/*
 * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2.0,
 * as published by the Free Software Foundation.
 *
 * This program is designed to work with certain software (including
 * but not limited to OpenSSL) that is licensed under separate terms,
 * as designated in a particular file or component or in included license
 * documentation.  The authors of MySQL hereby grant you an additional
 * permission to link the program and your derivative works with the
 * separately licensed software that they have either included with
 * the program or referenced in the documentation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License, version 2.0, for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef PLUGIN_X_SRC_INTERFACE_SOCKET_EVENTS_H_
#define PLUGIN_X_SRC_INTERFACE_SOCKET_EVENTS_H_

#include <functional>
#include <memory>

#include "plugin/x/src/interface/connection_acceptor.h"
#include "plugin/x/src/interface/socket.h"
#include "plugin/x/src/ngs/thread.h"

namespace xpl {
namespace iface {

class Socket_events {
 public:
  virtual ~Socket_events() = default;

  virtual bool listen(std::shared_ptr<Socket> s,
                      std::function<void(Connection_acceptor &)> callback) = 0;

  virtual void add_timer(const std::size_t delay_ms,
                         std::function<bool()> callback) = 0;
  virtual void loop() = 0;
  virtual void break_loop() = 0;
};

}  // namespace iface
}  // namespace xpl

#endif  // PLUGIN_X_SRC_INTERFACE_SOCKET_EVENTS_H_
