/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.

   This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
   separately licensed software that they have either included with
   the program or referenced in the documentation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License, version 2.0, for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

/*
  This header file contains weight tables and weight length table of
  UCA8.0.0, as well as some constant values and table of decomposition.
*/

#ifndef UCA900_DATA_H
#define UCA900_DATA_H

#include "m_ctype.h"
#include "strings/str_uca_type.h"

#define MY_UCA_900_CE_SIZE 3
const uint16 CASE_FIRST_UPPER_MASK= 0x0100;
const uint16 CASE_FIRST_MIXED_MASK= 0x0200;
const uint16 CASE_FIRST_LOWER_MASK= 0x0300;

enum enum_char_category{
  CHAR_CATEGORY_CC= 1, CHAR_CATEGORY_CF, CHAR_CATEGORY_CO, CHAR_CATEGORY_CS,
  CHAR_CATEGORY_LL, CHAR_CATEGORY_LM, CHAR_CATEGORY_LO, CHAR_CATEGORY_LT,
  CHAR_CATEGORY_LU, CHAR_CATEGORY_MC, CHAR_CATEGORY_ME, CHAR_CATEGORY_MN,
  CHAR_CATEGORY_ND, CHAR_CATEGORY_NL, CHAR_CATEGORY_NO, CHAR_CATEGORY_PC,
  CHAR_CATEGORY_PD, CHAR_CATEGORY_PE, CHAR_CATEGORY_PF, CHAR_CATEGORY_PI,
  CHAR_CATEGORY_PO, CHAR_CATEGORY_PS, CHAR_CATEGORY_SC, CHAR_CATEGORY_SK,
  CHAR_CATEGORY_SM, CHAR_CATEGORY_SO, CHAR_CATEGORY_ZL, CHAR_CATEGORY_ZP,
  CHAR_CATEGORY_ZS
};

enum enum_decomp_tag{
  DECOMP_TAG_NONE,      DECOMP_TAG_CIRCLE,   DECOMP_TAG_COMPAT,
  DECOMP_TAG_FINAL,     DECOMP_TAG_FONT,     DECOMP_TAG_FRACTION,
  DECOMP_TAG_INITIAL,   DECOMP_TAG_ISOLATED, DECOMP_TAG_MEDIAL,
  DECOMP_TAG_NARROW,    DECOMP_TAG_NOBREAK,  DECOMP_TAG_SMALL,
  DECOMP_TAG_SQUARE,    DECOMP_TAG_SUB,      DECOMP_TAG_SUPER,
  DECOMP_TAG_VERTICAL,  DECOMP_TAG_WIDE
};

struct Unidata_decomp
{
  my_wc_t charcode;
  enum_char_category category;
  enum_decomp_tag    decomp_tag;
  my_wc_t dec_codes[MY_UCA_MAX_CONTRACTION];
};

Unidata_decomp uni_dec[]= {
  {0x00A0, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00A8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00AA, CHAR_CATEGORY_LO, DECOMP_TAG_SUPER,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00AF, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B2, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B3, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B4, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B9, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00BA, CHAR_CATEGORY_LO, DECOMP_TAG_SUPER,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00BC, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0034, 0x0000, 0x0000, 0x0000}},
  {0x00BD, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0032, 0x0000, 0x0000, 0x0000}},
  {0x00BE, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0034, 0x0000, 0x0000, 0x0000}},
  {0x00C0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C7, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E8, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0100, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0101, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0102, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0103, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0104, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0105, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0106, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0107, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0108, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0109, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0112, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0113, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0114, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0115, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0116, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0117, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0118, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0119, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0120, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0121, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0122, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0123, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0124, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0125, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0128, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0129, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0130, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0132, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0049, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0133, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0069, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0134, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0135, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0136, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0137, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0139, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013F, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004C, 0x00B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0140, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006C, 0x00B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0143, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0144, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0145, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0146, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0147, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0148, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0149, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x02BC, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0150, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0151, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0154, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0155, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0156, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0157, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0158, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0159, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0160, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0161, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0162, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0163, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0164, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0165, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0168, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0169, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0170, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0171, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0172, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0173, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0174, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0175, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0176, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0177, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0178, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0179, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017F, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01A0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01A1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01AF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01B0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C4, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0044, 0x017D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C5, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x0044, 0x017E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C6, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0064, 0x017E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C7, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004C, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C8, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x004C, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C9, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006C, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CA, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004E, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CB, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x004E, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CC, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006E, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01D6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01D7, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x01D8, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x01D9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x030C, 0x0000, 0x0000, 0x0000}},
  {0x01DA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x030C, 0x0000, 0x0000, 0x0000}},
  {0x01DB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x01DC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x01DE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01DF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01E0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0307, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01E1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0307, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01E2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0328, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0328, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x01EE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01B7, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0292, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F1, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0044, 0x005A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F2, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x0044, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F3, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0064, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030A, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x01FB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030A, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x01FC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C6, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E6, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0200, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0201, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0202, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0203, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0204, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0205, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0206, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0207, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0208, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0209, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0210, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0211, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0212, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0213, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0214, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0215, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0216, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0217, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0218, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0219, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0226, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0227, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0228, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0229, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x022B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0308, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x022C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0303, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x022D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0303, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x022E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0230, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0307, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x0231, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0307, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x0232, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0233, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0266, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x027B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0281, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02D8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02D9, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DA, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DB, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DC, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DD, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0263, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0340, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0300, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0341, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0301, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0343, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0313, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0344, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0308, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0374, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x02B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x037A, CHAR_CATEGORY_LM, DECOMP_TAG_COMPAT,
   {0x0020, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x037E, CHAR_CATEGORY_PO, DECOMP_TAG_NONE,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0384, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0385, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0386, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0387, CHAR_CATEGORY_PO, DECOMP_TAG_NONE,
   {0x00B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0388, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0389, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0390, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x03AA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03B0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x03CA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D0, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D1, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D2, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03D2, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03D2, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D6, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F0, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F1, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F2, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F4, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F9, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0400, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0401, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0403, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0413, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0407, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0406, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x041A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0419, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0439, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0450, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0451, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0453, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0433, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0457, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0456, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x043A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0476, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0474, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0477, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0475, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04C1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0416, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04C2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0436, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0410, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0430, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0410, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0430, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x04D8, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x04D9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0416, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0436, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0417, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0437, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x041E, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x043E, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x04E8, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x04E9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x042D, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x044D, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0427, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0447, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x042B, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x044B, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0587, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0565, 0x0582, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0622, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0653, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0623, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0624, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0648, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0625, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0655, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0626, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x064A, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0675, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0627, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0676, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0648, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0677, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x06C7, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0678, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x064A, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06D5, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06C1, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06D2, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0928, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0930, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0933, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0915, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0916, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0917, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x091C, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0921, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0922, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x092B, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x092F, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09CB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x09C7, 0x09BE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09CC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x09C7, 0x09D7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09A1, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09A2, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09AF, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A32, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A38, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A59, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A16, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A17, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A1C, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A2B, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B48, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B56, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B4B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B4C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B57, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B21, 0x0B3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B5D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B22, 0x0B3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B94, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B92, 0x0BD7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC6, 0x0BBE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC7, 0x0BBE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC6, 0x0BD7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0C48, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0C46, 0x0C56, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC0, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CBF, 0x0CD5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC7, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CD5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC8, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CD6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CCA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CC2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CCB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CC2, 0x0CD5, 0x0000, 0x0000, 0x0000}},
  {0x0D4A, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D46, 0x0D3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0D4B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D47, 0x0D3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0D4C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D46, 0x0D57, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DCA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DCF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDD, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DCF, 0x0DCA, 0x0000, 0x0000, 0x0000}},
  {0x0DDE, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DDF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0E33, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0E4D, 0x0E32, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EB3, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0ECD, 0x0EB2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EDC, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0EAB, 0x0E99, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EDD, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0EAB, 0x0EA1, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F0C, CHAR_CATEGORY_PO, DECOMP_TAG_NOBREAK,
   {0x0F0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F42, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F4C, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F52, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F51, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F57, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F56, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F5B, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F69, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F40, 0x0FB5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F73, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F72, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F75, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F74, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F76, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FB2, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F77, CHAR_CATEGORY_MN, DECOMP_TAG_COMPAT,
   {0x0FB2, 0x0F81, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F78, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FB3, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F79, CHAR_CATEGORY_MN, DECOMP_TAG_COMPAT,
   {0x0FB3, 0x0F81, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F81, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F93, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F92, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F9D, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F9C, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FA2, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FA1, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FA7, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FA6, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FAC, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FAB, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FB9, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F90, 0x0FB5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1026, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1025, 0x102E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x10FC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x10DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B05, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B07, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B09, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B0B, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B0D, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B11, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B3B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3A, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B3D, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3C, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B40, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3E, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B41, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3F, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B43, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B42, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x00C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D30, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D31, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D32, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x018E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D33, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D34, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D35, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D36, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D37, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D38, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D39, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0222, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0250, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0251, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x014B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0254, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D17, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D1D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D25, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x043D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0252, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0255, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x00F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0261, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0265, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0268, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0269, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D7B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x029D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAA, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAB, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x029F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0271, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAD, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0270, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAE, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAF, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0273, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0274, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0275, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0278, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0282, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0283, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x01AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0289, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D1C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBA, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBB, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0290, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBD, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBE, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0292, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBF, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E00, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0325, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E01, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0325, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E02, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E03, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E04, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E05, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E06, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E07, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E08, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0327, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E09, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0327, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E0A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E10, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E11, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E12, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E13, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E14, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0304, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1E15, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0304, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1E16, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0304, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E17, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0304, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E18, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E19, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0327, 0x0306, 0x0000, 0x0000, 0x0000}},
  {0x1E1D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0327, 0x0306, 0x0000, 0x0000, 0x0000}},
  {0x1E1E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0046, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0066, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E20, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E21, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E22, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E23, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E24, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E25, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E26, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E27, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E28, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E29, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x032E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x032E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E2F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E30, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E31, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E32, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E33, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E34, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E35, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E36, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E37, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E38, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0323, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x1E39, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0323, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x1E3A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E40, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E41, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E42, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E43, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E44, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E45, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E46, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E47, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E48, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E49, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0303, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E4D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0303, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E4E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0303, 0x0308, 0x0000, 0x0000, 0x0000}},
  {0x1E4F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0303, 0x0308, 0x0000, 0x0000, 0x0000}},
  {0x1E50, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0304, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1E51, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0304, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1E52, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0304, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E53, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0304, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E54, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0050, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E55, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0070, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E56, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0050, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E57, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0070, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E58, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E59, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0323, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x1E5D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0323, 0x0304, 0x0000, 0x0000, 0x0000}},
  {0x1E5E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E60, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E61, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E62, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E63, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E64, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0301, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E65, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0301, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E66, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x030C, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E67, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x030C, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E68, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0323, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E69, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0323, 0x0307, 0x0000, 0x0000, 0x0000}},
  {0x1E6A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E70, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E71, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E72, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0324, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E73, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0324, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E74, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E75, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E76, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E77, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E78, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0303, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E79, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0303, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1E7A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0304, 0x0308, 0x0000, 0x0000, 0x0000}},
  {0x1E7B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0304, 0x0308, 0x0000, 0x0000, 0x0000}},
  {0x1E7C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0056, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0076, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0056, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0076, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E80, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E81, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E82, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E83, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E84, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E85, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E86, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E87, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E88, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E89, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0058, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0078, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0058, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0078, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E90, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E91, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E92, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E93, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E94, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E95, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E96, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E97, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E98, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E99, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E9A, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0061, 0x02BE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E9B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x017F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EA5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EA6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EA7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EA8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EA9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EAA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EAB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EAC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1EAD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1EAE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EAF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EB0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EB1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EB2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EB3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EB4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EB5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EB6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0323, 0x0306, 0x0000, 0x0000, 0x0000}},
  {0x1EB7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0323, 0x0306, 0x0000, 0x0000, 0x0000}},
  {0x1EB8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EBF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EC0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EC1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EC2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EC3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EC4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EC5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EC6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1EC7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1EC8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1ED1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1ED2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1ED3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1ED4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1ED5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1ED6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1ED7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1ED8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1ED9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0323, 0x0302, 0x0000, 0x0000, 0x0000}},
  {0x1EDA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EDB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EDC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EDD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EDE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EDF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EE0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EE1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EE2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0323, 0x0000, 0x0000, 0x0000}},
  {0x1EE3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0323, 0x0000, 0x0000, 0x0000}},
  {0x1EE4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EE9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1EEA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EEB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1EEC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0309, 0x0000, 0x0000, 0x0000}},
  {0x1EEE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EEF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0303, 0x0000, 0x0000, 0x0000}},
  {0x1EF0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0323, 0x0000, 0x0000, 0x0000}},
  {0x1EF1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0323, 0x0000, 0x0000, 0x0000}},
  {0x1EF2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F00, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F01, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F02, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F03, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F04, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F05, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F06, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F07, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F08, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F09, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F0B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F0C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F0D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F0E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F0F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F10, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F11, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F13, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F14, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F15, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F18, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F19, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F1A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F1B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F1C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F1D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F20, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F23, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F24, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F25, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F26, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F27, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F28, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F29, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F2B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F2C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F2D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F2E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F2F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F30, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F31, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F32, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F33, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F34, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F35, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F36, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F37, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F38, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F39, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F3B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F3C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F3D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F3E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F3F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F40, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F41, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F42, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F43, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F44, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F45, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F48, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F49, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F4A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F4B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F4C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F4D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F50, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F51, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F52, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F53, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F54, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F55, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F56, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F57, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F59, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F5B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F5D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F5F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F60, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F61, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F62, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F63, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F64, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F65, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F66, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F67, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F68, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F69, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F6B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1F6C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F6D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1F6E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F6F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1F70, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F71, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F72, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F73, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F74, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F75, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F76, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F77, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F78, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F79, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F80, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F81, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F82, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F83, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F84, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F85, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F86, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F87, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F88, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F89, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F8A, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F8B, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F8C, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F8D, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F8E, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F8F, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F90, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F91, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F92, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F93, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F94, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F95, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F96, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F97, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F98, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F99, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1F9A, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F9B, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1F9C, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F9D, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1F9E, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1F9F, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1FA0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FA1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FA2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1FA3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1FA4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1FA5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1FA6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1FA7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1FA8, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FA9, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FAA, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1FAB, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0300, 0x0345, 0x0000, 0x0000}},
  {0x1FAC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1FAD, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0301, 0x0345, 0x0000, 0x0000}},
  {0x1FAE, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1FAF, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0342, 0x0345, 0x0000, 0x0000}},
  {0x1FB0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0300, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FB3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0301, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FB6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0342, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FB8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBD, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBF, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC0, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC1, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0300, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FC3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0301, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FC6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0342, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FC8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1FD3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1FD6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1FD8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0300, 0x0000, 0x0000, 0x0000}},
  {0x1FE3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0301, 0x0000, 0x0000, 0x0000}},
  {0x1FE4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C1, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0342, 0x0000, 0x0000, 0x0000}},
  {0x1FE8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FED, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x0060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0300, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FF3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0301, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FF6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0342, 0x0345, 0x0000, 0x0000, 0x0000}},
  {0x1FF8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFE, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2000, CHAR_CATEGORY_ZS, DECOMP_TAG_NONE,
   {0x2002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2001, CHAR_CATEGORY_ZS, DECOMP_TAG_NONE,
   {0x2003, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2002, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2003, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2004, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2005, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2006, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2007, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2008, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2009, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x200A, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2011, CHAR_CATEGORY_PD, DECOMP_TAG_NOBREAK,
   {0x2010, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2017, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0020, 0x0333, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2024, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2025, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2026, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x002E, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x202F, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2033, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2034, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x2032, 0x0000, 0x0000, 0x0000}},
  {0x2036, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2035, 0x2035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2037, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2035, 0x2035, 0x2035, 0x0000, 0x0000, 0x0000}},
  {0x203C, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0021, 0x0021, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x203E, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0020, 0x0305, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2047, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x003F, 0x003F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2048, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x003F, 0x0021, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2049, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0021, 0x003F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2057, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x2032, 0x2032, 0x0000, 0x0000}},
  {0x205F, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2070, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2071, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2074, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2075, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2076, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2077, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2078, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2079, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207A, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207B, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x2212, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207C, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207D, CHAR_CATEGORY_PS, DECOMP_TAG_SUPER,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207E, CHAR_CATEGORY_PE, DECOMP_TAG_SUPER,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2080, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2081, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2082, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2083, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2084, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2085, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2086, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2087, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2088, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2089, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208A, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208B, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x2212, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208C, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208D, CHAR_CATEGORY_PS, DECOMP_TAG_SUB,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208E, CHAR_CATEGORY_PE, DECOMP_TAG_SUB,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2090, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2091, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2092, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2093, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2094, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2095, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2096, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2097, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2098, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2099, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209A, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209B, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209C, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x20A8, CHAR_CATEGORY_SC, DECOMP_TAG_COMPAT,
   {0x0052, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2100, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0061, 0x002F, 0x0063, 0x0000, 0x0000, 0x0000}},
  {0x2101, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0061, 0x002F, 0x0073, 0x0000, 0x0000, 0x0000}},
  {0x2102, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2103, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x00B0, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2105, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0063, 0x002F, 0x006F, 0x0000, 0x0000, 0x0000}},
  {0x2106, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0063, 0x002F, 0x0075, 0x0000, 0x0000, 0x0000}},
  {0x2107, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2109, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x00B0, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0127, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2110, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2111, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2112, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2113, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2115, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2116, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x004E, 0x006F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2119, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2120, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x0053, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2121, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0054, 0x0045, 0x004C, 0x0000, 0x0000, 0x0000}},
  {0x2122, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x0054, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2124, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2126, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2128, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2130, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2131, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2133, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2134, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2135, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2136, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2137, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2138, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2139, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0046, 0x0041, 0x0058, 0x0000, 0x0000, 0x0000}},
  {0x213C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2140, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2211, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2145, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2146, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2147, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2148, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2149, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2150, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0037, 0x0000, 0x0000, 0x0000}},
  {0x2151, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0039, 0x0000, 0x0000, 0x0000}},
  {0x2152, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0031, 0x0030, 0x0000, 0x0000}},
  {0x2153, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x2154, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0032, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x2155, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2156, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0032, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2157, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2158, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0034, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2159, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0036, 0x0000, 0x0000, 0x0000}},
  {0x215A, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0035, 0x2044, 0x0036, 0x0000, 0x0000, 0x0000}},
  {0x215B, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215C, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215D, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0035, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215E, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0037, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215F, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2160, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2161, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2162, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x2163, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2164, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2165, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2166, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x2167, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0049, 0x0049, 0x0000, 0x0000}},
  {0x2168, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0058, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2169, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216B, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x216C, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216D, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216E, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216F, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2170, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2171, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2172, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x2173, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0076, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2174, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2175, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2176, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x2177, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0069, 0x0069, 0x0000, 0x0000}},
  {0x2178, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0078, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2179, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217B, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x217C, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217D, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217E, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217F, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2189, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0030, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x219A, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2190, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x219B, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2192, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21AE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2194, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CD, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x21D0, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x21D4, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CF, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x21D2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2204, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2203, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2209, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2208, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x220C, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x220B, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2224, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2223, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2226, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2225, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x222C, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x222D, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x222B, 0x0000, 0x0000, 0x0000}},
  {0x222F, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222E, 0x222E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2230, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222E, 0x222E, 0x222E, 0x0000, 0x0000, 0x0000}},
  {0x2241, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x223C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2244, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2243, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2247, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2245, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2249, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2248, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2260, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2262, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2261, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226D, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x224D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226E, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226F, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003E, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2270, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2264, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2271, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2265, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2274, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2272, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2275, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2273, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2278, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2276, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2279, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2277, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2280, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227A, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2281, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227B, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2284, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2282, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2285, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2283, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2288, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2286, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2289, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2287, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AD, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A8, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A9, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AF, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22AB, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E0, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E1, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E2, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2291, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E3, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2292, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EA, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EB, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B3, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B4, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22ED, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B5, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2329, CHAR_CATEGORY_PS, DECOMP_TAG_NONE,
   {0x3008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x232A, CHAR_CATEGORY_PE, DECOMP_TAG_NONE,
   {0x3009, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2460, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2461, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2462, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2463, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2464, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2465, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2466, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2467, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2468, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2469, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246A, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246B, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246C, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246D, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246E, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246F, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2470, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2471, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2472, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2473, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2474, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2475, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0032, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2476, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0033, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2477, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0034, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2478, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0035, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2479, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0036, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0037, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0038, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247C, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0039, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247D, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0030, 0x0029, 0x0000, 0x0000}},
  {0x247E, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0031, 0x0029, 0x0000, 0x0000}},
  {0x247F, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0032, 0x0029, 0x0000, 0x0000}},
  {0x2480, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0033, 0x0029, 0x0000, 0x0000}},
  {0x2481, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0034, 0x0029, 0x0000, 0x0000}},
  {0x2482, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0035, 0x0029, 0x0000, 0x0000}},
  {0x2483, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0036, 0x0029, 0x0000, 0x0000}},
  {0x2484, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0037, 0x0029, 0x0000, 0x0000}},
  {0x2485, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0038, 0x0029, 0x0000, 0x0000}},
  {0x2486, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0039, 0x0029, 0x0000, 0x0000}},
  {0x2487, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0032, 0x0030, 0x0029, 0x0000, 0x0000}},
  {0x2488, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2489, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0033, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0034, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248C, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0035, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248D, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0036, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248E, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0037, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248F, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0038, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2490, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0039, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2491, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2492, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2493, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2494, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2495, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2496, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2497, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2498, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2499, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0061, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0062, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0063, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0064, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0065, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0066, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0067, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0068, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0069, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0070, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0071, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0072, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0073, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0074, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0075, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0076, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0077, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0078, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0079, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x007A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24EA, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2A0C, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x222B, 0x222B, 0x0000, 0x0000}},
  {0x2A74, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003A, 0x003A, 0x003D, 0x0000, 0x0000, 0x0000}},
  {0x2A75, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003D, 0x003D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2A76, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003D, 0x003D, 0x003D, 0x0000, 0x0000, 0x0000}},
  {0x2ADC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2ADD, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2C7C, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2C7D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2D6F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x2D61, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2E9F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2EF3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F00, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F01, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F02, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F03, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E3F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F04, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E59, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F05, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F06, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F07, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4EA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F08, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F09, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x513F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5165, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x516B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5182, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5196, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F10, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F11, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F12, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x529B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F13, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x52F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F14, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5315, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F15, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x531A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F16, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5338, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F17, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F18, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x535C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F19, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5369, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5382, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x56D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x571F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F20, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x58EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F21, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5902, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F22, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x590A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F23, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5915, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F24, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5927, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F25, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F26, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5B50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F27, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5B80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F28, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5BF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F29, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C71, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F30, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DF1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F31, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F32, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F33, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F34, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F35, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5EF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F36, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F37, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F38, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F39, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F61, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F73, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5FC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6208, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6236, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x624B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F40, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x652F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F41, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6534, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F42, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F43, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6597, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F44, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F45, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F46, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F47, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F48, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x66F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F49, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F50, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BD4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F51, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F52, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F53, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C14, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F54, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F55, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x706B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F56, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x722A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F57, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7236, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F58, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x723B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F59, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x723F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7247, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x725B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x72AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7384, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7389, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F60, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x74DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F61, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x74E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F62, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7518, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F63, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x751F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F64, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7528, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F65, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7530, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F66, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x758B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F67, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7592, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F68, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7676, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F69, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x767D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F70, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x793A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F71, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x79B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F72, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x79BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F73, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7A74, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F74, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F75, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7AF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F76, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7C73, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F77, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7CF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F78, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F79, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F51, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7FBD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x800C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x807F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8089, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x820C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x821B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x821F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x826E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8278, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x864D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x866B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8840, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x884C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8863, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x897E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x898B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x89D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8A00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C46, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C55, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C78, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8D64, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8D70, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8DB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8EAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8F9B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8FB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8FB5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9091, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9577, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9580, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x961C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9751, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x975E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9762, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9769, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9801, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9999, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x99AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9AA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9AD8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ADF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B25, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B32, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B3C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B5A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9CE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9E75, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EA5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ECD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F0E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F4A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F52, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9FA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3000, CHAR_CATEGORY_ZS, DECOMP_TAG_WIDE,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3036, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3038, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3039, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5344, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x303A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5345, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x304C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x304E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3050, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3052, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3051, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3054, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3053, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3056, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3055, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3058, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3057, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3059, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3060, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3062, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3061, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3065, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3064, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3067, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3066, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3069, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3068, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3070, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x306F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3071, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x306F, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3073, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3072, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3074, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3072, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3076, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3075, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3077, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3075, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3079, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3078, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3078, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x307B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x307B, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3094, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3046, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309B, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309C, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309E, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x309D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309F, CHAR_CATEGORY_LO, DECOMP_TAG_VERTICAL,
   {0x3088, 0x308A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B3, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B5, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B7, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B9, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C4, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C6, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C8, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30CF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30CF, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D2, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D2, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D5, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D5, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D8, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D8, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30DB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30DB, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30A6, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30EF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F0, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F2, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FE, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x30FD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FF, CHAR_CATEGORY_LO, DECOMP_TAG_VERTICAL,
   {0x30B3, 0x30C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3131, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1100, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3132, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1101, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3133, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3134, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1102, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3135, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3136, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3137, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3138, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1104, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3139, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3140, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3141, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1106, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3142, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1107, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3143, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1108, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3144, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1121, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3145, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1109, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3146, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3147, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3148, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3149, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1110, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1112, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1161, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3150, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1162, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3151, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1163, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3152, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3153, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1165, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3154, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1166, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3155, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1167, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3156, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3157, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1169, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3158, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3159, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1170, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1171, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3160, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1172, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3161, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1173, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3162, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1174, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3163, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1175, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3164, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3165, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1114, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3166, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1115, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3167, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3168, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3169, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3170, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3171, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3172, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3173, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1120, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3174, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1122, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3175, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1123, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3176, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1127, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3177, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1129, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3178, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3179, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1132, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1136, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3180, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1147, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3181, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x114C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3182, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3183, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3184, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1157, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3185, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1158, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3186, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1159, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3187, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1184, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3188, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1185, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3189, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1188, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1191, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1194, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x119E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3192, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3193, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3194, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3195, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x56DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3196, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E0A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3197, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3198, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3199, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x7532, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319A, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E59, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319B, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319C, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319D, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x5929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319E, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x5730, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319F, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3200, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1100, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3201, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1102, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3202, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1103, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3203, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1105, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3204, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1106, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3205, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1107, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3206, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1109, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3207, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3208, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3209, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1110, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1111, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1112, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1100, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x320F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1102, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3210, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1103, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3211, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1105, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3212, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1106, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3213, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1107, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3214, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1109, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3215, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3216, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3217, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110E, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3218, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110F, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3219, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1110, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1111, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1112, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x116E, 0x0029, 0x0000, 0x0000}},
  {0x321D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1169, 0x110C, 0x1165, 0x11AB}},
  {0x321E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1169, 0x1112, 0x116E, 0x0029}},
  {0x3220, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E00, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3221, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E8C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3222, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E09, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3223, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x56DB, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3224, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E94, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3225, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x516D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3226, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E03, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3227, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x516B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3228, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E5D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3229, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5341, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6708, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x706B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6C34, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6728, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x91D1, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x571F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3230, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x65E5, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3231, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x682A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3232, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6709, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3233, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x793E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3234, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x540D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3235, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x7279, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3236, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x8CA1, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3237, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x795D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3238, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x52B4, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3239, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4EE3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x547C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5B66, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x76E3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4F01, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x8CC7, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5354, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3240, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x796D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3241, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4F11, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3242, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x81EA, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3243, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x81F3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3244, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x554F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3245, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5E7C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3246, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3247, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7B8F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3250, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0054, 0x0045, 0x0000, 0x0000, 0x0000}},
  {0x3251, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3252, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3253, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3254, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3255, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3256, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3257, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3258, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3259, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325A, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325B, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325C, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325D, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325E, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325F, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3260, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1100, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3261, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1102, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3262, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3263, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3264, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1106, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3265, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1107, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3266, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1109, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3267, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3268, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3269, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1110, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1112, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1100, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1102, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3270, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1103, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3271, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1105, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3272, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1106, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3273, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1107, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3274, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1109, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3275, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3276, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3277, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3278, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110F, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3279, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1110, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1111, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1112, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x1161, 0x11B7, 0x1100, 0x1169, 0x0000}},
  {0x327D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x116E, 0x110B, 0x1174, 0x0000, 0x0000}},
  {0x327E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x116E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3280, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3281, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3282, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3283, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x56DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3284, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E94, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3285, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x516D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3286, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3287, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x516B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3288, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E5D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3289, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x706B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6C34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x571F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3290, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x65E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3291, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x682A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3292, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6709, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3293, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x793E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3294, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x540D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3295, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3296, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x8CA1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3297, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x795D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3298, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x52B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3299, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x79D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7537, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x9069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x512A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5370, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6CE8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x9805, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4F11, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5199, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6B63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E0A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x53F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x533B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5B97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5B66, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x76E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4F01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x8CC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5354, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x591C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B1, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B2, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B3, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B4, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B5, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B6, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B7, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B8, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B9, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BA, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BB, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BC, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BD, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BE, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BF, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0035, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32CD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0065, 0x0072, 0x0067, 0x0000, 0x0000, 0x0000}},
  {0x32CE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0065, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32CF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004C, 0x0054, 0x0044, 0x0000, 0x0000, 0x0000}},
  {0x32D0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32ED, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3300, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30D1, 0x30FC, 0x30C8, 0x0000, 0x0000}},
  {0x3301, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30EB, 0x30D5, 0x30A1, 0x0000, 0x0000}},
  {0x3302, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30F3, 0x30DA, 0x30A2, 0x0000, 0x0000}},
  {0x3303, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3304, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A4, 0x30CB, 0x30F3, 0x30B0, 0x0000, 0x0000}},
  {0x3305, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A4, 0x30F3, 0x30C1, 0x0000, 0x0000, 0x0000}},
  {0x3306, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A6, 0x30A9, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3307, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A8, 0x30B9, 0x30AF, 0x30FC, 0x30C9, 0x0000}},
  {0x3308, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A8, 0x30FC, 0x30AB, 0x30FC, 0x0000, 0x0000}},
  {0x3309, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AA, 0x30F3, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x330A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AA, 0x30FC, 0x30E0, 0x0000, 0x0000, 0x0000}},
  {0x330B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30A4, 0x30EA, 0x0000, 0x0000, 0x0000}},
  {0x330C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30E9, 0x30C3, 0x30C8, 0x0000, 0x0000}},
  {0x330D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30ED, 0x30EA, 0x30FC, 0x0000, 0x0000}},
  {0x330E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AC, 0x30ED, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x330F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AC, 0x30F3, 0x30DE, 0x0000, 0x0000, 0x0000}},
  {0x3310, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30AC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3311, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30CB, 0x30FC, 0x0000, 0x0000, 0x0000}},
  {0x3312, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30E5, 0x30EA, 0x30FC, 0x0000, 0x0000}},
  {0x3313, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30EB, 0x30C0, 0x30FC, 0x0000, 0x0000}},
  {0x3314, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3315, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30B0, 0x30E9, 0x30E0, 0x0000}},
  {0x3316, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30E1, 0x30FC, 0x30C8, 0x30EB}},
  {0x3317, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30EF, 0x30C3, 0x30C8, 0x0000}},
  {0x3318, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B0, 0x30E9, 0x30E0, 0x0000, 0x0000, 0x0000}},
  {0x3319, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B0, 0x30E9, 0x30E0, 0x30C8, 0x30F3, 0x0000}},
  {0x331A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AF, 0x30EB, 0x30BC, 0x30A4, 0x30ED, 0x0000}},
  {0x331B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AF, 0x30ED, 0x30FC, 0x30CD, 0x0000, 0x0000}},
  {0x331C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B1, 0x30FC, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x331D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30EB, 0x30CA, 0x0000, 0x0000, 0x0000}},
  {0x331E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30FC, 0x30DD, 0x0000, 0x0000, 0x0000}},
  {0x331F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x30A4, 0x30AF, 0x30EB, 0x0000, 0x0000}},
  {0x3320, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x30F3, 0x30C1, 0x30FC, 0x30E0, 0x0000}},
  {0x3321, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B7, 0x30EA, 0x30F3, 0x30B0, 0x0000, 0x0000}},
  {0x3322, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30BB, 0x30F3, 0x30C1, 0x0000, 0x0000, 0x0000}},
  {0x3323, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30BB, 0x30F3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x3324, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C0, 0x30FC, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x3325, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C7, 0x30B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3326, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C9, 0x30EB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3327, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C8, 0x30F3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3328, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CA, 0x30CE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3329, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CE, 0x30C3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x332A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CF, 0x30A4, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x332B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D1, 0x30FC, 0x30BB, 0x30F3, 0x30C8, 0x0000}},
  {0x332C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D1, 0x30FC, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x332D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D0, 0x30FC, 0x30EC, 0x30EB, 0x0000, 0x0000}},
  {0x332E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30A2, 0x30B9, 0x30C8, 0x30EB, 0x0000}},
  {0x332F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30AF, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3330, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3331, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D3, 0x30EB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3332, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30A1, 0x30E9, 0x30C3, 0x30C9, 0x0000}},
  {0x3333, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30A3, 0x30FC, 0x30C8, 0x0000, 0x0000}},
  {0x3334, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D6, 0x30C3, 0x30B7, 0x30A7, 0x30EB, 0x0000}},
  {0x3335, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30E9, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3336, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D8, 0x30AF, 0x30BF, 0x30FC, 0x30EB, 0x0000}},
  {0x3337, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30BD, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3338, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30CB, 0x30D2, 0x0000, 0x0000, 0x0000}},
  {0x3339, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D8, 0x30EB, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x333A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30F3, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x333B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30FC, 0x30B8, 0x0000, 0x0000, 0x0000}},
  {0x333C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D9, 0x30FC, 0x30BF, 0x0000, 0x0000, 0x0000}},
  {0x333D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DD, 0x30A4, 0x30F3, 0x30C8, 0x0000, 0x0000}},
  {0x333E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DC, 0x30EB, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x333F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30F3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3340, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DD, 0x30F3, 0x30C9, 0x0000, 0x0000, 0x0000}},
  {0x3341, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3342, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30FC, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3343, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30A4, 0x30AF, 0x30ED, 0x0000, 0x0000}},
  {0x3344, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30A4, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3345, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30C3, 0x30CF, 0x0000, 0x0000, 0x0000}},
  {0x3346, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30EB, 0x30AF, 0x0000, 0x0000, 0x0000}},
  {0x3347, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30F3, 0x30B7, 0x30E7, 0x30F3, 0x0000}},
  {0x3348, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30AF, 0x30ED, 0x30F3, 0x0000, 0x0000}},
  {0x3349, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30EA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x334A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30EA, 0x30D0, 0x30FC, 0x30EB, 0x0000}},
  {0x334B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30AC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x334C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30AC, 0x30C8, 0x30F3, 0x0000, 0x0000}},
  {0x334D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30FC, 0x30C8, 0x30EB, 0x0000, 0x0000}},
  {0x334E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E4, 0x30FC, 0x30C9, 0x0000, 0x0000, 0x0000}},
  {0x334F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E4, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3350, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E6, 0x30A2, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3351, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EA, 0x30C3, 0x30C8, 0x30EB, 0x0000, 0x0000}},
  {0x3352, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EA, 0x30E9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3353, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EB, 0x30D4, 0x30FC, 0x0000, 0x0000, 0x0000}},
  {0x3354, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EB, 0x30FC, 0x30D6, 0x30EB, 0x0000, 0x0000}},
  {0x3355, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EC, 0x30E0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3356, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EC, 0x30F3, 0x30C8, 0x30B2, 0x30F3, 0x0000}},
  {0x3357, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EF, 0x30C3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x3358, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0030, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3359, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3360, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3361, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3362, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3363, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3364, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3365, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3366, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3367, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3368, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3369, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0031, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0032, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0033, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3370, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0034, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3371, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0068, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x3372, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3373, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x0055, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3374, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0062, 0x0061, 0x0072, 0x0000, 0x0000, 0x0000}},
  {0x3375, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006F, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3376, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0063, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3377, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3378, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x3379, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x337A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0049, 0x0055, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5E73, 0x6210, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x662D, 0x548C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5927, 0x6B63, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x660E, 0x6CBB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x682A, 0x5F0F, 0x4F1A, 0x793E, 0x0000, 0x0000}},
  {0x3380, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3381, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3382, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3383, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3384, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3385, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3386, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3387, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3388, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0061, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x3389, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0063, 0x0061, 0x006C, 0x0000, 0x0000}},
  {0x338A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3390, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3391, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3392, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3393, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3394, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0054, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3395, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3396, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3397, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3398, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3399, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0066, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x00B2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33A2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x00B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33A6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2215, 0x0073, 0x0000, 0x0000, 0x0000}},
  {0x33A8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2215, 0x0073, 0x00B2, 0x0000, 0x0000}},
  {0x33A9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33AA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x0000, 0x0000, 0x0000}},
  {0x33AE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x2215, 0x0073, 0x0000}},
  {0x33AF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x2215, 0x0073, 0x00B2}},
  {0x33B0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x03A9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x03A9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0061, 0x002E, 0x006D, 0x002E, 0x0000, 0x0000}},
  {0x33C3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0042, 0x0071, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0063, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0064, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x2215, 0x006B, 0x0067, 0x0000, 0x0000}},
  {0x33C7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x006F, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x33C8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0079, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0068, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0050, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0069, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x004B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006F, 0x0067, 0x0000, 0x0000, 0x0000}},
  {0x33D3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x0078, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0062, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0069, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x33D6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006F, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x33D7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0048, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x002E, 0x006D, 0x002E, 0x0000, 0x0000}},
  {0x33D9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0050, 0x004D, 0x0000, 0x0000, 0x0000}},
  {0x33DA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0052, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0073, 0x0072, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0076, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0062, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0056, 0x2215, 0x006D, 0x0000, 0x0000, 0x0000}},
  {0x33DF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x2215, 0x006D, 0x0000, 0x0000, 0x0000}},
  {0x33E0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33ED, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0032, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0033, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0034, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0035, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0036, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0037, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0038, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0039, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0067, 0x0061, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0xA69C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x044A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA69D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x044C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA770, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xA76F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA7F8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0126, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA7F9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0153, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xA727, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xAB37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xAB52, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF900, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8C48, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF901, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF902, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF903, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF904, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF905, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E32, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF906, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF907, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF908, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF909, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5951, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5948, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7669, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF910, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x863F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF911, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x87BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF912, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88F8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF913, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x908F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF914, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF915, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF916, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF917, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF918, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x843D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF919, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x916A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E82, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5375, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B04, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x721B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x862D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF920, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF921, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF922, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FEB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF923, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x85CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF924, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8964, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF925, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF926, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF927, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x881F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF928, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D6A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF930, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x64C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6AD3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF932, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7210, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF933, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF935, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF936, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x865C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF937, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DEF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF938, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9732, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF939, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B6F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9DFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x788C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x797F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9304, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF940, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF941, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AD6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF942, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF943, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F04, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF944, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF945, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x807E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF946, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7262, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF947, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF948, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF949, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96F7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DDA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF950, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF951, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x964B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF952, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF953, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x808B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF954, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF955, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF956, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A1C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF957, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DBE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9675, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF960, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF961, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7387, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF962, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7570, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF963, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5317, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF964, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78FB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF965, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FBF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF966, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FA9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF967, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E0D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF968, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CCC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF969, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6578, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7701, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8449, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AAA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF970, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF971, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8FB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF972, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C88, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF973, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62FE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF974, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF975, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF976, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7565, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF977, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF978, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5169, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF979, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6881, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x826F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AD2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF980, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5442, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF981, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF982, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EEC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF983, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF984, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF985, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x792A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF986, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x95AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF987, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9A6A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF988, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF989, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9ECE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x529B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E74, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF990, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF991, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x649A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF992, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F23, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF993, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF994, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7489, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF995, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF996, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF997, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x806F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF998, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F26, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF999, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x934A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5217, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AAA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EC9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FF5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x637B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7375, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x56F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x601C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7469, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F9A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9234, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9748, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9818, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F8B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BEE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C3F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x71CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x907C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x962E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5289, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x677B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x67F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7409, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7559, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x786B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D10, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x985E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x516D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x622E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9678, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x502B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DEA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F2A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F8B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6144, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6817, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7387, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5229, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x540F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C65, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6613, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x674E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x68A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7406, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x75E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x533F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x541D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x71D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7498, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x85FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9C57, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6797, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7B20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C92, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B58, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8336, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x523A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA00, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA01, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EA6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA02, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA03, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CD6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA04, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA05, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA07, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x884C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA09, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x964D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x898B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA10, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA15, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA16, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x732A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA17, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA18, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x793C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA19, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x795E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7965, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CBE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7FBD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA20, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8612, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA22, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA25, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA26, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90FD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9928, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9DB4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA30, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA31, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA32, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x514D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA34, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA35, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5351, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x559D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA37, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA38, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5668, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA39, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5840, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C64, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA40, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA41, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x654F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA42, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA44, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6885, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA45, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA46, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E1A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA47, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA48, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x716E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA49, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x722B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7422, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7891, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x793E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7949, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7948, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7950, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA50, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7956, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA51, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x795D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA52, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA53, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA54, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA55, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A81, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA56, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA57, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA58, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA59, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8457, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA60, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8910, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA61, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA62, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA63, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA64, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CD3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA65, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA66, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8FB6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA67, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA68, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA69, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97FF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x983B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x242EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8218, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA70, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E26, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA71, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA72, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA73, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA74, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA75, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5180, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA76, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA77, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA78, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x559D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA79, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5555, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5944, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5954, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA80, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA81, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA82, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA83, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA84, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA85, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FAD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA86, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA87, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA88, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6108, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA89, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6234, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x641C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6452, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA90, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6556, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA91, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA92, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA93, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x671B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA94, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA95, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA96, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA97, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA98, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA99, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ECB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x716E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x77A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7235, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x732A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7471, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7506, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x753B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x761D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x761F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x774A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7740, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7AB1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C7B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D5B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8352, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8779, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8941, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8986, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ABF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9199, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9276, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x967C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97FF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x983B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B12, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2284A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22844, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x233D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4018, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25249, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25CD0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27ED3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F43, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F8E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB00, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB01, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB02, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x006C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB03, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0xFB04, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0xFB05, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x017F, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB06, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0073, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB13, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x0576, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB14, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x0565, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB15, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x056B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB16, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x057E, 0x0576, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB17, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x056D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D9, 0x05B4, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB1F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05F2, 0x05B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB20, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB21, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB22, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB23, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB24, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB25, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB26, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB27, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB28, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB29, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05C1, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05C2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05BC, 0x05C1, 0x0000, 0x0000, 0x0000}},
  {0xFB2D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05BC, 0x05C2, 0x0000, 0x0000, 0x0000}},
  {0xFB2E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05B8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB30, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB31, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D1, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB32, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D2, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D3, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB34, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D4, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB35, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D5, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D6, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB38, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D8, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB39, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D9, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DA, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DB, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DC, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DE, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB40, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E0, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB41, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E1, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E3, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB44, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E4, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB46, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E6, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB47, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E7, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB48, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E8, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB49, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05EA, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D5, 0x05B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D1, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DB, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E4, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D0, 0x05DC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB50, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0671, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB51, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0671, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB52, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB53, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB54, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB55, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB56, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB57, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB58, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB59, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB60, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB61, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB62, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB63, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB64, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB65, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB66, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB68, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB69, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB70, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB71, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB72, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB73, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB74, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB75, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB76, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB77, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB78, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB79, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB80, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB81, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB82, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB83, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB84, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB86, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB88, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB89, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0698, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0698, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB90, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB91, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB92, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB93, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB94, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB95, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB96, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB97, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB99, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAD, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBB0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBB1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD3, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD4, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD6, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0677, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE7, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06D5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBED, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06D5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0648, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0648, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC00, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC01, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC02, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC03, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC04, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC05, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC06, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC07, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC08, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC09, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC10, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC11, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC12, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC13, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC14, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC15, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC16, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC17, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC18, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC19, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC20, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC21, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC22, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC23, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC24, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC25, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC26, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC27, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC28, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC29, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC30, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC31, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC32, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC33, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC34, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC35, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC36, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC37, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC38, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC39, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC40, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC41, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC42, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC43, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC44, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC45, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC46, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC47, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC48, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC49, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC50, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC51, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC52, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC53, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC54, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC55, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC56, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC57, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC58, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC59, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0630, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0649, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064C, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC5F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064D, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC60, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064E, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC61, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064F, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC62, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0650, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC63, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0651, 0x0670, 0x0000, 0x0000, 0x0000}},
  {0xFC64, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC65, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC66, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC68, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC69, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC70, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC71, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC72, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC73, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC74, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC75, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC76, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC77, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC78, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC79, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC80, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC81, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC83, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC86, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC88, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC89, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC90, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0649, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC91, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC92, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC93, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC94, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC95, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC97, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC99, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9A, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE1, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE2, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE5, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE6, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE7, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEA, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEB, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCED, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEE, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF1, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF2, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064E, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064F, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0650, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF8, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD00, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD01, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD02, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD03, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD04, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD05, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD06, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD07, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD08, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD09, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD10, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD11, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD12, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD13, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD14, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD15, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD16, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD17, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD18, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD19, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD20, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD21, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD22, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD23, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD24, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD25, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD26, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD27, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD28, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD29, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD30, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD31, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD32, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD33, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD34, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD35, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD36, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD37, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD38, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD39, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3A, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3B, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0627, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD50, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD51, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD52, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD53, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD54, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD55, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD56, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD57, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD58, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD59, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD5A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD5B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD5C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD5D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD5E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD5F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD60, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD61, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD62, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD63, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD64, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x062D, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD65, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062D, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD66, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD68, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD69, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD6A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD6B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD6C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD6D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD6E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD70, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD71, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD72, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD73, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD74, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD75, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD76, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD77, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD78, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD79, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD7C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD80, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD81, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD83, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD86, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD88, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD89, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD8A, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD8C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD8D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD8E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD8F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD92, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD93, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD94, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD95, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD97, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD99, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD9A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD9B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD9C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD9D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD9E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD9F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062E, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA4, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062E, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA9, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDB5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDB6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDB9, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDBA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDBE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDBF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDF0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x06D2, 0x0000, 0x0000, 0x0000}},
  {0xFDF1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0644, 0x06D2, 0x0000, 0x0000, 0x0000}},
  {0xFDF2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0644, 0x0644, 0x0647, 0x0000, 0x0000}},
  {0xFDF3, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0643, 0x0628, 0x0631, 0x0000, 0x0000}},
  {0xFDF4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062D, 0x0645, 0x062F, 0x0000, 0x0000}},
  {0xFDF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0639, 0x0645, 0x0000, 0x0000}},
  {0xFDF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0633, 0x0648, 0x0644, 0x0000, 0x0000}},
  {0xFDF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0644, 0x064A, 0x0647, 0x0000, 0x0000}},
  {0xFDF8, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0648, 0x0633, 0x0644, 0x0645, 0x0000, 0x0000}},
  {0xFDF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDFA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0649, 0x0020, 0x0627, 0x0644}},
  {0xFDFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0644, 0x0020, 0x062C, 0x0644, 0x0627}},
  {0xFDFC, CHAR_CATEGORY_SC, DECOMP_TAG_ISOLATED,
   {0x0631, 0x06CC, 0x0627, 0x0644, 0x0000, 0x0000}},
  {0xFE10, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE11, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE12, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x3002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE13, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE14, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE15, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE16, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE17, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3016, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE18, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3017, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE19, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x2026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE30, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x2025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE31, CHAR_CATEGORY_PD, DECOMP_TAG_VERTICAL,
   {0x2014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE32, CHAR_CATEGORY_PD, DECOMP_TAG_VERTICAL,
   {0x2013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE33, CHAR_CATEGORY_PC, DECOMP_TAG_VERTICAL,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE34, CHAR_CATEGORY_PC, DECOMP_TAG_VERTICAL,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE35, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE36, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE37, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE38, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE39, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3A, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3015, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3B, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3010, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3C, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3011, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3D, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3E, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3F, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE40, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3009, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE41, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE42, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE43, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE44, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE47, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x005B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE48, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x005D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE49, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4A, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4B, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4C, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4D, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4E, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4F, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE50, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE51, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE52, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE54, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE55, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE56, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE57, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE58, CHAR_CATEGORY_PD, DECOMP_TAG_SMALL,
   {0x2014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE59, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5A, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5B, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5C, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5D, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x3014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5E, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x3015, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5F, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE60, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE61, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE62, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE63, CHAR_CATEGORY_PD, DECOMP_TAG_SMALL,
   {0x002D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE64, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE65, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE66, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE68, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x005C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE69, CHAR_CATEGORY_SC, DECOMP_TAG_SMALL,
   {0x0024, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE6A, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE6B, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0040, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE70, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE71, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE72, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE74, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE76, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE77, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE78, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE79, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0650, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7B, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0650, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0651, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0651, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0652, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7F, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0652, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE80, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0621, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE81, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0622, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0622, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE83, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0623, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0623, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE85, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0624, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE86, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0624, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE87, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0625, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE88, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0625, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE89, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8C, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE90, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE91, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE92, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE93, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0629, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE94, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0629, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE95, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE97, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE98, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE99, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9C, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEED, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0622, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0622, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0623, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0623, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0625, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0625, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF01, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF02, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0022, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF03, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF04, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x0024, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF05, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF06, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF07, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0027, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF08, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF09, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0A, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0B, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0C, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0D, CHAR_CATEGORY_PD, DECOMP_TAG_WIDE,
   {0x002D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0E, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0F, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF10, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF11, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF12, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF13, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF14, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF15, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF16, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF17, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF18, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF19, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1A, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1B, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1C, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1D, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1E, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1F, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF20, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0040, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF21, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF22, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF23, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF24, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF25, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF26, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF27, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF28, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF29, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2A, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2B, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2C, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2D, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2E, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2F, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF30, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF31, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF32, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF33, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF34, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF35, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF36, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF37, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF38, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF39, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3A, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3B, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x005B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3C, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x005C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3D, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x005D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3E, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x005E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3F, CHAR_CATEGORY_PC, DECOMP_TAG_WIDE,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF40, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x0060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF41, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF42, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF43, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF44, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF45, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF46, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF47, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF48, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF49, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4A, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4B, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4C, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4D, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4E, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4F, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF50, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF51, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF52, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF53, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF54, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF55, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF56, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF57, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF58, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF59, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5A, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5B, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5C, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x007C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5D, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5E, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x007E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5F, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x2985, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF60, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x2986, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF61, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x3002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF62, CHAR_CATEGORY_PS, DECOMP_TAG_NARROW,
   {0x300C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF63, CHAR_CATEGORY_PE, DECOMP_TAG_NARROW,
   {0x300D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF64, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF65, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x30FB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF66, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF67, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF68, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF69, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF70, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x30FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF71, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF72, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF73, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF74, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF75, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF76, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF77, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF78, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF79, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF80, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF81, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF82, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF83, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF84, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF85, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF86, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF87, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF88, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF89, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF90, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF91, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF92, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF93, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF94, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF95, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF96, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF97, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF98, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF99, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9E, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x3099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9F, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x309A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA0, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA1, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3131, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3132, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3133, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3134, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3135, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3136, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3137, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA8, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3138, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA9, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3139, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAF, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB0, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB1, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3141, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3142, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3143, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3144, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3146, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3147, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB8, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3148, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB9, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3150, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3151, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3152, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3153, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3154, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3155, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3156, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3157, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3158, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3159, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCF, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3161, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3162, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3163, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE0, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE1, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE2, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x00AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE3, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x00AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE4, CHAR_CATEGORY_SO, DECOMP_TAG_WIDE,
   {0x00A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE5, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE6, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x20A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE8, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x2502, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE9, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEA, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2191, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEB, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEC, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2193, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFED, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x25A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEE, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x25CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1109A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x11099, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1109C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1109B, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x110AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x110A5, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1112E, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x11131, 0x11127, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1112F, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x11132, 0x11127, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1134B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x11347, 0x1133E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1134C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x11347, 0x11357, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114B0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BE, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114BD, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x115BA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x115B8, 0x115AF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x115BB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x115B9, 0x115AF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D15E, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D157, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D15F, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D160, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x1D16E, 0x0000, 0x0000, 0x0000}},
  {0x1D161, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x1D16F, 0x0000, 0x0000, 0x0000}},
  {0x1D162, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x1D170, 0x0000, 0x0000, 0x0000}},
  {0x1D163, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x1D171, 0x0000, 0x0000, 0x0000}},
  {0x1D164, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x1D172, 0x0000, 0x0000, 0x0000}},
  {0x1D1BB, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1B9, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BC, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BA, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BD, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1B9, 0x1D165, 0x1D16E, 0x0000, 0x0000, 0x0000}},
  {0x1D1BE, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BA, 0x1D165, 0x1D16E, 0x0000, 0x0000, 0x0000}},
  {0x1D1BF, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1B9, 0x1D165, 0x1D16F, 0x0000, 0x0000, 0x0000}},
  {0x1D1C0, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BA, 0x1D165, 0x1D16F, 0x0000, 0x0000, 0x0000}},
  {0x1D400, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D401, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D402, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D403, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D404, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D405, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D406, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D407, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D408, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D409, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D410, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D411, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D412, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D413, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D414, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D415, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D416, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D417, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D418, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D419, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D420, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D421, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D422, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D423, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D424, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D425, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D426, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D427, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D428, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D429, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D430, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D431, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D432, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D433, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D434, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D435, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D436, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D437, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D438, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D439, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D440, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D441, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D442, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D443, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D444, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D445, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D446, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D447, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D448, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D449, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D450, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D451, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D452, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D453, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D454, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D456, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D457, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D458, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D459, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D460, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D461, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D462, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D463, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D464, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D465, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D466, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D467, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D468, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D469, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D470, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D471, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D472, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D473, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D474, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D475, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D476, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D477, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D478, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D479, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D480, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D481, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D482, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D483, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D484, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D485, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D486, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D487, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D488, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D489, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D490, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D491, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D492, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D493, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D494, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D495, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D496, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D497, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D498, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D499, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4ED, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D500, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D501, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D502, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D503, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D504, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D505, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D507, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D508, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D509, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D510, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D511, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D512, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D513, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D514, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D516, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D517, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D518, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D519, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D520, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D521, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D522, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D523, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D524, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D525, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D526, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D527, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D528, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D529, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D530, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D531, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D532, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D533, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D534, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D535, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D536, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D537, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D538, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D539, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D540, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D541, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D542, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D543, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D544, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D546, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D550, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D552, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D553, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D554, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D555, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D556, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D557, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D558, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D559, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D560, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D561, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D562, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D563, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D564, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D565, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D566, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D567, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D568, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D569, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D570, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D571, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D572, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D573, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D574, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D575, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D576, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D577, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D578, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D579, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D580, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D581, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D582, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D583, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D584, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D585, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D586, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D587, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D588, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D589, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D590, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D591, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D592, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D593, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D594, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D595, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D596, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D597, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D598, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D599, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5ED, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D600, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D601, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D602, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D603, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D604, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D605, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D606, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D607, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D608, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D609, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D610, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D611, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D612, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D613, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D614, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D615, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D616, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D617, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D618, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D619, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D620, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D621, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D622, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D623, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D624, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D625, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D626, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D627, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D628, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D629, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D630, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D631, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D632, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D633, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D634, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D635, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D636, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D637, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D638, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D639, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D640, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D641, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D642, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D643, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D644, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D645, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D646, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D647, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D648, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D649, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D650, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D651, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D652, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D653, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D654, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D655, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D656, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D657, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D658, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D659, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D660, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D661, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D662, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D663, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D664, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D665, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D666, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D667, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D668, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D669, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D670, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D671, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D672, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D673, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D674, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D675, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D676, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D677, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D678, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D679, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D680, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D681, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D682, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D683, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D684, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D685, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D686, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D687, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D688, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D689, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D690, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D691, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D692, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D693, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D694, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D695, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D696, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D697, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D698, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D699, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0131, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0237, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C1, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DB, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6ED, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FB, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D700, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D701, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D702, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D703, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D704, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D705, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D706, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D707, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D708, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D709, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D710, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D711, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D712, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D713, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D714, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D715, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D716, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D717, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D718, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D719, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D720, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D721, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D722, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D723, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D724, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D725, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D726, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D727, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D728, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D729, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D730, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D731, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D732, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D733, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D734, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D735, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D736, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D737, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D738, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D739, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D740, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D741, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D742, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D743, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D744, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D745, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D746, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D747, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D748, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D749, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74F, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D750, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D751, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D752, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D753, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D754, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D755, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D756, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D757, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D758, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D759, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D760, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D761, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D762, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D763, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D764, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D765, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D766, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D767, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D768, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D769, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76F, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D770, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D771, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D772, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D773, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D774, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D775, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D776, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D777, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D778, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D779, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D780, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D781, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D782, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D783, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D784, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D785, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D786, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D787, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D788, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D789, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D790, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D791, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D792, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D793, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D794, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D795, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D796, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D797, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D798, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D799, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A9, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C3, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DD, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7ED, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FD, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE00, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE01, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE02, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE03, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE05, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE06, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE07, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE08, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE09, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE10, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE11, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE12, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE13, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE14, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE15, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE16, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE17, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE18, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE19, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE21, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE22, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE24, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE27, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE29, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE30, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE31, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE32, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE34, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE35, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE36, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE37, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE39, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE3B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE42, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE47, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE49, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE51, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE52, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE54, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE57, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE59, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE61, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE62, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE64, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE67, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE68, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE69, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE70, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE71, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE72, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE74, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE75, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE76, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE77, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE79, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE80, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE81, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE82, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE83, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE84, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE85, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE86, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE87, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE88, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE89, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE90, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE91, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE92, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE93, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE94, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE95, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE96, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE97, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE98, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE99, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE9A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE9B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA1, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA2, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA3, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA5, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA6, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA7, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA8, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA9, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAB, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAC, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAD, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAE, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAF, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB0, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB1, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB2, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB3, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB4, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB5, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB6, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB7, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB8, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB9, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEBA, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEBB, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F100, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0030, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F101, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0030, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F102, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F103, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F104, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0033, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F105, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0034, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F106, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0035, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F107, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0036, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F108, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0037, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F109, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0038, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F10A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0039, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F110, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0041, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F111, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0042, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F112, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0043, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F113, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0044, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F114, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0045, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F115, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0046, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F116, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0047, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F117, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0048, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F118, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0049, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F119, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0050, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F120, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0051, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F121, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0052, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F122, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0053, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F123, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0054, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F124, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0055, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F125, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0056, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F126, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0057, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F127, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0058, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F128, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0059, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F129, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x005A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F12A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x0053, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F12B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0057, 0x005A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F130, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F131, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F132, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F133, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F134, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F135, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F136, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F137, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F138, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F139, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F140, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F141, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F142, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F143, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F144, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F145, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F146, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F147, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F148, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F149, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0053, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0050, 0x0056, 0x0000, 0x0000, 0x0000}},
  {0x1F14F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F16A, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x004D, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F16B, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x004D, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F190, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0044, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F200, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x307B, 0x304B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F201, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F202, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F210, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x624B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F211, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5B57, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F212, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x53CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F213, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F214, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F215, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x591A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F216, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x89E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F217, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F218, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4EA4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F219, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6620, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7121, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x524D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5F8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x518D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x65B0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F220, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x521D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F221, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7D42, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F222, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x751F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F223, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x8CA9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F224, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x58F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F225, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5439, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F226, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6F14, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F227, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F228, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6355, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F229, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x904A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x53F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6307, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F230, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x8D70, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F231, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6253, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F232, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7981, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F233, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7A7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F234, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5408, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F235, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6E80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F236, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6709, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F237, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F238, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7533, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F239, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F23A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x55B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F23B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x914D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F240, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x672C, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F241, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x4E09, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F242, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x4E8C, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F243, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x5B89, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F244, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x70B9, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F245, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x6253, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F246, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x76D7, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F247, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x52DD, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F248, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x6557, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F250, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5F97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F251, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x53EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F800, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E3D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F801, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F802, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F803, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20122, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F804, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F805, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F806, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F807, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F808, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x507A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F809, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x349E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x514D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5154, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F810, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F811, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5177, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F812, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2051C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F813, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F814, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5167, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F815, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x518D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F816, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2054B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F817, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5197, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F818, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F819, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4ECC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x291DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5203, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F820, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x523B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F821, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5246, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F822, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F823, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5277, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F824, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3515, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F825, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F826, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F827, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F828, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F829, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5305, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5306, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5317, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5349, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5351, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x535A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5373, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F830, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F831, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F832, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F833, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F834, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20A2C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F835, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F836, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F837, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F838, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20B63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F839, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5406, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x549E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5438, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5448, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5468, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F840, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F841, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F842, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5510, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F843, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5553, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F844, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5563, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F845, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5584, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F846, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5584, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F847, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F848, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F849, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5716, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5651, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F850, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F851, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F852, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x57CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F853, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x57F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F854, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x580D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F855, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x578B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F856, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5832, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F857, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5831, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F858, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F859, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x214E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58F7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5906, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x591A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5922, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5962, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F860, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x216A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F861, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x216EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F862, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x59EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F863, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F864, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A27, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F865, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x59D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F866, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A66, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F867, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x36EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F868, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x36FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F869, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x219C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BD8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F870, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BF3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F871, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21B18, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F872, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F873, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C06, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F874, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F53, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F875, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F876, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3781, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F877, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F878, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F879, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5CC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21DE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D43, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F880, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D7C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F881, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DE1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F882, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DE2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F883, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x382F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F884, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F885, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F886, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E3D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F887, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F888, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3862, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F889, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22183, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x387C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F890, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F891, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22331, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F892, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22331, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F893, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8201, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F894, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F895, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F896, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x38C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F897, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x232B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F898, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x261DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F899, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x38E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F9A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FD7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6081, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x393A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x391C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x226D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6148, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x617A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6210, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x621B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x625D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6350, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22B0C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x633D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6368, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6383, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22BF1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6422, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3A2E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6469, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x647E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x649D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6477, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3A6C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x654F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x656C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2300A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66F8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3AE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5195, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6700, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x669C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x80AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x43D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x671B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6721, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x675E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6753, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x233C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B49, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x67FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6785, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6852, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6885, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2346D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x688E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x681F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6914, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6942, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x69A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x69EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6AA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x236A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ADB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3C18, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B21, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x238A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B54, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3C4E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23A8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21D0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23AFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C4E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23CBC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CBF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C67, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F900, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F901, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F902, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F903, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F904, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D78, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F905, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F906, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23D1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F907, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F908, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F909, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3D33, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ECB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F910, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23F5E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F911, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23F8E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F912, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FC6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F913, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F914, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F915, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F916, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3D96, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F917, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x704A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F918, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x707D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F919, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20525, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24263, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x719C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x243AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F920, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7228, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F921, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7235, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F922, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7250, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F923, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24608, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F924, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7280, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F925, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F926, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24735, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F927, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24814, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F928, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x737A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x738B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EAC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7447, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x745C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F930, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7471, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7485, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F932, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x74CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F933, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3F1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7524, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F935, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24C36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F936, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x753E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F937, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24C92, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F938, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7570, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F939, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2219F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7610, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24FA1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24FB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3FFC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F940, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F941, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x250F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F942, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x250F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F943, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25119, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F944, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25133, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F945, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F946, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F947, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F948, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x774A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F949, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x778B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4096, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2541D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x784E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x788C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F950, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F951, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x40E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F952, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F953, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7956, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F954, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2569A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F955, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x256C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F956, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F957, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x412F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A4A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A4F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2597C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25AA7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25AA7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7AEE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F960, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F961, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25BAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F962, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F963, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F964, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4227, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F965, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25C80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F966, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CD2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F967, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x42A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F968, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F969, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25F86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4301, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F970, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E45, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F971, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4334, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F972, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26228, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F973, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26247, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F974, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4359, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F975, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x262D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F976, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F977, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2633E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F978, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F95, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F979, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7FFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x264DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26523, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x265A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F980, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2335F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F981, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x43D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F982, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x80B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F983, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F984, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x440B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F985, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x813E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F986, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5AB5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F987, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x267A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F988, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x267B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F989, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2339C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8201, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8204, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F9E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x446B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F990, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x828B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F991, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x829D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F992, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F993, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F994, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F995, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F996, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F997, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26B3C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F998, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F999, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x831D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8363, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8323, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8457, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8353, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26C36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26D6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26CD5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x452B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8516, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x273CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8564, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26F2C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x455D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4561, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26FB1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x270D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x456B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8650, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x865C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8667, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8669, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x86A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x870E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x86E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8779, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x876B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8786, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x45D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x87E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8801, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x45F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8860, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8863, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27667, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x278AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27966, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x46BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x46C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8C55, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27CA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27F2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20804, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DBC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DF0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x208DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ED4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x285D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x285ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2872E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x911B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9238, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x92D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x92D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x927C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x93F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9415, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x28BFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x958B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4995, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x95B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x28D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x49E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DB2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9723, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2921A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4A6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4A76, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2940A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4AB2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29496, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA00, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9829, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA01, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x295B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA02, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA03, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4B33, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA04, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA05, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA07, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99FE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4BCE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA09, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29B30, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B12, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9C40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9CFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CCE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9D67, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA10, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A0CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA11, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA13, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A20E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA14, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA15, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA16, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4D56, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA17, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA18, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA19, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F05, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
};

struct Combining_mark
{
  my_wc_t   charcode;
  int       ccc; //Canonical_Combining_Class
};

/*
  All the combining marks whose ccc value is greater than 0.
  Gerneral_Category is Mn.
*/
Combining_mark combining_marks[]= {
  {0x0300, 230}, {0x0301, 230}, {0x0302, 230}, {0x0303, 230},
  {0x0304, 230}, {0x0305, 230}, {0x0306, 230}, {0x0307, 230},
  {0x0308, 230}, {0x0309, 230}, {0x030A, 230}, {0x030B, 230},
  {0x030C, 230}, {0x030D, 230}, {0x030E, 230}, {0x030F, 230},
  {0x0310, 230}, {0x0311, 230}, {0x0312, 230}, {0x0313, 230},
  {0x0314, 230}, {0x0315, 232}, {0x0316, 220}, {0x0317, 220},
  {0x0318, 220}, {0x0319, 220}, {0x031A, 232}, {0x031B, 216},
  {0x031C, 220}, {0x031D, 220}, {0x031E, 220}, {0x031F, 220},
  {0x0320, 220}, {0x0321, 202}, {0x0322, 202}, {0x0323, 220},
  {0x0324, 220}, {0x0325, 220}, {0x0326, 220}, {0x0327, 202},
  {0x0328, 202}, {0x0329, 220}, {0x032A, 220}, {0x032B, 220},
  {0x032C, 220}, {0x032D, 220}, {0x032E, 220}, {0x032F, 220},
  {0x0330, 220}, {0x0331, 220}, {0x0332, 220}, {0x0333, 220},
  {0x0334, 1},   {0x0335, 1},   {0x0336, 1},   {0x0337, 1},
  {0x0338, 1},   {0x0339, 220}, {0x033A, 220}, {0x033B, 220},
  {0x033C, 220}, {0x033D, 230}, {0x033E, 230}, {0x033F, 230},
  {0x0340, 230}, {0x0341, 230}, {0x0342, 230}, {0x0343, 230},
  {0x0344, 230}, {0x0345, 240}, {0x0346, 230}, {0x0347, 220},
  {0x0348, 220}, {0x0349, 220}, {0x034A, 230}, {0x034B, 230},
  {0x034C, 230}, {0x034D, 220}, {0x034E, 220}, {0x0350, 230},
  {0x0351, 230}, {0x0352, 230}, {0x0353, 220}, {0x0354, 220},
  {0x0355, 220}, {0x0356, 220}, {0x0357, 230}, {0x0358, 232},
  {0x0359, 220}, {0x035A, 220}, {0x035B, 230}, {0x035C, 233},
  {0x035D, 234}, {0x035E, 234}, {0x035F, 233}, {0x0360, 234},
  {0x0361, 234}, {0x0362, 233}, {0x0363, 230}, {0x0364, 230},
  {0x0365, 230}, {0x0366, 230}, {0x0367, 230}, {0x0368, 230},
  {0x0369, 230}, {0x036A, 230}, {0x036B, 230}, {0x036C, 230},
  {0x036D, 230}, {0x036E, 230}, {0x036F, 230}, {0x0483, 230},
  {0x0484, 230}, {0x0485, 230}, {0x0486, 230}, {0x0487, 230},
  {0x0591, 220}, {0x0592, 230}, {0x0593, 230}, {0x0594, 230},
  {0x0595, 230}, {0x0596, 220}, {0x0597, 230}, {0x0598, 230},
  {0x0599, 230}, {0x059A, 222}, {0x059B, 220}, {0x059C, 230},
  {0x059D, 230}, {0x059E, 230}, {0x059F, 230}, {0x05A0, 230},
  {0x05A1, 230}, {0x05A2, 220}, {0x05A3, 220}, {0x05A4, 220},
  {0x05A5, 220}, {0x05A6, 220}, {0x05A7, 220}, {0x05A8, 230},
  {0x05A9, 230}, {0x05AA, 220}, {0x05AB, 230}, {0x05AC, 230},
  {0x05AD, 222}, {0x05AE, 228}, {0x05AF, 230}, {0x05B0, 10},
  {0x05B1, 11},  {0x05B2, 12},  {0x05B3, 13},  {0x05B4, 14},
  {0x05B5, 15},  {0x05B6, 16},  {0x05B7, 17},  {0x05B8, 18},
  {0x05B9, 19},  {0x05BA, 19},  {0x05BB, 20},  {0x05BC, 21},
  {0x05BD, 22},  {0x05BF, 23},  {0x05C1, 24},  {0x05C2, 25},
  {0x05C4, 230}, {0x05C5, 220}, {0x05C7, 18},  {0x0610, 230},
  {0x0611, 230}, {0x0612, 230}, {0x0613, 230}, {0x0614, 230},
  {0x0615, 230}, {0x0616, 230}, {0x0617, 230}, {0x0618, 30},
  {0x0619, 31},  {0x061A, 32},  {0x064B, 27},  {0x064C, 28},
  {0x064D, 29},  {0x064E, 30},  {0x064F, 31},  {0x0650, 32},
  {0x0651, 33},  {0x0652, 34},  {0x0653, 230}, {0x0654, 230},
  {0x0655, 220}, {0x0656, 220}, {0x0657, 230}, {0x0658, 230},
  {0x0659, 230}, {0x065A, 230}, {0x065B, 230}, {0x065C, 220},
  {0x065D, 230}, {0x065E, 230}, {0x065F, 220}, {0x0670, 35},
  {0x06D6, 230}, {0x06D7, 230}, {0x06D8, 230}, {0x06D9, 230},
  {0x06DA, 230}, {0x06DB, 230}, {0x06DC, 230}, {0x06DF, 230},
  {0x06E0, 230}, {0x06E1, 230}, {0x06E2, 230}, {0x06E3, 220},
  {0x06E4, 230}, {0x06E7, 230}, {0x06E8, 230}, {0x06EA, 220},
  {0x06EB, 230}, {0x06EC, 230}, {0x06ED, 220}, {0x0711, 36},
  {0x0730, 230}, {0x0731, 220}, {0x0732, 230}, {0x0733, 230},
  {0x0734, 220}, {0x0735, 230}, {0x0736, 230}, {0x0737, 220},
  {0x0738, 220}, {0x0739, 220}, {0x073A, 230}, {0x073B, 220},
  {0x073C, 220}, {0x073D, 230}, {0x073E, 220}, {0x073F, 230},
  {0x0740, 230}, {0x0741, 230}, {0x0742, 220}, {0x0743, 230},
  {0x0744, 220}, {0x0745, 230}, {0x0746, 220}, {0x0747, 230},
  {0x0748, 220}, {0x0749, 230}, {0x074A, 230}, {0x07EB, 230},
  {0x07EC, 230}, {0x07ED, 230}, {0x07EE, 230}, {0x07EF, 230},
  {0x07F0, 230}, {0x07F1, 230}, {0x07F2, 220}, {0x07F3, 230},
  {0x0816, 230}, {0x0817, 230}, {0x0818, 230}, {0x0819, 230},
  {0x081B, 230}, {0x081C, 230}, {0x081D, 230}, {0x081E, 230},
  {0x081F, 230}, {0x0820, 230}, {0x0821, 230}, {0x0822, 230},
  {0x0823, 230}, {0x0825, 230}, {0x0826, 230}, {0x0827, 230},
  {0x0829, 230}, {0x082A, 230}, {0x082B, 230}, {0x082C, 230},
  {0x082D, 230}, {0x0859, 220}, {0x085A, 220}, {0x085B, 220},
  {0x08D4, 230}, {0x08D5, 230}, {0x08D6, 230}, {0x08D7, 230},
  {0x08D8, 230}, {0x08D9, 230}, {0x08DA, 230}, {0x08DB, 230},
  {0x08DC, 230}, {0x08DD, 230}, {0x08DE, 230}, {0x08DF, 230},
  {0x08E0, 230}, {0x08E1, 230}, {0x08E3, 220}, {0x08E4, 230},
  {0x08E5, 230}, {0x08E6, 220}, {0x08E7, 230}, {0x08E8, 230},
  {0x08E9, 220}, {0x08EA, 230}, {0x08EB, 230}, {0x08EC, 230},
  {0x08ED, 220}, {0x08EE, 220}, {0x08EF, 220}, {0x08F0, 27},
  {0x08F1, 28},  {0x08F2, 29},  {0x08F3, 230}, {0x08F4, 230},
  {0x08F5, 230}, {0x08F6, 220}, {0x08F7, 230}, {0x08F8, 230},
  {0x08F9, 220}, {0x08FA, 220}, {0x08FB, 230}, {0x08FC, 230},
  {0x08FD, 230}, {0x08FE, 230}, {0x08FF, 230}, {0x093C, 7},
  {0x094D, 9},   {0x0951, 230}, {0x0952, 220}, {0x0953, 230},
  {0x0954, 230}, {0x09BC, 7},   {0x09CD, 9},   {0x0A3C, 7},
  {0x0A4D, 9},   {0x0ABC, 7},   {0x0ACD, 9},   {0x0B3C, 7},
  {0x0B4D, 9},   {0x0BCD, 9},   {0x0C4D, 9},   {0x0C55, 84},
  {0x0C56, 91},  {0x0CBC, 7},   {0x0CCD, 9},   {0x0D4D, 9},
  {0x0DCA, 9},   {0x0E38, 103}, {0x0E39, 103}, {0x0E3A, 9},
  {0x0E48, 107}, {0x0E49, 107}, {0x0E4A, 107}, {0x0E4B, 107},
  {0x0EB8, 118}, {0x0EB9, 118}, {0x0EC8, 122}, {0x0EC9, 122},
  {0x0ECA, 122}, {0x0ECB, 122}, {0x0F18, 220}, {0x0F19, 220},
  {0x0F35, 220}, {0x0F37, 220}, {0x0F39, 216}, {0x0F71, 129},
  {0x0F72, 130}, {0x0F74, 132}, {0x0F7A, 130}, {0x0F7B, 130},
  {0x0F7C, 130}, {0x0F7D, 130}, {0x0F80, 130}, {0x0F82, 230},
  {0x0F83, 230}, {0x0F84, 9},   {0x0F86, 230}, {0x0F87, 230},
  {0x0FC6, 220}, {0x1037, 7},   {0x1039, 9},   {0x103A, 9},
  {0x108D, 220}, {0x135D, 230}, {0x135E, 230}, {0x135F, 230},
  {0x1714, 9},   {0x1734, 9},   {0x17D2, 9},   {0x17DD, 230},
  {0x18A9, 228}, {0x1939, 222}, {0x193A, 230}, {0x193B, 220},
  {0x1A17, 230}, {0x1A18, 220}, {0x1A60, 9},   {0x1A75, 230},
  {0x1A76, 230}, {0x1A77, 230}, {0x1A78, 230}, {0x1A79, 230},
  {0x1A7A, 230}, {0x1A7B, 230}, {0x1A7C, 230}, {0x1A7F, 220},
  {0x1AB0, 230}, {0x1AB1, 230}, {0x1AB2, 230}, {0x1AB3, 230},
  {0x1AB4, 230}, {0x1AB5, 220}, {0x1AB6, 220}, {0x1AB7, 220},
  {0x1AB8, 220}, {0x1AB9, 220}, {0x1ABA, 220}, {0x1ABB, 230},
  {0x1ABC, 230}, {0x1ABD, 220}, {0x1B34, 7},   {0x1B6B, 230},
  {0x1B6C, 220}, {0x1B6D, 230}, {0x1B6E, 230}, {0x1B6F, 230},
  {0x1B70, 230}, {0x1B71, 230}, {0x1B72, 230}, {0x1B73, 230},
  {0x1BAB, 9},   {0x1BE6, 7},   {0x1C37, 7},   {0x1CD0, 230},
  {0x1CD1, 230}, {0x1CD2, 230}, {0x1CD4, 1},   {0x1CD5, 220},
  {0x1CD6, 220}, {0x1CD7, 220}, {0x1CD8, 220}, {0x1CD9, 220},
  {0x1CDA, 230}, {0x1CDB, 230}, {0x1CDC, 220}, {0x1CDD, 220},
  {0x1CDE, 220}, {0x1CDF, 220}, {0x1CE0, 230}, {0x1CE2, 1},
  {0x1CE3, 1},   {0x1CE4, 1},   {0x1CE5, 1},   {0x1CE6, 1},
  {0x1CE7, 1},   {0x1CE8, 1},   {0x1CED, 220}, {0x1CF4, 230},
  {0x1CF8, 230}, {0x1CF9, 230}, {0x1DC0, 230}, {0x1DC1, 230},
  {0x1DC2, 220}, {0x1DC3, 230}, {0x1DC4, 230}, {0x1DC5, 230},
  {0x1DC6, 230}, {0x1DC7, 230}, {0x1DC8, 230}, {0x1DC9, 230},
  {0x1DCA, 220}, {0x1DCB, 230}, {0x1DCC, 230}, {0x1DCD, 234},
  {0x1DCE, 214}, {0x1DCF, 220}, {0x1DD0, 202}, {0x1DD1, 230},
  {0x1DD2, 230}, {0x1DD3, 230}, {0x1DD4, 230}, {0x1DD5, 230},
  {0x1DD6, 230}, {0x1DD7, 230}, {0x1DD8, 230}, {0x1DD9, 230},
  {0x1DDA, 230}, {0x1DDB, 230}, {0x1DDC, 230}, {0x1DDD, 230},
  {0x1DDE, 230}, {0x1DDF, 230}, {0x1DE0, 230}, {0x1DE1, 230},
  {0x1DE2, 230}, {0x1DE3, 230}, {0x1DE4, 230}, {0x1DE5, 230},
  {0x1DE6, 230}, {0x1DE7, 230}, {0x1DE8, 230}, {0x1DE9, 230},
  {0x1DEA, 230}, {0x1DEB, 230}, {0x1DEC, 230}, {0x1DED, 230},
  {0x1DEE, 230}, {0x1DEF, 230}, {0x1DF0, 230}, {0x1DF1, 230},
  {0x1DF2, 230}, {0x1DF3, 230}, {0x1DF4, 230}, {0x1DF5, 230},
  {0x1DFB, 230}, {0x1DFC, 233}, {0x1DFD, 220}, {0x1DFE, 230},
  {0x1DFF, 220}, {0x20D0, 230}, {0x20D1, 230}, {0x20D2, 1},
  {0x20D3, 1},   {0x20D4, 230}, {0x20D5, 230}, {0x20D6, 230},
  {0x20D7, 230}, {0x20D8, 1},   {0x20D9, 1},   {0x20DA, 1},
  {0x20DB, 230}, {0x20DC, 230}, {0x20E1, 230}, {0x20E5, 1},
  {0x20E6, 1},   {0x20E7, 230}, {0x20E8, 220}, {0x20E9, 230},
  {0x20EA, 1},   {0x20EB, 1},   {0x20EC, 220}, {0x20ED, 220},
  {0x20EE, 220}, {0x20EF, 220}, {0x20F0, 230}, {0x2CEF, 230},
  {0x2CF0, 230}, {0x2CF1, 230}, {0x2D7F, 9},   {0x2DE0, 230},
  {0x2DE1, 230}, {0x2DE2, 230}, {0x2DE3, 230}, {0x2DE4, 230},
  {0x2DE5, 230}, {0x2DE6, 230}, {0x2DE7, 230}, {0x2DE8, 230},
  {0x2DE9, 230}, {0x2DEA, 230}, {0x2DEB, 230}, {0x2DEC, 230},
  {0x2DED, 230}, {0x2DEE, 230}, {0x2DEF, 230}, {0x2DF0, 230},
  {0x2DF1, 230}, {0x2DF2, 230}, {0x2DF3, 230}, {0x2DF4, 230},
  {0x2DF5, 230}, {0x2DF6, 230}, {0x2DF7, 230}, {0x2DF8, 230},
  {0x2DF9, 230}, {0x2DFA, 230}, {0x2DFB, 230}, {0x2DFC, 230},
  {0x2DFD, 230}, {0x2DFE, 230}, {0x2DFF, 230}, {0x302A, 218},
  {0x302B, 228}, {0x302C, 232}, {0x302D, 222}, {0x3099, 8},
  {0x309A, 8},   {0xA66F, 230}, {0xA674, 230}, {0xA675, 230},
  {0xA676, 230}, {0xA677, 230}, {0xA678, 230}, {0xA679, 230},
  {0xA67A, 230}, {0xA67B, 230}, {0xA67C, 230}, {0xA67D, 230},
  {0xA69E, 230}, {0xA69F, 230}, {0xA6F0, 230}, {0xA6F1, 230},
  {0xA806, 9},   {0xA8C4, 9},   {0xA8E0, 230}, {0xA8E1, 230},
  {0xA8E2, 230}, {0xA8E3, 230}, {0xA8E4, 230}, {0xA8E5, 230},
  {0xA8E6, 230}, {0xA8E7, 230}, {0xA8E8, 230}, {0xA8E9, 230},
  {0xA8EA, 230}, {0xA8EB, 230}, {0xA8EC, 230}, {0xA8ED, 230},
  {0xA8EE, 230}, {0xA8EF, 230}, {0xA8F0, 230}, {0xA8F1, 230},
  {0xA92B, 220}, {0xA92C, 220}, {0xA92D, 220}, {0xA9B3, 7},
  {0xAAB0, 230}, {0xAAB2, 230}, {0xAAB3, 230}, {0xAAB4, 220},
  {0xAAB7, 230}, {0xAAB8, 230}, {0xAABE, 230}, {0xAABF, 230},
  {0xAAC1, 230}, {0xAAF6, 9},   {0xABED, 9},   {0xFB1E, 26},
  {0xFE20, 230}, {0xFE21, 230}, {0xFE22, 230}, {0xFE23, 230},
  {0xFE24, 230}, {0xFE25, 230}, {0xFE26, 230}, {0xFE27, 220},
  {0xFE28, 220}, {0xFE29, 220}, {0xFE2A, 220}, {0xFE2B, 220},
  {0xFE2C, 220}, {0xFE2D, 220}, {0xFE2E, 230}, {0xFE2F, 230},
  {0x101FD, 220},{0x102E0, 220},{0x10376, 230},{0x10377, 230},
  {0x10378, 230},{0x10379, 230},{0x1037A, 230},{0x10A0D, 220},
  {0x10A0F, 230},{0x10A38, 230},{0x10A39, 1},  {0x10A3A, 220},
  {0x10A3F, 9},  {0x10AE5, 230},{0x10AE6, 220},{0x11046, 9},
  {0x1107F, 9},  {0x110B9, 9},  {0x110BA, 7},  {0x11100, 230},
  {0x11101, 230},{0x11102, 230},{0x11133, 9},  {0x11134, 9},
  {0x11173, 7},  {0x111CA, 7},  {0x11236, 7},  {0x112E9, 7},
  {0x112EA, 9},  {0x1133C, 7},  {0x11366, 230},{0x11367, 230},
  {0x11368, 230},{0x11369, 230},{0x1136A, 230},{0x1136B, 230},
  {0x1136C, 230},{0x11370, 230},{0x11371, 230},{0x11372, 230},
  {0x11373, 230},{0x11374, 230},{0x11442, 9},  {0x11446, 7},
  {0x114C2, 9},  {0x114C3, 7},  {0x115BF, 9},  {0x115C0, 7},
  {0x1163F, 9},  {0x116B7, 7},  {0x1172B, 9},  {0x11C3F, 9},
  {0x16AF0, 1},  {0x16AF1, 1},  {0x16AF2, 1},  {0x16AF3, 1},
  {0x16AF4, 1},  {0x16B30, 230},{0x16B31, 230},{0x16B32, 230},
  {0x16B33, 230},{0x16B34, 230},{0x16B35, 230},{0x16B36, 230},
  {0x1BC9E, 1},  {0x1D167, 1},  {0x1D168, 1},  {0x1D169, 1},
  {0x1D17B, 220},{0x1D17C, 220},{0x1D17D, 220},{0x1D17E, 220},
  {0x1D17F, 220},{0x1D180, 220},{0x1D181, 220},{0x1D182, 220},
  {0x1D185, 230},{0x1D186, 230},{0x1D187, 230},{0x1D188, 230},
  {0x1D189, 230},{0x1D18A, 220},{0x1D18B, 220},{0x1D1AA, 230},
  {0x1D1AB, 230},{0x1D1AC, 230},{0x1D1AD, 230},{0x1D242, 230},
  {0x1D243, 230},{0x1D244, 230},{0x1E000, 230},{0x1E001, 230},
  {0x1E002, 230},{0x1E003, 230},{0x1E004, 230},{0x1E005, 230},
  {0x1E006, 230},{0x1E008, 230},{0x1E009, 230},{0x1E00A, 230},
  {0x1E00B, 230},{0x1E00C, 230},{0x1E00D, 230},{0x1E00E, 230},
  {0x1E00F, 230},{0x1E010, 230},{0x1E011, 230},{0x1E012, 230},
  {0x1E013, 230},{0x1E014, 230},{0x1E015, 230},{0x1E016, 230},
  {0x1E017, 230},{0x1E018, 230},{0x1E01B, 230},{0x1E01C, 230},
  {0x1E01D, 230},{0x1E01E, 230},{0x1E01F, 230},{0x1E020, 230},
  {0x1E021, 230},{0x1E023, 230},{0x1E024, 230},{0x1E026, 230},
  {0x1E027, 230},{0x1E028, 230},{0x1E029, 230},{0x1E02A, 230},
  {0x1E8D0, 220},{0x1E8D1, 220},{0x1E8D2, 220},{0x1E8D3, 220},
  {0x1E8D4, 220},{0x1E8D5, 220},{0x1E8D6, 220},{0x1E944, 230},
  {0x1E945, 230},{0x1E946, 230},{0x1E947, 230},{0x1E948, 230},
  {0x1E949, 230},{0x1E94A, 7}
};

struct Char_grp_info
{
  enum_char_grp    group;
  Weight_boundary  grp_wt_bdy;
};

const uint16 START_WEIGHT_TO_REORDER= 0x1C47;

Char_grp_info char_grp_infos[]= {
  {CHARGRP_CORE,     {0,      0x1C46}}, //space, punctuation, symbol,
                                        //currency, digit and all
                                        //primary ignorable characters
  {CHARGRP_LATIN,    {0x1C47, 0x1FB5}},
  {CHARGRP_CYRILLIC, {0x2022, 0x21E1}},
  {CHARGRP_ARAB,     {0x22FD, 0x23D6}},
  {CHARGRP_KANA,     {0x3D5A, 0x3D8B}}
};

/*
  In UCA 9.0.0, every collation element has three weights.
  To make one character's weight data aligned with those
  who have more collation elements in same page, we have
  to add zero values. To track the real collation element
  number, add one additional number at the start to record it.
  Use the macros below to look up the weights for the first
  collation element for each level.
*/
#define UCA900_NUM_OF_CE(page, subcode) ((page)[(subcode)])
#define UCA900_WEIGHT_ADDR(page, level, subcode) \
  ((page) + 256 + (level) * 256 + (subcode))
#define UCA900_WEIGHT(page, level, subcode) \
  (page)[256 + (level) * 256 + (subcode)]
#define UCA900_DISTANCE_BETWEEN_LEVELS 256
#define UCA900_DISTANCE_BETWEEN_WEIGHTS (MY_UCA_900_CE_SIZE * 256)

uint16 uca900_p000[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 1, 
  2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1, 3, 
  2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0201,   /* U+0009 */
  0x0202,   /* U+000A */
  0x0203,   /* U+000B */
  0x0204,   /* U+000C */
  0x0205,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0209,   /* U+0020 */
  0x0260,   /* U+0021 */
  0x030C,   /* U+0022 */
  0x0398,   /* U+0023 */
  0x1C12,   /* U+0024 */
  0x0399,   /* U+0025 */
  0x0396,   /* U+0026 */
  0x0305,   /* U+0027 */
  0x0317,   /* U+0028 */
  0x0318,   /* U+0029 */
  0x038F,   /* U+002A */
  0x0616,   /* U+002B */
  0x0222,   /* U+002C */
  0x020D,   /* U+002D */
  0x0277,   /* U+002E */
  0x0394,   /* U+002F */
  0x1C3D,   /* U+0030 */
  0x1C3E,   /* U+0031 */
  0x1C3F,   /* U+0032 */
  0x1C40,   /* U+0033 */
  0x1C41,   /* U+0034 */
  0x1C42,   /* U+0035 */
  0x1C43,   /* U+0036 */
  0x1C44,   /* U+0037 */
  0x1C45,   /* U+0038 */
  0x1C46,   /* U+0039 */
  0x0239,   /* U+003A */
  0x0234,   /* U+003B */
  0x061A,   /* U+003C */
  0x061B,   /* U+003D */
  0x061C,   /* U+003E */
  0x0266,   /* U+003F */
  0x038E,   /* U+0040 */
  0x1C47,   /* U+0041 */
  0x1C60,   /* U+0042 */
  0x1C7A,   /* U+0043 */
  0x1C8F,   /* U+0044 */
  0x1CAA,   /* U+0045 */
  0x1CE5,   /* U+0046 */
  0x1CF4,   /* U+0047 */
  0x1D18,   /* U+0048 */
  0x1D32,   /* U+0049 */
  0x1D4C,   /* U+004A */
  0x1D65,   /* U+004B */
  0x1D77,   /* U+004C */
  0x1DAA,   /* U+004D */
  0x1DB9,   /* U+004E */
  0x1DDD,   /* U+004F */
  0x1E0C,   /* U+0050 */
  0x1E21,   /* U+0051 */
  0x1E33,   /* U+0052 */
  0x1E71,   /* U+0053 */
  0x1E95,   /* U+0054 */
  0x1EB5,   /* U+0055 */
  0x1EE3,   /* U+0056 */
  0x1EF5,   /* U+0057 */
  0x1EFF,   /* U+0058 */
  0x1F0B,   /* U+0059 */
  0x1F21,   /* U+005A */
  0x0319,   /* U+005B */
  0x0395,   /* U+005C */
  0x031A,   /* U+005D */
  0x0485,   /* U+005E */
  0x020B,   /* U+005F */
  0x0482,   /* U+0060 */
  0x1C47,   /* U+0061 */
  0x1C60,   /* U+0062 */
  0x1C7A,   /* U+0063 */
  0x1C8F,   /* U+0064 */
  0x1CAA,   /* U+0065 */
  0x1CE5,   /* U+0066 */
  0x1CF4,   /* U+0067 */
  0x1D18,   /* U+0068 */
  0x1D32,   /* U+0069 */
  0x1D4C,   /* U+006A */
  0x1D65,   /* U+006B */
  0x1D77,   /* U+006C */
  0x1DAA,   /* U+006D */
  0x1DB9,   /* U+006E */
  0x1DDD,   /* U+006F */
  0x1E0C,   /* U+0070 */
  0x1E21,   /* U+0071 */
  0x1E33,   /* U+0072 */
  0x1E71,   /* U+0073 */
  0x1E95,   /* U+0074 */
  0x1EB5,   /* U+0075 */
  0x1EE3,   /* U+0076 */
  0x1EF5,   /* U+0077 */
  0x1EFF,   /* U+0078 */
  0x1F0B,   /* U+0079 */
  0x1F21,   /* U+007A */
  0x031B,   /* U+007B */
  0x061E,   /* U+007C */
  0x031C,   /* U+007D */
  0x0620,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0206,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0209,   /* U+00A0 */
  0x0261,   /* U+00A1 */
  0x1C11,   /* U+00A2 */
  0x1C13,   /* U+00A3 */
  0x1C10,   /* U+00A4 */
  0x1C14,   /* U+00A5 */
  0x061F,   /* U+00A6 */
  0x0389,   /* U+00A7 */
  0x0489,   /* U+00A8 */
  0x0584,   /* U+00A9 */
  0x1C47,   /* U+00AA */
  0x0315,   /* U+00AB */
  0x061D,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0585,   /* U+00AE */
  0x0486,   /* U+00AF */
  0x04F6,   /* U+00B0 */
  0x0617,   /* U+00B1 */
  0x1C3F,   /* U+00B2 */
  0x1C40,   /* U+00B3 */
  0x0483,   /* U+00B4 */
  0x1FCB,   /* U+00B5 */
  0x038B,   /* U+00B6 */
  0x028B,   /* U+00B7 */
  0x048C,   /* U+00B8 */
  0x1C3E,   /* U+00B9 */
  0x1DDD,   /* U+00BA */
  0x0316,   /* U+00BB */
  0x1C3E,   /* U+00BC */
  0x1C3E,   /* U+00BD */
  0x1C40,   /* U+00BE */
  0x0267,   /* U+00BF */
  0x1C47,   /* U+00C0 */
  0x1C47,   /* U+00C1 */
  0x1C47,   /* U+00C2 */
  0x1C47,   /* U+00C3 */
  0x1C47,   /* U+00C4 */
  0x1C47,   /* U+00C5 */
  0x1C47,   /* U+00C6 */
  0x1C7A,   /* U+00C7 */
  0x1CAA,   /* U+00C8 */
  0x1CAA,   /* U+00C9 */
  0x1CAA,   /* U+00CA */
  0x1CAA,   /* U+00CB */
  0x1D32,   /* U+00CC */
  0x1D32,   /* U+00CD */
  0x1D32,   /* U+00CE */
  0x1D32,   /* U+00CF */
  0x1C8F,   /* U+00D0 */
  0x1DB9,   /* U+00D1 */
  0x1DDD,   /* U+00D2 */
  0x1DDD,   /* U+00D3 */
  0x1DDD,   /* U+00D4 */
  0x1DDD,   /* U+00D5 */
  0x1DDD,   /* U+00D6 */
  0x0619,   /* U+00D7 */
  0x1DDD,   /* U+00D8 */
  0x1EB5,   /* U+00D9 */
  0x1EB5,   /* U+00DA */
  0x1EB5,   /* U+00DB */
  0x1EB5,   /* U+00DC */
  0x1F0B,   /* U+00DD */
  0x1F50,   /* U+00DE */
  0x1E71,   /* U+00DF */
  0x1C47,   /* U+00E0 */
  0x1C47,   /* U+00E1 */
  0x1C47,   /* U+00E2 */
  0x1C47,   /* U+00E3 */
  0x1C47,   /* U+00E4 */
  0x1C47,   /* U+00E5 */
  0x1C47,   /* U+00E6 */
  0x1C7A,   /* U+00E7 */
  0x1CAA,   /* U+00E8 */
  0x1CAA,   /* U+00E9 */
  0x1CAA,   /* U+00EA */
  0x1CAA,   /* U+00EB */
  0x1D32,   /* U+00EC */
  0x1D32,   /* U+00ED */
  0x1D32,   /* U+00EE */
  0x1D32,   /* U+00EF */
  0x1C8F,   /* U+00F0 */
  0x1DB9,   /* U+00F1 */
  0x1DDD,   /* U+00F2 */
  0x1DDD,   /* U+00F3 */
  0x1DDD,   /* U+00F4 */
  0x1DDD,   /* U+00F5 */
  0x1DDD,   /* U+00F6 */
  0x0618,   /* U+00F7 */
  0x1DDD,   /* U+00F8 */
  0x1EB5,   /* U+00F9 */
  0x1EB5,   /* U+00FA */
  0x1EB5,   /* U+00FB */
  0x1EB5,   /* U+00FC */
  0x1F0B,   /* U+00FD */
  0x1F50,   /* U+00FE */
  0x1F0B,   /* U+00FF */

  /* Secondary weight 1 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0020,   /* U+0009 */
  0x0020,   /* U+000A */
  0x0020,   /* U+000B */
  0x0020,   /* U+000C */
  0x0020,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0020,   /* U+0020 */
  0x0020,   /* U+0021 */
  0x0020,   /* U+0022 */
  0x0020,   /* U+0023 */
  0x0020,   /* U+0024 */
  0x0020,   /* U+0025 */
  0x0020,   /* U+0026 */
  0x0020,   /* U+0027 */
  0x0020,   /* U+0028 */
  0x0020,   /* U+0029 */
  0x0020,   /* U+002A */
  0x0020,   /* U+002B */
  0x0020,   /* U+002C */
  0x0020,   /* U+002D */
  0x0020,   /* U+002E */
  0x0020,   /* U+002F */
  0x0020,   /* U+0030 */
  0x0020,   /* U+0031 */
  0x0020,   /* U+0032 */
  0x0020,   /* U+0033 */
  0x0020,   /* U+0034 */
  0x0020,   /* U+0035 */
  0x0020,   /* U+0036 */
  0x0020,   /* U+0037 */
  0x0020,   /* U+0038 */
  0x0020,   /* U+0039 */
  0x0020,   /* U+003A */
  0x0020,   /* U+003B */
  0x0020,   /* U+003C */
  0x0020,   /* U+003D */
  0x0020,   /* U+003E */
  0x0020,   /* U+003F */
  0x0020,   /* U+0040 */
  0x0020,   /* U+0041 */
  0x0020,   /* U+0042 */
  0x0020,   /* U+0043 */
  0x0020,   /* U+0044 */
  0x0020,   /* U+0045 */
  0x0020,   /* U+0046 */
  0x0020,   /* U+0047 */
  0x0020,   /* U+0048 */
  0x0020,   /* U+0049 */
  0x0020,   /* U+004A */
  0x0020,   /* U+004B */
  0x0020,   /* U+004C */
  0x0020,   /* U+004D */
  0x0020,   /* U+004E */
  0x0020,   /* U+004F */
  0x0020,   /* U+0050 */
  0x0020,   /* U+0051 */
  0x0020,   /* U+0052 */
  0x0020,   /* U+0053 */
  0x0020,   /* U+0054 */
  0x0020,   /* U+0055 */
  0x0020,   /* U+0056 */
  0x0020,   /* U+0057 */
  0x0020,   /* U+0058 */
  0x0020,   /* U+0059 */
  0x0020,   /* U+005A */
  0x0020,   /* U+005B */
  0x0020,   /* U+005C */
  0x0020,   /* U+005D */
  0x0020,   /* U+005E */
  0x0020,   /* U+005F */
  0x0020,   /* U+0060 */
  0x0020,   /* U+0061 */
  0x0020,   /* U+0062 */
  0x0020,   /* U+0063 */
  0x0020,   /* U+0064 */
  0x0020,   /* U+0065 */
  0x0020,   /* U+0066 */
  0x0020,   /* U+0067 */
  0x0020,   /* U+0068 */
  0x0020,   /* U+0069 */
  0x0020,   /* U+006A */
  0x0020,   /* U+006B */
  0x0020,   /* U+006C */
  0x0020,   /* U+006D */
  0x0020,   /* U+006E */
  0x0020,   /* U+006F */
  0x0020,   /* U+0070 */
  0x0020,   /* U+0071 */
  0x0020,   /* U+0072 */
  0x0020,   /* U+0073 */
  0x0020,   /* U+0074 */
  0x0020,   /* U+0075 */
  0x0020,   /* U+0076 */
  0x0020,   /* U+0077 */
  0x0020,   /* U+0078 */
  0x0020,   /* U+0079 */
  0x0020,   /* U+007A */
  0x0020,   /* U+007B */
  0x0020,   /* U+007C */
  0x0020,   /* U+007D */
  0x0020,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0020,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0020,   /* U+00A0 */
  0x0020,   /* U+00A1 */
  0x0020,   /* U+00A2 */
  0x0020,   /* U+00A3 */
  0x0020,   /* U+00A4 */
  0x0020,   /* U+00A5 */
  0x0020,   /* U+00A6 */
  0x0020,   /* U+00A7 */
  0x0020,   /* U+00A8 */
  0x0020,   /* U+00A9 */
  0x0020,   /* U+00AA */
  0x0020,   /* U+00AB */
  0x0020,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0020,   /* U+00AE */
  0x0020,   /* U+00AF */
  0x0020,   /* U+00B0 */
  0x0020,   /* U+00B1 */
  0x0020,   /* U+00B2 */
  0x0020,   /* U+00B3 */
  0x0020,   /* U+00B4 */
  0x0020,   /* U+00B5 */
  0x0020,   /* U+00B6 */
  0x0020,   /* U+00B7 */
  0x0020,   /* U+00B8 */
  0x0020,   /* U+00B9 */
  0x0020,   /* U+00BA */
  0x0020,   /* U+00BB */
  0x0020,   /* U+00BC */
  0x0020,   /* U+00BD */
  0x0020,   /* U+00BE */
  0x0020,   /* U+00BF */
  0x0020,   /* U+00C0 */
  0x0020,   /* U+00C1 */
  0x0020,   /* U+00C2 */
  0x0020,   /* U+00C3 */
  0x0020,   /* U+00C4 */
  0x0020,   /* U+00C5 */
  0x0020,   /* U+00C6 */
  0x0020,   /* U+00C7 */
  0x0020,   /* U+00C8 */
  0x0020,   /* U+00C9 */
  0x0020,   /* U+00CA */
  0x0020,   /* U+00CB */
  0x0020,   /* U+00CC */
  0x0020,   /* U+00CD */
  0x0020,   /* U+00CE */
  0x0020,   /* U+00CF */
  0x0020,   /* U+00D0 */
  0x0020,   /* U+00D1 */
  0x0020,   /* U+00D2 */
  0x0020,   /* U+00D3 */
  0x0020,   /* U+00D4 */
  0x0020,   /* U+00D5 */
  0x0020,   /* U+00D6 */
  0x0020,   /* U+00D7 */
  0x0020,   /* U+00D8 */
  0x0020,   /* U+00D9 */
  0x0020,   /* U+00DA */
  0x0020,   /* U+00DB */
  0x0020,   /* U+00DC */
  0x0020,   /* U+00DD */
  0x0020,   /* U+00DE */
  0x0020,   /* U+00DF */
  0x0020,   /* U+00E0 */
  0x0020,   /* U+00E1 */
  0x0020,   /* U+00E2 */
  0x0020,   /* U+00E3 */
  0x0020,   /* U+00E4 */
  0x0020,   /* U+00E5 */
  0x0020,   /* U+00E6 */
  0x0020,   /* U+00E7 */
  0x0020,   /* U+00E8 */
  0x0020,   /* U+00E9 */
  0x0020,   /* U+00EA */
  0x0020,   /* U+00EB */
  0x0020,   /* U+00EC */
  0x0020,   /* U+00ED */
  0x0020,   /* U+00EE */
  0x0020,   /* U+00EF */
  0x0020,   /* U+00F0 */
  0x0020,   /* U+00F1 */
  0x0020,   /* U+00F2 */
  0x0020,   /* U+00F3 */
  0x0020,   /* U+00F4 */
  0x0020,   /* U+00F5 */
  0x0020,   /* U+00F6 */
  0x0020,   /* U+00F7 */
  0x0020,   /* U+00F8 */
  0x0020,   /* U+00F9 */
  0x0020,   /* U+00FA */
  0x0020,   /* U+00FB */
  0x0020,   /* U+00FC */
  0x0020,   /* U+00FD */
  0x0020,   /* U+00FE */
  0x0020,   /* U+00FF */

  /* Tertiary weight 1 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0002,   /* U+0009 */
  0x0002,   /* U+000A */
  0x0002,   /* U+000B */
  0x0002,   /* U+000C */
  0x0002,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0002,   /* U+0020 */
  0x0002,   /* U+0021 */
  0x0002,   /* U+0022 */
  0x0002,   /* U+0023 */
  0x0002,   /* U+0024 */
  0x0002,   /* U+0025 */
  0x0002,   /* U+0026 */
  0x0002,   /* U+0027 */
  0x0002,   /* U+0028 */
  0x0002,   /* U+0029 */
  0x0002,   /* U+002A */
  0x0002,   /* U+002B */
  0x0002,   /* U+002C */
  0x0002,   /* U+002D */
  0x0002,   /* U+002E */
  0x0002,   /* U+002F */
  0x0002,   /* U+0030 */
  0x0002,   /* U+0031 */
  0x0002,   /* U+0032 */
  0x0002,   /* U+0033 */
  0x0002,   /* U+0034 */
  0x0002,   /* U+0035 */
  0x0002,   /* U+0036 */
  0x0002,   /* U+0037 */
  0x0002,   /* U+0038 */
  0x0002,   /* U+0039 */
  0x0002,   /* U+003A */
  0x0002,   /* U+003B */
  0x0002,   /* U+003C */
  0x0002,   /* U+003D */
  0x0002,   /* U+003E */
  0x0002,   /* U+003F */
  0x0002,   /* U+0040 */
  0x0008,   /* U+0041 */
  0x0008,   /* U+0042 */
  0x0008,   /* U+0043 */
  0x0008,   /* U+0044 */
  0x0008,   /* U+0045 */
  0x0008,   /* U+0046 */
  0x0008,   /* U+0047 */
  0x0008,   /* U+0048 */
  0x0008,   /* U+0049 */
  0x0008,   /* U+004A */
  0x0008,   /* U+004B */
  0x0008,   /* U+004C */
  0x0008,   /* U+004D */
  0x0008,   /* U+004E */
  0x0008,   /* U+004F */
  0x0008,   /* U+0050 */
  0x0008,   /* U+0051 */
  0x0008,   /* U+0052 */
  0x0008,   /* U+0053 */
  0x0008,   /* U+0054 */
  0x0008,   /* U+0055 */
  0x0008,   /* U+0056 */
  0x0008,   /* U+0057 */
  0x0008,   /* U+0058 */
  0x0008,   /* U+0059 */
  0x0008,   /* U+005A */
  0x0002,   /* U+005B */
  0x0002,   /* U+005C */
  0x0002,   /* U+005D */
  0x0002,   /* U+005E */
  0x0002,   /* U+005F */
  0x0002,   /* U+0060 */
  0x0002,   /* U+0061 */
  0x0002,   /* U+0062 */
  0x0002,   /* U+0063 */
  0x0002,   /* U+0064 */
  0x0002,   /* U+0065 */
  0x0002,   /* U+0066 */
  0x0002,   /* U+0067 */
  0x0002,   /* U+0068 */
  0x0002,   /* U+0069 */
  0x0002,   /* U+006A */
  0x0002,   /* U+006B */
  0x0002,   /* U+006C */
  0x0002,   /* U+006D */
  0x0002,   /* U+006E */
  0x0002,   /* U+006F */
  0x0002,   /* U+0070 */
  0x0002,   /* U+0071 */
  0x0002,   /* U+0072 */
  0x0002,   /* U+0073 */
  0x0002,   /* U+0074 */
  0x0002,   /* U+0075 */
  0x0002,   /* U+0076 */
  0x0002,   /* U+0077 */
  0x0002,   /* U+0078 */
  0x0002,   /* U+0079 */
  0x0002,   /* U+007A */
  0x0002,   /* U+007B */
  0x0002,   /* U+007C */
  0x0002,   /* U+007D */
  0x0002,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0002,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x001B,   /* U+00A0 */
  0x0002,   /* U+00A1 */
  0x0002,   /* U+00A2 */
  0x0002,   /* U+00A3 */
  0x0002,   /* U+00A4 */
  0x0002,   /* U+00A5 */
  0x0002,   /* U+00A6 */
  0x0002,   /* U+00A7 */
  0x0002,   /* U+00A8 */
  0x0002,   /* U+00A9 */
  0x0014,   /* U+00AA */
  0x0002,   /* U+00AB */
  0x0002,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0002,   /* U+00AE */
  0x0002,   /* U+00AF */
  0x0002,   /* U+00B0 */
  0x0002,   /* U+00B1 */
  0x0014,   /* U+00B2 */
  0x0014,   /* U+00B3 */
  0x0002,   /* U+00B4 */
  0x0004,   /* U+00B5 */
  0x0002,   /* U+00B6 */
  0x0002,   /* U+00B7 */
  0x0002,   /* U+00B8 */
  0x0014,   /* U+00B9 */
  0x0014,   /* U+00BA */
  0x0002,   /* U+00BB */
  0x001E,   /* U+00BC */
  0x001E,   /* U+00BD */
  0x001E,   /* U+00BE */
  0x0002,   /* U+00BF */
  0x0008,   /* U+00C0 */
  0x0008,   /* U+00C1 */
  0x0008,   /* U+00C2 */
  0x0008,   /* U+00C3 */
  0x0008,   /* U+00C4 */
  0x0008,   /* U+00C5 */
  0x000A,   /* U+00C6 */
  0x0008,   /* U+00C7 */
  0x0008,   /* U+00C8 */
  0x0008,   /* U+00C9 */
  0x0008,   /* U+00CA */
  0x0008,   /* U+00CB */
  0x0008,   /* U+00CC */
  0x0008,   /* U+00CD */
  0x0008,   /* U+00CE */
  0x0008,   /* U+00CF */
  0x000A,   /* U+00D0 */
  0x0008,   /* U+00D1 */
  0x0008,   /* U+00D2 */
  0x0008,   /* U+00D3 */
  0x0008,   /* U+00D4 */
  0x0008,   /* U+00D5 */
  0x0008,   /* U+00D6 */
  0x0002,   /* U+00D7 */
  0x0008,   /* U+00D8 */
  0x0008,   /* U+00D9 */
  0x0008,   /* U+00DA */
  0x0008,   /* U+00DB */
  0x0008,   /* U+00DC */
  0x0008,   /* U+00DD */
  0x0008,   /* U+00DE */
  0x0004,   /* U+00DF */
  0x0002,   /* U+00E0 */
  0x0002,   /* U+00E1 */
  0x0002,   /* U+00E2 */
  0x0002,   /* U+00E3 */
  0x0002,   /* U+00E4 */
  0x0002,   /* U+00E5 */
  0x0004,   /* U+00E6 */
  0x0002,   /* U+00E7 */
  0x0002,   /* U+00E8 */
  0x0002,   /* U+00E9 */
  0x0002,   /* U+00EA */
  0x0002,   /* U+00EB */
  0x0002,   /* U+00EC */
  0x0002,   /* U+00ED */
  0x0002,   /* U+00EE */
  0x0002,   /* U+00EF */
  0x0004,   /* U+00F0 */
  0x0002,   /* U+00F1 */
  0x0002,   /* U+00F2 */
  0x0002,   /* U+00F3 */
  0x0002,   /* U+00F4 */
  0x0002,   /* U+00F5 */
  0x0002,   /* U+00F6 */
  0x0002,   /* U+00F7 */
  0x0002,   /* U+00F8 */
  0x0002,   /* U+00F9 */
  0x0002,   /* U+00FA */
  0x0002,   /* U+00FB */
  0x0002,   /* U+00FC */
  0x0002,   /* U+00FD */
  0x0002,   /* U+00FE */
  0x0002,   /* U+00FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x0626,   /* U+00BC */
  0x0626,   /* U+00BD */
  0x0626,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0000,   /* U+00C0 */
  0x0000,   /* U+00C1 */
  0x0000,   /* U+00C2 */
  0x0000,   /* U+00C3 */
  0x0000,   /* U+00C4 */
  0x0000,   /* U+00C5 */
  0x0000,   /* U+00C6 */
  0x0000,   /* U+00C7 */
  0x0000,   /* U+00C8 */
  0x0000,   /* U+00C9 */
  0x0000,   /* U+00CA */
  0x0000,   /* U+00CB */
  0x0000,   /* U+00CC */
  0x0000,   /* U+00CD */
  0x0000,   /* U+00CE */
  0x0000,   /* U+00CF */
  0x0000,   /* U+00D0 */
  0x0000,   /* U+00D1 */
  0x0000,   /* U+00D2 */
  0x0000,   /* U+00D3 */
  0x0000,   /* U+00D4 */
  0x0000,   /* U+00D5 */
  0x0000,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x0000,   /* U+00D8 */
  0x0000,   /* U+00D9 */
  0x0000,   /* U+00DA */
  0x0000,   /* U+00DB */
  0x0000,   /* U+00DC */
  0x0000,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x0000,   /* U+00DF */
  0x0000,   /* U+00E0 */
  0x0000,   /* U+00E1 */
  0x0000,   /* U+00E2 */
  0x0000,   /* U+00E3 */
  0x0000,   /* U+00E4 */
  0x0000,   /* U+00E5 */
  0x0000,   /* U+00E6 */
  0x0000,   /* U+00E7 */
  0x0000,   /* U+00E8 */
  0x0000,   /* U+00E9 */
  0x0000,   /* U+00EA */
  0x0000,   /* U+00EB */
  0x0000,   /* U+00EC */
  0x0000,   /* U+00ED */
  0x0000,   /* U+00EE */
  0x0000,   /* U+00EF */
  0x0000,   /* U+00F0 */
  0x0000,   /* U+00F1 */
  0x0000,   /* U+00F2 */
  0x0000,   /* U+00F3 */
  0x0000,   /* U+00F4 */
  0x0000,   /* U+00F5 */
  0x0000,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x0000,   /* U+00F8 */
  0x0000,   /* U+00F9 */
  0x0000,   /* U+00FA */
  0x0000,   /* U+00FB */
  0x0000,   /* U+00FC */
  0x0000,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x0000,   /* U+00FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x0020,   /* U+00BC */
  0x0020,   /* U+00BD */
  0x0020,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0025,   /* U+00C0 */
  0x0024,   /* U+00C1 */
  0x0027,   /* U+00C2 */
  0x002D,   /* U+00C3 */
  0x002B,   /* U+00C4 */
  0x0029,   /* U+00C5 */
  0x0110,   /* U+00C6 */
  0x0030,   /* U+00C7 */
  0x0025,   /* U+00C8 */
  0x0024,   /* U+00C9 */
  0x0027,   /* U+00CA */
  0x002B,   /* U+00CB */
  0x0025,   /* U+00CC */
  0x0024,   /* U+00CD */
  0x0027,   /* U+00CE */
  0x002B,   /* U+00CF */
  0x0110,   /* U+00D0 */
  0x002D,   /* U+00D1 */
  0x0025,   /* U+00D2 */
  0x0024,   /* U+00D3 */
  0x0027,   /* U+00D4 */
  0x002D,   /* U+00D5 */
  0x002B,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x002F,   /* U+00D8 */
  0x0025,   /* U+00D9 */
  0x0024,   /* U+00DA */
  0x0027,   /* U+00DB */
  0x002B,   /* U+00DC */
  0x0024,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x0110,   /* U+00DF */
  0x0025,   /* U+00E0 */
  0x0024,   /* U+00E1 */
  0x0027,   /* U+00E2 */
  0x002D,   /* U+00E3 */
  0x002B,   /* U+00E4 */
  0x0029,   /* U+00E5 */
  0x0110,   /* U+00E6 */
  0x0030,   /* U+00E7 */
  0x0025,   /* U+00E8 */
  0x0024,   /* U+00E9 */
  0x0027,   /* U+00EA */
  0x002B,   /* U+00EB */
  0x0025,   /* U+00EC */
  0x0024,   /* U+00ED */
  0x0027,   /* U+00EE */
  0x002B,   /* U+00EF */
  0x0110,   /* U+00F0 */
  0x002D,   /* U+00F1 */
  0x0025,   /* U+00F2 */
  0x0024,   /* U+00F3 */
  0x0027,   /* U+00F4 */
  0x002D,   /* U+00F5 */
  0x002B,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x002F,   /* U+00F8 */
  0x0025,   /* U+00F9 */
  0x0024,   /* U+00FA */
  0x0027,   /* U+00FB */
  0x002B,   /* U+00FC */
  0x0024,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x002B,   /* U+00FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x001E,   /* U+00BC */
  0x001E,   /* U+00BD */
  0x001E,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0002,   /* U+00C0 */
  0x0002,   /* U+00C1 */
  0x0002,   /* U+00C2 */
  0x0002,   /* U+00C3 */
  0x0002,   /* U+00C4 */
  0x0002,   /* U+00C5 */
  0x0004,   /* U+00C6 */
  0x0002,   /* U+00C7 */
  0x0002,   /* U+00C8 */
  0x0002,   /* U+00C9 */
  0x0002,   /* U+00CA */
  0x0002,   /* U+00CB */
  0x0002,   /* U+00CC */
  0x0002,   /* U+00CD */
  0x0002,   /* U+00CE */
  0x0002,   /* U+00CF */
  0x0004,   /* U+00D0 */
  0x0002,   /* U+00D1 */
  0x0002,   /* U+00D2 */
  0x0002,   /* U+00D3 */
  0x0002,   /* U+00D4 */
  0x0002,   /* U+00D5 */
  0x0002,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x0002,   /* U+00D8 */
  0x0002,   /* U+00D9 */
  0x0002,   /* U+00DA */
  0x0002,   /* U+00DB */
  0x0002,   /* U+00DC */
  0x0002,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x0004,   /* U+00DF */
  0x0002,   /* U+00E0 */
  0x0002,   /* U+00E1 */
  0x0002,   /* U+00E2 */
  0x0002,   /* U+00E3 */
  0x0002,   /* U+00E4 */
  0x0002,   /* U+00E5 */
  0x0004,   /* U+00E6 */
  0x0002,   /* U+00E7 */
  0x0002,   /* U+00E8 */
  0x0002,   /* U+00E9 */
  0x0002,   /* U+00EA */
  0x0002,   /* U+00EB */
  0x0002,   /* U+00EC */
  0x0002,   /* U+00ED */
  0x0002,   /* U+00EE */
  0x0002,   /* U+00EF */
  0x0004,   /* U+00F0 */
  0x0002,   /* U+00F1 */
  0x0002,   /* U+00F2 */
  0x0002,   /* U+00F3 */
  0x0002,   /* U+00F4 */
  0x0002,   /* U+00F5 */
  0x0002,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x0002,   /* U+00F8 */
  0x0002,   /* U+00F9 */
  0x0002,   /* U+00FA */
  0x0002,   /* U+00FB */
  0x0002,   /* U+00FC */
  0x0002,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x0002,   /* U+00FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x1C41,   /* U+00BC */
  0x1C3F,   /* U+00BD */
  0x1C41,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0000,   /* U+00C0 */
  0x0000,   /* U+00C1 */
  0x0000,   /* U+00C2 */
  0x0000,   /* U+00C3 */
  0x0000,   /* U+00C4 */
  0x0000,   /* U+00C5 */
  0x1CAA,   /* U+00C6 */
  0x0000,   /* U+00C7 */
  0x0000,   /* U+00C8 */
  0x0000,   /* U+00C9 */
  0x0000,   /* U+00CA */
  0x0000,   /* U+00CB */
  0x0000,   /* U+00CC */
  0x0000,   /* U+00CD */
  0x0000,   /* U+00CE */
  0x0000,   /* U+00CF */
  0x0000,   /* U+00D0 */
  0x0000,   /* U+00D1 */
  0x0000,   /* U+00D2 */
  0x0000,   /* U+00D3 */
  0x0000,   /* U+00D4 */
  0x0000,   /* U+00D5 */
  0x0000,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x0000,   /* U+00D8 */
  0x0000,   /* U+00D9 */
  0x0000,   /* U+00DA */
  0x0000,   /* U+00DB */
  0x0000,   /* U+00DC */
  0x0000,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x1E71,   /* U+00DF */
  0x0000,   /* U+00E0 */
  0x0000,   /* U+00E1 */
  0x0000,   /* U+00E2 */
  0x0000,   /* U+00E3 */
  0x0000,   /* U+00E4 */
  0x0000,   /* U+00E5 */
  0x1CAA,   /* U+00E6 */
  0x0000,   /* U+00E7 */
  0x0000,   /* U+00E8 */
  0x0000,   /* U+00E9 */
  0x0000,   /* U+00EA */
  0x0000,   /* U+00EB */
  0x0000,   /* U+00EC */
  0x0000,   /* U+00ED */
  0x0000,   /* U+00EE */
  0x0000,   /* U+00EF */
  0x0000,   /* U+00F0 */
  0x0000,   /* U+00F1 */
  0x0000,   /* U+00F2 */
  0x0000,   /* U+00F3 */
  0x0000,   /* U+00F4 */
  0x0000,   /* U+00F5 */
  0x0000,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x0000,   /* U+00F8 */
  0x0000,   /* U+00F9 */
  0x0000,   /* U+00FA */
  0x0000,   /* U+00FB */
  0x0000,   /* U+00FC */
  0x0000,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x0000,   /* U+00FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x0020,   /* U+00BC */
  0x0020,   /* U+00BD */
  0x0020,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0000,   /* U+00C0 */
  0x0000,   /* U+00C1 */
  0x0000,   /* U+00C2 */
  0x0000,   /* U+00C3 */
  0x0000,   /* U+00C4 */
  0x0000,   /* U+00C5 */
  0x0020,   /* U+00C6 */
  0x0000,   /* U+00C7 */
  0x0000,   /* U+00C8 */
  0x0000,   /* U+00C9 */
  0x0000,   /* U+00CA */
  0x0000,   /* U+00CB */
  0x0000,   /* U+00CC */
  0x0000,   /* U+00CD */
  0x0000,   /* U+00CE */
  0x0000,   /* U+00CF */
  0x0000,   /* U+00D0 */
  0x0000,   /* U+00D1 */
  0x0000,   /* U+00D2 */
  0x0000,   /* U+00D3 */
  0x0000,   /* U+00D4 */
  0x0000,   /* U+00D5 */
  0x0000,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x0000,   /* U+00D8 */
  0x0000,   /* U+00D9 */
  0x0000,   /* U+00DA */
  0x0000,   /* U+00DB */
  0x0000,   /* U+00DC */
  0x0000,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x0020,   /* U+00DF */
  0x0000,   /* U+00E0 */
  0x0000,   /* U+00E1 */
  0x0000,   /* U+00E2 */
  0x0000,   /* U+00E3 */
  0x0000,   /* U+00E4 */
  0x0000,   /* U+00E5 */
  0x0020,   /* U+00E6 */
  0x0000,   /* U+00E7 */
  0x0000,   /* U+00E8 */
  0x0000,   /* U+00E9 */
  0x0000,   /* U+00EA */
  0x0000,   /* U+00EB */
  0x0000,   /* U+00EC */
  0x0000,   /* U+00ED */
  0x0000,   /* U+00EE */
  0x0000,   /* U+00EF */
  0x0000,   /* U+00F0 */
  0x0000,   /* U+00F1 */
  0x0000,   /* U+00F2 */
  0x0000,   /* U+00F3 */
  0x0000,   /* U+00F4 */
  0x0000,   /* U+00F5 */
  0x0000,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x0000,   /* U+00F8 */
  0x0000,   /* U+00F9 */
  0x0000,   /* U+00FA */
  0x0000,   /* U+00FB */
  0x0000,   /* U+00FC */
  0x0000,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x0000,   /* U+00FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+0000 */
  0x0000,   /* U+0001 */
  0x0000,   /* U+0002 */
  0x0000,   /* U+0003 */
  0x0000,   /* U+0004 */
  0x0000,   /* U+0005 */
  0x0000,   /* U+0006 */
  0x0000,   /* U+0007 */
  0x0000,   /* U+0008 */
  0x0000,   /* U+0009 */
  0x0000,   /* U+000A */
  0x0000,   /* U+000B */
  0x0000,   /* U+000C */
  0x0000,   /* U+000D */
  0x0000,   /* U+000E */
  0x0000,   /* U+000F */
  0x0000,   /* U+0010 */
  0x0000,   /* U+0011 */
  0x0000,   /* U+0012 */
  0x0000,   /* U+0013 */
  0x0000,   /* U+0014 */
  0x0000,   /* U+0015 */
  0x0000,   /* U+0016 */
  0x0000,   /* U+0017 */
  0x0000,   /* U+0018 */
  0x0000,   /* U+0019 */
  0x0000,   /* U+001A */
  0x0000,   /* U+001B */
  0x0000,   /* U+001C */
  0x0000,   /* U+001D */
  0x0000,   /* U+001E */
  0x0000,   /* U+001F */
  0x0000,   /* U+0020 */
  0x0000,   /* U+0021 */
  0x0000,   /* U+0022 */
  0x0000,   /* U+0023 */
  0x0000,   /* U+0024 */
  0x0000,   /* U+0025 */
  0x0000,   /* U+0026 */
  0x0000,   /* U+0027 */
  0x0000,   /* U+0028 */
  0x0000,   /* U+0029 */
  0x0000,   /* U+002A */
  0x0000,   /* U+002B */
  0x0000,   /* U+002C */
  0x0000,   /* U+002D */
  0x0000,   /* U+002E */
  0x0000,   /* U+002F */
  0x0000,   /* U+0030 */
  0x0000,   /* U+0031 */
  0x0000,   /* U+0032 */
  0x0000,   /* U+0033 */
  0x0000,   /* U+0034 */
  0x0000,   /* U+0035 */
  0x0000,   /* U+0036 */
  0x0000,   /* U+0037 */
  0x0000,   /* U+0038 */
  0x0000,   /* U+0039 */
  0x0000,   /* U+003A */
  0x0000,   /* U+003B */
  0x0000,   /* U+003C */
  0x0000,   /* U+003D */
  0x0000,   /* U+003E */
  0x0000,   /* U+003F */
  0x0000,   /* U+0040 */
  0x0000,   /* U+0041 */
  0x0000,   /* U+0042 */
  0x0000,   /* U+0043 */
  0x0000,   /* U+0044 */
  0x0000,   /* U+0045 */
  0x0000,   /* U+0046 */
  0x0000,   /* U+0047 */
  0x0000,   /* U+0048 */
  0x0000,   /* U+0049 */
  0x0000,   /* U+004A */
  0x0000,   /* U+004B */
  0x0000,   /* U+004C */
  0x0000,   /* U+004D */
  0x0000,   /* U+004E */
  0x0000,   /* U+004F */
  0x0000,   /* U+0050 */
  0x0000,   /* U+0051 */
  0x0000,   /* U+0052 */
  0x0000,   /* U+0053 */
  0x0000,   /* U+0054 */
  0x0000,   /* U+0055 */
  0x0000,   /* U+0056 */
  0x0000,   /* U+0057 */
  0x0000,   /* U+0058 */
  0x0000,   /* U+0059 */
  0x0000,   /* U+005A */
  0x0000,   /* U+005B */
  0x0000,   /* U+005C */
  0x0000,   /* U+005D */
  0x0000,   /* U+005E */
  0x0000,   /* U+005F */
  0x0000,   /* U+0060 */
  0x0000,   /* U+0061 */
  0x0000,   /* U+0062 */
  0x0000,   /* U+0063 */
  0x0000,   /* U+0064 */
  0x0000,   /* U+0065 */
  0x0000,   /* U+0066 */
  0x0000,   /* U+0067 */
  0x0000,   /* U+0068 */
  0x0000,   /* U+0069 */
  0x0000,   /* U+006A */
  0x0000,   /* U+006B */
  0x0000,   /* U+006C */
  0x0000,   /* U+006D */
  0x0000,   /* U+006E */
  0x0000,   /* U+006F */
  0x0000,   /* U+0070 */
  0x0000,   /* U+0071 */
  0x0000,   /* U+0072 */
  0x0000,   /* U+0073 */
  0x0000,   /* U+0074 */
  0x0000,   /* U+0075 */
  0x0000,   /* U+0076 */
  0x0000,   /* U+0077 */
  0x0000,   /* U+0078 */
  0x0000,   /* U+0079 */
  0x0000,   /* U+007A */
  0x0000,   /* U+007B */
  0x0000,   /* U+007C */
  0x0000,   /* U+007D */
  0x0000,   /* U+007E */
  0x0000,   /* U+007F */
  0x0000,   /* U+0080 */
  0x0000,   /* U+0081 */
  0x0000,   /* U+0082 */
  0x0000,   /* U+0083 */
  0x0000,   /* U+0084 */
  0x0000,   /* U+0085 */
  0x0000,   /* U+0086 */
  0x0000,   /* U+0087 */
  0x0000,   /* U+0088 */
  0x0000,   /* U+0089 */
  0x0000,   /* U+008A */
  0x0000,   /* U+008B */
  0x0000,   /* U+008C */
  0x0000,   /* U+008D */
  0x0000,   /* U+008E */
  0x0000,   /* U+008F */
  0x0000,   /* U+0090 */
  0x0000,   /* U+0091 */
  0x0000,   /* U+0092 */
  0x0000,   /* U+0093 */
  0x0000,   /* U+0094 */
  0x0000,   /* U+0095 */
  0x0000,   /* U+0096 */
  0x0000,   /* U+0097 */
  0x0000,   /* U+0098 */
  0x0000,   /* U+0099 */
  0x0000,   /* U+009A */
  0x0000,   /* U+009B */
  0x0000,   /* U+009C */
  0x0000,   /* U+009D */
  0x0000,   /* U+009E */
  0x0000,   /* U+009F */
  0x0000,   /* U+00A0 */
  0x0000,   /* U+00A1 */
  0x0000,   /* U+00A2 */
  0x0000,   /* U+00A3 */
  0x0000,   /* U+00A4 */
  0x0000,   /* U+00A5 */
  0x0000,   /* U+00A6 */
  0x0000,   /* U+00A7 */
  0x0000,   /* U+00A8 */
  0x0000,   /* U+00A9 */
  0x0000,   /* U+00AA */
  0x0000,   /* U+00AB */
  0x0000,   /* U+00AC */
  0x0000,   /* U+00AD */
  0x0000,   /* U+00AE */
  0x0000,   /* U+00AF */
  0x0000,   /* U+00B0 */
  0x0000,   /* U+00B1 */
  0x0000,   /* U+00B2 */
  0x0000,   /* U+00B3 */
  0x0000,   /* U+00B4 */
  0x0000,   /* U+00B5 */
  0x0000,   /* U+00B6 */
  0x0000,   /* U+00B7 */
  0x0000,   /* U+00B8 */
  0x0000,   /* U+00B9 */
  0x0000,   /* U+00BA */
  0x0000,   /* U+00BB */
  0x001E,   /* U+00BC */
  0x001E,   /* U+00BD */
  0x001E,   /* U+00BE */
  0x0000,   /* U+00BF */
  0x0000,   /* U+00C0 */
  0x0000,   /* U+00C1 */
  0x0000,   /* U+00C2 */
  0x0000,   /* U+00C3 */
  0x0000,   /* U+00C4 */
  0x0000,   /* U+00C5 */
  0x000A,   /* U+00C6 */
  0x0000,   /* U+00C7 */
  0x0000,   /* U+00C8 */
  0x0000,   /* U+00C9 */
  0x0000,   /* U+00CA */
  0x0000,   /* U+00CB */
  0x0000,   /* U+00CC */
  0x0000,   /* U+00CD */
  0x0000,   /* U+00CE */
  0x0000,   /* U+00CF */
  0x0000,   /* U+00D0 */
  0x0000,   /* U+00D1 */
  0x0000,   /* U+00D2 */
  0x0000,   /* U+00D3 */
  0x0000,   /* U+00D4 */
  0x0000,   /* U+00D5 */
  0x0000,   /* U+00D6 */
  0x0000,   /* U+00D7 */
  0x0000,   /* U+00D8 */
  0x0000,   /* U+00D9 */
  0x0000,   /* U+00DA */
  0x0000,   /* U+00DB */
  0x0000,   /* U+00DC */
  0x0000,   /* U+00DD */
  0x0000,   /* U+00DE */
  0x0004,   /* U+00DF */
  0x0000,   /* U+00E0 */
  0x0000,   /* U+00E1 */
  0x0000,   /* U+00E2 */
  0x0000,   /* U+00E3 */
  0x0000,   /* U+00E4 */
  0x0000,   /* U+00E5 */
  0x0004,   /* U+00E6 */
  0x0000,   /* U+00E7 */
  0x0000,   /* U+00E8 */
  0x0000,   /* U+00E9 */
  0x0000,   /* U+00EA */
  0x0000,   /* U+00EB */
  0x0000,   /* U+00EC */
  0x0000,   /* U+00ED */
  0x0000,   /* U+00EE */
  0x0000,   /* U+00EF */
  0x0000,   /* U+00F0 */
  0x0000,   /* U+00F1 */
  0x0000,   /* U+00F2 */
  0x0000,   /* U+00F3 */
  0x0000,   /* U+00F4 */
  0x0000,   /* U+00F5 */
  0x0000,   /* U+00F6 */
  0x0000,   /* U+00F7 */
  0x0000,   /* U+00F8 */
  0x0000,   /* U+00F9 */
  0x0000,   /* U+00FA */
  0x0000,   /* U+00FB */
  0x0000,   /* U+00FC */
  0x0000,   /* U+00FD */
  0x0000,   /* U+00FE */
  0x0000,   /* U+00FF */
};

uint16 uca900_p001[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 
  2, 2, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 1, 3, 3, 
  3, 3, 4, 4, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 3, 3, 4, 4, 3, 3, 

  /* Primary weight 1 for each character. */
  0x1C47,   /* U+0100 */
  0x1C47,   /* U+0101 */
  0x1C47,   /* U+0102 */
  0x1C47,   /* U+0103 */
  0x1C47,   /* U+0104 */
  0x1C47,   /* U+0105 */
  0x1C7A,   /* U+0106 */
  0x1C7A,   /* U+0107 */
  0x1C7A,   /* U+0108 */
  0x1C7A,   /* U+0109 */
  0x1C7A,   /* U+010A */
  0x1C7A,   /* U+010B */
  0x1C7A,   /* U+010C */
  0x1C7A,   /* U+010D */
  0x1C8F,   /* U+010E */
  0x1C8F,   /* U+010F */
  0x1C8F,   /* U+0110 */
  0x1C8F,   /* U+0111 */
  0x1CAA,   /* U+0112 */
  0x1CAA,   /* U+0113 */
  0x1CAA,   /* U+0114 */
  0x1CAA,   /* U+0115 */
  0x1CAA,   /* U+0116 */
  0x1CAA,   /* U+0117 */
  0x1CAA,   /* U+0118 */
  0x1CAA,   /* U+0119 */
  0x1CAA,   /* U+011A */
  0x1CAA,   /* U+011B */
  0x1CF4,   /* U+011C */
  0x1CF4,   /* U+011D */
  0x1CF4,   /* U+011E */
  0x1CF4,   /* U+011F */
  0x1CF4,   /* U+0120 */
  0x1CF4,   /* U+0121 */
  0x1CF4,   /* U+0122 */
  0x1CF4,   /* U+0123 */
  0x1D18,   /* U+0124 */
  0x1D18,   /* U+0125 */
  0x1D18,   /* U+0126 */
  0x1D18,   /* U+0127 */
  0x1D32,   /* U+0128 */
  0x1D32,   /* U+0129 */
  0x1D32,   /* U+012A */
  0x1D32,   /* U+012B */
  0x1D32,   /* U+012C */
  0x1D32,   /* U+012D */
  0x1D32,   /* U+012E */
  0x1D32,   /* U+012F */
  0x1D32,   /* U+0130 */
  0x1D36,   /* U+0131 */
  0x1D32,   /* U+0132 */
  0x1D32,   /* U+0133 */
  0x1D4C,   /* U+0134 */
  0x1D4C,   /* U+0135 */
  0x1D65,   /* U+0136 */
  0x1D65,   /* U+0137 */
  0x1E2F,   /* U+0138 */
  0x1D77,   /* U+0139 */
  0x1D77,   /* U+013A */
  0x1D77,   /* U+013B */
  0x1D77,   /* U+013C */
  0x1D77,   /* U+013D */
  0x1D77,   /* U+013E */
  0x1D77,   /* U+013F */
  0x1D77,   /* U+0140 */
  0x1D77,   /* U+0141 */
  0x1D77,   /* U+0142 */
  0x1DB9,   /* U+0143 */
  0x1DB9,   /* U+0144 */
  0x1DB9,   /* U+0145 */
  0x1DB9,   /* U+0146 */
  0x1DB9,   /* U+0147 */
  0x1DB9,   /* U+0148 */
  0x1F7E,   /* U+0149 */
  0x1DD8,   /* U+014A */
  0x1DD8,   /* U+014B */
  0x1DDD,   /* U+014C */
  0x1DDD,   /* U+014D */
  0x1DDD,   /* U+014E */
  0x1DDD,   /* U+014F */
  0x1DDD,   /* U+0150 */
  0x1DDD,   /* U+0151 */
  0x1DDD,   /* U+0152 */
  0x1DDD,   /* U+0153 */
  0x1E33,   /* U+0154 */
  0x1E33,   /* U+0155 */
  0x1E33,   /* U+0156 */
  0x1E33,   /* U+0157 */
  0x1E33,   /* U+0158 */
  0x1E33,   /* U+0159 */
  0x1E71,   /* U+015A */
  0x1E71,   /* U+015B */
  0x1E71,   /* U+015C */
  0x1E71,   /* U+015D */
  0x1E71,   /* U+015E */
  0x1E71,   /* U+015F */
  0x1E71,   /* U+0160 */
  0x1E71,   /* U+0161 */
  0x1E95,   /* U+0162 */
  0x1E95,   /* U+0163 */
  0x1E95,   /* U+0164 */
  0x1E95,   /* U+0165 */
  0x1E9A,   /* U+0166 */
  0x1E9A,   /* U+0167 */
  0x1EB5,   /* U+0168 */
  0x1EB5,   /* U+0169 */
  0x1EB5,   /* U+016A */
  0x1EB5,   /* U+016B */
  0x1EB5,   /* U+016C */
  0x1EB5,   /* U+016D */
  0x1EB5,   /* U+016E */
  0x1EB5,   /* U+016F */
  0x1EB5,   /* U+0170 */
  0x1EB5,   /* U+0171 */
  0x1EB5,   /* U+0172 */
  0x1EB5,   /* U+0173 */
  0x1EF5,   /* U+0174 */
  0x1EF5,   /* U+0175 */
  0x1F0B,   /* U+0176 */
  0x1F0B,   /* U+0177 */
  0x1F0B,   /* U+0178 */
  0x1F21,   /* U+0179 */
  0x1F21,   /* U+017A */
  0x1F21,   /* U+017B */
  0x1F21,   /* U+017C */
  0x1F21,   /* U+017D */
  0x1F21,   /* U+017E */
  0x1E71,   /* U+017F */
  0x1C68,   /* U+0180 */
  0x1C71,   /* U+0181 */
  0x1C75,   /* U+0182 */
  0x1C75,   /* U+0183 */
  0x1F71,   /* U+0184 */
  0x1F71,   /* U+0185 */
  0x1DF0,   /* U+0186 */
  0x1C85,   /* U+0187 */
  0x1C85,   /* U+0188 */
  0x1C97,   /* U+0189 */
  0x1C9B,   /* U+018A */
  0x1CA0,   /* U+018B */
  0x1CA0,   /* U+018C */
  0x1F21,   /* U+018D */
  0x1CB8,   /* U+018E */
  0x1CBD,   /* U+018F */
  0x1CC2,   /* U+0190 */
  0x1CEE,   /* U+0191 */
  0x1CEE,   /* U+0192 */
  0x1D06,   /* U+0193 */
  0x1D10,   /* U+0194 */
  0x1D20,   /* U+0195 */
  0x1D47,   /* U+0196 */
  0x1D41,   /* U+0197 */
  0x1D6B,   /* U+0198 */
  0x1D6B,   /* U+0199 */
  0x1D82,   /* U+019A */
  0x1DA2,   /* U+019B */
  0x1ED4,   /* U+019C */
  0x1DC4,   /* U+019D */
  0x1DC8,   /* U+019E */
  0x1DFD,   /* U+019F */
  0x1DDD,   /* U+01A0 */
  0x1DDD,   /* U+01A1 */
  0x1D14,   /* U+01A2 */
  0x1D14,   /* U+01A3 */
  0x1E15,   /* U+01A4 */
  0x1E15,   /* U+01A5 */
  0x1E38,   /* U+01A6 */
  0x1F69,   /* U+01A7 */
  0x1F69,   /* U+01A8 */
  0x1E82,   /* U+01A9 */
  0x1E88,   /* U+01AA */
  0x1EA0,   /* U+01AB */
  0x1EA4,   /* U+01AC */
  0x1EA4,   /* U+01AD */
  0x1EA8,   /* U+01AE */
  0x1EB5,   /* U+01AF */
  0x1EB5,   /* U+01B0 */
  0x1EDE,   /* U+01B1 */
  0x1EEA,   /* U+01B2 */
  0x1F17,   /* U+01B3 */
  0x1F17,   /* U+01B4 */
  0x1F26,   /* U+01B5 */
  0x1F26,   /* U+01B6 */
  0x1F3E,   /* U+01B7 */
  0x1F43,   /* U+01B8 */
  0x1F43,   /* U+01B9 */
  0x1F48,   /* U+01BA */
  0x1F62,   /* U+01BB */
  0x1F6D,   /* U+01BC */
  0x1F6D,   /* U+01BD */
  0x1E95,   /* U+01BE */
  0x1F56,   /* U+01BF */
  0x1F99,   /* U+01C0 */
  0x1F9D,   /* U+01C1 */
  0x1FA1,   /* U+01C2 */
  0x1FA5,   /* U+01C3 */
  0x1C8F,   /* U+01C4 */
  0x1C8F,   /* U+01C5 */
  0x1C8F,   /* U+01C6 */
  0x1D77,   /* U+01C7 */
  0x1D77,   /* U+01C8 */
  0x1D77,   /* U+01C9 */
  0x1DB9,   /* U+01CA */
  0x1DB9,   /* U+01CB */
  0x1DB9,   /* U+01CC */
  0x1C47,   /* U+01CD */
  0x1C47,   /* U+01CE */
  0x1D32,   /* U+01CF */
  0x1D32,   /* U+01D0 */
  0x1DDD,   /* U+01D1 */
  0x1DDD,   /* U+01D2 */
  0x1EB5,   /* U+01D3 */
  0x1EB5,   /* U+01D4 */
  0x1EB5,   /* U+01D5 */
  0x1EB5,   /* U+01D6 */
  0x1EB5,   /* U+01D7 */
  0x1EB5,   /* U+01D8 */
  0x1EB5,   /* U+01D9 */
  0x1EB5,   /* U+01DA */
  0x1EB5,   /* U+01DB */
  0x1EB5,   /* U+01DC */
  0x1CB8,   /* U+01DD */
  0x1C47,   /* U+01DE */
  0x1C47,   /* U+01DF */
  0x1C47,   /* U+01E0 */
  0x1C47,   /* U+01E1 */
  0x1C47,   /* U+01E2 */
  0x1C47,   /* U+01E3 */
  0x1D01,   /* U+01E4 */
  0x1D01,   /* U+01E5 */
  0x1CF4,   /* U+01E6 */
  0x1CF4,   /* U+01E7 */
  0x1D65,   /* U+01E8 */
  0x1D65,   /* U+01E9 */
  0x1DDD,   /* U+01EA */
  0x1DDD,   /* U+01EB */
  0x1DDD,   /* U+01EC */
  0x1DDD,   /* U+01ED */
  0x1F3E,   /* U+01EE */
  0x1F3E,   /* U+01EF */
  0x1D4C,   /* U+01F0 */
  0x1C8F,   /* U+01F1 */
  0x1C8F,   /* U+01F2 */
  0x1C8F,   /* U+01F3 */
  0x1CF4,   /* U+01F4 */
  0x1CF4,   /* U+01F5 */
  0x1D20,   /* U+01F6 */
  0x1F56,   /* U+01F7 */
  0x1DB9,   /* U+01F8 */
  0x1DB9,   /* U+01F9 */
  0x1C47,   /* U+01FA */
  0x1C47,   /* U+01FB */
  0x1C47,   /* U+01FC */
  0x1C47,   /* U+01FD */
  0x1DDD,   /* U+01FE */
  0x1DDD,   /* U+01FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0100 */
  0x0020,   /* U+0101 */
  0x0020,   /* U+0102 */
  0x0020,   /* U+0103 */
  0x0020,   /* U+0104 */
  0x0020,   /* U+0105 */
  0x0020,   /* U+0106 */
  0x0020,   /* U+0107 */
  0x0020,   /* U+0108 */
  0x0020,   /* U+0109 */
  0x0020,   /* U+010A */
  0x0020,   /* U+010B */
  0x0020,   /* U+010C */
  0x0020,   /* U+010D */
  0x0020,   /* U+010E */
  0x0020,   /* U+010F */
  0x0020,   /* U+0110 */
  0x0020,   /* U+0111 */
  0x0020,   /* U+0112 */
  0x0020,   /* U+0113 */
  0x0020,   /* U+0114 */
  0x0020,   /* U+0115 */
  0x0020,   /* U+0116 */
  0x0020,   /* U+0117 */
  0x0020,   /* U+0118 */
  0x0020,   /* U+0119 */
  0x0020,   /* U+011A */
  0x0020,   /* U+011B */
  0x0020,   /* U+011C */
  0x0020,   /* U+011D */
  0x0020,   /* U+011E */
  0x0020,   /* U+011F */
  0x0020,   /* U+0120 */
  0x0020,   /* U+0121 */
  0x0020,   /* U+0122 */
  0x0020,   /* U+0123 */
  0x0020,   /* U+0124 */
  0x0020,   /* U+0125 */
  0x0020,   /* U+0126 */
  0x0020,   /* U+0127 */
  0x0020,   /* U+0128 */
  0x0020,   /* U+0129 */
  0x0020,   /* U+012A */
  0x0020,   /* U+012B */
  0x0020,   /* U+012C */
  0x0020,   /* U+012D */
  0x0020,   /* U+012E */
  0x0020,   /* U+012F */
  0x0020,   /* U+0130 */
  0x0020,   /* U+0131 */
  0x0020,   /* U+0132 */
  0x0020,   /* U+0133 */
  0x0020,   /* U+0134 */
  0x0020,   /* U+0135 */
  0x0020,   /* U+0136 */
  0x0020,   /* U+0137 */
  0x0020,   /* U+0138 */
  0x0020,   /* U+0139 */
  0x0020,   /* U+013A */
  0x0020,   /* U+013B */
  0x0020,   /* U+013C */
  0x0020,   /* U+013D */
  0x0020,   /* U+013E */
  0x0020,   /* U+013F */
  0x0020,   /* U+0140 */
  0x0020,   /* U+0141 */
  0x0020,   /* U+0142 */
  0x0020,   /* U+0143 */
  0x0020,   /* U+0144 */
  0x0020,   /* U+0145 */
  0x0020,   /* U+0146 */
  0x0020,   /* U+0147 */
  0x0020,   /* U+0148 */
  0x0020,   /* U+0149 */
  0x0020,   /* U+014A */
  0x0020,   /* U+014B */
  0x0020,   /* U+014C */
  0x0020,   /* U+014D */
  0x0020,   /* U+014E */
  0x0020,   /* U+014F */
  0x0020,   /* U+0150 */
  0x0020,   /* U+0151 */
  0x0020,   /* U+0152 */
  0x0020,   /* U+0153 */
  0x0020,   /* U+0154 */
  0x0020,   /* U+0155 */
  0x0020,   /* U+0156 */
  0x0020,   /* U+0157 */
  0x0020,   /* U+0158 */
  0x0020,   /* U+0159 */
  0x0020,   /* U+015A */
  0x0020,   /* U+015B */
  0x0020,   /* U+015C */
  0x0020,   /* U+015D */
  0x0020,   /* U+015E */
  0x0020,   /* U+015F */
  0x0020,   /* U+0160 */
  0x0020,   /* U+0161 */
  0x0020,   /* U+0162 */
  0x0020,   /* U+0163 */
  0x0020,   /* U+0164 */
  0x0020,   /* U+0165 */
  0x0020,   /* U+0166 */
  0x0020,   /* U+0167 */
  0x0020,   /* U+0168 */
  0x0020,   /* U+0169 */
  0x0020,   /* U+016A */
  0x0020,   /* U+016B */
  0x0020,   /* U+016C */
  0x0020,   /* U+016D */
  0x0020,   /* U+016E */
  0x0020,   /* U+016F */
  0x0020,   /* U+0170 */
  0x0020,   /* U+0171 */
  0x0020,   /* U+0172 */
  0x0020,   /* U+0173 */
  0x0020,   /* U+0174 */
  0x0020,   /* U+0175 */
  0x0020,   /* U+0176 */
  0x0020,   /* U+0177 */
  0x0020,   /* U+0178 */
  0x0020,   /* U+0179 */
  0x0020,   /* U+017A */
  0x0020,   /* U+017B */
  0x0020,   /* U+017C */
  0x0020,   /* U+017D */
  0x0020,   /* U+017E */
  0x0020,   /* U+017F */
  0x0020,   /* U+0180 */
  0x0020,   /* U+0181 */
  0x0020,   /* U+0182 */
  0x0020,   /* U+0183 */
  0x0020,   /* U+0184 */
  0x0020,   /* U+0185 */
  0x0020,   /* U+0186 */
  0x0020,   /* U+0187 */
  0x0020,   /* U+0188 */
  0x0020,   /* U+0189 */
  0x0020,   /* U+018A */
  0x0020,   /* U+018B */
  0x0020,   /* U+018C */
  0x0020,   /* U+018D */
  0x0020,   /* U+018E */
  0x0020,   /* U+018F */
  0x0020,   /* U+0190 */
  0x0020,   /* U+0191 */
  0x0020,   /* U+0192 */
  0x0020,   /* U+0193 */
  0x0020,   /* U+0194 */
  0x0020,   /* U+0195 */
  0x0020,   /* U+0196 */
  0x0020,   /* U+0197 */
  0x0020,   /* U+0198 */
  0x0020,   /* U+0199 */
  0x0020,   /* U+019A */
  0x0020,   /* U+019B */
  0x0020,   /* U+019C */
  0x0020,   /* U+019D */
  0x0020,   /* U+019E */
  0x0020,   /* U+019F */
  0x0020,   /* U+01A0 */
  0x0020,   /* U+01A1 */
  0x0020,   /* U+01A2 */
  0x0020,   /* U+01A3 */
  0x0020,   /* U+01A4 */
  0x0020,   /* U+01A5 */
  0x0020,   /* U+01A6 */
  0x0020,   /* U+01A7 */
  0x0020,   /* U+01A8 */
  0x0020,   /* U+01A9 */
  0x0020,   /* U+01AA */
  0x0020,   /* U+01AB */
  0x0020,   /* U+01AC */
  0x0020,   /* U+01AD */
  0x0020,   /* U+01AE */
  0x0020,   /* U+01AF */
  0x0020,   /* U+01B0 */
  0x0020,   /* U+01B1 */
  0x0020,   /* U+01B2 */
  0x0020,   /* U+01B3 */
  0x0020,   /* U+01B4 */
  0x0020,   /* U+01B5 */
  0x0020,   /* U+01B6 */
  0x0020,   /* U+01B7 */
  0x0020,   /* U+01B8 */
  0x0020,   /* U+01B9 */
  0x0020,   /* U+01BA */
  0x0020,   /* U+01BB */
  0x0020,   /* U+01BC */
  0x0020,   /* U+01BD */
  0x0020,   /* U+01BE */
  0x0020,   /* U+01BF */
  0x0020,   /* U+01C0 */
  0x0020,   /* U+01C1 */
  0x0020,   /* U+01C2 */
  0x0020,   /* U+01C3 */
  0x0020,   /* U+01C4 */
  0x0020,   /* U+01C5 */
  0x0020,   /* U+01C6 */
  0x0020,   /* U+01C7 */
  0x0020,   /* U+01C8 */
  0x0020,   /* U+01C9 */
  0x0020,   /* U+01CA */
  0x0020,   /* U+01CB */
  0x0020,   /* U+01CC */
  0x0020,   /* U+01CD */
  0x0020,   /* U+01CE */
  0x0020,   /* U+01CF */
  0x0020,   /* U+01D0 */
  0x0020,   /* U+01D1 */
  0x0020,   /* U+01D2 */
  0x0020,   /* U+01D3 */
  0x0020,   /* U+01D4 */
  0x0020,   /* U+01D5 */
  0x0020,   /* U+01D6 */
  0x0020,   /* U+01D7 */
  0x0020,   /* U+01D8 */
  0x0020,   /* U+01D9 */
  0x0020,   /* U+01DA */
  0x0020,   /* U+01DB */
  0x0020,   /* U+01DC */
  0x0020,   /* U+01DD */
  0x0020,   /* U+01DE */
  0x0020,   /* U+01DF */
  0x0020,   /* U+01E0 */
  0x0020,   /* U+01E1 */
  0x0020,   /* U+01E2 */
  0x0020,   /* U+01E3 */
  0x0020,   /* U+01E4 */
  0x0020,   /* U+01E5 */
  0x0020,   /* U+01E6 */
  0x0020,   /* U+01E7 */
  0x0020,   /* U+01E8 */
  0x0020,   /* U+01E9 */
  0x0020,   /* U+01EA */
  0x0020,   /* U+01EB */
  0x0020,   /* U+01EC */
  0x0020,   /* U+01ED */
  0x0020,   /* U+01EE */
  0x0020,   /* U+01EF */
  0x0020,   /* U+01F0 */
  0x0020,   /* U+01F1 */
  0x0020,   /* U+01F2 */
  0x0020,   /* U+01F3 */
  0x0020,   /* U+01F4 */
  0x0020,   /* U+01F5 */
  0x0020,   /* U+01F6 */
  0x0020,   /* U+01F7 */
  0x0020,   /* U+01F8 */
  0x0020,   /* U+01F9 */
  0x0020,   /* U+01FA */
  0x0020,   /* U+01FB */
  0x0020,   /* U+01FC */
  0x0020,   /* U+01FD */
  0x0020,   /* U+01FE */
  0x0020,   /* U+01FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+0100 */
  0x0002,   /* U+0101 */
  0x0008,   /* U+0102 */
  0x0002,   /* U+0103 */
  0x0008,   /* U+0104 */
  0x0002,   /* U+0105 */
  0x0008,   /* U+0106 */
  0x0002,   /* U+0107 */
  0x0008,   /* U+0108 */
  0x0002,   /* U+0109 */
  0x0008,   /* U+010A */
  0x0002,   /* U+010B */
  0x0008,   /* U+010C */
  0x0002,   /* U+010D */
  0x0008,   /* U+010E */
  0x0002,   /* U+010F */
  0x0008,   /* U+0110 */
  0x0002,   /* U+0111 */
  0x0008,   /* U+0112 */
  0x0002,   /* U+0113 */
  0x0008,   /* U+0114 */
  0x0002,   /* U+0115 */
  0x0008,   /* U+0116 */
  0x0002,   /* U+0117 */
  0x0008,   /* U+0118 */
  0x0002,   /* U+0119 */
  0x0008,   /* U+011A */
  0x0002,   /* U+011B */
  0x0008,   /* U+011C */
  0x0002,   /* U+011D */
  0x0008,   /* U+011E */
  0x0002,   /* U+011F */
  0x0008,   /* U+0120 */
  0x0002,   /* U+0121 */
  0x0008,   /* U+0122 */
  0x0002,   /* U+0123 */
  0x0008,   /* U+0124 */
  0x0002,   /* U+0125 */
  0x0008,   /* U+0126 */
  0x0002,   /* U+0127 */
  0x0008,   /* U+0128 */
  0x0002,   /* U+0129 */
  0x0008,   /* U+012A */
  0x0002,   /* U+012B */
  0x0008,   /* U+012C */
  0x0002,   /* U+012D */
  0x0008,   /* U+012E */
  0x0002,   /* U+012F */
  0x0008,   /* U+0130 */
  0x0002,   /* U+0131 */
  0x000A,   /* U+0132 */
  0x0004,   /* U+0133 */
  0x0008,   /* U+0134 */
  0x0002,   /* U+0135 */
  0x0008,   /* U+0136 */
  0x0002,   /* U+0137 */
  0x0002,   /* U+0138 */
  0x0008,   /* U+0139 */
  0x0002,   /* U+013A */
  0x0008,   /* U+013B */
  0x0002,   /* U+013C */
  0x0008,   /* U+013D */
  0x0002,   /* U+013E */
  0x0008,   /* U+013F */
  0x0002,   /* U+0140 */
  0x0008,   /* U+0141 */
  0x0002,   /* U+0142 */
  0x0008,   /* U+0143 */
  0x0002,   /* U+0144 */
  0x0008,   /* U+0145 */
  0x0002,   /* U+0146 */
  0x0008,   /* U+0147 */
  0x0002,   /* U+0148 */
  0x0004,   /* U+0149 */
  0x0008,   /* U+014A */
  0x0002,   /* U+014B */
  0x0008,   /* U+014C */
  0x0002,   /* U+014D */
  0x0008,   /* U+014E */
  0x0002,   /* U+014F */
  0x0008,   /* U+0150 */
  0x0002,   /* U+0151 */
  0x000A,   /* U+0152 */
  0x0004,   /* U+0153 */
  0x0008,   /* U+0154 */
  0x0002,   /* U+0155 */
  0x0008,   /* U+0156 */
  0x0002,   /* U+0157 */
  0x0008,   /* U+0158 */
  0x0002,   /* U+0159 */
  0x0008,   /* U+015A */
  0x0002,   /* U+015B */
  0x0008,   /* U+015C */
  0x0002,   /* U+015D */
  0x0008,   /* U+015E */
  0x0002,   /* U+015F */
  0x0008,   /* U+0160 */
  0x0002,   /* U+0161 */
  0x0008,   /* U+0162 */
  0x0002,   /* U+0163 */
  0x0008,   /* U+0164 */
  0x0002,   /* U+0165 */
  0x0008,   /* U+0166 */
  0x0002,   /* U+0167 */
  0x0008,   /* U+0168 */
  0x0002,   /* U+0169 */
  0x0008,   /* U+016A */
  0x0002,   /* U+016B */
  0x0008,   /* U+016C */
  0x0002,   /* U+016D */
  0x0008,   /* U+016E */
  0x0002,   /* U+016F */
  0x0008,   /* U+0170 */
  0x0002,   /* U+0171 */
  0x0008,   /* U+0172 */
  0x0002,   /* U+0173 */
  0x0008,   /* U+0174 */
  0x0002,   /* U+0175 */
  0x0008,   /* U+0176 */
  0x0002,   /* U+0177 */
  0x0008,   /* U+0178 */
  0x0008,   /* U+0179 */
  0x0002,   /* U+017A */
  0x0008,   /* U+017B */
  0x0002,   /* U+017C */
  0x0008,   /* U+017D */
  0x0002,   /* U+017E */
  0x0004,   /* U+017F */
  0x0002,   /* U+0180 */
  0x0008,   /* U+0181 */
  0x0008,   /* U+0182 */
  0x0002,   /* U+0183 */
  0x0008,   /* U+0184 */
  0x0002,   /* U+0185 */
  0x0008,   /* U+0186 */
  0x0008,   /* U+0187 */
  0x0002,   /* U+0188 */
  0x0008,   /* U+0189 */
  0x0008,   /* U+018A */
  0x0008,   /* U+018B */
  0x0002,   /* U+018C */
  0x0004,   /* U+018D */
  0x0008,   /* U+018E */
  0x0008,   /* U+018F */
  0x0008,   /* U+0190 */
  0x0008,   /* U+0191 */
  0x0002,   /* U+0192 */
  0x0008,   /* U+0193 */
  0x0008,   /* U+0194 */
  0x0002,   /* U+0195 */
  0x0008,   /* U+0196 */
  0x0008,   /* U+0197 */
  0x0008,   /* U+0198 */
  0x0002,   /* U+0199 */
  0x0002,   /* U+019A */
  0x0002,   /* U+019B */
  0x0008,   /* U+019C */
  0x0008,   /* U+019D */
  0x0002,   /* U+019E */
  0x0008,   /* U+019F */
  0x0008,   /* U+01A0 */
  0x0002,   /* U+01A1 */
  0x0008,   /* U+01A2 */
  0x0002,   /* U+01A3 */
  0x0008,   /* U+01A4 */
  0x0002,   /* U+01A5 */
  0x0008,   /* U+01A6 */
  0x0008,   /* U+01A7 */
  0x0002,   /* U+01A8 */
  0x0008,   /* U+01A9 */
  0x0002,   /* U+01AA */
  0x0002,   /* U+01AB */
  0x0008,   /* U+01AC */
  0x0002,   /* U+01AD */
  0x0008,   /* U+01AE */
  0x0008,   /* U+01AF */
  0x0002,   /* U+01B0 */
  0x0008,   /* U+01B1 */
  0x0008,   /* U+01B2 */
  0x0008,   /* U+01B3 */
  0x0002,   /* U+01B4 */
  0x0008,   /* U+01B5 */
  0x0002,   /* U+01B6 */
  0x0008,   /* U+01B7 */
  0x0008,   /* U+01B8 */
  0x0002,   /* U+01B9 */
  0x0002,   /* U+01BA */
  0x0002,   /* U+01BB */
  0x0008,   /* U+01BC */
  0x0002,   /* U+01BD */
  0x0004,   /* U+01BE */
  0x0002,   /* U+01BF */
  0x0002,   /* U+01C0 */
  0x0002,   /* U+01C1 */
  0x0002,   /* U+01C2 */
  0x0002,   /* U+01C3 */
  0x000A,   /* U+01C4 */
  0x000A,   /* U+01C5 */
  0x0004,   /* U+01C6 */
  0x000A,   /* U+01C7 */
  0x000A,   /* U+01C8 */
  0x0004,   /* U+01C9 */
  0x000A,   /* U+01CA */
  0x000A,   /* U+01CB */
  0x0004,   /* U+01CC */
  0x0008,   /* U+01CD */
  0x0002,   /* U+01CE */
  0x0008,   /* U+01CF */
  0x0002,   /* U+01D0 */
  0x0008,   /* U+01D1 */
  0x0002,   /* U+01D2 */
  0x0008,   /* U+01D3 */
  0x0002,   /* U+01D4 */
  0x0008,   /* U+01D5 */
  0x0002,   /* U+01D6 */
  0x0008,   /* U+01D7 */
  0x0002,   /* U+01D8 */
  0x0008,   /* U+01D9 */
  0x0002,   /* U+01DA */
  0x0008,   /* U+01DB */
  0x0002,   /* U+01DC */
  0x0002,   /* U+01DD */
  0x0008,   /* U+01DE */
  0x0002,   /* U+01DF */
  0x0008,   /* U+01E0 */
  0x0002,   /* U+01E1 */
  0x000A,   /* U+01E2 */
  0x0004,   /* U+01E3 */
  0x0008,   /* U+01E4 */
  0x0002,   /* U+01E5 */
  0x0008,   /* U+01E6 */
  0x0002,   /* U+01E7 */
  0x0008,   /* U+01E8 */
  0x0002,   /* U+01E9 */
  0x0008,   /* U+01EA */
  0x0002,   /* U+01EB */
  0x0008,   /* U+01EC */
  0x0002,   /* U+01ED */
  0x0008,   /* U+01EE */
  0x0002,   /* U+01EF */
  0x0002,   /* U+01F0 */
  0x000A,   /* U+01F1 */
  0x000A,   /* U+01F2 */
  0x0004,   /* U+01F3 */
  0x0008,   /* U+01F4 */
  0x0002,   /* U+01F5 */
  0x0008,   /* U+01F6 */
  0x0008,   /* U+01F7 */
  0x0008,   /* U+01F8 */
  0x0002,   /* U+01F9 */
  0x0008,   /* U+01FA */
  0x0002,   /* U+01FB */
  0x000A,   /* U+01FC */
  0x0004,   /* U+01FD */
  0x0008,   /* U+01FE */
  0x0002,   /* U+01FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x1D4C,   /* U+0132 */
  0x1D4C,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x1DB9,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x0000,   /* U+0152 */
  0x0000,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x1EF5,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x1E71,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x1F21,   /* U+01C4 */
  0x1F21,   /* U+01C5 */
  0x1F21,   /* U+01C6 */
  0x1D4C,   /* U+01C7 */
  0x1D4C,   /* U+01C8 */
  0x1D4C,   /* U+01C9 */
  0x1D4C,   /* U+01CA */
  0x1D4C,   /* U+01CB */
  0x1D4C,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0000,   /* U+01D5 */
  0x0000,   /* U+01D6 */
  0x0000,   /* U+01D7 */
  0x0000,   /* U+01D8 */
  0x0000,   /* U+01D9 */
  0x0000,   /* U+01DA */
  0x0000,   /* U+01DB */
  0x0000,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0000,   /* U+01DE */
  0x0000,   /* U+01DF */
  0x0000,   /* U+01E0 */
  0x0000,   /* U+01E1 */
  0x0000,   /* U+01E2 */
  0x0000,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0000,   /* U+01EC */
  0x0000,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x1F21,   /* U+01F1 */
  0x1F21,   /* U+01F2 */
  0x1F21,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0000,   /* U+01FA */
  0x0000,   /* U+01FB */
  0x0000,   /* U+01FC */
  0x0000,   /* U+01FD */
  0x0000,   /* U+01FE */
  0x0000,   /* U+01FF */

  /* Secondary weight 2 for each character. */
  0x0032,   /* U+0100 */
  0x0032,   /* U+0101 */
  0x0026,   /* U+0102 */
  0x0026,   /* U+0103 */
  0x0031,   /* U+0104 */
  0x0031,   /* U+0105 */
  0x0024,   /* U+0106 */
  0x0024,   /* U+0107 */
  0x0027,   /* U+0108 */
  0x0027,   /* U+0109 */
  0x002E,   /* U+010A */
  0x002E,   /* U+010B */
  0x0028,   /* U+010C */
  0x0028,   /* U+010D */
  0x0028,   /* U+010E */
  0x0028,   /* U+010F */
  0x0039,   /* U+0110 */
  0x0039,   /* U+0111 */
  0x0032,   /* U+0112 */
  0x0032,   /* U+0113 */
  0x0026,   /* U+0114 */
  0x0026,   /* U+0115 */
  0x002E,   /* U+0116 */
  0x002E,   /* U+0117 */
  0x0031,   /* U+0118 */
  0x0031,   /* U+0119 */
  0x0028,   /* U+011A */
  0x0028,   /* U+011B */
  0x0027,   /* U+011C */
  0x0027,   /* U+011D */
  0x0026,   /* U+011E */
  0x0026,   /* U+011F */
  0x002E,   /* U+0120 */
  0x002E,   /* U+0121 */
  0x0030,   /* U+0122 */
  0x0030,   /* U+0123 */
  0x0027,   /* U+0124 */
  0x0027,   /* U+0125 */
  0x0039,   /* U+0126 */
  0x0039,   /* U+0127 */
  0x002D,   /* U+0128 */
  0x002D,   /* U+0129 */
  0x0032,   /* U+012A */
  0x0032,   /* U+012B */
  0x0026,   /* U+012C */
  0x0026,   /* U+012D */
  0x0031,   /* U+012E */
  0x0031,   /* U+012F */
  0x002E,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0020,   /* U+0132 */
  0x0020,   /* U+0133 */
  0x0027,   /* U+0134 */
  0x0027,   /* U+0135 */
  0x0030,   /* U+0136 */
  0x0030,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0024,   /* U+0139 */
  0x0024,   /* U+013A */
  0x0030,   /* U+013B */
  0x0030,   /* U+013C */
  0x0028,   /* U+013D */
  0x0028,   /* U+013E */
  0x0110,   /* U+013F */
  0x0110,   /* U+0140 */
  0x0039,   /* U+0141 */
  0x0039,   /* U+0142 */
  0x0024,   /* U+0143 */
  0x0024,   /* U+0144 */
  0x0030,   /* U+0145 */
  0x0030,   /* U+0146 */
  0x0028,   /* U+0147 */
  0x0028,   /* U+0148 */
  0x0020,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0032,   /* U+014C */
  0x0032,   /* U+014D */
  0x0026,   /* U+014E */
  0x0026,   /* U+014F */
  0x002C,   /* U+0150 */
  0x002C,   /* U+0151 */
  0x0110,   /* U+0152 */
  0x0110,   /* U+0153 */
  0x0024,   /* U+0154 */
  0x0024,   /* U+0155 */
  0x0030,   /* U+0156 */
  0x0030,   /* U+0157 */
  0x0028,   /* U+0158 */
  0x0028,   /* U+0159 */
  0x0024,   /* U+015A */
  0x0024,   /* U+015B */
  0x0027,   /* U+015C */
  0x0027,   /* U+015D */
  0x0030,   /* U+015E */
  0x0030,   /* U+015F */
  0x0028,   /* U+0160 */
  0x0028,   /* U+0161 */
  0x0030,   /* U+0162 */
  0x0030,   /* U+0163 */
  0x0028,   /* U+0164 */
  0x0028,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x002D,   /* U+0168 */
  0x002D,   /* U+0169 */
  0x0032,   /* U+016A */
  0x0032,   /* U+016B */
  0x0026,   /* U+016C */
  0x0026,   /* U+016D */
  0x0029,   /* U+016E */
  0x0029,   /* U+016F */
  0x002C,   /* U+0170 */
  0x002C,   /* U+0171 */
  0x0031,   /* U+0172 */
  0x0031,   /* U+0173 */
  0x0027,   /* U+0174 */
  0x0027,   /* U+0175 */
  0x0027,   /* U+0176 */
  0x0027,   /* U+0177 */
  0x002B,   /* U+0178 */
  0x0024,   /* U+0179 */
  0x0024,   /* U+017A */
  0x002E,   /* U+017B */
  0x002E,   /* U+017C */
  0x0028,   /* U+017D */
  0x0028,   /* U+017E */
  0x0111,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0020,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x003F,   /* U+01A0 */
  0x003F,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x003F,   /* U+01AF */
  0x003F,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0020,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0020,   /* U+01C4 */
  0x0020,   /* U+01C5 */
  0x0020,   /* U+01C6 */
  0x0020,   /* U+01C7 */
  0x0020,   /* U+01C8 */
  0x0020,   /* U+01C9 */
  0x0020,   /* U+01CA */
  0x0020,   /* U+01CB */
  0x0020,   /* U+01CC */
  0x0028,   /* U+01CD */
  0x0028,   /* U+01CE */
  0x0028,   /* U+01CF */
  0x0028,   /* U+01D0 */
  0x0028,   /* U+01D1 */
  0x0028,   /* U+01D2 */
  0x0028,   /* U+01D3 */
  0x0028,   /* U+01D4 */
  0x002B,   /* U+01D5 */
  0x002B,   /* U+01D6 */
  0x002B,   /* U+01D7 */
  0x002B,   /* U+01D8 */
  0x002B,   /* U+01D9 */
  0x002B,   /* U+01DA */
  0x002B,   /* U+01DB */
  0x002B,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x002B,   /* U+01DE */
  0x002B,   /* U+01DF */
  0x002E,   /* U+01E0 */
  0x002E,   /* U+01E1 */
  0x0110,   /* U+01E2 */
  0x0110,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0028,   /* U+01E6 */
  0x0028,   /* U+01E7 */
  0x0028,   /* U+01E8 */
  0x0028,   /* U+01E9 */
  0x0031,   /* U+01EA */
  0x0031,   /* U+01EB */
  0x0031,   /* U+01EC */
  0x0031,   /* U+01ED */
  0x0028,   /* U+01EE */
  0x0028,   /* U+01EF */
  0x0028,   /* U+01F0 */
  0x0020,   /* U+01F1 */
  0x0020,   /* U+01F2 */
  0x0020,   /* U+01F3 */
  0x0024,   /* U+01F4 */
  0x0024,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0025,   /* U+01F8 */
  0x0025,   /* U+01F9 */
  0x0029,   /* U+01FA */
  0x0029,   /* U+01FB */
  0x0110,   /* U+01FC */
  0x0110,   /* U+01FD */
  0x002F,   /* U+01FE */
  0x002F,   /* U+01FF */

  /* Tertiary weight 2 for each character. */
  0x0002,   /* U+0100 */
  0x0002,   /* U+0101 */
  0x0002,   /* U+0102 */
  0x0002,   /* U+0103 */
  0x0002,   /* U+0104 */
  0x0002,   /* U+0105 */
  0x0002,   /* U+0106 */
  0x0002,   /* U+0107 */
  0x0002,   /* U+0108 */
  0x0002,   /* U+0109 */
  0x0002,   /* U+010A */
  0x0002,   /* U+010B */
  0x0002,   /* U+010C */
  0x0002,   /* U+010D */
  0x0002,   /* U+010E */
  0x0002,   /* U+010F */
  0x0002,   /* U+0110 */
  0x0002,   /* U+0111 */
  0x0002,   /* U+0112 */
  0x0002,   /* U+0113 */
  0x0002,   /* U+0114 */
  0x0002,   /* U+0115 */
  0x0002,   /* U+0116 */
  0x0002,   /* U+0117 */
  0x0002,   /* U+0118 */
  0x0002,   /* U+0119 */
  0x0002,   /* U+011A */
  0x0002,   /* U+011B */
  0x0002,   /* U+011C */
  0x0002,   /* U+011D */
  0x0002,   /* U+011E */
  0x0002,   /* U+011F */
  0x0002,   /* U+0120 */
  0x0002,   /* U+0121 */
  0x0002,   /* U+0122 */
  0x0002,   /* U+0123 */
  0x0002,   /* U+0124 */
  0x0002,   /* U+0125 */
  0x0002,   /* U+0126 */
  0x0002,   /* U+0127 */
  0x0002,   /* U+0128 */
  0x0002,   /* U+0129 */
  0x0002,   /* U+012A */
  0x0002,   /* U+012B */
  0x0002,   /* U+012C */
  0x0002,   /* U+012D */
  0x0002,   /* U+012E */
  0x0002,   /* U+012F */
  0x0002,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x000A,   /* U+0132 */
  0x0004,   /* U+0133 */
  0x0002,   /* U+0134 */
  0x0002,   /* U+0135 */
  0x0002,   /* U+0136 */
  0x0002,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0002,   /* U+0139 */
  0x0002,   /* U+013A */
  0x0002,   /* U+013B */
  0x0002,   /* U+013C */
  0x0002,   /* U+013D */
  0x0002,   /* U+013E */
  0x0002,   /* U+013F */
  0x0002,   /* U+0140 */
  0x0002,   /* U+0141 */
  0x0002,   /* U+0142 */
  0x0002,   /* U+0143 */
  0x0002,   /* U+0144 */
  0x0002,   /* U+0145 */
  0x0002,   /* U+0146 */
  0x0002,   /* U+0147 */
  0x0002,   /* U+0148 */
  0x0004,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0002,   /* U+014C */
  0x0002,   /* U+014D */
  0x0002,   /* U+014E */
  0x0002,   /* U+014F */
  0x0002,   /* U+0150 */
  0x0002,   /* U+0151 */
  0x0004,   /* U+0152 */
  0x0004,   /* U+0153 */
  0x0002,   /* U+0154 */
  0x0002,   /* U+0155 */
  0x0002,   /* U+0156 */
  0x0002,   /* U+0157 */
  0x0002,   /* U+0158 */
  0x0002,   /* U+0159 */
  0x0002,   /* U+015A */
  0x0002,   /* U+015B */
  0x0002,   /* U+015C */
  0x0002,   /* U+015D */
  0x0002,   /* U+015E */
  0x0002,   /* U+015F */
  0x0002,   /* U+0160 */
  0x0002,   /* U+0161 */
  0x0002,   /* U+0162 */
  0x0002,   /* U+0163 */
  0x0002,   /* U+0164 */
  0x0002,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0002,   /* U+0168 */
  0x0002,   /* U+0169 */
  0x0002,   /* U+016A */
  0x0002,   /* U+016B */
  0x0002,   /* U+016C */
  0x0002,   /* U+016D */
  0x0002,   /* U+016E */
  0x0002,   /* U+016F */
  0x0002,   /* U+0170 */
  0x0002,   /* U+0171 */
  0x0002,   /* U+0172 */
  0x0002,   /* U+0173 */
  0x0002,   /* U+0174 */
  0x0002,   /* U+0175 */
  0x0002,   /* U+0176 */
  0x0002,   /* U+0177 */
  0x0002,   /* U+0178 */
  0x0002,   /* U+0179 */
  0x0002,   /* U+017A */
  0x0002,   /* U+017B */
  0x0002,   /* U+017C */
  0x0002,   /* U+017D */
  0x0002,   /* U+017E */
  0x0004,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0004,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0002,   /* U+01A0 */
  0x0002,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0002,   /* U+01AF */
  0x0002,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0004,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x000A,   /* U+01C4 */
  0x0004,   /* U+01C5 */
  0x0004,   /* U+01C6 */
  0x000A,   /* U+01C7 */
  0x0004,   /* U+01C8 */
  0x0004,   /* U+01C9 */
  0x000A,   /* U+01CA */
  0x0004,   /* U+01CB */
  0x0004,   /* U+01CC */
  0x0002,   /* U+01CD */
  0x0002,   /* U+01CE */
  0x0002,   /* U+01CF */
  0x0002,   /* U+01D0 */
  0x0002,   /* U+01D1 */
  0x0002,   /* U+01D2 */
  0x0002,   /* U+01D3 */
  0x0002,   /* U+01D4 */
  0x0002,   /* U+01D5 */
  0x0002,   /* U+01D6 */
  0x0002,   /* U+01D7 */
  0x0002,   /* U+01D8 */
  0x0002,   /* U+01D9 */
  0x0002,   /* U+01DA */
  0x0002,   /* U+01DB */
  0x0002,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0002,   /* U+01DE */
  0x0002,   /* U+01DF */
  0x0002,   /* U+01E0 */
  0x0002,   /* U+01E1 */
  0x0004,   /* U+01E2 */
  0x0004,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0002,   /* U+01E6 */
  0x0002,   /* U+01E7 */
  0x0002,   /* U+01E8 */
  0x0002,   /* U+01E9 */
  0x0002,   /* U+01EA */
  0x0002,   /* U+01EB */
  0x0002,   /* U+01EC */
  0x0002,   /* U+01ED */
  0x0002,   /* U+01EE */
  0x0002,   /* U+01EF */
  0x0002,   /* U+01F0 */
  0x000A,   /* U+01F1 */
  0x0004,   /* U+01F2 */
  0x0004,   /* U+01F3 */
  0x0002,   /* U+01F4 */
  0x0002,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0002,   /* U+01F8 */
  0x0002,   /* U+01F9 */
  0x0002,   /* U+01FA */
  0x0002,   /* U+01FB */
  0x0004,   /* U+01FC */
  0x0004,   /* U+01FD */
  0x0002,   /* U+01FE */
  0x0002,   /* U+01FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x1CAA,   /* U+0152 */
  0x1CAA,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0000,   /* U+01C4 */
  0x0000,   /* U+01C5 */
  0x0000,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0000,   /* U+01D5 */
  0x0000,   /* U+01D6 */
  0x0000,   /* U+01D7 */
  0x0000,   /* U+01D8 */
  0x0000,   /* U+01D9 */
  0x0000,   /* U+01DA */
  0x0000,   /* U+01DB */
  0x0000,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0000,   /* U+01DE */
  0x0000,   /* U+01DF */
  0x0000,   /* U+01E0 */
  0x0000,   /* U+01E1 */
  0x1CAA,   /* U+01E2 */
  0x1CAA,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0000,   /* U+01EC */
  0x0000,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0000,   /* U+01FA */
  0x0000,   /* U+01FB */
  0x1CAA,   /* U+01FC */
  0x1CAA,   /* U+01FD */
  0x0000,   /* U+01FE */
  0x0000,   /* U+01FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x0020,   /* U+0152 */
  0x0020,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0028,   /* U+01C4 */
  0x0028,   /* U+01C5 */
  0x0028,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0032,   /* U+01D5 */
  0x0032,   /* U+01D6 */
  0x0024,   /* U+01D7 */
  0x0024,   /* U+01D8 */
  0x0028,   /* U+01D9 */
  0x0028,   /* U+01DA */
  0x0025,   /* U+01DB */
  0x0025,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0032,   /* U+01DE */
  0x0032,   /* U+01DF */
  0x0032,   /* U+01E0 */
  0x0032,   /* U+01E1 */
  0x0020,   /* U+01E2 */
  0x0020,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0032,   /* U+01EC */
  0x0032,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0024,   /* U+01FA */
  0x0024,   /* U+01FB */
  0x0020,   /* U+01FC */
  0x0020,   /* U+01FD */
  0x0024,   /* U+01FE */
  0x0024,   /* U+01FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x000A,   /* U+0152 */
  0x0004,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0004,   /* U+01C4 */
  0x0004,   /* U+01C5 */
  0x0004,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0002,   /* U+01D5 */
  0x0002,   /* U+01D6 */
  0x0002,   /* U+01D7 */
  0x0002,   /* U+01D8 */
  0x0002,   /* U+01D9 */
  0x0002,   /* U+01DA */
  0x0002,   /* U+01DB */
  0x0002,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0002,   /* U+01DE */
  0x0002,   /* U+01DF */
  0x0002,   /* U+01E0 */
  0x0002,   /* U+01E1 */
  0x000A,   /* U+01E2 */
  0x0004,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0002,   /* U+01EC */
  0x0002,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0002,   /* U+01FA */
  0x0002,   /* U+01FB */
  0x000A,   /* U+01FC */
  0x0004,   /* U+01FD */
  0x0002,   /* U+01FE */
  0x0002,   /* U+01FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x0000,   /* U+0152 */
  0x0000,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0000,   /* U+01C4 */
  0x0000,   /* U+01C5 */
  0x0000,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0000,   /* U+01D5 */
  0x0000,   /* U+01D6 */
  0x0000,   /* U+01D7 */
  0x0000,   /* U+01D8 */
  0x0000,   /* U+01D9 */
  0x0000,   /* U+01DA */
  0x0000,   /* U+01DB */
  0x0000,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0000,   /* U+01DE */
  0x0000,   /* U+01DF */
  0x0000,   /* U+01E0 */
  0x0000,   /* U+01E1 */
  0x0000,   /* U+01E2 */
  0x0000,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0000,   /* U+01EC */
  0x0000,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0000,   /* U+01FA */
  0x0000,   /* U+01FB */
  0x0000,   /* U+01FC */
  0x0000,   /* U+01FD */
  0x0000,   /* U+01FE */
  0x0000,   /* U+01FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x0000,   /* U+0152 */
  0x0000,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0000,   /* U+01C4 */
  0x0000,   /* U+01C5 */
  0x0000,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0000,   /* U+01D5 */
  0x0000,   /* U+01D6 */
  0x0000,   /* U+01D7 */
  0x0000,   /* U+01D8 */
  0x0000,   /* U+01D9 */
  0x0000,   /* U+01DA */
  0x0000,   /* U+01DB */
  0x0000,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0000,   /* U+01DE */
  0x0000,   /* U+01DF */
  0x0000,   /* U+01E0 */
  0x0000,   /* U+01E1 */
  0x0032,   /* U+01E2 */
  0x0032,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0000,   /* U+01EC */
  0x0000,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0000,   /* U+01FA */
  0x0000,   /* U+01FB */
  0x0024,   /* U+01FC */
  0x0024,   /* U+01FD */
  0x0000,   /* U+01FE */
  0x0000,   /* U+01FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+0100 */
  0x0000,   /* U+0101 */
  0x0000,   /* U+0102 */
  0x0000,   /* U+0103 */
  0x0000,   /* U+0104 */
  0x0000,   /* U+0105 */
  0x0000,   /* U+0106 */
  0x0000,   /* U+0107 */
  0x0000,   /* U+0108 */
  0x0000,   /* U+0109 */
  0x0000,   /* U+010A */
  0x0000,   /* U+010B */
  0x0000,   /* U+010C */
  0x0000,   /* U+010D */
  0x0000,   /* U+010E */
  0x0000,   /* U+010F */
  0x0000,   /* U+0110 */
  0x0000,   /* U+0111 */
  0x0000,   /* U+0112 */
  0x0000,   /* U+0113 */
  0x0000,   /* U+0114 */
  0x0000,   /* U+0115 */
  0x0000,   /* U+0116 */
  0x0000,   /* U+0117 */
  0x0000,   /* U+0118 */
  0x0000,   /* U+0119 */
  0x0000,   /* U+011A */
  0x0000,   /* U+011B */
  0x0000,   /* U+011C */
  0x0000,   /* U+011D */
  0x0000,   /* U+011E */
  0x0000,   /* U+011F */
  0x0000,   /* U+0120 */
  0x0000,   /* U+0121 */
  0x0000,   /* U+0122 */
  0x0000,   /* U+0123 */
  0x0000,   /* U+0124 */
  0x0000,   /* U+0125 */
  0x0000,   /* U+0126 */
  0x0000,   /* U+0127 */
  0x0000,   /* U+0128 */
  0x0000,   /* U+0129 */
  0x0000,   /* U+012A */
  0x0000,   /* U+012B */
  0x0000,   /* U+012C */
  0x0000,   /* U+012D */
  0x0000,   /* U+012E */
  0x0000,   /* U+012F */
  0x0000,   /* U+0130 */
  0x0000,   /* U+0131 */
  0x0000,   /* U+0132 */
  0x0000,   /* U+0133 */
  0x0000,   /* U+0134 */
  0x0000,   /* U+0135 */
  0x0000,   /* U+0136 */
  0x0000,   /* U+0137 */
  0x0000,   /* U+0138 */
  0x0000,   /* U+0139 */
  0x0000,   /* U+013A */
  0x0000,   /* U+013B */
  0x0000,   /* U+013C */
  0x0000,   /* U+013D */
  0x0000,   /* U+013E */
  0x0000,   /* U+013F */
  0x0000,   /* U+0140 */
  0x0000,   /* U+0141 */
  0x0000,   /* U+0142 */
  0x0000,   /* U+0143 */
  0x0000,   /* U+0144 */
  0x0000,   /* U+0145 */
  0x0000,   /* U+0146 */
  0x0000,   /* U+0147 */
  0x0000,   /* U+0148 */
  0x0000,   /* U+0149 */
  0x0000,   /* U+014A */
  0x0000,   /* U+014B */
  0x0000,   /* U+014C */
  0x0000,   /* U+014D */
  0x0000,   /* U+014E */
  0x0000,   /* U+014F */
  0x0000,   /* U+0150 */
  0x0000,   /* U+0151 */
  0x0000,   /* U+0152 */
  0x0000,   /* U+0153 */
  0x0000,   /* U+0154 */
  0x0000,   /* U+0155 */
  0x0000,   /* U+0156 */
  0x0000,   /* U+0157 */
  0x0000,   /* U+0158 */
  0x0000,   /* U+0159 */
  0x0000,   /* U+015A */
  0x0000,   /* U+015B */
  0x0000,   /* U+015C */
  0x0000,   /* U+015D */
  0x0000,   /* U+015E */
  0x0000,   /* U+015F */
  0x0000,   /* U+0160 */
  0x0000,   /* U+0161 */
  0x0000,   /* U+0162 */
  0x0000,   /* U+0163 */
  0x0000,   /* U+0164 */
  0x0000,   /* U+0165 */
  0x0000,   /* U+0166 */
  0x0000,   /* U+0167 */
  0x0000,   /* U+0168 */
  0x0000,   /* U+0169 */
  0x0000,   /* U+016A */
  0x0000,   /* U+016B */
  0x0000,   /* U+016C */
  0x0000,   /* U+016D */
  0x0000,   /* U+016E */
  0x0000,   /* U+016F */
  0x0000,   /* U+0170 */
  0x0000,   /* U+0171 */
  0x0000,   /* U+0172 */
  0x0000,   /* U+0173 */
  0x0000,   /* U+0174 */
  0x0000,   /* U+0175 */
  0x0000,   /* U+0176 */
  0x0000,   /* U+0177 */
  0x0000,   /* U+0178 */
  0x0000,   /* U+0179 */
  0x0000,   /* U+017A */
  0x0000,   /* U+017B */
  0x0000,   /* U+017C */
  0x0000,   /* U+017D */
  0x0000,   /* U+017E */
  0x0000,   /* U+017F */
  0x0000,   /* U+0180 */
  0x0000,   /* U+0181 */
  0x0000,   /* U+0182 */
  0x0000,   /* U+0183 */
  0x0000,   /* U+0184 */
  0x0000,   /* U+0185 */
  0x0000,   /* U+0186 */
  0x0000,   /* U+0187 */
  0x0000,   /* U+0188 */
  0x0000,   /* U+0189 */
  0x0000,   /* U+018A */
  0x0000,   /* U+018B */
  0x0000,   /* U+018C */
  0x0000,   /* U+018D */
  0x0000,   /* U+018E */
  0x0000,   /* U+018F */
  0x0000,   /* U+0190 */
  0x0000,   /* U+0191 */
  0x0000,   /* U+0192 */
  0x0000,   /* U+0193 */
  0x0000,   /* U+0194 */
  0x0000,   /* U+0195 */
  0x0000,   /* U+0196 */
  0x0000,   /* U+0197 */
  0x0000,   /* U+0198 */
  0x0000,   /* U+0199 */
  0x0000,   /* U+019A */
  0x0000,   /* U+019B */
  0x0000,   /* U+019C */
  0x0000,   /* U+019D */
  0x0000,   /* U+019E */
  0x0000,   /* U+019F */
  0x0000,   /* U+01A0 */
  0x0000,   /* U+01A1 */
  0x0000,   /* U+01A2 */
  0x0000,   /* U+01A3 */
  0x0000,   /* U+01A4 */
  0x0000,   /* U+01A5 */
  0x0000,   /* U+01A6 */
  0x0000,   /* U+01A7 */
  0x0000,   /* U+01A8 */
  0x0000,   /* U+01A9 */
  0x0000,   /* U+01AA */
  0x0000,   /* U+01AB */
  0x0000,   /* U+01AC */
  0x0000,   /* U+01AD */
  0x0000,   /* U+01AE */
  0x0000,   /* U+01AF */
  0x0000,   /* U+01B0 */
  0x0000,   /* U+01B1 */
  0x0000,   /* U+01B2 */
  0x0000,   /* U+01B3 */
  0x0000,   /* U+01B4 */
  0x0000,   /* U+01B5 */
  0x0000,   /* U+01B6 */
  0x0000,   /* U+01B7 */
  0x0000,   /* U+01B8 */
  0x0000,   /* U+01B9 */
  0x0000,   /* U+01BA */
  0x0000,   /* U+01BB */
  0x0000,   /* U+01BC */
  0x0000,   /* U+01BD */
  0x0000,   /* U+01BE */
  0x0000,   /* U+01BF */
  0x0000,   /* U+01C0 */
  0x0000,   /* U+01C1 */
  0x0000,   /* U+01C2 */
  0x0000,   /* U+01C3 */
  0x0000,   /* U+01C4 */
  0x0000,   /* U+01C5 */
  0x0000,   /* U+01C6 */
  0x0000,   /* U+01C7 */
  0x0000,   /* U+01C8 */
  0x0000,   /* U+01C9 */
  0x0000,   /* U+01CA */
  0x0000,   /* U+01CB */
  0x0000,   /* U+01CC */
  0x0000,   /* U+01CD */
  0x0000,   /* U+01CE */
  0x0000,   /* U+01CF */
  0x0000,   /* U+01D0 */
  0x0000,   /* U+01D1 */
  0x0000,   /* U+01D2 */
  0x0000,   /* U+01D3 */
  0x0000,   /* U+01D4 */
  0x0000,   /* U+01D5 */
  0x0000,   /* U+01D6 */
  0x0000,   /* U+01D7 */
  0x0000,   /* U+01D8 */
  0x0000,   /* U+01D9 */
  0x0000,   /* U+01DA */
  0x0000,   /* U+01DB */
  0x0000,   /* U+01DC */
  0x0000,   /* U+01DD */
  0x0000,   /* U+01DE */
  0x0000,   /* U+01DF */
  0x0000,   /* U+01E0 */
  0x0000,   /* U+01E1 */
  0x0002,   /* U+01E2 */
  0x0002,   /* U+01E3 */
  0x0000,   /* U+01E4 */
  0x0000,   /* U+01E5 */
  0x0000,   /* U+01E6 */
  0x0000,   /* U+01E7 */
  0x0000,   /* U+01E8 */
  0x0000,   /* U+01E9 */
  0x0000,   /* U+01EA */
  0x0000,   /* U+01EB */
  0x0000,   /* U+01EC */
  0x0000,   /* U+01ED */
  0x0000,   /* U+01EE */
  0x0000,   /* U+01EF */
  0x0000,   /* U+01F0 */
  0x0000,   /* U+01F1 */
  0x0000,   /* U+01F2 */
  0x0000,   /* U+01F3 */
  0x0000,   /* U+01F4 */
  0x0000,   /* U+01F5 */
  0x0000,   /* U+01F6 */
  0x0000,   /* U+01F7 */
  0x0000,   /* U+01F8 */
  0x0000,   /* U+01F9 */
  0x0000,   /* U+01FA */
  0x0000,   /* U+01FB */
  0x0002,   /* U+01FC */
  0x0002,   /* U+01FD */
  0x0000,   /* U+01FE */
  0x0000,   /* U+01FF */
};

uint16 uca900_p002[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 
  3, 3, 2, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C47,   /* U+0200 */
  0x1C47,   /* U+0201 */
  0x1C47,   /* U+0202 */
  0x1C47,   /* U+0203 */
  0x1CAA,   /* U+0204 */
  0x1CAA,   /* U+0205 */
  0x1CAA,   /* U+0206 */
  0x1CAA,   /* U+0207 */
  0x1D32,   /* U+0208 */
  0x1D32,   /* U+0209 */
  0x1D32,   /* U+020A */
  0x1D32,   /* U+020B */
  0x1DDD,   /* U+020C */
  0x1DDD,   /* U+020D */
  0x1DDD,   /* U+020E */
  0x1DDD,   /* U+020F */
  0x1E33,   /* U+0210 */
  0x1E33,   /* U+0211 */
  0x1E33,   /* U+0212 */
  0x1E33,   /* U+0213 */
  0x1EB5,   /* U+0214 */
  0x1EB5,   /* U+0215 */
  0x1EB5,   /* U+0216 */
  0x1EB5,   /* U+0217 */
  0x1E71,   /* U+0218 */
  0x1E71,   /* U+0219 */
  0x1E95,   /* U+021A */
  0x1E95,   /* U+021B */
  0x1F1D,   /* U+021C */
  0x1F1D,   /* U+021D */
  0x1D18,   /* U+021E */
  0x1D18,   /* U+021F */
  0x1DC8,   /* U+0220 */
  0x1CA4,   /* U+0221 */
  0x1E07,   /* U+0222 */
  0x1E07,   /* U+0223 */
  0x1F2C,   /* U+0224 */
  0x1F2C,   /* U+0225 */
  0x1C47,   /* U+0226 */
  0x1C47,   /* U+0227 */
  0x1CAA,   /* U+0228 */
  0x1CAA,   /* U+0229 */
  0x1DDD,   /* U+022A */
  0x1DDD,   /* U+022B */
  0x1DDD,   /* U+022C */
  0x1DDD,   /* U+022D */
  0x1DDD,   /* U+022E */
  0x1DDD,   /* U+022F */
  0x1DDD,   /* U+0230 */
  0x1DDD,   /* U+0231 */
  0x1F0B,   /* U+0232 */
  0x1F0B,   /* U+0233 */
  0x1D98,   /* U+0234 */
  0x1DD2,   /* U+0235 */
  0x1EAC,   /* U+0236 */
  0x1D50,   /* U+0237 */
  0x1C8F,   /* U+0238 */
  0x1E21,   /* U+0239 */
  0x1C4C,   /* U+023A */
  0x1C7F,   /* U+023B */
  0x1C7F,   /* U+023C */
  0x1D82,   /* U+023D */
  0x1E9E,   /* U+023E */
  0x1E7C,   /* U+023F */
  0x1F38,   /* U+0240 */
  0x1F79,   /* U+0241 */
  0x1F79,   /* U+0242 */
  0x1C68,   /* U+0243 */
  0x1EC0,   /* U+0244 */
  0x1EF1,   /* U+0245 */
  0x1CB1,   /* U+0246 */
  0x1CB1,   /* U+0247 */
  0x1D55,   /* U+0248 */
  0x1D55,   /* U+0249 */
  0x1E2B,   /* U+024A */
  0x1E2B,   /* U+024B */
  0x1E3F,   /* U+024C */
  0x1E3F,   /* U+024D */
  0x1F13,   /* U+024E */
  0x1F13,   /* U+024F */
  0x1C51,   /* U+0250 */
  0x1C55,   /* U+0251 */
  0x1C5B,   /* U+0252 */
  0x1C71,   /* U+0253 */
  0x1DF0,   /* U+0254 */
  0x1C89,   /* U+0255 */
  0x1C97,   /* U+0256 */
  0x1C9B,   /* U+0257 */
  0x1CC7,   /* U+0258 */
  0x1CBD,   /* U+0259 */
  0x1CCB,   /* U+025A */
  0x1CC2,   /* U+025B */
  0x1CCF,   /* U+025C */
  0x1CD5,   /* U+025D */
  0x1CD9,   /* U+025E */
  0x1D5D,   /* U+025F */
  0x1D06,   /* U+0260 */
  0x1CF8,   /* U+0261 */
  0x1CFD,   /* U+0262 */
  0x1D10,   /* U+0263 */
  0x1CE1,   /* U+0264 */
  0x1EC8,   /* U+0265 */
  0x1D25,   /* U+0266 */
  0x1D2C,   /* U+0267 */
  0x1D41,   /* U+0268 */
  0x1D47,   /* U+0269 */
  0x1D3A,   /* U+026A */
  0x1D87,   /* U+026B */
  0x1D8D,   /* U+026C */
  0x1D93,   /* U+026D */
  0x1D9D,   /* U+026E */
  0x1ED4,   /* U+026F */
  0x1EDA,   /* U+0270 */
  0x1DB1,   /* U+0271 */
  0x1DC4,   /* U+0272 */
  0x1DCE,   /* U+0273 */
  0x1DBD,   /* U+0274 */
  0x1DFD,   /* U+0275 */
  0x1DE4,   /* U+0276 */
  0x1E02,   /* U+0277 */
  0x1E1C,   /* U+0278 */
  0x1E44,   /* U+0279 */
  0x1E49,   /* U+027A */
  0x1E4E,   /* U+027B */
  0x1E53,   /* U+027C */
  0x1E57,   /* U+027D */
  0x1E5C,   /* U+027E */
  0x1E61,   /* U+027F */
  0x1E38,   /* U+0280 */
  0x1E6A,   /* U+0281 */
  0x1E78,   /* U+0282 */
  0x1E82,   /* U+0283 */
  0x1D61,   /* U+0284 */
  0x1E8C,   /* U+0285 */
  0x1E91,   /* U+0286 */
  0x1EB1,   /* U+0287 */
  0x1EA8,   /* U+0288 */
  0x1EC0,   /* U+0289 */
  0x1EDE,   /* U+028A */
  0x1EEA,   /* U+028B */
  0x1EF1,   /* U+028C */
  0x1EFB,   /* U+028D */
  0x1DA6,   /* U+028E */
  0x1F0F,   /* U+028F */
  0x1F30,   /* U+0290 */
  0x1F34,   /* U+0291 */
  0x1F3E,   /* U+0292 */
  0x1F4C,   /* U+0293 */
  0x1F75,   /* U+0294 */
  0x1F84,   /* U+0295 */
  0x1F95,   /* U+0296 */
  0x1FA9,   /* U+0297 */
  0x1FAD,   /* U+0298 */
  0x1C64,   /* U+0299 */
  0x1CDD,   /* U+029A */
  0x1D0A,   /* U+029B */
  0x1D1C,   /* U+029C */
  0x1D59,   /* U+029D */
  0x1D73,   /* U+029E */
  0x1D7B,   /* U+029F */
  0x1E27,   /* U+02A0 */
  0x1F8D,   /* U+02A1 */
  0x1F91,   /* U+02A2 */
  0x1C8F,   /* U+02A3 */
  0x1C8F,   /* U+02A4 */
  0x1C8F,   /* U+02A5 */
  0x1E95,   /* U+02A6 */
  0x1E95,   /* U+02A7 */
  0x1E95,   /* U+02A8 */
  0x1CE5,   /* U+02A9 */
  0x1D77,   /* U+02AA */
  0x1D77,   /* U+02AB */
  0x1FB1,   /* U+02AC */
  0x1FB5,   /* U+02AD */
  0x1ECC,   /* U+02AE */
  0x1ED0,   /* U+02AF */
  0x1D18,   /* U+02B0 */
  0x1D25,   /* U+02B1 */
  0x1D4C,   /* U+02B2 */
  0x1E33,   /* U+02B3 */
  0x1E44,   /* U+02B4 */
  0x1E4E,   /* U+02B5 */
  0x1E6A,   /* U+02B6 */
  0x1EF5,   /* U+02B7 */
  0x1F0B,   /* U+02B8 */
  0x0493,   /* U+02B9 */
  0x0495,   /* U+02BA */
  0x1D30,   /* U+02BB */
  0x1F7E,   /* U+02BC */
  0x1D31,   /* U+02BD */
  0x1F80,   /* U+02BE */
  0x1F88,   /* U+02BF */
  0x1F7D,   /* U+02C0 */
  0x1F89,   /* U+02C1 */
  0x0496,   /* U+02C2 */
  0x0497,   /* U+02C3 */
  0x0498,   /* U+02C4 */
  0x0499,   /* U+02C5 */
  0x049A,   /* U+02C6 */
  0x049B,   /* U+02C7 */
  0x049C,   /* U+02C8 */
  0x049D,   /* U+02C9 */
  0x049E,   /* U+02CA */
  0x049F,   /* U+02CB */
  0x04A0,   /* U+02CC */
  0x04A1,   /* U+02CD */
  0x04A2,   /* U+02CE */
  0x04A3,   /* U+02CF */
  0x1BF8,   /* U+02D0 */
  0x1BF9,   /* U+02D1 */
  0x04A4,   /* U+02D2 */
  0x04A5,   /* U+02D3 */
  0x04A6,   /* U+02D4 */
  0x04A7,   /* U+02D5 */
  0x04A8,   /* U+02D6 */
  0x04A9,   /* U+02D7 */
  0x0487,   /* U+02D8 */
  0x0488,   /* U+02D9 */
  0x048A,   /* U+02DA */
  0x048D,   /* U+02DB */
  0x0484,   /* U+02DC */
  0x048B,   /* U+02DD */
  0x04AA,   /* U+02DE */
  0x04AB,   /* U+02DF */
  0x1D10,   /* U+02E0 */
  0x1D77,   /* U+02E1 */
  0x1E71,   /* U+02E2 */
  0x1EFF,   /* U+02E3 */
  0x1F84,   /* U+02E4 */
  0x04AC,   /* U+02E5 */
  0x04AD,   /* U+02E6 */
  0x04AE,   /* U+02E7 */
  0x04AF,   /* U+02E8 */
  0x04B0,   /* U+02E9 */
  0x04B1,   /* U+02EA */
  0x04B2,   /* U+02EB */
  0x04B3,   /* U+02EC */
  0x04B4,   /* U+02ED */
  0x1F7F,   /* U+02EE */
  0x04B5,   /* U+02EF */
  0x04B6,   /* U+02F0 */
  0x04B7,   /* U+02F1 */
  0x04B8,   /* U+02F2 */
  0x04B9,   /* U+02F3 */
  0x04BA,   /* U+02F4 */
  0x04BB,   /* U+02F5 */
  0x04BC,   /* U+02F6 */
  0x04BD,   /* U+02F7 */
  0x04BE,   /* U+02F8 */
  0x04BF,   /* U+02F9 */
  0x04C0,   /* U+02FA */
  0x04C1,   /* U+02FB */
  0x04C2,   /* U+02FC */
  0x04C3,   /* U+02FD */
  0x04C4,   /* U+02FE */
  0x04C5,   /* U+02FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0200 */
  0x0020,   /* U+0201 */
  0x0020,   /* U+0202 */
  0x0020,   /* U+0203 */
  0x0020,   /* U+0204 */
  0x0020,   /* U+0205 */
  0x0020,   /* U+0206 */
  0x0020,   /* U+0207 */
  0x0020,   /* U+0208 */
  0x0020,   /* U+0209 */
  0x0020,   /* U+020A */
  0x0020,   /* U+020B */
  0x0020,   /* U+020C */
  0x0020,   /* U+020D */
  0x0020,   /* U+020E */
  0x0020,   /* U+020F */
  0x0020,   /* U+0210 */
  0x0020,   /* U+0211 */
  0x0020,   /* U+0212 */
  0x0020,   /* U+0213 */
  0x0020,   /* U+0214 */
  0x0020,   /* U+0215 */
  0x0020,   /* U+0216 */
  0x0020,   /* U+0217 */
  0x0020,   /* U+0218 */
  0x0020,   /* U+0219 */
  0x0020,   /* U+021A */
  0x0020,   /* U+021B */
  0x0020,   /* U+021C */
  0x0020,   /* U+021D */
  0x0020,   /* U+021E */
  0x0020,   /* U+021F */
  0x0020,   /* U+0220 */
  0x0020,   /* U+0221 */
  0x0020,   /* U+0222 */
  0x0020,   /* U+0223 */
  0x0020,   /* U+0224 */
  0x0020,   /* U+0225 */
  0x0020,   /* U+0226 */
  0x0020,   /* U+0227 */
  0x0020,   /* U+0228 */
  0x0020,   /* U+0229 */
  0x0020,   /* U+022A */
  0x0020,   /* U+022B */
  0x0020,   /* U+022C */
  0x0020,   /* U+022D */
  0x0020,   /* U+022E */
  0x0020,   /* U+022F */
  0x0020,   /* U+0230 */
  0x0020,   /* U+0231 */
  0x0020,   /* U+0232 */
  0x0020,   /* U+0233 */
  0x0020,   /* U+0234 */
  0x0020,   /* U+0235 */
  0x0020,   /* U+0236 */
  0x0020,   /* U+0237 */
  0x0020,   /* U+0238 */
  0x0020,   /* U+0239 */
  0x0020,   /* U+023A */
  0x0020,   /* U+023B */
  0x0020,   /* U+023C */
  0x0020,   /* U+023D */
  0x0020,   /* U+023E */
  0x0020,   /* U+023F */
  0x0020,   /* U+0240 */
  0x0020,   /* U+0241 */
  0x0020,   /* U+0242 */
  0x0020,   /* U+0243 */
  0x0020,   /* U+0244 */
  0x0020,   /* U+0245 */
  0x0020,   /* U+0246 */
  0x0020,   /* U+0247 */
  0x0020,   /* U+0248 */
  0x0020,   /* U+0249 */
  0x0020,   /* U+024A */
  0x0020,   /* U+024B */
  0x0020,   /* U+024C */
  0x0020,   /* U+024D */
  0x0020,   /* U+024E */
  0x0020,   /* U+024F */
  0x0020,   /* U+0250 */
  0x0020,   /* U+0251 */
  0x0020,   /* U+0252 */
  0x0020,   /* U+0253 */
  0x0020,   /* U+0254 */
  0x0020,   /* U+0255 */
  0x0020,   /* U+0256 */
  0x0020,   /* U+0257 */
  0x0020,   /* U+0258 */
  0x0020,   /* U+0259 */
  0x0020,   /* U+025A */
  0x0020,   /* U+025B */
  0x0020,   /* U+025C */
  0x0020,   /* U+025D */
  0x0020,   /* U+025E */
  0x0020,   /* U+025F */
  0x0020,   /* U+0260 */
  0x0020,   /* U+0261 */
  0x0020,   /* U+0262 */
  0x0020,   /* U+0263 */
  0x0020,   /* U+0264 */
  0x0020,   /* U+0265 */
  0x0020,   /* U+0266 */
  0x0020,   /* U+0267 */
  0x0020,   /* U+0268 */
  0x0020,   /* U+0269 */
  0x0020,   /* U+026A */
  0x0020,   /* U+026B */
  0x0020,   /* U+026C */
  0x0020,   /* U+026D */
  0x0020,   /* U+026E */
  0x0020,   /* U+026F */
  0x0020,   /* U+0270 */
  0x0020,   /* U+0271 */
  0x0020,   /* U+0272 */
  0x0020,   /* U+0273 */
  0x0020,   /* U+0274 */
  0x0020,   /* U+0275 */
  0x0020,   /* U+0276 */
  0x0020,   /* U+0277 */
  0x0020,   /* U+0278 */
  0x0020,   /* U+0279 */
  0x0020,   /* U+027A */
  0x0020,   /* U+027B */
  0x0020,   /* U+027C */
  0x0020,   /* U+027D */
  0x0020,   /* U+027E */
  0x0020,   /* U+027F */
  0x0020,   /* U+0280 */
  0x0020,   /* U+0281 */
  0x0020,   /* U+0282 */
  0x0020,   /* U+0283 */
  0x0020,   /* U+0284 */
  0x0020,   /* U+0285 */
  0x0020,   /* U+0286 */
  0x0020,   /* U+0287 */
  0x0020,   /* U+0288 */
  0x0020,   /* U+0289 */
  0x0020,   /* U+028A */
  0x0020,   /* U+028B */
  0x0020,   /* U+028C */
  0x0020,   /* U+028D */
  0x0020,   /* U+028E */
  0x0020,   /* U+028F */
  0x0020,   /* U+0290 */
  0x0020,   /* U+0291 */
  0x0020,   /* U+0292 */
  0x0020,   /* U+0293 */
  0x0020,   /* U+0294 */
  0x0020,   /* U+0295 */
  0x0020,   /* U+0296 */
  0x0020,   /* U+0297 */
  0x0020,   /* U+0298 */
  0x0020,   /* U+0299 */
  0x0020,   /* U+029A */
  0x0020,   /* U+029B */
  0x0020,   /* U+029C */
  0x0020,   /* U+029D */
  0x0020,   /* U+029E */
  0x0020,   /* U+029F */
  0x0020,   /* U+02A0 */
  0x0020,   /* U+02A1 */
  0x0020,   /* U+02A2 */
  0x0020,   /* U+02A3 */
  0x0020,   /* U+02A4 */
  0x0020,   /* U+02A5 */
  0x0020,   /* U+02A6 */
  0x0020,   /* U+02A7 */
  0x0020,   /* U+02A8 */
  0x0020,   /* U+02A9 */
  0x0020,   /* U+02AA */
  0x0020,   /* U+02AB */
  0x0020,   /* U+02AC */
  0x0020,   /* U+02AD */
  0x0020,   /* U+02AE */
  0x0020,   /* U+02AF */
  0x0020,   /* U+02B0 */
  0x0020,   /* U+02B1 */
  0x0020,   /* U+02B2 */
  0x0020,   /* U+02B3 */
  0x0020,   /* U+02B4 */
  0x0020,   /* U+02B5 */
  0x0020,   /* U+02B6 */
  0x0020,   /* U+02B7 */
  0x0020,   /* U+02B8 */
  0x0020,   /* U+02B9 */
  0x0020,   /* U+02BA */
  0x0020,   /* U+02BB */
  0x0020,   /* U+02BC */
  0x0020,   /* U+02BD */
  0x0020,   /* U+02BE */
  0x0020,   /* U+02BF */
  0x0020,   /* U+02C0 */
  0x0020,   /* U+02C1 */
  0x0020,   /* U+02C2 */
  0x0020,   /* U+02C3 */
  0x0020,   /* U+02C4 */
  0x0020,   /* U+02C5 */
  0x0020,   /* U+02C6 */
  0x0020,   /* U+02C7 */
  0x0020,   /* U+02C8 */
  0x0020,   /* U+02C9 */
  0x0020,   /* U+02CA */
  0x0020,   /* U+02CB */
  0x0020,   /* U+02CC */
  0x0020,   /* U+02CD */
  0x0020,   /* U+02CE */
  0x0020,   /* U+02CF */
  0x0020,   /* U+02D0 */
  0x0020,   /* U+02D1 */
  0x0020,   /* U+02D2 */
  0x0020,   /* U+02D3 */
  0x0020,   /* U+02D4 */
  0x0020,   /* U+02D5 */
  0x0020,   /* U+02D6 */
  0x0020,   /* U+02D7 */
  0x0020,   /* U+02D8 */
  0x0020,   /* U+02D9 */
  0x0020,   /* U+02DA */
  0x0020,   /* U+02DB */
  0x0020,   /* U+02DC */
  0x0020,   /* U+02DD */
  0x0020,   /* U+02DE */
  0x0020,   /* U+02DF */
  0x0020,   /* U+02E0 */
  0x0020,   /* U+02E1 */
  0x0020,   /* U+02E2 */
  0x0020,   /* U+02E3 */
  0x0020,   /* U+02E4 */
  0x0020,   /* U+02E5 */
  0x0020,   /* U+02E6 */
  0x0020,   /* U+02E7 */
  0x0020,   /* U+02E8 */
  0x0020,   /* U+02E9 */
  0x0020,   /* U+02EA */
  0x0020,   /* U+02EB */
  0x0020,   /* U+02EC */
  0x0020,   /* U+02ED */
  0x0020,   /* U+02EE */
  0x0020,   /* U+02EF */
  0x0020,   /* U+02F0 */
  0x0020,   /* U+02F1 */
  0x0020,   /* U+02F2 */
  0x0020,   /* U+02F3 */
  0x0020,   /* U+02F4 */
  0x0020,   /* U+02F5 */
  0x0020,   /* U+02F6 */
  0x0020,   /* U+02F7 */
  0x0020,   /* U+02F8 */
  0x0020,   /* U+02F9 */
  0x0020,   /* U+02FA */
  0x0020,   /* U+02FB */
  0x0020,   /* U+02FC */
  0x0020,   /* U+02FD */
  0x0020,   /* U+02FE */
  0x0020,   /* U+02FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+0200 */
  0x0002,   /* U+0201 */
  0x0008,   /* U+0202 */
  0x0002,   /* U+0203 */
  0x0008,   /* U+0204 */
  0x0002,   /* U+0205 */
  0x0008,   /* U+0206 */
  0x0002,   /* U+0207 */
  0x0008,   /* U+0208 */
  0x0002,   /* U+0209 */
  0x0008,   /* U+020A */
  0x0002,   /* U+020B */
  0x0008,   /* U+020C */
  0x0002,   /* U+020D */
  0x0008,   /* U+020E */
  0x0002,   /* U+020F */
  0x0008,   /* U+0210 */
  0x0002,   /* U+0211 */
  0x0008,   /* U+0212 */
  0x0002,   /* U+0213 */
  0x0008,   /* U+0214 */
  0x0002,   /* U+0215 */
  0x0008,   /* U+0216 */
  0x0002,   /* U+0217 */
  0x0008,   /* U+0218 */
  0x0002,   /* U+0219 */
  0x0008,   /* U+021A */
  0x0002,   /* U+021B */
  0x0008,   /* U+021C */
  0x0002,   /* U+021D */
  0x0008,   /* U+021E */
  0x0002,   /* U+021F */
  0x0008,   /* U+0220 */
  0x0002,   /* U+0221 */
  0x0008,   /* U+0222 */
  0x0002,   /* U+0223 */
  0x0008,   /* U+0224 */
  0x0002,   /* U+0225 */
  0x0008,   /* U+0226 */
  0x0002,   /* U+0227 */
  0x0008,   /* U+0228 */
  0x0002,   /* U+0229 */
  0x0008,   /* U+022A */
  0x0002,   /* U+022B */
  0x0008,   /* U+022C */
  0x0002,   /* U+022D */
  0x0008,   /* U+022E */
  0x0002,   /* U+022F */
  0x0008,   /* U+0230 */
  0x0002,   /* U+0231 */
  0x0008,   /* U+0232 */
  0x0002,   /* U+0233 */
  0x0002,   /* U+0234 */
  0x0002,   /* U+0235 */
  0x0002,   /* U+0236 */
  0x0002,   /* U+0237 */
  0x0004,   /* U+0238 */
  0x0004,   /* U+0239 */
  0x0008,   /* U+023A */
  0x0008,   /* U+023B */
  0x0002,   /* U+023C */
  0x0008,   /* U+023D */
  0x0008,   /* U+023E */
  0x0002,   /* U+023F */
  0x0002,   /* U+0240 */
  0x0008,   /* U+0241 */
  0x0002,   /* U+0242 */
  0x0008,   /* U+0243 */
  0x0008,   /* U+0244 */
  0x0008,   /* U+0245 */
  0x0008,   /* U+0246 */
  0x0002,   /* U+0247 */
  0x0008,   /* U+0248 */
  0x0002,   /* U+0249 */
  0x0008,   /* U+024A */
  0x0002,   /* U+024B */
  0x0008,   /* U+024C */
  0x0002,   /* U+024D */
  0x0008,   /* U+024E */
  0x0002,   /* U+024F */
  0x0002,   /* U+0250 */
  0x0002,   /* U+0251 */
  0x0002,   /* U+0252 */
  0x0002,   /* U+0253 */
  0x0002,   /* U+0254 */
  0x0002,   /* U+0255 */
  0x0002,   /* U+0256 */
  0x0002,   /* U+0257 */
  0x0002,   /* U+0258 */
  0x0002,   /* U+0259 */
  0x0002,   /* U+025A */
  0x0002,   /* U+025B */
  0x0002,   /* U+025C */
  0x0002,   /* U+025D */
  0x0002,   /* U+025E */
  0x0002,   /* U+025F */
  0x0002,   /* U+0260 */
  0x0002,   /* U+0261 */
  0x0002,   /* U+0262 */
  0x0002,   /* U+0263 */
  0x0002,   /* U+0264 */
  0x0002,   /* U+0265 */
  0x0002,   /* U+0266 */
  0x0002,   /* U+0267 */
  0x0002,   /* U+0268 */
  0x0002,   /* U+0269 */
  0x0002,   /* U+026A */
  0x0002,   /* U+026B */
  0x0002,   /* U+026C */
  0x0002,   /* U+026D */
  0x0002,   /* U+026E */
  0x0002,   /* U+026F */
  0x0002,   /* U+0270 */
  0x0002,   /* U+0271 */
  0x0002,   /* U+0272 */
  0x0002,   /* U+0273 */
  0x0002,   /* U+0274 */
  0x0002,   /* U+0275 */
  0x0002,   /* U+0276 */
  0x0002,   /* U+0277 */
  0x0002,   /* U+0278 */
  0x0002,   /* U+0279 */
  0x0002,   /* U+027A */
  0x0002,   /* U+027B */
  0x0002,   /* U+027C */
  0x0002,   /* U+027D */
  0x0002,   /* U+027E */
  0x0002,   /* U+027F */
  0x0002,   /* U+0280 */
  0x0002,   /* U+0281 */
  0x0002,   /* U+0282 */
  0x0002,   /* U+0283 */
  0x0002,   /* U+0284 */
  0x0002,   /* U+0285 */
  0x0002,   /* U+0286 */
  0x0002,   /* U+0287 */
  0x0002,   /* U+0288 */
  0x0002,   /* U+0289 */
  0x0002,   /* U+028A */
  0x0002,   /* U+028B */
  0x0002,   /* U+028C */
  0x0002,   /* U+028D */
  0x0002,   /* U+028E */
  0x0002,   /* U+028F */
  0x0002,   /* U+0290 */
  0x0002,   /* U+0291 */
  0x0002,   /* U+0292 */
  0x0002,   /* U+0293 */
  0x0002,   /* U+0294 */
  0x0002,   /* U+0295 */
  0x0002,   /* U+0296 */
  0x0002,   /* U+0297 */
  0x0002,   /* U+0298 */
  0x0002,   /* U+0299 */
  0x0002,   /* U+029A */
  0x0002,   /* U+029B */
  0x0002,   /* U+029C */
  0x0002,   /* U+029D */
  0x0002,   /* U+029E */
  0x0002,   /* U+029F */
  0x0002,   /* U+02A0 */
  0x0002,   /* U+02A1 */
  0x0002,   /* U+02A2 */
  0x0004,   /* U+02A3 */
  0x0004,   /* U+02A4 */
  0x0004,   /* U+02A5 */
  0x0004,   /* U+02A6 */
  0x0004,   /* U+02A7 */
  0x0004,   /* U+02A8 */
  0x0004,   /* U+02A9 */
  0x0004,   /* U+02AA */
  0x0004,   /* U+02AB */
  0x0002,   /* U+02AC */
  0x0002,   /* U+02AD */
  0x0002,   /* U+02AE */
  0x0002,   /* U+02AF */
  0x0014,   /* U+02B0 */
  0x0014,   /* U+02B1 */
  0x0014,   /* U+02B2 */
  0x0014,   /* U+02B3 */
  0x0014,   /* U+02B4 */
  0x0014,   /* U+02B5 */
  0x0014,   /* U+02B6 */
  0x0014,   /* U+02B7 */
  0x0014,   /* U+02B8 */
  0x0002,   /* U+02B9 */
  0x0002,   /* U+02BA */
  0x0002,   /* U+02BB */
  0x0002,   /* U+02BC */
  0x0002,   /* U+02BD */
  0x0002,   /* U+02BE */
  0x0002,   /* U+02BF */
  0x0002,   /* U+02C0 */
  0x0002,   /* U+02C1 */
  0x0002,   /* U+02C2 */
  0x0002,   /* U+02C3 */
  0x0002,   /* U+02C4 */
  0x0002,   /* U+02C5 */
  0x0002,   /* U+02C6 */
  0x0002,   /* U+02C7 */
  0x0002,   /* U+02C8 */
  0x0002,   /* U+02C9 */
  0x0002,   /* U+02CA */
  0x0002,   /* U+02CB */
  0x0002,   /* U+02CC */
  0x0002,   /* U+02CD */
  0x0002,   /* U+02CE */
  0x0002,   /* U+02CF */
  0x0002,   /* U+02D0 */
  0x0002,   /* U+02D1 */
  0x0002,   /* U+02D2 */
  0x0002,   /* U+02D3 */
  0x0002,   /* U+02D4 */
  0x0002,   /* U+02D5 */
  0x0002,   /* U+02D6 */
  0x0002,   /* U+02D7 */
  0x0002,   /* U+02D8 */
  0x0002,   /* U+02D9 */
  0x0002,   /* U+02DA */
  0x0002,   /* U+02DB */
  0x0002,   /* U+02DC */
  0x0002,   /* U+02DD */
  0x0002,   /* U+02DE */
  0x0002,   /* U+02DF */
  0x0014,   /* U+02E0 */
  0x0014,   /* U+02E1 */
  0x0014,   /* U+02E2 */
  0x0014,   /* U+02E3 */
  0x0014,   /* U+02E4 */
  0x0002,   /* U+02E5 */
  0x0002,   /* U+02E6 */
  0x0002,   /* U+02E7 */
  0x0002,   /* U+02E8 */
  0x0002,   /* U+02E9 */
  0x0002,   /* U+02EA */
  0x0002,   /* U+02EB */
  0x0002,   /* U+02EC */
  0x0002,   /* U+02ED */
  0x0002,   /* U+02EE */
  0x0002,   /* U+02EF */
  0x0002,   /* U+02F0 */
  0x0002,   /* U+02F1 */
  0x0002,   /* U+02F2 */
  0x0002,   /* U+02F3 */
  0x0002,   /* U+02F4 */
  0x0002,   /* U+02F5 */
  0x0002,   /* U+02F6 */
  0x0002,   /* U+02F7 */
  0x0002,   /* U+02F8 */
  0x0002,   /* U+02F9 */
  0x0002,   /* U+02FA */
  0x0002,   /* U+02FB */
  0x0002,   /* U+02FC */
  0x0002,   /* U+02FD */
  0x0002,   /* U+02FE */
  0x0002,   /* U+02FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0200 */
  0x0000,   /* U+0201 */
  0x0000,   /* U+0202 */
  0x0000,   /* U+0203 */
  0x0000,   /* U+0204 */
  0x0000,   /* U+0205 */
  0x0000,   /* U+0206 */
  0x0000,   /* U+0207 */
  0x0000,   /* U+0208 */
  0x0000,   /* U+0209 */
  0x0000,   /* U+020A */
  0x0000,   /* U+020B */
  0x0000,   /* U+020C */
  0x0000,   /* U+020D */
  0x0000,   /* U+020E */
  0x0000,   /* U+020F */
  0x0000,   /* U+0210 */
  0x0000,   /* U+0211 */
  0x0000,   /* U+0212 */
  0x0000,   /* U+0213 */
  0x0000,   /* U+0214 */
  0x0000,   /* U+0215 */
  0x0000,   /* U+0216 */
  0x0000,   /* U+0217 */
  0x0000,   /* U+0218 */
  0x0000,   /* U+0219 */
  0x0000,   /* U+021A */
  0x0000,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0000,   /* U+021E */
  0x0000,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x0000,   /* U+0226 */
  0x0000,   /* U+0227 */
  0x0000,   /* U+0228 */
  0x0000,   /* U+0229 */
  0x0000,   /* U+022A */
  0x0000,   /* U+022B */
  0x0000,   /* U+022C */
  0x0000,   /* U+022D */
  0x0000,   /* U+022E */
  0x0000,   /* U+022F */
  0x0000,   /* U+0230 */
  0x0000,   /* U+0231 */
  0x0000,   /* U+0232 */
  0x0000,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x1C60,   /* U+0238 */
  0x1E0C,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x1F21,   /* U+02A3 */
  0x1F3E,   /* U+02A4 */
  0x1F34,   /* U+02A5 */
  0x1E71,   /* U+02A6 */
  0x1E82,   /* U+02A7 */
  0x1C89,   /* U+02A8 */
  0x1DD8,   /* U+02A9 */
  0x1E71,   /* U+02AA */
  0x1F21,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */

  /* Secondary weight 2 for each character. */
  0x003C,   /* U+0200 */
  0x003C,   /* U+0201 */
  0x003E,   /* U+0202 */
  0x003E,   /* U+0203 */
  0x003C,   /* U+0204 */
  0x003C,   /* U+0205 */
  0x003E,   /* U+0206 */
  0x003E,   /* U+0207 */
  0x003C,   /* U+0208 */
  0x003C,   /* U+0209 */
  0x003E,   /* U+020A */
  0x003E,   /* U+020B */
  0x003C,   /* U+020C */
  0x003C,   /* U+020D */
  0x003E,   /* U+020E */
  0x003E,   /* U+020F */
  0x003C,   /* U+0210 */
  0x003C,   /* U+0211 */
  0x003E,   /* U+0212 */
  0x003E,   /* U+0213 */
  0x003C,   /* U+0214 */
  0x003C,   /* U+0215 */
  0x003E,   /* U+0216 */
  0x003E,   /* U+0217 */
  0x0045,   /* U+0218 */
  0x0045,   /* U+0219 */
  0x0045,   /* U+021A */
  0x0045,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0028,   /* U+021E */
  0x0028,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x002E,   /* U+0226 */
  0x002E,   /* U+0227 */
  0x0030,   /* U+0228 */
  0x0030,   /* U+0229 */
  0x002B,   /* U+022A */
  0x002B,   /* U+022B */
  0x002D,   /* U+022C */
  0x002D,   /* U+022D */
  0x002E,   /* U+022E */
  0x002E,   /* U+022F */
  0x002E,   /* U+0230 */
  0x002E,   /* U+0231 */
  0x0032,   /* U+0232 */
  0x0032,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x0020,   /* U+0238 */
  0x0020,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x0020,   /* U+02A3 */
  0x0020,   /* U+02A4 */
  0x0020,   /* U+02A5 */
  0x0020,   /* U+02A6 */
  0x0020,   /* U+02A7 */
  0x0020,   /* U+02A8 */
  0x0020,   /* U+02A9 */
  0x0020,   /* U+02AA */
  0x0020,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */

  /* Tertiary weight 2 for each character. */
  0x0002,   /* U+0200 */
  0x0002,   /* U+0201 */
  0x0002,   /* U+0202 */
  0x0002,   /* U+0203 */
  0x0002,   /* U+0204 */
  0x0002,   /* U+0205 */
  0x0002,   /* U+0206 */
  0x0002,   /* U+0207 */
  0x0002,   /* U+0208 */
  0x0002,   /* U+0209 */
  0x0002,   /* U+020A */
  0x0002,   /* U+020B */
  0x0002,   /* U+020C */
  0x0002,   /* U+020D */
  0x0002,   /* U+020E */
  0x0002,   /* U+020F */
  0x0002,   /* U+0210 */
  0x0002,   /* U+0211 */
  0x0002,   /* U+0212 */
  0x0002,   /* U+0213 */
  0x0002,   /* U+0214 */
  0x0002,   /* U+0215 */
  0x0002,   /* U+0216 */
  0x0002,   /* U+0217 */
  0x0002,   /* U+0218 */
  0x0002,   /* U+0219 */
  0x0002,   /* U+021A */
  0x0002,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0002,   /* U+021E */
  0x0002,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x0002,   /* U+0226 */
  0x0002,   /* U+0227 */
  0x0002,   /* U+0228 */
  0x0002,   /* U+0229 */
  0x0002,   /* U+022A */
  0x0002,   /* U+022B */
  0x0002,   /* U+022C */
  0x0002,   /* U+022D */
  0x0002,   /* U+022E */
  0x0002,   /* U+022F */
  0x0002,   /* U+0230 */
  0x0002,   /* U+0231 */
  0x0002,   /* U+0232 */
  0x0002,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x0004,   /* U+0238 */
  0x0004,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x0004,   /* U+02A3 */
  0x0004,   /* U+02A4 */
  0x0004,   /* U+02A5 */
  0x0004,   /* U+02A6 */
  0x0004,   /* U+02A7 */
  0x0004,   /* U+02A8 */
  0x0004,   /* U+02A9 */
  0x0004,   /* U+02AA */
  0x0004,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+0200 */
  0x0000,   /* U+0201 */
  0x0000,   /* U+0202 */
  0x0000,   /* U+0203 */
  0x0000,   /* U+0204 */
  0x0000,   /* U+0205 */
  0x0000,   /* U+0206 */
  0x0000,   /* U+0207 */
  0x0000,   /* U+0208 */
  0x0000,   /* U+0209 */
  0x0000,   /* U+020A */
  0x0000,   /* U+020B */
  0x0000,   /* U+020C */
  0x0000,   /* U+020D */
  0x0000,   /* U+020E */
  0x0000,   /* U+020F */
  0x0000,   /* U+0210 */
  0x0000,   /* U+0211 */
  0x0000,   /* U+0212 */
  0x0000,   /* U+0213 */
  0x0000,   /* U+0214 */
  0x0000,   /* U+0215 */
  0x0000,   /* U+0216 */
  0x0000,   /* U+0217 */
  0x0000,   /* U+0218 */
  0x0000,   /* U+0219 */
  0x0000,   /* U+021A */
  0x0000,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0000,   /* U+021E */
  0x0000,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x0000,   /* U+0226 */
  0x0000,   /* U+0227 */
  0x0000,   /* U+0228 */
  0x0000,   /* U+0229 */
  0x0000,   /* U+022A */
  0x0000,   /* U+022B */
  0x0000,   /* U+022C */
  0x0000,   /* U+022D */
  0x0000,   /* U+022E */
  0x0000,   /* U+022F */
  0x0000,   /* U+0230 */
  0x0000,   /* U+0231 */
  0x0000,   /* U+0232 */
  0x0000,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x0000,   /* U+0238 */
  0x0000,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x0000,   /* U+02A3 */
  0x0000,   /* U+02A4 */
  0x0000,   /* U+02A5 */
  0x0000,   /* U+02A6 */
  0x0000,   /* U+02A7 */
  0x0000,   /* U+02A8 */
  0x0000,   /* U+02A9 */
  0x0000,   /* U+02AA */
  0x0000,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+0200 */
  0x0000,   /* U+0201 */
  0x0000,   /* U+0202 */
  0x0000,   /* U+0203 */
  0x0000,   /* U+0204 */
  0x0000,   /* U+0205 */
  0x0000,   /* U+0206 */
  0x0000,   /* U+0207 */
  0x0000,   /* U+0208 */
  0x0000,   /* U+0209 */
  0x0000,   /* U+020A */
  0x0000,   /* U+020B */
  0x0000,   /* U+020C */
  0x0000,   /* U+020D */
  0x0000,   /* U+020E */
  0x0000,   /* U+020F */
  0x0000,   /* U+0210 */
  0x0000,   /* U+0211 */
  0x0000,   /* U+0212 */
  0x0000,   /* U+0213 */
  0x0000,   /* U+0214 */
  0x0000,   /* U+0215 */
  0x0000,   /* U+0216 */
  0x0000,   /* U+0217 */
  0x0000,   /* U+0218 */
  0x0000,   /* U+0219 */
  0x0000,   /* U+021A */
  0x0000,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0000,   /* U+021E */
  0x0000,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x0000,   /* U+0226 */
  0x0000,   /* U+0227 */
  0x0000,   /* U+0228 */
  0x0000,   /* U+0229 */
  0x0032,   /* U+022A */
  0x0032,   /* U+022B */
  0x0032,   /* U+022C */
  0x0032,   /* U+022D */
  0x0000,   /* U+022E */
  0x0000,   /* U+022F */
  0x0032,   /* U+0230 */
  0x0032,   /* U+0231 */
  0x0000,   /* U+0232 */
  0x0000,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x0000,   /* U+0238 */
  0x0000,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x0000,   /* U+02A3 */
  0x0000,   /* U+02A4 */
  0x0000,   /* U+02A5 */
  0x0000,   /* U+02A6 */
  0x0000,   /* U+02A7 */
  0x0000,   /* U+02A8 */
  0x0000,   /* U+02A9 */
  0x0000,   /* U+02AA */
  0x0000,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+0200 */
  0x0000,   /* U+0201 */
  0x0000,   /* U+0202 */
  0x0000,   /* U+0203 */
  0x0000,   /* U+0204 */
  0x0000,   /* U+0205 */
  0x0000,   /* U+0206 */
  0x0000,   /* U+0207 */
  0x0000,   /* U+0208 */
  0x0000,   /* U+0209 */
  0x0000,   /* U+020A */
  0x0000,   /* U+020B */
  0x0000,   /* U+020C */
  0x0000,   /* U+020D */
  0x0000,   /* U+020E */
  0x0000,   /* U+020F */
  0x0000,   /* U+0210 */
  0x0000,   /* U+0211 */
  0x0000,   /* U+0212 */
  0x0000,   /* U+0213 */
  0x0000,   /* U+0214 */
  0x0000,   /* U+0215 */
  0x0000,   /* U+0216 */
  0x0000,   /* U+0217 */
  0x0000,   /* U+0218 */
  0x0000,   /* U+0219 */
  0x0000,   /* U+021A */
  0x0000,   /* U+021B */
  0x0000,   /* U+021C */
  0x0000,   /* U+021D */
  0x0000,   /* U+021E */
  0x0000,   /* U+021F */
  0x0000,   /* U+0220 */
  0x0000,   /* U+0221 */
  0x0000,   /* U+0222 */
  0x0000,   /* U+0223 */
  0x0000,   /* U+0224 */
  0x0000,   /* U+0225 */
  0x0000,   /* U+0226 */
  0x0000,   /* U+0227 */
  0x0000,   /* U+0228 */
  0x0000,   /* U+0229 */
  0x0002,   /* U+022A */
  0x0002,   /* U+022B */
  0x0002,   /* U+022C */
  0x0002,   /* U+022D */
  0x0000,   /* U+022E */
  0x0000,   /* U+022F */
  0x0002,   /* U+0230 */
  0x0002,   /* U+0231 */
  0x0000,   /* U+0232 */
  0x0000,   /* U+0233 */
  0x0000,   /* U+0234 */
  0x0000,   /* U+0235 */
  0x0000,   /* U+0236 */
  0x0000,   /* U+0237 */
  0x0000,   /* U+0238 */
  0x0000,   /* U+0239 */
  0x0000,   /* U+023A */
  0x0000,   /* U+023B */
  0x0000,   /* U+023C */
  0x0000,   /* U+023D */
  0x0000,   /* U+023E */
  0x0000,   /* U+023F */
  0x0000,   /* U+0240 */
  0x0000,   /* U+0241 */
  0x0000,   /* U+0242 */
  0x0000,   /* U+0243 */
  0x0000,   /* U+0244 */
  0x0000,   /* U+0245 */
  0x0000,   /* U+0246 */
  0x0000,   /* U+0247 */
  0x0000,   /* U+0248 */
  0x0000,   /* U+0249 */
  0x0000,   /* U+024A */
  0x0000,   /* U+024B */
  0x0000,   /* U+024C */
  0x0000,   /* U+024D */
  0x0000,   /* U+024E */
  0x0000,   /* U+024F */
  0x0000,   /* U+0250 */
  0x0000,   /* U+0251 */
  0x0000,   /* U+0252 */
  0x0000,   /* U+0253 */
  0x0000,   /* U+0254 */
  0x0000,   /* U+0255 */
  0x0000,   /* U+0256 */
  0x0000,   /* U+0257 */
  0x0000,   /* U+0258 */
  0x0000,   /* U+0259 */
  0x0000,   /* U+025A */
  0x0000,   /* U+025B */
  0x0000,   /* U+025C */
  0x0000,   /* U+025D */
  0x0000,   /* U+025E */
  0x0000,   /* U+025F */
  0x0000,   /* U+0260 */
  0x0000,   /* U+0261 */
  0x0000,   /* U+0262 */
  0x0000,   /* U+0263 */
  0x0000,   /* U+0264 */
  0x0000,   /* U+0265 */
  0x0000,   /* U+0266 */
  0x0000,   /* U+0267 */
  0x0000,   /* U+0268 */
  0x0000,   /* U+0269 */
  0x0000,   /* U+026A */
  0x0000,   /* U+026B */
  0x0000,   /* U+026C */
  0x0000,   /* U+026D */
  0x0000,   /* U+026E */
  0x0000,   /* U+026F */
  0x0000,   /* U+0270 */
  0x0000,   /* U+0271 */
  0x0000,   /* U+0272 */
  0x0000,   /* U+0273 */
  0x0000,   /* U+0274 */
  0x0000,   /* U+0275 */
  0x0000,   /* U+0276 */
  0x0000,   /* U+0277 */
  0x0000,   /* U+0278 */
  0x0000,   /* U+0279 */
  0x0000,   /* U+027A */
  0x0000,   /* U+027B */
  0x0000,   /* U+027C */
  0x0000,   /* U+027D */
  0x0000,   /* U+027E */
  0x0000,   /* U+027F */
  0x0000,   /* U+0280 */
  0x0000,   /* U+0281 */
  0x0000,   /* U+0282 */
  0x0000,   /* U+0283 */
  0x0000,   /* U+0284 */
  0x0000,   /* U+0285 */
  0x0000,   /* U+0286 */
  0x0000,   /* U+0287 */
  0x0000,   /* U+0288 */
  0x0000,   /* U+0289 */
  0x0000,   /* U+028A */
  0x0000,   /* U+028B */
  0x0000,   /* U+028C */
  0x0000,   /* U+028D */
  0x0000,   /* U+028E */
  0x0000,   /* U+028F */
  0x0000,   /* U+0290 */
  0x0000,   /* U+0291 */
  0x0000,   /* U+0292 */
  0x0000,   /* U+0293 */
  0x0000,   /* U+0294 */
  0x0000,   /* U+0295 */
  0x0000,   /* U+0296 */
  0x0000,   /* U+0297 */
  0x0000,   /* U+0298 */
  0x0000,   /* U+0299 */
  0x0000,   /* U+029A */
  0x0000,   /* U+029B */
  0x0000,   /* U+029C */
  0x0000,   /* U+029D */
  0x0000,   /* U+029E */
  0x0000,   /* U+029F */
  0x0000,   /* U+02A0 */
  0x0000,   /* U+02A1 */
  0x0000,   /* U+02A2 */
  0x0000,   /* U+02A3 */
  0x0000,   /* U+02A4 */
  0x0000,   /* U+02A5 */
  0x0000,   /* U+02A6 */
  0x0000,   /* U+02A7 */
  0x0000,   /* U+02A8 */
  0x0000,   /* U+02A9 */
  0x0000,   /* U+02AA */
  0x0000,   /* U+02AB */
  0x0000,   /* U+02AC */
  0x0000,   /* U+02AD */
  0x0000,   /* U+02AE */
  0x0000,   /* U+02AF */
  0x0000,   /* U+02B0 */
  0x0000,   /* U+02B1 */
  0x0000,   /* U+02B2 */
  0x0000,   /* U+02B3 */
  0x0000,   /* U+02B4 */
  0x0000,   /* U+02B5 */
  0x0000,   /* U+02B6 */
  0x0000,   /* U+02B7 */
  0x0000,   /* U+02B8 */
  0x0000,   /* U+02B9 */
  0x0000,   /* U+02BA */
  0x0000,   /* U+02BB */
  0x0000,   /* U+02BC */
  0x0000,   /* U+02BD */
  0x0000,   /* U+02BE */
  0x0000,   /* U+02BF */
  0x0000,   /* U+02C0 */
  0x0000,   /* U+02C1 */
  0x0000,   /* U+02C2 */
  0x0000,   /* U+02C3 */
  0x0000,   /* U+02C4 */
  0x0000,   /* U+02C5 */
  0x0000,   /* U+02C6 */
  0x0000,   /* U+02C7 */
  0x0000,   /* U+02C8 */
  0x0000,   /* U+02C9 */
  0x0000,   /* U+02CA */
  0x0000,   /* U+02CB */
  0x0000,   /* U+02CC */
  0x0000,   /* U+02CD */
  0x0000,   /* U+02CE */
  0x0000,   /* U+02CF */
  0x0000,   /* U+02D0 */
  0x0000,   /* U+02D1 */
  0x0000,   /* U+02D2 */
  0x0000,   /* U+02D3 */
  0x0000,   /* U+02D4 */
  0x0000,   /* U+02D5 */
  0x0000,   /* U+02D6 */
  0x0000,   /* U+02D7 */
  0x0000,   /* U+02D8 */
  0x0000,   /* U+02D9 */
  0x0000,   /* U+02DA */
  0x0000,   /* U+02DB */
  0x0000,   /* U+02DC */
  0x0000,   /* U+02DD */
  0x0000,   /* U+02DE */
  0x0000,   /* U+02DF */
  0x0000,   /* U+02E0 */
  0x0000,   /* U+02E1 */
  0x0000,   /* U+02E2 */
  0x0000,   /* U+02E3 */
  0x0000,   /* U+02E4 */
  0x0000,   /* U+02E5 */
  0x0000,   /* U+02E6 */
  0x0000,   /* U+02E7 */
  0x0000,   /* U+02E8 */
  0x0000,   /* U+02E9 */
  0x0000,   /* U+02EA */
  0x0000,   /* U+02EB */
  0x0000,   /* U+02EC */
  0x0000,   /* U+02ED */
  0x0000,   /* U+02EE */
  0x0000,   /* U+02EF */
  0x0000,   /* U+02F0 */
  0x0000,   /* U+02F1 */
  0x0000,   /* U+02F2 */
  0x0000,   /* U+02F3 */
  0x0000,   /* U+02F4 */
  0x0000,   /* U+02F5 */
  0x0000,   /* U+02F6 */
  0x0000,   /* U+02F7 */
  0x0000,   /* U+02F8 */
  0x0000,   /* U+02F9 */
  0x0000,   /* U+02FA */
  0x0000,   /* U+02FB */
  0x0000,   /* U+02FC */
  0x0000,   /* U+02FD */
  0x0000,   /* U+02FE */
  0x0000,   /* U+02FF */
};

uint16 uca900_p003[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 1, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 
  1, 1, 1, 2, 2, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0000,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x1C47,   /* U+0363 */
  0x1CAA,   /* U+0364 */
  0x1D32,   /* U+0365 */
  0x1DDD,   /* U+0366 */
  0x1EB5,   /* U+0367 */
  0x1C7A,   /* U+0368 */
  0x1C8F,   /* U+0369 */
  0x1D18,   /* U+036A */
  0x1DAA,   /* U+036B */
  0x1E33,   /* U+036C */
  0x1E95,   /* U+036D */
  0x1EE3,   /* U+036E */
  0x1EFF,   /* U+036F */
  0x1FC3,   /* U+0370 */
  0x1FC3,   /* U+0371 */
  0x1FE4,   /* U+0372 */
  0x1FE4,   /* U+0373 */
  0x0493,   /* U+0374 */
  0x0494,   /* U+0375 */
  0x1FC0,   /* U+0376 */
  0x1FC0,   /* U+0377 */
  0xFBC0,   /* U+0378 */
  0xFBC0,   /* U+0379 */
  0x1FC6,   /* U+037A */
  0x1FD9,   /* U+037B */
  0x1FD8,   /* U+037C */
  0x1FDA,   /* U+037D */
  0x0234,   /* U+037E */
  0x1FC7,   /* U+037F */
  0xFBC0,   /* U+0380 */
  0xFBC0,   /* U+0381 */
  0xFBC0,   /* U+0382 */
  0xFBC0,   /* U+0383 */
  0x0483,   /* U+0384 */
  0x0489,   /* U+0385 */
  0x1FB9,   /* U+0386 */
  0x028B,   /* U+0387 */
  0x1FBE,   /* U+0388 */
  0x1FC4,   /* U+0389 */
  0x1FC6,   /* U+038A */
  0xFBC0,   /* U+038B */
  0x1FCE,   /* U+038C */
  0xFBC0,   /* U+038D */
  0x1FDC,   /* U+038E */
  0x1FE1,   /* U+038F */
  0x1FC6,   /* U+0390 */
  0x1FB9,   /* U+0391 */
  0x1FBA,   /* U+0392 */
  0x1FBB,   /* U+0393 */
  0x1FBD,   /* U+0394 */
  0x1FBE,   /* U+0395 */
  0x1FC2,   /* U+0396 */
  0x1FC4,   /* U+0397 */
  0x1FC5,   /* U+0398 */
  0x1FC6,   /* U+0399 */
  0x1FC8,   /* U+039A */
  0x1FC9,   /* U+039B */
  0x1FCB,   /* U+039C */
  0x1FCC,   /* U+039D */
  0x1FCD,   /* U+039E */
  0x1FCE,   /* U+039F */
  0x1FCF,   /* U+03A0 */
  0x1FD4,   /* U+03A1 */
  0xFBC0,   /* U+03A2 */
  0x1FD7,   /* U+03A3 */
  0x1FDB,   /* U+03A4 */
  0x1FDC,   /* U+03A5 */
  0x1FDD,   /* U+03A6 */
  0x1FDE,   /* U+03A7 */
  0x1FDF,   /* U+03A8 */
  0x1FE1,   /* U+03A9 */
  0x1FC6,   /* U+03AA */
  0x1FDC,   /* U+03AB */
  0x1FB9,   /* U+03AC */
  0x1FBE,   /* U+03AD */
  0x1FC4,   /* U+03AE */
  0x1FC6,   /* U+03AF */
  0x1FDC,   /* U+03B0 */
  0x1FB9,   /* U+03B1 */
  0x1FBA,   /* U+03B2 */
  0x1FBB,   /* U+03B3 */
  0x1FBD,   /* U+03B4 */
  0x1FBE,   /* U+03B5 */
  0x1FC2,   /* U+03B6 */
  0x1FC4,   /* U+03B7 */
  0x1FC5,   /* U+03B8 */
  0x1FC6,   /* U+03B9 */
  0x1FC8,   /* U+03BA */
  0x1FC9,   /* U+03BB */
  0x1FCB,   /* U+03BC */
  0x1FCC,   /* U+03BD */
  0x1FCD,   /* U+03BE */
  0x1FCE,   /* U+03BF */
  0x1FCF,   /* U+03C0 */
  0x1FD4,   /* U+03C1 */
  0x1FD7,   /* U+03C2 */
  0x1FD7,   /* U+03C3 */
  0x1FDB,   /* U+03C4 */
  0x1FDC,   /* U+03C5 */
  0x1FDD,   /* U+03C6 */
  0x1FDE,   /* U+03C7 */
  0x1FDF,   /* U+03C8 */
  0x1FE1,   /* U+03C9 */
  0x1FC6,   /* U+03CA */
  0x1FDC,   /* U+03CB */
  0x1FCE,   /* U+03CC */
  0x1FDC,   /* U+03CD */
  0x1FE1,   /* U+03CE */
  0x1FC8,   /* U+03CF */
  0x1FBA,   /* U+03D0 */
  0x1FC5,   /* U+03D1 */
  0x1FDC,   /* U+03D2 */
  0x1FDC,   /* U+03D3 */
  0x1FDC,   /* U+03D4 */
  0x1FDD,   /* U+03D5 */
  0x1FCF,   /* U+03D6 */
  0x1FC8,   /* U+03D7 */
  0x1FD3,   /* U+03D8 */
  0x1FD3,   /* U+03D9 */
  0x1FC1,   /* U+03DA */
  0x1FC1,   /* U+03DB */
  0x1FBF,   /* U+03DC */
  0x1FBF,   /* U+03DD */
  0x1FD2,   /* U+03DE */
  0x1FD2,   /* U+03DF */
  0x1FE3,   /* U+03E0 */
  0x1FE3,   /* U+03E1 */
  0x2005,   /* U+03E2 */
  0x2005,   /* U+03E3 */
  0x200A,   /* U+03E4 */
  0x200A,   /* U+03E5 */
  0x200B,   /* U+03E6 */
  0x200B,   /* U+03E7 */
  0x200E,   /* U+03E8 */
  0x200E,   /* U+03E9 */
  0x2015,   /* U+03EA */
  0x2015,   /* U+03EB */
  0x2018,   /* U+03EC */
  0x2018,   /* U+03ED */
  0x201C,   /* U+03EE */
  0x201C,   /* U+03EF */
  0x1FC8,   /* U+03F0 */
  0x1FD4,   /* U+03F1 */
  0x1FD7,   /* U+03F2 */
  0x1FC7,   /* U+03F3 */
  0x1FC5,   /* U+03F4 */
  0x1FBE,   /* U+03F5 */
  0x0611,   /* U+03F6 */
  0x1FE5,   /* U+03F7 */
  0x1FE5,   /* U+03F8 */
  0x1FD7,   /* U+03F9 */
  0x1FD1,   /* U+03FA */
  0x1FD1,   /* U+03FB */
  0x1FD6,   /* U+03FC */
  0x1FD9,   /* U+03FD */
  0x1FD8,   /* U+03FE */
  0x1FDA,   /* U+03FF */

  /* Secondary weight 1 for each character. */
  0x0025,   /* U+0300 */
  0x0024,   /* U+0301 */
  0x0027,   /* U+0302 */
  0x002D,   /* U+0303 */
  0x0032,   /* U+0304 */
  0x003A,   /* U+0305 */
  0x0026,   /* U+0306 */
  0x002E,   /* U+0307 */
  0x002B,   /* U+0308 */
  0x003B,   /* U+0309 */
  0x0029,   /* U+030A */
  0x002C,   /* U+030B */
  0x0028,   /* U+030C */
  0x0033,   /* U+030D */
  0x0033,   /* U+030E */
  0x003C,   /* U+030F */
  0x003D,   /* U+0310 */
  0x003E,   /* U+0311 */
  0x0033,   /* U+0312 */
  0x0022,   /* U+0313 */
  0x0023,   /* U+0314 */
  0x0033,   /* U+0315 */
  0x0034,   /* U+0316 */
  0x0034,   /* U+0317 */
  0x0034,   /* U+0318 */
  0x0034,   /* U+0319 */
  0x0033,   /* U+031A */
  0x003F,   /* U+031B */
  0x0034,   /* U+031C */
  0x0034,   /* U+031D */
  0x0034,   /* U+031E */
  0x0034,   /* U+031F */
  0x0034,   /* U+0320 */
  0x0040,   /* U+0321 */
  0x0041,   /* U+0322 */
  0x0042,   /* U+0323 */
  0x0043,   /* U+0324 */
  0x0044,   /* U+0325 */
  0x0045,   /* U+0326 */
  0x0030,   /* U+0327 */
  0x0031,   /* U+0328 */
  0x0034,   /* U+0329 */
  0x0034,   /* U+032A */
  0x0034,   /* U+032B */
  0x0034,   /* U+032C */
  0x0046,   /* U+032D */
  0x0047,   /* U+032E */
  0x0034,   /* U+032F */
  0x0048,   /* U+0330 */
  0x0049,   /* U+0331 */
  0x0021,   /* U+0332 */
  0x0034,   /* U+0333 */
  0x004A,   /* U+0334 */
  0x0039,   /* U+0335 */
  0x0035,   /* U+0336 */
  0x0035,   /* U+0337 */
  0x002F,   /* U+0338 */
  0x004B,   /* U+0339 */
  0x0034,   /* U+033A */
  0x0034,   /* U+033B */
  0x0034,   /* U+033C */
  0x0033,   /* U+033D */
  0x0033,   /* U+033E */
  0x0033,   /* U+033F */
  0x0025,   /* U+0340 */
  0x0024,   /* U+0341 */
  0x002A,   /* U+0342 */
  0x0022,   /* U+0343 */
  0x002B,   /* U+0344 */
  0x004C,   /* U+0345 */
  0x0033,   /* U+0346 */
  0x0034,   /* U+0347 */
  0x0034,   /* U+0348 */
  0x0034,   /* U+0349 */
  0x0033,   /* U+034A */
  0x0033,   /* U+034B */
  0x0033,   /* U+034C */
  0x0034,   /* U+034D */
  0x0034,   /* U+034E */
  0x0000,   /* U+034F */
  0x0033,   /* U+0350 */
  0x0033,   /* U+0351 */
  0x0033,   /* U+0352 */
  0x0034,   /* U+0353 */
  0x0034,   /* U+0354 */
  0x0034,   /* U+0355 */
  0x0034,   /* U+0356 */
  0x0033,   /* U+0357 */
  0x004D,   /* U+0358 */
  0x0034,   /* U+0359 */
  0x0034,   /* U+035A */
  0x0033,   /* U+035B */
  0x0034,   /* U+035C */
  0x0033,   /* U+035D */
  0x0033,   /* U+035E */
  0x0034,   /* U+035F */
  0x004E,   /* U+0360 */
  0x004F,   /* U+0361 */
  0x0034,   /* U+0362 */
  0x0020,   /* U+0363 */
  0x0020,   /* U+0364 */
  0x0020,   /* U+0365 */
  0x0020,   /* U+0366 */
  0x0020,   /* U+0367 */
  0x0020,   /* U+0368 */
  0x0020,   /* U+0369 */
  0x0020,   /* U+036A */
  0x0020,   /* U+036B */
  0x0020,   /* U+036C */
  0x0020,   /* U+036D */
  0x0020,   /* U+036E */
  0x0020,   /* U+036F */
  0x0020,   /* U+0370 */
  0x0020,   /* U+0371 */
  0x0020,   /* U+0372 */
  0x0020,   /* U+0373 */
  0x0020,   /* U+0374 */
  0x0020,   /* U+0375 */
  0x0020,   /* U+0376 */
  0x0020,   /* U+0377 */
  0x0020,   /* U+0378 */
  0x0020,   /* U+0379 */
  0x0020,   /* U+037A */
  0x0020,   /* U+037B */
  0x0020,   /* U+037C */
  0x0020,   /* U+037D */
  0x0020,   /* U+037E */
  0x0020,   /* U+037F */
  0x0020,   /* U+0380 */
  0x0020,   /* U+0381 */
  0x0020,   /* U+0382 */
  0x0020,   /* U+0383 */
  0x0020,   /* U+0384 */
  0x0020,   /* U+0385 */
  0x0020,   /* U+0386 */
  0x0020,   /* U+0387 */
  0x0020,   /* U+0388 */
  0x0020,   /* U+0389 */
  0x0020,   /* U+038A */
  0x0020,   /* U+038B */
  0x0020,   /* U+038C */
  0x0020,   /* U+038D */
  0x0020,   /* U+038E */
  0x0020,   /* U+038F */
  0x0020,   /* U+0390 */
  0x0020,   /* U+0391 */
  0x0020,   /* U+0392 */
  0x0020,   /* U+0393 */
  0x0020,   /* U+0394 */
  0x0020,   /* U+0395 */
  0x0020,   /* U+0396 */
  0x0020,   /* U+0397 */
  0x0020,   /* U+0398 */
  0x0020,   /* U+0399 */
  0x0020,   /* U+039A */
  0x0020,   /* U+039B */
  0x0020,   /* U+039C */
  0x0020,   /* U+039D */
  0x0020,   /* U+039E */
  0x0020,   /* U+039F */
  0x0020,   /* U+03A0 */
  0x0020,   /* U+03A1 */
  0x0020,   /* U+03A2 */
  0x0020,   /* U+03A3 */
  0x0020,   /* U+03A4 */
  0x0020,   /* U+03A5 */
  0x0020,   /* U+03A6 */
  0x0020,   /* U+03A7 */
  0x0020,   /* U+03A8 */
  0x0020,   /* U+03A9 */
  0x0020,   /* U+03AA */
  0x0020,   /* U+03AB */
  0x0020,   /* U+03AC */
  0x0020,   /* U+03AD */
  0x0020,   /* U+03AE */
  0x0020,   /* U+03AF */
  0x0020,   /* U+03B0 */
  0x0020,   /* U+03B1 */
  0x0020,   /* U+03B2 */
  0x0020,   /* U+03B3 */
  0x0020,   /* U+03B4 */
  0x0020,   /* U+03B5 */
  0x0020,   /* U+03B6 */
  0x0020,   /* U+03B7 */
  0x0020,   /* U+03B8 */
  0x0020,   /* U+03B9 */
  0x0020,   /* U+03BA */
  0x0020,   /* U+03BB */
  0x0020,   /* U+03BC */
  0x0020,   /* U+03BD */
  0x0020,   /* U+03BE */
  0x0020,   /* U+03BF */
  0x0020,   /* U+03C0 */
  0x0020,   /* U+03C1 */
  0x0020,   /* U+03C2 */
  0x0020,   /* U+03C3 */
  0x0020,   /* U+03C4 */
  0x0020,   /* U+03C5 */
  0x0020,   /* U+03C6 */
  0x0020,   /* U+03C7 */
  0x0020,   /* U+03C8 */
  0x0020,   /* U+03C9 */
  0x0020,   /* U+03CA */
  0x0020,   /* U+03CB */
  0x0020,   /* U+03CC */
  0x0020,   /* U+03CD */
  0x0020,   /* U+03CE */
  0x0020,   /* U+03CF */
  0x0020,   /* U+03D0 */
  0x0020,   /* U+03D1 */
  0x0020,   /* U+03D2 */
  0x0020,   /* U+03D3 */
  0x0020,   /* U+03D4 */
  0x0020,   /* U+03D5 */
  0x0020,   /* U+03D6 */
  0x0020,   /* U+03D7 */
  0x0020,   /* U+03D8 */
  0x0020,   /* U+03D9 */
  0x0020,   /* U+03DA */
  0x0020,   /* U+03DB */
  0x0020,   /* U+03DC */
  0x0020,   /* U+03DD */
  0x0020,   /* U+03DE */
  0x0020,   /* U+03DF */
  0x0020,   /* U+03E0 */
  0x0020,   /* U+03E1 */
  0x0020,   /* U+03E2 */
  0x0020,   /* U+03E3 */
  0x0020,   /* U+03E4 */
  0x0020,   /* U+03E5 */
  0x0020,   /* U+03E6 */
  0x0020,   /* U+03E7 */
  0x0020,   /* U+03E8 */
  0x0020,   /* U+03E9 */
  0x0020,   /* U+03EA */
  0x0020,   /* U+03EB */
  0x0020,   /* U+03EC */
  0x0020,   /* U+03ED */
  0x0020,   /* U+03EE */
  0x0020,   /* U+03EF */
  0x0020,   /* U+03F0 */
  0x0020,   /* U+03F1 */
  0x0020,   /* U+03F2 */
  0x0020,   /* U+03F3 */
  0x0020,   /* U+03F4 */
  0x0020,   /* U+03F5 */
  0x0020,   /* U+03F6 */
  0x0020,   /* U+03F7 */
  0x0020,   /* U+03F8 */
  0x0020,   /* U+03F9 */
  0x0020,   /* U+03FA */
  0x0020,   /* U+03FB */
  0x0020,   /* U+03FC */
  0x0020,   /* U+03FD */
  0x0020,   /* U+03FE */
  0x0020,   /* U+03FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0300 */
  0x0002,   /* U+0301 */
  0x0002,   /* U+0302 */
  0x0002,   /* U+0303 */
  0x0002,   /* U+0304 */
  0x0002,   /* U+0305 */
  0x0002,   /* U+0306 */
  0x0002,   /* U+0307 */
  0x0002,   /* U+0308 */
  0x0002,   /* U+0309 */
  0x0002,   /* U+030A */
  0x0002,   /* U+030B */
  0x0002,   /* U+030C */
  0x0002,   /* U+030D */
  0x0002,   /* U+030E */
  0x0002,   /* U+030F */
  0x0002,   /* U+0310 */
  0x0002,   /* U+0311 */
  0x0002,   /* U+0312 */
  0x0002,   /* U+0313 */
  0x0002,   /* U+0314 */
  0x0002,   /* U+0315 */
  0x0002,   /* U+0316 */
  0x0002,   /* U+0317 */
  0x0002,   /* U+0318 */
  0x0002,   /* U+0319 */
  0x0002,   /* U+031A */
  0x0002,   /* U+031B */
  0x0002,   /* U+031C */
  0x0002,   /* U+031D */
  0x0002,   /* U+031E */
  0x0002,   /* U+031F */
  0x0002,   /* U+0320 */
  0x0002,   /* U+0321 */
  0x0002,   /* U+0322 */
  0x0002,   /* U+0323 */
  0x0002,   /* U+0324 */
  0x0002,   /* U+0325 */
  0x0002,   /* U+0326 */
  0x0002,   /* U+0327 */
  0x0002,   /* U+0328 */
  0x0002,   /* U+0329 */
  0x0002,   /* U+032A */
  0x0002,   /* U+032B */
  0x0002,   /* U+032C */
  0x0002,   /* U+032D */
  0x0002,   /* U+032E */
  0x0002,   /* U+032F */
  0x0002,   /* U+0330 */
  0x0002,   /* U+0331 */
  0x0002,   /* U+0332 */
  0x0002,   /* U+0333 */
  0x0002,   /* U+0334 */
  0x0002,   /* U+0335 */
  0x0002,   /* U+0336 */
  0x0002,   /* U+0337 */
  0x0002,   /* U+0338 */
  0x0002,   /* U+0339 */
  0x0002,   /* U+033A */
  0x0002,   /* U+033B */
  0x0002,   /* U+033C */
  0x0002,   /* U+033D */
  0x0002,   /* U+033E */
  0x0002,   /* U+033F */
  0x0002,   /* U+0340 */
  0x0002,   /* U+0341 */
  0x0002,   /* U+0342 */
  0x0002,   /* U+0343 */
  0x0002,   /* U+0344 */
  0x0002,   /* U+0345 */
  0x0002,   /* U+0346 */
  0x0002,   /* U+0347 */
  0x0002,   /* U+0348 */
  0x0002,   /* U+0349 */
  0x0002,   /* U+034A */
  0x0002,   /* U+034B */
  0x0002,   /* U+034C */
  0x0002,   /* U+034D */
  0x0002,   /* U+034E */
  0x0000,   /* U+034F */
  0x0002,   /* U+0350 */
  0x0002,   /* U+0351 */
  0x0002,   /* U+0352 */
  0x0002,   /* U+0353 */
  0x0002,   /* U+0354 */
  0x0002,   /* U+0355 */
  0x0002,   /* U+0356 */
  0x0002,   /* U+0357 */
  0x0002,   /* U+0358 */
  0x0002,   /* U+0359 */
  0x0002,   /* U+035A */
  0x0002,   /* U+035B */
  0x0002,   /* U+035C */
  0x0002,   /* U+035D */
  0x0002,   /* U+035E */
  0x0002,   /* U+035F */
  0x0002,   /* U+0360 */
  0x0002,   /* U+0361 */
  0x0002,   /* U+0362 */
  0x0004,   /* U+0363 */
  0x0004,   /* U+0364 */
  0x0004,   /* U+0365 */
  0x0004,   /* U+0366 */
  0x0004,   /* U+0367 */
  0x0004,   /* U+0368 */
  0x0004,   /* U+0369 */
  0x0004,   /* U+036A */
  0x0004,   /* U+036B */
  0x0004,   /* U+036C */
  0x0004,   /* U+036D */
  0x0004,   /* U+036E */
  0x0004,   /* U+036F */
  0x0008,   /* U+0370 */
  0x0002,   /* U+0371 */
  0x0008,   /* U+0372 */
  0x0002,   /* U+0373 */
  0x0002,   /* U+0374 */
  0x0002,   /* U+0375 */
  0x0008,   /* U+0376 */
  0x0002,   /* U+0377 */
  0x0002,   /* U+0378 */
  0x0002,   /* U+0379 */
  0x0004,   /* U+037A */
  0x0002,   /* U+037B */
  0x0002,   /* U+037C */
  0x0002,   /* U+037D */
  0x0002,   /* U+037E */
  0x0008,   /* U+037F */
  0x0002,   /* U+0380 */
  0x0002,   /* U+0381 */
  0x0002,   /* U+0382 */
  0x0002,   /* U+0383 */
  0x0002,   /* U+0384 */
  0x0002,   /* U+0385 */
  0x0008,   /* U+0386 */
  0x0002,   /* U+0387 */
  0x0008,   /* U+0388 */
  0x0008,   /* U+0389 */
  0x0008,   /* U+038A */
  0x0002,   /* U+038B */
  0x0008,   /* U+038C */
  0x0002,   /* U+038D */
  0x0008,   /* U+038E */
  0x0008,   /* U+038F */
  0x0002,   /* U+0390 */
  0x0008,   /* U+0391 */
  0x0008,   /* U+0392 */
  0x0008,   /* U+0393 */
  0x0008,   /* U+0394 */
  0x0008,   /* U+0395 */
  0x0008,   /* U+0396 */
  0x0008,   /* U+0397 */
  0x0008,   /* U+0398 */
  0x0008,   /* U+0399 */
  0x0008,   /* U+039A */
  0x0008,   /* U+039B */
  0x0008,   /* U+039C */
  0x0008,   /* U+039D */
  0x0008,   /* U+039E */
  0x0008,   /* U+039F */
  0x0008,   /* U+03A0 */
  0x0008,   /* U+03A1 */
  0x0002,   /* U+03A2 */
  0x0008,   /* U+03A3 */
  0x0008,   /* U+03A4 */
  0x0008,   /* U+03A5 */
  0x0008,   /* U+03A6 */
  0x0008,   /* U+03A7 */
  0x0008,   /* U+03A8 */
  0x0008,   /* U+03A9 */
  0x0008,   /* U+03AA */
  0x0008,   /* U+03AB */
  0x0002,   /* U+03AC */
  0x0002,   /* U+03AD */
  0x0002,   /* U+03AE */
  0x0002,   /* U+03AF */
  0x0002,   /* U+03B0 */
  0x0002,   /* U+03B1 */
  0x0002,   /* U+03B2 */
  0x0002,   /* U+03B3 */
  0x0002,   /* U+03B4 */
  0x0002,   /* U+03B5 */
  0x0002,   /* U+03B6 */
  0x0002,   /* U+03B7 */
  0x0002,   /* U+03B8 */
  0x0002,   /* U+03B9 */
  0x0002,   /* U+03BA */
  0x0002,   /* U+03BB */
  0x0002,   /* U+03BC */
  0x0002,   /* U+03BD */
  0x0002,   /* U+03BE */
  0x0002,   /* U+03BF */
  0x0002,   /* U+03C0 */
  0x0002,   /* U+03C1 */
  0x0019,   /* U+03C2 */
  0x0002,   /* U+03C3 */
  0x0002,   /* U+03C4 */
  0x0002,   /* U+03C5 */
  0x0002,   /* U+03C6 */
  0x0002,   /* U+03C7 */
  0x0002,   /* U+03C8 */
  0x0002,   /* U+03C9 */
  0x0002,   /* U+03CA */
  0x0002,   /* U+03CB */
  0x0002,   /* U+03CC */
  0x0002,   /* U+03CD */
  0x0002,   /* U+03CE */
  0x000A,   /* U+03CF */
  0x0004,   /* U+03D0 */
  0x0004,   /* U+03D1 */
  0x000A,   /* U+03D2 */
  0x000A,   /* U+03D3 */
  0x000A,   /* U+03D4 */
  0x0004,   /* U+03D5 */
  0x0004,   /* U+03D6 */
  0x0004,   /* U+03D7 */
  0x0008,   /* U+03D8 */
  0x0002,   /* U+03D9 */
  0x0008,   /* U+03DA */
  0x0002,   /* U+03DB */
  0x0008,   /* U+03DC */
  0x0002,   /* U+03DD */
  0x0008,   /* U+03DE */
  0x0002,   /* U+03DF */
  0x0008,   /* U+03E0 */
  0x0002,   /* U+03E1 */
  0x0008,   /* U+03E2 */
  0x0002,   /* U+03E3 */
  0x0008,   /* U+03E4 */
  0x0002,   /* U+03E5 */
  0x0008,   /* U+03E6 */
  0x0002,   /* U+03E7 */
  0x0008,   /* U+03E8 */
  0x0002,   /* U+03E9 */
  0x0008,   /* U+03EA */
  0x0002,   /* U+03EB */
  0x0008,   /* U+03EC */
  0x0002,   /* U+03ED */
  0x0008,   /* U+03EE */
  0x0002,   /* U+03EF */
  0x0004,   /* U+03F0 */
  0x0004,   /* U+03F1 */
  0x0004,   /* U+03F2 */
  0x0002,   /* U+03F3 */
  0x000A,   /* U+03F4 */
  0x0004,   /* U+03F5 */
  0x0002,   /* U+03F6 */
  0x0008,   /* U+03F7 */
  0x0002,   /* U+03F8 */
  0x000A,   /* U+03F9 */
  0x0008,   /* U+03FA */
  0x0002,   /* U+03FB */
  0x0002,   /* U+03FC */
  0x0008,   /* U+03FD */
  0x0008,   /* U+03FE */
  0x0008,   /* U+03FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0000,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x8378,   /* U+0378 */
  0x8379,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x8380,   /* U+0380 */
  0x8381,   /* U+0381 */
  0x8382,   /* U+0382 */
  0x8383,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0000,   /* U+0385 */
  0x0000,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0000,   /* U+0388 */
  0x0000,   /* U+0389 */
  0x0000,   /* U+038A */
  0x838B,   /* U+038B */
  0x0000,   /* U+038C */
  0x838D,   /* U+038D */
  0x0000,   /* U+038E */
  0x0000,   /* U+038F */
  0x0000,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x83A2,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x0000,   /* U+03AA */
  0x0000,   /* U+03AB */
  0x0000,   /* U+03AC */
  0x0000,   /* U+03AD */
  0x0000,   /* U+03AE */
  0x0000,   /* U+03AF */
  0x0000,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x0000,   /* U+03CA */
  0x0000,   /* U+03CB */
  0x0000,   /* U+03CC */
  0x0000,   /* U+03CD */
  0x0000,   /* U+03CE */
  0x1FB9,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0000,   /* U+03D3 */
  0x0000,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x1FB9,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0024,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x0000,   /* U+0378 */
  0x0000,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x0000,   /* U+0380 */
  0x0000,   /* U+0381 */
  0x0000,   /* U+0382 */
  0x0000,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0024,   /* U+0385 */
  0x0024,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0024,   /* U+0388 */
  0x0024,   /* U+0389 */
  0x0024,   /* U+038A */
  0x0000,   /* U+038B */
  0x0024,   /* U+038C */
  0x0000,   /* U+038D */
  0x0024,   /* U+038E */
  0x0024,   /* U+038F */
  0x002B,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x0000,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x002B,   /* U+03AA */
  0x002B,   /* U+03AB */
  0x0024,   /* U+03AC */
  0x0024,   /* U+03AD */
  0x0024,   /* U+03AE */
  0x0024,   /* U+03AF */
  0x002B,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x002B,   /* U+03CA */
  0x002B,   /* U+03CB */
  0x0024,   /* U+03CC */
  0x0024,   /* U+03CD */
  0x0024,   /* U+03CE */
  0x0020,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0024,   /* U+03D3 */
  0x002B,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x0020,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0002,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x0000,   /* U+0378 */
  0x0000,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x0000,   /* U+0380 */
  0x0000,   /* U+0381 */
  0x0000,   /* U+0382 */
  0x0000,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0002,   /* U+0385 */
  0x0002,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0002,   /* U+0388 */
  0x0002,   /* U+0389 */
  0x0002,   /* U+038A */
  0x0000,   /* U+038B */
  0x0002,   /* U+038C */
  0x0000,   /* U+038D */
  0x0002,   /* U+038E */
  0x0002,   /* U+038F */
  0x0002,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x0000,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x0002,   /* U+03AA */
  0x0002,   /* U+03AB */
  0x0002,   /* U+03AC */
  0x0002,   /* U+03AD */
  0x0002,   /* U+03AE */
  0x0002,   /* U+03AF */
  0x0002,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x0002,   /* U+03CA */
  0x0002,   /* U+03CB */
  0x0002,   /* U+03CC */
  0x0002,   /* U+03CD */
  0x0002,   /* U+03CE */
  0x0004,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0002,   /* U+03D3 */
  0x0002,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x0004,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0000,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x0000,   /* U+0378 */
  0x0000,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x0000,   /* U+0380 */
  0x0000,   /* U+0381 */
  0x0000,   /* U+0382 */
  0x0000,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0000,   /* U+0385 */
  0x0000,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0000,   /* U+0388 */
  0x0000,   /* U+0389 */
  0x0000,   /* U+038A */
  0x0000,   /* U+038B */
  0x0000,   /* U+038C */
  0x0000,   /* U+038D */
  0x0000,   /* U+038E */
  0x0000,   /* U+038F */
  0x0000,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x0000,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x0000,   /* U+03AA */
  0x0000,   /* U+03AB */
  0x0000,   /* U+03AC */
  0x0000,   /* U+03AD */
  0x0000,   /* U+03AE */
  0x0000,   /* U+03AF */
  0x0000,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x0000,   /* U+03CA */
  0x0000,   /* U+03CB */
  0x0000,   /* U+03CC */
  0x0000,   /* U+03CD */
  0x0000,   /* U+03CE */
  0x1FC6,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0000,   /* U+03D3 */
  0x0000,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x1FC6,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0000,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x0000,   /* U+0378 */
  0x0000,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x0000,   /* U+0380 */
  0x0000,   /* U+0381 */
  0x0000,   /* U+0382 */
  0x0000,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0000,   /* U+0385 */
  0x0000,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0000,   /* U+0388 */
  0x0000,   /* U+0389 */
  0x0000,   /* U+038A */
  0x0000,   /* U+038B */
  0x0000,   /* U+038C */
  0x0000,   /* U+038D */
  0x0000,   /* U+038E */
  0x0000,   /* U+038F */
  0x0024,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x0000,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x0000,   /* U+03AA */
  0x0000,   /* U+03AB */
  0x0000,   /* U+03AC */
  0x0000,   /* U+03AD */
  0x0000,   /* U+03AE */
  0x0000,   /* U+03AF */
  0x0024,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x0000,   /* U+03CA */
  0x0000,   /* U+03CB */
  0x0000,   /* U+03CC */
  0x0000,   /* U+03CD */
  0x0000,   /* U+03CE */
  0x0020,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0000,   /* U+03D3 */
  0x0000,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x0020,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+0300 */
  0x0000,   /* U+0301 */
  0x0000,   /* U+0302 */
  0x0000,   /* U+0303 */
  0x0000,   /* U+0304 */
  0x0000,   /* U+0305 */
  0x0000,   /* U+0306 */
  0x0000,   /* U+0307 */
  0x0000,   /* U+0308 */
  0x0000,   /* U+0309 */
  0x0000,   /* U+030A */
  0x0000,   /* U+030B */
  0x0000,   /* U+030C */
  0x0000,   /* U+030D */
  0x0000,   /* U+030E */
  0x0000,   /* U+030F */
  0x0000,   /* U+0310 */
  0x0000,   /* U+0311 */
  0x0000,   /* U+0312 */
  0x0000,   /* U+0313 */
  0x0000,   /* U+0314 */
  0x0000,   /* U+0315 */
  0x0000,   /* U+0316 */
  0x0000,   /* U+0317 */
  0x0000,   /* U+0318 */
  0x0000,   /* U+0319 */
  0x0000,   /* U+031A */
  0x0000,   /* U+031B */
  0x0000,   /* U+031C */
  0x0000,   /* U+031D */
  0x0000,   /* U+031E */
  0x0000,   /* U+031F */
  0x0000,   /* U+0320 */
  0x0000,   /* U+0321 */
  0x0000,   /* U+0322 */
  0x0000,   /* U+0323 */
  0x0000,   /* U+0324 */
  0x0000,   /* U+0325 */
  0x0000,   /* U+0326 */
  0x0000,   /* U+0327 */
  0x0000,   /* U+0328 */
  0x0000,   /* U+0329 */
  0x0000,   /* U+032A */
  0x0000,   /* U+032B */
  0x0000,   /* U+032C */
  0x0000,   /* U+032D */
  0x0000,   /* U+032E */
  0x0000,   /* U+032F */
  0x0000,   /* U+0330 */
  0x0000,   /* U+0331 */
  0x0000,   /* U+0332 */
  0x0000,   /* U+0333 */
  0x0000,   /* U+0334 */
  0x0000,   /* U+0335 */
  0x0000,   /* U+0336 */
  0x0000,   /* U+0337 */
  0x0000,   /* U+0338 */
  0x0000,   /* U+0339 */
  0x0000,   /* U+033A */
  0x0000,   /* U+033B */
  0x0000,   /* U+033C */
  0x0000,   /* U+033D */
  0x0000,   /* U+033E */
  0x0000,   /* U+033F */
  0x0000,   /* U+0340 */
  0x0000,   /* U+0341 */
  0x0000,   /* U+0342 */
  0x0000,   /* U+0343 */
  0x0000,   /* U+0344 */
  0x0000,   /* U+0345 */
  0x0000,   /* U+0346 */
  0x0000,   /* U+0347 */
  0x0000,   /* U+0348 */
  0x0000,   /* U+0349 */
  0x0000,   /* U+034A */
  0x0000,   /* U+034B */
  0x0000,   /* U+034C */
  0x0000,   /* U+034D */
  0x0000,   /* U+034E */
  0x0000,   /* U+034F */
  0x0000,   /* U+0350 */
  0x0000,   /* U+0351 */
  0x0000,   /* U+0352 */
  0x0000,   /* U+0353 */
  0x0000,   /* U+0354 */
  0x0000,   /* U+0355 */
  0x0000,   /* U+0356 */
  0x0000,   /* U+0357 */
  0x0000,   /* U+0358 */
  0x0000,   /* U+0359 */
  0x0000,   /* U+035A */
  0x0000,   /* U+035B */
  0x0000,   /* U+035C */
  0x0000,   /* U+035D */
  0x0000,   /* U+035E */
  0x0000,   /* U+035F */
  0x0000,   /* U+0360 */
  0x0000,   /* U+0361 */
  0x0000,   /* U+0362 */
  0x0000,   /* U+0363 */
  0x0000,   /* U+0364 */
  0x0000,   /* U+0365 */
  0x0000,   /* U+0366 */
  0x0000,   /* U+0367 */
  0x0000,   /* U+0368 */
  0x0000,   /* U+0369 */
  0x0000,   /* U+036A */
  0x0000,   /* U+036B */
  0x0000,   /* U+036C */
  0x0000,   /* U+036D */
  0x0000,   /* U+036E */
  0x0000,   /* U+036F */
  0x0000,   /* U+0370 */
  0x0000,   /* U+0371 */
  0x0000,   /* U+0372 */
  0x0000,   /* U+0373 */
  0x0000,   /* U+0374 */
  0x0000,   /* U+0375 */
  0x0000,   /* U+0376 */
  0x0000,   /* U+0377 */
  0x0000,   /* U+0378 */
  0x0000,   /* U+0379 */
  0x0000,   /* U+037A */
  0x0000,   /* U+037B */
  0x0000,   /* U+037C */
  0x0000,   /* U+037D */
  0x0000,   /* U+037E */
  0x0000,   /* U+037F */
  0x0000,   /* U+0380 */
  0x0000,   /* U+0381 */
  0x0000,   /* U+0382 */
  0x0000,   /* U+0383 */
  0x0000,   /* U+0384 */
  0x0000,   /* U+0385 */
  0x0000,   /* U+0386 */
  0x0000,   /* U+0387 */
  0x0000,   /* U+0388 */
  0x0000,   /* U+0389 */
  0x0000,   /* U+038A */
  0x0000,   /* U+038B */
  0x0000,   /* U+038C */
  0x0000,   /* U+038D */
  0x0000,   /* U+038E */
  0x0000,   /* U+038F */
  0x0002,   /* U+0390 */
  0x0000,   /* U+0391 */
  0x0000,   /* U+0392 */
  0x0000,   /* U+0393 */
  0x0000,   /* U+0394 */
  0x0000,   /* U+0395 */
  0x0000,   /* U+0396 */
  0x0000,   /* U+0397 */
  0x0000,   /* U+0398 */
  0x0000,   /* U+0399 */
  0x0000,   /* U+039A */
  0x0000,   /* U+039B */
  0x0000,   /* U+039C */
  0x0000,   /* U+039D */
  0x0000,   /* U+039E */
  0x0000,   /* U+039F */
  0x0000,   /* U+03A0 */
  0x0000,   /* U+03A1 */
  0x0000,   /* U+03A2 */
  0x0000,   /* U+03A3 */
  0x0000,   /* U+03A4 */
  0x0000,   /* U+03A5 */
  0x0000,   /* U+03A6 */
  0x0000,   /* U+03A7 */
  0x0000,   /* U+03A8 */
  0x0000,   /* U+03A9 */
  0x0000,   /* U+03AA */
  0x0000,   /* U+03AB */
  0x0000,   /* U+03AC */
  0x0000,   /* U+03AD */
  0x0000,   /* U+03AE */
  0x0000,   /* U+03AF */
  0x0002,   /* U+03B0 */
  0x0000,   /* U+03B1 */
  0x0000,   /* U+03B2 */
  0x0000,   /* U+03B3 */
  0x0000,   /* U+03B4 */
  0x0000,   /* U+03B5 */
  0x0000,   /* U+03B6 */
  0x0000,   /* U+03B7 */
  0x0000,   /* U+03B8 */
  0x0000,   /* U+03B9 */
  0x0000,   /* U+03BA */
  0x0000,   /* U+03BB */
  0x0000,   /* U+03BC */
  0x0000,   /* U+03BD */
  0x0000,   /* U+03BE */
  0x0000,   /* U+03BF */
  0x0000,   /* U+03C0 */
  0x0000,   /* U+03C1 */
  0x0000,   /* U+03C2 */
  0x0000,   /* U+03C3 */
  0x0000,   /* U+03C4 */
  0x0000,   /* U+03C5 */
  0x0000,   /* U+03C6 */
  0x0000,   /* U+03C7 */
  0x0000,   /* U+03C8 */
  0x0000,   /* U+03C9 */
  0x0000,   /* U+03CA */
  0x0000,   /* U+03CB */
  0x0000,   /* U+03CC */
  0x0000,   /* U+03CD */
  0x0000,   /* U+03CE */
  0x0004,   /* U+03CF */
  0x0000,   /* U+03D0 */
  0x0000,   /* U+03D1 */
  0x0000,   /* U+03D2 */
  0x0000,   /* U+03D3 */
  0x0000,   /* U+03D4 */
  0x0000,   /* U+03D5 */
  0x0000,   /* U+03D6 */
  0x0004,   /* U+03D7 */
  0x0000,   /* U+03D8 */
  0x0000,   /* U+03D9 */
  0x0000,   /* U+03DA */
  0x0000,   /* U+03DB */
  0x0000,   /* U+03DC */
  0x0000,   /* U+03DD */
  0x0000,   /* U+03DE */
  0x0000,   /* U+03DF */
  0x0000,   /* U+03E0 */
  0x0000,   /* U+03E1 */
  0x0000,   /* U+03E2 */
  0x0000,   /* U+03E3 */
  0x0000,   /* U+03E4 */
  0x0000,   /* U+03E5 */
  0x0000,   /* U+03E6 */
  0x0000,   /* U+03E7 */
  0x0000,   /* U+03E8 */
  0x0000,   /* U+03E9 */
  0x0000,   /* U+03EA */
  0x0000,   /* U+03EB */
  0x0000,   /* U+03EC */
  0x0000,   /* U+03ED */
  0x0000,   /* U+03EE */
  0x0000,   /* U+03EF */
  0x0000,   /* U+03F0 */
  0x0000,   /* U+03F1 */
  0x0000,   /* U+03F2 */
  0x0000,   /* U+03F3 */
  0x0000,   /* U+03F4 */
  0x0000,   /* U+03F5 */
  0x0000,   /* U+03F6 */
  0x0000,   /* U+03F7 */
  0x0000,   /* U+03F8 */
  0x0000,   /* U+03F9 */
  0x0000,   /* U+03FA */
  0x0000,   /* U+03FB */
  0x0000,   /* U+03FC */
  0x0000,   /* U+03FD */
  0x0000,   /* U+03FE */
  0x0000,   /* U+03FF */
};

uint16 uca900_p004[]= {
  /* Number of CEs for each character. */
  2, 2, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x205A,   /* U+0400 */
  0x205A,   /* U+0401 */
  0x2050,   /* U+0402 */
  0x2036,   /* U+0403 */
  0x205E,   /* U+0404 */
  0x2074,   /* U+0405 */
  0x2088,   /* U+0406 */
  0x2088,   /* U+0407 */
  0x2091,   /* U+0408 */
  0x20BC,   /* U+0409 */
  0x20E2,   /* U+040A */
  0x2119,   /* U+040B */
  0x2096,   /* U+040C */
  0x2080,   /* U+040D */
  0x211D,   /* U+040E */
  0x217F,   /* U+040F */
  0x2022,   /* U+0410 */
  0x202E,   /* U+0411 */
  0x2032,   /* U+0412 */
  0x2036,   /* U+0413 */
  0x204A,   /* U+0414 */
  0x205A,   /* U+0415 */
  0x2062,   /* U+0416 */
  0x206C,   /* U+0417 */
  0x2080,   /* U+0418 */
  0x208D,   /* U+0419 */
  0x2096,   /* U+041A */
  0x20B0,   /* U+041B */
  0x20C3,   /* U+041C */
  0x20CC,   /* U+041D */
  0x20E7,   /* U+041E */
  0x20EF,   /* U+041F */
  0x20FC,   /* U+0420 */
  0x2105,   /* U+0421 */
  0x210E,   /* U+0422 */
  0x211D,   /* U+0423 */
  0x212E,   /* U+0424 */
  0x2132,   /* U+0425 */
  0x2159,   /* U+0426 */
  0x2164,   /* U+0427 */
  0x2183,   /* U+0428 */
  0x2188,   /* U+0429 */
  0x218F,   /* U+042A */
  0x2194,   /* U+042B */
  0x2198,   /* U+042C */
  0x21A5,   /* U+042D */
  0x21A9,   /* U+042E */
  0x21AF,   /* U+042F */
  0x2022,   /* U+0430 */
  0x202E,   /* U+0431 */
  0x2032,   /* U+0432 */
  0x2036,   /* U+0433 */
  0x204A,   /* U+0434 */
  0x205A,   /* U+0435 */
  0x2062,   /* U+0436 */
  0x206C,   /* U+0437 */
  0x2080,   /* U+0438 */
  0x208D,   /* U+0439 */
  0x2096,   /* U+043A */
  0x20B0,   /* U+043B */
  0x20C3,   /* U+043C */
  0x20CC,   /* U+043D */
  0x20E7,   /* U+043E */
  0x20EF,   /* U+043F */
  0x20FC,   /* U+0440 */
  0x2105,   /* U+0441 */
  0x210E,   /* U+0442 */
  0x211D,   /* U+0443 */
  0x212E,   /* U+0444 */
  0x2132,   /* U+0445 */
  0x2159,   /* U+0446 */
  0x2164,   /* U+0447 */
  0x2183,   /* U+0448 */
  0x2188,   /* U+0449 */
  0x218F,   /* U+044A */
  0x2194,   /* U+044B */
  0x2198,   /* U+044C */
  0x21A5,   /* U+044D */
  0x21A9,   /* U+044E */
  0x21AF,   /* U+044F */
  0x205A,   /* U+0450 */
  0x205A,   /* U+0451 */
  0x2050,   /* U+0452 */
  0x2036,   /* U+0453 */
  0x205E,   /* U+0454 */
  0x2074,   /* U+0455 */
  0x2088,   /* U+0456 */
  0x2088,   /* U+0457 */
  0x2091,   /* U+0458 */
  0x20BC,   /* U+0459 */
  0x20E2,   /* U+045A */
  0x2119,   /* U+045B */
  0x2096,   /* U+045C */
  0x2080,   /* U+045D */
  0x211D,   /* U+045E */
  0x217F,   /* U+045F */
  0x2148,   /* U+0460 */
  0x2148,   /* U+0461 */
  0x21A0,   /* U+0462 */
  0x21A0,   /* U+0463 */
  0x21B4,   /* U+0464 */
  0x21B4,   /* U+0465 */
  0x21B8,   /* U+0466 */
  0x21B8,   /* U+0467 */
  0x21C2,   /* U+0468 */
  0x21C2,   /* U+0469 */
  0x21BD,   /* U+046A */
  0x21BD,   /* U+046B */
  0x21C7,   /* U+046C */
  0x21C7,   /* U+046D */
  0x21CB,   /* U+046E */
  0x21CB,   /* U+046F */
  0x21CF,   /* U+0470 */
  0x21CF,   /* U+0471 */
  0x21D3,   /* U+0472 */
  0x21D3,   /* U+0473 */
  0x21D7,   /* U+0474 */
  0x21D7,   /* U+0475 */
  0x21D7,   /* U+0476 */
  0x21D7,   /* U+0477 */
  0x212A,   /* U+0478 */
  0x212A,   /* U+0479 */
  0x2155,   /* U+047A */
  0x2155,   /* U+047B */
  0x2151,   /* U+047C */
  0x2151,   /* U+047D */
  0x214C,   /* U+047E */
  0x214C,   /* U+047F */
  0x20F8,   /* U+0480 */
  0x20F8,   /* U+0481 */
  0x04F7,   /* U+0482 */
  0x0000,   /* U+0483 */
  0x0000,   /* U+0484 */
  0x0000,   /* U+0485 */
  0x0000,   /* U+0486 */
  0x0000,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x2084,   /* U+048A */
  0x2084,   /* U+048B */
  0x219C,   /* U+048C */
  0x219C,   /* U+048D */
  0x2100,   /* U+048E */
  0x2100,   /* U+048F */
  0x2036,   /* U+0490 */
  0x2036,   /* U+0491 */
  0x203A,   /* U+0492 */
  0x203A,   /* U+0493 */
  0x2042,   /* U+0494 */
  0x2042,   /* U+0495 */
  0x2068,   /* U+0496 */
  0x2068,   /* U+0497 */
  0x2056,   /* U+0498 */
  0x2056,   /* U+0499 */
  0x209A,   /* U+049A */
  0x209A,   /* U+049B */
  0x20AA,   /* U+049C */
  0x20AA,   /* U+049D */
  0x20A6,   /* U+049E */
  0x20A6,   /* U+049F */
  0x20A2,   /* U+04A0 */
  0x20A2,   /* U+04A1 */
  0x20D5,   /* U+04A2 */
  0x20D5,   /* U+04A3 */
  0x20DE,   /* U+04A4 */
  0x20DE,   /* U+04A5 */
  0x20F4,   /* U+04A6 */
  0x20F4,   /* U+04A7 */
  0x21DC,   /* U+04A8 */
  0x21DC,   /* U+04A9 */
  0x210A,   /* U+04AA */
  0x210A,   /* U+04AB */
  0x2114,   /* U+04AC */
  0x2114,   /* U+04AD */
  0x2121,   /* U+04AE */
  0x2121,   /* U+04AF */
  0x2125,   /* U+04B0 */
  0x2125,   /* U+04B1 */
  0x213E,   /* U+04B2 */
  0x213E,   /* U+04B3 */
  0x215F,   /* U+04B4 */
  0x215F,   /* U+04B5 */
  0x216A,   /* U+04B6 */
  0x216A,   /* U+04B7 */
  0x2172,   /* U+04B8 */
  0x2172,   /* U+04B9 */
  0x2142,   /* U+04BA */
  0x2142,   /* U+04BB */
  0x2177,   /* U+04BC */
  0x2177,   /* U+04BD */
  0x217B,   /* U+04BE */
  0x217B,   /* U+04BF */
  0x21E1,   /* U+04C0 */
  0x2062,   /* U+04C1 */
  0x2062,   /* U+04C2 */
  0x209E,   /* U+04C3 */
  0x209E,   /* U+04C4 */
  0x20B5,   /* U+04C5 */
  0x20B5,   /* U+04C6 */
  0x20D9,   /* U+04C7 */
  0x20D9,   /* U+04C8 */
  0x20D1,   /* U+04C9 */
  0x20D1,   /* U+04CA */
  0x216E,   /* U+04CB */
  0x216E,   /* U+04CC */
  0x20C7,   /* U+04CD */
  0x20C7,   /* U+04CE */
  0x21E1,   /* U+04CF */
  0x2022,   /* U+04D0 */
  0x2022,   /* U+04D1 */
  0x2022,   /* U+04D2 */
  0x2022,   /* U+04D3 */
  0x202A,   /* U+04D4 */
  0x202A,   /* U+04D5 */
  0x205A,   /* U+04D6 */
  0x205A,   /* U+04D7 */
  0x2026,   /* U+04D8 */
  0x2026,   /* U+04D9 */
  0x2026,   /* U+04DA */
  0x2026,   /* U+04DB */
  0x2062,   /* U+04DC */
  0x2062,   /* U+04DD */
  0x206C,   /* U+04DE */
  0x206C,   /* U+04DF */
  0x2079,   /* U+04E0 */
  0x2079,   /* U+04E1 */
  0x2080,   /* U+04E2 */
  0x2080,   /* U+04E3 */
  0x2080,   /* U+04E4 */
  0x2080,   /* U+04E5 */
  0x20E7,   /* U+04E6 */
  0x20E7,   /* U+04E7 */
  0x20EB,   /* U+04E8 */
  0x20EB,   /* U+04E9 */
  0x20EB,   /* U+04EA */
  0x20EB,   /* U+04EB */
  0x21A5,   /* U+04EC */
  0x21A5,   /* U+04ED */
  0x211D,   /* U+04EE */
  0x211D,   /* U+04EF */
  0x211D,   /* U+04F0 */
  0x211D,   /* U+04F1 */
  0x211D,   /* U+04F2 */
  0x211D,   /* U+04F3 */
  0x2164,   /* U+04F4 */
  0x2164,   /* U+04F5 */
  0x2046,   /* U+04F6 */
  0x2046,   /* U+04F7 */
  0x2194,   /* U+04F8 */
  0x2194,   /* U+04F9 */
  0x203E,   /* U+04FA */
  0x203E,   /* U+04FB */
  0x2136,   /* U+04FC */
  0x2136,   /* U+04FD */
  0x213A,   /* U+04FE */
  0x213A,   /* U+04FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0400 */
  0x0020,   /* U+0401 */
  0x0020,   /* U+0402 */
  0x0020,   /* U+0403 */
  0x0020,   /* U+0404 */
  0x0020,   /* U+0405 */
  0x0020,   /* U+0406 */
  0x0020,   /* U+0407 */
  0x0020,   /* U+0408 */
  0x0020,   /* U+0409 */
  0x0020,   /* U+040A */
  0x0020,   /* U+040B */
  0x0020,   /* U+040C */
  0x0020,   /* U+040D */
  0x0020,   /* U+040E */
  0x0020,   /* U+040F */
  0x0020,   /* U+0410 */
  0x0020,   /* U+0411 */
  0x0020,   /* U+0412 */
  0x0020,   /* U+0413 */
  0x0020,   /* U+0414 */
  0x0020,   /* U+0415 */
  0x0020,   /* U+0416 */
  0x0020,   /* U+0417 */
  0x0020,   /* U+0418 */
  0x0020,   /* U+0419 */
  0x0020,   /* U+041A */
  0x0020,   /* U+041B */
  0x0020,   /* U+041C */
  0x0020,   /* U+041D */
  0x0020,   /* U+041E */
  0x0020,   /* U+041F */
  0x0020,   /* U+0420 */
  0x0020,   /* U+0421 */
  0x0020,   /* U+0422 */
  0x0020,   /* U+0423 */
  0x0020,   /* U+0424 */
  0x0020,   /* U+0425 */
  0x0020,   /* U+0426 */
  0x0020,   /* U+0427 */
  0x0020,   /* U+0428 */
  0x0020,   /* U+0429 */
  0x0020,   /* U+042A */
  0x0020,   /* U+042B */
  0x0020,   /* U+042C */
  0x0020,   /* U+042D */
  0x0020,   /* U+042E */
  0x0020,   /* U+042F */
  0x0020,   /* U+0430 */
  0x0020,   /* U+0431 */
  0x0020,   /* U+0432 */
  0x0020,   /* U+0433 */
  0x0020,   /* U+0434 */
  0x0020,   /* U+0435 */
  0x0020,   /* U+0436 */
  0x0020,   /* U+0437 */
  0x0020,   /* U+0438 */
  0x0020,   /* U+0439 */
  0x0020,   /* U+043A */
  0x0020,   /* U+043B */
  0x0020,   /* U+043C */
  0x0020,   /* U+043D */
  0x0020,   /* U+043E */
  0x0020,   /* U+043F */
  0x0020,   /* U+0440 */
  0x0020,   /* U+0441 */
  0x0020,   /* U+0442 */
  0x0020,   /* U+0443 */
  0x0020,   /* U+0444 */
  0x0020,   /* U+0445 */
  0x0020,   /* U+0446 */
  0x0020,   /* U+0447 */
  0x0020,   /* U+0448 */
  0x0020,   /* U+0449 */
  0x0020,   /* U+044A */
  0x0020,   /* U+044B */
  0x0020,   /* U+044C */
  0x0020,   /* U+044D */
  0x0020,   /* U+044E */
  0x0020,   /* U+044F */
  0x0020,   /* U+0450 */
  0x0020,   /* U+0451 */
  0x0020,   /* U+0452 */
  0x0020,   /* U+0453 */
  0x0020,   /* U+0454 */
  0x0020,   /* U+0455 */
  0x0020,   /* U+0456 */
  0x0020,   /* U+0457 */
  0x0020,   /* U+0458 */
  0x0020,   /* U+0459 */
  0x0020,   /* U+045A */
  0x0020,   /* U+045B */
  0x0020,   /* U+045C */
  0x0020,   /* U+045D */
  0x0020,   /* U+045E */
  0x0020,   /* U+045F */
  0x0020,   /* U+0460 */
  0x0020,   /* U+0461 */
  0x0020,   /* U+0462 */
  0x0020,   /* U+0463 */
  0x0020,   /* U+0464 */
  0x0020,   /* U+0465 */
  0x0020,   /* U+0466 */
  0x0020,   /* U+0467 */
  0x0020,   /* U+0468 */
  0x0020,   /* U+0469 */
  0x0020,   /* U+046A */
  0x0020,   /* U+046B */
  0x0020,   /* U+046C */
  0x0020,   /* U+046D */
  0x0020,   /* U+046E */
  0x0020,   /* U+046F */
  0x0020,   /* U+0470 */
  0x0020,   /* U+0471 */
  0x0020,   /* U+0472 */
  0x0020,   /* U+0473 */
  0x0020,   /* U+0474 */
  0x0020,   /* U+0475 */
  0x0020,   /* U+0476 */
  0x0020,   /* U+0477 */
  0x0020,   /* U+0478 */
  0x0020,   /* U+0479 */
  0x0020,   /* U+047A */
  0x0020,   /* U+047B */
  0x0020,   /* U+047C */
  0x0020,   /* U+047D */
  0x0020,   /* U+047E */
  0x0020,   /* U+047F */
  0x0020,   /* U+0480 */
  0x0020,   /* U+0481 */
  0x0020,   /* U+0482 */
  0x0050,   /* U+0483 */
  0x0033,   /* U+0484 */
  0x0023,   /* U+0485 */
  0x0022,   /* U+0486 */
  0x0033,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x0020,   /* U+048A */
  0x0020,   /* U+048B */
  0x0020,   /* U+048C */
  0x0020,   /* U+048D */
  0x0020,   /* U+048E */
  0x0020,   /* U+048F */
  0x0020,   /* U+0490 */
  0x0020,   /* U+0491 */
  0x0020,   /* U+0492 */
  0x0020,   /* U+0493 */
  0x0020,   /* U+0494 */
  0x0020,   /* U+0495 */
  0x0020,   /* U+0496 */
  0x0020,   /* U+0497 */
  0x0020,   /* U+0498 */
  0x0020,   /* U+0499 */
  0x0020,   /* U+049A */
  0x0020,   /* U+049B */
  0x0020,   /* U+049C */
  0x0020,   /* U+049D */
  0x0020,   /* U+049E */
  0x0020,   /* U+049F */
  0x0020,   /* U+04A0 */
  0x0020,   /* U+04A1 */
  0x0020,   /* U+04A2 */
  0x0020,   /* U+04A3 */
  0x0020,   /* U+04A4 */
  0x0020,   /* U+04A5 */
  0x0020,   /* U+04A6 */
  0x0020,   /* U+04A7 */
  0x0020,   /* U+04A8 */
  0x0020,   /* U+04A9 */
  0x0020,   /* U+04AA */
  0x0020,   /* U+04AB */
  0x0020,   /* U+04AC */
  0x0020,   /* U+04AD */
  0x0020,   /* U+04AE */
  0x0020,   /* U+04AF */
  0x0020,   /* U+04B0 */
  0x0020,   /* U+04B1 */
  0x0020,   /* U+04B2 */
  0x0020,   /* U+04B3 */
  0x0020,   /* U+04B4 */
  0x0020,   /* U+04B5 */
  0x0020,   /* U+04B6 */
  0x0020,   /* U+04B7 */
  0x0020,   /* U+04B8 */
  0x0020,   /* U+04B9 */
  0x0020,   /* U+04BA */
  0x0020,   /* U+04BB */
  0x0020,   /* U+04BC */
  0x0020,   /* U+04BD */
  0x0020,   /* U+04BE */
  0x0020,   /* U+04BF */
  0x0020,   /* U+04C0 */
  0x0020,   /* U+04C1 */
  0x0020,   /* U+04C2 */
  0x0020,   /* U+04C3 */
  0x0020,   /* U+04C4 */
  0x0020,   /* U+04C5 */
  0x0020,   /* U+04C6 */
  0x0020,   /* U+04C7 */
  0x0020,   /* U+04C8 */
  0x0020,   /* U+04C9 */
  0x0020,   /* U+04CA */
  0x0020,   /* U+04CB */
  0x0020,   /* U+04CC */
  0x0020,   /* U+04CD */
  0x0020,   /* U+04CE */
  0x0020,   /* U+04CF */
  0x0020,   /* U+04D0 */
  0x0020,   /* U+04D1 */
  0x0020,   /* U+04D2 */
  0x0020,   /* U+04D3 */
  0x0020,   /* U+04D4 */
  0x0020,   /* U+04D5 */
  0x0020,   /* U+04D6 */
  0x0020,   /* U+04D7 */
  0x0020,   /* U+04D8 */
  0x0020,   /* U+04D9 */
  0x0020,   /* U+04DA */
  0x0020,   /* U+04DB */
  0x0020,   /* U+04DC */
  0x0020,   /* U+04DD */
  0x0020,   /* U+04DE */
  0x0020,   /* U+04DF */
  0x0020,   /* U+04E0 */
  0x0020,   /* U+04E1 */
  0x0020,   /* U+04E2 */
  0x0020,   /* U+04E3 */
  0x0020,   /* U+04E4 */
  0x0020,   /* U+04E5 */
  0x0020,   /* U+04E6 */
  0x0020,   /* U+04E7 */
  0x0020,   /* U+04E8 */
  0x0020,   /* U+04E9 */
  0x0020,   /* U+04EA */
  0x0020,   /* U+04EB */
  0x0020,   /* U+04EC */
  0x0020,   /* U+04ED */
  0x0020,   /* U+04EE */
  0x0020,   /* U+04EF */
  0x0020,   /* U+04F0 */
  0x0020,   /* U+04F1 */
  0x0020,   /* U+04F2 */
  0x0020,   /* U+04F3 */
  0x0020,   /* U+04F4 */
  0x0020,   /* U+04F5 */
  0x0020,   /* U+04F6 */
  0x0020,   /* U+04F7 */
  0x0020,   /* U+04F8 */
  0x0020,   /* U+04F9 */
  0x0020,   /* U+04FA */
  0x0020,   /* U+04FB */
  0x0020,   /* U+04FC */
  0x0020,   /* U+04FD */
  0x0020,   /* U+04FE */
  0x0020,   /* U+04FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+0400 */
  0x0008,   /* U+0401 */
  0x0008,   /* U+0402 */
  0x0008,   /* U+0403 */
  0x0008,   /* U+0404 */
  0x0008,   /* U+0405 */
  0x0008,   /* U+0406 */
  0x0008,   /* U+0407 */
  0x0008,   /* U+0408 */
  0x0008,   /* U+0409 */
  0x0008,   /* U+040A */
  0x0008,   /* U+040B */
  0x0008,   /* U+040C */
  0x0008,   /* U+040D */
  0x0008,   /* U+040E */
  0x0008,   /* U+040F */
  0x0008,   /* U+0410 */
  0x0008,   /* U+0411 */
  0x0008,   /* U+0412 */
  0x0008,   /* U+0413 */
  0x0008,   /* U+0414 */
  0x0008,   /* U+0415 */
  0x0008,   /* U+0416 */
  0x0008,   /* U+0417 */
  0x0008,   /* U+0418 */
  0x0008,   /* U+0419 */
  0x0008,   /* U+041A */
  0x0008,   /* U+041B */
  0x0008,   /* U+041C */
  0x0008,   /* U+041D */
  0x0008,   /* U+041E */
  0x0008,   /* U+041F */
  0x0008,   /* U+0420 */
  0x0008,   /* U+0421 */
  0x0008,   /* U+0422 */
  0x0008,   /* U+0423 */
  0x0008,   /* U+0424 */
  0x0008,   /* U+0425 */
  0x0008,   /* U+0426 */
  0x0008,   /* U+0427 */
  0x0008,   /* U+0428 */
  0x0008,   /* U+0429 */
  0x0008,   /* U+042A */
  0x0008,   /* U+042B */
  0x0008,   /* U+042C */
  0x0008,   /* U+042D */
  0x0008,   /* U+042E */
  0x0008,   /* U+042F */
  0x0002,   /* U+0430 */
  0x0002,   /* U+0431 */
  0x0002,   /* U+0432 */
  0x0002,   /* U+0433 */
  0x0002,   /* U+0434 */
  0x0002,   /* U+0435 */
  0x0002,   /* U+0436 */
  0x0002,   /* U+0437 */
  0x0002,   /* U+0438 */
  0x0002,   /* U+0439 */
  0x0002,   /* U+043A */
  0x0002,   /* U+043B */
  0x0002,   /* U+043C */
  0x0002,   /* U+043D */
  0x0002,   /* U+043E */
  0x0002,   /* U+043F */
  0x0002,   /* U+0440 */
  0x0002,   /* U+0441 */
  0x0002,   /* U+0442 */
  0x0002,   /* U+0443 */
  0x0002,   /* U+0444 */
  0x0002,   /* U+0445 */
  0x0002,   /* U+0446 */
  0x0002,   /* U+0447 */
  0x0002,   /* U+0448 */
  0x0002,   /* U+0449 */
  0x0002,   /* U+044A */
  0x0002,   /* U+044B */
  0x0002,   /* U+044C */
  0x0002,   /* U+044D */
  0x0002,   /* U+044E */
  0x0002,   /* U+044F */
  0x0002,   /* U+0450 */
  0x0002,   /* U+0451 */
  0x0002,   /* U+0452 */
  0x0002,   /* U+0453 */
  0x0002,   /* U+0454 */
  0x0002,   /* U+0455 */
  0x0002,   /* U+0456 */
  0x0002,   /* U+0457 */
  0x0002,   /* U+0458 */
  0x0002,   /* U+0459 */
  0x0002,   /* U+045A */
  0x0002,   /* U+045B */
  0x0002,   /* U+045C */
  0x0002,   /* U+045D */
  0x0002,   /* U+045E */
  0x0002,   /* U+045F */
  0x0008,   /* U+0460 */
  0x0002,   /* U+0461 */
  0x0008,   /* U+0462 */
  0x0002,   /* U+0463 */
  0x0008,   /* U+0464 */
  0x0002,   /* U+0465 */
  0x0008,   /* U+0466 */
  0x0002,   /* U+0467 */
  0x0008,   /* U+0468 */
  0x0002,   /* U+0469 */
  0x0008,   /* U+046A */
  0x0002,   /* U+046B */
  0x0008,   /* U+046C */
  0x0002,   /* U+046D */
  0x0008,   /* U+046E */
  0x0002,   /* U+046F */
  0x0008,   /* U+0470 */
  0x0002,   /* U+0471 */
  0x0008,   /* U+0472 */
  0x0002,   /* U+0473 */
  0x0008,   /* U+0474 */
  0x0002,   /* U+0475 */
  0x0008,   /* U+0476 */
  0x0002,   /* U+0477 */
  0x0008,   /* U+0478 */
  0x0002,   /* U+0479 */
  0x0008,   /* U+047A */
  0x0002,   /* U+047B */
  0x0008,   /* U+047C */
  0x0002,   /* U+047D */
  0x0008,   /* U+047E */
  0x0002,   /* U+047F */
  0x0008,   /* U+0480 */
  0x0002,   /* U+0481 */
  0x0002,   /* U+0482 */
  0x0002,   /* U+0483 */
  0x0002,   /* U+0484 */
  0x0002,   /* U+0485 */
  0x0002,   /* U+0486 */
  0x0002,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x0008,   /* U+048A */
  0x0002,   /* U+048B */
  0x0008,   /* U+048C */
  0x0002,   /* U+048D */
  0x0008,   /* U+048E */
  0x0002,   /* U+048F */
  0x000A,   /* U+0490 */
  0x0004,   /* U+0491 */
  0x0008,   /* U+0492 */
  0x0002,   /* U+0493 */
  0x0008,   /* U+0494 */
  0x0002,   /* U+0495 */
  0x0008,   /* U+0496 */
  0x0002,   /* U+0497 */
  0x0008,   /* U+0498 */
  0x0002,   /* U+0499 */
  0x0008,   /* U+049A */
  0x0002,   /* U+049B */
  0x0008,   /* U+049C */
  0x0002,   /* U+049D */
  0x0008,   /* U+049E */
  0x0002,   /* U+049F */
  0x0008,   /* U+04A0 */
  0x0002,   /* U+04A1 */
  0x0008,   /* U+04A2 */
  0x0002,   /* U+04A3 */
  0x0008,   /* U+04A4 */
  0x0002,   /* U+04A5 */
  0x0008,   /* U+04A6 */
  0x0002,   /* U+04A7 */
  0x0008,   /* U+04A8 */
  0x0002,   /* U+04A9 */
  0x0008,   /* U+04AA */
  0x0002,   /* U+04AB */
  0x0008,   /* U+04AC */
  0x0002,   /* U+04AD */
  0x0008,   /* U+04AE */
  0x0002,   /* U+04AF */
  0x0008,   /* U+04B0 */
  0x0002,   /* U+04B1 */
  0x0008,   /* U+04B2 */
  0x0002,   /* U+04B3 */
  0x0008,   /* U+04B4 */
  0x0002,   /* U+04B5 */
  0x0008,   /* U+04B6 */
  0x0002,   /* U+04B7 */
  0x0008,   /* U+04B8 */
  0x0002,   /* U+04B9 */
  0x0008,   /* U+04BA */
  0x0002,   /* U+04BB */
  0x0008,   /* U+04BC */
  0x0002,   /* U+04BD */
  0x0008,   /* U+04BE */
  0x0002,   /* U+04BF */
  0x0008,   /* U+04C0 */
  0x0008,   /* U+04C1 */
  0x0002,   /* U+04C2 */
  0x0008,   /* U+04C3 */
  0x0002,   /* U+04C4 */
  0x0008,   /* U+04C5 */
  0x0002,   /* U+04C6 */
  0x0008,   /* U+04C7 */
  0x0002,   /* U+04C8 */
  0x0008,   /* U+04C9 */
  0x0002,   /* U+04CA */
  0x0008,   /* U+04CB */
  0x0002,   /* U+04CC */
  0x0008,   /* U+04CD */
  0x0002,   /* U+04CE */
  0x0002,   /* U+04CF */
  0x0008,   /* U+04D0 */
  0x0002,   /* U+04D1 */
  0x0008,   /* U+04D2 */
  0x0002,   /* U+04D3 */
  0x0008,   /* U+04D4 */
  0x0002,   /* U+04D5 */
  0x0008,   /* U+04D6 */
  0x0002,   /* U+04D7 */
  0x0008,   /* U+04D8 */
  0x0002,   /* U+04D9 */
  0x0008,   /* U+04DA */
  0x0002,   /* U+04DB */
  0x0008,   /* U+04DC */
  0x0002,   /* U+04DD */
  0x0008,   /* U+04DE */
  0x0002,   /* U+04DF */
  0x0008,   /* U+04E0 */
  0x0002,   /* U+04E1 */
  0x0008,   /* U+04E2 */
  0x0002,   /* U+04E3 */
  0x0008,   /* U+04E4 */
  0x0002,   /* U+04E5 */
  0x0008,   /* U+04E6 */
  0x0002,   /* U+04E7 */
  0x0008,   /* U+04E8 */
  0x0002,   /* U+04E9 */
  0x0008,   /* U+04EA */
  0x0002,   /* U+04EB */
  0x0008,   /* U+04EC */
  0x0002,   /* U+04ED */
  0x0008,   /* U+04EE */
  0x0002,   /* U+04EF */
  0x0008,   /* U+04F0 */
  0x0002,   /* U+04F1 */
  0x0008,   /* U+04F2 */
  0x0002,   /* U+04F3 */
  0x0008,   /* U+04F4 */
  0x0002,   /* U+04F5 */
  0x0008,   /* U+04F6 */
  0x0002,   /* U+04F7 */
  0x0008,   /* U+04F8 */
  0x0002,   /* U+04F9 */
  0x0008,   /* U+04FA */
  0x0002,   /* U+04FB */
  0x0008,   /* U+04FC */
  0x0002,   /* U+04FD */
  0x0008,   /* U+04FE */
  0x0002,   /* U+04FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0400 */
  0x0000,   /* U+0401 */
  0x0000,   /* U+0402 */
  0x0000,   /* U+0403 */
  0x0000,   /* U+0404 */
  0x0000,   /* U+0405 */
  0x0000,   /* U+0406 */
  0x0000,   /* U+0407 */
  0x0000,   /* U+0408 */
  0x0000,   /* U+0409 */
  0x0000,   /* U+040A */
  0x0000,   /* U+040B */
  0x0000,   /* U+040C */
  0x0000,   /* U+040D */
  0x0000,   /* U+040E */
  0x0000,   /* U+040F */
  0x0000,   /* U+0410 */
  0x0000,   /* U+0411 */
  0x0000,   /* U+0412 */
  0x0000,   /* U+0413 */
  0x0000,   /* U+0414 */
  0x0000,   /* U+0415 */
  0x0000,   /* U+0416 */
  0x0000,   /* U+0417 */
  0x0000,   /* U+0418 */
  0x0000,   /* U+0419 */
  0x0000,   /* U+041A */
  0x0000,   /* U+041B */
  0x0000,   /* U+041C */
  0x0000,   /* U+041D */
  0x0000,   /* U+041E */
  0x0000,   /* U+041F */
  0x0000,   /* U+0420 */
  0x0000,   /* U+0421 */
  0x0000,   /* U+0422 */
  0x0000,   /* U+0423 */
  0x0000,   /* U+0424 */
  0x0000,   /* U+0425 */
  0x0000,   /* U+0426 */
  0x0000,   /* U+0427 */
  0x0000,   /* U+0428 */
  0x0000,   /* U+0429 */
  0x0000,   /* U+042A */
  0x0000,   /* U+042B */
  0x0000,   /* U+042C */
  0x0000,   /* U+042D */
  0x0000,   /* U+042E */
  0x0000,   /* U+042F */
  0x0000,   /* U+0430 */
  0x0000,   /* U+0431 */
  0x0000,   /* U+0432 */
  0x0000,   /* U+0433 */
  0x0000,   /* U+0434 */
  0x0000,   /* U+0435 */
  0x0000,   /* U+0436 */
  0x0000,   /* U+0437 */
  0x0000,   /* U+0438 */
  0x0000,   /* U+0439 */
  0x0000,   /* U+043A */
  0x0000,   /* U+043B */
  0x0000,   /* U+043C */
  0x0000,   /* U+043D */
  0x0000,   /* U+043E */
  0x0000,   /* U+043F */
  0x0000,   /* U+0440 */
  0x0000,   /* U+0441 */
  0x0000,   /* U+0442 */
  0x0000,   /* U+0443 */
  0x0000,   /* U+0444 */
  0x0000,   /* U+0445 */
  0x0000,   /* U+0446 */
  0x0000,   /* U+0447 */
  0x0000,   /* U+0448 */
  0x0000,   /* U+0449 */
  0x0000,   /* U+044A */
  0x0000,   /* U+044B */
  0x0000,   /* U+044C */
  0x0000,   /* U+044D */
  0x0000,   /* U+044E */
  0x0000,   /* U+044F */
  0x0000,   /* U+0450 */
  0x0000,   /* U+0451 */
  0x0000,   /* U+0452 */
  0x0000,   /* U+0453 */
  0x0000,   /* U+0454 */
  0x0000,   /* U+0455 */
  0x0000,   /* U+0456 */
  0x0000,   /* U+0457 */
  0x0000,   /* U+0458 */
  0x0000,   /* U+0459 */
  0x0000,   /* U+045A */
  0x0000,   /* U+045B */
  0x0000,   /* U+045C */
  0x0000,   /* U+045D */
  0x0000,   /* U+045E */
  0x0000,   /* U+045F */
  0x0000,   /* U+0460 */
  0x0000,   /* U+0461 */
  0x0000,   /* U+0462 */
  0x0000,   /* U+0463 */
  0x0000,   /* U+0464 */
  0x0000,   /* U+0465 */
  0x0000,   /* U+0466 */
  0x0000,   /* U+0467 */
  0x0000,   /* U+0468 */
  0x0000,   /* U+0469 */
  0x0000,   /* U+046A */
  0x0000,   /* U+046B */
  0x0000,   /* U+046C */
  0x0000,   /* U+046D */
  0x0000,   /* U+046E */
  0x0000,   /* U+046F */
  0x0000,   /* U+0470 */
  0x0000,   /* U+0471 */
  0x0000,   /* U+0472 */
  0x0000,   /* U+0473 */
  0x0000,   /* U+0474 */
  0x0000,   /* U+0475 */
  0x0000,   /* U+0476 */
  0x0000,   /* U+0477 */
  0x0000,   /* U+0478 */
  0x0000,   /* U+0479 */
  0x0000,   /* U+047A */
  0x0000,   /* U+047B */
  0x0000,   /* U+047C */
  0x0000,   /* U+047D */
  0x0000,   /* U+047E */
  0x0000,   /* U+047F */
  0x0000,   /* U+0480 */
  0x0000,   /* U+0481 */
  0x0000,   /* U+0482 */
  0x0000,   /* U+0483 */
  0x0000,   /* U+0484 */
  0x0000,   /* U+0485 */
  0x0000,   /* U+0486 */
  0x0000,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x0000,   /* U+048A */
  0x0000,   /* U+048B */
  0x0000,   /* U+048C */
  0x0000,   /* U+048D */
  0x0000,   /* U+048E */
  0x0000,   /* U+048F */
  0x0000,   /* U+0490 */
  0x0000,   /* U+0491 */
  0x0000,   /* U+0492 */
  0x0000,   /* U+0493 */
  0x0000,   /* U+0494 */
  0x0000,   /* U+0495 */
  0x0000,   /* U+0496 */
  0x0000,   /* U+0497 */
  0x0000,   /* U+0498 */
  0x0000,   /* U+0499 */
  0x0000,   /* U+049A */
  0x0000,   /* U+049B */
  0x0000,   /* U+049C */
  0x0000,   /* U+049D */
  0x0000,   /* U+049E */
  0x0000,   /* U+049F */
  0x0000,   /* U+04A0 */
  0x0000,   /* U+04A1 */
  0x0000,   /* U+04A2 */
  0x0000,   /* U+04A3 */
  0x0000,   /* U+04A4 */
  0x0000,   /* U+04A5 */
  0x0000,   /* U+04A6 */
  0x0000,   /* U+04A7 */
  0x0000,   /* U+04A8 */
  0x0000,   /* U+04A9 */
  0x0000,   /* U+04AA */
  0x0000,   /* U+04AB */
  0x0000,   /* U+04AC */
  0x0000,   /* U+04AD */
  0x0000,   /* U+04AE */
  0x0000,   /* U+04AF */
  0x0000,   /* U+04B0 */
  0x0000,   /* U+04B1 */
  0x0000,   /* U+04B2 */
  0x0000,   /* U+04B3 */
  0x0000,   /* U+04B4 */
  0x0000,   /* U+04B5 */
  0x0000,   /* U+04B6 */
  0x0000,   /* U+04B7 */
  0x0000,   /* U+04B8 */
  0x0000,   /* U+04B9 */
  0x0000,   /* U+04BA */
  0x0000,   /* U+04BB */
  0x0000,   /* U+04BC */
  0x0000,   /* U+04BD */
  0x0000,   /* U+04BE */
  0x0000,   /* U+04BF */
  0x0000,   /* U+04C0 */
  0x0000,   /* U+04C1 */
  0x0000,   /* U+04C2 */
  0x0000,   /* U+04C3 */
  0x0000,   /* U+04C4 */
  0x0000,   /* U+04C5 */
  0x0000,   /* U+04C6 */
  0x0000,   /* U+04C7 */
  0x0000,   /* U+04C8 */
  0x0000,   /* U+04C9 */
  0x0000,   /* U+04CA */
  0x0000,   /* U+04CB */
  0x0000,   /* U+04CC */
  0x0000,   /* U+04CD */
  0x0000,   /* U+04CE */
  0x0000,   /* U+04CF */
  0x0000,   /* U+04D0 */
  0x0000,   /* U+04D1 */
  0x0000,   /* U+04D2 */
  0x0000,   /* U+04D3 */
  0x0000,   /* U+04D4 */
  0x0000,   /* U+04D5 */
  0x0000,   /* U+04D6 */
  0x0000,   /* U+04D7 */
  0x0000,   /* U+04D8 */
  0x0000,   /* U+04D9 */
  0x0000,   /* U+04DA */
  0x0000,   /* U+04DB */
  0x0000,   /* U+04DC */
  0x0000,   /* U+04DD */
  0x0000,   /* U+04DE */
  0x0000,   /* U+04DF */
  0x0000,   /* U+04E0 */
  0x0000,   /* U+04E1 */
  0x0000,   /* U+04E2 */
  0x0000,   /* U+04E3 */
  0x0000,   /* U+04E4 */
  0x0000,   /* U+04E5 */
  0x0000,   /* U+04E6 */
  0x0000,   /* U+04E7 */
  0x0000,   /* U+04E8 */
  0x0000,   /* U+04E9 */
  0x0000,   /* U+04EA */
  0x0000,   /* U+04EB */
  0x0000,   /* U+04EC */
  0x0000,   /* U+04ED */
  0x0000,   /* U+04EE */
  0x0000,   /* U+04EF */
  0x0000,   /* U+04F0 */
  0x0000,   /* U+04F1 */
  0x0000,   /* U+04F2 */
  0x0000,   /* U+04F3 */
  0x0000,   /* U+04F4 */
  0x0000,   /* U+04F5 */
  0x0000,   /* U+04F6 */
  0x0000,   /* U+04F7 */
  0x0000,   /* U+04F8 */
  0x0000,   /* U+04F9 */
  0x0000,   /* U+04FA */
  0x0000,   /* U+04FB */
  0x0000,   /* U+04FC */
  0x0000,   /* U+04FD */
  0x0000,   /* U+04FE */
  0x0000,   /* U+04FF */

  /* Secondary weight 2 for each character. */
  0x0025,   /* U+0400 */
  0x002B,   /* U+0401 */
  0x0000,   /* U+0402 */
  0x0024,   /* U+0403 */
  0x0000,   /* U+0404 */
  0x0000,   /* U+0405 */
  0x0000,   /* U+0406 */
  0x002B,   /* U+0407 */
  0x0000,   /* U+0408 */
  0x0000,   /* U+0409 */
  0x0000,   /* U+040A */
  0x0000,   /* U+040B */
  0x0024,   /* U+040C */
  0x0025,   /* U+040D */
  0x0026,   /* U+040E */
  0x0000,   /* U+040F */
  0x0000,   /* U+0410 */
  0x0000,   /* U+0411 */
  0x0000,   /* U+0412 */
  0x0000,   /* U+0413 */
  0x0000,   /* U+0414 */
  0x0000,   /* U+0415 */
  0x0000,   /* U+0416 */
  0x0000,   /* U+0417 */
  0x0000,   /* U+0418 */
  0x0000,   /* U+0419 */
  0x0000,   /* U+041A */
  0x0000,   /* U+041B */
  0x0000,   /* U+041C */
  0x0000,   /* U+041D */
  0x0000,   /* U+041E */
  0x0000,   /* U+041F */
  0x0000,   /* U+0420 */
  0x0000,   /* U+0421 */
  0x0000,   /* U+0422 */
  0x0000,   /* U+0423 */
  0x0000,   /* U+0424 */
  0x0000,   /* U+0425 */
  0x0000,   /* U+0426 */
  0x0000,   /* U+0427 */
  0x0000,   /* U+0428 */
  0x0000,   /* U+0429 */
  0x0000,   /* U+042A */
  0x0000,   /* U+042B */
  0x0000,   /* U+042C */
  0x0000,   /* U+042D */
  0x0000,   /* U+042E */
  0x0000,   /* U+042F */
  0x0000,   /* U+0430 */
  0x0000,   /* U+0431 */
  0x0000,   /* U+0432 */
  0x0000,   /* U+0433 */
  0x0000,   /* U+0434 */
  0x0000,   /* U+0435 */
  0x0000,   /* U+0436 */
  0x0000,   /* U+0437 */
  0x0000,   /* U+0438 */
  0x0000,   /* U+0439 */
  0x0000,   /* U+043A */
  0x0000,   /* U+043B */
  0x0000,   /* U+043C */
  0x0000,   /* U+043D */
  0x0000,   /* U+043E */
  0x0000,   /* U+043F */
  0x0000,   /* U+0440 */
  0x0000,   /* U+0441 */
  0x0000,   /* U+0442 */
  0x0000,   /* U+0443 */
  0x0000,   /* U+0444 */
  0x0000,   /* U+0445 */
  0x0000,   /* U+0446 */
  0x0000,   /* U+0447 */
  0x0000,   /* U+0448 */
  0x0000,   /* U+0449 */
  0x0000,   /* U+044A */
  0x0000,   /* U+044B */
  0x0000,   /* U+044C */
  0x0000,   /* U+044D */
  0x0000,   /* U+044E */
  0x0000,   /* U+044F */
  0x0025,   /* U+0450 */
  0x002B,   /* U+0451 */
  0x0000,   /* U+0452 */
  0x0024,   /* U+0453 */
  0x0000,   /* U+0454 */
  0x0000,   /* U+0455 */
  0x0000,   /* U+0456 */
  0x002B,   /* U+0457 */
  0x0000,   /* U+0458 */
  0x0000,   /* U+0459 */
  0x0000,   /* U+045A */
  0x0000,   /* U+045B */
  0x0024,   /* U+045C */
  0x0025,   /* U+045D */
  0x0026,   /* U+045E */
  0x0000,   /* U+045F */
  0x0000,   /* U+0460 */
  0x0000,   /* U+0461 */
  0x0000,   /* U+0462 */
  0x0000,   /* U+0463 */
  0x0000,   /* U+0464 */
  0x0000,   /* U+0465 */
  0x0000,   /* U+0466 */
  0x0000,   /* U+0467 */
  0x0000,   /* U+0468 */
  0x0000,   /* U+0469 */
  0x0000,   /* U+046A */
  0x0000,   /* U+046B */
  0x0000,   /* U+046C */
  0x0000,   /* U+046D */
  0x0000,   /* U+046E */
  0x0000,   /* U+046F */
  0x0000,   /* U+0470 */
  0x0000,   /* U+0471 */
  0x0000,   /* U+0472 */
  0x0000,   /* U+0473 */
  0x0000,   /* U+0474 */
  0x0000,   /* U+0475 */
  0x003C,   /* U+0476 */
  0x003C,   /* U+0477 */
  0x0000,   /* U+0478 */
  0x0000,   /* U+0479 */
  0x0000,   /* U+047A */
  0x0000,   /* U+047B */
  0x0000,   /* U+047C */
  0x0000,   /* U+047D */
  0x0000,   /* U+047E */
  0x0000,   /* U+047F */
  0x0000,   /* U+0480 */
  0x0000,   /* U+0481 */
  0x0000,   /* U+0482 */
  0x0000,   /* U+0483 */
  0x0000,   /* U+0484 */
  0x0000,   /* U+0485 */
  0x0000,   /* U+0486 */
  0x0000,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x0000,   /* U+048A */
  0x0000,   /* U+048B */
  0x0000,   /* U+048C */
  0x0000,   /* U+048D */
  0x0000,   /* U+048E */
  0x0000,   /* U+048F */
  0x0111,   /* U+0490 */
  0x0111,   /* U+0491 */
  0x0000,   /* U+0492 */
  0x0000,   /* U+0493 */
  0x0000,   /* U+0494 */
  0x0000,   /* U+0495 */
  0x0000,   /* U+0496 */
  0x0000,   /* U+0497 */
  0x0000,   /* U+0498 */
  0x0000,   /* U+0499 */
  0x0000,   /* U+049A */
  0x0000,   /* U+049B */
  0x0000,   /* U+049C */
  0x0000,   /* U+049D */
  0x0000,   /* U+049E */
  0x0000,   /* U+049F */
  0x0000,   /* U+04A0 */
  0x0000,   /* U+04A1 */
  0x0000,   /* U+04A2 */
  0x0000,   /* U+04A3 */
  0x0000,   /* U+04A4 */
  0x0000,   /* U+04A5 */
  0x0000,   /* U+04A6 */
  0x0000,   /* U+04A7 */
  0x0000,   /* U+04A8 */
  0x0000,   /* U+04A9 */
  0x0000,   /* U+04AA */
  0x0000,   /* U+04AB */
  0x0000,   /* U+04AC */
  0x0000,   /* U+04AD */
  0x0000,   /* U+04AE */
  0x0000,   /* U+04AF */
  0x0000,   /* U+04B0 */
  0x0000,   /* U+04B1 */
  0x0000,   /* U+04B2 */
  0x0000,   /* U+04B3 */
  0x0000,   /* U+04B4 */
  0x0000,   /* U+04B5 */
  0x0000,   /* U+04B6 */
  0x0000,   /* U+04B7 */
  0x0000,   /* U+04B8 */
  0x0000,   /* U+04B9 */
  0x0000,   /* U+04BA */
  0x0000,   /* U+04BB */
  0x0000,   /* U+04BC */
  0x0000,   /* U+04BD */
  0x0000,   /* U+04BE */
  0x0000,   /* U+04BF */
  0x0000,   /* U+04C0 */
  0x0026,   /* U+04C1 */
  0x0026,   /* U+04C2 */
  0x0000,   /* U+04C3 */
  0x0000,   /* U+04C4 */
  0x0000,   /* U+04C5 */
  0x0000,   /* U+04C6 */
  0x0000,   /* U+04C7 */
  0x0000,   /* U+04C8 */
  0x0000,   /* U+04C9 */
  0x0000,   /* U+04CA */
  0x0000,   /* U+04CB */
  0x0000,   /* U+04CC */
  0x0000,   /* U+04CD */
  0x0000,   /* U+04CE */
  0x0000,   /* U+04CF */
  0x0026,   /* U+04D0 */
  0x0026,   /* U+04D1 */
  0x002B,   /* U+04D2 */
  0x002B,   /* U+04D3 */
  0x0000,   /* U+04D4 */
  0x0000,   /* U+04D5 */
  0x0026,   /* U+04D6 */
  0x0026,   /* U+04D7 */
  0x0000,   /* U+04D8 */
  0x0000,   /* U+04D9 */
  0x002B,   /* U+04DA */
  0x002B,   /* U+04DB */
  0x002B,   /* U+04DC */
  0x002B,   /* U+04DD */
  0x002B,   /* U+04DE */
  0x002B,   /* U+04DF */
  0x0000,   /* U+04E0 */
  0x0000,   /* U+04E1 */
  0x0032,   /* U+04E2 */
  0x0032,   /* U+04E3 */
  0x002B,   /* U+04E4 */
  0x002B,   /* U+04E5 */
  0x002B,   /* U+04E6 */
  0x002B,   /* U+04E7 */
  0x0000,   /* U+04E8 */
  0x0000,   /* U+04E9 */
  0x002B,   /* U+04EA */
  0x002B,   /* U+04EB */
  0x002B,   /* U+04EC */
  0x002B,   /* U+04ED */
  0x0032,   /* U+04EE */
  0x0032,   /* U+04EF */
  0x002B,   /* U+04F0 */
  0x002B,   /* U+04F1 */
  0x002C,   /* U+04F2 */
  0x002C,   /* U+04F3 */
  0x002B,   /* U+04F4 */
  0x002B,   /* U+04F5 */
  0x0000,   /* U+04F6 */
  0x0000,   /* U+04F7 */
  0x002B,   /* U+04F8 */
  0x002B,   /* U+04F9 */
  0x0000,   /* U+04FA */
  0x0000,   /* U+04FB */
  0x0000,   /* U+04FC */
  0x0000,   /* U+04FD */
  0x0000,   /* U+04FE */
  0x0000,   /* U+04FF */

  /* Tertiary weight 2 for each character. */
  0x0002,   /* U+0400 */
  0x0002,   /* U+0401 */
  0x0000,   /* U+0402 */
  0x0002,   /* U+0403 */
  0x0000,   /* U+0404 */
  0x0000,   /* U+0405 */
  0x0000,   /* U+0406 */
  0x0002,   /* U+0407 */
  0x0000,   /* U+0408 */
  0x0000,   /* U+0409 */
  0x0000,   /* U+040A */
  0x0000,   /* U+040B */
  0x0002,   /* U+040C */
  0x0002,   /* U+040D */
  0x0002,   /* U+040E */
  0x0000,   /* U+040F */
  0x0000,   /* U+0410 */
  0x0000,   /* U+0411 */
  0x0000,   /* U+0412 */
  0x0000,   /* U+0413 */
  0x0000,   /* U+0414 */
  0x0000,   /* U+0415 */
  0x0000,   /* U+0416 */
  0x0000,   /* U+0417 */
  0x0000,   /* U+0418 */
  0x0000,   /* U+0419 */
  0x0000,   /* U+041A */
  0x0000,   /* U+041B */
  0x0000,   /* U+041C */
  0x0000,   /* U+041D */
  0x0000,   /* U+041E */
  0x0000,   /* U+041F */
  0x0000,   /* U+0420 */
  0x0000,   /* U+0421 */
  0x0000,   /* U+0422 */
  0x0000,   /* U+0423 */
  0x0000,   /* U+0424 */
  0x0000,   /* U+0425 */
  0x0000,   /* U+0426 */
  0x0000,   /* U+0427 */
  0x0000,   /* U+0428 */
  0x0000,   /* U+0429 */
  0x0000,   /* U+042A */
  0x0000,   /* U+042B */
  0x0000,   /* U+042C */
  0x0000,   /* U+042D */
  0x0000,   /* U+042E */
  0x0000,   /* U+042F */
  0x0000,   /* U+0430 */
  0x0000,   /* U+0431 */
  0x0000,   /* U+0432 */
  0x0000,   /* U+0433 */
  0x0000,   /* U+0434 */
  0x0000,   /* U+0435 */
  0x0000,   /* U+0436 */
  0x0000,   /* U+0437 */
  0x0000,   /* U+0438 */
  0x0000,   /* U+0439 */
  0x0000,   /* U+043A */
  0x0000,   /* U+043B */
  0x0000,   /* U+043C */
  0x0000,   /* U+043D */
  0x0000,   /* U+043E */
  0x0000,   /* U+043F */
  0x0000,   /* U+0440 */
  0x0000,   /* U+0441 */
  0x0000,   /* U+0442 */
  0x0000,   /* U+0443 */
  0x0000,   /* U+0444 */
  0x0000,   /* U+0445 */
  0x0000,   /* U+0446 */
  0x0000,   /* U+0447 */
  0x0000,   /* U+0448 */
  0x0000,   /* U+0449 */
  0x0000,   /* U+044A */
  0x0000,   /* U+044B */
  0x0000,   /* U+044C */
  0x0000,   /* U+044D */
  0x0000,   /* U+044E */
  0x0000,   /* U+044F */
  0x0002,   /* U+0450 */
  0x0002,   /* U+0451 */
  0x0000,   /* U+0452 */
  0x0002,   /* U+0453 */
  0x0000,   /* U+0454 */
  0x0000,   /* U+0455 */
  0x0000,   /* U+0456 */
  0x0002,   /* U+0457 */
  0x0000,   /* U+0458 */
  0x0000,   /* U+0459 */
  0x0000,   /* U+045A */
  0x0000,   /* U+045B */
  0x0002,   /* U+045C */
  0x0002,   /* U+045D */
  0x0002,   /* U+045E */
  0x0000,   /* U+045F */
  0x0000,   /* U+0460 */
  0x0000,   /* U+0461 */
  0x0000,   /* U+0462 */
  0x0000,   /* U+0463 */
  0x0000,   /* U+0464 */
  0x0000,   /* U+0465 */
  0x0000,   /* U+0466 */
  0x0000,   /* U+0467 */
  0x0000,   /* U+0468 */
  0x0000,   /* U+0469 */
  0x0000,   /* U+046A */
  0x0000,   /* U+046B */
  0x0000,   /* U+046C */
  0x0000,   /* U+046D */
  0x0000,   /* U+046E */
  0x0000,   /* U+046F */
  0x0000,   /* U+0470 */
  0x0000,   /* U+0471 */
  0x0000,   /* U+0472 */
  0x0000,   /* U+0473 */
  0x0000,   /* U+0474 */
  0x0000,   /* U+0475 */
  0x0002,   /* U+0476 */
  0x0002,   /* U+0477 */
  0x0000,   /* U+0478 */
  0x0000,   /* U+0479 */
  0x0000,   /* U+047A */
  0x0000,   /* U+047B */
  0x0000,   /* U+047C */
  0x0000,   /* U+047D */
  0x0000,   /* U+047E */
  0x0000,   /* U+047F */
  0x0000,   /* U+0480 */
  0x0000,   /* U+0481 */
  0x0000,   /* U+0482 */
  0x0000,   /* U+0483 */
  0x0000,   /* U+0484 */
  0x0000,   /* U+0485 */
  0x0000,   /* U+0486 */
  0x0000,   /* U+0487 */
  0x0000,   /* U+0488 */
  0x0000,   /* U+0489 */
  0x0000,   /* U+048A */
  0x0000,   /* U+048B */
  0x0000,   /* U+048C */
  0x0000,   /* U+048D */
  0x0000,   /* U+048E */
  0x0000,   /* U+048F */
  0x0004,   /* U+0490 */
  0x0004,   /* U+0491 */
  0x0000,   /* U+0492 */
  0x0000,   /* U+0493 */
  0x0000,   /* U+0494 */
  0x0000,   /* U+0495 */
  0x0000,   /* U+0496 */
  0x0000,   /* U+0497 */
  0x0000,   /* U+0498 */
  0x0000,   /* U+0499 */
  0x0000,   /* U+049A */
  0x0000,   /* U+049B */
  0x0000,   /* U+049C */
  0x0000,   /* U+049D */
  0x0000,   /* U+049E */
  0x0000,   /* U+049F */
  0x0000,   /* U+04A0 */
  0x0000,   /* U+04A1 */
  0x0000,   /* U+04A2 */
  0x0000,   /* U+04A3 */
  0x0000,   /* U+04A4 */
  0x0000,   /* U+04A5 */
  0x0000,   /* U+04A6 */
  0x0000,   /* U+04A7 */
  0x0000,   /* U+04A8 */
  0x0000,   /* U+04A9 */
  0x0000,   /* U+04AA */
  0x0000,   /* U+04AB */
  0x0000,   /* U+04AC */
  0x0000,   /* U+04AD */
  0x0000,   /* U+04AE */
  0x0000,   /* U+04AF */
  0x0000,   /* U+04B0 */
  0x0000,   /* U+04B1 */
  0x0000,   /* U+04B2 */
  0x0000,   /* U+04B3 */
  0x0000,   /* U+04B4 */
  0x0000,   /* U+04B5 */
  0x0000,   /* U+04B6 */
  0x0000,   /* U+04B7 */
  0x0000,   /* U+04B8 */
  0x0000,   /* U+04B9 */
  0x0000,   /* U+04BA */
  0x0000,   /* U+04BB */
  0x0000,   /* U+04BC */
  0x0000,   /* U+04BD */
  0x0000,   /* U+04BE */
  0x0000,   /* U+04BF */
  0x0000,   /* U+04C0 */
  0x0002,   /* U+04C1 */
  0x0002,   /* U+04C2 */
  0x0000,   /* U+04C3 */
  0x0000,   /* U+04C4 */
  0x0000,   /* U+04C5 */
  0x0000,   /* U+04C6 */
  0x0000,   /* U+04C7 */
  0x0000,   /* U+04C8 */
  0x0000,   /* U+04C9 */
  0x0000,   /* U+04CA */
  0x0000,   /* U+04CB */
  0x0000,   /* U+04CC */
  0x0000,   /* U+04CD */
  0x0000,   /* U+04CE */
  0x0000,   /* U+04CF */
  0x0002,   /* U+04D0 */
  0x0002,   /* U+04D1 */
  0x0002,   /* U+04D2 */
  0x0002,   /* U+04D3 */
  0x0000,   /* U+04D4 */
  0x0000,   /* U+04D5 */
  0x0002,   /* U+04D6 */
  0x0002,   /* U+04D7 */
  0x0000,   /* U+04D8 */
  0x0000,   /* U+04D9 */
  0x0002,   /* U+04DA */
  0x0002,   /* U+04DB */
  0x0002,   /* U+04DC */
  0x0002,   /* U+04DD */
  0x0002,   /* U+04DE */
  0x0002,   /* U+04DF */
  0x0000,   /* U+04E0 */
  0x0000,   /* U+04E1 */
  0x0002,   /* U+04E2 */
  0x0002,   /* U+04E3 */
  0x0002,   /* U+04E4 */
  0x0002,   /* U+04E5 */
  0x0002,   /* U+04E6 */
  0x0002,   /* U+04E7 */
  0x0000,   /* U+04E8 */
  0x0000,   /* U+04E9 */
  0x0002,   /* U+04EA */
  0x0002,   /* U+04EB */
  0x0002,   /* U+04EC */
  0x0002,   /* U+04ED */
  0x0002,   /* U+04EE */
  0x0002,   /* U+04EF */
  0x0002,   /* U+04F0 */
  0x0002,   /* U+04F1 */
  0x0002,   /* U+04F2 */
  0x0002,   /* U+04F3 */
  0x0002,   /* U+04F4 */
  0x0002,   /* U+04F5 */
  0x0000,   /* U+04F6 */
  0x0000,   /* U+04F7 */
  0x0002,   /* U+04F8 */
  0x0002,   /* U+04F9 */
  0x0000,   /* U+04FA */
  0x0000,   /* U+04FB */
  0x0000,   /* U+04FC */
  0x0000,   /* U+04FD */
  0x0000,   /* U+04FE */
  0x0000,   /* U+04FF */
};

uint16 uca900_p005[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x204E,   /* U+0500 */
  0x204E,   /* U+0501 */
  0x2055,   /* U+0502 */
  0x2055,   /* U+0503 */
  0x2071,   /* U+0504 */
  0x2071,   /* U+0505 */
  0x207E,   /* U+0506 */
  0x207E,   /* U+0507 */
  0x20C1,   /* U+0508 */
  0x20C1,   /* U+0509 */
  0x20E6,   /* U+050A */
  0x20E6,   /* U+050B */
  0x2109,   /* U+050C */
  0x2109,   /* U+050D */
  0x2113,   /* U+050E */
  0x2113,   /* U+050F */
  0x2072,   /* U+0510 */
  0x2072,   /* U+0511 */
  0x20BA,   /* U+0512 */
  0x20BA,   /* U+0513 */
  0x20C2,   /* U+0514 */
  0x20C2,   /* U+0515 */
  0x2104,   /* U+0516 */
  0x2104,   /* U+0517 */
  0x21B3,   /* U+0518 */
  0x21B3,   /* U+0519 */
  0x20AF,   /* U+051A */
  0x20AF,   /* U+051B */
  0x21E0,   /* U+051C */
  0x21E0,   /* U+051D */
  0x20AE,   /* U+051E */
  0x20AE,   /* U+051F */
  0x20BB,   /* U+0520 */
  0x20BB,   /* U+0521 */
  0x20DD,   /* U+0522 */
  0x20DD,   /* U+0523 */
  0x20F3,   /* U+0524 */
  0x20F3,   /* U+0525 */
  0x2146,   /* U+0526 */
  0x2146,   /* U+0527 */
  0x20D0,   /* U+0528 */
  0x20D0,   /* U+0529 */
  0x2066,   /* U+052A */
  0x2066,   /* U+052B */
  0x2168,   /* U+052C */
  0x2168,   /* U+052D */
  0x20B9,   /* U+052E */
  0x20B9,   /* U+052F */
  0xFBC0,   /* U+0530 */
  0x2290,   /* U+0531 */
  0x2291,   /* U+0532 */
  0x2292,   /* U+0533 */
  0x2293,   /* U+0534 */
  0x2294,   /* U+0535 */
  0x2295,   /* U+0536 */
  0x2296,   /* U+0537 */
  0x2297,   /* U+0538 */
  0x2298,   /* U+0539 */
  0x2299,   /* U+053A */
  0x229A,   /* U+053B */
  0x229B,   /* U+053C */
  0x229C,   /* U+053D */
  0x229D,   /* U+053E */
  0x229E,   /* U+053F */
  0x229F,   /* U+0540 */
  0x22A0,   /* U+0541 */
  0x22A1,   /* U+0542 */
  0x22A2,   /* U+0543 */
  0x22A3,   /* U+0544 */
  0x22A4,   /* U+0545 */
  0x22A5,   /* U+0546 */
  0x22A6,   /* U+0547 */
  0x22A7,   /* U+0548 */
  0x22A8,   /* U+0549 */
  0x22A9,   /* U+054A */
  0x22AA,   /* U+054B */
  0x22AB,   /* U+054C */
  0x22AC,   /* U+054D */
  0x22AD,   /* U+054E */
  0x22AE,   /* U+054F */
  0x22AF,   /* U+0550 */
  0x22B0,   /* U+0551 */
  0x22B1,   /* U+0552 */
  0x22B2,   /* U+0553 */
  0x22B3,   /* U+0554 */
  0x22B4,   /* U+0555 */
  0x22B5,   /* U+0556 */
  0xFBC0,   /* U+0557 */
  0xFBC0,   /* U+0558 */
  0x22B6,   /* U+0559 */
  0x03CB,   /* U+055A */
  0x03CC,   /* U+055B */
  0x0262,   /* U+055C */
  0x0226,   /* U+055D */
  0x0269,   /* U+055E */
  0x03CD,   /* U+055F */
  0xFBC0,   /* U+0560 */
  0x2290,   /* U+0561 */
  0x2291,   /* U+0562 */
  0x2292,   /* U+0563 */
  0x2293,   /* U+0564 */
  0x2294,   /* U+0565 */
  0x2295,   /* U+0566 */
  0x2296,   /* U+0567 */
  0x2297,   /* U+0568 */
  0x2298,   /* U+0569 */
  0x2299,   /* U+056A */
  0x229A,   /* U+056B */
  0x229B,   /* U+056C */
  0x229C,   /* U+056D */
  0x229D,   /* U+056E */
  0x229E,   /* U+056F */
  0x229F,   /* U+0570 */
  0x22A0,   /* U+0571 */
  0x22A1,   /* U+0572 */
  0x22A2,   /* U+0573 */
  0x22A3,   /* U+0574 */
  0x22A4,   /* U+0575 */
  0x22A5,   /* U+0576 */
  0x22A6,   /* U+0577 */
  0x22A7,   /* U+0578 */
  0x22A8,   /* U+0579 */
  0x22A9,   /* U+057A */
  0x22AA,   /* U+057B */
  0x22AB,   /* U+057C */
  0x22AC,   /* U+057D */
  0x22AD,   /* U+057E */
  0x22AE,   /* U+057F */
  0x22AF,   /* U+0580 */
  0x22B0,   /* U+0581 */
  0x22B1,   /* U+0582 */
  0x22B2,   /* U+0583 */
  0x22B3,   /* U+0584 */
  0x22B4,   /* U+0585 */
  0x22B5,   /* U+0586 */
  0x2294,   /* U+0587 */
  0xFBC0,   /* U+0588 */
  0x023A,   /* U+0589 */
  0x020E,   /* U+058A */
  0xFBC0,   /* U+058B */
  0xFBC0,   /* U+058C */
  0x04F8,   /* U+058D */
  0x04F9,   /* U+058E */
  0x1C15,   /* U+058F */
  0xFBC0,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0000,   /* U+05B0 */
  0x0000,   /* U+05B1 */
  0x0000,   /* U+05B2 */
  0x0000,   /* U+05B3 */
  0x0000,   /* U+05B4 */
  0x0000,   /* U+05B5 */
  0x0000,   /* U+05B6 */
  0x0000,   /* U+05B7 */
  0x0000,   /* U+05B8 */
  0x0000,   /* U+05B9 */
  0x0000,   /* U+05BA */
  0x0000,   /* U+05BB */
  0x0000,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x03CE,   /* U+05BE */
  0x0000,   /* U+05BF */
  0x03CF,   /* U+05C0 */
  0x0000,   /* U+05C1 */
  0x0000,   /* U+05C2 */
  0x03D0,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x03D1,   /* U+05C6 */
  0x0000,   /* U+05C7 */
  0xFBC0,   /* U+05C8 */
  0xFBC0,   /* U+05C9 */
  0xFBC0,   /* U+05CA */
  0xFBC0,   /* U+05CB */
  0xFBC0,   /* U+05CC */
  0xFBC0,   /* U+05CD */
  0xFBC0,   /* U+05CE */
  0xFBC0,   /* U+05CF */
  0x22B7,   /* U+05D0 */
  0x22B8,   /* U+05D1 */
  0x22B9,   /* U+05D2 */
  0x22BA,   /* U+05D3 */
  0x22BB,   /* U+05D4 */
  0x22BC,   /* U+05D5 */
  0x22BD,   /* U+05D6 */
  0x22BE,   /* U+05D7 */
  0x22BF,   /* U+05D8 */
  0x22C0,   /* U+05D9 */
  0x22C1,   /* U+05DA */
  0x22C1,   /* U+05DB */
  0x22C2,   /* U+05DC */
  0x22C3,   /* U+05DD */
  0x22C3,   /* U+05DE */
  0x22C4,   /* U+05DF */
  0x22C4,   /* U+05E0 */
  0x22C5,   /* U+05E1 */
  0x22C6,   /* U+05E2 */
  0x22C7,   /* U+05E3 */
  0x22C7,   /* U+05E4 */
  0x22C8,   /* U+05E5 */
  0x22C8,   /* U+05E6 */
  0x22C9,   /* U+05E7 */
  0x22CA,   /* U+05E8 */
  0x22CB,   /* U+05E9 */
  0x22CC,   /* U+05EA */
  0xFBC0,   /* U+05EB */
  0xFBC0,   /* U+05EC */
  0xFBC0,   /* U+05ED */
  0xFBC0,   /* U+05EE */
  0xFBC0,   /* U+05EF */
  0x22BC,   /* U+05F0 */
  0x22BC,   /* U+05F1 */
  0x22C0,   /* U+05F2 */
  0x03D2,   /* U+05F3 */
  0x03D3,   /* U+05F4 */
  0xFBC0,   /* U+05F5 */
  0xFBC0,   /* U+05F6 */
  0xFBC0,   /* U+05F7 */
  0xFBC0,   /* U+05F8 */
  0xFBC0,   /* U+05F9 */
  0xFBC0,   /* U+05FA */
  0xFBC0,   /* U+05FB */
  0xFBC0,   /* U+05FC */
  0xFBC0,   /* U+05FD */
  0xFBC0,   /* U+05FE */
  0xFBC0,   /* U+05FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0500 */
  0x0020,   /* U+0501 */
  0x0020,   /* U+0502 */
  0x0020,   /* U+0503 */
  0x0020,   /* U+0504 */
  0x0020,   /* U+0505 */
  0x0020,   /* U+0506 */
  0x0020,   /* U+0507 */
  0x0020,   /* U+0508 */
  0x0020,   /* U+0509 */
  0x0020,   /* U+050A */
  0x0020,   /* U+050B */
  0x0020,   /* U+050C */
  0x0020,   /* U+050D */
  0x0020,   /* U+050E */
  0x0020,   /* U+050F */
  0x0020,   /* U+0510 */
  0x0020,   /* U+0511 */
  0x0020,   /* U+0512 */
  0x0020,   /* U+0513 */
  0x0020,   /* U+0514 */
  0x0020,   /* U+0515 */
  0x0020,   /* U+0516 */
  0x0020,   /* U+0517 */
  0x0020,   /* U+0518 */
  0x0020,   /* U+0519 */
  0x0020,   /* U+051A */
  0x0020,   /* U+051B */
  0x0020,   /* U+051C */
  0x0020,   /* U+051D */
  0x0020,   /* U+051E */
  0x0020,   /* U+051F */
  0x0020,   /* U+0520 */
  0x0020,   /* U+0521 */
  0x0020,   /* U+0522 */
  0x0020,   /* U+0523 */
  0x0020,   /* U+0524 */
  0x0020,   /* U+0525 */
  0x0020,   /* U+0526 */
  0x0020,   /* U+0527 */
  0x0020,   /* U+0528 */
  0x0020,   /* U+0529 */
  0x0020,   /* U+052A */
  0x0020,   /* U+052B */
  0x0020,   /* U+052C */
  0x0020,   /* U+052D */
  0x0020,   /* U+052E */
  0x0020,   /* U+052F */
  0x0020,   /* U+0530 */
  0x0020,   /* U+0531 */
  0x0020,   /* U+0532 */
  0x0020,   /* U+0533 */
  0x0020,   /* U+0534 */
  0x0020,   /* U+0535 */
  0x0020,   /* U+0536 */
  0x0020,   /* U+0537 */
  0x0020,   /* U+0538 */
  0x0020,   /* U+0539 */
  0x0020,   /* U+053A */
  0x0020,   /* U+053B */
  0x0020,   /* U+053C */
  0x0020,   /* U+053D */
  0x0020,   /* U+053E */
  0x0020,   /* U+053F */
  0x0020,   /* U+0540 */
  0x0020,   /* U+0541 */
  0x0020,   /* U+0542 */
  0x0020,   /* U+0543 */
  0x0020,   /* U+0544 */
  0x0020,   /* U+0545 */
  0x0020,   /* U+0546 */
  0x0020,   /* U+0547 */
  0x0020,   /* U+0548 */
  0x0020,   /* U+0549 */
  0x0020,   /* U+054A */
  0x0020,   /* U+054B */
  0x0020,   /* U+054C */
  0x0020,   /* U+054D */
  0x0020,   /* U+054E */
  0x0020,   /* U+054F */
  0x0020,   /* U+0550 */
  0x0020,   /* U+0551 */
  0x0020,   /* U+0552 */
  0x0020,   /* U+0553 */
  0x0020,   /* U+0554 */
  0x0020,   /* U+0555 */
  0x0020,   /* U+0556 */
  0x0020,   /* U+0557 */
  0x0020,   /* U+0558 */
  0x0020,   /* U+0559 */
  0x0020,   /* U+055A */
  0x0020,   /* U+055B */
  0x0020,   /* U+055C */
  0x0020,   /* U+055D */
  0x0020,   /* U+055E */
  0x0020,   /* U+055F */
  0x0020,   /* U+0560 */
  0x0020,   /* U+0561 */
  0x0020,   /* U+0562 */
  0x0020,   /* U+0563 */
  0x0020,   /* U+0564 */
  0x0020,   /* U+0565 */
  0x0020,   /* U+0566 */
  0x0020,   /* U+0567 */
  0x0020,   /* U+0568 */
  0x0020,   /* U+0569 */
  0x0020,   /* U+056A */
  0x0020,   /* U+056B */
  0x0020,   /* U+056C */
  0x0020,   /* U+056D */
  0x0020,   /* U+056E */
  0x0020,   /* U+056F */
  0x0020,   /* U+0570 */
  0x0020,   /* U+0571 */
  0x0020,   /* U+0572 */
  0x0020,   /* U+0573 */
  0x0020,   /* U+0574 */
  0x0020,   /* U+0575 */
  0x0020,   /* U+0576 */
  0x0020,   /* U+0577 */
  0x0020,   /* U+0578 */
  0x0020,   /* U+0579 */
  0x0020,   /* U+057A */
  0x0020,   /* U+057B */
  0x0020,   /* U+057C */
  0x0020,   /* U+057D */
  0x0020,   /* U+057E */
  0x0020,   /* U+057F */
  0x0020,   /* U+0580 */
  0x0020,   /* U+0581 */
  0x0020,   /* U+0582 */
  0x0020,   /* U+0583 */
  0x0020,   /* U+0584 */
  0x0020,   /* U+0585 */
  0x0020,   /* U+0586 */
  0x0020,   /* U+0587 */
  0x0020,   /* U+0588 */
  0x0020,   /* U+0589 */
  0x0020,   /* U+058A */
  0x0020,   /* U+058B */
  0x0020,   /* U+058C */
  0x0020,   /* U+058D */
  0x0020,   /* U+058E */
  0x0020,   /* U+058F */
  0x0020,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0052,   /* U+05B0 */
  0x0053,   /* U+05B1 */
  0x0054,   /* U+05B2 */
  0x0055,   /* U+05B3 */
  0x0056,   /* U+05B4 */
  0x0057,   /* U+05B5 */
  0x0058,   /* U+05B6 */
  0x0059,   /* U+05B7 */
  0x005A,   /* U+05B8 */
  0x005B,   /* U+05B9 */
  0x005B,   /* U+05BA */
  0x005C,   /* U+05BB */
  0x005F,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x0020,   /* U+05BE */
  0x0060,   /* U+05BF */
  0x0020,   /* U+05C0 */
  0x005E,   /* U+05C1 */
  0x005D,   /* U+05C2 */
  0x0020,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x0020,   /* U+05C6 */
  0x005A,   /* U+05C7 */
  0x0020,   /* U+05C8 */
  0x0020,   /* U+05C9 */
  0x0020,   /* U+05CA */
  0x0020,   /* U+05CB */
  0x0020,   /* U+05CC */
  0x0020,   /* U+05CD */
  0x0020,   /* U+05CE */
  0x0020,   /* U+05CF */
  0x0020,   /* U+05D0 */
  0x0020,   /* U+05D1 */
  0x0020,   /* U+05D2 */
  0x0020,   /* U+05D3 */
  0x0020,   /* U+05D4 */
  0x0020,   /* U+05D5 */
  0x0020,   /* U+05D6 */
  0x0020,   /* U+05D7 */
  0x0020,   /* U+05D8 */
  0x0020,   /* U+05D9 */
  0x0020,   /* U+05DA */
  0x0020,   /* U+05DB */
  0x0020,   /* U+05DC */
  0x0020,   /* U+05DD */
  0x0020,   /* U+05DE */
  0x0020,   /* U+05DF */
  0x0020,   /* U+05E0 */
  0x0020,   /* U+05E1 */
  0x0020,   /* U+05E2 */
  0x0020,   /* U+05E3 */
  0x0020,   /* U+05E4 */
  0x0020,   /* U+05E5 */
  0x0020,   /* U+05E6 */
  0x0020,   /* U+05E7 */
  0x0020,   /* U+05E8 */
  0x0020,   /* U+05E9 */
  0x0020,   /* U+05EA */
  0x0020,   /* U+05EB */
  0x0020,   /* U+05EC */
  0x0020,   /* U+05ED */
  0x0020,   /* U+05EE */
  0x0020,   /* U+05EF */
  0x0020,   /* U+05F0 */
  0x0020,   /* U+05F1 */
  0x0020,   /* U+05F2 */
  0x0020,   /* U+05F3 */
  0x0020,   /* U+05F4 */
  0x0020,   /* U+05F5 */
  0x0020,   /* U+05F6 */
  0x0020,   /* U+05F7 */
  0x0020,   /* U+05F8 */
  0x0020,   /* U+05F9 */
  0x0020,   /* U+05FA */
  0x0020,   /* U+05FB */
  0x0020,   /* U+05FC */
  0x0020,   /* U+05FD */
  0x0020,   /* U+05FE */
  0x0020,   /* U+05FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+0500 */
  0x0002,   /* U+0501 */
  0x0008,   /* U+0502 */
  0x0002,   /* U+0503 */
  0x0008,   /* U+0504 */
  0x0002,   /* U+0505 */
  0x0008,   /* U+0506 */
  0x0002,   /* U+0507 */
  0x0008,   /* U+0508 */
  0x0002,   /* U+0509 */
  0x0008,   /* U+050A */
  0x0002,   /* U+050B */
  0x0008,   /* U+050C */
  0x0002,   /* U+050D */
  0x0008,   /* U+050E */
  0x0002,   /* U+050F */
  0x0008,   /* U+0510 */
  0x0002,   /* U+0511 */
  0x0008,   /* U+0512 */
  0x0002,   /* U+0513 */
  0x0008,   /* U+0514 */
  0x0002,   /* U+0515 */
  0x0008,   /* U+0516 */
  0x0002,   /* U+0517 */
  0x0008,   /* U+0518 */
  0x0002,   /* U+0519 */
  0x0008,   /* U+051A */
  0x0002,   /* U+051B */
  0x0008,   /* U+051C */
  0x0002,   /* U+051D */
  0x0008,   /* U+051E */
  0x0002,   /* U+051F */
  0x0008,   /* U+0520 */
  0x0002,   /* U+0521 */
  0x0008,   /* U+0522 */
  0x0002,   /* U+0523 */
  0x0008,   /* U+0524 */
  0x0002,   /* U+0525 */
  0x0008,   /* U+0526 */
  0x0002,   /* U+0527 */
  0x0008,   /* U+0528 */
  0x0002,   /* U+0529 */
  0x0008,   /* U+052A */
  0x0002,   /* U+052B */
  0x0008,   /* U+052C */
  0x0002,   /* U+052D */
  0x0008,   /* U+052E */
  0x0002,   /* U+052F */
  0x0002,   /* U+0530 */
  0x0008,   /* U+0531 */
  0x0008,   /* U+0532 */
  0x0008,   /* U+0533 */
  0x0008,   /* U+0534 */
  0x0008,   /* U+0535 */
  0x0008,   /* U+0536 */
  0x0008,   /* U+0537 */
  0x0008,   /* U+0538 */
  0x0008,   /* U+0539 */
  0x0008,   /* U+053A */
  0x0008,   /* U+053B */
  0x0008,   /* U+053C */
  0x0008,   /* U+053D */
  0x0008,   /* U+053E */
  0x0008,   /* U+053F */
  0x0008,   /* U+0540 */
  0x0008,   /* U+0541 */
  0x0008,   /* U+0542 */
  0x0008,   /* U+0543 */
  0x0008,   /* U+0544 */
  0x0008,   /* U+0545 */
  0x0008,   /* U+0546 */
  0x0008,   /* U+0547 */
  0x0008,   /* U+0548 */
  0x0008,   /* U+0549 */
  0x0008,   /* U+054A */
  0x0008,   /* U+054B */
  0x0008,   /* U+054C */
  0x0008,   /* U+054D */
  0x0008,   /* U+054E */
  0x0008,   /* U+054F */
  0x0008,   /* U+0550 */
  0x0008,   /* U+0551 */
  0x0008,   /* U+0552 */
  0x0008,   /* U+0553 */
  0x0008,   /* U+0554 */
  0x0008,   /* U+0555 */
  0x0008,   /* U+0556 */
  0x0002,   /* U+0557 */
  0x0002,   /* U+0558 */
  0x0002,   /* U+0559 */
  0x0002,   /* U+055A */
  0x0002,   /* U+055B */
  0x0002,   /* U+055C */
  0x0002,   /* U+055D */
  0x0002,   /* U+055E */
  0x0002,   /* U+055F */
  0x0002,   /* U+0560 */
  0x0002,   /* U+0561 */
  0x0002,   /* U+0562 */
  0x0002,   /* U+0563 */
  0x0002,   /* U+0564 */
  0x0002,   /* U+0565 */
  0x0002,   /* U+0566 */
  0x0002,   /* U+0567 */
  0x0002,   /* U+0568 */
  0x0002,   /* U+0569 */
  0x0002,   /* U+056A */
  0x0002,   /* U+056B */
  0x0002,   /* U+056C */
  0x0002,   /* U+056D */
  0x0002,   /* U+056E */
  0x0002,   /* U+056F */
  0x0002,   /* U+0570 */
  0x0002,   /* U+0571 */
  0x0002,   /* U+0572 */
  0x0002,   /* U+0573 */
  0x0002,   /* U+0574 */
  0x0002,   /* U+0575 */
  0x0002,   /* U+0576 */
  0x0002,   /* U+0577 */
  0x0002,   /* U+0578 */
  0x0002,   /* U+0579 */
  0x0002,   /* U+057A */
  0x0002,   /* U+057B */
  0x0002,   /* U+057C */
  0x0002,   /* U+057D */
  0x0002,   /* U+057E */
  0x0002,   /* U+057F */
  0x0002,   /* U+0580 */
  0x0002,   /* U+0581 */
  0x0002,   /* U+0582 */
  0x0002,   /* U+0583 */
  0x0002,   /* U+0584 */
  0x0002,   /* U+0585 */
  0x0002,   /* U+0586 */
  0x0004,   /* U+0587 */
  0x0002,   /* U+0588 */
  0x0002,   /* U+0589 */
  0x0002,   /* U+058A */
  0x0002,   /* U+058B */
  0x0002,   /* U+058C */
  0x0002,   /* U+058D */
  0x0002,   /* U+058E */
  0x0002,   /* U+058F */
  0x0002,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0002,   /* U+05B0 */
  0x0002,   /* U+05B1 */
  0x0002,   /* U+05B2 */
  0x0002,   /* U+05B3 */
  0x0002,   /* U+05B4 */
  0x0002,   /* U+05B5 */
  0x0002,   /* U+05B6 */
  0x0002,   /* U+05B7 */
  0x0002,   /* U+05B8 */
  0x0002,   /* U+05B9 */
  0x0002,   /* U+05BA */
  0x0002,   /* U+05BB */
  0x0002,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x0002,   /* U+05BE */
  0x0002,   /* U+05BF */
  0x0002,   /* U+05C0 */
  0x0002,   /* U+05C1 */
  0x0002,   /* U+05C2 */
  0x0002,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x0002,   /* U+05C6 */
  0x0002,   /* U+05C7 */
  0x0002,   /* U+05C8 */
  0x0002,   /* U+05C9 */
  0x0002,   /* U+05CA */
  0x0002,   /* U+05CB */
  0x0002,   /* U+05CC */
  0x0002,   /* U+05CD */
  0x0002,   /* U+05CE */
  0x0002,   /* U+05CF */
  0x0002,   /* U+05D0 */
  0x0002,   /* U+05D1 */
  0x0002,   /* U+05D2 */
  0x0002,   /* U+05D3 */
  0x0002,   /* U+05D4 */
  0x0002,   /* U+05D5 */
  0x0002,   /* U+05D6 */
  0x0002,   /* U+05D7 */
  0x0002,   /* U+05D8 */
  0x0002,   /* U+05D9 */
  0x0019,   /* U+05DA */
  0x0002,   /* U+05DB */
  0x0002,   /* U+05DC */
  0x0019,   /* U+05DD */
  0x0002,   /* U+05DE */
  0x0019,   /* U+05DF */
  0x0002,   /* U+05E0 */
  0x0002,   /* U+05E1 */
  0x0002,   /* U+05E2 */
  0x0019,   /* U+05E3 */
  0x0002,   /* U+05E4 */
  0x0019,   /* U+05E5 */
  0x0002,   /* U+05E6 */
  0x0002,   /* U+05E7 */
  0x0002,   /* U+05E8 */
  0x0002,   /* U+05E9 */
  0x0002,   /* U+05EA */
  0x0002,   /* U+05EB */
  0x0002,   /* U+05EC */
  0x0002,   /* U+05ED */
  0x0002,   /* U+05EE */
  0x0002,   /* U+05EF */
  0x0004,   /* U+05F0 */
  0x0004,   /* U+05F1 */
  0x0004,   /* U+05F2 */
  0x0002,   /* U+05F3 */
  0x0002,   /* U+05F4 */
  0x0002,   /* U+05F5 */
  0x0002,   /* U+05F6 */
  0x0002,   /* U+05F7 */
  0x0002,   /* U+05F8 */
  0x0002,   /* U+05F9 */
  0x0002,   /* U+05FA */
  0x0002,   /* U+05FB */
  0x0002,   /* U+05FC */
  0x0002,   /* U+05FD */
  0x0002,   /* U+05FE */
  0x0002,   /* U+05FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0500 */
  0x0000,   /* U+0501 */
  0x0000,   /* U+0502 */
  0x0000,   /* U+0503 */
  0x0000,   /* U+0504 */
  0x0000,   /* U+0505 */
  0x0000,   /* U+0506 */
  0x0000,   /* U+0507 */
  0x0000,   /* U+0508 */
  0x0000,   /* U+0509 */
  0x0000,   /* U+050A */
  0x0000,   /* U+050B */
  0x0000,   /* U+050C */
  0x0000,   /* U+050D */
  0x0000,   /* U+050E */
  0x0000,   /* U+050F */
  0x0000,   /* U+0510 */
  0x0000,   /* U+0511 */
  0x0000,   /* U+0512 */
  0x0000,   /* U+0513 */
  0x0000,   /* U+0514 */
  0x0000,   /* U+0515 */
  0x0000,   /* U+0516 */
  0x0000,   /* U+0517 */
  0x0000,   /* U+0518 */
  0x0000,   /* U+0519 */
  0x0000,   /* U+051A */
  0x0000,   /* U+051B */
  0x0000,   /* U+051C */
  0x0000,   /* U+051D */
  0x0000,   /* U+051E */
  0x0000,   /* U+051F */
  0x0000,   /* U+0520 */
  0x0000,   /* U+0521 */
  0x0000,   /* U+0522 */
  0x0000,   /* U+0523 */
  0x0000,   /* U+0524 */
  0x0000,   /* U+0525 */
  0x0000,   /* U+0526 */
  0x0000,   /* U+0527 */
  0x0000,   /* U+0528 */
  0x0000,   /* U+0529 */
  0x0000,   /* U+052A */
  0x0000,   /* U+052B */
  0x0000,   /* U+052C */
  0x0000,   /* U+052D */
  0x0000,   /* U+052E */
  0x0000,   /* U+052F */
  0x8530,   /* U+0530 */
  0x0000,   /* U+0531 */
  0x0000,   /* U+0532 */
  0x0000,   /* U+0533 */
  0x0000,   /* U+0534 */
  0x0000,   /* U+0535 */
  0x0000,   /* U+0536 */
  0x0000,   /* U+0537 */
  0x0000,   /* U+0538 */
  0x0000,   /* U+0539 */
  0x0000,   /* U+053A */
  0x0000,   /* U+053B */
  0x0000,   /* U+053C */
  0x0000,   /* U+053D */
  0x0000,   /* U+053E */
  0x0000,   /* U+053F */
  0x0000,   /* U+0540 */
  0x0000,   /* U+0541 */
  0x0000,   /* U+0542 */
  0x0000,   /* U+0543 */
  0x0000,   /* U+0544 */
  0x0000,   /* U+0545 */
  0x0000,   /* U+0546 */
  0x0000,   /* U+0547 */
  0x0000,   /* U+0548 */
  0x0000,   /* U+0549 */
  0x0000,   /* U+054A */
  0x0000,   /* U+054B */
  0x0000,   /* U+054C */
  0x0000,   /* U+054D */
  0x0000,   /* U+054E */
  0x0000,   /* U+054F */
  0x0000,   /* U+0550 */
  0x0000,   /* U+0551 */
  0x0000,   /* U+0552 */
  0x0000,   /* U+0553 */
  0x0000,   /* U+0554 */
  0x0000,   /* U+0555 */
  0x0000,   /* U+0556 */
  0x8557,   /* U+0557 */
  0x8558,   /* U+0558 */
  0x0000,   /* U+0559 */
  0x0000,   /* U+055A */
  0x0000,   /* U+055B */
  0x0000,   /* U+055C */
  0x0000,   /* U+055D */
  0x0000,   /* U+055E */
  0x0000,   /* U+055F */
  0x8560,   /* U+0560 */
  0x0000,   /* U+0561 */
  0x0000,   /* U+0562 */
  0x0000,   /* U+0563 */
  0x0000,   /* U+0564 */
  0x0000,   /* U+0565 */
  0x0000,   /* U+0566 */
  0x0000,   /* U+0567 */
  0x0000,   /* U+0568 */
  0x0000,   /* U+0569 */
  0x0000,   /* U+056A */
  0x0000,   /* U+056B */
  0x0000,   /* U+056C */
  0x0000,   /* U+056D */
  0x0000,   /* U+056E */
  0x0000,   /* U+056F */
  0x0000,   /* U+0570 */
  0x0000,   /* U+0571 */
  0x0000,   /* U+0572 */
  0x0000,   /* U+0573 */
  0x0000,   /* U+0574 */
  0x0000,   /* U+0575 */
  0x0000,   /* U+0576 */
  0x0000,   /* U+0577 */
  0x0000,   /* U+0578 */
  0x0000,   /* U+0579 */
  0x0000,   /* U+057A */
  0x0000,   /* U+057B */
  0x0000,   /* U+057C */
  0x0000,   /* U+057D */
  0x0000,   /* U+057E */
  0x0000,   /* U+057F */
  0x0000,   /* U+0580 */
  0x0000,   /* U+0581 */
  0x0000,   /* U+0582 */
  0x0000,   /* U+0583 */
  0x0000,   /* U+0584 */
  0x0000,   /* U+0585 */
  0x0000,   /* U+0586 */
  0x22B1,   /* U+0587 */
  0x8588,   /* U+0588 */
  0x0000,   /* U+0589 */
  0x0000,   /* U+058A */
  0x858B,   /* U+058B */
  0x858C,   /* U+058C */
  0x0000,   /* U+058D */
  0x0000,   /* U+058E */
  0x0000,   /* U+058F */
  0x8590,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0000,   /* U+05B0 */
  0x0000,   /* U+05B1 */
  0x0000,   /* U+05B2 */
  0x0000,   /* U+05B3 */
  0x0000,   /* U+05B4 */
  0x0000,   /* U+05B5 */
  0x0000,   /* U+05B6 */
  0x0000,   /* U+05B7 */
  0x0000,   /* U+05B8 */
  0x0000,   /* U+05B9 */
  0x0000,   /* U+05BA */
  0x0000,   /* U+05BB */
  0x0000,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x0000,   /* U+05BE */
  0x0000,   /* U+05BF */
  0x0000,   /* U+05C0 */
  0x0000,   /* U+05C1 */
  0x0000,   /* U+05C2 */
  0x0000,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x0000,   /* U+05C6 */
  0x0000,   /* U+05C7 */
  0x85C8,   /* U+05C8 */
  0x85C9,   /* U+05C9 */
  0x85CA,   /* U+05CA */
  0x85CB,   /* U+05CB */
  0x85CC,   /* U+05CC */
  0x85CD,   /* U+05CD */
  0x85CE,   /* U+05CE */
  0x85CF,   /* U+05CF */
  0x0000,   /* U+05D0 */
  0x0000,   /* U+05D1 */
  0x0000,   /* U+05D2 */
  0x0000,   /* U+05D3 */
  0x0000,   /* U+05D4 */
  0x0000,   /* U+05D5 */
  0x0000,   /* U+05D6 */
  0x0000,   /* U+05D7 */
  0x0000,   /* U+05D8 */
  0x0000,   /* U+05D9 */
  0x0000,   /* U+05DA */
  0x0000,   /* U+05DB */
  0x0000,   /* U+05DC */
  0x0000,   /* U+05DD */
  0x0000,   /* U+05DE */
  0x0000,   /* U+05DF */
  0x0000,   /* U+05E0 */
  0x0000,   /* U+05E1 */
  0x0000,   /* U+05E2 */
  0x0000,   /* U+05E3 */
  0x0000,   /* U+05E4 */
  0x0000,   /* U+05E5 */
  0x0000,   /* U+05E6 */
  0x0000,   /* U+05E7 */
  0x0000,   /* U+05E8 */
  0x0000,   /* U+05E9 */
  0x0000,   /* U+05EA */
  0x85EB,   /* U+05EB */
  0x85EC,   /* U+05EC */
  0x85ED,   /* U+05ED */
  0x85EE,   /* U+05EE */
  0x85EF,   /* U+05EF */
  0x22BC,   /* U+05F0 */
  0x22C0,   /* U+05F1 */
  0x22C0,   /* U+05F2 */
  0x0000,   /* U+05F3 */
  0x0000,   /* U+05F4 */
  0x85F5,   /* U+05F5 */
  0x85F6,   /* U+05F6 */
  0x85F7,   /* U+05F7 */
  0x85F8,   /* U+05F8 */
  0x85F9,   /* U+05F9 */
  0x85FA,   /* U+05FA */
  0x85FB,   /* U+05FB */
  0x85FC,   /* U+05FC */
  0x85FD,   /* U+05FD */
  0x85FE,   /* U+05FE */
  0x85FF,   /* U+05FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0500 */
  0x0000,   /* U+0501 */
  0x0000,   /* U+0502 */
  0x0000,   /* U+0503 */
  0x0000,   /* U+0504 */
  0x0000,   /* U+0505 */
  0x0000,   /* U+0506 */
  0x0000,   /* U+0507 */
  0x0000,   /* U+0508 */
  0x0000,   /* U+0509 */
  0x0000,   /* U+050A */
  0x0000,   /* U+050B */
  0x0000,   /* U+050C */
  0x0000,   /* U+050D */
  0x0000,   /* U+050E */
  0x0000,   /* U+050F */
  0x0000,   /* U+0510 */
  0x0000,   /* U+0511 */
  0x0000,   /* U+0512 */
  0x0000,   /* U+0513 */
  0x0000,   /* U+0514 */
  0x0000,   /* U+0515 */
  0x0000,   /* U+0516 */
  0x0000,   /* U+0517 */
  0x0000,   /* U+0518 */
  0x0000,   /* U+0519 */
  0x0000,   /* U+051A */
  0x0000,   /* U+051B */
  0x0000,   /* U+051C */
  0x0000,   /* U+051D */
  0x0000,   /* U+051E */
  0x0000,   /* U+051F */
  0x0000,   /* U+0520 */
  0x0000,   /* U+0521 */
  0x0000,   /* U+0522 */
  0x0000,   /* U+0523 */
  0x0000,   /* U+0524 */
  0x0000,   /* U+0525 */
  0x0000,   /* U+0526 */
  0x0000,   /* U+0527 */
  0x0000,   /* U+0528 */
  0x0000,   /* U+0529 */
  0x0000,   /* U+052A */
  0x0000,   /* U+052B */
  0x0000,   /* U+052C */
  0x0000,   /* U+052D */
  0x0000,   /* U+052E */
  0x0000,   /* U+052F */
  0x0000,   /* U+0530 */
  0x0000,   /* U+0531 */
  0x0000,   /* U+0532 */
  0x0000,   /* U+0533 */
  0x0000,   /* U+0534 */
  0x0000,   /* U+0535 */
  0x0000,   /* U+0536 */
  0x0000,   /* U+0537 */
  0x0000,   /* U+0538 */
  0x0000,   /* U+0539 */
  0x0000,   /* U+053A */
  0x0000,   /* U+053B */
  0x0000,   /* U+053C */
  0x0000,   /* U+053D */
  0x0000,   /* U+053E */
  0x0000,   /* U+053F */
  0x0000,   /* U+0540 */
  0x0000,   /* U+0541 */
  0x0000,   /* U+0542 */
  0x0000,   /* U+0543 */
  0x0000,   /* U+0544 */
  0x0000,   /* U+0545 */
  0x0000,   /* U+0546 */
  0x0000,   /* U+0547 */
  0x0000,   /* U+0548 */
  0x0000,   /* U+0549 */
  0x0000,   /* U+054A */
  0x0000,   /* U+054B */
  0x0000,   /* U+054C */
  0x0000,   /* U+054D */
  0x0000,   /* U+054E */
  0x0000,   /* U+054F */
  0x0000,   /* U+0550 */
  0x0000,   /* U+0551 */
  0x0000,   /* U+0552 */
  0x0000,   /* U+0553 */
  0x0000,   /* U+0554 */
  0x0000,   /* U+0555 */
  0x0000,   /* U+0556 */
  0x0000,   /* U+0557 */
  0x0000,   /* U+0558 */
  0x0000,   /* U+0559 */
  0x0000,   /* U+055A */
  0x0000,   /* U+055B */
  0x0000,   /* U+055C */
  0x0000,   /* U+055D */
  0x0000,   /* U+055E */
  0x0000,   /* U+055F */
  0x0000,   /* U+0560 */
  0x0000,   /* U+0561 */
  0x0000,   /* U+0562 */
  0x0000,   /* U+0563 */
  0x0000,   /* U+0564 */
  0x0000,   /* U+0565 */
  0x0000,   /* U+0566 */
  0x0000,   /* U+0567 */
  0x0000,   /* U+0568 */
  0x0000,   /* U+0569 */
  0x0000,   /* U+056A */
  0x0000,   /* U+056B */
  0x0000,   /* U+056C */
  0x0000,   /* U+056D */
  0x0000,   /* U+056E */
  0x0000,   /* U+056F */
  0x0000,   /* U+0570 */
  0x0000,   /* U+0571 */
  0x0000,   /* U+0572 */
  0x0000,   /* U+0573 */
  0x0000,   /* U+0574 */
  0x0000,   /* U+0575 */
  0x0000,   /* U+0576 */
  0x0000,   /* U+0577 */
  0x0000,   /* U+0578 */
  0x0000,   /* U+0579 */
  0x0000,   /* U+057A */
  0x0000,   /* U+057B */
  0x0000,   /* U+057C */
  0x0000,   /* U+057D */
  0x0000,   /* U+057E */
  0x0000,   /* U+057F */
  0x0000,   /* U+0580 */
  0x0000,   /* U+0581 */
  0x0000,   /* U+0582 */
  0x0000,   /* U+0583 */
  0x0000,   /* U+0584 */
  0x0000,   /* U+0585 */
  0x0000,   /* U+0586 */
  0x0020,   /* U+0587 */
  0x0000,   /* U+0588 */
  0x0000,   /* U+0589 */
  0x0000,   /* U+058A */
  0x0000,   /* U+058B */
  0x0000,   /* U+058C */
  0x0000,   /* U+058D */
  0x0000,   /* U+058E */
  0x0000,   /* U+058F */
  0x0000,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0000,   /* U+05B0 */
  0x0000,   /* U+05B1 */
  0x0000,   /* U+05B2 */
  0x0000,   /* U+05B3 */
  0x0000,   /* U+05B4 */
  0x0000,   /* U+05B5 */
  0x0000,   /* U+05B6 */
  0x0000,   /* U+05B7 */
  0x0000,   /* U+05B8 */
  0x0000,   /* U+05B9 */
  0x0000,   /* U+05BA */
  0x0000,   /* U+05BB */
  0x0000,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x0000,   /* U+05BE */
  0x0000,   /* U+05BF */
  0x0000,   /* U+05C0 */
  0x0000,   /* U+05C1 */
  0x0000,   /* U+05C2 */
  0x0000,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x0000,   /* U+05C6 */
  0x0000,   /* U+05C7 */
  0x0000,   /* U+05C8 */
  0x0000,   /* U+05C9 */
  0x0000,   /* U+05CA */
  0x0000,   /* U+05CB */
  0x0000,   /* U+05CC */
  0x0000,   /* U+05CD */
  0x0000,   /* U+05CE */
  0x0000,   /* U+05CF */
  0x0000,   /* U+05D0 */
  0x0000,   /* U+05D1 */
  0x0000,   /* U+05D2 */
  0x0000,   /* U+05D3 */
  0x0000,   /* U+05D4 */
  0x0000,   /* U+05D5 */
  0x0000,   /* U+05D6 */
  0x0000,   /* U+05D7 */
  0x0000,   /* U+05D8 */
  0x0000,   /* U+05D9 */
  0x0000,   /* U+05DA */
  0x0000,   /* U+05DB */
  0x0000,   /* U+05DC */
  0x0000,   /* U+05DD */
  0x0000,   /* U+05DE */
  0x0000,   /* U+05DF */
  0x0000,   /* U+05E0 */
  0x0000,   /* U+05E1 */
  0x0000,   /* U+05E2 */
  0x0000,   /* U+05E3 */
  0x0000,   /* U+05E4 */
  0x0000,   /* U+05E5 */
  0x0000,   /* U+05E6 */
  0x0000,   /* U+05E7 */
  0x0000,   /* U+05E8 */
  0x0000,   /* U+05E9 */
  0x0000,   /* U+05EA */
  0x0000,   /* U+05EB */
  0x0000,   /* U+05EC */
  0x0000,   /* U+05ED */
  0x0000,   /* U+05EE */
  0x0000,   /* U+05EF */
  0x0020,   /* U+05F0 */
  0x0020,   /* U+05F1 */
  0x0020,   /* U+05F2 */
  0x0000,   /* U+05F3 */
  0x0000,   /* U+05F4 */
  0x0000,   /* U+05F5 */
  0x0000,   /* U+05F6 */
  0x0000,   /* U+05F7 */
  0x0000,   /* U+05F8 */
  0x0000,   /* U+05F9 */
  0x0000,   /* U+05FA */
  0x0000,   /* U+05FB */
  0x0000,   /* U+05FC */
  0x0000,   /* U+05FD */
  0x0000,   /* U+05FE */
  0x0000,   /* U+05FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0500 */
  0x0000,   /* U+0501 */
  0x0000,   /* U+0502 */
  0x0000,   /* U+0503 */
  0x0000,   /* U+0504 */
  0x0000,   /* U+0505 */
  0x0000,   /* U+0506 */
  0x0000,   /* U+0507 */
  0x0000,   /* U+0508 */
  0x0000,   /* U+0509 */
  0x0000,   /* U+050A */
  0x0000,   /* U+050B */
  0x0000,   /* U+050C */
  0x0000,   /* U+050D */
  0x0000,   /* U+050E */
  0x0000,   /* U+050F */
  0x0000,   /* U+0510 */
  0x0000,   /* U+0511 */
  0x0000,   /* U+0512 */
  0x0000,   /* U+0513 */
  0x0000,   /* U+0514 */
  0x0000,   /* U+0515 */
  0x0000,   /* U+0516 */
  0x0000,   /* U+0517 */
  0x0000,   /* U+0518 */
  0x0000,   /* U+0519 */
  0x0000,   /* U+051A */
  0x0000,   /* U+051B */
  0x0000,   /* U+051C */
  0x0000,   /* U+051D */
  0x0000,   /* U+051E */
  0x0000,   /* U+051F */
  0x0000,   /* U+0520 */
  0x0000,   /* U+0521 */
  0x0000,   /* U+0522 */
  0x0000,   /* U+0523 */
  0x0000,   /* U+0524 */
  0x0000,   /* U+0525 */
  0x0000,   /* U+0526 */
  0x0000,   /* U+0527 */
  0x0000,   /* U+0528 */
  0x0000,   /* U+0529 */
  0x0000,   /* U+052A */
  0x0000,   /* U+052B */
  0x0000,   /* U+052C */
  0x0000,   /* U+052D */
  0x0000,   /* U+052E */
  0x0000,   /* U+052F */
  0x0000,   /* U+0530 */
  0x0000,   /* U+0531 */
  0x0000,   /* U+0532 */
  0x0000,   /* U+0533 */
  0x0000,   /* U+0534 */
  0x0000,   /* U+0535 */
  0x0000,   /* U+0536 */
  0x0000,   /* U+0537 */
  0x0000,   /* U+0538 */
  0x0000,   /* U+0539 */
  0x0000,   /* U+053A */
  0x0000,   /* U+053B */
  0x0000,   /* U+053C */
  0x0000,   /* U+053D */
  0x0000,   /* U+053E */
  0x0000,   /* U+053F */
  0x0000,   /* U+0540 */
  0x0000,   /* U+0541 */
  0x0000,   /* U+0542 */
  0x0000,   /* U+0543 */
  0x0000,   /* U+0544 */
  0x0000,   /* U+0545 */
  0x0000,   /* U+0546 */
  0x0000,   /* U+0547 */
  0x0000,   /* U+0548 */
  0x0000,   /* U+0549 */
  0x0000,   /* U+054A */
  0x0000,   /* U+054B */
  0x0000,   /* U+054C */
  0x0000,   /* U+054D */
  0x0000,   /* U+054E */
  0x0000,   /* U+054F */
  0x0000,   /* U+0550 */
  0x0000,   /* U+0551 */
  0x0000,   /* U+0552 */
  0x0000,   /* U+0553 */
  0x0000,   /* U+0554 */
  0x0000,   /* U+0555 */
  0x0000,   /* U+0556 */
  0x0000,   /* U+0557 */
  0x0000,   /* U+0558 */
  0x0000,   /* U+0559 */
  0x0000,   /* U+055A */
  0x0000,   /* U+055B */
  0x0000,   /* U+055C */
  0x0000,   /* U+055D */
  0x0000,   /* U+055E */
  0x0000,   /* U+055F */
  0x0000,   /* U+0560 */
  0x0000,   /* U+0561 */
  0x0000,   /* U+0562 */
  0x0000,   /* U+0563 */
  0x0000,   /* U+0564 */
  0x0000,   /* U+0565 */
  0x0000,   /* U+0566 */
  0x0000,   /* U+0567 */
  0x0000,   /* U+0568 */
  0x0000,   /* U+0569 */
  0x0000,   /* U+056A */
  0x0000,   /* U+056B */
  0x0000,   /* U+056C */
  0x0000,   /* U+056D */
  0x0000,   /* U+056E */
  0x0000,   /* U+056F */
  0x0000,   /* U+0570 */
  0x0000,   /* U+0571 */
  0x0000,   /* U+0572 */
  0x0000,   /* U+0573 */
  0x0000,   /* U+0574 */
  0x0000,   /* U+0575 */
  0x0000,   /* U+0576 */
  0x0000,   /* U+0577 */
  0x0000,   /* U+0578 */
  0x0000,   /* U+0579 */
  0x0000,   /* U+057A */
  0x0000,   /* U+057B */
  0x0000,   /* U+057C */
  0x0000,   /* U+057D */
  0x0000,   /* U+057E */
  0x0000,   /* U+057F */
  0x0000,   /* U+0580 */
  0x0000,   /* U+0581 */
  0x0000,   /* U+0582 */
  0x0000,   /* U+0583 */
  0x0000,   /* U+0584 */
  0x0000,   /* U+0585 */
  0x0000,   /* U+0586 */
  0x0004,   /* U+0587 */
  0x0000,   /* U+0588 */
  0x0000,   /* U+0589 */
  0x0000,   /* U+058A */
  0x0000,   /* U+058B */
  0x0000,   /* U+058C */
  0x0000,   /* U+058D */
  0x0000,   /* U+058E */
  0x0000,   /* U+058F */
  0x0000,   /* U+0590 */
  0x0000,   /* U+0591 */
  0x0000,   /* U+0592 */
  0x0000,   /* U+0593 */
  0x0000,   /* U+0594 */
  0x0000,   /* U+0595 */
  0x0000,   /* U+0596 */
  0x0000,   /* U+0597 */
  0x0000,   /* U+0598 */
  0x0000,   /* U+0599 */
  0x0000,   /* U+059A */
  0x0000,   /* U+059B */
  0x0000,   /* U+059C */
  0x0000,   /* U+059D */
  0x0000,   /* U+059E */
  0x0000,   /* U+059F */
  0x0000,   /* U+05A0 */
  0x0000,   /* U+05A1 */
  0x0000,   /* U+05A2 */
  0x0000,   /* U+05A3 */
  0x0000,   /* U+05A4 */
  0x0000,   /* U+05A5 */
  0x0000,   /* U+05A6 */
  0x0000,   /* U+05A7 */
  0x0000,   /* U+05A8 */
  0x0000,   /* U+05A9 */
  0x0000,   /* U+05AA */
  0x0000,   /* U+05AB */
  0x0000,   /* U+05AC */
  0x0000,   /* U+05AD */
  0x0000,   /* U+05AE */
  0x0000,   /* U+05AF */
  0x0000,   /* U+05B0 */
  0x0000,   /* U+05B1 */
  0x0000,   /* U+05B2 */
  0x0000,   /* U+05B3 */
  0x0000,   /* U+05B4 */
  0x0000,   /* U+05B5 */
  0x0000,   /* U+05B6 */
  0x0000,   /* U+05B7 */
  0x0000,   /* U+05B8 */
  0x0000,   /* U+05B9 */
  0x0000,   /* U+05BA */
  0x0000,   /* U+05BB */
  0x0000,   /* U+05BC */
  0x0000,   /* U+05BD */
  0x0000,   /* U+05BE */
  0x0000,   /* U+05BF */
  0x0000,   /* U+05C0 */
  0x0000,   /* U+05C1 */
  0x0000,   /* U+05C2 */
  0x0000,   /* U+05C3 */
  0x0000,   /* U+05C4 */
  0x0000,   /* U+05C5 */
  0x0000,   /* U+05C6 */
  0x0000,   /* U+05C7 */
  0x0000,   /* U+05C8 */
  0x0000,   /* U+05C9 */
  0x0000,   /* U+05CA */
  0x0000,   /* U+05CB */
  0x0000,   /* U+05CC */
  0x0000,   /* U+05CD */
  0x0000,   /* U+05CE */
  0x0000,   /* U+05CF */
  0x0000,   /* U+05D0 */
  0x0000,   /* U+05D1 */
  0x0000,   /* U+05D2 */
  0x0000,   /* U+05D3 */
  0x0000,   /* U+05D4 */
  0x0000,   /* U+05D5 */
  0x0000,   /* U+05D6 */
  0x0000,   /* U+05D7 */
  0x0000,   /* U+05D8 */
  0x0000,   /* U+05D9 */
  0x0000,   /* U+05DA */
  0x0000,   /* U+05DB */
  0x0000,   /* U+05DC */
  0x0000,   /* U+05DD */
  0x0000,   /* U+05DE */
  0x0000,   /* U+05DF */
  0x0000,   /* U+05E0 */
  0x0000,   /* U+05E1 */
  0x0000,   /* U+05E2 */
  0x0000,   /* U+05E3 */
  0x0000,   /* U+05E4 */
  0x0000,   /* U+05E5 */
  0x0000,   /* U+05E6 */
  0x0000,   /* U+05E7 */
  0x0000,   /* U+05E8 */
  0x0000,   /* U+05E9 */
  0x0000,   /* U+05EA */
  0x0000,   /* U+05EB */
  0x0000,   /* U+05EC */
  0x0000,   /* U+05ED */
  0x0000,   /* U+05EE */
  0x0000,   /* U+05EF */
  0x0004,   /* U+05F0 */
  0x0004,   /* U+05F1 */
  0x0004,   /* U+05F2 */
  0x0000,   /* U+05F3 */
  0x0000,   /* U+05F4 */
  0x0000,   /* U+05F5 */
  0x0000,   /* U+05F6 */
  0x0000,   /* U+05F7 */
  0x0000,   /* U+05F8 */
  0x0000,   /* U+05F9 */
  0x0000,   /* U+05FA */
  0x0000,   /* U+05FB */
  0x0000,   /* U+05FC */
  0x0000,   /* U+05FD */
  0x0000,   /* U+05FE */
  0x0000,   /* U+05FF */
};

uint16 uca900_p006[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x062D,   /* U+0606 */
  0x062F,   /* U+0607 */
  0x04FA,   /* U+0608 */
  0x039C,   /* U+0609 */
  0x039E,   /* U+060A */
  0x1C16,   /* U+060B */
  0x0227,   /* U+060C */
  0x0228,   /* U+060D */
  0x04FD,   /* U+060E */
  0x04FE,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0235,   /* U+061B */
  0x0000,   /* U+061C */
  0xFBC0,   /* U+061D */
  0x023B,   /* U+061E */
  0x026A,   /* U+061F */
  0x23CF,   /* U+0620 */
  0x22FD,   /* U+0621 */
  0x22FE,   /* U+0622 */
  0x22FF,   /* U+0623 */
  0x2302,   /* U+0624 */
  0x2303,   /* U+0625 */
  0x2307,   /* U+0626 */
  0x230B,   /* U+0627 */
  0x230D,   /* U+0628 */
  0x231C,   /* U+0629 */
  0x231D,   /* U+062A */
  0x231E,   /* U+062B */
  0x2325,   /* U+062C */
  0x232C,   /* U+062D */
  0x232D,   /* U+062E */
  0x2337,   /* U+062F */
  0x2338,   /* U+0630 */
  0x2346,   /* U+0631 */
  0x2347,   /* U+0632 */
  0x2359,   /* U+0633 */
  0x235A,   /* U+0634 */
  0x2364,   /* U+0635 */
  0x2365,   /* U+0636 */
  0x236A,   /* U+0637 */
  0x236B,   /* U+0638 */
  0x236E,   /* U+0639 */
  0x236F,   /* U+063A */
  0x2398,   /* U+063B */
  0x2399,   /* U+063C */
  0x23CC,   /* U+063D */
  0x23CD,   /* U+063E */
  0x23CE,   /* U+063F */
  0x0000,   /* U+0640 */
  0x2376,   /* U+0641 */
  0x2382,   /* U+0642 */
  0x2387,   /* U+0643 */
  0x239C,   /* U+0644 */
  0x23A3,   /* U+0645 */
  0x23A7,   /* U+0646 */
  0x23B1,   /* U+0647 */
  0x23B7,   /* U+0648 */
  0x23C5,   /* U+0649 */
  0x23C6,   /* U+064A */
  0x0000,   /* U+064B */
  0x0000,   /* U+064C */
  0x0000,   /* U+064D */
  0x0000,   /* U+064E */
  0x0000,   /* U+064F */
  0x0000,   /* U+0650 */
  0x0000,   /* U+0651 */
  0x0000,   /* U+0652 */
  0x0000,   /* U+0653 */
  0x0000,   /* U+0654 */
  0x0000,   /* U+0655 */
  0x0000,   /* U+0656 */
  0x0000,   /* U+0657 */
  0x0000,   /* U+0658 */
  0x0000,   /* U+0659 */
  0x0000,   /* U+065A */
  0x0000,   /* U+065B */
  0x0000,   /* U+065C */
  0x0000,   /* U+065D */
  0x0000,   /* U+065E */
  0x0000,   /* U+065F */
  0x1C3D,   /* U+0660 */
  0x1C3E,   /* U+0661 */
  0x1C3F,   /* U+0662 */
  0x1C40,   /* U+0663 */
  0x1C41,   /* U+0664 */
  0x1C42,   /* U+0665 */
  0x1C43,   /* U+0666 */
  0x1C44,   /* U+0667 */
  0x1C45,   /* U+0668 */
  0x1C46,   /* U+0669 */
  0x039A,   /* U+066A */
  0x0229,   /* U+066B */
  0x022A,   /* U+066C */
  0x0392,   /* U+066D */
  0x230C,   /* U+066E */
  0x2381,   /* U+066F */
  0x0000,   /* U+0670 */
  0x2301,   /* U+0671 */
  0x2300,   /* U+0672 */
  0x2304,   /* U+0673 */
  0x22FD,   /* U+0674 */
  0x230B,   /* U+0675 */
  0x23B7,   /* U+0676 */
  0x23BB,   /* U+0677 */
  0x23C6,   /* U+0678 */
  0x231F,   /* U+0679 */
  0x2320,   /* U+067A */
  0x230E,   /* U+067B */
  0x2321,   /* U+067C */
  0x2322,   /* U+067D */
  0x230F,   /* U+067E */
  0x2323,   /* U+067F */
  0x2310,   /* U+0680 */
  0x232E,   /* U+0681 */
  0x232F,   /* U+0682 */
  0x2326,   /* U+0683 */
  0x2327,   /* U+0684 */
  0x2330,   /* U+0685 */
  0x2328,   /* U+0686 */
  0x232A,   /* U+0687 */
  0x2339,   /* U+0688 */
  0x233A,   /* U+0689 */
  0x233B,   /* U+068A */
  0x233C,   /* U+068B */
  0x233D,   /* U+068C */
  0x233E,   /* U+068D */
  0x2340,   /* U+068E */
  0x2341,   /* U+068F */
  0x2342,   /* U+0690 */
  0x2348,   /* U+0691 */
  0x2349,   /* U+0692 */
  0x234A,   /* U+0693 */
  0x234B,   /* U+0694 */
  0x234C,   /* U+0695 */
  0x234D,   /* U+0696 */
  0x234E,   /* U+0697 */
  0x234F,   /* U+0698 */
  0x2350,   /* U+0699 */
  0x235B,   /* U+069A */
  0x235C,   /* U+069B */
  0x235D,   /* U+069C */
  0x2366,   /* U+069D */
  0x2368,   /* U+069E */
  0x236C,   /* U+069F */
  0x2370,   /* U+06A0 */
  0x2377,   /* U+06A1 */
  0x2378,   /* U+06A2 */
  0x237A,   /* U+06A3 */
  0x237B,   /* U+06A4 */
  0x237D,   /* U+06A5 */
  0x237E,   /* U+06A6 */
  0x2383,   /* U+06A7 */
  0x2385,   /* U+06A8 */
  0x2388,   /* U+06A9 */
  0x2389,   /* U+06AA */
  0x238A,   /* U+06AB */
  0x238B,   /* U+06AC */
  0x238D,   /* U+06AD */
  0x238E,   /* U+06AE */
  0x2390,   /* U+06AF */
  0x2392,   /* U+06B0 */
  0x2393,   /* U+06B1 */
  0x2394,   /* U+06B2 */
  0x2395,   /* U+06B3 */
  0x2396,   /* U+06B4 */
  0x239D,   /* U+06B5 */
  0x239E,   /* U+06B6 */
  0x239F,   /* U+06B7 */
  0x23A0,   /* U+06B8 */
  0x23AD,   /* U+06B9 */
  0x23A8,   /* U+06BA */
  0x23AA,   /* U+06BB */
  0x23AB,   /* U+06BC */
  0x23AC,   /* U+06BD */
  0x23B2,   /* U+06BE */
  0x2329,   /* U+06BF */
  0x23B6,   /* U+06C0 */
  0x23B3,   /* U+06C1 */
  0x23B3,   /* U+06C2 */
  0x23B4,   /* U+06C3 */
  0x23B8,   /* U+06C4 */
  0x23B9,   /* U+06C5 */
  0x23BA,   /* U+06C6 */
  0x23BB,   /* U+06C7 */
  0x23BC,   /* U+06C8 */
  0x23BD,   /* U+06C9 */
  0x23BE,   /* U+06CA */
  0x23BF,   /* U+06CB */
  0x23C7,   /* U+06CC */
  0x23C8,   /* U+06CD */
  0x23C9,   /* U+06CE */
  0x23C1,   /* U+06CF */
  0x23CA,   /* U+06D0 */
  0x23CB,   /* U+06D1 */
  0x23D4,   /* U+06D2 */
  0x23D4,   /* U+06D3 */
  0x0279,   /* U+06D4 */
  0x23B6,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x04FF,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x23B7,   /* U+06E5 */
  0x23C6,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0500,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x2343,   /* U+06EE */
  0x2351,   /* U+06EF */
  0x1C3D,   /* U+06F0 */
  0x1C3E,   /* U+06F1 */
  0x1C3F,   /* U+06F2 */
  0x1C40,   /* U+06F3 */
  0x1C41,   /* U+06F4 */
  0x1C42,   /* U+06F5 */
  0x1C43,   /* U+06F6 */
  0x1C44,   /* U+06F7 */
  0x1C45,   /* U+06F8 */
  0x1C46,   /* U+06F9 */
  0x235E,   /* U+06FA */
  0x2369,   /* U+06FB */
  0x2371,   /* U+06FC */
  0x22FD,   /* U+06FD */
  0x23A3,   /* U+06FE */
  0x23B5,   /* U+06FF */

  /* Secondary weight 1 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x0020,   /* U+0606 */
  0x0020,   /* U+0607 */
  0x0020,   /* U+0608 */
  0x0020,   /* U+0609 */
  0x0020,   /* U+060A */
  0x0020,   /* U+060B */
  0x0020,   /* U+060C */
  0x0020,   /* U+060D */
  0x0020,   /* U+060E */
  0x0020,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0020,   /* U+061B */
  0x0000,   /* U+061C */
  0x0020,   /* U+061D */
  0x0020,   /* U+061E */
  0x0020,   /* U+061F */
  0x0020,   /* U+0620 */
  0x0020,   /* U+0621 */
  0x0020,   /* U+0622 */
  0x0020,   /* U+0623 */
  0x0020,   /* U+0624 */
  0x0020,   /* U+0625 */
  0x0020,   /* U+0626 */
  0x0020,   /* U+0627 */
  0x0020,   /* U+0628 */
  0x0020,   /* U+0629 */
  0x0020,   /* U+062A */
  0x0020,   /* U+062B */
  0x0020,   /* U+062C */
  0x0020,   /* U+062D */
  0x0020,   /* U+062E */
  0x0020,   /* U+062F */
  0x0020,   /* U+0630 */
  0x0020,   /* U+0631 */
  0x0020,   /* U+0632 */
  0x0020,   /* U+0633 */
  0x0020,   /* U+0634 */
  0x0020,   /* U+0635 */
  0x0020,   /* U+0636 */
  0x0020,   /* U+0637 */
  0x0020,   /* U+0638 */
  0x0020,   /* U+0639 */
  0x0020,   /* U+063A */
  0x0020,   /* U+063B */
  0x0020,   /* U+063C */
  0x0020,   /* U+063D */
  0x0020,   /* U+063E */
  0x0020,   /* U+063F */
  0x0000,   /* U+0640 */
  0x0020,   /* U+0641 */
  0x0020,   /* U+0642 */
  0x0020,   /* U+0643 */
  0x0020,   /* U+0644 */
  0x0020,   /* U+0645 */
  0x0020,   /* U+0646 */
  0x0020,   /* U+0647 */
  0x0020,   /* U+0648 */
  0x0020,   /* U+0649 */
  0x0020,   /* U+064A */
  0x006D,   /* U+064B */
  0x0070,   /* U+064C */
  0x0073,   /* U+064D */
  0x0076,   /* U+064E */
  0x007A,   /* U+064F */
  0x007D,   /* U+0650 */
  0x0080,   /* U+0651 */
  0x0081,   /* U+0652 */
  0x0082,   /* U+0653 */
  0x0083,   /* U+0654 */
  0x0084,   /* U+0655 */
  0x0086,   /* U+0656 */
  0x0087,   /* U+0657 */
  0x0088,   /* U+0658 */
  0x008A,   /* U+0659 */
  0x008B,   /* U+065A */
  0x008C,   /* U+065B */
  0x008D,   /* U+065C */
  0x008E,   /* U+065D */
  0x008F,   /* U+065E */
  0x0085,   /* U+065F */
  0x0020,   /* U+0660 */
  0x0020,   /* U+0661 */
  0x0020,   /* U+0662 */
  0x0020,   /* U+0663 */
  0x0020,   /* U+0664 */
  0x0020,   /* U+0665 */
  0x0020,   /* U+0666 */
  0x0020,   /* U+0667 */
  0x0020,   /* U+0668 */
  0x0020,   /* U+0669 */
  0x0020,   /* U+066A */
  0x0020,   /* U+066B */
  0x0020,   /* U+066C */
  0x0020,   /* U+066D */
  0x0020,   /* U+066E */
  0x0020,   /* U+066F */
  0x0098,   /* U+0670 */
  0x0020,   /* U+0671 */
  0x0020,   /* U+0672 */
  0x0020,   /* U+0673 */
  0x0020,   /* U+0674 */
  0x0020,   /* U+0675 */
  0x0020,   /* U+0676 */
  0x0020,   /* U+0677 */
  0x0020,   /* U+0678 */
  0x0020,   /* U+0679 */
  0x0020,   /* U+067A */
  0x0020,   /* U+067B */
  0x0020,   /* U+067C */
  0x0020,   /* U+067D */
  0x0020,   /* U+067E */
  0x0020,   /* U+067F */
  0x0020,   /* U+0680 */
  0x0020,   /* U+0681 */
  0x0020,   /* U+0682 */
  0x0020,   /* U+0683 */
  0x0020,   /* U+0684 */
  0x0020,   /* U+0685 */
  0x0020,   /* U+0686 */
  0x0020,   /* U+0687 */
  0x0020,   /* U+0688 */
  0x0020,   /* U+0689 */
  0x0020,   /* U+068A */
  0x0020,   /* U+068B */
  0x0020,   /* U+068C */
  0x0020,   /* U+068D */
  0x0020,   /* U+068E */
  0x0020,   /* U+068F */
  0x0020,   /* U+0690 */
  0x0020,   /* U+0691 */
  0x0020,   /* U+0692 */
  0x0020,   /* U+0693 */
  0x0020,   /* U+0694 */
  0x0020,   /* U+0695 */
  0x0020,   /* U+0696 */
  0x0020,   /* U+0697 */
  0x0020,   /* U+0698 */
  0x0020,   /* U+0699 */
  0x0020,   /* U+069A */
  0x0020,   /* U+069B */
  0x0020,   /* U+069C */
  0x0020,   /* U+069D */
  0x0020,   /* U+069E */
  0x0020,   /* U+069F */
  0x0020,   /* U+06A0 */
  0x0020,   /* U+06A1 */
  0x0020,   /* U+06A2 */
  0x0020,   /* U+06A3 */
  0x0020,   /* U+06A4 */
  0x0020,   /* U+06A5 */
  0x0020,   /* U+06A6 */
  0x0020,   /* U+06A7 */
  0x0020,   /* U+06A8 */
  0x0020,   /* U+06A9 */
  0x0020,   /* U+06AA */
  0x0020,   /* U+06AB */
  0x0020,   /* U+06AC */
  0x0020,   /* U+06AD */
  0x0020,   /* U+06AE */
  0x0020,   /* U+06AF */
  0x0020,   /* U+06B0 */
  0x0020,   /* U+06B1 */
  0x0020,   /* U+06B2 */
  0x0020,   /* U+06B3 */
  0x0020,   /* U+06B4 */
  0x0020,   /* U+06B5 */
  0x0020,   /* U+06B6 */
  0x0020,   /* U+06B7 */
  0x0020,   /* U+06B8 */
  0x0020,   /* U+06B9 */
  0x0020,   /* U+06BA */
  0x0020,   /* U+06BB */
  0x0020,   /* U+06BC */
  0x0020,   /* U+06BD */
  0x0020,   /* U+06BE */
  0x0020,   /* U+06BF */
  0x0020,   /* U+06C0 */
  0x0020,   /* U+06C1 */
  0x0020,   /* U+06C2 */
  0x0020,   /* U+06C3 */
  0x0020,   /* U+06C4 */
  0x0020,   /* U+06C5 */
  0x0020,   /* U+06C6 */
  0x0020,   /* U+06C7 */
  0x0020,   /* U+06C8 */
  0x0020,   /* U+06C9 */
  0x0020,   /* U+06CA */
  0x0020,   /* U+06CB */
  0x0020,   /* U+06CC */
  0x0020,   /* U+06CD */
  0x0020,   /* U+06CE */
  0x0020,   /* U+06CF */
  0x0020,   /* U+06D0 */
  0x0020,   /* U+06D1 */
  0x0020,   /* U+06D2 */
  0x0020,   /* U+06D3 */
  0x0020,   /* U+06D4 */
  0x0020,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x0020,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x0020,   /* U+06E5 */
  0x0020,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0020,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x0020,   /* U+06EE */
  0x0020,   /* U+06EF */
  0x0020,   /* U+06F0 */
  0x0020,   /* U+06F1 */
  0x0020,   /* U+06F2 */
  0x0020,   /* U+06F3 */
  0x0020,   /* U+06F4 */
  0x0020,   /* U+06F5 */
  0x0020,   /* U+06F6 */
  0x0020,   /* U+06F7 */
  0x0020,   /* U+06F8 */
  0x0020,   /* U+06F9 */
  0x0020,   /* U+06FA */
  0x0020,   /* U+06FB */
  0x0020,   /* U+06FC */
  0x0020,   /* U+06FD */
  0x0020,   /* U+06FE */
  0x0020,   /* U+06FF */

  /* Tertiary weight 1 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x0002,   /* U+0606 */
  0x0002,   /* U+0607 */
  0x0002,   /* U+0608 */
  0x0002,   /* U+0609 */
  0x0002,   /* U+060A */
  0x0002,   /* U+060B */
  0x0002,   /* U+060C */
  0x0002,   /* U+060D */
  0x0002,   /* U+060E */
  0x0002,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0002,   /* U+061B */
  0x0000,   /* U+061C */
  0x0002,   /* U+061D */
  0x0002,   /* U+061E */
  0x0002,   /* U+061F */
  0x0002,   /* U+0620 */
  0x0002,   /* U+0621 */
  0x0002,   /* U+0622 */
  0x0002,   /* U+0623 */
  0x0002,   /* U+0624 */
  0x0002,   /* U+0625 */
  0x0002,   /* U+0626 */
  0x0002,   /* U+0627 */
  0x0002,   /* U+0628 */
  0x0002,   /* U+0629 */
  0x0002,   /* U+062A */
  0x0002,   /* U+062B */
  0x0002,   /* U+062C */
  0x0002,   /* U+062D */
  0x0002,   /* U+062E */
  0x0002,   /* U+062F */
  0x0002,   /* U+0630 */
  0x0002,   /* U+0631 */
  0x0002,   /* U+0632 */
  0x0002,   /* U+0633 */
  0x0002,   /* U+0634 */
  0x0002,   /* U+0635 */
  0x0002,   /* U+0636 */
  0x0002,   /* U+0637 */
  0x0002,   /* U+0638 */
  0x0002,   /* U+0639 */
  0x0002,   /* U+063A */
  0x0002,   /* U+063B */
  0x0002,   /* U+063C */
  0x0002,   /* U+063D */
  0x0002,   /* U+063E */
  0x0002,   /* U+063F */
  0x0000,   /* U+0640 */
  0x0002,   /* U+0641 */
  0x0002,   /* U+0642 */
  0x0002,   /* U+0643 */
  0x0002,   /* U+0644 */
  0x0002,   /* U+0645 */
  0x0002,   /* U+0646 */
  0x0002,   /* U+0647 */
  0x0002,   /* U+0648 */
  0x0002,   /* U+0649 */
  0x0002,   /* U+064A */
  0x0002,   /* U+064B */
  0x0002,   /* U+064C */
  0x0002,   /* U+064D */
  0x0002,   /* U+064E */
  0x0002,   /* U+064F */
  0x0002,   /* U+0650 */
  0x0002,   /* U+0651 */
  0x0002,   /* U+0652 */
  0x0002,   /* U+0653 */
  0x0002,   /* U+0654 */
  0x0002,   /* U+0655 */
  0x0002,   /* U+0656 */
  0x0002,   /* U+0657 */
  0x0002,   /* U+0658 */
  0x0002,   /* U+0659 */
  0x0002,   /* U+065A */
  0x0002,   /* U+065B */
  0x0002,   /* U+065C */
  0x0002,   /* U+065D */
  0x0002,   /* U+065E */
  0x0002,   /* U+065F */
  0x0002,   /* U+0660 */
  0x0002,   /* U+0661 */
  0x0002,   /* U+0662 */
  0x0002,   /* U+0663 */
  0x0002,   /* U+0664 */
  0x0002,   /* U+0665 */
  0x0002,   /* U+0666 */
  0x0002,   /* U+0667 */
  0x0002,   /* U+0668 */
  0x0002,   /* U+0669 */
  0x0002,   /* U+066A */
  0x0002,   /* U+066B */
  0x0002,   /* U+066C */
  0x0002,   /* U+066D */
  0x0002,   /* U+066E */
  0x0002,   /* U+066F */
  0x0002,   /* U+0670 */
  0x0002,   /* U+0671 */
  0x0002,   /* U+0672 */
  0x0002,   /* U+0673 */
  0x0004,   /* U+0674 */
  0x0004,   /* U+0675 */
  0x0004,   /* U+0676 */
  0x0004,   /* U+0677 */
  0x0004,   /* U+0678 */
  0x0002,   /* U+0679 */
  0x0002,   /* U+067A */
  0x0002,   /* U+067B */
  0x0002,   /* U+067C */
  0x0002,   /* U+067D */
  0x0002,   /* U+067E */
  0x0002,   /* U+067F */
  0x0002,   /* U+0680 */
  0x0002,   /* U+0681 */
  0x0002,   /* U+0682 */
  0x0002,   /* U+0683 */
  0x0002,   /* U+0684 */
  0x0002,   /* U+0685 */
  0x0002,   /* U+0686 */
  0x0002,   /* U+0687 */
  0x0002,   /* U+0688 */
  0x0002,   /* U+0689 */
  0x0002,   /* U+068A */
  0x0002,   /* U+068B */
  0x0002,   /* U+068C */
  0x0002,   /* U+068D */
  0x0002,   /* U+068E */
  0x0002,   /* U+068F */
  0x0002,   /* U+0690 */
  0x0002,   /* U+0691 */
  0x0002,   /* U+0692 */
  0x0002,   /* U+0693 */
  0x0002,   /* U+0694 */
  0x0002,   /* U+0695 */
  0x0002,   /* U+0696 */
  0x0002,   /* U+0697 */
  0x0002,   /* U+0698 */
  0x0002,   /* U+0699 */
  0x0002,   /* U+069A */
  0x0002,   /* U+069B */
  0x0002,   /* U+069C */
  0x0002,   /* U+069D */
  0x0002,   /* U+069E */
  0x0002,   /* U+069F */
  0x0002,   /* U+06A0 */
  0x0002,   /* U+06A1 */
  0x0002,   /* U+06A2 */
  0x0002,   /* U+06A3 */
  0x0002,   /* U+06A4 */
  0x0002,   /* U+06A5 */
  0x0002,   /* U+06A6 */
  0x0002,   /* U+06A7 */
  0x0002,   /* U+06A8 */
  0x0002,   /* U+06A9 */
  0x0002,   /* U+06AA */
  0x0002,   /* U+06AB */
  0x0002,   /* U+06AC */
  0x0002,   /* U+06AD */
  0x0002,   /* U+06AE */
  0x0002,   /* U+06AF */
  0x0002,   /* U+06B0 */
  0x0002,   /* U+06B1 */
  0x0002,   /* U+06B2 */
  0x0002,   /* U+06B3 */
  0x0002,   /* U+06B4 */
  0x0002,   /* U+06B5 */
  0x0002,   /* U+06B6 */
  0x0002,   /* U+06B7 */
  0x0002,   /* U+06B8 */
  0x0002,   /* U+06B9 */
  0x0002,   /* U+06BA */
  0x0002,   /* U+06BB */
  0x0002,   /* U+06BC */
  0x0002,   /* U+06BD */
  0x0002,   /* U+06BE */
  0x0002,   /* U+06BF */
  0x0002,   /* U+06C0 */
  0x0002,   /* U+06C1 */
  0x0002,   /* U+06C2 */
  0x0002,   /* U+06C3 */
  0x0002,   /* U+06C4 */
  0x0002,   /* U+06C5 */
  0x0002,   /* U+06C6 */
  0x0002,   /* U+06C7 */
  0x0002,   /* U+06C8 */
  0x0002,   /* U+06C9 */
  0x0002,   /* U+06CA */
  0x0002,   /* U+06CB */
  0x0002,   /* U+06CC */
  0x0002,   /* U+06CD */
  0x0002,   /* U+06CE */
  0x0002,   /* U+06CF */
  0x0002,   /* U+06D0 */
  0x0002,   /* U+06D1 */
  0x0002,   /* U+06D2 */
  0x0002,   /* U+06D3 */
  0x0002,   /* U+06D4 */
  0x0002,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x0002,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x0004,   /* U+06E5 */
  0x0004,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0002,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x0002,   /* U+06EE */
  0x0002,   /* U+06EF */
  0x0002,   /* U+06F0 */
  0x0002,   /* U+06F1 */
  0x0002,   /* U+06F2 */
  0x0002,   /* U+06F3 */
  0x0002,   /* U+06F4 */
  0x0002,   /* U+06F5 */
  0x0002,   /* U+06F6 */
  0x0002,   /* U+06F7 */
  0x0002,   /* U+06F8 */
  0x0002,   /* U+06F9 */
  0x0002,   /* U+06FA */
  0x0002,   /* U+06FB */
  0x0002,   /* U+06FC */
  0x0004,   /* U+06FD */
  0x0004,   /* U+06FE */
  0x0002,   /* U+06FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x0000,   /* U+0606 */
  0x0000,   /* U+0607 */
  0x0000,   /* U+0608 */
  0x0000,   /* U+0609 */
  0x0000,   /* U+060A */
  0x0000,   /* U+060B */
  0x0000,   /* U+060C */
  0x0000,   /* U+060D */
  0x0000,   /* U+060E */
  0x0000,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0000,   /* U+061B */
  0x0000,   /* U+061C */
  0x861D,   /* U+061D */
  0x0000,   /* U+061E */
  0x0000,   /* U+061F */
  0x0000,   /* U+0620 */
  0x0000,   /* U+0621 */
  0x0000,   /* U+0622 */
  0x0000,   /* U+0623 */
  0x0000,   /* U+0624 */
  0x0000,   /* U+0625 */
  0x0000,   /* U+0626 */
  0x0000,   /* U+0627 */
  0x0000,   /* U+0628 */
  0x0000,   /* U+0629 */
  0x0000,   /* U+062A */
  0x0000,   /* U+062B */
  0x0000,   /* U+062C */
  0x0000,   /* U+062D */
  0x0000,   /* U+062E */
  0x0000,   /* U+062F */
  0x0000,   /* U+0630 */
  0x0000,   /* U+0631 */
  0x0000,   /* U+0632 */
  0x0000,   /* U+0633 */
  0x0000,   /* U+0634 */
  0x0000,   /* U+0635 */
  0x0000,   /* U+0636 */
  0x0000,   /* U+0637 */
  0x0000,   /* U+0638 */
  0x0000,   /* U+0639 */
  0x0000,   /* U+063A */
  0x0000,   /* U+063B */
  0x0000,   /* U+063C */
  0x0000,   /* U+063D */
  0x0000,   /* U+063E */
  0x0000,   /* U+063F */
  0x0000,   /* U+0640 */
  0x0000,   /* U+0641 */
  0x0000,   /* U+0642 */
  0x0000,   /* U+0643 */
  0x0000,   /* U+0644 */
  0x0000,   /* U+0645 */
  0x0000,   /* U+0646 */
  0x0000,   /* U+0647 */
  0x0000,   /* U+0648 */
  0x0000,   /* U+0649 */
  0x0000,   /* U+064A */
  0x0000,   /* U+064B */
  0x0000,   /* U+064C */
  0x0000,   /* U+064D */
  0x0000,   /* U+064E */
  0x0000,   /* U+064F */
  0x0000,   /* U+0650 */
  0x0000,   /* U+0651 */
  0x0000,   /* U+0652 */
  0x0000,   /* U+0653 */
  0x0000,   /* U+0654 */
  0x0000,   /* U+0655 */
  0x0000,   /* U+0656 */
  0x0000,   /* U+0657 */
  0x0000,   /* U+0658 */
  0x0000,   /* U+0659 */
  0x0000,   /* U+065A */
  0x0000,   /* U+065B */
  0x0000,   /* U+065C */
  0x0000,   /* U+065D */
  0x0000,   /* U+065E */
  0x0000,   /* U+065F */
  0x0000,   /* U+0660 */
  0x0000,   /* U+0661 */
  0x0000,   /* U+0662 */
  0x0000,   /* U+0663 */
  0x0000,   /* U+0664 */
  0x0000,   /* U+0665 */
  0x0000,   /* U+0666 */
  0x0000,   /* U+0667 */
  0x0000,   /* U+0668 */
  0x0000,   /* U+0669 */
  0x0000,   /* U+066A */
  0x0000,   /* U+066B */
  0x0000,   /* U+066C */
  0x0000,   /* U+066D */
  0x0000,   /* U+066E */
  0x0000,   /* U+066F */
  0x0000,   /* U+0670 */
  0x0000,   /* U+0671 */
  0x0000,   /* U+0672 */
  0x0000,   /* U+0673 */
  0x0000,   /* U+0674 */
  0x22FD,   /* U+0675 */
  0x22FD,   /* U+0676 */
  0x22FD,   /* U+0677 */
  0x22FD,   /* U+0678 */
  0x0000,   /* U+0679 */
  0x0000,   /* U+067A */
  0x0000,   /* U+067B */
  0x0000,   /* U+067C */
  0x0000,   /* U+067D */
  0x0000,   /* U+067E */
  0x0000,   /* U+067F */
  0x0000,   /* U+0680 */
  0x0000,   /* U+0681 */
  0x0000,   /* U+0682 */
  0x0000,   /* U+0683 */
  0x0000,   /* U+0684 */
  0x0000,   /* U+0685 */
  0x0000,   /* U+0686 */
  0x0000,   /* U+0687 */
  0x0000,   /* U+0688 */
  0x0000,   /* U+0689 */
  0x0000,   /* U+068A */
  0x0000,   /* U+068B */
  0x0000,   /* U+068C */
  0x0000,   /* U+068D */
  0x0000,   /* U+068E */
  0x0000,   /* U+068F */
  0x0000,   /* U+0690 */
  0x0000,   /* U+0691 */
  0x0000,   /* U+0692 */
  0x0000,   /* U+0693 */
  0x0000,   /* U+0694 */
  0x0000,   /* U+0695 */
  0x0000,   /* U+0696 */
  0x0000,   /* U+0697 */
  0x0000,   /* U+0698 */
  0x0000,   /* U+0699 */
  0x0000,   /* U+069A */
  0x0000,   /* U+069B */
  0x0000,   /* U+069C */
  0x0000,   /* U+069D */
  0x0000,   /* U+069E */
  0x0000,   /* U+069F */
  0x0000,   /* U+06A0 */
  0x0000,   /* U+06A1 */
  0x0000,   /* U+06A2 */
  0x0000,   /* U+06A3 */
  0x0000,   /* U+06A4 */
  0x0000,   /* U+06A5 */
  0x0000,   /* U+06A6 */
  0x0000,   /* U+06A7 */
  0x0000,   /* U+06A8 */
  0x0000,   /* U+06A9 */
  0x0000,   /* U+06AA */
  0x0000,   /* U+06AB */
  0x0000,   /* U+06AC */
  0x0000,   /* U+06AD */
  0x0000,   /* U+06AE */
  0x0000,   /* U+06AF */
  0x0000,   /* U+06B0 */
  0x0000,   /* U+06B1 */
  0x0000,   /* U+06B2 */
  0x0000,   /* U+06B3 */
  0x0000,   /* U+06B4 */
  0x0000,   /* U+06B5 */
  0x0000,   /* U+06B6 */
  0x0000,   /* U+06B7 */
  0x0000,   /* U+06B8 */
  0x0000,   /* U+06B9 */
  0x0000,   /* U+06BA */
  0x0000,   /* U+06BB */
  0x0000,   /* U+06BC */
  0x0000,   /* U+06BD */
  0x0000,   /* U+06BE */
  0x0000,   /* U+06BF */
  0x0000,   /* U+06C0 */
  0x0000,   /* U+06C1 */
  0x0000,   /* U+06C2 */
  0x0000,   /* U+06C3 */
  0x0000,   /* U+06C4 */
  0x0000,   /* U+06C5 */
  0x0000,   /* U+06C6 */
  0x0000,   /* U+06C7 */
  0x0000,   /* U+06C8 */
  0x0000,   /* U+06C9 */
  0x0000,   /* U+06CA */
  0x0000,   /* U+06CB */
  0x0000,   /* U+06CC */
  0x0000,   /* U+06CD */
  0x0000,   /* U+06CE */
  0x0000,   /* U+06CF */
  0x0000,   /* U+06D0 */
  0x0000,   /* U+06D1 */
  0x0000,   /* U+06D2 */
  0x0000,   /* U+06D3 */
  0x0000,   /* U+06D4 */
  0x0000,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x0000,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x0000,   /* U+06E5 */
  0x0000,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0000,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x0000,   /* U+06EE */
  0x0000,   /* U+06EF */
  0x0000,   /* U+06F0 */
  0x0000,   /* U+06F1 */
  0x0000,   /* U+06F2 */
  0x0000,   /* U+06F3 */
  0x0000,   /* U+06F4 */
  0x0000,   /* U+06F5 */
  0x0000,   /* U+06F6 */
  0x0000,   /* U+06F7 */
  0x0000,   /* U+06F8 */
  0x0000,   /* U+06F9 */
  0x0000,   /* U+06FA */
  0x0000,   /* U+06FB */
  0x0000,   /* U+06FC */
  0x0000,   /* U+06FD */
  0x0000,   /* U+06FE */
  0x0000,   /* U+06FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x0000,   /* U+0606 */
  0x0000,   /* U+0607 */
  0x0000,   /* U+0608 */
  0x0000,   /* U+0609 */
  0x0000,   /* U+060A */
  0x0000,   /* U+060B */
  0x0000,   /* U+060C */
  0x0000,   /* U+060D */
  0x0000,   /* U+060E */
  0x0000,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0000,   /* U+061B */
  0x0000,   /* U+061C */
  0x0000,   /* U+061D */
  0x0000,   /* U+061E */
  0x0000,   /* U+061F */
  0x0000,   /* U+0620 */
  0x0000,   /* U+0621 */
  0x0000,   /* U+0622 */
  0x0000,   /* U+0623 */
  0x0000,   /* U+0624 */
  0x0000,   /* U+0625 */
  0x0000,   /* U+0626 */
  0x0000,   /* U+0627 */
  0x0000,   /* U+0628 */
  0x0000,   /* U+0629 */
  0x0000,   /* U+062A */
  0x0000,   /* U+062B */
  0x0000,   /* U+062C */
  0x0000,   /* U+062D */
  0x0000,   /* U+062E */
  0x0000,   /* U+062F */
  0x0000,   /* U+0630 */
  0x0000,   /* U+0631 */
  0x0000,   /* U+0632 */
  0x0000,   /* U+0633 */
  0x0000,   /* U+0634 */
  0x0000,   /* U+0635 */
  0x0000,   /* U+0636 */
  0x0000,   /* U+0637 */
  0x0000,   /* U+0638 */
  0x0000,   /* U+0639 */
  0x0000,   /* U+063A */
  0x0000,   /* U+063B */
  0x0000,   /* U+063C */
  0x0000,   /* U+063D */
  0x0000,   /* U+063E */
  0x0000,   /* U+063F */
  0x0000,   /* U+0640 */
  0x0000,   /* U+0641 */
  0x0000,   /* U+0642 */
  0x0000,   /* U+0643 */
  0x0000,   /* U+0644 */
  0x0000,   /* U+0645 */
  0x0000,   /* U+0646 */
  0x0000,   /* U+0647 */
  0x0000,   /* U+0648 */
  0x0000,   /* U+0649 */
  0x0000,   /* U+064A */
  0x0000,   /* U+064B */
  0x0000,   /* U+064C */
  0x0000,   /* U+064D */
  0x0000,   /* U+064E */
  0x0000,   /* U+064F */
  0x0000,   /* U+0650 */
  0x0000,   /* U+0651 */
  0x0000,   /* U+0652 */
  0x0000,   /* U+0653 */
  0x0000,   /* U+0654 */
  0x0000,   /* U+0655 */
  0x0000,   /* U+0656 */
  0x0000,   /* U+0657 */
  0x0000,   /* U+0658 */
  0x0000,   /* U+0659 */
  0x0000,   /* U+065A */
  0x0000,   /* U+065B */
  0x0000,   /* U+065C */
  0x0000,   /* U+065D */
  0x0000,   /* U+065E */
  0x0000,   /* U+065F */
  0x0000,   /* U+0660 */
  0x0000,   /* U+0661 */
  0x0000,   /* U+0662 */
  0x0000,   /* U+0663 */
  0x0000,   /* U+0664 */
  0x0000,   /* U+0665 */
  0x0000,   /* U+0666 */
  0x0000,   /* U+0667 */
  0x0000,   /* U+0668 */
  0x0000,   /* U+0669 */
  0x0000,   /* U+066A */
  0x0000,   /* U+066B */
  0x0000,   /* U+066C */
  0x0000,   /* U+066D */
  0x0000,   /* U+066E */
  0x0000,   /* U+066F */
  0x0000,   /* U+0670 */
  0x0000,   /* U+0671 */
  0x0000,   /* U+0672 */
  0x0000,   /* U+0673 */
  0x0000,   /* U+0674 */
  0x0020,   /* U+0675 */
  0x0020,   /* U+0676 */
  0x0020,   /* U+0677 */
  0x0020,   /* U+0678 */
  0x0000,   /* U+0679 */
  0x0000,   /* U+067A */
  0x0000,   /* U+067B */
  0x0000,   /* U+067C */
  0x0000,   /* U+067D */
  0x0000,   /* U+067E */
  0x0000,   /* U+067F */
  0x0000,   /* U+0680 */
  0x0000,   /* U+0681 */
  0x0000,   /* U+0682 */
  0x0000,   /* U+0683 */
  0x0000,   /* U+0684 */
  0x0000,   /* U+0685 */
  0x0000,   /* U+0686 */
  0x0000,   /* U+0687 */
  0x0000,   /* U+0688 */
  0x0000,   /* U+0689 */
  0x0000,   /* U+068A */
  0x0000,   /* U+068B */
  0x0000,   /* U+068C */
  0x0000,   /* U+068D */
  0x0000,   /* U+068E */
  0x0000,   /* U+068F */
  0x0000,   /* U+0690 */
  0x0000,   /* U+0691 */
  0x0000,   /* U+0692 */
  0x0000,   /* U+0693 */
  0x0000,   /* U+0694 */
  0x0000,   /* U+0695 */
  0x0000,   /* U+0696 */
  0x0000,   /* U+0697 */
  0x0000,   /* U+0698 */
  0x0000,   /* U+0699 */
  0x0000,   /* U+069A */
  0x0000,   /* U+069B */
  0x0000,   /* U+069C */
  0x0000,   /* U+069D */
  0x0000,   /* U+069E */
  0x0000,   /* U+069F */
  0x0000,   /* U+06A0 */
  0x0000,   /* U+06A1 */
  0x0000,   /* U+06A2 */
  0x0000,   /* U+06A3 */
  0x0000,   /* U+06A4 */
  0x0000,   /* U+06A5 */
  0x0000,   /* U+06A6 */
  0x0000,   /* U+06A7 */
  0x0000,   /* U+06A8 */
  0x0000,   /* U+06A9 */
  0x0000,   /* U+06AA */
  0x0000,   /* U+06AB */
  0x0000,   /* U+06AC */
  0x0000,   /* U+06AD */
  0x0000,   /* U+06AE */
  0x0000,   /* U+06AF */
  0x0000,   /* U+06B0 */
  0x0000,   /* U+06B1 */
  0x0000,   /* U+06B2 */
  0x0000,   /* U+06B3 */
  0x0000,   /* U+06B4 */
  0x0000,   /* U+06B5 */
  0x0000,   /* U+06B6 */
  0x0000,   /* U+06B7 */
  0x0000,   /* U+06B8 */
  0x0000,   /* U+06B9 */
  0x0000,   /* U+06BA */
  0x0000,   /* U+06BB */
  0x0000,   /* U+06BC */
  0x0000,   /* U+06BD */
  0x0000,   /* U+06BE */
  0x0000,   /* U+06BF */
  0x0083,   /* U+06C0 */
  0x0000,   /* U+06C1 */
  0x0083,   /* U+06C2 */
  0x0000,   /* U+06C3 */
  0x0000,   /* U+06C4 */
  0x0000,   /* U+06C5 */
  0x0000,   /* U+06C6 */
  0x0000,   /* U+06C7 */
  0x0000,   /* U+06C8 */
  0x0000,   /* U+06C9 */
  0x0000,   /* U+06CA */
  0x0000,   /* U+06CB */
  0x0000,   /* U+06CC */
  0x0000,   /* U+06CD */
  0x0000,   /* U+06CE */
  0x0000,   /* U+06CF */
  0x0000,   /* U+06D0 */
  0x0000,   /* U+06D1 */
  0x0000,   /* U+06D2 */
  0x0083,   /* U+06D3 */
  0x0000,   /* U+06D4 */
  0x0000,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x0000,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x0000,   /* U+06E5 */
  0x0000,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0000,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x0000,   /* U+06EE */
  0x0000,   /* U+06EF */
  0x0000,   /* U+06F0 */
  0x0000,   /* U+06F1 */
  0x0000,   /* U+06F2 */
  0x0000,   /* U+06F3 */
  0x0000,   /* U+06F4 */
  0x0000,   /* U+06F5 */
  0x0000,   /* U+06F6 */
  0x0000,   /* U+06F7 */
  0x0000,   /* U+06F8 */
  0x0000,   /* U+06F9 */
  0x0000,   /* U+06FA */
  0x0000,   /* U+06FB */
  0x0000,   /* U+06FC */
  0x0111,   /* U+06FD */
  0x0111,   /* U+06FE */
  0x0000,   /* U+06FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0600 */
  0x0000,   /* U+0601 */
  0x0000,   /* U+0602 */
  0x0000,   /* U+0603 */
  0x0000,   /* U+0604 */
  0x0000,   /* U+0605 */
  0x0000,   /* U+0606 */
  0x0000,   /* U+0607 */
  0x0000,   /* U+0608 */
  0x0000,   /* U+0609 */
  0x0000,   /* U+060A */
  0x0000,   /* U+060B */
  0x0000,   /* U+060C */
  0x0000,   /* U+060D */
  0x0000,   /* U+060E */
  0x0000,   /* U+060F */
  0x0000,   /* U+0610 */
  0x0000,   /* U+0611 */
  0x0000,   /* U+0612 */
  0x0000,   /* U+0613 */
  0x0000,   /* U+0614 */
  0x0000,   /* U+0615 */
  0x0000,   /* U+0616 */
  0x0000,   /* U+0617 */
  0x0000,   /* U+0618 */
  0x0000,   /* U+0619 */
  0x0000,   /* U+061A */
  0x0000,   /* U+061B */
  0x0000,   /* U+061C */
  0x0000,   /* U+061D */
  0x0000,   /* U+061E */
  0x0000,   /* U+061F */
  0x0000,   /* U+0620 */
  0x0000,   /* U+0621 */
  0x0000,   /* U+0622 */
  0x0000,   /* U+0623 */
  0x0000,   /* U+0624 */
  0x0000,   /* U+0625 */
  0x0000,   /* U+0626 */
  0x0000,   /* U+0627 */
  0x0000,   /* U+0628 */
  0x0000,   /* U+0629 */
  0x0000,   /* U+062A */
  0x0000,   /* U+062B */
  0x0000,   /* U+062C */
  0x0000,   /* U+062D */
  0x0000,   /* U+062E */
  0x0000,   /* U+062F */
  0x0000,   /* U+0630 */
  0x0000,   /* U+0631 */
  0x0000,   /* U+0632 */
  0x0000,   /* U+0633 */
  0x0000,   /* U+0634 */
  0x0000,   /* U+0635 */
  0x0000,   /* U+0636 */
  0x0000,   /* U+0637 */
  0x0000,   /* U+0638 */
  0x0000,   /* U+0639 */
  0x0000,   /* U+063A */
  0x0000,   /* U+063B */
  0x0000,   /* U+063C */
  0x0000,   /* U+063D */
  0x0000,   /* U+063E */
  0x0000,   /* U+063F */
  0x0000,   /* U+0640 */
  0x0000,   /* U+0641 */
  0x0000,   /* U+0642 */
  0x0000,   /* U+0643 */
  0x0000,   /* U+0644 */
  0x0000,   /* U+0645 */
  0x0000,   /* U+0646 */
  0x0000,   /* U+0647 */
  0x0000,   /* U+0648 */
  0x0000,   /* U+0649 */
  0x0000,   /* U+064A */
  0x0000,   /* U+064B */
  0x0000,   /* U+064C */
  0x0000,   /* U+064D */
  0x0000,   /* U+064E */
  0x0000,   /* U+064F */
  0x0000,   /* U+0650 */
  0x0000,   /* U+0651 */
  0x0000,   /* U+0652 */
  0x0000,   /* U+0653 */
  0x0000,   /* U+0654 */
  0x0000,   /* U+0655 */
  0x0000,   /* U+0656 */
  0x0000,   /* U+0657 */
  0x0000,   /* U+0658 */
  0x0000,   /* U+0659 */
  0x0000,   /* U+065A */
  0x0000,   /* U+065B */
  0x0000,   /* U+065C */
  0x0000,   /* U+065D */
  0x0000,   /* U+065E */
  0x0000,   /* U+065F */
  0x0000,   /* U+0660 */
  0x0000,   /* U+0661 */
  0x0000,   /* U+0662 */
  0x0000,   /* U+0663 */
  0x0000,   /* U+0664 */
  0x0000,   /* U+0665 */
  0x0000,   /* U+0666 */
  0x0000,   /* U+0667 */
  0x0000,   /* U+0668 */
  0x0000,   /* U+0669 */
  0x0000,   /* U+066A */
  0x0000,   /* U+066B */
  0x0000,   /* U+066C */
  0x0000,   /* U+066D */
  0x0000,   /* U+066E */
  0x0000,   /* U+066F */
  0x0000,   /* U+0670 */
  0x0000,   /* U+0671 */
  0x0000,   /* U+0672 */
  0x0000,   /* U+0673 */
  0x0000,   /* U+0674 */
  0x0004,   /* U+0675 */
  0x0004,   /* U+0676 */
  0x0004,   /* U+0677 */
  0x0004,   /* U+0678 */
  0x0000,   /* U+0679 */
  0x0000,   /* U+067A */
  0x0000,   /* U+067B */
  0x0000,   /* U+067C */
  0x0000,   /* U+067D */
  0x0000,   /* U+067E */
  0x0000,   /* U+067F */
  0x0000,   /* U+0680 */
  0x0000,   /* U+0681 */
  0x0000,   /* U+0682 */
  0x0000,   /* U+0683 */
  0x0000,   /* U+0684 */
  0x0000,   /* U+0685 */
  0x0000,   /* U+0686 */
  0x0000,   /* U+0687 */
  0x0000,   /* U+0688 */
  0x0000,   /* U+0689 */
  0x0000,   /* U+068A */
  0x0000,   /* U+068B */
  0x0000,   /* U+068C */
  0x0000,   /* U+068D */
  0x0000,   /* U+068E */
  0x0000,   /* U+068F */
  0x0000,   /* U+0690 */
  0x0000,   /* U+0691 */
  0x0000,   /* U+0692 */
  0x0000,   /* U+0693 */
  0x0000,   /* U+0694 */
  0x0000,   /* U+0695 */
  0x0000,   /* U+0696 */
  0x0000,   /* U+0697 */
  0x0000,   /* U+0698 */
  0x0000,   /* U+0699 */
  0x0000,   /* U+069A */
  0x0000,   /* U+069B */
  0x0000,   /* U+069C */
  0x0000,   /* U+069D */
  0x0000,   /* U+069E */
  0x0000,   /* U+069F */
  0x0000,   /* U+06A0 */
  0x0000,   /* U+06A1 */
  0x0000,   /* U+06A2 */
  0x0000,   /* U+06A3 */
  0x0000,   /* U+06A4 */
  0x0000,   /* U+06A5 */
  0x0000,   /* U+06A6 */
  0x0000,   /* U+06A7 */
  0x0000,   /* U+06A8 */
  0x0000,   /* U+06A9 */
  0x0000,   /* U+06AA */
  0x0000,   /* U+06AB */
  0x0000,   /* U+06AC */
  0x0000,   /* U+06AD */
  0x0000,   /* U+06AE */
  0x0000,   /* U+06AF */
  0x0000,   /* U+06B0 */
  0x0000,   /* U+06B1 */
  0x0000,   /* U+06B2 */
  0x0000,   /* U+06B3 */
  0x0000,   /* U+06B4 */
  0x0000,   /* U+06B5 */
  0x0000,   /* U+06B6 */
  0x0000,   /* U+06B7 */
  0x0000,   /* U+06B8 */
  0x0000,   /* U+06B9 */
  0x0000,   /* U+06BA */
  0x0000,   /* U+06BB */
  0x0000,   /* U+06BC */
  0x0000,   /* U+06BD */
  0x0000,   /* U+06BE */
  0x0000,   /* U+06BF */
  0x0002,   /* U+06C0 */
  0x0000,   /* U+06C1 */
  0x0002,   /* U+06C2 */
  0x0000,   /* U+06C3 */
  0x0000,   /* U+06C4 */
  0x0000,   /* U+06C5 */
  0x0000,   /* U+06C6 */
  0x0000,   /* U+06C7 */
  0x0000,   /* U+06C8 */
  0x0000,   /* U+06C9 */
  0x0000,   /* U+06CA */
  0x0000,   /* U+06CB */
  0x0000,   /* U+06CC */
  0x0000,   /* U+06CD */
  0x0000,   /* U+06CE */
  0x0000,   /* U+06CF */
  0x0000,   /* U+06D0 */
  0x0000,   /* U+06D1 */
  0x0000,   /* U+06D2 */
  0x0002,   /* U+06D3 */
  0x0000,   /* U+06D4 */
  0x0000,   /* U+06D5 */
  0x0000,   /* U+06D6 */
  0x0000,   /* U+06D7 */
  0x0000,   /* U+06D8 */
  0x0000,   /* U+06D9 */
  0x0000,   /* U+06DA */
  0x0000,   /* U+06DB */
  0x0000,   /* U+06DC */
  0x0000,   /* U+06DD */
  0x0000,   /* U+06DE */
  0x0000,   /* U+06DF */
  0x0000,   /* U+06E0 */
  0x0000,   /* U+06E1 */
  0x0000,   /* U+06E2 */
  0x0000,   /* U+06E3 */
  0x0000,   /* U+06E4 */
  0x0000,   /* U+06E5 */
  0x0000,   /* U+06E6 */
  0x0000,   /* U+06E7 */
  0x0000,   /* U+06E8 */
  0x0000,   /* U+06E9 */
  0x0000,   /* U+06EA */
  0x0000,   /* U+06EB */
  0x0000,   /* U+06EC */
  0x0000,   /* U+06ED */
  0x0000,   /* U+06EE */
  0x0000,   /* U+06EF */
  0x0000,   /* U+06F0 */
  0x0000,   /* U+06F1 */
  0x0000,   /* U+06F2 */
  0x0000,   /* U+06F3 */
  0x0000,   /* U+06F4 */
  0x0000,   /* U+06F5 */
  0x0000,   /* U+06F6 */
  0x0000,   /* U+06F7 */
  0x0000,   /* U+06F8 */
  0x0000,   /* U+06F9 */
  0x0000,   /* U+06FA */
  0x0000,   /* U+06FB */
  0x0000,   /* U+06FC */
  0x0004,   /* U+06FD */
  0x0004,   /* U+06FE */
  0x0000,   /* U+06FF */
};

uint16 uca900_p007[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x02C4,   /* U+0700 */
  0x027A,   /* U+0701 */
  0x027B,   /* U+0702 */
  0x023C,   /* U+0703 */
  0x023D,   /* U+0704 */
  0x023E,   /* U+0705 */
  0x023F,   /* U+0706 */
  0x0240,   /* U+0707 */
  0x0241,   /* U+0708 */
  0x026B,   /* U+0709 */
  0x03D4,   /* U+070A */
  0x03D5,   /* U+070B */
  0x03D6,   /* U+070C */
  0x03D7,   /* U+070D */
  0xFBC0,   /* U+070E */
  0x0000,   /* U+070F */
  0x23D7,   /* U+0710 */
  0x0000,   /* U+0711 */
  0x23D8,   /* U+0712 */
  0x23D9,   /* U+0713 */
  0x23D9,   /* U+0714 */
  0x23DB,   /* U+0715 */
  0x23DA,   /* U+0716 */
  0x23DC,   /* U+0717 */
  0x23DD,   /* U+0718 */
  0x23DE,   /* U+0719 */
  0x23E0,   /* U+071A */
  0x23E1,   /* U+071B */
  0x23E1,   /* U+071C */
  0x23E2,   /* U+071D */
  0x23E3,   /* U+071E */
  0x23E4,   /* U+071F */
  0x23E6,   /* U+0720 */
  0x23E7,   /* U+0721 */
  0x23E8,   /* U+0722 */
  0x23E9,   /* U+0723 */
  0x23E9,   /* U+0724 */
  0x23EA,   /* U+0725 */
  0x23EB,   /* U+0726 */
  0x23EB,   /* U+0727 */
  0x23ED,   /* U+0728 */
  0x23EE,   /* U+0729 */
  0x23EF,   /* U+072A */
  0x23F0,   /* U+072B */
  0x23F1,   /* U+072C */
  0x23D8,   /* U+072D */
  0x23D9,   /* U+072E */
  0x23DB,   /* U+072F */
  0x0000,   /* U+0730 */
  0x0000,   /* U+0731 */
  0x0000,   /* U+0732 */
  0x0000,   /* U+0733 */
  0x0000,   /* U+0734 */
  0x0000,   /* U+0735 */
  0x0000,   /* U+0736 */
  0x0000,   /* U+0737 */
  0x0000,   /* U+0738 */
  0x0000,   /* U+0739 */
  0x0000,   /* U+073A */
  0x0000,   /* U+073B */
  0x0000,   /* U+073C */
  0x0000,   /* U+073D */
  0x0000,   /* U+073E */
  0x0000,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0000,   /* U+0741 */
  0x0000,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0000,   /* U+0745 */
  0x0000,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0xFBC0,   /* U+074B */
  0xFBC0,   /* U+074C */
  0x23DF,   /* U+074D */
  0x23E5,   /* U+074E */
  0x23EC,   /* U+074F */
  0x2311,   /* U+0750 */
  0x2312,   /* U+0751 */
  0x2313,   /* U+0752 */
  0x2314,   /* U+0753 */
  0x2315,   /* U+0754 */
  0x2316,   /* U+0755 */
  0x2318,   /* U+0756 */
  0x2331,   /* U+0757 */
  0x2332,   /* U+0758 */
  0x2344,   /* U+0759 */
  0x2345,   /* U+075A */
  0x2352,   /* U+075B */
  0x235F,   /* U+075C */
  0x2372,   /* U+075D */
  0x2373,   /* U+075E */
  0x2374,   /* U+075F */
  0x237F,   /* U+0760 */
  0x2380,   /* U+0761 */
  0x2397,   /* U+0762 */
  0x239A,   /* U+0763 */
  0x239B,   /* U+0764 */
  0x23A4,   /* U+0765 */
  0x23A5,   /* U+0766 */
  0x23AE,   /* U+0767 */
  0x23AF,   /* U+0768 */
  0x23B0,   /* U+0769 */
  0x23A1,   /* U+076A */
  0x2353,   /* U+076B */
  0x2354,   /* U+076C */
  0x2360,   /* U+076D */
  0x2333,   /* U+076E */
  0x2334,   /* U+076F */
  0x2361,   /* U+0770 */
  0x2355,   /* U+0771 */
  0x2335,   /* U+0772 */
  0x2305,   /* U+0773 */
  0x2306,   /* U+0774 */
  0x23D0,   /* U+0775 */
  0x23D1,   /* U+0776 */
  0x23D2,   /* U+0777 */
  0x23C2,   /* U+0778 */
  0x23C3,   /* U+0779 */
  0x23D5,   /* U+077A */
  0x23D6,   /* U+077B */
  0x2336,   /* U+077C */
  0x2362,   /* U+077D */
  0x2363,   /* U+077E */
  0x238C,   /* U+077F */
  0x240B,   /* U+0780 */
  0x240E,   /* U+0781 */
  0x240F,   /* U+0782 */
  0x2410,   /* U+0783 */
  0x2412,   /* U+0784 */
  0x2413,   /* U+0785 */
  0x2414,   /* U+0786 */
  0x2415,   /* U+0787 */
  0x2418,   /* U+0788 */
  0x241A,   /* U+0789 */
  0x241B,   /* U+078A */
  0x241C,   /* U+078B */
  0x241E,   /* U+078C */
  0x2422,   /* U+078D */
  0x2423,   /* U+078E */
  0x2425,   /* U+078F */
  0x2426,   /* U+0790 */
  0x242A,   /* U+0791 */
  0x242B,   /* U+0792 */
  0x242C,   /* U+0793 */
  0x242D,   /* U+0794 */
  0x242E,   /* U+0795 */
  0x242F,   /* U+0796 */
  0x2430,   /* U+0797 */
  0x241F,   /* U+0798 */
  0x240C,   /* U+0799 */
  0x240D,   /* U+079A */
  0x241D,   /* U+079B */
  0x2411,   /* U+079C */
  0x2427,   /* U+079D */
  0x2428,   /* U+079E */
  0x2429,   /* U+079F */
  0x2420,   /* U+07A0 */
  0x2421,   /* U+07A1 */
  0x2416,   /* U+07A2 */
  0x2417,   /* U+07A3 */
  0x2424,   /* U+07A4 */
  0x2419,   /* U+07A5 */
  0x2432,   /* U+07A6 */
  0x2433,   /* U+07A7 */
  0x2434,   /* U+07A8 */
  0x2435,   /* U+07A9 */
  0x2436,   /* U+07AA */
  0x2437,   /* U+07AB */
  0x2438,   /* U+07AC */
  0x2439,   /* U+07AD */
  0x243A,   /* U+07AE */
  0x243B,   /* U+07AF */
  0x243C,   /* U+07B0 */
  0x2431,   /* U+07B1 */
  0xFBC0,   /* U+07B2 */
  0xFBC0,   /* U+07B3 */
  0xFBC0,   /* U+07B4 */
  0xFBC0,   /* U+07B5 */
  0xFBC0,   /* U+07B6 */
  0xFBC0,   /* U+07B7 */
  0xFBC0,   /* U+07B8 */
  0xFBC0,   /* U+07B9 */
  0xFBC0,   /* U+07BA */
  0xFBC0,   /* U+07BB */
  0xFBC0,   /* U+07BC */
  0xFBC0,   /* U+07BD */
  0xFBC0,   /* U+07BE */
  0xFBC0,   /* U+07BF */
  0x1C3D,   /* U+07C0 */
  0x1C3E,   /* U+07C1 */
  0x1C3F,   /* U+07C2 */
  0x1C40,   /* U+07C3 */
  0x1C41,   /* U+07C4 */
  0x1C42,   /* U+07C5 */
  0x1C43,   /* U+07C6 */
  0x1C44,   /* U+07C7 */
  0x1C45,   /* U+07C8 */
  0x1C46,   /* U+07C9 */
  0x243D,   /* U+07CA */
  0x243E,   /* U+07CB */
  0x243F,   /* U+07CC */
  0x2440,   /* U+07CD */
  0x2441,   /* U+07CE */
  0x2442,   /* U+07CF */
  0x2443,   /* U+07D0 */
  0x2444,   /* U+07D1 */
  0x2445,   /* U+07D2 */
  0x2446,   /* U+07D3 */
  0x2447,   /* U+07D4 */
  0x2448,   /* U+07D5 */
  0x2449,   /* U+07D6 */
  0x244A,   /* U+07D7 */
  0x244B,   /* U+07D8 */
  0x244C,   /* U+07D9 */
  0x244D,   /* U+07DA */
  0x244E,   /* U+07DB */
  0x244F,   /* U+07DC */
  0x2450,   /* U+07DD */
  0x2451,   /* U+07DE */
  0x2452,   /* U+07DF */
  0x2453,   /* U+07E0 */
  0x2454,   /* U+07E1 */
  0x2455,   /* U+07E2 */
  0x2456,   /* U+07E3 */
  0x2457,   /* U+07E4 */
  0x2458,   /* U+07E5 */
  0x2459,   /* U+07E6 */
  0x245A,   /* U+07E7 */
  0x2449,   /* U+07E8 */
  0x244A,   /* U+07E9 */
  0x244C,   /* U+07EA */
  0x0000,   /* U+07EB */
  0x0000,   /* U+07EC */
  0x0000,   /* U+07ED */
  0x0000,   /* U+07EE */
  0x0000,   /* U+07EF */
  0x0000,   /* U+07F0 */
  0x0000,   /* U+07F1 */
  0x0000,   /* U+07F2 */
  0x0000,   /* U+07F3 */
  0x245B,   /* U+07F4 */
  0x245C,   /* U+07F5 */
  0x0512,   /* U+07F6 */
  0x02C5,   /* U+07F7 */
  0x022B,   /* U+07F8 */
  0x0263,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0xFBC0,   /* U+07FB */
  0xFBC0,   /* U+07FC */
  0xFBC0,   /* U+07FD */
  0xFBC0,   /* U+07FE */
  0xFBC0,   /* U+07FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0700 */
  0x0020,   /* U+0701 */
  0x0020,   /* U+0702 */
  0x0020,   /* U+0703 */
  0x0020,   /* U+0704 */
  0x0020,   /* U+0705 */
  0x0020,   /* U+0706 */
  0x0020,   /* U+0707 */
  0x0020,   /* U+0708 */
  0x0020,   /* U+0709 */
  0x0020,   /* U+070A */
  0x0020,   /* U+070B */
  0x0020,   /* U+070C */
  0x0020,   /* U+070D */
  0x0020,   /* U+070E */
  0x0000,   /* U+070F */
  0x0020,   /* U+0710 */
  0x0099,   /* U+0711 */
  0x0020,   /* U+0712 */
  0x0020,   /* U+0713 */
  0x0020,   /* U+0714 */
  0x0020,   /* U+0715 */
  0x0020,   /* U+0716 */
  0x0020,   /* U+0717 */
  0x0020,   /* U+0718 */
  0x0020,   /* U+0719 */
  0x0020,   /* U+071A */
  0x0020,   /* U+071B */
  0x0020,   /* U+071C */
  0x0020,   /* U+071D */
  0x0020,   /* U+071E */
  0x0020,   /* U+071F */
  0x0020,   /* U+0720 */
  0x0020,   /* U+0721 */
  0x0020,   /* U+0722 */
  0x0020,   /* U+0723 */
  0x0020,   /* U+0724 */
  0x0020,   /* U+0725 */
  0x0020,   /* U+0726 */
  0x0020,   /* U+0727 */
  0x0020,   /* U+0728 */
  0x0020,   /* U+0729 */
  0x0020,   /* U+072A */
  0x0020,   /* U+072B */
  0x0020,   /* U+072C */
  0x0020,   /* U+072D */
  0x0020,   /* U+072E */
  0x0020,   /* U+072F */
  0x009A,   /* U+0730 */
  0x009B,   /* U+0731 */
  0x009C,   /* U+0732 */
  0x009D,   /* U+0733 */
  0x009E,   /* U+0734 */
  0x009F,   /* U+0735 */
  0x00A0,   /* U+0736 */
  0x00A1,   /* U+0737 */
  0x00A2,   /* U+0738 */
  0x00A3,   /* U+0739 */
  0x00A4,   /* U+073A */
  0x00A5,   /* U+073B */
  0x00A6,   /* U+073C */
  0x00A7,   /* U+073D */
  0x00A8,   /* U+073E */
  0x00A9,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0033,   /* U+0741 */
  0x0034,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0033,   /* U+0745 */
  0x0034,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0x0020,   /* U+074B */
  0x0020,   /* U+074C */
  0x0020,   /* U+074D */
  0x0020,   /* U+074E */
  0x0020,   /* U+074F */
  0x0020,   /* U+0750 */
  0x0020,   /* U+0751 */
  0x0020,   /* U+0752 */
  0x0020,   /* U+0753 */
  0x0020,   /* U+0754 */
  0x0020,   /* U+0755 */
  0x0020,   /* U+0756 */
  0x0020,   /* U+0757 */
  0x0020,   /* U+0758 */
  0x0020,   /* U+0759 */
  0x0020,   /* U+075A */
  0x0020,   /* U+075B */
  0x0020,   /* U+075C */
  0x0020,   /* U+075D */
  0x0020,   /* U+075E */
  0x0020,   /* U+075F */
  0x0020,   /* U+0760 */
  0x0020,   /* U+0761 */
  0x0020,   /* U+0762 */
  0x0020,   /* U+0763 */
  0x0020,   /* U+0764 */
  0x0020,   /* U+0765 */
  0x0020,   /* U+0766 */
  0x0020,   /* U+0767 */
  0x0020,   /* U+0768 */
  0x0020,   /* U+0769 */
  0x0020,   /* U+076A */
  0x0020,   /* U+076B */
  0x0020,   /* U+076C */
  0x0020,   /* U+076D */
  0x0020,   /* U+076E */
  0x0020,   /* U+076F */
  0x0020,   /* U+0770 */
  0x0020,   /* U+0771 */
  0x0020,   /* U+0772 */
  0x0020,   /* U+0773 */
  0x0020,   /* U+0774 */
  0x0020,   /* U+0775 */
  0x0020,   /* U+0776 */
  0x0020,   /* U+0777 */
  0x0020,   /* U+0778 */
  0x0020,   /* U+0779 */
  0x0020,   /* U+077A */
  0x0020,   /* U+077B */
  0x0020,   /* U+077C */
  0x0020,   /* U+077D */
  0x0020,   /* U+077E */
  0x0020,   /* U+077F */
  0x0020,   /* U+0780 */
  0x0020,   /* U+0781 */
  0x0020,   /* U+0782 */
  0x0020,   /* U+0783 */
  0x0020,   /* U+0784 */
  0x0020,   /* U+0785 */
  0x0020,   /* U+0786 */
  0x0020,   /* U+0787 */
  0x0020,   /* U+0788 */
  0x0020,   /* U+0789 */
  0x0020,   /* U+078A */
  0x0020,   /* U+078B */
  0x0020,   /* U+078C */
  0x0020,   /* U+078D */
  0x0020,   /* U+078E */
  0x0020,   /* U+078F */
  0x0020,   /* U+0790 */
  0x0020,   /* U+0791 */
  0x0020,   /* U+0792 */
  0x0020,   /* U+0793 */
  0x0020,   /* U+0794 */
  0x0020,   /* U+0795 */
  0x0020,   /* U+0796 */
  0x0020,   /* U+0797 */
  0x0020,   /* U+0798 */
  0x0020,   /* U+0799 */
  0x0020,   /* U+079A */
  0x0020,   /* U+079B */
  0x0020,   /* U+079C */
  0x0020,   /* U+079D */
  0x0020,   /* U+079E */
  0x0020,   /* U+079F */
  0x0020,   /* U+07A0 */
  0x0020,   /* U+07A1 */
  0x0020,   /* U+07A2 */
  0x0020,   /* U+07A3 */
  0x0020,   /* U+07A4 */
  0x0020,   /* U+07A5 */
  0x0020,   /* U+07A6 */
  0x0020,   /* U+07A7 */
  0x0020,   /* U+07A8 */
  0x0020,   /* U+07A9 */
  0x0020,   /* U+07AA */
  0x0020,   /* U+07AB */
  0x0020,   /* U+07AC */
  0x0020,   /* U+07AD */
  0x0020,   /* U+07AE */
  0x0020,   /* U+07AF */
  0x0020,   /* U+07B0 */
  0x0020,   /* U+07B1 */
  0x0020,   /* U+07B2 */
  0x0020,   /* U+07B3 */
  0x0020,   /* U+07B4 */
  0x0020,   /* U+07B5 */
  0x0020,   /* U+07B6 */
  0x0020,   /* U+07B7 */
  0x0020,   /* U+07B8 */
  0x0020,   /* U+07B9 */
  0x0020,   /* U+07BA */
  0x0020,   /* U+07BB */
  0x0020,   /* U+07BC */
  0x0020,   /* U+07BD */
  0x0020,   /* U+07BE */
  0x0020,   /* U+07BF */
  0x0020,   /* U+07C0 */
  0x0020,   /* U+07C1 */
  0x0020,   /* U+07C2 */
  0x0020,   /* U+07C3 */
  0x0020,   /* U+07C4 */
  0x0020,   /* U+07C5 */
  0x0020,   /* U+07C6 */
  0x0020,   /* U+07C7 */
  0x0020,   /* U+07C8 */
  0x0020,   /* U+07C9 */
  0x0020,   /* U+07CA */
  0x0020,   /* U+07CB */
  0x0020,   /* U+07CC */
  0x0020,   /* U+07CD */
  0x0020,   /* U+07CE */
  0x0020,   /* U+07CF */
  0x0020,   /* U+07D0 */
  0x0020,   /* U+07D1 */
  0x0020,   /* U+07D2 */
  0x0020,   /* U+07D3 */
  0x0020,   /* U+07D4 */
  0x0020,   /* U+07D5 */
  0x0020,   /* U+07D6 */
  0x0020,   /* U+07D7 */
  0x0020,   /* U+07D8 */
  0x0020,   /* U+07D9 */
  0x0020,   /* U+07DA */
  0x0020,   /* U+07DB */
  0x0020,   /* U+07DC */
  0x0020,   /* U+07DD */
  0x0020,   /* U+07DE */
  0x0020,   /* U+07DF */
  0x0020,   /* U+07E0 */
  0x0020,   /* U+07E1 */
  0x0020,   /* U+07E2 */
  0x0020,   /* U+07E3 */
  0x0020,   /* U+07E4 */
  0x0020,   /* U+07E5 */
  0x0020,   /* U+07E6 */
  0x0020,   /* U+07E7 */
  0x0020,   /* U+07E8 */
  0x0020,   /* U+07E9 */
  0x0020,   /* U+07EA */
  0x00AA,   /* U+07EB */
  0x00AB,   /* U+07EC */
  0x00AC,   /* U+07ED */
  0x00AD,   /* U+07EE */
  0x00AE,   /* U+07EF */
  0x00AF,   /* U+07F0 */
  0x00B0,   /* U+07F1 */
  0x00B1,   /* U+07F2 */
  0x00B2,   /* U+07F3 */
  0x0020,   /* U+07F4 */
  0x0020,   /* U+07F5 */
  0x0020,   /* U+07F6 */
  0x0020,   /* U+07F7 */
  0x0020,   /* U+07F8 */
  0x0020,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0x0020,   /* U+07FB */
  0x0020,   /* U+07FC */
  0x0020,   /* U+07FD */
  0x0020,   /* U+07FE */
  0x0020,   /* U+07FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0700 */
  0x0002,   /* U+0701 */
  0x0002,   /* U+0702 */
  0x0002,   /* U+0703 */
  0x0002,   /* U+0704 */
  0x0002,   /* U+0705 */
  0x0002,   /* U+0706 */
  0x0002,   /* U+0707 */
  0x0002,   /* U+0708 */
  0x0002,   /* U+0709 */
  0x0002,   /* U+070A */
  0x0002,   /* U+070B */
  0x0002,   /* U+070C */
  0x0002,   /* U+070D */
  0x0002,   /* U+070E */
  0x0000,   /* U+070F */
  0x0002,   /* U+0710 */
  0x0002,   /* U+0711 */
  0x0002,   /* U+0712 */
  0x0002,   /* U+0713 */
  0x0004,   /* U+0714 */
  0x0002,   /* U+0715 */
  0x0002,   /* U+0716 */
  0x0002,   /* U+0717 */
  0x0002,   /* U+0718 */
  0x0002,   /* U+0719 */
  0x0002,   /* U+071A */
  0x0002,   /* U+071B */
  0x0004,   /* U+071C */
  0x0002,   /* U+071D */
  0x0002,   /* U+071E */
  0x0002,   /* U+071F */
  0x0002,   /* U+0720 */
  0x0002,   /* U+0721 */
  0x0002,   /* U+0722 */
  0x0002,   /* U+0723 */
  0x0019,   /* U+0724 */
  0x0002,   /* U+0725 */
  0x0002,   /* U+0726 */
  0x0004,   /* U+0727 */
  0x0002,   /* U+0728 */
  0x0002,   /* U+0729 */
  0x0002,   /* U+072A */
  0x0002,   /* U+072B */
  0x0002,   /* U+072C */
  0x0004,   /* U+072D */
  0x0004,   /* U+072E */
  0x0004,   /* U+072F */
  0x0002,   /* U+0730 */
  0x0002,   /* U+0731 */
  0x0002,   /* U+0732 */
  0x0002,   /* U+0733 */
  0x0002,   /* U+0734 */
  0x0002,   /* U+0735 */
  0x0002,   /* U+0736 */
  0x0002,   /* U+0737 */
  0x0002,   /* U+0738 */
  0x0002,   /* U+0739 */
  0x0002,   /* U+073A */
  0x0002,   /* U+073B */
  0x0002,   /* U+073C */
  0x0002,   /* U+073D */
  0x0002,   /* U+073E */
  0x0002,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0002,   /* U+0741 */
  0x0002,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0002,   /* U+0745 */
  0x0002,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0x0002,   /* U+074B */
  0x0002,   /* U+074C */
  0x0002,   /* U+074D */
  0x0002,   /* U+074E */
  0x0002,   /* U+074F */
  0x0002,   /* U+0750 */
  0x0002,   /* U+0751 */
  0x0002,   /* U+0752 */
  0x0002,   /* U+0753 */
  0x0002,   /* U+0754 */
  0x0002,   /* U+0755 */
  0x0002,   /* U+0756 */
  0x0002,   /* U+0757 */
  0x0002,   /* U+0758 */
  0x0002,   /* U+0759 */
  0x0002,   /* U+075A */
  0x0002,   /* U+075B */
  0x0002,   /* U+075C */
  0x0002,   /* U+075D */
  0x0002,   /* U+075E */
  0x0002,   /* U+075F */
  0x0002,   /* U+0760 */
  0x0002,   /* U+0761 */
  0x0002,   /* U+0762 */
  0x0002,   /* U+0763 */
  0x0002,   /* U+0764 */
  0x0002,   /* U+0765 */
  0x0002,   /* U+0766 */
  0x0002,   /* U+0767 */
  0x0002,   /* U+0768 */
  0x0002,   /* U+0769 */
  0x0002,   /* U+076A */
  0x0002,   /* U+076B */
  0x0002,   /* U+076C */
  0x0002,   /* U+076D */
  0x0002,   /* U+076E */
  0x0002,   /* U+076F */
  0x0002,   /* U+0770 */
  0x0002,   /* U+0771 */
  0x0002,   /* U+0772 */
  0x0002,   /* U+0773 */
  0x0002,   /* U+0774 */
  0x0002,   /* U+0775 */
  0x0002,   /* U+0776 */
  0x0002,   /* U+0777 */
  0x0002,   /* U+0778 */
  0x0002,   /* U+0779 */
  0x0002,   /* U+077A */
  0x0002,   /* U+077B */
  0x0002,   /* U+077C */
  0x0002,   /* U+077D */
  0x0002,   /* U+077E */
  0x0002,   /* U+077F */
  0x0002,   /* U+0780 */
  0x0002,   /* U+0781 */
  0x0002,   /* U+0782 */
  0x0002,   /* U+0783 */
  0x0002,   /* U+0784 */
  0x0002,   /* U+0785 */
  0x0002,   /* U+0786 */
  0x0002,   /* U+0787 */
  0x0002,   /* U+0788 */
  0x0002,   /* U+0789 */
  0x0002,   /* U+078A */
  0x0002,   /* U+078B */
  0x0002,   /* U+078C */
  0x0002,   /* U+078D */
  0x0002,   /* U+078E */
  0x0002,   /* U+078F */
  0x0002,   /* U+0790 */
  0x0002,   /* U+0791 */
  0x0002,   /* U+0792 */
  0x0002,   /* U+0793 */
  0x0002,   /* U+0794 */
  0x0002,   /* U+0795 */
  0x0002,   /* U+0796 */
  0x0002,   /* U+0797 */
  0x0002,   /* U+0798 */
  0x0002,   /* U+0799 */
  0x0002,   /* U+079A */
  0x0002,   /* U+079B */
  0x0002,   /* U+079C */
  0x0002,   /* U+079D */
  0x0002,   /* U+079E */
  0x0002,   /* U+079F */
  0x0002,   /* U+07A0 */
  0x0002,   /* U+07A1 */
  0x0002,   /* U+07A2 */
  0x0002,   /* U+07A3 */
  0x0002,   /* U+07A4 */
  0x0002,   /* U+07A5 */
  0x0002,   /* U+07A6 */
  0x0002,   /* U+07A7 */
  0x0002,   /* U+07A8 */
  0x0002,   /* U+07A9 */
  0x0002,   /* U+07AA */
  0x0002,   /* U+07AB */
  0x0002,   /* U+07AC */
  0x0002,   /* U+07AD */
  0x0002,   /* U+07AE */
  0x0002,   /* U+07AF */
  0x0002,   /* U+07B0 */
  0x0002,   /* U+07B1 */
  0x0002,   /* U+07B2 */
  0x0002,   /* U+07B3 */
  0x0002,   /* U+07B4 */
  0x0002,   /* U+07B5 */
  0x0002,   /* U+07B6 */
  0x0002,   /* U+07B7 */
  0x0002,   /* U+07B8 */
  0x0002,   /* U+07B9 */
  0x0002,   /* U+07BA */
  0x0002,   /* U+07BB */
  0x0002,   /* U+07BC */
  0x0002,   /* U+07BD */
  0x0002,   /* U+07BE */
  0x0002,   /* U+07BF */
  0x0002,   /* U+07C0 */
  0x0002,   /* U+07C1 */
  0x0002,   /* U+07C2 */
  0x0002,   /* U+07C3 */
  0x0002,   /* U+07C4 */
  0x0002,   /* U+07C5 */
  0x0002,   /* U+07C6 */
  0x0002,   /* U+07C7 */
  0x0002,   /* U+07C8 */
  0x0002,   /* U+07C9 */
  0x0002,   /* U+07CA */
  0x0002,   /* U+07CB */
  0x0002,   /* U+07CC */
  0x0002,   /* U+07CD */
  0x0002,   /* U+07CE */
  0x0002,   /* U+07CF */
  0x0002,   /* U+07D0 */
  0x0002,   /* U+07D1 */
  0x0002,   /* U+07D2 */
  0x0002,   /* U+07D3 */
  0x0002,   /* U+07D4 */
  0x0002,   /* U+07D5 */
  0x0002,   /* U+07D6 */
  0x0002,   /* U+07D7 */
  0x0002,   /* U+07D8 */
  0x0002,   /* U+07D9 */
  0x0002,   /* U+07DA */
  0x0002,   /* U+07DB */
  0x0002,   /* U+07DC */
  0x0002,   /* U+07DD */
  0x0002,   /* U+07DE */
  0x0002,   /* U+07DF */
  0x0002,   /* U+07E0 */
  0x0002,   /* U+07E1 */
  0x0002,   /* U+07E2 */
  0x0002,   /* U+07E3 */
  0x0002,   /* U+07E4 */
  0x0002,   /* U+07E5 */
  0x0002,   /* U+07E6 */
  0x0002,   /* U+07E7 */
  0x0004,   /* U+07E8 */
  0x0004,   /* U+07E9 */
  0x0004,   /* U+07EA */
  0x0002,   /* U+07EB */
  0x0002,   /* U+07EC */
  0x0002,   /* U+07ED */
  0x0002,   /* U+07EE */
  0x0002,   /* U+07EF */
  0x0002,   /* U+07F0 */
  0x0002,   /* U+07F1 */
  0x0002,   /* U+07F2 */
  0x0002,   /* U+07F3 */
  0x0002,   /* U+07F4 */
  0x0002,   /* U+07F5 */
  0x0002,   /* U+07F6 */
  0x0002,   /* U+07F7 */
  0x0002,   /* U+07F8 */
  0x0002,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0x0002,   /* U+07FB */
  0x0002,   /* U+07FC */
  0x0002,   /* U+07FD */
  0x0002,   /* U+07FE */
  0x0002,   /* U+07FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0700 */
  0x0000,   /* U+0701 */
  0x0000,   /* U+0702 */
  0x0000,   /* U+0703 */
  0x0000,   /* U+0704 */
  0x0000,   /* U+0705 */
  0x0000,   /* U+0706 */
  0x0000,   /* U+0707 */
  0x0000,   /* U+0708 */
  0x0000,   /* U+0709 */
  0x0000,   /* U+070A */
  0x0000,   /* U+070B */
  0x0000,   /* U+070C */
  0x0000,   /* U+070D */
  0x870E,   /* U+070E */
  0x0000,   /* U+070F */
  0x0000,   /* U+0710 */
  0x0000,   /* U+0711 */
  0x0000,   /* U+0712 */
  0x0000,   /* U+0713 */
  0x0000,   /* U+0714 */
  0x0000,   /* U+0715 */
  0x0000,   /* U+0716 */
  0x0000,   /* U+0717 */
  0x0000,   /* U+0718 */
  0x0000,   /* U+0719 */
  0x0000,   /* U+071A */
  0x0000,   /* U+071B */
  0x0000,   /* U+071C */
  0x0000,   /* U+071D */
  0x0000,   /* U+071E */
  0x0000,   /* U+071F */
  0x0000,   /* U+0720 */
  0x0000,   /* U+0721 */
  0x0000,   /* U+0722 */
  0x0000,   /* U+0723 */
  0x0000,   /* U+0724 */
  0x0000,   /* U+0725 */
  0x0000,   /* U+0726 */
  0x0000,   /* U+0727 */
  0x0000,   /* U+0728 */
  0x0000,   /* U+0729 */
  0x0000,   /* U+072A */
  0x0000,   /* U+072B */
  0x0000,   /* U+072C */
  0x0000,   /* U+072D */
  0x0000,   /* U+072E */
  0x0000,   /* U+072F */
  0x0000,   /* U+0730 */
  0x0000,   /* U+0731 */
  0x0000,   /* U+0732 */
  0x0000,   /* U+0733 */
  0x0000,   /* U+0734 */
  0x0000,   /* U+0735 */
  0x0000,   /* U+0736 */
  0x0000,   /* U+0737 */
  0x0000,   /* U+0738 */
  0x0000,   /* U+0739 */
  0x0000,   /* U+073A */
  0x0000,   /* U+073B */
  0x0000,   /* U+073C */
  0x0000,   /* U+073D */
  0x0000,   /* U+073E */
  0x0000,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0000,   /* U+0741 */
  0x0000,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0000,   /* U+0745 */
  0x0000,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0x874B,   /* U+074B */
  0x874C,   /* U+074C */
  0x0000,   /* U+074D */
  0x0000,   /* U+074E */
  0x0000,   /* U+074F */
  0x0000,   /* U+0750 */
  0x0000,   /* U+0751 */
  0x0000,   /* U+0752 */
  0x0000,   /* U+0753 */
  0x0000,   /* U+0754 */
  0x0000,   /* U+0755 */
  0x0000,   /* U+0756 */
  0x0000,   /* U+0757 */
  0x0000,   /* U+0758 */
  0x0000,   /* U+0759 */
  0x0000,   /* U+075A */
  0x0000,   /* U+075B */
  0x0000,   /* U+075C */
  0x0000,   /* U+075D */
  0x0000,   /* U+075E */
  0x0000,   /* U+075F */
  0x0000,   /* U+0760 */
  0x0000,   /* U+0761 */
  0x0000,   /* U+0762 */
  0x0000,   /* U+0763 */
  0x0000,   /* U+0764 */
  0x0000,   /* U+0765 */
  0x0000,   /* U+0766 */
  0x0000,   /* U+0767 */
  0x0000,   /* U+0768 */
  0x0000,   /* U+0769 */
  0x0000,   /* U+076A */
  0x0000,   /* U+076B */
  0x0000,   /* U+076C */
  0x0000,   /* U+076D */
  0x0000,   /* U+076E */
  0x0000,   /* U+076F */
  0x0000,   /* U+0770 */
  0x0000,   /* U+0771 */
  0x0000,   /* U+0772 */
  0x0000,   /* U+0773 */
  0x0000,   /* U+0774 */
  0x0000,   /* U+0775 */
  0x0000,   /* U+0776 */
  0x0000,   /* U+0777 */
  0x0000,   /* U+0778 */
  0x0000,   /* U+0779 */
  0x0000,   /* U+077A */
  0x0000,   /* U+077B */
  0x0000,   /* U+077C */
  0x0000,   /* U+077D */
  0x0000,   /* U+077E */
  0x0000,   /* U+077F */
  0x0000,   /* U+0780 */
  0x0000,   /* U+0781 */
  0x0000,   /* U+0782 */
  0x0000,   /* U+0783 */
  0x0000,   /* U+0784 */
  0x0000,   /* U+0785 */
  0x0000,   /* U+0786 */
  0x0000,   /* U+0787 */
  0x0000,   /* U+0788 */
  0x0000,   /* U+0789 */
  0x0000,   /* U+078A */
  0x0000,   /* U+078B */
  0x0000,   /* U+078C */
  0x0000,   /* U+078D */
  0x0000,   /* U+078E */
  0x0000,   /* U+078F */
  0x0000,   /* U+0790 */
  0x0000,   /* U+0791 */
  0x0000,   /* U+0792 */
  0x0000,   /* U+0793 */
  0x0000,   /* U+0794 */
  0x0000,   /* U+0795 */
  0x0000,   /* U+0796 */
  0x0000,   /* U+0797 */
  0x0000,   /* U+0798 */
  0x0000,   /* U+0799 */
  0x0000,   /* U+079A */
  0x0000,   /* U+079B */
  0x0000,   /* U+079C */
  0x0000,   /* U+079D */
  0x0000,   /* U+079E */
  0x0000,   /* U+079F */
  0x0000,   /* U+07A0 */
  0x0000,   /* U+07A1 */
  0x0000,   /* U+07A2 */
  0x0000,   /* U+07A3 */
  0x0000,   /* U+07A4 */
  0x0000,   /* U+07A5 */
  0x0000,   /* U+07A6 */
  0x0000,   /* U+07A7 */
  0x0000,   /* U+07A8 */
  0x0000,   /* U+07A9 */
  0x0000,   /* U+07AA */
  0x0000,   /* U+07AB */
  0x0000,   /* U+07AC */
  0x0000,   /* U+07AD */
  0x0000,   /* U+07AE */
  0x0000,   /* U+07AF */
  0x0000,   /* U+07B0 */
  0x0000,   /* U+07B1 */
  0x87B2,   /* U+07B2 */
  0x87B3,   /* U+07B3 */
  0x87B4,   /* U+07B4 */
  0x87B5,   /* U+07B5 */
  0x87B6,   /* U+07B6 */
  0x87B7,   /* U+07B7 */
  0x87B8,   /* U+07B8 */
  0x87B9,   /* U+07B9 */
  0x87BA,   /* U+07BA */
  0x87BB,   /* U+07BB */
  0x87BC,   /* U+07BC */
  0x87BD,   /* U+07BD */
  0x87BE,   /* U+07BE */
  0x87BF,   /* U+07BF */
  0x0000,   /* U+07C0 */
  0x0000,   /* U+07C1 */
  0x0000,   /* U+07C2 */
  0x0000,   /* U+07C3 */
  0x0000,   /* U+07C4 */
  0x0000,   /* U+07C5 */
  0x0000,   /* U+07C6 */
  0x0000,   /* U+07C7 */
  0x0000,   /* U+07C8 */
  0x0000,   /* U+07C9 */
  0x0000,   /* U+07CA */
  0x0000,   /* U+07CB */
  0x0000,   /* U+07CC */
  0x0000,   /* U+07CD */
  0x0000,   /* U+07CE */
  0x0000,   /* U+07CF */
  0x0000,   /* U+07D0 */
  0x0000,   /* U+07D1 */
  0x0000,   /* U+07D2 */
  0x0000,   /* U+07D3 */
  0x0000,   /* U+07D4 */
  0x0000,   /* U+07D5 */
  0x0000,   /* U+07D6 */
  0x0000,   /* U+07D7 */
  0x0000,   /* U+07D8 */
  0x0000,   /* U+07D9 */
  0x0000,   /* U+07DA */
  0x0000,   /* U+07DB */
  0x0000,   /* U+07DC */
  0x0000,   /* U+07DD */
  0x0000,   /* U+07DE */
  0x0000,   /* U+07DF */
  0x0000,   /* U+07E0 */
  0x0000,   /* U+07E1 */
  0x0000,   /* U+07E2 */
  0x0000,   /* U+07E3 */
  0x0000,   /* U+07E4 */
  0x0000,   /* U+07E5 */
  0x0000,   /* U+07E6 */
  0x0000,   /* U+07E7 */
  0x0000,   /* U+07E8 */
  0x0000,   /* U+07E9 */
  0x0000,   /* U+07EA */
  0x0000,   /* U+07EB */
  0x0000,   /* U+07EC */
  0x0000,   /* U+07ED */
  0x0000,   /* U+07EE */
  0x0000,   /* U+07EF */
  0x0000,   /* U+07F0 */
  0x0000,   /* U+07F1 */
  0x0000,   /* U+07F2 */
  0x0000,   /* U+07F3 */
  0x0000,   /* U+07F4 */
  0x0000,   /* U+07F5 */
  0x0000,   /* U+07F6 */
  0x0000,   /* U+07F7 */
  0x0000,   /* U+07F8 */
  0x0000,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0x87FB,   /* U+07FB */
  0x87FC,   /* U+07FC */
  0x87FD,   /* U+07FD */
  0x87FE,   /* U+07FE */
  0x87FF,   /* U+07FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0700 */
  0x0000,   /* U+0701 */
  0x0000,   /* U+0702 */
  0x0000,   /* U+0703 */
  0x0000,   /* U+0704 */
  0x0000,   /* U+0705 */
  0x0000,   /* U+0706 */
  0x0000,   /* U+0707 */
  0x0000,   /* U+0708 */
  0x0000,   /* U+0709 */
  0x0000,   /* U+070A */
  0x0000,   /* U+070B */
  0x0000,   /* U+070C */
  0x0000,   /* U+070D */
  0x0000,   /* U+070E */
  0x0000,   /* U+070F */
  0x0000,   /* U+0710 */
  0x0000,   /* U+0711 */
  0x0000,   /* U+0712 */
  0x0000,   /* U+0713 */
  0x0111,   /* U+0714 */
  0x0000,   /* U+0715 */
  0x0000,   /* U+0716 */
  0x0000,   /* U+0717 */
  0x0000,   /* U+0718 */
  0x0000,   /* U+0719 */
  0x0000,   /* U+071A */
  0x0000,   /* U+071B */
  0x0111,   /* U+071C */
  0x0000,   /* U+071D */
  0x0000,   /* U+071E */
  0x0000,   /* U+071F */
  0x0000,   /* U+0720 */
  0x0000,   /* U+0721 */
  0x0000,   /* U+0722 */
  0x0000,   /* U+0723 */
  0x0000,   /* U+0724 */
  0x0000,   /* U+0725 */
  0x0000,   /* U+0726 */
  0x0111,   /* U+0727 */
  0x0000,   /* U+0728 */
  0x0000,   /* U+0729 */
  0x0000,   /* U+072A */
  0x0000,   /* U+072B */
  0x0000,   /* U+072C */
  0x0112,   /* U+072D */
  0x0112,   /* U+072E */
  0x0112,   /* U+072F */
  0x0000,   /* U+0730 */
  0x0000,   /* U+0731 */
  0x0000,   /* U+0732 */
  0x0000,   /* U+0733 */
  0x0000,   /* U+0734 */
  0x0000,   /* U+0735 */
  0x0000,   /* U+0736 */
  0x0000,   /* U+0737 */
  0x0000,   /* U+0738 */
  0x0000,   /* U+0739 */
  0x0000,   /* U+073A */
  0x0000,   /* U+073B */
  0x0000,   /* U+073C */
  0x0000,   /* U+073D */
  0x0000,   /* U+073E */
  0x0000,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0000,   /* U+0741 */
  0x0000,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0000,   /* U+0745 */
  0x0000,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0x0000,   /* U+074B */
  0x0000,   /* U+074C */
  0x0000,   /* U+074D */
  0x0000,   /* U+074E */
  0x0000,   /* U+074F */
  0x0000,   /* U+0750 */
  0x0000,   /* U+0751 */
  0x0000,   /* U+0752 */
  0x0000,   /* U+0753 */
  0x0000,   /* U+0754 */
  0x0000,   /* U+0755 */
  0x0000,   /* U+0756 */
  0x0000,   /* U+0757 */
  0x0000,   /* U+0758 */
  0x0000,   /* U+0759 */
  0x0000,   /* U+075A */
  0x0000,   /* U+075B */
  0x0000,   /* U+075C */
  0x0000,   /* U+075D */
  0x0000,   /* U+075E */
  0x0000,   /* U+075F */
  0x0000,   /* U+0760 */
  0x0000,   /* U+0761 */
  0x0000,   /* U+0762 */
  0x0000,   /* U+0763 */
  0x0000,   /* U+0764 */
  0x0000,   /* U+0765 */
  0x0000,   /* U+0766 */
  0x0000,   /* U+0767 */
  0x0000,   /* U+0768 */
  0x0000,   /* U+0769 */
  0x0000,   /* U+076A */
  0x0000,   /* U+076B */
  0x0000,   /* U+076C */
  0x0000,   /* U+076D */
  0x0000,   /* U+076E */
  0x0000,   /* U+076F */
  0x0000,   /* U+0770 */
  0x0000,   /* U+0771 */
  0x0000,   /* U+0772 */
  0x0000,   /* U+0773 */
  0x0000,   /* U+0774 */
  0x0000,   /* U+0775 */
  0x0000,   /* U+0776 */
  0x0000,   /* U+0777 */
  0x0000,   /* U+0778 */
  0x0000,   /* U+0779 */
  0x0000,   /* U+077A */
  0x0000,   /* U+077B */
  0x0000,   /* U+077C */
  0x0000,   /* U+077D */
  0x0000,   /* U+077E */
  0x0000,   /* U+077F */
  0x0000,   /* U+0780 */
  0x0000,   /* U+0781 */
  0x0000,   /* U+0782 */
  0x0000,   /* U+0783 */
  0x0000,   /* U+0784 */
  0x0000,   /* U+0785 */
  0x0000,   /* U+0786 */
  0x0000,   /* U+0787 */
  0x0000,   /* U+0788 */
  0x0000,   /* U+0789 */
  0x0000,   /* U+078A */
  0x0000,   /* U+078B */
  0x0000,   /* U+078C */
  0x0000,   /* U+078D */
  0x0000,   /* U+078E */
  0x0000,   /* U+078F */
  0x0000,   /* U+0790 */
  0x0000,   /* U+0791 */
  0x0000,   /* U+0792 */
  0x0000,   /* U+0793 */
  0x0000,   /* U+0794 */
  0x0000,   /* U+0795 */
  0x0000,   /* U+0796 */
  0x0000,   /* U+0797 */
  0x0000,   /* U+0798 */
  0x0000,   /* U+0799 */
  0x0000,   /* U+079A */
  0x0000,   /* U+079B */
  0x0000,   /* U+079C */
  0x0000,   /* U+079D */
  0x0000,   /* U+079E */
  0x0000,   /* U+079F */
  0x0000,   /* U+07A0 */
  0x0000,   /* U+07A1 */
  0x0000,   /* U+07A2 */
  0x0000,   /* U+07A3 */
  0x0000,   /* U+07A4 */
  0x0000,   /* U+07A5 */
  0x0000,   /* U+07A6 */
  0x0000,   /* U+07A7 */
  0x0000,   /* U+07A8 */
  0x0000,   /* U+07A9 */
  0x0000,   /* U+07AA */
  0x0000,   /* U+07AB */
  0x0000,   /* U+07AC */
  0x0000,   /* U+07AD */
  0x0000,   /* U+07AE */
  0x0000,   /* U+07AF */
  0x0000,   /* U+07B0 */
  0x0000,   /* U+07B1 */
  0x0000,   /* U+07B2 */
  0x0000,   /* U+07B3 */
  0x0000,   /* U+07B4 */
  0x0000,   /* U+07B5 */
  0x0000,   /* U+07B6 */
  0x0000,   /* U+07B7 */
  0x0000,   /* U+07B8 */
  0x0000,   /* U+07B9 */
  0x0000,   /* U+07BA */
  0x0000,   /* U+07BB */
  0x0000,   /* U+07BC */
  0x0000,   /* U+07BD */
  0x0000,   /* U+07BE */
  0x0000,   /* U+07BF */
  0x0000,   /* U+07C0 */
  0x0000,   /* U+07C1 */
  0x0000,   /* U+07C2 */
  0x0000,   /* U+07C3 */
  0x0000,   /* U+07C4 */
  0x0000,   /* U+07C5 */
  0x0000,   /* U+07C6 */
  0x0000,   /* U+07C7 */
  0x0000,   /* U+07C8 */
  0x0000,   /* U+07C9 */
  0x0000,   /* U+07CA */
  0x0000,   /* U+07CB */
  0x0000,   /* U+07CC */
  0x0000,   /* U+07CD */
  0x0000,   /* U+07CE */
  0x0000,   /* U+07CF */
  0x0000,   /* U+07D0 */
  0x0000,   /* U+07D1 */
  0x0000,   /* U+07D2 */
  0x0000,   /* U+07D3 */
  0x0000,   /* U+07D4 */
  0x0000,   /* U+07D5 */
  0x0000,   /* U+07D6 */
  0x0000,   /* U+07D7 */
  0x0000,   /* U+07D8 */
  0x0000,   /* U+07D9 */
  0x0000,   /* U+07DA */
  0x0000,   /* U+07DB */
  0x0000,   /* U+07DC */
  0x0000,   /* U+07DD */
  0x0000,   /* U+07DE */
  0x0000,   /* U+07DF */
  0x0000,   /* U+07E0 */
  0x0000,   /* U+07E1 */
  0x0000,   /* U+07E2 */
  0x0000,   /* U+07E3 */
  0x0000,   /* U+07E4 */
  0x0000,   /* U+07E5 */
  0x0000,   /* U+07E6 */
  0x0000,   /* U+07E7 */
  0x0110,   /* U+07E8 */
  0x0110,   /* U+07E9 */
  0x0110,   /* U+07EA */
  0x0000,   /* U+07EB */
  0x0000,   /* U+07EC */
  0x0000,   /* U+07ED */
  0x0000,   /* U+07EE */
  0x0000,   /* U+07EF */
  0x0000,   /* U+07F0 */
  0x0000,   /* U+07F1 */
  0x0000,   /* U+07F2 */
  0x0000,   /* U+07F3 */
  0x0000,   /* U+07F4 */
  0x0000,   /* U+07F5 */
  0x0000,   /* U+07F6 */
  0x0000,   /* U+07F7 */
  0x0000,   /* U+07F8 */
  0x0000,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0x0000,   /* U+07FB */
  0x0000,   /* U+07FC */
  0x0000,   /* U+07FD */
  0x0000,   /* U+07FE */
  0x0000,   /* U+07FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0700 */
  0x0000,   /* U+0701 */
  0x0000,   /* U+0702 */
  0x0000,   /* U+0703 */
  0x0000,   /* U+0704 */
  0x0000,   /* U+0705 */
  0x0000,   /* U+0706 */
  0x0000,   /* U+0707 */
  0x0000,   /* U+0708 */
  0x0000,   /* U+0709 */
  0x0000,   /* U+070A */
  0x0000,   /* U+070B */
  0x0000,   /* U+070C */
  0x0000,   /* U+070D */
  0x0000,   /* U+070E */
  0x0000,   /* U+070F */
  0x0000,   /* U+0710 */
  0x0000,   /* U+0711 */
  0x0000,   /* U+0712 */
  0x0000,   /* U+0713 */
  0x0004,   /* U+0714 */
  0x0000,   /* U+0715 */
  0x0000,   /* U+0716 */
  0x0000,   /* U+0717 */
  0x0000,   /* U+0718 */
  0x0000,   /* U+0719 */
  0x0000,   /* U+071A */
  0x0000,   /* U+071B */
  0x0004,   /* U+071C */
  0x0000,   /* U+071D */
  0x0000,   /* U+071E */
  0x0000,   /* U+071F */
  0x0000,   /* U+0720 */
  0x0000,   /* U+0721 */
  0x0000,   /* U+0722 */
  0x0000,   /* U+0723 */
  0x0000,   /* U+0724 */
  0x0000,   /* U+0725 */
  0x0000,   /* U+0726 */
  0x0004,   /* U+0727 */
  0x0000,   /* U+0728 */
  0x0000,   /* U+0729 */
  0x0000,   /* U+072A */
  0x0000,   /* U+072B */
  0x0000,   /* U+072C */
  0x0004,   /* U+072D */
  0x0004,   /* U+072E */
  0x0004,   /* U+072F */
  0x0000,   /* U+0730 */
  0x0000,   /* U+0731 */
  0x0000,   /* U+0732 */
  0x0000,   /* U+0733 */
  0x0000,   /* U+0734 */
  0x0000,   /* U+0735 */
  0x0000,   /* U+0736 */
  0x0000,   /* U+0737 */
  0x0000,   /* U+0738 */
  0x0000,   /* U+0739 */
  0x0000,   /* U+073A */
  0x0000,   /* U+073B */
  0x0000,   /* U+073C */
  0x0000,   /* U+073D */
  0x0000,   /* U+073E */
  0x0000,   /* U+073F */
  0x0000,   /* U+0740 */
  0x0000,   /* U+0741 */
  0x0000,   /* U+0742 */
  0x0000,   /* U+0743 */
  0x0000,   /* U+0744 */
  0x0000,   /* U+0745 */
  0x0000,   /* U+0746 */
  0x0000,   /* U+0747 */
  0x0000,   /* U+0748 */
  0x0000,   /* U+0749 */
  0x0000,   /* U+074A */
  0x0000,   /* U+074B */
  0x0000,   /* U+074C */
  0x0000,   /* U+074D */
  0x0000,   /* U+074E */
  0x0000,   /* U+074F */
  0x0000,   /* U+0750 */
  0x0000,   /* U+0751 */
  0x0000,   /* U+0752 */
  0x0000,   /* U+0753 */
  0x0000,   /* U+0754 */
  0x0000,   /* U+0755 */
  0x0000,   /* U+0756 */
  0x0000,   /* U+0757 */
  0x0000,   /* U+0758 */
  0x0000,   /* U+0759 */
  0x0000,   /* U+075A */
  0x0000,   /* U+075B */
  0x0000,   /* U+075C */
  0x0000,   /* U+075D */
  0x0000,   /* U+075E */
  0x0000,   /* U+075F */
  0x0000,   /* U+0760 */
  0x0000,   /* U+0761 */
  0x0000,   /* U+0762 */
  0x0000,   /* U+0763 */
  0x0000,   /* U+0764 */
  0x0000,   /* U+0765 */
  0x0000,   /* U+0766 */
  0x0000,   /* U+0767 */
  0x0000,   /* U+0768 */
  0x0000,   /* U+0769 */
  0x0000,   /* U+076A */
  0x0000,   /* U+076B */
  0x0000,   /* U+076C */
  0x0000,   /* U+076D */
  0x0000,   /* U+076E */
  0x0000,   /* U+076F */
  0x0000,   /* U+0770 */
  0x0000,   /* U+0771 */
  0x0000,   /* U+0772 */
  0x0000,   /* U+0773 */
  0x0000,   /* U+0774 */
  0x0000,   /* U+0775 */
  0x0000,   /* U+0776 */
  0x0000,   /* U+0777 */
  0x0000,   /* U+0778 */
  0x0000,   /* U+0779 */
  0x0000,   /* U+077A */
  0x0000,   /* U+077B */
  0x0000,   /* U+077C */
  0x0000,   /* U+077D */
  0x0000,   /* U+077E */
  0x0000,   /* U+077F */
  0x0000,   /* U+0780 */
  0x0000,   /* U+0781 */
  0x0000,   /* U+0782 */
  0x0000,   /* U+0783 */
  0x0000,   /* U+0784 */
  0x0000,   /* U+0785 */
  0x0000,   /* U+0786 */
  0x0000,   /* U+0787 */
  0x0000,   /* U+0788 */
  0x0000,   /* U+0789 */
  0x0000,   /* U+078A */
  0x0000,   /* U+078B */
  0x0000,   /* U+078C */
  0x0000,   /* U+078D */
  0x0000,   /* U+078E */
  0x0000,   /* U+078F */
  0x0000,   /* U+0790 */
  0x0000,   /* U+0791 */
  0x0000,   /* U+0792 */
  0x0000,   /* U+0793 */
  0x0000,   /* U+0794 */
  0x0000,   /* U+0795 */
  0x0000,   /* U+0796 */
  0x0000,   /* U+0797 */
  0x0000,   /* U+0798 */
  0x0000,   /* U+0799 */
  0x0000,   /* U+079A */
  0x0000,   /* U+079B */
  0x0000,   /* U+079C */
  0x0000,   /* U+079D */
  0x0000,   /* U+079E */
  0x0000,   /* U+079F */
  0x0000,   /* U+07A0 */
  0x0000,   /* U+07A1 */
  0x0000,   /* U+07A2 */
  0x0000,   /* U+07A3 */
  0x0000,   /* U+07A4 */
  0x0000,   /* U+07A5 */
  0x0000,   /* U+07A6 */
  0x0000,   /* U+07A7 */
  0x0000,   /* U+07A8 */
  0x0000,   /* U+07A9 */
  0x0000,   /* U+07AA */
  0x0000,   /* U+07AB */
  0x0000,   /* U+07AC */
  0x0000,   /* U+07AD */
  0x0000,   /* U+07AE */
  0x0000,   /* U+07AF */
  0x0000,   /* U+07B0 */
  0x0000,   /* U+07B1 */
  0x0000,   /* U+07B2 */
  0x0000,   /* U+07B3 */
  0x0000,   /* U+07B4 */
  0x0000,   /* U+07B5 */
  0x0000,   /* U+07B6 */
  0x0000,   /* U+07B7 */
  0x0000,   /* U+07B8 */
  0x0000,   /* U+07B9 */
  0x0000,   /* U+07BA */
  0x0000,   /* U+07BB */
  0x0000,   /* U+07BC */
  0x0000,   /* U+07BD */
  0x0000,   /* U+07BE */
  0x0000,   /* U+07BF */
  0x0000,   /* U+07C0 */
  0x0000,   /* U+07C1 */
  0x0000,   /* U+07C2 */
  0x0000,   /* U+07C3 */
  0x0000,   /* U+07C4 */
  0x0000,   /* U+07C5 */
  0x0000,   /* U+07C6 */
  0x0000,   /* U+07C7 */
  0x0000,   /* U+07C8 */
  0x0000,   /* U+07C9 */
  0x0000,   /* U+07CA */
  0x0000,   /* U+07CB */
  0x0000,   /* U+07CC */
  0x0000,   /* U+07CD */
  0x0000,   /* U+07CE */
  0x0000,   /* U+07CF */
  0x0000,   /* U+07D0 */
  0x0000,   /* U+07D1 */
  0x0000,   /* U+07D2 */
  0x0000,   /* U+07D3 */
  0x0000,   /* U+07D4 */
  0x0000,   /* U+07D5 */
  0x0000,   /* U+07D6 */
  0x0000,   /* U+07D7 */
  0x0000,   /* U+07D8 */
  0x0000,   /* U+07D9 */
  0x0000,   /* U+07DA */
  0x0000,   /* U+07DB */
  0x0000,   /* U+07DC */
  0x0000,   /* U+07DD */
  0x0000,   /* U+07DE */
  0x0000,   /* U+07DF */
  0x0000,   /* U+07E0 */
  0x0000,   /* U+07E1 */
  0x0000,   /* U+07E2 */
  0x0000,   /* U+07E3 */
  0x0000,   /* U+07E4 */
  0x0000,   /* U+07E5 */
  0x0000,   /* U+07E6 */
  0x0000,   /* U+07E7 */
  0x0004,   /* U+07E8 */
  0x0004,   /* U+07E9 */
  0x0004,   /* U+07EA */
  0x0000,   /* U+07EB */
  0x0000,   /* U+07EC */
  0x0000,   /* U+07ED */
  0x0000,   /* U+07EE */
  0x0000,   /* U+07EF */
  0x0000,   /* U+07F0 */
  0x0000,   /* U+07F1 */
  0x0000,   /* U+07F2 */
  0x0000,   /* U+07F3 */
  0x0000,   /* U+07F4 */
  0x0000,   /* U+07F5 */
  0x0000,   /* U+07F6 */
  0x0000,   /* U+07F7 */
  0x0000,   /* U+07F8 */
  0x0000,   /* U+07F9 */
  0x0000,   /* U+07FA */
  0x0000,   /* U+07FB */
  0x0000,   /* U+07FC */
  0x0000,   /* U+07FD */
  0x0000,   /* U+07FE */
  0x0000,   /* U+07FF */
};

uint16 uca900_p008[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x22E3,   /* U+0800 */
  0x22E4,   /* U+0801 */
  0x22E5,   /* U+0802 */
  0x22E6,   /* U+0803 */
  0x22E7,   /* U+0804 */
  0x22E8,   /* U+0805 */
  0x22E9,   /* U+0806 */
  0x22EA,   /* U+0807 */
  0x22EB,   /* U+0808 */
  0x22EC,   /* U+0809 */
  0x22ED,   /* U+080A */
  0x22EE,   /* U+080B */
  0x22EF,   /* U+080C */
  0x22F0,   /* U+080D */
  0x22F1,   /* U+080E */
  0x22F2,   /* U+080F */
  0x22F3,   /* U+0810 */
  0x22F4,   /* U+0811 */
  0x22F5,   /* U+0812 */
  0x22F6,   /* U+0813 */
  0x22F7,   /* U+0814 */
  0x22F8,   /* U+0815 */
  0x22F9,   /* U+0816 */
  0x22FA,   /* U+0817 */
  0x0000,   /* U+0818 */
  0x0000,   /* U+0819 */
  0x22FB,   /* U+081A */
  0x22FC,   /* U+081B */
  0x0000,   /* U+081C */
  0x0000,   /* U+081D */
  0x0000,   /* U+081E */
  0x0000,   /* U+081F */
  0x0000,   /* U+0820 */
  0x0000,   /* U+0821 */
  0x0000,   /* U+0822 */
  0x0000,   /* U+0823 */
  0x0000,   /* U+0824 */
  0x0000,   /* U+0825 */
  0x0000,   /* U+0826 */
  0x0000,   /* U+0827 */
  0x0000,   /* U+0828 */
  0x0000,   /* U+0829 */
  0x0000,   /* U+082A */
  0x0000,   /* U+082B */
  0x0000,   /* U+082C */
  0x0000,   /* U+082D */
  0xFBC0,   /* U+082E */
  0xFBC0,   /* U+082F */
  0x0242,   /* U+0830 */
  0x0243,   /* U+0831 */
  0x0244,   /* U+0832 */
  0x0245,   /* U+0833 */
  0x0246,   /* U+0834 */
  0x0247,   /* U+0835 */
  0x0248,   /* U+0836 */
  0x0249,   /* U+0837 */
  0x024A,   /* U+0838 */
  0x024B,   /* U+0839 */
  0x024C,   /* U+083A */
  0x024D,   /* U+083B */
  0x024E,   /* U+083C */
  0x024F,   /* U+083D */
  0x0250,   /* U+083E */
  0xFBC0,   /* U+083F */
  0x23F2,   /* U+0840 */
  0x23F3,   /* U+0841 */
  0x23F4,   /* U+0842 */
  0x23F5,   /* U+0843 */
  0x23F6,   /* U+0844 */
  0x23F7,   /* U+0845 */
  0x23F8,   /* U+0846 */
  0x23F9,   /* U+0847 */
  0x23FA,   /* U+0848 */
  0x23FB,   /* U+0849 */
  0x23FC,   /* U+084A */
  0x23FD,   /* U+084B */
  0x23FE,   /* U+084C */
  0x23FF,   /* U+084D */
  0x2400,   /* U+084E */
  0x2401,   /* U+084F */
  0x2402,   /* U+0850 */
  0x2403,   /* U+0851 */
  0x2404,   /* U+0852 */
  0x2405,   /* U+0853 */
  0x2406,   /* U+0854 */
  0x2407,   /* U+0855 */
  0x2408,   /* U+0856 */
  0x2409,   /* U+0857 */
  0x240A,   /* U+0858 */
  0x0000,   /* U+0859 */
  0x0000,   /* U+085A */
  0x0000,   /* U+085B */
  0xFBC0,   /* U+085C */
  0xFBC0,   /* U+085D */
  0x03D8,   /* U+085E */
  0xFBC0,   /* U+085F */
  0xFBC0,   /* U+0860 */
  0xFBC0,   /* U+0861 */
  0xFBC0,   /* U+0862 */
  0xFBC0,   /* U+0863 */
  0xFBC0,   /* U+0864 */
  0xFBC0,   /* U+0865 */
  0xFBC0,   /* U+0866 */
  0xFBC0,   /* U+0867 */
  0xFBC0,   /* U+0868 */
  0xFBC0,   /* U+0869 */
  0xFBC0,   /* U+086A */
  0xFBC0,   /* U+086B */
  0xFBC0,   /* U+086C */
  0xFBC0,   /* U+086D */
  0xFBC0,   /* U+086E */
  0xFBC0,   /* U+086F */
  0xFBC0,   /* U+0870 */
  0xFBC0,   /* U+0871 */
  0xFBC0,   /* U+0872 */
  0xFBC0,   /* U+0873 */
  0xFBC0,   /* U+0874 */
  0xFBC0,   /* U+0875 */
  0xFBC0,   /* U+0876 */
  0xFBC0,   /* U+0877 */
  0xFBC0,   /* U+0878 */
  0xFBC0,   /* U+0879 */
  0xFBC0,   /* U+087A */
  0xFBC0,   /* U+087B */
  0xFBC0,   /* U+087C */
  0xFBC0,   /* U+087D */
  0xFBC0,   /* U+087E */
  0xFBC0,   /* U+087F */
  0xFBC0,   /* U+0880 */
  0xFBC0,   /* U+0881 */
  0xFBC0,   /* U+0882 */
  0xFBC0,   /* U+0883 */
  0xFBC0,   /* U+0884 */
  0xFBC0,   /* U+0885 */
  0xFBC0,   /* U+0886 */
  0xFBC0,   /* U+0887 */
  0xFBC0,   /* U+0888 */
  0xFBC0,   /* U+0889 */
  0xFBC0,   /* U+088A */
  0xFBC0,   /* U+088B */
  0xFBC0,   /* U+088C */
  0xFBC0,   /* U+088D */
  0xFBC0,   /* U+088E */
  0xFBC0,   /* U+088F */
  0xFBC0,   /* U+0890 */
  0xFBC0,   /* U+0891 */
  0xFBC0,   /* U+0892 */
  0xFBC0,   /* U+0893 */
  0xFBC0,   /* U+0894 */
  0xFBC0,   /* U+0895 */
  0xFBC0,   /* U+0896 */
  0xFBC0,   /* U+0897 */
  0xFBC0,   /* U+0898 */
  0xFBC0,   /* U+0899 */
  0xFBC0,   /* U+089A */
  0xFBC0,   /* U+089B */
  0xFBC0,   /* U+089C */
  0xFBC0,   /* U+089D */
  0xFBC0,   /* U+089E */
  0xFBC0,   /* U+089F */
  0x2317,   /* U+08A0 */
  0x2319,   /* U+08A1 */
  0x232B,   /* U+08A2 */
  0x236D,   /* U+08A3 */
  0x237C,   /* U+08A4 */
  0x2386,   /* U+08A5 */
  0x23A2,   /* U+08A6 */
  0x23A6,   /* U+08A7 */
  0x2308,   /* U+08A8 */
  0x2309,   /* U+08A9 */
  0x2356,   /* U+08AA */
  0x23C4,   /* U+08AB */
  0x230A,   /* U+08AC */
  0x230B,   /* U+08AD */
  0x233F,   /* U+08AE */
  0x2367,   /* U+08AF */
  0x2391,   /* U+08B0 */
  0x23C0,   /* U+08B1 */
  0x2357,   /* U+08B2 */
  0x2375,   /* U+08B3 */
  0x238F,   /* U+08B4 */
  0xFBC0,   /* U+08B5 */
  0x231A,   /* U+08B6 */
  0x231B,   /* U+08B7 */
  0x2324,   /* U+08B8 */
  0x2358,   /* U+08B9 */
  0x23D3,   /* U+08BA */
  0x2379,   /* U+08BB */
  0x2384,   /* U+08BC */
  0x23A9,   /* U+08BD */
  0xFBC0,   /* U+08BE */
  0xFBC0,   /* U+08BF */
  0xFBC0,   /* U+08C0 */
  0xFBC0,   /* U+08C1 */
  0xFBC0,   /* U+08C2 */
  0xFBC0,   /* U+08C3 */
  0xFBC0,   /* U+08C4 */
  0xFBC0,   /* U+08C5 */
  0xFBC0,   /* U+08C6 */
  0xFBC0,   /* U+08C7 */
  0xFBC0,   /* U+08C8 */
  0xFBC0,   /* U+08C9 */
  0xFBC0,   /* U+08CA */
  0xFBC0,   /* U+08CB */
  0xFBC0,   /* U+08CC */
  0xFBC0,   /* U+08CD */
  0xFBC0,   /* U+08CE */
  0xFBC0,   /* U+08CF */
  0xFBC0,   /* U+08D0 */
  0xFBC0,   /* U+08D1 */
  0xFBC0,   /* U+08D2 */
  0xFBC0,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0000,   /* U+08E3 */
  0x0000,   /* U+08E4 */
  0x0000,   /* U+08E5 */
  0x0000,   /* U+08E6 */
  0x0000,   /* U+08E7 */
  0x0000,   /* U+08E8 */
  0x0000,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x0000,   /* U+08F0 */
  0x0000,   /* U+08F1 */
  0x0000,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0000,   /* U+08F4 */
  0x0000,   /* U+08F5 */
  0x0000,   /* U+08F6 */
  0x0000,   /* U+08F7 */
  0x0000,   /* U+08F8 */
  0x0000,   /* U+08F9 */
  0x0000,   /* U+08FA */
  0x0000,   /* U+08FB */
  0x0000,   /* U+08FC */
  0x0000,   /* U+08FD */
  0x0000,   /* U+08FE */
  0x0000,   /* U+08FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0800 */
  0x0020,   /* U+0801 */
  0x0020,   /* U+0802 */
  0x0020,   /* U+0803 */
  0x0020,   /* U+0804 */
  0x0020,   /* U+0805 */
  0x0020,   /* U+0806 */
  0x0020,   /* U+0807 */
  0x0020,   /* U+0808 */
  0x0020,   /* U+0809 */
  0x0020,   /* U+080A */
  0x0020,   /* U+080B */
  0x0020,   /* U+080C */
  0x0020,   /* U+080D */
  0x0020,   /* U+080E */
  0x0020,   /* U+080F */
  0x0020,   /* U+0810 */
  0x0020,   /* U+0811 */
  0x0020,   /* U+0812 */
  0x0020,   /* U+0813 */
  0x0020,   /* U+0814 */
  0x0020,   /* U+0815 */
  0x0020,   /* U+0816 */
  0x0020,   /* U+0817 */
  0x006A,   /* U+0818 */
  0x006B,   /* U+0819 */
  0x0020,   /* U+081A */
  0x0020,   /* U+081B */
  0x0062,   /* U+081C */
  0x0062,   /* U+081D */
  0x0063,   /* U+081E */
  0x0063,   /* U+081F */
  0x0063,   /* U+0820 */
  0x0064,   /* U+0821 */
  0x0064,   /* U+0822 */
  0x0064,   /* U+0823 */
  0x0065,   /* U+0824 */
  0x0065,   /* U+0825 */
  0x0066,   /* U+0826 */
  0x0066,   /* U+0827 */
  0x0067,   /* U+0828 */
  0x0067,   /* U+0829 */
  0x0067,   /* U+082A */
  0x0068,   /* U+082B */
  0x0069,   /* U+082C */
  0x006C,   /* U+082D */
  0x0020,   /* U+082E */
  0x0020,   /* U+082F */
  0x0020,   /* U+0830 */
  0x0020,   /* U+0831 */
  0x0020,   /* U+0832 */
  0x0020,   /* U+0833 */
  0x0020,   /* U+0834 */
  0x0020,   /* U+0835 */
  0x0020,   /* U+0836 */
  0x0020,   /* U+0837 */
  0x0020,   /* U+0838 */
  0x0020,   /* U+0839 */
  0x0020,   /* U+083A */
  0x0020,   /* U+083B */
  0x0020,   /* U+083C */
  0x0020,   /* U+083D */
  0x0020,   /* U+083E */
  0x0020,   /* U+083F */
  0x0020,   /* U+0840 */
  0x0020,   /* U+0841 */
  0x0020,   /* U+0842 */
  0x0020,   /* U+0843 */
  0x0020,   /* U+0844 */
  0x0020,   /* U+0845 */
  0x0020,   /* U+0846 */
  0x0020,   /* U+0847 */
  0x0020,   /* U+0848 */
  0x0020,   /* U+0849 */
  0x0020,   /* U+084A */
  0x0020,   /* U+084B */
  0x0020,   /* U+084C */
  0x0020,   /* U+084D */
  0x0020,   /* U+084E */
  0x0020,   /* U+084F */
  0x0020,   /* U+0850 */
  0x0020,   /* U+0851 */
  0x0020,   /* U+0852 */
  0x0020,   /* U+0853 */
  0x0020,   /* U+0854 */
  0x0020,   /* U+0855 */
  0x0020,   /* U+0856 */
  0x0020,   /* U+0857 */
  0x0020,   /* U+0858 */
  0x0034,   /* U+0859 */
  0x0034,   /* U+085A */
  0x0034,   /* U+085B */
  0x0020,   /* U+085C */
  0x0020,   /* U+085D */
  0x0020,   /* U+085E */
  0x0020,   /* U+085F */
  0x0020,   /* U+0860 */
  0x0020,   /* U+0861 */
  0x0020,   /* U+0862 */
  0x0020,   /* U+0863 */
  0x0020,   /* U+0864 */
  0x0020,   /* U+0865 */
  0x0020,   /* U+0866 */
  0x0020,   /* U+0867 */
  0x0020,   /* U+0868 */
  0x0020,   /* U+0869 */
  0x0020,   /* U+086A */
  0x0020,   /* U+086B */
  0x0020,   /* U+086C */
  0x0020,   /* U+086D */
  0x0020,   /* U+086E */
  0x0020,   /* U+086F */
  0x0020,   /* U+0870 */
  0x0020,   /* U+0871 */
  0x0020,   /* U+0872 */
  0x0020,   /* U+0873 */
  0x0020,   /* U+0874 */
  0x0020,   /* U+0875 */
  0x0020,   /* U+0876 */
  0x0020,   /* U+0877 */
  0x0020,   /* U+0878 */
  0x0020,   /* U+0879 */
  0x0020,   /* U+087A */
  0x0020,   /* U+087B */
  0x0020,   /* U+087C */
  0x0020,   /* U+087D */
  0x0020,   /* U+087E */
  0x0020,   /* U+087F */
  0x0020,   /* U+0880 */
  0x0020,   /* U+0881 */
  0x0020,   /* U+0882 */
  0x0020,   /* U+0883 */
  0x0020,   /* U+0884 */
  0x0020,   /* U+0885 */
  0x0020,   /* U+0886 */
  0x0020,   /* U+0887 */
  0x0020,   /* U+0888 */
  0x0020,   /* U+0889 */
  0x0020,   /* U+088A */
  0x0020,   /* U+088B */
  0x0020,   /* U+088C */
  0x0020,   /* U+088D */
  0x0020,   /* U+088E */
  0x0020,   /* U+088F */
  0x0020,   /* U+0890 */
  0x0020,   /* U+0891 */
  0x0020,   /* U+0892 */
  0x0020,   /* U+0893 */
  0x0020,   /* U+0894 */
  0x0020,   /* U+0895 */
  0x0020,   /* U+0896 */
  0x0020,   /* U+0897 */
  0x0020,   /* U+0898 */
  0x0020,   /* U+0899 */
  0x0020,   /* U+089A */
  0x0020,   /* U+089B */
  0x0020,   /* U+089C */
  0x0020,   /* U+089D */
  0x0020,   /* U+089E */
  0x0020,   /* U+089F */
  0x0020,   /* U+08A0 */
  0x0020,   /* U+08A1 */
  0x0020,   /* U+08A2 */
  0x0020,   /* U+08A3 */
  0x0020,   /* U+08A4 */
  0x0020,   /* U+08A5 */
  0x0020,   /* U+08A6 */
  0x0020,   /* U+08A7 */
  0x0020,   /* U+08A8 */
  0x0020,   /* U+08A9 */
  0x0020,   /* U+08AA */
  0x0020,   /* U+08AB */
  0x0020,   /* U+08AC */
  0x0020,   /* U+08AD */
  0x0020,   /* U+08AE */
  0x0020,   /* U+08AF */
  0x0020,   /* U+08B0 */
  0x0020,   /* U+08B1 */
  0x0020,   /* U+08B2 */
  0x0020,   /* U+08B3 */
  0x0020,   /* U+08B4 */
  0x0020,   /* U+08B5 */
  0x0020,   /* U+08B6 */
  0x0020,   /* U+08B7 */
  0x0020,   /* U+08B8 */
  0x0020,   /* U+08B9 */
  0x0020,   /* U+08BA */
  0x0020,   /* U+08BB */
  0x0020,   /* U+08BC */
  0x0020,   /* U+08BD */
  0x0020,   /* U+08BE */
  0x0020,   /* U+08BF */
  0x0020,   /* U+08C0 */
  0x0020,   /* U+08C1 */
  0x0020,   /* U+08C2 */
  0x0020,   /* U+08C3 */
  0x0020,   /* U+08C4 */
  0x0020,   /* U+08C5 */
  0x0020,   /* U+08C6 */
  0x0020,   /* U+08C7 */
  0x0020,   /* U+08C8 */
  0x0020,   /* U+08C9 */
  0x0020,   /* U+08CA */
  0x0020,   /* U+08CB */
  0x0020,   /* U+08CC */
  0x0020,   /* U+08CD */
  0x0020,   /* U+08CE */
  0x0020,   /* U+08CF */
  0x0020,   /* U+08D0 */
  0x0020,   /* U+08D1 */
  0x0020,   /* U+08D2 */
  0x0020,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0090,   /* U+08E3 */
  0x0077,   /* U+08E4 */
  0x007B,   /* U+08E5 */
  0x007E,   /* U+08E6 */
  0x006F,   /* U+08E7 */
  0x0072,   /* U+08E8 */
  0x0075,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x006E,   /* U+08F0 */
  0x0071,   /* U+08F1 */
  0x0074,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0078,   /* U+08F4 */
  0x0079,   /* U+08F5 */
  0x007F,   /* U+08F6 */
  0x0091,   /* U+08F7 */
  0x0092,   /* U+08F8 */
  0x0096,   /* U+08F9 */
  0x0097,   /* U+08FA */
  0x0094,   /* U+08FB */
  0x0095,   /* U+08FC */
  0x0093,   /* U+08FD */
  0x007C,   /* U+08FE */
  0x0089,   /* U+08FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0800 */
  0x0002,   /* U+0801 */
  0x0002,   /* U+0802 */
  0x0002,   /* U+0803 */
  0x0002,   /* U+0804 */
  0x0002,   /* U+0805 */
  0x0002,   /* U+0806 */
  0x0002,   /* U+0807 */
  0x0002,   /* U+0808 */
  0x0002,   /* U+0809 */
  0x0002,   /* U+080A */
  0x0002,   /* U+080B */
  0x0002,   /* U+080C */
  0x0002,   /* U+080D */
  0x0002,   /* U+080E */
  0x0002,   /* U+080F */
  0x0002,   /* U+0810 */
  0x0002,   /* U+0811 */
  0x0002,   /* U+0812 */
  0x0002,   /* U+0813 */
  0x0002,   /* U+0814 */
  0x0002,   /* U+0815 */
  0x0002,   /* U+0816 */
  0x0002,   /* U+0817 */
  0x0002,   /* U+0818 */
  0x0002,   /* U+0819 */
  0x0002,   /* U+081A */
  0x0002,   /* U+081B */
  0x0002,   /* U+081C */
  0x0002,   /* U+081D */
  0x0002,   /* U+081E */
  0x0002,   /* U+081F */
  0x0002,   /* U+0820 */
  0x0002,   /* U+0821 */
  0x0002,   /* U+0822 */
  0x0002,   /* U+0823 */
  0x0002,   /* U+0824 */
  0x0002,   /* U+0825 */
  0x0002,   /* U+0826 */
  0x0002,   /* U+0827 */
  0x0002,   /* U+0828 */
  0x0002,   /* U+0829 */
  0x0002,   /* U+082A */
  0x0002,   /* U+082B */
  0x0002,   /* U+082C */
  0x0002,   /* U+082D */
  0x0002,   /* U+082E */
  0x0002,   /* U+082F */
  0x0002,   /* U+0830 */
  0x0002,   /* U+0831 */
  0x0002,   /* U+0832 */
  0x0002,   /* U+0833 */
  0x0002,   /* U+0834 */
  0x0002,   /* U+0835 */
  0x0002,   /* U+0836 */
  0x0002,   /* U+0837 */
  0x0002,   /* U+0838 */
  0x0002,   /* U+0839 */
  0x0002,   /* U+083A */
  0x0002,   /* U+083B */
  0x0002,   /* U+083C */
  0x0002,   /* U+083D */
  0x0002,   /* U+083E */
  0x0002,   /* U+083F */
  0x0002,   /* U+0840 */
  0x0002,   /* U+0841 */
  0x0002,   /* U+0842 */
  0x0002,   /* U+0843 */
  0x0002,   /* U+0844 */
  0x0002,   /* U+0845 */
  0x0002,   /* U+0846 */
  0x0002,   /* U+0847 */
  0x0002,   /* U+0848 */
  0x0002,   /* U+0849 */
  0x0002,   /* U+084A */
  0x0002,   /* U+084B */
  0x0002,   /* U+084C */
  0x0002,   /* U+084D */
  0x0002,   /* U+084E */
  0x0002,   /* U+084F */
  0x0002,   /* U+0850 */
  0x0002,   /* U+0851 */
  0x0002,   /* U+0852 */
  0x0002,   /* U+0853 */
  0x0002,   /* U+0854 */
  0x0002,   /* U+0855 */
  0x0002,   /* U+0856 */
  0x0002,   /* U+0857 */
  0x0002,   /* U+0858 */
  0x0002,   /* U+0859 */
  0x0002,   /* U+085A */
  0x0002,   /* U+085B */
  0x0002,   /* U+085C */
  0x0002,   /* U+085D */
  0x0002,   /* U+085E */
  0x0002,   /* U+085F */
  0x0002,   /* U+0860 */
  0x0002,   /* U+0861 */
  0x0002,   /* U+0862 */
  0x0002,   /* U+0863 */
  0x0002,   /* U+0864 */
  0x0002,   /* U+0865 */
  0x0002,   /* U+0866 */
  0x0002,   /* U+0867 */
  0x0002,   /* U+0868 */
  0x0002,   /* U+0869 */
  0x0002,   /* U+086A */
  0x0002,   /* U+086B */
  0x0002,   /* U+086C */
  0x0002,   /* U+086D */
  0x0002,   /* U+086E */
  0x0002,   /* U+086F */
  0x0002,   /* U+0870 */
  0x0002,   /* U+0871 */
  0x0002,   /* U+0872 */
  0x0002,   /* U+0873 */
  0x0002,   /* U+0874 */
  0x0002,   /* U+0875 */
  0x0002,   /* U+0876 */
  0x0002,   /* U+0877 */
  0x0002,   /* U+0878 */
  0x0002,   /* U+0879 */
  0x0002,   /* U+087A */
  0x0002,   /* U+087B */
  0x0002,   /* U+087C */
  0x0002,   /* U+087D */
  0x0002,   /* U+087E */
  0x0002,   /* U+087F */
  0x0002,   /* U+0880 */
  0x0002,   /* U+0881 */
  0x0002,   /* U+0882 */
  0x0002,   /* U+0883 */
  0x0002,   /* U+0884 */
  0x0002,   /* U+0885 */
  0x0002,   /* U+0886 */
  0x0002,   /* U+0887 */
  0x0002,   /* U+0888 */
  0x0002,   /* U+0889 */
  0x0002,   /* U+088A */
  0x0002,   /* U+088B */
  0x0002,   /* U+088C */
  0x0002,   /* U+088D */
  0x0002,   /* U+088E */
  0x0002,   /* U+088F */
  0x0002,   /* U+0890 */
  0x0002,   /* U+0891 */
  0x0002,   /* U+0892 */
  0x0002,   /* U+0893 */
  0x0002,   /* U+0894 */
  0x0002,   /* U+0895 */
  0x0002,   /* U+0896 */
  0x0002,   /* U+0897 */
  0x0002,   /* U+0898 */
  0x0002,   /* U+0899 */
  0x0002,   /* U+089A */
  0x0002,   /* U+089B */
  0x0002,   /* U+089C */
  0x0002,   /* U+089D */
  0x0002,   /* U+089E */
  0x0002,   /* U+089F */
  0x0002,   /* U+08A0 */
  0x0002,   /* U+08A1 */
  0x0002,   /* U+08A2 */
  0x0002,   /* U+08A3 */
  0x0002,   /* U+08A4 */
  0x0002,   /* U+08A5 */
  0x0002,   /* U+08A6 */
  0x0002,   /* U+08A7 */
  0x0002,   /* U+08A8 */
  0x0002,   /* U+08A9 */
  0x0002,   /* U+08AA */
  0x0002,   /* U+08AB */
  0x0002,   /* U+08AC */
  0x0004,   /* U+08AD */
  0x0002,   /* U+08AE */
  0x0002,   /* U+08AF */
  0x0002,   /* U+08B0 */
  0x0002,   /* U+08B1 */
  0x0002,   /* U+08B2 */
  0x0002,   /* U+08B3 */
  0x0002,   /* U+08B4 */
  0x0002,   /* U+08B5 */
  0x0002,   /* U+08B6 */
  0x0002,   /* U+08B7 */
  0x0002,   /* U+08B8 */
  0x0002,   /* U+08B9 */
  0x0002,   /* U+08BA */
  0x0002,   /* U+08BB */
  0x0002,   /* U+08BC */
  0x0002,   /* U+08BD */
  0x0002,   /* U+08BE */
  0x0002,   /* U+08BF */
  0x0002,   /* U+08C0 */
  0x0002,   /* U+08C1 */
  0x0002,   /* U+08C2 */
  0x0002,   /* U+08C3 */
  0x0002,   /* U+08C4 */
  0x0002,   /* U+08C5 */
  0x0002,   /* U+08C6 */
  0x0002,   /* U+08C7 */
  0x0002,   /* U+08C8 */
  0x0002,   /* U+08C9 */
  0x0002,   /* U+08CA */
  0x0002,   /* U+08CB */
  0x0002,   /* U+08CC */
  0x0002,   /* U+08CD */
  0x0002,   /* U+08CE */
  0x0002,   /* U+08CF */
  0x0002,   /* U+08D0 */
  0x0002,   /* U+08D1 */
  0x0002,   /* U+08D2 */
  0x0002,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0002,   /* U+08E3 */
  0x0002,   /* U+08E4 */
  0x0002,   /* U+08E5 */
  0x0002,   /* U+08E6 */
  0x0002,   /* U+08E7 */
  0x0002,   /* U+08E8 */
  0x0002,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x0002,   /* U+08F0 */
  0x0002,   /* U+08F1 */
  0x0002,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0002,   /* U+08F4 */
  0x0002,   /* U+08F5 */
  0x0002,   /* U+08F6 */
  0x0002,   /* U+08F7 */
  0x0002,   /* U+08F8 */
  0x0002,   /* U+08F9 */
  0x0002,   /* U+08FA */
  0x0002,   /* U+08FB */
  0x0002,   /* U+08FC */
  0x0002,   /* U+08FD */
  0x0002,   /* U+08FE */
  0x0002,   /* U+08FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0800 */
  0x0000,   /* U+0801 */
  0x0000,   /* U+0802 */
  0x0000,   /* U+0803 */
  0x0000,   /* U+0804 */
  0x0000,   /* U+0805 */
  0x0000,   /* U+0806 */
  0x0000,   /* U+0807 */
  0x0000,   /* U+0808 */
  0x0000,   /* U+0809 */
  0x0000,   /* U+080A */
  0x0000,   /* U+080B */
  0x0000,   /* U+080C */
  0x0000,   /* U+080D */
  0x0000,   /* U+080E */
  0x0000,   /* U+080F */
  0x0000,   /* U+0810 */
  0x0000,   /* U+0811 */
  0x0000,   /* U+0812 */
  0x0000,   /* U+0813 */
  0x0000,   /* U+0814 */
  0x0000,   /* U+0815 */
  0x0000,   /* U+0816 */
  0x0000,   /* U+0817 */
  0x0000,   /* U+0818 */
  0x0000,   /* U+0819 */
  0x0000,   /* U+081A */
  0x0000,   /* U+081B */
  0x0000,   /* U+081C */
  0x0000,   /* U+081D */
  0x0000,   /* U+081E */
  0x0000,   /* U+081F */
  0x0000,   /* U+0820 */
  0x0000,   /* U+0821 */
  0x0000,   /* U+0822 */
  0x0000,   /* U+0823 */
  0x0000,   /* U+0824 */
  0x0000,   /* U+0825 */
  0x0000,   /* U+0826 */
  0x0000,   /* U+0827 */
  0x0000,   /* U+0828 */
  0x0000,   /* U+0829 */
  0x0000,   /* U+082A */
  0x0000,   /* U+082B */
  0x0000,   /* U+082C */
  0x0000,   /* U+082D */
  0x882E,   /* U+082E */
  0x882F,   /* U+082F */
  0x0000,   /* U+0830 */
  0x0000,   /* U+0831 */
  0x0000,   /* U+0832 */
  0x0000,   /* U+0833 */
  0x0000,   /* U+0834 */
  0x0000,   /* U+0835 */
  0x0000,   /* U+0836 */
  0x0000,   /* U+0837 */
  0x0000,   /* U+0838 */
  0x0000,   /* U+0839 */
  0x0000,   /* U+083A */
  0x0000,   /* U+083B */
  0x0000,   /* U+083C */
  0x0000,   /* U+083D */
  0x0000,   /* U+083E */
  0x883F,   /* U+083F */
  0x0000,   /* U+0840 */
  0x0000,   /* U+0841 */
  0x0000,   /* U+0842 */
  0x0000,   /* U+0843 */
  0x0000,   /* U+0844 */
  0x0000,   /* U+0845 */
  0x0000,   /* U+0846 */
  0x0000,   /* U+0847 */
  0x0000,   /* U+0848 */
  0x0000,   /* U+0849 */
  0x0000,   /* U+084A */
  0x0000,   /* U+084B */
  0x0000,   /* U+084C */
  0x0000,   /* U+084D */
  0x0000,   /* U+084E */
  0x0000,   /* U+084F */
  0x0000,   /* U+0850 */
  0x0000,   /* U+0851 */
  0x0000,   /* U+0852 */
  0x0000,   /* U+0853 */
  0x0000,   /* U+0854 */
  0x0000,   /* U+0855 */
  0x0000,   /* U+0856 */
  0x0000,   /* U+0857 */
  0x0000,   /* U+0858 */
  0x0000,   /* U+0859 */
  0x0000,   /* U+085A */
  0x0000,   /* U+085B */
  0x885C,   /* U+085C */
  0x885D,   /* U+085D */
  0x0000,   /* U+085E */
  0x885F,   /* U+085F */
  0x8860,   /* U+0860 */
  0x8861,   /* U+0861 */
  0x8862,   /* U+0862 */
  0x8863,   /* U+0863 */
  0x8864,   /* U+0864 */
  0x8865,   /* U+0865 */
  0x8866,   /* U+0866 */
  0x8867,   /* U+0867 */
  0x8868,   /* U+0868 */
  0x8869,   /* U+0869 */
  0x886A,   /* U+086A */
  0x886B,   /* U+086B */
  0x886C,   /* U+086C */
  0x886D,   /* U+086D */
  0x886E,   /* U+086E */
  0x886F,   /* U+086F */
  0x8870,   /* U+0870 */
  0x8871,   /* U+0871 */
  0x8872,   /* U+0872 */
  0x8873,   /* U+0873 */
  0x8874,   /* U+0874 */
  0x8875,   /* U+0875 */
  0x8876,   /* U+0876 */
  0x8877,   /* U+0877 */
  0x8878,   /* U+0878 */
  0x8879,   /* U+0879 */
  0x887A,   /* U+087A */
  0x887B,   /* U+087B */
  0x887C,   /* U+087C */
  0x887D,   /* U+087D */
  0x887E,   /* U+087E */
  0x887F,   /* U+087F */
  0x8880,   /* U+0880 */
  0x8881,   /* U+0881 */
  0x8882,   /* U+0882 */
  0x8883,   /* U+0883 */
  0x8884,   /* U+0884 */
  0x8885,   /* U+0885 */
  0x8886,   /* U+0886 */
  0x8887,   /* U+0887 */
  0x8888,   /* U+0888 */
  0x8889,   /* U+0889 */
  0x888A,   /* U+088A */
  0x888B,   /* U+088B */
  0x888C,   /* U+088C */
  0x888D,   /* U+088D */
  0x888E,   /* U+088E */
  0x888F,   /* U+088F */
  0x8890,   /* U+0890 */
  0x8891,   /* U+0891 */
  0x8892,   /* U+0892 */
  0x8893,   /* U+0893 */
  0x8894,   /* U+0894 */
  0x8895,   /* U+0895 */
  0x8896,   /* U+0896 */
  0x8897,   /* U+0897 */
  0x8898,   /* U+0898 */
  0x8899,   /* U+0899 */
  0x889A,   /* U+089A */
  0x889B,   /* U+089B */
  0x889C,   /* U+089C */
  0x889D,   /* U+089D */
  0x889E,   /* U+089E */
  0x889F,   /* U+089F */
  0x0000,   /* U+08A0 */
  0x0000,   /* U+08A1 */
  0x0000,   /* U+08A2 */
  0x0000,   /* U+08A3 */
  0x0000,   /* U+08A4 */
  0x0000,   /* U+08A5 */
  0x0000,   /* U+08A6 */
  0x0000,   /* U+08A7 */
  0x0000,   /* U+08A8 */
  0x0000,   /* U+08A9 */
  0x0000,   /* U+08AA */
  0x0000,   /* U+08AB */
  0x0000,   /* U+08AC */
  0x0000,   /* U+08AD */
  0x0000,   /* U+08AE */
  0x0000,   /* U+08AF */
  0x0000,   /* U+08B0 */
  0x0000,   /* U+08B1 */
  0x0000,   /* U+08B2 */
  0x0000,   /* U+08B3 */
  0x0000,   /* U+08B4 */
  0x88B5,   /* U+08B5 */
  0x0000,   /* U+08B6 */
  0x0000,   /* U+08B7 */
  0x0000,   /* U+08B8 */
  0x0000,   /* U+08B9 */
  0x0000,   /* U+08BA */
  0x0000,   /* U+08BB */
  0x0000,   /* U+08BC */
  0x0000,   /* U+08BD */
  0x88BE,   /* U+08BE */
  0x88BF,   /* U+08BF */
  0x88C0,   /* U+08C0 */
  0x88C1,   /* U+08C1 */
  0x88C2,   /* U+08C2 */
  0x88C3,   /* U+08C3 */
  0x88C4,   /* U+08C4 */
  0x88C5,   /* U+08C5 */
  0x88C6,   /* U+08C6 */
  0x88C7,   /* U+08C7 */
  0x88C8,   /* U+08C8 */
  0x88C9,   /* U+08C9 */
  0x88CA,   /* U+08CA */
  0x88CB,   /* U+08CB */
  0x88CC,   /* U+08CC */
  0x88CD,   /* U+08CD */
  0x88CE,   /* U+08CE */
  0x88CF,   /* U+08CF */
  0x88D0,   /* U+08D0 */
  0x88D1,   /* U+08D1 */
  0x88D2,   /* U+08D2 */
  0x88D3,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0000,   /* U+08E3 */
  0x0000,   /* U+08E4 */
  0x0000,   /* U+08E5 */
  0x0000,   /* U+08E6 */
  0x0000,   /* U+08E7 */
  0x0000,   /* U+08E8 */
  0x0000,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x0000,   /* U+08F0 */
  0x0000,   /* U+08F1 */
  0x0000,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0000,   /* U+08F4 */
  0x0000,   /* U+08F5 */
  0x0000,   /* U+08F6 */
  0x0000,   /* U+08F7 */
  0x0000,   /* U+08F8 */
  0x0000,   /* U+08F9 */
  0x0000,   /* U+08FA */
  0x0000,   /* U+08FB */
  0x0000,   /* U+08FC */
  0x0000,   /* U+08FD */
  0x0000,   /* U+08FE */
  0x0000,   /* U+08FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0800 */
  0x0000,   /* U+0801 */
  0x0000,   /* U+0802 */
  0x0000,   /* U+0803 */
  0x0000,   /* U+0804 */
  0x0000,   /* U+0805 */
  0x0000,   /* U+0806 */
  0x0000,   /* U+0807 */
  0x0000,   /* U+0808 */
  0x0000,   /* U+0809 */
  0x0000,   /* U+080A */
  0x0000,   /* U+080B */
  0x0000,   /* U+080C */
  0x0000,   /* U+080D */
  0x0000,   /* U+080E */
  0x0000,   /* U+080F */
  0x0000,   /* U+0810 */
  0x0000,   /* U+0811 */
  0x0000,   /* U+0812 */
  0x0000,   /* U+0813 */
  0x0000,   /* U+0814 */
  0x0000,   /* U+0815 */
  0x0000,   /* U+0816 */
  0x0000,   /* U+0817 */
  0x0000,   /* U+0818 */
  0x0000,   /* U+0819 */
  0x0000,   /* U+081A */
  0x0000,   /* U+081B */
  0x0000,   /* U+081C */
  0x0000,   /* U+081D */
  0x0000,   /* U+081E */
  0x0000,   /* U+081F */
  0x0000,   /* U+0820 */
  0x0000,   /* U+0821 */
  0x0000,   /* U+0822 */
  0x0000,   /* U+0823 */
  0x0000,   /* U+0824 */
  0x0000,   /* U+0825 */
  0x0000,   /* U+0826 */
  0x0000,   /* U+0827 */
  0x0000,   /* U+0828 */
  0x0000,   /* U+0829 */
  0x0000,   /* U+082A */
  0x0000,   /* U+082B */
  0x0000,   /* U+082C */
  0x0000,   /* U+082D */
  0x0000,   /* U+082E */
  0x0000,   /* U+082F */
  0x0000,   /* U+0830 */
  0x0000,   /* U+0831 */
  0x0000,   /* U+0832 */
  0x0000,   /* U+0833 */
  0x0000,   /* U+0834 */
  0x0000,   /* U+0835 */
  0x0000,   /* U+0836 */
  0x0000,   /* U+0837 */
  0x0000,   /* U+0838 */
  0x0000,   /* U+0839 */
  0x0000,   /* U+083A */
  0x0000,   /* U+083B */
  0x0000,   /* U+083C */
  0x0000,   /* U+083D */
  0x0000,   /* U+083E */
  0x0000,   /* U+083F */
  0x0000,   /* U+0840 */
  0x0000,   /* U+0841 */
  0x0000,   /* U+0842 */
  0x0000,   /* U+0843 */
  0x0000,   /* U+0844 */
  0x0000,   /* U+0845 */
  0x0000,   /* U+0846 */
  0x0000,   /* U+0847 */
  0x0000,   /* U+0848 */
  0x0000,   /* U+0849 */
  0x0000,   /* U+084A */
  0x0000,   /* U+084B */
  0x0000,   /* U+084C */
  0x0000,   /* U+084D */
  0x0000,   /* U+084E */
  0x0000,   /* U+084F */
  0x0000,   /* U+0850 */
  0x0000,   /* U+0851 */
  0x0000,   /* U+0852 */
  0x0000,   /* U+0853 */
  0x0000,   /* U+0854 */
  0x0000,   /* U+0855 */
  0x0000,   /* U+0856 */
  0x0000,   /* U+0857 */
  0x0000,   /* U+0858 */
  0x0000,   /* U+0859 */
  0x0000,   /* U+085A */
  0x0000,   /* U+085B */
  0x0000,   /* U+085C */
  0x0000,   /* U+085D */
  0x0000,   /* U+085E */
  0x0000,   /* U+085F */
  0x0000,   /* U+0860 */
  0x0000,   /* U+0861 */
  0x0000,   /* U+0862 */
  0x0000,   /* U+0863 */
  0x0000,   /* U+0864 */
  0x0000,   /* U+0865 */
  0x0000,   /* U+0866 */
  0x0000,   /* U+0867 */
  0x0000,   /* U+0868 */
  0x0000,   /* U+0869 */
  0x0000,   /* U+086A */
  0x0000,   /* U+086B */
  0x0000,   /* U+086C */
  0x0000,   /* U+086D */
  0x0000,   /* U+086E */
  0x0000,   /* U+086F */
  0x0000,   /* U+0870 */
  0x0000,   /* U+0871 */
  0x0000,   /* U+0872 */
  0x0000,   /* U+0873 */
  0x0000,   /* U+0874 */
  0x0000,   /* U+0875 */
  0x0000,   /* U+0876 */
  0x0000,   /* U+0877 */
  0x0000,   /* U+0878 */
  0x0000,   /* U+0879 */
  0x0000,   /* U+087A */
  0x0000,   /* U+087B */
  0x0000,   /* U+087C */
  0x0000,   /* U+087D */
  0x0000,   /* U+087E */
  0x0000,   /* U+087F */
  0x0000,   /* U+0880 */
  0x0000,   /* U+0881 */
  0x0000,   /* U+0882 */
  0x0000,   /* U+0883 */
  0x0000,   /* U+0884 */
  0x0000,   /* U+0885 */
  0x0000,   /* U+0886 */
  0x0000,   /* U+0887 */
  0x0000,   /* U+0888 */
  0x0000,   /* U+0889 */
  0x0000,   /* U+088A */
  0x0000,   /* U+088B */
  0x0000,   /* U+088C */
  0x0000,   /* U+088D */
  0x0000,   /* U+088E */
  0x0000,   /* U+088F */
  0x0000,   /* U+0890 */
  0x0000,   /* U+0891 */
  0x0000,   /* U+0892 */
  0x0000,   /* U+0893 */
  0x0000,   /* U+0894 */
  0x0000,   /* U+0895 */
  0x0000,   /* U+0896 */
  0x0000,   /* U+0897 */
  0x0000,   /* U+0898 */
  0x0000,   /* U+0899 */
  0x0000,   /* U+089A */
  0x0000,   /* U+089B */
  0x0000,   /* U+089C */
  0x0000,   /* U+089D */
  0x0000,   /* U+089E */
  0x0000,   /* U+089F */
  0x0000,   /* U+08A0 */
  0x0000,   /* U+08A1 */
  0x0000,   /* U+08A2 */
  0x0000,   /* U+08A3 */
  0x0000,   /* U+08A4 */
  0x0000,   /* U+08A5 */
  0x0000,   /* U+08A6 */
  0x0000,   /* U+08A7 */
  0x0000,   /* U+08A8 */
  0x0000,   /* U+08A9 */
  0x0000,   /* U+08AA */
  0x0000,   /* U+08AB */
  0x0000,   /* U+08AC */
  0x0000,   /* U+08AD */
  0x0000,   /* U+08AE */
  0x0000,   /* U+08AF */
  0x0000,   /* U+08B0 */
  0x0000,   /* U+08B1 */
  0x0000,   /* U+08B2 */
  0x0000,   /* U+08B3 */
  0x0000,   /* U+08B4 */
  0x0000,   /* U+08B5 */
  0x0000,   /* U+08B6 */
  0x0000,   /* U+08B7 */
  0x0000,   /* U+08B8 */
  0x0000,   /* U+08B9 */
  0x0000,   /* U+08BA */
  0x0000,   /* U+08BB */
  0x0000,   /* U+08BC */
  0x0000,   /* U+08BD */
  0x0000,   /* U+08BE */
  0x0000,   /* U+08BF */
  0x0000,   /* U+08C0 */
  0x0000,   /* U+08C1 */
  0x0000,   /* U+08C2 */
  0x0000,   /* U+08C3 */
  0x0000,   /* U+08C4 */
  0x0000,   /* U+08C5 */
  0x0000,   /* U+08C6 */
  0x0000,   /* U+08C7 */
  0x0000,   /* U+08C8 */
  0x0000,   /* U+08C9 */
  0x0000,   /* U+08CA */
  0x0000,   /* U+08CB */
  0x0000,   /* U+08CC */
  0x0000,   /* U+08CD */
  0x0000,   /* U+08CE */
  0x0000,   /* U+08CF */
  0x0000,   /* U+08D0 */
  0x0000,   /* U+08D1 */
  0x0000,   /* U+08D2 */
  0x0000,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0000,   /* U+08E3 */
  0x0000,   /* U+08E4 */
  0x0000,   /* U+08E5 */
  0x0000,   /* U+08E6 */
  0x0000,   /* U+08E7 */
  0x0000,   /* U+08E8 */
  0x0000,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x0000,   /* U+08F0 */
  0x0000,   /* U+08F1 */
  0x0000,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0000,   /* U+08F4 */
  0x0000,   /* U+08F5 */
  0x0000,   /* U+08F6 */
  0x0000,   /* U+08F7 */
  0x0000,   /* U+08F8 */
  0x0000,   /* U+08F9 */
  0x0000,   /* U+08FA */
  0x0000,   /* U+08FB */
  0x0000,   /* U+08FC */
  0x0000,   /* U+08FD */
  0x0000,   /* U+08FE */
  0x0000,   /* U+08FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0800 */
  0x0000,   /* U+0801 */
  0x0000,   /* U+0802 */
  0x0000,   /* U+0803 */
  0x0000,   /* U+0804 */
  0x0000,   /* U+0805 */
  0x0000,   /* U+0806 */
  0x0000,   /* U+0807 */
  0x0000,   /* U+0808 */
  0x0000,   /* U+0809 */
  0x0000,   /* U+080A */
  0x0000,   /* U+080B */
  0x0000,   /* U+080C */
  0x0000,   /* U+080D */
  0x0000,   /* U+080E */
  0x0000,   /* U+080F */
  0x0000,   /* U+0810 */
  0x0000,   /* U+0811 */
  0x0000,   /* U+0812 */
  0x0000,   /* U+0813 */
  0x0000,   /* U+0814 */
  0x0000,   /* U+0815 */
  0x0000,   /* U+0816 */
  0x0000,   /* U+0817 */
  0x0000,   /* U+0818 */
  0x0000,   /* U+0819 */
  0x0000,   /* U+081A */
  0x0000,   /* U+081B */
  0x0000,   /* U+081C */
  0x0000,   /* U+081D */
  0x0000,   /* U+081E */
  0x0000,   /* U+081F */
  0x0000,   /* U+0820 */
  0x0000,   /* U+0821 */
  0x0000,   /* U+0822 */
  0x0000,   /* U+0823 */
  0x0000,   /* U+0824 */
  0x0000,   /* U+0825 */
  0x0000,   /* U+0826 */
  0x0000,   /* U+0827 */
  0x0000,   /* U+0828 */
  0x0000,   /* U+0829 */
  0x0000,   /* U+082A */
  0x0000,   /* U+082B */
  0x0000,   /* U+082C */
  0x0000,   /* U+082D */
  0x0000,   /* U+082E */
  0x0000,   /* U+082F */
  0x0000,   /* U+0830 */
  0x0000,   /* U+0831 */
  0x0000,   /* U+0832 */
  0x0000,   /* U+0833 */
  0x0000,   /* U+0834 */
  0x0000,   /* U+0835 */
  0x0000,   /* U+0836 */
  0x0000,   /* U+0837 */
  0x0000,   /* U+0838 */
  0x0000,   /* U+0839 */
  0x0000,   /* U+083A */
  0x0000,   /* U+083B */
  0x0000,   /* U+083C */
  0x0000,   /* U+083D */
  0x0000,   /* U+083E */
  0x0000,   /* U+083F */
  0x0000,   /* U+0840 */
  0x0000,   /* U+0841 */
  0x0000,   /* U+0842 */
  0x0000,   /* U+0843 */
  0x0000,   /* U+0844 */
  0x0000,   /* U+0845 */
  0x0000,   /* U+0846 */
  0x0000,   /* U+0847 */
  0x0000,   /* U+0848 */
  0x0000,   /* U+0849 */
  0x0000,   /* U+084A */
  0x0000,   /* U+084B */
  0x0000,   /* U+084C */
  0x0000,   /* U+084D */
  0x0000,   /* U+084E */
  0x0000,   /* U+084F */
  0x0000,   /* U+0850 */
  0x0000,   /* U+0851 */
  0x0000,   /* U+0852 */
  0x0000,   /* U+0853 */
  0x0000,   /* U+0854 */
  0x0000,   /* U+0855 */
  0x0000,   /* U+0856 */
  0x0000,   /* U+0857 */
  0x0000,   /* U+0858 */
  0x0000,   /* U+0859 */
  0x0000,   /* U+085A */
  0x0000,   /* U+085B */
  0x0000,   /* U+085C */
  0x0000,   /* U+085D */
  0x0000,   /* U+085E */
  0x0000,   /* U+085F */
  0x0000,   /* U+0860 */
  0x0000,   /* U+0861 */
  0x0000,   /* U+0862 */
  0x0000,   /* U+0863 */
  0x0000,   /* U+0864 */
  0x0000,   /* U+0865 */
  0x0000,   /* U+0866 */
  0x0000,   /* U+0867 */
  0x0000,   /* U+0868 */
  0x0000,   /* U+0869 */
  0x0000,   /* U+086A */
  0x0000,   /* U+086B */
  0x0000,   /* U+086C */
  0x0000,   /* U+086D */
  0x0000,   /* U+086E */
  0x0000,   /* U+086F */
  0x0000,   /* U+0870 */
  0x0000,   /* U+0871 */
  0x0000,   /* U+0872 */
  0x0000,   /* U+0873 */
  0x0000,   /* U+0874 */
  0x0000,   /* U+0875 */
  0x0000,   /* U+0876 */
  0x0000,   /* U+0877 */
  0x0000,   /* U+0878 */
  0x0000,   /* U+0879 */
  0x0000,   /* U+087A */
  0x0000,   /* U+087B */
  0x0000,   /* U+087C */
  0x0000,   /* U+087D */
  0x0000,   /* U+087E */
  0x0000,   /* U+087F */
  0x0000,   /* U+0880 */
  0x0000,   /* U+0881 */
  0x0000,   /* U+0882 */
  0x0000,   /* U+0883 */
  0x0000,   /* U+0884 */
  0x0000,   /* U+0885 */
  0x0000,   /* U+0886 */
  0x0000,   /* U+0887 */
  0x0000,   /* U+0888 */
  0x0000,   /* U+0889 */
  0x0000,   /* U+088A */
  0x0000,   /* U+088B */
  0x0000,   /* U+088C */
  0x0000,   /* U+088D */
  0x0000,   /* U+088E */
  0x0000,   /* U+088F */
  0x0000,   /* U+0890 */
  0x0000,   /* U+0891 */
  0x0000,   /* U+0892 */
  0x0000,   /* U+0893 */
  0x0000,   /* U+0894 */
  0x0000,   /* U+0895 */
  0x0000,   /* U+0896 */
  0x0000,   /* U+0897 */
  0x0000,   /* U+0898 */
  0x0000,   /* U+0899 */
  0x0000,   /* U+089A */
  0x0000,   /* U+089B */
  0x0000,   /* U+089C */
  0x0000,   /* U+089D */
  0x0000,   /* U+089E */
  0x0000,   /* U+089F */
  0x0000,   /* U+08A0 */
  0x0000,   /* U+08A1 */
  0x0000,   /* U+08A2 */
  0x0000,   /* U+08A3 */
  0x0000,   /* U+08A4 */
  0x0000,   /* U+08A5 */
  0x0000,   /* U+08A6 */
  0x0000,   /* U+08A7 */
  0x0000,   /* U+08A8 */
  0x0000,   /* U+08A9 */
  0x0000,   /* U+08AA */
  0x0000,   /* U+08AB */
  0x0000,   /* U+08AC */
  0x0000,   /* U+08AD */
  0x0000,   /* U+08AE */
  0x0000,   /* U+08AF */
  0x0000,   /* U+08B0 */
  0x0000,   /* U+08B1 */
  0x0000,   /* U+08B2 */
  0x0000,   /* U+08B3 */
  0x0000,   /* U+08B4 */
  0x0000,   /* U+08B5 */
  0x0000,   /* U+08B6 */
  0x0000,   /* U+08B7 */
  0x0000,   /* U+08B8 */
  0x0000,   /* U+08B9 */
  0x0000,   /* U+08BA */
  0x0000,   /* U+08BB */
  0x0000,   /* U+08BC */
  0x0000,   /* U+08BD */
  0x0000,   /* U+08BE */
  0x0000,   /* U+08BF */
  0x0000,   /* U+08C0 */
  0x0000,   /* U+08C1 */
  0x0000,   /* U+08C2 */
  0x0000,   /* U+08C3 */
  0x0000,   /* U+08C4 */
  0x0000,   /* U+08C5 */
  0x0000,   /* U+08C6 */
  0x0000,   /* U+08C7 */
  0x0000,   /* U+08C8 */
  0x0000,   /* U+08C9 */
  0x0000,   /* U+08CA */
  0x0000,   /* U+08CB */
  0x0000,   /* U+08CC */
  0x0000,   /* U+08CD */
  0x0000,   /* U+08CE */
  0x0000,   /* U+08CF */
  0x0000,   /* U+08D0 */
  0x0000,   /* U+08D1 */
  0x0000,   /* U+08D2 */
  0x0000,   /* U+08D3 */
  0x0000,   /* U+08D4 */
  0x0000,   /* U+08D5 */
  0x0000,   /* U+08D6 */
  0x0000,   /* U+08D7 */
  0x0000,   /* U+08D8 */
  0x0000,   /* U+08D9 */
  0x0000,   /* U+08DA */
  0x0000,   /* U+08DB */
  0x0000,   /* U+08DC */
  0x0000,   /* U+08DD */
  0x0000,   /* U+08DE */
  0x0000,   /* U+08DF */
  0x0000,   /* U+08E0 */
  0x0000,   /* U+08E1 */
  0x0000,   /* U+08E2 */
  0x0000,   /* U+08E3 */
  0x0000,   /* U+08E4 */
  0x0000,   /* U+08E5 */
  0x0000,   /* U+08E6 */
  0x0000,   /* U+08E7 */
  0x0000,   /* U+08E8 */
  0x0000,   /* U+08E9 */
  0x0000,   /* U+08EA */
  0x0000,   /* U+08EB */
  0x0000,   /* U+08EC */
  0x0000,   /* U+08ED */
  0x0000,   /* U+08EE */
  0x0000,   /* U+08EF */
  0x0000,   /* U+08F0 */
  0x0000,   /* U+08F1 */
  0x0000,   /* U+08F2 */
  0x0000,   /* U+08F3 */
  0x0000,   /* U+08F4 */
  0x0000,   /* U+08F5 */
  0x0000,   /* U+08F6 */
  0x0000,   /* U+08F7 */
  0x0000,   /* U+08F8 */
  0x0000,   /* U+08F9 */
  0x0000,   /* U+08FA */
  0x0000,   /* U+08FB */
  0x0000,   /* U+08FC */
  0x0000,   /* U+08FD */
  0x0000,   /* U+08FE */
  0x0000,   /* U+08FF */
};

uint16 uca900_p009[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 2, 2, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+0900 */
  0x0000,   /* U+0901 */
  0x0000,   /* U+0902 */
  0x0000,   /* U+0903 */
  0x265E,   /* U+0904 */
  0x265F,   /* U+0905 */
  0x2660,   /* U+0906 */
  0x2666,   /* U+0907 */
  0x2667,   /* U+0908 */
  0x2668,   /* U+0909 */
  0x2669,   /* U+090A */
  0x266A,   /* U+090B */
  0x266C,   /* U+090C */
  0x266E,   /* U+090D */
  0x266F,   /* U+090E */
  0x2670,   /* U+090F */
  0x2671,   /* U+0910 */
  0x2672,   /* U+0911 */
  0x2673,   /* U+0912 */
  0x2674,   /* U+0913 */
  0x2675,   /* U+0914 */
  0x2676,   /* U+0915 */
  0x2677,   /* U+0916 */
  0x2678,   /* U+0917 */
  0x267A,   /* U+0918 */
  0x267B,   /* U+0919 */
  0x267C,   /* U+091A */
  0x267D,   /* U+091B */
  0x267E,   /* U+091C */
  0x2681,   /* U+091D */
  0x2682,   /* U+091E */
  0x2683,   /* U+091F */
  0x2684,   /* U+0920 */
  0x2686,   /* U+0921 */
  0x2688,   /* U+0922 */
  0x2689,   /* U+0923 */
  0x268A,   /* U+0924 */
  0x268B,   /* U+0925 */
  0x268C,   /* U+0926 */
  0x268D,   /* U+0927 */
  0x268E,   /* U+0928 */
  0x268E,   /* U+0929 */
  0x268F,   /* U+092A */
  0x2690,   /* U+092B */
  0x2691,   /* U+092C */
  0x2693,   /* U+092D */
  0x2694,   /* U+092E */
  0x2695,   /* U+092F */
  0x2697,   /* U+0930 */
  0x2697,   /* U+0931 */
  0x2698,   /* U+0932 */
  0x2699,   /* U+0933 */
  0x2699,   /* U+0934 */
  0x269A,   /* U+0935 */
  0x269B,   /* U+0936 */
  0x269C,   /* U+0937 */
  0x269D,   /* U+0938 */
  0x269E,   /* U+0939 */
  0x26A7,   /* U+093A */
  0x26A8,   /* U+093B */
  0x0000,   /* U+093C */
  0x269F,   /* U+093D */
  0x26A6,   /* U+093E */
  0x26AC,   /* U+093F */
  0x26AD,   /* U+0940 */
  0x26AE,   /* U+0941 */
  0x26AF,   /* U+0942 */
  0x26B0,   /* U+0943 */
  0x26B1,   /* U+0944 */
  0x26B4,   /* U+0945 */
  0x26B6,   /* U+0946 */
  0x26B7,   /* U+0947 */
  0x26B9,   /* U+0948 */
  0x26BA,   /* U+0949 */
  0x26BB,   /* U+094A */
  0x26BC,   /* U+094B */
  0x26BD,   /* U+094C */
  0x26BE,   /* U+094D */
  0x26B8,   /* U+094E */
  0x26A9,   /* U+094F */
  0x265B,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0000,   /* U+0953 */
  0x0000,   /* U+0954 */
  0x26B5,   /* U+0955 */
  0x26AA,   /* U+0956 */
  0x26AB,   /* U+0957 */
  0x2676,   /* U+0958 */
  0x2677,   /* U+0959 */
  0x2678,   /* U+095A */
  0x267E,   /* U+095B */
  0x2686,   /* U+095C */
  0x2688,   /* U+095D */
  0x2690,   /* U+095E */
  0x2695,   /* U+095F */
  0x266B,   /* U+0960 */
  0x266D,   /* U+0961 */
  0x26B2,   /* U+0962 */
  0x26B3,   /* U+0963 */
  0x028E,   /* U+0964 */
  0x028F,   /* U+0965 */
  0x1C3D,   /* U+0966 */
  0x1C3E,   /* U+0967 */
  0x1C3F,   /* U+0968 */
  0x1C40,   /* U+0969 */
  0x1C41,   /* U+096A */
  0x1C42,   /* U+096B */
  0x1C43,   /* U+096C */
  0x1C44,   /* U+096D */
  0x1C45,   /* U+096E */
  0x1C46,   /* U+096F */
  0x03E7,   /* U+0970 */
  0x1BFA,   /* U+0971 */
  0x265D,   /* U+0972 */
  0x2661,   /* U+0973 */
  0x2662,   /* U+0974 */
  0x2663,   /* U+0975 */
  0x2664,   /* U+0976 */
  0x2665,   /* U+0977 */
  0x2685,   /* U+0978 */
  0x267F,   /* U+0979 */
  0x2696,   /* U+097A */
  0x2679,   /* U+097B */
  0x2680,   /* U+097C */
  0x26A0,   /* U+097D */
  0x2687,   /* U+097E */
  0x2692,   /* U+097F */
  0x26BF,   /* U+0980 */
  0x0000,   /* U+0981 */
  0x0000,   /* U+0982 */
  0x0000,   /* U+0983 */
  0xFBC0,   /* U+0984 */
  0x26C0,   /* U+0985 */
  0x26C1,   /* U+0986 */
  0x26C2,   /* U+0987 */
  0x26C3,   /* U+0988 */
  0x26C4,   /* U+0989 */
  0x26C5,   /* U+098A */
  0x26C6,   /* U+098B */
  0x26C8,   /* U+098C */
  0xFBC0,   /* U+098D */
  0xFBC0,   /* U+098E */
  0x26CA,   /* U+098F */
  0x26CB,   /* U+0990 */
  0xFBC0,   /* U+0991 */
  0xFBC0,   /* U+0992 */
  0x26CC,   /* U+0993 */
  0x26CD,   /* U+0994 */
  0x26CE,   /* U+0995 */
  0x26CF,   /* U+0996 */
  0x26D0,   /* U+0997 */
  0x26D1,   /* U+0998 */
  0x26D2,   /* U+0999 */
  0x26D3,   /* U+099A */
  0x26D4,   /* U+099B */
  0x26D5,   /* U+099C */
  0x26D6,   /* U+099D */
  0x26D7,   /* U+099E */
  0x26D8,   /* U+099F */
  0x26D9,   /* U+09A0 */
  0x26DA,   /* U+09A1 */
  0x26DB,   /* U+09A2 */
  0x26DC,   /* U+09A3 */
  0x26DD,   /* U+09A4 */
  0x26DE,   /* U+09A5 */
  0x26DF,   /* U+09A6 */
  0x26E0,   /* U+09A7 */
  0x26E1,   /* U+09A8 */
  0xFBC0,   /* U+09A9 */
  0x26E2,   /* U+09AA */
  0x26E3,   /* U+09AB */
  0x26E4,   /* U+09AC */
  0x26E5,   /* U+09AD */
  0x26E6,   /* U+09AE */
  0x26E7,   /* U+09AF */
  0x26E8,   /* U+09B0 */
  0xFBC0,   /* U+09B1 */
  0x26EA,   /* U+09B2 */
  0xFBC0,   /* U+09B3 */
  0xFBC0,   /* U+09B4 */
  0xFBC0,   /* U+09B5 */
  0x26EC,   /* U+09B6 */
  0x26ED,   /* U+09B7 */
  0x26EE,   /* U+09B8 */
  0x26EF,   /* U+09B9 */
  0xFBC0,   /* U+09BA */
  0xFBC0,   /* U+09BB */
  0x0000,   /* U+09BC */
  0x26F0,   /* U+09BD */
  0x26F1,   /* U+09BE */
  0x26F2,   /* U+09BF */
  0x26F3,   /* U+09C0 */
  0x26F4,   /* U+09C1 */
  0x26F5,   /* U+09C2 */
  0x26F6,   /* U+09C3 */
  0x26F7,   /* U+09C4 */
  0xFBC0,   /* U+09C5 */
  0xFBC0,   /* U+09C6 */
  0x26FA,   /* U+09C7 */
  0x26FB,   /* U+09C8 */
  0xFBC0,   /* U+09C9 */
  0xFBC0,   /* U+09CA */
  0x26FC,   /* U+09CB */
  0x26FD,   /* U+09CC */
  0x26FE,   /* U+09CD */
  0x26DD,   /* U+09CE */
  0xFBC0,   /* U+09CF */
  0xFBC0,   /* U+09D0 */
  0xFBC0,   /* U+09D1 */
  0xFBC0,   /* U+09D2 */
  0xFBC0,   /* U+09D3 */
  0xFBC0,   /* U+09D4 */
  0xFBC0,   /* U+09D5 */
  0xFBC0,   /* U+09D6 */
  0x26FF,   /* U+09D7 */
  0xFBC0,   /* U+09D8 */
  0xFBC0,   /* U+09D9 */
  0xFBC0,   /* U+09DA */
  0xFBC0,   /* U+09DB */
  0x26DA,   /* U+09DC */
  0x26DB,   /* U+09DD */
  0xFBC0,   /* U+09DE */
  0x26E7,   /* U+09DF */
  0x26C7,   /* U+09E0 */
  0x26C9,   /* U+09E1 */
  0x26F8,   /* U+09E2 */
  0x26F9,   /* U+09E3 */
  0xFBC0,   /* U+09E4 */
  0xFBC0,   /* U+09E5 */
  0x1C3D,   /* U+09E6 */
  0x1C3E,   /* U+09E7 */
  0x1C3F,   /* U+09E8 */
  0x1C40,   /* U+09E9 */
  0x1C41,   /* U+09EA */
  0x1C42,   /* U+09EB */
  0x1C43,   /* U+09EC */
  0x1C44,   /* U+09ED */
  0x1C45,   /* U+09EE */
  0x1C46,   /* U+09EF */
  0x26E9,   /* U+09F0 */
  0x26EB,   /* U+09F1 */
  0x1C17,   /* U+09F2 */
  0x1C18,   /* U+09F3 */
  0x1A96,   /* U+09F4 */
  0x1A97,   /* U+09F5 */
  0x1A98,   /* U+09F6 */
  0x1A99,   /* U+09F7 */
  0x1A9A,   /* U+09F8 */
  0x1A9B,   /* U+09F9 */
  0x0513,   /* U+09FA */
  0x1C19,   /* U+09FB */
  0xFBC0,   /* U+09FC */
  0xFBC0,   /* U+09FD */
  0xFBC0,   /* U+09FE */
  0xFBC0,   /* U+09FF */

  /* Secondary weight 1 for each character. */
  0x00C3,   /* U+0900 */
  0x00C3,   /* U+0901 */
  0x00C4,   /* U+0902 */
  0x00C5,   /* U+0903 */
  0x0020,   /* U+0904 */
  0x0020,   /* U+0905 */
  0x0020,   /* U+0906 */
  0x0020,   /* U+0907 */
  0x0020,   /* U+0908 */
  0x0020,   /* U+0909 */
  0x0020,   /* U+090A */
  0x0020,   /* U+090B */
  0x0020,   /* U+090C */
  0x0020,   /* U+090D */
  0x0020,   /* U+090E */
  0x0020,   /* U+090F */
  0x0020,   /* U+0910 */
  0x0020,   /* U+0911 */
  0x0020,   /* U+0912 */
  0x0020,   /* U+0913 */
  0x0020,   /* U+0914 */
  0x0020,   /* U+0915 */
  0x0020,   /* U+0916 */
  0x0020,   /* U+0917 */
  0x0020,   /* U+0918 */
  0x0020,   /* U+0919 */
  0x0020,   /* U+091A */
  0x0020,   /* U+091B */
  0x0020,   /* U+091C */
  0x0020,   /* U+091D */
  0x0020,   /* U+091E */
  0x0020,   /* U+091F */
  0x0020,   /* U+0920 */
  0x0020,   /* U+0921 */
  0x0020,   /* U+0922 */
  0x0020,   /* U+0923 */
  0x0020,   /* U+0924 */
  0x0020,   /* U+0925 */
  0x0020,   /* U+0926 */
  0x0020,   /* U+0927 */
  0x0020,   /* U+0928 */
  0x0020,   /* U+0929 */
  0x0020,   /* U+092A */
  0x0020,   /* U+092B */
  0x0020,   /* U+092C */
  0x0020,   /* U+092D */
  0x0020,   /* U+092E */
  0x0020,   /* U+092F */
  0x0020,   /* U+0930 */
  0x0020,   /* U+0931 */
  0x0020,   /* U+0932 */
  0x0020,   /* U+0933 */
  0x0020,   /* U+0934 */
  0x0020,   /* U+0935 */
  0x0020,   /* U+0936 */
  0x0020,   /* U+0937 */
  0x0020,   /* U+0938 */
  0x0020,   /* U+0939 */
  0x0020,   /* U+093A */
  0x0020,   /* U+093B */
  0x00C2,   /* U+093C */
  0x0020,   /* U+093D */
  0x0020,   /* U+093E */
  0x0020,   /* U+093F */
  0x0020,   /* U+0940 */
  0x0020,   /* U+0941 */
  0x0020,   /* U+0942 */
  0x0020,   /* U+0943 */
  0x0020,   /* U+0944 */
  0x0020,   /* U+0945 */
  0x0020,   /* U+0946 */
  0x0020,   /* U+0947 */
  0x0020,   /* U+0948 */
  0x0020,   /* U+0949 */
  0x0020,   /* U+094A */
  0x0020,   /* U+094B */
  0x0020,   /* U+094C */
  0x0020,   /* U+094D */
  0x0020,   /* U+094E */
  0x0020,   /* U+094F */
  0x0020,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0025,   /* U+0953 */
  0x0024,   /* U+0954 */
  0x0020,   /* U+0955 */
  0x0020,   /* U+0956 */
  0x0020,   /* U+0957 */
  0x0020,   /* U+0958 */
  0x0020,   /* U+0959 */
  0x0020,   /* U+095A */
  0x0020,   /* U+095B */
  0x0020,   /* U+095C */
  0x0020,   /* U+095D */
  0x0020,   /* U+095E */
  0x0020,   /* U+095F */
  0x0020,   /* U+0960 */
  0x0020,   /* U+0961 */
  0x0020,   /* U+0962 */
  0x0020,   /* U+0963 */
  0x0020,   /* U+0964 */
  0x0020,   /* U+0965 */
  0x0020,   /* U+0966 */
  0x0020,   /* U+0967 */
  0x0020,   /* U+0968 */
  0x0020,   /* U+0969 */
  0x0020,   /* U+096A */
  0x0020,   /* U+096B */
  0x0020,   /* U+096C */
  0x0020,   /* U+096D */
  0x0020,   /* U+096E */
  0x0020,   /* U+096F */
  0x0020,   /* U+0970 */
  0x0020,   /* U+0971 */
  0x0020,   /* U+0972 */
  0x0020,   /* U+0973 */
  0x0020,   /* U+0974 */
  0x0020,   /* U+0975 */
  0x0020,   /* U+0976 */
  0x0020,   /* U+0977 */
  0x0020,   /* U+0978 */
  0x0020,   /* U+0979 */
  0x0020,   /* U+097A */
  0x0020,   /* U+097B */
  0x0020,   /* U+097C */
  0x0020,   /* U+097D */
  0x0020,   /* U+097E */
  0x0020,   /* U+097F */
  0x0020,   /* U+0980 */
  0x00C3,   /* U+0981 */
  0x00C4,   /* U+0982 */
  0x00C5,   /* U+0983 */
  0x0020,   /* U+0984 */
  0x0020,   /* U+0985 */
  0x0020,   /* U+0986 */
  0x0020,   /* U+0987 */
  0x0020,   /* U+0988 */
  0x0020,   /* U+0989 */
  0x0020,   /* U+098A */
  0x0020,   /* U+098B */
  0x0020,   /* U+098C */
  0x0020,   /* U+098D */
  0x0020,   /* U+098E */
  0x0020,   /* U+098F */
  0x0020,   /* U+0990 */
  0x0020,   /* U+0991 */
  0x0020,   /* U+0992 */
  0x0020,   /* U+0993 */
  0x0020,   /* U+0994 */
  0x0020,   /* U+0995 */
  0x0020,   /* U+0996 */
  0x0020,   /* U+0997 */
  0x0020,   /* U+0998 */
  0x0020,   /* U+0999 */
  0x0020,   /* U+099A */
  0x0020,   /* U+099B */
  0x0020,   /* U+099C */
  0x0020,   /* U+099D */
  0x0020,   /* U+099E */
  0x0020,   /* U+099F */
  0x0020,   /* U+09A0 */
  0x0020,   /* U+09A1 */
  0x0020,   /* U+09A2 */
  0x0020,   /* U+09A3 */
  0x0020,   /* U+09A4 */
  0x0020,   /* U+09A5 */
  0x0020,   /* U+09A6 */
  0x0020,   /* U+09A7 */
  0x0020,   /* U+09A8 */
  0x0020,   /* U+09A9 */
  0x0020,   /* U+09AA */
  0x0020,   /* U+09AB */
  0x0020,   /* U+09AC */
  0x0020,   /* U+09AD */
  0x0020,   /* U+09AE */
  0x0020,   /* U+09AF */
  0x0020,   /* U+09B0 */
  0x0020,   /* U+09B1 */
  0x0020,   /* U+09B2 */
  0x0020,   /* U+09B3 */
  0x0020,   /* U+09B4 */
  0x0020,   /* U+09B5 */
  0x0020,   /* U+09B6 */
  0x0020,   /* U+09B7 */
  0x0020,   /* U+09B8 */
  0x0020,   /* U+09B9 */
  0x0020,   /* U+09BA */
  0x0020,   /* U+09BB */
  0x00C2,   /* U+09BC */
  0x0020,   /* U+09BD */
  0x0020,   /* U+09BE */
  0x0020,   /* U+09BF */
  0x0020,   /* U+09C0 */
  0x0020,   /* U+09C1 */
  0x0020,   /* U+09C2 */
  0x0020,   /* U+09C3 */
  0x0020,   /* U+09C4 */
  0x0020,   /* U+09C5 */
  0x0020,   /* U+09C6 */
  0x0020,   /* U+09C7 */
  0x0020,   /* U+09C8 */
  0x0020,   /* U+09C9 */
  0x0020,   /* U+09CA */
  0x0020,   /* U+09CB */
  0x0020,   /* U+09CC */
  0x0020,   /* U+09CD */
  0x0020,   /* U+09CE */
  0x0020,   /* U+09CF */
  0x0020,   /* U+09D0 */
  0x0020,   /* U+09D1 */
  0x0020,   /* U+09D2 */
  0x0020,   /* U+09D3 */
  0x0020,   /* U+09D4 */
  0x0020,   /* U+09D5 */
  0x0020,   /* U+09D6 */
  0x0020,   /* U+09D7 */
  0x0020,   /* U+09D8 */
  0x0020,   /* U+09D9 */
  0x0020,   /* U+09DA */
  0x0020,   /* U+09DB */
  0x0020,   /* U+09DC */
  0x0020,   /* U+09DD */
  0x0020,   /* U+09DE */
  0x0020,   /* U+09DF */
  0x0020,   /* U+09E0 */
  0x0020,   /* U+09E1 */
  0x0020,   /* U+09E2 */
  0x0020,   /* U+09E3 */
  0x0020,   /* U+09E4 */
  0x0020,   /* U+09E5 */
  0x0020,   /* U+09E6 */
  0x0020,   /* U+09E7 */
  0x0020,   /* U+09E8 */
  0x0020,   /* U+09E9 */
  0x0020,   /* U+09EA */
  0x0020,   /* U+09EB */
  0x0020,   /* U+09EC */
  0x0020,   /* U+09ED */
  0x0020,   /* U+09EE */
  0x0020,   /* U+09EF */
  0x0020,   /* U+09F0 */
  0x0020,   /* U+09F1 */
  0x0020,   /* U+09F2 */
  0x0020,   /* U+09F3 */
  0x0020,   /* U+09F4 */
  0x0020,   /* U+09F5 */
  0x0020,   /* U+09F6 */
  0x0020,   /* U+09F7 */
  0x0020,   /* U+09F8 */
  0x0020,   /* U+09F9 */
  0x0020,   /* U+09FA */
  0x0020,   /* U+09FB */
  0x0020,   /* U+09FC */
  0x0020,   /* U+09FD */
  0x0020,   /* U+09FE */
  0x0020,   /* U+09FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0900 */
  0x0002,   /* U+0901 */
  0x0002,   /* U+0902 */
  0x0002,   /* U+0903 */
  0x0002,   /* U+0904 */
  0x0002,   /* U+0905 */
  0x0002,   /* U+0906 */
  0x0002,   /* U+0907 */
  0x0002,   /* U+0908 */
  0x0002,   /* U+0909 */
  0x0002,   /* U+090A */
  0x0002,   /* U+090B */
  0x0002,   /* U+090C */
  0x0002,   /* U+090D */
  0x0002,   /* U+090E */
  0x0002,   /* U+090F */
  0x0002,   /* U+0910 */
  0x0002,   /* U+0911 */
  0x0002,   /* U+0912 */
  0x0002,   /* U+0913 */
  0x0002,   /* U+0914 */
  0x0002,   /* U+0915 */
  0x0002,   /* U+0916 */
  0x0002,   /* U+0917 */
  0x0002,   /* U+0918 */
  0x0002,   /* U+0919 */
  0x0002,   /* U+091A */
  0x0002,   /* U+091B */
  0x0002,   /* U+091C */
  0x0002,   /* U+091D */
  0x0002,   /* U+091E */
  0x0002,   /* U+091F */
  0x0002,   /* U+0920 */
  0x0002,   /* U+0921 */
  0x0002,   /* U+0922 */
  0x0002,   /* U+0923 */
  0x0002,   /* U+0924 */
  0x0002,   /* U+0925 */
  0x0002,   /* U+0926 */
  0x0002,   /* U+0927 */
  0x0002,   /* U+0928 */
  0x0002,   /* U+0929 */
  0x0002,   /* U+092A */
  0x0002,   /* U+092B */
  0x0002,   /* U+092C */
  0x0002,   /* U+092D */
  0x0002,   /* U+092E */
  0x0002,   /* U+092F */
  0x0002,   /* U+0930 */
  0x0002,   /* U+0931 */
  0x0002,   /* U+0932 */
  0x0002,   /* U+0933 */
  0x0002,   /* U+0934 */
  0x0002,   /* U+0935 */
  0x0002,   /* U+0936 */
  0x0002,   /* U+0937 */
  0x0002,   /* U+0938 */
  0x0002,   /* U+0939 */
  0x0002,   /* U+093A */
  0x0002,   /* U+093B */
  0x0002,   /* U+093C */
  0x0002,   /* U+093D */
  0x0002,   /* U+093E */
  0x0002,   /* U+093F */
  0x0002,   /* U+0940 */
  0x0002,   /* U+0941 */
  0x0002,   /* U+0942 */
  0x0002,   /* U+0943 */
  0x0002,   /* U+0944 */
  0x0002,   /* U+0945 */
  0x0002,   /* U+0946 */
  0x0002,   /* U+0947 */
  0x0002,   /* U+0948 */
  0x0002,   /* U+0949 */
  0x0002,   /* U+094A */
  0x0002,   /* U+094B */
  0x0002,   /* U+094C */
  0x0002,   /* U+094D */
  0x0002,   /* U+094E */
  0x0002,   /* U+094F */
  0x0002,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0002,   /* U+0953 */
  0x0002,   /* U+0954 */
  0x0002,   /* U+0955 */
  0x0002,   /* U+0956 */
  0x0002,   /* U+0957 */
  0x0002,   /* U+0958 */
  0x0002,   /* U+0959 */
  0x0002,   /* U+095A */
  0x0002,   /* U+095B */
  0x0002,   /* U+095C */
  0x0002,   /* U+095D */
  0x0002,   /* U+095E */
  0x0002,   /* U+095F */
  0x0002,   /* U+0960 */
  0x0002,   /* U+0961 */
  0x0002,   /* U+0962 */
  0x0002,   /* U+0963 */
  0x0002,   /* U+0964 */
  0x0002,   /* U+0965 */
  0x0002,   /* U+0966 */
  0x0002,   /* U+0967 */
  0x0002,   /* U+0968 */
  0x0002,   /* U+0969 */
  0x0002,   /* U+096A */
  0x0002,   /* U+096B */
  0x0002,   /* U+096C */
  0x0002,   /* U+096D */
  0x0002,   /* U+096E */
  0x0002,   /* U+096F */
  0x0002,   /* U+0970 */
  0x0002,   /* U+0971 */
  0x0002,   /* U+0972 */
  0x0002,   /* U+0973 */
  0x0002,   /* U+0974 */
  0x0002,   /* U+0975 */
  0x0002,   /* U+0976 */
  0x0002,   /* U+0977 */
  0x0002,   /* U+0978 */
  0x0002,   /* U+0979 */
  0x0002,   /* U+097A */
  0x0002,   /* U+097B */
  0x0002,   /* U+097C */
  0x0002,   /* U+097D */
  0x0002,   /* U+097E */
  0x0002,   /* U+097F */
  0x0002,   /* U+0980 */
  0x0002,   /* U+0981 */
  0x0002,   /* U+0982 */
  0x0002,   /* U+0983 */
  0x0002,   /* U+0984 */
  0x0002,   /* U+0985 */
  0x0002,   /* U+0986 */
  0x0002,   /* U+0987 */
  0x0002,   /* U+0988 */
  0x0002,   /* U+0989 */
  0x0002,   /* U+098A */
  0x0002,   /* U+098B */
  0x0002,   /* U+098C */
  0x0002,   /* U+098D */
  0x0002,   /* U+098E */
  0x0002,   /* U+098F */
  0x0002,   /* U+0990 */
  0x0002,   /* U+0991 */
  0x0002,   /* U+0992 */
  0x0002,   /* U+0993 */
  0x0002,   /* U+0994 */
  0x0002,   /* U+0995 */
  0x0002,   /* U+0996 */
  0x0002,   /* U+0997 */
  0x0002,   /* U+0998 */
  0x0002,   /* U+0999 */
  0x0002,   /* U+099A */
  0x0002,   /* U+099B */
  0x0002,   /* U+099C */
  0x0002,   /* U+099D */
  0x0002,   /* U+099E */
  0x0002,   /* U+099F */
  0x0002,   /* U+09A0 */
  0x0002,   /* U+09A1 */
  0x0002,   /* U+09A2 */
  0x0002,   /* U+09A3 */
  0x0002,   /* U+09A4 */
  0x0002,   /* U+09A5 */
  0x0002,   /* U+09A6 */
  0x0002,   /* U+09A7 */
  0x0002,   /* U+09A8 */
  0x0002,   /* U+09A9 */
  0x0002,   /* U+09AA */
  0x0002,   /* U+09AB */
  0x0002,   /* U+09AC */
  0x0002,   /* U+09AD */
  0x0002,   /* U+09AE */
  0x0002,   /* U+09AF */
  0x0002,   /* U+09B0 */
  0x0002,   /* U+09B1 */
  0x0002,   /* U+09B2 */
  0x0002,   /* U+09B3 */
  0x0002,   /* U+09B4 */
  0x0002,   /* U+09B5 */
  0x0002,   /* U+09B6 */
  0x0002,   /* U+09B7 */
  0x0002,   /* U+09B8 */
  0x0002,   /* U+09B9 */
  0x0002,   /* U+09BA */
  0x0002,   /* U+09BB */
  0x0002,   /* U+09BC */
  0x0002,   /* U+09BD */
  0x0002,   /* U+09BE */
  0x0002,   /* U+09BF */
  0x0002,   /* U+09C0 */
  0x0002,   /* U+09C1 */
  0x0002,   /* U+09C2 */
  0x0002,   /* U+09C3 */
  0x0002,   /* U+09C4 */
  0x0002,   /* U+09C5 */
  0x0002,   /* U+09C6 */
  0x0002,   /* U+09C7 */
  0x0002,   /* U+09C8 */
  0x0002,   /* U+09C9 */
  0x0002,   /* U+09CA */
  0x0002,   /* U+09CB */
  0x0002,   /* U+09CC */
  0x0002,   /* U+09CD */
  0x0004,   /* U+09CE */
  0x0002,   /* U+09CF */
  0x0002,   /* U+09D0 */
  0x0002,   /* U+09D1 */
  0x0002,   /* U+09D2 */
  0x0002,   /* U+09D3 */
  0x0002,   /* U+09D4 */
  0x0002,   /* U+09D5 */
  0x0002,   /* U+09D6 */
  0x0002,   /* U+09D7 */
  0x0002,   /* U+09D8 */
  0x0002,   /* U+09D9 */
  0x0002,   /* U+09DA */
  0x0002,   /* U+09DB */
  0x0002,   /* U+09DC */
  0x0002,   /* U+09DD */
  0x0002,   /* U+09DE */
  0x0002,   /* U+09DF */
  0x0002,   /* U+09E0 */
  0x0002,   /* U+09E1 */
  0x0002,   /* U+09E2 */
  0x0002,   /* U+09E3 */
  0x0002,   /* U+09E4 */
  0x0002,   /* U+09E5 */
  0x0002,   /* U+09E6 */
  0x0002,   /* U+09E7 */
  0x0002,   /* U+09E8 */
  0x0002,   /* U+09E9 */
  0x0002,   /* U+09EA */
  0x0002,   /* U+09EB */
  0x0002,   /* U+09EC */
  0x0002,   /* U+09ED */
  0x0002,   /* U+09EE */
  0x0002,   /* U+09EF */
  0x0002,   /* U+09F0 */
  0x0002,   /* U+09F1 */
  0x0002,   /* U+09F2 */
  0x0002,   /* U+09F3 */
  0x0002,   /* U+09F4 */
  0x0002,   /* U+09F5 */
  0x0002,   /* U+09F6 */
  0x0002,   /* U+09F7 */
  0x0002,   /* U+09F8 */
  0x0002,   /* U+09F9 */
  0x0002,   /* U+09FA */
  0x0002,   /* U+09FB */
  0x0002,   /* U+09FC */
  0x0002,   /* U+09FD */
  0x0002,   /* U+09FE */
  0x0002,   /* U+09FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0900 */
  0x0000,   /* U+0901 */
  0x0000,   /* U+0902 */
  0x0000,   /* U+0903 */
  0x0000,   /* U+0904 */
  0x0000,   /* U+0905 */
  0x0000,   /* U+0906 */
  0x0000,   /* U+0907 */
  0x0000,   /* U+0908 */
  0x0000,   /* U+0909 */
  0x0000,   /* U+090A */
  0x0000,   /* U+090B */
  0x0000,   /* U+090C */
  0x0000,   /* U+090D */
  0x0000,   /* U+090E */
  0x0000,   /* U+090F */
  0x0000,   /* U+0910 */
  0x0000,   /* U+0911 */
  0x0000,   /* U+0912 */
  0x0000,   /* U+0913 */
  0x0000,   /* U+0914 */
  0x0000,   /* U+0915 */
  0x0000,   /* U+0916 */
  0x0000,   /* U+0917 */
  0x0000,   /* U+0918 */
  0x0000,   /* U+0919 */
  0x0000,   /* U+091A */
  0x0000,   /* U+091B */
  0x0000,   /* U+091C */
  0x0000,   /* U+091D */
  0x0000,   /* U+091E */
  0x0000,   /* U+091F */
  0x0000,   /* U+0920 */
  0x0000,   /* U+0921 */
  0x0000,   /* U+0922 */
  0x0000,   /* U+0923 */
  0x0000,   /* U+0924 */
  0x0000,   /* U+0925 */
  0x0000,   /* U+0926 */
  0x0000,   /* U+0927 */
  0x0000,   /* U+0928 */
  0x0000,   /* U+0929 */
  0x0000,   /* U+092A */
  0x0000,   /* U+092B */
  0x0000,   /* U+092C */
  0x0000,   /* U+092D */
  0x0000,   /* U+092E */
  0x0000,   /* U+092F */
  0x0000,   /* U+0930 */
  0x0000,   /* U+0931 */
  0x0000,   /* U+0932 */
  0x0000,   /* U+0933 */
  0x0000,   /* U+0934 */
  0x0000,   /* U+0935 */
  0x0000,   /* U+0936 */
  0x0000,   /* U+0937 */
  0x0000,   /* U+0938 */
  0x0000,   /* U+0939 */
  0x0000,   /* U+093A */
  0x0000,   /* U+093B */
  0x0000,   /* U+093C */
  0x0000,   /* U+093D */
  0x0000,   /* U+093E */
  0x0000,   /* U+093F */
  0x0000,   /* U+0940 */
  0x0000,   /* U+0941 */
  0x0000,   /* U+0942 */
  0x0000,   /* U+0943 */
  0x0000,   /* U+0944 */
  0x0000,   /* U+0945 */
  0x0000,   /* U+0946 */
  0x0000,   /* U+0947 */
  0x0000,   /* U+0948 */
  0x0000,   /* U+0949 */
  0x0000,   /* U+094A */
  0x0000,   /* U+094B */
  0x0000,   /* U+094C */
  0x0000,   /* U+094D */
  0x0000,   /* U+094E */
  0x0000,   /* U+094F */
  0x0000,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0000,   /* U+0953 */
  0x0000,   /* U+0954 */
  0x0000,   /* U+0955 */
  0x0000,   /* U+0956 */
  0x0000,   /* U+0957 */
  0x0000,   /* U+0958 */
  0x0000,   /* U+0959 */
  0x0000,   /* U+095A */
  0x0000,   /* U+095B */
  0x0000,   /* U+095C */
  0x0000,   /* U+095D */
  0x0000,   /* U+095E */
  0x0000,   /* U+095F */
  0x0000,   /* U+0960 */
  0x0000,   /* U+0961 */
  0x0000,   /* U+0962 */
  0x0000,   /* U+0963 */
  0x0000,   /* U+0964 */
  0x0000,   /* U+0965 */
  0x0000,   /* U+0966 */
  0x0000,   /* U+0967 */
  0x0000,   /* U+0968 */
  0x0000,   /* U+0969 */
  0x0000,   /* U+096A */
  0x0000,   /* U+096B */
  0x0000,   /* U+096C */
  0x0000,   /* U+096D */
  0x0000,   /* U+096E */
  0x0000,   /* U+096F */
  0x0000,   /* U+0970 */
  0x0000,   /* U+0971 */
  0x0000,   /* U+0972 */
  0x0000,   /* U+0973 */
  0x0000,   /* U+0974 */
  0x0000,   /* U+0975 */
  0x0000,   /* U+0976 */
  0x0000,   /* U+0977 */
  0x0000,   /* U+0978 */
  0x0000,   /* U+0979 */
  0x0000,   /* U+097A */
  0x0000,   /* U+097B */
  0x0000,   /* U+097C */
  0x0000,   /* U+097D */
  0x0000,   /* U+097E */
  0x0000,   /* U+097F */
  0x0000,   /* U+0980 */
  0x0000,   /* U+0981 */
  0x0000,   /* U+0982 */
  0x0000,   /* U+0983 */
  0x8984,   /* U+0984 */
  0x0000,   /* U+0985 */
  0x0000,   /* U+0986 */
  0x0000,   /* U+0987 */
  0x0000,   /* U+0988 */
  0x0000,   /* U+0989 */
  0x0000,   /* U+098A */
  0x0000,   /* U+098B */
  0x0000,   /* U+098C */
  0x898D,   /* U+098D */
  0x898E,   /* U+098E */
  0x0000,   /* U+098F */
  0x0000,   /* U+0990 */
  0x8991,   /* U+0991 */
  0x8992,   /* U+0992 */
  0x0000,   /* U+0993 */
  0x0000,   /* U+0994 */
  0x0000,   /* U+0995 */
  0x0000,   /* U+0996 */
  0x0000,   /* U+0997 */
  0x0000,   /* U+0998 */
  0x0000,   /* U+0999 */
  0x0000,   /* U+099A */
  0x0000,   /* U+099B */
  0x0000,   /* U+099C */
  0x0000,   /* U+099D */
  0x0000,   /* U+099E */
  0x0000,   /* U+099F */
  0x0000,   /* U+09A0 */
  0x0000,   /* U+09A1 */
  0x0000,   /* U+09A2 */
  0x0000,   /* U+09A3 */
  0x0000,   /* U+09A4 */
  0x0000,   /* U+09A5 */
  0x0000,   /* U+09A6 */
  0x0000,   /* U+09A7 */
  0x0000,   /* U+09A8 */
  0x89A9,   /* U+09A9 */
  0x0000,   /* U+09AA */
  0x0000,   /* U+09AB */
  0x0000,   /* U+09AC */
  0x0000,   /* U+09AD */
  0x0000,   /* U+09AE */
  0x0000,   /* U+09AF */
  0x0000,   /* U+09B0 */
  0x89B1,   /* U+09B1 */
  0x0000,   /* U+09B2 */
  0x89B3,   /* U+09B3 */
  0x89B4,   /* U+09B4 */
  0x89B5,   /* U+09B5 */
  0x0000,   /* U+09B6 */
  0x0000,   /* U+09B7 */
  0x0000,   /* U+09B8 */
  0x0000,   /* U+09B9 */
  0x89BA,   /* U+09BA */
  0x89BB,   /* U+09BB */
  0x0000,   /* U+09BC */
  0x0000,   /* U+09BD */
  0x0000,   /* U+09BE */
  0x0000,   /* U+09BF */
  0x0000,   /* U+09C0 */
  0x0000,   /* U+09C1 */
  0x0000,   /* U+09C2 */
  0x0000,   /* U+09C3 */
  0x0000,   /* U+09C4 */
  0x89C5,   /* U+09C5 */
  0x89C6,   /* U+09C6 */
  0x0000,   /* U+09C7 */
  0x0000,   /* U+09C8 */
  0x89C9,   /* U+09C9 */
  0x89CA,   /* U+09CA */
  0x0000,   /* U+09CB */
  0x0000,   /* U+09CC */
  0x0000,   /* U+09CD */
  0x26FE,   /* U+09CE */
  0x89CF,   /* U+09CF */
  0x89D0,   /* U+09D0 */
  0x89D1,   /* U+09D1 */
  0x89D2,   /* U+09D2 */
  0x89D3,   /* U+09D3 */
  0x89D4,   /* U+09D4 */
  0x89D5,   /* U+09D5 */
  0x89D6,   /* U+09D6 */
  0x0000,   /* U+09D7 */
  0x89D8,   /* U+09D8 */
  0x89D9,   /* U+09D9 */
  0x89DA,   /* U+09DA */
  0x89DB,   /* U+09DB */
  0x0000,   /* U+09DC */
  0x0000,   /* U+09DD */
  0x89DE,   /* U+09DE */
  0x0000,   /* U+09DF */
  0x0000,   /* U+09E0 */
  0x0000,   /* U+09E1 */
  0x0000,   /* U+09E2 */
  0x0000,   /* U+09E3 */
  0x89E4,   /* U+09E4 */
  0x89E5,   /* U+09E5 */
  0x0000,   /* U+09E6 */
  0x0000,   /* U+09E7 */
  0x0000,   /* U+09E8 */
  0x0000,   /* U+09E9 */
  0x0000,   /* U+09EA */
  0x0000,   /* U+09EB */
  0x0000,   /* U+09EC */
  0x0000,   /* U+09ED */
  0x0000,   /* U+09EE */
  0x0000,   /* U+09EF */
  0x0000,   /* U+09F0 */
  0x0000,   /* U+09F1 */
  0x0000,   /* U+09F2 */
  0x0000,   /* U+09F3 */
  0x0000,   /* U+09F4 */
  0x0000,   /* U+09F5 */
  0x0000,   /* U+09F6 */
  0x0000,   /* U+09F7 */
  0x0000,   /* U+09F8 */
  0x0000,   /* U+09F9 */
  0x0000,   /* U+09FA */
  0x0000,   /* U+09FB */
  0x89FC,   /* U+09FC */
  0x89FD,   /* U+09FD */
  0x89FE,   /* U+09FE */
  0x89FF,   /* U+09FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0900 */
  0x0000,   /* U+0901 */
  0x0000,   /* U+0902 */
  0x0000,   /* U+0903 */
  0x0000,   /* U+0904 */
  0x0000,   /* U+0905 */
  0x0000,   /* U+0906 */
  0x0000,   /* U+0907 */
  0x0000,   /* U+0908 */
  0x0000,   /* U+0909 */
  0x0000,   /* U+090A */
  0x0000,   /* U+090B */
  0x0000,   /* U+090C */
  0x0000,   /* U+090D */
  0x0000,   /* U+090E */
  0x0000,   /* U+090F */
  0x0000,   /* U+0910 */
  0x0000,   /* U+0911 */
  0x0000,   /* U+0912 */
  0x0000,   /* U+0913 */
  0x0000,   /* U+0914 */
  0x0000,   /* U+0915 */
  0x0000,   /* U+0916 */
  0x0000,   /* U+0917 */
  0x0000,   /* U+0918 */
  0x0000,   /* U+0919 */
  0x0000,   /* U+091A */
  0x0000,   /* U+091B */
  0x0000,   /* U+091C */
  0x0000,   /* U+091D */
  0x0000,   /* U+091E */
  0x0000,   /* U+091F */
  0x0000,   /* U+0920 */
  0x0000,   /* U+0921 */
  0x0000,   /* U+0922 */
  0x0000,   /* U+0923 */
  0x0000,   /* U+0924 */
  0x0000,   /* U+0925 */
  0x0000,   /* U+0926 */
  0x0000,   /* U+0927 */
  0x0000,   /* U+0928 */
  0x00C2,   /* U+0929 */
  0x0000,   /* U+092A */
  0x0000,   /* U+092B */
  0x0000,   /* U+092C */
  0x0000,   /* U+092D */
  0x0000,   /* U+092E */
  0x0000,   /* U+092F */
  0x0000,   /* U+0930 */
  0x00C2,   /* U+0931 */
  0x0000,   /* U+0932 */
  0x0000,   /* U+0933 */
  0x00C2,   /* U+0934 */
  0x0000,   /* U+0935 */
  0x0000,   /* U+0936 */
  0x0000,   /* U+0937 */
  0x0000,   /* U+0938 */
  0x0000,   /* U+0939 */
  0x0000,   /* U+093A */
  0x0000,   /* U+093B */
  0x0000,   /* U+093C */
  0x0000,   /* U+093D */
  0x0000,   /* U+093E */
  0x0000,   /* U+093F */
  0x0000,   /* U+0940 */
  0x0000,   /* U+0941 */
  0x0000,   /* U+0942 */
  0x0000,   /* U+0943 */
  0x0000,   /* U+0944 */
  0x0000,   /* U+0945 */
  0x0000,   /* U+0946 */
  0x0000,   /* U+0947 */
  0x0000,   /* U+0948 */
  0x0000,   /* U+0949 */
  0x0000,   /* U+094A */
  0x0000,   /* U+094B */
  0x0000,   /* U+094C */
  0x0000,   /* U+094D */
  0x0000,   /* U+094E */
  0x0000,   /* U+094F */
  0x0000,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0000,   /* U+0953 */
  0x0000,   /* U+0954 */
  0x0000,   /* U+0955 */
  0x0000,   /* U+0956 */
  0x0000,   /* U+0957 */
  0x00C2,   /* U+0958 */
  0x00C2,   /* U+0959 */
  0x00C2,   /* U+095A */
  0x00C2,   /* U+095B */
  0x00C2,   /* U+095C */
  0x00C2,   /* U+095D */
  0x00C2,   /* U+095E */
  0x00C2,   /* U+095F */
  0x0000,   /* U+0960 */
  0x0000,   /* U+0961 */
  0x0000,   /* U+0962 */
  0x0000,   /* U+0963 */
  0x0000,   /* U+0964 */
  0x0000,   /* U+0965 */
  0x0000,   /* U+0966 */
  0x0000,   /* U+0967 */
  0x0000,   /* U+0968 */
  0x0000,   /* U+0969 */
  0x0000,   /* U+096A */
  0x0000,   /* U+096B */
  0x0000,   /* U+096C */
  0x0000,   /* U+096D */
  0x0000,   /* U+096E */
  0x0000,   /* U+096F */
  0x0000,   /* U+0970 */
  0x0000,   /* U+0971 */
  0x0000,   /* U+0972 */
  0x0000,   /* U+0973 */
  0x0000,   /* U+0974 */
  0x0000,   /* U+0975 */
  0x0000,   /* U+0976 */
  0x0000,   /* U+0977 */
  0x0000,   /* U+0978 */
  0x0000,   /* U+0979 */
  0x0000,   /* U+097A */
  0x0000,   /* U+097B */
  0x0000,   /* U+097C */
  0x0000,   /* U+097D */
  0x0000,   /* U+097E */
  0x0000,   /* U+097F */
  0x0000,   /* U+0980 */
  0x0000,   /* U+0981 */
  0x0000,   /* U+0982 */
  0x0000,   /* U+0983 */
  0x0000,   /* U+0984 */
  0x0000,   /* U+0985 */
  0x0000,   /* U+0986 */
  0x0000,   /* U+0987 */
  0x0000,   /* U+0988 */
  0x0000,   /* U+0989 */
  0x0000,   /* U+098A */
  0x0000,   /* U+098B */
  0x0000,   /* U+098C */
  0x0000,   /* U+098D */
  0x0000,   /* U+098E */
  0x0000,   /* U+098F */
  0x0000,   /* U+0990 */
  0x0000,   /* U+0991 */
  0x0000,   /* U+0992 */
  0x0000,   /* U+0993 */
  0x0000,   /* U+0994 */
  0x0000,   /* U+0995 */
  0x0000,   /* U+0996 */
  0x0000,   /* U+0997 */
  0x0000,   /* U+0998 */
  0x0000,   /* U+0999 */
  0x0000,   /* U+099A */
  0x0000,   /* U+099B */
  0x0000,   /* U+099C */
  0x0000,   /* U+099D */
  0x0000,   /* U+099E */
  0x0000,   /* U+099F */
  0x0000,   /* U+09A0 */
  0x0000,   /* U+09A1 */
  0x0000,   /* U+09A2 */
  0x0000,   /* U+09A3 */
  0x0000,   /* U+09A4 */
  0x0000,   /* U+09A5 */
  0x0000,   /* U+09A6 */
  0x0000,   /* U+09A7 */
  0x0000,   /* U+09A8 */
  0x0000,   /* U+09A9 */
  0x0000,   /* U+09AA */
  0x0000,   /* U+09AB */
  0x0000,   /* U+09AC */
  0x0000,   /* U+09AD */
  0x0000,   /* U+09AE */
  0x0000,   /* U+09AF */
  0x0000,   /* U+09B0 */
  0x0000,   /* U+09B1 */
  0x0000,   /* U+09B2 */
  0x0000,   /* U+09B3 */
  0x0000,   /* U+09B4 */
  0x0000,   /* U+09B5 */
  0x0000,   /* U+09B6 */
  0x0000,   /* U+09B7 */
  0x0000,   /* U+09B8 */
  0x0000,   /* U+09B9 */
  0x0000,   /* U+09BA */
  0x0000,   /* U+09BB */
  0x0000,   /* U+09BC */
  0x0000,   /* U+09BD */
  0x0000,   /* U+09BE */
  0x0000,   /* U+09BF */
  0x0000,   /* U+09C0 */
  0x0000,   /* U+09C1 */
  0x0000,   /* U+09C2 */
  0x0000,   /* U+09C3 */
  0x0000,   /* U+09C4 */
  0x0000,   /* U+09C5 */
  0x0000,   /* U+09C6 */
  0x0000,   /* U+09C7 */
  0x0000,   /* U+09C8 */
  0x0000,   /* U+09C9 */
  0x0000,   /* U+09CA */
  0x0000,   /* U+09CB */
  0x0000,   /* U+09CC */
  0x0000,   /* U+09CD */
  0x0020,   /* U+09CE */
  0x0000,   /* U+09CF */
  0x0000,   /* U+09D0 */
  0x0000,   /* U+09D1 */
  0x0000,   /* U+09D2 */
  0x0000,   /* U+09D3 */
  0x0000,   /* U+09D4 */
  0x0000,   /* U+09D5 */
  0x0000,   /* U+09D6 */
  0x0000,   /* U+09D7 */
  0x0000,   /* U+09D8 */
  0x0000,   /* U+09D9 */
  0x0000,   /* U+09DA */
  0x0000,   /* U+09DB */
  0x00C2,   /* U+09DC */
  0x00C2,   /* U+09DD */
  0x0000,   /* U+09DE */
  0x00C2,   /* U+09DF */
  0x0000,   /* U+09E0 */
  0x0000,   /* U+09E1 */
  0x0000,   /* U+09E2 */
  0x0000,   /* U+09E3 */
  0x0000,   /* U+09E4 */
  0x0000,   /* U+09E5 */
  0x0000,   /* U+09E6 */
  0x0000,   /* U+09E7 */
  0x0000,   /* U+09E8 */
  0x0000,   /* U+09E9 */
  0x0000,   /* U+09EA */
  0x0000,   /* U+09EB */
  0x0000,   /* U+09EC */
  0x0000,   /* U+09ED */
  0x0000,   /* U+09EE */
  0x0000,   /* U+09EF */
  0x0000,   /* U+09F0 */
  0x0000,   /* U+09F1 */
  0x0000,   /* U+09F2 */
  0x0000,   /* U+09F3 */
  0x0000,   /* U+09F4 */
  0x0000,   /* U+09F5 */
  0x0000,   /* U+09F6 */
  0x0000,   /* U+09F7 */
  0x0000,   /* U+09F8 */
  0x0000,   /* U+09F9 */
  0x0000,   /* U+09FA */
  0x0000,   /* U+09FB */
  0x0000,   /* U+09FC */
  0x0000,   /* U+09FD */
  0x0000,   /* U+09FE */
  0x0000,   /* U+09FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0900 */
  0x0000,   /* U+0901 */
  0x0000,   /* U+0902 */
  0x0000,   /* U+0903 */
  0x0000,   /* U+0904 */
  0x0000,   /* U+0905 */
  0x0000,   /* U+0906 */
  0x0000,   /* U+0907 */
  0x0000,   /* U+0908 */
  0x0000,   /* U+0909 */
  0x0000,   /* U+090A */
  0x0000,   /* U+090B */
  0x0000,   /* U+090C */
  0x0000,   /* U+090D */
  0x0000,   /* U+090E */
  0x0000,   /* U+090F */
  0x0000,   /* U+0910 */
  0x0000,   /* U+0911 */
  0x0000,   /* U+0912 */
  0x0000,   /* U+0913 */
  0x0000,   /* U+0914 */
  0x0000,   /* U+0915 */
  0x0000,   /* U+0916 */
  0x0000,   /* U+0917 */
  0x0000,   /* U+0918 */
  0x0000,   /* U+0919 */
  0x0000,   /* U+091A */
  0x0000,   /* U+091B */
  0x0000,   /* U+091C */
  0x0000,   /* U+091D */
  0x0000,   /* U+091E */
  0x0000,   /* U+091F */
  0x0000,   /* U+0920 */
  0x0000,   /* U+0921 */
  0x0000,   /* U+0922 */
  0x0000,   /* U+0923 */
  0x0000,   /* U+0924 */
  0x0000,   /* U+0925 */
  0x0000,   /* U+0926 */
  0x0000,   /* U+0927 */
  0x0000,   /* U+0928 */
  0x0002,   /* U+0929 */
  0x0000,   /* U+092A */
  0x0000,   /* U+092B */
  0x0000,   /* U+092C */
  0x0000,   /* U+092D */
  0x0000,   /* U+092E */
  0x0000,   /* U+092F */
  0x0000,   /* U+0930 */
  0x0002,   /* U+0931 */
  0x0000,   /* U+0932 */
  0x0000,   /* U+0933 */
  0x0002,   /* U+0934 */
  0x0000,   /* U+0935 */
  0x0000,   /* U+0936 */
  0x0000,   /* U+0937 */
  0x0000,   /* U+0938 */
  0x0000,   /* U+0939 */
  0x0000,   /* U+093A */
  0x0000,   /* U+093B */
  0x0000,   /* U+093C */
  0x0000,   /* U+093D */
  0x0000,   /* U+093E */
  0x0000,   /* U+093F */
  0x0000,   /* U+0940 */
  0x0000,   /* U+0941 */
  0x0000,   /* U+0942 */
  0x0000,   /* U+0943 */
  0x0000,   /* U+0944 */
  0x0000,   /* U+0945 */
  0x0000,   /* U+0946 */
  0x0000,   /* U+0947 */
  0x0000,   /* U+0948 */
  0x0000,   /* U+0949 */
  0x0000,   /* U+094A */
  0x0000,   /* U+094B */
  0x0000,   /* U+094C */
  0x0000,   /* U+094D */
  0x0000,   /* U+094E */
  0x0000,   /* U+094F */
  0x0000,   /* U+0950 */
  0x0000,   /* U+0951 */
  0x0000,   /* U+0952 */
  0x0000,   /* U+0953 */
  0x0000,   /* U+0954 */
  0x0000,   /* U+0955 */
  0x0000,   /* U+0956 */
  0x0000,   /* U+0957 */
  0x0002,   /* U+0958 */
  0x0002,   /* U+0959 */
  0x0002,   /* U+095A */
  0x0002,   /* U+095B */
  0x0002,   /* U+095C */
  0x0002,   /* U+095D */
  0x0002,   /* U+095E */
  0x0002,   /* U+095F */
  0x0000,   /* U+0960 */
  0x0000,   /* U+0961 */
  0x0000,   /* U+0962 */
  0x0000,   /* U+0963 */
  0x0000,   /* U+0964 */
  0x0000,   /* U+0965 */
  0x0000,   /* U+0966 */
  0x0000,   /* U+0967 */
  0x0000,   /* U+0968 */
  0x0000,   /* U+0969 */
  0x0000,   /* U+096A */
  0x0000,   /* U+096B */
  0x0000,   /* U+096C */
  0x0000,   /* U+096D */
  0x0000,   /* U+096E */
  0x0000,   /* U+096F */
  0x0000,   /* U+0970 */
  0x0000,   /* U+0971 */
  0x0000,   /* U+0972 */
  0x0000,   /* U+0973 */
  0x0000,   /* U+0974 */
  0x0000,   /* U+0975 */
  0x0000,   /* U+0976 */
  0x0000,   /* U+0977 */
  0x0000,   /* U+0978 */
  0x0000,   /* U+0979 */
  0x0000,   /* U+097A */
  0x0000,   /* U+097B */
  0x0000,   /* U+097C */
  0x0000,   /* U+097D */
  0x0000,   /* U+097E */
  0x0000,   /* U+097F */
  0x0000,   /* U+0980 */
  0x0000,   /* U+0981 */
  0x0000,   /* U+0982 */
  0x0000,   /* U+0983 */
  0x0000,   /* U+0984 */
  0x0000,   /* U+0985 */
  0x0000,   /* U+0986 */
  0x0000,   /* U+0987 */
  0x0000,   /* U+0988 */
  0x0000,   /* U+0989 */
  0x0000,   /* U+098A */
  0x0000,   /* U+098B */
  0x0000,   /* U+098C */
  0x0000,   /* U+098D */
  0x0000,   /* U+098E */
  0x0000,   /* U+098F */
  0x0000,   /* U+0990 */
  0x0000,   /* U+0991 */
  0x0000,   /* U+0992 */
  0x0000,   /* U+0993 */
  0x0000,   /* U+0994 */
  0x0000,   /* U+0995 */
  0x0000,   /* U+0996 */
  0x0000,   /* U+0997 */
  0x0000,   /* U+0998 */
  0x0000,   /* U+0999 */
  0x0000,   /* U+099A */
  0x0000,   /* U+099B */
  0x0000,   /* U+099C */
  0x0000,   /* U+099D */
  0x0000,   /* U+099E */
  0x0000,   /* U+099F */
  0x0000,   /* U+09A0 */
  0x0000,   /* U+09A1 */
  0x0000,   /* U+09A2 */
  0x0000,   /* U+09A3 */
  0x0000,   /* U+09A4 */
  0x0000,   /* U+09A5 */
  0x0000,   /* U+09A6 */
  0x0000,   /* U+09A7 */
  0x0000,   /* U+09A8 */
  0x0000,   /* U+09A9 */
  0x0000,   /* U+09AA */
  0x0000,   /* U+09AB */
  0x0000,   /* U+09AC */
  0x0000,   /* U+09AD */
  0x0000,   /* U+09AE */
  0x0000,   /* U+09AF */
  0x0000,   /* U+09B0 */
  0x0000,   /* U+09B1 */
  0x0000,   /* U+09B2 */
  0x0000,   /* U+09B3 */
  0x0000,   /* U+09B4 */
  0x0000,   /* U+09B5 */
  0x0000,   /* U+09B6 */
  0x0000,   /* U+09B7 */
  0x0000,   /* U+09B8 */
  0x0000,   /* U+09B9 */
  0x0000,   /* U+09BA */
  0x0000,   /* U+09BB */
  0x0000,   /* U+09BC */
  0x0000,   /* U+09BD */
  0x0000,   /* U+09BE */
  0x0000,   /* U+09BF */
  0x0000,   /* U+09C0 */
  0x0000,   /* U+09C1 */
  0x0000,   /* U+09C2 */
  0x0000,   /* U+09C3 */
  0x0000,   /* U+09C4 */
  0x0000,   /* U+09C5 */
  0x0000,   /* U+09C6 */
  0x0000,   /* U+09C7 */
  0x0000,   /* U+09C8 */
  0x0000,   /* U+09C9 */
  0x0000,   /* U+09CA */
  0x0000,   /* U+09CB */
  0x0000,   /* U+09CC */
  0x0000,   /* U+09CD */
  0x0004,   /* U+09CE */
  0x0000,   /* U+09CF */
  0x0000,   /* U+09D0 */
  0x0000,   /* U+09D1 */
  0x0000,   /* U+09D2 */
  0x0000,   /* U+09D3 */
  0x0000,   /* U+09D4 */
  0x0000,   /* U+09D5 */
  0x0000,   /* U+09D6 */
  0x0000,   /* U+09D7 */
  0x0000,   /* U+09D8 */
  0x0000,   /* U+09D9 */
  0x0000,   /* U+09DA */
  0x0000,   /* U+09DB */
  0x0002,   /* U+09DC */
  0x0002,   /* U+09DD */
  0x0000,   /* U+09DE */
  0x0002,   /* U+09DF */
  0x0000,   /* U+09E0 */
  0x0000,   /* U+09E1 */
  0x0000,   /* U+09E2 */
  0x0000,   /* U+09E3 */
  0x0000,   /* U+09E4 */
  0x0000,   /* U+09E5 */
  0x0000,   /* U+09E6 */
  0x0000,   /* U+09E7 */
  0x0000,   /* U+09E8 */
  0x0000,   /* U+09E9 */
  0x0000,   /* U+09EA */
  0x0000,   /* U+09EB */
  0x0000,   /* U+09EC */
  0x0000,   /* U+09ED */
  0x0000,   /* U+09EE */
  0x0000,   /* U+09EF */
  0x0000,   /* U+09F0 */
  0x0000,   /* U+09F1 */
  0x0000,   /* U+09F2 */
  0x0000,   /* U+09F3 */
  0x0000,   /* U+09F4 */
  0x0000,   /* U+09F5 */
  0x0000,   /* U+09F6 */
  0x0000,   /* U+09F7 */
  0x0000,   /* U+09F8 */
  0x0000,   /* U+09F9 */
  0x0000,   /* U+09FA */
  0x0000,   /* U+09FB */
  0x0000,   /* U+09FC */
  0x0000,   /* U+09FD */
  0x0000,   /* U+09FE */
  0x0000,   /* U+09FF */
};

uint16 uca900_p00A[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 2, 2, 1, 2, 2, 1, 1, 2, 2, 1, 2, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 1, 1, 2, 2, 1, 1, 1, 2, 2, 
  2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC0,   /* U+0A00 */
  0x0000,   /* U+0A01 */
  0x0000,   /* U+0A02 */
  0x0000,   /* U+0A03 */
  0xFBC0,   /* U+0A04 */
  0x2705,   /* U+0A05 */
  0x2706,   /* U+0A06 */
  0x270A,   /* U+0A07 */
  0x270B,   /* U+0A08 */
  0x2702,   /* U+0A09 */
  0x2703,   /* U+0A0A */
  0xFBC0,   /* U+0A0B */
  0xFBC0,   /* U+0A0C */
  0xFBC0,   /* U+0A0D */
  0xFBC0,   /* U+0A0E */
  0x270C,   /* U+0A0F */
  0x2707,   /* U+0A10 */
  0xFBC0,   /* U+0A11 */
  0xFBC0,   /* U+0A12 */
  0x2704,   /* U+0A13 */
  0x2708,   /* U+0A14 */
  0x2710,   /* U+0A15 */
  0x2711,   /* U+0A16 */
  0x2712,   /* U+0A17 */
  0x2713,   /* U+0A18 */
  0x2714,   /* U+0A19 */
  0x2715,   /* U+0A1A */
  0x2716,   /* U+0A1B */
  0x2717,   /* U+0A1C */
  0x2718,   /* U+0A1D */
  0x2719,   /* U+0A1E */
  0x271A,   /* U+0A1F */
  0x271B,   /* U+0A20 */
  0x271C,   /* U+0A21 */
  0x271D,   /* U+0A22 */
  0x271E,   /* U+0A23 */
  0x271F,   /* U+0A24 */
  0x2720,   /* U+0A25 */
  0x2721,   /* U+0A26 */
  0x2722,   /* U+0A27 */
  0x2723,   /* U+0A28 */
  0xFBC0,   /* U+0A29 */
  0x2724,   /* U+0A2A */
  0x2725,   /* U+0A2B */
  0x2726,   /* U+0A2C */
  0x2727,   /* U+0A2D */
  0x2728,   /* U+0A2E */
  0x2729,   /* U+0A2F */
  0x272B,   /* U+0A30 */
  0xFBC0,   /* U+0A31 */
  0x272C,   /* U+0A32 */
  0x272C,   /* U+0A33 */
  0xFBC0,   /* U+0A34 */
  0x272D,   /* U+0A35 */
  0x270D,   /* U+0A36 */
  0xFBC0,   /* U+0A37 */
  0x270D,   /* U+0A38 */
  0x270E,   /* U+0A39 */
  0xFBC0,   /* U+0A3A */
  0xFBC0,   /* U+0A3B */
  0x0000,   /* U+0A3C */
  0xFBC0,   /* U+0A3D */
  0x272F,   /* U+0A3E */
  0x2730,   /* U+0A3F */
  0x2731,   /* U+0A40 */
  0x2732,   /* U+0A41 */
  0x2733,   /* U+0A42 */
  0xFBC0,   /* U+0A43 */
  0xFBC0,   /* U+0A44 */
  0xFBC0,   /* U+0A45 */
  0xFBC0,   /* U+0A46 */
  0x2734,   /* U+0A47 */
  0x2735,   /* U+0A48 */
  0xFBC0,   /* U+0A49 */
  0xFBC0,   /* U+0A4A */
  0x2736,   /* U+0A4B */
  0x2737,   /* U+0A4C */
  0x2738,   /* U+0A4D */
  0xFBC0,   /* U+0A4E */
  0xFBC0,   /* U+0A4F */
  0xFBC0,   /* U+0A50 */
  0x270F,   /* U+0A51 */
  0xFBC0,   /* U+0A52 */
  0xFBC0,   /* U+0A53 */
  0xFBC0,   /* U+0A54 */
  0xFBC0,   /* U+0A55 */
  0xFBC0,   /* U+0A56 */
  0xFBC0,   /* U+0A57 */
  0xFBC0,   /* U+0A58 */
  0x2711,   /* U+0A59 */
  0x2712,   /* U+0A5A */
  0x2717,   /* U+0A5B */
  0x272E,   /* U+0A5C */
  0xFBC0,   /* U+0A5D */
  0x2725,   /* U+0A5E */
  0xFBC0,   /* U+0A5F */
  0xFBC0,   /* U+0A60 */
  0xFBC0,   /* U+0A61 */
  0xFBC0,   /* U+0A62 */
  0xFBC0,   /* U+0A63 */
  0xFBC0,   /* U+0A64 */
  0xFBC0,   /* U+0A65 */
  0x1C3D,   /* U+0A66 */
  0x1C3E,   /* U+0A67 */
  0x1C3F,   /* U+0A68 */
  0x1C40,   /* U+0A69 */
  0x1C41,   /* U+0A6A */
  0x1C42,   /* U+0A6B */
  0x1C43,   /* U+0A6C */
  0x1C44,   /* U+0A6D */
  0x1C45,   /* U+0A6E */
  0x1C46,   /* U+0A6F */
  0x0000,   /* U+0A70 */
  0x0000,   /* U+0A71 */
  0x2709,   /* U+0A72 */
  0x2701,   /* U+0A73 */
  0x2700,   /* U+0A74 */
  0x272A,   /* U+0A75 */
  0xFBC0,   /* U+0A76 */
  0xFBC0,   /* U+0A77 */
  0xFBC0,   /* U+0A78 */
  0xFBC0,   /* U+0A79 */
  0xFBC0,   /* U+0A7A */
  0xFBC0,   /* U+0A7B */
  0xFBC0,   /* U+0A7C */
  0xFBC0,   /* U+0A7D */
  0xFBC0,   /* U+0A7E */
  0xFBC0,   /* U+0A7F */
  0xFBC0,   /* U+0A80 */
  0x0000,   /* U+0A81 */
  0x0000,   /* U+0A82 */
  0x0000,   /* U+0A83 */
  0xFBC0,   /* U+0A84 */
  0x273A,   /* U+0A85 */
  0x273B,   /* U+0A86 */
  0x273C,   /* U+0A87 */
  0x273D,   /* U+0A88 */
  0x273E,   /* U+0A89 */
  0x273F,   /* U+0A8A */
  0x2740,   /* U+0A8B */
  0x2742,   /* U+0A8C */
  0x2744,   /* U+0A8D */
  0xFBC0,   /* U+0A8E */
  0x2745,   /* U+0A8F */
  0x2746,   /* U+0A90 */
  0x2747,   /* U+0A91 */
  0xFBC0,   /* U+0A92 */
  0x2748,   /* U+0A93 */
  0x2749,   /* U+0A94 */
  0x274A,   /* U+0A95 */
  0x274B,   /* U+0A96 */
  0x274C,   /* U+0A97 */
  0x274D,   /* U+0A98 */
  0x274E,   /* U+0A99 */
  0x274F,   /* U+0A9A */
  0x2750,   /* U+0A9B */
  0x2751,   /* U+0A9C */
  0x2753,   /* U+0A9D */
  0x2754,   /* U+0A9E */
  0x2755,   /* U+0A9F */
  0x2756,   /* U+0AA0 */
  0x2757,   /* U+0AA1 */
  0x2758,   /* U+0AA2 */
  0x2759,   /* U+0AA3 */
  0x275A,   /* U+0AA4 */
  0x275B,   /* U+0AA5 */
  0x275C,   /* U+0AA6 */
  0x275D,   /* U+0AA7 */
  0x275E,   /* U+0AA8 */
  0xFBC0,   /* U+0AA9 */
  0x275F,   /* U+0AAA */
  0x2760,   /* U+0AAB */
  0x2761,   /* U+0AAC */
  0x2762,   /* U+0AAD */
  0x2763,   /* U+0AAE */
  0x2764,   /* U+0AAF */
  0x2765,   /* U+0AB0 */
  0xFBC0,   /* U+0AB1 */
  0x2766,   /* U+0AB2 */
  0x276C,   /* U+0AB3 */
  0xFBC0,   /* U+0AB4 */
  0x2767,   /* U+0AB5 */
  0x2768,   /* U+0AB6 */
  0x2769,   /* U+0AB7 */
  0x276A,   /* U+0AB8 */
  0x276B,   /* U+0AB9 */
  0xFBC0,   /* U+0ABA */
  0xFBC0,   /* U+0ABB */
  0x0000,   /* U+0ABC */
  0x276D,   /* U+0ABD */
  0x276E,   /* U+0ABE */
  0x276F,   /* U+0ABF */
  0x2770,   /* U+0AC0 */
  0x2771,   /* U+0AC1 */
  0x2772,   /* U+0AC2 */
  0x2773,   /* U+0AC3 */
  0x2774,   /* U+0AC4 */
  0x2777,   /* U+0AC5 */
  0xFBC0,   /* U+0AC6 */
  0x2778,   /* U+0AC7 */
  0x2779,   /* U+0AC8 */
  0x277A,   /* U+0AC9 */
  0xFBC0,   /* U+0ACA */
  0x277B,   /* U+0ACB */
  0x277C,   /* U+0ACC */
  0x277D,   /* U+0ACD */
  0xFBC0,   /* U+0ACE */
  0xFBC0,   /* U+0ACF */
  0x2739,   /* U+0AD0 */
  0xFBC0,   /* U+0AD1 */
  0xFBC0,   /* U+0AD2 */
  0xFBC0,   /* U+0AD3 */
  0xFBC0,   /* U+0AD4 */
  0xFBC0,   /* U+0AD5 */
  0xFBC0,   /* U+0AD6 */
  0xFBC0,   /* U+0AD7 */
  0xFBC0,   /* U+0AD8 */
  0xFBC0,   /* U+0AD9 */
  0xFBC0,   /* U+0ADA */
  0xFBC0,   /* U+0ADB */
  0xFBC0,   /* U+0ADC */
  0xFBC0,   /* U+0ADD */
  0xFBC0,   /* U+0ADE */
  0xFBC0,   /* U+0ADF */
  0x2741,   /* U+0AE0 */
  0x2743,   /* U+0AE1 */
  0x2775,   /* U+0AE2 */
  0x2776,   /* U+0AE3 */
  0xFBC0,   /* U+0AE4 */
  0xFBC0,   /* U+0AE5 */
  0x1C3D,   /* U+0AE6 */
  0x1C3E,   /* U+0AE7 */
  0x1C3F,   /* U+0AE8 */
  0x1C40,   /* U+0AE9 */
  0x1C41,   /* U+0AEA */
  0x1C42,   /* U+0AEB */
  0x1C43,   /* U+0AEC */
  0x1C44,   /* U+0AED */
  0x1C45,   /* U+0AEE */
  0x1C46,   /* U+0AEF */
  0x03EC,   /* U+0AF0 */
  0x1C1A,   /* U+0AF1 */
  0xFBC0,   /* U+0AF2 */
  0xFBC0,   /* U+0AF3 */
  0xFBC0,   /* U+0AF4 */
  0xFBC0,   /* U+0AF5 */
  0xFBC0,   /* U+0AF6 */
  0xFBC0,   /* U+0AF7 */
  0xFBC0,   /* U+0AF8 */
  0x2752,   /* U+0AF9 */
  0xFBC0,   /* U+0AFA */
  0xFBC0,   /* U+0AFB */
  0xFBC0,   /* U+0AFC */
  0xFBC0,   /* U+0AFD */
  0xFBC0,   /* U+0AFE */
  0xFBC0,   /* U+0AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0A00 */
  0x00C3,   /* U+0A01 */
  0x00C4,   /* U+0A02 */
  0x00C5,   /* U+0A03 */
  0x0020,   /* U+0A04 */
  0x0020,   /* U+0A05 */
  0x0020,   /* U+0A06 */
  0x0020,   /* U+0A07 */
  0x0020,   /* U+0A08 */
  0x0020,   /* U+0A09 */
  0x0020,   /* U+0A0A */
  0x0020,   /* U+0A0B */
  0x0020,   /* U+0A0C */
  0x0020,   /* U+0A0D */
  0x0020,   /* U+0A0E */
  0x0020,   /* U+0A0F */
  0x0020,   /* U+0A10 */
  0x0020,   /* U+0A11 */
  0x0020,   /* U+0A12 */
  0x0020,   /* U+0A13 */
  0x0020,   /* U+0A14 */
  0x0020,   /* U+0A15 */
  0x0020,   /* U+0A16 */
  0x0020,   /* U+0A17 */
  0x0020,   /* U+0A18 */
  0x0020,   /* U+0A19 */
  0x0020,   /* U+0A1A */
  0x0020,   /* U+0A1B */
  0x0020,   /* U+0A1C */
  0x0020,   /* U+0A1D */
  0x0020,   /* U+0A1E */
  0x0020,   /* U+0A1F */
  0x0020,   /* U+0A20 */
  0x0020,   /* U+0A21 */
  0x0020,   /* U+0A22 */
  0x0020,   /* U+0A23 */
  0x0020,   /* U+0A24 */
  0x0020,   /* U+0A25 */
  0x0020,   /* U+0A26 */
  0x0020,   /* U+0A27 */
  0x0020,   /* U+0A28 */
  0x0020,   /* U+0A29 */
  0x0020,   /* U+0A2A */
  0x0020,   /* U+0A2B */
  0x0020,   /* U+0A2C */
  0x0020,   /* U+0A2D */
  0x0020,   /* U+0A2E */
  0x0020,   /* U+0A2F */
  0x0020,   /* U+0A30 */
  0x0020,   /* U+0A31 */
  0x0020,   /* U+0A32 */
  0x0020,   /* U+0A33 */
  0x0020,   /* U+0A34 */
  0x0020,   /* U+0A35 */
  0x0020,   /* U+0A36 */
  0x0020,   /* U+0A37 */
  0x0020,   /* U+0A38 */
  0x0020,   /* U+0A39 */
  0x0020,   /* U+0A3A */
  0x0020,   /* U+0A3B */
  0x00C2,   /* U+0A3C */
  0x0020,   /* U+0A3D */
  0x0020,   /* U+0A3E */
  0x0020,   /* U+0A3F */
  0x0020,   /* U+0A40 */
  0x0020,   /* U+0A41 */
  0x0020,   /* U+0A42 */
  0x0020,   /* U+0A43 */
  0x0020,   /* U+0A44 */
  0x0020,   /* U+0A45 */
  0x0020,   /* U+0A46 */
  0x0020,   /* U+0A47 */
  0x0020,   /* U+0A48 */
  0x0020,   /* U+0A49 */
  0x0020,   /* U+0A4A */
  0x0020,   /* U+0A4B */
  0x0020,   /* U+0A4C */
  0x0020,   /* U+0A4D */
  0x0020,   /* U+0A4E */
  0x0020,   /* U+0A4F */
  0x0020,   /* U+0A50 */
  0x0020,   /* U+0A51 */
  0x0020,   /* U+0A52 */
  0x0020,   /* U+0A53 */
  0x0020,   /* U+0A54 */
  0x0020,   /* U+0A55 */
  0x0020,   /* U+0A56 */
  0x0020,   /* U+0A57 */
  0x0020,   /* U+0A58 */
  0x0020,   /* U+0A59 */
  0x0020,   /* U+0A5A */
  0x0020,   /* U+0A5B */
  0x0020,   /* U+0A5C */
  0x0020,   /* U+0A5D */
  0x0020,   /* U+0A5E */
  0x0020,   /* U+0A5F */
  0x0020,   /* U+0A60 */
  0x0020,   /* U+0A61 */
  0x0020,   /* U+0A62 */
  0x0020,   /* U+0A63 */
  0x0020,   /* U+0A64 */
  0x0020,   /* U+0A65 */
  0x0020,   /* U+0A66 */
  0x0020,   /* U+0A67 */
  0x0020,   /* U+0A68 */
  0x0020,   /* U+0A69 */
  0x0020,   /* U+0A6A */
  0x0020,   /* U+0A6B */
  0x0020,   /* U+0A6C */
  0x0020,   /* U+0A6D */
  0x0020,   /* U+0A6E */
  0x0020,   /* U+0A6F */
  0x00C6,   /* U+0A70 */
  0x00C7,   /* U+0A71 */
  0x0020,   /* U+0A72 */
  0x0020,   /* U+0A73 */
  0x0020,   /* U+0A74 */
  0x0020,   /* U+0A75 */
  0x0020,   /* U+0A76 */
  0x0020,   /* U+0A77 */
  0x0020,   /* U+0A78 */
  0x0020,   /* U+0A79 */
  0x0020,   /* U+0A7A */
  0x0020,   /* U+0A7B */
  0x0020,   /* U+0A7C */
  0x0020,   /* U+0A7D */
  0x0020,   /* U+0A7E */
  0x0020,   /* U+0A7F */
  0x0020,   /* U+0A80 */
  0x00C3,   /* U+0A81 */
  0x00C4,   /* U+0A82 */
  0x00C5,   /* U+0A83 */
  0x0020,   /* U+0A84 */
  0x0020,   /* U+0A85 */
  0x0020,   /* U+0A86 */
  0x0020,   /* U+0A87 */
  0x0020,   /* U+0A88 */
  0x0020,   /* U+0A89 */
  0x0020,   /* U+0A8A */
  0x0020,   /* U+0A8B */
  0x0020,   /* U+0A8C */
  0x0020,   /* U+0A8D */
  0x0020,   /* U+0A8E */
  0x0020,   /* U+0A8F */
  0x0020,   /* U+0A90 */
  0x0020,   /* U+0A91 */
  0x0020,   /* U+0A92 */
  0x0020,   /* U+0A93 */
  0x0020,   /* U+0A94 */
  0x0020,   /* U+0A95 */
  0x0020,   /* U+0A96 */
  0x0020,   /* U+0A97 */
  0x0020,   /* U+0A98 */
  0x0020,   /* U+0A99 */
  0x0020,   /* U+0A9A */
  0x0020,   /* U+0A9B */
  0x0020,   /* U+0A9C */
  0x0020,   /* U+0A9D */
  0x0020,   /* U+0A9E */
  0x0020,   /* U+0A9F */
  0x0020,   /* U+0AA0 */
  0x0020,   /* U+0AA1 */
  0x0020,   /* U+0AA2 */
  0x0020,   /* U+0AA3 */
  0x0020,   /* U+0AA4 */
  0x0020,   /* U+0AA5 */
  0x0020,   /* U+0AA6 */
  0x0020,   /* U+0AA7 */
  0x0020,   /* U+0AA8 */
  0x0020,   /* U+0AA9 */
  0x0020,   /* U+0AAA */
  0x0020,   /* U+0AAB */
  0x0020,   /* U+0AAC */
  0x0020,   /* U+0AAD */
  0x0020,   /* U+0AAE */
  0x0020,   /* U+0AAF */
  0x0020,   /* U+0AB0 */
  0x0020,   /* U+0AB1 */
  0x0020,   /* U+0AB2 */
  0x0020,   /* U+0AB3 */
  0x0020,   /* U+0AB4 */
  0x0020,   /* U+0AB5 */
  0x0020,   /* U+0AB6 */
  0x0020,   /* U+0AB7 */
  0x0020,   /* U+0AB8 */
  0x0020,   /* U+0AB9 */
  0x0020,   /* U+0ABA */
  0x0020,   /* U+0ABB */
  0x00C2,   /* U+0ABC */
  0x0020,   /* U+0ABD */
  0x0020,   /* U+0ABE */
  0x0020,   /* U+0ABF */
  0x0020,   /* U+0AC0 */
  0x0020,   /* U+0AC1 */
  0x0020,   /* U+0AC2 */
  0x0020,   /* U+0AC3 */
  0x0020,   /* U+0AC4 */
  0x0020,   /* U+0AC5 */
  0x0020,   /* U+0AC6 */
  0x0020,   /* U+0AC7 */
  0x0020,   /* U+0AC8 */
  0x0020,   /* U+0AC9 */
  0x0020,   /* U+0ACA */
  0x0020,   /* U+0ACB */
  0x0020,   /* U+0ACC */
  0x0020,   /* U+0ACD */
  0x0020,   /* U+0ACE */
  0x0020,   /* U+0ACF */
  0x0020,   /* U+0AD0 */
  0x0020,   /* U+0AD1 */
  0x0020,   /* U+0AD2 */
  0x0020,   /* U+0AD3 */
  0x0020,   /* U+0AD4 */
  0x0020,   /* U+0AD5 */
  0x0020,   /* U+0AD6 */
  0x0020,   /* U+0AD7 */
  0x0020,   /* U+0AD8 */
  0x0020,   /* U+0AD9 */
  0x0020,   /* U+0ADA */
  0x0020,   /* U+0ADB */
  0x0020,   /* U+0ADC */
  0x0020,   /* U+0ADD */
  0x0020,   /* U+0ADE */
  0x0020,   /* U+0ADF */
  0x0020,   /* U+0AE0 */
  0x0020,   /* U+0AE1 */
  0x0020,   /* U+0AE2 */
  0x0020,   /* U+0AE3 */
  0x0020,   /* U+0AE4 */
  0x0020,   /* U+0AE5 */
  0x0020,   /* U+0AE6 */
  0x0020,   /* U+0AE7 */
  0x0020,   /* U+0AE8 */
  0x0020,   /* U+0AE9 */
  0x0020,   /* U+0AEA */
  0x0020,   /* U+0AEB */
  0x0020,   /* U+0AEC */
  0x0020,   /* U+0AED */
  0x0020,   /* U+0AEE */
  0x0020,   /* U+0AEF */
  0x0020,   /* U+0AF0 */
  0x0020,   /* U+0AF1 */
  0x0020,   /* U+0AF2 */
  0x0020,   /* U+0AF3 */
  0x0020,   /* U+0AF4 */
  0x0020,   /* U+0AF5 */
  0x0020,   /* U+0AF6 */
  0x0020,   /* U+0AF7 */
  0x0020,   /* U+0AF8 */
  0x0020,   /* U+0AF9 */
  0x0020,   /* U+0AFA */
  0x0020,   /* U+0AFB */
  0x0020,   /* U+0AFC */
  0x0020,   /* U+0AFD */
  0x0020,   /* U+0AFE */
  0x0020,   /* U+0AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0A00 */
  0x0002,   /* U+0A01 */
  0x0002,   /* U+0A02 */
  0x0002,   /* U+0A03 */
  0x0002,   /* U+0A04 */
  0x0002,   /* U+0A05 */
  0x0002,   /* U+0A06 */
  0x0002,   /* U+0A07 */
  0x0002,   /* U+0A08 */
  0x0002,   /* U+0A09 */
  0x0002,   /* U+0A0A */
  0x0002,   /* U+0A0B */
  0x0002,   /* U+0A0C */
  0x0002,   /* U+0A0D */
  0x0002,   /* U+0A0E */
  0x0002,   /* U+0A0F */
  0x0002,   /* U+0A10 */
  0x0002,   /* U+0A11 */
  0x0002,   /* U+0A12 */
  0x0002,   /* U+0A13 */
  0x0002,   /* U+0A14 */
  0x0002,   /* U+0A15 */
  0x0002,   /* U+0A16 */
  0x0002,   /* U+0A17 */
  0x0002,   /* U+0A18 */
  0x0002,   /* U+0A19 */
  0x0002,   /* U+0A1A */
  0x0002,   /* U+0A1B */
  0x0002,   /* U+0A1C */
  0x0002,   /* U+0A1D */
  0x0002,   /* U+0A1E */
  0x0002,   /* U+0A1F */
  0x0002,   /* U+0A20 */
  0x0002,   /* U+0A21 */
  0x0002,   /* U+0A22 */
  0x0002,   /* U+0A23 */
  0x0002,   /* U+0A24 */
  0x0002,   /* U+0A25 */
  0x0002,   /* U+0A26 */
  0x0002,   /* U+0A27 */
  0x0002,   /* U+0A28 */
  0x0002,   /* U+0A29 */
  0x0002,   /* U+0A2A */
  0x0002,   /* U+0A2B */
  0x0002,   /* U+0A2C */
  0x0002,   /* U+0A2D */
  0x0002,   /* U+0A2E */
  0x0002,   /* U+0A2F */
  0x0002,   /* U+0A30 */
  0x0002,   /* U+0A31 */
  0x0002,   /* U+0A32 */
  0x0002,   /* U+0A33 */
  0x0002,   /* U+0A34 */
  0x0002,   /* U+0A35 */
  0x0002,   /* U+0A36 */
  0x0002,   /* U+0A37 */
  0x0002,   /* U+0A38 */
  0x0002,   /* U+0A39 */
  0x0002,   /* U+0A3A */
  0x0002,   /* U+0A3B */
  0x0002,   /* U+0A3C */
  0x0002,   /* U+0A3D */
  0x0002,   /* U+0A3E */
  0x0002,   /* U+0A3F */
  0x0002,   /* U+0A40 */
  0x0002,   /* U+0A41 */
  0x0002,   /* U+0A42 */
  0x0002,   /* U+0A43 */
  0x0002,   /* U+0A44 */
  0x0002,   /* U+0A45 */
  0x0002,   /* U+0A46 */
  0x0002,   /* U+0A47 */
  0x0002,   /* U+0A48 */
  0x0002,   /* U+0A49 */
  0x0002,   /* U+0A4A */
  0x0002,   /* U+0A4B */
  0x0002,   /* U+0A4C */
  0x0002,   /* U+0A4D */
  0x0002,   /* U+0A4E */
  0x0002,   /* U+0A4F */
  0x0002,   /* U+0A50 */
  0x0002,   /* U+0A51 */
  0x0002,   /* U+0A52 */
  0x0002,   /* U+0A53 */
  0x0002,   /* U+0A54 */
  0x0002,   /* U+0A55 */
  0x0002,   /* U+0A56 */
  0x0002,   /* U+0A57 */
  0x0002,   /* U+0A58 */
  0x0002,   /* U+0A59 */
  0x0002,   /* U+0A5A */
  0x0002,   /* U+0A5B */
  0x0002,   /* U+0A5C */
  0x0002,   /* U+0A5D */
  0x0002,   /* U+0A5E */
  0x0002,   /* U+0A5F */
  0x0002,   /* U+0A60 */
  0x0002,   /* U+0A61 */
  0x0002,   /* U+0A62 */
  0x0002,   /* U+0A63 */
  0x0002,   /* U+0A64 */
  0x0002,   /* U+0A65 */
  0x0002,   /* U+0A66 */
  0x0002,   /* U+0A67 */
  0x0002,   /* U+0A68 */
  0x0002,   /* U+0A69 */
  0x0002,   /* U+0A6A */
  0x0002,   /* U+0A6B */
  0x0002,   /* U+0A6C */
  0x0002,   /* U+0A6D */
  0x0002,   /* U+0A6E */
  0x0002,   /* U+0A6F */
  0x0002,   /* U+0A70 */
  0x0002,   /* U+0A71 */
  0x0002,   /* U+0A72 */
  0x0002,   /* U+0A73 */
  0x0002,   /* U+0A74 */
  0x0002,   /* U+0A75 */
  0x0002,   /* U+0A76 */
  0x0002,   /* U+0A77 */
  0x0002,   /* U+0A78 */
  0x0002,   /* U+0A79 */
  0x0002,   /* U+0A7A */
  0x0002,   /* U+0A7B */
  0x0002,   /* U+0A7C */
  0x0002,   /* U+0A7D */
  0x0002,   /* U+0A7E */
  0x0002,   /* U+0A7F */
  0x0002,   /* U+0A80 */
  0x0002,   /* U+0A81 */
  0x0002,   /* U+0A82 */
  0x0002,   /* U+0A83 */
  0x0002,   /* U+0A84 */
  0x0002,   /* U+0A85 */
  0x0002,   /* U+0A86 */
  0x0002,   /* U+0A87 */
  0x0002,   /* U+0A88 */
  0x0002,   /* U+0A89 */
  0x0002,   /* U+0A8A */
  0x0002,   /* U+0A8B */
  0x0002,   /* U+0A8C */
  0x0002,   /* U+0A8D */
  0x0002,   /* U+0A8E */
  0x0002,   /* U+0A8F */
  0x0002,   /* U+0A90 */
  0x0002,   /* U+0A91 */
  0x0002,   /* U+0A92 */
  0x0002,   /* U+0A93 */
  0x0002,   /* U+0A94 */
  0x0002,   /* U+0A95 */
  0x0002,   /* U+0A96 */
  0x0002,   /* U+0A97 */
  0x0002,   /* U+0A98 */
  0x0002,   /* U+0A99 */
  0x0002,   /* U+0A9A */
  0x0002,   /* U+0A9B */
  0x0002,   /* U+0A9C */
  0x0002,   /* U+0A9D */
  0x0002,   /* U+0A9E */
  0x0002,   /* U+0A9F */
  0x0002,   /* U+0AA0 */
  0x0002,   /* U+0AA1 */
  0x0002,   /* U+0AA2 */
  0x0002,   /* U+0AA3 */
  0x0002,   /* U+0AA4 */
  0x0002,   /* U+0AA5 */
  0x0002,   /* U+0AA6 */
  0x0002,   /* U+0AA7 */
  0x0002,   /* U+0AA8 */
  0x0002,   /* U+0AA9 */
  0x0002,   /* U+0AAA */
  0x0002,   /* U+0AAB */
  0x0002,   /* U+0AAC */
  0x0002,   /* U+0AAD */
  0x0002,   /* U+0AAE */
  0x0002,   /* U+0AAF */
  0x0002,   /* U+0AB0 */
  0x0002,   /* U+0AB1 */
  0x0002,   /* U+0AB2 */
  0x0002,   /* U+0AB3 */
  0x0002,   /* U+0AB4 */
  0x0002,   /* U+0AB5 */
  0x0002,   /* U+0AB6 */
  0x0002,   /* U+0AB7 */
  0x0002,   /* U+0AB8 */
  0x0002,   /* U+0AB9 */
  0x0002,   /* U+0ABA */
  0x0002,   /* U+0ABB */
  0x0002,   /* U+0ABC */
  0x0002,   /* U+0ABD */
  0x0002,   /* U+0ABE */
  0x0002,   /* U+0ABF */
  0x0002,   /* U+0AC0 */
  0x0002,   /* U+0AC1 */
  0x0002,   /* U+0AC2 */
  0x0002,   /* U+0AC3 */
  0x0002,   /* U+0AC4 */
  0x0002,   /* U+0AC5 */
  0x0002,   /* U+0AC6 */
  0x0002,   /* U+0AC7 */
  0x0002,   /* U+0AC8 */
  0x0002,   /* U+0AC9 */
  0x0002,   /* U+0ACA */
  0x0002,   /* U+0ACB */
  0x0002,   /* U+0ACC */
  0x0002,   /* U+0ACD */
  0x0002,   /* U+0ACE */
  0x0002,   /* U+0ACF */
  0x0002,   /* U+0AD0 */
  0x0002,   /* U+0AD1 */
  0x0002,   /* U+0AD2 */
  0x0002,   /* U+0AD3 */
  0x0002,   /* U+0AD4 */
  0x0002,   /* U+0AD5 */
  0x0002,   /* U+0AD6 */
  0x0002,   /* U+0AD7 */
  0x0002,   /* U+0AD8 */
  0x0002,   /* U+0AD9 */
  0x0002,   /* U+0ADA */
  0x0002,   /* U+0ADB */
  0x0002,   /* U+0ADC */
  0x0002,   /* U+0ADD */
  0x0002,   /* U+0ADE */
  0x0002,   /* U+0ADF */
  0x0002,   /* U+0AE0 */
  0x0002,   /* U+0AE1 */
  0x0002,   /* U+0AE2 */
  0x0002,   /* U+0AE3 */
  0x0002,   /* U+0AE4 */
  0x0002,   /* U+0AE5 */
  0x0002,   /* U+0AE6 */
  0x0002,   /* U+0AE7 */
  0x0002,   /* U+0AE8 */
  0x0002,   /* U+0AE9 */
  0x0002,   /* U+0AEA */
  0x0002,   /* U+0AEB */
  0x0002,   /* U+0AEC */
  0x0002,   /* U+0AED */
  0x0002,   /* U+0AEE */
  0x0002,   /* U+0AEF */
  0x0002,   /* U+0AF0 */
  0x0002,   /* U+0AF1 */
  0x0002,   /* U+0AF2 */
  0x0002,   /* U+0AF3 */
  0x0002,   /* U+0AF4 */
  0x0002,   /* U+0AF5 */
  0x0002,   /* U+0AF6 */
  0x0002,   /* U+0AF7 */
  0x0002,   /* U+0AF8 */
  0x0002,   /* U+0AF9 */
  0x0002,   /* U+0AFA */
  0x0002,   /* U+0AFB */
  0x0002,   /* U+0AFC */
  0x0002,   /* U+0AFD */
  0x0002,   /* U+0AFE */
  0x0002,   /* U+0AFF */

  /* Primary weight 2 for each character. */
  0x8A00,   /* U+0A00 */
  0x0000,   /* U+0A01 */
  0x0000,   /* U+0A02 */
  0x0000,   /* U+0A03 */
  0x8A04,   /* U+0A04 */
  0x0000,   /* U+0A05 */
  0x0000,   /* U+0A06 */
  0x0000,   /* U+0A07 */
  0x0000,   /* U+0A08 */
  0x0000,   /* U+0A09 */
  0x0000,   /* U+0A0A */
  0x8A0B,   /* U+0A0B */
  0x8A0C,   /* U+0A0C */
  0x8A0D,   /* U+0A0D */
  0x8A0E,   /* U+0A0E */
  0x0000,   /* U+0A0F */
  0x0000,   /* U+0A10 */
  0x8A11,   /* U+0A11 */
  0x8A12,   /* U+0A12 */
  0x0000,   /* U+0A13 */
  0x0000,   /* U+0A14 */
  0x0000,   /* U+0A15 */
  0x0000,   /* U+0A16 */
  0x0000,   /* U+0A17 */
  0x0000,   /* U+0A18 */
  0x0000,   /* U+0A19 */
  0x0000,   /* U+0A1A */
  0x0000,   /* U+0A1B */
  0x0000,   /* U+0A1C */
  0x0000,   /* U+0A1D */
  0x0000,   /* U+0A1E */
  0x0000,   /* U+0A1F */
  0x0000,   /* U+0A20 */
  0x0000,   /* U+0A21 */
  0x0000,   /* U+0A22 */
  0x0000,   /* U+0A23 */
  0x0000,   /* U+0A24 */
  0x0000,   /* U+0A25 */
  0x0000,   /* U+0A26 */
  0x0000,   /* U+0A27 */
  0x0000,   /* U+0A28 */
  0x8A29,   /* U+0A29 */
  0x0000,   /* U+0A2A */
  0x0000,   /* U+0A2B */
  0x0000,   /* U+0A2C */
  0x0000,   /* U+0A2D */
  0x0000,   /* U+0A2E */
  0x0000,   /* U+0A2F */
  0x0000,   /* U+0A30 */
  0x8A31,   /* U+0A31 */
  0x0000,   /* U+0A32 */
  0x0000,   /* U+0A33 */
  0x8A34,   /* U+0A34 */
  0x0000,   /* U+0A35 */
  0x0000,   /* U+0A36 */
  0x8A37,   /* U+0A37 */
  0x0000,   /* U+0A38 */
  0x0000,   /* U+0A39 */
  0x8A3A,   /* U+0A3A */
  0x8A3B,   /* U+0A3B */
  0x0000,   /* U+0A3C */
  0x8A3D,   /* U+0A3D */
  0x0000,   /* U+0A3E */
  0x0000,   /* U+0A3F */
  0x0000,   /* U+0A40 */
  0x0000,   /* U+0A41 */
  0x0000,   /* U+0A42 */
  0x8A43,   /* U+0A43 */
  0x8A44,   /* U+0A44 */
  0x8A45,   /* U+0A45 */
  0x8A46,   /* U+0A46 */
  0x0000,   /* U+0A47 */
  0x0000,   /* U+0A48 */
  0x8A49,   /* U+0A49 */
  0x8A4A,   /* U+0A4A */
  0x0000,   /* U+0A4B */
  0x0000,   /* U+0A4C */
  0x0000,   /* U+0A4D */
  0x8A4E,   /* U+0A4E */
  0x8A4F,   /* U+0A4F */
  0x8A50,   /* U+0A50 */
  0x0000,   /* U+0A51 */
  0x8A52,   /* U+0A52 */
  0x8A53,   /* U+0A53 */
  0x8A54,   /* U+0A54 */
  0x8A55,   /* U+0A55 */
  0x8A56,   /* U+0A56 */
  0x8A57,   /* U+0A57 */
  0x8A58,   /* U+0A58 */
  0x0000,   /* U+0A59 */
  0x0000,   /* U+0A5A */
  0x0000,   /* U+0A5B */
  0x0000,   /* U+0A5C */
  0x8A5D,   /* U+0A5D */
  0x0000,   /* U+0A5E */
  0x8A5F,   /* U+0A5F */
  0x8A60,   /* U+0A60 */
  0x8A61,   /* U+0A61 */
  0x8A62,   /* U+0A62 */
  0x8A63,   /* U+0A63 */
  0x8A64,   /* U+0A64 */
  0x8A65,   /* U+0A65 */
  0x0000,   /* U+0A66 */
  0x0000,   /* U+0A67 */
  0x0000,   /* U+0A68 */
  0x0000,   /* U+0A69 */
  0x0000,   /* U+0A6A */
  0x0000,   /* U+0A6B */
  0x0000,   /* U+0A6C */
  0x0000,   /* U+0A6D */
  0x0000,   /* U+0A6E */
  0x0000,   /* U+0A6F */
  0x0000,   /* U+0A70 */
  0x0000,   /* U+0A71 */
  0x0000,   /* U+0A72 */
  0x0000,   /* U+0A73 */
  0x0000,   /* U+0A74 */
  0x0000,   /* U+0A75 */
  0x8A76,   /* U+0A76 */
  0x8A77,   /* U+0A77 */
  0x8A78,   /* U+0A78 */
  0x8A79,   /* U+0A79 */
  0x8A7A,   /* U+0A7A */
  0x8A7B,   /* U+0A7B */
  0x8A7C,   /* U+0A7C */
  0x8A7D,   /* U+0A7D */
  0x8A7E,   /* U+0A7E */
  0x8A7F,   /* U+0A7F */
  0x8A80,   /* U+0A80 */
  0x0000,   /* U+0A81 */
  0x0000,   /* U+0A82 */
  0x0000,   /* U+0A83 */
  0x8A84,   /* U+0A84 */
  0x0000,   /* U+0A85 */
  0x0000,   /* U+0A86 */
  0x0000,   /* U+0A87 */
  0x0000,   /* U+0A88 */
  0x0000,   /* U+0A89 */
  0x0000,   /* U+0A8A */
  0x0000,   /* U+0A8B */
  0x0000,   /* U+0A8C */
  0x0000,   /* U+0A8D */
  0x8A8E,   /* U+0A8E */
  0x0000,   /* U+0A8F */
  0x0000,   /* U+0A90 */
  0x0000,   /* U+0A91 */
  0x8A92,   /* U+0A92 */
  0x0000,   /* U+0A93 */
  0x0000,   /* U+0A94 */
  0x0000,   /* U+0A95 */
  0x0000,   /* U+0A96 */
  0x0000,   /* U+0A97 */
  0x0000,   /* U+0A98 */
  0x0000,   /* U+0A99 */
  0x0000,   /* U+0A9A */
  0x0000,   /* U+0A9B */
  0x0000,   /* U+0A9C */
  0x0000,   /* U+0A9D */
  0x0000,   /* U+0A9E */
  0x0000,   /* U+0A9F */
  0x0000,   /* U+0AA0 */
  0x0000,   /* U+0AA1 */
  0x0000,   /* U+0AA2 */
  0x0000,   /* U+0AA3 */
  0x0000,   /* U+0AA4 */
  0x0000,   /* U+0AA5 */
  0x0000,   /* U+0AA6 */
  0x0000,   /* U+0AA7 */
  0x0000,   /* U+0AA8 */
  0x8AA9,   /* U+0AA9 */
  0x0000,   /* U+0AAA */
  0x0000,   /* U+0AAB */
  0x0000,   /* U+0AAC */
  0x0000,   /* U+0AAD */
  0x0000,   /* U+0AAE */
  0x0000,   /* U+0AAF */
  0x0000,   /* U+0AB0 */
  0x8AB1,   /* U+0AB1 */
  0x0000,   /* U+0AB2 */
  0x0000,   /* U+0AB3 */
  0x8AB4,   /* U+0AB4 */
  0x0000,   /* U+0AB5 */
  0x0000,   /* U+0AB6 */
  0x0000,   /* U+0AB7 */
  0x0000,   /* U+0AB8 */
  0x0000,   /* U+0AB9 */
  0x8ABA,   /* U+0ABA */
  0x8ABB,   /* U+0ABB */
  0x0000,   /* U+0ABC */
  0x0000,   /* U+0ABD */
  0x0000,   /* U+0ABE */
  0x0000,   /* U+0ABF */
  0x0000,   /* U+0AC0 */
  0x0000,   /* U+0AC1 */
  0x0000,   /* U+0AC2 */
  0x0000,   /* U+0AC3 */
  0x0000,   /* U+0AC4 */
  0x0000,   /* U+0AC5 */
  0x8AC6,   /* U+0AC6 */
  0x0000,   /* U+0AC7 */
  0x0000,   /* U+0AC8 */
  0x0000,   /* U+0AC9 */
  0x8ACA,   /* U+0ACA */
  0x0000,   /* U+0ACB */
  0x0000,   /* U+0ACC */
  0x0000,   /* U+0ACD */
  0x8ACE,   /* U+0ACE */
  0x8ACF,   /* U+0ACF */
  0x0000,   /* U+0AD0 */
  0x8AD1,   /* U+0AD1 */
  0x8AD2,   /* U+0AD2 */
  0x8AD3,   /* U+0AD3 */
  0x8AD4,   /* U+0AD4 */
  0x8AD5,   /* U+0AD5 */
  0x8AD6,   /* U+0AD6 */
  0x8AD7,   /* U+0AD7 */
  0x8AD8,   /* U+0AD8 */
  0x8AD9,   /* U+0AD9 */
  0x8ADA,   /* U+0ADA */
  0x8ADB,   /* U+0ADB */
  0x8ADC,   /* U+0ADC */
  0x8ADD,   /* U+0ADD */
  0x8ADE,   /* U+0ADE */
  0x8ADF,   /* U+0ADF */
  0x0000,   /* U+0AE0 */
  0x0000,   /* U+0AE1 */
  0x0000,   /* U+0AE2 */
  0x0000,   /* U+0AE3 */
  0x8AE4,   /* U+0AE4 */
  0x8AE5,   /* U+0AE5 */
  0x0000,   /* U+0AE6 */
  0x0000,   /* U+0AE7 */
  0x0000,   /* U+0AE8 */
  0x0000,   /* U+0AE9 */
  0x0000,   /* U+0AEA */
  0x0000,   /* U+0AEB */
  0x0000,   /* U+0AEC */
  0x0000,   /* U+0AED */
  0x0000,   /* U+0AEE */
  0x0000,   /* U+0AEF */
  0x0000,   /* U+0AF0 */
  0x0000,   /* U+0AF1 */
  0x8AF2,   /* U+0AF2 */
  0x8AF3,   /* U+0AF3 */
  0x8AF4,   /* U+0AF4 */
  0x8AF5,   /* U+0AF5 */
  0x8AF6,   /* U+0AF6 */
  0x8AF7,   /* U+0AF7 */
  0x8AF8,   /* U+0AF8 */
  0x0000,   /* U+0AF9 */
  0x8AFA,   /* U+0AFA */
  0x8AFB,   /* U+0AFB */
  0x8AFC,   /* U+0AFC */
  0x8AFD,   /* U+0AFD */
  0x8AFE,   /* U+0AFE */
  0x8AFF,   /* U+0AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0A00 */
  0x0000,   /* U+0A01 */
  0x0000,   /* U+0A02 */
  0x0000,   /* U+0A03 */
  0x0000,   /* U+0A04 */
  0x0000,   /* U+0A05 */
  0x0000,   /* U+0A06 */
  0x0000,   /* U+0A07 */
  0x0000,   /* U+0A08 */
  0x0000,   /* U+0A09 */
  0x0000,   /* U+0A0A */
  0x0000,   /* U+0A0B */
  0x0000,   /* U+0A0C */
  0x0000,   /* U+0A0D */
  0x0000,   /* U+0A0E */
  0x0000,   /* U+0A0F */
  0x0000,   /* U+0A10 */
  0x0000,   /* U+0A11 */
  0x0000,   /* U+0A12 */
  0x0000,   /* U+0A13 */
  0x0000,   /* U+0A14 */
  0x0000,   /* U+0A15 */
  0x0000,   /* U+0A16 */
  0x0000,   /* U+0A17 */
  0x0000,   /* U+0A18 */
  0x0000,   /* U+0A19 */
  0x0000,   /* U+0A1A */
  0x0000,   /* U+0A1B */
  0x0000,   /* U+0A1C */
  0x0000,   /* U+0A1D */
  0x0000,   /* U+0A1E */
  0x0000,   /* U+0A1F */
  0x0000,   /* U+0A20 */
  0x0000,   /* U+0A21 */
  0x0000,   /* U+0A22 */
  0x0000,   /* U+0A23 */
  0x0000,   /* U+0A24 */
  0x0000,   /* U+0A25 */
  0x0000,   /* U+0A26 */
  0x0000,   /* U+0A27 */
  0x0000,   /* U+0A28 */
  0x0000,   /* U+0A29 */
  0x0000,   /* U+0A2A */
  0x0000,   /* U+0A2B */
  0x0000,   /* U+0A2C */
  0x0000,   /* U+0A2D */
  0x0000,   /* U+0A2E */
  0x0000,   /* U+0A2F */
  0x0000,   /* U+0A30 */
  0x0000,   /* U+0A31 */
  0x0000,   /* U+0A32 */
  0x00C2,   /* U+0A33 */
  0x0000,   /* U+0A34 */
  0x0000,   /* U+0A35 */
  0x00C2,   /* U+0A36 */
  0x0000,   /* U+0A37 */
  0x0000,   /* U+0A38 */
  0x0000,   /* U+0A39 */
  0x0000,   /* U+0A3A */
  0x0000,   /* U+0A3B */
  0x0000,   /* U+0A3C */
  0x0000,   /* U+0A3D */
  0x0000,   /* U+0A3E */
  0x0000,   /* U+0A3F */
  0x0000,   /* U+0A40 */
  0x0000,   /* U+0A41 */
  0x0000,   /* U+0A42 */
  0x0000,   /* U+0A43 */
  0x0000,   /* U+0A44 */
  0x0000,   /* U+0A45 */
  0x0000,   /* U+0A46 */
  0x0000,   /* U+0A47 */
  0x0000,   /* U+0A48 */
  0x0000,   /* U+0A49 */
  0x0000,   /* U+0A4A */
  0x0000,   /* U+0A4B */
  0x0000,   /* U+0A4C */
  0x0000,   /* U+0A4D */
  0x0000,   /* U+0A4E */
  0x0000,   /* U+0A4F */
  0x0000,   /* U+0A50 */
  0x0000,   /* U+0A51 */
  0x0000,   /* U+0A52 */
  0x0000,   /* U+0A53 */
  0x0000,   /* U+0A54 */
  0x0000,   /* U+0A55 */
  0x0000,   /* U+0A56 */
  0x0000,   /* U+0A57 */
  0x0000,   /* U+0A58 */
  0x00C2,   /* U+0A59 */
  0x00C2,   /* U+0A5A */
  0x00C2,   /* U+0A5B */
  0x0000,   /* U+0A5C */
  0x0000,   /* U+0A5D */
  0x00C2,   /* U+0A5E */
  0x0000,   /* U+0A5F */
  0x0000,   /* U+0A60 */
  0x0000,   /* U+0A61 */
  0x0000,   /* U+0A62 */
  0x0000,   /* U+0A63 */
  0x0000,   /* U+0A64 */
  0x0000,   /* U+0A65 */
  0x0000,   /* U+0A66 */
  0x0000,   /* U+0A67 */
  0x0000,   /* U+0A68 */
  0x0000,   /* U+0A69 */
  0x0000,   /* U+0A6A */
  0x0000,   /* U+0A6B */
  0x0000,   /* U+0A6C */
  0x0000,   /* U+0A6D */
  0x0000,   /* U+0A6E */
  0x0000,   /* U+0A6F */
  0x0000,   /* U+0A70 */
  0x0000,   /* U+0A71 */
  0x0000,   /* U+0A72 */
  0x0000,   /* U+0A73 */
  0x0000,   /* U+0A74 */
  0x0000,   /* U+0A75 */
  0x0000,   /* U+0A76 */
  0x0000,   /* U+0A77 */
  0x0000,   /* U+0A78 */
  0x0000,   /* U+0A79 */
  0x0000,   /* U+0A7A */
  0x0000,   /* U+0A7B */
  0x0000,   /* U+0A7C */
  0x0000,   /* U+0A7D */
  0x0000,   /* U+0A7E */
  0x0000,   /* U+0A7F */
  0x0000,   /* U+0A80 */
  0x0000,   /* U+0A81 */
  0x0000,   /* U+0A82 */
  0x0000,   /* U+0A83 */
  0x0000,   /* U+0A84 */
  0x0000,   /* U+0A85 */
  0x0000,   /* U+0A86 */
  0x0000,   /* U+0A87 */
  0x0000,   /* U+0A88 */
  0x0000,   /* U+0A89 */
  0x0000,   /* U+0A8A */
  0x0000,   /* U+0A8B */
  0x0000,   /* U+0A8C */
  0x0000,   /* U+0A8D */
  0x0000,   /* U+0A8E */
  0x0000,   /* U+0A8F */
  0x0000,   /* U+0A90 */
  0x0000,   /* U+0A91 */
  0x0000,   /* U+0A92 */
  0x0000,   /* U+0A93 */
  0x0000,   /* U+0A94 */
  0x0000,   /* U+0A95 */
  0x0000,   /* U+0A96 */
  0x0000,   /* U+0A97 */
  0x0000,   /* U+0A98 */
  0x0000,   /* U+0A99 */
  0x0000,   /* U+0A9A */
  0x0000,   /* U+0A9B */
  0x0000,   /* U+0A9C */
  0x0000,   /* U+0A9D */
  0x0000,   /* U+0A9E */
  0x0000,   /* U+0A9F */
  0x0000,   /* U+0AA0 */
  0x0000,   /* U+0AA1 */
  0x0000,   /* U+0AA2 */
  0x0000,   /* U+0AA3 */
  0x0000,   /* U+0AA4 */
  0x0000,   /* U+0AA5 */
  0x0000,   /* U+0AA6 */
  0x0000,   /* U+0AA7 */
  0x0000,   /* U+0AA8 */
  0x0000,   /* U+0AA9 */
  0x0000,   /* U+0AAA */
  0x0000,   /* U+0AAB */
  0x0000,   /* U+0AAC */
  0x0000,   /* U+0AAD */
  0x0000,   /* U+0AAE */
  0x0000,   /* U+0AAF */
  0x0000,   /* U+0AB0 */
  0x0000,   /* U+0AB1 */
  0x0000,   /* U+0AB2 */
  0x0000,   /* U+0AB3 */
  0x0000,   /* U+0AB4 */
  0x0000,   /* U+0AB5 */
  0x0000,   /* U+0AB6 */
  0x0000,   /* U+0AB7 */
  0x0000,   /* U+0AB8 */
  0x0000,   /* U+0AB9 */
  0x0000,   /* U+0ABA */
  0x0000,   /* U+0ABB */
  0x0000,   /* U+0ABC */
  0x0000,   /* U+0ABD */
  0x0000,   /* U+0ABE */
  0x0000,   /* U+0ABF */
  0x0000,   /* U+0AC0 */
  0x0000,   /* U+0AC1 */
  0x0000,   /* U+0AC2 */
  0x0000,   /* U+0AC3 */
  0x0000,   /* U+0AC4 */
  0x0000,   /* U+0AC5 */
  0x0000,   /* U+0AC6 */
  0x0000,   /* U+0AC7 */
  0x0000,   /* U+0AC8 */
  0x0000,   /* U+0AC9 */
  0x0000,   /* U+0ACA */
  0x0000,   /* U+0ACB */
  0x0000,   /* U+0ACC */
  0x0000,   /* U+0ACD */
  0x0000,   /* U+0ACE */
  0x0000,   /* U+0ACF */
  0x0000,   /* U+0AD0 */
  0x0000,   /* U+0AD1 */
  0x0000,   /* U+0AD2 */
  0x0000,   /* U+0AD3 */
  0x0000,   /* U+0AD4 */
  0x0000,   /* U+0AD5 */
  0x0000,   /* U+0AD6 */
  0x0000,   /* U+0AD7 */
  0x0000,   /* U+0AD8 */
  0x0000,   /* U+0AD9 */
  0x0000,   /* U+0ADA */
  0x0000,   /* U+0ADB */
  0x0000,   /* U+0ADC */
  0x0000,   /* U+0ADD */
  0x0000,   /* U+0ADE */
  0x0000,   /* U+0ADF */
  0x0000,   /* U+0AE0 */
  0x0000,   /* U+0AE1 */
  0x0000,   /* U+0AE2 */
  0x0000,   /* U+0AE3 */
  0x0000,   /* U+0AE4 */
  0x0000,   /* U+0AE5 */
  0x0000,   /* U+0AE6 */
  0x0000,   /* U+0AE7 */
  0x0000,   /* U+0AE8 */
  0x0000,   /* U+0AE9 */
  0x0000,   /* U+0AEA */
  0x0000,   /* U+0AEB */
  0x0000,   /* U+0AEC */
  0x0000,   /* U+0AED */
  0x0000,   /* U+0AEE */
  0x0000,   /* U+0AEF */
  0x0000,   /* U+0AF0 */
  0x0000,   /* U+0AF1 */
  0x0000,   /* U+0AF2 */
  0x0000,   /* U+0AF3 */
  0x0000,   /* U+0AF4 */
  0x0000,   /* U+0AF5 */
  0x0000,   /* U+0AF6 */
  0x0000,   /* U+0AF7 */
  0x0000,   /* U+0AF8 */
  0x0000,   /* U+0AF9 */
  0x0000,   /* U+0AFA */
  0x0000,   /* U+0AFB */
  0x0000,   /* U+0AFC */
  0x0000,   /* U+0AFD */
  0x0000,   /* U+0AFE */
  0x0000,   /* U+0AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0A00 */
  0x0000,   /* U+0A01 */
  0x0000,   /* U+0A02 */
  0x0000,   /* U+0A03 */
  0x0000,   /* U+0A04 */
  0x0000,   /* U+0A05 */
  0x0000,   /* U+0A06 */
  0x0000,   /* U+0A07 */
  0x0000,   /* U+0A08 */
  0x0000,   /* U+0A09 */
  0x0000,   /* U+0A0A */
  0x0000,   /* U+0A0B */
  0x0000,   /* U+0A0C */
  0x0000,   /* U+0A0D */
  0x0000,   /* U+0A0E */
  0x0000,   /* U+0A0F */
  0x0000,   /* U+0A10 */
  0x0000,   /* U+0A11 */
  0x0000,   /* U+0A12 */
  0x0000,   /* U+0A13 */
  0x0000,   /* U+0A14 */
  0x0000,   /* U+0A15 */
  0x0000,   /* U+0A16 */
  0x0000,   /* U+0A17 */
  0x0000,   /* U+0A18 */
  0x0000,   /* U+0A19 */
  0x0000,   /* U+0A1A */
  0x0000,   /* U+0A1B */
  0x0000,   /* U+0A1C */
  0x0000,   /* U+0A1D */
  0x0000,   /* U+0A1E */
  0x0000,   /* U+0A1F */
  0x0000,   /* U+0A20 */
  0x0000,   /* U+0A21 */
  0x0000,   /* U+0A22 */
  0x0000,   /* U+0A23 */
  0x0000,   /* U+0A24 */
  0x0000,   /* U+0A25 */
  0x0000,   /* U+0A26 */
  0x0000,   /* U+0A27 */
  0x0000,   /* U+0A28 */
  0x0000,   /* U+0A29 */
  0x0000,   /* U+0A2A */
  0x0000,   /* U+0A2B */
  0x0000,   /* U+0A2C */
  0x0000,   /* U+0A2D */
  0x0000,   /* U+0A2E */
  0x0000,   /* U+0A2F */
  0x0000,   /* U+0A30 */
  0x0000,   /* U+0A31 */
  0x0000,   /* U+0A32 */
  0x0002,   /* U+0A33 */
  0x0000,   /* U+0A34 */
  0x0000,   /* U+0A35 */
  0x0002,   /* U+0A36 */
  0x0000,   /* U+0A37 */
  0x0000,   /* U+0A38 */
  0x0000,   /* U+0A39 */
  0x0000,   /* U+0A3A */
  0x0000,   /* U+0A3B */
  0x0000,   /* U+0A3C */
  0x0000,   /* U+0A3D */
  0x0000,   /* U+0A3E */
  0x0000,   /* U+0A3F */
  0x0000,   /* U+0A40 */
  0x0000,   /* U+0A41 */
  0x0000,   /* U+0A42 */
  0x0000,   /* U+0A43 */
  0x0000,   /* U+0A44 */
  0x0000,   /* U+0A45 */
  0x0000,   /* U+0A46 */
  0x0000,   /* U+0A47 */
  0x0000,   /* U+0A48 */
  0x0000,   /* U+0A49 */
  0x0000,   /* U+0A4A */
  0x0000,   /* U+0A4B */
  0x0000,   /* U+0A4C */
  0x0000,   /* U+0A4D */
  0x0000,   /* U+0A4E */
  0x0000,   /* U+0A4F */
  0x0000,   /* U+0A50 */
  0x0000,   /* U+0A51 */
  0x0000,   /* U+0A52 */
  0x0000,   /* U+0A53 */
  0x0000,   /* U+0A54 */
  0x0000,   /* U+0A55 */
  0x0000,   /* U+0A56 */
  0x0000,   /* U+0A57 */
  0x0000,   /* U+0A58 */
  0x0002,   /* U+0A59 */
  0x0002,   /* U+0A5A */
  0x0002,   /* U+0A5B */
  0x0000,   /* U+0A5C */
  0x0000,   /* U+0A5D */
  0x0002,   /* U+0A5E */
  0x0000,   /* U+0A5F */
  0x0000,   /* U+0A60 */
  0x0000,   /* U+0A61 */
  0x0000,   /* U+0A62 */
  0x0000,   /* U+0A63 */
  0x0000,   /* U+0A64 */
  0x0000,   /* U+0A65 */
  0x0000,   /* U+0A66 */
  0x0000,   /* U+0A67 */
  0x0000,   /* U+0A68 */
  0x0000,   /* U+0A69 */
  0x0000,   /* U+0A6A */
  0x0000,   /* U+0A6B */
  0x0000,   /* U+0A6C */
  0x0000,   /* U+0A6D */
  0x0000,   /* U+0A6E */
  0x0000,   /* U+0A6F */
  0x0000,   /* U+0A70 */
  0x0000,   /* U+0A71 */
  0x0000,   /* U+0A72 */
  0x0000,   /* U+0A73 */
  0x0000,   /* U+0A74 */
  0x0000,   /* U+0A75 */
  0x0000,   /* U+0A76 */
  0x0000,   /* U+0A77 */
  0x0000,   /* U+0A78 */
  0x0000,   /* U+0A79 */
  0x0000,   /* U+0A7A */
  0x0000,   /* U+0A7B */
  0x0000,   /* U+0A7C */
  0x0000,   /* U+0A7D */
  0x0000,   /* U+0A7E */
  0x0000,   /* U+0A7F */
  0x0000,   /* U+0A80 */
  0x0000,   /* U+0A81 */
  0x0000,   /* U+0A82 */
  0x0000,   /* U+0A83 */
  0x0000,   /* U+0A84 */
  0x0000,   /* U+0A85 */
  0x0000,   /* U+0A86 */
  0x0000,   /* U+0A87 */
  0x0000,   /* U+0A88 */
  0x0000,   /* U+0A89 */
  0x0000,   /* U+0A8A */
  0x0000,   /* U+0A8B */
  0x0000,   /* U+0A8C */
  0x0000,   /* U+0A8D */
  0x0000,   /* U+0A8E */
  0x0000,   /* U+0A8F */
  0x0000,   /* U+0A90 */
  0x0000,   /* U+0A91 */
  0x0000,   /* U+0A92 */
  0x0000,   /* U+0A93 */
  0x0000,   /* U+0A94 */
  0x0000,   /* U+0A95 */
  0x0000,   /* U+0A96 */
  0x0000,   /* U+0A97 */
  0x0000,   /* U+0A98 */
  0x0000,   /* U+0A99 */
  0x0000,   /* U+0A9A */
  0x0000,   /* U+0A9B */
  0x0000,   /* U+0A9C */
  0x0000,   /* U+0A9D */
  0x0000,   /* U+0A9E */
  0x0000,   /* U+0A9F */
  0x0000,   /* U+0AA0 */
  0x0000,   /* U+0AA1 */
  0x0000,   /* U+0AA2 */
  0x0000,   /* U+0AA3 */
  0x0000,   /* U+0AA4 */
  0x0000,   /* U+0AA5 */
  0x0000,   /* U+0AA6 */
  0x0000,   /* U+0AA7 */
  0x0000,   /* U+0AA8 */
  0x0000,   /* U+0AA9 */
  0x0000,   /* U+0AAA */
  0x0000,   /* U+0AAB */
  0x0000,   /* U+0AAC */
  0x0000,   /* U+0AAD */
  0x0000,   /* U+0AAE */
  0x0000,   /* U+0AAF */
  0x0000,   /* U+0AB0 */
  0x0000,   /* U+0AB1 */
  0x0000,   /* U+0AB2 */
  0x0000,   /* U+0AB3 */
  0x0000,   /* U+0AB4 */
  0x0000,   /* U+0AB5 */
  0x0000,   /* U+0AB6 */
  0x0000,   /* U+0AB7 */
  0x0000,   /* U+0AB8 */
  0x0000,   /* U+0AB9 */
  0x0000,   /* U+0ABA */
  0x0000,   /* U+0ABB */
  0x0000,   /* U+0ABC */
  0x0000,   /* U+0ABD */
  0x0000,   /* U+0ABE */
  0x0000,   /* U+0ABF */
  0x0000,   /* U+0AC0 */
  0x0000,   /* U+0AC1 */
  0x0000,   /* U+0AC2 */
  0x0000,   /* U+0AC3 */
  0x0000,   /* U+0AC4 */
  0x0000,   /* U+0AC5 */
  0x0000,   /* U+0AC6 */
  0x0000,   /* U+0AC7 */
  0x0000,   /* U+0AC8 */
  0x0000,   /* U+0AC9 */
  0x0000,   /* U+0ACA */
  0x0000,   /* U+0ACB */
  0x0000,   /* U+0ACC */
  0x0000,   /* U+0ACD */
  0x0000,   /* U+0ACE */
  0x0000,   /* U+0ACF */
  0x0000,   /* U+0AD0 */
  0x0000,   /* U+0AD1 */
  0x0000,   /* U+0AD2 */
  0x0000,   /* U+0AD3 */
  0x0000,   /* U+0AD4 */
  0x0000,   /* U+0AD5 */
  0x0000,   /* U+0AD6 */
  0x0000,   /* U+0AD7 */
  0x0000,   /* U+0AD8 */
  0x0000,   /* U+0AD9 */
  0x0000,   /* U+0ADA */
  0x0000,   /* U+0ADB */
  0x0000,   /* U+0ADC */
  0x0000,   /* U+0ADD */
  0x0000,   /* U+0ADE */
  0x0000,   /* U+0ADF */
  0x0000,   /* U+0AE0 */
  0x0000,   /* U+0AE1 */
  0x0000,   /* U+0AE2 */
  0x0000,   /* U+0AE3 */
  0x0000,   /* U+0AE4 */
  0x0000,   /* U+0AE5 */
  0x0000,   /* U+0AE6 */
  0x0000,   /* U+0AE7 */
  0x0000,   /* U+0AE8 */
  0x0000,   /* U+0AE9 */
  0x0000,   /* U+0AEA */
  0x0000,   /* U+0AEB */
  0x0000,   /* U+0AEC */
  0x0000,   /* U+0AED */
  0x0000,   /* U+0AEE */
  0x0000,   /* U+0AEF */
  0x0000,   /* U+0AF0 */
  0x0000,   /* U+0AF1 */
  0x0000,   /* U+0AF2 */
  0x0000,   /* U+0AF3 */
  0x0000,   /* U+0AF4 */
  0x0000,   /* U+0AF5 */
  0x0000,   /* U+0AF6 */
  0x0000,   /* U+0AF7 */
  0x0000,   /* U+0AF8 */
  0x0000,   /* U+0AF9 */
  0x0000,   /* U+0AFA */
  0x0000,   /* U+0AFB */
  0x0000,   /* U+0AFC */
  0x0000,   /* U+0AFD */
  0x0000,   /* U+0AFE */
  0x0000,   /* U+0AFF */
};

uint16 uca900_p00B[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 
  1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 1, 2, 1, 2, 1, 1, 
  2, 2, 2, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC0,   /* U+0B00 */
  0x0000,   /* U+0B01 */
  0x0000,   /* U+0B02 */
  0x0000,   /* U+0B03 */
  0xFBC0,   /* U+0B04 */
  0x277E,   /* U+0B05 */
  0x277F,   /* U+0B06 */
  0x2780,   /* U+0B07 */
  0x2781,   /* U+0B08 */
  0x2782,   /* U+0B09 */
  0x2783,   /* U+0B0A */
  0x2784,   /* U+0B0B */
  0x2786,   /* U+0B0C */
  0xFBC0,   /* U+0B0D */
  0xFBC0,   /* U+0B0E */
  0x2788,   /* U+0B0F */
  0x2789,   /* U+0B10 */
  0xFBC0,   /* U+0B11 */
  0xFBC0,   /* U+0B12 */
  0x278A,   /* U+0B13 */
  0x278B,   /* U+0B14 */
  0x278C,   /* U+0B15 */
  0x278D,   /* U+0B16 */
  0x278E,   /* U+0B17 */
  0x278F,   /* U+0B18 */
  0x2790,   /* U+0B19 */
  0x2791,   /* U+0B1A */
  0x2792,   /* U+0B1B */
  0x2793,   /* U+0B1C */
  0x2794,   /* U+0B1D */
  0x2795,   /* U+0B1E */
  0x2796,   /* U+0B1F */
  0x2797,   /* U+0B20 */
  0x2798,   /* U+0B21 */
  0x2799,   /* U+0B22 */
  0x279A,   /* U+0B23 */
  0x279B,   /* U+0B24 */
  0x279C,   /* U+0B25 */
  0x279D,   /* U+0B26 */
  0x279E,   /* U+0B27 */
  0x279F,   /* U+0B28 */
  0xFBC0,   /* U+0B29 */
  0x27A0,   /* U+0B2A */
  0x27A1,   /* U+0B2B */
  0x27A2,   /* U+0B2C */
  0x27A3,   /* U+0B2D */
  0x27A4,   /* U+0B2E */
  0x27A5,   /* U+0B2F */
  0x27A7,   /* U+0B30 */
  0xFBC0,   /* U+0B31 */
  0x27A8,   /* U+0B32 */
  0x27A9,   /* U+0B33 */
  0xFBC0,   /* U+0B34 */
  0x27AA,   /* U+0B35 */
  0x27AC,   /* U+0B36 */
  0x27AD,   /* U+0B37 */
  0x27AE,   /* U+0B38 */
  0x27AF,   /* U+0B39 */
  0xFBC0,   /* U+0B3A */
  0xFBC0,   /* U+0B3B */
  0x0000,   /* U+0B3C */
  0x27B0,   /* U+0B3D */
  0x27B1,   /* U+0B3E */
  0x27B2,   /* U+0B3F */
  0x27B3,   /* U+0B40 */
  0x27B4,   /* U+0B41 */
  0x27B5,   /* U+0B42 */
  0x27B6,   /* U+0B43 */
  0x27B7,   /* U+0B44 */
  0xFBC0,   /* U+0B45 */
  0xFBC0,   /* U+0B46 */
  0x27BA,   /* U+0B47 */
  0x27BB,   /* U+0B48 */
  0xFBC0,   /* U+0B49 */
  0xFBC0,   /* U+0B4A */
  0x27BC,   /* U+0B4B */
  0x27BD,   /* U+0B4C */
  0x27BE,   /* U+0B4D */
  0xFBC0,   /* U+0B4E */
  0xFBC0,   /* U+0B4F */
  0xFBC0,   /* U+0B50 */
  0xFBC0,   /* U+0B51 */
  0xFBC0,   /* U+0B52 */
  0xFBC0,   /* U+0B53 */
  0xFBC0,   /* U+0B54 */
  0xFBC0,   /* U+0B55 */
  0x27BF,   /* U+0B56 */
  0x27C0,   /* U+0B57 */
  0xFBC0,   /* U+0B58 */
  0xFBC0,   /* U+0B59 */
  0xFBC0,   /* U+0B5A */
  0xFBC0,   /* U+0B5B */
  0x2798,   /* U+0B5C */
  0x2799,   /* U+0B5D */
  0xFBC0,   /* U+0B5E */
  0x27A6,   /* U+0B5F */
  0x2785,   /* U+0B60 */
  0x2787,   /* U+0B61 */
  0x27B8,   /* U+0B62 */
  0x27B9,   /* U+0B63 */
  0xFBC0,   /* U+0B64 */
  0xFBC0,   /* U+0B65 */
  0x1C3D,   /* U+0B66 */
  0x1C3E,   /* U+0B67 */
  0x1C3F,   /* U+0B68 */
  0x1C40,   /* U+0B69 */
  0x1C41,   /* U+0B6A */
  0x1C42,   /* U+0B6B */
  0x1C43,   /* U+0B6C */
  0x1C44,   /* U+0B6D */
  0x1C45,   /* U+0B6E */
  0x1C46,   /* U+0B6F */
  0x0514,   /* U+0B70 */
  0x27AB,   /* U+0B71 */
  0x1A9C,   /* U+0B72 */
  0x1A9D,   /* U+0B73 */
  0x1A9E,   /* U+0B74 */
  0x1A9F,   /* U+0B75 */
  0x1AA0,   /* U+0B76 */
  0x1AA1,   /* U+0B77 */
  0xFBC0,   /* U+0B78 */
  0xFBC0,   /* U+0B79 */
  0xFBC0,   /* U+0B7A */
  0xFBC0,   /* U+0B7B */
  0xFBC0,   /* U+0B7C */
  0xFBC0,   /* U+0B7D */
  0xFBC0,   /* U+0B7E */
  0xFBC0,   /* U+0B7F */
  0xFBC0,   /* U+0B80 */
  0xFBC0,   /* U+0B81 */
  0x0000,   /* U+0B82 */
  0x27CE,   /* U+0B83 */
  0xFBC0,   /* U+0B84 */
  0x27C2,   /* U+0B85 */
  0x27C3,   /* U+0B86 */
  0x27C4,   /* U+0B87 */
  0x27C5,   /* U+0B88 */
  0x27C6,   /* U+0B89 */
  0x27C7,   /* U+0B8A */
  0xFBC0,   /* U+0B8B */
  0xFBC0,   /* U+0B8C */
  0xFBC0,   /* U+0B8D */
  0x27C8,   /* U+0B8E */
  0x27C9,   /* U+0B8F */
  0x27CA,   /* U+0B90 */
  0xFBC0,   /* U+0B91 */
  0x27CB,   /* U+0B92 */
  0x27CC,   /* U+0B93 */
  0x27CD,   /* U+0B94 */
  0x27CF,   /* U+0B95 */
  0xFBC0,   /* U+0B96 */
  0xFBC0,   /* U+0B97 */
  0xFBC0,   /* U+0B98 */
  0x27D0,   /* U+0B99 */
  0x27D1,   /* U+0B9A */
  0xFBC0,   /* U+0B9B */
  0x27E1,   /* U+0B9C */
  0xFBC0,   /* U+0B9D */
  0x27D2,   /* U+0B9E */
  0x27D3,   /* U+0B9F */
  0xFBC0,   /* U+0BA0 */
  0xFBC0,   /* U+0BA1 */
  0xFBC0,   /* U+0BA2 */
  0x27D4,   /* U+0BA3 */
  0x27D5,   /* U+0BA4 */
  0xFBC0,   /* U+0BA5 */
  0xFBC0,   /* U+0BA6 */
  0xFBC0,   /* U+0BA7 */
  0x27D6,   /* U+0BA8 */
  0x27E0,   /* U+0BA9 */
  0x27D7,   /* U+0BAA */
  0xFBC0,   /* U+0BAB */
  0xFBC0,   /* U+0BAC */
  0xFBC0,   /* U+0BAD */
  0x27D8,   /* U+0BAE */
  0x27D9,   /* U+0BAF */
  0x27DA,   /* U+0BB0 */
  0x27DF,   /* U+0BB1 */
  0x27DB,   /* U+0BB2 */
  0x27DE,   /* U+0BB3 */
  0x27DD,   /* U+0BB4 */
  0x27DC,   /* U+0BB5 */
  0x27E2,   /* U+0BB6 */
  0x27E3,   /* U+0BB7 */
  0x27E4,   /* U+0BB8 */
  0x27E5,   /* U+0BB9 */
  0xFBC0,   /* U+0BBA */
  0xFBC0,   /* U+0BBB */
  0xFBC0,   /* U+0BBC */
  0xFBC0,   /* U+0BBD */
  0x27E6,   /* U+0BBE */
  0x27E7,   /* U+0BBF */
  0x27E8,   /* U+0BC0 */
  0x27E9,   /* U+0BC1 */
  0x27EA,   /* U+0BC2 */
  0xFBC0,   /* U+0BC3 */
  0xFBC0,   /* U+0BC4 */
  0xFBC0,   /* U+0BC5 */
  0x27EB,   /* U+0BC6 */
  0x27EC,   /* U+0BC7 */
  0x27ED,   /* U+0BC8 */
  0xFBC0,   /* U+0BC9 */
  0x27EE,   /* U+0BCA */
  0x27EF,   /* U+0BCB */
  0x27F0,   /* U+0BCC */
  0x27F1,   /* U+0BCD */
  0xFBC0,   /* U+0BCE */
  0xFBC0,   /* U+0BCF */
  0x27C1,   /* U+0BD0 */
  0xFBC0,   /* U+0BD1 */
  0xFBC0,   /* U+0BD2 */
  0xFBC0,   /* U+0BD3 */
  0xFBC0,   /* U+0BD4 */
  0xFBC0,   /* U+0BD5 */
  0xFBC0,   /* U+0BD6 */
  0x27F2,   /* U+0BD7 */
  0xFBC0,   /* U+0BD8 */
  0xFBC0,   /* U+0BD9 */
  0xFBC0,   /* U+0BDA */
  0xFBC0,   /* U+0BDB */
  0xFBC0,   /* U+0BDC */
  0xFBC0,   /* U+0BDD */
  0xFBC0,   /* U+0BDE */
  0xFBC0,   /* U+0BDF */
  0xFBC0,   /* U+0BE0 */
  0xFBC0,   /* U+0BE1 */
  0xFBC0,   /* U+0BE2 */
  0xFBC0,   /* U+0BE3 */
  0xFBC0,   /* U+0BE4 */
  0xFBC0,   /* U+0BE5 */
  0x1C3D,   /* U+0BE6 */
  0x1C3E,   /* U+0BE7 */
  0x1C3F,   /* U+0BE8 */
  0x1C40,   /* U+0BE9 */
  0x1C41,   /* U+0BEA */
  0x1C42,   /* U+0BEB */
  0x1C43,   /* U+0BEC */
  0x1C44,   /* U+0BED */
  0x1C45,   /* U+0BEE */
  0x1C46,   /* U+0BEF */
  0x1AA8,   /* U+0BF0 */
  0x1AA9,   /* U+0BF1 */
  0x1AAA,   /* U+0BF2 */
  0x0515,   /* U+0BF3 */
  0x0516,   /* U+0BF4 */
  0x0517,   /* U+0BF5 */
  0x0518,   /* U+0BF6 */
  0x0519,   /* U+0BF7 */
  0x051A,   /* U+0BF8 */
  0x1C1C,   /* U+0BF9 */
  0x051B,   /* U+0BFA */
  0xFBC0,   /* U+0BFB */
  0xFBC0,   /* U+0BFC */
  0xFBC0,   /* U+0BFD */
  0xFBC0,   /* U+0BFE */
  0xFBC0,   /* U+0BFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0B00 */
  0x00C3,   /* U+0B01 */
  0x00C4,   /* U+0B02 */
  0x00C5,   /* U+0B03 */
  0x0020,   /* U+0B04 */
  0x0020,   /* U+0B05 */
  0x0020,   /* U+0B06 */
  0x0020,   /* U+0B07 */
  0x0020,   /* U+0B08 */
  0x0020,   /* U+0B09 */
  0x0020,   /* U+0B0A */
  0x0020,   /* U+0B0B */
  0x0020,   /* U+0B0C */
  0x0020,   /* U+0B0D */
  0x0020,   /* U+0B0E */
  0x0020,   /* U+0B0F */
  0x0020,   /* U+0B10 */
  0x0020,   /* U+0B11 */
  0x0020,   /* U+0B12 */
  0x0020,   /* U+0B13 */
  0x0020,   /* U+0B14 */
  0x0020,   /* U+0B15 */
  0x0020,   /* U+0B16 */
  0x0020,   /* U+0B17 */
  0x0020,   /* U+0B18 */
  0x0020,   /* U+0B19 */
  0x0020,   /* U+0B1A */
  0x0020,   /* U+0B1B */
  0x0020,   /* U+0B1C */
  0x0020,   /* U+0B1D */
  0x0020,   /* U+0B1E */
  0x0020,   /* U+0B1F */
  0x0020,   /* U+0B20 */
  0x0020,   /* U+0B21 */
  0x0020,   /* U+0B22 */
  0x0020,   /* U+0B23 */
  0x0020,   /* U+0B24 */
  0x0020,   /* U+0B25 */
  0x0020,   /* U+0B26 */
  0x0020,   /* U+0B27 */
  0x0020,   /* U+0B28 */
  0x0020,   /* U+0B29 */
  0x0020,   /* U+0B2A */
  0x0020,   /* U+0B2B */
  0x0020,   /* U+0B2C */
  0x0020,   /* U+0B2D */
  0x0020,   /* U+0B2E */
  0x0020,   /* U+0B2F */
  0x0020,   /* U+0B30 */
  0x0020,   /* U+0B31 */
  0x0020,   /* U+0B32 */
  0x0020,   /* U+0B33 */
  0x0020,   /* U+0B34 */
  0x0020,   /* U+0B35 */
  0x0020,   /* U+0B36 */
  0x0020,   /* U+0B37 */
  0x0020,   /* U+0B38 */
  0x0020,   /* U+0B39 */
  0x0020,   /* U+0B3A */
  0x0020,   /* U+0B3B */
  0x00C2,   /* U+0B3C */
  0x0020,   /* U+0B3D */
  0x0020,   /* U+0B3E */
  0x0020,   /* U+0B3F */
  0x0020,   /* U+0B40 */
  0x0020,   /* U+0B41 */
  0x0020,   /* U+0B42 */
  0x0020,   /* U+0B43 */
  0x0020,   /* U+0B44 */
  0x0020,   /* U+0B45 */
  0x0020,   /* U+0B46 */
  0x0020,   /* U+0B47 */
  0x0020,   /* U+0B48 */
  0x0020,   /* U+0B49 */
  0x0020,   /* U+0B4A */
  0x0020,   /* U+0B4B */
  0x0020,   /* U+0B4C */
  0x0020,   /* U+0B4D */
  0x0020,   /* U+0B4E */
  0x0020,   /* U+0B4F */
  0x0020,   /* U+0B50 */
  0x0020,   /* U+0B51 */
  0x0020,   /* U+0B52 */
  0x0020,   /* U+0B53 */
  0x0020,   /* U+0B54 */
  0x0020,   /* U+0B55 */
  0x0020,   /* U+0B56 */
  0x0020,   /* U+0B57 */
  0x0020,   /* U+0B58 */
  0x0020,   /* U+0B59 */
  0x0020,   /* U+0B5A */
  0x0020,   /* U+0B5B */
  0x0020,   /* U+0B5C */
  0x0020,   /* U+0B5D */
  0x0020,   /* U+0B5E */
  0x0020,   /* U+0B5F */
  0x0020,   /* U+0B60 */
  0x0020,   /* U+0B61 */
  0x0020,   /* U+0B62 */
  0x0020,   /* U+0B63 */
  0x0020,   /* U+0B64 */
  0x0020,   /* U+0B65 */
  0x0020,   /* U+0B66 */
  0x0020,   /* U+0B67 */
  0x0020,   /* U+0B68 */
  0x0020,   /* U+0B69 */
  0x0020,   /* U+0B6A */
  0x0020,   /* U+0B6B */
  0x0020,   /* U+0B6C */
  0x0020,   /* U+0B6D */
  0x0020,   /* U+0B6E */
  0x0020,   /* U+0B6F */
  0x0020,   /* U+0B70 */
  0x0020,   /* U+0B71 */
  0x0020,   /* U+0B72 */
  0x0020,   /* U+0B73 */
  0x0020,   /* U+0B74 */
  0x0020,   /* U+0B75 */
  0x0020,   /* U+0B76 */
  0x0020,   /* U+0B77 */
  0x0020,   /* U+0B78 */
  0x0020,   /* U+0B79 */
  0x0020,   /* U+0B7A */
  0x0020,   /* U+0B7B */
  0x0020,   /* U+0B7C */
  0x0020,   /* U+0B7D */
  0x0020,   /* U+0B7E */
  0x0020,   /* U+0B7F */
  0x0020,   /* U+0B80 */
  0x0020,   /* U+0B81 */
  0x00C4,   /* U+0B82 */
  0x0020,   /* U+0B83 */
  0x0020,   /* U+0B84 */
  0x0020,   /* U+0B85 */
  0x0020,   /* U+0B86 */
  0x0020,   /* U+0B87 */
  0x0020,   /* U+0B88 */
  0x0020,   /* U+0B89 */
  0x0020,   /* U+0B8A */
  0x0020,   /* U+0B8B */
  0x0020,   /* U+0B8C */
  0x0020,   /* U+0B8D */
  0x0020,   /* U+0B8E */
  0x0020,   /* U+0B8F */
  0x0020,   /* U+0B90 */
  0x0020,   /* U+0B91 */
  0x0020,   /* U+0B92 */
  0x0020,   /* U+0B93 */
  0x0020,   /* U+0B94 */
  0x0020,   /* U+0B95 */
  0x0020,   /* U+0B96 */
  0x0020,   /* U+0B97 */
  0x0020,   /* U+0B98 */
  0x0020,   /* U+0B99 */
  0x0020,   /* U+0B9A */
  0x0020,   /* U+0B9B */
  0x0020,   /* U+0B9C */
  0x0020,   /* U+0B9D */
  0x0020,   /* U+0B9E */
  0x0020,   /* U+0B9F */
  0x0020,   /* U+0BA0 */
  0x0020,   /* U+0BA1 */
  0x0020,   /* U+0BA2 */
  0x0020,   /* U+0BA3 */
  0x0020,   /* U+0BA4 */
  0x0020,   /* U+0BA5 */
  0x0020,   /* U+0BA6 */
  0x0020,   /* U+0BA7 */
  0x0020,   /* U+0BA8 */
  0x0020,   /* U+0BA9 */
  0x0020,   /* U+0BAA */
  0x0020,   /* U+0BAB */
  0x0020,   /* U+0BAC */
  0x0020,   /* U+0BAD */
  0x0020,   /* U+0BAE */
  0x0020,   /* U+0BAF */
  0x0020,   /* U+0BB0 */
  0x0020,   /* U+0BB1 */
  0x0020,   /* U+0BB2 */
  0x0020,   /* U+0BB3 */
  0x0020,   /* U+0BB4 */
  0x0020,   /* U+0BB5 */
  0x0020,   /* U+0BB6 */
  0x0020,   /* U+0BB7 */
  0x0020,   /* U+0BB8 */
  0x0020,   /* U+0BB9 */
  0x0020,   /* U+0BBA */
  0x0020,   /* U+0BBB */
  0x0020,   /* U+0BBC */
  0x0020,   /* U+0BBD */
  0x0020,   /* U+0BBE */
  0x0020,   /* U+0BBF */
  0x0020,   /* U+0BC0 */
  0x0020,   /* U+0BC1 */
  0x0020,   /* U+0BC2 */
  0x0020,   /* U+0BC3 */
  0x0020,   /* U+0BC4 */
  0x0020,   /* U+0BC5 */
  0x0020,   /* U+0BC6 */
  0x0020,   /* U+0BC7 */
  0x0020,   /* U+0BC8 */
  0x0020,   /* U+0BC9 */
  0x0020,   /* U+0BCA */
  0x0020,   /* U+0BCB */
  0x0020,   /* U+0BCC */
  0x0020,   /* U+0BCD */
  0x0020,   /* U+0BCE */
  0x0020,   /* U+0BCF */
  0x0020,   /* U+0BD0 */
  0x0020,   /* U+0BD1 */
  0x0020,   /* U+0BD2 */
  0x0020,   /* U+0BD3 */
  0x0020,   /* U+0BD4 */
  0x0020,   /* U+0BD5 */
  0x0020,   /* U+0BD6 */
  0x0020,   /* U+0BD7 */
  0x0020,   /* U+0BD8 */
  0x0020,   /* U+0BD9 */
  0x0020,   /* U+0BDA */
  0x0020,   /* U+0BDB */
  0x0020,   /* U+0BDC */
  0x0020,   /* U+0BDD */
  0x0020,   /* U+0BDE */
  0x0020,   /* U+0BDF */
  0x0020,   /* U+0BE0 */
  0x0020,   /* U+0BE1 */
  0x0020,   /* U+0BE2 */
  0x0020,   /* U+0BE3 */
  0x0020,   /* U+0BE4 */
  0x0020,   /* U+0BE5 */
  0x0020,   /* U+0BE6 */
  0x0020,   /* U+0BE7 */
  0x0020,   /* U+0BE8 */
  0x0020,   /* U+0BE9 */
  0x0020,   /* U+0BEA */
  0x0020,   /* U+0BEB */
  0x0020,   /* U+0BEC */
  0x0020,   /* U+0BED */
  0x0020,   /* U+0BEE */
  0x0020,   /* U+0BEF */
  0x0020,   /* U+0BF0 */
  0x0020,   /* U+0BF1 */
  0x0020,   /* U+0BF2 */
  0x0020,   /* U+0BF3 */
  0x0020,   /* U+0BF4 */
  0x0020,   /* U+0BF5 */
  0x0020,   /* U+0BF6 */
  0x0020,   /* U+0BF7 */
  0x0020,   /* U+0BF8 */
  0x0020,   /* U+0BF9 */
  0x0020,   /* U+0BFA */
  0x0020,   /* U+0BFB */
  0x0020,   /* U+0BFC */
  0x0020,   /* U+0BFD */
  0x0020,   /* U+0BFE */
  0x0020,   /* U+0BFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0B00 */
  0x0002,   /* U+0B01 */
  0x0002,   /* U+0B02 */
  0x0002,   /* U+0B03 */
  0x0002,   /* U+0B04 */
  0x0002,   /* U+0B05 */
  0x0002,   /* U+0B06 */
  0x0002,   /* U+0B07 */
  0x0002,   /* U+0B08 */
  0x0002,   /* U+0B09 */
  0x0002,   /* U+0B0A */
  0x0002,   /* U+0B0B */
  0x0002,   /* U+0B0C */
  0x0002,   /* U+0B0D */
  0x0002,   /* U+0B0E */
  0x0002,   /* U+0B0F */
  0x0002,   /* U+0B10 */
  0x0002,   /* U+0B11 */
  0x0002,   /* U+0B12 */
  0x0002,   /* U+0B13 */
  0x0002,   /* U+0B14 */
  0x0002,   /* U+0B15 */
  0x0002,   /* U+0B16 */
  0x0002,   /* U+0B17 */
  0x0002,   /* U+0B18 */
  0x0002,   /* U+0B19 */
  0x0002,   /* U+0B1A */
  0x0002,   /* U+0B1B */
  0x0002,   /* U+0B1C */
  0x0002,   /* U+0B1D */
  0x0002,   /* U+0B1E */
  0x0002,   /* U+0B1F */
  0x0002,   /* U+0B20 */
  0x0002,   /* U+0B21 */
  0x0002,   /* U+0B22 */
  0x0002,   /* U+0B23 */
  0x0002,   /* U+0B24 */
  0x0002,   /* U+0B25 */
  0x0002,   /* U+0B26 */
  0x0002,   /* U+0B27 */
  0x0002,   /* U+0B28 */
  0x0002,   /* U+0B29 */
  0x0002,   /* U+0B2A */
  0x0002,   /* U+0B2B */
  0x0002,   /* U+0B2C */
  0x0002,   /* U+0B2D */
  0x0002,   /* U+0B2E */
  0x0002,   /* U+0B2F */
  0x0002,   /* U+0B30 */
  0x0002,   /* U+0B31 */
  0x0002,   /* U+0B32 */
  0x0002,   /* U+0B33 */
  0x0002,   /* U+0B34 */
  0x0002,   /* U+0B35 */
  0x0002,   /* U+0B36 */
  0x0002,   /* U+0B37 */
  0x0002,   /* U+0B38 */
  0x0002,   /* U+0B39 */
  0x0002,   /* U+0B3A */
  0x0002,   /* U+0B3B */
  0x0002,   /* U+0B3C */
  0x0002,   /* U+0B3D */
  0x0002,   /* U+0B3E */
  0x0002,   /* U+0B3F */
  0x0002,   /* U+0B40 */
  0x0002,   /* U+0B41 */
  0x0002,   /* U+0B42 */
  0x0002,   /* U+0B43 */
  0x0002,   /* U+0B44 */
  0x0002,   /* U+0B45 */
  0x0002,   /* U+0B46 */
  0x0002,   /* U+0B47 */
  0x0002,   /* U+0B48 */
  0x0002,   /* U+0B49 */
  0x0002,   /* U+0B4A */
  0x0002,   /* U+0B4B */
  0x0002,   /* U+0B4C */
  0x0002,   /* U+0B4D */
  0x0002,   /* U+0B4E */
  0x0002,   /* U+0B4F */
  0x0002,   /* U+0B50 */
  0x0002,   /* U+0B51 */
  0x0002,   /* U+0B52 */
  0x0002,   /* U+0B53 */
  0x0002,   /* U+0B54 */
  0x0002,   /* U+0B55 */
  0x0002,   /* U+0B56 */
  0x0002,   /* U+0B57 */
  0x0002,   /* U+0B58 */
  0x0002,   /* U+0B59 */
  0x0002,   /* U+0B5A */
  0x0002,   /* U+0B5B */
  0x0002,   /* U+0B5C */
  0x0002,   /* U+0B5D */
  0x0002,   /* U+0B5E */
  0x0002,   /* U+0B5F */
  0x0002,   /* U+0B60 */
  0x0002,   /* U+0B61 */
  0x0002,   /* U+0B62 */
  0x0002,   /* U+0B63 */
  0x0002,   /* U+0B64 */
  0x0002,   /* U+0B65 */
  0x0002,   /* U+0B66 */
  0x0002,   /* U+0B67 */
  0x0002,   /* U+0B68 */
  0x0002,   /* U+0B69 */
  0x0002,   /* U+0B6A */
  0x0002,   /* U+0B6B */
  0x0002,   /* U+0B6C */
  0x0002,   /* U+0B6D */
  0x0002,   /* U+0B6E */
  0x0002,   /* U+0B6F */
  0x0002,   /* U+0B70 */
  0x0002,   /* U+0B71 */
  0x0002,   /* U+0B72 */
  0x0002,   /* U+0B73 */
  0x0002,   /* U+0B74 */
  0x0002,   /* U+0B75 */
  0x0002,   /* U+0B76 */
  0x0002,   /* U+0B77 */
  0x0002,   /* U+0B78 */
  0x0002,   /* U+0B79 */
  0x0002,   /* U+0B7A */
  0x0002,   /* U+0B7B */
  0x0002,   /* U+0B7C */
  0x0002,   /* U+0B7D */
  0x0002,   /* U+0B7E */
  0x0002,   /* U+0B7F */
  0x0002,   /* U+0B80 */
  0x0002,   /* U+0B81 */
  0x0002,   /* U+0B82 */
  0x0002,   /* U+0B83 */
  0x0002,   /* U+0B84 */
  0x0002,   /* U+0B85 */
  0x0002,   /* U+0B86 */
  0x0002,   /* U+0B87 */
  0x0002,   /* U+0B88 */
  0x0002,   /* U+0B89 */
  0x0002,   /* U+0B8A */
  0x0002,   /* U+0B8B */
  0x0002,   /* U+0B8C */
  0x0002,   /* U+0B8D */
  0x0002,   /* U+0B8E */
  0x0002,   /* U+0B8F */
  0x0002,   /* U+0B90 */
  0x0002,   /* U+0B91 */
  0x0002,   /* U+0B92 */
  0x0002,   /* U+0B93 */
  0x0002,   /* U+0B94 */
  0x0002,   /* U+0B95 */
  0x0002,   /* U+0B96 */
  0x0002,   /* U+0B97 */
  0x0002,   /* U+0B98 */
  0x0002,   /* U+0B99 */
  0x0002,   /* U+0B9A */
  0x0002,   /* U+0B9B */
  0x0002,   /* U+0B9C */
  0x0002,   /* U+0B9D */
  0x0002,   /* U+0B9E */
  0x0002,   /* U+0B9F */
  0x0002,   /* U+0BA0 */
  0x0002,   /* U+0BA1 */
  0x0002,   /* U+0BA2 */
  0x0002,   /* U+0BA3 */
  0x0002,   /* U+0BA4 */
  0x0002,   /* U+0BA5 */
  0x0002,   /* U+0BA6 */
  0x0002,   /* U+0BA7 */
  0x0002,   /* U+0BA8 */
  0x0002,   /* U+0BA9 */
  0x0002,   /* U+0BAA */
  0x0002,   /* U+0BAB */
  0x0002,   /* U+0BAC */
  0x0002,   /* U+0BAD */
  0x0002,   /* U+0BAE */
  0x0002,   /* U+0BAF */
  0x0002,   /* U+0BB0 */
  0x0002,   /* U+0BB1 */
  0x0002,   /* U+0BB2 */
  0x0002,   /* U+0BB3 */
  0x0002,   /* U+0BB4 */
  0x0002,   /* U+0BB5 */
  0x0002,   /* U+0BB6 */
  0x0002,   /* U+0BB7 */
  0x0002,   /* U+0BB8 */
  0x0002,   /* U+0BB9 */
  0x0002,   /* U+0BBA */
  0x0002,   /* U+0BBB */
  0x0002,   /* U+0BBC */
  0x0002,   /* U+0BBD */
  0x0002,   /* U+0BBE */
  0x0002,   /* U+0BBF */
  0x0002,   /* U+0BC0 */
  0x0002,   /* U+0BC1 */
  0x0002,   /* U+0BC2 */
  0x0002,   /* U+0BC3 */
  0x0002,   /* U+0BC4 */
  0x0002,   /* U+0BC5 */
  0x0002,   /* U+0BC6 */
  0x0002,   /* U+0BC7 */
  0x0002,   /* U+0BC8 */
  0x0002,   /* U+0BC9 */
  0x0002,   /* U+0BCA */
  0x0002,   /* U+0BCB */
  0x0002,   /* U+0BCC */
  0x0002,   /* U+0BCD */
  0x0002,   /* U+0BCE */
  0x0002,   /* U+0BCF */
  0x0002,   /* U+0BD0 */
  0x0002,   /* U+0BD1 */
  0x0002,   /* U+0BD2 */
  0x0002,   /* U+0BD3 */
  0x0002,   /* U+0BD4 */
  0x0002,   /* U+0BD5 */
  0x0002,   /* U+0BD6 */
  0x0002,   /* U+0BD7 */
  0x0002,   /* U+0BD8 */
  0x0002,   /* U+0BD9 */
  0x0002,   /* U+0BDA */
  0x0002,   /* U+0BDB */
  0x0002,   /* U+0BDC */
  0x0002,   /* U+0BDD */
  0x0002,   /* U+0BDE */
  0x0002,   /* U+0BDF */
  0x0002,   /* U+0BE0 */
  0x0002,   /* U+0BE1 */
  0x0002,   /* U+0BE2 */
  0x0002,   /* U+0BE3 */
  0x0002,   /* U+0BE4 */
  0x0002,   /* U+0BE5 */
  0x0002,   /* U+0BE6 */
  0x0002,   /* U+0BE7 */
  0x0002,   /* U+0BE8 */
  0x0002,   /* U+0BE9 */
  0x0002,   /* U+0BEA */
  0x0002,   /* U+0BEB */
  0x0002,   /* U+0BEC */
  0x0002,   /* U+0BED */
  0x0002,   /* U+0BEE */
  0x0002,   /* U+0BEF */
  0x0002,   /* U+0BF0 */
  0x0002,   /* U+0BF1 */
  0x0002,   /* U+0BF2 */
  0x0002,   /* U+0BF3 */
  0x0002,   /* U+0BF4 */
  0x0002,   /* U+0BF5 */
  0x0002,   /* U+0BF6 */
  0x0002,   /* U+0BF7 */
  0x0002,   /* U+0BF8 */
  0x0002,   /* U+0BF9 */
  0x0002,   /* U+0BFA */
  0x0002,   /* U+0BFB */
  0x0002,   /* U+0BFC */
  0x0002,   /* U+0BFD */
  0x0002,   /* U+0BFE */
  0x0002,   /* U+0BFF */

  /* Primary weight 2 for each character. */
  0x8B00,   /* U+0B00 */
  0x0000,   /* U+0B01 */
  0x0000,   /* U+0B02 */
  0x0000,   /* U+0B03 */
  0x8B04,   /* U+0B04 */
  0x0000,   /* U+0B05 */
  0x0000,   /* U+0B06 */
  0x0000,   /* U+0B07 */
  0x0000,   /* U+0B08 */
  0x0000,   /* U+0B09 */
  0x0000,   /* U+0B0A */
  0x0000,   /* U+0B0B */
  0x0000,   /* U+0B0C */
  0x8B0D,   /* U+0B0D */
  0x8B0E,   /* U+0B0E */
  0x0000,   /* U+0B0F */
  0x0000,   /* U+0B10 */
  0x8B11,   /* U+0B11 */
  0x8B12,   /* U+0B12 */
  0x0000,   /* U+0B13 */
  0x0000,   /* U+0B14 */
  0x0000,   /* U+0B15 */
  0x0000,   /* U+0B16 */
  0x0000,   /* U+0B17 */
  0x0000,   /* U+0B18 */
  0x0000,   /* U+0B19 */
  0x0000,   /* U+0B1A */
  0x0000,   /* U+0B1B */
  0x0000,   /* U+0B1C */
  0x0000,   /* U+0B1D */
  0x0000,   /* U+0B1E */
  0x0000,   /* U+0B1F */
  0x0000,   /* U+0B20 */
  0x0000,   /* U+0B21 */
  0x0000,   /* U+0B22 */
  0x0000,   /* U+0B23 */
  0x0000,   /* U+0B24 */
  0x0000,   /* U+0B25 */
  0x0000,   /* U+0B26 */
  0x0000,   /* U+0B27 */
  0x0000,   /* U+0B28 */
  0x8B29,   /* U+0B29 */
  0x0000,   /* U+0B2A */
  0x0000,   /* U+0B2B */
  0x0000,   /* U+0B2C */
  0x0000,   /* U+0B2D */
  0x0000,   /* U+0B2E */
  0x0000,   /* U+0B2F */
  0x0000,   /* U+0B30 */
  0x8B31,   /* U+0B31 */
  0x0000,   /* U+0B32 */
  0x0000,   /* U+0B33 */
  0x8B34,   /* U+0B34 */
  0x0000,   /* U+0B35 */
  0x0000,   /* U+0B36 */
  0x0000,   /* U+0B37 */
  0x0000,   /* U+0B38 */
  0x0000,   /* U+0B39 */
  0x8B3A,   /* U+0B3A */
  0x8B3B,   /* U+0B3B */
  0x0000,   /* U+0B3C */
  0x0000,   /* U+0B3D */
  0x0000,   /* U+0B3E */
  0x0000,   /* U+0B3F */
  0x0000,   /* U+0B40 */
  0x0000,   /* U+0B41 */
  0x0000,   /* U+0B42 */
  0x0000,   /* U+0B43 */
  0x0000,   /* U+0B44 */
  0x8B45,   /* U+0B45 */
  0x8B46,   /* U+0B46 */
  0x0000,   /* U+0B47 */
  0x0000,   /* U+0B48 */
  0x8B49,   /* U+0B49 */
  0x8B4A,   /* U+0B4A */
  0x0000,   /* U+0B4B */
  0x0000,   /* U+0B4C */
  0x0000,   /* U+0B4D */
  0x8B4E,   /* U+0B4E */
  0x8B4F,   /* U+0B4F */
  0x8B50,   /* U+0B50 */
  0x8B51,   /* U+0B51 */
  0x8B52,   /* U+0B52 */
  0x8B53,   /* U+0B53 */
  0x8B54,   /* U+0B54 */
  0x8B55,   /* U+0B55 */
  0x0000,   /* U+0B56 */
  0x0000,   /* U+0B57 */
  0x8B58,   /* U+0B58 */
  0x8B59,   /* U+0B59 */
  0x8B5A,   /* U+0B5A */
  0x8B5B,   /* U+0B5B */
  0x0000,   /* U+0B5C */
  0x0000,   /* U+0B5D */
  0x8B5E,   /* U+0B5E */
  0x0000,   /* U+0B5F */
  0x0000,   /* U+0B60 */
  0x0000,   /* U+0B61 */
  0x0000,   /* U+0B62 */
  0x0000,   /* U+0B63 */
  0x8B64,   /* U+0B64 */
  0x8B65,   /* U+0B65 */
  0x0000,   /* U+0B66 */
  0x0000,   /* U+0B67 */
  0x0000,   /* U+0B68 */
  0x0000,   /* U+0B69 */
  0x0000,   /* U+0B6A */
  0x0000,   /* U+0B6B */
  0x0000,   /* U+0B6C */
  0x0000,   /* U+0B6D */
  0x0000,   /* U+0B6E */
  0x0000,   /* U+0B6F */
  0x0000,   /* U+0B70 */
  0x0000,   /* U+0B71 */
  0x0000,   /* U+0B72 */
  0x0000,   /* U+0B73 */
  0x0000,   /* U+0B74 */
  0x0000,   /* U+0B75 */
  0x0000,   /* U+0B76 */
  0x0000,   /* U+0B77 */
  0x8B78,   /* U+0B78 */
  0x8B79,   /* U+0B79 */
  0x8B7A,   /* U+0B7A */
  0x8B7B,   /* U+0B7B */
  0x8B7C,   /* U+0B7C */
  0x8B7D,   /* U+0B7D */
  0x8B7E,   /* U+0B7E */
  0x8B7F,   /* U+0B7F */
  0x8B80,   /* U+0B80 */
  0x8B81,   /* U+0B81 */
  0x0000,   /* U+0B82 */
  0x0000,   /* U+0B83 */
  0x8B84,   /* U+0B84 */
  0x0000,   /* U+0B85 */
  0x0000,   /* U+0B86 */
  0x0000,   /* U+0B87 */
  0x0000,   /* U+0B88 */
  0x0000,   /* U+0B89 */
  0x0000,   /* U+0B8A */
  0x8B8B,   /* U+0B8B */
  0x8B8C,   /* U+0B8C */
  0x8B8D,   /* U+0B8D */
  0x0000,   /* U+0B8E */
  0x0000,   /* U+0B8F */
  0x0000,   /* U+0B90 */
  0x8B91,   /* U+0B91 */
  0x0000,   /* U+0B92 */
  0x0000,   /* U+0B93 */
  0x0000,   /* U+0B94 */
  0x0000,   /* U+0B95 */
  0x8B96,   /* U+0B96 */
  0x8B97,   /* U+0B97 */
  0x8B98,   /* U+0B98 */
  0x0000,   /* U+0B99 */
  0x0000,   /* U+0B9A */
  0x8B9B,   /* U+0B9B */
  0x0000,   /* U+0B9C */
  0x8B9D,   /* U+0B9D */
  0x0000,   /* U+0B9E */
  0x0000,   /* U+0B9F */
  0x8BA0,   /* U+0BA0 */
  0x8BA1,   /* U+0BA1 */
  0x8BA2,   /* U+0BA2 */
  0x0000,   /* U+0BA3 */
  0x0000,   /* U+0BA4 */
  0x8BA5,   /* U+0BA5 */
  0x8BA6,   /* U+0BA6 */
  0x8BA7,   /* U+0BA7 */
  0x0000,   /* U+0BA8 */
  0x0000,   /* U+0BA9 */
  0x0000,   /* U+0BAA */
  0x8BAB,   /* U+0BAB */
  0x8BAC,   /* U+0BAC */
  0x8BAD,   /* U+0BAD */
  0x0000,   /* U+0BAE */
  0x0000,   /* U+0BAF */
  0x0000,   /* U+0BB0 */
  0x0000,   /* U+0BB1 */
  0x0000,   /* U+0BB2 */
  0x0000,   /* U+0BB3 */
  0x0000,   /* U+0BB4 */
  0x0000,   /* U+0BB5 */
  0x0000,   /* U+0BB6 */
  0x0000,   /* U+0BB7 */
  0x0000,   /* U+0BB8 */
  0x0000,   /* U+0BB9 */
  0x8BBA,   /* U+0BBA */
  0x8BBB,   /* U+0BBB */
  0x8BBC,   /* U+0BBC */
  0x8BBD,   /* U+0BBD */
  0x0000,   /* U+0BBE */
  0x0000,   /* U+0BBF */
  0x0000,   /* U+0BC0 */
  0x0000,   /* U+0BC1 */
  0x0000,   /* U+0BC2 */
  0x8BC3,   /* U+0BC3 */
  0x8BC4,   /* U+0BC4 */
  0x8BC5,   /* U+0BC5 */
  0x0000,   /* U+0BC6 */
  0x0000,   /* U+0BC7 */
  0x0000,   /* U+0BC8 */
  0x8BC9,   /* U+0BC9 */
  0x0000,   /* U+0BCA */
  0x0000,   /* U+0BCB */
  0x0000,   /* U+0BCC */
  0x0000,   /* U+0BCD */
  0x8BCE,   /* U+0BCE */
  0x8BCF,   /* U+0BCF */
  0x0000,   /* U+0BD0 */
  0x8BD1,   /* U+0BD1 */
  0x8BD2,   /* U+0BD2 */
  0x8BD3,   /* U+0BD3 */
  0x8BD4,   /* U+0BD4 */
  0x8BD5,   /* U+0BD5 */
  0x8BD6,   /* U+0BD6 */
  0x0000,   /* U+0BD7 */
  0x8BD8,   /* U+0BD8 */
  0x8BD9,   /* U+0BD9 */
  0x8BDA,   /* U+0BDA */
  0x8BDB,   /* U+0BDB */
  0x8BDC,   /* U+0BDC */
  0x8BDD,   /* U+0BDD */
  0x8BDE,   /* U+0BDE */
  0x8BDF,   /* U+0BDF */
  0x8BE0,   /* U+0BE0 */
  0x8BE1,   /* U+0BE1 */
  0x8BE2,   /* U+0BE2 */
  0x8BE3,   /* U+0BE3 */
  0x8BE4,   /* U+0BE4 */
  0x8BE5,   /* U+0BE5 */
  0x0000,   /* U+0BE6 */
  0x0000,   /* U+0BE7 */
  0x0000,   /* U+0BE8 */
  0x0000,   /* U+0BE9 */
  0x0000,   /* U+0BEA */
  0x0000,   /* U+0BEB */
  0x0000,   /* U+0BEC */
  0x0000,   /* U+0BED */
  0x0000,   /* U+0BEE */
  0x0000,   /* U+0BEF */
  0x0000,   /* U+0BF0 */
  0x0000,   /* U+0BF1 */
  0x0000,   /* U+0BF2 */
  0x0000,   /* U+0BF3 */
  0x0000,   /* U+0BF4 */
  0x0000,   /* U+0BF5 */
  0x0000,   /* U+0BF6 */
  0x0000,   /* U+0BF7 */
  0x0000,   /* U+0BF8 */
  0x0000,   /* U+0BF9 */
  0x0000,   /* U+0BFA */
  0x8BFB,   /* U+0BFB */
  0x8BFC,   /* U+0BFC */
  0x8BFD,   /* U+0BFD */
  0x8BFE,   /* U+0BFE */
  0x8BFF,   /* U+0BFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0B00 */
  0x0000,   /* U+0B01 */
  0x0000,   /* U+0B02 */
  0x0000,   /* U+0B03 */
  0x0000,   /* U+0B04 */
  0x0000,   /* U+0B05 */
  0x0000,   /* U+0B06 */
  0x0000,   /* U+0B07 */
  0x0000,   /* U+0B08 */
  0x0000,   /* U+0B09 */
  0x0000,   /* U+0B0A */
  0x0000,   /* U+0B0B */
  0x0000,   /* U+0B0C */
  0x0000,   /* U+0B0D */
  0x0000,   /* U+0B0E */
  0x0000,   /* U+0B0F */
  0x0000,   /* U+0B10 */
  0x0000,   /* U+0B11 */
  0x0000,   /* U+0B12 */
  0x0000,   /* U+0B13 */
  0x0000,   /* U+0B14 */
  0x0000,   /* U+0B15 */
  0x0000,   /* U+0B16 */
  0x0000,   /* U+0B17 */
  0x0000,   /* U+0B18 */
  0x0000,   /* U+0B19 */
  0x0000,   /* U+0B1A */
  0x0000,   /* U+0B1B */
  0x0000,   /* U+0B1C */
  0x0000,   /* U+0B1D */
  0x0000,   /* U+0B1E */
  0x0000,   /* U+0B1F */
  0x0000,   /* U+0B20 */
  0x0000,   /* U+0B21 */
  0x0000,   /* U+0B22 */
  0x0000,   /* U+0B23 */
  0x0000,   /* U+0B24 */
  0x0000,   /* U+0B25 */
  0x0000,   /* U+0B26 */
  0x0000,   /* U+0B27 */
  0x0000,   /* U+0B28 */
  0x0000,   /* U+0B29 */
  0x0000,   /* U+0B2A */
  0x0000,   /* U+0B2B */
  0x0000,   /* U+0B2C */
  0x0000,   /* U+0B2D */
  0x0000,   /* U+0B2E */
  0x0000,   /* U+0B2F */
  0x0000,   /* U+0B30 */
  0x0000,   /* U+0B31 */
  0x0000,   /* U+0B32 */
  0x0000,   /* U+0B33 */
  0x0000,   /* U+0B34 */
  0x0000,   /* U+0B35 */
  0x0000,   /* U+0B36 */
  0x0000,   /* U+0B37 */
  0x0000,   /* U+0B38 */
  0x0000,   /* U+0B39 */
  0x0000,   /* U+0B3A */
  0x0000,   /* U+0B3B */
  0x0000,   /* U+0B3C */
  0x0000,   /* U+0B3D */
  0x0000,   /* U+0B3E */
  0x0000,   /* U+0B3F */
  0x0000,   /* U+0B40 */
  0x0000,   /* U+0B41 */
  0x0000,   /* U+0B42 */
  0x0000,   /* U+0B43 */
  0x0000,   /* U+0B44 */
  0x0000,   /* U+0B45 */
  0x0000,   /* U+0B46 */
  0x0000,   /* U+0B47 */
  0x0000,   /* U+0B48 */
  0x0000,   /* U+0B49 */
  0x0000,   /* U+0B4A */
  0x0000,   /* U+0B4B */
  0x0000,   /* U+0B4C */
  0x0000,   /* U+0B4D */
  0x0000,   /* U+0B4E */
  0x0000,   /* U+0B4F */
  0x0000,   /* U+0B50 */
  0x0000,   /* U+0B51 */
  0x0000,   /* U+0B52 */
  0x0000,   /* U+0B53 */
  0x0000,   /* U+0B54 */
  0x0000,   /* U+0B55 */
  0x0000,   /* U+0B56 */
  0x0000,   /* U+0B57 */
  0x0000,   /* U+0B58 */
  0x0000,   /* U+0B59 */
  0x0000,   /* U+0B5A */
  0x0000,   /* U+0B5B */
  0x00C2,   /* U+0B5C */
  0x00C2,   /* U+0B5D */
  0x0000,   /* U+0B5E */
  0x0000,   /* U+0B5F */
  0x0000,   /* U+0B60 */
  0x0000,   /* U+0B61 */
  0x0000,   /* U+0B62 */
  0x0000,   /* U+0B63 */
  0x0000,   /* U+0B64 */
  0x0000,   /* U+0B65 */
  0x0000,   /* U+0B66 */
  0x0000,   /* U+0B67 */
  0x0000,   /* U+0B68 */
  0x0000,   /* U+0B69 */
  0x0000,   /* U+0B6A */
  0x0000,   /* U+0B6B */
  0x0000,   /* U+0B6C */
  0x0000,   /* U+0B6D */
  0x0000,   /* U+0B6E */
  0x0000,   /* U+0B6F */
  0x0000,   /* U+0B70 */
  0x0000,   /* U+0B71 */
  0x0000,   /* U+0B72 */
  0x0000,   /* U+0B73 */
  0x0000,   /* U+0B74 */
  0x0000,   /* U+0B75 */
  0x0000,   /* U+0B76 */
  0x0000,   /* U+0B77 */
  0x0000,   /* U+0B78 */
  0x0000,   /* U+0B79 */
  0x0000,   /* U+0B7A */
  0x0000,   /* U+0B7B */
  0x0000,   /* U+0B7C */
  0x0000,   /* U+0B7D */
  0x0000,   /* U+0B7E */
  0x0000,   /* U+0B7F */
  0x0000,   /* U+0B80 */
  0x0000,   /* U+0B81 */
  0x0000,   /* U+0B82 */
  0x0000,   /* U+0B83 */
  0x0000,   /* U+0B84 */
  0x0000,   /* U+0B85 */
  0x0000,   /* U+0B86 */
  0x0000,   /* U+0B87 */
  0x0000,   /* U+0B88 */
  0x0000,   /* U+0B89 */
  0x0000,   /* U+0B8A */
  0x0000,   /* U+0B8B */
  0x0000,   /* U+0B8C */
  0x0000,   /* U+0B8D */
  0x0000,   /* U+0B8E */
  0x0000,   /* U+0B8F */
  0x0000,   /* U+0B90 */
  0x0000,   /* U+0B91 */
  0x0000,   /* U+0B92 */
  0x0000,   /* U+0B93 */
  0x0000,   /* U+0B94 */
  0x0000,   /* U+0B95 */
  0x0000,   /* U+0B96 */
  0x0000,   /* U+0B97 */
  0x0000,   /* U+0B98 */
  0x0000,   /* U+0B99 */
  0x0000,   /* U+0B9A */
  0x0000,   /* U+0B9B */
  0x0000,   /* U+0B9C */
  0x0000,   /* U+0B9D */
  0x0000,   /* U+0B9E */
  0x0000,   /* U+0B9F */
  0x0000,   /* U+0BA0 */
  0x0000,   /* U+0BA1 */
  0x0000,   /* U+0BA2 */
  0x0000,   /* U+0BA3 */
  0x0000,   /* U+0BA4 */
  0x0000,   /* U+0BA5 */
  0x0000,   /* U+0BA6 */
  0x0000,   /* U+0BA7 */
  0x0000,   /* U+0BA8 */
  0x0000,   /* U+0BA9 */
  0x0000,   /* U+0BAA */
  0x0000,   /* U+0BAB */
  0x0000,   /* U+0BAC */
  0x0000,   /* U+0BAD */
  0x0000,   /* U+0BAE */
  0x0000,   /* U+0BAF */
  0x0000,   /* U+0BB0 */
  0x0000,   /* U+0BB1 */
  0x0000,   /* U+0BB2 */
  0x0000,   /* U+0BB3 */
  0x0000,   /* U+0BB4 */
  0x0000,   /* U+0BB5 */
  0x0000,   /* U+0BB6 */
  0x0000,   /* U+0BB7 */
  0x0000,   /* U+0BB8 */
  0x0000,   /* U+0BB9 */
  0x0000,   /* U+0BBA */
  0x0000,   /* U+0BBB */
  0x0000,   /* U+0BBC */
  0x0000,   /* U+0BBD */
  0x0000,   /* U+0BBE */
  0x0000,   /* U+0BBF */
  0x0000,   /* U+0BC0 */
  0x0000,   /* U+0BC1 */
  0x0000,   /* U+0BC2 */
  0x0000,   /* U+0BC3 */
  0x0000,   /* U+0BC4 */
  0x0000,   /* U+0BC5 */
  0x0000,   /* U+0BC6 */
  0x0000,   /* U+0BC7 */
  0x0000,   /* U+0BC8 */
  0x0000,   /* U+0BC9 */
  0x0000,   /* U+0BCA */
  0x0000,   /* U+0BCB */
  0x0000,   /* U+0BCC */
  0x0000,   /* U+0BCD */
  0x0000,   /* U+0BCE */
  0x0000,   /* U+0BCF */
  0x0000,   /* U+0BD0 */
  0x0000,   /* U+0BD1 */
  0x0000,   /* U+0BD2 */
  0x0000,   /* U+0BD3 */
  0x0000,   /* U+0BD4 */
  0x0000,   /* U+0BD5 */
  0x0000,   /* U+0BD6 */
  0x0000,   /* U+0BD7 */
  0x0000,   /* U+0BD8 */
  0x0000,   /* U+0BD9 */
  0x0000,   /* U+0BDA */
  0x0000,   /* U+0BDB */
  0x0000,   /* U+0BDC */
  0x0000,   /* U+0BDD */
  0x0000,   /* U+0BDE */
  0x0000,   /* U+0BDF */
  0x0000,   /* U+0BE0 */
  0x0000,   /* U+0BE1 */
  0x0000,   /* U+0BE2 */
  0x0000,   /* U+0BE3 */
  0x0000,   /* U+0BE4 */
  0x0000,   /* U+0BE5 */
  0x0000,   /* U+0BE6 */
  0x0000,   /* U+0BE7 */
  0x0000,   /* U+0BE8 */
  0x0000,   /* U+0BE9 */
  0x0000,   /* U+0BEA */
  0x0000,   /* U+0BEB */
  0x0000,   /* U+0BEC */
  0x0000,   /* U+0BED */
  0x0000,   /* U+0BEE */
  0x0000,   /* U+0BEF */
  0x0000,   /* U+0BF0 */
  0x0000,   /* U+0BF1 */
  0x0000,   /* U+0BF2 */
  0x0000,   /* U+0BF3 */
  0x0000,   /* U+0BF4 */
  0x0000,   /* U+0BF5 */
  0x0000,   /* U+0BF6 */
  0x0000,   /* U+0BF7 */
  0x0000,   /* U+0BF8 */
  0x0000,   /* U+0BF9 */
  0x0000,   /* U+0BFA */
  0x0000,   /* U+0BFB */
  0x0000,   /* U+0BFC */
  0x0000,   /* U+0BFD */
  0x0000,   /* U+0BFE */
  0x0000,   /* U+0BFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0B00 */
  0x0000,   /* U+0B01 */
  0x0000,   /* U+0B02 */
  0x0000,   /* U+0B03 */
  0x0000,   /* U+0B04 */
  0x0000,   /* U+0B05 */
  0x0000,   /* U+0B06 */
  0x0000,   /* U+0B07 */
  0x0000,   /* U+0B08 */
  0x0000,   /* U+0B09 */
  0x0000,   /* U+0B0A */
  0x0000,   /* U+0B0B */
  0x0000,   /* U+0B0C */
  0x0000,   /* U+0B0D */
  0x0000,   /* U+0B0E */
  0x0000,   /* U+0B0F */
  0x0000,   /* U+0B10 */
  0x0000,   /* U+0B11 */
  0x0000,   /* U+0B12 */
  0x0000,   /* U+0B13 */
  0x0000,   /* U+0B14 */
  0x0000,   /* U+0B15 */
  0x0000,   /* U+0B16 */
  0x0000,   /* U+0B17 */
  0x0000,   /* U+0B18 */
  0x0000,   /* U+0B19 */
  0x0000,   /* U+0B1A */
  0x0000,   /* U+0B1B */
  0x0000,   /* U+0B1C */
  0x0000,   /* U+0B1D */
  0x0000,   /* U+0B1E */
  0x0000,   /* U+0B1F */
  0x0000,   /* U+0B20 */
  0x0000,   /* U+0B21 */
  0x0000,   /* U+0B22 */
  0x0000,   /* U+0B23 */
  0x0000,   /* U+0B24 */
  0x0000,   /* U+0B25 */
  0x0000,   /* U+0B26 */
  0x0000,   /* U+0B27 */
  0x0000,   /* U+0B28 */
  0x0000,   /* U+0B29 */
  0x0000,   /* U+0B2A */
  0x0000,   /* U+0B2B */
  0x0000,   /* U+0B2C */
  0x0000,   /* U+0B2D */
  0x0000,   /* U+0B2E */
  0x0000,   /* U+0B2F */
  0x0000,   /* U+0B30 */
  0x0000,   /* U+0B31 */
  0x0000,   /* U+0B32 */
  0x0000,   /* U+0B33 */
  0x0000,   /* U+0B34 */
  0x0000,   /* U+0B35 */
  0x0000,   /* U+0B36 */
  0x0000,   /* U+0B37 */
  0x0000,   /* U+0B38 */
  0x0000,   /* U+0B39 */
  0x0000,   /* U+0B3A */
  0x0000,   /* U+0B3B */
  0x0000,   /* U+0B3C */
  0x0000,   /* U+0B3D */
  0x0000,   /* U+0B3E */
  0x0000,   /* U+0B3F */
  0x0000,   /* U+0B40 */
  0x0000,   /* U+0B41 */
  0x0000,   /* U+0B42 */
  0x0000,   /* U+0B43 */
  0x0000,   /* U+0B44 */
  0x0000,   /* U+0B45 */
  0x0000,   /* U+0B46 */
  0x0000,   /* U+0B47 */
  0x0000,   /* U+0B48 */
  0x0000,   /* U+0B49 */
  0x0000,   /* U+0B4A */
  0x0000,   /* U+0B4B */
  0x0000,   /* U+0B4C */
  0x0000,   /* U+0B4D */
  0x0000,   /* U+0B4E */
  0x0000,   /* U+0B4F */
  0x0000,   /* U+0B50 */
  0x0000,   /* U+0B51 */
  0x0000,   /* U+0B52 */
  0x0000,   /* U+0B53 */
  0x0000,   /* U+0B54 */
  0x0000,   /* U+0B55 */
  0x0000,   /* U+0B56 */
  0x0000,   /* U+0B57 */
  0x0000,   /* U+0B58 */
  0x0000,   /* U+0B59 */
  0x0000,   /* U+0B5A */
  0x0000,   /* U+0B5B */
  0x0002,   /* U+0B5C */
  0x0002,   /* U+0B5D */
  0x0000,   /* U+0B5E */
  0x0000,   /* U+0B5F */
  0x0000,   /* U+0B60 */
  0x0000,   /* U+0B61 */
  0x0000,   /* U+0B62 */
  0x0000,   /* U+0B63 */
  0x0000,   /* U+0B64 */
  0x0000,   /* U+0B65 */
  0x0000,   /* U+0B66 */
  0x0000,   /* U+0B67 */
  0x0000,   /* U+0B68 */
  0x0000,   /* U+0B69 */
  0x0000,   /* U+0B6A */
  0x0000,   /* U+0B6B */
  0x0000,   /* U+0B6C */
  0x0000,   /* U+0B6D */
  0x0000,   /* U+0B6E */
  0x0000,   /* U+0B6F */
  0x0000,   /* U+0B70 */
  0x0000,   /* U+0B71 */
  0x0000,   /* U+0B72 */
  0x0000,   /* U+0B73 */
  0x0000,   /* U+0B74 */
  0x0000,   /* U+0B75 */
  0x0000,   /* U+0B76 */
  0x0000,   /* U+0B77 */
  0x0000,   /* U+0B78 */
  0x0000,   /* U+0B79 */
  0x0000,   /* U+0B7A */
  0x0000,   /* U+0B7B */
  0x0000,   /* U+0B7C */
  0x0000,   /* U+0B7D */
  0x0000,   /* U+0B7E */
  0x0000,   /* U+0B7F */
  0x0000,   /* U+0B80 */
  0x0000,   /* U+0B81 */
  0x0000,   /* U+0B82 */
  0x0000,   /* U+0B83 */
  0x0000,   /* U+0B84 */
  0x0000,   /* U+0B85 */
  0x0000,   /* U+0B86 */
  0x0000,   /* U+0B87 */
  0x0000,   /* U+0B88 */
  0x0000,   /* U+0B89 */
  0x0000,   /* U+0B8A */
  0x0000,   /* U+0B8B */
  0x0000,   /* U+0B8C */
  0x0000,   /* U+0B8D */
  0x0000,   /* U+0B8E */
  0x0000,   /* U+0B8F */
  0x0000,   /* U+0B90 */
  0x0000,   /* U+0B91 */
  0x0000,   /* U+0B92 */
  0x0000,   /* U+0B93 */
  0x0000,   /* U+0B94 */
  0x0000,   /* U+0B95 */
  0x0000,   /* U+0B96 */
  0x0000,   /* U+0B97 */
  0x0000,   /* U+0B98 */
  0x0000,   /* U+0B99 */
  0x0000,   /* U+0B9A */
  0x0000,   /* U+0B9B */
  0x0000,   /* U+0B9C */
  0x0000,   /* U+0B9D */
  0x0000,   /* U+0B9E */
  0x0000,   /* U+0B9F */
  0x0000,   /* U+0BA0 */
  0x0000,   /* U+0BA1 */
  0x0000,   /* U+0BA2 */
  0x0000,   /* U+0BA3 */
  0x0000,   /* U+0BA4 */
  0x0000,   /* U+0BA5 */
  0x0000,   /* U+0BA6 */
  0x0000,   /* U+0BA7 */
  0x0000,   /* U+0BA8 */
  0x0000,   /* U+0BA9 */
  0x0000,   /* U+0BAA */
  0x0000,   /* U+0BAB */
  0x0000,   /* U+0BAC */
  0x0000,   /* U+0BAD */
  0x0000,   /* U+0BAE */
  0x0000,   /* U+0BAF */
  0x0000,   /* U+0BB0 */
  0x0000,   /* U+0BB1 */
  0x0000,   /* U+0BB2 */
  0x0000,   /* U+0BB3 */
  0x0000,   /* U+0BB4 */
  0x0000,   /* U+0BB5 */
  0x0000,   /* U+0BB6 */
  0x0000,   /* U+0BB7 */
  0x0000,   /* U+0BB8 */
  0x0000,   /* U+0BB9 */
  0x0000,   /* U+0BBA */
  0x0000,   /* U+0BBB */
  0x0000,   /* U+0BBC */
  0x0000,   /* U+0BBD */
  0x0000,   /* U+0BBE */
  0x0000,   /* U+0BBF */
  0x0000,   /* U+0BC0 */
  0x0000,   /* U+0BC1 */
  0x0000,   /* U+0BC2 */
  0x0000,   /* U+0BC3 */
  0x0000,   /* U+0BC4 */
  0x0000,   /* U+0BC5 */
  0x0000,   /* U+0BC6 */
  0x0000,   /* U+0BC7 */
  0x0000,   /* U+0BC8 */
  0x0000,   /* U+0BC9 */
  0x0000,   /* U+0BCA */
  0x0000,   /* U+0BCB */
  0x0000,   /* U+0BCC */
  0x0000,   /* U+0BCD */
  0x0000,   /* U+0BCE */
  0x0000,   /* U+0BCF */
  0x0000,   /* U+0BD0 */
  0x0000,   /* U+0BD1 */
  0x0000,   /* U+0BD2 */
  0x0000,   /* U+0BD3 */
  0x0000,   /* U+0BD4 */
  0x0000,   /* U+0BD5 */
  0x0000,   /* U+0BD6 */
  0x0000,   /* U+0BD7 */
  0x0000,   /* U+0BD8 */
  0x0000,   /* U+0BD9 */
  0x0000,   /* U+0BDA */
  0x0000,   /* U+0BDB */
  0x0000,   /* U+0BDC */
  0x0000,   /* U+0BDD */
  0x0000,   /* U+0BDE */
  0x0000,   /* U+0BDF */
  0x0000,   /* U+0BE0 */
  0x0000,   /* U+0BE1 */
  0x0000,   /* U+0BE2 */
  0x0000,   /* U+0BE3 */
  0x0000,   /* U+0BE4 */
  0x0000,   /* U+0BE5 */
  0x0000,   /* U+0BE6 */
  0x0000,   /* U+0BE7 */
  0x0000,   /* U+0BE8 */
  0x0000,   /* U+0BE9 */
  0x0000,   /* U+0BEA */
  0x0000,   /* U+0BEB */
  0x0000,   /* U+0BEC */
  0x0000,   /* U+0BED */
  0x0000,   /* U+0BEE */
  0x0000,   /* U+0BEF */
  0x0000,   /* U+0BF0 */
  0x0000,   /* U+0BF1 */
  0x0000,   /* U+0BF2 */
  0x0000,   /* U+0BF3 */
  0x0000,   /* U+0BF4 */
  0x0000,   /* U+0BF5 */
  0x0000,   /* U+0BF6 */
  0x0000,   /* U+0BF7 */
  0x0000,   /* U+0BF8 */
  0x0000,   /* U+0BF9 */
  0x0000,   /* U+0BFA */
  0x0000,   /* U+0BFB */
  0x0000,   /* U+0BFC */
  0x0000,   /* U+0BFD */
  0x0000,   /* U+0BFE */
  0x0000,   /* U+0BFF */
};

uint16 uca900_p00C[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 1, 1, 2, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 2, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+0C00 */
  0x0000,   /* U+0C01 */
  0x0000,   /* U+0C02 */
  0x0000,   /* U+0C03 */
  0xFBC0,   /* U+0C04 */
  0x27F3,   /* U+0C05 */
  0x27F4,   /* U+0C06 */
  0x27F5,   /* U+0C07 */
  0x27F6,   /* U+0C08 */
  0x27F7,   /* U+0C09 */
  0x27F8,   /* U+0C0A */
  0x27F9,   /* U+0C0B */
  0x27FB,   /* U+0C0C */
  0xFBC0,   /* U+0C0D */
  0x27FD,   /* U+0C0E */
  0x27FE,   /* U+0C0F */
  0x27FF,   /* U+0C10 */
  0xFBC0,   /* U+0C11 */
  0x2800,   /* U+0C12 */
  0x2801,   /* U+0C13 */
  0x2802,   /* U+0C14 */
  0x2803,   /* U+0C15 */
  0x2804,   /* U+0C16 */
  0x2805,   /* U+0C17 */
  0x2806,   /* U+0C18 */
  0x2807,   /* U+0C19 */
  0x2808,   /* U+0C1A */
  0x280A,   /* U+0C1B */
  0x280B,   /* U+0C1C */
  0x280D,   /* U+0C1D */
  0x280E,   /* U+0C1E */
  0x280F,   /* U+0C1F */
  0x2810,   /* U+0C20 */
  0x2811,   /* U+0C21 */
  0x2812,   /* U+0C22 */
  0x2813,   /* U+0C23 */
  0x2814,   /* U+0C24 */
  0x2815,   /* U+0C25 */
  0x2816,   /* U+0C26 */
  0x2817,   /* U+0C27 */
  0x2818,   /* U+0C28 */
  0xFBC0,   /* U+0C29 */
  0x2819,   /* U+0C2A */
  0x281A,   /* U+0C2B */
  0x281B,   /* U+0C2C */
  0x281C,   /* U+0C2D */
  0x281D,   /* U+0C2E */
  0x281E,   /* U+0C2F */
  0x281F,   /* U+0C30 */
  0x2820,   /* U+0C31 */
  0x2821,   /* U+0C32 */
  0x2827,   /* U+0C33 */
  0x2828,   /* U+0C34 */
  0x2822,   /* U+0C35 */
  0x2823,   /* U+0C36 */
  0x2824,   /* U+0C37 */
  0x2825,   /* U+0C38 */
  0x2826,   /* U+0C39 */
  0xFBC0,   /* U+0C3A */
  0xFBC0,   /* U+0C3B */
  0xFBC0,   /* U+0C3C */
  0x282A,   /* U+0C3D */
  0x282B,   /* U+0C3E */
  0x282C,   /* U+0C3F */
  0x282D,   /* U+0C40 */
  0x282E,   /* U+0C41 */
  0x282F,   /* U+0C42 */
  0x2830,   /* U+0C43 */
  0x2831,   /* U+0C44 */
  0xFBC0,   /* U+0C45 */
  0x2834,   /* U+0C46 */
  0x2835,   /* U+0C47 */
  0x2836,   /* U+0C48 */
  0xFBC0,   /* U+0C49 */
  0x2837,   /* U+0C4A */
  0x2838,   /* U+0C4B */
  0x2839,   /* U+0C4C */
  0x283A,   /* U+0C4D */
  0xFBC0,   /* U+0C4E */
  0xFBC0,   /* U+0C4F */
  0xFBC0,   /* U+0C50 */
  0xFBC0,   /* U+0C51 */
  0xFBC0,   /* U+0C52 */
  0xFBC0,   /* U+0C53 */
  0xFBC0,   /* U+0C54 */
  0x283B,   /* U+0C55 */
  0x283C,   /* U+0C56 */
  0xFBC0,   /* U+0C57 */
  0x2809,   /* U+0C58 */
  0x280C,   /* U+0C59 */
  0x2829,   /* U+0C5A */
  0xFBC0,   /* U+0C5B */
  0xFBC0,   /* U+0C5C */
  0xFBC0,   /* U+0C5D */
  0xFBC0,   /* U+0C5E */
  0xFBC0,   /* U+0C5F */
  0x27FA,   /* U+0C60 */
  0x27FC,   /* U+0C61 */
  0x2832,   /* U+0C62 */
  0x2833,   /* U+0C63 */
  0xFBC0,   /* U+0C64 */
  0xFBC0,   /* U+0C65 */
  0x1C3D,   /* U+0C66 */
  0x1C3E,   /* U+0C67 */
  0x1C3F,   /* U+0C68 */
  0x1C40,   /* U+0C69 */
  0x1C41,   /* U+0C6A */
  0x1C42,   /* U+0C6B */
  0x1C43,   /* U+0C6C */
  0x1C44,   /* U+0C6D */
  0x1C45,   /* U+0C6E */
  0x1C46,   /* U+0C6F */
  0xFBC0,   /* U+0C70 */
  0xFBC0,   /* U+0C71 */
  0xFBC0,   /* U+0C72 */
  0xFBC0,   /* U+0C73 */
  0xFBC0,   /* U+0C74 */
  0xFBC0,   /* U+0C75 */
  0xFBC0,   /* U+0C76 */
  0xFBC0,   /* U+0C77 */
  0x1C3D,   /* U+0C78 */
  0x1C3E,   /* U+0C79 */
  0x1C3F,   /* U+0C7A */
  0x1C40,   /* U+0C7B */
  0x1C3E,   /* U+0C7C */
  0x1C3F,   /* U+0C7D */
  0x1C40,   /* U+0C7E */
  0x051C,   /* U+0C7F */
  0x2874,   /* U+0C80 */
  0x0000,   /* U+0C81 */
  0x0000,   /* U+0C82 */
  0x0000,   /* U+0C83 */
  0xFBC0,   /* U+0C84 */
  0x283D,   /* U+0C85 */
  0x283E,   /* U+0C86 */
  0x283F,   /* U+0C87 */
  0x2840,   /* U+0C88 */
  0x2841,   /* U+0C89 */
  0x2842,   /* U+0C8A */
  0x2843,   /* U+0C8B */
  0x2845,   /* U+0C8C */
  0xFBC0,   /* U+0C8D */
  0x2847,   /* U+0C8E */
  0x2848,   /* U+0C8F */
  0x2849,   /* U+0C90 */
  0xFBC0,   /* U+0C91 */
  0x284A,   /* U+0C92 */
  0x284B,   /* U+0C93 */
  0x284C,   /* U+0C94 */
  0x284D,   /* U+0C95 */
  0x284E,   /* U+0C96 */
  0x284F,   /* U+0C97 */
  0x2850,   /* U+0C98 */
  0x2851,   /* U+0C99 */
  0x2852,   /* U+0C9A */
  0x2853,   /* U+0C9B */
  0x2854,   /* U+0C9C */
  0x2855,   /* U+0C9D */
  0x2856,   /* U+0C9E */
  0x2857,   /* U+0C9F */
  0x2858,   /* U+0CA0 */
  0x2859,   /* U+0CA1 */
  0x285A,   /* U+0CA2 */
  0x285B,   /* U+0CA3 */
  0x285C,   /* U+0CA4 */
  0x285D,   /* U+0CA5 */
  0x285E,   /* U+0CA6 */
  0x285F,   /* U+0CA7 */
  0x2860,   /* U+0CA8 */
  0xFBC0,   /* U+0CA9 */
  0x2861,   /* U+0CAA */
  0x2862,   /* U+0CAB */
  0x2863,   /* U+0CAC */
  0x2864,   /* U+0CAD */
  0x2865,   /* U+0CAE */
  0x2866,   /* U+0CAF */
  0x2867,   /* U+0CB0 */
  0x2868,   /* U+0CB1 */
  0x2869,   /* U+0CB2 */
  0x286F,   /* U+0CB3 */
  0xFBC0,   /* U+0CB4 */
  0x286A,   /* U+0CB5 */
  0x286B,   /* U+0CB6 */
  0x286C,   /* U+0CB7 */
  0x286D,   /* U+0CB8 */
  0x286E,   /* U+0CB9 */
  0xFBC0,   /* U+0CBA */
  0xFBC0,   /* U+0CBB */
  0x0000,   /* U+0CBC */
  0x2871,   /* U+0CBD */
  0x2875,   /* U+0CBE */
  0x2876,   /* U+0CBF */
  0x2877,   /* U+0CC0 */
  0x2878,   /* U+0CC1 */
  0x2879,   /* U+0CC2 */
  0x287A,   /* U+0CC3 */
  0x287B,   /* U+0CC4 */
  0xFBC0,   /* U+0CC5 */
  0x287E,   /* U+0CC6 */
  0x287F,   /* U+0CC7 */
  0x2880,   /* U+0CC8 */
  0xFBC0,   /* U+0CC9 */
  0x2881,   /* U+0CCA */
  0x2882,   /* U+0CCB */
  0x2883,   /* U+0CCC */
  0x2884,   /* U+0CCD */
  0xFBC0,   /* U+0CCE */
  0xFBC0,   /* U+0CCF */
  0xFBC0,   /* U+0CD0 */
  0xFBC0,   /* U+0CD1 */
  0xFBC0,   /* U+0CD2 */
  0xFBC0,   /* U+0CD3 */
  0xFBC0,   /* U+0CD4 */
  0x2885,   /* U+0CD5 */
  0x2886,   /* U+0CD6 */
  0xFBC0,   /* U+0CD7 */
  0xFBC0,   /* U+0CD8 */
  0xFBC0,   /* U+0CD9 */
  0xFBC0,   /* U+0CDA */
  0xFBC0,   /* U+0CDB */
  0xFBC0,   /* U+0CDC */
  0xFBC0,   /* U+0CDD */
  0x2870,   /* U+0CDE */
  0xFBC0,   /* U+0CDF */
  0x2844,   /* U+0CE0 */
  0x2846,   /* U+0CE1 */
  0x287C,   /* U+0CE2 */
  0x287D,   /* U+0CE3 */
  0xFBC0,   /* U+0CE4 */
  0xFBC0,   /* U+0CE5 */
  0x1C3D,   /* U+0CE6 */
  0x1C3E,   /* U+0CE7 */
  0x1C3F,   /* U+0CE8 */
  0x1C40,   /* U+0CE9 */
  0x1C41,   /* U+0CEA */
  0x1C42,   /* U+0CEB */
  0x1C43,   /* U+0CEC */
  0x1C44,   /* U+0CED */
  0x1C45,   /* U+0CEE */
  0x1C46,   /* U+0CEF */
  0xFBC0,   /* U+0CF0 */
  0x2872,   /* U+0CF1 */
  0x2873,   /* U+0CF2 */
  0xFBC0,   /* U+0CF3 */
  0xFBC0,   /* U+0CF4 */
  0xFBC0,   /* U+0CF5 */
  0xFBC0,   /* U+0CF6 */
  0xFBC0,   /* U+0CF7 */
  0xFBC0,   /* U+0CF8 */
  0xFBC0,   /* U+0CF9 */
  0xFBC0,   /* U+0CFA */
  0xFBC0,   /* U+0CFB */
  0xFBC0,   /* U+0CFC */
  0xFBC0,   /* U+0CFD */
  0xFBC0,   /* U+0CFE */
  0xFBC0,   /* U+0CFF */

  /* Secondary weight 1 for each character. */
  0x00C3,   /* U+0C00 */
  0x00C3,   /* U+0C01 */
  0x00C4,   /* U+0C02 */
  0x00C5,   /* U+0C03 */
  0x0020,   /* U+0C04 */
  0x0020,   /* U+0C05 */
  0x0020,   /* U+0C06 */
  0x0020,   /* U+0C07 */
  0x0020,   /* U+0C08 */
  0x0020,   /* U+0C09 */
  0x0020,   /* U+0C0A */
  0x0020,   /* U+0C0B */
  0x0020,   /* U+0C0C */
  0x0020,   /* U+0C0D */
  0x0020,   /* U+0C0E */
  0x0020,   /* U+0C0F */
  0x0020,   /* U+0C10 */
  0x0020,   /* U+0C11 */
  0x0020,   /* U+0C12 */
  0x0020,   /* U+0C13 */
  0x0020,   /* U+0C14 */
  0x0020,   /* U+0C15 */
  0x0020,   /* U+0C16 */
  0x0020,   /* U+0C17 */
  0x0020,   /* U+0C18 */
  0x0020,   /* U+0C19 */
  0x0020,   /* U+0C1A */
  0x0020,   /* U+0C1B */
  0x0020,   /* U+0C1C */
  0x0020,   /* U+0C1D */
  0x0020,   /* U+0C1E */
  0x0020,   /* U+0C1F */
  0x0020,   /* U+0C20 */
  0x0020,   /* U+0C21 */
  0x0020,   /* U+0C22 */
  0x0020,   /* U+0C23 */
  0x0020,   /* U+0C24 */
  0x0020,   /* U+0C25 */
  0x0020,   /* U+0C26 */
  0x0020,   /* U+0C27 */
  0x0020,   /* U+0C28 */
  0x0020,   /* U+0C29 */
  0x0020,   /* U+0C2A */
  0x0020,   /* U+0C2B */
  0x0020,   /* U+0C2C */
  0x0020,   /* U+0C2D */
  0x0020,   /* U+0C2E */
  0x0020,   /* U+0C2F */
  0x0020,   /* U+0C30 */
  0x0020,   /* U+0C31 */
  0x0020,   /* U+0C32 */
  0x0020,   /* U+0C33 */
  0x0020,   /* U+0C34 */
  0x0020,   /* U+0C35 */
  0x0020,   /* U+0C36 */
  0x0020,   /* U+0C37 */
  0x0020,   /* U+0C38 */
  0x0020,   /* U+0C39 */
  0x0020,   /* U+0C3A */
  0x0020,   /* U+0C3B */
  0x0020,   /* U+0C3C */
  0x0020,   /* U+0C3D */
  0x0020,   /* U+0C3E */
  0x0020,   /* U+0C3F */
  0x0020,   /* U+0C40 */
  0x0020,   /* U+0C41 */
  0x0020,   /* U+0C42 */
  0x0020,   /* U+0C43 */
  0x0020,   /* U+0C44 */
  0x0020,   /* U+0C45 */
  0x0020,   /* U+0C46 */
  0x0020,   /* U+0C47 */
  0x0020,   /* U+0C48 */
  0x0020,   /* U+0C49 */
  0x0020,   /* U+0C4A */
  0x0020,   /* U+0C4B */
  0x0020,   /* U+0C4C */
  0x0020,   /* U+0C4D */
  0x0020,   /* U+0C4E */
  0x0020,   /* U+0C4F */
  0x0020,   /* U+0C50 */
  0x0020,   /* U+0C51 */
  0x0020,   /* U+0C52 */
  0x0020,   /* U+0C53 */
  0x0020,   /* U+0C54 */
  0x0020,   /* U+0C55 */
  0x0020,   /* U+0C56 */
  0x0020,   /* U+0C57 */
  0x0020,   /* U+0C58 */
  0x0020,   /* U+0C59 */
  0x0020,   /* U+0C5A */
  0x0020,   /* U+0C5B */
  0x0020,   /* U+0C5C */
  0x0020,   /* U+0C5D */
  0x0020,   /* U+0C5E */
  0x0020,   /* U+0C5F */
  0x0020,   /* U+0C60 */
  0x0020,   /* U+0C61 */
  0x0020,   /* U+0C62 */
  0x0020,   /* U+0C63 */
  0x0020,   /* U+0C64 */
  0x0020,   /* U+0C65 */
  0x0020,   /* U+0C66 */
  0x0020,   /* U+0C67 */
  0x0020,   /* U+0C68 */
  0x0020,   /* U+0C69 */
  0x0020,   /* U+0C6A */
  0x0020,   /* U+0C6B */
  0x0020,   /* U+0C6C */
  0x0020,   /* U+0C6D */
  0x0020,   /* U+0C6E */
  0x0020,   /* U+0C6F */
  0x0020,   /* U+0C70 */
  0x0020,   /* U+0C71 */
  0x0020,   /* U+0C72 */
  0x0020,   /* U+0C73 */
  0x0020,   /* U+0C74 */
  0x0020,   /* U+0C75 */
  0x0020,   /* U+0C76 */
  0x0020,   /* U+0C77 */
  0x0020,   /* U+0C78 */
  0x0020,   /* U+0C79 */
  0x0020,   /* U+0C7A */
  0x0020,   /* U+0C7B */
  0x0020,   /* U+0C7C */
  0x0020,   /* U+0C7D */
  0x0020,   /* U+0C7E */
  0x0020,   /* U+0C7F */
  0x0020,   /* U+0C80 */
  0x00C3,   /* U+0C81 */
  0x00C4,   /* U+0C82 */
  0x00C5,   /* U+0C83 */
  0x0020,   /* U+0C84 */
  0x0020,   /* U+0C85 */
  0x0020,   /* U+0C86 */
  0x0020,   /* U+0C87 */
  0x0020,   /* U+0C88 */
  0x0020,   /* U+0C89 */
  0x0020,   /* U+0C8A */
  0x0020,   /* U+0C8B */
  0x0020,   /* U+0C8C */
  0x0020,   /* U+0C8D */
  0x0020,   /* U+0C8E */
  0x0020,   /* U+0C8F */
  0x0020,   /* U+0C90 */
  0x0020,   /* U+0C91 */
  0x0020,   /* U+0C92 */
  0x0020,   /* U+0C93 */
  0x0020,   /* U+0C94 */
  0x0020,   /* U+0C95 */
  0x0020,   /* U+0C96 */
  0x0020,   /* U+0C97 */
  0x0020,   /* U+0C98 */
  0x0020,   /* U+0C99 */
  0x0020,   /* U+0C9A */
  0x0020,   /* U+0C9B */
  0x0020,   /* U+0C9C */
  0x0020,   /* U+0C9D */
  0x0020,   /* U+0C9E */
  0x0020,   /* U+0C9F */
  0x0020,   /* U+0CA0 */
  0x0020,   /* U+0CA1 */
  0x0020,   /* U+0CA2 */
  0x0020,   /* U+0CA3 */
  0x0020,   /* U+0CA4 */
  0x0020,   /* U+0CA5 */
  0x0020,   /* U+0CA6 */
  0x0020,   /* U+0CA7 */
  0x0020,   /* U+0CA8 */
  0x0020,   /* U+0CA9 */
  0x0020,   /* U+0CAA */
  0x0020,   /* U+0CAB */
  0x0020,   /* U+0CAC */
  0x0020,   /* U+0CAD */
  0x0020,   /* U+0CAE */
  0x0020,   /* U+0CAF */
  0x0020,   /* U+0CB0 */
  0x0020,   /* U+0CB1 */
  0x0020,   /* U+0CB2 */
  0x0020,   /* U+0CB3 */
  0x0020,   /* U+0CB4 */
  0x0020,   /* U+0CB5 */
  0x0020,   /* U+0CB6 */
  0x0020,   /* U+0CB7 */
  0x0020,   /* U+0CB8 */
  0x0020,   /* U+0CB9 */
  0x0020,   /* U+0CBA */
  0x0020,   /* U+0CBB */
  0x00C2,   /* U+0CBC */
  0x0020,   /* U+0CBD */
  0x0020,   /* U+0CBE */
  0x0020,   /* U+0CBF */
  0x0020,   /* U+0CC0 */
  0x0020,   /* U+0CC1 */
  0x0020,   /* U+0CC2 */
  0x0020,   /* U+0CC3 */
  0x0020,   /* U+0CC4 */
  0x0020,   /* U+0CC5 */
  0x0020,   /* U+0CC6 */
  0x0020,   /* U+0CC7 */
  0x0020,   /* U+0CC8 */
  0x0020,   /* U+0CC9 */
  0x0020,   /* U+0CCA */
  0x0020,   /* U+0CCB */
  0x0020,   /* U+0CCC */
  0x0020,   /* U+0CCD */
  0x0020,   /* U+0CCE */
  0x0020,   /* U+0CCF */
  0x0020,   /* U+0CD0 */
  0x0020,   /* U+0CD1 */
  0x0020,   /* U+0CD2 */
  0x0020,   /* U+0CD3 */
  0x0020,   /* U+0CD4 */
  0x0020,   /* U+0CD5 */
  0x0020,   /* U+0CD6 */
  0x0020,   /* U+0CD7 */
  0x0020,   /* U+0CD8 */
  0x0020,   /* U+0CD9 */
  0x0020,   /* U+0CDA */
  0x0020,   /* U+0CDB */
  0x0020,   /* U+0CDC */
  0x0020,   /* U+0CDD */
  0x0020,   /* U+0CDE */
  0x0020,   /* U+0CDF */
  0x0020,   /* U+0CE0 */
  0x0020,   /* U+0CE1 */
  0x0020,   /* U+0CE2 */
  0x0020,   /* U+0CE3 */
  0x0020,   /* U+0CE4 */
  0x0020,   /* U+0CE5 */
  0x0020,   /* U+0CE6 */
  0x0020,   /* U+0CE7 */
  0x0020,   /* U+0CE8 */
  0x0020,   /* U+0CE9 */
  0x0020,   /* U+0CEA */
  0x0020,   /* U+0CEB */
  0x0020,   /* U+0CEC */
  0x0020,   /* U+0CED */
  0x0020,   /* U+0CEE */
  0x0020,   /* U+0CEF */
  0x0020,   /* U+0CF0 */
  0x0020,   /* U+0CF1 */
  0x0020,   /* U+0CF2 */
  0x0020,   /* U+0CF3 */
  0x0020,   /* U+0CF4 */
  0x0020,   /* U+0CF5 */
  0x0020,   /* U+0CF6 */
  0x0020,   /* U+0CF7 */
  0x0020,   /* U+0CF8 */
  0x0020,   /* U+0CF9 */
  0x0020,   /* U+0CFA */
  0x0020,   /* U+0CFB */
  0x0020,   /* U+0CFC */
  0x0020,   /* U+0CFD */
  0x0020,   /* U+0CFE */
  0x0020,   /* U+0CFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0C00 */
  0x0002,   /* U+0C01 */
  0x0002,   /* U+0C02 */
  0x0002,   /* U+0C03 */
  0x0002,   /* U+0C04 */
  0x0002,   /* U+0C05 */
  0x0002,   /* U+0C06 */
  0x0002,   /* U+0C07 */
  0x0002,   /* U+0C08 */
  0x0002,   /* U+0C09 */
  0x0002,   /* U+0C0A */
  0x0002,   /* U+0C0B */
  0x0002,   /* U+0C0C */
  0x0002,   /* U+0C0D */
  0x0002,   /* U+0C0E */
  0x0002,   /* U+0C0F */
  0x0002,   /* U+0C10 */
  0x0002,   /* U+0C11 */
  0x0002,   /* U+0C12 */
  0x0002,   /* U+0C13 */
  0x0002,   /* U+0C14 */
  0x0002,   /* U+0C15 */
  0x0002,   /* U+0C16 */
  0x0002,   /* U+0C17 */
  0x0002,   /* U+0C18 */
  0x0002,   /* U+0C19 */
  0x0002,   /* U+0C1A */
  0x0002,   /* U+0C1B */
  0x0002,   /* U+0C1C */
  0x0002,   /* U+0C1D */
  0x0002,   /* U+0C1E */
  0x0002,   /* U+0C1F */
  0x0002,   /* U+0C20 */
  0x0002,   /* U+0C21 */
  0x0002,   /* U+0C22 */
  0x0002,   /* U+0C23 */
  0x0002,   /* U+0C24 */
  0x0002,   /* U+0C25 */
  0x0002,   /* U+0C26 */
  0x0002,   /* U+0C27 */
  0x0002,   /* U+0C28 */
  0x0002,   /* U+0C29 */
  0x0002,   /* U+0C2A */
  0x0002,   /* U+0C2B */
  0x0002,   /* U+0C2C */
  0x0002,   /* U+0C2D */
  0x0002,   /* U+0C2E */
  0x0002,   /* U+0C2F */
  0x0002,   /* U+0C30 */
  0x0002,   /* U+0C31 */
  0x0002,   /* U+0C32 */
  0x0002,   /* U+0C33 */
  0x0002,   /* U+0C34 */
  0x0002,   /* U+0C35 */
  0x0002,   /* U+0C36 */
  0x0002,   /* U+0C37 */
  0x0002,   /* U+0C38 */
  0x0002,   /* U+0C39 */
  0x0002,   /* U+0C3A */
  0x0002,   /* U+0C3B */
  0x0002,   /* U+0C3C */
  0x0002,   /* U+0C3D */
  0x0002,   /* U+0C3E */
  0x0002,   /* U+0C3F */
  0x0002,   /* U+0C40 */
  0x0002,   /* U+0C41 */
  0x0002,   /* U+0C42 */
  0x0002,   /* U+0C43 */
  0x0002,   /* U+0C44 */
  0x0002,   /* U+0C45 */
  0x0002,   /* U+0C46 */
  0x0002,   /* U+0C47 */
  0x0002,   /* U+0C48 */
  0x0002,   /* U+0C49 */
  0x0002,   /* U+0C4A */
  0x0002,   /* U+0C4B */
  0x0002,   /* U+0C4C */
  0x0002,   /* U+0C4D */
  0x0002,   /* U+0C4E */
  0x0002,   /* U+0C4F */
  0x0002,   /* U+0C50 */
  0x0002,   /* U+0C51 */
  0x0002,   /* U+0C52 */
  0x0002,   /* U+0C53 */
  0x0002,   /* U+0C54 */
  0x0002,   /* U+0C55 */
  0x0002,   /* U+0C56 */
  0x0002,   /* U+0C57 */
  0x0002,   /* U+0C58 */
  0x0002,   /* U+0C59 */
  0x0002,   /* U+0C5A */
  0x0002,   /* U+0C5B */
  0x0002,   /* U+0C5C */
  0x0002,   /* U+0C5D */
  0x0002,   /* U+0C5E */
  0x0002,   /* U+0C5F */
  0x0002,   /* U+0C60 */
  0x0002,   /* U+0C61 */
  0x0002,   /* U+0C62 */
  0x0002,   /* U+0C63 */
  0x0002,   /* U+0C64 */
  0x0002,   /* U+0C65 */
  0x0002,   /* U+0C66 */
  0x0002,   /* U+0C67 */
  0x0002,   /* U+0C68 */
  0x0002,   /* U+0C69 */
  0x0002,   /* U+0C6A */
  0x0002,   /* U+0C6B */
  0x0002,   /* U+0C6C */
  0x0002,   /* U+0C6D */
  0x0002,   /* U+0C6E */
  0x0002,   /* U+0C6F */
  0x0002,   /* U+0C70 */
  0x0002,   /* U+0C71 */
  0x0002,   /* U+0C72 */
  0x0002,   /* U+0C73 */
  0x0002,   /* U+0C74 */
  0x0002,   /* U+0C75 */
  0x0002,   /* U+0C76 */
  0x0002,   /* U+0C77 */
  0x0002,   /* U+0C78 */
  0x0002,   /* U+0C79 */
  0x0002,   /* U+0C7A */
  0x0002,   /* U+0C7B */
  0x0002,   /* U+0C7C */
  0x0002,   /* U+0C7D */
  0x0002,   /* U+0C7E */
  0x0002,   /* U+0C7F */
  0x0002,   /* U+0C80 */
  0x0002,   /* U+0C81 */
  0x0002,   /* U+0C82 */
  0x0002,   /* U+0C83 */
  0x0002,   /* U+0C84 */
  0x0002,   /* U+0C85 */
  0x0002,   /* U+0C86 */
  0x0002,   /* U+0C87 */
  0x0002,   /* U+0C88 */
  0x0002,   /* U+0C89 */
  0x0002,   /* U+0C8A */
  0x0002,   /* U+0C8B */
  0x0002,   /* U+0C8C */
  0x0002,   /* U+0C8D */
  0x0002,   /* U+0C8E */
  0x0002,   /* U+0C8F */
  0x0002,   /* U+0C90 */
  0x0002,   /* U+0C91 */
  0x0002,   /* U+0C92 */
  0x0002,   /* U+0C93 */
  0x0002,   /* U+0C94 */
  0x0002,   /* U+0C95 */
  0x0002,   /* U+0C96 */
  0x0002,   /* U+0C97 */
  0x0002,   /* U+0C98 */
  0x0002,   /* U+0C99 */
  0x0002,   /* U+0C9A */
  0x0002,   /* U+0C9B */
  0x0002,   /* U+0C9C */
  0x0002,   /* U+0C9D */
  0x0002,   /* U+0C9E */
  0x0002,   /* U+0C9F */
  0x0002,   /* U+0CA0 */
  0x0002,   /* U+0CA1 */
  0x0002,   /* U+0CA2 */
  0x0002,   /* U+0CA3 */
  0x0002,   /* U+0CA4 */
  0x0002,   /* U+0CA5 */
  0x0002,   /* U+0CA6 */
  0x0002,   /* U+0CA7 */
  0x0002,   /* U+0CA8 */
  0x0002,   /* U+0CA9 */
  0x0002,   /* U+0CAA */
  0x0002,   /* U+0CAB */
  0x0002,   /* U+0CAC */
  0x0002,   /* U+0CAD */
  0x0002,   /* U+0CAE */
  0x0002,   /* U+0CAF */
  0x0002,   /* U+0CB0 */
  0x0002,   /* U+0CB1 */
  0x0002,   /* U+0CB2 */
  0x0002,   /* U+0CB3 */
  0x0002,   /* U+0CB4 */
  0x0002,   /* U+0CB5 */
  0x0002,   /* U+0CB6 */
  0x0002,   /* U+0CB7 */
  0x0002,   /* U+0CB8 */
  0x0002,   /* U+0CB9 */
  0x0002,   /* U+0CBA */
  0x0002,   /* U+0CBB */
  0x0002,   /* U+0CBC */
  0x0002,   /* U+0CBD */
  0x0002,   /* U+0CBE */
  0x0002,   /* U+0CBF */
  0x0002,   /* U+0CC0 */
  0x0002,   /* U+0CC1 */
  0x0002,   /* U+0CC2 */
  0x0002,   /* U+0CC3 */
  0x0002,   /* U+0CC4 */
  0x0002,   /* U+0CC5 */
  0x0002,   /* U+0CC6 */
  0x0002,   /* U+0CC7 */
  0x0002,   /* U+0CC8 */
  0x0002,   /* U+0CC9 */
  0x0002,   /* U+0CCA */
  0x0002,   /* U+0CCB */
  0x0002,   /* U+0CCC */
  0x0002,   /* U+0CCD */
  0x0002,   /* U+0CCE */
  0x0002,   /* U+0CCF */
  0x0002,   /* U+0CD0 */
  0x0002,   /* U+0CD1 */
  0x0002,   /* U+0CD2 */
  0x0002,   /* U+0CD3 */
  0x0002,   /* U+0CD4 */
  0x0002,   /* U+0CD5 */
  0x0002,   /* U+0CD6 */
  0x0002,   /* U+0CD7 */
  0x0002,   /* U+0CD8 */
  0x0002,   /* U+0CD9 */
  0x0002,   /* U+0CDA */
  0x0002,   /* U+0CDB */
  0x0002,   /* U+0CDC */
  0x0002,   /* U+0CDD */
  0x0002,   /* U+0CDE */
  0x0002,   /* U+0CDF */
  0x0002,   /* U+0CE0 */
  0x0002,   /* U+0CE1 */
  0x0002,   /* U+0CE2 */
  0x0002,   /* U+0CE3 */
  0x0002,   /* U+0CE4 */
  0x0002,   /* U+0CE5 */
  0x0002,   /* U+0CE6 */
  0x0002,   /* U+0CE7 */
  0x0002,   /* U+0CE8 */
  0x0002,   /* U+0CE9 */
  0x0002,   /* U+0CEA */
  0x0002,   /* U+0CEB */
  0x0002,   /* U+0CEC */
  0x0002,   /* U+0CED */
  0x0002,   /* U+0CEE */
  0x0002,   /* U+0CEF */
  0x0002,   /* U+0CF0 */
  0x0002,   /* U+0CF1 */
  0x0002,   /* U+0CF2 */
  0x0002,   /* U+0CF3 */
  0x0002,   /* U+0CF4 */
  0x0002,   /* U+0CF5 */
  0x0002,   /* U+0CF6 */
  0x0002,   /* U+0CF7 */
  0x0002,   /* U+0CF8 */
  0x0002,   /* U+0CF9 */
  0x0002,   /* U+0CFA */
  0x0002,   /* U+0CFB */
  0x0002,   /* U+0CFC */
  0x0002,   /* U+0CFD */
  0x0002,   /* U+0CFE */
  0x0002,   /* U+0CFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+0C00 */
  0x0000,   /* U+0C01 */
  0x0000,   /* U+0C02 */
  0x0000,   /* U+0C03 */
  0x8C04,   /* U+0C04 */
  0x0000,   /* U+0C05 */
  0x0000,   /* U+0C06 */
  0x0000,   /* U+0C07 */
  0x0000,   /* U+0C08 */
  0x0000,   /* U+0C09 */
  0x0000,   /* U+0C0A */
  0x0000,   /* U+0C0B */
  0x0000,   /* U+0C0C */
  0x8C0D,   /* U+0C0D */
  0x0000,   /* U+0C0E */
  0x0000,   /* U+0C0F */
  0x0000,   /* U+0C10 */
  0x8C11,   /* U+0C11 */
  0x0000,   /* U+0C12 */
  0x0000,   /* U+0C13 */
  0x0000,   /* U+0C14 */
  0x0000,   /* U+0C15 */
  0x0000,   /* U+0C16 */
  0x0000,   /* U+0C17 */
  0x0000,   /* U+0C18 */
  0x0000,   /* U+0C19 */
  0x0000,   /* U+0C1A */
  0x0000,   /* U+0C1B */
  0x0000,   /* U+0C1C */
  0x0000,   /* U+0C1D */
  0x0000,   /* U+0C1E */
  0x0000,   /* U+0C1F */
  0x0000,   /* U+0C20 */
  0x0000,   /* U+0C21 */
  0x0000,   /* U+0C22 */
  0x0000,   /* U+0C23 */
  0x0000,   /* U+0C24 */
  0x0000,   /* U+0C25 */
  0x0000,   /* U+0C26 */
  0x0000,   /* U+0C27 */
  0x0000,   /* U+0C28 */
  0x8C29,   /* U+0C29 */
  0x0000,   /* U+0C2A */
  0x0000,   /* U+0C2B */
  0x0000,   /* U+0C2C */
  0x0000,   /* U+0C2D */
  0x0000,   /* U+0C2E */
  0x0000,   /* U+0C2F */
  0x0000,   /* U+0C30 */
  0x0000,   /* U+0C31 */
  0x0000,   /* U+0C32 */
  0x0000,   /* U+0C33 */
  0x0000,   /* U+0C34 */
  0x0000,   /* U+0C35 */
  0x0000,   /* U+0C36 */
  0x0000,   /* U+0C37 */
  0x0000,   /* U+0C38 */
  0x0000,   /* U+0C39 */
  0x8C3A,   /* U+0C3A */
  0x8C3B,   /* U+0C3B */
  0x8C3C,   /* U+0C3C */
  0x0000,   /* U+0C3D */
  0x0000,   /* U+0C3E */
  0x0000,   /* U+0C3F */
  0x0000,   /* U+0C40 */
  0x0000,   /* U+0C41 */
  0x0000,   /* U+0C42 */
  0x0000,   /* U+0C43 */
  0x0000,   /* U+0C44 */
  0x8C45,   /* U+0C45 */
  0x0000,   /* U+0C46 */
  0x0000,   /* U+0C47 */
  0x0000,   /* U+0C48 */
  0x8C49,   /* U+0C49 */
  0x0000,   /* U+0C4A */
  0x0000,   /* U+0C4B */
  0x0000,   /* U+0C4C */
  0x0000,   /* U+0C4D */
  0x8C4E,   /* U+0C4E */
  0x8C4F,   /* U+0C4F */
  0x8C50,   /* U+0C50 */
  0x8C51,   /* U+0C51 */
  0x8C52,   /* U+0C52 */
  0x8C53,   /* U+0C53 */
  0x8C54,   /* U+0C54 */
  0x0000,   /* U+0C55 */
  0x0000,   /* U+0C56 */
  0x8C57,   /* U+0C57 */
  0x0000,   /* U+0C58 */
  0x0000,   /* U+0C59 */
  0x0000,   /* U+0C5A */
  0x8C5B,   /* U+0C5B */
  0x8C5C,   /* U+0C5C */
  0x8C5D,   /* U+0C5D */
  0x8C5E,   /* U+0C5E */
  0x8C5F,   /* U+0C5F */
  0x0000,   /* U+0C60 */
  0x0000,   /* U+0C61 */
  0x0000,   /* U+0C62 */
  0x0000,   /* U+0C63 */
  0x8C64,   /* U+0C64 */
  0x8C65,   /* U+0C65 */
  0x0000,   /* U+0C66 */
  0x0000,   /* U+0C67 */
  0x0000,   /* U+0C68 */
  0x0000,   /* U+0C69 */
  0x0000,   /* U+0C6A */
  0x0000,   /* U+0C6B */
  0x0000,   /* U+0C6C */
  0x0000,   /* U+0C6D */
  0x0000,   /* U+0C6E */
  0x0000,   /* U+0C6F */
  0x8C70,   /* U+0C70 */
  0x8C71,   /* U+0C71 */
  0x8C72,   /* U+0C72 */
  0x8C73,   /* U+0C73 */
  0x8C74,   /* U+0C74 */
  0x8C75,   /* U+0C75 */
  0x8C76,   /* U+0C76 */
  0x8C77,   /* U+0C77 */
  0x0000,   /* U+0C78 */
  0x0000,   /* U+0C79 */
  0x0000,   /* U+0C7A */
  0x0000,   /* U+0C7B */
  0x0000,   /* U+0C7C */
  0x0000,   /* U+0C7D */
  0x0000,   /* U+0C7E */
  0x0000,   /* U+0C7F */
  0x0000,   /* U+0C80 */
  0x0000,   /* U+0C81 */
  0x0000,   /* U+0C82 */
  0x0000,   /* U+0C83 */
  0x8C84,   /* U+0C84 */
  0x0000,   /* U+0C85 */
  0x0000,   /* U+0C86 */
  0x0000,   /* U+0C87 */
  0x0000,   /* U+0C88 */
  0x0000,   /* U+0C89 */
  0x0000,   /* U+0C8A */
  0x0000,   /* U+0C8B */
  0x0000,   /* U+0C8C */
  0x8C8D,   /* U+0C8D */
  0x0000,   /* U+0C8E */
  0x0000,   /* U+0C8F */
  0x0000,   /* U+0C90 */
  0x8C91,   /* U+0C91 */
  0x0000,   /* U+0C92 */
  0x0000,   /* U+0C93 */
  0x0000,   /* U+0C94 */
  0x0000,   /* U+0C95 */
  0x0000,   /* U+0C96 */
  0x0000,   /* U+0C97 */
  0x0000,   /* U+0C98 */
  0x0000,   /* U+0C99 */
  0x0000,   /* U+0C9A */
  0x0000,   /* U+0C9B */
  0x0000,   /* U+0C9C */
  0x0000,   /* U+0C9D */
  0x0000,   /* U+0C9E */
  0x0000,   /* U+0C9F */
  0x0000,   /* U+0CA0 */
  0x0000,   /* U+0CA1 */
  0x0000,   /* U+0CA2 */
  0x0000,   /* U+0CA3 */
  0x0000,   /* U+0CA4 */
  0x0000,   /* U+0CA5 */
  0x0000,   /* U+0CA6 */
  0x0000,   /* U+0CA7 */
  0x0000,   /* U+0CA8 */
  0x8CA9,   /* U+0CA9 */
  0x0000,   /* U+0CAA */
  0x0000,   /* U+0CAB */
  0x0000,   /* U+0CAC */
  0x0000,   /* U+0CAD */
  0x0000,   /* U+0CAE */
  0x0000,   /* U+0CAF */
  0x0000,   /* U+0CB0 */
  0x0000,   /* U+0CB1 */
  0x0000,   /* U+0CB2 */
  0x0000,   /* U+0CB3 */
  0x8CB4,   /* U+0CB4 */
  0x0000,   /* U+0CB5 */
  0x0000,   /* U+0CB6 */
  0x0000,   /* U+0CB7 */
  0x0000,   /* U+0CB8 */
  0x0000,   /* U+0CB9 */
  0x8CBA,   /* U+0CBA */
  0x8CBB,   /* U+0CBB */
  0x0000,   /* U+0CBC */
  0x0000,   /* U+0CBD */
  0x0000,   /* U+0CBE */
  0x0000,   /* U+0CBF */
  0x0000,   /* U+0CC0 */
  0x0000,   /* U+0CC1 */
  0x0000,   /* U+0CC2 */
  0x0000,   /* U+0CC3 */
  0x0000,   /* U+0CC4 */
  0x8CC5,   /* U+0CC5 */
  0x0000,   /* U+0CC6 */
  0x0000,   /* U+0CC7 */
  0x0000,   /* U+0CC8 */
  0x8CC9,   /* U+0CC9 */
  0x0000,   /* U+0CCA */
  0x0000,   /* U+0CCB */
  0x0000,   /* U+0CCC */
  0x0000,   /* U+0CCD */
  0x8CCE,   /* U+0CCE */
  0x8CCF,   /* U+0CCF */
  0x8CD0,   /* U+0CD0 */
  0x8CD1,   /* U+0CD1 */
  0x8CD2,   /* U+0CD2 */
  0x8CD3,   /* U+0CD3 */
  0x8CD4,   /* U+0CD4 */
  0x0000,   /* U+0CD5 */
  0x0000,   /* U+0CD6 */
  0x8CD7,   /* U+0CD7 */
  0x8CD8,   /* U+0CD8 */
  0x8CD9,   /* U+0CD9 */
  0x8CDA,   /* U+0CDA */
  0x8CDB,   /* U+0CDB */
  0x8CDC,   /* U+0CDC */
  0x8CDD,   /* U+0CDD */
  0x0000,   /* U+0CDE */
  0x8CDF,   /* U+0CDF */
  0x0000,   /* U+0CE0 */
  0x0000,   /* U+0CE1 */
  0x0000,   /* U+0CE2 */
  0x0000,   /* U+0CE3 */
  0x8CE4,   /* U+0CE4 */
  0x8CE5,   /* U+0CE5 */
  0x0000,   /* U+0CE6 */
  0x0000,   /* U+0CE7 */
  0x0000,   /* U+0CE8 */
  0x0000,   /* U+0CE9 */
  0x0000,   /* U+0CEA */
  0x0000,   /* U+0CEB */
  0x0000,   /* U+0CEC */
  0x0000,   /* U+0CED */
  0x0000,   /* U+0CEE */
  0x0000,   /* U+0CEF */
  0x8CF0,   /* U+0CF0 */
  0x0000,   /* U+0CF1 */
  0x0000,   /* U+0CF2 */
  0x8CF3,   /* U+0CF3 */
  0x8CF4,   /* U+0CF4 */
  0x8CF5,   /* U+0CF5 */
  0x8CF6,   /* U+0CF6 */
  0x8CF7,   /* U+0CF7 */
  0x8CF8,   /* U+0CF8 */
  0x8CF9,   /* U+0CF9 */
  0x8CFA,   /* U+0CFA */
  0x8CFB,   /* U+0CFB */
  0x8CFC,   /* U+0CFC */
  0x8CFD,   /* U+0CFD */
  0x8CFE,   /* U+0CFE */
  0x8CFF,   /* U+0CFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0C00 */
  0x0000,   /* U+0C01 */
  0x0000,   /* U+0C02 */
  0x0000,   /* U+0C03 */
  0x0000,   /* U+0C04 */
  0x0000,   /* U+0C05 */
  0x0000,   /* U+0C06 */
  0x0000,   /* U+0C07 */
  0x0000,   /* U+0C08 */
  0x0000,   /* U+0C09 */
  0x0000,   /* U+0C0A */
  0x0000,   /* U+0C0B */
  0x0000,   /* U+0C0C */
  0x0000,   /* U+0C0D */
  0x0000,   /* U+0C0E */
  0x0000,   /* U+0C0F */
  0x0000,   /* U+0C10 */
  0x0000,   /* U+0C11 */
  0x0000,   /* U+0C12 */
  0x0000,   /* U+0C13 */
  0x0000,   /* U+0C14 */
  0x0000,   /* U+0C15 */
  0x0000,   /* U+0C16 */
  0x0000,   /* U+0C17 */
  0x0000,   /* U+0C18 */
  0x0000,   /* U+0C19 */
  0x0000,   /* U+0C1A */
  0x0000,   /* U+0C1B */
  0x0000,   /* U+0C1C */
  0x0000,   /* U+0C1D */
  0x0000,   /* U+0C1E */
  0x0000,   /* U+0C1F */
  0x0000,   /* U+0C20 */
  0x0000,   /* U+0C21 */
  0x0000,   /* U+0C22 */
  0x0000,   /* U+0C23 */
  0x0000,   /* U+0C24 */
  0x0000,   /* U+0C25 */
  0x0000,   /* U+0C26 */
  0x0000,   /* U+0C27 */
  0x0000,   /* U+0C28 */
  0x0000,   /* U+0C29 */
  0x0000,   /* U+0C2A */
  0x0000,   /* U+0C2B */
  0x0000,   /* U+0C2C */
  0x0000,   /* U+0C2D */
  0x0000,   /* U+0C2E */
  0x0000,   /* U+0C2F */
  0x0000,   /* U+0C30 */
  0x0000,   /* U+0C31 */
  0x0000,   /* U+0C32 */
  0x0000,   /* U+0C33 */
  0x0000,   /* U+0C34 */
  0x0000,   /* U+0C35 */
  0x0000,   /* U+0C36 */
  0x0000,   /* U+0C37 */
  0x0000,   /* U+0C38 */
  0x0000,   /* U+0C39 */
  0x0000,   /* U+0C3A */
  0x0000,   /* U+0C3B */
  0x0000,   /* U+0C3C */
  0x0000,   /* U+0C3D */
  0x0000,   /* U+0C3E */
  0x0000,   /* U+0C3F */
  0x0000,   /* U+0C40 */
  0x0000,   /* U+0C41 */
  0x0000,   /* U+0C42 */
  0x0000,   /* U+0C43 */
  0x0000,   /* U+0C44 */
  0x0000,   /* U+0C45 */
  0x0000,   /* U+0C46 */
  0x0000,   /* U+0C47 */
  0x0000,   /* U+0C48 */
  0x0000,   /* U+0C49 */
  0x0000,   /* U+0C4A */
  0x0000,   /* U+0C4B */
  0x0000,   /* U+0C4C */
  0x0000,   /* U+0C4D */
  0x0000,   /* U+0C4E */
  0x0000,   /* U+0C4F */
  0x0000,   /* U+0C50 */
  0x0000,   /* U+0C51 */
  0x0000,   /* U+0C52 */
  0x0000,   /* U+0C53 */
  0x0000,   /* U+0C54 */
  0x0000,   /* U+0C55 */
  0x0000,   /* U+0C56 */
  0x0000,   /* U+0C57 */
  0x0000,   /* U+0C58 */
  0x0000,   /* U+0C59 */
  0x0000,   /* U+0C5A */
  0x0000,   /* U+0C5B */
  0x0000,   /* U+0C5C */
  0x0000,   /* U+0C5D */
  0x0000,   /* U+0C5E */
  0x0000,   /* U+0C5F */
  0x0000,   /* U+0C60 */
  0x0000,   /* U+0C61 */
  0x0000,   /* U+0C62 */
  0x0000,   /* U+0C63 */
  0x0000,   /* U+0C64 */
  0x0000,   /* U+0C65 */
  0x0000,   /* U+0C66 */
  0x0000,   /* U+0C67 */
  0x0000,   /* U+0C68 */
  0x0000,   /* U+0C69 */
  0x0000,   /* U+0C6A */
  0x0000,   /* U+0C6B */
  0x0000,   /* U+0C6C */
  0x0000,   /* U+0C6D */
  0x0000,   /* U+0C6E */
  0x0000,   /* U+0C6F */
  0x0000,   /* U+0C70 */
  0x0000,   /* U+0C71 */
  0x0000,   /* U+0C72 */
  0x0000,   /* U+0C73 */
  0x0000,   /* U+0C74 */
  0x0000,   /* U+0C75 */
  0x0000,   /* U+0C76 */
  0x0000,   /* U+0C77 */
  0x0000,   /* U+0C78 */
  0x0000,   /* U+0C79 */
  0x0000,   /* U+0C7A */
  0x0000,   /* U+0C7B */
  0x0000,   /* U+0C7C */
  0x0000,   /* U+0C7D */
  0x0000,   /* U+0C7E */
  0x0000,   /* U+0C7F */
  0x0000,   /* U+0C80 */
  0x0000,   /* U+0C81 */
  0x0000,   /* U+0C82 */
  0x0000,   /* U+0C83 */
  0x0000,   /* U+0C84 */
  0x0000,   /* U+0C85 */
  0x0000,   /* U+0C86 */
  0x0000,   /* U+0C87 */
  0x0000,   /* U+0C88 */
  0x0000,   /* U+0C89 */
  0x0000,   /* U+0C8A */
  0x0000,   /* U+0C8B */
  0x0000,   /* U+0C8C */
  0x0000,   /* U+0C8D */
  0x0000,   /* U+0C8E */
  0x0000,   /* U+0C8F */
  0x0000,   /* U+0C90 */
  0x0000,   /* U+0C91 */
  0x0000,   /* U+0C92 */
  0x0000,   /* U+0C93 */
  0x0000,   /* U+0C94 */
  0x0000,   /* U+0C95 */
  0x0000,   /* U+0C96 */
  0x0000,   /* U+0C97 */
  0x0000,   /* U+0C98 */
  0x0000,   /* U+0C99 */
  0x0000,   /* U+0C9A */
  0x0000,   /* U+0C9B */
  0x0000,   /* U+0C9C */
  0x0000,   /* U+0C9D */
  0x0000,   /* U+0C9E */
  0x0000,   /* U+0C9F */
  0x0000,   /* U+0CA0 */
  0x0000,   /* U+0CA1 */
  0x0000,   /* U+0CA2 */
  0x0000,   /* U+0CA3 */
  0x0000,   /* U+0CA4 */
  0x0000,   /* U+0CA5 */
  0x0000,   /* U+0CA6 */
  0x0000,   /* U+0CA7 */
  0x0000,   /* U+0CA8 */
  0x0000,   /* U+0CA9 */
  0x0000,   /* U+0CAA */
  0x0000,   /* U+0CAB */
  0x0000,   /* U+0CAC */
  0x0000,   /* U+0CAD */
  0x0000,   /* U+0CAE */
  0x0000,   /* U+0CAF */
  0x0000,   /* U+0CB0 */
  0x0000,   /* U+0CB1 */
  0x0000,   /* U+0CB2 */
  0x0000,   /* U+0CB3 */
  0x0000,   /* U+0CB4 */
  0x0000,   /* U+0CB5 */
  0x0000,   /* U+0CB6 */
  0x0000,   /* U+0CB7 */
  0x0000,   /* U+0CB8 */
  0x0000,   /* U+0CB9 */
  0x0000,   /* U+0CBA */
  0x0000,   /* U+0CBB */
  0x0000,   /* U+0CBC */
  0x0000,   /* U+0CBD */
  0x0000,   /* U+0CBE */
  0x0000,   /* U+0CBF */
  0x0000,   /* U+0CC0 */
  0x0000,   /* U+0CC1 */
  0x0000,   /* U+0CC2 */
  0x0000,   /* U+0CC3 */
  0x0000,   /* U+0CC4 */
  0x0000,   /* U+0CC5 */
  0x0000,   /* U+0CC6 */
  0x0000,   /* U+0CC7 */
  0x0000,   /* U+0CC8 */
  0x0000,   /* U+0CC9 */
  0x0000,   /* U+0CCA */
  0x0000,   /* U+0CCB */
  0x0000,   /* U+0CCC */
  0x0000,   /* U+0CCD */
  0x0000,   /* U+0CCE */
  0x0000,   /* U+0CCF */
  0x0000,   /* U+0CD0 */
  0x0000,   /* U+0CD1 */
  0x0000,   /* U+0CD2 */
  0x0000,   /* U+0CD3 */
  0x0000,   /* U+0CD4 */
  0x0000,   /* U+0CD5 */
  0x0000,   /* U+0CD6 */
  0x0000,   /* U+0CD7 */
  0x0000,   /* U+0CD8 */
  0x0000,   /* U+0CD9 */
  0x0000,   /* U+0CDA */
  0x0000,   /* U+0CDB */
  0x0000,   /* U+0CDC */
  0x0000,   /* U+0CDD */
  0x0000,   /* U+0CDE */
  0x0000,   /* U+0CDF */
  0x0000,   /* U+0CE0 */
  0x0000,   /* U+0CE1 */
  0x0000,   /* U+0CE2 */
  0x0000,   /* U+0CE3 */
  0x0000,   /* U+0CE4 */
  0x0000,   /* U+0CE5 */
  0x0000,   /* U+0CE6 */
  0x0000,   /* U+0CE7 */
  0x0000,   /* U+0CE8 */
  0x0000,   /* U+0CE9 */
  0x0000,   /* U+0CEA */
  0x0000,   /* U+0CEB */
  0x0000,   /* U+0CEC */
  0x0000,   /* U+0CED */
  0x0000,   /* U+0CEE */
  0x0000,   /* U+0CEF */
  0x0000,   /* U+0CF0 */
  0x0000,   /* U+0CF1 */
  0x0000,   /* U+0CF2 */
  0x0000,   /* U+0CF3 */
  0x0000,   /* U+0CF4 */
  0x0000,   /* U+0CF5 */
  0x0000,   /* U+0CF6 */
  0x0000,   /* U+0CF7 */
  0x0000,   /* U+0CF8 */
  0x0000,   /* U+0CF9 */
  0x0000,   /* U+0CFA */
  0x0000,   /* U+0CFB */
  0x0000,   /* U+0CFC */
  0x0000,   /* U+0CFD */
  0x0000,   /* U+0CFE */
  0x0000,   /* U+0CFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0C00 */
  0x0000,   /* U+0C01 */
  0x0000,   /* U+0C02 */
  0x0000,   /* U+0C03 */
  0x0000,   /* U+0C04 */
  0x0000,   /* U+0C05 */
  0x0000,   /* U+0C06 */
  0x0000,   /* U+0C07 */
  0x0000,   /* U+0C08 */
  0x0000,   /* U+0C09 */
  0x0000,   /* U+0C0A */
  0x0000,   /* U+0C0B */
  0x0000,   /* U+0C0C */
  0x0000,   /* U+0C0D */
  0x0000,   /* U+0C0E */
  0x0000,   /* U+0C0F */
  0x0000,   /* U+0C10 */
  0x0000,   /* U+0C11 */
  0x0000,   /* U+0C12 */
  0x0000,   /* U+0C13 */
  0x0000,   /* U+0C14 */
  0x0000,   /* U+0C15 */
  0x0000,   /* U+0C16 */
  0x0000,   /* U+0C17 */
  0x0000,   /* U+0C18 */
  0x0000,   /* U+0C19 */
  0x0000,   /* U+0C1A */
  0x0000,   /* U+0C1B */
  0x0000,   /* U+0C1C */
  0x0000,   /* U+0C1D */
  0x0000,   /* U+0C1E */
  0x0000,   /* U+0C1F */
  0x0000,   /* U+0C20 */
  0x0000,   /* U+0C21 */
  0x0000,   /* U+0C22 */
  0x0000,   /* U+0C23 */
  0x0000,   /* U+0C24 */
  0x0000,   /* U+0C25 */
  0x0000,   /* U+0C26 */
  0x0000,   /* U+0C27 */
  0x0000,   /* U+0C28 */
  0x0000,   /* U+0C29 */
  0x0000,   /* U+0C2A */
  0x0000,   /* U+0C2B */
  0x0000,   /* U+0C2C */
  0x0000,   /* U+0C2D */
  0x0000,   /* U+0C2E */
  0x0000,   /* U+0C2F */
  0x0000,   /* U+0C30 */
  0x0000,   /* U+0C31 */
  0x0000,   /* U+0C32 */
  0x0000,   /* U+0C33 */
  0x0000,   /* U+0C34 */
  0x0000,   /* U+0C35 */
  0x0000,   /* U+0C36 */
  0x0000,   /* U+0C37 */
  0x0000,   /* U+0C38 */
  0x0000,   /* U+0C39 */
  0x0000,   /* U+0C3A */
  0x0000,   /* U+0C3B */
  0x0000,   /* U+0C3C */
  0x0000,   /* U+0C3D */
  0x0000,   /* U+0C3E */
  0x0000,   /* U+0C3F */
  0x0000,   /* U+0C40 */
  0x0000,   /* U+0C41 */
  0x0000,   /* U+0C42 */
  0x0000,   /* U+0C43 */
  0x0000,   /* U+0C44 */
  0x0000,   /* U+0C45 */
  0x0000,   /* U+0C46 */
  0x0000,   /* U+0C47 */
  0x0000,   /* U+0C48 */
  0x0000,   /* U+0C49 */
  0x0000,   /* U+0C4A */
  0x0000,   /* U+0C4B */
  0x0000,   /* U+0C4C */
  0x0000,   /* U+0C4D */
  0x0000,   /* U+0C4E */
  0x0000,   /* U+0C4F */
  0x0000,   /* U+0C50 */
  0x0000,   /* U+0C51 */
  0x0000,   /* U+0C52 */
  0x0000,   /* U+0C53 */
  0x0000,   /* U+0C54 */
  0x0000,   /* U+0C55 */
  0x0000,   /* U+0C56 */
  0x0000,   /* U+0C57 */
  0x0000,   /* U+0C58 */
  0x0000,   /* U+0C59 */
  0x0000,   /* U+0C5A */
  0x0000,   /* U+0C5B */
  0x0000,   /* U+0C5C */
  0x0000,   /* U+0C5D */
  0x0000,   /* U+0C5E */
  0x0000,   /* U+0C5F */
  0x0000,   /* U+0C60 */
  0x0000,   /* U+0C61 */
  0x0000,   /* U+0C62 */
  0x0000,   /* U+0C63 */
  0x0000,   /* U+0C64 */
  0x0000,   /* U+0C65 */
  0x0000,   /* U+0C66 */
  0x0000,   /* U+0C67 */
  0x0000,   /* U+0C68 */
  0x0000,   /* U+0C69 */
  0x0000,   /* U+0C6A */
  0x0000,   /* U+0C6B */
  0x0000,   /* U+0C6C */
  0x0000,   /* U+0C6D */
  0x0000,   /* U+0C6E */
  0x0000,   /* U+0C6F */
  0x0000,   /* U+0C70 */
  0x0000,   /* U+0C71 */
  0x0000,   /* U+0C72 */
  0x0000,   /* U+0C73 */
  0x0000,   /* U+0C74 */
  0x0000,   /* U+0C75 */
  0x0000,   /* U+0C76 */
  0x0000,   /* U+0C77 */
  0x0000,   /* U+0C78 */
  0x0000,   /* U+0C79 */
  0x0000,   /* U+0C7A */
  0x0000,   /* U+0C7B */
  0x0000,   /* U+0C7C */
  0x0000,   /* U+0C7D */
  0x0000,   /* U+0C7E */
  0x0000,   /* U+0C7F */
  0x0000,   /* U+0C80 */
  0x0000,   /* U+0C81 */
  0x0000,   /* U+0C82 */
  0x0000,   /* U+0C83 */
  0x0000,   /* U+0C84 */
  0x0000,   /* U+0C85 */
  0x0000,   /* U+0C86 */
  0x0000,   /* U+0C87 */
  0x0000,   /* U+0C88 */
  0x0000,   /* U+0C89 */
  0x0000,   /* U+0C8A */
  0x0000,   /* U+0C8B */
  0x0000,   /* U+0C8C */
  0x0000,   /* U+0C8D */
  0x0000,   /* U+0C8E */
  0x0000,   /* U+0C8F */
  0x0000,   /* U+0C90 */
  0x0000,   /* U+0C91 */
  0x0000,   /* U+0C92 */
  0x0000,   /* U+0C93 */
  0x0000,   /* U+0C94 */
  0x0000,   /* U+0C95 */
  0x0000,   /* U+0C96 */
  0x0000,   /* U+0C97 */
  0x0000,   /* U+0C98 */
  0x0000,   /* U+0C99 */
  0x0000,   /* U+0C9A */
  0x0000,   /* U+0C9B */
  0x0000,   /* U+0C9C */
  0x0000,   /* U+0C9D */
  0x0000,   /* U+0C9E */
  0x0000,   /* U+0C9F */
  0x0000,   /* U+0CA0 */
  0x0000,   /* U+0CA1 */
  0x0000,   /* U+0CA2 */
  0x0000,   /* U+0CA3 */
  0x0000,   /* U+0CA4 */
  0x0000,   /* U+0CA5 */
  0x0000,   /* U+0CA6 */
  0x0000,   /* U+0CA7 */
  0x0000,   /* U+0CA8 */
  0x0000,   /* U+0CA9 */
  0x0000,   /* U+0CAA */
  0x0000,   /* U+0CAB */
  0x0000,   /* U+0CAC */
  0x0000,   /* U+0CAD */
  0x0000,   /* U+0CAE */
  0x0000,   /* U+0CAF */
  0x0000,   /* U+0CB0 */
  0x0000,   /* U+0CB1 */
  0x0000,   /* U+0CB2 */
  0x0000,   /* U+0CB3 */
  0x0000,   /* U+0CB4 */
  0x0000,   /* U+0CB5 */
  0x0000,   /* U+0CB6 */
  0x0000,   /* U+0CB7 */
  0x0000,   /* U+0CB8 */
  0x0000,   /* U+0CB9 */
  0x0000,   /* U+0CBA */
  0x0000,   /* U+0CBB */
  0x0000,   /* U+0CBC */
  0x0000,   /* U+0CBD */
  0x0000,   /* U+0CBE */
  0x0000,   /* U+0CBF */
  0x0000,   /* U+0CC0 */
  0x0000,   /* U+0CC1 */
  0x0000,   /* U+0CC2 */
  0x0000,   /* U+0CC3 */
  0x0000,   /* U+0CC4 */
  0x0000,   /* U+0CC5 */
  0x0000,   /* U+0CC6 */
  0x0000,   /* U+0CC7 */
  0x0000,   /* U+0CC8 */
  0x0000,   /* U+0CC9 */
  0x0000,   /* U+0CCA */
  0x0000,   /* U+0CCB */
  0x0000,   /* U+0CCC */
  0x0000,   /* U+0CCD */
  0x0000,   /* U+0CCE */
  0x0000,   /* U+0CCF */
  0x0000,   /* U+0CD0 */
  0x0000,   /* U+0CD1 */
  0x0000,   /* U+0CD2 */
  0x0000,   /* U+0CD3 */
  0x0000,   /* U+0CD4 */
  0x0000,   /* U+0CD5 */
  0x0000,   /* U+0CD6 */
  0x0000,   /* U+0CD7 */
  0x0000,   /* U+0CD8 */
  0x0000,   /* U+0CD9 */
  0x0000,   /* U+0CDA */
  0x0000,   /* U+0CDB */
  0x0000,   /* U+0CDC */
  0x0000,   /* U+0CDD */
  0x0000,   /* U+0CDE */
  0x0000,   /* U+0CDF */
  0x0000,   /* U+0CE0 */
  0x0000,   /* U+0CE1 */
  0x0000,   /* U+0CE2 */
  0x0000,   /* U+0CE3 */
  0x0000,   /* U+0CE4 */
  0x0000,   /* U+0CE5 */
  0x0000,   /* U+0CE6 */
  0x0000,   /* U+0CE7 */
  0x0000,   /* U+0CE8 */
  0x0000,   /* U+0CE9 */
  0x0000,   /* U+0CEA */
  0x0000,   /* U+0CEB */
  0x0000,   /* U+0CEC */
  0x0000,   /* U+0CED */
  0x0000,   /* U+0CEE */
  0x0000,   /* U+0CEF */
  0x0000,   /* U+0CF0 */
  0x0000,   /* U+0CF1 */
  0x0000,   /* U+0CF2 */
  0x0000,   /* U+0CF3 */
  0x0000,   /* U+0CF4 */
  0x0000,   /* U+0CF5 */
  0x0000,   /* U+0CF6 */
  0x0000,   /* U+0CF7 */
  0x0000,   /* U+0CF8 */
  0x0000,   /* U+0CF9 */
  0x0000,   /* U+0CFA */
  0x0000,   /* U+0CFB */
  0x0000,   /* U+0CFC */
  0x0000,   /* U+0CFD */
  0x0000,   /* U+0CFE */
  0x0000,   /* U+0CFF */
};

uint16 uca900_p00D[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 
  2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC0,   /* U+0D00 */
  0x0000,   /* U+0D01 */
  0x0000,   /* U+0D02 */
  0x0000,   /* U+0D03 */
  0xFBC0,   /* U+0D04 */
  0x2887,   /* U+0D05 */
  0x2888,   /* U+0D06 */
  0x2889,   /* U+0D07 */
  0x288A,   /* U+0D08 */
  0x288C,   /* U+0D09 */
  0x288D,   /* U+0D0A */
  0x288E,   /* U+0D0B */
  0x2890,   /* U+0D0C */
  0xFBC0,   /* U+0D0D */
  0x2892,   /* U+0D0E */
  0x2893,   /* U+0D0F */
  0x2894,   /* U+0D10 */
  0xFBC0,   /* U+0D11 */
  0x2895,   /* U+0D12 */
  0x2896,   /* U+0D13 */
  0x2897,   /* U+0D14 */
  0x2898,   /* U+0D15 */
  0x2899,   /* U+0D16 */
  0x289A,   /* U+0D17 */
  0x289B,   /* U+0D18 */
  0x289C,   /* U+0D19 */
  0x289D,   /* U+0D1A */
  0x289E,   /* U+0D1B */
  0x289F,   /* U+0D1C */
  0x28A0,   /* U+0D1D */
  0x28A1,   /* U+0D1E */
  0x28A2,   /* U+0D1F */
  0x28A3,   /* U+0D20 */
  0x28A4,   /* U+0D21 */
  0x28A5,   /* U+0D22 */
  0x28A6,   /* U+0D23 */
  0x28A7,   /* U+0D24 */
  0x28A8,   /* U+0D25 */
  0x28A9,   /* U+0D26 */
  0x28AA,   /* U+0D27 */
  0x28AB,   /* U+0D28 */
  0x28AC,   /* U+0D29 */
  0x28AD,   /* U+0D2A */
  0x28AE,   /* U+0D2B */
  0x28AF,   /* U+0D2C */
  0x28B0,   /* U+0D2D */
  0x28B1,   /* U+0D2E */
  0x28B2,   /* U+0D2F */
  0x28B3,   /* U+0D30 */
  0x28BC,   /* U+0D31 */
  0x28B4,   /* U+0D32 */
  0x28BA,   /* U+0D33 */
  0x28BB,   /* U+0D34 */
  0x28B5,   /* U+0D35 */
  0x28B6,   /* U+0D36 */
  0x28B7,   /* U+0D37 */
  0x28B8,   /* U+0D38 */
  0x28B9,   /* U+0D39 */
  0x28BD,   /* U+0D3A */
  0xFBC0,   /* U+0D3B */
  0xFBC0,   /* U+0D3C */
  0x28BE,   /* U+0D3D */
  0x28BF,   /* U+0D3E */
  0x28C0,   /* U+0D3F */
  0x28C1,   /* U+0D40 */
  0x28C2,   /* U+0D41 */
  0x28C3,   /* U+0D42 */
  0x28C4,   /* U+0D43 */
  0x28C5,   /* U+0D44 */
  0xFBC0,   /* U+0D45 */
  0x28C8,   /* U+0D46 */
  0x28C9,   /* U+0D47 */
  0x28CA,   /* U+0D48 */
  0xFBC0,   /* U+0D49 */
  0x28CB,   /* U+0D4A */
  0x28CC,   /* U+0D4B */
  0x28CD,   /* U+0D4C */
  0x28CF,   /* U+0D4D */
  0x28B3,   /* U+0D4E */
  0x051D,   /* U+0D4F */
  0xFBC0,   /* U+0D50 */
  0xFBC0,   /* U+0D51 */
  0xFBC0,   /* U+0D52 */
  0xFBC0,   /* U+0D53 */
  0x28B1,   /* U+0D54 */
  0x28B2,   /* U+0D55 */
  0x28BB,   /* U+0D56 */
  0x28CE,   /* U+0D57 */
  0x1AAB,   /* U+0D58 */
  0x1AAC,   /* U+0D59 */
  0x1AAD,   /* U+0D5A */
  0x1AAE,   /* U+0D5B */
  0x1AAF,   /* U+0D5C */
  0x1AB0,   /* U+0D5D */
  0x1AB1,   /* U+0D5E */
  0x288B,   /* U+0D5F */
  0x288F,   /* U+0D60 */
  0x2891,   /* U+0D61 */
  0x28C6,   /* U+0D62 */
  0x28C7,   /* U+0D63 */
  0xFBC0,   /* U+0D64 */
  0xFBC0,   /* U+0D65 */
  0x1C3D,   /* U+0D66 */
  0x1C3E,   /* U+0D67 */
  0x1C3F,   /* U+0D68 */
  0x1C40,   /* U+0D69 */
  0x1C41,   /* U+0D6A */
  0x1C42,   /* U+0D6B */
  0x1C43,   /* U+0D6C */
  0x1C44,   /* U+0D6D */
  0x1C45,   /* U+0D6E */
  0x1C46,   /* U+0D6F */
  0x1AB2,   /* U+0D70 */
  0x1AB3,   /* U+0D71 */
  0x1AB4,   /* U+0D72 */
  0x1AB5,   /* U+0D73 */
  0x1AB6,   /* U+0D74 */
  0x1AB7,   /* U+0D75 */
  0x1AB8,   /* U+0D76 */
  0x1AB9,   /* U+0D77 */
  0x1ABA,   /* U+0D78 */
  0x051E,   /* U+0D79 */
  0x28A6,   /* U+0D7A */
  0x28AB,   /* U+0D7B */
  0x28B3,   /* U+0D7C */
  0x28B4,   /* U+0D7D */
  0x28BA,   /* U+0D7E */
  0x2898,   /* U+0D7F */
  0xFBC0,   /* U+0D80 */
  0xFBC0,   /* U+0D81 */
  0x0000,   /* U+0D82 */
  0x0000,   /* U+0D83 */
  0xFBC0,   /* U+0D84 */
  0x28D0,   /* U+0D85 */
  0x28D1,   /* U+0D86 */
  0x28D2,   /* U+0D87 */
  0x28D3,   /* U+0D88 */
  0x28D4,   /* U+0D89 */
  0x28D5,   /* U+0D8A */
  0x28D6,   /* U+0D8B */
  0x28D7,   /* U+0D8C */
  0x28D8,   /* U+0D8D */
  0x28D9,   /* U+0D8E */
  0x28DA,   /* U+0D8F */
  0x28DB,   /* U+0D90 */
  0x28DC,   /* U+0D91 */
  0x28DD,   /* U+0D92 */
  0x28DE,   /* U+0D93 */
  0x28DF,   /* U+0D94 */
  0x28E0,   /* U+0D95 */
  0x28E1,   /* U+0D96 */
  0xFBC0,   /* U+0D97 */
  0xFBC0,   /* U+0D98 */
  0xFBC0,   /* U+0D99 */
  0x28E2,   /* U+0D9A */
  0x28E3,   /* U+0D9B */
  0x28E4,   /* U+0D9C */
  0x28E5,   /* U+0D9D */
  0x28E6,   /* U+0D9E */
  0x28E7,   /* U+0D9F */
  0x28E8,   /* U+0DA0 */
  0x28E9,   /* U+0DA1 */
  0x28EA,   /* U+0DA2 */
  0x28EB,   /* U+0DA3 */
  0x28EC,   /* U+0DA4 */
  0x28ED,   /* U+0DA5 */
  0x28EE,   /* U+0DA6 */
  0x28EF,   /* U+0DA7 */
  0x28F0,   /* U+0DA8 */
  0x28F1,   /* U+0DA9 */
  0x28F2,   /* U+0DAA */
  0x28F3,   /* U+0DAB */
  0x28F4,   /* U+0DAC */
  0x28F5,   /* U+0DAD */
  0x28F6,   /* U+0DAE */
  0x28F7,   /* U+0DAF */
  0x28F8,   /* U+0DB0 */
  0x28F9,   /* U+0DB1 */
  0xFBC0,   /* U+0DB2 */
  0x28FA,   /* U+0DB3 */
  0x28FB,   /* U+0DB4 */
  0x28FC,   /* U+0DB5 */
  0x28FD,   /* U+0DB6 */
  0x28FE,   /* U+0DB7 */
  0x28FF,   /* U+0DB8 */
  0x2900,   /* U+0DB9 */
  0x2901,   /* U+0DBA */
  0x2902,   /* U+0DBB */
  0xFBC0,   /* U+0DBC */
  0x2903,   /* U+0DBD */
  0xFBC0,   /* U+0DBE */
  0xFBC0,   /* U+0DBF */
  0x2904,   /* U+0DC0 */
  0x2905,   /* U+0DC1 */
  0x2906,   /* U+0DC2 */
  0x2907,   /* U+0DC3 */
  0x2908,   /* U+0DC4 */
  0x2909,   /* U+0DC5 */
  0x290A,   /* U+0DC6 */
  0xFBC0,   /* U+0DC7 */
  0xFBC0,   /* U+0DC8 */
  0xFBC0,   /* U+0DC9 */
  0x291C,   /* U+0DCA */
  0xFBC0,   /* U+0DCB */
  0xFBC0,   /* U+0DCC */
  0xFBC0,   /* U+0DCD */
  0xFBC0,   /* U+0DCE */
  0x290B,   /* U+0DCF */
  0x290C,   /* U+0DD0 */
  0x290D,   /* U+0DD1 */
  0x290E,   /* U+0DD2 */
  0x290F,   /* U+0DD3 */
  0x2910,   /* U+0DD4 */
  0xFBC0,   /* U+0DD5 */
  0x2911,   /* U+0DD6 */
  0xFBC0,   /* U+0DD7 */
  0x2912,   /* U+0DD8 */
  0x2916,   /* U+0DD9 */
  0x2917,   /* U+0DDA */
  0x2918,   /* U+0DDB */
  0x2919,   /* U+0DDC */
  0x291A,   /* U+0DDD */
  0x291B,   /* U+0DDE */
  0x2914,   /* U+0DDF */
  0xFBC0,   /* U+0DE0 */
  0xFBC0,   /* U+0DE1 */
  0xFBC0,   /* U+0DE2 */
  0xFBC0,   /* U+0DE3 */
  0xFBC0,   /* U+0DE4 */
  0xFBC0,   /* U+0DE5 */
  0x1C3D,   /* U+0DE6 */
  0x1C3E,   /* U+0DE7 */
  0x1C3F,   /* U+0DE8 */
  0x1C40,   /* U+0DE9 */
  0x1C41,   /* U+0DEA */
  0x1C42,   /* U+0DEB */
  0x1C43,   /* U+0DEC */
  0x1C44,   /* U+0DED */
  0x1C45,   /* U+0DEE */
  0x1C46,   /* U+0DEF */
  0xFBC0,   /* U+0DF0 */
  0xFBC0,   /* U+0DF1 */
  0x2913,   /* U+0DF2 */
  0x2915,   /* U+0DF3 */
  0x03ED,   /* U+0DF4 */
  0xFBC0,   /* U+0DF5 */
  0xFBC0,   /* U+0DF6 */
  0xFBC0,   /* U+0DF7 */
  0xFBC0,   /* U+0DF8 */
  0xFBC0,   /* U+0DF9 */
  0xFBC0,   /* U+0DFA */
  0xFBC0,   /* U+0DFB */
  0xFBC0,   /* U+0DFC */
  0xFBC0,   /* U+0DFD */
  0xFBC0,   /* U+0DFE */
  0xFBC0,   /* U+0DFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0D00 */
  0x00C3,   /* U+0D01 */
  0x00C4,   /* U+0D02 */
  0x00C5,   /* U+0D03 */
  0x0020,   /* U+0D04 */
  0x0020,   /* U+0D05 */
  0x0020,   /* U+0D06 */
  0x0020,   /* U+0D07 */
  0x0020,   /* U+0D08 */
  0x0020,   /* U+0D09 */
  0x0020,   /* U+0D0A */
  0x0020,   /* U+0D0B */
  0x0020,   /* U+0D0C */
  0x0020,   /* U+0D0D */
  0x0020,   /* U+0D0E */
  0x0020,   /* U+0D0F */
  0x0020,   /* U+0D10 */
  0x0020,   /* U+0D11 */
  0x0020,   /* U+0D12 */
  0x0020,   /* U+0D13 */
  0x0020,   /* U+0D14 */
  0x0020,   /* U+0D15 */
  0x0020,   /* U+0D16 */
  0x0020,   /* U+0D17 */
  0x0020,   /* U+0D18 */
  0x0020,   /* U+0D19 */
  0x0020,   /* U+0D1A */
  0x0020,   /* U+0D1B */
  0x0020,   /* U+0D1C */
  0x0020,   /* U+0D1D */
  0x0020,   /* U+0D1E */
  0x0020,   /* U+0D1F */
  0x0020,   /* U+0D20 */
  0x0020,   /* U+0D21 */
  0x0020,   /* U+0D22 */
  0x0020,   /* U+0D23 */
  0x0020,   /* U+0D24 */
  0x0020,   /* U+0D25 */
  0x0020,   /* U+0D26 */
  0x0020,   /* U+0D27 */
  0x0020,   /* U+0D28 */
  0x0020,   /* U+0D29 */
  0x0020,   /* U+0D2A */
  0x0020,   /* U+0D2B */
  0x0020,   /* U+0D2C */
  0x0020,   /* U+0D2D */
  0x0020,   /* U+0D2E */
  0x0020,   /* U+0D2F */
  0x0020,   /* U+0D30 */
  0x0020,   /* U+0D31 */
  0x0020,   /* U+0D32 */
  0x0020,   /* U+0D33 */
  0x0020,   /* U+0D34 */
  0x0020,   /* U+0D35 */
  0x0020,   /* U+0D36 */
  0x0020,   /* U+0D37 */
  0x0020,   /* U+0D38 */
  0x0020,   /* U+0D39 */
  0x0020,   /* U+0D3A */
  0x0020,   /* U+0D3B */
  0x0020,   /* U+0D3C */
  0x0020,   /* U+0D3D */
  0x0020,   /* U+0D3E */
  0x0020,   /* U+0D3F */
  0x0020,   /* U+0D40 */
  0x0020,   /* U+0D41 */
  0x0020,   /* U+0D42 */
  0x0020,   /* U+0D43 */
  0x0020,   /* U+0D44 */
  0x0020,   /* U+0D45 */
  0x0020,   /* U+0D46 */
  0x0020,   /* U+0D47 */
  0x0020,   /* U+0D48 */
  0x0020,   /* U+0D49 */
  0x0020,   /* U+0D4A */
  0x0020,   /* U+0D4B */
  0x0020,   /* U+0D4C */
  0x0020,   /* U+0D4D */
  0x0020,   /* U+0D4E */
  0x0020,   /* U+0D4F */
  0x0020,   /* U+0D50 */
  0x0020,   /* U+0D51 */
  0x0020,   /* U+0D52 */
  0x0020,   /* U+0D53 */
  0x0020,   /* U+0D54 */
  0x0020,   /* U+0D55 */
  0x0020,   /* U+0D56 */
  0x0020,   /* U+0D57 */
  0x0020,   /* U+0D58 */
  0x0020,   /* U+0D59 */
  0x0020,   /* U+0D5A */
  0x0020,   /* U+0D5B */
  0x0020,   /* U+0D5C */
  0x0020,   /* U+0D5D */
  0x0020,   /* U+0D5E */
  0x0020,   /* U+0D5F */
  0x0020,   /* U+0D60 */
  0x0020,   /* U+0D61 */
  0x0020,   /* U+0D62 */
  0x0020,   /* U+0D63 */
  0x0020,   /* U+0D64 */
  0x0020,   /* U+0D65 */
  0x0020,   /* U+0D66 */
  0x0020,   /* U+0D67 */
  0x0020,   /* U+0D68 */
  0x0020,   /* U+0D69 */
  0x0020,   /* U+0D6A */
  0x0020,   /* U+0D6B */
  0x0020,   /* U+0D6C */
  0x0020,   /* U+0D6D */
  0x0020,   /* U+0D6E */
  0x0020,   /* U+0D6F */
  0x0020,   /* U+0D70 */
  0x0020,   /* U+0D71 */
  0x0020,   /* U+0D72 */
  0x0020,   /* U+0D73 */
  0x0020,   /* U+0D74 */
  0x0020,   /* U+0D75 */
  0x0020,   /* U+0D76 */
  0x0020,   /* U+0D77 */
  0x0020,   /* U+0D78 */
  0x0020,   /* U+0D79 */
  0x0020,   /* U+0D7A */
  0x0020,   /* U+0D7B */
  0x0020,   /* U+0D7C */
  0x0020,   /* U+0D7D */
  0x0020,   /* U+0D7E */
  0x0020,   /* U+0D7F */
  0x0020,   /* U+0D80 */
  0x0020,   /* U+0D81 */
  0x00C4,   /* U+0D82 */
  0x00C5,   /* U+0D83 */
  0x0020,   /* U+0D84 */
  0x0020,   /* U+0D85 */
  0x0020,   /* U+0D86 */
  0x0020,   /* U+0D87 */
  0x0020,   /* U+0D88 */
  0x0020,   /* U+0D89 */
  0x0020,   /* U+0D8A */
  0x0020,   /* U+0D8B */
  0x0020,   /* U+0D8C */
  0x0020,   /* U+0D8D */
  0x0020,   /* U+0D8E */
  0x0020,   /* U+0D8F */
  0x0020,   /* U+0D90 */
  0x0020,   /* U+0D91 */
  0x0020,   /* U+0D92 */
  0x0020,   /* U+0D93 */
  0x0020,   /* U+0D94 */
  0x0020,   /* U+0D95 */
  0x0020,   /* U+0D96 */
  0x0020,   /* U+0D97 */
  0x0020,   /* U+0D98 */
  0x0020,   /* U+0D99 */
  0x0020,   /* U+0D9A */
  0x0020,   /* U+0D9B */
  0x0020,   /* U+0D9C */
  0x0020,   /* U+0D9D */
  0x0020,   /* U+0D9E */
  0x0020,   /* U+0D9F */
  0x0020,   /* U+0DA0 */
  0x0020,   /* U+0DA1 */
  0x0020,   /* U+0DA2 */
  0x0020,   /* U+0DA3 */
  0x0020,   /* U+0DA4 */
  0x0020,   /* U+0DA5 */
  0x0020,   /* U+0DA6 */
  0x0020,   /* U+0DA7 */
  0x0020,   /* U+0DA8 */
  0x0020,   /* U+0DA9 */
  0x0020,   /* U+0DAA */
  0x0020,   /* U+0DAB */
  0x0020,   /* U+0DAC */
  0x0020,   /* U+0DAD */
  0x0020,   /* U+0DAE */
  0x0020,   /* U+0DAF */
  0x0020,   /* U+0DB0 */
  0x0020,   /* U+0DB1 */
  0x0020,   /* U+0DB2 */
  0x0020,   /* U+0DB3 */
  0x0020,   /* U+0DB4 */
  0x0020,   /* U+0DB5 */
  0x0020,   /* U+0DB6 */
  0x0020,   /* U+0DB7 */
  0x0020,   /* U+0DB8 */
  0x0020,   /* U+0DB9 */
  0x0020,   /* U+0DBA */
  0x0020,   /* U+0DBB */
  0x0020,   /* U+0DBC */
  0x0020,   /* U+0DBD */
  0x0020,   /* U+0DBE */
  0x0020,   /* U+0DBF */
  0x0020,   /* U+0DC0 */
  0x0020,   /* U+0DC1 */
  0x0020,   /* U+0DC2 */
  0x0020,   /* U+0DC3 */
  0x0020,   /* U+0DC4 */
  0x0020,   /* U+0DC5 */
  0x0020,   /* U+0DC6 */
  0x0020,   /* U+0DC7 */
  0x0020,   /* U+0DC8 */
  0x0020,   /* U+0DC9 */
  0x0020,   /* U+0DCA */
  0x0020,   /* U+0DCB */
  0x0020,   /* U+0DCC */
  0x0020,   /* U+0DCD */
  0x0020,   /* U+0DCE */
  0x0020,   /* U+0DCF */
  0x0020,   /* U+0DD0 */
  0x0020,   /* U+0DD1 */
  0x0020,   /* U+0DD2 */
  0x0020,   /* U+0DD3 */
  0x0020,   /* U+0DD4 */
  0x0020,   /* U+0DD5 */
  0x0020,   /* U+0DD6 */
  0x0020,   /* U+0DD7 */
  0x0020,   /* U+0DD8 */
  0x0020,   /* U+0DD9 */
  0x0020,   /* U+0DDA */
  0x0020,   /* U+0DDB */
  0x0020,   /* U+0DDC */
  0x0020,   /* U+0DDD */
  0x0020,   /* U+0DDE */
  0x0020,   /* U+0DDF */
  0x0020,   /* U+0DE0 */
  0x0020,   /* U+0DE1 */
  0x0020,   /* U+0DE2 */
  0x0020,   /* U+0DE3 */
  0x0020,   /* U+0DE4 */
  0x0020,   /* U+0DE5 */
  0x0020,   /* U+0DE6 */
  0x0020,   /* U+0DE7 */
  0x0020,   /* U+0DE8 */
  0x0020,   /* U+0DE9 */
  0x0020,   /* U+0DEA */
  0x0020,   /* U+0DEB */
  0x0020,   /* U+0DEC */
  0x0020,   /* U+0DED */
  0x0020,   /* U+0DEE */
  0x0020,   /* U+0DEF */
  0x0020,   /* U+0DF0 */
  0x0020,   /* U+0DF1 */
  0x0020,   /* U+0DF2 */
  0x0020,   /* U+0DF3 */
  0x0020,   /* U+0DF4 */
  0x0020,   /* U+0DF5 */
  0x0020,   /* U+0DF6 */
  0x0020,   /* U+0DF7 */
  0x0020,   /* U+0DF8 */
  0x0020,   /* U+0DF9 */
  0x0020,   /* U+0DFA */
  0x0020,   /* U+0DFB */
  0x0020,   /* U+0DFC */
  0x0020,   /* U+0DFD */
  0x0020,   /* U+0DFE */
  0x0020,   /* U+0DFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0D00 */
  0x0002,   /* U+0D01 */
  0x0002,   /* U+0D02 */
  0x0002,   /* U+0D03 */
  0x0002,   /* U+0D04 */
  0x0002,   /* U+0D05 */
  0x0002,   /* U+0D06 */
  0x0002,   /* U+0D07 */
  0x0002,   /* U+0D08 */
  0x0002,   /* U+0D09 */
  0x0002,   /* U+0D0A */
  0x0002,   /* U+0D0B */
  0x0002,   /* U+0D0C */
  0x0002,   /* U+0D0D */
  0x0002,   /* U+0D0E */
  0x0002,   /* U+0D0F */
  0x0002,   /* U+0D10 */
  0x0002,   /* U+0D11 */
  0x0002,   /* U+0D12 */
  0x0002,   /* U+0D13 */
  0x0002,   /* U+0D14 */
  0x0002,   /* U+0D15 */
  0x0002,   /* U+0D16 */
  0x0002,   /* U+0D17 */
  0x0002,   /* U+0D18 */
  0x0002,   /* U+0D19 */
  0x0002,   /* U+0D1A */
  0x0002,   /* U+0D1B */
  0x0002,   /* U+0D1C */
  0x0002,   /* U+0D1D */
  0x0002,   /* U+0D1E */
  0x0002,   /* U+0D1F */
  0x0002,   /* U+0D20 */
  0x0002,   /* U+0D21 */
  0x0002,   /* U+0D22 */
  0x0002,   /* U+0D23 */
  0x0002,   /* U+0D24 */
  0x0002,   /* U+0D25 */
  0x0002,   /* U+0D26 */
  0x0002,   /* U+0D27 */
  0x0002,   /* U+0D28 */
  0x0002,   /* U+0D29 */
  0x0002,   /* U+0D2A */
  0x0002,   /* U+0D2B */
  0x0002,   /* U+0D2C */
  0x0002,   /* U+0D2D */
  0x0002,   /* U+0D2E */
  0x0002,   /* U+0D2F */
  0x0002,   /* U+0D30 */
  0x0002,   /* U+0D31 */
  0x0002,   /* U+0D32 */
  0x0002,   /* U+0D33 */
  0x0002,   /* U+0D34 */
  0x0002,   /* U+0D35 */
  0x0002,   /* U+0D36 */
  0x0002,   /* U+0D37 */
  0x0002,   /* U+0D38 */
  0x0002,   /* U+0D39 */
  0x0002,   /* U+0D3A */
  0x0002,   /* U+0D3B */
  0x0002,   /* U+0D3C */
  0x0002,   /* U+0D3D */
  0x0002,   /* U+0D3E */
  0x0002,   /* U+0D3F */
  0x0002,   /* U+0D40 */
  0x0002,   /* U+0D41 */
  0x0002,   /* U+0D42 */
  0x0002,   /* U+0D43 */
  0x0002,   /* U+0D44 */
  0x0002,   /* U+0D45 */
  0x0002,   /* U+0D46 */
  0x0002,   /* U+0D47 */
  0x0002,   /* U+0D48 */
  0x0002,   /* U+0D49 */
  0x0002,   /* U+0D4A */
  0x0002,   /* U+0D4B */
  0x0002,   /* U+0D4C */
  0x0002,   /* U+0D4D */
  0x0004,   /* U+0D4E */
  0x0002,   /* U+0D4F */
  0x0002,   /* U+0D50 */
  0x0002,   /* U+0D51 */
  0x0002,   /* U+0D52 */
  0x0002,   /* U+0D53 */
  0x0004,   /* U+0D54 */
  0x0004,   /* U+0D55 */
  0x0004,   /* U+0D56 */
  0x0002,   /* U+0D57 */
  0x0002,   /* U+0D58 */
  0x0002,   /* U+0D59 */
  0x0002,   /* U+0D5A */
  0x0002,   /* U+0D5B */
  0x0002,   /* U+0D5C */
  0x0002,   /* U+0D5D */
  0x0002,   /* U+0D5E */
  0x0002,   /* U+0D5F */
  0x0002,   /* U+0D60 */
  0x0002,   /* U+0D61 */
  0x0002,   /* U+0D62 */
  0x0002,   /* U+0D63 */
  0x0002,   /* U+0D64 */
  0x0002,   /* U+0D65 */
  0x0002,   /* U+0D66 */
  0x0002,   /* U+0D67 */
  0x0002,   /* U+0D68 */
  0x0002,   /* U+0D69 */
  0x0002,   /* U+0D6A */
  0x0002,   /* U+0D6B */
  0x0002,   /* U+0D6C */
  0x0002,   /* U+0D6D */
  0x0002,   /* U+0D6E */
  0x0002,   /* U+0D6F */
  0x0002,   /* U+0D70 */
  0x0002,   /* U+0D71 */
  0x0002,   /* U+0D72 */
  0x0002,   /* U+0D73 */
  0x0002,   /* U+0D74 */
  0x0002,   /* U+0D75 */
  0x0002,   /* U+0D76 */
  0x0002,   /* U+0D77 */
  0x0002,   /* U+0D78 */
  0x0002,   /* U+0D79 */
  0x0004,   /* U+0D7A */
  0x0004,   /* U+0D7B */
  0x0004,   /* U+0D7C */
  0x0004,   /* U+0D7D */
  0x0004,   /* U+0D7E */
  0x0004,   /* U+0D7F */
  0x0002,   /* U+0D80 */
  0x0002,   /* U+0D81 */
  0x0002,   /* U+0D82 */
  0x0002,   /* U+0D83 */
  0x0002,   /* U+0D84 */
  0x0002,   /* U+0D85 */
  0x0002,   /* U+0D86 */
  0x0002,   /* U+0D87 */
  0x0002,   /* U+0D88 */
  0x0002,   /* U+0D89 */
  0x0002,   /* U+0D8A */
  0x0002,   /* U+0D8B */
  0x0002,   /* U+0D8C */
  0x0002,   /* U+0D8D */
  0x0002,   /* U+0D8E */
  0x0002,   /* U+0D8F */
  0x0002,   /* U+0D90 */
  0x0002,   /* U+0D91 */
  0x0002,   /* U+0D92 */
  0x0002,   /* U+0D93 */
  0x0002,   /* U+0D94 */
  0x0002,   /* U+0D95 */
  0x0002,   /* U+0D96 */
  0x0002,   /* U+0D97 */
  0x0002,   /* U+0D98 */
  0x0002,   /* U+0D99 */
  0x0002,   /* U+0D9A */
  0x0002,   /* U+0D9B */
  0x0002,   /* U+0D9C */
  0x0002,   /* U+0D9D */
  0x0002,   /* U+0D9E */
  0x0002,   /* U+0D9F */
  0x0002,   /* U+0DA0 */
  0x0002,   /* U+0DA1 */
  0x0002,   /* U+0DA2 */
  0x0002,   /* U+0DA3 */
  0x0002,   /* U+0DA4 */
  0x0002,   /* U+0DA5 */
  0x0002,   /* U+0DA6 */
  0x0002,   /* U+0DA7 */
  0x0002,   /* U+0DA8 */
  0x0002,   /* U+0DA9 */
  0x0002,   /* U+0DAA */
  0x0002,   /* U+0DAB */
  0x0002,   /* U+0DAC */
  0x0002,   /* U+0DAD */
  0x0002,   /* U+0DAE */
  0x0002,   /* U+0DAF */
  0x0002,   /* U+0DB0 */
  0x0002,   /* U+0DB1 */
  0x0002,   /* U+0DB2 */
  0x0002,   /* U+0DB3 */
  0x0002,   /* U+0DB4 */
  0x0002,   /* U+0DB5 */
  0x0002,   /* U+0DB6 */
  0x0002,   /* U+0DB7 */
  0x0002,   /* U+0DB8 */
  0x0002,   /* U+0DB9 */
  0x0002,   /* U+0DBA */
  0x0002,   /* U+0DBB */
  0x0002,   /* U+0DBC */
  0x0002,   /* U+0DBD */
  0x0002,   /* U+0DBE */
  0x0002,   /* U+0DBF */
  0x0002,   /* U+0DC0 */
  0x0002,   /* U+0DC1 */
  0x0002,   /* U+0DC2 */
  0x0002,   /* U+0DC3 */
  0x0002,   /* U+0DC4 */
  0x0002,   /* U+0DC5 */
  0x0002,   /* U+0DC6 */
  0x0002,   /* U+0DC7 */
  0x0002,   /* U+0DC8 */
  0x0002,   /* U+0DC9 */
  0x0002,   /* U+0DCA */
  0x0002,   /* U+0DCB */
  0x0002,   /* U+0DCC */
  0x0002,   /* U+0DCD */
  0x0002,   /* U+0DCE */
  0x0002,   /* U+0DCF */
  0x0002,   /* U+0DD0 */
  0x0002,   /* U+0DD1 */
  0x0002,   /* U+0DD2 */
  0x0002,   /* U+0DD3 */
  0x0002,   /* U+0DD4 */
  0x0002,   /* U+0DD5 */
  0x0002,   /* U+0DD6 */
  0x0002,   /* U+0DD7 */
  0x0002,   /* U+0DD8 */
  0x0002,   /* U+0DD9 */
  0x0002,   /* U+0DDA */
  0x0002,   /* U+0DDB */
  0x0002,   /* U+0DDC */
  0x0002,   /* U+0DDD */
  0x0002,   /* U+0DDE */
  0x0002,   /* U+0DDF */
  0x0002,   /* U+0DE0 */
  0x0002,   /* U+0DE1 */
  0x0002,   /* U+0DE2 */
  0x0002,   /* U+0DE3 */
  0x0002,   /* U+0DE4 */
  0x0002,   /* U+0DE5 */
  0x0002,   /* U+0DE6 */
  0x0002,   /* U+0DE7 */
  0x0002,   /* U+0DE8 */
  0x0002,   /* U+0DE9 */
  0x0002,   /* U+0DEA */
  0x0002,   /* U+0DEB */
  0x0002,   /* U+0DEC */
  0x0002,   /* U+0DED */
  0x0002,   /* U+0DEE */
  0x0002,   /* U+0DEF */
  0x0002,   /* U+0DF0 */
  0x0002,   /* U+0DF1 */
  0x0002,   /* U+0DF2 */
  0x0002,   /* U+0DF3 */
  0x0002,   /* U+0DF4 */
  0x0002,   /* U+0DF5 */
  0x0002,   /* U+0DF6 */
  0x0002,   /* U+0DF7 */
  0x0002,   /* U+0DF8 */
  0x0002,   /* U+0DF9 */
  0x0002,   /* U+0DFA */
  0x0002,   /* U+0DFB */
  0x0002,   /* U+0DFC */
  0x0002,   /* U+0DFD */
  0x0002,   /* U+0DFE */
  0x0002,   /* U+0DFF */

  /* Primary weight 2 for each character. */
  0x8D00,   /* U+0D00 */
  0x0000,   /* U+0D01 */
  0x0000,   /* U+0D02 */
  0x0000,   /* U+0D03 */
  0x8D04,   /* U+0D04 */
  0x0000,   /* U+0D05 */
  0x0000,   /* U+0D06 */
  0x0000,   /* U+0D07 */
  0x0000,   /* U+0D08 */
  0x0000,   /* U+0D09 */
  0x0000,   /* U+0D0A */
  0x0000,   /* U+0D0B */
  0x0000,   /* U+0D0C */
  0x8D0D,   /* U+0D0D */
  0x0000,   /* U+0D0E */
  0x0000,   /* U+0D0F */
  0x0000,   /* U+0D10 */
  0x8D11,   /* U+0D11 */
  0x0000,   /* U+0D12 */
  0x0000,   /* U+0D13 */
  0x0000,   /* U+0D14 */
  0x0000,   /* U+0D15 */
  0x0000,   /* U+0D16 */
  0x0000,   /* U+0D17 */
  0x0000,   /* U+0D18 */
  0x0000,   /* U+0D19 */
  0x0000,   /* U+0D1A */
  0x0000,   /* U+0D1B */
  0x0000,   /* U+0D1C */
  0x0000,   /* U+0D1D */
  0x0000,   /* U+0D1E */
  0x0000,   /* U+0D1F */
  0x0000,   /* U+0D20 */
  0x0000,   /* U+0D21 */
  0x0000,   /* U+0D22 */
  0x0000,   /* U+0D23 */
  0x0000,   /* U+0D24 */
  0x0000,   /* U+0D25 */
  0x0000,   /* U+0D26 */
  0x0000,   /* U+0D27 */
  0x0000,   /* U+0D28 */
  0x0000,   /* U+0D29 */
  0x0000,   /* U+0D2A */
  0x0000,   /* U+0D2B */
  0x0000,   /* U+0D2C */
  0x0000,   /* U+0D2D */
  0x0000,   /* U+0D2E */
  0x0000,   /* U+0D2F */
  0x0000,   /* U+0D30 */
  0x0000,   /* U+0D31 */
  0x0000,   /* U+0D32 */
  0x0000,   /* U+0D33 */
  0x0000,   /* U+0D34 */
  0x0000,   /* U+0D35 */
  0x0000,   /* U+0D36 */
  0x0000,   /* U+0D37 */
  0x0000,   /* U+0D38 */
  0x0000,   /* U+0D39 */
  0x0000,   /* U+0D3A */
  0x8D3B,   /* U+0D3B */
  0x8D3C,   /* U+0D3C */
  0x0000,   /* U+0D3D */
  0x0000,   /* U+0D3E */
  0x0000,   /* U+0D3F */
  0x0000,   /* U+0D40 */
  0x0000,   /* U+0D41 */
  0x0000,   /* U+0D42 */
  0x0000,   /* U+0D43 */
  0x0000,   /* U+0D44 */
  0x8D45,   /* U+0D45 */
  0x0000,   /* U+0D46 */
  0x0000,   /* U+0D47 */
  0x0000,   /* U+0D48 */
  0x8D49,   /* U+0D49 */
  0x0000,   /* U+0D4A */
  0x0000,   /* U+0D4B */
  0x0000,   /* U+0D4C */
  0x0000,   /* U+0D4D */
  0x28CF,   /* U+0D4E */
  0x0000,   /* U+0D4F */
  0x8D50,   /* U+0D50 */
  0x8D51,   /* U+0D51 */
  0x8D52,   /* U+0D52 */
  0x8D53,   /* U+0D53 */
  0x28CF,   /* U+0D54 */
  0x28CF,   /* U+0D55 */
  0x28CF,   /* U+0D56 */
  0x0000,   /* U+0D57 */
  0x0000,   /* U+0D58 */
  0x0000,   /* U+0D59 */
  0x0000,   /* U+0D5A */
  0x0000,   /* U+0D5B */
  0x0000,   /* U+0D5C */
  0x0000,   /* U+0D5D */
  0x0000,   /* U+0D5E */
  0x0000,   /* U+0D5F */
  0x0000,   /* U+0D60 */
  0x0000,   /* U+0D61 */
  0x0000,   /* U+0D62 */
  0x0000,   /* U+0D63 */
  0x8D64,   /* U+0D64 */
  0x8D65,   /* U+0D65 */
  0x0000,   /* U+0D66 */
  0x0000,   /* U+0D67 */
  0x0000,   /* U+0D68 */
  0x0000,   /* U+0D69 */
  0x0000,   /* U+0D6A */
  0x0000,   /* U+0D6B */
  0x0000,   /* U+0D6C */
  0x0000,   /* U+0D6D */
  0x0000,   /* U+0D6E */
  0x0000,   /* U+0D6F */
  0x0000,   /* U+0D70 */
  0x0000,   /* U+0D71 */
  0x0000,   /* U+0D72 */
  0x0000,   /* U+0D73 */
  0x0000,   /* U+0D74 */
  0x0000,   /* U+0D75 */
  0x0000,   /* U+0D76 */
  0x0000,   /* U+0D77 */
  0x0000,   /* U+0D78 */
  0x0000,   /* U+0D79 */
  0x28CF,   /* U+0D7A */
  0x28CF,   /* U+0D7B */
  0x28CF,   /* U+0D7C */
  0x28CF,   /* U+0D7D */
  0x28CF,   /* U+0D7E */
  0x28CF,   /* U+0D7F */
  0x8D80,   /* U+0D80 */
  0x8D81,   /* U+0D81 */
  0x0000,   /* U+0D82 */
  0x0000,   /* U+0D83 */
  0x8D84,   /* U+0D84 */
  0x0000,   /* U+0D85 */
  0x0000,   /* U+0D86 */
  0x0000,   /* U+0D87 */
  0x0000,   /* U+0D88 */
  0x0000,   /* U+0D89 */
  0x0000,   /* U+0D8A */
  0x0000,   /* U+0D8B */
  0x0000,   /* U+0D8C */
  0x0000,   /* U+0D8D */
  0x0000,   /* U+0D8E */
  0x0000,   /* U+0D8F */
  0x0000,   /* U+0D90 */
  0x0000,   /* U+0D91 */
  0x0000,   /* U+0D92 */
  0x0000,   /* U+0D93 */
  0x0000,   /* U+0D94 */
  0x0000,   /* U+0D95 */
  0x0000,   /* U+0D96 */
  0x8D97,   /* U+0D97 */
  0x8D98,   /* U+0D98 */
  0x8D99,   /* U+0D99 */
  0x0000,   /* U+0D9A */
  0x0000,   /* U+0D9B */
  0x0000,   /* U+0D9C */
  0x0000,   /* U+0D9D */
  0x0000,   /* U+0D9E */
  0x0000,   /* U+0D9F */
  0x0000,   /* U+0DA0 */
  0x0000,   /* U+0DA1 */
  0x0000,   /* U+0DA2 */
  0x0000,   /* U+0DA3 */
  0x0000,   /* U+0DA4 */
  0x0000,   /* U+0DA5 */
  0x0000,   /* U+0DA6 */
  0x0000,   /* U+0DA7 */
  0x0000,   /* U+0DA8 */
  0x0000,   /* U+0DA9 */
  0x0000,   /* U+0DAA */
  0x0000,   /* U+0DAB */
  0x0000,   /* U+0DAC */
  0x0000,   /* U+0DAD */
  0x0000,   /* U+0DAE */
  0x0000,   /* U+0DAF */
  0x0000,   /* U+0DB0 */
  0x0000,   /* U+0DB1 */
  0x8DB2,   /* U+0DB2 */
  0x0000,   /* U+0DB3 */
  0x0000,   /* U+0DB4 */
  0x0000,   /* U+0DB5 */
  0x0000,   /* U+0DB6 */
  0x0000,   /* U+0DB7 */
  0x0000,   /* U+0DB8 */
  0x0000,   /* U+0DB9 */
  0x0000,   /* U+0DBA */
  0x0000,   /* U+0DBB */
  0x8DBC,   /* U+0DBC */
  0x0000,   /* U+0DBD */
  0x8DBE,   /* U+0DBE */
  0x8DBF,   /* U+0DBF */
  0x0000,   /* U+0DC0 */
  0x0000,   /* U+0DC1 */
  0x0000,   /* U+0DC2 */
  0x0000,   /* U+0DC3 */
  0x0000,   /* U+0DC4 */
  0x0000,   /* U+0DC5 */
  0x0000,   /* U+0DC6 */
  0x8DC7,   /* U+0DC7 */
  0x8DC8,   /* U+0DC8 */
  0x8DC9,   /* U+0DC9 */
  0x0000,   /* U+0DCA */
  0x8DCB,   /* U+0DCB */
  0x8DCC,   /* U+0DCC */
  0x8DCD,   /* U+0DCD */
  0x8DCE,   /* U+0DCE */
  0x0000,   /* U+0DCF */
  0x0000,   /* U+0DD0 */
  0x0000,   /* U+0DD1 */
  0x0000,   /* U+0DD2 */
  0x0000,   /* U+0DD3 */
  0x0000,   /* U+0DD4 */
  0x8DD5,   /* U+0DD5 */
  0x0000,   /* U+0DD6 */
  0x8DD7,   /* U+0DD7 */
  0x0000,   /* U+0DD8 */
  0x0000,   /* U+0DD9 */
  0x0000,   /* U+0DDA */
  0x0000,   /* U+0DDB */
  0x0000,   /* U+0DDC */
  0x0000,   /* U+0DDD */
  0x0000,   /* U+0DDE */
  0x0000,   /* U+0DDF */
  0x8DE0,   /* U+0DE0 */
  0x8DE1,   /* U+0DE1 */
  0x8DE2,   /* U+0DE2 */
  0x8DE3,   /* U+0DE3 */
  0x8DE4,   /* U+0DE4 */
  0x8DE5,   /* U+0DE5 */
  0x0000,   /* U+0DE6 */
  0x0000,   /* U+0DE7 */
  0x0000,   /* U+0DE8 */
  0x0000,   /* U+0DE9 */
  0x0000,   /* U+0DEA */
  0x0000,   /* U+0DEB */
  0x0000,   /* U+0DEC */
  0x0000,   /* U+0DED */
  0x0000,   /* U+0DEE */
  0x0000,   /* U+0DEF */
  0x8DF0,   /* U+0DF0 */
  0x8DF1,   /* U+0DF1 */
  0x0000,   /* U+0DF2 */
  0x0000,   /* U+0DF3 */
  0x0000,   /* U+0DF4 */
  0x8DF5,   /* U+0DF5 */
  0x8DF6,   /* U+0DF6 */
  0x8DF7,   /* U+0DF7 */
  0x8DF8,   /* U+0DF8 */
  0x8DF9,   /* U+0DF9 */
  0x8DFA,   /* U+0DFA */
  0x8DFB,   /* U+0DFB */
  0x8DFC,   /* U+0DFC */
  0x8DFD,   /* U+0DFD */
  0x8DFE,   /* U+0DFE */
  0x8DFF,   /* U+0DFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0D00 */
  0x0000,   /* U+0D01 */
  0x0000,   /* U+0D02 */
  0x0000,   /* U+0D03 */
  0x0000,   /* U+0D04 */
  0x0000,   /* U+0D05 */
  0x0000,   /* U+0D06 */
  0x0000,   /* U+0D07 */
  0x0000,   /* U+0D08 */
  0x0000,   /* U+0D09 */
  0x0000,   /* U+0D0A */
  0x0000,   /* U+0D0B */
  0x0000,   /* U+0D0C */
  0x0000,   /* U+0D0D */
  0x0000,   /* U+0D0E */
  0x0000,   /* U+0D0F */
  0x0000,   /* U+0D10 */
  0x0000,   /* U+0D11 */
  0x0000,   /* U+0D12 */
  0x0000,   /* U+0D13 */
  0x0000,   /* U+0D14 */
  0x0000,   /* U+0D15 */
  0x0000,   /* U+0D16 */
  0x0000,   /* U+0D17 */
  0x0000,   /* U+0D18 */
  0x0000,   /* U+0D19 */
  0x0000,   /* U+0D1A */
  0x0000,   /* U+0D1B */
  0x0000,   /* U+0D1C */
  0x0000,   /* U+0D1D */
  0x0000,   /* U+0D1E */
  0x0000,   /* U+0D1F */
  0x0000,   /* U+0D20 */
  0x0000,   /* U+0D21 */
  0x0000,   /* U+0D22 */
  0x0000,   /* U+0D23 */
  0x0000,   /* U+0D24 */
  0x0000,   /* U+0D25 */
  0x0000,   /* U+0D26 */
  0x0000,   /* U+0D27 */
  0x0000,   /* U+0D28 */
  0x0000,   /* U+0D29 */
  0x0000,   /* U+0D2A */
  0x0000,   /* U+0D2B */
  0x0000,   /* U+0D2C */
  0x0000,   /* U+0D2D */
  0x0000,   /* U+0D2E */
  0x0000,   /* U+0D2F */
  0x0000,   /* U+0D30 */
  0x0000,   /* U+0D31 */
  0x0000,   /* U+0D32 */
  0x0000,   /* U+0D33 */
  0x0000,   /* U+0D34 */
  0x0000,   /* U+0D35 */
  0x0000,   /* U+0D36 */
  0x0000,   /* U+0D37 */
  0x0000,   /* U+0D38 */
  0x0000,   /* U+0D39 */
  0x0000,   /* U+0D3A */
  0x0000,   /* U+0D3B */
  0x0000,   /* U+0D3C */
  0x0000,   /* U+0D3D */
  0x0000,   /* U+0D3E */
  0x0000,   /* U+0D3F */
  0x0000,   /* U+0D40 */
  0x0000,   /* U+0D41 */
  0x0000,   /* U+0D42 */
  0x0000,   /* U+0D43 */
  0x0000,   /* U+0D44 */
  0x0000,   /* U+0D45 */
  0x0000,   /* U+0D46 */
  0x0000,   /* U+0D47 */
  0x0000,   /* U+0D48 */
  0x0000,   /* U+0D49 */
  0x0000,   /* U+0D4A */
  0x0000,   /* U+0D4B */
  0x0000,   /* U+0D4C */
  0x0000,   /* U+0D4D */
  0x0020,   /* U+0D4E */
  0x0000,   /* U+0D4F */
  0x0000,   /* U+0D50 */
  0x0000,   /* U+0D51 */
  0x0000,   /* U+0D52 */
  0x0000,   /* U+0D53 */
  0x0020,   /* U+0D54 */
  0x0020,   /* U+0D55 */
  0x0020,   /* U+0D56 */
  0x0000,   /* U+0D57 */
  0x0000,   /* U+0D58 */
  0x0000,   /* U+0D59 */
  0x0000,   /* U+0D5A */
  0x0000,   /* U+0D5B */
  0x0000,   /* U+0D5C */
  0x0000,   /* U+0D5D */
  0x0000,   /* U+0D5E */
  0x0000,   /* U+0D5F */
  0x0000,   /* U+0D60 */
  0x0000,   /* U+0D61 */
  0x0000,   /* U+0D62 */
  0x0000,   /* U+0D63 */
  0x0000,   /* U+0D64 */
  0x0000,   /* U+0D65 */
  0x0000,   /* U+0D66 */
  0x0000,   /* U+0D67 */
  0x0000,   /* U+0D68 */
  0x0000,   /* U+0D69 */
  0x0000,   /* U+0D6A */
  0x0000,   /* U+0D6B */
  0x0000,   /* U+0D6C */
  0x0000,   /* U+0D6D */
  0x0000,   /* U+0D6E */
  0x0000,   /* U+0D6F */
  0x0000,   /* U+0D70 */
  0x0000,   /* U+0D71 */
  0x0000,   /* U+0D72 */
  0x0000,   /* U+0D73 */
  0x0000,   /* U+0D74 */
  0x0000,   /* U+0D75 */
  0x0000,   /* U+0D76 */
  0x0000,   /* U+0D77 */
  0x0000,   /* U+0D78 */
  0x0000,   /* U+0D79 */
  0x0020,   /* U+0D7A */
  0x0020,   /* U+0D7B */
  0x0020,   /* U+0D7C */
  0x0020,   /* U+0D7D */
  0x0020,   /* U+0D7E */
  0x0020,   /* U+0D7F */
  0x0000,   /* U+0D80 */
  0x0000,   /* U+0D81 */
  0x0000,   /* U+0D82 */
  0x0000,   /* U+0D83 */
  0x0000,   /* U+0D84 */
  0x0000,   /* U+0D85 */
  0x0000,   /* U+0D86 */
  0x0000,   /* U+0D87 */
  0x0000,   /* U+0D88 */
  0x0000,   /* U+0D89 */
  0x0000,   /* U+0D8A */
  0x0000,   /* U+0D8B */
  0x0000,   /* U+0D8C */
  0x0000,   /* U+0D8D */
  0x0000,   /* U+0D8E */
  0x0000,   /* U+0D8F */
  0x0000,   /* U+0D90 */
  0x0000,   /* U+0D91 */
  0x0000,   /* U+0D92 */
  0x0000,   /* U+0D93 */
  0x0000,   /* U+0D94 */
  0x0000,   /* U+0D95 */
  0x0000,   /* U+0D96 */
  0x0000,   /* U+0D97 */
  0x0000,   /* U+0D98 */
  0x0000,   /* U+0D99 */
  0x0000,   /* U+0D9A */
  0x0000,   /* U+0D9B */
  0x0000,   /* U+0D9C */
  0x0000,   /* U+0D9D */
  0x0000,   /* U+0D9E */
  0x0000,   /* U+0D9F */
  0x0000,   /* U+0DA0 */
  0x0000,   /* U+0DA1 */
  0x0000,   /* U+0DA2 */
  0x0000,   /* U+0DA3 */
  0x0000,   /* U+0DA4 */
  0x0000,   /* U+0DA5 */
  0x0000,   /* U+0DA6 */
  0x0000,   /* U+0DA7 */
  0x0000,   /* U+0DA8 */
  0x0000,   /* U+0DA9 */
  0x0000,   /* U+0DAA */
  0x0000,   /* U+0DAB */
  0x0000,   /* U+0DAC */
  0x0000,   /* U+0DAD */
  0x0000,   /* U+0DAE */
  0x0000,   /* U+0DAF */
  0x0000,   /* U+0DB0 */
  0x0000,   /* U+0DB1 */
  0x0000,   /* U+0DB2 */
  0x0000,   /* U+0DB3 */
  0x0000,   /* U+0DB4 */
  0x0000,   /* U+0DB5 */
  0x0000,   /* U+0DB6 */
  0x0000,   /* U+0DB7 */
  0x0000,   /* U+0DB8 */
  0x0000,   /* U+0DB9 */
  0x0000,   /* U+0DBA */
  0x0000,   /* U+0DBB */
  0x0000,   /* U+0DBC */
  0x0000,   /* U+0DBD */
  0x0000,   /* U+0DBE */
  0x0000,   /* U+0DBF */
  0x0000,   /* U+0DC0 */
  0x0000,   /* U+0DC1 */
  0x0000,   /* U+0DC2 */
  0x0000,   /* U+0DC3 */
  0x0000,   /* U+0DC4 */
  0x0000,   /* U+0DC5 */
  0x0000,   /* U+0DC6 */
  0x0000,   /* U+0DC7 */
  0x0000,   /* U+0DC8 */
  0x0000,   /* U+0DC9 */
  0x0000,   /* U+0DCA */
  0x0000,   /* U+0DCB */
  0x0000,   /* U+0DCC */
  0x0000,   /* U+0DCD */
  0x0000,   /* U+0DCE */
  0x0000,   /* U+0DCF */
  0x0000,   /* U+0DD0 */
  0x0000,   /* U+0DD1 */
  0x0000,   /* U+0DD2 */
  0x0000,   /* U+0DD3 */
  0x0000,   /* U+0DD4 */
  0x0000,   /* U+0DD5 */
  0x0000,   /* U+0DD6 */
  0x0000,   /* U+0DD7 */
  0x0000,   /* U+0DD8 */
  0x0000,   /* U+0DD9 */
  0x0000,   /* U+0DDA */
  0x0000,   /* U+0DDB */
  0x0000,   /* U+0DDC */
  0x0000,   /* U+0DDD */
  0x0000,   /* U+0DDE */
  0x0000,   /* U+0DDF */
  0x0000,   /* U+0DE0 */
  0x0000,   /* U+0DE1 */
  0x0000,   /* U+0DE2 */
  0x0000,   /* U+0DE3 */
  0x0000,   /* U+0DE4 */
  0x0000,   /* U+0DE5 */
  0x0000,   /* U+0DE6 */
  0x0000,   /* U+0DE7 */
  0x0000,   /* U+0DE8 */
  0x0000,   /* U+0DE9 */
  0x0000,   /* U+0DEA */
  0x0000,   /* U+0DEB */
  0x0000,   /* U+0DEC */
  0x0000,   /* U+0DED */
  0x0000,   /* U+0DEE */
  0x0000,   /* U+0DEF */
  0x0000,   /* U+0DF0 */
  0x0000,   /* U+0DF1 */
  0x0000,   /* U+0DF2 */
  0x0000,   /* U+0DF3 */
  0x0000,   /* U+0DF4 */
  0x0000,   /* U+0DF5 */
  0x0000,   /* U+0DF6 */
  0x0000,   /* U+0DF7 */
  0x0000,   /* U+0DF8 */
  0x0000,   /* U+0DF9 */
  0x0000,   /* U+0DFA */
  0x0000,   /* U+0DFB */
  0x0000,   /* U+0DFC */
  0x0000,   /* U+0DFD */
  0x0000,   /* U+0DFE */
  0x0000,   /* U+0DFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0D00 */
  0x0000,   /* U+0D01 */
  0x0000,   /* U+0D02 */
  0x0000,   /* U+0D03 */
  0x0000,   /* U+0D04 */
  0x0000,   /* U+0D05 */
  0x0000,   /* U+0D06 */
  0x0000,   /* U+0D07 */
  0x0000,   /* U+0D08 */
  0x0000,   /* U+0D09 */
  0x0000,   /* U+0D0A */
  0x0000,   /* U+0D0B */
  0x0000,   /* U+0D0C */
  0x0000,   /* U+0D0D */
  0x0000,   /* U+0D0E */
  0x0000,   /* U+0D0F */
  0x0000,   /* U+0D10 */
  0x0000,   /* U+0D11 */
  0x0000,   /* U+0D12 */
  0x0000,   /* U+0D13 */
  0x0000,   /* U+0D14 */
  0x0000,   /* U+0D15 */
  0x0000,   /* U+0D16 */
  0x0000,   /* U+0D17 */
  0x0000,   /* U+0D18 */
  0x0000,   /* U+0D19 */
  0x0000,   /* U+0D1A */
  0x0000,   /* U+0D1B */
  0x0000,   /* U+0D1C */
  0x0000,   /* U+0D1D */
  0x0000,   /* U+0D1E */
  0x0000,   /* U+0D1F */
  0x0000,   /* U+0D20 */
  0x0000,   /* U+0D21 */
  0x0000,   /* U+0D22 */
  0x0000,   /* U+0D23 */
  0x0000,   /* U+0D24 */
  0x0000,   /* U+0D25 */
  0x0000,   /* U+0D26 */
  0x0000,   /* U+0D27 */
  0x0000,   /* U+0D28 */
  0x0000,   /* U+0D29 */
  0x0000,   /* U+0D2A */
  0x0000,   /* U+0D2B */
  0x0000,   /* U+0D2C */
  0x0000,   /* U+0D2D */
  0x0000,   /* U+0D2E */
  0x0000,   /* U+0D2F */
  0x0000,   /* U+0D30 */
  0x0000,   /* U+0D31 */
  0x0000,   /* U+0D32 */
  0x0000,   /* U+0D33 */
  0x0000,   /* U+0D34 */
  0x0000,   /* U+0D35 */
  0x0000,   /* U+0D36 */
  0x0000,   /* U+0D37 */
  0x0000,   /* U+0D38 */
  0x0000,   /* U+0D39 */
  0x0000,   /* U+0D3A */
  0x0000,   /* U+0D3B */
  0x0000,   /* U+0D3C */
  0x0000,   /* U+0D3D */
  0x0000,   /* U+0D3E */
  0x0000,   /* U+0D3F */
  0x0000,   /* U+0D40 */
  0x0000,   /* U+0D41 */
  0x0000,   /* U+0D42 */
  0x0000,   /* U+0D43 */
  0x0000,   /* U+0D44 */
  0x0000,   /* U+0D45 */
  0x0000,   /* U+0D46 */
  0x0000,   /* U+0D47 */
  0x0000,   /* U+0D48 */
  0x0000,   /* U+0D49 */
  0x0000,   /* U+0D4A */
  0x0000,   /* U+0D4B */
  0x0000,   /* U+0D4C */
  0x0000,   /* U+0D4D */
  0x0004,   /* U+0D4E */
  0x0000,   /* U+0D4F */
  0x0000,   /* U+0D50 */
  0x0000,   /* U+0D51 */
  0x0000,   /* U+0D52 */
  0x0000,   /* U+0D53 */
  0x0004,   /* U+0D54 */
  0x0004,   /* U+0D55 */
  0x0004,   /* U+0D56 */
  0x0000,   /* U+0D57 */
  0x0000,   /* U+0D58 */
  0x0000,   /* U+0D59 */
  0x0000,   /* U+0D5A */
  0x0000,   /* U+0D5B */
  0x0000,   /* U+0D5C */
  0x0000,   /* U+0D5D */
  0x0000,   /* U+0D5E */
  0x0000,   /* U+0D5F */
  0x0000,   /* U+0D60 */
  0x0000,   /* U+0D61 */
  0x0000,   /* U+0D62 */
  0x0000,   /* U+0D63 */
  0x0000,   /* U+0D64 */
  0x0000,   /* U+0D65 */
  0x0000,   /* U+0D66 */
  0x0000,   /* U+0D67 */
  0x0000,   /* U+0D68 */
  0x0000,   /* U+0D69 */
  0x0000,   /* U+0D6A */
  0x0000,   /* U+0D6B */
  0x0000,   /* U+0D6C */
  0x0000,   /* U+0D6D */
  0x0000,   /* U+0D6E */
  0x0000,   /* U+0D6F */
  0x0000,   /* U+0D70 */
  0x0000,   /* U+0D71 */
  0x0000,   /* U+0D72 */
  0x0000,   /* U+0D73 */
  0x0000,   /* U+0D74 */
  0x0000,   /* U+0D75 */
  0x0000,   /* U+0D76 */
  0x0000,   /* U+0D77 */
  0x0000,   /* U+0D78 */
  0x0000,   /* U+0D79 */
  0x0004,   /* U+0D7A */
  0x0004,   /* U+0D7B */
  0x0004,   /* U+0D7C */
  0x0004,   /* U+0D7D */
  0x0004,   /* U+0D7E */
  0x0004,   /* U+0D7F */
  0x0000,   /* U+0D80 */
  0x0000,   /* U+0D81 */
  0x0000,   /* U+0D82 */
  0x0000,   /* U+0D83 */
  0x0000,   /* U+0D84 */
  0x0000,   /* U+0D85 */
  0x0000,   /* U+0D86 */
  0x0000,   /* U+0D87 */
  0x0000,   /* U+0D88 */
  0x0000,   /* U+0D89 */
  0x0000,   /* U+0D8A */
  0x0000,   /* U+0D8B */
  0x0000,   /* U+0D8C */
  0x0000,   /* U+0D8D */
  0x0000,   /* U+0D8E */
  0x0000,   /* U+0D8F */
  0x0000,   /* U+0D90 */
  0x0000,   /* U+0D91 */
  0x0000,   /* U+0D92 */
  0x0000,   /* U+0D93 */
  0x0000,   /* U+0D94 */
  0x0000,   /* U+0D95 */
  0x0000,   /* U+0D96 */
  0x0000,   /* U+0D97 */
  0x0000,   /* U+0D98 */
  0x0000,   /* U+0D99 */
  0x0000,   /* U+0D9A */
  0x0000,   /* U+0D9B */
  0x0000,   /* U+0D9C */
  0x0000,   /* U+0D9D */
  0x0000,   /* U+0D9E */
  0x0000,   /* U+0D9F */
  0x0000,   /* U+0DA0 */
  0x0000,   /* U+0DA1 */
  0x0000,   /* U+0DA2 */
  0x0000,   /* U+0DA3 */
  0x0000,   /* U+0DA4 */
  0x0000,   /* U+0DA5 */
  0x0000,   /* U+0DA6 */
  0x0000,   /* U+0DA7 */
  0x0000,   /* U+0DA8 */
  0x0000,   /* U+0DA9 */
  0x0000,   /* U+0DAA */
  0x0000,   /* U+0DAB */
  0x0000,   /* U+0DAC */
  0x0000,   /* U+0DAD */
  0x0000,   /* U+0DAE */
  0x0000,   /* U+0DAF */
  0x0000,   /* U+0DB0 */
  0x0000,   /* U+0DB1 */
  0x0000,   /* U+0DB2 */
  0x0000,   /* U+0DB3 */
  0x0000,   /* U+0DB4 */
  0x0000,   /* U+0DB5 */
  0x0000,   /* U+0DB6 */
  0x0000,   /* U+0DB7 */
  0x0000,   /* U+0DB8 */
  0x0000,   /* U+0DB9 */
  0x0000,   /* U+0DBA */
  0x0000,   /* U+0DBB */
  0x0000,   /* U+0DBC */
  0x0000,   /* U+0DBD */
  0x0000,   /* U+0DBE */
  0x0000,   /* U+0DBF */
  0x0000,   /* U+0DC0 */
  0x0000,   /* U+0DC1 */
  0x0000,   /* U+0DC2 */
  0x0000,   /* U+0DC3 */
  0x0000,   /* U+0DC4 */
  0x0000,   /* U+0DC5 */
  0x0000,   /* U+0DC6 */
  0x0000,   /* U+0DC7 */
  0x0000,   /* U+0DC8 */
  0x0000,   /* U+0DC9 */
  0x0000,   /* U+0DCA */
  0x0000,   /* U+0DCB */
  0x0000,   /* U+0DCC */
  0x0000,   /* U+0DCD */
  0x0000,   /* U+0DCE */
  0x0000,   /* U+0DCF */
  0x0000,   /* U+0DD0 */
  0x0000,   /* U+0DD1 */
  0x0000,   /* U+0DD2 */
  0x0000,   /* U+0DD3 */
  0x0000,   /* U+0DD4 */
  0x0000,   /* U+0DD5 */
  0x0000,   /* U+0DD6 */
  0x0000,   /* U+0DD7 */
  0x0000,   /* U+0DD8 */
  0x0000,   /* U+0DD9 */
  0x0000,   /* U+0DDA */
  0x0000,   /* U+0DDB */
  0x0000,   /* U+0DDC */
  0x0000,   /* U+0DDD */
  0x0000,   /* U+0DDE */
  0x0000,   /* U+0DDF */
  0x0000,   /* U+0DE0 */
  0x0000,   /* U+0DE1 */
  0x0000,   /* U+0DE2 */
  0x0000,   /* U+0DE3 */
  0x0000,   /* U+0DE4 */
  0x0000,   /* U+0DE5 */
  0x0000,   /* U+0DE6 */
  0x0000,   /* U+0DE7 */
  0x0000,   /* U+0DE8 */
  0x0000,   /* U+0DE9 */
  0x0000,   /* U+0DEA */
  0x0000,   /* U+0DEB */
  0x0000,   /* U+0DEC */
  0x0000,   /* U+0DED */
  0x0000,   /* U+0DEE */
  0x0000,   /* U+0DEF */
  0x0000,   /* U+0DF0 */
  0x0000,   /* U+0DF1 */
  0x0000,   /* U+0DF2 */
  0x0000,   /* U+0DF3 */
  0x0000,   /* U+0DF4 */
  0x0000,   /* U+0DF5 */
  0x0000,   /* U+0DF6 */
  0x0000,   /* U+0DF7 */
  0x0000,   /* U+0DF8 */
  0x0000,   /* U+0DF9 */
  0x0000,   /* U+0DFA */
  0x0000,   /* U+0DFB */
  0x0000,   /* U+0DFC */
  0x0000,   /* U+0DFD */
  0x0000,   /* U+0DFE */
  0x0000,   /* U+0DFF */
};

uint16 uca900_p00E[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 1, 1, 2, 1, 2, 2, 1, 1, 2, 1, 2, 2, 1, 2, 2, 
  2, 2, 2, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 2, 1, 2, 1, 2, 2, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC0,   /* U+0E00 */
  0x2D73,   /* U+0E01 */
  0x2D74,   /* U+0E02 */
  0x2D75,   /* U+0E03 */
  0x2D76,   /* U+0E04 */
  0x2D77,   /* U+0E05 */
  0x2D78,   /* U+0E06 */
  0x2D79,   /* U+0E07 */
  0x2D7A,   /* U+0E08 */
  0x2D7B,   /* U+0E09 */
  0x2D7C,   /* U+0E0A */
  0x2D7D,   /* U+0E0B */
  0x2D7E,   /* U+0E0C */
  0x2D7F,   /* U+0E0D */
  0x2D80,   /* U+0E0E */
  0x2D81,   /* U+0E0F */
  0x2D82,   /* U+0E10 */
  0x2D83,   /* U+0E11 */
  0x2D84,   /* U+0E12 */
  0x2D85,   /* U+0E13 */
  0x2D86,   /* U+0E14 */
  0x2D87,   /* U+0E15 */
  0x2D88,   /* U+0E16 */
  0x2D89,   /* U+0E17 */
  0x2D8A,   /* U+0E18 */
  0x2D8B,   /* U+0E19 */
  0x2D8C,   /* U+0E1A */
  0x2D8D,   /* U+0E1B */
  0x2D8E,   /* U+0E1C */
  0x2D8F,   /* U+0E1D */
  0x2D90,   /* U+0E1E */
  0x2D91,   /* U+0E1F */
  0x2D92,   /* U+0E20 */
  0x2D93,   /* U+0E21 */
  0x2D94,   /* U+0E22 */
  0x2D95,   /* U+0E23 */
  0x2D96,   /* U+0E24 */
  0x2D97,   /* U+0E25 */
  0x2D98,   /* U+0E26 */
  0x2D99,   /* U+0E27 */
  0x2D9A,   /* U+0E28 */
  0x2D9B,   /* U+0E29 */
  0x2D9C,   /* U+0E2A */
  0x2D9D,   /* U+0E2B */
  0x2D9E,   /* U+0E2C */
  0x2D9F,   /* U+0E2D */
  0x2DA0,   /* U+0E2E */
  0x2DA1,   /* U+0E2F */
  0x2DA2,   /* U+0E30 */
  0x2DA3,   /* U+0E31 */
  0x2DA4,   /* U+0E32 */
  0x2DA5,   /* U+0E33 */
  0x2DA6,   /* U+0E34 */
  0x2DA7,   /* U+0E35 */
  0x2DA8,   /* U+0E36 */
  0x2DA9,   /* U+0E37 */
  0x2DAA,   /* U+0E38 */
  0x2DAB,   /* U+0E39 */
  0x2DAC,   /* U+0E3A */
  0xFBC0,   /* U+0E3B */
  0xFBC0,   /* U+0E3C */
  0xFBC0,   /* U+0E3D */
  0xFBC0,   /* U+0E3E */
  0x1C1D,   /* U+0E3F */
  0x2DAD,   /* U+0E40 */
  0x2DAE,   /* U+0E41 */
  0x2DAF,   /* U+0E42 */
  0x2DB0,   /* U+0E43 */
  0x2DB1,   /* U+0E44 */
  0x2DB2,   /* U+0E45 */
  0x1BFB,   /* U+0E46 */
  0x0000,   /* U+0E47 */
  0x0000,   /* U+0E48 */
  0x0000,   /* U+0E49 */
  0x0000,   /* U+0E4A */
  0x0000,   /* U+0E4B */
  0x0000,   /* U+0E4C */
  0x0000,   /* U+0E4D */
  0x0000,   /* U+0E4E */
  0x03EE,   /* U+0E4F */
  0x1C3D,   /* U+0E50 */
  0x1C3E,   /* U+0E51 */
  0x1C3F,   /* U+0E52 */
  0x1C40,   /* U+0E53 */
  0x1C41,   /* U+0E54 */
  0x1C42,   /* U+0E55 */
  0x1C43,   /* U+0E56 */
  0x1C44,   /* U+0E57 */
  0x1C45,   /* U+0E58 */
  0x1C46,   /* U+0E59 */
  0x03EF,   /* U+0E5A */
  0x03F0,   /* U+0E5B */
  0xFBC0,   /* U+0E5C */
  0xFBC0,   /* U+0E5D */
  0xFBC0,   /* U+0E5E */
  0xFBC0,   /* U+0E5F */
  0xFBC0,   /* U+0E60 */
  0xFBC0,   /* U+0E61 */
  0xFBC0,   /* U+0E62 */
  0xFBC0,   /* U+0E63 */
  0xFBC0,   /* U+0E64 */
  0xFBC0,   /* U+0E65 */
  0xFBC0,   /* U+0E66 */
  0xFBC0,   /* U+0E67 */
  0xFBC0,   /* U+0E68 */
  0xFBC0,   /* U+0E69 */
  0xFBC0,   /* U+0E6A */
  0xFBC0,   /* U+0E6B */
  0xFBC0,   /* U+0E6C */
  0xFBC0,   /* U+0E6D */
  0xFBC0,   /* U+0E6E */
  0xFBC0,   /* U+0E6F */
  0xFBC0,   /* U+0E70 */
  0xFBC0,   /* U+0E71 */
  0xFBC0,   /* U+0E72 */
  0xFBC0,   /* U+0E73 */
  0xFBC0,   /* U+0E74 */
  0xFBC0,   /* U+0E75 */
  0xFBC0,   /* U+0E76 */
  0xFBC0,   /* U+0E77 */
  0xFBC0,   /* U+0E78 */
  0xFBC0,   /* U+0E79 */
  0xFBC0,   /* U+0E7A */
  0xFBC0,   /* U+0E7B */
  0xFBC0,   /* U+0E7C */
  0xFBC0,   /* U+0E7D */
  0xFBC0,   /* U+0E7E */
  0xFBC0,   /* U+0E7F */
  0xFBC0,   /* U+0E80 */
  0x2DB4,   /* U+0E81 */
  0x2DB5,   /* U+0E82 */
  0xFBC0,   /* U+0E83 */
  0x2DB6,   /* U+0E84 */
  0xFBC0,   /* U+0E85 */
  0xFBC0,   /* U+0E86 */
  0x2DB7,   /* U+0E87 */
  0x2DB8,   /* U+0E88 */
  0xFBC0,   /* U+0E89 */
  0x2DBA,   /* U+0E8A */
  0xFBC0,   /* U+0E8B */
  0xFBC0,   /* U+0E8C */
  0x2DBC,   /* U+0E8D */
  0xFBC0,   /* U+0E8E */
  0xFBC0,   /* U+0E8F */
  0xFBC0,   /* U+0E90 */
  0xFBC0,   /* U+0E91 */
  0xFBC0,   /* U+0E92 */
  0xFBC0,   /* U+0E93 */
  0x2DBD,   /* U+0E94 */
  0x2DBE,   /* U+0E95 */
  0x2DBF,   /* U+0E96 */
  0x2DC0,   /* U+0E97 */
  0xFBC0,   /* U+0E98 */
  0x2DC1,   /* U+0E99 */
  0x2DC2,   /* U+0E9A */
  0x2DC3,   /* U+0E9B */
  0x2DC4,   /* U+0E9C */
  0x2DC5,   /* U+0E9D */
  0x2DC6,   /* U+0E9E */
  0x2DC7,   /* U+0E9F */
  0xFBC0,   /* U+0EA0 */
  0x2DC8,   /* U+0EA1 */
  0x2DC9,   /* U+0EA2 */
  0x2DCA,   /* U+0EA3 */
  0xFBC0,   /* U+0EA4 */
  0x2DCB,   /* U+0EA5 */
  0xFBC0,   /* U+0EA6 */
  0x2DCC,   /* U+0EA7 */
  0xFBC0,   /* U+0EA8 */
  0xFBC0,   /* U+0EA9 */
  0x2DB9,   /* U+0EAA */
  0x2DCD,   /* U+0EAB */
  0xFBC0,   /* U+0EAC */
  0x2DCE,   /* U+0EAD */
  0x2DCF,   /* U+0EAE */
  0x2DD0,   /* U+0EAF */
  0x2DD1,   /* U+0EB0 */
  0x2DD2,   /* U+0EB1 */
  0x2DD3,   /* U+0EB2 */
  0x2DD4,   /* U+0EB3 */
  0x2DD5,   /* U+0EB4 */
  0x2DD6,   /* U+0EB5 */
  0x2DD7,   /* U+0EB6 */
  0x2DD8,   /* U+0EB7 */
  0x2DD9,   /* U+0EB8 */
  0x2DDA,   /* U+0EB9 */
  0xFBC0,   /* U+0EBA */
  0x2DDB,   /* U+0EBB */
  0x2DDC,   /* U+0EBC */
  0x2DDD,   /* U+0EBD */
  0xFBC0,   /* U+0EBE */
  0xFBC0,   /* U+0EBF */
  0x2DDE,   /* U+0EC0 */
  0x2DDF,   /* U+0EC1 */
  0x2DE0,   /* U+0EC2 */
  0x2DE1,   /* U+0EC3 */
  0x2DE2,   /* U+0EC4 */
  0xFBC0,   /* U+0EC5 */
  0x1BFC,   /* U+0EC6 */
  0xFBC0,   /* U+0EC7 */
  0x0000,   /* U+0EC8 */
  0x0000,   /* U+0EC9 */
  0x0000,   /* U+0ECA */
  0x0000,   /* U+0ECB */
  0x0000,   /* U+0ECC */
  0x0000,   /* U+0ECD */
  0xFBC0,   /* U+0ECE */
  0xFBC0,   /* U+0ECF */
  0x1C3D,   /* U+0ED0 */
  0x1C3E,   /* U+0ED1 */
  0x1C3F,   /* U+0ED2 */
  0x1C40,   /* U+0ED3 */
  0x1C41,   /* U+0ED4 */
  0x1C42,   /* U+0ED5 */
  0x1C43,   /* U+0ED6 */
  0x1C44,   /* U+0ED7 */
  0x1C45,   /* U+0ED8 */
  0x1C46,   /* U+0ED9 */
  0xFBC0,   /* U+0EDA */
  0xFBC0,   /* U+0EDB */
  0x2DCD,   /* U+0EDC */
  0x2DCD,   /* U+0EDD */
  0x2DB3,   /* U+0EDE */
  0x2DBB,   /* U+0EDF */
  0xFBC0,   /* U+0EE0 */
  0xFBC0,   /* U+0EE1 */
  0xFBC0,   /* U+0EE2 */
  0xFBC0,   /* U+0EE3 */
  0xFBC0,   /* U+0EE4 */
  0xFBC0,   /* U+0EE5 */
  0xFBC0,   /* U+0EE6 */
  0xFBC0,   /* U+0EE7 */
  0xFBC0,   /* U+0EE8 */
  0xFBC0,   /* U+0EE9 */
  0xFBC0,   /* U+0EEA */
  0xFBC0,   /* U+0EEB */
  0xFBC0,   /* U+0EEC */
  0xFBC0,   /* U+0EED */
  0xFBC0,   /* U+0EEE */
  0xFBC0,   /* U+0EEF */
  0xFBC0,   /* U+0EF0 */
  0xFBC0,   /* U+0EF1 */
  0xFBC0,   /* U+0EF2 */
  0xFBC0,   /* U+0EF3 */
  0xFBC0,   /* U+0EF4 */
  0xFBC0,   /* U+0EF5 */
  0xFBC0,   /* U+0EF6 */
  0xFBC0,   /* U+0EF7 */
  0xFBC0,   /* U+0EF8 */
  0xFBC0,   /* U+0EF9 */
  0xFBC0,   /* U+0EFA */
  0xFBC0,   /* U+0EFB */
  0xFBC0,   /* U+0EFC */
  0xFBC0,   /* U+0EFD */
  0xFBC0,   /* U+0EFE */
  0xFBC0,   /* U+0EFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0E00 */
  0x0020,   /* U+0E01 */
  0x0020,   /* U+0E02 */
  0x0020,   /* U+0E03 */
  0x0020,   /* U+0E04 */
  0x0020,   /* U+0E05 */
  0x0020,   /* U+0E06 */
  0x0020,   /* U+0E07 */
  0x0020,   /* U+0E08 */
  0x0020,   /* U+0E09 */
  0x0020,   /* U+0E0A */
  0x0020,   /* U+0E0B */
  0x0020,   /* U+0E0C */
  0x0020,   /* U+0E0D */
  0x0020,   /* U+0E0E */
  0x0020,   /* U+0E0F */
  0x0020,   /* U+0E10 */
  0x0020,   /* U+0E11 */
  0x0020,   /* U+0E12 */
  0x0020,   /* U+0E13 */
  0x0020,   /* U+0E14 */
  0x0020,   /* U+0E15 */
  0x0020,   /* U+0E16 */
  0x0020,   /* U+0E17 */
  0x0020,   /* U+0E18 */
  0x0020,   /* U+0E19 */
  0x0020,   /* U+0E1A */
  0x0020,   /* U+0E1B */
  0x0020,   /* U+0E1C */
  0x0020,   /* U+0E1D */
  0x0020,   /* U+0E1E */
  0x0020,   /* U+0E1F */
  0x0020,   /* U+0E20 */
  0x0020,   /* U+0E21 */
  0x0020,   /* U+0E22 */
  0x0020,   /* U+0E23 */
  0x0020,   /* U+0E24 */
  0x0020,   /* U+0E25 */
  0x0020,   /* U+0E26 */
  0x0020,   /* U+0E27 */
  0x0020,   /* U+0E28 */
  0x0020,   /* U+0E29 */
  0x0020,   /* U+0E2A */
  0x0020,   /* U+0E2B */
  0x0020,   /* U+0E2C */
  0x0020,   /* U+0E2D */
  0x0020,   /* U+0E2E */
  0x0020,   /* U+0E2F */
  0x0020,   /* U+0E30 */
  0x0020,   /* U+0E31 */
  0x0020,   /* U+0E32 */
  0x0020,   /* U+0E33 */
  0x0020,   /* U+0E34 */
  0x0020,   /* U+0E35 */
  0x0020,   /* U+0E36 */
  0x0020,   /* U+0E37 */
  0x0020,   /* U+0E38 */
  0x0020,   /* U+0E39 */
  0x0020,   /* U+0E3A */
  0x0020,   /* U+0E3B */
  0x0020,   /* U+0E3C */
  0x0020,   /* U+0E3D */
  0x0020,   /* U+0E3E */
  0x0020,   /* U+0E3F */
  0x0020,   /* U+0E40 */
  0x0020,   /* U+0E41 */
  0x0020,   /* U+0E42 */
  0x0020,   /* U+0E43 */
  0x0020,   /* U+0E44 */
  0x0020,   /* U+0E45 */
  0x0020,   /* U+0E46 */
  0x00D2,   /* U+0E47 */
  0x00D3,   /* U+0E48 */
  0x00D4,   /* U+0E49 */
  0x00D5,   /* U+0E4A */
  0x00D6,   /* U+0E4B */
  0x00D7,   /* U+0E4C */
  0x00D8,   /* U+0E4D */
  0x00D1,   /* U+0E4E */
  0x0020,   /* U+0E4F */
  0x0020,   /* U+0E50 */
  0x0020,   /* U+0E51 */
  0x0020,   /* U+0E52 */
  0x0020,   /* U+0E53 */
  0x0020,   /* U+0E54 */
  0x0020,   /* U+0E55 */
  0x0020,   /* U+0E56 */
  0x0020,   /* U+0E57 */
  0x0020,   /* U+0E58 */
  0x0020,   /* U+0E59 */
  0x0020,   /* U+0E5A */
  0x0020,   /* U+0E5B */
  0x0020,   /* U+0E5C */
  0x0020,   /* U+0E5D */
  0x0020,   /* U+0E5E */
  0x0020,   /* U+0E5F */
  0x0020,   /* U+0E60 */
  0x0020,   /* U+0E61 */
  0x0020,   /* U+0E62 */
  0x0020,   /* U+0E63 */
  0x0020,   /* U+0E64 */
  0x0020,   /* U+0E65 */
  0x0020,   /* U+0E66 */
  0x0020,   /* U+0E67 */
  0x0020,   /* U+0E68 */
  0x0020,   /* U+0E69 */
  0x0020,   /* U+0E6A */
  0x0020,   /* U+0E6B */
  0x0020,   /* U+0E6C */
  0x0020,   /* U+0E6D */
  0x0020,   /* U+0E6E */
  0x0020,   /* U+0E6F */
  0x0020,   /* U+0E70 */
  0x0020,   /* U+0E71 */
  0x0020,   /* U+0E72 */
  0x0020,   /* U+0E73 */
  0x0020,   /* U+0E74 */
  0x0020,   /* U+0E75 */
  0x0020,   /* U+0E76 */
  0x0020,   /* U+0E77 */
  0x0020,   /* U+0E78 */
  0x0020,   /* U+0E79 */
  0x0020,   /* U+0E7A */
  0x0020,   /* U+0E7B */
  0x0020,   /* U+0E7C */
  0x0020,   /* U+0E7D */
  0x0020,   /* U+0E7E */
  0x0020,   /* U+0E7F */
  0x0020,   /* U+0E80 */
  0x0020,   /* U+0E81 */
  0x0020,   /* U+0E82 */
  0x0020,   /* U+0E83 */
  0x0020,   /* U+0E84 */
  0x0020,   /* U+0E85 */
  0x0020,   /* U+0E86 */
  0x0020,   /* U+0E87 */
  0x0020,   /* U+0E88 */
  0x0020,   /* U+0E89 */
  0x0020,   /* U+0E8A */
  0x0020,   /* U+0E8B */
  0x0020,   /* U+0E8C */
  0x0020,   /* U+0E8D */
  0x0020,   /* U+0E8E */
  0x0020,   /* U+0E8F */
  0x0020,   /* U+0E90 */
  0x0020,   /* U+0E91 */
  0x0020,   /* U+0E92 */
  0x0020,   /* U+0E93 */
  0x0020,   /* U+0E94 */
  0x0020,   /* U+0E95 */
  0x0020,   /* U+0E96 */
  0x0020,   /* U+0E97 */
  0x0020,   /* U+0E98 */
  0x0020,   /* U+0E99 */
  0x0020,   /* U+0E9A */
  0x0020,   /* U+0E9B */
  0x0020,   /* U+0E9C */
  0x0020,   /* U+0E9D */
  0x0020,   /* U+0E9E */
  0x0020,   /* U+0E9F */
  0x0020,   /* U+0EA0 */
  0x0020,   /* U+0EA1 */
  0x0020,   /* U+0EA2 */
  0x0020,   /* U+0EA3 */
  0x0020,   /* U+0EA4 */
  0x0020,   /* U+0EA5 */
  0x0020,   /* U+0EA6 */
  0x0020,   /* U+0EA7 */
  0x0020,   /* U+0EA8 */
  0x0020,   /* U+0EA9 */
  0x0020,   /* U+0EAA */
  0x0020,   /* U+0EAB */
  0x0020,   /* U+0EAC */
  0x0020,   /* U+0EAD */
  0x0020,   /* U+0EAE */
  0x0020,   /* U+0EAF */
  0x0020,   /* U+0EB0 */
  0x0020,   /* U+0EB1 */
  0x0020,   /* U+0EB2 */
  0x0020,   /* U+0EB3 */
  0x0020,   /* U+0EB4 */
  0x0020,   /* U+0EB5 */
  0x0020,   /* U+0EB6 */
  0x0020,   /* U+0EB7 */
  0x0020,   /* U+0EB8 */
  0x0020,   /* U+0EB9 */
  0x0020,   /* U+0EBA */
  0x0020,   /* U+0EBB */
  0x0020,   /* U+0EBC */
  0x0020,   /* U+0EBD */
  0x0020,   /* U+0EBE */
  0x0020,   /* U+0EBF */
  0x0020,   /* U+0EC0 */
  0x0020,   /* U+0EC1 */
  0x0020,   /* U+0EC2 */
  0x0020,   /* U+0EC3 */
  0x0020,   /* U+0EC4 */
  0x0020,   /* U+0EC5 */
  0x0020,   /* U+0EC6 */
  0x0020,   /* U+0EC7 */
  0x00D9,   /* U+0EC8 */
  0x00DA,   /* U+0EC9 */
  0x00DB,   /* U+0ECA */
  0x00DC,   /* U+0ECB */
  0x00DD,   /* U+0ECC */
  0x00DE,   /* U+0ECD */
  0x0020,   /* U+0ECE */
  0x0020,   /* U+0ECF */
  0x0020,   /* U+0ED0 */
  0x0020,   /* U+0ED1 */
  0x0020,   /* U+0ED2 */
  0x0020,   /* U+0ED3 */
  0x0020,   /* U+0ED4 */
  0x0020,   /* U+0ED5 */
  0x0020,   /* U+0ED6 */
  0x0020,   /* U+0ED7 */
  0x0020,   /* U+0ED8 */
  0x0020,   /* U+0ED9 */
  0x0020,   /* U+0EDA */
  0x0020,   /* U+0EDB */
  0x0020,   /* U+0EDC */
  0x0020,   /* U+0EDD */
  0x0020,   /* U+0EDE */
  0x0020,   /* U+0EDF */
  0x0020,   /* U+0EE0 */
  0x0020,   /* U+0EE1 */
  0x0020,   /* U+0EE2 */
  0x0020,   /* U+0EE3 */
  0x0020,   /* U+0EE4 */
  0x0020,   /* U+0EE5 */
  0x0020,   /* U+0EE6 */
  0x0020,   /* U+0EE7 */
  0x0020,   /* U+0EE8 */
  0x0020,   /* U+0EE9 */
  0x0020,   /* U+0EEA */
  0x0020,   /* U+0EEB */
  0x0020,   /* U+0EEC */
  0x0020,   /* U+0EED */
  0x0020,   /* U+0EEE */
  0x0020,   /* U+0EEF */
  0x0020,   /* U+0EF0 */
  0x0020,   /* U+0EF1 */
  0x0020,   /* U+0EF2 */
  0x0020,   /* U+0EF3 */
  0x0020,   /* U+0EF4 */
  0x0020,   /* U+0EF5 */
  0x0020,   /* U+0EF6 */
  0x0020,   /* U+0EF7 */
  0x0020,   /* U+0EF8 */
  0x0020,   /* U+0EF9 */
  0x0020,   /* U+0EFA */
  0x0020,   /* U+0EFB */
  0x0020,   /* U+0EFC */
  0x0020,   /* U+0EFD */
  0x0020,   /* U+0EFE */
  0x0020,   /* U+0EFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+0E00 */
  0x0002,   /* U+0E01 */
  0x0002,   /* U+0E02 */
  0x0002,   /* U+0E03 */
  0x0002,   /* U+0E04 */
  0x0002,   /* U+0E05 */
  0x0002,   /* U+0E06 */
  0x0002,   /* U+0E07 */
  0x0002,   /* U+0E08 */
  0x0002,   /* U+0E09 */
  0x0002,   /* U+0E0A */
  0x0002,   /* U+0E0B */
  0x0002,   /* U+0E0C */
  0x0002,   /* U+0E0D */
  0x0002,   /* U+0E0E */
  0x0002,   /* U+0E0F */
  0x0002,   /* U+0E10 */
  0x0002,   /* U+0E11 */
  0x0002,   /* U+0E12 */
  0x0002,   /* U+0E13 */
  0x0002,   /* U+0E14 */
  0x0002,   /* U+0E15 */
  0x0002,   /* U+0E16 */
  0x0002,   /* U+0E17 */
  0x0002,   /* U+0E18 */
  0x0002,   /* U+0E19 */
  0x0002,   /* U+0E1A */
  0x0002,   /* U+0E1B */
  0x0002,   /* U+0E1C */
  0x0002,   /* U+0E1D */
  0x0002,   /* U+0E1E */
  0x0002,   /* U+0E1F */
  0x0002,   /* U+0E20 */
  0x0002,   /* U+0E21 */
  0x0002,   /* U+0E22 */
  0x0002,   /* U+0E23 */
  0x0002,   /* U+0E24 */
  0x0002,   /* U+0E25 */
  0x0002,   /* U+0E26 */
  0x0002,   /* U+0E27 */
  0x0002,   /* U+0E28 */
  0x0002,   /* U+0E29 */
  0x0002,   /* U+0E2A */
  0x0002,   /* U+0E2B */
  0x0002,   /* U+0E2C */
  0x0002,   /* U+0E2D */
  0x0002,   /* U+0E2E */
  0x0002,   /* U+0E2F */
  0x0002,   /* U+0E30 */
  0x0002,   /* U+0E31 */
  0x0002,   /* U+0E32 */
  0x0002,   /* U+0E33 */
  0x0002,   /* U+0E34 */
  0x0002,   /* U+0E35 */
  0x0002,   /* U+0E36 */
  0x0002,   /* U+0E37 */
  0x0002,   /* U+0E38 */
  0x0002,   /* U+0E39 */
  0x0002,   /* U+0E3A */
  0x0002,   /* U+0E3B */
  0x0002,   /* U+0E3C */
  0x0002,   /* U+0E3D */
  0x0002,   /* U+0E3E */
  0x0002,   /* U+0E3F */
  0x0002,   /* U+0E40 */
  0x0002,   /* U+0E41 */
  0x0002,   /* U+0E42 */
  0x0002,   /* U+0E43 */
  0x0002,   /* U+0E44 */
  0x0002,   /* U+0E45 */
  0x0002,   /* U+0E46 */
  0x0002,   /* U+0E47 */
  0x0002,   /* U+0E48 */
  0x0002,   /* U+0E49 */
  0x0002,   /* U+0E4A */
  0x0002,   /* U+0E4B */
  0x0002,   /* U+0E4C */
  0x0002,   /* U+0E4D */
  0x0002,   /* U+0E4E */
  0x0002,   /* U+0E4F */
  0x0002,   /* U+0E50 */
  0x0002,   /* U+0E51 */
  0x0002,   /* U+0E52 */
  0x0002,   /* U+0E53 */
  0x0002,   /* U+0E54 */
  0x0002,   /* U+0E55 */
  0x0002,   /* U+0E56 */
  0x0002,   /* U+0E57 */
  0x0002,   /* U+0E58 */
  0x0002,   /* U+0E59 */
  0x0002,   /* U+0E5A */
  0x0002,   /* U+0E5B */
  0x0002,   /* U+0E5C */
  0x0002,   /* U+0E5D */
  0x0002,   /* U+0E5E */
  0x0002,   /* U+0E5F */
  0x0002,   /* U+0E60 */
  0x0002,   /* U+0E61 */
  0x0002,   /* U+0E62 */
  0x0002,   /* U+0E63 */
  0x0002,   /* U+0E64 */
  0x0002,   /* U+0E65 */
  0x0002,   /* U+0E66 */
  0x0002,   /* U+0E67 */
  0x0002,   /* U+0E68 */
  0x0002,   /* U+0E69 */
  0x0002,   /* U+0E6A */
  0x0002,   /* U+0E6B */
  0x0002,   /* U+0E6C */
  0x0002,   /* U+0E6D */
  0x0002,   /* U+0E6E */
  0x0002,   /* U+0E6F */
  0x0002,   /* U+0E70 */
  0x0002,   /* U+0E71 */
  0x0002,   /* U+0E72 */
  0x0002,   /* U+0E73 */
  0x0002,   /* U+0E74 */
  0x0002,   /* U+0E75 */
  0x0002,   /* U+0E76 */
  0x0002,   /* U+0E77 */
  0x0002,   /* U+0E78 */
  0x0002,   /* U+0E79 */
  0x0002,   /* U+0E7A */
  0x0002,   /* U+0E7B */
  0x0002,   /* U+0E7C */
  0x0002,   /* U+0E7D */
  0x0002,   /* U+0E7E */
  0x0002,   /* U+0E7F */
  0x0002,   /* U+0E80 */
  0x0002,   /* U+0E81 */
  0x0002,   /* U+0E82 */
  0x0002,   /* U+0E83 */
  0x0002,   /* U+0E84 */
  0x0002,   /* U+0E85 */
  0x0002,   /* U+0E86 */
  0x0002,   /* U+0E87 */
  0x0002,   /* U+0E88 */
  0x0002,   /* U+0E89 */
  0x0002,   /* U+0E8A */
  0x0002,   /* U+0E8B */
  0x0002,   /* U+0E8C */
  0x0002,   /* U+0E8D */
  0x0002,   /* U+0E8E */
  0x0002,   /* U+0E8F */
  0x0002,   /* U+0E90 */
  0x0002,   /* U+0E91 */
  0x0002,   /* U+0E92 */
  0x0002,   /* U+0E93 */
  0x0002,   /* U+0E94 */
  0x0002,   /* U+0E95 */
  0x0002,   /* U+0E96 */
  0x0002,   /* U+0E97 */
  0x0002,   /* U+0E98 */
  0x0002,   /* U+0E99 */
  0x0002,   /* U+0E9A */
  0x0002,   /* U+0E9B */
  0x0002,   /* U+0E9C */
  0x0002,   /* U+0E9D */
  0x0002,   /* U+0E9E */
  0x0002,   /* U+0E9F */
  0x0002,   /* U+0EA0 */
  0x0002,   /* U+0EA1 */
  0x0002,   /* U+0EA2 */
  0x0002,   /* U+0EA3 */
  0x0002,   /* U+0EA4 */
  0x0002,   /* U+0EA5 */
  0x0002,   /* U+0EA6 */
  0x0002,   /* U+0EA7 */
  0x0002,   /* U+0EA8 */
  0x0002,   /* U+0EA9 */
  0x0002,   /* U+0EAA */
  0x0002,   /* U+0EAB */
  0x0002,   /* U+0EAC */
  0x0002,   /* U+0EAD */
  0x0002,   /* U+0EAE */
  0x0002,   /* U+0EAF */
  0x0002,   /* U+0EB0 */
  0x0002,   /* U+0EB1 */
  0x0002,   /* U+0EB2 */
  0x0002,   /* U+0EB3 */
  0x0002,   /* U+0EB4 */
  0x0002,   /* U+0EB5 */
  0x0002,   /* U+0EB6 */
  0x0002,   /* U+0EB7 */
  0x0002,   /* U+0EB8 */
  0x0002,   /* U+0EB9 */
  0x0002,   /* U+0EBA */
  0x0002,   /* U+0EBB */
  0x0002,   /* U+0EBC */
  0x0002,   /* U+0EBD */
  0x0002,   /* U+0EBE */
  0x0002,   /* U+0EBF */
  0x0002,   /* U+0EC0 */
  0x0002,   /* U+0EC1 */
  0x0002,   /* U+0EC2 */
  0x0002,   /* U+0EC3 */
  0x0002,   /* U+0EC4 */
  0x0002,   /* U+0EC5 */
  0x0002,   /* U+0EC6 */
  0x0002,   /* U+0EC7 */
  0x0002,   /* U+0EC8 */
  0x0002,   /* U+0EC9 */
  0x0002,   /* U+0ECA */
  0x0002,   /* U+0ECB */
  0x0002,   /* U+0ECC */
  0x0002,   /* U+0ECD */
  0x0002,   /* U+0ECE */
  0x0002,   /* U+0ECF */
  0x0002,   /* U+0ED0 */
  0x0002,   /* U+0ED1 */
  0x0002,   /* U+0ED2 */
  0x0002,   /* U+0ED3 */
  0x0002,   /* U+0ED4 */
  0x0002,   /* U+0ED5 */
  0x0002,   /* U+0ED6 */
  0x0002,   /* U+0ED7 */
  0x0002,   /* U+0ED8 */
  0x0002,   /* U+0ED9 */
  0x0002,   /* U+0EDA */
  0x0002,   /* U+0EDB */
  0x0004,   /* U+0EDC */
  0x0004,   /* U+0EDD */
  0x0002,   /* U+0EDE */
  0x0002,   /* U+0EDF */
  0x0002,   /* U+0EE0 */
  0x0002,   /* U+0EE1 */
  0x0002,   /* U+0EE2 */
  0x0002,   /* U+0EE3 */
  0x0002,   /* U+0EE4 */
  0x0002,   /* U+0EE5 */
  0x0002,   /* U+0EE6 */
  0x0002,   /* U+0EE7 */
  0x0002,   /* U+0EE8 */
  0x0002,   /* U+0EE9 */
  0x0002,   /* U+0EEA */
  0x0002,   /* U+0EEB */
  0x0002,   /* U+0EEC */
  0x0002,   /* U+0EED */
  0x0002,   /* U+0EEE */
  0x0002,   /* U+0EEF */
  0x0002,   /* U+0EF0 */
  0x0002,   /* U+0EF1 */
  0x0002,   /* U+0EF2 */
  0x0002,   /* U+0EF3 */
  0x0002,   /* U+0EF4 */
  0x0002,   /* U+0EF5 */
  0x0002,   /* U+0EF6 */
  0x0002,   /* U+0EF7 */
  0x0002,   /* U+0EF8 */
  0x0002,   /* U+0EF9 */
  0x0002,   /* U+0EFA */
  0x0002,   /* U+0EFB */
  0x0002,   /* U+0EFC */
  0x0002,   /* U+0EFD */
  0x0002,   /* U+0EFE */
  0x0002,   /* U+0EFF */

  /* Primary weight 2 for each character. */
  0x8E00,   /* U+0E00 */
  0x0000,   /* U+0E01 */
  0x0000,   /* U+0E02 */
  0x0000,   /* U+0E03 */
  0x0000,   /* U+0E04 */
  0x0000,   /* U+0E05 */
  0x0000,   /* U+0E06 */
  0x0000,   /* U+0E07 */
  0x0000,   /* U+0E08 */
  0x0000,   /* U+0E09 */
  0x0000,   /* U+0E0A */
  0x0000,   /* U+0E0B */
  0x0000,   /* U+0E0C */
  0x0000,   /* U+0E0D */
  0x0000,   /* U+0E0E */
  0x0000,   /* U+0E0F */
  0x0000,   /* U+0E10 */
  0x0000,   /* U+0E11 */
  0x0000,   /* U+0E12 */
  0x0000,   /* U+0E13 */
  0x0000,   /* U+0E14 */
  0x0000,   /* U+0E15 */
  0x0000,   /* U+0E16 */
  0x0000,   /* U+0E17 */
  0x0000,   /* U+0E18 */
  0x0000,   /* U+0E19 */
  0x0000,   /* U+0E1A */
  0x0000,   /* U+0E1B */
  0x0000,   /* U+0E1C */
  0x0000,   /* U+0E1D */
  0x0000,   /* U+0E1E */
  0x0000,   /* U+0E1F */
  0x0000,   /* U+0E20 */
  0x0000,   /* U+0E21 */
  0x0000,   /* U+0E22 */
  0x0000,   /* U+0E23 */
  0x0000,   /* U+0E24 */
  0x0000,   /* U+0E25 */
  0x0000,   /* U+0E26 */
  0x0000,   /* U+0E27 */
  0x0000,   /* U+0E28 */
  0x0000,   /* U+0E29 */
  0x0000,   /* U+0E2A */
  0x0000,   /* U+0E2B */
  0x0000,   /* U+0E2C */
  0x0000,   /* U+0E2D */
  0x0000,   /* U+0E2E */
  0x0000,   /* U+0E2F */
  0x0000,   /* U+0E30 */
  0x0000,   /* U+0E31 */
  0x0000,   /* U+0E32 */
  0x0000,   /* U+0E33 */
  0x0000,   /* U+0E34 */
  0x0000,   /* U+0E35 */
  0x0000,   /* U+0E36 */
  0x0000,   /* U+0E37 */
  0x0000,   /* U+0E38 */
  0x0000,   /* U+0E39 */
  0x0000,   /* U+0E3A */
  0x8E3B,   /* U+0E3B */
  0x8E3C,   /* U+0E3C */
  0x8E3D,   /* U+0E3D */
  0x8E3E,   /* U+0E3E */
  0x0000,   /* U+0E3F */
  0x0000,   /* U+0E40 */
  0x0000,   /* U+0E41 */
  0x0000,   /* U+0E42 */
  0x0000,   /* U+0E43 */
  0x0000,   /* U+0E44 */
  0x0000,   /* U+0E45 */
  0x0000,   /* U+0E46 */
  0x0000,   /* U+0E47 */
  0x0000,   /* U+0E48 */
  0x0000,   /* U+0E49 */
  0x0000,   /* U+0E4A */
  0x0000,   /* U+0E4B */
  0x0000,   /* U+0E4C */
  0x0000,   /* U+0E4D */
  0x0000,   /* U+0E4E */
  0x0000,   /* U+0E4F */
  0x0000,   /* U+0E50 */
  0x0000,   /* U+0E51 */
  0x0000,   /* U+0E52 */
  0x0000,   /* U+0E53 */
  0x0000,   /* U+0E54 */
  0x0000,   /* U+0E55 */
  0x0000,   /* U+0E56 */
  0x0000,   /* U+0E57 */
  0x0000,   /* U+0E58 */
  0x0000,   /* U+0E59 */
  0x0000,   /* U+0E5A */
  0x0000,   /* U+0E5B */
  0x8E5C,   /* U+0E5C */
  0x8E5D,   /* U+0E5D */
  0x8E5E,   /* U+0E5E */
  0x8E5F,   /* U+0E5F */
  0x8E60,   /* U+0E60 */
  0x8E61,   /* U+0E61 */
  0x8E62,   /* U+0E62 */
  0x8E63,   /* U+0E63 */
  0x8E64,   /* U+0E64 */
  0x8E65,   /* U+0E65 */
  0x8E66,   /* U+0E66 */
  0x8E67,   /* U+0E67 */
  0x8E68,   /* U+0E68 */
  0x8E69,   /* U+0E69 */
  0x8E6A,   /* U+0E6A */
  0x8E6B,   /* U+0E6B */
  0x8E6C,   /* U+0E6C */
  0x8E6D,   /* U+0E6D */
  0x8E6E,   /* U+0E6E */
  0x8E6F,   /* U+0E6F */
  0x8E70,   /* U+0E70 */
  0x8E71,   /* U+0E71 */
  0x8E72,   /* U+0E72 */
  0x8E73,   /* U+0E73 */
  0x8E74,   /* U+0E74 */
  0x8E75,   /* U+0E75 */
  0x8E76,   /* U+0E76 */
  0x8E77,   /* U+0E77 */
  0x8E78,   /* U+0E78 */
  0x8E79,   /* U+0E79 */
  0x8E7A,   /* U+0E7A */
  0x8E7B,   /* U+0E7B */
  0x8E7C,   /* U+0E7C */
  0x8E7D,   /* U+0E7D */
  0x8E7E,   /* U+0E7E */
  0x8E7F,   /* U+0E7F */
  0x8E80,   /* U+0E80 */
  0x0000,   /* U+0E81 */
  0x0000,   /* U+0E82 */
  0x8E83,   /* U+0E83 */
  0x0000,   /* U+0E84 */
  0x8E85,   /* U+0E85 */
  0x8E86,   /* U+0E86 */
  0x0000,   /* U+0E87 */
  0x0000,   /* U+0E88 */
  0x8E89,   /* U+0E89 */
  0x0000,   /* U+0E8A */
  0x8E8B,   /* U+0E8B */
  0x8E8C,   /* U+0E8C */
  0x0000,   /* U+0E8D */
  0x8E8E,   /* U+0E8E */
  0x8E8F,   /* U+0E8F */
  0x8E90,   /* U+0E90 */
  0x8E91,   /* U+0E91 */
  0x8E92,   /* U+0E92 */
  0x8E93,   /* U+0E93 */
  0x0000,   /* U+0E94 */
  0x0000,   /* U+0E95 */
  0x0000,   /* U+0E96 */
  0x0000,   /* U+0E97 */
  0x8E98,   /* U+0E98 */
  0x0000,   /* U+0E99 */
  0x0000,   /* U+0E9A */
  0x0000,   /* U+0E9B */
  0x0000,   /* U+0E9C */
  0x0000,   /* U+0E9D */
  0x0000,   /* U+0E9E */
  0x0000,   /* U+0E9F */
  0x8EA0,   /* U+0EA0 */
  0x0000,   /* U+0EA1 */
  0x0000,   /* U+0EA2 */
  0x0000,   /* U+0EA3 */
  0x8EA4,   /* U+0EA4 */
  0x0000,   /* U+0EA5 */
  0x8EA6,   /* U+0EA6 */
  0x0000,   /* U+0EA7 */
  0x8EA8,   /* U+0EA8 */
  0x8EA9,   /* U+0EA9 */
  0x0000,   /* U+0EAA */
  0x0000,   /* U+0EAB */
  0x8EAC,   /* U+0EAC */
  0x0000,   /* U+0EAD */
  0x0000,   /* U+0EAE */
  0x0000,   /* U+0EAF */
  0x0000,   /* U+0EB0 */
  0x0000,   /* U+0EB1 */
  0x0000,   /* U+0EB2 */
  0x0000,   /* U+0EB3 */
  0x0000,   /* U+0EB4 */
  0x0000,   /* U+0EB5 */
  0x0000,   /* U+0EB6 */
  0x0000,   /* U+0EB7 */
  0x0000,   /* U+0EB8 */
  0x0000,   /* U+0EB9 */
  0x8EBA,   /* U+0EBA */
  0x0000,   /* U+0EBB */
  0x0000,   /* U+0EBC */
  0x0000,   /* U+0EBD */
  0x8EBE,   /* U+0EBE */
  0x8EBF,   /* U+0EBF */
  0x0000,   /* U+0EC0 */
  0x0000,   /* U+0EC1 */
  0x0000,   /* U+0EC2 */
  0x0000,   /* U+0EC3 */
  0x0000,   /* U+0EC4 */
  0x8EC5,   /* U+0EC5 */
  0x0000,   /* U+0EC6 */
  0x8EC7,   /* U+0EC7 */
  0x0000,   /* U+0EC8 */
  0x0000,   /* U+0EC9 */
  0x0000,   /* U+0ECA */
  0x0000,   /* U+0ECB */
  0x0000,   /* U+0ECC */
  0x0000,   /* U+0ECD */
  0x8ECE,   /* U+0ECE */
  0x8ECF,   /* U+0ECF */
  0x0000,   /* U+0ED0 */
  0x0000,   /* U+0ED1 */
  0x0000,   /* U+0ED2 */
  0x0000,   /* U+0ED3 */
  0x0000,   /* U+0ED4 */
  0x0000,   /* U+0ED5 */
  0x0000,   /* U+0ED6 */
  0x0000,   /* U+0ED7 */
  0x0000,   /* U+0ED8 */
  0x0000,   /* U+0ED9 */
  0x8EDA,   /* U+0EDA */
  0x8EDB,   /* U+0EDB */
  0x2DC1,   /* U+0EDC */
  0x2DC8,   /* U+0EDD */
  0x0000,   /* U+0EDE */
  0x0000,   /* U+0EDF */
  0x8EE0,   /* U+0EE0 */
  0x8EE1,   /* U+0EE1 */
  0x8EE2,   /* U+0EE2 */
  0x8EE3,   /* U+0EE3 */
  0x8EE4,   /* U+0EE4 */
  0x8EE5,   /* U+0EE5 */
  0x8EE6,   /* U+0EE6 */
  0x8EE7,   /* U+0EE7 */
  0x8EE8,   /* U+0EE8 */
  0x8EE9,   /* U+0EE9 */
  0x8EEA,   /* U+0EEA */
  0x8EEB,   /* U+0EEB */
  0x8EEC,   /* U+0EEC */
  0x8EED,   /* U+0EED */
  0x8EEE,   /* U+0EEE */
  0x8EEF,   /* U+0EEF */
  0x8EF0,   /* U+0EF0 */
  0x8EF1,   /* U+0EF1 */
  0x8EF2,   /* U+0EF2 */
  0x8EF3,   /* U+0EF3 */
  0x8EF4,   /* U+0EF4 */
  0x8EF5,   /* U+0EF5 */
  0x8EF6,   /* U+0EF6 */
  0x8EF7,   /* U+0EF7 */
  0x8EF8,   /* U+0EF8 */
  0x8EF9,   /* U+0EF9 */
  0x8EFA,   /* U+0EFA */
  0x8EFB,   /* U+0EFB */
  0x8EFC,   /* U+0EFC */
  0x8EFD,   /* U+0EFD */
  0x8EFE,   /* U+0EFE */
  0x8EFF,   /* U+0EFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+0E00 */
  0x0000,   /* U+0E01 */
  0x0000,   /* U+0E02 */
  0x0000,   /* U+0E03 */
  0x0000,   /* U+0E04 */
  0x0000,   /* U+0E05 */
  0x0000,   /* U+0E06 */
  0x0000,   /* U+0E07 */
  0x0000,   /* U+0E08 */
  0x0000,   /* U+0E09 */
  0x0000,   /* U+0E0A */
  0x0000,   /* U+0E0B */
  0x0000,   /* U+0E0C */
  0x0000,   /* U+0E0D */
  0x0000,   /* U+0E0E */
  0x0000,   /* U+0E0F */
  0x0000,   /* U+0E10 */
  0x0000,   /* U+0E11 */
  0x0000,   /* U+0E12 */
  0x0000,   /* U+0E13 */
  0x0000,   /* U+0E14 */
  0x0000,   /* U+0E15 */
  0x0000,   /* U+0E16 */
  0x0000,   /* U+0E17 */
  0x0000,   /* U+0E18 */
  0x0000,   /* U+0E19 */
  0x0000,   /* U+0E1A */
  0x0000,   /* U+0E1B */
  0x0000,   /* U+0E1C */
  0x0000,   /* U+0E1D */
  0x0000,   /* U+0E1E */
  0x0000,   /* U+0E1F */
  0x0000,   /* U+0E20 */
  0x0000,   /* U+0E21 */
  0x0000,   /* U+0E22 */
  0x0000,   /* U+0E23 */
  0x0000,   /* U+0E24 */
  0x0000,   /* U+0E25 */
  0x0000,   /* U+0E26 */
  0x0000,   /* U+0E27 */
  0x0000,   /* U+0E28 */
  0x0000,   /* U+0E29 */
  0x0000,   /* U+0E2A */
  0x0000,   /* U+0E2B */
  0x0000,   /* U+0E2C */
  0x0000,   /* U+0E2D */
  0x0000,   /* U+0E2E */
  0x0000,   /* U+0E2F */
  0x0000,   /* U+0E30 */
  0x0000,   /* U+0E31 */
  0x0000,   /* U+0E32 */
  0x0000,   /* U+0E33 */
  0x0000,   /* U+0E34 */
  0x0000,   /* U+0E35 */
  0x0000,   /* U+0E36 */
  0x0000,   /* U+0E37 */
  0x0000,   /* U+0E38 */
  0x0000,   /* U+0E39 */
  0x0000,   /* U+0E3A */
  0x0000,   /* U+0E3B */
  0x0000,   /* U+0E3C */
  0x0000,   /* U+0E3D */
  0x0000,   /* U+0E3E */
  0x0000,   /* U+0E3F */
  0x0000,   /* U+0E40 */
  0x0000,   /* U+0E41 */
  0x0000,   /* U+0E42 */
  0x0000,   /* U+0E43 */
  0x0000,   /* U+0E44 */
  0x0000,   /* U+0E45 */
  0x0000,   /* U+0E46 */
  0x0000,   /* U+0E47 */
  0x0000,   /* U+0E48 */
  0x0000,   /* U+0E49 */
  0x0000,   /* U+0E4A */
  0x0000,   /* U+0E4B */
  0x0000,   /* U+0E4C */
  0x0000,   /* U+0E4D */
  0x0000,   /* U+0E4E */
  0x0000,   /* U+0E4F */
  0x0000,   /* U+0E50 */
  0x0000,   /* U+0E51 */
  0x0000,   /* U+0E52 */
  0x0000,   /* U+0E53 */
  0x0000,   /* U+0E54 */
  0x0000,   /* U+0E55 */
  0x0000,   /* U+0E56 */
  0x0000,   /* U+0E57 */
  0x0000,   /* U+0E58 */
  0x0000,   /* U+0E59 */
  0x0000,   /* U+0E5A */
  0x0000,   /* U+0E5B */
  0x0000,   /* U+0E5C */
  0x0000,   /* U+0E5D */
  0x0000,   /* U+0E5E */
  0x0000,   /* U+0E5F */
  0x0000,   /* U+0E60 */
  0x0000,   /* U+0E61 */
  0x0000,   /* U+0E62 */
  0x0000,   /* U+0E63 */
  0x0000,   /* U+0E64 */
  0x0000,   /* U+0E65 */
  0x0000,   /* U+0E66 */
  0x0000,   /* U+0E67 */
  0x0000,   /* U+0E68 */
  0x0000,   /* U+0E69 */
  0x0000,   /* U+0E6A */
  0x0000,   /* U+0E6B */
  0x0000,   /* U+0E6C */
  0x0000,   /* U+0E6D */
  0x0000,   /* U+0E6E */
  0x0000,   /* U+0E6F */
  0x0000,   /* U+0E70 */
  0x0000,   /* U+0E71 */
  0x0000,   /* U+0E72 */
  0x0000,   /* U+0E73 */
  0x0000,   /* U+0E74 */
  0x0000,   /* U+0E75 */
  0x0000,   /* U+0E76 */
  0x0000,   /* U+0E77 */
  0x0000,   /* U+0E78 */
  0x0000,   /* U+0E79 */
  0x0000,   /* U+0E7A */
  0x0000,   /* U+0E7B */
  0x0000,   /* U+0E7C */
  0x0000,   /* U+0E7D */
  0x0000,   /* U+0E7E */
  0x0000,   /* U+0E7F */
  0x0000,   /* U+0E80 */
  0x0000,   /* U+0E81 */
  0x0000,   /* U+0E82 */
  0x0000,   /* U+0E83 */
  0x0000,   /* U+0E84 */
  0x0000,   /* U+0E85 */
  0x0000,   /* U+0E86 */
  0x0000,   /* U+0E87 */
  0x0000,   /* U+0E88 */
  0x0000,   /* U+0E89 */
  0x0000,   /* U+0E8A */
  0x0000,   /* U+0E8B */
  0x0000,   /* U+0E8C */
  0x0000,   /* U+0E8D */
  0x0000,   /* U+0E8E */
  0x0000,   /* U+0E8F */
  0x0000,   /* U+0E90 */
  0x0000,   /* U+0E91 */
  0x0000,   /* U+0E92 */
  0x0000,   /* U+0E93 */
  0x0000,   /* U+0E94 */
  0x0000,   /* U+0E95 */
  0x0000,   /* U+0E96 */
  0x0000,   /* U+0E97 */
  0x0000,   /* U+0E98 */
  0x0000,   /* U+0E99 */
  0x0000,   /* U+0E9A */
  0x0000,   /* U+0E9B */
  0x0000,   /* U+0E9C */
  0x0000,   /* U+0E9D */
  0x0000,   /* U+0E9E */
  0x0000,   /* U+0E9F */
  0x0000,   /* U+0EA0 */
  0x0000,   /* U+0EA1 */
  0x0000,   /* U+0EA2 */
  0x0000,   /* U+0EA3 */
  0x0000,   /* U+0EA4 */
  0x0000,   /* U+0EA5 */
  0x0000,   /* U+0EA6 */
  0x0000,   /* U+0EA7 */
  0x0000,   /* U+0EA8 */
  0x0000,   /* U+0EA9 */
  0x0000,   /* U+0EAA */
  0x0000,   /* U+0EAB */
  0x0000,   /* U+0EAC */
  0x0000,   /* U+0EAD */
  0x0000,   /* U+0EAE */
  0x0000,   /* U+0EAF */
  0x0000,   /* U+0EB0 */
  0x0000,   /* U+0EB1 */
  0x0000,   /* U+0EB2 */
  0x0000,   /* U+0EB3 */
  0x0000,   /* U+0EB4 */
  0x0000,   /* U+0EB5 */
  0x0000,   /* U+0EB6 */
  0x0000,   /* U+0EB7 */
  0x0000,   /* U+0EB8 */
  0x0000,   /* U+0EB9 */
  0x0000,   /* U+0EBA */
  0x0000,   /* U+0EBB */
  0x0000,   /* U+0EBC */
  0x0000,   /* U+0EBD */
  0x0000,   /* U+0EBE */
  0x0000,   /* U+0EBF */
  0x0000,   /* U+0EC0 */
  0x0000,   /* U+0EC1 */
  0x0000,   /* U+0EC2 */
  0x0000,   /* U+0EC3 */
  0x0000,   /* U+0EC4 */
  0x0000,   /* U+0EC5 */
  0x0000,   /* U+0EC6 */
  0x0000,   /* U+0EC7 */
  0x0000,   /* U+0EC8 */
  0x0000,   /* U+0EC9 */
  0x0000,   /* U+0ECA */
  0x0000,   /* U+0ECB */
  0x0000,   /* U+0ECC */
  0x0000,   /* U+0ECD */
  0x0000,   /* U+0ECE */
  0x0000,   /* U+0ECF */
  0x0000,   /* U+0ED0 */
  0x0000,   /* U+0ED1 */
  0x0000,   /* U+0ED2 */
  0x0000,   /* U+0ED3 */
  0x0000,   /* U+0ED4 */
  0x0000,   /* U+0ED5 */
  0x0000,   /* U+0ED6 */
  0x0000,   /* U+0ED7 */
  0x0000,   /* U+0ED8 */
  0x0000,   /* U+0ED9 */
  0x0000,   /* U+0EDA */
  0x0000,   /* U+0EDB */
  0x0020,   /* U+0EDC */
  0x0020,   /* U+0EDD */
  0x0000,   /* U+0EDE */
  0x0000,   /* U+0EDF */
  0x0000,   /* U+0EE0 */
  0x0000,   /* U+0EE1 */
  0x0000,   /* U+0EE2 */
  0x0000,   /* U+0EE3 */
  0x0000,   /* U+0EE4 */
  0x0000,   /* U+0EE5 */
  0x0000,   /* U+0EE6 */
  0x0000,   /* U+0EE7 */
  0x0000,   /* U+0EE8 */
  0x0000,   /* U+0EE9 */
  0x0000,   /* U+0EEA */
  0x0000,   /* U+0EEB */
  0x0000,   /* U+0EEC */
  0x0000,   /* U+0EED */
  0x0000,   /* U+0EEE */
  0x0000,   /* U+0EEF */
  0x0000,   /* U+0EF0 */
  0x0000,   /* U+0EF1 */
  0x0000,   /* U+0EF2 */
  0x0000,   /* U+0EF3 */
  0x0000,   /* U+0EF4 */
  0x0000,   /* U+0EF5 */
  0x0000,   /* U+0EF6 */
  0x0000,   /* U+0EF7 */
  0x0000,   /* U+0EF8 */
  0x0000,   /* U+0EF9 */
  0x0000,   /* U+0EFA */
  0x0000,   /* U+0EFB */
  0x0000,   /* U+0EFC */
  0x0000,   /* U+0EFD */
  0x0000,   /* U+0EFE */
  0x0000,   /* U+0EFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+0E00 */
  0x0000,   /* U+0E01 */
  0x0000,   /* U+0E02 */
  0x0000,   /* U+0E03 */
  0x0000,   /* U+0E04 */
  0x0000,   /* U+0E05 */
  0x0000,   /* U+0E06 */
  0x0000,   /* U+0E07 */
  0x0000,   /* U+0E08 */
  0x0000,   /* U+0E09 */
  0x0000,   /* U+0E0A */
  0x0000,   /* U+0E0B */
  0x0000,   /* U+0E0C */
  0x0000,   /* U+0E0D */
  0x0000,   /* U+0E0E */
  0x0000,   /* U+0E0F */
  0x0000,   /* U+0E10 */
  0x0000,   /* U+0E11 */
  0x0000,   /* U+0E12 */
  0x0000,   /* U+0E13 */
  0x0000,   /* U+0E14 */
  0x0000,   /* U+0E15 */
  0x0000,   /* U+0E16 */
  0x0000,   /* U+0E17 */
  0x0000,   /* U+0E18 */
  0x0000,   /* U+0E19 */
  0x0000,   /* U+0E1A */
  0x0000,   /* U+0E1B */
  0x0000,   /* U+0E1C */
  0x0000,   /* U+0E1D */
  0x0000,   /* U+0E1E */
  0x0000,   /* U+0E1F */
  0x0000,   /* U+0E20 */
  0x0000,   /* U+0E21 */
  0x0000,   /* U+0E22 */
  0x0000,   /* U+0E23 */
  0x0000,   /* U+0E24 */
  0x0000,   /* U+0E25 */
  0x0000,   /* U+0E26 */
  0x0000,   /* U+0E27 */
  0x0000,   /* U+0E28 */
  0x0000,   /* U+0E29 */
  0x0000,   /* U+0E2A */
  0x0000,   /* U+0E2B */
  0x0000,   /* U+0E2C */
  0x0000,   /* U+0E2D */
  0x0000,   /* U+0E2E */
  0x0000,   /* U+0E2F */
  0x0000,   /* U+0E30 */
  0x0000,   /* U+0E31 */
  0x0000,   /* U+0E32 */
  0x0000,   /* U+0E33 */
  0x0000,   /* U+0E34 */
  0x0000,   /* U+0E35 */
  0x0000,   /* U+0E36 */
  0x0000,   /* U+0E37 */
  0x0000,   /* U+0E38 */
  0x0000,   /* U+0E39 */
  0x0000,   /* U+0E3A */
  0x0000,   /* U+0E3B */
  0x0000,   /* U+0E3C */
  0x0000,   /* U+0E3D */
  0x0000,   /* U+0E3E */
  0x0000,   /* U+0E3F */
  0x0000,   /* U+0E40 */
  0x0000,   /* U+0E41 */
  0x0000,   /* U+0E42 */
  0x0000,   /* U+0E43 */
  0x0000,   /* U+0E44 */
  0x0000,   /* U+0E45 */
  0x0000,   /* U+0E46 */
  0x0000,   /* U+0E47 */
  0x0000,   /* U+0E48 */
  0x0000,   /* U+0E49 */
  0x0000,   /* U+0E4A */
  0x0000,   /* U+0E4B */
  0x0000,   /* U+0E4C */
  0x0000,   /* U+0E4D */
  0x0000,   /* U+0E4E */
  0x0000,   /* U+0E4F */
  0x0000,   /* U+0E50 */
  0x0000,   /* U+0E51 */
  0x0000,   /* U+0E52 */
  0x0000,   /* U+0E53 */
  0x0000,   /* U+0E54 */
  0x0000,   /* U+0E55 */
  0x0000,   /* U+0E56 */
  0x0000,   /* U+0E57 */
  0x0000,   /* U+0E58 */
  0x0000,   /* U+0E59 */
  0x0000,   /* U+0E5A */
  0x0000,   /* U+0E5B */
  0x0000,   /* U+0E5C */
  0x0000,   /* U+0E5D */
  0x0000,   /* U+0E5E */
  0x0000,   /* U+0E5F */
  0x0000,   /* U+0E60 */
  0x0000,   /* U+0E61 */
  0x0000,   /* U+0E62 */
  0x0000,   /* U+0E63 */
  0x0000,   /* U+0E64 */
  0x0000,   /* U+0E65 */
  0x0000,   /* U+0E66 */
  0x0000,   /* U+0E67 */
  0x0000,   /* U+0E68 */
  0x0000,   /* U+0E69 */
  0x0000,   /* U+0E6A */
  0x0000,   /* U+0E6B */
  0x0000,   /* U+0E6C */
  0x0000,   /* U+0E6D */
  0x0000,   /* U+0E6E */
  0x0000,   /* U+0E6F */
  0x0000,   /* U+0E70 */
  0x0000,   /* U+0E71 */
  0x0000,   /* U+0E72 */
  0x0000,   /* U+0E73 */
  0x0000,   /* U+0E74 */
  0x0000,   /* U+0E75 */
  0x0000,   /* U+0E76 */
  0x0000,   /* U+0E77 */
  0x0000,   /* U+0E78 */
  0x0000,   /* U+0E79 */
  0x0000,   /* U+0E7A */
  0x0000,   /* U+0E7B */
  0x0000,   /* U+0E7C */
  0x0000,   /* U+0E7D */
  0x0000,   /* U+0E7E */
  0x0000,   /* U+0E7F */
  0x0000,   /* U+0E80 */
  0x0000,   /* U+0E81 */
  0x0000,   /* U+0E82 */
  0x0000,   /* U+0E83 */
  0x0000,   /* U+0E84 */
  0x0000,   /* U+0E85 */
  0x0000,   /* U+0E86 */
  0x0000,   /* U+0E87 */
  0x0000,   /* U+0E88 */
  0x0000,   /* U+0E89 */
  0x0000,   /* U+0E8A */
  0x0000,   /* U+0E8B */
  0x0000,   /* U+0E8C */
  0x0000,   /* U+0E8D */
  0x0000,   /* U+0E8E */
  0x0000,   /* U+0E8F */
  0x0000,   /* U+0E90 */
  0x0000,   /* U+0E91 */
  0x0000,   /* U+0E92 */
  0x0000,   /* U+0E93 */
  0x0000,   /* U+0E94 */
  0x0000,   /* U+0E95 */
  0x0000,   /* U+0E96 */
  0x0000,   /* U+0E97 */
  0x0000,   /* U+0E98 */
  0x0000,   /* U+0E99 */
  0x0000,   /* U+0E9A */
  0x0000,   /* U+0E9B */
  0x0000,   /* U+0E9C */
  0x0000,   /* U+0E9D */
  0x0000,   /* U+0E9E */
  0x0000,   /* U+0E9F */
  0x0000,   /* U+0EA0 */
  0x0000,   /* U+0EA1 */
  0x0000,   /* U+0EA2 */
  0x0000,   /* U+0EA3 */
  0x0000,   /* U+0EA4 */
  0x0000,   /* U+0EA5 */
  0x0000,   /* U+0EA6 */
  0x0000,   /* U+0EA7 */
  0x0000,   /* U+0EA8 */
  0x0000,   /* U+0EA9 */
  0x0000,   /* U+0EAA */
  0x0000,   /* U+0EAB */
  0x0000,   /* U+0EAC */
  0x0000,   /* U+0EAD */
  0x0000,   /* U+0EAE */
  0x0000,   /* U+0EAF */
  0x0000,   /* U+0EB0 */
  0x0000,   /* U+0EB1 */
  0x0000,   /* U+0EB2 */
  0x0000,   /* U+0EB3 */
  0x0000,   /* U+0EB4 */
  0x0000,   /* U+0EB5 */
  0x0000,   /* U+0EB6 */
  0x0000,   /* U+0EB7 */
  0x0000,   /* U+0EB8 */
  0x0000,   /* U+0EB9 */
  0x0000,   /* U+0EBA */
  0x0000,   /* U+0EBB */
  0x0000,   /* U+0EBC */
  0x0000,   /* U+0EBD */
  0x0000,   /* U+0EBE */
  0x0000,   /* U+0EBF */
  0x0000,   /* U+0EC0 */
  0x0000,   /* U+0EC1 */
  0x0000,   /* U+0EC2 */
  0x0000,   /* U+0EC3 */
  0x0000,   /* U+0EC4 */
  0x0000,   /* U+0EC5 */
  0x0000,   /* U+0EC6 */
  0x0000,   /* U+0EC7 */
  0x0000,   /* U+0EC8 */
  0x0000,   /* U+0EC9 */
  0x0000,   /* U+0ECA */
  0x0000,   /* U+0ECB */
  0x0000,   /* U+0ECC */
  0x0000,   /* U+0ECD */
  0x0000,   /* U+0ECE */
  0x0000,   /* U+0ECF */
  0x0000,   /* U+0ED0 */
  0x0000,   /* U+0ED1 */
  0x0000,   /* U+0ED2 */
  0x0000,   /* U+0ED3 */
  0x0000,   /* U+0ED4 */
  0x0000,   /* U+0ED5 */
  0x0000,   /* U+0ED6 */
  0x0000,   /* U+0ED7 */
  0x0000,   /* U+0ED8 */
  0x0000,   /* U+0ED9 */
  0x0000,   /* U+0EDA */
  0x0000,   /* U+0EDB */
  0x0004,   /* U+0EDC */
  0x0004,   /* U+0EDD */
  0x0000,   /* U+0EDE */
  0x0000,   /* U+0EDF */
  0x0000,   /* U+0EE0 */
  0x0000,   /* U+0EE1 */
  0x0000,   /* U+0EE2 */
  0x0000,   /* U+0EE3 */
  0x0000,   /* U+0EE4 */
  0x0000,   /* U+0EE5 */
  0x0000,   /* U+0EE6 */
  0x0000,   /* U+0EE7 */
  0x0000,   /* U+0EE8 */
  0x0000,   /* U+0EE9 */
  0x0000,   /* U+0EEA */
  0x0000,   /* U+0EEB */
  0x0000,   /* U+0EEC */
  0x0000,   /* U+0EED */
  0x0000,   /* U+0EEE */
  0x0000,   /* U+0EEF */
  0x0000,   /* U+0EF0 */
  0x0000,   /* U+0EF1 */
  0x0000,   /* U+0EF2 */
  0x0000,   /* U+0EF3 */
  0x0000,   /* U+0EF4 */
  0x0000,   /* U+0EF5 */
  0x0000,   /* U+0EF6 */
  0x0000,   /* U+0EF7 */
  0x0000,   /* U+0EF8 */
  0x0000,   /* U+0EF9 */
  0x0000,   /* U+0EFA */
  0x0000,   /* U+0EFB */
  0x0000,   /* U+0EFC */
  0x0000,   /* U+0EFD */
  0x0000,   /* U+0EFE */
  0x0000,   /* U+0EFF */
};

uint16 uca900_p00F[]= {
  /* Number of CEs for each character. */
  3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 2, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2E6C,   /* U+0F00 */
  0x0526,   /* U+0F01 */
  0x0527,   /* U+0F02 */
  0x0528,   /* U+0F03 */
  0x03F3,   /* U+0F04 */
  0x03F4,   /* U+0F05 */
  0x03F5,   /* U+0F06 */
  0x03F6,   /* U+0F07 */
  0x03F7,   /* U+0F08 */
  0x03F8,   /* U+0F09 */
  0x03F9,   /* U+0F0A */
  0x03FC,   /* U+0F0B */
  0x03FC,   /* U+0F0C */
  0x03FD,   /* U+0F0D */
  0x03FE,   /* U+0F0E */
  0x03FF,   /* U+0F0F */
  0x0400,   /* U+0F10 */
  0x0401,   /* U+0F11 */
  0x0402,   /* U+0F12 */
  0x0529,   /* U+0F13 */
  0x0258,   /* U+0F14 */
  0x052A,   /* U+0F15 */
  0x052B,   /* U+0F16 */
  0x052C,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x052D,   /* U+0F1A */
  0x052E,   /* U+0F1B */
  0x052F,   /* U+0F1C */
  0x0530,   /* U+0F1D */
  0x0531,   /* U+0F1E */
  0x0532,   /* U+0F1F */
  0x1C3D,   /* U+0F20 */
  0x1C3E,   /* U+0F21 */
  0x1C3F,   /* U+0F22 */
  0x1C40,   /* U+0F23 */
  0x1C41,   /* U+0F24 */
  0x1C42,   /* U+0F25 */
  0x1C43,   /* U+0F26 */
  0x1C44,   /* U+0F27 */
  0x1C45,   /* U+0F28 */
  0x1C46,   /* U+0F29 */
  0x1C3E,   /* U+0F2A */
  0x1C3F,   /* U+0F2B */
  0x1C40,   /* U+0F2C */
  0x1C41,   /* U+0F2D */
  0x1C42,   /* U+0F2E */
  0x1C43,   /* U+0F2F */
  0x1C44,   /* U+0F30 */
  0x1C45,   /* U+0F31 */
  0x1C46,   /* U+0F32 */
  0x1C3D,   /* U+0F33 */
  0x0533,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0534,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0535,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x031D,   /* U+0F3A */
  0x031E,   /* U+0F3B */
  0x031F,   /* U+0F3C */
  0x0320,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x2E26,   /* U+0F40 */
  0x2E29,   /* U+0F41 */
  0x2E2B,   /* U+0F42 */
  0x2E2B,   /* U+0F43 */
  0x2E2D,   /* U+0F44 */
  0x2E2F,   /* U+0F45 */
  0x2E31,   /* U+0F46 */
  0x2E33,   /* U+0F47 */
  0xFBC0,   /* U+0F48 */
  0x2E35,   /* U+0F49 */
  0x2E37,   /* U+0F4A */
  0x2E39,   /* U+0F4B */
  0x2E3B,   /* U+0F4C */
  0x2E3B,   /* U+0F4D */
  0x2E3D,   /* U+0F4E */
  0x2E3F,   /* U+0F4F */
  0x2E41,   /* U+0F50 */
  0x2E43,   /* U+0F51 */
  0x2E43,   /* U+0F52 */
  0x2E45,   /* U+0F53 */
  0x2E47,   /* U+0F54 */
  0x2E49,   /* U+0F55 */
  0x2E4B,   /* U+0F56 */
  0x2E4B,   /* U+0F57 */
  0x2E4D,   /* U+0F58 */
  0x2E4F,   /* U+0F59 */
  0x2E51,   /* U+0F5A */
  0x2E53,   /* U+0F5B */
  0x2E53,   /* U+0F5C */
  0x2E55,   /* U+0F5D */
  0x2E57,   /* U+0F5E */
  0x2E59,   /* U+0F5F */
  0x2E5B,   /* U+0F60 */
  0x2E5D,   /* U+0F61 */
  0x2E5F,   /* U+0F62 */
  0x2E62,   /* U+0F63 */
  0x2E64,   /* U+0F64 */
  0x2E66,   /* U+0F65 */
  0x2E68,   /* U+0F66 */
  0x2E6A,   /* U+0F67 */
  0x2E6C,   /* U+0F68 */
  0x2E26,   /* U+0F69 */
  0x2E5F,   /* U+0F6A */
  0x2E28,   /* U+0F6B */
  0x2E61,   /* U+0F6C */
  0xFBC0,   /* U+0F6D */
  0xFBC0,   /* U+0F6E */
  0xFBC0,   /* U+0F6F */
  0xFBC0,   /* U+0F70 */
  0x2E76,   /* U+0F71 */
  0x2E77,   /* U+0F72 */
  0x2E78,   /* U+0F73 */
  0x2E7B,   /* U+0F74 */
  0x2E7C,   /* U+0F75 */
  0x2E7D,   /* U+0F76 */
  0x2E7E,   /* U+0F77 */
  0x2E7F,   /* U+0F78 */
  0x2E80,   /* U+0F79 */
  0x2E81,   /* U+0F7A */
  0x2E82,   /* U+0F7B */
  0x2E83,   /* U+0F7C */
  0x2E84,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x2E79,   /* U+0F80 */
  0x2E7A,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x2E85,   /* U+0F84 */
  0x0403,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x2E6E,   /* U+0F88 */
  0x2E70,   /* U+0F89 */
  0x2E74,   /* U+0F8A */
  0x2E75,   /* U+0F8B */
  0x2E72,   /* U+0F8C */
  0x2E6F,   /* U+0F8D */
  0x2E71,   /* U+0F8E */
  0x2E73,   /* U+0F8F */
  0x2E27,   /* U+0F90 */
  0x2E2A,   /* U+0F91 */
  0x2E2C,   /* U+0F92 */
  0x2E2C,   /* U+0F93 */
  0x2E2E,   /* U+0F94 */
  0x2E30,   /* U+0F95 */
  0x2E32,   /* U+0F96 */
  0x2E34,   /* U+0F97 */
  0xFBC0,   /* U+0F98 */
  0x2E36,   /* U+0F99 */
  0x2E38,   /* U+0F9A */
  0x2E3A,   /* U+0F9B */
  0x2E3C,   /* U+0F9C */
  0x2E3C,   /* U+0F9D */
  0x2E3E,   /* U+0F9E */
  0x2E40,   /* U+0F9F */
  0x2E42,   /* U+0FA0 */
  0x2E44,   /* U+0FA1 */
  0x2E44,   /* U+0FA2 */
  0x2E46,   /* U+0FA3 */
  0x2E48,   /* U+0FA4 */
  0x2E4A,   /* U+0FA5 */
  0x2E4C,   /* U+0FA6 */
  0x2E4C,   /* U+0FA7 */
  0x2E4E,   /* U+0FA8 */
  0x2E50,   /* U+0FA9 */
  0x2E52,   /* U+0FAA */
  0x2E54,   /* U+0FAB */
  0x2E54,   /* U+0FAC */
  0x2E56,   /* U+0FAD */
  0x2E58,   /* U+0FAE */
  0x2E5A,   /* U+0FAF */
  0x2E5C,   /* U+0FB0 */
  0x2E5E,   /* U+0FB1 */
  0x2E60,   /* U+0FB2 */
  0x2E63,   /* U+0FB3 */
  0x2E65,   /* U+0FB4 */
  0x2E67,   /* U+0FB5 */
  0x2E69,   /* U+0FB6 */
  0x2E6B,   /* U+0FB7 */
  0x2E6D,   /* U+0FB8 */
  0x2E27,   /* U+0FB9 */
  0x2E56,   /* U+0FBA */
  0x2E5E,   /* U+0FBB */
  0x2E60,   /* U+0FBC */
  0xFBC0,   /* U+0FBD */
  0x0536,   /* U+0FBE */
  0x0537,   /* U+0FBF */
  0x0538,   /* U+0FC0 */
  0x0539,   /* U+0FC1 */
  0x053A,   /* U+0FC2 */
  0x053B,   /* U+0FC3 */
  0x053C,   /* U+0FC4 */
  0x053D,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x053E,   /* U+0FC7 */
  0x053F,   /* U+0FC8 */
  0x0540,   /* U+0FC9 */
  0x0541,   /* U+0FCA */
  0x0542,   /* U+0FCB */
  0x0543,   /* U+0FCC */
  0xFBC0,   /* U+0FCD */
  0x0544,   /* U+0FCE */
  0x0545,   /* U+0FCF */
  0x03FA,   /* U+0FD0 */
  0x03FB,   /* U+0FD1 */
  0x0404,   /* U+0FD2 */
  0x0405,   /* U+0FD3 */
  0x0406,   /* U+0FD4 */
  0x0546,   /* U+0FD5 */
  0x0547,   /* U+0FD6 */
  0x0548,   /* U+0FD7 */
  0x0549,   /* U+0FD8 */
  0x0407,   /* U+0FD9 */
  0x0408,   /* U+0FDA */
  0xFBC0,   /* U+0FDB */
  0xFBC0,   /* U+0FDC */
  0xFBC0,   /* U+0FDD */
  0xFBC0,   /* U+0FDE */
  0xFBC0,   /* U+0FDF */
  0xFBC0,   /* U+0FE0 */
  0xFBC0,   /* U+0FE1 */
  0xFBC0,   /* U+0FE2 */
  0xFBC0,   /* U+0FE3 */
  0xFBC0,   /* U+0FE4 */
  0xFBC0,   /* U+0FE5 */
  0xFBC0,   /* U+0FE6 */
  0xFBC0,   /* U+0FE7 */
  0xFBC0,   /* U+0FE8 */
  0xFBC0,   /* U+0FE9 */
  0xFBC0,   /* U+0FEA */
  0xFBC0,   /* U+0FEB */
  0xFBC0,   /* U+0FEC */
  0xFBC0,   /* U+0FED */
  0xFBC0,   /* U+0FEE */
  0xFBC0,   /* U+0FEF */
  0xFBC0,   /* U+0FF0 */
  0xFBC0,   /* U+0FF1 */
  0xFBC0,   /* U+0FF2 */
  0xFBC0,   /* U+0FF3 */
  0xFBC0,   /* U+0FF4 */
  0xFBC0,   /* U+0FF5 */
  0xFBC0,   /* U+0FF6 */
  0xFBC0,   /* U+0FF7 */
  0xFBC0,   /* U+0FF8 */
  0xFBC0,   /* U+0FF9 */
  0xFBC0,   /* U+0FFA */
  0xFBC0,   /* U+0FFB */
  0xFBC0,   /* U+0FFC */
  0xFBC0,   /* U+0FFD */
  0xFBC0,   /* U+0FFE */
  0xFBC0,   /* U+0FFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+0F00 */
  0x0020,   /* U+0F01 */
  0x0020,   /* U+0F02 */
  0x0020,   /* U+0F03 */
  0x0020,   /* U+0F04 */
  0x0020,   /* U+0F05 */
  0x0020,   /* U+0F06 */
  0x0020,   /* U+0F07 */
  0x0020,   /* U+0F08 */
  0x0020,   /* U+0F09 */
  0x0020,   /* U+0F0A */
  0x0020,   /* U+0F0B */
  0x0020,   /* U+0F0C */
  0x0020,   /* U+0F0D */
  0x0020,   /* U+0F0E */
  0x0020,   /* U+0F0F */
  0x0020,   /* U+0F10 */
  0x0020,   /* U+0F11 */
  0x0020,   /* U+0F12 */
  0x0020,   /* U+0F13 */
  0x0020,   /* U+0F14 */
  0x0020,   /* U+0F15 */
  0x0020,   /* U+0F16 */
  0x0020,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0020,   /* U+0F1A */
  0x0020,   /* U+0F1B */
  0x0020,   /* U+0F1C */
  0x0020,   /* U+0F1D */
  0x0020,   /* U+0F1E */
  0x0020,   /* U+0F1F */
  0x0020,   /* U+0F20 */
  0x0020,   /* U+0F21 */
  0x0020,   /* U+0F22 */
  0x0020,   /* U+0F23 */
  0x0020,   /* U+0F24 */
  0x0020,   /* U+0F25 */
  0x0020,   /* U+0F26 */
  0x0020,   /* U+0F27 */
  0x0020,   /* U+0F28 */
  0x0020,   /* U+0F29 */
  0x0020,   /* U+0F2A */
  0x0020,   /* U+0F2B */
  0x0020,   /* U+0F2C */
  0x0020,   /* U+0F2D */
  0x0020,   /* U+0F2E */
  0x0020,   /* U+0F2F */
  0x0020,   /* U+0F30 */
  0x0020,   /* U+0F31 */
  0x0020,   /* U+0F32 */
  0x0020,   /* U+0F33 */
  0x0020,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0020,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0020,   /* U+0F38 */
  0x00E1,   /* U+0F39 */
  0x0020,   /* U+0F3A */
  0x0020,   /* U+0F3B */
  0x0020,   /* U+0F3C */
  0x0020,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0020,   /* U+0F40 */
  0x0020,   /* U+0F41 */
  0x0020,   /* U+0F42 */
  0x0020,   /* U+0F43 */
  0x0020,   /* U+0F44 */
  0x0020,   /* U+0F45 */
  0x0020,   /* U+0F46 */
  0x0020,   /* U+0F47 */
  0x0020,   /* U+0F48 */
  0x0020,   /* U+0F49 */
  0x0020,   /* U+0F4A */
  0x0020,   /* U+0F4B */
  0x0020,   /* U+0F4C */
  0x0020,   /* U+0F4D */
  0x0020,   /* U+0F4E */
  0x0020,   /* U+0F4F */
  0x0020,   /* U+0F50 */
  0x0020,   /* U+0F51 */
  0x0020,   /* U+0F52 */
  0x0020,   /* U+0F53 */
  0x0020,   /* U+0F54 */
  0x0020,   /* U+0F55 */
  0x0020,   /* U+0F56 */
  0x0020,   /* U+0F57 */
  0x0020,   /* U+0F58 */
  0x0020,   /* U+0F59 */
  0x0020,   /* U+0F5A */
  0x0020,   /* U+0F5B */
  0x0020,   /* U+0F5C */
  0x0020,   /* U+0F5D */
  0x0020,   /* U+0F5E */
  0x0020,   /* U+0F5F */
  0x0020,   /* U+0F60 */
  0x0020,   /* U+0F61 */
  0x0020,   /* U+0F62 */
  0x0020,   /* U+0F63 */
  0x0020,   /* U+0F64 */
  0x0020,   /* U+0F65 */
  0x0020,   /* U+0F66 */
  0x0020,   /* U+0F67 */
  0x0020,   /* U+0F68 */
  0x0020,   /* U+0F69 */
  0x0020,   /* U+0F6A */
  0x0020,   /* U+0F6B */
  0x0020,   /* U+0F6C */
  0x0020,   /* U+0F6D */
  0x0020,   /* U+0F6E */
  0x0020,   /* U+0F6F */
  0x0020,   /* U+0F70 */
  0x0020,   /* U+0F71 */
  0x0020,   /* U+0F72 */
  0x0020,   /* U+0F73 */
  0x0020,   /* U+0F74 */
  0x0020,   /* U+0F75 */
  0x0020,   /* U+0F76 */
  0x0020,   /* U+0F77 */
  0x0020,   /* U+0F78 */
  0x0020,   /* U+0F79 */
  0x0020,   /* U+0F7A */
  0x0020,   /* U+0F7B */
  0x0020,   /* U+0F7C */
  0x0020,   /* U+0F7D */
  0x00C4,   /* U+0F7E */
  0x00C5,   /* U+0F7F */
  0x0020,   /* U+0F80 */
  0x0020,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0020,   /* U+0F84 */
  0x0020,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0020,   /* U+0F88 */
  0x0020,   /* U+0F89 */
  0x0020,   /* U+0F8A */
  0x0020,   /* U+0F8B */
  0x0020,   /* U+0F8C */
  0x0020,   /* U+0F8D */
  0x0020,   /* U+0F8E */
  0x0020,   /* U+0F8F */
  0x0020,   /* U+0F90 */
  0x0020,   /* U+0F91 */
  0x0020,   /* U+0F92 */
  0x0020,   /* U+0F93 */
  0x0020,   /* U+0F94 */
  0x0020,   /* U+0F95 */
  0x0020,   /* U+0F96 */
  0x0020,   /* U+0F97 */
  0x0020,   /* U+0F98 */
  0x0020,   /* U+0F99 */
  0x0020,   /* U+0F9A */
  0x0020,   /* U+0F9B */
  0x0020,   /* U+0F9C */
  0x0020,   /* U+0F9D */
  0x0020,   /* U+0F9E */
  0x0020,   /* U+0F9F */
  0x0020,   /* U+0FA0 */
  0x0020,   /* U+0FA1 */
  0x0020,   /* U+0FA2 */
  0x0020,   /* U+0FA3 */
  0x0020,   /* U+0FA4 */
  0x0020,   /* U+0FA5 */
  0x0020,   /* U+0FA6 */
  0x0020,   /* U+0FA7 */
  0x0020,   /* U+0FA8 */
  0x0020,   /* U+0FA9 */
  0x0020,   /* U+0FAA */
  0x0020,   /* U+0FAB */
  0x0020,   /* U+0FAC */
  0x0020,   /* U+0FAD */
  0x0020,   /* U+0FAE */
  0x0020,   /* U+0FAF */
  0x0020,   /* U+0FB0 */
  0x0020,   /* U+0FB1 */
  0x0020,   /* U+0FB2 */
  0x0020,   /* U+0FB3 */
  0x0020,   /* U+0FB4 */
  0x0020,   /* U+0FB5 */
  0x0020,   /* U+0FB6 */
  0x0020,   /* U+0FB7 */
  0x0020,   /* U+0FB8 */
  0x0020,   /* U+0FB9 */
  0x0020,   /* U+0FBA */
  0x0020,   /* U+0FBB */
  0x0020,   /* U+0FBC */
  0x0020,   /* U+0FBD */
  0x0020,   /* U+0FBE */
  0x0020,   /* U+0FBF */
  0x0020,   /* U+0FC0 */
  0x0020,   /* U+0FC1 */
  0x0020,   /* U+0FC2 */
  0x0020,   /* U+0FC3 */
  0x0020,   /* U+0FC4 */
  0x0020,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0020,   /* U+0FC7 */
  0x0020,   /* U+0FC8 */
  0x0020,   /* U+0FC9 */
  0x0020,   /* U+0FCA */
  0x0020,   /* U+0FCB */
  0x0020,   /* U+0FCC */
  0x0020,   /* U+0FCD */
  0x0020,   /* U+0FCE */
  0x0020,   /* U+0FCF */
  0x0020,   /* U+0FD0 */
  0x0020,   /* U+0FD1 */
  0x0020,   /* U+0FD2 */
  0x0020,   /* U+0FD3 */
  0x0020,   /* U+0FD4 */
  0x0020,   /* U+0FD5 */
  0x0020,   /* U+0FD6 */
  0x0020,   /* U+0FD7 */
  0x0020,   /* U+0FD8 */
  0x0020,   /* U+0FD9 */
  0x0020,   /* U+0FDA */
  0x0020,   /* U+0FDB */
  0x0020,   /* U+0FDC */
  0x0020,   /* U+0FDD */
  0x0020,   /* U+0FDE */
  0x0020,   /* U+0FDF */
  0x0020,   /* U+0FE0 */
  0x0020,   /* U+0FE1 */
  0x0020,   /* U+0FE2 */
  0x0020,   /* U+0FE3 */
  0x0020,   /* U+0FE4 */
  0x0020,   /* U+0FE5 */
  0x0020,   /* U+0FE6 */
  0x0020,   /* U+0FE7 */
  0x0020,   /* U+0FE8 */
  0x0020,   /* U+0FE9 */
  0x0020,   /* U+0FEA */
  0x0020,   /* U+0FEB */
  0x0020,   /* U+0FEC */
  0x0020,   /* U+0FED */
  0x0020,   /* U+0FEE */
  0x0020,   /* U+0FEF */
  0x0020,   /* U+0FF0 */
  0x0020,   /* U+0FF1 */
  0x0020,   /* U+0FF2 */
  0x0020,   /* U+0FF3 */
  0x0020,   /* U+0FF4 */
  0x0020,   /* U+0FF5 */
  0x0020,   /* U+0FF6 */
  0x0020,   /* U+0FF7 */
  0x0020,   /* U+0FF8 */
  0x0020,   /* U+0FF9 */
  0x0020,   /* U+0FFA */
  0x0020,   /* U+0FFB */
  0x0020,   /* U+0FFC */
  0x0020,   /* U+0FFD */
  0x0020,   /* U+0FFE */
  0x0020,   /* U+0FFF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+0F00 */
  0x0002,   /* U+0F01 */
  0x0002,   /* U+0F02 */
  0x0002,   /* U+0F03 */
  0x0002,   /* U+0F04 */
  0x0002,   /* U+0F05 */
  0x0002,   /* U+0F06 */
  0x0002,   /* U+0F07 */
  0x0002,   /* U+0F08 */
  0x0002,   /* U+0F09 */
  0x0002,   /* U+0F0A */
  0x0002,   /* U+0F0B */
  0x001B,   /* U+0F0C */
  0x0002,   /* U+0F0D */
  0x0002,   /* U+0F0E */
  0x0002,   /* U+0F0F */
  0x0002,   /* U+0F10 */
  0x0002,   /* U+0F11 */
  0x0002,   /* U+0F12 */
  0x0002,   /* U+0F13 */
  0x0002,   /* U+0F14 */
  0x0002,   /* U+0F15 */
  0x0002,   /* U+0F16 */
  0x0002,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0002,   /* U+0F1A */
  0x0002,   /* U+0F1B */
  0x0002,   /* U+0F1C */
  0x0002,   /* U+0F1D */
  0x0002,   /* U+0F1E */
  0x0002,   /* U+0F1F */
  0x0002,   /* U+0F20 */
  0x0002,   /* U+0F21 */
  0x0002,   /* U+0F22 */
  0x0002,   /* U+0F23 */
  0x0002,   /* U+0F24 */
  0x0002,   /* U+0F25 */
  0x0002,   /* U+0F26 */
  0x0002,   /* U+0F27 */
  0x0002,   /* U+0F28 */
  0x0002,   /* U+0F29 */
  0x0004,   /* U+0F2A */
  0x0004,   /* U+0F2B */
  0x0004,   /* U+0F2C */
  0x0004,   /* U+0F2D */
  0x0004,   /* U+0F2E */
  0x0004,   /* U+0F2F */
  0x0004,   /* U+0F30 */
  0x0004,   /* U+0F31 */
  0x0004,   /* U+0F32 */
  0x0004,   /* U+0F33 */
  0x0002,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0002,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0002,   /* U+0F38 */
  0x0002,   /* U+0F39 */
  0x0002,   /* U+0F3A */
  0x0002,   /* U+0F3B */
  0x0002,   /* U+0F3C */
  0x0002,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0002,   /* U+0F40 */
  0x0002,   /* U+0F41 */
  0x0002,   /* U+0F42 */
  0x0002,   /* U+0F43 */
  0x0002,   /* U+0F44 */
  0x0002,   /* U+0F45 */
  0x0002,   /* U+0F46 */
  0x0002,   /* U+0F47 */
  0x0002,   /* U+0F48 */
  0x0002,   /* U+0F49 */
  0x0002,   /* U+0F4A */
  0x0002,   /* U+0F4B */
  0x0002,   /* U+0F4C */
  0x0002,   /* U+0F4D */
  0x0002,   /* U+0F4E */
  0x0002,   /* U+0F4F */
  0x0002,   /* U+0F50 */
  0x0002,   /* U+0F51 */
  0x0002,   /* U+0F52 */
  0x0002,   /* U+0F53 */
  0x0002,   /* U+0F54 */
  0x0002,   /* U+0F55 */
  0x0002,   /* U+0F56 */
  0x0002,   /* U+0F57 */
  0x0002,   /* U+0F58 */
  0x0002,   /* U+0F59 */
  0x0002,   /* U+0F5A */
  0x0002,   /* U+0F5B */
  0x0002,   /* U+0F5C */
  0x0002,   /* U+0F5D */
  0x0002,   /* U+0F5E */
  0x0002,   /* U+0F5F */
  0x0002,   /* U+0F60 */
  0x0002,   /* U+0F61 */
  0x0002,   /* U+0F62 */
  0x0002,   /* U+0F63 */
  0x0002,   /* U+0F64 */
  0x0002,   /* U+0F65 */
  0x0002,   /* U+0F66 */
  0x0002,   /* U+0F67 */
  0x0002,   /* U+0F68 */
  0x0002,   /* U+0F69 */
  0x0004,   /* U+0F6A */
  0x0002,   /* U+0F6B */
  0x0002,   /* U+0F6C */
  0x0002,   /* U+0F6D */
  0x0002,   /* U+0F6E */
  0x0002,   /* U+0F6F */
  0x0002,   /* U+0F70 */
  0x0002,   /* U+0F71 */
  0x0002,   /* U+0F72 */
  0x0002,   /* U+0F73 */
  0x0002,   /* U+0F74 */
  0x0002,   /* U+0F75 */
  0x0002,   /* U+0F76 */
  0x0002,   /* U+0F77 */
  0x0002,   /* U+0F78 */
  0x0002,   /* U+0F79 */
  0x0002,   /* U+0F7A */
  0x0002,   /* U+0F7B */
  0x0002,   /* U+0F7C */
  0x0002,   /* U+0F7D */
  0x0002,   /* U+0F7E */
  0x0002,   /* U+0F7F */
  0x0002,   /* U+0F80 */
  0x0002,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0002,   /* U+0F84 */
  0x0002,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0002,   /* U+0F88 */
  0x0002,   /* U+0F89 */
  0x0002,   /* U+0F8A */
  0x0002,   /* U+0F8B */
  0x0002,   /* U+0F8C */
  0x0002,   /* U+0F8D */
  0x0002,   /* U+0F8E */
  0x0002,   /* U+0F8F */
  0x0002,   /* U+0F90 */
  0x0002,   /* U+0F91 */
  0x0002,   /* U+0F92 */
  0x0002,   /* U+0F93 */
  0x0002,   /* U+0F94 */
  0x0002,   /* U+0F95 */
  0x0002,   /* U+0F96 */
  0x0002,   /* U+0F97 */
  0x0002,   /* U+0F98 */
  0x0002,   /* U+0F99 */
  0x0002,   /* U+0F9A */
  0x0002,   /* U+0F9B */
  0x0002,   /* U+0F9C */
  0x0002,   /* U+0F9D */
  0x0002,   /* U+0F9E */
  0x0002,   /* U+0F9F */
  0x0002,   /* U+0FA0 */
  0x0002,   /* U+0FA1 */
  0x0002,   /* U+0FA2 */
  0x0002,   /* U+0FA3 */
  0x0002,   /* U+0FA4 */
  0x0002,   /* U+0FA5 */
  0x0002,   /* U+0FA6 */
  0x0002,   /* U+0FA7 */
  0x0002,   /* U+0FA8 */
  0x0002,   /* U+0FA9 */
  0x0002,   /* U+0FAA */
  0x0002,   /* U+0FAB */
  0x0002,   /* U+0FAC */
  0x0002,   /* U+0FAD */
  0x0002,   /* U+0FAE */
  0x0002,   /* U+0FAF */
  0x0002,   /* U+0FB0 */
  0x0002,   /* U+0FB1 */
  0x0002,   /* U+0FB2 */
  0x0002,   /* U+0FB3 */
  0x0002,   /* U+0FB4 */
  0x0002,   /* U+0FB5 */
  0x0002,   /* U+0FB6 */
  0x0002,   /* U+0FB7 */
  0x0002,   /* U+0FB8 */
  0x0002,   /* U+0FB9 */
  0x0004,   /* U+0FBA */
  0x0004,   /* U+0FBB */
  0x0004,   /* U+0FBC */
  0x0002,   /* U+0FBD */
  0x0002,   /* U+0FBE */
  0x0002,   /* U+0FBF */
  0x0002,   /* U+0FC0 */
  0x0002,   /* U+0FC1 */
  0x0002,   /* U+0FC2 */
  0x0002,   /* U+0FC3 */
  0x0002,   /* U+0FC4 */
  0x0002,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0002,   /* U+0FC7 */
  0x0002,   /* U+0FC8 */
  0x0002,   /* U+0FC9 */
  0x0002,   /* U+0FCA */
  0x0002,   /* U+0FCB */
  0x0002,   /* U+0FCC */
  0x0002,   /* U+0FCD */
  0x0002,   /* U+0FCE */
  0x0002,   /* U+0FCF */
  0x0002,   /* U+0FD0 */
  0x0002,   /* U+0FD1 */
  0x0002,   /* U+0FD2 */
  0x0002,   /* U+0FD3 */
  0x0002,   /* U+0FD4 */
  0x0002,   /* U+0FD5 */
  0x0002,   /* U+0FD6 */
  0x0002,   /* U+0FD7 */
  0x0002,   /* U+0FD8 */
  0x0002,   /* U+0FD9 */
  0x0002,   /* U+0FDA */
  0x0002,   /* U+0FDB */
  0x0002,   /* U+0FDC */
  0x0002,   /* U+0FDD */
  0x0002,   /* U+0FDE */
  0x0002,   /* U+0FDF */
  0x0002,   /* U+0FE0 */
  0x0002,   /* U+0FE1 */
  0x0002,   /* U+0FE2 */
  0x0002,   /* U+0FE3 */
  0x0002,   /* U+0FE4 */
  0x0002,   /* U+0FE5 */
  0x0002,   /* U+0FE6 */
  0x0002,   /* U+0FE7 */
  0x0002,   /* U+0FE8 */
  0x0002,   /* U+0FE9 */
  0x0002,   /* U+0FEA */
  0x0002,   /* U+0FEB */
  0x0002,   /* U+0FEC */
  0x0002,   /* U+0FED */
  0x0002,   /* U+0FEE */
  0x0002,   /* U+0FEF */
  0x0002,   /* U+0FF0 */
  0x0002,   /* U+0FF1 */
  0x0002,   /* U+0FF2 */
  0x0002,   /* U+0FF3 */
  0x0002,   /* U+0FF4 */
  0x0002,   /* U+0FF5 */
  0x0002,   /* U+0FF6 */
  0x0002,   /* U+0FF7 */
  0x0002,   /* U+0FF8 */
  0x0002,   /* U+0FF9 */
  0x0002,   /* U+0FFA */
  0x0002,   /* U+0FFB */
  0x0002,   /* U+0FFC */
  0x0002,   /* U+0FFD */
  0x0002,   /* U+0FFE */
  0x0002,   /* U+0FFF */

  /* Primary weight 2 for each character. */
  0x2E83,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x2E6B,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x8F48,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x2E6B,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x2E6B,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x2E6B,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x2E6B,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x2E67,   /* U+0F69 */
  0x0000,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x8F6D,   /* U+0F6D */
  0x8F6E,   /* U+0F6E */
  0x8F6F,   /* U+0F6F */
  0x8F70,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x2E6B,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x8F98,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x2E6B,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x2E6B,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x2E6B,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x2E6B,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x2E67,   /* U+0FB9 */
  0x0000,   /* U+0FBA */
  0x0000,   /* U+0FBB */
  0x0000,   /* U+0FBC */
  0x8FBD,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x8FCD,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x8FDB,   /* U+0FDB */
  0x8FDC,   /* U+0FDC */
  0x8FDD,   /* U+0FDD */
  0x8FDE,   /* U+0FDE */
  0x8FDF,   /* U+0FDF */
  0x8FE0,   /* U+0FE0 */
  0x8FE1,   /* U+0FE1 */
  0x8FE2,   /* U+0FE2 */
  0x8FE3,   /* U+0FE3 */
  0x8FE4,   /* U+0FE4 */
  0x8FE5,   /* U+0FE5 */
  0x8FE6,   /* U+0FE6 */
  0x8FE7,   /* U+0FE7 */
  0x8FE8,   /* U+0FE8 */
  0x8FE9,   /* U+0FE9 */
  0x8FEA,   /* U+0FEA */
  0x8FEB,   /* U+0FEB */
  0x8FEC,   /* U+0FEC */
  0x8FED,   /* U+0FED */
  0x8FEE,   /* U+0FEE */
  0x8FEF,   /* U+0FEF */
  0x8FF0,   /* U+0FF0 */
  0x8FF1,   /* U+0FF1 */
  0x8FF2,   /* U+0FF2 */
  0x8FF3,   /* U+0FF3 */
  0x8FF4,   /* U+0FF4 */
  0x8FF5,   /* U+0FF5 */
  0x8FF6,   /* U+0FF6 */
  0x8FF7,   /* U+0FF7 */
  0x8FF8,   /* U+0FF8 */
  0x8FF9,   /* U+0FF9 */
  0x8FFA,   /* U+0FFA */
  0x8FFB,   /* U+0FFB */
  0x8FFC,   /* U+0FFC */
  0x8FFD,   /* U+0FFD */
  0x8FFE,   /* U+0FFE */
  0x8FFF,   /* U+0FFF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x0020,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x0000,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x0020,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x0020,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x0020,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x0020,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x0020,   /* U+0F69 */
  0x0111,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x0000,   /* U+0F6D */
  0x0000,   /* U+0F6E */
  0x0000,   /* U+0F6F */
  0x0000,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x0020,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x0000,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x0020,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x0020,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x0020,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x0020,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x0020,   /* U+0FB9 */
  0x0111,   /* U+0FBA */
  0x0111,   /* U+0FBB */
  0x0111,   /* U+0FBC */
  0x0000,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x0000,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x0000,   /* U+0FDB */
  0x0000,   /* U+0FDC */
  0x0000,   /* U+0FDD */
  0x0000,   /* U+0FDE */
  0x0000,   /* U+0FDF */
  0x0000,   /* U+0FE0 */
  0x0000,   /* U+0FE1 */
  0x0000,   /* U+0FE2 */
  0x0000,   /* U+0FE3 */
  0x0000,   /* U+0FE4 */
  0x0000,   /* U+0FE5 */
  0x0000,   /* U+0FE6 */
  0x0000,   /* U+0FE7 */
  0x0000,   /* U+0FE8 */
  0x0000,   /* U+0FE9 */
  0x0000,   /* U+0FEA */
  0x0000,   /* U+0FEB */
  0x0000,   /* U+0FEC */
  0x0000,   /* U+0FED */
  0x0000,   /* U+0FEE */
  0x0000,   /* U+0FEF */
  0x0000,   /* U+0FF0 */
  0x0000,   /* U+0FF1 */
  0x0000,   /* U+0FF2 */
  0x0000,   /* U+0FF3 */
  0x0000,   /* U+0FF4 */
  0x0000,   /* U+0FF5 */
  0x0000,   /* U+0FF6 */
  0x0000,   /* U+0FF7 */
  0x0000,   /* U+0FF8 */
  0x0000,   /* U+0FF9 */
  0x0000,   /* U+0FFA */
  0x0000,   /* U+0FFB */
  0x0000,   /* U+0FFC */
  0x0000,   /* U+0FFD */
  0x0000,   /* U+0FFE */
  0x0000,   /* U+0FFF */

  /* Tertiary weight 2 for each character. */
  0x0004,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x0002,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x0000,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x0002,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x0002,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x0002,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x0002,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x0002,   /* U+0F69 */
  0x0004,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x0000,   /* U+0F6D */
  0x0000,   /* U+0F6E */
  0x0000,   /* U+0F6F */
  0x0000,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x0002,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x0000,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x0002,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x0002,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x0002,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x0002,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x0002,   /* U+0FB9 */
  0x0004,   /* U+0FBA */
  0x0004,   /* U+0FBB */
  0x0004,   /* U+0FBC */
  0x0000,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x0000,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x0000,   /* U+0FDB */
  0x0000,   /* U+0FDC */
  0x0000,   /* U+0FDD */
  0x0000,   /* U+0FDE */
  0x0000,   /* U+0FDF */
  0x0000,   /* U+0FE0 */
  0x0000,   /* U+0FE1 */
  0x0000,   /* U+0FE2 */
  0x0000,   /* U+0FE3 */
  0x0000,   /* U+0FE4 */
  0x0000,   /* U+0FE5 */
  0x0000,   /* U+0FE6 */
  0x0000,   /* U+0FE7 */
  0x0000,   /* U+0FE8 */
  0x0000,   /* U+0FE9 */
  0x0000,   /* U+0FEA */
  0x0000,   /* U+0FEB */
  0x0000,   /* U+0FEC */
  0x0000,   /* U+0FED */
  0x0000,   /* U+0FEE */
  0x0000,   /* U+0FEF */
  0x0000,   /* U+0FF0 */
  0x0000,   /* U+0FF1 */
  0x0000,   /* U+0FF2 */
  0x0000,   /* U+0FF3 */
  0x0000,   /* U+0FF4 */
  0x0000,   /* U+0FF5 */
  0x0000,   /* U+0FF6 */
  0x0000,   /* U+0FF7 */
  0x0000,   /* U+0FF8 */
  0x0000,   /* U+0FF9 */
  0x0000,   /* U+0FFA */
  0x0000,   /* U+0FFB */
  0x0000,   /* U+0FFC */
  0x0000,   /* U+0FFD */
  0x0000,   /* U+0FFE */
  0x0000,   /* U+0FFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x0000,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x0000,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x0000,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x0000,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x0000,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x0000,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x0000,   /* U+0F69 */
  0x0000,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x0000,   /* U+0F6D */
  0x0000,   /* U+0F6E */
  0x0000,   /* U+0F6F */
  0x0000,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x0000,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x0000,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x0000,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x0000,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x0000,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x0000,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x0000,   /* U+0FB9 */
  0x0000,   /* U+0FBA */
  0x0000,   /* U+0FBB */
  0x0000,   /* U+0FBC */
  0x0000,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x0000,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x0000,   /* U+0FDB */
  0x0000,   /* U+0FDC */
  0x0000,   /* U+0FDD */
  0x0000,   /* U+0FDE */
  0x0000,   /* U+0FDF */
  0x0000,   /* U+0FE0 */
  0x0000,   /* U+0FE1 */
  0x0000,   /* U+0FE2 */
  0x0000,   /* U+0FE3 */
  0x0000,   /* U+0FE4 */
  0x0000,   /* U+0FE5 */
  0x0000,   /* U+0FE6 */
  0x0000,   /* U+0FE7 */
  0x0000,   /* U+0FE8 */
  0x0000,   /* U+0FE9 */
  0x0000,   /* U+0FEA */
  0x0000,   /* U+0FEB */
  0x0000,   /* U+0FEC */
  0x0000,   /* U+0FED */
  0x0000,   /* U+0FEE */
  0x0000,   /* U+0FEF */
  0x0000,   /* U+0FF0 */
  0x0000,   /* U+0FF1 */
  0x0000,   /* U+0FF2 */
  0x0000,   /* U+0FF3 */
  0x0000,   /* U+0FF4 */
  0x0000,   /* U+0FF5 */
  0x0000,   /* U+0FF6 */
  0x0000,   /* U+0FF7 */
  0x0000,   /* U+0FF8 */
  0x0000,   /* U+0FF9 */
  0x0000,   /* U+0FFA */
  0x0000,   /* U+0FFB */
  0x0000,   /* U+0FFC */
  0x0000,   /* U+0FFD */
  0x0000,   /* U+0FFE */
  0x0000,   /* U+0FFF */

  /* Secondary weight 3 for each character. */
  0x00C4,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x0000,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x0000,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x0000,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x0000,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x0000,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x0000,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x0000,   /* U+0F69 */
  0x0000,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x0000,   /* U+0F6D */
  0x0000,   /* U+0F6E */
  0x0000,   /* U+0F6F */
  0x0000,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x0000,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x0000,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x0000,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x0000,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x0000,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x0000,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x0000,   /* U+0FB9 */
  0x0000,   /* U+0FBA */
  0x0000,   /* U+0FBB */
  0x0000,   /* U+0FBC */
  0x0000,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x0000,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x0000,   /* U+0FDB */
  0x0000,   /* U+0FDC */
  0x0000,   /* U+0FDD */
  0x0000,   /* U+0FDE */
  0x0000,   /* U+0FDF */
  0x0000,   /* U+0FE0 */
  0x0000,   /* U+0FE1 */
  0x0000,   /* U+0FE2 */
  0x0000,   /* U+0FE3 */
  0x0000,   /* U+0FE4 */
  0x0000,   /* U+0FE5 */
  0x0000,   /* U+0FE6 */
  0x0000,   /* U+0FE7 */
  0x0000,   /* U+0FE8 */
  0x0000,   /* U+0FE9 */
  0x0000,   /* U+0FEA */
  0x0000,   /* U+0FEB */
  0x0000,   /* U+0FEC */
  0x0000,   /* U+0FED */
  0x0000,   /* U+0FEE */
  0x0000,   /* U+0FEF */
  0x0000,   /* U+0FF0 */
  0x0000,   /* U+0FF1 */
  0x0000,   /* U+0FF2 */
  0x0000,   /* U+0FF3 */
  0x0000,   /* U+0FF4 */
  0x0000,   /* U+0FF5 */
  0x0000,   /* U+0FF6 */
  0x0000,   /* U+0FF7 */
  0x0000,   /* U+0FF8 */
  0x0000,   /* U+0FF9 */
  0x0000,   /* U+0FFA */
  0x0000,   /* U+0FFB */
  0x0000,   /* U+0FFC */
  0x0000,   /* U+0FFD */
  0x0000,   /* U+0FFE */
  0x0000,   /* U+0FFF */

  /* Tertiary weight 3 for each character. */
  0x0004,   /* U+0F00 */
  0x0000,   /* U+0F01 */
  0x0000,   /* U+0F02 */
  0x0000,   /* U+0F03 */
  0x0000,   /* U+0F04 */
  0x0000,   /* U+0F05 */
  0x0000,   /* U+0F06 */
  0x0000,   /* U+0F07 */
  0x0000,   /* U+0F08 */
  0x0000,   /* U+0F09 */
  0x0000,   /* U+0F0A */
  0x0000,   /* U+0F0B */
  0x0000,   /* U+0F0C */
  0x0000,   /* U+0F0D */
  0x0000,   /* U+0F0E */
  0x0000,   /* U+0F0F */
  0x0000,   /* U+0F10 */
  0x0000,   /* U+0F11 */
  0x0000,   /* U+0F12 */
  0x0000,   /* U+0F13 */
  0x0000,   /* U+0F14 */
  0x0000,   /* U+0F15 */
  0x0000,   /* U+0F16 */
  0x0000,   /* U+0F17 */
  0x0000,   /* U+0F18 */
  0x0000,   /* U+0F19 */
  0x0000,   /* U+0F1A */
  0x0000,   /* U+0F1B */
  0x0000,   /* U+0F1C */
  0x0000,   /* U+0F1D */
  0x0000,   /* U+0F1E */
  0x0000,   /* U+0F1F */
  0x0000,   /* U+0F20 */
  0x0000,   /* U+0F21 */
  0x0000,   /* U+0F22 */
  0x0000,   /* U+0F23 */
  0x0000,   /* U+0F24 */
  0x0000,   /* U+0F25 */
  0x0000,   /* U+0F26 */
  0x0000,   /* U+0F27 */
  0x0000,   /* U+0F28 */
  0x0000,   /* U+0F29 */
  0x0000,   /* U+0F2A */
  0x0000,   /* U+0F2B */
  0x0000,   /* U+0F2C */
  0x0000,   /* U+0F2D */
  0x0000,   /* U+0F2E */
  0x0000,   /* U+0F2F */
  0x0000,   /* U+0F30 */
  0x0000,   /* U+0F31 */
  0x0000,   /* U+0F32 */
  0x0000,   /* U+0F33 */
  0x0000,   /* U+0F34 */
  0x0000,   /* U+0F35 */
  0x0000,   /* U+0F36 */
  0x0000,   /* U+0F37 */
  0x0000,   /* U+0F38 */
  0x0000,   /* U+0F39 */
  0x0000,   /* U+0F3A */
  0x0000,   /* U+0F3B */
  0x0000,   /* U+0F3C */
  0x0000,   /* U+0F3D */
  0x0000,   /* U+0F3E */
  0x0000,   /* U+0F3F */
  0x0000,   /* U+0F40 */
  0x0000,   /* U+0F41 */
  0x0000,   /* U+0F42 */
  0x0000,   /* U+0F43 */
  0x0000,   /* U+0F44 */
  0x0000,   /* U+0F45 */
  0x0000,   /* U+0F46 */
  0x0000,   /* U+0F47 */
  0x0000,   /* U+0F48 */
  0x0000,   /* U+0F49 */
  0x0000,   /* U+0F4A */
  0x0000,   /* U+0F4B */
  0x0000,   /* U+0F4C */
  0x0000,   /* U+0F4D */
  0x0000,   /* U+0F4E */
  0x0000,   /* U+0F4F */
  0x0000,   /* U+0F50 */
  0x0000,   /* U+0F51 */
  0x0000,   /* U+0F52 */
  0x0000,   /* U+0F53 */
  0x0000,   /* U+0F54 */
  0x0000,   /* U+0F55 */
  0x0000,   /* U+0F56 */
  0x0000,   /* U+0F57 */
  0x0000,   /* U+0F58 */
  0x0000,   /* U+0F59 */
  0x0000,   /* U+0F5A */
  0x0000,   /* U+0F5B */
  0x0000,   /* U+0F5C */
  0x0000,   /* U+0F5D */
  0x0000,   /* U+0F5E */
  0x0000,   /* U+0F5F */
  0x0000,   /* U+0F60 */
  0x0000,   /* U+0F61 */
  0x0000,   /* U+0F62 */
  0x0000,   /* U+0F63 */
  0x0000,   /* U+0F64 */
  0x0000,   /* U+0F65 */
  0x0000,   /* U+0F66 */
  0x0000,   /* U+0F67 */
  0x0000,   /* U+0F68 */
  0x0000,   /* U+0F69 */
  0x0000,   /* U+0F6A */
  0x0000,   /* U+0F6B */
  0x0000,   /* U+0F6C */
  0x0000,   /* U+0F6D */
  0x0000,   /* U+0F6E */
  0x0000,   /* U+0F6F */
  0x0000,   /* U+0F70 */
  0x0000,   /* U+0F71 */
  0x0000,   /* U+0F72 */
  0x0000,   /* U+0F73 */
  0x0000,   /* U+0F74 */
  0x0000,   /* U+0F75 */
  0x0000,   /* U+0F76 */
  0x0000,   /* U+0F77 */
  0x0000,   /* U+0F78 */
  0x0000,   /* U+0F79 */
  0x0000,   /* U+0F7A */
  0x0000,   /* U+0F7B */
  0x0000,   /* U+0F7C */
  0x0000,   /* U+0F7D */
  0x0000,   /* U+0F7E */
  0x0000,   /* U+0F7F */
  0x0000,   /* U+0F80 */
  0x0000,   /* U+0F81 */
  0x0000,   /* U+0F82 */
  0x0000,   /* U+0F83 */
  0x0000,   /* U+0F84 */
  0x0000,   /* U+0F85 */
  0x0000,   /* U+0F86 */
  0x0000,   /* U+0F87 */
  0x0000,   /* U+0F88 */
  0x0000,   /* U+0F89 */
  0x0000,   /* U+0F8A */
  0x0000,   /* U+0F8B */
  0x0000,   /* U+0F8C */
  0x0000,   /* U+0F8D */
  0x0000,   /* U+0F8E */
  0x0000,   /* U+0F8F */
  0x0000,   /* U+0F90 */
  0x0000,   /* U+0F91 */
  0x0000,   /* U+0F92 */
  0x0000,   /* U+0F93 */
  0x0000,   /* U+0F94 */
  0x0000,   /* U+0F95 */
  0x0000,   /* U+0F96 */
  0x0000,   /* U+0F97 */
  0x0000,   /* U+0F98 */
  0x0000,   /* U+0F99 */
  0x0000,   /* U+0F9A */
  0x0000,   /* U+0F9B */
  0x0000,   /* U+0F9C */
  0x0000,   /* U+0F9D */
  0x0000,   /* U+0F9E */
  0x0000,   /* U+0F9F */
  0x0000,   /* U+0FA0 */
  0x0000,   /* U+0FA1 */
  0x0000,   /* U+0FA2 */
  0x0000,   /* U+0FA3 */
  0x0000,   /* U+0FA4 */
  0x0000,   /* U+0FA5 */
  0x0000,   /* U+0FA6 */
  0x0000,   /* U+0FA7 */
  0x0000,   /* U+0FA8 */
  0x0000,   /* U+0FA9 */
  0x0000,   /* U+0FAA */
  0x0000,   /* U+0FAB */
  0x0000,   /* U+0FAC */
  0x0000,   /* U+0FAD */
  0x0000,   /* U+0FAE */
  0x0000,   /* U+0FAF */
  0x0000,   /* U+0FB0 */
  0x0000,   /* U+0FB1 */
  0x0000,   /* U+0FB2 */
  0x0000,   /* U+0FB3 */
  0x0000,   /* U+0FB4 */
  0x0000,   /* U+0FB5 */
  0x0000,   /* U+0FB6 */
  0x0000,   /* U+0FB7 */
  0x0000,   /* U+0FB8 */
  0x0000,   /* U+0FB9 */
  0x0000,   /* U+0FBA */
  0x0000,   /* U+0FBB */
  0x0000,   /* U+0FBC */
  0x0000,   /* U+0FBD */
  0x0000,   /* U+0FBE */
  0x0000,   /* U+0FBF */
  0x0000,   /* U+0FC0 */
  0x0000,   /* U+0FC1 */
  0x0000,   /* U+0FC2 */
  0x0000,   /* U+0FC3 */
  0x0000,   /* U+0FC4 */
  0x0000,   /* U+0FC5 */
  0x0000,   /* U+0FC6 */
  0x0000,   /* U+0FC7 */
  0x0000,   /* U+0FC8 */
  0x0000,   /* U+0FC9 */
  0x0000,   /* U+0FCA */
  0x0000,   /* U+0FCB */
  0x0000,   /* U+0FCC */
  0x0000,   /* U+0FCD */
  0x0000,   /* U+0FCE */
  0x0000,   /* U+0FCF */
  0x0000,   /* U+0FD0 */
  0x0000,   /* U+0FD1 */
  0x0000,   /* U+0FD2 */
  0x0000,   /* U+0FD3 */
  0x0000,   /* U+0FD4 */
  0x0000,   /* U+0FD5 */
  0x0000,   /* U+0FD6 */
  0x0000,   /* U+0FD7 */
  0x0000,   /* U+0FD8 */
  0x0000,   /* U+0FD9 */
  0x0000,   /* U+0FDA */
  0x0000,   /* U+0FDB */
  0x0000,   /* U+0FDC */
  0x0000,   /* U+0FDD */
  0x0000,   /* U+0FDE */
  0x0000,   /* U+0FDF */
  0x0000,   /* U+0FE0 */
  0x0000,   /* U+0FE1 */
  0x0000,   /* U+0FE2 */
  0x0000,   /* U+0FE3 */
  0x0000,   /* U+0FE4 */
  0x0000,   /* U+0FE5 */
  0x0000,   /* U+0FE6 */
  0x0000,   /* U+0FE7 */
  0x0000,   /* U+0FE8 */
  0x0000,   /* U+0FE9 */
  0x0000,   /* U+0FEA */
  0x0000,   /* U+0FEB */
  0x0000,   /* U+0FEC */
  0x0000,   /* U+0FED */
  0x0000,   /* U+0FEE */
  0x0000,   /* U+0FEF */
  0x0000,   /* U+0FF0 */
  0x0000,   /* U+0FF1 */
  0x0000,   /* U+0FF2 */
  0x0000,   /* U+0FF3 */
  0x0000,   /* U+0FF4 */
  0x0000,   /* U+0FF5 */
  0x0000,   /* U+0FF6 */
  0x0000,   /* U+0FF7 */
  0x0000,   /* U+0FF8 */
  0x0000,   /* U+0FF9 */
  0x0000,   /* U+0FFA */
  0x0000,   /* U+0FFB */
  0x0000,   /* U+0FFC */
  0x0000,   /* U+0FFD */
  0x0000,   /* U+0FFE */
  0x0000,   /* U+0FFF */
};

uint16 uca900_p010[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 2, 2, 2, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x3035,   /* U+1000 */
  0x3037,   /* U+1001 */
  0x3039,   /* U+1002 */
  0x303D,   /* U+1003 */
  0x3040,   /* U+1004 */
  0x3042,   /* U+1005 */
  0x3045,   /* U+1006 */
  0x3049,   /* U+1007 */
  0x304E,   /* U+1008 */
  0x3055,   /* U+1009 */
  0x3059,   /* U+100A */
  0x305A,   /* U+100B */
  0x305C,   /* U+100C */
  0x305E,   /* U+100D */
  0x3061,   /* U+100E */
  0x3064,   /* U+100F */
  0x3068,   /* U+1010 */
  0x3069,   /* U+1011 */
  0x306A,   /* U+1012 */
  0x306D,   /* U+1013 */
  0x3070,   /* U+1014 */
  0x3074,   /* U+1015 */
  0x3075,   /* U+1016 */
  0x307B,   /* U+1017 */
  0x307E,   /* U+1018 */
  0x3081,   /* U+1019 */
  0x3083,   /* U+101A */
  0x3085,   /* U+101B */
  0x3089,   /* U+101C */
  0x308B,   /* U+101D */
  0x3092,   /* U+101E */
  0x3094,   /* U+101F */
  0x309A,   /* U+1020 */
  0x30A1,   /* U+1021 */
  0x30A2,   /* U+1022 */
  0x30A3,   /* U+1023 */
  0x30A4,   /* U+1024 */
  0x30A5,   /* U+1025 */
  0x30A6,   /* U+1026 */
  0x30AB,   /* U+1027 */
  0x30AC,   /* U+1028 */
  0x30AD,   /* U+1029 */
  0x30AE,   /* U+102A */
  0x30AF,   /* U+102B */
  0x30AF,   /* U+102C */
  0x30B3,   /* U+102D */
  0x30B5,   /* U+102E */
  0x30B7,   /* U+102F */
  0x30BA,   /* U+1030 */
  0x30BF,   /* U+1031 */
  0x30C3,   /* U+1032 */
  0x30B6,   /* U+1033 */
  0x30C5,   /* U+1034 */
  0x30C1,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x30CB,   /* U+1039 */
  0x30CC,   /* U+103A */
  0x3084,   /* U+103B */
  0x3088,   /* U+103C */
  0x308C,   /* U+103D */
  0x3097,   /* U+103E */
  0x3092,   /* U+103F */
  0x1C3D,   /* U+1040 */
  0x1C3E,   /* U+1041 */
  0x1C3F,   /* U+1042 */
  0x1C40,   /* U+1043 */
  0x1C41,   /* U+1044 */
  0x1C42,   /* U+1045 */
  0x1C43,   /* U+1046 */
  0x1C44,   /* U+1047 */
  0x1C45,   /* U+1048 */
  0x1C46,   /* U+1049 */
  0x0299,   /* U+104A */
  0x029A,   /* U+104B */
  0x040E,   /* U+104C */
  0x040F,   /* U+104D */
  0x0410,   /* U+104E */
  0x0411,   /* U+104F */
  0x308F,   /* U+1050 */
  0x3090,   /* U+1051 */
  0x30A7,   /* U+1052 */
  0x30A8,   /* U+1053 */
  0x30A9,   /* U+1054 */
  0x30AA,   /* U+1055 */
  0x30BB,   /* U+1056 */
  0x30BC,   /* U+1057 */
  0x30BD,   /* U+1058 */
  0x30BE,   /* U+1059 */
  0x3041,   /* U+105A */
  0x304F,   /* U+105B */
  0x309C,   /* U+105C */
  0x309D,   /* U+105D */
  0x3073,   /* U+105E */
  0x3082,   /* U+105F */
  0x308A,   /* U+1060 */
  0x3053,   /* U+1061 */
  0x30C6,   /* U+1062 */
  0x30CD,   /* U+1063 */
  0x30CE,   /* U+1064 */
  0x3091,   /* U+1065 */
  0x30A0,   /* U+1066 */
  0x30C7,   /* U+1067 */
  0x30C8,   /* U+1068 */
  0x30CF,   /* U+1069 */
  0x30D0,   /* U+106A */
  0x30D1,   /* U+106B */
  0x30D2,   /* U+106C */
  0x30D3,   /* U+106D */
  0x3065,   /* U+106E */
  0x309E,   /* U+106F */
  0x309F,   /* U+1070 */
  0x30B4,   /* U+1071 */
  0x30B1,   /* U+1072 */
  0x30B8,   /* U+1073 */
  0x30B9,   /* U+1074 */
  0x3036,   /* U+1075 */
  0x3038,   /* U+1076 */
  0x303A,   /* U+1077 */
  0x3043,   /* U+1078 */
  0x304C,   /* U+1079 */
  0x3056,   /* U+107A */
  0x306B,   /* U+107B */
  0x3071,   /* U+107C */
  0x3076,   /* U+107D */
  0x3077,   /* U+107E */
  0x307C,   /* U+107F */
  0x308E,   /* U+1080 */
  0x3095,   /* U+1081 */
  0x308D,   /* U+1082 */
  0x30B0,   /* U+1083 */
  0x30C0,   /* U+1084 */
  0x30C2,   /* U+1085 */
  0x30CA,   /* U+1086 */
  0x30D4,   /* U+1087 */
  0x30D6,   /* U+1088 */
  0x30D9,   /* U+1089 */
  0x30DA,   /* U+108A */
  0x30D5,   /* U+108B */
  0x30D7,   /* U+108C */
  0x30D8,   /* U+108D */
  0x3079,   /* U+108E */
  0x30DB,   /* U+108F */
  0x1C3D,   /* U+1090 */
  0x1C3E,   /* U+1091 */
  0x1C3F,   /* U+1092 */
  0x1C40,   /* U+1093 */
  0x1C41,   /* U+1094 */
  0x1C42,   /* U+1095 */
  0x1C43,   /* U+1096 */
  0x1C44,   /* U+1097 */
  0x1C45,   /* U+1098 */
  0x1C46,   /* U+1099 */
  0x30DC,   /* U+109A */
  0x30DD,   /* U+109B */
  0x30B2,   /* U+109C */
  0x30C4,   /* U+109D */
  0x054B,   /* U+109E */
  0x054C,   /* U+109F */
  0x223B,   /* U+10A0 */
  0x223D,   /* U+10A1 */
  0x223F,   /* U+10A2 */
  0x2241,   /* U+10A3 */
  0x2243,   /* U+10A4 */
  0x2245,   /* U+10A5 */
  0x2247,   /* U+10A6 */
  0x224B,   /* U+10A7 */
  0x224D,   /* U+10A8 */
  0x224F,   /* U+10A9 */
  0x2251,   /* U+10AA */
  0x2253,   /* U+10AB */
  0x2255,   /* U+10AC */
  0x2259,   /* U+10AD */
  0x225B,   /* U+10AE */
  0x225D,   /* U+10AF */
  0x225F,   /* U+10B0 */
  0x2261,   /* U+10B1 */
  0x2263,   /* U+10B2 */
  0x2267,   /* U+10B3 */
  0x2269,   /* U+10B4 */
  0x226B,   /* U+10B5 */
  0x226D,   /* U+10B6 */
  0x226F,   /* U+10B7 */
  0x2271,   /* U+10B8 */
  0x2273,   /* U+10B9 */
  0x2275,   /* U+10BA */
  0x2277,   /* U+10BB */
  0x2279,   /* U+10BC */
  0x227B,   /* U+10BD */
  0x227D,   /* U+10BE */
  0x2281,   /* U+10BF */
  0x2283,   /* U+10C0 */
  0x2249,   /* U+10C1 */
  0x2257,   /* U+10C2 */
  0x2265,   /* U+10C3 */
  0x227F,   /* U+10C4 */
  0x2285,   /* U+10C5 */
  0xFBC0,   /* U+10C6 */
  0x2288,   /* U+10C7 */
  0xFBC0,   /* U+10C8 */
  0xFBC0,   /* U+10C9 */
  0xFBC0,   /* U+10CA */
  0xFBC0,   /* U+10CB */
  0xFBC0,   /* U+10CC */
  0x228D,   /* U+10CD */
  0xFBC0,   /* U+10CE */
  0xFBC0,   /* U+10CF */
  0x223A,   /* U+10D0 */
  0x223C,   /* U+10D1 */
  0x223E,   /* U+10D2 */
  0x2240,   /* U+10D3 */
  0x2242,   /* U+10D4 */
  0x2244,   /* U+10D5 */
  0x2246,   /* U+10D6 */
  0x224A,   /* U+10D7 */
  0x224C,   /* U+10D8 */
  0x224E,   /* U+10D9 */
  0x2250,   /* U+10DA */
  0x2252,   /* U+10DB */
  0x2254,   /* U+10DC */
  0x2258,   /* U+10DD */
  0x225A,   /* U+10DE */
  0x225C,   /* U+10DF */
  0x225E,   /* U+10E0 */
  0x2260,   /* U+10E1 */
  0x2262,   /* U+10E2 */
  0x2266,   /* U+10E3 */
  0x2268,   /* U+10E4 */
  0x226A,   /* U+10E5 */
  0x226C,   /* U+10E6 */
  0x226E,   /* U+10E7 */
  0x2270,   /* U+10E8 */
  0x2272,   /* U+10E9 */
  0x2274,   /* U+10EA */
  0x2276,   /* U+10EB */
  0x2278,   /* U+10EC */
  0x227A,   /* U+10ED */
  0x227C,   /* U+10EE */
  0x2280,   /* U+10EF */
  0x2282,   /* U+10F0 */
  0x2248,   /* U+10F1 */
  0x2256,   /* U+10F2 */
  0x2264,   /* U+10F3 */
  0x227E,   /* U+10F4 */
  0x2284,   /* U+10F5 */
  0x2286,   /* U+10F6 */
  0x2287,   /* U+10F7 */
  0x2289,   /* U+10F8 */
  0x228A,   /* U+10F9 */
  0x228B,   /* U+10FA */
  0x02C6,   /* U+10FB */
  0x2254,   /* U+10FC */
  0x228C,   /* U+10FD */
  0x228E,   /* U+10FE */
  0x228F,   /* U+10FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1000 */
  0x0020,   /* U+1001 */
  0x0020,   /* U+1002 */
  0x0020,   /* U+1003 */
  0x0020,   /* U+1004 */
  0x0020,   /* U+1005 */
  0x0020,   /* U+1006 */
  0x0020,   /* U+1007 */
  0x0020,   /* U+1008 */
  0x0020,   /* U+1009 */
  0x0020,   /* U+100A */
  0x0020,   /* U+100B */
  0x0020,   /* U+100C */
  0x0020,   /* U+100D */
  0x0020,   /* U+100E */
  0x0020,   /* U+100F */
  0x0020,   /* U+1010 */
  0x0020,   /* U+1011 */
  0x0020,   /* U+1012 */
  0x0020,   /* U+1013 */
  0x0020,   /* U+1014 */
  0x0020,   /* U+1015 */
  0x0020,   /* U+1016 */
  0x0020,   /* U+1017 */
  0x0020,   /* U+1018 */
  0x0020,   /* U+1019 */
  0x0020,   /* U+101A */
  0x0020,   /* U+101B */
  0x0020,   /* U+101C */
  0x0020,   /* U+101D */
  0x0020,   /* U+101E */
  0x0020,   /* U+101F */
  0x0020,   /* U+1020 */
  0x0020,   /* U+1021 */
  0x0020,   /* U+1022 */
  0x0020,   /* U+1023 */
  0x0020,   /* U+1024 */
  0x0020,   /* U+1025 */
  0x0020,   /* U+1026 */
  0x0020,   /* U+1027 */
  0x0020,   /* U+1028 */
  0x0020,   /* U+1029 */
  0x0020,   /* U+102A */
  0x0020,   /* U+102B */
  0x0020,   /* U+102C */
  0x0020,   /* U+102D */
  0x0020,   /* U+102E */
  0x0020,   /* U+102F */
  0x0020,   /* U+1030 */
  0x0020,   /* U+1031 */
  0x0020,   /* U+1032 */
  0x0020,   /* U+1033 */
  0x0020,   /* U+1034 */
  0x0020,   /* U+1035 */
  0x00C4,   /* U+1036 */
  0x00E5,   /* U+1037 */
  0x00C5,   /* U+1038 */
  0x0020,   /* U+1039 */
  0x0020,   /* U+103A */
  0x0020,   /* U+103B */
  0x0020,   /* U+103C */
  0x0020,   /* U+103D */
  0x0020,   /* U+103E */
  0x0020,   /* U+103F */
  0x0020,   /* U+1040 */
  0x0020,   /* U+1041 */
  0x0020,   /* U+1042 */
  0x0020,   /* U+1043 */
  0x0020,   /* U+1044 */
  0x0020,   /* U+1045 */
  0x0020,   /* U+1046 */
  0x0020,   /* U+1047 */
  0x0020,   /* U+1048 */
  0x0020,   /* U+1049 */
  0x0020,   /* U+104A */
  0x0020,   /* U+104B */
  0x0020,   /* U+104C */
  0x0020,   /* U+104D */
  0x0020,   /* U+104E */
  0x0020,   /* U+104F */
  0x0020,   /* U+1050 */
  0x0020,   /* U+1051 */
  0x0020,   /* U+1052 */
  0x0020,   /* U+1053 */
  0x0020,   /* U+1054 */
  0x0020,   /* U+1055 */
  0x0020,   /* U+1056 */
  0x0020,   /* U+1057 */
  0x0020,   /* U+1058 */
  0x0020,   /* U+1059 */
  0x0020,   /* U+105A */
  0x0020,   /* U+105B */
  0x0020,   /* U+105C */
  0x0020,   /* U+105D */
  0x0020,   /* U+105E */
  0x0020,   /* U+105F */
  0x0020,   /* U+1060 */
  0x0020,   /* U+1061 */
  0x0020,   /* U+1062 */
  0x0020,   /* U+1063 */
  0x0020,   /* U+1064 */
  0x0020,   /* U+1065 */
  0x0020,   /* U+1066 */
  0x0020,   /* U+1067 */
  0x0020,   /* U+1068 */
  0x0020,   /* U+1069 */
  0x0020,   /* U+106A */
  0x0020,   /* U+106B */
  0x0020,   /* U+106C */
  0x0020,   /* U+106D */
  0x0020,   /* U+106E */
  0x0020,   /* U+106F */
  0x0020,   /* U+1070 */
  0x0020,   /* U+1071 */
  0x0020,   /* U+1072 */
  0x0020,   /* U+1073 */
  0x0020,   /* U+1074 */
  0x0020,   /* U+1075 */
  0x0020,   /* U+1076 */
  0x0020,   /* U+1077 */
  0x0020,   /* U+1078 */
  0x0020,   /* U+1079 */
  0x0020,   /* U+107A */
  0x0020,   /* U+107B */
  0x0020,   /* U+107C */
  0x0020,   /* U+107D */
  0x0020,   /* U+107E */
  0x0020,   /* U+107F */
  0x0020,   /* U+1080 */
  0x0020,   /* U+1081 */
  0x0020,   /* U+1082 */
  0x0020,   /* U+1083 */
  0x0020,   /* U+1084 */
  0x0020,   /* U+1085 */
  0x0020,   /* U+1086 */
  0x0020,   /* U+1087 */
  0x0020,   /* U+1088 */
  0x0020,   /* U+1089 */
  0x0020,   /* U+108A */
  0x0020,   /* U+108B */
  0x0020,   /* U+108C */
  0x0020,   /* U+108D */
  0x0020,   /* U+108E */
  0x0020,   /* U+108F */
  0x0020,   /* U+1090 */
  0x0020,   /* U+1091 */
  0x0020,   /* U+1092 */
  0x0020,   /* U+1093 */
  0x0020,   /* U+1094 */
  0x0020,   /* U+1095 */
  0x0020,   /* U+1096 */
  0x0020,   /* U+1097 */
  0x0020,   /* U+1098 */
  0x0020,   /* U+1099 */
  0x0020,   /* U+109A */
  0x0020,   /* U+109B */
  0x0020,   /* U+109C */
  0x0020,   /* U+109D */
  0x0020,   /* U+109E */
  0x0020,   /* U+109F */
  0x0020,   /* U+10A0 */
  0x0020,   /* U+10A1 */
  0x0020,   /* U+10A2 */
  0x0020,   /* U+10A3 */
  0x0020,   /* U+10A4 */
  0x0020,   /* U+10A5 */
  0x0020,   /* U+10A6 */
  0x0020,   /* U+10A7 */
  0x0020,   /* U+10A8 */
  0x0020,   /* U+10A9 */
  0x0020,   /* U+10AA */
  0x0020,   /* U+10AB */
  0x0020,   /* U+10AC */
  0x0020,   /* U+10AD */
  0x0020,   /* U+10AE */
  0x0020,   /* U+10AF */
  0x0020,   /* U+10B0 */
  0x0020,   /* U+10B1 */
  0x0020,   /* U+10B2 */
  0x0020,   /* U+10B3 */
  0x0020,   /* U+10B4 */
  0x0020,   /* U+10B5 */
  0x0020,   /* U+10B6 */
  0x0020,   /* U+10B7 */
  0x0020,   /* U+10B8 */
  0x0020,   /* U+10B9 */
  0x0020,   /* U+10BA */
  0x0020,   /* U+10BB */
  0x0020,   /* U+10BC */
  0x0020,   /* U+10BD */
  0x0020,   /* U+10BE */
  0x0020,   /* U+10BF */
  0x0020,   /* U+10C0 */
  0x0020,   /* U+10C1 */
  0x0020,   /* U+10C2 */
  0x0020,   /* U+10C3 */
  0x0020,   /* U+10C4 */
  0x0020,   /* U+10C5 */
  0x0020,   /* U+10C6 */
  0x0020,   /* U+10C7 */
  0x0020,   /* U+10C8 */
  0x0020,   /* U+10C9 */
  0x0020,   /* U+10CA */
  0x0020,   /* U+10CB */
  0x0020,   /* U+10CC */
  0x0020,   /* U+10CD */
  0x0020,   /* U+10CE */
  0x0020,   /* U+10CF */
  0x0020,   /* U+10D0 */
  0x0020,   /* U+10D1 */
  0x0020,   /* U+10D2 */
  0x0020,   /* U+10D3 */
  0x0020,   /* U+10D4 */
  0x0020,   /* U+10D5 */
  0x0020,   /* U+10D6 */
  0x0020,   /* U+10D7 */
  0x0020,   /* U+10D8 */
  0x0020,   /* U+10D9 */
  0x0020,   /* U+10DA */
  0x0020,   /* U+10DB */
  0x0020,   /* U+10DC */
  0x0020,   /* U+10DD */
  0x0020,   /* U+10DE */
  0x0020,   /* U+10DF */
  0x0020,   /* U+10E0 */
  0x0020,   /* U+10E1 */
  0x0020,   /* U+10E2 */
  0x0020,   /* U+10E3 */
  0x0020,   /* U+10E4 */
  0x0020,   /* U+10E5 */
  0x0020,   /* U+10E6 */
  0x0020,   /* U+10E7 */
  0x0020,   /* U+10E8 */
  0x0020,   /* U+10E9 */
  0x0020,   /* U+10EA */
  0x0020,   /* U+10EB */
  0x0020,   /* U+10EC */
  0x0020,   /* U+10ED */
  0x0020,   /* U+10EE */
  0x0020,   /* U+10EF */
  0x0020,   /* U+10F0 */
  0x0020,   /* U+10F1 */
  0x0020,   /* U+10F2 */
  0x0020,   /* U+10F3 */
  0x0020,   /* U+10F4 */
  0x0020,   /* U+10F5 */
  0x0020,   /* U+10F6 */
  0x0020,   /* U+10F7 */
  0x0020,   /* U+10F8 */
  0x0020,   /* U+10F9 */
  0x0020,   /* U+10FA */
  0x0020,   /* U+10FB */
  0x0020,   /* U+10FC */
  0x0020,   /* U+10FD */
  0x0020,   /* U+10FE */
  0x0020,   /* U+10FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1000 */
  0x0002,   /* U+1001 */
  0x0002,   /* U+1002 */
  0x0002,   /* U+1003 */
  0x0002,   /* U+1004 */
  0x0002,   /* U+1005 */
  0x0002,   /* U+1006 */
  0x0002,   /* U+1007 */
  0x0002,   /* U+1008 */
  0x0002,   /* U+1009 */
  0x0002,   /* U+100A */
  0x0002,   /* U+100B */
  0x0002,   /* U+100C */
  0x0002,   /* U+100D */
  0x0002,   /* U+100E */
  0x0002,   /* U+100F */
  0x0002,   /* U+1010 */
  0x0002,   /* U+1011 */
  0x0002,   /* U+1012 */
  0x0002,   /* U+1013 */
  0x0002,   /* U+1014 */
  0x0002,   /* U+1015 */
  0x0002,   /* U+1016 */
  0x0002,   /* U+1017 */
  0x0002,   /* U+1018 */
  0x0002,   /* U+1019 */
  0x0002,   /* U+101A */
  0x0002,   /* U+101B */
  0x0002,   /* U+101C */
  0x0002,   /* U+101D */
  0x0002,   /* U+101E */
  0x0002,   /* U+101F */
  0x0002,   /* U+1020 */
  0x0002,   /* U+1021 */
  0x0002,   /* U+1022 */
  0x0002,   /* U+1023 */
  0x0002,   /* U+1024 */
  0x0002,   /* U+1025 */
  0x0002,   /* U+1026 */
  0x0002,   /* U+1027 */
  0x0002,   /* U+1028 */
  0x0002,   /* U+1029 */
  0x0002,   /* U+102A */
  0x0004,   /* U+102B */
  0x0002,   /* U+102C */
  0x0002,   /* U+102D */
  0x0002,   /* U+102E */
  0x0002,   /* U+102F */
  0x0002,   /* U+1030 */
  0x0002,   /* U+1031 */
  0x0002,   /* U+1032 */
  0x0002,   /* U+1033 */
  0x0002,   /* U+1034 */
  0x0002,   /* U+1035 */
  0x0002,   /* U+1036 */
  0x0002,   /* U+1037 */
  0x0002,   /* U+1038 */
  0x0002,   /* U+1039 */
  0x0002,   /* U+103A */
  0x0002,   /* U+103B */
  0x0002,   /* U+103C */
  0x0002,   /* U+103D */
  0x0002,   /* U+103E */
  0x0004,   /* U+103F */
  0x0002,   /* U+1040 */
  0x0002,   /* U+1041 */
  0x0002,   /* U+1042 */
  0x0002,   /* U+1043 */
  0x0002,   /* U+1044 */
  0x0002,   /* U+1045 */
  0x0002,   /* U+1046 */
  0x0002,   /* U+1047 */
  0x0002,   /* U+1048 */
  0x0002,   /* U+1049 */
  0x0002,   /* U+104A */
  0x0002,   /* U+104B */
  0x0002,   /* U+104C */
  0x0002,   /* U+104D */
  0x0002,   /* U+104E */
  0x0002,   /* U+104F */
  0x0002,   /* U+1050 */
  0x0002,   /* U+1051 */
  0x0002,   /* U+1052 */
  0x0002,   /* U+1053 */
  0x0002,   /* U+1054 */
  0x0002,   /* U+1055 */
  0x0002,   /* U+1056 */
  0x0002,   /* U+1057 */
  0x0002,   /* U+1058 */
  0x0002,   /* U+1059 */
  0x0002,   /* U+105A */
  0x0002,   /* U+105B */
  0x0002,   /* U+105C */
  0x0002,   /* U+105D */
  0x0002,   /* U+105E */
  0x0002,   /* U+105F */
  0x0002,   /* U+1060 */
  0x0002,   /* U+1061 */
  0x0002,   /* U+1062 */
  0x0002,   /* U+1063 */
  0x0002,   /* U+1064 */
  0x0002,   /* U+1065 */
  0x0002,   /* U+1066 */
  0x0002,   /* U+1067 */
  0x0002,   /* U+1068 */
  0x0002,   /* U+1069 */
  0x0002,   /* U+106A */
  0x0002,   /* U+106B */
  0x0002,   /* U+106C */
  0x0002,   /* U+106D */
  0x0002,   /* U+106E */
  0x0002,   /* U+106F */
  0x0002,   /* U+1070 */
  0x0002,   /* U+1071 */
  0x0002,   /* U+1072 */
  0x0002,   /* U+1073 */
  0x0002,   /* U+1074 */
  0x0002,   /* U+1075 */
  0x0002,   /* U+1076 */
  0x0002,   /* U+1077 */
  0x0002,   /* U+1078 */
  0x0002,   /* U+1079 */
  0x0002,   /* U+107A */
  0x0002,   /* U+107B */
  0x0002,   /* U+107C */
  0x0002,   /* U+107D */
  0x0002,   /* U+107E */
  0x0002,   /* U+107F */
  0x0002,   /* U+1080 */
  0x0002,   /* U+1081 */
  0x0002,   /* U+1082 */
  0x0002,   /* U+1083 */
  0x0002,   /* U+1084 */
  0x0002,   /* U+1085 */
  0x0002,   /* U+1086 */
  0x0002,   /* U+1087 */
  0x0002,   /* U+1088 */
  0x0002,   /* U+1089 */
  0x0002,   /* U+108A */
  0x0002,   /* U+108B */
  0x0002,   /* U+108C */
  0x0002,   /* U+108D */
  0x0002,   /* U+108E */
  0x0002,   /* U+108F */
  0x0002,   /* U+1090 */
  0x0002,   /* U+1091 */
  0x0002,   /* U+1092 */
  0x0002,   /* U+1093 */
  0x0002,   /* U+1094 */
  0x0002,   /* U+1095 */
  0x0002,   /* U+1096 */
  0x0002,   /* U+1097 */
  0x0002,   /* U+1098 */
  0x0002,   /* U+1099 */
  0x0002,   /* U+109A */
  0x0002,   /* U+109B */
  0x0002,   /* U+109C */
  0x0002,   /* U+109D */
  0x0002,   /* U+109E */
  0x0002,   /* U+109F */
  0x0008,   /* U+10A0 */
  0x0008,   /* U+10A1 */
  0x0008,   /* U+10A2 */
  0x0008,   /* U+10A3 */
  0x0008,   /* U+10A4 */
  0x0008,   /* U+10A5 */
  0x0008,   /* U+10A6 */
  0x0008,   /* U+10A7 */
  0x0008,   /* U+10A8 */
  0x0008,   /* U+10A9 */
  0x0008,   /* U+10AA */
  0x0008,   /* U+10AB */
  0x0008,   /* U+10AC */
  0x0008,   /* U+10AD */
  0x0008,   /* U+10AE */
  0x0008,   /* U+10AF */
  0x0008,   /* U+10B0 */
  0x0008,   /* U+10B1 */
  0x0008,   /* U+10B2 */
  0x0008,   /* U+10B3 */
  0x0008,   /* U+10B4 */
  0x0008,   /* U+10B5 */
  0x0008,   /* U+10B6 */
  0x0008,   /* U+10B7 */
  0x0008,   /* U+10B8 */
  0x0008,   /* U+10B9 */
  0x0008,   /* U+10BA */
  0x0008,   /* U+10BB */
  0x0008,   /* U+10BC */
  0x0008,   /* U+10BD */
  0x0008,   /* U+10BE */
  0x0008,   /* U+10BF */
  0x0008,   /* U+10C0 */
  0x0008,   /* U+10C1 */
  0x0008,   /* U+10C2 */
  0x0008,   /* U+10C3 */
  0x0008,   /* U+10C4 */
  0x0008,   /* U+10C5 */
  0x0002,   /* U+10C6 */
  0x0008,   /* U+10C7 */
  0x0002,   /* U+10C8 */
  0x0002,   /* U+10C9 */
  0x0002,   /* U+10CA */
  0x0002,   /* U+10CB */
  0x0002,   /* U+10CC */
  0x0008,   /* U+10CD */
  0x0002,   /* U+10CE */
  0x0002,   /* U+10CF */
  0x0002,   /* U+10D0 */
  0x0002,   /* U+10D1 */
  0x0002,   /* U+10D2 */
  0x0002,   /* U+10D3 */
  0x0002,   /* U+10D4 */
  0x0002,   /* U+10D5 */
  0x0002,   /* U+10D6 */
  0x0002,   /* U+10D7 */
  0x0002,   /* U+10D8 */
  0x0002,   /* U+10D9 */
  0x0002,   /* U+10DA */
  0x0002,   /* U+10DB */
  0x0002,   /* U+10DC */
  0x0002,   /* U+10DD */
  0x0002,   /* U+10DE */
  0x0002,   /* U+10DF */
  0x0002,   /* U+10E0 */
  0x0002,   /* U+10E1 */
  0x0002,   /* U+10E2 */
  0x0002,   /* U+10E3 */
  0x0002,   /* U+10E4 */
  0x0002,   /* U+10E5 */
  0x0002,   /* U+10E6 */
  0x0002,   /* U+10E7 */
  0x0002,   /* U+10E8 */
  0x0002,   /* U+10E9 */
  0x0002,   /* U+10EA */
  0x0002,   /* U+10EB */
  0x0002,   /* U+10EC */
  0x0002,   /* U+10ED */
  0x0002,   /* U+10EE */
  0x0002,   /* U+10EF */
  0x0002,   /* U+10F0 */
  0x0002,   /* U+10F1 */
  0x0002,   /* U+10F2 */
  0x0002,   /* U+10F3 */
  0x0002,   /* U+10F4 */
  0x0002,   /* U+10F5 */
  0x0002,   /* U+10F6 */
  0x0002,   /* U+10F7 */
  0x0002,   /* U+10F8 */
  0x0002,   /* U+10F9 */
  0x0002,   /* U+10FA */
  0x0002,   /* U+10FB */
  0x0014,   /* U+10FC */
  0x0002,   /* U+10FD */
  0x0002,   /* U+10FE */
  0x0002,   /* U+10FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x30CB,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x90C6,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x90C8,   /* U+10C8 */
  0x90C9,   /* U+10C9 */
  0x90CA,   /* U+10CA */
  0x90CB,   /* U+10CB */
  0x90CC,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x90CE,   /* U+10CE */
  0x90CF,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x0020,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x0000,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x0000,   /* U+10C8 */
  0x0000,   /* U+10C9 */
  0x0000,   /* U+10CA */
  0x0000,   /* U+10CB */
  0x0000,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x0000,   /* U+10CE */
  0x0000,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x0004,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x0000,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x0000,   /* U+10C8 */
  0x0000,   /* U+10C9 */
  0x0000,   /* U+10CA */
  0x0000,   /* U+10CB */
  0x0000,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x0000,   /* U+10CE */
  0x0000,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x3092,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x0000,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x0000,   /* U+10C8 */
  0x0000,   /* U+10C9 */
  0x0000,   /* U+10CA */
  0x0000,   /* U+10CB */
  0x0000,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x0000,   /* U+10CE */
  0x0000,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x0020,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x0000,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x0000,   /* U+10C8 */
  0x0000,   /* U+10C9 */
  0x0000,   /* U+10CA */
  0x0000,   /* U+10CB */
  0x0000,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x0000,   /* U+10CE */
  0x0000,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1000 */
  0x0000,   /* U+1001 */
  0x0000,   /* U+1002 */
  0x0000,   /* U+1003 */
  0x0000,   /* U+1004 */
  0x0000,   /* U+1005 */
  0x0000,   /* U+1006 */
  0x0000,   /* U+1007 */
  0x0000,   /* U+1008 */
  0x0000,   /* U+1009 */
  0x0000,   /* U+100A */
  0x0000,   /* U+100B */
  0x0000,   /* U+100C */
  0x0000,   /* U+100D */
  0x0000,   /* U+100E */
  0x0000,   /* U+100F */
  0x0000,   /* U+1010 */
  0x0000,   /* U+1011 */
  0x0000,   /* U+1012 */
  0x0000,   /* U+1013 */
  0x0000,   /* U+1014 */
  0x0000,   /* U+1015 */
  0x0000,   /* U+1016 */
  0x0000,   /* U+1017 */
  0x0000,   /* U+1018 */
  0x0000,   /* U+1019 */
  0x0000,   /* U+101A */
  0x0000,   /* U+101B */
  0x0000,   /* U+101C */
  0x0000,   /* U+101D */
  0x0000,   /* U+101E */
  0x0000,   /* U+101F */
  0x0000,   /* U+1020 */
  0x0000,   /* U+1021 */
  0x0000,   /* U+1022 */
  0x0000,   /* U+1023 */
  0x0000,   /* U+1024 */
  0x0000,   /* U+1025 */
  0x0000,   /* U+1026 */
  0x0000,   /* U+1027 */
  0x0000,   /* U+1028 */
  0x0000,   /* U+1029 */
  0x0000,   /* U+102A */
  0x0000,   /* U+102B */
  0x0000,   /* U+102C */
  0x0000,   /* U+102D */
  0x0000,   /* U+102E */
  0x0000,   /* U+102F */
  0x0000,   /* U+1030 */
  0x0000,   /* U+1031 */
  0x0000,   /* U+1032 */
  0x0000,   /* U+1033 */
  0x0000,   /* U+1034 */
  0x0000,   /* U+1035 */
  0x0000,   /* U+1036 */
  0x0000,   /* U+1037 */
  0x0000,   /* U+1038 */
  0x0000,   /* U+1039 */
  0x0000,   /* U+103A */
  0x0000,   /* U+103B */
  0x0000,   /* U+103C */
  0x0000,   /* U+103D */
  0x0000,   /* U+103E */
  0x0004,   /* U+103F */
  0x0000,   /* U+1040 */
  0x0000,   /* U+1041 */
  0x0000,   /* U+1042 */
  0x0000,   /* U+1043 */
  0x0000,   /* U+1044 */
  0x0000,   /* U+1045 */
  0x0000,   /* U+1046 */
  0x0000,   /* U+1047 */
  0x0000,   /* U+1048 */
  0x0000,   /* U+1049 */
  0x0000,   /* U+104A */
  0x0000,   /* U+104B */
  0x0000,   /* U+104C */
  0x0000,   /* U+104D */
  0x0000,   /* U+104E */
  0x0000,   /* U+104F */
  0x0000,   /* U+1050 */
  0x0000,   /* U+1051 */
  0x0000,   /* U+1052 */
  0x0000,   /* U+1053 */
  0x0000,   /* U+1054 */
  0x0000,   /* U+1055 */
  0x0000,   /* U+1056 */
  0x0000,   /* U+1057 */
  0x0000,   /* U+1058 */
  0x0000,   /* U+1059 */
  0x0000,   /* U+105A */
  0x0000,   /* U+105B */
  0x0000,   /* U+105C */
  0x0000,   /* U+105D */
  0x0000,   /* U+105E */
  0x0000,   /* U+105F */
  0x0000,   /* U+1060 */
  0x0000,   /* U+1061 */
  0x0000,   /* U+1062 */
  0x0000,   /* U+1063 */
  0x0000,   /* U+1064 */
  0x0000,   /* U+1065 */
  0x0000,   /* U+1066 */
  0x0000,   /* U+1067 */
  0x0000,   /* U+1068 */
  0x0000,   /* U+1069 */
  0x0000,   /* U+106A */
  0x0000,   /* U+106B */
  0x0000,   /* U+106C */
  0x0000,   /* U+106D */
  0x0000,   /* U+106E */
  0x0000,   /* U+106F */
  0x0000,   /* U+1070 */
  0x0000,   /* U+1071 */
  0x0000,   /* U+1072 */
  0x0000,   /* U+1073 */
  0x0000,   /* U+1074 */
  0x0000,   /* U+1075 */
  0x0000,   /* U+1076 */
  0x0000,   /* U+1077 */
  0x0000,   /* U+1078 */
  0x0000,   /* U+1079 */
  0x0000,   /* U+107A */
  0x0000,   /* U+107B */
  0x0000,   /* U+107C */
  0x0000,   /* U+107D */
  0x0000,   /* U+107E */
  0x0000,   /* U+107F */
  0x0000,   /* U+1080 */
  0x0000,   /* U+1081 */
  0x0000,   /* U+1082 */
  0x0000,   /* U+1083 */
  0x0000,   /* U+1084 */
  0x0000,   /* U+1085 */
  0x0000,   /* U+1086 */
  0x0000,   /* U+1087 */
  0x0000,   /* U+1088 */
  0x0000,   /* U+1089 */
  0x0000,   /* U+108A */
  0x0000,   /* U+108B */
  0x0000,   /* U+108C */
  0x0000,   /* U+108D */
  0x0000,   /* U+108E */
  0x0000,   /* U+108F */
  0x0000,   /* U+1090 */
  0x0000,   /* U+1091 */
  0x0000,   /* U+1092 */
  0x0000,   /* U+1093 */
  0x0000,   /* U+1094 */
  0x0000,   /* U+1095 */
  0x0000,   /* U+1096 */
  0x0000,   /* U+1097 */
  0x0000,   /* U+1098 */
  0x0000,   /* U+1099 */
  0x0000,   /* U+109A */
  0x0000,   /* U+109B */
  0x0000,   /* U+109C */
  0x0000,   /* U+109D */
  0x0000,   /* U+109E */
  0x0000,   /* U+109F */
  0x0000,   /* U+10A0 */
  0x0000,   /* U+10A1 */
  0x0000,   /* U+10A2 */
  0x0000,   /* U+10A3 */
  0x0000,   /* U+10A4 */
  0x0000,   /* U+10A5 */
  0x0000,   /* U+10A6 */
  0x0000,   /* U+10A7 */
  0x0000,   /* U+10A8 */
  0x0000,   /* U+10A9 */
  0x0000,   /* U+10AA */
  0x0000,   /* U+10AB */
  0x0000,   /* U+10AC */
  0x0000,   /* U+10AD */
  0x0000,   /* U+10AE */
  0x0000,   /* U+10AF */
  0x0000,   /* U+10B0 */
  0x0000,   /* U+10B1 */
  0x0000,   /* U+10B2 */
  0x0000,   /* U+10B3 */
  0x0000,   /* U+10B4 */
  0x0000,   /* U+10B5 */
  0x0000,   /* U+10B6 */
  0x0000,   /* U+10B7 */
  0x0000,   /* U+10B8 */
  0x0000,   /* U+10B9 */
  0x0000,   /* U+10BA */
  0x0000,   /* U+10BB */
  0x0000,   /* U+10BC */
  0x0000,   /* U+10BD */
  0x0000,   /* U+10BE */
  0x0000,   /* U+10BF */
  0x0000,   /* U+10C0 */
  0x0000,   /* U+10C1 */
  0x0000,   /* U+10C2 */
  0x0000,   /* U+10C3 */
  0x0000,   /* U+10C4 */
  0x0000,   /* U+10C5 */
  0x0000,   /* U+10C6 */
  0x0000,   /* U+10C7 */
  0x0000,   /* U+10C8 */
  0x0000,   /* U+10C9 */
  0x0000,   /* U+10CA */
  0x0000,   /* U+10CB */
  0x0000,   /* U+10CC */
  0x0000,   /* U+10CD */
  0x0000,   /* U+10CE */
  0x0000,   /* U+10CF */
  0x0000,   /* U+10D0 */
  0x0000,   /* U+10D1 */
  0x0000,   /* U+10D2 */
  0x0000,   /* U+10D3 */
  0x0000,   /* U+10D4 */
  0x0000,   /* U+10D5 */
  0x0000,   /* U+10D6 */
  0x0000,   /* U+10D7 */
  0x0000,   /* U+10D8 */
  0x0000,   /* U+10D9 */
  0x0000,   /* U+10DA */
  0x0000,   /* U+10DB */
  0x0000,   /* U+10DC */
  0x0000,   /* U+10DD */
  0x0000,   /* U+10DE */
  0x0000,   /* U+10DF */
  0x0000,   /* U+10E0 */
  0x0000,   /* U+10E1 */
  0x0000,   /* U+10E2 */
  0x0000,   /* U+10E3 */
  0x0000,   /* U+10E4 */
  0x0000,   /* U+10E5 */
  0x0000,   /* U+10E6 */
  0x0000,   /* U+10E7 */
  0x0000,   /* U+10E8 */
  0x0000,   /* U+10E9 */
  0x0000,   /* U+10EA */
  0x0000,   /* U+10EB */
  0x0000,   /* U+10EC */
  0x0000,   /* U+10ED */
  0x0000,   /* U+10EE */
  0x0000,   /* U+10EF */
  0x0000,   /* U+10F0 */
  0x0000,   /* U+10F1 */
  0x0000,   /* U+10F2 */
  0x0000,   /* U+10F3 */
  0x0000,   /* U+10F4 */
  0x0000,   /* U+10F5 */
  0x0000,   /* U+10F6 */
  0x0000,   /* U+10F7 */
  0x0000,   /* U+10F8 */
  0x0000,   /* U+10F9 */
  0x0000,   /* U+10FA */
  0x0000,   /* U+10FB */
  0x0000,   /* U+10FC */
  0x0000,   /* U+10FD */
  0x0000,   /* U+10FE */
  0x0000,   /* U+10FF */
};

uint16 uca900_p011[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x3BF5,   /* U+1100 */
  0x3BF6,   /* U+1101 */
  0x3BF7,   /* U+1102 */
  0x3BF8,   /* U+1103 */
  0x3BF9,   /* U+1104 */
  0x3BFA,   /* U+1105 */
  0x3BFB,   /* U+1106 */
  0x3BFC,   /* U+1107 */
  0x3BFD,   /* U+1108 */
  0x3BFE,   /* U+1109 */
  0x3BFF,   /* U+110A */
  0x3C00,   /* U+110B */
  0x3C01,   /* U+110C */
  0x3C02,   /* U+110D */
  0x3C03,   /* U+110E */
  0x3C04,   /* U+110F */
  0x3C05,   /* U+1110 */
  0x3C06,   /* U+1111 */
  0x3C07,   /* U+1112 */
  0x3C08,   /* U+1113 */
  0x3C09,   /* U+1114 */
  0x3C0A,   /* U+1115 */
  0x3C0B,   /* U+1116 */
  0x3C0C,   /* U+1117 */
  0x3C0D,   /* U+1118 */
  0x3C0E,   /* U+1119 */
  0x3C0F,   /* U+111A */
  0x3C10,   /* U+111B */
  0x3C11,   /* U+111C */
  0x3C12,   /* U+111D */
  0x3C13,   /* U+111E */
  0x3C14,   /* U+111F */
  0x3C15,   /* U+1120 */
  0x3C16,   /* U+1121 */
  0x3C17,   /* U+1122 */
  0x3C18,   /* U+1123 */
  0x3C19,   /* U+1124 */
  0x3C1A,   /* U+1125 */
  0x3C1B,   /* U+1126 */
  0x3C1C,   /* U+1127 */
  0x3C1D,   /* U+1128 */
  0x3C1E,   /* U+1129 */
  0x3C1F,   /* U+112A */
  0x3C20,   /* U+112B */
  0x3C21,   /* U+112C */
  0x3C22,   /* U+112D */
  0x3C23,   /* U+112E */
  0x3C24,   /* U+112F */
  0x3C25,   /* U+1130 */
  0x3C26,   /* U+1131 */
  0x3C27,   /* U+1132 */
  0x3C28,   /* U+1133 */
  0x3C29,   /* U+1134 */
  0x3C2A,   /* U+1135 */
  0x3C2B,   /* U+1136 */
  0x3C2C,   /* U+1137 */
  0x3C2D,   /* U+1138 */
  0x3C2E,   /* U+1139 */
  0x3C2F,   /* U+113A */
  0x3C30,   /* U+113B */
  0x3C31,   /* U+113C */
  0x3C32,   /* U+113D */
  0x3C33,   /* U+113E */
  0x3C34,   /* U+113F */
  0x3C35,   /* U+1140 */
  0x3C36,   /* U+1141 */
  0x3C37,   /* U+1142 */
  0x3C38,   /* U+1143 */
  0x3C39,   /* U+1144 */
  0x3C3A,   /* U+1145 */
  0x3C3B,   /* U+1146 */
  0x3C3C,   /* U+1147 */
  0x3C3D,   /* U+1148 */
  0x3C3E,   /* U+1149 */
  0x3C3F,   /* U+114A */
  0x3C40,   /* U+114B */
  0x3C41,   /* U+114C */
  0x3C42,   /* U+114D */
  0x3C43,   /* U+114E */
  0x3C44,   /* U+114F */
  0x3C45,   /* U+1150 */
  0x3C46,   /* U+1151 */
  0x3C47,   /* U+1152 */
  0x3C48,   /* U+1153 */
  0x3C49,   /* U+1154 */
  0x3C4A,   /* U+1155 */
  0x3C4B,   /* U+1156 */
  0x3C4C,   /* U+1157 */
  0x3C4D,   /* U+1158 */
  0x3C4E,   /* U+1159 */
  0x3C4F,   /* U+115A */
  0x3C50,   /* U+115B */
  0x3C51,   /* U+115C */
  0x3C52,   /* U+115D */
  0x3C53,   /* U+115E */
  0x3C71,   /* U+115F */
  0x3C72,   /* U+1160 */
  0x3C73,   /* U+1161 */
  0x3C74,   /* U+1162 */
  0x3C75,   /* U+1163 */
  0x3C76,   /* U+1164 */
  0x3C77,   /* U+1165 */
  0x3C78,   /* U+1166 */
  0x3C79,   /* U+1167 */
  0x3C7A,   /* U+1168 */
  0x3C7B,   /* U+1169 */
  0x3C7C,   /* U+116A */
  0x3C7D,   /* U+116B */
  0x3C7E,   /* U+116C */
  0x3C7F,   /* U+116D */
  0x3C80,   /* U+116E */
  0x3C81,   /* U+116F */
  0x3C82,   /* U+1170 */
  0x3C83,   /* U+1171 */
  0x3C84,   /* U+1172 */
  0x3C85,   /* U+1173 */
  0x3C86,   /* U+1174 */
  0x3C87,   /* U+1175 */
  0x3C88,   /* U+1176 */
  0x3C89,   /* U+1177 */
  0x3C8A,   /* U+1178 */
  0x3C8B,   /* U+1179 */
  0x3C8C,   /* U+117A */
  0x3C8D,   /* U+117B */
  0x3C8E,   /* U+117C */
  0x3C8F,   /* U+117D */
  0x3C90,   /* U+117E */
  0x3C91,   /* U+117F */
  0x3C92,   /* U+1180 */
  0x3C93,   /* U+1181 */
  0x3C94,   /* U+1182 */
  0x3C95,   /* U+1183 */
  0x3C96,   /* U+1184 */
  0x3C97,   /* U+1185 */
  0x3C98,   /* U+1186 */
  0x3C99,   /* U+1187 */
  0x3C9A,   /* U+1188 */
  0x3C9B,   /* U+1189 */
  0x3C9C,   /* U+118A */
  0x3C9D,   /* U+118B */
  0x3C9E,   /* U+118C */
  0x3C9F,   /* U+118D */
  0x3CA0,   /* U+118E */
  0x3CA1,   /* U+118F */
  0x3CA2,   /* U+1190 */
  0x3CA3,   /* U+1191 */
  0x3CA4,   /* U+1192 */
  0x3CA5,   /* U+1193 */
  0x3CA6,   /* U+1194 */
  0x3CA7,   /* U+1195 */
  0x3CA8,   /* U+1196 */
  0x3CA9,   /* U+1197 */
  0x3CAA,   /* U+1198 */
  0x3CAB,   /* U+1199 */
  0x3CAC,   /* U+119A */
  0x3CAD,   /* U+119B */
  0x3CAE,   /* U+119C */
  0x3CAF,   /* U+119D */
  0x3CB0,   /* U+119E */
  0x3CB1,   /* U+119F */
  0x3CB2,   /* U+11A0 */
  0x3CB3,   /* U+11A1 */
  0x3CB4,   /* U+11A2 */
  0x3CB5,   /* U+11A3 */
  0x3CB6,   /* U+11A4 */
  0x3CB7,   /* U+11A5 */
  0x3CB8,   /* U+11A6 */
  0x3CB9,   /* U+11A7 */
  0x3CD1,   /* U+11A8 */
  0x3CD2,   /* U+11A9 */
  0x3CD3,   /* U+11AA */
  0x3CD4,   /* U+11AB */
  0x3CD5,   /* U+11AC */
  0x3CD6,   /* U+11AD */
  0x3CD7,   /* U+11AE */
  0x3CD8,   /* U+11AF */
  0x3CD9,   /* U+11B0 */
  0x3CDA,   /* U+11B1 */
  0x3CDB,   /* U+11B2 */
  0x3CDC,   /* U+11B3 */
  0x3CDD,   /* U+11B4 */
  0x3CDE,   /* U+11B5 */
  0x3CDF,   /* U+11B6 */
  0x3CE0,   /* U+11B7 */
  0x3CE1,   /* U+11B8 */
  0x3CE2,   /* U+11B9 */
  0x3CE3,   /* U+11BA */
  0x3CE4,   /* U+11BB */
  0x3CE5,   /* U+11BC */
  0x3CE6,   /* U+11BD */
  0x3CE7,   /* U+11BE */
  0x3CE8,   /* U+11BF */
  0x3CE9,   /* U+11C0 */
  0x3CEA,   /* U+11C1 */
  0x3CEB,   /* U+11C2 */
  0x3CEC,   /* U+11C3 */
  0x3CED,   /* U+11C4 */
  0x3CEE,   /* U+11C5 */
  0x3CEF,   /* U+11C6 */
  0x3CF0,   /* U+11C7 */
  0x3CF1,   /* U+11C8 */
  0x3CF2,   /* U+11C9 */
  0x3CF3,   /* U+11CA */
  0x3CF4,   /* U+11CB */
  0x3CF5,   /* U+11CC */
  0x3CF6,   /* U+11CD */
  0x3CF7,   /* U+11CE */
  0x3CF8,   /* U+11CF */
  0x3CF9,   /* U+11D0 */
  0x3CFA,   /* U+11D1 */
  0x3CFB,   /* U+11D2 */
  0x3CFC,   /* U+11D3 */
  0x3CFD,   /* U+11D4 */
  0x3CFE,   /* U+11D5 */
  0x3CFF,   /* U+11D6 */
  0x3D00,   /* U+11D7 */
  0x3D01,   /* U+11D8 */
  0x3D02,   /* U+11D9 */
  0x3D03,   /* U+11DA */
  0x3D04,   /* U+11DB */
  0x3D05,   /* U+11DC */
  0x3D06,   /* U+11DD */
  0x3D07,   /* U+11DE */
  0x3D08,   /* U+11DF */
  0x3D09,   /* U+11E0 */
  0x3D0A,   /* U+11E1 */
  0x3D0B,   /* U+11E2 */
  0x3D0C,   /* U+11E3 */
  0x3D0D,   /* U+11E4 */
  0x3D0E,   /* U+11E5 */
  0x3D0F,   /* U+11E6 */
  0x3D10,   /* U+11E7 */
  0x3D11,   /* U+11E8 */
  0x3D12,   /* U+11E9 */
  0x3D13,   /* U+11EA */
  0x3D14,   /* U+11EB */
  0x3D15,   /* U+11EC */
  0x3D16,   /* U+11ED */
  0x3D17,   /* U+11EE */
  0x3D18,   /* U+11EF */
  0x3D19,   /* U+11F0 */
  0x3D1A,   /* U+11F1 */
  0x3D1B,   /* U+11F2 */
  0x3D1C,   /* U+11F3 */
  0x3D1D,   /* U+11F4 */
  0x3D1E,   /* U+11F5 */
  0x3D1F,   /* U+11F6 */
  0x3D20,   /* U+11F7 */
  0x3D21,   /* U+11F8 */
  0x3D22,   /* U+11F9 */
  0x3D23,   /* U+11FA */
  0x3D24,   /* U+11FB */
  0x3D25,   /* U+11FC */
  0x3D26,   /* U+11FD */
  0x3D27,   /* U+11FE */
  0x3D28,   /* U+11FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1100 */
  0x0020,   /* U+1101 */
  0x0020,   /* U+1102 */
  0x0020,   /* U+1103 */
  0x0020,   /* U+1104 */
  0x0020,   /* U+1105 */
  0x0020,   /* U+1106 */
  0x0020,   /* U+1107 */
  0x0020,   /* U+1108 */
  0x0020,   /* U+1109 */
  0x0020,   /* U+110A */
  0x0020,   /* U+110B */
  0x0020,   /* U+110C */
  0x0020,   /* U+110D */
  0x0020,   /* U+110E */
  0x0020,   /* U+110F */
  0x0020,   /* U+1110 */
  0x0020,   /* U+1111 */
  0x0020,   /* U+1112 */
  0x0020,   /* U+1113 */
  0x0020,   /* U+1114 */
  0x0020,   /* U+1115 */
  0x0020,   /* U+1116 */
  0x0020,   /* U+1117 */
  0x0020,   /* U+1118 */
  0x0020,   /* U+1119 */
  0x0020,   /* U+111A */
  0x0020,   /* U+111B */
  0x0020,   /* U+111C */
  0x0020,   /* U+111D */
  0x0020,   /* U+111E */
  0x0020,   /* U+111F */
  0x0020,   /* U+1120 */
  0x0020,   /* U+1121 */
  0x0020,   /* U+1122 */
  0x0020,   /* U+1123 */
  0x0020,   /* U+1124 */
  0x0020,   /* U+1125 */
  0x0020,   /* U+1126 */
  0x0020,   /* U+1127 */
  0x0020,   /* U+1128 */
  0x0020,   /* U+1129 */
  0x0020,   /* U+112A */
  0x0020,   /* U+112B */
  0x0020,   /* U+112C */
  0x0020,   /* U+112D */
  0x0020,   /* U+112E */
  0x0020,   /* U+112F */
  0x0020,   /* U+1130 */
  0x0020,   /* U+1131 */
  0x0020,   /* U+1132 */
  0x0020,   /* U+1133 */
  0x0020,   /* U+1134 */
  0x0020,   /* U+1135 */
  0x0020,   /* U+1136 */
  0x0020,   /* U+1137 */
  0x0020,   /* U+1138 */
  0x0020,   /* U+1139 */
  0x0020,   /* U+113A */
  0x0020,   /* U+113B */
  0x0020,   /* U+113C */
  0x0020,   /* U+113D */
  0x0020,   /* U+113E */
  0x0020,   /* U+113F */
  0x0020,   /* U+1140 */
  0x0020,   /* U+1141 */
  0x0020,   /* U+1142 */
  0x0020,   /* U+1143 */
  0x0020,   /* U+1144 */
  0x0020,   /* U+1145 */
  0x0020,   /* U+1146 */
  0x0020,   /* U+1147 */
  0x0020,   /* U+1148 */
  0x0020,   /* U+1149 */
  0x0020,   /* U+114A */
  0x0020,   /* U+114B */
  0x0020,   /* U+114C */
  0x0020,   /* U+114D */
  0x0020,   /* U+114E */
  0x0020,   /* U+114F */
  0x0020,   /* U+1150 */
  0x0020,   /* U+1151 */
  0x0020,   /* U+1152 */
  0x0020,   /* U+1153 */
  0x0020,   /* U+1154 */
  0x0020,   /* U+1155 */
  0x0020,   /* U+1156 */
  0x0020,   /* U+1157 */
  0x0020,   /* U+1158 */
  0x0020,   /* U+1159 */
  0x0020,   /* U+115A */
  0x0020,   /* U+115B */
  0x0020,   /* U+115C */
  0x0020,   /* U+115D */
  0x0020,   /* U+115E */
  0x0020,   /* U+115F */
  0x0020,   /* U+1160 */
  0x0020,   /* U+1161 */
  0x0020,   /* U+1162 */
  0x0020,   /* U+1163 */
  0x0020,   /* U+1164 */
  0x0020,   /* U+1165 */
  0x0020,   /* U+1166 */
  0x0020,   /* U+1167 */
  0x0020,   /* U+1168 */
  0x0020,   /* U+1169 */
  0x0020,   /* U+116A */
  0x0020,   /* U+116B */
  0x0020,   /* U+116C */
  0x0020,   /* U+116D */
  0x0020,   /* U+116E */
  0x0020,   /* U+116F */
  0x0020,   /* U+1170 */
  0x0020,   /* U+1171 */
  0x0020,   /* U+1172 */
  0x0020,   /* U+1173 */
  0x0020,   /* U+1174 */
  0x0020,   /* U+1175 */
  0x0020,   /* U+1176 */
  0x0020,   /* U+1177 */
  0x0020,   /* U+1178 */
  0x0020,   /* U+1179 */
  0x0020,   /* U+117A */
  0x0020,   /* U+117B */
  0x0020,   /* U+117C */
  0x0020,   /* U+117D */
  0x0020,   /* U+117E */
  0x0020,   /* U+117F */
  0x0020,   /* U+1180 */
  0x0020,   /* U+1181 */
  0x0020,   /* U+1182 */
  0x0020,   /* U+1183 */
  0x0020,   /* U+1184 */
  0x0020,   /* U+1185 */
  0x0020,   /* U+1186 */
  0x0020,   /* U+1187 */
  0x0020,   /* U+1188 */
  0x0020,   /* U+1189 */
  0x0020,   /* U+118A */
  0x0020,   /* U+118B */
  0x0020,   /* U+118C */
  0x0020,   /* U+118D */
  0x0020,   /* U+118E */
  0x0020,   /* U+118F */
  0x0020,   /* U+1190 */
  0x0020,   /* U+1191 */
  0x0020,   /* U+1192 */
  0x0020,   /* U+1193 */
  0x0020,   /* U+1194 */
  0x0020,   /* U+1195 */
  0x0020,   /* U+1196 */
  0x0020,   /* U+1197 */
  0x0020,   /* U+1198 */
  0x0020,   /* U+1199 */
  0x0020,   /* U+119A */
  0x0020,   /* U+119B */
  0x0020,   /* U+119C */
  0x0020,   /* U+119D */
  0x0020,   /* U+119E */
  0x0020,   /* U+119F */
  0x0020,   /* U+11A0 */
  0x0020,   /* U+11A1 */
  0x0020,   /* U+11A2 */
  0x0020,   /* U+11A3 */
  0x0020,   /* U+11A4 */
  0x0020,   /* U+11A5 */
  0x0020,   /* U+11A6 */
  0x0020,   /* U+11A7 */
  0x0020,   /* U+11A8 */
  0x0020,   /* U+11A9 */
  0x0020,   /* U+11AA */
  0x0020,   /* U+11AB */
  0x0020,   /* U+11AC */
  0x0020,   /* U+11AD */
  0x0020,   /* U+11AE */
  0x0020,   /* U+11AF */
  0x0020,   /* U+11B0 */
  0x0020,   /* U+11B1 */
  0x0020,   /* U+11B2 */
  0x0020,   /* U+11B3 */
  0x0020,   /* U+11B4 */
  0x0020,   /* U+11B5 */
  0x0020,   /* U+11B6 */
  0x0020,   /* U+11B7 */
  0x0020,   /* U+11B8 */
  0x0020,   /* U+11B9 */
  0x0020,   /* U+11BA */
  0x0020,   /* U+11BB */
  0x0020,   /* U+11BC */
  0x0020,   /* U+11BD */
  0x0020,   /* U+11BE */
  0x0020,   /* U+11BF */
  0x0020,   /* U+11C0 */
  0x0020,   /* U+11C1 */
  0x0020,   /* U+11C2 */
  0x0020,   /* U+11C3 */
  0x0020,   /* U+11C4 */
  0x0020,   /* U+11C5 */
  0x0020,   /* U+11C6 */
  0x0020,   /* U+11C7 */
  0x0020,   /* U+11C8 */
  0x0020,   /* U+11C9 */
  0x0020,   /* U+11CA */
  0x0020,   /* U+11CB */
  0x0020,   /* U+11CC */
  0x0020,   /* U+11CD */
  0x0020,   /* U+11CE */
  0x0020,   /* U+11CF */
  0x0020,   /* U+11D0 */
  0x0020,   /* U+11D1 */
  0x0020,   /* U+11D2 */
  0x0020,   /* U+11D3 */
  0x0020,   /* U+11D4 */
  0x0020,   /* U+11D5 */
  0x0020,   /* U+11D6 */
  0x0020,   /* U+11D7 */
  0x0020,   /* U+11D8 */
  0x0020,   /* U+11D9 */
  0x0020,   /* U+11DA */
  0x0020,   /* U+11DB */
  0x0020,   /* U+11DC */
  0x0020,   /* U+11DD */
  0x0020,   /* U+11DE */
  0x0020,   /* U+11DF */
  0x0020,   /* U+11E0 */
  0x0020,   /* U+11E1 */
  0x0020,   /* U+11E2 */
  0x0020,   /* U+11E3 */
  0x0020,   /* U+11E4 */
  0x0020,   /* U+11E5 */
  0x0020,   /* U+11E6 */
  0x0020,   /* U+11E7 */
  0x0020,   /* U+11E8 */
  0x0020,   /* U+11E9 */
  0x0020,   /* U+11EA */
  0x0020,   /* U+11EB */
  0x0020,   /* U+11EC */
  0x0020,   /* U+11ED */
  0x0020,   /* U+11EE */
  0x0020,   /* U+11EF */
  0x0020,   /* U+11F0 */
  0x0020,   /* U+11F1 */
  0x0020,   /* U+11F2 */
  0x0020,   /* U+11F3 */
  0x0020,   /* U+11F4 */
  0x0020,   /* U+11F5 */
  0x0020,   /* U+11F6 */
  0x0020,   /* U+11F7 */
  0x0020,   /* U+11F8 */
  0x0020,   /* U+11F9 */
  0x0020,   /* U+11FA */
  0x0020,   /* U+11FB */
  0x0020,   /* U+11FC */
  0x0020,   /* U+11FD */
  0x0020,   /* U+11FE */
  0x0020,   /* U+11FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1100 */
  0x0002,   /* U+1101 */
  0x0002,   /* U+1102 */
  0x0002,   /* U+1103 */
  0x0002,   /* U+1104 */
  0x0002,   /* U+1105 */
  0x0002,   /* U+1106 */
  0x0002,   /* U+1107 */
  0x0002,   /* U+1108 */
  0x0002,   /* U+1109 */
  0x0002,   /* U+110A */
  0x0002,   /* U+110B */
  0x0002,   /* U+110C */
  0x0002,   /* U+110D */
  0x0002,   /* U+110E */
  0x0002,   /* U+110F */
  0x0002,   /* U+1110 */
  0x0002,   /* U+1111 */
  0x0002,   /* U+1112 */
  0x0002,   /* U+1113 */
  0x0002,   /* U+1114 */
  0x0002,   /* U+1115 */
  0x0002,   /* U+1116 */
  0x0002,   /* U+1117 */
  0x0002,   /* U+1118 */
  0x0002,   /* U+1119 */
  0x0002,   /* U+111A */
  0x0002,   /* U+111B */
  0x0002,   /* U+111C */
  0x0002,   /* U+111D */
  0x0002,   /* U+111E */
  0x0002,   /* U+111F */
  0x0002,   /* U+1120 */
  0x0002,   /* U+1121 */
  0x0002,   /* U+1122 */
  0x0002,   /* U+1123 */
  0x0002,   /* U+1124 */
  0x0002,   /* U+1125 */
  0x0002,   /* U+1126 */
  0x0002,   /* U+1127 */
  0x0002,   /* U+1128 */
  0x0002,   /* U+1129 */
  0x0002,   /* U+112A */
  0x0002,   /* U+112B */
  0x0002,   /* U+112C */
  0x0002,   /* U+112D */
  0x0002,   /* U+112E */
  0x0002,   /* U+112F */
  0x0002,   /* U+1130 */
  0x0002,   /* U+1131 */
  0x0002,   /* U+1132 */
  0x0002,   /* U+1133 */
  0x0002,   /* U+1134 */
  0x0002,   /* U+1135 */
  0x0002,   /* U+1136 */
  0x0002,   /* U+1137 */
  0x0002,   /* U+1138 */
  0x0002,   /* U+1139 */
  0x0002,   /* U+113A */
  0x0002,   /* U+113B */
  0x0002,   /* U+113C */
  0x0002,   /* U+113D */
  0x0002,   /* U+113E */
  0x0002,   /* U+113F */
  0x0002,   /* U+1140 */
  0x0002,   /* U+1141 */
  0x0002,   /* U+1142 */
  0x0002,   /* U+1143 */
  0x0002,   /* U+1144 */
  0x0002,   /* U+1145 */
  0x0002,   /* U+1146 */
  0x0002,   /* U+1147 */
  0x0002,   /* U+1148 */
  0x0002,   /* U+1149 */
  0x0002,   /* U+114A */
  0x0002,   /* U+114B */
  0x0002,   /* U+114C */
  0x0002,   /* U+114D */
  0x0002,   /* U+114E */
  0x0002,   /* U+114F */
  0x0002,   /* U+1150 */
  0x0002,   /* U+1151 */
  0x0002,   /* U+1152 */
  0x0002,   /* U+1153 */
  0x0002,   /* U+1154 */
  0x0002,   /* U+1155 */
  0x0002,   /* U+1156 */
  0x0002,   /* U+1157 */
  0x0002,   /* U+1158 */
  0x0002,   /* U+1159 */
  0x0002,   /* U+115A */
  0x0002,   /* U+115B */
  0x0002,   /* U+115C */
  0x0002,   /* U+115D */
  0x0002,   /* U+115E */
  0x0002,   /* U+115F */
  0x0002,   /* U+1160 */
  0x0002,   /* U+1161 */
  0x0002,   /* U+1162 */
  0x0002,   /* U+1163 */
  0x0002,   /* U+1164 */
  0x0002,   /* U+1165 */
  0x0002,   /* U+1166 */
  0x0002,   /* U+1167 */
  0x0002,   /* U+1168 */
  0x0002,   /* U+1169 */
  0x0002,   /* U+116A */
  0x0002,   /* U+116B */
  0x0002,   /* U+116C */
  0x0002,   /* U+116D */
  0x0002,   /* U+116E */
  0x0002,   /* U+116F */
  0x0002,   /* U+1170 */
  0x0002,   /* U+1171 */
  0x0002,   /* U+1172 */
  0x0002,   /* U+1173 */
  0x0002,   /* U+1174 */
  0x0002,   /* U+1175 */
  0x0002,   /* U+1176 */
  0x0002,   /* U+1177 */
  0x0002,   /* U+1178 */
  0x0002,   /* U+1179 */
  0x0002,   /* U+117A */
  0x0002,   /* U+117B */
  0x0002,   /* U+117C */
  0x0002,   /* U+117D */
  0x0002,   /* U+117E */
  0x0002,   /* U+117F */
  0x0002,   /* U+1180 */
  0x0002,   /* U+1181 */
  0x0002,   /* U+1182 */
  0x0002,   /* U+1183 */
  0x0002,   /* U+1184 */
  0x0002,   /* U+1185 */
  0x0002,   /* U+1186 */
  0x0002,   /* U+1187 */
  0x0002,   /* U+1188 */
  0x0002,   /* U+1189 */
  0x0002,   /* U+118A */
  0x0002,   /* U+118B */
  0x0002,   /* U+118C */
  0x0002,   /* U+118D */
  0x0002,   /* U+118E */
  0x0002,   /* U+118F */
  0x0002,   /* U+1190 */
  0x0002,   /* U+1191 */
  0x0002,   /* U+1192 */
  0x0002,   /* U+1193 */
  0x0002,   /* U+1194 */
  0x0002,   /* U+1195 */
  0x0002,   /* U+1196 */
  0x0002,   /* U+1197 */
  0x0002,   /* U+1198 */
  0x0002,   /* U+1199 */
  0x0002,   /* U+119A */
  0x0002,   /* U+119B */
  0x0002,   /* U+119C */
  0x0002,   /* U+119D */
  0x0002,   /* U+119E */
  0x0002,   /* U+119F */
  0x0002,   /* U+11A0 */
  0x0002,   /* U+11A1 */
  0x0002,   /* U+11A2 */
  0x0002,   /* U+11A3 */
  0x0002,   /* U+11A4 */
  0x0002,   /* U+11A5 */
  0x0002,   /* U+11A6 */
  0x0002,   /* U+11A7 */
  0x0002,   /* U+11A8 */
  0x0002,   /* U+11A9 */
  0x0002,   /* U+11AA */
  0x0002,   /* U+11AB */
  0x0002,   /* U+11AC */
  0x0002,   /* U+11AD */
  0x0002,   /* U+11AE */
  0x0002,   /* U+11AF */
  0x0002,   /* U+11B0 */
  0x0002,   /* U+11B1 */
  0x0002,   /* U+11B2 */
  0x0002,   /* U+11B3 */
  0x0002,   /* U+11B4 */
  0x0002,   /* U+11B5 */
  0x0002,   /* U+11B6 */
  0x0002,   /* U+11B7 */
  0x0002,   /* U+11B8 */
  0x0002,   /* U+11B9 */
  0x0002,   /* U+11BA */
  0x0002,   /* U+11BB */
  0x0002,   /* U+11BC */
  0x0002,   /* U+11BD */
  0x0002,   /* U+11BE */
  0x0002,   /* U+11BF */
  0x0002,   /* U+11C0 */
  0x0002,   /* U+11C1 */
  0x0002,   /* U+11C2 */
  0x0002,   /* U+11C3 */
  0x0002,   /* U+11C4 */
  0x0002,   /* U+11C5 */
  0x0002,   /* U+11C6 */
  0x0002,   /* U+11C7 */
  0x0002,   /* U+11C8 */
  0x0002,   /* U+11C9 */
  0x0002,   /* U+11CA */
  0x0002,   /* U+11CB */
  0x0002,   /* U+11CC */
  0x0002,   /* U+11CD */
  0x0002,   /* U+11CE */
  0x0002,   /* U+11CF */
  0x0002,   /* U+11D0 */
  0x0002,   /* U+11D1 */
  0x0002,   /* U+11D2 */
  0x0002,   /* U+11D3 */
  0x0002,   /* U+11D4 */
  0x0002,   /* U+11D5 */
  0x0002,   /* U+11D6 */
  0x0002,   /* U+11D7 */
  0x0002,   /* U+11D8 */
  0x0002,   /* U+11D9 */
  0x0002,   /* U+11DA */
  0x0002,   /* U+11DB */
  0x0002,   /* U+11DC */
  0x0002,   /* U+11DD */
  0x0002,   /* U+11DE */
  0x0002,   /* U+11DF */
  0x0002,   /* U+11E0 */
  0x0002,   /* U+11E1 */
  0x0002,   /* U+11E2 */
  0x0002,   /* U+11E3 */
  0x0002,   /* U+11E4 */
  0x0002,   /* U+11E5 */
  0x0002,   /* U+11E6 */
  0x0002,   /* U+11E7 */
  0x0002,   /* U+11E8 */
  0x0002,   /* U+11E9 */
  0x0002,   /* U+11EA */
  0x0002,   /* U+11EB */
  0x0002,   /* U+11EC */
  0x0002,   /* U+11ED */
  0x0002,   /* U+11EE */
  0x0002,   /* U+11EF */
  0x0002,   /* U+11F0 */
  0x0002,   /* U+11F1 */
  0x0002,   /* U+11F2 */
  0x0002,   /* U+11F3 */
  0x0002,   /* U+11F4 */
  0x0002,   /* U+11F5 */
  0x0002,   /* U+11F6 */
  0x0002,   /* U+11F7 */
  0x0002,   /* U+11F8 */
  0x0002,   /* U+11F9 */
  0x0002,   /* U+11FA */
  0x0002,   /* U+11FB */
  0x0002,   /* U+11FC */
  0x0002,   /* U+11FD */
  0x0002,   /* U+11FE */
  0x0002,   /* U+11FF */
};

uint16 uca900_p012[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x2496,   /* U+1200 */
  0x2497,   /* U+1201 */
  0x2498,   /* U+1202 */
  0x2499,   /* U+1203 */
  0x249A,   /* U+1204 */
  0x249B,   /* U+1205 */
  0x249C,   /* U+1206 */
  0x249D,   /* U+1207 */
  0x249E,   /* U+1208 */
  0x249F,   /* U+1209 */
  0x24A0,   /* U+120A */
  0x24A1,   /* U+120B */
  0x24A2,   /* U+120C */
  0x24A3,   /* U+120D */
  0x24A4,   /* U+120E */
  0x24A5,   /* U+120F */
  0x24A7,   /* U+1210 */
  0x24A8,   /* U+1211 */
  0x24A9,   /* U+1212 */
  0x24AA,   /* U+1213 */
  0x24AB,   /* U+1214 */
  0x24AC,   /* U+1215 */
  0x24AD,   /* U+1216 */
  0x24AE,   /* U+1217 */
  0x24AF,   /* U+1218 */
  0x24B0,   /* U+1219 */
  0x24B1,   /* U+121A */
  0x24B2,   /* U+121B */
  0x24B3,   /* U+121C */
  0x24B4,   /* U+121D */
  0x24B5,   /* U+121E */
  0x24B6,   /* U+121F */
  0x24BC,   /* U+1220 */
  0x24BD,   /* U+1221 */
  0x24BE,   /* U+1222 */
  0x24BF,   /* U+1223 */
  0x24C0,   /* U+1224 */
  0x24C1,   /* U+1225 */
  0x24C2,   /* U+1226 */
  0x24C3,   /* U+1227 */
  0x24C4,   /* U+1228 */
  0x24C5,   /* U+1229 */
  0x24C6,   /* U+122A */
  0x24C7,   /* U+122B */
  0x24C8,   /* U+122C */
  0x24C9,   /* U+122D */
  0x24CA,   /* U+122E */
  0x24CB,   /* U+122F */
  0x24CD,   /* U+1230 */
  0x24CE,   /* U+1231 */
  0x24CF,   /* U+1232 */
  0x24D0,   /* U+1233 */
  0x24D1,   /* U+1234 */
  0x24D2,   /* U+1235 */
  0x24D3,   /* U+1236 */
  0x24D4,   /* U+1237 */
  0x24DC,   /* U+1238 */
  0x24DD,   /* U+1239 */
  0x24DE,   /* U+123A */
  0x24DF,   /* U+123B */
  0x24E0,   /* U+123C */
  0x24E1,   /* U+123D */
  0x24E2,   /* U+123E */
  0x24E3,   /* U+123F */
  0x24E5,   /* U+1240 */
  0x24E6,   /* U+1241 */
  0x24E7,   /* U+1242 */
  0x24E8,   /* U+1243 */
  0x24E9,   /* U+1244 */
  0x24EA,   /* U+1245 */
  0x24EB,   /* U+1246 */
  0x24EC,   /* U+1247 */
  0x24ED,   /* U+1248 */
  0xFBC0,   /* U+1249 */
  0x24EE,   /* U+124A */
  0x24EF,   /* U+124B */
  0x24F0,   /* U+124C */
  0x24F1,   /* U+124D */
  0xFBC0,   /* U+124E */
  0xFBC0,   /* U+124F */
  0x24F2,   /* U+1250 */
  0x24F3,   /* U+1251 */
  0x24F4,   /* U+1252 */
  0x24F5,   /* U+1253 */
  0x24F6,   /* U+1254 */
  0x24F7,   /* U+1255 */
  0x24F8,   /* U+1256 */
  0xFBC0,   /* U+1257 */
  0x24F9,   /* U+1258 */
  0xFBC0,   /* U+1259 */
  0x24FA,   /* U+125A */
  0x24FB,   /* U+125B */
  0x24FC,   /* U+125C */
  0x24FD,   /* U+125D */
  0xFBC0,   /* U+125E */
  0xFBC0,   /* U+125F */
  0x24FE,   /* U+1260 */
  0x24FF,   /* U+1261 */
  0x2500,   /* U+1262 */
  0x2501,   /* U+1263 */
  0x2502,   /* U+1264 */
  0x2503,   /* U+1265 */
  0x2504,   /* U+1266 */
  0x2505,   /* U+1267 */
  0x250B,   /* U+1268 */
  0x250C,   /* U+1269 */
  0x250D,   /* U+126A */
  0x250E,   /* U+126B */
  0x250F,   /* U+126C */
  0x2510,   /* U+126D */
  0x2511,   /* U+126E */
  0x2512,   /* U+126F */
  0x2513,   /* U+1270 */
  0x2514,   /* U+1271 */
  0x2515,   /* U+1272 */
  0x2516,   /* U+1273 */
  0x2517,   /* U+1274 */
  0x2518,   /* U+1275 */
  0x2519,   /* U+1276 */
  0x251A,   /* U+1277 */
  0x251C,   /* U+1278 */
  0x251D,   /* U+1279 */
  0x251E,   /* U+127A */
  0x251F,   /* U+127B */
  0x2520,   /* U+127C */
  0x2521,   /* U+127D */
  0x2522,   /* U+127E */
  0x2523,   /* U+127F */
  0x2525,   /* U+1280 */
  0x2526,   /* U+1281 */
  0x2527,   /* U+1282 */
  0x2528,   /* U+1283 */
  0x2529,   /* U+1284 */
  0x252A,   /* U+1285 */
  0x252B,   /* U+1286 */
  0x252C,   /* U+1287 */
  0x252D,   /* U+1288 */
  0xFBC0,   /* U+1289 */
  0x252E,   /* U+128A */
  0x252F,   /* U+128B */
  0x2530,   /* U+128C */
  0x2531,   /* U+128D */
  0xFBC0,   /* U+128E */
  0xFBC0,   /* U+128F */
  0x2532,   /* U+1290 */
  0x2533,   /* U+1291 */
  0x2534,   /* U+1292 */
  0x2535,   /* U+1293 */
  0x2536,   /* U+1294 */
  0x2537,   /* U+1295 */
  0x2538,   /* U+1296 */
  0x2539,   /* U+1297 */
  0x253B,   /* U+1298 */
  0x253C,   /* U+1299 */
  0x253D,   /* U+129A */
  0x253E,   /* U+129B */
  0x253F,   /* U+129C */
  0x2540,   /* U+129D */
  0x2541,   /* U+129E */
  0x2542,   /* U+129F */
  0x2544,   /* U+12A0 */
  0x2545,   /* U+12A1 */
  0x2546,   /* U+12A2 */
  0x2547,   /* U+12A3 */
  0x2548,   /* U+12A4 */
  0x2549,   /* U+12A5 */
  0x254A,   /* U+12A6 */
  0x254B,   /* U+12A7 */
  0x254D,   /* U+12A8 */
  0x254E,   /* U+12A9 */
  0x254F,   /* U+12AA */
  0x2550,   /* U+12AB */
  0x2551,   /* U+12AC */
  0x2552,   /* U+12AD */
  0x2553,   /* U+12AE */
  0x2554,   /* U+12AF */
  0x2555,   /* U+12B0 */
  0xFBC0,   /* U+12B1 */
  0x2556,   /* U+12B2 */
  0x2557,   /* U+12B3 */
  0x2558,   /* U+12B4 */
  0x2559,   /* U+12B5 */
  0xFBC0,   /* U+12B6 */
  0xFBC0,   /* U+12B7 */
  0x255A,   /* U+12B8 */
  0x255B,   /* U+12B9 */
  0x255C,   /* U+12BA */
  0x255D,   /* U+12BB */
  0x255E,   /* U+12BC */
  0x255F,   /* U+12BD */
  0x2560,   /* U+12BE */
  0xFBC0,   /* U+12BF */
  0x2561,   /* U+12C0 */
  0xFBC0,   /* U+12C1 */
  0x2562,   /* U+12C2 */
  0x2563,   /* U+12C3 */
  0x2564,   /* U+12C4 */
  0x2565,   /* U+12C5 */
  0xFBC0,   /* U+12C6 */
  0xFBC0,   /* U+12C7 */
  0x2566,   /* U+12C8 */
  0x2567,   /* U+12C9 */
  0x2568,   /* U+12CA */
  0x2569,   /* U+12CB */
  0x256A,   /* U+12CC */
  0x256B,   /* U+12CD */
  0x256C,   /* U+12CE */
  0x256D,   /* U+12CF */
  0x256E,   /* U+12D0 */
  0x256F,   /* U+12D1 */
  0x2570,   /* U+12D2 */
  0x2571,   /* U+12D3 */
  0x2572,   /* U+12D4 */
  0x2573,   /* U+12D5 */
  0x2574,   /* U+12D6 */
  0xFBC0,   /* U+12D7 */
  0x2575,   /* U+12D8 */
  0x2576,   /* U+12D9 */
  0x2577,   /* U+12DA */
  0x2578,   /* U+12DB */
  0x2579,   /* U+12DC */
  0x257A,   /* U+12DD */
  0x257B,   /* U+12DE */
  0x257C,   /* U+12DF */
  0x2584,   /* U+12E0 */
  0x2585,   /* U+12E1 */
  0x2586,   /* U+12E2 */
  0x2587,   /* U+12E3 */
  0x2588,   /* U+12E4 */
  0x2589,   /* U+12E5 */
  0x258A,   /* U+12E6 */
  0x258B,   /* U+12E7 */
  0x258C,   /* U+12E8 */
  0x258D,   /* U+12E9 */
  0x258E,   /* U+12EA */
  0x258F,   /* U+12EB */
  0x2590,   /* U+12EC */
  0x2591,   /* U+12ED */
  0x2592,   /* U+12EE */
  0x2593,   /* U+12EF */
  0x2594,   /* U+12F0 */
  0x2595,   /* U+12F1 */
  0x2596,   /* U+12F2 */
  0x2597,   /* U+12F3 */
  0x2598,   /* U+12F4 */
  0x2599,   /* U+12F5 */
  0x259A,   /* U+12F6 */
  0x259B,   /* U+12F7 */
  0x25A3,   /* U+12F8 */
  0x25A4,   /* U+12F9 */
  0x25A5,   /* U+12FA */
  0x25A6,   /* U+12FB */
  0x25A7,   /* U+12FC */
  0x25A8,   /* U+12FD */
  0x25A9,   /* U+12FE */
  0x25AA,   /* U+12FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1200 */
  0x0020,   /* U+1201 */
  0x0020,   /* U+1202 */
  0x0020,   /* U+1203 */
  0x0020,   /* U+1204 */
  0x0020,   /* U+1205 */
  0x0020,   /* U+1206 */
  0x0020,   /* U+1207 */
  0x0020,   /* U+1208 */
  0x0020,   /* U+1209 */
  0x0020,   /* U+120A */
  0x0020,   /* U+120B */
  0x0020,   /* U+120C */
  0x0020,   /* U+120D */
  0x0020,   /* U+120E */
  0x0020,   /* U+120F */
  0x0020,   /* U+1210 */
  0x0020,   /* U+1211 */
  0x0020,   /* U+1212 */
  0x0020,   /* U+1213 */
  0x0020,   /* U+1214 */
  0x0020,   /* U+1215 */
  0x0020,   /* U+1216 */
  0x0020,   /* U+1217 */
  0x0020,   /* U+1218 */
  0x0020,   /* U+1219 */
  0x0020,   /* U+121A */
  0x0020,   /* U+121B */
  0x0020,   /* U+121C */
  0x0020,   /* U+121D */
  0x0020,   /* U+121E */
  0x0020,   /* U+121F */
  0x0020,   /* U+1220 */
  0x0020,   /* U+1221 */
  0x0020,   /* U+1222 */
  0x0020,   /* U+1223 */
  0x0020,   /* U+1224 */
  0x0020,   /* U+1225 */
  0x0020,   /* U+1226 */
  0x0020,   /* U+1227 */
  0x0020,   /* U+1228 */
  0x0020,   /* U+1229 */
  0x0020,   /* U+122A */
  0x0020,   /* U+122B */
  0x0020,   /* U+122C */
  0x0020,   /* U+122D */
  0x0020,   /* U+122E */
  0x0020,   /* U+122F */
  0x0020,   /* U+1230 */
  0x0020,   /* U+1231 */
  0x0020,   /* U+1232 */
  0x0020,   /* U+1233 */
  0x0020,   /* U+1234 */
  0x0020,   /* U+1235 */
  0x0020,   /* U+1236 */
  0x0020,   /* U+1237 */
  0x0020,   /* U+1238 */
  0x0020,   /* U+1239 */
  0x0020,   /* U+123A */
  0x0020,   /* U+123B */
  0x0020,   /* U+123C */
  0x0020,   /* U+123D */
  0x0020,   /* U+123E */
  0x0020,   /* U+123F */
  0x0020,   /* U+1240 */
  0x0020,   /* U+1241 */
  0x0020,   /* U+1242 */
  0x0020,   /* U+1243 */
  0x0020,   /* U+1244 */
  0x0020,   /* U+1245 */
  0x0020,   /* U+1246 */
  0x0020,   /* U+1247 */
  0x0020,   /* U+1248 */
  0x0020,   /* U+1249 */
  0x0020,   /* U+124A */
  0x0020,   /* U+124B */
  0x0020,   /* U+124C */
  0x0020,   /* U+124D */
  0x0020,   /* U+124E */
  0x0020,   /* U+124F */
  0x0020,   /* U+1250 */
  0x0020,   /* U+1251 */
  0x0020,   /* U+1252 */
  0x0020,   /* U+1253 */
  0x0020,   /* U+1254 */
  0x0020,   /* U+1255 */
  0x0020,   /* U+1256 */
  0x0020,   /* U+1257 */
  0x0020,   /* U+1258 */
  0x0020,   /* U+1259 */
  0x0020,   /* U+125A */
  0x0020,   /* U+125B */
  0x0020,   /* U+125C */
  0x0020,   /* U+125D */
  0x0020,   /* U+125E */
  0x0020,   /* U+125F */
  0x0020,   /* U+1260 */
  0x0020,   /* U+1261 */
  0x0020,   /* U+1262 */
  0x0020,   /* U+1263 */
  0x0020,   /* U+1264 */
  0x0020,   /* U+1265 */
  0x0020,   /* U+1266 */
  0x0020,   /* U+1267 */
  0x0020,   /* U+1268 */
  0x0020,   /* U+1269 */
  0x0020,   /* U+126A */
  0x0020,   /* U+126B */
  0x0020,   /* U+126C */
  0x0020,   /* U+126D */
  0x0020,   /* U+126E */
  0x0020,   /* U+126F */
  0x0020,   /* U+1270 */
  0x0020,   /* U+1271 */
  0x0020,   /* U+1272 */
  0x0020,   /* U+1273 */
  0x0020,   /* U+1274 */
  0x0020,   /* U+1275 */
  0x0020,   /* U+1276 */
  0x0020,   /* U+1277 */
  0x0020,   /* U+1278 */
  0x0020,   /* U+1279 */
  0x0020,   /* U+127A */
  0x0020,   /* U+127B */
  0x0020,   /* U+127C */
  0x0020,   /* U+127D */
  0x0020,   /* U+127E */
  0x0020,   /* U+127F */
  0x0020,   /* U+1280 */
  0x0020,   /* U+1281 */
  0x0020,   /* U+1282 */
  0x0020,   /* U+1283 */
  0x0020,   /* U+1284 */
  0x0020,   /* U+1285 */
  0x0020,   /* U+1286 */
  0x0020,   /* U+1287 */
  0x0020,   /* U+1288 */
  0x0020,   /* U+1289 */
  0x0020,   /* U+128A */
  0x0020,   /* U+128B */
  0x0020,   /* U+128C */
  0x0020,   /* U+128D */
  0x0020,   /* U+128E */
  0x0020,   /* U+128F */
  0x0020,   /* U+1290 */
  0x0020,   /* U+1291 */
  0x0020,   /* U+1292 */
  0x0020,   /* U+1293 */
  0x0020,   /* U+1294 */
  0x0020,   /* U+1295 */
  0x0020,   /* U+1296 */
  0x0020,   /* U+1297 */
  0x0020,   /* U+1298 */
  0x0020,   /* U+1299 */
  0x0020,   /* U+129A */
  0x0020,   /* U+129B */
  0x0020,   /* U+129C */
  0x0020,   /* U+129D */
  0x0020,   /* U+129E */
  0x0020,   /* U+129F */
  0x0020,   /* U+12A0 */
  0x0020,   /* U+12A1 */
  0x0020,   /* U+12A2 */
  0x0020,   /* U+12A3 */
  0x0020,   /* U+12A4 */
  0x0020,   /* U+12A5 */
  0x0020,   /* U+12A6 */
  0x0020,   /* U+12A7 */
  0x0020,   /* U+12A8 */
  0x0020,   /* U+12A9 */
  0x0020,   /* U+12AA */
  0x0020,   /* U+12AB */
  0x0020,   /* U+12AC */
  0x0020,   /* U+12AD */
  0x0020,   /* U+12AE */
  0x0020,   /* U+12AF */
  0x0020,   /* U+12B0 */
  0x0020,   /* U+12B1 */
  0x0020,   /* U+12B2 */
  0x0020,   /* U+12B3 */
  0x0020,   /* U+12B4 */
  0x0020,   /* U+12B5 */
  0x0020,   /* U+12B6 */
  0x0020,   /* U+12B7 */
  0x0020,   /* U+12B8 */
  0x0020,   /* U+12B9 */
  0x0020,   /* U+12BA */
  0x0020,   /* U+12BB */
  0x0020,   /* U+12BC */
  0x0020,   /* U+12BD */
  0x0020,   /* U+12BE */
  0x0020,   /* U+12BF */
  0x0020,   /* U+12C0 */
  0x0020,   /* U+12C1 */
  0x0020,   /* U+12C2 */
  0x0020,   /* U+12C3 */
  0x0020,   /* U+12C4 */
  0x0020,   /* U+12C5 */
  0x0020,   /* U+12C6 */
  0x0020,   /* U+12C7 */
  0x0020,   /* U+12C8 */
  0x0020,   /* U+12C9 */
  0x0020,   /* U+12CA */
  0x0020,   /* U+12CB */
  0x0020,   /* U+12CC */
  0x0020,   /* U+12CD */
  0x0020,   /* U+12CE */
  0x0020,   /* U+12CF */
  0x0020,   /* U+12D0 */
  0x0020,   /* U+12D1 */
  0x0020,   /* U+12D2 */
  0x0020,   /* U+12D3 */
  0x0020,   /* U+12D4 */
  0x0020,   /* U+12D5 */
  0x0020,   /* U+12D6 */
  0x0020,   /* U+12D7 */
  0x0020,   /* U+12D8 */
  0x0020,   /* U+12D9 */
  0x0020,   /* U+12DA */
  0x0020,   /* U+12DB */
  0x0020,   /* U+12DC */
  0x0020,   /* U+12DD */
  0x0020,   /* U+12DE */
  0x0020,   /* U+12DF */
  0x0020,   /* U+12E0 */
  0x0020,   /* U+12E1 */
  0x0020,   /* U+12E2 */
  0x0020,   /* U+12E3 */
  0x0020,   /* U+12E4 */
  0x0020,   /* U+12E5 */
  0x0020,   /* U+12E6 */
  0x0020,   /* U+12E7 */
  0x0020,   /* U+12E8 */
  0x0020,   /* U+12E9 */
  0x0020,   /* U+12EA */
  0x0020,   /* U+12EB */
  0x0020,   /* U+12EC */
  0x0020,   /* U+12ED */
  0x0020,   /* U+12EE */
  0x0020,   /* U+12EF */
  0x0020,   /* U+12F0 */
  0x0020,   /* U+12F1 */
  0x0020,   /* U+12F2 */
  0x0020,   /* U+12F3 */
  0x0020,   /* U+12F4 */
  0x0020,   /* U+12F5 */
  0x0020,   /* U+12F6 */
  0x0020,   /* U+12F7 */
  0x0020,   /* U+12F8 */
  0x0020,   /* U+12F9 */
  0x0020,   /* U+12FA */
  0x0020,   /* U+12FB */
  0x0020,   /* U+12FC */
  0x0020,   /* U+12FD */
  0x0020,   /* U+12FE */
  0x0020,   /* U+12FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1200 */
  0x0002,   /* U+1201 */
  0x0002,   /* U+1202 */
  0x0002,   /* U+1203 */
  0x0002,   /* U+1204 */
  0x0002,   /* U+1205 */
  0x0002,   /* U+1206 */
  0x0002,   /* U+1207 */
  0x0002,   /* U+1208 */
  0x0002,   /* U+1209 */
  0x0002,   /* U+120A */
  0x0002,   /* U+120B */
  0x0002,   /* U+120C */
  0x0002,   /* U+120D */
  0x0002,   /* U+120E */
  0x0002,   /* U+120F */
  0x0002,   /* U+1210 */
  0x0002,   /* U+1211 */
  0x0002,   /* U+1212 */
  0x0002,   /* U+1213 */
  0x0002,   /* U+1214 */
  0x0002,   /* U+1215 */
  0x0002,   /* U+1216 */
  0x0002,   /* U+1217 */
  0x0002,   /* U+1218 */
  0x0002,   /* U+1219 */
  0x0002,   /* U+121A */
  0x0002,   /* U+121B */
  0x0002,   /* U+121C */
  0x0002,   /* U+121D */
  0x0002,   /* U+121E */
  0x0002,   /* U+121F */
  0x0002,   /* U+1220 */
  0x0002,   /* U+1221 */
  0x0002,   /* U+1222 */
  0x0002,   /* U+1223 */
  0x0002,   /* U+1224 */
  0x0002,   /* U+1225 */
  0x0002,   /* U+1226 */
  0x0002,   /* U+1227 */
  0x0002,   /* U+1228 */
  0x0002,   /* U+1229 */
  0x0002,   /* U+122A */
  0x0002,   /* U+122B */
  0x0002,   /* U+122C */
  0x0002,   /* U+122D */
  0x0002,   /* U+122E */
  0x0002,   /* U+122F */
  0x0002,   /* U+1230 */
  0x0002,   /* U+1231 */
  0x0002,   /* U+1232 */
  0x0002,   /* U+1233 */
  0x0002,   /* U+1234 */
  0x0002,   /* U+1235 */
  0x0002,   /* U+1236 */
  0x0002,   /* U+1237 */
  0x0002,   /* U+1238 */
  0x0002,   /* U+1239 */
  0x0002,   /* U+123A */
  0x0002,   /* U+123B */
  0x0002,   /* U+123C */
  0x0002,   /* U+123D */
  0x0002,   /* U+123E */
  0x0002,   /* U+123F */
  0x0002,   /* U+1240 */
  0x0002,   /* U+1241 */
  0x0002,   /* U+1242 */
  0x0002,   /* U+1243 */
  0x0002,   /* U+1244 */
  0x0002,   /* U+1245 */
  0x0002,   /* U+1246 */
  0x0002,   /* U+1247 */
  0x0002,   /* U+1248 */
  0x0002,   /* U+1249 */
  0x0002,   /* U+124A */
  0x0002,   /* U+124B */
  0x0002,   /* U+124C */
  0x0002,   /* U+124D */
  0x0002,   /* U+124E */
  0x0002,   /* U+124F */
  0x0002,   /* U+1250 */
  0x0002,   /* U+1251 */
  0x0002,   /* U+1252 */
  0x0002,   /* U+1253 */
  0x0002,   /* U+1254 */
  0x0002,   /* U+1255 */
  0x0002,   /* U+1256 */
  0x0002,   /* U+1257 */
  0x0002,   /* U+1258 */
  0x0002,   /* U+1259 */
  0x0002,   /* U+125A */
  0x0002,   /* U+125B */
  0x0002,   /* U+125C */
  0x0002,   /* U+125D */
  0x0002,   /* U+125E */
  0x0002,   /* U+125F */
  0x0002,   /* U+1260 */
  0x0002,   /* U+1261 */
  0x0002,   /* U+1262 */
  0x0002,   /* U+1263 */
  0x0002,   /* U+1264 */
  0x0002,   /* U+1265 */
  0x0002,   /* U+1266 */
  0x0002,   /* U+1267 */
  0x0002,   /* U+1268 */
  0x0002,   /* U+1269 */
  0x0002,   /* U+126A */
  0x0002,   /* U+126B */
  0x0002,   /* U+126C */
  0x0002,   /* U+126D */
  0x0002,   /* U+126E */
  0x0002,   /* U+126F */
  0x0002,   /* U+1270 */
  0x0002,   /* U+1271 */
  0x0002,   /* U+1272 */
  0x0002,   /* U+1273 */
  0x0002,   /* U+1274 */
  0x0002,   /* U+1275 */
  0x0002,   /* U+1276 */
  0x0002,   /* U+1277 */
  0x0002,   /* U+1278 */
  0x0002,   /* U+1279 */
  0x0002,   /* U+127A */
  0x0002,   /* U+127B */
  0x0002,   /* U+127C */
  0x0002,   /* U+127D */
  0x0002,   /* U+127E */
  0x0002,   /* U+127F */
  0x0002,   /* U+1280 */
  0x0002,   /* U+1281 */
  0x0002,   /* U+1282 */
  0x0002,   /* U+1283 */
  0x0002,   /* U+1284 */
  0x0002,   /* U+1285 */
  0x0002,   /* U+1286 */
  0x0002,   /* U+1287 */
  0x0002,   /* U+1288 */
  0x0002,   /* U+1289 */
  0x0002,   /* U+128A */
  0x0002,   /* U+128B */
  0x0002,   /* U+128C */
  0x0002,   /* U+128D */
  0x0002,   /* U+128E */
  0x0002,   /* U+128F */
  0x0002,   /* U+1290 */
  0x0002,   /* U+1291 */
  0x0002,   /* U+1292 */
  0x0002,   /* U+1293 */
  0x0002,   /* U+1294 */
  0x0002,   /* U+1295 */
  0x0002,   /* U+1296 */
  0x0002,   /* U+1297 */
  0x0002,   /* U+1298 */
  0x0002,   /* U+1299 */
  0x0002,   /* U+129A */
  0x0002,   /* U+129B */
  0x0002,   /* U+129C */
  0x0002,   /* U+129D */
  0x0002,   /* U+129E */
  0x0002,   /* U+129F */
  0x0002,   /* U+12A0 */
  0x0002,   /* U+12A1 */
  0x0002,   /* U+12A2 */
  0x0002,   /* U+12A3 */
  0x0002,   /* U+12A4 */
  0x0002,   /* U+12A5 */
  0x0002,   /* U+12A6 */
  0x0002,   /* U+12A7 */
  0x0002,   /* U+12A8 */
  0x0002,   /* U+12A9 */
  0x0002,   /* U+12AA */
  0x0002,   /* U+12AB */
  0x0002,   /* U+12AC */
  0x0002,   /* U+12AD */
  0x0002,   /* U+12AE */
  0x0002,   /* U+12AF */
  0x0002,   /* U+12B0 */
  0x0002,   /* U+12B1 */
  0x0002,   /* U+12B2 */
  0x0002,   /* U+12B3 */
  0x0002,   /* U+12B4 */
  0x0002,   /* U+12B5 */
  0x0002,   /* U+12B6 */
  0x0002,   /* U+12B7 */
  0x0002,   /* U+12B8 */
  0x0002,   /* U+12B9 */
  0x0002,   /* U+12BA */
  0x0002,   /* U+12BB */
  0x0002,   /* U+12BC */
  0x0002,   /* U+12BD */
  0x0002,   /* U+12BE */
  0x0002,   /* U+12BF */
  0x0002,   /* U+12C0 */
  0x0002,   /* U+12C1 */
  0x0002,   /* U+12C2 */
  0x0002,   /* U+12C3 */
  0x0002,   /* U+12C4 */
  0x0002,   /* U+12C5 */
  0x0002,   /* U+12C6 */
  0x0002,   /* U+12C7 */
  0x0002,   /* U+12C8 */
  0x0002,   /* U+12C9 */
  0x0002,   /* U+12CA */
  0x0002,   /* U+12CB */
  0x0002,   /* U+12CC */
  0x0002,   /* U+12CD */
  0x0002,   /* U+12CE */
  0x0002,   /* U+12CF */
  0x0002,   /* U+12D0 */
  0x0002,   /* U+12D1 */
  0x0002,   /* U+12D2 */
  0x0002,   /* U+12D3 */
  0x0002,   /* U+12D4 */
  0x0002,   /* U+12D5 */
  0x0002,   /* U+12D6 */
  0x0002,   /* U+12D7 */
  0x0002,   /* U+12D8 */
  0x0002,   /* U+12D9 */
  0x0002,   /* U+12DA */
  0x0002,   /* U+12DB */
  0x0002,   /* U+12DC */
  0x0002,   /* U+12DD */
  0x0002,   /* U+12DE */
  0x0002,   /* U+12DF */
  0x0002,   /* U+12E0 */
  0x0002,   /* U+12E1 */
  0x0002,   /* U+12E2 */
  0x0002,   /* U+12E3 */
  0x0002,   /* U+12E4 */
  0x0002,   /* U+12E5 */
  0x0002,   /* U+12E6 */
  0x0002,   /* U+12E7 */
  0x0002,   /* U+12E8 */
  0x0002,   /* U+12E9 */
  0x0002,   /* U+12EA */
  0x0002,   /* U+12EB */
  0x0002,   /* U+12EC */
  0x0002,   /* U+12ED */
  0x0002,   /* U+12EE */
  0x0002,   /* U+12EF */
  0x0002,   /* U+12F0 */
  0x0002,   /* U+12F1 */
  0x0002,   /* U+12F2 */
  0x0002,   /* U+12F3 */
  0x0002,   /* U+12F4 */
  0x0002,   /* U+12F5 */
  0x0002,   /* U+12F6 */
  0x0002,   /* U+12F7 */
  0x0002,   /* U+12F8 */
  0x0002,   /* U+12F9 */
  0x0002,   /* U+12FA */
  0x0002,   /* U+12FB */
  0x0002,   /* U+12FC */
  0x0002,   /* U+12FD */
  0x0002,   /* U+12FE */
  0x0002,   /* U+12FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1200 */
  0x0000,   /* U+1201 */
  0x0000,   /* U+1202 */
  0x0000,   /* U+1203 */
  0x0000,   /* U+1204 */
  0x0000,   /* U+1205 */
  0x0000,   /* U+1206 */
  0x0000,   /* U+1207 */
  0x0000,   /* U+1208 */
  0x0000,   /* U+1209 */
  0x0000,   /* U+120A */
  0x0000,   /* U+120B */
  0x0000,   /* U+120C */
  0x0000,   /* U+120D */
  0x0000,   /* U+120E */
  0x0000,   /* U+120F */
  0x0000,   /* U+1210 */
  0x0000,   /* U+1211 */
  0x0000,   /* U+1212 */
  0x0000,   /* U+1213 */
  0x0000,   /* U+1214 */
  0x0000,   /* U+1215 */
  0x0000,   /* U+1216 */
  0x0000,   /* U+1217 */
  0x0000,   /* U+1218 */
  0x0000,   /* U+1219 */
  0x0000,   /* U+121A */
  0x0000,   /* U+121B */
  0x0000,   /* U+121C */
  0x0000,   /* U+121D */
  0x0000,   /* U+121E */
  0x0000,   /* U+121F */
  0x0000,   /* U+1220 */
  0x0000,   /* U+1221 */
  0x0000,   /* U+1222 */
  0x0000,   /* U+1223 */
  0x0000,   /* U+1224 */
  0x0000,   /* U+1225 */
  0x0000,   /* U+1226 */
  0x0000,   /* U+1227 */
  0x0000,   /* U+1228 */
  0x0000,   /* U+1229 */
  0x0000,   /* U+122A */
  0x0000,   /* U+122B */
  0x0000,   /* U+122C */
  0x0000,   /* U+122D */
  0x0000,   /* U+122E */
  0x0000,   /* U+122F */
  0x0000,   /* U+1230 */
  0x0000,   /* U+1231 */
  0x0000,   /* U+1232 */
  0x0000,   /* U+1233 */
  0x0000,   /* U+1234 */
  0x0000,   /* U+1235 */
  0x0000,   /* U+1236 */
  0x0000,   /* U+1237 */
  0x0000,   /* U+1238 */
  0x0000,   /* U+1239 */
  0x0000,   /* U+123A */
  0x0000,   /* U+123B */
  0x0000,   /* U+123C */
  0x0000,   /* U+123D */
  0x0000,   /* U+123E */
  0x0000,   /* U+123F */
  0x0000,   /* U+1240 */
  0x0000,   /* U+1241 */
  0x0000,   /* U+1242 */
  0x0000,   /* U+1243 */
  0x0000,   /* U+1244 */
  0x0000,   /* U+1245 */
  0x0000,   /* U+1246 */
  0x0000,   /* U+1247 */
  0x0000,   /* U+1248 */
  0x9249,   /* U+1249 */
  0x0000,   /* U+124A */
  0x0000,   /* U+124B */
  0x0000,   /* U+124C */
  0x0000,   /* U+124D */
  0x924E,   /* U+124E */
  0x924F,   /* U+124F */
  0x0000,   /* U+1250 */
  0x0000,   /* U+1251 */
  0x0000,   /* U+1252 */
  0x0000,   /* U+1253 */
  0x0000,   /* U+1254 */
  0x0000,   /* U+1255 */
  0x0000,   /* U+1256 */
  0x9257,   /* U+1257 */
  0x0000,   /* U+1258 */
  0x9259,   /* U+1259 */
  0x0000,   /* U+125A */
  0x0000,   /* U+125B */
  0x0000,   /* U+125C */
  0x0000,   /* U+125D */
  0x925E,   /* U+125E */
  0x925F,   /* U+125F */
  0x0000,   /* U+1260 */
  0x0000,   /* U+1261 */
  0x0000,   /* U+1262 */
  0x0000,   /* U+1263 */
  0x0000,   /* U+1264 */
  0x0000,   /* U+1265 */
  0x0000,   /* U+1266 */
  0x0000,   /* U+1267 */
  0x0000,   /* U+1268 */
  0x0000,   /* U+1269 */
  0x0000,   /* U+126A */
  0x0000,   /* U+126B */
  0x0000,   /* U+126C */
  0x0000,   /* U+126D */
  0x0000,   /* U+126E */
  0x0000,   /* U+126F */
  0x0000,   /* U+1270 */
  0x0000,   /* U+1271 */
  0x0000,   /* U+1272 */
  0x0000,   /* U+1273 */
  0x0000,   /* U+1274 */
  0x0000,   /* U+1275 */
  0x0000,   /* U+1276 */
  0x0000,   /* U+1277 */
  0x0000,   /* U+1278 */
  0x0000,   /* U+1279 */
  0x0000,   /* U+127A */
  0x0000,   /* U+127B */
  0x0000,   /* U+127C */
  0x0000,   /* U+127D */
  0x0000,   /* U+127E */
  0x0000,   /* U+127F */
  0x0000,   /* U+1280 */
  0x0000,   /* U+1281 */
  0x0000,   /* U+1282 */
  0x0000,   /* U+1283 */
  0x0000,   /* U+1284 */
  0x0000,   /* U+1285 */
  0x0000,   /* U+1286 */
  0x0000,   /* U+1287 */
  0x0000,   /* U+1288 */
  0x9289,   /* U+1289 */
  0x0000,   /* U+128A */
  0x0000,   /* U+128B */
  0x0000,   /* U+128C */
  0x0000,   /* U+128D */
  0x928E,   /* U+128E */
  0x928F,   /* U+128F */
  0x0000,   /* U+1290 */
  0x0000,   /* U+1291 */
  0x0000,   /* U+1292 */
  0x0000,   /* U+1293 */
  0x0000,   /* U+1294 */
  0x0000,   /* U+1295 */
  0x0000,   /* U+1296 */
  0x0000,   /* U+1297 */
  0x0000,   /* U+1298 */
  0x0000,   /* U+1299 */
  0x0000,   /* U+129A */
  0x0000,   /* U+129B */
  0x0000,   /* U+129C */
  0x0000,   /* U+129D */
  0x0000,   /* U+129E */
  0x0000,   /* U+129F */
  0x0000,   /* U+12A0 */
  0x0000,   /* U+12A1 */
  0x0000,   /* U+12A2 */
  0x0000,   /* U+12A3 */
  0x0000,   /* U+12A4 */
  0x0000,   /* U+12A5 */
  0x0000,   /* U+12A6 */
  0x0000,   /* U+12A7 */
  0x0000,   /* U+12A8 */
  0x0000,   /* U+12A9 */
  0x0000,   /* U+12AA */
  0x0000,   /* U+12AB */
  0x0000,   /* U+12AC */
  0x0000,   /* U+12AD */
  0x0000,   /* U+12AE */
  0x0000,   /* U+12AF */
  0x0000,   /* U+12B0 */
  0x92B1,   /* U+12B1 */
  0x0000,   /* U+12B2 */
  0x0000,   /* U+12B3 */
  0x0000,   /* U+12B4 */
  0x0000,   /* U+12B5 */
  0x92B6,   /* U+12B6 */
  0x92B7,   /* U+12B7 */
  0x0000,   /* U+12B8 */
  0x0000,   /* U+12B9 */
  0x0000,   /* U+12BA */
  0x0000,   /* U+12BB */
  0x0000,   /* U+12BC */
  0x0000,   /* U+12BD */
  0x0000,   /* U+12BE */
  0x92BF,   /* U+12BF */
  0x0000,   /* U+12C0 */
  0x92C1,   /* U+12C1 */
  0x0000,   /* U+12C2 */
  0x0000,   /* U+12C3 */
  0x0000,   /* U+12C4 */
  0x0000,   /* U+12C5 */
  0x92C6,   /* U+12C6 */
  0x92C7,   /* U+12C7 */
  0x0000,   /* U+12C8 */
  0x0000,   /* U+12C9 */
  0x0000,   /* U+12CA */
  0x0000,   /* U+12CB */
  0x0000,   /* U+12CC */
  0x0000,   /* U+12CD */
  0x0000,   /* U+12CE */
  0x0000,   /* U+12CF */
  0x0000,   /* U+12D0 */
  0x0000,   /* U+12D1 */
  0x0000,   /* U+12D2 */
  0x0000,   /* U+12D3 */
  0x0000,   /* U+12D4 */
  0x0000,   /* U+12D5 */
  0x0000,   /* U+12D6 */
  0x92D7,   /* U+12D7 */
  0x0000,   /* U+12D8 */
  0x0000,   /* U+12D9 */
  0x0000,   /* U+12DA */
  0x0000,   /* U+12DB */
  0x0000,   /* U+12DC */
  0x0000,   /* U+12DD */
  0x0000,   /* U+12DE */
  0x0000,   /* U+12DF */
  0x0000,   /* U+12E0 */
  0x0000,   /* U+12E1 */
  0x0000,   /* U+12E2 */
  0x0000,   /* U+12E3 */
  0x0000,   /* U+12E4 */
  0x0000,   /* U+12E5 */
  0x0000,   /* U+12E6 */
  0x0000,   /* U+12E7 */
  0x0000,   /* U+12E8 */
  0x0000,   /* U+12E9 */
  0x0000,   /* U+12EA */
  0x0000,   /* U+12EB */
  0x0000,   /* U+12EC */
  0x0000,   /* U+12ED */
  0x0000,   /* U+12EE */
  0x0000,   /* U+12EF */
  0x0000,   /* U+12F0 */
  0x0000,   /* U+12F1 */
  0x0000,   /* U+12F2 */
  0x0000,   /* U+12F3 */
  0x0000,   /* U+12F4 */
  0x0000,   /* U+12F5 */
  0x0000,   /* U+12F6 */
  0x0000,   /* U+12F7 */
  0x0000,   /* U+12F8 */
  0x0000,   /* U+12F9 */
  0x0000,   /* U+12FA */
  0x0000,   /* U+12FB */
  0x0000,   /* U+12FC */
  0x0000,   /* U+12FD */
  0x0000,   /* U+12FE */
  0x0000,   /* U+12FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1200 */
  0x0000,   /* U+1201 */
  0x0000,   /* U+1202 */
  0x0000,   /* U+1203 */
  0x0000,   /* U+1204 */
  0x0000,   /* U+1205 */
  0x0000,   /* U+1206 */
  0x0000,   /* U+1207 */
  0x0000,   /* U+1208 */
  0x0000,   /* U+1209 */
  0x0000,   /* U+120A */
  0x0000,   /* U+120B */
  0x0000,   /* U+120C */
  0x0000,   /* U+120D */
  0x0000,   /* U+120E */
  0x0000,   /* U+120F */
  0x0000,   /* U+1210 */
  0x0000,   /* U+1211 */
  0x0000,   /* U+1212 */
  0x0000,   /* U+1213 */
  0x0000,   /* U+1214 */
  0x0000,   /* U+1215 */
  0x0000,   /* U+1216 */
  0x0000,   /* U+1217 */
  0x0000,   /* U+1218 */
  0x0000,   /* U+1219 */
  0x0000,   /* U+121A */
  0x0000,   /* U+121B */
  0x0000,   /* U+121C */
  0x0000,   /* U+121D */
  0x0000,   /* U+121E */
  0x0000,   /* U+121F */
  0x0000,   /* U+1220 */
  0x0000,   /* U+1221 */
  0x0000,   /* U+1222 */
  0x0000,   /* U+1223 */
  0x0000,   /* U+1224 */
  0x0000,   /* U+1225 */
  0x0000,   /* U+1226 */
  0x0000,   /* U+1227 */
  0x0000,   /* U+1228 */
  0x0000,   /* U+1229 */
  0x0000,   /* U+122A */
  0x0000,   /* U+122B */
  0x0000,   /* U+122C */
  0x0000,   /* U+122D */
  0x0000,   /* U+122E */
  0x0000,   /* U+122F */
  0x0000,   /* U+1230 */
  0x0000,   /* U+1231 */
  0x0000,   /* U+1232 */
  0x0000,   /* U+1233 */
  0x0000,   /* U+1234 */
  0x0000,   /* U+1235 */
  0x0000,   /* U+1236 */
  0x0000,   /* U+1237 */
  0x0000,   /* U+1238 */
  0x0000,   /* U+1239 */
  0x0000,   /* U+123A */
  0x0000,   /* U+123B */
  0x0000,   /* U+123C */
  0x0000,   /* U+123D */
  0x0000,   /* U+123E */
  0x0000,   /* U+123F */
  0x0000,   /* U+1240 */
  0x0000,   /* U+1241 */
  0x0000,   /* U+1242 */
  0x0000,   /* U+1243 */
  0x0000,   /* U+1244 */
  0x0000,   /* U+1245 */
  0x0000,   /* U+1246 */
  0x0000,   /* U+1247 */
  0x0000,   /* U+1248 */
  0x0000,   /* U+1249 */
  0x0000,   /* U+124A */
  0x0000,   /* U+124B */
  0x0000,   /* U+124C */
  0x0000,   /* U+124D */
  0x0000,   /* U+124E */
  0x0000,   /* U+124F */
  0x0000,   /* U+1250 */
  0x0000,   /* U+1251 */
  0x0000,   /* U+1252 */
  0x0000,   /* U+1253 */
  0x0000,   /* U+1254 */
  0x0000,   /* U+1255 */
  0x0000,   /* U+1256 */
  0x0000,   /* U+1257 */
  0x0000,   /* U+1258 */
  0x0000,   /* U+1259 */
  0x0000,   /* U+125A */
  0x0000,   /* U+125B */
  0x0000,   /* U+125C */
  0x0000,   /* U+125D */
  0x0000,   /* U+125E */
  0x0000,   /* U+125F */
  0x0000,   /* U+1260 */
  0x0000,   /* U+1261 */
  0x0000,   /* U+1262 */
  0x0000,   /* U+1263 */
  0x0000,   /* U+1264 */
  0x0000,   /* U+1265 */
  0x0000,   /* U+1266 */
  0x0000,   /* U+1267 */
  0x0000,   /* U+1268 */
  0x0000,   /* U+1269 */
  0x0000,   /* U+126A */
  0x0000,   /* U+126B */
  0x0000,   /* U+126C */
  0x0000,   /* U+126D */
  0x0000,   /* U+126E */
  0x0000,   /* U+126F */
  0x0000,   /* U+1270 */
  0x0000,   /* U+1271 */
  0x0000,   /* U+1272 */
  0x0000,   /* U+1273 */
  0x0000,   /* U+1274 */
  0x0000,   /* U+1275 */
  0x0000,   /* U+1276 */
  0x0000,   /* U+1277 */
  0x0000,   /* U+1278 */
  0x0000,   /* U+1279 */
  0x0000,   /* U+127A */
  0x0000,   /* U+127B */
  0x0000,   /* U+127C */
  0x0000,   /* U+127D */
  0x0000,   /* U+127E */
  0x0000,   /* U+127F */
  0x0000,   /* U+1280 */
  0x0000,   /* U+1281 */
  0x0000,   /* U+1282 */
  0x0000,   /* U+1283 */
  0x0000,   /* U+1284 */
  0x0000,   /* U+1285 */
  0x0000,   /* U+1286 */
  0x0000,   /* U+1287 */
  0x0000,   /* U+1288 */
  0x0000,   /* U+1289 */
  0x0000,   /* U+128A */
  0x0000,   /* U+128B */
  0x0000,   /* U+128C */
  0x0000,   /* U+128D */
  0x0000,   /* U+128E */
  0x0000,   /* U+128F */
  0x0000,   /* U+1290 */
  0x0000,   /* U+1291 */
  0x0000,   /* U+1292 */
  0x0000,   /* U+1293 */
  0x0000,   /* U+1294 */
  0x0000,   /* U+1295 */
  0x0000,   /* U+1296 */
  0x0000,   /* U+1297 */
  0x0000,   /* U+1298 */
  0x0000,   /* U+1299 */
  0x0000,   /* U+129A */
  0x0000,   /* U+129B */
  0x0000,   /* U+129C */
  0x0000,   /* U+129D */
  0x0000,   /* U+129E */
  0x0000,   /* U+129F */
  0x0000,   /* U+12A0 */
  0x0000,   /* U+12A1 */
  0x0000,   /* U+12A2 */
  0x0000,   /* U+12A3 */
  0x0000,   /* U+12A4 */
  0x0000,   /* U+12A5 */
  0x0000,   /* U+12A6 */
  0x0000,   /* U+12A7 */
  0x0000,   /* U+12A8 */
  0x0000,   /* U+12A9 */
  0x0000,   /* U+12AA */
  0x0000,   /* U+12AB */
  0x0000,   /* U+12AC */
  0x0000,   /* U+12AD */
  0x0000,   /* U+12AE */
  0x0000,   /* U+12AF */
  0x0000,   /* U+12B0 */
  0x0000,   /* U+12B1 */
  0x0000,   /* U+12B2 */
  0x0000,   /* U+12B3 */
  0x0000,   /* U+12B4 */
  0x0000,   /* U+12B5 */
  0x0000,   /* U+12B6 */
  0x0000,   /* U+12B7 */
  0x0000,   /* U+12B8 */
  0x0000,   /* U+12B9 */
  0x0000,   /* U+12BA */
  0x0000,   /* U+12BB */
  0x0000,   /* U+12BC */
  0x0000,   /* U+12BD */
  0x0000,   /* U+12BE */
  0x0000,   /* U+12BF */
  0x0000,   /* U+12C0 */
  0x0000,   /* U+12C1 */
  0x0000,   /* U+12C2 */
  0x0000,   /* U+12C3 */
  0x0000,   /* U+12C4 */
  0x0000,   /* U+12C5 */
  0x0000,   /* U+12C6 */
  0x0000,   /* U+12C7 */
  0x0000,   /* U+12C8 */
  0x0000,   /* U+12C9 */
  0x0000,   /* U+12CA */
  0x0000,   /* U+12CB */
  0x0000,   /* U+12CC */
  0x0000,   /* U+12CD */
  0x0000,   /* U+12CE */
  0x0000,   /* U+12CF */
  0x0000,   /* U+12D0 */
  0x0000,   /* U+12D1 */
  0x0000,   /* U+12D2 */
  0x0000,   /* U+12D3 */
  0x0000,   /* U+12D4 */
  0x0000,   /* U+12D5 */
  0x0000,   /* U+12D6 */
  0x0000,   /* U+12D7 */
  0x0000,   /* U+12D8 */
  0x0000,   /* U+12D9 */
  0x0000,   /* U+12DA */
  0x0000,   /* U+12DB */
  0x0000,   /* U+12DC */
  0x0000,   /* U+12DD */
  0x0000,   /* U+12DE */
  0x0000,   /* U+12DF */
  0x0000,   /* U+12E0 */
  0x0000,   /* U+12E1 */
  0x0000,   /* U+12E2 */
  0x0000,   /* U+12E3 */
  0x0000,   /* U+12E4 */
  0x0000,   /* U+12E5 */
  0x0000,   /* U+12E6 */
  0x0000,   /* U+12E7 */
  0x0000,   /* U+12E8 */
  0x0000,   /* U+12E9 */
  0x0000,   /* U+12EA */
  0x0000,   /* U+12EB */
  0x0000,   /* U+12EC */
  0x0000,   /* U+12ED */
  0x0000,   /* U+12EE */
  0x0000,   /* U+12EF */
  0x0000,   /* U+12F0 */
  0x0000,   /* U+12F1 */
  0x0000,   /* U+12F2 */
  0x0000,   /* U+12F3 */
  0x0000,   /* U+12F4 */
  0x0000,   /* U+12F5 */
  0x0000,   /* U+12F6 */
  0x0000,   /* U+12F7 */
  0x0000,   /* U+12F8 */
  0x0000,   /* U+12F9 */
  0x0000,   /* U+12FA */
  0x0000,   /* U+12FB */
  0x0000,   /* U+12FC */
  0x0000,   /* U+12FD */
  0x0000,   /* U+12FE */
  0x0000,   /* U+12FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1200 */
  0x0000,   /* U+1201 */
  0x0000,   /* U+1202 */
  0x0000,   /* U+1203 */
  0x0000,   /* U+1204 */
  0x0000,   /* U+1205 */
  0x0000,   /* U+1206 */
  0x0000,   /* U+1207 */
  0x0000,   /* U+1208 */
  0x0000,   /* U+1209 */
  0x0000,   /* U+120A */
  0x0000,   /* U+120B */
  0x0000,   /* U+120C */
  0x0000,   /* U+120D */
  0x0000,   /* U+120E */
  0x0000,   /* U+120F */
  0x0000,   /* U+1210 */
  0x0000,   /* U+1211 */
  0x0000,   /* U+1212 */
  0x0000,   /* U+1213 */
  0x0000,   /* U+1214 */
  0x0000,   /* U+1215 */
  0x0000,   /* U+1216 */
  0x0000,   /* U+1217 */
  0x0000,   /* U+1218 */
  0x0000,   /* U+1219 */
  0x0000,   /* U+121A */
  0x0000,   /* U+121B */
  0x0000,   /* U+121C */
  0x0000,   /* U+121D */
  0x0000,   /* U+121E */
  0x0000,   /* U+121F */
  0x0000,   /* U+1220 */
  0x0000,   /* U+1221 */
  0x0000,   /* U+1222 */
  0x0000,   /* U+1223 */
  0x0000,   /* U+1224 */
  0x0000,   /* U+1225 */
  0x0000,   /* U+1226 */
  0x0000,   /* U+1227 */
  0x0000,   /* U+1228 */
  0x0000,   /* U+1229 */
  0x0000,   /* U+122A */
  0x0000,   /* U+122B */
  0x0000,   /* U+122C */
  0x0000,   /* U+122D */
  0x0000,   /* U+122E */
  0x0000,   /* U+122F */
  0x0000,   /* U+1230 */
  0x0000,   /* U+1231 */
  0x0000,   /* U+1232 */
  0x0000,   /* U+1233 */
  0x0000,   /* U+1234 */
  0x0000,   /* U+1235 */
  0x0000,   /* U+1236 */
  0x0000,   /* U+1237 */
  0x0000,   /* U+1238 */
  0x0000,   /* U+1239 */
  0x0000,   /* U+123A */
  0x0000,   /* U+123B */
  0x0000,   /* U+123C */
  0x0000,   /* U+123D */
  0x0000,   /* U+123E */
  0x0000,   /* U+123F */
  0x0000,   /* U+1240 */
  0x0000,   /* U+1241 */
  0x0000,   /* U+1242 */
  0x0000,   /* U+1243 */
  0x0000,   /* U+1244 */
  0x0000,   /* U+1245 */
  0x0000,   /* U+1246 */
  0x0000,   /* U+1247 */
  0x0000,   /* U+1248 */
  0x0000,   /* U+1249 */
  0x0000,   /* U+124A */
  0x0000,   /* U+124B */
  0x0000,   /* U+124C */
  0x0000,   /* U+124D */
  0x0000,   /* U+124E */
  0x0000,   /* U+124F */
  0x0000,   /* U+1250 */
  0x0000,   /* U+1251 */
  0x0000,   /* U+1252 */
  0x0000,   /* U+1253 */
  0x0000,   /* U+1254 */
  0x0000,   /* U+1255 */
  0x0000,   /* U+1256 */
  0x0000,   /* U+1257 */
  0x0000,   /* U+1258 */
  0x0000,   /* U+1259 */
  0x0000,   /* U+125A */
  0x0000,   /* U+125B */
  0x0000,   /* U+125C */
  0x0000,   /* U+125D */
  0x0000,   /* U+125E */
  0x0000,   /* U+125F */
  0x0000,   /* U+1260 */
  0x0000,   /* U+1261 */
  0x0000,   /* U+1262 */
  0x0000,   /* U+1263 */
  0x0000,   /* U+1264 */
  0x0000,   /* U+1265 */
  0x0000,   /* U+1266 */
  0x0000,   /* U+1267 */
  0x0000,   /* U+1268 */
  0x0000,   /* U+1269 */
  0x0000,   /* U+126A */
  0x0000,   /* U+126B */
  0x0000,   /* U+126C */
  0x0000,   /* U+126D */
  0x0000,   /* U+126E */
  0x0000,   /* U+126F */
  0x0000,   /* U+1270 */
  0x0000,   /* U+1271 */
  0x0000,   /* U+1272 */
  0x0000,   /* U+1273 */
  0x0000,   /* U+1274 */
  0x0000,   /* U+1275 */
  0x0000,   /* U+1276 */
  0x0000,   /* U+1277 */
  0x0000,   /* U+1278 */
  0x0000,   /* U+1279 */
  0x0000,   /* U+127A */
  0x0000,   /* U+127B */
  0x0000,   /* U+127C */
  0x0000,   /* U+127D */
  0x0000,   /* U+127E */
  0x0000,   /* U+127F */
  0x0000,   /* U+1280 */
  0x0000,   /* U+1281 */
  0x0000,   /* U+1282 */
  0x0000,   /* U+1283 */
  0x0000,   /* U+1284 */
  0x0000,   /* U+1285 */
  0x0000,   /* U+1286 */
  0x0000,   /* U+1287 */
  0x0000,   /* U+1288 */
  0x0000,   /* U+1289 */
  0x0000,   /* U+128A */
  0x0000,   /* U+128B */
  0x0000,   /* U+128C */
  0x0000,   /* U+128D */
  0x0000,   /* U+128E */
  0x0000,   /* U+128F */
  0x0000,   /* U+1290 */
  0x0000,   /* U+1291 */
  0x0000,   /* U+1292 */
  0x0000,   /* U+1293 */
  0x0000,   /* U+1294 */
  0x0000,   /* U+1295 */
  0x0000,   /* U+1296 */
  0x0000,   /* U+1297 */
  0x0000,   /* U+1298 */
  0x0000,   /* U+1299 */
  0x0000,   /* U+129A */
  0x0000,   /* U+129B */
  0x0000,   /* U+129C */
  0x0000,   /* U+129D */
  0x0000,   /* U+129E */
  0x0000,   /* U+129F */
  0x0000,   /* U+12A0 */
  0x0000,   /* U+12A1 */
  0x0000,   /* U+12A2 */
  0x0000,   /* U+12A3 */
  0x0000,   /* U+12A4 */
  0x0000,   /* U+12A5 */
  0x0000,   /* U+12A6 */
  0x0000,   /* U+12A7 */
  0x0000,   /* U+12A8 */
  0x0000,   /* U+12A9 */
  0x0000,   /* U+12AA */
  0x0000,   /* U+12AB */
  0x0000,   /* U+12AC */
  0x0000,   /* U+12AD */
  0x0000,   /* U+12AE */
  0x0000,   /* U+12AF */
  0x0000,   /* U+12B0 */
  0x0000,   /* U+12B1 */
  0x0000,   /* U+12B2 */
  0x0000,   /* U+12B3 */
  0x0000,   /* U+12B4 */
  0x0000,   /* U+12B5 */
  0x0000,   /* U+12B6 */
  0x0000,   /* U+12B7 */
  0x0000,   /* U+12B8 */
  0x0000,   /* U+12B9 */
  0x0000,   /* U+12BA */
  0x0000,   /* U+12BB */
  0x0000,   /* U+12BC */
  0x0000,   /* U+12BD */
  0x0000,   /* U+12BE */
  0x0000,   /* U+12BF */
  0x0000,   /* U+12C0 */
  0x0000,   /* U+12C1 */
  0x0000,   /* U+12C2 */
  0x0000,   /* U+12C3 */
  0x0000,   /* U+12C4 */
  0x0000,   /* U+12C5 */
  0x0000,   /* U+12C6 */
  0x0000,   /* U+12C7 */
  0x0000,   /* U+12C8 */
  0x0000,   /* U+12C9 */
  0x0000,   /* U+12CA */
  0x0000,   /* U+12CB */
  0x0000,   /* U+12CC */
  0x0000,   /* U+12CD */
  0x0000,   /* U+12CE */
  0x0000,   /* U+12CF */
  0x0000,   /* U+12D0 */
  0x0000,   /* U+12D1 */
  0x0000,   /* U+12D2 */
  0x0000,   /* U+12D3 */
  0x0000,   /* U+12D4 */
  0x0000,   /* U+12D5 */
  0x0000,   /* U+12D6 */
  0x0000,   /* U+12D7 */
  0x0000,   /* U+12D8 */
  0x0000,   /* U+12D9 */
  0x0000,   /* U+12DA */
  0x0000,   /* U+12DB */
  0x0000,   /* U+12DC */
  0x0000,   /* U+12DD */
  0x0000,   /* U+12DE */
  0x0000,   /* U+12DF */
  0x0000,   /* U+12E0 */
  0x0000,   /* U+12E1 */
  0x0000,   /* U+12E2 */
  0x0000,   /* U+12E3 */
  0x0000,   /* U+12E4 */
  0x0000,   /* U+12E5 */
  0x0000,   /* U+12E6 */
  0x0000,   /* U+12E7 */
  0x0000,   /* U+12E8 */
  0x0000,   /* U+12E9 */
  0x0000,   /* U+12EA */
  0x0000,   /* U+12EB */
  0x0000,   /* U+12EC */
  0x0000,   /* U+12ED */
  0x0000,   /* U+12EE */
  0x0000,   /* U+12EF */
  0x0000,   /* U+12F0 */
  0x0000,   /* U+12F1 */
  0x0000,   /* U+12F2 */
  0x0000,   /* U+12F3 */
  0x0000,   /* U+12F4 */
  0x0000,   /* U+12F5 */
  0x0000,   /* U+12F6 */
  0x0000,   /* U+12F7 */
  0x0000,   /* U+12F8 */
  0x0000,   /* U+12F9 */
  0x0000,   /* U+12FA */
  0x0000,   /* U+12FB */
  0x0000,   /* U+12FC */
  0x0000,   /* U+12FD */
  0x0000,   /* U+12FE */
  0x0000,   /* U+12FF */
};

uint16 uca900_p013[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0x25AC,   /* U+1300 */
  0x25AD,   /* U+1301 */
  0x25AE,   /* U+1302 */
  0x25AF,   /* U+1303 */
  0x25B0,   /* U+1304 */
  0x25B1,   /* U+1305 */
  0x25B2,   /* U+1306 */
  0x25B3,   /* U+1307 */
  0x25B5,   /* U+1308 */
  0x25B6,   /* U+1309 */
  0x25B7,   /* U+130A */
  0x25B8,   /* U+130B */
  0x25B9,   /* U+130C */
  0x25BA,   /* U+130D */
  0x25BB,   /* U+130E */
  0x25BC,   /* U+130F */
  0x25BD,   /* U+1310 */
  0xFBC0,   /* U+1311 */
  0x25BE,   /* U+1312 */
  0x25BF,   /* U+1313 */
  0x25C0,   /* U+1314 */
  0x25C1,   /* U+1315 */
  0xFBC0,   /* U+1316 */
  0xFBC0,   /* U+1317 */
  0x25C2,   /* U+1318 */
  0x25C3,   /* U+1319 */
  0x25C4,   /* U+131A */
  0x25C5,   /* U+131B */
  0x25C6,   /* U+131C */
  0x25C7,   /* U+131D */
  0x25C8,   /* U+131E */
  0x25C9,   /* U+131F */
  0x25CE,   /* U+1320 */
  0x25CF,   /* U+1321 */
  0x25D0,   /* U+1322 */
  0x25D1,   /* U+1323 */
  0x25D2,   /* U+1324 */
  0x25D3,   /* U+1325 */
  0x25D4,   /* U+1326 */
  0x25D5,   /* U+1327 */
  0x25D7,   /* U+1328 */
  0x25D8,   /* U+1329 */
  0x25D9,   /* U+132A */
  0x25DA,   /* U+132B */
  0x25DB,   /* U+132C */
  0x25DC,   /* U+132D */
  0x25DD,   /* U+132E */
  0x25DE,   /* U+132F */
  0x25E7,   /* U+1330 */
  0x25E8,   /* U+1331 */
  0x25E9,   /* U+1332 */
  0x25EA,   /* U+1333 */
  0x25EB,   /* U+1334 */
  0x25EC,   /* U+1335 */
  0x25ED,   /* U+1336 */
  0x25EE,   /* U+1337 */
  0x25F0,   /* U+1338 */
  0x25F1,   /* U+1339 */
  0x25F2,   /* U+133A */
  0x25F3,   /* U+133B */
  0x25F4,   /* U+133C */
  0x25F5,   /* U+133D */
  0x25F6,   /* U+133E */
  0x25F7,   /* U+133F */
  0x25FF,   /* U+1340 */
  0x2600,   /* U+1341 */
  0x2601,   /* U+1342 */
  0x2602,   /* U+1343 */
  0x2603,   /* U+1344 */
  0x2604,   /* U+1345 */
  0x2605,   /* U+1346 */
  0x2606,   /* U+1347 */
  0x2607,   /* U+1348 */
  0x2608,   /* U+1349 */
  0x2609,   /* U+134A */
  0x260A,   /* U+134B */
  0x260B,   /* U+134C */
  0x260C,   /* U+134D */
  0x260D,   /* U+134E */
  0x260E,   /* U+134F */
  0x2613,   /* U+1350 */
  0x2614,   /* U+1351 */
  0x2615,   /* U+1352 */
  0x2616,   /* U+1353 */
  0x2617,   /* U+1354 */
  0x2618,   /* U+1355 */
  0x2619,   /* U+1356 */
  0x261A,   /* U+1357 */
  0x2620,   /* U+1358 */
  0x2621,   /* U+1359 */
  0x2622,   /* U+135A */
  0xFBC0,   /* U+135B */
  0xFBC0,   /* U+135C */
  0x0000,   /* U+135D */
  0x0000,   /* U+135E */
  0x0000,   /* U+135F */
  0x02C7,   /* U+1360 */
  0x0251,   /* U+1361 */
  0x027C,   /* U+1362 */
  0x0252,   /* U+1363 */
  0x0253,   /* U+1364 */
  0x0254,   /* U+1365 */
  0x0255,   /* U+1366 */
  0x026C,   /* U+1367 */
  0x02C8,   /* U+1368 */
  0x1C3E,   /* U+1369 */
  0x1C3F,   /* U+136A */
  0x1C40,   /* U+136B */
  0x1C41,   /* U+136C */
  0x1C42,   /* U+136D */
  0x1C43,   /* U+136E */
  0x1C44,   /* U+136F */
  0x1C45,   /* U+1370 */
  0x1C46,   /* U+1371 */
  0x1ABB,   /* U+1372 */
  0x1ABC,   /* U+1373 */
  0x1ABD,   /* U+1374 */
  0x1ABE,   /* U+1375 */
  0x1ABF,   /* U+1376 */
  0x1AC0,   /* U+1377 */
  0x1AC1,   /* U+1378 */
  0x1AC2,   /* U+1379 */
  0x1AC3,   /* U+137A */
  0x1AC4,   /* U+137B */
  0x1AC5,   /* U+137C */
  0xFBC0,   /* U+137D */
  0xFBC0,   /* U+137E */
  0xFBC0,   /* U+137F */
  0x24B7,   /* U+1380 */
  0x24B8,   /* U+1381 */
  0x24B9,   /* U+1382 */
  0x24BA,   /* U+1383 */
  0x2506,   /* U+1384 */
  0x2507,   /* U+1385 */
  0x2508,   /* U+1386 */
  0x2509,   /* U+1387 */
  0x260F,   /* U+1388 */
  0x2610,   /* U+1389 */
  0x2611,   /* U+138A */
  0x2612,   /* U+138B */
  0x261B,   /* U+138C */
  0x261C,   /* U+138D */
  0x261D,   /* U+138E */
  0x261E,   /* U+138F */
  0x04C6,   /* U+1390 */
  0x04C7,   /* U+1391 */
  0x04C8,   /* U+1392 */
  0x04C9,   /* U+1393 */
  0x04CA,   /* U+1394 */
  0x04CB,   /* U+1395 */
  0x04CC,   /* U+1396 */
  0x04CD,   /* U+1397 */
  0x04CE,   /* U+1398 */
  0x04CF,   /* U+1399 */
  0xFBC0,   /* U+139A */
  0xFBC0,   /* U+139B */
  0xFBC0,   /* U+139C */
  0xFBC0,   /* U+139D */
  0xFBC0,   /* U+139E */
  0xFBC0,   /* U+139F */
  0x337F,   /* U+13A0 */
  0x3380,   /* U+13A1 */
  0x3381,   /* U+13A2 */
  0x3382,   /* U+13A3 */
  0x3383,   /* U+13A4 */
  0x3384,   /* U+13A5 */
  0x3385,   /* U+13A6 */
  0x3386,   /* U+13A7 */
  0x3387,   /* U+13A8 */
  0x3388,   /* U+13A9 */
  0x3389,   /* U+13AA */
  0x338A,   /* U+13AB */
  0x338B,   /* U+13AC */
  0x338C,   /* U+13AD */
  0x338D,   /* U+13AE */
  0x338E,   /* U+13AF */
  0x338F,   /* U+13B0 */
  0x3390,   /* U+13B1 */
  0x3391,   /* U+13B2 */
  0x3392,   /* U+13B3 */
  0x3393,   /* U+13B4 */
  0x3394,   /* U+13B5 */
  0x3395,   /* U+13B6 */
  0x3396,   /* U+13B7 */
  0x3397,   /* U+13B8 */
  0x3398,   /* U+13B9 */
  0x3399,   /* U+13BA */
  0x339A,   /* U+13BB */
  0x339B,   /* U+13BC */
  0x339C,   /* U+13BD */
  0x339D,   /* U+13BE */
  0x339E,   /* U+13BF */
  0x339F,   /* U+13C0 */
  0x33A0,   /* U+13C1 */
  0x33A1,   /* U+13C2 */
  0x33A2,   /* U+13C3 */
  0x33A3,   /* U+13C4 */
  0x33A4,   /* U+13C5 */
  0x33A5,   /* U+13C6 */
  0x33A6,   /* U+13C7 */
  0x33A7,   /* U+13C8 */
  0x33A8,   /* U+13C9 */
  0x33A9,   /* U+13CA */
  0x33AA,   /* U+13CB */
  0x33AB,   /* U+13CC */
  0x33AC,   /* U+13CD */
  0x33AD,   /* U+13CE */
  0x33AE,   /* U+13CF */
  0x33AF,   /* U+13D0 */
  0x33B0,   /* U+13D1 */
  0x33B1,   /* U+13D2 */
  0x33B2,   /* U+13D3 */
  0x33B3,   /* U+13D4 */
  0x33B4,   /* U+13D5 */
  0x33B5,   /* U+13D6 */
  0x33B6,   /* U+13D7 */
  0x33B7,   /* U+13D8 */
  0x33B8,   /* U+13D9 */
  0x33B9,   /* U+13DA */
  0x33BA,   /* U+13DB */
  0x33BB,   /* U+13DC */
  0x33BC,   /* U+13DD */
  0x33BD,   /* U+13DE */
  0x33BE,   /* U+13DF */
  0x33BF,   /* U+13E0 */
  0x33C0,   /* U+13E1 */
  0x33C1,   /* U+13E2 */
  0x33C2,   /* U+13E3 */
  0x33C3,   /* U+13E4 */
  0x33C4,   /* U+13E5 */
  0x33C5,   /* U+13E6 */
  0x33C6,   /* U+13E7 */
  0x33C7,   /* U+13E8 */
  0x33C8,   /* U+13E9 */
  0x33C9,   /* U+13EA */
  0x33CA,   /* U+13EB */
  0x33CB,   /* U+13EC */
  0x33CC,   /* U+13ED */
  0x33CD,   /* U+13EE */
  0x33CE,   /* U+13EF */
  0x33CF,   /* U+13F0 */
  0x33D0,   /* U+13F1 */
  0x33D1,   /* U+13F2 */
  0x33D2,   /* U+13F3 */
  0x33D3,   /* U+13F4 */
  0x33D4,   /* U+13F5 */
  0xFBC0,   /* U+13F6 */
  0xFBC0,   /* U+13F7 */
  0x33CF,   /* U+13F8 */
  0x33D0,   /* U+13F9 */
  0x33D1,   /* U+13FA */
  0x33D2,   /* U+13FB */
  0x33D3,   /* U+13FC */
  0x33D4,   /* U+13FD */
  0xFBC0,   /* U+13FE */
  0xFBC0,   /* U+13FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1300 */
  0x0020,   /* U+1301 */
  0x0020,   /* U+1302 */
  0x0020,   /* U+1303 */
  0x0020,   /* U+1304 */
  0x0020,   /* U+1305 */
  0x0020,   /* U+1306 */
  0x0020,   /* U+1307 */
  0x0020,   /* U+1308 */
  0x0020,   /* U+1309 */
  0x0020,   /* U+130A */
  0x0020,   /* U+130B */
  0x0020,   /* U+130C */
  0x0020,   /* U+130D */
  0x0020,   /* U+130E */
  0x0020,   /* U+130F */
  0x0020,   /* U+1310 */
  0x0020,   /* U+1311 */
  0x0020,   /* U+1312 */
  0x0020,   /* U+1313 */
  0x0020,   /* U+1314 */
  0x0020,   /* U+1315 */
  0x0020,   /* U+1316 */
  0x0020,   /* U+1317 */
  0x0020,   /* U+1318 */
  0x0020,   /* U+1319 */
  0x0020,   /* U+131A */
  0x0020,   /* U+131B */
  0x0020,   /* U+131C */
  0x0020,   /* U+131D */
  0x0020,   /* U+131E */
  0x0020,   /* U+131F */
  0x0020,   /* U+1320 */
  0x0020,   /* U+1321 */
  0x0020,   /* U+1322 */
  0x0020,   /* U+1323 */
  0x0020,   /* U+1324 */
  0x0020,   /* U+1325 */
  0x0020,   /* U+1326 */
  0x0020,   /* U+1327 */
  0x0020,   /* U+1328 */
  0x0020,   /* U+1329 */
  0x0020,   /* U+132A */
  0x0020,   /* U+132B */
  0x0020,   /* U+132C */
  0x0020,   /* U+132D */
  0x0020,   /* U+132E */
  0x0020,   /* U+132F */
  0x0020,   /* U+1330 */
  0x0020,   /* U+1331 */
  0x0020,   /* U+1332 */
  0x0020,   /* U+1333 */
  0x0020,   /* U+1334 */
  0x0020,   /* U+1335 */
  0x0020,   /* U+1336 */
  0x0020,   /* U+1337 */
  0x0020,   /* U+1338 */
  0x0020,   /* U+1339 */
  0x0020,   /* U+133A */
  0x0020,   /* U+133B */
  0x0020,   /* U+133C */
  0x0020,   /* U+133D */
  0x0020,   /* U+133E */
  0x0020,   /* U+133F */
  0x0020,   /* U+1340 */
  0x0020,   /* U+1341 */
  0x0020,   /* U+1342 */
  0x0020,   /* U+1343 */
  0x0020,   /* U+1344 */
  0x0020,   /* U+1345 */
  0x0020,   /* U+1346 */
  0x0020,   /* U+1347 */
  0x0020,   /* U+1348 */
  0x0020,   /* U+1349 */
  0x0020,   /* U+134A */
  0x0020,   /* U+134B */
  0x0020,   /* U+134C */
  0x0020,   /* U+134D */
  0x0020,   /* U+134E */
  0x0020,   /* U+134F */
  0x0020,   /* U+1350 */
  0x0020,   /* U+1351 */
  0x0020,   /* U+1352 */
  0x0020,   /* U+1353 */
  0x0020,   /* U+1354 */
  0x0020,   /* U+1355 */
  0x0020,   /* U+1356 */
  0x0020,   /* U+1357 */
  0x0020,   /* U+1358 */
  0x0020,   /* U+1359 */
  0x0020,   /* U+135A */
  0x0020,   /* U+135B */
  0x0020,   /* U+135C */
  0x00B5,   /* U+135D */
  0x00B4,   /* U+135E */
  0x00B3,   /* U+135F */
  0x0020,   /* U+1360 */
  0x0020,   /* U+1361 */
  0x0020,   /* U+1362 */
  0x0020,   /* U+1363 */
  0x0020,   /* U+1364 */
  0x0020,   /* U+1365 */
  0x0020,   /* U+1366 */
  0x0020,   /* U+1367 */
  0x0020,   /* U+1368 */
  0x0020,   /* U+1369 */
  0x0020,   /* U+136A */
  0x0020,   /* U+136B */
  0x0020,   /* U+136C */
  0x0020,   /* U+136D */
  0x0020,   /* U+136E */
  0x0020,   /* U+136F */
  0x0020,   /* U+1370 */
  0x0020,   /* U+1371 */
  0x0020,   /* U+1372 */
  0x0020,   /* U+1373 */
  0x0020,   /* U+1374 */
  0x0020,   /* U+1375 */
  0x0020,   /* U+1376 */
  0x0020,   /* U+1377 */
  0x0020,   /* U+1378 */
  0x0020,   /* U+1379 */
  0x0020,   /* U+137A */
  0x0020,   /* U+137B */
  0x0020,   /* U+137C */
  0x0020,   /* U+137D */
  0x0020,   /* U+137E */
  0x0020,   /* U+137F */
  0x0020,   /* U+1380 */
  0x0020,   /* U+1381 */
  0x0020,   /* U+1382 */
  0x0020,   /* U+1383 */
  0x0020,   /* U+1384 */
  0x0020,   /* U+1385 */
  0x0020,   /* U+1386 */
  0x0020,   /* U+1387 */
  0x0020,   /* U+1388 */
  0x0020,   /* U+1389 */
  0x0020,   /* U+138A */
  0x0020,   /* U+138B */
  0x0020,   /* U+138C */
  0x0020,   /* U+138D */
  0x0020,   /* U+138E */
  0x0020,   /* U+138F */
  0x0020,   /* U+1390 */
  0x0020,   /* U+1391 */
  0x0020,   /* U+1392 */
  0x0020,   /* U+1393 */
  0x0020,   /* U+1394 */
  0x0020,   /* U+1395 */
  0x0020,   /* U+1396 */
  0x0020,   /* U+1397 */
  0x0020,   /* U+1398 */
  0x0020,   /* U+1399 */
  0x0020,   /* U+139A */
  0x0020,   /* U+139B */
  0x0020,   /* U+139C */
  0x0020,   /* U+139D */
  0x0020,   /* U+139E */
  0x0020,   /* U+139F */
  0x0020,   /* U+13A0 */
  0x0020,   /* U+13A1 */
  0x0020,   /* U+13A2 */
  0x0020,   /* U+13A3 */
  0x0020,   /* U+13A4 */
  0x0020,   /* U+13A5 */
  0x0020,   /* U+13A6 */
  0x0020,   /* U+13A7 */
  0x0020,   /* U+13A8 */
  0x0020,   /* U+13A9 */
  0x0020,   /* U+13AA */
  0x0020,   /* U+13AB */
  0x0020,   /* U+13AC */
  0x0020,   /* U+13AD */
  0x0020,   /* U+13AE */
  0x0020,   /* U+13AF */
  0x0020,   /* U+13B0 */
  0x0020,   /* U+13B1 */
  0x0020,   /* U+13B2 */
  0x0020,   /* U+13B3 */
  0x0020,   /* U+13B4 */
  0x0020,   /* U+13B5 */
  0x0020,   /* U+13B6 */
  0x0020,   /* U+13B7 */
  0x0020,   /* U+13B8 */
  0x0020,   /* U+13B9 */
  0x0020,   /* U+13BA */
  0x0020,   /* U+13BB */
  0x0020,   /* U+13BC */
  0x0020,   /* U+13BD */
  0x0020,   /* U+13BE */
  0x0020,   /* U+13BF */
  0x0020,   /* U+13C0 */
  0x0020,   /* U+13C1 */
  0x0020,   /* U+13C2 */
  0x0020,   /* U+13C3 */
  0x0020,   /* U+13C4 */
  0x0020,   /* U+13C5 */
  0x0020,   /* U+13C6 */
  0x0020,   /* U+13C7 */
  0x0020,   /* U+13C8 */
  0x0020,   /* U+13C9 */
  0x0020,   /* U+13CA */
  0x0020,   /* U+13CB */
  0x0020,   /* U+13CC */
  0x0020,   /* U+13CD */
  0x0020,   /* U+13CE */
  0x0020,   /* U+13CF */
  0x0020,   /* U+13D0 */
  0x0020,   /* U+13D1 */
  0x0020,   /* U+13D2 */
  0x0020,   /* U+13D3 */
  0x0020,   /* U+13D4 */
  0x0020,   /* U+13D5 */
  0x0020,   /* U+13D6 */
  0x0020,   /* U+13D7 */
  0x0020,   /* U+13D8 */
  0x0020,   /* U+13D9 */
  0x0020,   /* U+13DA */
  0x0020,   /* U+13DB */
  0x0020,   /* U+13DC */
  0x0020,   /* U+13DD */
  0x0020,   /* U+13DE */
  0x0020,   /* U+13DF */
  0x0020,   /* U+13E0 */
  0x0020,   /* U+13E1 */
  0x0020,   /* U+13E2 */
  0x0020,   /* U+13E3 */
  0x0020,   /* U+13E4 */
  0x0020,   /* U+13E5 */
  0x0020,   /* U+13E6 */
  0x0020,   /* U+13E7 */
  0x0020,   /* U+13E8 */
  0x0020,   /* U+13E9 */
  0x0020,   /* U+13EA */
  0x0020,   /* U+13EB */
  0x0020,   /* U+13EC */
  0x0020,   /* U+13ED */
  0x0020,   /* U+13EE */
  0x0020,   /* U+13EF */
  0x0020,   /* U+13F0 */
  0x0020,   /* U+13F1 */
  0x0020,   /* U+13F2 */
  0x0020,   /* U+13F3 */
  0x0020,   /* U+13F4 */
  0x0020,   /* U+13F5 */
  0x0020,   /* U+13F6 */
  0x0020,   /* U+13F7 */
  0x0020,   /* U+13F8 */
  0x0020,   /* U+13F9 */
  0x0020,   /* U+13FA */
  0x0020,   /* U+13FB */
  0x0020,   /* U+13FC */
  0x0020,   /* U+13FD */
  0x0020,   /* U+13FE */
  0x0020,   /* U+13FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1300 */
  0x0002,   /* U+1301 */
  0x0002,   /* U+1302 */
  0x0002,   /* U+1303 */
  0x0002,   /* U+1304 */
  0x0002,   /* U+1305 */
  0x0002,   /* U+1306 */
  0x0002,   /* U+1307 */
  0x0002,   /* U+1308 */
  0x0002,   /* U+1309 */
  0x0002,   /* U+130A */
  0x0002,   /* U+130B */
  0x0002,   /* U+130C */
  0x0002,   /* U+130D */
  0x0002,   /* U+130E */
  0x0002,   /* U+130F */
  0x0002,   /* U+1310 */
  0x0002,   /* U+1311 */
  0x0002,   /* U+1312 */
  0x0002,   /* U+1313 */
  0x0002,   /* U+1314 */
  0x0002,   /* U+1315 */
  0x0002,   /* U+1316 */
  0x0002,   /* U+1317 */
  0x0002,   /* U+1318 */
  0x0002,   /* U+1319 */
  0x0002,   /* U+131A */
  0x0002,   /* U+131B */
  0x0002,   /* U+131C */
  0x0002,   /* U+131D */
  0x0002,   /* U+131E */
  0x0002,   /* U+131F */
  0x0002,   /* U+1320 */
  0x0002,   /* U+1321 */
  0x0002,   /* U+1322 */
  0x0002,   /* U+1323 */
  0x0002,   /* U+1324 */
  0x0002,   /* U+1325 */
  0x0002,   /* U+1326 */
  0x0002,   /* U+1327 */
  0x0002,   /* U+1328 */
  0x0002,   /* U+1329 */
  0x0002,   /* U+132A */
  0x0002,   /* U+132B */
  0x0002,   /* U+132C */
  0x0002,   /* U+132D */
  0x0002,   /* U+132E */
  0x0002,   /* U+132F */
  0x0002,   /* U+1330 */
  0x0002,   /* U+1331 */
  0x0002,   /* U+1332 */
  0x0002,   /* U+1333 */
  0x0002,   /* U+1334 */
  0x0002,   /* U+1335 */
  0x0002,   /* U+1336 */
  0x0002,   /* U+1337 */
  0x0002,   /* U+1338 */
  0x0002,   /* U+1339 */
  0x0002,   /* U+133A */
  0x0002,   /* U+133B */
  0x0002,   /* U+133C */
  0x0002,   /* U+133D */
  0x0002,   /* U+133E */
  0x0002,   /* U+133F */
  0x0002,   /* U+1340 */
  0x0002,   /* U+1341 */
  0x0002,   /* U+1342 */
  0x0002,   /* U+1343 */
  0x0002,   /* U+1344 */
  0x0002,   /* U+1345 */
  0x0002,   /* U+1346 */
  0x0002,   /* U+1347 */
  0x0002,   /* U+1348 */
  0x0002,   /* U+1349 */
  0x0002,   /* U+134A */
  0x0002,   /* U+134B */
  0x0002,   /* U+134C */
  0x0002,   /* U+134D */
  0x0002,   /* U+134E */
  0x0002,   /* U+134F */
  0x0002,   /* U+1350 */
  0x0002,   /* U+1351 */
  0x0002,   /* U+1352 */
  0x0002,   /* U+1353 */
  0x0002,   /* U+1354 */
  0x0002,   /* U+1355 */
  0x0002,   /* U+1356 */
  0x0002,   /* U+1357 */
  0x0002,   /* U+1358 */
  0x0002,   /* U+1359 */
  0x0002,   /* U+135A */
  0x0002,   /* U+135B */
  0x0002,   /* U+135C */
  0x0002,   /* U+135D */
  0x0002,   /* U+135E */
  0x0002,   /* U+135F */
  0x0002,   /* U+1360 */
  0x0002,   /* U+1361 */
  0x0002,   /* U+1362 */
  0x0002,   /* U+1363 */
  0x0002,   /* U+1364 */
  0x0002,   /* U+1365 */
  0x0002,   /* U+1366 */
  0x0002,   /* U+1367 */
  0x0002,   /* U+1368 */
  0x0002,   /* U+1369 */
  0x0002,   /* U+136A */
  0x0002,   /* U+136B */
  0x0002,   /* U+136C */
  0x0002,   /* U+136D */
  0x0002,   /* U+136E */
  0x0002,   /* U+136F */
  0x0002,   /* U+1370 */
  0x0002,   /* U+1371 */
  0x0002,   /* U+1372 */
  0x0002,   /* U+1373 */
  0x0002,   /* U+1374 */
  0x0002,   /* U+1375 */
  0x0002,   /* U+1376 */
  0x0002,   /* U+1377 */
  0x0002,   /* U+1378 */
  0x0002,   /* U+1379 */
  0x0002,   /* U+137A */
  0x0002,   /* U+137B */
  0x0002,   /* U+137C */
  0x0002,   /* U+137D */
  0x0002,   /* U+137E */
  0x0002,   /* U+137F */
  0x0002,   /* U+1380 */
  0x0002,   /* U+1381 */
  0x0002,   /* U+1382 */
  0x0002,   /* U+1383 */
  0x0002,   /* U+1384 */
  0x0002,   /* U+1385 */
  0x0002,   /* U+1386 */
  0x0002,   /* U+1387 */
  0x0002,   /* U+1388 */
  0x0002,   /* U+1389 */
  0x0002,   /* U+138A */
  0x0002,   /* U+138B */
  0x0002,   /* U+138C */
  0x0002,   /* U+138D */
  0x0002,   /* U+138E */
  0x0002,   /* U+138F */
  0x0002,   /* U+1390 */
  0x0002,   /* U+1391 */
  0x0002,   /* U+1392 */
  0x0002,   /* U+1393 */
  0x0002,   /* U+1394 */
  0x0002,   /* U+1395 */
  0x0002,   /* U+1396 */
  0x0002,   /* U+1397 */
  0x0002,   /* U+1398 */
  0x0002,   /* U+1399 */
  0x0002,   /* U+139A */
  0x0002,   /* U+139B */
  0x0002,   /* U+139C */
  0x0002,   /* U+139D */
  0x0002,   /* U+139E */
  0x0002,   /* U+139F */
  0x0008,   /* U+13A0 */
  0x0008,   /* U+13A1 */
  0x0008,   /* U+13A2 */
  0x0008,   /* U+13A3 */
  0x0008,   /* U+13A4 */
  0x0008,   /* U+13A5 */
  0x0008,   /* U+13A6 */
  0x0008,   /* U+13A7 */
  0x0008,   /* U+13A8 */
  0x0008,   /* U+13A9 */
  0x0008,   /* U+13AA */
  0x0008,   /* U+13AB */
  0x0008,   /* U+13AC */
  0x0008,   /* U+13AD */
  0x0008,   /* U+13AE */
  0x0008,   /* U+13AF */
  0x0008,   /* U+13B0 */
  0x0008,   /* U+13B1 */
  0x0008,   /* U+13B2 */
  0x0008,   /* U+13B3 */
  0x0008,   /* U+13B4 */
  0x0008,   /* U+13B5 */
  0x0008,   /* U+13B6 */
  0x0008,   /* U+13B7 */
  0x0008,   /* U+13B8 */
  0x0008,   /* U+13B9 */
  0x0008,   /* U+13BA */
  0x0008,   /* U+13BB */
  0x0008,   /* U+13BC */
  0x0008,   /* U+13BD */
  0x0008,   /* U+13BE */
  0x0008,   /* U+13BF */
  0x0008,   /* U+13C0 */
  0x0008,   /* U+13C1 */
  0x0008,   /* U+13C2 */
  0x0008,   /* U+13C3 */
  0x0008,   /* U+13C4 */
  0x0008,   /* U+13C5 */
  0x0008,   /* U+13C6 */
  0x0008,   /* U+13C7 */
  0x0008,   /* U+13C8 */
  0x0008,   /* U+13C9 */
  0x0008,   /* U+13CA */
  0x0008,   /* U+13CB */
  0x0008,   /* U+13CC */
  0x0008,   /* U+13CD */
  0x0008,   /* U+13CE */
  0x0008,   /* U+13CF */
  0x0008,   /* U+13D0 */
  0x0008,   /* U+13D1 */
  0x0008,   /* U+13D2 */
  0x0008,   /* U+13D3 */
  0x0008,   /* U+13D4 */
  0x0008,   /* U+13D5 */
  0x0008,   /* U+13D6 */
  0x0008,   /* U+13D7 */
  0x0008,   /* U+13D8 */
  0x0008,   /* U+13D9 */
  0x0008,   /* U+13DA */
  0x0008,   /* U+13DB */
  0x0008,   /* U+13DC */
  0x0008,   /* U+13DD */
  0x0008,   /* U+13DE */
  0x0008,   /* U+13DF */
  0x0008,   /* U+13E0 */
  0x0008,   /* U+13E1 */
  0x0008,   /* U+13E2 */
  0x0008,   /* U+13E3 */
  0x0008,   /* U+13E4 */
  0x0008,   /* U+13E5 */
  0x0008,   /* U+13E6 */
  0x0008,   /* U+13E7 */
  0x0008,   /* U+13E8 */
  0x0008,   /* U+13E9 */
  0x0008,   /* U+13EA */
  0x0008,   /* U+13EB */
  0x0008,   /* U+13EC */
  0x0008,   /* U+13ED */
  0x0008,   /* U+13EE */
  0x0008,   /* U+13EF */
  0x0008,   /* U+13F0 */
  0x0008,   /* U+13F1 */
  0x0008,   /* U+13F2 */
  0x0008,   /* U+13F3 */
  0x0008,   /* U+13F4 */
  0x0008,   /* U+13F5 */
  0x0002,   /* U+13F6 */
  0x0002,   /* U+13F7 */
  0x0002,   /* U+13F8 */
  0x0002,   /* U+13F9 */
  0x0002,   /* U+13FA */
  0x0002,   /* U+13FB */
  0x0002,   /* U+13FC */
  0x0002,   /* U+13FD */
  0x0002,   /* U+13FE */
  0x0002,   /* U+13FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1300 */
  0x0000,   /* U+1301 */
  0x0000,   /* U+1302 */
  0x0000,   /* U+1303 */
  0x0000,   /* U+1304 */
  0x0000,   /* U+1305 */
  0x0000,   /* U+1306 */
  0x0000,   /* U+1307 */
  0x0000,   /* U+1308 */
  0x0000,   /* U+1309 */
  0x0000,   /* U+130A */
  0x0000,   /* U+130B */
  0x0000,   /* U+130C */
  0x0000,   /* U+130D */
  0x0000,   /* U+130E */
  0x0000,   /* U+130F */
  0x0000,   /* U+1310 */
  0x9311,   /* U+1311 */
  0x0000,   /* U+1312 */
  0x0000,   /* U+1313 */
  0x0000,   /* U+1314 */
  0x0000,   /* U+1315 */
  0x9316,   /* U+1316 */
  0x9317,   /* U+1317 */
  0x0000,   /* U+1318 */
  0x0000,   /* U+1319 */
  0x0000,   /* U+131A */
  0x0000,   /* U+131B */
  0x0000,   /* U+131C */
  0x0000,   /* U+131D */
  0x0000,   /* U+131E */
  0x0000,   /* U+131F */
  0x0000,   /* U+1320 */
  0x0000,   /* U+1321 */
  0x0000,   /* U+1322 */
  0x0000,   /* U+1323 */
  0x0000,   /* U+1324 */
  0x0000,   /* U+1325 */
  0x0000,   /* U+1326 */
  0x0000,   /* U+1327 */
  0x0000,   /* U+1328 */
  0x0000,   /* U+1329 */
  0x0000,   /* U+132A */
  0x0000,   /* U+132B */
  0x0000,   /* U+132C */
  0x0000,   /* U+132D */
  0x0000,   /* U+132E */
  0x0000,   /* U+132F */
  0x0000,   /* U+1330 */
  0x0000,   /* U+1331 */
  0x0000,   /* U+1332 */
  0x0000,   /* U+1333 */
  0x0000,   /* U+1334 */
  0x0000,   /* U+1335 */
  0x0000,   /* U+1336 */
  0x0000,   /* U+1337 */
  0x0000,   /* U+1338 */
  0x0000,   /* U+1339 */
  0x0000,   /* U+133A */
  0x0000,   /* U+133B */
  0x0000,   /* U+133C */
  0x0000,   /* U+133D */
  0x0000,   /* U+133E */
  0x0000,   /* U+133F */
  0x0000,   /* U+1340 */
  0x0000,   /* U+1341 */
  0x0000,   /* U+1342 */
  0x0000,   /* U+1343 */
  0x0000,   /* U+1344 */
  0x0000,   /* U+1345 */
  0x0000,   /* U+1346 */
  0x0000,   /* U+1347 */
  0x0000,   /* U+1348 */
  0x0000,   /* U+1349 */
  0x0000,   /* U+134A */
  0x0000,   /* U+134B */
  0x0000,   /* U+134C */
  0x0000,   /* U+134D */
  0x0000,   /* U+134E */
  0x0000,   /* U+134F */
  0x0000,   /* U+1350 */
  0x0000,   /* U+1351 */
  0x0000,   /* U+1352 */
  0x0000,   /* U+1353 */
  0x0000,   /* U+1354 */
  0x0000,   /* U+1355 */
  0x0000,   /* U+1356 */
  0x0000,   /* U+1357 */
  0x0000,   /* U+1358 */
  0x0000,   /* U+1359 */
  0x0000,   /* U+135A */
  0x935B,   /* U+135B */
  0x935C,   /* U+135C */
  0x0000,   /* U+135D */
  0x0000,   /* U+135E */
  0x0000,   /* U+135F */
  0x0000,   /* U+1360 */
  0x0000,   /* U+1361 */
  0x0000,   /* U+1362 */
  0x0000,   /* U+1363 */
  0x0000,   /* U+1364 */
  0x0000,   /* U+1365 */
  0x0000,   /* U+1366 */
  0x0000,   /* U+1367 */
  0x0000,   /* U+1368 */
  0x0000,   /* U+1369 */
  0x0000,   /* U+136A */
  0x0000,   /* U+136B */
  0x0000,   /* U+136C */
  0x0000,   /* U+136D */
  0x0000,   /* U+136E */
  0x0000,   /* U+136F */
  0x0000,   /* U+1370 */
  0x0000,   /* U+1371 */
  0x0000,   /* U+1372 */
  0x0000,   /* U+1373 */
  0x0000,   /* U+1374 */
  0x0000,   /* U+1375 */
  0x0000,   /* U+1376 */
  0x0000,   /* U+1377 */
  0x0000,   /* U+1378 */
  0x0000,   /* U+1379 */
  0x0000,   /* U+137A */
  0x0000,   /* U+137B */
  0x0000,   /* U+137C */
  0x937D,   /* U+137D */
  0x937E,   /* U+137E */
  0x937F,   /* U+137F */
  0x0000,   /* U+1380 */
  0x0000,   /* U+1381 */
  0x0000,   /* U+1382 */
  0x0000,   /* U+1383 */
  0x0000,   /* U+1384 */
  0x0000,   /* U+1385 */
  0x0000,   /* U+1386 */
  0x0000,   /* U+1387 */
  0x0000,   /* U+1388 */
  0x0000,   /* U+1389 */
  0x0000,   /* U+138A */
  0x0000,   /* U+138B */
  0x0000,   /* U+138C */
  0x0000,   /* U+138D */
  0x0000,   /* U+138E */
  0x0000,   /* U+138F */
  0x0000,   /* U+1390 */
  0x0000,   /* U+1391 */
  0x0000,   /* U+1392 */
  0x0000,   /* U+1393 */
  0x0000,   /* U+1394 */
  0x0000,   /* U+1395 */
  0x0000,   /* U+1396 */
  0x0000,   /* U+1397 */
  0x0000,   /* U+1398 */
  0x0000,   /* U+1399 */
  0x939A,   /* U+139A */
  0x939B,   /* U+139B */
  0x939C,   /* U+139C */
  0x939D,   /* U+139D */
  0x939E,   /* U+139E */
  0x939F,   /* U+139F */
  0x0000,   /* U+13A0 */
  0x0000,   /* U+13A1 */
  0x0000,   /* U+13A2 */
  0x0000,   /* U+13A3 */
  0x0000,   /* U+13A4 */
  0x0000,   /* U+13A5 */
  0x0000,   /* U+13A6 */
  0x0000,   /* U+13A7 */
  0x0000,   /* U+13A8 */
  0x0000,   /* U+13A9 */
  0x0000,   /* U+13AA */
  0x0000,   /* U+13AB */
  0x0000,   /* U+13AC */
  0x0000,   /* U+13AD */
  0x0000,   /* U+13AE */
  0x0000,   /* U+13AF */
  0x0000,   /* U+13B0 */
  0x0000,   /* U+13B1 */
  0x0000,   /* U+13B2 */
  0x0000,   /* U+13B3 */
  0x0000,   /* U+13B4 */
  0x0000,   /* U+13B5 */
  0x0000,   /* U+13B6 */
  0x0000,   /* U+13B7 */
  0x0000,   /* U+13B8 */
  0x0000,   /* U+13B9 */
  0x0000,   /* U+13BA */
  0x0000,   /* U+13BB */
  0x0000,   /* U+13BC */
  0x0000,   /* U+13BD */
  0x0000,   /* U+13BE */
  0x0000,   /* U+13BF */
  0x0000,   /* U+13C0 */
  0x0000,   /* U+13C1 */
  0x0000,   /* U+13C2 */
  0x0000,   /* U+13C3 */
  0x0000,   /* U+13C4 */
  0x0000,   /* U+13C5 */
  0x0000,   /* U+13C6 */
  0x0000,   /* U+13C7 */
  0x0000,   /* U+13C8 */
  0x0000,   /* U+13C9 */
  0x0000,   /* U+13CA */
  0x0000,   /* U+13CB */
  0x0000,   /* U+13CC */
  0x0000,   /* U+13CD */
  0x0000,   /* U+13CE */
  0x0000,   /* U+13CF */
  0x0000,   /* U+13D0 */
  0x0000,   /* U+13D1 */
  0x0000,   /* U+13D2 */
  0x0000,   /* U+13D3 */
  0x0000,   /* U+13D4 */
  0x0000,   /* U+13D5 */
  0x0000,   /* U+13D6 */
  0x0000,   /* U+13D7 */
  0x0000,   /* U+13D8 */
  0x0000,   /* U+13D9 */
  0x0000,   /* U+13DA */
  0x0000,   /* U+13DB */
  0x0000,   /* U+13DC */
  0x0000,   /* U+13DD */
  0x0000,   /* U+13DE */
  0x0000,   /* U+13DF */
  0x0000,   /* U+13E0 */
  0x0000,   /* U+13E1 */
  0x0000,   /* U+13E2 */
  0x0000,   /* U+13E3 */
  0x0000,   /* U+13E4 */
  0x0000,   /* U+13E5 */
  0x0000,   /* U+13E6 */
  0x0000,   /* U+13E7 */
  0x0000,   /* U+13E8 */
  0x0000,   /* U+13E9 */
  0x0000,   /* U+13EA */
  0x0000,   /* U+13EB */
  0x0000,   /* U+13EC */
  0x0000,   /* U+13ED */
  0x0000,   /* U+13EE */
  0x0000,   /* U+13EF */
  0x0000,   /* U+13F0 */
  0x0000,   /* U+13F1 */
  0x0000,   /* U+13F2 */
  0x0000,   /* U+13F3 */
  0x0000,   /* U+13F4 */
  0x0000,   /* U+13F5 */
  0x93F6,   /* U+13F6 */
  0x93F7,   /* U+13F7 */
  0x0000,   /* U+13F8 */
  0x0000,   /* U+13F9 */
  0x0000,   /* U+13FA */
  0x0000,   /* U+13FB */
  0x0000,   /* U+13FC */
  0x0000,   /* U+13FD */
  0x93FE,   /* U+13FE */
  0x93FF,   /* U+13FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1300 */
  0x0000,   /* U+1301 */
  0x0000,   /* U+1302 */
  0x0000,   /* U+1303 */
  0x0000,   /* U+1304 */
  0x0000,   /* U+1305 */
  0x0000,   /* U+1306 */
  0x0000,   /* U+1307 */
  0x0000,   /* U+1308 */
  0x0000,   /* U+1309 */
  0x0000,   /* U+130A */
  0x0000,   /* U+130B */
  0x0000,   /* U+130C */
  0x0000,   /* U+130D */
  0x0000,   /* U+130E */
  0x0000,   /* U+130F */
  0x0000,   /* U+1310 */
  0x0000,   /* U+1311 */
  0x0000,   /* U+1312 */
  0x0000,   /* U+1313 */
  0x0000,   /* U+1314 */
  0x0000,   /* U+1315 */
  0x0000,   /* U+1316 */
  0x0000,   /* U+1317 */
  0x0000,   /* U+1318 */
  0x0000,   /* U+1319 */
  0x0000,   /* U+131A */
  0x0000,   /* U+131B */
  0x0000,   /* U+131C */
  0x0000,   /* U+131D */
  0x0000,   /* U+131E */
  0x0000,   /* U+131F */
  0x0000,   /* U+1320 */
  0x0000,   /* U+1321 */
  0x0000,   /* U+1322 */
  0x0000,   /* U+1323 */
  0x0000,   /* U+1324 */
  0x0000,   /* U+1325 */
  0x0000,   /* U+1326 */
  0x0000,   /* U+1327 */
  0x0000,   /* U+1328 */
  0x0000,   /* U+1329 */
  0x0000,   /* U+132A */
  0x0000,   /* U+132B */
  0x0000,   /* U+132C */
  0x0000,   /* U+132D */
  0x0000,   /* U+132E */
  0x0000,   /* U+132F */
  0x0000,   /* U+1330 */
  0x0000,   /* U+1331 */
  0x0000,   /* U+1332 */
  0x0000,   /* U+1333 */
  0x0000,   /* U+1334 */
  0x0000,   /* U+1335 */
  0x0000,   /* U+1336 */
  0x0000,   /* U+1337 */
  0x0000,   /* U+1338 */
  0x0000,   /* U+1339 */
  0x0000,   /* U+133A */
  0x0000,   /* U+133B */
  0x0000,   /* U+133C */
  0x0000,   /* U+133D */
  0x0000,   /* U+133E */
  0x0000,   /* U+133F */
  0x0000,   /* U+1340 */
  0x0000,   /* U+1341 */
  0x0000,   /* U+1342 */
  0x0000,   /* U+1343 */
  0x0000,   /* U+1344 */
  0x0000,   /* U+1345 */
  0x0000,   /* U+1346 */
  0x0000,   /* U+1347 */
  0x0000,   /* U+1348 */
  0x0000,   /* U+1349 */
  0x0000,   /* U+134A */
  0x0000,   /* U+134B */
  0x0000,   /* U+134C */
  0x0000,   /* U+134D */
  0x0000,   /* U+134E */
  0x0000,   /* U+134F */
  0x0000,   /* U+1350 */
  0x0000,   /* U+1351 */
  0x0000,   /* U+1352 */
  0x0000,   /* U+1353 */
  0x0000,   /* U+1354 */
  0x0000,   /* U+1355 */
  0x0000,   /* U+1356 */
  0x0000,   /* U+1357 */
  0x0000,   /* U+1358 */
  0x0000,   /* U+1359 */
  0x0000,   /* U+135A */
  0x0000,   /* U+135B */
  0x0000,   /* U+135C */
  0x0000,   /* U+135D */
  0x0000,   /* U+135E */
  0x0000,   /* U+135F */
  0x0000,   /* U+1360 */
  0x0000,   /* U+1361 */
  0x0000,   /* U+1362 */
  0x0000,   /* U+1363 */
  0x0000,   /* U+1364 */
  0x0000,   /* U+1365 */
  0x0000,   /* U+1366 */
  0x0000,   /* U+1367 */
  0x0000,   /* U+1368 */
  0x0000,   /* U+1369 */
  0x0000,   /* U+136A */
  0x0000,   /* U+136B */
  0x0000,   /* U+136C */
  0x0000,   /* U+136D */
  0x0000,   /* U+136E */
  0x0000,   /* U+136F */
  0x0000,   /* U+1370 */
  0x0000,   /* U+1371 */
  0x0000,   /* U+1372 */
  0x0000,   /* U+1373 */
  0x0000,   /* U+1374 */
  0x0000,   /* U+1375 */
  0x0000,   /* U+1376 */
  0x0000,   /* U+1377 */
  0x0000,   /* U+1378 */
  0x0000,   /* U+1379 */
  0x0000,   /* U+137A */
  0x0000,   /* U+137B */
  0x0000,   /* U+137C */
  0x0000,   /* U+137D */
  0x0000,   /* U+137E */
  0x0000,   /* U+137F */
  0x0000,   /* U+1380 */
  0x0000,   /* U+1381 */
  0x0000,   /* U+1382 */
  0x0000,   /* U+1383 */
  0x0000,   /* U+1384 */
  0x0000,   /* U+1385 */
  0x0000,   /* U+1386 */
  0x0000,   /* U+1387 */
  0x0000,   /* U+1388 */
  0x0000,   /* U+1389 */
  0x0000,   /* U+138A */
  0x0000,   /* U+138B */
  0x0000,   /* U+138C */
  0x0000,   /* U+138D */
  0x0000,   /* U+138E */
  0x0000,   /* U+138F */
  0x0000,   /* U+1390 */
  0x0000,   /* U+1391 */
  0x0000,   /* U+1392 */
  0x0000,   /* U+1393 */
  0x0000,   /* U+1394 */
  0x0000,   /* U+1395 */
  0x0000,   /* U+1396 */
  0x0000,   /* U+1397 */
  0x0000,   /* U+1398 */
  0x0000,   /* U+1399 */
  0x0000,   /* U+139A */
  0x0000,   /* U+139B */
  0x0000,   /* U+139C */
  0x0000,   /* U+139D */
  0x0000,   /* U+139E */
  0x0000,   /* U+139F */
  0x0000,   /* U+13A0 */
  0x0000,   /* U+13A1 */
  0x0000,   /* U+13A2 */
  0x0000,   /* U+13A3 */
  0x0000,   /* U+13A4 */
  0x0000,   /* U+13A5 */
  0x0000,   /* U+13A6 */
  0x0000,   /* U+13A7 */
  0x0000,   /* U+13A8 */
  0x0000,   /* U+13A9 */
  0x0000,   /* U+13AA */
  0x0000,   /* U+13AB */
  0x0000,   /* U+13AC */
  0x0000,   /* U+13AD */
  0x0000,   /* U+13AE */
  0x0000,   /* U+13AF */
  0x0000,   /* U+13B0 */
  0x0000,   /* U+13B1 */
  0x0000,   /* U+13B2 */
  0x0000,   /* U+13B3 */
  0x0000,   /* U+13B4 */
  0x0000,   /* U+13B5 */
  0x0000,   /* U+13B6 */
  0x0000,   /* U+13B7 */
  0x0000,   /* U+13B8 */
  0x0000,   /* U+13B9 */
  0x0000,   /* U+13BA */
  0x0000,   /* U+13BB */
  0x0000,   /* U+13BC */
  0x0000,   /* U+13BD */
  0x0000,   /* U+13BE */
  0x0000,   /* U+13BF */
  0x0000,   /* U+13C0 */
  0x0000,   /* U+13C1 */
  0x0000,   /* U+13C2 */
  0x0000,   /* U+13C3 */
  0x0000,   /* U+13C4 */
  0x0000,   /* U+13C5 */
  0x0000,   /* U+13C6 */
  0x0000,   /* U+13C7 */
  0x0000,   /* U+13C8 */
  0x0000,   /* U+13C9 */
  0x0000,   /* U+13CA */
  0x0000,   /* U+13CB */
  0x0000,   /* U+13CC */
  0x0000,   /* U+13CD */
  0x0000,   /* U+13CE */
  0x0000,   /* U+13CF */
  0x0000,   /* U+13D0 */
  0x0000,   /* U+13D1 */
  0x0000,   /* U+13D2 */
  0x0000,   /* U+13D3 */
  0x0000,   /* U+13D4 */
  0x0000,   /* U+13D5 */
  0x0000,   /* U+13D6 */
  0x0000,   /* U+13D7 */
  0x0000,   /* U+13D8 */
  0x0000,   /* U+13D9 */
  0x0000,   /* U+13DA */
  0x0000,   /* U+13DB */
  0x0000,   /* U+13DC */
  0x0000,   /* U+13DD */
  0x0000,   /* U+13DE */
  0x0000,   /* U+13DF */
  0x0000,   /* U+13E0 */
  0x0000,   /* U+13E1 */
  0x0000,   /* U+13E2 */
  0x0000,   /* U+13E3 */
  0x0000,   /* U+13E4 */
  0x0000,   /* U+13E5 */
  0x0000,   /* U+13E6 */
  0x0000,   /* U+13E7 */
  0x0000,   /* U+13E8 */
  0x0000,   /* U+13E9 */
  0x0000,   /* U+13EA */
  0x0000,   /* U+13EB */
  0x0000,   /* U+13EC */
  0x0000,   /* U+13ED */
  0x0000,   /* U+13EE */
  0x0000,   /* U+13EF */
  0x0000,   /* U+13F0 */
  0x0000,   /* U+13F1 */
  0x0000,   /* U+13F2 */
  0x0000,   /* U+13F3 */
  0x0000,   /* U+13F4 */
  0x0000,   /* U+13F5 */
  0x0000,   /* U+13F6 */
  0x0000,   /* U+13F7 */
  0x0000,   /* U+13F8 */
  0x0000,   /* U+13F9 */
  0x0000,   /* U+13FA */
  0x0000,   /* U+13FB */
  0x0000,   /* U+13FC */
  0x0000,   /* U+13FD */
  0x0000,   /* U+13FE */
  0x0000,   /* U+13FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1300 */
  0x0000,   /* U+1301 */
  0x0000,   /* U+1302 */
  0x0000,   /* U+1303 */
  0x0000,   /* U+1304 */
  0x0000,   /* U+1305 */
  0x0000,   /* U+1306 */
  0x0000,   /* U+1307 */
  0x0000,   /* U+1308 */
  0x0000,   /* U+1309 */
  0x0000,   /* U+130A */
  0x0000,   /* U+130B */
  0x0000,   /* U+130C */
  0x0000,   /* U+130D */
  0x0000,   /* U+130E */
  0x0000,   /* U+130F */
  0x0000,   /* U+1310 */
  0x0000,   /* U+1311 */
  0x0000,   /* U+1312 */
  0x0000,   /* U+1313 */
  0x0000,   /* U+1314 */
  0x0000,   /* U+1315 */
  0x0000,   /* U+1316 */
  0x0000,   /* U+1317 */
  0x0000,   /* U+1318 */
  0x0000,   /* U+1319 */
  0x0000,   /* U+131A */
  0x0000,   /* U+131B */
  0x0000,   /* U+131C */
  0x0000,   /* U+131D */
  0x0000,   /* U+131E */
  0x0000,   /* U+131F */
  0x0000,   /* U+1320 */
  0x0000,   /* U+1321 */
  0x0000,   /* U+1322 */
  0x0000,   /* U+1323 */
  0x0000,   /* U+1324 */
  0x0000,   /* U+1325 */
  0x0000,   /* U+1326 */
  0x0000,   /* U+1327 */
  0x0000,   /* U+1328 */
  0x0000,   /* U+1329 */
  0x0000,   /* U+132A */
  0x0000,   /* U+132B */
  0x0000,   /* U+132C */
  0x0000,   /* U+132D */
  0x0000,   /* U+132E */
  0x0000,   /* U+132F */
  0x0000,   /* U+1330 */
  0x0000,   /* U+1331 */
  0x0000,   /* U+1332 */
  0x0000,   /* U+1333 */
  0x0000,   /* U+1334 */
  0x0000,   /* U+1335 */
  0x0000,   /* U+1336 */
  0x0000,   /* U+1337 */
  0x0000,   /* U+1338 */
  0x0000,   /* U+1339 */
  0x0000,   /* U+133A */
  0x0000,   /* U+133B */
  0x0000,   /* U+133C */
  0x0000,   /* U+133D */
  0x0000,   /* U+133E */
  0x0000,   /* U+133F */
  0x0000,   /* U+1340 */
  0x0000,   /* U+1341 */
  0x0000,   /* U+1342 */
  0x0000,   /* U+1343 */
  0x0000,   /* U+1344 */
  0x0000,   /* U+1345 */
  0x0000,   /* U+1346 */
  0x0000,   /* U+1347 */
  0x0000,   /* U+1348 */
  0x0000,   /* U+1349 */
  0x0000,   /* U+134A */
  0x0000,   /* U+134B */
  0x0000,   /* U+134C */
  0x0000,   /* U+134D */
  0x0000,   /* U+134E */
  0x0000,   /* U+134F */
  0x0000,   /* U+1350 */
  0x0000,   /* U+1351 */
  0x0000,   /* U+1352 */
  0x0000,   /* U+1353 */
  0x0000,   /* U+1354 */
  0x0000,   /* U+1355 */
  0x0000,   /* U+1356 */
  0x0000,   /* U+1357 */
  0x0000,   /* U+1358 */
  0x0000,   /* U+1359 */
  0x0000,   /* U+135A */
  0x0000,   /* U+135B */
  0x0000,   /* U+135C */
  0x0000,   /* U+135D */
  0x0000,   /* U+135E */
  0x0000,   /* U+135F */
  0x0000,   /* U+1360 */
  0x0000,   /* U+1361 */
  0x0000,   /* U+1362 */
  0x0000,   /* U+1363 */
  0x0000,   /* U+1364 */
  0x0000,   /* U+1365 */
  0x0000,   /* U+1366 */
  0x0000,   /* U+1367 */
  0x0000,   /* U+1368 */
  0x0000,   /* U+1369 */
  0x0000,   /* U+136A */
  0x0000,   /* U+136B */
  0x0000,   /* U+136C */
  0x0000,   /* U+136D */
  0x0000,   /* U+136E */
  0x0000,   /* U+136F */
  0x0000,   /* U+1370 */
  0x0000,   /* U+1371 */
  0x0000,   /* U+1372 */
  0x0000,   /* U+1373 */
  0x0000,   /* U+1374 */
  0x0000,   /* U+1375 */
  0x0000,   /* U+1376 */
  0x0000,   /* U+1377 */
  0x0000,   /* U+1378 */
  0x0000,   /* U+1379 */
  0x0000,   /* U+137A */
  0x0000,   /* U+137B */
  0x0000,   /* U+137C */
  0x0000,   /* U+137D */
  0x0000,   /* U+137E */
  0x0000,   /* U+137F */
  0x0000,   /* U+1380 */
  0x0000,   /* U+1381 */
  0x0000,   /* U+1382 */
  0x0000,   /* U+1383 */
  0x0000,   /* U+1384 */
  0x0000,   /* U+1385 */
  0x0000,   /* U+1386 */
  0x0000,   /* U+1387 */
  0x0000,   /* U+1388 */
  0x0000,   /* U+1389 */
  0x0000,   /* U+138A */
  0x0000,   /* U+138B */
  0x0000,   /* U+138C */
  0x0000,   /* U+138D */
  0x0000,   /* U+138E */
  0x0000,   /* U+138F */
  0x0000,   /* U+1390 */
  0x0000,   /* U+1391 */
  0x0000,   /* U+1392 */
  0x0000,   /* U+1393 */
  0x0000,   /* U+1394 */
  0x0000,   /* U+1395 */
  0x0000,   /* U+1396 */
  0x0000,   /* U+1397 */
  0x0000,   /* U+1398 */
  0x0000,   /* U+1399 */
  0x0000,   /* U+139A */
  0x0000,   /* U+139B */
  0x0000,   /* U+139C */
  0x0000,   /* U+139D */
  0x0000,   /* U+139E */
  0x0000,   /* U+139F */
  0x0000,   /* U+13A0 */
  0x0000,   /* U+13A1 */
  0x0000,   /* U+13A2 */
  0x0000,   /* U+13A3 */
  0x0000,   /* U+13A4 */
  0x0000,   /* U+13A5 */
  0x0000,   /* U+13A6 */
  0x0000,   /* U+13A7 */
  0x0000,   /* U+13A8 */
  0x0000,   /* U+13A9 */
  0x0000,   /* U+13AA */
  0x0000,   /* U+13AB */
  0x0000,   /* U+13AC */
  0x0000,   /* U+13AD */
  0x0000,   /* U+13AE */
  0x0000,   /* U+13AF */
  0x0000,   /* U+13B0 */
  0x0000,   /* U+13B1 */
  0x0000,   /* U+13B2 */
  0x0000,   /* U+13B3 */
  0x0000,   /* U+13B4 */
  0x0000,   /* U+13B5 */
  0x0000,   /* U+13B6 */
  0x0000,   /* U+13B7 */
  0x0000,   /* U+13B8 */
  0x0000,   /* U+13B9 */
  0x0000,   /* U+13BA */
  0x0000,   /* U+13BB */
  0x0000,   /* U+13BC */
  0x0000,   /* U+13BD */
  0x0000,   /* U+13BE */
  0x0000,   /* U+13BF */
  0x0000,   /* U+13C0 */
  0x0000,   /* U+13C1 */
  0x0000,   /* U+13C2 */
  0x0000,   /* U+13C3 */
  0x0000,   /* U+13C4 */
  0x0000,   /* U+13C5 */
  0x0000,   /* U+13C6 */
  0x0000,   /* U+13C7 */
  0x0000,   /* U+13C8 */
  0x0000,   /* U+13C9 */
  0x0000,   /* U+13CA */
  0x0000,   /* U+13CB */
  0x0000,   /* U+13CC */
  0x0000,   /* U+13CD */
  0x0000,   /* U+13CE */
  0x0000,   /* U+13CF */
  0x0000,   /* U+13D0 */
  0x0000,   /* U+13D1 */
  0x0000,   /* U+13D2 */
  0x0000,   /* U+13D3 */
  0x0000,   /* U+13D4 */
  0x0000,   /* U+13D5 */
  0x0000,   /* U+13D6 */
  0x0000,   /* U+13D7 */
  0x0000,   /* U+13D8 */
  0x0000,   /* U+13D9 */
  0x0000,   /* U+13DA */
  0x0000,   /* U+13DB */
  0x0000,   /* U+13DC */
  0x0000,   /* U+13DD */
  0x0000,   /* U+13DE */
  0x0000,   /* U+13DF */
  0x0000,   /* U+13E0 */
  0x0000,   /* U+13E1 */
  0x0000,   /* U+13E2 */
  0x0000,   /* U+13E3 */
  0x0000,   /* U+13E4 */
  0x0000,   /* U+13E5 */
  0x0000,   /* U+13E6 */
  0x0000,   /* U+13E7 */
  0x0000,   /* U+13E8 */
  0x0000,   /* U+13E9 */
  0x0000,   /* U+13EA */
  0x0000,   /* U+13EB */
  0x0000,   /* U+13EC */
  0x0000,   /* U+13ED */
  0x0000,   /* U+13EE */
  0x0000,   /* U+13EF */
  0x0000,   /* U+13F0 */
  0x0000,   /* U+13F1 */
  0x0000,   /* U+13F2 */
  0x0000,   /* U+13F3 */
  0x0000,   /* U+13F4 */
  0x0000,   /* U+13F5 */
  0x0000,   /* U+13F6 */
  0x0000,   /* U+13F7 */
  0x0000,   /* U+13F8 */
  0x0000,   /* U+13F9 */
  0x0000,   /* U+13FA */
  0x0000,   /* U+13FB */
  0x0000,   /* U+13FC */
  0x0000,   /* U+13FD */
  0x0000,   /* U+13FE */
  0x0000,   /* U+13FF */
};

uint16 uca900_p014[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x020F,   /* U+1400 */
  0x33F9,   /* U+1401 */
  0x33FA,   /* U+1402 */
  0x33FB,   /* U+1403 */
  0x33FC,   /* U+1404 */
  0x33FD,   /* U+1405 */
  0x33FE,   /* U+1406 */
  0x33FF,   /* U+1407 */
  0x3400,   /* U+1408 */
  0x3401,   /* U+1409 */
  0x3402,   /* U+140A */
  0x3403,   /* U+140B */
  0x3404,   /* U+140C */
  0x3405,   /* U+140D */
  0x3406,   /* U+140E */
  0x3407,   /* U+140F */
  0x3408,   /* U+1410 */
  0x3409,   /* U+1411 */
  0x340A,   /* U+1412 */
  0x340B,   /* U+1413 */
  0x340C,   /* U+1414 */
  0x340D,   /* U+1415 */
  0x340E,   /* U+1416 */
  0x340F,   /* U+1417 */
  0x3410,   /* U+1418 */
  0x3411,   /* U+1419 */
  0x3412,   /* U+141A */
  0x3413,   /* U+141B */
  0x3414,   /* U+141C */
  0x3415,   /* U+141D */
  0x3416,   /* U+141E */
  0x3417,   /* U+141F */
  0x3418,   /* U+1420 */
  0x3419,   /* U+1421 */
  0x341A,   /* U+1422 */
  0x341B,   /* U+1423 */
  0x341C,   /* U+1424 */
  0x341D,   /* U+1425 */
  0x341E,   /* U+1426 */
  0x341F,   /* U+1427 */
  0x3420,   /* U+1428 */
  0x3421,   /* U+1429 */
  0x3422,   /* U+142A */
  0x3423,   /* U+142B */
  0x3424,   /* U+142C */
  0x3425,   /* U+142D */
  0x3426,   /* U+142E */
  0x3427,   /* U+142F */
  0x3428,   /* U+1430 */
  0x3429,   /* U+1431 */
  0x342A,   /* U+1432 */
  0x342B,   /* U+1433 */
  0x342C,   /* U+1434 */
  0x342D,   /* U+1435 */
  0x342E,   /* U+1436 */
  0x342F,   /* U+1437 */
  0x3430,   /* U+1438 */
  0x3431,   /* U+1439 */
  0x3432,   /* U+143A */
  0x3433,   /* U+143B */
  0x3434,   /* U+143C */
  0x3435,   /* U+143D */
  0x3436,   /* U+143E */
  0x3437,   /* U+143F */
  0x3438,   /* U+1440 */
  0x3439,   /* U+1441 */
  0x343A,   /* U+1442 */
  0x343B,   /* U+1443 */
  0x343C,   /* U+1444 */
  0x343D,   /* U+1445 */
  0x343E,   /* U+1446 */
  0x343F,   /* U+1447 */
  0x3440,   /* U+1448 */
  0x3441,   /* U+1449 */
  0x3442,   /* U+144A */
  0x3443,   /* U+144B */
  0x3444,   /* U+144C */
  0x3445,   /* U+144D */
  0x3446,   /* U+144E */
  0x3447,   /* U+144F */
  0x3448,   /* U+1450 */
  0x3449,   /* U+1451 */
  0x344A,   /* U+1452 */
  0x344B,   /* U+1453 */
  0x344C,   /* U+1454 */
  0x344D,   /* U+1455 */
  0x344E,   /* U+1456 */
  0x344F,   /* U+1457 */
  0x3450,   /* U+1458 */
  0x3451,   /* U+1459 */
  0x3452,   /* U+145A */
  0x3453,   /* U+145B */
  0x3454,   /* U+145C */
  0x3455,   /* U+145D */
  0x3456,   /* U+145E */
  0x3457,   /* U+145F */
  0x3458,   /* U+1460 */
  0x3459,   /* U+1461 */
  0x345A,   /* U+1462 */
  0x345B,   /* U+1463 */
  0x345C,   /* U+1464 */
  0x345D,   /* U+1465 */
  0x345E,   /* U+1466 */
  0x345F,   /* U+1467 */
  0x3460,   /* U+1468 */
  0x3461,   /* U+1469 */
  0x3462,   /* U+146A */
  0x3463,   /* U+146B */
  0x3464,   /* U+146C */
  0x3465,   /* U+146D */
  0x3466,   /* U+146E */
  0x3467,   /* U+146F */
  0x3468,   /* U+1470 */
  0x3469,   /* U+1471 */
  0x346A,   /* U+1472 */
  0x346B,   /* U+1473 */
  0x346C,   /* U+1474 */
  0x346D,   /* U+1475 */
  0x346E,   /* U+1476 */
  0x346F,   /* U+1477 */
  0x3470,   /* U+1478 */
  0x3471,   /* U+1479 */
  0x3472,   /* U+147A */
  0x3473,   /* U+147B */
  0x3474,   /* U+147C */
  0x3475,   /* U+147D */
  0x3476,   /* U+147E */
  0x3477,   /* U+147F */
  0x3478,   /* U+1480 */
  0x3479,   /* U+1481 */
  0x347A,   /* U+1482 */
  0x347B,   /* U+1483 */
  0x347C,   /* U+1484 */
  0x347D,   /* U+1485 */
  0x347E,   /* U+1486 */
  0x347F,   /* U+1487 */
  0x3480,   /* U+1488 */
  0x3481,   /* U+1489 */
  0x3482,   /* U+148A */
  0x3483,   /* U+148B */
  0x3484,   /* U+148C */
  0x3485,   /* U+148D */
  0x3486,   /* U+148E */
  0x3487,   /* U+148F */
  0x3488,   /* U+1490 */
  0x3489,   /* U+1491 */
  0x348A,   /* U+1492 */
  0x348B,   /* U+1493 */
  0x348C,   /* U+1494 */
  0x348D,   /* U+1495 */
  0x348E,   /* U+1496 */
  0x348F,   /* U+1497 */
  0x3490,   /* U+1498 */
  0x3491,   /* U+1499 */
  0x3492,   /* U+149A */
  0x3493,   /* U+149B */
  0x3494,   /* U+149C */
  0x3495,   /* U+149D */
  0x3496,   /* U+149E */
  0x3497,   /* U+149F */
  0x3498,   /* U+14A0 */
  0x3499,   /* U+14A1 */
  0x349A,   /* U+14A2 */
  0x349B,   /* U+14A3 */
  0x349C,   /* U+14A4 */
  0x349D,   /* U+14A5 */
  0x349E,   /* U+14A6 */
  0x349F,   /* U+14A7 */
  0x34A0,   /* U+14A8 */
  0x34A1,   /* U+14A9 */
  0x34A2,   /* U+14AA */
  0x34A3,   /* U+14AB */
  0x34A4,   /* U+14AC */
  0x34A5,   /* U+14AD */
  0x34A6,   /* U+14AE */
  0x34A7,   /* U+14AF */
  0x34A8,   /* U+14B0 */
  0x34A9,   /* U+14B1 */
  0x34AA,   /* U+14B2 */
  0x34AB,   /* U+14B3 */
  0x34AC,   /* U+14B4 */
  0x34AD,   /* U+14B5 */
  0x34AE,   /* U+14B6 */
  0x34AF,   /* U+14B7 */
  0x34B0,   /* U+14B8 */
  0x34B1,   /* U+14B9 */
  0x34B2,   /* U+14BA */
  0x34B3,   /* U+14BB */
  0x34B4,   /* U+14BC */
  0x34B5,   /* U+14BD */
  0x34B6,   /* U+14BE */
  0x34B7,   /* U+14BF */
  0x34B8,   /* U+14C0 */
  0x34B9,   /* U+14C1 */
  0x34BA,   /* U+14C2 */
  0x34BB,   /* U+14C3 */
  0x34BC,   /* U+14C4 */
  0x34BD,   /* U+14C5 */
  0x34BE,   /* U+14C6 */
  0x34BF,   /* U+14C7 */
  0x34C0,   /* U+14C8 */
  0x34C1,   /* U+14C9 */
  0x34C2,   /* U+14CA */
  0x34C3,   /* U+14CB */
  0x34C4,   /* U+14CC */
  0x34C5,   /* U+14CD */
  0x34C6,   /* U+14CE */
  0x34C7,   /* U+14CF */
  0x34C8,   /* U+14D0 */
  0x34C9,   /* U+14D1 */
  0x34CA,   /* U+14D2 */
  0x34CB,   /* U+14D3 */
  0x34CC,   /* U+14D4 */
  0x34CD,   /* U+14D5 */
  0x34CE,   /* U+14D6 */
  0x34CF,   /* U+14D7 */
  0x34D0,   /* U+14D8 */
  0x34D1,   /* U+14D9 */
  0x34D2,   /* U+14DA */
  0x34D3,   /* U+14DB */
  0x34D4,   /* U+14DC */
  0x34D5,   /* U+14DD */
  0x34D6,   /* U+14DE */
  0x34D7,   /* U+14DF */
  0x34D8,   /* U+14E0 */
  0x34D9,   /* U+14E1 */
  0x34DA,   /* U+14E2 */
  0x34DB,   /* U+14E3 */
  0x34DC,   /* U+14E4 */
  0x34DD,   /* U+14E5 */
  0x34DE,   /* U+14E6 */
  0x34DF,   /* U+14E7 */
  0x34E0,   /* U+14E8 */
  0x34E1,   /* U+14E9 */
  0x34E2,   /* U+14EA */
  0x34E3,   /* U+14EB */
  0x34E4,   /* U+14EC */
  0x34E5,   /* U+14ED */
  0x34E6,   /* U+14EE */
  0x34E7,   /* U+14EF */
  0x34E8,   /* U+14F0 */
  0x34E9,   /* U+14F1 */
  0x34EA,   /* U+14F2 */
  0x34EB,   /* U+14F3 */
  0x34EC,   /* U+14F4 */
  0x34ED,   /* U+14F5 */
  0x34EE,   /* U+14F6 */
  0x34EF,   /* U+14F7 */
  0x34F0,   /* U+14F8 */
  0x34F1,   /* U+14F9 */
  0x34F2,   /* U+14FA */
  0x34F3,   /* U+14FB */
  0x34F4,   /* U+14FC */
  0x34F5,   /* U+14FD */
  0x34F6,   /* U+14FE */
  0x34F7,   /* U+14FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1400 */
  0x0020,   /* U+1401 */
  0x0020,   /* U+1402 */
  0x0020,   /* U+1403 */
  0x0020,   /* U+1404 */
  0x0020,   /* U+1405 */
  0x0020,   /* U+1406 */
  0x0020,   /* U+1407 */
  0x0020,   /* U+1408 */
  0x0020,   /* U+1409 */
  0x0020,   /* U+140A */
  0x0020,   /* U+140B */
  0x0020,   /* U+140C */
  0x0020,   /* U+140D */
  0x0020,   /* U+140E */
  0x0020,   /* U+140F */
  0x0020,   /* U+1410 */
  0x0020,   /* U+1411 */
  0x0020,   /* U+1412 */
  0x0020,   /* U+1413 */
  0x0020,   /* U+1414 */
  0x0020,   /* U+1415 */
  0x0020,   /* U+1416 */
  0x0020,   /* U+1417 */
  0x0020,   /* U+1418 */
  0x0020,   /* U+1419 */
  0x0020,   /* U+141A */
  0x0020,   /* U+141B */
  0x0020,   /* U+141C */
  0x0020,   /* U+141D */
  0x0020,   /* U+141E */
  0x0020,   /* U+141F */
  0x0020,   /* U+1420 */
  0x0020,   /* U+1421 */
  0x0020,   /* U+1422 */
  0x0020,   /* U+1423 */
  0x0020,   /* U+1424 */
  0x0020,   /* U+1425 */
  0x0020,   /* U+1426 */
  0x0020,   /* U+1427 */
  0x0020,   /* U+1428 */
  0x0020,   /* U+1429 */
  0x0020,   /* U+142A */
  0x0020,   /* U+142B */
  0x0020,   /* U+142C */
  0x0020,   /* U+142D */
  0x0020,   /* U+142E */
  0x0020,   /* U+142F */
  0x0020,   /* U+1430 */
  0x0020,   /* U+1431 */
  0x0020,   /* U+1432 */
  0x0020,   /* U+1433 */
  0x0020,   /* U+1434 */
  0x0020,   /* U+1435 */
  0x0020,   /* U+1436 */
  0x0020,   /* U+1437 */
  0x0020,   /* U+1438 */
  0x0020,   /* U+1439 */
  0x0020,   /* U+143A */
  0x0020,   /* U+143B */
  0x0020,   /* U+143C */
  0x0020,   /* U+143D */
  0x0020,   /* U+143E */
  0x0020,   /* U+143F */
  0x0020,   /* U+1440 */
  0x0020,   /* U+1441 */
  0x0020,   /* U+1442 */
  0x0020,   /* U+1443 */
  0x0020,   /* U+1444 */
  0x0020,   /* U+1445 */
  0x0020,   /* U+1446 */
  0x0020,   /* U+1447 */
  0x0020,   /* U+1448 */
  0x0020,   /* U+1449 */
  0x0020,   /* U+144A */
  0x0020,   /* U+144B */
  0x0020,   /* U+144C */
  0x0020,   /* U+144D */
  0x0020,   /* U+144E */
  0x0020,   /* U+144F */
  0x0020,   /* U+1450 */
  0x0020,   /* U+1451 */
  0x0020,   /* U+1452 */
  0x0020,   /* U+1453 */
  0x0020,   /* U+1454 */
  0x0020,   /* U+1455 */
  0x0020,   /* U+1456 */
  0x0020,   /* U+1457 */
  0x0020,   /* U+1458 */
  0x0020,   /* U+1459 */
  0x0020,   /* U+145A */
  0x0020,   /* U+145B */
  0x0020,   /* U+145C */
  0x0020,   /* U+145D */
  0x0020,   /* U+145E */
  0x0020,   /* U+145F */
  0x0020,   /* U+1460 */
  0x0020,   /* U+1461 */
  0x0020,   /* U+1462 */
  0x0020,   /* U+1463 */
  0x0020,   /* U+1464 */
  0x0020,   /* U+1465 */
  0x0020,   /* U+1466 */
  0x0020,   /* U+1467 */
  0x0020,   /* U+1468 */
  0x0020,   /* U+1469 */
  0x0020,   /* U+146A */
  0x0020,   /* U+146B */
  0x0020,   /* U+146C */
  0x0020,   /* U+146D */
  0x0020,   /* U+146E */
  0x0020,   /* U+146F */
  0x0020,   /* U+1470 */
  0x0020,   /* U+1471 */
  0x0020,   /* U+1472 */
  0x0020,   /* U+1473 */
  0x0020,   /* U+1474 */
  0x0020,   /* U+1475 */
  0x0020,   /* U+1476 */
  0x0020,   /* U+1477 */
  0x0020,   /* U+1478 */
  0x0020,   /* U+1479 */
  0x0020,   /* U+147A */
  0x0020,   /* U+147B */
  0x0020,   /* U+147C */
  0x0020,   /* U+147D */
  0x0020,   /* U+147E */
  0x0020,   /* U+147F */
  0x0020,   /* U+1480 */
  0x0020,   /* U+1481 */
  0x0020,   /* U+1482 */
  0x0020,   /* U+1483 */
  0x0020,   /* U+1484 */
  0x0020,   /* U+1485 */
  0x0020,   /* U+1486 */
  0x0020,   /* U+1487 */
  0x0020,   /* U+1488 */
  0x0020,   /* U+1489 */
  0x0020,   /* U+148A */
  0x0020,   /* U+148B */
  0x0020,   /* U+148C */
  0x0020,   /* U+148D */
  0x0020,   /* U+148E */
  0x0020,   /* U+148F */
  0x0020,   /* U+1490 */
  0x0020,   /* U+1491 */
  0x0020,   /* U+1492 */
  0x0020,   /* U+1493 */
  0x0020,   /* U+1494 */
  0x0020,   /* U+1495 */
  0x0020,   /* U+1496 */
  0x0020,   /* U+1497 */
  0x0020,   /* U+1498 */
  0x0020,   /* U+1499 */
  0x0020,   /* U+149A */
  0x0020,   /* U+149B */
  0x0020,   /* U+149C */
  0x0020,   /* U+149D */
  0x0020,   /* U+149E */
  0x0020,   /* U+149F */
  0x0020,   /* U+14A0 */
  0x0020,   /* U+14A1 */
  0x0020,   /* U+14A2 */
  0x0020,   /* U+14A3 */
  0x0020,   /* U+14A4 */
  0x0020,   /* U+14A5 */
  0x0020,   /* U+14A6 */
  0x0020,   /* U+14A7 */
  0x0020,   /* U+14A8 */
  0x0020,   /* U+14A9 */
  0x0020,   /* U+14AA */
  0x0020,   /* U+14AB */
  0x0020,   /* U+14AC */
  0x0020,   /* U+14AD */
  0x0020,   /* U+14AE */
  0x0020,   /* U+14AF */
  0x0020,   /* U+14B0 */
  0x0020,   /* U+14B1 */
  0x0020,   /* U+14B2 */
  0x0020,   /* U+14B3 */
  0x0020,   /* U+14B4 */
  0x0020,   /* U+14B5 */
  0x0020,   /* U+14B6 */
  0x0020,   /* U+14B7 */
  0x0020,   /* U+14B8 */
  0x0020,   /* U+14B9 */
  0x0020,   /* U+14BA */
  0x0020,   /* U+14BB */
  0x0020,   /* U+14BC */
  0x0020,   /* U+14BD */
  0x0020,   /* U+14BE */
  0x0020,   /* U+14BF */
  0x0020,   /* U+14C0 */
  0x0020,   /* U+14C1 */
  0x0020,   /* U+14C2 */
  0x0020,   /* U+14C3 */
  0x0020,   /* U+14C4 */
  0x0020,   /* U+14C5 */
  0x0020,   /* U+14C6 */
  0x0020,   /* U+14C7 */
  0x0020,   /* U+14C8 */
  0x0020,   /* U+14C9 */
  0x0020,   /* U+14CA */
  0x0020,   /* U+14CB */
  0x0020,   /* U+14CC */
  0x0020,   /* U+14CD */
  0x0020,   /* U+14CE */
  0x0020,   /* U+14CF */
  0x0020,   /* U+14D0 */
  0x0020,   /* U+14D1 */
  0x0020,   /* U+14D2 */
  0x0020,   /* U+14D3 */
  0x0020,   /* U+14D4 */
  0x0020,   /* U+14D5 */
  0x0020,   /* U+14D6 */
  0x0020,   /* U+14D7 */
  0x0020,   /* U+14D8 */
  0x0020,   /* U+14D9 */
  0x0020,   /* U+14DA */
  0x0020,   /* U+14DB */
  0x0020,   /* U+14DC */
  0x0020,   /* U+14DD */
  0x0020,   /* U+14DE */
  0x0020,   /* U+14DF */
  0x0020,   /* U+14E0 */
  0x0020,   /* U+14E1 */
  0x0020,   /* U+14E2 */
  0x0020,   /* U+14E3 */
  0x0020,   /* U+14E4 */
  0x0020,   /* U+14E5 */
  0x0020,   /* U+14E6 */
  0x0020,   /* U+14E7 */
  0x0020,   /* U+14E8 */
  0x0020,   /* U+14E9 */
  0x0020,   /* U+14EA */
  0x0020,   /* U+14EB */
  0x0020,   /* U+14EC */
  0x0020,   /* U+14ED */
  0x0020,   /* U+14EE */
  0x0020,   /* U+14EF */
  0x0020,   /* U+14F0 */
  0x0020,   /* U+14F1 */
  0x0020,   /* U+14F2 */
  0x0020,   /* U+14F3 */
  0x0020,   /* U+14F4 */
  0x0020,   /* U+14F5 */
  0x0020,   /* U+14F6 */
  0x0020,   /* U+14F7 */
  0x0020,   /* U+14F8 */
  0x0020,   /* U+14F9 */
  0x0020,   /* U+14FA */
  0x0020,   /* U+14FB */
  0x0020,   /* U+14FC */
  0x0020,   /* U+14FD */
  0x0020,   /* U+14FE */
  0x0020,   /* U+14FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1400 */
  0x0002,   /* U+1401 */
  0x0002,   /* U+1402 */
  0x0002,   /* U+1403 */
  0x0002,   /* U+1404 */
  0x0002,   /* U+1405 */
  0x0002,   /* U+1406 */
  0x0002,   /* U+1407 */
  0x0002,   /* U+1408 */
  0x0002,   /* U+1409 */
  0x0002,   /* U+140A */
  0x0002,   /* U+140B */
  0x0002,   /* U+140C */
  0x0002,   /* U+140D */
  0x0002,   /* U+140E */
  0x0002,   /* U+140F */
  0x0002,   /* U+1410 */
  0x0002,   /* U+1411 */
  0x0002,   /* U+1412 */
  0x0002,   /* U+1413 */
  0x0002,   /* U+1414 */
  0x0002,   /* U+1415 */
  0x0002,   /* U+1416 */
  0x0002,   /* U+1417 */
  0x0002,   /* U+1418 */
  0x0002,   /* U+1419 */
  0x0002,   /* U+141A */
  0x0002,   /* U+141B */
  0x0002,   /* U+141C */
  0x0002,   /* U+141D */
  0x0002,   /* U+141E */
  0x0002,   /* U+141F */
  0x0002,   /* U+1420 */
  0x0002,   /* U+1421 */
  0x0002,   /* U+1422 */
  0x0002,   /* U+1423 */
  0x0002,   /* U+1424 */
  0x0002,   /* U+1425 */
  0x0002,   /* U+1426 */
  0x0002,   /* U+1427 */
  0x0002,   /* U+1428 */
  0x0002,   /* U+1429 */
  0x0002,   /* U+142A */
  0x0002,   /* U+142B */
  0x0002,   /* U+142C */
  0x0002,   /* U+142D */
  0x0002,   /* U+142E */
  0x0002,   /* U+142F */
  0x0002,   /* U+1430 */
  0x0002,   /* U+1431 */
  0x0002,   /* U+1432 */
  0x0002,   /* U+1433 */
  0x0002,   /* U+1434 */
  0x0002,   /* U+1435 */
  0x0002,   /* U+1436 */
  0x0002,   /* U+1437 */
  0x0002,   /* U+1438 */
  0x0002,   /* U+1439 */
  0x0002,   /* U+143A */
  0x0002,   /* U+143B */
  0x0002,   /* U+143C */
  0x0002,   /* U+143D */
  0x0002,   /* U+143E */
  0x0002,   /* U+143F */
  0x0002,   /* U+1440 */
  0x0002,   /* U+1441 */
  0x0002,   /* U+1442 */
  0x0002,   /* U+1443 */
  0x0002,   /* U+1444 */
  0x0002,   /* U+1445 */
  0x0002,   /* U+1446 */
  0x0002,   /* U+1447 */
  0x0002,   /* U+1448 */
  0x0002,   /* U+1449 */
  0x0002,   /* U+144A */
  0x0002,   /* U+144B */
  0x0002,   /* U+144C */
  0x0002,   /* U+144D */
  0x0002,   /* U+144E */
  0x0002,   /* U+144F */
  0x0002,   /* U+1450 */
  0x0002,   /* U+1451 */
  0x0002,   /* U+1452 */
  0x0002,   /* U+1453 */
  0x0002,   /* U+1454 */
  0x0002,   /* U+1455 */
  0x0002,   /* U+1456 */
  0x0002,   /* U+1457 */
  0x0002,   /* U+1458 */
  0x0002,   /* U+1459 */
  0x0002,   /* U+145A */
  0x0002,   /* U+145B */
  0x0002,   /* U+145C */
  0x0002,   /* U+145D */
  0x0002,   /* U+145E */
  0x0002,   /* U+145F */
  0x0002,   /* U+1460 */
  0x0002,   /* U+1461 */
  0x0002,   /* U+1462 */
  0x0002,   /* U+1463 */
  0x0002,   /* U+1464 */
  0x0002,   /* U+1465 */
  0x0002,   /* U+1466 */
  0x0002,   /* U+1467 */
  0x0002,   /* U+1468 */
  0x0002,   /* U+1469 */
  0x0002,   /* U+146A */
  0x0002,   /* U+146B */
  0x0002,   /* U+146C */
  0x0002,   /* U+146D */
  0x0002,   /* U+146E */
  0x0002,   /* U+146F */
  0x0002,   /* U+1470 */
  0x0002,   /* U+1471 */
  0x0002,   /* U+1472 */
  0x0002,   /* U+1473 */
  0x0002,   /* U+1474 */
  0x0002,   /* U+1475 */
  0x0002,   /* U+1476 */
  0x0002,   /* U+1477 */
  0x0002,   /* U+1478 */
  0x0002,   /* U+1479 */
  0x0002,   /* U+147A */
  0x0002,   /* U+147B */
  0x0002,   /* U+147C */
  0x0002,   /* U+147D */
  0x0002,   /* U+147E */
  0x0002,   /* U+147F */
  0x0002,   /* U+1480 */
  0x0002,   /* U+1481 */
  0x0002,   /* U+1482 */
  0x0002,   /* U+1483 */
  0x0002,   /* U+1484 */
  0x0002,   /* U+1485 */
  0x0002,   /* U+1486 */
  0x0002,   /* U+1487 */
  0x0002,   /* U+1488 */
  0x0002,   /* U+1489 */
  0x0002,   /* U+148A */
  0x0002,   /* U+148B */
  0x0002,   /* U+148C */
  0x0002,   /* U+148D */
  0x0002,   /* U+148E */
  0x0002,   /* U+148F */
  0x0002,   /* U+1490 */
  0x0002,   /* U+1491 */
  0x0002,   /* U+1492 */
  0x0002,   /* U+1493 */
  0x0002,   /* U+1494 */
  0x0002,   /* U+1495 */
  0x0002,   /* U+1496 */
  0x0002,   /* U+1497 */
  0x0002,   /* U+1498 */
  0x0002,   /* U+1499 */
  0x0002,   /* U+149A */
  0x0002,   /* U+149B */
  0x0002,   /* U+149C */
  0x0002,   /* U+149D */
  0x0002,   /* U+149E */
  0x0002,   /* U+149F */
  0x0002,   /* U+14A0 */
  0x0002,   /* U+14A1 */
  0x0002,   /* U+14A2 */
  0x0002,   /* U+14A3 */
  0x0002,   /* U+14A4 */
  0x0002,   /* U+14A5 */
  0x0002,   /* U+14A6 */
  0x0002,   /* U+14A7 */
  0x0002,   /* U+14A8 */
  0x0002,   /* U+14A9 */
  0x0002,   /* U+14AA */
  0x0002,   /* U+14AB */
  0x0002,   /* U+14AC */
  0x0002,   /* U+14AD */
  0x0002,   /* U+14AE */
  0x0002,   /* U+14AF */
  0x0002,   /* U+14B0 */
  0x0002,   /* U+14B1 */
  0x0002,   /* U+14B2 */
  0x0002,   /* U+14B3 */
  0x0002,   /* U+14B4 */
  0x0002,   /* U+14B5 */
  0x0002,   /* U+14B6 */
  0x0002,   /* U+14B7 */
  0x0002,   /* U+14B8 */
  0x0002,   /* U+14B9 */
  0x0002,   /* U+14BA */
  0x0002,   /* U+14BB */
  0x0002,   /* U+14BC */
  0x0002,   /* U+14BD */
  0x0002,   /* U+14BE */
  0x0002,   /* U+14BF */
  0x0002,   /* U+14C0 */
  0x0002,   /* U+14C1 */
  0x0002,   /* U+14C2 */
  0x0002,   /* U+14C3 */
  0x0002,   /* U+14C4 */
  0x0002,   /* U+14C5 */
  0x0002,   /* U+14C6 */
  0x0002,   /* U+14C7 */
  0x0002,   /* U+14C8 */
  0x0002,   /* U+14C9 */
  0x0002,   /* U+14CA */
  0x0002,   /* U+14CB */
  0x0002,   /* U+14CC */
  0x0002,   /* U+14CD */
  0x0002,   /* U+14CE */
  0x0002,   /* U+14CF */
  0x0002,   /* U+14D0 */
  0x0002,   /* U+14D1 */
  0x0002,   /* U+14D2 */
  0x0002,   /* U+14D3 */
  0x0002,   /* U+14D4 */
  0x0002,   /* U+14D5 */
  0x0002,   /* U+14D6 */
  0x0002,   /* U+14D7 */
  0x0002,   /* U+14D8 */
  0x0002,   /* U+14D9 */
  0x0002,   /* U+14DA */
  0x0002,   /* U+14DB */
  0x0002,   /* U+14DC */
  0x0002,   /* U+14DD */
  0x0002,   /* U+14DE */
  0x0002,   /* U+14DF */
  0x0002,   /* U+14E0 */
  0x0002,   /* U+14E1 */
  0x0002,   /* U+14E2 */
  0x0002,   /* U+14E3 */
  0x0002,   /* U+14E4 */
  0x0002,   /* U+14E5 */
  0x0002,   /* U+14E6 */
  0x0002,   /* U+14E7 */
  0x0002,   /* U+14E8 */
  0x0002,   /* U+14E9 */
  0x0002,   /* U+14EA */
  0x0002,   /* U+14EB */
  0x0002,   /* U+14EC */
  0x0002,   /* U+14ED */
  0x0002,   /* U+14EE */
  0x0002,   /* U+14EF */
  0x0002,   /* U+14F0 */
  0x0002,   /* U+14F1 */
  0x0002,   /* U+14F2 */
  0x0002,   /* U+14F3 */
  0x0002,   /* U+14F4 */
  0x0002,   /* U+14F5 */
  0x0002,   /* U+14F6 */
  0x0002,   /* U+14F7 */
  0x0002,   /* U+14F8 */
  0x0002,   /* U+14F9 */
  0x0002,   /* U+14FA */
  0x0002,   /* U+14FB */
  0x0002,   /* U+14FC */
  0x0002,   /* U+14FD */
  0x0002,   /* U+14FE */
  0x0002,   /* U+14FF */
};

uint16 uca900_p015[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x34F8,   /* U+1500 */
  0x34F9,   /* U+1501 */
  0x34FA,   /* U+1502 */
  0x34FB,   /* U+1503 */
  0x34FC,   /* U+1504 */
  0x34FD,   /* U+1505 */
  0x34FE,   /* U+1506 */
  0x34FF,   /* U+1507 */
  0x3500,   /* U+1508 */
  0x3501,   /* U+1509 */
  0x3502,   /* U+150A */
  0x3503,   /* U+150B */
  0x3504,   /* U+150C */
  0x3505,   /* U+150D */
  0x3506,   /* U+150E */
  0x3507,   /* U+150F */
  0x3508,   /* U+1510 */
  0x3509,   /* U+1511 */
  0x350A,   /* U+1512 */
  0x350B,   /* U+1513 */
  0x350C,   /* U+1514 */
  0x350D,   /* U+1515 */
  0x350E,   /* U+1516 */
  0x350F,   /* U+1517 */
  0x3510,   /* U+1518 */
  0x3511,   /* U+1519 */
  0x3512,   /* U+151A */
  0x3513,   /* U+151B */
  0x3514,   /* U+151C */
  0x3515,   /* U+151D */
  0x3516,   /* U+151E */
  0x3517,   /* U+151F */
  0x3518,   /* U+1520 */
  0x3519,   /* U+1521 */
  0x351A,   /* U+1522 */
  0x351B,   /* U+1523 */
  0x351C,   /* U+1524 */
  0x351D,   /* U+1525 */
  0x351E,   /* U+1526 */
  0x351F,   /* U+1527 */
  0x3520,   /* U+1528 */
  0x3521,   /* U+1529 */
  0x3522,   /* U+152A */
  0x3523,   /* U+152B */
  0x3524,   /* U+152C */
  0x3525,   /* U+152D */
  0x3526,   /* U+152E */
  0x3527,   /* U+152F */
  0x3528,   /* U+1530 */
  0x3529,   /* U+1531 */
  0x352A,   /* U+1532 */
  0x352B,   /* U+1533 */
  0x352C,   /* U+1534 */
  0x352D,   /* U+1535 */
  0x352E,   /* U+1536 */
  0x352F,   /* U+1537 */
  0x3530,   /* U+1538 */
  0x3531,   /* U+1539 */
  0x3532,   /* U+153A */
  0x3533,   /* U+153B */
  0x3534,   /* U+153C */
  0x3535,   /* U+153D */
  0x3536,   /* U+153E */
  0x3537,   /* U+153F */
  0x3538,   /* U+1540 */
  0x3539,   /* U+1541 */
  0x353A,   /* U+1542 */
  0x353B,   /* U+1543 */
  0x353C,   /* U+1544 */
  0x353D,   /* U+1545 */
  0x353E,   /* U+1546 */
  0x353F,   /* U+1547 */
  0x3540,   /* U+1548 */
  0x3541,   /* U+1549 */
  0x3542,   /* U+154A */
  0x3543,   /* U+154B */
  0x3544,   /* U+154C */
  0x3545,   /* U+154D */
  0x3546,   /* U+154E */
  0x3547,   /* U+154F */
  0x3548,   /* U+1550 */
  0x3549,   /* U+1551 */
  0x354A,   /* U+1552 */
  0x354B,   /* U+1553 */
  0x354C,   /* U+1554 */
  0x354D,   /* U+1555 */
  0x354E,   /* U+1556 */
  0x354F,   /* U+1557 */
  0x3550,   /* U+1558 */
  0x3551,   /* U+1559 */
  0x3552,   /* U+155A */
  0x3553,   /* U+155B */
  0x3554,   /* U+155C */
  0x3555,   /* U+155D */
  0x3556,   /* U+155E */
  0x3557,   /* U+155F */
  0x3558,   /* U+1560 */
  0x3559,   /* U+1561 */
  0x355A,   /* U+1562 */
  0x355B,   /* U+1563 */
  0x355C,   /* U+1564 */
  0x355D,   /* U+1565 */
  0x355E,   /* U+1566 */
  0x355F,   /* U+1567 */
  0x3560,   /* U+1568 */
  0x3561,   /* U+1569 */
  0x3562,   /* U+156A */
  0x3563,   /* U+156B */
  0x3564,   /* U+156C */
  0x3565,   /* U+156D */
  0x3566,   /* U+156E */
  0x3567,   /* U+156F */
  0x3568,   /* U+1570 */
  0x3569,   /* U+1571 */
  0x356A,   /* U+1572 */
  0x356B,   /* U+1573 */
  0x356C,   /* U+1574 */
  0x356D,   /* U+1575 */
  0x356E,   /* U+1576 */
  0x356F,   /* U+1577 */
  0x3570,   /* U+1578 */
  0x3571,   /* U+1579 */
  0x3572,   /* U+157A */
  0x3573,   /* U+157B */
  0x35A6,   /* U+157C */
  0x3574,   /* U+157D */
  0x3576,   /* U+157E */
  0x3577,   /* U+157F */
  0x3578,   /* U+1580 */
  0x3579,   /* U+1581 */
  0x357A,   /* U+1582 */
  0x357B,   /* U+1583 */
  0x357C,   /* U+1584 */
  0x357D,   /* U+1585 */
  0x357E,   /* U+1586 */
  0x357F,   /* U+1587 */
  0x3580,   /* U+1588 */
  0x3581,   /* U+1589 */
  0x3582,   /* U+158A */
  0x3583,   /* U+158B */
  0x3584,   /* U+158C */
  0x3585,   /* U+158D */
  0x3587,   /* U+158E */
  0x3588,   /* U+158F */
  0x3589,   /* U+1590 */
  0x358A,   /* U+1591 */
  0x358B,   /* U+1592 */
  0x358C,   /* U+1593 */
  0x358D,   /* U+1594 */
  0x358E,   /* U+1595 */
  0x3595,   /* U+1596 */
  0x3596,   /* U+1597 */
  0x3597,   /* U+1598 */
  0x3598,   /* U+1599 */
  0x3599,   /* U+159A */
  0x359A,   /* U+159B */
  0x359B,   /* U+159C */
  0x359C,   /* U+159D */
  0x359D,   /* U+159E */
  0x359E,   /* U+159F */
  0x359F,   /* U+15A0 */
  0x35A0,   /* U+15A1 */
  0x35A1,   /* U+15A2 */
  0x35A2,   /* U+15A3 */
  0x35A3,   /* U+15A4 */
  0x35A4,   /* U+15A5 */
  0x35A5,   /* U+15A6 */
  0x35A7,   /* U+15A7 */
  0x35A8,   /* U+15A8 */
  0x35A9,   /* U+15A9 */
  0x35AA,   /* U+15AA */
  0x35AB,   /* U+15AB */
  0x35AC,   /* U+15AC */
  0x35AD,   /* U+15AD */
  0x35AE,   /* U+15AE */
  0x35AF,   /* U+15AF */
  0x35B0,   /* U+15B0 */
  0x35B1,   /* U+15B1 */
  0x35B2,   /* U+15B2 */
  0x35B3,   /* U+15B3 */
  0x35B4,   /* U+15B4 */
  0x35B5,   /* U+15B5 */
  0x35B6,   /* U+15B6 */
  0x35B7,   /* U+15B7 */
  0x35B8,   /* U+15B8 */
  0x35B9,   /* U+15B9 */
  0x35BA,   /* U+15BA */
  0x35BB,   /* U+15BB */
  0x35BC,   /* U+15BC */
  0x35BD,   /* U+15BD */
  0x35BE,   /* U+15BE */
  0x35BF,   /* U+15BF */
  0x35C0,   /* U+15C0 */
  0x35C1,   /* U+15C1 */
  0x35C2,   /* U+15C2 */
  0x35C3,   /* U+15C3 */
  0x35C4,   /* U+15C4 */
  0x35C5,   /* U+15C5 */
  0x35C6,   /* U+15C6 */
  0x35C7,   /* U+15C7 */
  0x35C8,   /* U+15C8 */
  0x35C9,   /* U+15C9 */
  0x35CA,   /* U+15CA */
  0x35CB,   /* U+15CB */
  0x35CC,   /* U+15CC */
  0x35CD,   /* U+15CD */
  0x35CE,   /* U+15CE */
  0x35CF,   /* U+15CF */
  0x35D0,   /* U+15D0 */
  0x35D1,   /* U+15D1 */
  0x35D2,   /* U+15D2 */
  0x35D3,   /* U+15D3 */
  0x35D4,   /* U+15D4 */
  0x35D5,   /* U+15D5 */
  0x35D6,   /* U+15D6 */
  0x35D7,   /* U+15D7 */
  0x35D8,   /* U+15D8 */
  0x35D9,   /* U+15D9 */
  0x35DA,   /* U+15DA */
  0x35DB,   /* U+15DB */
  0x35DC,   /* U+15DC */
  0x35DD,   /* U+15DD */
  0x35DE,   /* U+15DE */
  0x35DF,   /* U+15DF */
  0x35E0,   /* U+15E0 */
  0x35E1,   /* U+15E1 */
  0x35E2,   /* U+15E2 */
  0x35E3,   /* U+15E3 */
  0x35E4,   /* U+15E4 */
  0x35E5,   /* U+15E5 */
  0x35E6,   /* U+15E6 */
  0x35E7,   /* U+15E7 */
  0x35E8,   /* U+15E8 */
  0x35E9,   /* U+15E9 */
  0x35EA,   /* U+15EA */
  0x35EB,   /* U+15EB */
  0x35EC,   /* U+15EC */
  0x35ED,   /* U+15ED */
  0x35EE,   /* U+15EE */
  0x35EF,   /* U+15EF */
  0x35F0,   /* U+15F0 */
  0x35F1,   /* U+15F1 */
  0x35F2,   /* U+15F2 */
  0x35F3,   /* U+15F3 */
  0x35F4,   /* U+15F4 */
  0x35F5,   /* U+15F5 */
  0x35F6,   /* U+15F6 */
  0x35F7,   /* U+15F7 */
  0x35F8,   /* U+15F8 */
  0x35F9,   /* U+15F9 */
  0x35FA,   /* U+15FA */
  0x35FB,   /* U+15FB */
  0x35FC,   /* U+15FC */
  0x35FD,   /* U+15FD */
  0x35FE,   /* U+15FE */
  0x35FF,   /* U+15FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1500 */
  0x0020,   /* U+1501 */
  0x0020,   /* U+1502 */
  0x0020,   /* U+1503 */
  0x0020,   /* U+1504 */
  0x0020,   /* U+1505 */
  0x0020,   /* U+1506 */
  0x0020,   /* U+1507 */
  0x0020,   /* U+1508 */
  0x0020,   /* U+1509 */
  0x0020,   /* U+150A */
  0x0020,   /* U+150B */
  0x0020,   /* U+150C */
  0x0020,   /* U+150D */
  0x0020,   /* U+150E */
  0x0020,   /* U+150F */
  0x0020,   /* U+1510 */
  0x0020,   /* U+1511 */
  0x0020,   /* U+1512 */
  0x0020,   /* U+1513 */
  0x0020,   /* U+1514 */
  0x0020,   /* U+1515 */
  0x0020,   /* U+1516 */
  0x0020,   /* U+1517 */
  0x0020,   /* U+1518 */
  0x0020,   /* U+1519 */
  0x0020,   /* U+151A */
  0x0020,   /* U+151B */
  0x0020,   /* U+151C */
  0x0020,   /* U+151D */
  0x0020,   /* U+151E */
  0x0020,   /* U+151F */
  0x0020,   /* U+1520 */
  0x0020,   /* U+1521 */
  0x0020,   /* U+1522 */
  0x0020,   /* U+1523 */
  0x0020,   /* U+1524 */
  0x0020,   /* U+1525 */
  0x0020,   /* U+1526 */
  0x0020,   /* U+1527 */
  0x0020,   /* U+1528 */
  0x0020,   /* U+1529 */
  0x0020,   /* U+152A */
  0x0020,   /* U+152B */
  0x0020,   /* U+152C */
  0x0020,   /* U+152D */
  0x0020,   /* U+152E */
  0x0020,   /* U+152F */
  0x0020,   /* U+1530 */
  0x0020,   /* U+1531 */
  0x0020,   /* U+1532 */
  0x0020,   /* U+1533 */
  0x0020,   /* U+1534 */
  0x0020,   /* U+1535 */
  0x0020,   /* U+1536 */
  0x0020,   /* U+1537 */
  0x0020,   /* U+1538 */
  0x0020,   /* U+1539 */
  0x0020,   /* U+153A */
  0x0020,   /* U+153B */
  0x0020,   /* U+153C */
  0x0020,   /* U+153D */
  0x0020,   /* U+153E */
  0x0020,   /* U+153F */
  0x0020,   /* U+1540 */
  0x0020,   /* U+1541 */
  0x0020,   /* U+1542 */
  0x0020,   /* U+1543 */
  0x0020,   /* U+1544 */
  0x0020,   /* U+1545 */
  0x0020,   /* U+1546 */
  0x0020,   /* U+1547 */
  0x0020,   /* U+1548 */
  0x0020,   /* U+1549 */
  0x0020,   /* U+154A */
  0x0020,   /* U+154B */
  0x0020,   /* U+154C */
  0x0020,   /* U+154D */
  0x0020,   /* U+154E */
  0x0020,   /* U+154F */
  0x0020,   /* U+1550 */
  0x0020,   /* U+1551 */
  0x0020,   /* U+1552 */
  0x0020,   /* U+1553 */
  0x0020,   /* U+1554 */
  0x0020,   /* U+1555 */
  0x0020,   /* U+1556 */
  0x0020,   /* U+1557 */
  0x0020,   /* U+1558 */
  0x0020,   /* U+1559 */
  0x0020,   /* U+155A */
  0x0020,   /* U+155B */
  0x0020,   /* U+155C */
  0x0020,   /* U+155D */
  0x0020,   /* U+155E */
  0x0020,   /* U+155F */
  0x0020,   /* U+1560 */
  0x0020,   /* U+1561 */
  0x0020,   /* U+1562 */
  0x0020,   /* U+1563 */
  0x0020,   /* U+1564 */
  0x0020,   /* U+1565 */
  0x0020,   /* U+1566 */
  0x0020,   /* U+1567 */
  0x0020,   /* U+1568 */
  0x0020,   /* U+1569 */
  0x0020,   /* U+156A */
  0x0020,   /* U+156B */
  0x0020,   /* U+156C */
  0x0020,   /* U+156D */
  0x0020,   /* U+156E */
  0x0020,   /* U+156F */
  0x0020,   /* U+1570 */
  0x0020,   /* U+1571 */
  0x0020,   /* U+1572 */
  0x0020,   /* U+1573 */
  0x0020,   /* U+1574 */
  0x0020,   /* U+1575 */
  0x0020,   /* U+1576 */
  0x0020,   /* U+1577 */
  0x0020,   /* U+1578 */
  0x0020,   /* U+1579 */
  0x0020,   /* U+157A */
  0x0020,   /* U+157B */
  0x0020,   /* U+157C */
  0x0020,   /* U+157D */
  0x0020,   /* U+157E */
  0x0020,   /* U+157F */
  0x0020,   /* U+1580 */
  0x0020,   /* U+1581 */
  0x0020,   /* U+1582 */
  0x0020,   /* U+1583 */
  0x0020,   /* U+1584 */
  0x0020,   /* U+1585 */
  0x0020,   /* U+1586 */
  0x0020,   /* U+1587 */
  0x0020,   /* U+1588 */
  0x0020,   /* U+1589 */
  0x0020,   /* U+158A */
  0x0020,   /* U+158B */
  0x0020,   /* U+158C */
  0x0020,   /* U+158D */
  0x0020,   /* U+158E */
  0x0020,   /* U+158F */
  0x0020,   /* U+1590 */
  0x0020,   /* U+1591 */
  0x0020,   /* U+1592 */
  0x0020,   /* U+1593 */
  0x0020,   /* U+1594 */
  0x0020,   /* U+1595 */
  0x0020,   /* U+1596 */
  0x0020,   /* U+1597 */
  0x0020,   /* U+1598 */
  0x0020,   /* U+1599 */
  0x0020,   /* U+159A */
  0x0020,   /* U+159B */
  0x0020,   /* U+159C */
  0x0020,   /* U+159D */
  0x0020,   /* U+159E */
  0x0020,   /* U+159F */
  0x0020,   /* U+15A0 */
  0x0020,   /* U+15A1 */
  0x0020,   /* U+15A2 */
  0x0020,   /* U+15A3 */
  0x0020,   /* U+15A4 */
  0x0020,   /* U+15A5 */
  0x0020,   /* U+15A6 */
  0x0020,   /* U+15A7 */
  0x0020,   /* U+15A8 */
  0x0020,   /* U+15A9 */
  0x0020,   /* U+15AA */
  0x0020,   /* U+15AB */
  0x0020,   /* U+15AC */
  0x0020,   /* U+15AD */
  0x0020,   /* U+15AE */
  0x0020,   /* U+15AF */
  0x0020,   /* U+15B0 */
  0x0020,   /* U+15B1 */
  0x0020,   /* U+15B2 */
  0x0020,   /* U+15B3 */
  0x0020,   /* U+15B4 */
  0x0020,   /* U+15B5 */
  0x0020,   /* U+15B6 */
  0x0020,   /* U+15B7 */
  0x0020,   /* U+15B8 */
  0x0020,   /* U+15B9 */
  0x0020,   /* U+15BA */
  0x0020,   /* U+15BB */
  0x0020,   /* U+15BC */
  0x0020,   /* U+15BD */
  0x0020,   /* U+15BE */
  0x0020,   /* U+15BF */
  0x0020,   /* U+15C0 */
  0x0020,   /* U+15C1 */
  0x0020,   /* U+15C2 */
  0x0020,   /* U+15C3 */
  0x0020,   /* U+15C4 */
  0x0020,   /* U+15C5 */
  0x0020,   /* U+15C6 */
  0x0020,   /* U+15C7 */
  0x0020,   /* U+15C8 */
  0x0020,   /* U+15C9 */
  0x0020,   /* U+15CA */
  0x0020,   /* U+15CB */
  0x0020,   /* U+15CC */
  0x0020,   /* U+15CD */
  0x0020,   /* U+15CE */
  0x0020,   /* U+15CF */
  0x0020,   /* U+15D0 */
  0x0020,   /* U+15D1 */
  0x0020,   /* U+15D2 */
  0x0020,   /* U+15D3 */
  0x0020,   /* U+15D4 */
  0x0020,   /* U+15D5 */
  0x0020,   /* U+15D6 */
  0x0020,   /* U+15D7 */
  0x0020,   /* U+15D8 */
  0x0020,   /* U+15D9 */
  0x0020,   /* U+15DA */
  0x0020,   /* U+15DB */
  0x0020,   /* U+15DC */
  0x0020,   /* U+15DD */
  0x0020,   /* U+15DE */
  0x0020,   /* U+15DF */
  0x0020,   /* U+15E0 */
  0x0020,   /* U+15E1 */
  0x0020,   /* U+15E2 */
  0x0020,   /* U+15E3 */
  0x0020,   /* U+15E4 */
  0x0020,   /* U+15E5 */
  0x0020,   /* U+15E6 */
  0x0020,   /* U+15E7 */
  0x0020,   /* U+15E8 */
  0x0020,   /* U+15E9 */
  0x0020,   /* U+15EA */
  0x0020,   /* U+15EB */
  0x0020,   /* U+15EC */
  0x0020,   /* U+15ED */
  0x0020,   /* U+15EE */
  0x0020,   /* U+15EF */
  0x0020,   /* U+15F0 */
  0x0020,   /* U+15F1 */
  0x0020,   /* U+15F2 */
  0x0020,   /* U+15F3 */
  0x0020,   /* U+15F4 */
  0x0020,   /* U+15F5 */
  0x0020,   /* U+15F6 */
  0x0020,   /* U+15F7 */
  0x0020,   /* U+15F8 */
  0x0020,   /* U+15F9 */
  0x0020,   /* U+15FA */
  0x0020,   /* U+15FB */
  0x0020,   /* U+15FC */
  0x0020,   /* U+15FD */
  0x0020,   /* U+15FE */
  0x0020,   /* U+15FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1500 */
  0x0002,   /* U+1501 */
  0x0002,   /* U+1502 */
  0x0002,   /* U+1503 */
  0x0002,   /* U+1504 */
  0x0002,   /* U+1505 */
  0x0002,   /* U+1506 */
  0x0002,   /* U+1507 */
  0x0002,   /* U+1508 */
  0x0002,   /* U+1509 */
  0x0002,   /* U+150A */
  0x0002,   /* U+150B */
  0x0002,   /* U+150C */
  0x0002,   /* U+150D */
  0x0002,   /* U+150E */
  0x0002,   /* U+150F */
  0x0002,   /* U+1510 */
  0x0002,   /* U+1511 */
  0x0002,   /* U+1512 */
  0x0002,   /* U+1513 */
  0x0002,   /* U+1514 */
  0x0002,   /* U+1515 */
  0x0002,   /* U+1516 */
  0x0002,   /* U+1517 */
  0x0002,   /* U+1518 */
  0x0002,   /* U+1519 */
  0x0002,   /* U+151A */
  0x0002,   /* U+151B */
  0x0002,   /* U+151C */
  0x0002,   /* U+151D */
  0x0002,   /* U+151E */
  0x0002,   /* U+151F */
  0x0002,   /* U+1520 */
  0x0002,   /* U+1521 */
  0x0002,   /* U+1522 */
  0x0002,   /* U+1523 */
  0x0002,   /* U+1524 */
  0x0002,   /* U+1525 */
  0x0002,   /* U+1526 */
  0x0002,   /* U+1527 */
  0x0002,   /* U+1528 */
  0x0002,   /* U+1529 */
  0x0002,   /* U+152A */
  0x0002,   /* U+152B */
  0x0002,   /* U+152C */
  0x0002,   /* U+152D */
  0x0002,   /* U+152E */
  0x0002,   /* U+152F */
  0x0002,   /* U+1530 */
  0x0002,   /* U+1531 */
  0x0002,   /* U+1532 */
  0x0002,   /* U+1533 */
  0x0002,   /* U+1534 */
  0x0002,   /* U+1535 */
  0x0002,   /* U+1536 */
  0x0002,   /* U+1537 */
  0x0002,   /* U+1538 */
  0x0002,   /* U+1539 */
  0x0002,   /* U+153A */
  0x0002,   /* U+153B */
  0x0002,   /* U+153C */
  0x0002,   /* U+153D */
  0x0002,   /* U+153E */
  0x0002,   /* U+153F */
  0x0002,   /* U+1540 */
  0x0002,   /* U+1541 */
  0x0002,   /* U+1542 */
  0x0002,   /* U+1543 */
  0x0002,   /* U+1544 */
  0x0002,   /* U+1545 */
  0x0002,   /* U+1546 */
  0x0002,   /* U+1547 */
  0x0002,   /* U+1548 */
  0x0002,   /* U+1549 */
  0x0002,   /* U+154A */
  0x0002,   /* U+154B */
  0x0002,   /* U+154C */
  0x0002,   /* U+154D */
  0x0002,   /* U+154E */
  0x0002,   /* U+154F */
  0x0002,   /* U+1550 */
  0x0002,   /* U+1551 */
  0x0002,   /* U+1552 */
  0x0002,   /* U+1553 */
  0x0002,   /* U+1554 */
  0x0002,   /* U+1555 */
  0x0002,   /* U+1556 */
  0x0002,   /* U+1557 */
  0x0002,   /* U+1558 */
  0x0002,   /* U+1559 */
  0x0002,   /* U+155A */
  0x0002,   /* U+155B */
  0x0002,   /* U+155C */
  0x0002,   /* U+155D */
  0x0002,   /* U+155E */
  0x0002,   /* U+155F */
  0x0002,   /* U+1560 */
  0x0002,   /* U+1561 */
  0x0002,   /* U+1562 */
  0x0002,   /* U+1563 */
  0x0002,   /* U+1564 */
  0x0002,   /* U+1565 */
  0x0002,   /* U+1566 */
  0x0002,   /* U+1567 */
  0x0002,   /* U+1568 */
  0x0002,   /* U+1569 */
  0x0002,   /* U+156A */
  0x0002,   /* U+156B */
  0x0002,   /* U+156C */
  0x0002,   /* U+156D */
  0x0002,   /* U+156E */
  0x0002,   /* U+156F */
  0x0002,   /* U+1570 */
  0x0002,   /* U+1571 */
  0x0002,   /* U+1572 */
  0x0002,   /* U+1573 */
  0x0002,   /* U+1574 */
  0x0002,   /* U+1575 */
  0x0002,   /* U+1576 */
  0x0002,   /* U+1577 */
  0x0002,   /* U+1578 */
  0x0002,   /* U+1579 */
  0x0002,   /* U+157A */
  0x0002,   /* U+157B */
  0x0002,   /* U+157C */
  0x0002,   /* U+157D */
  0x0002,   /* U+157E */
  0x0002,   /* U+157F */
  0x0002,   /* U+1580 */
  0x0002,   /* U+1581 */
  0x0002,   /* U+1582 */
  0x0002,   /* U+1583 */
  0x0002,   /* U+1584 */
  0x0002,   /* U+1585 */
  0x0002,   /* U+1586 */
  0x0002,   /* U+1587 */
  0x0002,   /* U+1588 */
  0x0002,   /* U+1589 */
  0x0002,   /* U+158A */
  0x0002,   /* U+158B */
  0x0002,   /* U+158C */
  0x0002,   /* U+158D */
  0x0002,   /* U+158E */
  0x0002,   /* U+158F */
  0x0002,   /* U+1590 */
  0x0002,   /* U+1591 */
  0x0002,   /* U+1592 */
  0x0002,   /* U+1593 */
  0x0002,   /* U+1594 */
  0x0002,   /* U+1595 */
  0x0002,   /* U+1596 */
  0x0002,   /* U+1597 */
  0x0002,   /* U+1598 */
  0x0002,   /* U+1599 */
  0x0002,   /* U+159A */
  0x0002,   /* U+159B */
  0x0002,   /* U+159C */
  0x0002,   /* U+159D */
  0x0002,   /* U+159E */
  0x0002,   /* U+159F */
  0x0002,   /* U+15A0 */
  0x0002,   /* U+15A1 */
  0x0002,   /* U+15A2 */
  0x0002,   /* U+15A3 */
  0x0002,   /* U+15A4 */
  0x0002,   /* U+15A5 */
  0x0002,   /* U+15A6 */
  0x0002,   /* U+15A7 */
  0x0002,   /* U+15A8 */
  0x0002,   /* U+15A9 */
  0x0002,   /* U+15AA */
  0x0002,   /* U+15AB */
  0x0002,   /* U+15AC */
  0x0002,   /* U+15AD */
  0x0002,   /* U+15AE */
  0x0002,   /* U+15AF */
  0x0002,   /* U+15B0 */
  0x0002,   /* U+15B1 */
  0x0002,   /* U+15B2 */
  0x0002,   /* U+15B3 */
  0x0002,   /* U+15B4 */
  0x0002,   /* U+15B5 */
  0x0002,   /* U+15B6 */
  0x0002,   /* U+15B7 */
  0x0002,   /* U+15B8 */
  0x0002,   /* U+15B9 */
  0x0002,   /* U+15BA */
  0x0002,   /* U+15BB */
  0x0002,   /* U+15BC */
  0x0002,   /* U+15BD */
  0x0002,   /* U+15BE */
  0x0002,   /* U+15BF */
  0x0002,   /* U+15C0 */
  0x0002,   /* U+15C1 */
  0x0002,   /* U+15C2 */
  0x0002,   /* U+15C3 */
  0x0002,   /* U+15C4 */
  0x0002,   /* U+15C5 */
  0x0002,   /* U+15C6 */
  0x0002,   /* U+15C7 */
  0x0002,   /* U+15C8 */
  0x0002,   /* U+15C9 */
  0x0002,   /* U+15CA */
  0x0002,   /* U+15CB */
  0x0002,   /* U+15CC */
  0x0002,   /* U+15CD */
  0x0002,   /* U+15CE */
  0x0002,   /* U+15CF */
  0x0002,   /* U+15D0 */
  0x0002,   /* U+15D1 */
  0x0002,   /* U+15D2 */
  0x0002,   /* U+15D3 */
  0x0002,   /* U+15D4 */
  0x0002,   /* U+15D5 */
  0x0002,   /* U+15D6 */
  0x0002,   /* U+15D7 */
  0x0002,   /* U+15D8 */
  0x0002,   /* U+15D9 */
  0x0002,   /* U+15DA */
  0x0002,   /* U+15DB */
  0x0002,   /* U+15DC */
  0x0002,   /* U+15DD */
  0x0002,   /* U+15DE */
  0x0002,   /* U+15DF */
  0x0002,   /* U+15E0 */
  0x0002,   /* U+15E1 */
  0x0002,   /* U+15E2 */
  0x0002,   /* U+15E3 */
  0x0002,   /* U+15E4 */
  0x0002,   /* U+15E5 */
  0x0002,   /* U+15E6 */
  0x0002,   /* U+15E7 */
  0x0002,   /* U+15E8 */
  0x0002,   /* U+15E9 */
  0x0002,   /* U+15EA */
  0x0002,   /* U+15EB */
  0x0002,   /* U+15EC */
  0x0002,   /* U+15ED */
  0x0002,   /* U+15EE */
  0x0002,   /* U+15EF */
  0x0002,   /* U+15F0 */
  0x0002,   /* U+15F1 */
  0x0002,   /* U+15F2 */
  0x0002,   /* U+15F3 */
  0x0002,   /* U+15F4 */
  0x0002,   /* U+15F5 */
  0x0002,   /* U+15F6 */
  0x0002,   /* U+15F7 */
  0x0002,   /* U+15F8 */
  0x0002,   /* U+15F9 */
  0x0002,   /* U+15FA */
  0x0002,   /* U+15FB */
  0x0002,   /* U+15FC */
  0x0002,   /* U+15FD */
  0x0002,   /* U+15FE */
  0x0002,   /* U+15FF */
};

uint16 uca900_p016[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 2, 1, 1, 2, 2, 1, 2, 1, 2, 1, 1, 2, 2, 2, 1, 
  1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 2, 2, 2, 1, 2, 
  2, 1, 2, 1, 2, 1, 2, 1, 1, 1, 1, 2, 2, 2, 2, 1, 
  2, 2, 1, 2, 2, 2, 1, 1, 2, 2, 1, 2, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 2, 4, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3600,   /* U+1600 */
  0x3601,   /* U+1601 */
  0x3602,   /* U+1602 */
  0x3603,   /* U+1603 */
  0x3604,   /* U+1604 */
  0x3605,   /* U+1605 */
  0x3606,   /* U+1606 */
  0x3607,   /* U+1607 */
  0x3608,   /* U+1608 */
  0x3609,   /* U+1609 */
  0x360A,   /* U+160A */
  0x360B,   /* U+160B */
  0x360C,   /* U+160C */
  0x360D,   /* U+160D */
  0x360E,   /* U+160E */
  0x360F,   /* U+160F */
  0x3610,   /* U+1610 */
  0x3611,   /* U+1611 */
  0x3612,   /* U+1612 */
  0x3613,   /* U+1613 */
  0x3614,   /* U+1614 */
  0x3615,   /* U+1615 */
  0x3616,   /* U+1616 */
  0x3617,   /* U+1617 */
  0x3618,   /* U+1618 */
  0x3619,   /* U+1619 */
  0x361A,   /* U+161A */
  0x361B,   /* U+161B */
  0x361C,   /* U+161C */
  0x361D,   /* U+161D */
  0x361E,   /* U+161E */
  0x361F,   /* U+161F */
  0x3620,   /* U+1620 */
  0x3621,   /* U+1621 */
  0x3622,   /* U+1622 */
  0x3623,   /* U+1623 */
  0x3624,   /* U+1624 */
  0x3625,   /* U+1625 */
  0x3626,   /* U+1626 */
  0x3627,   /* U+1627 */
  0x3628,   /* U+1628 */
  0x3629,   /* U+1629 */
  0x362A,   /* U+162A */
  0x362B,   /* U+162B */
  0x362C,   /* U+162C */
  0x362D,   /* U+162D */
  0x362E,   /* U+162E */
  0x362F,   /* U+162F */
  0x3630,   /* U+1630 */
  0x3631,   /* U+1631 */
  0x3632,   /* U+1632 */
  0x3633,   /* U+1633 */
  0x3634,   /* U+1634 */
  0x3635,   /* U+1635 */
  0x3636,   /* U+1636 */
  0x3637,   /* U+1637 */
  0x3638,   /* U+1638 */
  0x3639,   /* U+1639 */
  0x363A,   /* U+163A */
  0x363B,   /* U+163B */
  0x363C,   /* U+163C */
  0x363D,   /* U+163D */
  0x363E,   /* U+163E */
  0x363F,   /* U+163F */
  0x3640,   /* U+1640 */
  0x3641,   /* U+1641 */
  0x3642,   /* U+1642 */
  0x3643,   /* U+1643 */
  0x3644,   /* U+1644 */
  0x3645,   /* U+1645 */
  0x3646,   /* U+1646 */
  0x3647,   /* U+1647 */
  0x3648,   /* U+1648 */
  0x3649,   /* U+1649 */
  0x364A,   /* U+164A */
  0x364B,   /* U+164B */
  0x364C,   /* U+164C */
  0x364D,   /* U+164D */
  0x364E,   /* U+164E */
  0x364F,   /* U+164F */
  0x3650,   /* U+1650 */
  0x3651,   /* U+1651 */
  0x3652,   /* U+1652 */
  0x3653,   /* U+1653 */
  0x3654,   /* U+1654 */
  0x3655,   /* U+1655 */
  0x3656,   /* U+1656 */
  0x3657,   /* U+1657 */
  0x3658,   /* U+1658 */
  0x3659,   /* U+1659 */
  0x365A,   /* U+165A */
  0x365B,   /* U+165B */
  0x365C,   /* U+165C */
  0x365D,   /* U+165D */
  0x365E,   /* U+165E */
  0x365F,   /* U+165F */
  0x3660,   /* U+1660 */
  0x3661,   /* U+1661 */
  0x3662,   /* U+1662 */
  0x3663,   /* U+1663 */
  0x3664,   /* U+1664 */
  0x3665,   /* U+1665 */
  0x3666,   /* U+1666 */
  0x3667,   /* U+1667 */
  0x3668,   /* U+1668 */
  0x3669,   /* U+1669 */
  0x366A,   /* U+166A */
  0x366B,   /* U+166B */
  0x366C,   /* U+166C */
  0x041E,   /* U+166D */
  0x027F,   /* U+166E */
  0x3575,   /* U+166F */
  0x3586,   /* U+1670 */
  0x358F,   /* U+1671 */
  0x3590,   /* U+1672 */
  0x3591,   /* U+1673 */
  0x3592,   /* U+1674 */
  0x3593,   /* U+1675 */
  0x3594,   /* U+1676 */
  0x366D,   /* U+1677 */
  0x366E,   /* U+1678 */
  0x366F,   /* U+1679 */
  0x3670,   /* U+167A */
  0x3671,   /* U+167B */
  0x3672,   /* U+167C */
  0x3673,   /* U+167D */
  0x3674,   /* U+167E */
  0x3675,   /* U+167F */
  0x0209,   /* U+1680 */
  0x36BC,   /* U+1681 */
  0x36BD,   /* U+1682 */
  0x36BE,   /* U+1683 */
  0x36BF,   /* U+1684 */
  0x36C0,   /* U+1685 */
  0x36C1,   /* U+1686 */
  0x36C2,   /* U+1687 */
  0x36C3,   /* U+1688 */
  0x36C4,   /* U+1689 */
  0x36C5,   /* U+168A */
  0x36C6,   /* U+168B */
  0x36C7,   /* U+168C */
  0x36C8,   /* U+168D */
  0x36C9,   /* U+168E */
  0x36CA,   /* U+168F */
  0x36CB,   /* U+1690 */
  0x36CC,   /* U+1691 */
  0x36CD,   /* U+1692 */
  0x36CE,   /* U+1693 */
  0x36CF,   /* U+1694 */
  0x36D0,   /* U+1695 */
  0x36D1,   /* U+1696 */
  0x36D2,   /* U+1697 */
  0x36D3,   /* U+1698 */
  0x36D4,   /* U+1699 */
  0x36D5,   /* U+169A */
  0x0321,   /* U+169B */
  0x0322,   /* U+169C */
  0xFBC0,   /* U+169D */
  0xFBC0,   /* U+169E */
  0xFBC0,   /* U+169F */
  0x36D6,   /* U+16A0 */
  0x36D6,   /* U+16A1 */
  0x36D7,   /* U+16A2 */
  0x36FB,   /* U+16A3 */
  0x36D7,   /* U+16A4 */
  0x36D7,   /* U+16A5 */
  0x36D8,   /* U+16A6 */
  0x36D8,   /* U+16A7 */
  0x36D9,   /* U+16A8 */
  0x36D9,   /* U+16A9 */
  0x36F7,   /* U+16AA */
  0x36F9,   /* U+16AB */
  0x36D9,   /* U+16AC */
  0x36D9,   /* U+16AD */
  0x36D9,   /* U+16AE */
  0x36DB,   /* U+16AF */
  0x36DC,   /* U+16B0 */
  0x36DD,   /* U+16B1 */
  0x36DE,   /* U+16B2 */
  0x36DE,   /* U+16B3 */
  0x36DE,   /* U+16B4 */
  0x36DE,   /* U+16B5 */
  0x36DE,   /* U+16B6 */
  0x36E0,   /* U+16B7 */
  0x36FE,   /* U+16B8 */
  0x36E1,   /* U+16B9 */
  0x36E2,   /* U+16BA */
  0x36E2,   /* U+16BB */
  0x36E2,   /* U+16BC */
  0x36E2,   /* U+16BD */
  0x36E3,   /* U+16BE */
  0x36E3,   /* U+16BF */
  0x36E3,   /* U+16C0 */
  0x36E4,   /* U+16C1 */
  0x36E4,   /* U+16C2 */
  0x36E6,   /* U+16C3 */
  0x36E6,   /* U+16C4 */
  0x36E7,   /* U+16C5 */
  0x36E7,   /* U+16C6 */
  0x36E8,   /* U+16C7 */
  0x36E9,   /* U+16C8 */
  0x36EA,   /* U+16C9 */
  0x36EB,   /* U+16CA */
  0x36EB,   /* U+16CB */
  0x36EB,   /* U+16CC */
  0x36EB,   /* U+16CD */
  0x36EB,   /* U+16CE */
  0x36ED,   /* U+16CF */
  0x36ED,   /* U+16D0 */
  0x36ED,   /* U+16D1 */
  0x36EE,   /* U+16D2 */
  0x36EE,   /* U+16D3 */
  0x36EE,   /* U+16D4 */
  0x36E9,   /* U+16D5 */
  0x36EF,   /* U+16D6 */
  0x36F1,   /* U+16D7 */
  0x36F1,   /* U+16D8 */
  0x36F1,   /* U+16D9 */
  0x36F2,   /* U+16DA */
  0x36F2,   /* U+16DB */
  0x36F3,   /* U+16DC */
  0x36F3,   /* U+16DD */
  0x36F4,   /* U+16DE */
  0x36F5,   /* U+16DF */
  0x36FC,   /* U+16E0 */
  0x3700,   /* U+16E1 */
  0x3701,   /* U+16E2 */
  0x36FD,   /* U+16E3 */
  0x36FF,   /* U+16E4 */
  0x3702,   /* U+16E5 */
  0x3703,   /* U+16E6 */
  0x3703,   /* U+16E7 */
  0x3703,   /* U+16E8 */
  0x36E1,   /* U+16E9 */
  0x36EB,   /* U+16EA */
  0x025C,   /* U+16EB */
  0x025D,   /* U+16EC */
  0x025E,   /* U+16ED */
  0x36E7,   /* U+16EE */
  0x36F1,   /* U+16EF */
  0x36D8,   /* U+16F0 */
  0x36DF,   /* U+16F1 */
  0x36EC,   /* U+16F2 */
  0x36F6,   /* U+16F3 */
  0x36DA,   /* U+16F4 */
  0x36E5,   /* U+16F5 */
  0x36F0,   /* U+16F6 */
  0x36F8,   /* U+16F7 */
  0x36FA,   /* U+16F8 */
  0xFBC0,   /* U+16F9 */
  0xFBC0,   /* U+16FA */
  0xFBC0,   /* U+16FB */
  0xFBC0,   /* U+16FC */
  0xFBC0,   /* U+16FD */
  0xFBC0,   /* U+16FE */
  0xFBC0,   /* U+16FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1600 */
  0x0020,   /* U+1601 */
  0x0020,   /* U+1602 */
  0x0020,   /* U+1603 */
  0x0020,   /* U+1604 */
  0x0020,   /* U+1605 */
  0x0020,   /* U+1606 */
  0x0020,   /* U+1607 */
  0x0020,   /* U+1608 */
  0x0020,   /* U+1609 */
  0x0020,   /* U+160A */
  0x0020,   /* U+160B */
  0x0020,   /* U+160C */
  0x0020,   /* U+160D */
  0x0020,   /* U+160E */
  0x0020,   /* U+160F */
  0x0020,   /* U+1610 */
  0x0020,   /* U+1611 */
  0x0020,   /* U+1612 */
  0x0020,   /* U+1613 */
  0x0020,   /* U+1614 */
  0x0020,   /* U+1615 */
  0x0020,   /* U+1616 */
  0x0020,   /* U+1617 */
  0x0020,   /* U+1618 */
  0x0020,   /* U+1619 */
  0x0020,   /* U+161A */
  0x0020,   /* U+161B */
  0x0020,   /* U+161C */
  0x0020,   /* U+161D */
  0x0020,   /* U+161E */
  0x0020,   /* U+161F */
  0x0020,   /* U+1620 */
  0x0020,   /* U+1621 */
  0x0020,   /* U+1622 */
  0x0020,   /* U+1623 */
  0x0020,   /* U+1624 */
  0x0020,   /* U+1625 */
  0x0020,   /* U+1626 */
  0x0020,   /* U+1627 */
  0x0020,   /* U+1628 */
  0x0020,   /* U+1629 */
  0x0020,   /* U+162A */
  0x0020,   /* U+162B */
  0x0020,   /* U+162C */
  0x0020,   /* U+162D */
  0x0020,   /* U+162E */
  0x0020,   /* U+162F */
  0x0020,   /* U+1630 */
  0x0020,   /* U+1631 */
  0x0020,   /* U+1632 */
  0x0020,   /* U+1633 */
  0x0020,   /* U+1634 */
  0x0020,   /* U+1635 */
  0x0020,   /* U+1636 */
  0x0020,   /* U+1637 */
  0x0020,   /* U+1638 */
  0x0020,   /* U+1639 */
  0x0020,   /* U+163A */
  0x0020,   /* U+163B */
  0x0020,   /* U+163C */
  0x0020,   /* U+163D */
  0x0020,   /* U+163E */
  0x0020,   /* U+163F */
  0x0020,   /* U+1640 */
  0x0020,   /* U+1641 */
  0x0020,   /* U+1642 */
  0x0020,   /* U+1643 */
  0x0020,   /* U+1644 */
  0x0020,   /* U+1645 */
  0x0020,   /* U+1646 */
  0x0020,   /* U+1647 */
  0x0020,   /* U+1648 */
  0x0020,   /* U+1649 */
  0x0020,   /* U+164A */
  0x0020,   /* U+164B */
  0x0020,   /* U+164C */
  0x0020,   /* U+164D */
  0x0020,   /* U+164E */
  0x0020,   /* U+164F */
  0x0020,   /* U+1650 */
  0x0020,   /* U+1651 */
  0x0020,   /* U+1652 */
  0x0020,   /* U+1653 */
  0x0020,   /* U+1654 */
  0x0020,   /* U+1655 */
  0x0020,   /* U+1656 */
  0x0020,   /* U+1657 */
  0x0020,   /* U+1658 */
  0x0020,   /* U+1659 */
  0x0020,   /* U+165A */
  0x0020,   /* U+165B */
  0x0020,   /* U+165C */
  0x0020,   /* U+165D */
  0x0020,   /* U+165E */
  0x0020,   /* U+165F */
  0x0020,   /* U+1660 */
  0x0020,   /* U+1661 */
  0x0020,   /* U+1662 */
  0x0020,   /* U+1663 */
  0x0020,   /* U+1664 */
  0x0020,   /* U+1665 */
  0x0020,   /* U+1666 */
  0x0020,   /* U+1667 */
  0x0020,   /* U+1668 */
  0x0020,   /* U+1669 */
  0x0020,   /* U+166A */
  0x0020,   /* U+166B */
  0x0020,   /* U+166C */
  0x0020,   /* U+166D */
  0x0020,   /* U+166E */
  0x0020,   /* U+166F */
  0x0020,   /* U+1670 */
  0x0020,   /* U+1671 */
  0x0020,   /* U+1672 */
  0x0020,   /* U+1673 */
  0x0020,   /* U+1674 */
  0x0020,   /* U+1675 */
  0x0020,   /* U+1676 */
  0x0020,   /* U+1677 */
  0x0020,   /* U+1678 */
  0x0020,   /* U+1679 */
  0x0020,   /* U+167A */
  0x0020,   /* U+167B */
  0x0020,   /* U+167C */
  0x0020,   /* U+167D */
  0x0020,   /* U+167E */
  0x0020,   /* U+167F */
  0x0020,   /* U+1680 */
  0x0020,   /* U+1681 */
  0x0020,   /* U+1682 */
  0x0020,   /* U+1683 */
  0x0020,   /* U+1684 */
  0x0020,   /* U+1685 */
  0x0020,   /* U+1686 */
  0x0020,   /* U+1687 */
  0x0020,   /* U+1688 */
  0x0020,   /* U+1689 */
  0x0020,   /* U+168A */
  0x0020,   /* U+168B */
  0x0020,   /* U+168C */
  0x0020,   /* U+168D */
  0x0020,   /* U+168E */
  0x0020,   /* U+168F */
  0x0020,   /* U+1690 */
  0x0020,   /* U+1691 */
  0x0020,   /* U+1692 */
  0x0020,   /* U+1693 */
  0x0020,   /* U+1694 */
  0x0020,   /* U+1695 */
  0x0020,   /* U+1696 */
  0x0020,   /* U+1697 */
  0x0020,   /* U+1698 */
  0x0020,   /* U+1699 */
  0x0020,   /* U+169A */
  0x0020,   /* U+169B */
  0x0020,   /* U+169C */
  0x0020,   /* U+169D */
  0x0020,   /* U+169E */
  0x0020,   /* U+169F */
  0x0020,   /* U+16A0 */
  0x0020,   /* U+16A1 */
  0x0020,   /* U+16A2 */
  0x0020,   /* U+16A3 */
  0x0020,   /* U+16A4 */
  0x0020,   /* U+16A5 */
  0x0020,   /* U+16A6 */
  0x0020,   /* U+16A7 */
  0x0020,   /* U+16A8 */
  0x0020,   /* U+16A9 */
  0x0020,   /* U+16AA */
  0x0020,   /* U+16AB */
  0x0020,   /* U+16AC */
  0x0020,   /* U+16AD */
  0x0020,   /* U+16AE */
  0x0020,   /* U+16AF */
  0x0020,   /* U+16B0 */
  0x0020,   /* U+16B1 */
  0x0020,   /* U+16B2 */
  0x0020,   /* U+16B3 */
  0x0020,   /* U+16B4 */
  0x0020,   /* U+16B5 */
  0x0020,   /* U+16B6 */
  0x0020,   /* U+16B7 */
  0x0020,   /* U+16B8 */
  0x0020,   /* U+16B9 */
  0x0020,   /* U+16BA */
  0x0020,   /* U+16BB */
  0x0020,   /* U+16BC */
  0x0020,   /* U+16BD */
  0x0020,   /* U+16BE */
  0x0020,   /* U+16BF */
  0x0020,   /* U+16C0 */
  0x0020,   /* U+16C1 */
  0x0020,   /* U+16C2 */
  0x0020,   /* U+16C3 */
  0x0020,   /* U+16C4 */
  0x0020,   /* U+16C5 */
  0x0020,   /* U+16C6 */
  0x0020,   /* U+16C7 */
  0x0020,   /* U+16C8 */
  0x0020,   /* U+16C9 */
  0x0020,   /* U+16CA */
  0x0020,   /* U+16CB */
  0x0020,   /* U+16CC */
  0x0020,   /* U+16CD */
  0x0020,   /* U+16CE */
  0x0020,   /* U+16CF */
  0x0020,   /* U+16D0 */
  0x0020,   /* U+16D1 */
  0x0020,   /* U+16D2 */
  0x0020,   /* U+16D3 */
  0x0020,   /* U+16D4 */
  0x0020,   /* U+16D5 */
  0x0020,   /* U+16D6 */
  0x0020,   /* U+16D7 */
  0x0020,   /* U+16D8 */
  0x0020,   /* U+16D9 */
  0x0020,   /* U+16DA */
  0x0020,   /* U+16DB */
  0x0020,   /* U+16DC */
  0x0020,   /* U+16DD */
  0x0020,   /* U+16DE */
  0x0020,   /* U+16DF */
  0x0020,   /* U+16E0 */
  0x0020,   /* U+16E1 */
  0x0020,   /* U+16E2 */
  0x0020,   /* U+16E3 */
  0x0020,   /* U+16E4 */
  0x0020,   /* U+16E5 */
  0x0020,   /* U+16E6 */
  0x0020,   /* U+16E7 */
  0x0020,   /* U+16E8 */
  0x0020,   /* U+16E9 */
  0x0020,   /* U+16EA */
  0x0020,   /* U+16EB */
  0x0020,   /* U+16EC */
  0x0020,   /* U+16ED */
  0x0020,   /* U+16EE */
  0x0020,   /* U+16EF */
  0x0020,   /* U+16F0 */
  0x0020,   /* U+16F1 */
  0x0020,   /* U+16F2 */
  0x0020,   /* U+16F3 */
  0x0020,   /* U+16F4 */
  0x0020,   /* U+16F5 */
  0x0020,   /* U+16F6 */
  0x0020,   /* U+16F7 */
  0x0020,   /* U+16F8 */
  0x0020,   /* U+16F9 */
  0x0020,   /* U+16FA */
  0x0020,   /* U+16FB */
  0x0020,   /* U+16FC */
  0x0020,   /* U+16FD */
  0x0020,   /* U+16FE */
  0x0020,   /* U+16FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1600 */
  0x0002,   /* U+1601 */
  0x0002,   /* U+1602 */
  0x0002,   /* U+1603 */
  0x0002,   /* U+1604 */
  0x0002,   /* U+1605 */
  0x0002,   /* U+1606 */
  0x0002,   /* U+1607 */
  0x0002,   /* U+1608 */
  0x0002,   /* U+1609 */
  0x0002,   /* U+160A */
  0x0002,   /* U+160B */
  0x0002,   /* U+160C */
  0x0002,   /* U+160D */
  0x0002,   /* U+160E */
  0x0002,   /* U+160F */
  0x0002,   /* U+1610 */
  0x0002,   /* U+1611 */
  0x0002,   /* U+1612 */
  0x0002,   /* U+1613 */
  0x0002,   /* U+1614 */
  0x0002,   /* U+1615 */
  0x0002,   /* U+1616 */
  0x0002,   /* U+1617 */
  0x0002,   /* U+1618 */
  0x0002,   /* U+1619 */
  0x0002,   /* U+161A */
  0x0002,   /* U+161B */
  0x0002,   /* U+161C */
  0x0002,   /* U+161D */
  0x0002,   /* U+161E */
  0x0002,   /* U+161F */
  0x0002,   /* U+1620 */
  0x0002,   /* U+1621 */
  0x0002,   /* U+1622 */
  0x0002,   /* U+1623 */
  0x0002,   /* U+1624 */
  0x0002,   /* U+1625 */
  0x0002,   /* U+1626 */
  0x0002,   /* U+1627 */
  0x0002,   /* U+1628 */
  0x0002,   /* U+1629 */
  0x0002,   /* U+162A */
  0x0002,   /* U+162B */
  0x0002,   /* U+162C */
  0x0002,   /* U+162D */
  0x0002,   /* U+162E */
  0x0002,   /* U+162F */
  0x0002,   /* U+1630 */
  0x0002,   /* U+1631 */
  0x0002,   /* U+1632 */
  0x0002,   /* U+1633 */
  0x0002,   /* U+1634 */
  0x0002,   /* U+1635 */
  0x0002,   /* U+1636 */
  0x0002,   /* U+1637 */
  0x0002,   /* U+1638 */
  0x0002,   /* U+1639 */
  0x0002,   /* U+163A */
  0x0002,   /* U+163B */
  0x0002,   /* U+163C */
  0x0002,   /* U+163D */
  0x0002,   /* U+163E */
  0x0002,   /* U+163F */
  0x0002,   /* U+1640 */
  0x0002,   /* U+1641 */
  0x0002,   /* U+1642 */
  0x0002,   /* U+1643 */
  0x0002,   /* U+1644 */
  0x0002,   /* U+1645 */
  0x0002,   /* U+1646 */
  0x0002,   /* U+1647 */
  0x0002,   /* U+1648 */
  0x0002,   /* U+1649 */
  0x0002,   /* U+164A */
  0x0002,   /* U+164B */
  0x0002,   /* U+164C */
  0x0002,   /* U+164D */
  0x0002,   /* U+164E */
  0x0002,   /* U+164F */
  0x0002,   /* U+1650 */
  0x0002,   /* U+1651 */
  0x0002,   /* U+1652 */
  0x0002,   /* U+1653 */
  0x0002,   /* U+1654 */
  0x0002,   /* U+1655 */
  0x0002,   /* U+1656 */
  0x0002,   /* U+1657 */
  0x0002,   /* U+1658 */
  0x0002,   /* U+1659 */
  0x0002,   /* U+165A */
  0x0002,   /* U+165B */
  0x0002,   /* U+165C */
  0x0002,   /* U+165D */
  0x0002,   /* U+165E */
  0x0002,   /* U+165F */
  0x0002,   /* U+1660 */
  0x0002,   /* U+1661 */
  0x0002,   /* U+1662 */
  0x0002,   /* U+1663 */
  0x0002,   /* U+1664 */
  0x0002,   /* U+1665 */
  0x0002,   /* U+1666 */
  0x0002,   /* U+1667 */
  0x0002,   /* U+1668 */
  0x0002,   /* U+1669 */
  0x0002,   /* U+166A */
  0x0002,   /* U+166B */
  0x0002,   /* U+166C */
  0x0002,   /* U+166D */
  0x0002,   /* U+166E */
  0x0002,   /* U+166F */
  0x0002,   /* U+1670 */
  0x0002,   /* U+1671 */
  0x0002,   /* U+1672 */
  0x0002,   /* U+1673 */
  0x0002,   /* U+1674 */
  0x0002,   /* U+1675 */
  0x0002,   /* U+1676 */
  0x0002,   /* U+1677 */
  0x0002,   /* U+1678 */
  0x0002,   /* U+1679 */
  0x0002,   /* U+167A */
  0x0002,   /* U+167B */
  0x0002,   /* U+167C */
  0x0002,   /* U+167D */
  0x0002,   /* U+167E */
  0x0002,   /* U+167F */
  0x0004,   /* U+1680 */
  0x0002,   /* U+1681 */
  0x0002,   /* U+1682 */
  0x0002,   /* U+1683 */
  0x0002,   /* U+1684 */
  0x0002,   /* U+1685 */
  0x0002,   /* U+1686 */
  0x0002,   /* U+1687 */
  0x0002,   /* U+1688 */
  0x0002,   /* U+1689 */
  0x0002,   /* U+168A */
  0x0002,   /* U+168B */
  0x0002,   /* U+168C */
  0x0002,   /* U+168D */
  0x0002,   /* U+168E */
  0x0002,   /* U+168F */
  0x0002,   /* U+1690 */
  0x0002,   /* U+1691 */
  0x0002,   /* U+1692 */
  0x0002,   /* U+1693 */
  0x0002,   /* U+1694 */
  0x0002,   /* U+1695 */
  0x0002,   /* U+1696 */
  0x0002,   /* U+1697 */
  0x0002,   /* U+1698 */
  0x0002,   /* U+1699 */
  0x0002,   /* U+169A */
  0x0002,   /* U+169B */
  0x0002,   /* U+169C */
  0x0002,   /* U+169D */
  0x0002,   /* U+169E */
  0x0002,   /* U+169F */
  0x0002,   /* U+16A0 */
  0x0004,   /* U+16A1 */
  0x0002,   /* U+16A2 */
  0x0002,   /* U+16A3 */
  0x0004,   /* U+16A4 */
  0x0004,   /* U+16A5 */
  0x0002,   /* U+16A6 */
  0x0004,   /* U+16A7 */
  0x0002,   /* U+16A8 */
  0x0004,   /* U+16A9 */
  0x0002,   /* U+16AA */
  0x0002,   /* U+16AB */
  0x0004,   /* U+16AC */
  0x0004,   /* U+16AD */
  0x0004,   /* U+16AE */
  0x0002,   /* U+16AF */
  0x0002,   /* U+16B0 */
  0x0002,   /* U+16B1 */
  0x0002,   /* U+16B2 */
  0x0004,   /* U+16B3 */
  0x0004,   /* U+16B4 */
  0x0004,   /* U+16B5 */
  0x0004,   /* U+16B6 */
  0x0002,   /* U+16B7 */
  0x0002,   /* U+16B8 */
  0x0002,   /* U+16B9 */
  0x0002,   /* U+16BA */
  0x0004,   /* U+16BB */
  0x0004,   /* U+16BC */
  0x0004,   /* U+16BD */
  0x0002,   /* U+16BE */
  0x0004,   /* U+16BF */
  0x0004,   /* U+16C0 */
  0x0002,   /* U+16C1 */
  0x0004,   /* U+16C2 */
  0x0002,   /* U+16C3 */
  0x0004,   /* U+16C4 */
  0x0002,   /* U+16C5 */
  0x0004,   /* U+16C6 */
  0x0002,   /* U+16C7 */
  0x0002,   /* U+16C8 */
  0x0002,   /* U+16C9 */
  0x0002,   /* U+16CA */
  0x0004,   /* U+16CB */
  0x0004,   /* U+16CC */
  0x0004,   /* U+16CD */
  0x0004,   /* U+16CE */
  0x0002,   /* U+16CF */
  0x0004,   /* U+16D0 */
  0x0004,   /* U+16D1 */
  0x0002,   /* U+16D2 */
  0x0004,   /* U+16D3 */
  0x0004,   /* U+16D4 */
  0x0004,   /* U+16D5 */
  0x0002,   /* U+16D6 */
  0x0002,   /* U+16D7 */
  0x0004,   /* U+16D8 */
  0x0004,   /* U+16D9 */
  0x0002,   /* U+16DA */
  0x0004,   /* U+16DB */
  0x0002,   /* U+16DC */
  0x0004,   /* U+16DD */
  0x0002,   /* U+16DE */
  0x0002,   /* U+16DF */
  0x0002,   /* U+16E0 */
  0x0002,   /* U+16E1 */
  0x0002,   /* U+16E2 */
  0x0002,   /* U+16E3 */
  0x0002,   /* U+16E4 */
  0x0002,   /* U+16E5 */
  0x0002,   /* U+16E6 */
  0x0004,   /* U+16E7 */
  0x0004,   /* U+16E8 */
  0x0004,   /* U+16E9 */
  0x0004,   /* U+16EA */
  0x0002,   /* U+16EB */
  0x0002,   /* U+16EC */
  0x0002,   /* U+16ED */
  0x0004,   /* U+16EE */
  0x0004,   /* U+16EF */
  0x0004,   /* U+16F0 */
  0x0002,   /* U+16F1 */
  0x0002,   /* U+16F2 */
  0x0002,   /* U+16F3 */
  0x0002,   /* U+16F4 */
  0x0002,   /* U+16F5 */
  0x0002,   /* U+16F6 */
  0x0002,   /* U+16F7 */
  0x0002,   /* U+16F8 */
  0x0002,   /* U+16F9 */
  0x0002,   /* U+16FA */
  0x0002,   /* U+16FB */
  0x0002,   /* U+16FC */
  0x0002,   /* U+16FD */
  0x0002,   /* U+16FE */
  0x0002,   /* U+16FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x969D,   /* U+169D */
  0x969E,   /* U+169E */
  0x969F,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x36F2,   /* U+16EE */
  0x0000,   /* U+16EF */
  0x36D8,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x96F9,   /* U+16F9 */
  0x96FA,   /* U+16FA */
  0x96FB,   /* U+16FB */
  0x96FC,   /* U+16FC */
  0x96FD,   /* U+16FD */
  0x96FE,   /* U+16FE */
  0x96FF,   /* U+16FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0110,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0110,   /* U+16A4 */
  0x0111,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0110,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0110,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0111,   /* U+16AC */
  0x0112,   /* U+16AD */
  0x0113,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0110,   /* U+16B3 */
  0x0111,   /* U+16B4 */
  0x0112,   /* U+16B5 */
  0x0113,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0110,   /* U+16BB */
  0x0111,   /* U+16BC */
  0x0112,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0110,   /* U+16BF */
  0x0111,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0110,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0110,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0110,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0110,   /* U+16CB */
  0x0112,   /* U+16CC */
  0x0113,   /* U+16CD */
  0x0114,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0110,   /* U+16D0 */
  0x0111,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0110,   /* U+16D3 */
  0x0111,   /* U+16D4 */
  0x0110,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0110,   /* U+16D8 */
  0x0111,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0110,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0110,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0110,   /* U+16E7 */
  0x0111,   /* U+16E8 */
  0x0110,   /* U+16E9 */
  0x0111,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0020,   /* U+16EE */
  0x0110,   /* U+16EF */
  0x0020,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0004,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0004,   /* U+16A4 */
  0x0004,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0004,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0004,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0004,   /* U+16AC */
  0x0004,   /* U+16AD */
  0x0004,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0004,   /* U+16B3 */
  0x0004,   /* U+16B4 */
  0x0004,   /* U+16B5 */
  0x0004,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0004,   /* U+16BB */
  0x0004,   /* U+16BC */
  0x0004,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0004,   /* U+16BF */
  0x0004,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0004,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0004,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0004,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0004,   /* U+16CB */
  0x0004,   /* U+16CC */
  0x0004,   /* U+16CD */
  0x0004,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0004,   /* U+16D0 */
  0x0004,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0004,   /* U+16D3 */
  0x0004,   /* U+16D4 */
  0x0004,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0004,   /* U+16D8 */
  0x0004,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0004,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0004,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0004,   /* U+16E7 */
  0x0004,   /* U+16E8 */
  0x0004,   /* U+16E9 */
  0x0004,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0004,   /* U+16EE */
  0x0004,   /* U+16EF */
  0x0004,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x36F1,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x0020,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x0004,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x0000,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x0110,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+1600 */
  0x0000,   /* U+1601 */
  0x0000,   /* U+1602 */
  0x0000,   /* U+1603 */
  0x0000,   /* U+1604 */
  0x0000,   /* U+1605 */
  0x0000,   /* U+1606 */
  0x0000,   /* U+1607 */
  0x0000,   /* U+1608 */
  0x0000,   /* U+1609 */
  0x0000,   /* U+160A */
  0x0000,   /* U+160B */
  0x0000,   /* U+160C */
  0x0000,   /* U+160D */
  0x0000,   /* U+160E */
  0x0000,   /* U+160F */
  0x0000,   /* U+1610 */
  0x0000,   /* U+1611 */
  0x0000,   /* U+1612 */
  0x0000,   /* U+1613 */
  0x0000,   /* U+1614 */
  0x0000,   /* U+1615 */
  0x0000,   /* U+1616 */
  0x0000,   /* U+1617 */
  0x0000,   /* U+1618 */
  0x0000,   /* U+1619 */
  0x0000,   /* U+161A */
  0x0000,   /* U+161B */
  0x0000,   /* U+161C */
  0x0000,   /* U+161D */
  0x0000,   /* U+161E */
  0x0000,   /* U+161F */
  0x0000,   /* U+1620 */
  0x0000,   /* U+1621 */
  0x0000,   /* U+1622 */
  0x0000,   /* U+1623 */
  0x0000,   /* U+1624 */
  0x0000,   /* U+1625 */
  0x0000,   /* U+1626 */
  0x0000,   /* U+1627 */
  0x0000,   /* U+1628 */
  0x0000,   /* U+1629 */
  0x0000,   /* U+162A */
  0x0000,   /* U+162B */
  0x0000,   /* U+162C */
  0x0000,   /* U+162D */
  0x0000,   /* U+162E */
  0x0000,   /* U+162F */
  0x0000,   /* U+1630 */
  0x0000,   /* U+1631 */
  0x0000,   /* U+1632 */
  0x0000,   /* U+1633 */
  0x0000,   /* U+1634 */
  0x0000,   /* U+1635 */
  0x0000,   /* U+1636 */
  0x0000,   /* U+1637 */
  0x0000,   /* U+1638 */
  0x0000,   /* U+1639 */
  0x0000,   /* U+163A */
  0x0000,   /* U+163B */
  0x0000,   /* U+163C */
  0x0000,   /* U+163D */
  0x0000,   /* U+163E */
  0x0000,   /* U+163F */
  0x0000,   /* U+1640 */
  0x0000,   /* U+1641 */
  0x0000,   /* U+1642 */
  0x0000,   /* U+1643 */
  0x0000,   /* U+1644 */
  0x0000,   /* U+1645 */
  0x0000,   /* U+1646 */
  0x0000,   /* U+1647 */
  0x0000,   /* U+1648 */
  0x0000,   /* U+1649 */
  0x0000,   /* U+164A */
  0x0000,   /* U+164B */
  0x0000,   /* U+164C */
  0x0000,   /* U+164D */
  0x0000,   /* U+164E */
  0x0000,   /* U+164F */
  0x0000,   /* U+1650 */
  0x0000,   /* U+1651 */
  0x0000,   /* U+1652 */
  0x0000,   /* U+1653 */
  0x0000,   /* U+1654 */
  0x0000,   /* U+1655 */
  0x0000,   /* U+1656 */
  0x0000,   /* U+1657 */
  0x0000,   /* U+1658 */
  0x0000,   /* U+1659 */
  0x0000,   /* U+165A */
  0x0000,   /* U+165B */
  0x0000,   /* U+165C */
  0x0000,   /* U+165D */
  0x0000,   /* U+165E */
  0x0000,   /* U+165F */
  0x0000,   /* U+1660 */
  0x0000,   /* U+1661 */
  0x0000,   /* U+1662 */
  0x0000,   /* U+1663 */
  0x0000,   /* U+1664 */
  0x0000,   /* U+1665 */
  0x0000,   /* U+1666 */
  0x0000,   /* U+1667 */
  0x0000,   /* U+1668 */
  0x0000,   /* U+1669 */
  0x0000,   /* U+166A */
  0x0000,   /* U+166B */
  0x0000,   /* U+166C */
  0x0000,   /* U+166D */
  0x0000,   /* U+166E */
  0x0000,   /* U+166F */
  0x0000,   /* U+1670 */
  0x0000,   /* U+1671 */
  0x0000,   /* U+1672 */
  0x0000,   /* U+1673 */
  0x0000,   /* U+1674 */
  0x0000,   /* U+1675 */
  0x0000,   /* U+1676 */
  0x0000,   /* U+1677 */
  0x0000,   /* U+1678 */
  0x0000,   /* U+1679 */
  0x0000,   /* U+167A */
  0x0000,   /* U+167B */
  0x0000,   /* U+167C */
  0x0000,   /* U+167D */
  0x0000,   /* U+167E */
  0x0000,   /* U+167F */
  0x0000,   /* U+1680 */
  0x0000,   /* U+1681 */
  0x0000,   /* U+1682 */
  0x0000,   /* U+1683 */
  0x0000,   /* U+1684 */
  0x0000,   /* U+1685 */
  0x0000,   /* U+1686 */
  0x0000,   /* U+1687 */
  0x0000,   /* U+1688 */
  0x0000,   /* U+1689 */
  0x0000,   /* U+168A */
  0x0000,   /* U+168B */
  0x0000,   /* U+168C */
  0x0000,   /* U+168D */
  0x0000,   /* U+168E */
  0x0000,   /* U+168F */
  0x0000,   /* U+1690 */
  0x0000,   /* U+1691 */
  0x0000,   /* U+1692 */
  0x0000,   /* U+1693 */
  0x0000,   /* U+1694 */
  0x0000,   /* U+1695 */
  0x0000,   /* U+1696 */
  0x0000,   /* U+1697 */
  0x0000,   /* U+1698 */
  0x0000,   /* U+1699 */
  0x0000,   /* U+169A */
  0x0000,   /* U+169B */
  0x0000,   /* U+169C */
  0x0000,   /* U+169D */
  0x0000,   /* U+169E */
  0x0000,   /* U+169F */
  0x0000,   /* U+16A0 */
  0x0000,   /* U+16A1 */
  0x0000,   /* U+16A2 */
  0x0000,   /* U+16A3 */
  0x0000,   /* U+16A4 */
  0x0000,   /* U+16A5 */
  0x0000,   /* U+16A6 */
  0x0000,   /* U+16A7 */
  0x0000,   /* U+16A8 */
  0x0000,   /* U+16A9 */
  0x0000,   /* U+16AA */
  0x0000,   /* U+16AB */
  0x0000,   /* U+16AC */
  0x0000,   /* U+16AD */
  0x0000,   /* U+16AE */
  0x0000,   /* U+16AF */
  0x0000,   /* U+16B0 */
  0x0000,   /* U+16B1 */
  0x0000,   /* U+16B2 */
  0x0000,   /* U+16B3 */
  0x0000,   /* U+16B4 */
  0x0000,   /* U+16B5 */
  0x0000,   /* U+16B6 */
  0x0000,   /* U+16B7 */
  0x0000,   /* U+16B8 */
  0x0000,   /* U+16B9 */
  0x0000,   /* U+16BA */
  0x0000,   /* U+16BB */
  0x0000,   /* U+16BC */
  0x0000,   /* U+16BD */
  0x0000,   /* U+16BE */
  0x0000,   /* U+16BF */
  0x0000,   /* U+16C0 */
  0x0000,   /* U+16C1 */
  0x0000,   /* U+16C2 */
  0x0000,   /* U+16C3 */
  0x0000,   /* U+16C4 */
  0x0000,   /* U+16C5 */
  0x0000,   /* U+16C6 */
  0x0000,   /* U+16C7 */
  0x0000,   /* U+16C8 */
  0x0000,   /* U+16C9 */
  0x0000,   /* U+16CA */
  0x0000,   /* U+16CB */
  0x0000,   /* U+16CC */
  0x0000,   /* U+16CD */
  0x0000,   /* U+16CE */
  0x0000,   /* U+16CF */
  0x0000,   /* U+16D0 */
  0x0000,   /* U+16D1 */
  0x0000,   /* U+16D2 */
  0x0000,   /* U+16D3 */
  0x0000,   /* U+16D4 */
  0x0000,   /* U+16D5 */
  0x0000,   /* U+16D6 */
  0x0000,   /* U+16D7 */
  0x0000,   /* U+16D8 */
  0x0000,   /* U+16D9 */
  0x0000,   /* U+16DA */
  0x0000,   /* U+16DB */
  0x0000,   /* U+16DC */
  0x0000,   /* U+16DD */
  0x0000,   /* U+16DE */
  0x0000,   /* U+16DF */
  0x0000,   /* U+16E0 */
  0x0000,   /* U+16E1 */
  0x0000,   /* U+16E2 */
  0x0000,   /* U+16E3 */
  0x0000,   /* U+16E4 */
  0x0000,   /* U+16E5 */
  0x0000,   /* U+16E6 */
  0x0000,   /* U+16E7 */
  0x0000,   /* U+16E8 */
  0x0000,   /* U+16E9 */
  0x0000,   /* U+16EA */
  0x0000,   /* U+16EB */
  0x0000,   /* U+16EC */
  0x0000,   /* U+16ED */
  0x0000,   /* U+16EE */
  0x0004,   /* U+16EF */
  0x0000,   /* U+16F0 */
  0x0000,   /* U+16F1 */
  0x0000,   /* U+16F2 */
  0x0000,   /* U+16F3 */
  0x0000,   /* U+16F4 */
  0x0000,   /* U+16F5 */
  0x0000,   /* U+16F6 */
  0x0000,   /* U+16F7 */
  0x0000,   /* U+16F8 */
  0x0000,   /* U+16F9 */
  0x0000,   /* U+16FA */
  0x0000,   /* U+16FB */
  0x0000,   /* U+16FC */
  0x0000,   /* U+16FD */
  0x0000,   /* U+16FE */
  0x0000,   /* U+16FF */
};

uint16 uca900_p017[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2F66,   /* U+1700 */
  0x2F67,   /* U+1701 */
  0x2F68,   /* U+1702 */
  0x2F69,   /* U+1703 */
  0x2F6A,   /* U+1704 */
  0x2F6B,   /* U+1705 */
  0x2F6C,   /* U+1706 */
  0x2F6D,   /* U+1707 */
  0x2F6E,   /* U+1708 */
  0x2F6F,   /* U+1709 */
  0x2F70,   /* U+170A */
  0x2F71,   /* U+170B */
  0x2F72,   /* U+170C */
  0xFBC0,   /* U+170D */
  0x2F73,   /* U+170E */
  0x2F74,   /* U+170F */
  0x2F75,   /* U+1710 */
  0x2F76,   /* U+1711 */
  0x2F77,   /* U+1712 */
  0x2F78,   /* U+1713 */
  0x2F79,   /* U+1714 */
  0xFBC0,   /* U+1715 */
  0xFBC0,   /* U+1716 */
  0xFBC0,   /* U+1717 */
  0xFBC0,   /* U+1718 */
  0xFBC0,   /* U+1719 */
  0xFBC0,   /* U+171A */
  0xFBC0,   /* U+171B */
  0xFBC0,   /* U+171C */
  0xFBC0,   /* U+171D */
  0xFBC0,   /* U+171E */
  0xFBC0,   /* U+171F */
  0x2F7A,   /* U+1720 */
  0x2F7B,   /* U+1721 */
  0x2F7C,   /* U+1722 */
  0x2F7D,   /* U+1723 */
  0x2F7E,   /* U+1724 */
  0x2F7F,   /* U+1725 */
  0x2F80,   /* U+1726 */
  0x2F81,   /* U+1727 */
  0x2F82,   /* U+1728 */
  0x2F83,   /* U+1729 */
  0x2F84,   /* U+172A */
  0x2F85,   /* U+172B */
  0x2F86,   /* U+172C */
  0x2F87,   /* U+172D */
  0x2F88,   /* U+172E */
  0x2F89,   /* U+172F */
  0x2F8A,   /* U+1730 */
  0x2F8B,   /* U+1731 */
  0x2F8C,   /* U+1732 */
  0x2F8D,   /* U+1733 */
  0x2F8E,   /* U+1734 */
  0x0296,   /* U+1735 */
  0x0297,   /* U+1736 */
  0xFBC0,   /* U+1737 */
  0xFBC0,   /* U+1738 */
  0xFBC0,   /* U+1739 */
  0xFBC0,   /* U+173A */
  0xFBC0,   /* U+173B */
  0xFBC0,   /* U+173C */
  0xFBC0,   /* U+173D */
  0xFBC0,   /* U+173E */
  0xFBC0,   /* U+173F */
  0x2F8F,   /* U+1740 */
  0x2F90,   /* U+1741 */
  0x2F91,   /* U+1742 */
  0x2F92,   /* U+1743 */
  0x2F93,   /* U+1744 */
  0x2F94,   /* U+1745 */
  0x2F95,   /* U+1746 */
  0x2F96,   /* U+1747 */
  0x2F97,   /* U+1748 */
  0x2F98,   /* U+1749 */
  0x2F99,   /* U+174A */
  0x2F9A,   /* U+174B */
  0x2F9B,   /* U+174C */
  0x2F9C,   /* U+174D */
  0x2F9D,   /* U+174E */
  0x2F9E,   /* U+174F */
  0x2F9F,   /* U+1750 */
  0x2FA0,   /* U+1751 */
  0x2FA1,   /* U+1752 */
  0x2FA2,   /* U+1753 */
  0xFBC0,   /* U+1754 */
  0xFBC0,   /* U+1755 */
  0xFBC0,   /* U+1756 */
  0xFBC0,   /* U+1757 */
  0xFBC0,   /* U+1758 */
  0xFBC0,   /* U+1759 */
  0xFBC0,   /* U+175A */
  0xFBC0,   /* U+175B */
  0xFBC0,   /* U+175C */
  0xFBC0,   /* U+175D */
  0xFBC0,   /* U+175E */
  0xFBC0,   /* U+175F */
  0x2FA3,   /* U+1760 */
  0x2FA4,   /* U+1761 */
  0x2FA5,   /* U+1762 */
  0x2FA6,   /* U+1763 */
  0x2FA7,   /* U+1764 */
  0x2FA8,   /* U+1765 */
  0x2FA9,   /* U+1766 */
  0x2FAA,   /* U+1767 */
  0x2FAB,   /* U+1768 */
  0x2FAC,   /* U+1769 */
  0x2FAD,   /* U+176A */
  0x2FAE,   /* U+176B */
  0x2FAF,   /* U+176C */
  0xFBC0,   /* U+176D */
  0x2FB0,   /* U+176E */
  0x2FB1,   /* U+176F */
  0x2FB2,   /* U+1770 */
  0xFBC0,   /* U+1771 */
  0x2FB3,   /* U+1772 */
  0x2FB4,   /* U+1773 */
  0xFBC0,   /* U+1774 */
  0xFBC0,   /* U+1775 */
  0xFBC0,   /* U+1776 */
  0xFBC0,   /* U+1777 */
  0xFBC0,   /* U+1778 */
  0xFBC0,   /* U+1779 */
  0xFBC0,   /* U+177A */
  0xFBC0,   /* U+177B */
  0xFBC0,   /* U+177C */
  0xFBC0,   /* U+177D */
  0xFBC0,   /* U+177E */
  0xFBC0,   /* U+177F */
  0x3116,   /* U+1780 */
  0x3117,   /* U+1781 */
  0x3118,   /* U+1782 */
  0x3119,   /* U+1783 */
  0x311A,   /* U+1784 */
  0x311B,   /* U+1785 */
  0x311C,   /* U+1786 */
  0x311D,   /* U+1787 */
  0x311E,   /* U+1788 */
  0x311F,   /* U+1789 */
  0x3120,   /* U+178A */
  0x3121,   /* U+178B */
  0x3122,   /* U+178C */
  0x3123,   /* U+178D */
  0x3124,   /* U+178E */
  0x3125,   /* U+178F */
  0x3126,   /* U+1790 */
  0x3127,   /* U+1791 */
  0x3128,   /* U+1792 */
  0x3129,   /* U+1793 */
  0x312A,   /* U+1794 */
  0x312B,   /* U+1795 */
  0x312C,   /* U+1796 */
  0x312D,   /* U+1797 */
  0x312E,   /* U+1798 */
  0x312F,   /* U+1799 */
  0x3130,   /* U+179A */
  0x3131,   /* U+179B */
  0x3132,   /* U+179C */
  0x3133,   /* U+179D */
  0x3134,   /* U+179E */
  0x3135,   /* U+179F */
  0x3136,   /* U+17A0 */
  0x3137,   /* U+17A1 */
  0x3138,   /* U+17A2 */
  0x313A,   /* U+17A3 */
  0x313B,   /* U+17A4 */
  0x313C,   /* U+17A5 */
  0x313D,   /* U+17A6 */
  0x313E,   /* U+17A7 */
  0x313F,   /* U+17A8 */
  0x3140,   /* U+17A9 */
  0x3141,   /* U+17AA */
  0x3142,   /* U+17AB */
  0x3143,   /* U+17AC */
  0x3144,   /* U+17AD */
  0x3145,   /* U+17AE */
  0x3146,   /* U+17AF */
  0x3147,   /* U+17B0 */
  0x3148,   /* U+17B1 */
  0x3149,   /* U+17B2 */
  0x314A,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x314B,   /* U+17B6 */
  0x314C,   /* U+17B7 */
  0x314D,   /* U+17B8 */
  0x314E,   /* U+17B9 */
  0x314F,   /* U+17BA */
  0x3150,   /* U+17BB */
  0x3151,   /* U+17BC */
  0x3152,   /* U+17BD */
  0x3153,   /* U+17BE */
  0x3154,   /* U+17BF */
  0x3155,   /* U+17C0 */
  0x3156,   /* U+17C1 */
  0x3157,   /* U+17C2 */
  0x3158,   /* U+17C3 */
  0x3159,   /* U+17C4 */
  0x315A,   /* U+17C5 */
  0x0000,   /* U+17C6 */
  0x0000,   /* U+17C7 */
  0x0000,   /* U+17C8 */
  0x0000,   /* U+17C9 */
  0x0000,   /* U+17CA */
  0x0000,   /* U+17CB */
  0x0000,   /* U+17CC */
  0x0000,   /* U+17CD */
  0x0000,   /* U+17CE */
  0x0000,   /* U+17CF */
  0x0000,   /* U+17D0 */
  0x0000,   /* U+17D1 */
  0x315B,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x029B,   /* U+17D4 */
  0x029C,   /* U+17D5 */
  0x0259,   /* U+17D6 */
  0x1BFD,   /* U+17D7 */
  0x0412,   /* U+17D8 */
  0x0413,   /* U+17D9 */
  0x0414,   /* U+17DA */
  0x1C1E,   /* U+17DB */
  0x3139,   /* U+17DC */
  0x0000,   /* U+17DD */
  0xFBC0,   /* U+17DE */
  0xFBC0,   /* U+17DF */
  0x1C3D,   /* U+17E0 */
  0x1C3E,   /* U+17E1 */
  0x1C3F,   /* U+17E2 */
  0x1C40,   /* U+17E3 */
  0x1C41,   /* U+17E4 */
  0x1C42,   /* U+17E5 */
  0x1C43,   /* U+17E6 */
  0x1C44,   /* U+17E7 */
  0x1C45,   /* U+17E8 */
  0x1C46,   /* U+17E9 */
  0xFBC0,   /* U+17EA */
  0xFBC0,   /* U+17EB */
  0xFBC0,   /* U+17EC */
  0xFBC0,   /* U+17ED */
  0xFBC0,   /* U+17EE */
  0xFBC0,   /* U+17EF */
  0x1C3D,   /* U+17F0 */
  0x1C3E,   /* U+17F1 */
  0x1C3F,   /* U+17F2 */
  0x1C40,   /* U+17F3 */
  0x1C41,   /* U+17F4 */
  0x1C42,   /* U+17F5 */
  0x1C43,   /* U+17F6 */
  0x1C44,   /* U+17F7 */
  0x1C45,   /* U+17F8 */
  0x1C46,   /* U+17F9 */
  0xFBC0,   /* U+17FA */
  0xFBC0,   /* U+17FB */
  0xFBC0,   /* U+17FC */
  0xFBC0,   /* U+17FD */
  0xFBC0,   /* U+17FE */
  0xFBC0,   /* U+17FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1700 */
  0x0020,   /* U+1701 */
  0x0020,   /* U+1702 */
  0x0020,   /* U+1703 */
  0x0020,   /* U+1704 */
  0x0020,   /* U+1705 */
  0x0020,   /* U+1706 */
  0x0020,   /* U+1707 */
  0x0020,   /* U+1708 */
  0x0020,   /* U+1709 */
  0x0020,   /* U+170A */
  0x0020,   /* U+170B */
  0x0020,   /* U+170C */
  0x0020,   /* U+170D */
  0x0020,   /* U+170E */
  0x0020,   /* U+170F */
  0x0020,   /* U+1710 */
  0x0020,   /* U+1711 */
  0x0020,   /* U+1712 */
  0x0020,   /* U+1713 */
  0x0020,   /* U+1714 */
  0x0020,   /* U+1715 */
  0x0020,   /* U+1716 */
  0x0020,   /* U+1717 */
  0x0020,   /* U+1718 */
  0x0020,   /* U+1719 */
  0x0020,   /* U+171A */
  0x0020,   /* U+171B */
  0x0020,   /* U+171C */
  0x0020,   /* U+171D */
  0x0020,   /* U+171E */
  0x0020,   /* U+171F */
  0x0020,   /* U+1720 */
  0x0020,   /* U+1721 */
  0x0020,   /* U+1722 */
  0x0020,   /* U+1723 */
  0x0020,   /* U+1724 */
  0x0020,   /* U+1725 */
  0x0020,   /* U+1726 */
  0x0020,   /* U+1727 */
  0x0020,   /* U+1728 */
  0x0020,   /* U+1729 */
  0x0020,   /* U+172A */
  0x0020,   /* U+172B */
  0x0020,   /* U+172C */
  0x0020,   /* U+172D */
  0x0020,   /* U+172E */
  0x0020,   /* U+172F */
  0x0020,   /* U+1730 */
  0x0020,   /* U+1731 */
  0x0020,   /* U+1732 */
  0x0020,   /* U+1733 */
  0x0020,   /* U+1734 */
  0x0020,   /* U+1735 */
  0x0020,   /* U+1736 */
  0x0020,   /* U+1737 */
  0x0020,   /* U+1738 */
  0x0020,   /* U+1739 */
  0x0020,   /* U+173A */
  0x0020,   /* U+173B */
  0x0020,   /* U+173C */
  0x0020,   /* U+173D */
  0x0020,   /* U+173E */
  0x0020,   /* U+173F */
  0x0020,   /* U+1740 */
  0x0020,   /* U+1741 */
  0x0020,   /* U+1742 */
  0x0020,   /* U+1743 */
  0x0020,   /* U+1744 */
  0x0020,   /* U+1745 */
  0x0020,   /* U+1746 */
  0x0020,   /* U+1747 */
  0x0020,   /* U+1748 */
  0x0020,   /* U+1749 */
  0x0020,   /* U+174A */
  0x0020,   /* U+174B */
  0x0020,   /* U+174C */
  0x0020,   /* U+174D */
  0x0020,   /* U+174E */
  0x0020,   /* U+174F */
  0x0020,   /* U+1750 */
  0x0020,   /* U+1751 */
  0x0020,   /* U+1752 */
  0x0020,   /* U+1753 */
  0x0020,   /* U+1754 */
  0x0020,   /* U+1755 */
  0x0020,   /* U+1756 */
  0x0020,   /* U+1757 */
  0x0020,   /* U+1758 */
  0x0020,   /* U+1759 */
  0x0020,   /* U+175A */
  0x0020,   /* U+175B */
  0x0020,   /* U+175C */
  0x0020,   /* U+175D */
  0x0020,   /* U+175E */
  0x0020,   /* U+175F */
  0x0020,   /* U+1760 */
  0x0020,   /* U+1761 */
  0x0020,   /* U+1762 */
  0x0020,   /* U+1763 */
  0x0020,   /* U+1764 */
  0x0020,   /* U+1765 */
  0x0020,   /* U+1766 */
  0x0020,   /* U+1767 */
  0x0020,   /* U+1768 */
  0x0020,   /* U+1769 */
  0x0020,   /* U+176A */
  0x0020,   /* U+176B */
  0x0020,   /* U+176C */
  0x0020,   /* U+176D */
  0x0020,   /* U+176E */
  0x0020,   /* U+176F */
  0x0020,   /* U+1770 */
  0x0020,   /* U+1771 */
  0x0020,   /* U+1772 */
  0x0020,   /* U+1773 */
  0x0020,   /* U+1774 */
  0x0020,   /* U+1775 */
  0x0020,   /* U+1776 */
  0x0020,   /* U+1777 */
  0x0020,   /* U+1778 */
  0x0020,   /* U+1779 */
  0x0020,   /* U+177A */
  0x0020,   /* U+177B */
  0x0020,   /* U+177C */
  0x0020,   /* U+177D */
  0x0020,   /* U+177E */
  0x0020,   /* U+177F */
  0x0020,   /* U+1780 */
  0x0020,   /* U+1781 */
  0x0020,   /* U+1782 */
  0x0020,   /* U+1783 */
  0x0020,   /* U+1784 */
  0x0020,   /* U+1785 */
  0x0020,   /* U+1786 */
  0x0020,   /* U+1787 */
  0x0020,   /* U+1788 */
  0x0020,   /* U+1789 */
  0x0020,   /* U+178A */
  0x0020,   /* U+178B */
  0x0020,   /* U+178C */
  0x0020,   /* U+178D */
  0x0020,   /* U+178E */
  0x0020,   /* U+178F */
  0x0020,   /* U+1790 */
  0x0020,   /* U+1791 */
  0x0020,   /* U+1792 */
  0x0020,   /* U+1793 */
  0x0020,   /* U+1794 */
  0x0020,   /* U+1795 */
  0x0020,   /* U+1796 */
  0x0020,   /* U+1797 */
  0x0020,   /* U+1798 */
  0x0020,   /* U+1799 */
  0x0020,   /* U+179A */
  0x0020,   /* U+179B */
  0x0020,   /* U+179C */
  0x0020,   /* U+179D */
  0x0020,   /* U+179E */
  0x0020,   /* U+179F */
  0x0020,   /* U+17A0 */
  0x0020,   /* U+17A1 */
  0x0020,   /* U+17A2 */
  0x0020,   /* U+17A3 */
  0x0020,   /* U+17A4 */
  0x0020,   /* U+17A5 */
  0x0020,   /* U+17A6 */
  0x0020,   /* U+17A7 */
  0x0020,   /* U+17A8 */
  0x0020,   /* U+17A9 */
  0x0020,   /* U+17AA */
  0x0020,   /* U+17AB */
  0x0020,   /* U+17AC */
  0x0020,   /* U+17AD */
  0x0020,   /* U+17AE */
  0x0020,   /* U+17AF */
  0x0020,   /* U+17B0 */
  0x0020,   /* U+17B1 */
  0x0020,   /* U+17B2 */
  0x0020,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x0020,   /* U+17B6 */
  0x0020,   /* U+17B7 */
  0x0020,   /* U+17B8 */
  0x0020,   /* U+17B9 */
  0x0020,   /* U+17BA */
  0x0020,   /* U+17BB */
  0x0020,   /* U+17BC */
  0x0020,   /* U+17BD */
  0x0020,   /* U+17BE */
  0x0020,   /* U+17BF */
  0x0020,   /* U+17C0 */
  0x0020,   /* U+17C1 */
  0x0020,   /* U+17C2 */
  0x0020,   /* U+17C3 */
  0x0020,   /* U+17C4 */
  0x0020,   /* U+17C5 */
  0x00C4,   /* U+17C6 */
  0x00C5,   /* U+17C7 */
  0x00E6,   /* U+17C8 */
  0x00E7,   /* U+17C9 */
  0x00E8,   /* U+17CA */
  0x0033,   /* U+17CB */
  0x0033,   /* U+17CC */
  0x0033,   /* U+17CD */
  0x0033,   /* U+17CE */
  0x0033,   /* U+17CF */
  0x0033,   /* U+17D0 */
  0x0033,   /* U+17D1 */
  0x0020,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x0020,   /* U+17D4 */
  0x0020,   /* U+17D5 */
  0x0020,   /* U+17D6 */
  0x0020,   /* U+17D7 */
  0x0020,   /* U+17D8 */
  0x0020,   /* U+17D9 */
  0x0020,   /* U+17DA */
  0x0020,   /* U+17DB */
  0x0020,   /* U+17DC */
  0x0033,   /* U+17DD */
  0x0020,   /* U+17DE */
  0x0020,   /* U+17DF */
  0x0020,   /* U+17E0 */
  0x0020,   /* U+17E1 */
  0x0020,   /* U+17E2 */
  0x0020,   /* U+17E3 */
  0x0020,   /* U+17E4 */
  0x0020,   /* U+17E5 */
  0x0020,   /* U+17E6 */
  0x0020,   /* U+17E7 */
  0x0020,   /* U+17E8 */
  0x0020,   /* U+17E9 */
  0x0020,   /* U+17EA */
  0x0020,   /* U+17EB */
  0x0020,   /* U+17EC */
  0x0020,   /* U+17ED */
  0x0020,   /* U+17EE */
  0x0020,   /* U+17EF */
  0x0020,   /* U+17F0 */
  0x0020,   /* U+17F1 */
  0x0020,   /* U+17F2 */
  0x0020,   /* U+17F3 */
  0x0020,   /* U+17F4 */
  0x0020,   /* U+17F5 */
  0x0020,   /* U+17F6 */
  0x0020,   /* U+17F7 */
  0x0020,   /* U+17F8 */
  0x0020,   /* U+17F9 */
  0x0020,   /* U+17FA */
  0x0020,   /* U+17FB */
  0x0020,   /* U+17FC */
  0x0020,   /* U+17FD */
  0x0020,   /* U+17FE */
  0x0020,   /* U+17FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1700 */
  0x0002,   /* U+1701 */
  0x0002,   /* U+1702 */
  0x0002,   /* U+1703 */
  0x0002,   /* U+1704 */
  0x0002,   /* U+1705 */
  0x0002,   /* U+1706 */
  0x0002,   /* U+1707 */
  0x0002,   /* U+1708 */
  0x0002,   /* U+1709 */
  0x0002,   /* U+170A */
  0x0002,   /* U+170B */
  0x0002,   /* U+170C */
  0x0002,   /* U+170D */
  0x0002,   /* U+170E */
  0x0002,   /* U+170F */
  0x0002,   /* U+1710 */
  0x0002,   /* U+1711 */
  0x0002,   /* U+1712 */
  0x0002,   /* U+1713 */
  0x0002,   /* U+1714 */
  0x0002,   /* U+1715 */
  0x0002,   /* U+1716 */
  0x0002,   /* U+1717 */
  0x0002,   /* U+1718 */
  0x0002,   /* U+1719 */
  0x0002,   /* U+171A */
  0x0002,   /* U+171B */
  0x0002,   /* U+171C */
  0x0002,   /* U+171D */
  0x0002,   /* U+171E */
  0x0002,   /* U+171F */
  0x0002,   /* U+1720 */
  0x0002,   /* U+1721 */
  0x0002,   /* U+1722 */
  0x0002,   /* U+1723 */
  0x0002,   /* U+1724 */
  0x0002,   /* U+1725 */
  0x0002,   /* U+1726 */
  0x0002,   /* U+1727 */
  0x0002,   /* U+1728 */
  0x0002,   /* U+1729 */
  0x0002,   /* U+172A */
  0x0002,   /* U+172B */
  0x0002,   /* U+172C */
  0x0002,   /* U+172D */
  0x0002,   /* U+172E */
  0x0002,   /* U+172F */
  0x0002,   /* U+1730 */
  0x0002,   /* U+1731 */
  0x0002,   /* U+1732 */
  0x0002,   /* U+1733 */
  0x0002,   /* U+1734 */
  0x0002,   /* U+1735 */
  0x0002,   /* U+1736 */
  0x0002,   /* U+1737 */
  0x0002,   /* U+1738 */
  0x0002,   /* U+1739 */
  0x0002,   /* U+173A */
  0x0002,   /* U+173B */
  0x0002,   /* U+173C */
  0x0002,   /* U+173D */
  0x0002,   /* U+173E */
  0x0002,   /* U+173F */
  0x0002,   /* U+1740 */
  0x0002,   /* U+1741 */
  0x0002,   /* U+1742 */
  0x0002,   /* U+1743 */
  0x0002,   /* U+1744 */
  0x0002,   /* U+1745 */
  0x0002,   /* U+1746 */
  0x0002,   /* U+1747 */
  0x0002,   /* U+1748 */
  0x0002,   /* U+1749 */
  0x0002,   /* U+174A */
  0x0002,   /* U+174B */
  0x0002,   /* U+174C */
  0x0002,   /* U+174D */
  0x0002,   /* U+174E */
  0x0002,   /* U+174F */
  0x0002,   /* U+1750 */
  0x0002,   /* U+1751 */
  0x0002,   /* U+1752 */
  0x0002,   /* U+1753 */
  0x0002,   /* U+1754 */
  0x0002,   /* U+1755 */
  0x0002,   /* U+1756 */
  0x0002,   /* U+1757 */
  0x0002,   /* U+1758 */
  0x0002,   /* U+1759 */
  0x0002,   /* U+175A */
  0x0002,   /* U+175B */
  0x0002,   /* U+175C */
  0x0002,   /* U+175D */
  0x0002,   /* U+175E */
  0x0002,   /* U+175F */
  0x0002,   /* U+1760 */
  0x0002,   /* U+1761 */
  0x0002,   /* U+1762 */
  0x0002,   /* U+1763 */
  0x0002,   /* U+1764 */
  0x0002,   /* U+1765 */
  0x0002,   /* U+1766 */
  0x0002,   /* U+1767 */
  0x0002,   /* U+1768 */
  0x0002,   /* U+1769 */
  0x0002,   /* U+176A */
  0x0002,   /* U+176B */
  0x0002,   /* U+176C */
  0x0002,   /* U+176D */
  0x0002,   /* U+176E */
  0x0002,   /* U+176F */
  0x0002,   /* U+1770 */
  0x0002,   /* U+1771 */
  0x0002,   /* U+1772 */
  0x0002,   /* U+1773 */
  0x0002,   /* U+1774 */
  0x0002,   /* U+1775 */
  0x0002,   /* U+1776 */
  0x0002,   /* U+1777 */
  0x0002,   /* U+1778 */
  0x0002,   /* U+1779 */
  0x0002,   /* U+177A */
  0x0002,   /* U+177B */
  0x0002,   /* U+177C */
  0x0002,   /* U+177D */
  0x0002,   /* U+177E */
  0x0002,   /* U+177F */
  0x0002,   /* U+1780 */
  0x0002,   /* U+1781 */
  0x0002,   /* U+1782 */
  0x0002,   /* U+1783 */
  0x0002,   /* U+1784 */
  0x0002,   /* U+1785 */
  0x0002,   /* U+1786 */
  0x0002,   /* U+1787 */
  0x0002,   /* U+1788 */
  0x0002,   /* U+1789 */
  0x0002,   /* U+178A */
  0x0002,   /* U+178B */
  0x0002,   /* U+178C */
  0x0002,   /* U+178D */
  0x0002,   /* U+178E */
  0x0002,   /* U+178F */
  0x0002,   /* U+1790 */
  0x0002,   /* U+1791 */
  0x0002,   /* U+1792 */
  0x0002,   /* U+1793 */
  0x0002,   /* U+1794 */
  0x0002,   /* U+1795 */
  0x0002,   /* U+1796 */
  0x0002,   /* U+1797 */
  0x0002,   /* U+1798 */
  0x0002,   /* U+1799 */
  0x0002,   /* U+179A */
  0x0002,   /* U+179B */
  0x0002,   /* U+179C */
  0x0002,   /* U+179D */
  0x0002,   /* U+179E */
  0x0002,   /* U+179F */
  0x0002,   /* U+17A0 */
  0x0002,   /* U+17A1 */
  0x0002,   /* U+17A2 */
  0x0002,   /* U+17A3 */
  0x0002,   /* U+17A4 */
  0x0002,   /* U+17A5 */
  0x0002,   /* U+17A6 */
  0x0002,   /* U+17A7 */
  0x0002,   /* U+17A8 */
  0x0002,   /* U+17A9 */
  0x0002,   /* U+17AA */
  0x0002,   /* U+17AB */
  0x0002,   /* U+17AC */
  0x0002,   /* U+17AD */
  0x0002,   /* U+17AE */
  0x0002,   /* U+17AF */
  0x0002,   /* U+17B0 */
  0x0002,   /* U+17B1 */
  0x0002,   /* U+17B2 */
  0x0002,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x0002,   /* U+17B6 */
  0x0002,   /* U+17B7 */
  0x0002,   /* U+17B8 */
  0x0002,   /* U+17B9 */
  0x0002,   /* U+17BA */
  0x0002,   /* U+17BB */
  0x0002,   /* U+17BC */
  0x0002,   /* U+17BD */
  0x0002,   /* U+17BE */
  0x0002,   /* U+17BF */
  0x0002,   /* U+17C0 */
  0x0002,   /* U+17C1 */
  0x0002,   /* U+17C2 */
  0x0002,   /* U+17C3 */
  0x0002,   /* U+17C4 */
  0x0002,   /* U+17C5 */
  0x0002,   /* U+17C6 */
  0x0002,   /* U+17C7 */
  0x0002,   /* U+17C8 */
  0x0002,   /* U+17C9 */
  0x0002,   /* U+17CA */
  0x0002,   /* U+17CB */
  0x0002,   /* U+17CC */
  0x0002,   /* U+17CD */
  0x0002,   /* U+17CE */
  0x0002,   /* U+17CF */
  0x0002,   /* U+17D0 */
  0x0002,   /* U+17D1 */
  0x0002,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x0002,   /* U+17D4 */
  0x0002,   /* U+17D5 */
  0x0002,   /* U+17D6 */
  0x0002,   /* U+17D7 */
  0x0002,   /* U+17D8 */
  0x0002,   /* U+17D9 */
  0x0002,   /* U+17DA */
  0x0002,   /* U+17DB */
  0x0002,   /* U+17DC */
  0x0002,   /* U+17DD */
  0x0002,   /* U+17DE */
  0x0002,   /* U+17DF */
  0x0002,   /* U+17E0 */
  0x0002,   /* U+17E1 */
  0x0002,   /* U+17E2 */
  0x0002,   /* U+17E3 */
  0x0002,   /* U+17E4 */
  0x0002,   /* U+17E5 */
  0x0002,   /* U+17E6 */
  0x0002,   /* U+17E7 */
  0x0002,   /* U+17E8 */
  0x0002,   /* U+17E9 */
  0x0002,   /* U+17EA */
  0x0002,   /* U+17EB */
  0x0002,   /* U+17EC */
  0x0002,   /* U+17ED */
  0x0002,   /* U+17EE */
  0x0002,   /* U+17EF */
  0x0002,   /* U+17F0 */
  0x0002,   /* U+17F1 */
  0x0002,   /* U+17F2 */
  0x0002,   /* U+17F3 */
  0x0002,   /* U+17F4 */
  0x0002,   /* U+17F5 */
  0x0002,   /* U+17F6 */
  0x0002,   /* U+17F7 */
  0x0002,   /* U+17F8 */
  0x0002,   /* U+17F9 */
  0x0002,   /* U+17FA */
  0x0002,   /* U+17FB */
  0x0002,   /* U+17FC */
  0x0002,   /* U+17FD */
  0x0002,   /* U+17FE */
  0x0002,   /* U+17FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1700 */
  0x0000,   /* U+1701 */
  0x0000,   /* U+1702 */
  0x0000,   /* U+1703 */
  0x0000,   /* U+1704 */
  0x0000,   /* U+1705 */
  0x0000,   /* U+1706 */
  0x0000,   /* U+1707 */
  0x0000,   /* U+1708 */
  0x0000,   /* U+1709 */
  0x0000,   /* U+170A */
  0x0000,   /* U+170B */
  0x0000,   /* U+170C */
  0x970D,   /* U+170D */
  0x0000,   /* U+170E */
  0x0000,   /* U+170F */
  0x0000,   /* U+1710 */
  0x0000,   /* U+1711 */
  0x0000,   /* U+1712 */
  0x0000,   /* U+1713 */
  0x0000,   /* U+1714 */
  0x9715,   /* U+1715 */
  0x9716,   /* U+1716 */
  0x9717,   /* U+1717 */
  0x9718,   /* U+1718 */
  0x9719,   /* U+1719 */
  0x971A,   /* U+171A */
  0x971B,   /* U+171B */
  0x971C,   /* U+171C */
  0x971D,   /* U+171D */
  0x971E,   /* U+171E */
  0x971F,   /* U+171F */
  0x0000,   /* U+1720 */
  0x0000,   /* U+1721 */
  0x0000,   /* U+1722 */
  0x0000,   /* U+1723 */
  0x0000,   /* U+1724 */
  0x0000,   /* U+1725 */
  0x0000,   /* U+1726 */
  0x0000,   /* U+1727 */
  0x0000,   /* U+1728 */
  0x0000,   /* U+1729 */
  0x0000,   /* U+172A */
  0x0000,   /* U+172B */
  0x0000,   /* U+172C */
  0x0000,   /* U+172D */
  0x0000,   /* U+172E */
  0x0000,   /* U+172F */
  0x0000,   /* U+1730 */
  0x0000,   /* U+1731 */
  0x0000,   /* U+1732 */
  0x0000,   /* U+1733 */
  0x0000,   /* U+1734 */
  0x0000,   /* U+1735 */
  0x0000,   /* U+1736 */
  0x9737,   /* U+1737 */
  0x9738,   /* U+1738 */
  0x9739,   /* U+1739 */
  0x973A,   /* U+173A */
  0x973B,   /* U+173B */
  0x973C,   /* U+173C */
  0x973D,   /* U+173D */
  0x973E,   /* U+173E */
  0x973F,   /* U+173F */
  0x0000,   /* U+1740 */
  0x0000,   /* U+1741 */
  0x0000,   /* U+1742 */
  0x0000,   /* U+1743 */
  0x0000,   /* U+1744 */
  0x0000,   /* U+1745 */
  0x0000,   /* U+1746 */
  0x0000,   /* U+1747 */
  0x0000,   /* U+1748 */
  0x0000,   /* U+1749 */
  0x0000,   /* U+174A */
  0x0000,   /* U+174B */
  0x0000,   /* U+174C */
  0x0000,   /* U+174D */
  0x0000,   /* U+174E */
  0x0000,   /* U+174F */
  0x0000,   /* U+1750 */
  0x0000,   /* U+1751 */
  0x0000,   /* U+1752 */
  0x0000,   /* U+1753 */
  0x9754,   /* U+1754 */
  0x9755,   /* U+1755 */
  0x9756,   /* U+1756 */
  0x9757,   /* U+1757 */
  0x9758,   /* U+1758 */
  0x9759,   /* U+1759 */
  0x975A,   /* U+175A */
  0x975B,   /* U+175B */
  0x975C,   /* U+175C */
  0x975D,   /* U+175D */
  0x975E,   /* U+175E */
  0x975F,   /* U+175F */
  0x0000,   /* U+1760 */
  0x0000,   /* U+1761 */
  0x0000,   /* U+1762 */
  0x0000,   /* U+1763 */
  0x0000,   /* U+1764 */
  0x0000,   /* U+1765 */
  0x0000,   /* U+1766 */
  0x0000,   /* U+1767 */
  0x0000,   /* U+1768 */
  0x0000,   /* U+1769 */
  0x0000,   /* U+176A */
  0x0000,   /* U+176B */
  0x0000,   /* U+176C */
  0x976D,   /* U+176D */
  0x0000,   /* U+176E */
  0x0000,   /* U+176F */
  0x0000,   /* U+1770 */
  0x9771,   /* U+1771 */
  0x0000,   /* U+1772 */
  0x0000,   /* U+1773 */
  0x9774,   /* U+1774 */
  0x9775,   /* U+1775 */
  0x9776,   /* U+1776 */
  0x9777,   /* U+1777 */
  0x9778,   /* U+1778 */
  0x9779,   /* U+1779 */
  0x977A,   /* U+177A */
  0x977B,   /* U+177B */
  0x977C,   /* U+177C */
  0x977D,   /* U+177D */
  0x977E,   /* U+177E */
  0x977F,   /* U+177F */
  0x0000,   /* U+1780 */
  0x0000,   /* U+1781 */
  0x0000,   /* U+1782 */
  0x0000,   /* U+1783 */
  0x0000,   /* U+1784 */
  0x0000,   /* U+1785 */
  0x0000,   /* U+1786 */
  0x0000,   /* U+1787 */
  0x0000,   /* U+1788 */
  0x0000,   /* U+1789 */
  0x0000,   /* U+178A */
  0x0000,   /* U+178B */
  0x0000,   /* U+178C */
  0x0000,   /* U+178D */
  0x0000,   /* U+178E */
  0x0000,   /* U+178F */
  0x0000,   /* U+1790 */
  0x0000,   /* U+1791 */
  0x0000,   /* U+1792 */
  0x0000,   /* U+1793 */
  0x0000,   /* U+1794 */
  0x0000,   /* U+1795 */
  0x0000,   /* U+1796 */
  0x0000,   /* U+1797 */
  0x0000,   /* U+1798 */
  0x0000,   /* U+1799 */
  0x0000,   /* U+179A */
  0x0000,   /* U+179B */
  0x0000,   /* U+179C */
  0x0000,   /* U+179D */
  0x0000,   /* U+179E */
  0x0000,   /* U+179F */
  0x0000,   /* U+17A0 */
  0x0000,   /* U+17A1 */
  0x0000,   /* U+17A2 */
  0x0000,   /* U+17A3 */
  0x0000,   /* U+17A4 */
  0x0000,   /* U+17A5 */
  0x0000,   /* U+17A6 */
  0x0000,   /* U+17A7 */
  0x0000,   /* U+17A8 */
  0x0000,   /* U+17A9 */
  0x0000,   /* U+17AA */
  0x0000,   /* U+17AB */
  0x0000,   /* U+17AC */
  0x0000,   /* U+17AD */
  0x0000,   /* U+17AE */
  0x0000,   /* U+17AF */
  0x0000,   /* U+17B0 */
  0x0000,   /* U+17B1 */
  0x0000,   /* U+17B2 */
  0x0000,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x0000,   /* U+17B6 */
  0x0000,   /* U+17B7 */
  0x0000,   /* U+17B8 */
  0x0000,   /* U+17B9 */
  0x0000,   /* U+17BA */
  0x0000,   /* U+17BB */
  0x0000,   /* U+17BC */
  0x0000,   /* U+17BD */
  0x0000,   /* U+17BE */
  0x0000,   /* U+17BF */
  0x0000,   /* U+17C0 */
  0x0000,   /* U+17C1 */
  0x0000,   /* U+17C2 */
  0x0000,   /* U+17C3 */
  0x0000,   /* U+17C4 */
  0x0000,   /* U+17C5 */
  0x0000,   /* U+17C6 */
  0x0000,   /* U+17C7 */
  0x0000,   /* U+17C8 */
  0x0000,   /* U+17C9 */
  0x0000,   /* U+17CA */
  0x0000,   /* U+17CB */
  0x0000,   /* U+17CC */
  0x0000,   /* U+17CD */
  0x0000,   /* U+17CE */
  0x0000,   /* U+17CF */
  0x0000,   /* U+17D0 */
  0x0000,   /* U+17D1 */
  0x0000,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x0000,   /* U+17D4 */
  0x0000,   /* U+17D5 */
  0x0000,   /* U+17D6 */
  0x0000,   /* U+17D7 */
  0x0000,   /* U+17D8 */
  0x0000,   /* U+17D9 */
  0x0000,   /* U+17DA */
  0x0000,   /* U+17DB */
  0x0000,   /* U+17DC */
  0x0000,   /* U+17DD */
  0x97DE,   /* U+17DE */
  0x97DF,   /* U+17DF */
  0x0000,   /* U+17E0 */
  0x0000,   /* U+17E1 */
  0x0000,   /* U+17E2 */
  0x0000,   /* U+17E3 */
  0x0000,   /* U+17E4 */
  0x0000,   /* U+17E5 */
  0x0000,   /* U+17E6 */
  0x0000,   /* U+17E7 */
  0x0000,   /* U+17E8 */
  0x0000,   /* U+17E9 */
  0x97EA,   /* U+17EA */
  0x97EB,   /* U+17EB */
  0x97EC,   /* U+17EC */
  0x97ED,   /* U+17ED */
  0x97EE,   /* U+17EE */
  0x97EF,   /* U+17EF */
  0x0000,   /* U+17F0 */
  0x0000,   /* U+17F1 */
  0x0000,   /* U+17F2 */
  0x0000,   /* U+17F3 */
  0x0000,   /* U+17F4 */
  0x0000,   /* U+17F5 */
  0x0000,   /* U+17F6 */
  0x0000,   /* U+17F7 */
  0x0000,   /* U+17F8 */
  0x0000,   /* U+17F9 */
  0x97FA,   /* U+17FA */
  0x97FB,   /* U+17FB */
  0x97FC,   /* U+17FC */
  0x97FD,   /* U+17FD */
  0x97FE,   /* U+17FE */
  0x97FF,   /* U+17FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1700 */
  0x0000,   /* U+1701 */
  0x0000,   /* U+1702 */
  0x0000,   /* U+1703 */
  0x0000,   /* U+1704 */
  0x0000,   /* U+1705 */
  0x0000,   /* U+1706 */
  0x0000,   /* U+1707 */
  0x0000,   /* U+1708 */
  0x0000,   /* U+1709 */
  0x0000,   /* U+170A */
  0x0000,   /* U+170B */
  0x0000,   /* U+170C */
  0x0000,   /* U+170D */
  0x0000,   /* U+170E */
  0x0000,   /* U+170F */
  0x0000,   /* U+1710 */
  0x0000,   /* U+1711 */
  0x0000,   /* U+1712 */
  0x0000,   /* U+1713 */
  0x0000,   /* U+1714 */
  0x0000,   /* U+1715 */
  0x0000,   /* U+1716 */
  0x0000,   /* U+1717 */
  0x0000,   /* U+1718 */
  0x0000,   /* U+1719 */
  0x0000,   /* U+171A */
  0x0000,   /* U+171B */
  0x0000,   /* U+171C */
  0x0000,   /* U+171D */
  0x0000,   /* U+171E */
  0x0000,   /* U+171F */
  0x0000,   /* U+1720 */
  0x0000,   /* U+1721 */
  0x0000,   /* U+1722 */
  0x0000,   /* U+1723 */
  0x0000,   /* U+1724 */
  0x0000,   /* U+1725 */
  0x0000,   /* U+1726 */
  0x0000,   /* U+1727 */
  0x0000,   /* U+1728 */
  0x0000,   /* U+1729 */
  0x0000,   /* U+172A */
  0x0000,   /* U+172B */
  0x0000,   /* U+172C */
  0x0000,   /* U+172D */
  0x0000,   /* U+172E */
  0x0000,   /* U+172F */
  0x0000,   /* U+1730 */
  0x0000,   /* U+1731 */
  0x0000,   /* U+1732 */
  0x0000,   /* U+1733 */
  0x0000,   /* U+1734 */
  0x0000,   /* U+1735 */
  0x0000,   /* U+1736 */
  0x0000,   /* U+1737 */
  0x0000,   /* U+1738 */
  0x0000,   /* U+1739 */
  0x0000,   /* U+173A */
  0x0000,   /* U+173B */
  0x0000,   /* U+173C */
  0x0000,   /* U+173D */
  0x0000,   /* U+173E */
  0x0000,   /* U+173F */
  0x0000,   /* U+1740 */
  0x0000,   /* U+1741 */
  0x0000,   /* U+1742 */
  0x0000,   /* U+1743 */
  0x0000,   /* U+1744 */
  0x0000,   /* U+1745 */
  0x0000,   /* U+1746 */
  0x0000,   /* U+1747 */
  0x0000,   /* U+1748 */
  0x0000,   /* U+1749 */
  0x0000,   /* U+174A */
  0x0000,   /* U+174B */
  0x0000,   /* U+174C */
  0x0000,   /* U+174D */
  0x0000,   /* U+174E */
  0x0000,   /* U+174F */
  0x0000,   /* U+1750 */
  0x0000,   /* U+1751 */
  0x0000,   /* U+1752 */
  0x0000,   /* U+1753 */
  0x0000,   /* U+1754 */
  0x0000,   /* U+1755 */
  0x0000,   /* U+1756 */
  0x0000,   /* U+1757 */
  0x0000,   /* U+1758 */
  0x0000,   /* U+1759 */
  0x0000,   /* U+175A */
  0x0000,   /* U+175B */
  0x0000,   /* U+175C */
  0x0000,   /* U+175D */
  0x0000,   /* U+175E */
  0x0000,   /* U+175F */
  0x0000,   /* U+1760 */
  0x0000,   /* U+1761 */
  0x0000,   /* U+1762 */
  0x0000,   /* U+1763 */
  0x0000,   /* U+1764 */
  0x0000,   /* U+1765 */
  0x0000,   /* U+1766 */
  0x0000,   /* U+1767 */
  0x0000,   /* U+1768 */
  0x0000,   /* U+1769 */
  0x0000,   /* U+176A */
  0x0000,   /* U+176B */
  0x0000,   /* U+176C */
  0x0000,   /* U+176D */
  0x0000,   /* U+176E */
  0x0000,   /* U+176F */
  0x0000,   /* U+1770 */
  0x0000,   /* U+1771 */
  0x0000,   /* U+1772 */
  0x0000,   /* U+1773 */
  0x0000,   /* U+1774 */
  0x0000,   /* U+1775 */
  0x0000,   /* U+1776 */
  0x0000,   /* U+1777 */
  0x0000,   /* U+1778 */
  0x0000,   /* U+1779 */
  0x0000,   /* U+177A */
  0x0000,   /* U+177B */
  0x0000,   /* U+177C */
  0x0000,   /* U+177D */
  0x0000,   /* U+177E */
  0x0000,   /* U+177F */
  0x0000,   /* U+1780 */
  0x0000,   /* U+1781 */
  0x0000,   /* U+1782 */
  0x0000,   /* U+1783 */
  0x0000,   /* U+1784 */
  0x0000,   /* U+1785 */
  0x0000,   /* U+1786 */
  0x0000,   /* U+1787 */
  0x0000,   /* U+1788 */
  0x0000,   /* U+1789 */
  0x0000,   /* U+178A */
  0x0000,   /* U+178B */
  0x0000,   /* U+178C */
  0x0000,   /* U+178D */
  0x0000,   /* U+178E */
  0x0000,   /* U+178F */
  0x0000,   /* U+1790 */
  0x0000,   /* U+1791 */
  0x0000,   /* U+1792 */
  0x0000,   /* U+1793 */
  0x0000,   /* U+1794 */
  0x0000,   /* U+1795 */
  0x0000,   /* U+1796 */
  0x0000,   /* U+1797 */
  0x0000,   /* U+1798 */
  0x0000,   /* U+1799 */
  0x0000,   /* U+179A */
  0x0000,   /* U+179B */
  0x0000,   /* U+179C */
  0x0000,   /* U+179D */
  0x0000,   /* U+179E */
  0x0000,   /* U+179F */
  0x0000,   /* U+17A0 */
  0x0000,   /* U+17A1 */
  0x0000,   /* U+17A2 */
  0x0000,   /* U+17A3 */
  0x0000,   /* U+17A4 */
  0x0000,   /* U+17A5 */
  0x0000,   /* U+17A6 */
  0x0000,   /* U+17A7 */
  0x0000,   /* U+17A8 */
  0x0000,   /* U+17A9 */
  0x0000,   /* U+17AA */
  0x0000,   /* U+17AB */
  0x0000,   /* U+17AC */
  0x0000,   /* U+17AD */
  0x0000,   /* U+17AE */
  0x0000,   /* U+17AF */
  0x0000,   /* U+17B0 */
  0x0000,   /* U+17B1 */
  0x0000,   /* U+17B2 */
  0x0000,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x0000,   /* U+17B6 */
  0x0000,   /* U+17B7 */
  0x0000,   /* U+17B8 */
  0x0000,   /* U+17B9 */
  0x0000,   /* U+17BA */
  0x0000,   /* U+17BB */
  0x0000,   /* U+17BC */
  0x0000,   /* U+17BD */
  0x0000,   /* U+17BE */
  0x0000,   /* U+17BF */
  0x0000,   /* U+17C0 */
  0x0000,   /* U+17C1 */
  0x0000,   /* U+17C2 */
  0x0000,   /* U+17C3 */
  0x0000,   /* U+17C4 */
  0x0000,   /* U+17C5 */
  0x0000,   /* U+17C6 */
  0x0000,   /* U+17C7 */
  0x0000,   /* U+17C8 */
  0x0000,   /* U+17C9 */
  0x0000,   /* U+17CA */
  0x0000,   /* U+17CB */
  0x0000,   /* U+17CC */
  0x0000,   /* U+17CD */
  0x0000,   /* U+17CE */
  0x0000,   /* U+17CF */
  0x0000,   /* U+17D0 */
  0x0000,   /* U+17D1 */
  0x0000,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x0000,   /* U+17D4 */
  0x0000,   /* U+17D5 */
  0x0000,   /* U+17D6 */
  0x0000,   /* U+17D7 */
  0x0000,   /* U+17D8 */
  0x0000,   /* U+17D9 */
  0x0000,   /* U+17DA */
  0x0000,   /* U+17DB */
  0x0000,   /* U+17DC */
  0x0000,   /* U+17DD */
  0x0000,   /* U+17DE */
  0x0000,   /* U+17DF */
  0x0000,   /* U+17E0 */
  0x0000,   /* U+17E1 */
  0x0000,   /* U+17E2 */
  0x0000,   /* U+17E3 */
  0x0000,   /* U+17E4 */
  0x0000,   /* U+17E5 */
  0x0000,   /* U+17E6 */
  0x0000,   /* U+17E7 */
  0x0000,   /* U+17E8 */
  0x0000,   /* U+17E9 */
  0x0000,   /* U+17EA */
  0x0000,   /* U+17EB */
  0x0000,   /* U+17EC */
  0x0000,   /* U+17ED */
  0x0000,   /* U+17EE */
  0x0000,   /* U+17EF */
  0x0000,   /* U+17F0 */
  0x0000,   /* U+17F1 */
  0x0000,   /* U+17F2 */
  0x0000,   /* U+17F3 */
  0x0000,   /* U+17F4 */
  0x0000,   /* U+17F5 */
  0x0000,   /* U+17F6 */
  0x0000,   /* U+17F7 */
  0x0000,   /* U+17F8 */
  0x0000,   /* U+17F9 */
  0x0000,   /* U+17FA */
  0x0000,   /* U+17FB */
  0x0000,   /* U+17FC */
  0x0000,   /* U+17FD */
  0x0000,   /* U+17FE */
  0x0000,   /* U+17FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1700 */
  0x0000,   /* U+1701 */
  0x0000,   /* U+1702 */
  0x0000,   /* U+1703 */
  0x0000,   /* U+1704 */
  0x0000,   /* U+1705 */
  0x0000,   /* U+1706 */
  0x0000,   /* U+1707 */
  0x0000,   /* U+1708 */
  0x0000,   /* U+1709 */
  0x0000,   /* U+170A */
  0x0000,   /* U+170B */
  0x0000,   /* U+170C */
  0x0000,   /* U+170D */
  0x0000,   /* U+170E */
  0x0000,   /* U+170F */
  0x0000,   /* U+1710 */
  0x0000,   /* U+1711 */
  0x0000,   /* U+1712 */
  0x0000,   /* U+1713 */
  0x0000,   /* U+1714 */
  0x0000,   /* U+1715 */
  0x0000,   /* U+1716 */
  0x0000,   /* U+1717 */
  0x0000,   /* U+1718 */
  0x0000,   /* U+1719 */
  0x0000,   /* U+171A */
  0x0000,   /* U+171B */
  0x0000,   /* U+171C */
  0x0000,   /* U+171D */
  0x0000,   /* U+171E */
  0x0000,   /* U+171F */
  0x0000,   /* U+1720 */
  0x0000,   /* U+1721 */
  0x0000,   /* U+1722 */
  0x0000,   /* U+1723 */
  0x0000,   /* U+1724 */
  0x0000,   /* U+1725 */
  0x0000,   /* U+1726 */
  0x0000,   /* U+1727 */
  0x0000,   /* U+1728 */
  0x0000,   /* U+1729 */
  0x0000,   /* U+172A */
  0x0000,   /* U+172B */
  0x0000,   /* U+172C */
  0x0000,   /* U+172D */
  0x0000,   /* U+172E */
  0x0000,   /* U+172F */
  0x0000,   /* U+1730 */
  0x0000,   /* U+1731 */
  0x0000,   /* U+1732 */
  0x0000,   /* U+1733 */
  0x0000,   /* U+1734 */
  0x0000,   /* U+1735 */
  0x0000,   /* U+1736 */
  0x0000,   /* U+1737 */
  0x0000,   /* U+1738 */
  0x0000,   /* U+1739 */
  0x0000,   /* U+173A */
  0x0000,   /* U+173B */
  0x0000,   /* U+173C */
  0x0000,   /* U+173D */
  0x0000,   /* U+173E */
  0x0000,   /* U+173F */
  0x0000,   /* U+1740 */
  0x0000,   /* U+1741 */
  0x0000,   /* U+1742 */
  0x0000,   /* U+1743 */
  0x0000,   /* U+1744 */
  0x0000,   /* U+1745 */
  0x0000,   /* U+1746 */
  0x0000,   /* U+1747 */
  0x0000,   /* U+1748 */
  0x0000,   /* U+1749 */
  0x0000,   /* U+174A */
  0x0000,   /* U+174B */
  0x0000,   /* U+174C */
  0x0000,   /* U+174D */
  0x0000,   /* U+174E */
  0x0000,   /* U+174F */
  0x0000,   /* U+1750 */
  0x0000,   /* U+1751 */
  0x0000,   /* U+1752 */
  0x0000,   /* U+1753 */
  0x0000,   /* U+1754 */
  0x0000,   /* U+1755 */
  0x0000,   /* U+1756 */
  0x0000,   /* U+1757 */
  0x0000,   /* U+1758 */
  0x0000,   /* U+1759 */
  0x0000,   /* U+175A */
  0x0000,   /* U+175B */
  0x0000,   /* U+175C */
  0x0000,   /* U+175D */
  0x0000,   /* U+175E */
  0x0000,   /* U+175F */
  0x0000,   /* U+1760 */
  0x0000,   /* U+1761 */
  0x0000,   /* U+1762 */
  0x0000,   /* U+1763 */
  0x0000,   /* U+1764 */
  0x0000,   /* U+1765 */
  0x0000,   /* U+1766 */
  0x0000,   /* U+1767 */
  0x0000,   /* U+1768 */
  0x0000,   /* U+1769 */
  0x0000,   /* U+176A */
  0x0000,   /* U+176B */
  0x0000,   /* U+176C */
  0x0000,   /* U+176D */
  0x0000,   /* U+176E */
  0x0000,   /* U+176F */
  0x0000,   /* U+1770 */
  0x0000,   /* U+1771 */
  0x0000,   /* U+1772 */
  0x0000,   /* U+1773 */
  0x0000,   /* U+1774 */
  0x0000,   /* U+1775 */
  0x0000,   /* U+1776 */
  0x0000,   /* U+1777 */
  0x0000,   /* U+1778 */
  0x0000,   /* U+1779 */
  0x0000,   /* U+177A */
  0x0000,   /* U+177B */
  0x0000,   /* U+177C */
  0x0000,   /* U+177D */
  0x0000,   /* U+177E */
  0x0000,   /* U+177F */
  0x0000,   /* U+1780 */
  0x0000,   /* U+1781 */
  0x0000,   /* U+1782 */
  0x0000,   /* U+1783 */
  0x0000,   /* U+1784 */
  0x0000,   /* U+1785 */
  0x0000,   /* U+1786 */
  0x0000,   /* U+1787 */
  0x0000,   /* U+1788 */
  0x0000,   /* U+1789 */
  0x0000,   /* U+178A */
  0x0000,   /* U+178B */
  0x0000,   /* U+178C */
  0x0000,   /* U+178D */
  0x0000,   /* U+178E */
  0x0000,   /* U+178F */
  0x0000,   /* U+1790 */
  0x0000,   /* U+1791 */
  0x0000,   /* U+1792 */
  0x0000,   /* U+1793 */
  0x0000,   /* U+1794 */
  0x0000,   /* U+1795 */
  0x0000,   /* U+1796 */
  0x0000,   /* U+1797 */
  0x0000,   /* U+1798 */
  0x0000,   /* U+1799 */
  0x0000,   /* U+179A */
  0x0000,   /* U+179B */
  0x0000,   /* U+179C */
  0x0000,   /* U+179D */
  0x0000,   /* U+179E */
  0x0000,   /* U+179F */
  0x0000,   /* U+17A0 */
  0x0000,   /* U+17A1 */
  0x0000,   /* U+17A2 */
  0x0000,   /* U+17A3 */
  0x0000,   /* U+17A4 */
  0x0000,   /* U+17A5 */
  0x0000,   /* U+17A6 */
  0x0000,   /* U+17A7 */
  0x0000,   /* U+17A8 */
  0x0000,   /* U+17A9 */
  0x0000,   /* U+17AA */
  0x0000,   /* U+17AB */
  0x0000,   /* U+17AC */
  0x0000,   /* U+17AD */
  0x0000,   /* U+17AE */
  0x0000,   /* U+17AF */
  0x0000,   /* U+17B0 */
  0x0000,   /* U+17B1 */
  0x0000,   /* U+17B2 */
  0x0000,   /* U+17B3 */
  0x0000,   /* U+17B4 */
  0x0000,   /* U+17B5 */
  0x0000,   /* U+17B6 */
  0x0000,   /* U+17B7 */
  0x0000,   /* U+17B8 */
  0x0000,   /* U+17B9 */
  0x0000,   /* U+17BA */
  0x0000,   /* U+17BB */
  0x0000,   /* U+17BC */
  0x0000,   /* U+17BD */
  0x0000,   /* U+17BE */
  0x0000,   /* U+17BF */
  0x0000,   /* U+17C0 */
  0x0000,   /* U+17C1 */
  0x0000,   /* U+17C2 */
  0x0000,   /* U+17C3 */
  0x0000,   /* U+17C4 */
  0x0000,   /* U+17C5 */
  0x0000,   /* U+17C6 */
  0x0000,   /* U+17C7 */
  0x0000,   /* U+17C8 */
  0x0000,   /* U+17C9 */
  0x0000,   /* U+17CA */
  0x0000,   /* U+17CB */
  0x0000,   /* U+17CC */
  0x0000,   /* U+17CD */
  0x0000,   /* U+17CE */
  0x0000,   /* U+17CF */
  0x0000,   /* U+17D0 */
  0x0000,   /* U+17D1 */
  0x0000,   /* U+17D2 */
  0x0000,   /* U+17D3 */
  0x0000,   /* U+17D4 */
  0x0000,   /* U+17D5 */
  0x0000,   /* U+17D6 */
  0x0000,   /* U+17D7 */
  0x0000,   /* U+17D8 */
  0x0000,   /* U+17D9 */
  0x0000,   /* U+17DA */
  0x0000,   /* U+17DB */
  0x0000,   /* U+17DC */
  0x0000,   /* U+17DD */
  0x0000,   /* U+17DE */
  0x0000,   /* U+17DF */
  0x0000,   /* U+17E0 */
  0x0000,   /* U+17E1 */
  0x0000,   /* U+17E2 */
  0x0000,   /* U+17E3 */
  0x0000,   /* U+17E4 */
  0x0000,   /* U+17E5 */
  0x0000,   /* U+17E6 */
  0x0000,   /* U+17E7 */
  0x0000,   /* U+17E8 */
  0x0000,   /* U+17E9 */
  0x0000,   /* U+17EA */
  0x0000,   /* U+17EB */
  0x0000,   /* U+17EC */
  0x0000,   /* U+17ED */
  0x0000,   /* U+17EE */
  0x0000,   /* U+17EF */
  0x0000,   /* U+17F0 */
  0x0000,   /* U+17F1 */
  0x0000,   /* U+17F2 */
  0x0000,   /* U+17F3 */
  0x0000,   /* U+17F4 */
  0x0000,   /* U+17F5 */
  0x0000,   /* U+17F6 */
  0x0000,   /* U+17F7 */
  0x0000,   /* U+17F8 */
  0x0000,   /* U+17F9 */
  0x0000,   /* U+17FA */
  0x0000,   /* U+17FB */
  0x0000,   /* U+17FC */
  0x0000,   /* U+17FD */
  0x0000,   /* U+17FE */
  0x0000,   /* U+17FF */
};

uint16 uca900_p018[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x03D9,   /* U+1800 */
  0x0278,   /* U+1801 */
  0x022C,   /* U+1802 */
  0x027D,   /* U+1803 */
  0x0256,   /* U+1804 */
  0x0257,   /* U+1805 */
  0x0211,   /* U+1806 */
  0x0212,   /* U+1807 */
  0x022D,   /* U+1808 */
  0x027E,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0xFBC0,   /* U+180F */
  0x1C3D,   /* U+1810 */
  0x1C3E,   /* U+1811 */
  0x1C3F,   /* U+1812 */
  0x1C40,   /* U+1813 */
  0x1C41,   /* U+1814 */
  0x1C42,   /* U+1815 */
  0x1C43,   /* U+1816 */
  0x1C44,   /* U+1817 */
  0x1C45,   /* U+1818 */
  0x1C46,   /* U+1819 */
  0xFBC0,   /* U+181A */
  0xFBC0,   /* U+181B */
  0xFBC0,   /* U+181C */
  0xFBC0,   /* U+181D */
  0xFBC0,   /* U+181E */
  0xFBC0,   /* U+181F */
  0x32E0,   /* U+1820 */
  0x32E2,   /* U+1821 */
  0x32E5,   /* U+1822 */
  0x32EB,   /* U+1823 */
  0x32ED,   /* U+1824 */
  0x32F0,   /* U+1825 */
  0x32F2,   /* U+1826 */
  0x32F5,   /* U+1827 */
  0x32F6,   /* U+1828 */
  0x32F7,   /* U+1829 */
  0x32FC,   /* U+182A */
  0x32FE,   /* U+182B */
  0x3301,   /* U+182C */
  0x3303,   /* U+182D */
  0x3308,   /* U+182E */
  0x330A,   /* U+182F */
  0x330B,   /* U+1830 */
  0x330C,   /* U+1831 */
  0x3313,   /* U+1832 */
  0x3316,   /* U+1833 */
  0x3319,   /* U+1834 */
  0x331E,   /* U+1835 */
  0x3322,   /* U+1836 */
  0x3325,   /* U+1837 */
  0x3327,   /* U+1838 */
  0x3329,   /* U+1839 */
  0x332C,   /* U+183A */
  0x3331,   /* U+183B */
  0x3332,   /* U+183C */
  0x3335,   /* U+183D */
  0x3339,   /* U+183E */
  0x333C,   /* U+183F */
  0x333D,   /* U+1840 */
  0x333E,   /* U+1841 */
  0x333F,   /* U+1842 */
  0x32DF,   /* U+1843 */
  0x32E3,   /* U+1844 */
  0x32E6,   /* U+1845 */
  0x32EC,   /* U+1846 */
  0x32EE,   /* U+1847 */
  0x32F1,   /* U+1848 */
  0x32F3,   /* U+1849 */
  0x32F8,   /* U+184A */
  0x32FD,   /* U+184B */
  0x32FF,   /* U+184C */
  0x3302,   /* U+184D */
  0x3304,   /* U+184E */
  0x3309,   /* U+184F */
  0x3314,   /* U+1850 */
  0x3317,   /* U+1851 */
  0x331A,   /* U+1852 */
  0x331F,   /* U+1853 */
  0x3333,   /* U+1854 */
  0x3323,   /* U+1855 */
  0x3328,   /* U+1856 */
  0x332D,   /* U+1857 */
  0x3337,   /* U+1858 */
  0x333A,   /* U+1859 */
  0x3340,   /* U+185A */
  0x3341,   /* U+185B */
  0x331C,   /* U+185C */
  0x32E4,   /* U+185D */
  0x32E7,   /* U+185E */
  0x32EA,   /* U+185F */
  0x32F4,   /* U+1860 */
  0x32EF,   /* U+1861 */
  0x32F9,   /* U+1862 */
  0x332E,   /* U+1863 */
  0x3305,   /* U+1864 */
  0x3307,   /* U+1865 */
  0x3300,   /* U+1866 */
  0x330D,   /* U+1867 */
  0x3315,   /* U+1868 */
  0x3318,   /* U+1869 */
  0x3320,   /* U+186A */
  0x332A,   /* U+186B */
  0x3338,   /* U+186C */
  0x333B,   /* U+186D */
  0x3334,   /* U+186E */
  0x3336,   /* U+186F */
  0x3342,   /* U+1870 */
  0x331B,   /* U+1871 */
  0x3324,   /* U+1872 */
  0x32E8,   /* U+1873 */
  0x332F,   /* U+1874 */
  0x3326,   /* U+1875 */
  0x332B,   /* U+1876 */
  0x3321,   /* U+1877 */
  0xFBC0,   /* U+1878 */
  0xFBC0,   /* U+1879 */
  0xFBC0,   /* U+187A */
  0xFBC0,   /* U+187B */
  0xFBC0,   /* U+187C */
  0xFBC0,   /* U+187D */
  0xFBC0,   /* U+187E */
  0xFBC0,   /* U+187F */
  0x32D8,   /* U+1880 */
  0x32D9,   /* U+1881 */
  0x32DA,   /* U+1882 */
  0x32DB,   /* U+1883 */
  0x32DC,   /* U+1884 */
  0x32DD,   /* U+1885 */
  0x32DE,   /* U+1886 */
  0x32E1,   /* U+1887 */
  0x32E9,   /* U+1888 */
  0x3330,   /* U+1889 */
  0x32FA,   /* U+188A */
  0x331D,   /* U+188B */
  0x3343,   /* U+188C */
  0x3345,   /* U+188D */
  0x3346,   /* U+188E */
  0x3348,   /* U+188F */
  0x3349,   /* U+1890 */
  0x334C,   /* U+1891 */
  0x334E,   /* U+1892 */
  0x334F,   /* U+1893 */
  0x3351,   /* U+1894 */
  0x3353,   /* U+1895 */
  0x3355,   /* U+1896 */
  0x3356,   /* U+1897 */
  0x334A,   /* U+1898 */
  0x3354,   /* U+1899 */
  0x3306,   /* U+189A */
  0x32FB,   /* U+189B */
  0x330E,   /* U+189C */
  0x330F,   /* U+189D */
  0x3344,   /* U+189E */
  0x3347,   /* U+189F */
  0x334B,   /* U+18A0 */
  0x334D,   /* U+18A1 */
  0x3310,   /* U+18A2 */
  0x3352,   /* U+18A3 */
  0x3311,   /* U+18A4 */
  0x3312,   /* U+18A5 */
  0x3357,   /* U+18A6 */
  0x3358,   /* U+18A7 */
  0x3350,   /* U+18A8 */
  0x335A,   /* U+18A9 */
  0x3359,   /* U+18AA */
  0xFBC0,   /* U+18AB */
  0xFBC0,   /* U+18AC */
  0xFBC0,   /* U+18AD */
  0xFBC0,   /* U+18AE */
  0xFBC0,   /* U+18AF */
  0x3676,   /* U+18B0 */
  0x3677,   /* U+18B1 */
  0x3678,   /* U+18B2 */
  0x3679,   /* U+18B3 */
  0x367A,   /* U+18B4 */
  0x367B,   /* U+18B5 */
  0x367C,   /* U+18B6 */
  0x367D,   /* U+18B7 */
  0x367E,   /* U+18B8 */
  0x367F,   /* U+18B9 */
  0x3680,   /* U+18BA */
  0x3681,   /* U+18BB */
  0x3682,   /* U+18BC */
  0x3683,   /* U+18BD */
  0x3684,   /* U+18BE */
  0x3685,   /* U+18BF */
  0x3686,   /* U+18C0 */
  0x3687,   /* U+18C1 */
  0x3688,   /* U+18C2 */
  0x3689,   /* U+18C3 */
  0x368A,   /* U+18C4 */
  0x368B,   /* U+18C5 */
  0x368C,   /* U+18C6 */
  0x368D,   /* U+18C7 */
  0x368E,   /* U+18C8 */
  0x368F,   /* U+18C9 */
  0x3690,   /* U+18CA */
  0x3691,   /* U+18CB */
  0x3692,   /* U+18CC */
  0x3693,   /* U+18CD */
  0x3694,   /* U+18CE */
  0x3695,   /* U+18CF */
  0x3696,   /* U+18D0 */
  0x3697,   /* U+18D1 */
  0x3698,   /* U+18D2 */
  0x3699,   /* U+18D3 */
  0x369A,   /* U+18D4 */
  0x369B,   /* U+18D5 */
  0x369C,   /* U+18D6 */
  0x369D,   /* U+18D7 */
  0x369E,   /* U+18D8 */
  0x369F,   /* U+18D9 */
  0x36A0,   /* U+18DA */
  0x36A1,   /* U+18DB */
  0x36A2,   /* U+18DC */
  0x36A3,   /* U+18DD */
  0x36A4,   /* U+18DE */
  0x36A5,   /* U+18DF */
  0x36A6,   /* U+18E0 */
  0x36A7,   /* U+18E1 */
  0x36A8,   /* U+18E2 */
  0x36A9,   /* U+18E3 */
  0x36AA,   /* U+18E4 */
  0x36AB,   /* U+18E5 */
  0x36AC,   /* U+18E6 */
  0x36AD,   /* U+18E7 */
  0x36AE,   /* U+18E8 */
  0x36AF,   /* U+18E9 */
  0x36B0,   /* U+18EA */
  0x36B1,   /* U+18EB */
  0x36B2,   /* U+18EC */
  0x36B3,   /* U+18ED */
  0x36B4,   /* U+18EE */
  0x36B5,   /* U+18EF */
  0x36B6,   /* U+18F0 */
  0x36B7,   /* U+18F1 */
  0x36B8,   /* U+18F2 */
  0x36B9,   /* U+18F3 */
  0x36BA,   /* U+18F4 */
  0x36BB,   /* U+18F5 */
  0xFBC0,   /* U+18F6 */
  0xFBC0,   /* U+18F7 */
  0xFBC0,   /* U+18F8 */
  0xFBC0,   /* U+18F9 */
  0xFBC0,   /* U+18FA */
  0xFBC0,   /* U+18FB */
  0xFBC0,   /* U+18FC */
  0xFBC0,   /* U+18FD */
  0xFBC0,   /* U+18FE */
  0xFBC0,   /* U+18FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1800 */
  0x0020,   /* U+1801 */
  0x0020,   /* U+1802 */
  0x0020,   /* U+1803 */
  0x0020,   /* U+1804 */
  0x0020,   /* U+1805 */
  0x0020,   /* U+1806 */
  0x0020,   /* U+1807 */
  0x0020,   /* U+1808 */
  0x0020,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0x0020,   /* U+180F */
  0x0020,   /* U+1810 */
  0x0020,   /* U+1811 */
  0x0020,   /* U+1812 */
  0x0020,   /* U+1813 */
  0x0020,   /* U+1814 */
  0x0020,   /* U+1815 */
  0x0020,   /* U+1816 */
  0x0020,   /* U+1817 */
  0x0020,   /* U+1818 */
  0x0020,   /* U+1819 */
  0x0020,   /* U+181A */
  0x0020,   /* U+181B */
  0x0020,   /* U+181C */
  0x0020,   /* U+181D */
  0x0020,   /* U+181E */
  0x0020,   /* U+181F */
  0x0020,   /* U+1820 */
  0x0020,   /* U+1821 */
  0x0020,   /* U+1822 */
  0x0020,   /* U+1823 */
  0x0020,   /* U+1824 */
  0x0020,   /* U+1825 */
  0x0020,   /* U+1826 */
  0x0020,   /* U+1827 */
  0x0020,   /* U+1828 */
  0x0020,   /* U+1829 */
  0x0020,   /* U+182A */
  0x0020,   /* U+182B */
  0x0020,   /* U+182C */
  0x0020,   /* U+182D */
  0x0020,   /* U+182E */
  0x0020,   /* U+182F */
  0x0020,   /* U+1830 */
  0x0020,   /* U+1831 */
  0x0020,   /* U+1832 */
  0x0020,   /* U+1833 */
  0x0020,   /* U+1834 */
  0x0020,   /* U+1835 */
  0x0020,   /* U+1836 */
  0x0020,   /* U+1837 */
  0x0020,   /* U+1838 */
  0x0020,   /* U+1839 */
  0x0020,   /* U+183A */
  0x0020,   /* U+183B */
  0x0020,   /* U+183C */
  0x0020,   /* U+183D */
  0x0020,   /* U+183E */
  0x0020,   /* U+183F */
  0x0020,   /* U+1840 */
  0x0020,   /* U+1841 */
  0x0020,   /* U+1842 */
  0x0020,   /* U+1843 */
  0x0020,   /* U+1844 */
  0x0020,   /* U+1845 */
  0x0020,   /* U+1846 */
  0x0020,   /* U+1847 */
  0x0020,   /* U+1848 */
  0x0020,   /* U+1849 */
  0x0020,   /* U+184A */
  0x0020,   /* U+184B */
  0x0020,   /* U+184C */
  0x0020,   /* U+184D */
  0x0020,   /* U+184E */
  0x0020,   /* U+184F */
  0x0020,   /* U+1850 */
  0x0020,   /* U+1851 */
  0x0020,   /* U+1852 */
  0x0020,   /* U+1853 */
  0x0020,   /* U+1854 */
  0x0020,   /* U+1855 */
  0x0020,   /* U+1856 */
  0x0020,   /* U+1857 */
  0x0020,   /* U+1858 */
  0x0020,   /* U+1859 */
  0x0020,   /* U+185A */
  0x0020,   /* U+185B */
  0x0020,   /* U+185C */
  0x0020,   /* U+185D */
  0x0020,   /* U+185E */
  0x0020,   /* U+185F */
  0x0020,   /* U+1860 */
  0x0020,   /* U+1861 */
  0x0020,   /* U+1862 */
  0x0020,   /* U+1863 */
  0x0020,   /* U+1864 */
  0x0020,   /* U+1865 */
  0x0020,   /* U+1866 */
  0x0020,   /* U+1867 */
  0x0020,   /* U+1868 */
  0x0020,   /* U+1869 */
  0x0020,   /* U+186A */
  0x0020,   /* U+186B */
  0x0020,   /* U+186C */
  0x0020,   /* U+186D */
  0x0020,   /* U+186E */
  0x0020,   /* U+186F */
  0x0020,   /* U+1870 */
  0x0020,   /* U+1871 */
  0x0020,   /* U+1872 */
  0x0020,   /* U+1873 */
  0x0020,   /* U+1874 */
  0x0020,   /* U+1875 */
  0x0020,   /* U+1876 */
  0x0020,   /* U+1877 */
  0x0020,   /* U+1878 */
  0x0020,   /* U+1879 */
  0x0020,   /* U+187A */
  0x0020,   /* U+187B */
  0x0020,   /* U+187C */
  0x0020,   /* U+187D */
  0x0020,   /* U+187E */
  0x0020,   /* U+187F */
  0x0020,   /* U+1880 */
  0x0020,   /* U+1881 */
  0x0020,   /* U+1882 */
  0x0020,   /* U+1883 */
  0x0020,   /* U+1884 */
  0x0020,   /* U+1885 */
  0x0020,   /* U+1886 */
  0x0020,   /* U+1887 */
  0x0020,   /* U+1888 */
  0x0020,   /* U+1889 */
  0x0020,   /* U+188A */
  0x0020,   /* U+188B */
  0x0020,   /* U+188C */
  0x0020,   /* U+188D */
  0x0020,   /* U+188E */
  0x0020,   /* U+188F */
  0x0020,   /* U+1890 */
  0x0020,   /* U+1891 */
  0x0020,   /* U+1892 */
  0x0020,   /* U+1893 */
  0x0020,   /* U+1894 */
  0x0020,   /* U+1895 */
  0x0020,   /* U+1896 */
  0x0020,   /* U+1897 */
  0x0020,   /* U+1898 */
  0x0020,   /* U+1899 */
  0x0020,   /* U+189A */
  0x0020,   /* U+189B */
  0x0020,   /* U+189C */
  0x0020,   /* U+189D */
  0x0020,   /* U+189E */
  0x0020,   /* U+189F */
  0x0020,   /* U+18A0 */
  0x0020,   /* U+18A1 */
  0x0020,   /* U+18A2 */
  0x0020,   /* U+18A3 */
  0x0020,   /* U+18A4 */
  0x0020,   /* U+18A5 */
  0x0020,   /* U+18A6 */
  0x0020,   /* U+18A7 */
  0x0020,   /* U+18A8 */
  0x0020,   /* U+18A9 */
  0x0020,   /* U+18AA */
  0x0020,   /* U+18AB */
  0x0020,   /* U+18AC */
  0x0020,   /* U+18AD */
  0x0020,   /* U+18AE */
  0x0020,   /* U+18AF */
  0x0020,   /* U+18B0 */
  0x0020,   /* U+18B1 */
  0x0020,   /* U+18B2 */
  0x0020,   /* U+18B3 */
  0x0020,   /* U+18B4 */
  0x0020,   /* U+18B5 */
  0x0020,   /* U+18B6 */
  0x0020,   /* U+18B7 */
  0x0020,   /* U+18B8 */
  0x0020,   /* U+18B9 */
  0x0020,   /* U+18BA */
  0x0020,   /* U+18BB */
  0x0020,   /* U+18BC */
  0x0020,   /* U+18BD */
  0x0020,   /* U+18BE */
  0x0020,   /* U+18BF */
  0x0020,   /* U+18C0 */
  0x0020,   /* U+18C1 */
  0x0020,   /* U+18C2 */
  0x0020,   /* U+18C3 */
  0x0020,   /* U+18C4 */
  0x0020,   /* U+18C5 */
  0x0020,   /* U+18C6 */
  0x0020,   /* U+18C7 */
  0x0020,   /* U+18C8 */
  0x0020,   /* U+18C9 */
  0x0020,   /* U+18CA */
  0x0020,   /* U+18CB */
  0x0020,   /* U+18CC */
  0x0020,   /* U+18CD */
  0x0020,   /* U+18CE */
  0x0020,   /* U+18CF */
  0x0020,   /* U+18D0 */
  0x0020,   /* U+18D1 */
  0x0020,   /* U+18D2 */
  0x0020,   /* U+18D3 */
  0x0020,   /* U+18D4 */
  0x0020,   /* U+18D5 */
  0x0020,   /* U+18D6 */
  0x0020,   /* U+18D7 */
  0x0020,   /* U+18D8 */
  0x0020,   /* U+18D9 */
  0x0020,   /* U+18DA */
  0x0020,   /* U+18DB */
  0x0020,   /* U+18DC */
  0x0020,   /* U+18DD */
  0x0020,   /* U+18DE */
  0x0020,   /* U+18DF */
  0x0020,   /* U+18E0 */
  0x0020,   /* U+18E1 */
  0x0020,   /* U+18E2 */
  0x0020,   /* U+18E3 */
  0x0020,   /* U+18E4 */
  0x0020,   /* U+18E5 */
  0x0020,   /* U+18E6 */
  0x0020,   /* U+18E7 */
  0x0020,   /* U+18E8 */
  0x0020,   /* U+18E9 */
  0x0020,   /* U+18EA */
  0x0020,   /* U+18EB */
  0x0020,   /* U+18EC */
  0x0020,   /* U+18ED */
  0x0020,   /* U+18EE */
  0x0020,   /* U+18EF */
  0x0020,   /* U+18F0 */
  0x0020,   /* U+18F1 */
  0x0020,   /* U+18F2 */
  0x0020,   /* U+18F3 */
  0x0020,   /* U+18F4 */
  0x0020,   /* U+18F5 */
  0x0020,   /* U+18F6 */
  0x0020,   /* U+18F7 */
  0x0020,   /* U+18F8 */
  0x0020,   /* U+18F9 */
  0x0020,   /* U+18FA */
  0x0020,   /* U+18FB */
  0x0020,   /* U+18FC */
  0x0020,   /* U+18FD */
  0x0020,   /* U+18FE */
  0x0020,   /* U+18FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1800 */
  0x0002,   /* U+1801 */
  0x0002,   /* U+1802 */
  0x0002,   /* U+1803 */
  0x0002,   /* U+1804 */
  0x0002,   /* U+1805 */
  0x0002,   /* U+1806 */
  0x0002,   /* U+1807 */
  0x0002,   /* U+1808 */
  0x0002,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0x0002,   /* U+180F */
  0x0002,   /* U+1810 */
  0x0002,   /* U+1811 */
  0x0002,   /* U+1812 */
  0x0002,   /* U+1813 */
  0x0002,   /* U+1814 */
  0x0002,   /* U+1815 */
  0x0002,   /* U+1816 */
  0x0002,   /* U+1817 */
  0x0002,   /* U+1818 */
  0x0002,   /* U+1819 */
  0x0002,   /* U+181A */
  0x0002,   /* U+181B */
  0x0002,   /* U+181C */
  0x0002,   /* U+181D */
  0x0002,   /* U+181E */
  0x0002,   /* U+181F */
  0x0002,   /* U+1820 */
  0x0002,   /* U+1821 */
  0x0002,   /* U+1822 */
  0x0002,   /* U+1823 */
  0x0002,   /* U+1824 */
  0x0002,   /* U+1825 */
  0x0002,   /* U+1826 */
  0x0002,   /* U+1827 */
  0x0002,   /* U+1828 */
  0x0002,   /* U+1829 */
  0x0002,   /* U+182A */
  0x0002,   /* U+182B */
  0x0002,   /* U+182C */
  0x0002,   /* U+182D */
  0x0002,   /* U+182E */
  0x0002,   /* U+182F */
  0x0002,   /* U+1830 */
  0x0002,   /* U+1831 */
  0x0002,   /* U+1832 */
  0x0002,   /* U+1833 */
  0x0002,   /* U+1834 */
  0x0002,   /* U+1835 */
  0x0002,   /* U+1836 */
  0x0002,   /* U+1837 */
  0x0002,   /* U+1838 */
  0x0002,   /* U+1839 */
  0x0002,   /* U+183A */
  0x0002,   /* U+183B */
  0x0002,   /* U+183C */
  0x0002,   /* U+183D */
  0x0002,   /* U+183E */
  0x0002,   /* U+183F */
  0x0002,   /* U+1840 */
  0x0002,   /* U+1841 */
  0x0002,   /* U+1842 */
  0x0002,   /* U+1843 */
  0x0002,   /* U+1844 */
  0x0002,   /* U+1845 */
  0x0002,   /* U+1846 */
  0x0002,   /* U+1847 */
  0x0002,   /* U+1848 */
  0x0002,   /* U+1849 */
  0x0002,   /* U+184A */
  0x0002,   /* U+184B */
  0x0002,   /* U+184C */
  0x0002,   /* U+184D */
  0x0002,   /* U+184E */
  0x0002,   /* U+184F */
  0x0002,   /* U+1850 */
  0x0002,   /* U+1851 */
  0x0002,   /* U+1852 */
  0x0002,   /* U+1853 */
  0x0002,   /* U+1854 */
  0x0002,   /* U+1855 */
  0x0002,   /* U+1856 */
  0x0002,   /* U+1857 */
  0x0002,   /* U+1858 */
  0x0002,   /* U+1859 */
  0x0002,   /* U+185A */
  0x0002,   /* U+185B */
  0x0002,   /* U+185C */
  0x0002,   /* U+185D */
  0x0002,   /* U+185E */
  0x0002,   /* U+185F */
  0x0002,   /* U+1860 */
  0x0002,   /* U+1861 */
  0x0002,   /* U+1862 */
  0x0002,   /* U+1863 */
  0x0002,   /* U+1864 */
  0x0002,   /* U+1865 */
  0x0002,   /* U+1866 */
  0x0002,   /* U+1867 */
  0x0002,   /* U+1868 */
  0x0002,   /* U+1869 */
  0x0002,   /* U+186A */
  0x0002,   /* U+186B */
  0x0002,   /* U+186C */
  0x0002,   /* U+186D */
  0x0002,   /* U+186E */
  0x0002,   /* U+186F */
  0x0002,   /* U+1870 */
  0x0002,   /* U+1871 */
  0x0002,   /* U+1872 */
  0x0002,   /* U+1873 */
  0x0002,   /* U+1874 */
  0x0002,   /* U+1875 */
  0x0002,   /* U+1876 */
  0x0002,   /* U+1877 */
  0x0002,   /* U+1878 */
  0x0002,   /* U+1879 */
  0x0002,   /* U+187A */
  0x0002,   /* U+187B */
  0x0002,   /* U+187C */
  0x0002,   /* U+187D */
  0x0002,   /* U+187E */
  0x0002,   /* U+187F */
  0x0002,   /* U+1880 */
  0x0002,   /* U+1881 */
  0x0002,   /* U+1882 */
  0x0002,   /* U+1883 */
  0x0002,   /* U+1884 */
  0x0002,   /* U+1885 */
  0x0002,   /* U+1886 */
  0x0002,   /* U+1887 */
  0x0002,   /* U+1888 */
  0x0002,   /* U+1889 */
  0x0002,   /* U+188A */
  0x0002,   /* U+188B */
  0x0002,   /* U+188C */
  0x0002,   /* U+188D */
  0x0002,   /* U+188E */
  0x0002,   /* U+188F */
  0x0002,   /* U+1890 */
  0x0002,   /* U+1891 */
  0x0002,   /* U+1892 */
  0x0002,   /* U+1893 */
  0x0002,   /* U+1894 */
  0x0002,   /* U+1895 */
  0x0002,   /* U+1896 */
  0x0002,   /* U+1897 */
  0x0002,   /* U+1898 */
  0x0002,   /* U+1899 */
  0x0002,   /* U+189A */
  0x0002,   /* U+189B */
  0x0002,   /* U+189C */
  0x0002,   /* U+189D */
  0x0002,   /* U+189E */
  0x0002,   /* U+189F */
  0x0002,   /* U+18A0 */
  0x0002,   /* U+18A1 */
  0x0002,   /* U+18A2 */
  0x0002,   /* U+18A3 */
  0x0002,   /* U+18A4 */
  0x0002,   /* U+18A5 */
  0x0002,   /* U+18A6 */
  0x0002,   /* U+18A7 */
  0x0002,   /* U+18A8 */
  0x0002,   /* U+18A9 */
  0x0002,   /* U+18AA */
  0x0002,   /* U+18AB */
  0x0002,   /* U+18AC */
  0x0002,   /* U+18AD */
  0x0002,   /* U+18AE */
  0x0002,   /* U+18AF */
  0x0002,   /* U+18B0 */
  0x0002,   /* U+18B1 */
  0x0002,   /* U+18B2 */
  0x0002,   /* U+18B3 */
  0x0002,   /* U+18B4 */
  0x0002,   /* U+18B5 */
  0x0002,   /* U+18B6 */
  0x0002,   /* U+18B7 */
  0x0002,   /* U+18B8 */
  0x0002,   /* U+18B9 */
  0x0002,   /* U+18BA */
  0x0002,   /* U+18BB */
  0x0002,   /* U+18BC */
  0x0002,   /* U+18BD */
  0x0002,   /* U+18BE */
  0x0002,   /* U+18BF */
  0x0002,   /* U+18C0 */
  0x0002,   /* U+18C1 */
  0x0002,   /* U+18C2 */
  0x0002,   /* U+18C3 */
  0x0002,   /* U+18C4 */
  0x0002,   /* U+18C5 */
  0x0002,   /* U+18C6 */
  0x0002,   /* U+18C7 */
  0x0002,   /* U+18C8 */
  0x0002,   /* U+18C9 */
  0x0002,   /* U+18CA */
  0x0002,   /* U+18CB */
  0x0002,   /* U+18CC */
  0x0002,   /* U+18CD */
  0x0002,   /* U+18CE */
  0x0002,   /* U+18CF */
  0x0002,   /* U+18D0 */
  0x0002,   /* U+18D1 */
  0x0002,   /* U+18D2 */
  0x0002,   /* U+18D3 */
  0x0002,   /* U+18D4 */
  0x0002,   /* U+18D5 */
  0x0002,   /* U+18D6 */
  0x0002,   /* U+18D7 */
  0x0002,   /* U+18D8 */
  0x0002,   /* U+18D9 */
  0x0002,   /* U+18DA */
  0x0002,   /* U+18DB */
  0x0002,   /* U+18DC */
  0x0002,   /* U+18DD */
  0x0002,   /* U+18DE */
  0x0002,   /* U+18DF */
  0x0002,   /* U+18E0 */
  0x0002,   /* U+18E1 */
  0x0002,   /* U+18E2 */
  0x0002,   /* U+18E3 */
  0x0002,   /* U+18E4 */
  0x0002,   /* U+18E5 */
  0x0002,   /* U+18E6 */
  0x0002,   /* U+18E7 */
  0x0002,   /* U+18E8 */
  0x0002,   /* U+18E9 */
  0x0002,   /* U+18EA */
  0x0002,   /* U+18EB */
  0x0002,   /* U+18EC */
  0x0002,   /* U+18ED */
  0x0002,   /* U+18EE */
  0x0002,   /* U+18EF */
  0x0002,   /* U+18F0 */
  0x0002,   /* U+18F1 */
  0x0002,   /* U+18F2 */
  0x0002,   /* U+18F3 */
  0x0002,   /* U+18F4 */
  0x0002,   /* U+18F5 */
  0x0002,   /* U+18F6 */
  0x0002,   /* U+18F7 */
  0x0002,   /* U+18F8 */
  0x0002,   /* U+18F9 */
  0x0002,   /* U+18FA */
  0x0002,   /* U+18FB */
  0x0002,   /* U+18FC */
  0x0002,   /* U+18FD */
  0x0002,   /* U+18FE */
  0x0002,   /* U+18FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1800 */
  0x0000,   /* U+1801 */
  0x0000,   /* U+1802 */
  0x0000,   /* U+1803 */
  0x0000,   /* U+1804 */
  0x0000,   /* U+1805 */
  0x0000,   /* U+1806 */
  0x0000,   /* U+1807 */
  0x0000,   /* U+1808 */
  0x0000,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0x980F,   /* U+180F */
  0x0000,   /* U+1810 */
  0x0000,   /* U+1811 */
  0x0000,   /* U+1812 */
  0x0000,   /* U+1813 */
  0x0000,   /* U+1814 */
  0x0000,   /* U+1815 */
  0x0000,   /* U+1816 */
  0x0000,   /* U+1817 */
  0x0000,   /* U+1818 */
  0x0000,   /* U+1819 */
  0x981A,   /* U+181A */
  0x981B,   /* U+181B */
  0x981C,   /* U+181C */
  0x981D,   /* U+181D */
  0x981E,   /* U+181E */
  0x981F,   /* U+181F */
  0x0000,   /* U+1820 */
  0x0000,   /* U+1821 */
  0x0000,   /* U+1822 */
  0x0000,   /* U+1823 */
  0x0000,   /* U+1824 */
  0x0000,   /* U+1825 */
  0x0000,   /* U+1826 */
  0x0000,   /* U+1827 */
  0x0000,   /* U+1828 */
  0x0000,   /* U+1829 */
  0x0000,   /* U+182A */
  0x0000,   /* U+182B */
  0x0000,   /* U+182C */
  0x0000,   /* U+182D */
  0x0000,   /* U+182E */
  0x0000,   /* U+182F */
  0x0000,   /* U+1830 */
  0x0000,   /* U+1831 */
  0x0000,   /* U+1832 */
  0x0000,   /* U+1833 */
  0x0000,   /* U+1834 */
  0x0000,   /* U+1835 */
  0x0000,   /* U+1836 */
  0x0000,   /* U+1837 */
  0x0000,   /* U+1838 */
  0x0000,   /* U+1839 */
  0x0000,   /* U+183A */
  0x0000,   /* U+183B */
  0x0000,   /* U+183C */
  0x0000,   /* U+183D */
  0x0000,   /* U+183E */
  0x0000,   /* U+183F */
  0x0000,   /* U+1840 */
  0x0000,   /* U+1841 */
  0x0000,   /* U+1842 */
  0x0000,   /* U+1843 */
  0x0000,   /* U+1844 */
  0x0000,   /* U+1845 */
  0x0000,   /* U+1846 */
  0x0000,   /* U+1847 */
  0x0000,   /* U+1848 */
  0x0000,   /* U+1849 */
  0x0000,   /* U+184A */
  0x0000,   /* U+184B */
  0x0000,   /* U+184C */
  0x0000,   /* U+184D */
  0x0000,   /* U+184E */
  0x0000,   /* U+184F */
  0x0000,   /* U+1850 */
  0x0000,   /* U+1851 */
  0x0000,   /* U+1852 */
  0x0000,   /* U+1853 */
  0x0000,   /* U+1854 */
  0x0000,   /* U+1855 */
  0x0000,   /* U+1856 */
  0x0000,   /* U+1857 */
  0x0000,   /* U+1858 */
  0x0000,   /* U+1859 */
  0x0000,   /* U+185A */
  0x0000,   /* U+185B */
  0x0000,   /* U+185C */
  0x0000,   /* U+185D */
  0x0000,   /* U+185E */
  0x0000,   /* U+185F */
  0x0000,   /* U+1860 */
  0x0000,   /* U+1861 */
  0x0000,   /* U+1862 */
  0x0000,   /* U+1863 */
  0x0000,   /* U+1864 */
  0x0000,   /* U+1865 */
  0x0000,   /* U+1866 */
  0x0000,   /* U+1867 */
  0x0000,   /* U+1868 */
  0x0000,   /* U+1869 */
  0x0000,   /* U+186A */
  0x0000,   /* U+186B */
  0x0000,   /* U+186C */
  0x0000,   /* U+186D */
  0x0000,   /* U+186E */
  0x0000,   /* U+186F */
  0x0000,   /* U+1870 */
  0x0000,   /* U+1871 */
  0x0000,   /* U+1872 */
  0x0000,   /* U+1873 */
  0x0000,   /* U+1874 */
  0x0000,   /* U+1875 */
  0x0000,   /* U+1876 */
  0x0000,   /* U+1877 */
  0x9878,   /* U+1878 */
  0x9879,   /* U+1879 */
  0x987A,   /* U+187A */
  0x987B,   /* U+187B */
  0x987C,   /* U+187C */
  0x987D,   /* U+187D */
  0x987E,   /* U+187E */
  0x987F,   /* U+187F */
  0x0000,   /* U+1880 */
  0x0000,   /* U+1881 */
  0x0000,   /* U+1882 */
  0x0000,   /* U+1883 */
  0x0000,   /* U+1884 */
  0x0000,   /* U+1885 */
  0x0000,   /* U+1886 */
  0x0000,   /* U+1887 */
  0x0000,   /* U+1888 */
  0x0000,   /* U+1889 */
  0x0000,   /* U+188A */
  0x0000,   /* U+188B */
  0x0000,   /* U+188C */
  0x0000,   /* U+188D */
  0x0000,   /* U+188E */
  0x0000,   /* U+188F */
  0x0000,   /* U+1890 */
  0x0000,   /* U+1891 */
  0x0000,   /* U+1892 */
  0x0000,   /* U+1893 */
  0x0000,   /* U+1894 */
  0x0000,   /* U+1895 */
  0x0000,   /* U+1896 */
  0x0000,   /* U+1897 */
  0x0000,   /* U+1898 */
  0x0000,   /* U+1899 */
  0x0000,   /* U+189A */
  0x0000,   /* U+189B */
  0x0000,   /* U+189C */
  0x0000,   /* U+189D */
  0x0000,   /* U+189E */
  0x0000,   /* U+189F */
  0x0000,   /* U+18A0 */
  0x0000,   /* U+18A1 */
  0x0000,   /* U+18A2 */
  0x0000,   /* U+18A3 */
  0x0000,   /* U+18A4 */
  0x0000,   /* U+18A5 */
  0x0000,   /* U+18A6 */
  0x0000,   /* U+18A7 */
  0x0000,   /* U+18A8 */
  0x0000,   /* U+18A9 */
  0x0000,   /* U+18AA */
  0x98AB,   /* U+18AB */
  0x98AC,   /* U+18AC */
  0x98AD,   /* U+18AD */
  0x98AE,   /* U+18AE */
  0x98AF,   /* U+18AF */
  0x0000,   /* U+18B0 */
  0x0000,   /* U+18B1 */
  0x0000,   /* U+18B2 */
  0x0000,   /* U+18B3 */
  0x0000,   /* U+18B4 */
  0x0000,   /* U+18B5 */
  0x0000,   /* U+18B6 */
  0x0000,   /* U+18B7 */
  0x0000,   /* U+18B8 */
  0x0000,   /* U+18B9 */
  0x0000,   /* U+18BA */
  0x0000,   /* U+18BB */
  0x0000,   /* U+18BC */
  0x0000,   /* U+18BD */
  0x0000,   /* U+18BE */
  0x0000,   /* U+18BF */
  0x0000,   /* U+18C0 */
  0x0000,   /* U+18C1 */
  0x0000,   /* U+18C2 */
  0x0000,   /* U+18C3 */
  0x0000,   /* U+18C4 */
  0x0000,   /* U+18C5 */
  0x0000,   /* U+18C6 */
  0x0000,   /* U+18C7 */
  0x0000,   /* U+18C8 */
  0x0000,   /* U+18C9 */
  0x0000,   /* U+18CA */
  0x0000,   /* U+18CB */
  0x0000,   /* U+18CC */
  0x0000,   /* U+18CD */
  0x0000,   /* U+18CE */
  0x0000,   /* U+18CF */
  0x0000,   /* U+18D0 */
  0x0000,   /* U+18D1 */
  0x0000,   /* U+18D2 */
  0x0000,   /* U+18D3 */
  0x0000,   /* U+18D4 */
  0x0000,   /* U+18D5 */
  0x0000,   /* U+18D6 */
  0x0000,   /* U+18D7 */
  0x0000,   /* U+18D8 */
  0x0000,   /* U+18D9 */
  0x0000,   /* U+18DA */
  0x0000,   /* U+18DB */
  0x0000,   /* U+18DC */
  0x0000,   /* U+18DD */
  0x0000,   /* U+18DE */
  0x0000,   /* U+18DF */
  0x0000,   /* U+18E0 */
  0x0000,   /* U+18E1 */
  0x0000,   /* U+18E2 */
  0x0000,   /* U+18E3 */
  0x0000,   /* U+18E4 */
  0x0000,   /* U+18E5 */
  0x0000,   /* U+18E6 */
  0x0000,   /* U+18E7 */
  0x0000,   /* U+18E8 */
  0x0000,   /* U+18E9 */
  0x0000,   /* U+18EA */
  0x0000,   /* U+18EB */
  0x0000,   /* U+18EC */
  0x0000,   /* U+18ED */
  0x0000,   /* U+18EE */
  0x0000,   /* U+18EF */
  0x0000,   /* U+18F0 */
  0x0000,   /* U+18F1 */
  0x0000,   /* U+18F2 */
  0x0000,   /* U+18F3 */
  0x0000,   /* U+18F4 */
  0x0000,   /* U+18F5 */
  0x98F6,   /* U+18F6 */
  0x98F7,   /* U+18F7 */
  0x98F8,   /* U+18F8 */
  0x98F9,   /* U+18F9 */
  0x98FA,   /* U+18FA */
  0x98FB,   /* U+18FB */
  0x98FC,   /* U+18FC */
  0x98FD,   /* U+18FD */
  0x98FE,   /* U+18FE */
  0x98FF,   /* U+18FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1800 */
  0x0000,   /* U+1801 */
  0x0000,   /* U+1802 */
  0x0000,   /* U+1803 */
  0x0000,   /* U+1804 */
  0x0000,   /* U+1805 */
  0x0000,   /* U+1806 */
  0x0000,   /* U+1807 */
  0x0000,   /* U+1808 */
  0x0000,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0x0000,   /* U+180F */
  0x0000,   /* U+1810 */
  0x0000,   /* U+1811 */
  0x0000,   /* U+1812 */
  0x0000,   /* U+1813 */
  0x0000,   /* U+1814 */
  0x0000,   /* U+1815 */
  0x0000,   /* U+1816 */
  0x0000,   /* U+1817 */
  0x0000,   /* U+1818 */
  0x0000,   /* U+1819 */
  0x0000,   /* U+181A */
  0x0000,   /* U+181B */
  0x0000,   /* U+181C */
  0x0000,   /* U+181D */
  0x0000,   /* U+181E */
  0x0000,   /* U+181F */
  0x0000,   /* U+1820 */
  0x0000,   /* U+1821 */
  0x0000,   /* U+1822 */
  0x0000,   /* U+1823 */
  0x0000,   /* U+1824 */
  0x0000,   /* U+1825 */
  0x0000,   /* U+1826 */
  0x0000,   /* U+1827 */
  0x0000,   /* U+1828 */
  0x0000,   /* U+1829 */
  0x0000,   /* U+182A */
  0x0000,   /* U+182B */
  0x0000,   /* U+182C */
  0x0000,   /* U+182D */
  0x0000,   /* U+182E */
  0x0000,   /* U+182F */
  0x0000,   /* U+1830 */
  0x0000,   /* U+1831 */
  0x0000,   /* U+1832 */
  0x0000,   /* U+1833 */
  0x0000,   /* U+1834 */
  0x0000,   /* U+1835 */
  0x0000,   /* U+1836 */
  0x0000,   /* U+1837 */
  0x0000,   /* U+1838 */
  0x0000,   /* U+1839 */
  0x0000,   /* U+183A */
  0x0000,   /* U+183B */
  0x0000,   /* U+183C */
  0x0000,   /* U+183D */
  0x0000,   /* U+183E */
  0x0000,   /* U+183F */
  0x0000,   /* U+1840 */
  0x0000,   /* U+1841 */
  0x0000,   /* U+1842 */
  0x0000,   /* U+1843 */
  0x0000,   /* U+1844 */
  0x0000,   /* U+1845 */
  0x0000,   /* U+1846 */
  0x0000,   /* U+1847 */
  0x0000,   /* U+1848 */
  0x0000,   /* U+1849 */
  0x0000,   /* U+184A */
  0x0000,   /* U+184B */
  0x0000,   /* U+184C */
  0x0000,   /* U+184D */
  0x0000,   /* U+184E */
  0x0000,   /* U+184F */
  0x0000,   /* U+1850 */
  0x0000,   /* U+1851 */
  0x0000,   /* U+1852 */
  0x0000,   /* U+1853 */
  0x0000,   /* U+1854 */
  0x0000,   /* U+1855 */
  0x0000,   /* U+1856 */
  0x0000,   /* U+1857 */
  0x0000,   /* U+1858 */
  0x0000,   /* U+1859 */
  0x0000,   /* U+185A */
  0x0000,   /* U+185B */
  0x0000,   /* U+185C */
  0x0000,   /* U+185D */
  0x0000,   /* U+185E */
  0x0000,   /* U+185F */
  0x0000,   /* U+1860 */
  0x0000,   /* U+1861 */
  0x0000,   /* U+1862 */
  0x0000,   /* U+1863 */
  0x0000,   /* U+1864 */
  0x0000,   /* U+1865 */
  0x0000,   /* U+1866 */
  0x0000,   /* U+1867 */
  0x0000,   /* U+1868 */
  0x0000,   /* U+1869 */
  0x0000,   /* U+186A */
  0x0000,   /* U+186B */
  0x0000,   /* U+186C */
  0x0000,   /* U+186D */
  0x0000,   /* U+186E */
  0x0000,   /* U+186F */
  0x0000,   /* U+1870 */
  0x0000,   /* U+1871 */
  0x0000,   /* U+1872 */
  0x0000,   /* U+1873 */
  0x0000,   /* U+1874 */
  0x0000,   /* U+1875 */
  0x0000,   /* U+1876 */
  0x0000,   /* U+1877 */
  0x0000,   /* U+1878 */
  0x0000,   /* U+1879 */
  0x0000,   /* U+187A */
  0x0000,   /* U+187B */
  0x0000,   /* U+187C */
  0x0000,   /* U+187D */
  0x0000,   /* U+187E */
  0x0000,   /* U+187F */
  0x0000,   /* U+1880 */
  0x0000,   /* U+1881 */
  0x0000,   /* U+1882 */
  0x0000,   /* U+1883 */
  0x0000,   /* U+1884 */
  0x0000,   /* U+1885 */
  0x0000,   /* U+1886 */
  0x0000,   /* U+1887 */
  0x0000,   /* U+1888 */
  0x0000,   /* U+1889 */
  0x0000,   /* U+188A */
  0x0000,   /* U+188B */
  0x0000,   /* U+188C */
  0x0000,   /* U+188D */
  0x0000,   /* U+188E */
  0x0000,   /* U+188F */
  0x0000,   /* U+1890 */
  0x0000,   /* U+1891 */
  0x0000,   /* U+1892 */
  0x0000,   /* U+1893 */
  0x0000,   /* U+1894 */
  0x0000,   /* U+1895 */
  0x0000,   /* U+1896 */
  0x0000,   /* U+1897 */
  0x0000,   /* U+1898 */
  0x0000,   /* U+1899 */
  0x0000,   /* U+189A */
  0x0000,   /* U+189B */
  0x0000,   /* U+189C */
  0x0000,   /* U+189D */
  0x0000,   /* U+189E */
  0x0000,   /* U+189F */
  0x0000,   /* U+18A0 */
  0x0000,   /* U+18A1 */
  0x0000,   /* U+18A2 */
  0x0000,   /* U+18A3 */
  0x0000,   /* U+18A4 */
  0x0000,   /* U+18A5 */
  0x0000,   /* U+18A6 */
  0x0000,   /* U+18A7 */
  0x0000,   /* U+18A8 */
  0x0000,   /* U+18A9 */
  0x0000,   /* U+18AA */
  0x0000,   /* U+18AB */
  0x0000,   /* U+18AC */
  0x0000,   /* U+18AD */
  0x0000,   /* U+18AE */
  0x0000,   /* U+18AF */
  0x0000,   /* U+18B0 */
  0x0000,   /* U+18B1 */
  0x0000,   /* U+18B2 */
  0x0000,   /* U+18B3 */
  0x0000,   /* U+18B4 */
  0x0000,   /* U+18B5 */
  0x0000,   /* U+18B6 */
  0x0000,   /* U+18B7 */
  0x0000,   /* U+18B8 */
  0x0000,   /* U+18B9 */
  0x0000,   /* U+18BA */
  0x0000,   /* U+18BB */
  0x0000,   /* U+18BC */
  0x0000,   /* U+18BD */
  0x0000,   /* U+18BE */
  0x0000,   /* U+18BF */
  0x0000,   /* U+18C0 */
  0x0000,   /* U+18C1 */
  0x0000,   /* U+18C2 */
  0x0000,   /* U+18C3 */
  0x0000,   /* U+18C4 */
  0x0000,   /* U+18C5 */
  0x0000,   /* U+18C6 */
  0x0000,   /* U+18C7 */
  0x0000,   /* U+18C8 */
  0x0000,   /* U+18C9 */
  0x0000,   /* U+18CA */
  0x0000,   /* U+18CB */
  0x0000,   /* U+18CC */
  0x0000,   /* U+18CD */
  0x0000,   /* U+18CE */
  0x0000,   /* U+18CF */
  0x0000,   /* U+18D0 */
  0x0000,   /* U+18D1 */
  0x0000,   /* U+18D2 */
  0x0000,   /* U+18D3 */
  0x0000,   /* U+18D4 */
  0x0000,   /* U+18D5 */
  0x0000,   /* U+18D6 */
  0x0000,   /* U+18D7 */
  0x0000,   /* U+18D8 */
  0x0000,   /* U+18D9 */
  0x0000,   /* U+18DA */
  0x0000,   /* U+18DB */
  0x0000,   /* U+18DC */
  0x0000,   /* U+18DD */
  0x0000,   /* U+18DE */
  0x0000,   /* U+18DF */
  0x0000,   /* U+18E0 */
  0x0000,   /* U+18E1 */
  0x0000,   /* U+18E2 */
  0x0000,   /* U+18E3 */
  0x0000,   /* U+18E4 */
  0x0000,   /* U+18E5 */
  0x0000,   /* U+18E6 */
  0x0000,   /* U+18E7 */
  0x0000,   /* U+18E8 */
  0x0000,   /* U+18E9 */
  0x0000,   /* U+18EA */
  0x0000,   /* U+18EB */
  0x0000,   /* U+18EC */
  0x0000,   /* U+18ED */
  0x0000,   /* U+18EE */
  0x0000,   /* U+18EF */
  0x0000,   /* U+18F0 */
  0x0000,   /* U+18F1 */
  0x0000,   /* U+18F2 */
  0x0000,   /* U+18F3 */
  0x0000,   /* U+18F4 */
  0x0000,   /* U+18F5 */
  0x0000,   /* U+18F6 */
  0x0000,   /* U+18F7 */
  0x0000,   /* U+18F8 */
  0x0000,   /* U+18F9 */
  0x0000,   /* U+18FA */
  0x0000,   /* U+18FB */
  0x0000,   /* U+18FC */
  0x0000,   /* U+18FD */
  0x0000,   /* U+18FE */
  0x0000,   /* U+18FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1800 */
  0x0000,   /* U+1801 */
  0x0000,   /* U+1802 */
  0x0000,   /* U+1803 */
  0x0000,   /* U+1804 */
  0x0000,   /* U+1805 */
  0x0000,   /* U+1806 */
  0x0000,   /* U+1807 */
  0x0000,   /* U+1808 */
  0x0000,   /* U+1809 */
  0x0000,   /* U+180A */
  0x0000,   /* U+180B */
  0x0000,   /* U+180C */
  0x0000,   /* U+180D */
  0x0000,   /* U+180E */
  0x0000,   /* U+180F */
  0x0000,   /* U+1810 */
  0x0000,   /* U+1811 */
  0x0000,   /* U+1812 */
  0x0000,   /* U+1813 */
  0x0000,   /* U+1814 */
  0x0000,   /* U+1815 */
  0x0000,   /* U+1816 */
  0x0000,   /* U+1817 */
  0x0000,   /* U+1818 */
  0x0000,   /* U+1819 */
  0x0000,   /* U+181A */
  0x0000,   /* U+181B */
  0x0000,   /* U+181C */
  0x0000,   /* U+181D */
  0x0000,   /* U+181E */
  0x0000,   /* U+181F */
  0x0000,   /* U+1820 */
  0x0000,   /* U+1821 */
  0x0000,   /* U+1822 */
  0x0000,   /* U+1823 */
  0x0000,   /* U+1824 */
  0x0000,   /* U+1825 */
  0x0000,   /* U+1826 */
  0x0000,   /* U+1827 */
  0x0000,   /* U+1828 */
  0x0000,   /* U+1829 */
  0x0000,   /* U+182A */
  0x0000,   /* U+182B */
  0x0000,   /* U+182C */
  0x0000,   /* U+182D */
  0x0000,   /* U+182E */
  0x0000,   /* U+182F */
  0x0000,   /* U+1830 */
  0x0000,   /* U+1831 */
  0x0000,   /* U+1832 */
  0x0000,   /* U+1833 */
  0x0000,   /* U+1834 */
  0x0000,   /* U+1835 */
  0x0000,   /* U+1836 */
  0x0000,   /* U+1837 */
  0x0000,   /* U+1838 */
  0x0000,   /* U+1839 */
  0x0000,   /* U+183A */
  0x0000,   /* U+183B */
  0x0000,   /* U+183C */
  0x0000,   /* U+183D */
  0x0000,   /* U+183E */
  0x0000,   /* U+183F */
  0x0000,   /* U+1840 */
  0x0000,   /* U+1841 */
  0x0000,   /* U+1842 */
  0x0000,   /* U+1843 */
  0x0000,   /* U+1844 */
  0x0000,   /* U+1845 */
  0x0000,   /* U+1846 */
  0x0000,   /* U+1847 */
  0x0000,   /* U+1848 */
  0x0000,   /* U+1849 */
  0x0000,   /* U+184A */
  0x0000,   /* U+184B */
  0x0000,   /* U+184C */
  0x0000,   /* U+184D */
  0x0000,   /* U+184E */
  0x0000,   /* U+184F */
  0x0000,   /* U+1850 */
  0x0000,   /* U+1851 */
  0x0000,   /* U+1852 */
  0x0000,   /* U+1853 */
  0x0000,   /* U+1854 */
  0x0000,   /* U+1855 */
  0x0000,   /* U+1856 */
  0x0000,   /* U+1857 */
  0x0000,   /* U+1858 */
  0x0000,   /* U+1859 */
  0x0000,   /* U+185A */
  0x0000,   /* U+185B */
  0x0000,   /* U+185C */
  0x0000,   /* U+185D */
  0x0000,   /* U+185E */
  0x0000,   /* U+185F */
  0x0000,   /* U+1860 */
  0x0000,   /* U+1861 */
  0x0000,   /* U+1862 */
  0x0000,   /* U+1863 */
  0x0000,   /* U+1864 */
  0x0000,   /* U+1865 */
  0x0000,   /* U+1866 */
  0x0000,   /* U+1867 */
  0x0000,   /* U+1868 */
  0x0000,   /* U+1869 */
  0x0000,   /* U+186A */
  0x0000,   /* U+186B */
  0x0000,   /* U+186C */
  0x0000,   /* U+186D */
  0x0000,   /* U+186E */
  0x0000,   /* U+186F */
  0x0000,   /* U+1870 */
  0x0000,   /* U+1871 */
  0x0000,   /* U+1872 */
  0x0000,   /* U+1873 */
  0x0000,   /* U+1874 */
  0x0000,   /* U+1875 */
  0x0000,   /* U+1876 */
  0x0000,   /* U+1877 */
  0x0000,   /* U+1878 */
  0x0000,   /* U+1879 */
  0x0000,   /* U+187A */
  0x0000,   /* U+187B */
  0x0000,   /* U+187C */
  0x0000,   /* U+187D */
  0x0000,   /* U+187E */
  0x0000,   /* U+187F */
  0x0000,   /* U+1880 */
  0x0000,   /* U+1881 */
  0x0000,   /* U+1882 */
  0x0000,   /* U+1883 */
  0x0000,   /* U+1884 */
  0x0000,   /* U+1885 */
  0x0000,   /* U+1886 */
  0x0000,   /* U+1887 */
  0x0000,   /* U+1888 */
  0x0000,   /* U+1889 */
  0x0000,   /* U+188A */
  0x0000,   /* U+188B */
  0x0000,   /* U+188C */
  0x0000,   /* U+188D */
  0x0000,   /* U+188E */
  0x0000,   /* U+188F */
  0x0000,   /* U+1890 */
  0x0000,   /* U+1891 */
  0x0000,   /* U+1892 */
  0x0000,   /* U+1893 */
  0x0000,   /* U+1894 */
  0x0000,   /* U+1895 */
  0x0000,   /* U+1896 */
  0x0000,   /* U+1897 */
  0x0000,   /* U+1898 */
  0x0000,   /* U+1899 */
  0x0000,   /* U+189A */
  0x0000,   /* U+189B */
  0x0000,   /* U+189C */
  0x0000,   /* U+189D */
  0x0000,   /* U+189E */
  0x0000,   /* U+189F */
  0x0000,   /* U+18A0 */
  0x0000,   /* U+18A1 */
  0x0000,   /* U+18A2 */
  0x0000,   /* U+18A3 */
  0x0000,   /* U+18A4 */
  0x0000,   /* U+18A5 */
  0x0000,   /* U+18A6 */
  0x0000,   /* U+18A7 */
  0x0000,   /* U+18A8 */
  0x0000,   /* U+18A9 */
  0x0000,   /* U+18AA */
  0x0000,   /* U+18AB */
  0x0000,   /* U+18AC */
  0x0000,   /* U+18AD */
  0x0000,   /* U+18AE */
  0x0000,   /* U+18AF */
  0x0000,   /* U+18B0 */
  0x0000,   /* U+18B1 */
  0x0000,   /* U+18B2 */
  0x0000,   /* U+18B3 */
  0x0000,   /* U+18B4 */
  0x0000,   /* U+18B5 */
  0x0000,   /* U+18B6 */
  0x0000,   /* U+18B7 */
  0x0000,   /* U+18B8 */
  0x0000,   /* U+18B9 */
  0x0000,   /* U+18BA */
  0x0000,   /* U+18BB */
  0x0000,   /* U+18BC */
  0x0000,   /* U+18BD */
  0x0000,   /* U+18BE */
  0x0000,   /* U+18BF */
  0x0000,   /* U+18C0 */
  0x0000,   /* U+18C1 */
  0x0000,   /* U+18C2 */
  0x0000,   /* U+18C3 */
  0x0000,   /* U+18C4 */
  0x0000,   /* U+18C5 */
  0x0000,   /* U+18C6 */
  0x0000,   /* U+18C7 */
  0x0000,   /* U+18C8 */
  0x0000,   /* U+18C9 */
  0x0000,   /* U+18CA */
  0x0000,   /* U+18CB */
  0x0000,   /* U+18CC */
  0x0000,   /* U+18CD */
  0x0000,   /* U+18CE */
  0x0000,   /* U+18CF */
  0x0000,   /* U+18D0 */
  0x0000,   /* U+18D1 */
  0x0000,   /* U+18D2 */
  0x0000,   /* U+18D3 */
  0x0000,   /* U+18D4 */
  0x0000,   /* U+18D5 */
  0x0000,   /* U+18D6 */
  0x0000,   /* U+18D7 */
  0x0000,   /* U+18D8 */
  0x0000,   /* U+18D9 */
  0x0000,   /* U+18DA */
  0x0000,   /* U+18DB */
  0x0000,   /* U+18DC */
  0x0000,   /* U+18DD */
  0x0000,   /* U+18DE */
  0x0000,   /* U+18DF */
  0x0000,   /* U+18E0 */
  0x0000,   /* U+18E1 */
  0x0000,   /* U+18E2 */
  0x0000,   /* U+18E3 */
  0x0000,   /* U+18E4 */
  0x0000,   /* U+18E5 */
  0x0000,   /* U+18E6 */
  0x0000,   /* U+18E7 */
  0x0000,   /* U+18E8 */
  0x0000,   /* U+18E9 */
  0x0000,   /* U+18EA */
  0x0000,   /* U+18EB */
  0x0000,   /* U+18EC */
  0x0000,   /* U+18ED */
  0x0000,   /* U+18EE */
  0x0000,   /* U+18EF */
  0x0000,   /* U+18F0 */
  0x0000,   /* U+18F1 */
  0x0000,   /* U+18F2 */
  0x0000,   /* U+18F3 */
  0x0000,   /* U+18F4 */
  0x0000,   /* U+18F5 */
  0x0000,   /* U+18F6 */
  0x0000,   /* U+18F7 */
  0x0000,   /* U+18F8 */
  0x0000,   /* U+18F9 */
  0x0000,   /* U+18FA */
  0x0000,   /* U+18FB */
  0x0000,   /* U+18FC */
  0x0000,   /* U+18FD */
  0x0000,   /* U+18FE */
  0x0000,   /* U+18FF */
};

uint16 uca900_p019[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x2F34,   /* U+1900 */
  0x2F35,   /* U+1901 */
  0x2F36,   /* U+1902 */
  0x2F37,   /* U+1903 */
  0x2F38,   /* U+1904 */
  0x2F39,   /* U+1905 */
  0x2F3A,   /* U+1906 */
  0x2F3B,   /* U+1907 */
  0x2F3C,   /* U+1908 */
  0x2F3D,   /* U+1909 */
  0x2F3E,   /* U+190A */
  0x2F3F,   /* U+190B */
  0x2F40,   /* U+190C */
  0x2F41,   /* U+190D */
  0x2F42,   /* U+190E */
  0x2F43,   /* U+190F */
  0x2F44,   /* U+1910 */
  0x2F45,   /* U+1911 */
  0x2F46,   /* U+1912 */
  0x2F47,   /* U+1913 */
  0x2F48,   /* U+1914 */
  0x2F49,   /* U+1915 */
  0x2F4A,   /* U+1916 */
  0x2F4B,   /* U+1917 */
  0x2F4C,   /* U+1918 */
  0x2F4D,   /* U+1919 */
  0x2F4E,   /* U+191A */
  0x2F4F,   /* U+191B */
  0x2F50,   /* U+191C */
  0x2F3C,   /* U+191D */
  0x2F3F,   /* U+191E */
  0xFBC0,   /* U+191F */
  0x2F51,   /* U+1920 */
  0x2F52,   /* U+1921 */
  0x2F53,   /* U+1922 */
  0x2F54,   /* U+1923 */
  0x2F55,   /* U+1924 */
  0x2F56,   /* U+1925 */
  0x2F57,   /* U+1926 */
  0x2F58,   /* U+1927 */
  0x2F59,   /* U+1928 */
  0x2F5A,   /* U+1929 */
  0x2F5B,   /* U+192A */
  0x2F5C,   /* U+192B */
  0xFBC0,   /* U+192C */
  0xFBC0,   /* U+192D */
  0xFBC0,   /* U+192E */
  0xFBC0,   /* U+192F */
  0x2F5D,   /* U+1930 */
  0x2F5E,   /* U+1931 */
  0x2F5F,   /* U+1932 */
  0x2F60,   /* U+1933 */
  0x2F61,   /* U+1934 */
  0x2F62,   /* U+1935 */
  0x2F63,   /* U+1936 */
  0x2F64,   /* U+1937 */
  0x2F65,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0xFBC0,   /* U+193C */
  0xFBC0,   /* U+193D */
  0xFBC0,   /* U+193E */
  0xFBC0,   /* U+193F */
  0x054A,   /* U+1940 */
  0xFBC0,   /* U+1941 */
  0xFBC0,   /* U+1942 */
  0xFBC0,   /* U+1943 */
  0x0264,   /* U+1944 */
  0x026D,   /* U+1945 */
  0x1C3D,   /* U+1946 */
  0x1C3E,   /* U+1947 */
  0x1C3F,   /* U+1948 */
  0x1C40,   /* U+1949 */
  0x1C41,   /* U+194A */
  0x1C42,   /* U+194B */
  0x1C43,   /* U+194C */
  0x1C44,   /* U+194D */
  0x1C45,   /* U+194E */
  0x1C46,   /* U+194F */
  0x315C,   /* U+1950 */
  0x315D,   /* U+1951 */
  0x315E,   /* U+1952 */
  0x315F,   /* U+1953 */
  0x3160,   /* U+1954 */
  0x3161,   /* U+1955 */
  0x3162,   /* U+1956 */
  0x3163,   /* U+1957 */
  0x3164,   /* U+1958 */
  0x3165,   /* U+1959 */
  0x3166,   /* U+195A */
  0x3167,   /* U+195B */
  0x3168,   /* U+195C */
  0x3169,   /* U+195D */
  0x316A,   /* U+195E */
  0x316B,   /* U+195F */
  0x316C,   /* U+1960 */
  0x316D,   /* U+1961 */
  0x316E,   /* U+1962 */
  0x316F,   /* U+1963 */
  0x3170,   /* U+1964 */
  0x3171,   /* U+1965 */
  0x3172,   /* U+1966 */
  0x3173,   /* U+1967 */
  0x3174,   /* U+1968 */
  0x3175,   /* U+1969 */
  0x3176,   /* U+196A */
  0x3177,   /* U+196B */
  0x3178,   /* U+196C */
  0x3179,   /* U+196D */
  0xFBC0,   /* U+196E */
  0xFBC0,   /* U+196F */
  0x317A,   /* U+1970 */
  0x317B,   /* U+1971 */
  0x317C,   /* U+1972 */
  0x317D,   /* U+1973 */
  0x317E,   /* U+1974 */
  0xFBC0,   /* U+1975 */
  0xFBC0,   /* U+1976 */
  0xFBC0,   /* U+1977 */
  0xFBC0,   /* U+1978 */
  0xFBC0,   /* U+1979 */
  0xFBC0,   /* U+197A */
  0xFBC0,   /* U+197B */
  0xFBC0,   /* U+197C */
  0xFBC0,   /* U+197D */
  0xFBC0,   /* U+197E */
  0xFBC0,   /* U+197F */
  0x317F,   /* U+1980 */
  0x3180,   /* U+1981 */
  0x3181,   /* U+1982 */
  0x3182,   /* U+1983 */
  0x3183,   /* U+1984 */
  0x3184,   /* U+1985 */
  0x3185,   /* U+1986 */
  0x3186,   /* U+1987 */
  0x3187,   /* U+1988 */
  0x3188,   /* U+1989 */
  0x3189,   /* U+198A */
  0x318A,   /* U+198B */
  0x318B,   /* U+198C */
  0x318C,   /* U+198D */
  0x318D,   /* U+198E */
  0x318E,   /* U+198F */
  0x318F,   /* U+1990 */
  0x3190,   /* U+1991 */
  0x3191,   /* U+1992 */
  0x3192,   /* U+1993 */
  0x3193,   /* U+1994 */
  0x3194,   /* U+1995 */
  0x3195,   /* U+1996 */
  0x3196,   /* U+1997 */
  0x3197,   /* U+1998 */
  0x3198,   /* U+1999 */
  0x3199,   /* U+199A */
  0x319A,   /* U+199B */
  0x319B,   /* U+199C */
  0x319C,   /* U+199D */
  0x319D,   /* U+199E */
  0x319E,   /* U+199F */
  0x319F,   /* U+19A0 */
  0x31A0,   /* U+19A1 */
  0x31A1,   /* U+19A2 */
  0x31A2,   /* U+19A3 */
  0x31A3,   /* U+19A4 */
  0x31A4,   /* U+19A5 */
  0x31A5,   /* U+19A6 */
  0x31A6,   /* U+19A7 */
  0x31A7,   /* U+19A8 */
  0x31A8,   /* U+19A9 */
  0x31A9,   /* U+19AA */
  0x31AA,   /* U+19AB */
  0xFBC0,   /* U+19AC */
  0xFBC0,   /* U+19AD */
  0xFBC0,   /* U+19AE */
  0xFBC0,   /* U+19AF */
  0x31AB,   /* U+19B0 */
  0x31AC,   /* U+19B1 */
  0x31AD,   /* U+19B2 */
  0x31AE,   /* U+19B3 */
  0x31AF,   /* U+19B4 */
  0x31B0,   /* U+19B5 */
  0x31B1,   /* U+19B6 */
  0x31B2,   /* U+19B7 */
  0x31B3,   /* U+19B8 */
  0x31B4,   /* U+19B9 */
  0x31B5,   /* U+19BA */
  0x31B6,   /* U+19BB */
  0x31B7,   /* U+19BC */
  0x31B8,   /* U+19BD */
  0x31B9,   /* U+19BE */
  0x31BA,   /* U+19BF */
  0x31BB,   /* U+19C0 */
  0x31BC,   /* U+19C1 */
  0x31BD,   /* U+19C2 */
  0x31BE,   /* U+19C3 */
  0x31BF,   /* U+19C4 */
  0x31C0,   /* U+19C5 */
  0x31C1,   /* U+19C6 */
  0x31C2,   /* U+19C7 */
  0x31C3,   /* U+19C8 */
  0x31C4,   /* U+19C9 */
  0xFBC0,   /* U+19CA */
  0xFBC0,   /* U+19CB */
  0xFBC0,   /* U+19CC */
  0xFBC0,   /* U+19CD */
  0xFBC0,   /* U+19CE */
  0xFBC0,   /* U+19CF */
  0x1C3D,   /* U+19D0 */
  0x1C3E,   /* U+19D1 */
  0x1C3F,   /* U+19D2 */
  0x1C40,   /* U+19D3 */
  0x1C41,   /* U+19D4 */
  0x1C42,   /* U+19D5 */
  0x1C43,   /* U+19D6 */
  0x1C44,   /* U+19D7 */
  0x1C45,   /* U+19D8 */
  0x1C46,   /* U+19D9 */
  0x1C3E,   /* U+19DA */
  0xFBC0,   /* U+19DB */
  0xFBC0,   /* U+19DC */
  0xFBC0,   /* U+19DD */
  0x319B,   /* U+19DE */
  0x319B,   /* U+19DF */
  0x0551,   /* U+19E0 */
  0x0552,   /* U+19E1 */
  0x0553,   /* U+19E2 */
  0x0554,   /* U+19E3 */
  0x0555,   /* U+19E4 */
  0x0556,   /* U+19E5 */
  0x0557,   /* U+19E6 */
  0x0558,   /* U+19E7 */
  0x0559,   /* U+19E8 */
  0x055A,   /* U+19E9 */
  0x055B,   /* U+19EA */
  0x055C,   /* U+19EB */
  0x055D,   /* U+19EC */
  0x055E,   /* U+19ED */
  0x055F,   /* U+19EE */
  0x0560,   /* U+19EF */
  0x0561,   /* U+19F0 */
  0x0562,   /* U+19F1 */
  0x0563,   /* U+19F2 */
  0x0564,   /* U+19F3 */
  0x0565,   /* U+19F4 */
  0x0566,   /* U+19F5 */
  0x0567,   /* U+19F6 */
  0x0568,   /* U+19F7 */
  0x0569,   /* U+19F8 */
  0x056A,   /* U+19F9 */
  0x056B,   /* U+19FA */
  0x056C,   /* U+19FB */
  0x056D,   /* U+19FC */
  0x056E,   /* U+19FD */
  0x056F,   /* U+19FE */
  0x0570,   /* U+19FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1900 */
  0x0020,   /* U+1901 */
  0x0020,   /* U+1902 */
  0x0020,   /* U+1903 */
  0x0020,   /* U+1904 */
  0x0020,   /* U+1905 */
  0x0020,   /* U+1906 */
  0x0020,   /* U+1907 */
  0x0020,   /* U+1908 */
  0x0020,   /* U+1909 */
  0x0020,   /* U+190A */
  0x0020,   /* U+190B */
  0x0020,   /* U+190C */
  0x0020,   /* U+190D */
  0x0020,   /* U+190E */
  0x0020,   /* U+190F */
  0x0020,   /* U+1910 */
  0x0020,   /* U+1911 */
  0x0020,   /* U+1912 */
  0x0020,   /* U+1913 */
  0x0020,   /* U+1914 */
  0x0020,   /* U+1915 */
  0x0020,   /* U+1916 */
  0x0020,   /* U+1917 */
  0x0020,   /* U+1918 */
  0x0020,   /* U+1919 */
  0x0020,   /* U+191A */
  0x0020,   /* U+191B */
  0x0020,   /* U+191C */
  0x0020,   /* U+191D */
  0x0020,   /* U+191E */
  0x0020,   /* U+191F */
  0x0020,   /* U+1920 */
  0x0020,   /* U+1921 */
  0x0020,   /* U+1922 */
  0x0020,   /* U+1923 */
  0x0020,   /* U+1924 */
  0x0020,   /* U+1925 */
  0x0020,   /* U+1926 */
  0x0020,   /* U+1927 */
  0x0020,   /* U+1928 */
  0x0020,   /* U+1929 */
  0x0020,   /* U+192A */
  0x0020,   /* U+192B */
  0x0020,   /* U+192C */
  0x0020,   /* U+192D */
  0x0020,   /* U+192E */
  0x0020,   /* U+192F */
  0x0020,   /* U+1930 */
  0x0020,   /* U+1931 */
  0x0020,   /* U+1932 */
  0x0020,   /* U+1933 */
  0x0020,   /* U+1934 */
  0x0020,   /* U+1935 */
  0x0020,   /* U+1936 */
  0x0020,   /* U+1937 */
  0x0020,   /* U+1938 */
  0x00F1,   /* U+1939 */
  0x00F2,   /* U+193A */
  0x00F3,   /* U+193B */
  0x0020,   /* U+193C */
  0x0020,   /* U+193D */
  0x0020,   /* U+193E */
  0x0020,   /* U+193F */
  0x0020,   /* U+1940 */
  0x0020,   /* U+1941 */
  0x0020,   /* U+1942 */
  0x0020,   /* U+1943 */
  0x0020,   /* U+1944 */
  0x0020,   /* U+1945 */
  0x0020,   /* U+1946 */
  0x0020,   /* U+1947 */
  0x0020,   /* U+1948 */
  0x0020,   /* U+1949 */
  0x0020,   /* U+194A */
  0x0020,   /* U+194B */
  0x0020,   /* U+194C */
  0x0020,   /* U+194D */
  0x0020,   /* U+194E */
  0x0020,   /* U+194F */
  0x0020,   /* U+1950 */
  0x0020,   /* U+1951 */
  0x0020,   /* U+1952 */
  0x0020,   /* U+1953 */
  0x0020,   /* U+1954 */
  0x0020,   /* U+1955 */
  0x0020,   /* U+1956 */
  0x0020,   /* U+1957 */
  0x0020,   /* U+1958 */
  0x0020,   /* U+1959 */
  0x0020,   /* U+195A */
  0x0020,   /* U+195B */
  0x0020,   /* U+195C */
  0x0020,   /* U+195D */
  0x0020,   /* U+195E */
  0x0020,   /* U+195F */
  0x0020,   /* U+1960 */
  0x0020,   /* U+1961 */
  0x0020,   /* U+1962 */
  0x0020,   /* U+1963 */
  0x0020,   /* U+1964 */
  0x0020,   /* U+1965 */
  0x0020,   /* U+1966 */
  0x0020,   /* U+1967 */
  0x0020,   /* U+1968 */
  0x0020,   /* U+1969 */
  0x0020,   /* U+196A */
  0x0020,   /* U+196B */
  0x0020,   /* U+196C */
  0x0020,   /* U+196D */
  0x0020,   /* U+196E */
  0x0020,   /* U+196F */
  0x0020,   /* U+1970 */
  0x0020,   /* U+1971 */
  0x0020,   /* U+1972 */
  0x0020,   /* U+1973 */
  0x0020,   /* U+1974 */
  0x0020,   /* U+1975 */
  0x0020,   /* U+1976 */
  0x0020,   /* U+1977 */
  0x0020,   /* U+1978 */
  0x0020,   /* U+1979 */
  0x0020,   /* U+197A */
  0x0020,   /* U+197B */
  0x0020,   /* U+197C */
  0x0020,   /* U+197D */
  0x0020,   /* U+197E */
  0x0020,   /* U+197F */
  0x0020,   /* U+1980 */
  0x0020,   /* U+1981 */
  0x0020,   /* U+1982 */
  0x0020,   /* U+1983 */
  0x0020,   /* U+1984 */
  0x0020,   /* U+1985 */
  0x0020,   /* U+1986 */
  0x0020,   /* U+1987 */
  0x0020,   /* U+1988 */
  0x0020,   /* U+1989 */
  0x0020,   /* U+198A */
  0x0020,   /* U+198B */
  0x0020,   /* U+198C */
  0x0020,   /* U+198D */
  0x0020,   /* U+198E */
  0x0020,   /* U+198F */
  0x0020,   /* U+1990 */
  0x0020,   /* U+1991 */
  0x0020,   /* U+1992 */
  0x0020,   /* U+1993 */
  0x0020,   /* U+1994 */
  0x0020,   /* U+1995 */
  0x0020,   /* U+1996 */
  0x0020,   /* U+1997 */
  0x0020,   /* U+1998 */
  0x0020,   /* U+1999 */
  0x0020,   /* U+199A */
  0x0020,   /* U+199B */
  0x0020,   /* U+199C */
  0x0020,   /* U+199D */
  0x0020,   /* U+199E */
  0x0020,   /* U+199F */
  0x0020,   /* U+19A0 */
  0x0020,   /* U+19A1 */
  0x0020,   /* U+19A2 */
  0x0020,   /* U+19A3 */
  0x0020,   /* U+19A4 */
  0x0020,   /* U+19A5 */
  0x0020,   /* U+19A6 */
  0x0020,   /* U+19A7 */
  0x0020,   /* U+19A8 */
  0x0020,   /* U+19A9 */
  0x0020,   /* U+19AA */
  0x0020,   /* U+19AB */
  0x0020,   /* U+19AC */
  0x0020,   /* U+19AD */
  0x0020,   /* U+19AE */
  0x0020,   /* U+19AF */
  0x0020,   /* U+19B0 */
  0x0020,   /* U+19B1 */
  0x0020,   /* U+19B2 */
  0x0020,   /* U+19B3 */
  0x0020,   /* U+19B4 */
  0x0020,   /* U+19B5 */
  0x0020,   /* U+19B6 */
  0x0020,   /* U+19B7 */
  0x0020,   /* U+19B8 */
  0x0020,   /* U+19B9 */
  0x0020,   /* U+19BA */
  0x0020,   /* U+19BB */
  0x0020,   /* U+19BC */
  0x0020,   /* U+19BD */
  0x0020,   /* U+19BE */
  0x0020,   /* U+19BF */
  0x0020,   /* U+19C0 */
  0x0020,   /* U+19C1 */
  0x0020,   /* U+19C2 */
  0x0020,   /* U+19C3 */
  0x0020,   /* U+19C4 */
  0x0020,   /* U+19C5 */
  0x0020,   /* U+19C6 */
  0x0020,   /* U+19C7 */
  0x0020,   /* U+19C8 */
  0x0020,   /* U+19C9 */
  0x0020,   /* U+19CA */
  0x0020,   /* U+19CB */
  0x0020,   /* U+19CC */
  0x0020,   /* U+19CD */
  0x0020,   /* U+19CE */
  0x0020,   /* U+19CF */
  0x0020,   /* U+19D0 */
  0x0020,   /* U+19D1 */
  0x0020,   /* U+19D2 */
  0x0020,   /* U+19D3 */
  0x0020,   /* U+19D4 */
  0x0020,   /* U+19D5 */
  0x0020,   /* U+19D6 */
  0x0020,   /* U+19D7 */
  0x0020,   /* U+19D8 */
  0x0020,   /* U+19D9 */
  0x0020,   /* U+19DA */
  0x0020,   /* U+19DB */
  0x0020,   /* U+19DC */
  0x0020,   /* U+19DD */
  0x0020,   /* U+19DE */
  0x0020,   /* U+19DF */
  0x0020,   /* U+19E0 */
  0x0020,   /* U+19E1 */
  0x0020,   /* U+19E2 */
  0x0020,   /* U+19E3 */
  0x0020,   /* U+19E4 */
  0x0020,   /* U+19E5 */
  0x0020,   /* U+19E6 */
  0x0020,   /* U+19E7 */
  0x0020,   /* U+19E8 */
  0x0020,   /* U+19E9 */
  0x0020,   /* U+19EA */
  0x0020,   /* U+19EB */
  0x0020,   /* U+19EC */
  0x0020,   /* U+19ED */
  0x0020,   /* U+19EE */
  0x0020,   /* U+19EF */
  0x0020,   /* U+19F0 */
  0x0020,   /* U+19F1 */
  0x0020,   /* U+19F2 */
  0x0020,   /* U+19F3 */
  0x0020,   /* U+19F4 */
  0x0020,   /* U+19F5 */
  0x0020,   /* U+19F6 */
  0x0020,   /* U+19F7 */
  0x0020,   /* U+19F8 */
  0x0020,   /* U+19F9 */
  0x0020,   /* U+19FA */
  0x0020,   /* U+19FB */
  0x0020,   /* U+19FC */
  0x0020,   /* U+19FD */
  0x0020,   /* U+19FE */
  0x0020,   /* U+19FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1900 */
  0x0002,   /* U+1901 */
  0x0002,   /* U+1902 */
  0x0002,   /* U+1903 */
  0x0002,   /* U+1904 */
  0x0002,   /* U+1905 */
  0x0002,   /* U+1906 */
  0x0002,   /* U+1907 */
  0x0002,   /* U+1908 */
  0x0002,   /* U+1909 */
  0x0002,   /* U+190A */
  0x0002,   /* U+190B */
  0x0002,   /* U+190C */
  0x0002,   /* U+190D */
  0x0002,   /* U+190E */
  0x0002,   /* U+190F */
  0x0002,   /* U+1910 */
  0x0002,   /* U+1911 */
  0x0002,   /* U+1912 */
  0x0002,   /* U+1913 */
  0x0002,   /* U+1914 */
  0x0002,   /* U+1915 */
  0x0002,   /* U+1916 */
  0x0002,   /* U+1917 */
  0x0002,   /* U+1918 */
  0x0002,   /* U+1919 */
  0x0002,   /* U+191A */
  0x0002,   /* U+191B */
  0x0002,   /* U+191C */
  0x0004,   /* U+191D */
  0x0004,   /* U+191E */
  0x0002,   /* U+191F */
  0x0002,   /* U+1920 */
  0x0002,   /* U+1921 */
  0x0002,   /* U+1922 */
  0x0002,   /* U+1923 */
  0x0002,   /* U+1924 */
  0x0002,   /* U+1925 */
  0x0002,   /* U+1926 */
  0x0002,   /* U+1927 */
  0x0002,   /* U+1928 */
  0x0002,   /* U+1929 */
  0x0002,   /* U+192A */
  0x0002,   /* U+192B */
  0x0002,   /* U+192C */
  0x0002,   /* U+192D */
  0x0002,   /* U+192E */
  0x0002,   /* U+192F */
  0x0002,   /* U+1930 */
  0x0002,   /* U+1931 */
  0x0002,   /* U+1932 */
  0x0002,   /* U+1933 */
  0x0002,   /* U+1934 */
  0x0002,   /* U+1935 */
  0x0002,   /* U+1936 */
  0x0002,   /* U+1937 */
  0x0002,   /* U+1938 */
  0x0002,   /* U+1939 */
  0x0002,   /* U+193A */
  0x0002,   /* U+193B */
  0x0002,   /* U+193C */
  0x0002,   /* U+193D */
  0x0002,   /* U+193E */
  0x0002,   /* U+193F */
  0x0002,   /* U+1940 */
  0x0002,   /* U+1941 */
  0x0002,   /* U+1942 */
  0x0002,   /* U+1943 */
  0x0002,   /* U+1944 */
  0x0002,   /* U+1945 */
  0x0002,   /* U+1946 */
  0x0002,   /* U+1947 */
  0x0002,   /* U+1948 */
  0x0002,   /* U+1949 */
  0x0002,   /* U+194A */
  0x0002,   /* U+194B */
  0x0002,   /* U+194C */
  0x0002,   /* U+194D */
  0x0002,   /* U+194E */
  0x0002,   /* U+194F */
  0x0002,   /* U+1950 */
  0x0002,   /* U+1951 */
  0x0002,   /* U+1952 */
  0x0002,   /* U+1953 */
  0x0002,   /* U+1954 */
  0x0002,   /* U+1955 */
  0x0002,   /* U+1956 */
  0x0002,   /* U+1957 */
  0x0002,   /* U+1958 */
  0x0002,   /* U+1959 */
  0x0002,   /* U+195A */
  0x0002,   /* U+195B */
  0x0002,   /* U+195C */
  0x0002,   /* U+195D */
  0x0002,   /* U+195E */
  0x0002,   /* U+195F */
  0x0002,   /* U+1960 */
  0x0002,   /* U+1961 */
  0x0002,   /* U+1962 */
  0x0002,   /* U+1963 */
  0x0002,   /* U+1964 */
  0x0002,   /* U+1965 */
  0x0002,   /* U+1966 */
  0x0002,   /* U+1967 */
  0x0002,   /* U+1968 */
  0x0002,   /* U+1969 */
  0x0002,   /* U+196A */
  0x0002,   /* U+196B */
  0x0002,   /* U+196C */
  0x0002,   /* U+196D */
  0x0002,   /* U+196E */
  0x0002,   /* U+196F */
  0x0002,   /* U+1970 */
  0x0002,   /* U+1971 */
  0x0002,   /* U+1972 */
  0x0002,   /* U+1973 */
  0x0002,   /* U+1974 */
  0x0002,   /* U+1975 */
  0x0002,   /* U+1976 */
  0x0002,   /* U+1977 */
  0x0002,   /* U+1978 */
  0x0002,   /* U+1979 */
  0x0002,   /* U+197A */
  0x0002,   /* U+197B */
  0x0002,   /* U+197C */
  0x0002,   /* U+197D */
  0x0002,   /* U+197E */
  0x0002,   /* U+197F */
  0x0002,   /* U+1980 */
  0x0002,   /* U+1981 */
  0x0002,   /* U+1982 */
  0x0002,   /* U+1983 */
  0x0002,   /* U+1984 */
  0x0002,   /* U+1985 */
  0x0002,   /* U+1986 */
  0x0002,   /* U+1987 */
  0x0002,   /* U+1988 */
  0x0002,   /* U+1989 */
  0x0002,   /* U+198A */
  0x0002,   /* U+198B */
  0x0002,   /* U+198C */
  0x0002,   /* U+198D */
  0x0002,   /* U+198E */
  0x0002,   /* U+198F */
  0x0002,   /* U+1990 */
  0x0002,   /* U+1991 */
  0x0002,   /* U+1992 */
  0x0002,   /* U+1993 */
  0x0002,   /* U+1994 */
  0x0002,   /* U+1995 */
  0x0002,   /* U+1996 */
  0x0002,   /* U+1997 */
  0x0002,   /* U+1998 */
  0x0002,   /* U+1999 */
  0x0002,   /* U+199A */
  0x0002,   /* U+199B */
  0x0002,   /* U+199C */
  0x0002,   /* U+199D */
  0x0002,   /* U+199E */
  0x0002,   /* U+199F */
  0x0002,   /* U+19A0 */
  0x0002,   /* U+19A1 */
  0x0002,   /* U+19A2 */
  0x0002,   /* U+19A3 */
  0x0002,   /* U+19A4 */
  0x0002,   /* U+19A5 */
  0x0002,   /* U+19A6 */
  0x0002,   /* U+19A7 */
  0x0002,   /* U+19A8 */
  0x0002,   /* U+19A9 */
  0x0002,   /* U+19AA */
  0x0002,   /* U+19AB */
  0x0002,   /* U+19AC */
  0x0002,   /* U+19AD */
  0x0002,   /* U+19AE */
  0x0002,   /* U+19AF */
  0x0002,   /* U+19B0 */
  0x0002,   /* U+19B1 */
  0x0002,   /* U+19B2 */
  0x0002,   /* U+19B3 */
  0x0002,   /* U+19B4 */
  0x0002,   /* U+19B5 */
  0x0002,   /* U+19B6 */
  0x0002,   /* U+19B7 */
  0x0002,   /* U+19B8 */
  0x0002,   /* U+19B9 */
  0x0002,   /* U+19BA */
  0x0002,   /* U+19BB */
  0x0002,   /* U+19BC */
  0x0002,   /* U+19BD */
  0x0002,   /* U+19BE */
  0x0002,   /* U+19BF */
  0x0002,   /* U+19C0 */
  0x0002,   /* U+19C1 */
  0x0002,   /* U+19C2 */
  0x0002,   /* U+19C3 */
  0x0002,   /* U+19C4 */
  0x0002,   /* U+19C5 */
  0x0002,   /* U+19C6 */
  0x0002,   /* U+19C7 */
  0x0002,   /* U+19C8 */
  0x0002,   /* U+19C9 */
  0x0002,   /* U+19CA */
  0x0002,   /* U+19CB */
  0x0002,   /* U+19CC */
  0x0002,   /* U+19CD */
  0x0002,   /* U+19CE */
  0x0002,   /* U+19CF */
  0x0002,   /* U+19D0 */
  0x0002,   /* U+19D1 */
  0x0002,   /* U+19D2 */
  0x0002,   /* U+19D3 */
  0x0002,   /* U+19D4 */
  0x0002,   /* U+19D5 */
  0x0002,   /* U+19D6 */
  0x0002,   /* U+19D7 */
  0x0002,   /* U+19D8 */
  0x0002,   /* U+19D9 */
  0x0002,   /* U+19DA */
  0x0002,   /* U+19DB */
  0x0002,   /* U+19DC */
  0x0002,   /* U+19DD */
  0x0004,   /* U+19DE */
  0x0004,   /* U+19DF */
  0x0002,   /* U+19E0 */
  0x0002,   /* U+19E1 */
  0x0002,   /* U+19E2 */
  0x0002,   /* U+19E3 */
  0x0002,   /* U+19E4 */
  0x0002,   /* U+19E5 */
  0x0002,   /* U+19E6 */
  0x0002,   /* U+19E7 */
  0x0002,   /* U+19E8 */
  0x0002,   /* U+19E9 */
  0x0002,   /* U+19EA */
  0x0002,   /* U+19EB */
  0x0002,   /* U+19EC */
  0x0002,   /* U+19ED */
  0x0002,   /* U+19EE */
  0x0002,   /* U+19EF */
  0x0002,   /* U+19F0 */
  0x0002,   /* U+19F1 */
  0x0002,   /* U+19F2 */
  0x0002,   /* U+19F3 */
  0x0002,   /* U+19F4 */
  0x0002,   /* U+19F5 */
  0x0002,   /* U+19F6 */
  0x0002,   /* U+19F7 */
  0x0002,   /* U+19F8 */
  0x0002,   /* U+19F9 */
  0x0002,   /* U+19FA */
  0x0002,   /* U+19FB */
  0x0002,   /* U+19FC */
  0x0002,   /* U+19FD */
  0x0002,   /* U+19FE */
  0x0002,   /* U+19FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x2F5A,   /* U+191D */
  0x2F5B,   /* U+191E */
  0x991F,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x992C,   /* U+192C */
  0x992D,   /* U+192D */
  0x992E,   /* U+192E */
  0x992F,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x993C,   /* U+193C */
  0x993D,   /* U+193D */
  0x993E,   /* U+193E */
  0x993F,   /* U+193F */
  0x0000,   /* U+1940 */
  0x9941,   /* U+1941 */
  0x9942,   /* U+1942 */
  0x9943,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x996E,   /* U+196E */
  0x996F,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x9975,   /* U+1975 */
  0x9976,   /* U+1976 */
  0x9977,   /* U+1977 */
  0x9978,   /* U+1978 */
  0x9979,   /* U+1979 */
  0x997A,   /* U+197A */
  0x997B,   /* U+197B */
  0x997C,   /* U+197C */
  0x997D,   /* U+197D */
  0x997E,   /* U+197E */
  0x997F,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x99AC,   /* U+19AC */
  0x99AD,   /* U+19AD */
  0x99AE,   /* U+19AE */
  0x99AF,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x99CA,   /* U+19CA */
  0x99CB,   /* U+19CB */
  0x99CC,   /* U+19CC */
  0x99CD,   /* U+19CD */
  0x99CE,   /* U+19CE */
  0x99CF,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x99DB,   /* U+19DB */
  0x99DC,   /* U+19DC */
  0x99DD,   /* U+19DD */
  0x31B1,   /* U+19DE */
  0x31B1,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x0020,   /* U+191D */
  0x0020,   /* U+191E */
  0x0000,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x0000,   /* U+192C */
  0x0000,   /* U+192D */
  0x0000,   /* U+192E */
  0x0000,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x0000,   /* U+193C */
  0x0000,   /* U+193D */
  0x0000,   /* U+193E */
  0x0000,   /* U+193F */
  0x0000,   /* U+1940 */
  0x0000,   /* U+1941 */
  0x0000,   /* U+1942 */
  0x0000,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x0000,   /* U+196E */
  0x0000,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x0000,   /* U+1975 */
  0x0000,   /* U+1976 */
  0x0000,   /* U+1977 */
  0x0000,   /* U+1978 */
  0x0000,   /* U+1979 */
  0x0000,   /* U+197A */
  0x0000,   /* U+197B */
  0x0000,   /* U+197C */
  0x0000,   /* U+197D */
  0x0000,   /* U+197E */
  0x0000,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x0000,   /* U+19AC */
  0x0000,   /* U+19AD */
  0x0000,   /* U+19AE */
  0x0000,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x0000,   /* U+19CA */
  0x0000,   /* U+19CB */
  0x0000,   /* U+19CC */
  0x0000,   /* U+19CD */
  0x0000,   /* U+19CE */
  0x0000,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x0000,   /* U+19DB */
  0x0000,   /* U+19DC */
  0x0000,   /* U+19DD */
  0x0020,   /* U+19DE */
  0x0020,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x0004,   /* U+191D */
  0x0004,   /* U+191E */
  0x0000,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x0000,   /* U+192C */
  0x0000,   /* U+192D */
  0x0000,   /* U+192E */
  0x0000,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x0000,   /* U+193C */
  0x0000,   /* U+193D */
  0x0000,   /* U+193E */
  0x0000,   /* U+193F */
  0x0000,   /* U+1940 */
  0x0000,   /* U+1941 */
  0x0000,   /* U+1942 */
  0x0000,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x0000,   /* U+196E */
  0x0000,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x0000,   /* U+1975 */
  0x0000,   /* U+1976 */
  0x0000,   /* U+1977 */
  0x0000,   /* U+1978 */
  0x0000,   /* U+1979 */
  0x0000,   /* U+197A */
  0x0000,   /* U+197B */
  0x0000,   /* U+197C */
  0x0000,   /* U+197D */
  0x0000,   /* U+197E */
  0x0000,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x0000,   /* U+19AC */
  0x0000,   /* U+19AD */
  0x0000,   /* U+19AE */
  0x0000,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x0000,   /* U+19CA */
  0x0000,   /* U+19CB */
  0x0000,   /* U+19CC */
  0x0000,   /* U+19CD */
  0x0000,   /* U+19CE */
  0x0000,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x0000,   /* U+19DB */
  0x0000,   /* U+19DC */
  0x0000,   /* U+19DD */
  0x0004,   /* U+19DE */
  0x0004,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x0000,   /* U+191D */
  0x0000,   /* U+191E */
  0x0000,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x0000,   /* U+192C */
  0x0000,   /* U+192D */
  0x0000,   /* U+192E */
  0x0000,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x0000,   /* U+193C */
  0x0000,   /* U+193D */
  0x0000,   /* U+193E */
  0x0000,   /* U+193F */
  0x0000,   /* U+1940 */
  0x0000,   /* U+1941 */
  0x0000,   /* U+1942 */
  0x0000,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x0000,   /* U+196E */
  0x0000,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x0000,   /* U+1975 */
  0x0000,   /* U+1976 */
  0x0000,   /* U+1977 */
  0x0000,   /* U+1978 */
  0x0000,   /* U+1979 */
  0x0000,   /* U+197A */
  0x0000,   /* U+197B */
  0x0000,   /* U+197C */
  0x0000,   /* U+197D */
  0x0000,   /* U+197E */
  0x0000,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x0000,   /* U+19AC */
  0x0000,   /* U+19AD */
  0x0000,   /* U+19AE */
  0x0000,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x0000,   /* U+19CA */
  0x0000,   /* U+19CB */
  0x0000,   /* U+19CC */
  0x0000,   /* U+19CD */
  0x0000,   /* U+19CE */
  0x0000,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x0000,   /* U+19DB */
  0x0000,   /* U+19DC */
  0x0000,   /* U+19DD */
  0x0000,   /* U+19DE */
  0x31BC,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x0000,   /* U+191D */
  0x0000,   /* U+191E */
  0x0000,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x0000,   /* U+192C */
  0x0000,   /* U+192D */
  0x0000,   /* U+192E */
  0x0000,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x0000,   /* U+193C */
  0x0000,   /* U+193D */
  0x0000,   /* U+193E */
  0x0000,   /* U+193F */
  0x0000,   /* U+1940 */
  0x0000,   /* U+1941 */
  0x0000,   /* U+1942 */
  0x0000,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x0000,   /* U+196E */
  0x0000,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x0000,   /* U+1975 */
  0x0000,   /* U+1976 */
  0x0000,   /* U+1977 */
  0x0000,   /* U+1978 */
  0x0000,   /* U+1979 */
  0x0000,   /* U+197A */
  0x0000,   /* U+197B */
  0x0000,   /* U+197C */
  0x0000,   /* U+197D */
  0x0000,   /* U+197E */
  0x0000,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x0000,   /* U+19AC */
  0x0000,   /* U+19AD */
  0x0000,   /* U+19AE */
  0x0000,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x0000,   /* U+19CA */
  0x0000,   /* U+19CB */
  0x0000,   /* U+19CC */
  0x0000,   /* U+19CD */
  0x0000,   /* U+19CE */
  0x0000,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x0000,   /* U+19DB */
  0x0000,   /* U+19DC */
  0x0000,   /* U+19DD */
  0x0000,   /* U+19DE */
  0x0020,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1900 */
  0x0000,   /* U+1901 */
  0x0000,   /* U+1902 */
  0x0000,   /* U+1903 */
  0x0000,   /* U+1904 */
  0x0000,   /* U+1905 */
  0x0000,   /* U+1906 */
  0x0000,   /* U+1907 */
  0x0000,   /* U+1908 */
  0x0000,   /* U+1909 */
  0x0000,   /* U+190A */
  0x0000,   /* U+190B */
  0x0000,   /* U+190C */
  0x0000,   /* U+190D */
  0x0000,   /* U+190E */
  0x0000,   /* U+190F */
  0x0000,   /* U+1910 */
  0x0000,   /* U+1911 */
  0x0000,   /* U+1912 */
  0x0000,   /* U+1913 */
  0x0000,   /* U+1914 */
  0x0000,   /* U+1915 */
  0x0000,   /* U+1916 */
  0x0000,   /* U+1917 */
  0x0000,   /* U+1918 */
  0x0000,   /* U+1919 */
  0x0000,   /* U+191A */
  0x0000,   /* U+191B */
  0x0000,   /* U+191C */
  0x0000,   /* U+191D */
  0x0000,   /* U+191E */
  0x0000,   /* U+191F */
  0x0000,   /* U+1920 */
  0x0000,   /* U+1921 */
  0x0000,   /* U+1922 */
  0x0000,   /* U+1923 */
  0x0000,   /* U+1924 */
  0x0000,   /* U+1925 */
  0x0000,   /* U+1926 */
  0x0000,   /* U+1927 */
  0x0000,   /* U+1928 */
  0x0000,   /* U+1929 */
  0x0000,   /* U+192A */
  0x0000,   /* U+192B */
  0x0000,   /* U+192C */
  0x0000,   /* U+192D */
  0x0000,   /* U+192E */
  0x0000,   /* U+192F */
  0x0000,   /* U+1930 */
  0x0000,   /* U+1931 */
  0x0000,   /* U+1932 */
  0x0000,   /* U+1933 */
  0x0000,   /* U+1934 */
  0x0000,   /* U+1935 */
  0x0000,   /* U+1936 */
  0x0000,   /* U+1937 */
  0x0000,   /* U+1938 */
  0x0000,   /* U+1939 */
  0x0000,   /* U+193A */
  0x0000,   /* U+193B */
  0x0000,   /* U+193C */
  0x0000,   /* U+193D */
  0x0000,   /* U+193E */
  0x0000,   /* U+193F */
  0x0000,   /* U+1940 */
  0x0000,   /* U+1941 */
  0x0000,   /* U+1942 */
  0x0000,   /* U+1943 */
  0x0000,   /* U+1944 */
  0x0000,   /* U+1945 */
  0x0000,   /* U+1946 */
  0x0000,   /* U+1947 */
  0x0000,   /* U+1948 */
  0x0000,   /* U+1949 */
  0x0000,   /* U+194A */
  0x0000,   /* U+194B */
  0x0000,   /* U+194C */
  0x0000,   /* U+194D */
  0x0000,   /* U+194E */
  0x0000,   /* U+194F */
  0x0000,   /* U+1950 */
  0x0000,   /* U+1951 */
  0x0000,   /* U+1952 */
  0x0000,   /* U+1953 */
  0x0000,   /* U+1954 */
  0x0000,   /* U+1955 */
  0x0000,   /* U+1956 */
  0x0000,   /* U+1957 */
  0x0000,   /* U+1958 */
  0x0000,   /* U+1959 */
  0x0000,   /* U+195A */
  0x0000,   /* U+195B */
  0x0000,   /* U+195C */
  0x0000,   /* U+195D */
  0x0000,   /* U+195E */
  0x0000,   /* U+195F */
  0x0000,   /* U+1960 */
  0x0000,   /* U+1961 */
  0x0000,   /* U+1962 */
  0x0000,   /* U+1963 */
  0x0000,   /* U+1964 */
  0x0000,   /* U+1965 */
  0x0000,   /* U+1966 */
  0x0000,   /* U+1967 */
  0x0000,   /* U+1968 */
  0x0000,   /* U+1969 */
  0x0000,   /* U+196A */
  0x0000,   /* U+196B */
  0x0000,   /* U+196C */
  0x0000,   /* U+196D */
  0x0000,   /* U+196E */
  0x0000,   /* U+196F */
  0x0000,   /* U+1970 */
  0x0000,   /* U+1971 */
  0x0000,   /* U+1972 */
  0x0000,   /* U+1973 */
  0x0000,   /* U+1974 */
  0x0000,   /* U+1975 */
  0x0000,   /* U+1976 */
  0x0000,   /* U+1977 */
  0x0000,   /* U+1978 */
  0x0000,   /* U+1979 */
  0x0000,   /* U+197A */
  0x0000,   /* U+197B */
  0x0000,   /* U+197C */
  0x0000,   /* U+197D */
  0x0000,   /* U+197E */
  0x0000,   /* U+197F */
  0x0000,   /* U+1980 */
  0x0000,   /* U+1981 */
  0x0000,   /* U+1982 */
  0x0000,   /* U+1983 */
  0x0000,   /* U+1984 */
  0x0000,   /* U+1985 */
  0x0000,   /* U+1986 */
  0x0000,   /* U+1987 */
  0x0000,   /* U+1988 */
  0x0000,   /* U+1989 */
  0x0000,   /* U+198A */
  0x0000,   /* U+198B */
  0x0000,   /* U+198C */
  0x0000,   /* U+198D */
  0x0000,   /* U+198E */
  0x0000,   /* U+198F */
  0x0000,   /* U+1990 */
  0x0000,   /* U+1991 */
  0x0000,   /* U+1992 */
  0x0000,   /* U+1993 */
  0x0000,   /* U+1994 */
  0x0000,   /* U+1995 */
  0x0000,   /* U+1996 */
  0x0000,   /* U+1997 */
  0x0000,   /* U+1998 */
  0x0000,   /* U+1999 */
  0x0000,   /* U+199A */
  0x0000,   /* U+199B */
  0x0000,   /* U+199C */
  0x0000,   /* U+199D */
  0x0000,   /* U+199E */
  0x0000,   /* U+199F */
  0x0000,   /* U+19A0 */
  0x0000,   /* U+19A1 */
  0x0000,   /* U+19A2 */
  0x0000,   /* U+19A3 */
  0x0000,   /* U+19A4 */
  0x0000,   /* U+19A5 */
  0x0000,   /* U+19A6 */
  0x0000,   /* U+19A7 */
  0x0000,   /* U+19A8 */
  0x0000,   /* U+19A9 */
  0x0000,   /* U+19AA */
  0x0000,   /* U+19AB */
  0x0000,   /* U+19AC */
  0x0000,   /* U+19AD */
  0x0000,   /* U+19AE */
  0x0000,   /* U+19AF */
  0x0000,   /* U+19B0 */
  0x0000,   /* U+19B1 */
  0x0000,   /* U+19B2 */
  0x0000,   /* U+19B3 */
  0x0000,   /* U+19B4 */
  0x0000,   /* U+19B5 */
  0x0000,   /* U+19B6 */
  0x0000,   /* U+19B7 */
  0x0000,   /* U+19B8 */
  0x0000,   /* U+19B9 */
  0x0000,   /* U+19BA */
  0x0000,   /* U+19BB */
  0x0000,   /* U+19BC */
  0x0000,   /* U+19BD */
  0x0000,   /* U+19BE */
  0x0000,   /* U+19BF */
  0x0000,   /* U+19C0 */
  0x0000,   /* U+19C1 */
  0x0000,   /* U+19C2 */
  0x0000,   /* U+19C3 */
  0x0000,   /* U+19C4 */
  0x0000,   /* U+19C5 */
  0x0000,   /* U+19C6 */
  0x0000,   /* U+19C7 */
  0x0000,   /* U+19C8 */
  0x0000,   /* U+19C9 */
  0x0000,   /* U+19CA */
  0x0000,   /* U+19CB */
  0x0000,   /* U+19CC */
  0x0000,   /* U+19CD */
  0x0000,   /* U+19CE */
  0x0000,   /* U+19CF */
  0x0000,   /* U+19D0 */
  0x0000,   /* U+19D1 */
  0x0000,   /* U+19D2 */
  0x0000,   /* U+19D3 */
  0x0000,   /* U+19D4 */
  0x0000,   /* U+19D5 */
  0x0000,   /* U+19D6 */
  0x0000,   /* U+19D7 */
  0x0000,   /* U+19D8 */
  0x0000,   /* U+19D9 */
  0x0000,   /* U+19DA */
  0x0000,   /* U+19DB */
  0x0000,   /* U+19DC */
  0x0000,   /* U+19DD */
  0x0000,   /* U+19DE */
  0x0004,   /* U+19DF */
  0x0000,   /* U+19E0 */
  0x0000,   /* U+19E1 */
  0x0000,   /* U+19E2 */
  0x0000,   /* U+19E3 */
  0x0000,   /* U+19E4 */
  0x0000,   /* U+19E5 */
  0x0000,   /* U+19E6 */
  0x0000,   /* U+19E7 */
  0x0000,   /* U+19E8 */
  0x0000,   /* U+19E9 */
  0x0000,   /* U+19EA */
  0x0000,   /* U+19EB */
  0x0000,   /* U+19EC */
  0x0000,   /* U+19ED */
  0x0000,   /* U+19EE */
  0x0000,   /* U+19EF */
  0x0000,   /* U+19F0 */
  0x0000,   /* U+19F1 */
  0x0000,   /* U+19F2 */
  0x0000,   /* U+19F3 */
  0x0000,   /* U+19F4 */
  0x0000,   /* U+19F5 */
  0x0000,   /* U+19F6 */
  0x0000,   /* U+19F7 */
  0x0000,   /* U+19F8 */
  0x0000,   /* U+19F9 */
  0x0000,   /* U+19FA */
  0x0000,   /* U+19FB */
  0x0000,   /* U+19FC */
  0x0000,   /* U+19FD */
  0x0000,   /* U+19FE */
  0x0000,   /* U+19FF */
};

uint16 uca900_p01A[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2FB5,   /* U+1A00 */
  0x2FB6,   /* U+1A01 */
  0x2FB7,   /* U+1A02 */
  0x2FB8,   /* U+1A03 */
  0x2FB9,   /* U+1A04 */
  0x2FBA,   /* U+1A05 */
  0x2FBB,   /* U+1A06 */
  0x2FBC,   /* U+1A07 */
  0x2FBD,   /* U+1A08 */
  0x2FBE,   /* U+1A09 */
  0x2FBF,   /* U+1A0A */
  0x2FC0,   /* U+1A0B */
  0x2FC1,   /* U+1A0C */
  0x2FC2,   /* U+1A0D */
  0x2FC3,   /* U+1A0E */
  0x2FC4,   /* U+1A0F */
  0x2FC5,   /* U+1A10 */
  0x2FC6,   /* U+1A11 */
  0x2FC7,   /* U+1A12 */
  0x2FC8,   /* U+1A13 */
  0x2FC9,   /* U+1A14 */
  0x2FCA,   /* U+1A15 */
  0x2FCB,   /* U+1A16 */
  0x2FCC,   /* U+1A17 */
  0x2FCD,   /* U+1A18 */
  0x2FCE,   /* U+1A19 */
  0x2FCF,   /* U+1A1A */
  0x2FD0,   /* U+1A1B */
  0xFBC0,   /* U+1A1C */
  0xFBC0,   /* U+1A1D */
  0x02C9,   /* U+1A1E */
  0x02CA,   /* U+1A1F */
  0x31C5,   /* U+1A20 */
  0x31C6,   /* U+1A21 */
  0x31C7,   /* U+1A22 */
  0x31C8,   /* U+1A23 */
  0x31C9,   /* U+1A24 */
  0x31CA,   /* U+1A25 */
  0x31CB,   /* U+1A26 */
  0x31CC,   /* U+1A27 */
  0x31CD,   /* U+1A28 */
  0x31CE,   /* U+1A29 */
  0x31CF,   /* U+1A2A */
  0x31D0,   /* U+1A2B */
  0x31D1,   /* U+1A2C */
  0x31D2,   /* U+1A2D */
  0x31D3,   /* U+1A2E */
  0x31D4,   /* U+1A2F */
  0x31D5,   /* U+1A30 */
  0x31D6,   /* U+1A31 */
  0x31D7,   /* U+1A32 */
  0x31D8,   /* U+1A33 */
  0x31D9,   /* U+1A34 */
  0x31DA,   /* U+1A35 */
  0x31DB,   /* U+1A36 */
  0x31DC,   /* U+1A37 */
  0x31DD,   /* U+1A38 */
  0x31DE,   /* U+1A39 */
  0x31DF,   /* U+1A3A */
  0x31E0,   /* U+1A3B */
  0x31E1,   /* U+1A3C */
  0x31E2,   /* U+1A3D */
  0x31E3,   /* U+1A3E */
  0x31E4,   /* U+1A3F */
  0x31E5,   /* U+1A40 */
  0x31E6,   /* U+1A41 */
  0x31E7,   /* U+1A42 */
  0x31E8,   /* U+1A43 */
  0x31E9,   /* U+1A44 */
  0x31EA,   /* U+1A45 */
  0x31EB,   /* U+1A46 */
  0x31EC,   /* U+1A47 */
  0x31ED,   /* U+1A48 */
  0x31EE,   /* U+1A49 */
  0x31EF,   /* U+1A4A */
  0x31F0,   /* U+1A4B */
  0x31F1,   /* U+1A4C */
  0x31FA,   /* U+1A4D */
  0x31FB,   /* U+1A4E */
  0x31FC,   /* U+1A4F */
  0x31FD,   /* U+1A50 */
  0x31FE,   /* U+1A51 */
  0x31FF,   /* U+1A52 */
  0x31F2,   /* U+1A53 */
  0x31EB,   /* U+1A54 */
  0x31F4,   /* U+1A55 */
  0x31F5,   /* U+1A56 */
  0x31F6,   /* U+1A57 */
  0x31CB,   /* U+1A58 */
  0x31CB,   /* U+1A59 */
  0x31E0,   /* U+1A5A */
  0x31E0,   /* U+1A5B */
  0x31F7,   /* U+1A5C */
  0x31F8,   /* U+1A5D */
  0x31F9,   /* U+1A5E */
  0xFBC0,   /* U+1A5F */
  0x3211,   /* U+1A60 */
  0x3200,   /* U+1A61 */
  0x3202,   /* U+1A62 */
  0x3203,   /* U+1A63 */
  0x3203,   /* U+1A64 */
  0x3204,   /* U+1A65 */
  0x3205,   /* U+1A66 */
  0x3206,   /* U+1A67 */
  0x3207,   /* U+1A68 */
  0x3208,   /* U+1A69 */
  0x3209,   /* U+1A6A */
  0x31F3,   /* U+1A6B */
  0x3201,   /* U+1A6C */
  0x3210,   /* U+1A6D */
  0x320A,   /* U+1A6E */
  0x320B,   /* U+1A6F */
  0x320D,   /* U+1A70 */
  0x320E,   /* U+1A71 */
  0x320F,   /* U+1A72 */
  0x320C,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0xFBC0,   /* U+1A7D */
  0xFBC0,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x1C3D,   /* U+1A80 */
  0x1C3E,   /* U+1A81 */
  0x1C3F,   /* U+1A82 */
  0x1C40,   /* U+1A83 */
  0x1C41,   /* U+1A84 */
  0x1C42,   /* U+1A85 */
  0x1C43,   /* U+1A86 */
  0x1C44,   /* U+1A87 */
  0x1C45,   /* U+1A88 */
  0x1C46,   /* U+1A89 */
  0xFBC0,   /* U+1A8A */
  0xFBC0,   /* U+1A8B */
  0xFBC0,   /* U+1A8C */
  0xFBC0,   /* U+1A8D */
  0xFBC0,   /* U+1A8E */
  0xFBC0,   /* U+1A8F */
  0x1C3D,   /* U+1A90 */
  0x1C3E,   /* U+1A91 */
  0x1C3F,   /* U+1A92 */
  0x1C40,   /* U+1A93 */
  0x1C41,   /* U+1A94 */
  0x1C42,   /* U+1A95 */
  0x1C43,   /* U+1A96 */
  0x1C44,   /* U+1A97 */
  0x1C45,   /* U+1A98 */
  0x1C46,   /* U+1A99 */
  0xFBC0,   /* U+1A9A */
  0xFBC0,   /* U+1A9B */
  0xFBC0,   /* U+1A9C */
  0xFBC0,   /* U+1A9D */
  0xFBC0,   /* U+1A9E */
  0xFBC0,   /* U+1A9F */
  0x0415,   /* U+1AA0 */
  0x0416,   /* U+1AA1 */
  0x0417,   /* U+1AA2 */
  0x0418,   /* U+1AA3 */
  0x0419,   /* U+1AA4 */
  0x041A,   /* U+1AA5 */
  0x041B,   /* U+1AA6 */
  0x1BFE,   /* U+1AA7 */
  0x029D,   /* U+1AA8 */
  0x029E,   /* U+1AA9 */
  0x029F,   /* U+1AAA */
  0x02A0,   /* U+1AAB */
  0x041C,   /* U+1AAC */
  0x041D,   /* U+1AAD */
  0xFBC0,   /* U+1AAE */
  0xFBC0,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0xFBC0,   /* U+1ABF */
  0xFBC0,   /* U+1AC0 */
  0xFBC0,   /* U+1AC1 */
  0xFBC0,   /* U+1AC2 */
  0xFBC0,   /* U+1AC3 */
  0xFBC0,   /* U+1AC4 */
  0xFBC0,   /* U+1AC5 */
  0xFBC0,   /* U+1AC6 */
  0xFBC0,   /* U+1AC7 */
  0xFBC0,   /* U+1AC8 */
  0xFBC0,   /* U+1AC9 */
  0xFBC0,   /* U+1ACA */
  0xFBC0,   /* U+1ACB */
  0xFBC0,   /* U+1ACC */
  0xFBC0,   /* U+1ACD */
  0xFBC0,   /* U+1ACE */
  0xFBC0,   /* U+1ACF */
  0xFBC0,   /* U+1AD0 */
  0xFBC0,   /* U+1AD1 */
  0xFBC0,   /* U+1AD2 */
  0xFBC0,   /* U+1AD3 */
  0xFBC0,   /* U+1AD4 */
  0xFBC0,   /* U+1AD5 */
  0xFBC0,   /* U+1AD6 */
  0xFBC0,   /* U+1AD7 */
  0xFBC0,   /* U+1AD8 */
  0xFBC0,   /* U+1AD9 */
  0xFBC0,   /* U+1ADA */
  0xFBC0,   /* U+1ADB */
  0xFBC0,   /* U+1ADC */
  0xFBC0,   /* U+1ADD */
  0xFBC0,   /* U+1ADE */
  0xFBC0,   /* U+1ADF */
  0xFBC0,   /* U+1AE0 */
  0xFBC0,   /* U+1AE1 */
  0xFBC0,   /* U+1AE2 */
  0xFBC0,   /* U+1AE3 */
  0xFBC0,   /* U+1AE4 */
  0xFBC0,   /* U+1AE5 */
  0xFBC0,   /* U+1AE6 */
  0xFBC0,   /* U+1AE7 */
  0xFBC0,   /* U+1AE8 */
  0xFBC0,   /* U+1AE9 */
  0xFBC0,   /* U+1AEA */
  0xFBC0,   /* U+1AEB */
  0xFBC0,   /* U+1AEC */
  0xFBC0,   /* U+1AED */
  0xFBC0,   /* U+1AEE */
  0xFBC0,   /* U+1AEF */
  0xFBC0,   /* U+1AF0 */
  0xFBC0,   /* U+1AF1 */
  0xFBC0,   /* U+1AF2 */
  0xFBC0,   /* U+1AF3 */
  0xFBC0,   /* U+1AF4 */
  0xFBC0,   /* U+1AF5 */
  0xFBC0,   /* U+1AF6 */
  0xFBC0,   /* U+1AF7 */
  0xFBC0,   /* U+1AF8 */
  0xFBC0,   /* U+1AF9 */
  0xFBC0,   /* U+1AFA */
  0xFBC0,   /* U+1AFB */
  0xFBC0,   /* U+1AFC */
  0xFBC0,   /* U+1AFD */
  0xFBC0,   /* U+1AFE */
  0xFBC0,   /* U+1AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1A00 */
  0x0020,   /* U+1A01 */
  0x0020,   /* U+1A02 */
  0x0020,   /* U+1A03 */
  0x0020,   /* U+1A04 */
  0x0020,   /* U+1A05 */
  0x0020,   /* U+1A06 */
  0x0020,   /* U+1A07 */
  0x0020,   /* U+1A08 */
  0x0020,   /* U+1A09 */
  0x0020,   /* U+1A0A */
  0x0020,   /* U+1A0B */
  0x0020,   /* U+1A0C */
  0x0020,   /* U+1A0D */
  0x0020,   /* U+1A0E */
  0x0020,   /* U+1A0F */
  0x0020,   /* U+1A10 */
  0x0020,   /* U+1A11 */
  0x0020,   /* U+1A12 */
  0x0020,   /* U+1A13 */
  0x0020,   /* U+1A14 */
  0x0020,   /* U+1A15 */
  0x0020,   /* U+1A16 */
  0x0020,   /* U+1A17 */
  0x0020,   /* U+1A18 */
  0x0020,   /* U+1A19 */
  0x0020,   /* U+1A1A */
  0x0020,   /* U+1A1B */
  0x0020,   /* U+1A1C */
  0x0020,   /* U+1A1D */
  0x0020,   /* U+1A1E */
  0x0020,   /* U+1A1F */
  0x0020,   /* U+1A20 */
  0x0020,   /* U+1A21 */
  0x0020,   /* U+1A22 */
  0x0020,   /* U+1A23 */
  0x0020,   /* U+1A24 */
  0x0020,   /* U+1A25 */
  0x0020,   /* U+1A26 */
  0x0020,   /* U+1A27 */
  0x0020,   /* U+1A28 */
  0x0020,   /* U+1A29 */
  0x0020,   /* U+1A2A */
  0x0020,   /* U+1A2B */
  0x0020,   /* U+1A2C */
  0x0020,   /* U+1A2D */
  0x0020,   /* U+1A2E */
  0x0020,   /* U+1A2F */
  0x0020,   /* U+1A30 */
  0x0020,   /* U+1A31 */
  0x0020,   /* U+1A32 */
  0x0020,   /* U+1A33 */
  0x0020,   /* U+1A34 */
  0x0020,   /* U+1A35 */
  0x0020,   /* U+1A36 */
  0x0020,   /* U+1A37 */
  0x0020,   /* U+1A38 */
  0x0020,   /* U+1A39 */
  0x0020,   /* U+1A3A */
  0x0020,   /* U+1A3B */
  0x0020,   /* U+1A3C */
  0x0020,   /* U+1A3D */
  0x0020,   /* U+1A3E */
  0x0020,   /* U+1A3F */
  0x0020,   /* U+1A40 */
  0x0020,   /* U+1A41 */
  0x0020,   /* U+1A42 */
  0x0020,   /* U+1A43 */
  0x0020,   /* U+1A44 */
  0x0020,   /* U+1A45 */
  0x0020,   /* U+1A46 */
  0x0020,   /* U+1A47 */
  0x0020,   /* U+1A48 */
  0x0020,   /* U+1A49 */
  0x0020,   /* U+1A4A */
  0x0020,   /* U+1A4B */
  0x0020,   /* U+1A4C */
  0x0020,   /* U+1A4D */
  0x0020,   /* U+1A4E */
  0x0020,   /* U+1A4F */
  0x0020,   /* U+1A50 */
  0x0020,   /* U+1A51 */
  0x0020,   /* U+1A52 */
  0x0020,   /* U+1A53 */
  0x0020,   /* U+1A54 */
  0x0020,   /* U+1A55 */
  0x0020,   /* U+1A56 */
  0x0020,   /* U+1A57 */
  0x0020,   /* U+1A58 */
  0x0020,   /* U+1A59 */
  0x0020,   /* U+1A5A */
  0x0020,   /* U+1A5B */
  0x0020,   /* U+1A5C */
  0x0020,   /* U+1A5D */
  0x0020,   /* U+1A5E */
  0x0020,   /* U+1A5F */
  0x0020,   /* U+1A60 */
  0x0020,   /* U+1A61 */
  0x0020,   /* U+1A62 */
  0x0020,   /* U+1A63 */
  0x0020,   /* U+1A64 */
  0x0020,   /* U+1A65 */
  0x0020,   /* U+1A66 */
  0x0020,   /* U+1A67 */
  0x0020,   /* U+1A68 */
  0x0020,   /* U+1A69 */
  0x0020,   /* U+1A6A */
  0x0020,   /* U+1A6B */
  0x0020,   /* U+1A6C */
  0x0020,   /* U+1A6D */
  0x0020,   /* U+1A6E */
  0x0020,   /* U+1A6F */
  0x0020,   /* U+1A70 */
  0x0020,   /* U+1A71 */
  0x0020,   /* U+1A72 */
  0x0020,   /* U+1A73 */
  0x00C4,   /* U+1A74 */
  0x00E9,   /* U+1A75 */
  0x00EA,   /* U+1A76 */
  0x00EB,   /* U+1A77 */
  0x00EC,   /* U+1A78 */
  0x00ED,   /* U+1A79 */
  0x00EE,   /* U+1A7A */
  0x00EF,   /* U+1A7B */
  0x00F0,   /* U+1A7C */
  0x0020,   /* U+1A7D */
  0x0020,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0020,   /* U+1A80 */
  0x0020,   /* U+1A81 */
  0x0020,   /* U+1A82 */
  0x0020,   /* U+1A83 */
  0x0020,   /* U+1A84 */
  0x0020,   /* U+1A85 */
  0x0020,   /* U+1A86 */
  0x0020,   /* U+1A87 */
  0x0020,   /* U+1A88 */
  0x0020,   /* U+1A89 */
  0x0020,   /* U+1A8A */
  0x0020,   /* U+1A8B */
  0x0020,   /* U+1A8C */
  0x0020,   /* U+1A8D */
  0x0020,   /* U+1A8E */
  0x0020,   /* U+1A8F */
  0x0020,   /* U+1A90 */
  0x0020,   /* U+1A91 */
  0x0020,   /* U+1A92 */
  0x0020,   /* U+1A93 */
  0x0020,   /* U+1A94 */
  0x0020,   /* U+1A95 */
  0x0020,   /* U+1A96 */
  0x0020,   /* U+1A97 */
  0x0020,   /* U+1A98 */
  0x0020,   /* U+1A99 */
  0x0020,   /* U+1A9A */
  0x0020,   /* U+1A9B */
  0x0020,   /* U+1A9C */
  0x0020,   /* U+1A9D */
  0x0020,   /* U+1A9E */
  0x0020,   /* U+1A9F */
  0x0020,   /* U+1AA0 */
  0x0020,   /* U+1AA1 */
  0x0020,   /* U+1AA2 */
  0x0020,   /* U+1AA3 */
  0x0020,   /* U+1AA4 */
  0x0020,   /* U+1AA5 */
  0x0020,   /* U+1AA6 */
  0x0020,   /* U+1AA7 */
  0x0020,   /* U+1AA8 */
  0x0020,   /* U+1AA9 */
  0x0020,   /* U+1AAA */
  0x0020,   /* U+1AAB */
  0x0020,   /* U+1AAC */
  0x0020,   /* U+1AAD */
  0x0020,   /* U+1AAE */
  0x0020,   /* U+1AAF */
  0x0033,   /* U+1AB0 */
  0x0033,   /* U+1AB1 */
  0x0033,   /* U+1AB2 */
  0x0033,   /* U+1AB3 */
  0x0033,   /* U+1AB4 */
  0x0034,   /* U+1AB5 */
  0x0034,   /* U+1AB6 */
  0x0034,   /* U+1AB7 */
  0x0034,   /* U+1AB8 */
  0x0034,   /* U+1AB9 */
  0x0034,   /* U+1ABA */
  0x0033,   /* U+1ABB */
  0x0033,   /* U+1ABC */
  0x0034,   /* U+1ABD */
  0x0036,   /* U+1ABE */
  0x0020,   /* U+1ABF */
  0x0020,   /* U+1AC0 */
  0x0020,   /* U+1AC1 */
  0x0020,   /* U+1AC2 */
  0x0020,   /* U+1AC3 */
  0x0020,   /* U+1AC4 */
  0x0020,   /* U+1AC5 */
  0x0020,   /* U+1AC6 */
  0x0020,   /* U+1AC7 */
  0x0020,   /* U+1AC8 */
  0x0020,   /* U+1AC9 */
  0x0020,   /* U+1ACA */
  0x0020,   /* U+1ACB */
  0x0020,   /* U+1ACC */
  0x0020,   /* U+1ACD */
  0x0020,   /* U+1ACE */
  0x0020,   /* U+1ACF */
  0x0020,   /* U+1AD0 */
  0x0020,   /* U+1AD1 */
  0x0020,   /* U+1AD2 */
  0x0020,   /* U+1AD3 */
  0x0020,   /* U+1AD4 */
  0x0020,   /* U+1AD5 */
  0x0020,   /* U+1AD6 */
  0x0020,   /* U+1AD7 */
  0x0020,   /* U+1AD8 */
  0x0020,   /* U+1AD9 */
  0x0020,   /* U+1ADA */
  0x0020,   /* U+1ADB */
  0x0020,   /* U+1ADC */
  0x0020,   /* U+1ADD */
  0x0020,   /* U+1ADE */
  0x0020,   /* U+1ADF */
  0x0020,   /* U+1AE0 */
  0x0020,   /* U+1AE1 */
  0x0020,   /* U+1AE2 */
  0x0020,   /* U+1AE3 */
  0x0020,   /* U+1AE4 */
  0x0020,   /* U+1AE5 */
  0x0020,   /* U+1AE6 */
  0x0020,   /* U+1AE7 */
  0x0020,   /* U+1AE8 */
  0x0020,   /* U+1AE9 */
  0x0020,   /* U+1AEA */
  0x0020,   /* U+1AEB */
  0x0020,   /* U+1AEC */
  0x0020,   /* U+1AED */
  0x0020,   /* U+1AEE */
  0x0020,   /* U+1AEF */
  0x0020,   /* U+1AF0 */
  0x0020,   /* U+1AF1 */
  0x0020,   /* U+1AF2 */
  0x0020,   /* U+1AF3 */
  0x0020,   /* U+1AF4 */
  0x0020,   /* U+1AF5 */
  0x0020,   /* U+1AF6 */
  0x0020,   /* U+1AF7 */
  0x0020,   /* U+1AF8 */
  0x0020,   /* U+1AF9 */
  0x0020,   /* U+1AFA */
  0x0020,   /* U+1AFB */
  0x0020,   /* U+1AFC */
  0x0020,   /* U+1AFD */
  0x0020,   /* U+1AFE */
  0x0020,   /* U+1AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1A00 */
  0x0002,   /* U+1A01 */
  0x0002,   /* U+1A02 */
  0x0002,   /* U+1A03 */
  0x0002,   /* U+1A04 */
  0x0002,   /* U+1A05 */
  0x0002,   /* U+1A06 */
  0x0002,   /* U+1A07 */
  0x0002,   /* U+1A08 */
  0x0002,   /* U+1A09 */
  0x0002,   /* U+1A0A */
  0x0002,   /* U+1A0B */
  0x0002,   /* U+1A0C */
  0x0002,   /* U+1A0D */
  0x0002,   /* U+1A0E */
  0x0002,   /* U+1A0F */
  0x0002,   /* U+1A10 */
  0x0002,   /* U+1A11 */
  0x0002,   /* U+1A12 */
  0x0002,   /* U+1A13 */
  0x0002,   /* U+1A14 */
  0x0002,   /* U+1A15 */
  0x0002,   /* U+1A16 */
  0x0002,   /* U+1A17 */
  0x0002,   /* U+1A18 */
  0x0002,   /* U+1A19 */
  0x0002,   /* U+1A1A */
  0x0002,   /* U+1A1B */
  0x0002,   /* U+1A1C */
  0x0002,   /* U+1A1D */
  0x0002,   /* U+1A1E */
  0x0002,   /* U+1A1F */
  0x0002,   /* U+1A20 */
  0x0002,   /* U+1A21 */
  0x0002,   /* U+1A22 */
  0x0002,   /* U+1A23 */
  0x0002,   /* U+1A24 */
  0x0002,   /* U+1A25 */
  0x0002,   /* U+1A26 */
  0x0002,   /* U+1A27 */
  0x0002,   /* U+1A28 */
  0x0002,   /* U+1A29 */
  0x0002,   /* U+1A2A */
  0x0002,   /* U+1A2B */
  0x0002,   /* U+1A2C */
  0x0002,   /* U+1A2D */
  0x0002,   /* U+1A2E */
  0x0002,   /* U+1A2F */
  0x0002,   /* U+1A30 */
  0x0002,   /* U+1A31 */
  0x0002,   /* U+1A32 */
  0x0002,   /* U+1A33 */
  0x0002,   /* U+1A34 */
  0x0002,   /* U+1A35 */
  0x0002,   /* U+1A36 */
  0x0002,   /* U+1A37 */
  0x0002,   /* U+1A38 */
  0x0002,   /* U+1A39 */
  0x0002,   /* U+1A3A */
  0x0002,   /* U+1A3B */
  0x0002,   /* U+1A3C */
  0x0002,   /* U+1A3D */
  0x0002,   /* U+1A3E */
  0x0002,   /* U+1A3F */
  0x0002,   /* U+1A40 */
  0x0002,   /* U+1A41 */
  0x0002,   /* U+1A42 */
  0x0002,   /* U+1A43 */
  0x0002,   /* U+1A44 */
  0x0002,   /* U+1A45 */
  0x0002,   /* U+1A46 */
  0x0002,   /* U+1A47 */
  0x0002,   /* U+1A48 */
  0x0002,   /* U+1A49 */
  0x0002,   /* U+1A4A */
  0x0002,   /* U+1A4B */
  0x0002,   /* U+1A4C */
  0x0002,   /* U+1A4D */
  0x0002,   /* U+1A4E */
  0x0002,   /* U+1A4F */
  0x0002,   /* U+1A50 */
  0x0002,   /* U+1A51 */
  0x0002,   /* U+1A52 */
  0x0002,   /* U+1A53 */
  0x0004,   /* U+1A54 */
  0x0002,   /* U+1A55 */
  0x0002,   /* U+1A56 */
  0x0002,   /* U+1A57 */
  0x0004,   /* U+1A58 */
  0x0004,   /* U+1A59 */
  0x0004,   /* U+1A5A */
  0x0004,   /* U+1A5B */
  0x0002,   /* U+1A5C */
  0x0002,   /* U+1A5D */
  0x0002,   /* U+1A5E */
  0x0002,   /* U+1A5F */
  0x0002,   /* U+1A60 */
  0x0002,   /* U+1A61 */
  0x0002,   /* U+1A62 */
  0x0002,   /* U+1A63 */
  0x0004,   /* U+1A64 */
  0x0002,   /* U+1A65 */
  0x0002,   /* U+1A66 */
  0x0002,   /* U+1A67 */
  0x0002,   /* U+1A68 */
  0x0002,   /* U+1A69 */
  0x0002,   /* U+1A6A */
  0x0002,   /* U+1A6B */
  0x0002,   /* U+1A6C */
  0x0002,   /* U+1A6D */
  0x0002,   /* U+1A6E */
  0x0002,   /* U+1A6F */
  0x0002,   /* U+1A70 */
  0x0002,   /* U+1A71 */
  0x0002,   /* U+1A72 */
  0x0002,   /* U+1A73 */
  0x0002,   /* U+1A74 */
  0x0002,   /* U+1A75 */
  0x0002,   /* U+1A76 */
  0x0002,   /* U+1A77 */
  0x0002,   /* U+1A78 */
  0x0002,   /* U+1A79 */
  0x0002,   /* U+1A7A */
  0x0002,   /* U+1A7B */
  0x0002,   /* U+1A7C */
  0x0002,   /* U+1A7D */
  0x0002,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0002,   /* U+1A80 */
  0x0002,   /* U+1A81 */
  0x0002,   /* U+1A82 */
  0x0002,   /* U+1A83 */
  0x0002,   /* U+1A84 */
  0x0002,   /* U+1A85 */
  0x0002,   /* U+1A86 */
  0x0002,   /* U+1A87 */
  0x0002,   /* U+1A88 */
  0x0002,   /* U+1A89 */
  0x0002,   /* U+1A8A */
  0x0002,   /* U+1A8B */
  0x0002,   /* U+1A8C */
  0x0002,   /* U+1A8D */
  0x0002,   /* U+1A8E */
  0x0002,   /* U+1A8F */
  0x0002,   /* U+1A90 */
  0x0002,   /* U+1A91 */
  0x0002,   /* U+1A92 */
  0x0002,   /* U+1A93 */
  0x0002,   /* U+1A94 */
  0x0002,   /* U+1A95 */
  0x0002,   /* U+1A96 */
  0x0002,   /* U+1A97 */
  0x0002,   /* U+1A98 */
  0x0002,   /* U+1A99 */
  0x0002,   /* U+1A9A */
  0x0002,   /* U+1A9B */
  0x0002,   /* U+1A9C */
  0x0002,   /* U+1A9D */
  0x0002,   /* U+1A9E */
  0x0002,   /* U+1A9F */
  0x0002,   /* U+1AA0 */
  0x0002,   /* U+1AA1 */
  0x0002,   /* U+1AA2 */
  0x0002,   /* U+1AA3 */
  0x0002,   /* U+1AA4 */
  0x0002,   /* U+1AA5 */
  0x0002,   /* U+1AA6 */
  0x0002,   /* U+1AA7 */
  0x0002,   /* U+1AA8 */
  0x0002,   /* U+1AA9 */
  0x0002,   /* U+1AAA */
  0x0002,   /* U+1AAB */
  0x0002,   /* U+1AAC */
  0x0002,   /* U+1AAD */
  0x0002,   /* U+1AAE */
  0x0002,   /* U+1AAF */
  0x0002,   /* U+1AB0 */
  0x0002,   /* U+1AB1 */
  0x0002,   /* U+1AB2 */
  0x0002,   /* U+1AB3 */
  0x0002,   /* U+1AB4 */
  0x0002,   /* U+1AB5 */
  0x0002,   /* U+1AB6 */
  0x0002,   /* U+1AB7 */
  0x0002,   /* U+1AB8 */
  0x0002,   /* U+1AB9 */
  0x0002,   /* U+1ABA */
  0x0002,   /* U+1ABB */
  0x0002,   /* U+1ABC */
  0x0002,   /* U+1ABD */
  0x0002,   /* U+1ABE */
  0x0002,   /* U+1ABF */
  0x0002,   /* U+1AC0 */
  0x0002,   /* U+1AC1 */
  0x0002,   /* U+1AC2 */
  0x0002,   /* U+1AC3 */
  0x0002,   /* U+1AC4 */
  0x0002,   /* U+1AC5 */
  0x0002,   /* U+1AC6 */
  0x0002,   /* U+1AC7 */
  0x0002,   /* U+1AC8 */
  0x0002,   /* U+1AC9 */
  0x0002,   /* U+1ACA */
  0x0002,   /* U+1ACB */
  0x0002,   /* U+1ACC */
  0x0002,   /* U+1ACD */
  0x0002,   /* U+1ACE */
  0x0002,   /* U+1ACF */
  0x0002,   /* U+1AD0 */
  0x0002,   /* U+1AD1 */
  0x0002,   /* U+1AD2 */
  0x0002,   /* U+1AD3 */
  0x0002,   /* U+1AD4 */
  0x0002,   /* U+1AD5 */
  0x0002,   /* U+1AD6 */
  0x0002,   /* U+1AD7 */
  0x0002,   /* U+1AD8 */
  0x0002,   /* U+1AD9 */
  0x0002,   /* U+1ADA */
  0x0002,   /* U+1ADB */
  0x0002,   /* U+1ADC */
  0x0002,   /* U+1ADD */
  0x0002,   /* U+1ADE */
  0x0002,   /* U+1ADF */
  0x0002,   /* U+1AE0 */
  0x0002,   /* U+1AE1 */
  0x0002,   /* U+1AE2 */
  0x0002,   /* U+1AE3 */
  0x0002,   /* U+1AE4 */
  0x0002,   /* U+1AE5 */
  0x0002,   /* U+1AE6 */
  0x0002,   /* U+1AE7 */
  0x0002,   /* U+1AE8 */
  0x0002,   /* U+1AE9 */
  0x0002,   /* U+1AEA */
  0x0002,   /* U+1AEB */
  0x0002,   /* U+1AEC */
  0x0002,   /* U+1AED */
  0x0002,   /* U+1AEE */
  0x0002,   /* U+1AEF */
  0x0002,   /* U+1AF0 */
  0x0002,   /* U+1AF1 */
  0x0002,   /* U+1AF2 */
  0x0002,   /* U+1AF3 */
  0x0002,   /* U+1AF4 */
  0x0002,   /* U+1AF5 */
  0x0002,   /* U+1AF6 */
  0x0002,   /* U+1AF7 */
  0x0002,   /* U+1AF8 */
  0x0002,   /* U+1AF9 */
  0x0002,   /* U+1AFA */
  0x0002,   /* U+1AFB */
  0x0002,   /* U+1AFC */
  0x0002,   /* U+1AFD */
  0x0002,   /* U+1AFE */
  0x0002,   /* U+1AFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x9A1C,   /* U+1A1C */
  0x9A1D,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x3211,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x9A5F,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x9A7D,   /* U+1A7D */
  0x9A7E,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x9A8A,   /* U+1A8A */
  0x9A8B,   /* U+1A8B */
  0x9A8C,   /* U+1A8C */
  0x9A8D,   /* U+1A8D */
  0x9A8E,   /* U+1A8E */
  0x9A8F,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x9A9A,   /* U+1A9A */
  0x9A9B,   /* U+1A9B */
  0x9A9C,   /* U+1A9C */
  0x9A9D,   /* U+1A9D */
  0x9A9E,   /* U+1A9E */
  0x9A9F,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x9AAE,   /* U+1AAE */
  0x9AAF,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x9ABF,   /* U+1ABF */
  0x9AC0,   /* U+1AC0 */
  0x9AC1,   /* U+1AC1 */
  0x9AC2,   /* U+1AC2 */
  0x9AC3,   /* U+1AC3 */
  0x9AC4,   /* U+1AC4 */
  0x9AC5,   /* U+1AC5 */
  0x9AC6,   /* U+1AC6 */
  0x9AC7,   /* U+1AC7 */
  0x9AC8,   /* U+1AC8 */
  0x9AC9,   /* U+1AC9 */
  0x9ACA,   /* U+1ACA */
  0x9ACB,   /* U+1ACB */
  0x9ACC,   /* U+1ACC */
  0x9ACD,   /* U+1ACD */
  0x9ACE,   /* U+1ACE */
  0x9ACF,   /* U+1ACF */
  0x9AD0,   /* U+1AD0 */
  0x9AD1,   /* U+1AD1 */
  0x9AD2,   /* U+1AD2 */
  0x9AD3,   /* U+1AD3 */
  0x9AD4,   /* U+1AD4 */
  0x9AD5,   /* U+1AD5 */
  0x9AD6,   /* U+1AD6 */
  0x9AD7,   /* U+1AD7 */
  0x9AD8,   /* U+1AD8 */
  0x9AD9,   /* U+1AD9 */
  0x9ADA,   /* U+1ADA */
  0x9ADB,   /* U+1ADB */
  0x9ADC,   /* U+1ADC */
  0x9ADD,   /* U+1ADD */
  0x9ADE,   /* U+1ADE */
  0x9ADF,   /* U+1ADF */
  0x9AE0,   /* U+1AE0 */
  0x9AE1,   /* U+1AE1 */
  0x9AE2,   /* U+1AE2 */
  0x9AE3,   /* U+1AE3 */
  0x9AE4,   /* U+1AE4 */
  0x9AE5,   /* U+1AE5 */
  0x9AE6,   /* U+1AE6 */
  0x9AE7,   /* U+1AE7 */
  0x9AE8,   /* U+1AE8 */
  0x9AE9,   /* U+1AE9 */
  0x9AEA,   /* U+1AEA */
  0x9AEB,   /* U+1AEB */
  0x9AEC,   /* U+1AEC */
  0x9AED,   /* U+1AED */
  0x9AEE,   /* U+1AEE */
  0x9AEF,   /* U+1AEF */
  0x9AF0,   /* U+1AF0 */
  0x9AF1,   /* U+1AF1 */
  0x9AF2,   /* U+1AF2 */
  0x9AF3,   /* U+1AF3 */
  0x9AF4,   /* U+1AF4 */
  0x9AF5,   /* U+1AF5 */
  0x9AF6,   /* U+1AF6 */
  0x9AF7,   /* U+1AF7 */
  0x9AF8,   /* U+1AF8 */
  0x9AF9,   /* U+1AF9 */
  0x9AFA,   /* U+1AFA */
  0x9AFB,   /* U+1AFB */
  0x9AFC,   /* U+1AFC */
  0x9AFD,   /* U+1AFD */
  0x9AFE,   /* U+1AFE */
  0x9AFF,   /* U+1AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x0000,   /* U+1A1C */
  0x0000,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x0020,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x0000,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x0000,   /* U+1A7D */
  0x0000,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x0000,   /* U+1A8A */
  0x0000,   /* U+1A8B */
  0x0000,   /* U+1A8C */
  0x0000,   /* U+1A8D */
  0x0000,   /* U+1A8E */
  0x0000,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x0000,   /* U+1A9A */
  0x0000,   /* U+1A9B */
  0x0000,   /* U+1A9C */
  0x0000,   /* U+1A9D */
  0x0000,   /* U+1A9E */
  0x0000,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x0000,   /* U+1AAE */
  0x0000,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x0000,   /* U+1ABF */
  0x0000,   /* U+1AC0 */
  0x0000,   /* U+1AC1 */
  0x0000,   /* U+1AC2 */
  0x0000,   /* U+1AC3 */
  0x0000,   /* U+1AC4 */
  0x0000,   /* U+1AC5 */
  0x0000,   /* U+1AC6 */
  0x0000,   /* U+1AC7 */
  0x0000,   /* U+1AC8 */
  0x0000,   /* U+1AC9 */
  0x0000,   /* U+1ACA */
  0x0000,   /* U+1ACB */
  0x0000,   /* U+1ACC */
  0x0000,   /* U+1ACD */
  0x0000,   /* U+1ACE */
  0x0000,   /* U+1ACF */
  0x0000,   /* U+1AD0 */
  0x0000,   /* U+1AD1 */
  0x0000,   /* U+1AD2 */
  0x0000,   /* U+1AD3 */
  0x0000,   /* U+1AD4 */
  0x0000,   /* U+1AD5 */
  0x0000,   /* U+1AD6 */
  0x0000,   /* U+1AD7 */
  0x0000,   /* U+1AD8 */
  0x0000,   /* U+1AD9 */
  0x0000,   /* U+1ADA */
  0x0000,   /* U+1ADB */
  0x0000,   /* U+1ADC */
  0x0000,   /* U+1ADD */
  0x0000,   /* U+1ADE */
  0x0000,   /* U+1ADF */
  0x0000,   /* U+1AE0 */
  0x0000,   /* U+1AE1 */
  0x0000,   /* U+1AE2 */
  0x0000,   /* U+1AE3 */
  0x0000,   /* U+1AE4 */
  0x0000,   /* U+1AE5 */
  0x0000,   /* U+1AE6 */
  0x0000,   /* U+1AE7 */
  0x0000,   /* U+1AE8 */
  0x0000,   /* U+1AE9 */
  0x0000,   /* U+1AEA */
  0x0000,   /* U+1AEB */
  0x0000,   /* U+1AEC */
  0x0000,   /* U+1AED */
  0x0000,   /* U+1AEE */
  0x0000,   /* U+1AEF */
  0x0000,   /* U+1AF0 */
  0x0000,   /* U+1AF1 */
  0x0000,   /* U+1AF2 */
  0x0000,   /* U+1AF3 */
  0x0000,   /* U+1AF4 */
  0x0000,   /* U+1AF5 */
  0x0000,   /* U+1AF6 */
  0x0000,   /* U+1AF7 */
  0x0000,   /* U+1AF8 */
  0x0000,   /* U+1AF9 */
  0x0000,   /* U+1AFA */
  0x0000,   /* U+1AFB */
  0x0000,   /* U+1AFC */
  0x0000,   /* U+1AFD */
  0x0000,   /* U+1AFE */
  0x0000,   /* U+1AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x0000,   /* U+1A1C */
  0x0000,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x0004,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x0000,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x0000,   /* U+1A7D */
  0x0000,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x0000,   /* U+1A8A */
  0x0000,   /* U+1A8B */
  0x0000,   /* U+1A8C */
  0x0000,   /* U+1A8D */
  0x0000,   /* U+1A8E */
  0x0000,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x0000,   /* U+1A9A */
  0x0000,   /* U+1A9B */
  0x0000,   /* U+1A9C */
  0x0000,   /* U+1A9D */
  0x0000,   /* U+1A9E */
  0x0000,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x0000,   /* U+1AAE */
  0x0000,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x0000,   /* U+1ABF */
  0x0000,   /* U+1AC0 */
  0x0000,   /* U+1AC1 */
  0x0000,   /* U+1AC2 */
  0x0000,   /* U+1AC3 */
  0x0000,   /* U+1AC4 */
  0x0000,   /* U+1AC5 */
  0x0000,   /* U+1AC6 */
  0x0000,   /* U+1AC7 */
  0x0000,   /* U+1AC8 */
  0x0000,   /* U+1AC9 */
  0x0000,   /* U+1ACA */
  0x0000,   /* U+1ACB */
  0x0000,   /* U+1ACC */
  0x0000,   /* U+1ACD */
  0x0000,   /* U+1ACE */
  0x0000,   /* U+1ACF */
  0x0000,   /* U+1AD0 */
  0x0000,   /* U+1AD1 */
  0x0000,   /* U+1AD2 */
  0x0000,   /* U+1AD3 */
  0x0000,   /* U+1AD4 */
  0x0000,   /* U+1AD5 */
  0x0000,   /* U+1AD6 */
  0x0000,   /* U+1AD7 */
  0x0000,   /* U+1AD8 */
  0x0000,   /* U+1AD9 */
  0x0000,   /* U+1ADA */
  0x0000,   /* U+1ADB */
  0x0000,   /* U+1ADC */
  0x0000,   /* U+1ADD */
  0x0000,   /* U+1ADE */
  0x0000,   /* U+1ADF */
  0x0000,   /* U+1AE0 */
  0x0000,   /* U+1AE1 */
  0x0000,   /* U+1AE2 */
  0x0000,   /* U+1AE3 */
  0x0000,   /* U+1AE4 */
  0x0000,   /* U+1AE5 */
  0x0000,   /* U+1AE6 */
  0x0000,   /* U+1AE7 */
  0x0000,   /* U+1AE8 */
  0x0000,   /* U+1AE9 */
  0x0000,   /* U+1AEA */
  0x0000,   /* U+1AEB */
  0x0000,   /* U+1AEC */
  0x0000,   /* U+1AED */
  0x0000,   /* U+1AEE */
  0x0000,   /* U+1AEF */
  0x0000,   /* U+1AF0 */
  0x0000,   /* U+1AF1 */
  0x0000,   /* U+1AF2 */
  0x0000,   /* U+1AF3 */
  0x0000,   /* U+1AF4 */
  0x0000,   /* U+1AF5 */
  0x0000,   /* U+1AF6 */
  0x0000,   /* U+1AF7 */
  0x0000,   /* U+1AF8 */
  0x0000,   /* U+1AF9 */
  0x0000,   /* U+1AFA */
  0x0000,   /* U+1AFB */
  0x0000,   /* U+1AFC */
  0x0000,   /* U+1AFD */
  0x0000,   /* U+1AFE */
  0x0000,   /* U+1AFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x0000,   /* U+1A1C */
  0x0000,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x31EB,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x0000,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x0000,   /* U+1A7D */
  0x0000,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x0000,   /* U+1A8A */
  0x0000,   /* U+1A8B */
  0x0000,   /* U+1A8C */
  0x0000,   /* U+1A8D */
  0x0000,   /* U+1A8E */
  0x0000,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x0000,   /* U+1A9A */
  0x0000,   /* U+1A9B */
  0x0000,   /* U+1A9C */
  0x0000,   /* U+1A9D */
  0x0000,   /* U+1A9E */
  0x0000,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x0000,   /* U+1AAE */
  0x0000,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x0000,   /* U+1ABF */
  0x0000,   /* U+1AC0 */
  0x0000,   /* U+1AC1 */
  0x0000,   /* U+1AC2 */
  0x0000,   /* U+1AC3 */
  0x0000,   /* U+1AC4 */
  0x0000,   /* U+1AC5 */
  0x0000,   /* U+1AC6 */
  0x0000,   /* U+1AC7 */
  0x0000,   /* U+1AC8 */
  0x0000,   /* U+1AC9 */
  0x0000,   /* U+1ACA */
  0x0000,   /* U+1ACB */
  0x0000,   /* U+1ACC */
  0x0000,   /* U+1ACD */
  0x0000,   /* U+1ACE */
  0x0000,   /* U+1ACF */
  0x0000,   /* U+1AD0 */
  0x0000,   /* U+1AD1 */
  0x0000,   /* U+1AD2 */
  0x0000,   /* U+1AD3 */
  0x0000,   /* U+1AD4 */
  0x0000,   /* U+1AD5 */
  0x0000,   /* U+1AD6 */
  0x0000,   /* U+1AD7 */
  0x0000,   /* U+1AD8 */
  0x0000,   /* U+1AD9 */
  0x0000,   /* U+1ADA */
  0x0000,   /* U+1ADB */
  0x0000,   /* U+1ADC */
  0x0000,   /* U+1ADD */
  0x0000,   /* U+1ADE */
  0x0000,   /* U+1ADF */
  0x0000,   /* U+1AE0 */
  0x0000,   /* U+1AE1 */
  0x0000,   /* U+1AE2 */
  0x0000,   /* U+1AE3 */
  0x0000,   /* U+1AE4 */
  0x0000,   /* U+1AE5 */
  0x0000,   /* U+1AE6 */
  0x0000,   /* U+1AE7 */
  0x0000,   /* U+1AE8 */
  0x0000,   /* U+1AE9 */
  0x0000,   /* U+1AEA */
  0x0000,   /* U+1AEB */
  0x0000,   /* U+1AEC */
  0x0000,   /* U+1AED */
  0x0000,   /* U+1AEE */
  0x0000,   /* U+1AEF */
  0x0000,   /* U+1AF0 */
  0x0000,   /* U+1AF1 */
  0x0000,   /* U+1AF2 */
  0x0000,   /* U+1AF3 */
  0x0000,   /* U+1AF4 */
  0x0000,   /* U+1AF5 */
  0x0000,   /* U+1AF6 */
  0x0000,   /* U+1AF7 */
  0x0000,   /* U+1AF8 */
  0x0000,   /* U+1AF9 */
  0x0000,   /* U+1AFA */
  0x0000,   /* U+1AFB */
  0x0000,   /* U+1AFC */
  0x0000,   /* U+1AFD */
  0x0000,   /* U+1AFE */
  0x0000,   /* U+1AFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x0000,   /* U+1A1C */
  0x0000,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x0020,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x0000,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x0000,   /* U+1A7D */
  0x0000,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x0000,   /* U+1A8A */
  0x0000,   /* U+1A8B */
  0x0000,   /* U+1A8C */
  0x0000,   /* U+1A8D */
  0x0000,   /* U+1A8E */
  0x0000,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x0000,   /* U+1A9A */
  0x0000,   /* U+1A9B */
  0x0000,   /* U+1A9C */
  0x0000,   /* U+1A9D */
  0x0000,   /* U+1A9E */
  0x0000,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x0000,   /* U+1AAE */
  0x0000,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x0000,   /* U+1ABF */
  0x0000,   /* U+1AC0 */
  0x0000,   /* U+1AC1 */
  0x0000,   /* U+1AC2 */
  0x0000,   /* U+1AC3 */
  0x0000,   /* U+1AC4 */
  0x0000,   /* U+1AC5 */
  0x0000,   /* U+1AC6 */
  0x0000,   /* U+1AC7 */
  0x0000,   /* U+1AC8 */
  0x0000,   /* U+1AC9 */
  0x0000,   /* U+1ACA */
  0x0000,   /* U+1ACB */
  0x0000,   /* U+1ACC */
  0x0000,   /* U+1ACD */
  0x0000,   /* U+1ACE */
  0x0000,   /* U+1ACF */
  0x0000,   /* U+1AD0 */
  0x0000,   /* U+1AD1 */
  0x0000,   /* U+1AD2 */
  0x0000,   /* U+1AD3 */
  0x0000,   /* U+1AD4 */
  0x0000,   /* U+1AD5 */
  0x0000,   /* U+1AD6 */
  0x0000,   /* U+1AD7 */
  0x0000,   /* U+1AD8 */
  0x0000,   /* U+1AD9 */
  0x0000,   /* U+1ADA */
  0x0000,   /* U+1ADB */
  0x0000,   /* U+1ADC */
  0x0000,   /* U+1ADD */
  0x0000,   /* U+1ADE */
  0x0000,   /* U+1ADF */
  0x0000,   /* U+1AE0 */
  0x0000,   /* U+1AE1 */
  0x0000,   /* U+1AE2 */
  0x0000,   /* U+1AE3 */
  0x0000,   /* U+1AE4 */
  0x0000,   /* U+1AE5 */
  0x0000,   /* U+1AE6 */
  0x0000,   /* U+1AE7 */
  0x0000,   /* U+1AE8 */
  0x0000,   /* U+1AE9 */
  0x0000,   /* U+1AEA */
  0x0000,   /* U+1AEB */
  0x0000,   /* U+1AEC */
  0x0000,   /* U+1AED */
  0x0000,   /* U+1AEE */
  0x0000,   /* U+1AEF */
  0x0000,   /* U+1AF0 */
  0x0000,   /* U+1AF1 */
  0x0000,   /* U+1AF2 */
  0x0000,   /* U+1AF3 */
  0x0000,   /* U+1AF4 */
  0x0000,   /* U+1AF5 */
  0x0000,   /* U+1AF6 */
  0x0000,   /* U+1AF7 */
  0x0000,   /* U+1AF8 */
  0x0000,   /* U+1AF9 */
  0x0000,   /* U+1AFA */
  0x0000,   /* U+1AFB */
  0x0000,   /* U+1AFC */
  0x0000,   /* U+1AFD */
  0x0000,   /* U+1AFE */
  0x0000,   /* U+1AFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1A00 */
  0x0000,   /* U+1A01 */
  0x0000,   /* U+1A02 */
  0x0000,   /* U+1A03 */
  0x0000,   /* U+1A04 */
  0x0000,   /* U+1A05 */
  0x0000,   /* U+1A06 */
  0x0000,   /* U+1A07 */
  0x0000,   /* U+1A08 */
  0x0000,   /* U+1A09 */
  0x0000,   /* U+1A0A */
  0x0000,   /* U+1A0B */
  0x0000,   /* U+1A0C */
  0x0000,   /* U+1A0D */
  0x0000,   /* U+1A0E */
  0x0000,   /* U+1A0F */
  0x0000,   /* U+1A10 */
  0x0000,   /* U+1A11 */
  0x0000,   /* U+1A12 */
  0x0000,   /* U+1A13 */
  0x0000,   /* U+1A14 */
  0x0000,   /* U+1A15 */
  0x0000,   /* U+1A16 */
  0x0000,   /* U+1A17 */
  0x0000,   /* U+1A18 */
  0x0000,   /* U+1A19 */
  0x0000,   /* U+1A1A */
  0x0000,   /* U+1A1B */
  0x0000,   /* U+1A1C */
  0x0000,   /* U+1A1D */
  0x0000,   /* U+1A1E */
  0x0000,   /* U+1A1F */
  0x0000,   /* U+1A20 */
  0x0000,   /* U+1A21 */
  0x0000,   /* U+1A22 */
  0x0000,   /* U+1A23 */
  0x0000,   /* U+1A24 */
  0x0000,   /* U+1A25 */
  0x0000,   /* U+1A26 */
  0x0000,   /* U+1A27 */
  0x0000,   /* U+1A28 */
  0x0000,   /* U+1A29 */
  0x0000,   /* U+1A2A */
  0x0000,   /* U+1A2B */
  0x0000,   /* U+1A2C */
  0x0000,   /* U+1A2D */
  0x0000,   /* U+1A2E */
  0x0000,   /* U+1A2F */
  0x0000,   /* U+1A30 */
  0x0000,   /* U+1A31 */
  0x0000,   /* U+1A32 */
  0x0000,   /* U+1A33 */
  0x0000,   /* U+1A34 */
  0x0000,   /* U+1A35 */
  0x0000,   /* U+1A36 */
  0x0000,   /* U+1A37 */
  0x0000,   /* U+1A38 */
  0x0000,   /* U+1A39 */
  0x0000,   /* U+1A3A */
  0x0000,   /* U+1A3B */
  0x0000,   /* U+1A3C */
  0x0000,   /* U+1A3D */
  0x0000,   /* U+1A3E */
  0x0000,   /* U+1A3F */
  0x0000,   /* U+1A40 */
  0x0000,   /* U+1A41 */
  0x0000,   /* U+1A42 */
  0x0000,   /* U+1A43 */
  0x0000,   /* U+1A44 */
  0x0000,   /* U+1A45 */
  0x0000,   /* U+1A46 */
  0x0000,   /* U+1A47 */
  0x0000,   /* U+1A48 */
  0x0000,   /* U+1A49 */
  0x0000,   /* U+1A4A */
  0x0000,   /* U+1A4B */
  0x0000,   /* U+1A4C */
  0x0000,   /* U+1A4D */
  0x0000,   /* U+1A4E */
  0x0000,   /* U+1A4F */
  0x0000,   /* U+1A50 */
  0x0000,   /* U+1A51 */
  0x0000,   /* U+1A52 */
  0x0000,   /* U+1A53 */
  0x0004,   /* U+1A54 */
  0x0000,   /* U+1A55 */
  0x0000,   /* U+1A56 */
  0x0000,   /* U+1A57 */
  0x0000,   /* U+1A58 */
  0x0000,   /* U+1A59 */
  0x0000,   /* U+1A5A */
  0x0000,   /* U+1A5B */
  0x0000,   /* U+1A5C */
  0x0000,   /* U+1A5D */
  0x0000,   /* U+1A5E */
  0x0000,   /* U+1A5F */
  0x0000,   /* U+1A60 */
  0x0000,   /* U+1A61 */
  0x0000,   /* U+1A62 */
  0x0000,   /* U+1A63 */
  0x0000,   /* U+1A64 */
  0x0000,   /* U+1A65 */
  0x0000,   /* U+1A66 */
  0x0000,   /* U+1A67 */
  0x0000,   /* U+1A68 */
  0x0000,   /* U+1A69 */
  0x0000,   /* U+1A6A */
  0x0000,   /* U+1A6B */
  0x0000,   /* U+1A6C */
  0x0000,   /* U+1A6D */
  0x0000,   /* U+1A6E */
  0x0000,   /* U+1A6F */
  0x0000,   /* U+1A70 */
  0x0000,   /* U+1A71 */
  0x0000,   /* U+1A72 */
  0x0000,   /* U+1A73 */
  0x0000,   /* U+1A74 */
  0x0000,   /* U+1A75 */
  0x0000,   /* U+1A76 */
  0x0000,   /* U+1A77 */
  0x0000,   /* U+1A78 */
  0x0000,   /* U+1A79 */
  0x0000,   /* U+1A7A */
  0x0000,   /* U+1A7B */
  0x0000,   /* U+1A7C */
  0x0000,   /* U+1A7D */
  0x0000,   /* U+1A7E */
  0x0000,   /* U+1A7F */
  0x0000,   /* U+1A80 */
  0x0000,   /* U+1A81 */
  0x0000,   /* U+1A82 */
  0x0000,   /* U+1A83 */
  0x0000,   /* U+1A84 */
  0x0000,   /* U+1A85 */
  0x0000,   /* U+1A86 */
  0x0000,   /* U+1A87 */
  0x0000,   /* U+1A88 */
  0x0000,   /* U+1A89 */
  0x0000,   /* U+1A8A */
  0x0000,   /* U+1A8B */
  0x0000,   /* U+1A8C */
  0x0000,   /* U+1A8D */
  0x0000,   /* U+1A8E */
  0x0000,   /* U+1A8F */
  0x0000,   /* U+1A90 */
  0x0000,   /* U+1A91 */
  0x0000,   /* U+1A92 */
  0x0000,   /* U+1A93 */
  0x0000,   /* U+1A94 */
  0x0000,   /* U+1A95 */
  0x0000,   /* U+1A96 */
  0x0000,   /* U+1A97 */
  0x0000,   /* U+1A98 */
  0x0000,   /* U+1A99 */
  0x0000,   /* U+1A9A */
  0x0000,   /* U+1A9B */
  0x0000,   /* U+1A9C */
  0x0000,   /* U+1A9D */
  0x0000,   /* U+1A9E */
  0x0000,   /* U+1A9F */
  0x0000,   /* U+1AA0 */
  0x0000,   /* U+1AA1 */
  0x0000,   /* U+1AA2 */
  0x0000,   /* U+1AA3 */
  0x0000,   /* U+1AA4 */
  0x0000,   /* U+1AA5 */
  0x0000,   /* U+1AA6 */
  0x0000,   /* U+1AA7 */
  0x0000,   /* U+1AA8 */
  0x0000,   /* U+1AA9 */
  0x0000,   /* U+1AAA */
  0x0000,   /* U+1AAB */
  0x0000,   /* U+1AAC */
  0x0000,   /* U+1AAD */
  0x0000,   /* U+1AAE */
  0x0000,   /* U+1AAF */
  0x0000,   /* U+1AB0 */
  0x0000,   /* U+1AB1 */
  0x0000,   /* U+1AB2 */
  0x0000,   /* U+1AB3 */
  0x0000,   /* U+1AB4 */
  0x0000,   /* U+1AB5 */
  0x0000,   /* U+1AB6 */
  0x0000,   /* U+1AB7 */
  0x0000,   /* U+1AB8 */
  0x0000,   /* U+1AB9 */
  0x0000,   /* U+1ABA */
  0x0000,   /* U+1ABB */
  0x0000,   /* U+1ABC */
  0x0000,   /* U+1ABD */
  0x0000,   /* U+1ABE */
  0x0000,   /* U+1ABF */
  0x0000,   /* U+1AC0 */
  0x0000,   /* U+1AC1 */
  0x0000,   /* U+1AC2 */
  0x0000,   /* U+1AC3 */
  0x0000,   /* U+1AC4 */
  0x0000,   /* U+1AC5 */
  0x0000,   /* U+1AC6 */
  0x0000,   /* U+1AC7 */
  0x0000,   /* U+1AC8 */
  0x0000,   /* U+1AC9 */
  0x0000,   /* U+1ACA */
  0x0000,   /* U+1ACB */
  0x0000,   /* U+1ACC */
  0x0000,   /* U+1ACD */
  0x0000,   /* U+1ACE */
  0x0000,   /* U+1ACF */
  0x0000,   /* U+1AD0 */
  0x0000,   /* U+1AD1 */
  0x0000,   /* U+1AD2 */
  0x0000,   /* U+1AD3 */
  0x0000,   /* U+1AD4 */
  0x0000,   /* U+1AD5 */
  0x0000,   /* U+1AD6 */
  0x0000,   /* U+1AD7 */
  0x0000,   /* U+1AD8 */
  0x0000,   /* U+1AD9 */
  0x0000,   /* U+1ADA */
  0x0000,   /* U+1ADB */
  0x0000,   /* U+1ADC */
  0x0000,   /* U+1ADD */
  0x0000,   /* U+1ADE */
  0x0000,   /* U+1ADF */
  0x0000,   /* U+1AE0 */
  0x0000,   /* U+1AE1 */
  0x0000,   /* U+1AE2 */
  0x0000,   /* U+1AE3 */
  0x0000,   /* U+1AE4 */
  0x0000,   /* U+1AE5 */
  0x0000,   /* U+1AE6 */
  0x0000,   /* U+1AE7 */
  0x0000,   /* U+1AE8 */
  0x0000,   /* U+1AE9 */
  0x0000,   /* U+1AEA */
  0x0000,   /* U+1AEB */
  0x0000,   /* U+1AEC */
  0x0000,   /* U+1AED */
  0x0000,   /* U+1AEE */
  0x0000,   /* U+1AEF */
  0x0000,   /* U+1AF0 */
  0x0000,   /* U+1AF1 */
  0x0000,   /* U+1AF2 */
  0x0000,   /* U+1AF3 */
  0x0000,   /* U+1AF4 */
  0x0000,   /* U+1AF5 */
  0x0000,   /* U+1AF6 */
  0x0000,   /* U+1AF7 */
  0x0000,   /* U+1AF8 */
  0x0000,   /* U+1AF9 */
  0x0000,   /* U+1AFA */
  0x0000,   /* U+1AFB */
  0x0000,   /* U+1AFC */
  0x0000,   /* U+1AFD */
  0x0000,   /* U+1AFE */
  0x0000,   /* U+1AFF */
};

uint16 uca900_p01B[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+1B00 */
  0x0000,   /* U+1B01 */
  0x0000,   /* U+1B02 */
  0x0000,   /* U+1B03 */
  0x0000,   /* U+1B04 */
  0x3257,   /* U+1B05 */
  0x3258,   /* U+1B06 */
  0x3259,   /* U+1B07 */
  0x325A,   /* U+1B08 */
  0x325B,   /* U+1B09 */
  0x325C,   /* U+1B0A */
  0x325D,   /* U+1B0B */
  0x325E,   /* U+1B0C */
  0x325F,   /* U+1B0D */
  0x3260,   /* U+1B0E */
  0x3261,   /* U+1B0F */
  0x3262,   /* U+1B10 */
  0x3263,   /* U+1B11 */
  0x3264,   /* U+1B12 */
  0x3265,   /* U+1B13 */
  0x3268,   /* U+1B14 */
  0x3269,   /* U+1B15 */
  0x326A,   /* U+1B16 */
  0x326B,   /* U+1B17 */
  0x326C,   /* U+1B18 */
  0x326D,   /* U+1B19 */
  0x326E,   /* U+1B1A */
  0x326F,   /* U+1B1B */
  0x3270,   /* U+1B1C */
  0x3271,   /* U+1B1D */
  0x3272,   /* U+1B1E */
  0x3273,   /* U+1B1F */
  0x3274,   /* U+1B20 */
  0x3275,   /* U+1B21 */
  0x3276,   /* U+1B22 */
  0x3278,   /* U+1B23 */
  0x3279,   /* U+1B24 */
  0x327A,   /* U+1B25 */
  0x327B,   /* U+1B26 */
  0x327C,   /* U+1B27 */
  0x327E,   /* U+1B28 */
  0x327F,   /* U+1B29 */
  0x3280,   /* U+1B2A */
  0x3281,   /* U+1B2B */
  0x3282,   /* U+1B2C */
  0x3283,   /* U+1B2D */
  0x3284,   /* U+1B2E */
  0x3285,   /* U+1B2F */
  0x3287,   /* U+1B30 */
  0x3288,   /* U+1B31 */
  0x3289,   /* U+1B32 */
  0x328C,   /* U+1B33 */
  0x0000,   /* U+1B34 */
  0x328D,   /* U+1B35 */
  0x328E,   /* U+1B36 */
  0x328F,   /* U+1B37 */
  0x3290,   /* U+1B38 */
  0x3291,   /* U+1B39 */
  0x3292,   /* U+1B3A */
  0x3293,   /* U+1B3B */
  0x3294,   /* U+1B3C */
  0x3295,   /* U+1B3D */
  0x3296,   /* U+1B3E */
  0x3297,   /* U+1B3F */
  0x3298,   /* U+1B40 */
  0x3299,   /* U+1B41 */
  0x329A,   /* U+1B42 */
  0x329B,   /* U+1B43 */
  0x329C,   /* U+1B44 */
  0x3266,   /* U+1B45 */
  0x3267,   /* U+1B46 */
  0x3277,   /* U+1B47 */
  0x327D,   /* U+1B48 */
  0x3286,   /* U+1B49 */
  0x328A,   /* U+1B4A */
  0x328B,   /* U+1B4B */
  0xFBC0,   /* U+1B4C */
  0xFBC0,   /* U+1B4D */
  0xFBC0,   /* U+1B4E */
  0xFBC0,   /* U+1B4F */
  0x1C3D,   /* U+1B50 */
  0x1C3E,   /* U+1B51 */
  0x1C3F,   /* U+1B52 */
  0x1C40,   /* U+1B53 */
  0x1C41,   /* U+1B54 */
  0x1C42,   /* U+1B55 */
  0x1C43,   /* U+1B56 */
  0x1C44,   /* U+1B57 */
  0x1C45,   /* U+1B58 */
  0x1C46,   /* U+1B59 */
  0x02CB,   /* U+1B5A */
  0x02CC,   /* U+1B5B */
  0x0280,   /* U+1B5C */
  0x025A,   /* U+1B5D */
  0x02A1,   /* U+1B5E */
  0x02A2,   /* U+1B5F */
  0x0210,   /* U+1B60 */
  0x0571,   /* U+1B61 */
  0x0572,   /* U+1B62 */
  0x0573,   /* U+1B63 */
  0x0574,   /* U+1B64 */
  0x0575,   /* U+1B65 */
  0x0576,   /* U+1B66 */
  0x0577,   /* U+1B67 */
  0x0578,   /* U+1B68 */
  0x0579,   /* U+1B69 */
  0x057A,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x057B,   /* U+1B74 */
  0x057C,   /* U+1B75 */
  0x057D,   /* U+1B76 */
  0x057E,   /* U+1B77 */
  0x057F,   /* U+1B78 */
  0x0580,   /* U+1B79 */
  0x0581,   /* U+1B7A */
  0x0582,   /* U+1B7B */
  0x0583,   /* U+1B7C */
  0xFBC0,   /* U+1B7D */
  0xFBC0,   /* U+1B7E */
  0xFBC0,   /* U+1B7F */
  0x0000,   /* U+1B80 */
  0x0000,   /* U+1B81 */
  0x0000,   /* U+1B82 */
  0x2C98,   /* U+1B83 */
  0x2C99,   /* U+1B84 */
  0x2C9A,   /* U+1B85 */
  0x2C9B,   /* U+1B86 */
  0x2C9C,   /* U+1B87 */
  0x2C9D,   /* U+1B88 */
  0x2C9E,   /* U+1B89 */
  0x2C9F,   /* U+1B8A */
  0x2CA1,   /* U+1B8B */
  0x2CA2,   /* U+1B8C */
  0x2CA3,   /* U+1B8D */
  0x2CA4,   /* U+1B8E */
  0x2CA5,   /* U+1B8F */
  0x2CA6,   /* U+1B90 */
  0x2CA7,   /* U+1B91 */
  0x2CA8,   /* U+1B92 */
  0x2CA9,   /* U+1B93 */
  0x2CAA,   /* U+1B94 */
  0x2CAB,   /* U+1B95 */
  0x2CAC,   /* U+1B96 */
  0x2CAD,   /* U+1B97 */
  0x2CAE,   /* U+1B98 */
  0x2CB0,   /* U+1B99 */
  0x2CB2,   /* U+1B9A */
  0x2CB4,   /* U+1B9B */
  0x2CB7,   /* U+1B9C */
  0x2CBA,   /* U+1B9D */
  0x2CBC,   /* U+1B9E */
  0x2CBD,   /* U+1B9F */
  0x2CBF,   /* U+1BA0 */
  0x2CB3,   /* U+1BA1 */
  0x2CB5,   /* U+1BA2 */
  0x2CB8,   /* U+1BA3 */
  0x2CC0,   /* U+1BA4 */
  0x2CC1,   /* U+1BA5 */
  0x2CC2,   /* U+1BA6 */
  0x2CC3,   /* U+1BA7 */
  0x2CC4,   /* U+1BA8 */
  0x2CC5,   /* U+1BA9 */
  0x2CC6,   /* U+1BAA */
  0x2CC7,   /* U+1BAB */
  0x2CB1,   /* U+1BAC */
  0x2CBB,   /* U+1BAD */
  0x2CA0,   /* U+1BAE */
  0x2CBE,   /* U+1BAF */
  0x1C3D,   /* U+1BB0 */
  0x1C3E,   /* U+1BB1 */
  0x1C3F,   /* U+1BB2 */
  0x1C40,   /* U+1BB3 */
  0x1C41,   /* U+1BB4 */
  0x1C42,   /* U+1BB5 */
  0x1C43,   /* U+1BB6 */
  0x1C44,   /* U+1BB7 */
  0x1C45,   /* U+1BB8 */
  0x1C46,   /* U+1BB9 */
  0x2C98,   /* U+1BBA */
  0x2CB6,   /* U+1BBB */
  0x2CB9,   /* U+1BBC */
  0x2CAF,   /* U+1BBD */
  0x2C9F,   /* U+1BBE */
  0x2CB0,   /* U+1BBF */
  0x2FD1,   /* U+1BC0 */
  0x2FD1,   /* U+1BC1 */
  0x2FD2,   /* U+1BC2 */
  0x2FD2,   /* U+1BC3 */
  0x2FD2,   /* U+1BC4 */
  0x2FD3,   /* U+1BC5 */
  0x2FD3,   /* U+1BC6 */
  0x2FD4,   /* U+1BC7 */
  0x2FD4,   /* U+1BC8 */
  0x2FD5,   /* U+1BC9 */
  0x2FD5,   /* U+1BCA */
  0x2FD6,   /* U+1BCB */
  0x2FD6,   /* U+1BCC */
  0x2FD6,   /* U+1BCD */
  0x2FD7,   /* U+1BCE */
  0x2FD7,   /* U+1BCF */
  0x2FD8,   /* U+1BD0 */
  0x2FD9,   /* U+1BD1 */
  0x2FDA,   /* U+1BD2 */
  0x2FDA,   /* U+1BD3 */
  0x2FDB,   /* U+1BD4 */
  0x2FDB,   /* U+1BD5 */
  0x2FDC,   /* U+1BD6 */
  0x2FDC,   /* U+1BD7 */
  0x2FDD,   /* U+1BD8 */
  0x2FDD,   /* U+1BD9 */
  0x2FDD,   /* U+1BDA */
  0x2FDE,   /* U+1BDB */
  0x2FDE,   /* U+1BDC */
  0x2FDF,   /* U+1BDD */
  0x2FE0,   /* U+1BDE */
  0x2FE0,   /* U+1BDF */
  0x2FE1,   /* U+1BE0 */
  0x2FE2,   /* U+1BE1 */
  0x2FE3,   /* U+1BE2 */
  0x2FE4,   /* U+1BE3 */
  0x2FE5,   /* U+1BE4 */
  0x2FE6,   /* U+1BE5 */
  0x0000,   /* U+1BE6 */
  0x2FE7,   /* U+1BE7 */
  0x2FE7,   /* U+1BE8 */
  0x2FE8,   /* U+1BE9 */
  0x2FE9,   /* U+1BEA */
  0x2FE9,   /* U+1BEB */
  0x2FEA,   /* U+1BEC */
  0x2FEA,   /* U+1BED */
  0x2FEB,   /* U+1BEE */
  0x2FEB,   /* U+1BEF */
  0x2FEC,   /* U+1BF0 */
  0x2FED,   /* U+1BF1 */
  0x2FEE,   /* U+1BF2 */
  0x2FEF,   /* U+1BF3 */
  0xFBC0,   /* U+1BF4 */
  0xFBC0,   /* U+1BF5 */
  0xFBC0,   /* U+1BF6 */
  0xFBC0,   /* U+1BF7 */
  0xFBC0,   /* U+1BF8 */
  0xFBC0,   /* U+1BF9 */
  0xFBC0,   /* U+1BFA */
  0xFBC0,   /* U+1BFB */
  0x042A,   /* U+1BFC */
  0x042B,   /* U+1BFD */
  0x042C,   /* U+1BFE */
  0x042D,   /* U+1BFF */

  /* Secondary weight 1 for each character. */
  0x00C3,   /* U+1B00 */
  0x00C3,   /* U+1B01 */
  0x00C4,   /* U+1B02 */
  0x00C8,   /* U+1B03 */
  0x00C5,   /* U+1B04 */
  0x0020,   /* U+1B05 */
  0x0020,   /* U+1B06 */
  0x0020,   /* U+1B07 */
  0x0020,   /* U+1B08 */
  0x0020,   /* U+1B09 */
  0x0020,   /* U+1B0A */
  0x0020,   /* U+1B0B */
  0x0020,   /* U+1B0C */
  0x0020,   /* U+1B0D */
  0x0020,   /* U+1B0E */
  0x0020,   /* U+1B0F */
  0x0020,   /* U+1B10 */
  0x0020,   /* U+1B11 */
  0x0020,   /* U+1B12 */
  0x0020,   /* U+1B13 */
  0x0020,   /* U+1B14 */
  0x0020,   /* U+1B15 */
  0x0020,   /* U+1B16 */
  0x0020,   /* U+1B17 */
  0x0020,   /* U+1B18 */
  0x0020,   /* U+1B19 */
  0x0020,   /* U+1B1A */
  0x0020,   /* U+1B1B */
  0x0020,   /* U+1B1C */
  0x0020,   /* U+1B1D */
  0x0020,   /* U+1B1E */
  0x0020,   /* U+1B1F */
  0x0020,   /* U+1B20 */
  0x0020,   /* U+1B21 */
  0x0020,   /* U+1B22 */
  0x0020,   /* U+1B23 */
  0x0020,   /* U+1B24 */
  0x0020,   /* U+1B25 */
  0x0020,   /* U+1B26 */
  0x0020,   /* U+1B27 */
  0x0020,   /* U+1B28 */
  0x0020,   /* U+1B29 */
  0x0020,   /* U+1B2A */
  0x0020,   /* U+1B2B */
  0x0020,   /* U+1B2C */
  0x0020,   /* U+1B2D */
  0x0020,   /* U+1B2E */
  0x0020,   /* U+1B2F */
  0x0020,   /* U+1B30 */
  0x0020,   /* U+1B31 */
  0x0020,   /* U+1B32 */
  0x0020,   /* U+1B33 */
  0x00C2,   /* U+1B34 */
  0x0020,   /* U+1B35 */
  0x0020,   /* U+1B36 */
  0x0020,   /* U+1B37 */
  0x0020,   /* U+1B38 */
  0x0020,   /* U+1B39 */
  0x0020,   /* U+1B3A */
  0x0020,   /* U+1B3B */
  0x0020,   /* U+1B3C */
  0x0020,   /* U+1B3D */
  0x0020,   /* U+1B3E */
  0x0020,   /* U+1B3F */
  0x0020,   /* U+1B40 */
  0x0020,   /* U+1B41 */
  0x0020,   /* U+1B42 */
  0x0020,   /* U+1B43 */
  0x0020,   /* U+1B44 */
  0x0020,   /* U+1B45 */
  0x0020,   /* U+1B46 */
  0x0020,   /* U+1B47 */
  0x0020,   /* U+1B48 */
  0x0020,   /* U+1B49 */
  0x0020,   /* U+1B4A */
  0x0020,   /* U+1B4B */
  0x0020,   /* U+1B4C */
  0x0020,   /* U+1B4D */
  0x0020,   /* U+1B4E */
  0x0020,   /* U+1B4F */
  0x0020,   /* U+1B50 */
  0x0020,   /* U+1B51 */
  0x0020,   /* U+1B52 */
  0x0020,   /* U+1B53 */
  0x0020,   /* U+1B54 */
  0x0020,   /* U+1B55 */
  0x0020,   /* U+1B56 */
  0x0020,   /* U+1B57 */
  0x0020,   /* U+1B58 */
  0x0020,   /* U+1B59 */
  0x0020,   /* U+1B5A */
  0x0020,   /* U+1B5B */
  0x0020,   /* U+1B5C */
  0x0020,   /* U+1B5D */
  0x0020,   /* U+1B5E */
  0x0020,   /* U+1B5F */
  0x0020,   /* U+1B60 */
  0x0020,   /* U+1B61 */
  0x0020,   /* U+1B62 */
  0x0020,   /* U+1B63 */
  0x0020,   /* U+1B64 */
  0x0020,   /* U+1B65 */
  0x0020,   /* U+1B66 */
  0x0020,   /* U+1B67 */
  0x0020,   /* U+1B68 */
  0x0020,   /* U+1B69 */
  0x0020,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x0020,   /* U+1B74 */
  0x0020,   /* U+1B75 */
  0x0020,   /* U+1B76 */
  0x0020,   /* U+1B77 */
  0x0020,   /* U+1B78 */
  0x0020,   /* U+1B79 */
  0x0020,   /* U+1B7A */
  0x0020,   /* U+1B7B */
  0x0020,   /* U+1B7C */
  0x0020,   /* U+1B7D */
  0x0020,   /* U+1B7E */
  0x0020,   /* U+1B7F */
  0x00C4,   /* U+1B80 */
  0x00CA,   /* U+1B81 */
  0x00C5,   /* U+1B82 */
  0x0020,   /* U+1B83 */
  0x0020,   /* U+1B84 */
  0x0020,   /* U+1B85 */
  0x0020,   /* U+1B86 */
  0x0020,   /* U+1B87 */
  0x0020,   /* U+1B88 */
  0x0020,   /* U+1B89 */
  0x0020,   /* U+1B8A */
  0x0020,   /* U+1B8B */
  0x0020,   /* U+1B8C */
  0x0020,   /* U+1B8D */
  0x0020,   /* U+1B8E */
  0x0020,   /* U+1B8F */
  0x0020,   /* U+1B90 */
  0x0020,   /* U+1B91 */
  0x0020,   /* U+1B92 */
  0x0020,   /* U+1B93 */
  0x0020,   /* U+1B94 */
  0x0020,   /* U+1B95 */
  0x0020,   /* U+1B96 */
  0x0020,   /* U+1B97 */
  0x0020,   /* U+1B98 */
  0x0020,   /* U+1B99 */
  0x0020,   /* U+1B9A */
  0x0020,   /* U+1B9B */
  0x0020,   /* U+1B9C */
  0x0020,   /* U+1B9D */
  0x0020,   /* U+1B9E */
  0x0020,   /* U+1B9F */
  0x0020,   /* U+1BA0 */
  0x0020,   /* U+1BA1 */
  0x0020,   /* U+1BA2 */
  0x0020,   /* U+1BA3 */
  0x0020,   /* U+1BA4 */
  0x0020,   /* U+1BA5 */
  0x0020,   /* U+1BA6 */
  0x0020,   /* U+1BA7 */
  0x0020,   /* U+1BA8 */
  0x0020,   /* U+1BA9 */
  0x0020,   /* U+1BAA */
  0x0020,   /* U+1BAB */
  0x0020,   /* U+1BAC */
  0x0020,   /* U+1BAD */
  0x0020,   /* U+1BAE */
  0x0020,   /* U+1BAF */
  0x0020,   /* U+1BB0 */
  0x0020,   /* U+1BB1 */
  0x0020,   /* U+1BB2 */
  0x0020,   /* U+1BB3 */
  0x0020,   /* U+1BB4 */
  0x0020,   /* U+1BB5 */
  0x0020,   /* U+1BB6 */
  0x0020,   /* U+1BB7 */
  0x0020,   /* U+1BB8 */
  0x0020,   /* U+1BB9 */
  0x0020,   /* U+1BBA */
  0x0020,   /* U+1BBB */
  0x0020,   /* U+1BBC */
  0x0020,   /* U+1BBD */
  0x0020,   /* U+1BBE */
  0x0020,   /* U+1BBF */
  0x0020,   /* U+1BC0 */
  0x0020,   /* U+1BC1 */
  0x0020,   /* U+1BC2 */
  0x0020,   /* U+1BC3 */
  0x0020,   /* U+1BC4 */
  0x0020,   /* U+1BC5 */
  0x0020,   /* U+1BC6 */
  0x0020,   /* U+1BC7 */
  0x0020,   /* U+1BC8 */
  0x0020,   /* U+1BC9 */
  0x0020,   /* U+1BCA */
  0x0020,   /* U+1BCB */
  0x0020,   /* U+1BCC */
  0x0020,   /* U+1BCD */
  0x0020,   /* U+1BCE */
  0x0020,   /* U+1BCF */
  0x0020,   /* U+1BD0 */
  0x0020,   /* U+1BD1 */
  0x0020,   /* U+1BD2 */
  0x0020,   /* U+1BD3 */
  0x0020,   /* U+1BD4 */
  0x0020,   /* U+1BD5 */
  0x0020,   /* U+1BD6 */
  0x0020,   /* U+1BD7 */
  0x0020,   /* U+1BD8 */
  0x0020,   /* U+1BD9 */
  0x0020,   /* U+1BDA */
  0x0020,   /* U+1BDB */
  0x0020,   /* U+1BDC */
  0x0020,   /* U+1BDD */
  0x0020,   /* U+1BDE */
  0x0020,   /* U+1BDF */
  0x0020,   /* U+1BE0 */
  0x0020,   /* U+1BE1 */
  0x0020,   /* U+1BE2 */
  0x0020,   /* U+1BE3 */
  0x0020,   /* U+1BE4 */
  0x0020,   /* U+1BE5 */
  0x00C2,   /* U+1BE6 */
  0x0020,   /* U+1BE7 */
  0x0020,   /* U+1BE8 */
  0x0020,   /* U+1BE9 */
  0x0020,   /* U+1BEA */
  0x0020,   /* U+1BEB */
  0x0020,   /* U+1BEC */
  0x0020,   /* U+1BED */
  0x0020,   /* U+1BEE */
  0x0020,   /* U+1BEF */
  0x0020,   /* U+1BF0 */
  0x0020,   /* U+1BF1 */
  0x0020,   /* U+1BF2 */
  0x0020,   /* U+1BF3 */
  0x0020,   /* U+1BF4 */
  0x0020,   /* U+1BF5 */
  0x0020,   /* U+1BF6 */
  0x0020,   /* U+1BF7 */
  0x0020,   /* U+1BF8 */
  0x0020,   /* U+1BF9 */
  0x0020,   /* U+1BFA */
  0x0020,   /* U+1BFB */
  0x0020,   /* U+1BFC */
  0x0020,   /* U+1BFD */
  0x0020,   /* U+1BFE */
  0x0020,   /* U+1BFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1B00 */
  0x0002,   /* U+1B01 */
  0x0002,   /* U+1B02 */
  0x0002,   /* U+1B03 */
  0x0002,   /* U+1B04 */
  0x0002,   /* U+1B05 */
  0x0002,   /* U+1B06 */
  0x0002,   /* U+1B07 */
  0x0002,   /* U+1B08 */
  0x0002,   /* U+1B09 */
  0x0002,   /* U+1B0A */
  0x0002,   /* U+1B0B */
  0x0002,   /* U+1B0C */
  0x0002,   /* U+1B0D */
  0x0002,   /* U+1B0E */
  0x0002,   /* U+1B0F */
  0x0002,   /* U+1B10 */
  0x0002,   /* U+1B11 */
  0x0002,   /* U+1B12 */
  0x0002,   /* U+1B13 */
  0x0002,   /* U+1B14 */
  0x0002,   /* U+1B15 */
  0x0002,   /* U+1B16 */
  0x0002,   /* U+1B17 */
  0x0002,   /* U+1B18 */
  0x0002,   /* U+1B19 */
  0x0002,   /* U+1B1A */
  0x0002,   /* U+1B1B */
  0x0002,   /* U+1B1C */
  0x0002,   /* U+1B1D */
  0x0002,   /* U+1B1E */
  0x0002,   /* U+1B1F */
  0x0002,   /* U+1B20 */
  0x0002,   /* U+1B21 */
  0x0002,   /* U+1B22 */
  0x0002,   /* U+1B23 */
  0x0002,   /* U+1B24 */
  0x0002,   /* U+1B25 */
  0x0002,   /* U+1B26 */
  0x0002,   /* U+1B27 */
  0x0002,   /* U+1B28 */
  0x0002,   /* U+1B29 */
  0x0002,   /* U+1B2A */
  0x0002,   /* U+1B2B */
  0x0002,   /* U+1B2C */
  0x0002,   /* U+1B2D */
  0x0002,   /* U+1B2E */
  0x0002,   /* U+1B2F */
  0x0002,   /* U+1B30 */
  0x0002,   /* U+1B31 */
  0x0002,   /* U+1B32 */
  0x0002,   /* U+1B33 */
  0x0002,   /* U+1B34 */
  0x0002,   /* U+1B35 */
  0x0002,   /* U+1B36 */
  0x0002,   /* U+1B37 */
  0x0002,   /* U+1B38 */
  0x0002,   /* U+1B39 */
  0x0002,   /* U+1B3A */
  0x0002,   /* U+1B3B */
  0x0002,   /* U+1B3C */
  0x0002,   /* U+1B3D */
  0x0002,   /* U+1B3E */
  0x0002,   /* U+1B3F */
  0x0002,   /* U+1B40 */
  0x0002,   /* U+1B41 */
  0x0002,   /* U+1B42 */
  0x0002,   /* U+1B43 */
  0x0002,   /* U+1B44 */
  0x0002,   /* U+1B45 */
  0x0002,   /* U+1B46 */
  0x0002,   /* U+1B47 */
  0x0002,   /* U+1B48 */
  0x0002,   /* U+1B49 */
  0x0002,   /* U+1B4A */
  0x0002,   /* U+1B4B */
  0x0002,   /* U+1B4C */
  0x0002,   /* U+1B4D */
  0x0002,   /* U+1B4E */
  0x0002,   /* U+1B4F */
  0x0002,   /* U+1B50 */
  0x0002,   /* U+1B51 */
  0x0002,   /* U+1B52 */
  0x0002,   /* U+1B53 */
  0x0002,   /* U+1B54 */
  0x0002,   /* U+1B55 */
  0x0002,   /* U+1B56 */
  0x0002,   /* U+1B57 */
  0x0002,   /* U+1B58 */
  0x0002,   /* U+1B59 */
  0x0002,   /* U+1B5A */
  0x0002,   /* U+1B5B */
  0x0002,   /* U+1B5C */
  0x0002,   /* U+1B5D */
  0x0002,   /* U+1B5E */
  0x0002,   /* U+1B5F */
  0x0002,   /* U+1B60 */
  0x0002,   /* U+1B61 */
  0x0002,   /* U+1B62 */
  0x0002,   /* U+1B63 */
  0x0002,   /* U+1B64 */
  0x0002,   /* U+1B65 */
  0x0002,   /* U+1B66 */
  0x0002,   /* U+1B67 */
  0x0002,   /* U+1B68 */
  0x0002,   /* U+1B69 */
  0x0002,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x0002,   /* U+1B74 */
  0x0002,   /* U+1B75 */
  0x0002,   /* U+1B76 */
  0x0002,   /* U+1B77 */
  0x0002,   /* U+1B78 */
  0x0002,   /* U+1B79 */
  0x0002,   /* U+1B7A */
  0x0002,   /* U+1B7B */
  0x0002,   /* U+1B7C */
  0x0002,   /* U+1B7D */
  0x0002,   /* U+1B7E */
  0x0002,   /* U+1B7F */
  0x0002,   /* U+1B80 */
  0x0002,   /* U+1B81 */
  0x0002,   /* U+1B82 */
  0x0002,   /* U+1B83 */
  0x0002,   /* U+1B84 */
  0x0002,   /* U+1B85 */
  0x0002,   /* U+1B86 */
  0x0002,   /* U+1B87 */
  0x0002,   /* U+1B88 */
  0x0002,   /* U+1B89 */
  0x0002,   /* U+1B8A */
  0x0002,   /* U+1B8B */
  0x0002,   /* U+1B8C */
  0x0002,   /* U+1B8D */
  0x0002,   /* U+1B8E */
  0x0002,   /* U+1B8F */
  0x0002,   /* U+1B90 */
  0x0002,   /* U+1B91 */
  0x0002,   /* U+1B92 */
  0x0002,   /* U+1B93 */
  0x0002,   /* U+1B94 */
  0x0002,   /* U+1B95 */
  0x0002,   /* U+1B96 */
  0x0002,   /* U+1B97 */
  0x0002,   /* U+1B98 */
  0x0002,   /* U+1B99 */
  0x0002,   /* U+1B9A */
  0x0002,   /* U+1B9B */
  0x0002,   /* U+1B9C */
  0x0002,   /* U+1B9D */
  0x0002,   /* U+1B9E */
  0x0002,   /* U+1B9F */
  0x0002,   /* U+1BA0 */
  0x0002,   /* U+1BA1 */
  0x0002,   /* U+1BA2 */
  0x0002,   /* U+1BA3 */
  0x0002,   /* U+1BA4 */
  0x0002,   /* U+1BA5 */
  0x0002,   /* U+1BA6 */
  0x0002,   /* U+1BA7 */
  0x0002,   /* U+1BA8 */
  0x0002,   /* U+1BA9 */
  0x0002,   /* U+1BAA */
  0x0002,   /* U+1BAB */
  0x0002,   /* U+1BAC */
  0x0002,   /* U+1BAD */
  0x0002,   /* U+1BAE */
  0x0002,   /* U+1BAF */
  0x0002,   /* U+1BB0 */
  0x0002,   /* U+1BB1 */
  0x0002,   /* U+1BB2 */
  0x0002,   /* U+1BB3 */
  0x0002,   /* U+1BB4 */
  0x0002,   /* U+1BB5 */
  0x0002,   /* U+1BB6 */
  0x0002,   /* U+1BB7 */
  0x0002,   /* U+1BB8 */
  0x0002,   /* U+1BB9 */
  0x0004,   /* U+1BBA */
  0x0002,   /* U+1BBB */
  0x0002,   /* U+1BBC */
  0x0002,   /* U+1BBD */
  0x0019,   /* U+1BBE */
  0x0019,   /* U+1BBF */
  0x0002,   /* U+1BC0 */
  0x0004,   /* U+1BC1 */
  0x0002,   /* U+1BC2 */
  0x0004,   /* U+1BC3 */
  0x0004,   /* U+1BC4 */
  0x0002,   /* U+1BC5 */
  0x0004,   /* U+1BC6 */
  0x0002,   /* U+1BC7 */
  0x0004,   /* U+1BC8 */
  0x0002,   /* U+1BC9 */
  0x0004,   /* U+1BCA */
  0x0002,   /* U+1BCB */
  0x0004,   /* U+1BCC */
  0x0004,   /* U+1BCD */
  0x0002,   /* U+1BCE */
  0x0004,   /* U+1BCF */
  0x0002,   /* U+1BD0 */
  0x0002,   /* U+1BD1 */
  0x0002,   /* U+1BD2 */
  0x0004,   /* U+1BD3 */
  0x0002,   /* U+1BD4 */
  0x0004,   /* U+1BD5 */
  0x0002,   /* U+1BD6 */
  0x0004,   /* U+1BD7 */
  0x0002,   /* U+1BD8 */
  0x0004,   /* U+1BD9 */
  0x0004,   /* U+1BDA */
  0x0002,   /* U+1BDB */
  0x0004,   /* U+1BDC */
  0x0002,   /* U+1BDD */
  0x0002,   /* U+1BDE */
  0x0004,   /* U+1BDF */
  0x0002,   /* U+1BE0 */
  0x0002,   /* U+1BE1 */
  0x0002,   /* U+1BE2 */
  0x0002,   /* U+1BE3 */
  0x0002,   /* U+1BE4 */
  0x0002,   /* U+1BE5 */
  0x0002,   /* U+1BE6 */
  0x0002,   /* U+1BE7 */
  0x0004,   /* U+1BE8 */
  0x0002,   /* U+1BE9 */
  0x0002,   /* U+1BEA */
  0x0004,   /* U+1BEB */
  0x0002,   /* U+1BEC */
  0x0004,   /* U+1BED */
  0x0002,   /* U+1BEE */
  0x0004,   /* U+1BEF */
  0x0002,   /* U+1BF0 */
  0x0002,   /* U+1BF1 */
  0x0002,   /* U+1BF2 */
  0x0002,   /* U+1BF3 */
  0x0002,   /* U+1BF4 */
  0x0002,   /* U+1BF5 */
  0x0002,   /* U+1BF6 */
  0x0002,   /* U+1BF7 */
  0x0002,   /* U+1BF8 */
  0x0002,   /* U+1BF9 */
  0x0002,   /* U+1BFA */
  0x0002,   /* U+1BFB */
  0x0002,   /* U+1BFC */
  0x0002,   /* U+1BFD */
  0x0002,   /* U+1BFE */
  0x0002,   /* U+1BFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1B00 */
  0x0000,   /* U+1B01 */
  0x0000,   /* U+1B02 */
  0x0000,   /* U+1B03 */
  0x0000,   /* U+1B04 */
  0x0000,   /* U+1B05 */
  0x0000,   /* U+1B06 */
  0x0000,   /* U+1B07 */
  0x0000,   /* U+1B08 */
  0x0000,   /* U+1B09 */
  0x0000,   /* U+1B0A */
  0x0000,   /* U+1B0B */
  0x0000,   /* U+1B0C */
  0x0000,   /* U+1B0D */
  0x0000,   /* U+1B0E */
  0x0000,   /* U+1B0F */
  0x0000,   /* U+1B10 */
  0x0000,   /* U+1B11 */
  0x0000,   /* U+1B12 */
  0x0000,   /* U+1B13 */
  0x0000,   /* U+1B14 */
  0x0000,   /* U+1B15 */
  0x0000,   /* U+1B16 */
  0x0000,   /* U+1B17 */
  0x0000,   /* U+1B18 */
  0x0000,   /* U+1B19 */
  0x0000,   /* U+1B1A */
  0x0000,   /* U+1B1B */
  0x0000,   /* U+1B1C */
  0x0000,   /* U+1B1D */
  0x0000,   /* U+1B1E */
  0x0000,   /* U+1B1F */
  0x0000,   /* U+1B20 */
  0x0000,   /* U+1B21 */
  0x0000,   /* U+1B22 */
  0x0000,   /* U+1B23 */
  0x0000,   /* U+1B24 */
  0x0000,   /* U+1B25 */
  0x0000,   /* U+1B26 */
  0x0000,   /* U+1B27 */
  0x0000,   /* U+1B28 */
  0x0000,   /* U+1B29 */
  0x0000,   /* U+1B2A */
  0x0000,   /* U+1B2B */
  0x0000,   /* U+1B2C */
  0x0000,   /* U+1B2D */
  0x0000,   /* U+1B2E */
  0x0000,   /* U+1B2F */
  0x0000,   /* U+1B30 */
  0x0000,   /* U+1B31 */
  0x0000,   /* U+1B32 */
  0x0000,   /* U+1B33 */
  0x0000,   /* U+1B34 */
  0x0000,   /* U+1B35 */
  0x0000,   /* U+1B36 */
  0x0000,   /* U+1B37 */
  0x0000,   /* U+1B38 */
  0x0000,   /* U+1B39 */
  0x0000,   /* U+1B3A */
  0x0000,   /* U+1B3B */
  0x0000,   /* U+1B3C */
  0x0000,   /* U+1B3D */
  0x0000,   /* U+1B3E */
  0x0000,   /* U+1B3F */
  0x0000,   /* U+1B40 */
  0x0000,   /* U+1B41 */
  0x0000,   /* U+1B42 */
  0x0000,   /* U+1B43 */
  0x0000,   /* U+1B44 */
  0x0000,   /* U+1B45 */
  0x0000,   /* U+1B46 */
  0x0000,   /* U+1B47 */
  0x0000,   /* U+1B48 */
  0x0000,   /* U+1B49 */
  0x0000,   /* U+1B4A */
  0x0000,   /* U+1B4B */
  0x9B4C,   /* U+1B4C */
  0x9B4D,   /* U+1B4D */
  0x9B4E,   /* U+1B4E */
  0x9B4F,   /* U+1B4F */
  0x0000,   /* U+1B50 */
  0x0000,   /* U+1B51 */
  0x0000,   /* U+1B52 */
  0x0000,   /* U+1B53 */
  0x0000,   /* U+1B54 */
  0x0000,   /* U+1B55 */
  0x0000,   /* U+1B56 */
  0x0000,   /* U+1B57 */
  0x0000,   /* U+1B58 */
  0x0000,   /* U+1B59 */
  0x0000,   /* U+1B5A */
  0x0000,   /* U+1B5B */
  0x0000,   /* U+1B5C */
  0x0000,   /* U+1B5D */
  0x0000,   /* U+1B5E */
  0x0000,   /* U+1B5F */
  0x0000,   /* U+1B60 */
  0x0000,   /* U+1B61 */
  0x0000,   /* U+1B62 */
  0x0000,   /* U+1B63 */
  0x0000,   /* U+1B64 */
  0x0000,   /* U+1B65 */
  0x0000,   /* U+1B66 */
  0x0000,   /* U+1B67 */
  0x0000,   /* U+1B68 */
  0x0000,   /* U+1B69 */
  0x0000,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x0000,   /* U+1B74 */
  0x0000,   /* U+1B75 */
  0x0000,   /* U+1B76 */
  0x0000,   /* U+1B77 */
  0x0000,   /* U+1B78 */
  0x0000,   /* U+1B79 */
  0x0000,   /* U+1B7A */
  0x0000,   /* U+1B7B */
  0x0000,   /* U+1B7C */
  0x9B7D,   /* U+1B7D */
  0x9B7E,   /* U+1B7E */
  0x9B7F,   /* U+1B7F */
  0x0000,   /* U+1B80 */
  0x0000,   /* U+1B81 */
  0x0000,   /* U+1B82 */
  0x0000,   /* U+1B83 */
  0x0000,   /* U+1B84 */
  0x0000,   /* U+1B85 */
  0x0000,   /* U+1B86 */
  0x0000,   /* U+1B87 */
  0x0000,   /* U+1B88 */
  0x0000,   /* U+1B89 */
  0x0000,   /* U+1B8A */
  0x0000,   /* U+1B8B */
  0x0000,   /* U+1B8C */
  0x0000,   /* U+1B8D */
  0x0000,   /* U+1B8E */
  0x0000,   /* U+1B8F */
  0x0000,   /* U+1B90 */
  0x0000,   /* U+1B91 */
  0x0000,   /* U+1B92 */
  0x0000,   /* U+1B93 */
  0x0000,   /* U+1B94 */
  0x0000,   /* U+1B95 */
  0x0000,   /* U+1B96 */
  0x0000,   /* U+1B97 */
  0x0000,   /* U+1B98 */
  0x0000,   /* U+1B99 */
  0x0000,   /* U+1B9A */
  0x0000,   /* U+1B9B */
  0x0000,   /* U+1B9C */
  0x0000,   /* U+1B9D */
  0x0000,   /* U+1B9E */
  0x0000,   /* U+1B9F */
  0x0000,   /* U+1BA0 */
  0x0000,   /* U+1BA1 */
  0x0000,   /* U+1BA2 */
  0x0000,   /* U+1BA3 */
  0x0000,   /* U+1BA4 */
  0x0000,   /* U+1BA5 */
  0x0000,   /* U+1BA6 */
  0x0000,   /* U+1BA7 */
  0x0000,   /* U+1BA8 */
  0x0000,   /* U+1BA9 */
  0x0000,   /* U+1BAA */
  0x0000,   /* U+1BAB */
  0x0000,   /* U+1BAC */
  0x0000,   /* U+1BAD */
  0x0000,   /* U+1BAE */
  0x0000,   /* U+1BAF */
  0x0000,   /* U+1BB0 */
  0x0000,   /* U+1BB1 */
  0x0000,   /* U+1BB2 */
  0x0000,   /* U+1BB3 */
  0x0000,   /* U+1BB4 */
  0x0000,   /* U+1BB5 */
  0x0000,   /* U+1BB6 */
  0x0000,   /* U+1BB7 */
  0x0000,   /* U+1BB8 */
  0x0000,   /* U+1BB9 */
  0x0000,   /* U+1BBA */
  0x0000,   /* U+1BBB */
  0x0000,   /* U+1BBC */
  0x0000,   /* U+1BBD */
  0x0000,   /* U+1BBE */
  0x0000,   /* U+1BBF */
  0x0000,   /* U+1BC0 */
  0x0000,   /* U+1BC1 */
  0x0000,   /* U+1BC2 */
  0x0000,   /* U+1BC3 */
  0x0000,   /* U+1BC4 */
  0x0000,   /* U+1BC5 */
  0x0000,   /* U+1BC6 */
  0x0000,   /* U+1BC7 */
  0x0000,   /* U+1BC8 */
  0x0000,   /* U+1BC9 */
  0x0000,   /* U+1BCA */
  0x0000,   /* U+1BCB */
  0x0000,   /* U+1BCC */
  0x0000,   /* U+1BCD */
  0x0000,   /* U+1BCE */
  0x0000,   /* U+1BCF */
  0x0000,   /* U+1BD0 */
  0x0000,   /* U+1BD1 */
  0x0000,   /* U+1BD2 */
  0x0000,   /* U+1BD3 */
  0x0000,   /* U+1BD4 */
  0x0000,   /* U+1BD5 */
  0x0000,   /* U+1BD6 */
  0x0000,   /* U+1BD7 */
  0x0000,   /* U+1BD8 */
  0x0000,   /* U+1BD9 */
  0x0000,   /* U+1BDA */
  0x0000,   /* U+1BDB */
  0x0000,   /* U+1BDC */
  0x0000,   /* U+1BDD */
  0x0000,   /* U+1BDE */
  0x0000,   /* U+1BDF */
  0x0000,   /* U+1BE0 */
  0x0000,   /* U+1BE1 */
  0x0000,   /* U+1BE2 */
  0x0000,   /* U+1BE3 */
  0x0000,   /* U+1BE4 */
  0x0000,   /* U+1BE5 */
  0x0000,   /* U+1BE6 */
  0x0000,   /* U+1BE7 */
  0x0000,   /* U+1BE8 */
  0x0000,   /* U+1BE9 */
  0x0000,   /* U+1BEA */
  0x0000,   /* U+1BEB */
  0x0000,   /* U+1BEC */
  0x0000,   /* U+1BED */
  0x0000,   /* U+1BEE */
  0x0000,   /* U+1BEF */
  0x0000,   /* U+1BF0 */
  0x0000,   /* U+1BF1 */
  0x0000,   /* U+1BF2 */
  0x0000,   /* U+1BF3 */
  0x9BF4,   /* U+1BF4 */
  0x9BF5,   /* U+1BF5 */
  0x9BF6,   /* U+1BF6 */
  0x9BF7,   /* U+1BF7 */
  0x9BF8,   /* U+1BF8 */
  0x9BF9,   /* U+1BF9 */
  0x9BFA,   /* U+1BFA */
  0x9BFB,   /* U+1BFB */
  0x0000,   /* U+1BFC */
  0x0000,   /* U+1BFD */
  0x0000,   /* U+1BFE */
  0x0000,   /* U+1BFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1B00 */
  0x0000,   /* U+1B01 */
  0x0000,   /* U+1B02 */
  0x0000,   /* U+1B03 */
  0x0000,   /* U+1B04 */
  0x0000,   /* U+1B05 */
  0x0000,   /* U+1B06 */
  0x0000,   /* U+1B07 */
  0x0000,   /* U+1B08 */
  0x0000,   /* U+1B09 */
  0x0000,   /* U+1B0A */
  0x0000,   /* U+1B0B */
  0x0000,   /* U+1B0C */
  0x0000,   /* U+1B0D */
  0x0000,   /* U+1B0E */
  0x0000,   /* U+1B0F */
  0x0000,   /* U+1B10 */
  0x0000,   /* U+1B11 */
  0x0000,   /* U+1B12 */
  0x0000,   /* U+1B13 */
  0x0000,   /* U+1B14 */
  0x0000,   /* U+1B15 */
  0x0000,   /* U+1B16 */
  0x0000,   /* U+1B17 */
  0x0000,   /* U+1B18 */
  0x0000,   /* U+1B19 */
  0x0000,   /* U+1B1A */
  0x0000,   /* U+1B1B */
  0x0000,   /* U+1B1C */
  0x0000,   /* U+1B1D */
  0x0000,   /* U+1B1E */
  0x0000,   /* U+1B1F */
  0x0000,   /* U+1B20 */
  0x0000,   /* U+1B21 */
  0x0000,   /* U+1B22 */
  0x0000,   /* U+1B23 */
  0x0000,   /* U+1B24 */
  0x0000,   /* U+1B25 */
  0x0000,   /* U+1B26 */
  0x0000,   /* U+1B27 */
  0x0000,   /* U+1B28 */
  0x0000,   /* U+1B29 */
  0x0000,   /* U+1B2A */
  0x0000,   /* U+1B2B */
  0x0000,   /* U+1B2C */
  0x0000,   /* U+1B2D */
  0x0000,   /* U+1B2E */
  0x0000,   /* U+1B2F */
  0x0000,   /* U+1B30 */
  0x0000,   /* U+1B31 */
  0x0000,   /* U+1B32 */
  0x0000,   /* U+1B33 */
  0x0000,   /* U+1B34 */
  0x0000,   /* U+1B35 */
  0x0000,   /* U+1B36 */
  0x0000,   /* U+1B37 */
  0x0000,   /* U+1B38 */
  0x0000,   /* U+1B39 */
  0x0000,   /* U+1B3A */
  0x0000,   /* U+1B3B */
  0x0000,   /* U+1B3C */
  0x0000,   /* U+1B3D */
  0x0000,   /* U+1B3E */
  0x0000,   /* U+1B3F */
  0x0000,   /* U+1B40 */
  0x0000,   /* U+1B41 */
  0x0000,   /* U+1B42 */
  0x0000,   /* U+1B43 */
  0x0000,   /* U+1B44 */
  0x0000,   /* U+1B45 */
  0x0000,   /* U+1B46 */
  0x0000,   /* U+1B47 */
  0x0000,   /* U+1B48 */
  0x0000,   /* U+1B49 */
  0x0000,   /* U+1B4A */
  0x0000,   /* U+1B4B */
  0x0000,   /* U+1B4C */
  0x0000,   /* U+1B4D */
  0x0000,   /* U+1B4E */
  0x0000,   /* U+1B4F */
  0x0000,   /* U+1B50 */
  0x0000,   /* U+1B51 */
  0x0000,   /* U+1B52 */
  0x0000,   /* U+1B53 */
  0x0000,   /* U+1B54 */
  0x0000,   /* U+1B55 */
  0x0000,   /* U+1B56 */
  0x0000,   /* U+1B57 */
  0x0000,   /* U+1B58 */
  0x0000,   /* U+1B59 */
  0x0000,   /* U+1B5A */
  0x0000,   /* U+1B5B */
  0x0000,   /* U+1B5C */
  0x0000,   /* U+1B5D */
  0x0000,   /* U+1B5E */
  0x0000,   /* U+1B5F */
  0x0000,   /* U+1B60 */
  0x0000,   /* U+1B61 */
  0x0000,   /* U+1B62 */
  0x0000,   /* U+1B63 */
  0x0000,   /* U+1B64 */
  0x0000,   /* U+1B65 */
  0x0000,   /* U+1B66 */
  0x0000,   /* U+1B67 */
  0x0000,   /* U+1B68 */
  0x0000,   /* U+1B69 */
  0x0000,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x0000,   /* U+1B74 */
  0x0000,   /* U+1B75 */
  0x0000,   /* U+1B76 */
  0x0000,   /* U+1B77 */
  0x0000,   /* U+1B78 */
  0x0000,   /* U+1B79 */
  0x0000,   /* U+1B7A */
  0x0000,   /* U+1B7B */
  0x0000,   /* U+1B7C */
  0x0000,   /* U+1B7D */
  0x0000,   /* U+1B7E */
  0x0000,   /* U+1B7F */
  0x0000,   /* U+1B80 */
  0x0000,   /* U+1B81 */
  0x0000,   /* U+1B82 */
  0x0000,   /* U+1B83 */
  0x0000,   /* U+1B84 */
  0x0000,   /* U+1B85 */
  0x0000,   /* U+1B86 */
  0x0000,   /* U+1B87 */
  0x0000,   /* U+1B88 */
  0x0000,   /* U+1B89 */
  0x0000,   /* U+1B8A */
  0x0000,   /* U+1B8B */
  0x0000,   /* U+1B8C */
  0x0000,   /* U+1B8D */
  0x0000,   /* U+1B8E */
  0x0000,   /* U+1B8F */
  0x0000,   /* U+1B90 */
  0x0000,   /* U+1B91 */
  0x0000,   /* U+1B92 */
  0x0000,   /* U+1B93 */
  0x0000,   /* U+1B94 */
  0x0000,   /* U+1B95 */
  0x0000,   /* U+1B96 */
  0x0000,   /* U+1B97 */
  0x0000,   /* U+1B98 */
  0x0000,   /* U+1B99 */
  0x0000,   /* U+1B9A */
  0x0000,   /* U+1B9B */
  0x0000,   /* U+1B9C */
  0x0000,   /* U+1B9D */
  0x0000,   /* U+1B9E */
  0x0000,   /* U+1B9F */
  0x0000,   /* U+1BA0 */
  0x0000,   /* U+1BA1 */
  0x0000,   /* U+1BA2 */
  0x0000,   /* U+1BA3 */
  0x0000,   /* U+1BA4 */
  0x0000,   /* U+1BA5 */
  0x0000,   /* U+1BA6 */
  0x0000,   /* U+1BA7 */
  0x0000,   /* U+1BA8 */
  0x0000,   /* U+1BA9 */
  0x0000,   /* U+1BAA */
  0x0000,   /* U+1BAB */
  0x0000,   /* U+1BAC */
  0x0000,   /* U+1BAD */
  0x0000,   /* U+1BAE */
  0x0000,   /* U+1BAF */
  0x0000,   /* U+1BB0 */
  0x0000,   /* U+1BB1 */
  0x0000,   /* U+1BB2 */
  0x0000,   /* U+1BB3 */
  0x0000,   /* U+1BB4 */
  0x0000,   /* U+1BB5 */
  0x0000,   /* U+1BB6 */
  0x0000,   /* U+1BB7 */
  0x0000,   /* U+1BB8 */
  0x0000,   /* U+1BB9 */
  0x0000,   /* U+1BBA */
  0x0000,   /* U+1BBB */
  0x0000,   /* U+1BBC */
  0x0000,   /* U+1BBD */
  0x0000,   /* U+1BBE */
  0x0000,   /* U+1BBF */
  0x0000,   /* U+1BC0 */
  0x0000,   /* U+1BC1 */
  0x0000,   /* U+1BC2 */
  0x0000,   /* U+1BC3 */
  0x0000,   /* U+1BC4 */
  0x0000,   /* U+1BC5 */
  0x0000,   /* U+1BC6 */
  0x0000,   /* U+1BC7 */
  0x0000,   /* U+1BC8 */
  0x0000,   /* U+1BC9 */
  0x0000,   /* U+1BCA */
  0x0000,   /* U+1BCB */
  0x0000,   /* U+1BCC */
  0x0000,   /* U+1BCD */
  0x0000,   /* U+1BCE */
  0x0000,   /* U+1BCF */
  0x0000,   /* U+1BD0 */
  0x0000,   /* U+1BD1 */
  0x0000,   /* U+1BD2 */
  0x0000,   /* U+1BD3 */
  0x0000,   /* U+1BD4 */
  0x0000,   /* U+1BD5 */
  0x0000,   /* U+1BD6 */
  0x0000,   /* U+1BD7 */
  0x0000,   /* U+1BD8 */
  0x0000,   /* U+1BD9 */
  0x0000,   /* U+1BDA */
  0x0000,   /* U+1BDB */
  0x0000,   /* U+1BDC */
  0x0000,   /* U+1BDD */
  0x0000,   /* U+1BDE */
  0x0000,   /* U+1BDF */
  0x0000,   /* U+1BE0 */
  0x0000,   /* U+1BE1 */
  0x0000,   /* U+1BE2 */
  0x0000,   /* U+1BE3 */
  0x0000,   /* U+1BE4 */
  0x0000,   /* U+1BE5 */
  0x0000,   /* U+1BE6 */
  0x0000,   /* U+1BE7 */
  0x0000,   /* U+1BE8 */
  0x0000,   /* U+1BE9 */
  0x0000,   /* U+1BEA */
  0x0000,   /* U+1BEB */
  0x0000,   /* U+1BEC */
  0x0000,   /* U+1BED */
  0x0000,   /* U+1BEE */
  0x0000,   /* U+1BEF */
  0x0000,   /* U+1BF0 */
  0x0000,   /* U+1BF1 */
  0x0000,   /* U+1BF2 */
  0x0000,   /* U+1BF3 */
  0x0000,   /* U+1BF4 */
  0x0000,   /* U+1BF5 */
  0x0000,   /* U+1BF6 */
  0x0000,   /* U+1BF7 */
  0x0000,   /* U+1BF8 */
  0x0000,   /* U+1BF9 */
  0x0000,   /* U+1BFA */
  0x0000,   /* U+1BFB */
  0x0000,   /* U+1BFC */
  0x0000,   /* U+1BFD */
  0x0000,   /* U+1BFE */
  0x0000,   /* U+1BFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1B00 */
  0x0000,   /* U+1B01 */
  0x0000,   /* U+1B02 */
  0x0000,   /* U+1B03 */
  0x0000,   /* U+1B04 */
  0x0000,   /* U+1B05 */
  0x0000,   /* U+1B06 */
  0x0000,   /* U+1B07 */
  0x0000,   /* U+1B08 */
  0x0000,   /* U+1B09 */
  0x0000,   /* U+1B0A */
  0x0000,   /* U+1B0B */
  0x0000,   /* U+1B0C */
  0x0000,   /* U+1B0D */
  0x0000,   /* U+1B0E */
  0x0000,   /* U+1B0F */
  0x0000,   /* U+1B10 */
  0x0000,   /* U+1B11 */
  0x0000,   /* U+1B12 */
  0x0000,   /* U+1B13 */
  0x0000,   /* U+1B14 */
  0x0000,   /* U+1B15 */
  0x0000,   /* U+1B16 */
  0x0000,   /* U+1B17 */
  0x0000,   /* U+1B18 */
  0x0000,   /* U+1B19 */
  0x0000,   /* U+1B1A */
  0x0000,   /* U+1B1B */
  0x0000,   /* U+1B1C */
  0x0000,   /* U+1B1D */
  0x0000,   /* U+1B1E */
  0x0000,   /* U+1B1F */
  0x0000,   /* U+1B20 */
  0x0000,   /* U+1B21 */
  0x0000,   /* U+1B22 */
  0x0000,   /* U+1B23 */
  0x0000,   /* U+1B24 */
  0x0000,   /* U+1B25 */
  0x0000,   /* U+1B26 */
  0x0000,   /* U+1B27 */
  0x0000,   /* U+1B28 */
  0x0000,   /* U+1B29 */
  0x0000,   /* U+1B2A */
  0x0000,   /* U+1B2B */
  0x0000,   /* U+1B2C */
  0x0000,   /* U+1B2D */
  0x0000,   /* U+1B2E */
  0x0000,   /* U+1B2F */
  0x0000,   /* U+1B30 */
  0x0000,   /* U+1B31 */
  0x0000,   /* U+1B32 */
  0x0000,   /* U+1B33 */
  0x0000,   /* U+1B34 */
  0x0000,   /* U+1B35 */
  0x0000,   /* U+1B36 */
  0x0000,   /* U+1B37 */
  0x0000,   /* U+1B38 */
  0x0000,   /* U+1B39 */
  0x0000,   /* U+1B3A */
  0x0000,   /* U+1B3B */
  0x0000,   /* U+1B3C */
  0x0000,   /* U+1B3D */
  0x0000,   /* U+1B3E */
  0x0000,   /* U+1B3F */
  0x0000,   /* U+1B40 */
  0x0000,   /* U+1B41 */
  0x0000,   /* U+1B42 */
  0x0000,   /* U+1B43 */
  0x0000,   /* U+1B44 */
  0x0000,   /* U+1B45 */
  0x0000,   /* U+1B46 */
  0x0000,   /* U+1B47 */
  0x0000,   /* U+1B48 */
  0x0000,   /* U+1B49 */
  0x0000,   /* U+1B4A */
  0x0000,   /* U+1B4B */
  0x0000,   /* U+1B4C */
  0x0000,   /* U+1B4D */
  0x0000,   /* U+1B4E */
  0x0000,   /* U+1B4F */
  0x0000,   /* U+1B50 */
  0x0000,   /* U+1B51 */
  0x0000,   /* U+1B52 */
  0x0000,   /* U+1B53 */
  0x0000,   /* U+1B54 */
  0x0000,   /* U+1B55 */
  0x0000,   /* U+1B56 */
  0x0000,   /* U+1B57 */
  0x0000,   /* U+1B58 */
  0x0000,   /* U+1B59 */
  0x0000,   /* U+1B5A */
  0x0000,   /* U+1B5B */
  0x0000,   /* U+1B5C */
  0x0000,   /* U+1B5D */
  0x0000,   /* U+1B5E */
  0x0000,   /* U+1B5F */
  0x0000,   /* U+1B60 */
  0x0000,   /* U+1B61 */
  0x0000,   /* U+1B62 */
  0x0000,   /* U+1B63 */
  0x0000,   /* U+1B64 */
  0x0000,   /* U+1B65 */
  0x0000,   /* U+1B66 */
  0x0000,   /* U+1B67 */
  0x0000,   /* U+1B68 */
  0x0000,   /* U+1B69 */
  0x0000,   /* U+1B6A */
  0x0000,   /* U+1B6B */
  0x0000,   /* U+1B6C */
  0x0000,   /* U+1B6D */
  0x0000,   /* U+1B6E */
  0x0000,   /* U+1B6F */
  0x0000,   /* U+1B70 */
  0x0000,   /* U+1B71 */
  0x0000,   /* U+1B72 */
  0x0000,   /* U+1B73 */
  0x0000,   /* U+1B74 */
  0x0000,   /* U+1B75 */
  0x0000,   /* U+1B76 */
  0x0000,   /* U+1B77 */
  0x0000,   /* U+1B78 */
  0x0000,   /* U+1B79 */
  0x0000,   /* U+1B7A */
  0x0000,   /* U+1B7B */
  0x0000,   /* U+1B7C */
  0x0000,   /* U+1B7D */
  0x0000,   /* U+1B7E */
  0x0000,   /* U+1B7F */
  0x0000,   /* U+1B80 */
  0x0000,   /* U+1B81 */
  0x0000,   /* U+1B82 */
  0x0000,   /* U+1B83 */
  0x0000,   /* U+1B84 */
  0x0000,   /* U+1B85 */
  0x0000,   /* U+1B86 */
  0x0000,   /* U+1B87 */
  0x0000,   /* U+1B88 */
  0x0000,   /* U+1B89 */
  0x0000,   /* U+1B8A */
  0x0000,   /* U+1B8B */
  0x0000,   /* U+1B8C */
  0x0000,   /* U+1B8D */
  0x0000,   /* U+1B8E */
  0x0000,   /* U+1B8F */
  0x0000,   /* U+1B90 */
  0x0000,   /* U+1B91 */
  0x0000,   /* U+1B92 */
  0x0000,   /* U+1B93 */
  0x0000,   /* U+1B94 */
  0x0000,   /* U+1B95 */
  0x0000,   /* U+1B96 */
  0x0000,   /* U+1B97 */
  0x0000,   /* U+1B98 */
  0x0000,   /* U+1B99 */
  0x0000,   /* U+1B9A */
  0x0000,   /* U+1B9B */
  0x0000,   /* U+1B9C */
  0x0000,   /* U+1B9D */
  0x0000,   /* U+1B9E */
  0x0000,   /* U+1B9F */
  0x0000,   /* U+1BA0 */
  0x0000,   /* U+1BA1 */
  0x0000,   /* U+1BA2 */
  0x0000,   /* U+1BA3 */
  0x0000,   /* U+1BA4 */
  0x0000,   /* U+1BA5 */
  0x0000,   /* U+1BA6 */
  0x0000,   /* U+1BA7 */
  0x0000,   /* U+1BA8 */
  0x0000,   /* U+1BA9 */
  0x0000,   /* U+1BAA */
  0x0000,   /* U+1BAB */
  0x0000,   /* U+1BAC */
  0x0000,   /* U+1BAD */
  0x0000,   /* U+1BAE */
  0x0000,   /* U+1BAF */
  0x0000,   /* U+1BB0 */
  0x0000,   /* U+1BB1 */
  0x0000,   /* U+1BB2 */
  0x0000,   /* U+1BB3 */
  0x0000,   /* U+1BB4 */
  0x0000,   /* U+1BB5 */
  0x0000,   /* U+1BB6 */
  0x0000,   /* U+1BB7 */
  0x0000,   /* U+1BB8 */
  0x0000,   /* U+1BB9 */
  0x0000,   /* U+1BBA */
  0x0000,   /* U+1BBB */
  0x0000,   /* U+1BBC */
  0x0000,   /* U+1BBD */
  0x0000,   /* U+1BBE */
  0x0000,   /* U+1BBF */
  0x0000,   /* U+1BC0 */
  0x0000,   /* U+1BC1 */
  0x0000,   /* U+1BC2 */
  0x0000,   /* U+1BC3 */
  0x0000,   /* U+1BC4 */
  0x0000,   /* U+1BC5 */
  0x0000,   /* U+1BC6 */
  0x0000,   /* U+1BC7 */
  0x0000,   /* U+1BC8 */
  0x0000,   /* U+1BC9 */
  0x0000,   /* U+1BCA */
  0x0000,   /* U+1BCB */
  0x0000,   /* U+1BCC */
  0x0000,   /* U+1BCD */
  0x0000,   /* U+1BCE */
  0x0000,   /* U+1BCF */
  0x0000,   /* U+1BD0 */
  0x0000,   /* U+1BD1 */
  0x0000,   /* U+1BD2 */
  0x0000,   /* U+1BD3 */
  0x0000,   /* U+1BD4 */
  0x0000,   /* U+1BD5 */
  0x0000,   /* U+1BD6 */
  0x0000,   /* U+1BD7 */
  0x0000,   /* U+1BD8 */
  0x0000,   /* U+1BD9 */
  0x0000,   /* U+1BDA */
  0x0000,   /* U+1BDB */
  0x0000,   /* U+1BDC */
  0x0000,   /* U+1BDD */
  0x0000,   /* U+1BDE */
  0x0000,   /* U+1BDF */
  0x0000,   /* U+1BE0 */
  0x0000,   /* U+1BE1 */
  0x0000,   /* U+1BE2 */
  0x0000,   /* U+1BE3 */
  0x0000,   /* U+1BE4 */
  0x0000,   /* U+1BE5 */
  0x0000,   /* U+1BE6 */
  0x0000,   /* U+1BE7 */
  0x0000,   /* U+1BE8 */
  0x0000,   /* U+1BE9 */
  0x0000,   /* U+1BEA */
  0x0000,   /* U+1BEB */
  0x0000,   /* U+1BEC */
  0x0000,   /* U+1BED */
  0x0000,   /* U+1BEE */
  0x0000,   /* U+1BEF */
  0x0000,   /* U+1BF0 */
  0x0000,   /* U+1BF1 */
  0x0000,   /* U+1BF2 */
  0x0000,   /* U+1BF3 */
  0x0000,   /* U+1BF4 */
  0x0000,   /* U+1BF5 */
  0x0000,   /* U+1BF6 */
  0x0000,   /* U+1BF7 */
  0x0000,   /* U+1BF8 */
  0x0000,   /* U+1BF9 */
  0x0000,   /* U+1BFA */
  0x0000,   /* U+1BFB */
  0x0000,   /* U+1BFC */
  0x0000,   /* U+1BFD */
  0x0000,   /* U+1BFE */
  0x0000,   /* U+1BFF */
};

uint16 uca900_p01C[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2EC6,   /* U+1C00 */
  0x2EC7,   /* U+1C01 */
  0x2EC8,   /* U+1C02 */
  0x2EC9,   /* U+1C03 */
  0x2ECA,   /* U+1C04 */
  0x2ECB,   /* U+1C05 */
  0x2ECC,   /* U+1C06 */
  0x2ECD,   /* U+1C07 */
  0x2ECE,   /* U+1C08 */
  0x2ECF,   /* U+1C09 */
  0x2ED3,   /* U+1C0A */
  0x2ED4,   /* U+1C0B */
  0x2ED5,   /* U+1C0C */
  0x2ED6,   /* U+1C0D */
  0x2ED7,   /* U+1C0E */
  0x2ED8,   /* U+1C0F */
  0x2ED9,   /* U+1C10 */
  0x2EDA,   /* U+1C11 */
  0x2EDB,   /* U+1C12 */
  0x2EDC,   /* U+1C13 */
  0x2EDD,   /* U+1C14 */
  0x2EDE,   /* U+1C15 */
  0x2EDF,   /* U+1C16 */
  0x2EE0,   /* U+1C17 */
  0x2EE1,   /* U+1C18 */
  0x2EE2,   /* U+1C19 */
  0x2EE3,   /* U+1C1A */
  0x2EE5,   /* U+1C1B */
  0x2EE7,   /* U+1C1C */
  0x2EE8,   /* U+1C1D */
  0x2EE9,   /* U+1C1E */
  0x2EEA,   /* U+1C1F */
  0x2EEB,   /* U+1C20 */
  0x2EEC,   /* U+1C21 */
  0x2EED,   /* U+1C22 */
  0x2EEE,   /* U+1C23 */
  0x2EE4,   /* U+1C24 */
  0x2EE6,   /* U+1C25 */
  0x2EF0,   /* U+1C26 */
  0x2EF1,   /* U+1C27 */
  0x2EF2,   /* U+1C28 */
  0x2EF3,   /* U+1C29 */
  0x2EF4,   /* U+1C2A */
  0x2EF5,   /* U+1C2B */
  0x2EF6,   /* U+1C2C */
  0x2EF7,   /* U+1C2D */
  0x2EF8,   /* U+1C2E */
  0x2EF9,   /* U+1C2F */
  0x2EFA,   /* U+1C30 */
  0x2EFB,   /* U+1C31 */
  0x2EFC,   /* U+1C32 */
  0x2EFD,   /* U+1C33 */
  0x2EFE,   /* U+1C34 */
  0x2EFF,   /* U+1C35 */
  0x2EEF,   /* U+1C36 */
  0x0000,   /* U+1C37 */
  0xFBC0,   /* U+1C38 */
  0xFBC0,   /* U+1C39 */
  0xFBC0,   /* U+1C3A */
  0x0292,   /* U+1C3B */
  0x0293,   /* U+1C3C */
  0x040B,   /* U+1C3D */
  0x040C,   /* U+1C3E */
  0x040D,   /* U+1C3F */
  0x1C3D,   /* U+1C40 */
  0x1C3E,   /* U+1C41 */
  0x1C3F,   /* U+1C42 */
  0x1C40,   /* U+1C43 */
  0x1C41,   /* U+1C44 */
  0x1C42,   /* U+1C45 */
  0x1C43,   /* U+1C46 */
  0x1C44,   /* U+1C47 */
  0x1C45,   /* U+1C48 */
  0x1C46,   /* U+1C49 */
  0xFBC0,   /* U+1C4A */
  0xFBC0,   /* U+1C4B */
  0xFBC0,   /* U+1C4C */
  0x2ED0,   /* U+1C4D */
  0x2ED1,   /* U+1C4E */
  0x2ED2,   /* U+1C4F */
  0x1C3D,   /* U+1C50 */
  0x1C3E,   /* U+1C51 */
  0x1C3F,   /* U+1C52 */
  0x1C40,   /* U+1C53 */
  0x1C41,   /* U+1C54 */
  0x1C42,   /* U+1C55 */
  0x1C43,   /* U+1C56 */
  0x1C44,   /* U+1C57 */
  0x1C45,   /* U+1C58 */
  0x1C46,   /* U+1C59 */
  0x335B,   /* U+1C5A */
  0x335C,   /* U+1C5B */
  0x335D,   /* U+1C5C */
  0x335E,   /* U+1C5D */
  0x335F,   /* U+1C5E */
  0x3360,   /* U+1C5F */
  0x3361,   /* U+1C60 */
  0x3362,   /* U+1C61 */
  0x3363,   /* U+1C62 */
  0x3364,   /* U+1C63 */
  0x3365,   /* U+1C64 */
  0x3366,   /* U+1C65 */
  0x3367,   /* U+1C66 */
  0x3368,   /* U+1C67 */
  0x3369,   /* U+1C68 */
  0x336A,   /* U+1C69 */
  0x336B,   /* U+1C6A */
  0x336C,   /* U+1C6B */
  0x336D,   /* U+1C6C */
  0x336E,   /* U+1C6D */
  0x336F,   /* U+1C6E */
  0x3370,   /* U+1C6F */
  0x3371,   /* U+1C70 */
  0x3372,   /* U+1C71 */
  0x3373,   /* U+1C72 */
  0x3374,   /* U+1C73 */
  0x3375,   /* U+1C74 */
  0x3376,   /* U+1C75 */
  0x3377,   /* U+1C76 */
  0x3378,   /* U+1C77 */
  0x3379,   /* U+1C78 */
  0x337A,   /* U+1C79 */
  0x337B,   /* U+1C7A */
  0x337C,   /* U+1C7B */
  0x337D,   /* U+1C7C */
  0x337E,   /* U+1C7D */
  0x02C2,   /* U+1C7E */
  0x02C3,   /* U+1C7F */
  0x2032,   /* U+1C80 */
  0x204A,   /* U+1C81 */
  0x20E7,   /* U+1C82 */
  0x2105,   /* U+1C83 */
  0x210E,   /* U+1C84 */
  0x210E,   /* U+1C85 */
  0x218F,   /* U+1C86 */
  0x21A0,   /* U+1C87 */
  0x2129,   /* U+1C88 */
  0xFBC0,   /* U+1C89 */
  0xFBC0,   /* U+1C8A */
  0xFBC0,   /* U+1C8B */
  0xFBC0,   /* U+1C8C */
  0xFBC0,   /* U+1C8D */
  0xFBC0,   /* U+1C8E */
  0xFBC0,   /* U+1C8F */
  0xFBC0,   /* U+1C90 */
  0xFBC0,   /* U+1C91 */
  0xFBC0,   /* U+1C92 */
  0xFBC0,   /* U+1C93 */
  0xFBC0,   /* U+1C94 */
  0xFBC0,   /* U+1C95 */
  0xFBC0,   /* U+1C96 */
  0xFBC0,   /* U+1C97 */
  0xFBC0,   /* U+1C98 */
  0xFBC0,   /* U+1C99 */
  0xFBC0,   /* U+1C9A */
  0xFBC0,   /* U+1C9B */
  0xFBC0,   /* U+1C9C */
  0xFBC0,   /* U+1C9D */
  0xFBC0,   /* U+1C9E */
  0xFBC0,   /* U+1C9F */
  0xFBC0,   /* U+1CA0 */
  0xFBC0,   /* U+1CA1 */
  0xFBC0,   /* U+1CA2 */
  0xFBC0,   /* U+1CA3 */
  0xFBC0,   /* U+1CA4 */
  0xFBC0,   /* U+1CA5 */
  0xFBC0,   /* U+1CA6 */
  0xFBC0,   /* U+1CA7 */
  0xFBC0,   /* U+1CA8 */
  0xFBC0,   /* U+1CA9 */
  0xFBC0,   /* U+1CAA */
  0xFBC0,   /* U+1CAB */
  0xFBC0,   /* U+1CAC */
  0xFBC0,   /* U+1CAD */
  0xFBC0,   /* U+1CAE */
  0xFBC0,   /* U+1CAF */
  0xFBC0,   /* U+1CB0 */
  0xFBC0,   /* U+1CB1 */
  0xFBC0,   /* U+1CB2 */
  0xFBC0,   /* U+1CB3 */
  0xFBC0,   /* U+1CB4 */
  0xFBC0,   /* U+1CB5 */
  0xFBC0,   /* U+1CB6 */
  0xFBC0,   /* U+1CB7 */
  0xFBC0,   /* U+1CB8 */
  0xFBC0,   /* U+1CB9 */
  0xFBC0,   /* U+1CBA */
  0xFBC0,   /* U+1CBB */
  0xFBC0,   /* U+1CBC */
  0xFBC0,   /* U+1CBD */
  0xFBC0,   /* U+1CBE */
  0xFBC0,   /* U+1CBF */
  0x041F,   /* U+1CC0 */
  0x0420,   /* U+1CC1 */
  0x0421,   /* U+1CC2 */
  0x0422,   /* U+1CC3 */
  0x0423,   /* U+1CC4 */
  0x0424,   /* U+1CC5 */
  0x0425,   /* U+1CC6 */
  0x0426,   /* U+1CC7 */
  0xFBC0,   /* U+1CC8 */
  0xFBC0,   /* U+1CC9 */
  0xFBC0,   /* U+1CCA */
  0xFBC0,   /* U+1CCB */
  0xFBC0,   /* U+1CCC */
  0xFBC0,   /* U+1CCD */
  0xFBC0,   /* U+1CCE */
  0xFBC0,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x26A1,   /* U+1CE9 */
  0x26A1,   /* U+1CEA */
  0x26A1,   /* U+1CEB */
  0x26A1,   /* U+1CEC */
  0x0000,   /* U+1CED */
  0x26A1,   /* U+1CEE */
  0x26A1,   /* U+1CEF */
  0x26A1,   /* U+1CF0 */
  0x26A1,   /* U+1CF1 */
  0x0000,   /* U+1CF2 */
  0x0000,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x26A2,   /* U+1CF5 */
  0x26A3,   /* U+1CF6 */
  0xFBC0,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0xFBC0,   /* U+1CFA */
  0xFBC0,   /* U+1CFB */
  0xFBC0,   /* U+1CFC */
  0xFBC0,   /* U+1CFD */
  0xFBC0,   /* U+1CFE */
  0xFBC0,   /* U+1CFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1C00 */
  0x0020,   /* U+1C01 */
  0x0020,   /* U+1C02 */
  0x0020,   /* U+1C03 */
  0x0020,   /* U+1C04 */
  0x0020,   /* U+1C05 */
  0x0020,   /* U+1C06 */
  0x0020,   /* U+1C07 */
  0x0020,   /* U+1C08 */
  0x0020,   /* U+1C09 */
  0x0020,   /* U+1C0A */
  0x0020,   /* U+1C0B */
  0x0020,   /* U+1C0C */
  0x0020,   /* U+1C0D */
  0x0020,   /* U+1C0E */
  0x0020,   /* U+1C0F */
  0x0020,   /* U+1C10 */
  0x0020,   /* U+1C11 */
  0x0020,   /* U+1C12 */
  0x0020,   /* U+1C13 */
  0x0020,   /* U+1C14 */
  0x0020,   /* U+1C15 */
  0x0020,   /* U+1C16 */
  0x0020,   /* U+1C17 */
  0x0020,   /* U+1C18 */
  0x0020,   /* U+1C19 */
  0x0020,   /* U+1C1A */
  0x0020,   /* U+1C1B */
  0x0020,   /* U+1C1C */
  0x0020,   /* U+1C1D */
  0x0020,   /* U+1C1E */
  0x0020,   /* U+1C1F */
  0x0020,   /* U+1C20 */
  0x0020,   /* U+1C21 */
  0x0020,   /* U+1C22 */
  0x0020,   /* U+1C23 */
  0x0020,   /* U+1C24 */
  0x0020,   /* U+1C25 */
  0x0020,   /* U+1C26 */
  0x0020,   /* U+1C27 */
  0x0020,   /* U+1C28 */
  0x0020,   /* U+1C29 */
  0x0020,   /* U+1C2A */
  0x0020,   /* U+1C2B */
  0x0020,   /* U+1C2C */
  0x0020,   /* U+1C2D */
  0x0020,   /* U+1C2E */
  0x0020,   /* U+1C2F */
  0x0020,   /* U+1C30 */
  0x0020,   /* U+1C31 */
  0x0020,   /* U+1C32 */
  0x0020,   /* U+1C33 */
  0x0020,   /* U+1C34 */
  0x0020,   /* U+1C35 */
  0x0020,   /* U+1C36 */
  0x00C2,   /* U+1C37 */
  0x0020,   /* U+1C38 */
  0x0020,   /* U+1C39 */
  0x0020,   /* U+1C3A */
  0x0020,   /* U+1C3B */
  0x0020,   /* U+1C3C */
  0x0020,   /* U+1C3D */
  0x0020,   /* U+1C3E */
  0x0020,   /* U+1C3F */
  0x0020,   /* U+1C40 */
  0x0020,   /* U+1C41 */
  0x0020,   /* U+1C42 */
  0x0020,   /* U+1C43 */
  0x0020,   /* U+1C44 */
  0x0020,   /* U+1C45 */
  0x0020,   /* U+1C46 */
  0x0020,   /* U+1C47 */
  0x0020,   /* U+1C48 */
  0x0020,   /* U+1C49 */
  0x0020,   /* U+1C4A */
  0x0020,   /* U+1C4B */
  0x0020,   /* U+1C4C */
  0x0020,   /* U+1C4D */
  0x0020,   /* U+1C4E */
  0x0020,   /* U+1C4F */
  0x0020,   /* U+1C50 */
  0x0020,   /* U+1C51 */
  0x0020,   /* U+1C52 */
  0x0020,   /* U+1C53 */
  0x0020,   /* U+1C54 */
  0x0020,   /* U+1C55 */
  0x0020,   /* U+1C56 */
  0x0020,   /* U+1C57 */
  0x0020,   /* U+1C58 */
  0x0020,   /* U+1C59 */
  0x0020,   /* U+1C5A */
  0x0020,   /* U+1C5B */
  0x0020,   /* U+1C5C */
  0x0020,   /* U+1C5D */
  0x0020,   /* U+1C5E */
  0x0020,   /* U+1C5F */
  0x0020,   /* U+1C60 */
  0x0020,   /* U+1C61 */
  0x0020,   /* U+1C62 */
  0x0020,   /* U+1C63 */
  0x0020,   /* U+1C64 */
  0x0020,   /* U+1C65 */
  0x0020,   /* U+1C66 */
  0x0020,   /* U+1C67 */
  0x0020,   /* U+1C68 */
  0x0020,   /* U+1C69 */
  0x0020,   /* U+1C6A */
  0x0020,   /* U+1C6B */
  0x0020,   /* U+1C6C */
  0x0020,   /* U+1C6D */
  0x0020,   /* U+1C6E */
  0x0020,   /* U+1C6F */
  0x0020,   /* U+1C70 */
  0x0020,   /* U+1C71 */
  0x0020,   /* U+1C72 */
  0x0020,   /* U+1C73 */
  0x0020,   /* U+1C74 */
  0x0020,   /* U+1C75 */
  0x0020,   /* U+1C76 */
  0x0020,   /* U+1C77 */
  0x0020,   /* U+1C78 */
  0x0020,   /* U+1C79 */
  0x0020,   /* U+1C7A */
  0x0020,   /* U+1C7B */
  0x0020,   /* U+1C7C */
  0x0020,   /* U+1C7D */
  0x0020,   /* U+1C7E */
  0x0020,   /* U+1C7F */
  0x0020,   /* U+1C80 */
  0x0020,   /* U+1C81 */
  0x0020,   /* U+1C82 */
  0x0020,   /* U+1C83 */
  0x0020,   /* U+1C84 */
  0x0020,   /* U+1C85 */
  0x0020,   /* U+1C86 */
  0x0020,   /* U+1C87 */
  0x0020,   /* U+1C88 */
  0x0020,   /* U+1C89 */
  0x0020,   /* U+1C8A */
  0x0020,   /* U+1C8B */
  0x0020,   /* U+1C8C */
  0x0020,   /* U+1C8D */
  0x0020,   /* U+1C8E */
  0x0020,   /* U+1C8F */
  0x0020,   /* U+1C90 */
  0x0020,   /* U+1C91 */
  0x0020,   /* U+1C92 */
  0x0020,   /* U+1C93 */
  0x0020,   /* U+1C94 */
  0x0020,   /* U+1C95 */
  0x0020,   /* U+1C96 */
  0x0020,   /* U+1C97 */
  0x0020,   /* U+1C98 */
  0x0020,   /* U+1C99 */
  0x0020,   /* U+1C9A */
  0x0020,   /* U+1C9B */
  0x0020,   /* U+1C9C */
  0x0020,   /* U+1C9D */
  0x0020,   /* U+1C9E */
  0x0020,   /* U+1C9F */
  0x0020,   /* U+1CA0 */
  0x0020,   /* U+1CA1 */
  0x0020,   /* U+1CA2 */
  0x0020,   /* U+1CA3 */
  0x0020,   /* U+1CA4 */
  0x0020,   /* U+1CA5 */
  0x0020,   /* U+1CA6 */
  0x0020,   /* U+1CA7 */
  0x0020,   /* U+1CA8 */
  0x0020,   /* U+1CA9 */
  0x0020,   /* U+1CAA */
  0x0020,   /* U+1CAB */
  0x0020,   /* U+1CAC */
  0x0020,   /* U+1CAD */
  0x0020,   /* U+1CAE */
  0x0020,   /* U+1CAF */
  0x0020,   /* U+1CB0 */
  0x0020,   /* U+1CB1 */
  0x0020,   /* U+1CB2 */
  0x0020,   /* U+1CB3 */
  0x0020,   /* U+1CB4 */
  0x0020,   /* U+1CB5 */
  0x0020,   /* U+1CB6 */
  0x0020,   /* U+1CB7 */
  0x0020,   /* U+1CB8 */
  0x0020,   /* U+1CB9 */
  0x0020,   /* U+1CBA */
  0x0020,   /* U+1CBB */
  0x0020,   /* U+1CBC */
  0x0020,   /* U+1CBD */
  0x0020,   /* U+1CBE */
  0x0020,   /* U+1CBF */
  0x0020,   /* U+1CC0 */
  0x0020,   /* U+1CC1 */
  0x0020,   /* U+1CC2 */
  0x0020,   /* U+1CC3 */
  0x0020,   /* U+1CC4 */
  0x0020,   /* U+1CC5 */
  0x0020,   /* U+1CC6 */
  0x0020,   /* U+1CC7 */
  0x0020,   /* U+1CC8 */
  0x0020,   /* U+1CC9 */
  0x0020,   /* U+1CCA */
  0x0020,   /* U+1CCB */
  0x0020,   /* U+1CCC */
  0x0020,   /* U+1CCD */
  0x0020,   /* U+1CCE */
  0x0020,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x0020,   /* U+1CE9 */
  0x0020,   /* U+1CEA */
  0x0020,   /* U+1CEB */
  0x0020,   /* U+1CEC */
  0x00C4,   /* U+1CED */
  0x0020,   /* U+1CEE */
  0x0020,   /* U+1CEF */
  0x0020,   /* U+1CF0 */
  0x0020,   /* U+1CF1 */
  0x00C5,   /* U+1CF2 */
  0x00C5,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x0020,   /* U+1CF5 */
  0x0020,   /* U+1CF6 */
  0x0020,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0x0020,   /* U+1CFA */
  0x0020,   /* U+1CFB */
  0x0020,   /* U+1CFC */
  0x0020,   /* U+1CFD */
  0x0020,   /* U+1CFE */
  0x0020,   /* U+1CFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1C00 */
  0x0002,   /* U+1C01 */
  0x0002,   /* U+1C02 */
  0x0002,   /* U+1C03 */
  0x0002,   /* U+1C04 */
  0x0002,   /* U+1C05 */
  0x0002,   /* U+1C06 */
  0x0002,   /* U+1C07 */
  0x0002,   /* U+1C08 */
  0x0002,   /* U+1C09 */
  0x0002,   /* U+1C0A */
  0x0002,   /* U+1C0B */
  0x0002,   /* U+1C0C */
  0x0002,   /* U+1C0D */
  0x0002,   /* U+1C0E */
  0x0002,   /* U+1C0F */
  0x0002,   /* U+1C10 */
  0x0002,   /* U+1C11 */
  0x0002,   /* U+1C12 */
  0x0002,   /* U+1C13 */
  0x0002,   /* U+1C14 */
  0x0002,   /* U+1C15 */
  0x0002,   /* U+1C16 */
  0x0002,   /* U+1C17 */
  0x0002,   /* U+1C18 */
  0x0002,   /* U+1C19 */
  0x0002,   /* U+1C1A */
  0x0002,   /* U+1C1B */
  0x0002,   /* U+1C1C */
  0x0002,   /* U+1C1D */
  0x0002,   /* U+1C1E */
  0x0002,   /* U+1C1F */
  0x0002,   /* U+1C20 */
  0x0002,   /* U+1C21 */
  0x0002,   /* U+1C22 */
  0x0002,   /* U+1C23 */
  0x0002,   /* U+1C24 */
  0x0002,   /* U+1C25 */
  0x0002,   /* U+1C26 */
  0x0002,   /* U+1C27 */
  0x0002,   /* U+1C28 */
  0x0002,   /* U+1C29 */
  0x0002,   /* U+1C2A */
  0x0002,   /* U+1C2B */
  0x0002,   /* U+1C2C */
  0x0002,   /* U+1C2D */
  0x0002,   /* U+1C2E */
  0x0002,   /* U+1C2F */
  0x0002,   /* U+1C30 */
  0x0002,   /* U+1C31 */
  0x0002,   /* U+1C32 */
  0x0002,   /* U+1C33 */
  0x0002,   /* U+1C34 */
  0x0002,   /* U+1C35 */
  0x0002,   /* U+1C36 */
  0x0002,   /* U+1C37 */
  0x0002,   /* U+1C38 */
  0x0002,   /* U+1C39 */
  0x0002,   /* U+1C3A */
  0x0002,   /* U+1C3B */
  0x0002,   /* U+1C3C */
  0x0002,   /* U+1C3D */
  0x0002,   /* U+1C3E */
  0x0002,   /* U+1C3F */
  0x0002,   /* U+1C40 */
  0x0002,   /* U+1C41 */
  0x0002,   /* U+1C42 */
  0x0002,   /* U+1C43 */
  0x0002,   /* U+1C44 */
  0x0002,   /* U+1C45 */
  0x0002,   /* U+1C46 */
  0x0002,   /* U+1C47 */
  0x0002,   /* U+1C48 */
  0x0002,   /* U+1C49 */
  0x0002,   /* U+1C4A */
  0x0002,   /* U+1C4B */
  0x0002,   /* U+1C4C */
  0x0002,   /* U+1C4D */
  0x0002,   /* U+1C4E */
  0x0002,   /* U+1C4F */
  0x0002,   /* U+1C50 */
  0x0002,   /* U+1C51 */
  0x0002,   /* U+1C52 */
  0x0002,   /* U+1C53 */
  0x0002,   /* U+1C54 */
  0x0002,   /* U+1C55 */
  0x0002,   /* U+1C56 */
  0x0002,   /* U+1C57 */
  0x0002,   /* U+1C58 */
  0x0002,   /* U+1C59 */
  0x0002,   /* U+1C5A */
  0x0002,   /* U+1C5B */
  0x0002,   /* U+1C5C */
  0x0002,   /* U+1C5D */
  0x0002,   /* U+1C5E */
  0x0002,   /* U+1C5F */
  0x0002,   /* U+1C60 */
  0x0002,   /* U+1C61 */
  0x0002,   /* U+1C62 */
  0x0002,   /* U+1C63 */
  0x0002,   /* U+1C64 */
  0x0002,   /* U+1C65 */
  0x0002,   /* U+1C66 */
  0x0002,   /* U+1C67 */
  0x0002,   /* U+1C68 */
  0x0002,   /* U+1C69 */
  0x0002,   /* U+1C6A */
  0x0002,   /* U+1C6B */
  0x0002,   /* U+1C6C */
  0x0002,   /* U+1C6D */
  0x0002,   /* U+1C6E */
  0x0002,   /* U+1C6F */
  0x0002,   /* U+1C70 */
  0x0002,   /* U+1C71 */
  0x0002,   /* U+1C72 */
  0x0002,   /* U+1C73 */
  0x0002,   /* U+1C74 */
  0x0002,   /* U+1C75 */
  0x0002,   /* U+1C76 */
  0x0002,   /* U+1C77 */
  0x0002,   /* U+1C78 */
  0x0002,   /* U+1C79 */
  0x0002,   /* U+1C7A */
  0x0002,   /* U+1C7B */
  0x0002,   /* U+1C7C */
  0x0002,   /* U+1C7D */
  0x0002,   /* U+1C7E */
  0x0002,   /* U+1C7F */
  0x0004,   /* U+1C80 */
  0x0004,   /* U+1C81 */
  0x0004,   /* U+1C82 */
  0x0004,   /* U+1C83 */
  0x0004,   /* U+1C84 */
  0x0004,   /* U+1C85 */
  0x0004,   /* U+1C86 */
  0x0004,   /* U+1C87 */
  0x0004,   /* U+1C88 */
  0x0002,   /* U+1C89 */
  0x0002,   /* U+1C8A */
  0x0002,   /* U+1C8B */
  0x0002,   /* U+1C8C */
  0x0002,   /* U+1C8D */
  0x0002,   /* U+1C8E */
  0x0002,   /* U+1C8F */
  0x0002,   /* U+1C90 */
  0x0002,   /* U+1C91 */
  0x0002,   /* U+1C92 */
  0x0002,   /* U+1C93 */
  0x0002,   /* U+1C94 */
  0x0002,   /* U+1C95 */
  0x0002,   /* U+1C96 */
  0x0002,   /* U+1C97 */
  0x0002,   /* U+1C98 */
  0x0002,   /* U+1C99 */
  0x0002,   /* U+1C9A */
  0x0002,   /* U+1C9B */
  0x0002,   /* U+1C9C */
  0x0002,   /* U+1C9D */
  0x0002,   /* U+1C9E */
  0x0002,   /* U+1C9F */
  0x0002,   /* U+1CA0 */
  0x0002,   /* U+1CA1 */
  0x0002,   /* U+1CA2 */
  0x0002,   /* U+1CA3 */
  0x0002,   /* U+1CA4 */
  0x0002,   /* U+1CA5 */
  0x0002,   /* U+1CA6 */
  0x0002,   /* U+1CA7 */
  0x0002,   /* U+1CA8 */
  0x0002,   /* U+1CA9 */
  0x0002,   /* U+1CAA */
  0x0002,   /* U+1CAB */
  0x0002,   /* U+1CAC */
  0x0002,   /* U+1CAD */
  0x0002,   /* U+1CAE */
  0x0002,   /* U+1CAF */
  0x0002,   /* U+1CB0 */
  0x0002,   /* U+1CB1 */
  0x0002,   /* U+1CB2 */
  0x0002,   /* U+1CB3 */
  0x0002,   /* U+1CB4 */
  0x0002,   /* U+1CB5 */
  0x0002,   /* U+1CB6 */
  0x0002,   /* U+1CB7 */
  0x0002,   /* U+1CB8 */
  0x0002,   /* U+1CB9 */
  0x0002,   /* U+1CBA */
  0x0002,   /* U+1CBB */
  0x0002,   /* U+1CBC */
  0x0002,   /* U+1CBD */
  0x0002,   /* U+1CBE */
  0x0002,   /* U+1CBF */
  0x0002,   /* U+1CC0 */
  0x0002,   /* U+1CC1 */
  0x0002,   /* U+1CC2 */
  0x0002,   /* U+1CC3 */
  0x0002,   /* U+1CC4 */
  0x0002,   /* U+1CC5 */
  0x0002,   /* U+1CC6 */
  0x0002,   /* U+1CC7 */
  0x0002,   /* U+1CC8 */
  0x0002,   /* U+1CC9 */
  0x0002,   /* U+1CCA */
  0x0002,   /* U+1CCB */
  0x0002,   /* U+1CCC */
  0x0002,   /* U+1CCD */
  0x0002,   /* U+1CCE */
  0x0002,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x0002,   /* U+1CE9 */
  0x0004,   /* U+1CEA */
  0x0004,   /* U+1CEB */
  0x0004,   /* U+1CEC */
  0x0002,   /* U+1CED */
  0x0004,   /* U+1CEE */
  0x0004,   /* U+1CEF */
  0x0004,   /* U+1CF0 */
  0x0004,   /* U+1CF1 */
  0x0002,   /* U+1CF2 */
  0x0002,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x0002,   /* U+1CF5 */
  0x0002,   /* U+1CF6 */
  0x0002,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0x0002,   /* U+1CFA */
  0x0002,   /* U+1CFB */
  0x0002,   /* U+1CFC */
  0x0002,   /* U+1CFD */
  0x0002,   /* U+1CFE */
  0x0002,   /* U+1CFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1C00 */
  0x0000,   /* U+1C01 */
  0x0000,   /* U+1C02 */
  0x0000,   /* U+1C03 */
  0x0000,   /* U+1C04 */
  0x0000,   /* U+1C05 */
  0x0000,   /* U+1C06 */
  0x0000,   /* U+1C07 */
  0x0000,   /* U+1C08 */
  0x0000,   /* U+1C09 */
  0x0000,   /* U+1C0A */
  0x0000,   /* U+1C0B */
  0x0000,   /* U+1C0C */
  0x0000,   /* U+1C0D */
  0x0000,   /* U+1C0E */
  0x0000,   /* U+1C0F */
  0x0000,   /* U+1C10 */
  0x0000,   /* U+1C11 */
  0x0000,   /* U+1C12 */
  0x0000,   /* U+1C13 */
  0x0000,   /* U+1C14 */
  0x0000,   /* U+1C15 */
  0x0000,   /* U+1C16 */
  0x0000,   /* U+1C17 */
  0x0000,   /* U+1C18 */
  0x0000,   /* U+1C19 */
  0x0000,   /* U+1C1A */
  0x0000,   /* U+1C1B */
  0x0000,   /* U+1C1C */
  0x0000,   /* U+1C1D */
  0x0000,   /* U+1C1E */
  0x0000,   /* U+1C1F */
  0x0000,   /* U+1C20 */
  0x0000,   /* U+1C21 */
  0x0000,   /* U+1C22 */
  0x0000,   /* U+1C23 */
  0x0000,   /* U+1C24 */
  0x0000,   /* U+1C25 */
  0x0000,   /* U+1C26 */
  0x0000,   /* U+1C27 */
  0x0000,   /* U+1C28 */
  0x0000,   /* U+1C29 */
  0x0000,   /* U+1C2A */
  0x0000,   /* U+1C2B */
  0x0000,   /* U+1C2C */
  0x0000,   /* U+1C2D */
  0x0000,   /* U+1C2E */
  0x0000,   /* U+1C2F */
  0x0000,   /* U+1C30 */
  0x0000,   /* U+1C31 */
  0x0000,   /* U+1C32 */
  0x0000,   /* U+1C33 */
  0x0000,   /* U+1C34 */
  0x0000,   /* U+1C35 */
  0x0000,   /* U+1C36 */
  0x0000,   /* U+1C37 */
  0x9C38,   /* U+1C38 */
  0x9C39,   /* U+1C39 */
  0x9C3A,   /* U+1C3A */
  0x0000,   /* U+1C3B */
  0x0000,   /* U+1C3C */
  0x0000,   /* U+1C3D */
  0x0000,   /* U+1C3E */
  0x0000,   /* U+1C3F */
  0x0000,   /* U+1C40 */
  0x0000,   /* U+1C41 */
  0x0000,   /* U+1C42 */
  0x0000,   /* U+1C43 */
  0x0000,   /* U+1C44 */
  0x0000,   /* U+1C45 */
  0x0000,   /* U+1C46 */
  0x0000,   /* U+1C47 */
  0x0000,   /* U+1C48 */
  0x0000,   /* U+1C49 */
  0x9C4A,   /* U+1C4A */
  0x9C4B,   /* U+1C4B */
  0x9C4C,   /* U+1C4C */
  0x0000,   /* U+1C4D */
  0x0000,   /* U+1C4E */
  0x0000,   /* U+1C4F */
  0x0000,   /* U+1C50 */
  0x0000,   /* U+1C51 */
  0x0000,   /* U+1C52 */
  0x0000,   /* U+1C53 */
  0x0000,   /* U+1C54 */
  0x0000,   /* U+1C55 */
  0x0000,   /* U+1C56 */
  0x0000,   /* U+1C57 */
  0x0000,   /* U+1C58 */
  0x0000,   /* U+1C59 */
  0x0000,   /* U+1C5A */
  0x0000,   /* U+1C5B */
  0x0000,   /* U+1C5C */
  0x0000,   /* U+1C5D */
  0x0000,   /* U+1C5E */
  0x0000,   /* U+1C5F */
  0x0000,   /* U+1C60 */
  0x0000,   /* U+1C61 */
  0x0000,   /* U+1C62 */
  0x0000,   /* U+1C63 */
  0x0000,   /* U+1C64 */
  0x0000,   /* U+1C65 */
  0x0000,   /* U+1C66 */
  0x0000,   /* U+1C67 */
  0x0000,   /* U+1C68 */
  0x0000,   /* U+1C69 */
  0x0000,   /* U+1C6A */
  0x0000,   /* U+1C6B */
  0x0000,   /* U+1C6C */
  0x0000,   /* U+1C6D */
  0x0000,   /* U+1C6E */
  0x0000,   /* U+1C6F */
  0x0000,   /* U+1C70 */
  0x0000,   /* U+1C71 */
  0x0000,   /* U+1C72 */
  0x0000,   /* U+1C73 */
  0x0000,   /* U+1C74 */
  0x0000,   /* U+1C75 */
  0x0000,   /* U+1C76 */
  0x0000,   /* U+1C77 */
  0x0000,   /* U+1C78 */
  0x0000,   /* U+1C79 */
  0x0000,   /* U+1C7A */
  0x0000,   /* U+1C7B */
  0x0000,   /* U+1C7C */
  0x0000,   /* U+1C7D */
  0x0000,   /* U+1C7E */
  0x0000,   /* U+1C7F */
  0x0000,   /* U+1C80 */
  0x0000,   /* U+1C81 */
  0x0000,   /* U+1C82 */
  0x0000,   /* U+1C83 */
  0x0000,   /* U+1C84 */
  0x0000,   /* U+1C85 */
  0x0000,   /* U+1C86 */
  0x0000,   /* U+1C87 */
  0x0000,   /* U+1C88 */
  0x9C89,   /* U+1C89 */
  0x9C8A,   /* U+1C8A */
  0x9C8B,   /* U+1C8B */
  0x9C8C,   /* U+1C8C */
  0x9C8D,   /* U+1C8D */
  0x9C8E,   /* U+1C8E */
  0x9C8F,   /* U+1C8F */
  0x9C90,   /* U+1C90 */
  0x9C91,   /* U+1C91 */
  0x9C92,   /* U+1C92 */
  0x9C93,   /* U+1C93 */
  0x9C94,   /* U+1C94 */
  0x9C95,   /* U+1C95 */
  0x9C96,   /* U+1C96 */
  0x9C97,   /* U+1C97 */
  0x9C98,   /* U+1C98 */
  0x9C99,   /* U+1C99 */
  0x9C9A,   /* U+1C9A */
  0x9C9B,   /* U+1C9B */
  0x9C9C,   /* U+1C9C */
  0x9C9D,   /* U+1C9D */
  0x9C9E,   /* U+1C9E */
  0x9C9F,   /* U+1C9F */
  0x9CA0,   /* U+1CA0 */
  0x9CA1,   /* U+1CA1 */
  0x9CA2,   /* U+1CA2 */
  0x9CA3,   /* U+1CA3 */
  0x9CA4,   /* U+1CA4 */
  0x9CA5,   /* U+1CA5 */
  0x9CA6,   /* U+1CA6 */
  0x9CA7,   /* U+1CA7 */
  0x9CA8,   /* U+1CA8 */
  0x9CA9,   /* U+1CA9 */
  0x9CAA,   /* U+1CAA */
  0x9CAB,   /* U+1CAB */
  0x9CAC,   /* U+1CAC */
  0x9CAD,   /* U+1CAD */
  0x9CAE,   /* U+1CAE */
  0x9CAF,   /* U+1CAF */
  0x9CB0,   /* U+1CB0 */
  0x9CB1,   /* U+1CB1 */
  0x9CB2,   /* U+1CB2 */
  0x9CB3,   /* U+1CB3 */
  0x9CB4,   /* U+1CB4 */
  0x9CB5,   /* U+1CB5 */
  0x9CB6,   /* U+1CB6 */
  0x9CB7,   /* U+1CB7 */
  0x9CB8,   /* U+1CB8 */
  0x9CB9,   /* U+1CB9 */
  0x9CBA,   /* U+1CBA */
  0x9CBB,   /* U+1CBB */
  0x9CBC,   /* U+1CBC */
  0x9CBD,   /* U+1CBD */
  0x9CBE,   /* U+1CBE */
  0x9CBF,   /* U+1CBF */
  0x0000,   /* U+1CC0 */
  0x0000,   /* U+1CC1 */
  0x0000,   /* U+1CC2 */
  0x0000,   /* U+1CC3 */
  0x0000,   /* U+1CC4 */
  0x0000,   /* U+1CC5 */
  0x0000,   /* U+1CC6 */
  0x0000,   /* U+1CC7 */
  0x9CC8,   /* U+1CC8 */
  0x9CC9,   /* U+1CC9 */
  0x9CCA,   /* U+1CCA */
  0x9CCB,   /* U+1CCB */
  0x9CCC,   /* U+1CCC */
  0x9CCD,   /* U+1CCD */
  0x9CCE,   /* U+1CCE */
  0x9CCF,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x0000,   /* U+1CE9 */
  0x0000,   /* U+1CEA */
  0x0000,   /* U+1CEB */
  0x0000,   /* U+1CEC */
  0x0000,   /* U+1CED */
  0x0000,   /* U+1CEE */
  0x0000,   /* U+1CEF */
  0x0000,   /* U+1CF0 */
  0x0000,   /* U+1CF1 */
  0x0000,   /* U+1CF2 */
  0x0000,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x0000,   /* U+1CF5 */
  0x0000,   /* U+1CF6 */
  0x9CF7,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0x9CFA,   /* U+1CFA */
  0x9CFB,   /* U+1CFB */
  0x9CFC,   /* U+1CFC */
  0x9CFD,   /* U+1CFD */
  0x9CFE,   /* U+1CFE */
  0x9CFF,   /* U+1CFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1C00 */
  0x0000,   /* U+1C01 */
  0x0000,   /* U+1C02 */
  0x0000,   /* U+1C03 */
  0x0000,   /* U+1C04 */
  0x0000,   /* U+1C05 */
  0x0000,   /* U+1C06 */
  0x0000,   /* U+1C07 */
  0x0000,   /* U+1C08 */
  0x0000,   /* U+1C09 */
  0x0000,   /* U+1C0A */
  0x0000,   /* U+1C0B */
  0x0000,   /* U+1C0C */
  0x0000,   /* U+1C0D */
  0x0000,   /* U+1C0E */
  0x0000,   /* U+1C0F */
  0x0000,   /* U+1C10 */
  0x0000,   /* U+1C11 */
  0x0000,   /* U+1C12 */
  0x0000,   /* U+1C13 */
  0x0000,   /* U+1C14 */
  0x0000,   /* U+1C15 */
  0x0000,   /* U+1C16 */
  0x0000,   /* U+1C17 */
  0x0000,   /* U+1C18 */
  0x0000,   /* U+1C19 */
  0x0000,   /* U+1C1A */
  0x0000,   /* U+1C1B */
  0x0000,   /* U+1C1C */
  0x0000,   /* U+1C1D */
  0x0000,   /* U+1C1E */
  0x0000,   /* U+1C1F */
  0x0000,   /* U+1C20 */
  0x0000,   /* U+1C21 */
  0x0000,   /* U+1C22 */
  0x0000,   /* U+1C23 */
  0x0000,   /* U+1C24 */
  0x0000,   /* U+1C25 */
  0x0000,   /* U+1C26 */
  0x0000,   /* U+1C27 */
  0x0000,   /* U+1C28 */
  0x0000,   /* U+1C29 */
  0x0000,   /* U+1C2A */
  0x0000,   /* U+1C2B */
  0x0000,   /* U+1C2C */
  0x0000,   /* U+1C2D */
  0x0000,   /* U+1C2E */
  0x0000,   /* U+1C2F */
  0x0000,   /* U+1C30 */
  0x0000,   /* U+1C31 */
  0x0000,   /* U+1C32 */
  0x0000,   /* U+1C33 */
  0x0000,   /* U+1C34 */
  0x0000,   /* U+1C35 */
  0x0000,   /* U+1C36 */
  0x0000,   /* U+1C37 */
  0x0000,   /* U+1C38 */
  0x0000,   /* U+1C39 */
  0x0000,   /* U+1C3A */
  0x0000,   /* U+1C3B */
  0x0000,   /* U+1C3C */
  0x0000,   /* U+1C3D */
  0x0000,   /* U+1C3E */
  0x0000,   /* U+1C3F */
  0x0000,   /* U+1C40 */
  0x0000,   /* U+1C41 */
  0x0000,   /* U+1C42 */
  0x0000,   /* U+1C43 */
  0x0000,   /* U+1C44 */
  0x0000,   /* U+1C45 */
  0x0000,   /* U+1C46 */
  0x0000,   /* U+1C47 */
  0x0000,   /* U+1C48 */
  0x0000,   /* U+1C49 */
  0x0000,   /* U+1C4A */
  0x0000,   /* U+1C4B */
  0x0000,   /* U+1C4C */
  0x0000,   /* U+1C4D */
  0x0000,   /* U+1C4E */
  0x0000,   /* U+1C4F */
  0x0000,   /* U+1C50 */
  0x0000,   /* U+1C51 */
  0x0000,   /* U+1C52 */
  0x0000,   /* U+1C53 */
  0x0000,   /* U+1C54 */
  0x0000,   /* U+1C55 */
  0x0000,   /* U+1C56 */
  0x0000,   /* U+1C57 */
  0x0000,   /* U+1C58 */
  0x0000,   /* U+1C59 */
  0x0000,   /* U+1C5A */
  0x0000,   /* U+1C5B */
  0x0000,   /* U+1C5C */
  0x0000,   /* U+1C5D */
  0x0000,   /* U+1C5E */
  0x0000,   /* U+1C5F */
  0x0000,   /* U+1C60 */
  0x0000,   /* U+1C61 */
  0x0000,   /* U+1C62 */
  0x0000,   /* U+1C63 */
  0x0000,   /* U+1C64 */
  0x0000,   /* U+1C65 */
  0x0000,   /* U+1C66 */
  0x0000,   /* U+1C67 */
  0x0000,   /* U+1C68 */
  0x0000,   /* U+1C69 */
  0x0000,   /* U+1C6A */
  0x0000,   /* U+1C6B */
  0x0000,   /* U+1C6C */
  0x0000,   /* U+1C6D */
  0x0000,   /* U+1C6E */
  0x0000,   /* U+1C6F */
  0x0000,   /* U+1C70 */
  0x0000,   /* U+1C71 */
  0x0000,   /* U+1C72 */
  0x0000,   /* U+1C73 */
  0x0000,   /* U+1C74 */
  0x0000,   /* U+1C75 */
  0x0000,   /* U+1C76 */
  0x0000,   /* U+1C77 */
  0x0000,   /* U+1C78 */
  0x0000,   /* U+1C79 */
  0x0000,   /* U+1C7A */
  0x0000,   /* U+1C7B */
  0x0000,   /* U+1C7C */
  0x0000,   /* U+1C7D */
  0x0000,   /* U+1C7E */
  0x0000,   /* U+1C7F */
  0x0000,   /* U+1C80 */
  0x0000,   /* U+1C81 */
  0x0000,   /* U+1C82 */
  0x0000,   /* U+1C83 */
  0x0000,   /* U+1C84 */
  0x0000,   /* U+1C85 */
  0x0000,   /* U+1C86 */
  0x0000,   /* U+1C87 */
  0x0000,   /* U+1C88 */
  0x0000,   /* U+1C89 */
  0x0000,   /* U+1C8A */
  0x0000,   /* U+1C8B */
  0x0000,   /* U+1C8C */
  0x0000,   /* U+1C8D */
  0x0000,   /* U+1C8E */
  0x0000,   /* U+1C8F */
  0x0000,   /* U+1C90 */
  0x0000,   /* U+1C91 */
  0x0000,   /* U+1C92 */
  0x0000,   /* U+1C93 */
  0x0000,   /* U+1C94 */
  0x0000,   /* U+1C95 */
  0x0000,   /* U+1C96 */
  0x0000,   /* U+1C97 */
  0x0000,   /* U+1C98 */
  0x0000,   /* U+1C99 */
  0x0000,   /* U+1C9A */
  0x0000,   /* U+1C9B */
  0x0000,   /* U+1C9C */
  0x0000,   /* U+1C9D */
  0x0000,   /* U+1C9E */
  0x0000,   /* U+1C9F */
  0x0000,   /* U+1CA0 */
  0x0000,   /* U+1CA1 */
  0x0000,   /* U+1CA2 */
  0x0000,   /* U+1CA3 */
  0x0000,   /* U+1CA4 */
  0x0000,   /* U+1CA5 */
  0x0000,   /* U+1CA6 */
  0x0000,   /* U+1CA7 */
  0x0000,   /* U+1CA8 */
  0x0000,   /* U+1CA9 */
  0x0000,   /* U+1CAA */
  0x0000,   /* U+1CAB */
  0x0000,   /* U+1CAC */
  0x0000,   /* U+1CAD */
  0x0000,   /* U+1CAE */
  0x0000,   /* U+1CAF */
  0x0000,   /* U+1CB0 */
  0x0000,   /* U+1CB1 */
  0x0000,   /* U+1CB2 */
  0x0000,   /* U+1CB3 */
  0x0000,   /* U+1CB4 */
  0x0000,   /* U+1CB5 */
  0x0000,   /* U+1CB6 */
  0x0000,   /* U+1CB7 */
  0x0000,   /* U+1CB8 */
  0x0000,   /* U+1CB9 */
  0x0000,   /* U+1CBA */
  0x0000,   /* U+1CBB */
  0x0000,   /* U+1CBC */
  0x0000,   /* U+1CBD */
  0x0000,   /* U+1CBE */
  0x0000,   /* U+1CBF */
  0x0000,   /* U+1CC0 */
  0x0000,   /* U+1CC1 */
  0x0000,   /* U+1CC2 */
  0x0000,   /* U+1CC3 */
  0x0000,   /* U+1CC4 */
  0x0000,   /* U+1CC5 */
  0x0000,   /* U+1CC6 */
  0x0000,   /* U+1CC7 */
  0x0000,   /* U+1CC8 */
  0x0000,   /* U+1CC9 */
  0x0000,   /* U+1CCA */
  0x0000,   /* U+1CCB */
  0x0000,   /* U+1CCC */
  0x0000,   /* U+1CCD */
  0x0000,   /* U+1CCE */
  0x0000,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x0000,   /* U+1CE9 */
  0x0000,   /* U+1CEA */
  0x0000,   /* U+1CEB */
  0x0000,   /* U+1CEC */
  0x0000,   /* U+1CED */
  0x0000,   /* U+1CEE */
  0x0000,   /* U+1CEF */
  0x0000,   /* U+1CF0 */
  0x0000,   /* U+1CF1 */
  0x0000,   /* U+1CF2 */
  0x0000,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x0000,   /* U+1CF5 */
  0x0000,   /* U+1CF6 */
  0x0000,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0x0000,   /* U+1CFA */
  0x0000,   /* U+1CFB */
  0x0000,   /* U+1CFC */
  0x0000,   /* U+1CFD */
  0x0000,   /* U+1CFE */
  0x0000,   /* U+1CFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1C00 */
  0x0000,   /* U+1C01 */
  0x0000,   /* U+1C02 */
  0x0000,   /* U+1C03 */
  0x0000,   /* U+1C04 */
  0x0000,   /* U+1C05 */
  0x0000,   /* U+1C06 */
  0x0000,   /* U+1C07 */
  0x0000,   /* U+1C08 */
  0x0000,   /* U+1C09 */
  0x0000,   /* U+1C0A */
  0x0000,   /* U+1C0B */
  0x0000,   /* U+1C0C */
  0x0000,   /* U+1C0D */
  0x0000,   /* U+1C0E */
  0x0000,   /* U+1C0F */
  0x0000,   /* U+1C10 */
  0x0000,   /* U+1C11 */
  0x0000,   /* U+1C12 */
  0x0000,   /* U+1C13 */
  0x0000,   /* U+1C14 */
  0x0000,   /* U+1C15 */
  0x0000,   /* U+1C16 */
  0x0000,   /* U+1C17 */
  0x0000,   /* U+1C18 */
  0x0000,   /* U+1C19 */
  0x0000,   /* U+1C1A */
  0x0000,   /* U+1C1B */
  0x0000,   /* U+1C1C */
  0x0000,   /* U+1C1D */
  0x0000,   /* U+1C1E */
  0x0000,   /* U+1C1F */
  0x0000,   /* U+1C20 */
  0x0000,   /* U+1C21 */
  0x0000,   /* U+1C22 */
  0x0000,   /* U+1C23 */
  0x0000,   /* U+1C24 */
  0x0000,   /* U+1C25 */
  0x0000,   /* U+1C26 */
  0x0000,   /* U+1C27 */
  0x0000,   /* U+1C28 */
  0x0000,   /* U+1C29 */
  0x0000,   /* U+1C2A */
  0x0000,   /* U+1C2B */
  0x0000,   /* U+1C2C */
  0x0000,   /* U+1C2D */
  0x0000,   /* U+1C2E */
  0x0000,   /* U+1C2F */
  0x0000,   /* U+1C30 */
  0x0000,   /* U+1C31 */
  0x0000,   /* U+1C32 */
  0x0000,   /* U+1C33 */
  0x0000,   /* U+1C34 */
  0x0000,   /* U+1C35 */
  0x0000,   /* U+1C36 */
  0x0000,   /* U+1C37 */
  0x0000,   /* U+1C38 */
  0x0000,   /* U+1C39 */
  0x0000,   /* U+1C3A */
  0x0000,   /* U+1C3B */
  0x0000,   /* U+1C3C */
  0x0000,   /* U+1C3D */
  0x0000,   /* U+1C3E */
  0x0000,   /* U+1C3F */
  0x0000,   /* U+1C40 */
  0x0000,   /* U+1C41 */
  0x0000,   /* U+1C42 */
  0x0000,   /* U+1C43 */
  0x0000,   /* U+1C44 */
  0x0000,   /* U+1C45 */
  0x0000,   /* U+1C46 */
  0x0000,   /* U+1C47 */
  0x0000,   /* U+1C48 */
  0x0000,   /* U+1C49 */
  0x0000,   /* U+1C4A */
  0x0000,   /* U+1C4B */
  0x0000,   /* U+1C4C */
  0x0000,   /* U+1C4D */
  0x0000,   /* U+1C4E */
  0x0000,   /* U+1C4F */
  0x0000,   /* U+1C50 */
  0x0000,   /* U+1C51 */
  0x0000,   /* U+1C52 */
  0x0000,   /* U+1C53 */
  0x0000,   /* U+1C54 */
  0x0000,   /* U+1C55 */
  0x0000,   /* U+1C56 */
  0x0000,   /* U+1C57 */
  0x0000,   /* U+1C58 */
  0x0000,   /* U+1C59 */
  0x0000,   /* U+1C5A */
  0x0000,   /* U+1C5B */
  0x0000,   /* U+1C5C */
  0x0000,   /* U+1C5D */
  0x0000,   /* U+1C5E */
  0x0000,   /* U+1C5F */
  0x0000,   /* U+1C60 */
  0x0000,   /* U+1C61 */
  0x0000,   /* U+1C62 */
  0x0000,   /* U+1C63 */
  0x0000,   /* U+1C64 */
  0x0000,   /* U+1C65 */
  0x0000,   /* U+1C66 */
  0x0000,   /* U+1C67 */
  0x0000,   /* U+1C68 */
  0x0000,   /* U+1C69 */
  0x0000,   /* U+1C6A */
  0x0000,   /* U+1C6B */
  0x0000,   /* U+1C6C */
  0x0000,   /* U+1C6D */
  0x0000,   /* U+1C6E */
  0x0000,   /* U+1C6F */
  0x0000,   /* U+1C70 */
  0x0000,   /* U+1C71 */
  0x0000,   /* U+1C72 */
  0x0000,   /* U+1C73 */
  0x0000,   /* U+1C74 */
  0x0000,   /* U+1C75 */
  0x0000,   /* U+1C76 */
  0x0000,   /* U+1C77 */
  0x0000,   /* U+1C78 */
  0x0000,   /* U+1C79 */
  0x0000,   /* U+1C7A */
  0x0000,   /* U+1C7B */
  0x0000,   /* U+1C7C */
  0x0000,   /* U+1C7D */
  0x0000,   /* U+1C7E */
  0x0000,   /* U+1C7F */
  0x0000,   /* U+1C80 */
  0x0000,   /* U+1C81 */
  0x0000,   /* U+1C82 */
  0x0000,   /* U+1C83 */
  0x0000,   /* U+1C84 */
  0x0000,   /* U+1C85 */
  0x0000,   /* U+1C86 */
  0x0000,   /* U+1C87 */
  0x0000,   /* U+1C88 */
  0x0000,   /* U+1C89 */
  0x0000,   /* U+1C8A */
  0x0000,   /* U+1C8B */
  0x0000,   /* U+1C8C */
  0x0000,   /* U+1C8D */
  0x0000,   /* U+1C8E */
  0x0000,   /* U+1C8F */
  0x0000,   /* U+1C90 */
  0x0000,   /* U+1C91 */
  0x0000,   /* U+1C92 */
  0x0000,   /* U+1C93 */
  0x0000,   /* U+1C94 */
  0x0000,   /* U+1C95 */
  0x0000,   /* U+1C96 */
  0x0000,   /* U+1C97 */
  0x0000,   /* U+1C98 */
  0x0000,   /* U+1C99 */
  0x0000,   /* U+1C9A */
  0x0000,   /* U+1C9B */
  0x0000,   /* U+1C9C */
  0x0000,   /* U+1C9D */
  0x0000,   /* U+1C9E */
  0x0000,   /* U+1C9F */
  0x0000,   /* U+1CA0 */
  0x0000,   /* U+1CA1 */
  0x0000,   /* U+1CA2 */
  0x0000,   /* U+1CA3 */
  0x0000,   /* U+1CA4 */
  0x0000,   /* U+1CA5 */
  0x0000,   /* U+1CA6 */
  0x0000,   /* U+1CA7 */
  0x0000,   /* U+1CA8 */
  0x0000,   /* U+1CA9 */
  0x0000,   /* U+1CAA */
  0x0000,   /* U+1CAB */
  0x0000,   /* U+1CAC */
  0x0000,   /* U+1CAD */
  0x0000,   /* U+1CAE */
  0x0000,   /* U+1CAF */
  0x0000,   /* U+1CB0 */
  0x0000,   /* U+1CB1 */
  0x0000,   /* U+1CB2 */
  0x0000,   /* U+1CB3 */
  0x0000,   /* U+1CB4 */
  0x0000,   /* U+1CB5 */
  0x0000,   /* U+1CB6 */
  0x0000,   /* U+1CB7 */
  0x0000,   /* U+1CB8 */
  0x0000,   /* U+1CB9 */
  0x0000,   /* U+1CBA */
  0x0000,   /* U+1CBB */
  0x0000,   /* U+1CBC */
  0x0000,   /* U+1CBD */
  0x0000,   /* U+1CBE */
  0x0000,   /* U+1CBF */
  0x0000,   /* U+1CC0 */
  0x0000,   /* U+1CC1 */
  0x0000,   /* U+1CC2 */
  0x0000,   /* U+1CC3 */
  0x0000,   /* U+1CC4 */
  0x0000,   /* U+1CC5 */
  0x0000,   /* U+1CC6 */
  0x0000,   /* U+1CC7 */
  0x0000,   /* U+1CC8 */
  0x0000,   /* U+1CC9 */
  0x0000,   /* U+1CCA */
  0x0000,   /* U+1CCB */
  0x0000,   /* U+1CCC */
  0x0000,   /* U+1CCD */
  0x0000,   /* U+1CCE */
  0x0000,   /* U+1CCF */
  0x0000,   /* U+1CD0 */
  0x0000,   /* U+1CD1 */
  0x0000,   /* U+1CD2 */
  0x0000,   /* U+1CD3 */
  0x0000,   /* U+1CD4 */
  0x0000,   /* U+1CD5 */
  0x0000,   /* U+1CD6 */
  0x0000,   /* U+1CD7 */
  0x0000,   /* U+1CD8 */
  0x0000,   /* U+1CD9 */
  0x0000,   /* U+1CDA */
  0x0000,   /* U+1CDB */
  0x0000,   /* U+1CDC */
  0x0000,   /* U+1CDD */
  0x0000,   /* U+1CDE */
  0x0000,   /* U+1CDF */
  0x0000,   /* U+1CE0 */
  0x0000,   /* U+1CE1 */
  0x0000,   /* U+1CE2 */
  0x0000,   /* U+1CE3 */
  0x0000,   /* U+1CE4 */
  0x0000,   /* U+1CE5 */
  0x0000,   /* U+1CE6 */
  0x0000,   /* U+1CE7 */
  0x0000,   /* U+1CE8 */
  0x0000,   /* U+1CE9 */
  0x0000,   /* U+1CEA */
  0x0000,   /* U+1CEB */
  0x0000,   /* U+1CEC */
  0x0000,   /* U+1CED */
  0x0000,   /* U+1CEE */
  0x0000,   /* U+1CEF */
  0x0000,   /* U+1CF0 */
  0x0000,   /* U+1CF1 */
  0x0000,   /* U+1CF2 */
  0x0000,   /* U+1CF3 */
  0x0000,   /* U+1CF4 */
  0x0000,   /* U+1CF5 */
  0x0000,   /* U+1CF6 */
  0x0000,   /* U+1CF7 */
  0x0000,   /* U+1CF8 */
  0x0000,   /* U+1CF9 */
  0x0000,   /* U+1CFA */
  0x0000,   /* U+1CFB */
  0x0000,   /* U+1CFC */
  0x0000,   /* U+1CFD */
  0x0000,   /* U+1CFE */
  0x0000,   /* U+1CFF */
};

uint16 uca900_p01D[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 3, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 3, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  2, 1, 2, 2, 2, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C4B,   /* U+1D00 */
  0x1C4E,   /* U+1D01 */
  0x1C4F,   /* U+1D02 */
  0x1C6D,   /* U+1D03 */
  0x1C7E,   /* U+1D04 */
  0x1C93,   /* U+1D05 */
  0x1C94,   /* U+1D06 */
  0x1CAE,   /* U+1D07 */
  0x1CD4,   /* U+1D08 */
  0x1D40,   /* U+1D09 */
  0x1D54,   /* U+1D0A */
  0x1D69,   /* U+1D0B */
  0x1D80,   /* U+1D0C */
  0x1DAE,   /* U+1D0D */
  0x1DC2,   /* U+1D0E */
  0x1DE1,   /* U+1D0F */
  0x1DF4,   /* U+1D10 */
  0x1DE2,   /* U+1D11 */
  0x1DF5,   /* U+1D12 */
  0x1DEE,   /* U+1D13 */
  0x1DE8,   /* U+1D14 */
  0x1E0B,   /* U+1D15 */
  0x1DFA,   /* U+1D16 */
  0x1DFB,   /* U+1D17 */
  0x1E10,   /* U+1D18 */
  0x1E3E,   /* U+1D19 */
  0x1E48,   /* U+1D1A */
  0x1E99,   /* U+1D1B */
  0x1EB9,   /* U+1D1C */
  0x1EBB,   /* U+1D1D */
  0x1EBC,   /* U+1D1E */
  0x1ED9,   /* U+1D1F */
  0x1EE7,   /* U+1D20 */
  0x1EF9,   /* U+1D21 */
  0x1F25,   /* U+1D22 */
  0x1F42,   /* U+1D23 */
  0x1F8A,   /* U+1D24 */
  0x1F8B,   /* U+1D25 */
  0x1FBC,   /* U+1D26 */
  0x1FCA,   /* U+1D27 */
  0x1FD0,   /* U+1D28 */
  0x1FD5,   /* U+1D29 */
  0x1FE0,   /* U+1D2A */
  0x20B4,   /* U+1D2B */
  0x1C47,   /* U+1D2C */
  0x1C47,   /* U+1D2D */
  0x1C60,   /* U+1D2E */
  0x1C6C,   /* U+1D2F */
  0x1C8F,   /* U+1D30 */
  0x1CAA,   /* U+1D31 */
  0x1CB8,   /* U+1D32 */
  0x1CF4,   /* U+1D33 */
  0x1D18,   /* U+1D34 */
  0x1D32,   /* U+1D35 */
  0x1D4C,   /* U+1D36 */
  0x1D65,   /* U+1D37 */
  0x1D77,   /* U+1D38 */
  0x1DAA,   /* U+1D39 */
  0x1DB9,   /* U+1D3A */
  0x1DC1,   /* U+1D3B */
  0x1DDD,   /* U+1D3C */
  0x1E07,   /* U+1D3D */
  0x1E0C,   /* U+1D3E */
  0x1E33,   /* U+1D3F */
  0x1E95,   /* U+1D40 */
  0x1EB5,   /* U+1D41 */
  0x1EF5,   /* U+1D42 */
  0x1C47,   /* U+1D43 */
  0x1C51,   /* U+1D44 */
  0x1C55,   /* U+1D45 */
  0x1C4F,   /* U+1D46 */
  0x1C60,   /* U+1D47 */
  0x1C8F,   /* U+1D48 */
  0x1CAA,   /* U+1D49 */
  0x1CBD,   /* U+1D4A */
  0x1CC2,   /* U+1D4B */
  0x1CD4,   /* U+1D4C */
  0x1CF4,   /* U+1D4D */
  0x1D40,   /* U+1D4E */
  0x1D65,   /* U+1D4F */
  0x1DAA,   /* U+1D50 */
  0x1DD8,   /* U+1D51 */
  0x1DDD,   /* U+1D52 */
  0x1DF0,   /* U+1D53 */
  0x1DFA,   /* U+1D54 */
  0x1DFB,   /* U+1D55 */
  0x1E0C,   /* U+1D56 */
  0x1E95,   /* U+1D57 */
  0x1EB5,   /* U+1D58 */
  0x1EBB,   /* U+1D59 */
  0x1ED4,   /* U+1D5A */
  0x1EE3,   /* U+1D5B */
  0x1F8B,   /* U+1D5C */
  0x1FBA,   /* U+1D5D */
  0x1FBB,   /* U+1D5E */
  0x1FBD,   /* U+1D5F */
  0x1FDD,   /* U+1D60 */
  0x1FDE,   /* U+1D61 */
  0x1D32,   /* U+1D62 */
  0x1E33,   /* U+1D63 */
  0x1EB5,   /* U+1D64 */
  0x1EE3,   /* U+1D65 */
  0x1FBA,   /* U+1D66 */
  0x1FBB,   /* U+1D67 */
  0x1FD4,   /* U+1D68 */
  0x1FDD,   /* U+1D69 */
  0x1FDE,   /* U+1D6A */
  0x1EBD,   /* U+1D6B */
  0x1C6E,   /* U+1D6C */
  0x1C95,   /* U+1D6D */
  0x1CEC,   /* U+1D6E */
  0x1DAF,   /* U+1D6F */
  0x1DC3,   /* U+1D70 */
  0x1E13,   /* U+1D71 */
  0x1E43,   /* U+1D72 */
  0x1E60,   /* U+1D73 */
  0x1E76,   /* U+1D74 */
  0x1E9F,   /* U+1D75 */
  0x1F2A,   /* U+1D76 */
  0x1D0E,   /* U+1D77 */
  0x20CC,   /* U+1D78 */
  0x1CF4,   /* U+1D79 */
  0x1E95,   /* U+1D7A */
  0x1D45,   /* U+1D7B */
  0x1D4B,   /* U+1D7C */
  0x1E11,   /* U+1D7D */
  0x1EC5,   /* U+1D7E */
  0x1EE2,   /* U+1D7F */
  0x1C70,   /* U+1D80 */
  0x1C96,   /* U+1D81 */
  0x1CED,   /* U+1D82 */
  0x1D05,   /* U+1D83 */
  0x1D6A,   /* U+1D84 */
  0x1D92,   /* U+1D85 */
  0x1DB0,   /* U+1D86 */
  0x1DCD,   /* U+1D87 */
  0x1E14,   /* U+1D88 */
  0x1E4D,   /* U+1D89 */
  0x1E77,   /* U+1D8A */
  0x1E87,   /* U+1D8B */
  0x1EE9,   /* U+1D8C */
  0x1F03,   /* U+1D8D */
  0x1F2B,   /* U+1D8E */
  0x1C4D,   /* U+1D8F */
  0x1C5A,   /* U+1D90 */
  0x1C9F,   /* U+1D91 */
  0x1CB5,   /* U+1D92 */
  0x1CC6,   /* U+1D93 */
  0x1CD3,   /* U+1D94 */
  0x1CC1,   /* U+1D95 */
  0x1D46,   /* U+1D96 */
  0x1DF7,   /* U+1D97 */
  0x1E90,   /* U+1D98 */
  0x1EC6,   /* U+1D99 */
  0x1F47,   /* U+1D9A */
  0x1C5B,   /* U+1D9B */
  0x1C7A,   /* U+1D9C */
  0x1C89,   /* U+1D9D */
  0x1C8F,   /* U+1D9E */
  0x1CCF,   /* U+1D9F */
  0x1CE5,   /* U+1DA0 */
  0x1D5D,   /* U+1DA1 */
  0x1CF8,   /* U+1DA2 */
  0x1EC8,   /* U+1DA3 */
  0x1D41,   /* U+1DA4 */
  0x1D47,   /* U+1DA5 */
  0x1D3A,   /* U+1DA6 */
  0x1D45,   /* U+1DA7 */
  0x1D59,   /* U+1DA8 */
  0x1D93,   /* U+1DA9 */
  0x1D92,   /* U+1DAA */
  0x1D7B,   /* U+1DAB */
  0x1DB1,   /* U+1DAC */
  0x1EDA,   /* U+1DAD */
  0x1DC4,   /* U+1DAE */
  0x1DCE,   /* U+1DAF */
  0x1DBD,   /* U+1DB0 */
  0x1DFD,   /* U+1DB1 */
  0x1E1C,   /* U+1DB2 */
  0x1E78,   /* U+1DB3 */
  0x1E82,   /* U+1DB4 */
  0x1EA0,   /* U+1DB5 */
  0x1EC0,   /* U+1DB6 */
  0x1EDE,   /* U+1DB7 */
  0x1EB9,   /* U+1DB8 */
  0x1EEA,   /* U+1DB9 */
  0x1EF1,   /* U+1DBA */
  0x1F21,   /* U+1DBB */
  0x1F30,   /* U+1DBC */
  0x1F34,   /* U+1DBD */
  0x1F3E,   /* U+1DBE */
  0x1FC5,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x1E33,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x1F60,   /* U+1DD2 */
  0x1C47,   /* U+1DD3 */
  0x1C47,   /* U+1DD4 */
  0x1C47,   /* U+1DD5 */
  0x1C47,   /* U+1DD6 */
  0x1C7A,   /* U+1DD7 */
  0x1C8F,   /* U+1DD8 */
  0x1C8F,   /* U+1DD9 */
  0x1CF4,   /* U+1DDA */
  0x1CFD,   /* U+1DDB */
  0x1D65,   /* U+1DDC */
  0x1D77,   /* U+1DDD */
  0x1D7B,   /* U+1DDE */
  0x1DAE,   /* U+1DDF */
  0x1DB9,   /* U+1DE0 */
  0x1DBD,   /* U+1DE1 */
  0x1E38,   /* U+1DE2 */
  0x1E3D,   /* U+1DE3 */
  0x1E71,   /* U+1DE4 */
  0x1E71,   /* U+1DE5 */
  0x1F21,   /* U+1DE6 */
  0x1C55,   /* U+1DE7 */
  0x1C60,   /* U+1DE8 */
  0x1C79,   /* U+1DE9 */
  0x1CBD,   /* U+1DEA */
  0x1CE5,   /* U+1DEB */
  0x1D8B,   /* U+1DEC */
  0x1DDD,   /* U+1DED */
  0x1E0C,   /* U+1DEE */
  0x1E82,   /* U+1DEF */
  0x1EB5,   /* U+1DF0 */
  0x1EF5,   /* U+1DF1 */
  0x1C47,   /* U+1DF2 */
  0x1DDD,   /* U+1DF3 */
  0x1EB5,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0xFBC0,   /* U+1DF6 */
  0xFBC0,   /* U+1DF7 */
  0xFBC0,   /* U+1DF8 */
  0xFBC0,   /* U+1DF9 */
  0xFBC0,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D00 */
  0x0020,   /* U+1D01 */
  0x0020,   /* U+1D02 */
  0x0020,   /* U+1D03 */
  0x0020,   /* U+1D04 */
  0x0020,   /* U+1D05 */
  0x0020,   /* U+1D06 */
  0x0020,   /* U+1D07 */
  0x0020,   /* U+1D08 */
  0x0020,   /* U+1D09 */
  0x0020,   /* U+1D0A */
  0x0020,   /* U+1D0B */
  0x0020,   /* U+1D0C */
  0x0020,   /* U+1D0D */
  0x0020,   /* U+1D0E */
  0x0020,   /* U+1D0F */
  0x0020,   /* U+1D10 */
  0x0020,   /* U+1D11 */
  0x0020,   /* U+1D12 */
  0x0020,   /* U+1D13 */
  0x0020,   /* U+1D14 */
  0x0020,   /* U+1D15 */
  0x0020,   /* U+1D16 */
  0x0020,   /* U+1D17 */
  0x0020,   /* U+1D18 */
  0x0020,   /* U+1D19 */
  0x0020,   /* U+1D1A */
  0x0020,   /* U+1D1B */
  0x0020,   /* U+1D1C */
  0x0020,   /* U+1D1D */
  0x0020,   /* U+1D1E */
  0x0020,   /* U+1D1F */
  0x0020,   /* U+1D20 */
  0x0020,   /* U+1D21 */
  0x0020,   /* U+1D22 */
  0x0020,   /* U+1D23 */
  0x0020,   /* U+1D24 */
  0x0020,   /* U+1D25 */
  0x0020,   /* U+1D26 */
  0x0020,   /* U+1D27 */
  0x0020,   /* U+1D28 */
  0x0020,   /* U+1D29 */
  0x0020,   /* U+1D2A */
  0x0020,   /* U+1D2B */
  0x0020,   /* U+1D2C */
  0x0020,   /* U+1D2D */
  0x0020,   /* U+1D2E */
  0x0020,   /* U+1D2F */
  0x0020,   /* U+1D30 */
  0x0020,   /* U+1D31 */
  0x0020,   /* U+1D32 */
  0x0020,   /* U+1D33 */
  0x0020,   /* U+1D34 */
  0x0020,   /* U+1D35 */
  0x0020,   /* U+1D36 */
  0x0020,   /* U+1D37 */
  0x0020,   /* U+1D38 */
  0x0020,   /* U+1D39 */
  0x0020,   /* U+1D3A */
  0x0020,   /* U+1D3B */
  0x0020,   /* U+1D3C */
  0x0020,   /* U+1D3D */
  0x0020,   /* U+1D3E */
  0x0020,   /* U+1D3F */
  0x0020,   /* U+1D40 */
  0x0020,   /* U+1D41 */
  0x0020,   /* U+1D42 */
  0x0020,   /* U+1D43 */
  0x0020,   /* U+1D44 */
  0x0020,   /* U+1D45 */
  0x0020,   /* U+1D46 */
  0x0020,   /* U+1D47 */
  0x0020,   /* U+1D48 */
  0x0020,   /* U+1D49 */
  0x0020,   /* U+1D4A */
  0x0020,   /* U+1D4B */
  0x0020,   /* U+1D4C */
  0x0020,   /* U+1D4D */
  0x0020,   /* U+1D4E */
  0x0020,   /* U+1D4F */
  0x0020,   /* U+1D50 */
  0x0020,   /* U+1D51 */
  0x0020,   /* U+1D52 */
  0x0020,   /* U+1D53 */
  0x0020,   /* U+1D54 */
  0x0020,   /* U+1D55 */
  0x0020,   /* U+1D56 */
  0x0020,   /* U+1D57 */
  0x0020,   /* U+1D58 */
  0x0020,   /* U+1D59 */
  0x0020,   /* U+1D5A */
  0x0020,   /* U+1D5B */
  0x0020,   /* U+1D5C */
  0x0020,   /* U+1D5D */
  0x0020,   /* U+1D5E */
  0x0020,   /* U+1D5F */
  0x0020,   /* U+1D60 */
  0x0020,   /* U+1D61 */
  0x0020,   /* U+1D62 */
  0x0020,   /* U+1D63 */
  0x0020,   /* U+1D64 */
  0x0020,   /* U+1D65 */
  0x0020,   /* U+1D66 */
  0x0020,   /* U+1D67 */
  0x0020,   /* U+1D68 */
  0x0020,   /* U+1D69 */
  0x0020,   /* U+1D6A */
  0x0020,   /* U+1D6B */
  0x0020,   /* U+1D6C */
  0x0020,   /* U+1D6D */
  0x0020,   /* U+1D6E */
  0x0020,   /* U+1D6F */
  0x0020,   /* U+1D70 */
  0x0020,   /* U+1D71 */
  0x0020,   /* U+1D72 */
  0x0020,   /* U+1D73 */
  0x0020,   /* U+1D74 */
  0x0020,   /* U+1D75 */
  0x0020,   /* U+1D76 */
  0x0020,   /* U+1D77 */
  0x0020,   /* U+1D78 */
  0x0020,   /* U+1D79 */
  0x0020,   /* U+1D7A */
  0x0020,   /* U+1D7B */
  0x0020,   /* U+1D7C */
  0x0020,   /* U+1D7D */
  0x0020,   /* U+1D7E */
  0x0020,   /* U+1D7F */
  0x0020,   /* U+1D80 */
  0x0020,   /* U+1D81 */
  0x0020,   /* U+1D82 */
  0x0020,   /* U+1D83 */
  0x0020,   /* U+1D84 */
  0x0020,   /* U+1D85 */
  0x0020,   /* U+1D86 */
  0x0020,   /* U+1D87 */
  0x0020,   /* U+1D88 */
  0x0020,   /* U+1D89 */
  0x0020,   /* U+1D8A */
  0x0020,   /* U+1D8B */
  0x0020,   /* U+1D8C */
  0x0020,   /* U+1D8D */
  0x0020,   /* U+1D8E */
  0x0020,   /* U+1D8F */
  0x0020,   /* U+1D90 */
  0x0020,   /* U+1D91 */
  0x0020,   /* U+1D92 */
  0x0020,   /* U+1D93 */
  0x0020,   /* U+1D94 */
  0x0020,   /* U+1D95 */
  0x0020,   /* U+1D96 */
  0x0020,   /* U+1D97 */
  0x0020,   /* U+1D98 */
  0x0020,   /* U+1D99 */
  0x0020,   /* U+1D9A */
  0x0020,   /* U+1D9B */
  0x0020,   /* U+1D9C */
  0x0020,   /* U+1D9D */
  0x0020,   /* U+1D9E */
  0x0020,   /* U+1D9F */
  0x0020,   /* U+1DA0 */
  0x0020,   /* U+1DA1 */
  0x0020,   /* U+1DA2 */
  0x0020,   /* U+1DA3 */
  0x0020,   /* U+1DA4 */
  0x0020,   /* U+1DA5 */
  0x0020,   /* U+1DA6 */
  0x0020,   /* U+1DA7 */
  0x0020,   /* U+1DA8 */
  0x0020,   /* U+1DA9 */
  0x0020,   /* U+1DAA */
  0x0020,   /* U+1DAB */
  0x0020,   /* U+1DAC */
  0x0020,   /* U+1DAD */
  0x0020,   /* U+1DAE */
  0x0020,   /* U+1DAF */
  0x0020,   /* U+1DB0 */
  0x0020,   /* U+1DB1 */
  0x0020,   /* U+1DB2 */
  0x0020,   /* U+1DB3 */
  0x0020,   /* U+1DB4 */
  0x0020,   /* U+1DB5 */
  0x0020,   /* U+1DB6 */
  0x0020,   /* U+1DB7 */
  0x0020,   /* U+1DB8 */
  0x0020,   /* U+1DB9 */
  0x0020,   /* U+1DBA */
  0x0020,   /* U+1DBB */
  0x0020,   /* U+1DBC */
  0x0020,   /* U+1DBD */
  0x0020,   /* U+1DBE */
  0x0020,   /* U+1DBF */
  0x0033,   /* U+1DC0 */
  0x0033,   /* U+1DC1 */
  0x0034,   /* U+1DC2 */
  0x0033,   /* U+1DC3 */
  0x0033,   /* U+1DC4 */
  0x0033,   /* U+1DC5 */
  0x0033,   /* U+1DC6 */
  0x0033,   /* U+1DC7 */
  0x0033,   /* U+1DC8 */
  0x0033,   /* U+1DC9 */
  0x0020,   /* U+1DCA */
  0x0033,   /* U+1DCB */
  0x0033,   /* U+1DCC */
  0x0033,   /* U+1DCD */
  0x0033,   /* U+1DCE */
  0x0034,   /* U+1DCF */
  0x0034,   /* U+1DD0 */
  0x0033,   /* U+1DD1 */
  0x0020,   /* U+1DD2 */
  0x0020,   /* U+1DD3 */
  0x0020,   /* U+1DD4 */
  0x0020,   /* U+1DD5 */
  0x0020,   /* U+1DD6 */
  0x0020,   /* U+1DD7 */
  0x0020,   /* U+1DD8 */
  0x0020,   /* U+1DD9 */
  0x0020,   /* U+1DDA */
  0x0020,   /* U+1DDB */
  0x0020,   /* U+1DDC */
  0x0020,   /* U+1DDD */
  0x0020,   /* U+1DDE */
  0x0020,   /* U+1DDF */
  0x0020,   /* U+1DE0 */
  0x0020,   /* U+1DE1 */
  0x0020,   /* U+1DE2 */
  0x0020,   /* U+1DE3 */
  0x0020,   /* U+1DE4 */
  0x0020,   /* U+1DE5 */
  0x0020,   /* U+1DE6 */
  0x0020,   /* U+1DE7 */
  0x0020,   /* U+1DE8 */
  0x0020,   /* U+1DE9 */
  0x0020,   /* U+1DEA */
  0x0020,   /* U+1DEB */
  0x0020,   /* U+1DEC */
  0x0020,   /* U+1DED */
  0x0020,   /* U+1DEE */
  0x0020,   /* U+1DEF */
  0x0020,   /* U+1DF0 */
  0x0020,   /* U+1DF1 */
  0x0020,   /* U+1DF2 */
  0x0020,   /* U+1DF3 */
  0x0020,   /* U+1DF4 */
  0x0033,   /* U+1DF5 */
  0x0020,   /* U+1DF6 */
  0x0020,   /* U+1DF7 */
  0x0020,   /* U+1DF8 */
  0x0020,   /* U+1DF9 */
  0x0020,   /* U+1DFA */
  0x0033,   /* U+1DFB */
  0x0034,   /* U+1DFC */
  0x0034,   /* U+1DFD */
  0x0033,   /* U+1DFE */
  0x0034,   /* U+1DFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D00 */
  0x0002,   /* U+1D01 */
  0x0002,   /* U+1D02 */
  0x0002,   /* U+1D03 */
  0x0002,   /* U+1D04 */
  0x0002,   /* U+1D05 */
  0x0002,   /* U+1D06 */
  0x0002,   /* U+1D07 */
  0x0002,   /* U+1D08 */
  0x0002,   /* U+1D09 */
  0x0002,   /* U+1D0A */
  0x0002,   /* U+1D0B */
  0x0002,   /* U+1D0C */
  0x0002,   /* U+1D0D */
  0x0002,   /* U+1D0E */
  0x0002,   /* U+1D0F */
  0x0002,   /* U+1D10 */
  0x0002,   /* U+1D11 */
  0x0002,   /* U+1D12 */
  0x0002,   /* U+1D13 */
  0x0002,   /* U+1D14 */
  0x0002,   /* U+1D15 */
  0x0002,   /* U+1D16 */
  0x0002,   /* U+1D17 */
  0x0002,   /* U+1D18 */
  0x0002,   /* U+1D19 */
  0x0002,   /* U+1D1A */
  0x0002,   /* U+1D1B */
  0x0002,   /* U+1D1C */
  0x0002,   /* U+1D1D */
  0x0002,   /* U+1D1E */
  0x0002,   /* U+1D1F */
  0x0002,   /* U+1D20 */
  0x0002,   /* U+1D21 */
  0x0002,   /* U+1D22 */
  0x0002,   /* U+1D23 */
  0x0002,   /* U+1D24 */
  0x0002,   /* U+1D25 */
  0x0002,   /* U+1D26 */
  0x0002,   /* U+1D27 */
  0x0002,   /* U+1D28 */
  0x0002,   /* U+1D29 */
  0x0002,   /* U+1D2A */
  0x0002,   /* U+1D2B */
  0x001D,   /* U+1D2C */
  0x0014,   /* U+1D2D */
  0x001D,   /* U+1D2E */
  0x0002,   /* U+1D2F */
  0x001D,   /* U+1D30 */
  0x001D,   /* U+1D31 */
  0x001D,   /* U+1D32 */
  0x001D,   /* U+1D33 */
  0x001D,   /* U+1D34 */
  0x001D,   /* U+1D35 */
  0x001D,   /* U+1D36 */
  0x001D,   /* U+1D37 */
  0x001D,   /* U+1D38 */
  0x001D,   /* U+1D39 */
  0x001D,   /* U+1D3A */
  0x0002,   /* U+1D3B */
  0x001D,   /* U+1D3C */
  0x001D,   /* U+1D3D */
  0x001D,   /* U+1D3E */
  0x001D,   /* U+1D3F */
  0x001D,   /* U+1D40 */
  0x001D,   /* U+1D41 */
  0x001D,   /* U+1D42 */
  0x0014,   /* U+1D43 */
  0x0014,   /* U+1D44 */
  0x0014,   /* U+1D45 */
  0x0014,   /* U+1D46 */
  0x0014,   /* U+1D47 */
  0x0014,   /* U+1D48 */
  0x0014,   /* U+1D49 */
  0x0014,   /* U+1D4A */
  0x0014,   /* U+1D4B */
  0x0014,   /* U+1D4C */
  0x0014,   /* U+1D4D */
  0x0014,   /* U+1D4E */
  0x0014,   /* U+1D4F */
  0x0014,   /* U+1D50 */
  0x0014,   /* U+1D51 */
  0x0014,   /* U+1D52 */
  0x0014,   /* U+1D53 */
  0x0014,   /* U+1D54 */
  0x0014,   /* U+1D55 */
  0x0014,   /* U+1D56 */
  0x0014,   /* U+1D57 */
  0x0014,   /* U+1D58 */
  0x0014,   /* U+1D59 */
  0x0014,   /* U+1D5A */
  0x0014,   /* U+1D5B */
  0x0014,   /* U+1D5C */
  0x0014,   /* U+1D5D */
  0x0014,   /* U+1D5E */
  0x0014,   /* U+1D5F */
  0x0014,   /* U+1D60 */
  0x0014,   /* U+1D61 */
  0x0015,   /* U+1D62 */
  0x0015,   /* U+1D63 */
  0x0015,   /* U+1D64 */
  0x0015,   /* U+1D65 */
  0x0015,   /* U+1D66 */
  0x0015,   /* U+1D67 */
  0x0015,   /* U+1D68 */
  0x0015,   /* U+1D69 */
  0x0015,   /* U+1D6A */
  0x0002,   /* U+1D6B */
  0x0002,   /* U+1D6C */
  0x0002,   /* U+1D6D */
  0x0002,   /* U+1D6E */
  0x0002,   /* U+1D6F */
  0x0002,   /* U+1D70 */
  0x0002,   /* U+1D71 */
  0x0002,   /* U+1D72 */
  0x0002,   /* U+1D73 */
  0x0002,   /* U+1D74 */
  0x0002,   /* U+1D75 */
  0x0002,   /* U+1D76 */
  0x0002,   /* U+1D77 */
  0x0014,   /* U+1D78 */
  0x0004,   /* U+1D79 */
  0x0004,   /* U+1D7A */
  0x0002,   /* U+1D7B */
  0x0002,   /* U+1D7C */
  0x0002,   /* U+1D7D */
  0x0002,   /* U+1D7E */
  0x0002,   /* U+1D7F */
  0x0002,   /* U+1D80 */
  0x0002,   /* U+1D81 */
  0x0002,   /* U+1D82 */
  0x0002,   /* U+1D83 */
  0x0002,   /* U+1D84 */
  0x0002,   /* U+1D85 */
  0x0002,   /* U+1D86 */
  0x0002,   /* U+1D87 */
  0x0002,   /* U+1D88 */
  0x0002,   /* U+1D89 */
  0x0002,   /* U+1D8A */
  0x0002,   /* U+1D8B */
  0x0002,   /* U+1D8C */
  0x0002,   /* U+1D8D */
  0x0002,   /* U+1D8E */
  0x0002,   /* U+1D8F */
  0x0002,   /* U+1D90 */
  0x0002,   /* U+1D91 */
  0x0002,   /* U+1D92 */
  0x0002,   /* U+1D93 */
  0x0002,   /* U+1D94 */
  0x0002,   /* U+1D95 */
  0x0002,   /* U+1D96 */
  0x0002,   /* U+1D97 */
  0x0002,   /* U+1D98 */
  0x0002,   /* U+1D99 */
  0x0002,   /* U+1D9A */
  0x0014,   /* U+1D9B */
  0x0014,   /* U+1D9C */
  0x0014,   /* U+1D9D */
  0x0014,   /* U+1D9E */
  0x0014,   /* U+1D9F */
  0x0014,   /* U+1DA0 */
  0x0014,   /* U+1DA1 */
  0x0014,   /* U+1DA2 */
  0x0014,   /* U+1DA3 */
  0x0014,   /* U+1DA4 */
  0x0014,   /* U+1DA5 */
  0x0014,   /* U+1DA6 */
  0x0014,   /* U+1DA7 */
  0x0014,   /* U+1DA8 */
  0x0014,   /* U+1DA9 */
  0x0014,   /* U+1DAA */
  0x0014,   /* U+1DAB */
  0x0014,   /* U+1DAC */
  0x0014,   /* U+1DAD */
  0x0014,   /* U+1DAE */
  0x0014,   /* U+1DAF */
  0x0014,   /* U+1DB0 */
  0x0014,   /* U+1DB1 */
  0x0014,   /* U+1DB2 */
  0x0014,   /* U+1DB3 */
  0x0014,   /* U+1DB4 */
  0x0014,   /* U+1DB5 */
  0x0014,   /* U+1DB6 */
  0x0014,   /* U+1DB7 */
  0x0014,   /* U+1DB8 */
  0x0014,   /* U+1DB9 */
  0x0014,   /* U+1DBA */
  0x0014,   /* U+1DBB */
  0x0014,   /* U+1DBC */
  0x0014,   /* U+1DBD */
  0x0014,   /* U+1DBE */
  0x0014,   /* U+1DBF */
  0x0002,   /* U+1DC0 */
  0x0002,   /* U+1DC1 */
  0x0002,   /* U+1DC2 */
  0x0002,   /* U+1DC3 */
  0x0002,   /* U+1DC4 */
  0x0002,   /* U+1DC5 */
  0x0002,   /* U+1DC6 */
  0x0002,   /* U+1DC7 */
  0x0002,   /* U+1DC8 */
  0x0002,   /* U+1DC9 */
  0x0004,   /* U+1DCA */
  0x0002,   /* U+1DCB */
  0x0002,   /* U+1DCC */
  0x0002,   /* U+1DCD */
  0x0002,   /* U+1DCE */
  0x0002,   /* U+1DCF */
  0x0002,   /* U+1DD0 */
  0x0002,   /* U+1DD1 */
  0x0004,   /* U+1DD2 */
  0x0004,   /* U+1DD3 */
  0x0004,   /* U+1DD4 */
  0x0004,   /* U+1DD5 */
  0x0004,   /* U+1DD6 */
  0x0004,   /* U+1DD7 */
  0x0004,   /* U+1DD8 */
  0x0004,   /* U+1DD9 */
  0x0004,   /* U+1DDA */
  0x0004,   /* U+1DDB */
  0x0004,   /* U+1DDC */
  0x0004,   /* U+1DDD */
  0x0004,   /* U+1DDE */
  0x0004,   /* U+1DDF */
  0x0004,   /* U+1DE0 */
  0x0004,   /* U+1DE1 */
  0x0004,   /* U+1DE2 */
  0x0004,   /* U+1DE3 */
  0x0004,   /* U+1DE4 */
  0x0004,   /* U+1DE5 */
  0x0004,   /* U+1DE6 */
  0x0004,   /* U+1DE7 */
  0x0004,   /* U+1DE8 */
  0x0004,   /* U+1DE9 */
  0x0004,   /* U+1DEA */
  0x0004,   /* U+1DEB */
  0x0004,   /* U+1DEC */
  0x0004,   /* U+1DED */
  0x0004,   /* U+1DEE */
  0x0004,   /* U+1DEF */
  0x0004,   /* U+1DF0 */
  0x0004,   /* U+1DF1 */
  0x0004,   /* U+1DF2 */
  0x0004,   /* U+1DF3 */
  0x0004,   /* U+1DF4 */
  0x0002,   /* U+1DF5 */
  0x0002,   /* U+1DF6 */
  0x0002,   /* U+1DF7 */
  0x0002,   /* U+1DF8 */
  0x0002,   /* U+1DF9 */
  0x0002,   /* U+1DFA */
  0x0002,   /* U+1DFB */
  0x0002,   /* U+1DFC */
  0x0002,   /* U+1DFD */
  0x0002,   /* U+1DFE */
  0x0002,   /* U+1DFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x0000,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0000,   /* U+1D79 */
  0x0000,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0000,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0000,   /* U+1DD3 */
  0x0000,   /* U+1DD4 */
  0x1DDD,   /* U+1DD5 */
  0x1EE3,   /* U+1DD6 */
  0x0000,   /* U+1DD7 */
  0x0000,   /* U+1DD8 */
  0x0000,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0000,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0000,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0000,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x0000,   /* U+1DF2 */
  0x0000,   /* U+1DF3 */
  0x0000,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x9DF6,   /* U+1DF6 */
  0x9DF7,   /* U+1DF7 */
  0x9DF8,   /* U+1DF8 */
  0x9DF9,   /* U+1DF9 */
  0x9DFA,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x0110,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0111,   /* U+1D79 */
  0x0110,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0110,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0110,   /* U+1DD3 */
  0x0110,   /* U+1DD4 */
  0x0020,   /* U+1DD5 */
  0x0020,   /* U+1DD6 */
  0x0030,   /* U+1DD7 */
  0x0111,   /* U+1DD8 */
  0x0110,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0111,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0034,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0034,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x002B,   /* U+1DF2 */
  0x002B,   /* U+1DF3 */
  0x002B,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x0000,   /* U+1DF6 */
  0x0000,   /* U+1DF7 */
  0x0000,   /* U+1DF8 */
  0x0000,   /* U+1DF9 */
  0x0000,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x0014,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0004,   /* U+1D79 */
  0x0004,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0014,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0004,   /* U+1DD3 */
  0x0004,   /* U+1DD4 */
  0x0004,   /* U+1DD5 */
  0x0004,   /* U+1DD6 */
  0x0004,   /* U+1DD7 */
  0x0004,   /* U+1DD8 */
  0x0004,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0004,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0004,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0004,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x0004,   /* U+1DF2 */
  0x0004,   /* U+1DF3 */
  0x0004,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x0000,   /* U+1DF6 */
  0x0000,   /* U+1DF7 */
  0x0000,   /* U+1DF8 */
  0x0000,   /* U+1DF9 */
  0x0000,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x1CAA,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0000,   /* U+1D79 */
  0x1D18,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0000,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0000,   /* U+1DD3 */
  0x1CAA,   /* U+1DD4 */
  0x0000,   /* U+1DD5 */
  0x0000,   /* U+1DD6 */
  0x0000,   /* U+1DD7 */
  0x0000,   /* U+1DD8 */
  0x0000,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0000,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0000,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0000,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x0000,   /* U+1DF2 */
  0x0000,   /* U+1DF3 */
  0x0000,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x0000,   /* U+1DF6 */
  0x0000,   /* U+1DF7 */
  0x0000,   /* U+1DF8 */
  0x0000,   /* U+1DF9 */
  0x0000,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x0020,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0000,   /* U+1D79 */
  0x0020,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0000,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0000,   /* U+1DD3 */
  0x0020,   /* U+1DD4 */
  0x0000,   /* U+1DD5 */
  0x0000,   /* U+1DD6 */
  0x0000,   /* U+1DD7 */
  0x0000,   /* U+1DD8 */
  0x0000,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0000,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0000,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0000,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x0000,   /* U+1DF2 */
  0x0000,   /* U+1DF3 */
  0x0000,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x0000,   /* U+1DF6 */
  0x0000,   /* U+1DF7 */
  0x0000,   /* U+1DF8 */
  0x0000,   /* U+1DF9 */
  0x0000,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1D00 */
  0x0000,   /* U+1D01 */
  0x0000,   /* U+1D02 */
  0x0000,   /* U+1D03 */
  0x0000,   /* U+1D04 */
  0x0000,   /* U+1D05 */
  0x0000,   /* U+1D06 */
  0x0000,   /* U+1D07 */
  0x0000,   /* U+1D08 */
  0x0000,   /* U+1D09 */
  0x0000,   /* U+1D0A */
  0x0000,   /* U+1D0B */
  0x0000,   /* U+1D0C */
  0x0000,   /* U+1D0D */
  0x0000,   /* U+1D0E */
  0x0000,   /* U+1D0F */
  0x0000,   /* U+1D10 */
  0x0000,   /* U+1D11 */
  0x0000,   /* U+1D12 */
  0x0000,   /* U+1D13 */
  0x0000,   /* U+1D14 */
  0x0000,   /* U+1D15 */
  0x0000,   /* U+1D16 */
  0x0000,   /* U+1D17 */
  0x0000,   /* U+1D18 */
  0x0000,   /* U+1D19 */
  0x0000,   /* U+1D1A */
  0x0000,   /* U+1D1B */
  0x0000,   /* U+1D1C */
  0x0000,   /* U+1D1D */
  0x0000,   /* U+1D1E */
  0x0000,   /* U+1D1F */
  0x0000,   /* U+1D20 */
  0x0000,   /* U+1D21 */
  0x0000,   /* U+1D22 */
  0x0000,   /* U+1D23 */
  0x0000,   /* U+1D24 */
  0x0000,   /* U+1D25 */
  0x0000,   /* U+1D26 */
  0x0000,   /* U+1D27 */
  0x0000,   /* U+1D28 */
  0x0000,   /* U+1D29 */
  0x0000,   /* U+1D2A */
  0x0000,   /* U+1D2B */
  0x0000,   /* U+1D2C */
  0x0014,   /* U+1D2D */
  0x0000,   /* U+1D2E */
  0x0000,   /* U+1D2F */
  0x0000,   /* U+1D30 */
  0x0000,   /* U+1D31 */
  0x0000,   /* U+1D32 */
  0x0000,   /* U+1D33 */
  0x0000,   /* U+1D34 */
  0x0000,   /* U+1D35 */
  0x0000,   /* U+1D36 */
  0x0000,   /* U+1D37 */
  0x0000,   /* U+1D38 */
  0x0000,   /* U+1D39 */
  0x0000,   /* U+1D3A */
  0x0000,   /* U+1D3B */
  0x0000,   /* U+1D3C */
  0x0000,   /* U+1D3D */
  0x0000,   /* U+1D3E */
  0x0000,   /* U+1D3F */
  0x0000,   /* U+1D40 */
  0x0000,   /* U+1D41 */
  0x0000,   /* U+1D42 */
  0x0000,   /* U+1D43 */
  0x0000,   /* U+1D44 */
  0x0000,   /* U+1D45 */
  0x0000,   /* U+1D46 */
  0x0000,   /* U+1D47 */
  0x0000,   /* U+1D48 */
  0x0000,   /* U+1D49 */
  0x0000,   /* U+1D4A */
  0x0000,   /* U+1D4B */
  0x0000,   /* U+1D4C */
  0x0000,   /* U+1D4D */
  0x0000,   /* U+1D4E */
  0x0000,   /* U+1D4F */
  0x0000,   /* U+1D50 */
  0x0000,   /* U+1D51 */
  0x0000,   /* U+1D52 */
  0x0000,   /* U+1D53 */
  0x0000,   /* U+1D54 */
  0x0000,   /* U+1D55 */
  0x0000,   /* U+1D56 */
  0x0000,   /* U+1D57 */
  0x0000,   /* U+1D58 */
  0x0000,   /* U+1D59 */
  0x0000,   /* U+1D5A */
  0x0000,   /* U+1D5B */
  0x0000,   /* U+1D5C */
  0x0000,   /* U+1D5D */
  0x0000,   /* U+1D5E */
  0x0000,   /* U+1D5F */
  0x0000,   /* U+1D60 */
  0x0000,   /* U+1D61 */
  0x0000,   /* U+1D62 */
  0x0000,   /* U+1D63 */
  0x0000,   /* U+1D64 */
  0x0000,   /* U+1D65 */
  0x0000,   /* U+1D66 */
  0x0000,   /* U+1D67 */
  0x0000,   /* U+1D68 */
  0x0000,   /* U+1D69 */
  0x0000,   /* U+1D6A */
  0x0000,   /* U+1D6B */
  0x0000,   /* U+1D6C */
  0x0000,   /* U+1D6D */
  0x0000,   /* U+1D6E */
  0x0000,   /* U+1D6F */
  0x0000,   /* U+1D70 */
  0x0000,   /* U+1D71 */
  0x0000,   /* U+1D72 */
  0x0000,   /* U+1D73 */
  0x0000,   /* U+1D74 */
  0x0000,   /* U+1D75 */
  0x0000,   /* U+1D76 */
  0x0000,   /* U+1D77 */
  0x0000,   /* U+1D78 */
  0x0000,   /* U+1D79 */
  0x0004,   /* U+1D7A */
  0x0000,   /* U+1D7B */
  0x0000,   /* U+1D7C */
  0x0000,   /* U+1D7D */
  0x0000,   /* U+1D7E */
  0x0000,   /* U+1D7F */
  0x0000,   /* U+1D80 */
  0x0000,   /* U+1D81 */
  0x0000,   /* U+1D82 */
  0x0000,   /* U+1D83 */
  0x0000,   /* U+1D84 */
  0x0000,   /* U+1D85 */
  0x0000,   /* U+1D86 */
  0x0000,   /* U+1D87 */
  0x0000,   /* U+1D88 */
  0x0000,   /* U+1D89 */
  0x0000,   /* U+1D8A */
  0x0000,   /* U+1D8B */
  0x0000,   /* U+1D8C */
  0x0000,   /* U+1D8D */
  0x0000,   /* U+1D8E */
  0x0000,   /* U+1D8F */
  0x0000,   /* U+1D90 */
  0x0000,   /* U+1D91 */
  0x0000,   /* U+1D92 */
  0x0000,   /* U+1D93 */
  0x0000,   /* U+1D94 */
  0x0000,   /* U+1D95 */
  0x0000,   /* U+1D96 */
  0x0000,   /* U+1D97 */
  0x0000,   /* U+1D98 */
  0x0000,   /* U+1D99 */
  0x0000,   /* U+1D9A */
  0x0000,   /* U+1D9B */
  0x0000,   /* U+1D9C */
  0x0000,   /* U+1D9D */
  0x0000,   /* U+1D9E */
  0x0000,   /* U+1D9F */
  0x0000,   /* U+1DA0 */
  0x0000,   /* U+1DA1 */
  0x0000,   /* U+1DA2 */
  0x0000,   /* U+1DA3 */
  0x0000,   /* U+1DA4 */
  0x0000,   /* U+1DA5 */
  0x0000,   /* U+1DA6 */
  0x0000,   /* U+1DA7 */
  0x0000,   /* U+1DA8 */
  0x0000,   /* U+1DA9 */
  0x0000,   /* U+1DAA */
  0x0000,   /* U+1DAB */
  0x0000,   /* U+1DAC */
  0x0000,   /* U+1DAD */
  0x0000,   /* U+1DAE */
  0x0000,   /* U+1DAF */
  0x0000,   /* U+1DB0 */
  0x0000,   /* U+1DB1 */
  0x0000,   /* U+1DB2 */
  0x0000,   /* U+1DB3 */
  0x0000,   /* U+1DB4 */
  0x0000,   /* U+1DB5 */
  0x0000,   /* U+1DB6 */
  0x0000,   /* U+1DB7 */
  0x0000,   /* U+1DB8 */
  0x0000,   /* U+1DB9 */
  0x0000,   /* U+1DBA */
  0x0000,   /* U+1DBB */
  0x0000,   /* U+1DBC */
  0x0000,   /* U+1DBD */
  0x0000,   /* U+1DBE */
  0x0000,   /* U+1DBF */
  0x0000,   /* U+1DC0 */
  0x0000,   /* U+1DC1 */
  0x0000,   /* U+1DC2 */
  0x0000,   /* U+1DC3 */
  0x0000,   /* U+1DC4 */
  0x0000,   /* U+1DC5 */
  0x0000,   /* U+1DC6 */
  0x0000,   /* U+1DC7 */
  0x0000,   /* U+1DC8 */
  0x0000,   /* U+1DC9 */
  0x0000,   /* U+1DCA */
  0x0000,   /* U+1DCB */
  0x0000,   /* U+1DCC */
  0x0000,   /* U+1DCD */
  0x0000,   /* U+1DCE */
  0x0000,   /* U+1DCF */
  0x0000,   /* U+1DD0 */
  0x0000,   /* U+1DD1 */
  0x0000,   /* U+1DD2 */
  0x0000,   /* U+1DD3 */
  0x0004,   /* U+1DD4 */
  0x0000,   /* U+1DD5 */
  0x0000,   /* U+1DD6 */
  0x0000,   /* U+1DD7 */
  0x0000,   /* U+1DD8 */
  0x0000,   /* U+1DD9 */
  0x0000,   /* U+1DDA */
  0x0000,   /* U+1DDB */
  0x0000,   /* U+1DDC */
  0x0000,   /* U+1DDD */
  0x0000,   /* U+1DDE */
  0x0000,   /* U+1DDF */
  0x0000,   /* U+1DE0 */
  0x0000,   /* U+1DE1 */
  0x0000,   /* U+1DE2 */
  0x0000,   /* U+1DE3 */
  0x0000,   /* U+1DE4 */
  0x0000,   /* U+1DE5 */
  0x0000,   /* U+1DE6 */
  0x0000,   /* U+1DE7 */
  0x0000,   /* U+1DE8 */
  0x0000,   /* U+1DE9 */
  0x0000,   /* U+1DEA */
  0x0000,   /* U+1DEB */
  0x0000,   /* U+1DEC */
  0x0000,   /* U+1DED */
  0x0000,   /* U+1DEE */
  0x0000,   /* U+1DEF */
  0x0000,   /* U+1DF0 */
  0x0000,   /* U+1DF1 */
  0x0000,   /* U+1DF2 */
  0x0000,   /* U+1DF3 */
  0x0000,   /* U+1DF4 */
  0x0000,   /* U+1DF5 */
  0x0000,   /* U+1DF6 */
  0x0000,   /* U+1DF7 */
  0x0000,   /* U+1DF8 */
  0x0000,   /* U+1DF9 */
  0x0000,   /* U+1DFA */
  0x0000,   /* U+1DFB */
  0x0000,   /* U+1DFC */
  0x0000,   /* U+1DFD */
  0x0000,   /* U+1DFE */
  0x0000,   /* U+1DFF */
};

uint16 uca900_p01E[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 2, 2, 3, 3, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 
  2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 
  3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 2, 2, 
  2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 1, 1, 3, 1, 
  2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C47,   /* U+1E00 */
  0x1C47,   /* U+1E01 */
  0x1C60,   /* U+1E02 */
  0x1C60,   /* U+1E03 */
  0x1C60,   /* U+1E04 */
  0x1C60,   /* U+1E05 */
  0x1C60,   /* U+1E06 */
  0x1C60,   /* U+1E07 */
  0x1C7A,   /* U+1E08 */
  0x1C7A,   /* U+1E09 */
  0x1C8F,   /* U+1E0A */
  0x1C8F,   /* U+1E0B */
  0x1C8F,   /* U+1E0C */
  0x1C8F,   /* U+1E0D */
  0x1C8F,   /* U+1E0E */
  0x1C8F,   /* U+1E0F */
  0x1C8F,   /* U+1E10 */
  0x1C8F,   /* U+1E11 */
  0x1C8F,   /* U+1E12 */
  0x1C8F,   /* U+1E13 */
  0x1CAA,   /* U+1E14 */
  0x1CAA,   /* U+1E15 */
  0x1CAA,   /* U+1E16 */
  0x1CAA,   /* U+1E17 */
  0x1CAA,   /* U+1E18 */
  0x1CAA,   /* U+1E19 */
  0x1CAA,   /* U+1E1A */
  0x1CAA,   /* U+1E1B */
  0x1CAA,   /* U+1E1C */
  0x1CAA,   /* U+1E1D */
  0x1CE5,   /* U+1E1E */
  0x1CE5,   /* U+1E1F */
  0x1CF4,   /* U+1E20 */
  0x1CF4,   /* U+1E21 */
  0x1D18,   /* U+1E22 */
  0x1D18,   /* U+1E23 */
  0x1D18,   /* U+1E24 */
  0x1D18,   /* U+1E25 */
  0x1D18,   /* U+1E26 */
  0x1D18,   /* U+1E27 */
  0x1D18,   /* U+1E28 */
  0x1D18,   /* U+1E29 */
  0x1D18,   /* U+1E2A */
  0x1D18,   /* U+1E2B */
  0x1D32,   /* U+1E2C */
  0x1D32,   /* U+1E2D */
  0x1D32,   /* U+1E2E */
  0x1D32,   /* U+1E2F */
  0x1D65,   /* U+1E30 */
  0x1D65,   /* U+1E31 */
  0x1D65,   /* U+1E32 */
  0x1D65,   /* U+1E33 */
  0x1D65,   /* U+1E34 */
  0x1D65,   /* U+1E35 */
  0x1D77,   /* U+1E36 */
  0x1D77,   /* U+1E37 */
  0x1D77,   /* U+1E38 */
  0x1D77,   /* U+1E39 */
  0x1D77,   /* U+1E3A */
  0x1D77,   /* U+1E3B */
  0x1D77,   /* U+1E3C */
  0x1D77,   /* U+1E3D */
  0x1DAA,   /* U+1E3E */
  0x1DAA,   /* U+1E3F */
  0x1DAA,   /* U+1E40 */
  0x1DAA,   /* U+1E41 */
  0x1DAA,   /* U+1E42 */
  0x1DAA,   /* U+1E43 */
  0x1DB9,   /* U+1E44 */
  0x1DB9,   /* U+1E45 */
  0x1DB9,   /* U+1E46 */
  0x1DB9,   /* U+1E47 */
  0x1DB9,   /* U+1E48 */
  0x1DB9,   /* U+1E49 */
  0x1DB9,   /* U+1E4A */
  0x1DB9,   /* U+1E4B */
  0x1DDD,   /* U+1E4C */
  0x1DDD,   /* U+1E4D */
  0x1DDD,   /* U+1E4E */
  0x1DDD,   /* U+1E4F */
  0x1DDD,   /* U+1E50 */
  0x1DDD,   /* U+1E51 */
  0x1DDD,   /* U+1E52 */
  0x1DDD,   /* U+1E53 */
  0x1E0C,   /* U+1E54 */
  0x1E0C,   /* U+1E55 */
  0x1E0C,   /* U+1E56 */
  0x1E0C,   /* U+1E57 */
  0x1E33,   /* U+1E58 */
  0x1E33,   /* U+1E59 */
  0x1E33,   /* U+1E5A */
  0x1E33,   /* U+1E5B */
  0x1E33,   /* U+1E5C */
  0x1E33,   /* U+1E5D */
  0x1E33,   /* U+1E5E */
  0x1E33,   /* U+1E5F */
  0x1E71,   /* U+1E60 */
  0x1E71,   /* U+1E61 */
  0x1E71,   /* U+1E62 */
  0x1E71,   /* U+1E63 */
  0x1E71,   /* U+1E64 */
  0x1E71,   /* U+1E65 */
  0x1E71,   /* U+1E66 */
  0x1E71,   /* U+1E67 */
  0x1E71,   /* U+1E68 */
  0x1E71,   /* U+1E69 */
  0x1E95,   /* U+1E6A */
  0x1E95,   /* U+1E6B */
  0x1E95,   /* U+1E6C */
  0x1E95,   /* U+1E6D */
  0x1E95,   /* U+1E6E */
  0x1E95,   /* U+1E6F */
  0x1E95,   /* U+1E70 */
  0x1E95,   /* U+1E71 */
  0x1EB5,   /* U+1E72 */
  0x1EB5,   /* U+1E73 */
  0x1EB5,   /* U+1E74 */
  0x1EB5,   /* U+1E75 */
  0x1EB5,   /* U+1E76 */
  0x1EB5,   /* U+1E77 */
  0x1EB5,   /* U+1E78 */
  0x1EB5,   /* U+1E79 */
  0x1EB5,   /* U+1E7A */
  0x1EB5,   /* U+1E7B */
  0x1EE3,   /* U+1E7C */
  0x1EE3,   /* U+1E7D */
  0x1EE3,   /* U+1E7E */
  0x1EE3,   /* U+1E7F */
  0x1EF5,   /* U+1E80 */
  0x1EF5,   /* U+1E81 */
  0x1EF5,   /* U+1E82 */
  0x1EF5,   /* U+1E83 */
  0x1EF5,   /* U+1E84 */
  0x1EF5,   /* U+1E85 */
  0x1EF5,   /* U+1E86 */
  0x1EF5,   /* U+1E87 */
  0x1EF5,   /* U+1E88 */
  0x1EF5,   /* U+1E89 */
  0x1EFF,   /* U+1E8A */
  0x1EFF,   /* U+1E8B */
  0x1EFF,   /* U+1E8C */
  0x1EFF,   /* U+1E8D */
  0x1F0B,   /* U+1E8E */
  0x1F0B,   /* U+1E8F */
  0x1F21,   /* U+1E90 */
  0x1F21,   /* U+1E91 */
  0x1F21,   /* U+1E92 */
  0x1F21,   /* U+1E93 */
  0x1F21,   /* U+1E94 */
  0x1F21,   /* U+1E95 */
  0x1D18,   /* U+1E96 */
  0x1E95,   /* U+1E97 */
  0x1EF5,   /* U+1E98 */
  0x1F0B,   /* U+1E99 */
  0x1C47,   /* U+1E9A */
  0x1E71,   /* U+1E9B */
  0x1E80,   /* U+1E9C */
  0x1E81,   /* U+1E9D */
  0x1E71,   /* U+1E9E */
  0x1CA9,   /* U+1E9F */
  0x1C47,   /* U+1EA0 */
  0x1C47,   /* U+1EA1 */
  0x1C47,   /* U+1EA2 */
  0x1C47,   /* U+1EA3 */
  0x1C47,   /* U+1EA4 */
  0x1C47,   /* U+1EA5 */
  0x1C47,   /* U+1EA6 */
  0x1C47,   /* U+1EA7 */
  0x1C47,   /* U+1EA8 */
  0x1C47,   /* U+1EA9 */
  0x1C47,   /* U+1EAA */
  0x1C47,   /* U+1EAB */
  0x1C47,   /* U+1EAC */
  0x1C47,   /* U+1EAD */
  0x1C47,   /* U+1EAE */
  0x1C47,   /* U+1EAF */
  0x1C47,   /* U+1EB0 */
  0x1C47,   /* U+1EB1 */
  0x1C47,   /* U+1EB2 */
  0x1C47,   /* U+1EB3 */
  0x1C47,   /* U+1EB4 */
  0x1C47,   /* U+1EB5 */
  0x1C47,   /* U+1EB6 */
  0x1C47,   /* U+1EB7 */
  0x1CAA,   /* U+1EB8 */
  0x1CAA,   /* U+1EB9 */
  0x1CAA,   /* U+1EBA */
  0x1CAA,   /* U+1EBB */
  0x1CAA,   /* U+1EBC */
  0x1CAA,   /* U+1EBD */
  0x1CAA,   /* U+1EBE */
  0x1CAA,   /* U+1EBF */
  0x1CAA,   /* U+1EC0 */
  0x1CAA,   /* U+1EC1 */
  0x1CAA,   /* U+1EC2 */
  0x1CAA,   /* U+1EC3 */
  0x1CAA,   /* U+1EC4 */
  0x1CAA,   /* U+1EC5 */
  0x1CAA,   /* U+1EC6 */
  0x1CAA,   /* U+1EC7 */
  0x1D32,   /* U+1EC8 */
  0x1D32,   /* U+1EC9 */
  0x1D32,   /* U+1ECA */
  0x1D32,   /* U+1ECB */
  0x1DDD,   /* U+1ECC */
  0x1DDD,   /* U+1ECD */
  0x1DDD,   /* U+1ECE */
  0x1DDD,   /* U+1ECF */
  0x1DDD,   /* U+1ED0 */
  0x1DDD,   /* U+1ED1 */
  0x1DDD,   /* U+1ED2 */
  0x1DDD,   /* U+1ED3 */
  0x1DDD,   /* U+1ED4 */
  0x1DDD,   /* U+1ED5 */
  0x1DDD,   /* U+1ED6 */
  0x1DDD,   /* U+1ED7 */
  0x1DDD,   /* U+1ED8 */
  0x1DDD,   /* U+1ED9 */
  0x1DDD,   /* U+1EDA */
  0x1DDD,   /* U+1EDB */
  0x1DDD,   /* U+1EDC */
  0x1DDD,   /* U+1EDD */
  0x1DDD,   /* U+1EDE */
  0x1DDD,   /* U+1EDF */
  0x1DDD,   /* U+1EE0 */
  0x1DDD,   /* U+1EE1 */
  0x1DDD,   /* U+1EE2 */
  0x1DDD,   /* U+1EE3 */
  0x1EB5,   /* U+1EE4 */
  0x1EB5,   /* U+1EE5 */
  0x1EB5,   /* U+1EE6 */
  0x1EB5,   /* U+1EE7 */
  0x1EB5,   /* U+1EE8 */
  0x1EB5,   /* U+1EE9 */
  0x1EB5,   /* U+1EEA */
  0x1EB5,   /* U+1EEB */
  0x1EB5,   /* U+1EEC */
  0x1EB5,   /* U+1EED */
  0x1EB5,   /* U+1EEE */
  0x1EB5,   /* U+1EEF */
  0x1EB5,   /* U+1EF0 */
  0x1EB5,   /* U+1EF1 */
  0x1F0B,   /* U+1EF2 */
  0x1F0B,   /* U+1EF3 */
  0x1F0B,   /* U+1EF4 */
  0x1F0B,   /* U+1EF5 */
  0x1F0B,   /* U+1EF6 */
  0x1F0B,   /* U+1EF7 */
  0x1F0B,   /* U+1EF8 */
  0x1F0B,   /* U+1EF9 */
  0x1D77,   /* U+1EFA */
  0x1D77,   /* U+1EFB */
  0x1EF0,   /* U+1EFC */
  0x1EF0,   /* U+1EFD */
  0x1F1B,   /* U+1EFE */
  0x1F1B,   /* U+1EFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1E00 */
  0x0020,   /* U+1E01 */
  0x0020,   /* U+1E02 */
  0x0020,   /* U+1E03 */
  0x0020,   /* U+1E04 */
  0x0020,   /* U+1E05 */
  0x0020,   /* U+1E06 */
  0x0020,   /* U+1E07 */
  0x0020,   /* U+1E08 */
  0x0020,   /* U+1E09 */
  0x0020,   /* U+1E0A */
  0x0020,   /* U+1E0B */
  0x0020,   /* U+1E0C */
  0x0020,   /* U+1E0D */
  0x0020,   /* U+1E0E */
  0x0020,   /* U+1E0F */
  0x0020,   /* U+1E10 */
  0x0020,   /* U+1E11 */
  0x0020,   /* U+1E12 */
  0x0020,   /* U+1E13 */
  0x0020,   /* U+1E14 */
  0x0020,   /* U+1E15 */
  0x0020,   /* U+1E16 */
  0x0020,   /* U+1E17 */
  0x0020,   /* U+1E18 */
  0x0020,   /* U+1E19 */
  0x0020,   /* U+1E1A */
  0x0020,   /* U+1E1B */
  0x0020,   /* U+1E1C */
  0x0020,   /* U+1E1D */
  0x0020,   /* U+1E1E */
  0x0020,   /* U+1E1F */
  0x0020,   /* U+1E20 */
  0x0020,   /* U+1E21 */
  0x0020,   /* U+1E22 */
  0x0020,   /* U+1E23 */
  0x0020,   /* U+1E24 */
  0x0020,   /* U+1E25 */
  0x0020,   /* U+1E26 */
  0x0020,   /* U+1E27 */
  0x0020,   /* U+1E28 */
  0x0020,   /* U+1E29 */
  0x0020,   /* U+1E2A */
  0x0020,   /* U+1E2B */
  0x0020,   /* U+1E2C */
  0x0020,   /* U+1E2D */
  0x0020,   /* U+1E2E */
  0x0020,   /* U+1E2F */
  0x0020,   /* U+1E30 */
  0x0020,   /* U+1E31 */
  0x0020,   /* U+1E32 */
  0x0020,   /* U+1E33 */
  0x0020,   /* U+1E34 */
  0x0020,   /* U+1E35 */
  0x0020,   /* U+1E36 */
  0x0020,   /* U+1E37 */
  0x0020,   /* U+1E38 */
  0x0020,   /* U+1E39 */
  0x0020,   /* U+1E3A */
  0x0020,   /* U+1E3B */
  0x0020,   /* U+1E3C */
  0x0020,   /* U+1E3D */
  0x0020,   /* U+1E3E */
  0x0020,   /* U+1E3F */
  0x0020,   /* U+1E40 */
  0x0020,   /* U+1E41 */
  0x0020,   /* U+1E42 */
  0x0020,   /* U+1E43 */
  0x0020,   /* U+1E44 */
  0x0020,   /* U+1E45 */
  0x0020,   /* U+1E46 */
  0x0020,   /* U+1E47 */
  0x0020,   /* U+1E48 */
  0x0020,   /* U+1E49 */
  0x0020,   /* U+1E4A */
  0x0020,   /* U+1E4B */
  0x0020,   /* U+1E4C */
  0x0020,   /* U+1E4D */
  0x0020,   /* U+1E4E */
  0x0020,   /* U+1E4F */
  0x0020,   /* U+1E50 */
  0x0020,   /* U+1E51 */
  0x0020,   /* U+1E52 */
  0x0020,   /* U+1E53 */
  0x0020,   /* U+1E54 */
  0x0020,   /* U+1E55 */
  0x0020,   /* U+1E56 */
  0x0020,   /* U+1E57 */
  0x0020,   /* U+1E58 */
  0x0020,   /* U+1E59 */
  0x0020,   /* U+1E5A */
  0x0020,   /* U+1E5B */
  0x0020,   /* U+1E5C */
  0x0020,   /* U+1E5D */
  0x0020,   /* U+1E5E */
  0x0020,   /* U+1E5F */
  0x0020,   /* U+1E60 */
  0x0020,   /* U+1E61 */
  0x0020,   /* U+1E62 */
  0x0020,   /* U+1E63 */
  0x0020,   /* U+1E64 */
  0x0020,   /* U+1E65 */
  0x0020,   /* U+1E66 */
  0x0020,   /* U+1E67 */
  0x0020,   /* U+1E68 */
  0x0020,   /* U+1E69 */
  0x0020,   /* U+1E6A */
  0x0020,   /* U+1E6B */
  0x0020,   /* U+1E6C */
  0x0020,   /* U+1E6D */
  0x0020,   /* U+1E6E */
  0x0020,   /* U+1E6F */
  0x0020,   /* U+1E70 */
  0x0020,   /* U+1E71 */
  0x0020,   /* U+1E72 */
  0x0020,   /* U+1E73 */
  0x0020,   /* U+1E74 */
  0x0020,   /* U+1E75 */
  0x0020,   /* U+1E76 */
  0x0020,   /* U+1E77 */
  0x0020,   /* U+1E78 */
  0x0020,   /* U+1E79 */
  0x0020,   /* U+1E7A */
  0x0020,   /* U+1E7B */
  0x0020,   /* U+1E7C */
  0x0020,   /* U+1E7D */
  0x0020,   /* U+1E7E */
  0x0020,   /* U+1E7F */
  0x0020,   /* U+1E80 */
  0x0020,   /* U+1E81 */
  0x0020,   /* U+1E82 */
  0x0020,   /* U+1E83 */
  0x0020,   /* U+1E84 */
  0x0020,   /* U+1E85 */
  0x0020,   /* U+1E86 */
  0x0020,   /* U+1E87 */
  0x0020,   /* U+1E88 */
  0x0020,   /* U+1E89 */
  0x0020,   /* U+1E8A */
  0x0020,   /* U+1E8B */
  0x0020,   /* U+1E8C */
  0x0020,   /* U+1E8D */
  0x0020,   /* U+1E8E */
  0x0020,   /* U+1E8F */
  0x0020,   /* U+1E90 */
  0x0020,   /* U+1E91 */
  0x0020,   /* U+1E92 */
  0x0020,   /* U+1E93 */
  0x0020,   /* U+1E94 */
  0x0020,   /* U+1E95 */
  0x0020,   /* U+1E96 */
  0x0020,   /* U+1E97 */
  0x0020,   /* U+1E98 */
  0x0020,   /* U+1E99 */
  0x0020,   /* U+1E9A */
  0x0020,   /* U+1E9B */
  0x0020,   /* U+1E9C */
  0x0020,   /* U+1E9D */
  0x0020,   /* U+1E9E */
  0x0020,   /* U+1E9F */
  0x0020,   /* U+1EA0 */
  0x0020,   /* U+1EA1 */
  0x0020,   /* U+1EA2 */
  0x0020,   /* U+1EA3 */
  0x0020,   /* U+1EA4 */
  0x0020,   /* U+1EA5 */
  0x0020,   /* U+1EA6 */
  0x0020,   /* U+1EA7 */
  0x0020,   /* U+1EA8 */
  0x0020,   /* U+1EA9 */
  0x0020,   /* U+1EAA */
  0x0020,   /* U+1EAB */
  0x0020,   /* U+1EAC */
  0x0020,   /* U+1EAD */
  0x0020,   /* U+1EAE */
  0x0020,   /* U+1EAF */
  0x0020,   /* U+1EB0 */
  0x0020,   /* U+1EB1 */
  0x0020,   /* U+1EB2 */
  0x0020,   /* U+1EB3 */
  0x0020,   /* U+1EB4 */
  0x0020,   /* U+1EB5 */
  0x0020,   /* U+1EB6 */
  0x0020,   /* U+1EB7 */
  0x0020,   /* U+1EB8 */
  0x0020,   /* U+1EB9 */
  0x0020,   /* U+1EBA */
  0x0020,   /* U+1EBB */
  0x0020,   /* U+1EBC */
  0x0020,   /* U+1EBD */
  0x0020,   /* U+1EBE */
  0x0020,   /* U+1EBF */
  0x0020,   /* U+1EC0 */
  0x0020,   /* U+1EC1 */
  0x0020,   /* U+1EC2 */
  0x0020,   /* U+1EC3 */
  0x0020,   /* U+1EC4 */
  0x0020,   /* U+1EC5 */
  0x0020,   /* U+1EC6 */
  0x0020,   /* U+1EC7 */
  0x0020,   /* U+1EC8 */
  0x0020,   /* U+1EC9 */
  0x0020,   /* U+1ECA */
  0x0020,   /* U+1ECB */
  0x0020,   /* U+1ECC */
  0x0020,   /* U+1ECD */
  0x0020,   /* U+1ECE */
  0x0020,   /* U+1ECF */
  0x0020,   /* U+1ED0 */
  0x0020,   /* U+1ED1 */
  0x0020,   /* U+1ED2 */
  0x0020,   /* U+1ED3 */
  0x0020,   /* U+1ED4 */
  0x0020,   /* U+1ED5 */
  0x0020,   /* U+1ED6 */
  0x0020,   /* U+1ED7 */
  0x0020,   /* U+1ED8 */
  0x0020,   /* U+1ED9 */
  0x0020,   /* U+1EDA */
  0x0020,   /* U+1EDB */
  0x0020,   /* U+1EDC */
  0x0020,   /* U+1EDD */
  0x0020,   /* U+1EDE */
  0x0020,   /* U+1EDF */
  0x0020,   /* U+1EE0 */
  0x0020,   /* U+1EE1 */
  0x0020,   /* U+1EE2 */
  0x0020,   /* U+1EE3 */
  0x0020,   /* U+1EE4 */
  0x0020,   /* U+1EE5 */
  0x0020,   /* U+1EE6 */
  0x0020,   /* U+1EE7 */
  0x0020,   /* U+1EE8 */
  0x0020,   /* U+1EE9 */
  0x0020,   /* U+1EEA */
  0x0020,   /* U+1EEB */
  0x0020,   /* U+1EEC */
  0x0020,   /* U+1EED */
  0x0020,   /* U+1EEE */
  0x0020,   /* U+1EEF */
  0x0020,   /* U+1EF0 */
  0x0020,   /* U+1EF1 */
  0x0020,   /* U+1EF2 */
  0x0020,   /* U+1EF3 */
  0x0020,   /* U+1EF4 */
  0x0020,   /* U+1EF5 */
  0x0020,   /* U+1EF6 */
  0x0020,   /* U+1EF7 */
  0x0020,   /* U+1EF8 */
  0x0020,   /* U+1EF9 */
  0x0020,   /* U+1EFA */
  0x0020,   /* U+1EFB */
  0x0020,   /* U+1EFC */
  0x0020,   /* U+1EFD */
  0x0020,   /* U+1EFE */
  0x0020,   /* U+1EFF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+1E00 */
  0x0002,   /* U+1E01 */
  0x0008,   /* U+1E02 */
  0x0002,   /* U+1E03 */
  0x0008,   /* U+1E04 */
  0x0002,   /* U+1E05 */
  0x0008,   /* U+1E06 */
  0x0002,   /* U+1E07 */
  0x0008,   /* U+1E08 */
  0x0002,   /* U+1E09 */
  0x0008,   /* U+1E0A */
  0x0002,   /* U+1E0B */
  0x0008,   /* U+1E0C */
  0x0002,   /* U+1E0D */
  0x0008,   /* U+1E0E */
  0x0002,   /* U+1E0F */
  0x0008,   /* U+1E10 */
  0x0002,   /* U+1E11 */
  0x0008,   /* U+1E12 */
  0x0002,   /* U+1E13 */
  0x0008,   /* U+1E14 */
  0x0002,   /* U+1E15 */
  0x0008,   /* U+1E16 */
  0x0002,   /* U+1E17 */
  0x0008,   /* U+1E18 */
  0x0002,   /* U+1E19 */
  0x0008,   /* U+1E1A */
  0x0002,   /* U+1E1B */
  0x0008,   /* U+1E1C */
  0x0002,   /* U+1E1D */
  0x0008,   /* U+1E1E */
  0x0002,   /* U+1E1F */
  0x0008,   /* U+1E20 */
  0x0002,   /* U+1E21 */
  0x0008,   /* U+1E22 */
  0x0002,   /* U+1E23 */
  0x0008,   /* U+1E24 */
  0x0002,   /* U+1E25 */
  0x0008,   /* U+1E26 */
  0x0002,   /* U+1E27 */
  0x0008,   /* U+1E28 */
  0x0002,   /* U+1E29 */
  0x0008,   /* U+1E2A */
  0x0002,   /* U+1E2B */
  0x0008,   /* U+1E2C */
  0x0002,   /* U+1E2D */
  0x0008,   /* U+1E2E */
  0x0002,   /* U+1E2F */
  0x0008,   /* U+1E30 */
  0x0002,   /* U+1E31 */
  0x0008,   /* U+1E32 */
  0x0002,   /* U+1E33 */
  0x0008,   /* U+1E34 */
  0x0002,   /* U+1E35 */
  0x0008,   /* U+1E36 */
  0x0002,   /* U+1E37 */
  0x0008,   /* U+1E38 */
  0x0002,   /* U+1E39 */
  0x0008,   /* U+1E3A */
  0x0002,   /* U+1E3B */
  0x0008,   /* U+1E3C */
  0x0002,   /* U+1E3D */
  0x0008,   /* U+1E3E */
  0x0002,   /* U+1E3F */
  0x0008,   /* U+1E40 */
  0x0002,   /* U+1E41 */
  0x0008,   /* U+1E42 */
  0x0002,   /* U+1E43 */
  0x0008,   /* U+1E44 */
  0x0002,   /* U+1E45 */
  0x0008,   /* U+1E46 */
  0x0002,   /* U+1E47 */
  0x0008,   /* U+1E48 */
  0x0002,   /* U+1E49 */
  0x0008,   /* U+1E4A */
  0x0002,   /* U+1E4B */
  0x0008,   /* U+1E4C */
  0x0002,   /* U+1E4D */
  0x0008,   /* U+1E4E */
  0x0002,   /* U+1E4F */
  0x0008,   /* U+1E50 */
  0x0002,   /* U+1E51 */
  0x0008,   /* U+1E52 */
  0x0002,   /* U+1E53 */
  0x0008,   /* U+1E54 */
  0x0002,   /* U+1E55 */
  0x0008,   /* U+1E56 */
  0x0002,   /* U+1E57 */
  0x0008,   /* U+1E58 */
  0x0002,   /* U+1E59 */
  0x0008,   /* U+1E5A */
  0x0002,   /* U+1E5B */
  0x0008,   /* U+1E5C */
  0x0002,   /* U+1E5D */
  0x0008,   /* U+1E5E */
  0x0002,   /* U+1E5F */
  0x0008,   /* U+1E60 */
  0x0002,   /* U+1E61 */
  0x0008,   /* U+1E62 */
  0x0002,   /* U+1E63 */
  0x0008,   /* U+1E64 */
  0x0002,   /* U+1E65 */
  0x0008,   /* U+1E66 */
  0x0002,   /* U+1E67 */
  0x0008,   /* U+1E68 */
  0x0002,   /* U+1E69 */
  0x0008,   /* U+1E6A */
  0x0002,   /* U+1E6B */
  0x0008,   /* U+1E6C */
  0x0002,   /* U+1E6D */
  0x0008,   /* U+1E6E */
  0x0002,   /* U+1E6F */
  0x0008,   /* U+1E70 */
  0x0002,   /* U+1E71 */
  0x0008,   /* U+1E72 */
  0x0002,   /* U+1E73 */
  0x0008,   /* U+1E74 */
  0x0002,   /* U+1E75 */
  0x0008,   /* U+1E76 */
  0x0002,   /* U+1E77 */
  0x0008,   /* U+1E78 */
  0x0002,   /* U+1E79 */
  0x0008,   /* U+1E7A */
  0x0002,   /* U+1E7B */
  0x0008,   /* U+1E7C */
  0x0002,   /* U+1E7D */
  0x0008,   /* U+1E7E */
  0x0002,   /* U+1E7F */
  0x0008,   /* U+1E80 */
  0x0002,   /* U+1E81 */
  0x0008,   /* U+1E82 */
  0x0002,   /* U+1E83 */
  0x0008,   /* U+1E84 */
  0x0002,   /* U+1E85 */
  0x0008,   /* U+1E86 */
  0x0002,   /* U+1E87 */
  0x0008,   /* U+1E88 */
  0x0002,   /* U+1E89 */
  0x0008,   /* U+1E8A */
  0x0002,   /* U+1E8B */
  0x0008,   /* U+1E8C */
  0x0002,   /* U+1E8D */
  0x0008,   /* U+1E8E */
  0x0002,   /* U+1E8F */
  0x0008,   /* U+1E90 */
  0x0002,   /* U+1E91 */
  0x0008,   /* U+1E92 */
  0x0002,   /* U+1E93 */
  0x0008,   /* U+1E94 */
  0x0002,   /* U+1E95 */
  0x0002,   /* U+1E96 */
  0x0002,   /* U+1E97 */
  0x0002,   /* U+1E98 */
  0x0002,   /* U+1E99 */
  0x0004,   /* U+1E9A */
  0x0004,   /* U+1E9B */
  0x0002,   /* U+1E9C */
  0x0002,   /* U+1E9D */
  0x000A,   /* U+1E9E */
  0x0002,   /* U+1E9F */
  0x0008,   /* U+1EA0 */
  0x0002,   /* U+1EA1 */
  0x0008,   /* U+1EA2 */
  0x0002,   /* U+1EA3 */
  0x0008,   /* U+1EA4 */
  0x0002,   /* U+1EA5 */
  0x0008,   /* U+1EA6 */
  0x0002,   /* U+1EA7 */
  0x0008,   /* U+1EA8 */
  0x0002,   /* U+1EA9 */
  0x0008,   /* U+1EAA */
  0x0002,   /* U+1EAB */
  0x0008,   /* U+1EAC */
  0x0002,   /* U+1EAD */
  0x0008,   /* U+1EAE */
  0x0002,   /* U+1EAF */
  0x0008,   /* U+1EB0 */
  0x0002,   /* U+1EB1 */
  0x0008,   /* U+1EB2 */
  0x0002,   /* U+1EB3 */
  0x0008,   /* U+1EB4 */
  0x0002,   /* U+1EB5 */
  0x0008,   /* U+1EB6 */
  0x0002,   /* U+1EB7 */
  0x0008,   /* U+1EB8 */
  0x0002,   /* U+1EB9 */
  0x0008,   /* U+1EBA */
  0x0002,   /* U+1EBB */
  0x0008,   /* U+1EBC */
  0x0002,   /* U+1EBD */
  0x0008,   /* U+1EBE */
  0x0002,   /* U+1EBF */
  0x0008,   /* U+1EC0 */
  0x0002,   /* U+1EC1 */
  0x0008,   /* U+1EC2 */
  0x0002,   /* U+1EC3 */
  0x0008,   /* U+1EC4 */
  0x0002,   /* U+1EC5 */
  0x0008,   /* U+1EC6 */
  0x0002,   /* U+1EC7 */
  0x0008,   /* U+1EC8 */
  0x0002,   /* U+1EC9 */
  0x0008,   /* U+1ECA */
  0x0002,   /* U+1ECB */
  0x0008,   /* U+1ECC */
  0x0002,   /* U+1ECD */
  0x0008,   /* U+1ECE */
  0x0002,   /* U+1ECF */
  0x0008,   /* U+1ED0 */
  0x0002,   /* U+1ED1 */
  0x0008,   /* U+1ED2 */
  0x0002,   /* U+1ED3 */
  0x0008,   /* U+1ED4 */
  0x0002,   /* U+1ED5 */
  0x0008,   /* U+1ED6 */
  0x0002,   /* U+1ED7 */
  0x0008,   /* U+1ED8 */
  0x0002,   /* U+1ED9 */
  0x0008,   /* U+1EDA */
  0x0002,   /* U+1EDB */
  0x0008,   /* U+1EDC */
  0x0002,   /* U+1EDD */
  0x0008,   /* U+1EDE */
  0x0002,   /* U+1EDF */
  0x0008,   /* U+1EE0 */
  0x0002,   /* U+1EE1 */
  0x0008,   /* U+1EE2 */
  0x0002,   /* U+1EE3 */
  0x0008,   /* U+1EE4 */
  0x0002,   /* U+1EE5 */
  0x0008,   /* U+1EE6 */
  0x0002,   /* U+1EE7 */
  0x0008,   /* U+1EE8 */
  0x0002,   /* U+1EE9 */
  0x0008,   /* U+1EEA */
  0x0002,   /* U+1EEB */
  0x0008,   /* U+1EEC */
  0x0002,   /* U+1EED */
  0x0008,   /* U+1EEE */
  0x0002,   /* U+1EEF */
  0x0008,   /* U+1EF0 */
  0x0002,   /* U+1EF1 */
  0x0008,   /* U+1EF2 */
  0x0002,   /* U+1EF3 */
  0x0008,   /* U+1EF4 */
  0x0002,   /* U+1EF5 */
  0x0008,   /* U+1EF6 */
  0x0002,   /* U+1EF7 */
  0x0008,   /* U+1EF8 */
  0x0002,   /* U+1EF9 */
  0x000A,   /* U+1EFA */
  0x0004,   /* U+1EFB */
  0x0008,   /* U+1EFC */
  0x0002,   /* U+1EFD */
  0x0008,   /* U+1EFE */
  0x0002,   /* U+1EFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1E00 */
  0x0000,   /* U+1E01 */
  0x0000,   /* U+1E02 */
  0x0000,   /* U+1E03 */
  0x0000,   /* U+1E04 */
  0x0000,   /* U+1E05 */
  0x0000,   /* U+1E06 */
  0x0000,   /* U+1E07 */
  0x0000,   /* U+1E08 */
  0x0000,   /* U+1E09 */
  0x0000,   /* U+1E0A */
  0x0000,   /* U+1E0B */
  0x0000,   /* U+1E0C */
  0x0000,   /* U+1E0D */
  0x0000,   /* U+1E0E */
  0x0000,   /* U+1E0F */
  0x0000,   /* U+1E10 */
  0x0000,   /* U+1E11 */
  0x0000,   /* U+1E12 */
  0x0000,   /* U+1E13 */
  0x0000,   /* U+1E14 */
  0x0000,   /* U+1E15 */
  0x0000,   /* U+1E16 */
  0x0000,   /* U+1E17 */
  0x0000,   /* U+1E18 */
  0x0000,   /* U+1E19 */
  0x0000,   /* U+1E1A */
  0x0000,   /* U+1E1B */
  0x0000,   /* U+1E1C */
  0x0000,   /* U+1E1D */
  0x0000,   /* U+1E1E */
  0x0000,   /* U+1E1F */
  0x0000,   /* U+1E20 */
  0x0000,   /* U+1E21 */
  0x0000,   /* U+1E22 */
  0x0000,   /* U+1E23 */
  0x0000,   /* U+1E24 */
  0x0000,   /* U+1E25 */
  0x0000,   /* U+1E26 */
  0x0000,   /* U+1E27 */
  0x0000,   /* U+1E28 */
  0x0000,   /* U+1E29 */
  0x0000,   /* U+1E2A */
  0x0000,   /* U+1E2B */
  0x0000,   /* U+1E2C */
  0x0000,   /* U+1E2D */
  0x0000,   /* U+1E2E */
  0x0000,   /* U+1E2F */
  0x0000,   /* U+1E30 */
  0x0000,   /* U+1E31 */
  0x0000,   /* U+1E32 */
  0x0000,   /* U+1E33 */
  0x0000,   /* U+1E34 */
  0x0000,   /* U+1E35 */
  0x0000,   /* U+1E36 */
  0x0000,   /* U+1E37 */
  0x0000,   /* U+1E38 */
  0x0000,   /* U+1E39 */
  0x0000,   /* U+1E3A */
  0x0000,   /* U+1E3B */
  0x0000,   /* U+1E3C */
  0x0000,   /* U+1E3D */
  0x0000,   /* U+1E3E */
  0x0000,   /* U+1E3F */
  0x0000,   /* U+1E40 */
  0x0000,   /* U+1E41 */
  0x0000,   /* U+1E42 */
  0x0000,   /* U+1E43 */
  0x0000,   /* U+1E44 */
  0x0000,   /* U+1E45 */
  0x0000,   /* U+1E46 */
  0x0000,   /* U+1E47 */
  0x0000,   /* U+1E48 */
  0x0000,   /* U+1E49 */
  0x0000,   /* U+1E4A */
  0x0000,   /* U+1E4B */
  0x0000,   /* U+1E4C */
  0x0000,   /* U+1E4D */
  0x0000,   /* U+1E4E */
  0x0000,   /* U+1E4F */
  0x0000,   /* U+1E50 */
  0x0000,   /* U+1E51 */
  0x0000,   /* U+1E52 */
  0x0000,   /* U+1E53 */
  0x0000,   /* U+1E54 */
  0x0000,   /* U+1E55 */
  0x0000,   /* U+1E56 */
  0x0000,   /* U+1E57 */
  0x0000,   /* U+1E58 */
  0x0000,   /* U+1E59 */
  0x0000,   /* U+1E5A */
  0x0000,   /* U+1E5B */
  0x0000,   /* U+1E5C */
  0x0000,   /* U+1E5D */
  0x0000,   /* U+1E5E */
  0x0000,   /* U+1E5F */
  0x0000,   /* U+1E60 */
  0x0000,   /* U+1E61 */
  0x0000,   /* U+1E62 */
  0x0000,   /* U+1E63 */
  0x0000,   /* U+1E64 */
  0x0000,   /* U+1E65 */
  0x0000,   /* U+1E66 */
  0x0000,   /* U+1E67 */
  0x0000,   /* U+1E68 */
  0x0000,   /* U+1E69 */
  0x0000,   /* U+1E6A */
  0x0000,   /* U+1E6B */
  0x0000,   /* U+1E6C */
  0x0000,   /* U+1E6D */
  0x0000,   /* U+1E6E */
  0x0000,   /* U+1E6F */
  0x0000,   /* U+1E70 */
  0x0000,   /* U+1E71 */
  0x0000,   /* U+1E72 */
  0x0000,   /* U+1E73 */
  0x0000,   /* U+1E74 */
  0x0000,   /* U+1E75 */
  0x0000,   /* U+1E76 */
  0x0000,   /* U+1E77 */
  0x0000,   /* U+1E78 */
  0x0000,   /* U+1E79 */
  0x0000,   /* U+1E7A */
  0x0000,   /* U+1E7B */
  0x0000,   /* U+1E7C */
  0x0000,   /* U+1E7D */
  0x0000,   /* U+1E7E */
  0x0000,   /* U+1E7F */
  0x0000,   /* U+1E80 */
  0x0000,   /* U+1E81 */
  0x0000,   /* U+1E82 */
  0x0000,   /* U+1E83 */
  0x0000,   /* U+1E84 */
  0x0000,   /* U+1E85 */
  0x0000,   /* U+1E86 */
  0x0000,   /* U+1E87 */
  0x0000,   /* U+1E88 */
  0x0000,   /* U+1E89 */
  0x0000,   /* U+1E8A */
  0x0000,   /* U+1E8B */
  0x0000,   /* U+1E8C */
  0x0000,   /* U+1E8D */
  0x0000,   /* U+1E8E */
  0x0000,   /* U+1E8F */
  0x0000,   /* U+1E90 */
  0x0000,   /* U+1E91 */
  0x0000,   /* U+1E92 */
  0x0000,   /* U+1E93 */
  0x0000,   /* U+1E94 */
  0x0000,   /* U+1E95 */
  0x0000,   /* U+1E96 */
  0x0000,   /* U+1E97 */
  0x0000,   /* U+1E98 */
  0x0000,   /* U+1E99 */
  0x1F80,   /* U+1E9A */
  0x0000,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x0000,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0000,   /* U+1EA0 */
  0x0000,   /* U+1EA1 */
  0x0000,   /* U+1EA2 */
  0x0000,   /* U+1EA3 */
  0x0000,   /* U+1EA4 */
  0x0000,   /* U+1EA5 */
  0x0000,   /* U+1EA6 */
  0x0000,   /* U+1EA7 */
  0x0000,   /* U+1EA8 */
  0x0000,   /* U+1EA9 */
  0x0000,   /* U+1EAA */
  0x0000,   /* U+1EAB */
  0x0000,   /* U+1EAC */
  0x0000,   /* U+1EAD */
  0x0000,   /* U+1EAE */
  0x0000,   /* U+1EAF */
  0x0000,   /* U+1EB0 */
  0x0000,   /* U+1EB1 */
  0x0000,   /* U+1EB2 */
  0x0000,   /* U+1EB3 */
  0x0000,   /* U+1EB4 */
  0x0000,   /* U+1EB5 */
  0x0000,   /* U+1EB6 */
  0x0000,   /* U+1EB7 */
  0x0000,   /* U+1EB8 */
  0x0000,   /* U+1EB9 */
  0x0000,   /* U+1EBA */
  0x0000,   /* U+1EBB */
  0x0000,   /* U+1EBC */
  0x0000,   /* U+1EBD */
  0x0000,   /* U+1EBE */
  0x0000,   /* U+1EBF */
  0x0000,   /* U+1EC0 */
  0x0000,   /* U+1EC1 */
  0x0000,   /* U+1EC2 */
  0x0000,   /* U+1EC3 */
  0x0000,   /* U+1EC4 */
  0x0000,   /* U+1EC5 */
  0x0000,   /* U+1EC6 */
  0x0000,   /* U+1EC7 */
  0x0000,   /* U+1EC8 */
  0x0000,   /* U+1EC9 */
  0x0000,   /* U+1ECA */
  0x0000,   /* U+1ECB */
  0x0000,   /* U+1ECC */
  0x0000,   /* U+1ECD */
  0x0000,   /* U+1ECE */
  0x0000,   /* U+1ECF */
  0x0000,   /* U+1ED0 */
  0x0000,   /* U+1ED1 */
  0x0000,   /* U+1ED2 */
  0x0000,   /* U+1ED3 */
  0x0000,   /* U+1ED4 */
  0x0000,   /* U+1ED5 */
  0x0000,   /* U+1ED6 */
  0x0000,   /* U+1ED7 */
  0x0000,   /* U+1ED8 */
  0x0000,   /* U+1ED9 */
  0x0000,   /* U+1EDA */
  0x0000,   /* U+1EDB */
  0x0000,   /* U+1EDC */
  0x0000,   /* U+1EDD */
  0x0000,   /* U+1EDE */
  0x0000,   /* U+1EDF */
  0x0000,   /* U+1EE0 */
  0x0000,   /* U+1EE1 */
  0x0000,   /* U+1EE2 */
  0x0000,   /* U+1EE3 */
  0x0000,   /* U+1EE4 */
  0x0000,   /* U+1EE5 */
  0x0000,   /* U+1EE6 */
  0x0000,   /* U+1EE7 */
  0x0000,   /* U+1EE8 */
  0x0000,   /* U+1EE9 */
  0x0000,   /* U+1EEA */
  0x0000,   /* U+1EEB */
  0x0000,   /* U+1EEC */
  0x0000,   /* U+1EED */
  0x0000,   /* U+1EEE */
  0x0000,   /* U+1EEF */
  0x0000,   /* U+1EF0 */
  0x0000,   /* U+1EF1 */
  0x0000,   /* U+1EF2 */
  0x0000,   /* U+1EF3 */
  0x0000,   /* U+1EF4 */
  0x0000,   /* U+1EF5 */
  0x0000,   /* U+1EF6 */
  0x0000,   /* U+1EF7 */
  0x0000,   /* U+1EF8 */
  0x0000,   /* U+1EF9 */
  0x1D77,   /* U+1EFA */
  0x1D77,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */

  /* Secondary weight 2 for each character. */
  0x0044,   /* U+1E00 */
  0x0044,   /* U+1E01 */
  0x002E,   /* U+1E02 */
  0x002E,   /* U+1E03 */
  0x0042,   /* U+1E04 */
  0x0042,   /* U+1E05 */
  0x0049,   /* U+1E06 */
  0x0049,   /* U+1E07 */
  0x0030,   /* U+1E08 */
  0x0030,   /* U+1E09 */
  0x002E,   /* U+1E0A */
  0x002E,   /* U+1E0B */
  0x0042,   /* U+1E0C */
  0x0042,   /* U+1E0D */
  0x0049,   /* U+1E0E */
  0x0049,   /* U+1E0F */
  0x0030,   /* U+1E10 */
  0x0030,   /* U+1E11 */
  0x0046,   /* U+1E12 */
  0x0046,   /* U+1E13 */
  0x0032,   /* U+1E14 */
  0x0032,   /* U+1E15 */
  0x0032,   /* U+1E16 */
  0x0032,   /* U+1E17 */
  0x0046,   /* U+1E18 */
  0x0046,   /* U+1E19 */
  0x0048,   /* U+1E1A */
  0x0048,   /* U+1E1B */
  0x0030,   /* U+1E1C */
  0x0030,   /* U+1E1D */
  0x002E,   /* U+1E1E */
  0x002E,   /* U+1E1F */
  0x0032,   /* U+1E20 */
  0x0032,   /* U+1E21 */
  0x002E,   /* U+1E22 */
  0x002E,   /* U+1E23 */
  0x0042,   /* U+1E24 */
  0x0042,   /* U+1E25 */
  0x002B,   /* U+1E26 */
  0x002B,   /* U+1E27 */
  0x0030,   /* U+1E28 */
  0x0030,   /* U+1E29 */
  0x0047,   /* U+1E2A */
  0x0047,   /* U+1E2B */
  0x0048,   /* U+1E2C */
  0x0048,   /* U+1E2D */
  0x002B,   /* U+1E2E */
  0x002B,   /* U+1E2F */
  0x0024,   /* U+1E30 */
  0x0024,   /* U+1E31 */
  0x0042,   /* U+1E32 */
  0x0042,   /* U+1E33 */
  0x0049,   /* U+1E34 */
  0x0049,   /* U+1E35 */
  0x0042,   /* U+1E36 */
  0x0042,   /* U+1E37 */
  0x0042,   /* U+1E38 */
  0x0042,   /* U+1E39 */
  0x0049,   /* U+1E3A */
  0x0049,   /* U+1E3B */
  0x0046,   /* U+1E3C */
  0x0046,   /* U+1E3D */
  0x0024,   /* U+1E3E */
  0x0024,   /* U+1E3F */
  0x002E,   /* U+1E40 */
  0x002E,   /* U+1E41 */
  0x0042,   /* U+1E42 */
  0x0042,   /* U+1E43 */
  0x002E,   /* U+1E44 */
  0x002E,   /* U+1E45 */
  0x0042,   /* U+1E46 */
  0x0042,   /* U+1E47 */
  0x0049,   /* U+1E48 */
  0x0049,   /* U+1E49 */
  0x0046,   /* U+1E4A */
  0x0046,   /* U+1E4B */
  0x002D,   /* U+1E4C */
  0x002D,   /* U+1E4D */
  0x002D,   /* U+1E4E */
  0x002D,   /* U+1E4F */
  0x0032,   /* U+1E50 */
  0x0032,   /* U+1E51 */
  0x0032,   /* U+1E52 */
  0x0032,   /* U+1E53 */
  0x0024,   /* U+1E54 */
  0x0024,   /* U+1E55 */
  0x002E,   /* U+1E56 */
  0x002E,   /* U+1E57 */
  0x002E,   /* U+1E58 */
  0x002E,   /* U+1E59 */
  0x0042,   /* U+1E5A */
  0x0042,   /* U+1E5B */
  0x0042,   /* U+1E5C */
  0x0042,   /* U+1E5D */
  0x0049,   /* U+1E5E */
  0x0049,   /* U+1E5F */
  0x002E,   /* U+1E60 */
  0x002E,   /* U+1E61 */
  0x0042,   /* U+1E62 */
  0x0042,   /* U+1E63 */
  0x0024,   /* U+1E64 */
  0x0024,   /* U+1E65 */
  0x0028,   /* U+1E66 */
  0x0028,   /* U+1E67 */
  0x0042,   /* U+1E68 */
  0x0042,   /* U+1E69 */
  0x002E,   /* U+1E6A */
  0x002E,   /* U+1E6B */
  0x0042,   /* U+1E6C */
  0x0042,   /* U+1E6D */
  0x0049,   /* U+1E6E */
  0x0049,   /* U+1E6F */
  0x0046,   /* U+1E70 */
  0x0046,   /* U+1E71 */
  0x0043,   /* U+1E72 */
  0x0043,   /* U+1E73 */
  0x0048,   /* U+1E74 */
  0x0048,   /* U+1E75 */
  0x0046,   /* U+1E76 */
  0x0046,   /* U+1E77 */
  0x002D,   /* U+1E78 */
  0x002D,   /* U+1E79 */
  0x0032,   /* U+1E7A */
  0x0032,   /* U+1E7B */
  0x002D,   /* U+1E7C */
  0x002D,   /* U+1E7D */
  0x0042,   /* U+1E7E */
  0x0042,   /* U+1E7F */
  0x0025,   /* U+1E80 */
  0x0025,   /* U+1E81 */
  0x0024,   /* U+1E82 */
  0x0024,   /* U+1E83 */
  0x002B,   /* U+1E84 */
  0x002B,   /* U+1E85 */
  0x002E,   /* U+1E86 */
  0x002E,   /* U+1E87 */
  0x0042,   /* U+1E88 */
  0x0042,   /* U+1E89 */
  0x002E,   /* U+1E8A */
  0x002E,   /* U+1E8B */
  0x002B,   /* U+1E8C */
  0x002B,   /* U+1E8D */
  0x002E,   /* U+1E8E */
  0x002E,   /* U+1E8F */
  0x0027,   /* U+1E90 */
  0x0027,   /* U+1E91 */
  0x0042,   /* U+1E92 */
  0x0042,   /* U+1E93 */
  0x0049,   /* U+1E94 */
  0x0049,   /* U+1E95 */
  0x0049,   /* U+1E96 */
  0x002B,   /* U+1E97 */
  0x0029,   /* U+1E98 */
  0x0029,   /* U+1E99 */
  0x0020,   /* U+1E9A */
  0x0111,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x0110,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0042,   /* U+1EA0 */
  0x0042,   /* U+1EA1 */
  0x003B,   /* U+1EA2 */
  0x003B,   /* U+1EA3 */
  0x0027,   /* U+1EA4 */
  0x0027,   /* U+1EA5 */
  0x0027,   /* U+1EA6 */
  0x0027,   /* U+1EA7 */
  0x0027,   /* U+1EA8 */
  0x0027,   /* U+1EA9 */
  0x0027,   /* U+1EAA */
  0x0027,   /* U+1EAB */
  0x0042,   /* U+1EAC */
  0x0042,   /* U+1EAD */
  0x0026,   /* U+1EAE */
  0x0026,   /* U+1EAF */
  0x0026,   /* U+1EB0 */
  0x0026,   /* U+1EB1 */
  0x0026,   /* U+1EB2 */
  0x0026,   /* U+1EB3 */
  0x0026,   /* U+1EB4 */
  0x0026,   /* U+1EB5 */
  0x0042,   /* U+1EB6 */
  0x0042,   /* U+1EB7 */
  0x0042,   /* U+1EB8 */
  0x0042,   /* U+1EB9 */
  0x003B,   /* U+1EBA */
  0x003B,   /* U+1EBB */
  0x002D,   /* U+1EBC */
  0x002D,   /* U+1EBD */
  0x0027,   /* U+1EBE */
  0x0027,   /* U+1EBF */
  0x0027,   /* U+1EC0 */
  0x0027,   /* U+1EC1 */
  0x0027,   /* U+1EC2 */
  0x0027,   /* U+1EC3 */
  0x0027,   /* U+1EC4 */
  0x0027,   /* U+1EC5 */
  0x0042,   /* U+1EC6 */
  0x0042,   /* U+1EC7 */
  0x003B,   /* U+1EC8 */
  0x003B,   /* U+1EC9 */
  0x0042,   /* U+1ECA */
  0x0042,   /* U+1ECB */
  0x0042,   /* U+1ECC */
  0x0042,   /* U+1ECD */
  0x003B,   /* U+1ECE */
  0x003B,   /* U+1ECF */
  0x0027,   /* U+1ED0 */
  0x0027,   /* U+1ED1 */
  0x0027,   /* U+1ED2 */
  0x0027,   /* U+1ED3 */
  0x0027,   /* U+1ED4 */
  0x0027,   /* U+1ED5 */
  0x0027,   /* U+1ED6 */
  0x0027,   /* U+1ED7 */
  0x0042,   /* U+1ED8 */
  0x0042,   /* U+1ED9 */
  0x003F,   /* U+1EDA */
  0x003F,   /* U+1EDB */
  0x003F,   /* U+1EDC */
  0x003F,   /* U+1EDD */
  0x003F,   /* U+1EDE */
  0x003F,   /* U+1EDF */
  0x003F,   /* U+1EE0 */
  0x003F,   /* U+1EE1 */
  0x003F,   /* U+1EE2 */
  0x003F,   /* U+1EE3 */
  0x0042,   /* U+1EE4 */
  0x0042,   /* U+1EE5 */
  0x003B,   /* U+1EE6 */
  0x003B,   /* U+1EE7 */
  0x003F,   /* U+1EE8 */
  0x003F,   /* U+1EE9 */
  0x003F,   /* U+1EEA */
  0x003F,   /* U+1EEB */
  0x003F,   /* U+1EEC */
  0x003F,   /* U+1EED */
  0x003F,   /* U+1EEE */
  0x003F,   /* U+1EEF */
  0x003F,   /* U+1EF0 */
  0x003F,   /* U+1EF1 */
  0x0025,   /* U+1EF2 */
  0x0025,   /* U+1EF3 */
  0x0042,   /* U+1EF4 */
  0x0042,   /* U+1EF5 */
  0x003B,   /* U+1EF6 */
  0x003B,   /* U+1EF7 */
  0x002D,   /* U+1EF8 */
  0x002D,   /* U+1EF9 */
  0x0020,   /* U+1EFA */
  0x0020,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */

  /* Tertiary weight 2 for each character. */
  0x0002,   /* U+1E00 */
  0x0002,   /* U+1E01 */
  0x0002,   /* U+1E02 */
  0x0002,   /* U+1E03 */
  0x0002,   /* U+1E04 */
  0x0002,   /* U+1E05 */
  0x0002,   /* U+1E06 */
  0x0002,   /* U+1E07 */
  0x0002,   /* U+1E08 */
  0x0002,   /* U+1E09 */
  0x0002,   /* U+1E0A */
  0x0002,   /* U+1E0B */
  0x0002,   /* U+1E0C */
  0x0002,   /* U+1E0D */
  0x0002,   /* U+1E0E */
  0x0002,   /* U+1E0F */
  0x0002,   /* U+1E10 */
  0x0002,   /* U+1E11 */
  0x0002,   /* U+1E12 */
  0x0002,   /* U+1E13 */
  0x0002,   /* U+1E14 */
  0x0002,   /* U+1E15 */
  0x0002,   /* U+1E16 */
  0x0002,   /* U+1E17 */
  0x0002,   /* U+1E18 */
  0x0002,   /* U+1E19 */
  0x0002,   /* U+1E1A */
  0x0002,   /* U+1E1B */
  0x0002,   /* U+1E1C */
  0x0002,   /* U+1E1D */
  0x0002,   /* U+1E1E */
  0x0002,   /* U+1E1F */
  0x0002,   /* U+1E20 */
  0x0002,   /* U+1E21 */
  0x0002,   /* U+1E22 */
  0x0002,   /* U+1E23 */
  0x0002,   /* U+1E24 */
  0x0002,   /* U+1E25 */
  0x0002,   /* U+1E26 */
  0x0002,   /* U+1E27 */
  0x0002,   /* U+1E28 */
  0x0002,   /* U+1E29 */
  0x0002,   /* U+1E2A */
  0x0002,   /* U+1E2B */
  0x0002,   /* U+1E2C */
  0x0002,   /* U+1E2D */
  0x0002,   /* U+1E2E */
  0x0002,   /* U+1E2F */
  0x0002,   /* U+1E30 */
  0x0002,   /* U+1E31 */
  0x0002,   /* U+1E32 */
  0x0002,   /* U+1E33 */
  0x0002,   /* U+1E34 */
  0x0002,   /* U+1E35 */
  0x0002,   /* U+1E36 */
  0x0002,   /* U+1E37 */
  0x0002,   /* U+1E38 */
  0x0002,   /* U+1E39 */
  0x0002,   /* U+1E3A */
  0x0002,   /* U+1E3B */
  0x0002,   /* U+1E3C */
  0x0002,   /* U+1E3D */
  0x0002,   /* U+1E3E */
  0x0002,   /* U+1E3F */
  0x0002,   /* U+1E40 */
  0x0002,   /* U+1E41 */
  0x0002,   /* U+1E42 */
  0x0002,   /* U+1E43 */
  0x0002,   /* U+1E44 */
  0x0002,   /* U+1E45 */
  0x0002,   /* U+1E46 */
  0x0002,   /* U+1E47 */
  0x0002,   /* U+1E48 */
  0x0002,   /* U+1E49 */
  0x0002,   /* U+1E4A */
  0x0002,   /* U+1E4B */
  0x0002,   /* U+1E4C */
  0x0002,   /* U+1E4D */
  0x0002,   /* U+1E4E */
  0x0002,   /* U+1E4F */
  0x0002,   /* U+1E50 */
  0x0002,   /* U+1E51 */
  0x0002,   /* U+1E52 */
  0x0002,   /* U+1E53 */
  0x0002,   /* U+1E54 */
  0x0002,   /* U+1E55 */
  0x0002,   /* U+1E56 */
  0x0002,   /* U+1E57 */
  0x0002,   /* U+1E58 */
  0x0002,   /* U+1E59 */
  0x0002,   /* U+1E5A */
  0x0002,   /* U+1E5B */
  0x0002,   /* U+1E5C */
  0x0002,   /* U+1E5D */
  0x0002,   /* U+1E5E */
  0x0002,   /* U+1E5F */
  0x0002,   /* U+1E60 */
  0x0002,   /* U+1E61 */
  0x0002,   /* U+1E62 */
  0x0002,   /* U+1E63 */
  0x0002,   /* U+1E64 */
  0x0002,   /* U+1E65 */
  0x0002,   /* U+1E66 */
  0x0002,   /* U+1E67 */
  0x0002,   /* U+1E68 */
  0x0002,   /* U+1E69 */
  0x0002,   /* U+1E6A */
  0x0002,   /* U+1E6B */
  0x0002,   /* U+1E6C */
  0x0002,   /* U+1E6D */
  0x0002,   /* U+1E6E */
  0x0002,   /* U+1E6F */
  0x0002,   /* U+1E70 */
  0x0002,   /* U+1E71 */
  0x0002,   /* U+1E72 */
  0x0002,   /* U+1E73 */
  0x0002,   /* U+1E74 */
  0x0002,   /* U+1E75 */
  0x0002,   /* U+1E76 */
  0x0002,   /* U+1E77 */
  0x0002,   /* U+1E78 */
  0x0002,   /* U+1E79 */
  0x0002,   /* U+1E7A */
  0x0002,   /* U+1E7B */
  0x0002,   /* U+1E7C */
  0x0002,   /* U+1E7D */
  0x0002,   /* U+1E7E */
  0x0002,   /* U+1E7F */
  0x0002,   /* U+1E80 */
  0x0002,   /* U+1E81 */
  0x0002,   /* U+1E82 */
  0x0002,   /* U+1E83 */
  0x0002,   /* U+1E84 */
  0x0002,   /* U+1E85 */
  0x0002,   /* U+1E86 */
  0x0002,   /* U+1E87 */
  0x0002,   /* U+1E88 */
  0x0002,   /* U+1E89 */
  0x0002,   /* U+1E8A */
  0x0002,   /* U+1E8B */
  0x0002,   /* U+1E8C */
  0x0002,   /* U+1E8D */
  0x0002,   /* U+1E8E */
  0x0002,   /* U+1E8F */
  0x0002,   /* U+1E90 */
  0x0002,   /* U+1E91 */
  0x0002,   /* U+1E92 */
  0x0002,   /* U+1E93 */
  0x0002,   /* U+1E94 */
  0x0002,   /* U+1E95 */
  0x0002,   /* U+1E96 */
  0x0002,   /* U+1E97 */
  0x0002,   /* U+1E98 */
  0x0002,   /* U+1E99 */
  0x0004,   /* U+1E9A */
  0x0004,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x0004,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0002,   /* U+1EA0 */
  0x0002,   /* U+1EA1 */
  0x0002,   /* U+1EA2 */
  0x0002,   /* U+1EA3 */
  0x0002,   /* U+1EA4 */
  0x0002,   /* U+1EA5 */
  0x0002,   /* U+1EA6 */
  0x0002,   /* U+1EA7 */
  0x0002,   /* U+1EA8 */
  0x0002,   /* U+1EA9 */
  0x0002,   /* U+1EAA */
  0x0002,   /* U+1EAB */
  0x0002,   /* U+1EAC */
  0x0002,   /* U+1EAD */
  0x0002,   /* U+1EAE */
  0x0002,   /* U+1EAF */
  0x0002,   /* U+1EB0 */
  0x0002,   /* U+1EB1 */
  0x0002,   /* U+1EB2 */
  0x0002,   /* U+1EB3 */
  0x0002,   /* U+1EB4 */
  0x0002,   /* U+1EB5 */
  0x0002,   /* U+1EB6 */
  0x0002,   /* U+1EB7 */
  0x0002,   /* U+1EB8 */
  0x0002,   /* U+1EB9 */
  0x0002,   /* U+1EBA */
  0x0002,   /* U+1EBB */
  0x0002,   /* U+1EBC */
  0x0002,   /* U+1EBD */
  0x0002,   /* U+1EBE */
  0x0002,   /* U+1EBF */
  0x0002,   /* U+1EC0 */
  0x0002,   /* U+1EC1 */
  0x0002,   /* U+1EC2 */
  0x0002,   /* U+1EC3 */
  0x0002,   /* U+1EC4 */
  0x0002,   /* U+1EC5 */
  0x0002,   /* U+1EC6 */
  0x0002,   /* U+1EC7 */
  0x0002,   /* U+1EC8 */
  0x0002,   /* U+1EC9 */
  0x0002,   /* U+1ECA */
  0x0002,   /* U+1ECB */
  0x0002,   /* U+1ECC */
  0x0002,   /* U+1ECD */
  0x0002,   /* U+1ECE */
  0x0002,   /* U+1ECF */
  0x0002,   /* U+1ED0 */
  0x0002,   /* U+1ED1 */
  0x0002,   /* U+1ED2 */
  0x0002,   /* U+1ED3 */
  0x0002,   /* U+1ED4 */
  0x0002,   /* U+1ED5 */
  0x0002,   /* U+1ED6 */
  0x0002,   /* U+1ED7 */
  0x0002,   /* U+1ED8 */
  0x0002,   /* U+1ED9 */
  0x0002,   /* U+1EDA */
  0x0002,   /* U+1EDB */
  0x0002,   /* U+1EDC */
  0x0002,   /* U+1EDD */
  0x0002,   /* U+1EDE */
  0x0002,   /* U+1EDF */
  0x0002,   /* U+1EE0 */
  0x0002,   /* U+1EE1 */
  0x0002,   /* U+1EE2 */
  0x0002,   /* U+1EE3 */
  0x0002,   /* U+1EE4 */
  0x0002,   /* U+1EE5 */
  0x0002,   /* U+1EE6 */
  0x0002,   /* U+1EE7 */
  0x0002,   /* U+1EE8 */
  0x0002,   /* U+1EE9 */
  0x0002,   /* U+1EEA */
  0x0002,   /* U+1EEB */
  0x0002,   /* U+1EEC */
  0x0002,   /* U+1EED */
  0x0002,   /* U+1EEE */
  0x0002,   /* U+1EEF */
  0x0002,   /* U+1EF0 */
  0x0002,   /* U+1EF1 */
  0x0002,   /* U+1EF2 */
  0x0002,   /* U+1EF3 */
  0x0002,   /* U+1EF4 */
  0x0002,   /* U+1EF5 */
  0x0002,   /* U+1EF6 */
  0x0002,   /* U+1EF7 */
  0x0002,   /* U+1EF8 */
  0x0002,   /* U+1EF9 */
  0x000A,   /* U+1EFA */
  0x0004,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1E00 */
  0x0000,   /* U+1E01 */
  0x0000,   /* U+1E02 */
  0x0000,   /* U+1E03 */
  0x0000,   /* U+1E04 */
  0x0000,   /* U+1E05 */
  0x0000,   /* U+1E06 */
  0x0000,   /* U+1E07 */
  0x0000,   /* U+1E08 */
  0x0000,   /* U+1E09 */
  0x0000,   /* U+1E0A */
  0x0000,   /* U+1E0B */
  0x0000,   /* U+1E0C */
  0x0000,   /* U+1E0D */
  0x0000,   /* U+1E0E */
  0x0000,   /* U+1E0F */
  0x0000,   /* U+1E10 */
  0x0000,   /* U+1E11 */
  0x0000,   /* U+1E12 */
  0x0000,   /* U+1E13 */
  0x0000,   /* U+1E14 */
  0x0000,   /* U+1E15 */
  0x0000,   /* U+1E16 */
  0x0000,   /* U+1E17 */
  0x0000,   /* U+1E18 */
  0x0000,   /* U+1E19 */
  0x0000,   /* U+1E1A */
  0x0000,   /* U+1E1B */
  0x0000,   /* U+1E1C */
  0x0000,   /* U+1E1D */
  0x0000,   /* U+1E1E */
  0x0000,   /* U+1E1F */
  0x0000,   /* U+1E20 */
  0x0000,   /* U+1E21 */
  0x0000,   /* U+1E22 */
  0x0000,   /* U+1E23 */
  0x0000,   /* U+1E24 */
  0x0000,   /* U+1E25 */
  0x0000,   /* U+1E26 */
  0x0000,   /* U+1E27 */
  0x0000,   /* U+1E28 */
  0x0000,   /* U+1E29 */
  0x0000,   /* U+1E2A */
  0x0000,   /* U+1E2B */
  0x0000,   /* U+1E2C */
  0x0000,   /* U+1E2D */
  0x0000,   /* U+1E2E */
  0x0000,   /* U+1E2F */
  0x0000,   /* U+1E30 */
  0x0000,   /* U+1E31 */
  0x0000,   /* U+1E32 */
  0x0000,   /* U+1E33 */
  0x0000,   /* U+1E34 */
  0x0000,   /* U+1E35 */
  0x0000,   /* U+1E36 */
  0x0000,   /* U+1E37 */
  0x0000,   /* U+1E38 */
  0x0000,   /* U+1E39 */
  0x0000,   /* U+1E3A */
  0x0000,   /* U+1E3B */
  0x0000,   /* U+1E3C */
  0x0000,   /* U+1E3D */
  0x0000,   /* U+1E3E */
  0x0000,   /* U+1E3F */
  0x0000,   /* U+1E40 */
  0x0000,   /* U+1E41 */
  0x0000,   /* U+1E42 */
  0x0000,   /* U+1E43 */
  0x0000,   /* U+1E44 */
  0x0000,   /* U+1E45 */
  0x0000,   /* U+1E46 */
  0x0000,   /* U+1E47 */
  0x0000,   /* U+1E48 */
  0x0000,   /* U+1E49 */
  0x0000,   /* U+1E4A */
  0x0000,   /* U+1E4B */
  0x0000,   /* U+1E4C */
  0x0000,   /* U+1E4D */
  0x0000,   /* U+1E4E */
  0x0000,   /* U+1E4F */
  0x0000,   /* U+1E50 */
  0x0000,   /* U+1E51 */
  0x0000,   /* U+1E52 */
  0x0000,   /* U+1E53 */
  0x0000,   /* U+1E54 */
  0x0000,   /* U+1E55 */
  0x0000,   /* U+1E56 */
  0x0000,   /* U+1E57 */
  0x0000,   /* U+1E58 */
  0x0000,   /* U+1E59 */
  0x0000,   /* U+1E5A */
  0x0000,   /* U+1E5B */
  0x0000,   /* U+1E5C */
  0x0000,   /* U+1E5D */
  0x0000,   /* U+1E5E */
  0x0000,   /* U+1E5F */
  0x0000,   /* U+1E60 */
  0x0000,   /* U+1E61 */
  0x0000,   /* U+1E62 */
  0x0000,   /* U+1E63 */
  0x0000,   /* U+1E64 */
  0x0000,   /* U+1E65 */
  0x0000,   /* U+1E66 */
  0x0000,   /* U+1E67 */
  0x0000,   /* U+1E68 */
  0x0000,   /* U+1E69 */
  0x0000,   /* U+1E6A */
  0x0000,   /* U+1E6B */
  0x0000,   /* U+1E6C */
  0x0000,   /* U+1E6D */
  0x0000,   /* U+1E6E */
  0x0000,   /* U+1E6F */
  0x0000,   /* U+1E70 */
  0x0000,   /* U+1E71 */
  0x0000,   /* U+1E72 */
  0x0000,   /* U+1E73 */
  0x0000,   /* U+1E74 */
  0x0000,   /* U+1E75 */
  0x0000,   /* U+1E76 */
  0x0000,   /* U+1E77 */
  0x0000,   /* U+1E78 */
  0x0000,   /* U+1E79 */
  0x0000,   /* U+1E7A */
  0x0000,   /* U+1E7B */
  0x0000,   /* U+1E7C */
  0x0000,   /* U+1E7D */
  0x0000,   /* U+1E7E */
  0x0000,   /* U+1E7F */
  0x0000,   /* U+1E80 */
  0x0000,   /* U+1E81 */
  0x0000,   /* U+1E82 */
  0x0000,   /* U+1E83 */
  0x0000,   /* U+1E84 */
  0x0000,   /* U+1E85 */
  0x0000,   /* U+1E86 */
  0x0000,   /* U+1E87 */
  0x0000,   /* U+1E88 */
  0x0000,   /* U+1E89 */
  0x0000,   /* U+1E8A */
  0x0000,   /* U+1E8B */
  0x0000,   /* U+1E8C */
  0x0000,   /* U+1E8D */
  0x0000,   /* U+1E8E */
  0x0000,   /* U+1E8F */
  0x0000,   /* U+1E90 */
  0x0000,   /* U+1E91 */
  0x0000,   /* U+1E92 */
  0x0000,   /* U+1E93 */
  0x0000,   /* U+1E94 */
  0x0000,   /* U+1E95 */
  0x0000,   /* U+1E96 */
  0x0000,   /* U+1E97 */
  0x0000,   /* U+1E98 */
  0x0000,   /* U+1E99 */
  0x0000,   /* U+1E9A */
  0x0000,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x1E71,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0000,   /* U+1EA0 */
  0x0000,   /* U+1EA1 */
  0x0000,   /* U+1EA2 */
  0x0000,   /* U+1EA3 */
  0x0000,   /* U+1EA4 */
  0x0000,   /* U+1EA5 */
  0x0000,   /* U+1EA6 */
  0x0000,   /* U+1EA7 */
  0x0000,   /* U+1EA8 */
  0x0000,   /* U+1EA9 */
  0x0000,   /* U+1EAA */
  0x0000,   /* U+1EAB */
  0x0000,   /* U+1EAC */
  0x0000,   /* U+1EAD */
  0x0000,   /* U+1EAE */
  0x0000,   /* U+1EAF */
  0x0000,   /* U+1EB0 */
  0x0000,   /* U+1EB1 */
  0x0000,   /* U+1EB2 */
  0x0000,   /* U+1EB3 */
  0x0000,   /* U+1EB4 */
  0x0000,   /* U+1EB5 */
  0x0000,   /* U+1EB6 */
  0x0000,   /* U+1EB7 */
  0x0000,   /* U+1EB8 */
  0x0000,   /* U+1EB9 */
  0x0000,   /* U+1EBA */
  0x0000,   /* U+1EBB */
  0x0000,   /* U+1EBC */
  0x0000,   /* U+1EBD */
  0x0000,   /* U+1EBE */
  0x0000,   /* U+1EBF */
  0x0000,   /* U+1EC0 */
  0x0000,   /* U+1EC1 */
  0x0000,   /* U+1EC2 */
  0x0000,   /* U+1EC3 */
  0x0000,   /* U+1EC4 */
  0x0000,   /* U+1EC5 */
  0x0000,   /* U+1EC6 */
  0x0000,   /* U+1EC7 */
  0x0000,   /* U+1EC8 */
  0x0000,   /* U+1EC9 */
  0x0000,   /* U+1ECA */
  0x0000,   /* U+1ECB */
  0x0000,   /* U+1ECC */
  0x0000,   /* U+1ECD */
  0x0000,   /* U+1ECE */
  0x0000,   /* U+1ECF */
  0x0000,   /* U+1ED0 */
  0x0000,   /* U+1ED1 */
  0x0000,   /* U+1ED2 */
  0x0000,   /* U+1ED3 */
  0x0000,   /* U+1ED4 */
  0x0000,   /* U+1ED5 */
  0x0000,   /* U+1ED6 */
  0x0000,   /* U+1ED7 */
  0x0000,   /* U+1ED8 */
  0x0000,   /* U+1ED9 */
  0x0000,   /* U+1EDA */
  0x0000,   /* U+1EDB */
  0x0000,   /* U+1EDC */
  0x0000,   /* U+1EDD */
  0x0000,   /* U+1EDE */
  0x0000,   /* U+1EDF */
  0x0000,   /* U+1EE0 */
  0x0000,   /* U+1EE1 */
  0x0000,   /* U+1EE2 */
  0x0000,   /* U+1EE3 */
  0x0000,   /* U+1EE4 */
  0x0000,   /* U+1EE5 */
  0x0000,   /* U+1EE6 */
  0x0000,   /* U+1EE7 */
  0x0000,   /* U+1EE8 */
  0x0000,   /* U+1EE9 */
  0x0000,   /* U+1EEA */
  0x0000,   /* U+1EEB */
  0x0000,   /* U+1EEC */
  0x0000,   /* U+1EED */
  0x0000,   /* U+1EEE */
  0x0000,   /* U+1EEF */
  0x0000,   /* U+1EF0 */
  0x0000,   /* U+1EF1 */
  0x0000,   /* U+1EF2 */
  0x0000,   /* U+1EF3 */
  0x0000,   /* U+1EF4 */
  0x0000,   /* U+1EF5 */
  0x0000,   /* U+1EF6 */
  0x0000,   /* U+1EF7 */
  0x0000,   /* U+1EF8 */
  0x0000,   /* U+1EF9 */
  0x0000,   /* U+1EFA */
  0x0000,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1E00 */
  0x0000,   /* U+1E01 */
  0x0000,   /* U+1E02 */
  0x0000,   /* U+1E03 */
  0x0000,   /* U+1E04 */
  0x0000,   /* U+1E05 */
  0x0000,   /* U+1E06 */
  0x0000,   /* U+1E07 */
  0x0024,   /* U+1E08 */
  0x0024,   /* U+1E09 */
  0x0000,   /* U+1E0A */
  0x0000,   /* U+1E0B */
  0x0000,   /* U+1E0C */
  0x0000,   /* U+1E0D */
  0x0000,   /* U+1E0E */
  0x0000,   /* U+1E0F */
  0x0000,   /* U+1E10 */
  0x0000,   /* U+1E11 */
  0x0000,   /* U+1E12 */
  0x0000,   /* U+1E13 */
  0x0025,   /* U+1E14 */
  0x0025,   /* U+1E15 */
  0x0024,   /* U+1E16 */
  0x0024,   /* U+1E17 */
  0x0000,   /* U+1E18 */
  0x0000,   /* U+1E19 */
  0x0000,   /* U+1E1A */
  0x0000,   /* U+1E1B */
  0x0026,   /* U+1E1C */
  0x0026,   /* U+1E1D */
  0x0000,   /* U+1E1E */
  0x0000,   /* U+1E1F */
  0x0000,   /* U+1E20 */
  0x0000,   /* U+1E21 */
  0x0000,   /* U+1E22 */
  0x0000,   /* U+1E23 */
  0x0000,   /* U+1E24 */
  0x0000,   /* U+1E25 */
  0x0000,   /* U+1E26 */
  0x0000,   /* U+1E27 */
  0x0000,   /* U+1E28 */
  0x0000,   /* U+1E29 */
  0x0000,   /* U+1E2A */
  0x0000,   /* U+1E2B */
  0x0000,   /* U+1E2C */
  0x0000,   /* U+1E2D */
  0x0024,   /* U+1E2E */
  0x0024,   /* U+1E2F */
  0x0000,   /* U+1E30 */
  0x0000,   /* U+1E31 */
  0x0000,   /* U+1E32 */
  0x0000,   /* U+1E33 */
  0x0000,   /* U+1E34 */
  0x0000,   /* U+1E35 */
  0x0000,   /* U+1E36 */
  0x0000,   /* U+1E37 */
  0x0032,   /* U+1E38 */
  0x0032,   /* U+1E39 */
  0x0000,   /* U+1E3A */
  0x0000,   /* U+1E3B */
  0x0000,   /* U+1E3C */
  0x0000,   /* U+1E3D */
  0x0000,   /* U+1E3E */
  0x0000,   /* U+1E3F */
  0x0000,   /* U+1E40 */
  0x0000,   /* U+1E41 */
  0x0000,   /* U+1E42 */
  0x0000,   /* U+1E43 */
  0x0000,   /* U+1E44 */
  0x0000,   /* U+1E45 */
  0x0000,   /* U+1E46 */
  0x0000,   /* U+1E47 */
  0x0000,   /* U+1E48 */
  0x0000,   /* U+1E49 */
  0x0000,   /* U+1E4A */
  0x0000,   /* U+1E4B */
  0x0024,   /* U+1E4C */
  0x0024,   /* U+1E4D */
  0x002B,   /* U+1E4E */
  0x002B,   /* U+1E4F */
  0x0025,   /* U+1E50 */
  0x0025,   /* U+1E51 */
  0x0024,   /* U+1E52 */
  0x0024,   /* U+1E53 */
  0x0000,   /* U+1E54 */
  0x0000,   /* U+1E55 */
  0x0000,   /* U+1E56 */
  0x0000,   /* U+1E57 */
  0x0000,   /* U+1E58 */
  0x0000,   /* U+1E59 */
  0x0000,   /* U+1E5A */
  0x0000,   /* U+1E5B */
  0x0032,   /* U+1E5C */
  0x0032,   /* U+1E5D */
  0x0000,   /* U+1E5E */
  0x0000,   /* U+1E5F */
  0x0000,   /* U+1E60 */
  0x0000,   /* U+1E61 */
  0x0000,   /* U+1E62 */
  0x0000,   /* U+1E63 */
  0x002E,   /* U+1E64 */
  0x002E,   /* U+1E65 */
  0x002E,   /* U+1E66 */
  0x002E,   /* U+1E67 */
  0x002E,   /* U+1E68 */
  0x002E,   /* U+1E69 */
  0x0000,   /* U+1E6A */
  0x0000,   /* U+1E6B */
  0x0000,   /* U+1E6C */
  0x0000,   /* U+1E6D */
  0x0000,   /* U+1E6E */
  0x0000,   /* U+1E6F */
  0x0000,   /* U+1E70 */
  0x0000,   /* U+1E71 */
  0x0000,   /* U+1E72 */
  0x0000,   /* U+1E73 */
  0x0000,   /* U+1E74 */
  0x0000,   /* U+1E75 */
  0x0000,   /* U+1E76 */
  0x0000,   /* U+1E77 */
  0x0024,   /* U+1E78 */
  0x0024,   /* U+1E79 */
  0x002B,   /* U+1E7A */
  0x002B,   /* U+1E7B */
  0x0000,   /* U+1E7C */
  0x0000,   /* U+1E7D */
  0x0000,   /* U+1E7E */
  0x0000,   /* U+1E7F */
  0x0000,   /* U+1E80 */
  0x0000,   /* U+1E81 */
  0x0000,   /* U+1E82 */
  0x0000,   /* U+1E83 */
  0x0000,   /* U+1E84 */
  0x0000,   /* U+1E85 */
  0x0000,   /* U+1E86 */
  0x0000,   /* U+1E87 */
  0x0000,   /* U+1E88 */
  0x0000,   /* U+1E89 */
  0x0000,   /* U+1E8A */
  0x0000,   /* U+1E8B */
  0x0000,   /* U+1E8C */
  0x0000,   /* U+1E8D */
  0x0000,   /* U+1E8E */
  0x0000,   /* U+1E8F */
  0x0000,   /* U+1E90 */
  0x0000,   /* U+1E91 */
  0x0000,   /* U+1E92 */
  0x0000,   /* U+1E93 */
  0x0000,   /* U+1E94 */
  0x0000,   /* U+1E95 */
  0x0000,   /* U+1E96 */
  0x0000,   /* U+1E97 */
  0x0000,   /* U+1E98 */
  0x0000,   /* U+1E99 */
  0x0000,   /* U+1E9A */
  0x002E,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x0020,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0000,   /* U+1EA0 */
  0x0000,   /* U+1EA1 */
  0x0000,   /* U+1EA2 */
  0x0000,   /* U+1EA3 */
  0x0024,   /* U+1EA4 */
  0x0024,   /* U+1EA5 */
  0x0025,   /* U+1EA6 */
  0x0025,   /* U+1EA7 */
  0x003B,   /* U+1EA8 */
  0x003B,   /* U+1EA9 */
  0x002D,   /* U+1EAA */
  0x002D,   /* U+1EAB */
  0x0027,   /* U+1EAC */
  0x0027,   /* U+1EAD */
  0x0024,   /* U+1EAE */
  0x0024,   /* U+1EAF */
  0x0025,   /* U+1EB0 */
  0x0025,   /* U+1EB1 */
  0x003B,   /* U+1EB2 */
  0x003B,   /* U+1EB3 */
  0x002D,   /* U+1EB4 */
  0x002D,   /* U+1EB5 */
  0x0026,   /* U+1EB6 */
  0x0026,   /* U+1EB7 */
  0x0000,   /* U+1EB8 */
  0x0000,   /* U+1EB9 */
  0x0000,   /* U+1EBA */
  0x0000,   /* U+1EBB */
  0x0000,   /* U+1EBC */
  0x0000,   /* U+1EBD */
  0x0024,   /* U+1EBE */
  0x0024,   /* U+1EBF */
  0x0025,   /* U+1EC0 */
  0x0025,   /* U+1EC1 */
  0x003B,   /* U+1EC2 */
  0x003B,   /* U+1EC3 */
  0x002D,   /* U+1EC4 */
  0x002D,   /* U+1EC5 */
  0x0027,   /* U+1EC6 */
  0x0027,   /* U+1EC7 */
  0x0000,   /* U+1EC8 */
  0x0000,   /* U+1EC9 */
  0x0000,   /* U+1ECA */
  0x0000,   /* U+1ECB */
  0x0000,   /* U+1ECC */
  0x0000,   /* U+1ECD */
  0x0000,   /* U+1ECE */
  0x0000,   /* U+1ECF */
  0x0024,   /* U+1ED0 */
  0x0024,   /* U+1ED1 */
  0x0025,   /* U+1ED2 */
  0x0025,   /* U+1ED3 */
  0x003B,   /* U+1ED4 */
  0x003B,   /* U+1ED5 */
  0x002D,   /* U+1ED6 */
  0x002D,   /* U+1ED7 */
  0x0027,   /* U+1ED8 */
  0x0027,   /* U+1ED9 */
  0x0024,   /* U+1EDA */
  0x0024,   /* U+1EDB */
  0x0025,   /* U+1EDC */
  0x0025,   /* U+1EDD */
  0x003B,   /* U+1EDE */
  0x003B,   /* U+1EDF */
  0x002D,   /* U+1EE0 */
  0x002D,   /* U+1EE1 */
  0x0042,   /* U+1EE2 */
  0x0042,   /* U+1EE3 */
  0x0000,   /* U+1EE4 */
  0x0000,   /* U+1EE5 */
  0x0000,   /* U+1EE6 */
  0x0000,   /* U+1EE7 */
  0x0024,   /* U+1EE8 */
  0x0024,   /* U+1EE9 */
  0x0025,   /* U+1EEA */
  0x0025,   /* U+1EEB */
  0x003B,   /* U+1EEC */
  0x003B,   /* U+1EED */
  0x002D,   /* U+1EEE */
  0x002D,   /* U+1EEF */
  0x0042,   /* U+1EF0 */
  0x0042,   /* U+1EF1 */
  0x0000,   /* U+1EF2 */
  0x0000,   /* U+1EF3 */
  0x0000,   /* U+1EF4 */
  0x0000,   /* U+1EF5 */
  0x0000,   /* U+1EF6 */
  0x0000,   /* U+1EF7 */
  0x0000,   /* U+1EF8 */
  0x0000,   /* U+1EF9 */
  0x0000,   /* U+1EFA */
  0x0000,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1E00 */
  0x0000,   /* U+1E01 */
  0x0000,   /* U+1E02 */
  0x0000,   /* U+1E03 */
  0x0000,   /* U+1E04 */
  0x0000,   /* U+1E05 */
  0x0000,   /* U+1E06 */
  0x0000,   /* U+1E07 */
  0x0002,   /* U+1E08 */
  0x0002,   /* U+1E09 */
  0x0000,   /* U+1E0A */
  0x0000,   /* U+1E0B */
  0x0000,   /* U+1E0C */
  0x0000,   /* U+1E0D */
  0x0000,   /* U+1E0E */
  0x0000,   /* U+1E0F */
  0x0000,   /* U+1E10 */
  0x0000,   /* U+1E11 */
  0x0000,   /* U+1E12 */
  0x0000,   /* U+1E13 */
  0x0002,   /* U+1E14 */
  0x0002,   /* U+1E15 */
  0x0002,   /* U+1E16 */
  0x0002,   /* U+1E17 */
  0x0000,   /* U+1E18 */
  0x0000,   /* U+1E19 */
  0x0000,   /* U+1E1A */
  0x0000,   /* U+1E1B */
  0x0002,   /* U+1E1C */
  0x0002,   /* U+1E1D */
  0x0000,   /* U+1E1E */
  0x0000,   /* U+1E1F */
  0x0000,   /* U+1E20 */
  0x0000,   /* U+1E21 */
  0x0000,   /* U+1E22 */
  0x0000,   /* U+1E23 */
  0x0000,   /* U+1E24 */
  0x0000,   /* U+1E25 */
  0x0000,   /* U+1E26 */
  0x0000,   /* U+1E27 */
  0x0000,   /* U+1E28 */
  0x0000,   /* U+1E29 */
  0x0000,   /* U+1E2A */
  0x0000,   /* U+1E2B */
  0x0000,   /* U+1E2C */
  0x0000,   /* U+1E2D */
  0x0002,   /* U+1E2E */
  0x0002,   /* U+1E2F */
  0x0000,   /* U+1E30 */
  0x0000,   /* U+1E31 */
  0x0000,   /* U+1E32 */
  0x0000,   /* U+1E33 */
  0x0000,   /* U+1E34 */
  0x0000,   /* U+1E35 */
  0x0000,   /* U+1E36 */
  0x0000,   /* U+1E37 */
  0x0002,   /* U+1E38 */
  0x0002,   /* U+1E39 */
  0x0000,   /* U+1E3A */
  0x0000,   /* U+1E3B */
  0x0000,   /* U+1E3C */
  0x0000,   /* U+1E3D */
  0x0000,   /* U+1E3E */
  0x0000,   /* U+1E3F */
  0x0000,   /* U+1E40 */
  0x0000,   /* U+1E41 */
  0x0000,   /* U+1E42 */
  0x0000,   /* U+1E43 */
  0x0000,   /* U+1E44 */
  0x0000,   /* U+1E45 */
  0x0000,   /* U+1E46 */
  0x0000,   /* U+1E47 */
  0x0000,   /* U+1E48 */
  0x0000,   /* U+1E49 */
  0x0000,   /* U+1E4A */
  0x0000,   /* U+1E4B */
  0x0002,   /* U+1E4C */
  0x0002,   /* U+1E4D */
  0x0002,   /* U+1E4E */
  0x0002,   /* U+1E4F */
  0x0002,   /* U+1E50 */
  0x0002,   /* U+1E51 */
  0x0002,   /* U+1E52 */
  0x0002,   /* U+1E53 */
  0x0000,   /* U+1E54 */
  0x0000,   /* U+1E55 */
  0x0000,   /* U+1E56 */
  0x0000,   /* U+1E57 */
  0x0000,   /* U+1E58 */
  0x0000,   /* U+1E59 */
  0x0000,   /* U+1E5A */
  0x0000,   /* U+1E5B */
  0x0002,   /* U+1E5C */
  0x0002,   /* U+1E5D */
  0x0000,   /* U+1E5E */
  0x0000,   /* U+1E5F */
  0x0000,   /* U+1E60 */
  0x0000,   /* U+1E61 */
  0x0000,   /* U+1E62 */
  0x0000,   /* U+1E63 */
  0x0002,   /* U+1E64 */
  0x0002,   /* U+1E65 */
  0x0002,   /* U+1E66 */
  0x0002,   /* U+1E67 */
  0x0002,   /* U+1E68 */
  0x0002,   /* U+1E69 */
  0x0000,   /* U+1E6A */
  0x0000,   /* U+1E6B */
  0x0000,   /* U+1E6C */
  0x0000,   /* U+1E6D */
  0x0000,   /* U+1E6E */
  0x0000,   /* U+1E6F */
  0x0000,   /* U+1E70 */
  0x0000,   /* U+1E71 */
  0x0000,   /* U+1E72 */
  0x0000,   /* U+1E73 */
  0x0000,   /* U+1E74 */
  0x0000,   /* U+1E75 */
  0x0000,   /* U+1E76 */
  0x0000,   /* U+1E77 */
  0x0002,   /* U+1E78 */
  0x0002,   /* U+1E79 */
  0x0002,   /* U+1E7A */
  0x0002,   /* U+1E7B */
  0x0000,   /* U+1E7C */
  0x0000,   /* U+1E7D */
  0x0000,   /* U+1E7E */
  0x0000,   /* U+1E7F */
  0x0000,   /* U+1E80 */
  0x0000,   /* U+1E81 */
  0x0000,   /* U+1E82 */
  0x0000,   /* U+1E83 */
  0x0000,   /* U+1E84 */
  0x0000,   /* U+1E85 */
  0x0000,   /* U+1E86 */
  0x0000,   /* U+1E87 */
  0x0000,   /* U+1E88 */
  0x0000,   /* U+1E89 */
  0x0000,   /* U+1E8A */
  0x0000,   /* U+1E8B */
  0x0000,   /* U+1E8C */
  0x0000,   /* U+1E8D */
  0x0000,   /* U+1E8E */
  0x0000,   /* U+1E8F */
  0x0000,   /* U+1E90 */
  0x0000,   /* U+1E91 */
  0x0000,   /* U+1E92 */
  0x0000,   /* U+1E93 */
  0x0000,   /* U+1E94 */
  0x0000,   /* U+1E95 */
  0x0000,   /* U+1E96 */
  0x0000,   /* U+1E97 */
  0x0000,   /* U+1E98 */
  0x0000,   /* U+1E99 */
  0x0000,   /* U+1E9A */
  0x0002,   /* U+1E9B */
  0x0000,   /* U+1E9C */
  0x0000,   /* U+1E9D */
  0x000A,   /* U+1E9E */
  0x0000,   /* U+1E9F */
  0x0000,   /* U+1EA0 */
  0x0000,   /* U+1EA1 */
  0x0000,   /* U+1EA2 */
  0x0000,   /* U+1EA3 */
  0x0002,   /* U+1EA4 */
  0x0002,   /* U+1EA5 */
  0x0002,   /* U+1EA6 */
  0x0002,   /* U+1EA7 */
  0x0002,   /* U+1EA8 */
  0x0002,   /* U+1EA9 */
  0x0002,   /* U+1EAA */
  0x0002,   /* U+1EAB */
  0x0002,   /* U+1EAC */
  0x0002,   /* U+1EAD */
  0x0002,   /* U+1EAE */
  0x0002,   /* U+1EAF */
  0x0002,   /* U+1EB0 */
  0x0002,   /* U+1EB1 */
  0x0002,   /* U+1EB2 */
  0x0002,   /* U+1EB3 */
  0x0002,   /* U+1EB4 */
  0x0002,   /* U+1EB5 */
  0x0002,   /* U+1EB6 */
  0x0002,   /* U+1EB7 */
  0x0000,   /* U+1EB8 */
  0x0000,   /* U+1EB9 */
  0x0000,   /* U+1EBA */
  0x0000,   /* U+1EBB */
  0x0000,   /* U+1EBC */
  0x0000,   /* U+1EBD */
  0x0002,   /* U+1EBE */
  0x0002,   /* U+1EBF */
  0x0002,   /* U+1EC0 */
  0x0002,   /* U+1EC1 */
  0x0002,   /* U+1EC2 */
  0x0002,   /* U+1EC3 */
  0x0002,   /* U+1EC4 */
  0x0002,   /* U+1EC5 */
  0x0002,   /* U+1EC6 */
  0x0002,   /* U+1EC7 */
  0x0000,   /* U+1EC8 */
  0x0000,   /* U+1EC9 */
  0x0000,   /* U+1ECA */
  0x0000,   /* U+1ECB */
  0x0000,   /* U+1ECC */
  0x0000,   /* U+1ECD */
  0x0000,   /* U+1ECE */
  0x0000,   /* U+1ECF */
  0x0002,   /* U+1ED0 */
  0x0002,   /* U+1ED1 */
  0x0002,   /* U+1ED2 */
  0x0002,   /* U+1ED3 */
  0x0002,   /* U+1ED4 */
  0x0002,   /* U+1ED5 */
  0x0002,   /* U+1ED6 */
  0x0002,   /* U+1ED7 */
  0x0002,   /* U+1ED8 */
  0x0002,   /* U+1ED9 */
  0x0002,   /* U+1EDA */
  0x0002,   /* U+1EDB */
  0x0002,   /* U+1EDC */
  0x0002,   /* U+1EDD */
  0x0002,   /* U+1EDE */
  0x0002,   /* U+1EDF */
  0x0002,   /* U+1EE0 */
  0x0002,   /* U+1EE1 */
  0x0002,   /* U+1EE2 */
  0x0002,   /* U+1EE3 */
  0x0000,   /* U+1EE4 */
  0x0000,   /* U+1EE5 */
  0x0000,   /* U+1EE6 */
  0x0000,   /* U+1EE7 */
  0x0002,   /* U+1EE8 */
  0x0002,   /* U+1EE9 */
  0x0002,   /* U+1EEA */
  0x0002,   /* U+1EEB */
  0x0002,   /* U+1EEC */
  0x0002,   /* U+1EED */
  0x0002,   /* U+1EEE */
  0x0002,   /* U+1EEF */
  0x0002,   /* U+1EF0 */
  0x0002,   /* U+1EF1 */
  0x0000,   /* U+1EF2 */
  0x0000,   /* U+1EF3 */
  0x0000,   /* U+1EF4 */
  0x0000,   /* U+1EF5 */
  0x0000,   /* U+1EF6 */
  0x0000,   /* U+1EF7 */
  0x0000,   /* U+1EF8 */
  0x0000,   /* U+1EF9 */
  0x0000,   /* U+1EFA */
  0x0000,   /* U+1EFB */
  0x0000,   /* U+1EFC */
  0x0000,   /* U+1EFD */
  0x0000,   /* U+1EFE */
  0x0000,   /* U+1EFF */
};

uint16 uca900_p01F[]= {
  /* Number of CEs for each character. */
  2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 3, 3, 3, 3, 3, 3, 
  2, 2, 3, 3, 3, 3, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 
  2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 3, 3, 3, 3, 3, 3, 
  2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 3, 3, 3, 3, 3, 3, 
  2, 2, 3, 3, 3, 3, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 
  2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 2, 3, 2, 3, 2, 3, 
  2, 2, 3, 3, 3, 3, 3, 3, 2, 2, 3, 3, 3, 3, 3, 3, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 4, 4, 4, 4, 4, 4, 3, 3, 4, 4, 4, 4, 4, 4, 
  3, 3, 4, 4, 4, 4, 4, 4, 3, 3, 4, 4, 4, 4, 4, 4, 
  3, 3, 4, 4, 4, 4, 4, 4, 3, 3, 4, 4, 4, 4, 4, 4, 
  2, 2, 3, 2, 3, 2, 2, 3, 2, 2, 2, 2, 2, 1, 1, 1, 
  1, 2, 3, 2, 3, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 3, 3, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 3, 3, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 1, 
  2, 2, 3, 2, 3, 2, 2, 3, 2, 2, 2, 2, 2, 1, 1, 2, 

  /* Primary weight 1 for each character. */
  0x1FB9,   /* U+1F00 */
  0x1FB9,   /* U+1F01 */
  0x1FB9,   /* U+1F02 */
  0x1FB9,   /* U+1F03 */
  0x1FB9,   /* U+1F04 */
  0x1FB9,   /* U+1F05 */
  0x1FB9,   /* U+1F06 */
  0x1FB9,   /* U+1F07 */
  0x1FB9,   /* U+1F08 */
  0x1FB9,   /* U+1F09 */
  0x1FB9,   /* U+1F0A */
  0x1FB9,   /* U+1F0B */
  0x1FB9,   /* U+1F0C */
  0x1FB9,   /* U+1F0D */
  0x1FB9,   /* U+1F0E */
  0x1FB9,   /* U+1F0F */
  0x1FBE,   /* U+1F10 */
  0x1FBE,   /* U+1F11 */
  0x1FBE,   /* U+1F12 */
  0x1FBE,   /* U+1F13 */
  0x1FBE,   /* U+1F14 */
  0x1FBE,   /* U+1F15 */
  0xFBC0,   /* U+1F16 */
  0xFBC0,   /* U+1F17 */
  0x1FBE,   /* U+1F18 */
  0x1FBE,   /* U+1F19 */
  0x1FBE,   /* U+1F1A */
  0x1FBE,   /* U+1F1B */
  0x1FBE,   /* U+1F1C */
  0x1FBE,   /* U+1F1D */
  0xFBC0,   /* U+1F1E */
  0xFBC0,   /* U+1F1F */
  0x1FC4,   /* U+1F20 */
  0x1FC4,   /* U+1F21 */
  0x1FC4,   /* U+1F22 */
  0x1FC4,   /* U+1F23 */
  0x1FC4,   /* U+1F24 */
  0x1FC4,   /* U+1F25 */
  0x1FC4,   /* U+1F26 */
  0x1FC4,   /* U+1F27 */
  0x1FC4,   /* U+1F28 */
  0x1FC4,   /* U+1F29 */
  0x1FC4,   /* U+1F2A */
  0x1FC4,   /* U+1F2B */
  0x1FC4,   /* U+1F2C */
  0x1FC4,   /* U+1F2D */
  0x1FC4,   /* U+1F2E */
  0x1FC4,   /* U+1F2F */
  0x1FC6,   /* U+1F30 */
  0x1FC6,   /* U+1F31 */
  0x1FC6,   /* U+1F32 */
  0x1FC6,   /* U+1F33 */
  0x1FC6,   /* U+1F34 */
  0x1FC6,   /* U+1F35 */
  0x1FC6,   /* U+1F36 */
  0x1FC6,   /* U+1F37 */
  0x1FC6,   /* U+1F38 */
  0x1FC6,   /* U+1F39 */
  0x1FC6,   /* U+1F3A */
  0x1FC6,   /* U+1F3B */
  0x1FC6,   /* U+1F3C */
  0x1FC6,   /* U+1F3D */
  0x1FC6,   /* U+1F3E */
  0x1FC6,   /* U+1F3F */
  0x1FCE,   /* U+1F40 */
  0x1FCE,   /* U+1F41 */
  0x1FCE,   /* U+1F42 */
  0x1FCE,   /* U+1F43 */
  0x1FCE,   /* U+1F44 */
  0x1FCE,   /* U+1F45 */
  0xFBC0,   /* U+1F46 */
  0xFBC0,   /* U+1F47 */
  0x1FCE,   /* U+1F48 */
  0x1FCE,   /* U+1F49 */
  0x1FCE,   /* U+1F4A */
  0x1FCE,   /* U+1F4B */
  0x1FCE,   /* U+1F4C */
  0x1FCE,   /* U+1F4D */
  0xFBC0,   /* U+1F4E */
  0xFBC0,   /* U+1F4F */
  0x1FDC,   /* U+1F50 */
  0x1FDC,   /* U+1F51 */
  0x1FDC,   /* U+1F52 */
  0x1FDC,   /* U+1F53 */
  0x1FDC,   /* U+1F54 */
  0x1FDC,   /* U+1F55 */
  0x1FDC,   /* U+1F56 */
  0x1FDC,   /* U+1F57 */
  0xFBC0,   /* U+1F58 */
  0x1FDC,   /* U+1F59 */
  0xFBC0,   /* U+1F5A */
  0x1FDC,   /* U+1F5B */
  0xFBC0,   /* U+1F5C */
  0x1FDC,   /* U+1F5D */
  0xFBC0,   /* U+1F5E */
  0x1FDC,   /* U+1F5F */
  0x1FE1,   /* U+1F60 */
  0x1FE1,   /* U+1F61 */
  0x1FE1,   /* U+1F62 */
  0x1FE1,   /* U+1F63 */
  0x1FE1,   /* U+1F64 */
  0x1FE1,   /* U+1F65 */
  0x1FE1,   /* U+1F66 */
  0x1FE1,   /* U+1F67 */
  0x1FE1,   /* U+1F68 */
  0x1FE1,   /* U+1F69 */
  0x1FE1,   /* U+1F6A */
  0x1FE1,   /* U+1F6B */
  0x1FE1,   /* U+1F6C */
  0x1FE1,   /* U+1F6D */
  0x1FE1,   /* U+1F6E */
  0x1FE1,   /* U+1F6F */
  0x1FB9,   /* U+1F70 */
  0x1FB9,   /* U+1F71 */
  0x1FBE,   /* U+1F72 */
  0x1FBE,   /* U+1F73 */
  0x1FC4,   /* U+1F74 */
  0x1FC4,   /* U+1F75 */
  0x1FC6,   /* U+1F76 */
  0x1FC6,   /* U+1F77 */
  0x1FCE,   /* U+1F78 */
  0x1FCE,   /* U+1F79 */
  0x1FDC,   /* U+1F7A */
  0x1FDC,   /* U+1F7B */
  0x1FE1,   /* U+1F7C */
  0x1FE1,   /* U+1F7D */
  0xFBC0,   /* U+1F7E */
  0xFBC0,   /* U+1F7F */
  0x1FB9,   /* U+1F80 */
  0x1FB9,   /* U+1F81 */
  0x1FB9,   /* U+1F82 */
  0x1FB9,   /* U+1F83 */
  0x1FB9,   /* U+1F84 */
  0x1FB9,   /* U+1F85 */
  0x1FB9,   /* U+1F86 */
  0x1FB9,   /* U+1F87 */
  0x1FB9,   /* U+1F88 */
  0x1FB9,   /* U+1F89 */
  0x1FB9,   /* U+1F8A */
  0x1FB9,   /* U+1F8B */
  0x1FB9,   /* U+1F8C */
  0x1FB9,   /* U+1F8D */
  0x1FB9,   /* U+1F8E */
  0x1FB9,   /* U+1F8F */
  0x1FC4,   /* U+1F90 */
  0x1FC4,   /* U+1F91 */
  0x1FC4,   /* U+1F92 */
  0x1FC4,   /* U+1F93 */
  0x1FC4,   /* U+1F94 */
  0x1FC4,   /* U+1F95 */
  0x1FC4,   /* U+1F96 */
  0x1FC4,   /* U+1F97 */
  0x1FC4,   /* U+1F98 */
  0x1FC4,   /* U+1F99 */
  0x1FC4,   /* U+1F9A */
  0x1FC4,   /* U+1F9B */
  0x1FC4,   /* U+1F9C */
  0x1FC4,   /* U+1F9D */
  0x1FC4,   /* U+1F9E */
  0x1FC4,   /* U+1F9F */
  0x1FE1,   /* U+1FA0 */
  0x1FE1,   /* U+1FA1 */
  0x1FE1,   /* U+1FA2 */
  0x1FE1,   /* U+1FA3 */
  0x1FE1,   /* U+1FA4 */
  0x1FE1,   /* U+1FA5 */
  0x1FE1,   /* U+1FA6 */
  0x1FE1,   /* U+1FA7 */
  0x1FE1,   /* U+1FA8 */
  0x1FE1,   /* U+1FA9 */
  0x1FE1,   /* U+1FAA */
  0x1FE1,   /* U+1FAB */
  0x1FE1,   /* U+1FAC */
  0x1FE1,   /* U+1FAD */
  0x1FE1,   /* U+1FAE */
  0x1FE1,   /* U+1FAF */
  0x1FB9,   /* U+1FB0 */
  0x1FB9,   /* U+1FB1 */
  0x1FB9,   /* U+1FB2 */
  0x1FB9,   /* U+1FB3 */
  0x1FB9,   /* U+1FB4 */
  0xFBC0,   /* U+1FB5 */
  0x1FB9,   /* U+1FB6 */
  0x1FB9,   /* U+1FB7 */
  0x1FB9,   /* U+1FB8 */
  0x1FB9,   /* U+1FB9 */
  0x1FB9,   /* U+1FBA */
  0x1FB9,   /* U+1FBB */
  0x1FB9,   /* U+1FBC */
  0x048E,   /* U+1FBD */
  0x1FC6,   /* U+1FBE */
  0x048E,   /* U+1FBF */
  0x0490,   /* U+1FC0 */
  0x0489,   /* U+1FC1 */
  0x1FC4,   /* U+1FC2 */
  0x1FC4,   /* U+1FC3 */
  0x1FC4,   /* U+1FC4 */
  0xFBC0,   /* U+1FC5 */
  0x1FC4,   /* U+1FC6 */
  0x1FC4,   /* U+1FC7 */
  0x1FBE,   /* U+1FC8 */
  0x1FBE,   /* U+1FC9 */
  0x1FC4,   /* U+1FCA */
  0x1FC4,   /* U+1FCB */
  0x1FC4,   /* U+1FCC */
  0x048E,   /* U+1FCD */
  0x048E,   /* U+1FCE */
  0x048E,   /* U+1FCF */
  0x1FC6,   /* U+1FD0 */
  0x1FC6,   /* U+1FD1 */
  0x1FC6,   /* U+1FD2 */
  0x1FC6,   /* U+1FD3 */
  0xFBC0,   /* U+1FD4 */
  0xFBC0,   /* U+1FD5 */
  0x1FC6,   /* U+1FD6 */
  0x1FC6,   /* U+1FD7 */
  0x1FC6,   /* U+1FD8 */
  0x1FC6,   /* U+1FD9 */
  0x1FC6,   /* U+1FDA */
  0x1FC6,   /* U+1FDB */
  0xFBC0,   /* U+1FDC */
  0x048F,   /* U+1FDD */
  0x048F,   /* U+1FDE */
  0x048F,   /* U+1FDF */
  0x1FDC,   /* U+1FE0 */
  0x1FDC,   /* U+1FE1 */
  0x1FDC,   /* U+1FE2 */
  0x1FDC,   /* U+1FE3 */
  0x1FD4,   /* U+1FE4 */
  0x1FD4,   /* U+1FE5 */
  0x1FDC,   /* U+1FE6 */
  0x1FDC,   /* U+1FE7 */
  0x1FDC,   /* U+1FE8 */
  0x1FDC,   /* U+1FE9 */
  0x1FDC,   /* U+1FEA */
  0x1FDC,   /* U+1FEB */
  0x1FD4,   /* U+1FEC */
  0x0489,   /* U+1FED */
  0x0489,   /* U+1FEE */
  0x0482,   /* U+1FEF */
  0xFBC0,   /* U+1FF0 */
  0xFBC0,   /* U+1FF1 */
  0x1FE1,   /* U+1FF2 */
  0x1FE1,   /* U+1FF3 */
  0x1FE1,   /* U+1FF4 */
  0xFBC0,   /* U+1FF5 */
  0x1FE1,   /* U+1FF6 */
  0x1FE1,   /* U+1FF7 */
  0x1FCE,   /* U+1FF8 */
  0x1FCE,   /* U+1FF9 */
  0x1FE1,   /* U+1FFA */
  0x1FE1,   /* U+1FFB */
  0x1FE1,   /* U+1FFC */
  0x0483,   /* U+1FFD */
  0x048F,   /* U+1FFE */
  0xFBC0,   /* U+1FFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F00 */
  0x0020,   /* U+1F01 */
  0x0020,   /* U+1F02 */
  0x0020,   /* U+1F03 */
  0x0020,   /* U+1F04 */
  0x0020,   /* U+1F05 */
  0x0020,   /* U+1F06 */
  0x0020,   /* U+1F07 */
  0x0020,   /* U+1F08 */
  0x0020,   /* U+1F09 */
  0x0020,   /* U+1F0A */
  0x0020,   /* U+1F0B */
  0x0020,   /* U+1F0C */
  0x0020,   /* U+1F0D */
  0x0020,   /* U+1F0E */
  0x0020,   /* U+1F0F */
  0x0020,   /* U+1F10 */
  0x0020,   /* U+1F11 */
  0x0020,   /* U+1F12 */
  0x0020,   /* U+1F13 */
  0x0020,   /* U+1F14 */
  0x0020,   /* U+1F15 */
  0x0020,   /* U+1F16 */
  0x0020,   /* U+1F17 */
  0x0020,   /* U+1F18 */
  0x0020,   /* U+1F19 */
  0x0020,   /* U+1F1A */
  0x0020,   /* U+1F1B */
  0x0020,   /* U+1F1C */
  0x0020,   /* U+1F1D */
  0x0020,   /* U+1F1E */
  0x0020,   /* U+1F1F */
  0x0020,   /* U+1F20 */
  0x0020,   /* U+1F21 */
  0x0020,   /* U+1F22 */
  0x0020,   /* U+1F23 */
  0x0020,   /* U+1F24 */
  0x0020,   /* U+1F25 */
  0x0020,   /* U+1F26 */
  0x0020,   /* U+1F27 */
  0x0020,   /* U+1F28 */
  0x0020,   /* U+1F29 */
  0x0020,   /* U+1F2A */
  0x0020,   /* U+1F2B */
  0x0020,   /* U+1F2C */
  0x0020,   /* U+1F2D */
  0x0020,   /* U+1F2E */
  0x0020,   /* U+1F2F */
  0x0020,   /* U+1F30 */
  0x0020,   /* U+1F31 */
  0x0020,   /* U+1F32 */
  0x0020,   /* U+1F33 */
  0x0020,   /* U+1F34 */
  0x0020,   /* U+1F35 */
  0x0020,   /* U+1F36 */
  0x0020,   /* U+1F37 */
  0x0020,   /* U+1F38 */
  0x0020,   /* U+1F39 */
  0x0020,   /* U+1F3A */
  0x0020,   /* U+1F3B */
  0x0020,   /* U+1F3C */
  0x0020,   /* U+1F3D */
  0x0020,   /* U+1F3E */
  0x0020,   /* U+1F3F */
  0x0020,   /* U+1F40 */
  0x0020,   /* U+1F41 */
  0x0020,   /* U+1F42 */
  0x0020,   /* U+1F43 */
  0x0020,   /* U+1F44 */
  0x0020,   /* U+1F45 */
  0x0020,   /* U+1F46 */
  0x0020,   /* U+1F47 */
  0x0020,   /* U+1F48 */
  0x0020,   /* U+1F49 */
  0x0020,   /* U+1F4A */
  0x0020,   /* U+1F4B */
  0x0020,   /* U+1F4C */
  0x0020,   /* U+1F4D */
  0x0020,   /* U+1F4E */
  0x0020,   /* U+1F4F */
  0x0020,   /* U+1F50 */
  0x0020,   /* U+1F51 */
  0x0020,   /* U+1F52 */
  0x0020,   /* U+1F53 */
  0x0020,   /* U+1F54 */
  0x0020,   /* U+1F55 */
  0x0020,   /* U+1F56 */
  0x0020,   /* U+1F57 */
  0x0020,   /* U+1F58 */
  0x0020,   /* U+1F59 */
  0x0020,   /* U+1F5A */
  0x0020,   /* U+1F5B */
  0x0020,   /* U+1F5C */
  0x0020,   /* U+1F5D */
  0x0020,   /* U+1F5E */
  0x0020,   /* U+1F5F */
  0x0020,   /* U+1F60 */
  0x0020,   /* U+1F61 */
  0x0020,   /* U+1F62 */
  0x0020,   /* U+1F63 */
  0x0020,   /* U+1F64 */
  0x0020,   /* U+1F65 */
  0x0020,   /* U+1F66 */
  0x0020,   /* U+1F67 */
  0x0020,   /* U+1F68 */
  0x0020,   /* U+1F69 */
  0x0020,   /* U+1F6A */
  0x0020,   /* U+1F6B */
  0x0020,   /* U+1F6C */
  0x0020,   /* U+1F6D */
  0x0020,   /* U+1F6E */
  0x0020,   /* U+1F6F */
  0x0020,   /* U+1F70 */
  0x0020,   /* U+1F71 */
  0x0020,   /* U+1F72 */
  0x0020,   /* U+1F73 */
  0x0020,   /* U+1F74 */
  0x0020,   /* U+1F75 */
  0x0020,   /* U+1F76 */
  0x0020,   /* U+1F77 */
  0x0020,   /* U+1F78 */
  0x0020,   /* U+1F79 */
  0x0020,   /* U+1F7A */
  0x0020,   /* U+1F7B */
  0x0020,   /* U+1F7C */
  0x0020,   /* U+1F7D */
  0x0020,   /* U+1F7E */
  0x0020,   /* U+1F7F */
  0x0020,   /* U+1F80 */
  0x0020,   /* U+1F81 */
  0x0020,   /* U+1F82 */
  0x0020,   /* U+1F83 */
  0x0020,   /* U+1F84 */
  0x0020,   /* U+1F85 */
  0x0020,   /* U+1F86 */
  0x0020,   /* U+1F87 */
  0x0020,   /* U+1F88 */
  0x0020,   /* U+1F89 */
  0x0020,   /* U+1F8A */
  0x0020,   /* U+1F8B */
  0x0020,   /* U+1F8C */
  0x0020,   /* U+1F8D */
  0x0020,   /* U+1F8E */
  0x0020,   /* U+1F8F */
  0x0020,   /* U+1F90 */
  0x0020,   /* U+1F91 */
  0x0020,   /* U+1F92 */
  0x0020,   /* U+1F93 */
  0x0020,   /* U+1F94 */
  0x0020,   /* U+1F95 */
  0x0020,   /* U+1F96 */
  0x0020,   /* U+1F97 */
  0x0020,   /* U+1F98 */
  0x0020,   /* U+1F99 */
  0x0020,   /* U+1F9A */
  0x0020,   /* U+1F9B */
  0x0020,   /* U+1F9C */
  0x0020,   /* U+1F9D */
  0x0020,   /* U+1F9E */
  0x0020,   /* U+1F9F */
  0x0020,   /* U+1FA0 */
  0x0020,   /* U+1FA1 */
  0x0020,   /* U+1FA2 */
  0x0020,   /* U+1FA3 */
  0x0020,   /* U+1FA4 */
  0x0020,   /* U+1FA5 */
  0x0020,   /* U+1FA6 */
  0x0020,   /* U+1FA7 */
  0x0020,   /* U+1FA8 */
  0x0020,   /* U+1FA9 */
  0x0020,   /* U+1FAA */
  0x0020,   /* U+1FAB */
  0x0020,   /* U+1FAC */
  0x0020,   /* U+1FAD */
  0x0020,   /* U+1FAE */
  0x0020,   /* U+1FAF */
  0x0020,   /* U+1FB0 */
  0x0020,   /* U+1FB1 */
  0x0020,   /* U+1FB2 */
  0x0020,   /* U+1FB3 */
  0x0020,   /* U+1FB4 */
  0x0020,   /* U+1FB5 */
  0x0020,   /* U+1FB6 */
  0x0020,   /* U+1FB7 */
  0x0020,   /* U+1FB8 */
  0x0020,   /* U+1FB9 */
  0x0020,   /* U+1FBA */
  0x0020,   /* U+1FBB */
  0x0020,   /* U+1FBC */
  0x0020,   /* U+1FBD */
  0x0020,   /* U+1FBE */
  0x0020,   /* U+1FBF */
  0x0020,   /* U+1FC0 */
  0x0020,   /* U+1FC1 */
  0x0020,   /* U+1FC2 */
  0x0020,   /* U+1FC3 */
  0x0020,   /* U+1FC4 */
  0x0020,   /* U+1FC5 */
  0x0020,   /* U+1FC6 */
  0x0020,   /* U+1FC7 */
  0x0020,   /* U+1FC8 */
  0x0020,   /* U+1FC9 */
  0x0020,   /* U+1FCA */
  0x0020,   /* U+1FCB */
  0x0020,   /* U+1FCC */
  0x0020,   /* U+1FCD */
  0x0020,   /* U+1FCE */
  0x0020,   /* U+1FCF */
  0x0020,   /* U+1FD0 */
  0x0020,   /* U+1FD1 */
  0x0020,   /* U+1FD2 */
  0x0020,   /* U+1FD3 */
  0x0020,   /* U+1FD4 */
  0x0020,   /* U+1FD5 */
  0x0020,   /* U+1FD6 */
  0x0020,   /* U+1FD7 */
  0x0020,   /* U+1FD8 */
  0x0020,   /* U+1FD9 */
  0x0020,   /* U+1FDA */
  0x0020,   /* U+1FDB */
  0x0020,   /* U+1FDC */
  0x0020,   /* U+1FDD */
  0x0020,   /* U+1FDE */
  0x0020,   /* U+1FDF */
  0x0020,   /* U+1FE0 */
  0x0020,   /* U+1FE1 */
  0x0020,   /* U+1FE2 */
  0x0020,   /* U+1FE3 */
  0x0020,   /* U+1FE4 */
  0x0020,   /* U+1FE5 */
  0x0020,   /* U+1FE6 */
  0x0020,   /* U+1FE7 */
  0x0020,   /* U+1FE8 */
  0x0020,   /* U+1FE9 */
  0x0020,   /* U+1FEA */
  0x0020,   /* U+1FEB */
  0x0020,   /* U+1FEC */
  0x0020,   /* U+1FED */
  0x0020,   /* U+1FEE */
  0x0020,   /* U+1FEF */
  0x0020,   /* U+1FF0 */
  0x0020,   /* U+1FF1 */
  0x0020,   /* U+1FF2 */
  0x0020,   /* U+1FF3 */
  0x0020,   /* U+1FF4 */
  0x0020,   /* U+1FF5 */
  0x0020,   /* U+1FF6 */
  0x0020,   /* U+1FF7 */
  0x0020,   /* U+1FF8 */
  0x0020,   /* U+1FF9 */
  0x0020,   /* U+1FFA */
  0x0020,   /* U+1FFB */
  0x0020,   /* U+1FFC */
  0x0020,   /* U+1FFD */
  0x0020,   /* U+1FFE */
  0x0020,   /* U+1FFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F00 */
  0x0002,   /* U+1F01 */
  0x0002,   /* U+1F02 */
  0x0002,   /* U+1F03 */
  0x0002,   /* U+1F04 */
  0x0002,   /* U+1F05 */
  0x0002,   /* U+1F06 */
  0x0002,   /* U+1F07 */
  0x0008,   /* U+1F08 */
  0x0008,   /* U+1F09 */
  0x0008,   /* U+1F0A */
  0x0008,   /* U+1F0B */
  0x0008,   /* U+1F0C */
  0x0008,   /* U+1F0D */
  0x0008,   /* U+1F0E */
  0x0008,   /* U+1F0F */
  0x0002,   /* U+1F10 */
  0x0002,   /* U+1F11 */
  0x0002,   /* U+1F12 */
  0x0002,   /* U+1F13 */
  0x0002,   /* U+1F14 */
  0x0002,   /* U+1F15 */
  0x0002,   /* U+1F16 */
  0x0002,   /* U+1F17 */
  0x0008,   /* U+1F18 */
  0x0008,   /* U+1F19 */
  0x0008,   /* U+1F1A */
  0x0008,   /* U+1F1B */
  0x0008,   /* U+1F1C */
  0x0008,   /* U+1F1D */
  0x0002,   /* U+1F1E */
  0x0002,   /* U+1F1F */
  0x0002,   /* U+1F20 */
  0x0002,   /* U+1F21 */
  0x0002,   /* U+1F22 */
  0x0002,   /* U+1F23 */
  0x0002,   /* U+1F24 */
  0x0002,   /* U+1F25 */
  0x0002,   /* U+1F26 */
  0x0002,   /* U+1F27 */
  0x0008,   /* U+1F28 */
  0x0008,   /* U+1F29 */
  0x0008,   /* U+1F2A */
  0x0008,   /* U+1F2B */
  0x0008,   /* U+1F2C */
  0x0008,   /* U+1F2D */
  0x0008,   /* U+1F2E */
  0x0008,   /* U+1F2F */
  0x0002,   /* U+1F30 */
  0x0002,   /* U+1F31 */
  0x0002,   /* U+1F32 */
  0x0002,   /* U+1F33 */
  0x0002,   /* U+1F34 */
  0x0002,   /* U+1F35 */
  0x0002,   /* U+1F36 */
  0x0002,   /* U+1F37 */
  0x0008,   /* U+1F38 */
  0x0008,   /* U+1F39 */
  0x0008,   /* U+1F3A */
  0x0008,   /* U+1F3B */
  0x0008,   /* U+1F3C */
  0x0008,   /* U+1F3D */
  0x0008,   /* U+1F3E */
  0x0008,   /* U+1F3F */
  0x0002,   /* U+1F40 */
  0x0002,   /* U+1F41 */
  0x0002,   /* U+1F42 */
  0x0002,   /* U+1F43 */
  0x0002,   /* U+1F44 */
  0x0002,   /* U+1F45 */
  0x0002,   /* U+1F46 */
  0x0002,   /* U+1F47 */
  0x0008,   /* U+1F48 */
  0x0008,   /* U+1F49 */
  0x0008,   /* U+1F4A */
  0x0008,   /* U+1F4B */
  0x0008,   /* U+1F4C */
  0x0008,   /* U+1F4D */
  0x0002,   /* U+1F4E */
  0x0002,   /* U+1F4F */
  0x0002,   /* U+1F50 */
  0x0002,   /* U+1F51 */
  0x0002,   /* U+1F52 */
  0x0002,   /* U+1F53 */
  0x0002,   /* U+1F54 */
  0x0002,   /* U+1F55 */
  0x0002,   /* U+1F56 */
  0x0002,   /* U+1F57 */
  0x0002,   /* U+1F58 */
  0x0008,   /* U+1F59 */
  0x0002,   /* U+1F5A */
  0x0008,   /* U+1F5B */
  0x0002,   /* U+1F5C */
  0x0008,   /* U+1F5D */
  0x0002,   /* U+1F5E */
  0x0008,   /* U+1F5F */
  0x0002,   /* U+1F60 */
  0x0002,   /* U+1F61 */
  0x0002,   /* U+1F62 */
  0x0002,   /* U+1F63 */
  0x0002,   /* U+1F64 */
  0x0002,   /* U+1F65 */
  0x0002,   /* U+1F66 */
  0x0002,   /* U+1F67 */
  0x0008,   /* U+1F68 */
  0x0008,   /* U+1F69 */
  0x0008,   /* U+1F6A */
  0x0008,   /* U+1F6B */
  0x0008,   /* U+1F6C */
  0x0008,   /* U+1F6D */
  0x0008,   /* U+1F6E */
  0x0008,   /* U+1F6F */
  0x0002,   /* U+1F70 */
  0x0002,   /* U+1F71 */
  0x0002,   /* U+1F72 */
  0x0002,   /* U+1F73 */
  0x0002,   /* U+1F74 */
  0x0002,   /* U+1F75 */
  0x0002,   /* U+1F76 */
  0x0002,   /* U+1F77 */
  0x0002,   /* U+1F78 */
  0x0002,   /* U+1F79 */
  0x0002,   /* U+1F7A */
  0x0002,   /* U+1F7B */
  0x0002,   /* U+1F7C */
  0x0002,   /* U+1F7D */
  0x0002,   /* U+1F7E */
  0x0002,   /* U+1F7F */
  0x0002,   /* U+1F80 */
  0x0002,   /* U+1F81 */
  0x0002,   /* U+1F82 */
  0x0002,   /* U+1F83 */
  0x0002,   /* U+1F84 */
  0x0002,   /* U+1F85 */
  0x0002,   /* U+1F86 */
  0x0002,   /* U+1F87 */
  0x0008,   /* U+1F88 */
  0x0008,   /* U+1F89 */
  0x0008,   /* U+1F8A */
  0x0008,   /* U+1F8B */
  0x0008,   /* U+1F8C */
  0x0008,   /* U+1F8D */
  0x0008,   /* U+1F8E */
  0x0008,   /* U+1F8F */
  0x0002,   /* U+1F90 */
  0x0002,   /* U+1F91 */
  0x0002,   /* U+1F92 */
  0x0002,   /* U+1F93 */
  0x0002,   /* U+1F94 */
  0x0002,   /* U+1F95 */
  0x0002,   /* U+1F96 */
  0x0002,   /* U+1F97 */
  0x0008,   /* U+1F98 */
  0x0008,   /* U+1F99 */
  0x0008,   /* U+1F9A */
  0x0008,   /* U+1F9B */
  0x0008,   /* U+1F9C */
  0x0008,   /* U+1F9D */
  0x0008,   /* U+1F9E */
  0x0008,   /* U+1F9F */
  0x0002,   /* U+1FA0 */
  0x0002,   /* U+1FA1 */
  0x0002,   /* U+1FA2 */
  0x0002,   /* U+1FA3 */
  0x0002,   /* U+1FA4 */
  0x0002,   /* U+1FA5 */
  0x0002,   /* U+1FA6 */
  0x0002,   /* U+1FA7 */
  0x0008,   /* U+1FA8 */
  0x0008,   /* U+1FA9 */
  0x0008,   /* U+1FAA */
  0x0008,   /* U+1FAB */
  0x0008,   /* U+1FAC */
  0x0008,   /* U+1FAD */
  0x0008,   /* U+1FAE */
  0x0008,   /* U+1FAF */
  0x0002,   /* U+1FB0 */
  0x0002,   /* U+1FB1 */
  0x0002,   /* U+1FB2 */
  0x0002,   /* U+1FB3 */
  0x0002,   /* U+1FB4 */
  0x0002,   /* U+1FB5 */
  0x0002,   /* U+1FB6 */
  0x0002,   /* U+1FB7 */
  0x0008,   /* U+1FB8 */
  0x0008,   /* U+1FB9 */
  0x0008,   /* U+1FBA */
  0x0008,   /* U+1FBB */
  0x0008,   /* U+1FBC */
  0x0002,   /* U+1FBD */
  0x0002,   /* U+1FBE */
  0x0002,   /* U+1FBF */
  0x0002,   /* U+1FC0 */
  0x0002,   /* U+1FC1 */
  0x0002,   /* U+1FC2 */
  0x0002,   /* U+1FC3 */
  0x0002,   /* U+1FC4 */
  0x0002,   /* U+1FC5 */
  0x0002,   /* U+1FC6 */
  0x0002,   /* U+1FC7 */
  0x0008,   /* U+1FC8 */
  0x0008,   /* U+1FC9 */
  0x0008,   /* U+1FCA */
  0x0008,   /* U+1FCB */
  0x0008,   /* U+1FCC */
  0x0002,   /* U+1FCD */
  0x0002,   /* U+1FCE */
  0x0002,   /* U+1FCF */
  0x0002,   /* U+1FD0 */
  0x0002,   /* U+1FD1 */
  0x0002,   /* U+1FD2 */
  0x0002,   /* U+1FD3 */
  0x0002,   /* U+1FD4 */
  0x0002,   /* U+1FD5 */
  0x0002,   /* U+1FD6 */
  0x0002,   /* U+1FD7 */
  0x0008,   /* U+1FD8 */
  0x0008,   /* U+1FD9 */
  0x0008,   /* U+1FDA */
  0x0008,   /* U+1FDB */
  0x0002,   /* U+1FDC */
  0x0002,   /* U+1FDD */
  0x0002,   /* U+1FDE */
  0x0002,   /* U+1FDF */
  0x0002,   /* U+1FE0 */
  0x0002,   /* U+1FE1 */
  0x0002,   /* U+1FE2 */
  0x0002,   /* U+1FE3 */
  0x0002,   /* U+1FE4 */
  0x0002,   /* U+1FE5 */
  0x0002,   /* U+1FE6 */
  0x0002,   /* U+1FE7 */
  0x0008,   /* U+1FE8 */
  0x0008,   /* U+1FE9 */
  0x0008,   /* U+1FEA */
  0x0008,   /* U+1FEB */
  0x0008,   /* U+1FEC */
  0x0002,   /* U+1FED */
  0x0002,   /* U+1FEE */
  0x0002,   /* U+1FEF */
  0x0002,   /* U+1FF0 */
  0x0002,   /* U+1FF1 */
  0x0002,   /* U+1FF2 */
  0x0002,   /* U+1FF3 */
  0x0002,   /* U+1FF4 */
  0x0002,   /* U+1FF5 */
  0x0002,   /* U+1FF6 */
  0x0002,   /* U+1FF7 */
  0x0008,   /* U+1FF8 */
  0x0008,   /* U+1FF9 */
  0x0008,   /* U+1FFA */
  0x0008,   /* U+1FFB */
  0x0008,   /* U+1FFC */
  0x0002,   /* U+1FFD */
  0x0002,   /* U+1FFE */
  0x0002,   /* U+1FFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0000,   /* U+1F02 */
  0x0000,   /* U+1F03 */
  0x0000,   /* U+1F04 */
  0x0000,   /* U+1F05 */
  0x0000,   /* U+1F06 */
  0x0000,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0000,   /* U+1F0A */
  0x0000,   /* U+1F0B */
  0x0000,   /* U+1F0C */
  0x0000,   /* U+1F0D */
  0x0000,   /* U+1F0E */
  0x0000,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0000,   /* U+1F12 */
  0x0000,   /* U+1F13 */
  0x0000,   /* U+1F14 */
  0x0000,   /* U+1F15 */
  0x9F16,   /* U+1F16 */
  0x9F17,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0000,   /* U+1F1A */
  0x0000,   /* U+1F1B */
  0x0000,   /* U+1F1C */
  0x0000,   /* U+1F1D */
  0x9F1E,   /* U+1F1E */
  0x9F1F,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0000,   /* U+1F22 */
  0x0000,   /* U+1F23 */
  0x0000,   /* U+1F24 */
  0x0000,   /* U+1F25 */
  0x0000,   /* U+1F26 */
  0x0000,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0000,   /* U+1F2A */
  0x0000,   /* U+1F2B */
  0x0000,   /* U+1F2C */
  0x0000,   /* U+1F2D */
  0x0000,   /* U+1F2E */
  0x0000,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0000,   /* U+1F32 */
  0x0000,   /* U+1F33 */
  0x0000,   /* U+1F34 */
  0x0000,   /* U+1F35 */
  0x0000,   /* U+1F36 */
  0x0000,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0000,   /* U+1F3A */
  0x0000,   /* U+1F3B */
  0x0000,   /* U+1F3C */
  0x0000,   /* U+1F3D */
  0x0000,   /* U+1F3E */
  0x0000,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0000,   /* U+1F42 */
  0x0000,   /* U+1F43 */
  0x0000,   /* U+1F44 */
  0x0000,   /* U+1F45 */
  0x9F46,   /* U+1F46 */
  0x9F47,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0000,   /* U+1F4A */
  0x0000,   /* U+1F4B */
  0x0000,   /* U+1F4C */
  0x0000,   /* U+1F4D */
  0x9F4E,   /* U+1F4E */
  0x9F4F,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0000,   /* U+1F52 */
  0x0000,   /* U+1F53 */
  0x0000,   /* U+1F54 */
  0x0000,   /* U+1F55 */
  0x0000,   /* U+1F56 */
  0x0000,   /* U+1F57 */
  0x9F58,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x9F5A,   /* U+1F5A */
  0x0000,   /* U+1F5B */
  0x9F5C,   /* U+1F5C */
  0x0000,   /* U+1F5D */
  0x9F5E,   /* U+1F5E */
  0x0000,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0000,   /* U+1F62 */
  0x0000,   /* U+1F63 */
  0x0000,   /* U+1F64 */
  0x0000,   /* U+1F65 */
  0x0000,   /* U+1F66 */
  0x0000,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0000,   /* U+1F6A */
  0x0000,   /* U+1F6B */
  0x0000,   /* U+1F6C */
  0x0000,   /* U+1F6D */
  0x0000,   /* U+1F6E */
  0x0000,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x9F7E,   /* U+1F7E */
  0x9F7F,   /* U+1F7F */
  0x0000,   /* U+1F80 */
  0x0000,   /* U+1F81 */
  0x0000,   /* U+1F82 */
  0x0000,   /* U+1F83 */
  0x0000,   /* U+1F84 */
  0x0000,   /* U+1F85 */
  0x0000,   /* U+1F86 */
  0x0000,   /* U+1F87 */
  0x0000,   /* U+1F88 */
  0x0000,   /* U+1F89 */
  0x0000,   /* U+1F8A */
  0x0000,   /* U+1F8B */
  0x0000,   /* U+1F8C */
  0x0000,   /* U+1F8D */
  0x0000,   /* U+1F8E */
  0x0000,   /* U+1F8F */
  0x0000,   /* U+1F90 */
  0x0000,   /* U+1F91 */
  0x0000,   /* U+1F92 */
  0x0000,   /* U+1F93 */
  0x0000,   /* U+1F94 */
  0x0000,   /* U+1F95 */
  0x0000,   /* U+1F96 */
  0x0000,   /* U+1F97 */
  0x0000,   /* U+1F98 */
  0x0000,   /* U+1F99 */
  0x0000,   /* U+1F9A */
  0x0000,   /* U+1F9B */
  0x0000,   /* U+1F9C */
  0x0000,   /* U+1F9D */
  0x0000,   /* U+1F9E */
  0x0000,   /* U+1F9F */
  0x0000,   /* U+1FA0 */
  0x0000,   /* U+1FA1 */
  0x0000,   /* U+1FA2 */
  0x0000,   /* U+1FA3 */
  0x0000,   /* U+1FA4 */
  0x0000,   /* U+1FA5 */
  0x0000,   /* U+1FA6 */
  0x0000,   /* U+1FA7 */
  0x0000,   /* U+1FA8 */
  0x0000,   /* U+1FA9 */
  0x0000,   /* U+1FAA */
  0x0000,   /* U+1FAB */
  0x0000,   /* U+1FAC */
  0x0000,   /* U+1FAD */
  0x0000,   /* U+1FAE */
  0x0000,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0000,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0000,   /* U+1FB4 */
  0x9FB5,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0000,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0000,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0000,   /* U+1FC4 */
  0x9FC5,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0000,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0000,   /* U+1FD2 */
  0x0000,   /* U+1FD3 */
  0x9FD4,   /* U+1FD4 */
  0x9FD5,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0000,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x9FDC,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0000,   /* U+1FE2 */
  0x0000,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0000,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x9FF0,   /* U+1FF0 */
  0x9FF1,   /* U+1FF1 */
  0x0000,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0000,   /* U+1FF4 */
  0x9FF5,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0000,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x9FFF,   /* U+1FFF */

  /* Secondary weight 2 for each character. */
  0x0022,   /* U+1F00 */
  0x0023,   /* U+1F01 */
  0x0022,   /* U+1F02 */
  0x0023,   /* U+1F03 */
  0x0022,   /* U+1F04 */
  0x0023,   /* U+1F05 */
  0x0022,   /* U+1F06 */
  0x0023,   /* U+1F07 */
  0x0022,   /* U+1F08 */
  0x0023,   /* U+1F09 */
  0x0022,   /* U+1F0A */
  0x0023,   /* U+1F0B */
  0x0022,   /* U+1F0C */
  0x0023,   /* U+1F0D */
  0x0022,   /* U+1F0E */
  0x0023,   /* U+1F0F */
  0x0022,   /* U+1F10 */
  0x0023,   /* U+1F11 */
  0x0022,   /* U+1F12 */
  0x0023,   /* U+1F13 */
  0x0022,   /* U+1F14 */
  0x0023,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0022,   /* U+1F18 */
  0x0023,   /* U+1F19 */
  0x0022,   /* U+1F1A */
  0x0023,   /* U+1F1B */
  0x0022,   /* U+1F1C */
  0x0023,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0022,   /* U+1F20 */
  0x0023,   /* U+1F21 */
  0x0022,   /* U+1F22 */
  0x0023,   /* U+1F23 */
  0x0022,   /* U+1F24 */
  0x0023,   /* U+1F25 */
  0x0022,   /* U+1F26 */
  0x0023,   /* U+1F27 */
  0x0022,   /* U+1F28 */
  0x0023,   /* U+1F29 */
  0x0022,   /* U+1F2A */
  0x0023,   /* U+1F2B */
  0x0022,   /* U+1F2C */
  0x0023,   /* U+1F2D */
  0x0022,   /* U+1F2E */
  0x0023,   /* U+1F2F */
  0x0022,   /* U+1F30 */
  0x0023,   /* U+1F31 */
  0x0022,   /* U+1F32 */
  0x0023,   /* U+1F33 */
  0x0022,   /* U+1F34 */
  0x0023,   /* U+1F35 */
  0x0022,   /* U+1F36 */
  0x0023,   /* U+1F37 */
  0x0022,   /* U+1F38 */
  0x0023,   /* U+1F39 */
  0x0022,   /* U+1F3A */
  0x0023,   /* U+1F3B */
  0x0022,   /* U+1F3C */
  0x0023,   /* U+1F3D */
  0x0022,   /* U+1F3E */
  0x0023,   /* U+1F3F */
  0x0022,   /* U+1F40 */
  0x0023,   /* U+1F41 */
  0x0022,   /* U+1F42 */
  0x0023,   /* U+1F43 */
  0x0022,   /* U+1F44 */
  0x0023,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0022,   /* U+1F48 */
  0x0023,   /* U+1F49 */
  0x0022,   /* U+1F4A */
  0x0023,   /* U+1F4B */
  0x0022,   /* U+1F4C */
  0x0023,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0022,   /* U+1F50 */
  0x0023,   /* U+1F51 */
  0x0022,   /* U+1F52 */
  0x0023,   /* U+1F53 */
  0x0022,   /* U+1F54 */
  0x0023,   /* U+1F55 */
  0x0022,   /* U+1F56 */
  0x0023,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0023,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0023,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0023,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0023,   /* U+1F5F */
  0x0022,   /* U+1F60 */
  0x0023,   /* U+1F61 */
  0x0022,   /* U+1F62 */
  0x0023,   /* U+1F63 */
  0x0022,   /* U+1F64 */
  0x0023,   /* U+1F65 */
  0x0022,   /* U+1F66 */
  0x0023,   /* U+1F67 */
  0x0022,   /* U+1F68 */
  0x0023,   /* U+1F69 */
  0x0022,   /* U+1F6A */
  0x0023,   /* U+1F6B */
  0x0022,   /* U+1F6C */
  0x0023,   /* U+1F6D */
  0x0022,   /* U+1F6E */
  0x0023,   /* U+1F6F */
  0x0025,   /* U+1F70 */
  0x0024,   /* U+1F71 */
  0x0025,   /* U+1F72 */
  0x0024,   /* U+1F73 */
  0x0025,   /* U+1F74 */
  0x0024,   /* U+1F75 */
  0x0025,   /* U+1F76 */
  0x0024,   /* U+1F77 */
  0x0025,   /* U+1F78 */
  0x0024,   /* U+1F79 */
  0x0025,   /* U+1F7A */
  0x0024,   /* U+1F7B */
  0x0025,   /* U+1F7C */
  0x0024,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0022,   /* U+1F80 */
  0x0023,   /* U+1F81 */
  0x0022,   /* U+1F82 */
  0x0023,   /* U+1F83 */
  0x0022,   /* U+1F84 */
  0x0023,   /* U+1F85 */
  0x0022,   /* U+1F86 */
  0x0023,   /* U+1F87 */
  0x0022,   /* U+1F88 */
  0x0023,   /* U+1F89 */
  0x0022,   /* U+1F8A */
  0x0023,   /* U+1F8B */
  0x0022,   /* U+1F8C */
  0x0023,   /* U+1F8D */
  0x0022,   /* U+1F8E */
  0x0023,   /* U+1F8F */
  0x0022,   /* U+1F90 */
  0x0023,   /* U+1F91 */
  0x0022,   /* U+1F92 */
  0x0023,   /* U+1F93 */
  0x0022,   /* U+1F94 */
  0x0023,   /* U+1F95 */
  0x0022,   /* U+1F96 */
  0x0023,   /* U+1F97 */
  0x0022,   /* U+1F98 */
  0x0023,   /* U+1F99 */
  0x0022,   /* U+1F9A */
  0x0023,   /* U+1F9B */
  0x0022,   /* U+1F9C */
  0x0023,   /* U+1F9D */
  0x0022,   /* U+1F9E */
  0x0023,   /* U+1F9F */
  0x0022,   /* U+1FA0 */
  0x0023,   /* U+1FA1 */
  0x0022,   /* U+1FA2 */
  0x0023,   /* U+1FA3 */
  0x0022,   /* U+1FA4 */
  0x0023,   /* U+1FA5 */
  0x0022,   /* U+1FA6 */
  0x0023,   /* U+1FA7 */
  0x0022,   /* U+1FA8 */
  0x0023,   /* U+1FA9 */
  0x0022,   /* U+1FAA */
  0x0023,   /* U+1FAB */
  0x0022,   /* U+1FAC */
  0x0023,   /* U+1FAD */
  0x0022,   /* U+1FAE */
  0x0023,   /* U+1FAF */
  0x0026,   /* U+1FB0 */
  0x0032,   /* U+1FB1 */
  0x0025,   /* U+1FB2 */
  0x004C,   /* U+1FB3 */
  0x0024,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x002A,   /* U+1FB6 */
  0x002A,   /* U+1FB7 */
  0x0026,   /* U+1FB8 */
  0x0032,   /* U+1FB9 */
  0x0025,   /* U+1FBA */
  0x0024,   /* U+1FBB */
  0x004C,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x002A,   /* U+1FC1 */
  0x0025,   /* U+1FC2 */
  0x004C,   /* U+1FC3 */
  0x0024,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x002A,   /* U+1FC6 */
  0x002A,   /* U+1FC7 */
  0x0025,   /* U+1FC8 */
  0x0024,   /* U+1FC9 */
  0x0025,   /* U+1FCA */
  0x0024,   /* U+1FCB */
  0x004C,   /* U+1FCC */
  0x0025,   /* U+1FCD */
  0x0024,   /* U+1FCE */
  0x002A,   /* U+1FCF */
  0x0026,   /* U+1FD0 */
  0x0032,   /* U+1FD1 */
  0x002B,   /* U+1FD2 */
  0x002B,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x002A,   /* U+1FD6 */
  0x002B,   /* U+1FD7 */
  0x0026,   /* U+1FD8 */
  0x0032,   /* U+1FD9 */
  0x0025,   /* U+1FDA */
  0x0024,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0025,   /* U+1FDD */
  0x0024,   /* U+1FDE */
  0x002A,   /* U+1FDF */
  0x0026,   /* U+1FE0 */
  0x0032,   /* U+1FE1 */
  0x002B,   /* U+1FE2 */
  0x002B,   /* U+1FE3 */
  0x0022,   /* U+1FE4 */
  0x0023,   /* U+1FE5 */
  0x002A,   /* U+1FE6 */
  0x002B,   /* U+1FE7 */
  0x0026,   /* U+1FE8 */
  0x0032,   /* U+1FE9 */
  0x0025,   /* U+1FEA */
  0x0024,   /* U+1FEB */
  0x0023,   /* U+1FEC */
  0x0025,   /* U+1FED */
  0x0024,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0025,   /* U+1FF2 */
  0x004C,   /* U+1FF3 */
  0x0024,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x002A,   /* U+1FF6 */
  0x002A,   /* U+1FF7 */
  0x0025,   /* U+1FF8 */
  0x0024,   /* U+1FF9 */
  0x0025,   /* U+1FFA */
  0x0024,   /* U+1FFB */
  0x004C,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Tertiary weight 2 for each character. */
  0x0002,   /* U+1F00 */
  0x0002,   /* U+1F01 */
  0x0002,   /* U+1F02 */
  0x0002,   /* U+1F03 */
  0x0002,   /* U+1F04 */
  0x0002,   /* U+1F05 */
  0x0002,   /* U+1F06 */
  0x0002,   /* U+1F07 */
  0x0002,   /* U+1F08 */
  0x0002,   /* U+1F09 */
  0x0002,   /* U+1F0A */
  0x0002,   /* U+1F0B */
  0x0002,   /* U+1F0C */
  0x0002,   /* U+1F0D */
  0x0002,   /* U+1F0E */
  0x0002,   /* U+1F0F */
  0x0002,   /* U+1F10 */
  0x0002,   /* U+1F11 */
  0x0002,   /* U+1F12 */
  0x0002,   /* U+1F13 */
  0x0002,   /* U+1F14 */
  0x0002,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0002,   /* U+1F18 */
  0x0002,   /* U+1F19 */
  0x0002,   /* U+1F1A */
  0x0002,   /* U+1F1B */
  0x0002,   /* U+1F1C */
  0x0002,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0002,   /* U+1F20 */
  0x0002,   /* U+1F21 */
  0x0002,   /* U+1F22 */
  0x0002,   /* U+1F23 */
  0x0002,   /* U+1F24 */
  0x0002,   /* U+1F25 */
  0x0002,   /* U+1F26 */
  0x0002,   /* U+1F27 */
  0x0002,   /* U+1F28 */
  0x0002,   /* U+1F29 */
  0x0002,   /* U+1F2A */
  0x0002,   /* U+1F2B */
  0x0002,   /* U+1F2C */
  0x0002,   /* U+1F2D */
  0x0002,   /* U+1F2E */
  0x0002,   /* U+1F2F */
  0x0002,   /* U+1F30 */
  0x0002,   /* U+1F31 */
  0x0002,   /* U+1F32 */
  0x0002,   /* U+1F33 */
  0x0002,   /* U+1F34 */
  0x0002,   /* U+1F35 */
  0x0002,   /* U+1F36 */
  0x0002,   /* U+1F37 */
  0x0002,   /* U+1F38 */
  0x0002,   /* U+1F39 */
  0x0002,   /* U+1F3A */
  0x0002,   /* U+1F3B */
  0x0002,   /* U+1F3C */
  0x0002,   /* U+1F3D */
  0x0002,   /* U+1F3E */
  0x0002,   /* U+1F3F */
  0x0002,   /* U+1F40 */
  0x0002,   /* U+1F41 */
  0x0002,   /* U+1F42 */
  0x0002,   /* U+1F43 */
  0x0002,   /* U+1F44 */
  0x0002,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0002,   /* U+1F48 */
  0x0002,   /* U+1F49 */
  0x0002,   /* U+1F4A */
  0x0002,   /* U+1F4B */
  0x0002,   /* U+1F4C */
  0x0002,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0002,   /* U+1F50 */
  0x0002,   /* U+1F51 */
  0x0002,   /* U+1F52 */
  0x0002,   /* U+1F53 */
  0x0002,   /* U+1F54 */
  0x0002,   /* U+1F55 */
  0x0002,   /* U+1F56 */
  0x0002,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0002,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0002,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0002,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0002,   /* U+1F5F */
  0x0002,   /* U+1F60 */
  0x0002,   /* U+1F61 */
  0x0002,   /* U+1F62 */
  0x0002,   /* U+1F63 */
  0x0002,   /* U+1F64 */
  0x0002,   /* U+1F65 */
  0x0002,   /* U+1F66 */
  0x0002,   /* U+1F67 */
  0x0002,   /* U+1F68 */
  0x0002,   /* U+1F69 */
  0x0002,   /* U+1F6A */
  0x0002,   /* U+1F6B */
  0x0002,   /* U+1F6C */
  0x0002,   /* U+1F6D */
  0x0002,   /* U+1F6E */
  0x0002,   /* U+1F6F */
  0x0002,   /* U+1F70 */
  0x0002,   /* U+1F71 */
  0x0002,   /* U+1F72 */
  0x0002,   /* U+1F73 */
  0x0002,   /* U+1F74 */
  0x0002,   /* U+1F75 */
  0x0002,   /* U+1F76 */
  0x0002,   /* U+1F77 */
  0x0002,   /* U+1F78 */
  0x0002,   /* U+1F79 */
  0x0002,   /* U+1F7A */
  0x0002,   /* U+1F7B */
  0x0002,   /* U+1F7C */
  0x0002,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0002,   /* U+1F80 */
  0x0002,   /* U+1F81 */
  0x0002,   /* U+1F82 */
  0x0002,   /* U+1F83 */
  0x0002,   /* U+1F84 */
  0x0002,   /* U+1F85 */
  0x0002,   /* U+1F86 */
  0x0002,   /* U+1F87 */
  0x0002,   /* U+1F88 */
  0x0002,   /* U+1F89 */
  0x0002,   /* U+1F8A */
  0x0002,   /* U+1F8B */
  0x0002,   /* U+1F8C */
  0x0002,   /* U+1F8D */
  0x0002,   /* U+1F8E */
  0x0002,   /* U+1F8F */
  0x0002,   /* U+1F90 */
  0x0002,   /* U+1F91 */
  0x0002,   /* U+1F92 */
  0x0002,   /* U+1F93 */
  0x0002,   /* U+1F94 */
  0x0002,   /* U+1F95 */
  0x0002,   /* U+1F96 */
  0x0002,   /* U+1F97 */
  0x0002,   /* U+1F98 */
  0x0002,   /* U+1F99 */
  0x0002,   /* U+1F9A */
  0x0002,   /* U+1F9B */
  0x0002,   /* U+1F9C */
  0x0002,   /* U+1F9D */
  0x0002,   /* U+1F9E */
  0x0002,   /* U+1F9F */
  0x0002,   /* U+1FA0 */
  0x0002,   /* U+1FA1 */
  0x0002,   /* U+1FA2 */
  0x0002,   /* U+1FA3 */
  0x0002,   /* U+1FA4 */
  0x0002,   /* U+1FA5 */
  0x0002,   /* U+1FA6 */
  0x0002,   /* U+1FA7 */
  0x0002,   /* U+1FA8 */
  0x0002,   /* U+1FA9 */
  0x0002,   /* U+1FAA */
  0x0002,   /* U+1FAB */
  0x0002,   /* U+1FAC */
  0x0002,   /* U+1FAD */
  0x0002,   /* U+1FAE */
  0x0002,   /* U+1FAF */
  0x0002,   /* U+1FB0 */
  0x0002,   /* U+1FB1 */
  0x0002,   /* U+1FB2 */
  0x0002,   /* U+1FB3 */
  0x0002,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0002,   /* U+1FB6 */
  0x0002,   /* U+1FB7 */
  0x0002,   /* U+1FB8 */
  0x0002,   /* U+1FB9 */
  0x0002,   /* U+1FBA */
  0x0002,   /* U+1FBB */
  0x0002,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0002,   /* U+1FC1 */
  0x0002,   /* U+1FC2 */
  0x0002,   /* U+1FC3 */
  0x0002,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0002,   /* U+1FC6 */
  0x0002,   /* U+1FC7 */
  0x0002,   /* U+1FC8 */
  0x0002,   /* U+1FC9 */
  0x0002,   /* U+1FCA */
  0x0002,   /* U+1FCB */
  0x0002,   /* U+1FCC */
  0x0002,   /* U+1FCD */
  0x0002,   /* U+1FCE */
  0x0002,   /* U+1FCF */
  0x0002,   /* U+1FD0 */
  0x0002,   /* U+1FD1 */
  0x0002,   /* U+1FD2 */
  0x0002,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0002,   /* U+1FD6 */
  0x0002,   /* U+1FD7 */
  0x0002,   /* U+1FD8 */
  0x0002,   /* U+1FD9 */
  0x0002,   /* U+1FDA */
  0x0002,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0002,   /* U+1FDD */
  0x0002,   /* U+1FDE */
  0x0002,   /* U+1FDF */
  0x0002,   /* U+1FE0 */
  0x0002,   /* U+1FE1 */
  0x0002,   /* U+1FE2 */
  0x0002,   /* U+1FE3 */
  0x0002,   /* U+1FE4 */
  0x0002,   /* U+1FE5 */
  0x0002,   /* U+1FE6 */
  0x0002,   /* U+1FE7 */
  0x0002,   /* U+1FE8 */
  0x0002,   /* U+1FE9 */
  0x0002,   /* U+1FEA */
  0x0002,   /* U+1FEB */
  0x0002,   /* U+1FEC */
  0x0002,   /* U+1FED */
  0x0002,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0002,   /* U+1FF2 */
  0x0002,   /* U+1FF3 */
  0x0002,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0002,   /* U+1FF6 */
  0x0002,   /* U+1FF7 */
  0x0002,   /* U+1FF8 */
  0x0002,   /* U+1FF9 */
  0x0002,   /* U+1FFA */
  0x0002,   /* U+1FFB */
  0x0002,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0000,   /* U+1F02 */
  0x0000,   /* U+1F03 */
  0x0000,   /* U+1F04 */
  0x0000,   /* U+1F05 */
  0x0000,   /* U+1F06 */
  0x0000,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0000,   /* U+1F0A */
  0x0000,   /* U+1F0B */
  0x0000,   /* U+1F0C */
  0x0000,   /* U+1F0D */
  0x0000,   /* U+1F0E */
  0x0000,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0000,   /* U+1F12 */
  0x0000,   /* U+1F13 */
  0x0000,   /* U+1F14 */
  0x0000,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0000,   /* U+1F1A */
  0x0000,   /* U+1F1B */
  0x0000,   /* U+1F1C */
  0x0000,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0000,   /* U+1F22 */
  0x0000,   /* U+1F23 */
  0x0000,   /* U+1F24 */
  0x0000,   /* U+1F25 */
  0x0000,   /* U+1F26 */
  0x0000,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0000,   /* U+1F2A */
  0x0000,   /* U+1F2B */
  0x0000,   /* U+1F2C */
  0x0000,   /* U+1F2D */
  0x0000,   /* U+1F2E */
  0x0000,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0000,   /* U+1F32 */
  0x0000,   /* U+1F33 */
  0x0000,   /* U+1F34 */
  0x0000,   /* U+1F35 */
  0x0000,   /* U+1F36 */
  0x0000,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0000,   /* U+1F3A */
  0x0000,   /* U+1F3B */
  0x0000,   /* U+1F3C */
  0x0000,   /* U+1F3D */
  0x0000,   /* U+1F3E */
  0x0000,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0000,   /* U+1F42 */
  0x0000,   /* U+1F43 */
  0x0000,   /* U+1F44 */
  0x0000,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0000,   /* U+1F4A */
  0x0000,   /* U+1F4B */
  0x0000,   /* U+1F4C */
  0x0000,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0000,   /* U+1F52 */
  0x0000,   /* U+1F53 */
  0x0000,   /* U+1F54 */
  0x0000,   /* U+1F55 */
  0x0000,   /* U+1F56 */
  0x0000,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0000,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0000,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0000,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0000,   /* U+1F62 */
  0x0000,   /* U+1F63 */
  0x0000,   /* U+1F64 */
  0x0000,   /* U+1F65 */
  0x0000,   /* U+1F66 */
  0x0000,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0000,   /* U+1F6A */
  0x0000,   /* U+1F6B */
  0x0000,   /* U+1F6C */
  0x0000,   /* U+1F6D */
  0x0000,   /* U+1F6E */
  0x0000,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0000,   /* U+1F80 */
  0x0000,   /* U+1F81 */
  0x0000,   /* U+1F82 */
  0x0000,   /* U+1F83 */
  0x0000,   /* U+1F84 */
  0x0000,   /* U+1F85 */
  0x0000,   /* U+1F86 */
  0x0000,   /* U+1F87 */
  0x0000,   /* U+1F88 */
  0x0000,   /* U+1F89 */
  0x0000,   /* U+1F8A */
  0x0000,   /* U+1F8B */
  0x0000,   /* U+1F8C */
  0x0000,   /* U+1F8D */
  0x0000,   /* U+1F8E */
  0x0000,   /* U+1F8F */
  0x0000,   /* U+1F90 */
  0x0000,   /* U+1F91 */
  0x0000,   /* U+1F92 */
  0x0000,   /* U+1F93 */
  0x0000,   /* U+1F94 */
  0x0000,   /* U+1F95 */
  0x0000,   /* U+1F96 */
  0x0000,   /* U+1F97 */
  0x0000,   /* U+1F98 */
  0x0000,   /* U+1F99 */
  0x0000,   /* U+1F9A */
  0x0000,   /* U+1F9B */
  0x0000,   /* U+1F9C */
  0x0000,   /* U+1F9D */
  0x0000,   /* U+1F9E */
  0x0000,   /* U+1F9F */
  0x0000,   /* U+1FA0 */
  0x0000,   /* U+1FA1 */
  0x0000,   /* U+1FA2 */
  0x0000,   /* U+1FA3 */
  0x0000,   /* U+1FA4 */
  0x0000,   /* U+1FA5 */
  0x0000,   /* U+1FA6 */
  0x0000,   /* U+1FA7 */
  0x0000,   /* U+1FA8 */
  0x0000,   /* U+1FA9 */
  0x0000,   /* U+1FAA */
  0x0000,   /* U+1FAB */
  0x0000,   /* U+1FAC */
  0x0000,   /* U+1FAD */
  0x0000,   /* U+1FAE */
  0x0000,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0000,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0000,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0000,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0000,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0000,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0000,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0000,   /* U+1FD2 */
  0x0000,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0000,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0000,   /* U+1FE2 */
  0x0000,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0000,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0000,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0000,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0000,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0025,   /* U+1F02 */
  0x0025,   /* U+1F03 */
  0x0024,   /* U+1F04 */
  0x0024,   /* U+1F05 */
  0x002A,   /* U+1F06 */
  0x002A,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0025,   /* U+1F0A */
  0x0025,   /* U+1F0B */
  0x0024,   /* U+1F0C */
  0x0024,   /* U+1F0D */
  0x002A,   /* U+1F0E */
  0x002A,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0025,   /* U+1F12 */
  0x0025,   /* U+1F13 */
  0x0024,   /* U+1F14 */
  0x0024,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0025,   /* U+1F1A */
  0x0025,   /* U+1F1B */
  0x0024,   /* U+1F1C */
  0x0024,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0025,   /* U+1F22 */
  0x0025,   /* U+1F23 */
  0x0024,   /* U+1F24 */
  0x0024,   /* U+1F25 */
  0x002A,   /* U+1F26 */
  0x002A,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0025,   /* U+1F2A */
  0x0025,   /* U+1F2B */
  0x0024,   /* U+1F2C */
  0x0024,   /* U+1F2D */
  0x002A,   /* U+1F2E */
  0x002A,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0025,   /* U+1F32 */
  0x0025,   /* U+1F33 */
  0x0024,   /* U+1F34 */
  0x0024,   /* U+1F35 */
  0x002A,   /* U+1F36 */
  0x002A,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0025,   /* U+1F3A */
  0x0025,   /* U+1F3B */
  0x0024,   /* U+1F3C */
  0x0024,   /* U+1F3D */
  0x002A,   /* U+1F3E */
  0x002A,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0025,   /* U+1F42 */
  0x0025,   /* U+1F43 */
  0x0024,   /* U+1F44 */
  0x0024,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0025,   /* U+1F4A */
  0x0025,   /* U+1F4B */
  0x0024,   /* U+1F4C */
  0x0024,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0025,   /* U+1F52 */
  0x0025,   /* U+1F53 */
  0x0024,   /* U+1F54 */
  0x0024,   /* U+1F55 */
  0x002A,   /* U+1F56 */
  0x002A,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0025,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0024,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x002A,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0025,   /* U+1F62 */
  0x0025,   /* U+1F63 */
  0x0024,   /* U+1F64 */
  0x0024,   /* U+1F65 */
  0x002A,   /* U+1F66 */
  0x002A,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0025,   /* U+1F6A */
  0x0025,   /* U+1F6B */
  0x0024,   /* U+1F6C */
  0x0024,   /* U+1F6D */
  0x002A,   /* U+1F6E */
  0x002A,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x004C,   /* U+1F80 */
  0x004C,   /* U+1F81 */
  0x0025,   /* U+1F82 */
  0x0025,   /* U+1F83 */
  0x0024,   /* U+1F84 */
  0x0024,   /* U+1F85 */
  0x002A,   /* U+1F86 */
  0x002A,   /* U+1F87 */
  0x004C,   /* U+1F88 */
  0x004C,   /* U+1F89 */
  0x0025,   /* U+1F8A */
  0x0025,   /* U+1F8B */
  0x0024,   /* U+1F8C */
  0x0024,   /* U+1F8D */
  0x002A,   /* U+1F8E */
  0x002A,   /* U+1F8F */
  0x004C,   /* U+1F90 */
  0x004C,   /* U+1F91 */
  0x0025,   /* U+1F92 */
  0x0025,   /* U+1F93 */
  0x0024,   /* U+1F94 */
  0x0024,   /* U+1F95 */
  0x002A,   /* U+1F96 */
  0x002A,   /* U+1F97 */
  0x004C,   /* U+1F98 */
  0x004C,   /* U+1F99 */
  0x0025,   /* U+1F9A */
  0x0025,   /* U+1F9B */
  0x0024,   /* U+1F9C */
  0x0024,   /* U+1F9D */
  0x002A,   /* U+1F9E */
  0x002A,   /* U+1F9F */
  0x004C,   /* U+1FA0 */
  0x004C,   /* U+1FA1 */
  0x0025,   /* U+1FA2 */
  0x0025,   /* U+1FA3 */
  0x0024,   /* U+1FA4 */
  0x0024,   /* U+1FA5 */
  0x002A,   /* U+1FA6 */
  0x002A,   /* U+1FA7 */
  0x004C,   /* U+1FA8 */
  0x004C,   /* U+1FA9 */
  0x0025,   /* U+1FAA */
  0x0025,   /* U+1FAB */
  0x0024,   /* U+1FAC */
  0x0024,   /* U+1FAD */
  0x002A,   /* U+1FAE */
  0x002A,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x004C,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x004C,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x004C,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x004C,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x004C,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x004C,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0025,   /* U+1FD2 */
  0x0024,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x002A,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0025,   /* U+1FE2 */
  0x0024,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x002A,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x004C,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x004C,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x004C,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0002,   /* U+1F02 */
  0x0002,   /* U+1F03 */
  0x0002,   /* U+1F04 */
  0x0002,   /* U+1F05 */
  0x0002,   /* U+1F06 */
  0x0002,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0002,   /* U+1F0A */
  0x0002,   /* U+1F0B */
  0x0002,   /* U+1F0C */
  0x0002,   /* U+1F0D */
  0x0002,   /* U+1F0E */
  0x0002,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0002,   /* U+1F12 */
  0x0002,   /* U+1F13 */
  0x0002,   /* U+1F14 */
  0x0002,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0002,   /* U+1F1A */
  0x0002,   /* U+1F1B */
  0x0002,   /* U+1F1C */
  0x0002,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0002,   /* U+1F22 */
  0x0002,   /* U+1F23 */
  0x0002,   /* U+1F24 */
  0x0002,   /* U+1F25 */
  0x0002,   /* U+1F26 */
  0x0002,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0002,   /* U+1F2A */
  0x0002,   /* U+1F2B */
  0x0002,   /* U+1F2C */
  0x0002,   /* U+1F2D */
  0x0002,   /* U+1F2E */
  0x0002,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0002,   /* U+1F32 */
  0x0002,   /* U+1F33 */
  0x0002,   /* U+1F34 */
  0x0002,   /* U+1F35 */
  0x0002,   /* U+1F36 */
  0x0002,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0002,   /* U+1F3A */
  0x0002,   /* U+1F3B */
  0x0002,   /* U+1F3C */
  0x0002,   /* U+1F3D */
  0x0002,   /* U+1F3E */
  0x0002,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0002,   /* U+1F42 */
  0x0002,   /* U+1F43 */
  0x0002,   /* U+1F44 */
  0x0002,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0002,   /* U+1F4A */
  0x0002,   /* U+1F4B */
  0x0002,   /* U+1F4C */
  0x0002,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0002,   /* U+1F52 */
  0x0002,   /* U+1F53 */
  0x0002,   /* U+1F54 */
  0x0002,   /* U+1F55 */
  0x0002,   /* U+1F56 */
  0x0002,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0002,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0002,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0002,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0002,   /* U+1F62 */
  0x0002,   /* U+1F63 */
  0x0002,   /* U+1F64 */
  0x0002,   /* U+1F65 */
  0x0002,   /* U+1F66 */
  0x0002,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0002,   /* U+1F6A */
  0x0002,   /* U+1F6B */
  0x0002,   /* U+1F6C */
  0x0002,   /* U+1F6D */
  0x0002,   /* U+1F6E */
  0x0002,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0002,   /* U+1F80 */
  0x0002,   /* U+1F81 */
  0x0002,   /* U+1F82 */
  0x0002,   /* U+1F83 */
  0x0002,   /* U+1F84 */
  0x0002,   /* U+1F85 */
  0x0002,   /* U+1F86 */
  0x0002,   /* U+1F87 */
  0x0002,   /* U+1F88 */
  0x0002,   /* U+1F89 */
  0x0002,   /* U+1F8A */
  0x0002,   /* U+1F8B */
  0x0002,   /* U+1F8C */
  0x0002,   /* U+1F8D */
  0x0002,   /* U+1F8E */
  0x0002,   /* U+1F8F */
  0x0002,   /* U+1F90 */
  0x0002,   /* U+1F91 */
  0x0002,   /* U+1F92 */
  0x0002,   /* U+1F93 */
  0x0002,   /* U+1F94 */
  0x0002,   /* U+1F95 */
  0x0002,   /* U+1F96 */
  0x0002,   /* U+1F97 */
  0x0002,   /* U+1F98 */
  0x0002,   /* U+1F99 */
  0x0002,   /* U+1F9A */
  0x0002,   /* U+1F9B */
  0x0002,   /* U+1F9C */
  0x0002,   /* U+1F9D */
  0x0002,   /* U+1F9E */
  0x0002,   /* U+1F9F */
  0x0002,   /* U+1FA0 */
  0x0002,   /* U+1FA1 */
  0x0002,   /* U+1FA2 */
  0x0002,   /* U+1FA3 */
  0x0002,   /* U+1FA4 */
  0x0002,   /* U+1FA5 */
  0x0002,   /* U+1FA6 */
  0x0002,   /* U+1FA7 */
  0x0002,   /* U+1FA8 */
  0x0002,   /* U+1FA9 */
  0x0002,   /* U+1FAA */
  0x0002,   /* U+1FAB */
  0x0002,   /* U+1FAC */
  0x0002,   /* U+1FAD */
  0x0002,   /* U+1FAE */
  0x0002,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0002,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0002,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0002,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0002,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0002,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0002,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0002,   /* U+1FD2 */
  0x0002,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0002,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0002,   /* U+1FE2 */
  0x0002,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0002,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0002,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0002,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0002,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0000,   /* U+1F02 */
  0x0000,   /* U+1F03 */
  0x0000,   /* U+1F04 */
  0x0000,   /* U+1F05 */
  0x0000,   /* U+1F06 */
  0x0000,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0000,   /* U+1F0A */
  0x0000,   /* U+1F0B */
  0x0000,   /* U+1F0C */
  0x0000,   /* U+1F0D */
  0x0000,   /* U+1F0E */
  0x0000,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0000,   /* U+1F12 */
  0x0000,   /* U+1F13 */
  0x0000,   /* U+1F14 */
  0x0000,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0000,   /* U+1F1A */
  0x0000,   /* U+1F1B */
  0x0000,   /* U+1F1C */
  0x0000,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0000,   /* U+1F22 */
  0x0000,   /* U+1F23 */
  0x0000,   /* U+1F24 */
  0x0000,   /* U+1F25 */
  0x0000,   /* U+1F26 */
  0x0000,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0000,   /* U+1F2A */
  0x0000,   /* U+1F2B */
  0x0000,   /* U+1F2C */
  0x0000,   /* U+1F2D */
  0x0000,   /* U+1F2E */
  0x0000,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0000,   /* U+1F32 */
  0x0000,   /* U+1F33 */
  0x0000,   /* U+1F34 */
  0x0000,   /* U+1F35 */
  0x0000,   /* U+1F36 */
  0x0000,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0000,   /* U+1F3A */
  0x0000,   /* U+1F3B */
  0x0000,   /* U+1F3C */
  0x0000,   /* U+1F3D */
  0x0000,   /* U+1F3E */
  0x0000,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0000,   /* U+1F42 */
  0x0000,   /* U+1F43 */
  0x0000,   /* U+1F44 */
  0x0000,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0000,   /* U+1F4A */
  0x0000,   /* U+1F4B */
  0x0000,   /* U+1F4C */
  0x0000,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0000,   /* U+1F52 */
  0x0000,   /* U+1F53 */
  0x0000,   /* U+1F54 */
  0x0000,   /* U+1F55 */
  0x0000,   /* U+1F56 */
  0x0000,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0000,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0000,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0000,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0000,   /* U+1F62 */
  0x0000,   /* U+1F63 */
  0x0000,   /* U+1F64 */
  0x0000,   /* U+1F65 */
  0x0000,   /* U+1F66 */
  0x0000,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0000,   /* U+1F6A */
  0x0000,   /* U+1F6B */
  0x0000,   /* U+1F6C */
  0x0000,   /* U+1F6D */
  0x0000,   /* U+1F6E */
  0x0000,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0000,   /* U+1F80 */
  0x0000,   /* U+1F81 */
  0x0000,   /* U+1F82 */
  0x0000,   /* U+1F83 */
  0x0000,   /* U+1F84 */
  0x0000,   /* U+1F85 */
  0x0000,   /* U+1F86 */
  0x0000,   /* U+1F87 */
  0x0000,   /* U+1F88 */
  0x0000,   /* U+1F89 */
  0x0000,   /* U+1F8A */
  0x0000,   /* U+1F8B */
  0x0000,   /* U+1F8C */
  0x0000,   /* U+1F8D */
  0x0000,   /* U+1F8E */
  0x0000,   /* U+1F8F */
  0x0000,   /* U+1F90 */
  0x0000,   /* U+1F91 */
  0x0000,   /* U+1F92 */
  0x0000,   /* U+1F93 */
  0x0000,   /* U+1F94 */
  0x0000,   /* U+1F95 */
  0x0000,   /* U+1F96 */
  0x0000,   /* U+1F97 */
  0x0000,   /* U+1F98 */
  0x0000,   /* U+1F99 */
  0x0000,   /* U+1F9A */
  0x0000,   /* U+1F9B */
  0x0000,   /* U+1F9C */
  0x0000,   /* U+1F9D */
  0x0000,   /* U+1F9E */
  0x0000,   /* U+1F9F */
  0x0000,   /* U+1FA0 */
  0x0000,   /* U+1FA1 */
  0x0000,   /* U+1FA2 */
  0x0000,   /* U+1FA3 */
  0x0000,   /* U+1FA4 */
  0x0000,   /* U+1FA5 */
  0x0000,   /* U+1FA6 */
  0x0000,   /* U+1FA7 */
  0x0000,   /* U+1FA8 */
  0x0000,   /* U+1FA9 */
  0x0000,   /* U+1FAA */
  0x0000,   /* U+1FAB */
  0x0000,   /* U+1FAC */
  0x0000,   /* U+1FAD */
  0x0000,   /* U+1FAE */
  0x0000,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0000,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0000,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0000,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0000,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0000,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0000,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0000,   /* U+1FD2 */
  0x0000,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0000,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0000,   /* U+1FE2 */
  0x0000,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0000,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0000,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0000,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0000,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0000,   /* U+1F02 */
  0x0000,   /* U+1F03 */
  0x0000,   /* U+1F04 */
  0x0000,   /* U+1F05 */
  0x0000,   /* U+1F06 */
  0x0000,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0000,   /* U+1F0A */
  0x0000,   /* U+1F0B */
  0x0000,   /* U+1F0C */
  0x0000,   /* U+1F0D */
  0x0000,   /* U+1F0E */
  0x0000,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0000,   /* U+1F12 */
  0x0000,   /* U+1F13 */
  0x0000,   /* U+1F14 */
  0x0000,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0000,   /* U+1F1A */
  0x0000,   /* U+1F1B */
  0x0000,   /* U+1F1C */
  0x0000,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0000,   /* U+1F22 */
  0x0000,   /* U+1F23 */
  0x0000,   /* U+1F24 */
  0x0000,   /* U+1F25 */
  0x0000,   /* U+1F26 */
  0x0000,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0000,   /* U+1F2A */
  0x0000,   /* U+1F2B */
  0x0000,   /* U+1F2C */
  0x0000,   /* U+1F2D */
  0x0000,   /* U+1F2E */
  0x0000,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0000,   /* U+1F32 */
  0x0000,   /* U+1F33 */
  0x0000,   /* U+1F34 */
  0x0000,   /* U+1F35 */
  0x0000,   /* U+1F36 */
  0x0000,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0000,   /* U+1F3A */
  0x0000,   /* U+1F3B */
  0x0000,   /* U+1F3C */
  0x0000,   /* U+1F3D */
  0x0000,   /* U+1F3E */
  0x0000,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0000,   /* U+1F42 */
  0x0000,   /* U+1F43 */
  0x0000,   /* U+1F44 */
  0x0000,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0000,   /* U+1F4A */
  0x0000,   /* U+1F4B */
  0x0000,   /* U+1F4C */
  0x0000,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0000,   /* U+1F52 */
  0x0000,   /* U+1F53 */
  0x0000,   /* U+1F54 */
  0x0000,   /* U+1F55 */
  0x0000,   /* U+1F56 */
  0x0000,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0000,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0000,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0000,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0000,   /* U+1F62 */
  0x0000,   /* U+1F63 */
  0x0000,   /* U+1F64 */
  0x0000,   /* U+1F65 */
  0x0000,   /* U+1F66 */
  0x0000,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0000,   /* U+1F6A */
  0x0000,   /* U+1F6B */
  0x0000,   /* U+1F6C */
  0x0000,   /* U+1F6D */
  0x0000,   /* U+1F6E */
  0x0000,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0000,   /* U+1F80 */
  0x0000,   /* U+1F81 */
  0x004C,   /* U+1F82 */
  0x004C,   /* U+1F83 */
  0x004C,   /* U+1F84 */
  0x004C,   /* U+1F85 */
  0x004C,   /* U+1F86 */
  0x004C,   /* U+1F87 */
  0x0000,   /* U+1F88 */
  0x0000,   /* U+1F89 */
  0x004C,   /* U+1F8A */
  0x004C,   /* U+1F8B */
  0x004C,   /* U+1F8C */
  0x004C,   /* U+1F8D */
  0x004C,   /* U+1F8E */
  0x004C,   /* U+1F8F */
  0x0000,   /* U+1F90 */
  0x0000,   /* U+1F91 */
  0x004C,   /* U+1F92 */
  0x004C,   /* U+1F93 */
  0x004C,   /* U+1F94 */
  0x004C,   /* U+1F95 */
  0x004C,   /* U+1F96 */
  0x004C,   /* U+1F97 */
  0x0000,   /* U+1F98 */
  0x0000,   /* U+1F99 */
  0x004C,   /* U+1F9A */
  0x004C,   /* U+1F9B */
  0x004C,   /* U+1F9C */
  0x004C,   /* U+1F9D */
  0x004C,   /* U+1F9E */
  0x004C,   /* U+1F9F */
  0x0000,   /* U+1FA0 */
  0x0000,   /* U+1FA1 */
  0x004C,   /* U+1FA2 */
  0x004C,   /* U+1FA3 */
  0x004C,   /* U+1FA4 */
  0x004C,   /* U+1FA5 */
  0x004C,   /* U+1FA6 */
  0x004C,   /* U+1FA7 */
  0x0000,   /* U+1FA8 */
  0x0000,   /* U+1FA9 */
  0x004C,   /* U+1FAA */
  0x004C,   /* U+1FAB */
  0x004C,   /* U+1FAC */
  0x004C,   /* U+1FAD */
  0x004C,   /* U+1FAE */
  0x004C,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0000,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0000,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0000,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0000,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0000,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0000,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0000,   /* U+1FD2 */
  0x0000,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0000,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0000,   /* U+1FE2 */
  0x0000,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0000,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0000,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0000,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0000,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+1F00 */
  0x0000,   /* U+1F01 */
  0x0000,   /* U+1F02 */
  0x0000,   /* U+1F03 */
  0x0000,   /* U+1F04 */
  0x0000,   /* U+1F05 */
  0x0000,   /* U+1F06 */
  0x0000,   /* U+1F07 */
  0x0000,   /* U+1F08 */
  0x0000,   /* U+1F09 */
  0x0000,   /* U+1F0A */
  0x0000,   /* U+1F0B */
  0x0000,   /* U+1F0C */
  0x0000,   /* U+1F0D */
  0x0000,   /* U+1F0E */
  0x0000,   /* U+1F0F */
  0x0000,   /* U+1F10 */
  0x0000,   /* U+1F11 */
  0x0000,   /* U+1F12 */
  0x0000,   /* U+1F13 */
  0x0000,   /* U+1F14 */
  0x0000,   /* U+1F15 */
  0x0000,   /* U+1F16 */
  0x0000,   /* U+1F17 */
  0x0000,   /* U+1F18 */
  0x0000,   /* U+1F19 */
  0x0000,   /* U+1F1A */
  0x0000,   /* U+1F1B */
  0x0000,   /* U+1F1C */
  0x0000,   /* U+1F1D */
  0x0000,   /* U+1F1E */
  0x0000,   /* U+1F1F */
  0x0000,   /* U+1F20 */
  0x0000,   /* U+1F21 */
  0x0000,   /* U+1F22 */
  0x0000,   /* U+1F23 */
  0x0000,   /* U+1F24 */
  0x0000,   /* U+1F25 */
  0x0000,   /* U+1F26 */
  0x0000,   /* U+1F27 */
  0x0000,   /* U+1F28 */
  0x0000,   /* U+1F29 */
  0x0000,   /* U+1F2A */
  0x0000,   /* U+1F2B */
  0x0000,   /* U+1F2C */
  0x0000,   /* U+1F2D */
  0x0000,   /* U+1F2E */
  0x0000,   /* U+1F2F */
  0x0000,   /* U+1F30 */
  0x0000,   /* U+1F31 */
  0x0000,   /* U+1F32 */
  0x0000,   /* U+1F33 */
  0x0000,   /* U+1F34 */
  0x0000,   /* U+1F35 */
  0x0000,   /* U+1F36 */
  0x0000,   /* U+1F37 */
  0x0000,   /* U+1F38 */
  0x0000,   /* U+1F39 */
  0x0000,   /* U+1F3A */
  0x0000,   /* U+1F3B */
  0x0000,   /* U+1F3C */
  0x0000,   /* U+1F3D */
  0x0000,   /* U+1F3E */
  0x0000,   /* U+1F3F */
  0x0000,   /* U+1F40 */
  0x0000,   /* U+1F41 */
  0x0000,   /* U+1F42 */
  0x0000,   /* U+1F43 */
  0x0000,   /* U+1F44 */
  0x0000,   /* U+1F45 */
  0x0000,   /* U+1F46 */
  0x0000,   /* U+1F47 */
  0x0000,   /* U+1F48 */
  0x0000,   /* U+1F49 */
  0x0000,   /* U+1F4A */
  0x0000,   /* U+1F4B */
  0x0000,   /* U+1F4C */
  0x0000,   /* U+1F4D */
  0x0000,   /* U+1F4E */
  0x0000,   /* U+1F4F */
  0x0000,   /* U+1F50 */
  0x0000,   /* U+1F51 */
  0x0000,   /* U+1F52 */
  0x0000,   /* U+1F53 */
  0x0000,   /* U+1F54 */
  0x0000,   /* U+1F55 */
  0x0000,   /* U+1F56 */
  0x0000,   /* U+1F57 */
  0x0000,   /* U+1F58 */
  0x0000,   /* U+1F59 */
  0x0000,   /* U+1F5A */
  0x0000,   /* U+1F5B */
  0x0000,   /* U+1F5C */
  0x0000,   /* U+1F5D */
  0x0000,   /* U+1F5E */
  0x0000,   /* U+1F5F */
  0x0000,   /* U+1F60 */
  0x0000,   /* U+1F61 */
  0x0000,   /* U+1F62 */
  0x0000,   /* U+1F63 */
  0x0000,   /* U+1F64 */
  0x0000,   /* U+1F65 */
  0x0000,   /* U+1F66 */
  0x0000,   /* U+1F67 */
  0x0000,   /* U+1F68 */
  0x0000,   /* U+1F69 */
  0x0000,   /* U+1F6A */
  0x0000,   /* U+1F6B */
  0x0000,   /* U+1F6C */
  0x0000,   /* U+1F6D */
  0x0000,   /* U+1F6E */
  0x0000,   /* U+1F6F */
  0x0000,   /* U+1F70 */
  0x0000,   /* U+1F71 */
  0x0000,   /* U+1F72 */
  0x0000,   /* U+1F73 */
  0x0000,   /* U+1F74 */
  0x0000,   /* U+1F75 */
  0x0000,   /* U+1F76 */
  0x0000,   /* U+1F77 */
  0x0000,   /* U+1F78 */
  0x0000,   /* U+1F79 */
  0x0000,   /* U+1F7A */
  0x0000,   /* U+1F7B */
  0x0000,   /* U+1F7C */
  0x0000,   /* U+1F7D */
  0x0000,   /* U+1F7E */
  0x0000,   /* U+1F7F */
  0x0000,   /* U+1F80 */
  0x0000,   /* U+1F81 */
  0x0002,   /* U+1F82 */
  0x0002,   /* U+1F83 */
  0x0002,   /* U+1F84 */
  0x0002,   /* U+1F85 */
  0x0002,   /* U+1F86 */
  0x0002,   /* U+1F87 */
  0x0000,   /* U+1F88 */
  0x0000,   /* U+1F89 */
  0x0002,   /* U+1F8A */
  0x0002,   /* U+1F8B */
  0x0002,   /* U+1F8C */
  0x0002,   /* U+1F8D */
  0x0002,   /* U+1F8E */
  0x0002,   /* U+1F8F */
  0x0000,   /* U+1F90 */
  0x0000,   /* U+1F91 */
  0x0002,   /* U+1F92 */
  0x0002,   /* U+1F93 */
  0x0002,   /* U+1F94 */
  0x0002,   /* U+1F95 */
  0x0002,   /* U+1F96 */
  0x0002,   /* U+1F97 */
  0x0000,   /* U+1F98 */
  0x0000,   /* U+1F99 */
  0x0002,   /* U+1F9A */
  0x0002,   /* U+1F9B */
  0x0002,   /* U+1F9C */
  0x0002,   /* U+1F9D */
  0x0002,   /* U+1F9E */
  0x0002,   /* U+1F9F */
  0x0000,   /* U+1FA0 */
  0x0000,   /* U+1FA1 */
  0x0002,   /* U+1FA2 */
  0x0002,   /* U+1FA3 */
  0x0002,   /* U+1FA4 */
  0x0002,   /* U+1FA5 */
  0x0002,   /* U+1FA6 */
  0x0002,   /* U+1FA7 */
  0x0000,   /* U+1FA8 */
  0x0000,   /* U+1FA9 */
  0x0002,   /* U+1FAA */
  0x0002,   /* U+1FAB */
  0x0002,   /* U+1FAC */
  0x0002,   /* U+1FAD */
  0x0002,   /* U+1FAE */
  0x0002,   /* U+1FAF */
  0x0000,   /* U+1FB0 */
  0x0000,   /* U+1FB1 */
  0x0000,   /* U+1FB2 */
  0x0000,   /* U+1FB3 */
  0x0000,   /* U+1FB4 */
  0x0000,   /* U+1FB5 */
  0x0000,   /* U+1FB6 */
  0x0000,   /* U+1FB7 */
  0x0000,   /* U+1FB8 */
  0x0000,   /* U+1FB9 */
  0x0000,   /* U+1FBA */
  0x0000,   /* U+1FBB */
  0x0000,   /* U+1FBC */
  0x0000,   /* U+1FBD */
  0x0000,   /* U+1FBE */
  0x0000,   /* U+1FBF */
  0x0000,   /* U+1FC0 */
  0x0000,   /* U+1FC1 */
  0x0000,   /* U+1FC2 */
  0x0000,   /* U+1FC3 */
  0x0000,   /* U+1FC4 */
  0x0000,   /* U+1FC5 */
  0x0000,   /* U+1FC6 */
  0x0000,   /* U+1FC7 */
  0x0000,   /* U+1FC8 */
  0x0000,   /* U+1FC9 */
  0x0000,   /* U+1FCA */
  0x0000,   /* U+1FCB */
  0x0000,   /* U+1FCC */
  0x0000,   /* U+1FCD */
  0x0000,   /* U+1FCE */
  0x0000,   /* U+1FCF */
  0x0000,   /* U+1FD0 */
  0x0000,   /* U+1FD1 */
  0x0000,   /* U+1FD2 */
  0x0000,   /* U+1FD3 */
  0x0000,   /* U+1FD4 */
  0x0000,   /* U+1FD5 */
  0x0000,   /* U+1FD6 */
  0x0000,   /* U+1FD7 */
  0x0000,   /* U+1FD8 */
  0x0000,   /* U+1FD9 */
  0x0000,   /* U+1FDA */
  0x0000,   /* U+1FDB */
  0x0000,   /* U+1FDC */
  0x0000,   /* U+1FDD */
  0x0000,   /* U+1FDE */
  0x0000,   /* U+1FDF */
  0x0000,   /* U+1FE0 */
  0x0000,   /* U+1FE1 */
  0x0000,   /* U+1FE2 */
  0x0000,   /* U+1FE3 */
  0x0000,   /* U+1FE4 */
  0x0000,   /* U+1FE5 */
  0x0000,   /* U+1FE6 */
  0x0000,   /* U+1FE7 */
  0x0000,   /* U+1FE8 */
  0x0000,   /* U+1FE9 */
  0x0000,   /* U+1FEA */
  0x0000,   /* U+1FEB */
  0x0000,   /* U+1FEC */
  0x0000,   /* U+1FED */
  0x0000,   /* U+1FEE */
  0x0000,   /* U+1FEF */
  0x0000,   /* U+1FF0 */
  0x0000,   /* U+1FF1 */
  0x0000,   /* U+1FF2 */
  0x0000,   /* U+1FF3 */
  0x0000,   /* U+1FF4 */
  0x0000,   /* U+1FF5 */
  0x0000,   /* U+1FF6 */
  0x0000,   /* U+1FF7 */
  0x0000,   /* U+1FF8 */
  0x0000,   /* U+1FF9 */
  0x0000,   /* U+1FFA */
  0x0000,   /* U+1FFB */
  0x0000,   /* U+1FFC */
  0x0000,   /* U+1FFD */
  0x0000,   /* U+1FFE */
  0x0000,   /* U+1FFF */
};

uint16 uca900_p020[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 3, 1, 2, 3, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 4, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0209,   /* U+2000 */
  0x0209,   /* U+2001 */
  0x0209,   /* U+2002 */
  0x0209,   /* U+2003 */
  0x0209,   /* U+2004 */
  0x0209,   /* U+2005 */
  0x0209,   /* U+2006 */
  0x0209,   /* U+2007 */
  0x0209,   /* U+2008 */
  0x0209,   /* U+2009 */
  0x0209,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0213,   /* U+2010 */
  0x0213,   /* U+2011 */
  0x0214,   /* U+2012 */
  0x0215,   /* U+2013 */
  0x0216,   /* U+2014 */
  0x0217,   /* U+2015 */
  0x0383,   /* U+2016 */
  0x020C,   /* U+2017 */
  0x0306,   /* U+2018 */
  0x0307,   /* U+2019 */
  0x0308,   /* U+201A */
  0x0309,   /* U+201B */
  0x030D,   /* U+201C */
  0x030E,   /* U+201D */
  0x030F,   /* U+201E */
  0x0310,   /* U+201F */
  0x039F,   /* U+2020 */
  0x03A0,   /* U+2021 */
  0x03A4,   /* U+2022 */
  0x03A5,   /* U+2023 */
  0x0277,   /* U+2024 */
  0x0277,   /* U+2025 */
  0x0277,   /* U+2026 */
  0x03A6,   /* U+2027 */
  0x0207,   /* U+2028 */
  0x0208,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0209,   /* U+202F */
  0x039B,   /* U+2030 */
  0x039D,   /* U+2031 */
  0x03AA,   /* U+2032 */
  0x03AA,   /* U+2033 */
  0x03AA,   /* U+2034 */
  0x03AB,   /* U+2035 */
  0x03AB,   /* U+2036 */
  0x03AB,   /* U+2037 */
  0x03AE,   /* U+2038 */
  0x030A,   /* U+2039 */
  0x030B,   /* U+203A */
  0x03AF,   /* U+203B */
  0x0260,   /* U+203C */
  0x0275,   /* U+203D */
  0x020A,   /* U+203E */
  0x03B0,   /* U+203F */
  0x03B2,   /* U+2040 */
  0x03B4,   /* U+2041 */
  0x03B5,   /* U+2042 */
  0x03A7,   /* U+2043 */
  0x0626,   /* U+2044 */
  0x0323,   /* U+2045 */
  0x0324,   /* U+2046 */
  0x0266,   /* U+2047 */
  0x0266,   /* U+2048 */
  0x0260,   /* U+2049 */
  0x0397,   /* U+204A */
  0x038C,   /* U+204B */
  0x03A8,   /* U+204C */
  0x03A9,   /* U+204D */
  0x0390,   /* U+204E */
  0x0236,   /* U+204F */
  0x03B3,   /* U+2050 */
  0x0391,   /* U+2051 */
  0x0622,   /* U+2052 */
  0x021A,   /* U+2053 */
  0x03B1,   /* U+2054 */
  0x02E8,   /* U+2055 */
  0x02E9,   /* U+2056 */
  0x03AA,   /* U+2057 */
  0x02EA,   /* U+2058 */
  0x02EB,   /* U+2059 */
  0x02EC,   /* U+205A */
  0x02ED,   /* U+205B */
  0x02EE,   /* U+205C */
  0x02EF,   /* U+205D */
  0x02F0,   /* U+205E */
  0x0209,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0xFBC0,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x1C3D,   /* U+2070 */
  0x1D32,   /* U+2071 */
  0xFBC0,   /* U+2072 */
  0xFBC0,   /* U+2073 */
  0x1C41,   /* U+2074 */
  0x1C42,   /* U+2075 */
  0x1C43,   /* U+2076 */
  0x1C44,   /* U+2077 */
  0x1C45,   /* U+2078 */
  0x1C46,   /* U+2079 */
  0x0616,   /* U+207A */
  0x0621,   /* U+207B */
  0x061B,   /* U+207C */
  0x0317,   /* U+207D */
  0x0318,   /* U+207E */
  0x1DB9,   /* U+207F */
  0x1C3D,   /* U+2080 */
  0x1C3E,   /* U+2081 */
  0x1C3F,   /* U+2082 */
  0x1C40,   /* U+2083 */
  0x1C41,   /* U+2084 */
  0x1C42,   /* U+2085 */
  0x1C43,   /* U+2086 */
  0x1C44,   /* U+2087 */
  0x1C45,   /* U+2088 */
  0x1C46,   /* U+2089 */
  0x0616,   /* U+208A */
  0x0621,   /* U+208B */
  0x061B,   /* U+208C */
  0x0317,   /* U+208D */
  0x0318,   /* U+208E */
  0xFBC0,   /* U+208F */
  0x1C47,   /* U+2090 */
  0x1CAA,   /* U+2091 */
  0x1DDD,   /* U+2092 */
  0x1EFF,   /* U+2093 */
  0x1CBD,   /* U+2094 */
  0x1D18,   /* U+2095 */
  0x1D65,   /* U+2096 */
  0x1D77,   /* U+2097 */
  0x1DAA,   /* U+2098 */
  0x1DB9,   /* U+2099 */
  0x1E0C,   /* U+209A */
  0x1E71,   /* U+209B */
  0x1E95,   /* U+209C */
  0xFBC0,   /* U+209D */
  0xFBC0,   /* U+209E */
  0xFBC0,   /* U+209F */
  0x1C1F,   /* U+20A0 */
  0x1C20,   /* U+20A1 */
  0x1C21,   /* U+20A2 */
  0x1C22,   /* U+20A3 */
  0x1C23,   /* U+20A4 */
  0x1C24,   /* U+20A5 */
  0x1C25,   /* U+20A6 */
  0x1C26,   /* U+20A7 */
  0x1E33,   /* U+20A8 */
  0x1C27,   /* U+20A9 */
  0x1C28,   /* U+20AA */
  0x1C29,   /* U+20AB */
  0x1C2A,   /* U+20AC */
  0x1C2B,   /* U+20AD */
  0x1C2C,   /* U+20AE */
  0x1C2D,   /* U+20AF */
  0x1C2E,   /* U+20B0 */
  0x1C2F,   /* U+20B1 */
  0x1C30,   /* U+20B2 */
  0x1C31,   /* U+20B3 */
  0x1C32,   /* U+20B4 */
  0x1C33,   /* U+20B5 */
  0x1C34,   /* U+20B6 */
  0x1C35,   /* U+20B7 */
  0x1C36,   /* U+20B8 */
  0x1C37,   /* U+20B9 */
  0x1C38,   /* U+20BA */
  0x1C39,   /* U+20BB */
  0x1C3A,   /* U+20BC */
  0x1C3B,   /* U+20BD */
  0x1C3C,   /* U+20BE */
  0xFBC0,   /* U+20BF */
  0xFBC0,   /* U+20C0 */
  0xFBC0,   /* U+20C1 */
  0xFBC0,   /* U+20C2 */
  0xFBC0,   /* U+20C3 */
  0xFBC0,   /* U+20C4 */
  0xFBC0,   /* U+20C5 */
  0xFBC0,   /* U+20C6 */
  0xFBC0,   /* U+20C7 */
  0xFBC0,   /* U+20C8 */
  0xFBC0,   /* U+20C9 */
  0xFBC0,   /* U+20CA */
  0xFBC0,   /* U+20CB */
  0xFBC0,   /* U+20CC */
  0xFBC0,   /* U+20CD */
  0xFBC0,   /* U+20CE */
  0xFBC0,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0xFBC0,   /* U+20F1 */
  0xFBC0,   /* U+20F2 */
  0xFBC0,   /* U+20F3 */
  0xFBC0,   /* U+20F4 */
  0xFBC0,   /* U+20F5 */
  0xFBC0,   /* U+20F6 */
  0xFBC0,   /* U+20F7 */
  0xFBC0,   /* U+20F8 */
  0xFBC0,   /* U+20F9 */
  0xFBC0,   /* U+20FA */
  0xFBC0,   /* U+20FB */
  0xFBC0,   /* U+20FC */
  0xFBC0,   /* U+20FD */
  0xFBC0,   /* U+20FE */
  0xFBC0,   /* U+20FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2000 */
  0x0020,   /* U+2001 */
  0x0020,   /* U+2002 */
  0x0020,   /* U+2003 */
  0x0020,   /* U+2004 */
  0x0020,   /* U+2005 */
  0x0020,   /* U+2006 */
  0x0020,   /* U+2007 */
  0x0020,   /* U+2008 */
  0x0020,   /* U+2009 */
  0x0020,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0020,   /* U+2010 */
  0x0020,   /* U+2011 */
  0x0020,   /* U+2012 */
  0x0020,   /* U+2013 */
  0x0020,   /* U+2014 */
  0x0020,   /* U+2015 */
  0x0020,   /* U+2016 */
  0x0020,   /* U+2017 */
  0x0020,   /* U+2018 */
  0x0020,   /* U+2019 */
  0x0020,   /* U+201A */
  0x0020,   /* U+201B */
  0x0020,   /* U+201C */
  0x0020,   /* U+201D */
  0x0020,   /* U+201E */
  0x0020,   /* U+201F */
  0x0020,   /* U+2020 */
  0x0020,   /* U+2021 */
  0x0020,   /* U+2022 */
  0x0020,   /* U+2023 */
  0x0020,   /* U+2024 */
  0x0020,   /* U+2025 */
  0x0020,   /* U+2026 */
  0x0020,   /* U+2027 */
  0x0020,   /* U+2028 */
  0x0020,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0020,   /* U+202F */
  0x0020,   /* U+2030 */
  0x0020,   /* U+2031 */
  0x0020,   /* U+2032 */
  0x0020,   /* U+2033 */
  0x0020,   /* U+2034 */
  0x0020,   /* U+2035 */
  0x0020,   /* U+2036 */
  0x0020,   /* U+2037 */
  0x0020,   /* U+2038 */
  0x0020,   /* U+2039 */
  0x0020,   /* U+203A */
  0x0020,   /* U+203B */
  0x0020,   /* U+203C */
  0x0020,   /* U+203D */
  0x0020,   /* U+203E */
  0x0020,   /* U+203F */
  0x0020,   /* U+2040 */
  0x0020,   /* U+2041 */
  0x0020,   /* U+2042 */
  0x0020,   /* U+2043 */
  0x0020,   /* U+2044 */
  0x0020,   /* U+2045 */
  0x0020,   /* U+2046 */
  0x0020,   /* U+2047 */
  0x0020,   /* U+2048 */
  0x0020,   /* U+2049 */
  0x0020,   /* U+204A */
  0x0020,   /* U+204B */
  0x0020,   /* U+204C */
  0x0020,   /* U+204D */
  0x0020,   /* U+204E */
  0x0020,   /* U+204F */
  0x0020,   /* U+2050 */
  0x0020,   /* U+2051 */
  0x0020,   /* U+2052 */
  0x0020,   /* U+2053 */
  0x0020,   /* U+2054 */
  0x0020,   /* U+2055 */
  0x0020,   /* U+2056 */
  0x0020,   /* U+2057 */
  0x0020,   /* U+2058 */
  0x0020,   /* U+2059 */
  0x0020,   /* U+205A */
  0x0020,   /* U+205B */
  0x0020,   /* U+205C */
  0x0020,   /* U+205D */
  0x0020,   /* U+205E */
  0x0020,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0020,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0020,   /* U+2070 */
  0x0020,   /* U+2071 */
  0x0020,   /* U+2072 */
  0x0020,   /* U+2073 */
  0x0020,   /* U+2074 */
  0x0020,   /* U+2075 */
  0x0020,   /* U+2076 */
  0x0020,   /* U+2077 */
  0x0020,   /* U+2078 */
  0x0020,   /* U+2079 */
  0x0020,   /* U+207A */
  0x0020,   /* U+207B */
  0x0020,   /* U+207C */
  0x0020,   /* U+207D */
  0x0020,   /* U+207E */
  0x0020,   /* U+207F */
  0x0020,   /* U+2080 */
  0x0020,   /* U+2081 */
  0x0020,   /* U+2082 */
  0x0020,   /* U+2083 */
  0x0020,   /* U+2084 */
  0x0020,   /* U+2085 */
  0x0020,   /* U+2086 */
  0x0020,   /* U+2087 */
  0x0020,   /* U+2088 */
  0x0020,   /* U+2089 */
  0x0020,   /* U+208A */
  0x0020,   /* U+208B */
  0x0020,   /* U+208C */
  0x0020,   /* U+208D */
  0x0020,   /* U+208E */
  0x0020,   /* U+208F */
  0x0020,   /* U+2090 */
  0x0020,   /* U+2091 */
  0x0020,   /* U+2092 */
  0x0020,   /* U+2093 */
  0x0020,   /* U+2094 */
  0x0020,   /* U+2095 */
  0x0020,   /* U+2096 */
  0x0020,   /* U+2097 */
  0x0020,   /* U+2098 */
  0x0020,   /* U+2099 */
  0x0020,   /* U+209A */
  0x0020,   /* U+209B */
  0x0020,   /* U+209C */
  0x0020,   /* U+209D */
  0x0020,   /* U+209E */
  0x0020,   /* U+209F */
  0x0020,   /* U+20A0 */
  0x0020,   /* U+20A1 */
  0x0020,   /* U+20A2 */
  0x0020,   /* U+20A3 */
  0x0020,   /* U+20A4 */
  0x0020,   /* U+20A5 */
  0x0020,   /* U+20A6 */
  0x0020,   /* U+20A7 */
  0x0020,   /* U+20A8 */
  0x0020,   /* U+20A9 */
  0x0020,   /* U+20AA */
  0x0020,   /* U+20AB */
  0x0020,   /* U+20AC */
  0x0020,   /* U+20AD */
  0x0020,   /* U+20AE */
  0x0020,   /* U+20AF */
  0x0020,   /* U+20B0 */
  0x0020,   /* U+20B1 */
  0x0020,   /* U+20B2 */
  0x0020,   /* U+20B3 */
  0x0020,   /* U+20B4 */
  0x0020,   /* U+20B5 */
  0x0020,   /* U+20B6 */
  0x0020,   /* U+20B7 */
  0x0020,   /* U+20B8 */
  0x0020,   /* U+20B9 */
  0x0020,   /* U+20BA */
  0x0020,   /* U+20BB */
  0x0020,   /* U+20BC */
  0x0020,   /* U+20BD */
  0x0020,   /* U+20BE */
  0x0020,   /* U+20BF */
  0x0020,   /* U+20C0 */
  0x0020,   /* U+20C1 */
  0x0020,   /* U+20C2 */
  0x0020,   /* U+20C3 */
  0x0020,   /* U+20C4 */
  0x0020,   /* U+20C5 */
  0x0020,   /* U+20C6 */
  0x0020,   /* U+20C7 */
  0x0020,   /* U+20C8 */
  0x0020,   /* U+20C9 */
  0x0020,   /* U+20CA */
  0x0020,   /* U+20CB */
  0x0020,   /* U+20CC */
  0x0020,   /* U+20CD */
  0x0020,   /* U+20CE */
  0x0020,   /* U+20CF */
  0x0101,   /* U+20D0 */
  0x0102,   /* U+20D1 */
  0x0103,   /* U+20D2 */
  0x0103,   /* U+20D3 */
  0x0104,   /* U+20D4 */
  0x0105,   /* U+20D5 */
  0x0106,   /* U+20D6 */
  0x0107,   /* U+20D7 */
  0x0035,   /* U+20D8 */
  0x0035,   /* U+20D9 */
  0x0035,   /* U+20DA */
  0x0108,   /* U+20DB */
  0x0109,   /* U+20DC */
  0x0036,   /* U+20DD */
  0x0036,   /* U+20DE */
  0x0036,   /* U+20DF */
  0x0036,   /* U+20E0 */
  0x010A,   /* U+20E1 */
  0x0036,   /* U+20E2 */
  0x0036,   /* U+20E3 */
  0x0036,   /* U+20E4 */
  0x0035,   /* U+20E5 */
  0x010B,   /* U+20E6 */
  0x010C,   /* U+20E7 */
  0x010D,   /* U+20E8 */
  0x010E,   /* U+20E9 */
  0x0035,   /* U+20EA */
  0x0035,   /* U+20EB */
  0x0034,   /* U+20EC */
  0x0034,   /* U+20ED */
  0x0034,   /* U+20EE */
  0x0034,   /* U+20EF */
  0x0033,   /* U+20F0 */
  0x0020,   /* U+20F1 */
  0x0020,   /* U+20F2 */
  0x0020,   /* U+20F3 */
  0x0020,   /* U+20F4 */
  0x0020,   /* U+20F5 */
  0x0020,   /* U+20F6 */
  0x0020,   /* U+20F7 */
  0x0020,   /* U+20F8 */
  0x0020,   /* U+20F9 */
  0x0020,   /* U+20FA */
  0x0020,   /* U+20FB */
  0x0020,   /* U+20FC */
  0x0020,   /* U+20FD */
  0x0020,   /* U+20FE */
  0x0020,   /* U+20FF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+2000 */
  0x0004,   /* U+2001 */
  0x0004,   /* U+2002 */
  0x0004,   /* U+2003 */
  0x0004,   /* U+2004 */
  0x0004,   /* U+2005 */
  0x0004,   /* U+2006 */
  0x001B,   /* U+2007 */
  0x0004,   /* U+2008 */
  0x0004,   /* U+2009 */
  0x0004,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0002,   /* U+2010 */
  0x001B,   /* U+2011 */
  0x0002,   /* U+2012 */
  0x0002,   /* U+2013 */
  0x0002,   /* U+2014 */
  0x0002,   /* U+2015 */
  0x0002,   /* U+2016 */
  0x0002,   /* U+2017 */
  0x0002,   /* U+2018 */
  0x0002,   /* U+2019 */
  0x0002,   /* U+201A */
  0x0002,   /* U+201B */
  0x0002,   /* U+201C */
  0x0002,   /* U+201D */
  0x0002,   /* U+201E */
  0x0002,   /* U+201F */
  0x0002,   /* U+2020 */
  0x0002,   /* U+2021 */
  0x0002,   /* U+2022 */
  0x0002,   /* U+2023 */
  0x0004,   /* U+2024 */
  0x0004,   /* U+2025 */
  0x0004,   /* U+2026 */
  0x0002,   /* U+2027 */
  0x0002,   /* U+2028 */
  0x0002,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x001B,   /* U+202F */
  0x0002,   /* U+2030 */
  0x0002,   /* U+2031 */
  0x0002,   /* U+2032 */
  0x0004,   /* U+2033 */
  0x0004,   /* U+2034 */
  0x0002,   /* U+2035 */
  0x0004,   /* U+2036 */
  0x0004,   /* U+2037 */
  0x0002,   /* U+2038 */
  0x0002,   /* U+2039 */
  0x0002,   /* U+203A */
  0x0002,   /* U+203B */
  0x0004,   /* U+203C */
  0x0002,   /* U+203D */
  0x0002,   /* U+203E */
  0x0002,   /* U+203F */
  0x0002,   /* U+2040 */
  0x0002,   /* U+2041 */
  0x0002,   /* U+2042 */
  0x0002,   /* U+2043 */
  0x0002,   /* U+2044 */
  0x0002,   /* U+2045 */
  0x0002,   /* U+2046 */
  0x0004,   /* U+2047 */
  0x0004,   /* U+2048 */
  0x0004,   /* U+2049 */
  0x0002,   /* U+204A */
  0x0002,   /* U+204B */
  0x0002,   /* U+204C */
  0x0002,   /* U+204D */
  0x0002,   /* U+204E */
  0x0002,   /* U+204F */
  0x0002,   /* U+2050 */
  0x0002,   /* U+2051 */
  0x0002,   /* U+2052 */
  0x0002,   /* U+2053 */
  0x0002,   /* U+2054 */
  0x0002,   /* U+2055 */
  0x0002,   /* U+2056 */
  0x0004,   /* U+2057 */
  0x0002,   /* U+2058 */
  0x0002,   /* U+2059 */
  0x0002,   /* U+205A */
  0x0002,   /* U+205B */
  0x0002,   /* U+205C */
  0x0002,   /* U+205D */
  0x0002,   /* U+205E */
  0x0004,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0002,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0014,   /* U+2070 */
  0x0014,   /* U+2071 */
  0x0002,   /* U+2072 */
  0x0002,   /* U+2073 */
  0x0014,   /* U+2074 */
  0x0014,   /* U+2075 */
  0x0014,   /* U+2076 */
  0x0014,   /* U+2077 */
  0x0014,   /* U+2078 */
  0x0014,   /* U+2079 */
  0x0014,   /* U+207A */
  0x0014,   /* U+207B */
  0x0014,   /* U+207C */
  0x0014,   /* U+207D */
  0x0014,   /* U+207E */
  0x0014,   /* U+207F */
  0x0015,   /* U+2080 */
  0x0015,   /* U+2081 */
  0x0015,   /* U+2082 */
  0x0015,   /* U+2083 */
  0x0015,   /* U+2084 */
  0x0015,   /* U+2085 */
  0x0015,   /* U+2086 */
  0x0015,   /* U+2087 */
  0x0015,   /* U+2088 */
  0x0015,   /* U+2089 */
  0x0015,   /* U+208A */
  0x0015,   /* U+208B */
  0x0015,   /* U+208C */
  0x0015,   /* U+208D */
  0x0015,   /* U+208E */
  0x0002,   /* U+208F */
  0x0015,   /* U+2090 */
  0x0015,   /* U+2091 */
  0x0015,   /* U+2092 */
  0x0015,   /* U+2093 */
  0x0015,   /* U+2094 */
  0x0015,   /* U+2095 */
  0x0015,   /* U+2096 */
  0x0015,   /* U+2097 */
  0x0015,   /* U+2098 */
  0x0015,   /* U+2099 */
  0x0015,   /* U+209A */
  0x0015,   /* U+209B */
  0x0015,   /* U+209C */
  0x0002,   /* U+209D */
  0x0002,   /* U+209E */
  0x0002,   /* U+209F */
  0x0002,   /* U+20A0 */
  0x0002,   /* U+20A1 */
  0x0002,   /* U+20A2 */
  0x0002,   /* U+20A3 */
  0x0002,   /* U+20A4 */
  0x0002,   /* U+20A5 */
  0x0002,   /* U+20A6 */
  0x0002,   /* U+20A7 */
  0x000A,   /* U+20A8 */
  0x0002,   /* U+20A9 */
  0x0002,   /* U+20AA */
  0x0002,   /* U+20AB */
  0x0002,   /* U+20AC */
  0x0002,   /* U+20AD */
  0x0002,   /* U+20AE */
  0x0002,   /* U+20AF */
  0x0002,   /* U+20B0 */
  0x0002,   /* U+20B1 */
  0x0002,   /* U+20B2 */
  0x0002,   /* U+20B3 */
  0x0002,   /* U+20B4 */
  0x0002,   /* U+20B5 */
  0x0002,   /* U+20B6 */
  0x0002,   /* U+20B7 */
  0x0002,   /* U+20B8 */
  0x0002,   /* U+20B9 */
  0x0002,   /* U+20BA */
  0x0002,   /* U+20BB */
  0x0002,   /* U+20BC */
  0x0002,   /* U+20BD */
  0x0002,   /* U+20BE */
  0x0002,   /* U+20BF */
  0x0002,   /* U+20C0 */
  0x0002,   /* U+20C1 */
  0x0002,   /* U+20C2 */
  0x0002,   /* U+20C3 */
  0x0002,   /* U+20C4 */
  0x0002,   /* U+20C5 */
  0x0002,   /* U+20C6 */
  0x0002,   /* U+20C7 */
  0x0002,   /* U+20C8 */
  0x0002,   /* U+20C9 */
  0x0002,   /* U+20CA */
  0x0002,   /* U+20CB */
  0x0002,   /* U+20CC */
  0x0002,   /* U+20CD */
  0x0002,   /* U+20CE */
  0x0002,   /* U+20CF */
  0x0002,   /* U+20D0 */
  0x0002,   /* U+20D1 */
  0x0002,   /* U+20D2 */
  0x0002,   /* U+20D3 */
  0x0002,   /* U+20D4 */
  0x0002,   /* U+20D5 */
  0x0002,   /* U+20D6 */
  0x0002,   /* U+20D7 */
  0x0002,   /* U+20D8 */
  0x0002,   /* U+20D9 */
  0x0002,   /* U+20DA */
  0x0002,   /* U+20DB */
  0x0002,   /* U+20DC */
  0x0002,   /* U+20DD */
  0x0002,   /* U+20DE */
  0x0002,   /* U+20DF */
  0x0002,   /* U+20E0 */
  0x0002,   /* U+20E1 */
  0x0002,   /* U+20E2 */
  0x0002,   /* U+20E3 */
  0x0002,   /* U+20E4 */
  0x0002,   /* U+20E5 */
  0x0002,   /* U+20E6 */
  0x0002,   /* U+20E7 */
  0x0002,   /* U+20E8 */
  0x0002,   /* U+20E9 */
  0x0002,   /* U+20EA */
  0x0002,   /* U+20EB */
  0x0002,   /* U+20EC */
  0x0002,   /* U+20ED */
  0x0002,   /* U+20EE */
  0x0002,   /* U+20EF */
  0x0002,   /* U+20F0 */
  0x0002,   /* U+20F1 */
  0x0002,   /* U+20F2 */
  0x0002,   /* U+20F3 */
  0x0002,   /* U+20F4 */
  0x0002,   /* U+20F5 */
  0x0002,   /* U+20F6 */
  0x0002,   /* U+20F7 */
  0x0002,   /* U+20F8 */
  0x0002,   /* U+20F9 */
  0x0002,   /* U+20FA */
  0x0002,   /* U+20FB */
  0x0002,   /* U+20FC */
  0x0002,   /* U+20FD */
  0x0002,   /* U+20FE */
  0x0002,   /* U+20FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0277,   /* U+2025 */
  0x0277,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x03AA,   /* U+2033 */
  0x03AA,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x03AB,   /* U+2036 */
  0x03AB,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0260,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0266,   /* U+2047 */
  0x0260,   /* U+2048 */
  0x0266,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x03AA,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0xA065,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0xA072,   /* U+2072 */
  0xA073,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0xA08F,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0xA09D,   /* U+209D */
  0xA09E,   /* U+209E */
  0xA09F,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x1E71,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0xA0BF,   /* U+20BF */
  0xA0C0,   /* U+20C0 */
  0xA0C1,   /* U+20C1 */
  0xA0C2,   /* U+20C2 */
  0xA0C3,   /* U+20C3 */
  0xA0C4,   /* U+20C4 */
  0xA0C5,   /* U+20C5 */
  0xA0C6,   /* U+20C6 */
  0xA0C7,   /* U+20C7 */
  0xA0C8,   /* U+20C8 */
  0xA0C9,   /* U+20C9 */
  0xA0CA,   /* U+20CA */
  0xA0CB,   /* U+20CB */
  0xA0CC,   /* U+20CC */
  0xA0CD,   /* U+20CD */
  0xA0CE,   /* U+20CE */
  0xA0CF,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0xA0F1,   /* U+20F1 */
  0xA0F2,   /* U+20F2 */
  0xA0F3,   /* U+20F3 */
  0xA0F4,   /* U+20F4 */
  0xA0F5,   /* U+20F5 */
  0xA0F6,   /* U+20F6 */
  0xA0F7,   /* U+20F7 */
  0xA0F8,   /* U+20F8 */
  0xA0F9,   /* U+20F9 */
  0xA0FA,   /* U+20FA */
  0xA0FB,   /* U+20FB */
  0xA0FC,   /* U+20FC */
  0xA0FD,   /* U+20FD */
  0xA0FE,   /* U+20FE */
  0xA0FF,   /* U+20FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0020,   /* U+2025 */
  0x0020,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0020,   /* U+2033 */
  0x0020,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0020,   /* U+2036 */
  0x0020,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0020,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0020,   /* U+2047 */
  0x0020,   /* U+2048 */
  0x0020,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0020,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0020,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0004,   /* U+2025 */
  0x0004,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0004,   /* U+2033 */
  0x0004,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0004,   /* U+2036 */
  0x0004,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0004,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0004,   /* U+2047 */
  0x0004,   /* U+2048 */
  0x0004,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0004,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0004,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0277,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x03AA,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x03AB,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x03AA,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0020,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x0020,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x0020,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0020,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0004,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x0004,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x0004,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0004,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0000,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x0000,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x0000,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x03AA,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0000,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x0000,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x0000,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0020,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+2000 */
  0x0000,   /* U+2001 */
  0x0000,   /* U+2002 */
  0x0000,   /* U+2003 */
  0x0000,   /* U+2004 */
  0x0000,   /* U+2005 */
  0x0000,   /* U+2006 */
  0x0000,   /* U+2007 */
  0x0000,   /* U+2008 */
  0x0000,   /* U+2009 */
  0x0000,   /* U+200A */
  0x0000,   /* U+200B */
  0x0000,   /* U+200C */
  0x0000,   /* U+200D */
  0x0000,   /* U+200E */
  0x0000,   /* U+200F */
  0x0000,   /* U+2010 */
  0x0000,   /* U+2011 */
  0x0000,   /* U+2012 */
  0x0000,   /* U+2013 */
  0x0000,   /* U+2014 */
  0x0000,   /* U+2015 */
  0x0000,   /* U+2016 */
  0x0000,   /* U+2017 */
  0x0000,   /* U+2018 */
  0x0000,   /* U+2019 */
  0x0000,   /* U+201A */
  0x0000,   /* U+201B */
  0x0000,   /* U+201C */
  0x0000,   /* U+201D */
  0x0000,   /* U+201E */
  0x0000,   /* U+201F */
  0x0000,   /* U+2020 */
  0x0000,   /* U+2021 */
  0x0000,   /* U+2022 */
  0x0000,   /* U+2023 */
  0x0000,   /* U+2024 */
  0x0000,   /* U+2025 */
  0x0000,   /* U+2026 */
  0x0000,   /* U+2027 */
  0x0000,   /* U+2028 */
  0x0000,   /* U+2029 */
  0x0000,   /* U+202A */
  0x0000,   /* U+202B */
  0x0000,   /* U+202C */
  0x0000,   /* U+202D */
  0x0000,   /* U+202E */
  0x0000,   /* U+202F */
  0x0000,   /* U+2030 */
  0x0000,   /* U+2031 */
  0x0000,   /* U+2032 */
  0x0000,   /* U+2033 */
  0x0000,   /* U+2034 */
  0x0000,   /* U+2035 */
  0x0000,   /* U+2036 */
  0x0000,   /* U+2037 */
  0x0000,   /* U+2038 */
  0x0000,   /* U+2039 */
  0x0000,   /* U+203A */
  0x0000,   /* U+203B */
  0x0000,   /* U+203C */
  0x0000,   /* U+203D */
  0x0000,   /* U+203E */
  0x0000,   /* U+203F */
  0x0000,   /* U+2040 */
  0x0000,   /* U+2041 */
  0x0000,   /* U+2042 */
  0x0000,   /* U+2043 */
  0x0000,   /* U+2044 */
  0x0000,   /* U+2045 */
  0x0000,   /* U+2046 */
  0x0000,   /* U+2047 */
  0x0000,   /* U+2048 */
  0x0000,   /* U+2049 */
  0x0000,   /* U+204A */
  0x0000,   /* U+204B */
  0x0000,   /* U+204C */
  0x0000,   /* U+204D */
  0x0000,   /* U+204E */
  0x0000,   /* U+204F */
  0x0000,   /* U+2050 */
  0x0000,   /* U+2051 */
  0x0000,   /* U+2052 */
  0x0000,   /* U+2053 */
  0x0000,   /* U+2054 */
  0x0000,   /* U+2055 */
  0x0000,   /* U+2056 */
  0x0004,   /* U+2057 */
  0x0000,   /* U+2058 */
  0x0000,   /* U+2059 */
  0x0000,   /* U+205A */
  0x0000,   /* U+205B */
  0x0000,   /* U+205C */
  0x0000,   /* U+205D */
  0x0000,   /* U+205E */
  0x0000,   /* U+205F */
  0x0000,   /* U+2060 */
  0x0000,   /* U+2061 */
  0x0000,   /* U+2062 */
  0x0000,   /* U+2063 */
  0x0000,   /* U+2064 */
  0x0000,   /* U+2065 */
  0x0000,   /* U+2066 */
  0x0000,   /* U+2067 */
  0x0000,   /* U+2068 */
  0x0000,   /* U+2069 */
  0x0000,   /* U+206A */
  0x0000,   /* U+206B */
  0x0000,   /* U+206C */
  0x0000,   /* U+206D */
  0x0000,   /* U+206E */
  0x0000,   /* U+206F */
  0x0000,   /* U+2070 */
  0x0000,   /* U+2071 */
  0x0000,   /* U+2072 */
  0x0000,   /* U+2073 */
  0x0000,   /* U+2074 */
  0x0000,   /* U+2075 */
  0x0000,   /* U+2076 */
  0x0000,   /* U+2077 */
  0x0000,   /* U+2078 */
  0x0000,   /* U+2079 */
  0x0000,   /* U+207A */
  0x0000,   /* U+207B */
  0x0000,   /* U+207C */
  0x0000,   /* U+207D */
  0x0000,   /* U+207E */
  0x0000,   /* U+207F */
  0x0000,   /* U+2080 */
  0x0000,   /* U+2081 */
  0x0000,   /* U+2082 */
  0x0000,   /* U+2083 */
  0x0000,   /* U+2084 */
  0x0000,   /* U+2085 */
  0x0000,   /* U+2086 */
  0x0000,   /* U+2087 */
  0x0000,   /* U+2088 */
  0x0000,   /* U+2089 */
  0x0000,   /* U+208A */
  0x0000,   /* U+208B */
  0x0000,   /* U+208C */
  0x0000,   /* U+208D */
  0x0000,   /* U+208E */
  0x0000,   /* U+208F */
  0x0000,   /* U+2090 */
  0x0000,   /* U+2091 */
  0x0000,   /* U+2092 */
  0x0000,   /* U+2093 */
  0x0000,   /* U+2094 */
  0x0000,   /* U+2095 */
  0x0000,   /* U+2096 */
  0x0000,   /* U+2097 */
  0x0000,   /* U+2098 */
  0x0000,   /* U+2099 */
  0x0000,   /* U+209A */
  0x0000,   /* U+209B */
  0x0000,   /* U+209C */
  0x0000,   /* U+209D */
  0x0000,   /* U+209E */
  0x0000,   /* U+209F */
  0x0000,   /* U+20A0 */
  0x0000,   /* U+20A1 */
  0x0000,   /* U+20A2 */
  0x0000,   /* U+20A3 */
  0x0000,   /* U+20A4 */
  0x0000,   /* U+20A5 */
  0x0000,   /* U+20A6 */
  0x0000,   /* U+20A7 */
  0x0000,   /* U+20A8 */
  0x0000,   /* U+20A9 */
  0x0000,   /* U+20AA */
  0x0000,   /* U+20AB */
  0x0000,   /* U+20AC */
  0x0000,   /* U+20AD */
  0x0000,   /* U+20AE */
  0x0000,   /* U+20AF */
  0x0000,   /* U+20B0 */
  0x0000,   /* U+20B1 */
  0x0000,   /* U+20B2 */
  0x0000,   /* U+20B3 */
  0x0000,   /* U+20B4 */
  0x0000,   /* U+20B5 */
  0x0000,   /* U+20B6 */
  0x0000,   /* U+20B7 */
  0x0000,   /* U+20B8 */
  0x0000,   /* U+20B9 */
  0x0000,   /* U+20BA */
  0x0000,   /* U+20BB */
  0x0000,   /* U+20BC */
  0x0000,   /* U+20BD */
  0x0000,   /* U+20BE */
  0x0000,   /* U+20BF */
  0x0000,   /* U+20C0 */
  0x0000,   /* U+20C1 */
  0x0000,   /* U+20C2 */
  0x0000,   /* U+20C3 */
  0x0000,   /* U+20C4 */
  0x0000,   /* U+20C5 */
  0x0000,   /* U+20C6 */
  0x0000,   /* U+20C7 */
  0x0000,   /* U+20C8 */
  0x0000,   /* U+20C9 */
  0x0000,   /* U+20CA */
  0x0000,   /* U+20CB */
  0x0000,   /* U+20CC */
  0x0000,   /* U+20CD */
  0x0000,   /* U+20CE */
  0x0000,   /* U+20CF */
  0x0000,   /* U+20D0 */
  0x0000,   /* U+20D1 */
  0x0000,   /* U+20D2 */
  0x0000,   /* U+20D3 */
  0x0000,   /* U+20D4 */
  0x0000,   /* U+20D5 */
  0x0000,   /* U+20D6 */
  0x0000,   /* U+20D7 */
  0x0000,   /* U+20D8 */
  0x0000,   /* U+20D9 */
  0x0000,   /* U+20DA */
  0x0000,   /* U+20DB */
  0x0000,   /* U+20DC */
  0x0000,   /* U+20DD */
  0x0000,   /* U+20DE */
  0x0000,   /* U+20DF */
  0x0000,   /* U+20E0 */
  0x0000,   /* U+20E1 */
  0x0000,   /* U+20E2 */
  0x0000,   /* U+20E3 */
  0x0000,   /* U+20E4 */
  0x0000,   /* U+20E5 */
  0x0000,   /* U+20E6 */
  0x0000,   /* U+20E7 */
  0x0000,   /* U+20E8 */
  0x0000,   /* U+20E9 */
  0x0000,   /* U+20EA */
  0x0000,   /* U+20EB */
  0x0000,   /* U+20EC */
  0x0000,   /* U+20ED */
  0x0000,   /* U+20EE */
  0x0000,   /* U+20EF */
  0x0000,   /* U+20F0 */
  0x0000,   /* U+20F1 */
  0x0000,   /* U+20F2 */
  0x0000,   /* U+20F3 */
  0x0000,   /* U+20F4 */
  0x0000,   /* U+20F5 */
  0x0000,   /* U+20F6 */
  0x0000,   /* U+20F7 */
  0x0000,   /* U+20F8 */
  0x0000,   /* U+20F9 */
  0x0000,   /* U+20FA */
  0x0000,   /* U+20FB */
  0x0000,   /* U+20FC */
  0x0000,   /* U+20FD */
  0x0000,   /* U+20FE */
  0x0000,   /* U+20FF */
};

uint16 uca900_p021[]= {
  /* Number of CEs for each character. */
  3, 3, 1, 2, 1, 3, 3, 1, 1, 2, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 3, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 
  3, 3, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 
  1, 2, 3, 2, 1, 2, 3, 4, 2, 1, 2, 3, 1, 1, 1, 1, 
  1, 2, 3, 2, 1, 2, 3, 4, 2, 1, 2, 3, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C47,   /* U+2100 */
  0x1C47,   /* U+2101 */
  0x1C7A,   /* U+2102 */
  0x04F6,   /* U+2103 */
  0x0586,   /* U+2104 */
  0x1C7A,   /* U+2105 */
  0x1C7A,   /* U+2106 */
  0x1CC2,   /* U+2107 */
  0x0587,   /* U+2108 */
  0x04F6,   /* U+2109 */
  0x1CF4,   /* U+210A */
  0x1D18,   /* U+210B */
  0x1D18,   /* U+210C */
  0x1D18,   /* U+210D */
  0x1D18,   /* U+210E */
  0x1D18,   /* U+210F */
  0x1D32,   /* U+2110 */
  0x1D32,   /* U+2111 */
  0x1D77,   /* U+2112 */
  0x1D77,   /* U+2113 */
  0x0588,   /* U+2114 */
  0x1DB9,   /* U+2115 */
  0x1DB9,   /* U+2116 */
  0x0589,   /* U+2117 */
  0x058A,   /* U+2118 */
  0x1E0C,   /* U+2119 */
  0x1E21,   /* U+211A */
  0x1E33,   /* U+211B */
  0x1E33,   /* U+211C */
  0x1E33,   /* U+211D */
  0x058B,   /* U+211E */
  0x058C,   /* U+211F */
  0x1E71,   /* U+2120 */
  0x1E95,   /* U+2121 */
  0x1E95,   /* U+2122 */
  0x058D,   /* U+2123 */
  0x1F21,   /* U+2124 */
  0x058E,   /* U+2125 */
  0x1FE1,   /* U+2126 */
  0x058F,   /* U+2127 */
  0x1F21,   /* U+2128 */
  0x0590,   /* U+2129 */
  0x1D65,   /* U+212A */
  0x1C47,   /* U+212B */
  0x1C60,   /* U+212C */
  0x1C7A,   /* U+212D */
  0x0591,   /* U+212E */
  0x1CAA,   /* U+212F */
  0x1CAA,   /* U+2130 */
  0x1CE5,   /* U+2131 */
  0x1CF2,   /* U+2132 */
  0x1DAA,   /* U+2133 */
  0x1DDD,   /* U+2134 */
  0x22B7,   /* U+2135 */
  0x22B8,   /* U+2136 */
  0x22B9,   /* U+2137 */
  0x22BA,   /* U+2138 */
  0x1D32,   /* U+2139 */
  0x0592,   /* U+213A */
  0x1CE5,   /* U+213B */
  0x1FCF,   /* U+213C */
  0x1FBB,   /* U+213D */
  0x1FBB,   /* U+213E */
  0x1FCF,   /* U+213F */
  0x0615,   /* U+2140 */
  0x0593,   /* U+2141 */
  0x0594,   /* U+2142 */
  0x0595,   /* U+2143 */
  0x0596,   /* U+2144 */
  0x1C8F,   /* U+2145 */
  0x1C8F,   /* U+2146 */
  0x1CAA,   /* U+2147 */
  0x1D32,   /* U+2148 */
  0x1D4C,   /* U+2149 */
  0x0597,   /* U+214A */
  0x06B1,   /* U+214B */
  0x0598,   /* U+214C */
  0x1C47,   /* U+214D */
  0x1CF2,   /* U+214E */
  0x0599,   /* U+214F */
  0x1C3E,   /* U+2150 */
  0x1C3E,   /* U+2151 */
  0x1C3E,   /* U+2152 */
  0x1C3E,   /* U+2153 */
  0x1C3F,   /* U+2154 */
  0x1C3E,   /* U+2155 */
  0x1C3F,   /* U+2156 */
  0x1C40,   /* U+2157 */
  0x1C41,   /* U+2158 */
  0x1C3E,   /* U+2159 */
  0x1C42,   /* U+215A */
  0x1C3E,   /* U+215B */
  0x1C40,   /* U+215C */
  0x1C42,   /* U+215D */
  0x1C44,   /* U+215E */
  0x1C3E,   /* U+215F */
  0x1D32,   /* U+2160 */
  0x1D32,   /* U+2161 */
  0x1D32,   /* U+2162 */
  0x1D32,   /* U+2163 */
  0x1EE3,   /* U+2164 */
  0x1EE3,   /* U+2165 */
  0x1EE3,   /* U+2166 */
  0x1EE3,   /* U+2167 */
  0x1D32,   /* U+2168 */
  0x1EFF,   /* U+2169 */
  0x1EFF,   /* U+216A */
  0x1EFF,   /* U+216B */
  0x1D77,   /* U+216C */
  0x1C7A,   /* U+216D */
  0x1C8F,   /* U+216E */
  0x1DAA,   /* U+216F */
  0x1D32,   /* U+2170 */
  0x1D32,   /* U+2171 */
  0x1D32,   /* U+2172 */
  0x1D32,   /* U+2173 */
  0x1EE3,   /* U+2174 */
  0x1EE3,   /* U+2175 */
  0x1EE3,   /* U+2176 */
  0x1EE3,   /* U+2177 */
  0x1D32,   /* U+2178 */
  0x1EFF,   /* U+2179 */
  0x1EFF,   /* U+217A */
  0x1EFF,   /* U+217B */
  0x1D77,   /* U+217C */
  0x1C7A,   /* U+217D */
  0x1C8F,   /* U+217E */
  0x1DAA,   /* U+217F */
  0x1AC6,   /* U+2180 */
  0x1AC7,   /* U+2181 */
  0x1AC8,   /* U+2182 */
  0x1C8D,   /* U+2183 */
  0x1C8D,   /* U+2184 */
  0x1C43,   /* U+2185 */
  0x1AC9,   /* U+2186 */
  0x1ACA,   /* U+2187 */
  0x1ACB,   /* U+2188 */
  0x1C3D,   /* U+2189 */
  0x059A,   /* U+218A */
  0x059B,   /* U+218B */
  0xFBC0,   /* U+218C */
  0xFBC0,   /* U+218D */
  0xFBC0,   /* U+218E */
  0xFBC0,   /* U+218F */
  0x059C,   /* U+2190 */
  0x059E,   /* U+2191 */
  0x059D,   /* U+2192 */
  0x059F,   /* U+2193 */
  0x05A0,   /* U+2194 */
  0x05A1,   /* U+2195 */
  0x05A2,   /* U+2196 */
  0x05A3,   /* U+2197 */
  0x05A4,   /* U+2198 */
  0x05A5,   /* U+2199 */
  0x059C,   /* U+219A */
  0x059D,   /* U+219B */
  0x05A6,   /* U+219C */
  0x05A7,   /* U+219D */
  0x05A8,   /* U+219E */
  0x05A9,   /* U+219F */
  0x05AA,   /* U+21A0 */
  0x05AB,   /* U+21A1 */
  0x05AC,   /* U+21A2 */
  0x05AD,   /* U+21A3 */
  0x05AE,   /* U+21A4 */
  0x05AF,   /* U+21A5 */
  0x05B0,   /* U+21A6 */
  0x05B1,   /* U+21A7 */
  0x05B2,   /* U+21A8 */
  0x05B3,   /* U+21A9 */
  0x05B4,   /* U+21AA */
  0x05B5,   /* U+21AB */
  0x05B6,   /* U+21AC */
  0x05B7,   /* U+21AD */
  0x05A0,   /* U+21AE */
  0x05B8,   /* U+21AF */
  0x05B9,   /* U+21B0 */
  0x05BA,   /* U+21B1 */
  0x05BB,   /* U+21B2 */
  0x05BC,   /* U+21B3 */
  0x05BD,   /* U+21B4 */
  0x05BE,   /* U+21B5 */
  0x05BF,   /* U+21B6 */
  0x05C0,   /* U+21B7 */
  0x05C1,   /* U+21B8 */
  0x05C2,   /* U+21B9 */
  0x05C3,   /* U+21BA */
  0x05C4,   /* U+21BB */
  0x05C5,   /* U+21BC */
  0x05C6,   /* U+21BD */
  0x05C7,   /* U+21BE */
  0x05C8,   /* U+21BF */
  0x05C9,   /* U+21C0 */
  0x05CA,   /* U+21C1 */
  0x05CB,   /* U+21C2 */
  0x05CC,   /* U+21C3 */
  0x05CD,   /* U+21C4 */
  0x05CE,   /* U+21C5 */
  0x05CF,   /* U+21C6 */
  0x05D0,   /* U+21C7 */
  0x05D1,   /* U+21C8 */
  0x05D2,   /* U+21C9 */
  0x05D3,   /* U+21CA */
  0x05D4,   /* U+21CB */
  0x05D5,   /* U+21CC */
  0x05D6,   /* U+21CD */
  0x05DA,   /* U+21CE */
  0x05D8,   /* U+21CF */
  0x05D6,   /* U+21D0 */
  0x05D7,   /* U+21D1 */
  0x05D8,   /* U+21D2 */
  0x05D9,   /* U+21D3 */
  0x05DA,   /* U+21D4 */
  0x05DB,   /* U+21D5 */
  0x05DC,   /* U+21D6 */
  0x05DD,   /* U+21D7 */
  0x05DE,   /* U+21D8 */
  0x05DF,   /* U+21D9 */
  0x05E0,   /* U+21DA */
  0x05E1,   /* U+21DB */
  0x05E2,   /* U+21DC */
  0x05E3,   /* U+21DD */
  0x05E4,   /* U+21DE */
  0x05E5,   /* U+21DF */
  0x05E6,   /* U+21E0 */
  0x05E7,   /* U+21E1 */
  0x05E8,   /* U+21E2 */
  0x05E9,   /* U+21E3 */
  0x05EA,   /* U+21E4 */
  0x05EB,   /* U+21E5 */
  0x05EC,   /* U+21E6 */
  0x05ED,   /* U+21E7 */
  0x05EE,   /* U+21E8 */
  0x05EF,   /* U+21E9 */
  0x05F0,   /* U+21EA */
  0x05F1,   /* U+21EB */
  0x05F2,   /* U+21EC */
  0x05F3,   /* U+21ED */
  0x05F4,   /* U+21EE */
  0x05F5,   /* U+21EF */
  0x05F6,   /* U+21F0 */
  0x05F7,   /* U+21F1 */
  0x05F8,   /* U+21F2 */
  0x05F9,   /* U+21F3 */
  0x05FA,   /* U+21F4 */
  0x05FB,   /* U+21F5 */
  0x05FC,   /* U+21F6 */
  0x05FD,   /* U+21F7 */
  0x05FE,   /* U+21F8 */
  0x05FF,   /* U+21F9 */
  0x0600,   /* U+21FA */
  0x0601,   /* U+21FB */
  0x0602,   /* U+21FC */
  0x0603,   /* U+21FD */
  0x0604,   /* U+21FE */
  0x0605,   /* U+21FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2100 */
  0x0020,   /* U+2101 */
  0x0020,   /* U+2102 */
  0x0020,   /* U+2103 */
  0x0020,   /* U+2104 */
  0x0020,   /* U+2105 */
  0x0020,   /* U+2106 */
  0x0020,   /* U+2107 */
  0x0020,   /* U+2108 */
  0x0020,   /* U+2109 */
  0x0020,   /* U+210A */
  0x0020,   /* U+210B */
  0x0020,   /* U+210C */
  0x0020,   /* U+210D */
  0x0020,   /* U+210E */
  0x0020,   /* U+210F */
  0x0020,   /* U+2110 */
  0x0020,   /* U+2111 */
  0x0020,   /* U+2112 */
  0x0020,   /* U+2113 */
  0x0020,   /* U+2114 */
  0x0020,   /* U+2115 */
  0x0020,   /* U+2116 */
  0x0020,   /* U+2117 */
  0x0020,   /* U+2118 */
  0x0020,   /* U+2119 */
  0x0020,   /* U+211A */
  0x0020,   /* U+211B */
  0x0020,   /* U+211C */
  0x0020,   /* U+211D */
  0x0020,   /* U+211E */
  0x0020,   /* U+211F */
  0x0020,   /* U+2120 */
  0x0020,   /* U+2121 */
  0x0020,   /* U+2122 */
  0x0020,   /* U+2123 */
  0x0020,   /* U+2124 */
  0x0020,   /* U+2125 */
  0x0020,   /* U+2126 */
  0x0020,   /* U+2127 */
  0x0020,   /* U+2128 */
  0x0020,   /* U+2129 */
  0x0020,   /* U+212A */
  0x0020,   /* U+212B */
  0x0020,   /* U+212C */
  0x0020,   /* U+212D */
  0x0020,   /* U+212E */
  0x0020,   /* U+212F */
  0x0020,   /* U+2130 */
  0x0020,   /* U+2131 */
  0x0020,   /* U+2132 */
  0x0020,   /* U+2133 */
  0x0020,   /* U+2134 */
  0x0020,   /* U+2135 */
  0x0020,   /* U+2136 */
  0x0020,   /* U+2137 */
  0x0020,   /* U+2138 */
  0x0020,   /* U+2139 */
  0x0020,   /* U+213A */
  0x0020,   /* U+213B */
  0x0020,   /* U+213C */
  0x0020,   /* U+213D */
  0x0020,   /* U+213E */
  0x0020,   /* U+213F */
  0x0020,   /* U+2140 */
  0x0020,   /* U+2141 */
  0x0020,   /* U+2142 */
  0x0020,   /* U+2143 */
  0x0020,   /* U+2144 */
  0x0020,   /* U+2145 */
  0x0020,   /* U+2146 */
  0x0020,   /* U+2147 */
  0x0020,   /* U+2148 */
  0x0020,   /* U+2149 */
  0x0020,   /* U+214A */
  0x0020,   /* U+214B */
  0x0020,   /* U+214C */
  0x0020,   /* U+214D */
  0x0020,   /* U+214E */
  0x0020,   /* U+214F */
  0x0020,   /* U+2150 */
  0x0020,   /* U+2151 */
  0x0020,   /* U+2152 */
  0x0020,   /* U+2153 */
  0x0020,   /* U+2154 */
  0x0020,   /* U+2155 */
  0x0020,   /* U+2156 */
  0x0020,   /* U+2157 */
  0x0020,   /* U+2158 */
  0x0020,   /* U+2159 */
  0x0020,   /* U+215A */
  0x0020,   /* U+215B */
  0x0020,   /* U+215C */
  0x0020,   /* U+215D */
  0x0020,   /* U+215E */
  0x0020,   /* U+215F */
  0x0020,   /* U+2160 */
  0x0020,   /* U+2161 */
  0x0020,   /* U+2162 */
  0x0020,   /* U+2163 */
  0x0020,   /* U+2164 */
  0x0020,   /* U+2165 */
  0x0020,   /* U+2166 */
  0x0020,   /* U+2167 */
  0x0020,   /* U+2168 */
  0x0020,   /* U+2169 */
  0x0020,   /* U+216A */
  0x0020,   /* U+216B */
  0x0020,   /* U+216C */
  0x0020,   /* U+216D */
  0x0020,   /* U+216E */
  0x0020,   /* U+216F */
  0x0020,   /* U+2170 */
  0x0020,   /* U+2171 */
  0x0020,   /* U+2172 */
  0x0020,   /* U+2173 */
  0x0020,   /* U+2174 */
  0x0020,   /* U+2175 */
  0x0020,   /* U+2176 */
  0x0020,   /* U+2177 */
  0x0020,   /* U+2178 */
  0x0020,   /* U+2179 */
  0x0020,   /* U+217A */
  0x0020,   /* U+217B */
  0x0020,   /* U+217C */
  0x0020,   /* U+217D */
  0x0020,   /* U+217E */
  0x0020,   /* U+217F */
  0x0020,   /* U+2180 */
  0x0020,   /* U+2181 */
  0x0020,   /* U+2182 */
  0x0020,   /* U+2183 */
  0x0020,   /* U+2184 */
  0x0020,   /* U+2185 */
  0x0020,   /* U+2186 */
  0x0020,   /* U+2187 */
  0x0020,   /* U+2188 */
  0x0020,   /* U+2189 */
  0x0020,   /* U+218A */
  0x0020,   /* U+218B */
  0x0020,   /* U+218C */
  0x0020,   /* U+218D */
  0x0020,   /* U+218E */
  0x0020,   /* U+218F */
  0x0020,   /* U+2190 */
  0x0020,   /* U+2191 */
  0x0020,   /* U+2192 */
  0x0020,   /* U+2193 */
  0x0020,   /* U+2194 */
  0x0020,   /* U+2195 */
  0x0020,   /* U+2196 */
  0x0020,   /* U+2197 */
  0x0020,   /* U+2198 */
  0x0020,   /* U+2199 */
  0x0020,   /* U+219A */
  0x0020,   /* U+219B */
  0x0020,   /* U+219C */
  0x0020,   /* U+219D */
  0x0020,   /* U+219E */
  0x0020,   /* U+219F */
  0x0020,   /* U+21A0 */
  0x0020,   /* U+21A1 */
  0x0020,   /* U+21A2 */
  0x0020,   /* U+21A3 */
  0x0020,   /* U+21A4 */
  0x0020,   /* U+21A5 */
  0x0020,   /* U+21A6 */
  0x0020,   /* U+21A7 */
  0x0020,   /* U+21A8 */
  0x0020,   /* U+21A9 */
  0x0020,   /* U+21AA */
  0x0020,   /* U+21AB */
  0x0020,   /* U+21AC */
  0x0020,   /* U+21AD */
  0x0020,   /* U+21AE */
  0x0020,   /* U+21AF */
  0x0020,   /* U+21B0 */
  0x0020,   /* U+21B1 */
  0x0020,   /* U+21B2 */
  0x0020,   /* U+21B3 */
  0x0020,   /* U+21B4 */
  0x0020,   /* U+21B5 */
  0x0020,   /* U+21B6 */
  0x0020,   /* U+21B7 */
  0x0020,   /* U+21B8 */
  0x0020,   /* U+21B9 */
  0x0020,   /* U+21BA */
  0x0020,   /* U+21BB */
  0x0020,   /* U+21BC */
  0x0020,   /* U+21BD */
  0x0020,   /* U+21BE */
  0x0020,   /* U+21BF */
  0x0020,   /* U+21C0 */
  0x0020,   /* U+21C1 */
  0x0020,   /* U+21C2 */
  0x0020,   /* U+21C3 */
  0x0020,   /* U+21C4 */
  0x0020,   /* U+21C5 */
  0x0020,   /* U+21C6 */
  0x0020,   /* U+21C7 */
  0x0020,   /* U+21C8 */
  0x0020,   /* U+21C9 */
  0x0020,   /* U+21CA */
  0x0020,   /* U+21CB */
  0x0020,   /* U+21CC */
  0x0020,   /* U+21CD */
  0x0020,   /* U+21CE */
  0x0020,   /* U+21CF */
  0x0020,   /* U+21D0 */
  0x0020,   /* U+21D1 */
  0x0020,   /* U+21D2 */
  0x0020,   /* U+21D3 */
  0x0020,   /* U+21D4 */
  0x0020,   /* U+21D5 */
  0x0020,   /* U+21D6 */
  0x0020,   /* U+21D7 */
  0x0020,   /* U+21D8 */
  0x0020,   /* U+21D9 */
  0x0020,   /* U+21DA */
  0x0020,   /* U+21DB */
  0x0020,   /* U+21DC */
  0x0020,   /* U+21DD */
  0x0020,   /* U+21DE */
  0x0020,   /* U+21DF */
  0x0020,   /* U+21E0 */
  0x0020,   /* U+21E1 */
  0x0020,   /* U+21E2 */
  0x0020,   /* U+21E3 */
  0x0020,   /* U+21E4 */
  0x0020,   /* U+21E5 */
  0x0020,   /* U+21E6 */
  0x0020,   /* U+21E7 */
  0x0020,   /* U+21E8 */
  0x0020,   /* U+21E9 */
  0x0020,   /* U+21EA */
  0x0020,   /* U+21EB */
  0x0020,   /* U+21EC */
  0x0020,   /* U+21ED */
  0x0020,   /* U+21EE */
  0x0020,   /* U+21EF */
  0x0020,   /* U+21F0 */
  0x0020,   /* U+21F1 */
  0x0020,   /* U+21F2 */
  0x0020,   /* U+21F3 */
  0x0020,   /* U+21F4 */
  0x0020,   /* U+21F5 */
  0x0020,   /* U+21F6 */
  0x0020,   /* U+21F7 */
  0x0020,   /* U+21F8 */
  0x0020,   /* U+21F9 */
  0x0020,   /* U+21FA */
  0x0020,   /* U+21FB */
  0x0020,   /* U+21FC */
  0x0020,   /* U+21FD */
  0x0020,   /* U+21FE */
  0x0020,   /* U+21FF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+2100 */
  0x0004,   /* U+2101 */
  0x000B,   /* U+2102 */
  0x0004,   /* U+2103 */
  0x0002,   /* U+2104 */
  0x0004,   /* U+2105 */
  0x0004,   /* U+2106 */
  0x000A,   /* U+2107 */
  0x0002,   /* U+2108 */
  0x0004,   /* U+2109 */
  0x0005,   /* U+210A */
  0x000B,   /* U+210B */
  0x000B,   /* U+210C */
  0x000B,   /* U+210D */
  0x0005,   /* U+210E */
  0x0002,   /* U+210F */
  0x000B,   /* U+2110 */
  0x000B,   /* U+2111 */
  0x000B,   /* U+2112 */
  0x0005,   /* U+2113 */
  0x0002,   /* U+2114 */
  0x000B,   /* U+2115 */
  0x000A,   /* U+2116 */
  0x0002,   /* U+2117 */
  0x0002,   /* U+2118 */
  0x000B,   /* U+2119 */
  0x000B,   /* U+211A */
  0x000B,   /* U+211B */
  0x000B,   /* U+211C */
  0x000B,   /* U+211D */
  0x0002,   /* U+211E */
  0x0002,   /* U+211F */
  0x0014,   /* U+2120 */
  0x000A,   /* U+2121 */
  0x0014,   /* U+2122 */
  0x0002,   /* U+2123 */
  0x000B,   /* U+2124 */
  0x0002,   /* U+2125 */
  0x0008,   /* U+2126 */
  0x0002,   /* U+2127 */
  0x000B,   /* U+2128 */
  0x0002,   /* U+2129 */
  0x0008,   /* U+212A */
  0x0008,   /* U+212B */
  0x000B,   /* U+212C */
  0x000B,   /* U+212D */
  0x0002,   /* U+212E */
  0x0005,   /* U+212F */
  0x000B,   /* U+2130 */
  0x000B,   /* U+2131 */
  0x0008,   /* U+2132 */
  0x000B,   /* U+2133 */
  0x0005,   /* U+2134 */
  0x0004,   /* U+2135 */
  0x0004,   /* U+2136 */
  0x0004,   /* U+2137 */
  0x0004,   /* U+2138 */
  0x0005,   /* U+2139 */
  0x0002,   /* U+213A */
  0x000A,   /* U+213B */
  0x0005,   /* U+213C */
  0x0005,   /* U+213D */
  0x000B,   /* U+213E */
  0x000B,   /* U+213F */
  0x0005,   /* U+2140 */
  0x0002,   /* U+2141 */
  0x0002,   /* U+2142 */
  0x0002,   /* U+2143 */
  0x0002,   /* U+2144 */
  0x000B,   /* U+2145 */
  0x0005,   /* U+2146 */
  0x0005,   /* U+2147 */
  0x0005,   /* U+2148 */
  0x0005,   /* U+2149 */
  0x0002,   /* U+214A */
  0x0002,   /* U+214B */
  0x0002,   /* U+214C */
  0x000A,   /* U+214D */
  0x0002,   /* U+214E */
  0x0002,   /* U+214F */
  0x001E,   /* U+2150 */
  0x001E,   /* U+2151 */
  0x001E,   /* U+2152 */
  0x001E,   /* U+2153 */
  0x001E,   /* U+2154 */
  0x001E,   /* U+2155 */
  0x001E,   /* U+2156 */
  0x001E,   /* U+2157 */
  0x001E,   /* U+2158 */
  0x001E,   /* U+2159 */
  0x001E,   /* U+215A */
  0x001E,   /* U+215B */
  0x001E,   /* U+215C */
  0x001E,   /* U+215D */
  0x001E,   /* U+215E */
  0x001E,   /* U+215F */
  0x000A,   /* U+2160 */
  0x000A,   /* U+2161 */
  0x000A,   /* U+2162 */
  0x000A,   /* U+2163 */
  0x000A,   /* U+2164 */
  0x000A,   /* U+2165 */
  0x000A,   /* U+2166 */
  0x000A,   /* U+2167 */
  0x000A,   /* U+2168 */
  0x000A,   /* U+2169 */
  0x000A,   /* U+216A */
  0x000A,   /* U+216B */
  0x000A,   /* U+216C */
  0x000A,   /* U+216D */
  0x000A,   /* U+216E */
  0x000A,   /* U+216F */
  0x0004,   /* U+2170 */
  0x0004,   /* U+2171 */
  0x0004,   /* U+2172 */
  0x0004,   /* U+2173 */
  0x0004,   /* U+2174 */
  0x0004,   /* U+2175 */
  0x0004,   /* U+2176 */
  0x0004,   /* U+2177 */
  0x0004,   /* U+2178 */
  0x0004,   /* U+2179 */
  0x0004,   /* U+217A */
  0x0004,   /* U+217B */
  0x0004,   /* U+217C */
  0x0004,   /* U+217D */
  0x0004,   /* U+217E */
  0x0004,   /* U+217F */
  0x0002,   /* U+2180 */
  0x0002,   /* U+2181 */
  0x0002,   /* U+2182 */
  0x0008,   /* U+2183 */
  0x0002,   /* U+2184 */
  0x0002,   /* U+2185 */
  0x0002,   /* U+2186 */
  0x0002,   /* U+2187 */
  0x0002,   /* U+2188 */
  0x001E,   /* U+2189 */
  0x0002,   /* U+218A */
  0x0002,   /* U+218B */
  0x0002,   /* U+218C */
  0x0002,   /* U+218D */
  0x0002,   /* U+218E */
  0x0002,   /* U+218F */
  0x0002,   /* U+2190 */
  0x0002,   /* U+2191 */
  0x0002,   /* U+2192 */
  0x0002,   /* U+2193 */
  0x0002,   /* U+2194 */
  0x0002,   /* U+2195 */
  0x0002,   /* U+2196 */
  0x0002,   /* U+2197 */
  0x0002,   /* U+2198 */
  0x0002,   /* U+2199 */
  0x0002,   /* U+219A */
  0x0002,   /* U+219B */
  0x0002,   /* U+219C */
  0x0002,   /* U+219D */
  0x0002,   /* U+219E */
  0x0002,   /* U+219F */
  0x0002,   /* U+21A0 */
  0x0002,   /* U+21A1 */
  0x0002,   /* U+21A2 */
  0x0002,   /* U+21A3 */
  0x0002,   /* U+21A4 */
  0x0002,   /* U+21A5 */
  0x0002,   /* U+21A6 */
  0x0002,   /* U+21A7 */
  0x0002,   /* U+21A8 */
  0x0002,   /* U+21A9 */
  0x0002,   /* U+21AA */
  0x0002,   /* U+21AB */
  0x0002,   /* U+21AC */
  0x0002,   /* U+21AD */
  0x0002,   /* U+21AE */
  0x0002,   /* U+21AF */
  0x0002,   /* U+21B0 */
  0x0002,   /* U+21B1 */
  0x0002,   /* U+21B2 */
  0x0002,   /* U+21B3 */
  0x0002,   /* U+21B4 */
  0x0002,   /* U+21B5 */
  0x0002,   /* U+21B6 */
  0x0002,   /* U+21B7 */
  0x0002,   /* U+21B8 */
  0x0002,   /* U+21B9 */
  0x0002,   /* U+21BA */
  0x0002,   /* U+21BB */
  0x0002,   /* U+21BC */
  0x0002,   /* U+21BD */
  0x0002,   /* U+21BE */
  0x0002,   /* U+21BF */
  0x0002,   /* U+21C0 */
  0x0002,   /* U+21C1 */
  0x0002,   /* U+21C2 */
  0x0002,   /* U+21C3 */
  0x0002,   /* U+21C4 */
  0x0002,   /* U+21C5 */
  0x0002,   /* U+21C6 */
  0x0002,   /* U+21C7 */
  0x0002,   /* U+21C8 */
  0x0002,   /* U+21C9 */
  0x0002,   /* U+21CA */
  0x0002,   /* U+21CB */
  0x0002,   /* U+21CC */
  0x0002,   /* U+21CD */
  0x0002,   /* U+21CE */
  0x0002,   /* U+21CF */
  0x0002,   /* U+21D0 */
  0x0002,   /* U+21D1 */
  0x0002,   /* U+21D2 */
  0x0002,   /* U+21D3 */
  0x0002,   /* U+21D4 */
  0x0002,   /* U+21D5 */
  0x0002,   /* U+21D6 */
  0x0002,   /* U+21D7 */
  0x0002,   /* U+21D8 */
  0x0002,   /* U+21D9 */
  0x0002,   /* U+21DA */
  0x0002,   /* U+21DB */
  0x0002,   /* U+21DC */
  0x0002,   /* U+21DD */
  0x0002,   /* U+21DE */
  0x0002,   /* U+21DF */
  0x0002,   /* U+21E0 */
  0x0002,   /* U+21E1 */
  0x0002,   /* U+21E2 */
  0x0002,   /* U+21E3 */
  0x0002,   /* U+21E4 */
  0x0002,   /* U+21E5 */
  0x0002,   /* U+21E6 */
  0x0002,   /* U+21E7 */
  0x0002,   /* U+21E8 */
  0x0002,   /* U+21E9 */
  0x0002,   /* U+21EA */
  0x0002,   /* U+21EB */
  0x0002,   /* U+21EC */
  0x0002,   /* U+21ED */
  0x0002,   /* U+21EE */
  0x0002,   /* U+21EF */
  0x0002,   /* U+21F0 */
  0x0002,   /* U+21F1 */
  0x0002,   /* U+21F2 */
  0x0002,   /* U+21F3 */
  0x0002,   /* U+21F4 */
  0x0002,   /* U+21F5 */
  0x0002,   /* U+21F6 */
  0x0002,   /* U+21F7 */
  0x0002,   /* U+21F8 */
  0x0002,   /* U+21F9 */
  0x0002,   /* U+21FA */
  0x0002,   /* U+21FB */
  0x0002,   /* U+21FC */
  0x0002,   /* U+21FD */
  0x0002,   /* U+21FE */
  0x0002,   /* U+21FF */

  /* Primary weight 2 for each character. */
  0x0394,   /* U+2100 */
  0x0394,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x1C7A,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0394,   /* U+2105 */
  0x0394,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x1CE5,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x1DDD,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x1DAA,   /* U+2120 */
  0x1CAA,   /* U+2121 */
  0x1DAA,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x1C47,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0394,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0626,   /* U+2150 */
  0x0626,   /* U+2151 */
  0x0626,   /* U+2152 */
  0x0626,   /* U+2153 */
  0x0626,   /* U+2154 */
  0x0626,   /* U+2155 */
  0x0626,   /* U+2156 */
  0x0626,   /* U+2157 */
  0x0626,   /* U+2158 */
  0x0626,   /* U+2159 */
  0x0626,   /* U+215A */
  0x0626,   /* U+215B */
  0x0626,   /* U+215C */
  0x0626,   /* U+215D */
  0x0626,   /* U+215E */
  0x0626,   /* U+215F */
  0x0000,   /* U+2160 */
  0x1D32,   /* U+2161 */
  0x1D32,   /* U+2162 */
  0x1EE3,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x1D32,   /* U+2165 */
  0x1D32,   /* U+2166 */
  0x1D32,   /* U+2167 */
  0x1EFF,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x1D32,   /* U+216A */
  0x1D32,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x1D32,   /* U+2171 */
  0x1D32,   /* U+2172 */
  0x1EE3,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x1D32,   /* U+2175 */
  0x1D32,   /* U+2176 */
  0x1D32,   /* U+2177 */
  0x1EFF,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x1D32,   /* U+217A */
  0x1D32,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0626,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0xA18C,   /* U+218C */
  0xA18D,   /* U+218D */
  0xA18E,   /* U+218E */
  0xA18F,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+2100 */
  0x0020,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0020,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0020,   /* U+2105 */
  0x0020,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0020,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0039,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0020,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0020,   /* U+2120 */
  0x0020,   /* U+2121 */
  0x0020,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0029,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x0020,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0020,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0020,   /* U+2150 */
  0x0020,   /* U+2151 */
  0x0020,   /* U+2152 */
  0x0020,   /* U+2153 */
  0x0020,   /* U+2154 */
  0x0020,   /* U+2155 */
  0x0020,   /* U+2156 */
  0x0020,   /* U+2157 */
  0x0020,   /* U+2158 */
  0x0020,   /* U+2159 */
  0x0020,   /* U+215A */
  0x0020,   /* U+215B */
  0x0020,   /* U+215C */
  0x0020,   /* U+215D */
  0x0020,   /* U+215E */
  0x0020,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0020,   /* U+2161 */
  0x0020,   /* U+2162 */
  0x0020,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0020,   /* U+2165 */
  0x0020,   /* U+2166 */
  0x0020,   /* U+2167 */
  0x0020,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0020,   /* U+216A */
  0x0020,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0020,   /* U+2171 */
  0x0020,   /* U+2172 */
  0x0020,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0020,   /* U+2175 */
  0x0020,   /* U+2176 */
  0x0020,   /* U+2177 */
  0x0020,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0020,   /* U+217A */
  0x0020,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0020,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x002F,   /* U+219A */
  0x002F,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x002F,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x002F,   /* U+21CD */
  0x002F,   /* U+21CE */
  0x002F,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Tertiary weight 2 for each character. */
  0x0004,   /* U+2100 */
  0x0004,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x000A,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0004,   /* U+2105 */
  0x0004,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x000A,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0002,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0004,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0014,   /* U+2120 */
  0x000A,   /* U+2121 */
  0x0014,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0002,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x000A,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0004,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x001E,   /* U+2150 */
  0x001E,   /* U+2151 */
  0x001E,   /* U+2152 */
  0x001E,   /* U+2153 */
  0x001E,   /* U+2154 */
  0x001E,   /* U+2155 */
  0x001E,   /* U+2156 */
  0x001E,   /* U+2157 */
  0x001E,   /* U+2158 */
  0x001E,   /* U+2159 */
  0x001E,   /* U+215A */
  0x001E,   /* U+215B */
  0x001E,   /* U+215C */
  0x001E,   /* U+215D */
  0x001E,   /* U+215E */
  0x001E,   /* U+215F */
  0x0000,   /* U+2160 */
  0x000A,   /* U+2161 */
  0x000A,   /* U+2162 */
  0x000A,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x000A,   /* U+2165 */
  0x000A,   /* U+2166 */
  0x000A,   /* U+2167 */
  0x000A,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x000A,   /* U+216A */
  0x000A,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0004,   /* U+2171 */
  0x0004,   /* U+2172 */
  0x0004,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0004,   /* U+2175 */
  0x0004,   /* U+2176 */
  0x0004,   /* U+2177 */
  0x0004,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0004,   /* U+217A */
  0x0004,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x001E,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0002,   /* U+219A */
  0x0002,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0002,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0002,   /* U+21CD */
  0x0002,   /* U+21CE */
  0x0002,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Primary weight 3 for each character. */
  0x1C7A,   /* U+2100 */
  0x1E71,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x1DDD,   /* U+2105 */
  0x1EB5,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x1D77,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x1EFF,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x1E71,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x1C44,   /* U+2150 */
  0x1C46,   /* U+2151 */
  0x1C3E,   /* U+2152 */
  0x1C40,   /* U+2153 */
  0x1C40,   /* U+2154 */
  0x1C42,   /* U+2155 */
  0x1C42,   /* U+2156 */
  0x1C42,   /* U+2157 */
  0x1C42,   /* U+2158 */
  0x1C43,   /* U+2159 */
  0x1C43,   /* U+215A */
  0x1C45,   /* U+215B */
  0x1C45,   /* U+215C */
  0x1C45,   /* U+215D */
  0x1C45,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x1D32,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x1D32,   /* U+2166 */
  0x1D32,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x1D32,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x1D32,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x1D32,   /* U+2176 */
  0x1D32,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x1D32,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x1C40,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Secondary weight 3 for each character. */
  0x0020,   /* U+2100 */
  0x0020,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0020,   /* U+2105 */
  0x0020,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x0020,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x0020,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0020,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0020,   /* U+2150 */
  0x0020,   /* U+2151 */
  0x0020,   /* U+2152 */
  0x0020,   /* U+2153 */
  0x0020,   /* U+2154 */
  0x0020,   /* U+2155 */
  0x0020,   /* U+2156 */
  0x0020,   /* U+2157 */
  0x0020,   /* U+2158 */
  0x0020,   /* U+2159 */
  0x0020,   /* U+215A */
  0x0020,   /* U+215B */
  0x0020,   /* U+215C */
  0x0020,   /* U+215D */
  0x0020,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x0020,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x0020,   /* U+2166 */
  0x0020,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x0020,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x0020,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x0020,   /* U+2176 */
  0x0020,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x0020,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0020,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Tertiary weight 3 for each character. */
  0x0004,   /* U+2100 */
  0x0004,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0004,   /* U+2105 */
  0x0004,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x000A,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x000A,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x000A,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x001E,   /* U+2150 */
  0x001E,   /* U+2151 */
  0x001E,   /* U+2152 */
  0x001E,   /* U+2153 */
  0x001E,   /* U+2154 */
  0x001E,   /* U+2155 */
  0x001E,   /* U+2156 */
  0x001E,   /* U+2157 */
  0x001E,   /* U+2158 */
  0x001E,   /* U+2159 */
  0x001E,   /* U+215A */
  0x001E,   /* U+215B */
  0x001E,   /* U+215C */
  0x001E,   /* U+215D */
  0x001E,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x000A,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x000A,   /* U+2166 */
  0x000A,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x000A,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x0004,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x0004,   /* U+2176 */
  0x0004,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x0004,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x001E,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+2100 */
  0x0000,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0000,   /* U+2105 */
  0x0000,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x0000,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x0000,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0000,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0000,   /* U+2150 */
  0x0000,   /* U+2151 */
  0x1C3D,   /* U+2152 */
  0x0000,   /* U+2153 */
  0x0000,   /* U+2154 */
  0x0000,   /* U+2155 */
  0x0000,   /* U+2156 */
  0x0000,   /* U+2157 */
  0x0000,   /* U+2158 */
  0x0000,   /* U+2159 */
  0x0000,   /* U+215A */
  0x0000,   /* U+215B */
  0x0000,   /* U+215C */
  0x0000,   /* U+215D */
  0x0000,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x0000,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x0000,   /* U+2166 */
  0x1D32,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x0000,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x0000,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x0000,   /* U+2176 */
  0x1D32,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x0000,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0000,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+2100 */
  0x0000,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0000,   /* U+2105 */
  0x0000,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x0000,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x0000,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0000,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0000,   /* U+2150 */
  0x0000,   /* U+2151 */
  0x0020,   /* U+2152 */
  0x0000,   /* U+2153 */
  0x0000,   /* U+2154 */
  0x0000,   /* U+2155 */
  0x0000,   /* U+2156 */
  0x0000,   /* U+2157 */
  0x0000,   /* U+2158 */
  0x0000,   /* U+2159 */
  0x0000,   /* U+215A */
  0x0000,   /* U+215B */
  0x0000,   /* U+215C */
  0x0000,   /* U+215D */
  0x0000,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x0000,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x0000,   /* U+2166 */
  0x0020,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x0000,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x0000,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x0000,   /* U+2176 */
  0x0020,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x0000,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0000,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+2100 */
  0x0000,   /* U+2101 */
  0x0000,   /* U+2102 */
  0x0000,   /* U+2103 */
  0x0000,   /* U+2104 */
  0x0000,   /* U+2105 */
  0x0000,   /* U+2106 */
  0x0000,   /* U+2107 */
  0x0000,   /* U+2108 */
  0x0000,   /* U+2109 */
  0x0000,   /* U+210A */
  0x0000,   /* U+210B */
  0x0000,   /* U+210C */
  0x0000,   /* U+210D */
  0x0000,   /* U+210E */
  0x0000,   /* U+210F */
  0x0000,   /* U+2110 */
  0x0000,   /* U+2111 */
  0x0000,   /* U+2112 */
  0x0000,   /* U+2113 */
  0x0000,   /* U+2114 */
  0x0000,   /* U+2115 */
  0x0000,   /* U+2116 */
  0x0000,   /* U+2117 */
  0x0000,   /* U+2118 */
  0x0000,   /* U+2119 */
  0x0000,   /* U+211A */
  0x0000,   /* U+211B */
  0x0000,   /* U+211C */
  0x0000,   /* U+211D */
  0x0000,   /* U+211E */
  0x0000,   /* U+211F */
  0x0000,   /* U+2120 */
  0x0000,   /* U+2121 */
  0x0000,   /* U+2122 */
  0x0000,   /* U+2123 */
  0x0000,   /* U+2124 */
  0x0000,   /* U+2125 */
  0x0000,   /* U+2126 */
  0x0000,   /* U+2127 */
  0x0000,   /* U+2128 */
  0x0000,   /* U+2129 */
  0x0000,   /* U+212A */
  0x0000,   /* U+212B */
  0x0000,   /* U+212C */
  0x0000,   /* U+212D */
  0x0000,   /* U+212E */
  0x0000,   /* U+212F */
  0x0000,   /* U+2130 */
  0x0000,   /* U+2131 */
  0x0000,   /* U+2132 */
  0x0000,   /* U+2133 */
  0x0000,   /* U+2134 */
  0x0000,   /* U+2135 */
  0x0000,   /* U+2136 */
  0x0000,   /* U+2137 */
  0x0000,   /* U+2138 */
  0x0000,   /* U+2139 */
  0x0000,   /* U+213A */
  0x0000,   /* U+213B */
  0x0000,   /* U+213C */
  0x0000,   /* U+213D */
  0x0000,   /* U+213E */
  0x0000,   /* U+213F */
  0x0000,   /* U+2140 */
  0x0000,   /* U+2141 */
  0x0000,   /* U+2142 */
  0x0000,   /* U+2143 */
  0x0000,   /* U+2144 */
  0x0000,   /* U+2145 */
  0x0000,   /* U+2146 */
  0x0000,   /* U+2147 */
  0x0000,   /* U+2148 */
  0x0000,   /* U+2149 */
  0x0000,   /* U+214A */
  0x0000,   /* U+214B */
  0x0000,   /* U+214C */
  0x0000,   /* U+214D */
  0x0000,   /* U+214E */
  0x0000,   /* U+214F */
  0x0000,   /* U+2150 */
  0x0000,   /* U+2151 */
  0x001E,   /* U+2152 */
  0x0000,   /* U+2153 */
  0x0000,   /* U+2154 */
  0x0000,   /* U+2155 */
  0x0000,   /* U+2156 */
  0x0000,   /* U+2157 */
  0x0000,   /* U+2158 */
  0x0000,   /* U+2159 */
  0x0000,   /* U+215A */
  0x0000,   /* U+215B */
  0x0000,   /* U+215C */
  0x0000,   /* U+215D */
  0x0000,   /* U+215E */
  0x0000,   /* U+215F */
  0x0000,   /* U+2160 */
  0x0000,   /* U+2161 */
  0x0000,   /* U+2162 */
  0x0000,   /* U+2163 */
  0x0000,   /* U+2164 */
  0x0000,   /* U+2165 */
  0x0000,   /* U+2166 */
  0x000A,   /* U+2167 */
  0x0000,   /* U+2168 */
  0x0000,   /* U+2169 */
  0x0000,   /* U+216A */
  0x0000,   /* U+216B */
  0x0000,   /* U+216C */
  0x0000,   /* U+216D */
  0x0000,   /* U+216E */
  0x0000,   /* U+216F */
  0x0000,   /* U+2170 */
  0x0000,   /* U+2171 */
  0x0000,   /* U+2172 */
  0x0000,   /* U+2173 */
  0x0000,   /* U+2174 */
  0x0000,   /* U+2175 */
  0x0000,   /* U+2176 */
  0x0004,   /* U+2177 */
  0x0000,   /* U+2178 */
  0x0000,   /* U+2179 */
  0x0000,   /* U+217A */
  0x0000,   /* U+217B */
  0x0000,   /* U+217C */
  0x0000,   /* U+217D */
  0x0000,   /* U+217E */
  0x0000,   /* U+217F */
  0x0000,   /* U+2180 */
  0x0000,   /* U+2181 */
  0x0000,   /* U+2182 */
  0x0000,   /* U+2183 */
  0x0000,   /* U+2184 */
  0x0000,   /* U+2185 */
  0x0000,   /* U+2186 */
  0x0000,   /* U+2187 */
  0x0000,   /* U+2188 */
  0x0000,   /* U+2189 */
  0x0000,   /* U+218A */
  0x0000,   /* U+218B */
  0x0000,   /* U+218C */
  0x0000,   /* U+218D */
  0x0000,   /* U+218E */
  0x0000,   /* U+218F */
  0x0000,   /* U+2190 */
  0x0000,   /* U+2191 */
  0x0000,   /* U+2192 */
  0x0000,   /* U+2193 */
  0x0000,   /* U+2194 */
  0x0000,   /* U+2195 */
  0x0000,   /* U+2196 */
  0x0000,   /* U+2197 */
  0x0000,   /* U+2198 */
  0x0000,   /* U+2199 */
  0x0000,   /* U+219A */
  0x0000,   /* U+219B */
  0x0000,   /* U+219C */
  0x0000,   /* U+219D */
  0x0000,   /* U+219E */
  0x0000,   /* U+219F */
  0x0000,   /* U+21A0 */
  0x0000,   /* U+21A1 */
  0x0000,   /* U+21A2 */
  0x0000,   /* U+21A3 */
  0x0000,   /* U+21A4 */
  0x0000,   /* U+21A5 */
  0x0000,   /* U+21A6 */
  0x0000,   /* U+21A7 */
  0x0000,   /* U+21A8 */
  0x0000,   /* U+21A9 */
  0x0000,   /* U+21AA */
  0x0000,   /* U+21AB */
  0x0000,   /* U+21AC */
  0x0000,   /* U+21AD */
  0x0000,   /* U+21AE */
  0x0000,   /* U+21AF */
  0x0000,   /* U+21B0 */
  0x0000,   /* U+21B1 */
  0x0000,   /* U+21B2 */
  0x0000,   /* U+21B3 */
  0x0000,   /* U+21B4 */
  0x0000,   /* U+21B5 */
  0x0000,   /* U+21B6 */
  0x0000,   /* U+21B7 */
  0x0000,   /* U+21B8 */
  0x0000,   /* U+21B9 */
  0x0000,   /* U+21BA */
  0x0000,   /* U+21BB */
  0x0000,   /* U+21BC */
  0x0000,   /* U+21BD */
  0x0000,   /* U+21BE */
  0x0000,   /* U+21BF */
  0x0000,   /* U+21C0 */
  0x0000,   /* U+21C1 */
  0x0000,   /* U+21C2 */
  0x0000,   /* U+21C3 */
  0x0000,   /* U+21C4 */
  0x0000,   /* U+21C5 */
  0x0000,   /* U+21C6 */
  0x0000,   /* U+21C7 */
  0x0000,   /* U+21C8 */
  0x0000,   /* U+21C9 */
  0x0000,   /* U+21CA */
  0x0000,   /* U+21CB */
  0x0000,   /* U+21CC */
  0x0000,   /* U+21CD */
  0x0000,   /* U+21CE */
  0x0000,   /* U+21CF */
  0x0000,   /* U+21D0 */
  0x0000,   /* U+21D1 */
  0x0000,   /* U+21D2 */
  0x0000,   /* U+21D3 */
  0x0000,   /* U+21D4 */
  0x0000,   /* U+21D5 */
  0x0000,   /* U+21D6 */
  0x0000,   /* U+21D7 */
  0x0000,   /* U+21D8 */
  0x0000,   /* U+21D9 */
  0x0000,   /* U+21DA */
  0x0000,   /* U+21DB */
  0x0000,   /* U+21DC */
  0x0000,   /* U+21DD */
  0x0000,   /* U+21DE */
  0x0000,   /* U+21DF */
  0x0000,   /* U+21E0 */
  0x0000,   /* U+21E1 */
  0x0000,   /* U+21E2 */
  0x0000,   /* U+21E3 */
  0x0000,   /* U+21E4 */
  0x0000,   /* U+21E5 */
  0x0000,   /* U+21E6 */
  0x0000,   /* U+21E7 */
  0x0000,   /* U+21E8 */
  0x0000,   /* U+21E9 */
  0x0000,   /* U+21EA */
  0x0000,   /* U+21EB */
  0x0000,   /* U+21EC */
  0x0000,   /* U+21ED */
  0x0000,   /* U+21EE */
  0x0000,   /* U+21EF */
  0x0000,   /* U+21F0 */
  0x0000,   /* U+21F1 */
  0x0000,   /* U+21F2 */
  0x0000,   /* U+21F3 */
  0x0000,   /* U+21F4 */
  0x0000,   /* U+21F5 */
  0x0000,   /* U+21F6 */
  0x0000,   /* U+21F7 */
  0x0000,   /* U+21F8 */
  0x0000,   /* U+21F9 */
  0x0000,   /* U+21FA */
  0x0000,   /* U+21FB */
  0x0000,   /* U+21FC */
  0x0000,   /* U+21FD */
  0x0000,   /* U+21FE */
  0x0000,   /* U+21FF */
};

uint16 uca900_p022[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 1, 2, 3, 1, 2, 
  3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 2, 1, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  2, 2, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0606,   /* U+2200 */
  0x0607,   /* U+2201 */
  0x0608,   /* U+2202 */
  0x0609,   /* U+2203 */
  0x0609,   /* U+2204 */
  0x060A,   /* U+2205 */
  0x060B,   /* U+2206 */
  0x060C,   /* U+2207 */
  0x060D,   /* U+2208 */
  0x060D,   /* U+2209 */
  0x060E,   /* U+220A */
  0x060F,   /* U+220B */
  0x060F,   /* U+220C */
  0x0610,   /* U+220D */
  0x0612,   /* U+220E */
  0x0613,   /* U+220F */
  0x0614,   /* U+2210 */
  0x0615,   /* U+2211 */
  0x0621,   /* U+2212 */
  0x0623,   /* U+2213 */
  0x0624,   /* U+2214 */
  0x0625,   /* U+2215 */
  0x0627,   /* U+2216 */
  0x0628,   /* U+2217 */
  0x0629,   /* U+2218 */
  0x062A,   /* U+2219 */
  0x062B,   /* U+221A */
  0x062C,   /* U+221B */
  0x062E,   /* U+221C */
  0x0630,   /* U+221D */
  0x0631,   /* U+221E */
  0x0632,   /* U+221F */
  0x0633,   /* U+2220 */
  0x0634,   /* U+2221 */
  0x0635,   /* U+2222 */
  0x0636,   /* U+2223 */
  0x0636,   /* U+2224 */
  0x0637,   /* U+2225 */
  0x0637,   /* U+2226 */
  0x0638,   /* U+2227 */
  0x0639,   /* U+2228 */
  0x063A,   /* U+2229 */
  0x063B,   /* U+222A */
  0x063C,   /* U+222B */
  0x063C,   /* U+222C */
  0x063C,   /* U+222D */
  0x063D,   /* U+222E */
  0x063D,   /* U+222F */
  0x063D,   /* U+2230 */
  0x063E,   /* U+2231 */
  0x063F,   /* U+2232 */
  0x0640,   /* U+2233 */
  0x0641,   /* U+2234 */
  0x0642,   /* U+2235 */
  0x0643,   /* U+2236 */
  0x0644,   /* U+2237 */
  0x0645,   /* U+2238 */
  0x0646,   /* U+2239 */
  0x0647,   /* U+223A */
  0x0648,   /* U+223B */
  0x0649,   /* U+223C */
  0x064A,   /* U+223D */
  0x064B,   /* U+223E */
  0x064C,   /* U+223F */
  0x064D,   /* U+2240 */
  0x0649,   /* U+2241 */
  0x064E,   /* U+2242 */
  0x064F,   /* U+2243 */
  0x064F,   /* U+2244 */
  0x0650,   /* U+2245 */
  0x0651,   /* U+2246 */
  0x0650,   /* U+2247 */
  0x0652,   /* U+2248 */
  0x0652,   /* U+2249 */
  0x0653,   /* U+224A */
  0x0654,   /* U+224B */
  0x0655,   /* U+224C */
  0x0656,   /* U+224D */
  0x0657,   /* U+224E */
  0x0658,   /* U+224F */
  0x0659,   /* U+2250 */
  0x065A,   /* U+2251 */
  0x065B,   /* U+2252 */
  0x065C,   /* U+2253 */
  0x065D,   /* U+2254 */
  0x065E,   /* U+2255 */
  0x065F,   /* U+2256 */
  0x0660,   /* U+2257 */
  0x0661,   /* U+2258 */
  0x0662,   /* U+2259 */
  0x0663,   /* U+225A */
  0x0664,   /* U+225B */
  0x0665,   /* U+225C */
  0x0666,   /* U+225D */
  0x0667,   /* U+225E */
  0x0668,   /* U+225F */
  0x061B,   /* U+2260 */
  0x0669,   /* U+2261 */
  0x0669,   /* U+2262 */
  0x066A,   /* U+2263 */
  0x066B,   /* U+2264 */
  0x066C,   /* U+2265 */
  0x066D,   /* U+2266 */
  0x066E,   /* U+2267 */
  0x066F,   /* U+2268 */
  0x0670,   /* U+2269 */
  0x0671,   /* U+226A */
  0x0672,   /* U+226B */
  0x0673,   /* U+226C */
  0x0656,   /* U+226D */
  0x061A,   /* U+226E */
  0x061C,   /* U+226F */
  0x066B,   /* U+2270 */
  0x066C,   /* U+2271 */
  0x0674,   /* U+2272 */
  0x0675,   /* U+2273 */
  0x0674,   /* U+2274 */
  0x0675,   /* U+2275 */
  0x0676,   /* U+2276 */
  0x0677,   /* U+2277 */
  0x0676,   /* U+2278 */
  0x0677,   /* U+2279 */
  0x0678,   /* U+227A */
  0x0679,   /* U+227B */
  0x067A,   /* U+227C */
  0x067B,   /* U+227D */
  0x067C,   /* U+227E */
  0x067D,   /* U+227F */
  0x0678,   /* U+2280 */
  0x0679,   /* U+2281 */
  0x067E,   /* U+2282 */
  0x067F,   /* U+2283 */
  0x067E,   /* U+2284 */
  0x067F,   /* U+2285 */
  0x0680,   /* U+2286 */
  0x0681,   /* U+2287 */
  0x0680,   /* U+2288 */
  0x0681,   /* U+2289 */
  0x0682,   /* U+228A */
  0x0683,   /* U+228B */
  0x0684,   /* U+228C */
  0x0685,   /* U+228D */
  0x0686,   /* U+228E */
  0x0687,   /* U+228F */
  0x0688,   /* U+2290 */
  0x0689,   /* U+2291 */
  0x068A,   /* U+2292 */
  0x068B,   /* U+2293 */
  0x068C,   /* U+2294 */
  0x068D,   /* U+2295 */
  0x068E,   /* U+2296 */
  0x068F,   /* U+2297 */
  0x0690,   /* U+2298 */
  0x0691,   /* U+2299 */
  0x0692,   /* U+229A */
  0x0693,   /* U+229B */
  0x0694,   /* U+229C */
  0x0695,   /* U+229D */
  0x0696,   /* U+229E */
  0x0697,   /* U+229F */
  0x0698,   /* U+22A0 */
  0x0699,   /* U+22A1 */
  0x069A,   /* U+22A2 */
  0x069B,   /* U+22A3 */
  0x069C,   /* U+22A4 */
  0x069D,   /* U+22A5 */
  0x069E,   /* U+22A6 */
  0x069F,   /* U+22A7 */
  0x06A0,   /* U+22A8 */
  0x06A1,   /* U+22A9 */
  0x06A2,   /* U+22AA */
  0x06A3,   /* U+22AB */
  0x069A,   /* U+22AC */
  0x06A0,   /* U+22AD */
  0x06A1,   /* U+22AE */
  0x06A3,   /* U+22AF */
  0x06A4,   /* U+22B0 */
  0x06A5,   /* U+22B1 */
  0x06A6,   /* U+22B2 */
  0x06A7,   /* U+22B3 */
  0x06A8,   /* U+22B4 */
  0x06A9,   /* U+22B5 */
  0x06AA,   /* U+22B6 */
  0x06AB,   /* U+22B7 */
  0x06AC,   /* U+22B8 */
  0x06AD,   /* U+22B9 */
  0x06AE,   /* U+22BA */
  0x06AF,   /* U+22BB */
  0x06B0,   /* U+22BC */
  0x06B2,   /* U+22BD */
  0x06B3,   /* U+22BE */
  0x06B4,   /* U+22BF */
  0x06B5,   /* U+22C0 */
  0x06B6,   /* U+22C1 */
  0x06B7,   /* U+22C2 */
  0x06B8,   /* U+22C3 */
  0x06B9,   /* U+22C4 */
  0x06BA,   /* U+22C5 */
  0x06BB,   /* U+22C6 */
  0x06BC,   /* U+22C7 */
  0x06BD,   /* U+22C8 */
  0x06BE,   /* U+22C9 */
  0x06BF,   /* U+22CA */
  0x06C0,   /* U+22CB */
  0x06C1,   /* U+22CC */
  0x06C2,   /* U+22CD */
  0x06C3,   /* U+22CE */
  0x06C4,   /* U+22CF */
  0x06C5,   /* U+22D0 */
  0x06C6,   /* U+22D1 */
  0x06C7,   /* U+22D2 */
  0x06C8,   /* U+22D3 */
  0x06C9,   /* U+22D4 */
  0x06CA,   /* U+22D5 */
  0x06CB,   /* U+22D6 */
  0x06CC,   /* U+22D7 */
  0x06CD,   /* U+22D8 */
  0x06CE,   /* U+22D9 */
  0x06CF,   /* U+22DA */
  0x06D0,   /* U+22DB */
  0x06D1,   /* U+22DC */
  0x06D2,   /* U+22DD */
  0x06D3,   /* U+22DE */
  0x06D4,   /* U+22DF */
  0x067A,   /* U+22E0 */
  0x067B,   /* U+22E1 */
  0x0689,   /* U+22E2 */
  0x068A,   /* U+22E3 */
  0x06D5,   /* U+22E4 */
  0x06D6,   /* U+22E5 */
  0x06D7,   /* U+22E6 */
  0x06D8,   /* U+22E7 */
  0x06D9,   /* U+22E8 */
  0x06DA,   /* U+22E9 */
  0x06A6,   /* U+22EA */
  0x06A7,   /* U+22EB */
  0x06A8,   /* U+22EC */
  0x06A9,   /* U+22ED */
  0x06DB,   /* U+22EE */
  0x06DC,   /* U+22EF */
  0x06DD,   /* U+22F0 */
  0x06DE,   /* U+22F1 */
  0x06DF,   /* U+22F2 */
  0x06E0,   /* U+22F3 */
  0x06E1,   /* U+22F4 */
  0x06E2,   /* U+22F5 */
  0x06E3,   /* U+22F6 */
  0x06E4,   /* U+22F7 */
  0x06E5,   /* U+22F8 */
  0x06E6,   /* U+22F9 */
  0x06E7,   /* U+22FA */
  0x06E8,   /* U+22FB */
  0x06E9,   /* U+22FC */
  0x06EA,   /* U+22FD */
  0x06EB,   /* U+22FE */
  0x06EC,   /* U+22FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2200 */
  0x0020,   /* U+2201 */
  0x0020,   /* U+2202 */
  0x0020,   /* U+2203 */
  0x0020,   /* U+2204 */
  0x0020,   /* U+2205 */
  0x0020,   /* U+2206 */
  0x0020,   /* U+2207 */
  0x0020,   /* U+2208 */
  0x0020,   /* U+2209 */
  0x0020,   /* U+220A */
  0x0020,   /* U+220B */
  0x0020,   /* U+220C */
  0x0020,   /* U+220D */
  0x0020,   /* U+220E */
  0x0020,   /* U+220F */
  0x0020,   /* U+2210 */
  0x0020,   /* U+2211 */
  0x0020,   /* U+2212 */
  0x0020,   /* U+2213 */
  0x0020,   /* U+2214 */
  0x0020,   /* U+2215 */
  0x0020,   /* U+2216 */
  0x0020,   /* U+2217 */
  0x0020,   /* U+2218 */
  0x0020,   /* U+2219 */
  0x0020,   /* U+221A */
  0x0020,   /* U+221B */
  0x0020,   /* U+221C */
  0x0020,   /* U+221D */
  0x0020,   /* U+221E */
  0x0020,   /* U+221F */
  0x0020,   /* U+2220 */
  0x0020,   /* U+2221 */
  0x0020,   /* U+2222 */
  0x0020,   /* U+2223 */
  0x0020,   /* U+2224 */
  0x0020,   /* U+2225 */
  0x0020,   /* U+2226 */
  0x0020,   /* U+2227 */
  0x0020,   /* U+2228 */
  0x0020,   /* U+2229 */
  0x0020,   /* U+222A */
  0x0020,   /* U+222B */
  0x0020,   /* U+222C */
  0x0020,   /* U+222D */
  0x0020,   /* U+222E */
  0x0020,   /* U+222F */
  0x0020,   /* U+2230 */
  0x0020,   /* U+2231 */
  0x0020,   /* U+2232 */
  0x0020,   /* U+2233 */
  0x0020,   /* U+2234 */
  0x0020,   /* U+2235 */
  0x0020,   /* U+2236 */
  0x0020,   /* U+2237 */
  0x0020,   /* U+2238 */
  0x0020,   /* U+2239 */
  0x0020,   /* U+223A */
  0x0020,   /* U+223B */
  0x0020,   /* U+223C */
  0x0020,   /* U+223D */
  0x0020,   /* U+223E */
  0x0020,   /* U+223F */
  0x0020,   /* U+2240 */
  0x0020,   /* U+2241 */
  0x0020,   /* U+2242 */
  0x0020,   /* U+2243 */
  0x0020,   /* U+2244 */
  0x0020,   /* U+2245 */
  0x0020,   /* U+2246 */
  0x0020,   /* U+2247 */
  0x0020,   /* U+2248 */
  0x0020,   /* U+2249 */
  0x0020,   /* U+224A */
  0x0020,   /* U+224B */
  0x0020,   /* U+224C */
  0x0020,   /* U+224D */
  0x0020,   /* U+224E */
  0x0020,   /* U+224F */
  0x0020,   /* U+2250 */
  0x0020,   /* U+2251 */
  0x0020,   /* U+2252 */
  0x0020,   /* U+2253 */
  0x0020,   /* U+2254 */
  0x0020,   /* U+2255 */
  0x0020,   /* U+2256 */
  0x0020,   /* U+2257 */
  0x0020,   /* U+2258 */
  0x0020,   /* U+2259 */
  0x0020,   /* U+225A */
  0x0020,   /* U+225B */
  0x0020,   /* U+225C */
  0x0020,   /* U+225D */
  0x0020,   /* U+225E */
  0x0020,   /* U+225F */
  0x0020,   /* U+2260 */
  0x0020,   /* U+2261 */
  0x0020,   /* U+2262 */
  0x0020,   /* U+2263 */
  0x0020,   /* U+2264 */
  0x0020,   /* U+2265 */
  0x0020,   /* U+2266 */
  0x0020,   /* U+2267 */
  0x0020,   /* U+2268 */
  0x0020,   /* U+2269 */
  0x0020,   /* U+226A */
  0x0020,   /* U+226B */
  0x0020,   /* U+226C */
  0x0020,   /* U+226D */
  0x0020,   /* U+226E */
  0x0020,   /* U+226F */
  0x0020,   /* U+2270 */
  0x0020,   /* U+2271 */
  0x0020,   /* U+2272 */
  0x0020,   /* U+2273 */
  0x0020,   /* U+2274 */
  0x0020,   /* U+2275 */
  0x0020,   /* U+2276 */
  0x0020,   /* U+2277 */
  0x0020,   /* U+2278 */
  0x0020,   /* U+2279 */
  0x0020,   /* U+227A */
  0x0020,   /* U+227B */
  0x0020,   /* U+227C */
  0x0020,   /* U+227D */
  0x0020,   /* U+227E */
  0x0020,   /* U+227F */
  0x0020,   /* U+2280 */
  0x0020,   /* U+2281 */
  0x0020,   /* U+2282 */
  0x0020,   /* U+2283 */
  0x0020,   /* U+2284 */
  0x0020,   /* U+2285 */
  0x0020,   /* U+2286 */
  0x0020,   /* U+2287 */
  0x0020,   /* U+2288 */
  0x0020,   /* U+2289 */
  0x0020,   /* U+228A */
  0x0020,   /* U+228B */
  0x0020,   /* U+228C */
  0x0020,   /* U+228D */
  0x0020,   /* U+228E */
  0x0020,   /* U+228F */
  0x0020,   /* U+2290 */
  0x0020,   /* U+2291 */
  0x0020,   /* U+2292 */
  0x0020,   /* U+2293 */
  0x0020,   /* U+2294 */
  0x0020,   /* U+2295 */
  0x0020,   /* U+2296 */
  0x0020,   /* U+2297 */
  0x0020,   /* U+2298 */
  0x0020,   /* U+2299 */
  0x0020,   /* U+229A */
  0x0020,   /* U+229B */
  0x0020,   /* U+229C */
  0x0020,   /* U+229D */
  0x0020,   /* U+229E */
  0x0020,   /* U+229F */
  0x0020,   /* U+22A0 */
  0x0020,   /* U+22A1 */
  0x0020,   /* U+22A2 */
  0x0020,   /* U+22A3 */
  0x0020,   /* U+22A4 */
  0x0020,   /* U+22A5 */
  0x0020,   /* U+22A6 */
  0x0020,   /* U+22A7 */
  0x0020,   /* U+22A8 */
  0x0020,   /* U+22A9 */
  0x0020,   /* U+22AA */
  0x0020,   /* U+22AB */
  0x0020,   /* U+22AC */
  0x0020,   /* U+22AD */
  0x0020,   /* U+22AE */
  0x0020,   /* U+22AF */
  0x0020,   /* U+22B0 */
  0x0020,   /* U+22B1 */
  0x0020,   /* U+22B2 */
  0x0020,   /* U+22B3 */
  0x0020,   /* U+22B4 */
  0x0020,   /* U+22B5 */
  0x0020,   /* U+22B6 */
  0x0020,   /* U+22B7 */
  0x0020,   /* U+22B8 */
  0x0020,   /* U+22B9 */
  0x0020,   /* U+22BA */
  0x0020,   /* U+22BB */
  0x0020,   /* U+22BC */
  0x0020,   /* U+22BD */
  0x0020,   /* U+22BE */
  0x0020,   /* U+22BF */
  0x0020,   /* U+22C0 */
  0x0020,   /* U+22C1 */
  0x0020,   /* U+22C2 */
  0x0020,   /* U+22C3 */
  0x0020,   /* U+22C4 */
  0x0020,   /* U+22C5 */
  0x0020,   /* U+22C6 */
  0x0020,   /* U+22C7 */
  0x0020,   /* U+22C8 */
  0x0020,   /* U+22C9 */
  0x0020,   /* U+22CA */
  0x0020,   /* U+22CB */
  0x0020,   /* U+22CC */
  0x0020,   /* U+22CD */
  0x0020,   /* U+22CE */
  0x0020,   /* U+22CF */
  0x0020,   /* U+22D0 */
  0x0020,   /* U+22D1 */
  0x0020,   /* U+22D2 */
  0x0020,   /* U+22D3 */
  0x0020,   /* U+22D4 */
  0x0020,   /* U+22D5 */
  0x0020,   /* U+22D6 */
  0x0020,   /* U+22D7 */
  0x0020,   /* U+22D8 */
  0x0020,   /* U+22D9 */
  0x0020,   /* U+22DA */
  0x0020,   /* U+22DB */
  0x0020,   /* U+22DC */
  0x0020,   /* U+22DD */
  0x0020,   /* U+22DE */
  0x0020,   /* U+22DF */
  0x0020,   /* U+22E0 */
  0x0020,   /* U+22E1 */
  0x0020,   /* U+22E2 */
  0x0020,   /* U+22E3 */
  0x0020,   /* U+22E4 */
  0x0020,   /* U+22E5 */
  0x0020,   /* U+22E6 */
  0x0020,   /* U+22E7 */
  0x0020,   /* U+22E8 */
  0x0020,   /* U+22E9 */
  0x0020,   /* U+22EA */
  0x0020,   /* U+22EB */
  0x0020,   /* U+22EC */
  0x0020,   /* U+22ED */
  0x0020,   /* U+22EE */
  0x0020,   /* U+22EF */
  0x0020,   /* U+22F0 */
  0x0020,   /* U+22F1 */
  0x0020,   /* U+22F2 */
  0x0020,   /* U+22F3 */
  0x0020,   /* U+22F4 */
  0x0020,   /* U+22F5 */
  0x0020,   /* U+22F6 */
  0x0020,   /* U+22F7 */
  0x0020,   /* U+22F8 */
  0x0020,   /* U+22F9 */
  0x0020,   /* U+22FA */
  0x0020,   /* U+22FB */
  0x0020,   /* U+22FC */
  0x0020,   /* U+22FD */
  0x0020,   /* U+22FE */
  0x0020,   /* U+22FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2200 */
  0x0002,   /* U+2201 */
  0x0002,   /* U+2202 */
  0x0002,   /* U+2203 */
  0x0002,   /* U+2204 */
  0x0002,   /* U+2205 */
  0x0002,   /* U+2206 */
  0x0002,   /* U+2207 */
  0x0002,   /* U+2208 */
  0x0002,   /* U+2209 */
  0x0002,   /* U+220A */
  0x0002,   /* U+220B */
  0x0002,   /* U+220C */
  0x0002,   /* U+220D */
  0x0002,   /* U+220E */
  0x0002,   /* U+220F */
  0x0002,   /* U+2210 */
  0x0002,   /* U+2211 */
  0x0002,   /* U+2212 */
  0x0002,   /* U+2213 */
  0x0002,   /* U+2214 */
  0x0002,   /* U+2215 */
  0x0002,   /* U+2216 */
  0x0002,   /* U+2217 */
  0x0002,   /* U+2218 */
  0x0002,   /* U+2219 */
  0x0002,   /* U+221A */
  0x0002,   /* U+221B */
  0x0002,   /* U+221C */
  0x0002,   /* U+221D */
  0x0002,   /* U+221E */
  0x0002,   /* U+221F */
  0x0002,   /* U+2220 */
  0x0002,   /* U+2221 */
  0x0002,   /* U+2222 */
  0x0002,   /* U+2223 */
  0x0002,   /* U+2224 */
  0x0002,   /* U+2225 */
  0x0002,   /* U+2226 */
  0x0002,   /* U+2227 */
  0x0002,   /* U+2228 */
  0x0002,   /* U+2229 */
  0x0002,   /* U+222A */
  0x0002,   /* U+222B */
  0x0004,   /* U+222C */
  0x0004,   /* U+222D */
  0x0002,   /* U+222E */
  0x0004,   /* U+222F */
  0x0004,   /* U+2230 */
  0x0002,   /* U+2231 */
  0x0002,   /* U+2232 */
  0x0002,   /* U+2233 */
  0x0002,   /* U+2234 */
  0x0002,   /* U+2235 */
  0x0002,   /* U+2236 */
  0x0002,   /* U+2237 */
  0x0002,   /* U+2238 */
  0x0002,   /* U+2239 */
  0x0002,   /* U+223A */
  0x0002,   /* U+223B */
  0x0002,   /* U+223C */
  0x0002,   /* U+223D */
  0x0002,   /* U+223E */
  0x0002,   /* U+223F */
  0x0002,   /* U+2240 */
  0x0002,   /* U+2241 */
  0x0002,   /* U+2242 */
  0x0002,   /* U+2243 */
  0x0002,   /* U+2244 */
  0x0002,   /* U+2245 */
  0x0002,   /* U+2246 */
  0x0002,   /* U+2247 */
  0x0002,   /* U+2248 */
  0x0002,   /* U+2249 */
  0x0002,   /* U+224A */
  0x0002,   /* U+224B */
  0x0002,   /* U+224C */
  0x0002,   /* U+224D */
  0x0002,   /* U+224E */
  0x0002,   /* U+224F */
  0x0002,   /* U+2250 */
  0x0002,   /* U+2251 */
  0x0002,   /* U+2252 */
  0x0002,   /* U+2253 */
  0x0002,   /* U+2254 */
  0x0002,   /* U+2255 */
  0x0002,   /* U+2256 */
  0x0002,   /* U+2257 */
  0x0002,   /* U+2258 */
  0x0002,   /* U+2259 */
  0x0002,   /* U+225A */
  0x0002,   /* U+225B */
  0x0002,   /* U+225C */
  0x0002,   /* U+225D */
  0x0002,   /* U+225E */
  0x0002,   /* U+225F */
  0x0002,   /* U+2260 */
  0x0002,   /* U+2261 */
  0x0002,   /* U+2262 */
  0x0002,   /* U+2263 */
  0x0002,   /* U+2264 */
  0x0002,   /* U+2265 */
  0x0002,   /* U+2266 */
  0x0002,   /* U+2267 */
  0x0002,   /* U+2268 */
  0x0002,   /* U+2269 */
  0x0002,   /* U+226A */
  0x0002,   /* U+226B */
  0x0002,   /* U+226C */
  0x0002,   /* U+226D */
  0x0002,   /* U+226E */
  0x0002,   /* U+226F */
  0x0002,   /* U+2270 */
  0x0002,   /* U+2271 */
  0x0002,   /* U+2272 */
  0x0002,   /* U+2273 */
  0x0002,   /* U+2274 */
  0x0002,   /* U+2275 */
  0x0002,   /* U+2276 */
  0x0002,   /* U+2277 */
  0x0002,   /* U+2278 */
  0x0002,   /* U+2279 */
  0x0002,   /* U+227A */
  0x0002,   /* U+227B */
  0x0002,   /* U+227C */
  0x0002,   /* U+227D */
  0x0002,   /* U+227E */
  0x0002,   /* U+227F */
  0x0002,   /* U+2280 */
  0x0002,   /* U+2281 */
  0x0002,   /* U+2282 */
  0x0002,   /* U+2283 */
  0x0002,   /* U+2284 */
  0x0002,   /* U+2285 */
  0x0002,   /* U+2286 */
  0x0002,   /* U+2287 */
  0x0002,   /* U+2288 */
  0x0002,   /* U+2289 */
  0x0002,   /* U+228A */
  0x0002,   /* U+228B */
  0x0002,   /* U+228C */
  0x0002,   /* U+228D */
  0x0002,   /* U+228E */
  0x0002,   /* U+228F */
  0x0002,   /* U+2290 */
  0x0002,   /* U+2291 */
  0x0002,   /* U+2292 */
  0x0002,   /* U+2293 */
  0x0002,   /* U+2294 */
  0x0002,   /* U+2295 */
  0x0002,   /* U+2296 */
  0x0002,   /* U+2297 */
  0x0002,   /* U+2298 */
  0x0002,   /* U+2299 */
  0x0002,   /* U+229A */
  0x0002,   /* U+229B */
  0x0002,   /* U+229C */
  0x0002,   /* U+229D */
  0x0002,   /* U+229E */
  0x0002,   /* U+229F */
  0x0002,   /* U+22A0 */
  0x0002,   /* U+22A1 */
  0x0002,   /* U+22A2 */
  0x0002,   /* U+22A3 */
  0x0002,   /* U+22A4 */
  0x0002,   /* U+22A5 */
  0x0002,   /* U+22A6 */
  0x0002,   /* U+22A7 */
  0x0002,   /* U+22A8 */
  0x0002,   /* U+22A9 */
  0x0002,   /* U+22AA */
  0x0002,   /* U+22AB */
  0x0002,   /* U+22AC */
  0x0002,   /* U+22AD */
  0x0002,   /* U+22AE */
  0x0002,   /* U+22AF */
  0x0002,   /* U+22B0 */
  0x0002,   /* U+22B1 */
  0x0002,   /* U+22B2 */
  0x0002,   /* U+22B3 */
  0x0002,   /* U+22B4 */
  0x0002,   /* U+22B5 */
  0x0002,   /* U+22B6 */
  0x0002,   /* U+22B7 */
  0x0002,   /* U+22B8 */
  0x0002,   /* U+22B9 */
  0x0002,   /* U+22BA */
  0x0002,   /* U+22BB */
  0x0002,   /* U+22BC */
  0x0002,   /* U+22BD */
  0x0002,   /* U+22BE */
  0x0002,   /* U+22BF */
  0x0002,   /* U+22C0 */
  0x0002,   /* U+22C1 */
  0x0002,   /* U+22C2 */
  0x0002,   /* U+22C3 */
  0x0002,   /* U+22C4 */
  0x0002,   /* U+22C5 */
  0x0002,   /* U+22C6 */
  0x0002,   /* U+22C7 */
  0x0002,   /* U+22C8 */
  0x0002,   /* U+22C9 */
  0x0002,   /* U+22CA */
  0x0002,   /* U+22CB */
  0x0002,   /* U+22CC */
  0x0002,   /* U+22CD */
  0x0002,   /* U+22CE */
  0x0002,   /* U+22CF */
  0x0002,   /* U+22D0 */
  0x0002,   /* U+22D1 */
  0x0002,   /* U+22D2 */
  0x0002,   /* U+22D3 */
  0x0002,   /* U+22D4 */
  0x0002,   /* U+22D5 */
  0x0002,   /* U+22D6 */
  0x0002,   /* U+22D7 */
  0x0002,   /* U+22D8 */
  0x0002,   /* U+22D9 */
  0x0002,   /* U+22DA */
  0x0002,   /* U+22DB */
  0x0002,   /* U+22DC */
  0x0002,   /* U+22DD */
  0x0002,   /* U+22DE */
  0x0002,   /* U+22DF */
  0x0002,   /* U+22E0 */
  0x0002,   /* U+22E1 */
  0x0002,   /* U+22E2 */
  0x0002,   /* U+22E3 */
  0x0002,   /* U+22E4 */
  0x0002,   /* U+22E5 */
  0x0002,   /* U+22E6 */
  0x0002,   /* U+22E7 */
  0x0002,   /* U+22E8 */
  0x0002,   /* U+22E9 */
  0x0002,   /* U+22EA */
  0x0002,   /* U+22EB */
  0x0002,   /* U+22EC */
  0x0002,   /* U+22ED */
  0x0002,   /* U+22EE */
  0x0002,   /* U+22EF */
  0x0002,   /* U+22F0 */
  0x0002,   /* U+22F1 */
  0x0002,   /* U+22F2 */
  0x0002,   /* U+22F3 */
  0x0002,   /* U+22F4 */
  0x0002,   /* U+22F5 */
  0x0002,   /* U+22F6 */
  0x0002,   /* U+22F7 */
  0x0002,   /* U+22F8 */
  0x0002,   /* U+22F9 */
  0x0002,   /* U+22FA */
  0x0002,   /* U+22FB */
  0x0002,   /* U+22FC */
  0x0002,   /* U+22FD */
  0x0002,   /* U+22FE */
  0x0002,   /* U+22FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x0000,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x0000,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x0000,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x0000,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x0000,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x063C,   /* U+222C */
  0x063C,   /* U+222D */
  0x0000,   /* U+222E */
  0x063D,   /* U+222F */
  0x063D,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x0000,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x0000,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x0000,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x0000,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x0000,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x0000,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x0000,   /* U+226D */
  0x0000,   /* U+226E */
  0x0000,   /* U+226F */
  0x0000,   /* U+2270 */
  0x0000,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x0000,   /* U+2274 */
  0x0000,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x0000,   /* U+2278 */
  0x0000,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x0000,   /* U+2280 */
  0x0000,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x0000,   /* U+2284 */
  0x0000,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x0000,   /* U+2288 */
  0x0000,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x0000,   /* U+22AC */
  0x0000,   /* U+22AD */
  0x0000,   /* U+22AE */
  0x0000,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x0000,   /* U+22E0 */
  0x0000,   /* U+22E1 */
  0x0000,   /* U+22E2 */
  0x0000,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x0000,   /* U+22EA */
  0x0000,   /* U+22EB */
  0x0000,   /* U+22EC */
  0x0000,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x002F,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x002F,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x002F,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x002F,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x002F,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x0020,   /* U+222C */
  0x0020,   /* U+222D */
  0x0000,   /* U+222E */
  0x0020,   /* U+222F */
  0x0020,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x002F,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x002F,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x002F,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x002F,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x002F,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x002F,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x002F,   /* U+226D */
  0x002F,   /* U+226E */
  0x002F,   /* U+226F */
  0x002F,   /* U+2270 */
  0x002F,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x002F,   /* U+2274 */
  0x002F,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x002F,   /* U+2278 */
  0x002F,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x002F,   /* U+2280 */
  0x002F,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x002F,   /* U+2284 */
  0x002F,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x002F,   /* U+2288 */
  0x002F,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x002F,   /* U+22AC */
  0x002F,   /* U+22AD */
  0x002F,   /* U+22AE */
  0x002F,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x002F,   /* U+22E0 */
  0x002F,   /* U+22E1 */
  0x002F,   /* U+22E2 */
  0x002F,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x002F,   /* U+22EA */
  0x002F,   /* U+22EB */
  0x002F,   /* U+22EC */
  0x002F,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x0002,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x0002,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x0002,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x0002,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x0002,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x0004,   /* U+222C */
  0x0004,   /* U+222D */
  0x0000,   /* U+222E */
  0x0004,   /* U+222F */
  0x0004,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x0002,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x0002,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x0002,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x0002,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x0002,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x0002,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x0002,   /* U+226D */
  0x0002,   /* U+226E */
  0x0002,   /* U+226F */
  0x0002,   /* U+2270 */
  0x0002,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x0002,   /* U+2274 */
  0x0002,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x0002,   /* U+2278 */
  0x0002,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x0002,   /* U+2280 */
  0x0002,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x0002,   /* U+2284 */
  0x0002,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x0002,   /* U+2288 */
  0x0002,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x0002,   /* U+22AC */
  0x0002,   /* U+22AD */
  0x0002,   /* U+22AE */
  0x0002,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x0002,   /* U+22E0 */
  0x0002,   /* U+22E1 */
  0x0002,   /* U+22E2 */
  0x0002,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x0002,   /* U+22EA */
  0x0002,   /* U+22EB */
  0x0002,   /* U+22EC */
  0x0002,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x0000,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x0000,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x0000,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x0000,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x0000,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x0000,   /* U+222C */
  0x063C,   /* U+222D */
  0x0000,   /* U+222E */
  0x0000,   /* U+222F */
  0x063D,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x0000,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x0000,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x0000,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x0000,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x0000,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x0000,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x0000,   /* U+226D */
  0x0000,   /* U+226E */
  0x0000,   /* U+226F */
  0x0000,   /* U+2270 */
  0x0000,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x0000,   /* U+2274 */
  0x0000,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x0000,   /* U+2278 */
  0x0000,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x0000,   /* U+2280 */
  0x0000,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x0000,   /* U+2284 */
  0x0000,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x0000,   /* U+2288 */
  0x0000,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x0000,   /* U+22AC */
  0x0000,   /* U+22AD */
  0x0000,   /* U+22AE */
  0x0000,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x0000,   /* U+22E0 */
  0x0000,   /* U+22E1 */
  0x0000,   /* U+22E2 */
  0x0000,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x0000,   /* U+22EA */
  0x0000,   /* U+22EB */
  0x0000,   /* U+22EC */
  0x0000,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x0000,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x0000,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x0000,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x0000,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x0000,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x0000,   /* U+222C */
  0x0020,   /* U+222D */
  0x0000,   /* U+222E */
  0x0000,   /* U+222F */
  0x0020,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x0000,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x0000,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x0000,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x0000,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x0000,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x0000,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x0000,   /* U+226D */
  0x0000,   /* U+226E */
  0x0000,   /* U+226F */
  0x0000,   /* U+2270 */
  0x0000,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x0000,   /* U+2274 */
  0x0000,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x0000,   /* U+2278 */
  0x0000,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x0000,   /* U+2280 */
  0x0000,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x0000,   /* U+2284 */
  0x0000,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x0000,   /* U+2288 */
  0x0000,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x0000,   /* U+22AC */
  0x0000,   /* U+22AD */
  0x0000,   /* U+22AE */
  0x0000,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x0000,   /* U+22E0 */
  0x0000,   /* U+22E1 */
  0x0000,   /* U+22E2 */
  0x0000,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x0000,   /* U+22EA */
  0x0000,   /* U+22EB */
  0x0000,   /* U+22EC */
  0x0000,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2200 */
  0x0000,   /* U+2201 */
  0x0000,   /* U+2202 */
  0x0000,   /* U+2203 */
  0x0000,   /* U+2204 */
  0x0000,   /* U+2205 */
  0x0000,   /* U+2206 */
  0x0000,   /* U+2207 */
  0x0000,   /* U+2208 */
  0x0000,   /* U+2209 */
  0x0000,   /* U+220A */
  0x0000,   /* U+220B */
  0x0000,   /* U+220C */
  0x0000,   /* U+220D */
  0x0000,   /* U+220E */
  0x0000,   /* U+220F */
  0x0000,   /* U+2210 */
  0x0000,   /* U+2211 */
  0x0000,   /* U+2212 */
  0x0000,   /* U+2213 */
  0x0000,   /* U+2214 */
  0x0000,   /* U+2215 */
  0x0000,   /* U+2216 */
  0x0000,   /* U+2217 */
  0x0000,   /* U+2218 */
  0x0000,   /* U+2219 */
  0x0000,   /* U+221A */
  0x0000,   /* U+221B */
  0x0000,   /* U+221C */
  0x0000,   /* U+221D */
  0x0000,   /* U+221E */
  0x0000,   /* U+221F */
  0x0000,   /* U+2220 */
  0x0000,   /* U+2221 */
  0x0000,   /* U+2222 */
  0x0000,   /* U+2223 */
  0x0000,   /* U+2224 */
  0x0000,   /* U+2225 */
  0x0000,   /* U+2226 */
  0x0000,   /* U+2227 */
  0x0000,   /* U+2228 */
  0x0000,   /* U+2229 */
  0x0000,   /* U+222A */
  0x0000,   /* U+222B */
  0x0000,   /* U+222C */
  0x0004,   /* U+222D */
  0x0000,   /* U+222E */
  0x0000,   /* U+222F */
  0x0004,   /* U+2230 */
  0x0000,   /* U+2231 */
  0x0000,   /* U+2232 */
  0x0000,   /* U+2233 */
  0x0000,   /* U+2234 */
  0x0000,   /* U+2235 */
  0x0000,   /* U+2236 */
  0x0000,   /* U+2237 */
  0x0000,   /* U+2238 */
  0x0000,   /* U+2239 */
  0x0000,   /* U+223A */
  0x0000,   /* U+223B */
  0x0000,   /* U+223C */
  0x0000,   /* U+223D */
  0x0000,   /* U+223E */
  0x0000,   /* U+223F */
  0x0000,   /* U+2240 */
  0x0000,   /* U+2241 */
  0x0000,   /* U+2242 */
  0x0000,   /* U+2243 */
  0x0000,   /* U+2244 */
  0x0000,   /* U+2245 */
  0x0000,   /* U+2246 */
  0x0000,   /* U+2247 */
  0x0000,   /* U+2248 */
  0x0000,   /* U+2249 */
  0x0000,   /* U+224A */
  0x0000,   /* U+224B */
  0x0000,   /* U+224C */
  0x0000,   /* U+224D */
  0x0000,   /* U+224E */
  0x0000,   /* U+224F */
  0x0000,   /* U+2250 */
  0x0000,   /* U+2251 */
  0x0000,   /* U+2252 */
  0x0000,   /* U+2253 */
  0x0000,   /* U+2254 */
  0x0000,   /* U+2255 */
  0x0000,   /* U+2256 */
  0x0000,   /* U+2257 */
  0x0000,   /* U+2258 */
  0x0000,   /* U+2259 */
  0x0000,   /* U+225A */
  0x0000,   /* U+225B */
  0x0000,   /* U+225C */
  0x0000,   /* U+225D */
  0x0000,   /* U+225E */
  0x0000,   /* U+225F */
  0x0000,   /* U+2260 */
  0x0000,   /* U+2261 */
  0x0000,   /* U+2262 */
  0x0000,   /* U+2263 */
  0x0000,   /* U+2264 */
  0x0000,   /* U+2265 */
  0x0000,   /* U+2266 */
  0x0000,   /* U+2267 */
  0x0000,   /* U+2268 */
  0x0000,   /* U+2269 */
  0x0000,   /* U+226A */
  0x0000,   /* U+226B */
  0x0000,   /* U+226C */
  0x0000,   /* U+226D */
  0x0000,   /* U+226E */
  0x0000,   /* U+226F */
  0x0000,   /* U+2270 */
  0x0000,   /* U+2271 */
  0x0000,   /* U+2272 */
  0x0000,   /* U+2273 */
  0x0000,   /* U+2274 */
  0x0000,   /* U+2275 */
  0x0000,   /* U+2276 */
  0x0000,   /* U+2277 */
  0x0000,   /* U+2278 */
  0x0000,   /* U+2279 */
  0x0000,   /* U+227A */
  0x0000,   /* U+227B */
  0x0000,   /* U+227C */
  0x0000,   /* U+227D */
  0x0000,   /* U+227E */
  0x0000,   /* U+227F */
  0x0000,   /* U+2280 */
  0x0000,   /* U+2281 */
  0x0000,   /* U+2282 */
  0x0000,   /* U+2283 */
  0x0000,   /* U+2284 */
  0x0000,   /* U+2285 */
  0x0000,   /* U+2286 */
  0x0000,   /* U+2287 */
  0x0000,   /* U+2288 */
  0x0000,   /* U+2289 */
  0x0000,   /* U+228A */
  0x0000,   /* U+228B */
  0x0000,   /* U+228C */
  0x0000,   /* U+228D */
  0x0000,   /* U+228E */
  0x0000,   /* U+228F */
  0x0000,   /* U+2290 */
  0x0000,   /* U+2291 */
  0x0000,   /* U+2292 */
  0x0000,   /* U+2293 */
  0x0000,   /* U+2294 */
  0x0000,   /* U+2295 */
  0x0000,   /* U+2296 */
  0x0000,   /* U+2297 */
  0x0000,   /* U+2298 */
  0x0000,   /* U+2299 */
  0x0000,   /* U+229A */
  0x0000,   /* U+229B */
  0x0000,   /* U+229C */
  0x0000,   /* U+229D */
  0x0000,   /* U+229E */
  0x0000,   /* U+229F */
  0x0000,   /* U+22A0 */
  0x0000,   /* U+22A1 */
  0x0000,   /* U+22A2 */
  0x0000,   /* U+22A3 */
  0x0000,   /* U+22A4 */
  0x0000,   /* U+22A5 */
  0x0000,   /* U+22A6 */
  0x0000,   /* U+22A7 */
  0x0000,   /* U+22A8 */
  0x0000,   /* U+22A9 */
  0x0000,   /* U+22AA */
  0x0000,   /* U+22AB */
  0x0000,   /* U+22AC */
  0x0000,   /* U+22AD */
  0x0000,   /* U+22AE */
  0x0000,   /* U+22AF */
  0x0000,   /* U+22B0 */
  0x0000,   /* U+22B1 */
  0x0000,   /* U+22B2 */
  0x0000,   /* U+22B3 */
  0x0000,   /* U+22B4 */
  0x0000,   /* U+22B5 */
  0x0000,   /* U+22B6 */
  0x0000,   /* U+22B7 */
  0x0000,   /* U+22B8 */
  0x0000,   /* U+22B9 */
  0x0000,   /* U+22BA */
  0x0000,   /* U+22BB */
  0x0000,   /* U+22BC */
  0x0000,   /* U+22BD */
  0x0000,   /* U+22BE */
  0x0000,   /* U+22BF */
  0x0000,   /* U+22C0 */
  0x0000,   /* U+22C1 */
  0x0000,   /* U+22C2 */
  0x0000,   /* U+22C3 */
  0x0000,   /* U+22C4 */
  0x0000,   /* U+22C5 */
  0x0000,   /* U+22C6 */
  0x0000,   /* U+22C7 */
  0x0000,   /* U+22C8 */
  0x0000,   /* U+22C9 */
  0x0000,   /* U+22CA */
  0x0000,   /* U+22CB */
  0x0000,   /* U+22CC */
  0x0000,   /* U+22CD */
  0x0000,   /* U+22CE */
  0x0000,   /* U+22CF */
  0x0000,   /* U+22D0 */
  0x0000,   /* U+22D1 */
  0x0000,   /* U+22D2 */
  0x0000,   /* U+22D3 */
  0x0000,   /* U+22D4 */
  0x0000,   /* U+22D5 */
  0x0000,   /* U+22D6 */
  0x0000,   /* U+22D7 */
  0x0000,   /* U+22D8 */
  0x0000,   /* U+22D9 */
  0x0000,   /* U+22DA */
  0x0000,   /* U+22DB */
  0x0000,   /* U+22DC */
  0x0000,   /* U+22DD */
  0x0000,   /* U+22DE */
  0x0000,   /* U+22DF */
  0x0000,   /* U+22E0 */
  0x0000,   /* U+22E1 */
  0x0000,   /* U+22E2 */
  0x0000,   /* U+22E3 */
  0x0000,   /* U+22E4 */
  0x0000,   /* U+22E5 */
  0x0000,   /* U+22E6 */
  0x0000,   /* U+22E7 */
  0x0000,   /* U+22E8 */
  0x0000,   /* U+22E9 */
  0x0000,   /* U+22EA */
  0x0000,   /* U+22EB */
  0x0000,   /* U+22EC */
  0x0000,   /* U+22ED */
  0x0000,   /* U+22EE */
  0x0000,   /* U+22EF */
  0x0000,   /* U+22F0 */
  0x0000,   /* U+22F1 */
  0x0000,   /* U+22F2 */
  0x0000,   /* U+22F3 */
  0x0000,   /* U+22F4 */
  0x0000,   /* U+22F5 */
  0x0000,   /* U+22F6 */
  0x0000,   /* U+22F7 */
  0x0000,   /* U+22F8 */
  0x0000,   /* U+22F9 */
  0x0000,   /* U+22FA */
  0x0000,   /* U+22FB */
  0x0000,   /* U+22FC */
  0x0000,   /* U+22FD */
  0x0000,   /* U+22FE */
  0x0000,   /* U+22FF */
};

uint16 uca900_p023[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 

  /* Primary weight 1 for each character. */
  0x06ED,   /* U+2300 */
  0x06EE,   /* U+2301 */
  0x06EF,   /* U+2302 */
  0x06F0,   /* U+2303 */
  0x06F1,   /* U+2304 */
  0x06F2,   /* U+2305 */
  0x06F3,   /* U+2306 */
  0x06F4,   /* U+2307 */
  0x0325,   /* U+2308 */
  0x0326,   /* U+2309 */
  0x0327,   /* U+230A */
  0x0328,   /* U+230B */
  0x06F5,   /* U+230C */
  0x06F6,   /* U+230D */
  0x06F7,   /* U+230E */
  0x06F8,   /* U+230F */
  0x06F9,   /* U+2310 */
  0x06FA,   /* U+2311 */
  0x06FB,   /* U+2312 */
  0x06FC,   /* U+2313 */
  0x06FD,   /* U+2314 */
  0x06FE,   /* U+2315 */
  0x06FF,   /* U+2316 */
  0x0700,   /* U+2317 */
  0x0701,   /* U+2318 */
  0x0702,   /* U+2319 */
  0x0703,   /* U+231A */
  0x0704,   /* U+231B */
  0x0705,   /* U+231C */
  0x0706,   /* U+231D */
  0x0707,   /* U+231E */
  0x0708,   /* U+231F */
  0x0709,   /* U+2320 */
  0x070A,   /* U+2321 */
  0x070B,   /* U+2322 */
  0x070C,   /* U+2323 */
  0x070D,   /* U+2324 */
  0x070E,   /* U+2325 */
  0x070F,   /* U+2326 */
  0x0710,   /* U+2327 */
  0x0711,   /* U+2328 */
  0x036F,   /* U+2329 */
  0x0370,   /* U+232A */
  0x0712,   /* U+232B */
  0x0713,   /* U+232C */
  0x0714,   /* U+232D */
  0x0715,   /* U+232E */
  0x0716,   /* U+232F */
  0x0717,   /* U+2330 */
  0x0718,   /* U+2331 */
  0x0719,   /* U+2332 */
  0x071A,   /* U+2333 */
  0x071B,   /* U+2334 */
  0x071C,   /* U+2335 */
  0x071D,   /* U+2336 */
  0x071E,   /* U+2337 */
  0x071F,   /* U+2338 */
  0x0720,   /* U+2339 */
  0x0721,   /* U+233A */
  0x0722,   /* U+233B */
  0x0723,   /* U+233C */
  0x0724,   /* U+233D */
  0x0725,   /* U+233E */
  0x0726,   /* U+233F */
  0x0727,   /* U+2340 */
  0x0728,   /* U+2341 */
  0x0729,   /* U+2342 */
  0x072A,   /* U+2343 */
  0x072B,   /* U+2344 */
  0x072C,   /* U+2345 */
  0x072D,   /* U+2346 */
  0x072E,   /* U+2347 */
  0x072F,   /* U+2348 */
  0x0730,   /* U+2349 */
  0x0731,   /* U+234A */
  0x0732,   /* U+234B */
  0x0733,   /* U+234C */
  0x0734,   /* U+234D */
  0x0735,   /* U+234E */
  0x0736,   /* U+234F */
  0x0737,   /* U+2350 */
  0x0738,   /* U+2351 */
  0x0739,   /* U+2352 */
  0x073A,   /* U+2353 */
  0x073B,   /* U+2354 */
  0x073C,   /* U+2355 */
  0x073D,   /* U+2356 */
  0x073E,   /* U+2357 */
  0x073F,   /* U+2358 */
  0x0740,   /* U+2359 */
  0x0741,   /* U+235A */
  0x0742,   /* U+235B */
  0x0743,   /* U+235C */
  0x0744,   /* U+235D */
  0x0745,   /* U+235E */
  0x0746,   /* U+235F */
  0x0747,   /* U+2360 */
  0x0748,   /* U+2361 */
  0x0749,   /* U+2362 */
  0x074A,   /* U+2363 */
  0x074B,   /* U+2364 */
  0x074C,   /* U+2365 */
  0x074D,   /* U+2366 */
  0x074E,   /* U+2367 */
  0x074F,   /* U+2368 */
  0x0750,   /* U+2369 */
  0x0751,   /* U+236A */
  0x0752,   /* U+236B */
  0x0753,   /* U+236C */
  0x0754,   /* U+236D */
  0x0755,   /* U+236E */
  0x0756,   /* U+236F */
  0x0757,   /* U+2370 */
  0x0758,   /* U+2371 */
  0x0759,   /* U+2372 */
  0x075A,   /* U+2373 */
  0x075B,   /* U+2374 */
  0x075C,   /* U+2375 */
  0x075D,   /* U+2376 */
  0x075E,   /* U+2377 */
  0x075F,   /* U+2378 */
  0x0760,   /* U+2379 */
  0x0761,   /* U+237A */
  0x0762,   /* U+237B */
  0x0763,   /* U+237C */
  0x0764,   /* U+237D */
  0x0765,   /* U+237E */
  0x0766,   /* U+237F */
  0x0767,   /* U+2380 */
  0x0768,   /* U+2381 */
  0x0769,   /* U+2382 */
  0x076A,   /* U+2383 */
  0x076B,   /* U+2384 */
  0x076C,   /* U+2385 */
  0x076D,   /* U+2386 */
  0x076E,   /* U+2387 */
  0x076F,   /* U+2388 */
  0x0770,   /* U+2389 */
  0x0771,   /* U+238A */
  0x0772,   /* U+238B */
  0x0773,   /* U+238C */
  0x0774,   /* U+238D */
  0x0775,   /* U+238E */
  0x0776,   /* U+238F */
  0x0777,   /* U+2390 */
  0x0778,   /* U+2391 */
  0x0779,   /* U+2392 */
  0x077A,   /* U+2393 */
  0x077B,   /* U+2394 */
  0x077C,   /* U+2395 */
  0x077D,   /* U+2396 */
  0x077E,   /* U+2397 */
  0x077F,   /* U+2398 */
  0x0780,   /* U+2399 */
  0x0781,   /* U+239A */
  0x0782,   /* U+239B */
  0x0783,   /* U+239C */
  0x0784,   /* U+239D */
  0x0785,   /* U+239E */
  0x0786,   /* U+239F */
  0x0787,   /* U+23A0 */
  0x0788,   /* U+23A1 */
  0x0789,   /* U+23A2 */
  0x078A,   /* U+23A3 */
  0x078B,   /* U+23A4 */
  0x078C,   /* U+23A5 */
  0x078D,   /* U+23A6 */
  0x078E,   /* U+23A7 */
  0x078F,   /* U+23A8 */
  0x0790,   /* U+23A9 */
  0x0791,   /* U+23AA */
  0x0792,   /* U+23AB */
  0x0793,   /* U+23AC */
  0x0794,   /* U+23AD */
  0x0795,   /* U+23AE */
  0x0796,   /* U+23AF */
  0x0797,   /* U+23B0 */
  0x0798,   /* U+23B1 */
  0x0799,   /* U+23B2 */
  0x079A,   /* U+23B3 */
  0x079B,   /* U+23B4 */
  0x079C,   /* U+23B5 */
  0x079D,   /* U+23B6 */
  0x079E,   /* U+23B7 */
  0x079F,   /* U+23B8 */
  0x07A0,   /* U+23B9 */
  0x07A1,   /* U+23BA */
  0x07A2,   /* U+23BB */
  0x07A3,   /* U+23BC */
  0x07A4,   /* U+23BD */
  0x07A5,   /* U+23BE */
  0x07A6,   /* U+23BF */
  0x07A7,   /* U+23C0 */
  0x07A8,   /* U+23C1 */
  0x07A9,   /* U+23C2 */
  0x07AA,   /* U+23C3 */
  0x07AB,   /* U+23C4 */
  0x07AC,   /* U+23C5 */
  0x07AD,   /* U+23C6 */
  0x07AE,   /* U+23C7 */
  0x07AF,   /* U+23C8 */
  0x07B0,   /* U+23C9 */
  0x07B1,   /* U+23CA */
  0x07B2,   /* U+23CB */
  0x07B3,   /* U+23CC */
  0x07B4,   /* U+23CD */
  0x07B5,   /* U+23CE */
  0x07B6,   /* U+23CF */
  0x07B7,   /* U+23D0 */
  0x07B8,   /* U+23D1 */
  0x07B9,   /* U+23D2 */
  0x07BA,   /* U+23D3 */
  0x07BB,   /* U+23D4 */
  0x07BC,   /* U+23D5 */
  0x07BD,   /* U+23D6 */
  0x07BE,   /* U+23D7 */
  0x07BF,   /* U+23D8 */
  0x07C0,   /* U+23D9 */
  0x07C1,   /* U+23DA */
  0x07C2,   /* U+23DB */
  0x07C3,   /* U+23DC */
  0x07C4,   /* U+23DD */
  0x07C5,   /* U+23DE */
  0x07C6,   /* U+23DF */
  0x07C7,   /* U+23E0 */
  0x07C8,   /* U+23E1 */
  0x07C9,   /* U+23E2 */
  0x07CA,   /* U+23E3 */
  0x07CB,   /* U+23E4 */
  0x07CC,   /* U+23E5 */
  0x07CD,   /* U+23E6 */
  0x07CE,   /* U+23E7 */
  0x07CF,   /* U+23E8 */
  0x07D0,   /* U+23E9 */
  0x07D1,   /* U+23EA */
  0x07D2,   /* U+23EB */
  0x07D3,   /* U+23EC */
  0x07D4,   /* U+23ED */
  0x07D5,   /* U+23EE */
  0x07D6,   /* U+23EF */
  0x07D7,   /* U+23F0 */
  0x07D8,   /* U+23F1 */
  0x07D9,   /* U+23F2 */
  0x07DA,   /* U+23F3 */
  0x07DB,   /* U+23F4 */
  0x07DC,   /* U+23F5 */
  0x07DD,   /* U+23F6 */
  0x07DE,   /* U+23F7 */
  0x07DF,   /* U+23F8 */
  0x07E0,   /* U+23F9 */
  0x07E1,   /* U+23FA */
  0x07E2,   /* U+23FB */
  0x07E3,   /* U+23FC */
  0x07E4,   /* U+23FD */
  0x07E5,   /* U+23FE */
  0xFBC0,   /* U+23FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2300 */
  0x0020,   /* U+2301 */
  0x0020,   /* U+2302 */
  0x0020,   /* U+2303 */
  0x0020,   /* U+2304 */
  0x0020,   /* U+2305 */
  0x0020,   /* U+2306 */
  0x0020,   /* U+2307 */
  0x0020,   /* U+2308 */
  0x0020,   /* U+2309 */
  0x0020,   /* U+230A */
  0x0020,   /* U+230B */
  0x0020,   /* U+230C */
  0x0020,   /* U+230D */
  0x0020,   /* U+230E */
  0x0020,   /* U+230F */
  0x0020,   /* U+2310 */
  0x0020,   /* U+2311 */
  0x0020,   /* U+2312 */
  0x0020,   /* U+2313 */
  0x0020,   /* U+2314 */
  0x0020,   /* U+2315 */
  0x0020,   /* U+2316 */
  0x0020,   /* U+2317 */
  0x0020,   /* U+2318 */
  0x0020,   /* U+2319 */
  0x0020,   /* U+231A */
  0x0020,   /* U+231B */
  0x0020,   /* U+231C */
  0x0020,   /* U+231D */
  0x0020,   /* U+231E */
  0x0020,   /* U+231F */
  0x0020,   /* U+2320 */
  0x0020,   /* U+2321 */
  0x0020,   /* U+2322 */
  0x0020,   /* U+2323 */
  0x0020,   /* U+2324 */
  0x0020,   /* U+2325 */
  0x0020,   /* U+2326 */
  0x0020,   /* U+2327 */
  0x0020,   /* U+2328 */
  0x0020,   /* U+2329 */
  0x0020,   /* U+232A */
  0x0020,   /* U+232B */
  0x0020,   /* U+232C */
  0x0020,   /* U+232D */
  0x0020,   /* U+232E */
  0x0020,   /* U+232F */
  0x0020,   /* U+2330 */
  0x0020,   /* U+2331 */
  0x0020,   /* U+2332 */
  0x0020,   /* U+2333 */
  0x0020,   /* U+2334 */
  0x0020,   /* U+2335 */
  0x0020,   /* U+2336 */
  0x0020,   /* U+2337 */
  0x0020,   /* U+2338 */
  0x0020,   /* U+2339 */
  0x0020,   /* U+233A */
  0x0020,   /* U+233B */
  0x0020,   /* U+233C */
  0x0020,   /* U+233D */
  0x0020,   /* U+233E */
  0x0020,   /* U+233F */
  0x0020,   /* U+2340 */
  0x0020,   /* U+2341 */
  0x0020,   /* U+2342 */
  0x0020,   /* U+2343 */
  0x0020,   /* U+2344 */
  0x0020,   /* U+2345 */
  0x0020,   /* U+2346 */
  0x0020,   /* U+2347 */
  0x0020,   /* U+2348 */
  0x0020,   /* U+2349 */
  0x0020,   /* U+234A */
  0x0020,   /* U+234B */
  0x0020,   /* U+234C */
  0x0020,   /* U+234D */
  0x0020,   /* U+234E */
  0x0020,   /* U+234F */
  0x0020,   /* U+2350 */
  0x0020,   /* U+2351 */
  0x0020,   /* U+2352 */
  0x0020,   /* U+2353 */
  0x0020,   /* U+2354 */
  0x0020,   /* U+2355 */
  0x0020,   /* U+2356 */
  0x0020,   /* U+2357 */
  0x0020,   /* U+2358 */
  0x0020,   /* U+2359 */
  0x0020,   /* U+235A */
  0x0020,   /* U+235B */
  0x0020,   /* U+235C */
  0x0020,   /* U+235D */
  0x0020,   /* U+235E */
  0x0020,   /* U+235F */
  0x0020,   /* U+2360 */
  0x0020,   /* U+2361 */
  0x0020,   /* U+2362 */
  0x0020,   /* U+2363 */
  0x0020,   /* U+2364 */
  0x0020,   /* U+2365 */
  0x0020,   /* U+2366 */
  0x0020,   /* U+2367 */
  0x0020,   /* U+2368 */
  0x0020,   /* U+2369 */
  0x0020,   /* U+236A */
  0x0020,   /* U+236B */
  0x0020,   /* U+236C */
  0x0020,   /* U+236D */
  0x0020,   /* U+236E */
  0x0020,   /* U+236F */
  0x0020,   /* U+2370 */
  0x0020,   /* U+2371 */
  0x0020,   /* U+2372 */
  0x0020,   /* U+2373 */
  0x0020,   /* U+2374 */
  0x0020,   /* U+2375 */
  0x0020,   /* U+2376 */
  0x0020,   /* U+2377 */
  0x0020,   /* U+2378 */
  0x0020,   /* U+2379 */
  0x0020,   /* U+237A */
  0x0020,   /* U+237B */
  0x0020,   /* U+237C */
  0x0020,   /* U+237D */
  0x0020,   /* U+237E */
  0x0020,   /* U+237F */
  0x0020,   /* U+2380 */
  0x0020,   /* U+2381 */
  0x0020,   /* U+2382 */
  0x0020,   /* U+2383 */
  0x0020,   /* U+2384 */
  0x0020,   /* U+2385 */
  0x0020,   /* U+2386 */
  0x0020,   /* U+2387 */
  0x0020,   /* U+2388 */
  0x0020,   /* U+2389 */
  0x0020,   /* U+238A */
  0x0020,   /* U+238B */
  0x0020,   /* U+238C */
  0x0020,   /* U+238D */
  0x0020,   /* U+238E */
  0x0020,   /* U+238F */
  0x0020,   /* U+2390 */
  0x0020,   /* U+2391 */
  0x0020,   /* U+2392 */
  0x0020,   /* U+2393 */
  0x0020,   /* U+2394 */
  0x0020,   /* U+2395 */
  0x0020,   /* U+2396 */
  0x0020,   /* U+2397 */
  0x0020,   /* U+2398 */
  0x0020,   /* U+2399 */
  0x0020,   /* U+239A */
  0x0020,   /* U+239B */
  0x0020,   /* U+239C */
  0x0020,   /* U+239D */
  0x0020,   /* U+239E */
  0x0020,   /* U+239F */
  0x0020,   /* U+23A0 */
  0x0020,   /* U+23A1 */
  0x0020,   /* U+23A2 */
  0x0020,   /* U+23A3 */
  0x0020,   /* U+23A4 */
  0x0020,   /* U+23A5 */
  0x0020,   /* U+23A6 */
  0x0020,   /* U+23A7 */
  0x0020,   /* U+23A8 */
  0x0020,   /* U+23A9 */
  0x0020,   /* U+23AA */
  0x0020,   /* U+23AB */
  0x0020,   /* U+23AC */
  0x0020,   /* U+23AD */
  0x0020,   /* U+23AE */
  0x0020,   /* U+23AF */
  0x0020,   /* U+23B0 */
  0x0020,   /* U+23B1 */
  0x0020,   /* U+23B2 */
  0x0020,   /* U+23B3 */
  0x0020,   /* U+23B4 */
  0x0020,   /* U+23B5 */
  0x0020,   /* U+23B6 */
  0x0020,   /* U+23B7 */
  0x0020,   /* U+23B8 */
  0x0020,   /* U+23B9 */
  0x0020,   /* U+23BA */
  0x0020,   /* U+23BB */
  0x0020,   /* U+23BC */
  0x0020,   /* U+23BD */
  0x0020,   /* U+23BE */
  0x0020,   /* U+23BF */
  0x0020,   /* U+23C0 */
  0x0020,   /* U+23C1 */
  0x0020,   /* U+23C2 */
  0x0020,   /* U+23C3 */
  0x0020,   /* U+23C4 */
  0x0020,   /* U+23C5 */
  0x0020,   /* U+23C6 */
  0x0020,   /* U+23C7 */
  0x0020,   /* U+23C8 */
  0x0020,   /* U+23C9 */
  0x0020,   /* U+23CA */
  0x0020,   /* U+23CB */
  0x0020,   /* U+23CC */
  0x0020,   /* U+23CD */
  0x0020,   /* U+23CE */
  0x0020,   /* U+23CF */
  0x0020,   /* U+23D0 */
  0x0020,   /* U+23D1 */
  0x0020,   /* U+23D2 */
  0x0020,   /* U+23D3 */
  0x0020,   /* U+23D4 */
  0x0020,   /* U+23D5 */
  0x0020,   /* U+23D6 */
  0x0020,   /* U+23D7 */
  0x0020,   /* U+23D8 */
  0x0020,   /* U+23D9 */
  0x0020,   /* U+23DA */
  0x0020,   /* U+23DB */
  0x0020,   /* U+23DC */
  0x0020,   /* U+23DD */
  0x0020,   /* U+23DE */
  0x0020,   /* U+23DF */
  0x0020,   /* U+23E0 */
  0x0020,   /* U+23E1 */
  0x0020,   /* U+23E2 */
  0x0020,   /* U+23E3 */
  0x0020,   /* U+23E4 */
  0x0020,   /* U+23E5 */
  0x0020,   /* U+23E6 */
  0x0020,   /* U+23E7 */
  0x0020,   /* U+23E8 */
  0x0020,   /* U+23E9 */
  0x0020,   /* U+23EA */
  0x0020,   /* U+23EB */
  0x0020,   /* U+23EC */
  0x0020,   /* U+23ED */
  0x0020,   /* U+23EE */
  0x0020,   /* U+23EF */
  0x0020,   /* U+23F0 */
  0x0020,   /* U+23F1 */
  0x0020,   /* U+23F2 */
  0x0020,   /* U+23F3 */
  0x0020,   /* U+23F4 */
  0x0020,   /* U+23F5 */
  0x0020,   /* U+23F6 */
  0x0020,   /* U+23F7 */
  0x0020,   /* U+23F8 */
  0x0020,   /* U+23F9 */
  0x0020,   /* U+23FA */
  0x0020,   /* U+23FB */
  0x0020,   /* U+23FC */
  0x0020,   /* U+23FD */
  0x0020,   /* U+23FE */
  0x0020,   /* U+23FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2300 */
  0x0002,   /* U+2301 */
  0x0002,   /* U+2302 */
  0x0002,   /* U+2303 */
  0x0002,   /* U+2304 */
  0x0002,   /* U+2305 */
  0x0002,   /* U+2306 */
  0x0002,   /* U+2307 */
  0x0002,   /* U+2308 */
  0x0002,   /* U+2309 */
  0x0002,   /* U+230A */
  0x0002,   /* U+230B */
  0x0002,   /* U+230C */
  0x0002,   /* U+230D */
  0x0002,   /* U+230E */
  0x0002,   /* U+230F */
  0x0002,   /* U+2310 */
  0x0002,   /* U+2311 */
  0x0002,   /* U+2312 */
  0x0002,   /* U+2313 */
  0x0002,   /* U+2314 */
  0x0002,   /* U+2315 */
  0x0002,   /* U+2316 */
  0x0002,   /* U+2317 */
  0x0002,   /* U+2318 */
  0x0002,   /* U+2319 */
  0x0002,   /* U+231A */
  0x0002,   /* U+231B */
  0x0002,   /* U+231C */
  0x0002,   /* U+231D */
  0x0002,   /* U+231E */
  0x0002,   /* U+231F */
  0x0002,   /* U+2320 */
  0x0002,   /* U+2321 */
  0x0002,   /* U+2322 */
  0x0002,   /* U+2323 */
  0x0002,   /* U+2324 */
  0x0002,   /* U+2325 */
  0x0002,   /* U+2326 */
  0x0002,   /* U+2327 */
  0x0002,   /* U+2328 */
  0x0002,   /* U+2329 */
  0x0002,   /* U+232A */
  0x0002,   /* U+232B */
  0x0002,   /* U+232C */
  0x0002,   /* U+232D */
  0x0002,   /* U+232E */
  0x0002,   /* U+232F */
  0x0002,   /* U+2330 */
  0x0002,   /* U+2331 */
  0x0002,   /* U+2332 */
  0x0002,   /* U+2333 */
  0x0002,   /* U+2334 */
  0x0002,   /* U+2335 */
  0x0002,   /* U+2336 */
  0x0002,   /* U+2337 */
  0x0002,   /* U+2338 */
  0x0002,   /* U+2339 */
  0x0002,   /* U+233A */
  0x0002,   /* U+233B */
  0x0002,   /* U+233C */
  0x0002,   /* U+233D */
  0x0002,   /* U+233E */
  0x0002,   /* U+233F */
  0x0002,   /* U+2340 */
  0x0002,   /* U+2341 */
  0x0002,   /* U+2342 */
  0x0002,   /* U+2343 */
  0x0002,   /* U+2344 */
  0x0002,   /* U+2345 */
  0x0002,   /* U+2346 */
  0x0002,   /* U+2347 */
  0x0002,   /* U+2348 */
  0x0002,   /* U+2349 */
  0x0002,   /* U+234A */
  0x0002,   /* U+234B */
  0x0002,   /* U+234C */
  0x0002,   /* U+234D */
  0x0002,   /* U+234E */
  0x0002,   /* U+234F */
  0x0002,   /* U+2350 */
  0x0002,   /* U+2351 */
  0x0002,   /* U+2352 */
  0x0002,   /* U+2353 */
  0x0002,   /* U+2354 */
  0x0002,   /* U+2355 */
  0x0002,   /* U+2356 */
  0x0002,   /* U+2357 */
  0x0002,   /* U+2358 */
  0x0002,   /* U+2359 */
  0x0002,   /* U+235A */
  0x0002,   /* U+235B */
  0x0002,   /* U+235C */
  0x0002,   /* U+235D */
  0x0002,   /* U+235E */
  0x0002,   /* U+235F */
  0x0002,   /* U+2360 */
  0x0002,   /* U+2361 */
  0x0002,   /* U+2362 */
  0x0002,   /* U+2363 */
  0x0002,   /* U+2364 */
  0x0002,   /* U+2365 */
  0x0002,   /* U+2366 */
  0x0002,   /* U+2367 */
  0x0002,   /* U+2368 */
  0x0002,   /* U+2369 */
  0x0002,   /* U+236A */
  0x0002,   /* U+236B */
  0x0002,   /* U+236C */
  0x0002,   /* U+236D */
  0x0002,   /* U+236E */
  0x0002,   /* U+236F */
  0x0002,   /* U+2370 */
  0x0002,   /* U+2371 */
  0x0002,   /* U+2372 */
  0x0002,   /* U+2373 */
  0x0002,   /* U+2374 */
  0x0002,   /* U+2375 */
  0x0002,   /* U+2376 */
  0x0002,   /* U+2377 */
  0x0002,   /* U+2378 */
  0x0002,   /* U+2379 */
  0x0002,   /* U+237A */
  0x0002,   /* U+237B */
  0x0002,   /* U+237C */
  0x0002,   /* U+237D */
  0x0002,   /* U+237E */
  0x0002,   /* U+237F */
  0x0002,   /* U+2380 */
  0x0002,   /* U+2381 */
  0x0002,   /* U+2382 */
  0x0002,   /* U+2383 */
  0x0002,   /* U+2384 */
  0x0002,   /* U+2385 */
  0x0002,   /* U+2386 */
  0x0002,   /* U+2387 */
  0x0002,   /* U+2388 */
  0x0002,   /* U+2389 */
  0x0002,   /* U+238A */
  0x0002,   /* U+238B */
  0x0002,   /* U+238C */
  0x0002,   /* U+238D */
  0x0002,   /* U+238E */
  0x0002,   /* U+238F */
  0x0002,   /* U+2390 */
  0x0002,   /* U+2391 */
  0x0002,   /* U+2392 */
  0x0002,   /* U+2393 */
  0x0002,   /* U+2394 */
  0x0002,   /* U+2395 */
  0x0002,   /* U+2396 */
  0x0002,   /* U+2397 */
  0x0002,   /* U+2398 */
  0x0002,   /* U+2399 */
  0x0002,   /* U+239A */
  0x0002,   /* U+239B */
  0x0002,   /* U+239C */
  0x0002,   /* U+239D */
  0x0002,   /* U+239E */
  0x0002,   /* U+239F */
  0x0002,   /* U+23A0 */
  0x0002,   /* U+23A1 */
  0x0002,   /* U+23A2 */
  0x0002,   /* U+23A3 */
  0x0002,   /* U+23A4 */
  0x0002,   /* U+23A5 */
  0x0002,   /* U+23A6 */
  0x0002,   /* U+23A7 */
  0x0002,   /* U+23A8 */
  0x0002,   /* U+23A9 */
  0x0002,   /* U+23AA */
  0x0002,   /* U+23AB */
  0x0002,   /* U+23AC */
  0x0002,   /* U+23AD */
  0x0002,   /* U+23AE */
  0x0002,   /* U+23AF */
  0x0002,   /* U+23B0 */
  0x0002,   /* U+23B1 */
  0x0002,   /* U+23B2 */
  0x0002,   /* U+23B3 */
  0x0002,   /* U+23B4 */
  0x0002,   /* U+23B5 */
  0x0002,   /* U+23B6 */
  0x0002,   /* U+23B7 */
  0x0002,   /* U+23B8 */
  0x0002,   /* U+23B9 */
  0x0002,   /* U+23BA */
  0x0002,   /* U+23BB */
  0x0002,   /* U+23BC */
  0x0002,   /* U+23BD */
  0x0002,   /* U+23BE */
  0x0002,   /* U+23BF */
  0x0002,   /* U+23C0 */
  0x0002,   /* U+23C1 */
  0x0002,   /* U+23C2 */
  0x0002,   /* U+23C3 */
  0x0002,   /* U+23C4 */
  0x0002,   /* U+23C5 */
  0x0002,   /* U+23C6 */
  0x0002,   /* U+23C7 */
  0x0002,   /* U+23C8 */
  0x0002,   /* U+23C9 */
  0x0002,   /* U+23CA */
  0x0002,   /* U+23CB */
  0x0002,   /* U+23CC */
  0x0002,   /* U+23CD */
  0x0002,   /* U+23CE */
  0x0002,   /* U+23CF */
  0x0002,   /* U+23D0 */
  0x0002,   /* U+23D1 */
  0x0002,   /* U+23D2 */
  0x0002,   /* U+23D3 */
  0x0002,   /* U+23D4 */
  0x0002,   /* U+23D5 */
  0x0002,   /* U+23D6 */
  0x0002,   /* U+23D7 */
  0x0002,   /* U+23D8 */
  0x0002,   /* U+23D9 */
  0x0002,   /* U+23DA */
  0x0002,   /* U+23DB */
  0x0002,   /* U+23DC */
  0x0002,   /* U+23DD */
  0x0002,   /* U+23DE */
  0x0002,   /* U+23DF */
  0x0002,   /* U+23E0 */
  0x0002,   /* U+23E1 */
  0x0002,   /* U+23E2 */
  0x0002,   /* U+23E3 */
  0x0002,   /* U+23E4 */
  0x0002,   /* U+23E5 */
  0x0002,   /* U+23E6 */
  0x0002,   /* U+23E7 */
  0x0002,   /* U+23E8 */
  0x0002,   /* U+23E9 */
  0x0002,   /* U+23EA */
  0x0002,   /* U+23EB */
  0x0002,   /* U+23EC */
  0x0002,   /* U+23ED */
  0x0002,   /* U+23EE */
  0x0002,   /* U+23EF */
  0x0002,   /* U+23F0 */
  0x0002,   /* U+23F1 */
  0x0002,   /* U+23F2 */
  0x0002,   /* U+23F3 */
  0x0002,   /* U+23F4 */
  0x0002,   /* U+23F5 */
  0x0002,   /* U+23F6 */
  0x0002,   /* U+23F7 */
  0x0002,   /* U+23F8 */
  0x0002,   /* U+23F9 */
  0x0002,   /* U+23FA */
  0x0002,   /* U+23FB */
  0x0002,   /* U+23FC */
  0x0002,   /* U+23FD */
  0x0002,   /* U+23FE */
  0x0002,   /* U+23FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2300 */
  0x0000,   /* U+2301 */
  0x0000,   /* U+2302 */
  0x0000,   /* U+2303 */
  0x0000,   /* U+2304 */
  0x0000,   /* U+2305 */
  0x0000,   /* U+2306 */
  0x0000,   /* U+2307 */
  0x0000,   /* U+2308 */
  0x0000,   /* U+2309 */
  0x0000,   /* U+230A */
  0x0000,   /* U+230B */
  0x0000,   /* U+230C */
  0x0000,   /* U+230D */
  0x0000,   /* U+230E */
  0x0000,   /* U+230F */
  0x0000,   /* U+2310 */
  0x0000,   /* U+2311 */
  0x0000,   /* U+2312 */
  0x0000,   /* U+2313 */
  0x0000,   /* U+2314 */
  0x0000,   /* U+2315 */
  0x0000,   /* U+2316 */
  0x0000,   /* U+2317 */
  0x0000,   /* U+2318 */
  0x0000,   /* U+2319 */
  0x0000,   /* U+231A */
  0x0000,   /* U+231B */
  0x0000,   /* U+231C */
  0x0000,   /* U+231D */
  0x0000,   /* U+231E */
  0x0000,   /* U+231F */
  0x0000,   /* U+2320 */
  0x0000,   /* U+2321 */
  0x0000,   /* U+2322 */
  0x0000,   /* U+2323 */
  0x0000,   /* U+2324 */
  0x0000,   /* U+2325 */
  0x0000,   /* U+2326 */
  0x0000,   /* U+2327 */
  0x0000,   /* U+2328 */
  0x0000,   /* U+2329 */
  0x0000,   /* U+232A */
  0x0000,   /* U+232B */
  0x0000,   /* U+232C */
  0x0000,   /* U+232D */
  0x0000,   /* U+232E */
  0x0000,   /* U+232F */
  0x0000,   /* U+2330 */
  0x0000,   /* U+2331 */
  0x0000,   /* U+2332 */
  0x0000,   /* U+2333 */
  0x0000,   /* U+2334 */
  0x0000,   /* U+2335 */
  0x0000,   /* U+2336 */
  0x0000,   /* U+2337 */
  0x0000,   /* U+2338 */
  0x0000,   /* U+2339 */
  0x0000,   /* U+233A */
  0x0000,   /* U+233B */
  0x0000,   /* U+233C */
  0x0000,   /* U+233D */
  0x0000,   /* U+233E */
  0x0000,   /* U+233F */
  0x0000,   /* U+2340 */
  0x0000,   /* U+2341 */
  0x0000,   /* U+2342 */
  0x0000,   /* U+2343 */
  0x0000,   /* U+2344 */
  0x0000,   /* U+2345 */
  0x0000,   /* U+2346 */
  0x0000,   /* U+2347 */
  0x0000,   /* U+2348 */
  0x0000,   /* U+2349 */
  0x0000,   /* U+234A */
  0x0000,   /* U+234B */
  0x0000,   /* U+234C */
  0x0000,   /* U+234D */
  0x0000,   /* U+234E */
  0x0000,   /* U+234F */
  0x0000,   /* U+2350 */
  0x0000,   /* U+2351 */
  0x0000,   /* U+2352 */
  0x0000,   /* U+2353 */
  0x0000,   /* U+2354 */
  0x0000,   /* U+2355 */
  0x0000,   /* U+2356 */
  0x0000,   /* U+2357 */
  0x0000,   /* U+2358 */
  0x0000,   /* U+2359 */
  0x0000,   /* U+235A */
  0x0000,   /* U+235B */
  0x0000,   /* U+235C */
  0x0000,   /* U+235D */
  0x0000,   /* U+235E */
  0x0000,   /* U+235F */
  0x0000,   /* U+2360 */
  0x0000,   /* U+2361 */
  0x0000,   /* U+2362 */
  0x0000,   /* U+2363 */
  0x0000,   /* U+2364 */
  0x0000,   /* U+2365 */
  0x0000,   /* U+2366 */
  0x0000,   /* U+2367 */
  0x0000,   /* U+2368 */
  0x0000,   /* U+2369 */
  0x0000,   /* U+236A */
  0x0000,   /* U+236B */
  0x0000,   /* U+236C */
  0x0000,   /* U+236D */
  0x0000,   /* U+236E */
  0x0000,   /* U+236F */
  0x0000,   /* U+2370 */
  0x0000,   /* U+2371 */
  0x0000,   /* U+2372 */
  0x0000,   /* U+2373 */
  0x0000,   /* U+2374 */
  0x0000,   /* U+2375 */
  0x0000,   /* U+2376 */
  0x0000,   /* U+2377 */
  0x0000,   /* U+2378 */
  0x0000,   /* U+2379 */
  0x0000,   /* U+237A */
  0x0000,   /* U+237B */
  0x0000,   /* U+237C */
  0x0000,   /* U+237D */
  0x0000,   /* U+237E */
  0x0000,   /* U+237F */
  0x0000,   /* U+2380 */
  0x0000,   /* U+2381 */
  0x0000,   /* U+2382 */
  0x0000,   /* U+2383 */
  0x0000,   /* U+2384 */
  0x0000,   /* U+2385 */
  0x0000,   /* U+2386 */
  0x0000,   /* U+2387 */
  0x0000,   /* U+2388 */
  0x0000,   /* U+2389 */
  0x0000,   /* U+238A */
  0x0000,   /* U+238B */
  0x0000,   /* U+238C */
  0x0000,   /* U+238D */
  0x0000,   /* U+238E */
  0x0000,   /* U+238F */
  0x0000,   /* U+2390 */
  0x0000,   /* U+2391 */
  0x0000,   /* U+2392 */
  0x0000,   /* U+2393 */
  0x0000,   /* U+2394 */
  0x0000,   /* U+2395 */
  0x0000,   /* U+2396 */
  0x0000,   /* U+2397 */
  0x0000,   /* U+2398 */
  0x0000,   /* U+2399 */
  0x0000,   /* U+239A */
  0x0000,   /* U+239B */
  0x0000,   /* U+239C */
  0x0000,   /* U+239D */
  0x0000,   /* U+239E */
  0x0000,   /* U+239F */
  0x0000,   /* U+23A0 */
  0x0000,   /* U+23A1 */
  0x0000,   /* U+23A2 */
  0x0000,   /* U+23A3 */
  0x0000,   /* U+23A4 */
  0x0000,   /* U+23A5 */
  0x0000,   /* U+23A6 */
  0x0000,   /* U+23A7 */
  0x0000,   /* U+23A8 */
  0x0000,   /* U+23A9 */
  0x0000,   /* U+23AA */
  0x0000,   /* U+23AB */
  0x0000,   /* U+23AC */
  0x0000,   /* U+23AD */
  0x0000,   /* U+23AE */
  0x0000,   /* U+23AF */
  0x0000,   /* U+23B0 */
  0x0000,   /* U+23B1 */
  0x0000,   /* U+23B2 */
  0x0000,   /* U+23B3 */
  0x0000,   /* U+23B4 */
  0x0000,   /* U+23B5 */
  0x0000,   /* U+23B6 */
  0x0000,   /* U+23B7 */
  0x0000,   /* U+23B8 */
  0x0000,   /* U+23B9 */
  0x0000,   /* U+23BA */
  0x0000,   /* U+23BB */
  0x0000,   /* U+23BC */
  0x0000,   /* U+23BD */
  0x0000,   /* U+23BE */
  0x0000,   /* U+23BF */
  0x0000,   /* U+23C0 */
  0x0000,   /* U+23C1 */
  0x0000,   /* U+23C2 */
  0x0000,   /* U+23C3 */
  0x0000,   /* U+23C4 */
  0x0000,   /* U+23C5 */
  0x0000,   /* U+23C6 */
  0x0000,   /* U+23C7 */
  0x0000,   /* U+23C8 */
  0x0000,   /* U+23C9 */
  0x0000,   /* U+23CA */
  0x0000,   /* U+23CB */
  0x0000,   /* U+23CC */
  0x0000,   /* U+23CD */
  0x0000,   /* U+23CE */
  0x0000,   /* U+23CF */
  0x0000,   /* U+23D0 */
  0x0000,   /* U+23D1 */
  0x0000,   /* U+23D2 */
  0x0000,   /* U+23D3 */
  0x0000,   /* U+23D4 */
  0x0000,   /* U+23D5 */
  0x0000,   /* U+23D6 */
  0x0000,   /* U+23D7 */
  0x0000,   /* U+23D8 */
  0x0000,   /* U+23D9 */
  0x0000,   /* U+23DA */
  0x0000,   /* U+23DB */
  0x0000,   /* U+23DC */
  0x0000,   /* U+23DD */
  0x0000,   /* U+23DE */
  0x0000,   /* U+23DF */
  0x0000,   /* U+23E0 */
  0x0000,   /* U+23E1 */
  0x0000,   /* U+23E2 */
  0x0000,   /* U+23E3 */
  0x0000,   /* U+23E4 */
  0x0000,   /* U+23E5 */
  0x0000,   /* U+23E6 */
  0x0000,   /* U+23E7 */
  0x0000,   /* U+23E8 */
  0x0000,   /* U+23E9 */
  0x0000,   /* U+23EA */
  0x0000,   /* U+23EB */
  0x0000,   /* U+23EC */
  0x0000,   /* U+23ED */
  0x0000,   /* U+23EE */
  0x0000,   /* U+23EF */
  0x0000,   /* U+23F0 */
  0x0000,   /* U+23F1 */
  0x0000,   /* U+23F2 */
  0x0000,   /* U+23F3 */
  0x0000,   /* U+23F4 */
  0x0000,   /* U+23F5 */
  0x0000,   /* U+23F6 */
  0x0000,   /* U+23F7 */
  0x0000,   /* U+23F8 */
  0x0000,   /* U+23F9 */
  0x0000,   /* U+23FA */
  0x0000,   /* U+23FB */
  0x0000,   /* U+23FC */
  0x0000,   /* U+23FD */
  0x0000,   /* U+23FE */
  0xA3FF,   /* U+23FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2300 */
  0x0000,   /* U+2301 */
  0x0000,   /* U+2302 */
  0x0000,   /* U+2303 */
  0x0000,   /* U+2304 */
  0x0000,   /* U+2305 */
  0x0000,   /* U+2306 */
  0x0000,   /* U+2307 */
  0x0000,   /* U+2308 */
  0x0000,   /* U+2309 */
  0x0000,   /* U+230A */
  0x0000,   /* U+230B */
  0x0000,   /* U+230C */
  0x0000,   /* U+230D */
  0x0000,   /* U+230E */
  0x0000,   /* U+230F */
  0x0000,   /* U+2310 */
  0x0000,   /* U+2311 */
  0x0000,   /* U+2312 */
  0x0000,   /* U+2313 */
  0x0000,   /* U+2314 */
  0x0000,   /* U+2315 */
  0x0000,   /* U+2316 */
  0x0000,   /* U+2317 */
  0x0000,   /* U+2318 */
  0x0000,   /* U+2319 */
  0x0000,   /* U+231A */
  0x0000,   /* U+231B */
  0x0000,   /* U+231C */
  0x0000,   /* U+231D */
  0x0000,   /* U+231E */
  0x0000,   /* U+231F */
  0x0000,   /* U+2320 */
  0x0000,   /* U+2321 */
  0x0000,   /* U+2322 */
  0x0000,   /* U+2323 */
  0x0000,   /* U+2324 */
  0x0000,   /* U+2325 */
  0x0000,   /* U+2326 */
  0x0000,   /* U+2327 */
  0x0000,   /* U+2328 */
  0x0000,   /* U+2329 */
  0x0000,   /* U+232A */
  0x0000,   /* U+232B */
  0x0000,   /* U+232C */
  0x0000,   /* U+232D */
  0x0000,   /* U+232E */
  0x0000,   /* U+232F */
  0x0000,   /* U+2330 */
  0x0000,   /* U+2331 */
  0x0000,   /* U+2332 */
  0x0000,   /* U+2333 */
  0x0000,   /* U+2334 */
  0x0000,   /* U+2335 */
  0x0000,   /* U+2336 */
  0x0000,   /* U+2337 */
  0x0000,   /* U+2338 */
  0x0000,   /* U+2339 */
  0x0000,   /* U+233A */
  0x0000,   /* U+233B */
  0x0000,   /* U+233C */
  0x0000,   /* U+233D */
  0x0000,   /* U+233E */
  0x0000,   /* U+233F */
  0x0000,   /* U+2340 */
  0x0000,   /* U+2341 */
  0x0000,   /* U+2342 */
  0x0000,   /* U+2343 */
  0x0000,   /* U+2344 */
  0x0000,   /* U+2345 */
  0x0000,   /* U+2346 */
  0x0000,   /* U+2347 */
  0x0000,   /* U+2348 */
  0x0000,   /* U+2349 */
  0x0000,   /* U+234A */
  0x0000,   /* U+234B */
  0x0000,   /* U+234C */
  0x0000,   /* U+234D */
  0x0000,   /* U+234E */
  0x0000,   /* U+234F */
  0x0000,   /* U+2350 */
  0x0000,   /* U+2351 */
  0x0000,   /* U+2352 */
  0x0000,   /* U+2353 */
  0x0000,   /* U+2354 */
  0x0000,   /* U+2355 */
  0x0000,   /* U+2356 */
  0x0000,   /* U+2357 */
  0x0000,   /* U+2358 */
  0x0000,   /* U+2359 */
  0x0000,   /* U+235A */
  0x0000,   /* U+235B */
  0x0000,   /* U+235C */
  0x0000,   /* U+235D */
  0x0000,   /* U+235E */
  0x0000,   /* U+235F */
  0x0000,   /* U+2360 */
  0x0000,   /* U+2361 */
  0x0000,   /* U+2362 */
  0x0000,   /* U+2363 */
  0x0000,   /* U+2364 */
  0x0000,   /* U+2365 */
  0x0000,   /* U+2366 */
  0x0000,   /* U+2367 */
  0x0000,   /* U+2368 */
  0x0000,   /* U+2369 */
  0x0000,   /* U+236A */
  0x0000,   /* U+236B */
  0x0000,   /* U+236C */
  0x0000,   /* U+236D */
  0x0000,   /* U+236E */
  0x0000,   /* U+236F */
  0x0000,   /* U+2370 */
  0x0000,   /* U+2371 */
  0x0000,   /* U+2372 */
  0x0000,   /* U+2373 */
  0x0000,   /* U+2374 */
  0x0000,   /* U+2375 */
  0x0000,   /* U+2376 */
  0x0000,   /* U+2377 */
  0x0000,   /* U+2378 */
  0x0000,   /* U+2379 */
  0x0000,   /* U+237A */
  0x0000,   /* U+237B */
  0x0000,   /* U+237C */
  0x0000,   /* U+237D */
  0x0000,   /* U+237E */
  0x0000,   /* U+237F */
  0x0000,   /* U+2380 */
  0x0000,   /* U+2381 */
  0x0000,   /* U+2382 */
  0x0000,   /* U+2383 */
  0x0000,   /* U+2384 */
  0x0000,   /* U+2385 */
  0x0000,   /* U+2386 */
  0x0000,   /* U+2387 */
  0x0000,   /* U+2388 */
  0x0000,   /* U+2389 */
  0x0000,   /* U+238A */
  0x0000,   /* U+238B */
  0x0000,   /* U+238C */
  0x0000,   /* U+238D */
  0x0000,   /* U+238E */
  0x0000,   /* U+238F */
  0x0000,   /* U+2390 */
  0x0000,   /* U+2391 */
  0x0000,   /* U+2392 */
  0x0000,   /* U+2393 */
  0x0000,   /* U+2394 */
  0x0000,   /* U+2395 */
  0x0000,   /* U+2396 */
  0x0000,   /* U+2397 */
  0x0000,   /* U+2398 */
  0x0000,   /* U+2399 */
  0x0000,   /* U+239A */
  0x0000,   /* U+239B */
  0x0000,   /* U+239C */
  0x0000,   /* U+239D */
  0x0000,   /* U+239E */
  0x0000,   /* U+239F */
  0x0000,   /* U+23A0 */
  0x0000,   /* U+23A1 */
  0x0000,   /* U+23A2 */
  0x0000,   /* U+23A3 */
  0x0000,   /* U+23A4 */
  0x0000,   /* U+23A5 */
  0x0000,   /* U+23A6 */
  0x0000,   /* U+23A7 */
  0x0000,   /* U+23A8 */
  0x0000,   /* U+23A9 */
  0x0000,   /* U+23AA */
  0x0000,   /* U+23AB */
  0x0000,   /* U+23AC */
  0x0000,   /* U+23AD */
  0x0000,   /* U+23AE */
  0x0000,   /* U+23AF */
  0x0000,   /* U+23B0 */
  0x0000,   /* U+23B1 */
  0x0000,   /* U+23B2 */
  0x0000,   /* U+23B3 */
  0x0000,   /* U+23B4 */
  0x0000,   /* U+23B5 */
  0x0000,   /* U+23B6 */
  0x0000,   /* U+23B7 */
  0x0000,   /* U+23B8 */
  0x0000,   /* U+23B9 */
  0x0000,   /* U+23BA */
  0x0000,   /* U+23BB */
  0x0000,   /* U+23BC */
  0x0000,   /* U+23BD */
  0x0000,   /* U+23BE */
  0x0000,   /* U+23BF */
  0x0000,   /* U+23C0 */
  0x0000,   /* U+23C1 */
  0x0000,   /* U+23C2 */
  0x0000,   /* U+23C3 */
  0x0000,   /* U+23C4 */
  0x0000,   /* U+23C5 */
  0x0000,   /* U+23C6 */
  0x0000,   /* U+23C7 */
  0x0000,   /* U+23C8 */
  0x0000,   /* U+23C9 */
  0x0000,   /* U+23CA */
  0x0000,   /* U+23CB */
  0x0000,   /* U+23CC */
  0x0000,   /* U+23CD */
  0x0000,   /* U+23CE */
  0x0000,   /* U+23CF */
  0x0000,   /* U+23D0 */
  0x0000,   /* U+23D1 */
  0x0000,   /* U+23D2 */
  0x0000,   /* U+23D3 */
  0x0000,   /* U+23D4 */
  0x0000,   /* U+23D5 */
  0x0000,   /* U+23D6 */
  0x0000,   /* U+23D7 */
  0x0000,   /* U+23D8 */
  0x0000,   /* U+23D9 */
  0x0000,   /* U+23DA */
  0x0000,   /* U+23DB */
  0x0000,   /* U+23DC */
  0x0000,   /* U+23DD */
  0x0000,   /* U+23DE */
  0x0000,   /* U+23DF */
  0x0000,   /* U+23E0 */
  0x0000,   /* U+23E1 */
  0x0000,   /* U+23E2 */
  0x0000,   /* U+23E3 */
  0x0000,   /* U+23E4 */
  0x0000,   /* U+23E5 */
  0x0000,   /* U+23E6 */
  0x0000,   /* U+23E7 */
  0x0000,   /* U+23E8 */
  0x0000,   /* U+23E9 */
  0x0000,   /* U+23EA */
  0x0000,   /* U+23EB */
  0x0000,   /* U+23EC */
  0x0000,   /* U+23ED */
  0x0000,   /* U+23EE */
  0x0000,   /* U+23EF */
  0x0000,   /* U+23F0 */
  0x0000,   /* U+23F1 */
  0x0000,   /* U+23F2 */
  0x0000,   /* U+23F3 */
  0x0000,   /* U+23F4 */
  0x0000,   /* U+23F5 */
  0x0000,   /* U+23F6 */
  0x0000,   /* U+23F7 */
  0x0000,   /* U+23F8 */
  0x0000,   /* U+23F9 */
  0x0000,   /* U+23FA */
  0x0000,   /* U+23FB */
  0x0000,   /* U+23FC */
  0x0000,   /* U+23FD */
  0x0000,   /* U+23FE */
  0x0000,   /* U+23FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2300 */
  0x0000,   /* U+2301 */
  0x0000,   /* U+2302 */
  0x0000,   /* U+2303 */
  0x0000,   /* U+2304 */
  0x0000,   /* U+2305 */
  0x0000,   /* U+2306 */
  0x0000,   /* U+2307 */
  0x0000,   /* U+2308 */
  0x0000,   /* U+2309 */
  0x0000,   /* U+230A */
  0x0000,   /* U+230B */
  0x0000,   /* U+230C */
  0x0000,   /* U+230D */
  0x0000,   /* U+230E */
  0x0000,   /* U+230F */
  0x0000,   /* U+2310 */
  0x0000,   /* U+2311 */
  0x0000,   /* U+2312 */
  0x0000,   /* U+2313 */
  0x0000,   /* U+2314 */
  0x0000,   /* U+2315 */
  0x0000,   /* U+2316 */
  0x0000,   /* U+2317 */
  0x0000,   /* U+2318 */
  0x0000,   /* U+2319 */
  0x0000,   /* U+231A */
  0x0000,   /* U+231B */
  0x0000,   /* U+231C */
  0x0000,   /* U+231D */
  0x0000,   /* U+231E */
  0x0000,   /* U+231F */
  0x0000,   /* U+2320 */
  0x0000,   /* U+2321 */
  0x0000,   /* U+2322 */
  0x0000,   /* U+2323 */
  0x0000,   /* U+2324 */
  0x0000,   /* U+2325 */
  0x0000,   /* U+2326 */
  0x0000,   /* U+2327 */
  0x0000,   /* U+2328 */
  0x0000,   /* U+2329 */
  0x0000,   /* U+232A */
  0x0000,   /* U+232B */
  0x0000,   /* U+232C */
  0x0000,   /* U+232D */
  0x0000,   /* U+232E */
  0x0000,   /* U+232F */
  0x0000,   /* U+2330 */
  0x0000,   /* U+2331 */
  0x0000,   /* U+2332 */
  0x0000,   /* U+2333 */
  0x0000,   /* U+2334 */
  0x0000,   /* U+2335 */
  0x0000,   /* U+2336 */
  0x0000,   /* U+2337 */
  0x0000,   /* U+2338 */
  0x0000,   /* U+2339 */
  0x0000,   /* U+233A */
  0x0000,   /* U+233B */
  0x0000,   /* U+233C */
  0x0000,   /* U+233D */
  0x0000,   /* U+233E */
  0x0000,   /* U+233F */
  0x0000,   /* U+2340 */
  0x0000,   /* U+2341 */
  0x0000,   /* U+2342 */
  0x0000,   /* U+2343 */
  0x0000,   /* U+2344 */
  0x0000,   /* U+2345 */
  0x0000,   /* U+2346 */
  0x0000,   /* U+2347 */
  0x0000,   /* U+2348 */
  0x0000,   /* U+2349 */
  0x0000,   /* U+234A */
  0x0000,   /* U+234B */
  0x0000,   /* U+234C */
  0x0000,   /* U+234D */
  0x0000,   /* U+234E */
  0x0000,   /* U+234F */
  0x0000,   /* U+2350 */
  0x0000,   /* U+2351 */
  0x0000,   /* U+2352 */
  0x0000,   /* U+2353 */
  0x0000,   /* U+2354 */
  0x0000,   /* U+2355 */
  0x0000,   /* U+2356 */
  0x0000,   /* U+2357 */
  0x0000,   /* U+2358 */
  0x0000,   /* U+2359 */
  0x0000,   /* U+235A */
  0x0000,   /* U+235B */
  0x0000,   /* U+235C */
  0x0000,   /* U+235D */
  0x0000,   /* U+235E */
  0x0000,   /* U+235F */
  0x0000,   /* U+2360 */
  0x0000,   /* U+2361 */
  0x0000,   /* U+2362 */
  0x0000,   /* U+2363 */
  0x0000,   /* U+2364 */
  0x0000,   /* U+2365 */
  0x0000,   /* U+2366 */
  0x0000,   /* U+2367 */
  0x0000,   /* U+2368 */
  0x0000,   /* U+2369 */
  0x0000,   /* U+236A */
  0x0000,   /* U+236B */
  0x0000,   /* U+236C */
  0x0000,   /* U+236D */
  0x0000,   /* U+236E */
  0x0000,   /* U+236F */
  0x0000,   /* U+2370 */
  0x0000,   /* U+2371 */
  0x0000,   /* U+2372 */
  0x0000,   /* U+2373 */
  0x0000,   /* U+2374 */
  0x0000,   /* U+2375 */
  0x0000,   /* U+2376 */
  0x0000,   /* U+2377 */
  0x0000,   /* U+2378 */
  0x0000,   /* U+2379 */
  0x0000,   /* U+237A */
  0x0000,   /* U+237B */
  0x0000,   /* U+237C */
  0x0000,   /* U+237D */
  0x0000,   /* U+237E */
  0x0000,   /* U+237F */
  0x0000,   /* U+2380 */
  0x0000,   /* U+2381 */
  0x0000,   /* U+2382 */
  0x0000,   /* U+2383 */
  0x0000,   /* U+2384 */
  0x0000,   /* U+2385 */
  0x0000,   /* U+2386 */
  0x0000,   /* U+2387 */
  0x0000,   /* U+2388 */
  0x0000,   /* U+2389 */
  0x0000,   /* U+238A */
  0x0000,   /* U+238B */
  0x0000,   /* U+238C */
  0x0000,   /* U+238D */
  0x0000,   /* U+238E */
  0x0000,   /* U+238F */
  0x0000,   /* U+2390 */
  0x0000,   /* U+2391 */
  0x0000,   /* U+2392 */
  0x0000,   /* U+2393 */
  0x0000,   /* U+2394 */
  0x0000,   /* U+2395 */
  0x0000,   /* U+2396 */
  0x0000,   /* U+2397 */
  0x0000,   /* U+2398 */
  0x0000,   /* U+2399 */
  0x0000,   /* U+239A */
  0x0000,   /* U+239B */
  0x0000,   /* U+239C */
  0x0000,   /* U+239D */
  0x0000,   /* U+239E */
  0x0000,   /* U+239F */
  0x0000,   /* U+23A0 */
  0x0000,   /* U+23A1 */
  0x0000,   /* U+23A2 */
  0x0000,   /* U+23A3 */
  0x0000,   /* U+23A4 */
  0x0000,   /* U+23A5 */
  0x0000,   /* U+23A6 */
  0x0000,   /* U+23A7 */
  0x0000,   /* U+23A8 */
  0x0000,   /* U+23A9 */
  0x0000,   /* U+23AA */
  0x0000,   /* U+23AB */
  0x0000,   /* U+23AC */
  0x0000,   /* U+23AD */
  0x0000,   /* U+23AE */
  0x0000,   /* U+23AF */
  0x0000,   /* U+23B0 */
  0x0000,   /* U+23B1 */
  0x0000,   /* U+23B2 */
  0x0000,   /* U+23B3 */
  0x0000,   /* U+23B4 */
  0x0000,   /* U+23B5 */
  0x0000,   /* U+23B6 */
  0x0000,   /* U+23B7 */
  0x0000,   /* U+23B8 */
  0x0000,   /* U+23B9 */
  0x0000,   /* U+23BA */
  0x0000,   /* U+23BB */
  0x0000,   /* U+23BC */
  0x0000,   /* U+23BD */
  0x0000,   /* U+23BE */
  0x0000,   /* U+23BF */
  0x0000,   /* U+23C0 */
  0x0000,   /* U+23C1 */
  0x0000,   /* U+23C2 */
  0x0000,   /* U+23C3 */
  0x0000,   /* U+23C4 */
  0x0000,   /* U+23C5 */
  0x0000,   /* U+23C6 */
  0x0000,   /* U+23C7 */
  0x0000,   /* U+23C8 */
  0x0000,   /* U+23C9 */
  0x0000,   /* U+23CA */
  0x0000,   /* U+23CB */
  0x0000,   /* U+23CC */
  0x0000,   /* U+23CD */
  0x0000,   /* U+23CE */
  0x0000,   /* U+23CF */
  0x0000,   /* U+23D0 */
  0x0000,   /* U+23D1 */
  0x0000,   /* U+23D2 */
  0x0000,   /* U+23D3 */
  0x0000,   /* U+23D4 */
  0x0000,   /* U+23D5 */
  0x0000,   /* U+23D6 */
  0x0000,   /* U+23D7 */
  0x0000,   /* U+23D8 */
  0x0000,   /* U+23D9 */
  0x0000,   /* U+23DA */
  0x0000,   /* U+23DB */
  0x0000,   /* U+23DC */
  0x0000,   /* U+23DD */
  0x0000,   /* U+23DE */
  0x0000,   /* U+23DF */
  0x0000,   /* U+23E0 */
  0x0000,   /* U+23E1 */
  0x0000,   /* U+23E2 */
  0x0000,   /* U+23E3 */
  0x0000,   /* U+23E4 */
  0x0000,   /* U+23E5 */
  0x0000,   /* U+23E6 */
  0x0000,   /* U+23E7 */
  0x0000,   /* U+23E8 */
  0x0000,   /* U+23E9 */
  0x0000,   /* U+23EA */
  0x0000,   /* U+23EB */
  0x0000,   /* U+23EC */
  0x0000,   /* U+23ED */
  0x0000,   /* U+23EE */
  0x0000,   /* U+23EF */
  0x0000,   /* U+23F0 */
  0x0000,   /* U+23F1 */
  0x0000,   /* U+23F2 */
  0x0000,   /* U+23F3 */
  0x0000,   /* U+23F4 */
  0x0000,   /* U+23F5 */
  0x0000,   /* U+23F6 */
  0x0000,   /* U+23F7 */
  0x0000,   /* U+23F8 */
  0x0000,   /* U+23F9 */
  0x0000,   /* U+23FA */
  0x0000,   /* U+23FB */
  0x0000,   /* U+23FC */
  0x0000,   /* U+23FD */
  0x0000,   /* U+23FE */
  0x0000,   /* U+23FF */
};

uint16 uca900_p024[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 

  /* Primary weight 1 for each character. */
  0x07E6,   /* U+2400 */
  0x07E7,   /* U+2401 */
  0x07E8,   /* U+2402 */
  0x07E9,   /* U+2403 */
  0x07EA,   /* U+2404 */
  0x07EB,   /* U+2405 */
  0x07EC,   /* U+2406 */
  0x07ED,   /* U+2407 */
  0x07EE,   /* U+2408 */
  0x07EF,   /* U+2409 */
  0x07F0,   /* U+240A */
  0x07F1,   /* U+240B */
  0x07F2,   /* U+240C */
  0x07F3,   /* U+240D */
  0x07F4,   /* U+240E */
  0x07F5,   /* U+240F */
  0x07F6,   /* U+2410 */
  0x07F7,   /* U+2411 */
  0x07F8,   /* U+2412 */
  0x07F9,   /* U+2413 */
  0x07FA,   /* U+2414 */
  0x07FB,   /* U+2415 */
  0x07FC,   /* U+2416 */
  0x07FD,   /* U+2417 */
  0x07FE,   /* U+2418 */
  0x07FF,   /* U+2419 */
  0x0800,   /* U+241A */
  0x0801,   /* U+241B */
  0x0802,   /* U+241C */
  0x0803,   /* U+241D */
  0x0804,   /* U+241E */
  0x0805,   /* U+241F */
  0x0806,   /* U+2420 */
  0x0807,   /* U+2421 */
  0x0808,   /* U+2422 */
  0x0809,   /* U+2423 */
  0x080A,   /* U+2424 */
  0x080B,   /* U+2425 */
  0x080C,   /* U+2426 */
  0xFBC0,   /* U+2427 */
  0xFBC0,   /* U+2428 */
  0xFBC0,   /* U+2429 */
  0xFBC0,   /* U+242A */
  0xFBC0,   /* U+242B */
  0xFBC0,   /* U+242C */
  0xFBC0,   /* U+242D */
  0xFBC0,   /* U+242E */
  0xFBC0,   /* U+242F */
  0xFBC0,   /* U+2430 */
  0xFBC0,   /* U+2431 */
  0xFBC0,   /* U+2432 */
  0xFBC0,   /* U+2433 */
  0xFBC0,   /* U+2434 */
  0xFBC0,   /* U+2435 */
  0xFBC0,   /* U+2436 */
  0xFBC0,   /* U+2437 */
  0xFBC0,   /* U+2438 */
  0xFBC0,   /* U+2439 */
  0xFBC0,   /* U+243A */
  0xFBC0,   /* U+243B */
  0xFBC0,   /* U+243C */
  0xFBC0,   /* U+243D */
  0xFBC0,   /* U+243E */
  0xFBC0,   /* U+243F */
  0x080D,   /* U+2440 */
  0x080E,   /* U+2441 */
  0x080F,   /* U+2442 */
  0x0810,   /* U+2443 */
  0x0811,   /* U+2444 */
  0x0812,   /* U+2445 */
  0x0813,   /* U+2446 */
  0x0814,   /* U+2447 */
  0x0815,   /* U+2448 */
  0x0816,   /* U+2449 */
  0x0817,   /* U+244A */
  0xFBC0,   /* U+244B */
  0xFBC0,   /* U+244C */
  0xFBC0,   /* U+244D */
  0xFBC0,   /* U+244E */
  0xFBC0,   /* U+244F */
  0xFBC0,   /* U+2450 */
  0xFBC0,   /* U+2451 */
  0xFBC0,   /* U+2452 */
  0xFBC0,   /* U+2453 */
  0xFBC0,   /* U+2454 */
  0xFBC0,   /* U+2455 */
  0xFBC0,   /* U+2456 */
  0xFBC0,   /* U+2457 */
  0xFBC0,   /* U+2458 */
  0xFBC0,   /* U+2459 */
  0xFBC0,   /* U+245A */
  0xFBC0,   /* U+245B */
  0xFBC0,   /* U+245C */
  0xFBC0,   /* U+245D */
  0xFBC0,   /* U+245E */
  0xFBC0,   /* U+245F */
  0x1C3E,   /* U+2460 */
  0x1C3F,   /* U+2461 */
  0x1C40,   /* U+2462 */
  0x1C41,   /* U+2463 */
  0x1C42,   /* U+2464 */
  0x1C43,   /* U+2465 */
  0x1C44,   /* U+2466 */
  0x1C45,   /* U+2467 */
  0x1C46,   /* U+2468 */
  0x1C3E,   /* U+2469 */
  0x1C3E,   /* U+246A */
  0x1C3E,   /* U+246B */
  0x1C3E,   /* U+246C */
  0x1C3E,   /* U+246D */
  0x1C3E,   /* U+246E */
  0x1C3E,   /* U+246F */
  0x1C3E,   /* U+2470 */
  0x1C3E,   /* U+2471 */
  0x1C3E,   /* U+2472 */
  0x1C3F,   /* U+2473 */
  0x0317,   /* U+2474 */
  0x0317,   /* U+2475 */
  0x0317,   /* U+2476 */
  0x0317,   /* U+2477 */
  0x0317,   /* U+2478 */
  0x0317,   /* U+2479 */
  0x0317,   /* U+247A */
  0x0317,   /* U+247B */
  0x0317,   /* U+247C */
  0x0317,   /* U+247D */
  0x0317,   /* U+247E */
  0x0317,   /* U+247F */
  0x0317,   /* U+2480 */
  0x0317,   /* U+2481 */
  0x0317,   /* U+2482 */
  0x0317,   /* U+2483 */
  0x0317,   /* U+2484 */
  0x0317,   /* U+2485 */
  0x0317,   /* U+2486 */
  0x0317,   /* U+2487 */
  0x1C3E,   /* U+2488 */
  0x1C3F,   /* U+2489 */
  0x1C40,   /* U+248A */
  0x1C41,   /* U+248B */
  0x1C42,   /* U+248C */
  0x1C43,   /* U+248D */
  0x1C44,   /* U+248E */
  0x1C45,   /* U+248F */
  0x1C46,   /* U+2490 */
  0x1C3E,   /* U+2491 */
  0x1C3E,   /* U+2492 */
  0x1C3E,   /* U+2493 */
  0x1C3E,   /* U+2494 */
  0x1C3E,   /* U+2495 */
  0x1C3E,   /* U+2496 */
  0x1C3E,   /* U+2497 */
  0x1C3E,   /* U+2498 */
  0x1C3E,   /* U+2499 */
  0x1C3E,   /* U+249A */
  0x1C3F,   /* U+249B */
  0x0317,   /* U+249C */
  0x0317,   /* U+249D */
  0x0317,   /* U+249E */
  0x0317,   /* U+249F */
  0x0317,   /* U+24A0 */
  0x0317,   /* U+24A1 */
  0x0317,   /* U+24A2 */
  0x0317,   /* U+24A3 */
  0x0317,   /* U+24A4 */
  0x0317,   /* U+24A5 */
  0x0317,   /* U+24A6 */
  0x0317,   /* U+24A7 */
  0x0317,   /* U+24A8 */
  0x0317,   /* U+24A9 */
  0x0317,   /* U+24AA */
  0x0317,   /* U+24AB */
  0x0317,   /* U+24AC */
  0x0317,   /* U+24AD */
  0x0317,   /* U+24AE */
  0x0317,   /* U+24AF */
  0x0317,   /* U+24B0 */
  0x0317,   /* U+24B1 */
  0x0317,   /* U+24B2 */
  0x0317,   /* U+24B3 */
  0x0317,   /* U+24B4 */
  0x0317,   /* U+24B5 */
  0x1C47,   /* U+24B6 */
  0x1C60,   /* U+24B7 */
  0x1C7A,   /* U+24B8 */
  0x1C8F,   /* U+24B9 */
  0x1CAA,   /* U+24BA */
  0x1CE5,   /* U+24BB */
  0x1CF4,   /* U+24BC */
  0x1D18,   /* U+24BD */
  0x1D32,   /* U+24BE */
  0x1D4C,   /* U+24BF */
  0x1D65,   /* U+24C0 */
  0x1D77,   /* U+24C1 */
  0x1DAA,   /* U+24C2 */
  0x1DB9,   /* U+24C3 */
  0x1DDD,   /* U+24C4 */
  0x1E0C,   /* U+24C5 */
  0x1E21,   /* U+24C6 */
  0x1E33,   /* U+24C7 */
  0x1E71,   /* U+24C8 */
  0x1E95,   /* U+24C9 */
  0x1EB5,   /* U+24CA */
  0x1EE3,   /* U+24CB */
  0x1EF5,   /* U+24CC */
  0x1EFF,   /* U+24CD */
  0x1F0B,   /* U+24CE */
  0x1F21,   /* U+24CF */
  0x1C47,   /* U+24D0 */
  0x1C60,   /* U+24D1 */
  0x1C7A,   /* U+24D2 */
  0x1C8F,   /* U+24D3 */
  0x1CAA,   /* U+24D4 */
  0x1CE5,   /* U+24D5 */
  0x1CF4,   /* U+24D6 */
  0x1D18,   /* U+24D7 */
  0x1D32,   /* U+24D8 */
  0x1D4C,   /* U+24D9 */
  0x1D65,   /* U+24DA */
  0x1D77,   /* U+24DB */
  0x1DAA,   /* U+24DC */
  0x1DB9,   /* U+24DD */
  0x1DDD,   /* U+24DE */
  0x1E0C,   /* U+24DF */
  0x1E21,   /* U+24E0 */
  0x1E33,   /* U+24E1 */
  0x1E71,   /* U+24E2 */
  0x1E95,   /* U+24E3 */
  0x1EB5,   /* U+24E4 */
  0x1EE3,   /* U+24E5 */
  0x1EF5,   /* U+24E6 */
  0x1EFF,   /* U+24E7 */
  0x1F0B,   /* U+24E8 */
  0x1F21,   /* U+24E9 */
  0x1C3D,   /* U+24EA */
  0x1C3E,   /* U+24EB */
  0x1C3E,   /* U+24EC */
  0x1C3E,   /* U+24ED */
  0x1C3E,   /* U+24EE */
  0x1C3E,   /* U+24EF */
  0x1C3E,   /* U+24F0 */
  0x1C3E,   /* U+24F1 */
  0x1C3E,   /* U+24F2 */
  0x1C3E,   /* U+24F3 */
  0x1C3F,   /* U+24F4 */
  0x1C3E,   /* U+24F5 */
  0x1C3F,   /* U+24F6 */
  0x1C40,   /* U+24F7 */
  0x1C41,   /* U+24F8 */
  0x1C42,   /* U+24F9 */
  0x1C43,   /* U+24FA */
  0x1C44,   /* U+24FB */
  0x1C45,   /* U+24FC */
  0x1C46,   /* U+24FD */
  0x1C3E,   /* U+24FE */
  0x1C3D,   /* U+24FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2400 */
  0x0020,   /* U+2401 */
  0x0020,   /* U+2402 */
  0x0020,   /* U+2403 */
  0x0020,   /* U+2404 */
  0x0020,   /* U+2405 */
  0x0020,   /* U+2406 */
  0x0020,   /* U+2407 */
  0x0020,   /* U+2408 */
  0x0020,   /* U+2409 */
  0x0020,   /* U+240A */
  0x0020,   /* U+240B */
  0x0020,   /* U+240C */
  0x0020,   /* U+240D */
  0x0020,   /* U+240E */
  0x0020,   /* U+240F */
  0x0020,   /* U+2410 */
  0x0020,   /* U+2411 */
  0x0020,   /* U+2412 */
  0x0020,   /* U+2413 */
  0x0020,   /* U+2414 */
  0x0020,   /* U+2415 */
  0x0020,   /* U+2416 */
  0x0020,   /* U+2417 */
  0x0020,   /* U+2418 */
  0x0020,   /* U+2419 */
  0x0020,   /* U+241A */
  0x0020,   /* U+241B */
  0x0020,   /* U+241C */
  0x0020,   /* U+241D */
  0x0020,   /* U+241E */
  0x0020,   /* U+241F */
  0x0020,   /* U+2420 */
  0x0020,   /* U+2421 */
  0x0020,   /* U+2422 */
  0x0020,   /* U+2423 */
  0x0020,   /* U+2424 */
  0x0020,   /* U+2425 */
  0x0020,   /* U+2426 */
  0x0020,   /* U+2427 */
  0x0020,   /* U+2428 */
  0x0020,   /* U+2429 */
  0x0020,   /* U+242A */
  0x0020,   /* U+242B */
  0x0020,   /* U+242C */
  0x0020,   /* U+242D */
  0x0020,   /* U+242E */
  0x0020,   /* U+242F */
  0x0020,   /* U+2430 */
  0x0020,   /* U+2431 */
  0x0020,   /* U+2432 */
  0x0020,   /* U+2433 */
  0x0020,   /* U+2434 */
  0x0020,   /* U+2435 */
  0x0020,   /* U+2436 */
  0x0020,   /* U+2437 */
  0x0020,   /* U+2438 */
  0x0020,   /* U+2439 */
  0x0020,   /* U+243A */
  0x0020,   /* U+243B */
  0x0020,   /* U+243C */
  0x0020,   /* U+243D */
  0x0020,   /* U+243E */
  0x0020,   /* U+243F */
  0x0020,   /* U+2440 */
  0x0020,   /* U+2441 */
  0x0020,   /* U+2442 */
  0x0020,   /* U+2443 */
  0x0020,   /* U+2444 */
  0x0020,   /* U+2445 */
  0x0020,   /* U+2446 */
  0x0020,   /* U+2447 */
  0x0020,   /* U+2448 */
  0x0020,   /* U+2449 */
  0x0020,   /* U+244A */
  0x0020,   /* U+244B */
  0x0020,   /* U+244C */
  0x0020,   /* U+244D */
  0x0020,   /* U+244E */
  0x0020,   /* U+244F */
  0x0020,   /* U+2450 */
  0x0020,   /* U+2451 */
  0x0020,   /* U+2452 */
  0x0020,   /* U+2453 */
  0x0020,   /* U+2454 */
  0x0020,   /* U+2455 */
  0x0020,   /* U+2456 */
  0x0020,   /* U+2457 */
  0x0020,   /* U+2458 */
  0x0020,   /* U+2459 */
  0x0020,   /* U+245A */
  0x0020,   /* U+245B */
  0x0020,   /* U+245C */
  0x0020,   /* U+245D */
  0x0020,   /* U+245E */
  0x0020,   /* U+245F */
  0x0020,   /* U+2460 */
  0x0020,   /* U+2461 */
  0x0020,   /* U+2462 */
  0x0020,   /* U+2463 */
  0x0020,   /* U+2464 */
  0x0020,   /* U+2465 */
  0x0020,   /* U+2466 */
  0x0020,   /* U+2467 */
  0x0020,   /* U+2468 */
  0x0020,   /* U+2469 */
  0x0020,   /* U+246A */
  0x0020,   /* U+246B */
  0x0020,   /* U+246C */
  0x0020,   /* U+246D */
  0x0020,   /* U+246E */
  0x0020,   /* U+246F */
  0x0020,   /* U+2470 */
  0x0020,   /* U+2471 */
  0x0020,   /* U+2472 */
  0x0020,   /* U+2473 */
  0x0020,   /* U+2474 */
  0x0020,   /* U+2475 */
  0x0020,   /* U+2476 */
  0x0020,   /* U+2477 */
  0x0020,   /* U+2478 */
  0x0020,   /* U+2479 */
  0x0020,   /* U+247A */
  0x0020,   /* U+247B */
  0x0020,   /* U+247C */
  0x0020,   /* U+247D */
  0x0020,   /* U+247E */
  0x0020,   /* U+247F */
  0x0020,   /* U+2480 */
  0x0020,   /* U+2481 */
  0x0020,   /* U+2482 */
  0x0020,   /* U+2483 */
  0x0020,   /* U+2484 */
  0x0020,   /* U+2485 */
  0x0020,   /* U+2486 */
  0x0020,   /* U+2487 */
  0x0020,   /* U+2488 */
  0x0020,   /* U+2489 */
  0x0020,   /* U+248A */
  0x0020,   /* U+248B */
  0x0020,   /* U+248C */
  0x0020,   /* U+248D */
  0x0020,   /* U+248E */
  0x0020,   /* U+248F */
  0x0020,   /* U+2490 */
  0x0020,   /* U+2491 */
  0x0020,   /* U+2492 */
  0x0020,   /* U+2493 */
  0x0020,   /* U+2494 */
  0x0020,   /* U+2495 */
  0x0020,   /* U+2496 */
  0x0020,   /* U+2497 */
  0x0020,   /* U+2498 */
  0x0020,   /* U+2499 */
  0x0020,   /* U+249A */
  0x0020,   /* U+249B */
  0x0020,   /* U+249C */
  0x0020,   /* U+249D */
  0x0020,   /* U+249E */
  0x0020,   /* U+249F */
  0x0020,   /* U+24A0 */
  0x0020,   /* U+24A1 */
  0x0020,   /* U+24A2 */
  0x0020,   /* U+24A3 */
  0x0020,   /* U+24A4 */
  0x0020,   /* U+24A5 */
  0x0020,   /* U+24A6 */
  0x0020,   /* U+24A7 */
  0x0020,   /* U+24A8 */
  0x0020,   /* U+24A9 */
  0x0020,   /* U+24AA */
  0x0020,   /* U+24AB */
  0x0020,   /* U+24AC */
  0x0020,   /* U+24AD */
  0x0020,   /* U+24AE */
  0x0020,   /* U+24AF */
  0x0020,   /* U+24B0 */
  0x0020,   /* U+24B1 */
  0x0020,   /* U+24B2 */
  0x0020,   /* U+24B3 */
  0x0020,   /* U+24B4 */
  0x0020,   /* U+24B5 */
  0x0020,   /* U+24B6 */
  0x0020,   /* U+24B7 */
  0x0020,   /* U+24B8 */
  0x0020,   /* U+24B9 */
  0x0020,   /* U+24BA */
  0x0020,   /* U+24BB */
  0x0020,   /* U+24BC */
  0x0020,   /* U+24BD */
  0x0020,   /* U+24BE */
  0x0020,   /* U+24BF */
  0x0020,   /* U+24C0 */
  0x0020,   /* U+24C1 */
  0x0020,   /* U+24C2 */
  0x0020,   /* U+24C3 */
  0x0020,   /* U+24C4 */
  0x0020,   /* U+24C5 */
  0x0020,   /* U+24C6 */
  0x0020,   /* U+24C7 */
  0x0020,   /* U+24C8 */
  0x0020,   /* U+24C9 */
  0x0020,   /* U+24CA */
  0x0020,   /* U+24CB */
  0x0020,   /* U+24CC */
  0x0020,   /* U+24CD */
  0x0020,   /* U+24CE */
  0x0020,   /* U+24CF */
  0x0020,   /* U+24D0 */
  0x0020,   /* U+24D1 */
  0x0020,   /* U+24D2 */
  0x0020,   /* U+24D3 */
  0x0020,   /* U+24D4 */
  0x0020,   /* U+24D5 */
  0x0020,   /* U+24D6 */
  0x0020,   /* U+24D7 */
  0x0020,   /* U+24D8 */
  0x0020,   /* U+24D9 */
  0x0020,   /* U+24DA */
  0x0020,   /* U+24DB */
  0x0020,   /* U+24DC */
  0x0020,   /* U+24DD */
  0x0020,   /* U+24DE */
  0x0020,   /* U+24DF */
  0x0020,   /* U+24E0 */
  0x0020,   /* U+24E1 */
  0x0020,   /* U+24E2 */
  0x0020,   /* U+24E3 */
  0x0020,   /* U+24E4 */
  0x0020,   /* U+24E5 */
  0x0020,   /* U+24E6 */
  0x0020,   /* U+24E7 */
  0x0020,   /* U+24E8 */
  0x0020,   /* U+24E9 */
  0x0020,   /* U+24EA */
  0x0020,   /* U+24EB */
  0x0020,   /* U+24EC */
  0x0020,   /* U+24ED */
  0x0020,   /* U+24EE */
  0x0020,   /* U+24EF */
  0x0020,   /* U+24F0 */
  0x0020,   /* U+24F1 */
  0x0020,   /* U+24F2 */
  0x0020,   /* U+24F3 */
  0x0020,   /* U+24F4 */
  0x0020,   /* U+24F5 */
  0x0020,   /* U+24F6 */
  0x0020,   /* U+24F7 */
  0x0020,   /* U+24F8 */
  0x0020,   /* U+24F9 */
  0x0020,   /* U+24FA */
  0x0020,   /* U+24FB */
  0x0020,   /* U+24FC */
  0x0020,   /* U+24FD */
  0x0020,   /* U+24FE */
  0x0020,   /* U+24FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2400 */
  0x0002,   /* U+2401 */
  0x0002,   /* U+2402 */
  0x0002,   /* U+2403 */
  0x0002,   /* U+2404 */
  0x0002,   /* U+2405 */
  0x0002,   /* U+2406 */
  0x0002,   /* U+2407 */
  0x0002,   /* U+2408 */
  0x0002,   /* U+2409 */
  0x0002,   /* U+240A */
  0x0002,   /* U+240B */
  0x0002,   /* U+240C */
  0x0002,   /* U+240D */
  0x0002,   /* U+240E */
  0x0002,   /* U+240F */
  0x0002,   /* U+2410 */
  0x0002,   /* U+2411 */
  0x0002,   /* U+2412 */
  0x0002,   /* U+2413 */
  0x0002,   /* U+2414 */
  0x0002,   /* U+2415 */
  0x0002,   /* U+2416 */
  0x0002,   /* U+2417 */
  0x0002,   /* U+2418 */
  0x0002,   /* U+2419 */
  0x0002,   /* U+241A */
  0x0002,   /* U+241B */
  0x0002,   /* U+241C */
  0x0002,   /* U+241D */
  0x0002,   /* U+241E */
  0x0002,   /* U+241F */
  0x0002,   /* U+2420 */
  0x0002,   /* U+2421 */
  0x0002,   /* U+2422 */
  0x0002,   /* U+2423 */
  0x0002,   /* U+2424 */
  0x0002,   /* U+2425 */
  0x0002,   /* U+2426 */
  0x0002,   /* U+2427 */
  0x0002,   /* U+2428 */
  0x0002,   /* U+2429 */
  0x0002,   /* U+242A */
  0x0002,   /* U+242B */
  0x0002,   /* U+242C */
  0x0002,   /* U+242D */
  0x0002,   /* U+242E */
  0x0002,   /* U+242F */
  0x0002,   /* U+2430 */
  0x0002,   /* U+2431 */
  0x0002,   /* U+2432 */
  0x0002,   /* U+2433 */
  0x0002,   /* U+2434 */
  0x0002,   /* U+2435 */
  0x0002,   /* U+2436 */
  0x0002,   /* U+2437 */
  0x0002,   /* U+2438 */
  0x0002,   /* U+2439 */
  0x0002,   /* U+243A */
  0x0002,   /* U+243B */
  0x0002,   /* U+243C */
  0x0002,   /* U+243D */
  0x0002,   /* U+243E */
  0x0002,   /* U+243F */
  0x0002,   /* U+2440 */
  0x0002,   /* U+2441 */
  0x0002,   /* U+2442 */
  0x0002,   /* U+2443 */
  0x0002,   /* U+2444 */
  0x0002,   /* U+2445 */
  0x0002,   /* U+2446 */
  0x0002,   /* U+2447 */
  0x0002,   /* U+2448 */
  0x0002,   /* U+2449 */
  0x0002,   /* U+244A */
  0x0002,   /* U+244B */
  0x0002,   /* U+244C */
  0x0002,   /* U+244D */
  0x0002,   /* U+244E */
  0x0002,   /* U+244F */
  0x0002,   /* U+2450 */
  0x0002,   /* U+2451 */
  0x0002,   /* U+2452 */
  0x0002,   /* U+2453 */
  0x0002,   /* U+2454 */
  0x0002,   /* U+2455 */
  0x0002,   /* U+2456 */
  0x0002,   /* U+2457 */
  0x0002,   /* U+2458 */
  0x0002,   /* U+2459 */
  0x0002,   /* U+245A */
  0x0002,   /* U+245B */
  0x0002,   /* U+245C */
  0x0002,   /* U+245D */
  0x0002,   /* U+245E */
  0x0002,   /* U+245F */
  0x0006,   /* U+2460 */
  0x0006,   /* U+2461 */
  0x0006,   /* U+2462 */
  0x0006,   /* U+2463 */
  0x0006,   /* U+2464 */
  0x0006,   /* U+2465 */
  0x0006,   /* U+2466 */
  0x0006,   /* U+2467 */
  0x0006,   /* U+2468 */
  0x0006,   /* U+2469 */
  0x0006,   /* U+246A */
  0x0006,   /* U+246B */
  0x0006,   /* U+246C */
  0x0006,   /* U+246D */
  0x0006,   /* U+246E */
  0x0006,   /* U+246F */
  0x0006,   /* U+2470 */
  0x0006,   /* U+2471 */
  0x0006,   /* U+2472 */
  0x0006,   /* U+2473 */
  0x0004,   /* U+2474 */
  0x0004,   /* U+2475 */
  0x0004,   /* U+2476 */
  0x0004,   /* U+2477 */
  0x0004,   /* U+2478 */
  0x0004,   /* U+2479 */
  0x0004,   /* U+247A */
  0x0004,   /* U+247B */
  0x0004,   /* U+247C */
  0x0004,   /* U+247D */
  0x0004,   /* U+247E */
  0x0004,   /* U+247F */
  0x0004,   /* U+2480 */
  0x0004,   /* U+2481 */
  0x0004,   /* U+2482 */
  0x0004,   /* U+2483 */
  0x0004,   /* U+2484 */
  0x0004,   /* U+2485 */
  0x0004,   /* U+2486 */
  0x0004,   /* U+2487 */
  0x0004,   /* U+2488 */
  0x0004,   /* U+2489 */
  0x0004,   /* U+248A */
  0x0004,   /* U+248B */
  0x0004,   /* U+248C */
  0x0004,   /* U+248D */
  0x0004,   /* U+248E */
  0x0004,   /* U+248F */
  0x0004,   /* U+2490 */
  0x0004,   /* U+2491 */
  0x0004,   /* U+2492 */
  0x0004,   /* U+2493 */
  0x0004,   /* U+2494 */
  0x0004,   /* U+2495 */
  0x0004,   /* U+2496 */
  0x0004,   /* U+2497 */
  0x0004,   /* U+2498 */
  0x0004,   /* U+2499 */
  0x0004,   /* U+249A */
  0x0004,   /* U+249B */
  0x0004,   /* U+249C */
  0x0004,   /* U+249D */
  0x0004,   /* U+249E */
  0x0004,   /* U+249F */
  0x0004,   /* U+24A0 */
  0x0004,   /* U+24A1 */
  0x0004,   /* U+24A2 */
  0x0004,   /* U+24A3 */
  0x0004,   /* U+24A4 */
  0x0004,   /* U+24A5 */
  0x0004,   /* U+24A6 */
  0x0004,   /* U+24A7 */
  0x0004,   /* U+24A8 */
  0x0004,   /* U+24A9 */
  0x0004,   /* U+24AA */
  0x0004,   /* U+24AB */
  0x0004,   /* U+24AC */
  0x0004,   /* U+24AD */
  0x0004,   /* U+24AE */
  0x0004,   /* U+24AF */
  0x0004,   /* U+24B0 */
  0x0004,   /* U+24B1 */
  0x0004,   /* U+24B2 */
  0x0004,   /* U+24B3 */
  0x0004,   /* U+24B4 */
  0x0004,   /* U+24B5 */
  0x000C,   /* U+24B6 */
  0x000C,   /* U+24B7 */
  0x000C,   /* U+24B8 */
  0x000C,   /* U+24B9 */
  0x000C,   /* U+24BA */
  0x000C,   /* U+24BB */
  0x000C,   /* U+24BC */
  0x000C,   /* U+24BD */
  0x000C,   /* U+24BE */
  0x000C,   /* U+24BF */
  0x000C,   /* U+24C0 */
  0x000C,   /* U+24C1 */
  0x000C,   /* U+24C2 */
  0x000C,   /* U+24C3 */
  0x000C,   /* U+24C4 */
  0x000C,   /* U+24C5 */
  0x000C,   /* U+24C6 */
  0x000C,   /* U+24C7 */
  0x000C,   /* U+24C8 */
  0x000C,   /* U+24C9 */
  0x000C,   /* U+24CA */
  0x000C,   /* U+24CB */
  0x000C,   /* U+24CC */
  0x000C,   /* U+24CD */
  0x000C,   /* U+24CE */
  0x000C,   /* U+24CF */
  0x0006,   /* U+24D0 */
  0x0006,   /* U+24D1 */
  0x0006,   /* U+24D2 */
  0x0006,   /* U+24D3 */
  0x0006,   /* U+24D4 */
  0x0006,   /* U+24D5 */
  0x0006,   /* U+24D6 */
  0x0006,   /* U+24D7 */
  0x0006,   /* U+24D8 */
  0x0006,   /* U+24D9 */
  0x0006,   /* U+24DA */
  0x0006,   /* U+24DB */
  0x0006,   /* U+24DC */
  0x0006,   /* U+24DD */
  0x0006,   /* U+24DE */
  0x0006,   /* U+24DF */
  0x0006,   /* U+24E0 */
  0x0006,   /* U+24E1 */
  0x0006,   /* U+24E2 */
  0x0006,   /* U+24E3 */
  0x0006,   /* U+24E4 */
  0x0006,   /* U+24E5 */
  0x0006,   /* U+24E6 */
  0x0006,   /* U+24E7 */
  0x0006,   /* U+24E8 */
  0x0006,   /* U+24E9 */
  0x0006,   /* U+24EA */
  0x0006,   /* U+24EB */
  0x0006,   /* U+24EC */
  0x0006,   /* U+24ED */
  0x0006,   /* U+24EE */
  0x0006,   /* U+24EF */
  0x0006,   /* U+24F0 */
  0x0006,   /* U+24F1 */
  0x0006,   /* U+24F2 */
  0x0006,   /* U+24F3 */
  0x0006,   /* U+24F4 */
  0x0006,   /* U+24F5 */
  0x0006,   /* U+24F6 */
  0x0006,   /* U+24F7 */
  0x0006,   /* U+24F8 */
  0x0006,   /* U+24F9 */
  0x0006,   /* U+24FA */
  0x0006,   /* U+24FB */
  0x0006,   /* U+24FC */
  0x0006,   /* U+24FD */
  0x0006,   /* U+24FE */
  0x0006,   /* U+24FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0xA427,   /* U+2427 */
  0xA428,   /* U+2428 */
  0xA429,   /* U+2429 */
  0xA42A,   /* U+242A */
  0xA42B,   /* U+242B */
  0xA42C,   /* U+242C */
  0xA42D,   /* U+242D */
  0xA42E,   /* U+242E */
  0xA42F,   /* U+242F */
  0xA430,   /* U+2430 */
  0xA431,   /* U+2431 */
  0xA432,   /* U+2432 */
  0xA433,   /* U+2433 */
  0xA434,   /* U+2434 */
  0xA435,   /* U+2435 */
  0xA436,   /* U+2436 */
  0xA437,   /* U+2437 */
  0xA438,   /* U+2438 */
  0xA439,   /* U+2439 */
  0xA43A,   /* U+243A */
  0xA43B,   /* U+243B */
  0xA43C,   /* U+243C */
  0xA43D,   /* U+243D */
  0xA43E,   /* U+243E */
  0xA43F,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0xA44B,   /* U+244B */
  0xA44C,   /* U+244C */
  0xA44D,   /* U+244D */
  0xA44E,   /* U+244E */
  0xA44F,   /* U+244F */
  0xA450,   /* U+2450 */
  0xA451,   /* U+2451 */
  0xA452,   /* U+2452 */
  0xA453,   /* U+2453 */
  0xA454,   /* U+2454 */
  0xA455,   /* U+2455 */
  0xA456,   /* U+2456 */
  0xA457,   /* U+2457 */
  0xA458,   /* U+2458 */
  0xA459,   /* U+2459 */
  0xA45A,   /* U+245A */
  0xA45B,   /* U+245B */
  0xA45C,   /* U+245C */
  0xA45D,   /* U+245D */
  0xA45E,   /* U+245E */
  0xA45F,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x1C3D,   /* U+2469 */
  0x1C3E,   /* U+246A */
  0x1C3F,   /* U+246B */
  0x1C40,   /* U+246C */
  0x1C41,   /* U+246D */
  0x1C42,   /* U+246E */
  0x1C43,   /* U+246F */
  0x1C44,   /* U+2470 */
  0x1C45,   /* U+2471 */
  0x1C46,   /* U+2472 */
  0x1C3D,   /* U+2473 */
  0x1C3E,   /* U+2474 */
  0x1C3F,   /* U+2475 */
  0x1C40,   /* U+2476 */
  0x1C41,   /* U+2477 */
  0x1C42,   /* U+2478 */
  0x1C43,   /* U+2479 */
  0x1C44,   /* U+247A */
  0x1C45,   /* U+247B */
  0x1C46,   /* U+247C */
  0x1C3E,   /* U+247D */
  0x1C3E,   /* U+247E */
  0x1C3E,   /* U+247F */
  0x1C3E,   /* U+2480 */
  0x1C3E,   /* U+2481 */
  0x1C3E,   /* U+2482 */
  0x1C3E,   /* U+2483 */
  0x1C3E,   /* U+2484 */
  0x1C3E,   /* U+2485 */
  0x1C3E,   /* U+2486 */
  0x1C3F,   /* U+2487 */
  0x0277,   /* U+2488 */
  0x0277,   /* U+2489 */
  0x0277,   /* U+248A */
  0x0277,   /* U+248B */
  0x0277,   /* U+248C */
  0x0277,   /* U+248D */
  0x0277,   /* U+248E */
  0x0277,   /* U+248F */
  0x0277,   /* U+2490 */
  0x1C3D,   /* U+2491 */
  0x1C3E,   /* U+2492 */
  0x1C3F,   /* U+2493 */
  0x1C40,   /* U+2494 */
  0x1C41,   /* U+2495 */
  0x1C42,   /* U+2496 */
  0x1C43,   /* U+2497 */
  0x1C44,   /* U+2498 */
  0x1C45,   /* U+2499 */
  0x1C46,   /* U+249A */
  0x1C3D,   /* U+249B */
  0x1C47,   /* U+249C */
  0x1C60,   /* U+249D */
  0x1C7A,   /* U+249E */
  0x1C8F,   /* U+249F */
  0x1CAA,   /* U+24A0 */
  0x1CE5,   /* U+24A1 */
  0x1CF4,   /* U+24A2 */
  0x1D18,   /* U+24A3 */
  0x1D32,   /* U+24A4 */
  0x1D4C,   /* U+24A5 */
  0x1D65,   /* U+24A6 */
  0x1D77,   /* U+24A7 */
  0x1DAA,   /* U+24A8 */
  0x1DB9,   /* U+24A9 */
  0x1DDD,   /* U+24AA */
  0x1E0C,   /* U+24AB */
  0x1E21,   /* U+24AC */
  0x1E33,   /* U+24AD */
  0x1E71,   /* U+24AE */
  0x1E95,   /* U+24AF */
  0x1EB5,   /* U+24B0 */
  0x1EE3,   /* U+24B1 */
  0x1EF5,   /* U+24B2 */
  0x1EFF,   /* U+24B3 */
  0x1F0B,   /* U+24B4 */
  0x1F21,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x1C3E,   /* U+24EB */
  0x1C3F,   /* U+24EC */
  0x1C40,   /* U+24ED */
  0x1C41,   /* U+24EE */
  0x1C42,   /* U+24EF */
  0x1C43,   /* U+24F0 */
  0x1C44,   /* U+24F1 */
  0x1C45,   /* U+24F2 */
  0x1C46,   /* U+24F3 */
  0x1C3D,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x1C3D,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0020,   /* U+2469 */
  0x0020,   /* U+246A */
  0x0020,   /* U+246B */
  0x0020,   /* U+246C */
  0x0020,   /* U+246D */
  0x0020,   /* U+246E */
  0x0020,   /* U+246F */
  0x0020,   /* U+2470 */
  0x0020,   /* U+2471 */
  0x0020,   /* U+2472 */
  0x0020,   /* U+2473 */
  0x0020,   /* U+2474 */
  0x0020,   /* U+2475 */
  0x0020,   /* U+2476 */
  0x0020,   /* U+2477 */
  0x0020,   /* U+2478 */
  0x0020,   /* U+2479 */
  0x0020,   /* U+247A */
  0x0020,   /* U+247B */
  0x0020,   /* U+247C */
  0x0020,   /* U+247D */
  0x0020,   /* U+247E */
  0x0020,   /* U+247F */
  0x0020,   /* U+2480 */
  0x0020,   /* U+2481 */
  0x0020,   /* U+2482 */
  0x0020,   /* U+2483 */
  0x0020,   /* U+2484 */
  0x0020,   /* U+2485 */
  0x0020,   /* U+2486 */
  0x0020,   /* U+2487 */
  0x0020,   /* U+2488 */
  0x0020,   /* U+2489 */
  0x0020,   /* U+248A */
  0x0020,   /* U+248B */
  0x0020,   /* U+248C */
  0x0020,   /* U+248D */
  0x0020,   /* U+248E */
  0x0020,   /* U+248F */
  0x0020,   /* U+2490 */
  0x0020,   /* U+2491 */
  0x0020,   /* U+2492 */
  0x0020,   /* U+2493 */
  0x0020,   /* U+2494 */
  0x0020,   /* U+2495 */
  0x0020,   /* U+2496 */
  0x0020,   /* U+2497 */
  0x0020,   /* U+2498 */
  0x0020,   /* U+2499 */
  0x0020,   /* U+249A */
  0x0020,   /* U+249B */
  0x0020,   /* U+249C */
  0x0020,   /* U+249D */
  0x0020,   /* U+249E */
  0x0020,   /* U+249F */
  0x0020,   /* U+24A0 */
  0x0020,   /* U+24A1 */
  0x0020,   /* U+24A2 */
  0x0020,   /* U+24A3 */
  0x0020,   /* U+24A4 */
  0x0020,   /* U+24A5 */
  0x0020,   /* U+24A6 */
  0x0020,   /* U+24A7 */
  0x0020,   /* U+24A8 */
  0x0020,   /* U+24A9 */
  0x0020,   /* U+24AA */
  0x0020,   /* U+24AB */
  0x0020,   /* U+24AC */
  0x0020,   /* U+24AD */
  0x0020,   /* U+24AE */
  0x0020,   /* U+24AF */
  0x0020,   /* U+24B0 */
  0x0020,   /* U+24B1 */
  0x0020,   /* U+24B2 */
  0x0020,   /* U+24B3 */
  0x0020,   /* U+24B4 */
  0x0020,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0020,   /* U+24EB */
  0x0020,   /* U+24EC */
  0x0020,   /* U+24ED */
  0x0020,   /* U+24EE */
  0x0020,   /* U+24EF */
  0x0020,   /* U+24F0 */
  0x0020,   /* U+24F1 */
  0x0020,   /* U+24F2 */
  0x0020,   /* U+24F3 */
  0x0020,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0020,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0006,   /* U+2469 */
  0x0006,   /* U+246A */
  0x0006,   /* U+246B */
  0x0006,   /* U+246C */
  0x0006,   /* U+246D */
  0x0006,   /* U+246E */
  0x0006,   /* U+246F */
  0x0006,   /* U+2470 */
  0x0006,   /* U+2471 */
  0x0006,   /* U+2472 */
  0x0006,   /* U+2473 */
  0x0004,   /* U+2474 */
  0x0004,   /* U+2475 */
  0x0004,   /* U+2476 */
  0x0004,   /* U+2477 */
  0x0004,   /* U+2478 */
  0x0004,   /* U+2479 */
  0x0004,   /* U+247A */
  0x0004,   /* U+247B */
  0x0004,   /* U+247C */
  0x0004,   /* U+247D */
  0x0004,   /* U+247E */
  0x0004,   /* U+247F */
  0x0004,   /* U+2480 */
  0x0004,   /* U+2481 */
  0x0004,   /* U+2482 */
  0x0004,   /* U+2483 */
  0x0004,   /* U+2484 */
  0x0004,   /* U+2485 */
  0x0004,   /* U+2486 */
  0x0004,   /* U+2487 */
  0x0004,   /* U+2488 */
  0x0004,   /* U+2489 */
  0x0004,   /* U+248A */
  0x0004,   /* U+248B */
  0x0004,   /* U+248C */
  0x0004,   /* U+248D */
  0x0004,   /* U+248E */
  0x0004,   /* U+248F */
  0x0004,   /* U+2490 */
  0x0004,   /* U+2491 */
  0x0004,   /* U+2492 */
  0x0004,   /* U+2493 */
  0x0004,   /* U+2494 */
  0x0004,   /* U+2495 */
  0x0004,   /* U+2496 */
  0x0004,   /* U+2497 */
  0x0004,   /* U+2498 */
  0x0004,   /* U+2499 */
  0x0004,   /* U+249A */
  0x0004,   /* U+249B */
  0x0004,   /* U+249C */
  0x0004,   /* U+249D */
  0x0004,   /* U+249E */
  0x0004,   /* U+249F */
  0x0004,   /* U+24A0 */
  0x0004,   /* U+24A1 */
  0x0004,   /* U+24A2 */
  0x0004,   /* U+24A3 */
  0x0004,   /* U+24A4 */
  0x0004,   /* U+24A5 */
  0x0004,   /* U+24A6 */
  0x0004,   /* U+24A7 */
  0x0004,   /* U+24A8 */
  0x0004,   /* U+24A9 */
  0x0004,   /* U+24AA */
  0x0004,   /* U+24AB */
  0x0004,   /* U+24AC */
  0x0004,   /* U+24AD */
  0x0004,   /* U+24AE */
  0x0004,   /* U+24AF */
  0x0004,   /* U+24B0 */
  0x0004,   /* U+24B1 */
  0x0004,   /* U+24B2 */
  0x0004,   /* U+24B3 */
  0x0004,   /* U+24B4 */
  0x0004,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0006,   /* U+24EB */
  0x0006,   /* U+24EC */
  0x0006,   /* U+24ED */
  0x0006,   /* U+24EE */
  0x0006,   /* U+24EF */
  0x0006,   /* U+24F0 */
  0x0006,   /* U+24F1 */
  0x0006,   /* U+24F2 */
  0x0006,   /* U+24F3 */
  0x0006,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0006,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0318,   /* U+2474 */
  0x0318,   /* U+2475 */
  0x0318,   /* U+2476 */
  0x0318,   /* U+2477 */
  0x0318,   /* U+2478 */
  0x0318,   /* U+2479 */
  0x0318,   /* U+247A */
  0x0318,   /* U+247B */
  0x0318,   /* U+247C */
  0x1C3D,   /* U+247D */
  0x1C3E,   /* U+247E */
  0x1C3F,   /* U+247F */
  0x1C40,   /* U+2480 */
  0x1C41,   /* U+2481 */
  0x1C42,   /* U+2482 */
  0x1C43,   /* U+2483 */
  0x1C44,   /* U+2484 */
  0x1C45,   /* U+2485 */
  0x1C46,   /* U+2486 */
  0x1C3D,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0277,   /* U+2491 */
  0x0277,   /* U+2492 */
  0x0277,   /* U+2493 */
  0x0277,   /* U+2494 */
  0x0277,   /* U+2495 */
  0x0277,   /* U+2496 */
  0x0277,   /* U+2497 */
  0x0277,   /* U+2498 */
  0x0277,   /* U+2499 */
  0x0277,   /* U+249A */
  0x0277,   /* U+249B */
  0x0318,   /* U+249C */
  0x0318,   /* U+249D */
  0x0318,   /* U+249E */
  0x0318,   /* U+249F */
  0x0318,   /* U+24A0 */
  0x0318,   /* U+24A1 */
  0x0318,   /* U+24A2 */
  0x0318,   /* U+24A3 */
  0x0318,   /* U+24A4 */
  0x0318,   /* U+24A5 */
  0x0318,   /* U+24A6 */
  0x0318,   /* U+24A7 */
  0x0318,   /* U+24A8 */
  0x0318,   /* U+24A9 */
  0x0318,   /* U+24AA */
  0x0318,   /* U+24AB */
  0x0318,   /* U+24AC */
  0x0318,   /* U+24AD */
  0x0318,   /* U+24AE */
  0x0318,   /* U+24AF */
  0x0318,   /* U+24B0 */
  0x0318,   /* U+24B1 */
  0x0318,   /* U+24B2 */
  0x0318,   /* U+24B3 */
  0x0318,   /* U+24B4 */
  0x0318,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0020,   /* U+2474 */
  0x0020,   /* U+2475 */
  0x0020,   /* U+2476 */
  0x0020,   /* U+2477 */
  0x0020,   /* U+2478 */
  0x0020,   /* U+2479 */
  0x0020,   /* U+247A */
  0x0020,   /* U+247B */
  0x0020,   /* U+247C */
  0x0020,   /* U+247D */
  0x0020,   /* U+247E */
  0x0020,   /* U+247F */
  0x0020,   /* U+2480 */
  0x0020,   /* U+2481 */
  0x0020,   /* U+2482 */
  0x0020,   /* U+2483 */
  0x0020,   /* U+2484 */
  0x0020,   /* U+2485 */
  0x0020,   /* U+2486 */
  0x0020,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0020,   /* U+2491 */
  0x0020,   /* U+2492 */
  0x0020,   /* U+2493 */
  0x0020,   /* U+2494 */
  0x0020,   /* U+2495 */
  0x0020,   /* U+2496 */
  0x0020,   /* U+2497 */
  0x0020,   /* U+2498 */
  0x0020,   /* U+2499 */
  0x0020,   /* U+249A */
  0x0020,   /* U+249B */
  0x0020,   /* U+249C */
  0x0020,   /* U+249D */
  0x0020,   /* U+249E */
  0x0020,   /* U+249F */
  0x0020,   /* U+24A0 */
  0x0020,   /* U+24A1 */
  0x0020,   /* U+24A2 */
  0x0020,   /* U+24A3 */
  0x0020,   /* U+24A4 */
  0x0020,   /* U+24A5 */
  0x0020,   /* U+24A6 */
  0x0020,   /* U+24A7 */
  0x0020,   /* U+24A8 */
  0x0020,   /* U+24A9 */
  0x0020,   /* U+24AA */
  0x0020,   /* U+24AB */
  0x0020,   /* U+24AC */
  0x0020,   /* U+24AD */
  0x0020,   /* U+24AE */
  0x0020,   /* U+24AF */
  0x0020,   /* U+24B0 */
  0x0020,   /* U+24B1 */
  0x0020,   /* U+24B2 */
  0x0020,   /* U+24B3 */
  0x0020,   /* U+24B4 */
  0x0020,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0004,   /* U+2474 */
  0x0004,   /* U+2475 */
  0x0004,   /* U+2476 */
  0x0004,   /* U+2477 */
  0x0004,   /* U+2478 */
  0x0004,   /* U+2479 */
  0x0004,   /* U+247A */
  0x0004,   /* U+247B */
  0x0004,   /* U+247C */
  0x0004,   /* U+247D */
  0x0004,   /* U+247E */
  0x0004,   /* U+247F */
  0x0004,   /* U+2480 */
  0x0004,   /* U+2481 */
  0x0004,   /* U+2482 */
  0x0004,   /* U+2483 */
  0x0004,   /* U+2484 */
  0x0004,   /* U+2485 */
  0x0004,   /* U+2486 */
  0x0004,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0004,   /* U+2491 */
  0x0004,   /* U+2492 */
  0x0004,   /* U+2493 */
  0x0004,   /* U+2494 */
  0x0004,   /* U+2495 */
  0x0004,   /* U+2496 */
  0x0004,   /* U+2497 */
  0x0004,   /* U+2498 */
  0x0004,   /* U+2499 */
  0x0004,   /* U+249A */
  0x0004,   /* U+249B */
  0x0004,   /* U+249C */
  0x0004,   /* U+249D */
  0x0004,   /* U+249E */
  0x0004,   /* U+249F */
  0x0004,   /* U+24A0 */
  0x0004,   /* U+24A1 */
  0x0004,   /* U+24A2 */
  0x0004,   /* U+24A3 */
  0x0004,   /* U+24A4 */
  0x0004,   /* U+24A5 */
  0x0004,   /* U+24A6 */
  0x0004,   /* U+24A7 */
  0x0004,   /* U+24A8 */
  0x0004,   /* U+24A9 */
  0x0004,   /* U+24AA */
  0x0004,   /* U+24AB */
  0x0004,   /* U+24AC */
  0x0004,   /* U+24AD */
  0x0004,   /* U+24AE */
  0x0004,   /* U+24AF */
  0x0004,   /* U+24B0 */
  0x0004,   /* U+24B1 */
  0x0004,   /* U+24B2 */
  0x0004,   /* U+24B3 */
  0x0004,   /* U+24B4 */
  0x0004,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0000,   /* U+2474 */
  0x0000,   /* U+2475 */
  0x0000,   /* U+2476 */
  0x0000,   /* U+2477 */
  0x0000,   /* U+2478 */
  0x0000,   /* U+2479 */
  0x0000,   /* U+247A */
  0x0000,   /* U+247B */
  0x0000,   /* U+247C */
  0x0318,   /* U+247D */
  0x0318,   /* U+247E */
  0x0318,   /* U+247F */
  0x0318,   /* U+2480 */
  0x0318,   /* U+2481 */
  0x0318,   /* U+2482 */
  0x0318,   /* U+2483 */
  0x0318,   /* U+2484 */
  0x0318,   /* U+2485 */
  0x0318,   /* U+2486 */
  0x0318,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0000,   /* U+2491 */
  0x0000,   /* U+2492 */
  0x0000,   /* U+2493 */
  0x0000,   /* U+2494 */
  0x0000,   /* U+2495 */
  0x0000,   /* U+2496 */
  0x0000,   /* U+2497 */
  0x0000,   /* U+2498 */
  0x0000,   /* U+2499 */
  0x0000,   /* U+249A */
  0x0000,   /* U+249B */
  0x0000,   /* U+249C */
  0x0000,   /* U+249D */
  0x0000,   /* U+249E */
  0x0000,   /* U+249F */
  0x0000,   /* U+24A0 */
  0x0000,   /* U+24A1 */
  0x0000,   /* U+24A2 */
  0x0000,   /* U+24A3 */
  0x0000,   /* U+24A4 */
  0x0000,   /* U+24A5 */
  0x0000,   /* U+24A6 */
  0x0000,   /* U+24A7 */
  0x0000,   /* U+24A8 */
  0x0000,   /* U+24A9 */
  0x0000,   /* U+24AA */
  0x0000,   /* U+24AB */
  0x0000,   /* U+24AC */
  0x0000,   /* U+24AD */
  0x0000,   /* U+24AE */
  0x0000,   /* U+24AF */
  0x0000,   /* U+24B0 */
  0x0000,   /* U+24B1 */
  0x0000,   /* U+24B2 */
  0x0000,   /* U+24B3 */
  0x0000,   /* U+24B4 */
  0x0000,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0000,   /* U+2474 */
  0x0000,   /* U+2475 */
  0x0000,   /* U+2476 */
  0x0000,   /* U+2477 */
  0x0000,   /* U+2478 */
  0x0000,   /* U+2479 */
  0x0000,   /* U+247A */
  0x0000,   /* U+247B */
  0x0000,   /* U+247C */
  0x0020,   /* U+247D */
  0x0020,   /* U+247E */
  0x0020,   /* U+247F */
  0x0020,   /* U+2480 */
  0x0020,   /* U+2481 */
  0x0020,   /* U+2482 */
  0x0020,   /* U+2483 */
  0x0020,   /* U+2484 */
  0x0020,   /* U+2485 */
  0x0020,   /* U+2486 */
  0x0020,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0000,   /* U+2491 */
  0x0000,   /* U+2492 */
  0x0000,   /* U+2493 */
  0x0000,   /* U+2494 */
  0x0000,   /* U+2495 */
  0x0000,   /* U+2496 */
  0x0000,   /* U+2497 */
  0x0000,   /* U+2498 */
  0x0000,   /* U+2499 */
  0x0000,   /* U+249A */
  0x0000,   /* U+249B */
  0x0000,   /* U+249C */
  0x0000,   /* U+249D */
  0x0000,   /* U+249E */
  0x0000,   /* U+249F */
  0x0000,   /* U+24A0 */
  0x0000,   /* U+24A1 */
  0x0000,   /* U+24A2 */
  0x0000,   /* U+24A3 */
  0x0000,   /* U+24A4 */
  0x0000,   /* U+24A5 */
  0x0000,   /* U+24A6 */
  0x0000,   /* U+24A7 */
  0x0000,   /* U+24A8 */
  0x0000,   /* U+24A9 */
  0x0000,   /* U+24AA */
  0x0000,   /* U+24AB */
  0x0000,   /* U+24AC */
  0x0000,   /* U+24AD */
  0x0000,   /* U+24AE */
  0x0000,   /* U+24AF */
  0x0000,   /* U+24B0 */
  0x0000,   /* U+24B1 */
  0x0000,   /* U+24B2 */
  0x0000,   /* U+24B3 */
  0x0000,   /* U+24B4 */
  0x0000,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+2400 */
  0x0000,   /* U+2401 */
  0x0000,   /* U+2402 */
  0x0000,   /* U+2403 */
  0x0000,   /* U+2404 */
  0x0000,   /* U+2405 */
  0x0000,   /* U+2406 */
  0x0000,   /* U+2407 */
  0x0000,   /* U+2408 */
  0x0000,   /* U+2409 */
  0x0000,   /* U+240A */
  0x0000,   /* U+240B */
  0x0000,   /* U+240C */
  0x0000,   /* U+240D */
  0x0000,   /* U+240E */
  0x0000,   /* U+240F */
  0x0000,   /* U+2410 */
  0x0000,   /* U+2411 */
  0x0000,   /* U+2412 */
  0x0000,   /* U+2413 */
  0x0000,   /* U+2414 */
  0x0000,   /* U+2415 */
  0x0000,   /* U+2416 */
  0x0000,   /* U+2417 */
  0x0000,   /* U+2418 */
  0x0000,   /* U+2419 */
  0x0000,   /* U+241A */
  0x0000,   /* U+241B */
  0x0000,   /* U+241C */
  0x0000,   /* U+241D */
  0x0000,   /* U+241E */
  0x0000,   /* U+241F */
  0x0000,   /* U+2420 */
  0x0000,   /* U+2421 */
  0x0000,   /* U+2422 */
  0x0000,   /* U+2423 */
  0x0000,   /* U+2424 */
  0x0000,   /* U+2425 */
  0x0000,   /* U+2426 */
  0x0000,   /* U+2427 */
  0x0000,   /* U+2428 */
  0x0000,   /* U+2429 */
  0x0000,   /* U+242A */
  0x0000,   /* U+242B */
  0x0000,   /* U+242C */
  0x0000,   /* U+242D */
  0x0000,   /* U+242E */
  0x0000,   /* U+242F */
  0x0000,   /* U+2430 */
  0x0000,   /* U+2431 */
  0x0000,   /* U+2432 */
  0x0000,   /* U+2433 */
  0x0000,   /* U+2434 */
  0x0000,   /* U+2435 */
  0x0000,   /* U+2436 */
  0x0000,   /* U+2437 */
  0x0000,   /* U+2438 */
  0x0000,   /* U+2439 */
  0x0000,   /* U+243A */
  0x0000,   /* U+243B */
  0x0000,   /* U+243C */
  0x0000,   /* U+243D */
  0x0000,   /* U+243E */
  0x0000,   /* U+243F */
  0x0000,   /* U+2440 */
  0x0000,   /* U+2441 */
  0x0000,   /* U+2442 */
  0x0000,   /* U+2443 */
  0x0000,   /* U+2444 */
  0x0000,   /* U+2445 */
  0x0000,   /* U+2446 */
  0x0000,   /* U+2447 */
  0x0000,   /* U+2448 */
  0x0000,   /* U+2449 */
  0x0000,   /* U+244A */
  0x0000,   /* U+244B */
  0x0000,   /* U+244C */
  0x0000,   /* U+244D */
  0x0000,   /* U+244E */
  0x0000,   /* U+244F */
  0x0000,   /* U+2450 */
  0x0000,   /* U+2451 */
  0x0000,   /* U+2452 */
  0x0000,   /* U+2453 */
  0x0000,   /* U+2454 */
  0x0000,   /* U+2455 */
  0x0000,   /* U+2456 */
  0x0000,   /* U+2457 */
  0x0000,   /* U+2458 */
  0x0000,   /* U+2459 */
  0x0000,   /* U+245A */
  0x0000,   /* U+245B */
  0x0000,   /* U+245C */
  0x0000,   /* U+245D */
  0x0000,   /* U+245E */
  0x0000,   /* U+245F */
  0x0000,   /* U+2460 */
  0x0000,   /* U+2461 */
  0x0000,   /* U+2462 */
  0x0000,   /* U+2463 */
  0x0000,   /* U+2464 */
  0x0000,   /* U+2465 */
  0x0000,   /* U+2466 */
  0x0000,   /* U+2467 */
  0x0000,   /* U+2468 */
  0x0000,   /* U+2469 */
  0x0000,   /* U+246A */
  0x0000,   /* U+246B */
  0x0000,   /* U+246C */
  0x0000,   /* U+246D */
  0x0000,   /* U+246E */
  0x0000,   /* U+246F */
  0x0000,   /* U+2470 */
  0x0000,   /* U+2471 */
  0x0000,   /* U+2472 */
  0x0000,   /* U+2473 */
  0x0000,   /* U+2474 */
  0x0000,   /* U+2475 */
  0x0000,   /* U+2476 */
  0x0000,   /* U+2477 */
  0x0000,   /* U+2478 */
  0x0000,   /* U+2479 */
  0x0000,   /* U+247A */
  0x0000,   /* U+247B */
  0x0000,   /* U+247C */
  0x0004,   /* U+247D */
  0x0004,   /* U+247E */
  0x0004,   /* U+247F */
  0x0004,   /* U+2480 */
  0x0004,   /* U+2481 */
  0x0004,   /* U+2482 */
  0x0004,   /* U+2483 */
  0x0004,   /* U+2484 */
  0x0004,   /* U+2485 */
  0x0004,   /* U+2486 */
  0x0004,   /* U+2487 */
  0x0000,   /* U+2488 */
  0x0000,   /* U+2489 */
  0x0000,   /* U+248A */
  0x0000,   /* U+248B */
  0x0000,   /* U+248C */
  0x0000,   /* U+248D */
  0x0000,   /* U+248E */
  0x0000,   /* U+248F */
  0x0000,   /* U+2490 */
  0x0000,   /* U+2491 */
  0x0000,   /* U+2492 */
  0x0000,   /* U+2493 */
  0x0000,   /* U+2494 */
  0x0000,   /* U+2495 */
  0x0000,   /* U+2496 */
  0x0000,   /* U+2497 */
  0x0000,   /* U+2498 */
  0x0000,   /* U+2499 */
  0x0000,   /* U+249A */
  0x0000,   /* U+249B */
  0x0000,   /* U+249C */
  0x0000,   /* U+249D */
  0x0000,   /* U+249E */
  0x0000,   /* U+249F */
  0x0000,   /* U+24A0 */
  0x0000,   /* U+24A1 */
  0x0000,   /* U+24A2 */
  0x0000,   /* U+24A3 */
  0x0000,   /* U+24A4 */
  0x0000,   /* U+24A5 */
  0x0000,   /* U+24A6 */
  0x0000,   /* U+24A7 */
  0x0000,   /* U+24A8 */
  0x0000,   /* U+24A9 */
  0x0000,   /* U+24AA */
  0x0000,   /* U+24AB */
  0x0000,   /* U+24AC */
  0x0000,   /* U+24AD */
  0x0000,   /* U+24AE */
  0x0000,   /* U+24AF */
  0x0000,   /* U+24B0 */
  0x0000,   /* U+24B1 */
  0x0000,   /* U+24B2 */
  0x0000,   /* U+24B3 */
  0x0000,   /* U+24B4 */
  0x0000,   /* U+24B5 */
  0x0000,   /* U+24B6 */
  0x0000,   /* U+24B7 */
  0x0000,   /* U+24B8 */
  0x0000,   /* U+24B9 */
  0x0000,   /* U+24BA */
  0x0000,   /* U+24BB */
  0x0000,   /* U+24BC */
  0x0000,   /* U+24BD */
  0x0000,   /* U+24BE */
  0x0000,   /* U+24BF */
  0x0000,   /* U+24C0 */
  0x0000,   /* U+24C1 */
  0x0000,   /* U+24C2 */
  0x0000,   /* U+24C3 */
  0x0000,   /* U+24C4 */
  0x0000,   /* U+24C5 */
  0x0000,   /* U+24C6 */
  0x0000,   /* U+24C7 */
  0x0000,   /* U+24C8 */
  0x0000,   /* U+24C9 */
  0x0000,   /* U+24CA */
  0x0000,   /* U+24CB */
  0x0000,   /* U+24CC */
  0x0000,   /* U+24CD */
  0x0000,   /* U+24CE */
  0x0000,   /* U+24CF */
  0x0000,   /* U+24D0 */
  0x0000,   /* U+24D1 */
  0x0000,   /* U+24D2 */
  0x0000,   /* U+24D3 */
  0x0000,   /* U+24D4 */
  0x0000,   /* U+24D5 */
  0x0000,   /* U+24D6 */
  0x0000,   /* U+24D7 */
  0x0000,   /* U+24D8 */
  0x0000,   /* U+24D9 */
  0x0000,   /* U+24DA */
  0x0000,   /* U+24DB */
  0x0000,   /* U+24DC */
  0x0000,   /* U+24DD */
  0x0000,   /* U+24DE */
  0x0000,   /* U+24DF */
  0x0000,   /* U+24E0 */
  0x0000,   /* U+24E1 */
  0x0000,   /* U+24E2 */
  0x0000,   /* U+24E3 */
  0x0000,   /* U+24E4 */
  0x0000,   /* U+24E5 */
  0x0000,   /* U+24E6 */
  0x0000,   /* U+24E7 */
  0x0000,   /* U+24E8 */
  0x0000,   /* U+24E9 */
  0x0000,   /* U+24EA */
  0x0000,   /* U+24EB */
  0x0000,   /* U+24EC */
  0x0000,   /* U+24ED */
  0x0000,   /* U+24EE */
  0x0000,   /* U+24EF */
  0x0000,   /* U+24F0 */
  0x0000,   /* U+24F1 */
  0x0000,   /* U+24F2 */
  0x0000,   /* U+24F3 */
  0x0000,   /* U+24F4 */
  0x0000,   /* U+24F5 */
  0x0000,   /* U+24F6 */
  0x0000,   /* U+24F7 */
  0x0000,   /* U+24F8 */
  0x0000,   /* U+24F9 */
  0x0000,   /* U+24FA */
  0x0000,   /* U+24FB */
  0x0000,   /* U+24FC */
  0x0000,   /* U+24FD */
  0x0000,   /* U+24FE */
  0x0000,   /* U+24FF */
};

uint16 uca900_p025[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0818,   /* U+2500 */
  0x0819,   /* U+2501 */
  0x081A,   /* U+2502 */
  0x081B,   /* U+2503 */
  0x081C,   /* U+2504 */
  0x081D,   /* U+2505 */
  0x081E,   /* U+2506 */
  0x081F,   /* U+2507 */
  0x0820,   /* U+2508 */
  0x0821,   /* U+2509 */
  0x0822,   /* U+250A */
  0x0823,   /* U+250B */
  0x0824,   /* U+250C */
  0x0825,   /* U+250D */
  0x0826,   /* U+250E */
  0x0827,   /* U+250F */
  0x0828,   /* U+2510 */
  0x0829,   /* U+2511 */
  0x082A,   /* U+2512 */
  0x082B,   /* U+2513 */
  0x082C,   /* U+2514 */
  0x082D,   /* U+2515 */
  0x082E,   /* U+2516 */
  0x082F,   /* U+2517 */
  0x0830,   /* U+2518 */
  0x0831,   /* U+2519 */
  0x0832,   /* U+251A */
  0x0833,   /* U+251B */
  0x0834,   /* U+251C */
  0x0835,   /* U+251D */
  0x0836,   /* U+251E */
  0x0837,   /* U+251F */
  0x0838,   /* U+2520 */
  0x0839,   /* U+2521 */
  0x083A,   /* U+2522 */
  0x083B,   /* U+2523 */
  0x083C,   /* U+2524 */
  0x083D,   /* U+2525 */
  0x083E,   /* U+2526 */
  0x083F,   /* U+2527 */
  0x0840,   /* U+2528 */
  0x0841,   /* U+2529 */
  0x0842,   /* U+252A */
  0x0843,   /* U+252B */
  0x0844,   /* U+252C */
  0x0845,   /* U+252D */
  0x0846,   /* U+252E */
  0x0847,   /* U+252F */
  0x0848,   /* U+2530 */
  0x0849,   /* U+2531 */
  0x084A,   /* U+2532 */
  0x084B,   /* U+2533 */
  0x084C,   /* U+2534 */
  0x084D,   /* U+2535 */
  0x084E,   /* U+2536 */
  0x084F,   /* U+2537 */
  0x0850,   /* U+2538 */
  0x0851,   /* U+2539 */
  0x0852,   /* U+253A */
  0x0853,   /* U+253B */
  0x0854,   /* U+253C */
  0x0855,   /* U+253D */
  0x0856,   /* U+253E */
  0x0857,   /* U+253F */
  0x0858,   /* U+2540 */
  0x0859,   /* U+2541 */
  0x085A,   /* U+2542 */
  0x085B,   /* U+2543 */
  0x085C,   /* U+2544 */
  0x085D,   /* U+2545 */
  0x085E,   /* U+2546 */
  0x085F,   /* U+2547 */
  0x0860,   /* U+2548 */
  0x0861,   /* U+2549 */
  0x0862,   /* U+254A */
  0x0863,   /* U+254B */
  0x0864,   /* U+254C */
  0x0865,   /* U+254D */
  0x0866,   /* U+254E */
  0x0867,   /* U+254F */
  0x0868,   /* U+2550 */
  0x0869,   /* U+2551 */
  0x086A,   /* U+2552 */
  0x086B,   /* U+2553 */
  0x086C,   /* U+2554 */
  0x086D,   /* U+2555 */
  0x086E,   /* U+2556 */
  0x086F,   /* U+2557 */
  0x0870,   /* U+2558 */
  0x0871,   /* U+2559 */
  0x0872,   /* U+255A */
  0x0873,   /* U+255B */
  0x0874,   /* U+255C */
  0x0875,   /* U+255D */
  0x0876,   /* U+255E */
  0x0877,   /* U+255F */
  0x0878,   /* U+2560 */
  0x0879,   /* U+2561 */
  0x087A,   /* U+2562 */
  0x087B,   /* U+2563 */
  0x087C,   /* U+2564 */
  0x087D,   /* U+2565 */
  0x087E,   /* U+2566 */
  0x087F,   /* U+2567 */
  0x0880,   /* U+2568 */
  0x0881,   /* U+2569 */
  0x0882,   /* U+256A */
  0x0883,   /* U+256B */
  0x0884,   /* U+256C */
  0x0885,   /* U+256D */
  0x0886,   /* U+256E */
  0x0887,   /* U+256F */
  0x0888,   /* U+2570 */
  0x0889,   /* U+2571 */
  0x088A,   /* U+2572 */
  0x088B,   /* U+2573 */
  0x088C,   /* U+2574 */
  0x088D,   /* U+2575 */
  0x088E,   /* U+2576 */
  0x088F,   /* U+2577 */
  0x0890,   /* U+2578 */
  0x0891,   /* U+2579 */
  0x0892,   /* U+257A */
  0x0893,   /* U+257B */
  0x0894,   /* U+257C */
  0x0895,   /* U+257D */
  0x0896,   /* U+257E */
  0x0897,   /* U+257F */
  0x0898,   /* U+2580 */
  0x0899,   /* U+2581 */
  0x089A,   /* U+2582 */
  0x089B,   /* U+2583 */
  0x089C,   /* U+2584 */
  0x089D,   /* U+2585 */
  0x089E,   /* U+2586 */
  0x089F,   /* U+2587 */
  0x08A0,   /* U+2588 */
  0x08A1,   /* U+2589 */
  0x08A2,   /* U+258A */
  0x08A3,   /* U+258B */
  0x08A4,   /* U+258C */
  0x08A5,   /* U+258D */
  0x08A6,   /* U+258E */
  0x08A7,   /* U+258F */
  0x08A8,   /* U+2590 */
  0x08A9,   /* U+2591 */
  0x08AA,   /* U+2592 */
  0x08AB,   /* U+2593 */
  0x08AC,   /* U+2594 */
  0x08AD,   /* U+2595 */
  0x08AE,   /* U+2596 */
  0x08AF,   /* U+2597 */
  0x08B0,   /* U+2598 */
  0x08B1,   /* U+2599 */
  0x08B2,   /* U+259A */
  0x08B3,   /* U+259B */
  0x08B4,   /* U+259C */
  0x08B5,   /* U+259D */
  0x08B6,   /* U+259E */
  0x08B7,   /* U+259F */
  0x08B8,   /* U+25A0 */
  0x08B9,   /* U+25A1 */
  0x08BA,   /* U+25A2 */
  0x08BB,   /* U+25A3 */
  0x08BC,   /* U+25A4 */
  0x08BD,   /* U+25A5 */
  0x08BE,   /* U+25A6 */
  0x08BF,   /* U+25A7 */
  0x08C0,   /* U+25A8 */
  0x08C1,   /* U+25A9 */
  0x08C2,   /* U+25AA */
  0x08C3,   /* U+25AB */
  0x08C4,   /* U+25AC */
  0x08C5,   /* U+25AD */
  0x08C6,   /* U+25AE */
  0x08C7,   /* U+25AF */
  0x08C8,   /* U+25B0 */
  0x08C9,   /* U+25B1 */
  0x08CA,   /* U+25B2 */
  0x08CB,   /* U+25B3 */
  0x08CC,   /* U+25B4 */
  0x08CD,   /* U+25B5 */
  0x08CE,   /* U+25B6 */
  0x08CF,   /* U+25B7 */
  0x08D0,   /* U+25B8 */
  0x08D1,   /* U+25B9 */
  0x08D2,   /* U+25BA */
  0x08D3,   /* U+25BB */
  0x08D4,   /* U+25BC */
  0x08D5,   /* U+25BD */
  0x08D6,   /* U+25BE */
  0x08D7,   /* U+25BF */
  0x08D8,   /* U+25C0 */
  0x08D9,   /* U+25C1 */
  0x08DA,   /* U+25C2 */
  0x08DB,   /* U+25C3 */
  0x08DC,   /* U+25C4 */
  0x08DD,   /* U+25C5 */
  0x08DE,   /* U+25C6 */
  0x08DF,   /* U+25C7 */
  0x08E0,   /* U+25C8 */
  0x08E1,   /* U+25C9 */
  0x08E2,   /* U+25CA */
  0x08E3,   /* U+25CB */
  0x08E4,   /* U+25CC */
  0x08E5,   /* U+25CD */
  0x08E6,   /* U+25CE */
  0x08E7,   /* U+25CF */
  0x08E8,   /* U+25D0 */
  0x08E9,   /* U+25D1 */
  0x08EA,   /* U+25D2 */
  0x08EB,   /* U+25D3 */
  0x08EC,   /* U+25D4 */
  0x08ED,   /* U+25D5 */
  0x08EE,   /* U+25D6 */
  0x08EF,   /* U+25D7 */
  0x08F0,   /* U+25D8 */
  0x08F1,   /* U+25D9 */
  0x08F2,   /* U+25DA */
  0x08F3,   /* U+25DB */
  0x08F4,   /* U+25DC */
  0x08F5,   /* U+25DD */
  0x08F6,   /* U+25DE */
  0x08F7,   /* U+25DF */
  0x08F8,   /* U+25E0 */
  0x08F9,   /* U+25E1 */
  0x08FA,   /* U+25E2 */
  0x08FB,   /* U+25E3 */
  0x08FC,   /* U+25E4 */
  0x08FD,   /* U+25E5 */
  0x08FE,   /* U+25E6 */
  0x08FF,   /* U+25E7 */
  0x0900,   /* U+25E8 */
  0x0901,   /* U+25E9 */
  0x0902,   /* U+25EA */
  0x0903,   /* U+25EB */
  0x0904,   /* U+25EC */
  0x0905,   /* U+25ED */
  0x0906,   /* U+25EE */
  0x0907,   /* U+25EF */
  0x0908,   /* U+25F0 */
  0x0909,   /* U+25F1 */
  0x090A,   /* U+25F2 */
  0x090B,   /* U+25F3 */
  0x090C,   /* U+25F4 */
  0x090D,   /* U+25F5 */
  0x090E,   /* U+25F6 */
  0x090F,   /* U+25F7 */
  0x0910,   /* U+25F8 */
  0x0911,   /* U+25F9 */
  0x0912,   /* U+25FA */
  0x0913,   /* U+25FB */
  0x0914,   /* U+25FC */
  0x0915,   /* U+25FD */
  0x0916,   /* U+25FE */
  0x0917,   /* U+25FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2500 */
  0x0020,   /* U+2501 */
  0x0020,   /* U+2502 */
  0x0020,   /* U+2503 */
  0x0020,   /* U+2504 */
  0x0020,   /* U+2505 */
  0x0020,   /* U+2506 */
  0x0020,   /* U+2507 */
  0x0020,   /* U+2508 */
  0x0020,   /* U+2509 */
  0x0020,   /* U+250A */
  0x0020,   /* U+250B */
  0x0020,   /* U+250C */
  0x0020,   /* U+250D */
  0x0020,   /* U+250E */
  0x0020,   /* U+250F */
  0x0020,   /* U+2510 */
  0x0020,   /* U+2511 */
  0x0020,   /* U+2512 */
  0x0020,   /* U+2513 */
  0x0020,   /* U+2514 */
  0x0020,   /* U+2515 */
  0x0020,   /* U+2516 */
  0x0020,   /* U+2517 */
  0x0020,   /* U+2518 */
  0x0020,   /* U+2519 */
  0x0020,   /* U+251A */
  0x0020,   /* U+251B */
  0x0020,   /* U+251C */
  0x0020,   /* U+251D */
  0x0020,   /* U+251E */
  0x0020,   /* U+251F */
  0x0020,   /* U+2520 */
  0x0020,   /* U+2521 */
  0x0020,   /* U+2522 */
  0x0020,   /* U+2523 */
  0x0020,   /* U+2524 */
  0x0020,   /* U+2525 */
  0x0020,   /* U+2526 */
  0x0020,   /* U+2527 */
  0x0020,   /* U+2528 */
  0x0020,   /* U+2529 */
  0x0020,   /* U+252A */
  0x0020,   /* U+252B */
  0x0020,   /* U+252C */
  0x0020,   /* U+252D */
  0x0020,   /* U+252E */
  0x0020,   /* U+252F */
  0x0020,   /* U+2530 */
  0x0020,   /* U+2531 */
  0x0020,   /* U+2532 */
  0x0020,   /* U+2533 */
  0x0020,   /* U+2534 */
  0x0020,   /* U+2535 */
  0x0020,   /* U+2536 */
  0x0020,   /* U+2537 */
  0x0020,   /* U+2538 */
  0x0020,   /* U+2539 */
  0x0020,   /* U+253A */
  0x0020,   /* U+253B */
  0x0020,   /* U+253C */
  0x0020,   /* U+253D */
  0x0020,   /* U+253E */
  0x0020,   /* U+253F */
  0x0020,   /* U+2540 */
  0x0020,   /* U+2541 */
  0x0020,   /* U+2542 */
  0x0020,   /* U+2543 */
  0x0020,   /* U+2544 */
  0x0020,   /* U+2545 */
  0x0020,   /* U+2546 */
  0x0020,   /* U+2547 */
  0x0020,   /* U+2548 */
  0x0020,   /* U+2549 */
  0x0020,   /* U+254A */
  0x0020,   /* U+254B */
  0x0020,   /* U+254C */
  0x0020,   /* U+254D */
  0x0020,   /* U+254E */
  0x0020,   /* U+254F */
  0x0020,   /* U+2550 */
  0x0020,   /* U+2551 */
  0x0020,   /* U+2552 */
  0x0020,   /* U+2553 */
  0x0020,   /* U+2554 */
  0x0020,   /* U+2555 */
  0x0020,   /* U+2556 */
  0x0020,   /* U+2557 */
  0x0020,   /* U+2558 */
  0x0020,   /* U+2559 */
  0x0020,   /* U+255A */
  0x0020,   /* U+255B */
  0x0020,   /* U+255C */
  0x0020,   /* U+255D */
  0x0020,   /* U+255E */
  0x0020,   /* U+255F */
  0x0020,   /* U+2560 */
  0x0020,   /* U+2561 */
  0x0020,   /* U+2562 */
  0x0020,   /* U+2563 */
  0x0020,   /* U+2564 */
  0x0020,   /* U+2565 */
  0x0020,   /* U+2566 */
  0x0020,   /* U+2567 */
  0x0020,   /* U+2568 */
  0x0020,   /* U+2569 */
  0x0020,   /* U+256A */
  0x0020,   /* U+256B */
  0x0020,   /* U+256C */
  0x0020,   /* U+256D */
  0x0020,   /* U+256E */
  0x0020,   /* U+256F */
  0x0020,   /* U+2570 */
  0x0020,   /* U+2571 */
  0x0020,   /* U+2572 */
  0x0020,   /* U+2573 */
  0x0020,   /* U+2574 */
  0x0020,   /* U+2575 */
  0x0020,   /* U+2576 */
  0x0020,   /* U+2577 */
  0x0020,   /* U+2578 */
  0x0020,   /* U+2579 */
  0x0020,   /* U+257A */
  0x0020,   /* U+257B */
  0x0020,   /* U+257C */
  0x0020,   /* U+257D */
  0x0020,   /* U+257E */
  0x0020,   /* U+257F */
  0x0020,   /* U+2580 */
  0x0020,   /* U+2581 */
  0x0020,   /* U+2582 */
  0x0020,   /* U+2583 */
  0x0020,   /* U+2584 */
  0x0020,   /* U+2585 */
  0x0020,   /* U+2586 */
  0x0020,   /* U+2587 */
  0x0020,   /* U+2588 */
  0x0020,   /* U+2589 */
  0x0020,   /* U+258A */
  0x0020,   /* U+258B */
  0x0020,   /* U+258C */
  0x0020,   /* U+258D */
  0x0020,   /* U+258E */
  0x0020,   /* U+258F */
  0x0020,   /* U+2590 */
  0x0020,   /* U+2591 */
  0x0020,   /* U+2592 */
  0x0020,   /* U+2593 */
  0x0020,   /* U+2594 */
  0x0020,   /* U+2595 */
  0x0020,   /* U+2596 */
  0x0020,   /* U+2597 */
  0x0020,   /* U+2598 */
  0x0020,   /* U+2599 */
  0x0020,   /* U+259A */
  0x0020,   /* U+259B */
  0x0020,   /* U+259C */
  0x0020,   /* U+259D */
  0x0020,   /* U+259E */
  0x0020,   /* U+259F */
  0x0020,   /* U+25A0 */
  0x0020,   /* U+25A1 */
  0x0020,   /* U+25A2 */
  0x0020,   /* U+25A3 */
  0x0020,   /* U+25A4 */
  0x0020,   /* U+25A5 */
  0x0020,   /* U+25A6 */
  0x0020,   /* U+25A7 */
  0x0020,   /* U+25A8 */
  0x0020,   /* U+25A9 */
  0x0020,   /* U+25AA */
  0x0020,   /* U+25AB */
  0x0020,   /* U+25AC */
  0x0020,   /* U+25AD */
  0x0020,   /* U+25AE */
  0x0020,   /* U+25AF */
  0x0020,   /* U+25B0 */
  0x0020,   /* U+25B1 */
  0x0020,   /* U+25B2 */
  0x0020,   /* U+25B3 */
  0x0020,   /* U+25B4 */
  0x0020,   /* U+25B5 */
  0x0020,   /* U+25B6 */
  0x0020,   /* U+25B7 */
  0x0020,   /* U+25B8 */
  0x0020,   /* U+25B9 */
  0x0020,   /* U+25BA */
  0x0020,   /* U+25BB */
  0x0020,   /* U+25BC */
  0x0020,   /* U+25BD */
  0x0020,   /* U+25BE */
  0x0020,   /* U+25BF */
  0x0020,   /* U+25C0 */
  0x0020,   /* U+25C1 */
  0x0020,   /* U+25C2 */
  0x0020,   /* U+25C3 */
  0x0020,   /* U+25C4 */
  0x0020,   /* U+25C5 */
  0x0020,   /* U+25C6 */
  0x0020,   /* U+25C7 */
  0x0020,   /* U+25C8 */
  0x0020,   /* U+25C9 */
  0x0020,   /* U+25CA */
  0x0020,   /* U+25CB */
  0x0020,   /* U+25CC */
  0x0020,   /* U+25CD */
  0x0020,   /* U+25CE */
  0x0020,   /* U+25CF */
  0x0020,   /* U+25D0 */
  0x0020,   /* U+25D1 */
  0x0020,   /* U+25D2 */
  0x0020,   /* U+25D3 */
  0x0020,   /* U+25D4 */
  0x0020,   /* U+25D5 */
  0x0020,   /* U+25D6 */
  0x0020,   /* U+25D7 */
  0x0020,   /* U+25D8 */
  0x0020,   /* U+25D9 */
  0x0020,   /* U+25DA */
  0x0020,   /* U+25DB */
  0x0020,   /* U+25DC */
  0x0020,   /* U+25DD */
  0x0020,   /* U+25DE */
  0x0020,   /* U+25DF */
  0x0020,   /* U+25E0 */
  0x0020,   /* U+25E1 */
  0x0020,   /* U+25E2 */
  0x0020,   /* U+25E3 */
  0x0020,   /* U+25E4 */
  0x0020,   /* U+25E5 */
  0x0020,   /* U+25E6 */
  0x0020,   /* U+25E7 */
  0x0020,   /* U+25E8 */
  0x0020,   /* U+25E9 */
  0x0020,   /* U+25EA */
  0x0020,   /* U+25EB */
  0x0020,   /* U+25EC */
  0x0020,   /* U+25ED */
  0x0020,   /* U+25EE */
  0x0020,   /* U+25EF */
  0x0020,   /* U+25F0 */
  0x0020,   /* U+25F1 */
  0x0020,   /* U+25F2 */
  0x0020,   /* U+25F3 */
  0x0020,   /* U+25F4 */
  0x0020,   /* U+25F5 */
  0x0020,   /* U+25F6 */
  0x0020,   /* U+25F7 */
  0x0020,   /* U+25F8 */
  0x0020,   /* U+25F9 */
  0x0020,   /* U+25FA */
  0x0020,   /* U+25FB */
  0x0020,   /* U+25FC */
  0x0020,   /* U+25FD */
  0x0020,   /* U+25FE */
  0x0020,   /* U+25FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2500 */
  0x0002,   /* U+2501 */
  0x0002,   /* U+2502 */
  0x0002,   /* U+2503 */
  0x0002,   /* U+2504 */
  0x0002,   /* U+2505 */
  0x0002,   /* U+2506 */
  0x0002,   /* U+2507 */
  0x0002,   /* U+2508 */
  0x0002,   /* U+2509 */
  0x0002,   /* U+250A */
  0x0002,   /* U+250B */
  0x0002,   /* U+250C */
  0x0002,   /* U+250D */
  0x0002,   /* U+250E */
  0x0002,   /* U+250F */
  0x0002,   /* U+2510 */
  0x0002,   /* U+2511 */
  0x0002,   /* U+2512 */
  0x0002,   /* U+2513 */
  0x0002,   /* U+2514 */
  0x0002,   /* U+2515 */
  0x0002,   /* U+2516 */
  0x0002,   /* U+2517 */
  0x0002,   /* U+2518 */
  0x0002,   /* U+2519 */
  0x0002,   /* U+251A */
  0x0002,   /* U+251B */
  0x0002,   /* U+251C */
  0x0002,   /* U+251D */
  0x0002,   /* U+251E */
  0x0002,   /* U+251F */
  0x0002,   /* U+2520 */
  0x0002,   /* U+2521 */
  0x0002,   /* U+2522 */
  0x0002,   /* U+2523 */
  0x0002,   /* U+2524 */
  0x0002,   /* U+2525 */
  0x0002,   /* U+2526 */
  0x0002,   /* U+2527 */
  0x0002,   /* U+2528 */
  0x0002,   /* U+2529 */
  0x0002,   /* U+252A */
  0x0002,   /* U+252B */
  0x0002,   /* U+252C */
  0x0002,   /* U+252D */
  0x0002,   /* U+252E */
  0x0002,   /* U+252F */
  0x0002,   /* U+2530 */
  0x0002,   /* U+2531 */
  0x0002,   /* U+2532 */
  0x0002,   /* U+2533 */
  0x0002,   /* U+2534 */
  0x0002,   /* U+2535 */
  0x0002,   /* U+2536 */
  0x0002,   /* U+2537 */
  0x0002,   /* U+2538 */
  0x0002,   /* U+2539 */
  0x0002,   /* U+253A */
  0x0002,   /* U+253B */
  0x0002,   /* U+253C */
  0x0002,   /* U+253D */
  0x0002,   /* U+253E */
  0x0002,   /* U+253F */
  0x0002,   /* U+2540 */
  0x0002,   /* U+2541 */
  0x0002,   /* U+2542 */
  0x0002,   /* U+2543 */
  0x0002,   /* U+2544 */
  0x0002,   /* U+2545 */
  0x0002,   /* U+2546 */
  0x0002,   /* U+2547 */
  0x0002,   /* U+2548 */
  0x0002,   /* U+2549 */
  0x0002,   /* U+254A */
  0x0002,   /* U+254B */
  0x0002,   /* U+254C */
  0x0002,   /* U+254D */
  0x0002,   /* U+254E */
  0x0002,   /* U+254F */
  0x0002,   /* U+2550 */
  0x0002,   /* U+2551 */
  0x0002,   /* U+2552 */
  0x0002,   /* U+2553 */
  0x0002,   /* U+2554 */
  0x0002,   /* U+2555 */
  0x0002,   /* U+2556 */
  0x0002,   /* U+2557 */
  0x0002,   /* U+2558 */
  0x0002,   /* U+2559 */
  0x0002,   /* U+255A */
  0x0002,   /* U+255B */
  0x0002,   /* U+255C */
  0x0002,   /* U+255D */
  0x0002,   /* U+255E */
  0x0002,   /* U+255F */
  0x0002,   /* U+2560 */
  0x0002,   /* U+2561 */
  0x0002,   /* U+2562 */
  0x0002,   /* U+2563 */
  0x0002,   /* U+2564 */
  0x0002,   /* U+2565 */
  0x0002,   /* U+2566 */
  0x0002,   /* U+2567 */
  0x0002,   /* U+2568 */
  0x0002,   /* U+2569 */
  0x0002,   /* U+256A */
  0x0002,   /* U+256B */
  0x0002,   /* U+256C */
  0x0002,   /* U+256D */
  0x0002,   /* U+256E */
  0x0002,   /* U+256F */
  0x0002,   /* U+2570 */
  0x0002,   /* U+2571 */
  0x0002,   /* U+2572 */
  0x0002,   /* U+2573 */
  0x0002,   /* U+2574 */
  0x0002,   /* U+2575 */
  0x0002,   /* U+2576 */
  0x0002,   /* U+2577 */
  0x0002,   /* U+2578 */
  0x0002,   /* U+2579 */
  0x0002,   /* U+257A */
  0x0002,   /* U+257B */
  0x0002,   /* U+257C */
  0x0002,   /* U+257D */
  0x0002,   /* U+257E */
  0x0002,   /* U+257F */
  0x0002,   /* U+2580 */
  0x0002,   /* U+2581 */
  0x0002,   /* U+2582 */
  0x0002,   /* U+2583 */
  0x0002,   /* U+2584 */
  0x0002,   /* U+2585 */
  0x0002,   /* U+2586 */
  0x0002,   /* U+2587 */
  0x0002,   /* U+2588 */
  0x0002,   /* U+2589 */
  0x0002,   /* U+258A */
  0x0002,   /* U+258B */
  0x0002,   /* U+258C */
  0x0002,   /* U+258D */
  0x0002,   /* U+258E */
  0x0002,   /* U+258F */
  0x0002,   /* U+2590 */
  0x0002,   /* U+2591 */
  0x0002,   /* U+2592 */
  0x0002,   /* U+2593 */
  0x0002,   /* U+2594 */
  0x0002,   /* U+2595 */
  0x0002,   /* U+2596 */
  0x0002,   /* U+2597 */
  0x0002,   /* U+2598 */
  0x0002,   /* U+2599 */
  0x0002,   /* U+259A */
  0x0002,   /* U+259B */
  0x0002,   /* U+259C */
  0x0002,   /* U+259D */
  0x0002,   /* U+259E */
  0x0002,   /* U+259F */
  0x0002,   /* U+25A0 */
  0x0002,   /* U+25A1 */
  0x0002,   /* U+25A2 */
  0x0002,   /* U+25A3 */
  0x0002,   /* U+25A4 */
  0x0002,   /* U+25A5 */
  0x0002,   /* U+25A6 */
  0x0002,   /* U+25A7 */
  0x0002,   /* U+25A8 */
  0x0002,   /* U+25A9 */
  0x0002,   /* U+25AA */
  0x0002,   /* U+25AB */
  0x0002,   /* U+25AC */
  0x0002,   /* U+25AD */
  0x0002,   /* U+25AE */
  0x0002,   /* U+25AF */
  0x0002,   /* U+25B0 */
  0x0002,   /* U+25B1 */
  0x0002,   /* U+25B2 */
  0x0002,   /* U+25B3 */
  0x0002,   /* U+25B4 */
  0x0002,   /* U+25B5 */
  0x0002,   /* U+25B6 */
  0x0002,   /* U+25B7 */
  0x0002,   /* U+25B8 */
  0x0002,   /* U+25B9 */
  0x0002,   /* U+25BA */
  0x0002,   /* U+25BB */
  0x0002,   /* U+25BC */
  0x0002,   /* U+25BD */
  0x0002,   /* U+25BE */
  0x0002,   /* U+25BF */
  0x0002,   /* U+25C0 */
  0x0002,   /* U+25C1 */
  0x0002,   /* U+25C2 */
  0x0002,   /* U+25C3 */
  0x0002,   /* U+25C4 */
  0x0002,   /* U+25C5 */
  0x0002,   /* U+25C6 */
  0x0002,   /* U+25C7 */
  0x0002,   /* U+25C8 */
  0x0002,   /* U+25C9 */
  0x0002,   /* U+25CA */
  0x0002,   /* U+25CB */
  0x0002,   /* U+25CC */
  0x0002,   /* U+25CD */
  0x0002,   /* U+25CE */
  0x0002,   /* U+25CF */
  0x0002,   /* U+25D0 */
  0x0002,   /* U+25D1 */
  0x0002,   /* U+25D2 */
  0x0002,   /* U+25D3 */
  0x0002,   /* U+25D4 */
  0x0002,   /* U+25D5 */
  0x0002,   /* U+25D6 */
  0x0002,   /* U+25D7 */
  0x0002,   /* U+25D8 */
  0x0002,   /* U+25D9 */
  0x0002,   /* U+25DA */
  0x0002,   /* U+25DB */
  0x0002,   /* U+25DC */
  0x0002,   /* U+25DD */
  0x0002,   /* U+25DE */
  0x0002,   /* U+25DF */
  0x0002,   /* U+25E0 */
  0x0002,   /* U+25E1 */
  0x0002,   /* U+25E2 */
  0x0002,   /* U+25E3 */
  0x0002,   /* U+25E4 */
  0x0002,   /* U+25E5 */
  0x0002,   /* U+25E6 */
  0x0002,   /* U+25E7 */
  0x0002,   /* U+25E8 */
  0x0002,   /* U+25E9 */
  0x0002,   /* U+25EA */
  0x0002,   /* U+25EB */
  0x0002,   /* U+25EC */
  0x0002,   /* U+25ED */
  0x0002,   /* U+25EE */
  0x0002,   /* U+25EF */
  0x0002,   /* U+25F0 */
  0x0002,   /* U+25F1 */
  0x0002,   /* U+25F2 */
  0x0002,   /* U+25F3 */
  0x0002,   /* U+25F4 */
  0x0002,   /* U+25F5 */
  0x0002,   /* U+25F6 */
  0x0002,   /* U+25F7 */
  0x0002,   /* U+25F8 */
  0x0002,   /* U+25F9 */
  0x0002,   /* U+25FA */
  0x0002,   /* U+25FB */
  0x0002,   /* U+25FC */
  0x0002,   /* U+25FD */
  0x0002,   /* U+25FE */
  0x0002,   /* U+25FF */
};

uint16 uca900_p026[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0918,   /* U+2600 */
  0x0919,   /* U+2601 */
  0x091A,   /* U+2602 */
  0x091B,   /* U+2603 */
  0x091C,   /* U+2604 */
  0x091D,   /* U+2605 */
  0x091E,   /* U+2606 */
  0x091F,   /* U+2607 */
  0x0920,   /* U+2608 */
  0x0921,   /* U+2609 */
  0x0922,   /* U+260A */
  0x0923,   /* U+260B */
  0x0924,   /* U+260C */
  0x0925,   /* U+260D */
  0x0926,   /* U+260E */
  0x0927,   /* U+260F */
  0x0928,   /* U+2610 */
  0x0929,   /* U+2611 */
  0x092A,   /* U+2612 */
  0x092B,   /* U+2613 */
  0x092C,   /* U+2614 */
  0x092D,   /* U+2615 */
  0x092E,   /* U+2616 */
  0x092F,   /* U+2617 */
  0x0930,   /* U+2618 */
  0x0931,   /* U+2619 */
  0x0932,   /* U+261A */
  0x0933,   /* U+261B */
  0x0934,   /* U+261C */
  0x0935,   /* U+261D */
  0x0936,   /* U+261E */
  0x0937,   /* U+261F */
  0x0938,   /* U+2620 */
  0x0939,   /* U+2621 */
  0x093A,   /* U+2622 */
  0x093B,   /* U+2623 */
  0x093C,   /* U+2624 */
  0x093D,   /* U+2625 */
  0x093E,   /* U+2626 */
  0x093F,   /* U+2627 */
  0x0940,   /* U+2628 */
  0x0941,   /* U+2629 */
  0x0942,   /* U+262A */
  0x0943,   /* U+262B */
  0x0944,   /* U+262C */
  0x0945,   /* U+262D */
  0x0946,   /* U+262E */
  0x0947,   /* U+262F */
  0x0EA2,   /* U+2630 */
  0x0EA3,   /* U+2631 */
  0x0EA4,   /* U+2632 */
  0x0EA5,   /* U+2633 */
  0x0EA6,   /* U+2634 */
  0x0EA7,   /* U+2635 */
  0x0EA8,   /* U+2636 */
  0x0EA9,   /* U+2637 */
  0x0948,   /* U+2638 */
  0x0949,   /* U+2639 */
  0x094A,   /* U+263A */
  0x094B,   /* U+263B */
  0x094C,   /* U+263C */
  0x094D,   /* U+263D */
  0x094E,   /* U+263E */
  0x094F,   /* U+263F */
  0x0950,   /* U+2640 */
  0x0951,   /* U+2641 */
  0x0952,   /* U+2642 */
  0x0953,   /* U+2643 */
  0x0954,   /* U+2644 */
  0x0955,   /* U+2645 */
  0x0956,   /* U+2646 */
  0x0957,   /* U+2647 */
  0x0958,   /* U+2648 */
  0x0959,   /* U+2649 */
  0x095A,   /* U+264A */
  0x095B,   /* U+264B */
  0x095C,   /* U+264C */
  0x095D,   /* U+264D */
  0x095E,   /* U+264E */
  0x095F,   /* U+264F */
  0x0960,   /* U+2650 */
  0x0961,   /* U+2651 */
  0x0962,   /* U+2652 */
  0x0963,   /* U+2653 */
  0x0964,   /* U+2654 */
  0x0965,   /* U+2655 */
  0x0966,   /* U+2656 */
  0x0967,   /* U+2657 */
  0x0968,   /* U+2658 */
  0x0969,   /* U+2659 */
  0x096A,   /* U+265A */
  0x096B,   /* U+265B */
  0x096C,   /* U+265C */
  0x096D,   /* U+265D */
  0x096E,   /* U+265E */
  0x096F,   /* U+265F */
  0x0970,   /* U+2660 */
  0x0971,   /* U+2661 */
  0x0972,   /* U+2662 */
  0x0973,   /* U+2663 */
  0x0974,   /* U+2664 */
  0x0975,   /* U+2665 */
  0x0976,   /* U+2666 */
  0x0977,   /* U+2667 */
  0x0978,   /* U+2668 */
  0x0979,   /* U+2669 */
  0x097A,   /* U+266A */
  0x097B,   /* U+266B */
  0x097C,   /* U+266C */
  0x10F3,   /* U+266D */
  0x10F4,   /* U+266E */
  0x10F5,   /* U+266F */
  0x097D,   /* U+2670 */
  0x097E,   /* U+2671 */
  0x097F,   /* U+2672 */
  0x0980,   /* U+2673 */
  0x0981,   /* U+2674 */
  0x0982,   /* U+2675 */
  0x0983,   /* U+2676 */
  0x0984,   /* U+2677 */
  0x0985,   /* U+2678 */
  0x0986,   /* U+2679 */
  0x0987,   /* U+267A */
  0x0988,   /* U+267B */
  0x0989,   /* U+267C */
  0x098A,   /* U+267D */
  0x098B,   /* U+267E */
  0x098C,   /* U+267F */
  0x098D,   /* U+2680 */
  0x098E,   /* U+2681 */
  0x098F,   /* U+2682 */
  0x0990,   /* U+2683 */
  0x0991,   /* U+2684 */
  0x0992,   /* U+2685 */
  0x0993,   /* U+2686 */
  0x0994,   /* U+2687 */
  0x0995,   /* U+2688 */
  0x0996,   /* U+2689 */
  0x0E9C,   /* U+268A */
  0x0E9D,   /* U+268B */
  0x0E9E,   /* U+268C */
  0x0E9F,   /* U+268D */
  0x0EA0,   /* U+268E */
  0x0EA1,   /* U+268F */
  0x0997,   /* U+2690 */
  0x0998,   /* U+2691 */
  0x0999,   /* U+2692 */
  0x099A,   /* U+2693 */
  0x099B,   /* U+2694 */
  0x099C,   /* U+2695 */
  0x099D,   /* U+2696 */
  0x099E,   /* U+2697 */
  0x099F,   /* U+2698 */
  0x09A0,   /* U+2699 */
  0x09A1,   /* U+269A */
  0x09A2,   /* U+269B */
  0x09A3,   /* U+269C */
  0x09A4,   /* U+269D */
  0x09A5,   /* U+269E */
  0x09A6,   /* U+269F */
  0x09A7,   /* U+26A0 */
  0x09A8,   /* U+26A1 */
  0x09A9,   /* U+26A2 */
  0x09AA,   /* U+26A3 */
  0x09AB,   /* U+26A4 */
  0x09AC,   /* U+26A5 */
  0x09AD,   /* U+26A6 */
  0x09AE,   /* U+26A7 */
  0x09AF,   /* U+26A8 */
  0x09B0,   /* U+26A9 */
  0x09B1,   /* U+26AA */
  0x09B2,   /* U+26AB */
  0x09B3,   /* U+26AC */
  0x09B4,   /* U+26AD */
  0x09B5,   /* U+26AE */
  0x09B6,   /* U+26AF */
  0x09B7,   /* U+26B0 */
  0x09B8,   /* U+26B1 */
  0x09B9,   /* U+26B2 */
  0x09BA,   /* U+26B3 */
  0x09BB,   /* U+26B4 */
  0x09BC,   /* U+26B5 */
  0x09BD,   /* U+26B6 */
  0x09BE,   /* U+26B7 */
  0x09BF,   /* U+26B8 */
  0x09C0,   /* U+26B9 */
  0x09C1,   /* U+26BA */
  0x09C2,   /* U+26BB */
  0x09C3,   /* U+26BC */
  0x09C4,   /* U+26BD */
  0x09C5,   /* U+26BE */
  0x09C6,   /* U+26BF */
  0x09C7,   /* U+26C0 */
  0x09C8,   /* U+26C1 */
  0x09C9,   /* U+26C2 */
  0x09CA,   /* U+26C3 */
  0x09CB,   /* U+26C4 */
  0x09CC,   /* U+26C5 */
  0x09CD,   /* U+26C6 */
  0x09CE,   /* U+26C7 */
  0x09CF,   /* U+26C8 */
  0x09D0,   /* U+26C9 */
  0x09D1,   /* U+26CA */
  0x09D2,   /* U+26CB */
  0x09D3,   /* U+26CC */
  0x09D4,   /* U+26CD */
  0x09D5,   /* U+26CE */
  0x09D6,   /* U+26CF */
  0x09D7,   /* U+26D0 */
  0x09D8,   /* U+26D1 */
  0x09D9,   /* U+26D2 */
  0x09DA,   /* U+26D3 */
  0x09DB,   /* U+26D4 */
  0x09DC,   /* U+26D5 */
  0x09DD,   /* U+26D6 */
  0x09DE,   /* U+26D7 */
  0x09DF,   /* U+26D8 */
  0x09E0,   /* U+26D9 */
  0x09E1,   /* U+26DA */
  0x09E2,   /* U+26DB */
  0x09E3,   /* U+26DC */
  0x09E4,   /* U+26DD */
  0x09E5,   /* U+26DE */
  0x09E6,   /* U+26DF */
  0x09E7,   /* U+26E0 */
  0x09E8,   /* U+26E1 */
  0x09E9,   /* U+26E2 */
  0x09EA,   /* U+26E3 */
  0x09EB,   /* U+26E4 */
  0x09EC,   /* U+26E5 */
  0x09ED,   /* U+26E6 */
  0x09EE,   /* U+26E7 */
  0x09EF,   /* U+26E8 */
  0x09F0,   /* U+26E9 */
  0x09F1,   /* U+26EA */
  0x09F2,   /* U+26EB */
  0x09F3,   /* U+26EC */
  0x09F4,   /* U+26ED */
  0x09F5,   /* U+26EE */
  0x09F6,   /* U+26EF */
  0x09F7,   /* U+26F0 */
  0x09F8,   /* U+26F1 */
  0x09F9,   /* U+26F2 */
  0x09FA,   /* U+26F3 */
  0x09FB,   /* U+26F4 */
  0x09FC,   /* U+26F5 */
  0x09FD,   /* U+26F6 */
  0x09FE,   /* U+26F7 */
  0x09FF,   /* U+26F8 */
  0x0A00,   /* U+26F9 */
  0x0A01,   /* U+26FA */
  0x0A02,   /* U+26FB */
  0x0A03,   /* U+26FC */
  0x0A04,   /* U+26FD */
  0x0A05,   /* U+26FE */
  0x0A06,   /* U+26FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2600 */
  0x0020,   /* U+2601 */
  0x0020,   /* U+2602 */
  0x0020,   /* U+2603 */
  0x0020,   /* U+2604 */
  0x0020,   /* U+2605 */
  0x0020,   /* U+2606 */
  0x0020,   /* U+2607 */
  0x0020,   /* U+2608 */
  0x0020,   /* U+2609 */
  0x0020,   /* U+260A */
  0x0020,   /* U+260B */
  0x0020,   /* U+260C */
  0x0020,   /* U+260D */
  0x0020,   /* U+260E */
  0x0020,   /* U+260F */
  0x0020,   /* U+2610 */
  0x0020,   /* U+2611 */
  0x0020,   /* U+2612 */
  0x0020,   /* U+2613 */
  0x0020,   /* U+2614 */
  0x0020,   /* U+2615 */
  0x0020,   /* U+2616 */
  0x0020,   /* U+2617 */
  0x0020,   /* U+2618 */
  0x0020,   /* U+2619 */
  0x0020,   /* U+261A */
  0x0020,   /* U+261B */
  0x0020,   /* U+261C */
  0x0020,   /* U+261D */
  0x0020,   /* U+261E */
  0x0020,   /* U+261F */
  0x0020,   /* U+2620 */
  0x0020,   /* U+2621 */
  0x0020,   /* U+2622 */
  0x0020,   /* U+2623 */
  0x0020,   /* U+2624 */
  0x0020,   /* U+2625 */
  0x0020,   /* U+2626 */
  0x0020,   /* U+2627 */
  0x0020,   /* U+2628 */
  0x0020,   /* U+2629 */
  0x0020,   /* U+262A */
  0x0020,   /* U+262B */
  0x0020,   /* U+262C */
  0x0020,   /* U+262D */
  0x0020,   /* U+262E */
  0x0020,   /* U+262F */
  0x0020,   /* U+2630 */
  0x0020,   /* U+2631 */
  0x0020,   /* U+2632 */
  0x0020,   /* U+2633 */
  0x0020,   /* U+2634 */
  0x0020,   /* U+2635 */
  0x0020,   /* U+2636 */
  0x0020,   /* U+2637 */
  0x0020,   /* U+2638 */
  0x0020,   /* U+2639 */
  0x0020,   /* U+263A */
  0x0020,   /* U+263B */
  0x0020,   /* U+263C */
  0x0020,   /* U+263D */
  0x0020,   /* U+263E */
  0x0020,   /* U+263F */
  0x0020,   /* U+2640 */
  0x0020,   /* U+2641 */
  0x0020,   /* U+2642 */
  0x0020,   /* U+2643 */
  0x0020,   /* U+2644 */
  0x0020,   /* U+2645 */
  0x0020,   /* U+2646 */
  0x0020,   /* U+2647 */
  0x0020,   /* U+2648 */
  0x0020,   /* U+2649 */
  0x0020,   /* U+264A */
  0x0020,   /* U+264B */
  0x0020,   /* U+264C */
  0x0020,   /* U+264D */
  0x0020,   /* U+264E */
  0x0020,   /* U+264F */
  0x0020,   /* U+2650 */
  0x0020,   /* U+2651 */
  0x0020,   /* U+2652 */
  0x0020,   /* U+2653 */
  0x0020,   /* U+2654 */
  0x0020,   /* U+2655 */
  0x0020,   /* U+2656 */
  0x0020,   /* U+2657 */
  0x0020,   /* U+2658 */
  0x0020,   /* U+2659 */
  0x0020,   /* U+265A */
  0x0020,   /* U+265B */
  0x0020,   /* U+265C */
  0x0020,   /* U+265D */
  0x0020,   /* U+265E */
  0x0020,   /* U+265F */
  0x0020,   /* U+2660 */
  0x0020,   /* U+2661 */
  0x0020,   /* U+2662 */
  0x0020,   /* U+2663 */
  0x0020,   /* U+2664 */
  0x0020,   /* U+2665 */
  0x0020,   /* U+2666 */
  0x0020,   /* U+2667 */
  0x0020,   /* U+2668 */
  0x0020,   /* U+2669 */
  0x0020,   /* U+266A */
  0x0020,   /* U+266B */
  0x0020,   /* U+266C */
  0x0020,   /* U+266D */
  0x0020,   /* U+266E */
  0x0020,   /* U+266F */
  0x0020,   /* U+2670 */
  0x0020,   /* U+2671 */
  0x0020,   /* U+2672 */
  0x0020,   /* U+2673 */
  0x0020,   /* U+2674 */
  0x0020,   /* U+2675 */
  0x0020,   /* U+2676 */
  0x0020,   /* U+2677 */
  0x0020,   /* U+2678 */
  0x0020,   /* U+2679 */
  0x0020,   /* U+267A */
  0x0020,   /* U+267B */
  0x0020,   /* U+267C */
  0x0020,   /* U+267D */
  0x0020,   /* U+267E */
  0x0020,   /* U+267F */
  0x0020,   /* U+2680 */
  0x0020,   /* U+2681 */
  0x0020,   /* U+2682 */
  0x0020,   /* U+2683 */
  0x0020,   /* U+2684 */
  0x0020,   /* U+2685 */
  0x0020,   /* U+2686 */
  0x0020,   /* U+2687 */
  0x0020,   /* U+2688 */
  0x0020,   /* U+2689 */
  0x0020,   /* U+268A */
  0x0020,   /* U+268B */
  0x0020,   /* U+268C */
  0x0020,   /* U+268D */
  0x0020,   /* U+268E */
  0x0020,   /* U+268F */
  0x0020,   /* U+2690 */
  0x0020,   /* U+2691 */
  0x0020,   /* U+2692 */
  0x0020,   /* U+2693 */
  0x0020,   /* U+2694 */
  0x0020,   /* U+2695 */
  0x0020,   /* U+2696 */
  0x0020,   /* U+2697 */
  0x0020,   /* U+2698 */
  0x0020,   /* U+2699 */
  0x0020,   /* U+269A */
  0x0020,   /* U+269B */
  0x0020,   /* U+269C */
  0x0020,   /* U+269D */
  0x0020,   /* U+269E */
  0x0020,   /* U+269F */
  0x0020,   /* U+26A0 */
  0x0020,   /* U+26A1 */
  0x0020,   /* U+26A2 */
  0x0020,   /* U+26A3 */
  0x0020,   /* U+26A4 */
  0x0020,   /* U+26A5 */
  0x0020,   /* U+26A6 */
  0x0020,   /* U+26A7 */
  0x0020,   /* U+26A8 */
  0x0020,   /* U+26A9 */
  0x0020,   /* U+26AA */
  0x0020,   /* U+26AB */
  0x0020,   /* U+26AC */
  0x0020,   /* U+26AD */
  0x0020,   /* U+26AE */
  0x0020,   /* U+26AF */
  0x0020,   /* U+26B0 */
  0x0020,   /* U+26B1 */
  0x0020,   /* U+26B2 */
  0x0020,   /* U+26B3 */
  0x0020,   /* U+26B4 */
  0x0020,   /* U+26B5 */
  0x0020,   /* U+26B6 */
  0x0020,   /* U+26B7 */
  0x0020,   /* U+26B8 */
  0x0020,   /* U+26B9 */
  0x0020,   /* U+26BA */
  0x0020,   /* U+26BB */
  0x0020,   /* U+26BC */
  0x0020,   /* U+26BD */
  0x0020,   /* U+26BE */
  0x0020,   /* U+26BF */
  0x0020,   /* U+26C0 */
  0x0020,   /* U+26C1 */
  0x0020,   /* U+26C2 */
  0x0020,   /* U+26C3 */
  0x0020,   /* U+26C4 */
  0x0020,   /* U+26C5 */
  0x0020,   /* U+26C6 */
  0x0020,   /* U+26C7 */
  0x0020,   /* U+26C8 */
  0x0020,   /* U+26C9 */
  0x0020,   /* U+26CA */
  0x0020,   /* U+26CB */
  0x0020,   /* U+26CC */
  0x0020,   /* U+26CD */
  0x0020,   /* U+26CE */
  0x0020,   /* U+26CF */
  0x0020,   /* U+26D0 */
  0x0020,   /* U+26D1 */
  0x0020,   /* U+26D2 */
  0x0020,   /* U+26D3 */
  0x0020,   /* U+26D4 */
  0x0020,   /* U+26D5 */
  0x0020,   /* U+26D6 */
  0x0020,   /* U+26D7 */
  0x0020,   /* U+26D8 */
  0x0020,   /* U+26D9 */
  0x0020,   /* U+26DA */
  0x0020,   /* U+26DB */
  0x0020,   /* U+26DC */
  0x0020,   /* U+26DD */
  0x0020,   /* U+26DE */
  0x0020,   /* U+26DF */
  0x0020,   /* U+26E0 */
  0x0020,   /* U+26E1 */
  0x0020,   /* U+26E2 */
  0x0020,   /* U+26E3 */
  0x0020,   /* U+26E4 */
  0x0020,   /* U+26E5 */
  0x0020,   /* U+26E6 */
  0x0020,   /* U+26E7 */
  0x0020,   /* U+26E8 */
  0x0020,   /* U+26E9 */
  0x0020,   /* U+26EA */
  0x0020,   /* U+26EB */
  0x0020,   /* U+26EC */
  0x0020,   /* U+26ED */
  0x0020,   /* U+26EE */
  0x0020,   /* U+26EF */
  0x0020,   /* U+26F0 */
  0x0020,   /* U+26F1 */
  0x0020,   /* U+26F2 */
  0x0020,   /* U+26F3 */
  0x0020,   /* U+26F4 */
  0x0020,   /* U+26F5 */
  0x0020,   /* U+26F6 */
  0x0020,   /* U+26F7 */
  0x0020,   /* U+26F8 */
  0x0020,   /* U+26F9 */
  0x0020,   /* U+26FA */
  0x0020,   /* U+26FB */
  0x0020,   /* U+26FC */
  0x0020,   /* U+26FD */
  0x0020,   /* U+26FE */
  0x0020,   /* U+26FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2600 */
  0x0002,   /* U+2601 */
  0x0002,   /* U+2602 */
  0x0002,   /* U+2603 */
  0x0002,   /* U+2604 */
  0x0002,   /* U+2605 */
  0x0002,   /* U+2606 */
  0x0002,   /* U+2607 */
  0x0002,   /* U+2608 */
  0x0002,   /* U+2609 */
  0x0002,   /* U+260A */
  0x0002,   /* U+260B */
  0x0002,   /* U+260C */
  0x0002,   /* U+260D */
  0x0002,   /* U+260E */
  0x0002,   /* U+260F */
  0x0002,   /* U+2610 */
  0x0002,   /* U+2611 */
  0x0002,   /* U+2612 */
  0x0002,   /* U+2613 */
  0x0002,   /* U+2614 */
  0x0002,   /* U+2615 */
  0x0002,   /* U+2616 */
  0x0002,   /* U+2617 */
  0x0002,   /* U+2618 */
  0x0002,   /* U+2619 */
  0x0002,   /* U+261A */
  0x0002,   /* U+261B */
  0x0002,   /* U+261C */
  0x0002,   /* U+261D */
  0x0002,   /* U+261E */
  0x0002,   /* U+261F */
  0x0002,   /* U+2620 */
  0x0002,   /* U+2621 */
  0x0002,   /* U+2622 */
  0x0002,   /* U+2623 */
  0x0002,   /* U+2624 */
  0x0002,   /* U+2625 */
  0x0002,   /* U+2626 */
  0x0002,   /* U+2627 */
  0x0002,   /* U+2628 */
  0x0002,   /* U+2629 */
  0x0002,   /* U+262A */
  0x0002,   /* U+262B */
  0x0002,   /* U+262C */
  0x0002,   /* U+262D */
  0x0002,   /* U+262E */
  0x0002,   /* U+262F */
  0x0002,   /* U+2630 */
  0x0002,   /* U+2631 */
  0x0002,   /* U+2632 */
  0x0002,   /* U+2633 */
  0x0002,   /* U+2634 */
  0x0002,   /* U+2635 */
  0x0002,   /* U+2636 */
  0x0002,   /* U+2637 */
  0x0002,   /* U+2638 */
  0x0002,   /* U+2639 */
  0x0002,   /* U+263A */
  0x0002,   /* U+263B */
  0x0002,   /* U+263C */
  0x0002,   /* U+263D */
  0x0002,   /* U+263E */
  0x0002,   /* U+263F */
  0x0002,   /* U+2640 */
  0x0002,   /* U+2641 */
  0x0002,   /* U+2642 */
  0x0002,   /* U+2643 */
  0x0002,   /* U+2644 */
  0x0002,   /* U+2645 */
  0x0002,   /* U+2646 */
  0x0002,   /* U+2647 */
  0x0002,   /* U+2648 */
  0x0002,   /* U+2649 */
  0x0002,   /* U+264A */
  0x0002,   /* U+264B */
  0x0002,   /* U+264C */
  0x0002,   /* U+264D */
  0x0002,   /* U+264E */
  0x0002,   /* U+264F */
  0x0002,   /* U+2650 */
  0x0002,   /* U+2651 */
  0x0002,   /* U+2652 */
  0x0002,   /* U+2653 */
  0x0002,   /* U+2654 */
  0x0002,   /* U+2655 */
  0x0002,   /* U+2656 */
  0x0002,   /* U+2657 */
  0x0002,   /* U+2658 */
  0x0002,   /* U+2659 */
  0x0002,   /* U+265A */
  0x0002,   /* U+265B */
  0x0002,   /* U+265C */
  0x0002,   /* U+265D */
  0x0002,   /* U+265E */
  0x0002,   /* U+265F */
  0x0002,   /* U+2660 */
  0x0002,   /* U+2661 */
  0x0002,   /* U+2662 */
  0x0002,   /* U+2663 */
  0x0002,   /* U+2664 */
  0x0002,   /* U+2665 */
  0x0002,   /* U+2666 */
  0x0002,   /* U+2667 */
  0x0002,   /* U+2668 */
  0x0002,   /* U+2669 */
  0x0002,   /* U+266A */
  0x0002,   /* U+266B */
  0x0002,   /* U+266C */
  0x0002,   /* U+266D */
  0x0002,   /* U+266E */
  0x0002,   /* U+266F */
  0x0002,   /* U+2670 */
  0x0002,   /* U+2671 */
  0x0002,   /* U+2672 */
  0x0002,   /* U+2673 */
  0x0002,   /* U+2674 */
  0x0002,   /* U+2675 */
  0x0002,   /* U+2676 */
  0x0002,   /* U+2677 */
  0x0002,   /* U+2678 */
  0x0002,   /* U+2679 */
  0x0002,   /* U+267A */
  0x0002,   /* U+267B */
  0x0002,   /* U+267C */
  0x0002,   /* U+267D */
  0x0002,   /* U+267E */
  0x0002,   /* U+267F */
  0x0002,   /* U+2680 */
  0x0002,   /* U+2681 */
  0x0002,   /* U+2682 */
  0x0002,   /* U+2683 */
  0x0002,   /* U+2684 */
  0x0002,   /* U+2685 */
  0x0002,   /* U+2686 */
  0x0002,   /* U+2687 */
  0x0002,   /* U+2688 */
  0x0002,   /* U+2689 */
  0x0002,   /* U+268A */
  0x0002,   /* U+268B */
  0x0002,   /* U+268C */
  0x0002,   /* U+268D */
  0x0002,   /* U+268E */
  0x0002,   /* U+268F */
  0x0002,   /* U+2690 */
  0x0002,   /* U+2691 */
  0x0002,   /* U+2692 */
  0x0002,   /* U+2693 */
  0x0002,   /* U+2694 */
  0x0002,   /* U+2695 */
  0x0002,   /* U+2696 */
  0x0002,   /* U+2697 */
  0x0002,   /* U+2698 */
  0x0002,   /* U+2699 */
  0x0002,   /* U+269A */
  0x0002,   /* U+269B */
  0x0002,   /* U+269C */
  0x0002,   /* U+269D */
  0x0002,   /* U+269E */
  0x0002,   /* U+269F */
  0x0002,   /* U+26A0 */
  0x0002,   /* U+26A1 */
  0x0002,   /* U+26A2 */
  0x0002,   /* U+26A3 */
  0x0002,   /* U+26A4 */
  0x0002,   /* U+26A5 */
  0x0002,   /* U+26A6 */
  0x0002,   /* U+26A7 */
  0x0002,   /* U+26A8 */
  0x0002,   /* U+26A9 */
  0x0002,   /* U+26AA */
  0x0002,   /* U+26AB */
  0x0002,   /* U+26AC */
  0x0002,   /* U+26AD */
  0x0002,   /* U+26AE */
  0x0002,   /* U+26AF */
  0x0002,   /* U+26B0 */
  0x0002,   /* U+26B1 */
  0x0002,   /* U+26B2 */
  0x0002,   /* U+26B3 */
  0x0002,   /* U+26B4 */
  0x0002,   /* U+26B5 */
  0x0002,   /* U+26B6 */
  0x0002,   /* U+26B7 */
  0x0002,   /* U+26B8 */
  0x0002,   /* U+26B9 */
  0x0002,   /* U+26BA */
  0x0002,   /* U+26BB */
  0x0002,   /* U+26BC */
  0x0002,   /* U+26BD */
  0x0002,   /* U+26BE */
  0x0002,   /* U+26BF */
  0x0002,   /* U+26C0 */
  0x0002,   /* U+26C1 */
  0x0002,   /* U+26C2 */
  0x0002,   /* U+26C3 */
  0x0002,   /* U+26C4 */
  0x0002,   /* U+26C5 */
  0x0002,   /* U+26C6 */
  0x0002,   /* U+26C7 */
  0x0002,   /* U+26C8 */
  0x0002,   /* U+26C9 */
  0x0002,   /* U+26CA */
  0x0002,   /* U+26CB */
  0x0002,   /* U+26CC */
  0x0002,   /* U+26CD */
  0x0002,   /* U+26CE */
  0x0002,   /* U+26CF */
  0x0002,   /* U+26D0 */
  0x0002,   /* U+26D1 */
  0x0002,   /* U+26D2 */
  0x0002,   /* U+26D3 */
  0x0002,   /* U+26D4 */
  0x0002,   /* U+26D5 */
  0x0002,   /* U+26D6 */
  0x0002,   /* U+26D7 */
  0x0002,   /* U+26D8 */
  0x0002,   /* U+26D9 */
  0x0002,   /* U+26DA */
  0x0002,   /* U+26DB */
  0x0002,   /* U+26DC */
  0x0002,   /* U+26DD */
  0x0002,   /* U+26DE */
  0x0002,   /* U+26DF */
  0x0002,   /* U+26E0 */
  0x0002,   /* U+26E1 */
  0x0002,   /* U+26E2 */
  0x0002,   /* U+26E3 */
  0x0002,   /* U+26E4 */
  0x0002,   /* U+26E5 */
  0x0002,   /* U+26E6 */
  0x0002,   /* U+26E7 */
  0x0002,   /* U+26E8 */
  0x0002,   /* U+26E9 */
  0x0002,   /* U+26EA */
  0x0002,   /* U+26EB */
  0x0002,   /* U+26EC */
  0x0002,   /* U+26ED */
  0x0002,   /* U+26EE */
  0x0002,   /* U+26EF */
  0x0002,   /* U+26F0 */
  0x0002,   /* U+26F1 */
  0x0002,   /* U+26F2 */
  0x0002,   /* U+26F3 */
  0x0002,   /* U+26F4 */
  0x0002,   /* U+26F5 */
  0x0002,   /* U+26F6 */
  0x0002,   /* U+26F7 */
  0x0002,   /* U+26F8 */
  0x0002,   /* U+26F9 */
  0x0002,   /* U+26FA */
  0x0002,   /* U+26FB */
  0x0002,   /* U+26FC */
  0x0002,   /* U+26FD */
  0x0002,   /* U+26FE */
  0x0002,   /* U+26FF */
};

uint16 uca900_p027[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0A21,   /* U+2700 */
  0x0A22,   /* U+2701 */
  0x0A23,   /* U+2702 */
  0x0A24,   /* U+2703 */
  0x0A25,   /* U+2704 */
  0x0A26,   /* U+2705 */
  0x0A27,   /* U+2706 */
  0x0A28,   /* U+2707 */
  0x0A29,   /* U+2708 */
  0x0A2A,   /* U+2709 */
  0x0A2B,   /* U+270A */
  0x0A2C,   /* U+270B */
  0x0A2D,   /* U+270C */
  0x0A2E,   /* U+270D */
  0x0A2F,   /* U+270E */
  0x0A30,   /* U+270F */
  0x0A31,   /* U+2710 */
  0x0A32,   /* U+2711 */
  0x0A33,   /* U+2712 */
  0x0A34,   /* U+2713 */
  0x0A35,   /* U+2714 */
  0x0A36,   /* U+2715 */
  0x0A37,   /* U+2716 */
  0x0A38,   /* U+2717 */
  0x0A39,   /* U+2718 */
  0x0A3A,   /* U+2719 */
  0x0A3B,   /* U+271A */
  0x0A3C,   /* U+271B */
  0x0A3D,   /* U+271C */
  0x0A3E,   /* U+271D */
  0x0A3F,   /* U+271E */
  0x0A40,   /* U+271F */
  0x0A41,   /* U+2720 */
  0x0A42,   /* U+2721 */
  0x0A43,   /* U+2722 */
  0x0A44,   /* U+2723 */
  0x0A45,   /* U+2724 */
  0x0A46,   /* U+2725 */
  0x0A47,   /* U+2726 */
  0x0A48,   /* U+2727 */
  0x0A49,   /* U+2728 */
  0x0A4A,   /* U+2729 */
  0x0A4B,   /* U+272A */
  0x0A4C,   /* U+272B */
  0x0A4D,   /* U+272C */
  0x0A4E,   /* U+272D */
  0x0A4F,   /* U+272E */
  0x0A50,   /* U+272F */
  0x0A51,   /* U+2730 */
  0x0A52,   /* U+2731 */
  0x0A53,   /* U+2732 */
  0x0A54,   /* U+2733 */
  0x0A55,   /* U+2734 */
  0x0A56,   /* U+2735 */
  0x0A57,   /* U+2736 */
  0x0A58,   /* U+2737 */
  0x0A59,   /* U+2738 */
  0x0A5A,   /* U+2739 */
  0x0A5B,   /* U+273A */
  0x0A5C,   /* U+273B */
  0x0A5D,   /* U+273C */
  0x0A5E,   /* U+273D */
  0x0A5F,   /* U+273E */
  0x0A60,   /* U+273F */
  0x0A61,   /* U+2740 */
  0x0A62,   /* U+2741 */
  0x0A63,   /* U+2742 */
  0x0A64,   /* U+2743 */
  0x0A65,   /* U+2744 */
  0x0A66,   /* U+2745 */
  0x0A67,   /* U+2746 */
  0x0A68,   /* U+2747 */
  0x0A69,   /* U+2748 */
  0x0A6A,   /* U+2749 */
  0x0A6B,   /* U+274A */
  0x0A6C,   /* U+274B */
  0x0A6D,   /* U+274C */
  0x0A6E,   /* U+274D */
  0x0A6F,   /* U+274E */
  0x0A70,   /* U+274F */
  0x0A71,   /* U+2750 */
  0x0A72,   /* U+2751 */
  0x0A73,   /* U+2752 */
  0x0A74,   /* U+2753 */
  0x0A75,   /* U+2754 */
  0x0A76,   /* U+2755 */
  0x0A77,   /* U+2756 */
  0x0A78,   /* U+2757 */
  0x0A79,   /* U+2758 */
  0x0A7A,   /* U+2759 */
  0x0A7B,   /* U+275A */
  0x0A7C,   /* U+275B */
  0x0A7D,   /* U+275C */
  0x0A7E,   /* U+275D */
  0x0A7F,   /* U+275E */
  0x0A80,   /* U+275F */
  0x0A81,   /* U+2760 */
  0x0A82,   /* U+2761 */
  0x0A83,   /* U+2762 */
  0x0A84,   /* U+2763 */
  0x0A85,   /* U+2764 */
  0x0A86,   /* U+2765 */
  0x0A87,   /* U+2766 */
  0x0A88,   /* U+2767 */
  0x034D,   /* U+2768 */
  0x034E,   /* U+2769 */
  0x034F,   /* U+276A */
  0x0350,   /* U+276B */
  0x0351,   /* U+276C */
  0x0352,   /* U+276D */
  0x0353,   /* U+276E */
  0x0354,   /* U+276F */
  0x0355,   /* U+2770 */
  0x0356,   /* U+2771 */
  0x0357,   /* U+2772 */
  0x0358,   /* U+2773 */
  0x0359,   /* U+2774 */
  0x035A,   /* U+2775 */
  0x1C3E,   /* U+2776 */
  0x1C3F,   /* U+2777 */
  0x1C40,   /* U+2778 */
  0x1C41,   /* U+2779 */
  0x1C42,   /* U+277A */
  0x1C43,   /* U+277B */
  0x1C44,   /* U+277C */
  0x1C45,   /* U+277D */
  0x1C46,   /* U+277E */
  0x1C3E,   /* U+277F */
  0x1C3E,   /* U+2780 */
  0x1C3F,   /* U+2781 */
  0x1C40,   /* U+2782 */
  0x1C41,   /* U+2783 */
  0x1C42,   /* U+2784 */
  0x1C43,   /* U+2785 */
  0x1C44,   /* U+2786 */
  0x1C45,   /* U+2787 */
  0x1C46,   /* U+2788 */
  0x1C3E,   /* U+2789 */
  0x1C3E,   /* U+278A */
  0x1C3F,   /* U+278B */
  0x1C40,   /* U+278C */
  0x1C41,   /* U+278D */
  0x1C42,   /* U+278E */
  0x1C43,   /* U+278F */
  0x1C44,   /* U+2790 */
  0x1C45,   /* U+2791 */
  0x1C46,   /* U+2792 */
  0x1C3E,   /* U+2793 */
  0x0A89,   /* U+2794 */
  0x0A8A,   /* U+2795 */
  0x0A8B,   /* U+2796 */
  0x0A8C,   /* U+2797 */
  0x0A8D,   /* U+2798 */
  0x0A8E,   /* U+2799 */
  0x0A8F,   /* U+279A */
  0x0A90,   /* U+279B */
  0x0A91,   /* U+279C */
  0x0A92,   /* U+279D */
  0x0A93,   /* U+279E */
  0x0A94,   /* U+279F */
  0x0A95,   /* U+27A0 */
  0x0A96,   /* U+27A1 */
  0x0A97,   /* U+27A2 */
  0x0A98,   /* U+27A3 */
  0x0A99,   /* U+27A4 */
  0x0A9A,   /* U+27A5 */
  0x0A9B,   /* U+27A6 */
  0x0A9C,   /* U+27A7 */
  0x0A9D,   /* U+27A8 */
  0x0A9E,   /* U+27A9 */
  0x0A9F,   /* U+27AA */
  0x0AA0,   /* U+27AB */
  0x0AA1,   /* U+27AC */
  0x0AA2,   /* U+27AD */
  0x0AA3,   /* U+27AE */
  0x0AA4,   /* U+27AF */
  0x0AA5,   /* U+27B0 */
  0x0AA6,   /* U+27B1 */
  0x0AA7,   /* U+27B2 */
  0x0AA8,   /* U+27B3 */
  0x0AA9,   /* U+27B4 */
  0x0AAA,   /* U+27B5 */
  0x0AAB,   /* U+27B6 */
  0x0AAC,   /* U+27B7 */
  0x0AAD,   /* U+27B8 */
  0x0AAE,   /* U+27B9 */
  0x0AAF,   /* U+27BA */
  0x0AB0,   /* U+27BB */
  0x0AB1,   /* U+27BC */
  0x0AB2,   /* U+27BD */
  0x0AB3,   /* U+27BE */
  0x0AB4,   /* U+27BF */
  0x0AB5,   /* U+27C0 */
  0x0AB6,   /* U+27C1 */
  0x0AB7,   /* U+27C2 */
  0x0AB8,   /* U+27C3 */
  0x0AB9,   /* U+27C4 */
  0x0341,   /* U+27C5 */
  0x0342,   /* U+27C6 */
  0x0ABA,   /* U+27C7 */
  0x0ABB,   /* U+27C8 */
  0x0ABC,   /* U+27C9 */
  0x0ABD,   /* U+27CA */
  0x0ABE,   /* U+27CB */
  0x0ABF,   /* U+27CC */
  0x0AC0,   /* U+27CD */
  0x0AC1,   /* U+27CE */
  0x0AC2,   /* U+27CF */
  0x0AC3,   /* U+27D0 */
  0x0AC4,   /* U+27D1 */
  0x0AC5,   /* U+27D2 */
  0x0AC6,   /* U+27D3 */
  0x0AC7,   /* U+27D4 */
  0x0AC8,   /* U+27D5 */
  0x0AC9,   /* U+27D6 */
  0x0ACA,   /* U+27D7 */
  0x0ACB,   /* U+27D8 */
  0x0ACC,   /* U+27D9 */
  0x0ACD,   /* U+27DA */
  0x0ACE,   /* U+27DB */
  0x0ACF,   /* U+27DC */
  0x0AD0,   /* U+27DD */
  0x0AD1,   /* U+27DE */
  0x0AD2,   /* U+27DF */
  0x0AD3,   /* U+27E0 */
  0x0AD4,   /* U+27E1 */
  0x0AD5,   /* U+27E2 */
  0x0AD6,   /* U+27E3 */
  0x0AD7,   /* U+27E4 */
  0x0AD8,   /* U+27E5 */
  0x0343,   /* U+27E6 */
  0x0344,   /* U+27E7 */
  0x0345,   /* U+27E8 */
  0x0346,   /* U+27E9 */
  0x0347,   /* U+27EA */
  0x0348,   /* U+27EB */
  0x0349,   /* U+27EC */
  0x034A,   /* U+27ED */
  0x034B,   /* U+27EE */
  0x034C,   /* U+27EF */
  0x0AD9,   /* U+27F0 */
  0x0ADA,   /* U+27F1 */
  0x0ADB,   /* U+27F2 */
  0x0ADC,   /* U+27F3 */
  0x0ADD,   /* U+27F4 */
  0x0ADE,   /* U+27F5 */
  0x0ADF,   /* U+27F6 */
  0x0AE0,   /* U+27F7 */
  0x0AE1,   /* U+27F8 */
  0x0AE2,   /* U+27F9 */
  0x0AE3,   /* U+27FA */
  0x0AE4,   /* U+27FB */
  0x0AE5,   /* U+27FC */
  0x0AE6,   /* U+27FD */
  0x0AE7,   /* U+27FE */
  0x0AE8,   /* U+27FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2700 */
  0x0020,   /* U+2701 */
  0x0020,   /* U+2702 */
  0x0020,   /* U+2703 */
  0x0020,   /* U+2704 */
  0x0020,   /* U+2705 */
  0x0020,   /* U+2706 */
  0x0020,   /* U+2707 */
  0x0020,   /* U+2708 */
  0x0020,   /* U+2709 */
  0x0020,   /* U+270A */
  0x0020,   /* U+270B */
  0x0020,   /* U+270C */
  0x0020,   /* U+270D */
  0x0020,   /* U+270E */
  0x0020,   /* U+270F */
  0x0020,   /* U+2710 */
  0x0020,   /* U+2711 */
  0x0020,   /* U+2712 */
  0x0020,   /* U+2713 */
  0x0020,   /* U+2714 */
  0x0020,   /* U+2715 */
  0x0020,   /* U+2716 */
  0x0020,   /* U+2717 */
  0x0020,   /* U+2718 */
  0x0020,   /* U+2719 */
  0x0020,   /* U+271A */
  0x0020,   /* U+271B */
  0x0020,   /* U+271C */
  0x0020,   /* U+271D */
  0x0020,   /* U+271E */
  0x0020,   /* U+271F */
  0x0020,   /* U+2720 */
  0x0020,   /* U+2721 */
  0x0020,   /* U+2722 */
  0x0020,   /* U+2723 */
  0x0020,   /* U+2724 */
  0x0020,   /* U+2725 */
  0x0020,   /* U+2726 */
  0x0020,   /* U+2727 */
  0x0020,   /* U+2728 */
  0x0020,   /* U+2729 */
  0x0020,   /* U+272A */
  0x0020,   /* U+272B */
  0x0020,   /* U+272C */
  0x0020,   /* U+272D */
  0x0020,   /* U+272E */
  0x0020,   /* U+272F */
  0x0020,   /* U+2730 */
  0x0020,   /* U+2731 */
  0x0020,   /* U+2732 */
  0x0020,   /* U+2733 */
  0x0020,   /* U+2734 */
  0x0020,   /* U+2735 */
  0x0020,   /* U+2736 */
  0x0020,   /* U+2737 */
  0x0020,   /* U+2738 */
  0x0020,   /* U+2739 */
  0x0020,   /* U+273A */
  0x0020,   /* U+273B */
  0x0020,   /* U+273C */
  0x0020,   /* U+273D */
  0x0020,   /* U+273E */
  0x0020,   /* U+273F */
  0x0020,   /* U+2740 */
  0x0020,   /* U+2741 */
  0x0020,   /* U+2742 */
  0x0020,   /* U+2743 */
  0x0020,   /* U+2744 */
  0x0020,   /* U+2745 */
  0x0020,   /* U+2746 */
  0x0020,   /* U+2747 */
  0x0020,   /* U+2748 */
  0x0020,   /* U+2749 */
  0x0020,   /* U+274A */
  0x0020,   /* U+274B */
  0x0020,   /* U+274C */
  0x0020,   /* U+274D */
  0x0020,   /* U+274E */
  0x0020,   /* U+274F */
  0x0020,   /* U+2750 */
  0x0020,   /* U+2751 */
  0x0020,   /* U+2752 */
  0x0020,   /* U+2753 */
  0x0020,   /* U+2754 */
  0x0020,   /* U+2755 */
  0x0020,   /* U+2756 */
  0x0020,   /* U+2757 */
  0x0020,   /* U+2758 */
  0x0020,   /* U+2759 */
  0x0020,   /* U+275A */
  0x0020,   /* U+275B */
  0x0020,   /* U+275C */
  0x0020,   /* U+275D */
  0x0020,   /* U+275E */
  0x0020,   /* U+275F */
  0x0020,   /* U+2760 */
  0x0020,   /* U+2761 */
  0x0020,   /* U+2762 */
  0x0020,   /* U+2763 */
  0x0020,   /* U+2764 */
  0x0020,   /* U+2765 */
  0x0020,   /* U+2766 */
  0x0020,   /* U+2767 */
  0x0020,   /* U+2768 */
  0x0020,   /* U+2769 */
  0x0020,   /* U+276A */
  0x0020,   /* U+276B */
  0x0020,   /* U+276C */
  0x0020,   /* U+276D */
  0x0020,   /* U+276E */
  0x0020,   /* U+276F */
  0x0020,   /* U+2770 */
  0x0020,   /* U+2771 */
  0x0020,   /* U+2772 */
  0x0020,   /* U+2773 */
  0x0020,   /* U+2774 */
  0x0020,   /* U+2775 */
  0x0020,   /* U+2776 */
  0x0020,   /* U+2777 */
  0x0020,   /* U+2778 */
  0x0020,   /* U+2779 */
  0x0020,   /* U+277A */
  0x0020,   /* U+277B */
  0x0020,   /* U+277C */
  0x0020,   /* U+277D */
  0x0020,   /* U+277E */
  0x0020,   /* U+277F */
  0x0020,   /* U+2780 */
  0x0020,   /* U+2781 */
  0x0020,   /* U+2782 */
  0x0020,   /* U+2783 */
  0x0020,   /* U+2784 */
  0x0020,   /* U+2785 */
  0x0020,   /* U+2786 */
  0x0020,   /* U+2787 */
  0x0020,   /* U+2788 */
  0x0020,   /* U+2789 */
  0x0020,   /* U+278A */
  0x0020,   /* U+278B */
  0x0020,   /* U+278C */
  0x0020,   /* U+278D */
  0x0020,   /* U+278E */
  0x0020,   /* U+278F */
  0x0020,   /* U+2790 */
  0x0020,   /* U+2791 */
  0x0020,   /* U+2792 */
  0x0020,   /* U+2793 */
  0x0020,   /* U+2794 */
  0x0020,   /* U+2795 */
  0x0020,   /* U+2796 */
  0x0020,   /* U+2797 */
  0x0020,   /* U+2798 */
  0x0020,   /* U+2799 */
  0x0020,   /* U+279A */
  0x0020,   /* U+279B */
  0x0020,   /* U+279C */
  0x0020,   /* U+279D */
  0x0020,   /* U+279E */
  0x0020,   /* U+279F */
  0x0020,   /* U+27A0 */
  0x0020,   /* U+27A1 */
  0x0020,   /* U+27A2 */
  0x0020,   /* U+27A3 */
  0x0020,   /* U+27A4 */
  0x0020,   /* U+27A5 */
  0x0020,   /* U+27A6 */
  0x0020,   /* U+27A7 */
  0x0020,   /* U+27A8 */
  0x0020,   /* U+27A9 */
  0x0020,   /* U+27AA */
  0x0020,   /* U+27AB */
  0x0020,   /* U+27AC */
  0x0020,   /* U+27AD */
  0x0020,   /* U+27AE */
  0x0020,   /* U+27AF */
  0x0020,   /* U+27B0 */
  0x0020,   /* U+27B1 */
  0x0020,   /* U+27B2 */
  0x0020,   /* U+27B3 */
  0x0020,   /* U+27B4 */
  0x0020,   /* U+27B5 */
  0x0020,   /* U+27B6 */
  0x0020,   /* U+27B7 */
  0x0020,   /* U+27B8 */
  0x0020,   /* U+27B9 */
  0x0020,   /* U+27BA */
  0x0020,   /* U+27BB */
  0x0020,   /* U+27BC */
  0x0020,   /* U+27BD */
  0x0020,   /* U+27BE */
  0x0020,   /* U+27BF */
  0x0020,   /* U+27C0 */
  0x0020,   /* U+27C1 */
  0x0020,   /* U+27C2 */
  0x0020,   /* U+27C3 */
  0x0020,   /* U+27C4 */
  0x0020,   /* U+27C5 */
  0x0020,   /* U+27C6 */
  0x0020,   /* U+27C7 */
  0x0020,   /* U+27C8 */
  0x0020,   /* U+27C9 */
  0x0020,   /* U+27CA */
  0x0020,   /* U+27CB */
  0x0020,   /* U+27CC */
  0x0020,   /* U+27CD */
  0x0020,   /* U+27CE */
  0x0020,   /* U+27CF */
  0x0020,   /* U+27D0 */
  0x0020,   /* U+27D1 */
  0x0020,   /* U+27D2 */
  0x0020,   /* U+27D3 */
  0x0020,   /* U+27D4 */
  0x0020,   /* U+27D5 */
  0x0020,   /* U+27D6 */
  0x0020,   /* U+27D7 */
  0x0020,   /* U+27D8 */
  0x0020,   /* U+27D9 */
  0x0020,   /* U+27DA */
  0x0020,   /* U+27DB */
  0x0020,   /* U+27DC */
  0x0020,   /* U+27DD */
  0x0020,   /* U+27DE */
  0x0020,   /* U+27DF */
  0x0020,   /* U+27E0 */
  0x0020,   /* U+27E1 */
  0x0020,   /* U+27E2 */
  0x0020,   /* U+27E3 */
  0x0020,   /* U+27E4 */
  0x0020,   /* U+27E5 */
  0x0020,   /* U+27E6 */
  0x0020,   /* U+27E7 */
  0x0020,   /* U+27E8 */
  0x0020,   /* U+27E9 */
  0x0020,   /* U+27EA */
  0x0020,   /* U+27EB */
  0x0020,   /* U+27EC */
  0x0020,   /* U+27ED */
  0x0020,   /* U+27EE */
  0x0020,   /* U+27EF */
  0x0020,   /* U+27F0 */
  0x0020,   /* U+27F1 */
  0x0020,   /* U+27F2 */
  0x0020,   /* U+27F3 */
  0x0020,   /* U+27F4 */
  0x0020,   /* U+27F5 */
  0x0020,   /* U+27F6 */
  0x0020,   /* U+27F7 */
  0x0020,   /* U+27F8 */
  0x0020,   /* U+27F9 */
  0x0020,   /* U+27FA */
  0x0020,   /* U+27FB */
  0x0020,   /* U+27FC */
  0x0020,   /* U+27FD */
  0x0020,   /* U+27FE */
  0x0020,   /* U+27FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2700 */
  0x0002,   /* U+2701 */
  0x0002,   /* U+2702 */
  0x0002,   /* U+2703 */
  0x0002,   /* U+2704 */
  0x0002,   /* U+2705 */
  0x0002,   /* U+2706 */
  0x0002,   /* U+2707 */
  0x0002,   /* U+2708 */
  0x0002,   /* U+2709 */
  0x0002,   /* U+270A */
  0x0002,   /* U+270B */
  0x0002,   /* U+270C */
  0x0002,   /* U+270D */
  0x0002,   /* U+270E */
  0x0002,   /* U+270F */
  0x0002,   /* U+2710 */
  0x0002,   /* U+2711 */
  0x0002,   /* U+2712 */
  0x0002,   /* U+2713 */
  0x0002,   /* U+2714 */
  0x0002,   /* U+2715 */
  0x0002,   /* U+2716 */
  0x0002,   /* U+2717 */
  0x0002,   /* U+2718 */
  0x0002,   /* U+2719 */
  0x0002,   /* U+271A */
  0x0002,   /* U+271B */
  0x0002,   /* U+271C */
  0x0002,   /* U+271D */
  0x0002,   /* U+271E */
  0x0002,   /* U+271F */
  0x0002,   /* U+2720 */
  0x0002,   /* U+2721 */
  0x0002,   /* U+2722 */
  0x0002,   /* U+2723 */
  0x0002,   /* U+2724 */
  0x0002,   /* U+2725 */
  0x0002,   /* U+2726 */
  0x0002,   /* U+2727 */
  0x0002,   /* U+2728 */
  0x0002,   /* U+2729 */
  0x0002,   /* U+272A */
  0x0002,   /* U+272B */
  0x0002,   /* U+272C */
  0x0002,   /* U+272D */
  0x0002,   /* U+272E */
  0x0002,   /* U+272F */
  0x0002,   /* U+2730 */
  0x0002,   /* U+2731 */
  0x0002,   /* U+2732 */
  0x0002,   /* U+2733 */
  0x0002,   /* U+2734 */
  0x0002,   /* U+2735 */
  0x0002,   /* U+2736 */
  0x0002,   /* U+2737 */
  0x0002,   /* U+2738 */
  0x0002,   /* U+2739 */
  0x0002,   /* U+273A */
  0x0002,   /* U+273B */
  0x0002,   /* U+273C */
  0x0002,   /* U+273D */
  0x0002,   /* U+273E */
  0x0002,   /* U+273F */
  0x0002,   /* U+2740 */
  0x0002,   /* U+2741 */
  0x0002,   /* U+2742 */
  0x0002,   /* U+2743 */
  0x0002,   /* U+2744 */
  0x0002,   /* U+2745 */
  0x0002,   /* U+2746 */
  0x0002,   /* U+2747 */
  0x0002,   /* U+2748 */
  0x0002,   /* U+2749 */
  0x0002,   /* U+274A */
  0x0002,   /* U+274B */
  0x0002,   /* U+274C */
  0x0002,   /* U+274D */
  0x0002,   /* U+274E */
  0x0002,   /* U+274F */
  0x0002,   /* U+2750 */
  0x0002,   /* U+2751 */
  0x0002,   /* U+2752 */
  0x0002,   /* U+2753 */
  0x0002,   /* U+2754 */
  0x0002,   /* U+2755 */
  0x0002,   /* U+2756 */
  0x0002,   /* U+2757 */
  0x0002,   /* U+2758 */
  0x0002,   /* U+2759 */
  0x0002,   /* U+275A */
  0x0002,   /* U+275B */
  0x0002,   /* U+275C */
  0x0002,   /* U+275D */
  0x0002,   /* U+275E */
  0x0002,   /* U+275F */
  0x0002,   /* U+2760 */
  0x0002,   /* U+2761 */
  0x0002,   /* U+2762 */
  0x0002,   /* U+2763 */
  0x0002,   /* U+2764 */
  0x0002,   /* U+2765 */
  0x0002,   /* U+2766 */
  0x0002,   /* U+2767 */
  0x0002,   /* U+2768 */
  0x0002,   /* U+2769 */
  0x0002,   /* U+276A */
  0x0002,   /* U+276B */
  0x0002,   /* U+276C */
  0x0002,   /* U+276D */
  0x0002,   /* U+276E */
  0x0002,   /* U+276F */
  0x0002,   /* U+2770 */
  0x0002,   /* U+2771 */
  0x0002,   /* U+2772 */
  0x0002,   /* U+2773 */
  0x0002,   /* U+2774 */
  0x0002,   /* U+2775 */
  0x0006,   /* U+2776 */
  0x0006,   /* U+2777 */
  0x0006,   /* U+2778 */
  0x0006,   /* U+2779 */
  0x0006,   /* U+277A */
  0x0006,   /* U+277B */
  0x0006,   /* U+277C */
  0x0006,   /* U+277D */
  0x0006,   /* U+277E */
  0x0006,   /* U+277F */
  0x0006,   /* U+2780 */
  0x0006,   /* U+2781 */
  0x0006,   /* U+2782 */
  0x0006,   /* U+2783 */
  0x0006,   /* U+2784 */
  0x0006,   /* U+2785 */
  0x0006,   /* U+2786 */
  0x0006,   /* U+2787 */
  0x0006,   /* U+2788 */
  0x0006,   /* U+2789 */
  0x0006,   /* U+278A */
  0x0006,   /* U+278B */
  0x0006,   /* U+278C */
  0x0006,   /* U+278D */
  0x0006,   /* U+278E */
  0x0006,   /* U+278F */
  0x0006,   /* U+2790 */
  0x0006,   /* U+2791 */
  0x0006,   /* U+2792 */
  0x0006,   /* U+2793 */
  0x0002,   /* U+2794 */
  0x0002,   /* U+2795 */
  0x0002,   /* U+2796 */
  0x0002,   /* U+2797 */
  0x0002,   /* U+2798 */
  0x0002,   /* U+2799 */
  0x0002,   /* U+279A */
  0x0002,   /* U+279B */
  0x0002,   /* U+279C */
  0x0002,   /* U+279D */
  0x0002,   /* U+279E */
  0x0002,   /* U+279F */
  0x0002,   /* U+27A0 */
  0x0002,   /* U+27A1 */
  0x0002,   /* U+27A2 */
  0x0002,   /* U+27A3 */
  0x0002,   /* U+27A4 */
  0x0002,   /* U+27A5 */
  0x0002,   /* U+27A6 */
  0x0002,   /* U+27A7 */
  0x0002,   /* U+27A8 */
  0x0002,   /* U+27A9 */
  0x0002,   /* U+27AA */
  0x0002,   /* U+27AB */
  0x0002,   /* U+27AC */
  0x0002,   /* U+27AD */
  0x0002,   /* U+27AE */
  0x0002,   /* U+27AF */
  0x0002,   /* U+27B0 */
  0x0002,   /* U+27B1 */
  0x0002,   /* U+27B2 */
  0x0002,   /* U+27B3 */
  0x0002,   /* U+27B4 */
  0x0002,   /* U+27B5 */
  0x0002,   /* U+27B6 */
  0x0002,   /* U+27B7 */
  0x0002,   /* U+27B8 */
  0x0002,   /* U+27B9 */
  0x0002,   /* U+27BA */
  0x0002,   /* U+27BB */
  0x0002,   /* U+27BC */
  0x0002,   /* U+27BD */
  0x0002,   /* U+27BE */
  0x0002,   /* U+27BF */
  0x0002,   /* U+27C0 */
  0x0002,   /* U+27C1 */
  0x0002,   /* U+27C2 */
  0x0002,   /* U+27C3 */
  0x0002,   /* U+27C4 */
  0x0002,   /* U+27C5 */
  0x0002,   /* U+27C6 */
  0x0002,   /* U+27C7 */
  0x0002,   /* U+27C8 */
  0x0002,   /* U+27C9 */
  0x0002,   /* U+27CA */
  0x0002,   /* U+27CB */
  0x0002,   /* U+27CC */
  0x0002,   /* U+27CD */
  0x0002,   /* U+27CE */
  0x0002,   /* U+27CF */
  0x0002,   /* U+27D0 */
  0x0002,   /* U+27D1 */
  0x0002,   /* U+27D2 */
  0x0002,   /* U+27D3 */
  0x0002,   /* U+27D4 */
  0x0002,   /* U+27D5 */
  0x0002,   /* U+27D6 */
  0x0002,   /* U+27D7 */
  0x0002,   /* U+27D8 */
  0x0002,   /* U+27D9 */
  0x0002,   /* U+27DA */
  0x0002,   /* U+27DB */
  0x0002,   /* U+27DC */
  0x0002,   /* U+27DD */
  0x0002,   /* U+27DE */
  0x0002,   /* U+27DF */
  0x0002,   /* U+27E0 */
  0x0002,   /* U+27E1 */
  0x0002,   /* U+27E2 */
  0x0002,   /* U+27E3 */
  0x0002,   /* U+27E4 */
  0x0002,   /* U+27E5 */
  0x0002,   /* U+27E6 */
  0x0002,   /* U+27E7 */
  0x0002,   /* U+27E8 */
  0x0002,   /* U+27E9 */
  0x0002,   /* U+27EA */
  0x0002,   /* U+27EB */
  0x0002,   /* U+27EC */
  0x0002,   /* U+27ED */
  0x0002,   /* U+27EE */
  0x0002,   /* U+27EF */
  0x0002,   /* U+27F0 */
  0x0002,   /* U+27F1 */
  0x0002,   /* U+27F2 */
  0x0002,   /* U+27F3 */
  0x0002,   /* U+27F4 */
  0x0002,   /* U+27F5 */
  0x0002,   /* U+27F6 */
  0x0002,   /* U+27F7 */
  0x0002,   /* U+27F8 */
  0x0002,   /* U+27F9 */
  0x0002,   /* U+27FA */
  0x0002,   /* U+27FB */
  0x0002,   /* U+27FC */
  0x0002,   /* U+27FD */
  0x0002,   /* U+27FE */
  0x0002,   /* U+27FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2700 */
  0x0000,   /* U+2701 */
  0x0000,   /* U+2702 */
  0x0000,   /* U+2703 */
  0x0000,   /* U+2704 */
  0x0000,   /* U+2705 */
  0x0000,   /* U+2706 */
  0x0000,   /* U+2707 */
  0x0000,   /* U+2708 */
  0x0000,   /* U+2709 */
  0x0000,   /* U+270A */
  0x0000,   /* U+270B */
  0x0000,   /* U+270C */
  0x0000,   /* U+270D */
  0x0000,   /* U+270E */
  0x0000,   /* U+270F */
  0x0000,   /* U+2710 */
  0x0000,   /* U+2711 */
  0x0000,   /* U+2712 */
  0x0000,   /* U+2713 */
  0x0000,   /* U+2714 */
  0x0000,   /* U+2715 */
  0x0000,   /* U+2716 */
  0x0000,   /* U+2717 */
  0x0000,   /* U+2718 */
  0x0000,   /* U+2719 */
  0x0000,   /* U+271A */
  0x0000,   /* U+271B */
  0x0000,   /* U+271C */
  0x0000,   /* U+271D */
  0x0000,   /* U+271E */
  0x0000,   /* U+271F */
  0x0000,   /* U+2720 */
  0x0000,   /* U+2721 */
  0x0000,   /* U+2722 */
  0x0000,   /* U+2723 */
  0x0000,   /* U+2724 */
  0x0000,   /* U+2725 */
  0x0000,   /* U+2726 */
  0x0000,   /* U+2727 */
  0x0000,   /* U+2728 */
  0x0000,   /* U+2729 */
  0x0000,   /* U+272A */
  0x0000,   /* U+272B */
  0x0000,   /* U+272C */
  0x0000,   /* U+272D */
  0x0000,   /* U+272E */
  0x0000,   /* U+272F */
  0x0000,   /* U+2730 */
  0x0000,   /* U+2731 */
  0x0000,   /* U+2732 */
  0x0000,   /* U+2733 */
  0x0000,   /* U+2734 */
  0x0000,   /* U+2735 */
  0x0000,   /* U+2736 */
  0x0000,   /* U+2737 */
  0x0000,   /* U+2738 */
  0x0000,   /* U+2739 */
  0x0000,   /* U+273A */
  0x0000,   /* U+273B */
  0x0000,   /* U+273C */
  0x0000,   /* U+273D */
  0x0000,   /* U+273E */
  0x0000,   /* U+273F */
  0x0000,   /* U+2740 */
  0x0000,   /* U+2741 */
  0x0000,   /* U+2742 */
  0x0000,   /* U+2743 */
  0x0000,   /* U+2744 */
  0x0000,   /* U+2745 */
  0x0000,   /* U+2746 */
  0x0000,   /* U+2747 */
  0x0000,   /* U+2748 */
  0x0000,   /* U+2749 */
  0x0000,   /* U+274A */
  0x0000,   /* U+274B */
  0x0000,   /* U+274C */
  0x0000,   /* U+274D */
  0x0000,   /* U+274E */
  0x0000,   /* U+274F */
  0x0000,   /* U+2750 */
  0x0000,   /* U+2751 */
  0x0000,   /* U+2752 */
  0x0000,   /* U+2753 */
  0x0000,   /* U+2754 */
  0x0000,   /* U+2755 */
  0x0000,   /* U+2756 */
  0x0000,   /* U+2757 */
  0x0000,   /* U+2758 */
  0x0000,   /* U+2759 */
  0x0000,   /* U+275A */
  0x0000,   /* U+275B */
  0x0000,   /* U+275C */
  0x0000,   /* U+275D */
  0x0000,   /* U+275E */
  0x0000,   /* U+275F */
  0x0000,   /* U+2760 */
  0x0000,   /* U+2761 */
  0x0000,   /* U+2762 */
  0x0000,   /* U+2763 */
  0x0000,   /* U+2764 */
  0x0000,   /* U+2765 */
  0x0000,   /* U+2766 */
  0x0000,   /* U+2767 */
  0x0000,   /* U+2768 */
  0x0000,   /* U+2769 */
  0x0000,   /* U+276A */
  0x0000,   /* U+276B */
  0x0000,   /* U+276C */
  0x0000,   /* U+276D */
  0x0000,   /* U+276E */
  0x0000,   /* U+276F */
  0x0000,   /* U+2770 */
  0x0000,   /* U+2771 */
  0x0000,   /* U+2772 */
  0x0000,   /* U+2773 */
  0x0000,   /* U+2774 */
  0x0000,   /* U+2775 */
  0x0000,   /* U+2776 */
  0x0000,   /* U+2777 */
  0x0000,   /* U+2778 */
  0x0000,   /* U+2779 */
  0x0000,   /* U+277A */
  0x0000,   /* U+277B */
  0x0000,   /* U+277C */
  0x0000,   /* U+277D */
  0x0000,   /* U+277E */
  0x1C3D,   /* U+277F */
  0x0000,   /* U+2780 */
  0x0000,   /* U+2781 */
  0x0000,   /* U+2782 */
  0x0000,   /* U+2783 */
  0x0000,   /* U+2784 */
  0x0000,   /* U+2785 */
  0x0000,   /* U+2786 */
  0x0000,   /* U+2787 */
  0x0000,   /* U+2788 */
  0x1C3D,   /* U+2789 */
  0x0000,   /* U+278A */
  0x0000,   /* U+278B */
  0x0000,   /* U+278C */
  0x0000,   /* U+278D */
  0x0000,   /* U+278E */
  0x0000,   /* U+278F */
  0x0000,   /* U+2790 */
  0x0000,   /* U+2791 */
  0x0000,   /* U+2792 */
  0x1C3D,   /* U+2793 */
  0x0000,   /* U+2794 */
  0x0000,   /* U+2795 */
  0x0000,   /* U+2796 */
  0x0000,   /* U+2797 */
  0x0000,   /* U+2798 */
  0x0000,   /* U+2799 */
  0x0000,   /* U+279A */
  0x0000,   /* U+279B */
  0x0000,   /* U+279C */
  0x0000,   /* U+279D */
  0x0000,   /* U+279E */
  0x0000,   /* U+279F */
  0x0000,   /* U+27A0 */
  0x0000,   /* U+27A1 */
  0x0000,   /* U+27A2 */
  0x0000,   /* U+27A3 */
  0x0000,   /* U+27A4 */
  0x0000,   /* U+27A5 */
  0x0000,   /* U+27A6 */
  0x0000,   /* U+27A7 */
  0x0000,   /* U+27A8 */
  0x0000,   /* U+27A9 */
  0x0000,   /* U+27AA */
  0x0000,   /* U+27AB */
  0x0000,   /* U+27AC */
  0x0000,   /* U+27AD */
  0x0000,   /* U+27AE */
  0x0000,   /* U+27AF */
  0x0000,   /* U+27B0 */
  0x0000,   /* U+27B1 */
  0x0000,   /* U+27B2 */
  0x0000,   /* U+27B3 */
  0x0000,   /* U+27B4 */
  0x0000,   /* U+27B5 */
  0x0000,   /* U+27B6 */
  0x0000,   /* U+27B7 */
  0x0000,   /* U+27B8 */
  0x0000,   /* U+27B9 */
  0x0000,   /* U+27BA */
  0x0000,   /* U+27BB */
  0x0000,   /* U+27BC */
  0x0000,   /* U+27BD */
  0x0000,   /* U+27BE */
  0x0000,   /* U+27BF */
  0x0000,   /* U+27C0 */
  0x0000,   /* U+27C1 */
  0x0000,   /* U+27C2 */
  0x0000,   /* U+27C3 */
  0x0000,   /* U+27C4 */
  0x0000,   /* U+27C5 */
  0x0000,   /* U+27C6 */
  0x0000,   /* U+27C7 */
  0x0000,   /* U+27C8 */
  0x0000,   /* U+27C9 */
  0x0000,   /* U+27CA */
  0x0000,   /* U+27CB */
  0x0000,   /* U+27CC */
  0x0000,   /* U+27CD */
  0x0000,   /* U+27CE */
  0x0000,   /* U+27CF */
  0x0000,   /* U+27D0 */
  0x0000,   /* U+27D1 */
  0x0000,   /* U+27D2 */
  0x0000,   /* U+27D3 */
  0x0000,   /* U+27D4 */
  0x0000,   /* U+27D5 */
  0x0000,   /* U+27D6 */
  0x0000,   /* U+27D7 */
  0x0000,   /* U+27D8 */
  0x0000,   /* U+27D9 */
  0x0000,   /* U+27DA */
  0x0000,   /* U+27DB */
  0x0000,   /* U+27DC */
  0x0000,   /* U+27DD */
  0x0000,   /* U+27DE */
  0x0000,   /* U+27DF */
  0x0000,   /* U+27E0 */
  0x0000,   /* U+27E1 */
  0x0000,   /* U+27E2 */
  0x0000,   /* U+27E3 */
  0x0000,   /* U+27E4 */
  0x0000,   /* U+27E5 */
  0x0000,   /* U+27E6 */
  0x0000,   /* U+27E7 */
  0x0000,   /* U+27E8 */
  0x0000,   /* U+27E9 */
  0x0000,   /* U+27EA */
  0x0000,   /* U+27EB */
  0x0000,   /* U+27EC */
  0x0000,   /* U+27ED */
  0x0000,   /* U+27EE */
  0x0000,   /* U+27EF */
  0x0000,   /* U+27F0 */
  0x0000,   /* U+27F1 */
  0x0000,   /* U+27F2 */
  0x0000,   /* U+27F3 */
  0x0000,   /* U+27F4 */
  0x0000,   /* U+27F5 */
  0x0000,   /* U+27F6 */
  0x0000,   /* U+27F7 */
  0x0000,   /* U+27F8 */
  0x0000,   /* U+27F9 */
  0x0000,   /* U+27FA */
  0x0000,   /* U+27FB */
  0x0000,   /* U+27FC */
  0x0000,   /* U+27FD */
  0x0000,   /* U+27FE */
  0x0000,   /* U+27FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2700 */
  0x0000,   /* U+2701 */
  0x0000,   /* U+2702 */
  0x0000,   /* U+2703 */
  0x0000,   /* U+2704 */
  0x0000,   /* U+2705 */
  0x0000,   /* U+2706 */
  0x0000,   /* U+2707 */
  0x0000,   /* U+2708 */
  0x0000,   /* U+2709 */
  0x0000,   /* U+270A */
  0x0000,   /* U+270B */
  0x0000,   /* U+270C */
  0x0000,   /* U+270D */
  0x0000,   /* U+270E */
  0x0000,   /* U+270F */
  0x0000,   /* U+2710 */
  0x0000,   /* U+2711 */
  0x0000,   /* U+2712 */
  0x0000,   /* U+2713 */
  0x0000,   /* U+2714 */
  0x0000,   /* U+2715 */
  0x0000,   /* U+2716 */
  0x0000,   /* U+2717 */
  0x0000,   /* U+2718 */
  0x0000,   /* U+2719 */
  0x0000,   /* U+271A */
  0x0000,   /* U+271B */
  0x0000,   /* U+271C */
  0x0000,   /* U+271D */
  0x0000,   /* U+271E */
  0x0000,   /* U+271F */
  0x0000,   /* U+2720 */
  0x0000,   /* U+2721 */
  0x0000,   /* U+2722 */
  0x0000,   /* U+2723 */
  0x0000,   /* U+2724 */
  0x0000,   /* U+2725 */
  0x0000,   /* U+2726 */
  0x0000,   /* U+2727 */
  0x0000,   /* U+2728 */
  0x0000,   /* U+2729 */
  0x0000,   /* U+272A */
  0x0000,   /* U+272B */
  0x0000,   /* U+272C */
  0x0000,   /* U+272D */
  0x0000,   /* U+272E */
  0x0000,   /* U+272F */
  0x0000,   /* U+2730 */
  0x0000,   /* U+2731 */
  0x0000,   /* U+2732 */
  0x0000,   /* U+2733 */
  0x0000,   /* U+2734 */
  0x0000,   /* U+2735 */
  0x0000,   /* U+2736 */
  0x0000,   /* U+2737 */
  0x0000,   /* U+2738 */
  0x0000,   /* U+2739 */
  0x0000,   /* U+273A */
  0x0000,   /* U+273B */
  0x0000,   /* U+273C */
  0x0000,   /* U+273D */
  0x0000,   /* U+273E */
  0x0000,   /* U+273F */
  0x0000,   /* U+2740 */
  0x0000,   /* U+2741 */
  0x0000,   /* U+2742 */
  0x0000,   /* U+2743 */
  0x0000,   /* U+2744 */
  0x0000,   /* U+2745 */
  0x0000,   /* U+2746 */
  0x0000,   /* U+2747 */
  0x0000,   /* U+2748 */
  0x0000,   /* U+2749 */
  0x0000,   /* U+274A */
  0x0000,   /* U+274B */
  0x0000,   /* U+274C */
  0x0000,   /* U+274D */
  0x0000,   /* U+274E */
  0x0000,   /* U+274F */
  0x0000,   /* U+2750 */
  0x0000,   /* U+2751 */
  0x0000,   /* U+2752 */
  0x0000,   /* U+2753 */
  0x0000,   /* U+2754 */
  0x0000,   /* U+2755 */
  0x0000,   /* U+2756 */
  0x0000,   /* U+2757 */
  0x0000,   /* U+2758 */
  0x0000,   /* U+2759 */
  0x0000,   /* U+275A */
  0x0000,   /* U+275B */
  0x0000,   /* U+275C */
  0x0000,   /* U+275D */
  0x0000,   /* U+275E */
  0x0000,   /* U+275F */
  0x0000,   /* U+2760 */
  0x0000,   /* U+2761 */
  0x0000,   /* U+2762 */
  0x0000,   /* U+2763 */
  0x0000,   /* U+2764 */
  0x0000,   /* U+2765 */
  0x0000,   /* U+2766 */
  0x0000,   /* U+2767 */
  0x0000,   /* U+2768 */
  0x0000,   /* U+2769 */
  0x0000,   /* U+276A */
  0x0000,   /* U+276B */
  0x0000,   /* U+276C */
  0x0000,   /* U+276D */
  0x0000,   /* U+276E */
  0x0000,   /* U+276F */
  0x0000,   /* U+2770 */
  0x0000,   /* U+2771 */
  0x0000,   /* U+2772 */
  0x0000,   /* U+2773 */
  0x0000,   /* U+2774 */
  0x0000,   /* U+2775 */
  0x0000,   /* U+2776 */
  0x0000,   /* U+2777 */
  0x0000,   /* U+2778 */
  0x0000,   /* U+2779 */
  0x0000,   /* U+277A */
  0x0000,   /* U+277B */
  0x0000,   /* U+277C */
  0x0000,   /* U+277D */
  0x0000,   /* U+277E */
  0x0020,   /* U+277F */
  0x0000,   /* U+2780 */
  0x0000,   /* U+2781 */
  0x0000,   /* U+2782 */
  0x0000,   /* U+2783 */
  0x0000,   /* U+2784 */
  0x0000,   /* U+2785 */
  0x0000,   /* U+2786 */
  0x0000,   /* U+2787 */
  0x0000,   /* U+2788 */
  0x0020,   /* U+2789 */
  0x0000,   /* U+278A */
  0x0000,   /* U+278B */
  0x0000,   /* U+278C */
  0x0000,   /* U+278D */
  0x0000,   /* U+278E */
  0x0000,   /* U+278F */
  0x0000,   /* U+2790 */
  0x0000,   /* U+2791 */
  0x0000,   /* U+2792 */
  0x0020,   /* U+2793 */
  0x0000,   /* U+2794 */
  0x0000,   /* U+2795 */
  0x0000,   /* U+2796 */
  0x0000,   /* U+2797 */
  0x0000,   /* U+2798 */
  0x0000,   /* U+2799 */
  0x0000,   /* U+279A */
  0x0000,   /* U+279B */
  0x0000,   /* U+279C */
  0x0000,   /* U+279D */
  0x0000,   /* U+279E */
  0x0000,   /* U+279F */
  0x0000,   /* U+27A0 */
  0x0000,   /* U+27A1 */
  0x0000,   /* U+27A2 */
  0x0000,   /* U+27A3 */
  0x0000,   /* U+27A4 */
  0x0000,   /* U+27A5 */
  0x0000,   /* U+27A6 */
  0x0000,   /* U+27A7 */
  0x0000,   /* U+27A8 */
  0x0000,   /* U+27A9 */
  0x0000,   /* U+27AA */
  0x0000,   /* U+27AB */
  0x0000,   /* U+27AC */
  0x0000,   /* U+27AD */
  0x0000,   /* U+27AE */
  0x0000,   /* U+27AF */
  0x0000,   /* U+27B0 */
  0x0000,   /* U+27B1 */
  0x0000,   /* U+27B2 */
  0x0000,   /* U+27B3 */
  0x0000,   /* U+27B4 */
  0x0000,   /* U+27B5 */
  0x0000,   /* U+27B6 */
  0x0000,   /* U+27B7 */
  0x0000,   /* U+27B8 */
  0x0000,   /* U+27B9 */
  0x0000,   /* U+27BA */
  0x0000,   /* U+27BB */
  0x0000,   /* U+27BC */
  0x0000,   /* U+27BD */
  0x0000,   /* U+27BE */
  0x0000,   /* U+27BF */
  0x0000,   /* U+27C0 */
  0x0000,   /* U+27C1 */
  0x0000,   /* U+27C2 */
  0x0000,   /* U+27C3 */
  0x0000,   /* U+27C4 */
  0x0000,   /* U+27C5 */
  0x0000,   /* U+27C6 */
  0x0000,   /* U+27C7 */
  0x0000,   /* U+27C8 */
  0x0000,   /* U+27C9 */
  0x0000,   /* U+27CA */
  0x0000,   /* U+27CB */
  0x0000,   /* U+27CC */
  0x0000,   /* U+27CD */
  0x0000,   /* U+27CE */
  0x0000,   /* U+27CF */
  0x0000,   /* U+27D0 */
  0x0000,   /* U+27D1 */
  0x0000,   /* U+27D2 */
  0x0000,   /* U+27D3 */
  0x0000,   /* U+27D4 */
  0x0000,   /* U+27D5 */
  0x0000,   /* U+27D6 */
  0x0000,   /* U+27D7 */
  0x0000,   /* U+27D8 */
  0x0000,   /* U+27D9 */
  0x0000,   /* U+27DA */
  0x0000,   /* U+27DB */
  0x0000,   /* U+27DC */
  0x0000,   /* U+27DD */
  0x0000,   /* U+27DE */
  0x0000,   /* U+27DF */
  0x0000,   /* U+27E0 */
  0x0000,   /* U+27E1 */
  0x0000,   /* U+27E2 */
  0x0000,   /* U+27E3 */
  0x0000,   /* U+27E4 */
  0x0000,   /* U+27E5 */
  0x0000,   /* U+27E6 */
  0x0000,   /* U+27E7 */
  0x0000,   /* U+27E8 */
  0x0000,   /* U+27E9 */
  0x0000,   /* U+27EA */
  0x0000,   /* U+27EB */
  0x0000,   /* U+27EC */
  0x0000,   /* U+27ED */
  0x0000,   /* U+27EE */
  0x0000,   /* U+27EF */
  0x0000,   /* U+27F0 */
  0x0000,   /* U+27F1 */
  0x0000,   /* U+27F2 */
  0x0000,   /* U+27F3 */
  0x0000,   /* U+27F4 */
  0x0000,   /* U+27F5 */
  0x0000,   /* U+27F6 */
  0x0000,   /* U+27F7 */
  0x0000,   /* U+27F8 */
  0x0000,   /* U+27F9 */
  0x0000,   /* U+27FA */
  0x0000,   /* U+27FB */
  0x0000,   /* U+27FC */
  0x0000,   /* U+27FD */
  0x0000,   /* U+27FE */
  0x0000,   /* U+27FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2700 */
  0x0000,   /* U+2701 */
  0x0000,   /* U+2702 */
  0x0000,   /* U+2703 */
  0x0000,   /* U+2704 */
  0x0000,   /* U+2705 */
  0x0000,   /* U+2706 */
  0x0000,   /* U+2707 */
  0x0000,   /* U+2708 */
  0x0000,   /* U+2709 */
  0x0000,   /* U+270A */
  0x0000,   /* U+270B */
  0x0000,   /* U+270C */
  0x0000,   /* U+270D */
  0x0000,   /* U+270E */
  0x0000,   /* U+270F */
  0x0000,   /* U+2710 */
  0x0000,   /* U+2711 */
  0x0000,   /* U+2712 */
  0x0000,   /* U+2713 */
  0x0000,   /* U+2714 */
  0x0000,   /* U+2715 */
  0x0000,   /* U+2716 */
  0x0000,   /* U+2717 */
  0x0000,   /* U+2718 */
  0x0000,   /* U+2719 */
  0x0000,   /* U+271A */
  0x0000,   /* U+271B */
  0x0000,   /* U+271C */
  0x0000,   /* U+271D */
  0x0000,   /* U+271E */
  0x0000,   /* U+271F */
  0x0000,   /* U+2720 */
  0x0000,   /* U+2721 */
  0x0000,   /* U+2722 */
  0x0000,   /* U+2723 */
  0x0000,   /* U+2724 */
  0x0000,   /* U+2725 */
  0x0000,   /* U+2726 */
  0x0000,   /* U+2727 */
  0x0000,   /* U+2728 */
  0x0000,   /* U+2729 */
  0x0000,   /* U+272A */
  0x0000,   /* U+272B */
  0x0000,   /* U+272C */
  0x0000,   /* U+272D */
  0x0000,   /* U+272E */
  0x0000,   /* U+272F */
  0x0000,   /* U+2730 */
  0x0000,   /* U+2731 */
  0x0000,   /* U+2732 */
  0x0000,   /* U+2733 */
  0x0000,   /* U+2734 */
  0x0000,   /* U+2735 */
  0x0000,   /* U+2736 */
  0x0000,   /* U+2737 */
  0x0000,   /* U+2738 */
  0x0000,   /* U+2739 */
  0x0000,   /* U+273A */
  0x0000,   /* U+273B */
  0x0000,   /* U+273C */
  0x0000,   /* U+273D */
  0x0000,   /* U+273E */
  0x0000,   /* U+273F */
  0x0000,   /* U+2740 */
  0x0000,   /* U+2741 */
  0x0000,   /* U+2742 */
  0x0000,   /* U+2743 */
  0x0000,   /* U+2744 */
  0x0000,   /* U+2745 */
  0x0000,   /* U+2746 */
  0x0000,   /* U+2747 */
  0x0000,   /* U+2748 */
  0x0000,   /* U+2749 */
  0x0000,   /* U+274A */
  0x0000,   /* U+274B */
  0x0000,   /* U+274C */
  0x0000,   /* U+274D */
  0x0000,   /* U+274E */
  0x0000,   /* U+274F */
  0x0000,   /* U+2750 */
  0x0000,   /* U+2751 */
  0x0000,   /* U+2752 */
  0x0000,   /* U+2753 */
  0x0000,   /* U+2754 */
  0x0000,   /* U+2755 */
  0x0000,   /* U+2756 */
  0x0000,   /* U+2757 */
  0x0000,   /* U+2758 */
  0x0000,   /* U+2759 */
  0x0000,   /* U+275A */
  0x0000,   /* U+275B */
  0x0000,   /* U+275C */
  0x0000,   /* U+275D */
  0x0000,   /* U+275E */
  0x0000,   /* U+275F */
  0x0000,   /* U+2760 */
  0x0000,   /* U+2761 */
  0x0000,   /* U+2762 */
  0x0000,   /* U+2763 */
  0x0000,   /* U+2764 */
  0x0000,   /* U+2765 */
  0x0000,   /* U+2766 */
  0x0000,   /* U+2767 */
  0x0000,   /* U+2768 */
  0x0000,   /* U+2769 */
  0x0000,   /* U+276A */
  0x0000,   /* U+276B */
  0x0000,   /* U+276C */
  0x0000,   /* U+276D */
  0x0000,   /* U+276E */
  0x0000,   /* U+276F */
  0x0000,   /* U+2770 */
  0x0000,   /* U+2771 */
  0x0000,   /* U+2772 */
  0x0000,   /* U+2773 */
  0x0000,   /* U+2774 */
  0x0000,   /* U+2775 */
  0x0000,   /* U+2776 */
  0x0000,   /* U+2777 */
  0x0000,   /* U+2778 */
  0x0000,   /* U+2779 */
  0x0000,   /* U+277A */
  0x0000,   /* U+277B */
  0x0000,   /* U+277C */
  0x0000,   /* U+277D */
  0x0000,   /* U+277E */
  0x0006,   /* U+277F */
  0x0000,   /* U+2780 */
  0x0000,   /* U+2781 */
  0x0000,   /* U+2782 */
  0x0000,   /* U+2783 */
  0x0000,   /* U+2784 */
  0x0000,   /* U+2785 */
  0x0000,   /* U+2786 */
  0x0000,   /* U+2787 */
  0x0000,   /* U+2788 */
  0x0006,   /* U+2789 */
  0x0000,   /* U+278A */
  0x0000,   /* U+278B */
  0x0000,   /* U+278C */
  0x0000,   /* U+278D */
  0x0000,   /* U+278E */
  0x0000,   /* U+278F */
  0x0000,   /* U+2790 */
  0x0000,   /* U+2791 */
  0x0000,   /* U+2792 */
  0x0006,   /* U+2793 */
  0x0000,   /* U+2794 */
  0x0000,   /* U+2795 */
  0x0000,   /* U+2796 */
  0x0000,   /* U+2797 */
  0x0000,   /* U+2798 */
  0x0000,   /* U+2799 */
  0x0000,   /* U+279A */
  0x0000,   /* U+279B */
  0x0000,   /* U+279C */
  0x0000,   /* U+279D */
  0x0000,   /* U+279E */
  0x0000,   /* U+279F */
  0x0000,   /* U+27A0 */
  0x0000,   /* U+27A1 */
  0x0000,   /* U+27A2 */
  0x0000,   /* U+27A3 */
  0x0000,   /* U+27A4 */
  0x0000,   /* U+27A5 */
  0x0000,   /* U+27A6 */
  0x0000,   /* U+27A7 */
  0x0000,   /* U+27A8 */
  0x0000,   /* U+27A9 */
  0x0000,   /* U+27AA */
  0x0000,   /* U+27AB */
  0x0000,   /* U+27AC */
  0x0000,   /* U+27AD */
  0x0000,   /* U+27AE */
  0x0000,   /* U+27AF */
  0x0000,   /* U+27B0 */
  0x0000,   /* U+27B1 */
  0x0000,   /* U+27B2 */
  0x0000,   /* U+27B3 */
  0x0000,   /* U+27B4 */
  0x0000,   /* U+27B5 */
  0x0000,   /* U+27B6 */
  0x0000,   /* U+27B7 */
  0x0000,   /* U+27B8 */
  0x0000,   /* U+27B9 */
  0x0000,   /* U+27BA */
  0x0000,   /* U+27BB */
  0x0000,   /* U+27BC */
  0x0000,   /* U+27BD */
  0x0000,   /* U+27BE */
  0x0000,   /* U+27BF */
  0x0000,   /* U+27C0 */
  0x0000,   /* U+27C1 */
  0x0000,   /* U+27C2 */
  0x0000,   /* U+27C3 */
  0x0000,   /* U+27C4 */
  0x0000,   /* U+27C5 */
  0x0000,   /* U+27C6 */
  0x0000,   /* U+27C7 */
  0x0000,   /* U+27C8 */
  0x0000,   /* U+27C9 */
  0x0000,   /* U+27CA */
  0x0000,   /* U+27CB */
  0x0000,   /* U+27CC */
  0x0000,   /* U+27CD */
  0x0000,   /* U+27CE */
  0x0000,   /* U+27CF */
  0x0000,   /* U+27D0 */
  0x0000,   /* U+27D1 */
  0x0000,   /* U+27D2 */
  0x0000,   /* U+27D3 */
  0x0000,   /* U+27D4 */
  0x0000,   /* U+27D5 */
  0x0000,   /* U+27D6 */
  0x0000,   /* U+27D7 */
  0x0000,   /* U+27D8 */
  0x0000,   /* U+27D9 */
  0x0000,   /* U+27DA */
  0x0000,   /* U+27DB */
  0x0000,   /* U+27DC */
  0x0000,   /* U+27DD */
  0x0000,   /* U+27DE */
  0x0000,   /* U+27DF */
  0x0000,   /* U+27E0 */
  0x0000,   /* U+27E1 */
  0x0000,   /* U+27E2 */
  0x0000,   /* U+27E3 */
  0x0000,   /* U+27E4 */
  0x0000,   /* U+27E5 */
  0x0000,   /* U+27E6 */
  0x0000,   /* U+27E7 */
  0x0000,   /* U+27E8 */
  0x0000,   /* U+27E9 */
  0x0000,   /* U+27EA */
  0x0000,   /* U+27EB */
  0x0000,   /* U+27EC */
  0x0000,   /* U+27ED */
  0x0000,   /* U+27EE */
  0x0000,   /* U+27EF */
  0x0000,   /* U+27F0 */
  0x0000,   /* U+27F1 */
  0x0000,   /* U+27F2 */
  0x0000,   /* U+27F3 */
  0x0000,   /* U+27F4 */
  0x0000,   /* U+27F5 */
  0x0000,   /* U+27F6 */
  0x0000,   /* U+27F7 */
  0x0000,   /* U+27F8 */
  0x0000,   /* U+27F9 */
  0x0000,   /* U+27FA */
  0x0000,   /* U+27FB */
  0x0000,   /* U+27FC */
  0x0000,   /* U+27FD */
  0x0000,   /* U+27FE */
  0x0000,   /* U+27FF */
};

uint16 uca900_p028[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0D9C,   /* U+2800 */
  0x0D9D,   /* U+2801 */
  0x0D9E,   /* U+2802 */
  0x0D9F,   /* U+2803 */
  0x0DA0,   /* U+2804 */
  0x0DA1,   /* U+2805 */
  0x0DA2,   /* U+2806 */
  0x0DA3,   /* U+2807 */
  0x0DA4,   /* U+2808 */
  0x0DA5,   /* U+2809 */
  0x0DA6,   /* U+280A */
  0x0DA7,   /* U+280B */
  0x0DA8,   /* U+280C */
  0x0DA9,   /* U+280D */
  0x0DAA,   /* U+280E */
  0x0DAB,   /* U+280F */
  0x0DAC,   /* U+2810 */
  0x0DAD,   /* U+2811 */
  0x0DAE,   /* U+2812 */
  0x0DAF,   /* U+2813 */
  0x0DB0,   /* U+2814 */
  0x0DB1,   /* U+2815 */
  0x0DB2,   /* U+2816 */
  0x0DB3,   /* U+2817 */
  0x0DB4,   /* U+2818 */
  0x0DB5,   /* U+2819 */
  0x0DB6,   /* U+281A */
  0x0DB7,   /* U+281B */
  0x0DB8,   /* U+281C */
  0x0DB9,   /* U+281D */
  0x0DBA,   /* U+281E */
  0x0DBB,   /* U+281F */
  0x0DBC,   /* U+2820 */
  0x0DBD,   /* U+2821 */
  0x0DBE,   /* U+2822 */
  0x0DBF,   /* U+2823 */
  0x0DC0,   /* U+2824 */
  0x0DC1,   /* U+2825 */
  0x0DC2,   /* U+2826 */
  0x0DC3,   /* U+2827 */
  0x0DC4,   /* U+2828 */
  0x0DC5,   /* U+2829 */
  0x0DC6,   /* U+282A */
  0x0DC7,   /* U+282B */
  0x0DC8,   /* U+282C */
  0x0DC9,   /* U+282D */
  0x0DCA,   /* U+282E */
  0x0DCB,   /* U+282F */
  0x0DCC,   /* U+2830 */
  0x0DCD,   /* U+2831 */
  0x0DCE,   /* U+2832 */
  0x0DCF,   /* U+2833 */
  0x0DD0,   /* U+2834 */
  0x0DD1,   /* U+2835 */
  0x0DD2,   /* U+2836 */
  0x0DD3,   /* U+2837 */
  0x0DD4,   /* U+2838 */
  0x0DD5,   /* U+2839 */
  0x0DD6,   /* U+283A */
  0x0DD7,   /* U+283B */
  0x0DD8,   /* U+283C */
  0x0DD9,   /* U+283D */
  0x0DDA,   /* U+283E */
  0x0DDB,   /* U+283F */
  0x0DDC,   /* U+2840 */
  0x0DDD,   /* U+2841 */
  0x0DDE,   /* U+2842 */
  0x0DDF,   /* U+2843 */
  0x0DE0,   /* U+2844 */
  0x0DE1,   /* U+2845 */
  0x0DE2,   /* U+2846 */
  0x0DE3,   /* U+2847 */
  0x0DE4,   /* U+2848 */
  0x0DE5,   /* U+2849 */
  0x0DE6,   /* U+284A */
  0x0DE7,   /* U+284B */
  0x0DE8,   /* U+284C */
  0x0DE9,   /* U+284D */
  0x0DEA,   /* U+284E */
  0x0DEB,   /* U+284F */
  0x0DEC,   /* U+2850 */
  0x0DED,   /* U+2851 */
  0x0DEE,   /* U+2852 */
  0x0DEF,   /* U+2853 */
  0x0DF0,   /* U+2854 */
  0x0DF1,   /* U+2855 */
  0x0DF2,   /* U+2856 */
  0x0DF3,   /* U+2857 */
  0x0DF4,   /* U+2858 */
  0x0DF5,   /* U+2859 */
  0x0DF6,   /* U+285A */
  0x0DF7,   /* U+285B */
  0x0DF8,   /* U+285C */
  0x0DF9,   /* U+285D */
  0x0DFA,   /* U+285E */
  0x0DFB,   /* U+285F */
  0x0DFC,   /* U+2860 */
  0x0DFD,   /* U+2861 */
  0x0DFE,   /* U+2862 */
  0x0DFF,   /* U+2863 */
  0x0E00,   /* U+2864 */
  0x0E01,   /* U+2865 */
  0x0E02,   /* U+2866 */
  0x0E03,   /* U+2867 */
  0x0E04,   /* U+2868 */
  0x0E05,   /* U+2869 */
  0x0E06,   /* U+286A */
  0x0E07,   /* U+286B */
  0x0E08,   /* U+286C */
  0x0E09,   /* U+286D */
  0x0E0A,   /* U+286E */
  0x0E0B,   /* U+286F */
  0x0E0C,   /* U+2870 */
  0x0E0D,   /* U+2871 */
  0x0E0E,   /* U+2872 */
  0x0E0F,   /* U+2873 */
  0x0E10,   /* U+2874 */
  0x0E11,   /* U+2875 */
  0x0E12,   /* U+2876 */
  0x0E13,   /* U+2877 */
  0x0E14,   /* U+2878 */
  0x0E15,   /* U+2879 */
  0x0E16,   /* U+287A */
  0x0E17,   /* U+287B */
  0x0E18,   /* U+287C */
  0x0E19,   /* U+287D */
  0x0E1A,   /* U+287E */
  0x0E1B,   /* U+287F */
  0x0E1C,   /* U+2880 */
  0x0E1D,   /* U+2881 */
  0x0E1E,   /* U+2882 */
  0x0E1F,   /* U+2883 */
  0x0E20,   /* U+2884 */
  0x0E21,   /* U+2885 */
  0x0E22,   /* U+2886 */
  0x0E23,   /* U+2887 */
  0x0E24,   /* U+2888 */
  0x0E25,   /* U+2889 */
  0x0E26,   /* U+288A */
  0x0E27,   /* U+288B */
  0x0E28,   /* U+288C */
  0x0E29,   /* U+288D */
  0x0E2A,   /* U+288E */
  0x0E2B,   /* U+288F */
  0x0E2C,   /* U+2890 */
  0x0E2D,   /* U+2891 */
  0x0E2E,   /* U+2892 */
  0x0E2F,   /* U+2893 */
  0x0E30,   /* U+2894 */
  0x0E31,   /* U+2895 */
  0x0E32,   /* U+2896 */
  0x0E33,   /* U+2897 */
  0x0E34,   /* U+2898 */
  0x0E35,   /* U+2899 */
  0x0E36,   /* U+289A */
  0x0E37,   /* U+289B */
  0x0E38,   /* U+289C */
  0x0E39,   /* U+289D */
  0x0E3A,   /* U+289E */
  0x0E3B,   /* U+289F */
  0x0E3C,   /* U+28A0 */
  0x0E3D,   /* U+28A1 */
  0x0E3E,   /* U+28A2 */
  0x0E3F,   /* U+28A3 */
  0x0E40,   /* U+28A4 */
  0x0E41,   /* U+28A5 */
  0x0E42,   /* U+28A6 */
  0x0E43,   /* U+28A7 */
  0x0E44,   /* U+28A8 */
  0x0E45,   /* U+28A9 */
  0x0E46,   /* U+28AA */
  0x0E47,   /* U+28AB */
  0x0E48,   /* U+28AC */
  0x0E49,   /* U+28AD */
  0x0E4A,   /* U+28AE */
  0x0E4B,   /* U+28AF */
  0x0E4C,   /* U+28B0 */
  0x0E4D,   /* U+28B1 */
  0x0E4E,   /* U+28B2 */
  0x0E4F,   /* U+28B3 */
  0x0E50,   /* U+28B4 */
  0x0E51,   /* U+28B5 */
  0x0E52,   /* U+28B6 */
  0x0E53,   /* U+28B7 */
  0x0E54,   /* U+28B8 */
  0x0E55,   /* U+28B9 */
  0x0E56,   /* U+28BA */
  0x0E57,   /* U+28BB */
  0x0E58,   /* U+28BC */
  0x0E59,   /* U+28BD */
  0x0E5A,   /* U+28BE */
  0x0E5B,   /* U+28BF */
  0x0E5C,   /* U+28C0 */
  0x0E5D,   /* U+28C1 */
  0x0E5E,   /* U+28C2 */
  0x0E5F,   /* U+28C3 */
  0x0E60,   /* U+28C4 */
  0x0E61,   /* U+28C5 */
  0x0E62,   /* U+28C6 */
  0x0E63,   /* U+28C7 */
  0x0E64,   /* U+28C8 */
  0x0E65,   /* U+28C9 */
  0x0E66,   /* U+28CA */
  0x0E67,   /* U+28CB */
  0x0E68,   /* U+28CC */
  0x0E69,   /* U+28CD */
  0x0E6A,   /* U+28CE */
  0x0E6B,   /* U+28CF */
  0x0E6C,   /* U+28D0 */
  0x0E6D,   /* U+28D1 */
  0x0E6E,   /* U+28D2 */
  0x0E6F,   /* U+28D3 */
  0x0E70,   /* U+28D4 */
  0x0E71,   /* U+28D5 */
  0x0E72,   /* U+28D6 */
  0x0E73,   /* U+28D7 */
  0x0E74,   /* U+28D8 */
  0x0E75,   /* U+28D9 */
  0x0E76,   /* U+28DA */
  0x0E77,   /* U+28DB */
  0x0E78,   /* U+28DC */
  0x0E79,   /* U+28DD */
  0x0E7A,   /* U+28DE */
  0x0E7B,   /* U+28DF */
  0x0E7C,   /* U+28E0 */
  0x0E7D,   /* U+28E1 */
  0x0E7E,   /* U+28E2 */
  0x0E7F,   /* U+28E3 */
  0x0E80,   /* U+28E4 */
  0x0E81,   /* U+28E5 */
  0x0E82,   /* U+28E6 */
  0x0E83,   /* U+28E7 */
  0x0E84,   /* U+28E8 */
  0x0E85,   /* U+28E9 */
  0x0E86,   /* U+28EA */
  0x0E87,   /* U+28EB */
  0x0E88,   /* U+28EC */
  0x0E89,   /* U+28ED */
  0x0E8A,   /* U+28EE */
  0x0E8B,   /* U+28EF */
  0x0E8C,   /* U+28F0 */
  0x0E8D,   /* U+28F1 */
  0x0E8E,   /* U+28F2 */
  0x0E8F,   /* U+28F3 */
  0x0E90,   /* U+28F4 */
  0x0E91,   /* U+28F5 */
  0x0E92,   /* U+28F6 */
  0x0E93,   /* U+28F7 */
  0x0E94,   /* U+28F8 */
  0x0E95,   /* U+28F9 */
  0x0E96,   /* U+28FA */
  0x0E97,   /* U+28FB */
  0x0E98,   /* U+28FC */
  0x0E99,   /* U+28FD */
  0x0E9A,   /* U+28FE */
  0x0E9B,   /* U+28FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2800 */
  0x0020,   /* U+2801 */
  0x0020,   /* U+2802 */
  0x0020,   /* U+2803 */
  0x0020,   /* U+2804 */
  0x0020,   /* U+2805 */
  0x0020,   /* U+2806 */
  0x0020,   /* U+2807 */
  0x0020,   /* U+2808 */
  0x0020,   /* U+2809 */
  0x0020,   /* U+280A */
  0x0020,   /* U+280B */
  0x0020,   /* U+280C */
  0x0020,   /* U+280D */
  0x0020,   /* U+280E */
  0x0020,   /* U+280F */
  0x0020,   /* U+2810 */
  0x0020,   /* U+2811 */
  0x0020,   /* U+2812 */
  0x0020,   /* U+2813 */
  0x0020,   /* U+2814 */
  0x0020,   /* U+2815 */
  0x0020,   /* U+2816 */
  0x0020,   /* U+2817 */
  0x0020,   /* U+2818 */
  0x0020,   /* U+2819 */
  0x0020,   /* U+281A */
  0x0020,   /* U+281B */
  0x0020,   /* U+281C */
  0x0020,   /* U+281D */
  0x0020,   /* U+281E */
  0x0020,   /* U+281F */
  0x0020,   /* U+2820 */
  0x0020,   /* U+2821 */
  0x0020,   /* U+2822 */
  0x0020,   /* U+2823 */
  0x0020,   /* U+2824 */
  0x0020,   /* U+2825 */
  0x0020,   /* U+2826 */
  0x0020,   /* U+2827 */
  0x0020,   /* U+2828 */
  0x0020,   /* U+2829 */
  0x0020,   /* U+282A */
  0x0020,   /* U+282B */
  0x0020,   /* U+282C */
  0x0020,   /* U+282D */
  0x0020,   /* U+282E */
  0x0020,   /* U+282F */
  0x0020,   /* U+2830 */
  0x0020,   /* U+2831 */
  0x0020,   /* U+2832 */
  0x0020,   /* U+2833 */
  0x0020,   /* U+2834 */
  0x0020,   /* U+2835 */
  0x0020,   /* U+2836 */
  0x0020,   /* U+2837 */
  0x0020,   /* U+2838 */
  0x0020,   /* U+2839 */
  0x0020,   /* U+283A */
  0x0020,   /* U+283B */
  0x0020,   /* U+283C */
  0x0020,   /* U+283D */
  0x0020,   /* U+283E */
  0x0020,   /* U+283F */
  0x0020,   /* U+2840 */
  0x0020,   /* U+2841 */
  0x0020,   /* U+2842 */
  0x0020,   /* U+2843 */
  0x0020,   /* U+2844 */
  0x0020,   /* U+2845 */
  0x0020,   /* U+2846 */
  0x0020,   /* U+2847 */
  0x0020,   /* U+2848 */
  0x0020,   /* U+2849 */
  0x0020,   /* U+284A */
  0x0020,   /* U+284B */
  0x0020,   /* U+284C */
  0x0020,   /* U+284D */
  0x0020,   /* U+284E */
  0x0020,   /* U+284F */
  0x0020,   /* U+2850 */
  0x0020,   /* U+2851 */
  0x0020,   /* U+2852 */
  0x0020,   /* U+2853 */
  0x0020,   /* U+2854 */
  0x0020,   /* U+2855 */
  0x0020,   /* U+2856 */
  0x0020,   /* U+2857 */
  0x0020,   /* U+2858 */
  0x0020,   /* U+2859 */
  0x0020,   /* U+285A */
  0x0020,   /* U+285B */
  0x0020,   /* U+285C */
  0x0020,   /* U+285D */
  0x0020,   /* U+285E */
  0x0020,   /* U+285F */
  0x0020,   /* U+2860 */
  0x0020,   /* U+2861 */
  0x0020,   /* U+2862 */
  0x0020,   /* U+2863 */
  0x0020,   /* U+2864 */
  0x0020,   /* U+2865 */
  0x0020,   /* U+2866 */
  0x0020,   /* U+2867 */
  0x0020,   /* U+2868 */
  0x0020,   /* U+2869 */
  0x0020,   /* U+286A */
  0x0020,   /* U+286B */
  0x0020,   /* U+286C */
  0x0020,   /* U+286D */
  0x0020,   /* U+286E */
  0x0020,   /* U+286F */
  0x0020,   /* U+2870 */
  0x0020,   /* U+2871 */
  0x0020,   /* U+2872 */
  0x0020,   /* U+2873 */
  0x0020,   /* U+2874 */
  0x0020,   /* U+2875 */
  0x0020,   /* U+2876 */
  0x0020,   /* U+2877 */
  0x0020,   /* U+2878 */
  0x0020,   /* U+2879 */
  0x0020,   /* U+287A */
  0x0020,   /* U+287B */
  0x0020,   /* U+287C */
  0x0020,   /* U+287D */
  0x0020,   /* U+287E */
  0x0020,   /* U+287F */
  0x0020,   /* U+2880 */
  0x0020,   /* U+2881 */
  0x0020,   /* U+2882 */
  0x0020,   /* U+2883 */
  0x0020,   /* U+2884 */
  0x0020,   /* U+2885 */
  0x0020,   /* U+2886 */
  0x0020,   /* U+2887 */
  0x0020,   /* U+2888 */
  0x0020,   /* U+2889 */
  0x0020,   /* U+288A */
  0x0020,   /* U+288B */
  0x0020,   /* U+288C */
  0x0020,   /* U+288D */
  0x0020,   /* U+288E */
  0x0020,   /* U+288F */
  0x0020,   /* U+2890 */
  0x0020,   /* U+2891 */
  0x0020,   /* U+2892 */
  0x0020,   /* U+2893 */
  0x0020,   /* U+2894 */
  0x0020,   /* U+2895 */
  0x0020,   /* U+2896 */
  0x0020,   /* U+2897 */
  0x0020,   /* U+2898 */
  0x0020,   /* U+2899 */
  0x0020,   /* U+289A */
  0x0020,   /* U+289B */
  0x0020,   /* U+289C */
  0x0020,   /* U+289D */
  0x0020,   /* U+289E */
  0x0020,   /* U+289F */
  0x0020,   /* U+28A0 */
  0x0020,   /* U+28A1 */
  0x0020,   /* U+28A2 */
  0x0020,   /* U+28A3 */
  0x0020,   /* U+28A4 */
  0x0020,   /* U+28A5 */
  0x0020,   /* U+28A6 */
  0x0020,   /* U+28A7 */
  0x0020,   /* U+28A8 */
  0x0020,   /* U+28A9 */
  0x0020,   /* U+28AA */
  0x0020,   /* U+28AB */
  0x0020,   /* U+28AC */
  0x0020,   /* U+28AD */
  0x0020,   /* U+28AE */
  0x0020,   /* U+28AF */
  0x0020,   /* U+28B0 */
  0x0020,   /* U+28B1 */
  0x0020,   /* U+28B2 */
  0x0020,   /* U+28B3 */
  0x0020,   /* U+28B4 */
  0x0020,   /* U+28B5 */
  0x0020,   /* U+28B6 */
  0x0020,   /* U+28B7 */
  0x0020,   /* U+28B8 */
  0x0020,   /* U+28B9 */
  0x0020,   /* U+28BA */
  0x0020,   /* U+28BB */
  0x0020,   /* U+28BC */
  0x0020,   /* U+28BD */
  0x0020,   /* U+28BE */
  0x0020,   /* U+28BF */
  0x0020,   /* U+28C0 */
  0x0020,   /* U+28C1 */
  0x0020,   /* U+28C2 */
  0x0020,   /* U+28C3 */
  0x0020,   /* U+28C4 */
  0x0020,   /* U+28C5 */
  0x0020,   /* U+28C6 */
  0x0020,   /* U+28C7 */
  0x0020,   /* U+28C8 */
  0x0020,   /* U+28C9 */
  0x0020,   /* U+28CA */
  0x0020,   /* U+28CB */
  0x0020,   /* U+28CC */
  0x0020,   /* U+28CD */
  0x0020,   /* U+28CE */
  0x0020,   /* U+28CF */
  0x0020,   /* U+28D0 */
  0x0020,   /* U+28D1 */
  0x0020,   /* U+28D2 */
  0x0020,   /* U+28D3 */
  0x0020,   /* U+28D4 */
  0x0020,   /* U+28D5 */
  0x0020,   /* U+28D6 */
  0x0020,   /* U+28D7 */
  0x0020,   /* U+28D8 */
  0x0020,   /* U+28D9 */
  0x0020,   /* U+28DA */
  0x0020,   /* U+28DB */
  0x0020,   /* U+28DC */
  0x0020,   /* U+28DD */
  0x0020,   /* U+28DE */
  0x0020,   /* U+28DF */
  0x0020,   /* U+28E0 */
  0x0020,   /* U+28E1 */
  0x0020,   /* U+28E2 */
  0x0020,   /* U+28E3 */
  0x0020,   /* U+28E4 */
  0x0020,   /* U+28E5 */
  0x0020,   /* U+28E6 */
  0x0020,   /* U+28E7 */
  0x0020,   /* U+28E8 */
  0x0020,   /* U+28E9 */
  0x0020,   /* U+28EA */
  0x0020,   /* U+28EB */
  0x0020,   /* U+28EC */
  0x0020,   /* U+28ED */
  0x0020,   /* U+28EE */
  0x0020,   /* U+28EF */
  0x0020,   /* U+28F0 */
  0x0020,   /* U+28F1 */
  0x0020,   /* U+28F2 */
  0x0020,   /* U+28F3 */
  0x0020,   /* U+28F4 */
  0x0020,   /* U+28F5 */
  0x0020,   /* U+28F6 */
  0x0020,   /* U+28F7 */
  0x0020,   /* U+28F8 */
  0x0020,   /* U+28F9 */
  0x0020,   /* U+28FA */
  0x0020,   /* U+28FB */
  0x0020,   /* U+28FC */
  0x0020,   /* U+28FD */
  0x0020,   /* U+28FE */
  0x0020,   /* U+28FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2800 */
  0x0002,   /* U+2801 */
  0x0002,   /* U+2802 */
  0x0002,   /* U+2803 */
  0x0002,   /* U+2804 */
  0x0002,   /* U+2805 */
  0x0002,   /* U+2806 */
  0x0002,   /* U+2807 */
  0x0002,   /* U+2808 */
  0x0002,   /* U+2809 */
  0x0002,   /* U+280A */
  0x0002,   /* U+280B */
  0x0002,   /* U+280C */
  0x0002,   /* U+280D */
  0x0002,   /* U+280E */
  0x0002,   /* U+280F */
  0x0002,   /* U+2810 */
  0x0002,   /* U+2811 */
  0x0002,   /* U+2812 */
  0x0002,   /* U+2813 */
  0x0002,   /* U+2814 */
  0x0002,   /* U+2815 */
  0x0002,   /* U+2816 */
  0x0002,   /* U+2817 */
  0x0002,   /* U+2818 */
  0x0002,   /* U+2819 */
  0x0002,   /* U+281A */
  0x0002,   /* U+281B */
  0x0002,   /* U+281C */
  0x0002,   /* U+281D */
  0x0002,   /* U+281E */
  0x0002,   /* U+281F */
  0x0002,   /* U+2820 */
  0x0002,   /* U+2821 */
  0x0002,   /* U+2822 */
  0x0002,   /* U+2823 */
  0x0002,   /* U+2824 */
  0x0002,   /* U+2825 */
  0x0002,   /* U+2826 */
  0x0002,   /* U+2827 */
  0x0002,   /* U+2828 */
  0x0002,   /* U+2829 */
  0x0002,   /* U+282A */
  0x0002,   /* U+282B */
  0x0002,   /* U+282C */
  0x0002,   /* U+282D */
  0x0002,   /* U+282E */
  0x0002,   /* U+282F */
  0x0002,   /* U+2830 */
  0x0002,   /* U+2831 */
  0x0002,   /* U+2832 */
  0x0002,   /* U+2833 */
  0x0002,   /* U+2834 */
  0x0002,   /* U+2835 */
  0x0002,   /* U+2836 */
  0x0002,   /* U+2837 */
  0x0002,   /* U+2838 */
  0x0002,   /* U+2839 */
  0x0002,   /* U+283A */
  0x0002,   /* U+283B */
  0x0002,   /* U+283C */
  0x0002,   /* U+283D */
  0x0002,   /* U+283E */
  0x0002,   /* U+283F */
  0x0002,   /* U+2840 */
  0x0002,   /* U+2841 */
  0x0002,   /* U+2842 */
  0x0002,   /* U+2843 */
  0x0002,   /* U+2844 */
  0x0002,   /* U+2845 */
  0x0002,   /* U+2846 */
  0x0002,   /* U+2847 */
  0x0002,   /* U+2848 */
  0x0002,   /* U+2849 */
  0x0002,   /* U+284A */
  0x0002,   /* U+284B */
  0x0002,   /* U+284C */
  0x0002,   /* U+284D */
  0x0002,   /* U+284E */
  0x0002,   /* U+284F */
  0x0002,   /* U+2850 */
  0x0002,   /* U+2851 */
  0x0002,   /* U+2852 */
  0x0002,   /* U+2853 */
  0x0002,   /* U+2854 */
  0x0002,   /* U+2855 */
  0x0002,   /* U+2856 */
  0x0002,   /* U+2857 */
  0x0002,   /* U+2858 */
  0x0002,   /* U+2859 */
  0x0002,   /* U+285A */
  0x0002,   /* U+285B */
  0x0002,   /* U+285C */
  0x0002,   /* U+285D */
  0x0002,   /* U+285E */
  0x0002,   /* U+285F */
  0x0002,   /* U+2860 */
  0x0002,   /* U+2861 */
  0x0002,   /* U+2862 */
  0x0002,   /* U+2863 */
  0x0002,   /* U+2864 */
  0x0002,   /* U+2865 */
  0x0002,   /* U+2866 */
  0x0002,   /* U+2867 */
  0x0002,   /* U+2868 */
  0x0002,   /* U+2869 */
  0x0002,   /* U+286A */
  0x0002,   /* U+286B */
  0x0002,   /* U+286C */
  0x0002,   /* U+286D */
  0x0002,   /* U+286E */
  0x0002,   /* U+286F */
  0x0002,   /* U+2870 */
  0x0002,   /* U+2871 */
  0x0002,   /* U+2872 */
  0x0002,   /* U+2873 */
  0x0002,   /* U+2874 */
  0x0002,   /* U+2875 */
  0x0002,   /* U+2876 */
  0x0002,   /* U+2877 */
  0x0002,   /* U+2878 */
  0x0002,   /* U+2879 */
  0x0002,   /* U+287A */
  0x0002,   /* U+287B */
  0x0002,   /* U+287C */
  0x0002,   /* U+287D */
  0x0002,   /* U+287E */
  0x0002,   /* U+287F */
  0x0002,   /* U+2880 */
  0x0002,   /* U+2881 */
  0x0002,   /* U+2882 */
  0x0002,   /* U+2883 */
  0x0002,   /* U+2884 */
  0x0002,   /* U+2885 */
  0x0002,   /* U+2886 */
  0x0002,   /* U+2887 */
  0x0002,   /* U+2888 */
  0x0002,   /* U+2889 */
  0x0002,   /* U+288A */
  0x0002,   /* U+288B */
  0x0002,   /* U+288C */
  0x0002,   /* U+288D */
  0x0002,   /* U+288E */
  0x0002,   /* U+288F */
  0x0002,   /* U+2890 */
  0x0002,   /* U+2891 */
  0x0002,   /* U+2892 */
  0x0002,   /* U+2893 */
  0x0002,   /* U+2894 */
  0x0002,   /* U+2895 */
  0x0002,   /* U+2896 */
  0x0002,   /* U+2897 */
  0x0002,   /* U+2898 */
  0x0002,   /* U+2899 */
  0x0002,   /* U+289A */
  0x0002,   /* U+289B */
  0x0002,   /* U+289C */
  0x0002,   /* U+289D */
  0x0002,   /* U+289E */
  0x0002,   /* U+289F */
  0x0002,   /* U+28A0 */
  0x0002,   /* U+28A1 */
  0x0002,   /* U+28A2 */
  0x0002,   /* U+28A3 */
  0x0002,   /* U+28A4 */
  0x0002,   /* U+28A5 */
  0x0002,   /* U+28A6 */
  0x0002,   /* U+28A7 */
  0x0002,   /* U+28A8 */
  0x0002,   /* U+28A9 */
  0x0002,   /* U+28AA */
  0x0002,   /* U+28AB */
  0x0002,   /* U+28AC */
  0x0002,   /* U+28AD */
  0x0002,   /* U+28AE */
  0x0002,   /* U+28AF */
  0x0002,   /* U+28B0 */
  0x0002,   /* U+28B1 */
  0x0002,   /* U+28B2 */
  0x0002,   /* U+28B3 */
  0x0002,   /* U+28B4 */
  0x0002,   /* U+28B5 */
  0x0002,   /* U+28B6 */
  0x0002,   /* U+28B7 */
  0x0002,   /* U+28B8 */
  0x0002,   /* U+28B9 */
  0x0002,   /* U+28BA */
  0x0002,   /* U+28BB */
  0x0002,   /* U+28BC */
  0x0002,   /* U+28BD */
  0x0002,   /* U+28BE */
  0x0002,   /* U+28BF */
  0x0002,   /* U+28C0 */
  0x0002,   /* U+28C1 */
  0x0002,   /* U+28C2 */
  0x0002,   /* U+28C3 */
  0x0002,   /* U+28C4 */
  0x0002,   /* U+28C5 */
  0x0002,   /* U+28C6 */
  0x0002,   /* U+28C7 */
  0x0002,   /* U+28C8 */
  0x0002,   /* U+28C9 */
  0x0002,   /* U+28CA */
  0x0002,   /* U+28CB */
  0x0002,   /* U+28CC */
  0x0002,   /* U+28CD */
  0x0002,   /* U+28CE */
  0x0002,   /* U+28CF */
  0x0002,   /* U+28D0 */
  0x0002,   /* U+28D1 */
  0x0002,   /* U+28D2 */
  0x0002,   /* U+28D3 */
  0x0002,   /* U+28D4 */
  0x0002,   /* U+28D5 */
  0x0002,   /* U+28D6 */
  0x0002,   /* U+28D7 */
  0x0002,   /* U+28D8 */
  0x0002,   /* U+28D9 */
  0x0002,   /* U+28DA */
  0x0002,   /* U+28DB */
  0x0002,   /* U+28DC */
  0x0002,   /* U+28DD */
  0x0002,   /* U+28DE */
  0x0002,   /* U+28DF */
  0x0002,   /* U+28E0 */
  0x0002,   /* U+28E1 */
  0x0002,   /* U+28E2 */
  0x0002,   /* U+28E3 */
  0x0002,   /* U+28E4 */
  0x0002,   /* U+28E5 */
  0x0002,   /* U+28E6 */
  0x0002,   /* U+28E7 */
  0x0002,   /* U+28E8 */
  0x0002,   /* U+28E9 */
  0x0002,   /* U+28EA */
  0x0002,   /* U+28EB */
  0x0002,   /* U+28EC */
  0x0002,   /* U+28ED */
  0x0002,   /* U+28EE */
  0x0002,   /* U+28EF */
  0x0002,   /* U+28F0 */
  0x0002,   /* U+28F1 */
  0x0002,   /* U+28F2 */
  0x0002,   /* U+28F3 */
  0x0002,   /* U+28F4 */
  0x0002,   /* U+28F5 */
  0x0002,   /* U+28F6 */
  0x0002,   /* U+28F7 */
  0x0002,   /* U+28F8 */
  0x0002,   /* U+28F9 */
  0x0002,   /* U+28FA */
  0x0002,   /* U+28FB */
  0x0002,   /* U+28FC */
  0x0002,   /* U+28FD */
  0x0002,   /* U+28FE */
  0x0002,   /* U+28FF */
};

uint16 uca900_p029[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0AE9,   /* U+2900 */
  0x0AEA,   /* U+2901 */
  0x0AEB,   /* U+2902 */
  0x0AEC,   /* U+2903 */
  0x0AED,   /* U+2904 */
  0x0AEE,   /* U+2905 */
  0x0AEF,   /* U+2906 */
  0x0AF0,   /* U+2907 */
  0x0AF1,   /* U+2908 */
  0x0AF2,   /* U+2909 */
  0x0AF3,   /* U+290A */
  0x0AF4,   /* U+290B */
  0x0AF5,   /* U+290C */
  0x0AF6,   /* U+290D */
  0x0AF7,   /* U+290E */
  0x0AF8,   /* U+290F */
  0x0AF9,   /* U+2910 */
  0x0AFA,   /* U+2911 */
  0x0AFB,   /* U+2912 */
  0x0AFC,   /* U+2913 */
  0x0AFD,   /* U+2914 */
  0x0AFE,   /* U+2915 */
  0x0AFF,   /* U+2916 */
  0x0B00,   /* U+2917 */
  0x0B01,   /* U+2918 */
  0x0B02,   /* U+2919 */
  0x0B03,   /* U+291A */
  0x0B04,   /* U+291B */
  0x0B05,   /* U+291C */
  0x0B06,   /* U+291D */
  0x0B07,   /* U+291E */
  0x0B08,   /* U+291F */
  0x0B09,   /* U+2920 */
  0x0B0A,   /* U+2921 */
  0x0B0B,   /* U+2922 */
  0x0B0C,   /* U+2923 */
  0x0B0D,   /* U+2924 */
  0x0B0E,   /* U+2925 */
  0x0B0F,   /* U+2926 */
  0x0B10,   /* U+2927 */
  0x0B11,   /* U+2928 */
  0x0B12,   /* U+2929 */
  0x0B13,   /* U+292A */
  0x0B14,   /* U+292B */
  0x0B15,   /* U+292C */
  0x0B16,   /* U+292D */
  0x0B17,   /* U+292E */
  0x0B18,   /* U+292F */
  0x0B19,   /* U+2930 */
  0x0B1A,   /* U+2931 */
  0x0B1B,   /* U+2932 */
  0x0B1C,   /* U+2933 */
  0x0B1D,   /* U+2934 */
  0x0B1E,   /* U+2935 */
  0x0B1F,   /* U+2936 */
  0x0B20,   /* U+2937 */
  0x0B21,   /* U+2938 */
  0x0B22,   /* U+2939 */
  0x0B23,   /* U+293A */
  0x0B24,   /* U+293B */
  0x0B25,   /* U+293C */
  0x0B26,   /* U+293D */
  0x0B27,   /* U+293E */
  0x0B28,   /* U+293F */
  0x0B29,   /* U+2940 */
  0x0B2A,   /* U+2941 */
  0x0B2B,   /* U+2942 */
  0x0B2C,   /* U+2943 */
  0x0B2D,   /* U+2944 */
  0x0B2E,   /* U+2945 */
  0x0B2F,   /* U+2946 */
  0x0B30,   /* U+2947 */
  0x0B31,   /* U+2948 */
  0x0B32,   /* U+2949 */
  0x0B33,   /* U+294A */
  0x0B34,   /* U+294B */
  0x0B35,   /* U+294C */
  0x0B36,   /* U+294D */
  0x0B37,   /* U+294E */
  0x0B38,   /* U+294F */
  0x0B39,   /* U+2950 */
  0x0B3A,   /* U+2951 */
  0x0B3B,   /* U+2952 */
  0x0B3C,   /* U+2953 */
  0x0B3D,   /* U+2954 */
  0x0B3E,   /* U+2955 */
  0x0B3F,   /* U+2956 */
  0x0B40,   /* U+2957 */
  0x0B41,   /* U+2958 */
  0x0B42,   /* U+2959 */
  0x0B43,   /* U+295A */
  0x0B44,   /* U+295B */
  0x0B45,   /* U+295C */
  0x0B46,   /* U+295D */
  0x0B47,   /* U+295E */
  0x0B48,   /* U+295F */
  0x0B49,   /* U+2960 */
  0x0B4A,   /* U+2961 */
  0x0B4B,   /* U+2962 */
  0x0B4C,   /* U+2963 */
  0x0B4D,   /* U+2964 */
  0x0B4E,   /* U+2965 */
  0x0B4F,   /* U+2966 */
  0x0B50,   /* U+2967 */
  0x0B51,   /* U+2968 */
  0x0B52,   /* U+2969 */
  0x0B53,   /* U+296A */
  0x0B54,   /* U+296B */
  0x0B55,   /* U+296C */
  0x0B56,   /* U+296D */
  0x0B57,   /* U+296E */
  0x0B58,   /* U+296F */
  0x0B59,   /* U+2970 */
  0x0B5A,   /* U+2971 */
  0x0B5B,   /* U+2972 */
  0x0B5C,   /* U+2973 */
  0x0B5D,   /* U+2974 */
  0x0B5E,   /* U+2975 */
  0x0B5F,   /* U+2976 */
  0x0B60,   /* U+2977 */
  0x0B61,   /* U+2978 */
  0x0B62,   /* U+2979 */
  0x0B63,   /* U+297A */
  0x0B64,   /* U+297B */
  0x0B65,   /* U+297C */
  0x0B66,   /* U+297D */
  0x0B67,   /* U+297E */
  0x0B68,   /* U+297F */
  0x0B69,   /* U+2980 */
  0x0B6A,   /* U+2981 */
  0x0B6B,   /* U+2982 */
  0x032B,   /* U+2983 */
  0x032C,   /* U+2984 */
  0x032D,   /* U+2985 */
  0x032E,   /* U+2986 */
  0x032F,   /* U+2987 */
  0x0330,   /* U+2988 */
  0x0331,   /* U+2989 */
  0x0332,   /* U+298A */
  0x0333,   /* U+298B */
  0x0334,   /* U+298C */
  0x0335,   /* U+298D */
  0x0336,   /* U+298E */
  0x0337,   /* U+298F */
  0x0338,   /* U+2990 */
  0x0339,   /* U+2991 */
  0x033A,   /* U+2992 */
  0x033B,   /* U+2993 */
  0x033C,   /* U+2994 */
  0x033D,   /* U+2995 */
  0x033E,   /* U+2996 */
  0x033F,   /* U+2997 */
  0x0340,   /* U+2998 */
  0x0B6C,   /* U+2999 */
  0x0B6D,   /* U+299A */
  0x0B6E,   /* U+299B */
  0x0B6F,   /* U+299C */
  0x0B70,   /* U+299D */
  0x0B71,   /* U+299E */
  0x0B72,   /* U+299F */
  0x0B73,   /* U+29A0 */
  0x0B74,   /* U+29A1 */
  0x0B75,   /* U+29A2 */
  0x0B76,   /* U+29A3 */
  0x0B77,   /* U+29A4 */
  0x0B78,   /* U+29A5 */
  0x0B79,   /* U+29A6 */
  0x0B7A,   /* U+29A7 */
  0x0B7B,   /* U+29A8 */
  0x0B7C,   /* U+29A9 */
  0x0B7D,   /* U+29AA */
  0x0B7E,   /* U+29AB */
  0x0B7F,   /* U+29AC */
  0x0B80,   /* U+29AD */
  0x0B81,   /* U+29AE */
  0x0B82,   /* U+29AF */
  0x0B83,   /* U+29B0 */
  0x0B84,   /* U+29B1 */
  0x0B85,   /* U+29B2 */
  0x0B86,   /* U+29B3 */
  0x0B87,   /* U+29B4 */
  0x0B88,   /* U+29B5 */
  0x0B89,   /* U+29B6 */
  0x0B8A,   /* U+29B7 */
  0x0B8B,   /* U+29B8 */
  0x0B8C,   /* U+29B9 */
  0x0B8D,   /* U+29BA */
  0x0B8E,   /* U+29BB */
  0x0B8F,   /* U+29BC */
  0x0B90,   /* U+29BD */
  0x0B91,   /* U+29BE */
  0x0B92,   /* U+29BF */
  0x0B93,   /* U+29C0 */
  0x0B94,   /* U+29C1 */
  0x0B95,   /* U+29C2 */
  0x0B96,   /* U+29C3 */
  0x0B97,   /* U+29C4 */
  0x0B98,   /* U+29C5 */
  0x0B99,   /* U+29C6 */
  0x0B9A,   /* U+29C7 */
  0x0B9B,   /* U+29C8 */
  0x0B9C,   /* U+29C9 */
  0x0B9D,   /* U+29CA */
  0x0B9E,   /* U+29CB */
  0x0B9F,   /* U+29CC */
  0x0BA0,   /* U+29CD */
  0x0BA1,   /* U+29CE */
  0x0BA2,   /* U+29CF */
  0x0BA3,   /* U+29D0 */
  0x0BA4,   /* U+29D1 */
  0x0BA5,   /* U+29D2 */
  0x0BA6,   /* U+29D3 */
  0x0BA7,   /* U+29D4 */
  0x0BA8,   /* U+29D5 */
  0x0BA9,   /* U+29D6 */
  0x0BAA,   /* U+29D7 */
  0x0385,   /* U+29D8 */
  0x0386,   /* U+29D9 */
  0x0387,   /* U+29DA */
  0x0388,   /* U+29DB */
  0x0BAB,   /* U+29DC */
  0x0BAC,   /* U+29DD */
  0x0BAD,   /* U+29DE */
  0x0BAE,   /* U+29DF */
  0x0BAF,   /* U+29E0 */
  0x0BB0,   /* U+29E1 */
  0x0BB1,   /* U+29E2 */
  0x0BB2,   /* U+29E3 */
  0x0BB3,   /* U+29E4 */
  0x0BB4,   /* U+29E5 */
  0x0BB5,   /* U+29E6 */
  0x0BB6,   /* U+29E7 */
  0x0BB7,   /* U+29E8 */
  0x0BB8,   /* U+29E9 */
  0x0BB9,   /* U+29EA */
  0x0BBA,   /* U+29EB */
  0x0BBB,   /* U+29EC */
  0x0BBC,   /* U+29ED */
  0x0BBD,   /* U+29EE */
  0x0BBE,   /* U+29EF */
  0x0BBF,   /* U+29F0 */
  0x0BC0,   /* U+29F1 */
  0x0BC1,   /* U+29F2 */
  0x0BC2,   /* U+29F3 */
  0x0BC3,   /* U+29F4 */
  0x0BC4,   /* U+29F5 */
  0x0BC5,   /* U+29F6 */
  0x0BC6,   /* U+29F7 */
  0x0BC7,   /* U+29F8 */
  0x0BC8,   /* U+29F9 */
  0x0BC9,   /* U+29FA */
  0x0BCA,   /* U+29FB */
  0x0329,   /* U+29FC */
  0x032A,   /* U+29FD */
  0x0BCB,   /* U+29FE */
  0x0BCC,   /* U+29FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2900 */
  0x0020,   /* U+2901 */
  0x0020,   /* U+2902 */
  0x0020,   /* U+2903 */
  0x0020,   /* U+2904 */
  0x0020,   /* U+2905 */
  0x0020,   /* U+2906 */
  0x0020,   /* U+2907 */
  0x0020,   /* U+2908 */
  0x0020,   /* U+2909 */
  0x0020,   /* U+290A */
  0x0020,   /* U+290B */
  0x0020,   /* U+290C */
  0x0020,   /* U+290D */
  0x0020,   /* U+290E */
  0x0020,   /* U+290F */
  0x0020,   /* U+2910 */
  0x0020,   /* U+2911 */
  0x0020,   /* U+2912 */
  0x0020,   /* U+2913 */
  0x0020,   /* U+2914 */
  0x0020,   /* U+2915 */
  0x0020,   /* U+2916 */
  0x0020,   /* U+2917 */
  0x0020,   /* U+2918 */
  0x0020,   /* U+2919 */
  0x0020,   /* U+291A */
  0x0020,   /* U+291B */
  0x0020,   /* U+291C */
  0x0020,   /* U+291D */
  0x0020,   /* U+291E */
  0x0020,   /* U+291F */
  0x0020,   /* U+2920 */
  0x0020,   /* U+2921 */
  0x0020,   /* U+2922 */
  0x0020,   /* U+2923 */
  0x0020,   /* U+2924 */
  0x0020,   /* U+2925 */
  0x0020,   /* U+2926 */
  0x0020,   /* U+2927 */
  0x0020,   /* U+2928 */
  0x0020,   /* U+2929 */
  0x0020,   /* U+292A */
  0x0020,   /* U+292B */
  0x0020,   /* U+292C */
  0x0020,   /* U+292D */
  0x0020,   /* U+292E */
  0x0020,   /* U+292F */
  0x0020,   /* U+2930 */
  0x0020,   /* U+2931 */
  0x0020,   /* U+2932 */
  0x0020,   /* U+2933 */
  0x0020,   /* U+2934 */
  0x0020,   /* U+2935 */
  0x0020,   /* U+2936 */
  0x0020,   /* U+2937 */
  0x0020,   /* U+2938 */
  0x0020,   /* U+2939 */
  0x0020,   /* U+293A */
  0x0020,   /* U+293B */
  0x0020,   /* U+293C */
  0x0020,   /* U+293D */
  0x0020,   /* U+293E */
  0x0020,   /* U+293F */
  0x0020,   /* U+2940 */
  0x0020,   /* U+2941 */
  0x0020,   /* U+2942 */
  0x0020,   /* U+2943 */
  0x0020,   /* U+2944 */
  0x0020,   /* U+2945 */
  0x0020,   /* U+2946 */
  0x0020,   /* U+2947 */
  0x0020,   /* U+2948 */
  0x0020,   /* U+2949 */
  0x0020,   /* U+294A */
  0x0020,   /* U+294B */
  0x0020,   /* U+294C */
  0x0020,   /* U+294D */
  0x0020,   /* U+294E */
  0x0020,   /* U+294F */
  0x0020,   /* U+2950 */
  0x0020,   /* U+2951 */
  0x0020,   /* U+2952 */
  0x0020,   /* U+2953 */
  0x0020,   /* U+2954 */
  0x0020,   /* U+2955 */
  0x0020,   /* U+2956 */
  0x0020,   /* U+2957 */
  0x0020,   /* U+2958 */
  0x0020,   /* U+2959 */
  0x0020,   /* U+295A */
  0x0020,   /* U+295B */
  0x0020,   /* U+295C */
  0x0020,   /* U+295D */
  0x0020,   /* U+295E */
  0x0020,   /* U+295F */
  0x0020,   /* U+2960 */
  0x0020,   /* U+2961 */
  0x0020,   /* U+2962 */
  0x0020,   /* U+2963 */
  0x0020,   /* U+2964 */
  0x0020,   /* U+2965 */
  0x0020,   /* U+2966 */
  0x0020,   /* U+2967 */
  0x0020,   /* U+2968 */
  0x0020,   /* U+2969 */
  0x0020,   /* U+296A */
  0x0020,   /* U+296B */
  0x0020,   /* U+296C */
  0x0020,   /* U+296D */
  0x0020,   /* U+296E */
  0x0020,   /* U+296F */
  0x0020,   /* U+2970 */
  0x0020,   /* U+2971 */
  0x0020,   /* U+2972 */
  0x0020,   /* U+2973 */
  0x0020,   /* U+2974 */
  0x0020,   /* U+2975 */
  0x0020,   /* U+2976 */
  0x0020,   /* U+2977 */
  0x0020,   /* U+2978 */
  0x0020,   /* U+2979 */
  0x0020,   /* U+297A */
  0x0020,   /* U+297B */
  0x0020,   /* U+297C */
  0x0020,   /* U+297D */
  0x0020,   /* U+297E */
  0x0020,   /* U+297F */
  0x0020,   /* U+2980 */
  0x0020,   /* U+2981 */
  0x0020,   /* U+2982 */
  0x0020,   /* U+2983 */
  0x0020,   /* U+2984 */
  0x0020,   /* U+2985 */
  0x0020,   /* U+2986 */
  0x0020,   /* U+2987 */
  0x0020,   /* U+2988 */
  0x0020,   /* U+2989 */
  0x0020,   /* U+298A */
  0x0020,   /* U+298B */
  0x0020,   /* U+298C */
  0x0020,   /* U+298D */
  0x0020,   /* U+298E */
  0x0020,   /* U+298F */
  0x0020,   /* U+2990 */
  0x0020,   /* U+2991 */
  0x0020,   /* U+2992 */
  0x0020,   /* U+2993 */
  0x0020,   /* U+2994 */
  0x0020,   /* U+2995 */
  0x0020,   /* U+2996 */
  0x0020,   /* U+2997 */
  0x0020,   /* U+2998 */
  0x0020,   /* U+2999 */
  0x0020,   /* U+299A */
  0x0020,   /* U+299B */
  0x0020,   /* U+299C */
  0x0020,   /* U+299D */
  0x0020,   /* U+299E */
  0x0020,   /* U+299F */
  0x0020,   /* U+29A0 */
  0x0020,   /* U+29A1 */
  0x0020,   /* U+29A2 */
  0x0020,   /* U+29A3 */
  0x0020,   /* U+29A4 */
  0x0020,   /* U+29A5 */
  0x0020,   /* U+29A6 */
  0x0020,   /* U+29A7 */
  0x0020,   /* U+29A8 */
  0x0020,   /* U+29A9 */
  0x0020,   /* U+29AA */
  0x0020,   /* U+29AB */
  0x0020,   /* U+29AC */
  0x0020,   /* U+29AD */
  0x0020,   /* U+29AE */
  0x0020,   /* U+29AF */
  0x0020,   /* U+29B0 */
  0x0020,   /* U+29B1 */
  0x0020,   /* U+29B2 */
  0x0020,   /* U+29B3 */
  0x0020,   /* U+29B4 */
  0x0020,   /* U+29B5 */
  0x0020,   /* U+29B6 */
  0x0020,   /* U+29B7 */
  0x0020,   /* U+29B8 */
  0x0020,   /* U+29B9 */
  0x0020,   /* U+29BA */
  0x0020,   /* U+29BB */
  0x0020,   /* U+29BC */
  0x0020,   /* U+29BD */
  0x0020,   /* U+29BE */
  0x0020,   /* U+29BF */
  0x0020,   /* U+29C0 */
  0x0020,   /* U+29C1 */
  0x0020,   /* U+29C2 */
  0x0020,   /* U+29C3 */
  0x0020,   /* U+29C4 */
  0x0020,   /* U+29C5 */
  0x0020,   /* U+29C6 */
  0x0020,   /* U+29C7 */
  0x0020,   /* U+29C8 */
  0x0020,   /* U+29C9 */
  0x0020,   /* U+29CA */
  0x0020,   /* U+29CB */
  0x0020,   /* U+29CC */
  0x0020,   /* U+29CD */
  0x0020,   /* U+29CE */
  0x0020,   /* U+29CF */
  0x0020,   /* U+29D0 */
  0x0020,   /* U+29D1 */
  0x0020,   /* U+29D2 */
  0x0020,   /* U+29D3 */
  0x0020,   /* U+29D4 */
  0x0020,   /* U+29D5 */
  0x0020,   /* U+29D6 */
  0x0020,   /* U+29D7 */
  0x0020,   /* U+29D8 */
  0x0020,   /* U+29D9 */
  0x0020,   /* U+29DA */
  0x0020,   /* U+29DB */
  0x0020,   /* U+29DC */
  0x0020,   /* U+29DD */
  0x0020,   /* U+29DE */
  0x0020,   /* U+29DF */
  0x0020,   /* U+29E0 */
  0x0020,   /* U+29E1 */
  0x0020,   /* U+29E2 */
  0x0020,   /* U+29E3 */
  0x0020,   /* U+29E4 */
  0x0020,   /* U+29E5 */
  0x0020,   /* U+29E6 */
  0x0020,   /* U+29E7 */
  0x0020,   /* U+29E8 */
  0x0020,   /* U+29E9 */
  0x0020,   /* U+29EA */
  0x0020,   /* U+29EB */
  0x0020,   /* U+29EC */
  0x0020,   /* U+29ED */
  0x0020,   /* U+29EE */
  0x0020,   /* U+29EF */
  0x0020,   /* U+29F0 */
  0x0020,   /* U+29F1 */
  0x0020,   /* U+29F2 */
  0x0020,   /* U+29F3 */
  0x0020,   /* U+29F4 */
  0x0020,   /* U+29F5 */
  0x0020,   /* U+29F6 */
  0x0020,   /* U+29F7 */
  0x0020,   /* U+29F8 */
  0x0020,   /* U+29F9 */
  0x0020,   /* U+29FA */
  0x0020,   /* U+29FB */
  0x0020,   /* U+29FC */
  0x0020,   /* U+29FD */
  0x0020,   /* U+29FE */
  0x0020,   /* U+29FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2900 */
  0x0002,   /* U+2901 */
  0x0002,   /* U+2902 */
  0x0002,   /* U+2903 */
  0x0002,   /* U+2904 */
  0x0002,   /* U+2905 */
  0x0002,   /* U+2906 */
  0x0002,   /* U+2907 */
  0x0002,   /* U+2908 */
  0x0002,   /* U+2909 */
  0x0002,   /* U+290A */
  0x0002,   /* U+290B */
  0x0002,   /* U+290C */
  0x0002,   /* U+290D */
  0x0002,   /* U+290E */
  0x0002,   /* U+290F */
  0x0002,   /* U+2910 */
  0x0002,   /* U+2911 */
  0x0002,   /* U+2912 */
  0x0002,   /* U+2913 */
  0x0002,   /* U+2914 */
  0x0002,   /* U+2915 */
  0x0002,   /* U+2916 */
  0x0002,   /* U+2917 */
  0x0002,   /* U+2918 */
  0x0002,   /* U+2919 */
  0x0002,   /* U+291A */
  0x0002,   /* U+291B */
  0x0002,   /* U+291C */
  0x0002,   /* U+291D */
  0x0002,   /* U+291E */
  0x0002,   /* U+291F */
  0x0002,   /* U+2920 */
  0x0002,   /* U+2921 */
  0x0002,   /* U+2922 */
  0x0002,   /* U+2923 */
  0x0002,   /* U+2924 */
  0x0002,   /* U+2925 */
  0x0002,   /* U+2926 */
  0x0002,   /* U+2927 */
  0x0002,   /* U+2928 */
  0x0002,   /* U+2929 */
  0x0002,   /* U+292A */
  0x0002,   /* U+292B */
  0x0002,   /* U+292C */
  0x0002,   /* U+292D */
  0x0002,   /* U+292E */
  0x0002,   /* U+292F */
  0x0002,   /* U+2930 */
  0x0002,   /* U+2931 */
  0x0002,   /* U+2932 */
  0x0002,   /* U+2933 */
  0x0002,   /* U+2934 */
  0x0002,   /* U+2935 */
  0x0002,   /* U+2936 */
  0x0002,   /* U+2937 */
  0x0002,   /* U+2938 */
  0x0002,   /* U+2939 */
  0x0002,   /* U+293A */
  0x0002,   /* U+293B */
  0x0002,   /* U+293C */
  0x0002,   /* U+293D */
  0x0002,   /* U+293E */
  0x0002,   /* U+293F */
  0x0002,   /* U+2940 */
  0x0002,   /* U+2941 */
  0x0002,   /* U+2942 */
  0x0002,   /* U+2943 */
  0x0002,   /* U+2944 */
  0x0002,   /* U+2945 */
  0x0002,   /* U+2946 */
  0x0002,   /* U+2947 */
  0x0002,   /* U+2948 */
  0x0002,   /* U+2949 */
  0x0002,   /* U+294A */
  0x0002,   /* U+294B */
  0x0002,   /* U+294C */
  0x0002,   /* U+294D */
  0x0002,   /* U+294E */
  0x0002,   /* U+294F */
  0x0002,   /* U+2950 */
  0x0002,   /* U+2951 */
  0x0002,   /* U+2952 */
  0x0002,   /* U+2953 */
  0x0002,   /* U+2954 */
  0x0002,   /* U+2955 */
  0x0002,   /* U+2956 */
  0x0002,   /* U+2957 */
  0x0002,   /* U+2958 */
  0x0002,   /* U+2959 */
  0x0002,   /* U+295A */
  0x0002,   /* U+295B */
  0x0002,   /* U+295C */
  0x0002,   /* U+295D */
  0x0002,   /* U+295E */
  0x0002,   /* U+295F */
  0x0002,   /* U+2960 */
  0x0002,   /* U+2961 */
  0x0002,   /* U+2962 */
  0x0002,   /* U+2963 */
  0x0002,   /* U+2964 */
  0x0002,   /* U+2965 */
  0x0002,   /* U+2966 */
  0x0002,   /* U+2967 */
  0x0002,   /* U+2968 */
  0x0002,   /* U+2969 */
  0x0002,   /* U+296A */
  0x0002,   /* U+296B */
  0x0002,   /* U+296C */
  0x0002,   /* U+296D */
  0x0002,   /* U+296E */
  0x0002,   /* U+296F */
  0x0002,   /* U+2970 */
  0x0002,   /* U+2971 */
  0x0002,   /* U+2972 */
  0x0002,   /* U+2973 */
  0x0002,   /* U+2974 */
  0x0002,   /* U+2975 */
  0x0002,   /* U+2976 */
  0x0002,   /* U+2977 */
  0x0002,   /* U+2978 */
  0x0002,   /* U+2979 */
  0x0002,   /* U+297A */
  0x0002,   /* U+297B */
  0x0002,   /* U+297C */
  0x0002,   /* U+297D */
  0x0002,   /* U+297E */
  0x0002,   /* U+297F */
  0x0002,   /* U+2980 */
  0x0002,   /* U+2981 */
  0x0002,   /* U+2982 */
  0x0002,   /* U+2983 */
  0x0002,   /* U+2984 */
  0x0002,   /* U+2985 */
  0x0002,   /* U+2986 */
  0x0002,   /* U+2987 */
  0x0002,   /* U+2988 */
  0x0002,   /* U+2989 */
  0x0002,   /* U+298A */
  0x0002,   /* U+298B */
  0x0002,   /* U+298C */
  0x0002,   /* U+298D */
  0x0002,   /* U+298E */
  0x0002,   /* U+298F */
  0x0002,   /* U+2990 */
  0x0002,   /* U+2991 */
  0x0002,   /* U+2992 */
  0x0002,   /* U+2993 */
  0x0002,   /* U+2994 */
  0x0002,   /* U+2995 */
  0x0002,   /* U+2996 */
  0x0002,   /* U+2997 */
  0x0002,   /* U+2998 */
  0x0002,   /* U+2999 */
  0x0002,   /* U+299A */
  0x0002,   /* U+299B */
  0x0002,   /* U+299C */
  0x0002,   /* U+299D */
  0x0002,   /* U+299E */
  0x0002,   /* U+299F */
  0x0002,   /* U+29A0 */
  0x0002,   /* U+29A1 */
  0x0002,   /* U+29A2 */
  0x0002,   /* U+29A3 */
  0x0002,   /* U+29A4 */
  0x0002,   /* U+29A5 */
  0x0002,   /* U+29A6 */
  0x0002,   /* U+29A7 */
  0x0002,   /* U+29A8 */
  0x0002,   /* U+29A9 */
  0x0002,   /* U+29AA */
  0x0002,   /* U+29AB */
  0x0002,   /* U+29AC */
  0x0002,   /* U+29AD */
  0x0002,   /* U+29AE */
  0x0002,   /* U+29AF */
  0x0002,   /* U+29B0 */
  0x0002,   /* U+29B1 */
  0x0002,   /* U+29B2 */
  0x0002,   /* U+29B3 */
  0x0002,   /* U+29B4 */
  0x0002,   /* U+29B5 */
  0x0002,   /* U+29B6 */
  0x0002,   /* U+29B7 */
  0x0002,   /* U+29B8 */
  0x0002,   /* U+29B9 */
  0x0002,   /* U+29BA */
  0x0002,   /* U+29BB */
  0x0002,   /* U+29BC */
  0x0002,   /* U+29BD */
  0x0002,   /* U+29BE */
  0x0002,   /* U+29BF */
  0x0002,   /* U+29C0 */
  0x0002,   /* U+29C1 */
  0x0002,   /* U+29C2 */
  0x0002,   /* U+29C3 */
  0x0002,   /* U+29C4 */
  0x0002,   /* U+29C5 */
  0x0002,   /* U+29C6 */
  0x0002,   /* U+29C7 */
  0x0002,   /* U+29C8 */
  0x0002,   /* U+29C9 */
  0x0002,   /* U+29CA */
  0x0002,   /* U+29CB */
  0x0002,   /* U+29CC */
  0x0002,   /* U+29CD */
  0x0002,   /* U+29CE */
  0x0002,   /* U+29CF */
  0x0002,   /* U+29D0 */
  0x0002,   /* U+29D1 */
  0x0002,   /* U+29D2 */
  0x0002,   /* U+29D3 */
  0x0002,   /* U+29D4 */
  0x0002,   /* U+29D5 */
  0x0002,   /* U+29D6 */
  0x0002,   /* U+29D7 */
  0x0002,   /* U+29D8 */
  0x0002,   /* U+29D9 */
  0x0002,   /* U+29DA */
  0x0002,   /* U+29DB */
  0x0002,   /* U+29DC */
  0x0002,   /* U+29DD */
  0x0002,   /* U+29DE */
  0x0002,   /* U+29DF */
  0x0002,   /* U+29E0 */
  0x0002,   /* U+29E1 */
  0x0002,   /* U+29E2 */
  0x0002,   /* U+29E3 */
  0x0002,   /* U+29E4 */
  0x0002,   /* U+29E5 */
  0x0002,   /* U+29E6 */
  0x0002,   /* U+29E7 */
  0x0002,   /* U+29E8 */
  0x0002,   /* U+29E9 */
  0x0002,   /* U+29EA */
  0x0002,   /* U+29EB */
  0x0002,   /* U+29EC */
  0x0002,   /* U+29ED */
  0x0002,   /* U+29EE */
  0x0002,   /* U+29EF */
  0x0002,   /* U+29F0 */
  0x0002,   /* U+29F1 */
  0x0002,   /* U+29F2 */
  0x0002,   /* U+29F3 */
  0x0002,   /* U+29F4 */
  0x0002,   /* U+29F5 */
  0x0002,   /* U+29F6 */
  0x0002,   /* U+29F7 */
  0x0002,   /* U+29F8 */
  0x0002,   /* U+29F9 */
  0x0002,   /* U+29FA */
  0x0002,   /* U+29FB */
  0x0002,   /* U+29FC */
  0x0002,   /* U+29FD */
  0x0002,   /* U+29FE */
  0x0002,   /* U+29FF */
};

uint16 uca900_p02A[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 4, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 3, 2, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x0BCD,   /* U+2A00 */
  0x0BCE,   /* U+2A01 */
  0x0BCF,   /* U+2A02 */
  0x0BD0,   /* U+2A03 */
  0x0BD1,   /* U+2A04 */
  0x0BD2,   /* U+2A05 */
  0x0BD3,   /* U+2A06 */
  0x0BD4,   /* U+2A07 */
  0x0BD5,   /* U+2A08 */
  0x0BD6,   /* U+2A09 */
  0x0BD7,   /* U+2A0A */
  0x0BD8,   /* U+2A0B */
  0x063C,   /* U+2A0C */
  0x0BD9,   /* U+2A0D */
  0x0BDA,   /* U+2A0E */
  0x0BDB,   /* U+2A0F */
  0x0BDC,   /* U+2A10 */
  0x0BDD,   /* U+2A11 */
  0x0BDE,   /* U+2A12 */
  0x0BDF,   /* U+2A13 */
  0x0BE0,   /* U+2A14 */
  0x0BE1,   /* U+2A15 */
  0x0BE2,   /* U+2A16 */
  0x0BE3,   /* U+2A17 */
  0x0BE4,   /* U+2A18 */
  0x0BE5,   /* U+2A19 */
  0x0BE6,   /* U+2A1A */
  0x0BE7,   /* U+2A1B */
  0x0BE8,   /* U+2A1C */
  0x0BE9,   /* U+2A1D */
  0x0BEA,   /* U+2A1E */
  0x0BEB,   /* U+2A1F */
  0x0BEC,   /* U+2A20 */
  0x0BED,   /* U+2A21 */
  0x0BEE,   /* U+2A22 */
  0x0BEF,   /* U+2A23 */
  0x0BF0,   /* U+2A24 */
  0x0BF1,   /* U+2A25 */
  0x0BF2,   /* U+2A26 */
  0x0BF3,   /* U+2A27 */
  0x0BF4,   /* U+2A28 */
  0x0BF5,   /* U+2A29 */
  0x0BF6,   /* U+2A2A */
  0x0BF7,   /* U+2A2B */
  0x0BF8,   /* U+2A2C */
  0x0BF9,   /* U+2A2D */
  0x0BFA,   /* U+2A2E */
  0x0BFB,   /* U+2A2F */
  0x0BFC,   /* U+2A30 */
  0x0BFD,   /* U+2A31 */
  0x0BFE,   /* U+2A32 */
  0x0BFF,   /* U+2A33 */
  0x0C00,   /* U+2A34 */
  0x0C01,   /* U+2A35 */
  0x0C02,   /* U+2A36 */
  0x0C03,   /* U+2A37 */
  0x0C04,   /* U+2A38 */
  0x0C05,   /* U+2A39 */
  0x0C06,   /* U+2A3A */
  0x0C07,   /* U+2A3B */
  0x0C08,   /* U+2A3C */
  0x0C09,   /* U+2A3D */
  0x0C0A,   /* U+2A3E */
  0x0C0B,   /* U+2A3F */
  0x0C0C,   /* U+2A40 */
  0x0C0D,   /* U+2A41 */
  0x0C0E,   /* U+2A42 */
  0x0C0F,   /* U+2A43 */
  0x0C10,   /* U+2A44 */
  0x0C11,   /* U+2A45 */
  0x0C12,   /* U+2A46 */
  0x0C13,   /* U+2A47 */
  0x0C14,   /* U+2A48 */
  0x0C15,   /* U+2A49 */
  0x0C16,   /* U+2A4A */
  0x0C17,   /* U+2A4B */
  0x0C18,   /* U+2A4C */
  0x0C19,   /* U+2A4D */
  0x0C1A,   /* U+2A4E */
  0x0C1B,   /* U+2A4F */
  0x0C1C,   /* U+2A50 */
  0x0C1D,   /* U+2A51 */
  0x0C1E,   /* U+2A52 */
  0x0C1F,   /* U+2A53 */
  0x0C20,   /* U+2A54 */
  0x0C21,   /* U+2A55 */
  0x0C22,   /* U+2A56 */
  0x0C23,   /* U+2A57 */
  0x0C24,   /* U+2A58 */
  0x0C25,   /* U+2A59 */
  0x0C26,   /* U+2A5A */
  0x0C27,   /* U+2A5B */
  0x0C28,   /* U+2A5C */
  0x0C29,   /* U+2A5D */
  0x0C2A,   /* U+2A5E */
  0x0C2B,   /* U+2A5F */
  0x0C2C,   /* U+2A60 */
  0x0C2D,   /* U+2A61 */
  0x0C2E,   /* U+2A62 */
  0x0C2F,   /* U+2A63 */
  0x0C30,   /* U+2A64 */
  0x0C31,   /* U+2A65 */
  0x0C32,   /* U+2A66 */
  0x0C33,   /* U+2A67 */
  0x0C34,   /* U+2A68 */
  0x0C35,   /* U+2A69 */
  0x0C36,   /* U+2A6A */
  0x0C37,   /* U+2A6B */
  0x0C38,   /* U+2A6C */
  0x0C39,   /* U+2A6D */
  0x0C3A,   /* U+2A6E */
  0x0C3B,   /* U+2A6F */
  0x0C3C,   /* U+2A70 */
  0x0C3D,   /* U+2A71 */
  0x0C3E,   /* U+2A72 */
  0x0C3F,   /* U+2A73 */
  0x0239,   /* U+2A74 */
  0x061B,   /* U+2A75 */
  0x061B,   /* U+2A76 */
  0x0C40,   /* U+2A77 */
  0x0C41,   /* U+2A78 */
  0x0C42,   /* U+2A79 */
  0x0C43,   /* U+2A7A */
  0x0C44,   /* U+2A7B */
  0x0C45,   /* U+2A7C */
  0x0C46,   /* U+2A7D */
  0x0C47,   /* U+2A7E */
  0x0C48,   /* U+2A7F */
  0x0C49,   /* U+2A80 */
  0x0C4A,   /* U+2A81 */
  0x0C4B,   /* U+2A82 */
  0x0C4C,   /* U+2A83 */
  0x0C4D,   /* U+2A84 */
  0x0C4E,   /* U+2A85 */
  0x0C4F,   /* U+2A86 */
  0x0C50,   /* U+2A87 */
  0x0C51,   /* U+2A88 */
  0x0C52,   /* U+2A89 */
  0x0C53,   /* U+2A8A */
  0x0C54,   /* U+2A8B */
  0x0C55,   /* U+2A8C */
  0x0C56,   /* U+2A8D */
  0x0C57,   /* U+2A8E */
  0x0C58,   /* U+2A8F */
  0x0C59,   /* U+2A90 */
  0x0C5A,   /* U+2A91 */
  0x0C5B,   /* U+2A92 */
  0x0C5C,   /* U+2A93 */
  0x0C5D,   /* U+2A94 */
  0x0C5E,   /* U+2A95 */
  0x0C5F,   /* U+2A96 */
  0x0C60,   /* U+2A97 */
  0x0C61,   /* U+2A98 */
  0x0C62,   /* U+2A99 */
  0x0C63,   /* U+2A9A */
  0x0C64,   /* U+2A9B */
  0x0C65,   /* U+2A9C */
  0x0C66,   /* U+2A9D */
  0x0C67,   /* U+2A9E */
  0x0C68,   /* U+2A9F */
  0x0C69,   /* U+2AA0 */
  0x0C6A,   /* U+2AA1 */
  0x0C6B,   /* U+2AA2 */
  0x0C6C,   /* U+2AA3 */
  0x0C6D,   /* U+2AA4 */
  0x0C6E,   /* U+2AA5 */
  0x0C6F,   /* U+2AA6 */
  0x0C70,   /* U+2AA7 */
  0x0C71,   /* U+2AA8 */
  0x0C72,   /* U+2AA9 */
  0x0C73,   /* U+2AAA */
  0x0C74,   /* U+2AAB */
  0x0C75,   /* U+2AAC */
  0x0C76,   /* U+2AAD */
  0x0C77,   /* U+2AAE */
  0x0C78,   /* U+2AAF */
  0x0C79,   /* U+2AB0 */
  0x0C7A,   /* U+2AB1 */
  0x0C7B,   /* U+2AB2 */
  0x0C7C,   /* U+2AB3 */
  0x0C7D,   /* U+2AB4 */
  0x0C7E,   /* U+2AB5 */
  0x0C7F,   /* U+2AB6 */
  0x0C80,   /* U+2AB7 */
  0x0C81,   /* U+2AB8 */
  0x0C82,   /* U+2AB9 */
  0x0C83,   /* U+2ABA */
  0x0C84,   /* U+2ABB */
  0x0C85,   /* U+2ABC */
  0x0C86,   /* U+2ABD */
  0x0C87,   /* U+2ABE */
  0x0C88,   /* U+2ABF */
  0x0C89,   /* U+2AC0 */
  0x0C8A,   /* U+2AC1 */
  0x0C8B,   /* U+2AC2 */
  0x0C8C,   /* U+2AC3 */
  0x0C8D,   /* U+2AC4 */
  0x0C8E,   /* U+2AC5 */
  0x0C8F,   /* U+2AC6 */
  0x0C90,   /* U+2AC7 */
  0x0C91,   /* U+2AC8 */
  0x0C92,   /* U+2AC9 */
  0x0C93,   /* U+2ACA */
  0x0C94,   /* U+2ACB */
  0x0C95,   /* U+2ACC */
  0x0C96,   /* U+2ACD */
  0x0C97,   /* U+2ACE */
  0x0C98,   /* U+2ACF */
  0x0C99,   /* U+2AD0 */
  0x0C9A,   /* U+2AD1 */
  0x0C9B,   /* U+2AD2 */
  0x0C9C,   /* U+2AD3 */
  0x0C9D,   /* U+2AD4 */
  0x0C9E,   /* U+2AD5 */
  0x0C9F,   /* U+2AD6 */
  0x0CA0,   /* U+2AD7 */
  0x0CA1,   /* U+2AD8 */
  0x0CA2,   /* U+2AD9 */
  0x0CA3,   /* U+2ADA */
  0x0CA4,   /* U+2ADB */
  0x0CA5,   /* U+2ADC */
  0x0CA5,   /* U+2ADD */
  0x0CA6,   /* U+2ADE */
  0x0CA7,   /* U+2ADF */
  0x0CA8,   /* U+2AE0 */
  0x0CA9,   /* U+2AE1 */
  0x0CAA,   /* U+2AE2 */
  0x0CAB,   /* U+2AE3 */
  0x0CAC,   /* U+2AE4 */
  0x0CAD,   /* U+2AE5 */
  0x0CAE,   /* U+2AE6 */
  0x0CAF,   /* U+2AE7 */
  0x0CB0,   /* U+2AE8 */
  0x0CB1,   /* U+2AE9 */
  0x0CB2,   /* U+2AEA */
  0x0CB3,   /* U+2AEB */
  0x0CB4,   /* U+2AEC */
  0x0CB5,   /* U+2AED */
  0x0CB6,   /* U+2AEE */
  0x0CB7,   /* U+2AEF */
  0x0CB8,   /* U+2AF0 */
  0x0CB9,   /* U+2AF1 */
  0x0CBA,   /* U+2AF2 */
  0x0CBB,   /* U+2AF3 */
  0x0CBC,   /* U+2AF4 */
  0x0CBD,   /* U+2AF5 */
  0x0CBE,   /* U+2AF6 */
  0x0CBF,   /* U+2AF7 */
  0x0CC0,   /* U+2AF8 */
  0x0CC1,   /* U+2AF9 */
  0x0CC2,   /* U+2AFA */
  0x0CC3,   /* U+2AFB */
  0x0CC4,   /* U+2AFC */
  0x0CC5,   /* U+2AFD */
  0x0CC6,   /* U+2AFE */
  0x0CC7,   /* U+2AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2A00 */
  0x0020,   /* U+2A01 */
  0x0020,   /* U+2A02 */
  0x0020,   /* U+2A03 */
  0x0020,   /* U+2A04 */
  0x0020,   /* U+2A05 */
  0x0020,   /* U+2A06 */
  0x0020,   /* U+2A07 */
  0x0020,   /* U+2A08 */
  0x0020,   /* U+2A09 */
  0x0020,   /* U+2A0A */
  0x0020,   /* U+2A0B */
  0x0020,   /* U+2A0C */
  0x0020,   /* U+2A0D */
  0x0020,   /* U+2A0E */
  0x0020,   /* U+2A0F */
  0x0020,   /* U+2A10 */
  0x0020,   /* U+2A11 */
  0x0020,   /* U+2A12 */
  0x0020,   /* U+2A13 */
  0x0020,   /* U+2A14 */
  0x0020,   /* U+2A15 */
  0x0020,   /* U+2A16 */
  0x0020,   /* U+2A17 */
  0x0020,   /* U+2A18 */
  0x0020,   /* U+2A19 */
  0x0020,   /* U+2A1A */
  0x0020,   /* U+2A1B */
  0x0020,   /* U+2A1C */
  0x0020,   /* U+2A1D */
  0x0020,   /* U+2A1E */
  0x0020,   /* U+2A1F */
  0x0020,   /* U+2A20 */
  0x0020,   /* U+2A21 */
  0x0020,   /* U+2A22 */
  0x0020,   /* U+2A23 */
  0x0020,   /* U+2A24 */
  0x0020,   /* U+2A25 */
  0x0020,   /* U+2A26 */
  0x0020,   /* U+2A27 */
  0x0020,   /* U+2A28 */
  0x0020,   /* U+2A29 */
  0x0020,   /* U+2A2A */
  0x0020,   /* U+2A2B */
  0x0020,   /* U+2A2C */
  0x0020,   /* U+2A2D */
  0x0020,   /* U+2A2E */
  0x0020,   /* U+2A2F */
  0x0020,   /* U+2A30 */
  0x0020,   /* U+2A31 */
  0x0020,   /* U+2A32 */
  0x0020,   /* U+2A33 */
  0x0020,   /* U+2A34 */
  0x0020,   /* U+2A35 */
  0x0020,   /* U+2A36 */
  0x0020,   /* U+2A37 */
  0x0020,   /* U+2A38 */
  0x0020,   /* U+2A39 */
  0x0020,   /* U+2A3A */
  0x0020,   /* U+2A3B */
  0x0020,   /* U+2A3C */
  0x0020,   /* U+2A3D */
  0x0020,   /* U+2A3E */
  0x0020,   /* U+2A3F */
  0x0020,   /* U+2A40 */
  0x0020,   /* U+2A41 */
  0x0020,   /* U+2A42 */
  0x0020,   /* U+2A43 */
  0x0020,   /* U+2A44 */
  0x0020,   /* U+2A45 */
  0x0020,   /* U+2A46 */
  0x0020,   /* U+2A47 */
  0x0020,   /* U+2A48 */
  0x0020,   /* U+2A49 */
  0x0020,   /* U+2A4A */
  0x0020,   /* U+2A4B */
  0x0020,   /* U+2A4C */
  0x0020,   /* U+2A4D */
  0x0020,   /* U+2A4E */
  0x0020,   /* U+2A4F */
  0x0020,   /* U+2A50 */
  0x0020,   /* U+2A51 */
  0x0020,   /* U+2A52 */
  0x0020,   /* U+2A53 */
  0x0020,   /* U+2A54 */
  0x0020,   /* U+2A55 */
  0x0020,   /* U+2A56 */
  0x0020,   /* U+2A57 */
  0x0020,   /* U+2A58 */
  0x0020,   /* U+2A59 */
  0x0020,   /* U+2A5A */
  0x0020,   /* U+2A5B */
  0x0020,   /* U+2A5C */
  0x0020,   /* U+2A5D */
  0x0020,   /* U+2A5E */
  0x0020,   /* U+2A5F */
  0x0020,   /* U+2A60 */
  0x0020,   /* U+2A61 */
  0x0020,   /* U+2A62 */
  0x0020,   /* U+2A63 */
  0x0020,   /* U+2A64 */
  0x0020,   /* U+2A65 */
  0x0020,   /* U+2A66 */
  0x0020,   /* U+2A67 */
  0x0020,   /* U+2A68 */
  0x0020,   /* U+2A69 */
  0x0020,   /* U+2A6A */
  0x0020,   /* U+2A6B */
  0x0020,   /* U+2A6C */
  0x0020,   /* U+2A6D */
  0x0020,   /* U+2A6E */
  0x0020,   /* U+2A6F */
  0x0020,   /* U+2A70 */
  0x0020,   /* U+2A71 */
  0x0020,   /* U+2A72 */
  0x0020,   /* U+2A73 */
  0x0020,   /* U+2A74 */
  0x0020,   /* U+2A75 */
  0x0020,   /* U+2A76 */
  0x0020,   /* U+2A77 */
  0x0020,   /* U+2A78 */
  0x0020,   /* U+2A79 */
  0x0020,   /* U+2A7A */
  0x0020,   /* U+2A7B */
  0x0020,   /* U+2A7C */
  0x0020,   /* U+2A7D */
  0x0020,   /* U+2A7E */
  0x0020,   /* U+2A7F */
  0x0020,   /* U+2A80 */
  0x0020,   /* U+2A81 */
  0x0020,   /* U+2A82 */
  0x0020,   /* U+2A83 */
  0x0020,   /* U+2A84 */
  0x0020,   /* U+2A85 */
  0x0020,   /* U+2A86 */
  0x0020,   /* U+2A87 */
  0x0020,   /* U+2A88 */
  0x0020,   /* U+2A89 */
  0x0020,   /* U+2A8A */
  0x0020,   /* U+2A8B */
  0x0020,   /* U+2A8C */
  0x0020,   /* U+2A8D */
  0x0020,   /* U+2A8E */
  0x0020,   /* U+2A8F */
  0x0020,   /* U+2A90 */
  0x0020,   /* U+2A91 */
  0x0020,   /* U+2A92 */
  0x0020,   /* U+2A93 */
  0x0020,   /* U+2A94 */
  0x0020,   /* U+2A95 */
  0x0020,   /* U+2A96 */
  0x0020,   /* U+2A97 */
  0x0020,   /* U+2A98 */
  0x0020,   /* U+2A99 */
  0x0020,   /* U+2A9A */
  0x0020,   /* U+2A9B */
  0x0020,   /* U+2A9C */
  0x0020,   /* U+2A9D */
  0x0020,   /* U+2A9E */
  0x0020,   /* U+2A9F */
  0x0020,   /* U+2AA0 */
  0x0020,   /* U+2AA1 */
  0x0020,   /* U+2AA2 */
  0x0020,   /* U+2AA3 */
  0x0020,   /* U+2AA4 */
  0x0020,   /* U+2AA5 */
  0x0020,   /* U+2AA6 */
  0x0020,   /* U+2AA7 */
  0x0020,   /* U+2AA8 */
  0x0020,   /* U+2AA9 */
  0x0020,   /* U+2AAA */
  0x0020,   /* U+2AAB */
  0x0020,   /* U+2AAC */
  0x0020,   /* U+2AAD */
  0x0020,   /* U+2AAE */
  0x0020,   /* U+2AAF */
  0x0020,   /* U+2AB0 */
  0x0020,   /* U+2AB1 */
  0x0020,   /* U+2AB2 */
  0x0020,   /* U+2AB3 */
  0x0020,   /* U+2AB4 */
  0x0020,   /* U+2AB5 */
  0x0020,   /* U+2AB6 */
  0x0020,   /* U+2AB7 */
  0x0020,   /* U+2AB8 */
  0x0020,   /* U+2AB9 */
  0x0020,   /* U+2ABA */
  0x0020,   /* U+2ABB */
  0x0020,   /* U+2ABC */
  0x0020,   /* U+2ABD */
  0x0020,   /* U+2ABE */
  0x0020,   /* U+2ABF */
  0x0020,   /* U+2AC0 */
  0x0020,   /* U+2AC1 */
  0x0020,   /* U+2AC2 */
  0x0020,   /* U+2AC3 */
  0x0020,   /* U+2AC4 */
  0x0020,   /* U+2AC5 */
  0x0020,   /* U+2AC6 */
  0x0020,   /* U+2AC7 */
  0x0020,   /* U+2AC8 */
  0x0020,   /* U+2AC9 */
  0x0020,   /* U+2ACA */
  0x0020,   /* U+2ACB */
  0x0020,   /* U+2ACC */
  0x0020,   /* U+2ACD */
  0x0020,   /* U+2ACE */
  0x0020,   /* U+2ACF */
  0x0020,   /* U+2AD0 */
  0x0020,   /* U+2AD1 */
  0x0020,   /* U+2AD2 */
  0x0020,   /* U+2AD3 */
  0x0020,   /* U+2AD4 */
  0x0020,   /* U+2AD5 */
  0x0020,   /* U+2AD6 */
  0x0020,   /* U+2AD7 */
  0x0020,   /* U+2AD8 */
  0x0020,   /* U+2AD9 */
  0x0020,   /* U+2ADA */
  0x0020,   /* U+2ADB */
  0x0020,   /* U+2ADC */
  0x0020,   /* U+2ADD */
  0x0020,   /* U+2ADE */
  0x0020,   /* U+2ADF */
  0x0020,   /* U+2AE0 */
  0x0020,   /* U+2AE1 */
  0x0020,   /* U+2AE2 */
  0x0020,   /* U+2AE3 */
  0x0020,   /* U+2AE4 */
  0x0020,   /* U+2AE5 */
  0x0020,   /* U+2AE6 */
  0x0020,   /* U+2AE7 */
  0x0020,   /* U+2AE8 */
  0x0020,   /* U+2AE9 */
  0x0020,   /* U+2AEA */
  0x0020,   /* U+2AEB */
  0x0020,   /* U+2AEC */
  0x0020,   /* U+2AED */
  0x0020,   /* U+2AEE */
  0x0020,   /* U+2AEF */
  0x0020,   /* U+2AF0 */
  0x0020,   /* U+2AF1 */
  0x0020,   /* U+2AF2 */
  0x0020,   /* U+2AF3 */
  0x0020,   /* U+2AF4 */
  0x0020,   /* U+2AF5 */
  0x0020,   /* U+2AF6 */
  0x0020,   /* U+2AF7 */
  0x0020,   /* U+2AF8 */
  0x0020,   /* U+2AF9 */
  0x0020,   /* U+2AFA */
  0x0020,   /* U+2AFB */
  0x0020,   /* U+2AFC */
  0x0020,   /* U+2AFD */
  0x0020,   /* U+2AFE */
  0x0020,   /* U+2AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2A00 */
  0x0002,   /* U+2A01 */
  0x0002,   /* U+2A02 */
  0x0002,   /* U+2A03 */
  0x0002,   /* U+2A04 */
  0x0002,   /* U+2A05 */
  0x0002,   /* U+2A06 */
  0x0002,   /* U+2A07 */
  0x0002,   /* U+2A08 */
  0x0002,   /* U+2A09 */
  0x0002,   /* U+2A0A */
  0x0002,   /* U+2A0B */
  0x0004,   /* U+2A0C */
  0x0002,   /* U+2A0D */
  0x0002,   /* U+2A0E */
  0x0002,   /* U+2A0F */
  0x0002,   /* U+2A10 */
  0x0002,   /* U+2A11 */
  0x0002,   /* U+2A12 */
  0x0002,   /* U+2A13 */
  0x0002,   /* U+2A14 */
  0x0002,   /* U+2A15 */
  0x0002,   /* U+2A16 */
  0x0002,   /* U+2A17 */
  0x0002,   /* U+2A18 */
  0x0002,   /* U+2A19 */
  0x0002,   /* U+2A1A */
  0x0002,   /* U+2A1B */
  0x0002,   /* U+2A1C */
  0x0002,   /* U+2A1D */
  0x0002,   /* U+2A1E */
  0x0002,   /* U+2A1F */
  0x0002,   /* U+2A20 */
  0x0002,   /* U+2A21 */
  0x0002,   /* U+2A22 */
  0x0002,   /* U+2A23 */
  0x0002,   /* U+2A24 */
  0x0002,   /* U+2A25 */
  0x0002,   /* U+2A26 */
  0x0002,   /* U+2A27 */
  0x0002,   /* U+2A28 */
  0x0002,   /* U+2A29 */
  0x0002,   /* U+2A2A */
  0x0002,   /* U+2A2B */
  0x0002,   /* U+2A2C */
  0x0002,   /* U+2A2D */
  0x0002,   /* U+2A2E */
  0x0002,   /* U+2A2F */
  0x0002,   /* U+2A30 */
  0x0002,   /* U+2A31 */
  0x0002,   /* U+2A32 */
  0x0002,   /* U+2A33 */
  0x0002,   /* U+2A34 */
  0x0002,   /* U+2A35 */
  0x0002,   /* U+2A36 */
  0x0002,   /* U+2A37 */
  0x0002,   /* U+2A38 */
  0x0002,   /* U+2A39 */
  0x0002,   /* U+2A3A */
  0x0002,   /* U+2A3B */
  0x0002,   /* U+2A3C */
  0x0002,   /* U+2A3D */
  0x0002,   /* U+2A3E */
  0x0002,   /* U+2A3F */
  0x0002,   /* U+2A40 */
  0x0002,   /* U+2A41 */
  0x0002,   /* U+2A42 */
  0x0002,   /* U+2A43 */
  0x0002,   /* U+2A44 */
  0x0002,   /* U+2A45 */
  0x0002,   /* U+2A46 */
  0x0002,   /* U+2A47 */
  0x0002,   /* U+2A48 */
  0x0002,   /* U+2A49 */
  0x0002,   /* U+2A4A */
  0x0002,   /* U+2A4B */
  0x0002,   /* U+2A4C */
  0x0002,   /* U+2A4D */
  0x0002,   /* U+2A4E */
  0x0002,   /* U+2A4F */
  0x0002,   /* U+2A50 */
  0x0002,   /* U+2A51 */
  0x0002,   /* U+2A52 */
  0x0002,   /* U+2A53 */
  0x0002,   /* U+2A54 */
  0x0002,   /* U+2A55 */
  0x0002,   /* U+2A56 */
  0x0002,   /* U+2A57 */
  0x0002,   /* U+2A58 */
  0x0002,   /* U+2A59 */
  0x0002,   /* U+2A5A */
  0x0002,   /* U+2A5B */
  0x0002,   /* U+2A5C */
  0x0002,   /* U+2A5D */
  0x0002,   /* U+2A5E */
  0x0002,   /* U+2A5F */
  0x0002,   /* U+2A60 */
  0x0002,   /* U+2A61 */
  0x0002,   /* U+2A62 */
  0x0002,   /* U+2A63 */
  0x0002,   /* U+2A64 */
  0x0002,   /* U+2A65 */
  0x0002,   /* U+2A66 */
  0x0002,   /* U+2A67 */
  0x0002,   /* U+2A68 */
  0x0002,   /* U+2A69 */
  0x0002,   /* U+2A6A */
  0x0002,   /* U+2A6B */
  0x0002,   /* U+2A6C */
  0x0002,   /* U+2A6D */
  0x0002,   /* U+2A6E */
  0x0002,   /* U+2A6F */
  0x0002,   /* U+2A70 */
  0x0002,   /* U+2A71 */
  0x0002,   /* U+2A72 */
  0x0002,   /* U+2A73 */
  0x0004,   /* U+2A74 */
  0x0004,   /* U+2A75 */
  0x0004,   /* U+2A76 */
  0x0002,   /* U+2A77 */
  0x0002,   /* U+2A78 */
  0x0002,   /* U+2A79 */
  0x0002,   /* U+2A7A */
  0x0002,   /* U+2A7B */
  0x0002,   /* U+2A7C */
  0x0002,   /* U+2A7D */
  0x0002,   /* U+2A7E */
  0x0002,   /* U+2A7F */
  0x0002,   /* U+2A80 */
  0x0002,   /* U+2A81 */
  0x0002,   /* U+2A82 */
  0x0002,   /* U+2A83 */
  0x0002,   /* U+2A84 */
  0x0002,   /* U+2A85 */
  0x0002,   /* U+2A86 */
  0x0002,   /* U+2A87 */
  0x0002,   /* U+2A88 */
  0x0002,   /* U+2A89 */
  0x0002,   /* U+2A8A */
  0x0002,   /* U+2A8B */
  0x0002,   /* U+2A8C */
  0x0002,   /* U+2A8D */
  0x0002,   /* U+2A8E */
  0x0002,   /* U+2A8F */
  0x0002,   /* U+2A90 */
  0x0002,   /* U+2A91 */
  0x0002,   /* U+2A92 */
  0x0002,   /* U+2A93 */
  0x0002,   /* U+2A94 */
  0x0002,   /* U+2A95 */
  0x0002,   /* U+2A96 */
  0x0002,   /* U+2A97 */
  0x0002,   /* U+2A98 */
  0x0002,   /* U+2A99 */
  0x0002,   /* U+2A9A */
  0x0002,   /* U+2A9B */
  0x0002,   /* U+2A9C */
  0x0002,   /* U+2A9D */
  0x0002,   /* U+2A9E */
  0x0002,   /* U+2A9F */
  0x0002,   /* U+2AA0 */
  0x0002,   /* U+2AA1 */
  0x0002,   /* U+2AA2 */
  0x0002,   /* U+2AA3 */
  0x0002,   /* U+2AA4 */
  0x0002,   /* U+2AA5 */
  0x0002,   /* U+2AA6 */
  0x0002,   /* U+2AA7 */
  0x0002,   /* U+2AA8 */
  0x0002,   /* U+2AA9 */
  0x0002,   /* U+2AAA */
  0x0002,   /* U+2AAB */
  0x0002,   /* U+2AAC */
  0x0002,   /* U+2AAD */
  0x0002,   /* U+2AAE */
  0x0002,   /* U+2AAF */
  0x0002,   /* U+2AB0 */
  0x0002,   /* U+2AB1 */
  0x0002,   /* U+2AB2 */
  0x0002,   /* U+2AB3 */
  0x0002,   /* U+2AB4 */
  0x0002,   /* U+2AB5 */
  0x0002,   /* U+2AB6 */
  0x0002,   /* U+2AB7 */
  0x0002,   /* U+2AB8 */
  0x0002,   /* U+2AB9 */
  0x0002,   /* U+2ABA */
  0x0002,   /* U+2ABB */
  0x0002,   /* U+2ABC */
  0x0002,   /* U+2ABD */
  0x0002,   /* U+2ABE */
  0x0002,   /* U+2ABF */
  0x0002,   /* U+2AC0 */
  0x0002,   /* U+2AC1 */
  0x0002,   /* U+2AC2 */
  0x0002,   /* U+2AC3 */
  0x0002,   /* U+2AC4 */
  0x0002,   /* U+2AC5 */
  0x0002,   /* U+2AC6 */
  0x0002,   /* U+2AC7 */
  0x0002,   /* U+2AC8 */
  0x0002,   /* U+2AC9 */
  0x0002,   /* U+2ACA */
  0x0002,   /* U+2ACB */
  0x0002,   /* U+2ACC */
  0x0002,   /* U+2ACD */
  0x0002,   /* U+2ACE */
  0x0002,   /* U+2ACF */
  0x0002,   /* U+2AD0 */
  0x0002,   /* U+2AD1 */
  0x0002,   /* U+2AD2 */
  0x0002,   /* U+2AD3 */
  0x0002,   /* U+2AD4 */
  0x0002,   /* U+2AD5 */
  0x0002,   /* U+2AD6 */
  0x0002,   /* U+2AD7 */
  0x0002,   /* U+2AD8 */
  0x0002,   /* U+2AD9 */
  0x0002,   /* U+2ADA */
  0x0002,   /* U+2ADB */
  0x0002,   /* U+2ADC */
  0x0002,   /* U+2ADD */
  0x0002,   /* U+2ADE */
  0x0002,   /* U+2ADF */
  0x0002,   /* U+2AE0 */
  0x0002,   /* U+2AE1 */
  0x0002,   /* U+2AE2 */
  0x0002,   /* U+2AE3 */
  0x0002,   /* U+2AE4 */
  0x0002,   /* U+2AE5 */
  0x0002,   /* U+2AE6 */
  0x0002,   /* U+2AE7 */
  0x0002,   /* U+2AE8 */
  0x0002,   /* U+2AE9 */
  0x0002,   /* U+2AEA */
  0x0002,   /* U+2AEB */
  0x0002,   /* U+2AEC */
  0x0002,   /* U+2AED */
  0x0002,   /* U+2AEE */
  0x0002,   /* U+2AEF */
  0x0002,   /* U+2AF0 */
  0x0002,   /* U+2AF1 */
  0x0002,   /* U+2AF2 */
  0x0002,   /* U+2AF3 */
  0x0002,   /* U+2AF4 */
  0x0002,   /* U+2AF5 */
  0x0002,   /* U+2AF6 */
  0x0002,   /* U+2AF7 */
  0x0002,   /* U+2AF8 */
  0x0002,   /* U+2AF9 */
  0x0002,   /* U+2AFA */
  0x0002,   /* U+2AFB */
  0x0002,   /* U+2AFC */
  0x0002,   /* U+2AFD */
  0x0002,   /* U+2AFE */
  0x0002,   /* U+2AFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x063C,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0239,   /* U+2A74 */
  0x061B,   /* U+2A75 */
  0x061B,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0020,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0020,   /* U+2A74 */
  0x0020,   /* U+2A75 */
  0x0020,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x002F,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0004,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0004,   /* U+2A74 */
  0x0004,   /* U+2A75 */
  0x0004,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0002,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x063C,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x061B,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x061B,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0020,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0020,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x0020,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0004,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0004,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x0004,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x063C,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0000,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x0000,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0020,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0000,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x0000,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+2A00 */
  0x0000,   /* U+2A01 */
  0x0000,   /* U+2A02 */
  0x0000,   /* U+2A03 */
  0x0000,   /* U+2A04 */
  0x0000,   /* U+2A05 */
  0x0000,   /* U+2A06 */
  0x0000,   /* U+2A07 */
  0x0000,   /* U+2A08 */
  0x0000,   /* U+2A09 */
  0x0000,   /* U+2A0A */
  0x0000,   /* U+2A0B */
  0x0004,   /* U+2A0C */
  0x0000,   /* U+2A0D */
  0x0000,   /* U+2A0E */
  0x0000,   /* U+2A0F */
  0x0000,   /* U+2A10 */
  0x0000,   /* U+2A11 */
  0x0000,   /* U+2A12 */
  0x0000,   /* U+2A13 */
  0x0000,   /* U+2A14 */
  0x0000,   /* U+2A15 */
  0x0000,   /* U+2A16 */
  0x0000,   /* U+2A17 */
  0x0000,   /* U+2A18 */
  0x0000,   /* U+2A19 */
  0x0000,   /* U+2A1A */
  0x0000,   /* U+2A1B */
  0x0000,   /* U+2A1C */
  0x0000,   /* U+2A1D */
  0x0000,   /* U+2A1E */
  0x0000,   /* U+2A1F */
  0x0000,   /* U+2A20 */
  0x0000,   /* U+2A21 */
  0x0000,   /* U+2A22 */
  0x0000,   /* U+2A23 */
  0x0000,   /* U+2A24 */
  0x0000,   /* U+2A25 */
  0x0000,   /* U+2A26 */
  0x0000,   /* U+2A27 */
  0x0000,   /* U+2A28 */
  0x0000,   /* U+2A29 */
  0x0000,   /* U+2A2A */
  0x0000,   /* U+2A2B */
  0x0000,   /* U+2A2C */
  0x0000,   /* U+2A2D */
  0x0000,   /* U+2A2E */
  0x0000,   /* U+2A2F */
  0x0000,   /* U+2A30 */
  0x0000,   /* U+2A31 */
  0x0000,   /* U+2A32 */
  0x0000,   /* U+2A33 */
  0x0000,   /* U+2A34 */
  0x0000,   /* U+2A35 */
  0x0000,   /* U+2A36 */
  0x0000,   /* U+2A37 */
  0x0000,   /* U+2A38 */
  0x0000,   /* U+2A39 */
  0x0000,   /* U+2A3A */
  0x0000,   /* U+2A3B */
  0x0000,   /* U+2A3C */
  0x0000,   /* U+2A3D */
  0x0000,   /* U+2A3E */
  0x0000,   /* U+2A3F */
  0x0000,   /* U+2A40 */
  0x0000,   /* U+2A41 */
  0x0000,   /* U+2A42 */
  0x0000,   /* U+2A43 */
  0x0000,   /* U+2A44 */
  0x0000,   /* U+2A45 */
  0x0000,   /* U+2A46 */
  0x0000,   /* U+2A47 */
  0x0000,   /* U+2A48 */
  0x0000,   /* U+2A49 */
  0x0000,   /* U+2A4A */
  0x0000,   /* U+2A4B */
  0x0000,   /* U+2A4C */
  0x0000,   /* U+2A4D */
  0x0000,   /* U+2A4E */
  0x0000,   /* U+2A4F */
  0x0000,   /* U+2A50 */
  0x0000,   /* U+2A51 */
  0x0000,   /* U+2A52 */
  0x0000,   /* U+2A53 */
  0x0000,   /* U+2A54 */
  0x0000,   /* U+2A55 */
  0x0000,   /* U+2A56 */
  0x0000,   /* U+2A57 */
  0x0000,   /* U+2A58 */
  0x0000,   /* U+2A59 */
  0x0000,   /* U+2A5A */
  0x0000,   /* U+2A5B */
  0x0000,   /* U+2A5C */
  0x0000,   /* U+2A5D */
  0x0000,   /* U+2A5E */
  0x0000,   /* U+2A5F */
  0x0000,   /* U+2A60 */
  0x0000,   /* U+2A61 */
  0x0000,   /* U+2A62 */
  0x0000,   /* U+2A63 */
  0x0000,   /* U+2A64 */
  0x0000,   /* U+2A65 */
  0x0000,   /* U+2A66 */
  0x0000,   /* U+2A67 */
  0x0000,   /* U+2A68 */
  0x0000,   /* U+2A69 */
  0x0000,   /* U+2A6A */
  0x0000,   /* U+2A6B */
  0x0000,   /* U+2A6C */
  0x0000,   /* U+2A6D */
  0x0000,   /* U+2A6E */
  0x0000,   /* U+2A6F */
  0x0000,   /* U+2A70 */
  0x0000,   /* U+2A71 */
  0x0000,   /* U+2A72 */
  0x0000,   /* U+2A73 */
  0x0000,   /* U+2A74 */
  0x0000,   /* U+2A75 */
  0x0000,   /* U+2A76 */
  0x0000,   /* U+2A77 */
  0x0000,   /* U+2A78 */
  0x0000,   /* U+2A79 */
  0x0000,   /* U+2A7A */
  0x0000,   /* U+2A7B */
  0x0000,   /* U+2A7C */
  0x0000,   /* U+2A7D */
  0x0000,   /* U+2A7E */
  0x0000,   /* U+2A7F */
  0x0000,   /* U+2A80 */
  0x0000,   /* U+2A81 */
  0x0000,   /* U+2A82 */
  0x0000,   /* U+2A83 */
  0x0000,   /* U+2A84 */
  0x0000,   /* U+2A85 */
  0x0000,   /* U+2A86 */
  0x0000,   /* U+2A87 */
  0x0000,   /* U+2A88 */
  0x0000,   /* U+2A89 */
  0x0000,   /* U+2A8A */
  0x0000,   /* U+2A8B */
  0x0000,   /* U+2A8C */
  0x0000,   /* U+2A8D */
  0x0000,   /* U+2A8E */
  0x0000,   /* U+2A8F */
  0x0000,   /* U+2A90 */
  0x0000,   /* U+2A91 */
  0x0000,   /* U+2A92 */
  0x0000,   /* U+2A93 */
  0x0000,   /* U+2A94 */
  0x0000,   /* U+2A95 */
  0x0000,   /* U+2A96 */
  0x0000,   /* U+2A97 */
  0x0000,   /* U+2A98 */
  0x0000,   /* U+2A99 */
  0x0000,   /* U+2A9A */
  0x0000,   /* U+2A9B */
  0x0000,   /* U+2A9C */
  0x0000,   /* U+2A9D */
  0x0000,   /* U+2A9E */
  0x0000,   /* U+2A9F */
  0x0000,   /* U+2AA0 */
  0x0000,   /* U+2AA1 */
  0x0000,   /* U+2AA2 */
  0x0000,   /* U+2AA3 */
  0x0000,   /* U+2AA4 */
  0x0000,   /* U+2AA5 */
  0x0000,   /* U+2AA6 */
  0x0000,   /* U+2AA7 */
  0x0000,   /* U+2AA8 */
  0x0000,   /* U+2AA9 */
  0x0000,   /* U+2AAA */
  0x0000,   /* U+2AAB */
  0x0000,   /* U+2AAC */
  0x0000,   /* U+2AAD */
  0x0000,   /* U+2AAE */
  0x0000,   /* U+2AAF */
  0x0000,   /* U+2AB0 */
  0x0000,   /* U+2AB1 */
  0x0000,   /* U+2AB2 */
  0x0000,   /* U+2AB3 */
  0x0000,   /* U+2AB4 */
  0x0000,   /* U+2AB5 */
  0x0000,   /* U+2AB6 */
  0x0000,   /* U+2AB7 */
  0x0000,   /* U+2AB8 */
  0x0000,   /* U+2AB9 */
  0x0000,   /* U+2ABA */
  0x0000,   /* U+2ABB */
  0x0000,   /* U+2ABC */
  0x0000,   /* U+2ABD */
  0x0000,   /* U+2ABE */
  0x0000,   /* U+2ABF */
  0x0000,   /* U+2AC0 */
  0x0000,   /* U+2AC1 */
  0x0000,   /* U+2AC2 */
  0x0000,   /* U+2AC3 */
  0x0000,   /* U+2AC4 */
  0x0000,   /* U+2AC5 */
  0x0000,   /* U+2AC6 */
  0x0000,   /* U+2AC7 */
  0x0000,   /* U+2AC8 */
  0x0000,   /* U+2AC9 */
  0x0000,   /* U+2ACA */
  0x0000,   /* U+2ACB */
  0x0000,   /* U+2ACC */
  0x0000,   /* U+2ACD */
  0x0000,   /* U+2ACE */
  0x0000,   /* U+2ACF */
  0x0000,   /* U+2AD0 */
  0x0000,   /* U+2AD1 */
  0x0000,   /* U+2AD2 */
  0x0000,   /* U+2AD3 */
  0x0000,   /* U+2AD4 */
  0x0000,   /* U+2AD5 */
  0x0000,   /* U+2AD6 */
  0x0000,   /* U+2AD7 */
  0x0000,   /* U+2AD8 */
  0x0000,   /* U+2AD9 */
  0x0000,   /* U+2ADA */
  0x0000,   /* U+2ADB */
  0x0000,   /* U+2ADC */
  0x0000,   /* U+2ADD */
  0x0000,   /* U+2ADE */
  0x0000,   /* U+2ADF */
  0x0000,   /* U+2AE0 */
  0x0000,   /* U+2AE1 */
  0x0000,   /* U+2AE2 */
  0x0000,   /* U+2AE3 */
  0x0000,   /* U+2AE4 */
  0x0000,   /* U+2AE5 */
  0x0000,   /* U+2AE6 */
  0x0000,   /* U+2AE7 */
  0x0000,   /* U+2AE8 */
  0x0000,   /* U+2AE9 */
  0x0000,   /* U+2AEA */
  0x0000,   /* U+2AEB */
  0x0000,   /* U+2AEC */
  0x0000,   /* U+2AED */
  0x0000,   /* U+2AEE */
  0x0000,   /* U+2AEF */
  0x0000,   /* U+2AF0 */
  0x0000,   /* U+2AF1 */
  0x0000,   /* U+2AF2 */
  0x0000,   /* U+2AF3 */
  0x0000,   /* U+2AF4 */
  0x0000,   /* U+2AF5 */
  0x0000,   /* U+2AF6 */
  0x0000,   /* U+2AF7 */
  0x0000,   /* U+2AF8 */
  0x0000,   /* U+2AF9 */
  0x0000,   /* U+2AFA */
  0x0000,   /* U+2AFB */
  0x0000,   /* U+2AFC */
  0x0000,   /* U+2AFD */
  0x0000,   /* U+2AFE */
  0x0000,   /* U+2AFF */
};

uint16 uca900_p02B[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0CC8,   /* U+2B00 */
  0x0CC9,   /* U+2B01 */
  0x0CCA,   /* U+2B02 */
  0x0CCB,   /* U+2B03 */
  0x0CCC,   /* U+2B04 */
  0x0CCD,   /* U+2B05 */
  0x0CCE,   /* U+2B06 */
  0x0CCF,   /* U+2B07 */
  0x0CD0,   /* U+2B08 */
  0x0CD1,   /* U+2B09 */
  0x0CD2,   /* U+2B0A */
  0x0CD3,   /* U+2B0B */
  0x0CD4,   /* U+2B0C */
  0x0CD5,   /* U+2B0D */
  0x0CD6,   /* U+2B0E */
  0x0CD7,   /* U+2B0F */
  0x0CD8,   /* U+2B10 */
  0x0CD9,   /* U+2B11 */
  0x0CDA,   /* U+2B12 */
  0x0CDB,   /* U+2B13 */
  0x0CDC,   /* U+2B14 */
  0x0CDD,   /* U+2B15 */
  0x0CDE,   /* U+2B16 */
  0x0CDF,   /* U+2B17 */
  0x0CE0,   /* U+2B18 */
  0x0CE1,   /* U+2B19 */
  0x0CE2,   /* U+2B1A */
  0x0CE3,   /* U+2B1B */
  0x0CE4,   /* U+2B1C */
  0x0CE5,   /* U+2B1D */
  0x0CE6,   /* U+2B1E */
  0x0CE7,   /* U+2B1F */
  0x0CE8,   /* U+2B20 */
  0x0CE9,   /* U+2B21 */
  0x0CEA,   /* U+2B22 */
  0x0CEB,   /* U+2B23 */
  0x0CEC,   /* U+2B24 */
  0x0CED,   /* U+2B25 */
  0x0CEE,   /* U+2B26 */
  0x0CEF,   /* U+2B27 */
  0x0CF0,   /* U+2B28 */
  0x0CF1,   /* U+2B29 */
  0x0CF2,   /* U+2B2A */
  0x0CF3,   /* U+2B2B */
  0x0CF4,   /* U+2B2C */
  0x0CF5,   /* U+2B2D */
  0x0CF6,   /* U+2B2E */
  0x0CF7,   /* U+2B2F */
  0x0CF8,   /* U+2B30 */
  0x0CF9,   /* U+2B31 */
  0x0CFA,   /* U+2B32 */
  0x0CFB,   /* U+2B33 */
  0x0CFC,   /* U+2B34 */
  0x0CFD,   /* U+2B35 */
  0x0CFE,   /* U+2B36 */
  0x0CFF,   /* U+2B37 */
  0x0D00,   /* U+2B38 */
  0x0D01,   /* U+2B39 */
  0x0D02,   /* U+2B3A */
  0x0D03,   /* U+2B3B */
  0x0D04,   /* U+2B3C */
  0x0D05,   /* U+2B3D */
  0x0D06,   /* U+2B3E */
  0x0D07,   /* U+2B3F */
  0x0D08,   /* U+2B40 */
  0x0D09,   /* U+2B41 */
  0x0D0A,   /* U+2B42 */
  0x0D0B,   /* U+2B43 */
  0x0D0C,   /* U+2B44 */
  0x0D0D,   /* U+2B45 */
  0x0D0E,   /* U+2B46 */
  0x0D0F,   /* U+2B47 */
  0x0D10,   /* U+2B48 */
  0x0D11,   /* U+2B49 */
  0x0D12,   /* U+2B4A */
  0x0D13,   /* U+2B4B */
  0x0D14,   /* U+2B4C */
  0x0D15,   /* U+2B4D */
  0x0D16,   /* U+2B4E */
  0x0D17,   /* U+2B4F */
  0x0D18,   /* U+2B50 */
  0x0D19,   /* U+2B51 */
  0x0D1A,   /* U+2B52 */
  0x0D1B,   /* U+2B53 */
  0x0D1C,   /* U+2B54 */
  0x0D1D,   /* U+2B55 */
  0x0D1E,   /* U+2B56 */
  0x0D1F,   /* U+2B57 */
  0x0D20,   /* U+2B58 */
  0x0D21,   /* U+2B59 */
  0x0D22,   /* U+2B5A */
  0x0D23,   /* U+2B5B */
  0x0D24,   /* U+2B5C */
  0x0D25,   /* U+2B5D */
  0x0D26,   /* U+2B5E */
  0x0D27,   /* U+2B5F */
  0x0D28,   /* U+2B60 */
  0x0D29,   /* U+2B61 */
  0x0D2A,   /* U+2B62 */
  0x0D2B,   /* U+2B63 */
  0x0D2C,   /* U+2B64 */
  0x0D2D,   /* U+2B65 */
  0x0D2E,   /* U+2B66 */
  0x0D2F,   /* U+2B67 */
  0x0D30,   /* U+2B68 */
  0x0D31,   /* U+2B69 */
  0x0D32,   /* U+2B6A */
  0x0D33,   /* U+2B6B */
  0x0D34,   /* U+2B6C */
  0x0D35,   /* U+2B6D */
  0x0D36,   /* U+2B6E */
  0x0D37,   /* U+2B6F */
  0x0D38,   /* U+2B70 */
  0x0D39,   /* U+2B71 */
  0x0D3A,   /* U+2B72 */
  0x0D3B,   /* U+2B73 */
  0xFBC0,   /* U+2B74 */
  0xFBC0,   /* U+2B75 */
  0x0D3C,   /* U+2B76 */
  0x0D3D,   /* U+2B77 */
  0x0D3E,   /* U+2B78 */
  0x0D3F,   /* U+2B79 */
  0x0D40,   /* U+2B7A */
  0x0D41,   /* U+2B7B */
  0x0D42,   /* U+2B7C */
  0x0D43,   /* U+2B7D */
  0x0D44,   /* U+2B7E */
  0x0D45,   /* U+2B7F */
  0x0D46,   /* U+2B80 */
  0x0D47,   /* U+2B81 */
  0x0D48,   /* U+2B82 */
  0x0D49,   /* U+2B83 */
  0x0D4A,   /* U+2B84 */
  0x0D4B,   /* U+2B85 */
  0x0D4C,   /* U+2B86 */
  0x0D4D,   /* U+2B87 */
  0x0D4E,   /* U+2B88 */
  0x0D4F,   /* U+2B89 */
  0x0D50,   /* U+2B8A */
  0x0D51,   /* U+2B8B */
  0x0D52,   /* U+2B8C */
  0x0D53,   /* U+2B8D */
  0x0D54,   /* U+2B8E */
  0x0D55,   /* U+2B8F */
  0x0D56,   /* U+2B90 */
  0x0D57,   /* U+2B91 */
  0x0D58,   /* U+2B92 */
  0x0D59,   /* U+2B93 */
  0x0D5A,   /* U+2B94 */
  0x0D5B,   /* U+2B95 */
  0xFBC0,   /* U+2B96 */
  0xFBC0,   /* U+2B97 */
  0x0D5C,   /* U+2B98 */
  0x0D5D,   /* U+2B99 */
  0x0D5E,   /* U+2B9A */
  0x0D5F,   /* U+2B9B */
  0x0D60,   /* U+2B9C */
  0x0D61,   /* U+2B9D */
  0x0D62,   /* U+2B9E */
  0x0D63,   /* U+2B9F */
  0x0D64,   /* U+2BA0 */
  0x0D65,   /* U+2BA1 */
  0x0D66,   /* U+2BA2 */
  0x0D67,   /* U+2BA3 */
  0x0D68,   /* U+2BA4 */
  0x0D69,   /* U+2BA5 */
  0x0D6A,   /* U+2BA6 */
  0x0D6B,   /* U+2BA7 */
  0x0D6C,   /* U+2BA8 */
  0x0D6D,   /* U+2BA9 */
  0x0D6E,   /* U+2BAA */
  0x0D6F,   /* U+2BAB */
  0x0D70,   /* U+2BAC */
  0x0D71,   /* U+2BAD */
  0x0D72,   /* U+2BAE */
  0x0D73,   /* U+2BAF */
  0x0D74,   /* U+2BB0 */
  0x0D75,   /* U+2BB1 */
  0x0D76,   /* U+2BB2 */
  0x0D77,   /* U+2BB3 */
  0x0D78,   /* U+2BB4 */
  0x0D79,   /* U+2BB5 */
  0x0D7A,   /* U+2BB6 */
  0x0D7B,   /* U+2BB7 */
  0x0D7C,   /* U+2BB8 */
  0x0D7D,   /* U+2BB9 */
  0xFBC0,   /* U+2BBA */
  0xFBC0,   /* U+2BBB */
  0xFBC0,   /* U+2BBC */
  0x0D7E,   /* U+2BBD */
  0x0D7F,   /* U+2BBE */
  0x0D80,   /* U+2BBF */
  0x0D81,   /* U+2BC0 */
  0x0D82,   /* U+2BC1 */
  0x0D83,   /* U+2BC2 */
  0x0D84,   /* U+2BC3 */
  0x0D85,   /* U+2BC4 */
  0x0D86,   /* U+2BC5 */
  0x0D87,   /* U+2BC6 */
  0x0D88,   /* U+2BC7 */
  0x0D89,   /* U+2BC8 */
  0xFBC0,   /* U+2BC9 */
  0x0D8A,   /* U+2BCA */
  0x0D8B,   /* U+2BCB */
  0x0D8C,   /* U+2BCC */
  0x0D8D,   /* U+2BCD */
  0x0D8E,   /* U+2BCE */
  0x0D8F,   /* U+2BCF */
  0x0D90,   /* U+2BD0 */
  0x0D91,   /* U+2BD1 */
  0xFBC0,   /* U+2BD2 */
  0xFBC0,   /* U+2BD3 */
  0xFBC0,   /* U+2BD4 */
  0xFBC0,   /* U+2BD5 */
  0xFBC0,   /* U+2BD6 */
  0xFBC0,   /* U+2BD7 */
  0xFBC0,   /* U+2BD8 */
  0xFBC0,   /* U+2BD9 */
  0xFBC0,   /* U+2BDA */
  0xFBC0,   /* U+2BDB */
  0xFBC0,   /* U+2BDC */
  0xFBC0,   /* U+2BDD */
  0xFBC0,   /* U+2BDE */
  0xFBC0,   /* U+2BDF */
  0xFBC0,   /* U+2BE0 */
  0xFBC0,   /* U+2BE1 */
  0xFBC0,   /* U+2BE2 */
  0xFBC0,   /* U+2BE3 */
  0xFBC0,   /* U+2BE4 */
  0xFBC0,   /* U+2BE5 */
  0xFBC0,   /* U+2BE6 */
  0xFBC0,   /* U+2BE7 */
  0xFBC0,   /* U+2BE8 */
  0xFBC0,   /* U+2BE9 */
  0xFBC0,   /* U+2BEA */
  0xFBC0,   /* U+2BEB */
  0x0D92,   /* U+2BEC */
  0x0D93,   /* U+2BED */
  0x0D94,   /* U+2BEE */
  0x0D95,   /* U+2BEF */
  0xFBC0,   /* U+2BF0 */
  0xFBC0,   /* U+2BF1 */
  0xFBC0,   /* U+2BF2 */
  0xFBC0,   /* U+2BF3 */
  0xFBC0,   /* U+2BF4 */
  0xFBC0,   /* U+2BF5 */
  0xFBC0,   /* U+2BF6 */
  0xFBC0,   /* U+2BF7 */
  0xFBC0,   /* U+2BF8 */
  0xFBC0,   /* U+2BF9 */
  0xFBC0,   /* U+2BFA */
  0xFBC0,   /* U+2BFB */
  0xFBC0,   /* U+2BFC */
  0xFBC0,   /* U+2BFD */
  0xFBC0,   /* U+2BFE */
  0xFBC0,   /* U+2BFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2B00 */
  0x0020,   /* U+2B01 */
  0x0020,   /* U+2B02 */
  0x0020,   /* U+2B03 */
  0x0020,   /* U+2B04 */
  0x0020,   /* U+2B05 */
  0x0020,   /* U+2B06 */
  0x0020,   /* U+2B07 */
  0x0020,   /* U+2B08 */
  0x0020,   /* U+2B09 */
  0x0020,   /* U+2B0A */
  0x0020,   /* U+2B0B */
  0x0020,   /* U+2B0C */
  0x0020,   /* U+2B0D */
  0x0020,   /* U+2B0E */
  0x0020,   /* U+2B0F */
  0x0020,   /* U+2B10 */
  0x0020,   /* U+2B11 */
  0x0020,   /* U+2B12 */
  0x0020,   /* U+2B13 */
  0x0020,   /* U+2B14 */
  0x0020,   /* U+2B15 */
  0x0020,   /* U+2B16 */
  0x0020,   /* U+2B17 */
  0x0020,   /* U+2B18 */
  0x0020,   /* U+2B19 */
  0x0020,   /* U+2B1A */
  0x0020,   /* U+2B1B */
  0x0020,   /* U+2B1C */
  0x0020,   /* U+2B1D */
  0x0020,   /* U+2B1E */
  0x0020,   /* U+2B1F */
  0x0020,   /* U+2B20 */
  0x0020,   /* U+2B21 */
  0x0020,   /* U+2B22 */
  0x0020,   /* U+2B23 */
  0x0020,   /* U+2B24 */
  0x0020,   /* U+2B25 */
  0x0020,   /* U+2B26 */
  0x0020,   /* U+2B27 */
  0x0020,   /* U+2B28 */
  0x0020,   /* U+2B29 */
  0x0020,   /* U+2B2A */
  0x0020,   /* U+2B2B */
  0x0020,   /* U+2B2C */
  0x0020,   /* U+2B2D */
  0x0020,   /* U+2B2E */
  0x0020,   /* U+2B2F */
  0x0020,   /* U+2B30 */
  0x0020,   /* U+2B31 */
  0x0020,   /* U+2B32 */
  0x0020,   /* U+2B33 */
  0x0020,   /* U+2B34 */
  0x0020,   /* U+2B35 */
  0x0020,   /* U+2B36 */
  0x0020,   /* U+2B37 */
  0x0020,   /* U+2B38 */
  0x0020,   /* U+2B39 */
  0x0020,   /* U+2B3A */
  0x0020,   /* U+2B3B */
  0x0020,   /* U+2B3C */
  0x0020,   /* U+2B3D */
  0x0020,   /* U+2B3E */
  0x0020,   /* U+2B3F */
  0x0020,   /* U+2B40 */
  0x0020,   /* U+2B41 */
  0x0020,   /* U+2B42 */
  0x0020,   /* U+2B43 */
  0x0020,   /* U+2B44 */
  0x0020,   /* U+2B45 */
  0x0020,   /* U+2B46 */
  0x0020,   /* U+2B47 */
  0x0020,   /* U+2B48 */
  0x0020,   /* U+2B49 */
  0x0020,   /* U+2B4A */
  0x0020,   /* U+2B4B */
  0x0020,   /* U+2B4C */
  0x0020,   /* U+2B4D */
  0x0020,   /* U+2B4E */
  0x0020,   /* U+2B4F */
  0x0020,   /* U+2B50 */
  0x0020,   /* U+2B51 */
  0x0020,   /* U+2B52 */
  0x0020,   /* U+2B53 */
  0x0020,   /* U+2B54 */
  0x0020,   /* U+2B55 */
  0x0020,   /* U+2B56 */
  0x0020,   /* U+2B57 */
  0x0020,   /* U+2B58 */
  0x0020,   /* U+2B59 */
  0x0020,   /* U+2B5A */
  0x0020,   /* U+2B5B */
  0x0020,   /* U+2B5C */
  0x0020,   /* U+2B5D */
  0x0020,   /* U+2B5E */
  0x0020,   /* U+2B5F */
  0x0020,   /* U+2B60 */
  0x0020,   /* U+2B61 */
  0x0020,   /* U+2B62 */
  0x0020,   /* U+2B63 */
  0x0020,   /* U+2B64 */
  0x0020,   /* U+2B65 */
  0x0020,   /* U+2B66 */
  0x0020,   /* U+2B67 */
  0x0020,   /* U+2B68 */
  0x0020,   /* U+2B69 */
  0x0020,   /* U+2B6A */
  0x0020,   /* U+2B6B */
  0x0020,   /* U+2B6C */
  0x0020,   /* U+2B6D */
  0x0020,   /* U+2B6E */
  0x0020,   /* U+2B6F */
  0x0020,   /* U+2B70 */
  0x0020,   /* U+2B71 */
  0x0020,   /* U+2B72 */
  0x0020,   /* U+2B73 */
  0x0020,   /* U+2B74 */
  0x0020,   /* U+2B75 */
  0x0020,   /* U+2B76 */
  0x0020,   /* U+2B77 */
  0x0020,   /* U+2B78 */
  0x0020,   /* U+2B79 */
  0x0020,   /* U+2B7A */
  0x0020,   /* U+2B7B */
  0x0020,   /* U+2B7C */
  0x0020,   /* U+2B7D */
  0x0020,   /* U+2B7E */
  0x0020,   /* U+2B7F */
  0x0020,   /* U+2B80 */
  0x0020,   /* U+2B81 */
  0x0020,   /* U+2B82 */
  0x0020,   /* U+2B83 */
  0x0020,   /* U+2B84 */
  0x0020,   /* U+2B85 */
  0x0020,   /* U+2B86 */
  0x0020,   /* U+2B87 */
  0x0020,   /* U+2B88 */
  0x0020,   /* U+2B89 */
  0x0020,   /* U+2B8A */
  0x0020,   /* U+2B8B */
  0x0020,   /* U+2B8C */
  0x0020,   /* U+2B8D */
  0x0020,   /* U+2B8E */
  0x0020,   /* U+2B8F */
  0x0020,   /* U+2B90 */
  0x0020,   /* U+2B91 */
  0x0020,   /* U+2B92 */
  0x0020,   /* U+2B93 */
  0x0020,   /* U+2B94 */
  0x0020,   /* U+2B95 */
  0x0020,   /* U+2B96 */
  0x0020,   /* U+2B97 */
  0x0020,   /* U+2B98 */
  0x0020,   /* U+2B99 */
  0x0020,   /* U+2B9A */
  0x0020,   /* U+2B9B */
  0x0020,   /* U+2B9C */
  0x0020,   /* U+2B9D */
  0x0020,   /* U+2B9E */
  0x0020,   /* U+2B9F */
  0x0020,   /* U+2BA0 */
  0x0020,   /* U+2BA1 */
  0x0020,   /* U+2BA2 */
  0x0020,   /* U+2BA3 */
  0x0020,   /* U+2BA4 */
  0x0020,   /* U+2BA5 */
  0x0020,   /* U+2BA6 */
  0x0020,   /* U+2BA7 */
  0x0020,   /* U+2BA8 */
  0x0020,   /* U+2BA9 */
  0x0020,   /* U+2BAA */
  0x0020,   /* U+2BAB */
  0x0020,   /* U+2BAC */
  0x0020,   /* U+2BAD */
  0x0020,   /* U+2BAE */
  0x0020,   /* U+2BAF */
  0x0020,   /* U+2BB0 */
  0x0020,   /* U+2BB1 */
  0x0020,   /* U+2BB2 */
  0x0020,   /* U+2BB3 */
  0x0020,   /* U+2BB4 */
  0x0020,   /* U+2BB5 */
  0x0020,   /* U+2BB6 */
  0x0020,   /* U+2BB7 */
  0x0020,   /* U+2BB8 */
  0x0020,   /* U+2BB9 */
  0x0020,   /* U+2BBA */
  0x0020,   /* U+2BBB */
  0x0020,   /* U+2BBC */
  0x0020,   /* U+2BBD */
  0x0020,   /* U+2BBE */
  0x0020,   /* U+2BBF */
  0x0020,   /* U+2BC0 */
  0x0020,   /* U+2BC1 */
  0x0020,   /* U+2BC2 */
  0x0020,   /* U+2BC3 */
  0x0020,   /* U+2BC4 */
  0x0020,   /* U+2BC5 */
  0x0020,   /* U+2BC6 */
  0x0020,   /* U+2BC7 */
  0x0020,   /* U+2BC8 */
  0x0020,   /* U+2BC9 */
  0x0020,   /* U+2BCA */
  0x0020,   /* U+2BCB */
  0x0020,   /* U+2BCC */
  0x0020,   /* U+2BCD */
  0x0020,   /* U+2BCE */
  0x0020,   /* U+2BCF */
  0x0020,   /* U+2BD0 */
  0x0020,   /* U+2BD1 */
  0x0020,   /* U+2BD2 */
  0x0020,   /* U+2BD3 */
  0x0020,   /* U+2BD4 */
  0x0020,   /* U+2BD5 */
  0x0020,   /* U+2BD6 */
  0x0020,   /* U+2BD7 */
  0x0020,   /* U+2BD8 */
  0x0020,   /* U+2BD9 */
  0x0020,   /* U+2BDA */
  0x0020,   /* U+2BDB */
  0x0020,   /* U+2BDC */
  0x0020,   /* U+2BDD */
  0x0020,   /* U+2BDE */
  0x0020,   /* U+2BDF */
  0x0020,   /* U+2BE0 */
  0x0020,   /* U+2BE1 */
  0x0020,   /* U+2BE2 */
  0x0020,   /* U+2BE3 */
  0x0020,   /* U+2BE4 */
  0x0020,   /* U+2BE5 */
  0x0020,   /* U+2BE6 */
  0x0020,   /* U+2BE7 */
  0x0020,   /* U+2BE8 */
  0x0020,   /* U+2BE9 */
  0x0020,   /* U+2BEA */
  0x0020,   /* U+2BEB */
  0x0020,   /* U+2BEC */
  0x0020,   /* U+2BED */
  0x0020,   /* U+2BEE */
  0x0020,   /* U+2BEF */
  0x0020,   /* U+2BF0 */
  0x0020,   /* U+2BF1 */
  0x0020,   /* U+2BF2 */
  0x0020,   /* U+2BF3 */
  0x0020,   /* U+2BF4 */
  0x0020,   /* U+2BF5 */
  0x0020,   /* U+2BF6 */
  0x0020,   /* U+2BF7 */
  0x0020,   /* U+2BF8 */
  0x0020,   /* U+2BF9 */
  0x0020,   /* U+2BFA */
  0x0020,   /* U+2BFB */
  0x0020,   /* U+2BFC */
  0x0020,   /* U+2BFD */
  0x0020,   /* U+2BFE */
  0x0020,   /* U+2BFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2B00 */
  0x0002,   /* U+2B01 */
  0x0002,   /* U+2B02 */
  0x0002,   /* U+2B03 */
  0x0002,   /* U+2B04 */
  0x0002,   /* U+2B05 */
  0x0002,   /* U+2B06 */
  0x0002,   /* U+2B07 */
  0x0002,   /* U+2B08 */
  0x0002,   /* U+2B09 */
  0x0002,   /* U+2B0A */
  0x0002,   /* U+2B0B */
  0x0002,   /* U+2B0C */
  0x0002,   /* U+2B0D */
  0x0002,   /* U+2B0E */
  0x0002,   /* U+2B0F */
  0x0002,   /* U+2B10 */
  0x0002,   /* U+2B11 */
  0x0002,   /* U+2B12 */
  0x0002,   /* U+2B13 */
  0x0002,   /* U+2B14 */
  0x0002,   /* U+2B15 */
  0x0002,   /* U+2B16 */
  0x0002,   /* U+2B17 */
  0x0002,   /* U+2B18 */
  0x0002,   /* U+2B19 */
  0x0002,   /* U+2B1A */
  0x0002,   /* U+2B1B */
  0x0002,   /* U+2B1C */
  0x0002,   /* U+2B1D */
  0x0002,   /* U+2B1E */
  0x0002,   /* U+2B1F */
  0x0002,   /* U+2B20 */
  0x0002,   /* U+2B21 */
  0x0002,   /* U+2B22 */
  0x0002,   /* U+2B23 */
  0x0002,   /* U+2B24 */
  0x0002,   /* U+2B25 */
  0x0002,   /* U+2B26 */
  0x0002,   /* U+2B27 */
  0x0002,   /* U+2B28 */
  0x0002,   /* U+2B29 */
  0x0002,   /* U+2B2A */
  0x0002,   /* U+2B2B */
  0x0002,   /* U+2B2C */
  0x0002,   /* U+2B2D */
  0x0002,   /* U+2B2E */
  0x0002,   /* U+2B2F */
  0x0002,   /* U+2B30 */
  0x0002,   /* U+2B31 */
  0x0002,   /* U+2B32 */
  0x0002,   /* U+2B33 */
  0x0002,   /* U+2B34 */
  0x0002,   /* U+2B35 */
  0x0002,   /* U+2B36 */
  0x0002,   /* U+2B37 */
  0x0002,   /* U+2B38 */
  0x0002,   /* U+2B39 */
  0x0002,   /* U+2B3A */
  0x0002,   /* U+2B3B */
  0x0002,   /* U+2B3C */
  0x0002,   /* U+2B3D */
  0x0002,   /* U+2B3E */
  0x0002,   /* U+2B3F */
  0x0002,   /* U+2B40 */
  0x0002,   /* U+2B41 */
  0x0002,   /* U+2B42 */
  0x0002,   /* U+2B43 */
  0x0002,   /* U+2B44 */
  0x0002,   /* U+2B45 */
  0x0002,   /* U+2B46 */
  0x0002,   /* U+2B47 */
  0x0002,   /* U+2B48 */
  0x0002,   /* U+2B49 */
  0x0002,   /* U+2B4A */
  0x0002,   /* U+2B4B */
  0x0002,   /* U+2B4C */
  0x0002,   /* U+2B4D */
  0x0002,   /* U+2B4E */
  0x0002,   /* U+2B4F */
  0x0002,   /* U+2B50 */
  0x0002,   /* U+2B51 */
  0x0002,   /* U+2B52 */
  0x0002,   /* U+2B53 */
  0x0002,   /* U+2B54 */
  0x0002,   /* U+2B55 */
  0x0002,   /* U+2B56 */
  0x0002,   /* U+2B57 */
  0x0002,   /* U+2B58 */
  0x0002,   /* U+2B59 */
  0x0002,   /* U+2B5A */
  0x0002,   /* U+2B5B */
  0x0002,   /* U+2B5C */
  0x0002,   /* U+2B5D */
  0x0002,   /* U+2B5E */
  0x0002,   /* U+2B5F */
  0x0002,   /* U+2B60 */
  0x0002,   /* U+2B61 */
  0x0002,   /* U+2B62 */
  0x0002,   /* U+2B63 */
  0x0002,   /* U+2B64 */
  0x0002,   /* U+2B65 */
  0x0002,   /* U+2B66 */
  0x0002,   /* U+2B67 */
  0x0002,   /* U+2B68 */
  0x0002,   /* U+2B69 */
  0x0002,   /* U+2B6A */
  0x0002,   /* U+2B6B */
  0x0002,   /* U+2B6C */
  0x0002,   /* U+2B6D */
  0x0002,   /* U+2B6E */
  0x0002,   /* U+2B6F */
  0x0002,   /* U+2B70 */
  0x0002,   /* U+2B71 */
  0x0002,   /* U+2B72 */
  0x0002,   /* U+2B73 */
  0x0002,   /* U+2B74 */
  0x0002,   /* U+2B75 */
  0x0002,   /* U+2B76 */
  0x0002,   /* U+2B77 */
  0x0002,   /* U+2B78 */
  0x0002,   /* U+2B79 */
  0x0002,   /* U+2B7A */
  0x0002,   /* U+2B7B */
  0x0002,   /* U+2B7C */
  0x0002,   /* U+2B7D */
  0x0002,   /* U+2B7E */
  0x0002,   /* U+2B7F */
  0x0002,   /* U+2B80 */
  0x0002,   /* U+2B81 */
  0x0002,   /* U+2B82 */
  0x0002,   /* U+2B83 */
  0x0002,   /* U+2B84 */
  0x0002,   /* U+2B85 */
  0x0002,   /* U+2B86 */
  0x0002,   /* U+2B87 */
  0x0002,   /* U+2B88 */
  0x0002,   /* U+2B89 */
  0x0002,   /* U+2B8A */
  0x0002,   /* U+2B8B */
  0x0002,   /* U+2B8C */
  0x0002,   /* U+2B8D */
  0x0002,   /* U+2B8E */
  0x0002,   /* U+2B8F */
  0x0002,   /* U+2B90 */
  0x0002,   /* U+2B91 */
  0x0002,   /* U+2B92 */
  0x0002,   /* U+2B93 */
  0x0002,   /* U+2B94 */
  0x0002,   /* U+2B95 */
  0x0002,   /* U+2B96 */
  0x0002,   /* U+2B97 */
  0x0002,   /* U+2B98 */
  0x0002,   /* U+2B99 */
  0x0002,   /* U+2B9A */
  0x0002,   /* U+2B9B */
  0x0002,   /* U+2B9C */
  0x0002,   /* U+2B9D */
  0x0002,   /* U+2B9E */
  0x0002,   /* U+2B9F */
  0x0002,   /* U+2BA0 */
  0x0002,   /* U+2BA1 */
  0x0002,   /* U+2BA2 */
  0x0002,   /* U+2BA3 */
  0x0002,   /* U+2BA4 */
  0x0002,   /* U+2BA5 */
  0x0002,   /* U+2BA6 */
  0x0002,   /* U+2BA7 */
  0x0002,   /* U+2BA8 */
  0x0002,   /* U+2BA9 */
  0x0002,   /* U+2BAA */
  0x0002,   /* U+2BAB */
  0x0002,   /* U+2BAC */
  0x0002,   /* U+2BAD */
  0x0002,   /* U+2BAE */
  0x0002,   /* U+2BAF */
  0x0002,   /* U+2BB0 */
  0x0002,   /* U+2BB1 */
  0x0002,   /* U+2BB2 */
  0x0002,   /* U+2BB3 */
  0x0002,   /* U+2BB4 */
  0x0002,   /* U+2BB5 */
  0x0002,   /* U+2BB6 */
  0x0002,   /* U+2BB7 */
  0x0002,   /* U+2BB8 */
  0x0002,   /* U+2BB9 */
  0x0002,   /* U+2BBA */
  0x0002,   /* U+2BBB */
  0x0002,   /* U+2BBC */
  0x0002,   /* U+2BBD */
  0x0002,   /* U+2BBE */
  0x0002,   /* U+2BBF */
  0x0002,   /* U+2BC0 */
  0x0002,   /* U+2BC1 */
  0x0002,   /* U+2BC2 */
  0x0002,   /* U+2BC3 */
  0x0002,   /* U+2BC4 */
  0x0002,   /* U+2BC5 */
  0x0002,   /* U+2BC6 */
  0x0002,   /* U+2BC7 */
  0x0002,   /* U+2BC8 */
  0x0002,   /* U+2BC9 */
  0x0002,   /* U+2BCA */
  0x0002,   /* U+2BCB */
  0x0002,   /* U+2BCC */
  0x0002,   /* U+2BCD */
  0x0002,   /* U+2BCE */
  0x0002,   /* U+2BCF */
  0x0002,   /* U+2BD0 */
  0x0002,   /* U+2BD1 */
  0x0002,   /* U+2BD2 */
  0x0002,   /* U+2BD3 */
  0x0002,   /* U+2BD4 */
  0x0002,   /* U+2BD5 */
  0x0002,   /* U+2BD6 */
  0x0002,   /* U+2BD7 */
  0x0002,   /* U+2BD8 */
  0x0002,   /* U+2BD9 */
  0x0002,   /* U+2BDA */
  0x0002,   /* U+2BDB */
  0x0002,   /* U+2BDC */
  0x0002,   /* U+2BDD */
  0x0002,   /* U+2BDE */
  0x0002,   /* U+2BDF */
  0x0002,   /* U+2BE0 */
  0x0002,   /* U+2BE1 */
  0x0002,   /* U+2BE2 */
  0x0002,   /* U+2BE3 */
  0x0002,   /* U+2BE4 */
  0x0002,   /* U+2BE5 */
  0x0002,   /* U+2BE6 */
  0x0002,   /* U+2BE7 */
  0x0002,   /* U+2BE8 */
  0x0002,   /* U+2BE9 */
  0x0002,   /* U+2BEA */
  0x0002,   /* U+2BEB */
  0x0002,   /* U+2BEC */
  0x0002,   /* U+2BED */
  0x0002,   /* U+2BEE */
  0x0002,   /* U+2BEF */
  0x0002,   /* U+2BF0 */
  0x0002,   /* U+2BF1 */
  0x0002,   /* U+2BF2 */
  0x0002,   /* U+2BF3 */
  0x0002,   /* U+2BF4 */
  0x0002,   /* U+2BF5 */
  0x0002,   /* U+2BF6 */
  0x0002,   /* U+2BF7 */
  0x0002,   /* U+2BF8 */
  0x0002,   /* U+2BF9 */
  0x0002,   /* U+2BFA */
  0x0002,   /* U+2BFB */
  0x0002,   /* U+2BFC */
  0x0002,   /* U+2BFD */
  0x0002,   /* U+2BFE */
  0x0002,   /* U+2BFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2B00 */
  0x0000,   /* U+2B01 */
  0x0000,   /* U+2B02 */
  0x0000,   /* U+2B03 */
  0x0000,   /* U+2B04 */
  0x0000,   /* U+2B05 */
  0x0000,   /* U+2B06 */
  0x0000,   /* U+2B07 */
  0x0000,   /* U+2B08 */
  0x0000,   /* U+2B09 */
  0x0000,   /* U+2B0A */
  0x0000,   /* U+2B0B */
  0x0000,   /* U+2B0C */
  0x0000,   /* U+2B0D */
  0x0000,   /* U+2B0E */
  0x0000,   /* U+2B0F */
  0x0000,   /* U+2B10 */
  0x0000,   /* U+2B11 */
  0x0000,   /* U+2B12 */
  0x0000,   /* U+2B13 */
  0x0000,   /* U+2B14 */
  0x0000,   /* U+2B15 */
  0x0000,   /* U+2B16 */
  0x0000,   /* U+2B17 */
  0x0000,   /* U+2B18 */
  0x0000,   /* U+2B19 */
  0x0000,   /* U+2B1A */
  0x0000,   /* U+2B1B */
  0x0000,   /* U+2B1C */
  0x0000,   /* U+2B1D */
  0x0000,   /* U+2B1E */
  0x0000,   /* U+2B1F */
  0x0000,   /* U+2B20 */
  0x0000,   /* U+2B21 */
  0x0000,   /* U+2B22 */
  0x0000,   /* U+2B23 */
  0x0000,   /* U+2B24 */
  0x0000,   /* U+2B25 */
  0x0000,   /* U+2B26 */
  0x0000,   /* U+2B27 */
  0x0000,   /* U+2B28 */
  0x0000,   /* U+2B29 */
  0x0000,   /* U+2B2A */
  0x0000,   /* U+2B2B */
  0x0000,   /* U+2B2C */
  0x0000,   /* U+2B2D */
  0x0000,   /* U+2B2E */
  0x0000,   /* U+2B2F */
  0x0000,   /* U+2B30 */
  0x0000,   /* U+2B31 */
  0x0000,   /* U+2B32 */
  0x0000,   /* U+2B33 */
  0x0000,   /* U+2B34 */
  0x0000,   /* U+2B35 */
  0x0000,   /* U+2B36 */
  0x0000,   /* U+2B37 */
  0x0000,   /* U+2B38 */
  0x0000,   /* U+2B39 */
  0x0000,   /* U+2B3A */
  0x0000,   /* U+2B3B */
  0x0000,   /* U+2B3C */
  0x0000,   /* U+2B3D */
  0x0000,   /* U+2B3E */
  0x0000,   /* U+2B3F */
  0x0000,   /* U+2B40 */
  0x0000,   /* U+2B41 */
  0x0000,   /* U+2B42 */
  0x0000,   /* U+2B43 */
  0x0000,   /* U+2B44 */
  0x0000,   /* U+2B45 */
  0x0000,   /* U+2B46 */
  0x0000,   /* U+2B47 */
  0x0000,   /* U+2B48 */
  0x0000,   /* U+2B49 */
  0x0000,   /* U+2B4A */
  0x0000,   /* U+2B4B */
  0x0000,   /* U+2B4C */
  0x0000,   /* U+2B4D */
  0x0000,   /* U+2B4E */
  0x0000,   /* U+2B4F */
  0x0000,   /* U+2B50 */
  0x0000,   /* U+2B51 */
  0x0000,   /* U+2B52 */
  0x0000,   /* U+2B53 */
  0x0000,   /* U+2B54 */
  0x0000,   /* U+2B55 */
  0x0000,   /* U+2B56 */
  0x0000,   /* U+2B57 */
  0x0000,   /* U+2B58 */
  0x0000,   /* U+2B59 */
  0x0000,   /* U+2B5A */
  0x0000,   /* U+2B5B */
  0x0000,   /* U+2B5C */
  0x0000,   /* U+2B5D */
  0x0000,   /* U+2B5E */
  0x0000,   /* U+2B5F */
  0x0000,   /* U+2B60 */
  0x0000,   /* U+2B61 */
  0x0000,   /* U+2B62 */
  0x0000,   /* U+2B63 */
  0x0000,   /* U+2B64 */
  0x0000,   /* U+2B65 */
  0x0000,   /* U+2B66 */
  0x0000,   /* U+2B67 */
  0x0000,   /* U+2B68 */
  0x0000,   /* U+2B69 */
  0x0000,   /* U+2B6A */
  0x0000,   /* U+2B6B */
  0x0000,   /* U+2B6C */
  0x0000,   /* U+2B6D */
  0x0000,   /* U+2B6E */
  0x0000,   /* U+2B6F */
  0x0000,   /* U+2B70 */
  0x0000,   /* U+2B71 */
  0x0000,   /* U+2B72 */
  0x0000,   /* U+2B73 */
  0xAB74,   /* U+2B74 */
  0xAB75,   /* U+2B75 */
  0x0000,   /* U+2B76 */
  0x0000,   /* U+2B77 */
  0x0000,   /* U+2B78 */
  0x0000,   /* U+2B79 */
  0x0000,   /* U+2B7A */
  0x0000,   /* U+2B7B */
  0x0000,   /* U+2B7C */
  0x0000,   /* U+2B7D */
  0x0000,   /* U+2B7E */
  0x0000,   /* U+2B7F */
  0x0000,   /* U+2B80 */
  0x0000,   /* U+2B81 */
  0x0000,   /* U+2B82 */
  0x0000,   /* U+2B83 */
  0x0000,   /* U+2B84 */
  0x0000,   /* U+2B85 */
  0x0000,   /* U+2B86 */
  0x0000,   /* U+2B87 */
  0x0000,   /* U+2B88 */
  0x0000,   /* U+2B89 */
  0x0000,   /* U+2B8A */
  0x0000,   /* U+2B8B */
  0x0000,   /* U+2B8C */
  0x0000,   /* U+2B8D */
  0x0000,   /* U+2B8E */
  0x0000,   /* U+2B8F */
  0x0000,   /* U+2B90 */
  0x0000,   /* U+2B91 */
  0x0000,   /* U+2B92 */
  0x0000,   /* U+2B93 */
  0x0000,   /* U+2B94 */
  0x0000,   /* U+2B95 */
  0xAB96,   /* U+2B96 */
  0xAB97,   /* U+2B97 */
  0x0000,   /* U+2B98 */
  0x0000,   /* U+2B99 */
  0x0000,   /* U+2B9A */
  0x0000,   /* U+2B9B */
  0x0000,   /* U+2B9C */
  0x0000,   /* U+2B9D */
  0x0000,   /* U+2B9E */
  0x0000,   /* U+2B9F */
  0x0000,   /* U+2BA0 */
  0x0000,   /* U+2BA1 */
  0x0000,   /* U+2BA2 */
  0x0000,   /* U+2BA3 */
  0x0000,   /* U+2BA4 */
  0x0000,   /* U+2BA5 */
  0x0000,   /* U+2BA6 */
  0x0000,   /* U+2BA7 */
  0x0000,   /* U+2BA8 */
  0x0000,   /* U+2BA9 */
  0x0000,   /* U+2BAA */
  0x0000,   /* U+2BAB */
  0x0000,   /* U+2BAC */
  0x0000,   /* U+2BAD */
  0x0000,   /* U+2BAE */
  0x0000,   /* U+2BAF */
  0x0000,   /* U+2BB0 */
  0x0000,   /* U+2BB1 */
  0x0000,   /* U+2BB2 */
  0x0000,   /* U+2BB3 */
  0x0000,   /* U+2BB4 */
  0x0000,   /* U+2BB5 */
  0x0000,   /* U+2BB6 */
  0x0000,   /* U+2BB7 */
  0x0000,   /* U+2BB8 */
  0x0000,   /* U+2BB9 */
  0xABBA,   /* U+2BBA */
  0xABBB,   /* U+2BBB */
  0xABBC,   /* U+2BBC */
  0x0000,   /* U+2BBD */
  0x0000,   /* U+2BBE */
  0x0000,   /* U+2BBF */
  0x0000,   /* U+2BC0 */
  0x0000,   /* U+2BC1 */
  0x0000,   /* U+2BC2 */
  0x0000,   /* U+2BC3 */
  0x0000,   /* U+2BC4 */
  0x0000,   /* U+2BC5 */
  0x0000,   /* U+2BC6 */
  0x0000,   /* U+2BC7 */
  0x0000,   /* U+2BC8 */
  0xABC9,   /* U+2BC9 */
  0x0000,   /* U+2BCA */
  0x0000,   /* U+2BCB */
  0x0000,   /* U+2BCC */
  0x0000,   /* U+2BCD */
  0x0000,   /* U+2BCE */
  0x0000,   /* U+2BCF */
  0x0000,   /* U+2BD0 */
  0x0000,   /* U+2BD1 */
  0xABD2,   /* U+2BD2 */
  0xABD3,   /* U+2BD3 */
  0xABD4,   /* U+2BD4 */
  0xABD5,   /* U+2BD5 */
  0xABD6,   /* U+2BD6 */
  0xABD7,   /* U+2BD7 */
  0xABD8,   /* U+2BD8 */
  0xABD9,   /* U+2BD9 */
  0xABDA,   /* U+2BDA */
  0xABDB,   /* U+2BDB */
  0xABDC,   /* U+2BDC */
  0xABDD,   /* U+2BDD */
  0xABDE,   /* U+2BDE */
  0xABDF,   /* U+2BDF */
  0xABE0,   /* U+2BE0 */
  0xABE1,   /* U+2BE1 */
  0xABE2,   /* U+2BE2 */
  0xABE3,   /* U+2BE3 */
  0xABE4,   /* U+2BE4 */
  0xABE5,   /* U+2BE5 */
  0xABE6,   /* U+2BE6 */
  0xABE7,   /* U+2BE7 */
  0xABE8,   /* U+2BE8 */
  0xABE9,   /* U+2BE9 */
  0xABEA,   /* U+2BEA */
  0xABEB,   /* U+2BEB */
  0x0000,   /* U+2BEC */
  0x0000,   /* U+2BED */
  0x0000,   /* U+2BEE */
  0x0000,   /* U+2BEF */
  0xABF0,   /* U+2BF0 */
  0xABF1,   /* U+2BF1 */
  0xABF2,   /* U+2BF2 */
  0xABF3,   /* U+2BF3 */
  0xABF4,   /* U+2BF4 */
  0xABF5,   /* U+2BF5 */
  0xABF6,   /* U+2BF6 */
  0xABF7,   /* U+2BF7 */
  0xABF8,   /* U+2BF8 */
  0xABF9,   /* U+2BF9 */
  0xABFA,   /* U+2BFA */
  0xABFB,   /* U+2BFB */
  0xABFC,   /* U+2BFC */
  0xABFD,   /* U+2BFD */
  0xABFE,   /* U+2BFE */
  0xABFF,   /* U+2BFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2B00 */
  0x0000,   /* U+2B01 */
  0x0000,   /* U+2B02 */
  0x0000,   /* U+2B03 */
  0x0000,   /* U+2B04 */
  0x0000,   /* U+2B05 */
  0x0000,   /* U+2B06 */
  0x0000,   /* U+2B07 */
  0x0000,   /* U+2B08 */
  0x0000,   /* U+2B09 */
  0x0000,   /* U+2B0A */
  0x0000,   /* U+2B0B */
  0x0000,   /* U+2B0C */
  0x0000,   /* U+2B0D */
  0x0000,   /* U+2B0E */
  0x0000,   /* U+2B0F */
  0x0000,   /* U+2B10 */
  0x0000,   /* U+2B11 */
  0x0000,   /* U+2B12 */
  0x0000,   /* U+2B13 */
  0x0000,   /* U+2B14 */
  0x0000,   /* U+2B15 */
  0x0000,   /* U+2B16 */
  0x0000,   /* U+2B17 */
  0x0000,   /* U+2B18 */
  0x0000,   /* U+2B19 */
  0x0000,   /* U+2B1A */
  0x0000,   /* U+2B1B */
  0x0000,   /* U+2B1C */
  0x0000,   /* U+2B1D */
  0x0000,   /* U+2B1E */
  0x0000,   /* U+2B1F */
  0x0000,   /* U+2B20 */
  0x0000,   /* U+2B21 */
  0x0000,   /* U+2B22 */
  0x0000,   /* U+2B23 */
  0x0000,   /* U+2B24 */
  0x0000,   /* U+2B25 */
  0x0000,   /* U+2B26 */
  0x0000,   /* U+2B27 */
  0x0000,   /* U+2B28 */
  0x0000,   /* U+2B29 */
  0x0000,   /* U+2B2A */
  0x0000,   /* U+2B2B */
  0x0000,   /* U+2B2C */
  0x0000,   /* U+2B2D */
  0x0000,   /* U+2B2E */
  0x0000,   /* U+2B2F */
  0x0000,   /* U+2B30 */
  0x0000,   /* U+2B31 */
  0x0000,   /* U+2B32 */
  0x0000,   /* U+2B33 */
  0x0000,   /* U+2B34 */
  0x0000,   /* U+2B35 */
  0x0000,   /* U+2B36 */
  0x0000,   /* U+2B37 */
  0x0000,   /* U+2B38 */
  0x0000,   /* U+2B39 */
  0x0000,   /* U+2B3A */
  0x0000,   /* U+2B3B */
  0x0000,   /* U+2B3C */
  0x0000,   /* U+2B3D */
  0x0000,   /* U+2B3E */
  0x0000,   /* U+2B3F */
  0x0000,   /* U+2B40 */
  0x0000,   /* U+2B41 */
  0x0000,   /* U+2B42 */
  0x0000,   /* U+2B43 */
  0x0000,   /* U+2B44 */
  0x0000,   /* U+2B45 */
  0x0000,   /* U+2B46 */
  0x0000,   /* U+2B47 */
  0x0000,   /* U+2B48 */
  0x0000,   /* U+2B49 */
  0x0000,   /* U+2B4A */
  0x0000,   /* U+2B4B */
  0x0000,   /* U+2B4C */
  0x0000,   /* U+2B4D */
  0x0000,   /* U+2B4E */
  0x0000,   /* U+2B4F */
  0x0000,   /* U+2B50 */
  0x0000,   /* U+2B51 */
  0x0000,   /* U+2B52 */
  0x0000,   /* U+2B53 */
  0x0000,   /* U+2B54 */
  0x0000,   /* U+2B55 */
  0x0000,   /* U+2B56 */
  0x0000,   /* U+2B57 */
  0x0000,   /* U+2B58 */
  0x0000,   /* U+2B59 */
  0x0000,   /* U+2B5A */
  0x0000,   /* U+2B5B */
  0x0000,   /* U+2B5C */
  0x0000,   /* U+2B5D */
  0x0000,   /* U+2B5E */
  0x0000,   /* U+2B5F */
  0x0000,   /* U+2B60 */
  0x0000,   /* U+2B61 */
  0x0000,   /* U+2B62 */
  0x0000,   /* U+2B63 */
  0x0000,   /* U+2B64 */
  0x0000,   /* U+2B65 */
  0x0000,   /* U+2B66 */
  0x0000,   /* U+2B67 */
  0x0000,   /* U+2B68 */
  0x0000,   /* U+2B69 */
  0x0000,   /* U+2B6A */
  0x0000,   /* U+2B6B */
  0x0000,   /* U+2B6C */
  0x0000,   /* U+2B6D */
  0x0000,   /* U+2B6E */
  0x0000,   /* U+2B6F */
  0x0000,   /* U+2B70 */
  0x0000,   /* U+2B71 */
  0x0000,   /* U+2B72 */
  0x0000,   /* U+2B73 */
  0x0000,   /* U+2B74 */
  0x0000,   /* U+2B75 */
  0x0000,   /* U+2B76 */
  0x0000,   /* U+2B77 */
  0x0000,   /* U+2B78 */
  0x0000,   /* U+2B79 */
  0x0000,   /* U+2B7A */
  0x0000,   /* U+2B7B */
  0x0000,   /* U+2B7C */
  0x0000,   /* U+2B7D */
  0x0000,   /* U+2B7E */
  0x0000,   /* U+2B7F */
  0x0000,   /* U+2B80 */
  0x0000,   /* U+2B81 */
  0x0000,   /* U+2B82 */
  0x0000,   /* U+2B83 */
  0x0000,   /* U+2B84 */
  0x0000,   /* U+2B85 */
  0x0000,   /* U+2B86 */
  0x0000,   /* U+2B87 */
  0x0000,   /* U+2B88 */
  0x0000,   /* U+2B89 */
  0x0000,   /* U+2B8A */
  0x0000,   /* U+2B8B */
  0x0000,   /* U+2B8C */
  0x0000,   /* U+2B8D */
  0x0000,   /* U+2B8E */
  0x0000,   /* U+2B8F */
  0x0000,   /* U+2B90 */
  0x0000,   /* U+2B91 */
  0x0000,   /* U+2B92 */
  0x0000,   /* U+2B93 */
  0x0000,   /* U+2B94 */
  0x0000,   /* U+2B95 */
  0x0000,   /* U+2B96 */
  0x0000,   /* U+2B97 */
  0x0000,   /* U+2B98 */
  0x0000,   /* U+2B99 */
  0x0000,   /* U+2B9A */
  0x0000,   /* U+2B9B */
  0x0000,   /* U+2B9C */
  0x0000,   /* U+2B9D */
  0x0000,   /* U+2B9E */
  0x0000,   /* U+2B9F */
  0x0000,   /* U+2BA0 */
  0x0000,   /* U+2BA1 */
  0x0000,   /* U+2BA2 */
  0x0000,   /* U+2BA3 */
  0x0000,   /* U+2BA4 */
  0x0000,   /* U+2BA5 */
  0x0000,   /* U+2BA6 */
  0x0000,   /* U+2BA7 */
  0x0000,   /* U+2BA8 */
  0x0000,   /* U+2BA9 */
  0x0000,   /* U+2BAA */
  0x0000,   /* U+2BAB */
  0x0000,   /* U+2BAC */
  0x0000,   /* U+2BAD */
  0x0000,   /* U+2BAE */
  0x0000,   /* U+2BAF */
  0x0000,   /* U+2BB0 */
  0x0000,   /* U+2BB1 */
  0x0000,   /* U+2BB2 */
  0x0000,   /* U+2BB3 */
  0x0000,   /* U+2BB4 */
  0x0000,   /* U+2BB5 */
  0x0000,   /* U+2BB6 */
  0x0000,   /* U+2BB7 */
  0x0000,   /* U+2BB8 */
  0x0000,   /* U+2BB9 */
  0x0000,   /* U+2BBA */
  0x0000,   /* U+2BBB */
  0x0000,   /* U+2BBC */
  0x0000,   /* U+2BBD */
  0x0000,   /* U+2BBE */
  0x0000,   /* U+2BBF */
  0x0000,   /* U+2BC0 */
  0x0000,   /* U+2BC1 */
  0x0000,   /* U+2BC2 */
  0x0000,   /* U+2BC3 */
  0x0000,   /* U+2BC4 */
  0x0000,   /* U+2BC5 */
  0x0000,   /* U+2BC6 */
  0x0000,   /* U+2BC7 */
  0x0000,   /* U+2BC8 */
  0x0000,   /* U+2BC9 */
  0x0000,   /* U+2BCA */
  0x0000,   /* U+2BCB */
  0x0000,   /* U+2BCC */
  0x0000,   /* U+2BCD */
  0x0000,   /* U+2BCE */
  0x0000,   /* U+2BCF */
  0x0000,   /* U+2BD0 */
  0x0000,   /* U+2BD1 */
  0x0000,   /* U+2BD2 */
  0x0000,   /* U+2BD3 */
  0x0000,   /* U+2BD4 */
  0x0000,   /* U+2BD5 */
  0x0000,   /* U+2BD6 */
  0x0000,   /* U+2BD7 */
  0x0000,   /* U+2BD8 */
  0x0000,   /* U+2BD9 */
  0x0000,   /* U+2BDA */
  0x0000,   /* U+2BDB */
  0x0000,   /* U+2BDC */
  0x0000,   /* U+2BDD */
  0x0000,   /* U+2BDE */
  0x0000,   /* U+2BDF */
  0x0000,   /* U+2BE0 */
  0x0000,   /* U+2BE1 */
  0x0000,   /* U+2BE2 */
  0x0000,   /* U+2BE3 */
  0x0000,   /* U+2BE4 */
  0x0000,   /* U+2BE5 */
  0x0000,   /* U+2BE6 */
  0x0000,   /* U+2BE7 */
  0x0000,   /* U+2BE8 */
  0x0000,   /* U+2BE9 */
  0x0000,   /* U+2BEA */
  0x0000,   /* U+2BEB */
  0x0000,   /* U+2BEC */
  0x0000,   /* U+2BED */
  0x0000,   /* U+2BEE */
  0x0000,   /* U+2BEF */
  0x0000,   /* U+2BF0 */
  0x0000,   /* U+2BF1 */
  0x0000,   /* U+2BF2 */
  0x0000,   /* U+2BF3 */
  0x0000,   /* U+2BF4 */
  0x0000,   /* U+2BF5 */
  0x0000,   /* U+2BF6 */
  0x0000,   /* U+2BF7 */
  0x0000,   /* U+2BF8 */
  0x0000,   /* U+2BF9 */
  0x0000,   /* U+2BFA */
  0x0000,   /* U+2BFB */
  0x0000,   /* U+2BFC */
  0x0000,   /* U+2BFD */
  0x0000,   /* U+2BFE */
  0x0000,   /* U+2BFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2B00 */
  0x0000,   /* U+2B01 */
  0x0000,   /* U+2B02 */
  0x0000,   /* U+2B03 */
  0x0000,   /* U+2B04 */
  0x0000,   /* U+2B05 */
  0x0000,   /* U+2B06 */
  0x0000,   /* U+2B07 */
  0x0000,   /* U+2B08 */
  0x0000,   /* U+2B09 */
  0x0000,   /* U+2B0A */
  0x0000,   /* U+2B0B */
  0x0000,   /* U+2B0C */
  0x0000,   /* U+2B0D */
  0x0000,   /* U+2B0E */
  0x0000,   /* U+2B0F */
  0x0000,   /* U+2B10 */
  0x0000,   /* U+2B11 */
  0x0000,   /* U+2B12 */
  0x0000,   /* U+2B13 */
  0x0000,   /* U+2B14 */
  0x0000,   /* U+2B15 */
  0x0000,   /* U+2B16 */
  0x0000,   /* U+2B17 */
  0x0000,   /* U+2B18 */
  0x0000,   /* U+2B19 */
  0x0000,   /* U+2B1A */
  0x0000,   /* U+2B1B */
  0x0000,   /* U+2B1C */
  0x0000,   /* U+2B1D */
  0x0000,   /* U+2B1E */
  0x0000,   /* U+2B1F */
  0x0000,   /* U+2B20 */
  0x0000,   /* U+2B21 */
  0x0000,   /* U+2B22 */
  0x0000,   /* U+2B23 */
  0x0000,   /* U+2B24 */
  0x0000,   /* U+2B25 */
  0x0000,   /* U+2B26 */
  0x0000,   /* U+2B27 */
  0x0000,   /* U+2B28 */
  0x0000,   /* U+2B29 */
  0x0000,   /* U+2B2A */
  0x0000,   /* U+2B2B */
  0x0000,   /* U+2B2C */
  0x0000,   /* U+2B2D */
  0x0000,   /* U+2B2E */
  0x0000,   /* U+2B2F */
  0x0000,   /* U+2B30 */
  0x0000,   /* U+2B31 */
  0x0000,   /* U+2B32 */
  0x0000,   /* U+2B33 */
  0x0000,   /* U+2B34 */
  0x0000,   /* U+2B35 */
  0x0000,   /* U+2B36 */
  0x0000,   /* U+2B37 */
  0x0000,   /* U+2B38 */
  0x0000,   /* U+2B39 */
  0x0000,   /* U+2B3A */
  0x0000,   /* U+2B3B */
  0x0000,   /* U+2B3C */
  0x0000,   /* U+2B3D */
  0x0000,   /* U+2B3E */
  0x0000,   /* U+2B3F */
  0x0000,   /* U+2B40 */
  0x0000,   /* U+2B41 */
  0x0000,   /* U+2B42 */
  0x0000,   /* U+2B43 */
  0x0000,   /* U+2B44 */
  0x0000,   /* U+2B45 */
  0x0000,   /* U+2B46 */
  0x0000,   /* U+2B47 */
  0x0000,   /* U+2B48 */
  0x0000,   /* U+2B49 */
  0x0000,   /* U+2B4A */
  0x0000,   /* U+2B4B */
  0x0000,   /* U+2B4C */
  0x0000,   /* U+2B4D */
  0x0000,   /* U+2B4E */
  0x0000,   /* U+2B4F */
  0x0000,   /* U+2B50 */
  0x0000,   /* U+2B51 */
  0x0000,   /* U+2B52 */
  0x0000,   /* U+2B53 */
  0x0000,   /* U+2B54 */
  0x0000,   /* U+2B55 */
  0x0000,   /* U+2B56 */
  0x0000,   /* U+2B57 */
  0x0000,   /* U+2B58 */
  0x0000,   /* U+2B59 */
  0x0000,   /* U+2B5A */
  0x0000,   /* U+2B5B */
  0x0000,   /* U+2B5C */
  0x0000,   /* U+2B5D */
  0x0000,   /* U+2B5E */
  0x0000,   /* U+2B5F */
  0x0000,   /* U+2B60 */
  0x0000,   /* U+2B61 */
  0x0000,   /* U+2B62 */
  0x0000,   /* U+2B63 */
  0x0000,   /* U+2B64 */
  0x0000,   /* U+2B65 */
  0x0000,   /* U+2B66 */
  0x0000,   /* U+2B67 */
  0x0000,   /* U+2B68 */
  0x0000,   /* U+2B69 */
  0x0000,   /* U+2B6A */
  0x0000,   /* U+2B6B */
  0x0000,   /* U+2B6C */
  0x0000,   /* U+2B6D */
  0x0000,   /* U+2B6E */
  0x0000,   /* U+2B6F */
  0x0000,   /* U+2B70 */
  0x0000,   /* U+2B71 */
  0x0000,   /* U+2B72 */
  0x0000,   /* U+2B73 */
  0x0000,   /* U+2B74 */
  0x0000,   /* U+2B75 */
  0x0000,   /* U+2B76 */
  0x0000,   /* U+2B77 */
  0x0000,   /* U+2B78 */
  0x0000,   /* U+2B79 */
  0x0000,   /* U+2B7A */
  0x0000,   /* U+2B7B */
  0x0000,   /* U+2B7C */
  0x0000,   /* U+2B7D */
  0x0000,   /* U+2B7E */
  0x0000,   /* U+2B7F */
  0x0000,   /* U+2B80 */
  0x0000,   /* U+2B81 */
  0x0000,   /* U+2B82 */
  0x0000,   /* U+2B83 */
  0x0000,   /* U+2B84 */
  0x0000,   /* U+2B85 */
  0x0000,   /* U+2B86 */
  0x0000,   /* U+2B87 */
  0x0000,   /* U+2B88 */
  0x0000,   /* U+2B89 */
  0x0000,   /* U+2B8A */
  0x0000,   /* U+2B8B */
  0x0000,   /* U+2B8C */
  0x0000,   /* U+2B8D */
  0x0000,   /* U+2B8E */
  0x0000,   /* U+2B8F */
  0x0000,   /* U+2B90 */
  0x0000,   /* U+2B91 */
  0x0000,   /* U+2B92 */
  0x0000,   /* U+2B93 */
  0x0000,   /* U+2B94 */
  0x0000,   /* U+2B95 */
  0x0000,   /* U+2B96 */
  0x0000,   /* U+2B97 */
  0x0000,   /* U+2B98 */
  0x0000,   /* U+2B99 */
  0x0000,   /* U+2B9A */
  0x0000,   /* U+2B9B */
  0x0000,   /* U+2B9C */
  0x0000,   /* U+2B9D */
  0x0000,   /* U+2B9E */
  0x0000,   /* U+2B9F */
  0x0000,   /* U+2BA0 */
  0x0000,   /* U+2BA1 */
  0x0000,   /* U+2BA2 */
  0x0000,   /* U+2BA3 */
  0x0000,   /* U+2BA4 */
  0x0000,   /* U+2BA5 */
  0x0000,   /* U+2BA6 */
  0x0000,   /* U+2BA7 */
  0x0000,   /* U+2BA8 */
  0x0000,   /* U+2BA9 */
  0x0000,   /* U+2BAA */
  0x0000,   /* U+2BAB */
  0x0000,   /* U+2BAC */
  0x0000,   /* U+2BAD */
  0x0000,   /* U+2BAE */
  0x0000,   /* U+2BAF */
  0x0000,   /* U+2BB0 */
  0x0000,   /* U+2BB1 */
  0x0000,   /* U+2BB2 */
  0x0000,   /* U+2BB3 */
  0x0000,   /* U+2BB4 */
  0x0000,   /* U+2BB5 */
  0x0000,   /* U+2BB6 */
  0x0000,   /* U+2BB7 */
  0x0000,   /* U+2BB8 */
  0x0000,   /* U+2BB9 */
  0x0000,   /* U+2BBA */
  0x0000,   /* U+2BBB */
  0x0000,   /* U+2BBC */
  0x0000,   /* U+2BBD */
  0x0000,   /* U+2BBE */
  0x0000,   /* U+2BBF */
  0x0000,   /* U+2BC0 */
  0x0000,   /* U+2BC1 */
  0x0000,   /* U+2BC2 */
  0x0000,   /* U+2BC3 */
  0x0000,   /* U+2BC4 */
  0x0000,   /* U+2BC5 */
  0x0000,   /* U+2BC6 */
  0x0000,   /* U+2BC7 */
  0x0000,   /* U+2BC8 */
  0x0000,   /* U+2BC9 */
  0x0000,   /* U+2BCA */
  0x0000,   /* U+2BCB */
  0x0000,   /* U+2BCC */
  0x0000,   /* U+2BCD */
  0x0000,   /* U+2BCE */
  0x0000,   /* U+2BCF */
  0x0000,   /* U+2BD0 */
  0x0000,   /* U+2BD1 */
  0x0000,   /* U+2BD2 */
  0x0000,   /* U+2BD3 */
  0x0000,   /* U+2BD4 */
  0x0000,   /* U+2BD5 */
  0x0000,   /* U+2BD6 */
  0x0000,   /* U+2BD7 */
  0x0000,   /* U+2BD8 */
  0x0000,   /* U+2BD9 */
  0x0000,   /* U+2BDA */
  0x0000,   /* U+2BDB */
  0x0000,   /* U+2BDC */
  0x0000,   /* U+2BDD */
  0x0000,   /* U+2BDE */
  0x0000,   /* U+2BDF */
  0x0000,   /* U+2BE0 */
  0x0000,   /* U+2BE1 */
  0x0000,   /* U+2BE2 */
  0x0000,   /* U+2BE3 */
  0x0000,   /* U+2BE4 */
  0x0000,   /* U+2BE5 */
  0x0000,   /* U+2BE6 */
  0x0000,   /* U+2BE7 */
  0x0000,   /* U+2BE8 */
  0x0000,   /* U+2BE9 */
  0x0000,   /* U+2BEA */
  0x0000,   /* U+2BEB */
  0x0000,   /* U+2BEC */
  0x0000,   /* U+2BED */
  0x0000,   /* U+2BEE */
  0x0000,   /* U+2BEF */
  0x0000,   /* U+2BF0 */
  0x0000,   /* U+2BF1 */
  0x0000,   /* U+2BF2 */
  0x0000,   /* U+2BF3 */
  0x0000,   /* U+2BF4 */
  0x0000,   /* U+2BF5 */
  0x0000,   /* U+2BF6 */
  0x0000,   /* U+2BF7 */
  0x0000,   /* U+2BF8 */
  0x0000,   /* U+2BF9 */
  0x0000,   /* U+2BFA */
  0x0000,   /* U+2BFB */
  0x0000,   /* U+2BFC */
  0x0000,   /* U+2BFD */
  0x0000,   /* U+2BFE */
  0x0000,   /* U+2BFF */
};

uint16 uca900_p02C[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x21E5,   /* U+2C00 */
  0x21E6,   /* U+2C01 */
  0x21E7,   /* U+2C02 */
  0x21E8,   /* U+2C03 */
  0x21E9,   /* U+2C04 */
  0x21EA,   /* U+2C05 */
  0x21EB,   /* U+2C06 */
  0x21EC,   /* U+2C07 */
  0x21ED,   /* U+2C08 */
  0x21EE,   /* U+2C09 */
  0x21EF,   /* U+2C0A */
  0x21F0,   /* U+2C0B */
  0x21F1,   /* U+2C0C */
  0x21F2,   /* U+2C0D */
  0x21F3,   /* U+2C0E */
  0x21F4,   /* U+2C0F */
  0x21F5,   /* U+2C10 */
  0x21F6,   /* U+2C11 */
  0x21F7,   /* U+2C12 */
  0x21F8,   /* U+2C13 */
  0x21F9,   /* U+2C14 */
  0x21FA,   /* U+2C15 */
  0x21FB,   /* U+2C16 */
  0x21FC,   /* U+2C17 */
  0x21FD,   /* U+2C18 */
  0x21FE,   /* U+2C19 */
  0x21FF,   /* U+2C1A */
  0x2200,   /* U+2C1B */
  0x2201,   /* U+2C1C */
  0x2202,   /* U+2C1D */
  0x2203,   /* U+2C1E */
  0x2204,   /* U+2C1F */
  0x2205,   /* U+2C20 */
  0x2206,   /* U+2C21 */
  0x2207,   /* U+2C22 */
  0x2208,   /* U+2C23 */
  0x2209,   /* U+2C24 */
  0x220A,   /* U+2C25 */
  0x220B,   /* U+2C26 */
  0x220C,   /* U+2C27 */
  0x220D,   /* U+2C28 */
  0x220E,   /* U+2C29 */
  0x220F,   /* U+2C2A */
  0x2210,   /* U+2C2B */
  0x2211,   /* U+2C2C */
  0x2212,   /* U+2C2D */
  0x2213,   /* U+2C2E */
  0xFBC0,   /* U+2C2F */
  0x21E5,   /* U+2C30 */
  0x21E6,   /* U+2C31 */
  0x21E7,   /* U+2C32 */
  0x21E8,   /* U+2C33 */
  0x21E9,   /* U+2C34 */
  0x21EA,   /* U+2C35 */
  0x21EB,   /* U+2C36 */
  0x21EC,   /* U+2C37 */
  0x21ED,   /* U+2C38 */
  0x21EE,   /* U+2C39 */
  0x21EF,   /* U+2C3A */
  0x21F0,   /* U+2C3B */
  0x21F1,   /* U+2C3C */
  0x21F2,   /* U+2C3D */
  0x21F3,   /* U+2C3E */
  0x21F4,   /* U+2C3F */
  0x21F5,   /* U+2C40 */
  0x21F6,   /* U+2C41 */
  0x21F7,   /* U+2C42 */
  0x21F8,   /* U+2C43 */
  0x21F9,   /* U+2C44 */
  0x21FA,   /* U+2C45 */
  0x21FB,   /* U+2C46 */
  0x21FC,   /* U+2C47 */
  0x21FD,   /* U+2C48 */
  0x21FE,   /* U+2C49 */
  0x21FF,   /* U+2C4A */
  0x2200,   /* U+2C4B */
  0x2201,   /* U+2C4C */
  0x2202,   /* U+2C4D */
  0x2203,   /* U+2C4E */
  0x2204,   /* U+2C4F */
  0x2205,   /* U+2C50 */
  0x2206,   /* U+2C51 */
  0x2207,   /* U+2C52 */
  0x2208,   /* U+2C53 */
  0x2209,   /* U+2C54 */
  0x220A,   /* U+2C55 */
  0x220B,   /* U+2C56 */
  0x220C,   /* U+2C57 */
  0x220D,   /* U+2C58 */
  0x220E,   /* U+2C59 */
  0x220F,   /* U+2C5A */
  0x2210,   /* U+2C5B */
  0x2211,   /* U+2C5C */
  0x2212,   /* U+2C5D */
  0x2213,   /* U+2C5E */
  0xFBC0,   /* U+2C5F */
  0x1D86,   /* U+2C60 */
  0x1D86,   /* U+2C61 */
  0x1D87,   /* U+2C62 */
  0x1E11,   /* U+2C63 */
  0x1E57,   /* U+2C64 */
  0x1C4C,   /* U+2C65 */
  0x1E9E,   /* U+2C66 */
  0x1D29,   /* U+2C67 */
  0x1D29,   /* U+2C68 */
  0x1D6F,   /* U+2C69 */
  0x1D6F,   /* U+2C6A */
  0x1F3C,   /* U+2C6B */
  0x1F3C,   /* U+2C6C */
  0x1C55,   /* U+2C6D */
  0x1DB1,   /* U+2C6E */
  0x1C51,   /* U+2C6F */
  0x1C5B,   /* U+2C70 */
  0x1EEE,   /* U+2C71 */
  0x1EFA,   /* U+2C72 */
  0x1EFA,   /* U+2C73 */
  0x1EEF,   /* U+2C74 */
  0x1D2A,   /* U+2C75 */
  0x1D2A,   /* U+2C76 */
  0x1E20,   /* U+2C77 */
  0x1CB7,   /* U+2C78 */
  0x1E52,   /* U+2C79 */
  0x1DFC,   /* U+2C7A */
  0x1CBC,   /* U+2C7B */
  0x1D4C,   /* U+2C7C */
  0x1EE3,   /* U+2C7D */
  0x1E7C,   /* U+2C7E */
  0x1F38,   /* U+2C7F */
  0x1FE6,   /* U+2C80 */
  0x1FE6,   /* U+2C81 */
  0x1FE7,   /* U+2C82 */
  0x1FE7,   /* U+2C83 */
  0x1FE8,   /* U+2C84 */
  0x1FE8,   /* U+2C85 */
  0x1FE9,   /* U+2C86 */
  0x1FE9,   /* U+2C87 */
  0x1FEA,   /* U+2C88 */
  0x1FEA,   /* U+2C89 */
  0x1FEC,   /* U+2C8A */
  0x1FEC,   /* U+2C8B */
  0x1FED,   /* U+2C8C */
  0x1FED,   /* U+2C8D */
  0x1FEE,   /* U+2C8E */
  0x1FEE,   /* U+2C8F */
  0x1FEF,   /* U+2C90 */
  0x1FEF,   /* U+2C91 */
  0x1FF0,   /* U+2C92 */
  0x1FF0,   /* U+2C93 */
  0x1FF1,   /* U+2C94 */
  0x1FF1,   /* U+2C95 */
  0x1FF3,   /* U+2C96 */
  0x1FF3,   /* U+2C97 */
  0x1FF4,   /* U+2C98 */
  0x1FF4,   /* U+2C99 */
  0x1FF5,   /* U+2C9A */
  0x1FF5,   /* U+2C9B */
  0x1FF8,   /* U+2C9C */
  0x1FF8,   /* U+2C9D */
  0x1FF9,   /* U+2C9E */
  0x1FF9,   /* U+2C9F */
  0x1FFA,   /* U+2CA0 */
  0x1FFA,   /* U+2CA1 */
  0x1FFB,   /* U+2CA2 */
  0x1FFB,   /* U+2CA3 */
  0x1FFC,   /* U+2CA4 */
  0x1FFC,   /* U+2CA5 */
  0x1FFD,   /* U+2CA6 */
  0x1FFD,   /* U+2CA7 */
  0x1FFE,   /* U+2CA8 */
  0x1FFE,   /* U+2CA9 */
  0x1FFF,   /* U+2CAA */
  0x1FFF,   /* U+2CAB */
  0x2000,   /* U+2CAC */
  0x2000,   /* U+2CAD */
  0x2001,   /* U+2CAE */
  0x2001,   /* U+2CAF */
  0x2002,   /* U+2CB0 */
  0x2002,   /* U+2CB1 */
  0x201D,   /* U+2CB2 */
  0x201D,   /* U+2CB3 */
  0x201E,   /* U+2CB4 */
  0x201E,   /* U+2CB5 */
  0x1FEB,   /* U+2CB6 */
  0x1FEB,   /* U+2CB7 */
  0x1FF2,   /* U+2CB8 */
  0x1FF2,   /* U+2CB9 */
  0x1FF6,   /* U+2CBA */
  0x1FF6,   /* U+2CBB */
  0x1FF7,   /* U+2CBC */
  0x1FF7,   /* U+2CBD */
  0x2003,   /* U+2CBE */
  0x2003,   /* U+2CBF */
  0x2004,   /* U+2CC0 */
  0x2004,   /* U+2CC1 */
  0x2007,   /* U+2CC2 */
  0x2007,   /* U+2CC3 */
  0x2008,   /* U+2CC4 */
  0x2008,   /* U+2CC5 */
  0x2009,   /* U+2CC6 */
  0x2009,   /* U+2CC7 */
  0x200D,   /* U+2CC8 */
  0x200D,   /* U+2CC9 */
  0x200F,   /* U+2CCA */
  0x200F,   /* U+2CCB */
  0x2010,   /* U+2CCC */
  0x2010,   /* U+2CCD */
  0x2011,   /* U+2CCE */
  0x2011,   /* U+2CCF */
  0x2012,   /* U+2CD0 */
  0x2012,   /* U+2CD1 */
  0x2013,   /* U+2CD2 */
  0x2013,   /* U+2CD3 */
  0x2014,   /* U+2CD4 */
  0x2014,   /* U+2CD5 */
  0x2017,   /* U+2CD6 */
  0x2017,   /* U+2CD7 */
  0x2019,   /* U+2CD8 */
  0x2019,   /* U+2CD9 */
  0x201A,   /* U+2CDA */
  0x201A,   /* U+2CDB */
  0x201B,   /* U+2CDC */
  0x201B,   /* U+2CDD */
  0x201F,   /* U+2CDE */
  0x201F,   /* U+2CDF */
  0x2020,   /* U+2CE0 */
  0x2020,   /* U+2CE1 */
  0x2021,   /* U+2CE2 */
  0x2021,   /* U+2CE3 */
  0x1FF1,   /* U+2CE4 */
  0x0D96,   /* U+2CE5 */
  0x0D97,   /* U+2CE6 */
  0x0D98,   /* U+2CE7 */
  0x0D99,   /* U+2CE8 */
  0x0D9A,   /* U+2CE9 */
  0x0D9B,   /* U+2CEA */
  0x2006,   /* U+2CEB */
  0x2006,   /* U+2CEC */
  0x2016,   /* U+2CED */
  0x2016,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x200C,   /* U+2CF2 */
  0x200C,   /* U+2CF3 */
  0xFBC0,   /* U+2CF4 */
  0xFBC0,   /* U+2CF5 */
  0xFBC0,   /* U+2CF6 */
  0xFBC0,   /* U+2CF7 */
  0xFBC0,   /* U+2CF8 */
  0x0281,   /* U+2CF9 */
  0x026E,   /* U+2CFA */
  0x026F,   /* U+2CFB */
  0x02F6,   /* U+2CFC */
  0x1AE2,   /* U+2CFD */
  0x0282,   /* U+2CFE */
  0x02F7,   /* U+2CFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2C00 */
  0x0020,   /* U+2C01 */
  0x0020,   /* U+2C02 */
  0x0020,   /* U+2C03 */
  0x0020,   /* U+2C04 */
  0x0020,   /* U+2C05 */
  0x0020,   /* U+2C06 */
  0x0020,   /* U+2C07 */
  0x0020,   /* U+2C08 */
  0x0020,   /* U+2C09 */
  0x0020,   /* U+2C0A */
  0x0020,   /* U+2C0B */
  0x0020,   /* U+2C0C */
  0x0020,   /* U+2C0D */
  0x0020,   /* U+2C0E */
  0x0020,   /* U+2C0F */
  0x0020,   /* U+2C10 */
  0x0020,   /* U+2C11 */
  0x0020,   /* U+2C12 */
  0x0020,   /* U+2C13 */
  0x0020,   /* U+2C14 */
  0x0020,   /* U+2C15 */
  0x0020,   /* U+2C16 */
  0x0020,   /* U+2C17 */
  0x0020,   /* U+2C18 */
  0x0020,   /* U+2C19 */
  0x0020,   /* U+2C1A */
  0x0020,   /* U+2C1B */
  0x0020,   /* U+2C1C */
  0x0020,   /* U+2C1D */
  0x0020,   /* U+2C1E */
  0x0020,   /* U+2C1F */
  0x0020,   /* U+2C20 */
  0x0020,   /* U+2C21 */
  0x0020,   /* U+2C22 */
  0x0020,   /* U+2C23 */
  0x0020,   /* U+2C24 */
  0x0020,   /* U+2C25 */
  0x0020,   /* U+2C26 */
  0x0020,   /* U+2C27 */
  0x0020,   /* U+2C28 */
  0x0020,   /* U+2C29 */
  0x0020,   /* U+2C2A */
  0x0020,   /* U+2C2B */
  0x0020,   /* U+2C2C */
  0x0020,   /* U+2C2D */
  0x0020,   /* U+2C2E */
  0x0020,   /* U+2C2F */
  0x0020,   /* U+2C30 */
  0x0020,   /* U+2C31 */
  0x0020,   /* U+2C32 */
  0x0020,   /* U+2C33 */
  0x0020,   /* U+2C34 */
  0x0020,   /* U+2C35 */
  0x0020,   /* U+2C36 */
  0x0020,   /* U+2C37 */
  0x0020,   /* U+2C38 */
  0x0020,   /* U+2C39 */
  0x0020,   /* U+2C3A */
  0x0020,   /* U+2C3B */
  0x0020,   /* U+2C3C */
  0x0020,   /* U+2C3D */
  0x0020,   /* U+2C3E */
  0x0020,   /* U+2C3F */
  0x0020,   /* U+2C40 */
  0x0020,   /* U+2C41 */
  0x0020,   /* U+2C42 */
  0x0020,   /* U+2C43 */
  0x0020,   /* U+2C44 */
  0x0020,   /* U+2C45 */
  0x0020,   /* U+2C46 */
  0x0020,   /* U+2C47 */
  0x0020,   /* U+2C48 */
  0x0020,   /* U+2C49 */
  0x0020,   /* U+2C4A */
  0x0020,   /* U+2C4B */
  0x0020,   /* U+2C4C */
  0x0020,   /* U+2C4D */
  0x0020,   /* U+2C4E */
  0x0020,   /* U+2C4F */
  0x0020,   /* U+2C50 */
  0x0020,   /* U+2C51 */
  0x0020,   /* U+2C52 */
  0x0020,   /* U+2C53 */
  0x0020,   /* U+2C54 */
  0x0020,   /* U+2C55 */
  0x0020,   /* U+2C56 */
  0x0020,   /* U+2C57 */
  0x0020,   /* U+2C58 */
  0x0020,   /* U+2C59 */
  0x0020,   /* U+2C5A */
  0x0020,   /* U+2C5B */
  0x0020,   /* U+2C5C */
  0x0020,   /* U+2C5D */
  0x0020,   /* U+2C5E */
  0x0020,   /* U+2C5F */
  0x0020,   /* U+2C60 */
  0x0020,   /* U+2C61 */
  0x0020,   /* U+2C62 */
  0x0020,   /* U+2C63 */
  0x0020,   /* U+2C64 */
  0x0020,   /* U+2C65 */
  0x0020,   /* U+2C66 */
  0x0020,   /* U+2C67 */
  0x0020,   /* U+2C68 */
  0x0020,   /* U+2C69 */
  0x0020,   /* U+2C6A */
  0x0020,   /* U+2C6B */
  0x0020,   /* U+2C6C */
  0x0020,   /* U+2C6D */
  0x0020,   /* U+2C6E */
  0x0020,   /* U+2C6F */
  0x0020,   /* U+2C70 */
  0x0020,   /* U+2C71 */
  0x0020,   /* U+2C72 */
  0x0020,   /* U+2C73 */
  0x0020,   /* U+2C74 */
  0x0020,   /* U+2C75 */
  0x0020,   /* U+2C76 */
  0x0020,   /* U+2C77 */
  0x0020,   /* U+2C78 */
  0x0020,   /* U+2C79 */
  0x0020,   /* U+2C7A */
  0x0020,   /* U+2C7B */
  0x0020,   /* U+2C7C */
  0x0020,   /* U+2C7D */
  0x0020,   /* U+2C7E */
  0x0020,   /* U+2C7F */
  0x0020,   /* U+2C80 */
  0x0020,   /* U+2C81 */
  0x0020,   /* U+2C82 */
  0x0020,   /* U+2C83 */
  0x0020,   /* U+2C84 */
  0x0020,   /* U+2C85 */
  0x0020,   /* U+2C86 */
  0x0020,   /* U+2C87 */
  0x0020,   /* U+2C88 */
  0x0020,   /* U+2C89 */
  0x0020,   /* U+2C8A */
  0x0020,   /* U+2C8B */
  0x0020,   /* U+2C8C */
  0x0020,   /* U+2C8D */
  0x0020,   /* U+2C8E */
  0x0020,   /* U+2C8F */
  0x0020,   /* U+2C90 */
  0x0020,   /* U+2C91 */
  0x0020,   /* U+2C92 */
  0x0020,   /* U+2C93 */
  0x0020,   /* U+2C94 */
  0x0020,   /* U+2C95 */
  0x0020,   /* U+2C96 */
  0x0020,   /* U+2C97 */
  0x0020,   /* U+2C98 */
  0x0020,   /* U+2C99 */
  0x0020,   /* U+2C9A */
  0x0020,   /* U+2C9B */
  0x0020,   /* U+2C9C */
  0x0020,   /* U+2C9D */
  0x0020,   /* U+2C9E */
  0x0020,   /* U+2C9F */
  0x0020,   /* U+2CA0 */
  0x0020,   /* U+2CA1 */
  0x0020,   /* U+2CA2 */
  0x0020,   /* U+2CA3 */
  0x0020,   /* U+2CA4 */
  0x0020,   /* U+2CA5 */
  0x0020,   /* U+2CA6 */
  0x0020,   /* U+2CA7 */
  0x0020,   /* U+2CA8 */
  0x0020,   /* U+2CA9 */
  0x0020,   /* U+2CAA */
  0x0020,   /* U+2CAB */
  0x0020,   /* U+2CAC */
  0x0020,   /* U+2CAD */
  0x0020,   /* U+2CAE */
  0x0020,   /* U+2CAF */
  0x0020,   /* U+2CB0 */
  0x0020,   /* U+2CB1 */
  0x0020,   /* U+2CB2 */
  0x0020,   /* U+2CB3 */
  0x0020,   /* U+2CB4 */
  0x0020,   /* U+2CB5 */
  0x0020,   /* U+2CB6 */
  0x0020,   /* U+2CB7 */
  0x0020,   /* U+2CB8 */
  0x0020,   /* U+2CB9 */
  0x0020,   /* U+2CBA */
  0x0020,   /* U+2CBB */
  0x0020,   /* U+2CBC */
  0x0020,   /* U+2CBD */
  0x0020,   /* U+2CBE */
  0x0020,   /* U+2CBF */
  0x0020,   /* U+2CC0 */
  0x0020,   /* U+2CC1 */
  0x0020,   /* U+2CC2 */
  0x0020,   /* U+2CC3 */
  0x0020,   /* U+2CC4 */
  0x0020,   /* U+2CC5 */
  0x0020,   /* U+2CC6 */
  0x0020,   /* U+2CC7 */
  0x0020,   /* U+2CC8 */
  0x0020,   /* U+2CC9 */
  0x0020,   /* U+2CCA */
  0x0020,   /* U+2CCB */
  0x0020,   /* U+2CCC */
  0x0020,   /* U+2CCD */
  0x0020,   /* U+2CCE */
  0x0020,   /* U+2CCF */
  0x0020,   /* U+2CD0 */
  0x0020,   /* U+2CD1 */
  0x0020,   /* U+2CD2 */
  0x0020,   /* U+2CD3 */
  0x0020,   /* U+2CD4 */
  0x0020,   /* U+2CD5 */
  0x0020,   /* U+2CD6 */
  0x0020,   /* U+2CD7 */
  0x0020,   /* U+2CD8 */
  0x0020,   /* U+2CD9 */
  0x0020,   /* U+2CDA */
  0x0020,   /* U+2CDB */
  0x0020,   /* U+2CDC */
  0x0020,   /* U+2CDD */
  0x0020,   /* U+2CDE */
  0x0020,   /* U+2CDF */
  0x0020,   /* U+2CE0 */
  0x0020,   /* U+2CE1 */
  0x0020,   /* U+2CE2 */
  0x0020,   /* U+2CE3 */
  0x0020,   /* U+2CE4 */
  0x0020,   /* U+2CE5 */
  0x0020,   /* U+2CE6 */
  0x0020,   /* U+2CE7 */
  0x0020,   /* U+2CE8 */
  0x0020,   /* U+2CE9 */
  0x0020,   /* U+2CEA */
  0x0020,   /* U+2CEB */
  0x0020,   /* U+2CEC */
  0x0020,   /* U+2CED */
  0x0020,   /* U+2CEE */
  0x0033,   /* U+2CEF */
  0x0023,   /* U+2CF0 */
  0x0022,   /* U+2CF1 */
  0x0020,   /* U+2CF2 */
  0x0020,   /* U+2CF3 */
  0x0020,   /* U+2CF4 */
  0x0020,   /* U+2CF5 */
  0x0020,   /* U+2CF6 */
  0x0020,   /* U+2CF7 */
  0x0020,   /* U+2CF8 */
  0x0020,   /* U+2CF9 */
  0x0020,   /* U+2CFA */
  0x0020,   /* U+2CFB */
  0x0020,   /* U+2CFC */
  0x0020,   /* U+2CFD */
  0x0020,   /* U+2CFE */
  0x0020,   /* U+2CFF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+2C00 */
  0x0008,   /* U+2C01 */
  0x0008,   /* U+2C02 */
  0x0008,   /* U+2C03 */
  0x0008,   /* U+2C04 */
  0x0008,   /* U+2C05 */
  0x0008,   /* U+2C06 */
  0x0008,   /* U+2C07 */
  0x0008,   /* U+2C08 */
  0x0008,   /* U+2C09 */
  0x0008,   /* U+2C0A */
  0x0008,   /* U+2C0B */
  0x0008,   /* U+2C0C */
  0x0008,   /* U+2C0D */
  0x0008,   /* U+2C0E */
  0x0008,   /* U+2C0F */
  0x0008,   /* U+2C10 */
  0x0008,   /* U+2C11 */
  0x0008,   /* U+2C12 */
  0x0008,   /* U+2C13 */
  0x0008,   /* U+2C14 */
  0x0008,   /* U+2C15 */
  0x0008,   /* U+2C16 */
  0x0008,   /* U+2C17 */
  0x0008,   /* U+2C18 */
  0x0008,   /* U+2C19 */
  0x0008,   /* U+2C1A */
  0x0008,   /* U+2C1B */
  0x0008,   /* U+2C1C */
  0x0008,   /* U+2C1D */
  0x0008,   /* U+2C1E */
  0x0008,   /* U+2C1F */
  0x0008,   /* U+2C20 */
  0x0008,   /* U+2C21 */
  0x0008,   /* U+2C22 */
  0x0008,   /* U+2C23 */
  0x0008,   /* U+2C24 */
  0x0008,   /* U+2C25 */
  0x0008,   /* U+2C26 */
  0x0008,   /* U+2C27 */
  0x0008,   /* U+2C28 */
  0x0008,   /* U+2C29 */
  0x0008,   /* U+2C2A */
  0x0008,   /* U+2C2B */
  0x0008,   /* U+2C2C */
  0x0008,   /* U+2C2D */
  0x0008,   /* U+2C2E */
  0x0002,   /* U+2C2F */
  0x0002,   /* U+2C30 */
  0x0002,   /* U+2C31 */
  0x0002,   /* U+2C32 */
  0x0002,   /* U+2C33 */
  0x0002,   /* U+2C34 */
  0x0002,   /* U+2C35 */
  0x0002,   /* U+2C36 */
  0x0002,   /* U+2C37 */
  0x0002,   /* U+2C38 */
  0x0002,   /* U+2C39 */
  0x0002,   /* U+2C3A */
  0x0002,   /* U+2C3B */
  0x0002,   /* U+2C3C */
  0x0002,   /* U+2C3D */
  0x0002,   /* U+2C3E */
  0x0002,   /* U+2C3F */
  0x0002,   /* U+2C40 */
  0x0002,   /* U+2C41 */
  0x0002,   /* U+2C42 */
  0x0002,   /* U+2C43 */
  0x0002,   /* U+2C44 */
  0x0002,   /* U+2C45 */
  0x0002,   /* U+2C46 */
  0x0002,   /* U+2C47 */
  0x0002,   /* U+2C48 */
  0x0002,   /* U+2C49 */
  0x0002,   /* U+2C4A */
  0x0002,   /* U+2C4B */
  0x0002,   /* U+2C4C */
  0x0002,   /* U+2C4D */
  0x0002,   /* U+2C4E */
  0x0002,   /* U+2C4F */
  0x0002,   /* U+2C50 */
  0x0002,   /* U+2C51 */
  0x0002,   /* U+2C52 */
  0x0002,   /* U+2C53 */
  0x0002,   /* U+2C54 */
  0x0002,   /* U+2C55 */
  0x0002,   /* U+2C56 */
  0x0002,   /* U+2C57 */
  0x0002,   /* U+2C58 */
  0x0002,   /* U+2C59 */
  0x0002,   /* U+2C5A */
  0x0002,   /* U+2C5B */
  0x0002,   /* U+2C5C */
  0x0002,   /* U+2C5D */
  0x0002,   /* U+2C5E */
  0x0002,   /* U+2C5F */
  0x0008,   /* U+2C60 */
  0x0002,   /* U+2C61 */
  0x0008,   /* U+2C62 */
  0x0008,   /* U+2C63 */
  0x0008,   /* U+2C64 */
  0x0002,   /* U+2C65 */
  0x0002,   /* U+2C66 */
  0x0008,   /* U+2C67 */
  0x0002,   /* U+2C68 */
  0x0008,   /* U+2C69 */
  0x0002,   /* U+2C6A */
  0x0008,   /* U+2C6B */
  0x0002,   /* U+2C6C */
  0x0008,   /* U+2C6D */
  0x0008,   /* U+2C6E */
  0x0008,   /* U+2C6F */
  0x0008,   /* U+2C70 */
  0x0002,   /* U+2C71 */
  0x0008,   /* U+2C72 */
  0x0002,   /* U+2C73 */
  0x0002,   /* U+2C74 */
  0x0008,   /* U+2C75 */
  0x0002,   /* U+2C76 */
  0x0002,   /* U+2C77 */
  0x0002,   /* U+2C78 */
  0x0002,   /* U+2C79 */
  0x0002,   /* U+2C7A */
  0x0002,   /* U+2C7B */
  0x0015,   /* U+2C7C */
  0x001D,   /* U+2C7D */
  0x0008,   /* U+2C7E */
  0x0008,   /* U+2C7F */
  0x0008,   /* U+2C80 */
  0x0002,   /* U+2C81 */
  0x0008,   /* U+2C82 */
  0x0002,   /* U+2C83 */
  0x0008,   /* U+2C84 */
  0x0002,   /* U+2C85 */
  0x0008,   /* U+2C86 */
  0x0002,   /* U+2C87 */
  0x0008,   /* U+2C88 */
  0x0002,   /* U+2C89 */
  0x0008,   /* U+2C8A */
  0x0002,   /* U+2C8B */
  0x0008,   /* U+2C8C */
  0x0002,   /* U+2C8D */
  0x0008,   /* U+2C8E */
  0x0002,   /* U+2C8F */
  0x0008,   /* U+2C90 */
  0x0002,   /* U+2C91 */
  0x0008,   /* U+2C92 */
  0x0002,   /* U+2C93 */
  0x0008,   /* U+2C94 */
  0x0002,   /* U+2C95 */
  0x0008,   /* U+2C96 */
  0x0002,   /* U+2C97 */
  0x0008,   /* U+2C98 */
  0x0002,   /* U+2C99 */
  0x0008,   /* U+2C9A */
  0x0002,   /* U+2C9B */
  0x0008,   /* U+2C9C */
  0x0002,   /* U+2C9D */
  0x0008,   /* U+2C9E */
  0x0002,   /* U+2C9F */
  0x0008,   /* U+2CA0 */
  0x0002,   /* U+2CA1 */
  0x0008,   /* U+2CA2 */
  0x0002,   /* U+2CA3 */
  0x0008,   /* U+2CA4 */
  0x0002,   /* U+2CA5 */
  0x0008,   /* U+2CA6 */
  0x0002,   /* U+2CA7 */
  0x0008,   /* U+2CA8 */
  0x0002,   /* U+2CA9 */
  0x0008,   /* U+2CAA */
  0x0002,   /* U+2CAB */
  0x0008,   /* U+2CAC */
  0x0002,   /* U+2CAD */
  0x0008,   /* U+2CAE */
  0x0002,   /* U+2CAF */
  0x0008,   /* U+2CB0 */
  0x0002,   /* U+2CB1 */
  0x0008,   /* U+2CB2 */
  0x0002,   /* U+2CB3 */
  0x0008,   /* U+2CB4 */
  0x0002,   /* U+2CB5 */
  0x0008,   /* U+2CB6 */
  0x0002,   /* U+2CB7 */
  0x0008,   /* U+2CB8 */
  0x0002,   /* U+2CB9 */
  0x0008,   /* U+2CBA */
  0x0002,   /* U+2CBB */
  0x0008,   /* U+2CBC */
  0x0002,   /* U+2CBD */
  0x0008,   /* U+2CBE */
  0x0002,   /* U+2CBF */
  0x0008,   /* U+2CC0 */
  0x0002,   /* U+2CC1 */
  0x0008,   /* U+2CC2 */
  0x0002,   /* U+2CC3 */
  0x0008,   /* U+2CC4 */
  0x0002,   /* U+2CC5 */
  0x0008,   /* U+2CC6 */
  0x0002,   /* U+2CC7 */
  0x0008,   /* U+2CC8 */
  0x0002,   /* U+2CC9 */
  0x0008,   /* U+2CCA */
  0x0002,   /* U+2CCB */
  0x0008,   /* U+2CCC */
  0x0002,   /* U+2CCD */
  0x0008,   /* U+2CCE */
  0x0002,   /* U+2CCF */
  0x0008,   /* U+2CD0 */
  0x0002,   /* U+2CD1 */
  0x0008,   /* U+2CD2 */
  0x0002,   /* U+2CD3 */
  0x0008,   /* U+2CD4 */
  0x0002,   /* U+2CD5 */
  0x0008,   /* U+2CD6 */
  0x0002,   /* U+2CD7 */
  0x0008,   /* U+2CD8 */
  0x0002,   /* U+2CD9 */
  0x0008,   /* U+2CDA */
  0x0002,   /* U+2CDB */
  0x0008,   /* U+2CDC */
  0x0002,   /* U+2CDD */
  0x0008,   /* U+2CDE */
  0x0002,   /* U+2CDF */
  0x0008,   /* U+2CE0 */
  0x0002,   /* U+2CE1 */
  0x0008,   /* U+2CE2 */
  0x0002,   /* U+2CE3 */
  0x0004,   /* U+2CE4 */
  0x0002,   /* U+2CE5 */
  0x0002,   /* U+2CE6 */
  0x0002,   /* U+2CE7 */
  0x0002,   /* U+2CE8 */
  0x0002,   /* U+2CE9 */
  0x0002,   /* U+2CEA */
  0x0008,   /* U+2CEB */
  0x0002,   /* U+2CEC */
  0x0008,   /* U+2CED */
  0x0002,   /* U+2CEE */
  0x0002,   /* U+2CEF */
  0x0002,   /* U+2CF0 */
  0x0002,   /* U+2CF1 */
  0x0008,   /* U+2CF2 */
  0x0002,   /* U+2CF3 */
  0x0002,   /* U+2CF4 */
  0x0002,   /* U+2CF5 */
  0x0002,   /* U+2CF6 */
  0x0002,   /* U+2CF7 */
  0x0002,   /* U+2CF8 */
  0x0002,   /* U+2CF9 */
  0x0002,   /* U+2CFA */
  0x0002,   /* U+2CFB */
  0x0002,   /* U+2CFC */
  0x0002,   /* U+2CFD */
  0x0002,   /* U+2CFE */
  0x0002,   /* U+2CFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0xAC2F,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0xAC5F,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x1FE6,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0xACF4,   /* U+2CF4 */
  0xACF5,   /* U+2CF5 */
  0xACF6,   /* U+2CF6 */
  0xACF7,   /* U+2CF7 */
  0xACF8,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0x0000,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0x0000,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x0020,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0x0000,   /* U+2CF4 */
  0x0000,   /* U+2CF5 */
  0x0000,   /* U+2CF6 */
  0x0000,   /* U+2CF7 */
  0x0000,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0x0000,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0x0000,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x0004,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0x0000,   /* U+2CF4 */
  0x0000,   /* U+2CF5 */
  0x0000,   /* U+2CF6 */
  0x0000,   /* U+2CF7 */
  0x0000,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0x0000,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0x0000,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x1FF0,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0x0000,   /* U+2CF4 */
  0x0000,   /* U+2CF5 */
  0x0000,   /* U+2CF6 */
  0x0000,   /* U+2CF7 */
  0x0000,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0x0000,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0x0000,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x0020,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0x0000,   /* U+2CF4 */
  0x0000,   /* U+2CF5 */
  0x0000,   /* U+2CF6 */
  0x0000,   /* U+2CF7 */
  0x0000,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2C00 */
  0x0000,   /* U+2C01 */
  0x0000,   /* U+2C02 */
  0x0000,   /* U+2C03 */
  0x0000,   /* U+2C04 */
  0x0000,   /* U+2C05 */
  0x0000,   /* U+2C06 */
  0x0000,   /* U+2C07 */
  0x0000,   /* U+2C08 */
  0x0000,   /* U+2C09 */
  0x0000,   /* U+2C0A */
  0x0000,   /* U+2C0B */
  0x0000,   /* U+2C0C */
  0x0000,   /* U+2C0D */
  0x0000,   /* U+2C0E */
  0x0000,   /* U+2C0F */
  0x0000,   /* U+2C10 */
  0x0000,   /* U+2C11 */
  0x0000,   /* U+2C12 */
  0x0000,   /* U+2C13 */
  0x0000,   /* U+2C14 */
  0x0000,   /* U+2C15 */
  0x0000,   /* U+2C16 */
  0x0000,   /* U+2C17 */
  0x0000,   /* U+2C18 */
  0x0000,   /* U+2C19 */
  0x0000,   /* U+2C1A */
  0x0000,   /* U+2C1B */
  0x0000,   /* U+2C1C */
  0x0000,   /* U+2C1D */
  0x0000,   /* U+2C1E */
  0x0000,   /* U+2C1F */
  0x0000,   /* U+2C20 */
  0x0000,   /* U+2C21 */
  0x0000,   /* U+2C22 */
  0x0000,   /* U+2C23 */
  0x0000,   /* U+2C24 */
  0x0000,   /* U+2C25 */
  0x0000,   /* U+2C26 */
  0x0000,   /* U+2C27 */
  0x0000,   /* U+2C28 */
  0x0000,   /* U+2C29 */
  0x0000,   /* U+2C2A */
  0x0000,   /* U+2C2B */
  0x0000,   /* U+2C2C */
  0x0000,   /* U+2C2D */
  0x0000,   /* U+2C2E */
  0x0000,   /* U+2C2F */
  0x0000,   /* U+2C30 */
  0x0000,   /* U+2C31 */
  0x0000,   /* U+2C32 */
  0x0000,   /* U+2C33 */
  0x0000,   /* U+2C34 */
  0x0000,   /* U+2C35 */
  0x0000,   /* U+2C36 */
  0x0000,   /* U+2C37 */
  0x0000,   /* U+2C38 */
  0x0000,   /* U+2C39 */
  0x0000,   /* U+2C3A */
  0x0000,   /* U+2C3B */
  0x0000,   /* U+2C3C */
  0x0000,   /* U+2C3D */
  0x0000,   /* U+2C3E */
  0x0000,   /* U+2C3F */
  0x0000,   /* U+2C40 */
  0x0000,   /* U+2C41 */
  0x0000,   /* U+2C42 */
  0x0000,   /* U+2C43 */
  0x0000,   /* U+2C44 */
  0x0000,   /* U+2C45 */
  0x0000,   /* U+2C46 */
  0x0000,   /* U+2C47 */
  0x0000,   /* U+2C48 */
  0x0000,   /* U+2C49 */
  0x0000,   /* U+2C4A */
  0x0000,   /* U+2C4B */
  0x0000,   /* U+2C4C */
  0x0000,   /* U+2C4D */
  0x0000,   /* U+2C4E */
  0x0000,   /* U+2C4F */
  0x0000,   /* U+2C50 */
  0x0000,   /* U+2C51 */
  0x0000,   /* U+2C52 */
  0x0000,   /* U+2C53 */
  0x0000,   /* U+2C54 */
  0x0000,   /* U+2C55 */
  0x0000,   /* U+2C56 */
  0x0000,   /* U+2C57 */
  0x0000,   /* U+2C58 */
  0x0000,   /* U+2C59 */
  0x0000,   /* U+2C5A */
  0x0000,   /* U+2C5B */
  0x0000,   /* U+2C5C */
  0x0000,   /* U+2C5D */
  0x0000,   /* U+2C5E */
  0x0000,   /* U+2C5F */
  0x0000,   /* U+2C60 */
  0x0000,   /* U+2C61 */
  0x0000,   /* U+2C62 */
  0x0000,   /* U+2C63 */
  0x0000,   /* U+2C64 */
  0x0000,   /* U+2C65 */
  0x0000,   /* U+2C66 */
  0x0000,   /* U+2C67 */
  0x0000,   /* U+2C68 */
  0x0000,   /* U+2C69 */
  0x0000,   /* U+2C6A */
  0x0000,   /* U+2C6B */
  0x0000,   /* U+2C6C */
  0x0000,   /* U+2C6D */
  0x0000,   /* U+2C6E */
  0x0000,   /* U+2C6F */
  0x0000,   /* U+2C70 */
  0x0000,   /* U+2C71 */
  0x0000,   /* U+2C72 */
  0x0000,   /* U+2C73 */
  0x0000,   /* U+2C74 */
  0x0000,   /* U+2C75 */
  0x0000,   /* U+2C76 */
  0x0000,   /* U+2C77 */
  0x0000,   /* U+2C78 */
  0x0000,   /* U+2C79 */
  0x0000,   /* U+2C7A */
  0x0000,   /* U+2C7B */
  0x0000,   /* U+2C7C */
  0x0000,   /* U+2C7D */
  0x0000,   /* U+2C7E */
  0x0000,   /* U+2C7F */
  0x0000,   /* U+2C80 */
  0x0000,   /* U+2C81 */
  0x0000,   /* U+2C82 */
  0x0000,   /* U+2C83 */
  0x0000,   /* U+2C84 */
  0x0000,   /* U+2C85 */
  0x0000,   /* U+2C86 */
  0x0000,   /* U+2C87 */
  0x0000,   /* U+2C88 */
  0x0000,   /* U+2C89 */
  0x0000,   /* U+2C8A */
  0x0000,   /* U+2C8B */
  0x0000,   /* U+2C8C */
  0x0000,   /* U+2C8D */
  0x0000,   /* U+2C8E */
  0x0000,   /* U+2C8F */
  0x0000,   /* U+2C90 */
  0x0000,   /* U+2C91 */
  0x0000,   /* U+2C92 */
  0x0000,   /* U+2C93 */
  0x0000,   /* U+2C94 */
  0x0000,   /* U+2C95 */
  0x0000,   /* U+2C96 */
  0x0000,   /* U+2C97 */
  0x0000,   /* U+2C98 */
  0x0000,   /* U+2C99 */
  0x0000,   /* U+2C9A */
  0x0000,   /* U+2C9B */
  0x0000,   /* U+2C9C */
  0x0000,   /* U+2C9D */
  0x0000,   /* U+2C9E */
  0x0000,   /* U+2C9F */
  0x0000,   /* U+2CA0 */
  0x0000,   /* U+2CA1 */
  0x0000,   /* U+2CA2 */
  0x0000,   /* U+2CA3 */
  0x0000,   /* U+2CA4 */
  0x0000,   /* U+2CA5 */
  0x0000,   /* U+2CA6 */
  0x0000,   /* U+2CA7 */
  0x0000,   /* U+2CA8 */
  0x0000,   /* U+2CA9 */
  0x0000,   /* U+2CAA */
  0x0000,   /* U+2CAB */
  0x0000,   /* U+2CAC */
  0x0000,   /* U+2CAD */
  0x0000,   /* U+2CAE */
  0x0000,   /* U+2CAF */
  0x0000,   /* U+2CB0 */
  0x0000,   /* U+2CB1 */
  0x0000,   /* U+2CB2 */
  0x0000,   /* U+2CB3 */
  0x0000,   /* U+2CB4 */
  0x0000,   /* U+2CB5 */
  0x0000,   /* U+2CB6 */
  0x0000,   /* U+2CB7 */
  0x0000,   /* U+2CB8 */
  0x0000,   /* U+2CB9 */
  0x0000,   /* U+2CBA */
  0x0000,   /* U+2CBB */
  0x0000,   /* U+2CBC */
  0x0000,   /* U+2CBD */
  0x0000,   /* U+2CBE */
  0x0000,   /* U+2CBF */
  0x0000,   /* U+2CC0 */
  0x0000,   /* U+2CC1 */
  0x0000,   /* U+2CC2 */
  0x0000,   /* U+2CC3 */
  0x0000,   /* U+2CC4 */
  0x0000,   /* U+2CC5 */
  0x0000,   /* U+2CC6 */
  0x0000,   /* U+2CC7 */
  0x0000,   /* U+2CC8 */
  0x0000,   /* U+2CC9 */
  0x0000,   /* U+2CCA */
  0x0000,   /* U+2CCB */
  0x0000,   /* U+2CCC */
  0x0000,   /* U+2CCD */
  0x0000,   /* U+2CCE */
  0x0000,   /* U+2CCF */
  0x0000,   /* U+2CD0 */
  0x0000,   /* U+2CD1 */
  0x0000,   /* U+2CD2 */
  0x0000,   /* U+2CD3 */
  0x0000,   /* U+2CD4 */
  0x0000,   /* U+2CD5 */
  0x0000,   /* U+2CD6 */
  0x0000,   /* U+2CD7 */
  0x0000,   /* U+2CD8 */
  0x0000,   /* U+2CD9 */
  0x0000,   /* U+2CDA */
  0x0000,   /* U+2CDB */
  0x0000,   /* U+2CDC */
  0x0000,   /* U+2CDD */
  0x0000,   /* U+2CDE */
  0x0000,   /* U+2CDF */
  0x0000,   /* U+2CE0 */
  0x0000,   /* U+2CE1 */
  0x0000,   /* U+2CE2 */
  0x0000,   /* U+2CE3 */
  0x0004,   /* U+2CE4 */
  0x0000,   /* U+2CE5 */
  0x0000,   /* U+2CE6 */
  0x0000,   /* U+2CE7 */
  0x0000,   /* U+2CE8 */
  0x0000,   /* U+2CE9 */
  0x0000,   /* U+2CEA */
  0x0000,   /* U+2CEB */
  0x0000,   /* U+2CEC */
  0x0000,   /* U+2CED */
  0x0000,   /* U+2CEE */
  0x0000,   /* U+2CEF */
  0x0000,   /* U+2CF0 */
  0x0000,   /* U+2CF1 */
  0x0000,   /* U+2CF2 */
  0x0000,   /* U+2CF3 */
  0x0000,   /* U+2CF4 */
  0x0000,   /* U+2CF5 */
  0x0000,   /* U+2CF6 */
  0x0000,   /* U+2CF7 */
  0x0000,   /* U+2CF8 */
  0x0000,   /* U+2CF9 */
  0x0000,   /* U+2CFA */
  0x0000,   /* U+2CFB */
  0x0000,   /* U+2CFC */
  0x0000,   /* U+2CFD */
  0x0000,   /* U+2CFE */
  0x0000,   /* U+2CFF */
};

uint16 uca900_p02D[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 2, 2, 2, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x223B,   /* U+2D00 */
  0x223D,   /* U+2D01 */
  0x223F,   /* U+2D02 */
  0x2241,   /* U+2D03 */
  0x2243,   /* U+2D04 */
  0x2245,   /* U+2D05 */
  0x2247,   /* U+2D06 */
  0x224B,   /* U+2D07 */
  0x224D,   /* U+2D08 */
  0x224F,   /* U+2D09 */
  0x2251,   /* U+2D0A */
  0x2253,   /* U+2D0B */
  0x2255,   /* U+2D0C */
  0x2259,   /* U+2D0D */
  0x225B,   /* U+2D0E */
  0x225D,   /* U+2D0F */
  0x225F,   /* U+2D10 */
  0x2261,   /* U+2D11 */
  0x2263,   /* U+2D12 */
  0x2267,   /* U+2D13 */
  0x2269,   /* U+2D14 */
  0x226B,   /* U+2D15 */
  0x226D,   /* U+2D16 */
  0x226F,   /* U+2D17 */
  0x2271,   /* U+2D18 */
  0x2273,   /* U+2D19 */
  0x2275,   /* U+2D1A */
  0x2277,   /* U+2D1B */
  0x2279,   /* U+2D1C */
  0x227B,   /* U+2D1D */
  0x227D,   /* U+2D1E */
  0x2281,   /* U+2D1F */
  0x2283,   /* U+2D20 */
  0x2249,   /* U+2D21 */
  0x2257,   /* U+2D22 */
  0x2265,   /* U+2D23 */
  0x227F,   /* U+2D24 */
  0x2285,   /* U+2D25 */
  0xFBC0,   /* U+2D26 */
  0x2288,   /* U+2D27 */
  0xFBC0,   /* U+2D28 */
  0xFBC0,   /* U+2D29 */
  0xFBC0,   /* U+2D2A */
  0xFBC0,   /* U+2D2B */
  0xFBC0,   /* U+2D2C */
  0x228D,   /* U+2D2D */
  0xFBC0,   /* U+2D2E */
  0xFBC0,   /* U+2D2F */
  0x245D,   /* U+2D30 */
  0x245E,   /* U+2D31 */
  0x245F,   /* U+2D32 */
  0x2460,   /* U+2D33 */
  0x2461,   /* U+2D34 */
  0x2462,   /* U+2D35 */
  0x2463,   /* U+2D36 */
  0x2464,   /* U+2D37 */
  0x2465,   /* U+2D38 */
  0x2466,   /* U+2D39 */
  0x2467,   /* U+2D3A */
  0x2468,   /* U+2D3B */
  0x246A,   /* U+2D3C */
  0x246B,   /* U+2D3D */
  0x246C,   /* U+2D3E */
  0x246D,   /* U+2D3F */
  0x246E,   /* U+2D40 */
  0x246F,   /* U+2D41 */
  0x2470,   /* U+2D42 */
  0x2471,   /* U+2D43 */
  0x2472,   /* U+2D44 */
  0x2473,   /* U+2D45 */
  0x2474,   /* U+2D46 */
  0x2475,   /* U+2D47 */
  0x2476,   /* U+2D48 */
  0x2477,   /* U+2D49 */
  0x2478,   /* U+2D4A */
  0x2479,   /* U+2D4B */
  0x247A,   /* U+2D4C */
  0x247B,   /* U+2D4D */
  0x247C,   /* U+2D4E */
  0x247D,   /* U+2D4F */
  0x247E,   /* U+2D50 */
  0x247F,   /* U+2D51 */
  0x2480,   /* U+2D52 */
  0x2481,   /* U+2D53 */
  0x2483,   /* U+2D54 */
  0x2484,   /* U+2D55 */
  0x2485,   /* U+2D56 */
  0x2486,   /* U+2D57 */
  0x2487,   /* U+2D58 */
  0x2488,   /* U+2D59 */
  0x2489,   /* U+2D5A */
  0x248A,   /* U+2D5B */
  0x248B,   /* U+2D5C */
  0x248C,   /* U+2D5D */
  0x248D,   /* U+2D5E */
  0x248E,   /* U+2D5F */
  0x248F,   /* U+2D60 */
  0x2490,   /* U+2D61 */
  0x2491,   /* U+2D62 */
  0x2492,   /* U+2D63 */
  0x2493,   /* U+2D64 */
  0x2494,   /* U+2D65 */
  0x2469,   /* U+2D66 */
  0x2482,   /* U+2D67 */
  0xFBC0,   /* U+2D68 */
  0xFBC0,   /* U+2D69 */
  0xFBC0,   /* U+2D6A */
  0xFBC0,   /* U+2D6B */
  0xFBC0,   /* U+2D6C */
  0xFBC0,   /* U+2D6D */
  0xFBC0,   /* U+2D6E */
  0x2495,   /* U+2D6F */
  0x0427,   /* U+2D70 */
  0xFBC0,   /* U+2D71 */
  0xFBC0,   /* U+2D72 */
  0xFBC0,   /* U+2D73 */
  0xFBC0,   /* U+2D74 */
  0xFBC0,   /* U+2D75 */
  0xFBC0,   /* U+2D76 */
  0xFBC0,   /* U+2D77 */
  0xFBC0,   /* U+2D78 */
  0xFBC0,   /* U+2D79 */
  0xFBC0,   /* U+2D7A */
  0xFBC0,   /* U+2D7B */
  0xFBC0,   /* U+2D7C */
  0xFBC0,   /* U+2D7D */
  0xFBC0,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x24A6,   /* U+2D80 */
  0x24BB,   /* U+2D81 */
  0x24CC,   /* U+2D82 */
  0x24D5,   /* U+2D83 */
  0x24E4,   /* U+2D84 */
  0x250A,   /* U+2D85 */
  0x251B,   /* U+2D86 */
  0x2524,   /* U+2D87 */
  0x253A,   /* U+2D88 */
  0x2543,   /* U+2D89 */
  0x254C,   /* U+2D8A */
  0x257D,   /* U+2D8B */
  0x259C,   /* U+2D8C */
  0x25AB,   /* U+2D8D */
  0x25B4,   /* U+2D8E */
  0x25D6,   /* U+2D8F */
  0x25DF,   /* U+2D90 */
  0x25EF,   /* U+2D91 */
  0x261F,   /* U+2D92 */
  0x25CA,   /* U+2D93 */
  0x25CB,   /* U+2D94 */
  0x25CC,   /* U+2D95 */
  0x25CD,   /* U+2D96 */
  0xFBC0,   /* U+2D97 */
  0xFBC0,   /* U+2D98 */
  0xFBC0,   /* U+2D99 */
  0xFBC0,   /* U+2D9A */
  0xFBC0,   /* U+2D9B */
  0xFBC0,   /* U+2D9C */
  0xFBC0,   /* U+2D9D */
  0xFBC0,   /* U+2D9E */
  0xFBC0,   /* U+2D9F */
  0x2623,   /* U+2DA0 */
  0x2624,   /* U+2DA1 */
  0x2625,   /* U+2DA2 */
  0x2626,   /* U+2DA3 */
  0x2627,   /* U+2DA4 */
  0x2628,   /* U+2DA5 */
  0x2629,   /* U+2DA6 */
  0xFBC0,   /* U+2DA7 */
  0x262A,   /* U+2DA8 */
  0x262B,   /* U+2DA9 */
  0x262C,   /* U+2DAA */
  0x262D,   /* U+2DAB */
  0x262E,   /* U+2DAC */
  0x262F,   /* U+2DAD */
  0x2630,   /* U+2DAE */
  0xFBC0,   /* U+2DAF */
  0x2631,   /* U+2DB0 */
  0x2632,   /* U+2DB1 */
  0x2633,   /* U+2DB2 */
  0x2634,   /* U+2DB3 */
  0x2635,   /* U+2DB4 */
  0x2636,   /* U+2DB5 */
  0x2637,   /* U+2DB6 */
  0xFBC0,   /* U+2DB7 */
  0x2638,   /* U+2DB8 */
  0x2639,   /* U+2DB9 */
  0x263A,   /* U+2DBA */
  0x263B,   /* U+2DBB */
  0x263C,   /* U+2DBC */
  0x263D,   /* U+2DBD */
  0x263E,   /* U+2DBE */
  0xFBC0,   /* U+2DBF */
  0x263F,   /* U+2DC0 */
  0x2640,   /* U+2DC1 */
  0x2641,   /* U+2DC2 */
  0x2642,   /* U+2DC3 */
  0x2643,   /* U+2DC4 */
  0x2644,   /* U+2DC5 */
  0x2645,   /* U+2DC6 */
  0xFBC0,   /* U+2DC7 */
  0x2646,   /* U+2DC8 */
  0x2647,   /* U+2DC9 */
  0x2648,   /* U+2DCA */
  0x2649,   /* U+2DCB */
  0x264A,   /* U+2DCC */
  0x264B,   /* U+2DCD */
  0x264C,   /* U+2DCE */
  0xFBC0,   /* U+2DCF */
  0x264D,   /* U+2DD0 */
  0x264E,   /* U+2DD1 */
  0x264F,   /* U+2DD2 */
  0x2650,   /* U+2DD3 */
  0x2651,   /* U+2DD4 */
  0x2652,   /* U+2DD5 */
  0x2653,   /* U+2DD6 */
  0xFBC0,   /* U+2DD7 */
  0x2654,   /* U+2DD8 */
  0x2655,   /* U+2DD9 */
  0x2656,   /* U+2DDA */
  0x2657,   /* U+2DDB */
  0x2658,   /* U+2DDC */
  0x2659,   /* U+2DDD */
  0x265A,   /* U+2DDE */
  0xFBC0,   /* U+2DDF */
  0x202E,   /* U+2DE0 */
  0x2032,   /* U+2DE1 */
  0x2036,   /* U+2DE2 */
  0x204A,   /* U+2DE3 */
  0x2062,   /* U+2DE4 */
  0x206C,   /* U+2DE5 */
  0x2096,   /* U+2DE6 */
  0x20B0,   /* U+2DE7 */
  0x20C3,   /* U+2DE8 */
  0x20CC,   /* U+2DE9 */
  0x20E7,   /* U+2DEA */
  0x20EF,   /* U+2DEB */
  0x20FC,   /* U+2DEC */
  0x2105,   /* U+2DED */
  0x210E,   /* U+2DEE */
  0x2132,   /* U+2DEF */
  0x2159,   /* U+2DF0 */
  0x2164,   /* U+2DF1 */
  0x2183,   /* U+2DF2 */
  0x2188,   /* U+2DF3 */
  0x21D3,   /* U+2DF4 */
  0x2105,   /* U+2DF5 */
  0x2022,   /* U+2DF6 */
  0x205A,   /* U+2DF7 */
  0x2095,   /* U+2DF8 */
  0x2129,   /* U+2DF9 */
  0x21A0,   /* U+2DFA */
  0x21A9,   /* U+2DFB */
  0x21AE,   /* U+2DFC */
  0x21B8,   /* U+2DFD */
  0x21BD,   /* U+2DFE */
  0x21C7,   /* U+2DFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2D00 */
  0x0020,   /* U+2D01 */
  0x0020,   /* U+2D02 */
  0x0020,   /* U+2D03 */
  0x0020,   /* U+2D04 */
  0x0020,   /* U+2D05 */
  0x0020,   /* U+2D06 */
  0x0020,   /* U+2D07 */
  0x0020,   /* U+2D08 */
  0x0020,   /* U+2D09 */
  0x0020,   /* U+2D0A */
  0x0020,   /* U+2D0B */
  0x0020,   /* U+2D0C */
  0x0020,   /* U+2D0D */
  0x0020,   /* U+2D0E */
  0x0020,   /* U+2D0F */
  0x0020,   /* U+2D10 */
  0x0020,   /* U+2D11 */
  0x0020,   /* U+2D12 */
  0x0020,   /* U+2D13 */
  0x0020,   /* U+2D14 */
  0x0020,   /* U+2D15 */
  0x0020,   /* U+2D16 */
  0x0020,   /* U+2D17 */
  0x0020,   /* U+2D18 */
  0x0020,   /* U+2D19 */
  0x0020,   /* U+2D1A */
  0x0020,   /* U+2D1B */
  0x0020,   /* U+2D1C */
  0x0020,   /* U+2D1D */
  0x0020,   /* U+2D1E */
  0x0020,   /* U+2D1F */
  0x0020,   /* U+2D20 */
  0x0020,   /* U+2D21 */
  0x0020,   /* U+2D22 */
  0x0020,   /* U+2D23 */
  0x0020,   /* U+2D24 */
  0x0020,   /* U+2D25 */
  0x0020,   /* U+2D26 */
  0x0020,   /* U+2D27 */
  0x0020,   /* U+2D28 */
  0x0020,   /* U+2D29 */
  0x0020,   /* U+2D2A */
  0x0020,   /* U+2D2B */
  0x0020,   /* U+2D2C */
  0x0020,   /* U+2D2D */
  0x0020,   /* U+2D2E */
  0x0020,   /* U+2D2F */
  0x0020,   /* U+2D30 */
  0x0020,   /* U+2D31 */
  0x0020,   /* U+2D32 */
  0x0020,   /* U+2D33 */
  0x0020,   /* U+2D34 */
  0x0020,   /* U+2D35 */
  0x0020,   /* U+2D36 */
  0x0020,   /* U+2D37 */
  0x0020,   /* U+2D38 */
  0x0020,   /* U+2D39 */
  0x0020,   /* U+2D3A */
  0x0020,   /* U+2D3B */
  0x0020,   /* U+2D3C */
  0x0020,   /* U+2D3D */
  0x0020,   /* U+2D3E */
  0x0020,   /* U+2D3F */
  0x0020,   /* U+2D40 */
  0x0020,   /* U+2D41 */
  0x0020,   /* U+2D42 */
  0x0020,   /* U+2D43 */
  0x0020,   /* U+2D44 */
  0x0020,   /* U+2D45 */
  0x0020,   /* U+2D46 */
  0x0020,   /* U+2D47 */
  0x0020,   /* U+2D48 */
  0x0020,   /* U+2D49 */
  0x0020,   /* U+2D4A */
  0x0020,   /* U+2D4B */
  0x0020,   /* U+2D4C */
  0x0020,   /* U+2D4D */
  0x0020,   /* U+2D4E */
  0x0020,   /* U+2D4F */
  0x0020,   /* U+2D50 */
  0x0020,   /* U+2D51 */
  0x0020,   /* U+2D52 */
  0x0020,   /* U+2D53 */
  0x0020,   /* U+2D54 */
  0x0020,   /* U+2D55 */
  0x0020,   /* U+2D56 */
  0x0020,   /* U+2D57 */
  0x0020,   /* U+2D58 */
  0x0020,   /* U+2D59 */
  0x0020,   /* U+2D5A */
  0x0020,   /* U+2D5B */
  0x0020,   /* U+2D5C */
  0x0020,   /* U+2D5D */
  0x0020,   /* U+2D5E */
  0x0020,   /* U+2D5F */
  0x0020,   /* U+2D60 */
  0x0020,   /* U+2D61 */
  0x0020,   /* U+2D62 */
  0x0020,   /* U+2D63 */
  0x0020,   /* U+2D64 */
  0x0020,   /* U+2D65 */
  0x0020,   /* U+2D66 */
  0x0020,   /* U+2D67 */
  0x0020,   /* U+2D68 */
  0x0020,   /* U+2D69 */
  0x0020,   /* U+2D6A */
  0x0020,   /* U+2D6B */
  0x0020,   /* U+2D6C */
  0x0020,   /* U+2D6D */
  0x0020,   /* U+2D6E */
  0x0020,   /* U+2D6F */
  0x0020,   /* U+2D70 */
  0x0020,   /* U+2D71 */
  0x0020,   /* U+2D72 */
  0x0020,   /* U+2D73 */
  0x0020,   /* U+2D74 */
  0x0020,   /* U+2D75 */
  0x0020,   /* U+2D76 */
  0x0020,   /* U+2D77 */
  0x0020,   /* U+2D78 */
  0x0020,   /* U+2D79 */
  0x0020,   /* U+2D7A */
  0x0020,   /* U+2D7B */
  0x0020,   /* U+2D7C */
  0x0020,   /* U+2D7D */
  0x0020,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x0020,   /* U+2D80 */
  0x0020,   /* U+2D81 */
  0x0020,   /* U+2D82 */
  0x0020,   /* U+2D83 */
  0x0020,   /* U+2D84 */
  0x0020,   /* U+2D85 */
  0x0020,   /* U+2D86 */
  0x0020,   /* U+2D87 */
  0x0020,   /* U+2D88 */
  0x0020,   /* U+2D89 */
  0x0020,   /* U+2D8A */
  0x0020,   /* U+2D8B */
  0x0020,   /* U+2D8C */
  0x0020,   /* U+2D8D */
  0x0020,   /* U+2D8E */
  0x0020,   /* U+2D8F */
  0x0020,   /* U+2D90 */
  0x0020,   /* U+2D91 */
  0x0020,   /* U+2D92 */
  0x0020,   /* U+2D93 */
  0x0020,   /* U+2D94 */
  0x0020,   /* U+2D95 */
  0x0020,   /* U+2D96 */
  0x0020,   /* U+2D97 */
  0x0020,   /* U+2D98 */
  0x0020,   /* U+2D99 */
  0x0020,   /* U+2D9A */
  0x0020,   /* U+2D9B */
  0x0020,   /* U+2D9C */
  0x0020,   /* U+2D9D */
  0x0020,   /* U+2D9E */
  0x0020,   /* U+2D9F */
  0x0020,   /* U+2DA0 */
  0x0020,   /* U+2DA1 */
  0x0020,   /* U+2DA2 */
  0x0020,   /* U+2DA3 */
  0x0020,   /* U+2DA4 */
  0x0020,   /* U+2DA5 */
  0x0020,   /* U+2DA6 */
  0x0020,   /* U+2DA7 */
  0x0020,   /* U+2DA8 */
  0x0020,   /* U+2DA9 */
  0x0020,   /* U+2DAA */
  0x0020,   /* U+2DAB */
  0x0020,   /* U+2DAC */
  0x0020,   /* U+2DAD */
  0x0020,   /* U+2DAE */
  0x0020,   /* U+2DAF */
  0x0020,   /* U+2DB0 */
  0x0020,   /* U+2DB1 */
  0x0020,   /* U+2DB2 */
  0x0020,   /* U+2DB3 */
  0x0020,   /* U+2DB4 */
  0x0020,   /* U+2DB5 */
  0x0020,   /* U+2DB6 */
  0x0020,   /* U+2DB7 */
  0x0020,   /* U+2DB8 */
  0x0020,   /* U+2DB9 */
  0x0020,   /* U+2DBA */
  0x0020,   /* U+2DBB */
  0x0020,   /* U+2DBC */
  0x0020,   /* U+2DBD */
  0x0020,   /* U+2DBE */
  0x0020,   /* U+2DBF */
  0x0020,   /* U+2DC0 */
  0x0020,   /* U+2DC1 */
  0x0020,   /* U+2DC2 */
  0x0020,   /* U+2DC3 */
  0x0020,   /* U+2DC4 */
  0x0020,   /* U+2DC5 */
  0x0020,   /* U+2DC6 */
  0x0020,   /* U+2DC7 */
  0x0020,   /* U+2DC8 */
  0x0020,   /* U+2DC9 */
  0x0020,   /* U+2DCA */
  0x0020,   /* U+2DCB */
  0x0020,   /* U+2DCC */
  0x0020,   /* U+2DCD */
  0x0020,   /* U+2DCE */
  0x0020,   /* U+2DCF */
  0x0020,   /* U+2DD0 */
  0x0020,   /* U+2DD1 */
  0x0020,   /* U+2DD2 */
  0x0020,   /* U+2DD3 */
  0x0020,   /* U+2DD4 */
  0x0020,   /* U+2DD5 */
  0x0020,   /* U+2DD6 */
  0x0020,   /* U+2DD7 */
  0x0020,   /* U+2DD8 */
  0x0020,   /* U+2DD9 */
  0x0020,   /* U+2DDA */
  0x0020,   /* U+2DDB */
  0x0020,   /* U+2DDC */
  0x0020,   /* U+2DDD */
  0x0020,   /* U+2DDE */
  0x0020,   /* U+2DDF */
  0x0020,   /* U+2DE0 */
  0x0020,   /* U+2DE1 */
  0x0020,   /* U+2DE2 */
  0x0020,   /* U+2DE3 */
  0x0020,   /* U+2DE4 */
  0x0020,   /* U+2DE5 */
  0x0020,   /* U+2DE6 */
  0x0020,   /* U+2DE7 */
  0x0020,   /* U+2DE8 */
  0x0020,   /* U+2DE9 */
  0x0020,   /* U+2DEA */
  0x0020,   /* U+2DEB */
  0x0020,   /* U+2DEC */
  0x0020,   /* U+2DED */
  0x0020,   /* U+2DEE */
  0x0020,   /* U+2DEF */
  0x0020,   /* U+2DF0 */
  0x0020,   /* U+2DF1 */
  0x0020,   /* U+2DF2 */
  0x0020,   /* U+2DF3 */
  0x0020,   /* U+2DF4 */
  0x0020,   /* U+2DF5 */
  0x0020,   /* U+2DF6 */
  0x0020,   /* U+2DF7 */
  0x0020,   /* U+2DF8 */
  0x0020,   /* U+2DF9 */
  0x0020,   /* U+2DFA */
  0x0020,   /* U+2DFB */
  0x0020,   /* U+2DFC */
  0x0020,   /* U+2DFD */
  0x0020,   /* U+2DFE */
  0x0020,   /* U+2DFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2D00 */
  0x0002,   /* U+2D01 */
  0x0002,   /* U+2D02 */
  0x0002,   /* U+2D03 */
  0x0002,   /* U+2D04 */
  0x0002,   /* U+2D05 */
  0x0002,   /* U+2D06 */
  0x0002,   /* U+2D07 */
  0x0002,   /* U+2D08 */
  0x0002,   /* U+2D09 */
  0x0002,   /* U+2D0A */
  0x0002,   /* U+2D0B */
  0x0002,   /* U+2D0C */
  0x0002,   /* U+2D0D */
  0x0002,   /* U+2D0E */
  0x0002,   /* U+2D0F */
  0x0002,   /* U+2D10 */
  0x0002,   /* U+2D11 */
  0x0002,   /* U+2D12 */
  0x0002,   /* U+2D13 */
  0x0002,   /* U+2D14 */
  0x0002,   /* U+2D15 */
  0x0002,   /* U+2D16 */
  0x0002,   /* U+2D17 */
  0x0002,   /* U+2D18 */
  0x0002,   /* U+2D19 */
  0x0002,   /* U+2D1A */
  0x0002,   /* U+2D1B */
  0x0002,   /* U+2D1C */
  0x0002,   /* U+2D1D */
  0x0002,   /* U+2D1E */
  0x0002,   /* U+2D1F */
  0x0002,   /* U+2D20 */
  0x0002,   /* U+2D21 */
  0x0002,   /* U+2D22 */
  0x0002,   /* U+2D23 */
  0x0002,   /* U+2D24 */
  0x0002,   /* U+2D25 */
  0x0002,   /* U+2D26 */
  0x0002,   /* U+2D27 */
  0x0002,   /* U+2D28 */
  0x0002,   /* U+2D29 */
  0x0002,   /* U+2D2A */
  0x0002,   /* U+2D2B */
  0x0002,   /* U+2D2C */
  0x0002,   /* U+2D2D */
  0x0002,   /* U+2D2E */
  0x0002,   /* U+2D2F */
  0x0002,   /* U+2D30 */
  0x0002,   /* U+2D31 */
  0x0002,   /* U+2D32 */
  0x0002,   /* U+2D33 */
  0x0002,   /* U+2D34 */
  0x0002,   /* U+2D35 */
  0x0002,   /* U+2D36 */
  0x0002,   /* U+2D37 */
  0x0002,   /* U+2D38 */
  0x0002,   /* U+2D39 */
  0x0002,   /* U+2D3A */
  0x0002,   /* U+2D3B */
  0x0002,   /* U+2D3C */
  0x0002,   /* U+2D3D */
  0x0002,   /* U+2D3E */
  0x0002,   /* U+2D3F */
  0x0002,   /* U+2D40 */
  0x0002,   /* U+2D41 */
  0x0002,   /* U+2D42 */
  0x0002,   /* U+2D43 */
  0x0002,   /* U+2D44 */
  0x0002,   /* U+2D45 */
  0x0002,   /* U+2D46 */
  0x0002,   /* U+2D47 */
  0x0002,   /* U+2D48 */
  0x0002,   /* U+2D49 */
  0x0002,   /* U+2D4A */
  0x0002,   /* U+2D4B */
  0x0002,   /* U+2D4C */
  0x0002,   /* U+2D4D */
  0x0002,   /* U+2D4E */
  0x0002,   /* U+2D4F */
  0x0002,   /* U+2D50 */
  0x0002,   /* U+2D51 */
  0x0002,   /* U+2D52 */
  0x0002,   /* U+2D53 */
  0x0002,   /* U+2D54 */
  0x0002,   /* U+2D55 */
  0x0002,   /* U+2D56 */
  0x0002,   /* U+2D57 */
  0x0002,   /* U+2D58 */
  0x0002,   /* U+2D59 */
  0x0002,   /* U+2D5A */
  0x0002,   /* U+2D5B */
  0x0002,   /* U+2D5C */
  0x0002,   /* U+2D5D */
  0x0002,   /* U+2D5E */
  0x0002,   /* U+2D5F */
  0x0002,   /* U+2D60 */
  0x0002,   /* U+2D61 */
  0x0002,   /* U+2D62 */
  0x0002,   /* U+2D63 */
  0x0002,   /* U+2D64 */
  0x0002,   /* U+2D65 */
  0x0002,   /* U+2D66 */
  0x0002,   /* U+2D67 */
  0x0002,   /* U+2D68 */
  0x0002,   /* U+2D69 */
  0x0002,   /* U+2D6A */
  0x0002,   /* U+2D6B */
  0x0002,   /* U+2D6C */
  0x0002,   /* U+2D6D */
  0x0002,   /* U+2D6E */
  0x0002,   /* U+2D6F */
  0x0002,   /* U+2D70 */
  0x0002,   /* U+2D71 */
  0x0002,   /* U+2D72 */
  0x0002,   /* U+2D73 */
  0x0002,   /* U+2D74 */
  0x0002,   /* U+2D75 */
  0x0002,   /* U+2D76 */
  0x0002,   /* U+2D77 */
  0x0002,   /* U+2D78 */
  0x0002,   /* U+2D79 */
  0x0002,   /* U+2D7A */
  0x0002,   /* U+2D7B */
  0x0002,   /* U+2D7C */
  0x0002,   /* U+2D7D */
  0x0002,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x0002,   /* U+2D80 */
  0x0002,   /* U+2D81 */
  0x0002,   /* U+2D82 */
  0x0002,   /* U+2D83 */
  0x0002,   /* U+2D84 */
  0x0002,   /* U+2D85 */
  0x0002,   /* U+2D86 */
  0x0002,   /* U+2D87 */
  0x0002,   /* U+2D88 */
  0x0002,   /* U+2D89 */
  0x0002,   /* U+2D8A */
  0x0002,   /* U+2D8B */
  0x0002,   /* U+2D8C */
  0x0002,   /* U+2D8D */
  0x0002,   /* U+2D8E */
  0x0002,   /* U+2D8F */
  0x0002,   /* U+2D90 */
  0x0002,   /* U+2D91 */
  0x0002,   /* U+2D92 */
  0x0002,   /* U+2D93 */
  0x0002,   /* U+2D94 */
  0x0002,   /* U+2D95 */
  0x0002,   /* U+2D96 */
  0x0002,   /* U+2D97 */
  0x0002,   /* U+2D98 */
  0x0002,   /* U+2D99 */
  0x0002,   /* U+2D9A */
  0x0002,   /* U+2D9B */
  0x0002,   /* U+2D9C */
  0x0002,   /* U+2D9D */
  0x0002,   /* U+2D9E */
  0x0002,   /* U+2D9F */
  0x0002,   /* U+2DA0 */
  0x0002,   /* U+2DA1 */
  0x0002,   /* U+2DA2 */
  0x0002,   /* U+2DA3 */
  0x0002,   /* U+2DA4 */
  0x0002,   /* U+2DA5 */
  0x0002,   /* U+2DA6 */
  0x0002,   /* U+2DA7 */
  0x0002,   /* U+2DA8 */
  0x0002,   /* U+2DA9 */
  0x0002,   /* U+2DAA */
  0x0002,   /* U+2DAB */
  0x0002,   /* U+2DAC */
  0x0002,   /* U+2DAD */
  0x0002,   /* U+2DAE */
  0x0002,   /* U+2DAF */
  0x0002,   /* U+2DB0 */
  0x0002,   /* U+2DB1 */
  0x0002,   /* U+2DB2 */
  0x0002,   /* U+2DB3 */
  0x0002,   /* U+2DB4 */
  0x0002,   /* U+2DB5 */
  0x0002,   /* U+2DB6 */
  0x0002,   /* U+2DB7 */
  0x0002,   /* U+2DB8 */
  0x0002,   /* U+2DB9 */
  0x0002,   /* U+2DBA */
  0x0002,   /* U+2DBB */
  0x0002,   /* U+2DBC */
  0x0002,   /* U+2DBD */
  0x0002,   /* U+2DBE */
  0x0002,   /* U+2DBF */
  0x0002,   /* U+2DC0 */
  0x0002,   /* U+2DC1 */
  0x0002,   /* U+2DC2 */
  0x0002,   /* U+2DC3 */
  0x0002,   /* U+2DC4 */
  0x0002,   /* U+2DC5 */
  0x0002,   /* U+2DC6 */
  0x0002,   /* U+2DC7 */
  0x0002,   /* U+2DC8 */
  0x0002,   /* U+2DC9 */
  0x0002,   /* U+2DCA */
  0x0002,   /* U+2DCB */
  0x0002,   /* U+2DCC */
  0x0002,   /* U+2DCD */
  0x0002,   /* U+2DCE */
  0x0002,   /* U+2DCF */
  0x0002,   /* U+2DD0 */
  0x0002,   /* U+2DD1 */
  0x0002,   /* U+2DD2 */
  0x0002,   /* U+2DD3 */
  0x0002,   /* U+2DD4 */
  0x0002,   /* U+2DD5 */
  0x0002,   /* U+2DD6 */
  0x0002,   /* U+2DD7 */
  0x0002,   /* U+2DD8 */
  0x0002,   /* U+2DD9 */
  0x0002,   /* U+2DDA */
  0x0002,   /* U+2DDB */
  0x0002,   /* U+2DDC */
  0x0002,   /* U+2DDD */
  0x0002,   /* U+2DDE */
  0x0002,   /* U+2DDF */
  0x0004,   /* U+2DE0 */
  0x0004,   /* U+2DE1 */
  0x0004,   /* U+2DE2 */
  0x0004,   /* U+2DE3 */
  0x0004,   /* U+2DE4 */
  0x0004,   /* U+2DE5 */
  0x0004,   /* U+2DE6 */
  0x0004,   /* U+2DE7 */
  0x0004,   /* U+2DE8 */
  0x0004,   /* U+2DE9 */
  0x0004,   /* U+2DEA */
  0x0004,   /* U+2DEB */
  0x0004,   /* U+2DEC */
  0x0004,   /* U+2DED */
  0x0004,   /* U+2DEE */
  0x0004,   /* U+2DEF */
  0x0004,   /* U+2DF0 */
  0x0004,   /* U+2DF1 */
  0x0004,   /* U+2DF2 */
  0x0004,   /* U+2DF3 */
  0x0004,   /* U+2DF4 */
  0x0004,   /* U+2DF5 */
  0x0004,   /* U+2DF6 */
  0x0004,   /* U+2DF7 */
  0x0004,   /* U+2DF8 */
  0x0004,   /* U+2DF9 */
  0x0004,   /* U+2DFA */
  0x0004,   /* U+2DFB */
  0x0004,   /* U+2DFC */
  0x0004,   /* U+2DFD */
  0x0004,   /* U+2DFE */
  0x0004,   /* U+2DFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2D00 */
  0x0000,   /* U+2D01 */
  0x0000,   /* U+2D02 */
  0x0000,   /* U+2D03 */
  0x0000,   /* U+2D04 */
  0x0000,   /* U+2D05 */
  0x0000,   /* U+2D06 */
  0x0000,   /* U+2D07 */
  0x0000,   /* U+2D08 */
  0x0000,   /* U+2D09 */
  0x0000,   /* U+2D0A */
  0x0000,   /* U+2D0B */
  0x0000,   /* U+2D0C */
  0x0000,   /* U+2D0D */
  0x0000,   /* U+2D0E */
  0x0000,   /* U+2D0F */
  0x0000,   /* U+2D10 */
  0x0000,   /* U+2D11 */
  0x0000,   /* U+2D12 */
  0x0000,   /* U+2D13 */
  0x0000,   /* U+2D14 */
  0x0000,   /* U+2D15 */
  0x0000,   /* U+2D16 */
  0x0000,   /* U+2D17 */
  0x0000,   /* U+2D18 */
  0x0000,   /* U+2D19 */
  0x0000,   /* U+2D1A */
  0x0000,   /* U+2D1B */
  0x0000,   /* U+2D1C */
  0x0000,   /* U+2D1D */
  0x0000,   /* U+2D1E */
  0x0000,   /* U+2D1F */
  0x0000,   /* U+2D20 */
  0x0000,   /* U+2D21 */
  0x0000,   /* U+2D22 */
  0x0000,   /* U+2D23 */
  0x0000,   /* U+2D24 */
  0x0000,   /* U+2D25 */
  0xAD26,   /* U+2D26 */
  0x0000,   /* U+2D27 */
  0xAD28,   /* U+2D28 */
  0xAD29,   /* U+2D29 */
  0xAD2A,   /* U+2D2A */
  0xAD2B,   /* U+2D2B */
  0xAD2C,   /* U+2D2C */
  0x0000,   /* U+2D2D */
  0xAD2E,   /* U+2D2E */
  0xAD2F,   /* U+2D2F */
  0x0000,   /* U+2D30 */
  0x0000,   /* U+2D31 */
  0x0000,   /* U+2D32 */
  0x0000,   /* U+2D33 */
  0x0000,   /* U+2D34 */
  0x0000,   /* U+2D35 */
  0x0000,   /* U+2D36 */
  0x0000,   /* U+2D37 */
  0x0000,   /* U+2D38 */
  0x0000,   /* U+2D39 */
  0x0000,   /* U+2D3A */
  0x0000,   /* U+2D3B */
  0x0000,   /* U+2D3C */
  0x0000,   /* U+2D3D */
  0x0000,   /* U+2D3E */
  0x0000,   /* U+2D3F */
  0x0000,   /* U+2D40 */
  0x0000,   /* U+2D41 */
  0x0000,   /* U+2D42 */
  0x0000,   /* U+2D43 */
  0x0000,   /* U+2D44 */
  0x0000,   /* U+2D45 */
  0x0000,   /* U+2D46 */
  0x0000,   /* U+2D47 */
  0x0000,   /* U+2D48 */
  0x0000,   /* U+2D49 */
  0x0000,   /* U+2D4A */
  0x0000,   /* U+2D4B */
  0x0000,   /* U+2D4C */
  0x0000,   /* U+2D4D */
  0x0000,   /* U+2D4E */
  0x0000,   /* U+2D4F */
  0x0000,   /* U+2D50 */
  0x0000,   /* U+2D51 */
  0x0000,   /* U+2D52 */
  0x0000,   /* U+2D53 */
  0x0000,   /* U+2D54 */
  0x0000,   /* U+2D55 */
  0x0000,   /* U+2D56 */
  0x0000,   /* U+2D57 */
  0x0000,   /* U+2D58 */
  0x0000,   /* U+2D59 */
  0x0000,   /* U+2D5A */
  0x0000,   /* U+2D5B */
  0x0000,   /* U+2D5C */
  0x0000,   /* U+2D5D */
  0x0000,   /* U+2D5E */
  0x0000,   /* U+2D5F */
  0x0000,   /* U+2D60 */
  0x0000,   /* U+2D61 */
  0x0000,   /* U+2D62 */
  0x0000,   /* U+2D63 */
  0x0000,   /* U+2D64 */
  0x0000,   /* U+2D65 */
  0x0000,   /* U+2D66 */
  0x0000,   /* U+2D67 */
  0xAD68,   /* U+2D68 */
  0xAD69,   /* U+2D69 */
  0xAD6A,   /* U+2D6A */
  0xAD6B,   /* U+2D6B */
  0xAD6C,   /* U+2D6C */
  0xAD6D,   /* U+2D6D */
  0xAD6E,   /* U+2D6E */
  0x0000,   /* U+2D6F */
  0x0000,   /* U+2D70 */
  0xAD71,   /* U+2D71 */
  0xAD72,   /* U+2D72 */
  0xAD73,   /* U+2D73 */
  0xAD74,   /* U+2D74 */
  0xAD75,   /* U+2D75 */
  0xAD76,   /* U+2D76 */
  0xAD77,   /* U+2D77 */
  0xAD78,   /* U+2D78 */
  0xAD79,   /* U+2D79 */
  0xAD7A,   /* U+2D7A */
  0xAD7B,   /* U+2D7B */
  0xAD7C,   /* U+2D7C */
  0xAD7D,   /* U+2D7D */
  0xAD7E,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x0000,   /* U+2D80 */
  0x0000,   /* U+2D81 */
  0x0000,   /* U+2D82 */
  0x0000,   /* U+2D83 */
  0x0000,   /* U+2D84 */
  0x0000,   /* U+2D85 */
  0x0000,   /* U+2D86 */
  0x0000,   /* U+2D87 */
  0x0000,   /* U+2D88 */
  0x0000,   /* U+2D89 */
  0x0000,   /* U+2D8A */
  0x0000,   /* U+2D8B */
  0x0000,   /* U+2D8C */
  0x0000,   /* U+2D8D */
  0x0000,   /* U+2D8E */
  0x0000,   /* U+2D8F */
  0x0000,   /* U+2D90 */
  0x0000,   /* U+2D91 */
  0x0000,   /* U+2D92 */
  0x0000,   /* U+2D93 */
  0x0000,   /* U+2D94 */
  0x0000,   /* U+2D95 */
  0x0000,   /* U+2D96 */
  0xAD97,   /* U+2D97 */
  0xAD98,   /* U+2D98 */
  0xAD99,   /* U+2D99 */
  0xAD9A,   /* U+2D9A */
  0xAD9B,   /* U+2D9B */
  0xAD9C,   /* U+2D9C */
  0xAD9D,   /* U+2D9D */
  0xAD9E,   /* U+2D9E */
  0xAD9F,   /* U+2D9F */
  0x0000,   /* U+2DA0 */
  0x0000,   /* U+2DA1 */
  0x0000,   /* U+2DA2 */
  0x0000,   /* U+2DA3 */
  0x0000,   /* U+2DA4 */
  0x0000,   /* U+2DA5 */
  0x0000,   /* U+2DA6 */
  0xADA7,   /* U+2DA7 */
  0x0000,   /* U+2DA8 */
  0x0000,   /* U+2DA9 */
  0x0000,   /* U+2DAA */
  0x0000,   /* U+2DAB */
  0x0000,   /* U+2DAC */
  0x0000,   /* U+2DAD */
  0x0000,   /* U+2DAE */
  0xADAF,   /* U+2DAF */
  0x0000,   /* U+2DB0 */
  0x0000,   /* U+2DB1 */
  0x0000,   /* U+2DB2 */
  0x0000,   /* U+2DB3 */
  0x0000,   /* U+2DB4 */
  0x0000,   /* U+2DB5 */
  0x0000,   /* U+2DB6 */
  0xADB7,   /* U+2DB7 */
  0x0000,   /* U+2DB8 */
  0x0000,   /* U+2DB9 */
  0x0000,   /* U+2DBA */
  0x0000,   /* U+2DBB */
  0x0000,   /* U+2DBC */
  0x0000,   /* U+2DBD */
  0x0000,   /* U+2DBE */
  0xADBF,   /* U+2DBF */
  0x0000,   /* U+2DC0 */
  0x0000,   /* U+2DC1 */
  0x0000,   /* U+2DC2 */
  0x0000,   /* U+2DC3 */
  0x0000,   /* U+2DC4 */
  0x0000,   /* U+2DC5 */
  0x0000,   /* U+2DC6 */
  0xADC7,   /* U+2DC7 */
  0x0000,   /* U+2DC8 */
  0x0000,   /* U+2DC9 */
  0x0000,   /* U+2DCA */
  0x0000,   /* U+2DCB */
  0x0000,   /* U+2DCC */
  0x0000,   /* U+2DCD */
  0x0000,   /* U+2DCE */
  0xADCF,   /* U+2DCF */
  0x0000,   /* U+2DD0 */
  0x0000,   /* U+2DD1 */
  0x0000,   /* U+2DD2 */
  0x0000,   /* U+2DD3 */
  0x0000,   /* U+2DD4 */
  0x0000,   /* U+2DD5 */
  0x0000,   /* U+2DD6 */
  0xADD7,   /* U+2DD7 */
  0x0000,   /* U+2DD8 */
  0x0000,   /* U+2DD9 */
  0x0000,   /* U+2DDA */
  0x0000,   /* U+2DDB */
  0x0000,   /* U+2DDC */
  0x0000,   /* U+2DDD */
  0x0000,   /* U+2DDE */
  0xADDF,   /* U+2DDF */
  0x0000,   /* U+2DE0 */
  0x0000,   /* U+2DE1 */
  0x0000,   /* U+2DE2 */
  0x0000,   /* U+2DE3 */
  0x0000,   /* U+2DE4 */
  0x0000,   /* U+2DE5 */
  0x0000,   /* U+2DE6 */
  0x0000,   /* U+2DE7 */
  0x0000,   /* U+2DE8 */
  0x0000,   /* U+2DE9 */
  0x0000,   /* U+2DEA */
  0x0000,   /* U+2DEB */
  0x0000,   /* U+2DEC */
  0x0000,   /* U+2DED */
  0x0000,   /* U+2DEE */
  0x0000,   /* U+2DEF */
  0x0000,   /* U+2DF0 */
  0x0000,   /* U+2DF1 */
  0x0000,   /* U+2DF2 */
  0x0000,   /* U+2DF3 */
  0x0000,   /* U+2DF4 */
  0x210E,   /* U+2DF5 */
  0x0000,   /* U+2DF6 */
  0x0000,   /* U+2DF7 */
  0x0000,   /* U+2DF8 */
  0x0000,   /* U+2DF9 */
  0x0000,   /* U+2DFA */
  0x0000,   /* U+2DFB */
  0x0000,   /* U+2DFC */
  0x0000,   /* U+2DFD */
  0x0000,   /* U+2DFE */
  0x0000,   /* U+2DFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2D00 */
  0x0000,   /* U+2D01 */
  0x0000,   /* U+2D02 */
  0x0000,   /* U+2D03 */
  0x0000,   /* U+2D04 */
  0x0000,   /* U+2D05 */
  0x0000,   /* U+2D06 */
  0x0000,   /* U+2D07 */
  0x0000,   /* U+2D08 */
  0x0000,   /* U+2D09 */
  0x0000,   /* U+2D0A */
  0x0000,   /* U+2D0B */
  0x0000,   /* U+2D0C */
  0x0000,   /* U+2D0D */
  0x0000,   /* U+2D0E */
  0x0000,   /* U+2D0F */
  0x0000,   /* U+2D10 */
  0x0000,   /* U+2D11 */
  0x0000,   /* U+2D12 */
  0x0000,   /* U+2D13 */
  0x0000,   /* U+2D14 */
  0x0000,   /* U+2D15 */
  0x0000,   /* U+2D16 */
  0x0000,   /* U+2D17 */
  0x0000,   /* U+2D18 */
  0x0000,   /* U+2D19 */
  0x0000,   /* U+2D1A */
  0x0000,   /* U+2D1B */
  0x0000,   /* U+2D1C */
  0x0000,   /* U+2D1D */
  0x0000,   /* U+2D1E */
  0x0000,   /* U+2D1F */
  0x0000,   /* U+2D20 */
  0x0000,   /* U+2D21 */
  0x0000,   /* U+2D22 */
  0x0000,   /* U+2D23 */
  0x0000,   /* U+2D24 */
  0x0000,   /* U+2D25 */
  0x0000,   /* U+2D26 */
  0x0000,   /* U+2D27 */
  0x0000,   /* U+2D28 */
  0x0000,   /* U+2D29 */
  0x0000,   /* U+2D2A */
  0x0000,   /* U+2D2B */
  0x0000,   /* U+2D2C */
  0x0000,   /* U+2D2D */
  0x0000,   /* U+2D2E */
  0x0000,   /* U+2D2F */
  0x0000,   /* U+2D30 */
  0x0000,   /* U+2D31 */
  0x0000,   /* U+2D32 */
  0x0000,   /* U+2D33 */
  0x0000,   /* U+2D34 */
  0x0000,   /* U+2D35 */
  0x0000,   /* U+2D36 */
  0x0000,   /* U+2D37 */
  0x0000,   /* U+2D38 */
  0x0000,   /* U+2D39 */
  0x0000,   /* U+2D3A */
  0x0000,   /* U+2D3B */
  0x0000,   /* U+2D3C */
  0x0000,   /* U+2D3D */
  0x0000,   /* U+2D3E */
  0x0000,   /* U+2D3F */
  0x0000,   /* U+2D40 */
  0x0000,   /* U+2D41 */
  0x0000,   /* U+2D42 */
  0x0000,   /* U+2D43 */
  0x0000,   /* U+2D44 */
  0x0000,   /* U+2D45 */
  0x0000,   /* U+2D46 */
  0x0000,   /* U+2D47 */
  0x0000,   /* U+2D48 */
  0x0000,   /* U+2D49 */
  0x0000,   /* U+2D4A */
  0x0000,   /* U+2D4B */
  0x0000,   /* U+2D4C */
  0x0000,   /* U+2D4D */
  0x0000,   /* U+2D4E */
  0x0000,   /* U+2D4F */
  0x0000,   /* U+2D50 */
  0x0000,   /* U+2D51 */
  0x0000,   /* U+2D52 */
  0x0000,   /* U+2D53 */
  0x0000,   /* U+2D54 */
  0x0000,   /* U+2D55 */
  0x0000,   /* U+2D56 */
  0x0000,   /* U+2D57 */
  0x0000,   /* U+2D58 */
  0x0000,   /* U+2D59 */
  0x0000,   /* U+2D5A */
  0x0000,   /* U+2D5B */
  0x0000,   /* U+2D5C */
  0x0000,   /* U+2D5D */
  0x0000,   /* U+2D5E */
  0x0000,   /* U+2D5F */
  0x0000,   /* U+2D60 */
  0x0000,   /* U+2D61 */
  0x0000,   /* U+2D62 */
  0x0000,   /* U+2D63 */
  0x0000,   /* U+2D64 */
  0x0000,   /* U+2D65 */
  0x0000,   /* U+2D66 */
  0x0000,   /* U+2D67 */
  0x0000,   /* U+2D68 */
  0x0000,   /* U+2D69 */
  0x0000,   /* U+2D6A */
  0x0000,   /* U+2D6B */
  0x0000,   /* U+2D6C */
  0x0000,   /* U+2D6D */
  0x0000,   /* U+2D6E */
  0x0000,   /* U+2D6F */
  0x0000,   /* U+2D70 */
  0x0000,   /* U+2D71 */
  0x0000,   /* U+2D72 */
  0x0000,   /* U+2D73 */
  0x0000,   /* U+2D74 */
  0x0000,   /* U+2D75 */
  0x0000,   /* U+2D76 */
  0x0000,   /* U+2D77 */
  0x0000,   /* U+2D78 */
  0x0000,   /* U+2D79 */
  0x0000,   /* U+2D7A */
  0x0000,   /* U+2D7B */
  0x0000,   /* U+2D7C */
  0x0000,   /* U+2D7D */
  0x0000,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x0000,   /* U+2D80 */
  0x0000,   /* U+2D81 */
  0x0000,   /* U+2D82 */
  0x0000,   /* U+2D83 */
  0x0000,   /* U+2D84 */
  0x0000,   /* U+2D85 */
  0x0000,   /* U+2D86 */
  0x0000,   /* U+2D87 */
  0x0000,   /* U+2D88 */
  0x0000,   /* U+2D89 */
  0x0000,   /* U+2D8A */
  0x0000,   /* U+2D8B */
  0x0000,   /* U+2D8C */
  0x0000,   /* U+2D8D */
  0x0000,   /* U+2D8E */
  0x0000,   /* U+2D8F */
  0x0000,   /* U+2D90 */
  0x0000,   /* U+2D91 */
  0x0000,   /* U+2D92 */
  0x0000,   /* U+2D93 */
  0x0000,   /* U+2D94 */
  0x0000,   /* U+2D95 */
  0x0000,   /* U+2D96 */
  0x0000,   /* U+2D97 */
  0x0000,   /* U+2D98 */
  0x0000,   /* U+2D99 */
  0x0000,   /* U+2D9A */
  0x0000,   /* U+2D9B */
  0x0000,   /* U+2D9C */
  0x0000,   /* U+2D9D */
  0x0000,   /* U+2D9E */
  0x0000,   /* U+2D9F */
  0x0000,   /* U+2DA0 */
  0x0000,   /* U+2DA1 */
  0x0000,   /* U+2DA2 */
  0x0000,   /* U+2DA3 */
  0x0000,   /* U+2DA4 */
  0x0000,   /* U+2DA5 */
  0x0000,   /* U+2DA6 */
  0x0000,   /* U+2DA7 */
  0x0000,   /* U+2DA8 */
  0x0000,   /* U+2DA9 */
  0x0000,   /* U+2DAA */
  0x0000,   /* U+2DAB */
  0x0000,   /* U+2DAC */
  0x0000,   /* U+2DAD */
  0x0000,   /* U+2DAE */
  0x0000,   /* U+2DAF */
  0x0000,   /* U+2DB0 */
  0x0000,   /* U+2DB1 */
  0x0000,   /* U+2DB2 */
  0x0000,   /* U+2DB3 */
  0x0000,   /* U+2DB4 */
  0x0000,   /* U+2DB5 */
  0x0000,   /* U+2DB6 */
  0x0000,   /* U+2DB7 */
  0x0000,   /* U+2DB8 */
  0x0000,   /* U+2DB9 */
  0x0000,   /* U+2DBA */
  0x0000,   /* U+2DBB */
  0x0000,   /* U+2DBC */
  0x0000,   /* U+2DBD */
  0x0000,   /* U+2DBE */
  0x0000,   /* U+2DBF */
  0x0000,   /* U+2DC0 */
  0x0000,   /* U+2DC1 */
  0x0000,   /* U+2DC2 */
  0x0000,   /* U+2DC3 */
  0x0000,   /* U+2DC4 */
  0x0000,   /* U+2DC5 */
  0x0000,   /* U+2DC6 */
  0x0000,   /* U+2DC7 */
  0x0000,   /* U+2DC8 */
  0x0000,   /* U+2DC9 */
  0x0000,   /* U+2DCA */
  0x0000,   /* U+2DCB */
  0x0000,   /* U+2DCC */
  0x0000,   /* U+2DCD */
  0x0000,   /* U+2DCE */
  0x0000,   /* U+2DCF */
  0x0000,   /* U+2DD0 */
  0x0000,   /* U+2DD1 */
  0x0000,   /* U+2DD2 */
  0x0000,   /* U+2DD3 */
  0x0000,   /* U+2DD4 */
  0x0000,   /* U+2DD5 */
  0x0000,   /* U+2DD6 */
  0x0000,   /* U+2DD7 */
  0x0000,   /* U+2DD8 */
  0x0000,   /* U+2DD9 */
  0x0000,   /* U+2DDA */
  0x0000,   /* U+2DDB */
  0x0000,   /* U+2DDC */
  0x0000,   /* U+2DDD */
  0x0000,   /* U+2DDE */
  0x0000,   /* U+2DDF */
  0x0000,   /* U+2DE0 */
  0x0000,   /* U+2DE1 */
  0x0000,   /* U+2DE2 */
  0x0000,   /* U+2DE3 */
  0x0000,   /* U+2DE4 */
  0x0000,   /* U+2DE5 */
  0x0000,   /* U+2DE6 */
  0x0000,   /* U+2DE7 */
  0x0000,   /* U+2DE8 */
  0x0000,   /* U+2DE9 */
  0x0000,   /* U+2DEA */
  0x0000,   /* U+2DEB */
  0x0000,   /* U+2DEC */
  0x0000,   /* U+2DED */
  0x0000,   /* U+2DEE */
  0x0000,   /* U+2DEF */
  0x0000,   /* U+2DF0 */
  0x0000,   /* U+2DF1 */
  0x0000,   /* U+2DF2 */
  0x0000,   /* U+2DF3 */
  0x0000,   /* U+2DF4 */
  0x0020,   /* U+2DF5 */
  0x0000,   /* U+2DF6 */
  0x0000,   /* U+2DF7 */
  0x0000,   /* U+2DF8 */
  0x0000,   /* U+2DF9 */
  0x0000,   /* U+2DFA */
  0x0000,   /* U+2DFB */
  0x0000,   /* U+2DFC */
  0x0000,   /* U+2DFD */
  0x0000,   /* U+2DFE */
  0x0000,   /* U+2DFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2D00 */
  0x0000,   /* U+2D01 */
  0x0000,   /* U+2D02 */
  0x0000,   /* U+2D03 */
  0x0000,   /* U+2D04 */
  0x0000,   /* U+2D05 */
  0x0000,   /* U+2D06 */
  0x0000,   /* U+2D07 */
  0x0000,   /* U+2D08 */
  0x0000,   /* U+2D09 */
  0x0000,   /* U+2D0A */
  0x0000,   /* U+2D0B */
  0x0000,   /* U+2D0C */
  0x0000,   /* U+2D0D */
  0x0000,   /* U+2D0E */
  0x0000,   /* U+2D0F */
  0x0000,   /* U+2D10 */
  0x0000,   /* U+2D11 */
  0x0000,   /* U+2D12 */
  0x0000,   /* U+2D13 */
  0x0000,   /* U+2D14 */
  0x0000,   /* U+2D15 */
  0x0000,   /* U+2D16 */
  0x0000,   /* U+2D17 */
  0x0000,   /* U+2D18 */
  0x0000,   /* U+2D19 */
  0x0000,   /* U+2D1A */
  0x0000,   /* U+2D1B */
  0x0000,   /* U+2D1C */
  0x0000,   /* U+2D1D */
  0x0000,   /* U+2D1E */
  0x0000,   /* U+2D1F */
  0x0000,   /* U+2D20 */
  0x0000,   /* U+2D21 */
  0x0000,   /* U+2D22 */
  0x0000,   /* U+2D23 */
  0x0000,   /* U+2D24 */
  0x0000,   /* U+2D25 */
  0x0000,   /* U+2D26 */
  0x0000,   /* U+2D27 */
  0x0000,   /* U+2D28 */
  0x0000,   /* U+2D29 */
  0x0000,   /* U+2D2A */
  0x0000,   /* U+2D2B */
  0x0000,   /* U+2D2C */
  0x0000,   /* U+2D2D */
  0x0000,   /* U+2D2E */
  0x0000,   /* U+2D2F */
  0x0000,   /* U+2D30 */
  0x0000,   /* U+2D31 */
  0x0000,   /* U+2D32 */
  0x0000,   /* U+2D33 */
  0x0000,   /* U+2D34 */
  0x0000,   /* U+2D35 */
  0x0000,   /* U+2D36 */
  0x0000,   /* U+2D37 */
  0x0000,   /* U+2D38 */
  0x0000,   /* U+2D39 */
  0x0000,   /* U+2D3A */
  0x0000,   /* U+2D3B */
  0x0000,   /* U+2D3C */
  0x0000,   /* U+2D3D */
  0x0000,   /* U+2D3E */
  0x0000,   /* U+2D3F */
  0x0000,   /* U+2D40 */
  0x0000,   /* U+2D41 */
  0x0000,   /* U+2D42 */
  0x0000,   /* U+2D43 */
  0x0000,   /* U+2D44 */
  0x0000,   /* U+2D45 */
  0x0000,   /* U+2D46 */
  0x0000,   /* U+2D47 */
  0x0000,   /* U+2D48 */
  0x0000,   /* U+2D49 */
  0x0000,   /* U+2D4A */
  0x0000,   /* U+2D4B */
  0x0000,   /* U+2D4C */
  0x0000,   /* U+2D4D */
  0x0000,   /* U+2D4E */
  0x0000,   /* U+2D4F */
  0x0000,   /* U+2D50 */
  0x0000,   /* U+2D51 */
  0x0000,   /* U+2D52 */
  0x0000,   /* U+2D53 */
  0x0000,   /* U+2D54 */
  0x0000,   /* U+2D55 */
  0x0000,   /* U+2D56 */
  0x0000,   /* U+2D57 */
  0x0000,   /* U+2D58 */
  0x0000,   /* U+2D59 */
  0x0000,   /* U+2D5A */
  0x0000,   /* U+2D5B */
  0x0000,   /* U+2D5C */
  0x0000,   /* U+2D5D */
  0x0000,   /* U+2D5E */
  0x0000,   /* U+2D5F */
  0x0000,   /* U+2D60 */
  0x0000,   /* U+2D61 */
  0x0000,   /* U+2D62 */
  0x0000,   /* U+2D63 */
  0x0000,   /* U+2D64 */
  0x0000,   /* U+2D65 */
  0x0000,   /* U+2D66 */
  0x0000,   /* U+2D67 */
  0x0000,   /* U+2D68 */
  0x0000,   /* U+2D69 */
  0x0000,   /* U+2D6A */
  0x0000,   /* U+2D6B */
  0x0000,   /* U+2D6C */
  0x0000,   /* U+2D6D */
  0x0000,   /* U+2D6E */
  0x0000,   /* U+2D6F */
  0x0000,   /* U+2D70 */
  0x0000,   /* U+2D71 */
  0x0000,   /* U+2D72 */
  0x0000,   /* U+2D73 */
  0x0000,   /* U+2D74 */
  0x0000,   /* U+2D75 */
  0x0000,   /* U+2D76 */
  0x0000,   /* U+2D77 */
  0x0000,   /* U+2D78 */
  0x0000,   /* U+2D79 */
  0x0000,   /* U+2D7A */
  0x0000,   /* U+2D7B */
  0x0000,   /* U+2D7C */
  0x0000,   /* U+2D7D */
  0x0000,   /* U+2D7E */
  0x0000,   /* U+2D7F */
  0x0000,   /* U+2D80 */
  0x0000,   /* U+2D81 */
  0x0000,   /* U+2D82 */
  0x0000,   /* U+2D83 */
  0x0000,   /* U+2D84 */
  0x0000,   /* U+2D85 */
  0x0000,   /* U+2D86 */
  0x0000,   /* U+2D87 */
  0x0000,   /* U+2D88 */
  0x0000,   /* U+2D89 */
  0x0000,   /* U+2D8A */
  0x0000,   /* U+2D8B */
  0x0000,   /* U+2D8C */
  0x0000,   /* U+2D8D */
  0x0000,   /* U+2D8E */
  0x0000,   /* U+2D8F */
  0x0000,   /* U+2D90 */
  0x0000,   /* U+2D91 */
  0x0000,   /* U+2D92 */
  0x0000,   /* U+2D93 */
  0x0000,   /* U+2D94 */
  0x0000,   /* U+2D95 */
  0x0000,   /* U+2D96 */
  0x0000,   /* U+2D97 */
  0x0000,   /* U+2D98 */
  0x0000,   /* U+2D99 */
  0x0000,   /* U+2D9A */
  0x0000,   /* U+2D9B */
  0x0000,   /* U+2D9C */
  0x0000,   /* U+2D9D */
  0x0000,   /* U+2D9E */
  0x0000,   /* U+2D9F */
  0x0000,   /* U+2DA0 */
  0x0000,   /* U+2DA1 */
  0x0000,   /* U+2DA2 */
  0x0000,   /* U+2DA3 */
  0x0000,   /* U+2DA4 */
  0x0000,   /* U+2DA5 */
  0x0000,   /* U+2DA6 */
  0x0000,   /* U+2DA7 */
  0x0000,   /* U+2DA8 */
  0x0000,   /* U+2DA9 */
  0x0000,   /* U+2DAA */
  0x0000,   /* U+2DAB */
  0x0000,   /* U+2DAC */
  0x0000,   /* U+2DAD */
  0x0000,   /* U+2DAE */
  0x0000,   /* U+2DAF */
  0x0000,   /* U+2DB0 */
  0x0000,   /* U+2DB1 */
  0x0000,   /* U+2DB2 */
  0x0000,   /* U+2DB3 */
  0x0000,   /* U+2DB4 */
  0x0000,   /* U+2DB5 */
  0x0000,   /* U+2DB6 */
  0x0000,   /* U+2DB7 */
  0x0000,   /* U+2DB8 */
  0x0000,   /* U+2DB9 */
  0x0000,   /* U+2DBA */
  0x0000,   /* U+2DBB */
  0x0000,   /* U+2DBC */
  0x0000,   /* U+2DBD */
  0x0000,   /* U+2DBE */
  0x0000,   /* U+2DBF */
  0x0000,   /* U+2DC0 */
  0x0000,   /* U+2DC1 */
  0x0000,   /* U+2DC2 */
  0x0000,   /* U+2DC3 */
  0x0000,   /* U+2DC4 */
  0x0000,   /* U+2DC5 */
  0x0000,   /* U+2DC6 */
  0x0000,   /* U+2DC7 */
  0x0000,   /* U+2DC8 */
  0x0000,   /* U+2DC9 */
  0x0000,   /* U+2DCA */
  0x0000,   /* U+2DCB */
  0x0000,   /* U+2DCC */
  0x0000,   /* U+2DCD */
  0x0000,   /* U+2DCE */
  0x0000,   /* U+2DCF */
  0x0000,   /* U+2DD0 */
  0x0000,   /* U+2DD1 */
  0x0000,   /* U+2DD2 */
  0x0000,   /* U+2DD3 */
  0x0000,   /* U+2DD4 */
  0x0000,   /* U+2DD5 */
  0x0000,   /* U+2DD6 */
  0x0000,   /* U+2DD7 */
  0x0000,   /* U+2DD8 */
  0x0000,   /* U+2DD9 */
  0x0000,   /* U+2DDA */
  0x0000,   /* U+2DDB */
  0x0000,   /* U+2DDC */
  0x0000,   /* U+2DDD */
  0x0000,   /* U+2DDE */
  0x0000,   /* U+2DDF */
  0x0000,   /* U+2DE0 */
  0x0000,   /* U+2DE1 */
  0x0000,   /* U+2DE2 */
  0x0000,   /* U+2DE3 */
  0x0000,   /* U+2DE4 */
  0x0000,   /* U+2DE5 */
  0x0000,   /* U+2DE6 */
  0x0000,   /* U+2DE7 */
  0x0000,   /* U+2DE8 */
  0x0000,   /* U+2DE9 */
  0x0000,   /* U+2DEA */
  0x0000,   /* U+2DEB */
  0x0000,   /* U+2DEC */
  0x0000,   /* U+2DED */
  0x0000,   /* U+2DEE */
  0x0000,   /* U+2DEF */
  0x0000,   /* U+2DF0 */
  0x0000,   /* U+2DF1 */
  0x0000,   /* U+2DF2 */
  0x0000,   /* U+2DF3 */
  0x0000,   /* U+2DF4 */
  0x0004,   /* U+2DF5 */
  0x0000,   /* U+2DF6 */
  0x0000,   /* U+2DF7 */
  0x0000,   /* U+2DF8 */
  0x0000,   /* U+2DF9 */
  0x0000,   /* U+2DFA */
  0x0000,   /* U+2DFB */
  0x0000,   /* U+2DFC */
  0x0000,   /* U+2DFD */
  0x0000,   /* U+2DFE */
  0x0000,   /* U+2DFF */
};

uint16 uca900_p02E[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 2, 2, 3, 2, 3, 3, 3, 2, 3, 3, 3, 3, 3, 2, 
  2, 3, 2, 2, 2, 3, 2, 3, 2, 2, 2, 2, 3, 3, 3, 2, 
  2, 2, 2, 2, 2, 3, 2, 3, 2, 3, 3, 3, 3, 2, 3, 2, 
  2, 2, 2, 3, 3, 3, 3, 3, 2, 2, 2, 3, 3, 3, 2, 3, 
  3, 2, 2, 2, 2, 2, 2, 3, 2, 2, 3, 2, 2, 3, 3, 3, 
  2, 2, 2, 2, 2, 3, 2, 3, 2, 2, 2, 2, 2, 2, 3, 2, 
  2, 3, 2, 3, 3, 2, 2, 3, 2, 2, 2, 3, 2, 3, 2, 3, 
  2, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x03B6,   /* U+2E00 */
  0x03B7,   /* U+2E01 */
  0x035B,   /* U+2E02 */
  0x035C,   /* U+2E03 */
  0x035D,   /* U+2E04 */
  0x035E,   /* U+2E05 */
  0x03B8,   /* U+2E06 */
  0x03B9,   /* U+2E07 */
  0x03BA,   /* U+2E08 */
  0x035F,   /* U+2E09 */
  0x0360,   /* U+2E0A */
  0x03BB,   /* U+2E0B */
  0x0361,   /* U+2E0C */
  0x0362,   /* U+2E0D */
  0x03BC,   /* U+2E0E */
  0x03BD,   /* U+2E0F */
  0x03BE,   /* U+2E10 */
  0x03BF,   /* U+2E11 */
  0x03C0,   /* U+2E12 */
  0x03C1,   /* U+2E13 */
  0x03C2,   /* U+2E14 */
  0x03C3,   /* U+2E15 */
  0x03C4,   /* U+2E16 */
  0x021C,   /* U+2E17 */
  0x0276,   /* U+2E18 */
  0x02F8,   /* U+2E19 */
  0x03C5,   /* U+2E1A */
  0x03C6,   /* U+2E1B */
  0x0363,   /* U+2E1C */
  0x0364,   /* U+2E1D */
  0x03C7,   /* U+2E1E */
  0x03C8,   /* U+2E1F */
  0x0365,   /* U+2E20 */
  0x0366,   /* U+2E21 */
  0x0367,   /* U+2E22 */
  0x0368,   /* U+2E23 */
  0x0369,   /* U+2E24 */
  0x036A,   /* U+2E25 */
  0x036B,   /* U+2E26 */
  0x036C,   /* U+2E27 */
  0x036D,   /* U+2E28 */
  0x036E,   /* U+2E29 */
  0x02F1,   /* U+2E2A */
  0x02F2,   /* U+2E2B */
  0x02F3,   /* U+2E2C */
  0x02F4,   /* U+2E2D */
  0x0268,   /* U+2E2E */
  0x218D,   /* U+2E2F */
  0x0283,   /* U+2E30 */
  0x028C,   /* U+2E31 */
  0x0224,   /* U+2E32 */
  0x028D,   /* U+2E33 */
  0x0223,   /* U+2E34 */
  0x0237,   /* U+2E35 */
  0x03A1,   /* U+2E36 */
  0x03A2,   /* U+2E37 */
  0x03A3,   /* U+2E38 */
  0x038A,   /* U+2E39 */
  0x0218,   /* U+2E3A */
  0x0219,   /* U+2E3B */
  0x0284,   /* U+2E3C */
  0x02F5,   /* U+2E3D */
  0x0384,   /* U+2E3E */
  0x038D,   /* U+2E3F */
  0x021D,   /* U+2E40 */
  0x0225,   /* U+2E41 */
  0x0311,   /* U+2E42 */
  0x021B,   /* U+2E43 */
  0x03C9,   /* U+2E44 */
  0xFBC0,   /* U+2E45 */
  0xFBC0,   /* U+2E46 */
  0xFBC0,   /* U+2E47 */
  0xFBC0,   /* U+2E48 */
  0xFBC0,   /* U+2E49 */
  0xFBC0,   /* U+2E4A */
  0xFBC0,   /* U+2E4B */
  0xFBC0,   /* U+2E4C */
  0xFBC0,   /* U+2E4D */
  0xFBC0,   /* U+2E4E */
  0xFBC0,   /* U+2E4F */
  0xFBC0,   /* U+2E50 */
  0xFBC0,   /* U+2E51 */
  0xFBC0,   /* U+2E52 */
  0xFBC0,   /* U+2E53 */
  0xFBC0,   /* U+2E54 */
  0xFBC0,   /* U+2E55 */
  0xFBC0,   /* U+2E56 */
  0xFBC0,   /* U+2E57 */
  0xFBC0,   /* U+2E58 */
  0xFBC0,   /* U+2E59 */
  0xFBC0,   /* U+2E5A */
  0xFBC0,   /* U+2E5B */
  0xFBC0,   /* U+2E5C */
  0xFBC0,   /* U+2E5D */
  0xFBC0,   /* U+2E5E */
  0xFBC0,   /* U+2E5F */
  0xFBC0,   /* U+2E60 */
  0xFBC0,   /* U+2E61 */
  0xFBC0,   /* U+2E62 */
  0xFBC0,   /* U+2E63 */
  0xFBC0,   /* U+2E64 */
  0xFBC0,   /* U+2E65 */
  0xFBC0,   /* U+2E66 */
  0xFBC0,   /* U+2E67 */
  0xFBC0,   /* U+2E68 */
  0xFBC0,   /* U+2E69 */
  0xFBC0,   /* U+2E6A */
  0xFBC0,   /* U+2E6B */
  0xFBC0,   /* U+2E6C */
  0xFBC0,   /* U+2E6D */
  0xFBC0,   /* U+2E6E */
  0xFBC0,   /* U+2E6F */
  0xFBC0,   /* U+2E70 */
  0xFBC0,   /* U+2E71 */
  0xFBC0,   /* U+2E72 */
  0xFBC0,   /* U+2E73 */
  0xFBC0,   /* U+2E74 */
  0xFBC0,   /* U+2E75 */
  0xFBC0,   /* U+2E76 */
  0xFBC0,   /* U+2E77 */
  0xFBC0,   /* U+2E78 */
  0xFBC0,   /* U+2E79 */
  0xFBC0,   /* U+2E7A */
  0xFBC0,   /* U+2E7B */
  0xFBC0,   /* U+2E7C */
  0xFBC0,   /* U+2E7D */
  0xFBC0,   /* U+2E7E */
  0xFBC0,   /* U+2E7F */
  0xFB40,   /* U+2E80 */
  0xFB40,   /* U+2E81 */
  0xFB40,   /* U+2E82 */
  0xFB40,   /* U+2E83 */
  0xFB40,   /* U+2E84 */
  0xFB40,   /* U+2E85 */
  0xFB40,   /* U+2E86 */
  0xFB40,   /* U+2E87 */
  0xFB40,   /* U+2E88 */
  0xFB40,   /* U+2E89 */
  0xFB40,   /* U+2E8A */
  0xFB40,   /* U+2E8B */
  0xFB40,   /* U+2E8C */
  0xFB40,   /* U+2E8D */
  0xFB40,   /* U+2E8E */
  0xFB40,   /* U+2E8F */
  0xFB40,   /* U+2E90 */
  0xFB40,   /* U+2E91 */
  0xFB40,   /* U+2E92 */
  0xFB40,   /* U+2E93 */
  0xFB40,   /* U+2E94 */
  0xFB40,   /* U+2E95 */
  0xFB40,   /* U+2E96 */
  0xFB40,   /* U+2E97 */
  0xFB40,   /* U+2E98 */
  0xFB40,   /* U+2E99 */
  0xFBC0,   /* U+2E9A */
  0xFB40,   /* U+2E9B */
  0xFB40,   /* U+2E9C */
  0xFB40,   /* U+2E9D */
  0xFB40,   /* U+2E9E */
  0xFB40,   /* U+2E9F */
  0xFB40,   /* U+2EA0 */
  0xFB40,   /* U+2EA1 */
  0xFB40,   /* U+2EA2 */
  0xFB40,   /* U+2EA3 */
  0xFB40,   /* U+2EA4 */
  0xFB40,   /* U+2EA5 */
  0xFB40,   /* U+2EA6 */
  0xFB40,   /* U+2EA7 */
  0xFB40,   /* U+2EA8 */
  0xFB40,   /* U+2EA9 */
  0xFB40,   /* U+2EAA */
  0xFB40,   /* U+2EAB */
  0xFB40,   /* U+2EAC */
  0xFB40,   /* U+2EAD */
  0xFB40,   /* U+2EAE */
  0xFB40,   /* U+2EAF */
  0xFB40,   /* U+2EB0 */
  0xFB40,   /* U+2EB1 */
  0xFB40,   /* U+2EB2 */
  0xFB40,   /* U+2EB3 */
  0xFB40,   /* U+2EB4 */
  0xFB40,   /* U+2EB5 */
  0xFB40,   /* U+2EB6 */
  0xFB40,   /* U+2EB7 */
  0xFB40,   /* U+2EB8 */
  0xFB41,   /* U+2EB9 */
  0xFB41,   /* U+2EBA */
  0xFB41,   /* U+2EBB */
  0xFB41,   /* U+2EBC */
  0xFB41,   /* U+2EBD */
  0xFB41,   /* U+2EBE */
  0xFB41,   /* U+2EBF */
  0xFB41,   /* U+2EC0 */
  0xFB41,   /* U+2EC1 */
  0xFB41,   /* U+2EC2 */
  0xFB41,   /* U+2EC3 */
  0xFB41,   /* U+2EC4 */
  0xFB41,   /* U+2EC5 */
  0xFB41,   /* U+2EC6 */
  0xFB41,   /* U+2EC7 */
  0xFB41,   /* U+2EC8 */
  0xFB41,   /* U+2EC9 */
  0xFB41,   /* U+2ECA */
  0xFB41,   /* U+2ECB */
  0xFB41,   /* U+2ECC */
  0xFB41,   /* U+2ECD */
  0xFB41,   /* U+2ECE */
  0xFB41,   /* U+2ECF */
  0xFB41,   /* U+2ED0 */
  0xFB41,   /* U+2ED1 */
  0xFB41,   /* U+2ED2 */
  0xFB41,   /* U+2ED3 */
  0xFB41,   /* U+2ED4 */
  0xFB41,   /* U+2ED5 */
  0xFB41,   /* U+2ED6 */
  0xFB41,   /* U+2ED7 */
  0xFB41,   /* U+2ED8 */
  0xFB41,   /* U+2ED9 */
  0xFB41,   /* U+2EDA */
  0xFB41,   /* U+2EDB */
  0xFB41,   /* U+2EDC */
  0xFB41,   /* U+2EDD */
  0xFB41,   /* U+2EDE */
  0xFB41,   /* U+2EDF */
  0xFB41,   /* U+2EE0 */
  0xFB41,   /* U+2EE1 */
  0xFB41,   /* U+2EE2 */
  0xFB41,   /* U+2EE3 */
  0xFB41,   /* U+2EE4 */
  0xFB41,   /* U+2EE5 */
  0xFB41,   /* U+2EE6 */
  0xFB41,   /* U+2EE7 */
  0xFB41,   /* U+2EE8 */
  0xFB41,   /* U+2EE9 */
  0xFB41,   /* U+2EEA */
  0xFB41,   /* U+2EEB */
  0xFB41,   /* U+2EEC */
  0xFB41,   /* U+2EED */
  0xFB41,   /* U+2EEE */
  0xFB41,   /* U+2EEF */
  0xFB41,   /* U+2EF0 */
  0xFB41,   /* U+2EF1 */
  0xFB41,   /* U+2EF2 */
  0xFB41,   /* U+2EF3 */
  0xFBC0,   /* U+2EF4 */
  0xFBC0,   /* U+2EF5 */
  0xFBC0,   /* U+2EF6 */
  0xFBC0,   /* U+2EF7 */
  0xFBC0,   /* U+2EF8 */
  0xFBC0,   /* U+2EF9 */
  0xFBC0,   /* U+2EFA */
  0xFBC0,   /* U+2EFB */
  0xFBC0,   /* U+2EFC */
  0xFBC0,   /* U+2EFD */
  0xFBC0,   /* U+2EFE */
  0xFBC0,   /* U+2EFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2E00 */
  0x0020,   /* U+2E01 */
  0x0020,   /* U+2E02 */
  0x0020,   /* U+2E03 */
  0x0020,   /* U+2E04 */
  0x0020,   /* U+2E05 */
  0x0020,   /* U+2E06 */
  0x0020,   /* U+2E07 */
  0x0020,   /* U+2E08 */
  0x0020,   /* U+2E09 */
  0x0020,   /* U+2E0A */
  0x0020,   /* U+2E0B */
  0x0020,   /* U+2E0C */
  0x0020,   /* U+2E0D */
  0x0020,   /* U+2E0E */
  0x0020,   /* U+2E0F */
  0x0020,   /* U+2E10 */
  0x0020,   /* U+2E11 */
  0x0020,   /* U+2E12 */
  0x0020,   /* U+2E13 */
  0x0020,   /* U+2E14 */
  0x0020,   /* U+2E15 */
  0x0020,   /* U+2E16 */
  0x0020,   /* U+2E17 */
  0x0020,   /* U+2E18 */
  0x0020,   /* U+2E19 */
  0x0020,   /* U+2E1A */
  0x0020,   /* U+2E1B */
  0x0020,   /* U+2E1C */
  0x0020,   /* U+2E1D */
  0x0020,   /* U+2E1E */
  0x0020,   /* U+2E1F */
  0x0020,   /* U+2E20 */
  0x0020,   /* U+2E21 */
  0x0020,   /* U+2E22 */
  0x0020,   /* U+2E23 */
  0x0020,   /* U+2E24 */
  0x0020,   /* U+2E25 */
  0x0020,   /* U+2E26 */
  0x0020,   /* U+2E27 */
  0x0020,   /* U+2E28 */
  0x0020,   /* U+2E29 */
  0x0020,   /* U+2E2A */
  0x0020,   /* U+2E2B */
  0x0020,   /* U+2E2C */
  0x0020,   /* U+2E2D */
  0x0020,   /* U+2E2E */
  0x0020,   /* U+2E2F */
  0x0020,   /* U+2E30 */
  0x0020,   /* U+2E31 */
  0x0020,   /* U+2E32 */
  0x0020,   /* U+2E33 */
  0x0020,   /* U+2E34 */
  0x0020,   /* U+2E35 */
  0x0020,   /* U+2E36 */
  0x0020,   /* U+2E37 */
  0x0020,   /* U+2E38 */
  0x0020,   /* U+2E39 */
  0x0020,   /* U+2E3A */
  0x0020,   /* U+2E3B */
  0x0020,   /* U+2E3C */
  0x0020,   /* U+2E3D */
  0x0020,   /* U+2E3E */
  0x0020,   /* U+2E3F */
  0x0020,   /* U+2E40 */
  0x0020,   /* U+2E41 */
  0x0020,   /* U+2E42 */
  0x0020,   /* U+2E43 */
  0x0020,   /* U+2E44 */
  0x0020,   /* U+2E45 */
  0x0020,   /* U+2E46 */
  0x0020,   /* U+2E47 */
  0x0020,   /* U+2E48 */
  0x0020,   /* U+2E49 */
  0x0020,   /* U+2E4A */
  0x0020,   /* U+2E4B */
  0x0020,   /* U+2E4C */
  0x0020,   /* U+2E4D */
  0x0020,   /* U+2E4E */
  0x0020,   /* U+2E4F */
  0x0020,   /* U+2E50 */
  0x0020,   /* U+2E51 */
  0x0020,   /* U+2E52 */
  0x0020,   /* U+2E53 */
  0x0020,   /* U+2E54 */
  0x0020,   /* U+2E55 */
  0x0020,   /* U+2E56 */
  0x0020,   /* U+2E57 */
  0x0020,   /* U+2E58 */
  0x0020,   /* U+2E59 */
  0x0020,   /* U+2E5A */
  0x0020,   /* U+2E5B */
  0x0020,   /* U+2E5C */
  0x0020,   /* U+2E5D */
  0x0020,   /* U+2E5E */
  0x0020,   /* U+2E5F */
  0x0020,   /* U+2E60 */
  0x0020,   /* U+2E61 */
  0x0020,   /* U+2E62 */
  0x0020,   /* U+2E63 */
  0x0020,   /* U+2E64 */
  0x0020,   /* U+2E65 */
  0x0020,   /* U+2E66 */
  0x0020,   /* U+2E67 */
  0x0020,   /* U+2E68 */
  0x0020,   /* U+2E69 */
  0x0020,   /* U+2E6A */
  0x0020,   /* U+2E6B */
  0x0020,   /* U+2E6C */
  0x0020,   /* U+2E6D */
  0x0020,   /* U+2E6E */
  0x0020,   /* U+2E6F */
  0x0020,   /* U+2E70 */
  0x0020,   /* U+2E71 */
  0x0020,   /* U+2E72 */
  0x0020,   /* U+2E73 */
  0x0020,   /* U+2E74 */
  0x0020,   /* U+2E75 */
  0x0020,   /* U+2E76 */
  0x0020,   /* U+2E77 */
  0x0020,   /* U+2E78 */
  0x0020,   /* U+2E79 */
  0x0020,   /* U+2E7A */
  0x0020,   /* U+2E7B */
  0x0020,   /* U+2E7C */
  0x0020,   /* U+2E7D */
  0x0020,   /* U+2E7E */
  0x0020,   /* U+2E7F */
  0x0020,   /* U+2E80 */
  0x0020,   /* U+2E81 */
  0x0020,   /* U+2E82 */
  0x0020,   /* U+2E83 */
  0x0020,   /* U+2E84 */
  0x0020,   /* U+2E85 */
  0x0020,   /* U+2E86 */
  0x0020,   /* U+2E87 */
  0x0020,   /* U+2E88 */
  0x0020,   /* U+2E89 */
  0x0020,   /* U+2E8A */
  0x0020,   /* U+2E8B */
  0x0020,   /* U+2E8C */
  0x0020,   /* U+2E8D */
  0x0020,   /* U+2E8E */
  0x0020,   /* U+2E8F */
  0x0020,   /* U+2E90 */
  0x0020,   /* U+2E91 */
  0x0020,   /* U+2E92 */
  0x0020,   /* U+2E93 */
  0x0020,   /* U+2E94 */
  0x0020,   /* U+2E95 */
  0x0020,   /* U+2E96 */
  0x0020,   /* U+2E97 */
  0x0020,   /* U+2E98 */
  0x0020,   /* U+2E99 */
  0x0020,   /* U+2E9A */
  0x0020,   /* U+2E9B */
  0x0020,   /* U+2E9C */
  0x0020,   /* U+2E9D */
  0x0020,   /* U+2E9E */
  0x0020,   /* U+2E9F */
  0x0020,   /* U+2EA0 */
  0x0020,   /* U+2EA1 */
  0x0020,   /* U+2EA2 */
  0x0020,   /* U+2EA3 */
  0x0020,   /* U+2EA4 */
  0x0020,   /* U+2EA5 */
  0x0020,   /* U+2EA6 */
  0x0020,   /* U+2EA7 */
  0x0020,   /* U+2EA8 */
  0x0020,   /* U+2EA9 */
  0x0020,   /* U+2EAA */
  0x0020,   /* U+2EAB */
  0x0020,   /* U+2EAC */
  0x0020,   /* U+2EAD */
  0x0020,   /* U+2EAE */
  0x0020,   /* U+2EAF */
  0x0020,   /* U+2EB0 */
  0x0020,   /* U+2EB1 */
  0x0020,   /* U+2EB2 */
  0x0020,   /* U+2EB3 */
  0x0020,   /* U+2EB4 */
  0x0020,   /* U+2EB5 */
  0x0020,   /* U+2EB6 */
  0x0020,   /* U+2EB7 */
  0x0020,   /* U+2EB8 */
  0x0020,   /* U+2EB9 */
  0x0020,   /* U+2EBA */
  0x0020,   /* U+2EBB */
  0x0020,   /* U+2EBC */
  0x0020,   /* U+2EBD */
  0x0020,   /* U+2EBE */
  0x0020,   /* U+2EBF */
  0x0020,   /* U+2EC0 */
  0x0020,   /* U+2EC1 */
  0x0020,   /* U+2EC2 */
  0x0020,   /* U+2EC3 */
  0x0020,   /* U+2EC4 */
  0x0020,   /* U+2EC5 */
  0x0020,   /* U+2EC6 */
  0x0020,   /* U+2EC7 */
  0x0020,   /* U+2EC8 */
  0x0020,   /* U+2EC9 */
  0x0020,   /* U+2ECA */
  0x0020,   /* U+2ECB */
  0x0020,   /* U+2ECC */
  0x0020,   /* U+2ECD */
  0x0020,   /* U+2ECE */
  0x0020,   /* U+2ECF */
  0x0020,   /* U+2ED0 */
  0x0020,   /* U+2ED1 */
  0x0020,   /* U+2ED2 */
  0x0020,   /* U+2ED3 */
  0x0020,   /* U+2ED4 */
  0x0020,   /* U+2ED5 */
  0x0020,   /* U+2ED6 */
  0x0020,   /* U+2ED7 */
  0x0020,   /* U+2ED8 */
  0x0020,   /* U+2ED9 */
  0x0020,   /* U+2EDA */
  0x0020,   /* U+2EDB */
  0x0020,   /* U+2EDC */
  0x0020,   /* U+2EDD */
  0x0020,   /* U+2EDE */
  0x0020,   /* U+2EDF */
  0x0020,   /* U+2EE0 */
  0x0020,   /* U+2EE1 */
  0x0020,   /* U+2EE2 */
  0x0020,   /* U+2EE3 */
  0x0020,   /* U+2EE4 */
  0x0020,   /* U+2EE5 */
  0x0020,   /* U+2EE6 */
  0x0020,   /* U+2EE7 */
  0x0020,   /* U+2EE8 */
  0x0020,   /* U+2EE9 */
  0x0020,   /* U+2EEA */
  0x0020,   /* U+2EEB */
  0x0020,   /* U+2EEC */
  0x0020,   /* U+2EED */
  0x0020,   /* U+2EEE */
  0x0020,   /* U+2EEF */
  0x0020,   /* U+2EF0 */
  0x0020,   /* U+2EF1 */
  0x0020,   /* U+2EF2 */
  0x0020,   /* U+2EF3 */
  0x0020,   /* U+2EF4 */
  0x0020,   /* U+2EF5 */
  0x0020,   /* U+2EF6 */
  0x0020,   /* U+2EF7 */
  0x0020,   /* U+2EF8 */
  0x0020,   /* U+2EF9 */
  0x0020,   /* U+2EFA */
  0x0020,   /* U+2EFB */
  0x0020,   /* U+2EFC */
  0x0020,   /* U+2EFD */
  0x0020,   /* U+2EFE */
  0x0020,   /* U+2EFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2E00 */
  0x0002,   /* U+2E01 */
  0x0002,   /* U+2E02 */
  0x0002,   /* U+2E03 */
  0x0002,   /* U+2E04 */
  0x0002,   /* U+2E05 */
  0x0002,   /* U+2E06 */
  0x0002,   /* U+2E07 */
  0x0002,   /* U+2E08 */
  0x0002,   /* U+2E09 */
  0x0002,   /* U+2E0A */
  0x0002,   /* U+2E0B */
  0x0002,   /* U+2E0C */
  0x0002,   /* U+2E0D */
  0x0002,   /* U+2E0E */
  0x0002,   /* U+2E0F */
  0x0002,   /* U+2E10 */
  0x0002,   /* U+2E11 */
  0x0002,   /* U+2E12 */
  0x0002,   /* U+2E13 */
  0x0002,   /* U+2E14 */
  0x0002,   /* U+2E15 */
  0x0002,   /* U+2E16 */
  0x0002,   /* U+2E17 */
  0x0002,   /* U+2E18 */
  0x0002,   /* U+2E19 */
  0x0002,   /* U+2E1A */
  0x0002,   /* U+2E1B */
  0x0002,   /* U+2E1C */
  0x0002,   /* U+2E1D */
  0x0002,   /* U+2E1E */
  0x0002,   /* U+2E1F */
  0x0002,   /* U+2E20 */
  0x0002,   /* U+2E21 */
  0x0002,   /* U+2E22 */
  0x0002,   /* U+2E23 */
  0x0002,   /* U+2E24 */
  0x0002,   /* U+2E25 */
  0x0002,   /* U+2E26 */
  0x0002,   /* U+2E27 */
  0x0002,   /* U+2E28 */
  0x0002,   /* U+2E29 */
  0x0002,   /* U+2E2A */
  0x0002,   /* U+2E2B */
  0x0002,   /* U+2E2C */
  0x0002,   /* U+2E2D */
  0x0002,   /* U+2E2E */
  0x0002,   /* U+2E2F */
  0x0002,   /* U+2E30 */
  0x0002,   /* U+2E31 */
  0x0002,   /* U+2E32 */
  0x0002,   /* U+2E33 */
  0x0002,   /* U+2E34 */
  0x0002,   /* U+2E35 */
  0x0002,   /* U+2E36 */
  0x0002,   /* U+2E37 */
  0x0002,   /* U+2E38 */
  0x0002,   /* U+2E39 */
  0x0002,   /* U+2E3A */
  0x0002,   /* U+2E3B */
  0x0002,   /* U+2E3C */
  0x0002,   /* U+2E3D */
  0x0002,   /* U+2E3E */
  0x0002,   /* U+2E3F */
  0x0002,   /* U+2E40 */
  0x0002,   /* U+2E41 */
  0x0002,   /* U+2E42 */
  0x0002,   /* U+2E43 */
  0x0002,   /* U+2E44 */
  0x0002,   /* U+2E45 */
  0x0002,   /* U+2E46 */
  0x0002,   /* U+2E47 */
  0x0002,   /* U+2E48 */
  0x0002,   /* U+2E49 */
  0x0002,   /* U+2E4A */
  0x0002,   /* U+2E4B */
  0x0002,   /* U+2E4C */
  0x0002,   /* U+2E4D */
  0x0002,   /* U+2E4E */
  0x0002,   /* U+2E4F */
  0x0002,   /* U+2E50 */
  0x0002,   /* U+2E51 */
  0x0002,   /* U+2E52 */
  0x0002,   /* U+2E53 */
  0x0002,   /* U+2E54 */
  0x0002,   /* U+2E55 */
  0x0002,   /* U+2E56 */
  0x0002,   /* U+2E57 */
  0x0002,   /* U+2E58 */
  0x0002,   /* U+2E59 */
  0x0002,   /* U+2E5A */
  0x0002,   /* U+2E5B */
  0x0002,   /* U+2E5C */
  0x0002,   /* U+2E5D */
  0x0002,   /* U+2E5E */
  0x0002,   /* U+2E5F */
  0x0002,   /* U+2E60 */
  0x0002,   /* U+2E61 */
  0x0002,   /* U+2E62 */
  0x0002,   /* U+2E63 */
  0x0002,   /* U+2E64 */
  0x0002,   /* U+2E65 */
  0x0002,   /* U+2E66 */
  0x0002,   /* U+2E67 */
  0x0002,   /* U+2E68 */
  0x0002,   /* U+2E69 */
  0x0002,   /* U+2E6A */
  0x0002,   /* U+2E6B */
  0x0002,   /* U+2E6C */
  0x0002,   /* U+2E6D */
  0x0002,   /* U+2E6E */
  0x0002,   /* U+2E6F */
  0x0002,   /* U+2E70 */
  0x0002,   /* U+2E71 */
  0x0002,   /* U+2E72 */
  0x0002,   /* U+2E73 */
  0x0002,   /* U+2E74 */
  0x0002,   /* U+2E75 */
  0x0002,   /* U+2E76 */
  0x0002,   /* U+2E77 */
  0x0002,   /* U+2E78 */
  0x0002,   /* U+2E79 */
  0x0002,   /* U+2E7A */
  0x0002,   /* U+2E7B */
  0x0002,   /* U+2E7C */
  0x0002,   /* U+2E7D */
  0x0002,   /* U+2E7E */
  0x0002,   /* U+2E7F */
  0x0004,   /* U+2E80 */
  0x0004,   /* U+2E81 */
  0x0004,   /* U+2E82 */
  0x0004,   /* U+2E83 */
  0x0004,   /* U+2E84 */
  0x0004,   /* U+2E85 */
  0x0004,   /* U+2E86 */
  0x0004,   /* U+2E87 */
  0x0004,   /* U+2E88 */
  0x0004,   /* U+2E89 */
  0x0004,   /* U+2E8A */
  0x0004,   /* U+2E8B */
  0x0004,   /* U+2E8C */
  0x0004,   /* U+2E8D */
  0x0004,   /* U+2E8E */
  0x0004,   /* U+2E8F */
  0x0004,   /* U+2E90 */
  0x0004,   /* U+2E91 */
  0x0004,   /* U+2E92 */
  0x0004,   /* U+2E93 */
  0x0004,   /* U+2E94 */
  0x0004,   /* U+2E95 */
  0x0004,   /* U+2E96 */
  0x0004,   /* U+2E97 */
  0x0004,   /* U+2E98 */
  0x0004,   /* U+2E99 */
  0x0002,   /* U+2E9A */
  0x0004,   /* U+2E9B */
  0x0004,   /* U+2E9C */
  0x0004,   /* U+2E9D */
  0x0004,   /* U+2E9E */
  0x0004,   /* U+2E9F */
  0x0004,   /* U+2EA0 */
  0x0004,   /* U+2EA1 */
  0x0004,   /* U+2EA2 */
  0x0004,   /* U+2EA3 */
  0x0004,   /* U+2EA4 */
  0x0004,   /* U+2EA5 */
  0x0004,   /* U+2EA6 */
  0x0004,   /* U+2EA7 */
  0x0004,   /* U+2EA8 */
  0x0004,   /* U+2EA9 */
  0x0004,   /* U+2EAA */
  0x0004,   /* U+2EAB */
  0x0004,   /* U+2EAC */
  0x0004,   /* U+2EAD */
  0x0004,   /* U+2EAE */
  0x0004,   /* U+2EAF */
  0x0004,   /* U+2EB0 */
  0x0004,   /* U+2EB1 */
  0x0004,   /* U+2EB2 */
  0x0004,   /* U+2EB3 */
  0x0004,   /* U+2EB4 */
  0x0004,   /* U+2EB5 */
  0x0004,   /* U+2EB6 */
  0x0004,   /* U+2EB7 */
  0x0004,   /* U+2EB8 */
  0x0004,   /* U+2EB9 */
  0x0004,   /* U+2EBA */
  0x0004,   /* U+2EBB */
  0x0004,   /* U+2EBC */
  0x0004,   /* U+2EBD */
  0x0004,   /* U+2EBE */
  0x0004,   /* U+2EBF */
  0x0004,   /* U+2EC0 */
  0x0004,   /* U+2EC1 */
  0x0004,   /* U+2EC2 */
  0x0004,   /* U+2EC3 */
  0x0004,   /* U+2EC4 */
  0x0004,   /* U+2EC5 */
  0x0004,   /* U+2EC6 */
  0x0004,   /* U+2EC7 */
  0x0004,   /* U+2EC8 */
  0x0004,   /* U+2EC9 */
  0x0004,   /* U+2ECA */
  0x0004,   /* U+2ECB */
  0x0004,   /* U+2ECC */
  0x0004,   /* U+2ECD */
  0x0004,   /* U+2ECE */
  0x0004,   /* U+2ECF */
  0x0004,   /* U+2ED0 */
  0x0004,   /* U+2ED1 */
  0x0004,   /* U+2ED2 */
  0x0004,   /* U+2ED3 */
  0x0004,   /* U+2ED4 */
  0x0004,   /* U+2ED5 */
  0x0004,   /* U+2ED6 */
  0x0004,   /* U+2ED7 */
  0x0004,   /* U+2ED8 */
  0x0004,   /* U+2ED9 */
  0x0004,   /* U+2EDA */
  0x0004,   /* U+2EDB */
  0x0004,   /* U+2EDC */
  0x0004,   /* U+2EDD */
  0x0004,   /* U+2EDE */
  0x0004,   /* U+2EDF */
  0x0004,   /* U+2EE0 */
  0x0004,   /* U+2EE1 */
  0x0004,   /* U+2EE2 */
  0x0004,   /* U+2EE3 */
  0x0004,   /* U+2EE4 */
  0x0004,   /* U+2EE5 */
  0x0004,   /* U+2EE6 */
  0x0004,   /* U+2EE7 */
  0x0004,   /* U+2EE8 */
  0x0004,   /* U+2EE9 */
  0x0004,   /* U+2EEA */
  0x0004,   /* U+2EEB */
  0x0004,   /* U+2EEC */
  0x0004,   /* U+2EED */
  0x0004,   /* U+2EEE */
  0x0004,   /* U+2EEF */
  0x0004,   /* U+2EF0 */
  0x0004,   /* U+2EF1 */
  0x0004,   /* U+2EF2 */
  0x0004,   /* U+2EF3 */
  0x0002,   /* U+2EF4 */
  0x0002,   /* U+2EF5 */
  0x0002,   /* U+2EF6 */
  0x0002,   /* U+2EF7 */
  0x0002,   /* U+2EF8 */
  0x0002,   /* U+2EF9 */
  0x0002,   /* U+2EFA */
  0x0002,   /* U+2EFB */
  0x0002,   /* U+2EFC */
  0x0002,   /* U+2EFD */
  0x0002,   /* U+2EFE */
  0x0002,   /* U+2EFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0xAE45,   /* U+2E45 */
  0xAE46,   /* U+2E46 */
  0xAE47,   /* U+2E47 */
  0xAE48,   /* U+2E48 */
  0xAE49,   /* U+2E49 */
  0xAE4A,   /* U+2E4A */
  0xAE4B,   /* U+2E4B */
  0xAE4C,   /* U+2E4C */
  0xAE4D,   /* U+2E4D */
  0xAE4E,   /* U+2E4E */
  0xAE4F,   /* U+2E4F */
  0xAE50,   /* U+2E50 */
  0xAE51,   /* U+2E51 */
  0xAE52,   /* U+2E52 */
  0xAE53,   /* U+2E53 */
  0xAE54,   /* U+2E54 */
  0xAE55,   /* U+2E55 */
  0xAE56,   /* U+2E56 */
  0xAE57,   /* U+2E57 */
  0xAE58,   /* U+2E58 */
  0xAE59,   /* U+2E59 */
  0xAE5A,   /* U+2E5A */
  0xAE5B,   /* U+2E5B */
  0xAE5C,   /* U+2E5C */
  0xAE5D,   /* U+2E5D */
  0xAE5E,   /* U+2E5E */
  0xAE5F,   /* U+2E5F */
  0xAE60,   /* U+2E60 */
  0xAE61,   /* U+2E61 */
  0xAE62,   /* U+2E62 */
  0xAE63,   /* U+2E63 */
  0xAE64,   /* U+2E64 */
  0xAE65,   /* U+2E65 */
  0xAE66,   /* U+2E66 */
  0xAE67,   /* U+2E67 */
  0xAE68,   /* U+2E68 */
  0xAE69,   /* U+2E69 */
  0xAE6A,   /* U+2E6A */
  0xAE6B,   /* U+2E6B */
  0xAE6C,   /* U+2E6C */
  0xAE6D,   /* U+2E6D */
  0xAE6E,   /* U+2E6E */
  0xAE6F,   /* U+2E6F */
  0xAE70,   /* U+2E70 */
  0xAE71,   /* U+2E71 */
  0xAE72,   /* U+2E72 */
  0xAE73,   /* U+2E73 */
  0xAE74,   /* U+2E74 */
  0xAE75,   /* U+2E75 */
  0xAE76,   /* U+2E76 */
  0xAE77,   /* U+2E77 */
  0xAE78,   /* U+2E78 */
  0xAE79,   /* U+2E79 */
  0xAE7A,   /* U+2E7A */
  0xAE7B,   /* U+2E7B */
  0xAE7C,   /* U+2E7C */
  0xAE7D,   /* U+2E7D */
  0xAE7E,   /* U+2E7E */
  0xAE7F,   /* U+2E7F */
  0xCE36,   /* U+2E80 */
  0xD382,   /* U+2E81 */
  0xCE5B,   /* U+2E82 */
  0xCE5A,   /* U+2E83 */
  0xCE59,   /* U+2E84 */
  0xCEBB,   /* U+2E85 */
  0xD182,   /* U+2E86 */
  0xD1E0,   /* U+2E87 */
  0xD200,   /* U+2E88 */
  0xD202,   /* U+2E89 */
  0xD35C,   /* U+2E8A */
  0xD369,   /* U+2E8B */
  0xDC0F,   /* U+2E8C */
  0xDC0F,   /* U+2E8D */
  0xDC22,   /* U+2E8E */
  0xDC23,   /* U+2E8F */
  0xDC22,   /* U+2E90 */
  0xDC23,   /* U+2E91 */
  0xDDF3,   /* U+2E92 */
  0xDE7A,   /* U+2E93 */
  0xDF51,   /* U+2E94 */
  0xDF50,   /* U+2E95 */
  0xDFC4,   /* U+2E96 */
  0xDFC3,   /* U+2E97 */
  0xE24C,   /* U+2E98 */
  0xE535,   /* U+2E99 */
  0xAE9A,   /* U+2E9A */
  0xE5E1,   /* U+2E9B */
  0xE5E5,   /* U+2E9C */
  0xE708,   /* U+2E9D */
  0xEB7A,   /* U+2E9E */
  0xEBCD,   /* U+2E9F */
  0xEC11,   /* U+2EA0 */
  0xEC35,   /* U+2EA1 */
  0xEC3A,   /* U+2EA2 */
  0xF06C,   /* U+2EA3 */
  0xF22B,   /* U+2EA4 */
  0xF22B,   /* U+2EA5 */
  0xCE2C,   /* U+2EA6 */
  0xF25B,   /* U+2EA7 */
  0xF2AD,   /* U+2EA8 */
  0xF38B,   /* U+2EA9 */
  0xF58B,   /* U+2EAA */
  0xF6EE,   /* U+2EAB */
  0xF93A,   /* U+2EAC */
  0xF93B,   /* U+2EAD */
  0xFAF9,   /* U+2EAE */
  0xFCF9,   /* U+2EAF */
  0xFE9F,   /* U+2EB0 */
  0xFF53,   /* U+2EB1 */
  0xFF52,   /* U+2EB2 */
  0xFF53,   /* U+2EB3 */
  0xFF53,   /* U+2EB4 */
  0xFF52,   /* U+2EB5 */
  0xFF8A,   /* U+2EB6 */
  0xFF8A,   /* U+2EB7 */
  0xFF8B,   /* U+2EB8 */
  0x8002,   /* U+2EB9 */
  0x8080,   /* U+2EBA */
  0x807F,   /* U+2EBB */
  0x8089,   /* U+2EBC */
  0x81FC,   /* U+2EBD */
  0x8279,   /* U+2EBE */
  0x8279,   /* U+2EBF */
  0x8279,   /* U+2EC0 */
  0x864E,   /* U+2EC1 */
  0x8864,   /* U+2EC2 */
  0x8980,   /* U+2EC3 */
  0x897F,   /* U+2EC4 */
  0x89C1,   /* U+2EC5 */
  0x89D2,   /* U+2EC6 */
  0x89D2,   /* U+2EC7 */
  0x8BA0,   /* U+2EC8 */
  0x8D1D,   /* U+2EC9 */
  0x8DB3,   /* U+2ECA */
  0x8F66,   /* U+2ECB */
  0x8FB6,   /* U+2ECC */
  0x8FB6,   /* U+2ECD */
  0x8FB6,   /* U+2ECE */
  0x9091,   /* U+2ECF */
  0x9485,   /* U+2ED0 */
  0x9577,   /* U+2ED1 */
  0x9578,   /* U+2ED2 */
  0x957F,   /* U+2ED3 */
  0x95E8,   /* U+2ED4 */
  0x961C,   /* U+2ED5 */
  0x961D,   /* U+2ED6 */
  0x96E8,   /* U+2ED7 */
  0x9752,   /* U+2ED8 */
  0x97E6,   /* U+2ED9 */
  0x9875,   /* U+2EDA */
  0x98CE,   /* U+2EDB */
  0x98DE,   /* U+2EDC */
  0x98DF,   /* U+2EDD */
  0x98E0,   /* U+2EDE */
  0x98E0,   /* U+2EDF */
  0x9963,   /* U+2EE0 */
  0x9996,   /* U+2EE1 */
  0x9A6C,   /* U+2EE2 */
  0x9AA8,   /* U+2EE3 */
  0x9B3C,   /* U+2EE4 */
  0x9C7C,   /* U+2EE5 */
  0x9E1F,   /* U+2EE6 */
  0x9E75,   /* U+2EE7 */
  0x9EA6,   /* U+2EE8 */
  0x9EC4,   /* U+2EE9 */
  0x9EFE,   /* U+2EEA */
  0x9F4A,   /* U+2EEB */
  0x9F50,   /* U+2EEC */
  0x9F52,   /* U+2EED */
  0x9F7F,   /* U+2EEE */
  0x9F8D,   /* U+2EEF */
  0x9F99,   /* U+2EF0 */
  0x9F9C,   /* U+2EF1 */
  0x9F9C,   /* U+2EF2 */
  0x9F9F,   /* U+2EF3 */
  0xAEF4,   /* U+2EF4 */
  0xAEF5,   /* U+2EF5 */
  0xAEF6,   /* U+2EF6 */
  0xAEF7,   /* U+2EF7 */
  0xAEF8,   /* U+2EF8 */
  0xAEF9,   /* U+2EF9 */
  0xAEFA,   /* U+2EFA */
  0xAEFB,   /* U+2EFB */
  0xAEFC,   /* U+2EFC */
  0xAEFD,   /* U+2EFD */
  0xAEFE,   /* U+2EFE */
  0xAEFF,   /* U+2EFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0x0000,   /* U+2E45 */
  0x0000,   /* U+2E46 */
  0x0000,   /* U+2E47 */
  0x0000,   /* U+2E48 */
  0x0000,   /* U+2E49 */
  0x0000,   /* U+2E4A */
  0x0000,   /* U+2E4B */
  0x0000,   /* U+2E4C */
  0x0000,   /* U+2E4D */
  0x0000,   /* U+2E4E */
  0x0000,   /* U+2E4F */
  0x0000,   /* U+2E50 */
  0x0000,   /* U+2E51 */
  0x0000,   /* U+2E52 */
  0x0000,   /* U+2E53 */
  0x0000,   /* U+2E54 */
  0x0000,   /* U+2E55 */
  0x0000,   /* U+2E56 */
  0x0000,   /* U+2E57 */
  0x0000,   /* U+2E58 */
  0x0000,   /* U+2E59 */
  0x0000,   /* U+2E5A */
  0x0000,   /* U+2E5B */
  0x0000,   /* U+2E5C */
  0x0000,   /* U+2E5D */
  0x0000,   /* U+2E5E */
  0x0000,   /* U+2E5F */
  0x0000,   /* U+2E60 */
  0x0000,   /* U+2E61 */
  0x0000,   /* U+2E62 */
  0x0000,   /* U+2E63 */
  0x0000,   /* U+2E64 */
  0x0000,   /* U+2E65 */
  0x0000,   /* U+2E66 */
  0x0000,   /* U+2E67 */
  0x0000,   /* U+2E68 */
  0x0000,   /* U+2E69 */
  0x0000,   /* U+2E6A */
  0x0000,   /* U+2E6B */
  0x0000,   /* U+2E6C */
  0x0000,   /* U+2E6D */
  0x0000,   /* U+2E6E */
  0x0000,   /* U+2E6F */
  0x0000,   /* U+2E70 */
  0x0000,   /* U+2E71 */
  0x0000,   /* U+2E72 */
  0x0000,   /* U+2E73 */
  0x0000,   /* U+2E74 */
  0x0000,   /* U+2E75 */
  0x0000,   /* U+2E76 */
  0x0000,   /* U+2E77 */
  0x0000,   /* U+2E78 */
  0x0000,   /* U+2E79 */
  0x0000,   /* U+2E7A */
  0x0000,   /* U+2E7B */
  0x0000,   /* U+2E7C */
  0x0000,   /* U+2E7D */
  0x0000,   /* U+2E7E */
  0x0000,   /* U+2E7F */
  0x0000,   /* U+2E80 */
  0x0000,   /* U+2E81 */
  0x0000,   /* U+2E82 */
  0x0000,   /* U+2E83 */
  0x0000,   /* U+2E84 */
  0x0000,   /* U+2E85 */
  0x0000,   /* U+2E86 */
  0x0000,   /* U+2E87 */
  0x0000,   /* U+2E88 */
  0x0000,   /* U+2E89 */
  0x0000,   /* U+2E8A */
  0x0000,   /* U+2E8B */
  0x0000,   /* U+2E8C */
  0x0000,   /* U+2E8D */
  0x0000,   /* U+2E8E */
  0x0000,   /* U+2E8F */
  0x0000,   /* U+2E90 */
  0x0000,   /* U+2E91 */
  0x0000,   /* U+2E92 */
  0x0000,   /* U+2E93 */
  0x0000,   /* U+2E94 */
  0x0000,   /* U+2E95 */
  0x0000,   /* U+2E96 */
  0x0000,   /* U+2E97 */
  0x0000,   /* U+2E98 */
  0x0000,   /* U+2E99 */
  0x0000,   /* U+2E9A */
  0x0000,   /* U+2E9B */
  0x0000,   /* U+2E9C */
  0x0000,   /* U+2E9D */
  0x0000,   /* U+2E9E */
  0x0000,   /* U+2E9F */
  0x0000,   /* U+2EA0 */
  0x0000,   /* U+2EA1 */
  0x0000,   /* U+2EA2 */
  0x0000,   /* U+2EA3 */
  0x0000,   /* U+2EA4 */
  0x0000,   /* U+2EA5 */
  0x0000,   /* U+2EA6 */
  0x0000,   /* U+2EA7 */
  0x0000,   /* U+2EA8 */
  0x0000,   /* U+2EA9 */
  0x0000,   /* U+2EAA */
  0x0000,   /* U+2EAB */
  0x0000,   /* U+2EAC */
  0x0000,   /* U+2EAD */
  0x0000,   /* U+2EAE */
  0x0000,   /* U+2EAF */
  0x0000,   /* U+2EB0 */
  0x0000,   /* U+2EB1 */
  0x0000,   /* U+2EB2 */
  0x0000,   /* U+2EB3 */
  0x0000,   /* U+2EB4 */
  0x0000,   /* U+2EB5 */
  0x0000,   /* U+2EB6 */
  0x0000,   /* U+2EB7 */
  0x0000,   /* U+2EB8 */
  0x0000,   /* U+2EB9 */
  0x0000,   /* U+2EBA */
  0x0000,   /* U+2EBB */
  0x0000,   /* U+2EBC */
  0x0000,   /* U+2EBD */
  0x0000,   /* U+2EBE */
  0x0000,   /* U+2EBF */
  0x0000,   /* U+2EC0 */
  0x0000,   /* U+2EC1 */
  0x0000,   /* U+2EC2 */
  0x0000,   /* U+2EC3 */
  0x0000,   /* U+2EC4 */
  0x0000,   /* U+2EC5 */
  0x0000,   /* U+2EC6 */
  0x0000,   /* U+2EC7 */
  0x0000,   /* U+2EC8 */
  0x0000,   /* U+2EC9 */
  0x0000,   /* U+2ECA */
  0x0000,   /* U+2ECB */
  0x0000,   /* U+2ECC */
  0x0000,   /* U+2ECD */
  0x0000,   /* U+2ECE */
  0x0000,   /* U+2ECF */
  0x0000,   /* U+2ED0 */
  0x0000,   /* U+2ED1 */
  0x0000,   /* U+2ED2 */
  0x0000,   /* U+2ED3 */
  0x0000,   /* U+2ED4 */
  0x0000,   /* U+2ED5 */
  0x0000,   /* U+2ED6 */
  0x0000,   /* U+2ED7 */
  0x0000,   /* U+2ED8 */
  0x0000,   /* U+2ED9 */
  0x0000,   /* U+2EDA */
  0x0000,   /* U+2EDB */
  0x0000,   /* U+2EDC */
  0x0000,   /* U+2EDD */
  0x0000,   /* U+2EDE */
  0x0000,   /* U+2EDF */
  0x0000,   /* U+2EE0 */
  0x0000,   /* U+2EE1 */
  0x0000,   /* U+2EE2 */
  0x0000,   /* U+2EE3 */
  0x0000,   /* U+2EE4 */
  0x0000,   /* U+2EE5 */
  0x0000,   /* U+2EE6 */
  0x0000,   /* U+2EE7 */
  0x0000,   /* U+2EE8 */
  0x0000,   /* U+2EE9 */
  0x0000,   /* U+2EEA */
  0x0000,   /* U+2EEB */
  0x0000,   /* U+2EEC */
  0x0000,   /* U+2EED */
  0x0000,   /* U+2EEE */
  0x0000,   /* U+2EEF */
  0x0000,   /* U+2EF0 */
  0x0000,   /* U+2EF1 */
  0x0000,   /* U+2EF2 */
  0x0000,   /* U+2EF3 */
  0x0000,   /* U+2EF4 */
  0x0000,   /* U+2EF5 */
  0x0000,   /* U+2EF6 */
  0x0000,   /* U+2EF7 */
  0x0000,   /* U+2EF8 */
  0x0000,   /* U+2EF9 */
  0x0000,   /* U+2EFA */
  0x0000,   /* U+2EFB */
  0x0000,   /* U+2EFC */
  0x0000,   /* U+2EFD */
  0x0000,   /* U+2EFE */
  0x0000,   /* U+2EFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0x0000,   /* U+2E45 */
  0x0000,   /* U+2E46 */
  0x0000,   /* U+2E47 */
  0x0000,   /* U+2E48 */
  0x0000,   /* U+2E49 */
  0x0000,   /* U+2E4A */
  0x0000,   /* U+2E4B */
  0x0000,   /* U+2E4C */
  0x0000,   /* U+2E4D */
  0x0000,   /* U+2E4E */
  0x0000,   /* U+2E4F */
  0x0000,   /* U+2E50 */
  0x0000,   /* U+2E51 */
  0x0000,   /* U+2E52 */
  0x0000,   /* U+2E53 */
  0x0000,   /* U+2E54 */
  0x0000,   /* U+2E55 */
  0x0000,   /* U+2E56 */
  0x0000,   /* U+2E57 */
  0x0000,   /* U+2E58 */
  0x0000,   /* U+2E59 */
  0x0000,   /* U+2E5A */
  0x0000,   /* U+2E5B */
  0x0000,   /* U+2E5C */
  0x0000,   /* U+2E5D */
  0x0000,   /* U+2E5E */
  0x0000,   /* U+2E5F */
  0x0000,   /* U+2E60 */
  0x0000,   /* U+2E61 */
  0x0000,   /* U+2E62 */
  0x0000,   /* U+2E63 */
  0x0000,   /* U+2E64 */
  0x0000,   /* U+2E65 */
  0x0000,   /* U+2E66 */
  0x0000,   /* U+2E67 */
  0x0000,   /* U+2E68 */
  0x0000,   /* U+2E69 */
  0x0000,   /* U+2E6A */
  0x0000,   /* U+2E6B */
  0x0000,   /* U+2E6C */
  0x0000,   /* U+2E6D */
  0x0000,   /* U+2E6E */
  0x0000,   /* U+2E6F */
  0x0000,   /* U+2E70 */
  0x0000,   /* U+2E71 */
  0x0000,   /* U+2E72 */
  0x0000,   /* U+2E73 */
  0x0000,   /* U+2E74 */
  0x0000,   /* U+2E75 */
  0x0000,   /* U+2E76 */
  0x0000,   /* U+2E77 */
  0x0000,   /* U+2E78 */
  0x0000,   /* U+2E79 */
  0x0000,   /* U+2E7A */
  0x0000,   /* U+2E7B */
  0x0000,   /* U+2E7C */
  0x0000,   /* U+2E7D */
  0x0000,   /* U+2E7E */
  0x0000,   /* U+2E7F */
  0x0000,   /* U+2E80 */
  0x0000,   /* U+2E81 */
  0x0000,   /* U+2E82 */
  0x0000,   /* U+2E83 */
  0x0000,   /* U+2E84 */
  0x0000,   /* U+2E85 */
  0x0000,   /* U+2E86 */
  0x0000,   /* U+2E87 */
  0x0000,   /* U+2E88 */
  0x0000,   /* U+2E89 */
  0x0000,   /* U+2E8A */
  0x0000,   /* U+2E8B */
  0x0000,   /* U+2E8C */
  0x0000,   /* U+2E8D */
  0x0000,   /* U+2E8E */
  0x0000,   /* U+2E8F */
  0x0000,   /* U+2E90 */
  0x0000,   /* U+2E91 */
  0x0000,   /* U+2E92 */
  0x0000,   /* U+2E93 */
  0x0000,   /* U+2E94 */
  0x0000,   /* U+2E95 */
  0x0000,   /* U+2E96 */
  0x0000,   /* U+2E97 */
  0x0000,   /* U+2E98 */
  0x0000,   /* U+2E99 */
  0x0000,   /* U+2E9A */
  0x0000,   /* U+2E9B */
  0x0000,   /* U+2E9C */
  0x0000,   /* U+2E9D */
  0x0000,   /* U+2E9E */
  0x0000,   /* U+2E9F */
  0x0000,   /* U+2EA0 */
  0x0000,   /* U+2EA1 */
  0x0000,   /* U+2EA2 */
  0x0000,   /* U+2EA3 */
  0x0000,   /* U+2EA4 */
  0x0000,   /* U+2EA5 */
  0x0000,   /* U+2EA6 */
  0x0000,   /* U+2EA7 */
  0x0000,   /* U+2EA8 */
  0x0000,   /* U+2EA9 */
  0x0000,   /* U+2EAA */
  0x0000,   /* U+2EAB */
  0x0000,   /* U+2EAC */
  0x0000,   /* U+2EAD */
  0x0000,   /* U+2EAE */
  0x0000,   /* U+2EAF */
  0x0000,   /* U+2EB0 */
  0x0000,   /* U+2EB1 */
  0x0000,   /* U+2EB2 */
  0x0000,   /* U+2EB3 */
  0x0000,   /* U+2EB4 */
  0x0000,   /* U+2EB5 */
  0x0000,   /* U+2EB6 */
  0x0000,   /* U+2EB7 */
  0x0000,   /* U+2EB8 */
  0x0000,   /* U+2EB9 */
  0x0000,   /* U+2EBA */
  0x0000,   /* U+2EBB */
  0x0000,   /* U+2EBC */
  0x0000,   /* U+2EBD */
  0x0000,   /* U+2EBE */
  0x0000,   /* U+2EBF */
  0x0000,   /* U+2EC0 */
  0x0000,   /* U+2EC1 */
  0x0000,   /* U+2EC2 */
  0x0000,   /* U+2EC3 */
  0x0000,   /* U+2EC4 */
  0x0000,   /* U+2EC5 */
  0x0000,   /* U+2EC6 */
  0x0000,   /* U+2EC7 */
  0x0000,   /* U+2EC8 */
  0x0000,   /* U+2EC9 */
  0x0000,   /* U+2ECA */
  0x0000,   /* U+2ECB */
  0x0000,   /* U+2ECC */
  0x0000,   /* U+2ECD */
  0x0000,   /* U+2ECE */
  0x0000,   /* U+2ECF */
  0x0000,   /* U+2ED0 */
  0x0000,   /* U+2ED1 */
  0x0000,   /* U+2ED2 */
  0x0000,   /* U+2ED3 */
  0x0000,   /* U+2ED4 */
  0x0000,   /* U+2ED5 */
  0x0000,   /* U+2ED6 */
  0x0000,   /* U+2ED7 */
  0x0000,   /* U+2ED8 */
  0x0000,   /* U+2ED9 */
  0x0000,   /* U+2EDA */
  0x0000,   /* U+2EDB */
  0x0000,   /* U+2EDC */
  0x0000,   /* U+2EDD */
  0x0000,   /* U+2EDE */
  0x0000,   /* U+2EDF */
  0x0000,   /* U+2EE0 */
  0x0000,   /* U+2EE1 */
  0x0000,   /* U+2EE2 */
  0x0000,   /* U+2EE3 */
  0x0000,   /* U+2EE4 */
  0x0000,   /* U+2EE5 */
  0x0000,   /* U+2EE6 */
  0x0000,   /* U+2EE7 */
  0x0000,   /* U+2EE8 */
  0x0000,   /* U+2EE9 */
  0x0000,   /* U+2EEA */
  0x0000,   /* U+2EEB */
  0x0000,   /* U+2EEC */
  0x0000,   /* U+2EED */
  0x0000,   /* U+2EEE */
  0x0000,   /* U+2EEF */
  0x0000,   /* U+2EF0 */
  0x0000,   /* U+2EF1 */
  0x0000,   /* U+2EF2 */
  0x0000,   /* U+2EF3 */
  0x0000,   /* U+2EF4 */
  0x0000,   /* U+2EF5 */
  0x0000,   /* U+2EF6 */
  0x0000,   /* U+2EF7 */
  0x0000,   /* U+2EF8 */
  0x0000,   /* U+2EF9 */
  0x0000,   /* U+2EFA */
  0x0000,   /* U+2EFB */
  0x0000,   /* U+2EFC */
  0x0000,   /* U+2EFD */
  0x0000,   /* U+2EFE */
  0x0000,   /* U+2EFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0x0000,   /* U+2E45 */
  0x0000,   /* U+2E46 */
  0x0000,   /* U+2E47 */
  0x0000,   /* U+2E48 */
  0x0000,   /* U+2E49 */
  0x0000,   /* U+2E4A */
  0x0000,   /* U+2E4B */
  0x0000,   /* U+2E4C */
  0x0000,   /* U+2E4D */
  0x0000,   /* U+2E4E */
  0x0000,   /* U+2E4F */
  0x0000,   /* U+2E50 */
  0x0000,   /* U+2E51 */
  0x0000,   /* U+2E52 */
  0x0000,   /* U+2E53 */
  0x0000,   /* U+2E54 */
  0x0000,   /* U+2E55 */
  0x0000,   /* U+2E56 */
  0x0000,   /* U+2E57 */
  0x0000,   /* U+2E58 */
  0x0000,   /* U+2E59 */
  0x0000,   /* U+2E5A */
  0x0000,   /* U+2E5B */
  0x0000,   /* U+2E5C */
  0x0000,   /* U+2E5D */
  0x0000,   /* U+2E5E */
  0x0000,   /* U+2E5F */
  0x0000,   /* U+2E60 */
  0x0000,   /* U+2E61 */
  0x0000,   /* U+2E62 */
  0x0000,   /* U+2E63 */
  0x0000,   /* U+2E64 */
  0x0000,   /* U+2E65 */
  0x0000,   /* U+2E66 */
  0x0000,   /* U+2E67 */
  0x0000,   /* U+2E68 */
  0x0000,   /* U+2E69 */
  0x0000,   /* U+2E6A */
  0x0000,   /* U+2E6B */
  0x0000,   /* U+2E6C */
  0x0000,   /* U+2E6D */
  0x0000,   /* U+2E6E */
  0x0000,   /* U+2E6F */
  0x0000,   /* U+2E70 */
  0x0000,   /* U+2E71 */
  0x0000,   /* U+2E72 */
  0x0000,   /* U+2E73 */
  0x0000,   /* U+2E74 */
  0x0000,   /* U+2E75 */
  0x0000,   /* U+2E76 */
  0x0000,   /* U+2E77 */
  0x0000,   /* U+2E78 */
  0x0000,   /* U+2E79 */
  0x0000,   /* U+2E7A */
  0x0000,   /* U+2E7B */
  0x0000,   /* U+2E7C */
  0x0000,   /* U+2E7D */
  0x0000,   /* U+2E7E */
  0x0000,   /* U+2E7F */
  0x0000,   /* U+2E80 */
  0x0000,   /* U+2E81 */
  0x0000,   /* U+2E82 */
  0x0000,   /* U+2E83 */
  0x0000,   /* U+2E84 */
  0x0000,   /* U+2E85 */
  0x0000,   /* U+2E86 */
  0x0000,   /* U+2E87 */
  0x0000,   /* U+2E88 */
  0x0000,   /* U+2E89 */
  0x0000,   /* U+2E8A */
  0x0000,   /* U+2E8B */
  0x0000,   /* U+2E8C */
  0x0000,   /* U+2E8D */
  0x0000,   /* U+2E8E */
  0x0000,   /* U+2E8F */
  0x0000,   /* U+2E90 */
  0x0000,   /* U+2E91 */
  0x0000,   /* U+2E92 */
  0x0000,   /* U+2E93 */
  0x0000,   /* U+2E94 */
  0x0000,   /* U+2E95 */
  0x0000,   /* U+2E96 */
  0x0000,   /* U+2E97 */
  0x0000,   /* U+2E98 */
  0x0000,   /* U+2E99 */
  0x0000,   /* U+2E9A */
  0x0000,   /* U+2E9B */
  0x0000,   /* U+2E9C */
  0x0000,   /* U+2E9D */
  0x0000,   /* U+2E9E */
  0x0000,   /* U+2E9F */
  0x0000,   /* U+2EA0 */
  0x0000,   /* U+2EA1 */
  0x0000,   /* U+2EA2 */
  0x0000,   /* U+2EA3 */
  0x0000,   /* U+2EA4 */
  0x0000,   /* U+2EA5 */
  0x0000,   /* U+2EA6 */
  0x0000,   /* U+2EA7 */
  0x0000,   /* U+2EA8 */
  0x0000,   /* U+2EA9 */
  0x0000,   /* U+2EAA */
  0x0000,   /* U+2EAB */
  0x0000,   /* U+2EAC */
  0x0000,   /* U+2EAD */
  0x0000,   /* U+2EAE */
  0x0000,   /* U+2EAF */
  0x0000,   /* U+2EB0 */
  0x0000,   /* U+2EB1 */
  0x0000,   /* U+2EB2 */
  0x0000,   /* U+2EB3 */
  0x0000,   /* U+2EB4 */
  0x0000,   /* U+2EB5 */
  0x0000,   /* U+2EB6 */
  0x0000,   /* U+2EB7 */
  0x0000,   /* U+2EB8 */
  0x0000,   /* U+2EB9 */
  0x0000,   /* U+2EBA */
  0x0000,   /* U+2EBB */
  0x0000,   /* U+2EBC */
  0x0000,   /* U+2EBD */
  0x0000,   /* U+2EBE */
  0x0000,   /* U+2EBF */
  0x0000,   /* U+2EC0 */
  0x0000,   /* U+2EC1 */
  0x0000,   /* U+2EC2 */
  0x0000,   /* U+2EC3 */
  0x0000,   /* U+2EC4 */
  0x0000,   /* U+2EC5 */
  0x0000,   /* U+2EC6 */
  0x0000,   /* U+2EC7 */
  0x0000,   /* U+2EC8 */
  0x0000,   /* U+2EC9 */
  0x0000,   /* U+2ECA */
  0x0000,   /* U+2ECB */
  0x0000,   /* U+2ECC */
  0x0000,   /* U+2ECD */
  0x0000,   /* U+2ECE */
  0x0000,   /* U+2ECF */
  0x0000,   /* U+2ED0 */
  0x0000,   /* U+2ED1 */
  0x0000,   /* U+2ED2 */
  0x0000,   /* U+2ED3 */
  0x0000,   /* U+2ED4 */
  0x0000,   /* U+2ED5 */
  0x0000,   /* U+2ED6 */
  0x0000,   /* U+2ED7 */
  0x0000,   /* U+2ED8 */
  0x0000,   /* U+2ED9 */
  0x0000,   /* U+2EDA */
  0x0000,   /* U+2EDB */
  0x0000,   /* U+2EDC */
  0x0000,   /* U+2EDD */
  0x0000,   /* U+2EDE */
  0x0000,   /* U+2EDF */
  0x0000,   /* U+2EE0 */
  0x0000,   /* U+2EE1 */
  0x0000,   /* U+2EE2 */
  0x0000,   /* U+2EE3 */
  0x0000,   /* U+2EE4 */
  0x0000,   /* U+2EE5 */
  0x0000,   /* U+2EE6 */
  0x0000,   /* U+2EE7 */
  0x0000,   /* U+2EE8 */
  0x0000,   /* U+2EE9 */
  0x0000,   /* U+2EEA */
  0x0000,   /* U+2EEB */
  0x0000,   /* U+2EEC */
  0x0000,   /* U+2EED */
  0x0000,   /* U+2EEE */
  0x0000,   /* U+2EEF */
  0x0000,   /* U+2EF0 */
  0x0000,   /* U+2EF1 */
  0x0000,   /* U+2EF2 */
  0x0000,   /* U+2EF3 */
  0x0000,   /* U+2EF4 */
  0x0000,   /* U+2EF5 */
  0x0000,   /* U+2EF6 */
  0x0000,   /* U+2EF7 */
  0x0000,   /* U+2EF8 */
  0x0000,   /* U+2EF9 */
  0x0000,   /* U+2EFA */
  0x0000,   /* U+2EFB */
  0x0000,   /* U+2EFC */
  0x0000,   /* U+2EFD */
  0x0000,   /* U+2EFE */
  0x0000,   /* U+2EFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0x0000,   /* U+2E45 */
  0x0000,   /* U+2E46 */
  0x0000,   /* U+2E47 */
  0x0000,   /* U+2E48 */
  0x0000,   /* U+2E49 */
  0x0000,   /* U+2E4A */
  0x0000,   /* U+2E4B */
  0x0000,   /* U+2E4C */
  0x0000,   /* U+2E4D */
  0x0000,   /* U+2E4E */
  0x0000,   /* U+2E4F */
  0x0000,   /* U+2E50 */
  0x0000,   /* U+2E51 */
  0x0000,   /* U+2E52 */
  0x0000,   /* U+2E53 */
  0x0000,   /* U+2E54 */
  0x0000,   /* U+2E55 */
  0x0000,   /* U+2E56 */
  0x0000,   /* U+2E57 */
  0x0000,   /* U+2E58 */
  0x0000,   /* U+2E59 */
  0x0000,   /* U+2E5A */
  0x0000,   /* U+2E5B */
  0x0000,   /* U+2E5C */
  0x0000,   /* U+2E5D */
  0x0000,   /* U+2E5E */
  0x0000,   /* U+2E5F */
  0x0000,   /* U+2E60 */
  0x0000,   /* U+2E61 */
  0x0000,   /* U+2E62 */
  0x0000,   /* U+2E63 */
  0x0000,   /* U+2E64 */
  0x0000,   /* U+2E65 */
  0x0000,   /* U+2E66 */
  0x0000,   /* U+2E67 */
  0x0000,   /* U+2E68 */
  0x0000,   /* U+2E69 */
  0x0000,   /* U+2E6A */
  0x0000,   /* U+2E6B */
  0x0000,   /* U+2E6C */
  0x0000,   /* U+2E6D */
  0x0000,   /* U+2E6E */
  0x0000,   /* U+2E6F */
  0x0000,   /* U+2E70 */
  0x0000,   /* U+2E71 */
  0x0000,   /* U+2E72 */
  0x0000,   /* U+2E73 */
  0x0000,   /* U+2E74 */
  0x0000,   /* U+2E75 */
  0x0000,   /* U+2E76 */
  0x0000,   /* U+2E77 */
  0x0000,   /* U+2E78 */
  0x0000,   /* U+2E79 */
  0x0000,   /* U+2E7A */
  0x0000,   /* U+2E7B */
  0x0000,   /* U+2E7C */
  0x0000,   /* U+2E7D */
  0x0000,   /* U+2E7E */
  0x0000,   /* U+2E7F */
  0x0110,   /* U+2E80 */
  0x0110,   /* U+2E81 */
  0x0000,   /* U+2E82 */
  0x0000,   /* U+2E83 */
  0x0110,   /* U+2E84 */
  0x0000,   /* U+2E85 */
  0x0110,   /* U+2E86 */
  0x0110,   /* U+2E87 */
  0x0110,   /* U+2E88 */
  0x0000,   /* U+2E89 */
  0x0110,   /* U+2E8A */
  0x0110,   /* U+2E8B */
  0x0110,   /* U+2E8C */
  0x0111,   /* U+2E8D */
  0x0110,   /* U+2E8E */
  0x0000,   /* U+2E8F */
  0x0000,   /* U+2E90 */
  0x0110,   /* U+2E91 */
  0x0000,   /* U+2E92 */
  0x0000,   /* U+2E93 */
  0x0000,   /* U+2E94 */
  0x0110,   /* U+2E95 */
  0x0000,   /* U+2E96 */
  0x0110,   /* U+2E97 */
  0x0000,   /* U+2E98 */
  0x0000,   /* U+2E99 */
  0x0000,   /* U+2E9A */
  0x0000,   /* U+2E9B */
  0x0110,   /* U+2E9C */
  0x0110,   /* U+2E9D */
  0x0110,   /* U+2E9E */
  0x0000,   /* U+2E9F */
  0x0000,   /* U+2EA0 */
  0x0000,   /* U+2EA1 */
  0x0000,   /* U+2EA2 */
  0x0000,   /* U+2EA3 */
  0x0000,   /* U+2EA4 */
  0x0110,   /* U+2EA5 */
  0x0000,   /* U+2EA6 */
  0x0110,   /* U+2EA7 */
  0x0000,   /* U+2EA8 */
  0x0110,   /* U+2EA9 */
  0x0110,   /* U+2EAA */
  0x0110,   /* U+2EAB */
  0x0110,   /* U+2EAC */
  0x0000,   /* U+2EAD */
  0x0110,   /* U+2EAE */
  0x0000,   /* U+2EAF */
  0x0000,   /* U+2EB0 */
  0x0000,   /* U+2EB1 */
  0x0000,   /* U+2EB2 */
  0x0110,   /* U+2EB3 */
  0x0111,   /* U+2EB4 */
  0x0110,   /* U+2EB5 */
  0x0110,   /* U+2EB6 */
  0x0111,   /* U+2EB7 */
  0x0000,   /* U+2EB8 */
  0x0000,   /* U+2EB9 */
  0x0000,   /* U+2EBA */
  0x0110,   /* U+2EBB */
  0x0110,   /* U+2EBC */
  0x0110,   /* U+2EBD */
  0x0000,   /* U+2EBE */
  0x0110,   /* U+2EBF */
  0x0111,   /* U+2EC0 */
  0x0000,   /* U+2EC1 */
  0x0000,   /* U+2EC2 */
  0x0000,   /* U+2EC3 */
  0x0000,   /* U+2EC4 */
  0x0000,   /* U+2EC5 */
  0x0000,   /* U+2EC6 */
  0x0110,   /* U+2EC7 */
  0x0000,   /* U+2EC8 */
  0x0000,   /* U+2EC9 */
  0x0110,   /* U+2ECA */
  0x0000,   /* U+2ECB */
  0x0000,   /* U+2ECC */
  0x0110,   /* U+2ECD */
  0x0111,   /* U+2ECE */
  0x0110,   /* U+2ECF */
  0x0000,   /* U+2ED0 */
  0x0000,   /* U+2ED1 */
  0x0000,   /* U+2ED2 */
  0x0000,   /* U+2ED3 */
  0x0000,   /* U+2ED4 */
  0x0110,   /* U+2ED5 */
  0x0000,   /* U+2ED6 */
  0x0110,   /* U+2ED7 */
  0x0000,   /* U+2ED8 */
  0x0000,   /* U+2ED9 */
  0x0000,   /* U+2EDA */
  0x0000,   /* U+2EDB */
  0x0000,   /* U+2EDC */
  0x0000,   /* U+2EDD */
  0x0110,   /* U+2EDE */
  0x0000,   /* U+2EDF */
  0x0000,   /* U+2EE0 */
  0x0110,   /* U+2EE1 */
  0x0000,   /* U+2EE2 */
  0x0110,   /* U+2EE3 */
  0x0110,   /* U+2EE4 */
  0x0000,   /* U+2EE5 */
  0x0000,   /* U+2EE6 */
  0x0110,   /* U+2EE7 */
  0x0000,   /* U+2EE8 */
  0x0000,   /* U+2EE9 */
  0x0000,   /* U+2EEA */
  0x0110,   /* U+2EEB */
  0x0000,   /* U+2EEC */
  0x0110,   /* U+2EED */
  0x0000,   /* U+2EEE */
  0x0110,   /* U+2EEF */
  0x0000,   /* U+2EF0 */
  0x0110,   /* U+2EF1 */
  0x0111,   /* U+2EF2 */
  0x0000,   /* U+2EF3 */
  0x0000,   /* U+2EF4 */
  0x0000,   /* U+2EF5 */
  0x0000,   /* U+2EF6 */
  0x0000,   /* U+2EF7 */
  0x0000,   /* U+2EF8 */
  0x0000,   /* U+2EF9 */
  0x0000,   /* U+2EFA */
  0x0000,   /* U+2EFB */
  0x0000,   /* U+2EFC */
  0x0000,   /* U+2EFD */
  0x0000,   /* U+2EFE */
  0x0000,   /* U+2EFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+2E00 */
  0x0000,   /* U+2E01 */
  0x0000,   /* U+2E02 */
  0x0000,   /* U+2E03 */
  0x0000,   /* U+2E04 */
  0x0000,   /* U+2E05 */
  0x0000,   /* U+2E06 */
  0x0000,   /* U+2E07 */
  0x0000,   /* U+2E08 */
  0x0000,   /* U+2E09 */
  0x0000,   /* U+2E0A */
  0x0000,   /* U+2E0B */
  0x0000,   /* U+2E0C */
  0x0000,   /* U+2E0D */
  0x0000,   /* U+2E0E */
  0x0000,   /* U+2E0F */
  0x0000,   /* U+2E10 */
  0x0000,   /* U+2E11 */
  0x0000,   /* U+2E12 */
  0x0000,   /* U+2E13 */
  0x0000,   /* U+2E14 */
  0x0000,   /* U+2E15 */
  0x0000,   /* U+2E16 */
  0x0000,   /* U+2E17 */
  0x0000,   /* U+2E18 */
  0x0000,   /* U+2E19 */
  0x0000,   /* U+2E1A */
  0x0000,   /* U+2E1B */
  0x0000,   /* U+2E1C */
  0x0000,   /* U+2E1D */
  0x0000,   /* U+2E1E */
  0x0000,   /* U+2E1F */
  0x0000,   /* U+2E20 */
  0x0000,   /* U+2E21 */
  0x0000,   /* U+2E22 */
  0x0000,   /* U+2E23 */
  0x0000,   /* U+2E24 */
  0x0000,   /* U+2E25 */
  0x0000,   /* U+2E26 */
  0x0000,   /* U+2E27 */
  0x0000,   /* U+2E28 */
  0x0000,   /* U+2E29 */
  0x0000,   /* U+2E2A */
  0x0000,   /* U+2E2B */
  0x0000,   /* U+2E2C */
  0x0000,   /* U+2E2D */
  0x0000,   /* U+2E2E */
  0x0000,   /* U+2E2F */
  0x0000,   /* U+2E30 */
  0x0000,   /* U+2E31 */
  0x0000,   /* U+2E32 */
  0x0000,   /* U+2E33 */
  0x0000,   /* U+2E34 */
  0x0000,   /* U+2E35 */
  0x0000,   /* U+2E36 */
  0x0000,   /* U+2E37 */
  0x0000,   /* U+2E38 */
  0x0000,   /* U+2E39 */
  0x0000,   /* U+2E3A */
  0x0000,   /* U+2E3B */
  0x0000,   /* U+2E3C */
  0x0000,   /* U+2E3D */
  0x0000,   /* U+2E3E */
  0x0000,   /* U+2E3F */
  0x0000,   /* U+2E40 */
  0x0000,   /* U+2E41 */
  0x0000,   /* U+2E42 */
  0x0000,   /* U+2E43 */
  0x0000,   /* U+2E44 */
  0x0000,   /* U+2E45 */
  0x0000,   /* U+2E46 */
  0x0000,   /* U+2E47 */
  0x0000,   /* U+2E48 */
  0x0000,   /* U+2E49 */
  0x0000,   /* U+2E4A */
  0x0000,   /* U+2E4B */
  0x0000,   /* U+2E4C */
  0x0000,   /* U+2E4D */
  0x0000,   /* U+2E4E */
  0x0000,   /* U+2E4F */
  0x0000,   /* U+2E50 */
  0x0000,   /* U+2E51 */
  0x0000,   /* U+2E52 */
  0x0000,   /* U+2E53 */
  0x0000,   /* U+2E54 */
  0x0000,   /* U+2E55 */
  0x0000,   /* U+2E56 */
  0x0000,   /* U+2E57 */
  0x0000,   /* U+2E58 */
  0x0000,   /* U+2E59 */
  0x0000,   /* U+2E5A */
  0x0000,   /* U+2E5B */
  0x0000,   /* U+2E5C */
  0x0000,   /* U+2E5D */
  0x0000,   /* U+2E5E */
  0x0000,   /* U+2E5F */
  0x0000,   /* U+2E60 */
  0x0000,   /* U+2E61 */
  0x0000,   /* U+2E62 */
  0x0000,   /* U+2E63 */
  0x0000,   /* U+2E64 */
  0x0000,   /* U+2E65 */
  0x0000,   /* U+2E66 */
  0x0000,   /* U+2E67 */
  0x0000,   /* U+2E68 */
  0x0000,   /* U+2E69 */
  0x0000,   /* U+2E6A */
  0x0000,   /* U+2E6B */
  0x0000,   /* U+2E6C */
  0x0000,   /* U+2E6D */
  0x0000,   /* U+2E6E */
  0x0000,   /* U+2E6F */
  0x0000,   /* U+2E70 */
  0x0000,   /* U+2E71 */
  0x0000,   /* U+2E72 */
  0x0000,   /* U+2E73 */
  0x0000,   /* U+2E74 */
  0x0000,   /* U+2E75 */
  0x0000,   /* U+2E76 */
  0x0000,   /* U+2E77 */
  0x0000,   /* U+2E78 */
  0x0000,   /* U+2E79 */
  0x0000,   /* U+2E7A */
  0x0000,   /* U+2E7B */
  0x0000,   /* U+2E7C */
  0x0000,   /* U+2E7D */
  0x0000,   /* U+2E7E */
  0x0000,   /* U+2E7F */
  0x0004,   /* U+2E80 */
  0x0004,   /* U+2E81 */
  0x0000,   /* U+2E82 */
  0x0000,   /* U+2E83 */
  0x0004,   /* U+2E84 */
  0x0000,   /* U+2E85 */
  0x0004,   /* U+2E86 */
  0x0004,   /* U+2E87 */
  0x0004,   /* U+2E88 */
  0x0000,   /* U+2E89 */
  0x0004,   /* U+2E8A */
  0x0004,   /* U+2E8B */
  0x0004,   /* U+2E8C */
  0x0004,   /* U+2E8D */
  0x0004,   /* U+2E8E */
  0x0000,   /* U+2E8F */
  0x0000,   /* U+2E90 */
  0x0004,   /* U+2E91 */
  0x0000,   /* U+2E92 */
  0x0000,   /* U+2E93 */
  0x0000,   /* U+2E94 */
  0x0004,   /* U+2E95 */
  0x0000,   /* U+2E96 */
  0x0004,   /* U+2E97 */
  0x0000,   /* U+2E98 */
  0x0000,   /* U+2E99 */
  0x0000,   /* U+2E9A */
  0x0000,   /* U+2E9B */
  0x0004,   /* U+2E9C */
  0x0004,   /* U+2E9D */
  0x0004,   /* U+2E9E */
  0x0000,   /* U+2E9F */
  0x0000,   /* U+2EA0 */
  0x0000,   /* U+2EA1 */
  0x0000,   /* U+2EA2 */
  0x0000,   /* U+2EA3 */
  0x0000,   /* U+2EA4 */
  0x0004,   /* U+2EA5 */
  0x0000,   /* U+2EA6 */
  0x0004,   /* U+2EA7 */
  0x0000,   /* U+2EA8 */
  0x0004,   /* U+2EA9 */
  0x0004,   /* U+2EAA */
  0x0004,   /* U+2EAB */
  0x0004,   /* U+2EAC */
  0x0000,   /* U+2EAD */
  0x0004,   /* U+2EAE */
  0x0000,   /* U+2EAF */
  0x0000,   /* U+2EB0 */
  0x0000,   /* U+2EB1 */
  0x0000,   /* U+2EB2 */
  0x0004,   /* U+2EB3 */
  0x0004,   /* U+2EB4 */
  0x0004,   /* U+2EB5 */
  0x0004,   /* U+2EB6 */
  0x0004,   /* U+2EB7 */
  0x0000,   /* U+2EB8 */
  0x0000,   /* U+2EB9 */
  0x0000,   /* U+2EBA */
  0x0004,   /* U+2EBB */
  0x0004,   /* U+2EBC */
  0x0004,   /* U+2EBD */
  0x0000,   /* U+2EBE */
  0x0004,   /* U+2EBF */
  0x0004,   /* U+2EC0 */
  0x0000,   /* U+2EC1 */
  0x0000,   /* U+2EC2 */
  0x0000,   /* U+2EC3 */
  0x0000,   /* U+2EC4 */
  0x0000,   /* U+2EC5 */
  0x0000,   /* U+2EC6 */
  0x0004,   /* U+2EC7 */
  0x0000,   /* U+2EC8 */
  0x0000,   /* U+2EC9 */
  0x0004,   /* U+2ECA */
  0x0000,   /* U+2ECB */
  0x0000,   /* U+2ECC */
  0x0004,   /* U+2ECD */
  0x0004,   /* U+2ECE */
  0x0004,   /* U+2ECF */
  0x0000,   /* U+2ED0 */
  0x0000,   /* U+2ED1 */
  0x0000,   /* U+2ED2 */
  0x0000,   /* U+2ED3 */
  0x0000,   /* U+2ED4 */
  0x0004,   /* U+2ED5 */
  0x0000,   /* U+2ED6 */
  0x0004,   /* U+2ED7 */
  0x0000,   /* U+2ED8 */
  0x0000,   /* U+2ED9 */
  0x0000,   /* U+2EDA */
  0x0000,   /* U+2EDB */
  0x0000,   /* U+2EDC */
  0x0000,   /* U+2EDD */
  0x0004,   /* U+2EDE */
  0x0000,   /* U+2EDF */
  0x0000,   /* U+2EE0 */
  0x0004,   /* U+2EE1 */
  0x0000,   /* U+2EE2 */
  0x0004,   /* U+2EE3 */
  0x0004,   /* U+2EE4 */
  0x0000,   /* U+2EE5 */
  0x0000,   /* U+2EE6 */
  0x0004,   /* U+2EE7 */
  0x0000,   /* U+2EE8 */
  0x0000,   /* U+2EE9 */
  0x0000,   /* U+2EEA */
  0x0004,   /* U+2EEB */
  0x0000,   /* U+2EEC */
  0x0004,   /* U+2EED */
  0x0000,   /* U+2EEE */
  0x0004,   /* U+2EEF */
  0x0000,   /* U+2EF0 */
  0x0004,   /* U+2EF1 */
  0x0004,   /* U+2EF2 */
  0x0000,   /* U+2EF3 */
  0x0000,   /* U+2EF4 */
  0x0000,   /* U+2EF5 */
  0x0000,   /* U+2EF6 */
  0x0000,   /* U+2EF7 */
  0x0000,   /* U+2EF8 */
  0x0000,   /* U+2EF9 */
  0x0000,   /* U+2EFA */
  0x0000,   /* U+2EFB */
  0x0000,   /* U+2EFC */
  0x0000,   /* U+2EFD */
  0x0000,   /* U+2EFE */
  0x0000,   /* U+2EFF */
};

uint16 uca900_p02F[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB40,   /* U+2F00 */
  0xFB40,   /* U+2F01 */
  0xFB40,   /* U+2F02 */
  0xFB40,   /* U+2F03 */
  0xFB40,   /* U+2F04 */
  0xFB40,   /* U+2F05 */
  0xFB40,   /* U+2F06 */
  0xFB40,   /* U+2F07 */
  0xFB40,   /* U+2F08 */
  0xFB40,   /* U+2F09 */
  0xFB40,   /* U+2F0A */
  0xFB40,   /* U+2F0B */
  0xFB40,   /* U+2F0C */
  0xFB40,   /* U+2F0D */
  0xFB40,   /* U+2F0E */
  0xFB40,   /* U+2F0F */
  0xFB40,   /* U+2F10 */
  0xFB40,   /* U+2F11 */
  0xFB40,   /* U+2F12 */
  0xFB40,   /* U+2F13 */
  0xFB40,   /* U+2F14 */
  0xFB40,   /* U+2F15 */
  0xFB40,   /* U+2F16 */
  0xFB40,   /* U+2F17 */
  0xFB40,   /* U+2F18 */
  0xFB40,   /* U+2F19 */
  0xFB40,   /* U+2F1A */
  0xFB40,   /* U+2F1B */
  0xFB40,   /* U+2F1C */
  0xFB40,   /* U+2F1D */
  0xFB40,   /* U+2F1E */
  0xFB40,   /* U+2F1F */
  0xFB40,   /* U+2F20 */
  0xFB40,   /* U+2F21 */
  0xFB40,   /* U+2F22 */
  0xFB40,   /* U+2F23 */
  0xFB40,   /* U+2F24 */
  0xFB40,   /* U+2F25 */
  0xFB40,   /* U+2F26 */
  0xFB40,   /* U+2F27 */
  0xFB40,   /* U+2F28 */
  0xFB40,   /* U+2F29 */
  0xFB40,   /* U+2F2A */
  0xFB40,   /* U+2F2B */
  0xFB40,   /* U+2F2C */
  0xFB40,   /* U+2F2D */
  0xFB40,   /* U+2F2E */
  0xFB40,   /* U+2F2F */
  0xFB40,   /* U+2F30 */
  0xFB40,   /* U+2F31 */
  0xFB40,   /* U+2F32 */
  0xFB40,   /* U+2F33 */
  0xFB40,   /* U+2F34 */
  0xFB40,   /* U+2F35 */
  0xFB40,   /* U+2F36 */
  0xFB40,   /* U+2F37 */
  0xFB40,   /* U+2F38 */
  0xFB40,   /* U+2F39 */
  0xFB40,   /* U+2F3A */
  0xFB40,   /* U+2F3B */
  0xFB40,   /* U+2F3C */
  0xFB40,   /* U+2F3D */
  0xFB40,   /* U+2F3E */
  0xFB40,   /* U+2F3F */
  0xFB40,   /* U+2F40 */
  0xFB40,   /* U+2F41 */
  0xFB40,   /* U+2F42 */
  0xFB40,   /* U+2F43 */
  0xFB40,   /* U+2F44 */
  0xFB40,   /* U+2F45 */
  0xFB40,   /* U+2F46 */
  0xFB40,   /* U+2F47 */
  0xFB40,   /* U+2F48 */
  0xFB40,   /* U+2F49 */
  0xFB40,   /* U+2F4A */
  0xFB40,   /* U+2F4B */
  0xFB40,   /* U+2F4C */
  0xFB40,   /* U+2F4D */
  0xFB40,   /* U+2F4E */
  0xFB40,   /* U+2F4F */
  0xFB40,   /* U+2F50 */
  0xFB40,   /* U+2F51 */
  0xFB40,   /* U+2F52 */
  0xFB40,   /* U+2F53 */
  0xFB40,   /* U+2F54 */
  0xFB40,   /* U+2F55 */
  0xFB40,   /* U+2F56 */
  0xFB40,   /* U+2F57 */
  0xFB40,   /* U+2F58 */
  0xFB40,   /* U+2F59 */
  0xFB40,   /* U+2F5A */
  0xFB40,   /* U+2F5B */
  0xFB40,   /* U+2F5C */
  0xFB40,   /* U+2F5D */
  0xFB40,   /* U+2F5E */
  0xFB40,   /* U+2F5F */
  0xFB40,   /* U+2F60 */
  0xFB40,   /* U+2F61 */
  0xFB40,   /* U+2F62 */
  0xFB40,   /* U+2F63 */
  0xFB40,   /* U+2F64 */
  0xFB40,   /* U+2F65 */
  0xFB40,   /* U+2F66 */
  0xFB40,   /* U+2F67 */
  0xFB40,   /* U+2F68 */
  0xFB40,   /* U+2F69 */
  0xFB40,   /* U+2F6A */
  0xFB40,   /* U+2F6B */
  0xFB40,   /* U+2F6C */
  0xFB40,   /* U+2F6D */
  0xFB40,   /* U+2F6E */
  0xFB40,   /* U+2F6F */
  0xFB40,   /* U+2F70 */
  0xFB40,   /* U+2F71 */
  0xFB40,   /* U+2F72 */
  0xFB40,   /* U+2F73 */
  0xFB40,   /* U+2F74 */
  0xFB40,   /* U+2F75 */
  0xFB40,   /* U+2F76 */
  0xFB40,   /* U+2F77 */
  0xFB40,   /* U+2F78 */
  0xFB40,   /* U+2F79 */
  0xFB40,   /* U+2F7A */
  0xFB40,   /* U+2F7B */
  0xFB41,   /* U+2F7C */
  0xFB41,   /* U+2F7D */
  0xFB41,   /* U+2F7E */
  0xFB41,   /* U+2F7F */
  0xFB41,   /* U+2F80 */
  0xFB41,   /* U+2F81 */
  0xFB41,   /* U+2F82 */
  0xFB41,   /* U+2F83 */
  0xFB41,   /* U+2F84 */
  0xFB41,   /* U+2F85 */
  0xFB41,   /* U+2F86 */
  0xFB41,   /* U+2F87 */
  0xFB41,   /* U+2F88 */
  0xFB41,   /* U+2F89 */
  0xFB41,   /* U+2F8A */
  0xFB41,   /* U+2F8B */
  0xFB41,   /* U+2F8C */
  0xFB41,   /* U+2F8D */
  0xFB41,   /* U+2F8E */
  0xFB41,   /* U+2F8F */
  0xFB41,   /* U+2F90 */
  0xFB41,   /* U+2F91 */
  0xFB41,   /* U+2F92 */
  0xFB41,   /* U+2F93 */
  0xFB41,   /* U+2F94 */
  0xFB41,   /* U+2F95 */
  0xFB41,   /* U+2F96 */
  0xFB41,   /* U+2F97 */
  0xFB41,   /* U+2F98 */
  0xFB41,   /* U+2F99 */
  0xFB41,   /* U+2F9A */
  0xFB41,   /* U+2F9B */
  0xFB41,   /* U+2F9C */
  0xFB41,   /* U+2F9D */
  0xFB41,   /* U+2F9E */
  0xFB41,   /* U+2F9F */
  0xFB41,   /* U+2FA0 */
  0xFB41,   /* U+2FA1 */
  0xFB41,   /* U+2FA2 */
  0xFB41,   /* U+2FA3 */
  0xFB41,   /* U+2FA4 */
  0xFB41,   /* U+2FA5 */
  0xFB41,   /* U+2FA6 */
  0xFB41,   /* U+2FA7 */
  0xFB41,   /* U+2FA8 */
  0xFB41,   /* U+2FA9 */
  0xFB41,   /* U+2FAA */
  0xFB41,   /* U+2FAB */
  0xFB41,   /* U+2FAC */
  0xFB41,   /* U+2FAD */
  0xFB41,   /* U+2FAE */
  0xFB41,   /* U+2FAF */
  0xFB41,   /* U+2FB0 */
  0xFB41,   /* U+2FB1 */
  0xFB41,   /* U+2FB2 */
  0xFB41,   /* U+2FB3 */
  0xFB41,   /* U+2FB4 */
  0xFB41,   /* U+2FB5 */
  0xFB41,   /* U+2FB6 */
  0xFB41,   /* U+2FB7 */
  0xFB41,   /* U+2FB8 */
  0xFB41,   /* U+2FB9 */
  0xFB41,   /* U+2FBA */
  0xFB41,   /* U+2FBB */
  0xFB41,   /* U+2FBC */
  0xFB41,   /* U+2FBD */
  0xFB41,   /* U+2FBE */
  0xFB41,   /* U+2FBF */
  0xFB41,   /* U+2FC0 */
  0xFB41,   /* U+2FC1 */
  0xFB41,   /* U+2FC2 */
  0xFB41,   /* U+2FC3 */
  0xFB41,   /* U+2FC4 */
  0xFB41,   /* U+2FC5 */
  0xFB41,   /* U+2FC6 */
  0xFB41,   /* U+2FC7 */
  0xFB41,   /* U+2FC8 */
  0xFB41,   /* U+2FC9 */
  0xFB41,   /* U+2FCA */
  0xFB41,   /* U+2FCB */
  0xFB41,   /* U+2FCC */
  0xFB41,   /* U+2FCD */
  0xFB41,   /* U+2FCE */
  0xFB41,   /* U+2FCF */
  0xFB41,   /* U+2FD0 */
  0xFB41,   /* U+2FD1 */
  0xFB41,   /* U+2FD2 */
  0xFB41,   /* U+2FD3 */
  0xFB41,   /* U+2FD4 */
  0xFB41,   /* U+2FD5 */
  0xFBC0,   /* U+2FD6 */
  0xFBC0,   /* U+2FD7 */
  0xFBC0,   /* U+2FD8 */
  0xFBC0,   /* U+2FD9 */
  0xFBC0,   /* U+2FDA */
  0xFBC0,   /* U+2FDB */
  0xFBC0,   /* U+2FDC */
  0xFBC0,   /* U+2FDD */
  0xFBC0,   /* U+2FDE */
  0xFBC0,   /* U+2FDF */
  0xFBC0,   /* U+2FE0 */
  0xFBC0,   /* U+2FE1 */
  0xFBC0,   /* U+2FE2 */
  0xFBC0,   /* U+2FE3 */
  0xFBC0,   /* U+2FE4 */
  0xFBC0,   /* U+2FE5 */
  0xFBC0,   /* U+2FE6 */
  0xFBC0,   /* U+2FE7 */
  0xFBC0,   /* U+2FE8 */
  0xFBC0,   /* U+2FE9 */
  0xFBC0,   /* U+2FEA */
  0xFBC0,   /* U+2FEB */
  0xFBC0,   /* U+2FEC */
  0xFBC0,   /* U+2FED */
  0xFBC0,   /* U+2FEE */
  0xFBC0,   /* U+2FEF */
  0x1A5B,   /* U+2FF0 */
  0x1A5C,   /* U+2FF1 */
  0x1A5D,   /* U+2FF2 */
  0x1A5E,   /* U+2FF3 */
  0x1A5F,   /* U+2FF4 */
  0x1A60,   /* U+2FF5 */
  0x1A61,   /* U+2FF6 */
  0x1A62,   /* U+2FF7 */
  0x1A63,   /* U+2FF8 */
  0x1A64,   /* U+2FF9 */
  0x1A65,   /* U+2FFA */
  0x1A66,   /* U+2FFB */
  0xFBC0,   /* U+2FFC */
  0xFBC0,   /* U+2FFD */
  0xFBC0,   /* U+2FFE */
  0xFBC0,   /* U+2FFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2F00 */
  0x0020,   /* U+2F01 */
  0x0020,   /* U+2F02 */
  0x0020,   /* U+2F03 */
  0x0020,   /* U+2F04 */
  0x0020,   /* U+2F05 */
  0x0020,   /* U+2F06 */
  0x0020,   /* U+2F07 */
  0x0020,   /* U+2F08 */
  0x0020,   /* U+2F09 */
  0x0020,   /* U+2F0A */
  0x0020,   /* U+2F0B */
  0x0020,   /* U+2F0C */
  0x0020,   /* U+2F0D */
  0x0020,   /* U+2F0E */
  0x0020,   /* U+2F0F */
  0x0020,   /* U+2F10 */
  0x0020,   /* U+2F11 */
  0x0020,   /* U+2F12 */
  0x0020,   /* U+2F13 */
  0x0020,   /* U+2F14 */
  0x0020,   /* U+2F15 */
  0x0020,   /* U+2F16 */
  0x0020,   /* U+2F17 */
  0x0020,   /* U+2F18 */
  0x0020,   /* U+2F19 */
  0x0020,   /* U+2F1A */
  0x0020,   /* U+2F1B */
  0x0020,   /* U+2F1C */
  0x0020,   /* U+2F1D */
  0x0020,   /* U+2F1E */
  0x0020,   /* U+2F1F */
  0x0020,   /* U+2F20 */
  0x0020,   /* U+2F21 */
  0x0020,   /* U+2F22 */
  0x0020,   /* U+2F23 */
  0x0020,   /* U+2F24 */
  0x0020,   /* U+2F25 */
  0x0020,   /* U+2F26 */
  0x0020,   /* U+2F27 */
  0x0020,   /* U+2F28 */
  0x0020,   /* U+2F29 */
  0x0020,   /* U+2F2A */
  0x0020,   /* U+2F2B */
  0x0020,   /* U+2F2C */
  0x0020,   /* U+2F2D */
  0x0020,   /* U+2F2E */
  0x0020,   /* U+2F2F */
  0x0020,   /* U+2F30 */
  0x0020,   /* U+2F31 */
  0x0020,   /* U+2F32 */
  0x0020,   /* U+2F33 */
  0x0020,   /* U+2F34 */
  0x0020,   /* U+2F35 */
  0x0020,   /* U+2F36 */
  0x0020,   /* U+2F37 */
  0x0020,   /* U+2F38 */
  0x0020,   /* U+2F39 */
  0x0020,   /* U+2F3A */
  0x0020,   /* U+2F3B */
  0x0020,   /* U+2F3C */
  0x0020,   /* U+2F3D */
  0x0020,   /* U+2F3E */
  0x0020,   /* U+2F3F */
  0x0020,   /* U+2F40 */
  0x0020,   /* U+2F41 */
  0x0020,   /* U+2F42 */
  0x0020,   /* U+2F43 */
  0x0020,   /* U+2F44 */
  0x0020,   /* U+2F45 */
  0x0020,   /* U+2F46 */
  0x0020,   /* U+2F47 */
  0x0020,   /* U+2F48 */
  0x0020,   /* U+2F49 */
  0x0020,   /* U+2F4A */
  0x0020,   /* U+2F4B */
  0x0020,   /* U+2F4C */
  0x0020,   /* U+2F4D */
  0x0020,   /* U+2F4E */
  0x0020,   /* U+2F4F */
  0x0020,   /* U+2F50 */
  0x0020,   /* U+2F51 */
  0x0020,   /* U+2F52 */
  0x0020,   /* U+2F53 */
  0x0020,   /* U+2F54 */
  0x0020,   /* U+2F55 */
  0x0020,   /* U+2F56 */
  0x0020,   /* U+2F57 */
  0x0020,   /* U+2F58 */
  0x0020,   /* U+2F59 */
  0x0020,   /* U+2F5A */
  0x0020,   /* U+2F5B */
  0x0020,   /* U+2F5C */
  0x0020,   /* U+2F5D */
  0x0020,   /* U+2F5E */
  0x0020,   /* U+2F5F */
  0x0020,   /* U+2F60 */
  0x0020,   /* U+2F61 */
  0x0020,   /* U+2F62 */
  0x0020,   /* U+2F63 */
  0x0020,   /* U+2F64 */
  0x0020,   /* U+2F65 */
  0x0020,   /* U+2F66 */
  0x0020,   /* U+2F67 */
  0x0020,   /* U+2F68 */
  0x0020,   /* U+2F69 */
  0x0020,   /* U+2F6A */
  0x0020,   /* U+2F6B */
  0x0020,   /* U+2F6C */
  0x0020,   /* U+2F6D */
  0x0020,   /* U+2F6E */
  0x0020,   /* U+2F6F */
  0x0020,   /* U+2F70 */
  0x0020,   /* U+2F71 */
  0x0020,   /* U+2F72 */
  0x0020,   /* U+2F73 */
  0x0020,   /* U+2F74 */
  0x0020,   /* U+2F75 */
  0x0020,   /* U+2F76 */
  0x0020,   /* U+2F77 */
  0x0020,   /* U+2F78 */
  0x0020,   /* U+2F79 */
  0x0020,   /* U+2F7A */
  0x0020,   /* U+2F7B */
  0x0020,   /* U+2F7C */
  0x0020,   /* U+2F7D */
  0x0020,   /* U+2F7E */
  0x0020,   /* U+2F7F */
  0x0020,   /* U+2F80 */
  0x0020,   /* U+2F81 */
  0x0020,   /* U+2F82 */
  0x0020,   /* U+2F83 */
  0x0020,   /* U+2F84 */
  0x0020,   /* U+2F85 */
  0x0020,   /* U+2F86 */
  0x0020,   /* U+2F87 */
  0x0020,   /* U+2F88 */
  0x0020,   /* U+2F89 */
  0x0020,   /* U+2F8A */
  0x0020,   /* U+2F8B */
  0x0020,   /* U+2F8C */
  0x0020,   /* U+2F8D */
  0x0020,   /* U+2F8E */
  0x0020,   /* U+2F8F */
  0x0020,   /* U+2F90 */
  0x0020,   /* U+2F91 */
  0x0020,   /* U+2F92 */
  0x0020,   /* U+2F93 */
  0x0020,   /* U+2F94 */
  0x0020,   /* U+2F95 */
  0x0020,   /* U+2F96 */
  0x0020,   /* U+2F97 */
  0x0020,   /* U+2F98 */
  0x0020,   /* U+2F99 */
  0x0020,   /* U+2F9A */
  0x0020,   /* U+2F9B */
  0x0020,   /* U+2F9C */
  0x0020,   /* U+2F9D */
  0x0020,   /* U+2F9E */
  0x0020,   /* U+2F9F */
  0x0020,   /* U+2FA0 */
  0x0020,   /* U+2FA1 */
  0x0020,   /* U+2FA2 */
  0x0020,   /* U+2FA3 */
  0x0020,   /* U+2FA4 */
  0x0020,   /* U+2FA5 */
  0x0020,   /* U+2FA6 */
  0x0020,   /* U+2FA7 */
  0x0020,   /* U+2FA8 */
  0x0020,   /* U+2FA9 */
  0x0020,   /* U+2FAA */
  0x0020,   /* U+2FAB */
  0x0020,   /* U+2FAC */
  0x0020,   /* U+2FAD */
  0x0020,   /* U+2FAE */
  0x0020,   /* U+2FAF */
  0x0020,   /* U+2FB0 */
  0x0020,   /* U+2FB1 */
  0x0020,   /* U+2FB2 */
  0x0020,   /* U+2FB3 */
  0x0020,   /* U+2FB4 */
  0x0020,   /* U+2FB5 */
  0x0020,   /* U+2FB6 */
  0x0020,   /* U+2FB7 */
  0x0020,   /* U+2FB8 */
  0x0020,   /* U+2FB9 */
  0x0020,   /* U+2FBA */
  0x0020,   /* U+2FBB */
  0x0020,   /* U+2FBC */
  0x0020,   /* U+2FBD */
  0x0020,   /* U+2FBE */
  0x0020,   /* U+2FBF */
  0x0020,   /* U+2FC0 */
  0x0020,   /* U+2FC1 */
  0x0020,   /* U+2FC2 */
  0x0020,   /* U+2FC3 */
  0x0020,   /* U+2FC4 */
  0x0020,   /* U+2FC5 */
  0x0020,   /* U+2FC6 */
  0x0020,   /* U+2FC7 */
  0x0020,   /* U+2FC8 */
  0x0020,   /* U+2FC9 */
  0x0020,   /* U+2FCA */
  0x0020,   /* U+2FCB */
  0x0020,   /* U+2FCC */
  0x0020,   /* U+2FCD */
  0x0020,   /* U+2FCE */
  0x0020,   /* U+2FCF */
  0x0020,   /* U+2FD0 */
  0x0020,   /* U+2FD1 */
  0x0020,   /* U+2FD2 */
  0x0020,   /* U+2FD3 */
  0x0020,   /* U+2FD4 */
  0x0020,   /* U+2FD5 */
  0x0020,   /* U+2FD6 */
  0x0020,   /* U+2FD7 */
  0x0020,   /* U+2FD8 */
  0x0020,   /* U+2FD9 */
  0x0020,   /* U+2FDA */
  0x0020,   /* U+2FDB */
  0x0020,   /* U+2FDC */
  0x0020,   /* U+2FDD */
  0x0020,   /* U+2FDE */
  0x0020,   /* U+2FDF */
  0x0020,   /* U+2FE0 */
  0x0020,   /* U+2FE1 */
  0x0020,   /* U+2FE2 */
  0x0020,   /* U+2FE3 */
  0x0020,   /* U+2FE4 */
  0x0020,   /* U+2FE5 */
  0x0020,   /* U+2FE6 */
  0x0020,   /* U+2FE7 */
  0x0020,   /* U+2FE8 */
  0x0020,   /* U+2FE9 */
  0x0020,   /* U+2FEA */
  0x0020,   /* U+2FEB */
  0x0020,   /* U+2FEC */
  0x0020,   /* U+2FED */
  0x0020,   /* U+2FEE */
  0x0020,   /* U+2FEF */
  0x0020,   /* U+2FF0 */
  0x0020,   /* U+2FF1 */
  0x0020,   /* U+2FF2 */
  0x0020,   /* U+2FF3 */
  0x0020,   /* U+2FF4 */
  0x0020,   /* U+2FF5 */
  0x0020,   /* U+2FF6 */
  0x0020,   /* U+2FF7 */
  0x0020,   /* U+2FF8 */
  0x0020,   /* U+2FF9 */
  0x0020,   /* U+2FFA */
  0x0020,   /* U+2FFB */
  0x0020,   /* U+2FFC */
  0x0020,   /* U+2FFD */
  0x0020,   /* U+2FFE */
  0x0020,   /* U+2FFF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+2F00 */
  0x0004,   /* U+2F01 */
  0x0004,   /* U+2F02 */
  0x0004,   /* U+2F03 */
  0x0004,   /* U+2F04 */
  0x0004,   /* U+2F05 */
  0x0004,   /* U+2F06 */
  0x0004,   /* U+2F07 */
  0x0004,   /* U+2F08 */
  0x0004,   /* U+2F09 */
  0x0004,   /* U+2F0A */
  0x0004,   /* U+2F0B */
  0x0004,   /* U+2F0C */
  0x0004,   /* U+2F0D */
  0x0004,   /* U+2F0E */
  0x0004,   /* U+2F0F */
  0x0004,   /* U+2F10 */
  0x0004,   /* U+2F11 */
  0x0004,   /* U+2F12 */
  0x0004,   /* U+2F13 */
  0x0004,   /* U+2F14 */
  0x0004,   /* U+2F15 */
  0x0004,   /* U+2F16 */
  0x0004,   /* U+2F17 */
  0x0004,   /* U+2F18 */
  0x0004,   /* U+2F19 */
  0x0004,   /* U+2F1A */
  0x0004,   /* U+2F1B */
  0x0004,   /* U+2F1C */
  0x0004,   /* U+2F1D */
  0x0004,   /* U+2F1E */
  0x0004,   /* U+2F1F */
  0x0004,   /* U+2F20 */
  0x0004,   /* U+2F21 */
  0x0004,   /* U+2F22 */
  0x0004,   /* U+2F23 */
  0x0004,   /* U+2F24 */
  0x0004,   /* U+2F25 */
  0x0004,   /* U+2F26 */
  0x0004,   /* U+2F27 */
  0x0004,   /* U+2F28 */
  0x0004,   /* U+2F29 */
  0x0004,   /* U+2F2A */
  0x0004,   /* U+2F2B */
  0x0004,   /* U+2F2C */
  0x0004,   /* U+2F2D */
  0x0004,   /* U+2F2E */
  0x0004,   /* U+2F2F */
  0x0004,   /* U+2F30 */
  0x0004,   /* U+2F31 */
  0x0004,   /* U+2F32 */
  0x0004,   /* U+2F33 */
  0x0004,   /* U+2F34 */
  0x0004,   /* U+2F35 */
  0x0004,   /* U+2F36 */
  0x0004,   /* U+2F37 */
  0x0004,   /* U+2F38 */
  0x0004,   /* U+2F39 */
  0x0004,   /* U+2F3A */
  0x0004,   /* U+2F3B */
  0x0004,   /* U+2F3C */
  0x0004,   /* U+2F3D */
  0x0004,   /* U+2F3E */
  0x0004,   /* U+2F3F */
  0x0004,   /* U+2F40 */
  0x0004,   /* U+2F41 */
  0x0004,   /* U+2F42 */
  0x0004,   /* U+2F43 */
  0x0004,   /* U+2F44 */
  0x0004,   /* U+2F45 */
  0x0004,   /* U+2F46 */
  0x0004,   /* U+2F47 */
  0x0004,   /* U+2F48 */
  0x0004,   /* U+2F49 */
  0x0004,   /* U+2F4A */
  0x0004,   /* U+2F4B */
  0x0004,   /* U+2F4C */
  0x0004,   /* U+2F4D */
  0x0004,   /* U+2F4E */
  0x0004,   /* U+2F4F */
  0x0004,   /* U+2F50 */
  0x0004,   /* U+2F51 */
  0x0004,   /* U+2F52 */
  0x0004,   /* U+2F53 */
  0x0004,   /* U+2F54 */
  0x0004,   /* U+2F55 */
  0x0004,   /* U+2F56 */
  0x0004,   /* U+2F57 */
  0x0004,   /* U+2F58 */
  0x0004,   /* U+2F59 */
  0x0004,   /* U+2F5A */
  0x0004,   /* U+2F5B */
  0x0004,   /* U+2F5C */
  0x0004,   /* U+2F5D */
  0x0004,   /* U+2F5E */
  0x0004,   /* U+2F5F */
  0x0004,   /* U+2F60 */
  0x0004,   /* U+2F61 */
  0x0004,   /* U+2F62 */
  0x0004,   /* U+2F63 */
  0x0004,   /* U+2F64 */
  0x0004,   /* U+2F65 */
  0x0004,   /* U+2F66 */
  0x0004,   /* U+2F67 */
  0x0004,   /* U+2F68 */
  0x0004,   /* U+2F69 */
  0x0004,   /* U+2F6A */
  0x0004,   /* U+2F6B */
  0x0004,   /* U+2F6C */
  0x0004,   /* U+2F6D */
  0x0004,   /* U+2F6E */
  0x0004,   /* U+2F6F */
  0x0004,   /* U+2F70 */
  0x0004,   /* U+2F71 */
  0x0004,   /* U+2F72 */
  0x0004,   /* U+2F73 */
  0x0004,   /* U+2F74 */
  0x0004,   /* U+2F75 */
  0x0004,   /* U+2F76 */
  0x0004,   /* U+2F77 */
  0x0004,   /* U+2F78 */
  0x0004,   /* U+2F79 */
  0x0004,   /* U+2F7A */
  0x0004,   /* U+2F7B */
  0x0004,   /* U+2F7C */
  0x0004,   /* U+2F7D */
  0x0004,   /* U+2F7E */
  0x0004,   /* U+2F7F */
  0x0004,   /* U+2F80 */
  0x0004,   /* U+2F81 */
  0x0004,   /* U+2F82 */
  0x0004,   /* U+2F83 */
  0x0004,   /* U+2F84 */
  0x0004,   /* U+2F85 */
  0x0004,   /* U+2F86 */
  0x0004,   /* U+2F87 */
  0x0004,   /* U+2F88 */
  0x0004,   /* U+2F89 */
  0x0004,   /* U+2F8A */
  0x0004,   /* U+2F8B */
  0x0004,   /* U+2F8C */
  0x0004,   /* U+2F8D */
  0x0004,   /* U+2F8E */
  0x0004,   /* U+2F8F */
  0x0004,   /* U+2F90 */
  0x0004,   /* U+2F91 */
  0x0004,   /* U+2F92 */
  0x0004,   /* U+2F93 */
  0x0004,   /* U+2F94 */
  0x0004,   /* U+2F95 */
  0x0004,   /* U+2F96 */
  0x0004,   /* U+2F97 */
  0x0004,   /* U+2F98 */
  0x0004,   /* U+2F99 */
  0x0004,   /* U+2F9A */
  0x0004,   /* U+2F9B */
  0x0004,   /* U+2F9C */
  0x0004,   /* U+2F9D */
  0x0004,   /* U+2F9E */
  0x0004,   /* U+2F9F */
  0x0004,   /* U+2FA0 */
  0x0004,   /* U+2FA1 */
  0x0004,   /* U+2FA2 */
  0x0004,   /* U+2FA3 */
  0x0004,   /* U+2FA4 */
  0x0004,   /* U+2FA5 */
  0x0004,   /* U+2FA6 */
  0x0004,   /* U+2FA7 */
  0x0004,   /* U+2FA8 */
  0x0004,   /* U+2FA9 */
  0x0004,   /* U+2FAA */
  0x0004,   /* U+2FAB */
  0x0004,   /* U+2FAC */
  0x0004,   /* U+2FAD */
  0x0004,   /* U+2FAE */
  0x0004,   /* U+2FAF */
  0x0004,   /* U+2FB0 */
  0x0004,   /* U+2FB1 */
  0x0004,   /* U+2FB2 */
  0x0004,   /* U+2FB3 */
  0x0004,   /* U+2FB4 */
  0x0004,   /* U+2FB5 */
  0x0004,   /* U+2FB6 */
  0x0004,   /* U+2FB7 */
  0x0004,   /* U+2FB8 */
  0x0004,   /* U+2FB9 */
  0x0004,   /* U+2FBA */
  0x0004,   /* U+2FBB */
  0x0004,   /* U+2FBC */
  0x0004,   /* U+2FBD */
  0x0004,   /* U+2FBE */
  0x0004,   /* U+2FBF */
  0x0004,   /* U+2FC0 */
  0x0004,   /* U+2FC1 */
  0x0004,   /* U+2FC2 */
  0x0004,   /* U+2FC3 */
  0x0004,   /* U+2FC4 */
  0x0004,   /* U+2FC5 */
  0x0004,   /* U+2FC6 */
  0x0004,   /* U+2FC7 */
  0x0004,   /* U+2FC8 */
  0x0004,   /* U+2FC9 */
  0x0004,   /* U+2FCA */
  0x0004,   /* U+2FCB */
  0x0004,   /* U+2FCC */
  0x0004,   /* U+2FCD */
  0x0004,   /* U+2FCE */
  0x0004,   /* U+2FCF */
  0x0004,   /* U+2FD0 */
  0x0004,   /* U+2FD1 */
  0x0004,   /* U+2FD2 */
  0x0004,   /* U+2FD3 */
  0x0004,   /* U+2FD4 */
  0x0004,   /* U+2FD5 */
  0x0002,   /* U+2FD6 */
  0x0002,   /* U+2FD7 */
  0x0002,   /* U+2FD8 */
  0x0002,   /* U+2FD9 */
  0x0002,   /* U+2FDA */
  0x0002,   /* U+2FDB */
  0x0002,   /* U+2FDC */
  0x0002,   /* U+2FDD */
  0x0002,   /* U+2FDE */
  0x0002,   /* U+2FDF */
  0x0002,   /* U+2FE0 */
  0x0002,   /* U+2FE1 */
  0x0002,   /* U+2FE2 */
  0x0002,   /* U+2FE3 */
  0x0002,   /* U+2FE4 */
  0x0002,   /* U+2FE5 */
  0x0002,   /* U+2FE6 */
  0x0002,   /* U+2FE7 */
  0x0002,   /* U+2FE8 */
  0x0002,   /* U+2FE9 */
  0x0002,   /* U+2FEA */
  0x0002,   /* U+2FEB */
  0x0002,   /* U+2FEC */
  0x0002,   /* U+2FED */
  0x0002,   /* U+2FEE */
  0x0002,   /* U+2FEF */
  0x0002,   /* U+2FF0 */
  0x0002,   /* U+2FF1 */
  0x0002,   /* U+2FF2 */
  0x0002,   /* U+2FF3 */
  0x0002,   /* U+2FF4 */
  0x0002,   /* U+2FF5 */
  0x0002,   /* U+2FF6 */
  0x0002,   /* U+2FF7 */
  0x0002,   /* U+2FF8 */
  0x0002,   /* U+2FF9 */
  0x0002,   /* U+2FFA */
  0x0002,   /* U+2FFB */
  0x0002,   /* U+2FFC */
  0x0002,   /* U+2FFD */
  0x0002,   /* U+2FFE */
  0x0002,   /* U+2FFF */

  /* Primary weight 2 for each character. */
  0xCE00,   /* U+2F00 */
  0xCE28,   /* U+2F01 */
  0xCE36,   /* U+2F02 */
  0xCE3F,   /* U+2F03 */
  0xCE59,   /* U+2F04 */
  0xCE85,   /* U+2F05 */
  0xCE8C,   /* U+2F06 */
  0xCEA0,   /* U+2F07 */
  0xCEBA,   /* U+2F08 */
  0xD13F,   /* U+2F09 */
  0xD165,   /* U+2F0A */
  0xD16B,   /* U+2F0B */
  0xD182,   /* U+2F0C */
  0xD196,   /* U+2F0D */
  0xD1AB,   /* U+2F0E */
  0xD1E0,   /* U+2F0F */
  0xD1F5,   /* U+2F10 */
  0xD200,   /* U+2F11 */
  0xD29B,   /* U+2F12 */
  0xD2F9,   /* U+2F13 */
  0xD315,   /* U+2F14 */
  0xD31A,   /* U+2F15 */
  0xD338,   /* U+2F16 */
  0xD341,   /* U+2F17 */
  0xD35C,   /* U+2F18 */
  0xD369,   /* U+2F19 */
  0xD382,   /* U+2F1A */
  0xD3B6,   /* U+2F1B */
  0xD3C8,   /* U+2F1C */
  0xD3E3,   /* U+2F1D */
  0xD6D7,   /* U+2F1E */
  0xD71F,   /* U+2F1F */
  0xD8EB,   /* U+2F20 */
  0xD902,   /* U+2F21 */
  0xD90A,   /* U+2F22 */
  0xD915,   /* U+2F23 */
  0xD927,   /* U+2F24 */
  0xD973,   /* U+2F25 */
  0xDB50,   /* U+2F26 */
  0xDB80,   /* U+2F27 */
  0xDBF8,   /* U+2F28 */
  0xDC0F,   /* U+2F29 */
  0xDC22,   /* U+2F2A */
  0xDC38,   /* U+2F2B */
  0xDC6E,   /* U+2F2C */
  0xDC71,   /* U+2F2D */
  0xDDDB,   /* U+2F2E */
  0xDDE5,   /* U+2F2F */
  0xDDF1,   /* U+2F30 */
  0xDDFE,   /* U+2F31 */
  0xDE72,   /* U+2F32 */
  0xDE7A,   /* U+2F33 */
  0xDE7F,   /* U+2F34 */
  0xDEF4,   /* U+2F35 */
  0xDEFE,   /* U+2F36 */
  0xDF0B,   /* U+2F37 */
  0xDF13,   /* U+2F38 */
  0xDF50,   /* U+2F39 */
  0xDF61,   /* U+2F3A */
  0xDF73,   /* U+2F3B */
  0xDFC3,   /* U+2F3C */
  0xE208,   /* U+2F3D */
  0xE236,   /* U+2F3E */
  0xE24B,   /* U+2F3F */
  0xE52F,   /* U+2F40 */
  0xE534,   /* U+2F41 */
  0xE587,   /* U+2F42 */
  0xE597,   /* U+2F43 */
  0xE5A4,   /* U+2F44 */
  0xE5B9,   /* U+2F45 */
  0xE5E0,   /* U+2F46 */
  0xE5E5,   /* U+2F47 */
  0xE6F0,   /* U+2F48 */
  0xE708,   /* U+2F49 */
  0xE728,   /* U+2F4A */
  0xEB20,   /* U+2F4B */
  0xEB62,   /* U+2F4C */
  0xEB79,   /* U+2F4D */
  0xEBB3,   /* U+2F4E */
  0xEBCB,   /* U+2F4F */
  0xEBD4,   /* U+2F50 */
  0xEBDB,   /* U+2F51 */
  0xEC0F,   /* U+2F52 */
  0xEC14,   /* U+2F53 */
  0xEC34,   /* U+2F54 */
  0xF06B,   /* U+2F55 */
  0xF22A,   /* U+2F56 */
  0xF236,   /* U+2F57 */
  0xF23B,   /* U+2F58 */
  0xF23F,   /* U+2F59 */
  0xF247,   /* U+2F5A */
  0xF259,   /* U+2F5B */
  0xF25B,   /* U+2F5C */
  0xF2AC,   /* U+2F5D */
  0xF384,   /* U+2F5E */
  0xF389,   /* U+2F5F */
  0xF4DC,   /* U+2F60 */
  0xF4E6,   /* U+2F61 */
  0xF518,   /* U+2F62 */
  0xF51F,   /* U+2F63 */
  0xF528,   /* U+2F64 */
  0xF530,   /* U+2F65 */
  0xF58B,   /* U+2F66 */
  0xF592,   /* U+2F67 */
  0xF676,   /* U+2F68 */
  0xF67D,   /* U+2F69 */
  0xF6AE,   /* U+2F6A */
  0xF6BF,   /* U+2F6B */
  0xF6EE,   /* U+2F6C */
  0xF7DB,   /* U+2F6D */
  0xF7E2,   /* U+2F6E */
  0xF7F3,   /* U+2F6F */
  0xF93A,   /* U+2F70 */
  0xF9B8,   /* U+2F71 */
  0xF9BE,   /* U+2F72 */
  0xFA74,   /* U+2F73 */
  0xFACB,   /* U+2F74 */
  0xFAF9,   /* U+2F75 */
  0xFC73,   /* U+2F76 */
  0xFCF8,   /* U+2F77 */
  0xFF36,   /* U+2F78 */
  0xFF51,   /* U+2F79 */
  0xFF8A,   /* U+2F7A */
  0xFFBD,   /* U+2F7B */
  0x8001,   /* U+2F7C */
  0x800C,   /* U+2F7D */
  0x8012,   /* U+2F7E */
  0x8033,   /* U+2F7F */
  0x807F,   /* U+2F80 */
  0x8089,   /* U+2F81 */
  0x81E3,   /* U+2F82 */
  0x81EA,   /* U+2F83 */
  0x81F3,   /* U+2F84 */
  0x81FC,   /* U+2F85 */
  0x820C,   /* U+2F86 */
  0x821B,   /* U+2F87 */
  0x821F,   /* U+2F88 */
  0x826E,   /* U+2F89 */
  0x8272,   /* U+2F8A */
  0x8278,   /* U+2F8B */
  0x864D,   /* U+2F8C */
  0x866B,   /* U+2F8D */
  0x8840,   /* U+2F8E */
  0x884C,   /* U+2F8F */
  0x8863,   /* U+2F90 */
  0x897E,   /* U+2F91 */
  0x898B,   /* U+2F92 */
  0x89D2,   /* U+2F93 */
  0x8A00,   /* U+2F94 */
  0x8C37,   /* U+2F95 */
  0x8C46,   /* U+2F96 */
  0x8C55,   /* U+2F97 */
  0x8C78,   /* U+2F98 */
  0x8C9D,   /* U+2F99 */
  0x8D64,   /* U+2F9A */
  0x8D70,   /* U+2F9B */
  0x8DB3,   /* U+2F9C */
  0x8EAB,   /* U+2F9D */
  0x8ECA,   /* U+2F9E */
  0x8F9B,   /* U+2F9F */
  0x8FB0,   /* U+2FA0 */
  0x8FB5,   /* U+2FA1 */
  0x9091,   /* U+2FA2 */
  0x9149,   /* U+2FA3 */
  0x91C6,   /* U+2FA4 */
  0x91CC,   /* U+2FA5 */
  0x91D1,   /* U+2FA6 */
  0x9577,   /* U+2FA7 */
  0x9580,   /* U+2FA8 */
  0x961C,   /* U+2FA9 */
  0x96B6,   /* U+2FAA */
  0x96B9,   /* U+2FAB */
  0x96E8,   /* U+2FAC */
  0x9751,   /* U+2FAD */
  0x975E,   /* U+2FAE */
  0x9762,   /* U+2FAF */
  0x9769,   /* U+2FB0 */
  0x97CB,   /* U+2FB1 */
  0x97ED,   /* U+2FB2 */
  0x97F3,   /* U+2FB3 */
  0x9801,   /* U+2FB4 */
  0x98A8,   /* U+2FB5 */
  0x98DB,   /* U+2FB6 */
  0x98DF,   /* U+2FB7 */
  0x9996,   /* U+2FB8 */
  0x9999,   /* U+2FB9 */
  0x99AC,   /* U+2FBA */
  0x9AA8,   /* U+2FBB */
  0x9AD8,   /* U+2FBC */
  0x9ADF,   /* U+2FBD */
  0x9B25,   /* U+2FBE */
  0x9B2F,   /* U+2FBF */
  0x9B32,   /* U+2FC0 */
  0x9B3C,   /* U+2FC1 */
  0x9B5A,   /* U+2FC2 */
  0x9CE5,   /* U+2FC3 */
  0x9E75,   /* U+2FC4 */
  0x9E7F,   /* U+2FC5 */
  0x9EA5,   /* U+2FC6 */
  0x9EBB,   /* U+2FC7 */
  0x9EC3,   /* U+2FC8 */
  0x9ECD,   /* U+2FC9 */
  0x9ED1,   /* U+2FCA */
  0x9EF9,   /* U+2FCB */
  0x9EFD,   /* U+2FCC */
  0x9F0E,   /* U+2FCD */
  0x9F13,   /* U+2FCE */
  0x9F20,   /* U+2FCF */
  0x9F3B,   /* U+2FD0 */
  0x9F4A,   /* U+2FD1 */
  0x9F52,   /* U+2FD2 */
  0x9F8D,   /* U+2FD3 */
  0x9F9C,   /* U+2FD4 */
  0x9FA0,   /* U+2FD5 */
  0xAFD6,   /* U+2FD6 */
  0xAFD7,   /* U+2FD7 */
  0xAFD8,   /* U+2FD8 */
  0xAFD9,   /* U+2FD9 */
  0xAFDA,   /* U+2FDA */
  0xAFDB,   /* U+2FDB */
  0xAFDC,   /* U+2FDC */
  0xAFDD,   /* U+2FDD */
  0xAFDE,   /* U+2FDE */
  0xAFDF,   /* U+2FDF */
  0xAFE0,   /* U+2FE0 */
  0xAFE1,   /* U+2FE1 */
  0xAFE2,   /* U+2FE2 */
  0xAFE3,   /* U+2FE3 */
  0xAFE4,   /* U+2FE4 */
  0xAFE5,   /* U+2FE5 */
  0xAFE6,   /* U+2FE6 */
  0xAFE7,   /* U+2FE7 */
  0xAFE8,   /* U+2FE8 */
  0xAFE9,   /* U+2FE9 */
  0xAFEA,   /* U+2FEA */
  0xAFEB,   /* U+2FEB */
  0xAFEC,   /* U+2FEC */
  0xAFED,   /* U+2FED */
  0xAFEE,   /* U+2FEE */
  0xAFEF,   /* U+2FEF */
  0x0000,   /* U+2FF0 */
  0x0000,   /* U+2FF1 */
  0x0000,   /* U+2FF2 */
  0x0000,   /* U+2FF3 */
  0x0000,   /* U+2FF4 */
  0x0000,   /* U+2FF5 */
  0x0000,   /* U+2FF6 */
  0x0000,   /* U+2FF7 */
  0x0000,   /* U+2FF8 */
  0x0000,   /* U+2FF9 */
  0x0000,   /* U+2FFA */
  0x0000,   /* U+2FFB */
  0xAFFC,   /* U+2FFC */
  0xAFFD,   /* U+2FFD */
  0xAFFE,   /* U+2FFE */
  0xAFFF,   /* U+2FFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2F00 */
  0x0000,   /* U+2F01 */
  0x0000,   /* U+2F02 */
  0x0000,   /* U+2F03 */
  0x0000,   /* U+2F04 */
  0x0000,   /* U+2F05 */
  0x0000,   /* U+2F06 */
  0x0000,   /* U+2F07 */
  0x0000,   /* U+2F08 */
  0x0000,   /* U+2F09 */
  0x0000,   /* U+2F0A */
  0x0000,   /* U+2F0B */
  0x0000,   /* U+2F0C */
  0x0000,   /* U+2F0D */
  0x0000,   /* U+2F0E */
  0x0000,   /* U+2F0F */
  0x0000,   /* U+2F10 */
  0x0000,   /* U+2F11 */
  0x0000,   /* U+2F12 */
  0x0000,   /* U+2F13 */
  0x0000,   /* U+2F14 */
  0x0000,   /* U+2F15 */
  0x0000,   /* U+2F16 */
  0x0000,   /* U+2F17 */
  0x0000,   /* U+2F18 */
  0x0000,   /* U+2F19 */
  0x0000,   /* U+2F1A */
  0x0000,   /* U+2F1B */
  0x0000,   /* U+2F1C */
  0x0000,   /* U+2F1D */
  0x0000,   /* U+2F1E */
  0x0000,   /* U+2F1F */
  0x0000,   /* U+2F20 */
  0x0000,   /* U+2F21 */
  0x0000,   /* U+2F22 */
  0x0000,   /* U+2F23 */
  0x0000,   /* U+2F24 */
  0x0000,   /* U+2F25 */
  0x0000,   /* U+2F26 */
  0x0000,   /* U+2F27 */
  0x0000,   /* U+2F28 */
  0x0000,   /* U+2F29 */
  0x0000,   /* U+2F2A */
  0x0000,   /* U+2F2B */
  0x0000,   /* U+2F2C */
  0x0000,   /* U+2F2D */
  0x0000,   /* U+2F2E */
  0x0000,   /* U+2F2F */
  0x0000,   /* U+2F30 */
  0x0000,   /* U+2F31 */
  0x0000,   /* U+2F32 */
  0x0000,   /* U+2F33 */
  0x0000,   /* U+2F34 */
  0x0000,   /* U+2F35 */
  0x0000,   /* U+2F36 */
  0x0000,   /* U+2F37 */
  0x0000,   /* U+2F38 */
  0x0000,   /* U+2F39 */
  0x0000,   /* U+2F3A */
  0x0000,   /* U+2F3B */
  0x0000,   /* U+2F3C */
  0x0000,   /* U+2F3D */
  0x0000,   /* U+2F3E */
  0x0000,   /* U+2F3F */
  0x0000,   /* U+2F40 */
  0x0000,   /* U+2F41 */
  0x0000,   /* U+2F42 */
  0x0000,   /* U+2F43 */
  0x0000,   /* U+2F44 */
  0x0000,   /* U+2F45 */
  0x0000,   /* U+2F46 */
  0x0000,   /* U+2F47 */
  0x0000,   /* U+2F48 */
  0x0000,   /* U+2F49 */
  0x0000,   /* U+2F4A */
  0x0000,   /* U+2F4B */
  0x0000,   /* U+2F4C */
  0x0000,   /* U+2F4D */
  0x0000,   /* U+2F4E */
  0x0000,   /* U+2F4F */
  0x0000,   /* U+2F50 */
  0x0000,   /* U+2F51 */
  0x0000,   /* U+2F52 */
  0x0000,   /* U+2F53 */
  0x0000,   /* U+2F54 */
  0x0000,   /* U+2F55 */
  0x0000,   /* U+2F56 */
  0x0000,   /* U+2F57 */
  0x0000,   /* U+2F58 */
  0x0000,   /* U+2F59 */
  0x0000,   /* U+2F5A */
  0x0000,   /* U+2F5B */
  0x0000,   /* U+2F5C */
  0x0000,   /* U+2F5D */
  0x0000,   /* U+2F5E */
  0x0000,   /* U+2F5F */
  0x0000,   /* U+2F60 */
  0x0000,   /* U+2F61 */
  0x0000,   /* U+2F62 */
  0x0000,   /* U+2F63 */
  0x0000,   /* U+2F64 */
  0x0000,   /* U+2F65 */
  0x0000,   /* U+2F66 */
  0x0000,   /* U+2F67 */
  0x0000,   /* U+2F68 */
  0x0000,   /* U+2F69 */
  0x0000,   /* U+2F6A */
  0x0000,   /* U+2F6B */
  0x0000,   /* U+2F6C */
  0x0000,   /* U+2F6D */
  0x0000,   /* U+2F6E */
  0x0000,   /* U+2F6F */
  0x0000,   /* U+2F70 */
  0x0000,   /* U+2F71 */
  0x0000,   /* U+2F72 */
  0x0000,   /* U+2F73 */
  0x0000,   /* U+2F74 */
  0x0000,   /* U+2F75 */
  0x0000,   /* U+2F76 */
  0x0000,   /* U+2F77 */
  0x0000,   /* U+2F78 */
  0x0000,   /* U+2F79 */
  0x0000,   /* U+2F7A */
  0x0000,   /* U+2F7B */
  0x0000,   /* U+2F7C */
  0x0000,   /* U+2F7D */
  0x0000,   /* U+2F7E */
  0x0000,   /* U+2F7F */
  0x0000,   /* U+2F80 */
  0x0000,   /* U+2F81 */
  0x0000,   /* U+2F82 */
  0x0000,   /* U+2F83 */
  0x0000,   /* U+2F84 */
  0x0000,   /* U+2F85 */
  0x0000,   /* U+2F86 */
  0x0000,   /* U+2F87 */
  0x0000,   /* U+2F88 */
  0x0000,   /* U+2F89 */
  0x0000,   /* U+2F8A */
  0x0000,   /* U+2F8B */
  0x0000,   /* U+2F8C */
  0x0000,   /* U+2F8D */
  0x0000,   /* U+2F8E */
  0x0000,   /* U+2F8F */
  0x0000,   /* U+2F90 */
  0x0000,   /* U+2F91 */
  0x0000,   /* U+2F92 */
  0x0000,   /* U+2F93 */
  0x0000,   /* U+2F94 */
  0x0000,   /* U+2F95 */
  0x0000,   /* U+2F96 */
  0x0000,   /* U+2F97 */
  0x0000,   /* U+2F98 */
  0x0000,   /* U+2F99 */
  0x0000,   /* U+2F9A */
  0x0000,   /* U+2F9B */
  0x0000,   /* U+2F9C */
  0x0000,   /* U+2F9D */
  0x0000,   /* U+2F9E */
  0x0000,   /* U+2F9F */
  0x0000,   /* U+2FA0 */
  0x0000,   /* U+2FA1 */
  0x0000,   /* U+2FA2 */
  0x0000,   /* U+2FA3 */
  0x0000,   /* U+2FA4 */
  0x0000,   /* U+2FA5 */
  0x0000,   /* U+2FA6 */
  0x0000,   /* U+2FA7 */
  0x0000,   /* U+2FA8 */
  0x0000,   /* U+2FA9 */
  0x0000,   /* U+2FAA */
  0x0000,   /* U+2FAB */
  0x0000,   /* U+2FAC */
  0x0000,   /* U+2FAD */
  0x0000,   /* U+2FAE */
  0x0000,   /* U+2FAF */
  0x0000,   /* U+2FB0 */
  0x0000,   /* U+2FB1 */
  0x0000,   /* U+2FB2 */
  0x0000,   /* U+2FB3 */
  0x0000,   /* U+2FB4 */
  0x0000,   /* U+2FB5 */
  0x0000,   /* U+2FB6 */
  0x0000,   /* U+2FB7 */
  0x0000,   /* U+2FB8 */
  0x0000,   /* U+2FB9 */
  0x0000,   /* U+2FBA */
  0x0000,   /* U+2FBB */
  0x0000,   /* U+2FBC */
  0x0000,   /* U+2FBD */
  0x0000,   /* U+2FBE */
  0x0000,   /* U+2FBF */
  0x0000,   /* U+2FC0 */
  0x0000,   /* U+2FC1 */
  0x0000,   /* U+2FC2 */
  0x0000,   /* U+2FC3 */
  0x0000,   /* U+2FC4 */
  0x0000,   /* U+2FC5 */
  0x0000,   /* U+2FC6 */
  0x0000,   /* U+2FC7 */
  0x0000,   /* U+2FC8 */
  0x0000,   /* U+2FC9 */
  0x0000,   /* U+2FCA */
  0x0000,   /* U+2FCB */
  0x0000,   /* U+2FCC */
  0x0000,   /* U+2FCD */
  0x0000,   /* U+2FCE */
  0x0000,   /* U+2FCF */
  0x0000,   /* U+2FD0 */
  0x0000,   /* U+2FD1 */
  0x0000,   /* U+2FD2 */
  0x0000,   /* U+2FD3 */
  0x0000,   /* U+2FD4 */
  0x0000,   /* U+2FD5 */
  0x0000,   /* U+2FD6 */
  0x0000,   /* U+2FD7 */
  0x0000,   /* U+2FD8 */
  0x0000,   /* U+2FD9 */
  0x0000,   /* U+2FDA */
  0x0000,   /* U+2FDB */
  0x0000,   /* U+2FDC */
  0x0000,   /* U+2FDD */
  0x0000,   /* U+2FDE */
  0x0000,   /* U+2FDF */
  0x0000,   /* U+2FE0 */
  0x0000,   /* U+2FE1 */
  0x0000,   /* U+2FE2 */
  0x0000,   /* U+2FE3 */
  0x0000,   /* U+2FE4 */
  0x0000,   /* U+2FE5 */
  0x0000,   /* U+2FE6 */
  0x0000,   /* U+2FE7 */
  0x0000,   /* U+2FE8 */
  0x0000,   /* U+2FE9 */
  0x0000,   /* U+2FEA */
  0x0000,   /* U+2FEB */
  0x0000,   /* U+2FEC */
  0x0000,   /* U+2FED */
  0x0000,   /* U+2FEE */
  0x0000,   /* U+2FEF */
  0x0000,   /* U+2FF0 */
  0x0000,   /* U+2FF1 */
  0x0000,   /* U+2FF2 */
  0x0000,   /* U+2FF3 */
  0x0000,   /* U+2FF4 */
  0x0000,   /* U+2FF5 */
  0x0000,   /* U+2FF6 */
  0x0000,   /* U+2FF7 */
  0x0000,   /* U+2FF8 */
  0x0000,   /* U+2FF9 */
  0x0000,   /* U+2FFA */
  0x0000,   /* U+2FFB */
  0x0000,   /* U+2FFC */
  0x0000,   /* U+2FFD */
  0x0000,   /* U+2FFE */
  0x0000,   /* U+2FFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2F00 */
  0x0000,   /* U+2F01 */
  0x0000,   /* U+2F02 */
  0x0000,   /* U+2F03 */
  0x0000,   /* U+2F04 */
  0x0000,   /* U+2F05 */
  0x0000,   /* U+2F06 */
  0x0000,   /* U+2F07 */
  0x0000,   /* U+2F08 */
  0x0000,   /* U+2F09 */
  0x0000,   /* U+2F0A */
  0x0000,   /* U+2F0B */
  0x0000,   /* U+2F0C */
  0x0000,   /* U+2F0D */
  0x0000,   /* U+2F0E */
  0x0000,   /* U+2F0F */
  0x0000,   /* U+2F10 */
  0x0000,   /* U+2F11 */
  0x0000,   /* U+2F12 */
  0x0000,   /* U+2F13 */
  0x0000,   /* U+2F14 */
  0x0000,   /* U+2F15 */
  0x0000,   /* U+2F16 */
  0x0000,   /* U+2F17 */
  0x0000,   /* U+2F18 */
  0x0000,   /* U+2F19 */
  0x0000,   /* U+2F1A */
  0x0000,   /* U+2F1B */
  0x0000,   /* U+2F1C */
  0x0000,   /* U+2F1D */
  0x0000,   /* U+2F1E */
  0x0000,   /* U+2F1F */
  0x0000,   /* U+2F20 */
  0x0000,   /* U+2F21 */
  0x0000,   /* U+2F22 */
  0x0000,   /* U+2F23 */
  0x0000,   /* U+2F24 */
  0x0000,   /* U+2F25 */
  0x0000,   /* U+2F26 */
  0x0000,   /* U+2F27 */
  0x0000,   /* U+2F28 */
  0x0000,   /* U+2F29 */
  0x0000,   /* U+2F2A */
  0x0000,   /* U+2F2B */
  0x0000,   /* U+2F2C */
  0x0000,   /* U+2F2D */
  0x0000,   /* U+2F2E */
  0x0000,   /* U+2F2F */
  0x0000,   /* U+2F30 */
  0x0000,   /* U+2F31 */
  0x0000,   /* U+2F32 */
  0x0000,   /* U+2F33 */
  0x0000,   /* U+2F34 */
  0x0000,   /* U+2F35 */
  0x0000,   /* U+2F36 */
  0x0000,   /* U+2F37 */
  0x0000,   /* U+2F38 */
  0x0000,   /* U+2F39 */
  0x0000,   /* U+2F3A */
  0x0000,   /* U+2F3B */
  0x0000,   /* U+2F3C */
  0x0000,   /* U+2F3D */
  0x0000,   /* U+2F3E */
  0x0000,   /* U+2F3F */
  0x0000,   /* U+2F40 */
  0x0000,   /* U+2F41 */
  0x0000,   /* U+2F42 */
  0x0000,   /* U+2F43 */
  0x0000,   /* U+2F44 */
  0x0000,   /* U+2F45 */
  0x0000,   /* U+2F46 */
  0x0000,   /* U+2F47 */
  0x0000,   /* U+2F48 */
  0x0000,   /* U+2F49 */
  0x0000,   /* U+2F4A */
  0x0000,   /* U+2F4B */
  0x0000,   /* U+2F4C */
  0x0000,   /* U+2F4D */
  0x0000,   /* U+2F4E */
  0x0000,   /* U+2F4F */
  0x0000,   /* U+2F50 */
  0x0000,   /* U+2F51 */
  0x0000,   /* U+2F52 */
  0x0000,   /* U+2F53 */
  0x0000,   /* U+2F54 */
  0x0000,   /* U+2F55 */
  0x0000,   /* U+2F56 */
  0x0000,   /* U+2F57 */
  0x0000,   /* U+2F58 */
  0x0000,   /* U+2F59 */
  0x0000,   /* U+2F5A */
  0x0000,   /* U+2F5B */
  0x0000,   /* U+2F5C */
  0x0000,   /* U+2F5D */
  0x0000,   /* U+2F5E */
  0x0000,   /* U+2F5F */
  0x0000,   /* U+2F60 */
  0x0000,   /* U+2F61 */
  0x0000,   /* U+2F62 */
  0x0000,   /* U+2F63 */
  0x0000,   /* U+2F64 */
  0x0000,   /* U+2F65 */
  0x0000,   /* U+2F66 */
  0x0000,   /* U+2F67 */
  0x0000,   /* U+2F68 */
  0x0000,   /* U+2F69 */
  0x0000,   /* U+2F6A */
  0x0000,   /* U+2F6B */
  0x0000,   /* U+2F6C */
  0x0000,   /* U+2F6D */
  0x0000,   /* U+2F6E */
  0x0000,   /* U+2F6F */
  0x0000,   /* U+2F70 */
  0x0000,   /* U+2F71 */
  0x0000,   /* U+2F72 */
  0x0000,   /* U+2F73 */
  0x0000,   /* U+2F74 */
  0x0000,   /* U+2F75 */
  0x0000,   /* U+2F76 */
  0x0000,   /* U+2F77 */
  0x0000,   /* U+2F78 */
  0x0000,   /* U+2F79 */
  0x0000,   /* U+2F7A */
  0x0000,   /* U+2F7B */
  0x0000,   /* U+2F7C */
  0x0000,   /* U+2F7D */
  0x0000,   /* U+2F7E */
  0x0000,   /* U+2F7F */
  0x0000,   /* U+2F80 */
  0x0000,   /* U+2F81 */
  0x0000,   /* U+2F82 */
  0x0000,   /* U+2F83 */
  0x0000,   /* U+2F84 */
  0x0000,   /* U+2F85 */
  0x0000,   /* U+2F86 */
  0x0000,   /* U+2F87 */
  0x0000,   /* U+2F88 */
  0x0000,   /* U+2F89 */
  0x0000,   /* U+2F8A */
  0x0000,   /* U+2F8B */
  0x0000,   /* U+2F8C */
  0x0000,   /* U+2F8D */
  0x0000,   /* U+2F8E */
  0x0000,   /* U+2F8F */
  0x0000,   /* U+2F90 */
  0x0000,   /* U+2F91 */
  0x0000,   /* U+2F92 */
  0x0000,   /* U+2F93 */
  0x0000,   /* U+2F94 */
  0x0000,   /* U+2F95 */
  0x0000,   /* U+2F96 */
  0x0000,   /* U+2F97 */
  0x0000,   /* U+2F98 */
  0x0000,   /* U+2F99 */
  0x0000,   /* U+2F9A */
  0x0000,   /* U+2F9B */
  0x0000,   /* U+2F9C */
  0x0000,   /* U+2F9D */
  0x0000,   /* U+2F9E */
  0x0000,   /* U+2F9F */
  0x0000,   /* U+2FA0 */
  0x0000,   /* U+2FA1 */
  0x0000,   /* U+2FA2 */
  0x0000,   /* U+2FA3 */
  0x0000,   /* U+2FA4 */
  0x0000,   /* U+2FA5 */
  0x0000,   /* U+2FA6 */
  0x0000,   /* U+2FA7 */
  0x0000,   /* U+2FA8 */
  0x0000,   /* U+2FA9 */
  0x0000,   /* U+2FAA */
  0x0000,   /* U+2FAB */
  0x0000,   /* U+2FAC */
  0x0000,   /* U+2FAD */
  0x0000,   /* U+2FAE */
  0x0000,   /* U+2FAF */
  0x0000,   /* U+2FB0 */
  0x0000,   /* U+2FB1 */
  0x0000,   /* U+2FB2 */
  0x0000,   /* U+2FB3 */
  0x0000,   /* U+2FB4 */
  0x0000,   /* U+2FB5 */
  0x0000,   /* U+2FB6 */
  0x0000,   /* U+2FB7 */
  0x0000,   /* U+2FB8 */
  0x0000,   /* U+2FB9 */
  0x0000,   /* U+2FBA */
  0x0000,   /* U+2FBB */
  0x0000,   /* U+2FBC */
  0x0000,   /* U+2FBD */
  0x0000,   /* U+2FBE */
  0x0000,   /* U+2FBF */
  0x0000,   /* U+2FC0 */
  0x0000,   /* U+2FC1 */
  0x0000,   /* U+2FC2 */
  0x0000,   /* U+2FC3 */
  0x0000,   /* U+2FC4 */
  0x0000,   /* U+2FC5 */
  0x0000,   /* U+2FC6 */
  0x0000,   /* U+2FC7 */
  0x0000,   /* U+2FC8 */
  0x0000,   /* U+2FC9 */
  0x0000,   /* U+2FCA */
  0x0000,   /* U+2FCB */
  0x0000,   /* U+2FCC */
  0x0000,   /* U+2FCD */
  0x0000,   /* U+2FCE */
  0x0000,   /* U+2FCF */
  0x0000,   /* U+2FD0 */
  0x0000,   /* U+2FD1 */
  0x0000,   /* U+2FD2 */
  0x0000,   /* U+2FD3 */
  0x0000,   /* U+2FD4 */
  0x0000,   /* U+2FD5 */
  0x0000,   /* U+2FD6 */
  0x0000,   /* U+2FD7 */
  0x0000,   /* U+2FD8 */
  0x0000,   /* U+2FD9 */
  0x0000,   /* U+2FDA */
  0x0000,   /* U+2FDB */
  0x0000,   /* U+2FDC */
  0x0000,   /* U+2FDD */
  0x0000,   /* U+2FDE */
  0x0000,   /* U+2FDF */
  0x0000,   /* U+2FE0 */
  0x0000,   /* U+2FE1 */
  0x0000,   /* U+2FE2 */
  0x0000,   /* U+2FE3 */
  0x0000,   /* U+2FE4 */
  0x0000,   /* U+2FE5 */
  0x0000,   /* U+2FE6 */
  0x0000,   /* U+2FE7 */
  0x0000,   /* U+2FE8 */
  0x0000,   /* U+2FE9 */
  0x0000,   /* U+2FEA */
  0x0000,   /* U+2FEB */
  0x0000,   /* U+2FEC */
  0x0000,   /* U+2FED */
  0x0000,   /* U+2FEE */
  0x0000,   /* U+2FEF */
  0x0000,   /* U+2FF0 */
  0x0000,   /* U+2FF1 */
  0x0000,   /* U+2FF2 */
  0x0000,   /* U+2FF3 */
  0x0000,   /* U+2FF4 */
  0x0000,   /* U+2FF5 */
  0x0000,   /* U+2FF6 */
  0x0000,   /* U+2FF7 */
  0x0000,   /* U+2FF8 */
  0x0000,   /* U+2FF9 */
  0x0000,   /* U+2FFA */
  0x0000,   /* U+2FFB */
  0x0000,   /* U+2FFC */
  0x0000,   /* U+2FFD */
  0x0000,   /* U+2FFE */
  0x0000,   /* U+2FFF */
};

uint16 uca900_p030[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 2, 1, 1, 1, 2, 2, 2, 1, 2, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 
  2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 
  2, 1, 2, 1, 1, 2, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 2, 2, 1, 2, 2, 1, 2, 2, 1, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 2, 2, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 
  2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 
  2, 1, 2, 1, 1, 2, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 2, 2, 1, 2, 2, 1, 2, 2, 1, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0209,   /* U+3000 */
  0x0231,   /* U+3001 */
  0x028A,   /* U+3002 */
  0x03AC,   /* U+3003 */
  0x1A8B,   /* U+3004 */
  0x1C07,   /* U+3005 */
  0x3D66,   /* U+3006 */
  0x1C3D,   /* U+3007 */
  0x036F,   /* U+3008 */
  0x0370,   /* U+3009 */
  0x0371,   /* U+300A */
  0x0372,   /* U+300B */
  0x0373,   /* U+300C */
  0x0374,   /* U+300D */
  0x0375,   /* U+300E */
  0x0376,   /* U+300F */
  0x0377,   /* U+3010 */
  0x0378,   /* U+3011 */
  0x1A8C,   /* U+3012 */
  0x1A8D,   /* U+3013 */
  0x0379,   /* U+3014 */
  0x037A,   /* U+3015 */
  0x037B,   /* U+3016 */
  0x037C,   /* U+3017 */
  0x037D,   /* U+3018 */
  0x037E,   /* U+3019 */
  0x037F,   /* U+301A */
  0x0380,   /* U+301B */
  0x021E,   /* U+301C */
  0x0312,   /* U+301D */
  0x0313,   /* U+301E */
  0x0314,   /* U+301F */
  0x1A8E,   /* U+3020 */
  0x1C3E,   /* U+3021 */
  0x1C3F,   /* U+3022 */
  0x1C40,   /* U+3023 */
  0x1C41,   /* U+3024 */
  0x1C42,   /* U+3025 */
  0x1C43,   /* U+3026 */
  0x1C44,   /* U+3027 */
  0x1C45,   /* U+3028 */
  0x1C46,   /* U+3029 */
  0x0000,   /* U+302A */
  0x0000,   /* U+302B */
  0x0000,   /* U+302C */
  0x0000,   /* U+302D */
  0x0000,   /* U+302E */
  0x0000,   /* U+302F */
  0x021F,   /* U+3030 */
  0x1C0A,   /* U+3031 */
  0x1C0A,   /* U+3032 */
  0x1C0B,   /* U+3033 */
  0x1C0B,   /* U+3034 */
  0x1C0C,   /* U+3035 */
  0x1A8C,   /* U+3036 */
  0x1A8F,   /* U+3037 */
  0xFB40,   /* U+3038 */
  0xFB40,   /* U+3039 */
  0xFB40,   /* U+303A */
  0x1C08,   /* U+303B */
  0x3D79,   /* U+303C */
  0x03AD,   /* U+303D */
  0x1A90,   /* U+303E */
  0x1A91,   /* U+303F */
  0xFBC0,   /* U+3040 */
  0x3D5A,   /* U+3041 */
  0x3D5A,   /* U+3042 */
  0x3D5B,   /* U+3043 */
  0x3D5B,   /* U+3044 */
  0x3D5C,   /* U+3045 */
  0x3D5C,   /* U+3046 */
  0x3D5E,   /* U+3047 */
  0x3D5E,   /* U+3048 */
  0x3D5F,   /* U+3049 */
  0x3D5F,   /* U+304A */
  0x3D60,   /* U+304B */
  0x3D60,   /* U+304C */
  0x3D61,   /* U+304D */
  0x3D61,   /* U+304E */
  0x3D62,   /* U+304F */
  0x3D62,   /* U+3050 */
  0x3D63,   /* U+3051 */
  0x3D63,   /* U+3052 */
  0x3D64,   /* U+3053 */
  0x3D64,   /* U+3054 */
  0x3D65,   /* U+3055 */
  0x3D65,   /* U+3056 */
  0x3D66,   /* U+3057 */
  0x3D66,   /* U+3058 */
  0x3D67,   /* U+3059 */
  0x3D67,   /* U+305A */
  0x3D68,   /* U+305B */
  0x3D68,   /* U+305C */
  0x3D69,   /* U+305D */
  0x3D69,   /* U+305E */
  0x3D6A,   /* U+305F */
  0x3D6A,   /* U+3060 */
  0x3D6B,   /* U+3061 */
  0x3D6B,   /* U+3062 */
  0x3D6C,   /* U+3063 */
  0x3D6C,   /* U+3064 */
  0x3D6C,   /* U+3065 */
  0x3D6D,   /* U+3066 */
  0x3D6D,   /* U+3067 */
  0x3D6E,   /* U+3068 */
  0x3D6E,   /* U+3069 */
  0x3D6F,   /* U+306A */
  0x3D70,   /* U+306B */
  0x3D71,   /* U+306C */
  0x3D72,   /* U+306D */
  0x3D73,   /* U+306E */
  0x3D74,   /* U+306F */
  0x3D74,   /* U+3070 */
  0x3D74,   /* U+3071 */
  0x3D75,   /* U+3072 */
  0x3D75,   /* U+3073 */
  0x3D75,   /* U+3074 */
  0x3D76,   /* U+3075 */
  0x3D76,   /* U+3076 */
  0x3D76,   /* U+3077 */
  0x3D77,   /* U+3078 */
  0x3D77,   /* U+3079 */
  0x3D77,   /* U+307A */
  0x3D78,   /* U+307B */
  0x3D78,   /* U+307C */
  0x3D78,   /* U+307D */
  0x3D79,   /* U+307E */
  0x3D7A,   /* U+307F */
  0x3D7B,   /* U+3080 */
  0x3D7C,   /* U+3081 */
  0x3D7D,   /* U+3082 */
  0x3D7E,   /* U+3083 */
  0x3D7E,   /* U+3084 */
  0x3D7F,   /* U+3085 */
  0x3D7F,   /* U+3086 */
  0x3D81,   /* U+3087 */
  0x3D81,   /* U+3088 */
  0x3D82,   /* U+3089 */
  0x3D83,   /* U+308A */
  0x3D84,   /* U+308B */
  0x3D85,   /* U+308C */
  0x3D86,   /* U+308D */
  0x3D87,   /* U+308E */
  0x3D87,   /* U+308F */
  0x3D88,   /* U+3090 */
  0x3D89,   /* U+3091 */
  0x3D8A,   /* U+3092 */
  0x3D8B,   /* U+3093 */
  0x3D5C,   /* U+3094 */
  0x3D60,   /* U+3095 */
  0x3D63,   /* U+3096 */
  0xFBC0,   /* U+3097 */
  0xFBC0,   /* U+3098 */
  0x0000,   /* U+3099 */
  0x0000,   /* U+309A */
  0x0491,   /* U+309B */
  0x0492,   /* U+309C */
  0x1C0D,   /* U+309D */
  0x1C0D,   /* U+309E */
  0x3D81,   /* U+309F */
  0x0220,   /* U+30A0 */
  0x3D5A,   /* U+30A1 */
  0x3D5A,   /* U+30A2 */
  0x3D5B,   /* U+30A3 */
  0x3D5B,   /* U+30A4 */
  0x3D5C,   /* U+30A5 */
  0x3D5C,   /* U+30A6 */
  0x3D5E,   /* U+30A7 */
  0x3D5E,   /* U+30A8 */
  0x3D5F,   /* U+30A9 */
  0x3D5F,   /* U+30AA */
  0x3D60,   /* U+30AB */
  0x3D60,   /* U+30AC */
  0x3D61,   /* U+30AD */
  0x3D61,   /* U+30AE */
  0x3D62,   /* U+30AF */
  0x3D62,   /* U+30B0 */
  0x3D63,   /* U+30B1 */
  0x3D63,   /* U+30B2 */
  0x3D64,   /* U+30B3 */
  0x3D64,   /* U+30B4 */
  0x3D65,   /* U+30B5 */
  0x3D65,   /* U+30B6 */
  0x3D66,   /* U+30B7 */
  0x3D66,   /* U+30B8 */
  0x3D67,   /* U+30B9 */
  0x3D67,   /* U+30BA */
  0x3D68,   /* U+30BB */
  0x3D68,   /* U+30BC */
  0x3D69,   /* U+30BD */
  0x3D69,   /* U+30BE */
  0x3D6A,   /* U+30BF */
  0x3D6A,   /* U+30C0 */
  0x3D6B,   /* U+30C1 */
  0x3D6B,   /* U+30C2 */
  0x3D6C,   /* U+30C3 */
  0x3D6C,   /* U+30C4 */
  0x3D6C,   /* U+30C5 */
  0x3D6D,   /* U+30C6 */
  0x3D6D,   /* U+30C7 */
  0x3D6E,   /* U+30C8 */
  0x3D6E,   /* U+30C9 */
  0x3D6F,   /* U+30CA */
  0x3D70,   /* U+30CB */
  0x3D71,   /* U+30CC */
  0x3D72,   /* U+30CD */
  0x3D73,   /* U+30CE */
  0x3D74,   /* U+30CF */
  0x3D74,   /* U+30D0 */
  0x3D74,   /* U+30D1 */
  0x3D75,   /* U+30D2 */
  0x3D75,   /* U+30D3 */
  0x3D75,   /* U+30D4 */
  0x3D76,   /* U+30D5 */
  0x3D76,   /* U+30D6 */
  0x3D76,   /* U+30D7 */
  0x3D77,   /* U+30D8 */
  0x3D77,   /* U+30D9 */
  0x3D77,   /* U+30DA */
  0x3D78,   /* U+30DB */
  0x3D78,   /* U+30DC */
  0x3D78,   /* U+30DD */
  0x3D79,   /* U+30DE */
  0x3D7A,   /* U+30DF */
  0x3D7B,   /* U+30E0 */
  0x3D7C,   /* U+30E1 */
  0x3D7D,   /* U+30E2 */
  0x3D7E,   /* U+30E3 */
  0x3D7E,   /* U+30E4 */
  0x3D7F,   /* U+30E5 */
  0x3D7F,   /* U+30E6 */
  0x3D81,   /* U+30E7 */
  0x3D81,   /* U+30E8 */
  0x3D82,   /* U+30E9 */
  0x3D83,   /* U+30EA */
  0x3D84,   /* U+30EB */
  0x3D85,   /* U+30EC */
  0x3D86,   /* U+30ED */
  0x3D87,   /* U+30EE */
  0x3D87,   /* U+30EF */
  0x3D88,   /* U+30F0 */
  0x3D89,   /* U+30F1 */
  0x3D8A,   /* U+30F2 */
  0x3D8B,   /* U+30F3 */
  0x3D5C,   /* U+30F4 */
  0x3D60,   /* U+30F5 */
  0x3D63,   /* U+30F6 */
  0x3D87,   /* U+30F7 */
  0x3D88,   /* U+30F8 */
  0x3D89,   /* U+30F9 */
  0x3D8A,   /* U+30FA */
  0x0221,   /* U+30FB */
  0x1C0E,   /* U+30FC */
  0x1C0F,   /* U+30FD */
  0x1C0F,   /* U+30FE */
  0x3D64,   /* U+30FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+3000 */
  0x0020,   /* U+3001 */
  0x0020,   /* U+3002 */
  0x0020,   /* U+3003 */
  0x0020,   /* U+3004 */
  0x0020,   /* U+3005 */
  0x0020,   /* U+3006 */
  0x0020,   /* U+3007 */
  0x0020,   /* U+3008 */
  0x0020,   /* U+3009 */
  0x0020,   /* U+300A */
  0x0020,   /* U+300B */
  0x0020,   /* U+300C */
  0x0020,   /* U+300D */
  0x0020,   /* U+300E */
  0x0020,   /* U+300F */
  0x0020,   /* U+3010 */
  0x0020,   /* U+3011 */
  0x0020,   /* U+3012 */
  0x0020,   /* U+3013 */
  0x0020,   /* U+3014 */
  0x0020,   /* U+3015 */
  0x0020,   /* U+3016 */
  0x0020,   /* U+3017 */
  0x0020,   /* U+3018 */
  0x0020,   /* U+3019 */
  0x0020,   /* U+301A */
  0x0020,   /* U+301B */
  0x0020,   /* U+301C */
  0x0020,   /* U+301D */
  0x0020,   /* U+301E */
  0x0020,   /* U+301F */
  0x0020,   /* U+3020 */
  0x0020,   /* U+3021 */
  0x0020,   /* U+3022 */
  0x0020,   /* U+3023 */
  0x0020,   /* U+3024 */
  0x0020,   /* U+3025 */
  0x0020,   /* U+3026 */
  0x0020,   /* U+3027 */
  0x0020,   /* U+3028 */
  0x0020,   /* U+3029 */
  0x00FB,   /* U+302A */
  0x00FC,   /* U+302B */
  0x00FD,   /* U+302C */
  0x00FE,   /* U+302D */
  0x00FF,   /* U+302E */
  0x0100,   /* U+302F */
  0x0020,   /* U+3030 */
  0x0020,   /* U+3031 */
  0x0020,   /* U+3032 */
  0x0020,   /* U+3033 */
  0x0020,   /* U+3034 */
  0x0020,   /* U+3035 */
  0x0020,   /* U+3036 */
  0x0020,   /* U+3037 */
  0x0020,   /* U+3038 */
  0x0020,   /* U+3039 */
  0x0020,   /* U+303A */
  0x0020,   /* U+303B */
  0x0020,   /* U+303C */
  0x0020,   /* U+303D */
  0x0020,   /* U+303E */
  0x0020,   /* U+303F */
  0x0020,   /* U+3040 */
  0x0020,   /* U+3041 */
  0x0020,   /* U+3042 */
  0x0020,   /* U+3043 */
  0x0020,   /* U+3044 */
  0x0020,   /* U+3045 */
  0x0020,   /* U+3046 */
  0x0020,   /* U+3047 */
  0x0020,   /* U+3048 */
  0x0020,   /* U+3049 */
  0x0020,   /* U+304A */
  0x0020,   /* U+304B */
  0x0020,   /* U+304C */
  0x0020,   /* U+304D */
  0x0020,   /* U+304E */
  0x0020,   /* U+304F */
  0x0020,   /* U+3050 */
  0x0020,   /* U+3051 */
  0x0020,   /* U+3052 */
  0x0020,   /* U+3053 */
  0x0020,   /* U+3054 */
  0x0020,   /* U+3055 */
  0x0020,   /* U+3056 */
  0x0020,   /* U+3057 */
  0x0020,   /* U+3058 */
  0x0020,   /* U+3059 */
  0x0020,   /* U+305A */
  0x0020,   /* U+305B */
  0x0020,   /* U+305C */
  0x0020,   /* U+305D */
  0x0020,   /* U+305E */
  0x0020,   /* U+305F */
  0x0020,   /* U+3060 */
  0x0020,   /* U+3061 */
  0x0020,   /* U+3062 */
  0x0020,   /* U+3063 */
  0x0020,   /* U+3064 */
  0x0020,   /* U+3065 */
  0x0020,   /* U+3066 */
  0x0020,   /* U+3067 */
  0x0020,   /* U+3068 */
  0x0020,   /* U+3069 */
  0x0020,   /* U+306A */
  0x0020,   /* U+306B */
  0x0020,   /* U+306C */
  0x0020,   /* U+306D */
  0x0020,   /* U+306E */
  0x0020,   /* U+306F */
  0x0020,   /* U+3070 */
  0x0020,   /* U+3071 */
  0x0020,   /* U+3072 */
  0x0020,   /* U+3073 */
  0x0020,   /* U+3074 */
  0x0020,   /* U+3075 */
  0x0020,   /* U+3076 */
  0x0020,   /* U+3077 */
  0x0020,   /* U+3078 */
  0x0020,   /* U+3079 */
  0x0020,   /* U+307A */
  0x0020,   /* U+307B */
  0x0020,   /* U+307C */
  0x0020,   /* U+307D */
  0x0020,   /* U+307E */
  0x0020,   /* U+307F */
  0x0020,   /* U+3080 */
  0x0020,   /* U+3081 */
  0x0020,   /* U+3082 */
  0x0020,   /* U+3083 */
  0x0020,   /* U+3084 */
  0x0020,   /* U+3085 */
  0x0020,   /* U+3086 */
  0x0020,   /* U+3087 */
  0x0020,   /* U+3088 */
  0x0020,   /* U+3089 */
  0x0020,   /* U+308A */
  0x0020,   /* U+308B */
  0x0020,   /* U+308C */
  0x0020,   /* U+308D */
  0x0020,   /* U+308E */
  0x0020,   /* U+308F */
  0x0020,   /* U+3090 */
  0x0020,   /* U+3091 */
  0x0020,   /* U+3092 */
  0x0020,   /* U+3093 */
  0x0020,   /* U+3094 */
  0x0020,   /* U+3095 */
  0x0020,   /* U+3096 */
  0x0020,   /* U+3097 */
  0x0020,   /* U+3098 */
  0x0037,   /* U+3099 */
  0x0038,   /* U+309A */
  0x0020,   /* U+309B */
  0x0020,   /* U+309C */
  0x0020,   /* U+309D */
  0x0020,   /* U+309E */
  0x0020,   /* U+309F */
  0x0020,   /* U+30A0 */
  0x0020,   /* U+30A1 */
  0x0020,   /* U+30A2 */
  0x0020,   /* U+30A3 */
  0x0020,   /* U+30A4 */
  0x0020,   /* U+30A5 */
  0x0020,   /* U+30A6 */
  0x0020,   /* U+30A7 */
  0x0020,   /* U+30A8 */
  0x0020,   /* U+30A9 */
  0x0020,   /* U+30AA */
  0x0020,   /* U+30AB */
  0x0020,   /* U+30AC */
  0x0020,   /* U+30AD */
  0x0020,   /* U+30AE */
  0x0020,   /* U+30AF */
  0x0020,   /* U+30B0 */
  0x0020,   /* U+30B1 */
  0x0020,   /* U+30B2 */
  0x0020,   /* U+30B3 */
  0x0020,   /* U+30B4 */
  0x0020,   /* U+30B5 */
  0x0020,   /* U+30B6 */
  0x0020,   /* U+30B7 */
  0x0020,   /* U+30B8 */
  0x0020,   /* U+30B9 */
  0x0020,   /* U+30BA */
  0x0020,   /* U+30BB */
  0x0020,   /* U+30BC */
  0x0020,   /* U+30BD */
  0x0020,   /* U+30BE */
  0x0020,   /* U+30BF */
  0x0020,   /* U+30C0 */
  0x0020,   /* U+30C1 */
  0x0020,   /* U+30C2 */
  0x0020,   /* U+30C3 */
  0x0020,   /* U+30C4 */
  0x0020,   /* U+30C5 */
  0x0020,   /* U+30C6 */
  0x0020,   /* U+30C7 */
  0x0020,   /* U+30C8 */
  0x0020,   /* U+30C9 */
  0x0020,   /* U+30CA */
  0x0020,   /* U+30CB */
  0x0020,   /* U+30CC */
  0x0020,   /* U+30CD */
  0x0020,   /* U+30CE */
  0x0020,   /* U+30CF */
  0x0020,   /* U+30D0 */
  0x0020,   /* U+30D1 */
  0x0020,   /* U+30D2 */
  0x0020,   /* U+30D3 */
  0x0020,   /* U+30D4 */
  0x0020,   /* U+30D5 */
  0x0020,   /* U+30D6 */
  0x0020,   /* U+30D7 */
  0x0020,   /* U+30D8 */
  0x0020,   /* U+30D9 */
  0x0020,   /* U+30DA */
  0x0020,   /* U+30DB */
  0x0020,   /* U+30DC */
  0x0020,   /* U+30DD */
  0x0020,   /* U+30DE */
  0x0020,   /* U+30DF */
  0x0020,   /* U+30E0 */
  0x0020,   /* U+30E1 */
  0x0020,   /* U+30E2 */
  0x0020,   /* U+30E3 */
  0x0020,   /* U+30E4 */
  0x0020,   /* U+30E5 */
  0x0020,   /* U+30E6 */
  0x0020,   /* U+30E7 */
  0x0020,   /* U+30E8 */
  0x0020,   /* U+30E9 */
  0x0020,   /* U+30EA */
  0x0020,   /* U+30EB */
  0x0020,   /* U+30EC */
  0x0020,   /* U+30ED */
  0x0020,   /* U+30EE */
  0x0020,   /* U+30EF */
  0x0020,   /* U+30F0 */
  0x0020,   /* U+30F1 */
  0x0020,   /* U+30F2 */
  0x0020,   /* U+30F3 */
  0x0020,   /* U+30F4 */
  0x0020,   /* U+30F5 */
  0x0020,   /* U+30F6 */
  0x0020,   /* U+30F7 */
  0x0020,   /* U+30F8 */
  0x0020,   /* U+30F9 */
  0x0020,   /* U+30FA */
  0x0020,   /* U+30FB */
  0x0020,   /* U+30FC */
  0x0020,   /* U+30FD */
  0x0020,   /* U+30FE */
  0x0020,   /* U+30FF */

  /* Tertiary weight 1 for each character. */
  0x0003,   /* U+3000 */
  0x0002,   /* U+3001 */
  0x0002,   /* U+3002 */
  0x0002,   /* U+3003 */
  0x0002,   /* U+3004 */
  0x0002,   /* U+3005 */
  0x0004,   /* U+3006 */
  0x0002,   /* U+3007 */
  0x0002,   /* U+3008 */
  0x0002,   /* U+3009 */
  0x0002,   /* U+300A */
  0x0002,   /* U+300B */
  0x0002,   /* U+300C */
  0x0002,   /* U+300D */
  0x0002,   /* U+300E */
  0x0002,   /* U+300F */
  0x0002,   /* U+3010 */
  0x0002,   /* U+3011 */
  0x0002,   /* U+3012 */
  0x0002,   /* U+3013 */
  0x0002,   /* U+3014 */
  0x0002,   /* U+3015 */
  0x0002,   /* U+3016 */
  0x0002,   /* U+3017 */
  0x0002,   /* U+3018 */
  0x0002,   /* U+3019 */
  0x0002,   /* U+301A */
  0x0002,   /* U+301B */
  0x0002,   /* U+301C */
  0x0002,   /* U+301D */
  0x0002,   /* U+301E */
  0x0002,   /* U+301F */
  0x0002,   /* U+3020 */
  0x0002,   /* U+3021 */
  0x0002,   /* U+3022 */
  0x0002,   /* U+3023 */
  0x0002,   /* U+3024 */
  0x0002,   /* U+3025 */
  0x0002,   /* U+3026 */
  0x0002,   /* U+3027 */
  0x0002,   /* U+3028 */
  0x0002,   /* U+3029 */
  0x0002,   /* U+302A */
  0x0002,   /* U+302B */
  0x0002,   /* U+302C */
  0x0002,   /* U+302D */
  0x0002,   /* U+302E */
  0x0002,   /* U+302F */
  0x0002,   /* U+3030 */
  0x0002,   /* U+3031 */
  0x0002,   /* U+3032 */
  0x0002,   /* U+3033 */
  0x0002,   /* U+3034 */
  0x0002,   /* U+3035 */
  0x0004,   /* U+3036 */
  0x0002,   /* U+3037 */
  0x0004,   /* U+3038 */
  0x0004,   /* U+3039 */
  0x0004,   /* U+303A */
  0x0002,   /* U+303B */
  0x0004,   /* U+303C */
  0x0002,   /* U+303D */
  0x0002,   /* U+303E */
  0x0002,   /* U+303F */
  0x0002,   /* U+3040 */
  0x000D,   /* U+3041 */
  0x000E,   /* U+3042 */
  0x000D,   /* U+3043 */
  0x000E,   /* U+3044 */
  0x000D,   /* U+3045 */
  0x000E,   /* U+3046 */
  0x000D,   /* U+3047 */
  0x000E,   /* U+3048 */
  0x000D,   /* U+3049 */
  0x000E,   /* U+304A */
  0x000E,   /* U+304B */
  0x000E,   /* U+304C */
  0x000E,   /* U+304D */
  0x000E,   /* U+304E */
  0x000E,   /* U+304F */
  0x000E,   /* U+3050 */
  0x000E,   /* U+3051 */
  0x000E,   /* U+3052 */
  0x000E,   /* U+3053 */
  0x000E,   /* U+3054 */
  0x000E,   /* U+3055 */
  0x000E,   /* U+3056 */
  0x000E,   /* U+3057 */
  0x000E,   /* U+3058 */
  0x000E,   /* U+3059 */
  0x000E,   /* U+305A */
  0x000E,   /* U+305B */
  0x000E,   /* U+305C */
  0x000E,   /* U+305D */
  0x000E,   /* U+305E */
  0x000E,   /* U+305F */
  0x000E,   /* U+3060 */
  0x000E,   /* U+3061 */
  0x000E,   /* U+3062 */
  0x000D,   /* U+3063 */
  0x000E,   /* U+3064 */
  0x000E,   /* U+3065 */
  0x000E,   /* U+3066 */
  0x000E,   /* U+3067 */
  0x000E,   /* U+3068 */
  0x000E,   /* U+3069 */
  0x000E,   /* U+306A */
  0x000E,   /* U+306B */
  0x000E,   /* U+306C */
  0x000E,   /* U+306D */
  0x000E,   /* U+306E */
  0x000E,   /* U+306F */
  0x000E,   /* U+3070 */
  0x000E,   /* U+3071 */
  0x000E,   /* U+3072 */
  0x000E,   /* U+3073 */
  0x000E,   /* U+3074 */
  0x000E,   /* U+3075 */
  0x000E,   /* U+3076 */
  0x000E,   /* U+3077 */
  0x000E,   /* U+3078 */
  0x000E,   /* U+3079 */
  0x000E,   /* U+307A */
  0x000E,   /* U+307B */
  0x000E,   /* U+307C */
  0x000E,   /* U+307D */
  0x000E,   /* U+307E */
  0x000E,   /* U+307F */
  0x000E,   /* U+3080 */
  0x000E,   /* U+3081 */
  0x000E,   /* U+3082 */
  0x000D,   /* U+3083 */
  0x000E,   /* U+3084 */
  0x000D,   /* U+3085 */
  0x000E,   /* U+3086 */
  0x000D,   /* U+3087 */
  0x000E,   /* U+3088 */
  0x000E,   /* U+3089 */
  0x000E,   /* U+308A */
  0x000E,   /* U+308B */
  0x000E,   /* U+308C */
  0x000E,   /* U+308D */
  0x000D,   /* U+308E */
  0x000E,   /* U+308F */
  0x000E,   /* U+3090 */
  0x000E,   /* U+3091 */
  0x000E,   /* U+3092 */
  0x000E,   /* U+3093 */
  0x000E,   /* U+3094 */
  0x000D,   /* U+3095 */
  0x000D,   /* U+3096 */
  0x0002,   /* U+3097 */
  0x0002,   /* U+3098 */
  0x0002,   /* U+3099 */
  0x0002,   /* U+309A */
  0x0002,   /* U+309B */
  0x0002,   /* U+309C */
  0x0002,   /* U+309D */
  0x0002,   /* U+309E */
  0x0016,   /* U+309F */
  0x0002,   /* U+30A0 */
  0x000F,   /* U+30A1 */
  0x0011,   /* U+30A2 */
  0x000F,   /* U+30A3 */
  0x0011,   /* U+30A4 */
  0x000F,   /* U+30A5 */
  0x0011,   /* U+30A6 */
  0x000F,   /* U+30A7 */
  0x0011,   /* U+30A8 */
  0x000F,   /* U+30A9 */
  0x0011,   /* U+30AA */
  0x0011,   /* U+30AB */
  0x0011,   /* U+30AC */
  0x0011,   /* U+30AD */
  0x0011,   /* U+30AE */
  0x0011,   /* U+30AF */
  0x0011,   /* U+30B0 */
  0x0011,   /* U+30B1 */
  0x0011,   /* U+30B2 */
  0x0011,   /* U+30B3 */
  0x0011,   /* U+30B4 */
  0x0011,   /* U+30B5 */
  0x0011,   /* U+30B6 */
  0x0011,   /* U+30B7 */
  0x0011,   /* U+30B8 */
  0x0011,   /* U+30B9 */
  0x0011,   /* U+30BA */
  0x0011,   /* U+30BB */
  0x0011,   /* U+30BC */
  0x0011,   /* U+30BD */
  0x0011,   /* U+30BE */
  0x0011,   /* U+30BF */
  0x0011,   /* U+30C0 */
  0x0011,   /* U+30C1 */
  0x0011,   /* U+30C2 */
  0x000F,   /* U+30C3 */
  0x0011,   /* U+30C4 */
  0x0011,   /* U+30C5 */
  0x0011,   /* U+30C6 */
  0x0011,   /* U+30C7 */
  0x0011,   /* U+30C8 */
  0x0011,   /* U+30C9 */
  0x0011,   /* U+30CA */
  0x0011,   /* U+30CB */
  0x0011,   /* U+30CC */
  0x0011,   /* U+30CD */
  0x0011,   /* U+30CE */
  0x0011,   /* U+30CF */
  0x0011,   /* U+30D0 */
  0x0011,   /* U+30D1 */
  0x0011,   /* U+30D2 */
  0x0011,   /* U+30D3 */
  0x0011,   /* U+30D4 */
  0x0011,   /* U+30D5 */
  0x0011,   /* U+30D6 */
  0x0011,   /* U+30D7 */
  0x0011,   /* U+30D8 */
  0x0011,   /* U+30D9 */
  0x0011,   /* U+30DA */
  0x0011,   /* U+30DB */
  0x0011,   /* U+30DC */
  0x0011,   /* U+30DD */
  0x0011,   /* U+30DE */
  0x0011,   /* U+30DF */
  0x0011,   /* U+30E0 */
  0x0011,   /* U+30E1 */
  0x0011,   /* U+30E2 */
  0x000F,   /* U+30E3 */
  0x0011,   /* U+30E4 */
  0x000F,   /* U+30E5 */
  0x0011,   /* U+30E6 */
  0x000F,   /* U+30E7 */
  0x0011,   /* U+30E8 */
  0x0011,   /* U+30E9 */
  0x0011,   /* U+30EA */
  0x0011,   /* U+30EB */
  0x0011,   /* U+30EC */
  0x0011,   /* U+30ED */
  0x000F,   /* U+30EE */
  0x0011,   /* U+30EF */
  0x0011,   /* U+30F0 */
  0x0011,   /* U+30F1 */
  0x0011,   /* U+30F2 */
  0x0011,   /* U+30F3 */
  0x0011,   /* U+30F4 */
  0x000F,   /* U+30F5 */
  0x000F,   /* U+30F6 */
  0x0011,   /* U+30F7 */
  0x0011,   /* U+30F8 */
  0x0011,   /* U+30F9 */
  0x0011,   /* U+30FA */
  0x0002,   /* U+30FB */
  0x0002,   /* U+30FC */
  0x0002,   /* U+30FD */
  0x0002,   /* U+30FE */
  0x0016,   /* U+30FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+3000 */
  0x0000,   /* U+3001 */
  0x0000,   /* U+3002 */
  0x0000,   /* U+3003 */
  0x0000,   /* U+3004 */
  0x0000,   /* U+3005 */
  0x3D7C,   /* U+3006 */
  0x0000,   /* U+3007 */
  0x0000,   /* U+3008 */
  0x0000,   /* U+3009 */
  0x0000,   /* U+300A */
  0x0000,   /* U+300B */
  0x0000,   /* U+300C */
  0x0000,   /* U+300D */
  0x0000,   /* U+300E */
  0x0000,   /* U+300F */
  0x0000,   /* U+3010 */
  0x0000,   /* U+3011 */
  0x0000,   /* U+3012 */
  0x0000,   /* U+3013 */
  0x0000,   /* U+3014 */
  0x0000,   /* U+3015 */
  0x0000,   /* U+3016 */
  0x0000,   /* U+3017 */
  0x0000,   /* U+3018 */
  0x0000,   /* U+3019 */
  0x0000,   /* U+301A */
  0x0000,   /* U+301B */
  0x0000,   /* U+301C */
  0x0000,   /* U+301D */
  0x0000,   /* U+301E */
  0x0000,   /* U+301F */
  0x0000,   /* U+3020 */
  0x0000,   /* U+3021 */
  0x0000,   /* U+3022 */
  0x0000,   /* U+3023 */
  0x0000,   /* U+3024 */
  0x0000,   /* U+3025 */
  0x0000,   /* U+3026 */
  0x0000,   /* U+3027 */
  0x0000,   /* U+3028 */
  0x0000,   /* U+3029 */
  0x0000,   /* U+302A */
  0x0000,   /* U+302B */
  0x0000,   /* U+302C */
  0x0000,   /* U+302D */
  0x0000,   /* U+302E */
  0x0000,   /* U+302F */
  0x0000,   /* U+3030 */
  0x0000,   /* U+3031 */
  0x0000,   /* U+3032 */
  0x0000,   /* U+3033 */
  0x0000,   /* U+3034 */
  0x0000,   /* U+3035 */
  0x0000,   /* U+3036 */
  0x0000,   /* U+3037 */
  0xD341,   /* U+3038 */
  0xD344,   /* U+3039 */
  0xD345,   /* U+303A */
  0x0000,   /* U+303B */
  0x3D67,   /* U+303C */
  0x0000,   /* U+303D */
  0x0000,   /* U+303E */
  0x0000,   /* U+303F */
  0xB040,   /* U+3040 */
  0x0000,   /* U+3041 */
  0x0000,   /* U+3042 */
  0x0000,   /* U+3043 */
  0x0000,   /* U+3044 */
  0x0000,   /* U+3045 */
  0x0000,   /* U+3046 */
  0x0000,   /* U+3047 */
  0x0000,   /* U+3048 */
  0x0000,   /* U+3049 */
  0x0000,   /* U+304A */
  0x0000,   /* U+304B */
  0x0000,   /* U+304C */
  0x0000,   /* U+304D */
  0x0000,   /* U+304E */
  0x0000,   /* U+304F */
  0x0000,   /* U+3050 */
  0x0000,   /* U+3051 */
  0x0000,   /* U+3052 */
  0x0000,   /* U+3053 */
  0x0000,   /* U+3054 */
  0x0000,   /* U+3055 */
  0x0000,   /* U+3056 */
  0x0000,   /* U+3057 */
  0x0000,   /* U+3058 */
  0x0000,   /* U+3059 */
  0x0000,   /* U+305A */
  0x0000,   /* U+305B */
  0x0000,   /* U+305C */
  0x0000,   /* U+305D */
  0x0000,   /* U+305E */
  0x0000,   /* U+305F */
  0x0000,   /* U+3060 */
  0x0000,   /* U+3061 */
  0x0000,   /* U+3062 */
  0x0000,   /* U+3063 */
  0x0000,   /* U+3064 */
  0x0000,   /* U+3065 */
  0x0000,   /* U+3066 */
  0x0000,   /* U+3067 */
  0x0000,   /* U+3068 */
  0x0000,   /* U+3069 */
  0x0000,   /* U+306A */
  0x0000,   /* U+306B */
  0x0000,   /* U+306C */
  0x0000,   /* U+306D */
  0x0000,   /* U+306E */
  0x0000,   /* U+306F */
  0x0000,   /* U+3070 */
  0x0000,   /* U+3071 */
  0x0000,   /* U+3072 */
  0x0000,   /* U+3073 */
  0x0000,   /* U+3074 */
  0x0000,   /* U+3075 */
  0x0000,   /* U+3076 */
  0x0000,   /* U+3077 */
  0x0000,   /* U+3078 */
  0x0000,   /* U+3079 */
  0x0000,   /* U+307A */
  0x0000,   /* U+307B */
  0x0000,   /* U+307C */
  0x0000,   /* U+307D */
  0x0000,   /* U+307E */
  0x0000,   /* U+307F */
  0x0000,   /* U+3080 */
  0x0000,   /* U+3081 */
  0x0000,   /* U+3082 */
  0x0000,   /* U+3083 */
  0x0000,   /* U+3084 */
  0x0000,   /* U+3085 */
  0x0000,   /* U+3086 */
  0x0000,   /* U+3087 */
  0x0000,   /* U+3088 */
  0x0000,   /* U+3089 */
  0x0000,   /* U+308A */
  0x0000,   /* U+308B */
  0x0000,   /* U+308C */
  0x0000,   /* U+308D */
  0x0000,   /* U+308E */
  0x0000,   /* U+308F */
  0x0000,   /* U+3090 */
  0x0000,   /* U+3091 */
  0x0000,   /* U+3092 */
  0x0000,   /* U+3093 */
  0x0000,   /* U+3094 */
  0x0000,   /* U+3095 */
  0x0000,   /* U+3096 */
  0xB097,   /* U+3097 */
  0xB098,   /* U+3098 */
  0x0000,   /* U+3099 */
  0x0000,   /* U+309A */
  0x0000,   /* U+309B */
  0x0000,   /* U+309C */
  0x0000,   /* U+309D */
  0x0000,   /* U+309E */
  0x3D83,   /* U+309F */
  0x0000,   /* U+30A0 */
  0x0000,   /* U+30A1 */
  0x0000,   /* U+30A2 */
  0x0000,   /* U+30A3 */
  0x0000,   /* U+30A4 */
  0x0000,   /* U+30A5 */
  0x0000,   /* U+30A6 */
  0x0000,   /* U+30A7 */
  0x0000,   /* U+30A8 */
  0x0000,   /* U+30A9 */
  0x0000,   /* U+30AA */
  0x0000,   /* U+30AB */
  0x0000,   /* U+30AC */
  0x0000,   /* U+30AD */
  0x0000,   /* U+30AE */
  0x0000,   /* U+30AF */
  0x0000,   /* U+30B0 */
  0x0000,   /* U+30B1 */
  0x0000,   /* U+30B2 */
  0x0000,   /* U+30B3 */
  0x0000,   /* U+30B4 */
  0x0000,   /* U+30B5 */
  0x0000,   /* U+30B6 */
  0x0000,   /* U+30B7 */
  0x0000,   /* U+30B8 */
  0x0000,   /* U+30B9 */
  0x0000,   /* U+30BA */
  0x0000,   /* U+30BB */
  0x0000,   /* U+30BC */
  0x0000,   /* U+30BD */
  0x0000,   /* U+30BE */
  0x0000,   /* U+30BF */
  0x0000,   /* U+30C0 */
  0x0000,   /* U+30C1 */
  0x0000,   /* U+30C2 */
  0x0000,   /* U+30C3 */
  0x0000,   /* U+30C4 */
  0x0000,   /* U+30C5 */
  0x0000,   /* U+30C6 */
  0x0000,   /* U+30C7 */
  0x0000,   /* U+30C8 */
  0x0000,   /* U+30C9 */
  0x0000,   /* U+30CA */
  0x0000,   /* U+30CB */
  0x0000,   /* U+30CC */
  0x0000,   /* U+30CD */
  0x0000,   /* U+30CE */
  0x0000,   /* U+30CF */
  0x0000,   /* U+30D0 */
  0x0000,   /* U+30D1 */
  0x0000,   /* U+30D2 */
  0x0000,   /* U+30D3 */
  0x0000,   /* U+30D4 */
  0x0000,   /* U+30D5 */
  0x0000,   /* U+30D6 */
  0x0000,   /* U+30D7 */
  0x0000,   /* U+30D8 */
  0x0000,   /* U+30D9 */
  0x0000,   /* U+30DA */
  0x0000,   /* U+30DB */
  0x0000,   /* U+30DC */
  0x0000,   /* U+30DD */
  0x0000,   /* U+30DE */
  0x0000,   /* U+30DF */
  0x0000,   /* U+30E0 */
  0x0000,   /* U+30E1 */
  0x0000,   /* U+30E2 */
  0x0000,   /* U+30E3 */
  0x0000,   /* U+30E4 */
  0x0000,   /* U+30E5 */
  0x0000,   /* U+30E6 */
  0x0000,   /* U+30E7 */
  0x0000,   /* U+30E8 */
  0x0000,   /* U+30E9 */
  0x0000,   /* U+30EA */
  0x0000,   /* U+30EB */
  0x0000,   /* U+30EC */
  0x0000,   /* U+30ED */
  0x0000,   /* U+30EE */
  0x0000,   /* U+30EF */
  0x0000,   /* U+30F0 */
  0x0000,   /* U+30F1 */
  0x0000,   /* U+30F2 */
  0x0000,   /* U+30F3 */
  0x0000,   /* U+30F4 */
  0x0000,   /* U+30F5 */
  0x0000,   /* U+30F6 */
  0x0000,   /* U+30F7 */
  0x0000,   /* U+30F8 */
  0x0000,   /* U+30F9 */
  0x0000,   /* U+30FA */
  0x0000,   /* U+30FB */
  0x0000,   /* U+30FC */
  0x0000,   /* U+30FD */
  0x0000,   /* U+30FE */
  0x3D6E,   /* U+30FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+3000 */
  0x0000,   /* U+3001 */
  0x0000,   /* U+3002 */
  0x0000,   /* U+3003 */
  0x0000,   /* U+3004 */
  0x0000,   /* U+3005 */
  0x0020,   /* U+3006 */
  0x0000,   /* U+3007 */
  0x0000,   /* U+3008 */
  0x0000,   /* U+3009 */
  0x0000,   /* U+300A */
  0x0000,   /* U+300B */
  0x0000,   /* U+300C */
  0x0000,   /* U+300D */
  0x0000,   /* U+300E */
  0x0000,   /* U+300F */
  0x0000,   /* U+3010 */
  0x0000,   /* U+3011 */
  0x0000,   /* U+3012 */
  0x0000,   /* U+3013 */
  0x0000,   /* U+3014 */
  0x0000,   /* U+3015 */
  0x0000,   /* U+3016 */
  0x0000,   /* U+3017 */
  0x0000,   /* U+3018 */
  0x0000,   /* U+3019 */
  0x0000,   /* U+301A */
  0x0000,   /* U+301B */
  0x0000,   /* U+301C */
  0x0000,   /* U+301D */
  0x0000,   /* U+301E */
  0x0000,   /* U+301F */
  0x0000,   /* U+3020 */
  0x0000,   /* U+3021 */
  0x0000,   /* U+3022 */
  0x0000,   /* U+3023 */
  0x0000,   /* U+3024 */
  0x0000,   /* U+3025 */
  0x0000,   /* U+3026 */
  0x0000,   /* U+3027 */
  0x0000,   /* U+3028 */
  0x0000,   /* U+3029 */
  0x0000,   /* U+302A */
  0x0000,   /* U+302B */
  0x0000,   /* U+302C */
  0x0000,   /* U+302D */
  0x0000,   /* U+302E */
  0x0000,   /* U+302F */
  0x0000,   /* U+3030 */
  0x0000,   /* U+3031 */
  0x0037,   /* U+3032 */
  0x0000,   /* U+3033 */
  0x0037,   /* U+3034 */
  0x0000,   /* U+3035 */
  0x0000,   /* U+3036 */
  0x0000,   /* U+3037 */
  0x0000,   /* U+3038 */
  0x0000,   /* U+3039 */
  0x0000,   /* U+303A */
  0x0000,   /* U+303B */
  0x0020,   /* U+303C */
  0x0000,   /* U+303D */
  0x0000,   /* U+303E */
  0x0000,   /* U+303F */
  0x0000,   /* U+3040 */
  0x0000,   /* U+3041 */
  0x0000,   /* U+3042 */
  0x0000,   /* U+3043 */
  0x0000,   /* U+3044 */
  0x0000,   /* U+3045 */
  0x0000,   /* U+3046 */
  0x0000,   /* U+3047 */
  0x0000,   /* U+3048 */
  0x0000,   /* U+3049 */
  0x0000,   /* U+304A */
  0x0000,   /* U+304B */
  0x0037,   /* U+304C */
  0x0000,   /* U+304D */
  0x0037,   /* U+304E */
  0x0000,   /* U+304F */
  0x0037,   /* U+3050 */
  0x0000,   /* U+3051 */
  0x0037,   /* U+3052 */
  0x0000,   /* U+3053 */
  0x0037,   /* U+3054 */
  0x0000,   /* U+3055 */
  0x0037,   /* U+3056 */
  0x0000,   /* U+3057 */
  0x0037,   /* U+3058 */
  0x0000,   /* U+3059 */
  0x0037,   /* U+305A */
  0x0000,   /* U+305B */
  0x0037,   /* U+305C */
  0x0000,   /* U+305D */
  0x0037,   /* U+305E */
  0x0000,   /* U+305F */
  0x0037,   /* U+3060 */
  0x0000,   /* U+3061 */
  0x0037,   /* U+3062 */
  0x0000,   /* U+3063 */
  0x0000,   /* U+3064 */
  0x0037,   /* U+3065 */
  0x0000,   /* U+3066 */
  0x0037,   /* U+3067 */
  0x0000,   /* U+3068 */
  0x0037,   /* U+3069 */
  0x0000,   /* U+306A */
  0x0000,   /* U+306B */
  0x0000,   /* U+306C */
  0x0000,   /* U+306D */
  0x0000,   /* U+306E */
  0x0000,   /* U+306F */
  0x0037,   /* U+3070 */
  0x0038,   /* U+3071 */
  0x0000,   /* U+3072 */
  0x0037,   /* U+3073 */
  0x0038,   /* U+3074 */
  0x0000,   /* U+3075 */
  0x0037,   /* U+3076 */
  0x0038,   /* U+3077 */
  0x0000,   /* U+3078 */
  0x0037,   /* U+3079 */
  0x0038,   /* U+307A */
  0x0000,   /* U+307B */
  0x0037,   /* U+307C */
  0x0038,   /* U+307D */
  0x0000,   /* U+307E */
  0x0000,   /* U+307F */
  0x0000,   /* U+3080 */
  0x0000,   /* U+3081 */
  0x0000,   /* U+3082 */
  0x0000,   /* U+3083 */
  0x0000,   /* U+3084 */
  0x0000,   /* U+3085 */
  0x0000,   /* U+3086 */
  0x0000,   /* U+3087 */
  0x0000,   /* U+3088 */
  0x0000,   /* U+3089 */
  0x0000,   /* U+308A */
  0x0000,   /* U+308B */
  0x0000,   /* U+308C */
  0x0000,   /* U+308D */
  0x0000,   /* U+308E */
  0x0000,   /* U+308F */
  0x0000,   /* U+3090 */
  0x0000,   /* U+3091 */
  0x0000,   /* U+3092 */
  0x0000,   /* U+3093 */
  0x0037,   /* U+3094 */
  0x0000,   /* U+3095 */
  0x0000,   /* U+3096 */
  0x0000,   /* U+3097 */
  0x0000,   /* U+3098 */
  0x0000,   /* U+3099 */
  0x0000,   /* U+309A */
  0x0000,   /* U+309B */
  0x0000,   /* U+309C */
  0x0000,   /* U+309D */
  0x0037,   /* U+309E */
  0x0020,   /* U+309F */
  0x0000,   /* U+30A0 */
  0x0000,   /* U+30A1 */
  0x0000,   /* U+30A2 */
  0x0000,   /* U+30A3 */
  0x0000,   /* U+30A4 */
  0x0000,   /* U+30A5 */
  0x0000,   /* U+30A6 */
  0x0000,   /* U+30A7 */
  0x0000,   /* U+30A8 */
  0x0000,   /* U+30A9 */
  0x0000,   /* U+30AA */
  0x0000,   /* U+30AB */
  0x0037,   /* U+30AC */
  0x0000,   /* U+30AD */
  0x0037,   /* U+30AE */
  0x0000,   /* U+30AF */
  0x0037,   /* U+30B0 */
  0x0000,   /* U+30B1 */
  0x0037,   /* U+30B2 */
  0x0000,   /* U+30B3 */
  0x0037,   /* U+30B4 */
  0x0000,   /* U+30B5 */
  0x0037,   /* U+30B6 */
  0x0000,   /* U+30B7 */
  0x0037,   /* U+30B8 */
  0x0000,   /* U+30B9 */
  0x0037,   /* U+30BA */
  0x0000,   /* U+30BB */
  0x0037,   /* U+30BC */
  0x0000,   /* U+30BD */
  0x0037,   /* U+30BE */
  0x0000,   /* U+30BF */
  0x0037,   /* U+30C0 */
  0x0000,   /* U+30C1 */
  0x0037,   /* U+30C2 */
  0x0000,   /* U+30C3 */
  0x0000,   /* U+30C4 */
  0x0037,   /* U+30C5 */
  0x0000,   /* U+30C6 */
  0x0037,   /* U+30C7 */
  0x0000,   /* U+30C8 */
  0x0037,   /* U+30C9 */
  0x0000,   /* U+30CA */
  0x0000,   /* U+30CB */
  0x0000,   /* U+30CC */
  0x0000,   /* U+30CD */
  0x0000,   /* U+30CE */
  0x0000,   /* U+30CF */
  0x0037,   /* U+30D0 */
  0x0038,   /* U+30D1 */
  0x0000,   /* U+30D2 */
  0x0037,   /* U+30D3 */
  0x0038,   /* U+30D4 */
  0x0000,   /* U+30D5 */
  0x0037,   /* U+30D6 */
  0x0038,   /* U+30D7 */
  0x0000,   /* U+30D8 */
  0x0037,   /* U+30D9 */
  0x0038,   /* U+30DA */
  0x0000,   /* U+30DB */
  0x0037,   /* U+30DC */
  0x0038,   /* U+30DD */
  0x0000,   /* U+30DE */
  0x0000,   /* U+30DF */
  0x0000,   /* U+30E0 */
  0x0000,   /* U+30E1 */
  0x0000,   /* U+30E2 */
  0x0000,   /* U+30E3 */
  0x0000,   /* U+30E4 */
  0x0000,   /* U+30E5 */
  0x0000,   /* U+30E6 */
  0x0000,   /* U+30E7 */
  0x0000,   /* U+30E8 */
  0x0000,   /* U+30E9 */
  0x0000,   /* U+30EA */
  0x0000,   /* U+30EB */
  0x0000,   /* U+30EC */
  0x0000,   /* U+30ED */
  0x0000,   /* U+30EE */
  0x0000,   /* U+30EF */
  0x0000,   /* U+30F0 */
  0x0000,   /* U+30F1 */
  0x0000,   /* U+30F2 */
  0x0000,   /* U+30F3 */
  0x0037,   /* U+30F4 */
  0x0000,   /* U+30F5 */
  0x0000,   /* U+30F6 */
  0x0037,   /* U+30F7 */
  0x0037,   /* U+30F8 */
  0x0037,   /* U+30F9 */
  0x0037,   /* U+30FA */
  0x0000,   /* U+30FB */
  0x0000,   /* U+30FC */
  0x0000,   /* U+30FD */
  0x0037,   /* U+30FE */
  0x0020,   /* U+30FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+3000 */
  0x0000,   /* U+3001 */
  0x0000,   /* U+3002 */
  0x0000,   /* U+3003 */
  0x0000,   /* U+3004 */
  0x0000,   /* U+3005 */
  0x0004,   /* U+3006 */
  0x0000,   /* U+3007 */
  0x0000,   /* U+3008 */
  0x0000,   /* U+3009 */
  0x0000,   /* U+300A */
  0x0000,   /* U+300B */
  0x0000,   /* U+300C */
  0x0000,   /* U+300D */
  0x0000,   /* U+300E */
  0x0000,   /* U+300F */
  0x0000,   /* U+3010 */
  0x0000,   /* U+3011 */
  0x0000,   /* U+3012 */
  0x0000,   /* U+3013 */
  0x0000,   /* U+3014 */
  0x0000,   /* U+3015 */
  0x0000,   /* U+3016 */
  0x0000,   /* U+3017 */
  0x0000,   /* U+3018 */
  0x0000,   /* U+3019 */
  0x0000,   /* U+301A */
  0x0000,   /* U+301B */
  0x0000,   /* U+301C */
  0x0000,   /* U+301D */
  0x0000,   /* U+301E */
  0x0000,   /* U+301F */
  0x0000,   /* U+3020 */
  0x0000,   /* U+3021 */
  0x0000,   /* U+3022 */
  0x0000,   /* U+3023 */
  0x0000,   /* U+3024 */
  0x0000,   /* U+3025 */
  0x0000,   /* U+3026 */
  0x0000,   /* U+3027 */
  0x0000,   /* U+3028 */
  0x0000,   /* U+3029 */
  0x0000,   /* U+302A */
  0x0000,   /* U+302B */
  0x0000,   /* U+302C */
  0x0000,   /* U+302D */
  0x0000,   /* U+302E */
  0x0000,   /* U+302F */
  0x0000,   /* U+3030 */
  0x0000,   /* U+3031 */
  0x0002,   /* U+3032 */
  0x0000,   /* U+3033 */
  0x0002,   /* U+3034 */
  0x0000,   /* U+3035 */
  0x0000,   /* U+3036 */
  0x0000,   /* U+3037 */
  0x0000,   /* U+3038 */
  0x0000,   /* U+3039 */
  0x0000,   /* U+303A */
  0x0000,   /* U+303B */
  0x0004,   /* U+303C */
  0x0000,   /* U+303D */
  0x0000,   /* U+303E */
  0x0000,   /* U+303F */
  0x0000,   /* U+3040 */
  0x0000,   /* U+3041 */
  0x0000,   /* U+3042 */
  0x0000,   /* U+3043 */
  0x0000,   /* U+3044 */
  0x0000,   /* U+3045 */
  0x0000,   /* U+3046 */
  0x0000,   /* U+3047 */
  0x0000,   /* U+3048 */
  0x0000,   /* U+3049 */
  0x0000,   /* U+304A */
  0x0000,   /* U+304B */
  0x0002,   /* U+304C */
  0x0000,   /* U+304D */
  0x0002,   /* U+304E */
  0x0000,   /* U+304F */
  0x0002,   /* U+3050 */
  0x0000,   /* U+3051 */
  0x0002,   /* U+3052 */
  0x0000,   /* U+3053 */
  0x0002,   /* U+3054 */
  0x0000,   /* U+3055 */
  0x0002,   /* U+3056 */
  0x0000,   /* U+3057 */
  0x0002,   /* U+3058 */
  0x0000,   /* U+3059 */
  0x0002,   /* U+305A */
  0x0000,   /* U+305B */
  0x0002,   /* U+305C */
  0x0000,   /* U+305D */
  0x0002,   /* U+305E */
  0x0000,   /* U+305F */
  0x0002,   /* U+3060 */
  0x0000,   /* U+3061 */
  0x0002,   /* U+3062 */
  0x0000,   /* U+3063 */
  0x0000,   /* U+3064 */
  0x0002,   /* U+3065 */
  0x0000,   /* U+3066 */
  0x0002,   /* U+3067 */
  0x0000,   /* U+3068 */
  0x0002,   /* U+3069 */
  0x0000,   /* U+306A */
  0x0000,   /* U+306B */
  0x0000,   /* U+306C */
  0x0000,   /* U+306D */
  0x0000,   /* U+306E */
  0x0000,   /* U+306F */
  0x0002,   /* U+3070 */
  0x0002,   /* U+3071 */
  0x0000,   /* U+3072 */
  0x0002,   /* U+3073 */
  0x0002,   /* U+3074 */
  0x0000,   /* U+3075 */
  0x0002,   /* U+3076 */
  0x0002,   /* U+3077 */
  0x0000,   /* U+3078 */
  0x0002,   /* U+3079 */
  0x0002,   /* U+307A */
  0x0000,   /* U+307B */
  0x0002,   /* U+307C */
  0x0002,   /* U+307D */
  0x0000,   /* U+307E */
  0x0000,   /* U+307F */
  0x0000,   /* U+3080 */
  0x0000,   /* U+3081 */
  0x0000,   /* U+3082 */
  0x0000,   /* U+3083 */
  0x0000,   /* U+3084 */
  0x0000,   /* U+3085 */
  0x0000,   /* U+3086 */
  0x0000,   /* U+3087 */
  0x0000,   /* U+3088 */
  0x0000,   /* U+3089 */
  0x0000,   /* U+308A */
  0x0000,   /* U+308B */
  0x0000,   /* U+308C */
  0x0000,   /* U+308D */
  0x0000,   /* U+308E */
  0x0000,   /* U+308F */
  0x0000,   /* U+3090 */
  0x0000,   /* U+3091 */
  0x0000,   /* U+3092 */
  0x0000,   /* U+3093 */
  0x0002,   /* U+3094 */
  0x0000,   /* U+3095 */
  0x0000,   /* U+3096 */
  0x0000,   /* U+3097 */
  0x0000,   /* U+3098 */
  0x0000,   /* U+3099 */
  0x0000,   /* U+309A */
  0x0000,   /* U+309B */
  0x0000,   /* U+309C */
  0x0000,   /* U+309D */
  0x0002,   /* U+309E */
  0x0016,   /* U+309F */
  0x0000,   /* U+30A0 */
  0x0000,   /* U+30A1 */
  0x0000,   /* U+30A2 */
  0x0000,   /* U+30A3 */
  0x0000,   /* U+30A4 */
  0x0000,   /* U+30A5 */
  0x0000,   /* U+30A6 */
  0x0000,   /* U+30A7 */
  0x0000,   /* U+30A8 */
  0x0000,   /* U+30A9 */
  0x0000,   /* U+30AA */
  0x0000,   /* U+30AB */
  0x0002,   /* U+30AC */
  0x0000,   /* U+30AD */
  0x0002,   /* U+30AE */
  0x0000,   /* U+30AF */
  0x0002,   /* U+30B0 */
  0x0000,   /* U+30B1 */
  0x0002,   /* U+30B2 */
  0x0000,   /* U+30B3 */
  0x0002,   /* U+30B4 */
  0x0000,   /* U+30B5 */
  0x0002,   /* U+30B6 */
  0x0000,   /* U+30B7 */
  0x0002,   /* U+30B8 */
  0x0000,   /* U+30B9 */
  0x0002,   /* U+30BA */
  0x0000,   /* U+30BB */
  0x0002,   /* U+30BC */
  0x0000,   /* U+30BD */
  0x0002,   /* U+30BE */
  0x0000,   /* U+30BF */
  0x0002,   /* U+30C0 */
  0x0000,   /* U+30C1 */
  0x0002,   /* U+30C2 */
  0x0000,   /* U+30C3 */
  0x0000,   /* U+30C4 */
  0x0002,   /* U+30C5 */
  0x0000,   /* U+30C6 */
  0x0002,   /* U+30C7 */
  0x0000,   /* U+30C8 */
  0x0002,   /* U+30C9 */
  0x0000,   /* U+30CA */
  0x0000,   /* U+30CB */
  0x0000,   /* U+30CC */
  0x0000,   /* U+30CD */
  0x0000,   /* U+30CE */
  0x0000,   /* U+30CF */
  0x0002,   /* U+30D0 */
  0x0002,   /* U+30D1 */
  0x0000,   /* U+30D2 */
  0x0002,   /* U+30D3 */
  0x0002,   /* U+30D4 */
  0x0000,   /* U+30D5 */
  0x0002,   /* U+30D6 */
  0x0002,   /* U+30D7 */
  0x0000,   /* U+30D8 */
  0x0002,   /* U+30D9 */
  0x0002,   /* U+30DA */
  0x0000,   /* U+30DB */
  0x0002,   /* U+30DC */
  0x0002,   /* U+30DD */
  0x0000,   /* U+30DE */
  0x0000,   /* U+30DF */
  0x0000,   /* U+30E0 */
  0x0000,   /* U+30E1 */
  0x0000,   /* U+30E2 */
  0x0000,   /* U+30E3 */
  0x0000,   /* U+30E4 */
  0x0000,   /* U+30E5 */
  0x0000,   /* U+30E6 */
  0x0000,   /* U+30E7 */
  0x0000,   /* U+30E8 */
  0x0000,   /* U+30E9 */
  0x0000,   /* U+30EA */
  0x0000,   /* U+30EB */
  0x0000,   /* U+30EC */
  0x0000,   /* U+30ED */
  0x0000,   /* U+30EE */
  0x0000,   /* U+30EF */
  0x0000,   /* U+30F0 */
  0x0000,   /* U+30F1 */
  0x0000,   /* U+30F2 */
  0x0000,   /* U+30F3 */
  0x0002,   /* U+30F4 */
  0x0000,   /* U+30F5 */
  0x0000,   /* U+30F6 */
  0x0002,   /* U+30F7 */
  0x0002,   /* U+30F8 */
  0x0002,   /* U+30F9 */
  0x0002,   /* U+30FA */
  0x0000,   /* U+30FB */
  0x0000,   /* U+30FC */
  0x0000,   /* U+30FD */
  0x0002,   /* U+30FE */
  0x0016,   /* U+30FF */
};

uint16 uca900_p031[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 1, 2, 1, 2, 2, 2, 2, 2, 1, 1, 2, 2, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0xFBC0,   /* U+3100 */
  0xFBC0,   /* U+3101 */
  0xFBC0,   /* U+3102 */
  0xFBC0,   /* U+3103 */
  0xFBC0,   /* U+3104 */
  0x3D8C,   /* U+3105 */
  0x3D8D,   /* U+3106 */
  0x3D8E,   /* U+3107 */
  0x3D8F,   /* U+3108 */
  0x3D91,   /* U+3109 */
  0x3D92,   /* U+310A */
  0x3D93,   /* U+310B */
  0x3D94,   /* U+310C */
  0x3D95,   /* U+310D */
  0x3D96,   /* U+310E */
  0x3D99,   /* U+310F */
  0x3D9A,   /* U+3110 */
  0x3D9B,   /* U+3111 */
  0x3D9C,   /* U+3112 */
  0x3D9E,   /* U+3113 */
  0x3D9F,   /* U+3114 */
  0x3DA0,   /* U+3115 */
  0x3DA1,   /* U+3116 */
  0x3DA2,   /* U+3117 */
  0x3DA3,   /* U+3118 */
  0x3DA4,   /* U+3119 */
  0x3DA8,   /* U+311A */
  0x3DA9,   /* U+311B */
  0x3DAB,   /* U+311C */
  0x3DAC,   /* U+311D */
  0x3DAE,   /* U+311E */
  0x3DAF,   /* U+311F */
  0x3DB0,   /* U+3120 */
  0x3DB1,   /* U+3121 */
  0x3DB2,   /* U+3122 */
  0x3DB3,   /* U+3123 */
  0x3DB4,   /* U+3124 */
  0x3DB6,   /* U+3125 */
  0x3DBA,   /* U+3126 */
  0x3DBB,   /* U+3127 */
  0x3DBC,   /* U+3128 */
  0x3DBD,   /* U+3129 */
  0x3D90,   /* U+312A */
  0x3D97,   /* U+312B */
  0x3D9D,   /* U+312C */
  0x3DBE,   /* U+312D */
  0xFBC0,   /* U+312E */
  0xFBC0,   /* U+312F */
  0xFBC0,   /* U+3130 */
  0x3BF5,   /* U+3131 */
  0x3BF6,   /* U+3132 */
  0x3CD3,   /* U+3133 */
  0x3BF7,   /* U+3134 */
  0x3CD5,   /* U+3135 */
  0x3CD6,   /* U+3136 */
  0x3BF8,   /* U+3137 */
  0x3BF9,   /* U+3138 */
  0x3BFA,   /* U+3139 */
  0x3CD9,   /* U+313A */
  0x3CDA,   /* U+313B */
  0x3CDB,   /* U+313C */
  0x3CDC,   /* U+313D */
  0x3CDD,   /* U+313E */
  0x3CDE,   /* U+313F */
  0x3C0F,   /* U+3140 */
  0x3BFB,   /* U+3141 */
  0x3BFC,   /* U+3142 */
  0x3BFD,   /* U+3143 */
  0x3C16,   /* U+3144 */
  0x3BFE,   /* U+3145 */
  0x3BFF,   /* U+3146 */
  0x3C00,   /* U+3147 */
  0x3C01,   /* U+3148 */
  0x3C02,   /* U+3149 */
  0x3C03,   /* U+314A */
  0x3C04,   /* U+314B */
  0x3C05,   /* U+314C */
  0x3C06,   /* U+314D */
  0x3C07,   /* U+314E */
  0x3C73,   /* U+314F */
  0x3C74,   /* U+3150 */
  0x3C75,   /* U+3151 */
  0x3C76,   /* U+3152 */
  0x3C77,   /* U+3153 */
  0x3C78,   /* U+3154 */
  0x3C79,   /* U+3155 */
  0x3C7A,   /* U+3156 */
  0x3C7B,   /* U+3157 */
  0x3C7C,   /* U+3158 */
  0x3C7D,   /* U+3159 */
  0x3C7E,   /* U+315A */
  0x3C7F,   /* U+315B */
  0x3C80,   /* U+315C */
  0x3C81,   /* U+315D */
  0x3C82,   /* U+315E */
  0x3C83,   /* U+315F */
  0x3C84,   /* U+3160 */
  0x3C85,   /* U+3161 */
  0x3C86,   /* U+3162 */
  0x3C87,   /* U+3163 */
  0x3C72,   /* U+3164 */
  0x3C09,   /* U+3165 */
  0x3C0A,   /* U+3166 */
  0x3CF0,   /* U+3167 */
  0x3CF1,   /* U+3168 */
  0x3CF5,   /* U+3169 */
  0x3CF7,   /* U+316A */
  0x3CFC,   /* U+316B */
  0x3D00,   /* U+316C */
  0x3D02,   /* U+316D */
  0x3C11,   /* U+316E */
  0x3D06,   /* U+316F */
  0x3D08,   /* U+3170 */
  0x3C12,   /* U+3171 */
  0x3C13,   /* U+3172 */
  0x3C15,   /* U+3173 */
  0x3C17,   /* U+3174 */
  0x3C18,   /* U+3175 */
  0x3C1C,   /* U+3176 */
  0x3C1E,   /* U+3177 */
  0x3C20,   /* U+3178 */
  0x3C21,   /* U+3179 */
  0x3C22,   /* U+317A */
  0x3C23,   /* U+317B */
  0x3C24,   /* U+317C */
  0x3C27,   /* U+317D */
  0x3C2B,   /* U+317E */
  0x3C35,   /* U+317F */
  0x3C3C,   /* U+3180 */
  0x3C41,   /* U+3181 */
  0x3D1A,   /* U+3182 */
  0x3D1B,   /* U+3183 */
  0x3C4C,   /* U+3184 */
  0x3C4D,   /* U+3185 */
  0x3C4E,   /* U+3186 */
  0x3C96,   /* U+3187 */
  0x3C97,   /* U+3188 */
  0x3C9A,   /* U+3189 */
  0x3CA3,   /* U+318A */
  0x3CA4,   /* U+318B */
  0x3CA6,   /* U+318C */
  0x3CB0,   /* U+318D */
  0x3CB3,   /* U+318E */
  0xFBC0,   /* U+318F */
  0x1A92,   /* U+3190 */
  0x1A93,   /* U+3191 */
  0xFB40,   /* U+3192 */
  0xFB40,   /* U+3193 */
  0xFB40,   /* U+3194 */
  0xFB40,   /* U+3195 */
  0xFB40,   /* U+3196 */
  0xFB40,   /* U+3197 */
  0xFB40,   /* U+3198 */
  0xFB40,   /* U+3199 */
  0xFB40,   /* U+319A */
  0xFB40,   /* U+319B */
  0xFB40,   /* U+319C */
  0xFB40,   /* U+319D */
  0xFB40,   /* U+319E */
  0xFB40,   /* U+319F */
  0x3D8C,   /* U+31A0 */
  0x3DA2,   /* U+31A1 */
  0x3D9A,   /* U+31A2 */
  0x3D95,   /* U+31A3 */
  0x3DAD,   /* U+31A4 */
  0x3DAD,   /* U+31A5 */
  0x3DAA,   /* U+31A6 */
  0x3DA9,   /* U+31A7 */
  0x3DBC,   /* U+31A8 */
  0x3DA8,   /* U+31A9 */
  0x3DBB,   /* U+31AA */
  0x3DBC,   /* U+31AB */
  0x3DB9,   /* U+31AC */
  0x3D98,   /* U+31AD */
  0x3DAE,   /* U+31AE */
  0x3DB0,   /* U+31AF */
  0x3DB7,   /* U+31B0 */
  0x3DB8,   /* U+31B1 */
  0x3DB5,   /* U+31B2 */
  0x3DBB,   /* U+31B3 */
  0x3D8D,   /* U+31B4 */
  0x3D92,   /* U+31B5 */
  0x3D96,   /* U+31B6 */
  0x3D99,   /* U+31B7 */
  0x3DA5,   /* U+31B8 */
  0x3DA6,   /* U+31B9 */
  0x3DA7,   /* U+31BA */
  0xFBC0,   /* U+31BB */
  0xFBC0,   /* U+31BC */
  0xFBC0,   /* U+31BD */
  0xFBC0,   /* U+31BE */
  0xFBC0,   /* U+31BF */
  0x1A67,   /* U+31C0 */
  0x1A68,   /* U+31C1 */
  0x1A69,   /* U+31C2 */
  0x1A6A,   /* U+31C3 */
  0x1A6B,   /* U+31C4 */
  0x1A6C,   /* U+31C5 */
  0x1A6D,   /* U+31C6 */
  0x1A6E,   /* U+31C7 */
  0x1A6F,   /* U+31C8 */
  0x1A70,   /* U+31C9 */
  0x1A71,   /* U+31CA */
  0x1A72,   /* U+31CB */
  0x1A73,   /* U+31CC */
  0x1A74,   /* U+31CD */
  0x1A75,   /* U+31CE */
  0x1A76,   /* U+31CF */
  0x1A77,   /* U+31D0 */
  0x1A78,   /* U+31D1 */
  0x1A79,   /* U+31D2 */
  0x1A7A,   /* U+31D3 */
  0x1A7B,   /* U+31D4 */
  0x1A7C,   /* U+31D5 */
  0x1A7D,   /* U+31D6 */
  0x1A7E,   /* U+31D7 */
  0x1A7F,   /* U+31D8 */
  0x1A80,   /* U+31D9 */
  0x1A81,   /* U+31DA */
  0x1A82,   /* U+31DB */
  0x1A83,   /* U+31DC */
  0x1A84,   /* U+31DD */
  0x1A85,   /* U+31DE */
  0x1A86,   /* U+31DF */
  0x1A87,   /* U+31E0 */
  0x1A88,   /* U+31E1 */
  0x1A89,   /* U+31E2 */
  0x1A8A,   /* U+31E3 */
  0xFBC0,   /* U+31E4 */
  0xFBC0,   /* U+31E5 */
  0xFBC0,   /* U+31E6 */
  0xFBC0,   /* U+31E7 */
  0xFBC0,   /* U+31E8 */
  0xFBC0,   /* U+31E9 */
  0xFBC0,   /* U+31EA */
  0xFBC0,   /* U+31EB */
  0xFBC0,   /* U+31EC */
  0xFBC0,   /* U+31ED */
  0xFBC0,   /* U+31EE */
  0xFBC0,   /* U+31EF */
  0x3D62,   /* U+31F0 */
  0x3D66,   /* U+31F1 */
  0x3D67,   /* U+31F2 */
  0x3D6E,   /* U+31F3 */
  0x3D71,   /* U+31F4 */
  0x3D74,   /* U+31F5 */
  0x3D75,   /* U+31F6 */
  0x3D76,   /* U+31F7 */
  0x3D77,   /* U+31F8 */
  0x3D78,   /* U+31F9 */
  0x3D7B,   /* U+31FA */
  0x3D82,   /* U+31FB */
  0x3D83,   /* U+31FC */
  0x3D84,   /* U+31FD */
  0x3D85,   /* U+31FE */
  0x3D86,   /* U+31FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+3100 */
  0x0020,   /* U+3101 */
  0x0020,   /* U+3102 */
  0x0020,   /* U+3103 */
  0x0020,   /* U+3104 */
  0x0020,   /* U+3105 */
  0x0020,   /* U+3106 */
  0x0020,   /* U+3107 */
  0x0020,   /* U+3108 */
  0x0020,   /* U+3109 */
  0x0020,   /* U+310A */
  0x0020,   /* U+310B */
  0x0020,   /* U+310C */
  0x0020,   /* U+310D */
  0x0020,   /* U+310E */
  0x0020,   /* U+310F */
  0x0020,   /* U+3110 */
  0x0020,   /* U+3111 */
  0x0020,   /* U+3112 */
  0x0020,   /* U+3113 */
  0x0020,   /* U+3114 */
  0x0020,   /* U+3115 */
  0x0020,   /* U+3116 */
  0x0020,   /* U+3117 */
  0x0020,   /* U+3118 */
  0x0020,   /* U+3119 */
  0x0020,   /* U+311A */
  0x0020,   /* U+311B */
  0x0020,   /* U+311C */
  0x0020,   /* U+311D */
  0x0020,   /* U+311E */
  0x0020,   /* U+311F */
  0x0020,   /* U+3120 */
  0x0020,   /* U+3121 */
  0x0020,   /* U+3122 */
  0x0020,   /* U+3123 */
  0x0020,   /* U+3124 */
  0x0020,   /* U+3125 */
  0x0020,   /* U+3126 */
  0x0020,   /* U+3127 */
  0x0020,   /* U+3128 */
  0x0020,   /* U+3129 */
  0x0020,   /* U+312A */
  0x0020,   /* U+312B */
  0x0020,   /* U+312C */
  0x0020,   /* U+312D */
  0x0020,   /* U+312E */
  0x0020,   /* U+312F */
  0x0020,   /* U+3130 */
  0x0020,   /* U+3131 */
  0x0020,   /* U+3132 */
  0x0020,   /* U+3133 */
  0x0020,   /* U+3134 */
  0x0020,   /* U+3135 */
  0x0020,   /* U+3136 */
  0x0020,   /* U+3137 */
  0x0020,   /* U+3138 */
  0x0020,   /* U+3139 */
  0x0020,   /* U+313A */
  0x0020,   /* U+313B */
  0x0020,   /* U+313C */
  0x0020,   /* U+313D */
  0x0020,   /* U+313E */
  0x0020,   /* U+313F */
  0x0020,   /* U+3140 */
  0x0020,   /* U+3141 */
  0x0020,   /* U+3142 */
  0x0020,   /* U+3143 */
  0x0020,   /* U+3144 */
  0x0020,   /* U+3145 */
  0x0020,   /* U+3146 */
  0x0020,   /* U+3147 */
  0x0020,   /* U+3148 */
  0x0020,   /* U+3149 */
  0x0020,   /* U+314A */
  0x0020,   /* U+314B */
  0x0020,   /* U+314C */
  0x0020,   /* U+314D */
  0x0020,   /* U+314E */
  0x0020,   /* U+314F */
  0x0020,   /* U+3150 */
  0x0020,   /* U+3151 */
  0x0020,   /* U+3152 */
  0x0020,   /* U+3153 */
  0x0020,   /* U+3154 */
  0x0020,   /* U+3155 */
  0x0020,   /* U+3156 */
  0x0020,   /* U+3157 */
  0x0020,   /* U+3158 */
  0x0020,   /* U+3159 */
  0x0020,   /* U+315A */
  0x0020,   /* U+315B */
  0x0020,   /* U+315C */
  0x0020,   /* U+315D */
  0x0020,   /* U+315E */
  0x0020,   /* U+315F */
  0x0020,   /* U+3160 */
  0x0020,   /* U+3161 */
  0x0020,   /* U+3162 */
  0x0020,   /* U+3163 */
  0x0020,   /* U+3164 */
  0x0020,   /* U+3165 */
  0x0020,   /* U+3166 */
  0x0020,   /* U+3167 */
  0x0020,   /* U+3168 */
  0x0020,   /* U+3169 */
  0x0020,   /* U+316A */
  0x0020,   /* U+316B */
  0x0020,   /* U+316C */
  0x0020,   /* U+316D */
  0x0020,   /* U+316E */
  0x0020,   /* U+316F */
  0x0020,   /* U+3170 */
  0x0020,   /* U+3171 */
  0x0020,   /* U+3172 */
  0x0020,   /* U+3173 */
  0x0020,   /* U+3174 */
  0x0020,   /* U+3175 */
  0x0020,   /* U+3176 */
  0x0020,   /* U+3177 */
  0x0020,   /* U+3178 */
  0x0020,   /* U+3179 */
  0x0020,   /* U+317A */
  0x0020,   /* U+317B */
  0x0020,   /* U+317C */
  0x0020,   /* U+317D */
  0x0020,   /* U+317E */
  0x0020,   /* U+317F */
  0x0020,   /* U+3180 */
  0x0020,   /* U+3181 */
  0x0020,   /* U+3182 */
  0x0020,   /* U+3183 */
  0x0020,   /* U+3184 */
  0x0020,   /* U+3185 */
  0x0020,   /* U+3186 */
  0x0020,   /* U+3187 */
  0x0020,   /* U+3188 */
  0x0020,   /* U+3189 */
  0x0020,   /* U+318A */
  0x0020,   /* U+318B */
  0x0020,   /* U+318C */
  0x0020,   /* U+318D */
  0x0020,   /* U+318E */
  0x0020,   /* U+318F */
  0x0020,   /* U+3190 */
  0x0020,   /* U+3191 */
  0x0020,   /* U+3192 */
  0x0020,   /* U+3193 */
  0x0020,   /* U+3194 */
  0x0020,   /* U+3195 */
  0x0020,   /* U+3196 */
  0x0020,   /* U+3197 */
  0x0020,   /* U+3198 */
  0x0020,   /* U+3199 */
  0x0020,   /* U+319A */
  0x0020,   /* U+319B */
  0x0020,   /* U+319C */
  0x0020,   /* U+319D */
  0x0020,   /* U+319E */
  0x0020,   /* U+319F */
  0x0020,   /* U+31A0 */
  0x0020,   /* U+31A1 */
  0x0020,   /* U+31A2 */
  0x0020,   /* U+31A3 */
  0x0020,   /* U+31A4 */
  0x0020,   /* U+31A5 */
  0x0020,   /* U+31A6 */
  0x0020,   /* U+31A7 */
  0x0020,   /* U+31A8 */
  0x0020,   /* U+31A9 */
  0x0020,   /* U+31AA */
  0x0020,   /* U+31AB */
  0x0020,   /* U+31AC */
  0x0020,   /* U+31AD */
  0x0020,   /* U+31AE */
  0x0020,   /* U+31AF */
  0x0020,   /* U+31B0 */
  0x0020,   /* U+31B1 */
  0x0020,   /* U+31B2 */
  0x0020,   /* U+31B3 */
  0x0020,   /* U+31B4 */
  0x0020,   /* U+31B5 */
  0x0020,   /* U+31B6 */
  0x0020,   /* U+31B7 */
  0x0020,   /* U+31B8 */
  0x0020,   /* U+31B9 */
  0x0020,   /* U+31BA */
  0x0020,   /* U+31BB */
  0x0020,   /* U+31BC */
  0x0020,   /* U+31BD */
  0x0020,   /* U+31BE */
  0x0020,   /* U+31BF */
  0x0020,   /* U+31C0 */
  0x0020,   /* U+31C1 */
  0x0020,   /* U+31C2 */
  0x0020,   /* U+31C3 */
  0x0020,   /* U+31C4 */
  0x0020,   /* U+31C5 */
  0x0020,   /* U+31C6 */
  0x0020,   /* U+31C7 */
  0x0020,   /* U+31C8 */
  0x0020,   /* U+31C9 */
  0x0020,   /* U+31CA */
  0x0020,   /* U+31CB */
  0x0020,   /* U+31CC */
  0x0020,   /* U+31CD */
  0x0020,   /* U+31CE */
  0x0020,   /* U+31CF */
  0x0020,   /* U+31D0 */
  0x0020,   /* U+31D1 */
  0x0020,   /* U+31D2 */
  0x0020,   /* U+31D3 */
  0x0020,   /* U+31D4 */
  0x0020,   /* U+31D5 */
  0x0020,   /* U+31D6 */
  0x0020,   /* U+31D7 */
  0x0020,   /* U+31D8 */
  0x0020,   /* U+31D9 */
  0x0020,   /* U+31DA */
  0x0020,   /* U+31DB */
  0x0020,   /* U+31DC */
  0x0020,   /* U+31DD */
  0x0020,   /* U+31DE */
  0x0020,   /* U+31DF */
  0x0020,   /* U+31E0 */
  0x0020,   /* U+31E1 */
  0x0020,   /* U+31E2 */
  0x0020,   /* U+31E3 */
  0x0020,   /* U+31E4 */
  0x0020,   /* U+31E5 */
  0x0020,   /* U+31E6 */
  0x0020,   /* U+31E7 */
  0x0020,   /* U+31E8 */
  0x0020,   /* U+31E9 */
  0x0020,   /* U+31EA */
  0x0020,   /* U+31EB */
  0x0020,   /* U+31EC */
  0x0020,   /* U+31ED */
  0x0020,   /* U+31EE */
  0x0020,   /* U+31EF */
  0x0020,   /* U+31F0 */
  0x0020,   /* U+31F1 */
  0x0020,   /* U+31F2 */
  0x0020,   /* U+31F3 */
  0x0020,   /* U+31F4 */
  0x0020,   /* U+31F5 */
  0x0020,   /* U+31F6 */
  0x0020,   /* U+31F7 */
  0x0020,   /* U+31F8 */
  0x0020,   /* U+31F9 */
  0x0020,   /* U+31FA */
  0x0020,   /* U+31FB */
  0x0020,   /* U+31FC */
  0x0020,   /* U+31FD */
  0x0020,   /* U+31FE */
  0x0020,   /* U+31FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+3100 */
  0x0002,   /* U+3101 */
  0x0002,   /* U+3102 */
  0x0002,   /* U+3103 */
  0x0002,   /* U+3104 */
  0x0002,   /* U+3105 */
  0x0002,   /* U+3106 */
  0x0002,   /* U+3107 */
  0x0002,   /* U+3108 */
  0x0002,   /* U+3109 */
  0x0002,   /* U+310A */
  0x0002,   /* U+310B */
  0x0002,   /* U+310C */
  0x0002,   /* U+310D */
  0x0002,   /* U+310E */
  0x0002,   /* U+310F */
  0x0002,   /* U+3110 */
  0x0002,   /* U+3111 */
  0x0002,   /* U+3112 */
  0x0002,   /* U+3113 */
  0x0002,   /* U+3114 */
  0x0002,   /* U+3115 */
  0x0002,   /* U+3116 */
  0x0002,   /* U+3117 */
  0x0002,   /* U+3118 */
  0x0002,   /* U+3119 */
  0x0002,   /* U+311A */
  0x0002,   /* U+311B */
  0x0002,   /* U+311C */
  0x0002,   /* U+311D */
  0x0002,   /* U+311E */
  0x0002,   /* U+311F */
  0x0002,   /* U+3120 */
  0x0002,   /* U+3121 */
  0x0002,   /* U+3122 */
  0x0002,   /* U+3123 */
  0x0002,   /* U+3124 */
  0x0002,   /* U+3125 */
  0x0002,   /* U+3126 */
  0x0002,   /* U+3127 */
  0x0002,   /* U+3128 */
  0x0002,   /* U+3129 */
  0x0002,   /* U+312A */
  0x0002,   /* U+312B */
  0x0002,   /* U+312C */
  0x0002,   /* U+312D */
  0x0002,   /* U+312E */
  0x0002,   /* U+312F */
  0x0002,   /* U+3130 */
  0x0004,   /* U+3131 */
  0x0004,   /* U+3132 */
  0x0004,   /* U+3133 */
  0x0004,   /* U+3134 */
  0x0004,   /* U+3135 */
  0x0004,   /* U+3136 */
  0x0004,   /* U+3137 */
  0x0004,   /* U+3138 */
  0x0004,   /* U+3139 */
  0x0004,   /* U+313A */
  0x0004,   /* U+313B */
  0x0004,   /* U+313C */
  0x0004,   /* U+313D */
  0x0004,   /* U+313E */
  0x0004,   /* U+313F */
  0x0004,   /* U+3140 */
  0x0004,   /* U+3141 */
  0x0004,   /* U+3142 */
  0x0004,   /* U+3143 */
  0x0004,   /* U+3144 */
  0x0004,   /* U+3145 */
  0x0004,   /* U+3146 */
  0x0004,   /* U+3147 */
  0x0004,   /* U+3148 */
  0x0004,   /* U+3149 */
  0x0004,   /* U+314A */
  0x0004,   /* U+314B */
  0x0004,   /* U+314C */
  0x0004,   /* U+314D */
  0x0004,   /* U+314E */
  0x0004,   /* U+314F */
  0x0004,   /* U+3150 */
  0x0004,   /* U+3151 */
  0x0004,   /* U+3152 */
  0x0004,   /* U+3153 */
  0x0004,   /* U+3154 */
  0x0004,   /* U+3155 */
  0x0004,   /* U+3156 */
  0x0004,   /* U+3157 */
  0x0004,   /* U+3158 */
  0x0004,   /* U+3159 */
  0x0004,   /* U+315A */
  0x0004,   /* U+315B */
  0x0004,   /* U+315C */
  0x0004,   /* U+315D */
  0x0004,   /* U+315E */
  0x0004,   /* U+315F */
  0x0004,   /* U+3160 */
  0x0004,   /* U+3161 */
  0x0004,   /* U+3162 */
  0x0004,   /* U+3163 */
  0x0004,   /* U+3164 */
  0x0004,   /* U+3165 */
  0x0004,   /* U+3166 */
  0x0004,   /* U+3167 */
  0x0004,   /* U+3168 */
  0x0004,   /* U+3169 */
  0x0004,   /* U+316A */
  0x0004,   /* U+316B */
  0x0004,   /* U+316C */
  0x0004,   /* U+316D */
  0x0004,   /* U+316E */
  0x0004,   /* U+316F */
  0x0004,   /* U+3170 */
  0x0004,   /* U+3171 */
  0x0004,   /* U+3172 */
  0x0004,   /* U+3173 */
  0x0004,   /* U+3174 */
  0x0004,   /* U+3175 */
  0x0004,   /* U+3176 */
  0x0004,   /* U+3177 */
  0x0004,   /* U+3178 */
  0x0004,   /* U+3179 */
  0x0004,   /* U+317A */
  0x0004,   /* U+317B */
  0x0004,   /* U+317C */
  0x0004,   /* U+317D */
  0x0004,   /* U+317E */
  0x0004,   /* U+317F */
  0x0004,   /* U+3180 */
  0x0004,   /* U+3181 */
  0x0004,   /* U+3182 */
  0x0004,   /* U+3183 */
  0x0004,   /* U+3184 */
  0x0004,   /* U+3185 */
  0x0004,   /* U+3186 */
  0x0004,   /* U+3187 */
  0x0004,   /* U+3188 */
  0x0004,   /* U+3189 */
  0x0004,   /* U+318A */
  0x0004,   /* U+318B */
  0x0004,   /* U+318C */
  0x0004,   /* U+318D */
  0x0004,   /* U+318E */
  0x0002,   /* U+318F */
  0x0002,   /* U+3190 */
  0x0002,   /* U+3191 */
  0x0014,   /* U+3192 */
  0x0014,   /* U+3193 */
  0x0014,   /* U+3194 */
  0x0014,   /* U+3195 */
  0x0014,   /* U+3196 */
  0x0014,   /* U+3197 */
  0x0014,   /* U+3198 */
  0x0014,   /* U+3199 */
  0x0014,   /* U+319A */
  0x0014,   /* U+319B */
  0x0014,   /* U+319C */
  0x0014,   /* U+319D */
  0x0014,   /* U+319E */
  0x0014,   /* U+319F */
  0x0004,   /* U+31A0 */
  0x0004,   /* U+31A1 */
  0x0004,   /* U+31A2 */
  0x0004,   /* U+31A3 */
  0x0002,   /* U+31A4 */
  0x0004,   /* U+31A5 */
  0x0002,   /* U+31A6 */
  0x0004,   /* U+31A7 */
  0x0004,   /* U+31A8 */
  0x0004,   /* U+31A9 */
  0x0004,   /* U+31AA */
  0x0004,   /* U+31AB */
  0x0002,   /* U+31AC */
  0x0002,   /* U+31AD */
  0x0004,   /* U+31AE */
  0x0004,   /* U+31AF */
  0x0002,   /* U+31B0 */
  0x0002,   /* U+31B1 */
  0x0002,   /* U+31B2 */
  0x0016,   /* U+31B3 */
  0x0019,   /* U+31B4 */
  0x0019,   /* U+31B5 */
  0x0019,   /* U+31B6 */
  0x0019,   /* U+31B7 */
  0x0002,   /* U+31B8 */
  0x0002,   /* U+31B9 */
  0x0002,   /* U+31BA */
  0x0002,   /* U+31BB */
  0x0002,   /* U+31BC */
  0x0002,   /* U+31BD */
  0x0002,   /* U+31BE */
  0x0002,   /* U+31BF */
  0x0002,   /* U+31C0 */
  0x0002,   /* U+31C1 */
  0x0002,   /* U+31C2 */
  0x0002,   /* U+31C3 */
  0x0002,   /* U+31C4 */
  0x0002,   /* U+31C5 */
  0x0002,   /* U+31C6 */
  0x0002,   /* U+31C7 */
  0x0002,   /* U+31C8 */
  0x0002,   /* U+31C9 */
  0x0002,   /* U+31CA */
  0x0002,   /* U+31CB */
  0x0002,   /* U+31CC */
  0x0002,   /* U+31CD */
  0x0002,   /* U+31CE */
  0x0002,   /* U+31CF */
  0x0002,   /* U+31D0 */
  0x0002,   /* U+31D1 */
  0x0002,   /* U+31D2 */
  0x0002,   /* U+31D3 */
  0x0002,   /* U+31D4 */
  0x0002,   /* U+31D5 */
  0x0002,   /* U+31D6 */
  0x0002,   /* U+31D7 */
  0x0002,   /* U+31D8 */
  0x0002,   /* U+31D9 */
  0x0002,   /* U+31DA */
  0x0002,   /* U+31DB */
  0x0002,   /* U+31DC */
  0x0002,   /* U+31DD */
  0x0002,   /* U+31DE */
  0x0002,   /* U+31DF */
  0x0002,   /* U+31E0 */
  0x0002,   /* U+31E1 */
  0x0002,   /* U+31E2 */
  0x0002,   /* U+31E3 */
  0x0002,   /* U+31E4 */
  0x0002,   /* U+31E5 */
  0x0002,   /* U+31E6 */
  0x0002,   /* U+31E7 */
  0x0002,   /* U+31E8 */
  0x0002,   /* U+31E9 */
  0x0002,   /* U+31EA */
  0x0002,   /* U+31EB */
  0x0002,   /* U+31EC */
  0x0002,   /* U+31ED */
  0x0002,   /* U+31EE */
  0x0002,   /* U+31EF */
  0x000F,   /* U+31F0 */
  0x000F,   /* U+31F1 */
  0x000F,   /* U+31F2 */
  0x000F,   /* U+31F3 */
  0x000F,   /* U+31F4 */
  0x000F,   /* U+31F5 */
  0x000F,   /* U+31F6 */
  0x000F,   /* U+31F7 */
  0x000F,   /* U+31F8 */
  0x000F,   /* U+31F9 */
  0x000F,   /* U+31FA */
  0x000F,   /* U+31FB */
  0x000F,   /* U+31FC */
  0x000F,   /* U+31FD */
  0x000F,   /* U+31FE */
  0x000F,   /* U+31FF */

  /* Primary weight 2 for each character. */
  0xB100,   /* U+3100 */
  0xB101,   /* U+3101 */
  0xB102,   /* U+3102 */
  0xB103,   /* U+3103 */
  0xB104,   /* U+3104 */
  0x0000,   /* U+3105 */
  0x0000,   /* U+3106 */
  0x0000,   /* U+3107 */
  0x0000,   /* U+3108 */
  0x0000,   /* U+3109 */
  0x0000,   /* U+310A */
  0x0000,   /* U+310B */
  0x0000,   /* U+310C */
  0x0000,   /* U+310D */
  0x0000,   /* U+310E */
  0x0000,   /* U+310F */
  0x0000,   /* U+3110 */
  0x0000,   /* U+3111 */
  0x0000,   /* U+3112 */
  0x0000,   /* U+3113 */
  0x0000,   /* U+3114 */
  0x0000,   /* U+3115 */
  0x0000,   /* U+3116 */
  0x0000,   /* U+3117 */
  0x0000,   /* U+3118 */
  0x0000,   /* U+3119 */
  0x0000,   /* U+311A */
  0x0000,   /* U+311B */
  0x0000,   /* U+311C */
  0x0000,   /* U+311D */
  0x0000,   /* U+311E */
  0x0000,   /* U+311F */
  0x0000,   /* U+3120 */
  0x0000,   /* U+3121 */
  0x0000,   /* U+3122 */
  0x0000,   /* U+3123 */
  0x0000,   /* U+3124 */
  0x0000,   /* U+3125 */
  0x0000,   /* U+3126 */
  0x0000,   /* U+3127 */
  0x0000,   /* U+3128 */
  0x0000,   /* U+3129 */
  0x0000,   /* U+312A */
  0x0000,   /* U+312B */
  0x0000,   /* U+312C */
  0x0000,   /* U+312D */
  0xB12E,   /* U+312E */
  0xB12F,   /* U+312F */
  0xB130,   /* U+3130 */
  0x0000,   /* U+3131 */
  0x0000,   /* U+3132 */
  0x0000,   /* U+3133 */
  0x0000,   /* U+3134 */
  0x0000,   /* U+3135 */
  0x0000,   /* U+3136 */
  0x0000,   /* U+3137 */
  0x0000,   /* U+3138 */
  0x0000,   /* U+3139 */
  0x0000,   /* U+313A */
  0x0000,   /* U+313B */
  0x0000,   /* U+313C */
  0x0000,   /* U+313D */
  0x0000,   /* U+313E */
  0x0000,   /* U+313F */
  0x0000,   /* U+3140 */
  0x0000,   /* U+3141 */
  0x0000,   /* U+3142 */
  0x0000,   /* U+3143 */
  0x0000,   /* U+3144 */
  0x0000,   /* U+3145 */
  0x0000,   /* U+3146 */
  0x0000,   /* U+3147 */
  0x0000,   /* U+3148 */
  0x0000,   /* U+3149 */
  0x0000,   /* U+314A */
  0x0000,   /* U+314B */
  0x0000,   /* U+314C */
  0x0000,   /* U+314D */
  0x0000,   /* U+314E */
  0x0000,   /* U+314F */
  0x0000,   /* U+3150 */
  0x0000,   /* U+3151 */
  0x0000,   /* U+3152 */
  0x0000,   /* U+3153 */
  0x0000,   /* U+3154 */
  0x0000,   /* U+3155 */
  0x0000,   /* U+3156 */
  0x0000,   /* U+3157 */
  0x0000,   /* U+3158 */
  0x0000,   /* U+3159 */
  0x0000,   /* U+315A */
  0x0000,   /* U+315B */
  0x0000,   /* U+315C */
  0x0000,   /* U+315D */
  0x0000,   /* U+315E */
  0x0000,   /* U+315F */
  0x0000,   /* U+3160 */
  0x0000,   /* U+3161 */
  0x0000,   /* U+3162 */
  0x0000,   /* U+3163 */
  0x0000,   /* U+3164 */
  0x0000,   /* U+3165 */
  0x0000,   /* U+3166 */
  0x0000,   /* U+3167 */
  0x0000,   /* U+3168 */
  0x0000,   /* U+3169 */
  0x0000,   /* U+316A */
  0x0000,   /* U+316B */
  0x0000,   /* U+316C */
  0x0000,   /* U+316D */
  0x0000,   /* U+316E */
  0x0000,   /* U+316F */
  0x0000,   /* U+3170 */
  0x0000,   /* U+3171 */
  0x0000,   /* U+3172 */
  0x0000,   /* U+3173 */
  0x0000,   /* U+3174 */
  0x0000,   /* U+3175 */
  0x0000,   /* U+3176 */
  0x0000,   /* U+3177 */
  0x0000,   /* U+3178 */
  0x0000,   /* U+3179 */
  0x0000,   /* U+317A */
  0x0000,   /* U+317B */
  0x0000,   /* U+317C */
  0x0000,   /* U+317D */
  0x0000,   /* U+317E */
  0x0000,   /* U+317F */
  0x0000,   /* U+3180 */
  0x0000,   /* U+3181 */
  0x0000,   /* U+3182 */
  0x0000,   /* U+3183 */
  0x0000,   /* U+3184 */
  0x0000,   /* U+3185 */
  0x0000,   /* U+3186 */
  0x0000,   /* U+3187 */
  0x0000,   /* U+3188 */
  0x0000,   /* U+3189 */
  0x0000,   /* U+318A */
  0x0000,   /* U+318B */
  0x0000,   /* U+318C */
  0x0000,   /* U+318D */
  0x0000,   /* U+318E */
  0xB18F,   /* U+318F */
  0x0000,   /* U+3190 */
  0x0000,   /* U+3191 */
  0xCE00,   /* U+3192 */
  0xCE8C,   /* U+3193 */
  0xCE09,   /* U+3194 */
  0xD6DB,   /* U+3195 */
  0xCE0A,   /* U+3196 */
  0xCE2D,   /* U+3197 */
  0xCE0B,   /* U+3198 */
  0xF532,   /* U+3199 */
  0xCE59,   /* U+319A */
  0xCE19,   /* U+319B */
  0xCE01,   /* U+319C */
  0xD929,   /* U+319D */
  0xD730,   /* U+319E */
  0xCEBA,   /* U+319F */
  0x0000,   /* U+31A0 */
  0x0000,   /* U+31A1 */
  0x0000,   /* U+31A2 */
  0x0000,   /* U+31A3 */
  0x0000,   /* U+31A4 */
  0x0000,   /* U+31A5 */
  0x0000,   /* U+31A6 */
  0x0000,   /* U+31A7 */
  0x0000,   /* U+31A8 */
  0x0000,   /* U+31A9 */
  0x0000,   /* U+31AA */
  0x0000,   /* U+31AB */
  0x0000,   /* U+31AC */
  0x0000,   /* U+31AD */
  0x0000,   /* U+31AE */
  0x0000,   /* U+31AF */
  0x0000,   /* U+31B0 */
  0x0000,   /* U+31B1 */
  0x0000,   /* U+31B2 */
  0x0000,   /* U+31B3 */
  0x0000,   /* U+31B4 */
  0x0000,   /* U+31B5 */
  0x0000,   /* U+31B6 */
  0x0000,   /* U+31B7 */
  0x0000,   /* U+31B8 */
  0x0000,   /* U+31B9 */
  0x0000,   /* U+31BA */
  0xB1BB,   /* U+31BB */
  0xB1BC,   /* U+31BC */
  0xB1BD,   /* U+31BD */
  0xB1BE,   /* U+31BE */
  0xB1BF,   /* U+31BF */
  0x0000,   /* U+31C0 */
  0x0000,   /* U+31C1 */
  0x0000,   /* U+31C2 */
  0x0000,   /* U+31C3 */
  0x0000,   /* U+31C4 */
  0x0000,   /* U+31C5 */
  0x0000,   /* U+31C6 */
  0x0000,   /* U+31C7 */
  0x0000,   /* U+31C8 */
  0x0000,   /* U+31C9 */
  0x0000,   /* U+31CA */
  0x0000,   /* U+31CB */
  0x0000,   /* U+31CC */
  0x0000,   /* U+31CD */
  0x0000,   /* U+31CE */
  0x0000,   /* U+31CF */
  0x0000,   /* U+31D0 */
  0x0000,   /* U+31D1 */
  0x0000,   /* U+31D2 */
  0x0000,   /* U+31D3 */
  0x0000,   /* U+31D4 */
  0x0000,   /* U+31D5 */
  0x0000,   /* U+31D6 */
  0x0000,   /* U+31D7 */
  0x0000,   /* U+31D8 */
  0x0000,   /* U+31D9 */
  0x0000,   /* U+31DA */
  0x0000,   /* U+31DB */
  0x0000,   /* U+31DC */
  0x0000,   /* U+31DD */
  0x0000,   /* U+31DE */
  0x0000,   /* U+31DF */
  0x0000,   /* U+31E0 */
  0x0000,   /* U+31E1 */
  0x0000,   /* U+31E2 */
  0x0000,   /* U+31E3 */
  0xB1E4,   /* U+31E4 */
  0xB1E5,   /* U+31E5 */
  0xB1E6,   /* U+31E6 */
  0xB1E7,   /* U+31E7 */
  0xB1E8,   /* U+31E8 */
  0xB1E9,   /* U+31E9 */
  0xB1EA,   /* U+31EA */
  0xB1EB,   /* U+31EB */
  0xB1EC,   /* U+31EC */
  0xB1ED,   /* U+31ED */
  0xB1EE,   /* U+31EE */
  0xB1EF,   /* U+31EF */
  0x0000,   /* U+31F0 */
  0x0000,   /* U+31F1 */
  0x0000,   /* U+31F2 */
  0x0000,   /* U+31F3 */
  0x0000,   /* U+31F4 */
  0x0000,   /* U+31F5 */
  0x0000,   /* U+31F6 */
  0x0000,   /* U+31F7 */
  0x0000,   /* U+31F8 */
  0x0000,   /* U+31F9 */
  0x0000,   /* U+31FA */
  0x0000,   /* U+31FB */
  0x0000,   /* U+31FC */
  0x0000,   /* U+31FD */
  0x0000,   /* U+31FE */
  0x0000,   /* U+31FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+3100 */
  0x0000,   /* U+3101 */
  0x0000,   /* U+3102 */
  0x0000,   /* U+3103 */
  0x0000,   /* U+3104 */
  0x0000,   /* U+3105 */
  0x0000,   /* U+3106 */
  0x0000,   /* U+3107 */
  0x0000,   /* U+3108 */
  0x0000,   /* U+3109 */
  0x0000,   /* U+310A */
  0x0000,   /* U+310B */
  0x0000,   /* U+310C */
  0x0000,   /* U+310D */
  0x0000,   /* U+310E */
  0x0000,   /* U+310F */
  0x0000,   /* U+3110 */
  0x0000,   /* U+3111 */
  0x0000,   /* U+3112 */
  0x0000,   /* U+3113 */
  0x0000,   /* U+3114 */
  0x0000,   /* U+3115 */
  0x0000,   /* U+3116 */
  0x0000,   /* U+3117 */
  0x0000,   /* U+3118 */
  0x0000,   /* U+3119 */
  0x0000,   /* U+311A */
  0x0000,   /* U+311B */
  0x0000,   /* U+311C */
  0x0000,   /* U+311D */
  0x0000,   /* U+311E */
  0x0000,   /* U+311F */
  0x0000,   /* U+3120 */
  0x0000,   /* U+3121 */
  0x0000,   /* U+3122 */
  0x0000,   /* U+3123 */
  0x0000,   /* U+3124 */
  0x0000,   /* U+3125 */
  0x0000,   /* U+3126 */
  0x0000,   /* U+3127 */
  0x0000,   /* U+3128 */
  0x0000,   /* U+3129 */
  0x0000,   /* U+312A */
  0x0000,   /* U+312B */
  0x0000,   /* U+312C */
  0x0000,   /* U+312D */
  0x0000,   /* U+312E */
  0x0000,   /* U+312F */
  0x0000,   /* U+3130 */
  0x0000,   /* U+3131 */
  0x0000,   /* U+3132 */
  0x0000,   /* U+3133 */
  0x0000,   /* U+3134 */
  0x0000,   /* U+3135 */
  0x0000,   /* U+3136 */
  0x0000,   /* U+3137 */
  0x0000,   /* U+3138 */
  0x0000,   /* U+3139 */
  0x0000,   /* U+313A */
  0x0000,   /* U+313B */
  0x0000,   /* U+313C */
  0x0000,   /* U+313D */
  0x0000,   /* U+313E */
  0x0000,   /* U+313F */
  0x0000,   /* U+3140 */
  0x0000,   /* U+3141 */
  0x0000,   /* U+3142 */
  0x0000,   /* U+3143 */
  0x0000,   /* U+3144 */
  0x0000,   /* U+3145 */
  0x0000,   /* U+3146 */
  0x0000,   /* U+3147 */
  0x0000,   /* U+3148 */
  0x0000,   /* U+3149 */
  0x0000,   /* U+314A */
  0x0000,   /* U+314B */
  0x0000,   /* U+314C */
  0x0000,   /* U+314D */
  0x0000,   /* U+314E */
  0x0000,   /* U+314F */
  0x0000,   /* U+3150 */
  0x0000,   /* U+3151 */
  0x0000,   /* U+3152 */
  0x0000,   /* U+3153 */
  0x0000,   /* U+3154 */
  0x0000,   /* U+3155 */
  0x0000,   /* U+3156 */
  0x0000,   /* U+3157 */
  0x0000,   /* U+3158 */
  0x0000,   /* U+3159 */
  0x0000,   /* U+315A */
  0x0000,   /* U+315B */
  0x0000,   /* U+315C */
  0x0000,   /* U+315D */
  0x0000,   /* U+315E */
  0x0000,   /* U+315F */
  0x0000,   /* U+3160 */
  0x0000,   /* U+3161 */
  0x0000,   /* U+3162 */
  0x0000,   /* U+3163 */
  0x0000,   /* U+3164 */
  0x0000,   /* U+3165 */
  0x0000,   /* U+3166 */
  0x0000,   /* U+3167 */
  0x0000,   /* U+3168 */
  0x0000,   /* U+3169 */
  0x0000,   /* U+316A */
  0x0000,   /* U+316B */
  0x0000,   /* U+316C */
  0x0000,   /* U+316D */
  0x0000,   /* U+316E */
  0x0000,   /* U+316F */
  0x0000,   /* U+3170 */
  0x0000,   /* U+3171 */
  0x0000,   /* U+3172 */
  0x0000,   /* U+3173 */
  0x0000,   /* U+3174 */
  0x0000,   /* U+3175 */
  0x0000,   /* U+3176 */
  0x0000,   /* U+3177 */
  0x0000,   /* U+3178 */
  0x0000,   /* U+3179 */
  0x0000,   /* U+317A */
  0x0000,   /* U+317B */
  0x0000,   /* U+317C */
  0x0000,   /* U+317D */
  0x0000,   /* U+317E */
  0x0000,   /* U+317F */
  0x0000,   /* U+3180 */
  0x0000,   /* U+3181 */
  0x0000,   /* U+3182 */
  0x0000,   /* U+3183 */
  0x0000,   /* U+3184 */
  0x0000,   /* U+3185 */
  0x0000,   /* U+3186 */
  0x0000,   /* U+3187 */
  0x0000,   /* U+3188 */
  0x0000,   /* U+3189 */
  0x0000,   /* U+318A */
  0x0000,   /* U+318B */
  0x0000,   /* U+318C */
  0x0000,   /* U+318D */
  0x0000,   /* U+318E */
  0x0000,   /* U+318F */
  0x0000,   /* U+3190 */
  0x0000,   /* U+3191 */
  0x0000,   /* U+3192 */
  0x0000,   /* U+3193 */
  0x0000,   /* U+3194 */
  0x0000,   /* U+3195 */
  0x0000,   /* U+3196 */
  0x0000,   /* U+3197 */
  0x0000,   /* U+3198 */
  0x0000,   /* U+3199 */
  0x0000,   /* U+319A */
  0x0000,   /* U+319B */
  0x0000,   /* U+319C */
  0x0000,   /* U+319D */
  0x0000,   /* U+319E */
  0x0000,   /* U+319F */
  0x0111,   /* U+31A0 */
  0x0111,   /* U+31A1 */
  0x0111,   /* U+31A2 */
  0x0111,   /* U+31A3 */
  0x0000,   /* U+31A4 */
  0x0111,   /* U+31A5 */
  0x0000,   /* U+31A6 */
  0x0111,   /* U+31A7 */
  0x0112,   /* U+31A8 */
  0x0111,   /* U+31A9 */
  0x0111,   /* U+31AA */
  0x0111,   /* U+31AB */
  0x0000,   /* U+31AC */
  0x0000,   /* U+31AD */
  0x0111,   /* U+31AE */
  0x0111,   /* U+31AF */
  0x0000,   /* U+31B0 */
  0x0000,   /* U+31B1 */
  0x0000,   /* U+31B2 */
  0x0111,   /* U+31B3 */
  0x0000,   /* U+31B4 */
  0x0000,   /* U+31B5 */
  0x0000,   /* U+31B6 */
  0x0000,   /* U+31B7 */
  0x0000,   /* U+31B8 */
  0x0000,   /* U+31B9 */
  0x0000,   /* U+31BA */
  0x0000,   /* U+31BB */
  0x0000,   /* U+31BC */
  0x0000,   /* U+31BD */
  0x0000,   /* U+31BE */
  0x0000,   /* U+31BF */
  0x0000,   /* U+31C0 */
  0x0000,   /* U+31C1 */
  0x0000,   /* U+31C2 */
  0x0000,   /* U+31C3 */
  0x0000,   /* U+31C4 */
  0x0000,   /* U+31C5 */
  0x0000,   /* U+31C6 */
  0x0000,   /* U+31C7 */
  0x0000,   /* U+31C8 */
  0x0000,   /* U+31C9 */
  0x0000,   /* U+31CA */
  0x0000,   /* U+31CB */
  0x0000,   /* U+31CC */
  0x0000,   /* U+31CD */
  0x0000,   /* U+31CE */
  0x0000,   /* U+31CF */
  0x0000,   /* U+31D0 */
  0x0000,   /* U+31D1 */
  0x0000,   /* U+31D2 */
  0x0000,   /* U+31D3 */
  0x0000,   /* U+31D4 */
  0x0000,   /* U+31D5 */
  0x0000,   /* U+31D6 */
  0x0000,   /* U+31D7 */
  0x0000,   /* U+31D8 */
  0x0000,   /* U+31D9 */
  0x0000,   /* U+31DA */
  0x0000,   /* U+31DB */
  0x0000,   /* U+31DC */
  0x0000,   /* U+31DD */
  0x0000,   /* U+31DE */
  0x0000,   /* U+31DF */
  0x0000,   /* U+31E0 */
  0x0000,   /* U+31E1 */
  0x0000,   /* U+31E2 */
  0x0000,   /* U+31E3 */
  0x0000,   /* U+31E4 */
  0x0000,   /* U+31E5 */
  0x0000,   /* U+31E6 */
  0x0000,   /* U+31E7 */
  0x0000,   /* U+31E8 */
  0x0000,   /* U+31E9 */
  0x0000,   /* U+31EA */
  0x0000,   /* U+31EB */
  0x0000,   /* U+31EC */
  0x0000,   /* U+31ED */
  0x0000,   /* U+31EE */
  0x0000,   /* U+31EF */
  0x0000,   /* U+31F0 */
  0x0000,   /* U+31F1 */
  0x0000,   /* U+31F2 */
  0x0000,   /* U+31F3 */
  0x0000,   /* U+31F4 */
  0x0000,   /* U+31F5 */
  0x0000,   /* U+31F6 */
  0x0000,   /* U+31F7 */
  0x0000,   /* U+31F8 */
  0x0000,   /* U+31F9 */
  0x0000,   /* U+31FA */
  0x0000,   /* U+31FB */
  0x0000,   /* U+31FC */
  0x0000,   /* U+31FD */
  0x0000,   /* U+31FE */
  0x0000,   /* U+31FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+3100 */
  0x0000,   /* U+3101 */
  0x0000,   /* U+3102 */
  0x0000,   /* U+3103 */
  0x0000,   /* U+3104 */
  0x0000,   /* U+3105 */
  0x0000,   /* U+3106 */
  0x0000,   /* U+3107 */
  0x0000,   /* U+3108 */
  0x0000,   /* U+3109 */
  0x0000,   /* U+310A */
  0x0000,   /* U+310B */
  0x0000,   /* U+310C */
  0x0000,   /* U+310D */
  0x0000,   /* U+310E */
  0x0000,   /* U+310F */
  0x0000,   /* U+3110 */
  0x0000,   /* U+3111 */
  0x0000,   /* U+3112 */
  0x0000,   /* U+3113 */
  0x0000,   /* U+3114 */
  0x0000,   /* U+3115 */
  0x0000,   /* U+3116 */
  0x0000,   /* U+3117 */
  0x0000,   /* U+3118 */
  0x0000,   /* U+3119 */
  0x0000,   /* U+311A */
  0x0000,   /* U+311B */
  0x0000,   /* U+311C */
  0x0000,   /* U+311D */
  0x0000,   /* U+311E */
  0x0000,   /* U+311F */
  0x0000,   /* U+3120 */
  0x0000,   /* U+3121 */
  0x0000,   /* U+3122 */
  0x0000,   /* U+3123 */
  0x0000,   /* U+3124 */
  0x0000,   /* U+3125 */
  0x0000,   /* U+3126 */
  0x0000,   /* U+3127 */
  0x0000,   /* U+3128 */
  0x0000,   /* U+3129 */
  0x0000,   /* U+312A */
  0x0000,   /* U+312B */
  0x0000,   /* U+312C */
  0x0000,   /* U+312D */
  0x0000,   /* U+312E */
  0x0000,   /* U+312F */
  0x0000,   /* U+3130 */
  0x0000,   /* U+3131 */
  0x0000,   /* U+3132 */
  0x0000,   /* U+3133 */
  0x0000,   /* U+3134 */
  0x0000,   /* U+3135 */
  0x0000,   /* U+3136 */
  0x0000,   /* U+3137 */
  0x0000,   /* U+3138 */
  0x0000,   /* U+3139 */
  0x0000,   /* U+313A */
  0x0000,   /* U+313B */
  0x0000,   /* U+313C */
  0x0000,   /* U+313D */
  0x0000,   /* U+313E */
  0x0000,   /* U+313F */
  0x0000,   /* U+3140 */
  0x0000,   /* U+3141 */
  0x0000,   /* U+3142 */
  0x0000,   /* U+3143 */
  0x0000,   /* U+3144 */
  0x0000,   /* U+3145 */
  0x0000,   /* U+3146 */
  0x0000,   /* U+3147 */
  0x0000,   /* U+3148 */
  0x0000,   /* U+3149 */
  0x0000,   /* U+314A */
  0x0000,   /* U+314B */
  0x0000,   /* U+314C */
  0x0000,   /* U+314D */
  0x0000,   /* U+314E */
  0x0000,   /* U+314F */
  0x0000,   /* U+3150 */
  0x0000,   /* U+3151 */
  0x0000,   /* U+3152 */
  0x0000,   /* U+3153 */
  0x0000,   /* U+3154 */
  0x0000,   /* U+3155 */
  0x0000,   /* U+3156 */
  0x0000,   /* U+3157 */
  0x0000,   /* U+3158 */
  0x0000,   /* U+3159 */
  0x0000,   /* U+315A */
  0x0000,   /* U+315B */
  0x0000,   /* U+315C */
  0x0000,   /* U+315D */
  0x0000,   /* U+315E */
  0x0000,   /* U+315F */
  0x0000,   /* U+3160 */
  0x0000,   /* U+3161 */
  0x0000,   /* U+3162 */
  0x0000,   /* U+3163 */
  0x0000,   /* U+3164 */
  0x0000,   /* U+3165 */
  0x0000,   /* U+3166 */
  0x0000,   /* U+3167 */
  0x0000,   /* U+3168 */
  0x0000,   /* U+3169 */
  0x0000,   /* U+316A */
  0x0000,   /* U+316B */
  0x0000,   /* U+316C */
  0x0000,   /* U+316D */
  0x0000,   /* U+316E */
  0x0000,   /* U+316F */
  0x0000,   /* U+3170 */
  0x0000,   /* U+3171 */
  0x0000,   /* U+3172 */
  0x0000,   /* U+3173 */
  0x0000,   /* U+3174 */
  0x0000,   /* U+3175 */
  0x0000,   /* U+3176 */
  0x0000,   /* U+3177 */
  0x0000,   /* U+3178 */
  0x0000,   /* U+3179 */
  0x0000,   /* U+317A */
  0x0000,   /* U+317B */
  0x0000,   /* U+317C */
  0x0000,   /* U+317D */
  0x0000,   /* U+317E */
  0x0000,   /* U+317F */
  0x0000,   /* U+3180 */
  0x0000,   /* U+3181 */
  0x0000,   /* U+3182 */
  0x0000,   /* U+3183 */
  0x0000,   /* U+3184 */
  0x0000,   /* U+3185 */
  0x0000,   /* U+3186 */
  0x0000,   /* U+3187 */
  0x0000,   /* U+3188 */
  0x0000,   /* U+3189 */
  0x0000,   /* U+318A */
  0x0000,   /* U+318B */
  0x0000,   /* U+318C */
  0x0000,   /* U+318D */
  0x0000,   /* U+318E */
  0x0000,   /* U+318F */
  0x0000,   /* U+3190 */
  0x0000,   /* U+3191 */
  0x0000,   /* U+3192 */
  0x0000,   /* U+3193 */
  0x0000,   /* U+3194 */
  0x0000,   /* U+3195 */
  0x0000,   /* U+3196 */
  0x0000,   /* U+3197 */
  0x0000,   /* U+3198 */
  0x0000,   /* U+3199 */
  0x0000,   /* U+319A */
  0x0000,   /* U+319B */
  0x0000,   /* U+319C */
  0x0000,   /* U+319D */
  0x0000,   /* U+319E */
  0x0000,   /* U+319F */
  0x0004,   /* U+31A0 */
  0x0004,   /* U+31A1 */
  0x0004,   /* U+31A2 */
  0x0004,   /* U+31A3 */
  0x0000,   /* U+31A4 */
  0x0004,   /* U+31A5 */
  0x0000,   /* U+31A6 */
  0x0004,   /* U+31A7 */
  0x0004,   /* U+31A8 */
  0x0004,   /* U+31A9 */
  0x0004,   /* U+31AA */
  0x0004,   /* U+31AB */
  0x0000,   /* U+31AC */
  0x0000,   /* U+31AD */
  0x0004,   /* U+31AE */
  0x0004,   /* U+31AF */
  0x0000,   /* U+31B0 */
  0x0000,   /* U+31B1 */
  0x0000,   /* U+31B2 */
  0x0016,   /* U+31B3 */
  0x0000,   /* U+31B4 */
  0x0000,   /* U+31B5 */
  0x0000,   /* U+31B6 */
  0x0000,   /* U+31B7 */
  0x0000,   /* U+31B8 */
  0x0000,   /* U+31B9 */
  0x0000,   /* U+31BA */
  0x0000,   /* U+31BB */
  0x0000,   /* U+31BC */
  0x0000,   /* U+31BD */
  0x0000,   /* U+31BE */
  0x0000,   /* U+31BF */
  0x0000,   /* U+31C0 */
  0x0000,   /* U+31C1 */
  0x0000,   /* U+31C2 */
  0x0000,   /* U+31C3 */
  0x0000,   /* U+31C4 */
  0x0000,   /* U+31C5 */
  0x0000,   /* U+31C6 */
  0x0000,   /* U+31C7 */
  0x0000,   /* U+31C8 */
  0x0000,   /* U+31C9 */
  0x0000,   /* U+31CA */
  0x0000,   /* U+31CB */
  0x0000,   /* U+31CC */
  0x0000,   /* U+31CD */
  0x0000,   /* U+31CE */
  0x0000,   /* U+31CF */
  0x0000,   /* U+31D0 */
  0x0000,   /* U+31D1 */
  0x0000,   /* U+31D2 */
  0x0000,   /* U+31D3 */
  0x0000,   /* U+31D4 */
  0x0000,   /* U+31D5 */
  0x0000,   /* U+31D6 */
  0x0000,   /* U+31D7 */
  0x0000,   /* U+31D8 */
  0x0000,   /* U+31D9 */
  0x0000,   /* U+31DA */
  0x0000,   /* U+31DB */
  0x0000,   /* U+31DC */
  0x0000,   /* U+31DD */
  0x0000,   /* U+31DE */
  0x0000,   /* U+31DF */
  0x0000,   /* U+31E0 */
  0x0000,   /* U+31E1 */
  0x0000,   /* U+31E2 */
  0x0000,   /* U+31E3 */
  0x0000,   /* U+31E4 */
  0x0000,   /* U+31E5 */
  0x0000,   /* U+31E6 */
  0x0000,   /* U+31E7 */
  0x0000,   /* U+31E8 */
  0x0000,   /* U+31E9 */
  0x0000,   /* U+31EA */
  0x0000,   /* U+31EB */
  0x0000,   /* U+31EC */
  0x0000,   /* U+31ED */
  0x0000,   /* U+31EE */
  0x0000,   /* U+31EF */
  0x0000,   /* U+31F0 */
  0x0000,   /* U+31F1 */
  0x0000,   /* U+31F2 */
  0x0000,   /* U+31F3 */
  0x0000,   /* U+31F4 */
  0x0000,   /* U+31F5 */
  0x0000,   /* U+31F6 */
  0x0000,   /* U+31F7 */
  0x0000,   /* U+31F8 */
  0x0000,   /* U+31F9 */
  0x0000,   /* U+31FA */
  0x0000,   /* U+31FB */
  0x0000,   /* U+31FC */
  0x0000,   /* U+31FD */
  0x0000,   /* U+31FE */
  0x0000,   /* U+31FF */
};

uint16 uca900_p032[]= {
  /* Number of CEs for each character. */
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 7, 6, 2, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 5, 4, 2, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 2, 3, 2, 3, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 

  /* Primary weight 1 for each character. */
  0x0317,   /* U+3200 */
  0x0317,   /* U+3201 */
  0x0317,   /* U+3202 */
  0x0317,   /* U+3203 */
  0x0317,   /* U+3204 */
  0x0317,   /* U+3205 */
  0x0317,   /* U+3206 */
  0x0317,   /* U+3207 */
  0x0317,   /* U+3208 */
  0x0317,   /* U+3209 */
  0x0317,   /* U+320A */
  0x0317,   /* U+320B */
  0x0317,   /* U+320C */
  0x0317,   /* U+320D */
  0x0317,   /* U+320E */
  0x0317,   /* U+320F */
  0x0317,   /* U+3210 */
  0x0317,   /* U+3211 */
  0x0317,   /* U+3212 */
  0x0317,   /* U+3213 */
  0x0317,   /* U+3214 */
  0x0317,   /* U+3215 */
  0x0317,   /* U+3216 */
  0x0317,   /* U+3217 */
  0x0317,   /* U+3218 */
  0x0317,   /* U+3219 */
  0x0317,   /* U+321A */
  0x0317,   /* U+321B */
  0x0317,   /* U+321C */
  0x0317,   /* U+321D */
  0x0317,   /* U+321E */
  0xFBC0,   /* U+321F */
  0x0317,   /* U+3220 */
  0x0317,   /* U+3221 */
  0x0317,   /* U+3222 */
  0x0317,   /* U+3223 */
  0x0317,   /* U+3224 */
  0x0317,   /* U+3225 */
  0x0317,   /* U+3226 */
  0x0317,   /* U+3227 */
  0x0317,   /* U+3228 */
  0x0317,   /* U+3229 */
  0x0317,   /* U+322A */
  0x0317,   /* U+322B */
  0x0317,   /* U+322C */
  0x0317,   /* U+322D */
  0x0317,   /* U+322E */
  0x0317,   /* U+322F */
  0x0317,   /* U+3230 */
  0x0317,   /* U+3231 */
  0x0317,   /* U+3232 */
  0x0317,   /* U+3233 */
  0x0317,   /* U+3234 */
  0x0317,   /* U+3235 */
  0x0317,   /* U+3236 */
  0x0317,   /* U+3237 */
  0x0317,   /* U+3238 */
  0x0317,   /* U+3239 */
  0x0317,   /* U+323A */
  0x0317,   /* U+323B */
  0x0317,   /* U+323C */
  0x0317,   /* U+323D */
  0x0317,   /* U+323E */
  0x0317,   /* U+323F */
  0x0317,   /* U+3240 */
  0x0317,   /* U+3241 */
  0x0317,   /* U+3242 */
  0x0317,   /* U+3243 */
  0xFB40,   /* U+3244 */
  0xFB40,   /* U+3245 */
  0xFB40,   /* U+3246 */
  0xFB40,   /* U+3247 */
  0x1C3E,   /* U+3248 */
  0x1C3F,   /* U+3249 */
  0x1C40,   /* U+324A */
  0x1C41,   /* U+324B */
  0x1C42,   /* U+324C */
  0x1C43,   /* U+324D */
  0x1C44,   /* U+324E */
  0x1C45,   /* U+324F */
  0x1E0C,   /* U+3250 */
  0x1C3F,   /* U+3251 */
  0x1C3F,   /* U+3252 */
  0x1C3F,   /* U+3253 */
  0x1C3F,   /* U+3254 */
  0x1C3F,   /* U+3255 */
  0x1C3F,   /* U+3256 */
  0x1C3F,   /* U+3257 */
  0x1C3F,   /* U+3258 */
  0x1C3F,   /* U+3259 */
  0x1C40,   /* U+325A */
  0x1C40,   /* U+325B */
  0x1C40,   /* U+325C */
  0x1C40,   /* U+325D */
  0x1C40,   /* U+325E */
  0x1C40,   /* U+325F */
  0x3BF5,   /* U+3260 */
  0x3BF7,   /* U+3261 */
  0x3BF8,   /* U+3262 */
  0x3BFA,   /* U+3263 */
  0x3BFB,   /* U+3264 */
  0x3BFC,   /* U+3265 */
  0x3BFE,   /* U+3266 */
  0x3C00,   /* U+3267 */
  0x3C01,   /* U+3268 */
  0x3C03,   /* U+3269 */
  0x3C04,   /* U+326A */
  0x3C05,   /* U+326B */
  0x3C06,   /* U+326C */
  0x3C07,   /* U+326D */
  0x3BF5,   /* U+326E */
  0x3BF7,   /* U+326F */
  0x3BF8,   /* U+3270 */
  0x3BFA,   /* U+3271 */
  0x3BFB,   /* U+3272 */
  0x3BFC,   /* U+3273 */
  0x3BFE,   /* U+3274 */
  0x3C00,   /* U+3275 */
  0x3C01,   /* U+3276 */
  0x3C03,   /* U+3277 */
  0x3C04,   /* U+3278 */
  0x3C05,   /* U+3279 */
  0x3C06,   /* U+327A */
  0x3C07,   /* U+327B */
  0x3C03,   /* U+327C */
  0x3C01,   /* U+327D */
  0x3C00,   /* U+327E */
  0x1A94,   /* U+327F */
  0xFB40,   /* U+3280 */
  0xFB40,   /* U+3281 */
  0xFB40,   /* U+3282 */
  0xFB40,   /* U+3283 */
  0xFB40,   /* U+3284 */
  0xFB40,   /* U+3285 */
  0xFB40,   /* U+3286 */
  0xFB40,   /* U+3287 */
  0xFB40,   /* U+3288 */
  0xFB40,   /* U+3289 */
  0xFB40,   /* U+328A */
  0xFB40,   /* U+328B */
  0xFB40,   /* U+328C */
  0xFB40,   /* U+328D */
  0xFB41,   /* U+328E */
  0xFB40,   /* U+328F */
  0xFB40,   /* U+3290 */
  0xFB40,   /* U+3291 */
  0xFB40,   /* U+3292 */
  0xFB40,   /* U+3293 */
  0xFB40,   /* U+3294 */
  0xFB40,   /* U+3295 */
  0xFB41,   /* U+3296 */
  0xFB40,   /* U+3297 */
  0xFB40,   /* U+3298 */
  0xFB40,   /* U+3299 */
  0xFB40,   /* U+329A */
  0xFB40,   /* U+329B */
  0xFB41,   /* U+329C */
  0xFB40,   /* U+329D */
  0xFB40,   /* U+329E */
  0xFB40,   /* U+329F */
  0xFB41,   /* U+32A0 */
  0xFB40,   /* U+32A1 */
  0xFB40,   /* U+32A2 */
  0xFB40,   /* U+32A3 */
  0xFB40,   /* U+32A4 */
  0xFB40,   /* U+32A5 */
  0xFB40,   /* U+32A6 */
  0xFB40,   /* U+32A7 */
  0xFB40,   /* U+32A8 */
  0xFB40,   /* U+32A9 */
  0xFB40,   /* U+32AA */
  0xFB40,   /* U+32AB */
  0xFB40,   /* U+32AC */
  0xFB40,   /* U+32AD */
  0xFB41,   /* U+32AE */
  0xFB40,   /* U+32AF */
  0xFB40,   /* U+32B0 */
  0x1C40,   /* U+32B1 */
  0x1C40,   /* U+32B2 */
  0x1C40,   /* U+32B3 */
  0x1C40,   /* U+32B4 */
  0x1C41,   /* U+32B5 */
  0x1C41,   /* U+32B6 */
  0x1C41,   /* U+32B7 */
  0x1C41,   /* U+32B8 */
  0x1C41,   /* U+32B9 */
  0x1C41,   /* U+32BA */
  0x1C41,   /* U+32BB */
  0x1C41,   /* U+32BC */
  0x1C41,   /* U+32BD */
  0x1C41,   /* U+32BE */
  0x1C42,   /* U+32BF */
  0x1C3E,   /* U+32C0 */
  0x1C3F,   /* U+32C1 */
  0x1C40,   /* U+32C2 */
  0x1C41,   /* U+32C3 */
  0x1C42,   /* U+32C4 */
  0x1C43,   /* U+32C5 */
  0x1C44,   /* U+32C6 */
  0x1C45,   /* U+32C7 */
  0x1C46,   /* U+32C8 */
  0x1C3E,   /* U+32C9 */
  0x1C3E,   /* U+32CA */
  0x1C3E,   /* U+32CB */
  0x1D18,   /* U+32CC */
  0x1CAA,   /* U+32CD */
  0x1CAA,   /* U+32CE */
  0x1D77,   /* U+32CF */
  0x3D5A,   /* U+32D0 */
  0x3D5B,   /* U+32D1 */
  0x3D5C,   /* U+32D2 */
  0x3D5E,   /* U+32D3 */
  0x3D5F,   /* U+32D4 */
  0x3D60,   /* U+32D5 */
  0x3D61,   /* U+32D6 */
  0x3D62,   /* U+32D7 */
  0x3D63,   /* U+32D8 */
  0x3D64,   /* U+32D9 */
  0x3D65,   /* U+32DA */
  0x3D66,   /* U+32DB */
  0x3D67,   /* U+32DC */
  0x3D68,   /* U+32DD */
  0x3D69,   /* U+32DE */
  0x3D6A,   /* U+32DF */
  0x3D6B,   /* U+32E0 */
  0x3D6C,   /* U+32E1 */
  0x3D6D,   /* U+32E2 */
  0x3D6E,   /* U+32E3 */
  0x3D6F,   /* U+32E4 */
  0x3D70,   /* U+32E5 */
  0x3D71,   /* U+32E6 */
  0x3D72,   /* U+32E7 */
  0x3D73,   /* U+32E8 */
  0x3D74,   /* U+32E9 */
  0x3D75,   /* U+32EA */
  0x3D76,   /* U+32EB */
  0x3D77,   /* U+32EC */
  0x3D78,   /* U+32ED */
  0x3D79,   /* U+32EE */
  0x3D7A,   /* U+32EF */
  0x3D7B,   /* U+32F0 */
  0x3D7C,   /* U+32F1 */
  0x3D7D,   /* U+32F2 */
  0x3D7E,   /* U+32F3 */
  0x3D7F,   /* U+32F4 */
  0x3D81,   /* U+32F5 */
  0x3D82,   /* U+32F6 */
  0x3D83,   /* U+32F7 */
  0x3D84,   /* U+32F8 */
  0x3D85,   /* U+32F9 */
  0x3D86,   /* U+32FA */
  0x3D87,   /* U+32FB */
  0x3D88,   /* U+32FC */
  0x3D89,   /* U+32FD */
  0x3D8A,   /* U+32FE */
  0xFBC0,   /* U+32FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+3200 */
  0x0020,   /* U+3201 */
  0x0020,   /* U+3202 */
  0x0020,   /* U+3203 */
  0x0020,   /* U+3204 */
  0x0020,   /* U+3205 */
  0x0020,   /* U+3206 */
  0x0020,   /* U+3207 */
  0x0020,   /* U+3208 */
  0x0020,   /* U+3209 */
  0x0020,   /* U+320A */
  0x0020,   /* U+320B */
  0x0020,   /* U+320C */
  0x0020,   /* U+320D */
  0x0020,   /* U+320E */
  0x0020,   /* U+320F */
  0x0020,   /* U+3210 */
  0x0020,   /* U+3211 */
  0x0020,   /* U+3212 */
  0x0020,   /* U+3213 */
  0x0020,   /* U+3214 */
  0x0020,   /* U+3215 */
  0x0020,   /* U+3216 */
  0x0020,   /* U+3217 */
  0x0020,   /* U+3218 */
  0x0020,   /* U+3219 */
  0x0020,   /* U+321A */
  0x0020,   /* U+321B */
  0x0020,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0020,   /* U+321F */
  0x0020,   /* U+3220 */
  0x0020,   /* U+3221 */
  0x0020,   /* U+3222 */
  0x0020,   /* U+3223 */
  0x0020,   /* U+3224 */
  0x0020,   /* U+3225 */
  0x0020,   /* U+3226 */
  0x0020,   /* U+3227 */
  0x0020,   /* U+3228 */
  0x0020,   /* U+3229 */
  0x0020,   /* U+322A */
  0x0020,   /* U+322B */
  0x0020,   /* U+322C */
  0x0020,   /* U+322D */
  0x0020,   /* U+322E */
  0x0020,   /* U+322F */
  0x0020,   /* U+3230 */
  0x0020,   /* U+3231 */
  0x0020,   /* U+3232 */
  0x0020,   /* U+3233 */
  0x0020,   /* U+3234 */
  0x0020,   /* U+3235 */
  0x0020,   /* U+3236 */
  0x0020,   /* U+3237 */
  0x0020,   /* U+3238 */
  0x0020,   /* U+3239 */
  0x0020,   /* U+323A */
  0x0020,   /* U+323B */
  0x0020,   /* U+323C */
  0x0020,   /* U+323D */
  0x0020,   /* U+323E */
  0x0020,   /* U+323F */
  0x0020,   /* U+3240 */
  0x0020,   /* U+3241 */
  0x0020,   /* U+3242 */
  0x0020,   /* U+3243 */
  0x0020,   /* U+3244 */
  0x0020,   /* U+3245 */
  0x0020,   /* U+3246 */
  0x0020,   /* U+3247 */
  0x0020,   /* U+3248 */
  0x0020,   /* U+3249 */
  0x0020,   /* U+324A */
  0x0020,   /* U+324B */
  0x0020,   /* U+324C */
  0x0020,   /* U+324D */
  0x0020,   /* U+324E */
  0x0020,   /* U+324F */
  0x0020,   /* U+3250 */
  0x0020,   /* U+3251 */
  0x0020,   /* U+3252 */
  0x0020,   /* U+3253 */
  0x0020,   /* U+3254 */
  0x0020,   /* U+3255 */
  0x0020,   /* U+3256 */
  0x0020,   /* U+3257 */
  0x0020,   /* U+3258 */
  0x0020,   /* U+3259 */
  0x0020,   /* U+325A */
  0x0020,   /* U+325B */
  0x0020,   /* U+325C */
  0x0020,   /* U+325D */
  0x0020,   /* U+325E */
  0x0020,   /* U+325F */
  0x0020,   /* U+3260 */
  0x0020,   /* U+3261 */
  0x0020,   /* U+3262 */
  0x0020,   /* U+3263 */
  0x0020,   /* U+3264 */
  0x0020,   /* U+3265 */
  0x0020,   /* U+3266 */
  0x0020,   /* U+3267 */
  0x0020,   /* U+3268 */
  0x0020,   /* U+3269 */
  0x0020,   /* U+326A */
  0x0020,   /* U+326B */
  0x0020,   /* U+326C */
  0x0020,   /* U+326D */
  0x0020,   /* U+326E */
  0x0020,   /* U+326F */
  0x0020,   /* U+3270 */
  0x0020,   /* U+3271 */
  0x0020,   /* U+3272 */
  0x0020,   /* U+3273 */
  0x0020,   /* U+3274 */
  0x0020,   /* U+3275 */
  0x0020,   /* U+3276 */
  0x0020,   /* U+3277 */
  0x0020,   /* U+3278 */
  0x0020,   /* U+3279 */
  0x0020,   /* U+327A */
  0x0020,   /* U+327B */
  0x0020,   /* U+327C */
  0x0020,   /* U+327D */
  0x0020,   /* U+327E */
  0x0020,   /* U+327F */
  0x0020,   /* U+3280 */
  0x0020,   /* U+3281 */
  0x0020,   /* U+3282 */
  0x0020,   /* U+3283 */
  0x0020,   /* U+3284 */
  0x0020,   /* U+3285 */
  0x0020,   /* U+3286 */
  0x0020,   /* U+3287 */
  0x0020,   /* U+3288 */
  0x0020,   /* U+3289 */
  0x0020,   /* U+328A */
  0x0020,   /* U+328B */
  0x0020,   /* U+328C */
  0x0020,   /* U+328D */
  0x0020,   /* U+328E */
  0x0020,   /* U+328F */
  0x0020,   /* U+3290 */
  0x0020,   /* U+3291 */
  0x0020,   /* U+3292 */
  0x0020,   /* U+3293 */
  0x0020,   /* U+3294 */
  0x0020,   /* U+3295 */
  0x0020,   /* U+3296 */
  0x0020,   /* U+3297 */
  0x0020,   /* U+3298 */
  0x0020,   /* U+3299 */
  0x0020,   /* U+329A */
  0x0020,   /* U+329B */
  0x0020,   /* U+329C */
  0x0020,   /* U+329D */
  0x0020,   /* U+329E */
  0x0020,   /* U+329F */
  0x0020,   /* U+32A0 */
  0x0020,   /* U+32A1 */
  0x0020,   /* U+32A2 */
  0x0020,   /* U+32A3 */
  0x0020,   /* U+32A4 */
  0x0020,   /* U+32A5 */
  0x0020,   /* U+32A6 */
  0x0020,   /* U+32A7 */
  0x0020,   /* U+32A8 */
  0x0020,   /* U+32A9 */
  0x0020,   /* U+32AA */
  0x0020,   /* U+32AB */
  0x0020,   /* U+32AC */
  0x0020,   /* U+32AD */
  0x0020,   /* U+32AE */
  0x0020,   /* U+32AF */
  0x0020,   /* U+32B0 */
  0x0020,   /* U+32B1 */
  0x0020,   /* U+32B2 */
  0x0020,   /* U+32B3 */
  0x0020,   /* U+32B4 */
  0x0020,   /* U+32B5 */
  0x0020,   /* U+32B6 */
  0x0020,   /* U+32B7 */
  0x0020,   /* U+32B8 */
  0x0020,   /* U+32B9 */
  0x0020,   /* U+32BA */
  0x0020,   /* U+32BB */
  0x0020,   /* U+32BC */
  0x0020,   /* U+32BD */
  0x0020,   /* U+32BE */
  0x0020,   /* U+32BF */
  0x0020,   /* U+32C0 */
  0x0020,   /* U+32C1 */
  0x0020,   /* U+32C2 */
  0x0020,   /* U+32C3 */
  0x0020,   /* U+32C4 */
  0x0020,   /* U+32C5 */
  0x0020,   /* U+32C6 */
  0x0020,   /* U+32C7 */
  0x0020,   /* U+32C8 */
  0x0020,   /* U+32C9 */
  0x0020,   /* U+32CA */
  0x0020,   /* U+32CB */
  0x0020,   /* U+32CC */
  0x0020,   /* U+32CD */
  0x0020,   /* U+32CE */
  0x0020,   /* U+32CF */
  0x0020,   /* U+32D0 */
  0x0020,   /* U+32D1 */
  0x0020,   /* U+32D2 */
  0x0020,   /* U+32D3 */
  0x0020,   /* U+32D4 */
  0x0020,   /* U+32D5 */
  0x0020,   /* U+32D6 */
  0x0020,   /* U+32D7 */
  0x0020,   /* U+32D8 */
  0x0020,   /* U+32D9 */
  0x0020,   /* U+32DA */
  0x0020,   /* U+32DB */
  0x0020,   /* U+32DC */
  0x0020,   /* U+32DD */
  0x0020,   /* U+32DE */
  0x0020,   /* U+32DF */
  0x0020,   /* U+32E0 */
  0x0020,   /* U+32E1 */
  0x0020,   /* U+32E2 */
  0x0020,   /* U+32E3 */
  0x0020,   /* U+32E4 */
  0x0020,   /* U+32E5 */
  0x0020,   /* U+32E6 */
  0x0020,   /* U+32E7 */
  0x0020,   /* U+32E8 */
  0x0020,   /* U+32E9 */
  0x0020,   /* U+32EA */
  0x0020,   /* U+32EB */
  0x0020,   /* U+32EC */
  0x0020,   /* U+32ED */
  0x0020,   /* U+32EE */
  0x0020,   /* U+32EF */
  0x0020,   /* U+32F0 */
  0x0020,   /* U+32F1 */
  0x0020,   /* U+32F2 */
  0x0020,   /* U+32F3 */
  0x0020,   /* U+32F4 */
  0x0020,   /* U+32F5 */
  0x0020,   /* U+32F6 */
  0x0020,   /* U+32F7 */
  0x0020,   /* U+32F8 */
  0x0020,   /* U+32F9 */
  0x0020,   /* U+32FA */
  0x0020,   /* U+32FB */
  0x0020,   /* U+32FC */
  0x0020,   /* U+32FD */
  0x0020,   /* U+32FE */
  0x0020,   /* U+32FF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+3200 */
  0x0004,   /* U+3201 */
  0x0004,   /* U+3202 */
  0x0004,   /* U+3203 */
  0x0004,   /* U+3204 */
  0x0004,   /* U+3205 */
  0x0004,   /* U+3206 */
  0x0004,   /* U+3207 */
  0x0004,   /* U+3208 */
  0x0004,   /* U+3209 */
  0x0004,   /* U+320A */
  0x0004,   /* U+320B */
  0x0004,   /* U+320C */
  0x0004,   /* U+320D */
  0x0004,   /* U+320E */
  0x0004,   /* U+320F */
  0x0004,   /* U+3210 */
  0x0004,   /* U+3211 */
  0x0004,   /* U+3212 */
  0x0004,   /* U+3213 */
  0x0004,   /* U+3214 */
  0x0004,   /* U+3215 */
  0x0004,   /* U+3216 */
  0x0004,   /* U+3217 */
  0x0004,   /* U+3218 */
  0x0004,   /* U+3219 */
  0x0004,   /* U+321A */
  0x0004,   /* U+321B */
  0x0004,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0002,   /* U+321F */
  0x0004,   /* U+3220 */
  0x0004,   /* U+3221 */
  0x0004,   /* U+3222 */
  0x0004,   /* U+3223 */
  0x0004,   /* U+3224 */
  0x0004,   /* U+3225 */
  0x0004,   /* U+3226 */
  0x0004,   /* U+3227 */
  0x0004,   /* U+3228 */
  0x0004,   /* U+3229 */
  0x0004,   /* U+322A */
  0x0004,   /* U+322B */
  0x0004,   /* U+322C */
  0x0004,   /* U+322D */
  0x0004,   /* U+322E */
  0x0004,   /* U+322F */
  0x0004,   /* U+3230 */
  0x0004,   /* U+3231 */
  0x0004,   /* U+3232 */
  0x0004,   /* U+3233 */
  0x0004,   /* U+3234 */
  0x0004,   /* U+3235 */
  0x0004,   /* U+3236 */
  0x0004,   /* U+3237 */
  0x0004,   /* U+3238 */
  0x0004,   /* U+3239 */
  0x0004,   /* U+323A */
  0x0004,   /* U+323B */
  0x0004,   /* U+323C */
  0x0004,   /* U+323D */
  0x0004,   /* U+323E */
  0x0004,   /* U+323F */
  0x0004,   /* U+3240 */
  0x0004,   /* U+3241 */
  0x0004,   /* U+3242 */
  0x0004,   /* U+3243 */
  0x0006,   /* U+3244 */
  0x0006,   /* U+3245 */
  0x0006,   /* U+3246 */
  0x0006,   /* U+3247 */
  0x0006,   /* U+3248 */
  0x0006,   /* U+3249 */
  0x0006,   /* U+324A */
  0x0006,   /* U+324B */
  0x0006,   /* U+324C */
  0x0006,   /* U+324D */
  0x0006,   /* U+324E */
  0x0006,   /* U+324F */
  0x001D,   /* U+3250 */
  0x0006,   /* U+3251 */
  0x0006,   /* U+3252 */
  0x0006,   /* U+3253 */
  0x0006,   /* U+3254 */
  0x0006,   /* U+3255 */
  0x0006,   /* U+3256 */
  0x0006,   /* U+3257 */
  0x0006,   /* U+3258 */
  0x0006,   /* U+3259 */
  0x0006,   /* U+325A */
  0x0006,   /* U+325B */
  0x0006,   /* U+325C */
  0x0006,   /* U+325D */
  0x0006,   /* U+325E */
  0x0006,   /* U+325F */
  0x0006,   /* U+3260 */
  0x0006,   /* U+3261 */
  0x0006,   /* U+3262 */
  0x0006,   /* U+3263 */
  0x0006,   /* U+3264 */
  0x0006,   /* U+3265 */
  0x0006,   /* U+3266 */
  0x0006,   /* U+3267 */
  0x0006,   /* U+3268 */
  0x0006,   /* U+3269 */
  0x0006,   /* U+326A */
  0x0006,   /* U+326B */
  0x0006,   /* U+326C */
  0x0006,   /* U+326D */
  0x0006,   /* U+326E */
  0x0006,   /* U+326F */
  0x0006,   /* U+3270 */
  0x0006,   /* U+3271 */
  0x0006,   /* U+3272 */
  0x0006,   /* U+3273 */
  0x0006,   /* U+3274 */
  0x0006,   /* U+3275 */
  0x0006,   /* U+3276 */
  0x0006,   /* U+3277 */
  0x0006,   /* U+3278 */
  0x0006,   /* U+3279 */
  0x0006,   /* U+327A */
  0x0006,   /* U+327B */
  0x0006,   /* U+327C */
  0x0006,   /* U+327D */
  0x0006,   /* U+327E */
  0x0002,   /* U+327F */
  0x0006,   /* U+3280 */
  0x0006,   /* U+3281 */
  0x0006,   /* U+3282 */
  0x0006,   /* U+3283 */
  0x0006,   /* U+3284 */
  0x0006,   /* U+3285 */
  0x0006,   /* U+3286 */
  0x0006,   /* U+3287 */
  0x0006,   /* U+3288 */
  0x0006,   /* U+3289 */
  0x0006,   /* U+328A */
  0x0006,   /* U+328B */
  0x0006,   /* U+328C */
  0x0006,   /* U+328D */
  0x0006,   /* U+328E */
  0x0006,   /* U+328F */
  0x0006,   /* U+3290 */
  0x0006,   /* U+3291 */
  0x0006,   /* U+3292 */
  0x0006,   /* U+3293 */
  0x0006,   /* U+3294 */
  0x0006,   /* U+3295 */
  0x0006,   /* U+3296 */
  0x0006,   /* U+3297 */
  0x0006,   /* U+3298 */
  0x0006,   /* U+3299 */
  0x0006,   /* U+329A */
  0x0006,   /* U+329B */
  0x0006,   /* U+329C */
  0x0006,   /* U+329D */
  0x0006,   /* U+329E */
  0x0006,   /* U+329F */
  0x0006,   /* U+32A0 */
  0x0006,   /* U+32A1 */
  0x0006,   /* U+32A2 */
  0x0006,   /* U+32A3 */
  0x0006,   /* U+32A4 */
  0x0006,   /* U+32A5 */
  0x0006,   /* U+32A6 */
  0x0006,   /* U+32A7 */
  0x0006,   /* U+32A8 */
  0x0006,   /* U+32A9 */
  0x0006,   /* U+32AA */
  0x0006,   /* U+32AB */
  0x0006,   /* U+32AC */
  0x0006,   /* U+32AD */
  0x0006,   /* U+32AE */
  0x0006,   /* U+32AF */
  0x0006,   /* U+32B0 */
  0x0006,   /* U+32B1 */
  0x0006,   /* U+32B2 */
  0x0006,   /* U+32B3 */
  0x0006,   /* U+32B4 */
  0x0006,   /* U+32B5 */
  0x0006,   /* U+32B6 */
  0x0006,   /* U+32B7 */
  0x0006,   /* U+32B8 */
  0x0006,   /* U+32B9 */
  0x0006,   /* U+32BA */
  0x0006,   /* U+32BB */
  0x0006,   /* U+32BC */
  0x0006,   /* U+32BD */
  0x0006,   /* U+32BE */
  0x0006,   /* U+32BF */
  0x0004,   /* U+32C0 */
  0x0004,   /* U+32C1 */
  0x0004,   /* U+32C2 */
  0x0004,   /* U+32C3 */
  0x0004,   /* U+32C4 */
  0x0004,   /* U+32C5 */
  0x0004,   /* U+32C6 */
  0x0004,   /* U+32C7 */
  0x0004,   /* U+32C8 */
  0x0004,   /* U+32C9 */
  0x0004,   /* U+32CA */
  0x0004,   /* U+32CB */
  0x001D,   /* U+32CC */
  0x001C,   /* U+32CD */
  0x001C,   /* U+32CE */
  0x001D,   /* U+32CF */
  0x0013,   /* U+32D0 */
  0x0013,   /* U+32D1 */
  0x0013,   /* U+32D2 */
  0x0013,   /* U+32D3 */
  0x0013,   /* U+32D4 */
  0x0013,   /* U+32D5 */
  0x0013,   /* U+32D6 */
  0x0013,   /* U+32D7 */
  0x0013,   /* U+32D8 */
  0x0013,   /* U+32D9 */
  0x0013,   /* U+32DA */
  0x0013,   /* U+32DB */
  0x0013,   /* U+32DC */
  0x0013,   /* U+32DD */
  0x0013,   /* U+32DE */
  0x0013,   /* U+32DF */
  0x0013,   /* U+32E0 */
  0x0013,   /* U+32E1 */
  0x0013,   /* U+32E2 */
  0x0013,   /* U+32E3 */
  0x0013,   /* U+32E4 */
  0x0013,   /* U+32E5 */
  0x0013,   /* U+32E6 */
  0x0013,   /* U+32E7 */
  0x0013,   /* U+32E8 */
  0x0013,   /* U+32E9 */
  0x0013,   /* U+32EA */
  0x0013,   /* U+32EB */
  0x0013,   /* U+32EC */
  0x0013,   /* U+32ED */
  0x0013,   /* U+32EE */
  0x0013,   /* U+32EF */
  0x0013,   /* U+32F0 */
  0x0013,   /* U+32F1 */
  0x0013,   /* U+32F2 */
  0x0013,   /* U+32F3 */
  0x0013,   /* U+32F4 */
  0x0013,   /* U+32F5 */
  0x0013,   /* U+32F6 */
  0x0013,   /* U+32F7 */
  0x0013,   /* U+32F8 */
  0x0013,   /* U+32F9 */
  0x0013,   /* U+32FA */
  0x0013,   /* U+32FB */
  0x0013,   /* U+32FC */
  0x0013,   /* U+32FD */
  0x0013,   /* U+32FE */
  0x0002,   /* U+32FF */

  /* Primary weight 2 for each character. */
  0x3BF5,   /* U+3200 */
  0x3BF7,   /* U+3201 */
  0x3BF8,   /* U+3202 */
  0x3BFA,   /* U+3203 */
  0x3BFB,   /* U+3204 */
  0x3BFC,   /* U+3205 */
  0x3BFE,   /* U+3206 */
  0x3C00,   /* U+3207 */
  0x3C01,   /* U+3208 */
  0x3C03,   /* U+3209 */
  0x3C04,   /* U+320A */
  0x3C05,   /* U+320B */
  0x3C06,   /* U+320C */
  0x3C07,   /* U+320D */
  0x3BF5,   /* U+320E */
  0x3BF7,   /* U+320F */
  0x3BF8,   /* U+3210 */
  0x3BFA,   /* U+3211 */
  0x3BFB,   /* U+3212 */
  0x3BFC,   /* U+3213 */
  0x3BFE,   /* U+3214 */
  0x3C00,   /* U+3215 */
  0x3C01,   /* U+3216 */
  0x3C03,   /* U+3217 */
  0x3C04,   /* U+3218 */
  0x3C05,   /* U+3219 */
  0x3C06,   /* U+321A */
  0x3C07,   /* U+321B */
  0x3C01,   /* U+321C */
  0x3C00,   /* U+321D */
  0x3C00,   /* U+321E */
  0xB21F,   /* U+321F */
  0xFB40,   /* U+3220 */
  0xFB40,   /* U+3221 */
  0xFB40,   /* U+3222 */
  0xFB40,   /* U+3223 */
  0xFB40,   /* U+3224 */
  0xFB40,   /* U+3225 */
  0xFB40,   /* U+3226 */
  0xFB40,   /* U+3227 */
  0xFB40,   /* U+3228 */
  0xFB40,   /* U+3229 */
  0xFB40,   /* U+322A */
  0xFB40,   /* U+322B */
  0xFB40,   /* U+322C */
  0xFB40,   /* U+322D */
  0xFB41,   /* U+322E */
  0xFB40,   /* U+322F */
  0xFB40,   /* U+3230 */
  0xFB40,   /* U+3231 */
  0xFB40,   /* U+3232 */
  0xFB40,   /* U+3233 */
  0xFB40,   /* U+3234 */
  0xFB40,   /* U+3235 */
  0xFB41,   /* U+3236 */
  0xFB40,   /* U+3237 */
  0xFB40,   /* U+3238 */
  0xFB40,   /* U+3239 */
  0xFB40,   /* U+323A */
  0xFB40,   /* U+323B */
  0xFB40,   /* U+323C */
  0xFB40,   /* U+323D */
  0xFB41,   /* U+323E */
  0xFB40,   /* U+323F */
  0xFB40,   /* U+3240 */
  0xFB40,   /* U+3241 */
  0xFB41,   /* U+3242 */
  0xFB41,   /* U+3243 */
  0xD54F,   /* U+3244 */
  0xDE7C,   /* U+3245 */
  0xE587,   /* U+3246 */
  0xFB8F,   /* U+3247 */
  0x1C3D,   /* U+3248 */
  0x1C3D,   /* U+3249 */
  0x1C3D,   /* U+324A */
  0x1C3D,   /* U+324B */
  0x1C3D,   /* U+324C */
  0x1C3D,   /* U+324D */
  0x1C3D,   /* U+324E */
  0x1C3D,   /* U+324F */
  0x1E95,   /* U+3250 */
  0x1C3E,   /* U+3251 */
  0x1C3F,   /* U+3252 */
  0x1C40,   /* U+3253 */
  0x1C41,   /* U+3254 */
  0x1C42,   /* U+3255 */
  0x1C43,   /* U+3256 */
  0x1C44,   /* U+3257 */
  0x1C45,   /* U+3258 */
  0x1C46,   /* U+3259 */
  0x1C3D,   /* U+325A */
  0x1C3E,   /* U+325B */
  0x1C3F,   /* U+325C */
  0x1C40,   /* U+325D */
  0x1C41,   /* U+325E */
  0x1C42,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x3C73,   /* U+326E */
  0x3C73,   /* U+326F */
  0x3C73,   /* U+3270 */
  0x3C73,   /* U+3271 */
  0x3C73,   /* U+3272 */
  0x3C73,   /* U+3273 */
  0x3C73,   /* U+3274 */
  0x3C73,   /* U+3275 */
  0x3C73,   /* U+3276 */
  0x3C73,   /* U+3277 */
  0x3C73,   /* U+3278 */
  0x3C73,   /* U+3279 */
  0x3C73,   /* U+327A */
  0x3C73,   /* U+327B */
  0x3C73,   /* U+327C */
  0x3C80,   /* U+327D */
  0x3C80,   /* U+327E */
  0x0000,   /* U+327F */
  0xCE00,   /* U+3280 */
  0xCE8C,   /* U+3281 */
  0xCE09,   /* U+3282 */
  0xD6DB,   /* U+3283 */
  0xCE94,   /* U+3284 */
  0xD16D,   /* U+3285 */
  0xCE03,   /* U+3286 */
  0xD16B,   /* U+3287 */
  0xCE5D,   /* U+3288 */
  0xD341,   /* U+3289 */
  0xE708,   /* U+328A */
  0xF06B,   /* U+328B */
  0xEC34,   /* U+328C */
  0xE728,   /* U+328D */
  0x91D1,   /* U+328E */
  0xD71F,   /* U+328F */
  0xE5E5,   /* U+3290 */
  0xE82A,   /* U+3291 */
  0xE709,   /* U+3292 */
  0xF93E,   /* U+3293 */
  0xD40D,   /* U+3294 */
  0xF279,   /* U+3295 */
  0x8CA1,   /* U+3296 */
  0xF95D,   /* U+3297 */
  0xD2B4,   /* U+3298 */
  0xF9D8,   /* U+3299 */
  0xF537,   /* U+329A */
  0xD973,   /* U+329B */
  0x9069,   /* U+329C */
  0xD12A,   /* U+329D */
  0xD370,   /* U+329E */
  0xECE8,   /* U+329F */
  0x9805,   /* U+32A0 */
  0xCF11,   /* U+32A1 */
  0xD199,   /* U+32A2 */
  0xEB63,   /* U+32A3 */
  0xCE0A,   /* U+32A4 */
  0xCE2D,   /* U+32A5 */
  0xCE0B,   /* U+32A6 */
  0xDDE6,   /* U+32A7 */
  0xD3F3,   /* U+32A8 */
  0xD33B,   /* U+32A9 */
  0xDB97,   /* U+32AA */
  0xDB66,   /* U+32AB */
  0xF6E3,   /* U+32AC */
  0xCF01,   /* U+32AD */
  0x8CC7,   /* U+32AE */
  0xD354,   /* U+32AF */
  0xD91C,   /* U+32B0 */
  0x1C43,   /* U+32B1 */
  0x1C44,   /* U+32B2 */
  0x1C45,   /* U+32B3 */
  0x1C46,   /* U+32B4 */
  0x1C3D,   /* U+32B5 */
  0x1C3E,   /* U+32B6 */
  0x1C3F,   /* U+32B7 */
  0x1C40,   /* U+32B8 */
  0x1C41,   /* U+32B9 */
  0x1C42,   /* U+32BA */
  0x1C43,   /* U+32BB */
  0x1C44,   /* U+32BC */
  0x1C45,   /* U+32BD */
  0x1C46,   /* U+32BE */
  0x1C3D,   /* U+32BF */
  0xFB40,   /* U+32C0 */
  0xFB40,   /* U+32C1 */
  0xFB40,   /* U+32C2 */
  0xFB40,   /* U+32C3 */
  0xFB40,   /* U+32C4 */
  0xFB40,   /* U+32C5 */
  0xFB40,   /* U+32C6 */
  0xFB40,   /* U+32C7 */
  0xFB40,   /* U+32C8 */
  0x1C3D,   /* U+32C9 */
  0x1C3E,   /* U+32CA */
  0x1C3F,   /* U+32CB */
  0x1CF4,   /* U+32CC */
  0x1E33,   /* U+32CD */
  0x1EE3,   /* U+32CE */
  0x1E95,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0xB2FF,   /* U+32FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+3200 */
  0x0020,   /* U+3201 */
  0x0020,   /* U+3202 */
  0x0020,   /* U+3203 */
  0x0020,   /* U+3204 */
  0x0020,   /* U+3205 */
  0x0020,   /* U+3206 */
  0x0020,   /* U+3207 */
  0x0020,   /* U+3208 */
  0x0020,   /* U+3209 */
  0x0020,   /* U+320A */
  0x0020,   /* U+320B */
  0x0020,   /* U+320C */
  0x0020,   /* U+320D */
  0x0020,   /* U+320E */
  0x0020,   /* U+320F */
  0x0020,   /* U+3210 */
  0x0020,   /* U+3211 */
  0x0020,   /* U+3212 */
  0x0020,   /* U+3213 */
  0x0020,   /* U+3214 */
  0x0020,   /* U+3215 */
  0x0020,   /* U+3216 */
  0x0020,   /* U+3217 */
  0x0020,   /* U+3218 */
  0x0020,   /* U+3219 */
  0x0020,   /* U+321A */
  0x0020,   /* U+321B */
  0x0020,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0000,   /* U+321F */
  0x0020,   /* U+3220 */
  0x0020,   /* U+3221 */
  0x0020,   /* U+3222 */
  0x0020,   /* U+3223 */
  0x0020,   /* U+3224 */
  0x0020,   /* U+3225 */
  0x0020,   /* U+3226 */
  0x0020,   /* U+3227 */
  0x0020,   /* U+3228 */
  0x0020,   /* U+3229 */
  0x0020,   /* U+322A */
  0x0020,   /* U+322B */
  0x0020,   /* U+322C */
  0x0020,   /* U+322D */
  0x0020,   /* U+322E */
  0x0020,   /* U+322F */
  0x0020,   /* U+3230 */
  0x0020,   /* U+3231 */
  0x0020,   /* U+3232 */
  0x0020,   /* U+3233 */
  0x0020,   /* U+3234 */
  0x0020,   /* U+3235 */
  0x0020,   /* U+3236 */
  0x0020,   /* U+3237 */
  0x0020,   /* U+3238 */
  0x0020,   /* U+3239 */
  0x0020,   /* U+323A */
  0x0020,   /* U+323B */
  0x0020,   /* U+323C */
  0x0020,   /* U+323D */
  0x0020,   /* U+323E */
  0x0020,   /* U+323F */
  0x0020,   /* U+3240 */
  0x0020,   /* U+3241 */
  0x0020,   /* U+3242 */
  0x0020,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0020,   /* U+3248 */
  0x0020,   /* U+3249 */
  0x0020,   /* U+324A */
  0x0020,   /* U+324B */
  0x0020,   /* U+324C */
  0x0020,   /* U+324D */
  0x0020,   /* U+324E */
  0x0020,   /* U+324F */
  0x0020,   /* U+3250 */
  0x0020,   /* U+3251 */
  0x0020,   /* U+3252 */
  0x0020,   /* U+3253 */
  0x0020,   /* U+3254 */
  0x0020,   /* U+3255 */
  0x0020,   /* U+3256 */
  0x0020,   /* U+3257 */
  0x0020,   /* U+3258 */
  0x0020,   /* U+3259 */
  0x0020,   /* U+325A */
  0x0020,   /* U+325B */
  0x0020,   /* U+325C */
  0x0020,   /* U+325D */
  0x0020,   /* U+325E */
  0x0020,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0020,   /* U+326E */
  0x0020,   /* U+326F */
  0x0020,   /* U+3270 */
  0x0020,   /* U+3271 */
  0x0020,   /* U+3272 */
  0x0020,   /* U+3273 */
  0x0020,   /* U+3274 */
  0x0020,   /* U+3275 */
  0x0020,   /* U+3276 */
  0x0020,   /* U+3277 */
  0x0020,   /* U+3278 */
  0x0020,   /* U+3279 */
  0x0020,   /* U+327A */
  0x0020,   /* U+327B */
  0x0020,   /* U+327C */
  0x0020,   /* U+327D */
  0x0020,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0020,   /* U+32B1 */
  0x0020,   /* U+32B2 */
  0x0020,   /* U+32B3 */
  0x0020,   /* U+32B4 */
  0x0020,   /* U+32B5 */
  0x0020,   /* U+32B6 */
  0x0020,   /* U+32B7 */
  0x0020,   /* U+32B8 */
  0x0020,   /* U+32B9 */
  0x0020,   /* U+32BA */
  0x0020,   /* U+32BB */
  0x0020,   /* U+32BC */
  0x0020,   /* U+32BD */
  0x0020,   /* U+32BE */
  0x0020,   /* U+32BF */
  0x0020,   /* U+32C0 */
  0x0020,   /* U+32C1 */
  0x0020,   /* U+32C2 */
  0x0020,   /* U+32C3 */
  0x0020,   /* U+32C4 */
  0x0020,   /* U+32C5 */
  0x0020,   /* U+32C6 */
  0x0020,   /* U+32C7 */
  0x0020,   /* U+32C8 */
  0x0020,   /* U+32C9 */
  0x0020,   /* U+32CA */
  0x0020,   /* U+32CB */
  0x0020,   /* U+32CC */
  0x0020,   /* U+32CD */
  0x0020,   /* U+32CE */
  0x0020,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 2 for each character. */
  0x0004,   /* U+3200 */
  0x0004,   /* U+3201 */
  0x0004,   /* U+3202 */
  0x0004,   /* U+3203 */
  0x0004,   /* U+3204 */
  0x0004,   /* U+3205 */
  0x0004,   /* U+3206 */
  0x0004,   /* U+3207 */
  0x0004,   /* U+3208 */
  0x0004,   /* U+3209 */
  0x0004,   /* U+320A */
  0x0004,   /* U+320B */
  0x0004,   /* U+320C */
  0x0004,   /* U+320D */
  0x0004,   /* U+320E */
  0x0004,   /* U+320F */
  0x0004,   /* U+3210 */
  0x0004,   /* U+3211 */
  0x0004,   /* U+3212 */
  0x0004,   /* U+3213 */
  0x0004,   /* U+3214 */
  0x0004,   /* U+3215 */
  0x0004,   /* U+3216 */
  0x0004,   /* U+3217 */
  0x0004,   /* U+3218 */
  0x0004,   /* U+3219 */
  0x0004,   /* U+321A */
  0x0004,   /* U+321B */
  0x0004,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0000,   /* U+321F */
  0x0004,   /* U+3220 */
  0x0004,   /* U+3221 */
  0x0004,   /* U+3222 */
  0x0004,   /* U+3223 */
  0x0004,   /* U+3224 */
  0x0004,   /* U+3225 */
  0x0004,   /* U+3226 */
  0x0004,   /* U+3227 */
  0x0004,   /* U+3228 */
  0x0004,   /* U+3229 */
  0x0004,   /* U+322A */
  0x0004,   /* U+322B */
  0x0004,   /* U+322C */
  0x0004,   /* U+322D */
  0x0004,   /* U+322E */
  0x0004,   /* U+322F */
  0x0004,   /* U+3230 */
  0x0004,   /* U+3231 */
  0x0004,   /* U+3232 */
  0x0004,   /* U+3233 */
  0x0004,   /* U+3234 */
  0x0004,   /* U+3235 */
  0x0004,   /* U+3236 */
  0x0004,   /* U+3237 */
  0x0004,   /* U+3238 */
  0x0004,   /* U+3239 */
  0x0004,   /* U+323A */
  0x0004,   /* U+323B */
  0x0004,   /* U+323C */
  0x0004,   /* U+323D */
  0x0004,   /* U+323E */
  0x0004,   /* U+323F */
  0x0004,   /* U+3240 */
  0x0004,   /* U+3241 */
  0x0004,   /* U+3242 */
  0x0004,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0006,   /* U+3248 */
  0x0006,   /* U+3249 */
  0x0006,   /* U+324A */
  0x0006,   /* U+324B */
  0x0006,   /* U+324C */
  0x0006,   /* U+324D */
  0x0006,   /* U+324E */
  0x0006,   /* U+324F */
  0x001D,   /* U+3250 */
  0x0006,   /* U+3251 */
  0x0006,   /* U+3252 */
  0x0006,   /* U+3253 */
  0x0006,   /* U+3254 */
  0x0006,   /* U+3255 */
  0x0006,   /* U+3256 */
  0x0006,   /* U+3257 */
  0x0006,   /* U+3258 */
  0x0006,   /* U+3259 */
  0x0006,   /* U+325A */
  0x0006,   /* U+325B */
  0x0006,   /* U+325C */
  0x0006,   /* U+325D */
  0x0006,   /* U+325E */
  0x0006,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0006,   /* U+326E */
  0x0006,   /* U+326F */
  0x0006,   /* U+3270 */
  0x0006,   /* U+3271 */
  0x0006,   /* U+3272 */
  0x0006,   /* U+3273 */
  0x0006,   /* U+3274 */
  0x0006,   /* U+3275 */
  0x0006,   /* U+3276 */
  0x0006,   /* U+3277 */
  0x0006,   /* U+3278 */
  0x0006,   /* U+3279 */
  0x0006,   /* U+327A */
  0x0006,   /* U+327B */
  0x0006,   /* U+327C */
  0x0006,   /* U+327D */
  0x0006,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0006,   /* U+32B1 */
  0x0006,   /* U+32B2 */
  0x0006,   /* U+32B3 */
  0x0006,   /* U+32B4 */
  0x0006,   /* U+32B5 */
  0x0006,   /* U+32B6 */
  0x0006,   /* U+32B7 */
  0x0006,   /* U+32B8 */
  0x0006,   /* U+32B9 */
  0x0006,   /* U+32BA */
  0x0006,   /* U+32BB */
  0x0006,   /* U+32BC */
  0x0006,   /* U+32BD */
  0x0006,   /* U+32BE */
  0x0006,   /* U+32BF */
  0x0004,   /* U+32C0 */
  0x0004,   /* U+32C1 */
  0x0004,   /* U+32C2 */
  0x0004,   /* U+32C3 */
  0x0004,   /* U+32C4 */
  0x0004,   /* U+32C5 */
  0x0004,   /* U+32C6 */
  0x0004,   /* U+32C7 */
  0x0004,   /* U+32C8 */
  0x0004,   /* U+32C9 */
  0x0004,   /* U+32CA */
  0x0004,   /* U+32CB */
  0x001C,   /* U+32CC */
  0x001C,   /* U+32CD */
  0x001D,   /* U+32CE */
  0x001D,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Primary weight 3 for each character. */
  0x0318,   /* U+3200 */
  0x0318,   /* U+3201 */
  0x0318,   /* U+3202 */
  0x0318,   /* U+3203 */
  0x0318,   /* U+3204 */
  0x0318,   /* U+3205 */
  0x0318,   /* U+3206 */
  0x0318,   /* U+3207 */
  0x0318,   /* U+3208 */
  0x0318,   /* U+3209 */
  0x0318,   /* U+320A */
  0x0318,   /* U+320B */
  0x0318,   /* U+320C */
  0x0318,   /* U+320D */
  0x3C73,   /* U+320E */
  0x3C73,   /* U+320F */
  0x3C73,   /* U+3210 */
  0x3C73,   /* U+3211 */
  0x3C73,   /* U+3212 */
  0x3C73,   /* U+3213 */
  0x3C73,   /* U+3214 */
  0x3C73,   /* U+3215 */
  0x3C73,   /* U+3216 */
  0x3C73,   /* U+3217 */
  0x3C73,   /* U+3218 */
  0x3C73,   /* U+3219 */
  0x3C73,   /* U+321A */
  0x3C73,   /* U+321B */
  0x3C80,   /* U+321C */
  0x3C7B,   /* U+321D */
  0x3C7B,   /* U+321E */
  0x0000,   /* U+321F */
  0xCE00,   /* U+3220 */
  0xCE8C,   /* U+3221 */
  0xCE09,   /* U+3222 */
  0xD6DB,   /* U+3223 */
  0xCE94,   /* U+3224 */
  0xD16D,   /* U+3225 */
  0xCE03,   /* U+3226 */
  0xD16B,   /* U+3227 */
  0xCE5D,   /* U+3228 */
  0xD341,   /* U+3229 */
  0xE708,   /* U+322A */
  0xF06B,   /* U+322B */
  0xEC34,   /* U+322C */
  0xE728,   /* U+322D */
  0x91D1,   /* U+322E */
  0xD71F,   /* U+322F */
  0xE5E5,   /* U+3230 */
  0xE82A,   /* U+3231 */
  0xE709,   /* U+3232 */
  0xF93E,   /* U+3233 */
  0xD40D,   /* U+3234 */
  0xF279,   /* U+3235 */
  0x8CA1,   /* U+3236 */
  0xF95D,   /* U+3237 */
  0xD2B4,   /* U+3238 */
  0xCEE3,   /* U+3239 */
  0xD47C,   /* U+323A */
  0xDB66,   /* U+323B */
  0xF6E3,   /* U+323C */
  0xCF01,   /* U+323D */
  0x8CC7,   /* U+323E */
  0xD354,   /* U+323F */
  0xF96D,   /* U+3240 */
  0xCF11,   /* U+3241 */
  0x81EA,   /* U+3242 */
  0x81F3,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x1CAA,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x3CE0,   /* U+327C */
  0x3C00,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0xE708,   /* U+32C0 */
  0xE708,   /* U+32C1 */
  0xE708,   /* U+32C2 */
  0xE708,   /* U+32C3 */
  0xE708,   /* U+32C4 */
  0xE708,   /* U+32C5 */
  0xE708,   /* U+32C6 */
  0xE708,   /* U+32C7 */
  0xE708,   /* U+32C8 */
  0xFB40,   /* U+32C9 */
  0xFB40,   /* U+32CA */
  0xFB40,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x1CF4,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x1C8F,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Secondary weight 3 for each character. */
  0x0020,   /* U+3200 */
  0x0020,   /* U+3201 */
  0x0020,   /* U+3202 */
  0x0020,   /* U+3203 */
  0x0020,   /* U+3204 */
  0x0020,   /* U+3205 */
  0x0020,   /* U+3206 */
  0x0020,   /* U+3207 */
  0x0020,   /* U+3208 */
  0x0020,   /* U+3209 */
  0x0020,   /* U+320A */
  0x0020,   /* U+320B */
  0x0020,   /* U+320C */
  0x0020,   /* U+320D */
  0x0020,   /* U+320E */
  0x0020,   /* U+320F */
  0x0020,   /* U+3210 */
  0x0020,   /* U+3211 */
  0x0020,   /* U+3212 */
  0x0020,   /* U+3213 */
  0x0020,   /* U+3214 */
  0x0020,   /* U+3215 */
  0x0020,   /* U+3216 */
  0x0020,   /* U+3217 */
  0x0020,   /* U+3218 */
  0x0020,   /* U+3219 */
  0x0020,   /* U+321A */
  0x0020,   /* U+321B */
  0x0020,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0020,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0020,   /* U+327C */
  0x0020,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0020,   /* U+32C9 */
  0x0020,   /* U+32CA */
  0x0020,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0020,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0020,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 3 for each character. */
  0x0004,   /* U+3200 */
  0x0004,   /* U+3201 */
  0x0004,   /* U+3202 */
  0x0004,   /* U+3203 */
  0x0004,   /* U+3204 */
  0x0004,   /* U+3205 */
  0x0004,   /* U+3206 */
  0x0004,   /* U+3207 */
  0x0004,   /* U+3208 */
  0x0004,   /* U+3209 */
  0x0004,   /* U+320A */
  0x0004,   /* U+320B */
  0x0004,   /* U+320C */
  0x0004,   /* U+320D */
  0x0004,   /* U+320E */
  0x0004,   /* U+320F */
  0x0004,   /* U+3210 */
  0x0004,   /* U+3211 */
  0x0004,   /* U+3212 */
  0x0004,   /* U+3213 */
  0x0004,   /* U+3214 */
  0x0004,   /* U+3215 */
  0x0004,   /* U+3216 */
  0x0004,   /* U+3217 */
  0x0004,   /* U+3218 */
  0x0004,   /* U+3219 */
  0x0004,   /* U+321A */
  0x0004,   /* U+321B */
  0x0004,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x001D,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0006,   /* U+327C */
  0x0006,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0004,   /* U+32C9 */
  0x0004,   /* U+32CA */
  0x0004,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x001C,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x001D,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0318,   /* U+320E */
  0x0318,   /* U+320F */
  0x0318,   /* U+3210 */
  0x0318,   /* U+3211 */
  0x0318,   /* U+3212 */
  0x0318,   /* U+3213 */
  0x0318,   /* U+3214 */
  0x0318,   /* U+3215 */
  0x0318,   /* U+3216 */
  0x0318,   /* U+3217 */
  0x0318,   /* U+3218 */
  0x0318,   /* U+3219 */
  0x0318,   /* U+321A */
  0x0318,   /* U+321B */
  0x0318,   /* U+321C */
  0x3C01,   /* U+321D */
  0x3C07,   /* U+321E */
  0x0000,   /* U+321F */
  0x0318,   /* U+3220 */
  0x0318,   /* U+3221 */
  0x0318,   /* U+3222 */
  0x0318,   /* U+3223 */
  0x0318,   /* U+3224 */
  0x0318,   /* U+3225 */
  0x0318,   /* U+3226 */
  0x0318,   /* U+3227 */
  0x0318,   /* U+3228 */
  0x0318,   /* U+3229 */
  0x0318,   /* U+322A */
  0x0318,   /* U+322B */
  0x0318,   /* U+322C */
  0x0318,   /* U+322D */
  0x0318,   /* U+322E */
  0x0318,   /* U+322F */
  0x0318,   /* U+3230 */
  0x0318,   /* U+3231 */
  0x0318,   /* U+3232 */
  0x0318,   /* U+3233 */
  0x0318,   /* U+3234 */
  0x0318,   /* U+3235 */
  0x0318,   /* U+3236 */
  0x0318,   /* U+3237 */
  0x0318,   /* U+3238 */
  0x0318,   /* U+3239 */
  0x0318,   /* U+323A */
  0x0318,   /* U+323B */
  0x0318,   /* U+323C */
  0x0318,   /* U+323D */
  0x0318,   /* U+323E */
  0x0318,   /* U+323F */
  0x0318,   /* U+3240 */
  0x0318,   /* U+3241 */
  0x0318,   /* U+3242 */
  0x0318,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x3BF5,   /* U+327C */
  0x3C86,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0xE708,   /* U+32C9 */
  0xE708,   /* U+32CA */
  0xE708,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0020,   /* U+320E */
  0x0020,   /* U+320F */
  0x0020,   /* U+3210 */
  0x0020,   /* U+3211 */
  0x0020,   /* U+3212 */
  0x0020,   /* U+3213 */
  0x0020,   /* U+3214 */
  0x0020,   /* U+3215 */
  0x0020,   /* U+3216 */
  0x0020,   /* U+3217 */
  0x0020,   /* U+3218 */
  0x0020,   /* U+3219 */
  0x0020,   /* U+321A */
  0x0020,   /* U+321B */
  0x0020,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0000,   /* U+321F */
  0x0020,   /* U+3220 */
  0x0020,   /* U+3221 */
  0x0020,   /* U+3222 */
  0x0020,   /* U+3223 */
  0x0020,   /* U+3224 */
  0x0020,   /* U+3225 */
  0x0020,   /* U+3226 */
  0x0020,   /* U+3227 */
  0x0020,   /* U+3228 */
  0x0020,   /* U+3229 */
  0x0020,   /* U+322A */
  0x0020,   /* U+322B */
  0x0020,   /* U+322C */
  0x0020,   /* U+322D */
  0x0020,   /* U+322E */
  0x0020,   /* U+322F */
  0x0020,   /* U+3230 */
  0x0020,   /* U+3231 */
  0x0020,   /* U+3232 */
  0x0020,   /* U+3233 */
  0x0020,   /* U+3234 */
  0x0020,   /* U+3235 */
  0x0020,   /* U+3236 */
  0x0020,   /* U+3237 */
  0x0020,   /* U+3238 */
  0x0020,   /* U+3239 */
  0x0020,   /* U+323A */
  0x0020,   /* U+323B */
  0x0020,   /* U+323C */
  0x0020,   /* U+323D */
  0x0020,   /* U+323E */
  0x0020,   /* U+323F */
  0x0020,   /* U+3240 */
  0x0020,   /* U+3241 */
  0x0020,   /* U+3242 */
  0x0020,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0020,   /* U+327C */
  0x0020,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0004,   /* U+320E */
  0x0004,   /* U+320F */
  0x0004,   /* U+3210 */
  0x0004,   /* U+3211 */
  0x0004,   /* U+3212 */
  0x0004,   /* U+3213 */
  0x0004,   /* U+3214 */
  0x0004,   /* U+3215 */
  0x0004,   /* U+3216 */
  0x0004,   /* U+3217 */
  0x0004,   /* U+3218 */
  0x0004,   /* U+3219 */
  0x0004,   /* U+321A */
  0x0004,   /* U+321B */
  0x0004,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0000,   /* U+321F */
  0x0004,   /* U+3220 */
  0x0004,   /* U+3221 */
  0x0004,   /* U+3222 */
  0x0004,   /* U+3223 */
  0x0004,   /* U+3224 */
  0x0004,   /* U+3225 */
  0x0004,   /* U+3226 */
  0x0004,   /* U+3227 */
  0x0004,   /* U+3228 */
  0x0004,   /* U+3229 */
  0x0004,   /* U+322A */
  0x0004,   /* U+322B */
  0x0004,   /* U+322C */
  0x0004,   /* U+322D */
  0x0004,   /* U+322E */
  0x0004,   /* U+322F */
  0x0004,   /* U+3230 */
  0x0004,   /* U+3231 */
  0x0004,   /* U+3232 */
  0x0004,   /* U+3233 */
  0x0004,   /* U+3234 */
  0x0004,   /* U+3235 */
  0x0004,   /* U+3236 */
  0x0004,   /* U+3237 */
  0x0004,   /* U+3238 */
  0x0004,   /* U+3239 */
  0x0004,   /* U+323A */
  0x0004,   /* U+323B */
  0x0004,   /* U+323C */
  0x0004,   /* U+323D */
  0x0004,   /* U+323E */
  0x0004,   /* U+323F */
  0x0004,   /* U+3240 */
  0x0004,   /* U+3241 */
  0x0004,   /* U+3242 */
  0x0004,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0006,   /* U+327C */
  0x0006,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Primary weight 5 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x3C77,   /* U+321D */
  0x3C80,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x3C7B,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Secondary weight 5 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0020,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 5 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0006,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Primary weight 6 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x3CD4,   /* U+321D */
  0x0318,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Secondary weight 6 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0020,   /* U+321D */
  0x0020,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 6 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0004,   /* U+321D */
  0x0004,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Primary weight 7 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0318,   /* U+321D */
  0x0000,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Secondary weight 7 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0020,   /* U+321D */
  0x0000,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */

  /* Tertiary weight 7 for each character. */
  0x0000,   /* U+3200 */
  0x0000,   /* U+3201 */
  0x0000,   /* U+3202 */
  0x0000,   /* U+3203 */
  0x0000,   /* U+3204 */
  0x0000,   /* U+3205 */
  0x0000,   /* U+3206 */
  0x0000,   /* U+3207 */
  0x0000,   /* U+3208 */
  0x0000,   /* U+3209 */
  0x0000,   /* U+320A */
  0x0000,   /* U+320B */
  0x0000,   /* U+320C */
  0x0000,   /* U+320D */
  0x0000,   /* U+320E */
  0x0000,   /* U+320F */
  0x0000,   /* U+3210 */
  0x0000,   /* U+3211 */
  0x0000,   /* U+3212 */
  0x0000,   /* U+3213 */
  0x0000,   /* U+3214 */
  0x0000,   /* U+3215 */
  0x0000,   /* U+3216 */
  0x0000,   /* U+3217 */
  0x0000,   /* U+3218 */
  0x0000,   /* U+3219 */
  0x0000,   /* U+321A */
  0x0000,   /* U+321B */
  0x0000,   /* U+321C */
  0x0004,   /* U+321D */
  0x0000,   /* U+321E */
  0x0000,   /* U+321F */
  0x0000,   /* U+3220 */
  0x0000,   /* U+3221 */
  0x0000,   /* U+3222 */
  0x0000,   /* U+3223 */
  0x0000,   /* U+3224 */
  0x0000,   /* U+3225 */
  0x0000,   /* U+3226 */
  0x0000,   /* U+3227 */
  0x0000,   /* U+3228 */
  0x0000,   /* U+3229 */
  0x0000,   /* U+322A */
  0x0000,   /* U+322B */
  0x0000,   /* U+322C */
  0x0000,   /* U+322D */
  0x0000,   /* U+322E */
  0x0000,   /* U+322F */
  0x0000,   /* U+3230 */
  0x0000,   /* U+3231 */
  0x0000,   /* U+3232 */
  0x0000,   /* U+3233 */
  0x0000,   /* U+3234 */
  0x0000,   /* U+3235 */
  0x0000,   /* U+3236 */
  0x0000,   /* U+3237 */
  0x0000,   /* U+3238 */
  0x0000,   /* U+3239 */
  0x0000,   /* U+323A */
  0x0000,   /* U+323B */
  0x0000,   /* U+323C */
  0x0000,   /* U+323D */
  0x0000,   /* U+323E */
  0x0000,   /* U+323F */
  0x0000,   /* U+3240 */
  0x0000,   /* U+3241 */
  0x0000,   /* U+3242 */
  0x0000,   /* U+3243 */
  0x0000,   /* U+3244 */
  0x0000,   /* U+3245 */
  0x0000,   /* U+3246 */
  0x0000,   /* U+3247 */
  0x0000,   /* U+3248 */
  0x0000,   /* U+3249 */
  0x0000,   /* U+324A */
  0x0000,   /* U+324B */
  0x0000,   /* U+324C */
  0x0000,   /* U+324D */
  0x0000,   /* U+324E */
  0x0000,   /* U+324F */
  0x0000,   /* U+3250 */
  0x0000,   /* U+3251 */
  0x0000,   /* U+3252 */
  0x0000,   /* U+3253 */
  0x0000,   /* U+3254 */
  0x0000,   /* U+3255 */
  0x0000,   /* U+3256 */
  0x0000,   /* U+3257 */
  0x0000,   /* U+3258 */
  0x0000,   /* U+3259 */
  0x0000,   /* U+325A */
  0x0000,   /* U+325B */
  0x0000,   /* U+325C */
  0x0000,   /* U+325D */
  0x0000,   /* U+325E */
  0x0000,   /* U+325F */
  0x0000,   /* U+3260 */
  0x0000,   /* U+3261 */
  0x0000,   /* U+3262 */
  0x0000,   /* U+3263 */
  0x0000,   /* U+3264 */
  0x0000,   /* U+3265 */
  0x0000,   /* U+3266 */
  0x0000,   /* U+3267 */
  0x0000,   /* U+3268 */
  0x0000,   /* U+3269 */
  0x0000,   /* U+326A */
  0x0000,   /* U+326B */
  0x0000,   /* U+326C */
  0x0000,   /* U+326D */
  0x0000,   /* U+326E */
  0x0000,   /* U+326F */
  0x0000,   /* U+3270 */
  0x0000,   /* U+3271 */
  0x0000,   /* U+3272 */
  0x0000,   /* U+3273 */
  0x0000,   /* U+3274 */
  0x0000,   /* U+3275 */
  0x0000,   /* U+3276 */
  0x0000,   /* U+3277 */
  0x0000,   /* U+3278 */
  0x0000,   /* U+3279 */
  0x0000,   /* U+327A */
  0x0000,   /* U+327B */
  0x0000,   /* U+327C */
  0x0000,   /* U+327D */
  0x0000,   /* U+327E */
  0x0000,   /* U+327F */
  0x0000,   /* U+3280 */
  0x0000,   /* U+3281 */
  0x0000,   /* U+3282 */
  0x0000,   /* U+3283 */
  0x0000,   /* U+3284 */
  0x0000,   /* U+3285 */
  0x0000,   /* U+3286 */
  0x0000,   /* U+3287 */
  0x0000,   /* U+3288 */
  0x0000,   /* U+3289 */
  0x0000,   /* U+328A */
  0x0000,   /* U+328B */
  0x0000,   /* U+328C */
  0x0000,   /* U+328D */
  0x0000,   /* U+328E */
  0x0000,   /* U+328F */
  0x0000,   /* U+3290 */
  0x0000,   /* U+3291 */
  0x0000,   /* U+3292 */
  0x0000,   /* U+3293 */
  0x0000,   /* U+3294 */
  0x0000,   /* U+3295 */
  0x0000,   /* U+3296 */
  0x0000,   /* U+3297 */
  0x0000,   /* U+3298 */
  0x0000,   /* U+3299 */
  0x0000,   /* U+329A */
  0x0000,   /* U+329B */
  0x0000,   /* U+329C */
  0x0000,   /* U+329D */
  0x0000,   /* U+329E */
  0x0000,   /* U+329F */
  0x0000,   /* U+32A0 */
  0x0000,   /* U+32A1 */
  0x0000,   /* U+32A2 */
  0x0000,   /* U+32A3 */
  0x0000,   /* U+32A4 */
  0x0000,   /* U+32A5 */
  0x0000,   /* U+32A6 */
  0x0000,   /* U+32A7 */
  0x0000,   /* U+32A8 */
  0x0000,   /* U+32A9 */
  0x0000,   /* U+32AA */
  0x0000,   /* U+32AB */
  0x0000,   /* U+32AC */
  0x0000,   /* U+32AD */
  0x0000,   /* U+32AE */
  0x0000,   /* U+32AF */
  0x0000,   /* U+32B0 */
  0x0000,   /* U+32B1 */
  0x0000,   /* U+32B2 */
  0x0000,   /* U+32B3 */
  0x0000,   /* U+32B4 */
  0x0000,   /* U+32B5 */
  0x0000,   /* U+32B6 */
  0x0000,   /* U+32B7 */
  0x0000,   /* U+32B8 */
  0x0000,   /* U+32B9 */
  0x0000,   /* U+32BA */
  0x0000,   /* U+32BB */
  0x0000,   /* U+32BC */
  0x0000,   /* U+32BD */
  0x0000,   /* U+32BE */
  0x0000,   /* U+32BF */
  0x0000,   /* U+32C0 */
  0x0000,   /* U+32C1 */
  0x0000,   /* U+32C2 */
  0x0000,   /* U+32C3 */
  0x0000,   /* U+32C4 */
  0x0000,   /* U+32C5 */
  0x0000,   /* U+32C6 */
  0x0000,   /* U+32C7 */
  0x0000,   /* U+32C8 */
  0x0000,   /* U+32C9 */
  0x0000,   /* U+32CA */
  0x0000,   /* U+32CB */
  0x0000,   /* U+32CC */
  0x0000,   /* U+32CD */
  0x0000,   /* U+32CE */
  0x0000,   /* U+32CF */
  0x0000,   /* U+32D0 */
  0x0000,   /* U+32D1 */
  0x0000,   /* U+32D2 */
  0x0000,   /* U+32D3 */
  0x0000,   /* U+32D4 */
  0x0000,   /* U+32D5 */
  0x0000,   /* U+32D6 */
  0x0000,   /* U+32D7 */
  0x0000,   /* U+32D8 */
  0x0000,   /* U+32D9 */
  0x0000,   /* U+32DA */
  0x0000,   /* U+32DB */
  0x0000,   /* U+32DC */
  0x0000,   /* U+32DD */
  0x0000,   /* U+32DE */
  0x0000,   /* U+32DF */
  0x0000,   /* U+32E0 */
  0x0000,   /* U+32E1 */
  0x0000,   /* U+32E2 */
  0x0000,   /* U+32E3 */
  0x0000,   /* U+32E4 */
  0x0000,   /* U+32E5 */
  0x0000,   /* U+32E6 */
  0x0000,   /* U+32E7 */
  0x0000,   /* U+32E8 */
  0x0000,   /* U+32E9 */
  0x0000,   /* U+32EA */
  0x0000,   /* U+32EB */
  0x0000,   /* U+32EC */
  0x0000,   /* U+32ED */
  0x0000,   /* U+32EE */
  0x0000,   /* U+32EF */
  0x0000,   /* U+32F0 */
  0x0000,   /* U+32F1 */
  0x0000,   /* U+32F2 */
  0x0000,   /* U+32F3 */
  0x0000,   /* U+32F4 */
  0x0000,   /* U+32F5 */
  0x0000,   /* U+32F6 */
  0x0000,   /* U+32F7 */
  0x0000,   /* U+32F8 */
  0x0000,   /* U+32F9 */
  0x0000,   /* U+32FA */
  0x0000,   /* U+32FB */
  0x0000,   /* U+32FC */
  0x0000,   /* U+32FD */
  0x0000,   /* U+32FE */
  0x0000,   /* U+32FF */
};

uint16 uca900_p033[]= {
  /* Number of CEs for each character. */
  5, 4, 5, 3, 5, 3, 3, 6, 4, 3, 3, 3, 4, 4, 4, 4, 
  4, 4, 4, 6, 2, 6, 6, 5, 4, 6, 6, 4, 3, 3, 4, 4, 
  5, 5, 3, 3, 4, 3, 3, 2, 2, 3, 3, 6, 4, 5, 6, 4, 
  3, 3, 6, 4, 6, 3, 5, 3, 4, 3, 4, 5, 4, 5, 4, 2, 
  5, 3, 3, 4, 3, 3, 3, 5, 4, 2, 6, 3, 5, 4, 4, 3, 
  3, 4, 2, 4, 5, 2, 6, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 3, 2, 2, 3, 2, 2, 2, 3, 3, 2, 4, 4, 4, 4, 8, 
  2, 2, 2, 2, 2, 2, 2, 2, 3, 4, 2, 2, 2, 2, 2, 2, 
  2, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 
  3, 2, 3, 3, 3, 2, 3, 3, 4, 2, 3, 3, 3, 3, 5, 6, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 4, 2, 2, 2, 4, 3, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 3, 2, 2, 3, 3, 2, 4, 3, 2, 2, 2, 2, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 3, 

  /* Primary weight 1 for each character. */
  0x3D5A,   /* U+3300 */
  0x3D5A,   /* U+3301 */
  0x3D5A,   /* U+3302 */
  0x3D5A,   /* U+3303 */
  0x3D5B,   /* U+3304 */
  0x3D5B,   /* U+3305 */
  0x3D5C,   /* U+3306 */
  0x3D5E,   /* U+3307 */
  0x3D5E,   /* U+3308 */
  0x3D5F,   /* U+3309 */
  0x3D5F,   /* U+330A */
  0x3D60,   /* U+330B */
  0x3D60,   /* U+330C */
  0x3D60,   /* U+330D */
  0x3D60,   /* U+330E */
  0x3D60,   /* U+330F */
  0x3D61,   /* U+3310 */
  0x3D61,   /* U+3311 */
  0x3D61,   /* U+3312 */
  0x3D61,   /* U+3313 */
  0x3D61,   /* U+3314 */
  0x3D61,   /* U+3315 */
  0x3D61,   /* U+3316 */
  0x3D61,   /* U+3317 */
  0x3D62,   /* U+3318 */
  0x3D62,   /* U+3319 */
  0x3D62,   /* U+331A */
  0x3D62,   /* U+331B */
  0x3D63,   /* U+331C */
  0x3D64,   /* U+331D */
  0x3D64,   /* U+331E */
  0x3D65,   /* U+331F */
  0x3D65,   /* U+3320 */
  0x3D66,   /* U+3321 */
  0x3D68,   /* U+3322 */
  0x3D68,   /* U+3323 */
  0x3D6A,   /* U+3324 */
  0x3D6D,   /* U+3325 */
  0x3D6E,   /* U+3326 */
  0x3D6E,   /* U+3327 */
  0x3D6F,   /* U+3328 */
  0x3D73,   /* U+3329 */
  0x3D74,   /* U+332A */
  0x3D74,   /* U+332B */
  0x3D74,   /* U+332C */
  0x3D74,   /* U+332D */
  0x3D75,   /* U+332E */
  0x3D75,   /* U+332F */
  0x3D75,   /* U+3330 */
  0x3D75,   /* U+3331 */
  0x3D76,   /* U+3332 */
  0x3D76,   /* U+3333 */
  0x3D76,   /* U+3334 */
  0x3D76,   /* U+3335 */
  0x3D77,   /* U+3336 */
  0x3D77,   /* U+3337 */
  0x3D77,   /* U+3338 */
  0x3D77,   /* U+3339 */
  0x3D77,   /* U+333A */
  0x3D77,   /* U+333B */
  0x3D77,   /* U+333C */
  0x3D78,   /* U+333D */
  0x3D78,   /* U+333E */
  0x3D78,   /* U+333F */
  0x3D78,   /* U+3340 */
  0x3D78,   /* U+3341 */
  0x3D78,   /* U+3342 */
  0x3D79,   /* U+3343 */
  0x3D79,   /* U+3344 */
  0x3D79,   /* U+3345 */
  0x3D79,   /* U+3346 */
  0x3D79,   /* U+3347 */
  0x3D7A,   /* U+3348 */
  0x3D7A,   /* U+3349 */
  0x3D7A,   /* U+334A */
  0x3D7C,   /* U+334B */
  0x3D7C,   /* U+334C */
  0x3D7C,   /* U+334D */
  0x3D7E,   /* U+334E */
  0x3D7E,   /* U+334F */
  0x3D7F,   /* U+3350 */
  0x3D83,   /* U+3351 */
  0x3D83,   /* U+3352 */
  0x3D84,   /* U+3353 */
  0x3D84,   /* U+3354 */
  0x3D85,   /* U+3355 */
  0x3D85,   /* U+3356 */
  0x3D87,   /* U+3357 */
  0x1C3D,   /* U+3358 */
  0x1C3E,   /* U+3359 */
  0x1C3F,   /* U+335A */
  0x1C40,   /* U+335B */
  0x1C41,   /* U+335C */
  0x1C42,   /* U+335D */
  0x1C43,   /* U+335E */
  0x1C44,   /* U+335F */
  0x1C45,   /* U+3360 */
  0x1C46,   /* U+3361 */
  0x1C3E,   /* U+3362 */
  0x1C3E,   /* U+3363 */
  0x1C3E,   /* U+3364 */
  0x1C3E,   /* U+3365 */
  0x1C3E,   /* U+3366 */
  0x1C3E,   /* U+3367 */
  0x1C3E,   /* U+3368 */
  0x1C3E,   /* U+3369 */
  0x1C3E,   /* U+336A */
  0x1C3E,   /* U+336B */
  0x1C3F,   /* U+336C */
  0x1C3F,   /* U+336D */
  0x1C3F,   /* U+336E */
  0x1C3F,   /* U+336F */
  0x1C3F,   /* U+3370 */
  0x1D18,   /* U+3371 */
  0x1C8F,   /* U+3372 */
  0x1C47,   /* U+3373 */
  0x1C60,   /* U+3374 */
  0x1DDD,   /* U+3375 */
  0x1E0C,   /* U+3376 */
  0x1C8F,   /* U+3377 */
  0x1C8F,   /* U+3378 */
  0x1C8F,   /* U+3379 */
  0x1D32,   /* U+337A */
  0xFB40,   /* U+337B */
  0xFB40,   /* U+337C */
  0xFB40,   /* U+337D */
  0xFB40,   /* U+337E */
  0xFB40,   /* U+337F */
  0x1E0C,   /* U+3380 */
  0x1DB9,   /* U+3381 */
  0x1FCB,   /* U+3382 */
  0x1DAA,   /* U+3383 */
  0x1D65,   /* U+3384 */
  0x1D65,   /* U+3385 */
  0x1DAA,   /* U+3386 */
  0x1CF4,   /* U+3387 */
  0x1C7A,   /* U+3388 */
  0x1D65,   /* U+3389 */
  0x1E0C,   /* U+338A */
  0x1DB9,   /* U+338B */
  0x1FCB,   /* U+338C */
  0x1FCB,   /* U+338D */
  0x1DAA,   /* U+338E */
  0x1D65,   /* U+338F */
  0x1D18,   /* U+3390 */
  0x1D65,   /* U+3391 */
  0x1DAA,   /* U+3392 */
  0x1CF4,   /* U+3393 */
  0x1E95,   /* U+3394 */
  0x1FCB,   /* U+3395 */
  0x1DAA,   /* U+3396 */
  0x1C8F,   /* U+3397 */
  0x1D65,   /* U+3398 */
  0x1CE5,   /* U+3399 */
  0x1DB9,   /* U+339A */
  0x1FCB,   /* U+339B */
  0x1DAA,   /* U+339C */
  0x1C7A,   /* U+339D */
  0x1D65,   /* U+339E */
  0x1DAA,   /* U+339F */
  0x1C7A,   /* U+33A0 */
  0x1DAA,   /* U+33A1 */
  0x1D65,   /* U+33A2 */
  0x1DAA,   /* U+33A3 */
  0x1C7A,   /* U+33A4 */
  0x1DAA,   /* U+33A5 */
  0x1D65,   /* U+33A6 */
  0x1DAA,   /* U+33A7 */
  0x1DAA,   /* U+33A8 */
  0x1E0C,   /* U+33A9 */
  0x1D65,   /* U+33AA */
  0x1DAA,   /* U+33AB */
  0x1CF4,   /* U+33AC */
  0x1E33,   /* U+33AD */
  0x1E33,   /* U+33AE */
  0x1E33,   /* U+33AF */
  0x1E0C,   /* U+33B0 */
  0x1DB9,   /* U+33B1 */
  0x1FCB,   /* U+33B2 */
  0x1DAA,   /* U+33B3 */
  0x1E0C,   /* U+33B4 */
  0x1DB9,   /* U+33B5 */
  0x1FCB,   /* U+33B6 */
  0x1DAA,   /* U+33B7 */
  0x1D65,   /* U+33B8 */
  0x1DAA,   /* U+33B9 */
  0x1E0C,   /* U+33BA */
  0x1DB9,   /* U+33BB */
  0x1FCB,   /* U+33BC */
  0x1DAA,   /* U+33BD */
  0x1D65,   /* U+33BE */
  0x1DAA,   /* U+33BF */
  0x1D65,   /* U+33C0 */
  0x1DAA,   /* U+33C1 */
  0x1C47,   /* U+33C2 */
  0x1C60,   /* U+33C3 */
  0x1C7A,   /* U+33C4 */
  0x1C7A,   /* U+33C5 */
  0x1C7A,   /* U+33C6 */
  0x1C7A,   /* U+33C7 */
  0x1C8F,   /* U+33C8 */
  0x1CF4,   /* U+33C9 */
  0x1D18,   /* U+33CA */
  0x1D18,   /* U+33CB */
  0x1D32,   /* U+33CC */
  0x1D65,   /* U+33CD */
  0x1D65,   /* U+33CE */
  0x1D65,   /* U+33CF */
  0x1D77,   /* U+33D0 */
  0x1D77,   /* U+33D1 */
  0x1D77,   /* U+33D2 */
  0x1D77,   /* U+33D3 */
  0x1DAA,   /* U+33D4 */
  0x1DAA,   /* U+33D5 */
  0x1DAA,   /* U+33D6 */
  0x1E0C,   /* U+33D7 */
  0x1E0C,   /* U+33D8 */
  0x1E0C,   /* U+33D9 */
  0x1E0C,   /* U+33DA */
  0x1E71,   /* U+33DB */
  0x1E71,   /* U+33DC */
  0x1EF5,   /* U+33DD */
  0x1EE3,   /* U+33DE */
  0x1C47,   /* U+33DF */
  0x1C3E,   /* U+33E0 */
  0x1C3F,   /* U+33E1 */
  0x1C40,   /* U+33E2 */
  0x1C41,   /* U+33E3 */
  0x1C42,   /* U+33E4 */
  0x1C43,   /* U+33E5 */
  0x1C44,   /* U+33E6 */
  0x1C45,   /* U+33E7 */
  0x1C46,   /* U+33E8 */
  0x1C3E,   /* U+33E9 */
  0x1C3E,   /* U+33EA */
  0x1C3E,   /* U+33EB */
  0x1C3E,   /* U+33EC */
  0x1C3E,   /* U+33ED */
  0x1C3E,   /* U+33EE */
  0x1C3E,   /* U+33EF */
  0x1C3E,   /* U+33F0 */
  0x1C3E,   /* U+33F1 */
  0x1C3E,   /* U+33F2 */
  0x1C3F,   /* U+33F3 */
  0x1C3F,   /* U+33F4 */
  0x1C3F,   /* U+33F5 */
  0x1C3F,   /* U+33F6 */
  0x1C3F,   /* U+33F7 */
  0x1C3F,   /* U+33F8 */
  0x1C3F,   /* U+33F9 */
  0x1C3F,   /* U+33FA */
  0x1C3F,   /* U+33FB */
  0x1C3F,   /* U+33FC */
  0x1C40,   /* U+33FD */
  0x1C40,   /* U+33FE */
  0x1CF4,   /* U+33FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+3300 */
  0x0020,   /* U+3301 */
  0x0020,   /* U+3302 */
  0x0020,   /* U+3303 */
  0x0020,   /* U+3304 */
  0x0020,   /* U+3305 */
  0x0020,   /* U+3306 */
  0x0020,   /* U+3307 */
  0x0020,   /* U+3308 */
  0x0020,   /* U+3309 */
  0x0020,   /* U+330A */
  0x0020,   /* U+330B */
  0x0020,   /* U+330C */
  0x0020,   /* U+330D */
  0x0020,   /* U+330E */
  0x0020,   /* U+330F */
  0x0020,   /* U+3310 */
  0x0020,   /* U+3311 */
  0x0020,   /* U+3312 */
  0x0020,   /* U+3313 */
  0x0020,   /* U+3314 */
  0x0020,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0020,   /* U+3317 */
  0x0020,   /* U+3318 */
  0x0020,   /* U+3319 */
  0x0020,   /* U+331A */
  0x0020,   /* U+331B */
  0x0020,   /* U+331C */
  0x0020,   /* U+331D */
  0x0020,   /* U+331E */
  0x0020,   /* U+331F */
  0x0020,   /* U+3320 */
  0x0020,   /* U+3321 */
  0x0020,   /* U+3322 */
  0x0020,   /* U+3323 */
  0x0020,   /* U+3324 */
  0x0020,   /* U+3325 */
  0x0020,   /* U+3326 */
  0x0020,   /* U+3327 */
  0x0020,   /* U+3328 */
  0x0020,   /* U+3329 */
  0x0020,   /* U+332A */
  0x0020,   /* U+332B */
  0x0020,   /* U+332C */
  0x0020,   /* U+332D */
  0x0020,   /* U+332E */
  0x0020,   /* U+332F */
  0x0020,   /* U+3330 */
  0x0020,   /* U+3331 */
  0x0020,   /* U+3332 */
  0x0020,   /* U+3333 */
  0x0020,   /* U+3334 */
  0x0020,   /* U+3335 */
  0x0020,   /* U+3336 */
  0x0020,   /* U+3337 */
  0x0020,   /* U+3338 */
  0x0020,   /* U+3339 */
  0x0020,   /* U+333A */
  0x0020,   /* U+333B */
  0x0020,   /* U+333C */
  0x0020,   /* U+333D */
  0x0020,   /* U+333E */
  0x0020,   /* U+333F */
  0x0020,   /* U+3340 */
  0x0020,   /* U+3341 */
  0x0020,   /* U+3342 */
  0x0020,   /* U+3343 */
  0x0020,   /* U+3344 */
  0x0020,   /* U+3345 */
  0x0020,   /* U+3346 */
  0x0020,   /* U+3347 */
  0x0020,   /* U+3348 */
  0x0020,   /* U+3349 */
  0x0020,   /* U+334A */
  0x0020,   /* U+334B */
  0x0020,   /* U+334C */
  0x0020,   /* U+334D */
  0x0020,   /* U+334E */
  0x0020,   /* U+334F */
  0x0020,   /* U+3350 */
  0x0020,   /* U+3351 */
  0x0020,   /* U+3352 */
  0x0020,   /* U+3353 */
  0x0020,   /* U+3354 */
  0x0020,   /* U+3355 */
  0x0020,   /* U+3356 */
  0x0020,   /* U+3357 */
  0x0020,   /* U+3358 */
  0x0020,   /* U+3359 */
  0x0020,   /* U+335A */
  0x0020,   /* U+335B */
  0x0020,   /* U+335C */
  0x0020,   /* U+335D */
  0x0020,   /* U+335E */
  0x0020,   /* U+335F */
  0x0020,   /* U+3360 */
  0x0020,   /* U+3361 */
  0x0020,   /* U+3362 */
  0x0020,   /* U+3363 */
  0x0020,   /* U+3364 */
  0x0020,   /* U+3365 */
  0x0020,   /* U+3366 */
  0x0020,   /* U+3367 */
  0x0020,   /* U+3368 */
  0x0020,   /* U+3369 */
  0x0020,   /* U+336A */
  0x0020,   /* U+336B */
  0x0020,   /* U+336C */
  0x0020,   /* U+336D */
  0x0020,   /* U+336E */
  0x0020,   /* U+336F */
  0x0020,   /* U+3370 */
  0x0020,   /* U+3371 */
  0x0020,   /* U+3372 */
  0x0020,   /* U+3373 */
  0x0020,   /* U+3374 */
  0x0020,   /* U+3375 */
  0x0020,   /* U+3376 */
  0x0020,   /* U+3377 */
  0x0020,   /* U+3378 */
  0x0020,   /* U+3379 */
  0x0020,   /* U+337A */
  0x0020,   /* U+337B */
  0x0020,   /* U+337C */
  0x0020,   /* U+337D */
  0x0020,   /* U+337E */
  0x0020,   /* U+337F */
  0x0020,   /* U+3380 */
  0x0020,   /* U+3381 */
  0x0020,   /* U+3382 */
  0x0020,   /* U+3383 */
  0x0020,   /* U+3384 */
  0x0020,   /* U+3385 */
  0x0020,   /* U+3386 */
  0x0020,   /* U+3387 */
  0x0020,   /* U+3388 */
  0x0020,   /* U+3389 */
  0x0020,   /* U+338A */
  0x0020,   /* U+338B */
  0x0020,   /* U+338C */
  0x0020,   /* U+338D */
  0x0020,   /* U+338E */
  0x0020,   /* U+338F */
  0x0020,   /* U+3390 */
  0x0020,   /* U+3391 */
  0x0020,   /* U+3392 */
  0x0020,   /* U+3393 */
  0x0020,   /* U+3394 */
  0x0020,   /* U+3395 */
  0x0020,   /* U+3396 */
  0x0020,   /* U+3397 */
  0x0020,   /* U+3398 */
  0x0020,   /* U+3399 */
  0x0020,   /* U+339A */
  0x0020,   /* U+339B */
  0x0020,   /* U+339C */
  0x0020,   /* U+339D */
  0x0020,   /* U+339E */
  0x0020,   /* U+339F */
  0x0020,   /* U+33A0 */
  0x0020,   /* U+33A1 */
  0x0020,   /* U+33A2 */
  0x0020,   /* U+33A3 */
  0x0020,   /* U+33A4 */
  0x0020,   /* U+33A5 */
  0x0020,   /* U+33A6 */
  0x0020,   /* U+33A7 */
  0x0020,   /* U+33A8 */
  0x0020,   /* U+33A9 */
  0x0020,   /* U+33AA */
  0x0020,   /* U+33AB */
  0x0020,   /* U+33AC */
  0x0020,   /* U+33AD */
  0x0020,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0020,   /* U+33B0 */
  0x0020,   /* U+33B1 */
  0x0020,   /* U+33B2 */
  0x0020,   /* U+33B3 */
  0x0020,   /* U+33B4 */
  0x0020,   /* U+33B5 */
  0x0020,   /* U+33B6 */
  0x0020,   /* U+33B7 */
  0x0020,   /* U+33B8 */
  0x0020,   /* U+33B9 */
  0x0020,   /* U+33BA */
  0x0020,   /* U+33BB */
  0x0020,   /* U+33BC */
  0x0020,   /* U+33BD */
  0x0020,   /* U+33BE */
  0x0020,   /* U+33BF */
  0x0020,   /* U+33C0 */
  0x0020,   /* U+33C1 */
  0x0020,   /* U+33C2 */
  0x0020,   /* U+33C3 */
  0x0020,   /* U+33C4 */
  0x0020,   /* U+33C5 */
  0x0020,   /* U+33C6 */
  0x0020,   /* U+33C7 */
  0x0020,   /* U+33C8 */
  0x0020,   /* U+33C9 */
  0x0020,   /* U+33CA */
  0x0020,   /* U+33CB */
  0x0020,   /* U+33CC */
  0x0020,   /* U+33CD */
  0x0020,   /* U+33CE */
  0x0020,   /* U+33CF */
  0x0020,   /* U+33D0 */
  0x0020,   /* U+33D1 */
  0x0020,   /* U+33D2 */
  0x0020,   /* U+33D3 */
  0x0020,   /* U+33D4 */
  0x0020,   /* U+33D5 */
  0x0020,   /* U+33D6 */
  0x0020,   /* U+33D7 */
  0x0020,   /* U+33D8 */
  0x0020,   /* U+33D9 */
  0x0020,   /* U+33DA */
  0x0020,   /* U+33DB */
  0x0020,   /* U+33DC */
  0x0020,   /* U+33DD */
  0x0020,   /* U+33DE */
  0x0020,   /* U+33DF */
  0x0020,   /* U+33E0 */
  0x0020,   /* U+33E1 */
  0x0020,   /* U+33E2 */
  0x0020,   /* U+33E3 */
  0x0020,   /* U+33E4 */
  0x0020,   /* U+33E5 */
  0x0020,   /* U+33E6 */
  0x0020,   /* U+33E7 */
  0x0020,   /* U+33E8 */
  0x0020,   /* U+33E9 */
  0x0020,   /* U+33EA */
  0x0020,   /* U+33EB */
  0x0020,   /* U+33EC */
  0x0020,   /* U+33ED */
  0x0020,   /* U+33EE */
  0x0020,   /* U+33EF */
  0x0020,   /* U+33F0 */
  0x0020,   /* U+33F1 */
  0x0020,   /* U+33F2 */
  0x0020,   /* U+33F3 */
  0x0020,   /* U+33F4 */
  0x0020,   /* U+33F5 */
  0x0020,   /* U+33F6 */
  0x0020,   /* U+33F7 */
  0x0020,   /* U+33F8 */
  0x0020,   /* U+33F9 */
  0x0020,   /* U+33FA */
  0x0020,   /* U+33FB */
  0x0020,   /* U+33FC */
  0x0020,   /* U+33FD */
  0x0020,   /* U+33FE */
  0x0020,   /* U+33FF */

  /* Tertiary weight 1 for each character. */
  0x001C,   /* U+3300 */
  0x001C,   /* U+3301 */
  0x001C,   /* U+3302 */
  0x001C,   /* U+3303 */
  0x001C,   /* U+3304 */
  0x001C,   /* U+3305 */
  0x001C,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x001C,   /* U+3308 */
  0x001C,   /* U+3309 */
  0x001C,   /* U+330A */
  0x001C,   /* U+330B */
  0x001C,   /* U+330C */
  0x001C,   /* U+330D */
  0x001C,   /* U+330E */
  0x001C,   /* U+330F */
  0x001C,   /* U+3310 */
  0x001C,   /* U+3311 */
  0x001C,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x001C,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x001C,   /* U+3317 */
  0x001C,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x001C,   /* U+331B */
  0x001C,   /* U+331C */
  0x001C,   /* U+331D */
  0x001C,   /* U+331E */
  0x001C,   /* U+331F */
  0x001C,   /* U+3320 */
  0x001C,   /* U+3321 */
  0x001C,   /* U+3322 */
  0x001C,   /* U+3323 */
  0x001C,   /* U+3324 */
  0x001C,   /* U+3325 */
  0x001C,   /* U+3326 */
  0x001C,   /* U+3327 */
  0x001C,   /* U+3328 */
  0x001C,   /* U+3329 */
  0x001C,   /* U+332A */
  0x001C,   /* U+332B */
  0x001C,   /* U+332C */
  0x001C,   /* U+332D */
  0x001C,   /* U+332E */
  0x001C,   /* U+332F */
  0x001C,   /* U+3330 */
  0x001C,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x001C,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x001C,   /* U+3335 */
  0x001C,   /* U+3336 */
  0x001C,   /* U+3337 */
  0x001C,   /* U+3338 */
  0x001C,   /* U+3339 */
  0x001C,   /* U+333A */
  0x001C,   /* U+333B */
  0x001C,   /* U+333C */
  0x001C,   /* U+333D */
  0x001C,   /* U+333E */
  0x001C,   /* U+333F */
  0x001C,   /* U+3340 */
  0x001C,   /* U+3341 */
  0x001C,   /* U+3342 */
  0x001C,   /* U+3343 */
  0x001C,   /* U+3344 */
  0x001C,   /* U+3345 */
  0x001C,   /* U+3346 */
  0x001C,   /* U+3347 */
  0x001C,   /* U+3348 */
  0x001C,   /* U+3349 */
  0x001C,   /* U+334A */
  0x001C,   /* U+334B */
  0x001C,   /* U+334C */
  0x001C,   /* U+334D */
  0x001C,   /* U+334E */
  0x001C,   /* U+334F */
  0x001C,   /* U+3350 */
  0x001C,   /* U+3351 */
  0x001C,   /* U+3352 */
  0x001C,   /* U+3353 */
  0x001C,   /* U+3354 */
  0x001C,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x001C,   /* U+3357 */
  0x0004,   /* U+3358 */
  0x0004,   /* U+3359 */
  0x0004,   /* U+335A */
  0x0004,   /* U+335B */
  0x0004,   /* U+335C */
  0x0004,   /* U+335D */
  0x0004,   /* U+335E */
  0x0004,   /* U+335F */
  0x0004,   /* U+3360 */
  0x0004,   /* U+3361 */
  0x0004,   /* U+3362 */
  0x0004,   /* U+3363 */
  0x0004,   /* U+3364 */
  0x0004,   /* U+3365 */
  0x0004,   /* U+3366 */
  0x0004,   /* U+3367 */
  0x0004,   /* U+3368 */
  0x0004,   /* U+3369 */
  0x0004,   /* U+336A */
  0x0004,   /* U+336B */
  0x0004,   /* U+336C */
  0x0004,   /* U+336D */
  0x0004,   /* U+336E */
  0x0004,   /* U+336F */
  0x0004,   /* U+3370 */
  0x001C,   /* U+3371 */
  0x001C,   /* U+3372 */
  0x001D,   /* U+3373 */
  0x001C,   /* U+3374 */
  0x001C,   /* U+3375 */
  0x001C,   /* U+3376 */
  0x001C,   /* U+3377 */
  0x001C,   /* U+3378 */
  0x001C,   /* U+3379 */
  0x001D,   /* U+337A */
  0x001C,   /* U+337B */
  0x001C,   /* U+337C */
  0x001C,   /* U+337D */
  0x001C,   /* U+337E */
  0x001C,   /* U+337F */
  0x001C,   /* U+3380 */
  0x001C,   /* U+3381 */
  0x001C,   /* U+3382 */
  0x001C,   /* U+3383 */
  0x001C,   /* U+3384 */
  0x001D,   /* U+3385 */
  0x001D,   /* U+3386 */
  0x001D,   /* U+3387 */
  0x001C,   /* U+3388 */
  0x001C,   /* U+3389 */
  0x001C,   /* U+338A */
  0x001C,   /* U+338B */
  0x001C,   /* U+338C */
  0x001C,   /* U+338D */
  0x001C,   /* U+338E */
  0x001C,   /* U+338F */
  0x001D,   /* U+3390 */
  0x001C,   /* U+3391 */
  0x001D,   /* U+3392 */
  0x001D,   /* U+3393 */
  0x001D,   /* U+3394 */
  0x001C,   /* U+3395 */
  0x001C,   /* U+3396 */
  0x001C,   /* U+3397 */
  0x001C,   /* U+3398 */
  0x001C,   /* U+3399 */
  0x001C,   /* U+339A */
  0x001C,   /* U+339B */
  0x001C,   /* U+339C */
  0x001C,   /* U+339D */
  0x001C,   /* U+339E */
  0x001C,   /* U+339F */
  0x001C,   /* U+33A0 */
  0x001C,   /* U+33A1 */
  0x001C,   /* U+33A2 */
  0x001C,   /* U+33A3 */
  0x001C,   /* U+33A4 */
  0x001C,   /* U+33A5 */
  0x001C,   /* U+33A6 */
  0x001C,   /* U+33A7 */
  0x001C,   /* U+33A8 */
  0x001D,   /* U+33A9 */
  0x001C,   /* U+33AA */
  0x001D,   /* U+33AB */
  0x001D,   /* U+33AC */
  0x001C,   /* U+33AD */
  0x001C,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x001C,   /* U+33B0 */
  0x001C,   /* U+33B1 */
  0x001C,   /* U+33B2 */
  0x001C,   /* U+33B3 */
  0x001C,   /* U+33B4 */
  0x001C,   /* U+33B5 */
  0x001C,   /* U+33B6 */
  0x001C,   /* U+33B7 */
  0x001C,   /* U+33B8 */
  0x001D,   /* U+33B9 */
  0x001C,   /* U+33BA */
  0x001C,   /* U+33BB */
  0x001C,   /* U+33BC */
  0x001C,   /* U+33BD */
  0x001C,   /* U+33BE */
  0x001D,   /* U+33BF */
  0x001C,   /* U+33C0 */
  0x001D,   /* U+33C1 */
  0x001C,   /* U+33C2 */
  0x001D,   /* U+33C3 */
  0x001C,   /* U+33C4 */
  0x001C,   /* U+33C5 */
  0x001D,   /* U+33C6 */
  0x001D,   /* U+33C7 */
  0x001C,   /* U+33C8 */
  0x001D,   /* U+33C9 */
  0x001C,   /* U+33CA */
  0x001D,   /* U+33CB */
  0x001C,   /* U+33CC */
  0x001D,   /* U+33CD */
  0x001D,   /* U+33CE */
  0x001C,   /* U+33CF */
  0x001C,   /* U+33D0 */
  0x001C,   /* U+33D1 */
  0x001C,   /* U+33D2 */
  0x001C,   /* U+33D3 */
  0x001C,   /* U+33D4 */
  0x001C,   /* U+33D5 */
  0x001C,   /* U+33D6 */
  0x001D,   /* U+33D7 */
  0x001C,   /* U+33D8 */
  0x001D,   /* U+33D9 */
  0x001D,   /* U+33DA */
  0x001C,   /* U+33DB */
  0x001D,   /* U+33DC */
  0x001D,   /* U+33DD */
  0x001D,   /* U+33DE */
  0x001D,   /* U+33DF */
  0x0004,   /* U+33E0 */
  0x0004,   /* U+33E1 */
  0x0004,   /* U+33E2 */
  0x0004,   /* U+33E3 */
  0x0004,   /* U+33E4 */
  0x0004,   /* U+33E5 */
  0x0004,   /* U+33E6 */
  0x0004,   /* U+33E7 */
  0x0004,   /* U+33E8 */
  0x0004,   /* U+33E9 */
  0x0004,   /* U+33EA */
  0x0004,   /* U+33EB */
  0x0004,   /* U+33EC */
  0x0004,   /* U+33ED */
  0x0004,   /* U+33EE */
  0x0004,   /* U+33EF */
  0x0004,   /* U+33F0 */
  0x0004,   /* U+33F1 */
  0x0004,   /* U+33F2 */
  0x0004,   /* U+33F3 */
  0x0004,   /* U+33F4 */
  0x0004,   /* U+33F5 */
  0x0004,   /* U+33F6 */
  0x0004,   /* U+33F7 */
  0x0004,   /* U+33F8 */
  0x0004,   /* U+33F9 */
  0x0004,   /* U+33FA */
  0x0004,   /* U+33FB */
  0x0004,   /* U+33FC */
  0x0004,   /* U+33FD */
  0x0004,   /* U+33FE */
  0x001C,   /* U+33FF */

  /* Primary weight 2 for each character. */
  0x3D74,   /* U+3300 */
  0x3D84,   /* U+3301 */
  0x3D8B,   /* U+3302 */
  0x1C0E,   /* U+3303 */
  0x3D70,   /* U+3304 */
  0x3D8B,   /* U+3305 */
  0x3D5F,   /* U+3306 */
  0x3D67,   /* U+3307 */
  0x1C0E,   /* U+3308 */
  0x3D8B,   /* U+3309 */
  0x1C0E,   /* U+330A */
  0x3D5B,   /* U+330B */
  0x3D82,   /* U+330C */
  0x3D86,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x3D7F,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x3D86,   /* U+3314 */
  0x3D86,   /* U+3315 */
  0x3D86,   /* U+3316 */
  0x3D86,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x3D84,   /* U+331A */
  0x3D86,   /* U+331B */
  0x1C0E,   /* U+331C */
  0x3D84,   /* U+331D */
  0x1C0E,   /* U+331E */
  0x3D5B,   /* U+331F */
  0x3D8B,   /* U+3320 */
  0x3D83,   /* U+3321 */
  0x3D8B,   /* U+3322 */
  0x3D8B,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x3D8B,   /* U+3327 */
  0x3D73,   /* U+3328 */
  0x3D6C,   /* U+3329 */
  0x3D5B,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x3D5A,   /* U+3332 */
  0x3D5B,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x3D82,   /* U+3335 */
  0x3D62,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x3D84,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x3D8B,   /* U+333F */
  0x0000,   /* U+3340 */
  0x1C0E,   /* U+3341 */
  0x1C0E,   /* U+3342 */
  0x3D5B,   /* U+3343 */
  0x3D5B,   /* U+3344 */
  0x3D6C,   /* U+3345 */
  0x3D84,   /* U+3346 */
  0x3D8B,   /* U+3347 */
  0x3D62,   /* U+3348 */
  0x3D83,   /* U+3349 */
  0x3D83,   /* U+334A */
  0x3D60,   /* U+334B */
  0x3D60,   /* U+334C */
  0x1C0E,   /* U+334D */
  0x1C0E,   /* U+334E */
  0x1C0E,   /* U+334F */
  0x3D5A,   /* U+3350 */
  0x3D6C,   /* U+3351 */
  0x3D82,   /* U+3352 */
  0x3D75,   /* U+3353 */
  0x1C0E,   /* U+3354 */
  0x3D7B,   /* U+3355 */
  0x3D8B,   /* U+3356 */
  0x3D6C,   /* U+3357 */
  0xFB40,   /* U+3358 */
  0xFB40,   /* U+3359 */
  0xFB40,   /* U+335A */
  0xFB40,   /* U+335B */
  0xFB40,   /* U+335C */
  0xFB40,   /* U+335D */
  0xFB40,   /* U+335E */
  0xFB40,   /* U+335F */
  0xFB40,   /* U+3360 */
  0xFB40,   /* U+3361 */
  0x1C3D,   /* U+3362 */
  0x1C3E,   /* U+3363 */
  0x1C3F,   /* U+3364 */
  0x1C40,   /* U+3365 */
  0x1C41,   /* U+3366 */
  0x1C42,   /* U+3367 */
  0x1C43,   /* U+3368 */
  0x1C44,   /* U+3369 */
  0x1C45,   /* U+336A */
  0x1C46,   /* U+336B */
  0x1C3D,   /* U+336C */
  0x1C3E,   /* U+336D */
  0x1C3F,   /* U+336E */
  0x1C40,   /* U+336F */
  0x1C41,   /* U+3370 */
  0x1E0C,   /* U+3371 */
  0x1C47,   /* U+3372 */
  0x1EB5,   /* U+3373 */
  0x1C47,   /* U+3374 */
  0x1EE3,   /* U+3375 */
  0x1C7A,   /* U+3376 */
  0x1DAA,   /* U+3377 */
  0x1DAA,   /* U+3378 */
  0x1DAA,   /* U+3379 */
  0x1EB5,   /* U+337A */
  0xDE73,   /* U+337B */
  0xE62D,   /* U+337C */
  0xD927,   /* U+337D */
  0xE60E,   /* U+337E */
  0xE82A,   /* U+337F */
  0x1C47,   /* U+3380 */
  0x1C47,   /* U+3381 */
  0x1C47,   /* U+3382 */
  0x1C47,   /* U+3383 */
  0x1C47,   /* U+3384 */
  0x1C60,   /* U+3385 */
  0x1C60,   /* U+3386 */
  0x1C60,   /* U+3387 */
  0x1C47,   /* U+3388 */
  0x1C7A,   /* U+3389 */
  0x1CE5,   /* U+338A */
  0x1CE5,   /* U+338B */
  0x1CE5,   /* U+338C */
  0x1CF4,   /* U+338D */
  0x1CF4,   /* U+338E */
  0x1CF4,   /* U+338F */
  0x1F21,   /* U+3390 */
  0x1D18,   /* U+3391 */
  0x1D18,   /* U+3392 */
  0x1D18,   /* U+3393 */
  0x1D18,   /* U+3394 */
  0x1D77,   /* U+3395 */
  0x1D77,   /* U+3396 */
  0x1D77,   /* U+3397 */
  0x1D77,   /* U+3398 */
  0x1DAA,   /* U+3399 */
  0x1DAA,   /* U+339A */
  0x1DAA,   /* U+339B */
  0x1DAA,   /* U+339C */
  0x1DAA,   /* U+339D */
  0x1DAA,   /* U+339E */
  0x1DAA,   /* U+339F */
  0x1DAA,   /* U+33A0 */
  0x1C3F,   /* U+33A1 */
  0x1DAA,   /* U+33A2 */
  0x1DAA,   /* U+33A3 */
  0x1DAA,   /* U+33A4 */
  0x1C40,   /* U+33A5 */
  0x1DAA,   /* U+33A6 */
  0x0625,   /* U+33A7 */
  0x0625,   /* U+33A8 */
  0x1C47,   /* U+33A9 */
  0x1E0C,   /* U+33AA */
  0x1E0C,   /* U+33AB */
  0x1E0C,   /* U+33AC */
  0x1C47,   /* U+33AD */
  0x1C47,   /* U+33AE */
  0x1C47,   /* U+33AF */
  0x1E71,   /* U+33B0 */
  0x1E71,   /* U+33B1 */
  0x1E71,   /* U+33B2 */
  0x1E71,   /* U+33B3 */
  0x1EE3,   /* U+33B4 */
  0x1EE3,   /* U+33B5 */
  0x1EE3,   /* U+33B6 */
  0x1EE3,   /* U+33B7 */
  0x1EE3,   /* U+33B8 */
  0x1EE3,   /* U+33B9 */
  0x1EF5,   /* U+33BA */
  0x1EF5,   /* U+33BB */
  0x1EF5,   /* U+33BC */
  0x1EF5,   /* U+33BD */
  0x1EF5,   /* U+33BE */
  0x1EF5,   /* U+33BF */
  0x1FE1,   /* U+33C0 */
  0x1FE1,   /* U+33C1 */
  0x0277,   /* U+33C2 */
  0x1E21,   /* U+33C3 */
  0x1C7A,   /* U+33C4 */
  0x1C8F,   /* U+33C5 */
  0x0625,   /* U+33C6 */
  0x1DDD,   /* U+33C7 */
  0x1C60,   /* U+33C8 */
  0x1F0B,   /* U+33C9 */
  0x1C47,   /* U+33CA */
  0x1E0C,   /* U+33CB */
  0x1DB9,   /* U+33CC */
  0x1D65,   /* U+33CD */
  0x1DAA,   /* U+33CE */
  0x1E95,   /* U+33CF */
  0x1DAA,   /* U+33D0 */
  0x1DB9,   /* U+33D1 */
  0x1DDD,   /* U+33D2 */
  0x1EFF,   /* U+33D3 */
  0x1C60,   /* U+33D4 */
  0x1D32,   /* U+33D5 */
  0x1DDD,   /* U+33D6 */
  0x1D18,   /* U+33D7 */
  0x0277,   /* U+33D8 */
  0x1E0C,   /* U+33D9 */
  0x1E33,   /* U+33DA */
  0x1E33,   /* U+33DB */
  0x1EE3,   /* U+33DC */
  0x1C60,   /* U+33DD */
  0x0625,   /* U+33DE */
  0x0625,   /* U+33DF */
  0xFB40,   /* U+33E0 */
  0xFB40,   /* U+33E1 */
  0xFB40,   /* U+33E2 */
  0xFB40,   /* U+33E3 */
  0xFB40,   /* U+33E4 */
  0xFB40,   /* U+33E5 */
  0xFB40,   /* U+33E6 */
  0xFB40,   /* U+33E7 */
  0xFB40,   /* U+33E8 */
  0x1C3D,   /* U+33E9 */
  0x1C3E,   /* U+33EA */
  0x1C3F,   /* U+33EB */
  0x1C40,   /* U+33EC */
  0x1C41,   /* U+33ED */
  0x1C42,   /* U+33EE */
  0x1C43,   /* U+33EF */
  0x1C44,   /* U+33F0 */
  0x1C45,   /* U+33F1 */
  0x1C46,   /* U+33F2 */
  0x1C3D,   /* U+33F3 */
  0x1C3E,   /* U+33F4 */
  0x1C3F,   /* U+33F5 */
  0x1C40,   /* U+33F6 */
  0x1C41,   /* U+33F7 */
  0x1C42,   /* U+33F8 */
  0x1C43,   /* U+33F9 */
  0x1C44,   /* U+33FA */
  0x1C45,   /* U+33FB */
  0x1C46,   /* U+33FC */
  0x1C3D,   /* U+33FD */
  0x1C3E,   /* U+33FE */
  0x1C47,   /* U+33FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+3300 */
  0x0020,   /* U+3301 */
  0x0020,   /* U+3302 */
  0x0020,   /* U+3303 */
  0x0020,   /* U+3304 */
  0x0020,   /* U+3305 */
  0x0020,   /* U+3306 */
  0x0020,   /* U+3307 */
  0x0020,   /* U+3308 */
  0x0020,   /* U+3309 */
  0x0020,   /* U+330A */
  0x0020,   /* U+330B */
  0x0020,   /* U+330C */
  0x0020,   /* U+330D */
  0x0037,   /* U+330E */
  0x0037,   /* U+330F */
  0x0037,   /* U+3310 */
  0x0037,   /* U+3311 */
  0x0020,   /* U+3312 */
  0x0037,   /* U+3313 */
  0x0020,   /* U+3314 */
  0x0020,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0020,   /* U+3317 */
  0x0037,   /* U+3318 */
  0x0037,   /* U+3319 */
  0x0020,   /* U+331A */
  0x0020,   /* U+331B */
  0x0020,   /* U+331C */
  0x0020,   /* U+331D */
  0x0020,   /* U+331E */
  0x0020,   /* U+331F */
  0x0020,   /* U+3320 */
  0x0020,   /* U+3321 */
  0x0020,   /* U+3322 */
  0x0020,   /* U+3323 */
  0x0037,   /* U+3324 */
  0x0037,   /* U+3325 */
  0x0037,   /* U+3326 */
  0x0020,   /* U+3327 */
  0x0020,   /* U+3328 */
  0x0020,   /* U+3329 */
  0x0020,   /* U+332A */
  0x0038,   /* U+332B */
  0x0038,   /* U+332C */
  0x0037,   /* U+332D */
  0x0038,   /* U+332E */
  0x0038,   /* U+332F */
  0x0038,   /* U+3330 */
  0x0037,   /* U+3331 */
  0x0020,   /* U+3332 */
  0x0020,   /* U+3333 */
  0x0037,   /* U+3334 */
  0x0020,   /* U+3335 */
  0x0020,   /* U+3336 */
  0x0038,   /* U+3337 */
  0x0038,   /* U+3338 */
  0x0020,   /* U+3339 */
  0x0038,   /* U+333A */
  0x0038,   /* U+333B */
  0x0037,   /* U+333C */
  0x0038,   /* U+333D */
  0x0037,   /* U+333E */
  0x0020,   /* U+333F */
  0x0038,   /* U+3340 */
  0x0020,   /* U+3341 */
  0x0020,   /* U+3342 */
  0x0020,   /* U+3343 */
  0x0020,   /* U+3344 */
  0x0020,   /* U+3345 */
  0x0020,   /* U+3346 */
  0x0020,   /* U+3347 */
  0x0020,   /* U+3348 */
  0x0020,   /* U+3349 */
  0x0020,   /* U+334A */
  0x0020,   /* U+334B */
  0x0020,   /* U+334C */
  0x0020,   /* U+334D */
  0x0020,   /* U+334E */
  0x0020,   /* U+334F */
  0x0020,   /* U+3350 */
  0x0020,   /* U+3351 */
  0x0020,   /* U+3352 */
  0x0020,   /* U+3353 */
  0x0020,   /* U+3354 */
  0x0020,   /* U+3355 */
  0x0020,   /* U+3356 */
  0x0020,   /* U+3357 */
  0x0020,   /* U+3358 */
  0x0020,   /* U+3359 */
  0x0020,   /* U+335A */
  0x0020,   /* U+335B */
  0x0020,   /* U+335C */
  0x0020,   /* U+335D */
  0x0020,   /* U+335E */
  0x0020,   /* U+335F */
  0x0020,   /* U+3360 */
  0x0020,   /* U+3361 */
  0x0020,   /* U+3362 */
  0x0020,   /* U+3363 */
  0x0020,   /* U+3364 */
  0x0020,   /* U+3365 */
  0x0020,   /* U+3366 */
  0x0020,   /* U+3367 */
  0x0020,   /* U+3368 */
  0x0020,   /* U+3369 */
  0x0020,   /* U+336A */
  0x0020,   /* U+336B */
  0x0020,   /* U+336C */
  0x0020,   /* U+336D */
  0x0020,   /* U+336E */
  0x0020,   /* U+336F */
  0x0020,   /* U+3370 */
  0x0020,   /* U+3371 */
  0x0020,   /* U+3372 */
  0x0020,   /* U+3373 */
  0x0020,   /* U+3374 */
  0x0020,   /* U+3375 */
  0x0020,   /* U+3376 */
  0x0020,   /* U+3377 */
  0x0020,   /* U+3378 */
  0x0020,   /* U+3379 */
  0x0020,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0020,   /* U+3380 */
  0x0020,   /* U+3381 */
  0x0020,   /* U+3382 */
  0x0020,   /* U+3383 */
  0x0020,   /* U+3384 */
  0x0020,   /* U+3385 */
  0x0020,   /* U+3386 */
  0x0020,   /* U+3387 */
  0x0020,   /* U+3388 */
  0x0020,   /* U+3389 */
  0x0020,   /* U+338A */
  0x0020,   /* U+338B */
  0x0020,   /* U+338C */
  0x0020,   /* U+338D */
  0x0020,   /* U+338E */
  0x0020,   /* U+338F */
  0x0020,   /* U+3390 */
  0x0020,   /* U+3391 */
  0x0020,   /* U+3392 */
  0x0020,   /* U+3393 */
  0x0020,   /* U+3394 */
  0x0020,   /* U+3395 */
  0x0020,   /* U+3396 */
  0x0020,   /* U+3397 */
  0x0020,   /* U+3398 */
  0x0020,   /* U+3399 */
  0x0020,   /* U+339A */
  0x0020,   /* U+339B */
  0x0020,   /* U+339C */
  0x0020,   /* U+339D */
  0x0020,   /* U+339E */
  0x0020,   /* U+339F */
  0x0020,   /* U+33A0 */
  0x0020,   /* U+33A1 */
  0x0020,   /* U+33A2 */
  0x0020,   /* U+33A3 */
  0x0020,   /* U+33A4 */
  0x0020,   /* U+33A5 */
  0x0020,   /* U+33A6 */
  0x0020,   /* U+33A7 */
  0x0020,   /* U+33A8 */
  0x0020,   /* U+33A9 */
  0x0020,   /* U+33AA */
  0x0020,   /* U+33AB */
  0x0020,   /* U+33AC */
  0x0020,   /* U+33AD */
  0x0020,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0020,   /* U+33B0 */
  0x0020,   /* U+33B1 */
  0x0020,   /* U+33B2 */
  0x0020,   /* U+33B3 */
  0x0020,   /* U+33B4 */
  0x0020,   /* U+33B5 */
  0x0020,   /* U+33B6 */
  0x0020,   /* U+33B7 */
  0x0020,   /* U+33B8 */
  0x0020,   /* U+33B9 */
  0x0020,   /* U+33BA */
  0x0020,   /* U+33BB */
  0x0020,   /* U+33BC */
  0x0020,   /* U+33BD */
  0x0020,   /* U+33BE */
  0x0020,   /* U+33BF */
  0x0020,   /* U+33C0 */
  0x0020,   /* U+33C1 */
  0x0020,   /* U+33C2 */
  0x0020,   /* U+33C3 */
  0x0020,   /* U+33C4 */
  0x0020,   /* U+33C5 */
  0x0020,   /* U+33C6 */
  0x0020,   /* U+33C7 */
  0x0020,   /* U+33C8 */
  0x0020,   /* U+33C9 */
  0x0020,   /* U+33CA */
  0x0020,   /* U+33CB */
  0x0020,   /* U+33CC */
  0x0020,   /* U+33CD */
  0x0020,   /* U+33CE */
  0x0020,   /* U+33CF */
  0x0020,   /* U+33D0 */
  0x0020,   /* U+33D1 */
  0x0020,   /* U+33D2 */
  0x0020,   /* U+33D3 */
  0x0020,   /* U+33D4 */
  0x0020,   /* U+33D5 */
  0x0020,   /* U+33D6 */
  0x0020,   /* U+33D7 */
  0x0020,   /* U+33D8 */
  0x0020,   /* U+33D9 */
  0x0020,   /* U+33DA */
  0x0020,   /* U+33DB */
  0x0020,   /* U+33DC */
  0x0020,   /* U+33DD */
  0x0020,   /* U+33DE */
  0x0020,   /* U+33DF */
  0x0020,   /* U+33E0 */
  0x0020,   /* U+33E1 */
  0x0020,   /* U+33E2 */
  0x0020,   /* U+33E3 */
  0x0020,   /* U+33E4 */
  0x0020,   /* U+33E5 */
  0x0020,   /* U+33E6 */
  0x0020,   /* U+33E7 */
  0x0020,   /* U+33E8 */
  0x0020,   /* U+33E9 */
  0x0020,   /* U+33EA */
  0x0020,   /* U+33EB */
  0x0020,   /* U+33EC */
  0x0020,   /* U+33ED */
  0x0020,   /* U+33EE */
  0x0020,   /* U+33EF */
  0x0020,   /* U+33F0 */
  0x0020,   /* U+33F1 */
  0x0020,   /* U+33F2 */
  0x0020,   /* U+33F3 */
  0x0020,   /* U+33F4 */
  0x0020,   /* U+33F5 */
  0x0020,   /* U+33F6 */
  0x0020,   /* U+33F7 */
  0x0020,   /* U+33F8 */
  0x0020,   /* U+33F9 */
  0x0020,   /* U+33FA */
  0x0020,   /* U+33FB */
  0x0020,   /* U+33FC */
  0x0020,   /* U+33FD */
  0x0020,   /* U+33FE */
  0x0020,   /* U+33FF */

  /* Tertiary weight 2 for each character. */
  0x001C,   /* U+3300 */
  0x001C,   /* U+3301 */
  0x001C,   /* U+3302 */
  0x001C,   /* U+3303 */
  0x001C,   /* U+3304 */
  0x001C,   /* U+3305 */
  0x001C,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x001C,   /* U+3308 */
  0x001C,   /* U+3309 */
  0x001C,   /* U+330A */
  0x001C,   /* U+330B */
  0x001C,   /* U+330C */
  0x001C,   /* U+330D */
  0x001C,   /* U+330E */
  0x001C,   /* U+330F */
  0x001C,   /* U+3310 */
  0x001C,   /* U+3311 */
  0x001C,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x001C,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x001C,   /* U+3317 */
  0x001C,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x001C,   /* U+331B */
  0x001C,   /* U+331C */
  0x001C,   /* U+331D */
  0x001C,   /* U+331E */
  0x001C,   /* U+331F */
  0x001C,   /* U+3320 */
  0x001C,   /* U+3321 */
  0x001C,   /* U+3322 */
  0x001C,   /* U+3323 */
  0x001C,   /* U+3324 */
  0x001C,   /* U+3325 */
  0x001C,   /* U+3326 */
  0x001C,   /* U+3327 */
  0x001C,   /* U+3328 */
  0x001C,   /* U+3329 */
  0x001C,   /* U+332A */
  0x001C,   /* U+332B */
  0x001C,   /* U+332C */
  0x001C,   /* U+332D */
  0x001C,   /* U+332E */
  0x001C,   /* U+332F */
  0x001C,   /* U+3330 */
  0x001C,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x001C,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x001C,   /* U+3335 */
  0x001C,   /* U+3336 */
  0x001C,   /* U+3337 */
  0x001C,   /* U+3338 */
  0x001C,   /* U+3339 */
  0x001C,   /* U+333A */
  0x001C,   /* U+333B */
  0x001C,   /* U+333C */
  0x001C,   /* U+333D */
  0x001C,   /* U+333E */
  0x001C,   /* U+333F */
  0x001C,   /* U+3340 */
  0x001C,   /* U+3341 */
  0x001C,   /* U+3342 */
  0x001C,   /* U+3343 */
  0x001C,   /* U+3344 */
  0x001C,   /* U+3345 */
  0x001C,   /* U+3346 */
  0x001C,   /* U+3347 */
  0x001C,   /* U+3348 */
  0x001C,   /* U+3349 */
  0x001C,   /* U+334A */
  0x001C,   /* U+334B */
  0x001C,   /* U+334C */
  0x001C,   /* U+334D */
  0x001C,   /* U+334E */
  0x001C,   /* U+334F */
  0x001C,   /* U+3350 */
  0x001C,   /* U+3351 */
  0x001C,   /* U+3352 */
  0x001C,   /* U+3353 */
  0x001C,   /* U+3354 */
  0x001C,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x001C,   /* U+3357 */
  0x0004,   /* U+3358 */
  0x0004,   /* U+3359 */
  0x0004,   /* U+335A */
  0x0004,   /* U+335B */
  0x0004,   /* U+335C */
  0x0004,   /* U+335D */
  0x0004,   /* U+335E */
  0x0004,   /* U+335F */
  0x0004,   /* U+3360 */
  0x0004,   /* U+3361 */
  0x0004,   /* U+3362 */
  0x0004,   /* U+3363 */
  0x0004,   /* U+3364 */
  0x0004,   /* U+3365 */
  0x0004,   /* U+3366 */
  0x0004,   /* U+3367 */
  0x0004,   /* U+3368 */
  0x0004,   /* U+3369 */
  0x0004,   /* U+336A */
  0x0004,   /* U+336B */
  0x0004,   /* U+336C */
  0x0004,   /* U+336D */
  0x0004,   /* U+336E */
  0x0004,   /* U+336F */
  0x0004,   /* U+3370 */
  0x001D,   /* U+3371 */
  0x001C,   /* U+3372 */
  0x001D,   /* U+3373 */
  0x001C,   /* U+3374 */
  0x001D,   /* U+3375 */
  0x001C,   /* U+3376 */
  0x001C,   /* U+3377 */
  0x001C,   /* U+3378 */
  0x001C,   /* U+3379 */
  0x001D,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x001D,   /* U+3380 */
  0x001D,   /* U+3381 */
  0x001D,   /* U+3382 */
  0x001D,   /* U+3383 */
  0x001D,   /* U+3384 */
  0x001D,   /* U+3385 */
  0x001D,   /* U+3386 */
  0x001D,   /* U+3387 */
  0x001C,   /* U+3388 */
  0x001C,   /* U+3389 */
  0x001D,   /* U+338A */
  0x001D,   /* U+338B */
  0x001D,   /* U+338C */
  0x001C,   /* U+338D */
  0x001C,   /* U+338E */
  0x001C,   /* U+338F */
  0x001C,   /* U+3390 */
  0x001D,   /* U+3391 */
  0x001D,   /* U+3392 */
  0x001D,   /* U+3393 */
  0x001D,   /* U+3394 */
  0x001C,   /* U+3395 */
  0x001C,   /* U+3396 */
  0x001C,   /* U+3397 */
  0x001C,   /* U+3398 */
  0x001C,   /* U+3399 */
  0x001C,   /* U+339A */
  0x001C,   /* U+339B */
  0x001C,   /* U+339C */
  0x001C,   /* U+339D */
  0x001C,   /* U+339E */
  0x001C,   /* U+339F */
  0x001C,   /* U+33A0 */
  0x001C,   /* U+33A1 */
  0x001C,   /* U+33A2 */
  0x001C,   /* U+33A3 */
  0x001C,   /* U+33A4 */
  0x001C,   /* U+33A5 */
  0x001C,   /* U+33A6 */
  0x001C,   /* U+33A7 */
  0x001C,   /* U+33A8 */
  0x001C,   /* U+33A9 */
  0x001D,   /* U+33AA */
  0x001D,   /* U+33AB */
  0x001D,   /* U+33AC */
  0x001C,   /* U+33AD */
  0x001C,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x001C,   /* U+33B0 */
  0x001C,   /* U+33B1 */
  0x001C,   /* U+33B2 */
  0x001C,   /* U+33B3 */
  0x001D,   /* U+33B4 */
  0x001D,   /* U+33B5 */
  0x001D,   /* U+33B6 */
  0x001D,   /* U+33B7 */
  0x001D,   /* U+33B8 */
  0x001D,   /* U+33B9 */
  0x001D,   /* U+33BA */
  0x001D,   /* U+33BB */
  0x001D,   /* U+33BC */
  0x001D,   /* U+33BD */
  0x001D,   /* U+33BE */
  0x001D,   /* U+33BF */
  0x001D,   /* U+33C0 */
  0x001D,   /* U+33C1 */
  0x001C,   /* U+33C2 */
  0x001C,   /* U+33C3 */
  0x001C,   /* U+33C4 */
  0x001C,   /* U+33C5 */
  0x001C,   /* U+33C6 */
  0x001C,   /* U+33C7 */
  0x001D,   /* U+33C8 */
  0x001C,   /* U+33C9 */
  0x001C,   /* U+33CA */
  0x001D,   /* U+33CB */
  0x001C,   /* U+33CC */
  0x001D,   /* U+33CD */
  0x001D,   /* U+33CE */
  0x001C,   /* U+33CF */
  0x001C,   /* U+33D0 */
  0x001C,   /* U+33D1 */
  0x001C,   /* U+33D2 */
  0x001C,   /* U+33D3 */
  0x001C,   /* U+33D4 */
  0x001C,   /* U+33D5 */
  0x001C,   /* U+33D6 */
  0x001D,   /* U+33D7 */
  0x001C,   /* U+33D8 */
  0x001D,   /* U+33D9 */
  0x001D,   /* U+33DA */
  0x001C,   /* U+33DB */
  0x001C,   /* U+33DC */
  0x001C,   /* U+33DD */
  0x001C,   /* U+33DE */
  0x001C,   /* U+33DF */
  0x0004,   /* U+33E0 */
  0x0004,   /* U+33E1 */
  0x0004,   /* U+33E2 */
  0x0004,   /* U+33E3 */
  0x0004,   /* U+33E4 */
  0x0004,   /* U+33E5 */
  0x0004,   /* U+33E6 */
  0x0004,   /* U+33E7 */
  0x0004,   /* U+33E8 */
  0x0004,   /* U+33E9 */
  0x0004,   /* U+33EA */
  0x0004,   /* U+33EB */
  0x0004,   /* U+33EC */
  0x0004,   /* U+33ED */
  0x0004,   /* U+33EE */
  0x0004,   /* U+33EF */
  0x0004,   /* U+33F0 */
  0x0004,   /* U+33F1 */
  0x0004,   /* U+33F2 */
  0x0004,   /* U+33F3 */
  0x0004,   /* U+33F4 */
  0x0004,   /* U+33F5 */
  0x0004,   /* U+33F6 */
  0x0004,   /* U+33F7 */
  0x0004,   /* U+33F8 */
  0x0004,   /* U+33F9 */
  0x0004,   /* U+33FA */
  0x0004,   /* U+33FB */
  0x0004,   /* U+33FC */
  0x0004,   /* U+33FD */
  0x0004,   /* U+33FE */
  0x001C,   /* U+33FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+3300 */
  0x3D76,   /* U+3301 */
  0x3D77,   /* U+3302 */
  0x3D84,   /* U+3303 */
  0x3D8B,   /* U+3304 */
  0x3D6B,   /* U+3305 */
  0x3D8B,   /* U+3306 */
  0x3D62,   /* U+3307 */
  0x3D60,   /* U+3308 */
  0x3D67,   /* U+3309 */
  0x3D7B,   /* U+330A */
  0x3D83,   /* U+330B */
  0x3D6C,   /* U+330C */
  0x3D83,   /* U+330D */
  0x3D86,   /* U+330E */
  0x3D8B,   /* U+330F */
  0x3D60,   /* U+3310 */
  0x3D70,   /* U+3311 */
  0x3D83,   /* U+3312 */
  0x3D84,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x3D62,   /* U+3315 */
  0x3D7C,   /* U+3316 */
  0x3D87,   /* U+3317 */
  0x3D82,   /* U+3318 */
  0x3D82,   /* U+3319 */
  0x3D68,   /* U+331A */
  0x1C0E,   /* U+331B */
  0x3D67,   /* U+331C */
  0x3D6F,   /* U+331D */
  0x3D78,   /* U+331E */
  0x3D62,   /* U+331F */
  0x3D6B,   /* U+3320 */
  0x3D8B,   /* U+3321 */
  0x3D6B,   /* U+3322 */
  0x3D6E,   /* U+3323 */
  0x1C0E,   /* U+3324 */
  0x3D66,   /* U+3325 */
  0x3D84,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x3D6E,   /* U+3329 */
  0x3D6C,   /* U+332A */
  0x1C0E,   /* U+332B */
  0x1C0E,   /* U+332C */
  0x1C0E,   /* U+332D */
  0x3D5A,   /* U+332E */
  0x3D62,   /* U+332F */
  0x3D64,   /* U+3330 */
  0x3D84,   /* U+3331 */
  0x3D82,   /* U+3332 */
  0x1C0E,   /* U+3333 */
  0x3D6C,   /* U+3334 */
  0x3D8B,   /* U+3335 */
  0x3D6A,   /* U+3336 */
  0x3D69,   /* U+3337 */
  0x3D70,   /* U+3338 */
  0x3D6C,   /* U+3339 */
  0x3D8B,   /* U+333A */
  0x1C0E,   /* U+333B */
  0x1C0E,   /* U+333C */
  0x3D5B,   /* U+333D */
  0x3D84,   /* U+333E */
  0x0000,   /* U+333F */
  0x3D8B,   /* U+3340 */
  0x3D84,   /* U+3341 */
  0x3D8B,   /* U+3342 */
  0x3D62,   /* U+3343 */
  0x3D84,   /* U+3344 */
  0x3D74,   /* U+3345 */
  0x3D62,   /* U+3346 */
  0x3D66,   /* U+3347 */
  0x3D86,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x3D74,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x3D6E,   /* U+334D */
  0x3D6E,   /* U+334E */
  0x3D84,   /* U+334F */
  0x3D8B,   /* U+3350 */
  0x3D6E,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x3D76,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x3D6E,   /* U+3356 */
  0x3D6E,   /* U+3357 */
  0xF0B9,   /* U+3358 */
  0xF0B9,   /* U+3359 */
  0xF0B9,   /* U+335A */
  0xF0B9,   /* U+335B */
  0xF0B9,   /* U+335C */
  0xF0B9,   /* U+335D */
  0xF0B9,   /* U+335E */
  0xF0B9,   /* U+335F */
  0xF0B9,   /* U+3360 */
  0xF0B9,   /* U+3361 */
  0xFB40,   /* U+3362 */
  0xFB40,   /* U+3363 */
  0xFB40,   /* U+3364 */
  0xFB40,   /* U+3365 */
  0xFB40,   /* U+3366 */
  0xFB40,   /* U+3367 */
  0xFB40,   /* U+3368 */
  0xFB40,   /* U+3369 */
  0xFB40,   /* U+336A */
  0xFB40,   /* U+336B */
  0xFB40,   /* U+336C */
  0xFB40,   /* U+336D */
  0xFB40,   /* U+336E */
  0xFB40,   /* U+336F */
  0xFB40,   /* U+3370 */
  0x1C47,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x1E33,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x1C3F,   /* U+3378 */
  0x1C40,   /* U+3379 */
  0x0000,   /* U+337A */
  0xFB40,   /* U+337B */
  0xFB40,   /* U+337C */
  0xFB40,   /* U+337D */
  0xFB40,   /* U+337E */
  0xFB40,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x1D77,   /* U+3388 */
  0x1C47,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x1F21,   /* U+3391 */
  0x1F21,   /* U+3392 */
  0x1F21,   /* U+3393 */
  0x1F21,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x1C3F,   /* U+339F */
  0x1C3F,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x1C3F,   /* U+33A2 */
  0x1C40,   /* U+33A3 */
  0x1C40,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x1C40,   /* U+33A6 */
  0x1E71,   /* U+33A7 */
  0x1E71,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x1C47,   /* U+33AA */
  0x1C47,   /* U+33AB */
  0x1C47,   /* U+33AC */
  0x1C8F,   /* U+33AD */
  0x1C8F,   /* U+33AE */
  0x1C8F,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x1DAA,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x1D65,   /* U+33C6 */
  0x0277,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x1CF4,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x1D77,   /* U+33D5 */
  0x1D77,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x1DAA,   /* U+33D8 */
  0x1DAA,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x1DAA,   /* U+33DE */
  0x1DAA,   /* U+33DF */
  0xE5E5,   /* U+33E0 */
  0xE5E5,   /* U+33E1 */
  0xE5E5,   /* U+33E2 */
  0xE5E5,   /* U+33E3 */
  0xE5E5,   /* U+33E4 */
  0xE5E5,   /* U+33E5 */
  0xE5E5,   /* U+33E6 */
  0xE5E5,   /* U+33E7 */
  0xE5E5,   /* U+33E8 */
  0xFB40,   /* U+33E9 */
  0xFB40,   /* U+33EA */
  0xFB40,   /* U+33EB */
  0xFB40,   /* U+33EC */
  0xFB40,   /* U+33ED */
  0xFB40,   /* U+33EE */
  0xFB40,   /* U+33EF */
  0xFB40,   /* U+33F0 */
  0xFB40,   /* U+33F1 */
  0xFB40,   /* U+33F2 */
  0xFB40,   /* U+33F3 */
  0xFB40,   /* U+33F4 */
  0xFB40,   /* U+33F5 */
  0xFB40,   /* U+33F6 */
  0xFB40,   /* U+33F7 */
  0xFB40,   /* U+33F8 */
  0xFB40,   /* U+33F9 */
  0xFB40,   /* U+33FA */
  0xFB40,   /* U+33FB */
  0xFB40,   /* U+33FC */
  0xFB40,   /* U+33FD */
  0xFB40,   /* U+33FE */
  0x1D77,   /* U+33FF */

  /* Secondary weight 3 for each character. */
  0x0038,   /* U+3300 */
  0x0020,   /* U+3301 */
  0x0020,   /* U+3302 */
  0x0020,   /* U+3303 */
  0x0020,   /* U+3304 */
  0x0020,   /* U+3305 */
  0x0020,   /* U+3306 */
  0x0020,   /* U+3307 */
  0x0020,   /* U+3308 */
  0x0020,   /* U+3309 */
  0x0020,   /* U+330A */
  0x0020,   /* U+330B */
  0x0020,   /* U+330C */
  0x0020,   /* U+330D */
  0x0020,   /* U+330E */
  0x0020,   /* U+330F */
  0x0020,   /* U+3310 */
  0x0020,   /* U+3311 */
  0x0020,   /* U+3312 */
  0x0020,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0020,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0020,   /* U+3317 */
  0x0020,   /* U+3318 */
  0x0020,   /* U+3319 */
  0x0020,   /* U+331A */
  0x0020,   /* U+331B */
  0x0020,   /* U+331C */
  0x0020,   /* U+331D */
  0x0020,   /* U+331E */
  0x0020,   /* U+331F */
  0x0020,   /* U+3320 */
  0x0020,   /* U+3321 */
  0x0020,   /* U+3322 */
  0x0020,   /* U+3323 */
  0x0020,   /* U+3324 */
  0x0020,   /* U+3325 */
  0x0020,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0020,   /* U+3329 */
  0x0020,   /* U+332A */
  0x0020,   /* U+332B */
  0x0020,   /* U+332C */
  0x0020,   /* U+332D */
  0x0020,   /* U+332E */
  0x0020,   /* U+332F */
  0x0020,   /* U+3330 */
  0x0020,   /* U+3331 */
  0x0020,   /* U+3332 */
  0x0020,   /* U+3333 */
  0x0020,   /* U+3334 */
  0x0020,   /* U+3335 */
  0x0020,   /* U+3336 */
  0x0020,   /* U+3337 */
  0x0020,   /* U+3338 */
  0x0020,   /* U+3339 */
  0x0020,   /* U+333A */
  0x0020,   /* U+333B */
  0x0020,   /* U+333C */
  0x0020,   /* U+333D */
  0x0020,   /* U+333E */
  0x0000,   /* U+333F */
  0x0020,   /* U+3340 */
  0x0020,   /* U+3341 */
  0x0020,   /* U+3342 */
  0x0020,   /* U+3343 */
  0x0020,   /* U+3344 */
  0x0020,   /* U+3345 */
  0x0020,   /* U+3346 */
  0x0020,   /* U+3347 */
  0x0020,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0020,   /* U+334A */
  0x0037,   /* U+334B */
  0x0037,   /* U+334C */
  0x0020,   /* U+334D */
  0x0020,   /* U+334E */
  0x0020,   /* U+334F */
  0x0020,   /* U+3350 */
  0x0020,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0038,   /* U+3353 */
  0x0020,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0020,   /* U+3356 */
  0x0020,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0020,   /* U+3362 */
  0x0020,   /* U+3363 */
  0x0020,   /* U+3364 */
  0x0020,   /* U+3365 */
  0x0020,   /* U+3366 */
  0x0020,   /* U+3367 */
  0x0020,   /* U+3368 */
  0x0020,   /* U+3369 */
  0x0020,   /* U+336A */
  0x0020,   /* U+336B */
  0x0020,   /* U+336C */
  0x0020,   /* U+336D */
  0x0020,   /* U+336E */
  0x0020,   /* U+336F */
  0x0020,   /* U+3370 */
  0x0020,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0020,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0020,   /* U+3378 */
  0x0020,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0020,   /* U+337B */
  0x0020,   /* U+337C */
  0x0020,   /* U+337D */
  0x0020,   /* U+337E */
  0x0020,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0020,   /* U+3388 */
  0x0020,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0020,   /* U+3391 */
  0x0020,   /* U+3392 */
  0x0020,   /* U+3393 */
  0x0020,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0020,   /* U+339F */
  0x0020,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0020,   /* U+33A2 */
  0x0020,   /* U+33A3 */
  0x0020,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0020,   /* U+33A6 */
  0x0020,   /* U+33A7 */
  0x0020,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0020,   /* U+33AA */
  0x0020,   /* U+33AB */
  0x0020,   /* U+33AC */
  0x0020,   /* U+33AD */
  0x0020,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0020,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0020,   /* U+33C6 */
  0x0020,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0020,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0020,   /* U+33D5 */
  0x0020,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0020,   /* U+33D8 */
  0x0020,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0020,   /* U+33DE */
  0x0020,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0020,   /* U+33E9 */
  0x0020,   /* U+33EA */
  0x0020,   /* U+33EB */
  0x0020,   /* U+33EC */
  0x0020,   /* U+33ED */
  0x0020,   /* U+33EE */
  0x0020,   /* U+33EF */
  0x0020,   /* U+33F0 */
  0x0020,   /* U+33F1 */
  0x0020,   /* U+33F2 */
  0x0020,   /* U+33F3 */
  0x0020,   /* U+33F4 */
  0x0020,   /* U+33F5 */
  0x0020,   /* U+33F6 */
  0x0020,   /* U+33F7 */
  0x0020,   /* U+33F8 */
  0x0020,   /* U+33F9 */
  0x0020,   /* U+33FA */
  0x0020,   /* U+33FB */
  0x0020,   /* U+33FC */
  0x0020,   /* U+33FD */
  0x0020,   /* U+33FE */
  0x0020,   /* U+33FF */

  /* Tertiary weight 3 for each character. */
  0x001C,   /* U+3300 */
  0x001C,   /* U+3301 */
  0x001C,   /* U+3302 */
  0x001C,   /* U+3303 */
  0x001C,   /* U+3304 */
  0x001C,   /* U+3305 */
  0x001C,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x001C,   /* U+3308 */
  0x001C,   /* U+3309 */
  0x001C,   /* U+330A */
  0x001C,   /* U+330B */
  0x001C,   /* U+330C */
  0x001C,   /* U+330D */
  0x001C,   /* U+330E */
  0x001C,   /* U+330F */
  0x001C,   /* U+3310 */
  0x001C,   /* U+3311 */
  0x001C,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x001C,   /* U+3317 */
  0x001C,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x001C,   /* U+331B */
  0x001C,   /* U+331C */
  0x001C,   /* U+331D */
  0x001C,   /* U+331E */
  0x001C,   /* U+331F */
  0x001C,   /* U+3320 */
  0x001C,   /* U+3321 */
  0x001C,   /* U+3322 */
  0x001C,   /* U+3323 */
  0x001C,   /* U+3324 */
  0x001C,   /* U+3325 */
  0x001C,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x001C,   /* U+3329 */
  0x001C,   /* U+332A */
  0x001C,   /* U+332B */
  0x001C,   /* U+332C */
  0x001C,   /* U+332D */
  0x001C,   /* U+332E */
  0x001C,   /* U+332F */
  0x001C,   /* U+3330 */
  0x001C,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x001C,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x001C,   /* U+3335 */
  0x001C,   /* U+3336 */
  0x001C,   /* U+3337 */
  0x001C,   /* U+3338 */
  0x001C,   /* U+3339 */
  0x001C,   /* U+333A */
  0x001C,   /* U+333B */
  0x001C,   /* U+333C */
  0x001C,   /* U+333D */
  0x001C,   /* U+333E */
  0x0000,   /* U+333F */
  0x001C,   /* U+3340 */
  0x001C,   /* U+3341 */
  0x001C,   /* U+3342 */
  0x001C,   /* U+3343 */
  0x001C,   /* U+3344 */
  0x001C,   /* U+3345 */
  0x001C,   /* U+3346 */
  0x001C,   /* U+3347 */
  0x001C,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x001C,   /* U+334A */
  0x001C,   /* U+334B */
  0x001C,   /* U+334C */
  0x001C,   /* U+334D */
  0x001C,   /* U+334E */
  0x001C,   /* U+334F */
  0x001C,   /* U+3350 */
  0x001C,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x001C,   /* U+3353 */
  0x001C,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x001C,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0004,   /* U+3362 */
  0x0004,   /* U+3363 */
  0x0004,   /* U+3364 */
  0x0004,   /* U+3365 */
  0x0004,   /* U+3366 */
  0x0004,   /* U+3367 */
  0x0004,   /* U+3368 */
  0x0004,   /* U+3369 */
  0x0004,   /* U+336A */
  0x0004,   /* U+336B */
  0x0004,   /* U+336C */
  0x0004,   /* U+336D */
  0x0004,   /* U+336E */
  0x0004,   /* U+336F */
  0x0004,   /* U+3370 */
  0x001C,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x001C,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x001C,   /* U+3378 */
  0x001C,   /* U+3379 */
  0x0000,   /* U+337A */
  0x001C,   /* U+337B */
  0x001C,   /* U+337C */
  0x001C,   /* U+337D */
  0x001C,   /* U+337E */
  0x001C,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x001C,   /* U+3388 */
  0x001C,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x001C,   /* U+3391 */
  0x001C,   /* U+3392 */
  0x001C,   /* U+3393 */
  0x001C,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x001C,   /* U+339F */
  0x001C,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x001C,   /* U+33A2 */
  0x001C,   /* U+33A3 */
  0x001C,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x001C,   /* U+33A6 */
  0x001C,   /* U+33A7 */
  0x001C,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x001C,   /* U+33AA */
  0x001C,   /* U+33AB */
  0x001C,   /* U+33AC */
  0x001C,   /* U+33AD */
  0x001C,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x001C,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x001C,   /* U+33C6 */
  0x001C,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x001C,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x001C,   /* U+33D5 */
  0x001C,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x001C,   /* U+33D8 */
  0x001D,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x001C,   /* U+33DE */
  0x001C,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0004,   /* U+33E9 */
  0x0004,   /* U+33EA */
  0x0004,   /* U+33EB */
  0x0004,   /* U+33EC */
  0x0004,   /* U+33ED */
  0x0004,   /* U+33EE */
  0x0004,   /* U+33EF */
  0x0004,   /* U+33F0 */
  0x0004,   /* U+33F1 */
  0x0004,   /* U+33F2 */
  0x0004,   /* U+33F3 */
  0x0004,   /* U+33F4 */
  0x0004,   /* U+33F5 */
  0x0004,   /* U+33F6 */
  0x0004,   /* U+33F7 */
  0x0004,   /* U+33F8 */
  0x0004,   /* U+33F9 */
  0x0004,   /* U+33FA */
  0x0004,   /* U+33FB */
  0x0004,   /* U+33FC */
  0x0004,   /* U+33FD */
  0x0004,   /* U+33FE */
  0x001C,   /* U+33FF */

  /* Primary weight 4 for each character. */
  0x1C0E,   /* U+3300 */
  0x3D5A,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x3D62,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x1C0E,   /* U+3307 */
  0x1C0E,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x3D6E,   /* U+330C */
  0x1C0E,   /* U+330D */
  0x3D8B,   /* U+330E */
  0x3D79,   /* U+330F */
  0x0000,   /* U+3310 */
  0x1C0E,   /* U+3311 */
  0x1C0E,   /* U+3312 */
  0x3D6A,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x1C0E,   /* U+3316 */
  0x3D6C,   /* U+3317 */
  0x3D7B,   /* U+3318 */
  0x3D7B,   /* U+3319 */
  0x0000,   /* U+331A */
  0x3D72,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x3D84,   /* U+331F */
  0x1C0E,   /* U+3320 */
  0x3D62,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x3D67,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x3D68,   /* U+332B */
  0x3D6C,   /* U+332C */
  0x3D85,   /* U+332D */
  0x3D67,   /* U+332E */
  0x3D84,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x3D6C,   /* U+3332 */
  0x3D6E,   /* U+3333 */
  0x3D66,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x1C0E,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x3D75,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x3D67,   /* U+333A */
  0x3D66,   /* U+333B */
  0x3D6A,   /* U+333C */
  0x3D8B,   /* U+333D */
  0x3D6E,   /* U+333E */
  0x0000,   /* U+333F */
  0x3D6E,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x3D86,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x3D81,   /* U+3347 */
  0x3D8B,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x3D6E,   /* U+334C */
  0x3D84,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x3D84,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x1C0E,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x3D63,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0xF0B9,   /* U+3362 */
  0xF0B9,   /* U+3363 */
  0xF0B9,   /* U+3364 */
  0xF0B9,   /* U+3365 */
  0xF0B9,   /* U+3366 */
  0xF0B9,   /* U+3367 */
  0xF0B9,   /* U+3368 */
  0xF0B9,   /* U+3369 */
  0xF0B9,   /* U+336A */
  0xF0B9,   /* U+336B */
  0xF0B9,   /* U+336C */
  0xF0B9,   /* U+336D */
  0xF0B9,   /* U+336E */
  0xF0B9,   /* U+336F */
  0xF0B9,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0xE210,   /* U+337B */
  0xD48C,   /* U+337C */
  0xEB63,   /* U+337D */
  0xECBB,   /* U+337E */
  0xDF0F,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x1D77,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x1C3F,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0625,   /* U+33AE */
  0x0625,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0277,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x1CF4,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0277,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0xE5E5,   /* U+33E9 */
  0xE5E5,   /* U+33EA */
  0xE5E5,   /* U+33EB */
  0xE5E5,   /* U+33EC */
  0xE5E5,   /* U+33ED */
  0xE5E5,   /* U+33EE */
  0xE5E5,   /* U+33EF */
  0xE5E5,   /* U+33F0 */
  0xE5E5,   /* U+33F1 */
  0xE5E5,   /* U+33F2 */
  0xE5E5,   /* U+33F3 */
  0xE5E5,   /* U+33F4 */
  0xE5E5,   /* U+33F5 */
  0xE5E5,   /* U+33F6 */
  0xE5E5,   /* U+33F7 */
  0xE5E5,   /* U+33F8 */
  0xE5E5,   /* U+33F9 */
  0xE5E5,   /* U+33FA */
  0xE5E5,   /* U+33FB */
  0xE5E5,   /* U+33FC */
  0xE5E5,   /* U+33FD */
  0xE5E5,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Secondary weight 4 for each character. */
  0x0020,   /* U+3300 */
  0x0020,   /* U+3301 */
  0x0038,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0020,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0020,   /* U+3307 */
  0x0020,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0020,   /* U+330C */
  0x0020,   /* U+330D */
  0x0020,   /* U+330E */
  0x0020,   /* U+330F */
  0x0037,   /* U+3310 */
  0x0020,   /* U+3311 */
  0x0020,   /* U+3312 */
  0x0020,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0037,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0020,   /* U+3317 */
  0x0020,   /* U+3318 */
  0x0020,   /* U+3319 */
  0x0037,   /* U+331A */
  0x0020,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0038,   /* U+331E */
  0x0020,   /* U+331F */
  0x0020,   /* U+3320 */
  0x0020,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0020,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0020,   /* U+332B */
  0x0020,   /* U+332C */
  0x0020,   /* U+332D */
  0x0020,   /* U+332E */
  0x0020,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0020,   /* U+3332 */
  0x0020,   /* U+3333 */
  0x0020,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0020,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0020,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0020,   /* U+333A */
  0x0020,   /* U+333B */
  0x0020,   /* U+333C */
  0x0020,   /* U+333D */
  0x0020,   /* U+333E */
  0x0000,   /* U+333F */
  0x0020,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0020,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0020,   /* U+3347 */
  0x0020,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0037,   /* U+334A */
  0x0000,   /* U+334B */
  0x0020,   /* U+334C */
  0x0020,   /* U+334D */
  0x0037,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0020,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0020,   /* U+3353 */
  0x0037,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0020,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0020,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0020,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0020,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0020,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0020,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0020,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Tertiary weight 4 for each character. */
  0x001C,   /* U+3300 */
  0x001C,   /* U+3301 */
  0x001C,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x001C,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x001C,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x001C,   /* U+330C */
  0x001C,   /* U+330D */
  0x001C,   /* U+330E */
  0x001C,   /* U+330F */
  0x001C,   /* U+3310 */
  0x001C,   /* U+3311 */
  0x001C,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x001C,   /* U+3317 */
  0x001C,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x001C,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x001C,   /* U+331E */
  0x001C,   /* U+331F */
  0x001C,   /* U+3320 */
  0x001C,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x001C,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x001C,   /* U+332B */
  0x001C,   /* U+332C */
  0x001C,   /* U+332D */
  0x001C,   /* U+332E */
  0x001C,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x001C,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x001C,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x001C,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x001C,   /* U+333A */
  0x001C,   /* U+333B */
  0x001C,   /* U+333C */
  0x001C,   /* U+333D */
  0x001C,   /* U+333E */
  0x0000,   /* U+333F */
  0x001C,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x001C,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x001C,   /* U+3347 */
  0x001C,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x001C,   /* U+334A */
  0x0000,   /* U+334B */
  0x001C,   /* U+334C */
  0x001C,   /* U+334D */
  0x001C,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x001C,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x001C,   /* U+3353 */
  0x001C,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x001C,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x001C,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x001C,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x001C,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x001C,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x001C,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Primary weight 5 for each character. */
  0x3D6E,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x3D5A,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x3D6E,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x3D82,   /* U+3315 */
  0x3D6E,   /* U+3316 */
  0x3D6E,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x3D6E,   /* U+3319 */
  0x3D5B,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x3D7B,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x3D8B,   /* U+332B */
  0x0000,   /* U+332C */
  0x3D84,   /* U+332D */
  0x3D6E,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x3D6E,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x3D5E,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x3D84,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x3D6E,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x3D8B,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x1C0E,   /* U+334A */
  0x0000,   /* U+334B */
  0x3D8B,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x3D84,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0xFB40,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x1E71,   /* U+33AE */
  0x1E71,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Secondary weight 5 for each character. */
  0x0020,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0020,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0037,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0020,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0037,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0020,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0020,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0020,   /* U+3319 */
  0x0020,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0020,   /* U+3320 */
  0x0037,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0020,   /* U+332B */
  0x0000,   /* U+332C */
  0x0020,   /* U+332D */
  0x0020,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0020,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0020,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0020,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0037,   /* U+333B */
  0x0000,   /* U+333C */
  0x0020,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0037,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0020,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0020,   /* U+334A */
  0x0000,   /* U+334B */
  0x0020,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0020,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0037,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0020,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0020,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Tertiary weight 5 for each character. */
  0x001C,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x001C,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x001C,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x001C,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x001C,   /* U+3320 */
  0x001C,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x001C,   /* U+332B */
  0x0000,   /* U+332C */
  0x001C,   /* U+332D */
  0x001C,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x001C,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x001C,   /* U+333B */
  0x0000,   /* U+333C */
  0x001C,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x001C,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x001C,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x001C,   /* U+334A */
  0x0000,   /* U+334B */
  0x001C,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x001C,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x001C,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x001C,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Primary weight 6 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x1C0E,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x3D7B,   /* U+3315 */
  0x3D84,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x3D8B,   /* U+3319 */
  0x3D86,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x3D6E,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x3D84,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x3D84,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x3D84,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x3D8B,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0xCF1A,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x1C3F,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Secondary weight 6 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0037,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0020,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0020,   /* U+3315 */
  0x0020,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0020,   /* U+3319 */
  0x0020,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0020,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0020,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0037,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0020,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0020,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0020,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0020,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Tertiary weight 6 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x001C,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x001C,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x001C,   /* U+3315 */
  0x001C,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x001C,   /* U+3319 */
  0x001C,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x001C,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x001C,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x001C,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x001C,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x001C,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x001C,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x001C,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Primary weight 7 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0xFB40,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Secondary weight 7 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0020,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Tertiary weight 7 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x001C,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Primary weight 8 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0xF93E,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Secondary weight 8 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */

  /* Tertiary weight 8 for each character. */
  0x0000,   /* U+3300 */
  0x0000,   /* U+3301 */
  0x0000,   /* U+3302 */
  0x0000,   /* U+3303 */
  0x0000,   /* U+3304 */
  0x0000,   /* U+3305 */
  0x0000,   /* U+3306 */
  0x0000,   /* U+3307 */
  0x0000,   /* U+3308 */
  0x0000,   /* U+3309 */
  0x0000,   /* U+330A */
  0x0000,   /* U+330B */
  0x0000,   /* U+330C */
  0x0000,   /* U+330D */
  0x0000,   /* U+330E */
  0x0000,   /* U+330F */
  0x0000,   /* U+3310 */
  0x0000,   /* U+3311 */
  0x0000,   /* U+3312 */
  0x0000,   /* U+3313 */
  0x0000,   /* U+3314 */
  0x0000,   /* U+3315 */
  0x0000,   /* U+3316 */
  0x0000,   /* U+3317 */
  0x0000,   /* U+3318 */
  0x0000,   /* U+3319 */
  0x0000,   /* U+331A */
  0x0000,   /* U+331B */
  0x0000,   /* U+331C */
  0x0000,   /* U+331D */
  0x0000,   /* U+331E */
  0x0000,   /* U+331F */
  0x0000,   /* U+3320 */
  0x0000,   /* U+3321 */
  0x0000,   /* U+3322 */
  0x0000,   /* U+3323 */
  0x0000,   /* U+3324 */
  0x0000,   /* U+3325 */
  0x0000,   /* U+3326 */
  0x0000,   /* U+3327 */
  0x0000,   /* U+3328 */
  0x0000,   /* U+3329 */
  0x0000,   /* U+332A */
  0x0000,   /* U+332B */
  0x0000,   /* U+332C */
  0x0000,   /* U+332D */
  0x0000,   /* U+332E */
  0x0000,   /* U+332F */
  0x0000,   /* U+3330 */
  0x0000,   /* U+3331 */
  0x0000,   /* U+3332 */
  0x0000,   /* U+3333 */
  0x0000,   /* U+3334 */
  0x0000,   /* U+3335 */
  0x0000,   /* U+3336 */
  0x0000,   /* U+3337 */
  0x0000,   /* U+3338 */
  0x0000,   /* U+3339 */
  0x0000,   /* U+333A */
  0x0000,   /* U+333B */
  0x0000,   /* U+333C */
  0x0000,   /* U+333D */
  0x0000,   /* U+333E */
  0x0000,   /* U+333F */
  0x0000,   /* U+3340 */
  0x0000,   /* U+3341 */
  0x0000,   /* U+3342 */
  0x0000,   /* U+3343 */
  0x0000,   /* U+3344 */
  0x0000,   /* U+3345 */
  0x0000,   /* U+3346 */
  0x0000,   /* U+3347 */
  0x0000,   /* U+3348 */
  0x0000,   /* U+3349 */
  0x0000,   /* U+334A */
  0x0000,   /* U+334B */
  0x0000,   /* U+334C */
  0x0000,   /* U+334D */
  0x0000,   /* U+334E */
  0x0000,   /* U+334F */
  0x0000,   /* U+3350 */
  0x0000,   /* U+3351 */
  0x0000,   /* U+3352 */
  0x0000,   /* U+3353 */
  0x0000,   /* U+3354 */
  0x0000,   /* U+3355 */
  0x0000,   /* U+3356 */
  0x0000,   /* U+3357 */
  0x0000,   /* U+3358 */
  0x0000,   /* U+3359 */
  0x0000,   /* U+335A */
  0x0000,   /* U+335B */
  0x0000,   /* U+335C */
  0x0000,   /* U+335D */
  0x0000,   /* U+335E */
  0x0000,   /* U+335F */
  0x0000,   /* U+3360 */
  0x0000,   /* U+3361 */
  0x0000,   /* U+3362 */
  0x0000,   /* U+3363 */
  0x0000,   /* U+3364 */
  0x0000,   /* U+3365 */
  0x0000,   /* U+3366 */
  0x0000,   /* U+3367 */
  0x0000,   /* U+3368 */
  0x0000,   /* U+3369 */
  0x0000,   /* U+336A */
  0x0000,   /* U+336B */
  0x0000,   /* U+336C */
  0x0000,   /* U+336D */
  0x0000,   /* U+336E */
  0x0000,   /* U+336F */
  0x0000,   /* U+3370 */
  0x0000,   /* U+3371 */
  0x0000,   /* U+3372 */
  0x0000,   /* U+3373 */
  0x0000,   /* U+3374 */
  0x0000,   /* U+3375 */
  0x0000,   /* U+3376 */
  0x0000,   /* U+3377 */
  0x0000,   /* U+3378 */
  0x0000,   /* U+3379 */
  0x0000,   /* U+337A */
  0x0000,   /* U+337B */
  0x0000,   /* U+337C */
  0x0000,   /* U+337D */
  0x0000,   /* U+337E */
  0x0000,   /* U+337F */
  0x0000,   /* U+3380 */
  0x0000,   /* U+3381 */
  0x0000,   /* U+3382 */
  0x0000,   /* U+3383 */
  0x0000,   /* U+3384 */
  0x0000,   /* U+3385 */
  0x0000,   /* U+3386 */
  0x0000,   /* U+3387 */
  0x0000,   /* U+3388 */
  0x0000,   /* U+3389 */
  0x0000,   /* U+338A */
  0x0000,   /* U+338B */
  0x0000,   /* U+338C */
  0x0000,   /* U+338D */
  0x0000,   /* U+338E */
  0x0000,   /* U+338F */
  0x0000,   /* U+3390 */
  0x0000,   /* U+3391 */
  0x0000,   /* U+3392 */
  0x0000,   /* U+3393 */
  0x0000,   /* U+3394 */
  0x0000,   /* U+3395 */
  0x0000,   /* U+3396 */
  0x0000,   /* U+3397 */
  0x0000,   /* U+3398 */
  0x0000,   /* U+3399 */
  0x0000,   /* U+339A */
  0x0000,   /* U+339B */
  0x0000,   /* U+339C */
  0x0000,   /* U+339D */
  0x0000,   /* U+339E */
  0x0000,   /* U+339F */
  0x0000,   /* U+33A0 */
  0x0000,   /* U+33A1 */
  0x0000,   /* U+33A2 */
  0x0000,   /* U+33A3 */
  0x0000,   /* U+33A4 */
  0x0000,   /* U+33A5 */
  0x0000,   /* U+33A6 */
  0x0000,   /* U+33A7 */
  0x0000,   /* U+33A8 */
  0x0000,   /* U+33A9 */
  0x0000,   /* U+33AA */
  0x0000,   /* U+33AB */
  0x0000,   /* U+33AC */
  0x0000,   /* U+33AD */
  0x0000,   /* U+33AE */
  0x0000,   /* U+33AF */
  0x0000,   /* U+33B0 */
  0x0000,   /* U+33B1 */
  0x0000,   /* U+33B2 */
  0x0000,   /* U+33B3 */
  0x0000,   /* U+33B4 */
  0x0000,   /* U+33B5 */
  0x0000,   /* U+33B6 */
  0x0000,   /* U+33B7 */
  0x0000,   /* U+33B8 */
  0x0000,   /* U+33B9 */
  0x0000,   /* U+33BA */
  0x0000,   /* U+33BB */
  0x0000,   /* U+33BC */
  0x0000,   /* U+33BD */
  0x0000,   /* U+33BE */
  0x0000,   /* U+33BF */
  0x0000,   /* U+33C0 */
  0x0000,   /* U+33C1 */
  0x0000,   /* U+33C2 */
  0x0000,   /* U+33C3 */
  0x0000,   /* U+33C4 */
  0x0000,   /* U+33C5 */
  0x0000,   /* U+33C6 */
  0x0000,   /* U+33C7 */
  0x0000,   /* U+33C8 */
  0x0000,   /* U+33C9 */
  0x0000,   /* U+33CA */
  0x0000,   /* U+33CB */
  0x0000,   /* U+33CC */
  0x0000,   /* U+33CD */
  0x0000,   /* U+33CE */
  0x0000,   /* U+33CF */
  0x0000,   /* U+33D0 */
  0x0000,   /* U+33D1 */
  0x0000,   /* U+33D2 */
  0x0000,   /* U+33D3 */
  0x0000,   /* U+33D4 */
  0x0000,   /* U+33D5 */
  0x0000,   /* U+33D6 */
  0x0000,   /* U+33D7 */
  0x0000,   /* U+33D8 */
  0x0000,   /* U+33D9 */
  0x0000,   /* U+33DA */
  0x0000,   /* U+33DB */
  0x0000,   /* U+33DC */
  0x0000,   /* U+33DD */
  0x0000,   /* U+33DE */
  0x0000,   /* U+33DF */
  0x0000,   /* U+33E0 */
  0x0000,   /* U+33E1 */
  0x0000,   /* U+33E2 */
  0x0000,   /* U+33E3 */
  0x0000,   /* U+33E4 */
  0x0000,   /* U+33E5 */
  0x0000,   /* U+33E6 */
  0x0000,   /* U+33E7 */
  0x0000,   /* U+33E8 */
  0x0000,   /* U+33E9 */
  0x0000,   /* U+33EA */
  0x0000,   /* U+33EB */
  0x0000,   /* U+33EC */
  0x0000,   /* U+33ED */
  0x0000,   /* U+33EE */
  0x0000,   /* U+33EF */
  0x0000,   /* U+33F0 */
  0x0000,   /* U+33F1 */
  0x0000,   /* U+33F2 */
  0x0000,   /* U+33F3 */
  0x0000,   /* U+33F4 */
  0x0000,   /* U+33F5 */
  0x0000,   /* U+33F6 */
  0x0000,   /* U+33F7 */
  0x0000,   /* U+33F8 */
  0x0000,   /* U+33F9 */
  0x0000,   /* U+33FA */
  0x0000,   /* U+33FB */
  0x0000,   /* U+33FC */
  0x0000,   /* U+33FD */
  0x0000,   /* U+33FE */
  0x0000,   /* U+33FF */
};

uint16 uca900_p04D[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0xFB80,   /* U+4D00 */
  0xFB80,   /* U+4D01 */
  0xFB80,   /* U+4D02 */
  0xFB80,   /* U+4D03 */
  0xFB80,   /* U+4D04 */
  0xFB80,   /* U+4D05 */
  0xFB80,   /* U+4D06 */
  0xFB80,   /* U+4D07 */
  0xFB80,   /* U+4D08 */
  0xFB80,   /* U+4D09 */
  0xFB80,   /* U+4D0A */
  0xFB80,   /* U+4D0B */
  0xFB80,   /* U+4D0C */
  0xFB80,   /* U+4D0D */
  0xFB80,   /* U+4D0E */
  0xFB80,   /* U+4D0F */
  0xFB80,   /* U+4D10 */
  0xFB80,   /* U+4D11 */
  0xFB80,   /* U+4D12 */
  0xFB80,   /* U+4D13 */
  0xFB80,   /* U+4D14 */
  0xFB80,   /* U+4D15 */
  0xFB80,   /* U+4D16 */
  0xFB80,   /* U+4D17 */
  0xFB80,   /* U+4D18 */
  0xFB80,   /* U+4D19 */
  0xFB80,   /* U+4D1A */
  0xFB80,   /* U+4D1B */
  0xFB80,   /* U+4D1C */
  0xFB80,   /* U+4D1D */
  0xFB80,   /* U+4D1E */
  0xFB80,   /* U+4D1F */
  0xFB80,   /* U+4D20 */
  0xFB80,   /* U+4D21 */
  0xFB80,   /* U+4D22 */
  0xFB80,   /* U+4D23 */
  0xFB80,   /* U+4D24 */
  0xFB80,   /* U+4D25 */
  0xFB80,   /* U+4D26 */
  0xFB80,   /* U+4D27 */
  0xFB80,   /* U+4D28 */
  0xFB80,   /* U+4D29 */
  0xFB80,   /* U+4D2A */
  0xFB80,   /* U+4D2B */
  0xFB80,   /* U+4D2C */
  0xFB80,   /* U+4D2D */
  0xFB80,   /* U+4D2E */
  0xFB80,   /* U+4D2F */
  0xFB80,   /* U+4D30 */
  0xFB80,   /* U+4D31 */
  0xFB80,   /* U+4D32 */
  0xFB80,   /* U+4D33 */
  0xFB80,   /* U+4D34 */
  0xFB80,   /* U+4D35 */
  0xFB80,   /* U+4D36 */
  0xFB80,   /* U+4D37 */
  0xFB80,   /* U+4D38 */
  0xFB80,   /* U+4D39 */
  0xFB80,   /* U+4D3A */
  0xFB80,   /* U+4D3B */
  0xFB80,   /* U+4D3C */
  0xFB80,   /* U+4D3D */
  0xFB80,   /* U+4D3E */
  0xFB80,   /* U+4D3F */
  0xFB80,   /* U+4D40 */
  0xFB80,   /* U+4D41 */
  0xFB80,   /* U+4D42 */
  0xFB80,   /* U+4D43 */
  0xFB80,   /* U+4D44 */
  0xFB80,   /* U+4D45 */
  0xFB80,   /* U+4D46 */
  0xFB80,   /* U+4D47 */
  0xFB80,   /* U+4D48 */
  0xFB80,   /* U+4D49 */
  0xFB80,   /* U+4D4A */
  0xFB80,   /* U+4D4B */
  0xFB80,   /* U+4D4C */
  0xFB80,   /* U+4D4D */
  0xFB80,   /* U+4D4E */
  0xFB80,   /* U+4D4F */
  0xFB80,   /* U+4D50 */
  0xFB80,   /* U+4D51 */
  0xFB80,   /* U+4D52 */
  0xFB80,   /* U+4D53 */
  0xFB80,   /* U+4D54 */
  0xFB80,   /* U+4D55 */
  0xFB80,   /* U+4D56 */
  0xFB80,   /* U+4D57 */
  0xFB80,   /* U+4D58 */
  0xFB80,   /* U+4D59 */
  0xFB80,   /* U+4D5A */
  0xFB80,   /* U+4D5B */
  0xFB80,   /* U+4D5C */
  0xFB80,   /* U+4D5D */
  0xFB80,   /* U+4D5E */
  0xFB80,   /* U+4D5F */
  0xFB80,   /* U+4D60 */
  0xFB80,   /* U+4D61 */
  0xFB80,   /* U+4D62 */
  0xFB80,   /* U+4D63 */
  0xFB80,   /* U+4D64 */
  0xFB80,   /* U+4D65 */
  0xFB80,   /* U+4D66 */
  0xFB80,   /* U+4D67 */
  0xFB80,   /* U+4D68 */
  0xFB80,   /* U+4D69 */
  0xFB80,   /* U+4D6A */
  0xFB80,   /* U+4D6B */
  0xFB80,   /* U+4D6C */
  0xFB80,   /* U+4D6D */
  0xFB80,   /* U+4D6E */
  0xFB80,   /* U+4D6F */
  0xFB80,   /* U+4D70 */
  0xFB80,   /* U+4D71 */
  0xFB80,   /* U+4D72 */
  0xFB80,   /* U+4D73 */
  0xFB80,   /* U+4D74 */
  0xFB80,   /* U+4D75 */
  0xFB80,   /* U+4D76 */
  0xFB80,   /* U+4D77 */
  0xFB80,   /* U+4D78 */
  0xFB80,   /* U+4D79 */
  0xFB80,   /* U+4D7A */
  0xFB80,   /* U+4D7B */
  0xFB80,   /* U+4D7C */
  0xFB80,   /* U+4D7D */
  0xFB80,   /* U+4D7E */
  0xFB80,   /* U+4D7F */
  0xFB80,   /* U+4D80 */
  0xFB80,   /* U+4D81 */
  0xFB80,   /* U+4D82 */
  0xFB80,   /* U+4D83 */
  0xFB80,   /* U+4D84 */
  0xFB80,   /* U+4D85 */
  0xFB80,   /* U+4D86 */
  0xFB80,   /* U+4D87 */
  0xFB80,   /* U+4D88 */
  0xFB80,   /* U+4D89 */
  0xFB80,   /* U+4D8A */
  0xFB80,   /* U+4D8B */
  0xFB80,   /* U+4D8C */
  0xFB80,   /* U+4D8D */
  0xFB80,   /* U+4D8E */
  0xFB80,   /* U+4D8F */
  0xFB80,   /* U+4D90 */
  0xFB80,   /* U+4D91 */
  0xFB80,   /* U+4D92 */
  0xFB80,   /* U+4D93 */
  0xFB80,   /* U+4D94 */
  0xFB80,   /* U+4D95 */
  0xFB80,   /* U+4D96 */
  0xFB80,   /* U+4D97 */
  0xFB80,   /* U+4D98 */
  0xFB80,   /* U+4D99 */
  0xFB80,   /* U+4D9A */
  0xFB80,   /* U+4D9B */
  0xFB80,   /* U+4D9C */
  0xFB80,   /* U+4D9D */
  0xFB80,   /* U+4D9E */
  0xFB80,   /* U+4D9F */
  0xFB80,   /* U+4DA0 */
  0xFB80,   /* U+4DA1 */
  0xFB80,   /* U+4DA2 */
  0xFB80,   /* U+4DA3 */
  0xFB80,   /* U+4DA4 */
  0xFB80,   /* U+4DA5 */
  0xFB80,   /* U+4DA6 */
  0xFB80,   /* U+4DA7 */
  0xFB80,   /* U+4DA8 */
  0xFB80,   /* U+4DA9 */
  0xFB80,   /* U+4DAA */
  0xFB80,   /* U+4DAB */
  0xFB80,   /* U+4DAC */
  0xFB80,   /* U+4DAD */
  0xFB80,   /* U+4DAE */
  0xFB80,   /* U+4DAF */
  0xFB80,   /* U+4DB0 */
  0xFB80,   /* U+4DB1 */
  0xFB80,   /* U+4DB2 */
  0xFB80,   /* U+4DB3 */
  0xFB80,   /* U+4DB4 */
  0xFB80,   /* U+4DB5 */
  0xFBC0,   /* U+4DB6 */
  0xFBC0,   /* U+4DB7 */
  0xFBC0,   /* U+4DB8 */
  0xFBC0,   /* U+4DB9 */
  0xFBC0,   /* U+4DBA */
  0xFBC0,   /* U+4DBB */
  0xFBC0,   /* U+4DBC */
  0xFBC0,   /* U+4DBD */
  0xFBC0,   /* U+4DBE */
  0xFBC0,   /* U+4DBF */
  0x0EAA,   /* U+4DC0 */
  0x0EAB,   /* U+4DC1 */
  0x0EAC,   /* U+4DC2 */
  0x0EAD,   /* U+4DC3 */
  0x0EAE,   /* U+4DC4 */
  0x0EAF,   /* U+4DC5 */
  0x0EB0,   /* U+4DC6 */
  0x0EB1,   /* U+4DC7 */
  0x0EB2,   /* U+4DC8 */
  0x0EB3,   /* U+4DC9 */
  0x0EB4,   /* U+4DCA */
  0x0EB5,   /* U+4DCB */
  0x0EB6,   /* U+4DCC */
  0x0EB7,   /* U+4DCD */
  0x0EB8,   /* U+4DCE */
  0x0EB9,   /* U+4DCF */
  0x0EBA,   /* U+4DD0 */
  0x0EBB,   /* U+4DD1 */
  0x0EBC,   /* U+4DD2 */
  0x0EBD,   /* U+4DD3 */
  0x0EBE,   /* U+4DD4 */
  0x0EBF,   /* U+4DD5 */
  0x0EC0,   /* U+4DD6 */
  0x0EC1,   /* U+4DD7 */
  0x0EC2,   /* U+4DD8 */
  0x0EC3,   /* U+4DD9 */
  0x0EC4,   /* U+4DDA */
  0x0EC5,   /* U+4DDB */
  0x0EC6,   /* U+4DDC */
  0x0EC7,   /* U+4DDD */
  0x0EC8,   /* U+4DDE */
  0x0EC9,   /* U+4DDF */
  0x0ECA,   /* U+4DE0 */
  0x0ECB,   /* U+4DE1 */
  0x0ECC,   /* U+4DE2 */
  0x0ECD,   /* U+4DE3 */
  0x0ECE,   /* U+4DE4 */
  0x0ECF,   /* U+4DE5 */
  0x0ED0,   /* U+4DE6 */
  0x0ED1,   /* U+4DE7 */
  0x0ED2,   /* U+4DE8 */
  0x0ED3,   /* U+4DE9 */
  0x0ED4,   /* U+4DEA */
  0x0ED5,   /* U+4DEB */
  0x0ED6,   /* U+4DEC */
  0x0ED7,   /* U+4DED */
  0x0ED8,   /* U+4DEE */
  0x0ED9,   /* U+4DEF */
  0x0EDA,   /* U+4DF0 */
  0x0EDB,   /* U+4DF1 */
  0x0EDC,   /* U+4DF2 */
  0x0EDD,   /* U+4DF3 */
  0x0EDE,   /* U+4DF4 */
  0x0EDF,   /* U+4DF5 */
  0x0EE0,   /* U+4DF6 */
  0x0EE1,   /* U+4DF7 */
  0x0EE2,   /* U+4DF8 */
  0x0EE3,   /* U+4DF9 */
  0x0EE4,   /* U+4DFA */
  0x0EE5,   /* U+4DFB */
  0x0EE6,   /* U+4DFC */
  0x0EE7,   /* U+4DFD */
  0x0EE8,   /* U+4DFE */
  0x0EE9,   /* U+4DFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+4D00 */
  0x0020,   /* U+4D01 */
  0x0020,   /* U+4D02 */
  0x0020,   /* U+4D03 */
  0x0020,   /* U+4D04 */
  0x0020,   /* U+4D05 */
  0x0020,   /* U+4D06 */
  0x0020,   /* U+4D07 */
  0x0020,   /* U+4D08 */
  0x0020,   /* U+4D09 */
  0x0020,   /* U+4D0A */
  0x0020,   /* U+4D0B */
  0x0020,   /* U+4D0C */
  0x0020,   /* U+4D0D */
  0x0020,   /* U+4D0E */
  0x0020,   /* U+4D0F */
  0x0020,   /* U+4D10 */
  0x0020,   /* U+4D11 */
  0x0020,   /* U+4D12 */
  0x0020,   /* U+4D13 */
  0x0020,   /* U+4D14 */
  0x0020,   /* U+4D15 */
  0x0020,   /* U+4D16 */
  0x0020,   /* U+4D17 */
  0x0020,   /* U+4D18 */
  0x0020,   /* U+4D19 */
  0x0020,   /* U+4D1A */
  0x0020,   /* U+4D1B */
  0x0020,   /* U+4D1C */
  0x0020,   /* U+4D1D */
  0x0020,   /* U+4D1E */
  0x0020,   /* U+4D1F */
  0x0020,   /* U+4D20 */
  0x0020,   /* U+4D21 */
  0x0020,   /* U+4D22 */
  0x0020,   /* U+4D23 */
  0x0020,   /* U+4D24 */
  0x0020,   /* U+4D25 */
  0x0020,   /* U+4D26 */
  0x0020,   /* U+4D27 */
  0x0020,   /* U+4D28 */
  0x0020,   /* U+4D29 */
  0x0020,   /* U+4D2A */
  0x0020,   /* U+4D2B */
  0x0020,   /* U+4D2C */
  0x0020,   /* U+4D2D */
  0x0020,   /* U+4D2E */
  0x0020,   /* U+4D2F */
  0x0020,   /* U+4D30 */
  0x0020,   /* U+4D31 */
  0x0020,   /* U+4D32 */
  0x0020,   /* U+4D33 */
  0x0020,   /* U+4D34 */
  0x0020,   /* U+4D35 */
  0x0020,   /* U+4D36 */
  0x0020,   /* U+4D37 */
  0x0020,   /* U+4D38 */
  0x0020,   /* U+4D39 */
  0x0020,   /* U+4D3A */
  0x0020,   /* U+4D3B */
  0x0020,   /* U+4D3C */
  0x0020,   /* U+4D3D */
  0x0020,   /* U+4D3E */
  0x0020,   /* U+4D3F */
  0x0020,   /* U+4D40 */
  0x0020,   /* U+4D41 */
  0x0020,   /* U+4D42 */
  0x0020,   /* U+4D43 */
  0x0020,   /* U+4D44 */
  0x0020,   /* U+4D45 */
  0x0020,   /* U+4D46 */
  0x0020,   /* U+4D47 */
  0x0020,   /* U+4D48 */
  0x0020,   /* U+4D49 */
  0x0020,   /* U+4D4A */
  0x0020,   /* U+4D4B */
  0x0020,   /* U+4D4C */
  0x0020,   /* U+4D4D */
  0x0020,   /* U+4D4E */
  0x0020,   /* U+4D4F */
  0x0020,   /* U+4D50 */
  0x0020,   /* U+4D51 */
  0x0020,   /* U+4D52 */
  0x0020,   /* U+4D53 */
  0x0020,   /* U+4D54 */
  0x0020,   /* U+4D55 */
  0x0020,   /* U+4D56 */
  0x0020,   /* U+4D57 */
  0x0020,   /* U+4D58 */
  0x0020,   /* U+4D59 */
  0x0020,   /* U+4D5A */
  0x0020,   /* U+4D5B */
  0x0020,   /* U+4D5C */
  0x0020,   /* U+4D5D */
  0x0020,   /* U+4D5E */
  0x0020,   /* U+4D5F */
  0x0020,   /* U+4D60 */
  0x0020,   /* U+4D61 */
  0x0020,   /* U+4D62 */
  0x0020,   /* U+4D63 */
  0x0020,   /* U+4D64 */
  0x0020,   /* U+4D65 */
  0x0020,   /* U+4D66 */
  0x0020,   /* U+4D67 */
  0x0020,   /* U+4D68 */
  0x0020,   /* U+4D69 */
  0x0020,   /* U+4D6A */
  0x0020,   /* U+4D6B */
  0x0020,   /* U+4D6C */
  0x0020,   /* U+4D6D */
  0x0020,   /* U+4D6E */
  0x0020,   /* U+4D6F */
  0x0020,   /* U+4D70 */
  0x0020,   /* U+4D71 */
  0x0020,   /* U+4D72 */
  0x0020,   /* U+4D73 */
  0x0020,   /* U+4D74 */
  0x0020,   /* U+4D75 */
  0x0020,   /* U+4D76 */
  0x0020,   /* U+4D77 */
  0x0020,   /* U+4D78 */
  0x0020,   /* U+4D79 */
  0x0020,   /* U+4D7A */
  0x0020,   /* U+4D7B */
  0x0020,   /* U+4D7C */
  0x0020,   /* U+4D7D */
  0x0020,   /* U+4D7E */
  0x0020,   /* U+4D7F */
  0x0020,   /* U+4D80 */
  0x0020,   /* U+4D81 */
  0x0020,   /* U+4D82 */
  0x0020,   /* U+4D83 */
  0x0020,   /* U+4D84 */
  0x0020,   /* U+4D85 */
  0x0020,   /* U+4D86 */
  0x0020,   /* U+4D87 */
  0x0020,   /* U+4D88 */
  0x0020,   /* U+4D89 */
  0x0020,   /* U+4D8A */
  0x0020,   /* U+4D8B */
  0x0020,   /* U+4D8C */
  0x0020,   /* U+4D8D */
  0x0020,   /* U+4D8E */
  0x0020,   /* U+4D8F */
  0x0020,   /* U+4D90 */
  0x0020,   /* U+4D91 */
  0x0020,   /* U+4D92 */
  0x0020,   /* U+4D93 */
  0x0020,   /* U+4D94 */
  0x0020,   /* U+4D95 */
  0x0020,   /* U+4D96 */
  0x0020,   /* U+4D97 */
  0x0020,   /* U+4D98 */
  0x0020,   /* U+4D99 */
  0x0020,   /* U+4D9A */
  0x0020,   /* U+4D9B */
  0x0020,   /* U+4D9C */
  0x0020,   /* U+4D9D */
  0x0020,   /* U+4D9E */
  0x0020,   /* U+4D9F */
  0x0020,   /* U+4DA0 */
  0x0020,   /* U+4DA1 */
  0x0020,   /* U+4DA2 */
  0x0020,   /* U+4DA3 */
  0x0020,   /* U+4DA4 */
  0x0020,   /* U+4DA5 */
  0x0020,   /* U+4DA6 */
  0x0020,   /* U+4DA7 */
  0x0020,   /* U+4DA8 */
  0x0020,   /* U+4DA9 */
  0x0020,   /* U+4DAA */
  0x0020,   /* U+4DAB */
  0x0020,   /* U+4DAC */
  0x0020,   /* U+4DAD */
  0x0020,   /* U+4DAE */
  0x0020,   /* U+4DAF */
  0x0020,   /* U+4DB0 */
  0x0020,   /* U+4DB1 */
  0x0020,   /* U+4DB2 */
  0x0020,   /* U+4DB3 */
  0x0020,   /* U+4DB4 */
  0x0020,   /* U+4DB5 */
  0x0020,   /* U+4DB6 */
  0x0020,   /* U+4DB7 */
  0x0020,   /* U+4DB8 */
  0x0020,   /* U+4DB9 */
  0x0020,   /* U+4DBA */
  0x0020,   /* U+4DBB */
  0x0020,   /* U+4DBC */
  0x0020,   /* U+4DBD */
  0x0020,   /* U+4DBE */
  0x0020,   /* U+4DBF */
  0x0020,   /* U+4DC0 */
  0x0020,   /* U+4DC1 */
  0x0020,   /* U+4DC2 */
  0x0020,   /* U+4DC3 */
  0x0020,   /* U+4DC4 */
  0x0020,   /* U+4DC5 */
  0x0020,   /* U+4DC6 */
  0x0020,   /* U+4DC7 */
  0x0020,   /* U+4DC8 */
  0x0020,   /* U+4DC9 */
  0x0020,   /* U+4DCA */
  0x0020,   /* U+4DCB */
  0x0020,   /* U+4DCC */
  0x0020,   /* U+4DCD */
  0x0020,   /* U+4DCE */
  0x0020,   /* U+4DCF */
  0x0020,   /* U+4DD0 */
  0x0020,   /* U+4DD1 */
  0x0020,   /* U+4DD2 */
  0x0020,   /* U+4DD3 */
  0x0020,   /* U+4DD4 */
  0x0020,   /* U+4DD5 */
  0x0020,   /* U+4DD6 */
  0x0020,   /* U+4DD7 */
  0x0020,   /* U+4DD8 */
  0x0020,   /* U+4DD9 */
  0x0020,   /* U+4DDA */
  0x0020,   /* U+4DDB */
  0x0020,   /* U+4DDC */
  0x0020,   /* U+4DDD */
  0x0020,   /* U+4DDE */
  0x0020,   /* U+4DDF */
  0x0020,   /* U+4DE0 */
  0x0020,   /* U+4DE1 */
  0x0020,   /* U+4DE2 */
  0x0020,   /* U+4DE3 */
  0x0020,   /* U+4DE4 */
  0x0020,   /* U+4DE5 */
  0x0020,   /* U+4DE6 */
  0x0020,   /* U+4DE7 */
  0x0020,   /* U+4DE8 */
  0x0020,   /* U+4DE9 */
  0x0020,   /* U+4DEA */
  0x0020,   /* U+4DEB */
  0x0020,   /* U+4DEC */
  0x0020,   /* U+4DED */
  0x0020,   /* U+4DEE */
  0x0020,   /* U+4DEF */
  0x0020,   /* U+4DF0 */
  0x0020,   /* U+4DF1 */
  0x0020,   /* U+4DF2 */
  0x0020,   /* U+4DF3 */
  0x0020,   /* U+4DF4 */
  0x0020,   /* U+4DF5 */
  0x0020,   /* U+4DF6 */
  0x0020,   /* U+4DF7 */
  0x0020,   /* U+4DF8 */
  0x0020,   /* U+4DF9 */
  0x0020,   /* U+4DFA */
  0x0020,   /* U+4DFB */
  0x0020,   /* U+4DFC */
  0x0020,   /* U+4DFD */
  0x0020,   /* U+4DFE */
  0x0020,   /* U+4DFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+4D00 */
  0x0002,   /* U+4D01 */
  0x0002,   /* U+4D02 */
  0x0002,   /* U+4D03 */
  0x0002,   /* U+4D04 */
  0x0002,   /* U+4D05 */
  0x0002,   /* U+4D06 */
  0x0002,   /* U+4D07 */
  0x0002,   /* U+4D08 */
  0x0002,   /* U+4D09 */
  0x0002,   /* U+4D0A */
  0x0002,   /* U+4D0B */
  0x0002,   /* U+4D0C */
  0x0002,   /* U+4D0D */
  0x0002,   /* U+4D0E */
  0x0002,   /* U+4D0F */
  0x0002,   /* U+4D10 */
  0x0002,   /* U+4D11 */
  0x0002,   /* U+4D12 */
  0x0002,   /* U+4D13 */
  0x0002,   /* U+4D14 */
  0x0002,   /* U+4D15 */
  0x0002,   /* U+4D16 */
  0x0002,   /* U+4D17 */
  0x0002,   /* U+4D18 */
  0x0002,   /* U+4D19 */
  0x0002,   /* U+4D1A */
  0x0002,   /* U+4D1B */
  0x0002,   /* U+4D1C */
  0x0002,   /* U+4D1D */
  0x0002,   /* U+4D1E */
  0x0002,   /* U+4D1F */
  0x0002,   /* U+4D20 */
  0x0002,   /* U+4D21 */
  0x0002,   /* U+4D22 */
  0x0002,   /* U+4D23 */
  0x0002,   /* U+4D24 */
  0x0002,   /* U+4D25 */
  0x0002,   /* U+4D26 */
  0x0002,   /* U+4D27 */
  0x0002,   /* U+4D28 */
  0x0002,   /* U+4D29 */
  0x0002,   /* U+4D2A */
  0x0002,   /* U+4D2B */
  0x0002,   /* U+4D2C */
  0x0002,   /* U+4D2D */
  0x0002,   /* U+4D2E */
  0x0002,   /* U+4D2F */
  0x0002,   /* U+4D30 */
  0x0002,   /* U+4D31 */
  0x0002,   /* U+4D32 */
  0x0002,   /* U+4D33 */
  0x0002,   /* U+4D34 */
  0x0002,   /* U+4D35 */
  0x0002,   /* U+4D36 */
  0x0002,   /* U+4D37 */
  0x0002,   /* U+4D38 */
  0x0002,   /* U+4D39 */
  0x0002,   /* U+4D3A */
  0x0002,   /* U+4D3B */
  0x0002,   /* U+4D3C */
  0x0002,   /* U+4D3D */
  0x0002,   /* U+4D3E */
  0x0002,   /* U+4D3F */
  0x0002,   /* U+4D40 */
  0x0002,   /* U+4D41 */
  0x0002,   /* U+4D42 */
  0x0002,   /* U+4D43 */
  0x0002,   /* U+4D44 */
  0x0002,   /* U+4D45 */
  0x0002,   /* U+4D46 */
  0x0002,   /* U+4D47 */
  0x0002,   /* U+4D48 */
  0x0002,   /* U+4D49 */
  0x0002,   /* U+4D4A */
  0x0002,   /* U+4D4B */
  0x0002,   /* U+4D4C */
  0x0002,   /* U+4D4D */
  0x0002,   /* U+4D4E */
  0x0002,   /* U+4D4F */
  0x0002,   /* U+4D50 */
  0x0002,   /* U+4D51 */
  0x0002,   /* U+4D52 */
  0x0002,   /* U+4D53 */
  0x0002,   /* U+4D54 */
  0x0002,   /* U+4D55 */
  0x0002,   /* U+4D56 */
  0x0002,   /* U+4D57 */
  0x0002,   /* U+4D58 */
  0x0002,   /* U+4D59 */
  0x0002,   /* U+4D5A */
  0x0002,   /* U+4D5B */
  0x0002,   /* U+4D5C */
  0x0002,   /* U+4D5D */
  0x0002,   /* U+4D5E */
  0x0002,   /* U+4D5F */
  0x0002,   /* U+4D60 */
  0x0002,   /* U+4D61 */
  0x0002,   /* U+4D62 */
  0x0002,   /* U+4D63 */
  0x0002,   /* U+4D64 */
  0x0002,   /* U+4D65 */
  0x0002,   /* U+4D66 */
  0x0002,   /* U+4D67 */
  0x0002,   /* U+4D68 */
  0x0002,   /* U+4D69 */
  0x0002,   /* U+4D6A */
  0x0002,   /* U+4D6B */
  0x0002,   /* U+4D6C */
  0x0002,   /* U+4D6D */
  0x0002,   /* U+4D6E */
  0x0002,   /* U+4D6F */
  0x0002,   /* U+4D70 */
  0x0002,   /* U+4D71 */
  0x0002,   /* U+4D72 */
  0x0002,   /* U+4D73 */
  0x0002,   /* U+4D74 */
  0x0002,   /* U+4D75 */
  0x0002,   /* U+4D76 */
  0x0002,   /* U+4D77 */
  0x0002,   /* U+4D78 */
  0x0002,   /* U+4D79 */
  0x0002,   /* U+4D7A */
  0x0002,   /* U+4D7B */
  0x0002,   /* U+4D7C */
  0x0002,   /* U+4D7D */
  0x0002,   /* U+4D7E */
  0x0002,   /* U+4D7F */
  0x0002,   /* U+4D80 */
  0x0002,   /* U+4D81 */
  0x0002,   /* U+4D82 */
  0x0002,   /* U+4D83 */
  0x0002,   /* U+4D84 */
  0x0002,   /* U+4D85 */
  0x0002,   /* U+4D86 */
  0x0002,   /* U+4D87 */
  0x0002,   /* U+4D88 */
  0x0002,   /* U+4D89 */
  0x0002,   /* U+4D8A */
  0x0002,   /* U+4D8B */
  0x0002,   /* U+4D8C */
  0x0002,   /* U+4D8D */
  0x0002,   /* U+4D8E */
  0x0002,   /* U+4D8F */
  0x0002,   /* U+4D90 */
  0x0002,   /* U+4D91 */
  0x0002,   /* U+4D92 */
  0x0002,   /* U+4D93 */
  0x0002,   /* U+4D94 */
  0x0002,   /* U+4D95 */
  0x0002,   /* U+4D96 */
  0x0002,   /* U+4D97 */
  0x0002,   /* U+4D98 */
  0x0002,   /* U+4D99 */
  0x0002,   /* U+4D9A */
  0x0002,   /* U+4D9B */
  0x0002,   /* U+4D9C */
  0x0002,   /* U+4D9D */
  0x0002,   /* U+4D9E */
  0x0002,   /* U+4D9F */
  0x0002,   /* U+4DA0 */
  0x0002,   /* U+4DA1 */
  0x0002,   /* U+4DA2 */
  0x0002,   /* U+4DA3 */
  0x0002,   /* U+4DA4 */
  0x0002,   /* U+4DA5 */
  0x0002,   /* U+4DA6 */
  0x0002,   /* U+4DA7 */
  0x0002,   /* U+4DA8 */
  0x0002,   /* U+4DA9 */
  0x0002,   /* U+4DAA */
  0x0002,   /* U+4DAB */
  0x0002,   /* U+4DAC */
  0x0002,   /* U+4DAD */
  0x0002,   /* U+4DAE */
  0x0002,   /* U+4DAF */
  0x0002,   /* U+4DB0 */
  0x0002,   /* U+4DB1 */
  0x0002,   /* U+4DB2 */
  0x0002,   /* U+4DB3 */
  0x0002,   /* U+4DB4 */
  0x0002,   /* U+4DB5 */
  0x0002,   /* U+4DB6 */
  0x0002,   /* U+4DB7 */
  0x0002,   /* U+4DB8 */
  0x0002,   /* U+4DB9 */
  0x0002,   /* U+4DBA */
  0x0002,   /* U+4DBB */
  0x0002,   /* U+4DBC */
  0x0002,   /* U+4DBD */
  0x0002,   /* U+4DBE */
  0x0002,   /* U+4DBF */
  0x0002,   /* U+4DC0 */
  0x0002,   /* U+4DC1 */
  0x0002,   /* U+4DC2 */
  0x0002,   /* U+4DC3 */
  0x0002,   /* U+4DC4 */
  0x0002,   /* U+4DC5 */
  0x0002,   /* U+4DC6 */
  0x0002,   /* U+4DC7 */
  0x0002,   /* U+4DC8 */
  0x0002,   /* U+4DC9 */
  0x0002,   /* U+4DCA */
  0x0002,   /* U+4DCB */
  0x0002,   /* U+4DCC */
  0x0002,   /* U+4DCD */
  0x0002,   /* U+4DCE */
  0x0002,   /* U+4DCF */
  0x0002,   /* U+4DD0 */
  0x0002,   /* U+4DD1 */
  0x0002,   /* U+4DD2 */
  0x0002,   /* U+4DD3 */
  0x0002,   /* U+4DD4 */
  0x0002,   /* U+4DD5 */
  0x0002,   /* U+4DD6 */
  0x0002,   /* U+4DD7 */
  0x0002,   /* U+4DD8 */
  0x0002,   /* U+4DD9 */
  0x0002,   /* U+4DDA */
  0x0002,   /* U+4DDB */
  0x0002,   /* U+4DDC */
  0x0002,   /* U+4DDD */
  0x0002,   /* U+4DDE */
  0x0002,   /* U+4DDF */
  0x0002,   /* U+4DE0 */
  0x0002,   /* U+4DE1 */
  0x0002,   /* U+4DE2 */
  0x0002,   /* U+4DE3 */
  0x0002,   /* U+4DE4 */
  0x0002,   /* U+4DE5 */
  0x0002,   /* U+4DE6 */
  0x0002,   /* U+4DE7 */
  0x0002,   /* U+4DE8 */
  0x0002,   /* U+4DE9 */
  0x0002,   /* U+4DEA */
  0x0002,   /* U+4DEB */
  0x0002,   /* U+4DEC */
  0x0002,   /* U+4DED */
  0x0002,   /* U+4DEE */
  0x0002,   /* U+4DEF */
  0x0002,   /* U+4DF0 */
  0x0002,   /* U+4DF1 */
  0x0002,   /* U+4DF2 */
  0x0002,   /* U+4DF3 */
  0x0002,   /* U+4DF4 */
  0x0002,   /* U+4DF5 */
  0x0002,   /* U+4DF6 */
  0x0002,   /* U+4DF7 */
  0x0002,   /* U+4DF8 */
  0x0002,   /* U+4DF9 */
  0x0002,   /* U+4DFA */
  0x0002,   /* U+4DFB */
  0x0002,   /* U+4DFC */
  0x0002,   /* U+4DFD */
  0x0002,   /* U+4DFE */
  0x0002,   /* U+4DFF */

  /* Primary weight 2 for each character. */
  0xCD00,   /* U+4D00 */
  0xCD01,   /* U+4D01 */
  0xCD02,   /* U+4D02 */
  0xCD03,   /* U+4D03 */
  0xCD04,   /* U+4D04 */
  0xCD05,   /* U+4D05 */
  0xCD06,   /* U+4D06 */
  0xCD07,   /* U+4D07 */
  0xCD08,   /* U+4D08 */
  0xCD09,   /* U+4D09 */
  0xCD0A,   /* U+4D0A */
  0xCD0B,   /* U+4D0B */
  0xCD0C,   /* U+4D0C */
  0xCD0D,   /* U+4D0D */
  0xCD0E,   /* U+4D0E */
  0xCD0F,   /* U+4D0F */
  0xCD10,   /* U+4D10 */
  0xCD11,   /* U+4D11 */
  0xCD12,   /* U+4D12 */
  0xCD13,   /* U+4D13 */
  0xCD14,   /* U+4D14 */
  0xCD15,   /* U+4D15 */
  0xCD16,   /* U+4D16 */
  0xCD17,   /* U+4D17 */
  0xCD18,   /* U+4D18 */
  0xCD19,   /* U+4D19 */
  0xCD1A,   /* U+4D1A */
  0xCD1B,   /* U+4D1B */
  0xCD1C,   /* U+4D1C */
  0xCD1D,   /* U+4D1D */
  0xCD1E,   /* U+4D1E */
  0xCD1F,   /* U+4D1F */
  0xCD20,   /* U+4D20 */
  0xCD21,   /* U+4D21 */
  0xCD22,   /* U+4D22 */
  0xCD23,   /* U+4D23 */
  0xCD24,   /* U+4D24 */
  0xCD25,   /* U+4D25 */
  0xCD26,   /* U+4D26 */
  0xCD27,   /* U+4D27 */
  0xCD28,   /* U+4D28 */
  0xCD29,   /* U+4D29 */
  0xCD2A,   /* U+4D2A */
  0xCD2B,   /* U+4D2B */
  0xCD2C,   /* U+4D2C */
  0xCD2D,   /* U+4D2D */
  0xCD2E,   /* U+4D2E */
  0xCD2F,   /* U+4D2F */
  0xCD30,   /* U+4D30 */
  0xCD31,   /* U+4D31 */
  0xCD32,   /* U+4D32 */
  0xCD33,   /* U+4D33 */
  0xCD34,   /* U+4D34 */
  0xCD35,   /* U+4D35 */
  0xCD36,   /* U+4D36 */
  0xCD37,   /* U+4D37 */
  0xCD38,   /* U+4D38 */
  0xCD39,   /* U+4D39 */
  0xCD3A,   /* U+4D3A */
  0xCD3B,   /* U+4D3B */
  0xCD3C,   /* U+4D3C */
  0xCD3D,   /* U+4D3D */
  0xCD3E,   /* U+4D3E */
  0xCD3F,   /* U+4D3F */
  0xCD40,   /* U+4D40 */
  0xCD41,   /* U+4D41 */
  0xCD42,   /* U+4D42 */
  0xCD43,   /* U+4D43 */
  0xCD44,   /* U+4D44 */
  0xCD45,   /* U+4D45 */
  0xCD46,   /* U+4D46 */
  0xCD47,   /* U+4D47 */
  0xCD48,   /* U+4D48 */
  0xCD49,   /* U+4D49 */
  0xCD4A,   /* U+4D4A */
  0xCD4B,   /* U+4D4B */
  0xCD4C,   /* U+4D4C */
  0xCD4D,   /* U+4D4D */
  0xCD4E,   /* U+4D4E */
  0xCD4F,   /* U+4D4F */
  0xCD50,   /* U+4D50 */
  0xCD51,   /* U+4D51 */
  0xCD52,   /* U+4D52 */
  0xCD53,   /* U+4D53 */
  0xCD54,   /* U+4D54 */
  0xCD55,   /* U+4D55 */
  0xCD56,   /* U+4D56 */
  0xCD57,   /* U+4D57 */
  0xCD58,   /* U+4D58 */
  0xCD59,   /* U+4D59 */
  0xCD5A,   /* U+4D5A */
  0xCD5B,   /* U+4D5B */
  0xCD5C,   /* U+4D5C */
  0xCD5D,   /* U+4D5D */
  0xCD5E,   /* U+4D5E */
  0xCD5F,   /* U+4D5F */
  0xCD60,   /* U+4D60 */
  0xCD61,   /* U+4D61 */
  0xCD62,   /* U+4D62 */
  0xCD63,   /* U+4D63 */
  0xCD64,   /* U+4D64 */
  0xCD65,   /* U+4D65 */
  0xCD66,   /* U+4D66 */
  0xCD67,   /* U+4D67 */
  0xCD68,   /* U+4D68 */
  0xCD69,   /* U+4D69 */
  0xCD6A,   /* U+4D6A */
  0xCD6B,   /* U+4D6B */
  0xCD6C,   /* U+4D6C */
  0xCD6D,   /* U+4D6D */
  0xCD6E,   /* U+4D6E */
  0xCD6F,   /* U+4D6F */
  0xCD70,   /* U+4D70 */
  0xCD71,   /* U+4D71 */
  0xCD72,   /* U+4D72 */
  0xCD73,   /* U+4D73 */
  0xCD74,   /* U+4D74 */
  0xCD75,   /* U+4D75 */
  0xCD76,   /* U+4D76 */
  0xCD77,   /* U+4D77 */
  0xCD78,   /* U+4D78 */
  0xCD79,   /* U+4D79 */
  0xCD7A,   /* U+4D7A */
  0xCD7B,   /* U+4D7B */
  0xCD7C,   /* U+4D7C */
  0xCD7D,   /* U+4D7D */
  0xCD7E,   /* U+4D7E */
  0xCD7F,   /* U+4D7F */
  0xCD80,   /* U+4D80 */
  0xCD81,   /* U+4D81 */
  0xCD82,   /* U+4D82 */
  0xCD83,   /* U+4D83 */
  0xCD84,   /* U+4D84 */
  0xCD85,   /* U+4D85 */
  0xCD86,   /* U+4D86 */
  0xCD87,   /* U+4D87 */
  0xCD88,   /* U+4D88 */
  0xCD89,   /* U+4D89 */
  0xCD8A,   /* U+4D8A */
  0xCD8B,   /* U+4D8B */
  0xCD8C,   /* U+4D8C */
  0xCD8D,   /* U+4D8D */
  0xCD8E,   /* U+4D8E */
  0xCD8F,   /* U+4D8F */
  0xCD90,   /* U+4D90 */
  0xCD91,   /* U+4D91 */
  0xCD92,   /* U+4D92 */
  0xCD93,   /* U+4D93 */
  0xCD94,   /* U+4D94 */
  0xCD95,   /* U+4D95 */
  0xCD96,   /* U+4D96 */
  0xCD97,   /* U+4D97 */
  0xCD98,   /* U+4D98 */
  0xCD99,   /* U+4D99 */
  0xCD9A,   /* U+4D9A */
  0xCD9B,   /* U+4D9B */
  0xCD9C,   /* U+4D9C */
  0xCD9D,   /* U+4D9D */
  0xCD9E,   /* U+4D9E */
  0xCD9F,   /* U+4D9F */
  0xCDA0,   /* U+4DA0 */
  0xCDA1,   /* U+4DA1 */
  0xCDA2,   /* U+4DA2 */
  0xCDA3,   /* U+4DA3 */
  0xCDA4,   /* U+4DA4 */
  0xCDA5,   /* U+4DA5 */
  0xCDA6,   /* U+4DA6 */
  0xCDA7,   /* U+4DA7 */
  0xCDA8,   /* U+4DA8 */
  0xCDA9,   /* U+4DA9 */
  0xCDAA,   /* U+4DAA */
  0xCDAB,   /* U+4DAB */
  0xCDAC,   /* U+4DAC */
  0xCDAD,   /* U+4DAD */
  0xCDAE,   /* U+4DAE */
  0xCDAF,   /* U+4DAF */
  0xCDB0,   /* U+4DB0 */
  0xCDB1,   /* U+4DB1 */
  0xCDB2,   /* U+4DB2 */
  0xCDB3,   /* U+4DB3 */
  0xCDB4,   /* U+4DB4 */
  0xCDB5,   /* U+4DB5 */
  0xCDB6,   /* U+4DB6 */
  0xCDB7,   /* U+4DB7 */
  0xCDB8,   /* U+4DB8 */
  0xCDB9,   /* U+4DB9 */
  0xCDBA,   /* U+4DBA */
  0xCDBB,   /* U+4DBB */
  0xCDBC,   /* U+4DBC */
  0xCDBD,   /* U+4DBD */
  0xCDBE,   /* U+4DBE */
  0xCDBF,   /* U+4DBF */
  0x0000,   /* U+4DC0 */
  0x0000,   /* U+4DC1 */
  0x0000,   /* U+4DC2 */
  0x0000,   /* U+4DC3 */
  0x0000,   /* U+4DC4 */
  0x0000,   /* U+4DC5 */
  0x0000,   /* U+4DC6 */
  0x0000,   /* U+4DC7 */
  0x0000,   /* U+4DC8 */
  0x0000,   /* U+4DC9 */
  0x0000,   /* U+4DCA */
  0x0000,   /* U+4DCB */
  0x0000,   /* U+4DCC */
  0x0000,   /* U+4DCD */
  0x0000,   /* U+4DCE */
  0x0000,   /* U+4DCF */
  0x0000,   /* U+4DD0 */
  0x0000,   /* U+4DD1 */
  0x0000,   /* U+4DD2 */
  0x0000,   /* U+4DD3 */
  0x0000,   /* U+4DD4 */
  0x0000,   /* U+4DD5 */
  0x0000,   /* U+4DD6 */
  0x0000,   /* U+4DD7 */
  0x0000,   /* U+4DD8 */
  0x0000,   /* U+4DD9 */
  0x0000,   /* U+4DDA */
  0x0000,   /* U+4DDB */
  0x0000,   /* U+4DDC */
  0x0000,   /* U+4DDD */
  0x0000,   /* U+4DDE */
  0x0000,   /* U+4DDF */
  0x0000,   /* U+4DE0 */
  0x0000,   /* U+4DE1 */
  0x0000,   /* U+4DE2 */
  0x0000,   /* U+4DE3 */
  0x0000,   /* U+4DE4 */
  0x0000,   /* U+4DE5 */
  0x0000,   /* U+4DE6 */
  0x0000,   /* U+4DE7 */
  0x0000,   /* U+4DE8 */
  0x0000,   /* U+4DE9 */
  0x0000,   /* U+4DEA */
  0x0000,   /* U+4DEB */
  0x0000,   /* U+4DEC */
  0x0000,   /* U+4DED */
  0x0000,   /* U+4DEE */
  0x0000,   /* U+4DEF */
  0x0000,   /* U+4DF0 */
  0x0000,   /* U+4DF1 */
  0x0000,   /* U+4DF2 */
  0x0000,   /* U+4DF3 */
  0x0000,   /* U+4DF4 */
  0x0000,   /* U+4DF5 */
  0x0000,   /* U+4DF6 */
  0x0000,   /* U+4DF7 */
  0x0000,   /* U+4DF8 */
  0x0000,   /* U+4DF9 */
  0x0000,   /* U+4DFA */
  0x0000,   /* U+4DFB */
  0x0000,   /* U+4DFC */
  0x0000,   /* U+4DFD */
  0x0000,   /* U+4DFE */
  0x0000,   /* U+4DFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+4D00 */
  0x0000,   /* U+4D01 */
  0x0000,   /* U+4D02 */
  0x0000,   /* U+4D03 */
  0x0000,   /* U+4D04 */
  0x0000,   /* U+4D05 */
  0x0000,   /* U+4D06 */
  0x0000,   /* U+4D07 */
  0x0000,   /* U+4D08 */
  0x0000,   /* U+4D09 */
  0x0000,   /* U+4D0A */
  0x0000,   /* U+4D0B */
  0x0000,   /* U+4D0C */
  0x0000,   /* U+4D0D */
  0x0000,   /* U+4D0E */
  0x0000,   /* U+4D0F */
  0x0000,   /* U+4D10 */
  0x0000,   /* U+4D11 */
  0x0000,   /* U+4D12 */
  0x0000,   /* U+4D13 */
  0x0000,   /* U+4D14 */
  0x0000,   /* U+4D15 */
  0x0000,   /* U+4D16 */
  0x0000,   /* U+4D17 */
  0x0000,   /* U+4D18 */
  0x0000,   /* U+4D19 */
  0x0000,   /* U+4D1A */
  0x0000,   /* U+4D1B */
  0x0000,   /* U+4D1C */
  0x0000,   /* U+4D1D */
  0x0000,   /* U+4D1E */
  0x0000,   /* U+4D1F */
  0x0000,   /* U+4D20 */
  0x0000,   /* U+4D21 */
  0x0000,   /* U+4D22 */
  0x0000,   /* U+4D23 */
  0x0000,   /* U+4D24 */
  0x0000,   /* U+4D25 */
  0x0000,   /* U+4D26 */
  0x0000,   /* U+4D27 */
  0x0000,   /* U+4D28 */
  0x0000,   /* U+4D29 */
  0x0000,   /* U+4D2A */
  0x0000,   /* U+4D2B */
  0x0000,   /* U+4D2C */
  0x0000,   /* U+4D2D */
  0x0000,   /* U+4D2E */
  0x0000,   /* U+4D2F */
  0x0000,   /* U+4D30 */
  0x0000,   /* U+4D31 */
  0x0000,   /* U+4D32 */
  0x0000,   /* U+4D33 */
  0x0000,   /* U+4D34 */
  0x0000,   /* U+4D35 */
  0x0000,   /* U+4D36 */
  0x0000,   /* U+4D37 */
  0x0000,   /* U+4D38 */
  0x0000,   /* U+4D39 */
  0x0000,   /* U+4D3A */
  0x0000,   /* U+4D3B */
  0x0000,   /* U+4D3C */
  0x0000,   /* U+4D3D */
  0x0000,   /* U+4D3E */
  0x0000,   /* U+4D3F */
  0x0000,   /* U+4D40 */
  0x0000,   /* U+4D41 */
  0x0000,   /* U+4D42 */
  0x0000,   /* U+4D43 */
  0x0000,   /* U+4D44 */
  0x0000,   /* U+4D45 */
  0x0000,   /* U+4D46 */
  0x0000,   /* U+4D47 */
  0x0000,   /* U+4D48 */
  0x0000,   /* U+4D49 */
  0x0000,   /* U+4D4A */
  0x0000,   /* U+4D4B */
  0x0000,   /* U+4D4C */
  0x0000,   /* U+4D4D */
  0x0000,   /* U+4D4E */
  0x0000,   /* U+4D4F */
  0x0000,   /* U+4D50 */
  0x0000,   /* U+4D51 */
  0x0000,   /* U+4D52 */
  0x0000,   /* U+4D53 */
  0x0000,   /* U+4D54 */
  0x0000,   /* U+4D55 */
  0x0000,   /* U+4D56 */
  0x0000,   /* U+4D57 */
  0x0000,   /* U+4D58 */
  0x0000,   /* U+4D59 */
  0x0000,   /* U+4D5A */
  0x0000,   /* U+4D5B */
  0x0000,   /* U+4D5C */
  0x0000,   /* U+4D5D */
  0x0000,   /* U+4D5E */
  0x0000,   /* U+4D5F */
  0x0000,   /* U+4D60 */
  0x0000,   /* U+4D61 */
  0x0000,   /* U+4D62 */
  0x0000,   /* U+4D63 */
  0x0000,   /* U+4D64 */
  0x0000,   /* U+4D65 */
  0x0000,   /* U+4D66 */
  0x0000,   /* U+4D67 */
  0x0000,   /* U+4D68 */
  0x0000,   /* U+4D69 */
  0x0000,   /* U+4D6A */
  0x0000,   /* U+4D6B */
  0x0000,   /* U+4D6C */
  0x0000,   /* U+4D6D */
  0x0000,   /* U+4D6E */
  0x0000,   /* U+4D6F */
  0x0000,   /* U+4D70 */
  0x0000,   /* U+4D71 */
  0x0000,   /* U+4D72 */
  0x0000,   /* U+4D73 */
  0x0000,   /* U+4D74 */
  0x0000,   /* U+4D75 */
  0x0000,   /* U+4D76 */
  0x0000,   /* U+4D77 */
  0x0000,   /* U+4D78 */
  0x0000,   /* U+4D79 */
  0x0000,   /* U+4D7A */
  0x0000,   /* U+4D7B */
  0x0000,   /* U+4D7C */
  0x0000,   /* U+4D7D */
  0x0000,   /* U+4D7E */
  0x0000,   /* U+4D7F */
  0x0000,   /* U+4D80 */
  0x0000,   /* U+4D81 */
  0x0000,   /* U+4D82 */
  0x0000,   /* U+4D83 */
  0x0000,   /* U+4D84 */
  0x0000,   /* U+4D85 */
  0x0000,   /* U+4D86 */
  0x0000,   /* U+4D87 */
  0x0000,   /* U+4D88 */
  0x0000,   /* U+4D89 */
  0x0000,   /* U+4D8A */
  0x0000,   /* U+4D8B */
  0x0000,   /* U+4D8C */
  0x0000,   /* U+4D8D */
  0x0000,   /* U+4D8E */
  0x0000,   /* U+4D8F */
  0x0000,   /* U+4D90 */
  0x0000,   /* U+4D91 */
  0x0000,   /* U+4D92 */
  0x0000,   /* U+4D93 */
  0x0000,   /* U+4D94 */
  0x0000,   /* U+4D95 */
  0x0000,   /* U+4D96 */
  0x0000,   /* U+4D97 */
  0x0000,   /* U+4D98 */
  0x0000,   /* U+4D99 */
  0x0000,   /* U+4D9A */
  0x0000,   /* U+4D9B */
  0x0000,   /* U+4D9C */
  0x0000,   /* U+4D9D */
  0x0000,   /* U+4D9E */
  0x0000,   /* U+4D9F */
  0x0000,   /* U+4DA0 */
  0x0000,   /* U+4DA1 */
  0x0000,   /* U+4DA2 */
  0x0000,   /* U+4DA3 */
  0x0000,   /* U+4DA4 */
  0x0000,   /* U+4DA5 */
  0x0000,   /* U+4DA6 */
  0x0000,   /* U+4DA7 */
  0x0000,   /* U+4DA8 */
  0x0000,   /* U+4DA9 */
  0x0000,   /* U+4DAA */
  0x0000,   /* U+4DAB */
  0x0000,   /* U+4DAC */
  0x0000,   /* U+4DAD */
  0x0000,   /* U+4DAE */
  0x0000,   /* U+4DAF */
  0x0000,   /* U+4DB0 */
  0x0000,   /* U+4DB1 */
  0x0000,   /* U+4DB2 */
  0x0000,   /* U+4DB3 */
  0x0000,   /* U+4DB4 */
  0x0000,   /* U+4DB5 */
  0x0000,   /* U+4DB6 */
  0x0000,   /* U+4DB7 */
  0x0000,   /* U+4DB8 */
  0x0000,   /* U+4DB9 */
  0x0000,   /* U+4DBA */
  0x0000,   /* U+4DBB */
  0x0000,   /* U+4DBC */
  0x0000,   /* U+4DBD */
  0x0000,   /* U+4DBE */
  0x0000,   /* U+4DBF */
  0x0000,   /* U+4DC0 */
  0x0000,   /* U+4DC1 */
  0x0000,   /* U+4DC2 */
  0x0000,   /* U+4DC3 */
  0x0000,   /* U+4DC4 */
  0x0000,   /* U+4DC5 */
  0x0000,   /* U+4DC6 */
  0x0000,   /* U+4DC7 */
  0x0000,   /* U+4DC8 */
  0x0000,   /* U+4DC9 */
  0x0000,   /* U+4DCA */
  0x0000,   /* U+4DCB */
  0x0000,   /* U+4DCC */
  0x0000,   /* U+4DCD */
  0x0000,   /* U+4DCE */
  0x0000,   /* U+4DCF */
  0x0000,   /* U+4DD0 */
  0x0000,   /* U+4DD1 */
  0x0000,   /* U+4DD2 */
  0x0000,   /* U+4DD3 */
  0x0000,   /* U+4DD4 */
  0x0000,   /* U+4DD5 */
  0x0000,   /* U+4DD6 */
  0x0000,   /* U+4DD7 */
  0x0000,   /* U+4DD8 */
  0x0000,   /* U+4DD9 */
  0x0000,   /* U+4DDA */
  0x0000,   /* U+4DDB */
  0x0000,   /* U+4DDC */
  0x0000,   /* U+4DDD */
  0x0000,   /* U+4DDE */
  0x0000,   /* U+4DDF */
  0x0000,   /* U+4DE0 */
  0x0000,   /* U+4DE1 */
  0x0000,   /* U+4DE2 */
  0x0000,   /* U+4DE3 */
  0x0000,   /* U+4DE4 */
  0x0000,   /* U+4DE5 */
  0x0000,   /* U+4DE6 */
  0x0000,   /* U+4DE7 */
  0x0000,   /* U+4DE8 */
  0x0000,   /* U+4DE9 */
  0x0000,   /* U+4DEA */
  0x0000,   /* U+4DEB */
  0x0000,   /* U+4DEC */
  0x0000,   /* U+4DED */
  0x0000,   /* U+4DEE */
  0x0000,   /* U+4DEF */
  0x0000,   /* U+4DF0 */
  0x0000,   /* U+4DF1 */
  0x0000,   /* U+4DF2 */
  0x0000,   /* U+4DF3 */
  0x0000,   /* U+4DF4 */
  0x0000,   /* U+4DF5 */
  0x0000,   /* U+4DF6 */
  0x0000,   /* U+4DF7 */
  0x0000,   /* U+4DF8 */
  0x0000,   /* U+4DF9 */
  0x0000,   /* U+4DFA */
  0x0000,   /* U+4DFB */
  0x0000,   /* U+4DFC */
  0x0000,   /* U+4DFD */
  0x0000,   /* U+4DFE */
  0x0000,   /* U+4DFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+4D00 */
  0x0000,   /* U+4D01 */
  0x0000,   /* U+4D02 */
  0x0000,   /* U+4D03 */
  0x0000,   /* U+4D04 */
  0x0000,   /* U+4D05 */
  0x0000,   /* U+4D06 */
  0x0000,   /* U+4D07 */
  0x0000,   /* U+4D08 */
  0x0000,   /* U+4D09 */
  0x0000,   /* U+4D0A */
  0x0000,   /* U+4D0B */
  0x0000,   /* U+4D0C */
  0x0000,   /* U+4D0D */
  0x0000,   /* U+4D0E */
  0x0000,   /* U+4D0F */
  0x0000,   /* U+4D10 */
  0x0000,   /* U+4D11 */
  0x0000,   /* U+4D12 */
  0x0000,   /* U+4D13 */
  0x0000,   /* U+4D14 */
  0x0000,   /* U+4D15 */
  0x0000,   /* U+4D16 */
  0x0000,   /* U+4D17 */
  0x0000,   /* U+4D18 */
  0x0000,   /* U+4D19 */
  0x0000,   /* U+4D1A */
  0x0000,   /* U+4D1B */
  0x0000,   /* U+4D1C */
  0x0000,   /* U+4D1D */
  0x0000,   /* U+4D1E */
  0x0000,   /* U+4D1F */
  0x0000,   /* U+4D20 */
  0x0000,   /* U+4D21 */
  0x0000,   /* U+4D22 */
  0x0000,   /* U+4D23 */
  0x0000,   /* U+4D24 */
  0x0000,   /* U+4D25 */
  0x0000,   /* U+4D26 */
  0x0000,   /* U+4D27 */
  0x0000,   /* U+4D28 */
  0x0000,   /* U+4D29 */
  0x0000,   /* U+4D2A */
  0x0000,   /* U+4D2B */
  0x0000,   /* U+4D2C */
  0x0000,   /* U+4D2D */
  0x0000,   /* U+4D2E */
  0x0000,   /* U+4D2F */
  0x0000,   /* U+4D30 */
  0x0000,   /* U+4D31 */
  0x0000,   /* U+4D32 */
  0x0000,   /* U+4D33 */
  0x0000,   /* U+4D34 */
  0x0000,   /* U+4D35 */
  0x0000,   /* U+4D36 */
  0x0000,   /* U+4D37 */
  0x0000,   /* U+4D38 */
  0x0000,   /* U+4D39 */
  0x0000,   /* U+4D3A */
  0x0000,   /* U+4D3B */
  0x0000,   /* U+4D3C */
  0x0000,   /* U+4D3D */
  0x0000,   /* U+4D3E */
  0x0000,   /* U+4D3F */
  0x0000,   /* U+4D40 */
  0x0000,   /* U+4D41 */
  0x0000,   /* U+4D42 */
  0x0000,   /* U+4D43 */
  0x0000,   /* U+4D44 */
  0x0000,   /* U+4D45 */
  0x0000,   /* U+4D46 */
  0x0000,   /* U+4D47 */
  0x0000,   /* U+4D48 */
  0x0000,   /* U+4D49 */
  0x0000,   /* U+4D4A */
  0x0000,   /* U+4D4B */
  0x0000,   /* U+4D4C */
  0x0000,   /* U+4D4D */
  0x0000,   /* U+4D4E */
  0x0000,   /* U+4D4F */
  0x0000,   /* U+4D50 */
  0x0000,   /* U+4D51 */
  0x0000,   /* U+4D52 */
  0x0000,   /* U+4D53 */
  0x0000,   /* U+4D54 */
  0x0000,   /* U+4D55 */
  0x0000,   /* U+4D56 */
  0x0000,   /* U+4D57 */
  0x0000,   /* U+4D58 */
  0x0000,   /* U+4D59 */
  0x0000,   /* U+4D5A */
  0x0000,   /* U+4D5B */
  0x0000,   /* U+4D5C */
  0x0000,   /* U+4D5D */
  0x0000,   /* U+4D5E */
  0x0000,   /* U+4D5F */
  0x0000,   /* U+4D60 */
  0x0000,   /* U+4D61 */
  0x0000,   /* U+4D62 */
  0x0000,   /* U+4D63 */
  0x0000,   /* U+4D64 */
  0x0000,   /* U+4D65 */
  0x0000,   /* U+4D66 */
  0x0000,   /* U+4D67 */
  0x0000,   /* U+4D68 */
  0x0000,   /* U+4D69 */
  0x0000,   /* U+4D6A */
  0x0000,   /* U+4D6B */
  0x0000,   /* U+4D6C */
  0x0000,   /* U+4D6D */
  0x0000,   /* U+4D6E */
  0x0000,   /* U+4D6F */
  0x0000,   /* U+4D70 */
  0x0000,   /* U+4D71 */
  0x0000,   /* U+4D72 */
  0x0000,   /* U+4D73 */
  0x0000,   /* U+4D74 */
  0x0000,   /* U+4D75 */
  0x0000,   /* U+4D76 */
  0x0000,   /* U+4D77 */
  0x0000,   /* U+4D78 */
  0x0000,   /* U+4D79 */
  0x0000,   /* U+4D7A */
  0x0000,   /* U+4D7B */
  0x0000,   /* U+4D7C */
  0x0000,   /* U+4D7D */
  0x0000,   /* U+4D7E */
  0x0000,   /* U+4D7F */
  0x0000,   /* U+4D80 */
  0x0000,   /* U+4D81 */
  0x0000,   /* U+4D82 */
  0x0000,   /* U+4D83 */
  0x0000,   /* U+4D84 */
  0x0000,   /* U+4D85 */
  0x0000,   /* U+4D86 */
  0x0000,   /* U+4D87 */
  0x0000,   /* U+4D88 */
  0x0000,   /* U+4D89 */
  0x0000,   /* U+4D8A */
  0x0000,   /* U+4D8B */
  0x0000,   /* U+4D8C */
  0x0000,   /* U+4D8D */
  0x0000,   /* U+4D8E */
  0x0000,   /* U+4D8F */
  0x0000,   /* U+4D90 */
  0x0000,   /* U+4D91 */
  0x0000,   /* U+4D92 */
  0x0000,   /* U+4D93 */
  0x0000,   /* U+4D94 */
  0x0000,   /* U+4D95 */
  0x0000,   /* U+4D96 */
  0x0000,   /* U+4D97 */
  0x0000,   /* U+4D98 */
  0x0000,   /* U+4D99 */
  0x0000,   /* U+4D9A */
  0x0000,   /* U+4D9B */
  0x0000,   /* U+4D9C */
  0x0000,   /* U+4D9D */
  0x0000,   /* U+4D9E */
  0x0000,   /* U+4D9F */
  0x0000,   /* U+4DA0 */
  0x0000,   /* U+4DA1 */
  0x0000,   /* U+4DA2 */
  0x0000,   /* U+4DA3 */
  0x0000,   /* U+4DA4 */
  0x0000,   /* U+4DA5 */
  0x0000,   /* U+4DA6 */
  0x0000,   /* U+4DA7 */
  0x0000,   /* U+4DA8 */
  0x0000,   /* U+4DA9 */
  0x0000,   /* U+4DAA */
  0x0000,   /* U+4DAB */
  0x0000,   /* U+4DAC */
  0x0000,   /* U+4DAD */
  0x0000,   /* U+4DAE */
  0x0000,   /* U+4DAF */
  0x0000,   /* U+4DB0 */
  0x0000,   /* U+4DB1 */
  0x0000,   /* U+4DB2 */
  0x0000,   /* U+4DB3 */
  0x0000,   /* U+4DB4 */
  0x0000,   /* U+4DB5 */
  0x0000,   /* U+4DB6 */
  0x0000,   /* U+4DB7 */
  0x0000,   /* U+4DB8 */
  0x0000,   /* U+4DB9 */
  0x0000,   /* U+4DBA */
  0x0000,   /* U+4DBB */
  0x0000,   /* U+4DBC */
  0x0000,   /* U+4DBD */
  0x0000,   /* U+4DBE */
  0x0000,   /* U+4DBF */
  0x0000,   /* U+4DC0 */
  0x0000,   /* U+4DC1 */
  0x0000,   /* U+4DC2 */
  0x0000,   /* U+4DC3 */
  0x0000,   /* U+4DC4 */
  0x0000,   /* U+4DC5 */
  0x0000,   /* U+4DC6 */
  0x0000,   /* U+4DC7 */
  0x0000,   /* U+4DC8 */
  0x0000,   /* U+4DC9 */
  0x0000,   /* U+4DCA */
  0x0000,   /* U+4DCB */
  0x0000,   /* U+4DCC */
  0x0000,   /* U+4DCD */
  0x0000,   /* U+4DCE */
  0x0000,   /* U+4DCF */
  0x0000,   /* U+4DD0 */
  0x0000,   /* U+4DD1 */
  0x0000,   /* U+4DD2 */
  0x0000,   /* U+4DD3 */
  0x0000,   /* U+4DD4 */
  0x0000,   /* U+4DD5 */
  0x0000,   /* U+4DD6 */
  0x0000,   /* U+4DD7 */
  0x0000,   /* U+4DD8 */
  0x0000,   /* U+4DD9 */
  0x0000,   /* U+4DDA */
  0x0000,   /* U+4DDB */
  0x0000,   /* U+4DDC */
  0x0000,   /* U+4DDD */
  0x0000,   /* U+4DDE */
  0x0000,   /* U+4DDF */
  0x0000,   /* U+4DE0 */
  0x0000,   /* U+4DE1 */
  0x0000,   /* U+4DE2 */
  0x0000,   /* U+4DE3 */
  0x0000,   /* U+4DE4 */
  0x0000,   /* U+4DE5 */
  0x0000,   /* U+4DE6 */
  0x0000,   /* U+4DE7 */
  0x0000,   /* U+4DE8 */
  0x0000,   /* U+4DE9 */
  0x0000,   /* U+4DEA */
  0x0000,   /* U+4DEB */
  0x0000,   /* U+4DEC */
  0x0000,   /* U+4DED */
  0x0000,   /* U+4DEE */
  0x0000,   /* U+4DEF */
  0x0000,   /* U+4DF0 */
  0x0000,   /* U+4DF1 */
  0x0000,   /* U+4DF2 */
  0x0000,   /* U+4DF3 */
  0x0000,   /* U+4DF4 */
  0x0000,   /* U+4DF5 */
  0x0000,   /* U+4DF6 */
  0x0000,   /* U+4DF7 */
  0x0000,   /* U+4DF8 */
  0x0000,   /* U+4DF9 */
  0x0000,   /* U+4DFA */
  0x0000,   /* U+4DFB */
  0x0000,   /* U+4DFC */
  0x0000,   /* U+4DFD */
  0x0000,   /* U+4DFE */
  0x0000,   /* U+4DFF */
};

uint16 uca900_p0A0[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x3DBF,   /* U+A000 */
  0x3DC0,   /* U+A001 */
  0x3DC1,   /* U+A002 */
  0x3DC2,   /* U+A003 */
  0x3DC3,   /* U+A004 */
  0x3DC4,   /* U+A005 */
  0x3DC5,   /* U+A006 */
  0x3DC6,   /* U+A007 */
  0x3DC7,   /* U+A008 */
  0x3DC8,   /* U+A009 */
  0x3DC9,   /* U+A00A */
  0x3DCA,   /* U+A00B */
  0x3DCB,   /* U+A00C */
  0x3DCC,   /* U+A00D */
  0x3DCD,   /* U+A00E */
  0x3DCE,   /* U+A00F */
  0x3DCF,   /* U+A010 */
  0x3DD0,   /* U+A011 */
  0x3DD1,   /* U+A012 */
  0x3DD2,   /* U+A013 */
  0x3DD3,   /* U+A014 */
  0x3DD4,   /* U+A015 */
  0x3DD5,   /* U+A016 */
  0x3DD6,   /* U+A017 */
  0x3DD7,   /* U+A018 */
  0x3DD8,   /* U+A019 */
  0x3DD9,   /* U+A01A */
  0x3DDA,   /* U+A01B */
  0x3DDB,   /* U+A01C */
  0x3DDC,   /* U+A01D */
  0x3DDD,   /* U+A01E */
  0x3DDE,   /* U+A01F */
  0x3DDF,   /* U+A020 */
  0x3DE0,   /* U+A021 */
  0x3DE1,   /* U+A022 */
  0x3DE2,   /* U+A023 */
  0x3DE3,   /* U+A024 */
  0x3DE4,   /* U+A025 */
  0x3DE5,   /* U+A026 */
  0x3DE6,   /* U+A027 */
  0x3DE7,   /* U+A028 */
  0x3DE8,   /* U+A029 */
  0x3DE9,   /* U+A02A */
  0x3DEA,   /* U+A02B */
  0x3DEB,   /* U+A02C */
  0x3DEC,   /* U+A02D */
  0x3DED,   /* U+A02E */
  0x3DEE,   /* U+A02F */
  0x3DEF,   /* U+A030 */
  0x3DF0,   /* U+A031 */
  0x3DF1,   /* U+A032 */
  0x3DF2,   /* U+A033 */
  0x3DF3,   /* U+A034 */
  0x3DF4,   /* U+A035 */
  0x3DF5,   /* U+A036 */
  0x3DF6,   /* U+A037 */
  0x3DF7,   /* U+A038 */
  0x3DF8,   /* U+A039 */
  0x3DF9,   /* U+A03A */
  0x3DFA,   /* U+A03B */
  0x3DFB,   /* U+A03C */
  0x3DFC,   /* U+A03D */
  0x3DFD,   /* U+A03E */
  0x3DFE,   /* U+A03F */
  0x3DFF,   /* U+A040 */
  0x3E00,   /* U+A041 */
  0x3E01,   /* U+A042 */
  0x3E02,   /* U+A043 */
  0x3E03,   /* U+A044 */
  0x3E04,   /* U+A045 */
  0x3E05,   /* U+A046 */
  0x3E06,   /* U+A047 */
  0x3E07,   /* U+A048 */
  0x3E08,   /* U+A049 */
  0x3E09,   /* U+A04A */
  0x3E0A,   /* U+A04B */
  0x3E0B,   /* U+A04C */
  0x3E0C,   /* U+A04D */
  0x3E0D,   /* U+A04E */
  0x3E0E,   /* U+A04F */
  0x3E0F,   /* U+A050 */
  0x3E10,   /* U+A051 */
  0x3E11,   /* U+A052 */
  0x3E12,   /* U+A053 */
  0x3E13,   /* U+A054 */
  0x3E14,   /* U+A055 */
  0x3E15,   /* U+A056 */
  0x3E16,   /* U+A057 */
  0x3E17,   /* U+A058 */
  0x3E18,   /* U+A059 */
  0x3E19,   /* U+A05A */
  0x3E1A,   /* U+A05B */
  0x3E1B,   /* U+A05C */
  0x3E1C,   /* U+A05D */
  0x3E1D,   /* U+A05E */
  0x3E1E,   /* U+A05F */
  0x3E1F,   /* U+A060 */
  0x3E20,   /* U+A061 */
  0x3E21,   /* U+A062 */
  0x3E22,   /* U+A063 */
  0x3E23,   /* U+A064 */
  0x3E24,   /* U+A065 */
  0x3E25,   /* U+A066 */
  0x3E26,   /* U+A067 */
  0x3E27,   /* U+A068 */
  0x3E28,   /* U+A069 */
  0x3E29,   /* U+A06A */
  0x3E2A,   /* U+A06B */
  0x3E2B,   /* U+A06C */
  0x3E2C,   /* U+A06D */
  0x3E2D,   /* U+A06E */
  0x3E2E,   /* U+A06F */
  0x3E2F,   /* U+A070 */
  0x3E30,   /* U+A071 */
  0x3E31,   /* U+A072 */
  0x3E32,   /* U+A073 */
  0x3E33,   /* U+A074 */
  0x3E34,   /* U+A075 */
  0x3E35,   /* U+A076 */
  0x3E36,   /* U+A077 */
  0x3E37,   /* U+A078 */
  0x3E38,   /* U+A079 */
  0x3E39,   /* U+A07A */
  0x3E3A,   /* U+A07B */
  0x3E3B,   /* U+A07C */
  0x3E3C,   /* U+A07D */
  0x3E3D,   /* U+A07E */
  0x3E3E,   /* U+A07F */
  0x3E3F,   /* U+A080 */
  0x3E40,   /* U+A081 */
  0x3E41,   /* U+A082 */
  0x3E42,   /* U+A083 */
  0x3E43,   /* U+A084 */
  0x3E44,   /* U+A085 */
  0x3E45,   /* U+A086 */
  0x3E46,   /* U+A087 */
  0x3E47,   /* U+A088 */
  0x3E48,   /* U+A089 */
  0x3E49,   /* U+A08A */
  0x3E4A,   /* U+A08B */
  0x3E4B,   /* U+A08C */
  0x3E4C,   /* U+A08D */
  0x3E4D,   /* U+A08E */
  0x3E4E,   /* U+A08F */
  0x3E4F,   /* U+A090 */
  0x3E50,   /* U+A091 */
  0x3E51,   /* U+A092 */
  0x3E52,   /* U+A093 */
  0x3E53,   /* U+A094 */
  0x3E54,   /* U+A095 */
  0x3E55,   /* U+A096 */
  0x3E56,   /* U+A097 */
  0x3E57,   /* U+A098 */
  0x3E58,   /* U+A099 */
  0x3E59,   /* U+A09A */
  0x3E5A,   /* U+A09B */
  0x3E5B,   /* U+A09C */
  0x3E5C,   /* U+A09D */
  0x3E5D,   /* U+A09E */
  0x3E5E,   /* U+A09F */
  0x3E5F,   /* U+A0A0 */
  0x3E60,   /* U+A0A1 */
  0x3E61,   /* U+A0A2 */
  0x3E62,   /* U+A0A3 */
  0x3E63,   /* U+A0A4 */
  0x3E64,   /* U+A0A5 */
  0x3E65,   /* U+A0A6 */
  0x3E66,   /* U+A0A7 */
  0x3E67,   /* U+A0A8 */
  0x3E68,   /* U+A0A9 */
  0x3E69,   /* U+A0AA */
  0x3E6A,   /* U+A0AB */
  0x3E6B,   /* U+A0AC */
  0x3E6C,   /* U+A0AD */
  0x3E6D,   /* U+A0AE */
  0x3E6E,   /* U+A0AF */
  0x3E6F,   /* U+A0B0 */
  0x3E70,   /* U+A0B1 */
  0x3E71,   /* U+A0B2 */
  0x3E72,   /* U+A0B3 */
  0x3E73,   /* U+A0B4 */
  0x3E74,   /* U+A0B5 */
  0x3E75,   /* U+A0B6 */
  0x3E76,   /* U+A0B7 */
  0x3E77,   /* U+A0B8 */
  0x3E78,   /* U+A0B9 */
  0x3E79,   /* U+A0BA */
  0x3E7A,   /* U+A0BB */
  0x3E7B,   /* U+A0BC */
  0x3E7C,   /* U+A0BD */
  0x3E7D,   /* U+A0BE */
  0x3E7E,   /* U+A0BF */
  0x3E7F,   /* U+A0C0 */
  0x3E80,   /* U+A0C1 */
  0x3E81,   /* U+A0C2 */
  0x3E82,   /* U+A0C3 */
  0x3E83,   /* U+A0C4 */
  0x3E84,   /* U+A0C5 */
  0x3E85,   /* U+A0C6 */
  0x3E86,   /* U+A0C7 */
  0x3E87,   /* U+A0C8 */
  0x3E88,   /* U+A0C9 */
  0x3E89,   /* U+A0CA */
  0x3E8A,   /* U+A0CB */
  0x3E8B,   /* U+A0CC */
  0x3E8C,   /* U+A0CD */
  0x3E8D,   /* U+A0CE */
  0x3E8E,   /* U+A0CF */
  0x3E8F,   /* U+A0D0 */
  0x3E90,   /* U+A0D1 */
  0x3E91,   /* U+A0D2 */
  0x3E92,   /* U+A0D3 */
  0x3E93,   /* U+A0D4 */
  0x3E94,   /* U+A0D5 */
  0x3E95,   /* U+A0D6 */
  0x3E96,   /* U+A0D7 */
  0x3E97,   /* U+A0D8 */
  0x3E98,   /* U+A0D9 */
  0x3E99,   /* U+A0DA */
  0x3E9A,   /* U+A0DB */
  0x3E9B,   /* U+A0DC */
  0x3E9C,   /* U+A0DD */
  0x3E9D,   /* U+A0DE */
  0x3E9E,   /* U+A0DF */
  0x3E9F,   /* U+A0E0 */
  0x3EA0,   /* U+A0E1 */
  0x3EA1,   /* U+A0E2 */
  0x3EA2,   /* U+A0E3 */
  0x3EA3,   /* U+A0E4 */
  0x3EA4,   /* U+A0E5 */
  0x3EA5,   /* U+A0E6 */
  0x3EA6,   /* U+A0E7 */
  0x3EA7,   /* U+A0E8 */
  0x3EA8,   /* U+A0E9 */
  0x3EA9,   /* U+A0EA */
  0x3EAA,   /* U+A0EB */
  0x3EAB,   /* U+A0EC */
  0x3EAC,   /* U+A0ED */
  0x3EAD,   /* U+A0EE */
  0x3EAE,   /* U+A0EF */
  0x3EAF,   /* U+A0F0 */
  0x3EB0,   /* U+A0F1 */
  0x3EB1,   /* U+A0F2 */
  0x3EB2,   /* U+A0F3 */
  0x3EB3,   /* U+A0F4 */
  0x3EB4,   /* U+A0F5 */
  0x3EB5,   /* U+A0F6 */
  0x3EB6,   /* U+A0F7 */
  0x3EB7,   /* U+A0F8 */
  0x3EB8,   /* U+A0F9 */
  0x3EB9,   /* U+A0FA */
  0x3EBA,   /* U+A0FB */
  0x3EBB,   /* U+A0FC */
  0x3EBC,   /* U+A0FD */
  0x3EBD,   /* U+A0FE */
  0x3EBE,   /* U+A0FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A000 */
  0x0020,   /* U+A001 */
  0x0020,   /* U+A002 */
  0x0020,   /* U+A003 */
  0x0020,   /* U+A004 */
  0x0020,   /* U+A005 */
  0x0020,   /* U+A006 */
  0x0020,   /* U+A007 */
  0x0020,   /* U+A008 */
  0x0020,   /* U+A009 */
  0x0020,   /* U+A00A */
  0x0020,   /* U+A00B */
  0x0020,   /* U+A00C */
  0x0020,   /* U+A00D */
  0x0020,   /* U+A00E */
  0x0020,   /* U+A00F */
  0x0020,   /* U+A010 */
  0x0020,   /* U+A011 */
  0x0020,   /* U+A012 */
  0x0020,   /* U+A013 */
  0x0020,   /* U+A014 */
  0x0020,   /* U+A015 */
  0x0020,   /* U+A016 */
  0x0020,   /* U+A017 */
  0x0020,   /* U+A018 */
  0x0020,   /* U+A019 */
  0x0020,   /* U+A01A */
  0x0020,   /* U+A01B */
  0x0020,   /* U+A01C */
  0x0020,   /* U+A01D */
  0x0020,   /* U+A01E */
  0x0020,   /* U+A01F */
  0x0020,   /* U+A020 */
  0x0020,   /* U+A021 */
  0x0020,   /* U+A022 */
  0x0020,   /* U+A023 */
  0x0020,   /* U+A024 */
  0x0020,   /* U+A025 */
  0x0020,   /* U+A026 */
  0x0020,   /* U+A027 */
  0x0020,   /* U+A028 */
  0x0020,   /* U+A029 */
  0x0020,   /* U+A02A */
  0x0020,   /* U+A02B */
  0x0020,   /* U+A02C */
  0x0020,   /* U+A02D */
  0x0020,   /* U+A02E */
  0x0020,   /* U+A02F */
  0x0020,   /* U+A030 */
  0x0020,   /* U+A031 */
  0x0020,   /* U+A032 */
  0x0020,   /* U+A033 */
  0x0020,   /* U+A034 */
  0x0020,   /* U+A035 */
  0x0020,   /* U+A036 */
  0x0020,   /* U+A037 */
  0x0020,   /* U+A038 */
  0x0020,   /* U+A039 */
  0x0020,   /* U+A03A */
  0x0020,   /* U+A03B */
  0x0020,   /* U+A03C */
  0x0020,   /* U+A03D */
  0x0020,   /* U+A03E */
  0x0020,   /* U+A03F */
  0x0020,   /* U+A040 */
  0x0020,   /* U+A041 */
  0x0020,   /* U+A042 */
  0x0020,   /* U+A043 */
  0x0020,   /* U+A044 */
  0x0020,   /* U+A045 */
  0x0020,   /* U+A046 */
  0x0020,   /* U+A047 */
  0x0020,   /* U+A048 */
  0x0020,   /* U+A049 */
  0x0020,   /* U+A04A */
  0x0020,   /* U+A04B */
  0x0020,   /* U+A04C */
  0x0020,   /* U+A04D */
  0x0020,   /* U+A04E */
  0x0020,   /* U+A04F */
  0x0020,   /* U+A050 */
  0x0020,   /* U+A051 */
  0x0020,   /* U+A052 */
  0x0020,   /* U+A053 */
  0x0020,   /* U+A054 */
  0x0020,   /* U+A055 */
  0x0020,   /* U+A056 */
  0x0020,   /* U+A057 */
  0x0020,   /* U+A058 */
  0x0020,   /* U+A059 */
  0x0020,   /* U+A05A */
  0x0020,   /* U+A05B */
  0x0020,   /* U+A05C */
  0x0020,   /* U+A05D */
  0x0020,   /* U+A05E */
  0x0020,   /* U+A05F */
  0x0020,   /* U+A060 */
  0x0020,   /* U+A061 */
  0x0020,   /* U+A062 */
  0x0020,   /* U+A063 */
  0x0020,   /* U+A064 */
  0x0020,   /* U+A065 */
  0x0020,   /* U+A066 */
  0x0020,   /* U+A067 */
  0x0020,   /* U+A068 */
  0x0020,   /* U+A069 */
  0x0020,   /* U+A06A */
  0x0020,   /* U+A06B */
  0x0020,   /* U+A06C */
  0x0020,   /* U+A06D */
  0x0020,   /* U+A06E */
  0x0020,   /* U+A06F */
  0x0020,   /* U+A070 */
  0x0020,   /* U+A071 */
  0x0020,   /* U+A072 */
  0x0020,   /* U+A073 */
  0x0020,   /* U+A074 */
  0x0020,   /* U+A075 */
  0x0020,   /* U+A076 */
  0x0020,   /* U+A077 */
  0x0020,   /* U+A078 */
  0x0020,   /* U+A079 */
  0x0020,   /* U+A07A */
  0x0020,   /* U+A07B */
  0x0020,   /* U+A07C */
  0x0020,   /* U+A07D */
  0x0020,   /* U+A07E */
  0x0020,   /* U+A07F */
  0x0020,   /* U+A080 */
  0x0020,   /* U+A081 */
  0x0020,   /* U+A082 */
  0x0020,   /* U+A083 */
  0x0020,   /* U+A084 */
  0x0020,   /* U+A085 */
  0x0020,   /* U+A086 */
  0x0020,   /* U+A087 */
  0x0020,   /* U+A088 */
  0x0020,   /* U+A089 */
  0x0020,   /* U+A08A */
  0x0020,   /* U+A08B */
  0x0020,   /* U+A08C */
  0x0020,   /* U+A08D */
  0x0020,   /* U+A08E */
  0x0020,   /* U+A08F */
  0x0020,   /* U+A090 */
  0x0020,   /* U+A091 */
  0x0020,   /* U+A092 */
  0x0020,   /* U+A093 */
  0x0020,   /* U+A094 */
  0x0020,   /* U+A095 */
  0x0020,   /* U+A096 */
  0x0020,   /* U+A097 */
  0x0020,   /* U+A098 */
  0x0020,   /* U+A099 */
  0x0020,   /* U+A09A */
  0x0020,   /* U+A09B */
  0x0020,   /* U+A09C */
  0x0020,   /* U+A09D */
  0x0020,   /* U+A09E */
  0x0020,   /* U+A09F */
  0x0020,   /* U+A0A0 */
  0x0020,   /* U+A0A1 */
  0x0020,   /* U+A0A2 */
  0x0020,   /* U+A0A3 */
  0x0020,   /* U+A0A4 */
  0x0020,   /* U+A0A5 */
  0x0020,   /* U+A0A6 */
  0x0020,   /* U+A0A7 */
  0x0020,   /* U+A0A8 */
  0x0020,   /* U+A0A9 */
  0x0020,   /* U+A0AA */
  0x0020,   /* U+A0AB */
  0x0020,   /* U+A0AC */
  0x0020,   /* U+A0AD */
  0x0020,   /* U+A0AE */
  0x0020,   /* U+A0AF */
  0x0020,   /* U+A0B0 */
  0x0020,   /* U+A0B1 */
  0x0020,   /* U+A0B2 */
  0x0020,   /* U+A0B3 */
  0x0020,   /* U+A0B4 */
  0x0020,   /* U+A0B5 */
  0x0020,   /* U+A0B6 */
  0x0020,   /* U+A0B7 */
  0x0020,   /* U+A0B8 */
  0x0020,   /* U+A0B9 */
  0x0020,   /* U+A0BA */
  0x0020,   /* U+A0BB */
  0x0020,   /* U+A0BC */
  0x0020,   /* U+A0BD */
  0x0020,   /* U+A0BE */
  0x0020,   /* U+A0BF */
  0x0020,   /* U+A0C0 */
  0x0020,   /* U+A0C1 */
  0x0020,   /* U+A0C2 */
  0x0020,   /* U+A0C3 */
  0x0020,   /* U+A0C4 */
  0x0020,   /* U+A0C5 */
  0x0020,   /* U+A0C6 */
  0x0020,   /* U+A0C7 */
  0x0020,   /* U+A0C8 */
  0x0020,   /* U+A0C9 */
  0x0020,   /* U+A0CA */
  0x0020,   /* U+A0CB */
  0x0020,   /* U+A0CC */
  0x0020,   /* U+A0CD */
  0x0020,   /* U+A0CE */
  0x0020,   /* U+A0CF */
  0x0020,   /* U+A0D0 */
  0x0020,   /* U+A0D1 */
  0x0020,   /* U+A0D2 */
  0x0020,   /* U+A0D3 */
  0x0020,   /* U+A0D4 */
  0x0020,   /* U+A0D5 */
  0x0020,   /* U+A0D6 */
  0x0020,   /* U+A0D7 */
  0x0020,   /* U+A0D8 */
  0x0020,   /* U+A0D9 */
  0x0020,   /* U+A0DA */
  0x0020,   /* U+A0DB */
  0x0020,   /* U+A0DC */
  0x0020,   /* U+A0DD */
  0x0020,   /* U+A0DE */
  0x0020,   /* U+A0DF */
  0x0020,   /* U+A0E0 */
  0x0020,   /* U+A0E1 */
  0x0020,   /* U+A0E2 */
  0x0020,   /* U+A0E3 */
  0x0020,   /* U+A0E4 */
  0x0020,   /* U+A0E5 */
  0x0020,   /* U+A0E6 */
  0x0020,   /* U+A0E7 */
  0x0020,   /* U+A0E8 */
  0x0020,   /* U+A0E9 */
  0x0020,   /* U+A0EA */
  0x0020,   /* U+A0EB */
  0x0020,   /* U+A0EC */
  0x0020,   /* U+A0ED */
  0x0020,   /* U+A0EE */
  0x0020,   /* U+A0EF */
  0x0020,   /* U+A0F0 */
  0x0020,   /* U+A0F1 */
  0x0020,   /* U+A0F2 */
  0x0020,   /* U+A0F3 */
  0x0020,   /* U+A0F4 */
  0x0020,   /* U+A0F5 */
  0x0020,   /* U+A0F6 */
  0x0020,   /* U+A0F7 */
  0x0020,   /* U+A0F8 */
  0x0020,   /* U+A0F9 */
  0x0020,   /* U+A0FA */
  0x0020,   /* U+A0FB */
  0x0020,   /* U+A0FC */
  0x0020,   /* U+A0FD */
  0x0020,   /* U+A0FE */
  0x0020,   /* U+A0FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A000 */
  0x0002,   /* U+A001 */
  0x0002,   /* U+A002 */
  0x0002,   /* U+A003 */
  0x0002,   /* U+A004 */
  0x0002,   /* U+A005 */
  0x0002,   /* U+A006 */
  0x0002,   /* U+A007 */
  0x0002,   /* U+A008 */
  0x0002,   /* U+A009 */
  0x0002,   /* U+A00A */
  0x0002,   /* U+A00B */
  0x0002,   /* U+A00C */
  0x0002,   /* U+A00D */
  0x0002,   /* U+A00E */
  0x0002,   /* U+A00F */
  0x0002,   /* U+A010 */
  0x0002,   /* U+A011 */
  0x0002,   /* U+A012 */
  0x0002,   /* U+A013 */
  0x0002,   /* U+A014 */
  0x0002,   /* U+A015 */
  0x0002,   /* U+A016 */
  0x0002,   /* U+A017 */
  0x0002,   /* U+A018 */
  0x0002,   /* U+A019 */
  0x0002,   /* U+A01A */
  0x0002,   /* U+A01B */
  0x0002,   /* U+A01C */
  0x0002,   /* U+A01D */
  0x0002,   /* U+A01E */
  0x0002,   /* U+A01F */
  0x0002,   /* U+A020 */
  0x0002,   /* U+A021 */
  0x0002,   /* U+A022 */
  0x0002,   /* U+A023 */
  0x0002,   /* U+A024 */
  0x0002,   /* U+A025 */
  0x0002,   /* U+A026 */
  0x0002,   /* U+A027 */
  0x0002,   /* U+A028 */
  0x0002,   /* U+A029 */
  0x0002,   /* U+A02A */
  0x0002,   /* U+A02B */
  0x0002,   /* U+A02C */
  0x0002,   /* U+A02D */
  0x0002,   /* U+A02E */
  0x0002,   /* U+A02F */
  0x0002,   /* U+A030 */
  0x0002,   /* U+A031 */
  0x0002,   /* U+A032 */
  0x0002,   /* U+A033 */
  0x0002,   /* U+A034 */
  0x0002,   /* U+A035 */
  0x0002,   /* U+A036 */
  0x0002,   /* U+A037 */
  0x0002,   /* U+A038 */
  0x0002,   /* U+A039 */
  0x0002,   /* U+A03A */
  0x0002,   /* U+A03B */
  0x0002,   /* U+A03C */
  0x0002,   /* U+A03D */
  0x0002,   /* U+A03E */
  0x0002,   /* U+A03F */
  0x0002,   /* U+A040 */
  0x0002,   /* U+A041 */
  0x0002,   /* U+A042 */
  0x0002,   /* U+A043 */
  0x0002,   /* U+A044 */
  0x0002,   /* U+A045 */
  0x0002,   /* U+A046 */
  0x0002,   /* U+A047 */
  0x0002,   /* U+A048 */
  0x0002,   /* U+A049 */
  0x0002,   /* U+A04A */
  0x0002,   /* U+A04B */
  0x0002,   /* U+A04C */
  0x0002,   /* U+A04D */
  0x0002,   /* U+A04E */
  0x0002,   /* U+A04F */
  0x0002,   /* U+A050 */
  0x0002,   /* U+A051 */
  0x0002,   /* U+A052 */
  0x0002,   /* U+A053 */
  0x0002,   /* U+A054 */
  0x0002,   /* U+A055 */
  0x0002,   /* U+A056 */
  0x0002,   /* U+A057 */
  0x0002,   /* U+A058 */
  0x0002,   /* U+A059 */
  0x0002,   /* U+A05A */
  0x0002,   /* U+A05B */
  0x0002,   /* U+A05C */
  0x0002,   /* U+A05D */
  0x0002,   /* U+A05E */
  0x0002,   /* U+A05F */
  0x0002,   /* U+A060 */
  0x0002,   /* U+A061 */
  0x0002,   /* U+A062 */
  0x0002,   /* U+A063 */
  0x0002,   /* U+A064 */
  0x0002,   /* U+A065 */
  0x0002,   /* U+A066 */
  0x0002,   /* U+A067 */
  0x0002,   /* U+A068 */
  0x0002,   /* U+A069 */
  0x0002,   /* U+A06A */
  0x0002,   /* U+A06B */
  0x0002,   /* U+A06C */
  0x0002,   /* U+A06D */
  0x0002,   /* U+A06E */
  0x0002,   /* U+A06F */
  0x0002,   /* U+A070 */
  0x0002,   /* U+A071 */
  0x0002,   /* U+A072 */
  0x0002,   /* U+A073 */
  0x0002,   /* U+A074 */
  0x0002,   /* U+A075 */
  0x0002,   /* U+A076 */
  0x0002,   /* U+A077 */
  0x0002,   /* U+A078 */
  0x0002,   /* U+A079 */
  0x0002,   /* U+A07A */
  0x0002,   /* U+A07B */
  0x0002,   /* U+A07C */
  0x0002,   /* U+A07D */
  0x0002,   /* U+A07E */
  0x0002,   /* U+A07F */
  0x0002,   /* U+A080 */
  0x0002,   /* U+A081 */
  0x0002,   /* U+A082 */
  0x0002,   /* U+A083 */
  0x0002,   /* U+A084 */
  0x0002,   /* U+A085 */
  0x0002,   /* U+A086 */
  0x0002,   /* U+A087 */
  0x0002,   /* U+A088 */
  0x0002,   /* U+A089 */
  0x0002,   /* U+A08A */
  0x0002,   /* U+A08B */
  0x0002,   /* U+A08C */
  0x0002,   /* U+A08D */
  0x0002,   /* U+A08E */
  0x0002,   /* U+A08F */
  0x0002,   /* U+A090 */
  0x0002,   /* U+A091 */
  0x0002,   /* U+A092 */
  0x0002,   /* U+A093 */
  0x0002,   /* U+A094 */
  0x0002,   /* U+A095 */
  0x0002,   /* U+A096 */
  0x0002,   /* U+A097 */
  0x0002,   /* U+A098 */
  0x0002,   /* U+A099 */
  0x0002,   /* U+A09A */
  0x0002,   /* U+A09B */
  0x0002,   /* U+A09C */
  0x0002,   /* U+A09D */
  0x0002,   /* U+A09E */
  0x0002,   /* U+A09F */
  0x0002,   /* U+A0A0 */
  0x0002,   /* U+A0A1 */
  0x0002,   /* U+A0A2 */
  0x0002,   /* U+A0A3 */
  0x0002,   /* U+A0A4 */
  0x0002,   /* U+A0A5 */
  0x0002,   /* U+A0A6 */
  0x0002,   /* U+A0A7 */
  0x0002,   /* U+A0A8 */
  0x0002,   /* U+A0A9 */
  0x0002,   /* U+A0AA */
  0x0002,   /* U+A0AB */
  0x0002,   /* U+A0AC */
  0x0002,   /* U+A0AD */
  0x0002,   /* U+A0AE */
  0x0002,   /* U+A0AF */
  0x0002,   /* U+A0B0 */
  0x0002,   /* U+A0B1 */
  0x0002,   /* U+A0B2 */
  0x0002,   /* U+A0B3 */
  0x0002,   /* U+A0B4 */
  0x0002,   /* U+A0B5 */
  0x0002,   /* U+A0B6 */
  0x0002,   /* U+A0B7 */
  0x0002,   /* U+A0B8 */
  0x0002,   /* U+A0B9 */
  0x0002,   /* U+A0BA */
  0x0002,   /* U+A0BB */
  0x0002,   /* U+A0BC */
  0x0002,   /* U+A0BD */
  0x0002,   /* U+A0BE */
  0x0002,   /* U+A0BF */
  0x0002,   /* U+A0C0 */
  0x0002,   /* U+A0C1 */
  0x0002,   /* U+A0C2 */
  0x0002,   /* U+A0C3 */
  0x0002,   /* U+A0C4 */
  0x0002,   /* U+A0C5 */
  0x0002,   /* U+A0C6 */
  0x0002,   /* U+A0C7 */
  0x0002,   /* U+A0C8 */
  0x0002,   /* U+A0C9 */
  0x0002,   /* U+A0CA */
  0x0002,   /* U+A0CB */
  0x0002,   /* U+A0CC */
  0x0002,   /* U+A0CD */
  0x0002,   /* U+A0CE */
  0x0002,   /* U+A0CF */
  0x0002,   /* U+A0D0 */
  0x0002,   /* U+A0D1 */
  0x0002,   /* U+A0D2 */
  0x0002,   /* U+A0D3 */
  0x0002,   /* U+A0D4 */
  0x0002,   /* U+A0D5 */
  0x0002,   /* U+A0D6 */
  0x0002,   /* U+A0D7 */
  0x0002,   /* U+A0D8 */
  0x0002,   /* U+A0D9 */
  0x0002,   /* U+A0DA */
  0x0002,   /* U+A0DB */
  0x0002,   /* U+A0DC */
  0x0002,   /* U+A0DD */
  0x0002,   /* U+A0DE */
  0x0002,   /* U+A0DF */
  0x0002,   /* U+A0E0 */
  0x0002,   /* U+A0E1 */
  0x0002,   /* U+A0E2 */
  0x0002,   /* U+A0E3 */
  0x0002,   /* U+A0E4 */
  0x0002,   /* U+A0E5 */
  0x0002,   /* U+A0E6 */
  0x0002,   /* U+A0E7 */
  0x0002,   /* U+A0E8 */
  0x0002,   /* U+A0E9 */
  0x0002,   /* U+A0EA */
  0x0002,   /* U+A0EB */
  0x0002,   /* U+A0EC */
  0x0002,   /* U+A0ED */
  0x0002,   /* U+A0EE */
  0x0002,   /* U+A0EF */
  0x0002,   /* U+A0F0 */
  0x0002,   /* U+A0F1 */
  0x0002,   /* U+A0F2 */
  0x0002,   /* U+A0F3 */
  0x0002,   /* U+A0F4 */
  0x0002,   /* U+A0F5 */
  0x0002,   /* U+A0F6 */
  0x0002,   /* U+A0F7 */
  0x0002,   /* U+A0F8 */
  0x0002,   /* U+A0F9 */
  0x0002,   /* U+A0FA */
  0x0002,   /* U+A0FB */
  0x0002,   /* U+A0FC */
  0x0002,   /* U+A0FD */
  0x0002,   /* U+A0FE */
  0x0002,   /* U+A0FF */
};

uint16 uca900_p0A1[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x3EBF,   /* U+A100 */
  0x3EC0,   /* U+A101 */
  0x3EC1,   /* U+A102 */
  0x3EC2,   /* U+A103 */
  0x3EC3,   /* U+A104 */
  0x3EC4,   /* U+A105 */
  0x3EC5,   /* U+A106 */
  0x3EC6,   /* U+A107 */
  0x3EC7,   /* U+A108 */
  0x3EC8,   /* U+A109 */
  0x3EC9,   /* U+A10A */
  0x3ECA,   /* U+A10B */
  0x3ECB,   /* U+A10C */
  0x3ECC,   /* U+A10D */
  0x3ECD,   /* U+A10E */
  0x3ECE,   /* U+A10F */
  0x3ECF,   /* U+A110 */
  0x3ED0,   /* U+A111 */
  0x3ED1,   /* U+A112 */
  0x3ED2,   /* U+A113 */
  0x3ED3,   /* U+A114 */
  0x3ED4,   /* U+A115 */
  0x3ED5,   /* U+A116 */
  0x3ED6,   /* U+A117 */
  0x3ED7,   /* U+A118 */
  0x3ED8,   /* U+A119 */
  0x3ED9,   /* U+A11A */
  0x3EDA,   /* U+A11B */
  0x3EDB,   /* U+A11C */
  0x3EDC,   /* U+A11D */
  0x3EDD,   /* U+A11E */
  0x3EDE,   /* U+A11F */
  0x3EDF,   /* U+A120 */
  0x3EE0,   /* U+A121 */
  0x3EE1,   /* U+A122 */
  0x3EE2,   /* U+A123 */
  0x3EE3,   /* U+A124 */
  0x3EE4,   /* U+A125 */
  0x3EE5,   /* U+A126 */
  0x3EE6,   /* U+A127 */
  0x3EE7,   /* U+A128 */
  0x3EE8,   /* U+A129 */
  0x3EE9,   /* U+A12A */
  0x3EEA,   /* U+A12B */
  0x3EEB,   /* U+A12C */
  0x3EEC,   /* U+A12D */
  0x3EED,   /* U+A12E */
  0x3EEE,   /* U+A12F */
  0x3EEF,   /* U+A130 */
  0x3EF0,   /* U+A131 */
  0x3EF1,   /* U+A132 */
  0x3EF2,   /* U+A133 */
  0x3EF3,   /* U+A134 */
  0x3EF4,   /* U+A135 */
  0x3EF5,   /* U+A136 */
  0x3EF6,   /* U+A137 */
  0x3EF7,   /* U+A138 */
  0x3EF8,   /* U+A139 */
  0x3EF9,   /* U+A13A */
  0x3EFA,   /* U+A13B */
  0x3EFB,   /* U+A13C */
  0x3EFC,   /* U+A13D */
  0x3EFD,   /* U+A13E */
  0x3EFE,   /* U+A13F */
  0x3EFF,   /* U+A140 */
  0x3F00,   /* U+A141 */
  0x3F01,   /* U+A142 */
  0x3F02,   /* U+A143 */
  0x3F03,   /* U+A144 */
  0x3F04,   /* U+A145 */
  0x3F05,   /* U+A146 */
  0x3F06,   /* U+A147 */
  0x3F07,   /* U+A148 */
  0x3F08,   /* U+A149 */
  0x3F09,   /* U+A14A */
  0x3F0A,   /* U+A14B */
  0x3F0B,   /* U+A14C */
  0x3F0C,   /* U+A14D */
  0x3F0D,   /* U+A14E */
  0x3F0E,   /* U+A14F */
  0x3F0F,   /* U+A150 */
  0x3F10,   /* U+A151 */
  0x3F11,   /* U+A152 */
  0x3F12,   /* U+A153 */
  0x3F13,   /* U+A154 */
  0x3F14,   /* U+A155 */
  0x3F15,   /* U+A156 */
  0x3F16,   /* U+A157 */
  0x3F17,   /* U+A158 */
  0x3F18,   /* U+A159 */
  0x3F19,   /* U+A15A */
  0x3F1A,   /* U+A15B */
  0x3F1B,   /* U+A15C */
  0x3F1C,   /* U+A15D */
  0x3F1D,   /* U+A15E */
  0x3F1E,   /* U+A15F */
  0x3F1F,   /* U+A160 */
  0x3F20,   /* U+A161 */
  0x3F21,   /* U+A162 */
  0x3F22,   /* U+A163 */
  0x3F23,   /* U+A164 */
  0x3F24,   /* U+A165 */
  0x3F25,   /* U+A166 */
  0x3F26,   /* U+A167 */
  0x3F27,   /* U+A168 */
  0x3F28,   /* U+A169 */
  0x3F29,   /* U+A16A */
  0x3F2A,   /* U+A16B */
  0x3F2B,   /* U+A16C */
  0x3F2C,   /* U+A16D */
  0x3F2D,   /* U+A16E */
  0x3F2E,   /* U+A16F */
  0x3F2F,   /* U+A170 */
  0x3F30,   /* U+A171 */
  0x3F31,   /* U+A172 */
  0x3F32,   /* U+A173 */
  0x3F33,   /* U+A174 */
  0x3F34,   /* U+A175 */
  0x3F35,   /* U+A176 */
  0x3F36,   /* U+A177 */
  0x3F37,   /* U+A178 */
  0x3F38,   /* U+A179 */
  0x3F39,   /* U+A17A */
  0x3F3A,   /* U+A17B */
  0x3F3B,   /* U+A17C */
  0x3F3C,   /* U+A17D */
  0x3F3D,   /* U+A17E */
  0x3F3E,   /* U+A17F */
  0x3F3F,   /* U+A180 */
  0x3F40,   /* U+A181 */
  0x3F41,   /* U+A182 */
  0x3F42,   /* U+A183 */
  0x3F43,   /* U+A184 */
  0x3F44,   /* U+A185 */
  0x3F45,   /* U+A186 */
  0x3F46,   /* U+A187 */
  0x3F47,   /* U+A188 */
  0x3F48,   /* U+A189 */
  0x3F49,   /* U+A18A */
  0x3F4A,   /* U+A18B */
  0x3F4B,   /* U+A18C */
  0x3F4C,   /* U+A18D */
  0x3F4D,   /* U+A18E */
  0x3F4E,   /* U+A18F */
  0x3F4F,   /* U+A190 */
  0x3F50,   /* U+A191 */
  0x3F51,   /* U+A192 */
  0x3F52,   /* U+A193 */
  0x3F53,   /* U+A194 */
  0x3F54,   /* U+A195 */
  0x3F55,   /* U+A196 */
  0x3F56,   /* U+A197 */
  0x3F57,   /* U+A198 */
  0x3F58,   /* U+A199 */
  0x3F59,   /* U+A19A */
  0x3F5A,   /* U+A19B */
  0x3F5B,   /* U+A19C */
  0x3F5C,   /* U+A19D */
  0x3F5D,   /* U+A19E */
  0x3F5E,   /* U+A19F */
  0x3F5F,   /* U+A1A0 */
  0x3F60,   /* U+A1A1 */
  0x3F61,   /* U+A1A2 */
  0x3F62,   /* U+A1A3 */
  0x3F63,   /* U+A1A4 */
  0x3F64,   /* U+A1A5 */
  0x3F65,   /* U+A1A6 */
  0x3F66,   /* U+A1A7 */
  0x3F67,   /* U+A1A8 */
  0x3F68,   /* U+A1A9 */
  0x3F69,   /* U+A1AA */
  0x3F6A,   /* U+A1AB */
  0x3F6B,   /* U+A1AC */
  0x3F6C,   /* U+A1AD */
  0x3F6D,   /* U+A1AE */
  0x3F6E,   /* U+A1AF */
  0x3F6F,   /* U+A1B0 */
  0x3F70,   /* U+A1B1 */
  0x3F71,   /* U+A1B2 */
  0x3F72,   /* U+A1B3 */
  0x3F73,   /* U+A1B4 */
  0x3F74,   /* U+A1B5 */
  0x3F75,   /* U+A1B6 */
  0x3F76,   /* U+A1B7 */
  0x3F77,   /* U+A1B8 */
  0x3F78,   /* U+A1B9 */
  0x3F79,   /* U+A1BA */
  0x3F7A,   /* U+A1BB */
  0x3F7B,   /* U+A1BC */
  0x3F7C,   /* U+A1BD */
  0x3F7D,   /* U+A1BE */
  0x3F7E,   /* U+A1BF */
  0x3F7F,   /* U+A1C0 */
  0x3F80,   /* U+A1C1 */
  0x3F81,   /* U+A1C2 */
  0x3F82,   /* U+A1C3 */
  0x3F83,   /* U+A1C4 */
  0x3F84,   /* U+A1C5 */
  0x3F85,   /* U+A1C6 */
  0x3F86,   /* U+A1C7 */
  0x3F87,   /* U+A1C8 */
  0x3F88,   /* U+A1C9 */
  0x3F89,   /* U+A1CA */
  0x3F8A,   /* U+A1CB */
  0x3F8B,   /* U+A1CC */
  0x3F8C,   /* U+A1CD */
  0x3F8D,   /* U+A1CE */
  0x3F8E,   /* U+A1CF */
  0x3F8F,   /* U+A1D0 */
  0x3F90,   /* U+A1D1 */
  0x3F91,   /* U+A1D2 */
  0x3F92,   /* U+A1D3 */
  0x3F93,   /* U+A1D4 */
  0x3F94,   /* U+A1D5 */
  0x3F95,   /* U+A1D6 */
  0x3F96,   /* U+A1D7 */
  0x3F97,   /* U+A1D8 */
  0x3F98,   /* U+A1D9 */
  0x3F99,   /* U+A1DA */
  0x3F9A,   /* U+A1DB */
  0x3F9B,   /* U+A1DC */
  0x3F9C,   /* U+A1DD */
  0x3F9D,   /* U+A1DE */
  0x3F9E,   /* U+A1DF */
  0x3F9F,   /* U+A1E0 */
  0x3FA0,   /* U+A1E1 */
  0x3FA1,   /* U+A1E2 */
  0x3FA2,   /* U+A1E3 */
  0x3FA3,   /* U+A1E4 */
  0x3FA4,   /* U+A1E5 */
  0x3FA5,   /* U+A1E6 */
  0x3FA6,   /* U+A1E7 */
  0x3FA7,   /* U+A1E8 */
  0x3FA8,   /* U+A1E9 */
  0x3FA9,   /* U+A1EA */
  0x3FAA,   /* U+A1EB */
  0x3FAB,   /* U+A1EC */
  0x3FAC,   /* U+A1ED */
  0x3FAD,   /* U+A1EE */
  0x3FAE,   /* U+A1EF */
  0x3FAF,   /* U+A1F0 */
  0x3FB0,   /* U+A1F1 */
  0x3FB1,   /* U+A1F2 */
  0x3FB2,   /* U+A1F3 */
  0x3FB3,   /* U+A1F4 */
  0x3FB4,   /* U+A1F5 */
  0x3FB5,   /* U+A1F6 */
  0x3FB6,   /* U+A1F7 */
  0x3FB7,   /* U+A1F8 */
  0x3FB8,   /* U+A1F9 */
  0x3FB9,   /* U+A1FA */
  0x3FBA,   /* U+A1FB */
  0x3FBB,   /* U+A1FC */
  0x3FBC,   /* U+A1FD */
  0x3FBD,   /* U+A1FE */
  0x3FBE,   /* U+A1FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A100 */
  0x0020,   /* U+A101 */
  0x0020,   /* U+A102 */
  0x0020,   /* U+A103 */
  0x0020,   /* U+A104 */
  0x0020,   /* U+A105 */
  0x0020,   /* U+A106 */
  0x0020,   /* U+A107 */
  0x0020,   /* U+A108 */
  0x0020,   /* U+A109 */
  0x0020,   /* U+A10A */
  0x0020,   /* U+A10B */
  0x0020,   /* U+A10C */
  0x0020,   /* U+A10D */
  0x0020,   /* U+A10E */
  0x0020,   /* U+A10F */
  0x0020,   /* U+A110 */
  0x0020,   /* U+A111 */
  0x0020,   /* U+A112 */
  0x0020,   /* U+A113 */
  0x0020,   /* U+A114 */
  0x0020,   /* U+A115 */
  0x0020,   /* U+A116 */
  0x0020,   /* U+A117 */
  0x0020,   /* U+A118 */
  0x0020,   /* U+A119 */
  0x0020,   /* U+A11A */
  0x0020,   /* U+A11B */
  0x0020,   /* U+A11C */
  0x0020,   /* U+A11D */
  0x0020,   /* U+A11E */
  0x0020,   /* U+A11F */
  0x0020,   /* U+A120 */
  0x0020,   /* U+A121 */
  0x0020,   /* U+A122 */
  0x0020,   /* U+A123 */
  0x0020,   /* U+A124 */
  0x0020,   /* U+A125 */
  0x0020,   /* U+A126 */
  0x0020,   /* U+A127 */
  0x0020,   /* U+A128 */
  0x0020,   /* U+A129 */
  0x0020,   /* U+A12A */
  0x0020,   /* U+A12B */
  0x0020,   /* U+A12C */
  0x0020,   /* U+A12D */
  0x0020,   /* U+A12E */
  0x0020,   /* U+A12F */
  0x0020,   /* U+A130 */
  0x0020,   /* U+A131 */
  0x0020,   /* U+A132 */
  0x0020,   /* U+A133 */
  0x0020,   /* U+A134 */
  0x0020,   /* U+A135 */
  0x0020,   /* U+A136 */
  0x0020,   /* U+A137 */
  0x0020,   /* U+A138 */
  0x0020,   /* U+A139 */
  0x0020,   /* U+A13A */
  0x0020,   /* U+A13B */
  0x0020,   /* U+A13C */
  0x0020,   /* U+A13D */
  0x0020,   /* U+A13E */
  0x0020,   /* U+A13F */
  0x0020,   /* U+A140 */
  0x0020,   /* U+A141 */
  0x0020,   /* U+A142 */
  0x0020,   /* U+A143 */
  0x0020,   /* U+A144 */
  0x0020,   /* U+A145 */
  0x0020,   /* U+A146 */
  0x0020,   /* U+A147 */
  0x0020,   /* U+A148 */
  0x0020,   /* U+A149 */
  0x0020,   /* U+A14A */
  0x0020,   /* U+A14B */
  0x0020,   /* U+A14C */
  0x0020,   /* U+A14D */
  0x0020,   /* U+A14E */
  0x0020,   /* U+A14F */
  0x0020,   /* U+A150 */
  0x0020,   /* U+A151 */
  0x0020,   /* U+A152 */
  0x0020,   /* U+A153 */
  0x0020,   /* U+A154 */
  0x0020,   /* U+A155 */
  0x0020,   /* U+A156 */
  0x0020,   /* U+A157 */
  0x0020,   /* U+A158 */
  0x0020,   /* U+A159 */
  0x0020,   /* U+A15A */
  0x0020,   /* U+A15B */
  0x0020,   /* U+A15C */
  0x0020,   /* U+A15D */
  0x0020,   /* U+A15E */
  0x0020,   /* U+A15F */
  0x0020,   /* U+A160 */
  0x0020,   /* U+A161 */
  0x0020,   /* U+A162 */
  0x0020,   /* U+A163 */
  0x0020,   /* U+A164 */
  0x0020,   /* U+A165 */
  0x0020,   /* U+A166 */
  0x0020,   /* U+A167 */
  0x0020,   /* U+A168 */
  0x0020,   /* U+A169 */
  0x0020,   /* U+A16A */
  0x0020,   /* U+A16B */
  0x0020,   /* U+A16C */
  0x0020,   /* U+A16D */
  0x0020,   /* U+A16E */
  0x0020,   /* U+A16F */
  0x0020,   /* U+A170 */
  0x0020,   /* U+A171 */
  0x0020,   /* U+A172 */
  0x0020,   /* U+A173 */
  0x0020,   /* U+A174 */
  0x0020,   /* U+A175 */
  0x0020,   /* U+A176 */
  0x0020,   /* U+A177 */
  0x0020,   /* U+A178 */
  0x0020,   /* U+A179 */
  0x0020,   /* U+A17A */
  0x0020,   /* U+A17B */
  0x0020,   /* U+A17C */
  0x0020,   /* U+A17D */
  0x0020,   /* U+A17E */
  0x0020,   /* U+A17F */
  0x0020,   /* U+A180 */
  0x0020,   /* U+A181 */
  0x0020,   /* U+A182 */
  0x0020,   /* U+A183 */
  0x0020,   /* U+A184 */
  0x0020,   /* U+A185 */
  0x0020,   /* U+A186 */
  0x0020,   /* U+A187 */
  0x0020,   /* U+A188 */
  0x0020,   /* U+A189 */
  0x0020,   /* U+A18A */
  0x0020,   /* U+A18B */
  0x0020,   /* U+A18C */
  0x0020,   /* U+A18D */
  0x0020,   /* U+A18E */
  0x0020,   /* U+A18F */
  0x0020,   /* U+A190 */
  0x0020,   /* U+A191 */
  0x0020,   /* U+A192 */
  0x0020,   /* U+A193 */
  0x0020,   /* U+A194 */
  0x0020,   /* U+A195 */
  0x0020,   /* U+A196 */
  0x0020,   /* U+A197 */
  0x0020,   /* U+A198 */
  0x0020,   /* U+A199 */
  0x0020,   /* U+A19A */
  0x0020,   /* U+A19B */
  0x0020,   /* U+A19C */
  0x0020,   /* U+A19D */
  0x0020,   /* U+A19E */
  0x0020,   /* U+A19F */
  0x0020,   /* U+A1A0 */
  0x0020,   /* U+A1A1 */
  0x0020,   /* U+A1A2 */
  0x0020,   /* U+A1A3 */
  0x0020,   /* U+A1A4 */
  0x0020,   /* U+A1A5 */
  0x0020,   /* U+A1A6 */
  0x0020,   /* U+A1A7 */
  0x0020,   /* U+A1A8 */
  0x0020,   /* U+A1A9 */
  0x0020,   /* U+A1AA */
  0x0020,   /* U+A1AB */
  0x0020,   /* U+A1AC */
  0x0020,   /* U+A1AD */
  0x0020,   /* U+A1AE */
  0x0020,   /* U+A1AF */
  0x0020,   /* U+A1B0 */
  0x0020,   /* U+A1B1 */
  0x0020,   /* U+A1B2 */
  0x0020,   /* U+A1B3 */
  0x0020,   /* U+A1B4 */
  0x0020,   /* U+A1B5 */
  0x0020,   /* U+A1B6 */
  0x0020,   /* U+A1B7 */
  0x0020,   /* U+A1B8 */
  0x0020,   /* U+A1B9 */
  0x0020,   /* U+A1BA */
  0x0020,   /* U+A1BB */
  0x0020,   /* U+A1BC */
  0x0020,   /* U+A1BD */
  0x0020,   /* U+A1BE */
  0x0020,   /* U+A1BF */
  0x0020,   /* U+A1C0 */
  0x0020,   /* U+A1C1 */
  0x0020,   /* U+A1C2 */
  0x0020,   /* U+A1C3 */
  0x0020,   /* U+A1C4 */
  0x0020,   /* U+A1C5 */
  0x0020,   /* U+A1C6 */
  0x0020,   /* U+A1C7 */
  0x0020,   /* U+A1C8 */
  0x0020,   /* U+A1C9 */
  0x0020,   /* U+A1CA */
  0x0020,   /* U+A1CB */
  0x0020,   /* U+A1CC */
  0x0020,   /* U+A1CD */
  0x0020,   /* U+A1CE */
  0x0020,   /* U+A1CF */
  0x0020,   /* U+A1D0 */
  0x0020,   /* U+A1D1 */
  0x0020,   /* U+A1D2 */
  0x0020,   /* U+A1D3 */
  0x0020,   /* U+A1D4 */
  0x0020,   /* U+A1D5 */
  0x0020,   /* U+A1D6 */
  0x0020,   /* U+A1D7 */
  0x0020,   /* U+A1D8 */
  0x0020,   /* U+A1D9 */
  0x0020,   /* U+A1DA */
  0x0020,   /* U+A1DB */
  0x0020,   /* U+A1DC */
  0x0020,   /* U+A1DD */
  0x0020,   /* U+A1DE */
  0x0020,   /* U+A1DF */
  0x0020,   /* U+A1E0 */
  0x0020,   /* U+A1E1 */
  0x0020,   /* U+A1E2 */
  0x0020,   /* U+A1E3 */
  0x0020,   /* U+A1E4 */
  0x0020,   /* U+A1E5 */
  0x0020,   /* U+A1E6 */
  0x0020,   /* U+A1E7 */
  0x0020,   /* U+A1E8 */
  0x0020,   /* U+A1E9 */
  0x0020,   /* U+A1EA */
  0x0020,   /* U+A1EB */
  0x0020,   /* U+A1EC */
  0x0020,   /* U+A1ED */
  0x0020,   /* U+A1EE */
  0x0020,   /* U+A1EF */
  0x0020,   /* U+A1F0 */
  0x0020,   /* U+A1F1 */
  0x0020,   /* U+A1F2 */
  0x0020,   /* U+A1F3 */
  0x0020,   /* U+A1F4 */
  0x0020,   /* U+A1F5 */
  0x0020,   /* U+A1F6 */
  0x0020,   /* U+A1F7 */
  0x0020,   /* U+A1F8 */
  0x0020,   /* U+A1F9 */
  0x0020,   /* U+A1FA */
  0x0020,   /* U+A1FB */
  0x0020,   /* U+A1FC */
  0x0020,   /* U+A1FD */
  0x0020,   /* U+A1FE */
  0x0020,   /* U+A1FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A100 */
  0x0002,   /* U+A101 */
  0x0002,   /* U+A102 */
  0x0002,   /* U+A103 */
  0x0002,   /* U+A104 */
  0x0002,   /* U+A105 */
  0x0002,   /* U+A106 */
  0x0002,   /* U+A107 */
  0x0002,   /* U+A108 */
  0x0002,   /* U+A109 */
  0x0002,   /* U+A10A */
  0x0002,   /* U+A10B */
  0x0002,   /* U+A10C */
  0x0002,   /* U+A10D */
  0x0002,   /* U+A10E */
  0x0002,   /* U+A10F */
  0x0002,   /* U+A110 */
  0x0002,   /* U+A111 */
  0x0002,   /* U+A112 */
  0x0002,   /* U+A113 */
  0x0002,   /* U+A114 */
  0x0002,   /* U+A115 */
  0x0002,   /* U+A116 */
  0x0002,   /* U+A117 */
  0x0002,   /* U+A118 */
  0x0002,   /* U+A119 */
  0x0002,   /* U+A11A */
  0x0002,   /* U+A11B */
  0x0002,   /* U+A11C */
  0x0002,   /* U+A11D */
  0x0002,   /* U+A11E */
  0x0002,   /* U+A11F */
  0x0002,   /* U+A120 */
  0x0002,   /* U+A121 */
  0x0002,   /* U+A122 */
  0x0002,   /* U+A123 */
  0x0002,   /* U+A124 */
  0x0002,   /* U+A125 */
  0x0002,   /* U+A126 */
  0x0002,   /* U+A127 */
  0x0002,   /* U+A128 */
  0x0002,   /* U+A129 */
  0x0002,   /* U+A12A */
  0x0002,   /* U+A12B */
  0x0002,   /* U+A12C */
  0x0002,   /* U+A12D */
  0x0002,   /* U+A12E */
  0x0002,   /* U+A12F */
  0x0002,   /* U+A130 */
  0x0002,   /* U+A131 */
  0x0002,   /* U+A132 */
  0x0002,   /* U+A133 */
  0x0002,   /* U+A134 */
  0x0002,   /* U+A135 */
  0x0002,   /* U+A136 */
  0x0002,   /* U+A137 */
  0x0002,   /* U+A138 */
  0x0002,   /* U+A139 */
  0x0002,   /* U+A13A */
  0x0002,   /* U+A13B */
  0x0002,   /* U+A13C */
  0x0002,   /* U+A13D */
  0x0002,   /* U+A13E */
  0x0002,   /* U+A13F */
  0x0002,   /* U+A140 */
  0x0002,   /* U+A141 */
  0x0002,   /* U+A142 */
  0x0002,   /* U+A143 */
  0x0002,   /* U+A144 */
  0x0002,   /* U+A145 */
  0x0002,   /* U+A146 */
  0x0002,   /* U+A147 */
  0x0002,   /* U+A148 */
  0x0002,   /* U+A149 */
  0x0002,   /* U+A14A */
  0x0002,   /* U+A14B */
  0x0002,   /* U+A14C */
  0x0002,   /* U+A14D */
  0x0002,   /* U+A14E */
  0x0002,   /* U+A14F */
  0x0002,   /* U+A150 */
  0x0002,   /* U+A151 */
  0x0002,   /* U+A152 */
  0x0002,   /* U+A153 */
  0x0002,   /* U+A154 */
  0x0002,   /* U+A155 */
  0x0002,   /* U+A156 */
  0x0002,   /* U+A157 */
  0x0002,   /* U+A158 */
  0x0002,   /* U+A159 */
  0x0002,   /* U+A15A */
  0x0002,   /* U+A15B */
  0x0002,   /* U+A15C */
  0x0002,   /* U+A15D */
  0x0002,   /* U+A15E */
  0x0002,   /* U+A15F */
  0x0002,   /* U+A160 */
  0x0002,   /* U+A161 */
  0x0002,   /* U+A162 */
  0x0002,   /* U+A163 */
  0x0002,   /* U+A164 */
  0x0002,   /* U+A165 */
  0x0002,   /* U+A166 */
  0x0002,   /* U+A167 */
  0x0002,   /* U+A168 */
  0x0002,   /* U+A169 */
  0x0002,   /* U+A16A */
  0x0002,   /* U+A16B */
  0x0002,   /* U+A16C */
  0x0002,   /* U+A16D */
  0x0002,   /* U+A16E */
  0x0002,   /* U+A16F */
  0x0002,   /* U+A170 */
  0x0002,   /* U+A171 */
  0x0002,   /* U+A172 */
  0x0002,   /* U+A173 */
  0x0002,   /* U+A174 */
  0x0002,   /* U+A175 */
  0x0002,   /* U+A176 */
  0x0002,   /* U+A177 */
  0x0002,   /* U+A178 */
  0x0002,   /* U+A179 */
  0x0002,   /* U+A17A */
  0x0002,   /* U+A17B */
  0x0002,   /* U+A17C */
  0x0002,   /* U+A17D */
  0x0002,   /* U+A17E */
  0x0002,   /* U+A17F */
  0x0002,   /* U+A180 */
  0x0002,   /* U+A181 */
  0x0002,   /* U+A182 */
  0x0002,   /* U+A183 */
  0x0002,   /* U+A184 */
  0x0002,   /* U+A185 */
  0x0002,   /* U+A186 */
  0x0002,   /* U+A187 */
  0x0002,   /* U+A188 */
  0x0002,   /* U+A189 */
  0x0002,   /* U+A18A */
  0x0002,   /* U+A18B */
  0x0002,   /* U+A18C */
  0x0002,   /* U+A18D */
  0x0002,   /* U+A18E */
  0x0002,   /* U+A18F */
  0x0002,   /* U+A190 */
  0x0002,   /* U+A191 */
  0x0002,   /* U+A192 */
  0x0002,   /* U+A193 */
  0x0002,   /* U+A194 */
  0x0002,   /* U+A195 */
  0x0002,   /* U+A196 */
  0x0002,   /* U+A197 */
  0x0002,   /* U+A198 */
  0x0002,   /* U+A199 */
  0x0002,   /* U+A19A */
  0x0002,   /* U+A19B */
  0x0002,   /* U+A19C */
  0x0002,   /* U+A19D */
  0x0002,   /* U+A19E */
  0x0002,   /* U+A19F */
  0x0002,   /* U+A1A0 */
  0x0002,   /* U+A1A1 */
  0x0002,   /* U+A1A2 */
  0x0002,   /* U+A1A3 */
  0x0002,   /* U+A1A4 */
  0x0002,   /* U+A1A5 */
  0x0002,   /* U+A1A6 */
  0x0002,   /* U+A1A7 */
  0x0002,   /* U+A1A8 */
  0x0002,   /* U+A1A9 */
  0x0002,   /* U+A1AA */
  0x0002,   /* U+A1AB */
  0x0002,   /* U+A1AC */
  0x0002,   /* U+A1AD */
  0x0002,   /* U+A1AE */
  0x0002,   /* U+A1AF */
  0x0002,   /* U+A1B0 */
  0x0002,   /* U+A1B1 */
  0x0002,   /* U+A1B2 */
  0x0002,   /* U+A1B3 */
  0x0002,   /* U+A1B4 */
  0x0002,   /* U+A1B5 */
  0x0002,   /* U+A1B6 */
  0x0002,   /* U+A1B7 */
  0x0002,   /* U+A1B8 */
  0x0002,   /* U+A1B9 */
  0x0002,   /* U+A1BA */
  0x0002,   /* U+A1BB */
  0x0002,   /* U+A1BC */
  0x0002,   /* U+A1BD */
  0x0002,   /* U+A1BE */
  0x0002,   /* U+A1BF */
  0x0002,   /* U+A1C0 */
  0x0002,   /* U+A1C1 */
  0x0002,   /* U+A1C2 */
  0x0002,   /* U+A1C3 */
  0x0002,   /* U+A1C4 */
  0x0002,   /* U+A1C5 */
  0x0002,   /* U+A1C6 */
  0x0002,   /* U+A1C7 */
  0x0002,   /* U+A1C8 */
  0x0002,   /* U+A1C9 */
  0x0002,   /* U+A1CA */
  0x0002,   /* U+A1CB */
  0x0002,   /* U+A1CC */
  0x0002,   /* U+A1CD */
  0x0002,   /* U+A1CE */
  0x0002,   /* U+A1CF */
  0x0002,   /* U+A1D0 */
  0x0002,   /* U+A1D1 */
  0x0002,   /* U+A1D2 */
  0x0002,   /* U+A1D3 */
  0x0002,   /* U+A1D4 */
  0x0002,   /* U+A1D5 */
  0x0002,   /* U+A1D6 */
  0x0002,   /* U+A1D7 */
  0x0002,   /* U+A1D8 */
  0x0002,   /* U+A1D9 */
  0x0002,   /* U+A1DA */
  0x0002,   /* U+A1DB */
  0x0002,   /* U+A1DC */
  0x0002,   /* U+A1DD */
  0x0002,   /* U+A1DE */
  0x0002,   /* U+A1DF */
  0x0002,   /* U+A1E0 */
  0x0002,   /* U+A1E1 */
  0x0002,   /* U+A1E2 */
  0x0002,   /* U+A1E3 */
  0x0002,   /* U+A1E4 */
  0x0002,   /* U+A1E5 */
  0x0002,   /* U+A1E6 */
  0x0002,   /* U+A1E7 */
  0x0002,   /* U+A1E8 */
  0x0002,   /* U+A1E9 */
  0x0002,   /* U+A1EA */
  0x0002,   /* U+A1EB */
  0x0002,   /* U+A1EC */
  0x0002,   /* U+A1ED */
  0x0002,   /* U+A1EE */
  0x0002,   /* U+A1EF */
  0x0002,   /* U+A1F0 */
  0x0002,   /* U+A1F1 */
  0x0002,   /* U+A1F2 */
  0x0002,   /* U+A1F3 */
  0x0002,   /* U+A1F4 */
  0x0002,   /* U+A1F5 */
  0x0002,   /* U+A1F6 */
  0x0002,   /* U+A1F7 */
  0x0002,   /* U+A1F8 */
  0x0002,   /* U+A1F9 */
  0x0002,   /* U+A1FA */
  0x0002,   /* U+A1FB */
  0x0002,   /* U+A1FC */
  0x0002,   /* U+A1FD */
  0x0002,   /* U+A1FE */
  0x0002,   /* U+A1FF */
};

uint16 uca900_p0A2[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x3FBF,   /* U+A200 */
  0x3FC0,   /* U+A201 */
  0x3FC1,   /* U+A202 */
  0x3FC2,   /* U+A203 */
  0x3FC3,   /* U+A204 */
  0x3FC4,   /* U+A205 */
  0x3FC5,   /* U+A206 */
  0x3FC6,   /* U+A207 */
  0x3FC7,   /* U+A208 */
  0x3FC8,   /* U+A209 */
  0x3FC9,   /* U+A20A */
  0x3FCA,   /* U+A20B */
  0x3FCB,   /* U+A20C */
  0x3FCC,   /* U+A20D */
  0x3FCD,   /* U+A20E */
  0x3FCE,   /* U+A20F */
  0x3FCF,   /* U+A210 */
  0x3FD0,   /* U+A211 */
  0x3FD1,   /* U+A212 */
  0x3FD2,   /* U+A213 */
  0x3FD3,   /* U+A214 */
  0x3FD4,   /* U+A215 */
  0x3FD5,   /* U+A216 */
  0x3FD6,   /* U+A217 */
  0x3FD7,   /* U+A218 */
  0x3FD8,   /* U+A219 */
  0x3FD9,   /* U+A21A */
  0x3FDA,   /* U+A21B */
  0x3FDB,   /* U+A21C */
  0x3FDC,   /* U+A21D */
  0x3FDD,   /* U+A21E */
  0x3FDE,   /* U+A21F */
  0x3FDF,   /* U+A220 */
  0x3FE0,   /* U+A221 */
  0x3FE1,   /* U+A222 */
  0x3FE2,   /* U+A223 */
  0x3FE3,   /* U+A224 */
  0x3FE4,   /* U+A225 */
  0x3FE5,   /* U+A226 */
  0x3FE6,   /* U+A227 */
  0x3FE7,   /* U+A228 */
  0x3FE8,   /* U+A229 */
  0x3FE9,   /* U+A22A */
  0x3FEA,   /* U+A22B */
  0x3FEB,   /* U+A22C */
  0x3FEC,   /* U+A22D */
  0x3FED,   /* U+A22E */
  0x3FEE,   /* U+A22F */
  0x3FEF,   /* U+A230 */
  0x3FF0,   /* U+A231 */
  0x3FF1,   /* U+A232 */
  0x3FF2,   /* U+A233 */
  0x3FF3,   /* U+A234 */
  0x3FF4,   /* U+A235 */
  0x3FF5,   /* U+A236 */
  0x3FF6,   /* U+A237 */
  0x3FF7,   /* U+A238 */
  0x3FF8,   /* U+A239 */
  0x3FF9,   /* U+A23A */
  0x3FFA,   /* U+A23B */
  0x3FFB,   /* U+A23C */
  0x3FFC,   /* U+A23D */
  0x3FFD,   /* U+A23E */
  0x3FFE,   /* U+A23F */
  0x3FFF,   /* U+A240 */
  0x4000,   /* U+A241 */
  0x4001,   /* U+A242 */
  0x4002,   /* U+A243 */
  0x4003,   /* U+A244 */
  0x4004,   /* U+A245 */
  0x4005,   /* U+A246 */
  0x4006,   /* U+A247 */
  0x4007,   /* U+A248 */
  0x4008,   /* U+A249 */
  0x4009,   /* U+A24A */
  0x400A,   /* U+A24B */
  0x400B,   /* U+A24C */
  0x400C,   /* U+A24D */
  0x400D,   /* U+A24E */
  0x400E,   /* U+A24F */
  0x400F,   /* U+A250 */
  0x4010,   /* U+A251 */
  0x4011,   /* U+A252 */
  0x4012,   /* U+A253 */
  0x4013,   /* U+A254 */
  0x4014,   /* U+A255 */
  0x4015,   /* U+A256 */
  0x4016,   /* U+A257 */
  0x4017,   /* U+A258 */
  0x4018,   /* U+A259 */
  0x4019,   /* U+A25A */
  0x401A,   /* U+A25B */
  0x401B,   /* U+A25C */
  0x401C,   /* U+A25D */
  0x401D,   /* U+A25E */
  0x401E,   /* U+A25F */
  0x401F,   /* U+A260 */
  0x4020,   /* U+A261 */
  0x4021,   /* U+A262 */
  0x4022,   /* U+A263 */
  0x4023,   /* U+A264 */
  0x4024,   /* U+A265 */
  0x4025,   /* U+A266 */
  0x4026,   /* U+A267 */
  0x4027,   /* U+A268 */
  0x4028,   /* U+A269 */
  0x4029,   /* U+A26A */
  0x402A,   /* U+A26B */
  0x402B,   /* U+A26C */
  0x402C,   /* U+A26D */
  0x402D,   /* U+A26E */
  0x402E,   /* U+A26F */
  0x402F,   /* U+A270 */
  0x4030,   /* U+A271 */
  0x4031,   /* U+A272 */
  0x4032,   /* U+A273 */
  0x4033,   /* U+A274 */
  0x4034,   /* U+A275 */
  0x4035,   /* U+A276 */
  0x4036,   /* U+A277 */
  0x4037,   /* U+A278 */
  0x4038,   /* U+A279 */
  0x4039,   /* U+A27A */
  0x403A,   /* U+A27B */
  0x403B,   /* U+A27C */
  0x403C,   /* U+A27D */
  0x403D,   /* U+A27E */
  0x403E,   /* U+A27F */
  0x403F,   /* U+A280 */
  0x4040,   /* U+A281 */
  0x4041,   /* U+A282 */
  0x4042,   /* U+A283 */
  0x4043,   /* U+A284 */
  0x4044,   /* U+A285 */
  0x4045,   /* U+A286 */
  0x4046,   /* U+A287 */
  0x4047,   /* U+A288 */
  0x4048,   /* U+A289 */
  0x4049,   /* U+A28A */
  0x404A,   /* U+A28B */
  0x404B,   /* U+A28C */
  0x404C,   /* U+A28D */
  0x404D,   /* U+A28E */
  0x404E,   /* U+A28F */
  0x404F,   /* U+A290 */
  0x4050,   /* U+A291 */
  0x4051,   /* U+A292 */
  0x4052,   /* U+A293 */
  0x4053,   /* U+A294 */
  0x4054,   /* U+A295 */
  0x4055,   /* U+A296 */
  0x4056,   /* U+A297 */
  0x4057,   /* U+A298 */
  0x4058,   /* U+A299 */
  0x4059,   /* U+A29A */
  0x405A,   /* U+A29B */
  0x405B,   /* U+A29C */
  0x405C,   /* U+A29D */
  0x405D,   /* U+A29E */
  0x405E,   /* U+A29F */
  0x405F,   /* U+A2A0 */
  0x4060,   /* U+A2A1 */
  0x4061,   /* U+A2A2 */
  0x4062,   /* U+A2A3 */
  0x4063,   /* U+A2A4 */
  0x4064,   /* U+A2A5 */
  0x4065,   /* U+A2A6 */
  0x4066,   /* U+A2A7 */
  0x4067,   /* U+A2A8 */
  0x4068,   /* U+A2A9 */
  0x4069,   /* U+A2AA */
  0x406A,   /* U+A2AB */
  0x406B,   /* U+A2AC */
  0x406C,   /* U+A2AD */
  0x406D,   /* U+A2AE */
  0x406E,   /* U+A2AF */
  0x406F,   /* U+A2B0 */
  0x4070,   /* U+A2B1 */
  0x4071,   /* U+A2B2 */
  0x4072,   /* U+A2B3 */
  0x4073,   /* U+A2B4 */
  0x4074,   /* U+A2B5 */
  0x4075,   /* U+A2B6 */
  0x4076,   /* U+A2B7 */
  0x4077,   /* U+A2B8 */
  0x4078,   /* U+A2B9 */
  0x4079,   /* U+A2BA */
  0x407A,   /* U+A2BB */
  0x407B,   /* U+A2BC */
  0x407C,   /* U+A2BD */
  0x407D,   /* U+A2BE */
  0x407E,   /* U+A2BF */
  0x407F,   /* U+A2C0 */
  0x4080,   /* U+A2C1 */
  0x4081,   /* U+A2C2 */
  0x4082,   /* U+A2C3 */
  0x4083,   /* U+A2C4 */
  0x4084,   /* U+A2C5 */
  0x4085,   /* U+A2C6 */
  0x4086,   /* U+A2C7 */
  0x4087,   /* U+A2C8 */
  0x4088,   /* U+A2C9 */
  0x4089,   /* U+A2CA */
  0x408A,   /* U+A2CB */
  0x408B,   /* U+A2CC */
  0x408C,   /* U+A2CD */
  0x408D,   /* U+A2CE */
  0x408E,   /* U+A2CF */
  0x408F,   /* U+A2D0 */
  0x4090,   /* U+A2D1 */
  0x4091,   /* U+A2D2 */
  0x4092,   /* U+A2D3 */
  0x4093,   /* U+A2D4 */
  0x4094,   /* U+A2D5 */
  0x4095,   /* U+A2D6 */
  0x4096,   /* U+A2D7 */
  0x4097,   /* U+A2D8 */
  0x4098,   /* U+A2D9 */
  0x4099,   /* U+A2DA */
  0x409A,   /* U+A2DB */
  0x409B,   /* U+A2DC */
  0x409C,   /* U+A2DD */
  0x409D,   /* U+A2DE */
  0x409E,   /* U+A2DF */
  0x409F,   /* U+A2E0 */
  0x40A0,   /* U+A2E1 */
  0x40A1,   /* U+A2E2 */
  0x40A2,   /* U+A2E3 */
  0x40A3,   /* U+A2E4 */
  0x40A4,   /* U+A2E5 */
  0x40A5,   /* U+A2E6 */
  0x40A6,   /* U+A2E7 */
  0x40A7,   /* U+A2E8 */
  0x40A8,   /* U+A2E9 */
  0x40A9,   /* U+A2EA */
  0x40AA,   /* U+A2EB */
  0x40AB,   /* U+A2EC */
  0x40AC,   /* U+A2ED */
  0x40AD,   /* U+A2EE */
  0x40AE,   /* U+A2EF */
  0x40AF,   /* U+A2F0 */
  0x40B0,   /* U+A2F1 */
  0x40B1,   /* U+A2F2 */
  0x40B2,   /* U+A2F3 */
  0x40B3,   /* U+A2F4 */
  0x40B4,   /* U+A2F5 */
  0x40B5,   /* U+A2F6 */
  0x40B6,   /* U+A2F7 */
  0x40B7,   /* U+A2F8 */
  0x40B8,   /* U+A2F9 */
  0x40B9,   /* U+A2FA */
  0x40BA,   /* U+A2FB */
  0x40BB,   /* U+A2FC */
  0x40BC,   /* U+A2FD */
  0x40BD,   /* U+A2FE */
  0x40BE,   /* U+A2FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A200 */
  0x0020,   /* U+A201 */
  0x0020,   /* U+A202 */
  0x0020,   /* U+A203 */
  0x0020,   /* U+A204 */
  0x0020,   /* U+A205 */
  0x0020,   /* U+A206 */
  0x0020,   /* U+A207 */
  0x0020,   /* U+A208 */
  0x0020,   /* U+A209 */
  0x0020,   /* U+A20A */
  0x0020,   /* U+A20B */
  0x0020,   /* U+A20C */
  0x0020,   /* U+A20D */
  0x0020,   /* U+A20E */
  0x0020,   /* U+A20F */
  0x0020,   /* U+A210 */
  0x0020,   /* U+A211 */
  0x0020,   /* U+A212 */
  0x0020,   /* U+A213 */
  0x0020,   /* U+A214 */
  0x0020,   /* U+A215 */
  0x0020,   /* U+A216 */
  0x0020,   /* U+A217 */
  0x0020,   /* U+A218 */
  0x0020,   /* U+A219 */
  0x0020,   /* U+A21A */
  0x0020,   /* U+A21B */
  0x0020,   /* U+A21C */
  0x0020,   /* U+A21D */
  0x0020,   /* U+A21E */
  0x0020,   /* U+A21F */
  0x0020,   /* U+A220 */
  0x0020,   /* U+A221 */
  0x0020,   /* U+A222 */
  0x0020,   /* U+A223 */
  0x0020,   /* U+A224 */
  0x0020,   /* U+A225 */
  0x0020,   /* U+A226 */
  0x0020,   /* U+A227 */
  0x0020,   /* U+A228 */
  0x0020,   /* U+A229 */
  0x0020,   /* U+A22A */
  0x0020,   /* U+A22B */
  0x0020,   /* U+A22C */
  0x0020,   /* U+A22D */
  0x0020,   /* U+A22E */
  0x0020,   /* U+A22F */
  0x0020,   /* U+A230 */
  0x0020,   /* U+A231 */
  0x0020,   /* U+A232 */
  0x0020,   /* U+A233 */
  0x0020,   /* U+A234 */
  0x0020,   /* U+A235 */
  0x0020,   /* U+A236 */
  0x0020,   /* U+A237 */
  0x0020,   /* U+A238 */
  0x0020,   /* U+A239 */
  0x0020,   /* U+A23A */
  0x0020,   /* U+A23B */
  0x0020,   /* U+A23C */
  0x0020,   /* U+A23D */
  0x0020,   /* U+A23E */
  0x0020,   /* U+A23F */
  0x0020,   /* U+A240 */
  0x0020,   /* U+A241 */
  0x0020,   /* U+A242 */
  0x0020,   /* U+A243 */
  0x0020,   /* U+A244 */
  0x0020,   /* U+A245 */
  0x0020,   /* U+A246 */
  0x0020,   /* U+A247 */
  0x0020,   /* U+A248 */
  0x0020,   /* U+A249 */
  0x0020,   /* U+A24A */
  0x0020,   /* U+A24B */
  0x0020,   /* U+A24C */
  0x0020,   /* U+A24D */
  0x0020,   /* U+A24E */
  0x0020,   /* U+A24F */
  0x0020,   /* U+A250 */
  0x0020,   /* U+A251 */
  0x0020,   /* U+A252 */
  0x0020,   /* U+A253 */
  0x0020,   /* U+A254 */
  0x0020,   /* U+A255 */
  0x0020,   /* U+A256 */
  0x0020,   /* U+A257 */
  0x0020,   /* U+A258 */
  0x0020,   /* U+A259 */
  0x0020,   /* U+A25A */
  0x0020,   /* U+A25B */
  0x0020,   /* U+A25C */
  0x0020,   /* U+A25D */
  0x0020,   /* U+A25E */
  0x0020,   /* U+A25F */
  0x0020,   /* U+A260 */
  0x0020,   /* U+A261 */
  0x0020,   /* U+A262 */
  0x0020,   /* U+A263 */
  0x0020,   /* U+A264 */
  0x0020,   /* U+A265 */
  0x0020,   /* U+A266 */
  0x0020,   /* U+A267 */
  0x0020,   /* U+A268 */
  0x0020,   /* U+A269 */
  0x0020,   /* U+A26A */
  0x0020,   /* U+A26B */
  0x0020,   /* U+A26C */
  0x0020,   /* U+A26D */
  0x0020,   /* U+A26E */
  0x0020,   /* U+A26F */
  0x0020,   /* U+A270 */
  0x0020,   /* U+A271 */
  0x0020,   /* U+A272 */
  0x0020,   /* U+A273 */
  0x0020,   /* U+A274 */
  0x0020,   /* U+A275 */
  0x0020,   /* U+A276 */
  0x0020,   /* U+A277 */
  0x0020,   /* U+A278 */
  0x0020,   /* U+A279 */
  0x0020,   /* U+A27A */
  0x0020,   /* U+A27B */
  0x0020,   /* U+A27C */
  0x0020,   /* U+A27D */
  0x0020,   /* U+A27E */
  0x0020,   /* U+A27F */
  0x0020,   /* U+A280 */
  0x0020,   /* U+A281 */
  0x0020,   /* U+A282 */
  0x0020,   /* U+A283 */
  0x0020,   /* U+A284 */
  0x0020,   /* U+A285 */
  0x0020,   /* U+A286 */
  0x0020,   /* U+A287 */
  0x0020,   /* U+A288 */
  0x0020,   /* U+A289 */
  0x0020,   /* U+A28A */
  0x0020,   /* U+A28B */
  0x0020,   /* U+A28C */
  0x0020,   /* U+A28D */
  0x0020,   /* U+A28E */
  0x0020,   /* U+A28F */
  0x0020,   /* U+A290 */
  0x0020,   /* U+A291 */
  0x0020,   /* U+A292 */
  0x0020,   /* U+A293 */
  0x0020,   /* U+A294 */
  0x0020,   /* U+A295 */
  0x0020,   /* U+A296 */
  0x0020,   /* U+A297 */
  0x0020,   /* U+A298 */
  0x0020,   /* U+A299 */
  0x0020,   /* U+A29A */
  0x0020,   /* U+A29B */
  0x0020,   /* U+A29C */
  0x0020,   /* U+A29D */
  0x0020,   /* U+A29E */
  0x0020,   /* U+A29F */
  0x0020,   /* U+A2A0 */
  0x0020,   /* U+A2A1 */
  0x0020,   /* U+A2A2 */
  0x0020,   /* U+A2A3 */
  0x0020,   /* U+A2A4 */
  0x0020,   /* U+A2A5 */
  0x0020,   /* U+A2A6 */
  0x0020,   /* U+A2A7 */
  0x0020,   /* U+A2A8 */
  0x0020,   /* U+A2A9 */
  0x0020,   /* U+A2AA */
  0x0020,   /* U+A2AB */
  0x0020,   /* U+A2AC */
  0x0020,   /* U+A2AD */
  0x0020,   /* U+A2AE */
  0x0020,   /* U+A2AF */
  0x0020,   /* U+A2B0 */
  0x0020,   /* U+A2B1 */
  0x0020,   /* U+A2B2 */
  0x0020,   /* U+A2B3 */
  0x0020,   /* U+A2B4 */
  0x0020,   /* U+A2B5 */
  0x0020,   /* U+A2B6 */
  0x0020,   /* U+A2B7 */
  0x0020,   /* U+A2B8 */
  0x0020,   /* U+A2B9 */
  0x0020,   /* U+A2BA */
  0x0020,   /* U+A2BB */
  0x0020,   /* U+A2BC */
  0x0020,   /* U+A2BD */
  0x0020,   /* U+A2BE */
  0x0020,   /* U+A2BF */
  0x0020,   /* U+A2C0 */
  0x0020,   /* U+A2C1 */
  0x0020,   /* U+A2C2 */
  0x0020,   /* U+A2C3 */
  0x0020,   /* U+A2C4 */
  0x0020,   /* U+A2C5 */
  0x0020,   /* U+A2C6 */
  0x0020,   /* U+A2C7 */
  0x0020,   /* U+A2C8 */
  0x0020,   /* U+A2C9 */
  0x0020,   /* U+A2CA */
  0x0020,   /* U+A2CB */
  0x0020,   /* U+A2CC */
  0x0020,   /* U+A2CD */
  0x0020,   /* U+A2CE */
  0x0020,   /* U+A2CF */
  0x0020,   /* U+A2D0 */
  0x0020,   /* U+A2D1 */
  0x0020,   /* U+A2D2 */
  0x0020,   /* U+A2D3 */
  0x0020,   /* U+A2D4 */
  0x0020,   /* U+A2D5 */
  0x0020,   /* U+A2D6 */
  0x0020,   /* U+A2D7 */
  0x0020,   /* U+A2D8 */
  0x0020,   /* U+A2D9 */
  0x0020,   /* U+A2DA */
  0x0020,   /* U+A2DB */
  0x0020,   /* U+A2DC */
  0x0020,   /* U+A2DD */
  0x0020,   /* U+A2DE */
  0x0020,   /* U+A2DF */
  0x0020,   /* U+A2E0 */
  0x0020,   /* U+A2E1 */
  0x0020,   /* U+A2E2 */
  0x0020,   /* U+A2E3 */
  0x0020,   /* U+A2E4 */
  0x0020,   /* U+A2E5 */
  0x0020,   /* U+A2E6 */
  0x0020,   /* U+A2E7 */
  0x0020,   /* U+A2E8 */
  0x0020,   /* U+A2E9 */
  0x0020,   /* U+A2EA */
  0x0020,   /* U+A2EB */
  0x0020,   /* U+A2EC */
  0x0020,   /* U+A2ED */
  0x0020,   /* U+A2EE */
  0x0020,   /* U+A2EF */
  0x0020,   /* U+A2F0 */
  0x0020,   /* U+A2F1 */
  0x0020,   /* U+A2F2 */
  0x0020,   /* U+A2F3 */
  0x0020,   /* U+A2F4 */
  0x0020,   /* U+A2F5 */
  0x0020,   /* U+A2F6 */
  0x0020,   /* U+A2F7 */
  0x0020,   /* U+A2F8 */
  0x0020,   /* U+A2F9 */
  0x0020,   /* U+A2FA */
  0x0020,   /* U+A2FB */
  0x0020,   /* U+A2FC */
  0x0020,   /* U+A2FD */
  0x0020,   /* U+A2FE */
  0x0020,   /* U+A2FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A200 */
  0x0002,   /* U+A201 */
  0x0002,   /* U+A202 */
  0x0002,   /* U+A203 */
  0x0002,   /* U+A204 */
  0x0002,   /* U+A205 */
  0x0002,   /* U+A206 */
  0x0002,   /* U+A207 */
  0x0002,   /* U+A208 */
  0x0002,   /* U+A209 */
  0x0002,   /* U+A20A */
  0x0002,   /* U+A20B */
  0x0002,   /* U+A20C */
  0x0002,   /* U+A20D */
  0x0002,   /* U+A20E */
  0x0002,   /* U+A20F */
  0x0002,   /* U+A210 */
  0x0002,   /* U+A211 */
  0x0002,   /* U+A212 */
  0x0002,   /* U+A213 */
  0x0002,   /* U+A214 */
  0x0002,   /* U+A215 */
  0x0002,   /* U+A216 */
  0x0002,   /* U+A217 */
  0x0002,   /* U+A218 */
  0x0002,   /* U+A219 */
  0x0002,   /* U+A21A */
  0x0002,   /* U+A21B */
  0x0002,   /* U+A21C */
  0x0002,   /* U+A21D */
  0x0002,   /* U+A21E */
  0x0002,   /* U+A21F */
  0x0002,   /* U+A220 */
  0x0002,   /* U+A221 */
  0x0002,   /* U+A222 */
  0x0002,   /* U+A223 */
  0x0002,   /* U+A224 */
  0x0002,   /* U+A225 */
  0x0002,   /* U+A226 */
  0x0002,   /* U+A227 */
  0x0002,   /* U+A228 */
  0x0002,   /* U+A229 */
  0x0002,   /* U+A22A */
  0x0002,   /* U+A22B */
  0x0002,   /* U+A22C */
  0x0002,   /* U+A22D */
  0x0002,   /* U+A22E */
  0x0002,   /* U+A22F */
  0x0002,   /* U+A230 */
  0x0002,   /* U+A231 */
  0x0002,   /* U+A232 */
  0x0002,   /* U+A233 */
  0x0002,   /* U+A234 */
  0x0002,   /* U+A235 */
  0x0002,   /* U+A236 */
  0x0002,   /* U+A237 */
  0x0002,   /* U+A238 */
  0x0002,   /* U+A239 */
  0x0002,   /* U+A23A */
  0x0002,   /* U+A23B */
  0x0002,   /* U+A23C */
  0x0002,   /* U+A23D */
  0x0002,   /* U+A23E */
  0x0002,   /* U+A23F */
  0x0002,   /* U+A240 */
  0x0002,   /* U+A241 */
  0x0002,   /* U+A242 */
  0x0002,   /* U+A243 */
  0x0002,   /* U+A244 */
  0x0002,   /* U+A245 */
  0x0002,   /* U+A246 */
  0x0002,   /* U+A247 */
  0x0002,   /* U+A248 */
  0x0002,   /* U+A249 */
  0x0002,   /* U+A24A */
  0x0002,   /* U+A24B */
  0x0002,   /* U+A24C */
  0x0002,   /* U+A24D */
  0x0002,   /* U+A24E */
  0x0002,   /* U+A24F */
  0x0002,   /* U+A250 */
  0x0002,   /* U+A251 */
  0x0002,   /* U+A252 */
  0x0002,   /* U+A253 */
  0x0002,   /* U+A254 */
  0x0002,   /* U+A255 */
  0x0002,   /* U+A256 */
  0x0002,   /* U+A257 */
  0x0002,   /* U+A258 */
  0x0002,   /* U+A259 */
  0x0002,   /* U+A25A */
  0x0002,   /* U+A25B */
  0x0002,   /* U+A25C */
  0x0002,   /* U+A25D */
  0x0002,   /* U+A25E */
  0x0002,   /* U+A25F */
  0x0002,   /* U+A260 */
  0x0002,   /* U+A261 */
  0x0002,   /* U+A262 */
  0x0002,   /* U+A263 */
  0x0002,   /* U+A264 */
  0x0002,   /* U+A265 */
  0x0002,   /* U+A266 */
  0x0002,   /* U+A267 */
  0x0002,   /* U+A268 */
  0x0002,   /* U+A269 */
  0x0002,   /* U+A26A */
  0x0002,   /* U+A26B */
  0x0002,   /* U+A26C */
  0x0002,   /* U+A26D */
  0x0002,   /* U+A26E */
  0x0002,   /* U+A26F */
  0x0002,   /* U+A270 */
  0x0002,   /* U+A271 */
  0x0002,   /* U+A272 */
  0x0002,   /* U+A273 */
  0x0002,   /* U+A274 */
  0x0002,   /* U+A275 */
  0x0002,   /* U+A276 */
  0x0002,   /* U+A277 */
  0x0002,   /* U+A278 */
  0x0002,   /* U+A279 */
  0x0002,   /* U+A27A */
  0x0002,   /* U+A27B */
  0x0002,   /* U+A27C */
  0x0002,   /* U+A27D */
  0x0002,   /* U+A27E */
  0x0002,   /* U+A27F */
  0x0002,   /* U+A280 */
  0x0002,   /* U+A281 */
  0x0002,   /* U+A282 */
  0x0002,   /* U+A283 */
  0x0002,   /* U+A284 */
  0x0002,   /* U+A285 */
  0x0002,   /* U+A286 */
  0x0002,   /* U+A287 */
  0x0002,   /* U+A288 */
  0x0002,   /* U+A289 */
  0x0002,   /* U+A28A */
  0x0002,   /* U+A28B */
  0x0002,   /* U+A28C */
  0x0002,   /* U+A28D */
  0x0002,   /* U+A28E */
  0x0002,   /* U+A28F */
  0x0002,   /* U+A290 */
  0x0002,   /* U+A291 */
  0x0002,   /* U+A292 */
  0x0002,   /* U+A293 */
  0x0002,   /* U+A294 */
  0x0002,   /* U+A295 */
  0x0002,   /* U+A296 */
  0x0002,   /* U+A297 */
  0x0002,   /* U+A298 */
  0x0002,   /* U+A299 */
  0x0002,   /* U+A29A */
  0x0002,   /* U+A29B */
  0x0002,   /* U+A29C */
  0x0002,   /* U+A29D */
  0x0002,   /* U+A29E */
  0x0002,   /* U+A29F */
  0x0002,   /* U+A2A0 */
  0x0002,   /* U+A2A1 */
  0x0002,   /* U+A2A2 */
  0x0002,   /* U+A2A3 */
  0x0002,   /* U+A2A4 */
  0x0002,   /* U+A2A5 */
  0x0002,   /* U+A2A6 */
  0x0002,   /* U+A2A7 */
  0x0002,   /* U+A2A8 */
  0x0002,   /* U+A2A9 */
  0x0002,   /* U+A2AA */
  0x0002,   /* U+A2AB */
  0x0002,   /* U+A2AC */
  0x0002,   /* U+A2AD */
  0x0002,   /* U+A2AE */
  0x0002,   /* U+A2AF */
  0x0002,   /* U+A2B0 */
  0x0002,   /* U+A2B1 */
  0x0002,   /* U+A2B2 */
  0x0002,   /* U+A2B3 */
  0x0002,   /* U+A2B4 */
  0x0002,   /* U+A2B5 */
  0x0002,   /* U+A2B6 */
  0x0002,   /* U+A2B7 */
  0x0002,   /* U+A2B8 */
  0x0002,   /* U+A2B9 */
  0x0002,   /* U+A2BA */
  0x0002,   /* U+A2BB */
  0x0002,   /* U+A2BC */
  0x0002,   /* U+A2BD */
  0x0002,   /* U+A2BE */
  0x0002,   /* U+A2BF */
  0x0002,   /* U+A2C0 */
  0x0002,   /* U+A2C1 */
  0x0002,   /* U+A2C2 */
  0x0002,   /* U+A2C3 */
  0x0002,   /* U+A2C4 */
  0x0002,   /* U+A2C5 */
  0x0002,   /* U+A2C6 */
  0x0002,   /* U+A2C7 */
  0x0002,   /* U+A2C8 */
  0x0002,   /* U+A2C9 */
  0x0002,   /* U+A2CA */
  0x0002,   /* U+A2CB */
  0x0002,   /* U+A2CC */
  0x0002,   /* U+A2CD */
  0x0002,   /* U+A2CE */
  0x0002,   /* U+A2CF */
  0x0002,   /* U+A2D0 */
  0x0002,   /* U+A2D1 */
  0x0002,   /* U+A2D2 */
  0x0002,   /* U+A2D3 */
  0x0002,   /* U+A2D4 */
  0x0002,   /* U+A2D5 */
  0x0002,   /* U+A2D6 */
  0x0002,   /* U+A2D7 */
  0x0002,   /* U+A2D8 */
  0x0002,   /* U+A2D9 */
  0x0002,   /* U+A2DA */
  0x0002,   /* U+A2DB */
  0x0002,   /* U+A2DC */
  0x0002,   /* U+A2DD */
  0x0002,   /* U+A2DE */
  0x0002,   /* U+A2DF */
  0x0002,   /* U+A2E0 */
  0x0002,   /* U+A2E1 */
  0x0002,   /* U+A2E2 */
  0x0002,   /* U+A2E3 */
  0x0002,   /* U+A2E4 */
  0x0002,   /* U+A2E5 */
  0x0002,   /* U+A2E6 */
  0x0002,   /* U+A2E7 */
  0x0002,   /* U+A2E8 */
  0x0002,   /* U+A2E9 */
  0x0002,   /* U+A2EA */
  0x0002,   /* U+A2EB */
  0x0002,   /* U+A2EC */
  0x0002,   /* U+A2ED */
  0x0002,   /* U+A2EE */
  0x0002,   /* U+A2EF */
  0x0002,   /* U+A2F0 */
  0x0002,   /* U+A2F1 */
  0x0002,   /* U+A2F2 */
  0x0002,   /* U+A2F3 */
  0x0002,   /* U+A2F4 */
  0x0002,   /* U+A2F5 */
  0x0002,   /* U+A2F6 */
  0x0002,   /* U+A2F7 */
  0x0002,   /* U+A2F8 */
  0x0002,   /* U+A2F9 */
  0x0002,   /* U+A2FA */
  0x0002,   /* U+A2FB */
  0x0002,   /* U+A2FC */
  0x0002,   /* U+A2FD */
  0x0002,   /* U+A2FE */
  0x0002,   /* U+A2FF */
};

uint16 uca900_p0A3[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x40BF,   /* U+A300 */
  0x40C0,   /* U+A301 */
  0x40C1,   /* U+A302 */
  0x40C2,   /* U+A303 */
  0x40C3,   /* U+A304 */
  0x40C4,   /* U+A305 */
  0x40C5,   /* U+A306 */
  0x40C6,   /* U+A307 */
  0x40C7,   /* U+A308 */
  0x40C8,   /* U+A309 */
  0x40C9,   /* U+A30A */
  0x40CA,   /* U+A30B */
  0x40CB,   /* U+A30C */
  0x40CC,   /* U+A30D */
  0x40CD,   /* U+A30E */
  0x40CE,   /* U+A30F */
  0x40CF,   /* U+A310 */
  0x40D0,   /* U+A311 */
  0x40D1,   /* U+A312 */
  0x40D2,   /* U+A313 */
  0x40D3,   /* U+A314 */
  0x40D4,   /* U+A315 */
  0x40D5,   /* U+A316 */
  0x40D6,   /* U+A317 */
  0x40D7,   /* U+A318 */
  0x40D8,   /* U+A319 */
  0x40D9,   /* U+A31A */
  0x40DA,   /* U+A31B */
  0x40DB,   /* U+A31C */
  0x40DC,   /* U+A31D */
  0x40DD,   /* U+A31E */
  0x40DE,   /* U+A31F */
  0x40DF,   /* U+A320 */
  0x40E0,   /* U+A321 */
  0x40E1,   /* U+A322 */
  0x40E2,   /* U+A323 */
  0x40E3,   /* U+A324 */
  0x40E4,   /* U+A325 */
  0x40E5,   /* U+A326 */
  0x40E6,   /* U+A327 */
  0x40E7,   /* U+A328 */
  0x40E8,   /* U+A329 */
  0x40E9,   /* U+A32A */
  0x40EA,   /* U+A32B */
  0x40EB,   /* U+A32C */
  0x40EC,   /* U+A32D */
  0x40ED,   /* U+A32E */
  0x40EE,   /* U+A32F */
  0x40EF,   /* U+A330 */
  0x40F0,   /* U+A331 */
  0x40F1,   /* U+A332 */
  0x40F2,   /* U+A333 */
  0x40F3,   /* U+A334 */
  0x40F4,   /* U+A335 */
  0x40F5,   /* U+A336 */
  0x40F6,   /* U+A337 */
  0x40F7,   /* U+A338 */
  0x40F8,   /* U+A339 */
  0x40F9,   /* U+A33A */
  0x40FA,   /* U+A33B */
  0x40FB,   /* U+A33C */
  0x40FC,   /* U+A33D */
  0x40FD,   /* U+A33E */
  0x40FE,   /* U+A33F */
  0x40FF,   /* U+A340 */
  0x4100,   /* U+A341 */
  0x4101,   /* U+A342 */
  0x4102,   /* U+A343 */
  0x4103,   /* U+A344 */
  0x4104,   /* U+A345 */
  0x4105,   /* U+A346 */
  0x4106,   /* U+A347 */
  0x4107,   /* U+A348 */
  0x4108,   /* U+A349 */
  0x4109,   /* U+A34A */
  0x410A,   /* U+A34B */
  0x410B,   /* U+A34C */
  0x410C,   /* U+A34D */
  0x410D,   /* U+A34E */
  0x410E,   /* U+A34F */
  0x410F,   /* U+A350 */
  0x4110,   /* U+A351 */
  0x4111,   /* U+A352 */
  0x4112,   /* U+A353 */
  0x4113,   /* U+A354 */
  0x4114,   /* U+A355 */
  0x4115,   /* U+A356 */
  0x4116,   /* U+A357 */
  0x4117,   /* U+A358 */
  0x4118,   /* U+A359 */
  0x4119,   /* U+A35A */
  0x411A,   /* U+A35B */
  0x411B,   /* U+A35C */
  0x411C,   /* U+A35D */
  0x411D,   /* U+A35E */
  0x411E,   /* U+A35F */
  0x411F,   /* U+A360 */
  0x4120,   /* U+A361 */
  0x4121,   /* U+A362 */
  0x4122,   /* U+A363 */
  0x4123,   /* U+A364 */
  0x4124,   /* U+A365 */
  0x4125,   /* U+A366 */
  0x4126,   /* U+A367 */
  0x4127,   /* U+A368 */
  0x4128,   /* U+A369 */
  0x4129,   /* U+A36A */
  0x412A,   /* U+A36B */
  0x412B,   /* U+A36C */
  0x412C,   /* U+A36D */
  0x412D,   /* U+A36E */
  0x412E,   /* U+A36F */
  0x412F,   /* U+A370 */
  0x4130,   /* U+A371 */
  0x4131,   /* U+A372 */
  0x4132,   /* U+A373 */
  0x4133,   /* U+A374 */
  0x4134,   /* U+A375 */
  0x4135,   /* U+A376 */
  0x4136,   /* U+A377 */
  0x4137,   /* U+A378 */
  0x4138,   /* U+A379 */
  0x4139,   /* U+A37A */
  0x413A,   /* U+A37B */
  0x413B,   /* U+A37C */
  0x413C,   /* U+A37D */
  0x413D,   /* U+A37E */
  0x413E,   /* U+A37F */
  0x413F,   /* U+A380 */
  0x4140,   /* U+A381 */
  0x4141,   /* U+A382 */
  0x4142,   /* U+A383 */
  0x4143,   /* U+A384 */
  0x4144,   /* U+A385 */
  0x4145,   /* U+A386 */
  0x4146,   /* U+A387 */
  0x4147,   /* U+A388 */
  0x4148,   /* U+A389 */
  0x4149,   /* U+A38A */
  0x414A,   /* U+A38B */
  0x414B,   /* U+A38C */
  0x414C,   /* U+A38D */
  0x414D,   /* U+A38E */
  0x414E,   /* U+A38F */
  0x414F,   /* U+A390 */
  0x4150,   /* U+A391 */
  0x4151,   /* U+A392 */
  0x4152,   /* U+A393 */
  0x4153,   /* U+A394 */
  0x4154,   /* U+A395 */
  0x4155,   /* U+A396 */
  0x4156,   /* U+A397 */
  0x4157,   /* U+A398 */
  0x4158,   /* U+A399 */
  0x4159,   /* U+A39A */
  0x415A,   /* U+A39B */
  0x415B,   /* U+A39C */
  0x415C,   /* U+A39D */
  0x415D,   /* U+A39E */
  0x415E,   /* U+A39F */
  0x415F,   /* U+A3A0 */
  0x4160,   /* U+A3A1 */
  0x4161,   /* U+A3A2 */
  0x4162,   /* U+A3A3 */
  0x4163,   /* U+A3A4 */
  0x4164,   /* U+A3A5 */
  0x4165,   /* U+A3A6 */
  0x4166,   /* U+A3A7 */
  0x4167,   /* U+A3A8 */
  0x4168,   /* U+A3A9 */
  0x4169,   /* U+A3AA */
  0x416A,   /* U+A3AB */
  0x416B,   /* U+A3AC */
  0x416C,   /* U+A3AD */
  0x416D,   /* U+A3AE */
  0x416E,   /* U+A3AF */
  0x416F,   /* U+A3B0 */
  0x4170,   /* U+A3B1 */
  0x4171,   /* U+A3B2 */
  0x4172,   /* U+A3B3 */
  0x4173,   /* U+A3B4 */
  0x4174,   /* U+A3B5 */
  0x4175,   /* U+A3B6 */
  0x4176,   /* U+A3B7 */
  0x4177,   /* U+A3B8 */
  0x4178,   /* U+A3B9 */
  0x4179,   /* U+A3BA */
  0x417A,   /* U+A3BB */
  0x417B,   /* U+A3BC */
  0x417C,   /* U+A3BD */
  0x417D,   /* U+A3BE */
  0x417E,   /* U+A3BF */
  0x417F,   /* U+A3C0 */
  0x4180,   /* U+A3C1 */
  0x4181,   /* U+A3C2 */
  0x4182,   /* U+A3C3 */
  0x4183,   /* U+A3C4 */
  0x4184,   /* U+A3C5 */
  0x4185,   /* U+A3C6 */
  0x4186,   /* U+A3C7 */
  0x4187,   /* U+A3C8 */
  0x4188,   /* U+A3C9 */
  0x4189,   /* U+A3CA */
  0x418A,   /* U+A3CB */
  0x418B,   /* U+A3CC */
  0x418C,   /* U+A3CD */
  0x418D,   /* U+A3CE */
  0x418E,   /* U+A3CF */
  0x418F,   /* U+A3D0 */
  0x4190,   /* U+A3D1 */
  0x4191,   /* U+A3D2 */
  0x4192,   /* U+A3D3 */
  0x4193,   /* U+A3D4 */
  0x4194,   /* U+A3D5 */
  0x4195,   /* U+A3D6 */
  0x4196,   /* U+A3D7 */
  0x4197,   /* U+A3D8 */
  0x4198,   /* U+A3D9 */
  0x4199,   /* U+A3DA */
  0x419A,   /* U+A3DB */
  0x419B,   /* U+A3DC */
  0x419C,   /* U+A3DD */
  0x419D,   /* U+A3DE */
  0x419E,   /* U+A3DF */
  0x419F,   /* U+A3E0 */
  0x41A0,   /* U+A3E1 */
  0x41A1,   /* U+A3E2 */
  0x41A2,   /* U+A3E3 */
  0x41A3,   /* U+A3E4 */
  0x41A4,   /* U+A3E5 */
  0x41A5,   /* U+A3E6 */
  0x41A6,   /* U+A3E7 */
  0x41A7,   /* U+A3E8 */
  0x41A8,   /* U+A3E9 */
  0x41A9,   /* U+A3EA */
  0x41AA,   /* U+A3EB */
  0x41AB,   /* U+A3EC */
  0x41AC,   /* U+A3ED */
  0x41AD,   /* U+A3EE */
  0x41AE,   /* U+A3EF */
  0x41AF,   /* U+A3F0 */
  0x41B0,   /* U+A3F1 */
  0x41B1,   /* U+A3F2 */
  0x41B2,   /* U+A3F3 */
  0x41B3,   /* U+A3F4 */
  0x41B4,   /* U+A3F5 */
  0x41B5,   /* U+A3F6 */
  0x41B6,   /* U+A3F7 */
  0x41B7,   /* U+A3F8 */
  0x41B8,   /* U+A3F9 */
  0x41B9,   /* U+A3FA */
  0x41BA,   /* U+A3FB */
  0x41BB,   /* U+A3FC */
  0x41BC,   /* U+A3FD */
  0x41BD,   /* U+A3FE */
  0x41BE,   /* U+A3FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A300 */
  0x0020,   /* U+A301 */
  0x0020,   /* U+A302 */
  0x0020,   /* U+A303 */
  0x0020,   /* U+A304 */
  0x0020,   /* U+A305 */
  0x0020,   /* U+A306 */
  0x0020,   /* U+A307 */
  0x0020,   /* U+A308 */
  0x0020,   /* U+A309 */
  0x0020,   /* U+A30A */
  0x0020,   /* U+A30B */
  0x0020,   /* U+A30C */
  0x0020,   /* U+A30D */
  0x0020,   /* U+A30E */
  0x0020,   /* U+A30F */
  0x0020,   /* U+A310 */
  0x0020,   /* U+A311 */
  0x0020,   /* U+A312 */
  0x0020,   /* U+A313 */
  0x0020,   /* U+A314 */
  0x0020,   /* U+A315 */
  0x0020,   /* U+A316 */
  0x0020,   /* U+A317 */
  0x0020,   /* U+A318 */
  0x0020,   /* U+A319 */
  0x0020,   /* U+A31A */
  0x0020,   /* U+A31B */
  0x0020,   /* U+A31C */
  0x0020,   /* U+A31D */
  0x0020,   /* U+A31E */
  0x0020,   /* U+A31F */
  0x0020,   /* U+A320 */
  0x0020,   /* U+A321 */
  0x0020,   /* U+A322 */
  0x0020,   /* U+A323 */
  0x0020,   /* U+A324 */
  0x0020,   /* U+A325 */
  0x0020,   /* U+A326 */
  0x0020,   /* U+A327 */
  0x0020,   /* U+A328 */
  0x0020,   /* U+A329 */
  0x0020,   /* U+A32A */
  0x0020,   /* U+A32B */
  0x0020,   /* U+A32C */
  0x0020,   /* U+A32D */
  0x0020,   /* U+A32E */
  0x0020,   /* U+A32F */
  0x0020,   /* U+A330 */
  0x0020,   /* U+A331 */
  0x0020,   /* U+A332 */
  0x0020,   /* U+A333 */
  0x0020,   /* U+A334 */
  0x0020,   /* U+A335 */
  0x0020,   /* U+A336 */
  0x0020,   /* U+A337 */
  0x0020,   /* U+A338 */
  0x0020,   /* U+A339 */
  0x0020,   /* U+A33A */
  0x0020,   /* U+A33B */
  0x0020,   /* U+A33C */
  0x0020,   /* U+A33D */
  0x0020,   /* U+A33E */
  0x0020,   /* U+A33F */
  0x0020,   /* U+A340 */
  0x0020,   /* U+A341 */
  0x0020,   /* U+A342 */
  0x0020,   /* U+A343 */
  0x0020,   /* U+A344 */
  0x0020,   /* U+A345 */
  0x0020,   /* U+A346 */
  0x0020,   /* U+A347 */
  0x0020,   /* U+A348 */
  0x0020,   /* U+A349 */
  0x0020,   /* U+A34A */
  0x0020,   /* U+A34B */
  0x0020,   /* U+A34C */
  0x0020,   /* U+A34D */
  0x0020,   /* U+A34E */
  0x0020,   /* U+A34F */
  0x0020,   /* U+A350 */
  0x0020,   /* U+A351 */
  0x0020,   /* U+A352 */
  0x0020,   /* U+A353 */
  0x0020,   /* U+A354 */
  0x0020,   /* U+A355 */
  0x0020,   /* U+A356 */
  0x0020,   /* U+A357 */
  0x0020,   /* U+A358 */
  0x0020,   /* U+A359 */
  0x0020,   /* U+A35A */
  0x0020,   /* U+A35B */
  0x0020,   /* U+A35C */
  0x0020,   /* U+A35D */
  0x0020,   /* U+A35E */
  0x0020,   /* U+A35F */
  0x0020,   /* U+A360 */
  0x0020,   /* U+A361 */
  0x0020,   /* U+A362 */
  0x0020,   /* U+A363 */
  0x0020,   /* U+A364 */
  0x0020,   /* U+A365 */
  0x0020,   /* U+A366 */
  0x0020,   /* U+A367 */
  0x0020,   /* U+A368 */
  0x0020,   /* U+A369 */
  0x0020,   /* U+A36A */
  0x0020,   /* U+A36B */
  0x0020,   /* U+A36C */
  0x0020,   /* U+A36D */
  0x0020,   /* U+A36E */
  0x0020,   /* U+A36F */
  0x0020,   /* U+A370 */
  0x0020,   /* U+A371 */
  0x0020,   /* U+A372 */
  0x0020,   /* U+A373 */
  0x0020,   /* U+A374 */
  0x0020,   /* U+A375 */
  0x0020,   /* U+A376 */
  0x0020,   /* U+A377 */
  0x0020,   /* U+A378 */
  0x0020,   /* U+A379 */
  0x0020,   /* U+A37A */
  0x0020,   /* U+A37B */
  0x0020,   /* U+A37C */
  0x0020,   /* U+A37D */
  0x0020,   /* U+A37E */
  0x0020,   /* U+A37F */
  0x0020,   /* U+A380 */
  0x0020,   /* U+A381 */
  0x0020,   /* U+A382 */
  0x0020,   /* U+A383 */
  0x0020,   /* U+A384 */
  0x0020,   /* U+A385 */
  0x0020,   /* U+A386 */
  0x0020,   /* U+A387 */
  0x0020,   /* U+A388 */
  0x0020,   /* U+A389 */
  0x0020,   /* U+A38A */
  0x0020,   /* U+A38B */
  0x0020,   /* U+A38C */
  0x0020,   /* U+A38D */
  0x0020,   /* U+A38E */
  0x0020,   /* U+A38F */
  0x0020,   /* U+A390 */
  0x0020,   /* U+A391 */
  0x0020,   /* U+A392 */
  0x0020,   /* U+A393 */
  0x0020,   /* U+A394 */
  0x0020,   /* U+A395 */
  0x0020,   /* U+A396 */
  0x0020,   /* U+A397 */
  0x0020,   /* U+A398 */
  0x0020,   /* U+A399 */
  0x0020,   /* U+A39A */
  0x0020,   /* U+A39B */
  0x0020,   /* U+A39C */
  0x0020,   /* U+A39D */
  0x0020,   /* U+A39E */
  0x0020,   /* U+A39F */
  0x0020,   /* U+A3A0 */
  0x0020,   /* U+A3A1 */
  0x0020,   /* U+A3A2 */
  0x0020,   /* U+A3A3 */
  0x0020,   /* U+A3A4 */
  0x0020,   /* U+A3A5 */
  0x0020,   /* U+A3A6 */
  0x0020,   /* U+A3A7 */
  0x0020,   /* U+A3A8 */
  0x0020,   /* U+A3A9 */
  0x0020,   /* U+A3AA */
  0x0020,   /* U+A3AB */
  0x0020,   /* U+A3AC */
  0x0020,   /* U+A3AD */
  0x0020,   /* U+A3AE */
  0x0020,   /* U+A3AF */
  0x0020,   /* U+A3B0 */
  0x0020,   /* U+A3B1 */
  0x0020,   /* U+A3B2 */
  0x0020,   /* U+A3B3 */
  0x0020,   /* U+A3B4 */
  0x0020,   /* U+A3B5 */
  0x0020,   /* U+A3B6 */
  0x0020,   /* U+A3B7 */
  0x0020,   /* U+A3B8 */
  0x0020,   /* U+A3B9 */
  0x0020,   /* U+A3BA */
  0x0020,   /* U+A3BB */
  0x0020,   /* U+A3BC */
  0x0020,   /* U+A3BD */
  0x0020,   /* U+A3BE */
  0x0020,   /* U+A3BF */
  0x0020,   /* U+A3C0 */
  0x0020,   /* U+A3C1 */
  0x0020,   /* U+A3C2 */
  0x0020,   /* U+A3C3 */
  0x0020,   /* U+A3C4 */
  0x0020,   /* U+A3C5 */
  0x0020,   /* U+A3C6 */
  0x0020,   /* U+A3C7 */
  0x0020,   /* U+A3C8 */
  0x0020,   /* U+A3C9 */
  0x0020,   /* U+A3CA */
  0x0020,   /* U+A3CB */
  0x0020,   /* U+A3CC */
  0x0020,   /* U+A3CD */
  0x0020,   /* U+A3CE */
  0x0020,   /* U+A3CF */
  0x0020,   /* U+A3D0 */
  0x0020,   /* U+A3D1 */
  0x0020,   /* U+A3D2 */
  0x0020,   /* U+A3D3 */
  0x0020,   /* U+A3D4 */
  0x0020,   /* U+A3D5 */
  0x0020,   /* U+A3D6 */
  0x0020,   /* U+A3D7 */
  0x0020,   /* U+A3D8 */
  0x0020,   /* U+A3D9 */
  0x0020,   /* U+A3DA */
  0x0020,   /* U+A3DB */
  0x0020,   /* U+A3DC */
  0x0020,   /* U+A3DD */
  0x0020,   /* U+A3DE */
  0x0020,   /* U+A3DF */
  0x0020,   /* U+A3E0 */
  0x0020,   /* U+A3E1 */
  0x0020,   /* U+A3E2 */
  0x0020,   /* U+A3E3 */
  0x0020,   /* U+A3E4 */
  0x0020,   /* U+A3E5 */
  0x0020,   /* U+A3E6 */
  0x0020,   /* U+A3E7 */
  0x0020,   /* U+A3E8 */
  0x0020,   /* U+A3E9 */
  0x0020,   /* U+A3EA */
  0x0020,   /* U+A3EB */
  0x0020,   /* U+A3EC */
  0x0020,   /* U+A3ED */
  0x0020,   /* U+A3EE */
  0x0020,   /* U+A3EF */
  0x0020,   /* U+A3F0 */
  0x0020,   /* U+A3F1 */
  0x0020,   /* U+A3F2 */
  0x0020,   /* U+A3F3 */
  0x0020,   /* U+A3F4 */
  0x0020,   /* U+A3F5 */
  0x0020,   /* U+A3F6 */
  0x0020,   /* U+A3F7 */
  0x0020,   /* U+A3F8 */
  0x0020,   /* U+A3F9 */
  0x0020,   /* U+A3FA */
  0x0020,   /* U+A3FB */
  0x0020,   /* U+A3FC */
  0x0020,   /* U+A3FD */
  0x0020,   /* U+A3FE */
  0x0020,   /* U+A3FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A300 */
  0x0002,   /* U+A301 */
  0x0002,   /* U+A302 */
  0x0002,   /* U+A303 */
  0x0002,   /* U+A304 */
  0x0002,   /* U+A305 */
  0x0002,   /* U+A306 */
  0x0002,   /* U+A307 */
  0x0002,   /* U+A308 */
  0x0002,   /* U+A309 */
  0x0002,   /* U+A30A */
  0x0002,   /* U+A30B */
  0x0002,   /* U+A30C */
  0x0002,   /* U+A30D */
  0x0002,   /* U+A30E */
  0x0002,   /* U+A30F */
  0x0002,   /* U+A310 */
  0x0002,   /* U+A311 */
  0x0002,   /* U+A312 */
  0x0002,   /* U+A313 */
  0x0002,   /* U+A314 */
  0x0002,   /* U+A315 */
  0x0002,   /* U+A316 */
  0x0002,   /* U+A317 */
  0x0002,   /* U+A318 */
  0x0002,   /* U+A319 */
  0x0002,   /* U+A31A */
  0x0002,   /* U+A31B */
  0x0002,   /* U+A31C */
  0x0002,   /* U+A31D */
  0x0002,   /* U+A31E */
  0x0002,   /* U+A31F */
  0x0002,   /* U+A320 */
  0x0002,   /* U+A321 */
  0x0002,   /* U+A322 */
  0x0002,   /* U+A323 */
  0x0002,   /* U+A324 */
  0x0002,   /* U+A325 */
  0x0002,   /* U+A326 */
  0x0002,   /* U+A327 */
  0x0002,   /* U+A328 */
  0x0002,   /* U+A329 */
  0x0002,   /* U+A32A */
  0x0002,   /* U+A32B */
  0x0002,   /* U+A32C */
  0x0002,   /* U+A32D */
  0x0002,   /* U+A32E */
  0x0002,   /* U+A32F */
  0x0002,   /* U+A330 */
  0x0002,   /* U+A331 */
  0x0002,   /* U+A332 */
  0x0002,   /* U+A333 */
  0x0002,   /* U+A334 */
  0x0002,   /* U+A335 */
  0x0002,   /* U+A336 */
  0x0002,   /* U+A337 */
  0x0002,   /* U+A338 */
  0x0002,   /* U+A339 */
  0x0002,   /* U+A33A */
  0x0002,   /* U+A33B */
  0x0002,   /* U+A33C */
  0x0002,   /* U+A33D */
  0x0002,   /* U+A33E */
  0x0002,   /* U+A33F */
  0x0002,   /* U+A340 */
  0x0002,   /* U+A341 */
  0x0002,   /* U+A342 */
  0x0002,   /* U+A343 */
  0x0002,   /* U+A344 */
  0x0002,   /* U+A345 */
  0x0002,   /* U+A346 */
  0x0002,   /* U+A347 */
  0x0002,   /* U+A348 */
  0x0002,   /* U+A349 */
  0x0002,   /* U+A34A */
  0x0002,   /* U+A34B */
  0x0002,   /* U+A34C */
  0x0002,   /* U+A34D */
  0x0002,   /* U+A34E */
  0x0002,   /* U+A34F */
  0x0002,   /* U+A350 */
  0x0002,   /* U+A351 */
  0x0002,   /* U+A352 */
  0x0002,   /* U+A353 */
  0x0002,   /* U+A354 */
  0x0002,   /* U+A355 */
  0x0002,   /* U+A356 */
  0x0002,   /* U+A357 */
  0x0002,   /* U+A358 */
  0x0002,   /* U+A359 */
  0x0002,   /* U+A35A */
  0x0002,   /* U+A35B */
  0x0002,   /* U+A35C */
  0x0002,   /* U+A35D */
  0x0002,   /* U+A35E */
  0x0002,   /* U+A35F */
  0x0002,   /* U+A360 */
  0x0002,   /* U+A361 */
  0x0002,   /* U+A362 */
  0x0002,   /* U+A363 */
  0x0002,   /* U+A364 */
  0x0002,   /* U+A365 */
  0x0002,   /* U+A366 */
  0x0002,   /* U+A367 */
  0x0002,   /* U+A368 */
  0x0002,   /* U+A369 */
  0x0002,   /* U+A36A */
  0x0002,   /* U+A36B */
  0x0002,   /* U+A36C */
  0x0002,   /* U+A36D */
  0x0002,   /* U+A36E */
  0x0002,   /* U+A36F */
  0x0002,   /* U+A370 */
  0x0002,   /* U+A371 */
  0x0002,   /* U+A372 */
  0x0002,   /* U+A373 */
  0x0002,   /* U+A374 */
  0x0002,   /* U+A375 */
  0x0002,   /* U+A376 */
  0x0002,   /* U+A377 */
  0x0002,   /* U+A378 */
  0x0002,   /* U+A379 */
  0x0002,   /* U+A37A */
  0x0002,   /* U+A37B */
  0x0002,   /* U+A37C */
  0x0002,   /* U+A37D */
  0x0002,   /* U+A37E */
  0x0002,   /* U+A37F */
  0x0002,   /* U+A380 */
  0x0002,   /* U+A381 */
  0x0002,   /* U+A382 */
  0x0002,   /* U+A383 */
  0x0002,   /* U+A384 */
  0x0002,   /* U+A385 */
  0x0002,   /* U+A386 */
  0x0002,   /* U+A387 */
  0x0002,   /* U+A388 */
  0x0002,   /* U+A389 */
  0x0002,   /* U+A38A */
  0x0002,   /* U+A38B */
  0x0002,   /* U+A38C */
  0x0002,   /* U+A38D */
  0x0002,   /* U+A38E */
  0x0002,   /* U+A38F */
  0x0002,   /* U+A390 */
  0x0002,   /* U+A391 */
  0x0002,   /* U+A392 */
  0x0002,   /* U+A393 */
  0x0002,   /* U+A394 */
  0x0002,   /* U+A395 */
  0x0002,   /* U+A396 */
  0x0002,   /* U+A397 */
  0x0002,   /* U+A398 */
  0x0002,   /* U+A399 */
  0x0002,   /* U+A39A */
  0x0002,   /* U+A39B */
  0x0002,   /* U+A39C */
  0x0002,   /* U+A39D */
  0x0002,   /* U+A39E */
  0x0002,   /* U+A39F */
  0x0002,   /* U+A3A0 */
  0x0002,   /* U+A3A1 */
  0x0002,   /* U+A3A2 */
  0x0002,   /* U+A3A3 */
  0x0002,   /* U+A3A4 */
  0x0002,   /* U+A3A5 */
  0x0002,   /* U+A3A6 */
  0x0002,   /* U+A3A7 */
  0x0002,   /* U+A3A8 */
  0x0002,   /* U+A3A9 */
  0x0002,   /* U+A3AA */
  0x0002,   /* U+A3AB */
  0x0002,   /* U+A3AC */
  0x0002,   /* U+A3AD */
  0x0002,   /* U+A3AE */
  0x0002,   /* U+A3AF */
  0x0002,   /* U+A3B0 */
  0x0002,   /* U+A3B1 */
  0x0002,   /* U+A3B2 */
  0x0002,   /* U+A3B3 */
  0x0002,   /* U+A3B4 */
  0x0002,   /* U+A3B5 */
  0x0002,   /* U+A3B6 */
  0x0002,   /* U+A3B7 */
  0x0002,   /* U+A3B8 */
  0x0002,   /* U+A3B9 */
  0x0002,   /* U+A3BA */
  0x0002,   /* U+A3BB */
  0x0002,   /* U+A3BC */
  0x0002,   /* U+A3BD */
  0x0002,   /* U+A3BE */
  0x0002,   /* U+A3BF */
  0x0002,   /* U+A3C0 */
  0x0002,   /* U+A3C1 */
  0x0002,   /* U+A3C2 */
  0x0002,   /* U+A3C3 */
  0x0002,   /* U+A3C4 */
  0x0002,   /* U+A3C5 */
  0x0002,   /* U+A3C6 */
  0x0002,   /* U+A3C7 */
  0x0002,   /* U+A3C8 */
  0x0002,   /* U+A3C9 */
  0x0002,   /* U+A3CA */
  0x0002,   /* U+A3CB */
  0x0002,   /* U+A3CC */
  0x0002,   /* U+A3CD */
  0x0002,   /* U+A3CE */
  0x0002,   /* U+A3CF */
  0x0002,   /* U+A3D0 */
  0x0002,   /* U+A3D1 */
  0x0002,   /* U+A3D2 */
  0x0002,   /* U+A3D3 */
  0x0002,   /* U+A3D4 */
  0x0002,   /* U+A3D5 */
  0x0002,   /* U+A3D6 */
  0x0002,   /* U+A3D7 */
  0x0002,   /* U+A3D8 */
  0x0002,   /* U+A3D9 */
  0x0002,   /* U+A3DA */
  0x0002,   /* U+A3DB */
  0x0002,   /* U+A3DC */
  0x0002,   /* U+A3DD */
  0x0002,   /* U+A3DE */
  0x0002,   /* U+A3DF */
  0x0002,   /* U+A3E0 */
  0x0002,   /* U+A3E1 */
  0x0002,   /* U+A3E2 */
  0x0002,   /* U+A3E3 */
  0x0002,   /* U+A3E4 */
  0x0002,   /* U+A3E5 */
  0x0002,   /* U+A3E6 */
  0x0002,   /* U+A3E7 */
  0x0002,   /* U+A3E8 */
  0x0002,   /* U+A3E9 */
  0x0002,   /* U+A3EA */
  0x0002,   /* U+A3EB */
  0x0002,   /* U+A3EC */
  0x0002,   /* U+A3ED */
  0x0002,   /* U+A3EE */
  0x0002,   /* U+A3EF */
  0x0002,   /* U+A3F0 */
  0x0002,   /* U+A3F1 */
  0x0002,   /* U+A3F2 */
  0x0002,   /* U+A3F3 */
  0x0002,   /* U+A3F4 */
  0x0002,   /* U+A3F5 */
  0x0002,   /* U+A3F6 */
  0x0002,   /* U+A3F7 */
  0x0002,   /* U+A3F8 */
  0x0002,   /* U+A3F9 */
  0x0002,   /* U+A3FA */
  0x0002,   /* U+A3FB */
  0x0002,   /* U+A3FC */
  0x0002,   /* U+A3FD */
  0x0002,   /* U+A3FE */
  0x0002,   /* U+A3FF */
};

uint16 uca900_p0A4[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x41BF,   /* U+A400 */
  0x41C0,   /* U+A401 */
  0x41C1,   /* U+A402 */
  0x41C2,   /* U+A403 */
  0x41C3,   /* U+A404 */
  0x41C4,   /* U+A405 */
  0x41C5,   /* U+A406 */
  0x41C6,   /* U+A407 */
  0x41C7,   /* U+A408 */
  0x41C8,   /* U+A409 */
  0x41C9,   /* U+A40A */
  0x41CA,   /* U+A40B */
  0x41CB,   /* U+A40C */
  0x41CC,   /* U+A40D */
  0x41CD,   /* U+A40E */
  0x41CE,   /* U+A40F */
  0x41CF,   /* U+A410 */
  0x41D0,   /* U+A411 */
  0x41D1,   /* U+A412 */
  0x41D2,   /* U+A413 */
  0x41D3,   /* U+A414 */
  0x41D4,   /* U+A415 */
  0x41D5,   /* U+A416 */
  0x41D6,   /* U+A417 */
  0x41D7,   /* U+A418 */
  0x41D8,   /* U+A419 */
  0x41D9,   /* U+A41A */
  0x41DA,   /* U+A41B */
  0x41DB,   /* U+A41C */
  0x41DC,   /* U+A41D */
  0x41DD,   /* U+A41E */
  0x41DE,   /* U+A41F */
  0x41DF,   /* U+A420 */
  0x41E0,   /* U+A421 */
  0x41E1,   /* U+A422 */
  0x41E2,   /* U+A423 */
  0x41E3,   /* U+A424 */
  0x41E4,   /* U+A425 */
  0x41E5,   /* U+A426 */
  0x41E6,   /* U+A427 */
  0x41E7,   /* U+A428 */
  0x41E8,   /* U+A429 */
  0x41E9,   /* U+A42A */
  0x41EA,   /* U+A42B */
  0x41EB,   /* U+A42C */
  0x41EC,   /* U+A42D */
  0x41ED,   /* U+A42E */
  0x41EE,   /* U+A42F */
  0x41EF,   /* U+A430 */
  0x41F0,   /* U+A431 */
  0x41F1,   /* U+A432 */
  0x41F2,   /* U+A433 */
  0x41F3,   /* U+A434 */
  0x41F4,   /* U+A435 */
  0x41F5,   /* U+A436 */
  0x41F6,   /* U+A437 */
  0x41F7,   /* U+A438 */
  0x41F8,   /* U+A439 */
  0x41F9,   /* U+A43A */
  0x41FA,   /* U+A43B */
  0x41FB,   /* U+A43C */
  0x41FC,   /* U+A43D */
  0x41FD,   /* U+A43E */
  0x41FE,   /* U+A43F */
  0x41FF,   /* U+A440 */
  0x4200,   /* U+A441 */
  0x4201,   /* U+A442 */
  0x4202,   /* U+A443 */
  0x4203,   /* U+A444 */
  0x4204,   /* U+A445 */
  0x4205,   /* U+A446 */
  0x4206,   /* U+A447 */
  0x4207,   /* U+A448 */
  0x4208,   /* U+A449 */
  0x4209,   /* U+A44A */
  0x420A,   /* U+A44B */
  0x420B,   /* U+A44C */
  0x420C,   /* U+A44D */
  0x420D,   /* U+A44E */
  0x420E,   /* U+A44F */
  0x420F,   /* U+A450 */
  0x4210,   /* U+A451 */
  0x4211,   /* U+A452 */
  0x4212,   /* U+A453 */
  0x4213,   /* U+A454 */
  0x4214,   /* U+A455 */
  0x4215,   /* U+A456 */
  0x4216,   /* U+A457 */
  0x4217,   /* U+A458 */
  0x4218,   /* U+A459 */
  0x4219,   /* U+A45A */
  0x421A,   /* U+A45B */
  0x421B,   /* U+A45C */
  0x421C,   /* U+A45D */
  0x421D,   /* U+A45E */
  0x421E,   /* U+A45F */
  0x421F,   /* U+A460 */
  0x4220,   /* U+A461 */
  0x4221,   /* U+A462 */
  0x4222,   /* U+A463 */
  0x4223,   /* U+A464 */
  0x4224,   /* U+A465 */
  0x4225,   /* U+A466 */
  0x4226,   /* U+A467 */
  0x4227,   /* U+A468 */
  0x4228,   /* U+A469 */
  0x4229,   /* U+A46A */
  0x422A,   /* U+A46B */
  0x422B,   /* U+A46C */
  0x422C,   /* U+A46D */
  0x422D,   /* U+A46E */
  0x422E,   /* U+A46F */
  0x422F,   /* U+A470 */
  0x4230,   /* U+A471 */
  0x4231,   /* U+A472 */
  0x4232,   /* U+A473 */
  0x4233,   /* U+A474 */
  0x4234,   /* U+A475 */
  0x4235,   /* U+A476 */
  0x4236,   /* U+A477 */
  0x4237,   /* U+A478 */
  0x4238,   /* U+A479 */
  0x4239,   /* U+A47A */
  0x423A,   /* U+A47B */
  0x423B,   /* U+A47C */
  0x423C,   /* U+A47D */
  0x423D,   /* U+A47E */
  0x423E,   /* U+A47F */
  0x423F,   /* U+A480 */
  0x4240,   /* U+A481 */
  0x4241,   /* U+A482 */
  0x4242,   /* U+A483 */
  0x4243,   /* U+A484 */
  0x4244,   /* U+A485 */
  0x4245,   /* U+A486 */
  0x4246,   /* U+A487 */
  0x4247,   /* U+A488 */
  0x4248,   /* U+A489 */
  0x4249,   /* U+A48A */
  0x424A,   /* U+A48B */
  0x424B,   /* U+A48C */
  0xFBC1,   /* U+A48D */
  0xFBC1,   /* U+A48E */
  0xFBC1,   /* U+A48F */
  0x0F41,   /* U+A490 */
  0x0F42,   /* U+A491 */
  0x0F43,   /* U+A492 */
  0x0F44,   /* U+A493 */
  0x0F45,   /* U+A494 */
  0x0F46,   /* U+A495 */
  0x0F47,   /* U+A496 */
  0x0F48,   /* U+A497 */
  0x0F49,   /* U+A498 */
  0x0F4A,   /* U+A499 */
  0x0F4B,   /* U+A49A */
  0x0F4C,   /* U+A49B */
  0x0F4D,   /* U+A49C */
  0x0F4E,   /* U+A49D */
  0x0F4F,   /* U+A49E */
  0x0F50,   /* U+A49F */
  0x0F51,   /* U+A4A0 */
  0x0F52,   /* U+A4A1 */
  0x0F53,   /* U+A4A2 */
  0x0F54,   /* U+A4A3 */
  0x0F55,   /* U+A4A4 */
  0x0F56,   /* U+A4A5 */
  0x0F57,   /* U+A4A6 */
  0x0F58,   /* U+A4A7 */
  0x0F59,   /* U+A4A8 */
  0x0F5A,   /* U+A4A9 */
  0x0F5B,   /* U+A4AA */
  0x0F5C,   /* U+A4AB */
  0x0F5D,   /* U+A4AC */
  0x0F5E,   /* U+A4AD */
  0x0F5F,   /* U+A4AE */
  0x0F60,   /* U+A4AF */
  0x0F61,   /* U+A4B0 */
  0x0F62,   /* U+A4B1 */
  0x0F63,   /* U+A4B2 */
  0x0F64,   /* U+A4B3 */
  0x0F65,   /* U+A4B4 */
  0x0F66,   /* U+A4B5 */
  0x0F67,   /* U+A4B6 */
  0x0F68,   /* U+A4B7 */
  0x0F69,   /* U+A4B8 */
  0x0F6A,   /* U+A4B9 */
  0x0F6B,   /* U+A4BA */
  0x0F6C,   /* U+A4BB */
  0x0F6D,   /* U+A4BC */
  0x0F6E,   /* U+A4BD */
  0x0F6F,   /* U+A4BE */
  0x0F70,   /* U+A4BF */
  0x0F71,   /* U+A4C0 */
  0x0F72,   /* U+A4C1 */
  0x0F73,   /* U+A4C2 */
  0x0F74,   /* U+A4C3 */
  0x0F75,   /* U+A4C4 */
  0x0F76,   /* U+A4C5 */
  0x0F77,   /* U+A4C6 */
  0xFBC1,   /* U+A4C7 */
  0xFBC1,   /* U+A4C8 */
  0xFBC1,   /* U+A4C9 */
  0xFBC1,   /* U+A4CA */
  0xFBC1,   /* U+A4CB */
  0xFBC1,   /* U+A4CC */
  0xFBC1,   /* U+A4CD */
  0xFBC1,   /* U+A4CE */
  0xFBC1,   /* U+A4CF */
  0x4252,   /* U+A4D0 */
  0x4253,   /* U+A4D1 */
  0x4254,   /* U+A4D2 */
  0x4255,   /* U+A4D3 */
  0x4256,   /* U+A4D4 */
  0x4257,   /* U+A4D5 */
  0x4258,   /* U+A4D6 */
  0x4259,   /* U+A4D7 */
  0x425A,   /* U+A4D8 */
  0x425B,   /* U+A4D9 */
  0x425C,   /* U+A4DA */
  0x425D,   /* U+A4DB */
  0x425E,   /* U+A4DC */
  0x425F,   /* U+A4DD */
  0x4260,   /* U+A4DE */
  0x4261,   /* U+A4DF */
  0x4262,   /* U+A4E0 */
  0x4263,   /* U+A4E1 */
  0x4264,   /* U+A4E2 */
  0x4265,   /* U+A4E3 */
  0x4266,   /* U+A4E4 */
  0x4267,   /* U+A4E5 */
  0x4268,   /* U+A4E6 */
  0x4269,   /* U+A4E7 */
  0x426A,   /* U+A4E8 */
  0x426B,   /* U+A4E9 */
  0x426E,   /* U+A4EA */
  0x426C,   /* U+A4EB */
  0x426F,   /* U+A4EC */
  0x426D,   /* U+A4ED */
  0x4270,   /* U+A4EE */
  0x4271,   /* U+A4EF */
  0x4272,   /* U+A4F0 */
  0x4273,   /* U+A4F1 */
  0x4274,   /* U+A4F2 */
  0x4275,   /* U+A4F3 */
  0x4276,   /* U+A4F4 */
  0x4277,   /* U+A4F5 */
  0x4278,   /* U+A4F6 */
  0x4279,   /* U+A4F7 */
  0x424C,   /* U+A4F8 */
  0x424D,   /* U+A4F9 */
  0x424E,   /* U+A4FA */
  0x424F,   /* U+A4FB */
  0x4251,   /* U+A4FC */
  0x4250,   /* U+A4FD */
  0x022E,   /* U+A4FE */
  0x0285,   /* U+A4FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A400 */
  0x0020,   /* U+A401 */
  0x0020,   /* U+A402 */
  0x0020,   /* U+A403 */
  0x0020,   /* U+A404 */
  0x0020,   /* U+A405 */
  0x0020,   /* U+A406 */
  0x0020,   /* U+A407 */
  0x0020,   /* U+A408 */
  0x0020,   /* U+A409 */
  0x0020,   /* U+A40A */
  0x0020,   /* U+A40B */
  0x0020,   /* U+A40C */
  0x0020,   /* U+A40D */
  0x0020,   /* U+A40E */
  0x0020,   /* U+A40F */
  0x0020,   /* U+A410 */
  0x0020,   /* U+A411 */
  0x0020,   /* U+A412 */
  0x0020,   /* U+A413 */
  0x0020,   /* U+A414 */
  0x0020,   /* U+A415 */
  0x0020,   /* U+A416 */
  0x0020,   /* U+A417 */
  0x0020,   /* U+A418 */
  0x0020,   /* U+A419 */
  0x0020,   /* U+A41A */
  0x0020,   /* U+A41B */
  0x0020,   /* U+A41C */
  0x0020,   /* U+A41D */
  0x0020,   /* U+A41E */
  0x0020,   /* U+A41F */
  0x0020,   /* U+A420 */
  0x0020,   /* U+A421 */
  0x0020,   /* U+A422 */
  0x0020,   /* U+A423 */
  0x0020,   /* U+A424 */
  0x0020,   /* U+A425 */
  0x0020,   /* U+A426 */
  0x0020,   /* U+A427 */
  0x0020,   /* U+A428 */
  0x0020,   /* U+A429 */
  0x0020,   /* U+A42A */
  0x0020,   /* U+A42B */
  0x0020,   /* U+A42C */
  0x0020,   /* U+A42D */
  0x0020,   /* U+A42E */
  0x0020,   /* U+A42F */
  0x0020,   /* U+A430 */
  0x0020,   /* U+A431 */
  0x0020,   /* U+A432 */
  0x0020,   /* U+A433 */
  0x0020,   /* U+A434 */
  0x0020,   /* U+A435 */
  0x0020,   /* U+A436 */
  0x0020,   /* U+A437 */
  0x0020,   /* U+A438 */
  0x0020,   /* U+A439 */
  0x0020,   /* U+A43A */
  0x0020,   /* U+A43B */
  0x0020,   /* U+A43C */
  0x0020,   /* U+A43D */
  0x0020,   /* U+A43E */
  0x0020,   /* U+A43F */
  0x0020,   /* U+A440 */
  0x0020,   /* U+A441 */
  0x0020,   /* U+A442 */
  0x0020,   /* U+A443 */
  0x0020,   /* U+A444 */
  0x0020,   /* U+A445 */
  0x0020,   /* U+A446 */
  0x0020,   /* U+A447 */
  0x0020,   /* U+A448 */
  0x0020,   /* U+A449 */
  0x0020,   /* U+A44A */
  0x0020,   /* U+A44B */
  0x0020,   /* U+A44C */
  0x0020,   /* U+A44D */
  0x0020,   /* U+A44E */
  0x0020,   /* U+A44F */
  0x0020,   /* U+A450 */
  0x0020,   /* U+A451 */
  0x0020,   /* U+A452 */
  0x0020,   /* U+A453 */
  0x0020,   /* U+A454 */
  0x0020,   /* U+A455 */
  0x0020,   /* U+A456 */
  0x0020,   /* U+A457 */
  0x0020,   /* U+A458 */
  0x0020,   /* U+A459 */
  0x0020,   /* U+A45A */
  0x0020,   /* U+A45B */
  0x0020,   /* U+A45C */
  0x0020,   /* U+A45D */
  0x0020,   /* U+A45E */
  0x0020,   /* U+A45F */
  0x0020,   /* U+A460 */
  0x0020,   /* U+A461 */
  0x0020,   /* U+A462 */
  0x0020,   /* U+A463 */
  0x0020,   /* U+A464 */
  0x0020,   /* U+A465 */
  0x0020,   /* U+A466 */
  0x0020,   /* U+A467 */
  0x0020,   /* U+A468 */
  0x0020,   /* U+A469 */
  0x0020,   /* U+A46A */
  0x0020,   /* U+A46B */
  0x0020,   /* U+A46C */
  0x0020,   /* U+A46D */
  0x0020,   /* U+A46E */
  0x0020,   /* U+A46F */
  0x0020,   /* U+A470 */
  0x0020,   /* U+A471 */
  0x0020,   /* U+A472 */
  0x0020,   /* U+A473 */
  0x0020,   /* U+A474 */
  0x0020,   /* U+A475 */
  0x0020,   /* U+A476 */
  0x0020,   /* U+A477 */
  0x0020,   /* U+A478 */
  0x0020,   /* U+A479 */
  0x0020,   /* U+A47A */
  0x0020,   /* U+A47B */
  0x0020,   /* U+A47C */
  0x0020,   /* U+A47D */
  0x0020,   /* U+A47E */
  0x0020,   /* U+A47F */
  0x0020,   /* U+A480 */
  0x0020,   /* U+A481 */
  0x0020,   /* U+A482 */
  0x0020,   /* U+A483 */
  0x0020,   /* U+A484 */
  0x0020,   /* U+A485 */
  0x0020,   /* U+A486 */
  0x0020,   /* U+A487 */
  0x0020,   /* U+A488 */
  0x0020,   /* U+A489 */
  0x0020,   /* U+A48A */
  0x0020,   /* U+A48B */
  0x0020,   /* U+A48C */
  0x0020,   /* U+A48D */
  0x0020,   /* U+A48E */
  0x0020,   /* U+A48F */
  0x0020,   /* U+A490 */
  0x0020,   /* U+A491 */
  0x0020,   /* U+A492 */
  0x0020,   /* U+A493 */
  0x0020,   /* U+A494 */
  0x0020,   /* U+A495 */
  0x0020,   /* U+A496 */
  0x0020,   /* U+A497 */
  0x0020,   /* U+A498 */
  0x0020,   /* U+A499 */
  0x0020,   /* U+A49A */
  0x0020,   /* U+A49B */
  0x0020,   /* U+A49C */
  0x0020,   /* U+A49D */
  0x0020,   /* U+A49E */
  0x0020,   /* U+A49F */
  0x0020,   /* U+A4A0 */
  0x0020,   /* U+A4A1 */
  0x0020,   /* U+A4A2 */
  0x0020,   /* U+A4A3 */
  0x0020,   /* U+A4A4 */
  0x0020,   /* U+A4A5 */
  0x0020,   /* U+A4A6 */
  0x0020,   /* U+A4A7 */
  0x0020,   /* U+A4A8 */
  0x0020,   /* U+A4A9 */
  0x0020,   /* U+A4AA */
  0x0020,   /* U+A4AB */
  0x0020,   /* U+A4AC */
  0x0020,   /* U+A4AD */
  0x0020,   /* U+A4AE */
  0x0020,   /* U+A4AF */
  0x0020,   /* U+A4B0 */
  0x0020,   /* U+A4B1 */
  0x0020,   /* U+A4B2 */
  0x0020,   /* U+A4B3 */
  0x0020,   /* U+A4B4 */
  0x0020,   /* U+A4B5 */
  0x0020,   /* U+A4B6 */
  0x0020,   /* U+A4B7 */
  0x0020,   /* U+A4B8 */
  0x0020,   /* U+A4B9 */
  0x0020,   /* U+A4BA */
  0x0020,   /* U+A4BB */
  0x0020,   /* U+A4BC */
  0x0020,   /* U+A4BD */
  0x0020,   /* U+A4BE */
  0x0020,   /* U+A4BF */
  0x0020,   /* U+A4C0 */
  0x0020,   /* U+A4C1 */
  0x0020,   /* U+A4C2 */
  0x0020,   /* U+A4C3 */
  0x0020,   /* U+A4C4 */
  0x0020,   /* U+A4C5 */
  0x0020,   /* U+A4C6 */
  0x0020,   /* U+A4C7 */
  0x0020,   /* U+A4C8 */
  0x0020,   /* U+A4C9 */
  0x0020,   /* U+A4CA */
  0x0020,   /* U+A4CB */
  0x0020,   /* U+A4CC */
  0x0020,   /* U+A4CD */
  0x0020,   /* U+A4CE */
  0x0020,   /* U+A4CF */
  0x0020,   /* U+A4D0 */
  0x0020,   /* U+A4D1 */
  0x0020,   /* U+A4D2 */
  0x0020,   /* U+A4D3 */
  0x0020,   /* U+A4D4 */
  0x0020,   /* U+A4D5 */
  0x0020,   /* U+A4D6 */
  0x0020,   /* U+A4D7 */
  0x0020,   /* U+A4D8 */
  0x0020,   /* U+A4D9 */
  0x0020,   /* U+A4DA */
  0x0020,   /* U+A4DB */
  0x0020,   /* U+A4DC */
  0x0020,   /* U+A4DD */
  0x0020,   /* U+A4DE */
  0x0020,   /* U+A4DF */
  0x0020,   /* U+A4E0 */
  0x0020,   /* U+A4E1 */
  0x0020,   /* U+A4E2 */
  0x0020,   /* U+A4E3 */
  0x0020,   /* U+A4E4 */
  0x0020,   /* U+A4E5 */
  0x0020,   /* U+A4E6 */
  0x0020,   /* U+A4E7 */
  0x0020,   /* U+A4E8 */
  0x0020,   /* U+A4E9 */
  0x0020,   /* U+A4EA */
  0x0020,   /* U+A4EB */
  0x0020,   /* U+A4EC */
  0x0020,   /* U+A4ED */
  0x0020,   /* U+A4EE */
  0x0020,   /* U+A4EF */
  0x0020,   /* U+A4F0 */
  0x0020,   /* U+A4F1 */
  0x0020,   /* U+A4F2 */
  0x0020,   /* U+A4F3 */
  0x0020,   /* U+A4F4 */
  0x0020,   /* U+A4F5 */
  0x0020,   /* U+A4F6 */
  0x0020,   /* U+A4F7 */
  0x0020,   /* U+A4F8 */
  0x0020,   /* U+A4F9 */
  0x0020,   /* U+A4FA */
  0x0020,   /* U+A4FB */
  0x0020,   /* U+A4FC */
  0x0020,   /* U+A4FD */
  0x0020,   /* U+A4FE */
  0x0020,   /* U+A4FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A400 */
  0x0002,   /* U+A401 */
  0x0002,   /* U+A402 */
  0x0002,   /* U+A403 */
  0x0002,   /* U+A404 */
  0x0002,   /* U+A405 */
  0x0002,   /* U+A406 */
  0x0002,   /* U+A407 */
  0x0002,   /* U+A408 */
  0x0002,   /* U+A409 */
  0x0002,   /* U+A40A */
  0x0002,   /* U+A40B */
  0x0002,   /* U+A40C */
  0x0002,   /* U+A40D */
  0x0002,   /* U+A40E */
  0x0002,   /* U+A40F */
  0x0002,   /* U+A410 */
  0x0002,   /* U+A411 */
  0x0002,   /* U+A412 */
  0x0002,   /* U+A413 */
  0x0002,   /* U+A414 */
  0x0002,   /* U+A415 */
  0x0002,   /* U+A416 */
  0x0002,   /* U+A417 */
  0x0002,   /* U+A418 */
  0x0002,   /* U+A419 */
  0x0002,   /* U+A41A */
  0x0002,   /* U+A41B */
  0x0002,   /* U+A41C */
  0x0002,   /* U+A41D */
  0x0002,   /* U+A41E */
  0x0002,   /* U+A41F */
  0x0002,   /* U+A420 */
  0x0002,   /* U+A421 */
  0x0002,   /* U+A422 */
  0x0002,   /* U+A423 */
  0x0002,   /* U+A424 */
  0x0002,   /* U+A425 */
  0x0002,   /* U+A426 */
  0x0002,   /* U+A427 */
  0x0002,   /* U+A428 */
  0x0002,   /* U+A429 */
  0x0002,   /* U+A42A */
  0x0002,   /* U+A42B */
  0x0002,   /* U+A42C */
  0x0002,   /* U+A42D */
  0x0002,   /* U+A42E */
  0x0002,   /* U+A42F */
  0x0002,   /* U+A430 */
  0x0002,   /* U+A431 */
  0x0002,   /* U+A432 */
  0x0002,   /* U+A433 */
  0x0002,   /* U+A434 */
  0x0002,   /* U+A435 */
  0x0002,   /* U+A436 */
  0x0002,   /* U+A437 */
  0x0002,   /* U+A438 */
  0x0002,   /* U+A439 */
  0x0002,   /* U+A43A */
  0x0002,   /* U+A43B */
  0x0002,   /* U+A43C */
  0x0002,   /* U+A43D */
  0x0002,   /* U+A43E */
  0x0002,   /* U+A43F */
  0x0002,   /* U+A440 */
  0x0002,   /* U+A441 */
  0x0002,   /* U+A442 */
  0x0002,   /* U+A443 */
  0x0002,   /* U+A444 */
  0x0002,   /* U+A445 */
  0x0002,   /* U+A446 */
  0x0002,   /* U+A447 */
  0x0002,   /* U+A448 */
  0x0002,   /* U+A449 */
  0x0002,   /* U+A44A */
  0x0002,   /* U+A44B */
  0x0002,   /* U+A44C */
  0x0002,   /* U+A44D */
  0x0002,   /* U+A44E */
  0x0002,   /* U+A44F */
  0x0002,   /* U+A450 */
  0x0002,   /* U+A451 */
  0x0002,   /* U+A452 */
  0x0002,   /* U+A453 */
  0x0002,   /* U+A454 */
  0x0002,   /* U+A455 */
  0x0002,   /* U+A456 */
  0x0002,   /* U+A457 */
  0x0002,   /* U+A458 */
  0x0002,   /* U+A459 */
  0x0002,   /* U+A45A */
  0x0002,   /* U+A45B */
  0x0002,   /* U+A45C */
  0x0002,   /* U+A45D */
  0x0002,   /* U+A45E */
  0x0002,   /* U+A45F */
  0x0002,   /* U+A460 */
  0x0002,   /* U+A461 */
  0x0002,   /* U+A462 */
  0x0002,   /* U+A463 */
  0x0002,   /* U+A464 */
  0x0002,   /* U+A465 */
  0x0002,   /* U+A466 */
  0x0002,   /* U+A467 */
  0x0002,   /* U+A468 */
  0x0002,   /* U+A469 */
  0x0002,   /* U+A46A */
  0x0002,   /* U+A46B */
  0x0002,   /* U+A46C */
  0x0002,   /* U+A46D */
  0x0002,   /* U+A46E */
  0x0002,   /* U+A46F */
  0x0002,   /* U+A470 */
  0x0002,   /* U+A471 */
  0x0002,   /* U+A472 */
  0x0002,   /* U+A473 */
  0x0002,   /* U+A474 */
  0x0002,   /* U+A475 */
  0x0002,   /* U+A476 */
  0x0002,   /* U+A477 */
  0x0002,   /* U+A478 */
  0x0002,   /* U+A479 */
  0x0002,   /* U+A47A */
  0x0002,   /* U+A47B */
  0x0002,   /* U+A47C */
  0x0002,   /* U+A47D */
  0x0002,   /* U+A47E */
  0x0002,   /* U+A47F */
  0x0002,   /* U+A480 */
  0x0002,   /* U+A481 */
  0x0002,   /* U+A482 */
  0x0002,   /* U+A483 */
  0x0002,   /* U+A484 */
  0x0002,   /* U+A485 */
  0x0002,   /* U+A486 */
  0x0002,   /* U+A487 */
  0x0002,   /* U+A488 */
  0x0002,   /* U+A489 */
  0x0002,   /* U+A48A */
  0x0002,   /* U+A48B */
  0x0002,   /* U+A48C */
  0x0002,   /* U+A48D */
  0x0002,   /* U+A48E */
  0x0002,   /* U+A48F */
  0x0002,   /* U+A490 */
  0x0002,   /* U+A491 */
  0x0002,   /* U+A492 */
  0x0002,   /* U+A493 */
  0x0002,   /* U+A494 */
  0x0002,   /* U+A495 */
  0x0002,   /* U+A496 */
  0x0002,   /* U+A497 */
  0x0002,   /* U+A498 */
  0x0002,   /* U+A499 */
  0x0002,   /* U+A49A */
  0x0002,   /* U+A49B */
  0x0002,   /* U+A49C */
  0x0002,   /* U+A49D */
  0x0002,   /* U+A49E */
  0x0002,   /* U+A49F */
  0x0002,   /* U+A4A0 */
  0x0002,   /* U+A4A1 */
  0x0002,   /* U+A4A2 */
  0x0002,   /* U+A4A3 */
  0x0002,   /* U+A4A4 */
  0x0002,   /* U+A4A5 */
  0x0002,   /* U+A4A6 */
  0x0002,   /* U+A4A7 */
  0x0002,   /* U+A4A8 */
  0x0002,   /* U+A4A9 */
  0x0002,   /* U+A4AA */
  0x0002,   /* U+A4AB */
  0x0002,   /* U+A4AC */
  0x0002,   /* U+A4AD */
  0x0002,   /* U+A4AE */
  0x0002,   /* U+A4AF */
  0x0002,   /* U+A4B0 */
  0x0002,   /* U+A4B1 */
  0x0002,   /* U+A4B2 */
  0x0002,   /* U+A4B3 */
  0x0002,   /* U+A4B4 */
  0x0002,   /* U+A4B5 */
  0x0002,   /* U+A4B6 */
  0x0002,   /* U+A4B7 */
  0x0002,   /* U+A4B8 */
  0x0002,   /* U+A4B9 */
  0x0002,   /* U+A4BA */
  0x0002,   /* U+A4BB */
  0x0002,   /* U+A4BC */
  0x0002,   /* U+A4BD */
  0x0002,   /* U+A4BE */
  0x0002,   /* U+A4BF */
  0x0002,   /* U+A4C0 */
  0x0002,   /* U+A4C1 */
  0x0002,   /* U+A4C2 */
  0x0002,   /* U+A4C3 */
  0x0002,   /* U+A4C4 */
  0x0002,   /* U+A4C5 */
  0x0002,   /* U+A4C6 */
  0x0002,   /* U+A4C7 */
  0x0002,   /* U+A4C8 */
  0x0002,   /* U+A4C9 */
  0x0002,   /* U+A4CA */
  0x0002,   /* U+A4CB */
  0x0002,   /* U+A4CC */
  0x0002,   /* U+A4CD */
  0x0002,   /* U+A4CE */
  0x0002,   /* U+A4CF */
  0x0002,   /* U+A4D0 */
  0x0002,   /* U+A4D1 */
  0x0002,   /* U+A4D2 */
  0x0002,   /* U+A4D3 */
  0x0002,   /* U+A4D4 */
  0x0002,   /* U+A4D5 */
  0x0002,   /* U+A4D6 */
  0x0002,   /* U+A4D7 */
  0x0002,   /* U+A4D8 */
  0x0002,   /* U+A4D9 */
  0x0002,   /* U+A4DA */
  0x0002,   /* U+A4DB */
  0x0002,   /* U+A4DC */
  0x0002,   /* U+A4DD */
  0x0002,   /* U+A4DE */
  0x0002,   /* U+A4DF */
  0x0002,   /* U+A4E0 */
  0x0002,   /* U+A4E1 */
  0x0002,   /* U+A4E2 */
  0x0002,   /* U+A4E3 */
  0x0002,   /* U+A4E4 */
  0x0002,   /* U+A4E5 */
  0x0002,   /* U+A4E6 */
  0x0002,   /* U+A4E7 */
  0x0002,   /* U+A4E8 */
  0x0002,   /* U+A4E9 */
  0x0002,   /* U+A4EA */
  0x0002,   /* U+A4EB */
  0x0002,   /* U+A4EC */
  0x0002,   /* U+A4ED */
  0x0002,   /* U+A4EE */
  0x0002,   /* U+A4EF */
  0x0002,   /* U+A4F0 */
  0x0002,   /* U+A4F1 */
  0x0002,   /* U+A4F2 */
  0x0002,   /* U+A4F3 */
  0x0002,   /* U+A4F4 */
  0x0002,   /* U+A4F5 */
  0x0002,   /* U+A4F6 */
  0x0002,   /* U+A4F7 */
  0x0002,   /* U+A4F8 */
  0x0002,   /* U+A4F9 */
  0x0002,   /* U+A4FA */
  0x0002,   /* U+A4FB */
  0x0002,   /* U+A4FC */
  0x0002,   /* U+A4FD */
  0x0002,   /* U+A4FE */
  0x0002,   /* U+A4FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+A400 */
  0x0000,   /* U+A401 */
  0x0000,   /* U+A402 */
  0x0000,   /* U+A403 */
  0x0000,   /* U+A404 */
  0x0000,   /* U+A405 */
  0x0000,   /* U+A406 */
  0x0000,   /* U+A407 */
  0x0000,   /* U+A408 */
  0x0000,   /* U+A409 */
  0x0000,   /* U+A40A */
  0x0000,   /* U+A40B */
  0x0000,   /* U+A40C */
  0x0000,   /* U+A40D */
  0x0000,   /* U+A40E */
  0x0000,   /* U+A40F */
  0x0000,   /* U+A410 */
  0x0000,   /* U+A411 */
  0x0000,   /* U+A412 */
  0x0000,   /* U+A413 */
  0x0000,   /* U+A414 */
  0x0000,   /* U+A415 */
  0x0000,   /* U+A416 */
  0x0000,   /* U+A417 */
  0x0000,   /* U+A418 */
  0x0000,   /* U+A419 */
  0x0000,   /* U+A41A */
  0x0000,   /* U+A41B */
  0x0000,   /* U+A41C */
  0x0000,   /* U+A41D */
  0x0000,   /* U+A41E */
  0x0000,   /* U+A41F */
  0x0000,   /* U+A420 */
  0x0000,   /* U+A421 */
  0x0000,   /* U+A422 */
  0x0000,   /* U+A423 */
  0x0000,   /* U+A424 */
  0x0000,   /* U+A425 */
  0x0000,   /* U+A426 */
  0x0000,   /* U+A427 */
  0x0000,   /* U+A428 */
  0x0000,   /* U+A429 */
  0x0000,   /* U+A42A */
  0x0000,   /* U+A42B */
  0x0000,   /* U+A42C */
  0x0000,   /* U+A42D */
  0x0000,   /* U+A42E */
  0x0000,   /* U+A42F */
  0x0000,   /* U+A430 */
  0x0000,   /* U+A431 */
  0x0000,   /* U+A432 */
  0x0000,   /* U+A433 */
  0x0000,   /* U+A434 */
  0x0000,   /* U+A435 */
  0x0000,   /* U+A436 */
  0x0000,   /* U+A437 */
  0x0000,   /* U+A438 */
  0x0000,   /* U+A439 */
  0x0000,   /* U+A43A */
  0x0000,   /* U+A43B */
  0x0000,   /* U+A43C */
  0x0000,   /* U+A43D */
  0x0000,   /* U+A43E */
  0x0000,   /* U+A43F */
  0x0000,   /* U+A440 */
  0x0000,   /* U+A441 */
  0x0000,   /* U+A442 */
  0x0000,   /* U+A443 */
  0x0000,   /* U+A444 */
  0x0000,   /* U+A445 */
  0x0000,   /* U+A446 */
  0x0000,   /* U+A447 */
  0x0000,   /* U+A448 */
  0x0000,   /* U+A449 */
  0x0000,   /* U+A44A */
  0x0000,   /* U+A44B */
  0x0000,   /* U+A44C */
  0x0000,   /* U+A44D */
  0x0000,   /* U+A44E */
  0x0000,   /* U+A44F */
  0x0000,   /* U+A450 */
  0x0000,   /* U+A451 */
  0x0000,   /* U+A452 */
  0x0000,   /* U+A453 */
  0x0000,   /* U+A454 */
  0x0000,   /* U+A455 */
  0x0000,   /* U+A456 */
  0x0000,   /* U+A457 */
  0x0000,   /* U+A458 */
  0x0000,   /* U+A459 */
  0x0000,   /* U+A45A */
  0x0000,   /* U+A45B */
  0x0000,   /* U+A45C */
  0x0000,   /* U+A45D */
  0x0000,   /* U+A45E */
  0x0000,   /* U+A45F */
  0x0000,   /* U+A460 */
  0x0000,   /* U+A461 */
  0x0000,   /* U+A462 */
  0x0000,   /* U+A463 */
  0x0000,   /* U+A464 */
  0x0000,   /* U+A465 */
  0x0000,   /* U+A466 */
  0x0000,   /* U+A467 */
  0x0000,   /* U+A468 */
  0x0000,   /* U+A469 */
  0x0000,   /* U+A46A */
  0x0000,   /* U+A46B */
  0x0000,   /* U+A46C */
  0x0000,   /* U+A46D */
  0x0000,   /* U+A46E */
  0x0000,   /* U+A46F */
  0x0000,   /* U+A470 */
  0x0000,   /* U+A471 */
  0x0000,   /* U+A472 */
  0x0000,   /* U+A473 */
  0x0000,   /* U+A474 */
  0x0000,   /* U+A475 */
  0x0000,   /* U+A476 */
  0x0000,   /* U+A477 */
  0x0000,   /* U+A478 */
  0x0000,   /* U+A479 */
  0x0000,   /* U+A47A */
  0x0000,   /* U+A47B */
  0x0000,   /* U+A47C */
  0x0000,   /* U+A47D */
  0x0000,   /* U+A47E */
  0x0000,   /* U+A47F */
  0x0000,   /* U+A480 */
  0x0000,   /* U+A481 */
  0x0000,   /* U+A482 */
  0x0000,   /* U+A483 */
  0x0000,   /* U+A484 */
  0x0000,   /* U+A485 */
  0x0000,   /* U+A486 */
  0x0000,   /* U+A487 */
  0x0000,   /* U+A488 */
  0x0000,   /* U+A489 */
  0x0000,   /* U+A48A */
  0x0000,   /* U+A48B */
  0x0000,   /* U+A48C */
  0xA48D,   /* U+A48D */
  0xA48E,   /* U+A48E */
  0xA48F,   /* U+A48F */
  0x0000,   /* U+A490 */
  0x0000,   /* U+A491 */
  0x0000,   /* U+A492 */
  0x0000,   /* U+A493 */
  0x0000,   /* U+A494 */
  0x0000,   /* U+A495 */
  0x0000,   /* U+A496 */
  0x0000,   /* U+A497 */
  0x0000,   /* U+A498 */
  0x0000,   /* U+A499 */
  0x0000,   /* U+A49A */
  0x0000,   /* U+A49B */
  0x0000,   /* U+A49C */
  0x0000,   /* U+A49D */
  0x0000,   /* U+A49E */
  0x0000,   /* U+A49F */
  0x0000,   /* U+A4A0 */
  0x0000,   /* U+A4A1 */
  0x0000,   /* U+A4A2 */
  0x0000,   /* U+A4A3 */
  0x0000,   /* U+A4A4 */
  0x0000,   /* U+A4A5 */
  0x0000,   /* U+A4A6 */
  0x0000,   /* U+A4A7 */
  0x0000,   /* U+A4A8 */
  0x0000,   /* U+A4A9 */
  0x0000,   /* U+A4AA */
  0x0000,   /* U+A4AB */
  0x0000,   /* U+A4AC */
  0x0000,   /* U+A4AD */
  0x0000,   /* U+A4AE */
  0x0000,   /* U+A4AF */
  0x0000,   /* U+A4B0 */
  0x0000,   /* U+A4B1 */
  0x0000,   /* U+A4B2 */
  0x0000,   /* U+A4B3 */
  0x0000,   /* U+A4B4 */
  0x0000,   /* U+A4B5 */
  0x0000,   /* U+A4B6 */
  0x0000,   /* U+A4B7 */
  0x0000,   /* U+A4B8 */
  0x0000,   /* U+A4B9 */
  0x0000,   /* U+A4BA */
  0x0000,   /* U+A4BB */
  0x0000,   /* U+A4BC */
  0x0000,   /* U+A4BD */
  0x0000,   /* U+A4BE */
  0x0000,   /* U+A4BF */
  0x0000,   /* U+A4C0 */
  0x0000,   /* U+A4C1 */
  0x0000,   /* U+A4C2 */
  0x0000,   /* U+A4C3 */
  0x0000,   /* U+A4C4 */
  0x0000,   /* U+A4C5 */
  0x0000,   /* U+A4C6 */
  0xA4C7,   /* U+A4C7 */
  0xA4C8,   /* U+A4C8 */
  0xA4C9,   /* U+A4C9 */
  0xA4CA,   /* U+A4CA */
  0xA4CB,   /* U+A4CB */
  0xA4CC,   /* U+A4CC */
  0xA4CD,   /* U+A4CD */
  0xA4CE,   /* U+A4CE */
  0xA4CF,   /* U+A4CF */
  0x0000,   /* U+A4D0 */
  0x0000,   /* U+A4D1 */
  0x0000,   /* U+A4D2 */
  0x0000,   /* U+A4D3 */
  0x0000,   /* U+A4D4 */
  0x0000,   /* U+A4D5 */
  0x0000,   /* U+A4D6 */
  0x0000,   /* U+A4D7 */
  0x0000,   /* U+A4D8 */
  0x0000,   /* U+A4D9 */
  0x0000,   /* U+A4DA */
  0x0000,   /* U+A4DB */
  0x0000,   /* U+A4DC */
  0x0000,   /* U+A4DD */
  0x0000,   /* U+A4DE */
  0x0000,   /* U+A4DF */
  0x0000,   /* U+A4E0 */
  0x0000,   /* U+A4E1 */
  0x0000,   /* U+A4E2 */
  0x0000,   /* U+A4E3 */
  0x0000,   /* U+A4E4 */
  0x0000,   /* U+A4E5 */
  0x0000,   /* U+A4E6 */
  0x0000,   /* U+A4E7 */
  0x0000,   /* U+A4E8 */
  0x0000,   /* U+A4E9 */
  0x0000,   /* U+A4EA */
  0x0000,   /* U+A4EB */
  0x0000,   /* U+A4EC */
  0x0000,   /* U+A4ED */
  0x0000,   /* U+A4EE */
  0x0000,   /* U+A4EF */
  0x0000,   /* U+A4F0 */
  0x0000,   /* U+A4F1 */
  0x0000,   /* U+A4F2 */
  0x0000,   /* U+A4F3 */
  0x0000,   /* U+A4F4 */
  0x0000,   /* U+A4F5 */
  0x0000,   /* U+A4F6 */
  0x0000,   /* U+A4F7 */
  0x0000,   /* U+A4F8 */
  0x0000,   /* U+A4F9 */
  0x0000,   /* U+A4FA */
  0x0000,   /* U+A4FB */
  0x0000,   /* U+A4FC */
  0x0000,   /* U+A4FD */
  0x0000,   /* U+A4FE */
  0x0000,   /* U+A4FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+A400 */
  0x0000,   /* U+A401 */
  0x0000,   /* U+A402 */
  0x0000,   /* U+A403 */
  0x0000,   /* U+A404 */
  0x0000,   /* U+A405 */
  0x0000,   /* U+A406 */
  0x0000,   /* U+A407 */
  0x0000,   /* U+A408 */
  0x0000,   /* U+A409 */
  0x0000,   /* U+A40A */
  0x0000,   /* U+A40B */
  0x0000,   /* U+A40C */
  0x0000,   /* U+A40D */
  0x0000,   /* U+A40E */
  0x0000,   /* U+A40F */
  0x0000,   /* U+A410 */
  0x0000,   /* U+A411 */
  0x0000,   /* U+A412 */
  0x0000,   /* U+A413 */
  0x0000,   /* U+A414 */
  0x0000,   /* U+A415 */
  0x0000,   /* U+A416 */
  0x0000,   /* U+A417 */
  0x0000,   /* U+A418 */
  0x0000,   /* U+A419 */
  0x0000,   /* U+A41A */
  0x0000,   /* U+A41B */
  0x0000,   /* U+A41C */
  0x0000,   /* U+A41D */
  0x0000,   /* U+A41E */
  0x0000,   /* U+A41F */
  0x0000,   /* U+A420 */
  0x0000,   /* U+A421 */
  0x0000,   /* U+A422 */
  0x0000,   /* U+A423 */
  0x0000,   /* U+A424 */
  0x0000,   /* U+A425 */
  0x0000,   /* U+A426 */
  0x0000,   /* U+A427 */
  0x0000,   /* U+A428 */
  0x0000,   /* U+A429 */
  0x0000,   /* U+A42A */
  0x0000,   /* U+A42B */
  0x0000,   /* U+A42C */
  0x0000,   /* U+A42D */
  0x0000,   /* U+A42E */
  0x0000,   /* U+A42F */
  0x0000,   /* U+A430 */
  0x0000,   /* U+A431 */
  0x0000,   /* U+A432 */
  0x0000,   /* U+A433 */
  0x0000,   /* U+A434 */
  0x0000,   /* U+A435 */
  0x0000,   /* U+A436 */
  0x0000,   /* U+A437 */
  0x0000,   /* U+A438 */
  0x0000,   /* U+A439 */
  0x0000,   /* U+A43A */
  0x0000,   /* U+A43B */
  0x0000,   /* U+A43C */
  0x0000,   /* U+A43D */
  0x0000,   /* U+A43E */
  0x0000,   /* U+A43F */
  0x0000,   /* U+A440 */
  0x0000,   /* U+A441 */
  0x0000,   /* U+A442 */
  0x0000,   /* U+A443 */
  0x0000,   /* U+A444 */
  0x0000,   /* U+A445 */
  0x0000,   /* U+A446 */
  0x0000,   /* U+A447 */
  0x0000,   /* U+A448 */
  0x0000,   /* U+A449 */
  0x0000,   /* U+A44A */
  0x0000,   /* U+A44B */
  0x0000,   /* U+A44C */
  0x0000,   /* U+A44D */
  0x0000,   /* U+A44E */
  0x0000,   /* U+A44F */
  0x0000,   /* U+A450 */
  0x0000,   /* U+A451 */
  0x0000,   /* U+A452 */
  0x0000,   /* U+A453 */
  0x0000,   /* U+A454 */
  0x0000,   /* U+A455 */
  0x0000,   /* U+A456 */
  0x0000,   /* U+A457 */
  0x0000,   /* U+A458 */
  0x0000,   /* U+A459 */
  0x0000,   /* U+A45A */
  0x0000,   /* U+A45B */
  0x0000,   /* U+A45C */
  0x0000,   /* U+A45D */
  0x0000,   /* U+A45E */
  0x0000,   /* U+A45F */
  0x0000,   /* U+A460 */
  0x0000,   /* U+A461 */
  0x0000,   /* U+A462 */
  0x0000,   /* U+A463 */
  0x0000,   /* U+A464 */
  0x0000,   /* U+A465 */
  0x0000,   /* U+A466 */
  0x0000,   /* U+A467 */
  0x0000,   /* U+A468 */
  0x0000,   /* U+A469 */
  0x0000,   /* U+A46A */
  0x0000,   /* U+A46B */
  0x0000,   /* U+A46C */
  0x0000,   /* U+A46D */
  0x0000,   /* U+A46E */
  0x0000,   /* U+A46F */
  0x0000,   /* U+A470 */
  0x0000,   /* U+A471 */
  0x0000,   /* U+A472 */
  0x0000,   /* U+A473 */
  0x0000,   /* U+A474 */
  0x0000,   /* U+A475 */
  0x0000,   /* U+A476 */
  0x0000,   /* U+A477 */
  0x0000,   /* U+A478 */
  0x0000,   /* U+A479 */
  0x0000,   /* U+A47A */
  0x0000,   /* U+A47B */
  0x0000,   /* U+A47C */
  0x0000,   /* U+A47D */
  0x0000,   /* U+A47E */
  0x0000,   /* U+A47F */
  0x0000,   /* U+A480 */
  0x0000,   /* U+A481 */
  0x0000,   /* U+A482 */
  0x0000,   /* U+A483 */
  0x0000,   /* U+A484 */
  0x0000,   /* U+A485 */
  0x0000,   /* U+A486 */
  0x0000,   /* U+A487 */
  0x0000,   /* U+A488 */
  0x0000,   /* U+A489 */
  0x0000,   /* U+A48A */
  0x0000,   /* U+A48B */
  0x0000,   /* U+A48C */
  0x0000,   /* U+A48D */
  0x0000,   /* U+A48E */
  0x0000,   /* U+A48F */
  0x0000,   /* U+A490 */
  0x0000,   /* U+A491 */
  0x0000,   /* U+A492 */
  0x0000,   /* U+A493 */
  0x0000,   /* U+A494 */
  0x0000,   /* U+A495 */
  0x0000,   /* U+A496 */
  0x0000,   /* U+A497 */
  0x0000,   /* U+A498 */
  0x0000,   /* U+A499 */
  0x0000,   /* U+A49A */
  0x0000,   /* U+A49B */
  0x0000,   /* U+A49C */
  0x0000,   /* U+A49D */
  0x0000,   /* U+A49E */
  0x0000,   /* U+A49F */
  0x0000,   /* U+A4A0 */
  0x0000,   /* U+A4A1 */
  0x0000,   /* U+A4A2 */
  0x0000,   /* U+A4A3 */
  0x0000,   /* U+A4A4 */
  0x0000,   /* U+A4A5 */
  0x0000,   /* U+A4A6 */
  0x0000,   /* U+A4A7 */
  0x0000,   /* U+A4A8 */
  0x0000,   /* U+A4A9 */
  0x0000,   /* U+A4AA */
  0x0000,   /* U+A4AB */
  0x0000,   /* U+A4AC */
  0x0000,   /* U+A4AD */
  0x0000,   /* U+A4AE */
  0x0000,   /* U+A4AF */
  0x0000,   /* U+A4B0 */
  0x0000,   /* U+A4B1 */
  0x0000,   /* U+A4B2 */
  0x0000,   /* U+A4B3 */
  0x0000,   /* U+A4B4 */
  0x0000,   /* U+A4B5 */
  0x0000,   /* U+A4B6 */
  0x0000,   /* U+A4B7 */
  0x0000,   /* U+A4B8 */
  0x0000,   /* U+A4B9 */
  0x0000,   /* U+A4BA */
  0x0000,   /* U+A4BB */
  0x0000,   /* U+A4BC */
  0x0000,   /* U+A4BD */
  0x0000,   /* U+A4BE */
  0x0000,   /* U+A4BF */
  0x0000,   /* U+A4C0 */
  0x0000,   /* U+A4C1 */
  0x0000,   /* U+A4C2 */
  0x0000,   /* U+A4C3 */
  0x0000,   /* U+A4C4 */
  0x0000,   /* U+A4C5 */
  0x0000,   /* U+A4C6 */
  0x0000,   /* U+A4C7 */
  0x0000,   /* U+A4C8 */
  0x0000,   /* U+A4C9 */
  0x0000,   /* U+A4CA */
  0x0000,   /* U+A4CB */
  0x0000,   /* U+A4CC */
  0x0000,   /* U+A4CD */
  0x0000,   /* U+A4CE */
  0x0000,   /* U+A4CF */
  0x0000,   /* U+A4D0 */
  0x0000,   /* U+A4D1 */
  0x0000,   /* U+A4D2 */
  0x0000,   /* U+A4D3 */
  0x0000,   /* U+A4D4 */
  0x0000,   /* U+A4D5 */
  0x0000,   /* U+A4D6 */
  0x0000,   /* U+A4D7 */
  0x0000,   /* U+A4D8 */
  0x0000,   /* U+A4D9 */
  0x0000,   /* U+A4DA */
  0x0000,   /* U+A4DB */
  0x0000,   /* U+A4DC */
  0x0000,   /* U+A4DD */
  0x0000,   /* U+A4DE */
  0x0000,   /* U+A4DF */
  0x0000,   /* U+A4E0 */
  0x0000,   /* U+A4E1 */
  0x0000,   /* U+A4E2 */
  0x0000,   /* U+A4E3 */
  0x0000,   /* U+A4E4 */
  0x0000,   /* U+A4E5 */
  0x0000,   /* U+A4E6 */
  0x0000,   /* U+A4E7 */
  0x0000,   /* U+A4E8 */
  0x0000,   /* U+A4E9 */
  0x0000,   /* U+A4EA */
  0x0000,   /* U+A4EB */
  0x0000,   /* U+A4EC */
  0x0000,   /* U+A4ED */
  0x0000,   /* U+A4EE */
  0x0000,   /* U+A4EF */
  0x0000,   /* U+A4F0 */
  0x0000,   /* U+A4F1 */
  0x0000,   /* U+A4F2 */
  0x0000,   /* U+A4F3 */
  0x0000,   /* U+A4F4 */
  0x0000,   /* U+A4F5 */
  0x0000,   /* U+A4F6 */
  0x0000,   /* U+A4F7 */
  0x0000,   /* U+A4F8 */
  0x0000,   /* U+A4F9 */
  0x0000,   /* U+A4FA */
  0x0000,   /* U+A4FB */
  0x0000,   /* U+A4FC */
  0x0000,   /* U+A4FD */
  0x0000,   /* U+A4FE */
  0x0000,   /* U+A4FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+A400 */
  0x0000,   /* U+A401 */
  0x0000,   /* U+A402 */
  0x0000,   /* U+A403 */
  0x0000,   /* U+A404 */
  0x0000,   /* U+A405 */
  0x0000,   /* U+A406 */
  0x0000,   /* U+A407 */
  0x0000,   /* U+A408 */
  0x0000,   /* U+A409 */
  0x0000,   /* U+A40A */
  0x0000,   /* U+A40B */
  0x0000,   /* U+A40C */
  0x0000,   /* U+A40D */
  0x0000,   /* U+A40E */
  0x0000,   /* U+A40F */
  0x0000,   /* U+A410 */
  0x0000,   /* U+A411 */
  0x0000,   /* U+A412 */
  0x0000,   /* U+A413 */
  0x0000,   /* U+A414 */
  0x0000,   /* U+A415 */
  0x0000,   /* U+A416 */
  0x0000,   /* U+A417 */
  0x0000,   /* U+A418 */
  0x0000,   /* U+A419 */
  0x0000,   /* U+A41A */
  0x0000,   /* U+A41B */
  0x0000,   /* U+A41C */
  0x0000,   /* U+A41D */
  0x0000,   /* U+A41E */
  0x0000,   /* U+A41F */
  0x0000,   /* U+A420 */
  0x0000,   /* U+A421 */
  0x0000,   /* U+A422 */
  0x0000,   /* U+A423 */
  0x0000,   /* U+A424 */
  0x0000,   /* U+A425 */
  0x0000,   /* U+A426 */
  0x0000,   /* U+A427 */
  0x0000,   /* U+A428 */
  0x0000,   /* U+A429 */
  0x0000,   /* U+A42A */
  0x0000,   /* U+A42B */
  0x0000,   /* U+A42C */
  0x0000,   /* U+A42D */
  0x0000,   /* U+A42E */
  0x0000,   /* U+A42F */
  0x0000,   /* U+A430 */
  0x0000,   /* U+A431 */
  0x0000,   /* U+A432 */
  0x0000,   /* U+A433 */
  0x0000,   /* U+A434 */
  0x0000,   /* U+A435 */
  0x0000,   /* U+A436 */
  0x0000,   /* U+A437 */
  0x0000,   /* U+A438 */
  0x0000,   /* U+A439 */
  0x0000,   /* U+A43A */
  0x0000,   /* U+A43B */
  0x0000,   /* U+A43C */
  0x0000,   /* U+A43D */
  0x0000,   /* U+A43E */
  0x0000,   /* U+A43F */
  0x0000,   /* U+A440 */
  0x0000,   /* U+A441 */
  0x0000,   /* U+A442 */
  0x0000,   /* U+A443 */
  0x0000,   /* U+A444 */
  0x0000,   /* U+A445 */
  0x0000,   /* U+A446 */
  0x0000,   /* U+A447 */
  0x0000,   /* U+A448 */
  0x0000,   /* U+A449 */
  0x0000,   /* U+A44A */
  0x0000,   /* U+A44B */
  0x0000,   /* U+A44C */
  0x0000,   /* U+A44D */
  0x0000,   /* U+A44E */
  0x0000,   /* U+A44F */
  0x0000,   /* U+A450 */
  0x0000,   /* U+A451 */
  0x0000,   /* U+A452 */
  0x0000,   /* U+A453 */
  0x0000,   /* U+A454 */
  0x0000,   /* U+A455 */
  0x0000,   /* U+A456 */
  0x0000,   /* U+A457 */
  0x0000,   /* U+A458 */
  0x0000,   /* U+A459 */
  0x0000,   /* U+A45A */
  0x0000,   /* U+A45B */
  0x0000,   /* U+A45C */
  0x0000,   /* U+A45D */
  0x0000,   /* U+A45E */
  0x0000,   /* U+A45F */
  0x0000,   /* U+A460 */
  0x0000,   /* U+A461 */
  0x0000,   /* U+A462 */
  0x0000,   /* U+A463 */
  0x0000,   /* U+A464 */
  0x0000,   /* U+A465 */
  0x0000,   /* U+A466 */
  0x0000,   /* U+A467 */
  0x0000,   /* U+A468 */
  0x0000,   /* U+A469 */
  0x0000,   /* U+A46A */
  0x0000,   /* U+A46B */
  0x0000,   /* U+A46C */
  0x0000,   /* U+A46D */
  0x0000,   /* U+A46E */
  0x0000,   /* U+A46F */
  0x0000,   /* U+A470 */
  0x0000,   /* U+A471 */
  0x0000,   /* U+A472 */
  0x0000,   /* U+A473 */
  0x0000,   /* U+A474 */
  0x0000,   /* U+A475 */
  0x0000,   /* U+A476 */
  0x0000,   /* U+A477 */
  0x0000,   /* U+A478 */
  0x0000,   /* U+A479 */
  0x0000,   /* U+A47A */
  0x0000,   /* U+A47B */
  0x0000,   /* U+A47C */
  0x0000,   /* U+A47D */
  0x0000,   /* U+A47E */
  0x0000,   /* U+A47F */
  0x0000,   /* U+A480 */
  0x0000,   /* U+A481 */
  0x0000,   /* U+A482 */
  0x0000,   /* U+A483 */
  0x0000,   /* U+A484 */
  0x0000,   /* U+A485 */
  0x0000,   /* U+A486 */
  0x0000,   /* U+A487 */
  0x0000,   /* U+A488 */
  0x0000,   /* U+A489 */
  0x0000,   /* U+A48A */
  0x0000,   /* U+A48B */
  0x0000,   /* U+A48C */
  0x0000,   /* U+A48D */
  0x0000,   /* U+A48E */
  0x0000,   /* U+A48F */
  0x0000,   /* U+A490 */
  0x0000,   /* U+A491 */
  0x0000,   /* U+A492 */
  0x0000,   /* U+A493 */
  0x0000,   /* U+A494 */
  0x0000,   /* U+A495 */
  0x0000,   /* U+A496 */
  0x0000,   /* U+A497 */
  0x0000,   /* U+A498 */
  0x0000,   /* U+A499 */
  0x0000,   /* U+A49A */
  0x0000,   /* U+A49B */
  0x0000,   /* U+A49C */
  0x0000,   /* U+A49D */
  0x0000,   /* U+A49E */
  0x0000,   /* U+A49F */
  0x0000,   /* U+A4A0 */
  0x0000,   /* U+A4A1 */
  0x0000,   /* U+A4A2 */
  0x0000,   /* U+A4A3 */
  0x0000,   /* U+A4A4 */
  0x0000,   /* U+A4A5 */
  0x0000,   /* U+A4A6 */
  0x0000,   /* U+A4A7 */
  0x0000,   /* U+A4A8 */
  0x0000,   /* U+A4A9 */
  0x0000,   /* U+A4AA */
  0x0000,   /* U+A4AB */
  0x0000,   /* U+A4AC */
  0x0000,   /* U+A4AD */
  0x0000,   /* U+A4AE */
  0x0000,   /* U+A4AF */
  0x0000,   /* U+A4B0 */
  0x0000,   /* U+A4B1 */
  0x0000,   /* U+A4B2 */
  0x0000,   /* U+A4B3 */
  0x0000,   /* U+A4B4 */
  0x0000,   /* U+A4B5 */
  0x0000,   /* U+A4B6 */
  0x0000,   /* U+A4B7 */
  0x0000,   /* U+A4B8 */
  0x0000,   /* U+A4B9 */
  0x0000,   /* U+A4BA */
  0x0000,   /* U+A4BB */
  0x0000,   /* U+A4BC */
  0x0000,   /* U+A4BD */
  0x0000,   /* U+A4BE */
  0x0000,   /* U+A4BF */
  0x0000,   /* U+A4C0 */
  0x0000,   /* U+A4C1 */
  0x0000,   /* U+A4C2 */
  0x0000,   /* U+A4C3 */
  0x0000,   /* U+A4C4 */
  0x0000,   /* U+A4C5 */
  0x0000,   /* U+A4C6 */
  0x0000,   /* U+A4C7 */
  0x0000,   /* U+A4C8 */
  0x0000,   /* U+A4C9 */
  0x0000,   /* U+A4CA */
  0x0000,   /* U+A4CB */
  0x0000,   /* U+A4CC */
  0x0000,   /* U+A4CD */
  0x0000,   /* U+A4CE */
  0x0000,   /* U+A4CF */
  0x0000,   /* U+A4D0 */
  0x0000,   /* U+A4D1 */
  0x0000,   /* U+A4D2 */
  0x0000,   /* U+A4D3 */
  0x0000,   /* U+A4D4 */
  0x0000,   /* U+A4D5 */
  0x0000,   /* U+A4D6 */
  0x0000,   /* U+A4D7 */
  0x0000,   /* U+A4D8 */
  0x0000,   /* U+A4D9 */
  0x0000,   /* U+A4DA */
  0x0000,   /* U+A4DB */
  0x0000,   /* U+A4DC */
  0x0000,   /* U+A4DD */
  0x0000,   /* U+A4DE */
  0x0000,   /* U+A4DF */
  0x0000,   /* U+A4E0 */
  0x0000,   /* U+A4E1 */
  0x0000,   /* U+A4E2 */
  0x0000,   /* U+A4E3 */
  0x0000,   /* U+A4E4 */
  0x0000,   /* U+A4E5 */
  0x0000,   /* U+A4E6 */
  0x0000,   /* U+A4E7 */
  0x0000,   /* U+A4E8 */
  0x0000,   /* U+A4E9 */
  0x0000,   /* U+A4EA */
  0x0000,   /* U+A4EB */
  0x0000,   /* U+A4EC */
  0x0000,   /* U+A4ED */
  0x0000,   /* U+A4EE */
  0x0000,   /* U+A4EF */
  0x0000,   /* U+A4F0 */
  0x0000,   /* U+A4F1 */
  0x0000,   /* U+A4F2 */
  0x0000,   /* U+A4F3 */
  0x0000,   /* U+A4F4 */
  0x0000,   /* U+A4F5 */
  0x0000,   /* U+A4F6 */
  0x0000,   /* U+A4F7 */
  0x0000,   /* U+A4F8 */
  0x0000,   /* U+A4F9 */
  0x0000,   /* U+A4FA */
  0x0000,   /* U+A4FB */
  0x0000,   /* U+A4FC */
  0x0000,   /* U+A4FD */
  0x0000,   /* U+A4FE */
  0x0000,   /* U+A4FF */
};

uint16 uca900_p0A5[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x375A,   /* U+A500 */
  0x375B,   /* U+A501 */
  0x375C,   /* U+A502 */
  0x375D,   /* U+A503 */
  0x375E,   /* U+A504 */
  0x375F,   /* U+A505 */
  0x3760,   /* U+A506 */
  0x3761,   /* U+A507 */
  0x3762,   /* U+A508 */
  0x3763,   /* U+A509 */
  0x3764,   /* U+A50A */
  0x3765,   /* U+A50B */
  0x3766,   /* U+A50C */
  0x3767,   /* U+A50D */
  0x3768,   /* U+A50E */
  0x3769,   /* U+A50F */
  0x376A,   /* U+A510 */
  0x376B,   /* U+A511 */
  0x376C,   /* U+A512 */
  0x376D,   /* U+A513 */
  0x376E,   /* U+A514 */
  0x376F,   /* U+A515 */
  0x3770,   /* U+A516 */
  0x3771,   /* U+A517 */
  0x3772,   /* U+A518 */
  0x3773,   /* U+A519 */
  0x3774,   /* U+A51A */
  0x3775,   /* U+A51B */
  0x3776,   /* U+A51C */
  0x3777,   /* U+A51D */
  0x3778,   /* U+A51E */
  0x3779,   /* U+A51F */
  0x377A,   /* U+A520 */
  0x377B,   /* U+A521 */
  0x377C,   /* U+A522 */
  0x377D,   /* U+A523 */
  0x377E,   /* U+A524 */
  0x377F,   /* U+A525 */
  0x3780,   /* U+A526 */
  0x3781,   /* U+A527 */
  0x3782,   /* U+A528 */
  0x3783,   /* U+A529 */
  0x3784,   /* U+A52A */
  0x3785,   /* U+A52B */
  0x3786,   /* U+A52C */
  0x3787,   /* U+A52D */
  0x3788,   /* U+A52E */
  0x3789,   /* U+A52F */
  0x378A,   /* U+A530 */
  0x378B,   /* U+A531 */
  0x378C,   /* U+A532 */
  0x378D,   /* U+A533 */
  0x378E,   /* U+A534 */
  0x378F,   /* U+A535 */
  0x3790,   /* U+A536 */
  0x3791,   /* U+A537 */
  0x3792,   /* U+A538 */
  0x3793,   /* U+A539 */
  0x3794,   /* U+A53A */
  0x3795,   /* U+A53B */
  0x3796,   /* U+A53C */
  0x3797,   /* U+A53D */
  0x3798,   /* U+A53E */
  0x3799,   /* U+A53F */
  0x379A,   /* U+A540 */
  0x379B,   /* U+A541 */
  0x379C,   /* U+A542 */
  0x379D,   /* U+A543 */
  0x379E,   /* U+A544 */
  0x379F,   /* U+A545 */
  0x37A0,   /* U+A546 */
  0x37A1,   /* U+A547 */
  0x37A2,   /* U+A548 */
  0x37A3,   /* U+A549 */
  0x37A4,   /* U+A54A */
  0x37A5,   /* U+A54B */
  0x37A6,   /* U+A54C */
  0x37A7,   /* U+A54D */
  0x37A8,   /* U+A54E */
  0x37A9,   /* U+A54F */
  0x37AA,   /* U+A550 */
  0x37AB,   /* U+A551 */
  0x37AC,   /* U+A552 */
  0x37AD,   /* U+A553 */
  0x37AE,   /* U+A554 */
  0x37AF,   /* U+A555 */
  0x37B0,   /* U+A556 */
  0x37B1,   /* U+A557 */
  0x37B2,   /* U+A558 */
  0x37B3,   /* U+A559 */
  0x37B4,   /* U+A55A */
  0x37B5,   /* U+A55B */
  0x37B6,   /* U+A55C */
  0x37B7,   /* U+A55D */
  0x37B8,   /* U+A55E */
  0x37B9,   /* U+A55F */
  0x37BA,   /* U+A560 */
  0x37BB,   /* U+A561 */
  0x37BC,   /* U+A562 */
  0x37BD,   /* U+A563 */
  0x37BE,   /* U+A564 */
  0x37BF,   /* U+A565 */
  0x37C0,   /* U+A566 */
  0x37C1,   /* U+A567 */
  0x37C2,   /* U+A568 */
  0x37C3,   /* U+A569 */
  0x37C4,   /* U+A56A */
  0x37C5,   /* U+A56B */
  0x37C6,   /* U+A56C */
  0x37C7,   /* U+A56D */
  0x37C8,   /* U+A56E */
  0x37C9,   /* U+A56F */
  0x37CA,   /* U+A570 */
  0x37CB,   /* U+A571 */
  0x37CC,   /* U+A572 */
  0x37CD,   /* U+A573 */
  0x37CE,   /* U+A574 */
  0x37CF,   /* U+A575 */
  0x37D0,   /* U+A576 */
  0x37D1,   /* U+A577 */
  0x37D2,   /* U+A578 */
  0x37D3,   /* U+A579 */
  0x37D4,   /* U+A57A */
  0x37D5,   /* U+A57B */
  0x37D6,   /* U+A57C */
  0x37D7,   /* U+A57D */
  0x37D8,   /* U+A57E */
  0x37D9,   /* U+A57F */
  0x37DA,   /* U+A580 */
  0x37DB,   /* U+A581 */
  0x37DC,   /* U+A582 */
  0x37DD,   /* U+A583 */
  0x37DE,   /* U+A584 */
  0x37DF,   /* U+A585 */
  0x37E0,   /* U+A586 */
  0x37E1,   /* U+A587 */
  0x37E2,   /* U+A588 */
  0x37E3,   /* U+A589 */
  0x37E4,   /* U+A58A */
  0x37E5,   /* U+A58B */
  0x37E6,   /* U+A58C */
  0x37E7,   /* U+A58D */
  0x37E8,   /* U+A58E */
  0x37E9,   /* U+A58F */
  0x37EA,   /* U+A590 */
  0x37EB,   /* U+A591 */
  0x37EC,   /* U+A592 */
  0x37ED,   /* U+A593 */
  0x37EE,   /* U+A594 */
  0x37EF,   /* U+A595 */
  0x37F0,   /* U+A596 */
  0x37F1,   /* U+A597 */
  0x37F2,   /* U+A598 */
  0x37F3,   /* U+A599 */
  0x37F4,   /* U+A59A */
  0x37F5,   /* U+A59B */
  0x37F6,   /* U+A59C */
  0x37F7,   /* U+A59D */
  0x37F8,   /* U+A59E */
  0x37F9,   /* U+A59F */
  0x37FA,   /* U+A5A0 */
  0x37FB,   /* U+A5A1 */
  0x37FC,   /* U+A5A2 */
  0x37FD,   /* U+A5A3 */
  0x37FE,   /* U+A5A4 */
  0x37FF,   /* U+A5A5 */
  0x3800,   /* U+A5A6 */
  0x3801,   /* U+A5A7 */
  0x3802,   /* U+A5A8 */
  0x3803,   /* U+A5A9 */
  0x3804,   /* U+A5AA */
  0x3805,   /* U+A5AB */
  0x3806,   /* U+A5AC */
  0x3807,   /* U+A5AD */
  0x3808,   /* U+A5AE */
  0x3809,   /* U+A5AF */
  0x380A,   /* U+A5B0 */
  0x380B,   /* U+A5B1 */
  0x380C,   /* U+A5B2 */
  0x380D,   /* U+A5B3 */
  0x380E,   /* U+A5B4 */
  0x380F,   /* U+A5B5 */
  0x3810,   /* U+A5B6 */
  0x3811,   /* U+A5B7 */
  0x3812,   /* U+A5B8 */
  0x3813,   /* U+A5B9 */
  0x3814,   /* U+A5BA */
  0x3815,   /* U+A5BB */
  0x3816,   /* U+A5BC */
  0x3817,   /* U+A5BD */
  0x3818,   /* U+A5BE */
  0x3819,   /* U+A5BF */
  0x381A,   /* U+A5C0 */
  0x381B,   /* U+A5C1 */
  0x381C,   /* U+A5C2 */
  0x381D,   /* U+A5C3 */
  0x381E,   /* U+A5C4 */
  0x381F,   /* U+A5C5 */
  0x3820,   /* U+A5C6 */
  0x3821,   /* U+A5C7 */
  0x3822,   /* U+A5C8 */
  0x3823,   /* U+A5C9 */
  0x3824,   /* U+A5CA */
  0x3825,   /* U+A5CB */
  0x3826,   /* U+A5CC */
  0x3827,   /* U+A5CD */
  0x3828,   /* U+A5CE */
  0x3829,   /* U+A5CF */
  0x382A,   /* U+A5D0 */
  0x382B,   /* U+A5D1 */
  0x382C,   /* U+A5D2 */
  0x382D,   /* U+A5D3 */
  0x382E,   /* U+A5D4 */
  0x382F,   /* U+A5D5 */
  0x3830,   /* U+A5D6 */
  0x3831,   /* U+A5D7 */
  0x3832,   /* U+A5D8 */
  0x3833,   /* U+A5D9 */
  0x3834,   /* U+A5DA */
  0x3835,   /* U+A5DB */
  0x3836,   /* U+A5DC */
  0x3837,   /* U+A5DD */
  0x3838,   /* U+A5DE */
  0x3839,   /* U+A5DF */
  0x383A,   /* U+A5E0 */
  0x383B,   /* U+A5E1 */
  0x383C,   /* U+A5E2 */
  0x383D,   /* U+A5E3 */
  0x383E,   /* U+A5E4 */
  0x383F,   /* U+A5E5 */
  0x3840,   /* U+A5E6 */
  0x3841,   /* U+A5E7 */
  0x3842,   /* U+A5E8 */
  0x3843,   /* U+A5E9 */
  0x3844,   /* U+A5EA */
  0x3845,   /* U+A5EB */
  0x3846,   /* U+A5EC */
  0x3847,   /* U+A5ED */
  0x3848,   /* U+A5EE */
  0x3849,   /* U+A5EF */
  0x384A,   /* U+A5F0 */
  0x384B,   /* U+A5F1 */
  0x384C,   /* U+A5F2 */
  0x384D,   /* U+A5F3 */
  0x384E,   /* U+A5F4 */
  0x384F,   /* U+A5F5 */
  0x3850,   /* U+A5F6 */
  0x3851,   /* U+A5F7 */
  0x3852,   /* U+A5F8 */
  0x3853,   /* U+A5F9 */
  0x3854,   /* U+A5FA */
  0x3855,   /* U+A5FB */
  0x3856,   /* U+A5FC */
  0x3857,   /* U+A5FD */
  0x3858,   /* U+A5FE */
  0x3859,   /* U+A5FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A500 */
  0x0020,   /* U+A501 */
  0x0020,   /* U+A502 */
  0x0020,   /* U+A503 */
  0x0020,   /* U+A504 */
  0x0020,   /* U+A505 */
  0x0020,   /* U+A506 */
  0x0020,   /* U+A507 */
  0x0020,   /* U+A508 */
  0x0020,   /* U+A509 */
  0x0020,   /* U+A50A */
  0x0020,   /* U+A50B */
  0x0020,   /* U+A50C */
  0x0020,   /* U+A50D */
  0x0020,   /* U+A50E */
  0x0020,   /* U+A50F */
  0x0020,   /* U+A510 */
  0x0020,   /* U+A511 */
  0x0020,   /* U+A512 */
  0x0020,   /* U+A513 */
  0x0020,   /* U+A514 */
  0x0020,   /* U+A515 */
  0x0020,   /* U+A516 */
  0x0020,   /* U+A517 */
  0x0020,   /* U+A518 */
  0x0020,   /* U+A519 */
  0x0020,   /* U+A51A */
  0x0020,   /* U+A51B */
  0x0020,   /* U+A51C */
  0x0020,   /* U+A51D */
  0x0020,   /* U+A51E */
  0x0020,   /* U+A51F */
  0x0020,   /* U+A520 */
  0x0020,   /* U+A521 */
  0x0020,   /* U+A522 */
  0x0020,   /* U+A523 */
  0x0020,   /* U+A524 */
  0x0020,   /* U+A525 */
  0x0020,   /* U+A526 */
  0x0020,   /* U+A527 */
  0x0020,   /* U+A528 */
  0x0020,   /* U+A529 */
  0x0020,   /* U+A52A */
  0x0020,   /* U+A52B */
  0x0020,   /* U+A52C */
  0x0020,   /* U+A52D */
  0x0020,   /* U+A52E */
  0x0020,   /* U+A52F */
  0x0020,   /* U+A530 */
  0x0020,   /* U+A531 */
  0x0020,   /* U+A532 */
  0x0020,   /* U+A533 */
  0x0020,   /* U+A534 */
  0x0020,   /* U+A535 */
  0x0020,   /* U+A536 */
  0x0020,   /* U+A537 */
  0x0020,   /* U+A538 */
  0x0020,   /* U+A539 */
  0x0020,   /* U+A53A */
  0x0020,   /* U+A53B */
  0x0020,   /* U+A53C */
  0x0020,   /* U+A53D */
  0x0020,   /* U+A53E */
  0x0020,   /* U+A53F */
  0x0020,   /* U+A540 */
  0x0020,   /* U+A541 */
  0x0020,   /* U+A542 */
  0x0020,   /* U+A543 */
  0x0020,   /* U+A544 */
  0x0020,   /* U+A545 */
  0x0020,   /* U+A546 */
  0x0020,   /* U+A547 */
  0x0020,   /* U+A548 */
  0x0020,   /* U+A549 */
  0x0020,   /* U+A54A */
  0x0020,   /* U+A54B */
  0x0020,   /* U+A54C */
  0x0020,   /* U+A54D */
  0x0020,   /* U+A54E */
  0x0020,   /* U+A54F */
  0x0020,   /* U+A550 */
  0x0020,   /* U+A551 */
  0x0020,   /* U+A552 */
  0x0020,   /* U+A553 */
  0x0020,   /* U+A554 */
  0x0020,   /* U+A555 */
  0x0020,   /* U+A556 */
  0x0020,   /* U+A557 */
  0x0020,   /* U+A558 */
  0x0020,   /* U+A559 */
  0x0020,   /* U+A55A */
  0x0020,   /* U+A55B */
  0x0020,   /* U+A55C */
  0x0020,   /* U+A55D */
  0x0020,   /* U+A55E */
  0x0020,   /* U+A55F */
  0x0020,   /* U+A560 */
  0x0020,   /* U+A561 */
  0x0020,   /* U+A562 */
  0x0020,   /* U+A563 */
  0x0020,   /* U+A564 */
  0x0020,   /* U+A565 */
  0x0020,   /* U+A566 */
  0x0020,   /* U+A567 */
  0x0020,   /* U+A568 */
  0x0020,   /* U+A569 */
  0x0020,   /* U+A56A */
  0x0020,   /* U+A56B */
  0x0020,   /* U+A56C */
  0x0020,   /* U+A56D */
  0x0020,   /* U+A56E */
  0x0020,   /* U+A56F */
  0x0020,   /* U+A570 */
  0x0020,   /* U+A571 */
  0x0020,   /* U+A572 */
  0x0020,   /* U+A573 */
  0x0020,   /* U+A574 */
  0x0020,   /* U+A575 */
  0x0020,   /* U+A576 */
  0x0020,   /* U+A577 */
  0x0020,   /* U+A578 */
  0x0020,   /* U+A579 */
  0x0020,   /* U+A57A */
  0x0020,   /* U+A57B */
  0x0020,   /* U+A57C */
  0x0020,   /* U+A57D */
  0x0020,   /* U+A57E */
  0x0020,   /* U+A57F */
  0x0020,   /* U+A580 */
  0x0020,   /* U+A581 */
  0x0020,   /* U+A582 */
  0x0020,   /* U+A583 */
  0x0020,   /* U+A584 */
  0x0020,   /* U+A585 */
  0x0020,   /* U+A586 */
  0x0020,   /* U+A587 */
  0x0020,   /* U+A588 */
  0x0020,   /* U+A589 */
  0x0020,   /* U+A58A */
  0x0020,   /* U+A58B */
  0x0020,   /* U+A58C */
  0x0020,   /* U+A58D */
  0x0020,   /* U+A58E */
  0x0020,   /* U+A58F */
  0x0020,   /* U+A590 */
  0x0020,   /* U+A591 */
  0x0020,   /* U+A592 */
  0x0020,   /* U+A593 */
  0x0020,   /* U+A594 */
  0x0020,   /* U+A595 */
  0x0020,   /* U+A596 */
  0x0020,   /* U+A597 */
  0x0020,   /* U+A598 */
  0x0020,   /* U+A599 */
  0x0020,   /* U+A59A */
  0x0020,   /* U+A59B */
  0x0020,   /* U+A59C */
  0x0020,   /* U+A59D */
  0x0020,   /* U+A59E */
  0x0020,   /* U+A59F */
  0x0020,   /* U+A5A0 */
  0x0020,   /* U+A5A1 */
  0x0020,   /* U+A5A2 */
  0x0020,   /* U+A5A3 */
  0x0020,   /* U+A5A4 */
  0x0020,   /* U+A5A5 */
  0x0020,   /* U+A5A6 */
  0x0020,   /* U+A5A7 */
  0x0020,   /* U+A5A8 */
  0x0020,   /* U+A5A9 */
  0x0020,   /* U+A5AA */
  0x0020,   /* U+A5AB */
  0x0020,   /* U+A5AC */
  0x0020,   /* U+A5AD */
  0x0020,   /* U+A5AE */
  0x0020,   /* U+A5AF */
  0x0020,   /* U+A5B0 */
  0x0020,   /* U+A5B1 */
  0x0020,   /* U+A5B2 */
  0x0020,   /* U+A5B3 */
  0x0020,   /* U+A5B4 */
  0x0020,   /* U+A5B5 */
  0x0020,   /* U+A5B6 */
  0x0020,   /* U+A5B7 */
  0x0020,   /* U+A5B8 */
  0x0020,   /* U+A5B9 */
  0x0020,   /* U+A5BA */
  0x0020,   /* U+A5BB */
  0x0020,   /* U+A5BC */
  0x0020,   /* U+A5BD */
  0x0020,   /* U+A5BE */
  0x0020,   /* U+A5BF */
  0x0020,   /* U+A5C0 */
  0x0020,   /* U+A5C1 */
  0x0020,   /* U+A5C2 */
  0x0020,   /* U+A5C3 */
  0x0020,   /* U+A5C4 */
  0x0020,   /* U+A5C5 */
  0x0020,   /* U+A5C6 */
  0x0020,   /* U+A5C7 */
  0x0020,   /* U+A5C8 */
  0x0020,   /* U+A5C9 */
  0x0020,   /* U+A5CA */
  0x0020,   /* U+A5CB */
  0x0020,   /* U+A5CC */
  0x0020,   /* U+A5CD */
  0x0020,   /* U+A5CE */
  0x0020,   /* U+A5CF */
  0x0020,   /* U+A5D0 */
  0x0020,   /* U+A5D1 */
  0x0020,   /* U+A5D2 */
  0x0020,   /* U+A5D3 */
  0x0020,   /* U+A5D4 */
  0x0020,   /* U+A5D5 */
  0x0020,   /* U+A5D6 */
  0x0020,   /* U+A5D7 */
  0x0020,   /* U+A5D8 */
  0x0020,   /* U+A5D9 */
  0x0020,   /* U+A5DA */
  0x0020,   /* U+A5DB */
  0x0020,   /* U+A5DC */
  0x0020,   /* U+A5DD */
  0x0020,   /* U+A5DE */
  0x0020,   /* U+A5DF */
  0x0020,   /* U+A5E0 */
  0x0020,   /* U+A5E1 */
  0x0020,   /* U+A5E2 */
  0x0020,   /* U+A5E3 */
  0x0020,   /* U+A5E4 */
  0x0020,   /* U+A5E5 */
  0x0020,   /* U+A5E6 */
  0x0020,   /* U+A5E7 */
  0x0020,   /* U+A5E8 */
  0x0020,   /* U+A5E9 */
  0x0020,   /* U+A5EA */
  0x0020,   /* U+A5EB */
  0x0020,   /* U+A5EC */
  0x0020,   /* U+A5ED */
  0x0020,   /* U+A5EE */
  0x0020,   /* U+A5EF */
  0x0020,   /* U+A5F0 */
  0x0020,   /* U+A5F1 */
  0x0020,   /* U+A5F2 */
  0x0020,   /* U+A5F3 */
  0x0020,   /* U+A5F4 */
  0x0020,   /* U+A5F5 */
  0x0020,   /* U+A5F6 */
  0x0020,   /* U+A5F7 */
  0x0020,   /* U+A5F8 */
  0x0020,   /* U+A5F9 */
  0x0020,   /* U+A5FA */
  0x0020,   /* U+A5FB */
  0x0020,   /* U+A5FC */
  0x0020,   /* U+A5FD */
  0x0020,   /* U+A5FE */
  0x0020,   /* U+A5FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A500 */
  0x0002,   /* U+A501 */
  0x0002,   /* U+A502 */
  0x0002,   /* U+A503 */
  0x0002,   /* U+A504 */
  0x0002,   /* U+A505 */
  0x0002,   /* U+A506 */
  0x0002,   /* U+A507 */
  0x0002,   /* U+A508 */
  0x0002,   /* U+A509 */
  0x0002,   /* U+A50A */
  0x0002,   /* U+A50B */
  0x0002,   /* U+A50C */
  0x0002,   /* U+A50D */
  0x0002,   /* U+A50E */
  0x0002,   /* U+A50F */
  0x0002,   /* U+A510 */
  0x0002,   /* U+A511 */
  0x0002,   /* U+A512 */
  0x0002,   /* U+A513 */
  0x0002,   /* U+A514 */
  0x0002,   /* U+A515 */
  0x0002,   /* U+A516 */
  0x0002,   /* U+A517 */
  0x0002,   /* U+A518 */
  0x0002,   /* U+A519 */
  0x0002,   /* U+A51A */
  0x0002,   /* U+A51B */
  0x0002,   /* U+A51C */
  0x0002,   /* U+A51D */
  0x0002,   /* U+A51E */
  0x0002,   /* U+A51F */
  0x0002,   /* U+A520 */
  0x0002,   /* U+A521 */
  0x0002,   /* U+A522 */
  0x0002,   /* U+A523 */
  0x0002,   /* U+A524 */
  0x0002,   /* U+A525 */
  0x0002,   /* U+A526 */
  0x0002,   /* U+A527 */
  0x0002,   /* U+A528 */
  0x0002,   /* U+A529 */
  0x0002,   /* U+A52A */
  0x0002,   /* U+A52B */
  0x0002,   /* U+A52C */
  0x0002,   /* U+A52D */
  0x0002,   /* U+A52E */
  0x0002,   /* U+A52F */
  0x0002,   /* U+A530 */
  0x0002,   /* U+A531 */
  0x0002,   /* U+A532 */
  0x0002,   /* U+A533 */
  0x0002,   /* U+A534 */
  0x0002,   /* U+A535 */
  0x0002,   /* U+A536 */
  0x0002,   /* U+A537 */
  0x0002,   /* U+A538 */
  0x0002,   /* U+A539 */
  0x0002,   /* U+A53A */
  0x0002,   /* U+A53B */
  0x0002,   /* U+A53C */
  0x0002,   /* U+A53D */
  0x0002,   /* U+A53E */
  0x0002,   /* U+A53F */
  0x0002,   /* U+A540 */
  0x0002,   /* U+A541 */
  0x0002,   /* U+A542 */
  0x0002,   /* U+A543 */
  0x0002,   /* U+A544 */
  0x0002,   /* U+A545 */
  0x0002,   /* U+A546 */
  0x0002,   /* U+A547 */
  0x0002,   /* U+A548 */
  0x0002,   /* U+A549 */
  0x0002,   /* U+A54A */
  0x0002,   /* U+A54B */
  0x0002,   /* U+A54C */
  0x0002,   /* U+A54D */
  0x0002,   /* U+A54E */
  0x0002,   /* U+A54F */
  0x0002,   /* U+A550 */
  0x0002,   /* U+A551 */
  0x0002,   /* U+A552 */
  0x0002,   /* U+A553 */
  0x0002,   /* U+A554 */
  0x0002,   /* U+A555 */
  0x0002,   /* U+A556 */
  0x0002,   /* U+A557 */
  0x0002,   /* U+A558 */
  0x0002,   /* U+A559 */
  0x0002,   /* U+A55A */
  0x0002,   /* U+A55B */
  0x0002,   /* U+A55C */
  0x0002,   /* U+A55D */
  0x0002,   /* U+A55E */
  0x0002,   /* U+A55F */
  0x0002,   /* U+A560 */
  0x0002,   /* U+A561 */
  0x0002,   /* U+A562 */
  0x0002,   /* U+A563 */
  0x0002,   /* U+A564 */
  0x0002,   /* U+A565 */
  0x0002,   /* U+A566 */
  0x0002,   /* U+A567 */
  0x0002,   /* U+A568 */
  0x0002,   /* U+A569 */
  0x0002,   /* U+A56A */
  0x0002,   /* U+A56B */
  0x0002,   /* U+A56C */
  0x0002,   /* U+A56D */
  0x0002,   /* U+A56E */
  0x0002,   /* U+A56F */
  0x0002,   /* U+A570 */
  0x0002,   /* U+A571 */
  0x0002,   /* U+A572 */
  0x0002,   /* U+A573 */
  0x0002,   /* U+A574 */
  0x0002,   /* U+A575 */
  0x0002,   /* U+A576 */
  0x0002,   /* U+A577 */
  0x0002,   /* U+A578 */
  0x0002,   /* U+A579 */
  0x0002,   /* U+A57A */
  0x0002,   /* U+A57B */
  0x0002,   /* U+A57C */
  0x0002,   /* U+A57D */
  0x0002,   /* U+A57E */
  0x0002,   /* U+A57F */
  0x0002,   /* U+A580 */
  0x0002,   /* U+A581 */
  0x0002,   /* U+A582 */
  0x0002,   /* U+A583 */
  0x0002,   /* U+A584 */
  0x0002,   /* U+A585 */
  0x0002,   /* U+A586 */
  0x0002,   /* U+A587 */
  0x0002,   /* U+A588 */
  0x0002,   /* U+A589 */
  0x0002,   /* U+A58A */
  0x0002,   /* U+A58B */
  0x0002,   /* U+A58C */
  0x0002,   /* U+A58D */
  0x0002,   /* U+A58E */
  0x0002,   /* U+A58F */
  0x0002,   /* U+A590 */
  0x0002,   /* U+A591 */
  0x0002,   /* U+A592 */
  0x0002,   /* U+A593 */
  0x0002,   /* U+A594 */
  0x0002,   /* U+A595 */
  0x0002,   /* U+A596 */
  0x0002,   /* U+A597 */
  0x0002,   /* U+A598 */
  0x0002,   /* U+A599 */
  0x0002,   /* U+A59A */
  0x0002,   /* U+A59B */
  0x0002,   /* U+A59C */
  0x0002,   /* U+A59D */
  0x0002,   /* U+A59E */
  0x0002,   /* U+A59F */
  0x0002,   /* U+A5A0 */
  0x0002,   /* U+A5A1 */
  0x0002,   /* U+A5A2 */
  0x0002,   /* U+A5A3 */
  0x0002,   /* U+A5A4 */
  0x0002,   /* U+A5A5 */
  0x0002,   /* U+A5A6 */
  0x0002,   /* U+A5A7 */
  0x0002,   /* U+A5A8 */
  0x0002,   /* U+A5A9 */
  0x0002,   /* U+A5AA */
  0x0002,   /* U+A5AB */
  0x0002,   /* U+A5AC */
  0x0002,   /* U+A5AD */
  0x0002,   /* U+A5AE */
  0x0002,   /* U+A5AF */
  0x0002,   /* U+A5B0 */
  0x0002,   /* U+A5B1 */
  0x0002,   /* U+A5B2 */
  0x0002,   /* U+A5B3 */
  0x0002,   /* U+A5B4 */
  0x0002,   /* U+A5B5 */
  0x0002,   /* U+A5B6 */
  0x0002,   /* U+A5B7 */
  0x0002,   /* U+A5B8 */
  0x0002,   /* U+A5B9 */
  0x0002,   /* U+A5BA */
  0x0002,   /* U+A5BB */
  0x0002,   /* U+A5BC */
  0x0002,   /* U+A5BD */
  0x0002,   /* U+A5BE */
  0x0002,   /* U+A5BF */
  0x0002,   /* U+A5C0 */
  0x0002,   /* U+A5C1 */
  0x0002,   /* U+A5C2 */
  0x0002,   /* U+A5C3 */
  0x0002,   /* U+A5C4 */
  0x0002,   /* U+A5C5 */
  0x0002,   /* U+A5C6 */
  0x0002,   /* U+A5C7 */
  0x0002,   /* U+A5C8 */
  0x0002,   /* U+A5C9 */
  0x0002,   /* U+A5CA */
  0x0002,   /* U+A5CB */
  0x0002,   /* U+A5CC */
  0x0002,   /* U+A5CD */
  0x0002,   /* U+A5CE */
  0x0002,   /* U+A5CF */
  0x0002,   /* U+A5D0 */
  0x0002,   /* U+A5D1 */
  0x0002,   /* U+A5D2 */
  0x0002,   /* U+A5D3 */
  0x0002,   /* U+A5D4 */
  0x0002,   /* U+A5D5 */
  0x0002,   /* U+A5D6 */
  0x0002,   /* U+A5D7 */
  0x0002,   /* U+A5D8 */
  0x0002,   /* U+A5D9 */
  0x0002,   /* U+A5DA */
  0x0002,   /* U+A5DB */
  0x0002,   /* U+A5DC */
  0x0002,   /* U+A5DD */
  0x0002,   /* U+A5DE */
  0x0002,   /* U+A5DF */
  0x0002,   /* U+A5E0 */
  0x0002,   /* U+A5E1 */
  0x0002,   /* U+A5E2 */
  0x0002,   /* U+A5E3 */
  0x0002,   /* U+A5E4 */
  0x0002,   /* U+A5E5 */
  0x0002,   /* U+A5E6 */
  0x0002,   /* U+A5E7 */
  0x0002,   /* U+A5E8 */
  0x0002,   /* U+A5E9 */
  0x0002,   /* U+A5EA */
  0x0002,   /* U+A5EB */
  0x0002,   /* U+A5EC */
  0x0002,   /* U+A5ED */
  0x0002,   /* U+A5EE */
  0x0002,   /* U+A5EF */
  0x0002,   /* U+A5F0 */
  0x0002,   /* U+A5F1 */
  0x0002,   /* U+A5F2 */
  0x0002,   /* U+A5F3 */
  0x0002,   /* U+A5F4 */
  0x0002,   /* U+A5F5 */
  0x0002,   /* U+A5F6 */
  0x0002,   /* U+A5F7 */
  0x0002,   /* U+A5F8 */
  0x0002,   /* U+A5F9 */
  0x0002,   /* U+A5FA */
  0x0002,   /* U+A5FB */
  0x0002,   /* U+A5FC */
  0x0002,   /* U+A5FD */
  0x0002,   /* U+A5FE */
  0x0002,   /* U+A5FF */
};

uint16 uca900_p0A6[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x385A,   /* U+A600 */
  0x385B,   /* U+A601 */
  0x385C,   /* U+A602 */
  0x385D,   /* U+A603 */
  0x385E,   /* U+A604 */
  0x385F,   /* U+A605 */
  0x3860,   /* U+A606 */
  0x3861,   /* U+A607 */
  0x3862,   /* U+A608 */
  0x3863,   /* U+A609 */
  0x3864,   /* U+A60A */
  0x3865,   /* U+A60B */
  0x3866,   /* U+A60C */
  0x022F,   /* U+A60D */
  0x0286,   /* U+A60E */
  0x0270,   /* U+A60F */
  0x37B2,   /* U+A610 */
  0x37C4,   /* U+A611 */
  0x37E1,   /* U+A612 */
  0x3766,   /* U+A613 */
  0x3778,   /* U+A614 */
  0x378D,   /* U+A615 */
  0x37A1,   /* U+A616 */
  0x37AC,   /* U+A617 */
  0x37B2,   /* U+A618 */
  0x37B4,   /* U+A619 */
  0x37BA,   /* U+A61A */
  0x37DF,   /* U+A61B */
  0x380E,   /* U+A61C */
  0x3825,   /* U+A61D */
  0x382B,   /* U+A61E */
  0x3832,   /* U+A61F */
  0x1C3D,   /* U+A620 */
  0x1C3E,   /* U+A621 */
  0x1C3F,   /* U+A622 */
  0x1C40,   /* U+A623 */
  0x1C41,   /* U+A624 */
  0x1C42,   /* U+A625 */
  0x1C43,   /* U+A626 */
  0x1C44,   /* U+A627 */
  0x1C45,   /* U+A628 */
  0x1C46,   /* U+A629 */
  0x37C8,   /* U+A62A */
  0x382B,   /* U+A62B */
  0xFBC1,   /* U+A62C */
  0xFBC1,   /* U+A62D */
  0xFBC1,   /* U+A62E */
  0xFBC1,   /* U+A62F */
  0xFBC1,   /* U+A630 */
  0xFBC1,   /* U+A631 */
  0xFBC1,   /* U+A632 */
  0xFBC1,   /* U+A633 */
  0xFBC1,   /* U+A634 */
  0xFBC1,   /* U+A635 */
  0xFBC1,   /* U+A636 */
  0xFBC1,   /* U+A637 */
  0xFBC1,   /* U+A638 */
  0xFBC1,   /* U+A639 */
  0xFBC1,   /* U+A63A */
  0xFBC1,   /* U+A63B */
  0xFBC1,   /* U+A63C */
  0xFBC1,   /* U+A63D */
  0xFBC1,   /* U+A63E */
  0xFBC1,   /* U+A63F */
  0x2070,   /* U+A640 */
  0x2070,   /* U+A641 */
  0x2073,   /* U+A642 */
  0x2073,   /* U+A643 */
  0x2078,   /* U+A644 */
  0x2078,   /* U+A645 */
  0x208C,   /* U+A646 */
  0x208C,   /* U+A647 */
  0x2095,   /* U+A648 */
  0x2095,   /* U+A649 */
  0x2129,   /* U+A64A */
  0x2129,   /* U+A64B */
  0x2150,   /* U+A64C */
  0x2150,   /* U+A64D */
  0x218C,   /* U+A64E */
  0x218C,   /* U+A64F */
  0x2193,   /* U+A650 */
  0x2193,   /* U+A651 */
  0x21A4,   /* U+A652 */
  0x21A4,   /* U+A653 */
  0x21AD,   /* U+A654 */
  0x21AD,   /* U+A655 */
  0x21AE,   /* U+A656 */
  0x21AE,   /* U+A657 */
  0x21BC,   /* U+A658 */
  0x21BC,   /* U+A659 */
  0x21C1,   /* U+A65A */
  0x21C1,   /* U+A65B */
  0x21C6,   /* U+A65C */
  0x21C6,   /* U+A65D */
  0x21DB,   /* U+A65E */
  0x21DB,   /* U+A65F */
  0x215D,   /* U+A660 */
  0x215D,   /* U+A661 */
  0x2054,   /* U+A662 */
  0x2054,   /* U+A663 */
  0x20C0,   /* U+A664 */
  0x20C0,   /* U+A665 */
  0x20CB,   /* U+A666 */
  0x20CB,   /* U+A667 */
  0x20E7,   /* U+A668 */
  0x20E7,   /* U+A669 */
  0x20E7,   /* U+A66A */
  0x20E7,   /* U+A66B */
  0x20E7,   /* U+A66C */
  0x20E7,   /* U+A66D */
  0x20E7,   /* U+A66E */
  0x0000,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0393,   /* U+A673 */
  0x205E,   /* U+A674 */
  0x2080,   /* U+A675 */
  0x2088,   /* U+A676 */
  0x211D,   /* U+A677 */
  0x218F,   /* U+A678 */
  0x2194,   /* U+A679 */
  0x2198,   /* U+A67A */
  0x2148,   /* U+A67B */
  0x0000,   /* U+A67C */
  0x0000,   /* U+A67D */
  0x03CA,   /* U+A67E */
  0x218E,   /* U+A67F */
  0x204F,   /* U+A680 */
  0x204F,   /* U+A681 */
  0x207F,   /* U+A682 */
  0x207F,   /* U+A683 */
  0x2067,   /* U+A684 */
  0x2067,   /* U+A685 */
  0x2176,   /* U+A686 */
  0x2176,   /* U+A687 */
  0x207D,   /* U+A688 */
  0x207D,   /* U+A689 */
  0x2118,   /* U+A68A */
  0x2118,   /* U+A68B */
  0x2112,   /* U+A68C */
  0x2112,   /* U+A68D */
  0x215E,   /* U+A68E */
  0x215E,   /* U+A68F */
  0x2163,   /* U+A690 */
  0x2163,   /* U+A691 */
  0x2169,   /* U+A692 */
  0x2169,   /* U+A693 */
  0x2147,   /* U+A694 */
  0x2147,   /* U+A695 */
  0x2187,   /* U+A696 */
  0x2187,   /* U+A697 */
  0x20E7,   /* U+A698 */
  0x20E7,   /* U+A699 */
  0x20E7,   /* U+A69A */
  0x20E7,   /* U+A69B */
  0x218F,   /* U+A69C */
  0x2198,   /* U+A69D */
  0x212E,   /* U+A69E */
  0x21B4,   /* U+A69F */
  0x3867,   /* U+A6A0 */
  0x3868,   /* U+A6A1 */
  0x3869,   /* U+A6A2 */
  0x386A,   /* U+A6A3 */
  0x386B,   /* U+A6A4 */
  0x386C,   /* U+A6A5 */
  0x386D,   /* U+A6A6 */
  0x386E,   /* U+A6A7 */
  0x386F,   /* U+A6A8 */
  0x3870,   /* U+A6A9 */
  0x3871,   /* U+A6AA */
  0x3872,   /* U+A6AB */
  0x3873,   /* U+A6AC */
  0x3874,   /* U+A6AD */
  0x3875,   /* U+A6AE */
  0x3876,   /* U+A6AF */
  0x3877,   /* U+A6B0 */
  0x3878,   /* U+A6B1 */
  0x3879,   /* U+A6B2 */
  0x387A,   /* U+A6B3 */
  0x387B,   /* U+A6B4 */
  0x387C,   /* U+A6B5 */
  0x387D,   /* U+A6B6 */
  0x387E,   /* U+A6B7 */
  0x387F,   /* U+A6B8 */
  0x3880,   /* U+A6B9 */
  0x3881,   /* U+A6BA */
  0x3882,   /* U+A6BB */
  0x3883,   /* U+A6BC */
  0x3884,   /* U+A6BD */
  0x3885,   /* U+A6BE */
  0x3886,   /* U+A6BF */
  0x3887,   /* U+A6C0 */
  0x3888,   /* U+A6C1 */
  0x3889,   /* U+A6C2 */
  0x388A,   /* U+A6C3 */
  0x388B,   /* U+A6C4 */
  0x388C,   /* U+A6C5 */
  0x388D,   /* U+A6C6 */
  0x388E,   /* U+A6C7 */
  0x388F,   /* U+A6C8 */
  0x3890,   /* U+A6C9 */
  0x3891,   /* U+A6CA */
  0x3892,   /* U+A6CB */
  0x3893,   /* U+A6CC */
  0x3894,   /* U+A6CD */
  0x3895,   /* U+A6CE */
  0x3896,   /* U+A6CF */
  0x3897,   /* U+A6D0 */
  0x3898,   /* U+A6D1 */
  0x3899,   /* U+A6D2 */
  0x389A,   /* U+A6D3 */
  0x389B,   /* U+A6D4 */
  0x389C,   /* U+A6D5 */
  0x389D,   /* U+A6D6 */
  0x389E,   /* U+A6D7 */
  0x389F,   /* U+A6D8 */
  0x38A0,   /* U+A6D9 */
  0x38A1,   /* U+A6DA */
  0x38A2,   /* U+A6DB */
  0x38A3,   /* U+A6DC */
  0x38A4,   /* U+A6DD */
  0x38A5,   /* U+A6DE */
  0x38A6,   /* U+A6DF */
  0x38A7,   /* U+A6E0 */
  0x38A8,   /* U+A6E1 */
  0x38A9,   /* U+A6E2 */
  0x38AA,   /* U+A6E3 */
  0x38AB,   /* U+A6E4 */
  0x38AC,   /* U+A6E5 */
  0x38AD,   /* U+A6E6 */
  0x38AE,   /* U+A6E7 */
  0x38AF,   /* U+A6E8 */
  0x38B0,   /* U+A6E9 */
  0x38B1,   /* U+A6EA */
  0x38B2,   /* U+A6EB */
  0x38B3,   /* U+A6EC */
  0x38B4,   /* U+A6ED */
  0x38B5,   /* U+A6EE */
  0x38B6,   /* U+A6EF */
  0x0000,   /* U+A6F0 */
  0x0000,   /* U+A6F1 */
  0x02D7,   /* U+A6F2 */
  0x0287,   /* U+A6F3 */
  0x025F,   /* U+A6F4 */
  0x0230,   /* U+A6F5 */
  0x0238,   /* U+A6F6 */
  0x0271,   /* U+A6F7 */
  0xFBC1,   /* U+A6F8 */
  0xFBC1,   /* U+A6F9 */
  0xFBC1,   /* U+A6FA */
  0xFBC1,   /* U+A6FB */
  0xFBC1,   /* U+A6FC */
  0xFBC1,   /* U+A6FD */
  0xFBC1,   /* U+A6FE */
  0xFBC1,   /* U+A6FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A600 */
  0x0020,   /* U+A601 */
  0x0020,   /* U+A602 */
  0x0020,   /* U+A603 */
  0x0020,   /* U+A604 */
  0x0020,   /* U+A605 */
  0x0020,   /* U+A606 */
  0x0020,   /* U+A607 */
  0x0020,   /* U+A608 */
  0x0020,   /* U+A609 */
  0x0020,   /* U+A60A */
  0x0020,   /* U+A60B */
  0x0020,   /* U+A60C */
  0x0020,   /* U+A60D */
  0x0020,   /* U+A60E */
  0x0020,   /* U+A60F */
  0x0020,   /* U+A610 */
  0x0020,   /* U+A611 */
  0x0020,   /* U+A612 */
  0x0020,   /* U+A613 */
  0x0020,   /* U+A614 */
  0x0020,   /* U+A615 */
  0x0020,   /* U+A616 */
  0x0020,   /* U+A617 */
  0x0020,   /* U+A618 */
  0x0020,   /* U+A619 */
  0x0020,   /* U+A61A */
  0x0020,   /* U+A61B */
  0x0020,   /* U+A61C */
  0x0020,   /* U+A61D */
  0x0020,   /* U+A61E */
  0x0020,   /* U+A61F */
  0x0020,   /* U+A620 */
  0x0020,   /* U+A621 */
  0x0020,   /* U+A622 */
  0x0020,   /* U+A623 */
  0x0020,   /* U+A624 */
  0x0020,   /* U+A625 */
  0x0020,   /* U+A626 */
  0x0020,   /* U+A627 */
  0x0020,   /* U+A628 */
  0x0020,   /* U+A629 */
  0x0020,   /* U+A62A */
  0x0020,   /* U+A62B */
  0x0020,   /* U+A62C */
  0x0020,   /* U+A62D */
  0x0020,   /* U+A62E */
  0x0020,   /* U+A62F */
  0x0020,   /* U+A630 */
  0x0020,   /* U+A631 */
  0x0020,   /* U+A632 */
  0x0020,   /* U+A633 */
  0x0020,   /* U+A634 */
  0x0020,   /* U+A635 */
  0x0020,   /* U+A636 */
  0x0020,   /* U+A637 */
  0x0020,   /* U+A638 */
  0x0020,   /* U+A639 */
  0x0020,   /* U+A63A */
  0x0020,   /* U+A63B */
  0x0020,   /* U+A63C */
  0x0020,   /* U+A63D */
  0x0020,   /* U+A63E */
  0x0020,   /* U+A63F */
  0x0020,   /* U+A640 */
  0x0020,   /* U+A641 */
  0x0020,   /* U+A642 */
  0x0020,   /* U+A643 */
  0x0020,   /* U+A644 */
  0x0020,   /* U+A645 */
  0x0020,   /* U+A646 */
  0x0020,   /* U+A647 */
  0x0020,   /* U+A648 */
  0x0020,   /* U+A649 */
  0x0020,   /* U+A64A */
  0x0020,   /* U+A64B */
  0x0020,   /* U+A64C */
  0x0020,   /* U+A64D */
  0x0020,   /* U+A64E */
  0x0020,   /* U+A64F */
  0x0020,   /* U+A650 */
  0x0020,   /* U+A651 */
  0x0020,   /* U+A652 */
  0x0020,   /* U+A653 */
  0x0020,   /* U+A654 */
  0x0020,   /* U+A655 */
  0x0020,   /* U+A656 */
  0x0020,   /* U+A657 */
  0x0020,   /* U+A658 */
  0x0020,   /* U+A659 */
  0x0020,   /* U+A65A */
  0x0020,   /* U+A65B */
  0x0020,   /* U+A65C */
  0x0020,   /* U+A65D */
  0x0020,   /* U+A65E */
  0x0020,   /* U+A65F */
  0x0020,   /* U+A660 */
  0x0020,   /* U+A661 */
  0x0020,   /* U+A662 */
  0x0020,   /* U+A663 */
  0x0020,   /* U+A664 */
  0x0020,   /* U+A665 */
  0x0020,   /* U+A666 */
  0x0020,   /* U+A667 */
  0x0020,   /* U+A668 */
  0x0020,   /* U+A669 */
  0x0020,   /* U+A66A */
  0x0020,   /* U+A66B */
  0x0020,   /* U+A66C */
  0x0020,   /* U+A66D */
  0x0020,   /* U+A66E */
  0x0051,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0020,   /* U+A673 */
  0x0020,   /* U+A674 */
  0x0020,   /* U+A675 */
  0x0020,   /* U+A676 */
  0x0020,   /* U+A677 */
  0x0020,   /* U+A678 */
  0x0020,   /* U+A679 */
  0x0020,   /* U+A67A */
  0x0020,   /* U+A67B */
  0x0033,   /* U+A67C */
  0x0033,   /* U+A67D */
  0x0020,   /* U+A67E */
  0x0020,   /* U+A67F */
  0x0020,   /* U+A680 */
  0x0020,   /* U+A681 */
  0x0020,   /* U+A682 */
  0x0020,   /* U+A683 */
  0x0020,   /* U+A684 */
  0x0020,   /* U+A685 */
  0x0020,   /* U+A686 */
  0x0020,   /* U+A687 */
  0x0020,   /* U+A688 */
  0x0020,   /* U+A689 */
  0x0020,   /* U+A68A */
  0x0020,   /* U+A68B */
  0x0020,   /* U+A68C */
  0x0020,   /* U+A68D */
  0x0020,   /* U+A68E */
  0x0020,   /* U+A68F */
  0x0020,   /* U+A690 */
  0x0020,   /* U+A691 */
  0x0020,   /* U+A692 */
  0x0020,   /* U+A693 */
  0x0020,   /* U+A694 */
  0x0020,   /* U+A695 */
  0x0020,   /* U+A696 */
  0x0020,   /* U+A697 */
  0x0020,   /* U+A698 */
  0x0020,   /* U+A699 */
  0x0020,   /* U+A69A */
  0x0020,   /* U+A69B */
  0x0020,   /* U+A69C */
  0x0020,   /* U+A69D */
  0x0020,   /* U+A69E */
  0x0020,   /* U+A69F */
  0x0020,   /* U+A6A0 */
  0x0020,   /* U+A6A1 */
  0x0020,   /* U+A6A2 */
  0x0020,   /* U+A6A3 */
  0x0020,   /* U+A6A4 */
  0x0020,   /* U+A6A5 */
  0x0020,   /* U+A6A6 */
  0x0020,   /* U+A6A7 */
  0x0020,   /* U+A6A8 */
  0x0020,   /* U+A6A9 */
  0x0020,   /* U+A6AA */
  0x0020,   /* U+A6AB */
  0x0020,   /* U+A6AC */
  0x0020,   /* U+A6AD */
  0x0020,   /* U+A6AE */
  0x0020,   /* U+A6AF */
  0x0020,   /* U+A6B0 */
  0x0020,   /* U+A6B1 */
  0x0020,   /* U+A6B2 */
  0x0020,   /* U+A6B3 */
  0x0020,   /* U+A6B4 */
  0x0020,   /* U+A6B5 */
  0x0020,   /* U+A6B6 */
  0x0020,   /* U+A6B7 */
  0x0020,   /* U+A6B8 */
  0x0020,   /* U+A6B9 */
  0x0020,   /* U+A6BA */
  0x0020,   /* U+A6BB */
  0x0020,   /* U+A6BC */
  0x0020,   /* U+A6BD */
  0x0020,   /* U+A6BE */
  0x0020,   /* U+A6BF */
  0x0020,   /* U+A6C0 */
  0x0020,   /* U+A6C1 */
  0x0020,   /* U+A6C2 */
  0x0020,   /* U+A6C3 */
  0x0020,   /* U+A6C4 */
  0x0020,   /* U+A6C5 */
  0x0020,   /* U+A6C6 */
  0x0020,   /* U+A6C7 */
  0x0020,   /* U+A6C8 */
  0x0020,   /* U+A6C9 */
  0x0020,   /* U+A6CA */
  0x0020,   /* U+A6CB */
  0x0020,   /* U+A6CC */
  0x0020,   /* U+A6CD */
  0x0020,   /* U+A6CE */
  0x0020,   /* U+A6CF */
  0x0020,   /* U+A6D0 */
  0x0020,   /* U+A6D1 */
  0x0020,   /* U+A6D2 */
  0x0020,   /* U+A6D3 */
  0x0020,   /* U+A6D4 */
  0x0020,   /* U+A6D5 */
  0x0020,   /* U+A6D6 */
  0x0020,   /* U+A6D7 */
  0x0020,   /* U+A6D8 */
  0x0020,   /* U+A6D9 */
  0x0020,   /* U+A6DA */
  0x0020,   /* U+A6DB */
  0x0020,   /* U+A6DC */
  0x0020,   /* U+A6DD */
  0x0020,   /* U+A6DE */
  0x0020,   /* U+A6DF */
  0x0020,   /* U+A6E0 */
  0x0020,   /* U+A6E1 */
  0x0020,   /* U+A6E2 */
  0x0020,   /* U+A6E3 */
  0x0020,   /* U+A6E4 */
  0x0020,   /* U+A6E5 */
  0x0020,   /* U+A6E6 */
  0x0020,   /* U+A6E7 */
  0x0020,   /* U+A6E8 */
  0x0020,   /* U+A6E9 */
  0x0020,   /* U+A6EA */
  0x0020,   /* U+A6EB */
  0x0020,   /* U+A6EC */
  0x0020,   /* U+A6ED */
  0x0020,   /* U+A6EE */
  0x0020,   /* U+A6EF */
  0x00B6,   /* U+A6F0 */
  0x00B7,   /* U+A6F1 */
  0x0020,   /* U+A6F2 */
  0x0020,   /* U+A6F3 */
  0x0020,   /* U+A6F4 */
  0x0020,   /* U+A6F5 */
  0x0020,   /* U+A6F6 */
  0x0020,   /* U+A6F7 */
  0x0020,   /* U+A6F8 */
  0x0020,   /* U+A6F9 */
  0x0020,   /* U+A6FA */
  0x0020,   /* U+A6FB */
  0x0020,   /* U+A6FC */
  0x0020,   /* U+A6FD */
  0x0020,   /* U+A6FE */
  0x0020,   /* U+A6FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A600 */
  0x0002,   /* U+A601 */
  0x0002,   /* U+A602 */
  0x0002,   /* U+A603 */
  0x0002,   /* U+A604 */
  0x0002,   /* U+A605 */
  0x0002,   /* U+A606 */
  0x0002,   /* U+A607 */
  0x0002,   /* U+A608 */
  0x0002,   /* U+A609 */
  0x0002,   /* U+A60A */
  0x0002,   /* U+A60B */
  0x0002,   /* U+A60C */
  0x0002,   /* U+A60D */
  0x0002,   /* U+A60E */
  0x0002,   /* U+A60F */
  0x0004,   /* U+A610 */
  0x0004,   /* U+A611 */
  0x0004,   /* U+A612 */
  0x0004,   /* U+A613 */
  0x0004,   /* U+A614 */
  0x0004,   /* U+A615 */
  0x0004,   /* U+A616 */
  0x0004,   /* U+A617 */
  0x0004,   /* U+A618 */
  0x0004,   /* U+A619 */
  0x0004,   /* U+A61A */
  0x0004,   /* U+A61B */
  0x0004,   /* U+A61C */
  0x0004,   /* U+A61D */
  0x0004,   /* U+A61E */
  0x0004,   /* U+A61F */
  0x0002,   /* U+A620 */
  0x0002,   /* U+A621 */
  0x0002,   /* U+A622 */
  0x0002,   /* U+A623 */
  0x0002,   /* U+A624 */
  0x0002,   /* U+A625 */
  0x0002,   /* U+A626 */
  0x0002,   /* U+A627 */
  0x0002,   /* U+A628 */
  0x0002,   /* U+A629 */
  0x0004,   /* U+A62A */
  0x0004,   /* U+A62B */
  0x0002,   /* U+A62C */
  0x0002,   /* U+A62D */
  0x0002,   /* U+A62E */
  0x0002,   /* U+A62F */
  0x0002,   /* U+A630 */
  0x0002,   /* U+A631 */
  0x0002,   /* U+A632 */
  0x0002,   /* U+A633 */
  0x0002,   /* U+A634 */
  0x0002,   /* U+A635 */
  0x0002,   /* U+A636 */
  0x0002,   /* U+A637 */
  0x0002,   /* U+A638 */
  0x0002,   /* U+A639 */
  0x0002,   /* U+A63A */
  0x0002,   /* U+A63B */
  0x0002,   /* U+A63C */
  0x0002,   /* U+A63D */
  0x0002,   /* U+A63E */
  0x0002,   /* U+A63F */
  0x0008,   /* U+A640 */
  0x0002,   /* U+A641 */
  0x0008,   /* U+A642 */
  0x0002,   /* U+A643 */
  0x0008,   /* U+A644 */
  0x0002,   /* U+A645 */
  0x0008,   /* U+A646 */
  0x0002,   /* U+A647 */
  0x0008,   /* U+A648 */
  0x0002,   /* U+A649 */
  0x0008,   /* U+A64A */
  0x0002,   /* U+A64B */
  0x0008,   /* U+A64C */
  0x0002,   /* U+A64D */
  0x0008,   /* U+A64E */
  0x0002,   /* U+A64F */
  0x0008,   /* U+A650 */
  0x0002,   /* U+A651 */
  0x0008,   /* U+A652 */
  0x0002,   /* U+A653 */
  0x0008,   /* U+A654 */
  0x0002,   /* U+A655 */
  0x0008,   /* U+A656 */
  0x0002,   /* U+A657 */
  0x0008,   /* U+A658 */
  0x0002,   /* U+A659 */
  0x0008,   /* U+A65A */
  0x0002,   /* U+A65B */
  0x0008,   /* U+A65C */
  0x0002,   /* U+A65D */
  0x0008,   /* U+A65E */
  0x0002,   /* U+A65F */
  0x0008,   /* U+A660 */
  0x0002,   /* U+A661 */
  0x0008,   /* U+A662 */
  0x0002,   /* U+A663 */
  0x0008,   /* U+A664 */
  0x0002,   /* U+A665 */
  0x0008,   /* U+A666 */
  0x0002,   /* U+A667 */
  0x000A,   /* U+A668 */
  0x0004,   /* U+A669 */
  0x000A,   /* U+A66A */
  0x0004,   /* U+A66B */
  0x000A,   /* U+A66C */
  0x0004,   /* U+A66D */
  0x0004,   /* U+A66E */
  0x0002,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0002,   /* U+A673 */
  0x0004,   /* U+A674 */
  0x0004,   /* U+A675 */
  0x0004,   /* U+A676 */
  0x0004,   /* U+A677 */
  0x0004,   /* U+A678 */
  0x0004,   /* U+A679 */
  0x0004,   /* U+A67A */
  0x0004,   /* U+A67B */
  0x0002,   /* U+A67C */
  0x0002,   /* U+A67D */
  0x0002,   /* U+A67E */
  0x0002,   /* U+A67F */
  0x0008,   /* U+A680 */
  0x0002,   /* U+A681 */
  0x0008,   /* U+A682 */
  0x0002,   /* U+A683 */
  0x0008,   /* U+A684 */
  0x0002,   /* U+A685 */
  0x0008,   /* U+A686 */
  0x0002,   /* U+A687 */
  0x0008,   /* U+A688 */
  0x0002,   /* U+A689 */
  0x0008,   /* U+A68A */
  0x0002,   /* U+A68B */
  0x0008,   /* U+A68C */
  0x0002,   /* U+A68D */
  0x0008,   /* U+A68E */
  0x0002,   /* U+A68F */
  0x0008,   /* U+A690 */
  0x0002,   /* U+A691 */
  0x0008,   /* U+A692 */
  0x0002,   /* U+A693 */
  0x0008,   /* U+A694 */
  0x0002,   /* U+A695 */
  0x0008,   /* U+A696 */
  0x0002,   /* U+A697 */
  0x000A,   /* U+A698 */
  0x0004,   /* U+A699 */
  0x000A,   /* U+A69A */
  0x0004,   /* U+A69B */
  0x0014,   /* U+A69C */
  0x0014,   /* U+A69D */
  0x0004,   /* U+A69E */
  0x0004,   /* U+A69F */
  0x0002,   /* U+A6A0 */
  0x0002,   /* U+A6A1 */
  0x0002,   /* U+A6A2 */
  0x0002,   /* U+A6A3 */
  0x0002,   /* U+A6A4 */
  0x0002,   /* U+A6A5 */
  0x0002,   /* U+A6A6 */
  0x0002,   /* U+A6A7 */
  0x0002,   /* U+A6A8 */
  0x0002,   /* U+A6A9 */
  0x0002,   /* U+A6AA */
  0x0002,   /* U+A6AB */
  0x0002,   /* U+A6AC */
  0x0002,   /* U+A6AD */
  0x0002,   /* U+A6AE */
  0x0002,   /* U+A6AF */
  0x0002,   /* U+A6B0 */
  0x0002,   /* U+A6B1 */
  0x0002,   /* U+A6B2 */
  0x0002,   /* U+A6B3 */
  0x0002,   /* U+A6B4 */
  0x0002,   /* U+A6B5 */
  0x0002,   /* U+A6B6 */
  0x0002,   /* U+A6B7 */
  0x0002,   /* U+A6B8 */
  0x0002,   /* U+A6B9 */
  0x0002,   /* U+A6BA */
  0x0002,   /* U+A6BB */
  0x0002,   /* U+A6BC */
  0x0002,   /* U+A6BD */
  0x0002,   /* U+A6BE */
  0x0002,   /* U+A6BF */
  0x0002,   /* U+A6C0 */
  0x0002,   /* U+A6C1 */
  0x0002,   /* U+A6C2 */
  0x0002,   /* U+A6C3 */
  0x0002,   /* U+A6C4 */
  0x0002,   /* U+A6C5 */
  0x0002,   /* U+A6C6 */
  0x0002,   /* U+A6C7 */
  0x0002,   /* U+A6C8 */
  0x0002,   /* U+A6C9 */
  0x0002,   /* U+A6CA */
  0x0002,   /* U+A6CB */
  0x0002,   /* U+A6CC */
  0x0002,   /* U+A6CD */
  0x0002,   /* U+A6CE */
  0x0002,   /* U+A6CF */
  0x0002,   /* U+A6D0 */
  0x0002,   /* U+A6D1 */
  0x0002,   /* U+A6D2 */
  0x0002,   /* U+A6D3 */
  0x0002,   /* U+A6D4 */
  0x0002,   /* U+A6D5 */
  0x0002,   /* U+A6D6 */
  0x0002,   /* U+A6D7 */
  0x0002,   /* U+A6D8 */
  0x0002,   /* U+A6D9 */
  0x0002,   /* U+A6DA */
  0x0002,   /* U+A6DB */
  0x0002,   /* U+A6DC */
  0x0002,   /* U+A6DD */
  0x0002,   /* U+A6DE */
  0x0002,   /* U+A6DF */
  0x0002,   /* U+A6E0 */
  0x0002,   /* U+A6E1 */
  0x0002,   /* U+A6E2 */
  0x0002,   /* U+A6E3 */
  0x0002,   /* U+A6E4 */
  0x0002,   /* U+A6E5 */
  0x0002,   /* U+A6E6 */
  0x0002,   /* U+A6E7 */
  0x0002,   /* U+A6E8 */
  0x0002,   /* U+A6E9 */
  0x0002,   /* U+A6EA */
  0x0002,   /* U+A6EB */
  0x0002,   /* U+A6EC */
  0x0002,   /* U+A6ED */
  0x0002,   /* U+A6EE */
  0x0002,   /* U+A6EF */
  0x0002,   /* U+A6F0 */
  0x0002,   /* U+A6F1 */
  0x0002,   /* U+A6F2 */
  0x0002,   /* U+A6F3 */
  0x0002,   /* U+A6F4 */
  0x0002,   /* U+A6F5 */
  0x0002,   /* U+A6F6 */
  0x0002,   /* U+A6F7 */
  0x0002,   /* U+A6F8 */
  0x0002,   /* U+A6F9 */
  0x0002,   /* U+A6FA */
  0x0002,   /* U+A6FB */
  0x0002,   /* U+A6FC */
  0x0002,   /* U+A6FD */
  0x0002,   /* U+A6FE */
  0x0002,   /* U+A6FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+A600 */
  0x0000,   /* U+A601 */
  0x0000,   /* U+A602 */
  0x0000,   /* U+A603 */
  0x0000,   /* U+A604 */
  0x0000,   /* U+A605 */
  0x0000,   /* U+A606 */
  0x0000,   /* U+A607 */
  0x0000,   /* U+A608 */
  0x0000,   /* U+A609 */
  0x0000,   /* U+A60A */
  0x0000,   /* U+A60B */
  0x0000,   /* U+A60C */
  0x0000,   /* U+A60D */
  0x0000,   /* U+A60E */
  0x0000,   /* U+A60F */
  0x0000,   /* U+A610 */
  0x0000,   /* U+A611 */
  0x0000,   /* U+A612 */
  0x3865,   /* U+A613 */
  0x3865,   /* U+A614 */
  0x3865,   /* U+A615 */
  0x3866,   /* U+A616 */
  0x3865,   /* U+A617 */
  0x3866,   /* U+A618 */
  0x3866,   /* U+A619 */
  0x3865,   /* U+A61A */
  0x3865,   /* U+A61B */
  0x3865,   /* U+A61C */
  0x3865,   /* U+A61D */
  0x3866,   /* U+A61E */
  0x3865,   /* U+A61F */
  0x0000,   /* U+A620 */
  0x0000,   /* U+A621 */
  0x0000,   /* U+A622 */
  0x0000,   /* U+A623 */
  0x0000,   /* U+A624 */
  0x0000,   /* U+A625 */
  0x0000,   /* U+A626 */
  0x0000,   /* U+A627 */
  0x0000,   /* U+A628 */
  0x0000,   /* U+A629 */
  0x0000,   /* U+A62A */
  0x0000,   /* U+A62B */
  0xA62C,   /* U+A62C */
  0xA62D,   /* U+A62D */
  0xA62E,   /* U+A62E */
  0xA62F,   /* U+A62F */
  0xA630,   /* U+A630 */
  0xA631,   /* U+A631 */
  0xA632,   /* U+A632 */
  0xA633,   /* U+A633 */
  0xA634,   /* U+A634 */
  0xA635,   /* U+A635 */
  0xA636,   /* U+A636 */
  0xA637,   /* U+A637 */
  0xA638,   /* U+A638 */
  0xA639,   /* U+A639 */
  0xA63A,   /* U+A63A */
  0xA63B,   /* U+A63B */
  0xA63C,   /* U+A63C */
  0xA63D,   /* U+A63D */
  0xA63E,   /* U+A63E */
  0xA63F,   /* U+A63F */
  0x0000,   /* U+A640 */
  0x0000,   /* U+A641 */
  0x0000,   /* U+A642 */
  0x0000,   /* U+A643 */
  0x0000,   /* U+A644 */
  0x0000,   /* U+A645 */
  0x0000,   /* U+A646 */
  0x0000,   /* U+A647 */
  0x0000,   /* U+A648 */
  0x0000,   /* U+A649 */
  0x0000,   /* U+A64A */
  0x0000,   /* U+A64B */
  0x0000,   /* U+A64C */
  0x0000,   /* U+A64D */
  0x0000,   /* U+A64E */
  0x0000,   /* U+A64F */
  0x0000,   /* U+A650 */
  0x0000,   /* U+A651 */
  0x0000,   /* U+A652 */
  0x0000,   /* U+A653 */
  0x0000,   /* U+A654 */
  0x0000,   /* U+A655 */
  0x0000,   /* U+A656 */
  0x0000,   /* U+A657 */
  0x0000,   /* U+A658 */
  0x0000,   /* U+A659 */
  0x0000,   /* U+A65A */
  0x0000,   /* U+A65B */
  0x0000,   /* U+A65C */
  0x0000,   /* U+A65D */
  0x0000,   /* U+A65E */
  0x0000,   /* U+A65F */
  0x0000,   /* U+A660 */
  0x0000,   /* U+A661 */
  0x0000,   /* U+A662 */
  0x0000,   /* U+A663 */
  0x0000,   /* U+A664 */
  0x0000,   /* U+A665 */
  0x0000,   /* U+A666 */
  0x0000,   /* U+A667 */
  0x0000,   /* U+A668 */
  0x0000,   /* U+A669 */
  0x0000,   /* U+A66A */
  0x0000,   /* U+A66B */
  0x0000,   /* U+A66C */
  0x0000,   /* U+A66D */
  0x0000,   /* U+A66E */
  0x0000,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0000,   /* U+A673 */
  0x0000,   /* U+A674 */
  0x0000,   /* U+A675 */
  0x0000,   /* U+A676 */
  0x0000,   /* U+A677 */
  0x0000,   /* U+A678 */
  0x0000,   /* U+A679 */
  0x0000,   /* U+A67A */
  0x0000,   /* U+A67B */
  0x0000,   /* U+A67C */
  0x0000,   /* U+A67D */
  0x0000,   /* U+A67E */
  0x0000,   /* U+A67F */
  0x0000,   /* U+A680 */
  0x0000,   /* U+A681 */
  0x0000,   /* U+A682 */
  0x0000,   /* U+A683 */
  0x0000,   /* U+A684 */
  0x0000,   /* U+A685 */
  0x0000,   /* U+A686 */
  0x0000,   /* U+A687 */
  0x0000,   /* U+A688 */
  0x0000,   /* U+A689 */
  0x0000,   /* U+A68A */
  0x0000,   /* U+A68B */
  0x0000,   /* U+A68C */
  0x0000,   /* U+A68D */
  0x0000,   /* U+A68E */
  0x0000,   /* U+A68F */
  0x0000,   /* U+A690 */
  0x0000,   /* U+A691 */
  0x0000,   /* U+A692 */
  0x0000,   /* U+A693 */
  0x0000,   /* U+A694 */
  0x0000,   /* U+A695 */
  0x0000,   /* U+A696 */
  0x0000,   /* U+A697 */
  0x0000,   /* U+A698 */
  0x0000,   /* U+A699 */
  0x0000,   /* U+A69A */
  0x0000,   /* U+A69B */
  0x0000,   /* U+A69C */
  0x0000,   /* U+A69D */
  0x0000,   /* U+A69E */
  0x0000,   /* U+A69F */
  0x0000,   /* U+A6A0 */
  0x0000,   /* U+A6A1 */
  0x0000,   /* U+A6A2 */
  0x0000,   /* U+A6A3 */
  0x0000,   /* U+A6A4 */
  0x0000,   /* U+A6A5 */
  0x0000,   /* U+A6A6 */
  0x0000,   /* U+A6A7 */
  0x0000,   /* U+A6A8 */
  0x0000,   /* U+A6A9 */
  0x0000,   /* U+A6AA */
  0x0000,   /* U+A6AB */
  0x0000,   /* U+A6AC */
  0x0000,   /* U+A6AD */
  0x0000,   /* U+A6AE */
  0x0000,   /* U+A6AF */
  0x0000,   /* U+A6B0 */
  0x0000,   /* U+A6B1 */
  0x0000,   /* U+A6B2 */
  0x0000,   /* U+A6B3 */
  0x0000,   /* U+A6B4 */
  0x0000,   /* U+A6B5 */
  0x0000,   /* U+A6B6 */
  0x0000,   /* U+A6B7 */
  0x0000,   /* U+A6B8 */
  0x0000,   /* U+A6B9 */
  0x0000,   /* U+A6BA */
  0x0000,   /* U+A6BB */
  0x0000,   /* U+A6BC */
  0x0000,   /* U+A6BD */
  0x0000,   /* U+A6BE */
  0x0000,   /* U+A6BF */
  0x0000,   /* U+A6C0 */
  0x0000,   /* U+A6C1 */
  0x0000,   /* U+A6C2 */
  0x0000,   /* U+A6C3 */
  0x0000,   /* U+A6C4 */
  0x0000,   /* U+A6C5 */
  0x0000,   /* U+A6C6 */
  0x0000,   /* U+A6C7 */
  0x0000,   /* U+A6C8 */
  0x0000,   /* U+A6C9 */
  0x0000,   /* U+A6CA */
  0x0000,   /* U+A6CB */
  0x0000,   /* U+A6CC */
  0x0000,   /* U+A6CD */
  0x0000,   /* U+A6CE */
  0x0000,   /* U+A6CF */
  0x0000,   /* U+A6D0 */
  0x0000,   /* U+A6D1 */
  0x0000,   /* U+A6D2 */
  0x0000,   /* U+A6D3 */
  0x0000,   /* U+A6D4 */
  0x0000,   /* U+A6D5 */
  0x0000,   /* U+A6D6 */
  0x0000,   /* U+A6D7 */
  0x0000,   /* U+A6D8 */
  0x0000,   /* U+A6D9 */
  0x0000,   /* U+A6DA */
  0x0000,   /* U+A6DB */
  0x0000,   /* U+A6DC */
  0x0000,   /* U+A6DD */
  0x0000,   /* U+A6DE */
  0x0000,   /* U+A6DF */
  0x0000,   /* U+A6E0 */
  0x0000,   /* U+A6E1 */
  0x0000,   /* U+A6E2 */
  0x0000,   /* U+A6E3 */
  0x0000,   /* U+A6E4 */
  0x0000,   /* U+A6E5 */
  0x0000,   /* U+A6E6 */
  0x0000,   /* U+A6E7 */
  0x0000,   /* U+A6E8 */
  0x0000,   /* U+A6E9 */
  0x0000,   /* U+A6EA */
  0x0000,   /* U+A6EB */
  0x0000,   /* U+A6EC */
  0x0000,   /* U+A6ED */
  0x0000,   /* U+A6EE */
  0x0000,   /* U+A6EF */
  0x0000,   /* U+A6F0 */
  0x0000,   /* U+A6F1 */
  0x0000,   /* U+A6F2 */
  0x0000,   /* U+A6F3 */
  0x0000,   /* U+A6F4 */
  0x0000,   /* U+A6F5 */
  0x0000,   /* U+A6F6 */
  0x0000,   /* U+A6F7 */
  0xA6F8,   /* U+A6F8 */
  0xA6F9,   /* U+A6F9 */
  0xA6FA,   /* U+A6FA */
  0xA6FB,   /* U+A6FB */
  0xA6FC,   /* U+A6FC */
  0xA6FD,   /* U+A6FD */
  0xA6FE,   /* U+A6FE */
  0xA6FF,   /* U+A6FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+A600 */
  0x0000,   /* U+A601 */
  0x0000,   /* U+A602 */
  0x0000,   /* U+A603 */
  0x0000,   /* U+A604 */
  0x0000,   /* U+A605 */
  0x0000,   /* U+A606 */
  0x0000,   /* U+A607 */
  0x0000,   /* U+A608 */
  0x0000,   /* U+A609 */
  0x0000,   /* U+A60A */
  0x0000,   /* U+A60B */
  0x0000,   /* U+A60C */
  0x0000,   /* U+A60D */
  0x0000,   /* U+A60E */
  0x0000,   /* U+A60F */
  0x0000,   /* U+A610 */
  0x0000,   /* U+A611 */
  0x0000,   /* U+A612 */
  0x0020,   /* U+A613 */
  0x0020,   /* U+A614 */
  0x0020,   /* U+A615 */
  0x0020,   /* U+A616 */
  0x0020,   /* U+A617 */
  0x0020,   /* U+A618 */
  0x0020,   /* U+A619 */
  0x0020,   /* U+A61A */
  0x0020,   /* U+A61B */
  0x0020,   /* U+A61C */
  0x0020,   /* U+A61D */
  0x0020,   /* U+A61E */
  0x0020,   /* U+A61F */
  0x0000,   /* U+A620 */
  0x0000,   /* U+A621 */
  0x0000,   /* U+A622 */
  0x0000,   /* U+A623 */
  0x0000,   /* U+A624 */
  0x0000,   /* U+A625 */
  0x0000,   /* U+A626 */
  0x0000,   /* U+A627 */
  0x0000,   /* U+A628 */
  0x0000,   /* U+A629 */
  0x0000,   /* U+A62A */
  0x0000,   /* U+A62B */
  0x0000,   /* U+A62C */
  0x0000,   /* U+A62D */
  0x0000,   /* U+A62E */
  0x0000,   /* U+A62F */
  0x0000,   /* U+A630 */
  0x0000,   /* U+A631 */
  0x0000,   /* U+A632 */
  0x0000,   /* U+A633 */
  0x0000,   /* U+A634 */
  0x0000,   /* U+A635 */
  0x0000,   /* U+A636 */
  0x0000,   /* U+A637 */
  0x0000,   /* U+A638 */
  0x0000,   /* U+A639 */
  0x0000,   /* U+A63A */
  0x0000,   /* U+A63B */
  0x0000,   /* U+A63C */
  0x0000,   /* U+A63D */
  0x0000,   /* U+A63E */
  0x0000,   /* U+A63F */
  0x0000,   /* U+A640 */
  0x0000,   /* U+A641 */
  0x0000,   /* U+A642 */
  0x0000,   /* U+A643 */
  0x0000,   /* U+A644 */
  0x0000,   /* U+A645 */
  0x0000,   /* U+A646 */
  0x0000,   /* U+A647 */
  0x0000,   /* U+A648 */
  0x0000,   /* U+A649 */
  0x0000,   /* U+A64A */
  0x0000,   /* U+A64B */
  0x0000,   /* U+A64C */
  0x0000,   /* U+A64D */
  0x0000,   /* U+A64E */
  0x0000,   /* U+A64F */
  0x0000,   /* U+A650 */
  0x0000,   /* U+A651 */
  0x0000,   /* U+A652 */
  0x0000,   /* U+A653 */
  0x0000,   /* U+A654 */
  0x0000,   /* U+A655 */
  0x0000,   /* U+A656 */
  0x0000,   /* U+A657 */
  0x0000,   /* U+A658 */
  0x0000,   /* U+A659 */
  0x0000,   /* U+A65A */
  0x0000,   /* U+A65B */
  0x0000,   /* U+A65C */
  0x0000,   /* U+A65D */
  0x0000,   /* U+A65E */
  0x0000,   /* U+A65F */
  0x0000,   /* U+A660 */
  0x0000,   /* U+A661 */
  0x0000,   /* U+A662 */
  0x0000,   /* U+A663 */
  0x0000,   /* U+A664 */
  0x0000,   /* U+A665 */
  0x0000,   /* U+A666 */
  0x0000,   /* U+A667 */
  0x0000,   /* U+A668 */
  0x0000,   /* U+A669 */
  0x0000,   /* U+A66A */
  0x0000,   /* U+A66B */
  0x0000,   /* U+A66C */
  0x0000,   /* U+A66D */
  0x0000,   /* U+A66E */
  0x0000,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0000,   /* U+A673 */
  0x0000,   /* U+A674 */
  0x0000,   /* U+A675 */
  0x002B,   /* U+A676 */
  0x0000,   /* U+A677 */
  0x0000,   /* U+A678 */
  0x0000,   /* U+A679 */
  0x0000,   /* U+A67A */
  0x0000,   /* U+A67B */
  0x0000,   /* U+A67C */
  0x0000,   /* U+A67D */
  0x0000,   /* U+A67E */
  0x0000,   /* U+A67F */
  0x0000,   /* U+A680 */
  0x0000,   /* U+A681 */
  0x0000,   /* U+A682 */
  0x0000,   /* U+A683 */
  0x0000,   /* U+A684 */
  0x0000,   /* U+A685 */
  0x0000,   /* U+A686 */
  0x0000,   /* U+A687 */
  0x0000,   /* U+A688 */
  0x0000,   /* U+A689 */
  0x0000,   /* U+A68A */
  0x0000,   /* U+A68B */
  0x0000,   /* U+A68C */
  0x0000,   /* U+A68D */
  0x0000,   /* U+A68E */
  0x0000,   /* U+A68F */
  0x0000,   /* U+A690 */
  0x0000,   /* U+A691 */
  0x0000,   /* U+A692 */
  0x0000,   /* U+A693 */
  0x0000,   /* U+A694 */
  0x0000,   /* U+A695 */
  0x0000,   /* U+A696 */
  0x0000,   /* U+A697 */
  0x0000,   /* U+A698 */
  0x0000,   /* U+A699 */
  0x0000,   /* U+A69A */
  0x0000,   /* U+A69B */
  0x0000,   /* U+A69C */
  0x0000,   /* U+A69D */
  0x0000,   /* U+A69E */
  0x0000,   /* U+A69F */
  0x0000,   /* U+A6A0 */
  0x0000,   /* U+A6A1 */
  0x0000,   /* U+A6A2 */
  0x0000,   /* U+A6A3 */
  0x0000,   /* U+A6A4 */
  0x0000,   /* U+A6A5 */
  0x0000,   /* U+A6A6 */
  0x0000,   /* U+A6A7 */
  0x0000,   /* U+A6A8 */
  0x0000,   /* U+A6A9 */
  0x0000,   /* U+A6AA */
  0x0000,   /* U+A6AB */
  0x0000,   /* U+A6AC */
  0x0000,   /* U+A6AD */
  0x0000,   /* U+A6AE */
  0x0000,   /* U+A6AF */
  0x0000,   /* U+A6B0 */
  0x0000,   /* U+A6B1 */
  0x0000,   /* U+A6B2 */
  0x0000,   /* U+A6B3 */
  0x0000,   /* U+A6B4 */
  0x0000,   /* U+A6B5 */
  0x0000,   /* U+A6B6 */
  0x0000,   /* U+A6B7 */
  0x0000,   /* U+A6B8 */
  0x0000,   /* U+A6B9 */
  0x0000,   /* U+A6BA */
  0x0000,   /* U+A6BB */
  0x0000,   /* U+A6BC */
  0x0000,   /* U+A6BD */
  0x0000,   /* U+A6BE */
  0x0000,   /* U+A6BF */
  0x0000,   /* U+A6C0 */
  0x0000,   /* U+A6C1 */
  0x0000,   /* U+A6C2 */
  0x0000,   /* U+A6C3 */
  0x0000,   /* U+A6C4 */
  0x0000,   /* U+A6C5 */
  0x0000,   /* U+A6C6 */
  0x0000,   /* U+A6C7 */
  0x0000,   /* U+A6C8 */
  0x0000,   /* U+A6C9 */
  0x0000,   /* U+A6CA */
  0x0000,   /* U+A6CB */
  0x0000,   /* U+A6CC */
  0x0000,   /* U+A6CD */
  0x0000,   /* U+A6CE */
  0x0000,   /* U+A6CF */
  0x0000,   /* U+A6D0 */
  0x0000,   /* U+A6D1 */
  0x0000,   /* U+A6D2 */
  0x0000,   /* U+A6D3 */
  0x0000,   /* U+A6D4 */
  0x0000,   /* U+A6D5 */
  0x0000,   /* U+A6D6 */
  0x0000,   /* U+A6D7 */
  0x0000,   /* U+A6D8 */
  0x0000,   /* U+A6D9 */
  0x0000,   /* U+A6DA */
  0x0000,   /* U+A6DB */
  0x0000,   /* U+A6DC */
  0x0000,   /* U+A6DD */
  0x0000,   /* U+A6DE */
  0x0000,   /* U+A6DF */
  0x0000,   /* U+A6E0 */
  0x0000,   /* U+A6E1 */
  0x0000,   /* U+A6E2 */
  0x0000,   /* U+A6E3 */
  0x0000,   /* U+A6E4 */
  0x0000,   /* U+A6E5 */
  0x0000,   /* U+A6E6 */
  0x0000,   /* U+A6E7 */
  0x0000,   /* U+A6E8 */
  0x0000,   /* U+A6E9 */
  0x0000,   /* U+A6EA */
  0x0000,   /* U+A6EB */
  0x0000,   /* U+A6EC */
  0x0000,   /* U+A6ED */
  0x0000,   /* U+A6EE */
  0x0000,   /* U+A6EF */
  0x0000,   /* U+A6F0 */
  0x0000,   /* U+A6F1 */
  0x0000,   /* U+A6F2 */
  0x0000,   /* U+A6F3 */
  0x0000,   /* U+A6F4 */
  0x0000,   /* U+A6F5 */
  0x0000,   /* U+A6F6 */
  0x0000,   /* U+A6F7 */
  0x0000,   /* U+A6F8 */
  0x0000,   /* U+A6F9 */
  0x0000,   /* U+A6FA */
  0x0000,   /* U+A6FB */
  0x0000,   /* U+A6FC */
  0x0000,   /* U+A6FD */
  0x0000,   /* U+A6FE */
  0x0000,   /* U+A6FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+A600 */
  0x0000,   /* U+A601 */
  0x0000,   /* U+A602 */
  0x0000,   /* U+A603 */
  0x0000,   /* U+A604 */
  0x0000,   /* U+A605 */
  0x0000,   /* U+A606 */
  0x0000,   /* U+A607 */
  0x0000,   /* U+A608 */
  0x0000,   /* U+A609 */
  0x0000,   /* U+A60A */
  0x0000,   /* U+A60B */
  0x0000,   /* U+A60C */
  0x0000,   /* U+A60D */
  0x0000,   /* U+A60E */
  0x0000,   /* U+A60F */
  0x0000,   /* U+A610 */
  0x0000,   /* U+A611 */
  0x0000,   /* U+A612 */
  0x0004,   /* U+A613 */
  0x0004,   /* U+A614 */
  0x0004,   /* U+A615 */
  0x0004,   /* U+A616 */
  0x0004,   /* U+A617 */
  0x0004,   /* U+A618 */
  0x0004,   /* U+A619 */
  0x0004,   /* U+A61A */
  0x0004,   /* U+A61B */
  0x0004,   /* U+A61C */
  0x0004,   /* U+A61D */
  0x0004,   /* U+A61E */
  0x0004,   /* U+A61F */
  0x0000,   /* U+A620 */
  0x0000,   /* U+A621 */
  0x0000,   /* U+A622 */
  0x0000,   /* U+A623 */
  0x0000,   /* U+A624 */
  0x0000,   /* U+A625 */
  0x0000,   /* U+A626 */
  0x0000,   /* U+A627 */
  0x0000,   /* U+A628 */
  0x0000,   /* U+A629 */
  0x0000,   /* U+A62A */
  0x0000,   /* U+A62B */
  0x0000,   /* U+A62C */
  0x0000,   /* U+A62D */
  0x0000,   /* U+A62E */
  0x0000,   /* U+A62F */
  0x0000,   /* U+A630 */
  0x0000,   /* U+A631 */
  0x0000,   /* U+A632 */
  0x0000,   /* U+A633 */
  0x0000,   /* U+A634 */
  0x0000,   /* U+A635 */
  0x0000,   /* U+A636 */
  0x0000,   /* U+A637 */
  0x0000,   /* U+A638 */
  0x0000,   /* U+A639 */
  0x0000,   /* U+A63A */
  0x0000,   /* U+A63B */
  0x0000,   /* U+A63C */
  0x0000,   /* U+A63D */
  0x0000,   /* U+A63E */
  0x0000,   /* U+A63F */
  0x0000,   /* U+A640 */
  0x0000,   /* U+A641 */
  0x0000,   /* U+A642 */
  0x0000,   /* U+A643 */
  0x0000,   /* U+A644 */
  0x0000,   /* U+A645 */
  0x0000,   /* U+A646 */
  0x0000,   /* U+A647 */
  0x0000,   /* U+A648 */
  0x0000,   /* U+A649 */
  0x0000,   /* U+A64A */
  0x0000,   /* U+A64B */
  0x0000,   /* U+A64C */
  0x0000,   /* U+A64D */
  0x0000,   /* U+A64E */
  0x0000,   /* U+A64F */
  0x0000,   /* U+A650 */
  0x0000,   /* U+A651 */
  0x0000,   /* U+A652 */
  0x0000,   /* U+A653 */
  0x0000,   /* U+A654 */
  0x0000,   /* U+A655 */
  0x0000,   /* U+A656 */
  0x0000,   /* U+A657 */
  0x0000,   /* U+A658 */
  0x0000,   /* U+A659 */
  0x0000,   /* U+A65A */
  0x0000,   /* U+A65B */
  0x0000,   /* U+A65C */
  0x0000,   /* U+A65D */
  0x0000,   /* U+A65E */
  0x0000,   /* U+A65F */
  0x0000,   /* U+A660 */
  0x0000,   /* U+A661 */
  0x0000,   /* U+A662 */
  0x0000,   /* U+A663 */
  0x0000,   /* U+A664 */
  0x0000,   /* U+A665 */
  0x0000,   /* U+A666 */
  0x0000,   /* U+A667 */
  0x0000,   /* U+A668 */
  0x0000,   /* U+A669 */
  0x0000,   /* U+A66A */
  0x0000,   /* U+A66B */
  0x0000,   /* U+A66C */
  0x0000,   /* U+A66D */
  0x0000,   /* U+A66E */
  0x0000,   /* U+A66F */
  0x0000,   /* U+A670 */
  0x0000,   /* U+A671 */
  0x0000,   /* U+A672 */
  0x0000,   /* U+A673 */
  0x0000,   /* U+A674 */
  0x0000,   /* U+A675 */
  0x0004,   /* U+A676 */
  0x0000,   /* U+A677 */
  0x0000,   /* U+A678 */
  0x0000,   /* U+A679 */
  0x0000,   /* U+A67A */
  0x0000,   /* U+A67B */
  0x0000,   /* U+A67C */
  0x0000,   /* U+A67D */
  0x0000,   /* U+A67E */
  0x0000,   /* U+A67F */
  0x0000,   /* U+A680 */
  0x0000,   /* U+A681 */
  0x0000,   /* U+A682 */
  0x0000,   /* U+A683 */
  0x0000,   /* U+A684 */
  0x0000,   /* U+A685 */
  0x0000,   /* U+A686 */
  0x0000,   /* U+A687 */
  0x0000,   /* U+A688 */
  0x0000,   /* U+A689 */
  0x0000,   /* U+A68A */
  0x0000,   /* U+A68B */
  0x0000,   /* U+A68C */
  0x0000,   /* U+A68D */
  0x0000,   /* U+A68E */
  0x0000,   /* U+A68F */
  0x0000,   /* U+A690 */
  0x0000,   /* U+A691 */
  0x0000,   /* U+A692 */
  0x0000,   /* U+A693 */
  0x0000,   /* U+A694 */
  0x0000,   /* U+A695 */
  0x0000,   /* U+A696 */
  0x0000,   /* U+A697 */
  0x0000,   /* U+A698 */
  0x0000,   /* U+A699 */
  0x0000,   /* U+A69A */
  0x0000,   /* U+A69B */
  0x0000,   /* U+A69C */
  0x0000,   /* U+A69D */
  0x0000,   /* U+A69E */
  0x0000,   /* U+A69F */
  0x0000,   /* U+A6A0 */
  0x0000,   /* U+A6A1 */
  0x0000,   /* U+A6A2 */
  0x0000,   /* U+A6A3 */
  0x0000,   /* U+A6A4 */
  0x0000,   /* U+A6A5 */
  0x0000,   /* U+A6A6 */
  0x0000,   /* U+A6A7 */
  0x0000,   /* U+A6A8 */
  0x0000,   /* U+A6A9 */
  0x0000,   /* U+A6AA */
  0x0000,   /* U+A6AB */
  0x0000,   /* U+A6AC */
  0x0000,   /* U+A6AD */
  0x0000,   /* U+A6AE */
  0x0000,   /* U+A6AF */
  0x0000,   /* U+A6B0 */
  0x0000,   /* U+A6B1 */
  0x0000,   /* U+A6B2 */
  0x0000,   /* U+A6B3 */
  0x0000,   /* U+A6B4 */
  0x0000,   /* U+A6B5 */
  0x0000,   /* U+A6B6 */
  0x0000,   /* U+A6B7 */
  0x0000,   /* U+A6B8 */
  0x0000,   /* U+A6B9 */
  0x0000,   /* U+A6BA */
  0x0000,   /* U+A6BB */
  0x0000,   /* U+A6BC */
  0x0000,   /* U+A6BD */
  0x0000,   /* U+A6BE */
  0x0000,   /* U+A6BF */
  0x0000,   /* U+A6C0 */
  0x0000,   /* U+A6C1 */
  0x0000,   /* U+A6C2 */
  0x0000,   /* U+A6C3 */
  0x0000,   /* U+A6C4 */
  0x0000,   /* U+A6C5 */
  0x0000,   /* U+A6C6 */
  0x0000,   /* U+A6C7 */
  0x0000,   /* U+A6C8 */
  0x0000,   /* U+A6C9 */
  0x0000,   /* U+A6CA */
  0x0000,   /* U+A6CB */
  0x0000,   /* U+A6CC */
  0x0000,   /* U+A6CD */
  0x0000,   /* U+A6CE */
  0x0000,   /* U+A6CF */
  0x0000,   /* U+A6D0 */
  0x0000,   /* U+A6D1 */
  0x0000,   /* U+A6D2 */
  0x0000,   /* U+A6D3 */
  0x0000,   /* U+A6D4 */
  0x0000,   /* U+A6D5 */
  0x0000,   /* U+A6D6 */
  0x0000,   /* U+A6D7 */
  0x0000,   /* U+A6D8 */
  0x0000,   /* U+A6D9 */
  0x0000,   /* U+A6DA */
  0x0000,   /* U+A6DB */
  0x0000,   /* U+A6DC */
  0x0000,   /* U+A6DD */
  0x0000,   /* U+A6DE */
  0x0000,   /* U+A6DF */
  0x0000,   /* U+A6E0 */
  0x0000,   /* U+A6E1 */
  0x0000,   /* U+A6E2 */
  0x0000,   /* U+A6E3 */
  0x0000,   /* U+A6E4 */
  0x0000,   /* U+A6E5 */
  0x0000,   /* U+A6E6 */
  0x0000,   /* U+A6E7 */
  0x0000,   /* U+A6E8 */
  0x0000,   /* U+A6E9 */
  0x0000,   /* U+A6EA */
  0x0000,   /* U+A6EB */
  0x0000,   /* U+A6EC */
  0x0000,   /* U+A6ED */
  0x0000,   /* U+A6EE */
  0x0000,   /* U+A6EF */
  0x0000,   /* U+A6F0 */
  0x0000,   /* U+A6F1 */
  0x0000,   /* U+A6F2 */
  0x0000,   /* U+A6F3 */
  0x0000,   /* U+A6F4 */
  0x0000,   /* U+A6F5 */
  0x0000,   /* U+A6F6 */
  0x0000,   /* U+A6F7 */
  0x0000,   /* U+A6F8 */
  0x0000,   /* U+A6F9 */
  0x0000,   /* U+A6FA */
  0x0000,   /* U+A6FB */
  0x0000,   /* U+A6FC */
  0x0000,   /* U+A6FD */
  0x0000,   /* U+A6FE */
  0x0000,   /* U+A6FF */
};

uint16 uca900_p0A7[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 1, 2, 3, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x04D0,   /* U+A700 */
  0x04D1,   /* U+A701 */
  0x04D2,   /* U+A702 */
  0x04D3,   /* U+A703 */
  0x04D4,   /* U+A704 */
  0x04D5,   /* U+A705 */
  0x04D6,   /* U+A706 */
  0x04D7,   /* U+A707 */
  0x04D8,   /* U+A708 */
  0x04D9,   /* U+A709 */
  0x04DA,   /* U+A70A */
  0x04DB,   /* U+A70B */
  0x04DC,   /* U+A70C */
  0x04DD,   /* U+A70D */
  0x04DE,   /* U+A70E */
  0x04DF,   /* U+A70F */
  0x04E0,   /* U+A710 */
  0x04E1,   /* U+A711 */
  0x04E2,   /* U+A712 */
  0x04E3,   /* U+A713 */
  0x04E4,   /* U+A714 */
  0x04E5,   /* U+A715 */
  0x04E6,   /* U+A716 */
  0x04E7,   /* U+A717 */
  0x04E8,   /* U+A718 */
  0x04E9,   /* U+A719 */
  0x04EA,   /* U+A71A */
  0x04EB,   /* U+A71B */
  0x04EC,   /* U+A71C */
  0x04ED,   /* U+A71D */
  0x04EE,   /* U+A71E */
  0x04EF,   /* U+A71F */
  0x04F0,   /* U+A720 */
  0x04F1,   /* U+A721 */
  0x1F81,   /* U+A722 */
  0x1F81,   /* U+A723 */
  0x1F8C,   /* U+A724 */
  0x1F8C,   /* U+A725 */
  0x1D2B,   /* U+A726 */
  0x1D2B,   /* U+A727 */
  0x1E95,   /* U+A728 */
  0x1E95,   /* U+A729 */
  0x1F66,   /* U+A72A */
  0x1F66,   /* U+A72B */
  0x1F67,   /* U+A72C */
  0x1F67,   /* U+A72D */
  0x1F68,   /* U+A72E */
  0x1F68,   /* U+A72F */
  0x1CE9,   /* U+A730 */
  0x1E75,   /* U+A731 */
  0x1C47,   /* U+A732 */
  0x1C47,   /* U+A733 */
  0x1C47,   /* U+A734 */
  0x1C47,   /* U+A735 */
  0x1C47,   /* U+A736 */
  0x1C47,   /* U+A737 */
  0x1C47,   /* U+A738 */
  0x1C47,   /* U+A739 */
  0x1C47,   /* U+A73A */
  0x1C47,   /* U+A73B */
  0x1C47,   /* U+A73C */
  0x1C47,   /* U+A73D */
  0x1C8E,   /* U+A73E */
  0x1C8E,   /* U+A73F */
  0x1D70,   /* U+A740 */
  0x1D70,   /* U+A741 */
  0x1D71,   /* U+A742 */
  0x1D71,   /* U+A743 */
  0x1D72,   /* U+A744 */
  0x1D72,   /* U+A745 */
  0x1D7F,   /* U+A746 */
  0x1D7F,   /* U+A747 */
  0x1D81,   /* U+A748 */
  0x1D81,   /* U+A749 */
  0x1E01,   /* U+A74A */
  0x1E01,   /* U+A74B */
  0x1DF9,   /* U+A74C */
  0x1DF9,   /* U+A74D */
  0x1DDD,   /* U+A74E */
  0x1DDD,   /* U+A74F */
  0x1E12,   /* U+A750 */
  0x1E12,   /* U+A751 */
  0x1E19,   /* U+A752 */
  0x1E19,   /* U+A753 */
  0x1E1A,   /* U+A754 */
  0x1E1A,   /* U+A755 */
  0x1E25,   /* U+A756 */
  0x1E25,   /* U+A757 */
  0x1E26,   /* U+A758 */
  0x1E26,   /* U+A759 */
  0x1E3D,   /* U+A75A */
  0x1E3D,   /* U+A75B */
  0x1E70,   /* U+A75C */
  0x1E70,   /* U+A75D */
  0x1EE8,   /* U+A75E */
  0x1EE8,   /* U+A75F */
  0x1EE3,   /* U+A760 */
  0x1EE3,   /* U+A761 */
  0x1F3D,   /* U+A762 */
  0x1F3D,   /* U+A763 */
  0x1F54,   /* U+A764 */
  0x1F54,   /* U+A765 */
  0x1F55,   /* U+A766 */
  0x1F55,   /* U+A767 */
  0x1F5A,   /* U+A768 */
  0x1F5A,   /* U+A769 */
  0x1F5E,   /* U+A76A */
  0x1F5E,   /* U+A76B */
  0x1F5F,   /* U+A76C */
  0x1F5F,   /* U+A76D */
  0x1F60,   /* U+A76E */
  0x1F60,   /* U+A76F */
  0x1F60,   /* U+A770 */
  0x1CA8,   /* U+A771 */
  0x1D9C,   /* U+A772 */
  0x1DB8,   /* U+A773 */
  0x1DD7,   /* U+A774 */
  0x1E6E,   /* U+A775 */
  0x1E6F,   /* U+A776 */
  0x1EB0,   /* U+A777 */
  0x1F61,   /* U+A778 */
  0x1C8F,   /* U+A779 */
  0x1C8F,   /* U+A77A */
  0x1CE5,   /* U+A77B */
  0x1CE5,   /* U+A77C */
  0x1CF4,   /* U+A77D */
  0x1D0F,   /* U+A77E */
  0x1D0F,   /* U+A77F */
  0x1DA1,   /* U+A780 */
  0x1DA1,   /* U+A781 */
  0x1E33,   /* U+A782 */
  0x1E33,   /* U+A783 */
  0x1E71,   /* U+A784 */
  0x1E71,   /* U+A785 */
  0x1E95,   /* U+A786 */
  0x1E95,   /* U+A787 */
  0x04F2,   /* U+A788 */
  0x04F3,   /* U+A789 */
  0x04F4,   /* U+A78A */
  0x1F82,   /* U+A78B */
  0x1F82,   /* U+A78C */
  0x1EC8,   /* U+A78D */
  0x1D97,   /* U+A78E */
  0x1F83,   /* U+A78F */
  0x1DCC,   /* U+A790 */
  0x1DCC,   /* U+A791 */
  0x1C83,   /* U+A792 */
  0x1C83,   /* U+A793 */
  0x1C84,   /* U+A794 */
  0x1D24,   /* U+A795 */
  0x1C6F,   /* U+A796 */
  0x1C6F,   /* U+A797 */
  0x1CEB,   /* U+A798 */
  0x1CEB,   /* U+A799 */
  0x1C47,   /* U+A79A */
  0x1C47,   /* U+A79B */
  0x1DDD,   /* U+A79C */
  0x1DDD,   /* U+A79D */
  0x1EB5,   /* U+A79E */
  0x1EB5,   /* U+A79F */
  0x1CF4,   /* U+A7A0 */
  0x1CF4,   /* U+A7A1 */
  0x1D65,   /* U+A7A2 */
  0x1D65,   /* U+A7A3 */
  0x1DB9,   /* U+A7A4 */
  0x1DB9,   /* U+A7A5 */
  0x1E33,   /* U+A7A6 */
  0x1E33,   /* U+A7A7 */
  0x1E71,   /* U+A7A8 */
  0x1E71,   /* U+A7A9 */
  0x1D25,   /* U+A7AA */
  0x1CCF,   /* U+A7AB */
  0x1CF8,   /* U+A7AC */
  0x1D8D,   /* U+A7AD */
  0x1D3A,   /* U+A7AE */
  0xFBC1,   /* U+A7AF */
  0x1D73,   /* U+A7B0 */
  0x1EB1,   /* U+A7B1 */
  0x1D59,   /* U+A7B2 */
  0x1F08,   /* U+A7B3 */
  0x1C79,   /* U+A7B4 */
  0x1C79,   /* U+A7B5 */
  0x1E06,   /* U+A7B6 */
  0x1E06,   /* U+A7B7 */
  0xFBC1,   /* U+A7B8 */
  0xFBC1,   /* U+A7B9 */
  0xFBC1,   /* U+A7BA */
  0xFBC1,   /* U+A7BB */
  0xFBC1,   /* U+A7BC */
  0xFBC1,   /* U+A7BD */
  0xFBC1,   /* U+A7BE */
  0xFBC1,   /* U+A7BF */
  0xFBC1,   /* U+A7C0 */
  0xFBC1,   /* U+A7C1 */
  0xFBC1,   /* U+A7C2 */
  0xFBC1,   /* U+A7C3 */
  0xFBC1,   /* U+A7C4 */
  0xFBC1,   /* U+A7C5 */
  0xFBC1,   /* U+A7C6 */
  0xFBC1,   /* U+A7C7 */
  0xFBC1,   /* U+A7C8 */
  0xFBC1,   /* U+A7C9 */
  0xFBC1,   /* U+A7CA */
  0xFBC1,   /* U+A7CB */
  0xFBC1,   /* U+A7CC */
  0xFBC1,   /* U+A7CD */
  0xFBC1,   /* U+A7CE */
  0xFBC1,   /* U+A7CF */
  0xFBC1,   /* U+A7D0 */
  0xFBC1,   /* U+A7D1 */
  0xFBC1,   /* U+A7D2 */
  0xFBC1,   /* U+A7D3 */
  0xFBC1,   /* U+A7D4 */
  0xFBC1,   /* U+A7D5 */
  0xFBC1,   /* U+A7D6 */
  0xFBC1,   /* U+A7D7 */
  0xFBC1,   /* U+A7D8 */
  0xFBC1,   /* U+A7D9 */
  0xFBC1,   /* U+A7DA */
  0xFBC1,   /* U+A7DB */
  0xFBC1,   /* U+A7DC */
  0xFBC1,   /* U+A7DD */
  0xFBC1,   /* U+A7DE */
  0xFBC1,   /* U+A7DF */
  0xFBC1,   /* U+A7E0 */
  0xFBC1,   /* U+A7E1 */
  0xFBC1,   /* U+A7E2 */
  0xFBC1,   /* U+A7E3 */
  0xFBC1,   /* U+A7E4 */
  0xFBC1,   /* U+A7E5 */
  0xFBC1,   /* U+A7E6 */
  0xFBC1,   /* U+A7E7 */
  0xFBC1,   /* U+A7E8 */
  0xFBC1,   /* U+A7E9 */
  0xFBC1,   /* U+A7EA */
  0xFBC1,   /* U+A7EB */
  0xFBC1,   /* U+A7EC */
  0xFBC1,   /* U+A7ED */
  0xFBC1,   /* U+A7EE */
  0xFBC1,   /* U+A7EF */
  0xFBC1,   /* U+A7F0 */
  0xFBC1,   /* U+A7F1 */
  0xFBC1,   /* U+A7F2 */
  0xFBC1,   /* U+A7F3 */
  0xFBC1,   /* U+A7F4 */
  0xFBC1,   /* U+A7F5 */
  0xFBC1,   /* U+A7F6 */
  0x1D3F,   /* U+A7F7 */
  0x1D18,   /* U+A7F8 */
  0x1DDD,   /* U+A7F9 */
  0x1ED8,   /* U+A7FA */
  0x1CF3,   /* U+A7FB */
  0x1E1B,   /* U+A7FC */
  0x1DB6,   /* U+A7FD */
  0x1D3E,   /* U+A7FE */
  0x1DB7,   /* U+A7FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A700 */
  0x0020,   /* U+A701 */
  0x0020,   /* U+A702 */
  0x0020,   /* U+A703 */
  0x0020,   /* U+A704 */
  0x0020,   /* U+A705 */
  0x0020,   /* U+A706 */
  0x0020,   /* U+A707 */
  0x0020,   /* U+A708 */
  0x0020,   /* U+A709 */
  0x0020,   /* U+A70A */
  0x0020,   /* U+A70B */
  0x0020,   /* U+A70C */
  0x0020,   /* U+A70D */
  0x0020,   /* U+A70E */
  0x0020,   /* U+A70F */
  0x0020,   /* U+A710 */
  0x0020,   /* U+A711 */
  0x0020,   /* U+A712 */
  0x0020,   /* U+A713 */
  0x0020,   /* U+A714 */
  0x0020,   /* U+A715 */
  0x0020,   /* U+A716 */
  0x0020,   /* U+A717 */
  0x0020,   /* U+A718 */
  0x0020,   /* U+A719 */
  0x0020,   /* U+A71A */
  0x0020,   /* U+A71B */
  0x0020,   /* U+A71C */
  0x0020,   /* U+A71D */
  0x0020,   /* U+A71E */
  0x0020,   /* U+A71F */
  0x0020,   /* U+A720 */
  0x0020,   /* U+A721 */
  0x0020,   /* U+A722 */
  0x0020,   /* U+A723 */
  0x0020,   /* U+A724 */
  0x0020,   /* U+A725 */
  0x0020,   /* U+A726 */
  0x0020,   /* U+A727 */
  0x0020,   /* U+A728 */
  0x0020,   /* U+A729 */
  0x0020,   /* U+A72A */
  0x0020,   /* U+A72B */
  0x0020,   /* U+A72C */
  0x0020,   /* U+A72D */
  0x0020,   /* U+A72E */
  0x0020,   /* U+A72F */
  0x0020,   /* U+A730 */
  0x0020,   /* U+A731 */
  0x0020,   /* U+A732 */
  0x0020,   /* U+A733 */
  0x0020,   /* U+A734 */
  0x0020,   /* U+A735 */
  0x0020,   /* U+A736 */
  0x0020,   /* U+A737 */
  0x0020,   /* U+A738 */
  0x0020,   /* U+A739 */
  0x0020,   /* U+A73A */
  0x0020,   /* U+A73B */
  0x0020,   /* U+A73C */
  0x0020,   /* U+A73D */
  0x0020,   /* U+A73E */
  0x0020,   /* U+A73F */
  0x0020,   /* U+A740 */
  0x0020,   /* U+A741 */
  0x0020,   /* U+A742 */
  0x0020,   /* U+A743 */
  0x0020,   /* U+A744 */
  0x0020,   /* U+A745 */
  0x0020,   /* U+A746 */
  0x0020,   /* U+A747 */
  0x0020,   /* U+A748 */
  0x0020,   /* U+A749 */
  0x0020,   /* U+A74A */
  0x0020,   /* U+A74B */
  0x0020,   /* U+A74C */
  0x0020,   /* U+A74D */
  0x0020,   /* U+A74E */
  0x0020,   /* U+A74F */
  0x0020,   /* U+A750 */
  0x0020,   /* U+A751 */
  0x0020,   /* U+A752 */
  0x0020,   /* U+A753 */
  0x0020,   /* U+A754 */
  0x0020,   /* U+A755 */
  0x0020,   /* U+A756 */
  0x0020,   /* U+A757 */
  0x0020,   /* U+A758 */
  0x0020,   /* U+A759 */
  0x0020,   /* U+A75A */
  0x0020,   /* U+A75B */
  0x0020,   /* U+A75C */
  0x0020,   /* U+A75D */
  0x0020,   /* U+A75E */
  0x0020,   /* U+A75F */
  0x0020,   /* U+A760 */
  0x0020,   /* U+A761 */
  0x0020,   /* U+A762 */
  0x0020,   /* U+A763 */
  0x0020,   /* U+A764 */
  0x0020,   /* U+A765 */
  0x0020,   /* U+A766 */
  0x0020,   /* U+A767 */
  0x0020,   /* U+A768 */
  0x0020,   /* U+A769 */
  0x0020,   /* U+A76A */
  0x0020,   /* U+A76B */
  0x0020,   /* U+A76C */
  0x0020,   /* U+A76D */
  0x0020,   /* U+A76E */
  0x0020,   /* U+A76F */
  0x0020,   /* U+A770 */
  0x0020,   /* U+A771 */
  0x0020,   /* U+A772 */
  0x0020,   /* U+A773 */
  0x0020,   /* U+A774 */
  0x0020,   /* U+A775 */
  0x0020,   /* U+A776 */
  0x0020,   /* U+A777 */
  0x0020,   /* U+A778 */
  0x0020,   /* U+A779 */
  0x0020,   /* U+A77A */
  0x0020,   /* U+A77B */
  0x0020,   /* U+A77C */
  0x0020,   /* U+A77D */
  0x0020,   /* U+A77E */
  0x0020,   /* U+A77F */
  0x0020,   /* U+A780 */
  0x0020,   /* U+A781 */
  0x0020,   /* U+A782 */
  0x0020,   /* U+A783 */
  0x0020,   /* U+A784 */
  0x0020,   /* U+A785 */
  0x0020,   /* U+A786 */
  0x0020,   /* U+A787 */
  0x0020,   /* U+A788 */
  0x0020,   /* U+A789 */
  0x0020,   /* U+A78A */
  0x0020,   /* U+A78B */
  0x0020,   /* U+A78C */
  0x0020,   /* U+A78D */
  0x0020,   /* U+A78E */
  0x0020,   /* U+A78F */
  0x0020,   /* U+A790 */
  0x0020,   /* U+A791 */
  0x0020,   /* U+A792 */
  0x0020,   /* U+A793 */
  0x0020,   /* U+A794 */
  0x0020,   /* U+A795 */
  0x0020,   /* U+A796 */
  0x0020,   /* U+A797 */
  0x0020,   /* U+A798 */
  0x0020,   /* U+A799 */
  0x0020,   /* U+A79A */
  0x0020,   /* U+A79B */
  0x0020,   /* U+A79C */
  0x0020,   /* U+A79D */
  0x0020,   /* U+A79E */
  0x0020,   /* U+A79F */
  0x0020,   /* U+A7A0 */
  0x0020,   /* U+A7A1 */
  0x0020,   /* U+A7A2 */
  0x0020,   /* U+A7A3 */
  0x0020,   /* U+A7A4 */
  0x0020,   /* U+A7A5 */
  0x0020,   /* U+A7A6 */
  0x0020,   /* U+A7A7 */
  0x0020,   /* U+A7A8 */
  0x0020,   /* U+A7A9 */
  0x0020,   /* U+A7AA */
  0x0020,   /* U+A7AB */
  0x0020,   /* U+A7AC */
  0x0020,   /* U+A7AD */
  0x0020,   /* U+A7AE */
  0x0020,   /* U+A7AF */
  0x0020,   /* U+A7B0 */
  0x0020,   /* U+A7B1 */
  0x0020,   /* U+A7B2 */
  0x0020,   /* U+A7B3 */
  0x0020,   /* U+A7B4 */
  0x0020,   /* U+A7B5 */
  0x0020,   /* U+A7B6 */
  0x0020,   /* U+A7B7 */
  0x0020,   /* U+A7B8 */
  0x0020,   /* U+A7B9 */
  0x0020,   /* U+A7BA */
  0x0020,   /* U+A7BB */
  0x0020,   /* U+A7BC */
  0x0020,   /* U+A7BD */
  0x0020,   /* U+A7BE */
  0x0020,   /* U+A7BF */
  0x0020,   /* U+A7C0 */
  0x0020,   /* U+A7C1 */
  0x0020,   /* U+A7C2 */
  0x0020,   /* U+A7C3 */
  0x0020,   /* U+A7C4 */
  0x0020,   /* U+A7C5 */
  0x0020,   /* U+A7C6 */
  0x0020,   /* U+A7C7 */
  0x0020,   /* U+A7C8 */
  0x0020,   /* U+A7C9 */
  0x0020,   /* U+A7CA */
  0x0020,   /* U+A7CB */
  0x0020,   /* U+A7CC */
  0x0020,   /* U+A7CD */
  0x0020,   /* U+A7CE */
  0x0020,   /* U+A7CF */
  0x0020,   /* U+A7D0 */
  0x0020,   /* U+A7D1 */
  0x0020,   /* U+A7D2 */
  0x0020,   /* U+A7D3 */
  0x0020,   /* U+A7D4 */
  0x0020,   /* U+A7D5 */
  0x0020,   /* U+A7D6 */
  0x0020,   /* U+A7D7 */
  0x0020,   /* U+A7D8 */
  0x0020,   /* U+A7D9 */
  0x0020,   /* U+A7DA */
  0x0020,   /* U+A7DB */
  0x0020,   /* U+A7DC */
  0x0020,   /* U+A7DD */
  0x0020,   /* U+A7DE */
  0x0020,   /* U+A7DF */
  0x0020,   /* U+A7E0 */
  0x0020,   /* U+A7E1 */
  0x0020,   /* U+A7E2 */
  0x0020,   /* U+A7E3 */
  0x0020,   /* U+A7E4 */
  0x0020,   /* U+A7E5 */
  0x0020,   /* U+A7E6 */
  0x0020,   /* U+A7E7 */
  0x0020,   /* U+A7E8 */
  0x0020,   /* U+A7E9 */
  0x0020,   /* U+A7EA */
  0x0020,   /* U+A7EB */
  0x0020,   /* U+A7EC */
  0x0020,   /* U+A7ED */
  0x0020,   /* U+A7EE */
  0x0020,   /* U+A7EF */
  0x0020,   /* U+A7F0 */
  0x0020,   /* U+A7F1 */
  0x0020,   /* U+A7F2 */
  0x0020,   /* U+A7F3 */
  0x0020,   /* U+A7F4 */
  0x0020,   /* U+A7F5 */
  0x0020,   /* U+A7F6 */
  0x0020,   /* U+A7F7 */
  0x0020,   /* U+A7F8 */
  0x0020,   /* U+A7F9 */
  0x0020,   /* U+A7FA */
  0x0020,   /* U+A7FB */
  0x0020,   /* U+A7FC */
  0x0020,   /* U+A7FD */
  0x0020,   /* U+A7FE */
  0x0020,   /* U+A7FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A700 */
  0x0002,   /* U+A701 */
  0x0002,   /* U+A702 */
  0x0002,   /* U+A703 */
  0x0002,   /* U+A704 */
  0x0002,   /* U+A705 */
  0x0002,   /* U+A706 */
  0x0002,   /* U+A707 */
  0x0002,   /* U+A708 */
  0x0002,   /* U+A709 */
  0x0002,   /* U+A70A */
  0x0002,   /* U+A70B */
  0x0002,   /* U+A70C */
  0x0002,   /* U+A70D */
  0x0002,   /* U+A70E */
  0x0002,   /* U+A70F */
  0x0002,   /* U+A710 */
  0x0002,   /* U+A711 */
  0x0002,   /* U+A712 */
  0x0002,   /* U+A713 */
  0x0002,   /* U+A714 */
  0x0002,   /* U+A715 */
  0x0002,   /* U+A716 */
  0x0002,   /* U+A717 */
  0x0002,   /* U+A718 */
  0x0002,   /* U+A719 */
  0x0002,   /* U+A71A */
  0x0002,   /* U+A71B */
  0x0002,   /* U+A71C */
  0x0002,   /* U+A71D */
  0x0002,   /* U+A71E */
  0x0002,   /* U+A71F */
  0x0002,   /* U+A720 */
  0x0002,   /* U+A721 */
  0x0008,   /* U+A722 */
  0x0002,   /* U+A723 */
  0x0008,   /* U+A724 */
  0x0002,   /* U+A725 */
  0x0008,   /* U+A726 */
  0x0002,   /* U+A727 */
  0x000A,   /* U+A728 */
  0x0004,   /* U+A729 */
  0x0008,   /* U+A72A */
  0x0002,   /* U+A72B */
  0x0008,   /* U+A72C */
  0x0002,   /* U+A72D */
  0x0008,   /* U+A72E */
  0x0002,   /* U+A72F */
  0x0002,   /* U+A730 */
  0x0002,   /* U+A731 */
  0x000A,   /* U+A732 */
  0x0004,   /* U+A733 */
  0x000A,   /* U+A734 */
  0x0004,   /* U+A735 */
  0x000A,   /* U+A736 */
  0x0004,   /* U+A737 */
  0x000A,   /* U+A738 */
  0x0004,   /* U+A739 */
  0x000A,   /* U+A73A */
  0x0004,   /* U+A73B */
  0x000A,   /* U+A73C */
  0x0004,   /* U+A73D */
  0x0008,   /* U+A73E */
  0x0002,   /* U+A73F */
  0x0008,   /* U+A740 */
  0x0002,   /* U+A741 */
  0x0008,   /* U+A742 */
  0x0002,   /* U+A743 */
  0x0008,   /* U+A744 */
  0x0002,   /* U+A745 */
  0x0008,   /* U+A746 */
  0x0002,   /* U+A747 */
  0x0008,   /* U+A748 */
  0x0002,   /* U+A749 */
  0x0008,   /* U+A74A */
  0x0002,   /* U+A74B */
  0x0008,   /* U+A74C */
  0x0002,   /* U+A74D */
  0x000A,   /* U+A74E */
  0x0004,   /* U+A74F */
  0x0008,   /* U+A750 */
  0x0002,   /* U+A751 */
  0x0008,   /* U+A752 */
  0x0002,   /* U+A753 */
  0x0008,   /* U+A754 */
  0x0002,   /* U+A755 */
  0x0008,   /* U+A756 */
  0x0002,   /* U+A757 */
  0x0008,   /* U+A758 */
  0x0002,   /* U+A759 */
  0x0008,   /* U+A75A */
  0x0002,   /* U+A75B */
  0x0008,   /* U+A75C */
  0x0002,   /* U+A75D */
  0x0008,   /* U+A75E */
  0x0002,   /* U+A75F */
  0x000A,   /* U+A760 */
  0x0004,   /* U+A761 */
  0x0008,   /* U+A762 */
  0x0002,   /* U+A763 */
  0x0008,   /* U+A764 */
  0x0002,   /* U+A765 */
  0x0008,   /* U+A766 */
  0x0002,   /* U+A767 */
  0x0008,   /* U+A768 */
  0x0002,   /* U+A769 */
  0x0008,   /* U+A76A */
  0x0002,   /* U+A76B */
  0x0008,   /* U+A76C */
  0x0002,   /* U+A76D */
  0x0008,   /* U+A76E */
  0x0002,   /* U+A76F */
  0x0014,   /* U+A770 */
  0x0002,   /* U+A771 */
  0x0002,   /* U+A772 */
  0x0002,   /* U+A773 */
  0x0002,   /* U+A774 */
  0x0002,   /* U+A775 */
  0x0002,   /* U+A776 */
  0x0002,   /* U+A777 */
  0x0002,   /* U+A778 */
  0x000A,   /* U+A779 */
  0x0004,   /* U+A77A */
  0x000A,   /* U+A77B */
  0x0004,   /* U+A77C */
  0x000A,   /* U+A77D */
  0x0008,   /* U+A77E */
  0x0002,   /* U+A77F */
  0x0008,   /* U+A780 */
  0x0002,   /* U+A781 */
  0x000A,   /* U+A782 */
  0x0004,   /* U+A783 */
  0x000A,   /* U+A784 */
  0x0004,   /* U+A785 */
  0x000A,   /* U+A786 */
  0x0004,   /* U+A787 */
  0x0002,   /* U+A788 */
  0x0002,   /* U+A789 */
  0x0002,   /* U+A78A */
  0x0008,   /* U+A78B */
  0x0002,   /* U+A78C */
  0x0008,   /* U+A78D */
  0x0002,   /* U+A78E */
  0x0002,   /* U+A78F */
  0x0008,   /* U+A790 */
  0x0002,   /* U+A791 */
  0x0008,   /* U+A792 */
  0x0002,   /* U+A793 */
  0x0002,   /* U+A794 */
  0x0002,   /* U+A795 */
  0x0008,   /* U+A796 */
  0x0002,   /* U+A797 */
  0x0008,   /* U+A798 */
  0x0002,   /* U+A799 */
  0x000A,   /* U+A79A */
  0x0004,   /* U+A79B */
  0x000A,   /* U+A79C */
  0x0004,   /* U+A79D */
  0x000A,   /* U+A79E */
  0x0004,   /* U+A79F */
  0x000A,   /* U+A7A0 */
  0x0004,   /* U+A7A1 */
  0x000A,   /* U+A7A2 */
  0x0004,   /* U+A7A3 */
  0x000A,   /* U+A7A4 */
  0x0004,   /* U+A7A5 */
  0x000A,   /* U+A7A6 */
  0x0004,   /* U+A7A7 */
  0x000A,   /* U+A7A8 */
  0x0004,   /* U+A7A9 */
  0x0008,   /* U+A7AA */
  0x0008,   /* U+A7AB */
  0x0008,   /* U+A7AC */
  0x0008,   /* U+A7AD */
  0x0008,   /* U+A7AE */
  0x0002,   /* U+A7AF */
  0x0008,   /* U+A7B0 */
  0x0008,   /* U+A7B1 */
  0x0008,   /* U+A7B2 */
  0x0008,   /* U+A7B3 */
  0x0008,   /* U+A7B4 */
  0x0002,   /* U+A7B5 */
  0x0008,   /* U+A7B6 */
  0x0002,   /* U+A7B7 */
  0x0002,   /* U+A7B8 */
  0x0002,   /* U+A7B9 */
  0x0002,   /* U+A7BA */
  0x0002,   /* U+A7BB */
  0x0002,   /* U+A7BC */
  0x0002,   /* U+A7BD */
  0x0002,   /* U+A7BE */
  0x0002,   /* U+A7BF */
  0x0002,   /* U+A7C0 */
  0x0002,   /* U+A7C1 */
  0x0002,   /* U+A7C2 */
  0x0002,   /* U+A7C3 */
  0x0002,   /* U+A7C4 */
  0x0002,   /* U+A7C5 */
  0x0002,   /* U+A7C6 */
  0x0002,   /* U+A7C7 */
  0x0002,   /* U+A7C8 */
  0x0002,   /* U+A7C9 */
  0x0002,   /* U+A7CA */
  0x0002,   /* U+A7CB */
  0x0002,   /* U+A7CC */
  0x0002,   /* U+A7CD */
  0x0002,   /* U+A7CE */
  0x0002,   /* U+A7CF */
  0x0002,   /* U+A7D0 */
  0x0002,   /* U+A7D1 */
  0x0002,   /* U+A7D2 */
  0x0002,   /* U+A7D3 */
  0x0002,   /* U+A7D4 */
  0x0002,   /* U+A7D5 */
  0x0002,   /* U+A7D6 */
  0x0002,   /* U+A7D7 */
  0x0002,   /* U+A7D8 */
  0x0002,   /* U+A7D9 */
  0x0002,   /* U+A7DA */
  0x0002,   /* U+A7DB */
  0x0002,   /* U+A7DC */
  0x0002,   /* U+A7DD */
  0x0002,   /* U+A7DE */
  0x0002,   /* U+A7DF */
  0x0002,   /* U+A7E0 */
  0x0002,   /* U+A7E1 */
  0x0002,   /* U+A7E2 */
  0x0002,   /* U+A7E3 */
  0x0002,   /* U+A7E4 */
  0x0002,   /* U+A7E5 */
  0x0002,   /* U+A7E6 */
  0x0002,   /* U+A7E7 */
  0x0002,   /* U+A7E8 */
  0x0002,   /* U+A7E9 */
  0x0002,   /* U+A7EA */
  0x0002,   /* U+A7EB */
  0x0002,   /* U+A7EC */
  0x0002,   /* U+A7ED */
  0x0002,   /* U+A7EE */
  0x0002,   /* U+A7EF */
  0x0002,   /* U+A7F0 */
  0x0002,   /* U+A7F1 */
  0x0002,   /* U+A7F2 */
  0x0002,   /* U+A7F3 */
  0x0002,   /* U+A7F4 */
  0x0002,   /* U+A7F5 */
  0x0002,   /* U+A7F6 */
  0x0002,   /* U+A7F7 */
  0x0014,   /* U+A7F8 */
  0x0014,   /* U+A7F9 */
  0x0002,   /* U+A7FA */
  0x0002,   /* U+A7FB */
  0x0002,   /* U+A7FC */
  0x0002,   /* U+A7FD */
  0x0002,   /* U+A7FE */
  0x0002,   /* U+A7FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x1F21,   /* U+A728 */
  0x1F21,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x1C47,   /* U+A732 */
  0x1C47,   /* U+A733 */
  0x1DDD,   /* U+A734 */
  0x1DDD,   /* U+A735 */
  0x1EB5,   /* U+A736 */
  0x1EB5,   /* U+A737 */
  0x1EE3,   /* U+A738 */
  0x1EE3,   /* U+A739 */
  0x0000,   /* U+A73A */
  0x0000,   /* U+A73B */
  0x1F0B,   /* U+A73C */
  0x1F0B,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x1DDD,   /* U+A74E */
  0x1DDD,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x1F0B,   /* U+A760 */
  0x1F0B,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0000,   /* U+A779 */
  0x0000,   /* U+A77A */
  0x0000,   /* U+A77B */
  0x0000,   /* U+A77C */
  0x0000,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0000,   /* U+A782 */
  0x0000,   /* U+A783 */
  0x0000,   /* U+A784 */
  0x0000,   /* U+A785 */
  0x0000,   /* U+A786 */
  0x0000,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x0000,   /* U+A79A */
  0x0000,   /* U+A79B */
  0x0000,   /* U+A79C */
  0x0000,   /* U+A79D */
  0x0000,   /* U+A79E */
  0x0000,   /* U+A79F */
  0x0000,   /* U+A7A0 */
  0x0000,   /* U+A7A1 */
  0x0000,   /* U+A7A2 */
  0x0000,   /* U+A7A3 */
  0x0000,   /* U+A7A4 */
  0x0000,   /* U+A7A5 */
  0x0000,   /* U+A7A6 */
  0x0000,   /* U+A7A7 */
  0x0000,   /* U+A7A8 */
  0x0000,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0xA7AF,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0xA7B8,   /* U+A7B8 */
  0xA7B9,   /* U+A7B9 */
  0xA7BA,   /* U+A7BA */
  0xA7BB,   /* U+A7BB */
  0xA7BC,   /* U+A7BC */
  0xA7BD,   /* U+A7BD */
  0xA7BE,   /* U+A7BE */
  0xA7BF,   /* U+A7BF */
  0xA7C0,   /* U+A7C0 */
  0xA7C1,   /* U+A7C1 */
  0xA7C2,   /* U+A7C2 */
  0xA7C3,   /* U+A7C3 */
  0xA7C4,   /* U+A7C4 */
  0xA7C5,   /* U+A7C5 */
  0xA7C6,   /* U+A7C6 */
  0xA7C7,   /* U+A7C7 */
  0xA7C8,   /* U+A7C8 */
  0xA7C9,   /* U+A7C9 */
  0xA7CA,   /* U+A7CA */
  0xA7CB,   /* U+A7CB */
  0xA7CC,   /* U+A7CC */
  0xA7CD,   /* U+A7CD */
  0xA7CE,   /* U+A7CE */
  0xA7CF,   /* U+A7CF */
  0xA7D0,   /* U+A7D0 */
  0xA7D1,   /* U+A7D1 */
  0xA7D2,   /* U+A7D2 */
  0xA7D3,   /* U+A7D3 */
  0xA7D4,   /* U+A7D4 */
  0xA7D5,   /* U+A7D5 */
  0xA7D6,   /* U+A7D6 */
  0xA7D7,   /* U+A7D7 */
  0xA7D8,   /* U+A7D8 */
  0xA7D9,   /* U+A7D9 */
  0xA7DA,   /* U+A7DA */
  0xA7DB,   /* U+A7DB */
  0xA7DC,   /* U+A7DC */
  0xA7DD,   /* U+A7DD */
  0xA7DE,   /* U+A7DE */
  0xA7DF,   /* U+A7DF */
  0xA7E0,   /* U+A7E0 */
  0xA7E1,   /* U+A7E1 */
  0xA7E2,   /* U+A7E2 */
  0xA7E3,   /* U+A7E3 */
  0xA7E4,   /* U+A7E4 */
  0xA7E5,   /* U+A7E5 */
  0xA7E6,   /* U+A7E6 */
  0xA7E7,   /* U+A7E7 */
  0xA7E8,   /* U+A7E8 */
  0xA7E9,   /* U+A7E9 */
  0xA7EA,   /* U+A7EA */
  0xA7EB,   /* U+A7EB */
  0xA7EC,   /* U+A7EC */
  0xA7ED,   /* U+A7ED */
  0xA7EE,   /* U+A7EE */
  0xA7EF,   /* U+A7EF */
  0xA7F0,   /* U+A7F0 */
  0xA7F1,   /* U+A7F1 */
  0xA7F2,   /* U+A7F2 */
  0xA7F3,   /* U+A7F3 */
  0xA7F4,   /* U+A7F4 */
  0xA7F5,   /* U+A7F5 */
  0xA7F6,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0000,   /* U+A7F8 */
  0x0000,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x0020,   /* U+A728 */
  0x0020,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x0020,   /* U+A732 */
  0x0020,   /* U+A733 */
  0x0020,   /* U+A734 */
  0x0020,   /* U+A735 */
  0x0020,   /* U+A736 */
  0x0020,   /* U+A737 */
  0x0020,   /* U+A738 */
  0x0020,   /* U+A739 */
  0x0110,   /* U+A73A */
  0x0110,   /* U+A73B */
  0x0020,   /* U+A73C */
  0x0020,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x0020,   /* U+A74E */
  0x0020,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x0020,   /* U+A760 */
  0x0020,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0111,   /* U+A779 */
  0x0111,   /* U+A77A */
  0x0111,   /* U+A77B */
  0x0111,   /* U+A77C */
  0x0111,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0111,   /* U+A782 */
  0x0111,   /* U+A783 */
  0x0111,   /* U+A784 */
  0x0111,   /* U+A785 */
  0x0111,   /* U+A786 */
  0x0111,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x002B,   /* U+A79A */
  0x002B,   /* U+A79B */
  0x002B,   /* U+A79C */
  0x002B,   /* U+A79D */
  0x002B,   /* U+A79E */
  0x002B,   /* U+A79F */
  0x0035,   /* U+A7A0 */
  0x0035,   /* U+A7A1 */
  0x0035,   /* U+A7A2 */
  0x0035,   /* U+A7A3 */
  0x0035,   /* U+A7A4 */
  0x0035,   /* U+A7A5 */
  0x0035,   /* U+A7A6 */
  0x0035,   /* U+A7A7 */
  0x0035,   /* U+A7A8 */
  0x0035,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0x0000,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0x0000,   /* U+A7B8 */
  0x0000,   /* U+A7B9 */
  0x0000,   /* U+A7BA */
  0x0000,   /* U+A7BB */
  0x0000,   /* U+A7BC */
  0x0000,   /* U+A7BD */
  0x0000,   /* U+A7BE */
  0x0000,   /* U+A7BF */
  0x0000,   /* U+A7C0 */
  0x0000,   /* U+A7C1 */
  0x0000,   /* U+A7C2 */
  0x0000,   /* U+A7C3 */
  0x0000,   /* U+A7C4 */
  0x0000,   /* U+A7C5 */
  0x0000,   /* U+A7C6 */
  0x0000,   /* U+A7C7 */
  0x0000,   /* U+A7C8 */
  0x0000,   /* U+A7C9 */
  0x0000,   /* U+A7CA */
  0x0000,   /* U+A7CB */
  0x0000,   /* U+A7CC */
  0x0000,   /* U+A7CD */
  0x0000,   /* U+A7CE */
  0x0000,   /* U+A7CF */
  0x0000,   /* U+A7D0 */
  0x0000,   /* U+A7D1 */
  0x0000,   /* U+A7D2 */
  0x0000,   /* U+A7D3 */
  0x0000,   /* U+A7D4 */
  0x0000,   /* U+A7D5 */
  0x0000,   /* U+A7D6 */
  0x0000,   /* U+A7D7 */
  0x0000,   /* U+A7D8 */
  0x0000,   /* U+A7D9 */
  0x0000,   /* U+A7DA */
  0x0000,   /* U+A7DB */
  0x0000,   /* U+A7DC */
  0x0000,   /* U+A7DD */
  0x0000,   /* U+A7DE */
  0x0000,   /* U+A7DF */
  0x0000,   /* U+A7E0 */
  0x0000,   /* U+A7E1 */
  0x0000,   /* U+A7E2 */
  0x0000,   /* U+A7E3 */
  0x0000,   /* U+A7E4 */
  0x0000,   /* U+A7E5 */
  0x0000,   /* U+A7E6 */
  0x0000,   /* U+A7E7 */
  0x0000,   /* U+A7E8 */
  0x0000,   /* U+A7E9 */
  0x0000,   /* U+A7EA */
  0x0000,   /* U+A7EB */
  0x0000,   /* U+A7EC */
  0x0000,   /* U+A7ED */
  0x0000,   /* U+A7EE */
  0x0000,   /* U+A7EF */
  0x0000,   /* U+A7F0 */
  0x0000,   /* U+A7F1 */
  0x0000,   /* U+A7F2 */
  0x0000,   /* U+A7F3 */
  0x0000,   /* U+A7F4 */
  0x0000,   /* U+A7F5 */
  0x0000,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0039,   /* U+A7F8 */
  0x0110,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x0004,   /* U+A728 */
  0x0004,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x000A,   /* U+A732 */
  0x0004,   /* U+A733 */
  0x000A,   /* U+A734 */
  0x0004,   /* U+A735 */
  0x000A,   /* U+A736 */
  0x0004,   /* U+A737 */
  0x000A,   /* U+A738 */
  0x0004,   /* U+A739 */
  0x0004,   /* U+A73A */
  0x0004,   /* U+A73B */
  0x000A,   /* U+A73C */
  0x0004,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x000A,   /* U+A74E */
  0x0004,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x000A,   /* U+A760 */
  0x0004,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0004,   /* U+A779 */
  0x0004,   /* U+A77A */
  0x0004,   /* U+A77B */
  0x0004,   /* U+A77C */
  0x0004,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0004,   /* U+A782 */
  0x0004,   /* U+A783 */
  0x0004,   /* U+A784 */
  0x0004,   /* U+A785 */
  0x0004,   /* U+A786 */
  0x0004,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x0004,   /* U+A79A */
  0x0004,   /* U+A79B */
  0x0004,   /* U+A79C */
  0x0004,   /* U+A79D */
  0x0004,   /* U+A79E */
  0x0004,   /* U+A79F */
  0x0004,   /* U+A7A0 */
  0x0004,   /* U+A7A1 */
  0x0004,   /* U+A7A2 */
  0x0004,   /* U+A7A3 */
  0x0004,   /* U+A7A4 */
  0x0004,   /* U+A7A5 */
  0x0004,   /* U+A7A6 */
  0x0004,   /* U+A7A7 */
  0x0004,   /* U+A7A8 */
  0x0004,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0x0000,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0x0000,   /* U+A7B8 */
  0x0000,   /* U+A7B9 */
  0x0000,   /* U+A7BA */
  0x0000,   /* U+A7BB */
  0x0000,   /* U+A7BC */
  0x0000,   /* U+A7BD */
  0x0000,   /* U+A7BE */
  0x0000,   /* U+A7BF */
  0x0000,   /* U+A7C0 */
  0x0000,   /* U+A7C1 */
  0x0000,   /* U+A7C2 */
  0x0000,   /* U+A7C3 */
  0x0000,   /* U+A7C4 */
  0x0000,   /* U+A7C5 */
  0x0000,   /* U+A7C6 */
  0x0000,   /* U+A7C7 */
  0x0000,   /* U+A7C8 */
  0x0000,   /* U+A7C9 */
  0x0000,   /* U+A7CA */
  0x0000,   /* U+A7CB */
  0x0000,   /* U+A7CC */
  0x0000,   /* U+A7CD */
  0x0000,   /* U+A7CE */
  0x0000,   /* U+A7CF */
  0x0000,   /* U+A7D0 */
  0x0000,   /* U+A7D1 */
  0x0000,   /* U+A7D2 */
  0x0000,   /* U+A7D3 */
  0x0000,   /* U+A7D4 */
  0x0000,   /* U+A7D5 */
  0x0000,   /* U+A7D6 */
  0x0000,   /* U+A7D7 */
  0x0000,   /* U+A7D8 */
  0x0000,   /* U+A7D9 */
  0x0000,   /* U+A7DA */
  0x0000,   /* U+A7DB */
  0x0000,   /* U+A7DC */
  0x0000,   /* U+A7DD */
  0x0000,   /* U+A7DE */
  0x0000,   /* U+A7DF */
  0x0000,   /* U+A7E0 */
  0x0000,   /* U+A7E1 */
  0x0000,   /* U+A7E2 */
  0x0000,   /* U+A7E3 */
  0x0000,   /* U+A7E4 */
  0x0000,   /* U+A7E5 */
  0x0000,   /* U+A7E6 */
  0x0000,   /* U+A7E7 */
  0x0000,   /* U+A7E8 */
  0x0000,   /* U+A7E9 */
  0x0000,   /* U+A7EA */
  0x0000,   /* U+A7EB */
  0x0000,   /* U+A7EC */
  0x0000,   /* U+A7ED */
  0x0000,   /* U+A7EE */
  0x0000,   /* U+A7EF */
  0x0000,   /* U+A7F0 */
  0x0000,   /* U+A7F1 */
  0x0000,   /* U+A7F2 */
  0x0000,   /* U+A7F3 */
  0x0000,   /* U+A7F4 */
  0x0000,   /* U+A7F5 */
  0x0000,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0014,   /* U+A7F8 */
  0x0014,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x0000,   /* U+A728 */
  0x0000,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x0000,   /* U+A732 */
  0x0000,   /* U+A733 */
  0x0000,   /* U+A734 */
  0x0000,   /* U+A735 */
  0x0000,   /* U+A736 */
  0x0000,   /* U+A737 */
  0x0000,   /* U+A738 */
  0x0000,   /* U+A739 */
  0x1EE3,   /* U+A73A */
  0x1EE3,   /* U+A73B */
  0x0000,   /* U+A73C */
  0x0000,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x0000,   /* U+A74E */
  0x0000,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x0000,   /* U+A760 */
  0x0000,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0000,   /* U+A779 */
  0x0000,   /* U+A77A */
  0x0000,   /* U+A77B */
  0x0000,   /* U+A77C */
  0x0000,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0000,   /* U+A782 */
  0x0000,   /* U+A783 */
  0x0000,   /* U+A784 */
  0x0000,   /* U+A785 */
  0x0000,   /* U+A786 */
  0x0000,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x0000,   /* U+A79A */
  0x0000,   /* U+A79B */
  0x0000,   /* U+A79C */
  0x0000,   /* U+A79D */
  0x0000,   /* U+A79E */
  0x0000,   /* U+A79F */
  0x0000,   /* U+A7A0 */
  0x0000,   /* U+A7A1 */
  0x0000,   /* U+A7A2 */
  0x0000,   /* U+A7A3 */
  0x0000,   /* U+A7A4 */
  0x0000,   /* U+A7A5 */
  0x0000,   /* U+A7A6 */
  0x0000,   /* U+A7A7 */
  0x0000,   /* U+A7A8 */
  0x0000,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0x0000,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0x0000,   /* U+A7B8 */
  0x0000,   /* U+A7B9 */
  0x0000,   /* U+A7BA */
  0x0000,   /* U+A7BB */
  0x0000,   /* U+A7BC */
  0x0000,   /* U+A7BD */
  0x0000,   /* U+A7BE */
  0x0000,   /* U+A7BF */
  0x0000,   /* U+A7C0 */
  0x0000,   /* U+A7C1 */
  0x0000,   /* U+A7C2 */
  0x0000,   /* U+A7C3 */
  0x0000,   /* U+A7C4 */
  0x0000,   /* U+A7C5 */
  0x0000,   /* U+A7C6 */
  0x0000,   /* U+A7C7 */
  0x0000,   /* U+A7C8 */
  0x0000,   /* U+A7C9 */
  0x0000,   /* U+A7CA */
  0x0000,   /* U+A7CB */
  0x0000,   /* U+A7CC */
  0x0000,   /* U+A7CD */
  0x0000,   /* U+A7CE */
  0x0000,   /* U+A7CF */
  0x0000,   /* U+A7D0 */
  0x0000,   /* U+A7D1 */
  0x0000,   /* U+A7D2 */
  0x0000,   /* U+A7D3 */
  0x0000,   /* U+A7D4 */
  0x0000,   /* U+A7D5 */
  0x0000,   /* U+A7D6 */
  0x0000,   /* U+A7D7 */
  0x0000,   /* U+A7D8 */
  0x0000,   /* U+A7D9 */
  0x0000,   /* U+A7DA */
  0x0000,   /* U+A7DB */
  0x0000,   /* U+A7DC */
  0x0000,   /* U+A7DD */
  0x0000,   /* U+A7DE */
  0x0000,   /* U+A7DF */
  0x0000,   /* U+A7E0 */
  0x0000,   /* U+A7E1 */
  0x0000,   /* U+A7E2 */
  0x0000,   /* U+A7E3 */
  0x0000,   /* U+A7E4 */
  0x0000,   /* U+A7E5 */
  0x0000,   /* U+A7E6 */
  0x0000,   /* U+A7E7 */
  0x0000,   /* U+A7E8 */
  0x0000,   /* U+A7E9 */
  0x0000,   /* U+A7EA */
  0x0000,   /* U+A7EB */
  0x0000,   /* U+A7EC */
  0x0000,   /* U+A7ED */
  0x0000,   /* U+A7EE */
  0x0000,   /* U+A7EF */
  0x0000,   /* U+A7F0 */
  0x0000,   /* U+A7F1 */
  0x0000,   /* U+A7F2 */
  0x0000,   /* U+A7F3 */
  0x0000,   /* U+A7F4 */
  0x0000,   /* U+A7F5 */
  0x0000,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0000,   /* U+A7F8 */
  0x1CAA,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x0000,   /* U+A728 */
  0x0000,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x0000,   /* U+A732 */
  0x0000,   /* U+A733 */
  0x0000,   /* U+A734 */
  0x0000,   /* U+A735 */
  0x0000,   /* U+A736 */
  0x0000,   /* U+A737 */
  0x0000,   /* U+A738 */
  0x0000,   /* U+A739 */
  0x0020,   /* U+A73A */
  0x0020,   /* U+A73B */
  0x0000,   /* U+A73C */
  0x0000,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x0000,   /* U+A74E */
  0x0000,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x0000,   /* U+A760 */
  0x0000,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0000,   /* U+A779 */
  0x0000,   /* U+A77A */
  0x0000,   /* U+A77B */
  0x0000,   /* U+A77C */
  0x0000,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0000,   /* U+A782 */
  0x0000,   /* U+A783 */
  0x0000,   /* U+A784 */
  0x0000,   /* U+A785 */
  0x0000,   /* U+A786 */
  0x0000,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x0000,   /* U+A79A */
  0x0000,   /* U+A79B */
  0x0000,   /* U+A79C */
  0x0000,   /* U+A79D */
  0x0000,   /* U+A79E */
  0x0000,   /* U+A79F */
  0x0000,   /* U+A7A0 */
  0x0000,   /* U+A7A1 */
  0x0000,   /* U+A7A2 */
  0x0000,   /* U+A7A3 */
  0x0000,   /* U+A7A4 */
  0x0000,   /* U+A7A5 */
  0x0000,   /* U+A7A6 */
  0x0000,   /* U+A7A7 */
  0x0000,   /* U+A7A8 */
  0x0000,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0x0000,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0x0000,   /* U+A7B8 */
  0x0000,   /* U+A7B9 */
  0x0000,   /* U+A7BA */
  0x0000,   /* U+A7BB */
  0x0000,   /* U+A7BC */
  0x0000,   /* U+A7BD */
  0x0000,   /* U+A7BE */
  0x0000,   /* U+A7BF */
  0x0000,   /* U+A7C0 */
  0x0000,   /* U+A7C1 */
  0x0000,   /* U+A7C2 */
  0x0000,   /* U+A7C3 */
  0x0000,   /* U+A7C4 */
  0x0000,   /* U+A7C5 */
  0x0000,   /* U+A7C6 */
  0x0000,   /* U+A7C7 */
  0x0000,   /* U+A7C8 */
  0x0000,   /* U+A7C9 */
  0x0000,   /* U+A7CA */
  0x0000,   /* U+A7CB */
  0x0000,   /* U+A7CC */
  0x0000,   /* U+A7CD */
  0x0000,   /* U+A7CE */
  0x0000,   /* U+A7CF */
  0x0000,   /* U+A7D0 */
  0x0000,   /* U+A7D1 */
  0x0000,   /* U+A7D2 */
  0x0000,   /* U+A7D3 */
  0x0000,   /* U+A7D4 */
  0x0000,   /* U+A7D5 */
  0x0000,   /* U+A7D6 */
  0x0000,   /* U+A7D7 */
  0x0000,   /* U+A7D8 */
  0x0000,   /* U+A7D9 */
  0x0000,   /* U+A7DA */
  0x0000,   /* U+A7DB */
  0x0000,   /* U+A7DC */
  0x0000,   /* U+A7DD */
  0x0000,   /* U+A7DE */
  0x0000,   /* U+A7DF */
  0x0000,   /* U+A7E0 */
  0x0000,   /* U+A7E1 */
  0x0000,   /* U+A7E2 */
  0x0000,   /* U+A7E3 */
  0x0000,   /* U+A7E4 */
  0x0000,   /* U+A7E5 */
  0x0000,   /* U+A7E6 */
  0x0000,   /* U+A7E7 */
  0x0000,   /* U+A7E8 */
  0x0000,   /* U+A7E9 */
  0x0000,   /* U+A7EA */
  0x0000,   /* U+A7EB */
  0x0000,   /* U+A7EC */
  0x0000,   /* U+A7ED */
  0x0000,   /* U+A7EE */
  0x0000,   /* U+A7EF */
  0x0000,   /* U+A7F0 */
  0x0000,   /* U+A7F1 */
  0x0000,   /* U+A7F2 */
  0x0000,   /* U+A7F3 */
  0x0000,   /* U+A7F4 */
  0x0000,   /* U+A7F5 */
  0x0000,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0000,   /* U+A7F8 */
  0x0020,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+A700 */
  0x0000,   /* U+A701 */
  0x0000,   /* U+A702 */
  0x0000,   /* U+A703 */
  0x0000,   /* U+A704 */
  0x0000,   /* U+A705 */
  0x0000,   /* U+A706 */
  0x0000,   /* U+A707 */
  0x0000,   /* U+A708 */
  0x0000,   /* U+A709 */
  0x0000,   /* U+A70A */
  0x0000,   /* U+A70B */
  0x0000,   /* U+A70C */
  0x0000,   /* U+A70D */
  0x0000,   /* U+A70E */
  0x0000,   /* U+A70F */
  0x0000,   /* U+A710 */
  0x0000,   /* U+A711 */
  0x0000,   /* U+A712 */
  0x0000,   /* U+A713 */
  0x0000,   /* U+A714 */
  0x0000,   /* U+A715 */
  0x0000,   /* U+A716 */
  0x0000,   /* U+A717 */
  0x0000,   /* U+A718 */
  0x0000,   /* U+A719 */
  0x0000,   /* U+A71A */
  0x0000,   /* U+A71B */
  0x0000,   /* U+A71C */
  0x0000,   /* U+A71D */
  0x0000,   /* U+A71E */
  0x0000,   /* U+A71F */
  0x0000,   /* U+A720 */
  0x0000,   /* U+A721 */
  0x0000,   /* U+A722 */
  0x0000,   /* U+A723 */
  0x0000,   /* U+A724 */
  0x0000,   /* U+A725 */
  0x0000,   /* U+A726 */
  0x0000,   /* U+A727 */
  0x0000,   /* U+A728 */
  0x0000,   /* U+A729 */
  0x0000,   /* U+A72A */
  0x0000,   /* U+A72B */
  0x0000,   /* U+A72C */
  0x0000,   /* U+A72D */
  0x0000,   /* U+A72E */
  0x0000,   /* U+A72F */
  0x0000,   /* U+A730 */
  0x0000,   /* U+A731 */
  0x0000,   /* U+A732 */
  0x0000,   /* U+A733 */
  0x0000,   /* U+A734 */
  0x0000,   /* U+A735 */
  0x0000,   /* U+A736 */
  0x0000,   /* U+A737 */
  0x0000,   /* U+A738 */
  0x0000,   /* U+A739 */
  0x000A,   /* U+A73A */
  0x0004,   /* U+A73B */
  0x0000,   /* U+A73C */
  0x0000,   /* U+A73D */
  0x0000,   /* U+A73E */
  0x0000,   /* U+A73F */
  0x0000,   /* U+A740 */
  0x0000,   /* U+A741 */
  0x0000,   /* U+A742 */
  0x0000,   /* U+A743 */
  0x0000,   /* U+A744 */
  0x0000,   /* U+A745 */
  0x0000,   /* U+A746 */
  0x0000,   /* U+A747 */
  0x0000,   /* U+A748 */
  0x0000,   /* U+A749 */
  0x0000,   /* U+A74A */
  0x0000,   /* U+A74B */
  0x0000,   /* U+A74C */
  0x0000,   /* U+A74D */
  0x0000,   /* U+A74E */
  0x0000,   /* U+A74F */
  0x0000,   /* U+A750 */
  0x0000,   /* U+A751 */
  0x0000,   /* U+A752 */
  0x0000,   /* U+A753 */
  0x0000,   /* U+A754 */
  0x0000,   /* U+A755 */
  0x0000,   /* U+A756 */
  0x0000,   /* U+A757 */
  0x0000,   /* U+A758 */
  0x0000,   /* U+A759 */
  0x0000,   /* U+A75A */
  0x0000,   /* U+A75B */
  0x0000,   /* U+A75C */
  0x0000,   /* U+A75D */
  0x0000,   /* U+A75E */
  0x0000,   /* U+A75F */
  0x0000,   /* U+A760 */
  0x0000,   /* U+A761 */
  0x0000,   /* U+A762 */
  0x0000,   /* U+A763 */
  0x0000,   /* U+A764 */
  0x0000,   /* U+A765 */
  0x0000,   /* U+A766 */
  0x0000,   /* U+A767 */
  0x0000,   /* U+A768 */
  0x0000,   /* U+A769 */
  0x0000,   /* U+A76A */
  0x0000,   /* U+A76B */
  0x0000,   /* U+A76C */
  0x0000,   /* U+A76D */
  0x0000,   /* U+A76E */
  0x0000,   /* U+A76F */
  0x0000,   /* U+A770 */
  0x0000,   /* U+A771 */
  0x0000,   /* U+A772 */
  0x0000,   /* U+A773 */
  0x0000,   /* U+A774 */
  0x0000,   /* U+A775 */
  0x0000,   /* U+A776 */
  0x0000,   /* U+A777 */
  0x0000,   /* U+A778 */
  0x0000,   /* U+A779 */
  0x0000,   /* U+A77A */
  0x0000,   /* U+A77B */
  0x0000,   /* U+A77C */
  0x0000,   /* U+A77D */
  0x0000,   /* U+A77E */
  0x0000,   /* U+A77F */
  0x0000,   /* U+A780 */
  0x0000,   /* U+A781 */
  0x0000,   /* U+A782 */
  0x0000,   /* U+A783 */
  0x0000,   /* U+A784 */
  0x0000,   /* U+A785 */
  0x0000,   /* U+A786 */
  0x0000,   /* U+A787 */
  0x0000,   /* U+A788 */
  0x0000,   /* U+A789 */
  0x0000,   /* U+A78A */
  0x0000,   /* U+A78B */
  0x0000,   /* U+A78C */
  0x0000,   /* U+A78D */
  0x0000,   /* U+A78E */
  0x0000,   /* U+A78F */
  0x0000,   /* U+A790 */
  0x0000,   /* U+A791 */
  0x0000,   /* U+A792 */
  0x0000,   /* U+A793 */
  0x0000,   /* U+A794 */
  0x0000,   /* U+A795 */
  0x0000,   /* U+A796 */
  0x0000,   /* U+A797 */
  0x0000,   /* U+A798 */
  0x0000,   /* U+A799 */
  0x0000,   /* U+A79A */
  0x0000,   /* U+A79B */
  0x0000,   /* U+A79C */
  0x0000,   /* U+A79D */
  0x0000,   /* U+A79E */
  0x0000,   /* U+A79F */
  0x0000,   /* U+A7A0 */
  0x0000,   /* U+A7A1 */
  0x0000,   /* U+A7A2 */
  0x0000,   /* U+A7A3 */
  0x0000,   /* U+A7A4 */
  0x0000,   /* U+A7A5 */
  0x0000,   /* U+A7A6 */
  0x0000,   /* U+A7A7 */
  0x0000,   /* U+A7A8 */
  0x0000,   /* U+A7A9 */
  0x0000,   /* U+A7AA */
  0x0000,   /* U+A7AB */
  0x0000,   /* U+A7AC */
  0x0000,   /* U+A7AD */
  0x0000,   /* U+A7AE */
  0x0000,   /* U+A7AF */
  0x0000,   /* U+A7B0 */
  0x0000,   /* U+A7B1 */
  0x0000,   /* U+A7B2 */
  0x0000,   /* U+A7B3 */
  0x0000,   /* U+A7B4 */
  0x0000,   /* U+A7B5 */
  0x0000,   /* U+A7B6 */
  0x0000,   /* U+A7B7 */
  0x0000,   /* U+A7B8 */
  0x0000,   /* U+A7B9 */
  0x0000,   /* U+A7BA */
  0x0000,   /* U+A7BB */
  0x0000,   /* U+A7BC */
  0x0000,   /* U+A7BD */
  0x0000,   /* U+A7BE */
  0x0000,   /* U+A7BF */
  0x0000,   /* U+A7C0 */
  0x0000,   /* U+A7C1 */
  0x0000,   /* U+A7C2 */
  0x0000,   /* U+A7C3 */
  0x0000,   /* U+A7C4 */
  0x0000,   /* U+A7C5 */
  0x0000,   /* U+A7C6 */
  0x0000,   /* U+A7C7 */
  0x0000,   /* U+A7C8 */
  0x0000,   /* U+A7C9 */
  0x0000,   /* U+A7CA */
  0x0000,   /* U+A7CB */
  0x0000,   /* U+A7CC */
  0x0000,   /* U+A7CD */
  0x0000,   /* U+A7CE */
  0x0000,   /* U+A7CF */
  0x0000,   /* U+A7D0 */
  0x0000,   /* U+A7D1 */
  0x0000,   /* U+A7D2 */
  0x0000,   /* U+A7D3 */
  0x0000,   /* U+A7D4 */
  0x0000,   /* U+A7D5 */
  0x0000,   /* U+A7D6 */
  0x0000,   /* U+A7D7 */
  0x0000,   /* U+A7D8 */
  0x0000,   /* U+A7D9 */
  0x0000,   /* U+A7DA */
  0x0000,   /* U+A7DB */
  0x0000,   /* U+A7DC */
  0x0000,   /* U+A7DD */
  0x0000,   /* U+A7DE */
  0x0000,   /* U+A7DF */
  0x0000,   /* U+A7E0 */
  0x0000,   /* U+A7E1 */
  0x0000,   /* U+A7E2 */
  0x0000,   /* U+A7E3 */
  0x0000,   /* U+A7E4 */
  0x0000,   /* U+A7E5 */
  0x0000,   /* U+A7E6 */
  0x0000,   /* U+A7E7 */
  0x0000,   /* U+A7E8 */
  0x0000,   /* U+A7E9 */
  0x0000,   /* U+A7EA */
  0x0000,   /* U+A7EB */
  0x0000,   /* U+A7EC */
  0x0000,   /* U+A7ED */
  0x0000,   /* U+A7EE */
  0x0000,   /* U+A7EF */
  0x0000,   /* U+A7F0 */
  0x0000,   /* U+A7F1 */
  0x0000,   /* U+A7F2 */
  0x0000,   /* U+A7F3 */
  0x0000,   /* U+A7F4 */
  0x0000,   /* U+A7F5 */
  0x0000,   /* U+A7F6 */
  0x0000,   /* U+A7F7 */
  0x0000,   /* U+A7F8 */
  0x0014,   /* U+A7F9 */
  0x0000,   /* U+A7FA */
  0x0000,   /* U+A7FB */
  0x0000,   /* U+A7FC */
  0x0000,   /* U+A7FD */
  0x0000,   /* U+A7FE */
  0x0000,   /* U+A7FF */
};

uint16 uca900_p0A8[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0x295C,   /* U+A800 */
  0x295D,   /* U+A801 */
  0x295E,   /* U+A802 */
  0x295F,   /* U+A803 */
  0x2960,   /* U+A804 */
  0x2961,   /* U+A805 */
  0x2962,   /* U+A806 */
  0x2963,   /* U+A807 */
  0x2964,   /* U+A808 */
  0x2965,   /* U+A809 */
  0x2966,   /* U+A80A */
  0x0000,   /* U+A80B */
  0x2967,   /* U+A80C */
  0x2968,   /* U+A80D */
  0x2969,   /* U+A80E */
  0x296A,   /* U+A80F */
  0x296B,   /* U+A810 */
  0x296C,   /* U+A811 */
  0x296D,   /* U+A812 */
  0x296E,   /* U+A813 */
  0x296F,   /* U+A814 */
  0x2970,   /* U+A815 */
  0x2971,   /* U+A816 */
  0x2972,   /* U+A817 */
  0x2973,   /* U+A818 */
  0x2974,   /* U+A819 */
  0x2975,   /* U+A81A */
  0x2976,   /* U+A81B */
  0x2977,   /* U+A81C */
  0x2978,   /* U+A81D */
  0x2979,   /* U+A81E */
  0x297A,   /* U+A81F */
  0x297B,   /* U+A820 */
  0x297C,   /* U+A821 */
  0x297D,   /* U+A822 */
  0x297E,   /* U+A823 */
  0x297F,   /* U+A824 */
  0x2980,   /* U+A825 */
  0x2981,   /* U+A826 */
  0x2982,   /* U+A827 */
  0x051F,   /* U+A828 */
  0x0520,   /* U+A829 */
  0x0521,   /* U+A82A */
  0x0522,   /* U+A82B */
  0xFBC1,   /* U+A82C */
  0xFBC1,   /* U+A82D */
  0xFBC1,   /* U+A82E */
  0xFBC1,   /* U+A82F */
  0x1AA2,   /* U+A830 */
  0x1AA3,   /* U+A831 */
  0x1AA4,   /* U+A832 */
  0x1AA5,   /* U+A833 */
  0x1AA6,   /* U+A834 */
  0x1AA7,   /* U+A835 */
  0x0523,   /* U+A836 */
  0x0524,   /* U+A837 */
  0x1C1B,   /* U+A838 */
  0x0525,   /* U+A839 */
  0xFBC1,   /* U+A83A */
  0xFBC1,   /* U+A83B */
  0xFBC1,   /* U+A83C */
  0xFBC1,   /* U+A83D */
  0xFBC1,   /* U+A83E */
  0xFBC1,   /* U+A83F */
  0x2F00,   /* U+A840 */
  0x2F01,   /* U+A841 */
  0x2F02,   /* U+A842 */
  0x2F03,   /* U+A843 */
  0x2F04,   /* U+A844 */
  0x2F05,   /* U+A845 */
  0x2F06,   /* U+A846 */
  0x2F07,   /* U+A847 */
  0x2F0C,   /* U+A848 */
  0x2F0D,   /* U+A849 */
  0x2F0E,   /* U+A84A */
  0x2F0F,   /* U+A84B */
  0x2F10,   /* U+A84C */
  0x2F11,   /* U+A84D */
  0x2F12,   /* U+A84E */
  0x2F13,   /* U+A84F */
  0x2F14,   /* U+A850 */
  0x2F15,   /* U+A851 */
  0x2F16,   /* U+A852 */
  0x2F17,   /* U+A853 */
  0x2F19,   /* U+A854 */
  0x2F1A,   /* U+A855 */
  0x2F1B,   /* U+A856 */
  0x2F1C,   /* U+A857 */
  0x2F1F,   /* U+A858 */
  0x2F22,   /* U+A859 */
  0x2F23,   /* U+A85A */
  0x2F25,   /* U+A85B */
  0x2F26,   /* U+A85C */
  0x2F29,   /* U+A85D */
  0x2F2E,   /* U+A85E */
  0x2F2F,   /* U+A85F */
  0x2F30,   /* U+A860 */
  0x2F31,   /* U+A861 */
  0x2F2A,   /* U+A862 */
  0x2F2B,   /* U+A863 */
  0x2F2C,   /* U+A864 */
  0x2F2D,   /* U+A865 */
  0x2F32,   /* U+A866 */
  0x2F18,   /* U+A867 */
  0x2F1D,   /* U+A868 */
  0x2F08,   /* U+A869 */
  0x2F09,   /* U+A86A */
  0x2F0A,   /* U+A86B */
  0x2F0B,   /* U+A86C */
  0x2F1E,   /* U+A86D */
  0x2F24,   /* U+A86E */
  0x2F27,   /* U+A86F */
  0x2F28,   /* U+A870 */
  0x2F20,   /* U+A871 */
  0x2F21,   /* U+A872 */
  0x2F33,   /* U+A873 */
  0x0428,   /* U+A874 */
  0x0429,   /* U+A875 */
  0x0294,   /* U+A876 */
  0x0295,   /* U+A877 */
  0xFBC1,   /* U+A878 */
  0xFBC1,   /* U+A879 */
  0xFBC1,   /* U+A87A */
  0xFBC1,   /* U+A87B */
  0xFBC1,   /* U+A87C */
  0xFBC1,   /* U+A87D */
  0xFBC1,   /* U+A87E */
  0xFBC1,   /* U+A87F */
  0x0000,   /* U+A880 */
  0x0000,   /* U+A881 */
  0x2983,   /* U+A882 */
  0x2984,   /* U+A883 */
  0x2985,   /* U+A884 */
  0x2986,   /* U+A885 */
  0x2987,   /* U+A886 */
  0x2988,   /* U+A887 */
  0x2989,   /* U+A888 */
  0x298A,   /* U+A889 */
  0x298B,   /* U+A88A */
  0x298C,   /* U+A88B */
  0x298D,   /* U+A88C */
  0x298E,   /* U+A88D */
  0x298F,   /* U+A88E */
  0x2990,   /* U+A88F */
  0x2991,   /* U+A890 */
  0x2992,   /* U+A891 */
  0x2993,   /* U+A892 */
  0x2994,   /* U+A893 */
  0x2995,   /* U+A894 */
  0x2996,   /* U+A895 */
  0x2997,   /* U+A896 */
  0x2998,   /* U+A897 */
  0x2999,   /* U+A898 */
  0x299A,   /* U+A899 */
  0x299B,   /* U+A89A */
  0x299C,   /* U+A89B */
  0x299D,   /* U+A89C */
  0x299E,   /* U+A89D */
  0x299F,   /* U+A89E */
  0x29A0,   /* U+A89F */
  0x29A1,   /* U+A8A0 */
  0x29A2,   /* U+A8A1 */
  0x29A3,   /* U+A8A2 */
  0x29A4,   /* U+A8A3 */
  0x29A5,   /* U+A8A4 */
  0x29A6,   /* U+A8A5 */
  0x29A7,   /* U+A8A6 */
  0x29A8,   /* U+A8A7 */
  0x29A9,   /* U+A8A8 */
  0x29AA,   /* U+A8A9 */
  0x29AB,   /* U+A8AA */
  0x29AC,   /* U+A8AB */
  0x29AD,   /* U+A8AC */
  0x29AE,   /* U+A8AD */
  0x29AF,   /* U+A8AE */
  0x29B0,   /* U+A8AF */
  0x29B1,   /* U+A8B0 */
  0x29B2,   /* U+A8B1 */
  0x29B3,   /* U+A8B2 */
  0x29B4,   /* U+A8B3 */
  0x29B5,   /* U+A8B4 */
  0x29B6,   /* U+A8B5 */
  0x29B7,   /* U+A8B6 */
  0x29B8,   /* U+A8B7 */
  0x29B9,   /* U+A8B8 */
  0x29BA,   /* U+A8B9 */
  0x29BB,   /* U+A8BA */
  0x29BC,   /* U+A8BB */
  0x29BD,   /* U+A8BC */
  0x29BE,   /* U+A8BD */
  0x29BF,   /* U+A8BE */
  0x29C0,   /* U+A8BF */
  0x29C1,   /* U+A8C0 */
  0x29C2,   /* U+A8C1 */
  0x29C3,   /* U+A8C2 */
  0x29C4,   /* U+A8C3 */
  0x29C5,   /* U+A8C4 */
  0x0000,   /* U+A8C5 */
  0xFBC1,   /* U+A8C6 */
  0xFBC1,   /* U+A8C7 */
  0xFBC1,   /* U+A8C8 */
  0xFBC1,   /* U+A8C9 */
  0xFBC1,   /* U+A8CA */
  0xFBC1,   /* U+A8CB */
  0xFBC1,   /* U+A8CC */
  0xFBC1,   /* U+A8CD */
  0x0290,   /* U+A8CE */
  0x0291,   /* U+A8CF */
  0x1C3D,   /* U+A8D0 */
  0x1C3E,   /* U+A8D1 */
  0x1C3F,   /* U+A8D2 */
  0x1C40,   /* U+A8D3 */
  0x1C41,   /* U+A8D4 */
  0x1C42,   /* U+A8D5 */
  0x1C43,   /* U+A8D6 */
  0x1C44,   /* U+A8D7 */
  0x1C45,   /* U+A8D8 */
  0x1C46,   /* U+A8D9 */
  0xFBC1,   /* U+A8DA */
  0xFBC1,   /* U+A8DB */
  0xFBC1,   /* U+A8DC */
  0xFBC1,   /* U+A8DD */
  0xFBC1,   /* U+A8DE */
  0xFBC1,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x26A4,   /* U+A8F2 */
  0x26A4,   /* U+A8F3 */
  0x26A4,   /* U+A8F4 */
  0x26A4,   /* U+A8F5 */
  0x26A4,   /* U+A8F6 */
  0x26A4,   /* U+A8F7 */
  0x03E8,   /* U+A8F8 */
  0x03E9,   /* U+A8F9 */
  0x03EA,   /* U+A8FA */
  0x26A5,   /* U+A8FB */
  0x03EB,   /* U+A8FC */
  0x265C,   /* U+A8FD */
  0xFBC1,   /* U+A8FE */
  0xFBC1,   /* U+A8FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A800 */
  0x0020,   /* U+A801 */
  0x0020,   /* U+A802 */
  0x0020,   /* U+A803 */
  0x0020,   /* U+A804 */
  0x0020,   /* U+A805 */
  0x0020,   /* U+A806 */
  0x0020,   /* U+A807 */
  0x0020,   /* U+A808 */
  0x0020,   /* U+A809 */
  0x0020,   /* U+A80A */
  0x00C4,   /* U+A80B */
  0x0020,   /* U+A80C */
  0x0020,   /* U+A80D */
  0x0020,   /* U+A80E */
  0x0020,   /* U+A80F */
  0x0020,   /* U+A810 */
  0x0020,   /* U+A811 */
  0x0020,   /* U+A812 */
  0x0020,   /* U+A813 */
  0x0020,   /* U+A814 */
  0x0020,   /* U+A815 */
  0x0020,   /* U+A816 */
  0x0020,   /* U+A817 */
  0x0020,   /* U+A818 */
  0x0020,   /* U+A819 */
  0x0020,   /* U+A81A */
  0x0020,   /* U+A81B */
  0x0020,   /* U+A81C */
  0x0020,   /* U+A81D */
  0x0020,   /* U+A81E */
  0x0020,   /* U+A81F */
  0x0020,   /* U+A820 */
  0x0020,   /* U+A821 */
  0x0020,   /* U+A822 */
  0x0020,   /* U+A823 */
  0x0020,   /* U+A824 */
  0x0020,   /* U+A825 */
  0x0020,   /* U+A826 */
  0x0020,   /* U+A827 */
  0x0020,   /* U+A828 */
  0x0020,   /* U+A829 */
  0x0020,   /* U+A82A */
  0x0020,   /* U+A82B */
  0x0020,   /* U+A82C */
  0x0020,   /* U+A82D */
  0x0020,   /* U+A82E */
  0x0020,   /* U+A82F */
  0x0020,   /* U+A830 */
  0x0020,   /* U+A831 */
  0x0020,   /* U+A832 */
  0x0020,   /* U+A833 */
  0x0020,   /* U+A834 */
  0x0020,   /* U+A835 */
  0x0020,   /* U+A836 */
  0x0020,   /* U+A837 */
  0x0020,   /* U+A838 */
  0x0020,   /* U+A839 */
  0x0020,   /* U+A83A */
  0x0020,   /* U+A83B */
  0x0020,   /* U+A83C */
  0x0020,   /* U+A83D */
  0x0020,   /* U+A83E */
  0x0020,   /* U+A83F */
  0x0020,   /* U+A840 */
  0x0020,   /* U+A841 */
  0x0020,   /* U+A842 */
  0x0020,   /* U+A843 */
  0x0020,   /* U+A844 */
  0x0020,   /* U+A845 */
  0x0020,   /* U+A846 */
  0x0020,   /* U+A847 */
  0x0020,   /* U+A848 */
  0x0020,   /* U+A849 */
  0x0020,   /* U+A84A */
  0x0020,   /* U+A84B */
  0x0020,   /* U+A84C */
  0x0020,   /* U+A84D */
  0x0020,   /* U+A84E */
  0x0020,   /* U+A84F */
  0x0020,   /* U+A850 */
  0x0020,   /* U+A851 */
  0x0020,   /* U+A852 */
  0x0020,   /* U+A853 */
  0x0020,   /* U+A854 */
  0x0020,   /* U+A855 */
  0x0020,   /* U+A856 */
  0x0020,   /* U+A857 */
  0x0020,   /* U+A858 */
  0x0020,   /* U+A859 */
  0x0020,   /* U+A85A */
  0x0020,   /* U+A85B */
  0x0020,   /* U+A85C */
  0x0020,   /* U+A85D */
  0x0020,   /* U+A85E */
  0x0020,   /* U+A85F */
  0x0020,   /* U+A860 */
  0x0020,   /* U+A861 */
  0x0020,   /* U+A862 */
  0x0020,   /* U+A863 */
  0x0020,   /* U+A864 */
  0x0020,   /* U+A865 */
  0x0020,   /* U+A866 */
  0x0020,   /* U+A867 */
  0x0020,   /* U+A868 */
  0x0020,   /* U+A869 */
  0x0020,   /* U+A86A */
  0x0020,   /* U+A86B */
  0x0020,   /* U+A86C */
  0x0020,   /* U+A86D */
  0x0020,   /* U+A86E */
  0x0020,   /* U+A86F */
  0x0020,   /* U+A870 */
  0x0020,   /* U+A871 */
  0x0020,   /* U+A872 */
  0x0020,   /* U+A873 */
  0x0020,   /* U+A874 */
  0x0020,   /* U+A875 */
  0x0020,   /* U+A876 */
  0x0020,   /* U+A877 */
  0x0020,   /* U+A878 */
  0x0020,   /* U+A879 */
  0x0020,   /* U+A87A */
  0x0020,   /* U+A87B */
  0x0020,   /* U+A87C */
  0x0020,   /* U+A87D */
  0x0020,   /* U+A87E */
  0x0020,   /* U+A87F */
  0x00C4,   /* U+A880 */
  0x00C5,   /* U+A881 */
  0x0020,   /* U+A882 */
  0x0020,   /* U+A883 */
  0x0020,   /* U+A884 */
  0x0020,   /* U+A885 */
  0x0020,   /* U+A886 */
  0x0020,   /* U+A887 */
  0x0020,   /* U+A888 */
  0x0020,   /* U+A889 */
  0x0020,   /* U+A88A */
  0x0020,   /* U+A88B */
  0x0020,   /* U+A88C */
  0x0020,   /* U+A88D */
  0x0020,   /* U+A88E */
  0x0020,   /* U+A88F */
  0x0020,   /* U+A890 */
  0x0020,   /* U+A891 */
  0x0020,   /* U+A892 */
  0x0020,   /* U+A893 */
  0x0020,   /* U+A894 */
  0x0020,   /* U+A895 */
  0x0020,   /* U+A896 */
  0x0020,   /* U+A897 */
  0x0020,   /* U+A898 */
  0x0020,   /* U+A899 */
  0x0020,   /* U+A89A */
  0x0020,   /* U+A89B */
  0x0020,   /* U+A89C */
  0x0020,   /* U+A89D */
  0x0020,   /* U+A89E */
  0x0020,   /* U+A89F */
  0x0020,   /* U+A8A0 */
  0x0020,   /* U+A8A1 */
  0x0020,   /* U+A8A2 */
  0x0020,   /* U+A8A3 */
  0x0020,   /* U+A8A4 */
  0x0020,   /* U+A8A5 */
  0x0020,   /* U+A8A6 */
  0x0020,   /* U+A8A7 */
  0x0020,   /* U+A8A8 */
  0x0020,   /* U+A8A9 */
  0x0020,   /* U+A8AA */
  0x0020,   /* U+A8AB */
  0x0020,   /* U+A8AC */
  0x0020,   /* U+A8AD */
  0x0020,   /* U+A8AE */
  0x0020,   /* U+A8AF */
  0x0020,   /* U+A8B0 */
  0x0020,   /* U+A8B1 */
  0x0020,   /* U+A8B2 */
  0x0020,   /* U+A8B3 */
  0x0020,   /* U+A8B4 */
  0x0020,   /* U+A8B5 */
  0x0020,   /* U+A8B6 */
  0x0020,   /* U+A8B7 */
  0x0020,   /* U+A8B8 */
  0x0020,   /* U+A8B9 */
  0x0020,   /* U+A8BA */
  0x0020,   /* U+A8BB */
  0x0020,   /* U+A8BC */
  0x0020,   /* U+A8BD */
  0x0020,   /* U+A8BE */
  0x0020,   /* U+A8BF */
  0x0020,   /* U+A8C0 */
  0x0020,   /* U+A8C1 */
  0x0020,   /* U+A8C2 */
  0x0020,   /* U+A8C3 */
  0x0020,   /* U+A8C4 */
  0x00C3,   /* U+A8C5 */
  0x0020,   /* U+A8C6 */
  0x0020,   /* U+A8C7 */
  0x0020,   /* U+A8C8 */
  0x0020,   /* U+A8C9 */
  0x0020,   /* U+A8CA */
  0x0020,   /* U+A8CB */
  0x0020,   /* U+A8CC */
  0x0020,   /* U+A8CD */
  0x0020,   /* U+A8CE */
  0x0020,   /* U+A8CF */
  0x0020,   /* U+A8D0 */
  0x0020,   /* U+A8D1 */
  0x0020,   /* U+A8D2 */
  0x0020,   /* U+A8D3 */
  0x0020,   /* U+A8D4 */
  0x0020,   /* U+A8D5 */
  0x0020,   /* U+A8D6 */
  0x0020,   /* U+A8D7 */
  0x0020,   /* U+A8D8 */
  0x0020,   /* U+A8D9 */
  0x0020,   /* U+A8DA */
  0x0020,   /* U+A8DB */
  0x0020,   /* U+A8DC */
  0x0020,   /* U+A8DD */
  0x0020,   /* U+A8DE */
  0x0020,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x0020,   /* U+A8F2 */
  0x0020,   /* U+A8F3 */
  0x0020,   /* U+A8F4 */
  0x0020,   /* U+A8F5 */
  0x0020,   /* U+A8F6 */
  0x0020,   /* U+A8F7 */
  0x0020,   /* U+A8F8 */
  0x0020,   /* U+A8F9 */
  0x0020,   /* U+A8FA */
  0x0020,   /* U+A8FB */
  0x0020,   /* U+A8FC */
  0x0020,   /* U+A8FD */
  0x0020,   /* U+A8FE */
  0x0020,   /* U+A8FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A800 */
  0x0002,   /* U+A801 */
  0x0002,   /* U+A802 */
  0x0002,   /* U+A803 */
  0x0002,   /* U+A804 */
  0x0002,   /* U+A805 */
  0x0002,   /* U+A806 */
  0x0002,   /* U+A807 */
  0x0002,   /* U+A808 */
  0x0002,   /* U+A809 */
  0x0002,   /* U+A80A */
  0x0002,   /* U+A80B */
  0x0002,   /* U+A80C */
  0x0002,   /* U+A80D */
  0x0002,   /* U+A80E */
  0x0002,   /* U+A80F */
  0x0002,   /* U+A810 */
  0x0002,   /* U+A811 */
  0x0002,   /* U+A812 */
  0x0002,   /* U+A813 */
  0x0002,   /* U+A814 */
  0x0002,   /* U+A815 */
  0x0002,   /* U+A816 */
  0x0002,   /* U+A817 */
  0x0002,   /* U+A818 */
  0x0002,   /* U+A819 */
  0x0002,   /* U+A81A */
  0x0002,   /* U+A81B */
  0x0002,   /* U+A81C */
  0x0002,   /* U+A81D */
  0x0002,   /* U+A81E */
  0x0002,   /* U+A81F */
  0x0002,   /* U+A820 */
  0x0002,   /* U+A821 */
  0x0002,   /* U+A822 */
  0x0002,   /* U+A823 */
  0x0002,   /* U+A824 */
  0x0002,   /* U+A825 */
  0x0002,   /* U+A826 */
  0x0002,   /* U+A827 */
  0x0002,   /* U+A828 */
  0x0002,   /* U+A829 */
  0x0002,   /* U+A82A */
  0x0002,   /* U+A82B */
  0x0002,   /* U+A82C */
  0x0002,   /* U+A82D */
  0x0002,   /* U+A82E */
  0x0002,   /* U+A82F */
  0x0002,   /* U+A830 */
  0x0002,   /* U+A831 */
  0x0002,   /* U+A832 */
  0x0002,   /* U+A833 */
  0x0002,   /* U+A834 */
  0x0002,   /* U+A835 */
  0x0002,   /* U+A836 */
  0x0002,   /* U+A837 */
  0x0002,   /* U+A838 */
  0x0002,   /* U+A839 */
  0x0002,   /* U+A83A */
  0x0002,   /* U+A83B */
  0x0002,   /* U+A83C */
  0x0002,   /* U+A83D */
  0x0002,   /* U+A83E */
  0x0002,   /* U+A83F */
  0x0002,   /* U+A840 */
  0x0002,   /* U+A841 */
  0x0002,   /* U+A842 */
  0x0002,   /* U+A843 */
  0x0002,   /* U+A844 */
  0x0002,   /* U+A845 */
  0x0002,   /* U+A846 */
  0x0002,   /* U+A847 */
  0x0002,   /* U+A848 */
  0x0002,   /* U+A849 */
  0x0002,   /* U+A84A */
  0x0002,   /* U+A84B */
  0x0002,   /* U+A84C */
  0x0002,   /* U+A84D */
  0x0002,   /* U+A84E */
  0x0002,   /* U+A84F */
  0x0002,   /* U+A850 */
  0x0002,   /* U+A851 */
  0x0002,   /* U+A852 */
  0x0002,   /* U+A853 */
  0x0002,   /* U+A854 */
  0x0002,   /* U+A855 */
  0x0002,   /* U+A856 */
  0x0002,   /* U+A857 */
  0x0002,   /* U+A858 */
  0x0002,   /* U+A859 */
  0x0002,   /* U+A85A */
  0x0002,   /* U+A85B */
  0x0002,   /* U+A85C */
  0x0002,   /* U+A85D */
  0x0002,   /* U+A85E */
  0x0002,   /* U+A85F */
  0x0002,   /* U+A860 */
  0x0002,   /* U+A861 */
  0x0002,   /* U+A862 */
  0x0002,   /* U+A863 */
  0x0002,   /* U+A864 */
  0x0002,   /* U+A865 */
  0x0002,   /* U+A866 */
  0x0002,   /* U+A867 */
  0x0002,   /* U+A868 */
  0x0002,   /* U+A869 */
  0x0002,   /* U+A86A */
  0x0002,   /* U+A86B */
  0x0002,   /* U+A86C */
  0x0002,   /* U+A86D */
  0x0002,   /* U+A86E */
  0x0002,   /* U+A86F */
  0x0002,   /* U+A870 */
  0x0002,   /* U+A871 */
  0x0002,   /* U+A872 */
  0x0002,   /* U+A873 */
  0x0002,   /* U+A874 */
  0x0002,   /* U+A875 */
  0x0002,   /* U+A876 */
  0x0002,   /* U+A877 */
  0x0002,   /* U+A878 */
  0x0002,   /* U+A879 */
  0x0002,   /* U+A87A */
  0x0002,   /* U+A87B */
  0x0002,   /* U+A87C */
  0x0002,   /* U+A87D */
  0x0002,   /* U+A87E */
  0x0002,   /* U+A87F */
  0x0002,   /* U+A880 */
  0x0002,   /* U+A881 */
  0x0002,   /* U+A882 */
  0x0002,   /* U+A883 */
  0x0002,   /* U+A884 */
  0x0002,   /* U+A885 */
  0x0002,   /* U+A886 */
  0x0002,   /* U+A887 */
  0x0002,   /* U+A888 */
  0x0002,   /* U+A889 */
  0x0002,   /* U+A88A */
  0x0002,   /* U+A88B */
  0x0002,   /* U+A88C */
  0x0002,   /* U+A88D */
  0x0002,   /* U+A88E */
  0x0002,   /* U+A88F */
  0x0002,   /* U+A890 */
  0x0002,   /* U+A891 */
  0x0002,   /* U+A892 */
  0x0002,   /* U+A893 */
  0x0002,   /* U+A894 */
  0x0002,   /* U+A895 */
  0x0002,   /* U+A896 */
  0x0002,   /* U+A897 */
  0x0002,   /* U+A898 */
  0x0002,   /* U+A899 */
  0x0002,   /* U+A89A */
  0x0002,   /* U+A89B */
  0x0002,   /* U+A89C */
  0x0002,   /* U+A89D */
  0x0002,   /* U+A89E */
  0x0002,   /* U+A89F */
  0x0002,   /* U+A8A0 */
  0x0002,   /* U+A8A1 */
  0x0002,   /* U+A8A2 */
  0x0002,   /* U+A8A3 */
  0x0002,   /* U+A8A4 */
  0x0002,   /* U+A8A5 */
  0x0002,   /* U+A8A6 */
  0x0002,   /* U+A8A7 */
  0x0002,   /* U+A8A8 */
  0x0002,   /* U+A8A9 */
  0x0002,   /* U+A8AA */
  0x0002,   /* U+A8AB */
  0x0002,   /* U+A8AC */
  0x0002,   /* U+A8AD */
  0x0002,   /* U+A8AE */
  0x0002,   /* U+A8AF */
  0x0002,   /* U+A8B0 */
  0x0002,   /* U+A8B1 */
  0x0002,   /* U+A8B2 */
  0x0002,   /* U+A8B3 */
  0x0002,   /* U+A8B4 */
  0x0002,   /* U+A8B5 */
  0x0002,   /* U+A8B6 */
  0x0002,   /* U+A8B7 */
  0x0002,   /* U+A8B8 */
  0x0002,   /* U+A8B9 */
  0x0002,   /* U+A8BA */
  0x0002,   /* U+A8BB */
  0x0002,   /* U+A8BC */
  0x0002,   /* U+A8BD */
  0x0002,   /* U+A8BE */
  0x0002,   /* U+A8BF */
  0x0002,   /* U+A8C0 */
  0x0002,   /* U+A8C1 */
  0x0002,   /* U+A8C2 */
  0x0002,   /* U+A8C3 */
  0x0002,   /* U+A8C4 */
  0x0002,   /* U+A8C5 */
  0x0002,   /* U+A8C6 */
  0x0002,   /* U+A8C7 */
  0x0002,   /* U+A8C8 */
  0x0002,   /* U+A8C9 */
  0x0002,   /* U+A8CA */
  0x0002,   /* U+A8CB */
  0x0002,   /* U+A8CC */
  0x0002,   /* U+A8CD */
  0x0002,   /* U+A8CE */
  0x0002,   /* U+A8CF */
  0x0002,   /* U+A8D0 */
  0x0002,   /* U+A8D1 */
  0x0002,   /* U+A8D2 */
  0x0002,   /* U+A8D3 */
  0x0002,   /* U+A8D4 */
  0x0002,   /* U+A8D5 */
  0x0002,   /* U+A8D6 */
  0x0002,   /* U+A8D7 */
  0x0002,   /* U+A8D8 */
  0x0002,   /* U+A8D9 */
  0x0002,   /* U+A8DA */
  0x0002,   /* U+A8DB */
  0x0002,   /* U+A8DC */
  0x0002,   /* U+A8DD */
  0x0002,   /* U+A8DE */
  0x0002,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x0002,   /* U+A8F2 */
  0x0004,   /* U+A8F3 */
  0x0004,   /* U+A8F4 */
  0x0004,   /* U+A8F5 */
  0x0004,   /* U+A8F6 */
  0x0004,   /* U+A8F7 */
  0x0002,   /* U+A8F8 */
  0x0002,   /* U+A8F9 */
  0x0002,   /* U+A8FA */
  0x0002,   /* U+A8FB */
  0x0002,   /* U+A8FC */
  0x0002,   /* U+A8FD */
  0x0002,   /* U+A8FE */
  0x0002,   /* U+A8FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+A800 */
  0x0000,   /* U+A801 */
  0x0000,   /* U+A802 */
  0x0000,   /* U+A803 */
  0x0000,   /* U+A804 */
  0x0000,   /* U+A805 */
  0x0000,   /* U+A806 */
  0x0000,   /* U+A807 */
  0x0000,   /* U+A808 */
  0x0000,   /* U+A809 */
  0x0000,   /* U+A80A */
  0x0000,   /* U+A80B */
  0x0000,   /* U+A80C */
  0x0000,   /* U+A80D */
  0x0000,   /* U+A80E */
  0x0000,   /* U+A80F */
  0x0000,   /* U+A810 */
  0x0000,   /* U+A811 */
  0x0000,   /* U+A812 */
  0x0000,   /* U+A813 */
  0x0000,   /* U+A814 */
  0x0000,   /* U+A815 */
  0x0000,   /* U+A816 */
  0x0000,   /* U+A817 */
  0x0000,   /* U+A818 */
  0x0000,   /* U+A819 */
  0x0000,   /* U+A81A */
  0x0000,   /* U+A81B */
  0x0000,   /* U+A81C */
  0x0000,   /* U+A81D */
  0x0000,   /* U+A81E */
  0x0000,   /* U+A81F */
  0x0000,   /* U+A820 */
  0x0000,   /* U+A821 */
  0x0000,   /* U+A822 */
  0x0000,   /* U+A823 */
  0x0000,   /* U+A824 */
  0x0000,   /* U+A825 */
  0x0000,   /* U+A826 */
  0x0000,   /* U+A827 */
  0x0000,   /* U+A828 */
  0x0000,   /* U+A829 */
  0x0000,   /* U+A82A */
  0x0000,   /* U+A82B */
  0xA82C,   /* U+A82C */
  0xA82D,   /* U+A82D */
  0xA82E,   /* U+A82E */
  0xA82F,   /* U+A82F */
  0x0000,   /* U+A830 */
  0x0000,   /* U+A831 */
  0x0000,   /* U+A832 */
  0x0000,   /* U+A833 */
  0x0000,   /* U+A834 */
  0x0000,   /* U+A835 */
  0x0000,   /* U+A836 */
  0x0000,   /* U+A837 */
  0x0000,   /* U+A838 */
  0x0000,   /* U+A839 */
  0xA83A,   /* U+A83A */
  0xA83B,   /* U+A83B */
  0xA83C,   /* U+A83C */
  0xA83D,   /* U+A83D */
  0xA83E,   /* U+A83E */
  0xA83F,   /* U+A83F */
  0x0000,   /* U+A840 */
  0x0000,   /* U+A841 */
  0x0000,   /* U+A842 */
  0x0000,   /* U+A843 */
  0x0000,   /* U+A844 */
  0x0000,   /* U+A845 */
  0x0000,   /* U+A846 */
  0x0000,   /* U+A847 */
  0x0000,   /* U+A848 */
  0x0000,   /* U+A849 */
  0x0000,   /* U+A84A */
  0x0000,   /* U+A84B */
  0x0000,   /* U+A84C */
  0x0000,   /* U+A84D */
  0x0000,   /* U+A84E */
  0x0000,   /* U+A84F */
  0x0000,   /* U+A850 */
  0x0000,   /* U+A851 */
  0x0000,   /* U+A852 */
  0x0000,   /* U+A853 */
  0x0000,   /* U+A854 */
  0x0000,   /* U+A855 */
  0x0000,   /* U+A856 */
  0x0000,   /* U+A857 */
  0x0000,   /* U+A858 */
  0x0000,   /* U+A859 */
  0x0000,   /* U+A85A */
  0x0000,   /* U+A85B */
  0x0000,   /* U+A85C */
  0x0000,   /* U+A85D */
  0x0000,   /* U+A85E */
  0x0000,   /* U+A85F */
  0x0000,   /* U+A860 */
  0x0000,   /* U+A861 */
  0x0000,   /* U+A862 */
  0x0000,   /* U+A863 */
  0x0000,   /* U+A864 */
  0x0000,   /* U+A865 */
  0x0000,   /* U+A866 */
  0x0000,   /* U+A867 */
  0x0000,   /* U+A868 */
  0x0000,   /* U+A869 */
  0x0000,   /* U+A86A */
  0x0000,   /* U+A86B */
  0x0000,   /* U+A86C */
  0x0000,   /* U+A86D */
  0x0000,   /* U+A86E */
  0x0000,   /* U+A86F */
  0x0000,   /* U+A870 */
  0x0000,   /* U+A871 */
  0x0000,   /* U+A872 */
  0x0000,   /* U+A873 */
  0x0000,   /* U+A874 */
  0x0000,   /* U+A875 */
  0x0000,   /* U+A876 */
  0x0000,   /* U+A877 */
  0xA878,   /* U+A878 */
  0xA879,   /* U+A879 */
  0xA87A,   /* U+A87A */
  0xA87B,   /* U+A87B */
  0xA87C,   /* U+A87C */
  0xA87D,   /* U+A87D */
  0xA87E,   /* U+A87E */
  0xA87F,   /* U+A87F */
  0x0000,   /* U+A880 */
  0x0000,   /* U+A881 */
  0x0000,   /* U+A882 */
  0x0000,   /* U+A883 */
  0x0000,   /* U+A884 */
  0x0000,   /* U+A885 */
  0x0000,   /* U+A886 */
  0x0000,   /* U+A887 */
  0x0000,   /* U+A888 */
  0x0000,   /* U+A889 */
  0x0000,   /* U+A88A */
  0x0000,   /* U+A88B */
  0x0000,   /* U+A88C */
  0x0000,   /* U+A88D */
  0x0000,   /* U+A88E */
  0x0000,   /* U+A88F */
  0x0000,   /* U+A890 */
  0x0000,   /* U+A891 */
  0x0000,   /* U+A892 */
  0x0000,   /* U+A893 */
  0x0000,   /* U+A894 */
  0x0000,   /* U+A895 */
  0x0000,   /* U+A896 */
  0x0000,   /* U+A897 */
  0x0000,   /* U+A898 */
  0x0000,   /* U+A899 */
  0x0000,   /* U+A89A */
  0x0000,   /* U+A89B */
  0x0000,   /* U+A89C */
  0x0000,   /* U+A89D */
  0x0000,   /* U+A89E */
  0x0000,   /* U+A89F */
  0x0000,   /* U+A8A0 */
  0x0000,   /* U+A8A1 */
  0x0000,   /* U+A8A2 */
  0x0000,   /* U+A8A3 */
  0x0000,   /* U+A8A4 */
  0x0000,   /* U+A8A5 */
  0x0000,   /* U+A8A6 */
  0x0000,   /* U+A8A7 */
  0x0000,   /* U+A8A8 */
  0x0000,   /* U+A8A9 */
  0x0000,   /* U+A8AA */
  0x0000,   /* U+A8AB */
  0x0000,   /* U+A8AC */
  0x0000,   /* U+A8AD */
  0x0000,   /* U+A8AE */
  0x0000,   /* U+A8AF */
  0x0000,   /* U+A8B0 */
  0x0000,   /* U+A8B1 */
  0x0000,   /* U+A8B2 */
  0x0000,   /* U+A8B3 */
  0x0000,   /* U+A8B4 */
  0x0000,   /* U+A8B5 */
  0x0000,   /* U+A8B6 */
  0x0000,   /* U+A8B7 */
  0x0000,   /* U+A8B8 */
  0x0000,   /* U+A8B9 */
  0x0000,   /* U+A8BA */
  0x0000,   /* U+A8BB */
  0x0000,   /* U+A8BC */
  0x0000,   /* U+A8BD */
  0x0000,   /* U+A8BE */
  0x0000,   /* U+A8BF */
  0x0000,   /* U+A8C0 */
  0x0000,   /* U+A8C1 */
  0x0000,   /* U+A8C2 */
  0x0000,   /* U+A8C3 */
  0x0000,   /* U+A8C4 */
  0x0000,   /* U+A8C5 */
  0xA8C6,   /* U+A8C6 */
  0xA8C7,   /* U+A8C7 */
  0xA8C8,   /* U+A8C8 */
  0xA8C9,   /* U+A8C9 */
  0xA8CA,   /* U+A8CA */
  0xA8CB,   /* U+A8CB */
  0xA8CC,   /* U+A8CC */
  0xA8CD,   /* U+A8CD */
  0x0000,   /* U+A8CE */
  0x0000,   /* U+A8CF */
  0x0000,   /* U+A8D0 */
  0x0000,   /* U+A8D1 */
  0x0000,   /* U+A8D2 */
  0x0000,   /* U+A8D3 */
  0x0000,   /* U+A8D4 */
  0x0000,   /* U+A8D5 */
  0x0000,   /* U+A8D6 */
  0x0000,   /* U+A8D7 */
  0x0000,   /* U+A8D8 */
  0x0000,   /* U+A8D9 */
  0xA8DA,   /* U+A8DA */
  0xA8DB,   /* U+A8DB */
  0xA8DC,   /* U+A8DC */
  0xA8DD,   /* U+A8DD */
  0xA8DE,   /* U+A8DE */
  0xA8DF,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x0000,   /* U+A8F2 */
  0x0000,   /* U+A8F3 */
  0x0000,   /* U+A8F4 */
  0x0000,   /* U+A8F5 */
  0x0000,   /* U+A8F6 */
  0x0000,   /* U+A8F7 */
  0x0000,   /* U+A8F8 */
  0x0000,   /* U+A8F9 */
  0x0000,   /* U+A8FA */
  0x0000,   /* U+A8FB */
  0x0000,   /* U+A8FC */
  0x0000,   /* U+A8FD */
  0xA8FE,   /* U+A8FE */
  0xA8FF,   /* U+A8FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+A800 */
  0x0000,   /* U+A801 */
  0x0000,   /* U+A802 */
  0x0000,   /* U+A803 */
  0x0000,   /* U+A804 */
  0x0000,   /* U+A805 */
  0x0000,   /* U+A806 */
  0x0000,   /* U+A807 */
  0x0000,   /* U+A808 */
  0x0000,   /* U+A809 */
  0x0000,   /* U+A80A */
  0x0000,   /* U+A80B */
  0x0000,   /* U+A80C */
  0x0000,   /* U+A80D */
  0x0000,   /* U+A80E */
  0x0000,   /* U+A80F */
  0x0000,   /* U+A810 */
  0x0000,   /* U+A811 */
  0x0000,   /* U+A812 */
  0x0000,   /* U+A813 */
  0x0000,   /* U+A814 */
  0x0000,   /* U+A815 */
  0x0000,   /* U+A816 */
  0x0000,   /* U+A817 */
  0x0000,   /* U+A818 */
  0x0000,   /* U+A819 */
  0x0000,   /* U+A81A */
  0x0000,   /* U+A81B */
  0x0000,   /* U+A81C */
  0x0000,   /* U+A81D */
  0x0000,   /* U+A81E */
  0x0000,   /* U+A81F */
  0x0000,   /* U+A820 */
  0x0000,   /* U+A821 */
  0x0000,   /* U+A822 */
  0x0000,   /* U+A823 */
  0x0000,   /* U+A824 */
  0x0000,   /* U+A825 */
  0x0000,   /* U+A826 */
  0x0000,   /* U+A827 */
  0x0000,   /* U+A828 */
  0x0000,   /* U+A829 */
  0x0000,   /* U+A82A */
  0x0000,   /* U+A82B */
  0x0000,   /* U+A82C */
  0x0000,   /* U+A82D */
  0x0000,   /* U+A82E */
  0x0000,   /* U+A82F */
  0x0000,   /* U+A830 */
  0x0000,   /* U+A831 */
  0x0000,   /* U+A832 */
  0x0000,   /* U+A833 */
  0x0000,   /* U+A834 */
  0x0000,   /* U+A835 */
  0x0000,   /* U+A836 */
  0x0000,   /* U+A837 */
  0x0000,   /* U+A838 */
  0x0000,   /* U+A839 */
  0x0000,   /* U+A83A */
  0x0000,   /* U+A83B */
  0x0000,   /* U+A83C */
  0x0000,   /* U+A83D */
  0x0000,   /* U+A83E */
  0x0000,   /* U+A83F */
  0x0000,   /* U+A840 */
  0x0000,   /* U+A841 */
  0x0000,   /* U+A842 */
  0x0000,   /* U+A843 */
  0x0000,   /* U+A844 */
  0x0000,   /* U+A845 */
  0x0000,   /* U+A846 */
  0x0000,   /* U+A847 */
  0x0000,   /* U+A848 */
  0x0000,   /* U+A849 */
  0x0000,   /* U+A84A */
  0x0000,   /* U+A84B */
  0x0000,   /* U+A84C */
  0x0000,   /* U+A84D */
  0x0000,   /* U+A84E */
  0x0000,   /* U+A84F */
  0x0000,   /* U+A850 */
  0x0000,   /* U+A851 */
  0x0000,   /* U+A852 */
  0x0000,   /* U+A853 */
  0x0000,   /* U+A854 */
  0x0000,   /* U+A855 */
  0x0000,   /* U+A856 */
  0x0000,   /* U+A857 */
  0x0000,   /* U+A858 */
  0x0000,   /* U+A859 */
  0x0000,   /* U+A85A */
  0x0000,   /* U+A85B */
  0x0000,   /* U+A85C */
  0x0000,   /* U+A85D */
  0x0000,   /* U+A85E */
  0x0000,   /* U+A85F */
  0x0000,   /* U+A860 */
  0x0000,   /* U+A861 */
  0x0000,   /* U+A862 */
  0x0000,   /* U+A863 */
  0x0000,   /* U+A864 */
  0x0000,   /* U+A865 */
  0x0000,   /* U+A866 */
  0x0000,   /* U+A867 */
  0x0000,   /* U+A868 */
  0x0000,   /* U+A869 */
  0x0000,   /* U+A86A */
  0x0000,   /* U+A86B */
  0x0000,   /* U+A86C */
  0x0000,   /* U+A86D */
  0x0000,   /* U+A86E */
  0x0000,   /* U+A86F */
  0x0000,   /* U+A870 */
  0x0000,   /* U+A871 */
  0x0000,   /* U+A872 */
  0x0000,   /* U+A873 */
  0x0000,   /* U+A874 */
  0x0000,   /* U+A875 */
  0x0000,   /* U+A876 */
  0x0000,   /* U+A877 */
  0x0000,   /* U+A878 */
  0x0000,   /* U+A879 */
  0x0000,   /* U+A87A */
  0x0000,   /* U+A87B */
  0x0000,   /* U+A87C */
  0x0000,   /* U+A87D */
  0x0000,   /* U+A87E */
  0x0000,   /* U+A87F */
  0x0000,   /* U+A880 */
  0x0000,   /* U+A881 */
  0x0000,   /* U+A882 */
  0x0000,   /* U+A883 */
  0x0000,   /* U+A884 */
  0x0000,   /* U+A885 */
  0x0000,   /* U+A886 */
  0x0000,   /* U+A887 */
  0x0000,   /* U+A888 */
  0x0000,   /* U+A889 */
  0x0000,   /* U+A88A */
  0x0000,   /* U+A88B */
  0x0000,   /* U+A88C */
  0x0000,   /* U+A88D */
  0x0000,   /* U+A88E */
  0x0000,   /* U+A88F */
  0x0000,   /* U+A890 */
  0x0000,   /* U+A891 */
  0x0000,   /* U+A892 */
  0x0000,   /* U+A893 */
  0x0000,   /* U+A894 */
  0x0000,   /* U+A895 */
  0x0000,   /* U+A896 */
  0x0000,   /* U+A897 */
  0x0000,   /* U+A898 */
  0x0000,   /* U+A899 */
  0x0000,   /* U+A89A */
  0x0000,   /* U+A89B */
  0x0000,   /* U+A89C */
  0x0000,   /* U+A89D */
  0x0000,   /* U+A89E */
  0x0000,   /* U+A89F */
  0x0000,   /* U+A8A0 */
  0x0000,   /* U+A8A1 */
  0x0000,   /* U+A8A2 */
  0x0000,   /* U+A8A3 */
  0x0000,   /* U+A8A4 */
  0x0000,   /* U+A8A5 */
  0x0000,   /* U+A8A6 */
  0x0000,   /* U+A8A7 */
  0x0000,   /* U+A8A8 */
  0x0000,   /* U+A8A9 */
  0x0000,   /* U+A8AA */
  0x0000,   /* U+A8AB */
  0x0000,   /* U+A8AC */
  0x0000,   /* U+A8AD */
  0x0000,   /* U+A8AE */
  0x0000,   /* U+A8AF */
  0x0000,   /* U+A8B0 */
  0x0000,   /* U+A8B1 */
  0x0000,   /* U+A8B2 */
  0x0000,   /* U+A8B3 */
  0x0000,   /* U+A8B4 */
  0x0000,   /* U+A8B5 */
  0x0000,   /* U+A8B6 */
  0x0000,   /* U+A8B7 */
  0x0000,   /* U+A8B8 */
  0x0000,   /* U+A8B9 */
  0x0000,   /* U+A8BA */
  0x0000,   /* U+A8BB */
  0x0000,   /* U+A8BC */
  0x0000,   /* U+A8BD */
  0x0000,   /* U+A8BE */
  0x0000,   /* U+A8BF */
  0x0000,   /* U+A8C0 */
  0x0000,   /* U+A8C1 */
  0x0000,   /* U+A8C2 */
  0x0000,   /* U+A8C3 */
  0x0000,   /* U+A8C4 */
  0x0000,   /* U+A8C5 */
  0x0000,   /* U+A8C6 */
  0x0000,   /* U+A8C7 */
  0x0000,   /* U+A8C8 */
  0x0000,   /* U+A8C9 */
  0x0000,   /* U+A8CA */
  0x0000,   /* U+A8CB */
  0x0000,   /* U+A8CC */
  0x0000,   /* U+A8CD */
  0x0000,   /* U+A8CE */
  0x0000,   /* U+A8CF */
  0x0000,   /* U+A8D0 */
  0x0000,   /* U+A8D1 */
  0x0000,   /* U+A8D2 */
  0x0000,   /* U+A8D3 */
  0x0000,   /* U+A8D4 */
  0x0000,   /* U+A8D5 */
  0x0000,   /* U+A8D6 */
  0x0000,   /* U+A8D7 */
  0x0000,   /* U+A8D8 */
  0x0000,   /* U+A8D9 */
  0x0000,   /* U+A8DA */
  0x0000,   /* U+A8DB */
  0x0000,   /* U+A8DC */
  0x0000,   /* U+A8DD */
  0x0000,   /* U+A8DE */
  0x0000,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x0000,   /* U+A8F2 */
  0x0000,   /* U+A8F3 */
  0x0000,   /* U+A8F4 */
  0x0000,   /* U+A8F5 */
  0x0000,   /* U+A8F6 */
  0x0000,   /* U+A8F7 */
  0x0000,   /* U+A8F8 */
  0x0000,   /* U+A8F9 */
  0x0000,   /* U+A8FA */
  0x0000,   /* U+A8FB */
  0x0000,   /* U+A8FC */
  0x0000,   /* U+A8FD */
  0x0000,   /* U+A8FE */
  0x0000,   /* U+A8FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+A800 */
  0x0000,   /* U+A801 */
  0x0000,   /* U+A802 */
  0x0000,   /* U+A803 */
  0x0000,   /* U+A804 */
  0x0000,   /* U+A805 */
  0x0000,   /* U+A806 */
  0x0000,   /* U+A807 */
  0x0000,   /* U+A808 */
  0x0000,   /* U+A809 */
  0x0000,   /* U+A80A */
  0x0000,   /* U+A80B */
  0x0000,   /* U+A80C */
  0x0000,   /* U+A80D */
  0x0000,   /* U+A80E */
  0x0000,   /* U+A80F */
  0x0000,   /* U+A810 */
  0x0000,   /* U+A811 */
  0x0000,   /* U+A812 */
  0x0000,   /* U+A813 */
  0x0000,   /* U+A814 */
  0x0000,   /* U+A815 */
  0x0000,   /* U+A816 */
  0x0000,   /* U+A817 */
  0x0000,   /* U+A818 */
  0x0000,   /* U+A819 */
  0x0000,   /* U+A81A */
  0x0000,   /* U+A81B */
  0x0000,   /* U+A81C */
  0x0000,   /* U+A81D */
  0x0000,   /* U+A81E */
  0x0000,   /* U+A81F */
  0x0000,   /* U+A820 */
  0x0000,   /* U+A821 */
  0x0000,   /* U+A822 */
  0x0000,   /* U+A823 */
  0x0000,   /* U+A824 */
  0x0000,   /* U+A825 */
  0x0000,   /* U+A826 */
  0x0000,   /* U+A827 */
  0x0000,   /* U+A828 */
  0x0000,   /* U+A829 */
  0x0000,   /* U+A82A */
  0x0000,   /* U+A82B */
  0x0000,   /* U+A82C */
  0x0000,   /* U+A82D */
  0x0000,   /* U+A82E */
  0x0000,   /* U+A82F */
  0x0000,   /* U+A830 */
  0x0000,   /* U+A831 */
  0x0000,   /* U+A832 */
  0x0000,   /* U+A833 */
  0x0000,   /* U+A834 */
  0x0000,   /* U+A835 */
  0x0000,   /* U+A836 */
  0x0000,   /* U+A837 */
  0x0000,   /* U+A838 */
  0x0000,   /* U+A839 */
  0x0000,   /* U+A83A */
  0x0000,   /* U+A83B */
  0x0000,   /* U+A83C */
  0x0000,   /* U+A83D */
  0x0000,   /* U+A83E */
  0x0000,   /* U+A83F */
  0x0000,   /* U+A840 */
  0x0000,   /* U+A841 */
  0x0000,   /* U+A842 */
  0x0000,   /* U+A843 */
  0x0000,   /* U+A844 */
  0x0000,   /* U+A845 */
  0x0000,   /* U+A846 */
  0x0000,   /* U+A847 */
  0x0000,   /* U+A848 */
  0x0000,   /* U+A849 */
  0x0000,   /* U+A84A */
  0x0000,   /* U+A84B */
  0x0000,   /* U+A84C */
  0x0000,   /* U+A84D */
  0x0000,   /* U+A84E */
  0x0000,   /* U+A84F */
  0x0000,   /* U+A850 */
  0x0000,   /* U+A851 */
  0x0000,   /* U+A852 */
  0x0000,   /* U+A853 */
  0x0000,   /* U+A854 */
  0x0000,   /* U+A855 */
  0x0000,   /* U+A856 */
  0x0000,   /* U+A857 */
  0x0000,   /* U+A858 */
  0x0000,   /* U+A859 */
  0x0000,   /* U+A85A */
  0x0000,   /* U+A85B */
  0x0000,   /* U+A85C */
  0x0000,   /* U+A85D */
  0x0000,   /* U+A85E */
  0x0000,   /* U+A85F */
  0x0000,   /* U+A860 */
  0x0000,   /* U+A861 */
  0x0000,   /* U+A862 */
  0x0000,   /* U+A863 */
  0x0000,   /* U+A864 */
  0x0000,   /* U+A865 */
  0x0000,   /* U+A866 */
  0x0000,   /* U+A867 */
  0x0000,   /* U+A868 */
  0x0000,   /* U+A869 */
  0x0000,   /* U+A86A */
  0x0000,   /* U+A86B */
  0x0000,   /* U+A86C */
  0x0000,   /* U+A86D */
  0x0000,   /* U+A86E */
  0x0000,   /* U+A86F */
  0x0000,   /* U+A870 */
  0x0000,   /* U+A871 */
  0x0000,   /* U+A872 */
  0x0000,   /* U+A873 */
  0x0000,   /* U+A874 */
  0x0000,   /* U+A875 */
  0x0000,   /* U+A876 */
  0x0000,   /* U+A877 */
  0x0000,   /* U+A878 */
  0x0000,   /* U+A879 */
  0x0000,   /* U+A87A */
  0x0000,   /* U+A87B */
  0x0000,   /* U+A87C */
  0x0000,   /* U+A87D */
  0x0000,   /* U+A87E */
  0x0000,   /* U+A87F */
  0x0000,   /* U+A880 */
  0x0000,   /* U+A881 */
  0x0000,   /* U+A882 */
  0x0000,   /* U+A883 */
  0x0000,   /* U+A884 */
  0x0000,   /* U+A885 */
  0x0000,   /* U+A886 */
  0x0000,   /* U+A887 */
  0x0000,   /* U+A888 */
  0x0000,   /* U+A889 */
  0x0000,   /* U+A88A */
  0x0000,   /* U+A88B */
  0x0000,   /* U+A88C */
  0x0000,   /* U+A88D */
  0x0000,   /* U+A88E */
  0x0000,   /* U+A88F */
  0x0000,   /* U+A890 */
  0x0000,   /* U+A891 */
  0x0000,   /* U+A892 */
  0x0000,   /* U+A893 */
  0x0000,   /* U+A894 */
  0x0000,   /* U+A895 */
  0x0000,   /* U+A896 */
  0x0000,   /* U+A897 */
  0x0000,   /* U+A898 */
  0x0000,   /* U+A899 */
  0x0000,   /* U+A89A */
  0x0000,   /* U+A89B */
  0x0000,   /* U+A89C */
  0x0000,   /* U+A89D */
  0x0000,   /* U+A89E */
  0x0000,   /* U+A89F */
  0x0000,   /* U+A8A0 */
  0x0000,   /* U+A8A1 */
  0x0000,   /* U+A8A2 */
  0x0000,   /* U+A8A3 */
  0x0000,   /* U+A8A4 */
  0x0000,   /* U+A8A5 */
  0x0000,   /* U+A8A6 */
  0x0000,   /* U+A8A7 */
  0x0000,   /* U+A8A8 */
  0x0000,   /* U+A8A9 */
  0x0000,   /* U+A8AA */
  0x0000,   /* U+A8AB */
  0x0000,   /* U+A8AC */
  0x0000,   /* U+A8AD */
  0x0000,   /* U+A8AE */
  0x0000,   /* U+A8AF */
  0x0000,   /* U+A8B0 */
  0x0000,   /* U+A8B1 */
  0x0000,   /* U+A8B2 */
  0x0000,   /* U+A8B3 */
  0x0000,   /* U+A8B4 */
  0x0000,   /* U+A8B5 */
  0x0000,   /* U+A8B6 */
  0x0000,   /* U+A8B7 */
  0x0000,   /* U+A8B8 */
  0x0000,   /* U+A8B9 */
  0x0000,   /* U+A8BA */
  0x0000,   /* U+A8BB */
  0x0000,   /* U+A8BC */
  0x0000,   /* U+A8BD */
  0x0000,   /* U+A8BE */
  0x0000,   /* U+A8BF */
  0x0000,   /* U+A8C0 */
  0x0000,   /* U+A8C1 */
  0x0000,   /* U+A8C2 */
  0x0000,   /* U+A8C3 */
  0x0000,   /* U+A8C4 */
  0x0000,   /* U+A8C5 */
  0x0000,   /* U+A8C6 */
  0x0000,   /* U+A8C7 */
  0x0000,   /* U+A8C8 */
  0x0000,   /* U+A8C9 */
  0x0000,   /* U+A8CA */
  0x0000,   /* U+A8CB */
  0x0000,   /* U+A8CC */
  0x0000,   /* U+A8CD */
  0x0000,   /* U+A8CE */
  0x0000,   /* U+A8CF */
  0x0000,   /* U+A8D0 */
  0x0000,   /* U+A8D1 */
  0x0000,   /* U+A8D2 */
  0x0000,   /* U+A8D3 */
  0x0000,   /* U+A8D4 */
  0x0000,   /* U+A8D5 */
  0x0000,   /* U+A8D6 */
  0x0000,   /* U+A8D7 */
  0x0000,   /* U+A8D8 */
  0x0000,   /* U+A8D9 */
  0x0000,   /* U+A8DA */
  0x0000,   /* U+A8DB */
  0x0000,   /* U+A8DC */
  0x0000,   /* U+A8DD */
  0x0000,   /* U+A8DE */
  0x0000,   /* U+A8DF */
  0x0000,   /* U+A8E0 */
  0x0000,   /* U+A8E1 */
  0x0000,   /* U+A8E2 */
  0x0000,   /* U+A8E3 */
  0x0000,   /* U+A8E4 */
  0x0000,   /* U+A8E5 */
  0x0000,   /* U+A8E6 */
  0x0000,   /* U+A8E7 */
  0x0000,   /* U+A8E8 */
  0x0000,   /* U+A8E9 */
  0x0000,   /* U+A8EA */
  0x0000,   /* U+A8EB */
  0x0000,   /* U+A8EC */
  0x0000,   /* U+A8ED */
  0x0000,   /* U+A8EE */
  0x0000,   /* U+A8EF */
  0x0000,   /* U+A8F0 */
  0x0000,   /* U+A8F1 */
  0x0000,   /* U+A8F2 */
  0x0000,   /* U+A8F3 */
  0x0000,   /* U+A8F4 */
  0x0000,   /* U+A8F5 */
  0x0000,   /* U+A8F6 */
  0x0000,   /* U+A8F7 */
  0x0000,   /* U+A8F8 */
  0x0000,   /* U+A8F9 */
  0x0000,   /* U+A8FA */
  0x0000,   /* U+A8FB */
  0x0000,   /* U+A8FC */
  0x0000,   /* U+A8FD */
  0x0000,   /* U+A8FE */
  0x0000,   /* U+A8FF */
};

uint16 uca900_p0A9[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 

  /* Primary weight 1 for each character. */
  0x1C3D,   /* U+A900 */
  0x1C3E,   /* U+A901 */
  0x1C3F,   /* U+A902 */
  0x1C40,   /* U+A903 */
  0x1C41,   /* U+A904 */
  0x1C42,   /* U+A905 */
  0x1C43,   /* U+A906 */
  0x1C44,   /* U+A907 */
  0x1C45,   /* U+A908 */
  0x1C46,   /* U+A909 */
  0x3014,   /* U+A90A */
  0x3015,   /* U+A90B */
  0x3016,   /* U+A90C */
  0x3017,   /* U+A90D */
  0x3018,   /* U+A90E */
  0x3019,   /* U+A90F */
  0x301A,   /* U+A910 */
  0x301B,   /* U+A911 */
  0x301C,   /* U+A912 */
  0x301D,   /* U+A913 */
  0x301E,   /* U+A914 */
  0x301F,   /* U+A915 */
  0x3020,   /* U+A916 */
  0x3021,   /* U+A917 */
  0x3022,   /* U+A918 */
  0x3023,   /* U+A919 */
  0x3024,   /* U+A91A */
  0x3025,   /* U+A91B */
  0x3026,   /* U+A91C */
  0x3027,   /* U+A91D */
  0x3028,   /* U+A91E */
  0x3029,   /* U+A91F */
  0x302A,   /* U+A920 */
  0x302B,   /* U+A921 */
  0x302C,   /* U+A922 */
  0x302D,   /* U+A923 */
  0x302E,   /* U+A924 */
  0x302F,   /* U+A925 */
  0x3030,   /* U+A926 */
  0x3031,   /* U+A927 */
  0x3032,   /* U+A928 */
  0x3033,   /* U+A929 */
  0x3034,   /* U+A92A */
  0x0000,   /* U+A92B */
  0x0000,   /* U+A92C */
  0x0000,   /* U+A92D */
  0x042E,   /* U+A92E */
  0x0298,   /* U+A92F */
  0x2FF0,   /* U+A930 */
  0x2FF1,   /* U+A931 */
  0x2FF2,   /* U+A932 */
  0x2FF3,   /* U+A933 */
  0x2FF4,   /* U+A934 */
  0x2FF5,   /* U+A935 */
  0x2FF6,   /* U+A936 */
  0x2FF7,   /* U+A937 */
  0x2FF8,   /* U+A938 */
  0x2FF9,   /* U+A939 */
  0x2FFA,   /* U+A93A */
  0x2FFB,   /* U+A93B */
  0x2FFC,   /* U+A93C */
  0x2FFD,   /* U+A93D */
  0x2FFE,   /* U+A93E */
  0x2FFF,   /* U+A93F */
  0x3000,   /* U+A940 */
  0x3001,   /* U+A941 */
  0x3002,   /* U+A942 */
  0x3003,   /* U+A943 */
  0x3004,   /* U+A944 */
  0x3005,   /* U+A945 */
  0x3006,   /* U+A946 */
  0x3007,   /* U+A947 */
  0x3008,   /* U+A948 */
  0x3009,   /* U+A949 */
  0x300A,   /* U+A94A */
  0x300B,   /* U+A94B */
  0x300C,   /* U+A94C */
  0x300D,   /* U+A94D */
  0x300E,   /* U+A94E */
  0x300F,   /* U+A94F */
  0x3010,   /* U+A950 */
  0x3011,   /* U+A951 */
  0x3012,   /* U+A952 */
  0x3013,   /* U+A953 */
  0xFBC1,   /* U+A954 */
  0xFBC1,   /* U+A955 */
  0xFBC1,   /* U+A956 */
  0xFBC1,   /* U+A957 */
  0xFBC1,   /* U+A958 */
  0xFBC1,   /* U+A959 */
  0xFBC1,   /* U+A95A */
  0xFBC1,   /* U+A95B */
  0xFBC1,   /* U+A95C */
  0xFBC1,   /* U+A95D */
  0xFBC1,   /* U+A95E */
  0x02D8,   /* U+A95F */
  0x3C54,   /* U+A960 */
  0x3C55,   /* U+A961 */
  0x3C56,   /* U+A962 */
  0x3C57,   /* U+A963 */
  0x3C58,   /* U+A964 */
  0x3C59,   /* U+A965 */
  0x3C5A,   /* U+A966 */
  0x3C5B,   /* U+A967 */
  0x3C5C,   /* U+A968 */
  0x3C5D,   /* U+A969 */
  0x3C5E,   /* U+A96A */
  0x3C5F,   /* U+A96B */
  0x3C60,   /* U+A96C */
  0x3C61,   /* U+A96D */
  0x3C62,   /* U+A96E */
  0x3C63,   /* U+A96F */
  0x3C64,   /* U+A970 */
  0x3C65,   /* U+A971 */
  0x3C66,   /* U+A972 */
  0x3C67,   /* U+A973 */
  0x3C68,   /* U+A974 */
  0x3C69,   /* U+A975 */
  0x3C6A,   /* U+A976 */
  0x3C6B,   /* U+A977 */
  0x3C6C,   /* U+A978 */
  0x3C6D,   /* U+A979 */
  0x3C6E,   /* U+A97A */
  0x3C6F,   /* U+A97B */
  0x3C70,   /* U+A97C */
  0xFBC1,   /* U+A97D */
  0xFBC1,   /* U+A97E */
  0xFBC1,   /* U+A97F */
  0x0000,   /* U+A980 */
  0x0000,   /* U+A981 */
  0x0000,   /* U+A982 */
  0x0000,   /* U+A983 */
  0x329D,   /* U+A984 */
  0x329E,   /* U+A985 */
  0x329F,   /* U+A986 */
  0x32A0,   /* U+A987 */
  0x32A1,   /* U+A988 */
  0x32A2,   /* U+A989 */
  0x32A3,   /* U+A98A */
  0x32A4,   /* U+A98B */
  0x32A5,   /* U+A98C */
  0x32A6,   /* U+A98D */
  0x32A7,   /* U+A98E */
  0x32A8,   /* U+A98F */
  0x32A9,   /* U+A990 */
  0x32AA,   /* U+A991 */
  0x32AB,   /* U+A992 */
  0x32AC,   /* U+A993 */
  0x32AD,   /* U+A994 */
  0x32AE,   /* U+A995 */
  0x32AF,   /* U+A996 */
  0x32B0,   /* U+A997 */
  0x32B1,   /* U+A998 */
  0x32B2,   /* U+A999 */
  0x32B3,   /* U+A99A */
  0x32B4,   /* U+A99B */
  0x32B5,   /* U+A99C */
  0x32B6,   /* U+A99D */
  0x32B7,   /* U+A99E */
  0x32B8,   /* U+A99F */
  0x32B9,   /* U+A9A0 */
  0x32BA,   /* U+A9A1 */
  0x32BB,   /* U+A9A2 */
  0x32BC,   /* U+A9A3 */
  0x32BD,   /* U+A9A4 */
  0x32BE,   /* U+A9A5 */
  0x32BF,   /* U+A9A6 */
  0x32C0,   /* U+A9A7 */
  0x32C1,   /* U+A9A8 */
  0x32C2,   /* U+A9A9 */
  0x32C3,   /* U+A9AA */
  0x32C5,   /* U+A9AB */
  0x32C5,   /* U+A9AC */
  0x32C7,   /* U+A9AD */
  0x32C8,   /* U+A9AE */
  0x32C9,   /* U+A9AF */
  0x32CA,   /* U+A9B0 */
  0x32CB,   /* U+A9B1 */
  0x32CC,   /* U+A9B2 */
  0x0000,   /* U+A9B3 */
  0x32CD,   /* U+A9B4 */
  0x32D6,   /* U+A9B5 */
  0x32CF,   /* U+A9B6 */
  0x32D0,   /* U+A9B7 */
  0x32D1,   /* U+A9B8 */
  0x32D2,   /* U+A9B9 */
  0x32D4,   /* U+A9BA */
  0x32D5,   /* U+A9BB */
  0x32CE,   /* U+A9BC */
  0x32D3,   /* U+A9BD */
  0x32C4,   /* U+A9BE */
  0x32C6,   /* U+A9BF */
  0x32D7,   /* U+A9C0 */
  0x02CD,   /* U+A9C1 */
  0x02CE,   /* U+A9C2 */
  0x02CF,   /* U+A9C3 */
  0x02D0,   /* U+A9C4 */
  0x02D1,   /* U+A9C5 */
  0x02D2,   /* U+A9C6 */
  0x025B,   /* U+A9C7 */
  0x02A3,   /* U+A9C8 */
  0x02A4,   /* U+A9C9 */
  0x02D3,   /* U+A9CA */
  0x02D4,   /* U+A9CB */
  0x02D5,   /* U+A9CC */
  0x02D6,   /* U+A9CD */
  0xFBC1,   /* U+A9CE */
  0x1BFF,   /* U+A9CF */
  0x1C3D,   /* U+A9D0 */
  0x1C3E,   /* U+A9D1 */
  0x1C3F,   /* U+A9D2 */
  0x1C40,   /* U+A9D3 */
  0x1C41,   /* U+A9D4 */
  0x1C42,   /* U+A9D5 */
  0x1C43,   /* U+A9D6 */
  0x1C44,   /* U+A9D7 */
  0x1C45,   /* U+A9D8 */
  0x1C46,   /* U+A9D9 */
  0xFBC1,   /* U+A9DA */
  0xFBC1,   /* U+A9DB */
  0xFBC1,   /* U+A9DC */
  0xFBC1,   /* U+A9DD */
  0x042F,   /* U+A9DE */
  0x0430,   /* U+A9DF */
  0x303E,   /* U+A9E0 */
  0x3046,   /* U+A9E1 */
  0x3050,   /* U+A9E2 */
  0x3066,   /* U+A9E3 */
  0x307F,   /* U+A9E4 */
  0x30C9,   /* U+A9E5 */
  0x1C00,   /* U+A9E6 */
  0x3058,   /* U+A9E7 */
  0x307A,   /* U+A9E8 */
  0x303C,   /* U+A9E9 */
  0x303F,   /* U+A9EA */
  0x304B,   /* U+A9EB */
  0x3052,   /* U+A9EC */
  0x3060,   /* U+A9ED */
  0x3063,   /* U+A9EE */
  0x3067,   /* U+A9EF */
  0x1C3D,   /* U+A9F0 */
  0x1C3E,   /* U+A9F1 */
  0x1C3F,   /* U+A9F2 */
  0x1C40,   /* U+A9F3 */
  0x1C41,   /* U+A9F4 */
  0x1C42,   /* U+A9F5 */
  0x1C43,   /* U+A9F6 */
  0x1C44,   /* U+A9F7 */
  0x1C45,   /* U+A9F8 */
  0x1C46,   /* U+A9F9 */
  0x309B,   /* U+A9FA */
  0x306C,   /* U+A9FB */
  0x306F,   /* U+A9FC */
  0x307D,   /* U+A9FD */
  0x3080,   /* U+A9FE */
  0xFBC1,   /* U+A9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+A900 */
  0x0020,   /* U+A901 */
  0x0020,   /* U+A902 */
  0x0020,   /* U+A903 */
  0x0020,   /* U+A904 */
  0x0020,   /* U+A905 */
  0x0020,   /* U+A906 */
  0x0020,   /* U+A907 */
  0x0020,   /* U+A908 */
  0x0020,   /* U+A909 */
  0x0020,   /* U+A90A */
  0x0020,   /* U+A90B */
  0x0020,   /* U+A90C */
  0x0020,   /* U+A90D */
  0x0020,   /* U+A90E */
  0x0020,   /* U+A90F */
  0x0020,   /* U+A910 */
  0x0020,   /* U+A911 */
  0x0020,   /* U+A912 */
  0x0020,   /* U+A913 */
  0x0020,   /* U+A914 */
  0x0020,   /* U+A915 */
  0x0020,   /* U+A916 */
  0x0020,   /* U+A917 */
  0x0020,   /* U+A918 */
  0x0020,   /* U+A919 */
  0x0020,   /* U+A91A */
  0x0020,   /* U+A91B */
  0x0020,   /* U+A91C */
  0x0020,   /* U+A91D */
  0x0020,   /* U+A91E */
  0x0020,   /* U+A91F */
  0x0020,   /* U+A920 */
  0x0020,   /* U+A921 */
  0x0020,   /* U+A922 */
  0x0020,   /* U+A923 */
  0x0020,   /* U+A924 */
  0x0020,   /* U+A925 */
  0x0020,   /* U+A926 */
  0x0020,   /* U+A927 */
  0x0020,   /* U+A928 */
  0x0020,   /* U+A929 */
  0x0020,   /* U+A92A */
  0x00E2,   /* U+A92B */
  0x00E3,   /* U+A92C */
  0x00E4,   /* U+A92D */
  0x0020,   /* U+A92E */
  0x0020,   /* U+A92F */
  0x0020,   /* U+A930 */
  0x0020,   /* U+A931 */
  0x0020,   /* U+A932 */
  0x0020,   /* U+A933 */
  0x0020,   /* U+A934 */
  0x0020,   /* U+A935 */
  0x0020,   /* U+A936 */
  0x0020,   /* U+A937 */
  0x0020,   /* U+A938 */
  0x0020,   /* U+A939 */
  0x0020,   /* U+A93A */
  0x0020,   /* U+A93B */
  0x0020,   /* U+A93C */
  0x0020,   /* U+A93D */
  0x0020,   /* U+A93E */
  0x0020,   /* U+A93F */
  0x0020,   /* U+A940 */
  0x0020,   /* U+A941 */
  0x0020,   /* U+A942 */
  0x0020,   /* U+A943 */
  0x0020,   /* U+A944 */
  0x0020,   /* U+A945 */
  0x0020,   /* U+A946 */
  0x0020,   /* U+A947 */
  0x0020,   /* U+A948 */
  0x0020,   /* U+A949 */
  0x0020,   /* U+A94A */
  0x0020,   /* U+A94B */
  0x0020,   /* U+A94C */
  0x0020,   /* U+A94D */
  0x0020,   /* U+A94E */
  0x0020,   /* U+A94F */
  0x0020,   /* U+A950 */
  0x0020,   /* U+A951 */
  0x0020,   /* U+A952 */
  0x0020,   /* U+A953 */
  0x0020,   /* U+A954 */
  0x0020,   /* U+A955 */
  0x0020,   /* U+A956 */
  0x0020,   /* U+A957 */
  0x0020,   /* U+A958 */
  0x0020,   /* U+A959 */
  0x0020,   /* U+A95A */
  0x0020,   /* U+A95B */
  0x0020,   /* U+A95C */
  0x0020,   /* U+A95D */
  0x0020,   /* U+A95E */
  0x0020,   /* U+A95F */
  0x0020,   /* U+A960 */
  0x0020,   /* U+A961 */
  0x0020,   /* U+A962 */
  0x0020,   /* U+A963 */
  0x0020,   /* U+A964 */
  0x0020,   /* U+A965 */
  0x0020,   /* U+A966 */
  0x0020,   /* U+A967 */
  0x0020,   /* U+A968 */
  0x0020,   /* U+A969 */
  0x0020,   /* U+A96A */
  0x0020,   /* U+A96B */
  0x0020,   /* U+A96C */
  0x0020,   /* U+A96D */
  0x0020,   /* U+A96E */
  0x0020,   /* U+A96F */
  0x0020,   /* U+A970 */
  0x0020,   /* U+A971 */
  0x0020,   /* U+A972 */
  0x0020,   /* U+A973 */
  0x0020,   /* U+A974 */
  0x0020,   /* U+A975 */
  0x0020,   /* U+A976 */
  0x0020,   /* U+A977 */
  0x0020,   /* U+A978 */
  0x0020,   /* U+A979 */
  0x0020,   /* U+A97A */
  0x0020,   /* U+A97B */
  0x0020,   /* U+A97C */
  0x0020,   /* U+A97D */
  0x0020,   /* U+A97E */
  0x0020,   /* U+A97F */
  0x00C3,   /* U+A980 */
  0x00C4,   /* U+A981 */
  0x00C9,   /* U+A982 */
  0x00C5,   /* U+A983 */
  0x0020,   /* U+A984 */
  0x0020,   /* U+A985 */
  0x0020,   /* U+A986 */
  0x0020,   /* U+A987 */
  0x0020,   /* U+A988 */
  0x0020,   /* U+A989 */
  0x0020,   /* U+A98A */
  0x0020,   /* U+A98B */
  0x0020,   /* U+A98C */
  0x0020,   /* U+A98D */
  0x0020,   /* U+A98E */
  0x0020,   /* U+A98F */
  0x0020,   /* U+A990 */
  0x0020,   /* U+A991 */
  0x0020,   /* U+A992 */
  0x0020,   /* U+A993 */
  0x0020,   /* U+A994 */
  0x0020,   /* U+A995 */
  0x0020,   /* U+A996 */
  0x0020,   /* U+A997 */
  0x0020,   /* U+A998 */
  0x0020,   /* U+A999 */
  0x0020,   /* U+A99A */
  0x0020,   /* U+A99B */
  0x0020,   /* U+A99C */
  0x0020,   /* U+A99D */
  0x0020,   /* U+A99E */
  0x0020,   /* U+A99F */
  0x0020,   /* U+A9A0 */
  0x0020,   /* U+A9A1 */
  0x0020,   /* U+A9A2 */
  0x0020,   /* U+A9A3 */
  0x0020,   /* U+A9A4 */
  0x0020,   /* U+A9A5 */
  0x0020,   /* U+A9A6 */
  0x0020,   /* U+A9A7 */
  0x0020,   /* U+A9A8 */
  0x0020,   /* U+A9A9 */
  0x0020,   /* U+A9AA */
  0x0020,   /* U+A9AB */
  0x0020,   /* U+A9AC */
  0x0020,   /* U+A9AD */
  0x0020,   /* U+A9AE */
  0x0020,   /* U+A9AF */
  0x0020,   /* U+A9B0 */
  0x0020,   /* U+A9B1 */
  0x0020,   /* U+A9B2 */
  0x00C2,   /* U+A9B3 */
  0x0020,   /* U+A9B4 */
  0x0020,   /* U+A9B5 */
  0x0020,   /* U+A9B6 */
  0x0020,   /* U+A9B7 */
  0x0020,   /* U+A9B8 */
  0x0020,   /* U+A9B9 */
  0x0020,   /* U+A9BA */
  0x0020,   /* U+A9BB */
  0x0020,   /* U+A9BC */
  0x0020,   /* U+A9BD */
  0x0020,   /* U+A9BE */
  0x0020,   /* U+A9BF */
  0x0020,   /* U+A9C0 */
  0x0020,   /* U+A9C1 */
  0x0020,   /* U+A9C2 */
  0x0020,   /* U+A9C3 */
  0x0020,   /* U+A9C4 */
  0x0020,   /* U+A9C5 */
  0x0020,   /* U+A9C6 */
  0x0020,   /* U+A9C7 */
  0x0020,   /* U+A9C8 */
  0x0020,   /* U+A9C9 */
  0x0020,   /* U+A9CA */
  0x0020,   /* U+A9CB */
  0x0020,   /* U+A9CC */
  0x0020,   /* U+A9CD */
  0x0020,   /* U+A9CE */
  0x0020,   /* U+A9CF */
  0x0020,   /* U+A9D0 */
  0x0020,   /* U+A9D1 */
  0x0020,   /* U+A9D2 */
  0x0020,   /* U+A9D3 */
  0x0020,   /* U+A9D4 */
  0x0020,   /* U+A9D5 */
  0x0020,   /* U+A9D6 */
  0x0020,   /* U+A9D7 */
  0x0020,   /* U+A9D8 */
  0x0020,   /* U+A9D9 */
  0x0020,   /* U+A9DA */
  0x0020,   /* U+A9DB */
  0x0020,   /* U+A9DC */
  0x0020,   /* U+A9DD */
  0x0020,   /* U+A9DE */
  0x0020,   /* U+A9DF */
  0x0020,   /* U+A9E0 */
  0x0020,   /* U+A9E1 */
  0x0020,   /* U+A9E2 */
  0x0020,   /* U+A9E3 */
  0x0020,   /* U+A9E4 */
  0x0020,   /* U+A9E5 */
  0x0020,   /* U+A9E6 */
  0x0020,   /* U+A9E7 */
  0x0020,   /* U+A9E8 */
  0x0020,   /* U+A9E9 */
  0x0020,   /* U+A9EA */
  0x0020,   /* U+A9EB */
  0x0020,   /* U+A9EC */
  0x0020,   /* U+A9ED */
  0x0020,   /* U+A9EE */
  0x0020,   /* U+A9EF */
  0x0020,   /* U+A9F0 */
  0x0020,   /* U+A9F1 */
  0x0020,   /* U+A9F2 */
  0x0020,   /* U+A9F3 */
  0x0020,   /* U+A9F4 */
  0x0020,   /* U+A9F5 */
  0x0020,   /* U+A9F6 */
  0x0020,   /* U+A9F7 */
  0x0020,   /* U+A9F8 */
  0x0020,   /* U+A9F9 */
  0x0020,   /* U+A9FA */
  0x0020,   /* U+A9FB */
  0x0020,   /* U+A9FC */
  0x0020,   /* U+A9FD */
  0x0020,   /* U+A9FE */
  0x0020,   /* U+A9FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+A900 */
  0x0002,   /* U+A901 */
  0x0002,   /* U+A902 */
  0x0002,   /* U+A903 */
  0x0002,   /* U+A904 */
  0x0002,   /* U+A905 */
  0x0002,   /* U+A906 */
  0x0002,   /* U+A907 */
  0x0002,   /* U+A908 */
  0x0002,   /* U+A909 */
  0x0002,   /* U+A90A */
  0x0002,   /* U+A90B */
  0x0002,   /* U+A90C */
  0x0002,   /* U+A90D */
  0x0002,   /* U+A90E */
  0x0002,   /* U+A90F */
  0x0002,   /* U+A910 */
  0x0002,   /* U+A911 */
  0x0002,   /* U+A912 */
  0x0002,   /* U+A913 */
  0x0002,   /* U+A914 */
  0x0002,   /* U+A915 */
  0x0002,   /* U+A916 */
  0x0002,   /* U+A917 */
  0x0002,   /* U+A918 */
  0x0002,   /* U+A919 */
  0x0002,   /* U+A91A */
  0x0002,   /* U+A91B */
  0x0002,   /* U+A91C */
  0x0002,   /* U+A91D */
  0x0002,   /* U+A91E */
  0x0002,   /* U+A91F */
  0x0002,   /* U+A920 */
  0x0002,   /* U+A921 */
  0x0002,   /* U+A922 */
  0x0002,   /* U+A923 */
  0x0002,   /* U+A924 */
  0x0002,   /* U+A925 */
  0x0002,   /* U+A926 */
  0x0002,   /* U+A927 */
  0x0002,   /* U+A928 */
  0x0002,   /* U+A929 */
  0x0002,   /* U+A92A */
  0x0002,   /* U+A92B */
  0x0002,   /* U+A92C */
  0x0002,   /* U+A92D */
  0x0002,   /* U+A92E */
  0x0002,   /* U+A92F */
  0x0002,   /* U+A930 */
  0x0002,   /* U+A931 */
  0x0002,   /* U+A932 */
  0x0002,   /* U+A933 */
  0x0002,   /* U+A934 */
  0x0002,   /* U+A935 */
  0x0002,   /* U+A936 */
  0x0002,   /* U+A937 */
  0x0002,   /* U+A938 */
  0x0002,   /* U+A939 */
  0x0002,   /* U+A93A */
  0x0002,   /* U+A93B */
  0x0002,   /* U+A93C */
  0x0002,   /* U+A93D */
  0x0002,   /* U+A93E */
  0x0002,   /* U+A93F */
  0x0002,   /* U+A940 */
  0x0002,   /* U+A941 */
  0x0002,   /* U+A942 */
  0x0002,   /* U+A943 */
  0x0002,   /* U+A944 */
  0x0002,   /* U+A945 */
  0x0002,   /* U+A946 */
  0x0002,   /* U+A947 */
  0x0002,   /* U+A948 */
  0x0002,   /* U+A949 */
  0x0002,   /* U+A94A */
  0x0002,   /* U+A94B */
  0x0002,   /* U+A94C */
  0x0002,   /* U+A94D */
  0x0002,   /* U+A94E */
  0x0002,   /* U+A94F */
  0x0002,   /* U+A950 */
  0x0002,   /* U+A951 */
  0x0002,   /* U+A952 */
  0x0002,   /* U+A953 */
  0x0002,   /* U+A954 */
  0x0002,   /* U+A955 */
  0x0002,   /* U+A956 */
  0x0002,   /* U+A957 */
  0x0002,   /* U+A958 */
  0x0002,   /* U+A959 */
  0x0002,   /* U+A95A */
  0x0002,   /* U+A95B */
  0x0002,   /* U+A95C */
  0x0002,   /* U+A95D */
  0x0002,   /* U+A95E */
  0x0002,   /* U+A95F */
  0x0002,   /* U+A960 */
  0x0002,   /* U+A961 */
  0x0002,   /* U+A962 */
  0x0002,   /* U+A963 */
  0x0002,   /* U+A964 */
  0x0002,   /* U+A965 */
  0x0002,   /* U+A966 */
  0x0002,   /* U+A967 */
  0x0002,   /* U+A968 */
  0x0002,   /* U+A969 */
  0x0002,   /* U+A96A */
  0x0002,   /* U+A96B */
  0x0002,   /* U+A96C */
  0x0002,   /* U+A96D */
  0x0002,   /* U+A96E */
  0x0002,   /* U+A96F */
  0x0002,   /* U+A970 */
  0x0002,   /* U+A971 */
  0x0002,   /* U+A972 */
  0x0002,   /* U+A973 */
  0x0002,   /* U+A974 */
  0x0002,   /* U+A975 */
  0x0002,   /* U+A976 */
  0x0002,   /* U+A977 */
  0x0002,   /* U+A978 */
  0x0002,   /* U+A979 */
  0x0002,   /* U+A97A */
  0x0002,   /* U+A97B */
  0x0002,   /* U+A97C */
  0x0002,   /* U+A97D */
  0x0002,   /* U+A97E */
  0x0002,   /* U+A97F */
  0x0002,   /* U+A980 */
  0x0002,   /* U+A981 */
  0x0002,   /* U+A982 */
  0x0002,   /* U+A983 */
  0x0002,   /* U+A984 */
  0x0002,   /* U+A985 */
  0x0002,   /* U+A986 */
  0x0002,   /* U+A987 */
  0x0002,   /* U+A988 */
  0x0002,   /* U+A989 */
  0x0002,   /* U+A98A */
  0x0002,   /* U+A98B */
  0x0002,   /* U+A98C */
  0x0002,   /* U+A98D */
  0x0002,   /* U+A98E */
  0x0002,   /* U+A98F */
  0x0002,   /* U+A990 */
  0x0002,   /* U+A991 */
  0x0002,   /* U+A992 */
  0x0002,   /* U+A993 */
  0x0002,   /* U+A994 */
  0x0002,   /* U+A995 */
  0x0002,   /* U+A996 */
  0x0002,   /* U+A997 */
  0x0002,   /* U+A998 */
  0x0002,   /* U+A999 */
  0x0002,   /* U+A99A */
  0x0002,   /* U+A99B */
  0x0002,   /* U+A99C */
  0x0002,   /* U+A99D */
  0x0002,   /* U+A99E */
  0x0002,   /* U+A99F */
  0x0002,   /* U+A9A0 */
  0x0002,   /* U+A9A1 */
  0x0002,   /* U+A9A2 */
  0x0002,   /* U+A9A3 */
  0x0002,   /* U+A9A4 */
  0x0002,   /* U+A9A5 */
  0x0002,   /* U+A9A6 */
  0x0002,   /* U+A9A7 */
  0x0002,   /* U+A9A8 */
  0x0002,   /* U+A9A9 */
  0x0002,   /* U+A9AA */
  0x0002,   /* U+A9AB */
  0x0004,   /* U+A9AC */
  0x0002,   /* U+A9AD */
  0x0002,   /* U+A9AE */
  0x0002,   /* U+A9AF */
  0x0002,   /* U+A9B0 */
  0x0002,   /* U+A9B1 */
  0x0002,   /* U+A9B2 */
  0x0002,   /* U+A9B3 */
  0x0002,   /* U+A9B4 */
  0x0002,   /* U+A9B5 */
  0x0002,   /* U+A9B6 */
  0x0002,   /* U+A9B7 */
  0x0002,   /* U+A9B8 */
  0x0002,   /* U+A9B9 */
  0x0002,   /* U+A9BA */
  0x0002,   /* U+A9BB */
  0x0002,   /* U+A9BC */
  0x0002,   /* U+A9BD */
  0x0002,   /* U+A9BE */
  0x0002,   /* U+A9BF */
  0x0002,   /* U+A9C0 */
  0x0002,   /* U+A9C1 */
  0x0002,   /* U+A9C2 */
  0x0002,   /* U+A9C3 */
  0x0002,   /* U+A9C4 */
  0x0002,   /* U+A9C5 */
  0x0002,   /* U+A9C6 */
  0x0002,   /* U+A9C7 */
  0x0002,   /* U+A9C8 */
  0x0002,   /* U+A9C9 */
  0x0002,   /* U+A9CA */
  0x0002,   /* U+A9CB */
  0x0002,   /* U+A9CC */
  0x0002,   /* U+A9CD */
  0x0002,   /* U+A9CE */
  0x0002,   /* U+A9CF */
  0x0002,   /* U+A9D0 */
  0x0002,   /* U+A9D1 */
  0x0002,   /* U+A9D2 */
  0x0002,   /* U+A9D3 */
  0x0002,   /* U+A9D4 */
  0x0002,   /* U+A9D5 */
  0x0002,   /* U+A9D6 */
  0x0002,   /* U+A9D7 */
  0x0002,   /* U+A9D8 */
  0x0002,   /* U+A9D9 */
  0x0002,   /* U+A9DA */
  0x0002,   /* U+A9DB */
  0x0002,   /* U+A9DC */
  0x0002,   /* U+A9DD */
  0x0002,   /* U+A9DE */
  0x0002,   /* U+A9DF */
  0x0002,   /* U+A9E0 */
  0x0002,   /* U+A9E1 */
  0x0002,   /* U+A9E2 */
  0x0002,   /* U+A9E3 */
  0x0002,   /* U+A9E4 */
  0x0002,   /* U+A9E5 */
  0x0002,   /* U+A9E6 */
  0x0002,   /* U+A9E7 */
  0x0002,   /* U+A9E8 */
  0x0002,   /* U+A9E9 */
  0x0002,   /* U+A9EA */
  0x0002,   /* U+A9EB */
  0x0002,   /* U+A9EC */
  0x0002,   /* U+A9ED */
  0x0002,   /* U+A9EE */
  0x0002,   /* U+A9EF */
  0x0002,   /* U+A9F0 */
  0x0002,   /* U+A9F1 */
  0x0002,   /* U+A9F2 */
  0x0002,   /* U+A9F3 */
  0x0002,   /* U+A9F4 */
  0x0002,   /* U+A9F5 */
  0x0002,   /* U+A9F6 */
  0x0002,   /* U+A9F7 */
  0x0002,   /* U+A9F8 */
  0x0002,   /* U+A9F9 */
  0x0002,   /* U+A9FA */
  0x0002,   /* U+A9FB */
  0x0002,   /* U+A9FC */
  0x0002,   /* U+A9FD */
  0x0002,   /* U+A9FE */
  0x0002,   /* U+A9FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+A900 */
  0x0000,   /* U+A901 */
  0x0000,   /* U+A902 */
  0x0000,   /* U+A903 */
  0x0000,   /* U+A904 */
  0x0000,   /* U+A905 */
  0x0000,   /* U+A906 */
  0x0000,   /* U+A907 */
  0x0000,   /* U+A908 */
  0x0000,   /* U+A909 */
  0x0000,   /* U+A90A */
  0x0000,   /* U+A90B */
  0x0000,   /* U+A90C */
  0x0000,   /* U+A90D */
  0x0000,   /* U+A90E */
  0x0000,   /* U+A90F */
  0x0000,   /* U+A910 */
  0x0000,   /* U+A911 */
  0x0000,   /* U+A912 */
  0x0000,   /* U+A913 */
  0x0000,   /* U+A914 */
  0x0000,   /* U+A915 */
  0x0000,   /* U+A916 */
  0x0000,   /* U+A917 */
  0x0000,   /* U+A918 */
  0x0000,   /* U+A919 */
  0x0000,   /* U+A91A */
  0x0000,   /* U+A91B */
  0x0000,   /* U+A91C */
  0x0000,   /* U+A91D */
  0x0000,   /* U+A91E */
  0x0000,   /* U+A91F */
  0x0000,   /* U+A920 */
  0x0000,   /* U+A921 */
  0x0000,   /* U+A922 */
  0x0000,   /* U+A923 */
  0x0000,   /* U+A924 */
  0x0000,   /* U+A925 */
  0x0000,   /* U+A926 */
  0x0000,   /* U+A927 */
  0x0000,   /* U+A928 */
  0x0000,   /* U+A929 */
  0x0000,   /* U+A92A */
  0x0000,   /* U+A92B */
  0x0000,   /* U+A92C */
  0x0000,   /* U+A92D */
  0x0000,   /* U+A92E */
  0x0000,   /* U+A92F */
  0x0000,   /* U+A930 */
  0x0000,   /* U+A931 */
  0x0000,   /* U+A932 */
  0x0000,   /* U+A933 */
  0x0000,   /* U+A934 */
  0x0000,   /* U+A935 */
  0x0000,   /* U+A936 */
  0x0000,   /* U+A937 */
  0x0000,   /* U+A938 */
  0x0000,   /* U+A939 */
  0x0000,   /* U+A93A */
  0x0000,   /* U+A93B */
  0x0000,   /* U+A93C */
  0x0000,   /* U+A93D */
  0x0000,   /* U+A93E */
  0x0000,   /* U+A93F */
  0x0000,   /* U+A940 */
  0x0000,   /* U+A941 */
  0x0000,   /* U+A942 */
  0x0000,   /* U+A943 */
  0x0000,   /* U+A944 */
  0x0000,   /* U+A945 */
  0x0000,   /* U+A946 */
  0x0000,   /* U+A947 */
  0x0000,   /* U+A948 */
  0x0000,   /* U+A949 */
  0x0000,   /* U+A94A */
  0x0000,   /* U+A94B */
  0x0000,   /* U+A94C */
  0x0000,   /* U+A94D */
  0x0000,   /* U+A94E */
  0x0000,   /* U+A94F */
  0x0000,   /* U+A950 */
  0x0000,   /* U+A951 */
  0x0000,   /* U+A952 */
  0x0000,   /* U+A953 */
  0xA954,   /* U+A954 */
  0xA955,   /* U+A955 */
  0xA956,   /* U+A956 */
  0xA957,   /* U+A957 */
  0xA958,   /* U+A958 */
  0xA959,   /* U+A959 */
  0xA95A,   /* U+A95A */
  0xA95B,   /* U+A95B */
  0xA95C,   /* U+A95C */
  0xA95D,   /* U+A95D */
  0xA95E,   /* U+A95E */
  0x0000,   /* U+A95F */
  0x0000,   /* U+A960 */
  0x0000,   /* U+A961 */
  0x0000,   /* U+A962 */
  0x0000,   /* U+A963 */
  0x0000,   /* U+A964 */
  0x0000,   /* U+A965 */
  0x0000,   /* U+A966 */
  0x0000,   /* U+A967 */
  0x0000,   /* U+A968 */
  0x0000,   /* U+A969 */
  0x0000,   /* U+A96A */
  0x0000,   /* U+A96B */
  0x0000,   /* U+A96C */
  0x0000,   /* U+A96D */
  0x0000,   /* U+A96E */
  0x0000,   /* U+A96F */
  0x0000,   /* U+A970 */
  0x0000,   /* U+A971 */
  0x0000,   /* U+A972 */
  0x0000,   /* U+A973 */
  0x0000,   /* U+A974 */
  0x0000,   /* U+A975 */
  0x0000,   /* U+A976 */
  0x0000,   /* U+A977 */
  0x0000,   /* U+A978 */
  0x0000,   /* U+A979 */
  0x0000,   /* U+A97A */
  0x0000,   /* U+A97B */
  0x0000,   /* U+A97C */
  0xA97D,   /* U+A97D */
  0xA97E,   /* U+A97E */
  0xA97F,   /* U+A97F */
  0x0000,   /* U+A980 */
  0x0000,   /* U+A981 */
  0x0000,   /* U+A982 */
  0x0000,   /* U+A983 */
  0x0000,   /* U+A984 */
  0x0000,   /* U+A985 */
  0x0000,   /* U+A986 */
  0x0000,   /* U+A987 */
  0x0000,   /* U+A988 */
  0x0000,   /* U+A989 */
  0x0000,   /* U+A98A */
  0x0000,   /* U+A98B */
  0x0000,   /* U+A98C */
  0x0000,   /* U+A98D */
  0x0000,   /* U+A98E */
  0x0000,   /* U+A98F */
  0x0000,   /* U+A990 */
  0x0000,   /* U+A991 */
  0x0000,   /* U+A992 */
  0x0000,   /* U+A993 */
  0x0000,   /* U+A994 */
  0x0000,   /* U+A995 */
  0x0000,   /* U+A996 */
  0x0000,   /* U+A997 */
  0x0000,   /* U+A998 */
  0x0000,   /* U+A999 */
  0x0000,   /* U+A99A */
  0x0000,   /* U+A99B */
  0x0000,   /* U+A99C */
  0x0000,   /* U+A99D */
  0x0000,   /* U+A99E */
  0x0000,   /* U+A99F */
  0x0000,   /* U+A9A0 */
  0x0000,   /* U+A9A1 */
  0x0000,   /* U+A9A2 */
  0x0000,   /* U+A9A3 */
  0x0000,   /* U+A9A4 */
  0x0000,   /* U+A9A5 */
  0x0000,   /* U+A9A6 */
  0x0000,   /* U+A9A7 */
  0x0000,   /* U+A9A8 */
  0x0000,   /* U+A9A9 */
  0x0000,   /* U+A9AA */
  0x0000,   /* U+A9AB */
  0x0000,   /* U+A9AC */
  0x0000,   /* U+A9AD */
  0x0000,   /* U+A9AE */
  0x0000,   /* U+A9AF */
  0x0000,   /* U+A9B0 */
  0x0000,   /* U+A9B1 */
  0x0000,   /* U+A9B2 */
  0x0000,   /* U+A9B3 */
  0x0000,   /* U+A9B4 */
  0x0000,   /* U+A9B5 */
  0x0000,   /* U+A9B6 */
  0x0000,   /* U+A9B7 */
  0x0000,   /* U+A9B8 */
  0x0000,   /* U+A9B9 */
  0x0000,   /* U+A9BA */
  0x0000,   /* U+A9BB */
  0x0000,   /* U+A9BC */
  0x0000,   /* U+A9BD */
  0x0000,   /* U+A9BE */
  0x0000,   /* U+A9BF */
  0x0000,   /* U+A9C0 */
  0x0000,   /* U+A9C1 */
  0x0000,   /* U+A9C2 */
  0x0000,   /* U+A9C3 */
  0x0000,   /* U+A9C4 */
  0x0000,   /* U+A9C5 */
  0x0000,   /* U+A9C6 */
  0x0000,   /* U+A9C7 */
  0x0000,   /* U+A9C8 */
  0x0000,   /* U+A9C9 */
  0x0000,   /* U+A9CA */
  0x0000,   /* U+A9CB */
  0x0000,   /* U+A9CC */
  0x0000,   /* U+A9CD */
  0xA9CE,   /* U+A9CE */
  0x0000,   /* U+A9CF */
  0x0000,   /* U+A9D0 */
  0x0000,   /* U+A9D1 */
  0x0000,   /* U+A9D2 */
  0x0000,   /* U+A9D3 */
  0x0000,   /* U+A9D4 */
  0x0000,   /* U+A9D5 */
  0x0000,   /* U+A9D6 */
  0x0000,   /* U+A9D7 */
  0x0000,   /* U+A9D8 */
  0x0000,   /* U+A9D9 */
  0xA9DA,   /* U+A9DA */
  0xA9DB,   /* U+A9DB */
  0xA9DC,   /* U+A9DC */
  0xA9DD,   /* U+A9DD */
  0x0000,   /* U+A9DE */
  0x0000,   /* U+A9DF */
  0x0000,   /* U+A9E0 */
  0x0000,   /* U+A9E1 */
  0x0000,   /* U+A9E2 */
  0x0000,   /* U+A9E3 */
  0x0000,   /* U+A9E4 */
  0x0000,   /* U+A9E5 */
  0x0000,   /* U+A9E6 */
  0x0000,   /* U+A9E7 */
  0x0000,   /* U+A9E8 */
  0x0000,   /* U+A9E9 */
  0x0000,   /* U+A9EA */
  0x0000,   /* U+A9EB */
  0x0000,   /* U+A9EC */
  0x0000,   /* U+A9ED */
  0x0000,   /* U+A9EE */
  0x0000,   /* U+A9EF */
  0x0000,   /* U+A9F0 */
  0x0000,   /* U+A9F1 */
  0x0000,   /* U+A9F2 */
  0x0000,   /* U+A9F3 */
  0x0000,   /* U+A9F4 */
  0x0000,   /* U+A9F5 */
  0x0000,   /* U+A9F6 */
  0x0000,   /* U+A9F7 */
  0x0000,   /* U+A9F8 */
  0x0000,   /* U+A9F9 */
  0x0000,   /* U+A9FA */
  0x0000,   /* U+A9FB */
  0x0000,   /* U+A9FC */
  0x0000,   /* U+A9FD */
  0x0000,   /* U+A9FE */
  0xA9FF,   /* U+A9FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+A900 */
  0x0000,   /* U+A901 */
  0x0000,   /* U+A902 */
  0x0000,   /* U+A903 */
  0x0000,   /* U+A904 */
  0x0000,   /* U+A905 */
  0x0000,   /* U+A906 */
  0x0000,   /* U+A907 */
  0x0000,   /* U+A908 */
  0x0000,   /* U+A909 */
  0x0000,   /* U+A90A */
  0x0000,   /* U+A90B */
  0x0000,   /* U+A90C */
  0x0000,   /* U+A90D */
  0x0000,   /* U+A90E */
  0x0000,   /* U+A90F */
  0x0000,   /* U+A910 */
  0x0000,   /* U+A911 */
  0x0000,   /* U+A912 */
  0x0000,   /* U+A913 */
  0x0000,   /* U+A914 */
  0x0000,   /* U+A915 */
  0x0000,   /* U+A916 */
  0x0000,   /* U+A917 */
  0x0000,   /* U+A918 */
  0x0000,   /* U+A919 */
  0x0000,   /* U+A91A */
  0x0000,   /* U+A91B */
  0x0000,   /* U+A91C */
  0x0000,   /* U+A91D */
  0x0000,   /* U+A91E */
  0x0000,   /* U+A91F */
  0x0000,   /* U+A920 */
  0x0000,   /* U+A921 */
  0x0000,   /* U+A922 */
  0x0000,   /* U+A923 */
  0x0000,   /* U+A924 */
  0x0000,   /* U+A925 */
  0x0000,   /* U+A926 */
  0x0000,   /* U+A927 */
  0x0000,   /* U+A928 */
  0x0000,   /* U+A929 */
  0x0000,   /* U+A92A */
  0x0000,   /* U+A92B */
  0x0000,   /* U+A92C */
  0x0000,   /* U+A92D */
  0x0000,   /* U+A92E */
  0x0000,   /* U+A92F */
  0x0000,   /* U+A930 */
  0x0000,   /* U+A931 */
  0x0000,   /* U+A932 */
  0x0000,   /* U+A933 */
  0x0000,   /* U+A934 */
  0x0000,   /* U+A935 */
  0x0000,   /* U+A936 */
  0x0000,   /* U+A937 */
  0x0000,   /* U+A938 */
  0x0000,   /* U+A939 */
  0x0000,   /* U+A93A */
  0x0000,   /* U+A93B */
  0x0000,   /* U+A93C */
  0x0000,   /* U+A93D */
  0x0000,   /* U+A93E */
  0x0000,   /* U+A93F */
  0x0000,   /* U+A940 */
  0x0000,   /* U+A941 */
  0x0000,   /* U+A942 */
  0x0000,   /* U+A943 */
  0x0000,   /* U+A944 */
  0x0000,   /* U+A945 */
  0x0000,   /* U+A946 */
  0x0000,   /* U+A947 */
  0x0000,   /* U+A948 */
  0x0000,   /* U+A949 */
  0x0000,   /* U+A94A */
  0x0000,   /* U+A94B */
  0x0000,   /* U+A94C */
  0x0000,   /* U+A94D */
  0x0000,   /* U+A94E */
  0x0000,   /* U+A94F */
  0x0000,   /* U+A950 */
  0x0000,   /* U+A951 */
  0x0000,   /* U+A952 */
  0x0000,   /* U+A953 */
  0x0000,   /* U+A954 */
  0x0000,   /* U+A955 */
  0x0000,   /* U+A956 */
  0x0000,   /* U+A957 */
  0x0000,   /* U+A958 */
  0x0000,   /* U+A959 */
  0x0000,   /* U+A95A */
  0x0000,   /* U+A95B */
  0x0000,   /* U+A95C */
  0x0000,   /* U+A95D */
  0x0000,   /* U+A95E */
  0x0000,   /* U+A95F */
  0x0000,   /* U+A960 */
  0x0000,   /* U+A961 */
  0x0000,   /* U+A962 */
  0x0000,   /* U+A963 */
  0x0000,   /* U+A964 */
  0x0000,   /* U+A965 */
  0x0000,   /* U+A966 */
  0x0000,   /* U+A967 */
  0x0000,   /* U+A968 */
  0x0000,   /* U+A969 */
  0x0000,   /* U+A96A */
  0x0000,   /* U+A96B */
  0x0000,   /* U+A96C */
  0x0000,   /* U+A96D */
  0x0000,   /* U+A96E */
  0x0000,   /* U+A96F */
  0x0000,   /* U+A970 */
  0x0000,   /* U+A971 */
  0x0000,   /* U+A972 */
  0x0000,   /* U+A973 */
  0x0000,   /* U+A974 */
  0x0000,   /* U+A975 */
  0x0000,   /* U+A976 */
  0x0000,   /* U+A977 */
  0x0000,   /* U+A978 */
  0x0000,   /* U+A979 */
  0x0000,   /* U+A97A */
  0x0000,   /* U+A97B */
  0x0000,   /* U+A97C */
  0x0000,   /* U+A97D */
  0x0000,   /* U+A97E */
  0x0000,   /* U+A97F */
  0x0000,   /* U+A980 */
  0x0000,   /* U+A981 */
  0x0000,   /* U+A982 */
  0x0000,   /* U+A983 */
  0x0000,   /* U+A984 */
  0x0000,   /* U+A985 */
  0x0000,   /* U+A986 */
  0x0000,   /* U+A987 */
  0x0000,   /* U+A988 */
  0x0000,   /* U+A989 */
  0x0000,   /* U+A98A */
  0x0000,   /* U+A98B */
  0x0000,   /* U+A98C */
  0x0000,   /* U+A98D */
  0x0000,   /* U+A98E */
  0x0000,   /* U+A98F */
  0x0000,   /* U+A990 */
  0x0000,   /* U+A991 */
  0x0000,   /* U+A992 */
  0x0000,   /* U+A993 */
  0x0000,   /* U+A994 */
  0x0000,   /* U+A995 */
  0x0000,   /* U+A996 */
  0x0000,   /* U+A997 */
  0x0000,   /* U+A998 */
  0x0000,   /* U+A999 */
  0x0000,   /* U+A99A */
  0x0000,   /* U+A99B */
  0x0000,   /* U+A99C */
  0x0000,   /* U+A99D */
  0x0000,   /* U+A99E */
  0x0000,   /* U+A99F */
  0x0000,   /* U+A9A0 */
  0x0000,   /* U+A9A1 */
  0x0000,   /* U+A9A2 */
  0x0000,   /* U+A9A3 */
  0x0000,   /* U+A9A4 */
  0x0000,   /* U+A9A5 */
  0x0000,   /* U+A9A6 */
  0x0000,   /* U+A9A7 */
  0x0000,   /* U+A9A8 */
  0x0000,   /* U+A9A9 */
  0x0000,   /* U+A9AA */
  0x0000,   /* U+A9AB */
  0x0000,   /* U+A9AC */
  0x0000,   /* U+A9AD */
  0x0000,   /* U+A9AE */
  0x0000,   /* U+A9AF */
  0x0000,   /* U+A9B0 */
  0x0000,   /* U+A9B1 */
  0x0000,   /* U+A9B2 */
  0x0000,   /* U+A9B3 */
  0x0000,   /* U+A9B4 */
  0x0000,   /* U+A9B5 */
  0x0000,   /* U+A9B6 */
  0x0000,   /* U+A9B7 */
  0x0000,   /* U+A9B8 */
  0x0000,   /* U+A9B9 */
  0x0000,   /* U+A9BA */
  0x0000,   /* U+A9BB */
  0x0000,   /* U+A9BC */
  0x0000,   /* U+A9BD */
  0x0000,   /* U+A9BE */
  0x0000,   /* U+A9BF */
  0x0000,   /* U+A9C0 */
  0x0000,   /* U+A9C1 */
  0x0000,   /* U+A9C2 */
  0x0000,   /* U+A9C3 */
  0x0000,   /* U+A9C4 */
  0x0000,   /* U+A9C5 */
  0x0000,   /* U+A9C6 */
  0x0000,   /* U+A9C7 */
  0x0000,   /* U+A9C8 */
  0x0000,   /* U+A9C9 */
  0x0000,   /* U+A9CA */
  0x0000,   /* U+A9CB */
  0x0000,   /* U+A9CC */
  0x0000,   /* U+A9CD */
  0x0000,   /* U+A9CE */
  0x0000,   /* U+A9CF */
  0x0000,   /* U+A9D0 */
  0x0000,   /* U+A9D1 */
  0x0000,   /* U+A9D2 */
  0x0000,   /* U+A9D3 */
  0x0000,   /* U+A9D4 */
  0x0000,   /* U+A9D5 */
  0x0000,   /* U+A9D6 */
  0x0000,   /* U+A9D7 */
  0x0000,   /* U+A9D8 */
  0x0000,   /* U+A9D9 */
  0x0000,   /* U+A9DA */
  0x0000,   /* U+A9DB */
  0x0000,   /* U+A9DC */
  0x0000,   /* U+A9DD */
  0x0000,   /* U+A9DE */
  0x0000,   /* U+A9DF */
  0x0000,   /* U+A9E0 */
  0x0000,   /* U+A9E1 */
  0x0000,   /* U+A9E2 */
  0x0000,   /* U+A9E3 */
  0x0000,   /* U+A9E4 */
  0x0000,   /* U+A9E5 */
  0x0000,   /* U+A9E6 */
  0x0000,   /* U+A9E7 */
  0x0000,   /* U+A9E8 */
  0x0000,   /* U+A9E9 */
  0x0000,   /* U+A9EA */
  0x0000,   /* U+A9EB */
  0x0000,   /* U+A9EC */
  0x0000,   /* U+A9ED */
  0x0000,   /* U+A9EE */
  0x0000,   /* U+A9EF */
  0x0000,   /* U+A9F0 */
  0x0000,   /* U+A9F1 */
  0x0000,   /* U+A9F2 */
  0x0000,   /* U+A9F3 */
  0x0000,   /* U+A9F4 */
  0x0000,   /* U+A9F5 */
  0x0000,   /* U+A9F6 */
  0x0000,   /* U+A9F7 */
  0x0000,   /* U+A9F8 */
  0x0000,   /* U+A9F9 */
  0x0000,   /* U+A9FA */
  0x0000,   /* U+A9FB */
  0x0000,   /* U+A9FC */
  0x0000,   /* U+A9FD */
  0x0000,   /* U+A9FE */
  0x0000,   /* U+A9FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+A900 */
  0x0000,   /* U+A901 */
  0x0000,   /* U+A902 */
  0x0000,   /* U+A903 */
  0x0000,   /* U+A904 */
  0x0000,   /* U+A905 */
  0x0000,   /* U+A906 */
  0x0000,   /* U+A907 */
  0x0000,   /* U+A908 */
  0x0000,   /* U+A909 */
  0x0000,   /* U+A90A */
  0x0000,   /* U+A90B */
  0x0000,   /* U+A90C */
  0x0000,   /* U+A90D */
  0x0000,   /* U+A90E */
  0x0000,   /* U+A90F */
  0x0000,   /* U+A910 */
  0x0000,   /* U+A911 */
  0x0000,   /* U+A912 */
  0x0000,   /* U+A913 */
  0x0000,   /* U+A914 */
  0x0000,   /* U+A915 */
  0x0000,   /* U+A916 */
  0x0000,   /* U+A917 */
  0x0000,   /* U+A918 */
  0x0000,   /* U+A919 */
  0x0000,   /* U+A91A */
  0x0000,   /* U+A91B */
  0x0000,   /* U+A91C */
  0x0000,   /* U+A91D */
  0x0000,   /* U+A91E */
  0x0000,   /* U+A91F */
  0x0000,   /* U+A920 */
  0x0000,   /* U+A921 */
  0x0000,   /* U+A922 */
  0x0000,   /* U+A923 */
  0x0000,   /* U+A924 */
  0x0000,   /* U+A925 */
  0x0000,   /* U+A926 */
  0x0000,   /* U+A927 */
  0x0000,   /* U+A928 */
  0x0000,   /* U+A929 */
  0x0000,   /* U+A92A */
  0x0000,   /* U+A92B */
  0x0000,   /* U+A92C */
  0x0000,   /* U+A92D */
  0x0000,   /* U+A92E */
  0x0000,   /* U+A92F */
  0x0000,   /* U+A930 */
  0x0000,   /* U+A931 */
  0x0000,   /* U+A932 */
  0x0000,   /* U+A933 */
  0x0000,   /* U+A934 */
  0x0000,   /* U+A935 */
  0x0000,   /* U+A936 */
  0x0000,   /* U+A937 */
  0x0000,   /* U+A938 */
  0x0000,   /* U+A939 */
  0x0000,   /* U+A93A */
  0x0000,   /* U+A93B */
  0x0000,   /* U+A93C */
  0x0000,   /* U+A93D */
  0x0000,   /* U+A93E */
  0x0000,   /* U+A93F */
  0x0000,   /* U+A940 */
  0x0000,   /* U+A941 */
  0x0000,   /* U+A942 */
  0x0000,   /* U+A943 */
  0x0000,   /* U+A944 */
  0x0000,   /* U+A945 */
  0x0000,   /* U+A946 */
  0x0000,   /* U+A947 */
  0x0000,   /* U+A948 */
  0x0000,   /* U+A949 */
  0x0000,   /* U+A94A */
  0x0000,   /* U+A94B */
  0x0000,   /* U+A94C */
  0x0000,   /* U+A94D */
  0x0000,   /* U+A94E */
  0x0000,   /* U+A94F */
  0x0000,   /* U+A950 */
  0x0000,   /* U+A951 */
  0x0000,   /* U+A952 */
  0x0000,   /* U+A953 */
  0x0000,   /* U+A954 */
  0x0000,   /* U+A955 */
  0x0000,   /* U+A956 */
  0x0000,   /* U+A957 */
  0x0000,   /* U+A958 */
  0x0000,   /* U+A959 */
  0x0000,   /* U+A95A */
  0x0000,   /* U+A95B */
  0x0000,   /* U+A95C */
  0x0000,   /* U+A95D */
  0x0000,   /* U+A95E */
  0x0000,   /* U+A95F */
  0x0000,   /* U+A960 */
  0x0000,   /* U+A961 */
  0x0000,   /* U+A962 */
  0x0000,   /* U+A963 */
  0x0000,   /* U+A964 */
  0x0000,   /* U+A965 */
  0x0000,   /* U+A966 */
  0x0000,   /* U+A967 */
  0x0000,   /* U+A968 */
  0x0000,   /* U+A969 */
  0x0000,   /* U+A96A */
  0x0000,   /* U+A96B */
  0x0000,   /* U+A96C */
  0x0000,   /* U+A96D */
  0x0000,   /* U+A96E */
  0x0000,   /* U+A96F */
  0x0000,   /* U+A970 */
  0x0000,   /* U+A971 */
  0x0000,   /* U+A972 */
  0x0000,   /* U+A973 */
  0x0000,   /* U+A974 */
  0x0000,   /* U+A975 */
  0x0000,   /* U+A976 */
  0x0000,   /* U+A977 */
  0x0000,   /* U+A978 */
  0x0000,   /* U+A979 */
  0x0000,   /* U+A97A */
  0x0000,   /* U+A97B */
  0x0000,   /* U+A97C */
  0x0000,   /* U+A97D */
  0x0000,   /* U+A97E */
  0x0000,   /* U+A97F */
  0x0000,   /* U+A980 */
  0x0000,   /* U+A981 */
  0x0000,   /* U+A982 */
  0x0000,   /* U+A983 */
  0x0000,   /* U+A984 */
  0x0000,   /* U+A985 */
  0x0000,   /* U+A986 */
  0x0000,   /* U+A987 */
  0x0000,   /* U+A988 */
  0x0000,   /* U+A989 */
  0x0000,   /* U+A98A */
  0x0000,   /* U+A98B */
  0x0000,   /* U+A98C */
  0x0000,   /* U+A98D */
  0x0000,   /* U+A98E */
  0x0000,   /* U+A98F */
  0x0000,   /* U+A990 */
  0x0000,   /* U+A991 */
  0x0000,   /* U+A992 */
  0x0000,   /* U+A993 */
  0x0000,   /* U+A994 */
  0x0000,   /* U+A995 */
  0x0000,   /* U+A996 */
  0x0000,   /* U+A997 */
  0x0000,   /* U+A998 */
  0x0000,   /* U+A999 */
  0x0000,   /* U+A99A */
  0x0000,   /* U+A99B */
  0x0000,   /* U+A99C */
  0x0000,   /* U+A99D */
  0x0000,   /* U+A99E */
  0x0000,   /* U+A99F */
  0x0000,   /* U+A9A0 */
  0x0000,   /* U+A9A1 */
  0x0000,   /* U+A9A2 */
  0x0000,   /* U+A9A3 */
  0x0000,   /* U+A9A4 */
  0x0000,   /* U+A9A5 */
  0x0000,   /* U+A9A6 */
  0x0000,   /* U+A9A7 */
  0x0000,   /* U+A9A8 */
  0x0000,   /* U+A9A9 */
  0x0000,   /* U+A9AA */
  0x0000,   /* U+A9AB */
  0x0000,   /* U+A9AC */
  0x0000,   /* U+A9AD */
  0x0000,   /* U+A9AE */
  0x0000,   /* U+A9AF */
  0x0000,   /* U+A9B0 */
  0x0000,   /* U+A9B1 */
  0x0000,   /* U+A9B2 */
  0x0000,   /* U+A9B3 */
  0x0000,   /* U+A9B4 */
  0x0000,   /* U+A9B5 */
  0x0000,   /* U+A9B6 */
  0x0000,   /* U+A9B7 */
  0x0000,   /* U+A9B8 */
  0x0000,   /* U+A9B9 */
  0x0000,   /* U+A9BA */
  0x0000,   /* U+A9BB */
  0x0000,   /* U+A9BC */
  0x0000,   /* U+A9BD */
  0x0000,   /* U+A9BE */
  0x0000,   /* U+A9BF */
  0x0000,   /* U+A9C0 */
  0x0000,   /* U+A9C1 */
  0x0000,   /* U+A9C2 */
  0x0000,   /* U+A9C3 */
  0x0000,   /* U+A9C4 */
  0x0000,   /* U+A9C5 */
  0x0000,   /* U+A9C6 */
  0x0000,   /* U+A9C7 */
  0x0000,   /* U+A9C8 */
  0x0000,   /* U+A9C9 */
  0x0000,   /* U+A9CA */
  0x0000,   /* U+A9CB */
  0x0000,   /* U+A9CC */
  0x0000,   /* U+A9CD */
  0x0000,   /* U+A9CE */
  0x0000,   /* U+A9CF */
  0x0000,   /* U+A9D0 */
  0x0000,   /* U+A9D1 */
  0x0000,   /* U+A9D2 */
  0x0000,   /* U+A9D3 */
  0x0000,   /* U+A9D4 */
  0x0000,   /* U+A9D5 */
  0x0000,   /* U+A9D6 */
  0x0000,   /* U+A9D7 */
  0x0000,   /* U+A9D8 */
  0x0000,   /* U+A9D9 */
  0x0000,   /* U+A9DA */
  0x0000,   /* U+A9DB */
  0x0000,   /* U+A9DC */
  0x0000,   /* U+A9DD */
  0x0000,   /* U+A9DE */
  0x0000,   /* U+A9DF */
  0x0000,   /* U+A9E0 */
  0x0000,   /* U+A9E1 */
  0x0000,   /* U+A9E2 */
  0x0000,   /* U+A9E3 */
  0x0000,   /* U+A9E4 */
  0x0000,   /* U+A9E5 */
  0x0000,   /* U+A9E6 */
  0x0000,   /* U+A9E7 */
  0x0000,   /* U+A9E8 */
  0x0000,   /* U+A9E9 */
  0x0000,   /* U+A9EA */
  0x0000,   /* U+A9EB */
  0x0000,   /* U+A9EC */
  0x0000,   /* U+A9ED */
  0x0000,   /* U+A9EE */
  0x0000,   /* U+A9EF */
  0x0000,   /* U+A9F0 */
  0x0000,   /* U+A9F1 */
  0x0000,   /* U+A9F2 */
  0x0000,   /* U+A9F3 */
  0x0000,   /* U+A9F4 */
  0x0000,   /* U+A9F5 */
  0x0000,   /* U+A9F6 */
  0x0000,   /* U+A9F7 */
  0x0000,   /* U+A9F8 */
  0x0000,   /* U+A9F9 */
  0x0000,   /* U+A9FA */
  0x0000,   /* U+A9FB */
  0x0000,   /* U+A9FC */
  0x0000,   /* U+A9FD */
  0x0000,   /* U+A9FE */
  0x0000,   /* U+A9FF */
};

uint16 uca900_p0AA[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3212,   /* U+AA00 */
  0x3213,   /* U+AA01 */
  0x3214,   /* U+AA02 */
  0x3215,   /* U+AA03 */
  0x3216,   /* U+AA04 */
  0x3217,   /* U+AA05 */
  0x3218,   /* U+AA06 */
  0x3219,   /* U+AA07 */
  0x321A,   /* U+AA08 */
  0x321B,   /* U+AA09 */
  0x321C,   /* U+AA0A */
  0x321D,   /* U+AA0B */
  0x321E,   /* U+AA0C */
  0x321F,   /* U+AA0D */
  0x3220,   /* U+AA0E */
  0x3221,   /* U+AA0F */
  0x3222,   /* U+AA10 */
  0x3223,   /* U+AA11 */
  0x3224,   /* U+AA12 */
  0x3225,   /* U+AA13 */
  0x3226,   /* U+AA14 */
  0x3227,   /* U+AA15 */
  0x3228,   /* U+AA16 */
  0x3229,   /* U+AA17 */
  0x322A,   /* U+AA18 */
  0x322B,   /* U+AA19 */
  0x322C,   /* U+AA1A */
  0x322D,   /* U+AA1B */
  0x322E,   /* U+AA1C */
  0x322F,   /* U+AA1D */
  0x3230,   /* U+AA1E */
  0x3231,   /* U+AA1F */
  0x3232,   /* U+AA20 */
  0x3233,   /* U+AA21 */
  0x3234,   /* U+AA22 */
  0x3235,   /* U+AA23 */
  0x3236,   /* U+AA24 */
  0x3237,   /* U+AA25 */
  0x3238,   /* U+AA26 */
  0x3239,   /* U+AA27 */
  0x323A,   /* U+AA28 */
  0x323F,   /* U+AA29 */
  0x3240,   /* U+AA2A */
  0x3241,   /* U+AA2B */
  0x3242,   /* U+AA2C */
  0x3243,   /* U+AA2D */
  0x3244,   /* U+AA2E */
  0x3245,   /* U+AA2F */
  0x3246,   /* U+AA30 */
  0x3247,   /* U+AA31 */
  0x3248,   /* U+AA32 */
  0x323B,   /* U+AA33 */
  0x323C,   /* U+AA34 */
  0x323D,   /* U+AA35 */
  0x323E,   /* U+AA36 */
  0xFBC1,   /* U+AA37 */
  0xFBC1,   /* U+AA38 */
  0xFBC1,   /* U+AA39 */
  0xFBC1,   /* U+AA3A */
  0xFBC1,   /* U+AA3B */
  0xFBC1,   /* U+AA3C */
  0xFBC1,   /* U+AA3D */
  0xFBC1,   /* U+AA3E */
  0xFBC1,   /* U+AA3F */
  0x3249,   /* U+AA40 */
  0x324A,   /* U+AA41 */
  0x324B,   /* U+AA42 */
  0x324C,   /* U+AA43 */
  0x324D,   /* U+AA44 */
  0x324E,   /* U+AA45 */
  0x324F,   /* U+AA46 */
  0x3250,   /* U+AA47 */
  0x3251,   /* U+AA48 */
  0x3252,   /* U+AA49 */
  0x3253,   /* U+AA4A */
  0x3254,   /* U+AA4B */
  0x3255,   /* U+AA4C */
  0x3256,   /* U+AA4D */
  0xFBC1,   /* U+AA4E */
  0xFBC1,   /* U+AA4F */
  0x1C3D,   /* U+AA50 */
  0x1C3E,   /* U+AA51 */
  0x1C3F,   /* U+AA52 */
  0x1C40,   /* U+AA53 */
  0x1C41,   /* U+AA54 */
  0x1C42,   /* U+AA55 */
  0x1C43,   /* U+AA56 */
  0x1C44,   /* U+AA57 */
  0x1C45,   /* U+AA58 */
  0x1C46,   /* U+AA59 */
  0xFBC1,   /* U+AA5A */
  0xFBC1,   /* U+AA5B */
  0x0431,   /* U+AA5C */
  0x02A5,   /* U+AA5D */
  0x02A6,   /* U+AA5E */
  0x02A7,   /* U+AA5F */
  0x303B,   /* U+AA60 */
  0x3044,   /* U+AA61 */
  0x3047,   /* U+AA62 */
  0x304A,   /* U+AA63 */
  0x3051,   /* U+AA64 */
  0x3057,   /* U+AA65 */
  0x305B,   /* U+AA66 */
  0x305D,   /* U+AA67 */
  0x305F,   /* U+AA68 */
  0x3062,   /* U+AA69 */
  0x306E,   /* U+AA6A */
  0x3072,   /* U+AA6B */
  0x3093,   /* U+AA6C */
  0x3096,   /* U+AA6D */
  0x3098,   /* U+AA6E */
  0x3078,   /* U+AA6F */
  0x1C01,   /* U+AA70 */
  0x3099,   /* U+AA71 */
  0x304D,   /* U+AA72 */
  0x3086,   /* U+AA73 */
  0x30E1,   /* U+AA74 */
  0x30E2,   /* U+AA75 */
  0x30E3,   /* U+AA76 */
  0x054D,   /* U+AA77 */
  0x054E,   /* U+AA78 */
  0x054F,   /* U+AA79 */
  0x3087,   /* U+AA7A */
  0x30DE,   /* U+AA7B */
  0x30DF,   /* U+AA7C */
  0x30E0,   /* U+AA7D */
  0x3048,   /* U+AA7E */
  0x3054,   /* U+AA7F */
  0x2DE3,   /* U+AA80 */
  0x2DE4,   /* U+AA81 */
  0x2DE5,   /* U+AA82 */
  0x2DE6,   /* U+AA83 */
  0x2DE7,   /* U+AA84 */
  0x2DE8,   /* U+AA85 */
  0x2DE9,   /* U+AA86 */
  0x2DEA,   /* U+AA87 */
  0x2DEB,   /* U+AA88 */
  0x2DEC,   /* U+AA89 */
  0x2DED,   /* U+AA8A */
  0x2DEE,   /* U+AA8B */
  0x2DEF,   /* U+AA8C */
  0x2DF0,   /* U+AA8D */
  0x2DF1,   /* U+AA8E */
  0x2DF2,   /* U+AA8F */
  0x2DF3,   /* U+AA90 */
  0x2DF4,   /* U+AA91 */
  0x2DF5,   /* U+AA92 */
  0x2DF6,   /* U+AA93 */
  0x2DF7,   /* U+AA94 */
  0x2DF8,   /* U+AA95 */
  0x2DF9,   /* U+AA96 */
  0x2DFA,   /* U+AA97 */
  0x2DFB,   /* U+AA98 */
  0x2DFC,   /* U+AA99 */
  0x2DFD,   /* U+AA9A */
  0x2DFE,   /* U+AA9B */
  0x2DFF,   /* U+AA9C */
  0x2E00,   /* U+AA9D */
  0x2E01,   /* U+AA9E */
  0x2E02,   /* U+AA9F */
  0x2E03,   /* U+AAA0 */
  0x2E04,   /* U+AAA1 */
  0x2E05,   /* U+AAA2 */
  0x2E06,   /* U+AAA3 */
  0x2E07,   /* U+AAA4 */
  0x2E08,   /* U+AAA5 */
  0x2E09,   /* U+AAA6 */
  0x2E0A,   /* U+AAA7 */
  0x2E0B,   /* U+AAA8 */
  0x2E0C,   /* U+AAA9 */
  0x2E0D,   /* U+AAAA */
  0x2E0E,   /* U+AAAB */
  0x2E0F,   /* U+AAAC */
  0x2E10,   /* U+AAAD */
  0x2E11,   /* U+AAAE */
  0x2E12,   /* U+AAAF */
  0x2E13,   /* U+AAB0 */
  0x2E14,   /* U+AAB1 */
  0x2E15,   /* U+AAB2 */
  0x2E16,   /* U+AAB3 */
  0x2E17,   /* U+AAB4 */
  0x2E18,   /* U+AAB5 */
  0x2E19,   /* U+AAB6 */
  0x2E1A,   /* U+AAB7 */
  0x2E1B,   /* U+AAB8 */
  0x2E1C,   /* U+AAB9 */
  0x2E1D,   /* U+AABA */
  0x2E1E,   /* U+AABB */
  0x2E1F,   /* U+AABC */
  0x2E20,   /* U+AABD */
  0x2E21,   /* U+AABE */
  0x0000,   /* U+AABF */
  0x2E22,   /* U+AAC0 */
  0x0000,   /* U+AAC1 */
  0x2E23,   /* U+AAC2 */
  0xFBC1,   /* U+AAC3 */
  0xFBC1,   /* U+AAC4 */
  0xFBC1,   /* U+AAC5 */
  0xFBC1,   /* U+AAC6 */
  0xFBC1,   /* U+AAC7 */
  0xFBC1,   /* U+AAC8 */
  0xFBC1,   /* U+AAC9 */
  0xFBC1,   /* U+AACA */
  0xFBC1,   /* U+AACB */
  0xFBC1,   /* U+AACC */
  0xFBC1,   /* U+AACD */
  0xFBC1,   /* U+AACE */
  0xFBC1,   /* U+AACF */
  0xFBC1,   /* U+AAD0 */
  0xFBC1,   /* U+AAD1 */
  0xFBC1,   /* U+AAD2 */
  0xFBC1,   /* U+AAD3 */
  0xFBC1,   /* U+AAD4 */
  0xFBC1,   /* U+AAD5 */
  0xFBC1,   /* U+AAD6 */
  0xFBC1,   /* U+AAD7 */
  0xFBC1,   /* U+AAD8 */
  0xFBC1,   /* U+AAD9 */
  0xFBC1,   /* U+AADA */
  0x2E24,   /* U+AADB */
  0x2E25,   /* U+AADC */
  0x1C02,   /* U+AADD */
  0x03F1,   /* U+AADE */
  0x03F2,   /* U+AADF */
  0x2939,   /* U+AAE0 */
  0x293A,   /* U+AAE1 */
  0x293B,   /* U+AAE2 */
  0x293C,   /* U+AAE3 */
  0x293D,   /* U+AAE4 */
  0x293E,   /* U+AAE5 */
  0x293F,   /* U+AAE6 */
  0x2940,   /* U+AAE7 */
  0x2941,   /* U+AAE8 */
  0x2942,   /* U+AAE9 */
  0x2943,   /* U+AAEA */
  0x294C,   /* U+AAEB */
  0x294D,   /* U+AAEC */
  0x294E,   /* U+AAED */
  0x294F,   /* U+AAEE */
  0x2950,   /* U+AAEF */
  0x02A8,   /* U+AAF0 */
  0x0272,   /* U+AAF1 */
  0x291D,   /* U+AAF2 */
  0x1C03,   /* U+AAF3 */
  0x1C04,   /* U+AAF4 */
  0x2951,   /* U+AAF5 */
  0x295B,   /* U+AAF6 */
  0xFBC1,   /* U+AAF7 */
  0xFBC1,   /* U+AAF8 */
  0xFBC1,   /* U+AAF9 */
  0xFBC1,   /* U+AAFA */
  0xFBC1,   /* U+AAFB */
  0xFBC1,   /* U+AAFC */
  0xFBC1,   /* U+AAFD */
  0xFBC1,   /* U+AAFE */
  0xFBC1,   /* U+AAFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+AA00 */
  0x0020,   /* U+AA01 */
  0x0020,   /* U+AA02 */
  0x0020,   /* U+AA03 */
  0x0020,   /* U+AA04 */
  0x0020,   /* U+AA05 */
  0x0020,   /* U+AA06 */
  0x0020,   /* U+AA07 */
  0x0020,   /* U+AA08 */
  0x0020,   /* U+AA09 */
  0x0020,   /* U+AA0A */
  0x0020,   /* U+AA0B */
  0x0020,   /* U+AA0C */
  0x0020,   /* U+AA0D */
  0x0020,   /* U+AA0E */
  0x0020,   /* U+AA0F */
  0x0020,   /* U+AA10 */
  0x0020,   /* U+AA11 */
  0x0020,   /* U+AA12 */
  0x0020,   /* U+AA13 */
  0x0020,   /* U+AA14 */
  0x0020,   /* U+AA15 */
  0x0020,   /* U+AA16 */
  0x0020,   /* U+AA17 */
  0x0020,   /* U+AA18 */
  0x0020,   /* U+AA19 */
  0x0020,   /* U+AA1A */
  0x0020,   /* U+AA1B */
  0x0020,   /* U+AA1C */
  0x0020,   /* U+AA1D */
  0x0020,   /* U+AA1E */
  0x0020,   /* U+AA1F */
  0x0020,   /* U+AA20 */
  0x0020,   /* U+AA21 */
  0x0020,   /* U+AA22 */
  0x0020,   /* U+AA23 */
  0x0020,   /* U+AA24 */
  0x0020,   /* U+AA25 */
  0x0020,   /* U+AA26 */
  0x0020,   /* U+AA27 */
  0x0020,   /* U+AA28 */
  0x0020,   /* U+AA29 */
  0x0020,   /* U+AA2A */
  0x0020,   /* U+AA2B */
  0x0020,   /* U+AA2C */
  0x0020,   /* U+AA2D */
  0x0020,   /* U+AA2E */
  0x0020,   /* U+AA2F */
  0x0020,   /* U+AA30 */
  0x0020,   /* U+AA31 */
  0x0020,   /* U+AA32 */
  0x0020,   /* U+AA33 */
  0x0020,   /* U+AA34 */
  0x0020,   /* U+AA35 */
  0x0020,   /* U+AA36 */
  0x0020,   /* U+AA37 */
  0x0020,   /* U+AA38 */
  0x0020,   /* U+AA39 */
  0x0020,   /* U+AA3A */
  0x0020,   /* U+AA3B */
  0x0020,   /* U+AA3C */
  0x0020,   /* U+AA3D */
  0x0020,   /* U+AA3E */
  0x0020,   /* U+AA3F */
  0x0020,   /* U+AA40 */
  0x0020,   /* U+AA41 */
  0x0020,   /* U+AA42 */
  0x0020,   /* U+AA43 */
  0x0020,   /* U+AA44 */
  0x0020,   /* U+AA45 */
  0x0020,   /* U+AA46 */
  0x0020,   /* U+AA47 */
  0x0020,   /* U+AA48 */
  0x0020,   /* U+AA49 */
  0x0020,   /* U+AA4A */
  0x0020,   /* U+AA4B */
  0x0020,   /* U+AA4C */
  0x0020,   /* U+AA4D */
  0x0020,   /* U+AA4E */
  0x0020,   /* U+AA4F */
  0x0020,   /* U+AA50 */
  0x0020,   /* U+AA51 */
  0x0020,   /* U+AA52 */
  0x0020,   /* U+AA53 */
  0x0020,   /* U+AA54 */
  0x0020,   /* U+AA55 */
  0x0020,   /* U+AA56 */
  0x0020,   /* U+AA57 */
  0x0020,   /* U+AA58 */
  0x0020,   /* U+AA59 */
  0x0020,   /* U+AA5A */
  0x0020,   /* U+AA5B */
  0x0020,   /* U+AA5C */
  0x0020,   /* U+AA5D */
  0x0020,   /* U+AA5E */
  0x0020,   /* U+AA5F */
  0x0020,   /* U+AA60 */
  0x0020,   /* U+AA61 */
  0x0020,   /* U+AA62 */
  0x0020,   /* U+AA63 */
  0x0020,   /* U+AA64 */
  0x0020,   /* U+AA65 */
  0x0020,   /* U+AA66 */
  0x0020,   /* U+AA67 */
  0x0020,   /* U+AA68 */
  0x0020,   /* U+AA69 */
  0x0020,   /* U+AA6A */
  0x0020,   /* U+AA6B */
  0x0020,   /* U+AA6C */
  0x0020,   /* U+AA6D */
  0x0020,   /* U+AA6E */
  0x0020,   /* U+AA6F */
  0x0020,   /* U+AA70 */
  0x0020,   /* U+AA71 */
  0x0020,   /* U+AA72 */
  0x0020,   /* U+AA73 */
  0x0020,   /* U+AA74 */
  0x0020,   /* U+AA75 */
  0x0020,   /* U+AA76 */
  0x0020,   /* U+AA77 */
  0x0020,   /* U+AA78 */
  0x0020,   /* U+AA79 */
  0x0020,   /* U+AA7A */
  0x0020,   /* U+AA7B */
  0x0020,   /* U+AA7C */
  0x0020,   /* U+AA7D */
  0x0020,   /* U+AA7E */
  0x0020,   /* U+AA7F */
  0x0020,   /* U+AA80 */
  0x0020,   /* U+AA81 */
  0x0020,   /* U+AA82 */
  0x0020,   /* U+AA83 */
  0x0020,   /* U+AA84 */
  0x0020,   /* U+AA85 */
  0x0020,   /* U+AA86 */
  0x0020,   /* U+AA87 */
  0x0020,   /* U+AA88 */
  0x0020,   /* U+AA89 */
  0x0020,   /* U+AA8A */
  0x0020,   /* U+AA8B */
  0x0020,   /* U+AA8C */
  0x0020,   /* U+AA8D */
  0x0020,   /* U+AA8E */
  0x0020,   /* U+AA8F */
  0x0020,   /* U+AA90 */
  0x0020,   /* U+AA91 */
  0x0020,   /* U+AA92 */
  0x0020,   /* U+AA93 */
  0x0020,   /* U+AA94 */
  0x0020,   /* U+AA95 */
  0x0020,   /* U+AA96 */
  0x0020,   /* U+AA97 */
  0x0020,   /* U+AA98 */
  0x0020,   /* U+AA99 */
  0x0020,   /* U+AA9A */
  0x0020,   /* U+AA9B */
  0x0020,   /* U+AA9C */
  0x0020,   /* U+AA9D */
  0x0020,   /* U+AA9E */
  0x0020,   /* U+AA9F */
  0x0020,   /* U+AAA0 */
  0x0020,   /* U+AAA1 */
  0x0020,   /* U+AAA2 */
  0x0020,   /* U+AAA3 */
  0x0020,   /* U+AAA4 */
  0x0020,   /* U+AAA5 */
  0x0020,   /* U+AAA6 */
  0x0020,   /* U+AAA7 */
  0x0020,   /* U+AAA8 */
  0x0020,   /* U+AAA9 */
  0x0020,   /* U+AAAA */
  0x0020,   /* U+AAAB */
  0x0020,   /* U+AAAC */
  0x0020,   /* U+AAAD */
  0x0020,   /* U+AAAE */
  0x0020,   /* U+AAAF */
  0x0020,   /* U+AAB0 */
  0x0020,   /* U+AAB1 */
  0x0020,   /* U+AAB2 */
  0x0020,   /* U+AAB3 */
  0x0020,   /* U+AAB4 */
  0x0020,   /* U+AAB5 */
  0x0020,   /* U+AAB6 */
  0x0020,   /* U+AAB7 */
  0x0020,   /* U+AAB8 */
  0x0020,   /* U+AAB9 */
  0x0020,   /* U+AABA */
  0x0020,   /* U+AABB */
  0x0020,   /* U+AABC */
  0x0020,   /* U+AABD */
  0x0020,   /* U+AABE */
  0x00DF,   /* U+AABF */
  0x0020,   /* U+AAC0 */
  0x00E0,   /* U+AAC1 */
  0x0020,   /* U+AAC2 */
  0x0020,   /* U+AAC3 */
  0x0020,   /* U+AAC4 */
  0x0020,   /* U+AAC5 */
  0x0020,   /* U+AAC6 */
  0x0020,   /* U+AAC7 */
  0x0020,   /* U+AAC8 */
  0x0020,   /* U+AAC9 */
  0x0020,   /* U+AACA */
  0x0020,   /* U+AACB */
  0x0020,   /* U+AACC */
  0x0020,   /* U+AACD */
  0x0020,   /* U+AACE */
  0x0020,   /* U+AACF */
  0x0020,   /* U+AAD0 */
  0x0020,   /* U+AAD1 */
  0x0020,   /* U+AAD2 */
  0x0020,   /* U+AAD3 */
  0x0020,   /* U+AAD4 */
  0x0020,   /* U+AAD5 */
  0x0020,   /* U+AAD6 */
  0x0020,   /* U+AAD7 */
  0x0020,   /* U+AAD8 */
  0x0020,   /* U+AAD9 */
  0x0020,   /* U+AADA */
  0x0020,   /* U+AADB */
  0x0020,   /* U+AADC */
  0x0020,   /* U+AADD */
  0x0020,   /* U+AADE */
  0x0020,   /* U+AADF */
  0x0020,   /* U+AAE0 */
  0x0020,   /* U+AAE1 */
  0x0020,   /* U+AAE2 */
  0x0020,   /* U+AAE3 */
  0x0020,   /* U+AAE4 */
  0x0020,   /* U+AAE5 */
  0x0020,   /* U+AAE6 */
  0x0020,   /* U+AAE7 */
  0x0020,   /* U+AAE8 */
  0x0020,   /* U+AAE9 */
  0x0020,   /* U+AAEA */
  0x0020,   /* U+AAEB */
  0x0020,   /* U+AAEC */
  0x0020,   /* U+AAED */
  0x0020,   /* U+AAEE */
  0x0020,   /* U+AAEF */
  0x0020,   /* U+AAF0 */
  0x0020,   /* U+AAF1 */
  0x0020,   /* U+AAF2 */
  0x0020,   /* U+AAF3 */
  0x0020,   /* U+AAF4 */
  0x0020,   /* U+AAF5 */
  0x0020,   /* U+AAF6 */
  0x0020,   /* U+AAF7 */
  0x0020,   /* U+AAF8 */
  0x0020,   /* U+AAF9 */
  0x0020,   /* U+AAFA */
  0x0020,   /* U+AAFB */
  0x0020,   /* U+AAFC */
  0x0020,   /* U+AAFD */
  0x0020,   /* U+AAFE */
  0x0020,   /* U+AAFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+AA00 */
  0x0002,   /* U+AA01 */
  0x0002,   /* U+AA02 */
  0x0002,   /* U+AA03 */
  0x0002,   /* U+AA04 */
  0x0002,   /* U+AA05 */
  0x0002,   /* U+AA06 */
  0x0002,   /* U+AA07 */
  0x0002,   /* U+AA08 */
  0x0002,   /* U+AA09 */
  0x0002,   /* U+AA0A */
  0x0002,   /* U+AA0B */
  0x0002,   /* U+AA0C */
  0x0002,   /* U+AA0D */
  0x0002,   /* U+AA0E */
  0x0002,   /* U+AA0F */
  0x0002,   /* U+AA10 */
  0x0002,   /* U+AA11 */
  0x0002,   /* U+AA12 */
  0x0002,   /* U+AA13 */
  0x0002,   /* U+AA14 */
  0x0002,   /* U+AA15 */
  0x0002,   /* U+AA16 */
  0x0002,   /* U+AA17 */
  0x0002,   /* U+AA18 */
  0x0002,   /* U+AA19 */
  0x0002,   /* U+AA1A */
  0x0002,   /* U+AA1B */
  0x0002,   /* U+AA1C */
  0x0002,   /* U+AA1D */
  0x0002,   /* U+AA1E */
  0x0002,   /* U+AA1F */
  0x0002,   /* U+AA20 */
  0x0002,   /* U+AA21 */
  0x0002,   /* U+AA22 */
  0x0002,   /* U+AA23 */
  0x0002,   /* U+AA24 */
  0x0002,   /* U+AA25 */
  0x0002,   /* U+AA26 */
  0x0002,   /* U+AA27 */
  0x0002,   /* U+AA28 */
  0x0002,   /* U+AA29 */
  0x0002,   /* U+AA2A */
  0x0002,   /* U+AA2B */
  0x0002,   /* U+AA2C */
  0x0002,   /* U+AA2D */
  0x0002,   /* U+AA2E */
  0x0002,   /* U+AA2F */
  0x0002,   /* U+AA30 */
  0x0002,   /* U+AA31 */
  0x0002,   /* U+AA32 */
  0x0002,   /* U+AA33 */
  0x0002,   /* U+AA34 */
  0x0002,   /* U+AA35 */
  0x0002,   /* U+AA36 */
  0x0002,   /* U+AA37 */
  0x0002,   /* U+AA38 */
  0x0002,   /* U+AA39 */
  0x0002,   /* U+AA3A */
  0x0002,   /* U+AA3B */
  0x0002,   /* U+AA3C */
  0x0002,   /* U+AA3D */
  0x0002,   /* U+AA3E */
  0x0002,   /* U+AA3F */
  0x0002,   /* U+AA40 */
  0x0002,   /* U+AA41 */
  0x0002,   /* U+AA42 */
  0x0002,   /* U+AA43 */
  0x0002,   /* U+AA44 */
  0x0002,   /* U+AA45 */
  0x0002,   /* U+AA46 */
  0x0002,   /* U+AA47 */
  0x0002,   /* U+AA48 */
  0x0002,   /* U+AA49 */
  0x0002,   /* U+AA4A */
  0x0002,   /* U+AA4B */
  0x0002,   /* U+AA4C */
  0x0002,   /* U+AA4D */
  0x0002,   /* U+AA4E */
  0x0002,   /* U+AA4F */
  0x0002,   /* U+AA50 */
  0x0002,   /* U+AA51 */
  0x0002,   /* U+AA52 */
  0x0002,   /* U+AA53 */
  0x0002,   /* U+AA54 */
  0x0002,   /* U+AA55 */
  0x0002,   /* U+AA56 */
  0x0002,   /* U+AA57 */
  0x0002,   /* U+AA58 */
  0x0002,   /* U+AA59 */
  0x0002,   /* U+AA5A */
  0x0002,   /* U+AA5B */
  0x0002,   /* U+AA5C */
  0x0002,   /* U+AA5D */
  0x0002,   /* U+AA5E */
  0x0002,   /* U+AA5F */
  0x0002,   /* U+AA60 */
  0x0002,   /* U+AA61 */
  0x0002,   /* U+AA62 */
  0x0002,   /* U+AA63 */
  0x0002,   /* U+AA64 */
  0x0002,   /* U+AA65 */
  0x0002,   /* U+AA66 */
  0x0002,   /* U+AA67 */
  0x0002,   /* U+AA68 */
  0x0002,   /* U+AA69 */
  0x0002,   /* U+AA6A */
  0x0002,   /* U+AA6B */
  0x0002,   /* U+AA6C */
  0x0002,   /* U+AA6D */
  0x0002,   /* U+AA6E */
  0x0002,   /* U+AA6F */
  0x0002,   /* U+AA70 */
  0x0002,   /* U+AA71 */
  0x0002,   /* U+AA72 */
  0x0002,   /* U+AA73 */
  0x0002,   /* U+AA74 */
  0x0002,   /* U+AA75 */
  0x0002,   /* U+AA76 */
  0x0002,   /* U+AA77 */
  0x0002,   /* U+AA78 */
  0x0002,   /* U+AA79 */
  0x0002,   /* U+AA7A */
  0x0002,   /* U+AA7B */
  0x0002,   /* U+AA7C */
  0x0002,   /* U+AA7D */
  0x0002,   /* U+AA7E */
  0x0002,   /* U+AA7F */
  0x0002,   /* U+AA80 */
  0x0002,   /* U+AA81 */
  0x0002,   /* U+AA82 */
  0x0002,   /* U+AA83 */
  0x0002,   /* U+AA84 */
  0x0002,   /* U+AA85 */
  0x0002,   /* U+AA86 */
  0x0002,   /* U+AA87 */
  0x0002,   /* U+AA88 */
  0x0002,   /* U+AA89 */
  0x0002,   /* U+AA8A */
  0x0002,   /* U+AA8B */
  0x0002,   /* U+AA8C */
  0x0002,   /* U+AA8D */
  0x0002,   /* U+AA8E */
  0x0002,   /* U+AA8F */
  0x0002,   /* U+AA90 */
  0x0002,   /* U+AA91 */
  0x0002,   /* U+AA92 */
  0x0002,   /* U+AA93 */
  0x0002,   /* U+AA94 */
  0x0002,   /* U+AA95 */
  0x0002,   /* U+AA96 */
  0x0002,   /* U+AA97 */
  0x0002,   /* U+AA98 */
  0x0002,   /* U+AA99 */
  0x0002,   /* U+AA9A */
  0x0002,   /* U+AA9B */
  0x0002,   /* U+AA9C */
  0x0002,   /* U+AA9D */
  0x0002,   /* U+AA9E */
  0x0002,   /* U+AA9F */
  0x0002,   /* U+AAA0 */
  0x0002,   /* U+AAA1 */
  0x0002,   /* U+AAA2 */
  0x0002,   /* U+AAA3 */
  0x0002,   /* U+AAA4 */
  0x0002,   /* U+AAA5 */
  0x0002,   /* U+AAA6 */
  0x0002,   /* U+AAA7 */
  0x0002,   /* U+AAA8 */
  0x0002,   /* U+AAA9 */
  0x0002,   /* U+AAAA */
  0x0002,   /* U+AAAB */
  0x0002,   /* U+AAAC */
  0x0002,   /* U+AAAD */
  0x0002,   /* U+AAAE */
  0x0002,   /* U+AAAF */
  0x0002,   /* U+AAB0 */
  0x0002,   /* U+AAB1 */
  0x0002,   /* U+AAB2 */
  0x0002,   /* U+AAB3 */
  0x0002,   /* U+AAB4 */
  0x0002,   /* U+AAB5 */
  0x0002,   /* U+AAB6 */
  0x0002,   /* U+AAB7 */
  0x0002,   /* U+AAB8 */
  0x0002,   /* U+AAB9 */
  0x0002,   /* U+AABA */
  0x0002,   /* U+AABB */
  0x0002,   /* U+AABC */
  0x0002,   /* U+AABD */
  0x0002,   /* U+AABE */
  0x0002,   /* U+AABF */
  0x0002,   /* U+AAC0 */
  0x0002,   /* U+AAC1 */
  0x0002,   /* U+AAC2 */
  0x0002,   /* U+AAC3 */
  0x0002,   /* U+AAC4 */
  0x0002,   /* U+AAC5 */
  0x0002,   /* U+AAC6 */
  0x0002,   /* U+AAC7 */
  0x0002,   /* U+AAC8 */
  0x0002,   /* U+AAC9 */
  0x0002,   /* U+AACA */
  0x0002,   /* U+AACB */
  0x0002,   /* U+AACC */
  0x0002,   /* U+AACD */
  0x0002,   /* U+AACE */
  0x0002,   /* U+AACF */
  0x0002,   /* U+AAD0 */
  0x0002,   /* U+AAD1 */
  0x0002,   /* U+AAD2 */
  0x0002,   /* U+AAD3 */
  0x0002,   /* U+AAD4 */
  0x0002,   /* U+AAD5 */
  0x0002,   /* U+AAD6 */
  0x0002,   /* U+AAD7 */
  0x0002,   /* U+AAD8 */
  0x0002,   /* U+AAD9 */
  0x0002,   /* U+AADA */
  0x0002,   /* U+AADB */
  0x0002,   /* U+AADC */
  0x0002,   /* U+AADD */
  0x0002,   /* U+AADE */
  0x0002,   /* U+AADF */
  0x0002,   /* U+AAE0 */
  0x0002,   /* U+AAE1 */
  0x0002,   /* U+AAE2 */
  0x0002,   /* U+AAE3 */
  0x0002,   /* U+AAE4 */
  0x0002,   /* U+AAE5 */
  0x0002,   /* U+AAE6 */
  0x0002,   /* U+AAE7 */
  0x0002,   /* U+AAE8 */
  0x0002,   /* U+AAE9 */
  0x0002,   /* U+AAEA */
  0x0002,   /* U+AAEB */
  0x0002,   /* U+AAEC */
  0x0002,   /* U+AAED */
  0x0002,   /* U+AAEE */
  0x0002,   /* U+AAEF */
  0x0002,   /* U+AAF0 */
  0x0002,   /* U+AAF1 */
  0x0002,   /* U+AAF2 */
  0x0002,   /* U+AAF3 */
  0x0002,   /* U+AAF4 */
  0x0002,   /* U+AAF5 */
  0x0002,   /* U+AAF6 */
  0x0002,   /* U+AAF7 */
  0x0002,   /* U+AAF8 */
  0x0002,   /* U+AAF9 */
  0x0002,   /* U+AAFA */
  0x0002,   /* U+AAFB */
  0x0002,   /* U+AAFC */
  0x0002,   /* U+AAFD */
  0x0002,   /* U+AAFE */
  0x0002,   /* U+AAFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+AA00 */
  0x0000,   /* U+AA01 */
  0x0000,   /* U+AA02 */
  0x0000,   /* U+AA03 */
  0x0000,   /* U+AA04 */
  0x0000,   /* U+AA05 */
  0x0000,   /* U+AA06 */
  0x0000,   /* U+AA07 */
  0x0000,   /* U+AA08 */
  0x0000,   /* U+AA09 */
  0x0000,   /* U+AA0A */
  0x0000,   /* U+AA0B */
  0x0000,   /* U+AA0C */
  0x0000,   /* U+AA0D */
  0x0000,   /* U+AA0E */
  0x0000,   /* U+AA0F */
  0x0000,   /* U+AA10 */
  0x0000,   /* U+AA11 */
  0x0000,   /* U+AA12 */
  0x0000,   /* U+AA13 */
  0x0000,   /* U+AA14 */
  0x0000,   /* U+AA15 */
  0x0000,   /* U+AA16 */
  0x0000,   /* U+AA17 */
  0x0000,   /* U+AA18 */
  0x0000,   /* U+AA19 */
  0x0000,   /* U+AA1A */
  0x0000,   /* U+AA1B */
  0x0000,   /* U+AA1C */
  0x0000,   /* U+AA1D */
  0x0000,   /* U+AA1E */
  0x0000,   /* U+AA1F */
  0x0000,   /* U+AA20 */
  0x0000,   /* U+AA21 */
  0x0000,   /* U+AA22 */
  0x0000,   /* U+AA23 */
  0x0000,   /* U+AA24 */
  0x0000,   /* U+AA25 */
  0x0000,   /* U+AA26 */
  0x0000,   /* U+AA27 */
  0x0000,   /* U+AA28 */
  0x0000,   /* U+AA29 */
  0x0000,   /* U+AA2A */
  0x0000,   /* U+AA2B */
  0x0000,   /* U+AA2C */
  0x0000,   /* U+AA2D */
  0x0000,   /* U+AA2E */
  0x0000,   /* U+AA2F */
  0x0000,   /* U+AA30 */
  0x0000,   /* U+AA31 */
  0x0000,   /* U+AA32 */
  0x0000,   /* U+AA33 */
  0x0000,   /* U+AA34 */
  0x0000,   /* U+AA35 */
  0x0000,   /* U+AA36 */
  0xAA37,   /* U+AA37 */
  0xAA38,   /* U+AA38 */
  0xAA39,   /* U+AA39 */
  0xAA3A,   /* U+AA3A */
  0xAA3B,   /* U+AA3B */
  0xAA3C,   /* U+AA3C */
  0xAA3D,   /* U+AA3D */
  0xAA3E,   /* U+AA3E */
  0xAA3F,   /* U+AA3F */
  0x0000,   /* U+AA40 */
  0x0000,   /* U+AA41 */
  0x0000,   /* U+AA42 */
  0x0000,   /* U+AA43 */
  0x0000,   /* U+AA44 */
  0x0000,   /* U+AA45 */
  0x0000,   /* U+AA46 */
  0x0000,   /* U+AA47 */
  0x0000,   /* U+AA48 */
  0x0000,   /* U+AA49 */
  0x0000,   /* U+AA4A */
  0x0000,   /* U+AA4B */
  0x0000,   /* U+AA4C */
  0x0000,   /* U+AA4D */
  0xAA4E,   /* U+AA4E */
  0xAA4F,   /* U+AA4F */
  0x0000,   /* U+AA50 */
  0x0000,   /* U+AA51 */
  0x0000,   /* U+AA52 */
  0x0000,   /* U+AA53 */
  0x0000,   /* U+AA54 */
  0x0000,   /* U+AA55 */
  0x0000,   /* U+AA56 */
  0x0000,   /* U+AA57 */
  0x0000,   /* U+AA58 */
  0x0000,   /* U+AA59 */
  0xAA5A,   /* U+AA5A */
  0xAA5B,   /* U+AA5B */
  0x0000,   /* U+AA5C */
  0x0000,   /* U+AA5D */
  0x0000,   /* U+AA5E */
  0x0000,   /* U+AA5F */
  0x0000,   /* U+AA60 */
  0x0000,   /* U+AA61 */
  0x0000,   /* U+AA62 */
  0x0000,   /* U+AA63 */
  0x0000,   /* U+AA64 */
  0x0000,   /* U+AA65 */
  0x0000,   /* U+AA66 */
  0x0000,   /* U+AA67 */
  0x0000,   /* U+AA68 */
  0x0000,   /* U+AA69 */
  0x0000,   /* U+AA6A */
  0x0000,   /* U+AA6B */
  0x0000,   /* U+AA6C */
  0x0000,   /* U+AA6D */
  0x0000,   /* U+AA6E */
  0x0000,   /* U+AA6F */
  0x0000,   /* U+AA70 */
  0x0000,   /* U+AA71 */
  0x0000,   /* U+AA72 */
  0x0000,   /* U+AA73 */
  0x0000,   /* U+AA74 */
  0x0000,   /* U+AA75 */
  0x0000,   /* U+AA76 */
  0x0000,   /* U+AA77 */
  0x0000,   /* U+AA78 */
  0x0000,   /* U+AA79 */
  0x0000,   /* U+AA7A */
  0x0000,   /* U+AA7B */
  0x0000,   /* U+AA7C */
  0x0000,   /* U+AA7D */
  0x0000,   /* U+AA7E */
  0x0000,   /* U+AA7F */
  0x0000,   /* U+AA80 */
  0x0000,   /* U+AA81 */
  0x0000,   /* U+AA82 */
  0x0000,   /* U+AA83 */
  0x0000,   /* U+AA84 */
  0x0000,   /* U+AA85 */
  0x0000,   /* U+AA86 */
  0x0000,   /* U+AA87 */
  0x0000,   /* U+AA88 */
  0x0000,   /* U+AA89 */
  0x0000,   /* U+AA8A */
  0x0000,   /* U+AA8B */
  0x0000,   /* U+AA8C */
  0x0000,   /* U+AA8D */
  0x0000,   /* U+AA8E */
  0x0000,   /* U+AA8F */
  0x0000,   /* U+AA90 */
  0x0000,   /* U+AA91 */
  0x0000,   /* U+AA92 */
  0x0000,   /* U+AA93 */
  0x0000,   /* U+AA94 */
  0x0000,   /* U+AA95 */
  0x0000,   /* U+AA96 */
  0x0000,   /* U+AA97 */
  0x0000,   /* U+AA98 */
  0x0000,   /* U+AA99 */
  0x0000,   /* U+AA9A */
  0x0000,   /* U+AA9B */
  0x0000,   /* U+AA9C */
  0x0000,   /* U+AA9D */
  0x0000,   /* U+AA9E */
  0x0000,   /* U+AA9F */
  0x0000,   /* U+AAA0 */
  0x0000,   /* U+AAA1 */
  0x0000,   /* U+AAA2 */
  0x0000,   /* U+AAA3 */
  0x0000,   /* U+AAA4 */
  0x0000,   /* U+AAA5 */
  0x0000,   /* U+AAA6 */
  0x0000,   /* U+AAA7 */
  0x0000,   /* U+AAA8 */
  0x0000,   /* U+AAA9 */
  0x0000,   /* U+AAAA */
  0x0000,   /* U+AAAB */
  0x0000,   /* U+AAAC */
  0x0000,   /* U+AAAD */
  0x0000,   /* U+AAAE */
  0x0000,   /* U+AAAF */
  0x0000,   /* U+AAB0 */
  0x0000,   /* U+AAB1 */
  0x0000,   /* U+AAB2 */
  0x0000,   /* U+AAB3 */
  0x0000,   /* U+AAB4 */
  0x0000,   /* U+AAB5 */
  0x0000,   /* U+AAB6 */
  0x0000,   /* U+AAB7 */
  0x0000,   /* U+AAB8 */
  0x0000,   /* U+AAB9 */
  0x0000,   /* U+AABA */
  0x0000,   /* U+AABB */
  0x0000,   /* U+AABC */
  0x0000,   /* U+AABD */
  0x0000,   /* U+AABE */
  0x0000,   /* U+AABF */
  0x0000,   /* U+AAC0 */
  0x0000,   /* U+AAC1 */
  0x0000,   /* U+AAC2 */
  0xAAC3,   /* U+AAC3 */
  0xAAC4,   /* U+AAC4 */
  0xAAC5,   /* U+AAC5 */
  0xAAC6,   /* U+AAC6 */
  0xAAC7,   /* U+AAC7 */
  0xAAC8,   /* U+AAC8 */
  0xAAC9,   /* U+AAC9 */
  0xAACA,   /* U+AACA */
  0xAACB,   /* U+AACB */
  0xAACC,   /* U+AACC */
  0xAACD,   /* U+AACD */
  0xAACE,   /* U+AACE */
  0xAACF,   /* U+AACF */
  0xAAD0,   /* U+AAD0 */
  0xAAD1,   /* U+AAD1 */
  0xAAD2,   /* U+AAD2 */
  0xAAD3,   /* U+AAD3 */
  0xAAD4,   /* U+AAD4 */
  0xAAD5,   /* U+AAD5 */
  0xAAD6,   /* U+AAD6 */
  0xAAD7,   /* U+AAD7 */
  0xAAD8,   /* U+AAD8 */
  0xAAD9,   /* U+AAD9 */
  0xAADA,   /* U+AADA */
  0x0000,   /* U+AADB */
  0x0000,   /* U+AADC */
  0x0000,   /* U+AADD */
  0x0000,   /* U+AADE */
  0x0000,   /* U+AADF */
  0x0000,   /* U+AAE0 */
  0x0000,   /* U+AAE1 */
  0x0000,   /* U+AAE2 */
  0x0000,   /* U+AAE3 */
  0x0000,   /* U+AAE4 */
  0x0000,   /* U+AAE5 */
  0x0000,   /* U+AAE6 */
  0x0000,   /* U+AAE7 */
  0x0000,   /* U+AAE8 */
  0x0000,   /* U+AAE9 */
  0x0000,   /* U+AAEA */
  0x0000,   /* U+AAEB */
  0x0000,   /* U+AAEC */
  0x0000,   /* U+AAED */
  0x0000,   /* U+AAEE */
  0x0000,   /* U+AAEF */
  0x0000,   /* U+AAF0 */
  0x0000,   /* U+AAF1 */
  0x0000,   /* U+AAF2 */
  0x0000,   /* U+AAF3 */
  0x0000,   /* U+AAF4 */
  0x0000,   /* U+AAF5 */
  0x0000,   /* U+AAF6 */
  0xAAF7,   /* U+AAF7 */
  0xAAF8,   /* U+AAF8 */
  0xAAF9,   /* U+AAF9 */
  0xAAFA,   /* U+AAFA */
  0xAAFB,   /* U+AAFB */
  0xAAFC,   /* U+AAFC */
  0xAAFD,   /* U+AAFD */
  0xAAFE,   /* U+AAFE */
  0xAAFF,   /* U+AAFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+AA00 */
  0x0000,   /* U+AA01 */
  0x0000,   /* U+AA02 */
  0x0000,   /* U+AA03 */
  0x0000,   /* U+AA04 */
  0x0000,   /* U+AA05 */
  0x0000,   /* U+AA06 */
  0x0000,   /* U+AA07 */
  0x0000,   /* U+AA08 */
  0x0000,   /* U+AA09 */
  0x0000,   /* U+AA0A */
  0x0000,   /* U+AA0B */
  0x0000,   /* U+AA0C */
  0x0000,   /* U+AA0D */
  0x0000,   /* U+AA0E */
  0x0000,   /* U+AA0F */
  0x0000,   /* U+AA10 */
  0x0000,   /* U+AA11 */
  0x0000,   /* U+AA12 */
  0x0000,   /* U+AA13 */
  0x0000,   /* U+AA14 */
  0x0000,   /* U+AA15 */
  0x0000,   /* U+AA16 */
  0x0000,   /* U+AA17 */
  0x0000,   /* U+AA18 */
  0x0000,   /* U+AA19 */
  0x0000,   /* U+AA1A */
  0x0000,   /* U+AA1B */
  0x0000,   /* U+AA1C */
  0x0000,   /* U+AA1D */
  0x0000,   /* U+AA1E */
  0x0000,   /* U+AA1F */
  0x0000,   /* U+AA20 */
  0x0000,   /* U+AA21 */
  0x0000,   /* U+AA22 */
  0x0000,   /* U+AA23 */
  0x0000,   /* U+AA24 */
  0x0000,   /* U+AA25 */
  0x0000,   /* U+AA26 */
  0x0000,   /* U+AA27 */
  0x0000,   /* U+AA28 */
  0x0000,   /* U+AA29 */
  0x0000,   /* U+AA2A */
  0x0000,   /* U+AA2B */
  0x0000,   /* U+AA2C */
  0x0000,   /* U+AA2D */
  0x0000,   /* U+AA2E */
  0x0000,   /* U+AA2F */
  0x0000,   /* U+AA30 */
  0x0000,   /* U+AA31 */
  0x0000,   /* U+AA32 */
  0x0000,   /* U+AA33 */
  0x0000,   /* U+AA34 */
  0x0000,   /* U+AA35 */
  0x0000,   /* U+AA36 */
  0x0000,   /* U+AA37 */
  0x0000,   /* U+AA38 */
  0x0000,   /* U+AA39 */
  0x0000,   /* U+AA3A */
  0x0000,   /* U+AA3B */
  0x0000,   /* U+AA3C */
  0x0000,   /* U+AA3D */
  0x0000,   /* U+AA3E */
  0x0000,   /* U+AA3F */
  0x0000,   /* U+AA40 */
  0x0000,   /* U+AA41 */
  0x0000,   /* U+AA42 */
  0x0000,   /* U+AA43 */
  0x0000,   /* U+AA44 */
  0x0000,   /* U+AA45 */
  0x0000,   /* U+AA46 */
  0x0000,   /* U+AA47 */
  0x0000,   /* U+AA48 */
  0x0000,   /* U+AA49 */
  0x0000,   /* U+AA4A */
  0x0000,   /* U+AA4B */
  0x0000,   /* U+AA4C */
  0x0000,   /* U+AA4D */
  0x0000,   /* U+AA4E */
  0x0000,   /* U+AA4F */
  0x0000,   /* U+AA50 */
  0x0000,   /* U+AA51 */
  0x0000,   /* U+AA52 */
  0x0000,   /* U+AA53 */
  0x0000,   /* U+AA54 */
  0x0000,   /* U+AA55 */
  0x0000,   /* U+AA56 */
  0x0000,   /* U+AA57 */
  0x0000,   /* U+AA58 */
  0x0000,   /* U+AA59 */
  0x0000,   /* U+AA5A */
  0x0000,   /* U+AA5B */
  0x0000,   /* U+AA5C */
  0x0000,   /* U+AA5D */
  0x0000,   /* U+AA5E */
  0x0000,   /* U+AA5F */
  0x0000,   /* U+AA60 */
  0x0000,   /* U+AA61 */
  0x0000,   /* U+AA62 */
  0x0000,   /* U+AA63 */
  0x0000,   /* U+AA64 */
  0x0000,   /* U+AA65 */
  0x0000,   /* U+AA66 */
  0x0000,   /* U+AA67 */
  0x0000,   /* U+AA68 */
  0x0000,   /* U+AA69 */
  0x0000,   /* U+AA6A */
  0x0000,   /* U+AA6B */
  0x0000,   /* U+AA6C */
  0x0000,   /* U+AA6D */
  0x0000,   /* U+AA6E */
  0x0000,   /* U+AA6F */
  0x0000,   /* U+AA70 */
  0x0000,   /* U+AA71 */
  0x0000,   /* U+AA72 */
  0x0000,   /* U+AA73 */
  0x0000,   /* U+AA74 */
  0x0000,   /* U+AA75 */
  0x0000,   /* U+AA76 */
  0x0000,   /* U+AA77 */
  0x0000,   /* U+AA78 */
  0x0000,   /* U+AA79 */
  0x0000,   /* U+AA7A */
  0x0000,   /* U+AA7B */
  0x0000,   /* U+AA7C */
  0x0000,   /* U+AA7D */
  0x0000,   /* U+AA7E */
  0x0000,   /* U+AA7F */
  0x0000,   /* U+AA80 */
  0x0000,   /* U+AA81 */
  0x0000,   /* U+AA82 */
  0x0000,   /* U+AA83 */
  0x0000,   /* U+AA84 */
  0x0000,   /* U+AA85 */
  0x0000,   /* U+AA86 */
  0x0000,   /* U+AA87 */
  0x0000,   /* U+AA88 */
  0x0000,   /* U+AA89 */
  0x0000,   /* U+AA8A */
  0x0000,   /* U+AA8B */
  0x0000,   /* U+AA8C */
  0x0000,   /* U+AA8D */
  0x0000,   /* U+AA8E */
  0x0000,   /* U+AA8F */
  0x0000,   /* U+AA90 */
  0x0000,   /* U+AA91 */
  0x0000,   /* U+AA92 */
  0x0000,   /* U+AA93 */
  0x0000,   /* U+AA94 */
  0x0000,   /* U+AA95 */
  0x0000,   /* U+AA96 */
  0x0000,   /* U+AA97 */
  0x0000,   /* U+AA98 */
  0x0000,   /* U+AA99 */
  0x0000,   /* U+AA9A */
  0x0000,   /* U+AA9B */
  0x0000,   /* U+AA9C */
  0x0000,   /* U+AA9D */
  0x0000,   /* U+AA9E */
  0x0000,   /* U+AA9F */
  0x0000,   /* U+AAA0 */
  0x0000,   /* U+AAA1 */
  0x0000,   /* U+AAA2 */
  0x0000,   /* U+AAA3 */
  0x0000,   /* U+AAA4 */
  0x0000,   /* U+AAA5 */
  0x0000,   /* U+AAA6 */
  0x0000,   /* U+AAA7 */
  0x0000,   /* U+AAA8 */
  0x0000,   /* U+AAA9 */
  0x0000,   /* U+AAAA */
  0x0000,   /* U+AAAB */
  0x0000,   /* U+AAAC */
  0x0000,   /* U+AAAD */
  0x0000,   /* U+AAAE */
  0x0000,   /* U+AAAF */
  0x0000,   /* U+AAB0 */
  0x0000,   /* U+AAB1 */
  0x0000,   /* U+AAB2 */
  0x0000,   /* U+AAB3 */
  0x0000,   /* U+AAB4 */
  0x0000,   /* U+AAB5 */
  0x0000,   /* U+AAB6 */
  0x0000,   /* U+AAB7 */
  0x0000,   /* U+AAB8 */
  0x0000,   /* U+AAB9 */
  0x0000,   /* U+AABA */
  0x0000,   /* U+AABB */
  0x0000,   /* U+AABC */
  0x0000,   /* U+AABD */
  0x0000,   /* U+AABE */
  0x0000,   /* U+AABF */
  0x0000,   /* U+AAC0 */
  0x0000,   /* U+AAC1 */
  0x0000,   /* U+AAC2 */
  0x0000,   /* U+AAC3 */
  0x0000,   /* U+AAC4 */
  0x0000,   /* U+AAC5 */
  0x0000,   /* U+AAC6 */
  0x0000,   /* U+AAC7 */
  0x0000,   /* U+AAC8 */
  0x0000,   /* U+AAC9 */
  0x0000,   /* U+AACA */
  0x0000,   /* U+AACB */
  0x0000,   /* U+AACC */
  0x0000,   /* U+AACD */
  0x0000,   /* U+AACE */
  0x0000,   /* U+AACF */
  0x0000,   /* U+AAD0 */
  0x0000,   /* U+AAD1 */
  0x0000,   /* U+AAD2 */
  0x0000,   /* U+AAD3 */
  0x0000,   /* U+AAD4 */
  0x0000,   /* U+AAD5 */
  0x0000,   /* U+AAD6 */
  0x0000,   /* U+AAD7 */
  0x0000,   /* U+AAD8 */
  0x0000,   /* U+AAD9 */
  0x0000,   /* U+AADA */
  0x0000,   /* U+AADB */
  0x0000,   /* U+AADC */
  0x0000,   /* U+AADD */
  0x0000,   /* U+AADE */
  0x0000,   /* U+AADF */
  0x0000,   /* U+AAE0 */
  0x0000,   /* U+AAE1 */
  0x0000,   /* U+AAE2 */
  0x0000,   /* U+AAE3 */
  0x0000,   /* U+AAE4 */
  0x0000,   /* U+AAE5 */
  0x0000,   /* U+AAE6 */
  0x0000,   /* U+AAE7 */
  0x0000,   /* U+AAE8 */
  0x0000,   /* U+AAE9 */
  0x0000,   /* U+AAEA */
  0x0000,   /* U+AAEB */
  0x0000,   /* U+AAEC */
  0x0000,   /* U+AAED */
  0x0000,   /* U+AAEE */
  0x0000,   /* U+AAEF */
  0x0000,   /* U+AAF0 */
  0x0000,   /* U+AAF1 */
  0x0000,   /* U+AAF2 */
  0x0000,   /* U+AAF3 */
  0x0000,   /* U+AAF4 */
  0x0000,   /* U+AAF5 */
  0x0000,   /* U+AAF6 */
  0x0000,   /* U+AAF7 */
  0x0000,   /* U+AAF8 */
  0x0000,   /* U+AAF9 */
  0x0000,   /* U+AAFA */
  0x0000,   /* U+AAFB */
  0x0000,   /* U+AAFC */
  0x0000,   /* U+AAFD */
  0x0000,   /* U+AAFE */
  0x0000,   /* U+AAFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+AA00 */
  0x0000,   /* U+AA01 */
  0x0000,   /* U+AA02 */
  0x0000,   /* U+AA03 */
  0x0000,   /* U+AA04 */
  0x0000,   /* U+AA05 */
  0x0000,   /* U+AA06 */
  0x0000,   /* U+AA07 */
  0x0000,   /* U+AA08 */
  0x0000,   /* U+AA09 */
  0x0000,   /* U+AA0A */
  0x0000,   /* U+AA0B */
  0x0000,   /* U+AA0C */
  0x0000,   /* U+AA0D */
  0x0000,   /* U+AA0E */
  0x0000,   /* U+AA0F */
  0x0000,   /* U+AA10 */
  0x0000,   /* U+AA11 */
  0x0000,   /* U+AA12 */
  0x0000,   /* U+AA13 */
  0x0000,   /* U+AA14 */
  0x0000,   /* U+AA15 */
  0x0000,   /* U+AA16 */
  0x0000,   /* U+AA17 */
  0x0000,   /* U+AA18 */
  0x0000,   /* U+AA19 */
  0x0000,   /* U+AA1A */
  0x0000,   /* U+AA1B */
  0x0000,   /* U+AA1C */
  0x0000,   /* U+AA1D */
  0x0000,   /* U+AA1E */
  0x0000,   /* U+AA1F */
  0x0000,   /* U+AA20 */
  0x0000,   /* U+AA21 */
  0x0000,   /* U+AA22 */
  0x0000,   /* U+AA23 */
  0x0000,   /* U+AA24 */
  0x0000,   /* U+AA25 */
  0x0000,   /* U+AA26 */
  0x0000,   /* U+AA27 */
  0x0000,   /* U+AA28 */
  0x0000,   /* U+AA29 */
  0x0000,   /* U+AA2A */
  0x0000,   /* U+AA2B */
  0x0000,   /* U+AA2C */
  0x0000,   /* U+AA2D */
  0x0000,   /* U+AA2E */
  0x0000,   /* U+AA2F */
  0x0000,   /* U+AA30 */
  0x0000,   /* U+AA31 */
  0x0000,   /* U+AA32 */
  0x0000,   /* U+AA33 */
  0x0000,   /* U+AA34 */
  0x0000,   /* U+AA35 */
  0x0000,   /* U+AA36 */
  0x0000,   /* U+AA37 */
  0x0000,   /* U+AA38 */
  0x0000,   /* U+AA39 */
  0x0000,   /* U+AA3A */
  0x0000,   /* U+AA3B */
  0x0000,   /* U+AA3C */
  0x0000,   /* U+AA3D */
  0x0000,   /* U+AA3E */
  0x0000,   /* U+AA3F */
  0x0000,   /* U+AA40 */
  0x0000,   /* U+AA41 */
  0x0000,   /* U+AA42 */
  0x0000,   /* U+AA43 */
  0x0000,   /* U+AA44 */
  0x0000,   /* U+AA45 */
  0x0000,   /* U+AA46 */
  0x0000,   /* U+AA47 */
  0x0000,   /* U+AA48 */
  0x0000,   /* U+AA49 */
  0x0000,   /* U+AA4A */
  0x0000,   /* U+AA4B */
  0x0000,   /* U+AA4C */
  0x0000,   /* U+AA4D */
  0x0000,   /* U+AA4E */
  0x0000,   /* U+AA4F */
  0x0000,   /* U+AA50 */
  0x0000,   /* U+AA51 */
  0x0000,   /* U+AA52 */
  0x0000,   /* U+AA53 */
  0x0000,   /* U+AA54 */
  0x0000,   /* U+AA55 */
  0x0000,   /* U+AA56 */
  0x0000,   /* U+AA57 */
  0x0000,   /* U+AA58 */
  0x0000,   /* U+AA59 */
  0x0000,   /* U+AA5A */
  0x0000,   /* U+AA5B */
  0x0000,   /* U+AA5C */
  0x0000,   /* U+AA5D */
  0x0000,   /* U+AA5E */
  0x0000,   /* U+AA5F */
  0x0000,   /* U+AA60 */
  0x0000,   /* U+AA61 */
  0x0000,   /* U+AA62 */
  0x0000,   /* U+AA63 */
  0x0000,   /* U+AA64 */
  0x0000,   /* U+AA65 */
  0x0000,   /* U+AA66 */
  0x0000,   /* U+AA67 */
  0x0000,   /* U+AA68 */
  0x0000,   /* U+AA69 */
  0x0000,   /* U+AA6A */
  0x0000,   /* U+AA6B */
  0x0000,   /* U+AA6C */
  0x0000,   /* U+AA6D */
  0x0000,   /* U+AA6E */
  0x0000,   /* U+AA6F */
  0x0000,   /* U+AA70 */
  0x0000,   /* U+AA71 */
  0x0000,   /* U+AA72 */
  0x0000,   /* U+AA73 */
  0x0000,   /* U+AA74 */
  0x0000,   /* U+AA75 */
  0x0000,   /* U+AA76 */
  0x0000,   /* U+AA77 */
  0x0000,   /* U+AA78 */
  0x0000,   /* U+AA79 */
  0x0000,   /* U+AA7A */
  0x0000,   /* U+AA7B */
  0x0000,   /* U+AA7C */
  0x0000,   /* U+AA7D */
  0x0000,   /* U+AA7E */
  0x0000,   /* U+AA7F */
  0x0000,   /* U+AA80 */
  0x0000,   /* U+AA81 */
  0x0000,   /* U+AA82 */
  0x0000,   /* U+AA83 */
  0x0000,   /* U+AA84 */
  0x0000,   /* U+AA85 */
  0x0000,   /* U+AA86 */
  0x0000,   /* U+AA87 */
  0x0000,   /* U+AA88 */
  0x0000,   /* U+AA89 */
  0x0000,   /* U+AA8A */
  0x0000,   /* U+AA8B */
  0x0000,   /* U+AA8C */
  0x0000,   /* U+AA8D */
  0x0000,   /* U+AA8E */
  0x0000,   /* U+AA8F */
  0x0000,   /* U+AA90 */
  0x0000,   /* U+AA91 */
  0x0000,   /* U+AA92 */
  0x0000,   /* U+AA93 */
  0x0000,   /* U+AA94 */
  0x0000,   /* U+AA95 */
  0x0000,   /* U+AA96 */
  0x0000,   /* U+AA97 */
  0x0000,   /* U+AA98 */
  0x0000,   /* U+AA99 */
  0x0000,   /* U+AA9A */
  0x0000,   /* U+AA9B */
  0x0000,   /* U+AA9C */
  0x0000,   /* U+AA9D */
  0x0000,   /* U+AA9E */
  0x0000,   /* U+AA9F */
  0x0000,   /* U+AAA0 */
  0x0000,   /* U+AAA1 */
  0x0000,   /* U+AAA2 */
  0x0000,   /* U+AAA3 */
  0x0000,   /* U+AAA4 */
  0x0000,   /* U+AAA5 */
  0x0000,   /* U+AAA6 */
  0x0000,   /* U+AAA7 */
  0x0000,   /* U+AAA8 */
  0x0000,   /* U+AAA9 */
  0x0000,   /* U+AAAA */
  0x0000,   /* U+AAAB */
  0x0000,   /* U+AAAC */
  0x0000,   /* U+AAAD */
  0x0000,   /* U+AAAE */
  0x0000,   /* U+AAAF */
  0x0000,   /* U+AAB0 */
  0x0000,   /* U+AAB1 */
  0x0000,   /* U+AAB2 */
  0x0000,   /* U+AAB3 */
  0x0000,   /* U+AAB4 */
  0x0000,   /* U+AAB5 */
  0x0000,   /* U+AAB6 */
  0x0000,   /* U+AAB7 */
  0x0000,   /* U+AAB8 */
  0x0000,   /* U+AAB9 */
  0x0000,   /* U+AABA */
  0x0000,   /* U+AABB */
  0x0000,   /* U+AABC */
  0x0000,   /* U+AABD */
  0x0000,   /* U+AABE */
  0x0000,   /* U+AABF */
  0x0000,   /* U+AAC0 */
  0x0000,   /* U+AAC1 */
  0x0000,   /* U+AAC2 */
  0x0000,   /* U+AAC3 */
  0x0000,   /* U+AAC4 */
  0x0000,   /* U+AAC5 */
  0x0000,   /* U+AAC6 */
  0x0000,   /* U+AAC7 */
  0x0000,   /* U+AAC8 */
  0x0000,   /* U+AAC9 */
  0x0000,   /* U+AACA */
  0x0000,   /* U+AACB */
  0x0000,   /* U+AACC */
  0x0000,   /* U+AACD */
  0x0000,   /* U+AACE */
  0x0000,   /* U+AACF */
  0x0000,   /* U+AAD0 */
  0x0000,   /* U+AAD1 */
  0x0000,   /* U+AAD2 */
  0x0000,   /* U+AAD3 */
  0x0000,   /* U+AAD4 */
  0x0000,   /* U+AAD5 */
  0x0000,   /* U+AAD6 */
  0x0000,   /* U+AAD7 */
  0x0000,   /* U+AAD8 */
  0x0000,   /* U+AAD9 */
  0x0000,   /* U+AADA */
  0x0000,   /* U+AADB */
  0x0000,   /* U+AADC */
  0x0000,   /* U+AADD */
  0x0000,   /* U+AADE */
  0x0000,   /* U+AADF */
  0x0000,   /* U+AAE0 */
  0x0000,   /* U+AAE1 */
  0x0000,   /* U+AAE2 */
  0x0000,   /* U+AAE3 */
  0x0000,   /* U+AAE4 */
  0x0000,   /* U+AAE5 */
  0x0000,   /* U+AAE6 */
  0x0000,   /* U+AAE7 */
  0x0000,   /* U+AAE8 */
  0x0000,   /* U+AAE9 */
  0x0000,   /* U+AAEA */
  0x0000,   /* U+AAEB */
  0x0000,   /* U+AAEC */
  0x0000,   /* U+AAED */
  0x0000,   /* U+AAEE */
  0x0000,   /* U+AAEF */
  0x0000,   /* U+AAF0 */
  0x0000,   /* U+AAF1 */
  0x0000,   /* U+AAF2 */
  0x0000,   /* U+AAF3 */
  0x0000,   /* U+AAF4 */
  0x0000,   /* U+AAF5 */
  0x0000,   /* U+AAF6 */
  0x0000,   /* U+AAF7 */
  0x0000,   /* U+AAF8 */
  0x0000,   /* U+AAF9 */
  0x0000,   /* U+AAFA */
  0x0000,   /* U+AAFB */
  0x0000,   /* U+AAFC */
  0x0000,   /* U+AAFD */
  0x0000,   /* U+AAFE */
  0x0000,   /* U+AAFF */
};

uint16 uca900_p0AB[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 2, 
  2, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC1,   /* U+AB00 */
  0x24D6,   /* U+AB01 */
  0x24D7,   /* U+AB02 */
  0x24D8,   /* U+AB03 */
  0x24D9,   /* U+AB04 */
  0x24DA,   /* U+AB05 */
  0x24DB,   /* U+AB06 */
  0xFBC1,   /* U+AB07 */
  0xFBC1,   /* U+AB08 */
  0x259D,   /* U+AB09 */
  0x259E,   /* U+AB0A */
  0x259F,   /* U+AB0B */
  0x25A0,   /* U+AB0C */
  0x25A1,   /* U+AB0D */
  0x25A2,   /* U+AB0E */
  0xFBC1,   /* U+AB0F */
  0xFBC1,   /* U+AB10 */
  0x257E,   /* U+AB11 */
  0x257F,   /* U+AB12 */
  0x2580,   /* U+AB13 */
  0x2581,   /* U+AB14 */
  0x2582,   /* U+AB15 */
  0x2583,   /* U+AB16 */
  0xFBC1,   /* U+AB17 */
  0xFBC1,   /* U+AB18 */
  0xFBC1,   /* U+AB19 */
  0xFBC1,   /* U+AB1A */
  0xFBC1,   /* U+AB1B */
  0xFBC1,   /* U+AB1C */
  0xFBC1,   /* U+AB1D */
  0xFBC1,   /* U+AB1E */
  0xFBC1,   /* U+AB1F */
  0x25E0,   /* U+AB20 */
  0x25E1,   /* U+AB21 */
  0x25E2,   /* U+AB22 */
  0x25E3,   /* U+AB23 */
  0x25E4,   /* U+AB24 */
  0x25E5,   /* U+AB25 */
  0x25E6,   /* U+AB26 */
  0xFBC1,   /* U+AB27 */
  0x25F8,   /* U+AB28 */
  0x25F9,   /* U+AB29 */
  0x25FA,   /* U+AB2A */
  0x25FB,   /* U+AB2B */
  0x25FC,   /* U+AB2C */
  0x25FD,   /* U+AB2D */
  0x25FE,   /* U+AB2E */
  0xFBC1,   /* U+AB2F */
  0x1C59,   /* U+AB30 */
  0x1C50,   /* U+AB31 */
  0x1CAF,   /* U+AB32 */
  0x1CB0,   /* U+AB33 */
  0x1CB6,   /* U+AB34 */
  0x1CEA,   /* U+AB35 */
  0x1CFC,   /* U+AB36 */
  0x1D91,   /* U+AB37 */
  0x1D8B,   /* U+AB38 */
  0x1D8C,   /* U+AB39 */
  0x1DB5,   /* U+AB3A */
  0x1DD6,   /* U+AB3B */
  0x1DDC,   /* U+AB3C */
  0x1DE3,   /* U+AB3D */
  0x1DEF,   /* U+AB3E */
  0x1DF6,   /* U+AB3F */
  0x1DEB,   /* U+AB40 */
  0x1DE9,   /* U+AB41 */
  0x1DEA,   /* U+AB42 */
  0x1DEC,   /* U+AB43 */
  0x1DED,   /* U+AB44 */
  0x1E37,   /* U+AB45 */
  0x1E3C,   /* U+AB46 */
  0x1E65,   /* U+AB47 */
  0x1E66,   /* U+AB48 */
  0x1E5B,   /* U+AB49 */
  0x1E67,   /* U+AB4A */
  0x1E68,   /* U+AB4B */
  0x1E69,   /* U+AB4C */
  0x1E86,   /* U+AB4D */
  0x1EBA,   /* U+AB4E */
  0x1EC4,   /* U+AB4F */
  0x1EBE,   /* U+AB50 */
  0x1EBF,   /* U+AB51 */
  0x1EC7,   /* U+AB52 */
  0x1F08,   /* U+AB53 */
  0x1F09,   /* U+AB54 */
  0x1F0A,   /* U+AB55 */
  0x1F04,   /* U+AB56 */
  0x1F05,   /* U+AB57 */
  0x1F06,   /* U+AB58 */
  0x1F07,   /* U+AB59 */
  0x1F1C,   /* U+AB5A */
  0x04F5,   /* U+AB5B */
  0x1D2B,   /* U+AB5C */
  0x1D91,   /* U+AB5D */
  0x1D87,   /* U+AB5E */
  0x1EC7,   /* U+AB5F */
  0x1F5B,   /* U+AB60 */
  0x1F5C,   /* U+AB61 */
  0x1DF8,   /* U+AB62 */
  0x1F5D,   /* U+AB63 */
  0x1C5F,   /* U+AB64 */
  0x1FE2,   /* U+AB65 */
  0xFBC1,   /* U+AB66 */
  0xFBC1,   /* U+AB67 */
  0xFBC1,   /* U+AB68 */
  0xFBC1,   /* U+AB69 */
  0xFBC1,   /* U+AB6A */
  0xFBC1,   /* U+AB6B */
  0xFBC1,   /* U+AB6C */
  0xFBC1,   /* U+AB6D */
  0xFBC1,   /* U+AB6E */
  0xFBC1,   /* U+AB6F */
  0x337F,   /* U+AB70 */
  0x3380,   /* U+AB71 */
  0x3381,   /* U+AB72 */
  0x3382,   /* U+AB73 */
  0x3383,   /* U+AB74 */
  0x3384,   /* U+AB75 */
  0x3385,   /* U+AB76 */
  0x3386,   /* U+AB77 */
  0x3387,   /* U+AB78 */
  0x3388,   /* U+AB79 */
  0x3389,   /* U+AB7A */
  0x338A,   /* U+AB7B */
  0x338B,   /* U+AB7C */
  0x338C,   /* U+AB7D */
  0x338D,   /* U+AB7E */
  0x338E,   /* U+AB7F */
  0x338F,   /* U+AB80 */
  0x3390,   /* U+AB81 */
  0x3391,   /* U+AB82 */
  0x3392,   /* U+AB83 */
  0x3393,   /* U+AB84 */
  0x3394,   /* U+AB85 */
  0x3395,   /* U+AB86 */
  0x3396,   /* U+AB87 */
  0x3397,   /* U+AB88 */
  0x3398,   /* U+AB89 */
  0x3399,   /* U+AB8A */
  0x339A,   /* U+AB8B */
  0x339B,   /* U+AB8C */
  0x339C,   /* U+AB8D */
  0x339D,   /* U+AB8E */
  0x339E,   /* U+AB8F */
  0x339F,   /* U+AB90 */
  0x33A0,   /* U+AB91 */
  0x33A1,   /* U+AB92 */
  0x33A2,   /* U+AB93 */
  0x33A3,   /* U+AB94 */
  0x33A4,   /* U+AB95 */
  0x33A5,   /* U+AB96 */
  0x33A6,   /* U+AB97 */
  0x33A7,   /* U+AB98 */
  0x33A8,   /* U+AB99 */
  0x33A9,   /* U+AB9A */
  0x33AA,   /* U+AB9B */
  0x33AB,   /* U+AB9C */
  0x33AC,   /* U+AB9D */
  0x33AD,   /* U+AB9E */
  0x33AE,   /* U+AB9F */
  0x33AF,   /* U+ABA0 */
  0x33B0,   /* U+ABA1 */
  0x33B1,   /* U+ABA2 */
  0x33B2,   /* U+ABA3 */
  0x33B3,   /* U+ABA4 */
  0x33B4,   /* U+ABA5 */
  0x33B5,   /* U+ABA6 */
  0x33B6,   /* U+ABA7 */
  0x33B7,   /* U+ABA8 */
  0x33B8,   /* U+ABA9 */
  0x33B9,   /* U+ABAA */
  0x33BA,   /* U+ABAB */
  0x33BB,   /* U+ABAC */
  0x33BC,   /* U+ABAD */
  0x33BD,   /* U+ABAE */
  0x33BE,   /* U+ABAF */
  0x33BF,   /* U+ABB0 */
  0x33C0,   /* U+ABB1 */
  0x33C1,   /* U+ABB2 */
  0x33C2,   /* U+ABB3 */
  0x33C3,   /* U+ABB4 */
  0x33C4,   /* U+ABB5 */
  0x33C5,   /* U+ABB6 */
  0x33C6,   /* U+ABB7 */
  0x33C7,   /* U+ABB8 */
  0x33C8,   /* U+ABB9 */
  0x33C9,   /* U+ABBA */
  0x33CA,   /* U+ABBB */
  0x33CB,   /* U+ABBC */
  0x33CC,   /* U+ABBD */
  0x33CD,   /* U+ABBE */
  0x33CE,   /* U+ABBF */
  0x291E,   /* U+ABC0 */
  0x291F,   /* U+ABC1 */
  0x2920,   /* U+ABC2 */
  0x2921,   /* U+ABC3 */
  0x2922,   /* U+ABC4 */
  0x2923,   /* U+ABC5 */
  0x2924,   /* U+ABC6 */
  0x2925,   /* U+ABC7 */
  0x2926,   /* U+ABC8 */
  0x2927,   /* U+ABC9 */
  0x2928,   /* U+ABCA */
  0x2929,   /* U+ABCB */
  0x292A,   /* U+ABCC */
  0x292B,   /* U+ABCD */
  0x292C,   /* U+ABCE */
  0x292D,   /* U+ABCF */
  0x292E,   /* U+ABD0 */
  0x292F,   /* U+ABD1 */
  0x2930,   /* U+ABD2 */
  0x2931,   /* U+ABD3 */
  0x2932,   /* U+ABD4 */
  0x2933,   /* U+ABD5 */
  0x2934,   /* U+ABD6 */
  0x2935,   /* U+ABD7 */
  0x2936,   /* U+ABD8 */
  0x2937,   /* U+ABD9 */
  0x2938,   /* U+ABDA */
  0x2952,   /* U+ABDB */
  0x2953,   /* U+ABDC */
  0x2954,   /* U+ABDD */
  0x2955,   /* U+ABDE */
  0x2956,   /* U+ABDF */
  0x2957,   /* U+ABE0 */
  0x2958,   /* U+ABE1 */
  0x2959,   /* U+ABE2 */
  0x2944,   /* U+ABE3 */
  0x2945,   /* U+ABE4 */
  0x2946,   /* U+ABE5 */
  0x2947,   /* U+ABE6 */
  0x2948,   /* U+ABE7 */
  0x2949,   /* U+ABE8 */
  0x294A,   /* U+ABE9 */
  0x294B,   /* U+ABEA */
  0x02A9,   /* U+ABEB */
  0x0000,   /* U+ABEC */
  0x295A,   /* U+ABED */
  0xFBC1,   /* U+ABEE */
  0xFBC1,   /* U+ABEF */
  0x1C3D,   /* U+ABF0 */
  0x1C3E,   /* U+ABF1 */
  0x1C3F,   /* U+ABF2 */
  0x1C40,   /* U+ABF3 */
  0x1C41,   /* U+ABF4 */
  0x1C42,   /* U+ABF5 */
  0x1C43,   /* U+ABF6 */
  0x1C44,   /* U+ABF7 */
  0x1C45,   /* U+ABF8 */
  0x1C46,   /* U+ABF9 */
  0xFBC1,   /* U+ABFA */
  0xFBC1,   /* U+ABFB */
  0xFBC1,   /* U+ABFC */
  0xFBC1,   /* U+ABFD */
  0xFBC1,   /* U+ABFE */
  0xFBC1,   /* U+ABFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+AB00 */
  0x0020,   /* U+AB01 */
  0x0020,   /* U+AB02 */
  0x0020,   /* U+AB03 */
  0x0020,   /* U+AB04 */
  0x0020,   /* U+AB05 */
  0x0020,   /* U+AB06 */
  0x0020,   /* U+AB07 */
  0x0020,   /* U+AB08 */
  0x0020,   /* U+AB09 */
  0x0020,   /* U+AB0A */
  0x0020,   /* U+AB0B */
  0x0020,   /* U+AB0C */
  0x0020,   /* U+AB0D */
  0x0020,   /* U+AB0E */
  0x0020,   /* U+AB0F */
  0x0020,   /* U+AB10 */
  0x0020,   /* U+AB11 */
  0x0020,   /* U+AB12 */
  0x0020,   /* U+AB13 */
  0x0020,   /* U+AB14 */
  0x0020,   /* U+AB15 */
  0x0020,   /* U+AB16 */
  0x0020,   /* U+AB17 */
  0x0020,   /* U+AB18 */
  0x0020,   /* U+AB19 */
  0x0020,   /* U+AB1A */
  0x0020,   /* U+AB1B */
  0x0020,   /* U+AB1C */
  0x0020,   /* U+AB1D */
  0x0020,   /* U+AB1E */
  0x0020,   /* U+AB1F */
  0x0020,   /* U+AB20 */
  0x0020,   /* U+AB21 */
  0x0020,   /* U+AB22 */
  0x0020,   /* U+AB23 */
  0x0020,   /* U+AB24 */
  0x0020,   /* U+AB25 */
  0x0020,   /* U+AB26 */
  0x0020,   /* U+AB27 */
  0x0020,   /* U+AB28 */
  0x0020,   /* U+AB29 */
  0x0020,   /* U+AB2A */
  0x0020,   /* U+AB2B */
  0x0020,   /* U+AB2C */
  0x0020,   /* U+AB2D */
  0x0020,   /* U+AB2E */
  0x0020,   /* U+AB2F */
  0x0020,   /* U+AB30 */
  0x0020,   /* U+AB31 */
  0x0020,   /* U+AB32 */
  0x0020,   /* U+AB33 */
  0x0020,   /* U+AB34 */
  0x0020,   /* U+AB35 */
  0x0020,   /* U+AB36 */
  0x0020,   /* U+AB37 */
  0x0020,   /* U+AB38 */
  0x0020,   /* U+AB39 */
  0x0020,   /* U+AB3A */
  0x0020,   /* U+AB3B */
  0x0020,   /* U+AB3C */
  0x0020,   /* U+AB3D */
  0x0020,   /* U+AB3E */
  0x0020,   /* U+AB3F */
  0x0020,   /* U+AB40 */
  0x0020,   /* U+AB41 */
  0x0020,   /* U+AB42 */
  0x0020,   /* U+AB43 */
  0x0020,   /* U+AB44 */
  0x0020,   /* U+AB45 */
  0x0020,   /* U+AB46 */
  0x0020,   /* U+AB47 */
  0x0020,   /* U+AB48 */
  0x0020,   /* U+AB49 */
  0x0020,   /* U+AB4A */
  0x0020,   /* U+AB4B */
  0x0020,   /* U+AB4C */
  0x0020,   /* U+AB4D */
  0x0020,   /* U+AB4E */
  0x0020,   /* U+AB4F */
  0x0020,   /* U+AB50 */
  0x0020,   /* U+AB51 */
  0x0020,   /* U+AB52 */
  0x0020,   /* U+AB53 */
  0x0020,   /* U+AB54 */
  0x0020,   /* U+AB55 */
  0x0020,   /* U+AB56 */
  0x0020,   /* U+AB57 */
  0x0020,   /* U+AB58 */
  0x0020,   /* U+AB59 */
  0x0020,   /* U+AB5A */
  0x0020,   /* U+AB5B */
  0x0020,   /* U+AB5C */
  0x0020,   /* U+AB5D */
  0x0020,   /* U+AB5E */
  0x0020,   /* U+AB5F */
  0x0020,   /* U+AB60 */
  0x0020,   /* U+AB61 */
  0x0020,   /* U+AB62 */
  0x0020,   /* U+AB63 */
  0x0020,   /* U+AB64 */
  0x0020,   /* U+AB65 */
  0x0020,   /* U+AB66 */
  0x0020,   /* U+AB67 */
  0x0020,   /* U+AB68 */
  0x0020,   /* U+AB69 */
  0x0020,   /* U+AB6A */
  0x0020,   /* U+AB6B */
  0x0020,   /* U+AB6C */
  0x0020,   /* U+AB6D */
  0x0020,   /* U+AB6E */
  0x0020,   /* U+AB6F */
  0x0020,   /* U+AB70 */
  0x0020,   /* U+AB71 */
  0x0020,   /* U+AB72 */
  0x0020,   /* U+AB73 */
  0x0020,   /* U+AB74 */
  0x0020,   /* U+AB75 */
  0x0020,   /* U+AB76 */
  0x0020,   /* U+AB77 */
  0x0020,   /* U+AB78 */
  0x0020,   /* U+AB79 */
  0x0020,   /* U+AB7A */
  0x0020,   /* U+AB7B */
  0x0020,   /* U+AB7C */
  0x0020,   /* U+AB7D */
  0x0020,   /* U+AB7E */
  0x0020,   /* U+AB7F */
  0x0020,   /* U+AB80 */
  0x0020,   /* U+AB81 */
  0x0020,   /* U+AB82 */
  0x0020,   /* U+AB83 */
  0x0020,   /* U+AB84 */
  0x0020,   /* U+AB85 */
  0x0020,   /* U+AB86 */
  0x0020,   /* U+AB87 */
  0x0020,   /* U+AB88 */
  0x0020,   /* U+AB89 */
  0x0020,   /* U+AB8A */
  0x0020,   /* U+AB8B */
  0x0020,   /* U+AB8C */
  0x0020,   /* U+AB8D */
  0x0020,   /* U+AB8E */
  0x0020,   /* U+AB8F */
  0x0020,   /* U+AB90 */
  0x0020,   /* U+AB91 */
  0x0020,   /* U+AB92 */
  0x0020,   /* U+AB93 */
  0x0020,   /* U+AB94 */
  0x0020,   /* U+AB95 */
  0x0020,   /* U+AB96 */
  0x0020,   /* U+AB97 */
  0x0020,   /* U+AB98 */
  0x0020,   /* U+AB99 */
  0x0020,   /* U+AB9A */
  0x0020,   /* U+AB9B */
  0x0020,   /* U+AB9C */
  0x0020,   /* U+AB9D */
  0x0020,   /* U+AB9E */
  0x0020,   /* U+AB9F */
  0x0020,   /* U+ABA0 */
  0x0020,   /* U+ABA1 */
  0x0020,   /* U+ABA2 */
  0x0020,   /* U+ABA3 */
  0x0020,   /* U+ABA4 */
  0x0020,   /* U+ABA5 */
  0x0020,   /* U+ABA6 */
  0x0020,   /* U+ABA7 */
  0x0020,   /* U+ABA8 */
  0x0020,   /* U+ABA9 */
  0x0020,   /* U+ABAA */
  0x0020,   /* U+ABAB */
  0x0020,   /* U+ABAC */
  0x0020,   /* U+ABAD */
  0x0020,   /* U+ABAE */
  0x0020,   /* U+ABAF */
  0x0020,   /* U+ABB0 */
  0x0020,   /* U+ABB1 */
  0x0020,   /* U+ABB2 */
  0x0020,   /* U+ABB3 */
  0x0020,   /* U+ABB4 */
  0x0020,   /* U+ABB5 */
  0x0020,   /* U+ABB6 */
  0x0020,   /* U+ABB7 */
  0x0020,   /* U+ABB8 */
  0x0020,   /* U+ABB9 */
  0x0020,   /* U+ABBA */
  0x0020,   /* U+ABBB */
  0x0020,   /* U+ABBC */
  0x0020,   /* U+ABBD */
  0x0020,   /* U+ABBE */
  0x0020,   /* U+ABBF */
  0x0020,   /* U+ABC0 */
  0x0020,   /* U+ABC1 */
  0x0020,   /* U+ABC2 */
  0x0020,   /* U+ABC3 */
  0x0020,   /* U+ABC4 */
  0x0020,   /* U+ABC5 */
  0x0020,   /* U+ABC6 */
  0x0020,   /* U+ABC7 */
  0x0020,   /* U+ABC8 */
  0x0020,   /* U+ABC9 */
  0x0020,   /* U+ABCA */
  0x0020,   /* U+ABCB */
  0x0020,   /* U+ABCC */
  0x0020,   /* U+ABCD */
  0x0020,   /* U+ABCE */
  0x0020,   /* U+ABCF */
  0x0020,   /* U+ABD0 */
  0x0020,   /* U+ABD1 */
  0x0020,   /* U+ABD2 */
  0x0020,   /* U+ABD3 */
  0x0020,   /* U+ABD4 */
  0x0020,   /* U+ABD5 */
  0x0020,   /* U+ABD6 */
  0x0020,   /* U+ABD7 */
  0x0020,   /* U+ABD8 */
  0x0020,   /* U+ABD9 */
  0x0020,   /* U+ABDA */
  0x0020,   /* U+ABDB */
  0x0020,   /* U+ABDC */
  0x0020,   /* U+ABDD */
  0x0020,   /* U+ABDE */
  0x0020,   /* U+ABDF */
  0x0020,   /* U+ABE0 */
  0x0020,   /* U+ABE1 */
  0x0020,   /* U+ABE2 */
  0x0020,   /* U+ABE3 */
  0x0020,   /* U+ABE4 */
  0x0020,   /* U+ABE5 */
  0x0020,   /* U+ABE6 */
  0x0020,   /* U+ABE7 */
  0x0020,   /* U+ABE8 */
  0x0020,   /* U+ABE9 */
  0x0020,   /* U+ABEA */
  0x0020,   /* U+ABEB */
  0x00CB,   /* U+ABEC */
  0x0020,   /* U+ABED */
  0x0020,   /* U+ABEE */
  0x0020,   /* U+ABEF */
  0x0020,   /* U+ABF0 */
  0x0020,   /* U+ABF1 */
  0x0020,   /* U+ABF2 */
  0x0020,   /* U+ABF3 */
  0x0020,   /* U+ABF4 */
  0x0020,   /* U+ABF5 */
  0x0020,   /* U+ABF6 */
  0x0020,   /* U+ABF7 */
  0x0020,   /* U+ABF8 */
  0x0020,   /* U+ABF9 */
  0x0020,   /* U+ABFA */
  0x0020,   /* U+ABFB */
  0x0020,   /* U+ABFC */
  0x0020,   /* U+ABFD */
  0x0020,   /* U+ABFE */
  0x0020,   /* U+ABFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+AB00 */
  0x0002,   /* U+AB01 */
  0x0002,   /* U+AB02 */
  0x0002,   /* U+AB03 */
  0x0002,   /* U+AB04 */
  0x0002,   /* U+AB05 */
  0x0002,   /* U+AB06 */
  0x0002,   /* U+AB07 */
  0x0002,   /* U+AB08 */
  0x0002,   /* U+AB09 */
  0x0002,   /* U+AB0A */
  0x0002,   /* U+AB0B */
  0x0002,   /* U+AB0C */
  0x0002,   /* U+AB0D */
  0x0002,   /* U+AB0E */
  0x0002,   /* U+AB0F */
  0x0002,   /* U+AB10 */
  0x0002,   /* U+AB11 */
  0x0002,   /* U+AB12 */
  0x0002,   /* U+AB13 */
  0x0002,   /* U+AB14 */
  0x0002,   /* U+AB15 */
  0x0002,   /* U+AB16 */
  0x0002,   /* U+AB17 */
  0x0002,   /* U+AB18 */
  0x0002,   /* U+AB19 */
  0x0002,   /* U+AB1A */
  0x0002,   /* U+AB1B */
  0x0002,   /* U+AB1C */
  0x0002,   /* U+AB1D */
  0x0002,   /* U+AB1E */
  0x0002,   /* U+AB1F */
  0x0002,   /* U+AB20 */
  0x0002,   /* U+AB21 */
  0x0002,   /* U+AB22 */
  0x0002,   /* U+AB23 */
  0x0002,   /* U+AB24 */
  0x0002,   /* U+AB25 */
  0x0002,   /* U+AB26 */
  0x0002,   /* U+AB27 */
  0x0002,   /* U+AB28 */
  0x0002,   /* U+AB29 */
  0x0002,   /* U+AB2A */
  0x0002,   /* U+AB2B */
  0x0002,   /* U+AB2C */
  0x0002,   /* U+AB2D */
  0x0002,   /* U+AB2E */
  0x0002,   /* U+AB2F */
  0x0002,   /* U+AB30 */
  0x0002,   /* U+AB31 */
  0x0002,   /* U+AB32 */
  0x0002,   /* U+AB33 */
  0x0002,   /* U+AB34 */
  0x0002,   /* U+AB35 */
  0x0002,   /* U+AB36 */
  0x0002,   /* U+AB37 */
  0x0002,   /* U+AB38 */
  0x0002,   /* U+AB39 */
  0x0002,   /* U+AB3A */
  0x0002,   /* U+AB3B */
  0x0002,   /* U+AB3C */
  0x0002,   /* U+AB3D */
  0x0002,   /* U+AB3E */
  0x0002,   /* U+AB3F */
  0x0002,   /* U+AB40 */
  0x0002,   /* U+AB41 */
  0x0002,   /* U+AB42 */
  0x0002,   /* U+AB43 */
  0x0002,   /* U+AB44 */
  0x0002,   /* U+AB45 */
  0x0002,   /* U+AB46 */
  0x0002,   /* U+AB47 */
  0x0002,   /* U+AB48 */
  0x0002,   /* U+AB49 */
  0x0002,   /* U+AB4A */
  0x0002,   /* U+AB4B */
  0x0002,   /* U+AB4C */
  0x0002,   /* U+AB4D */
  0x0002,   /* U+AB4E */
  0x0002,   /* U+AB4F */
  0x0002,   /* U+AB50 */
  0x0002,   /* U+AB51 */
  0x0002,   /* U+AB52 */
  0x0002,   /* U+AB53 */
  0x0002,   /* U+AB54 */
  0x0002,   /* U+AB55 */
  0x0002,   /* U+AB56 */
  0x0002,   /* U+AB57 */
  0x0002,   /* U+AB58 */
  0x0002,   /* U+AB59 */
  0x0002,   /* U+AB5A */
  0x0002,   /* U+AB5B */
  0x0014,   /* U+AB5C */
  0x0014,   /* U+AB5D */
  0x0014,   /* U+AB5E */
  0x0014,   /* U+AB5F */
  0x0002,   /* U+AB60 */
  0x0002,   /* U+AB61 */
  0x0002,   /* U+AB62 */
  0x0002,   /* U+AB63 */
  0x0002,   /* U+AB64 */
  0x0002,   /* U+AB65 */
  0x0002,   /* U+AB66 */
  0x0002,   /* U+AB67 */
  0x0002,   /* U+AB68 */
  0x0002,   /* U+AB69 */
  0x0002,   /* U+AB6A */
  0x0002,   /* U+AB6B */
  0x0002,   /* U+AB6C */
  0x0002,   /* U+AB6D */
  0x0002,   /* U+AB6E */
  0x0002,   /* U+AB6F */
  0x0002,   /* U+AB70 */
  0x0002,   /* U+AB71 */
  0x0002,   /* U+AB72 */
  0x0002,   /* U+AB73 */
  0x0002,   /* U+AB74 */
  0x0002,   /* U+AB75 */
  0x0002,   /* U+AB76 */
  0x0002,   /* U+AB77 */
  0x0002,   /* U+AB78 */
  0x0002,   /* U+AB79 */
  0x0002,   /* U+AB7A */
  0x0002,   /* U+AB7B */
  0x0002,   /* U+AB7C */
  0x0002,   /* U+AB7D */
  0x0002,   /* U+AB7E */
  0x0002,   /* U+AB7F */
  0x0002,   /* U+AB80 */
  0x0002,   /* U+AB81 */
  0x0002,   /* U+AB82 */
  0x0002,   /* U+AB83 */
  0x0002,   /* U+AB84 */
  0x0002,   /* U+AB85 */
  0x0002,   /* U+AB86 */
  0x0002,   /* U+AB87 */
  0x0002,   /* U+AB88 */
  0x0002,   /* U+AB89 */
  0x0002,   /* U+AB8A */
  0x0002,   /* U+AB8B */
  0x0002,   /* U+AB8C */
  0x0002,   /* U+AB8D */
  0x0002,   /* U+AB8E */
  0x0002,   /* U+AB8F */
  0x0002,   /* U+AB90 */
  0x0002,   /* U+AB91 */
  0x0002,   /* U+AB92 */
  0x0002,   /* U+AB93 */
  0x0002,   /* U+AB94 */
  0x0002,   /* U+AB95 */
  0x0002,   /* U+AB96 */
  0x0002,   /* U+AB97 */
  0x0002,   /* U+AB98 */
  0x0002,   /* U+AB99 */
  0x0002,   /* U+AB9A */
  0x0002,   /* U+AB9B */
  0x0002,   /* U+AB9C */
  0x0002,   /* U+AB9D */
  0x0002,   /* U+AB9E */
  0x0002,   /* U+AB9F */
  0x0002,   /* U+ABA0 */
  0x0002,   /* U+ABA1 */
  0x0002,   /* U+ABA2 */
  0x0002,   /* U+ABA3 */
  0x0002,   /* U+ABA4 */
  0x0002,   /* U+ABA5 */
  0x0002,   /* U+ABA6 */
  0x0002,   /* U+ABA7 */
  0x0002,   /* U+ABA8 */
  0x0002,   /* U+ABA9 */
  0x0002,   /* U+ABAA */
  0x0002,   /* U+ABAB */
  0x0002,   /* U+ABAC */
  0x0002,   /* U+ABAD */
  0x0002,   /* U+ABAE */
  0x0002,   /* U+ABAF */
  0x0002,   /* U+ABB0 */
  0x0002,   /* U+ABB1 */
  0x0002,   /* U+ABB2 */
  0x0002,   /* U+ABB3 */
  0x0002,   /* U+ABB4 */
  0x0002,   /* U+ABB5 */
  0x0002,   /* U+ABB6 */
  0x0002,   /* U+ABB7 */
  0x0002,   /* U+ABB8 */
  0x0002,   /* U+ABB9 */
  0x0002,   /* U+ABBA */
  0x0002,   /* U+ABBB */
  0x0002,   /* U+ABBC */
  0x0002,   /* U+ABBD */
  0x0002,   /* U+ABBE */
  0x0002,   /* U+ABBF */
  0x0002,   /* U+ABC0 */
  0x0002,   /* U+ABC1 */
  0x0002,   /* U+ABC2 */
  0x0002,   /* U+ABC3 */
  0x0002,   /* U+ABC4 */
  0x0002,   /* U+ABC5 */
  0x0002,   /* U+ABC6 */
  0x0002,   /* U+ABC7 */
  0x0002,   /* U+ABC8 */
  0x0002,   /* U+ABC9 */
  0x0002,   /* U+ABCA */
  0x0002,   /* U+ABCB */
  0x0002,   /* U+ABCC */
  0x0002,   /* U+ABCD */
  0x0002,   /* U+ABCE */
  0x0002,   /* U+ABCF */
  0x0002,   /* U+ABD0 */
  0x0002,   /* U+ABD1 */
  0x0002,   /* U+ABD2 */
  0x0002,   /* U+ABD3 */
  0x0002,   /* U+ABD4 */
  0x0002,   /* U+ABD5 */
  0x0002,   /* U+ABD6 */
  0x0002,   /* U+ABD7 */
  0x0002,   /* U+ABD8 */
  0x0002,   /* U+ABD9 */
  0x0002,   /* U+ABDA */
  0x0002,   /* U+ABDB */
  0x0002,   /* U+ABDC */
  0x0002,   /* U+ABDD */
  0x0002,   /* U+ABDE */
  0x0002,   /* U+ABDF */
  0x0002,   /* U+ABE0 */
  0x0002,   /* U+ABE1 */
  0x0002,   /* U+ABE2 */
  0x0002,   /* U+ABE3 */
  0x0002,   /* U+ABE4 */
  0x0002,   /* U+ABE5 */
  0x0002,   /* U+ABE6 */
  0x0002,   /* U+ABE7 */
  0x0002,   /* U+ABE8 */
  0x0002,   /* U+ABE9 */
  0x0002,   /* U+ABEA */
  0x0002,   /* U+ABEB */
  0x0002,   /* U+ABEC */
  0x0002,   /* U+ABED */
  0x0002,   /* U+ABEE */
  0x0002,   /* U+ABEF */
  0x0002,   /* U+ABF0 */
  0x0002,   /* U+ABF1 */
  0x0002,   /* U+ABF2 */
  0x0002,   /* U+ABF3 */
  0x0002,   /* U+ABF4 */
  0x0002,   /* U+ABF5 */
  0x0002,   /* U+ABF6 */
  0x0002,   /* U+ABF7 */
  0x0002,   /* U+ABF8 */
  0x0002,   /* U+ABF9 */
  0x0002,   /* U+ABFA */
  0x0002,   /* U+ABFB */
  0x0002,   /* U+ABFC */
  0x0002,   /* U+ABFD */
  0x0002,   /* U+ABFE */
  0x0002,   /* U+ABFF */

  /* Primary weight 2 for each character. */
  0xAB00,   /* U+AB00 */
  0x0000,   /* U+AB01 */
  0x0000,   /* U+AB02 */
  0x0000,   /* U+AB03 */
  0x0000,   /* U+AB04 */
  0x0000,   /* U+AB05 */
  0x0000,   /* U+AB06 */
  0xAB07,   /* U+AB07 */
  0xAB08,   /* U+AB08 */
  0x0000,   /* U+AB09 */
  0x0000,   /* U+AB0A */
  0x0000,   /* U+AB0B */
  0x0000,   /* U+AB0C */
  0x0000,   /* U+AB0D */
  0x0000,   /* U+AB0E */
  0xAB0F,   /* U+AB0F */
  0xAB10,   /* U+AB10 */
  0x0000,   /* U+AB11 */
  0x0000,   /* U+AB12 */
  0x0000,   /* U+AB13 */
  0x0000,   /* U+AB14 */
  0x0000,   /* U+AB15 */
  0x0000,   /* U+AB16 */
  0xAB17,   /* U+AB17 */
  0xAB18,   /* U+AB18 */
  0xAB19,   /* U+AB19 */
  0xAB1A,   /* U+AB1A */
  0xAB1B,   /* U+AB1B */
  0xAB1C,   /* U+AB1C */
  0xAB1D,   /* U+AB1D */
  0xAB1E,   /* U+AB1E */
  0xAB1F,   /* U+AB1F */
  0x0000,   /* U+AB20 */
  0x0000,   /* U+AB21 */
  0x0000,   /* U+AB22 */
  0x0000,   /* U+AB23 */
  0x0000,   /* U+AB24 */
  0x0000,   /* U+AB25 */
  0x0000,   /* U+AB26 */
  0xAB27,   /* U+AB27 */
  0x0000,   /* U+AB28 */
  0x0000,   /* U+AB29 */
  0x0000,   /* U+AB2A */
  0x0000,   /* U+AB2B */
  0x0000,   /* U+AB2C */
  0x0000,   /* U+AB2D */
  0x0000,   /* U+AB2E */
  0xAB2F,   /* U+AB2F */
  0x0000,   /* U+AB30 */
  0x0000,   /* U+AB31 */
  0x0000,   /* U+AB32 */
  0x0000,   /* U+AB33 */
  0x0000,   /* U+AB34 */
  0x0000,   /* U+AB35 */
  0x0000,   /* U+AB36 */
  0x0000,   /* U+AB37 */
  0x0000,   /* U+AB38 */
  0x0000,   /* U+AB39 */
  0x0000,   /* U+AB3A */
  0x0000,   /* U+AB3B */
  0x0000,   /* U+AB3C */
  0x0000,   /* U+AB3D */
  0x0000,   /* U+AB3E */
  0x0000,   /* U+AB3F */
  0x0000,   /* U+AB40 */
  0x0000,   /* U+AB41 */
  0x0000,   /* U+AB42 */
  0x0000,   /* U+AB43 */
  0x0000,   /* U+AB44 */
  0x0000,   /* U+AB45 */
  0x0000,   /* U+AB46 */
  0x0000,   /* U+AB47 */
  0x0000,   /* U+AB48 */
  0x0000,   /* U+AB49 */
  0x0000,   /* U+AB4A */
  0x0000,   /* U+AB4B */
  0x0000,   /* U+AB4C */
  0x0000,   /* U+AB4D */
  0x0000,   /* U+AB4E */
  0x0000,   /* U+AB4F */
  0x0000,   /* U+AB50 */
  0x0000,   /* U+AB51 */
  0x0000,   /* U+AB52 */
  0x0000,   /* U+AB53 */
  0x0000,   /* U+AB54 */
  0x0000,   /* U+AB55 */
  0x0000,   /* U+AB56 */
  0x0000,   /* U+AB57 */
  0x0000,   /* U+AB58 */
  0x0000,   /* U+AB59 */
  0x0000,   /* U+AB5A */
  0x0000,   /* U+AB5B */
  0x0000,   /* U+AB5C */
  0x0000,   /* U+AB5D */
  0x0000,   /* U+AB5E */
  0x0000,   /* U+AB5F */
  0x0000,   /* U+AB60 */
  0x0000,   /* U+AB61 */
  0x0000,   /* U+AB62 */
  0x0000,   /* U+AB63 */
  0x0000,   /* U+AB64 */
  0x0000,   /* U+AB65 */
  0xAB66,   /* U+AB66 */
  0xAB67,   /* U+AB67 */
  0xAB68,   /* U+AB68 */
  0xAB69,   /* U+AB69 */
  0xAB6A,   /* U+AB6A */
  0xAB6B,   /* U+AB6B */
  0xAB6C,   /* U+AB6C */
  0xAB6D,   /* U+AB6D */
  0xAB6E,   /* U+AB6E */
  0xAB6F,   /* U+AB6F */
  0x0000,   /* U+AB70 */
  0x0000,   /* U+AB71 */
  0x0000,   /* U+AB72 */
  0x0000,   /* U+AB73 */
  0x0000,   /* U+AB74 */
  0x0000,   /* U+AB75 */
  0x0000,   /* U+AB76 */
  0x0000,   /* U+AB77 */
  0x0000,   /* U+AB78 */
  0x0000,   /* U+AB79 */
  0x0000,   /* U+AB7A */
  0x0000,   /* U+AB7B */
  0x0000,   /* U+AB7C */
  0x0000,   /* U+AB7D */
  0x0000,   /* U+AB7E */
  0x0000,   /* U+AB7F */
  0x0000,   /* U+AB80 */
  0x0000,   /* U+AB81 */
  0x0000,   /* U+AB82 */
  0x0000,   /* U+AB83 */
  0x0000,   /* U+AB84 */
  0x0000,   /* U+AB85 */
  0x0000,   /* U+AB86 */
  0x0000,   /* U+AB87 */
  0x0000,   /* U+AB88 */
  0x0000,   /* U+AB89 */
  0x0000,   /* U+AB8A */
  0x0000,   /* U+AB8B */
  0x0000,   /* U+AB8C */
  0x0000,   /* U+AB8D */
  0x0000,   /* U+AB8E */
  0x0000,   /* U+AB8F */
  0x0000,   /* U+AB90 */
  0x0000,   /* U+AB91 */
  0x0000,   /* U+AB92 */
  0x0000,   /* U+AB93 */
  0x0000,   /* U+AB94 */
  0x0000,   /* U+AB95 */
  0x0000,   /* U+AB96 */
  0x0000,   /* U+AB97 */
  0x0000,   /* U+AB98 */
  0x0000,   /* U+AB99 */
  0x0000,   /* U+AB9A */
  0x0000,   /* U+AB9B */
  0x0000,   /* U+AB9C */
  0x0000,   /* U+AB9D */
  0x0000,   /* U+AB9E */
  0x0000,   /* U+AB9F */
  0x0000,   /* U+ABA0 */
  0x0000,   /* U+ABA1 */
  0x0000,   /* U+ABA2 */
  0x0000,   /* U+ABA3 */
  0x0000,   /* U+ABA4 */
  0x0000,   /* U+ABA5 */
  0x0000,   /* U+ABA6 */
  0x0000,   /* U+ABA7 */
  0x0000,   /* U+ABA8 */
  0x0000,   /* U+ABA9 */
  0x0000,   /* U+ABAA */
  0x0000,   /* U+ABAB */
  0x0000,   /* U+ABAC */
  0x0000,   /* U+ABAD */
  0x0000,   /* U+ABAE */
  0x0000,   /* U+ABAF */
  0x0000,   /* U+ABB0 */
  0x0000,   /* U+ABB1 */
  0x0000,   /* U+ABB2 */
  0x0000,   /* U+ABB3 */
  0x0000,   /* U+ABB4 */
  0x0000,   /* U+ABB5 */
  0x0000,   /* U+ABB6 */
  0x0000,   /* U+ABB7 */
  0x0000,   /* U+ABB8 */
  0x0000,   /* U+ABB9 */
  0x0000,   /* U+ABBA */
  0x0000,   /* U+ABBB */
  0x0000,   /* U+ABBC */
  0x0000,   /* U+ABBD */
  0x0000,   /* U+ABBE */
  0x0000,   /* U+ABBF */
  0x0000,   /* U+ABC0 */
  0x0000,   /* U+ABC1 */
  0x0000,   /* U+ABC2 */
  0x0000,   /* U+ABC3 */
  0x0000,   /* U+ABC4 */
  0x0000,   /* U+ABC5 */
  0x0000,   /* U+ABC6 */
  0x0000,   /* U+ABC7 */
  0x0000,   /* U+ABC8 */
  0x0000,   /* U+ABC9 */
  0x0000,   /* U+ABCA */
  0x0000,   /* U+ABCB */
  0x0000,   /* U+ABCC */
  0x0000,   /* U+ABCD */
  0x0000,   /* U+ABCE */
  0x0000,   /* U+ABCF */
  0x0000,   /* U+ABD0 */
  0x0000,   /* U+ABD1 */
  0x0000,   /* U+ABD2 */
  0x0000,   /* U+ABD3 */
  0x0000,   /* U+ABD4 */
  0x0000,   /* U+ABD5 */
  0x0000,   /* U+ABD6 */
  0x0000,   /* U+ABD7 */
  0x0000,   /* U+ABD8 */
  0x0000,   /* U+ABD9 */
  0x0000,   /* U+ABDA */
  0x0000,   /* U+ABDB */
  0x0000,   /* U+ABDC */
  0x0000,   /* U+ABDD */
  0x0000,   /* U+ABDE */
  0x0000,   /* U+ABDF */
  0x0000,   /* U+ABE0 */
  0x0000,   /* U+ABE1 */
  0x0000,   /* U+ABE2 */
  0x0000,   /* U+ABE3 */
  0x0000,   /* U+ABE4 */
  0x0000,   /* U+ABE5 */
  0x0000,   /* U+ABE6 */
  0x0000,   /* U+ABE7 */
  0x0000,   /* U+ABE8 */
  0x0000,   /* U+ABE9 */
  0x0000,   /* U+ABEA */
  0x0000,   /* U+ABEB */
  0x0000,   /* U+ABEC */
  0x0000,   /* U+ABED */
  0xABEE,   /* U+ABEE */
  0xABEF,   /* U+ABEF */
  0x0000,   /* U+ABF0 */
  0x0000,   /* U+ABF1 */
  0x0000,   /* U+ABF2 */
  0x0000,   /* U+ABF3 */
  0x0000,   /* U+ABF4 */
  0x0000,   /* U+ABF5 */
  0x0000,   /* U+ABF6 */
  0x0000,   /* U+ABF7 */
  0x0000,   /* U+ABF8 */
  0x0000,   /* U+ABF9 */
  0xABFA,   /* U+ABFA */
  0xABFB,   /* U+ABFB */
  0xABFC,   /* U+ABFC */
  0xABFD,   /* U+ABFD */
  0xABFE,   /* U+ABFE */
  0xABFF,   /* U+ABFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+AB00 */
  0x0000,   /* U+AB01 */
  0x0000,   /* U+AB02 */
  0x0000,   /* U+AB03 */
  0x0000,   /* U+AB04 */
  0x0000,   /* U+AB05 */
  0x0000,   /* U+AB06 */
  0x0000,   /* U+AB07 */
  0x0000,   /* U+AB08 */
  0x0000,   /* U+AB09 */
  0x0000,   /* U+AB0A */
  0x0000,   /* U+AB0B */
  0x0000,   /* U+AB0C */
  0x0000,   /* U+AB0D */
  0x0000,   /* U+AB0E */
  0x0000,   /* U+AB0F */
  0x0000,   /* U+AB10 */
  0x0000,   /* U+AB11 */
  0x0000,   /* U+AB12 */
  0x0000,   /* U+AB13 */
  0x0000,   /* U+AB14 */
  0x0000,   /* U+AB15 */
  0x0000,   /* U+AB16 */
  0x0000,   /* U+AB17 */
  0x0000,   /* U+AB18 */
  0x0000,   /* U+AB19 */
  0x0000,   /* U+AB1A */
  0x0000,   /* U+AB1B */
  0x0000,   /* U+AB1C */
  0x0000,   /* U+AB1D */
  0x0000,   /* U+AB1E */
  0x0000,   /* U+AB1F */
  0x0000,   /* U+AB20 */
  0x0000,   /* U+AB21 */
  0x0000,   /* U+AB22 */
  0x0000,   /* U+AB23 */
  0x0000,   /* U+AB24 */
  0x0000,   /* U+AB25 */
  0x0000,   /* U+AB26 */
  0x0000,   /* U+AB27 */
  0x0000,   /* U+AB28 */
  0x0000,   /* U+AB29 */
  0x0000,   /* U+AB2A */
  0x0000,   /* U+AB2B */
  0x0000,   /* U+AB2C */
  0x0000,   /* U+AB2D */
  0x0000,   /* U+AB2E */
  0x0000,   /* U+AB2F */
  0x0000,   /* U+AB30 */
  0x0000,   /* U+AB31 */
  0x0000,   /* U+AB32 */
  0x0000,   /* U+AB33 */
  0x0000,   /* U+AB34 */
  0x0000,   /* U+AB35 */
  0x0000,   /* U+AB36 */
  0x0000,   /* U+AB37 */
  0x0000,   /* U+AB38 */
  0x0000,   /* U+AB39 */
  0x0000,   /* U+AB3A */
  0x0000,   /* U+AB3B */
  0x0000,   /* U+AB3C */
  0x0000,   /* U+AB3D */
  0x0000,   /* U+AB3E */
  0x0000,   /* U+AB3F */
  0x0000,   /* U+AB40 */
  0x0000,   /* U+AB41 */
  0x0000,   /* U+AB42 */
  0x0000,   /* U+AB43 */
  0x0000,   /* U+AB44 */
  0x0000,   /* U+AB45 */
  0x0000,   /* U+AB46 */
  0x0000,   /* U+AB47 */
  0x0000,   /* U+AB48 */
  0x0000,   /* U+AB49 */
  0x0000,   /* U+AB4A */
  0x0000,   /* U+AB4B */
  0x0000,   /* U+AB4C */
  0x0000,   /* U+AB4D */
  0x0000,   /* U+AB4E */
  0x0000,   /* U+AB4F */
  0x0000,   /* U+AB50 */
  0x0000,   /* U+AB51 */
  0x0000,   /* U+AB52 */
  0x0000,   /* U+AB53 */
  0x0000,   /* U+AB54 */
  0x0000,   /* U+AB55 */
  0x0000,   /* U+AB56 */
  0x0000,   /* U+AB57 */
  0x0000,   /* U+AB58 */
  0x0000,   /* U+AB59 */
  0x0000,   /* U+AB5A */
  0x0000,   /* U+AB5B */
  0x0000,   /* U+AB5C */
  0x0000,   /* U+AB5D */
  0x0000,   /* U+AB5E */
  0x0000,   /* U+AB5F */
  0x0000,   /* U+AB60 */
  0x0000,   /* U+AB61 */
  0x0000,   /* U+AB62 */
  0x0000,   /* U+AB63 */
  0x0000,   /* U+AB64 */
  0x0000,   /* U+AB65 */
  0x0000,   /* U+AB66 */
  0x0000,   /* U+AB67 */
  0x0000,   /* U+AB68 */
  0x0000,   /* U+AB69 */
  0x0000,   /* U+AB6A */
  0x0000,   /* U+AB6B */
  0x0000,   /* U+AB6C */
  0x0000,   /* U+AB6D */
  0x0000,   /* U+AB6E */
  0x0000,   /* U+AB6F */
  0x0000,   /* U+AB70 */
  0x0000,   /* U+AB71 */
  0x0000,   /* U+AB72 */
  0x0000,   /* U+AB73 */
  0x0000,   /* U+AB74 */
  0x0000,   /* U+AB75 */
  0x0000,   /* U+AB76 */
  0x0000,   /* U+AB77 */
  0x0000,   /* U+AB78 */
  0x0000,   /* U+AB79 */
  0x0000,   /* U+AB7A */
  0x0000,   /* U+AB7B */
  0x0000,   /* U+AB7C */
  0x0000,   /* U+AB7D */
  0x0000,   /* U+AB7E */
  0x0000,   /* U+AB7F */
  0x0000,   /* U+AB80 */
  0x0000,   /* U+AB81 */
  0x0000,   /* U+AB82 */
  0x0000,   /* U+AB83 */
  0x0000,   /* U+AB84 */
  0x0000,   /* U+AB85 */
  0x0000,   /* U+AB86 */
  0x0000,   /* U+AB87 */
  0x0000,   /* U+AB88 */
  0x0000,   /* U+AB89 */
  0x0000,   /* U+AB8A */
  0x0000,   /* U+AB8B */
  0x0000,   /* U+AB8C */
  0x0000,   /* U+AB8D */
  0x0000,   /* U+AB8E */
  0x0000,   /* U+AB8F */
  0x0000,   /* U+AB90 */
  0x0000,   /* U+AB91 */
  0x0000,   /* U+AB92 */
  0x0000,   /* U+AB93 */
  0x0000,   /* U+AB94 */
  0x0000,   /* U+AB95 */
  0x0000,   /* U+AB96 */
  0x0000,   /* U+AB97 */
  0x0000,   /* U+AB98 */
  0x0000,   /* U+AB99 */
  0x0000,   /* U+AB9A */
  0x0000,   /* U+AB9B */
  0x0000,   /* U+AB9C */
  0x0000,   /* U+AB9D */
  0x0000,   /* U+AB9E */
  0x0000,   /* U+AB9F */
  0x0000,   /* U+ABA0 */
  0x0000,   /* U+ABA1 */
  0x0000,   /* U+ABA2 */
  0x0000,   /* U+ABA3 */
  0x0000,   /* U+ABA4 */
  0x0000,   /* U+ABA5 */
  0x0000,   /* U+ABA6 */
  0x0000,   /* U+ABA7 */
  0x0000,   /* U+ABA8 */
  0x0000,   /* U+ABA9 */
  0x0000,   /* U+ABAA */
  0x0000,   /* U+ABAB */
  0x0000,   /* U+ABAC */
  0x0000,   /* U+ABAD */
  0x0000,   /* U+ABAE */
  0x0000,   /* U+ABAF */
  0x0000,   /* U+ABB0 */
  0x0000,   /* U+ABB1 */
  0x0000,   /* U+ABB2 */
  0x0000,   /* U+ABB3 */
  0x0000,   /* U+ABB4 */
  0x0000,   /* U+ABB5 */
  0x0000,   /* U+ABB6 */
  0x0000,   /* U+ABB7 */
  0x0000,   /* U+ABB8 */
  0x0000,   /* U+ABB9 */
  0x0000,   /* U+ABBA */
  0x0000,   /* U+ABBB */
  0x0000,   /* U+ABBC */
  0x0000,   /* U+ABBD */
  0x0000,   /* U+ABBE */
  0x0000,   /* U+ABBF */
  0x0000,   /* U+ABC0 */
  0x0000,   /* U+ABC1 */
  0x0000,   /* U+ABC2 */
  0x0000,   /* U+ABC3 */
  0x0000,   /* U+ABC4 */
  0x0000,   /* U+ABC5 */
  0x0000,   /* U+ABC6 */
  0x0000,   /* U+ABC7 */
  0x0000,   /* U+ABC8 */
  0x0000,   /* U+ABC9 */
  0x0000,   /* U+ABCA */
  0x0000,   /* U+ABCB */
  0x0000,   /* U+ABCC */
  0x0000,   /* U+ABCD */
  0x0000,   /* U+ABCE */
  0x0000,   /* U+ABCF */
  0x0000,   /* U+ABD0 */
  0x0000,   /* U+ABD1 */
  0x0000,   /* U+ABD2 */
  0x0000,   /* U+ABD3 */
  0x0000,   /* U+ABD4 */
  0x0000,   /* U+ABD5 */
  0x0000,   /* U+ABD6 */
  0x0000,   /* U+ABD7 */
  0x0000,   /* U+ABD8 */
  0x0000,   /* U+ABD9 */
  0x0000,   /* U+ABDA */
  0x0000,   /* U+ABDB */
  0x0000,   /* U+ABDC */
  0x0000,   /* U+ABDD */
  0x0000,   /* U+ABDE */
  0x0000,   /* U+ABDF */
  0x0000,   /* U+ABE0 */
  0x0000,   /* U+ABE1 */
  0x0000,   /* U+ABE2 */
  0x0000,   /* U+ABE3 */
  0x0000,   /* U+ABE4 */
  0x0000,   /* U+ABE5 */
  0x0000,   /* U+ABE6 */
  0x0000,   /* U+ABE7 */
  0x0000,   /* U+ABE8 */
  0x0000,   /* U+ABE9 */
  0x0000,   /* U+ABEA */
  0x0000,   /* U+ABEB */
  0x0000,   /* U+ABEC */
  0x0000,   /* U+ABED */
  0x0000,   /* U+ABEE */
  0x0000,   /* U+ABEF */
  0x0000,   /* U+ABF0 */
  0x0000,   /* U+ABF1 */
  0x0000,   /* U+ABF2 */
  0x0000,   /* U+ABF3 */
  0x0000,   /* U+ABF4 */
  0x0000,   /* U+ABF5 */
  0x0000,   /* U+ABF6 */
  0x0000,   /* U+ABF7 */
  0x0000,   /* U+ABF8 */
  0x0000,   /* U+ABF9 */
  0x0000,   /* U+ABFA */
  0x0000,   /* U+ABFB */
  0x0000,   /* U+ABFC */
  0x0000,   /* U+ABFD */
  0x0000,   /* U+ABFE */
  0x0000,   /* U+ABFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+AB00 */
  0x0000,   /* U+AB01 */
  0x0000,   /* U+AB02 */
  0x0000,   /* U+AB03 */
  0x0000,   /* U+AB04 */
  0x0000,   /* U+AB05 */
  0x0000,   /* U+AB06 */
  0x0000,   /* U+AB07 */
  0x0000,   /* U+AB08 */
  0x0000,   /* U+AB09 */
  0x0000,   /* U+AB0A */
  0x0000,   /* U+AB0B */
  0x0000,   /* U+AB0C */
  0x0000,   /* U+AB0D */
  0x0000,   /* U+AB0E */
  0x0000,   /* U+AB0F */
  0x0000,   /* U+AB10 */
  0x0000,   /* U+AB11 */
  0x0000,   /* U+AB12 */
  0x0000,   /* U+AB13 */
  0x0000,   /* U+AB14 */
  0x0000,   /* U+AB15 */
  0x0000,   /* U+AB16 */
  0x0000,   /* U+AB17 */
  0x0000,   /* U+AB18 */
  0x0000,   /* U+AB19 */
  0x0000,   /* U+AB1A */
  0x0000,   /* U+AB1B */
  0x0000,   /* U+AB1C */
  0x0000,   /* U+AB1D */
  0x0000,   /* U+AB1E */
  0x0000,   /* U+AB1F */
  0x0000,   /* U+AB20 */
  0x0000,   /* U+AB21 */
  0x0000,   /* U+AB22 */
  0x0000,   /* U+AB23 */
  0x0000,   /* U+AB24 */
  0x0000,   /* U+AB25 */
  0x0000,   /* U+AB26 */
  0x0000,   /* U+AB27 */
  0x0000,   /* U+AB28 */
  0x0000,   /* U+AB29 */
  0x0000,   /* U+AB2A */
  0x0000,   /* U+AB2B */
  0x0000,   /* U+AB2C */
  0x0000,   /* U+AB2D */
  0x0000,   /* U+AB2E */
  0x0000,   /* U+AB2F */
  0x0000,   /* U+AB30 */
  0x0000,   /* U+AB31 */
  0x0000,   /* U+AB32 */
  0x0000,   /* U+AB33 */
  0x0000,   /* U+AB34 */
  0x0000,   /* U+AB35 */
  0x0000,   /* U+AB36 */
  0x0000,   /* U+AB37 */
  0x0000,   /* U+AB38 */
  0x0000,   /* U+AB39 */
  0x0000,   /* U+AB3A */
  0x0000,   /* U+AB3B */
  0x0000,   /* U+AB3C */
  0x0000,   /* U+AB3D */
  0x0000,   /* U+AB3E */
  0x0000,   /* U+AB3F */
  0x0000,   /* U+AB40 */
  0x0000,   /* U+AB41 */
  0x0000,   /* U+AB42 */
  0x0000,   /* U+AB43 */
  0x0000,   /* U+AB44 */
  0x0000,   /* U+AB45 */
  0x0000,   /* U+AB46 */
  0x0000,   /* U+AB47 */
  0x0000,   /* U+AB48 */
  0x0000,   /* U+AB49 */
  0x0000,   /* U+AB4A */
  0x0000,   /* U+AB4B */
  0x0000,   /* U+AB4C */
  0x0000,   /* U+AB4D */
  0x0000,   /* U+AB4E */
  0x0000,   /* U+AB4F */
  0x0000,   /* U+AB50 */
  0x0000,   /* U+AB51 */
  0x0000,   /* U+AB52 */
  0x0000,   /* U+AB53 */
  0x0000,   /* U+AB54 */
  0x0000,   /* U+AB55 */
  0x0000,   /* U+AB56 */
  0x0000,   /* U+AB57 */
  0x0000,   /* U+AB58 */
  0x0000,   /* U+AB59 */
  0x0000,   /* U+AB5A */
  0x0000,   /* U+AB5B */
  0x0000,   /* U+AB5C */
  0x0000,   /* U+AB5D */
  0x0000,   /* U+AB5E */
  0x0000,   /* U+AB5F */
  0x0000,   /* U+AB60 */
  0x0000,   /* U+AB61 */
  0x0000,   /* U+AB62 */
  0x0000,   /* U+AB63 */
  0x0000,   /* U+AB64 */
  0x0000,   /* U+AB65 */
  0x0000,   /* U+AB66 */
  0x0000,   /* U+AB67 */
  0x0000,   /* U+AB68 */
  0x0000,   /* U+AB69 */
  0x0000,   /* U+AB6A */
  0x0000,   /* U+AB6B */
  0x0000,   /* U+AB6C */
  0x0000,   /* U+AB6D */
  0x0000,   /* U+AB6E */
  0x0000,   /* U+AB6F */
  0x0000,   /* U+AB70 */
  0x0000,   /* U+AB71 */
  0x0000,   /* U+AB72 */
  0x0000,   /* U+AB73 */
  0x0000,   /* U+AB74 */
  0x0000,   /* U+AB75 */
  0x0000,   /* U+AB76 */
  0x0000,   /* U+AB77 */
  0x0000,   /* U+AB78 */
  0x0000,   /* U+AB79 */
  0x0000,   /* U+AB7A */
  0x0000,   /* U+AB7B */
  0x0000,   /* U+AB7C */
  0x0000,   /* U+AB7D */
  0x0000,   /* U+AB7E */
  0x0000,   /* U+AB7F */
  0x0000,   /* U+AB80 */
  0x0000,   /* U+AB81 */
  0x0000,   /* U+AB82 */
  0x0000,   /* U+AB83 */
  0x0000,   /* U+AB84 */
  0x0000,   /* U+AB85 */
  0x0000,   /* U+AB86 */
  0x0000,   /* U+AB87 */
  0x0000,   /* U+AB88 */
  0x0000,   /* U+AB89 */
  0x0000,   /* U+AB8A */
  0x0000,   /* U+AB8B */
  0x0000,   /* U+AB8C */
  0x0000,   /* U+AB8D */
  0x0000,   /* U+AB8E */
  0x0000,   /* U+AB8F */
  0x0000,   /* U+AB90 */
  0x0000,   /* U+AB91 */
  0x0000,   /* U+AB92 */
  0x0000,   /* U+AB93 */
  0x0000,   /* U+AB94 */
  0x0000,   /* U+AB95 */
  0x0000,   /* U+AB96 */
  0x0000,   /* U+AB97 */
  0x0000,   /* U+AB98 */
  0x0000,   /* U+AB99 */
  0x0000,   /* U+AB9A */
  0x0000,   /* U+AB9B */
  0x0000,   /* U+AB9C */
  0x0000,   /* U+AB9D */
  0x0000,   /* U+AB9E */
  0x0000,   /* U+AB9F */
  0x0000,   /* U+ABA0 */
  0x0000,   /* U+ABA1 */
  0x0000,   /* U+ABA2 */
  0x0000,   /* U+ABA3 */
  0x0000,   /* U+ABA4 */
  0x0000,   /* U+ABA5 */
  0x0000,   /* U+ABA6 */
  0x0000,   /* U+ABA7 */
  0x0000,   /* U+ABA8 */
  0x0000,   /* U+ABA9 */
  0x0000,   /* U+ABAA */
  0x0000,   /* U+ABAB */
  0x0000,   /* U+ABAC */
  0x0000,   /* U+ABAD */
  0x0000,   /* U+ABAE */
  0x0000,   /* U+ABAF */
  0x0000,   /* U+ABB0 */
  0x0000,   /* U+ABB1 */
  0x0000,   /* U+ABB2 */
  0x0000,   /* U+ABB3 */
  0x0000,   /* U+ABB4 */
  0x0000,   /* U+ABB5 */
  0x0000,   /* U+ABB6 */
  0x0000,   /* U+ABB7 */
  0x0000,   /* U+ABB8 */
  0x0000,   /* U+ABB9 */
  0x0000,   /* U+ABBA */
  0x0000,   /* U+ABBB */
  0x0000,   /* U+ABBC */
  0x0000,   /* U+ABBD */
  0x0000,   /* U+ABBE */
  0x0000,   /* U+ABBF */
  0x0000,   /* U+ABC0 */
  0x0000,   /* U+ABC1 */
  0x0000,   /* U+ABC2 */
  0x0000,   /* U+ABC3 */
  0x0000,   /* U+ABC4 */
  0x0000,   /* U+ABC5 */
  0x0000,   /* U+ABC6 */
  0x0000,   /* U+ABC7 */
  0x0000,   /* U+ABC8 */
  0x0000,   /* U+ABC9 */
  0x0000,   /* U+ABCA */
  0x0000,   /* U+ABCB */
  0x0000,   /* U+ABCC */
  0x0000,   /* U+ABCD */
  0x0000,   /* U+ABCE */
  0x0000,   /* U+ABCF */
  0x0000,   /* U+ABD0 */
  0x0000,   /* U+ABD1 */
  0x0000,   /* U+ABD2 */
  0x0000,   /* U+ABD3 */
  0x0000,   /* U+ABD4 */
  0x0000,   /* U+ABD5 */
  0x0000,   /* U+ABD6 */
  0x0000,   /* U+ABD7 */
  0x0000,   /* U+ABD8 */
  0x0000,   /* U+ABD9 */
  0x0000,   /* U+ABDA */
  0x0000,   /* U+ABDB */
  0x0000,   /* U+ABDC */
  0x0000,   /* U+ABDD */
  0x0000,   /* U+ABDE */
  0x0000,   /* U+ABDF */
  0x0000,   /* U+ABE0 */
  0x0000,   /* U+ABE1 */
  0x0000,   /* U+ABE2 */
  0x0000,   /* U+ABE3 */
  0x0000,   /* U+ABE4 */
  0x0000,   /* U+ABE5 */
  0x0000,   /* U+ABE6 */
  0x0000,   /* U+ABE7 */
  0x0000,   /* U+ABE8 */
  0x0000,   /* U+ABE9 */
  0x0000,   /* U+ABEA */
  0x0000,   /* U+ABEB */
  0x0000,   /* U+ABEC */
  0x0000,   /* U+ABED */
  0x0000,   /* U+ABEE */
  0x0000,   /* U+ABEF */
  0x0000,   /* U+ABF0 */
  0x0000,   /* U+ABF1 */
  0x0000,   /* U+ABF2 */
  0x0000,   /* U+ABF3 */
  0x0000,   /* U+ABF4 */
  0x0000,   /* U+ABF5 */
  0x0000,   /* U+ABF6 */
  0x0000,   /* U+ABF7 */
  0x0000,   /* U+ABF8 */
  0x0000,   /* U+ABF9 */
  0x0000,   /* U+ABFA */
  0x0000,   /* U+ABFB */
  0x0000,   /* U+ABFC */
  0x0000,   /* U+ABFD */
  0x0000,   /* U+ABFE */
  0x0000,   /* U+ABFF */
};

uint16 uca900_p0D7[]= {
  /* Number of CEs for each character. */
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 2, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 2, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3C07,   /* U+D700 */
  0x3C07,   /* U+D701 */
  0x3C07,   /* U+D702 */
  0x3C07,   /* U+D703 */
  0x3C07,   /* U+D704 */
  0x3C07,   /* U+D705 */
  0x3C07,   /* U+D706 */
  0x3C07,   /* U+D707 */
  0x3C07,   /* U+D708 */
  0x3C07,   /* U+D709 */
  0x3C07,   /* U+D70A */
  0x3C07,   /* U+D70B */
  0x3C07,   /* U+D70C */
  0x3C07,   /* U+D70D */
  0x3C07,   /* U+D70E */
  0x3C07,   /* U+D70F */
  0x3C07,   /* U+D710 */
  0x3C07,   /* U+D711 */
  0x3C07,   /* U+D712 */
  0x3C07,   /* U+D713 */
  0x3C07,   /* U+D714 */
  0x3C07,   /* U+D715 */
  0x3C07,   /* U+D716 */
  0x3C07,   /* U+D717 */
  0x3C07,   /* U+D718 */
  0x3C07,   /* U+D719 */
  0x3C07,   /* U+D71A */
  0x3C07,   /* U+D71B */
  0x3C07,   /* U+D71C */
  0x3C07,   /* U+D71D */
  0x3C07,   /* U+D71E */
  0x3C07,   /* U+D71F */
  0x3C07,   /* U+D720 */
  0x3C07,   /* U+D721 */
  0x3C07,   /* U+D722 */
  0x3C07,   /* U+D723 */
  0x3C07,   /* U+D724 */
  0x3C07,   /* U+D725 */
  0x3C07,   /* U+D726 */
  0x3C07,   /* U+D727 */
  0x3C07,   /* U+D728 */
  0x3C07,   /* U+D729 */
  0x3C07,   /* U+D72A */
  0x3C07,   /* U+D72B */
  0x3C07,   /* U+D72C */
  0x3C07,   /* U+D72D */
  0x3C07,   /* U+D72E */
  0x3C07,   /* U+D72F */
  0x3C07,   /* U+D730 */
  0x3C07,   /* U+D731 */
  0x3C07,   /* U+D732 */
  0x3C07,   /* U+D733 */
  0x3C07,   /* U+D734 */
  0x3C07,   /* U+D735 */
  0x3C07,   /* U+D736 */
  0x3C07,   /* U+D737 */
  0x3C07,   /* U+D738 */
  0x3C07,   /* U+D739 */
  0x3C07,   /* U+D73A */
  0x3C07,   /* U+D73B */
  0x3C07,   /* U+D73C */
  0x3C07,   /* U+D73D */
  0x3C07,   /* U+D73E */
  0x3C07,   /* U+D73F */
  0x3C07,   /* U+D740 */
  0x3C07,   /* U+D741 */
  0x3C07,   /* U+D742 */
  0x3C07,   /* U+D743 */
  0x3C07,   /* U+D744 */
  0x3C07,   /* U+D745 */
  0x3C07,   /* U+D746 */
  0x3C07,   /* U+D747 */
  0x3C07,   /* U+D748 */
  0x3C07,   /* U+D749 */
  0x3C07,   /* U+D74A */
  0x3C07,   /* U+D74B */
  0x3C07,   /* U+D74C */
  0x3C07,   /* U+D74D */
  0x3C07,   /* U+D74E */
  0x3C07,   /* U+D74F */
  0x3C07,   /* U+D750 */
  0x3C07,   /* U+D751 */
  0x3C07,   /* U+D752 */
  0x3C07,   /* U+D753 */
  0x3C07,   /* U+D754 */
  0x3C07,   /* U+D755 */
  0x3C07,   /* U+D756 */
  0x3C07,   /* U+D757 */
  0x3C07,   /* U+D758 */
  0x3C07,   /* U+D759 */
  0x3C07,   /* U+D75A */
  0x3C07,   /* U+D75B */
  0x3C07,   /* U+D75C */
  0x3C07,   /* U+D75D */
  0x3C07,   /* U+D75E */
  0x3C07,   /* U+D75F */
  0x3C07,   /* U+D760 */
  0x3C07,   /* U+D761 */
  0x3C07,   /* U+D762 */
  0x3C07,   /* U+D763 */
  0x3C07,   /* U+D764 */
  0x3C07,   /* U+D765 */
  0x3C07,   /* U+D766 */
  0x3C07,   /* U+D767 */
  0x3C07,   /* U+D768 */
  0x3C07,   /* U+D769 */
  0x3C07,   /* U+D76A */
  0x3C07,   /* U+D76B */
  0x3C07,   /* U+D76C */
  0x3C07,   /* U+D76D */
  0x3C07,   /* U+D76E */
  0x3C07,   /* U+D76F */
  0x3C07,   /* U+D770 */
  0x3C07,   /* U+D771 */
  0x3C07,   /* U+D772 */
  0x3C07,   /* U+D773 */
  0x3C07,   /* U+D774 */
  0x3C07,   /* U+D775 */
  0x3C07,   /* U+D776 */
  0x3C07,   /* U+D777 */
  0x3C07,   /* U+D778 */
  0x3C07,   /* U+D779 */
  0x3C07,   /* U+D77A */
  0x3C07,   /* U+D77B */
  0x3C07,   /* U+D77C */
  0x3C07,   /* U+D77D */
  0x3C07,   /* U+D77E */
  0x3C07,   /* U+D77F */
  0x3C07,   /* U+D780 */
  0x3C07,   /* U+D781 */
  0x3C07,   /* U+D782 */
  0x3C07,   /* U+D783 */
  0x3C07,   /* U+D784 */
  0x3C07,   /* U+D785 */
  0x3C07,   /* U+D786 */
  0x3C07,   /* U+D787 */
  0x3C07,   /* U+D788 */
  0x3C07,   /* U+D789 */
  0x3C07,   /* U+D78A */
  0x3C07,   /* U+D78B */
  0x3C07,   /* U+D78C */
  0x3C07,   /* U+D78D */
  0x3C07,   /* U+D78E */
  0x3C07,   /* U+D78F */
  0x3C07,   /* U+D790 */
  0x3C07,   /* U+D791 */
  0x3C07,   /* U+D792 */
  0x3C07,   /* U+D793 */
  0x3C07,   /* U+D794 */
  0x3C07,   /* U+D795 */
  0x3C07,   /* U+D796 */
  0x3C07,   /* U+D797 */
  0x3C07,   /* U+D798 */
  0x3C07,   /* U+D799 */
  0x3C07,   /* U+D79A */
  0x3C07,   /* U+D79B */
  0x3C07,   /* U+D79C */
  0x3C07,   /* U+D79D */
  0x3C07,   /* U+D79E */
  0x3C07,   /* U+D79F */
  0x3C07,   /* U+D7A0 */
  0x3C07,   /* U+D7A1 */
  0x3C07,   /* U+D7A2 */
  0x3C07,   /* U+D7A3 */
  0x3C08,   /* U+D7A4 */
  0x3C08,   /* U+D7A5 */
  0x3C08,   /* U+D7A6 */
  0x3C08,   /* U+D7A7 */
  0x3C08,   /* U+D7A8 */
  0x3C08,   /* U+D7A9 */
  0x3C08,   /* U+D7AA */
  0x3C08,   /* U+D7AB */
  0x3C08,   /* U+D7AC */
  0x3C08,   /* U+D7AD */
  0x3C08,   /* U+D7AE */
  0x3C08,   /* U+D7AF */
  0x3CBA,   /* U+D7B0 */
  0x3CBB,   /* U+D7B1 */
  0x3CBC,   /* U+D7B2 */
  0x3CBD,   /* U+D7B3 */
  0x3CBE,   /* U+D7B4 */
  0x3CBF,   /* U+D7B5 */
  0x3CC0,   /* U+D7B6 */
  0x3CC1,   /* U+D7B7 */
  0x3CC2,   /* U+D7B8 */
  0x3CC3,   /* U+D7B9 */
  0x3CC4,   /* U+D7BA */
  0x3CC5,   /* U+D7BB */
  0x3CC6,   /* U+D7BC */
  0x3CC7,   /* U+D7BD */
  0x3CC8,   /* U+D7BE */
  0x3CC9,   /* U+D7BF */
  0x3CCA,   /* U+D7C0 */
  0x3CCB,   /* U+D7C1 */
  0x3CCC,   /* U+D7C2 */
  0x3CCD,   /* U+D7C3 */
  0x3CCE,   /* U+D7C4 */
  0x3CCF,   /* U+D7C5 */
  0x3CD0,   /* U+D7C6 */
  0xFBC1,   /* U+D7C7 */
  0xFBC1,   /* U+D7C8 */
  0xFBC1,   /* U+D7C9 */
  0xFBC1,   /* U+D7CA */
  0x3D29,   /* U+D7CB */
  0x3D2A,   /* U+D7CC */
  0x3D2B,   /* U+D7CD */
  0x3D2C,   /* U+D7CE */
  0x3D2D,   /* U+D7CF */
  0x3D2E,   /* U+D7D0 */
  0x3D2F,   /* U+D7D1 */
  0x3D30,   /* U+D7D2 */
  0x3D31,   /* U+D7D3 */
  0x3D32,   /* U+D7D4 */
  0x3D33,   /* U+D7D5 */
  0x3D34,   /* U+D7D6 */
  0x3D35,   /* U+D7D7 */
  0x3D36,   /* U+D7D8 */
  0x3D37,   /* U+D7D9 */
  0x3D38,   /* U+D7DA */
  0x3D39,   /* U+D7DB */
  0x3D3A,   /* U+D7DC */
  0x3D3B,   /* U+D7DD */
  0x3D3C,   /* U+D7DE */
  0x3D3D,   /* U+D7DF */
  0x3D3E,   /* U+D7E0 */
  0x3D3F,   /* U+D7E1 */
  0x3D40,   /* U+D7E2 */
  0x3D41,   /* U+D7E3 */
  0x3D42,   /* U+D7E4 */
  0x3D43,   /* U+D7E5 */
  0x3D44,   /* U+D7E6 */
  0x3D45,   /* U+D7E7 */
  0x3D46,   /* U+D7E8 */
  0x3D47,   /* U+D7E9 */
  0x3D48,   /* U+D7EA */
  0x3D49,   /* U+D7EB */
  0x3D4A,   /* U+D7EC */
  0x3D4B,   /* U+D7ED */
  0x3D4C,   /* U+D7EE */
  0x3D4D,   /* U+D7EF */
  0x3D4E,   /* U+D7F0 */
  0x3D4F,   /* U+D7F1 */
  0x3D50,   /* U+D7F2 */
  0x3D51,   /* U+D7F3 */
  0x3D52,   /* U+D7F4 */
  0x3D53,   /* U+D7F5 */
  0x3D54,   /* U+D7F6 */
  0x3D55,   /* U+D7F7 */
  0x3D56,   /* U+D7F8 */
  0x3D57,   /* U+D7F9 */
  0x3D58,   /* U+D7FA */
  0x3D59,   /* U+D7FB */
  0xFBC1,   /* U+D7FC */
  0xFBC1,   /* U+D7FD */
  0xFBC1,   /* U+D7FE */
  0xFBC1,   /* U+D7FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+D700 */
  0x0020,   /* U+D701 */
  0x0020,   /* U+D702 */
  0x0020,   /* U+D703 */
  0x0020,   /* U+D704 */
  0x0020,   /* U+D705 */
  0x0020,   /* U+D706 */
  0x0020,   /* U+D707 */
  0x0020,   /* U+D708 */
  0x0020,   /* U+D709 */
  0x0020,   /* U+D70A */
  0x0020,   /* U+D70B */
  0x0020,   /* U+D70C */
  0x0020,   /* U+D70D */
  0x0020,   /* U+D70E */
  0x0020,   /* U+D70F */
  0x0020,   /* U+D710 */
  0x0020,   /* U+D711 */
  0x0020,   /* U+D712 */
  0x0020,   /* U+D713 */
  0x0020,   /* U+D714 */
  0x0020,   /* U+D715 */
  0x0020,   /* U+D716 */
  0x0020,   /* U+D717 */
  0x0020,   /* U+D718 */
  0x0020,   /* U+D719 */
  0x0020,   /* U+D71A */
  0x0020,   /* U+D71B */
  0x0020,   /* U+D71C */
  0x0020,   /* U+D71D */
  0x0020,   /* U+D71E */
  0x0020,   /* U+D71F */
  0x0020,   /* U+D720 */
  0x0020,   /* U+D721 */
  0x0020,   /* U+D722 */
  0x0020,   /* U+D723 */
  0x0020,   /* U+D724 */
  0x0020,   /* U+D725 */
  0x0020,   /* U+D726 */
  0x0020,   /* U+D727 */
  0x0020,   /* U+D728 */
  0x0020,   /* U+D729 */
  0x0020,   /* U+D72A */
  0x0020,   /* U+D72B */
  0x0020,   /* U+D72C */
  0x0020,   /* U+D72D */
  0x0020,   /* U+D72E */
  0x0020,   /* U+D72F */
  0x0020,   /* U+D730 */
  0x0020,   /* U+D731 */
  0x0020,   /* U+D732 */
  0x0020,   /* U+D733 */
  0x0020,   /* U+D734 */
  0x0020,   /* U+D735 */
  0x0020,   /* U+D736 */
  0x0020,   /* U+D737 */
  0x0020,   /* U+D738 */
  0x0020,   /* U+D739 */
  0x0020,   /* U+D73A */
  0x0020,   /* U+D73B */
  0x0020,   /* U+D73C */
  0x0020,   /* U+D73D */
  0x0020,   /* U+D73E */
  0x0020,   /* U+D73F */
  0x0020,   /* U+D740 */
  0x0020,   /* U+D741 */
  0x0020,   /* U+D742 */
  0x0020,   /* U+D743 */
  0x0020,   /* U+D744 */
  0x0020,   /* U+D745 */
  0x0020,   /* U+D746 */
  0x0020,   /* U+D747 */
  0x0020,   /* U+D748 */
  0x0020,   /* U+D749 */
  0x0020,   /* U+D74A */
  0x0020,   /* U+D74B */
  0x0020,   /* U+D74C */
  0x0020,   /* U+D74D */
  0x0020,   /* U+D74E */
  0x0020,   /* U+D74F */
  0x0020,   /* U+D750 */
  0x0020,   /* U+D751 */
  0x0020,   /* U+D752 */
  0x0020,   /* U+D753 */
  0x0020,   /* U+D754 */
  0x0020,   /* U+D755 */
  0x0020,   /* U+D756 */
  0x0020,   /* U+D757 */
  0x0020,   /* U+D758 */
  0x0020,   /* U+D759 */
  0x0020,   /* U+D75A */
  0x0020,   /* U+D75B */
  0x0020,   /* U+D75C */
  0x0020,   /* U+D75D */
  0x0020,   /* U+D75E */
  0x0020,   /* U+D75F */
  0x0020,   /* U+D760 */
  0x0020,   /* U+D761 */
  0x0020,   /* U+D762 */
  0x0020,   /* U+D763 */
  0x0020,   /* U+D764 */
  0x0020,   /* U+D765 */
  0x0020,   /* U+D766 */
  0x0020,   /* U+D767 */
  0x0020,   /* U+D768 */
  0x0020,   /* U+D769 */
  0x0020,   /* U+D76A */
  0x0020,   /* U+D76B */
  0x0020,   /* U+D76C */
  0x0020,   /* U+D76D */
  0x0020,   /* U+D76E */
  0x0020,   /* U+D76F */
  0x0020,   /* U+D770 */
  0x0020,   /* U+D771 */
  0x0020,   /* U+D772 */
  0x0020,   /* U+D773 */
  0x0020,   /* U+D774 */
  0x0020,   /* U+D775 */
  0x0020,   /* U+D776 */
  0x0020,   /* U+D777 */
  0x0020,   /* U+D778 */
  0x0020,   /* U+D779 */
  0x0020,   /* U+D77A */
  0x0020,   /* U+D77B */
  0x0020,   /* U+D77C */
  0x0020,   /* U+D77D */
  0x0020,   /* U+D77E */
  0x0020,   /* U+D77F */
  0x0020,   /* U+D780 */
  0x0020,   /* U+D781 */
  0x0020,   /* U+D782 */
  0x0020,   /* U+D783 */
  0x0020,   /* U+D784 */
  0x0020,   /* U+D785 */
  0x0020,   /* U+D786 */
  0x0020,   /* U+D787 */
  0x0020,   /* U+D788 */
  0x0020,   /* U+D789 */
  0x0020,   /* U+D78A */
  0x0020,   /* U+D78B */
  0x0020,   /* U+D78C */
  0x0020,   /* U+D78D */
  0x0020,   /* U+D78E */
  0x0020,   /* U+D78F */
  0x0020,   /* U+D790 */
  0x0020,   /* U+D791 */
  0x0020,   /* U+D792 */
  0x0020,   /* U+D793 */
  0x0020,   /* U+D794 */
  0x0020,   /* U+D795 */
  0x0020,   /* U+D796 */
  0x0020,   /* U+D797 */
  0x0020,   /* U+D798 */
  0x0020,   /* U+D799 */
  0x0020,   /* U+D79A */
  0x0020,   /* U+D79B */
  0x0020,   /* U+D79C */
  0x0020,   /* U+D79D */
  0x0020,   /* U+D79E */
  0x0020,   /* U+D79F */
  0x0020,   /* U+D7A0 */
  0x0020,   /* U+D7A1 */
  0x0020,   /* U+D7A2 */
  0x0020,   /* U+D7A3 */
  0x0020,   /* U+D7A4 */
  0x0020,   /* U+D7A5 */
  0x0020,   /* U+D7A6 */
  0x0020,   /* U+D7A7 */
  0x0020,   /* U+D7A8 */
  0x0020,   /* U+D7A9 */
  0x0020,   /* U+D7AA */
  0x0020,   /* U+D7AB */
  0x0020,   /* U+D7AC */
  0x0020,   /* U+D7AD */
  0x0020,   /* U+D7AE */
  0x0020,   /* U+D7AF */
  0x0020,   /* U+D7B0 */
  0x0020,   /* U+D7B1 */
  0x0020,   /* U+D7B2 */
  0x0020,   /* U+D7B3 */
  0x0020,   /* U+D7B4 */
  0x0020,   /* U+D7B5 */
  0x0020,   /* U+D7B6 */
  0x0020,   /* U+D7B7 */
  0x0020,   /* U+D7B8 */
  0x0020,   /* U+D7B9 */
  0x0020,   /* U+D7BA */
  0x0020,   /* U+D7BB */
  0x0020,   /* U+D7BC */
  0x0020,   /* U+D7BD */
  0x0020,   /* U+D7BE */
  0x0020,   /* U+D7BF */
  0x0020,   /* U+D7C0 */
  0x0020,   /* U+D7C1 */
  0x0020,   /* U+D7C2 */
  0x0020,   /* U+D7C3 */
  0x0020,   /* U+D7C4 */
  0x0020,   /* U+D7C5 */
  0x0020,   /* U+D7C6 */
  0x0020,   /* U+D7C7 */
  0x0020,   /* U+D7C8 */
  0x0020,   /* U+D7C9 */
  0x0020,   /* U+D7CA */
  0x0020,   /* U+D7CB */
  0x0020,   /* U+D7CC */
  0x0020,   /* U+D7CD */
  0x0020,   /* U+D7CE */
  0x0020,   /* U+D7CF */
  0x0020,   /* U+D7D0 */
  0x0020,   /* U+D7D1 */
  0x0020,   /* U+D7D2 */
  0x0020,   /* U+D7D3 */
  0x0020,   /* U+D7D4 */
  0x0020,   /* U+D7D5 */
  0x0020,   /* U+D7D6 */
  0x0020,   /* U+D7D7 */
  0x0020,   /* U+D7D8 */
  0x0020,   /* U+D7D9 */
  0x0020,   /* U+D7DA */
  0x0020,   /* U+D7DB */
  0x0020,   /* U+D7DC */
  0x0020,   /* U+D7DD */
  0x0020,   /* U+D7DE */
  0x0020,   /* U+D7DF */
  0x0020,   /* U+D7E0 */
  0x0020,   /* U+D7E1 */
  0x0020,   /* U+D7E2 */
  0x0020,   /* U+D7E3 */
  0x0020,   /* U+D7E4 */
  0x0020,   /* U+D7E5 */
  0x0020,   /* U+D7E6 */
  0x0020,   /* U+D7E7 */
  0x0020,   /* U+D7E8 */
  0x0020,   /* U+D7E9 */
  0x0020,   /* U+D7EA */
  0x0020,   /* U+D7EB */
  0x0020,   /* U+D7EC */
  0x0020,   /* U+D7ED */
  0x0020,   /* U+D7EE */
  0x0020,   /* U+D7EF */
  0x0020,   /* U+D7F0 */
  0x0020,   /* U+D7F1 */
  0x0020,   /* U+D7F2 */
  0x0020,   /* U+D7F3 */
  0x0020,   /* U+D7F4 */
  0x0020,   /* U+D7F5 */
  0x0020,   /* U+D7F6 */
  0x0020,   /* U+D7F7 */
  0x0020,   /* U+D7F8 */
  0x0020,   /* U+D7F9 */
  0x0020,   /* U+D7FA */
  0x0020,   /* U+D7FB */
  0x0020,   /* U+D7FC */
  0x0020,   /* U+D7FD */
  0x0020,   /* U+D7FE */
  0x0020,   /* U+D7FF */

  /* Tertiary weight 1 for each character. */
  0x0003,   /* U+D700 */
  0x0003,   /* U+D701 */
  0x0003,   /* U+D702 */
  0x0003,   /* U+D703 */
  0x0003,   /* U+D704 */
  0x0003,   /* U+D705 */
  0x0003,   /* U+D706 */
  0x0003,   /* U+D707 */
  0x0003,   /* U+D708 */
  0x0003,   /* U+D709 */
  0x0003,   /* U+D70A */
  0x0003,   /* U+D70B */
  0x0003,   /* U+D70C */
  0x0003,   /* U+D70D */
  0x0003,   /* U+D70E */
  0x0003,   /* U+D70F */
  0x0003,   /* U+D710 */
  0x0003,   /* U+D711 */
  0x0003,   /* U+D712 */
  0x0003,   /* U+D713 */
  0x0003,   /* U+D714 */
  0x0003,   /* U+D715 */
  0x0003,   /* U+D716 */
  0x0003,   /* U+D717 */
  0x0003,   /* U+D718 */
  0x0003,   /* U+D719 */
  0x0003,   /* U+D71A */
  0x0003,   /* U+D71B */
  0x0003,   /* U+D71C */
  0x0003,   /* U+D71D */
  0x0003,   /* U+D71E */
  0x0003,   /* U+D71F */
  0x0003,   /* U+D720 */
  0x0003,   /* U+D721 */
  0x0003,   /* U+D722 */
  0x0003,   /* U+D723 */
  0x0003,   /* U+D724 */
  0x0003,   /* U+D725 */
  0x0003,   /* U+D726 */
  0x0003,   /* U+D727 */
  0x0003,   /* U+D728 */
  0x0003,   /* U+D729 */
  0x0003,   /* U+D72A */
  0x0003,   /* U+D72B */
  0x0003,   /* U+D72C */
  0x0003,   /* U+D72D */
  0x0003,   /* U+D72E */
  0x0003,   /* U+D72F */
  0x0003,   /* U+D730 */
  0x0003,   /* U+D731 */
  0x0003,   /* U+D732 */
  0x0003,   /* U+D733 */
  0x0003,   /* U+D734 */
  0x0003,   /* U+D735 */
  0x0003,   /* U+D736 */
  0x0003,   /* U+D737 */
  0x0003,   /* U+D738 */
  0x0003,   /* U+D739 */
  0x0003,   /* U+D73A */
  0x0003,   /* U+D73B */
  0x0003,   /* U+D73C */
  0x0003,   /* U+D73D */
  0x0003,   /* U+D73E */
  0x0003,   /* U+D73F */
  0x0003,   /* U+D740 */
  0x0003,   /* U+D741 */
  0x0003,   /* U+D742 */
  0x0003,   /* U+D743 */
  0x0003,   /* U+D744 */
  0x0003,   /* U+D745 */
  0x0003,   /* U+D746 */
  0x0003,   /* U+D747 */
  0x0003,   /* U+D748 */
  0x0003,   /* U+D749 */
  0x0003,   /* U+D74A */
  0x0003,   /* U+D74B */
  0x0003,   /* U+D74C */
  0x0003,   /* U+D74D */
  0x0003,   /* U+D74E */
  0x0003,   /* U+D74F */
  0x0003,   /* U+D750 */
  0x0003,   /* U+D751 */
  0x0003,   /* U+D752 */
  0x0003,   /* U+D753 */
  0x0003,   /* U+D754 */
  0x0003,   /* U+D755 */
  0x0003,   /* U+D756 */
  0x0003,   /* U+D757 */
  0x0003,   /* U+D758 */
  0x0003,   /* U+D759 */
  0x0003,   /* U+D75A */
  0x0003,   /* U+D75B */
  0x0003,   /* U+D75C */
  0x0003,   /* U+D75D */
  0x0003,   /* U+D75E */
  0x0003,   /* U+D75F */
  0x0003,   /* U+D760 */
  0x0003,   /* U+D761 */
  0x0003,   /* U+D762 */
  0x0003,   /* U+D763 */
  0x0003,   /* U+D764 */
  0x0003,   /* U+D765 */
  0x0003,   /* U+D766 */
  0x0003,   /* U+D767 */
  0x0003,   /* U+D768 */
  0x0003,   /* U+D769 */
  0x0003,   /* U+D76A */
  0x0003,   /* U+D76B */
  0x0003,   /* U+D76C */
  0x0003,   /* U+D76D */
  0x0003,   /* U+D76E */
  0x0003,   /* U+D76F */
  0x0003,   /* U+D770 */
  0x0003,   /* U+D771 */
  0x0003,   /* U+D772 */
  0x0003,   /* U+D773 */
  0x0003,   /* U+D774 */
  0x0003,   /* U+D775 */
  0x0003,   /* U+D776 */
  0x0003,   /* U+D777 */
  0x0003,   /* U+D778 */
  0x0003,   /* U+D779 */
  0x0003,   /* U+D77A */
  0x0003,   /* U+D77B */
  0x0003,   /* U+D77C */
  0x0003,   /* U+D77D */
  0x0003,   /* U+D77E */
  0x0003,   /* U+D77F */
  0x0003,   /* U+D780 */
  0x0003,   /* U+D781 */
  0x0003,   /* U+D782 */
  0x0003,   /* U+D783 */
  0x0003,   /* U+D784 */
  0x0003,   /* U+D785 */
  0x0003,   /* U+D786 */
  0x0003,   /* U+D787 */
  0x0003,   /* U+D788 */
  0x0003,   /* U+D789 */
  0x0003,   /* U+D78A */
  0x0003,   /* U+D78B */
  0x0003,   /* U+D78C */
  0x0003,   /* U+D78D */
  0x0003,   /* U+D78E */
  0x0003,   /* U+D78F */
  0x0003,   /* U+D790 */
  0x0003,   /* U+D791 */
  0x0003,   /* U+D792 */
  0x0003,   /* U+D793 */
  0x0003,   /* U+D794 */
  0x0003,   /* U+D795 */
  0x0003,   /* U+D796 */
  0x0003,   /* U+D797 */
  0x0003,   /* U+D798 */
  0x0003,   /* U+D799 */
  0x0003,   /* U+D79A */
  0x0003,   /* U+D79B */
  0x0003,   /* U+D79C */
  0x0003,   /* U+D79D */
  0x0003,   /* U+D79E */
  0x0003,   /* U+D79F */
  0x0003,   /* U+D7A0 */
  0x0003,   /* U+D7A1 */
  0x0003,   /* U+D7A2 */
  0x0003,   /* U+D7A3 */
  0x0003,   /* U+D7A4 */
  0x0003,   /* U+D7A5 */
  0x0003,   /* U+D7A6 */
  0x0003,   /* U+D7A7 */
  0x0003,   /* U+D7A8 */
  0x0003,   /* U+D7A9 */
  0x0003,   /* U+D7AA */
  0x0003,   /* U+D7AB */
  0x0003,   /* U+D7AC */
  0x0003,   /* U+D7AD */
  0x0003,   /* U+D7AE */
  0x0003,   /* U+D7AF */
  0x0002,   /* U+D7B0 */
  0x0002,   /* U+D7B1 */
  0x0002,   /* U+D7B2 */
  0x0002,   /* U+D7B3 */
  0x0002,   /* U+D7B4 */
  0x0002,   /* U+D7B5 */
  0x0002,   /* U+D7B6 */
  0x0002,   /* U+D7B7 */
  0x0002,   /* U+D7B8 */
  0x0002,   /* U+D7B9 */
  0x0002,   /* U+D7BA */
  0x0002,   /* U+D7BB */
  0x0002,   /* U+D7BC */
  0x0002,   /* U+D7BD */
  0x0002,   /* U+D7BE */
  0x0002,   /* U+D7BF */
  0x0002,   /* U+D7C0 */
  0x0002,   /* U+D7C1 */
  0x0002,   /* U+D7C2 */
  0x0002,   /* U+D7C3 */
  0x0002,   /* U+D7C4 */
  0x0002,   /* U+D7C5 */
  0x0002,   /* U+D7C6 */
  0x0002,   /* U+D7C7 */
  0x0002,   /* U+D7C8 */
  0x0002,   /* U+D7C9 */
  0x0002,   /* U+D7CA */
  0x0002,   /* U+D7CB */
  0x0002,   /* U+D7CC */
  0x0002,   /* U+D7CD */
  0x0002,   /* U+D7CE */
  0x0002,   /* U+D7CF */
  0x0002,   /* U+D7D0 */
  0x0002,   /* U+D7D1 */
  0x0002,   /* U+D7D2 */
  0x0002,   /* U+D7D3 */
  0x0002,   /* U+D7D4 */
  0x0002,   /* U+D7D5 */
  0x0002,   /* U+D7D6 */
  0x0002,   /* U+D7D7 */
  0x0002,   /* U+D7D8 */
  0x0002,   /* U+D7D9 */
  0x0002,   /* U+D7DA */
  0x0002,   /* U+D7DB */
  0x0002,   /* U+D7DC */
  0x0002,   /* U+D7DD */
  0x0002,   /* U+D7DE */
  0x0002,   /* U+D7DF */
  0x0002,   /* U+D7E0 */
  0x0002,   /* U+D7E1 */
  0x0002,   /* U+D7E2 */
  0x0002,   /* U+D7E3 */
  0x0002,   /* U+D7E4 */
  0x0002,   /* U+D7E5 */
  0x0002,   /* U+D7E6 */
  0x0002,   /* U+D7E7 */
  0x0002,   /* U+D7E8 */
  0x0002,   /* U+D7E9 */
  0x0002,   /* U+D7EA */
  0x0002,   /* U+D7EB */
  0x0002,   /* U+D7EC */
  0x0002,   /* U+D7ED */
  0x0002,   /* U+D7EE */
  0x0002,   /* U+D7EF */
  0x0002,   /* U+D7F0 */
  0x0002,   /* U+D7F1 */
  0x0002,   /* U+D7F2 */
  0x0002,   /* U+D7F3 */
  0x0002,   /* U+D7F4 */
  0x0002,   /* U+D7F5 */
  0x0002,   /* U+D7F6 */
  0x0002,   /* U+D7F7 */
  0x0002,   /* U+D7F8 */
  0x0002,   /* U+D7F9 */
  0x0002,   /* U+D7FA */
  0x0002,   /* U+D7FB */
  0x0002,   /* U+D7FC */
  0x0002,   /* U+D7FD */
  0x0002,   /* U+D7FE */
  0x0002,   /* U+D7FF */

  /* Primary weight 2 for each character. */
  0x3C82,   /* U+D700 */
  0x3C82,   /* U+D701 */
  0x3C82,   /* U+D702 */
  0x3C82,   /* U+D703 */
  0x3C82,   /* U+D704 */
  0x3C82,   /* U+D705 */
  0x3C82,   /* U+D706 */
  0x3C82,   /* U+D707 */
  0x3C82,   /* U+D708 */
  0x3C82,   /* U+D709 */
  0x3C82,   /* U+D70A */
  0x3C82,   /* U+D70B */
  0x3C82,   /* U+D70C */
  0x3C82,   /* U+D70D */
  0x3C82,   /* U+D70E */
  0x3C82,   /* U+D70F */
  0x3C82,   /* U+D710 */
  0x3C82,   /* U+D711 */
  0x3C82,   /* U+D712 */
  0x3C82,   /* U+D713 */
  0x3C82,   /* U+D714 */
  0x3C82,   /* U+D715 */
  0x3C82,   /* U+D716 */
  0x3C82,   /* U+D717 */
  0x3C83,   /* U+D718 */
  0x3C83,   /* U+D719 */
  0x3C83,   /* U+D71A */
  0x3C83,   /* U+D71B */
  0x3C83,   /* U+D71C */
  0x3C83,   /* U+D71D */
  0x3C83,   /* U+D71E */
  0x3C83,   /* U+D71F */
  0x3C83,   /* U+D720 */
  0x3C83,   /* U+D721 */
  0x3C83,   /* U+D722 */
  0x3C83,   /* U+D723 */
  0x3C83,   /* U+D724 */
  0x3C83,   /* U+D725 */
  0x3C83,   /* U+D726 */
  0x3C83,   /* U+D727 */
  0x3C83,   /* U+D728 */
  0x3C83,   /* U+D729 */
  0x3C83,   /* U+D72A */
  0x3C83,   /* U+D72B */
  0x3C83,   /* U+D72C */
  0x3C83,   /* U+D72D */
  0x3C83,   /* U+D72E */
  0x3C83,   /* U+D72F */
  0x3C83,   /* U+D730 */
  0x3C83,   /* U+D731 */
  0x3C83,   /* U+D732 */
  0x3C83,   /* U+D733 */
  0x3C84,   /* U+D734 */
  0x3C84,   /* U+D735 */
  0x3C84,   /* U+D736 */
  0x3C84,   /* U+D737 */
  0x3C84,   /* U+D738 */
  0x3C84,   /* U+D739 */
  0x3C84,   /* U+D73A */
  0x3C84,   /* U+D73B */
  0x3C84,   /* U+D73C */
  0x3C84,   /* U+D73D */
  0x3C84,   /* U+D73E */
  0x3C84,   /* U+D73F */
  0x3C84,   /* U+D740 */
  0x3C84,   /* U+D741 */
  0x3C84,   /* U+D742 */
  0x3C84,   /* U+D743 */
  0x3C84,   /* U+D744 */
  0x3C84,   /* U+D745 */
  0x3C84,   /* U+D746 */
  0x3C84,   /* U+D747 */
  0x3C84,   /* U+D748 */
  0x3C84,   /* U+D749 */
  0x3C84,   /* U+D74A */
  0x3C84,   /* U+D74B */
  0x3C84,   /* U+D74C */
  0x3C84,   /* U+D74D */
  0x3C84,   /* U+D74E */
  0x3C84,   /* U+D74F */
  0x3C85,   /* U+D750 */
  0x3C85,   /* U+D751 */
  0x3C85,   /* U+D752 */
  0x3C85,   /* U+D753 */
  0x3C85,   /* U+D754 */
  0x3C85,   /* U+D755 */
  0x3C85,   /* U+D756 */
  0x3C85,   /* U+D757 */
  0x3C85,   /* U+D758 */
  0x3C85,   /* U+D759 */
  0x3C85,   /* U+D75A */
  0x3C85,   /* U+D75B */
  0x3C85,   /* U+D75C */
  0x3C85,   /* U+D75D */
  0x3C85,   /* U+D75E */
  0x3C85,   /* U+D75F */
  0x3C85,   /* U+D760 */
  0x3C85,   /* U+D761 */
  0x3C85,   /* U+D762 */
  0x3C85,   /* U+D763 */
  0x3C85,   /* U+D764 */
  0x3C85,   /* U+D765 */
  0x3C85,   /* U+D766 */
  0x3C85,   /* U+D767 */
  0x3C85,   /* U+D768 */
  0x3C85,   /* U+D769 */
  0x3C85,   /* U+D76A */
  0x3C85,   /* U+D76B */
  0x3C86,   /* U+D76C */
  0x3C86,   /* U+D76D */
  0x3C86,   /* U+D76E */
  0x3C86,   /* U+D76F */
  0x3C86,   /* U+D770 */
  0x3C86,   /* U+D771 */
  0x3C86,   /* U+D772 */
  0x3C86,   /* U+D773 */
  0x3C86,   /* U+D774 */
  0x3C86,   /* U+D775 */
  0x3C86,   /* U+D776 */
  0x3C86,   /* U+D777 */
  0x3C86,   /* U+D778 */
  0x3C86,   /* U+D779 */
  0x3C86,   /* U+D77A */
  0x3C86,   /* U+D77B */
  0x3C86,   /* U+D77C */
  0x3C86,   /* U+D77D */
  0x3C86,   /* U+D77E */
  0x3C86,   /* U+D77F */
  0x3C86,   /* U+D780 */
  0x3C86,   /* U+D781 */
  0x3C86,   /* U+D782 */
  0x3C86,   /* U+D783 */
  0x3C86,   /* U+D784 */
  0x3C86,   /* U+D785 */
  0x3C86,   /* U+D786 */
  0x3C86,   /* U+D787 */
  0x3C87,   /* U+D788 */
  0x3C87,   /* U+D789 */
  0x3C87,   /* U+D78A */
  0x3C87,   /* U+D78B */
  0x3C87,   /* U+D78C */
  0x3C87,   /* U+D78D */
  0x3C87,   /* U+D78E */
  0x3C87,   /* U+D78F */
  0x3C87,   /* U+D790 */
  0x3C87,   /* U+D791 */
  0x3C87,   /* U+D792 */
  0x3C87,   /* U+D793 */
  0x3C87,   /* U+D794 */
  0x3C87,   /* U+D795 */
  0x3C87,   /* U+D796 */
  0x3C87,   /* U+D797 */
  0x3C87,   /* U+D798 */
  0x3C87,   /* U+D799 */
  0x3C87,   /* U+D79A */
  0x3C87,   /* U+D79B */
  0x3C87,   /* U+D79C */
  0x3C87,   /* U+D79D */
  0x3C87,   /* U+D79E */
  0x3C87,   /* U+D79F */
  0x3C87,   /* U+D7A0 */
  0x3C87,   /* U+D7A1 */
  0x3C87,   /* U+D7A2 */
  0x3C87,   /* U+D7A3 */
  0x3C73,   /* U+D7A4 */
  0x3C73,   /* U+D7A5 */
  0x3C73,   /* U+D7A6 */
  0x3C73,   /* U+D7A7 */
  0x3C73,   /* U+D7A8 */
  0x3C73,   /* U+D7A9 */
  0x3C73,   /* U+D7AA */
  0x3C73,   /* U+D7AB */
  0x3C73,   /* U+D7AC */
  0x3C73,   /* U+D7AD */
  0x3C73,   /* U+D7AE */
  0x3C73,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0xD7C7,   /* U+D7C7 */
  0xD7C8,   /* U+D7C8 */
  0xD7C9,   /* U+D7C9 */
  0xD7CA,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0xD7FC,   /* U+D7FC */
  0xD7FD,   /* U+D7FD */
  0xD7FE,   /* U+D7FE */
  0xD7FF,   /* U+D7FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+D700 */
  0x0020,   /* U+D701 */
  0x0020,   /* U+D702 */
  0x0020,   /* U+D703 */
  0x0020,   /* U+D704 */
  0x0020,   /* U+D705 */
  0x0020,   /* U+D706 */
  0x0020,   /* U+D707 */
  0x0020,   /* U+D708 */
  0x0020,   /* U+D709 */
  0x0020,   /* U+D70A */
  0x0020,   /* U+D70B */
  0x0020,   /* U+D70C */
  0x0020,   /* U+D70D */
  0x0020,   /* U+D70E */
  0x0020,   /* U+D70F */
  0x0020,   /* U+D710 */
  0x0020,   /* U+D711 */
  0x0020,   /* U+D712 */
  0x0020,   /* U+D713 */
  0x0020,   /* U+D714 */
  0x0020,   /* U+D715 */
  0x0020,   /* U+D716 */
  0x0020,   /* U+D717 */
  0x0020,   /* U+D718 */
  0x0020,   /* U+D719 */
  0x0020,   /* U+D71A */
  0x0020,   /* U+D71B */
  0x0020,   /* U+D71C */
  0x0020,   /* U+D71D */
  0x0020,   /* U+D71E */
  0x0020,   /* U+D71F */
  0x0020,   /* U+D720 */
  0x0020,   /* U+D721 */
  0x0020,   /* U+D722 */
  0x0020,   /* U+D723 */
  0x0020,   /* U+D724 */
  0x0020,   /* U+D725 */
  0x0020,   /* U+D726 */
  0x0020,   /* U+D727 */
  0x0020,   /* U+D728 */
  0x0020,   /* U+D729 */
  0x0020,   /* U+D72A */
  0x0020,   /* U+D72B */
  0x0020,   /* U+D72C */
  0x0020,   /* U+D72D */
  0x0020,   /* U+D72E */
  0x0020,   /* U+D72F */
  0x0020,   /* U+D730 */
  0x0020,   /* U+D731 */
  0x0020,   /* U+D732 */
  0x0020,   /* U+D733 */
  0x0020,   /* U+D734 */
  0x0020,   /* U+D735 */
  0x0020,   /* U+D736 */
  0x0020,   /* U+D737 */
  0x0020,   /* U+D738 */
  0x0020,   /* U+D739 */
  0x0020,   /* U+D73A */
  0x0020,   /* U+D73B */
  0x0020,   /* U+D73C */
  0x0020,   /* U+D73D */
  0x0020,   /* U+D73E */
  0x0020,   /* U+D73F */
  0x0020,   /* U+D740 */
  0x0020,   /* U+D741 */
  0x0020,   /* U+D742 */
  0x0020,   /* U+D743 */
  0x0020,   /* U+D744 */
  0x0020,   /* U+D745 */
  0x0020,   /* U+D746 */
  0x0020,   /* U+D747 */
  0x0020,   /* U+D748 */
  0x0020,   /* U+D749 */
  0x0020,   /* U+D74A */
  0x0020,   /* U+D74B */
  0x0020,   /* U+D74C */
  0x0020,   /* U+D74D */
  0x0020,   /* U+D74E */
  0x0020,   /* U+D74F */
  0x0020,   /* U+D750 */
  0x0020,   /* U+D751 */
  0x0020,   /* U+D752 */
  0x0020,   /* U+D753 */
  0x0020,   /* U+D754 */
  0x0020,   /* U+D755 */
  0x0020,   /* U+D756 */
  0x0020,   /* U+D757 */
  0x0020,   /* U+D758 */
  0x0020,   /* U+D759 */
  0x0020,   /* U+D75A */
  0x0020,   /* U+D75B */
  0x0020,   /* U+D75C */
  0x0020,   /* U+D75D */
  0x0020,   /* U+D75E */
  0x0020,   /* U+D75F */
  0x0020,   /* U+D760 */
  0x0020,   /* U+D761 */
  0x0020,   /* U+D762 */
  0x0020,   /* U+D763 */
  0x0020,   /* U+D764 */
  0x0020,   /* U+D765 */
  0x0020,   /* U+D766 */
  0x0020,   /* U+D767 */
  0x0020,   /* U+D768 */
  0x0020,   /* U+D769 */
  0x0020,   /* U+D76A */
  0x0020,   /* U+D76B */
  0x0020,   /* U+D76C */
  0x0020,   /* U+D76D */
  0x0020,   /* U+D76E */
  0x0020,   /* U+D76F */
  0x0020,   /* U+D770 */
  0x0020,   /* U+D771 */
  0x0020,   /* U+D772 */
  0x0020,   /* U+D773 */
  0x0020,   /* U+D774 */
  0x0020,   /* U+D775 */
  0x0020,   /* U+D776 */
  0x0020,   /* U+D777 */
  0x0020,   /* U+D778 */
  0x0020,   /* U+D779 */
  0x0020,   /* U+D77A */
  0x0020,   /* U+D77B */
  0x0020,   /* U+D77C */
  0x0020,   /* U+D77D */
  0x0020,   /* U+D77E */
  0x0020,   /* U+D77F */
  0x0020,   /* U+D780 */
  0x0020,   /* U+D781 */
  0x0020,   /* U+D782 */
  0x0020,   /* U+D783 */
  0x0020,   /* U+D784 */
  0x0020,   /* U+D785 */
  0x0020,   /* U+D786 */
  0x0020,   /* U+D787 */
  0x0020,   /* U+D788 */
  0x0020,   /* U+D789 */
  0x0020,   /* U+D78A */
  0x0020,   /* U+D78B */
  0x0020,   /* U+D78C */
  0x0020,   /* U+D78D */
  0x0020,   /* U+D78E */
  0x0020,   /* U+D78F */
  0x0020,   /* U+D790 */
  0x0020,   /* U+D791 */
  0x0020,   /* U+D792 */
  0x0020,   /* U+D793 */
  0x0020,   /* U+D794 */
  0x0020,   /* U+D795 */
  0x0020,   /* U+D796 */
  0x0020,   /* U+D797 */
  0x0020,   /* U+D798 */
  0x0020,   /* U+D799 */
  0x0020,   /* U+D79A */
  0x0020,   /* U+D79B */
  0x0020,   /* U+D79C */
  0x0020,   /* U+D79D */
  0x0020,   /* U+D79E */
  0x0020,   /* U+D79F */
  0x0020,   /* U+D7A0 */
  0x0020,   /* U+D7A1 */
  0x0020,   /* U+D7A2 */
  0x0020,   /* U+D7A3 */
  0x0020,   /* U+D7A4 */
  0x0020,   /* U+D7A5 */
  0x0020,   /* U+D7A6 */
  0x0020,   /* U+D7A7 */
  0x0020,   /* U+D7A8 */
  0x0020,   /* U+D7A9 */
  0x0020,   /* U+D7AA */
  0x0020,   /* U+D7AB */
  0x0020,   /* U+D7AC */
  0x0020,   /* U+D7AD */
  0x0020,   /* U+D7AE */
  0x0020,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0x0000,   /* U+D7C7 */
  0x0000,   /* U+D7C8 */
  0x0000,   /* U+D7C9 */
  0x0000,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0x0000,   /* U+D7FC */
  0x0000,   /* U+D7FD */
  0x0000,   /* U+D7FE */
  0x0000,   /* U+D7FF */

  /* Tertiary weight 2 for each character. */
  0x0003,   /* U+D700 */
  0x0003,   /* U+D701 */
  0x0003,   /* U+D702 */
  0x0003,   /* U+D703 */
  0x0003,   /* U+D704 */
  0x0003,   /* U+D705 */
  0x0003,   /* U+D706 */
  0x0003,   /* U+D707 */
  0x0003,   /* U+D708 */
  0x0003,   /* U+D709 */
  0x0003,   /* U+D70A */
  0x0003,   /* U+D70B */
  0x0003,   /* U+D70C */
  0x0003,   /* U+D70D */
  0x0003,   /* U+D70E */
  0x0003,   /* U+D70F */
  0x0003,   /* U+D710 */
  0x0003,   /* U+D711 */
  0x0003,   /* U+D712 */
  0x0003,   /* U+D713 */
  0x0003,   /* U+D714 */
  0x0003,   /* U+D715 */
  0x0003,   /* U+D716 */
  0x0003,   /* U+D717 */
  0x0003,   /* U+D718 */
  0x0003,   /* U+D719 */
  0x0003,   /* U+D71A */
  0x0003,   /* U+D71B */
  0x0003,   /* U+D71C */
  0x0003,   /* U+D71D */
  0x0003,   /* U+D71E */
  0x0003,   /* U+D71F */
  0x0003,   /* U+D720 */
  0x0003,   /* U+D721 */
  0x0003,   /* U+D722 */
  0x0003,   /* U+D723 */
  0x0003,   /* U+D724 */
  0x0003,   /* U+D725 */
  0x0003,   /* U+D726 */
  0x0003,   /* U+D727 */
  0x0003,   /* U+D728 */
  0x0003,   /* U+D729 */
  0x0003,   /* U+D72A */
  0x0003,   /* U+D72B */
  0x0003,   /* U+D72C */
  0x0003,   /* U+D72D */
  0x0003,   /* U+D72E */
  0x0003,   /* U+D72F */
  0x0003,   /* U+D730 */
  0x0003,   /* U+D731 */
  0x0003,   /* U+D732 */
  0x0003,   /* U+D733 */
  0x0003,   /* U+D734 */
  0x0003,   /* U+D735 */
  0x0003,   /* U+D736 */
  0x0003,   /* U+D737 */
  0x0003,   /* U+D738 */
  0x0003,   /* U+D739 */
  0x0003,   /* U+D73A */
  0x0003,   /* U+D73B */
  0x0003,   /* U+D73C */
  0x0003,   /* U+D73D */
  0x0003,   /* U+D73E */
  0x0003,   /* U+D73F */
  0x0003,   /* U+D740 */
  0x0003,   /* U+D741 */
  0x0003,   /* U+D742 */
  0x0003,   /* U+D743 */
  0x0003,   /* U+D744 */
  0x0003,   /* U+D745 */
  0x0003,   /* U+D746 */
  0x0003,   /* U+D747 */
  0x0003,   /* U+D748 */
  0x0003,   /* U+D749 */
  0x0003,   /* U+D74A */
  0x0003,   /* U+D74B */
  0x0003,   /* U+D74C */
  0x0003,   /* U+D74D */
  0x0003,   /* U+D74E */
  0x0003,   /* U+D74F */
  0x0003,   /* U+D750 */
  0x0003,   /* U+D751 */
  0x0003,   /* U+D752 */
  0x0003,   /* U+D753 */
  0x0003,   /* U+D754 */
  0x0003,   /* U+D755 */
  0x0003,   /* U+D756 */
  0x0003,   /* U+D757 */
  0x0003,   /* U+D758 */
  0x0003,   /* U+D759 */
  0x0003,   /* U+D75A */
  0x0003,   /* U+D75B */
  0x0003,   /* U+D75C */
  0x0003,   /* U+D75D */
  0x0003,   /* U+D75E */
  0x0003,   /* U+D75F */
  0x0003,   /* U+D760 */
  0x0003,   /* U+D761 */
  0x0003,   /* U+D762 */
  0x0003,   /* U+D763 */
  0x0003,   /* U+D764 */
  0x0003,   /* U+D765 */
  0x0003,   /* U+D766 */
  0x0003,   /* U+D767 */
  0x0003,   /* U+D768 */
  0x0003,   /* U+D769 */
  0x0003,   /* U+D76A */
  0x0003,   /* U+D76B */
  0x0003,   /* U+D76C */
  0x0003,   /* U+D76D */
  0x0003,   /* U+D76E */
  0x0003,   /* U+D76F */
  0x0003,   /* U+D770 */
  0x0003,   /* U+D771 */
  0x0003,   /* U+D772 */
  0x0003,   /* U+D773 */
  0x0003,   /* U+D774 */
  0x0003,   /* U+D775 */
  0x0003,   /* U+D776 */
  0x0003,   /* U+D777 */
  0x0003,   /* U+D778 */
  0x0003,   /* U+D779 */
  0x0003,   /* U+D77A */
  0x0003,   /* U+D77B */
  0x0003,   /* U+D77C */
  0x0003,   /* U+D77D */
  0x0003,   /* U+D77E */
  0x0003,   /* U+D77F */
  0x0003,   /* U+D780 */
  0x0003,   /* U+D781 */
  0x0003,   /* U+D782 */
  0x0003,   /* U+D783 */
  0x0003,   /* U+D784 */
  0x0003,   /* U+D785 */
  0x0003,   /* U+D786 */
  0x0003,   /* U+D787 */
  0x0003,   /* U+D788 */
  0x0003,   /* U+D789 */
  0x0003,   /* U+D78A */
  0x0003,   /* U+D78B */
  0x0003,   /* U+D78C */
  0x0003,   /* U+D78D */
  0x0003,   /* U+D78E */
  0x0003,   /* U+D78F */
  0x0003,   /* U+D790 */
  0x0003,   /* U+D791 */
  0x0003,   /* U+D792 */
  0x0003,   /* U+D793 */
  0x0003,   /* U+D794 */
  0x0003,   /* U+D795 */
  0x0003,   /* U+D796 */
  0x0003,   /* U+D797 */
  0x0003,   /* U+D798 */
  0x0003,   /* U+D799 */
  0x0003,   /* U+D79A */
  0x0003,   /* U+D79B */
  0x0003,   /* U+D79C */
  0x0003,   /* U+D79D */
  0x0003,   /* U+D79E */
  0x0003,   /* U+D79F */
  0x0003,   /* U+D7A0 */
  0x0003,   /* U+D7A1 */
  0x0003,   /* U+D7A2 */
  0x0003,   /* U+D7A3 */
  0x0003,   /* U+D7A4 */
  0x0003,   /* U+D7A5 */
  0x0003,   /* U+D7A6 */
  0x0003,   /* U+D7A7 */
  0x0003,   /* U+D7A8 */
  0x0003,   /* U+D7A9 */
  0x0003,   /* U+D7AA */
  0x0003,   /* U+D7AB */
  0x0003,   /* U+D7AC */
  0x0003,   /* U+D7AD */
  0x0003,   /* U+D7AE */
  0x0003,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0x0000,   /* U+D7C7 */
  0x0000,   /* U+D7C8 */
  0x0000,   /* U+D7C9 */
  0x0000,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0x0000,   /* U+D7FC */
  0x0000,   /* U+D7FD */
  0x0000,   /* U+D7FE */
  0x0000,   /* U+D7FF */

  /* Primary weight 3 for each character. */
  0x3CD4,   /* U+D700 */
  0x3CD5,   /* U+D701 */
  0x3CD6,   /* U+D702 */
  0x3CD7,   /* U+D703 */
  0x3CD8,   /* U+D704 */
  0x3CD9,   /* U+D705 */
  0x3CDA,   /* U+D706 */
  0x3CDB,   /* U+D707 */
  0x3CDC,   /* U+D708 */
  0x3CDD,   /* U+D709 */
  0x3CDE,   /* U+D70A */
  0x3CDF,   /* U+D70B */
  0x3CE0,   /* U+D70C */
  0x3CE1,   /* U+D70D */
  0x3CE2,   /* U+D70E */
  0x3CE3,   /* U+D70F */
  0x3CE4,   /* U+D710 */
  0x3CE5,   /* U+D711 */
  0x3CE6,   /* U+D712 */
  0x3CE7,   /* U+D713 */
  0x3CE8,   /* U+D714 */
  0x3CE9,   /* U+D715 */
  0x3CEA,   /* U+D716 */
  0x3CEB,   /* U+D717 */
  0x0000,   /* U+D718 */
  0x3CD1,   /* U+D719 */
  0x3CD2,   /* U+D71A */
  0x3CD3,   /* U+D71B */
  0x3CD4,   /* U+D71C */
  0x3CD5,   /* U+D71D */
  0x3CD6,   /* U+D71E */
  0x3CD7,   /* U+D71F */
  0x3CD8,   /* U+D720 */
  0x3CD9,   /* U+D721 */
  0x3CDA,   /* U+D722 */
  0x3CDB,   /* U+D723 */
  0x3CDC,   /* U+D724 */
  0x3CDD,   /* U+D725 */
  0x3CDE,   /* U+D726 */
  0x3CDF,   /* U+D727 */
  0x3CE0,   /* U+D728 */
  0x3CE1,   /* U+D729 */
  0x3CE2,   /* U+D72A */
  0x3CE3,   /* U+D72B */
  0x3CE4,   /* U+D72C */
  0x3CE5,   /* U+D72D */
  0x3CE6,   /* U+D72E */
  0x3CE7,   /* U+D72F */
  0x3CE8,   /* U+D730 */
  0x3CE9,   /* U+D731 */
  0x3CEA,   /* U+D732 */
  0x3CEB,   /* U+D733 */
  0x0000,   /* U+D734 */
  0x3CD1,   /* U+D735 */
  0x3CD2,   /* U+D736 */
  0x3CD3,   /* U+D737 */
  0x3CD4,   /* U+D738 */
  0x3CD5,   /* U+D739 */
  0x3CD6,   /* U+D73A */
  0x3CD7,   /* U+D73B */
  0x3CD8,   /* U+D73C */
  0x3CD9,   /* U+D73D */
  0x3CDA,   /* U+D73E */
  0x3CDB,   /* U+D73F */
  0x3CDC,   /* U+D740 */
  0x3CDD,   /* U+D741 */
  0x3CDE,   /* U+D742 */
  0x3CDF,   /* U+D743 */
  0x3CE0,   /* U+D744 */
  0x3CE1,   /* U+D745 */
  0x3CE2,   /* U+D746 */
  0x3CE3,   /* U+D747 */
  0x3CE4,   /* U+D748 */
  0x3CE5,   /* U+D749 */
  0x3CE6,   /* U+D74A */
  0x3CE7,   /* U+D74B */
  0x3CE8,   /* U+D74C */
  0x3CE9,   /* U+D74D */
  0x3CEA,   /* U+D74E */
  0x3CEB,   /* U+D74F */
  0x0000,   /* U+D750 */
  0x3CD1,   /* U+D751 */
  0x3CD2,   /* U+D752 */
  0x3CD3,   /* U+D753 */
  0x3CD4,   /* U+D754 */
  0x3CD5,   /* U+D755 */
  0x3CD6,   /* U+D756 */
  0x3CD7,   /* U+D757 */
  0x3CD8,   /* U+D758 */
  0x3CD9,   /* U+D759 */
  0x3CDA,   /* U+D75A */
  0x3CDB,   /* U+D75B */
  0x3CDC,   /* U+D75C */
  0x3CDD,   /* U+D75D */
  0x3CDE,   /* U+D75E */
  0x3CDF,   /* U+D75F */
  0x3CE0,   /* U+D760 */
  0x3CE1,   /* U+D761 */
  0x3CE2,   /* U+D762 */
  0x3CE3,   /* U+D763 */
  0x3CE4,   /* U+D764 */
  0x3CE5,   /* U+D765 */
  0x3CE6,   /* U+D766 */
  0x3CE7,   /* U+D767 */
  0x3CE8,   /* U+D768 */
  0x3CE9,   /* U+D769 */
  0x3CEA,   /* U+D76A */
  0x3CEB,   /* U+D76B */
  0x0000,   /* U+D76C */
  0x3CD1,   /* U+D76D */
  0x3CD2,   /* U+D76E */
  0x3CD3,   /* U+D76F */
  0x3CD4,   /* U+D770 */
  0x3CD5,   /* U+D771 */
  0x3CD6,   /* U+D772 */
  0x3CD7,   /* U+D773 */
  0x3CD8,   /* U+D774 */
  0x3CD9,   /* U+D775 */
  0x3CDA,   /* U+D776 */
  0x3CDB,   /* U+D777 */
  0x3CDC,   /* U+D778 */
  0x3CDD,   /* U+D779 */
  0x3CDE,   /* U+D77A */
  0x3CDF,   /* U+D77B */
  0x3CE0,   /* U+D77C */
  0x3CE1,   /* U+D77D */
  0x3CE2,   /* U+D77E */
  0x3CE3,   /* U+D77F */
  0x3CE4,   /* U+D780 */
  0x3CE5,   /* U+D781 */
  0x3CE6,   /* U+D782 */
  0x3CE7,   /* U+D783 */
  0x3CE8,   /* U+D784 */
  0x3CE9,   /* U+D785 */
  0x3CEA,   /* U+D786 */
  0x3CEB,   /* U+D787 */
  0x0000,   /* U+D788 */
  0x3CD1,   /* U+D789 */
  0x3CD2,   /* U+D78A */
  0x3CD3,   /* U+D78B */
  0x3CD4,   /* U+D78C */
  0x3CD5,   /* U+D78D */
  0x3CD6,   /* U+D78E */
  0x3CD7,   /* U+D78F */
  0x3CD8,   /* U+D790 */
  0x3CD9,   /* U+D791 */
  0x3CDA,   /* U+D792 */
  0x3CDB,   /* U+D793 */
  0x3CDC,   /* U+D794 */
  0x3CDD,   /* U+D795 */
  0x3CDE,   /* U+D796 */
  0x3CDF,   /* U+D797 */
  0x3CE0,   /* U+D798 */
  0x3CE1,   /* U+D799 */
  0x3CE2,   /* U+D79A */
  0x3CE3,   /* U+D79B */
  0x3CE4,   /* U+D79C */
  0x3CE5,   /* U+D79D */
  0x3CE6,   /* U+D79E */
  0x3CE7,   /* U+D79F */
  0x3CE8,   /* U+D7A0 */
  0x3CE9,   /* U+D7A1 */
  0x3CEA,   /* U+D7A2 */
  0x3CEB,   /* U+D7A3 */
  0x0000,   /* U+D7A4 */
  0x3CD1,   /* U+D7A5 */
  0x3CD2,   /* U+D7A6 */
  0x3CD3,   /* U+D7A7 */
  0x3CD4,   /* U+D7A8 */
  0x3CD5,   /* U+D7A9 */
  0x3CD6,   /* U+D7AA */
  0x3CD7,   /* U+D7AB */
  0x3CD8,   /* U+D7AC */
  0x3CD9,   /* U+D7AD */
  0x3CDA,   /* U+D7AE */
  0x3CDB,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0x0000,   /* U+D7C7 */
  0x0000,   /* U+D7C8 */
  0x0000,   /* U+D7C9 */
  0x0000,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0x0000,   /* U+D7FC */
  0x0000,   /* U+D7FD */
  0x0000,   /* U+D7FE */
  0x0000,   /* U+D7FF */

  /* Secondary weight 3 for each character. */
  0x0020,   /* U+D700 */
  0x0020,   /* U+D701 */
  0x0020,   /* U+D702 */
  0x0020,   /* U+D703 */
  0x0020,   /* U+D704 */
  0x0020,   /* U+D705 */
  0x0020,   /* U+D706 */
  0x0020,   /* U+D707 */
  0x0020,   /* U+D708 */
  0x0020,   /* U+D709 */
  0x0020,   /* U+D70A */
  0x0020,   /* U+D70B */
  0x0020,   /* U+D70C */
  0x0020,   /* U+D70D */
  0x0020,   /* U+D70E */
  0x0020,   /* U+D70F */
  0x0020,   /* U+D710 */
  0x0020,   /* U+D711 */
  0x0020,   /* U+D712 */
  0x0020,   /* U+D713 */
  0x0020,   /* U+D714 */
  0x0020,   /* U+D715 */
  0x0020,   /* U+D716 */
  0x0020,   /* U+D717 */
  0x0000,   /* U+D718 */
  0x0020,   /* U+D719 */
  0x0020,   /* U+D71A */
  0x0020,   /* U+D71B */
  0x0020,   /* U+D71C */
  0x0020,   /* U+D71D */
  0x0020,   /* U+D71E */
  0x0020,   /* U+D71F */
  0x0020,   /* U+D720 */
  0x0020,   /* U+D721 */
  0x0020,   /* U+D722 */
  0x0020,   /* U+D723 */
  0x0020,   /* U+D724 */
  0x0020,   /* U+D725 */
  0x0020,   /* U+D726 */
  0x0020,   /* U+D727 */
  0x0020,   /* U+D728 */
  0x0020,   /* U+D729 */
  0x0020,   /* U+D72A */
  0x0020,   /* U+D72B */
  0x0020,   /* U+D72C */
  0x0020,   /* U+D72D */
  0x0020,   /* U+D72E */
  0x0020,   /* U+D72F */
  0x0020,   /* U+D730 */
  0x0020,   /* U+D731 */
  0x0020,   /* U+D732 */
  0x0020,   /* U+D733 */
  0x0000,   /* U+D734 */
  0x0020,   /* U+D735 */
  0x0020,   /* U+D736 */
  0x0020,   /* U+D737 */
  0x0020,   /* U+D738 */
  0x0020,   /* U+D739 */
  0x0020,   /* U+D73A */
  0x0020,   /* U+D73B */
  0x0020,   /* U+D73C */
  0x0020,   /* U+D73D */
  0x0020,   /* U+D73E */
  0x0020,   /* U+D73F */
  0x0020,   /* U+D740 */
  0x0020,   /* U+D741 */
  0x0020,   /* U+D742 */
  0x0020,   /* U+D743 */
  0x0020,   /* U+D744 */
  0x0020,   /* U+D745 */
  0x0020,   /* U+D746 */
  0x0020,   /* U+D747 */
  0x0020,   /* U+D748 */
  0x0020,   /* U+D749 */
  0x0020,   /* U+D74A */
  0x0020,   /* U+D74B */
  0x0020,   /* U+D74C */
  0x0020,   /* U+D74D */
  0x0020,   /* U+D74E */
  0x0020,   /* U+D74F */
  0x0000,   /* U+D750 */
  0x0020,   /* U+D751 */
  0x0020,   /* U+D752 */
  0x0020,   /* U+D753 */
  0x0020,   /* U+D754 */
  0x0020,   /* U+D755 */
  0x0020,   /* U+D756 */
  0x0020,   /* U+D757 */
  0x0020,   /* U+D758 */
  0x0020,   /* U+D759 */
  0x0020,   /* U+D75A */
  0x0020,   /* U+D75B */
  0x0020,   /* U+D75C */
  0x0020,   /* U+D75D */
  0x0020,   /* U+D75E */
  0x0020,   /* U+D75F */
  0x0020,   /* U+D760 */
  0x0020,   /* U+D761 */
  0x0020,   /* U+D762 */
  0x0020,   /* U+D763 */
  0x0020,   /* U+D764 */
  0x0020,   /* U+D765 */
  0x0020,   /* U+D766 */
  0x0020,   /* U+D767 */
  0x0020,   /* U+D768 */
  0x0020,   /* U+D769 */
  0x0020,   /* U+D76A */
  0x0020,   /* U+D76B */
  0x0000,   /* U+D76C */
  0x0020,   /* U+D76D */
  0x0020,   /* U+D76E */
  0x0020,   /* U+D76F */
  0x0020,   /* U+D770 */
  0x0020,   /* U+D771 */
  0x0020,   /* U+D772 */
  0x0020,   /* U+D773 */
  0x0020,   /* U+D774 */
  0x0020,   /* U+D775 */
  0x0020,   /* U+D776 */
  0x0020,   /* U+D777 */
  0x0020,   /* U+D778 */
  0x0020,   /* U+D779 */
  0x0020,   /* U+D77A */
  0x0020,   /* U+D77B */
  0x0020,   /* U+D77C */
  0x0020,   /* U+D77D */
  0x0020,   /* U+D77E */
  0x0020,   /* U+D77F */
  0x0020,   /* U+D780 */
  0x0020,   /* U+D781 */
  0x0020,   /* U+D782 */
  0x0020,   /* U+D783 */
  0x0020,   /* U+D784 */
  0x0020,   /* U+D785 */
  0x0020,   /* U+D786 */
  0x0020,   /* U+D787 */
  0x0000,   /* U+D788 */
  0x0020,   /* U+D789 */
  0x0020,   /* U+D78A */
  0x0020,   /* U+D78B */
  0x0020,   /* U+D78C */
  0x0020,   /* U+D78D */
  0x0020,   /* U+D78E */
  0x0020,   /* U+D78F */
  0x0020,   /* U+D790 */
  0x0020,   /* U+D791 */
  0x0020,   /* U+D792 */
  0x0020,   /* U+D793 */
  0x0020,   /* U+D794 */
  0x0020,   /* U+D795 */
  0x0020,   /* U+D796 */
  0x0020,   /* U+D797 */
  0x0020,   /* U+D798 */
  0x0020,   /* U+D799 */
  0x0020,   /* U+D79A */
  0x0020,   /* U+D79B */
  0x0020,   /* U+D79C */
  0x0020,   /* U+D79D */
  0x0020,   /* U+D79E */
  0x0020,   /* U+D79F */
  0x0020,   /* U+D7A0 */
  0x0020,   /* U+D7A1 */
  0x0020,   /* U+D7A2 */
  0x0020,   /* U+D7A3 */
  0x0000,   /* U+D7A4 */
  0x0020,   /* U+D7A5 */
  0x0020,   /* U+D7A6 */
  0x0020,   /* U+D7A7 */
  0x0020,   /* U+D7A8 */
  0x0020,   /* U+D7A9 */
  0x0020,   /* U+D7AA */
  0x0020,   /* U+D7AB */
  0x0020,   /* U+D7AC */
  0x0020,   /* U+D7AD */
  0x0020,   /* U+D7AE */
  0x0020,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0x0000,   /* U+D7C7 */
  0x0000,   /* U+D7C8 */
  0x0000,   /* U+D7C9 */
  0x0000,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0x0000,   /* U+D7FC */
  0x0000,   /* U+D7FD */
  0x0000,   /* U+D7FE */
  0x0000,   /* U+D7FF */

  /* Tertiary weight 3 for each character. */
  0x0003,   /* U+D700 */
  0x0003,   /* U+D701 */
  0x0003,   /* U+D702 */
  0x0003,   /* U+D703 */
  0x0003,   /* U+D704 */
  0x0003,   /* U+D705 */
  0x0003,   /* U+D706 */
  0x0003,   /* U+D707 */
  0x0003,   /* U+D708 */
  0x0003,   /* U+D709 */
  0x0003,   /* U+D70A */
  0x0003,   /* U+D70B */
  0x0003,   /* U+D70C */
  0x0003,   /* U+D70D */
  0x0003,   /* U+D70E */
  0x0003,   /* U+D70F */
  0x0003,   /* U+D710 */
  0x0003,   /* U+D711 */
  0x0003,   /* U+D712 */
  0x0003,   /* U+D713 */
  0x0003,   /* U+D714 */
  0x0003,   /* U+D715 */
  0x0003,   /* U+D716 */
  0x0003,   /* U+D717 */
  0x0000,   /* U+D718 */
  0x0003,   /* U+D719 */
  0x0003,   /* U+D71A */
  0x0003,   /* U+D71B */
  0x0003,   /* U+D71C */
  0x0003,   /* U+D71D */
  0x0003,   /* U+D71E */
  0x0003,   /* U+D71F */
  0x0003,   /* U+D720 */
  0x0003,   /* U+D721 */
  0x0003,   /* U+D722 */
  0x0003,   /* U+D723 */
  0x0003,   /* U+D724 */
  0x0003,   /* U+D725 */
  0x0003,   /* U+D726 */
  0x0003,   /* U+D727 */
  0x0003,   /* U+D728 */
  0x0003,   /* U+D729 */
  0x0003,   /* U+D72A */
  0x0003,   /* U+D72B */
  0x0003,   /* U+D72C */
  0x0003,   /* U+D72D */
  0x0003,   /* U+D72E */
  0x0003,   /* U+D72F */
  0x0003,   /* U+D730 */
  0x0003,   /* U+D731 */
  0x0003,   /* U+D732 */
  0x0003,   /* U+D733 */
  0x0000,   /* U+D734 */
  0x0003,   /* U+D735 */
  0x0003,   /* U+D736 */
  0x0003,   /* U+D737 */
  0x0003,   /* U+D738 */
  0x0003,   /* U+D739 */
  0x0003,   /* U+D73A */
  0x0003,   /* U+D73B */
  0x0003,   /* U+D73C */
  0x0003,   /* U+D73D */
  0x0003,   /* U+D73E */
  0x0003,   /* U+D73F */
  0x0003,   /* U+D740 */
  0x0003,   /* U+D741 */
  0x0003,   /* U+D742 */
  0x0003,   /* U+D743 */
  0x0003,   /* U+D744 */
  0x0003,   /* U+D745 */
  0x0003,   /* U+D746 */
  0x0003,   /* U+D747 */
  0x0003,   /* U+D748 */
  0x0003,   /* U+D749 */
  0x0003,   /* U+D74A */
  0x0003,   /* U+D74B */
  0x0003,   /* U+D74C */
  0x0003,   /* U+D74D */
  0x0003,   /* U+D74E */
  0x0003,   /* U+D74F */
  0x0000,   /* U+D750 */
  0x0003,   /* U+D751 */
  0x0003,   /* U+D752 */
  0x0003,   /* U+D753 */
  0x0003,   /* U+D754 */
  0x0003,   /* U+D755 */
  0x0003,   /* U+D756 */
  0x0003,   /* U+D757 */
  0x0003,   /* U+D758 */
  0x0003,   /* U+D759 */
  0x0003,   /* U+D75A */
  0x0003,   /* U+D75B */
  0x0003,   /* U+D75C */
  0x0003,   /* U+D75D */
  0x0003,   /* U+D75E */
  0x0003,   /* U+D75F */
  0x0003,   /* U+D760 */
  0x0003,   /* U+D761 */
  0x0003,   /* U+D762 */
  0x0003,   /* U+D763 */
  0x0003,   /* U+D764 */
  0x0003,   /* U+D765 */
  0x0003,   /* U+D766 */
  0x0003,   /* U+D767 */
  0x0003,   /* U+D768 */
  0x0003,   /* U+D769 */
  0x0003,   /* U+D76A */
  0x0003,   /* U+D76B */
  0x0000,   /* U+D76C */
  0x0003,   /* U+D76D */
  0x0003,   /* U+D76E */
  0x0003,   /* U+D76F */
  0x0003,   /* U+D770 */
  0x0003,   /* U+D771 */
  0x0003,   /* U+D772 */
  0x0003,   /* U+D773 */
  0x0003,   /* U+D774 */
  0x0003,   /* U+D775 */
  0x0003,   /* U+D776 */
  0x0003,   /* U+D777 */
  0x0003,   /* U+D778 */
  0x0003,   /* U+D779 */
  0x0003,   /* U+D77A */
  0x0003,   /* U+D77B */
  0x0003,   /* U+D77C */
  0x0003,   /* U+D77D */
  0x0003,   /* U+D77E */
  0x0003,   /* U+D77F */
  0x0003,   /* U+D780 */
  0x0003,   /* U+D781 */
  0x0003,   /* U+D782 */
  0x0003,   /* U+D783 */
  0x0003,   /* U+D784 */
  0x0003,   /* U+D785 */
  0x0003,   /* U+D786 */
  0x0003,   /* U+D787 */
  0x0000,   /* U+D788 */
  0x0003,   /* U+D789 */
  0x0003,   /* U+D78A */
  0x0003,   /* U+D78B */
  0x0003,   /* U+D78C */
  0x0003,   /* U+D78D */
  0x0003,   /* U+D78E */
  0x0003,   /* U+D78F */
  0x0003,   /* U+D790 */
  0x0003,   /* U+D791 */
  0x0003,   /* U+D792 */
  0x0003,   /* U+D793 */
  0x0003,   /* U+D794 */
  0x0003,   /* U+D795 */
  0x0003,   /* U+D796 */
  0x0003,   /* U+D797 */
  0x0003,   /* U+D798 */
  0x0003,   /* U+D799 */
  0x0003,   /* U+D79A */
  0x0003,   /* U+D79B */
  0x0003,   /* U+D79C */
  0x0003,   /* U+D79D */
  0x0003,   /* U+D79E */
  0x0003,   /* U+D79F */
  0x0003,   /* U+D7A0 */
  0x0003,   /* U+D7A1 */
  0x0003,   /* U+D7A2 */
  0x0003,   /* U+D7A3 */
  0x0000,   /* U+D7A4 */
  0x0003,   /* U+D7A5 */
  0x0003,   /* U+D7A6 */
  0x0003,   /* U+D7A7 */
  0x0003,   /* U+D7A8 */
  0x0003,   /* U+D7A9 */
  0x0003,   /* U+D7AA */
  0x0003,   /* U+D7AB */
  0x0003,   /* U+D7AC */
  0x0003,   /* U+D7AD */
  0x0003,   /* U+D7AE */
  0x0003,   /* U+D7AF */
  0x0000,   /* U+D7B0 */
  0x0000,   /* U+D7B1 */
  0x0000,   /* U+D7B2 */
  0x0000,   /* U+D7B3 */
  0x0000,   /* U+D7B4 */
  0x0000,   /* U+D7B5 */
  0x0000,   /* U+D7B6 */
  0x0000,   /* U+D7B7 */
  0x0000,   /* U+D7B8 */
  0x0000,   /* U+D7B9 */
  0x0000,   /* U+D7BA */
  0x0000,   /* U+D7BB */
  0x0000,   /* U+D7BC */
  0x0000,   /* U+D7BD */
  0x0000,   /* U+D7BE */
  0x0000,   /* U+D7BF */
  0x0000,   /* U+D7C0 */
  0x0000,   /* U+D7C1 */
  0x0000,   /* U+D7C2 */
  0x0000,   /* U+D7C3 */
  0x0000,   /* U+D7C4 */
  0x0000,   /* U+D7C5 */
  0x0000,   /* U+D7C6 */
  0x0000,   /* U+D7C7 */
  0x0000,   /* U+D7C8 */
  0x0000,   /* U+D7C9 */
  0x0000,   /* U+D7CA */
  0x0000,   /* U+D7CB */
  0x0000,   /* U+D7CC */
  0x0000,   /* U+D7CD */
  0x0000,   /* U+D7CE */
  0x0000,   /* U+D7CF */
  0x0000,   /* U+D7D0 */
  0x0000,   /* U+D7D1 */
  0x0000,   /* U+D7D2 */
  0x0000,   /* U+D7D3 */
  0x0000,   /* U+D7D4 */
  0x0000,   /* U+D7D5 */
  0x0000,   /* U+D7D6 */
  0x0000,   /* U+D7D7 */
  0x0000,   /* U+D7D8 */
  0x0000,   /* U+D7D9 */
  0x0000,   /* U+D7DA */
  0x0000,   /* U+D7DB */
  0x0000,   /* U+D7DC */
  0x0000,   /* U+D7DD */
  0x0000,   /* U+D7DE */
  0x0000,   /* U+D7DF */
  0x0000,   /* U+D7E0 */
  0x0000,   /* U+D7E1 */
  0x0000,   /* U+D7E2 */
  0x0000,   /* U+D7E3 */
  0x0000,   /* U+D7E4 */
  0x0000,   /* U+D7E5 */
  0x0000,   /* U+D7E6 */
  0x0000,   /* U+D7E7 */
  0x0000,   /* U+D7E8 */
  0x0000,   /* U+D7E9 */
  0x0000,   /* U+D7EA */
  0x0000,   /* U+D7EB */
  0x0000,   /* U+D7EC */
  0x0000,   /* U+D7ED */
  0x0000,   /* U+D7EE */
  0x0000,   /* U+D7EF */
  0x0000,   /* U+D7F0 */
  0x0000,   /* U+D7F1 */
  0x0000,   /* U+D7F2 */
  0x0000,   /* U+D7F3 */
  0x0000,   /* U+D7F4 */
  0x0000,   /* U+D7F5 */
  0x0000,   /* U+D7F6 */
  0x0000,   /* U+D7F7 */
  0x0000,   /* U+D7F8 */
  0x0000,   /* U+D7F9 */
  0x0000,   /* U+D7FA */
  0x0000,   /* U+D7FB */
  0x0000,   /* U+D7FC */
  0x0000,   /* U+D7FD */
  0x0000,   /* U+D7FE */
  0x0000,   /* U+D7FF */
};

uint16 uca900_p0F9[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB41,   /* U+F900 */
  0xFB40,   /* U+F901 */
  0xFB41,   /* U+F902 */
  0xFB41,   /* U+F903 */
  0xFB40,   /* U+F904 */
  0xFB40,   /* U+F905 */
  0xFB40,   /* U+F906 */
  0xFB41,   /* U+F907 */
  0xFB41,   /* U+F908 */
  0xFB40,   /* U+F909 */
  0xFB41,   /* U+F90A */
  0xFB40,   /* U+F90B */
  0xFB40,   /* U+F90C */
  0xFB40,   /* U+F90D */
  0xFB40,   /* U+F90E */
  0xFB40,   /* U+F90F */
  0xFB41,   /* U+F910 */
  0xFB41,   /* U+F911 */
  0xFB41,   /* U+F912 */
  0xFB41,   /* U+F913 */
  0xFB40,   /* U+F914 */
  0xFB40,   /* U+F915 */
  0xFB40,   /* U+F916 */
  0xFB40,   /* U+F917 */
  0xFB41,   /* U+F918 */
  0xFB41,   /* U+F919 */
  0xFB41,   /* U+F91A */
  0xFB40,   /* U+F91B */
  0xFB40,   /* U+F91C */
  0xFB40,   /* U+F91D */
  0xFB40,   /* U+F91E */
  0xFB41,   /* U+F91F */
  0xFB41,   /* U+F920 */
  0xFB40,   /* U+F921 */
  0xFB40,   /* U+F922 */
  0xFB41,   /* U+F923 */
  0xFB41,   /* U+F924 */
  0xFB40,   /* U+F925 */
  0xFB41,   /* U+F926 */
  0xFB41,   /* U+F927 */
  0xFB40,   /* U+F928 */
  0xFB40,   /* U+F929 */
  0xFB40,   /* U+F92A */
  0xFB40,   /* U+F92B */
  0xFB41,   /* U+F92C */
  0xFB40,   /* U+F92D */
  0xFB40,   /* U+F92E */
  0xFB40,   /* U+F92F */
  0xFB40,   /* U+F930 */
  0xFB40,   /* U+F931 */
  0xFB40,   /* U+F932 */
  0xFB40,   /* U+F933 */
  0xFB41,   /* U+F934 */
  0xFB41,   /* U+F935 */
  0xFB41,   /* U+F936 */
  0xFB41,   /* U+F937 */
  0xFB41,   /* U+F938 */
  0xFB41,   /* U+F939 */
  0xFB41,   /* U+F93A */
  0xFB40,   /* U+F93B */
  0xFB40,   /* U+F93C */
  0xFB40,   /* U+F93D */
  0xFB41,   /* U+F93E */
  0xFB41,   /* U+F93F */
  0xFB41,   /* U+F940 */
  0xFB41,   /* U+F941 */
  0xFB40,   /* U+F942 */
  0xFB40,   /* U+F943 */
  0xFB40,   /* U+F944 */
  0xFB41,   /* U+F945 */
  0xFB40,   /* U+F946 */
  0xFB40,   /* U+F947 */
  0xFB41,   /* U+F948 */
  0xFB41,   /* U+F949 */
  0xFB40,   /* U+F94A */
  0xFB40,   /* U+F94B */
  0xFB40,   /* U+F94C */
  0xFB40,   /* U+F94D */
  0xFB40,   /* U+F94E */
  0xFB40,   /* U+F94F */
  0xFB40,   /* U+F950 */
  0xFB41,   /* U+F951 */
  0xFB40,   /* U+F952 */
  0xFB41,   /* U+F953 */
  0xFB40,   /* U+F954 */
  0xFB40,   /* U+F955 */
  0xFB40,   /* U+F956 */
  0xFB40,   /* U+F957 */
  0xFB41,   /* U+F958 */
  0xFB41,   /* U+F959 */
  0xFB41,   /* U+F95A */
  0xFB40,   /* U+F95B */
  0xFB40,   /* U+F95C */
  0xFB41,   /* U+F95D */
  0xFB40,   /* U+F95E */
  0xFB40,   /* U+F95F */
  0xFB40,   /* U+F960 */
  0xFB40,   /* U+F961 */
  0xFB40,   /* U+F962 */
  0xFB40,   /* U+F963 */
  0xFB40,   /* U+F964 */
  0xFB40,   /* U+F965 */
  0xFB40,   /* U+F966 */
  0xFB40,   /* U+F967 */
  0xFB40,   /* U+F968 */
  0xFB40,   /* U+F969 */
  0xFB40,   /* U+F96A */
  0xFB40,   /* U+F96B */
  0xFB40,   /* U+F96C */
  0xFB40,   /* U+F96D */
  0xFB41,   /* U+F96E */
  0xFB41,   /* U+F96F */
  0xFB40,   /* U+F970 */
  0xFB41,   /* U+F971 */
  0xFB40,   /* U+F972 */
  0xFB40,   /* U+F973 */
  0xFB41,   /* U+F974 */
  0xFB40,   /* U+F975 */
  0xFB40,   /* U+F976 */
  0xFB40,   /* U+F977 */
  0xFB40,   /* U+F978 */
  0xFB40,   /* U+F979 */
  0xFB40,   /* U+F97A */
  0xFB40,   /* U+F97B */
  0xFB41,   /* U+F97C */
  0xFB41,   /* U+F97D */
  0xFB41,   /* U+F97E */
  0xFB40,   /* U+F97F */
  0xFB40,   /* U+F980 */
  0xFB40,   /* U+F981 */
  0xFB40,   /* U+F982 */
  0xFB40,   /* U+F983 */
  0xFB40,   /* U+F984 */
  0xFB40,   /* U+F985 */
  0xFB41,   /* U+F986 */
  0xFB41,   /* U+F987 */
  0xFB41,   /* U+F988 */
  0xFB41,   /* U+F989 */
  0xFB40,   /* U+F98A */
  0xFB40,   /* U+F98B */
  0xFB40,   /* U+F98C */
  0xFB41,   /* U+F98D */
  0xFB40,   /* U+F98E */
  0xFB40,   /* U+F98F */
  0xFB40,   /* U+F990 */
  0xFB40,   /* U+F991 */
  0xFB40,   /* U+F992 */
  0xFB40,   /* U+F993 */
  0xFB40,   /* U+F994 */
  0xFB40,   /* U+F995 */
  0xFB40,   /* U+F996 */
  0xFB41,   /* U+F997 */
  0xFB41,   /* U+F998 */
  0xFB41,   /* U+F999 */
  0xFB41,   /* U+F99A */
  0xFB41,   /* U+F99B */
  0xFB40,   /* U+F99C */
  0xFB40,   /* U+F99D */
  0xFB40,   /* U+F99E */
  0xFB40,   /* U+F99F */
  0xFB41,   /* U+F9A0 */
  0xFB41,   /* U+F9A1 */
  0xFB40,   /* U+F9A2 */
  0xFB40,   /* U+F9A3 */
  0xFB40,   /* U+F9A4 */
  0xFB40,   /* U+F9A5 */
  0xFB40,   /* U+F9A6 */
  0xFB40,   /* U+F9A7 */
  0xFB40,   /* U+F9A8 */
  0xFB40,   /* U+F9A9 */
  0xFB40,   /* U+F9AA */
  0xFB40,   /* U+F9AB */
  0xFB40,   /* U+F9AC */
  0xFB40,   /* U+F9AD */
  0xFB40,   /* U+F9AE */
  0xFB40,   /* U+F9AF */
  0xFB41,   /* U+F9B0 */
  0xFB41,   /* U+F9B1 */
  0xFB41,   /* U+F9B2 */
  0xFB41,   /* U+F9B3 */
  0xFB41,   /* U+F9B4 */
  0xFB40,   /* U+F9B5 */
  0xFB40,   /* U+F9B6 */
  0xFB41,   /* U+F9B7 */
  0xFB41,   /* U+F9B8 */
  0xFB40,   /* U+F9B9 */
  0xFB40,   /* U+F9BA */
  0xFB40,   /* U+F9BB */
  0xFB40,   /* U+F9BC */
  0xFB40,   /* U+F9BD */
  0xFB40,   /* U+F9BE */
  0xFB40,   /* U+F9BF */
  0xFB40,   /* U+F9C0 */
  0xFB40,   /* U+F9C1 */
  0xFB41,   /* U+F9C2 */
  0xFB41,   /* U+F9C3 */
  0xFB41,   /* U+F9C4 */
  0xFB40,   /* U+F9C5 */
  0xFB41,   /* U+F9C6 */
  0xFB40,   /* U+F9C7 */
  0xFB40,   /* U+F9C8 */
  0xFB40,   /* U+F9C9 */
  0xFB40,   /* U+F9CA */
  0xFB40,   /* U+F9CB */
  0xFB40,   /* U+F9CC */
  0xFB40,   /* U+F9CD */
  0xFB40,   /* U+F9CE */
  0xFB40,   /* U+F9CF */
  0xFB41,   /* U+F9D0 */
  0xFB40,   /* U+F9D1 */
  0xFB40,   /* U+F9D2 */
  0xFB41,   /* U+F9D3 */
  0xFB40,   /* U+F9D4 */
  0xFB40,   /* U+F9D5 */
  0xFB40,   /* U+F9D6 */
  0xFB41,   /* U+F9D7 */
  0xFB40,   /* U+F9D8 */
  0xFB40,   /* U+F9D9 */
  0xFB40,   /* U+F9DA */
  0xFB40,   /* U+F9DB */
  0xFB41,   /* U+F9DC */
  0xFB40,   /* U+F9DD */
  0xFB40,   /* U+F9DE */
  0xFB40,   /* U+F9DF */
  0xFB40,   /* U+F9E0 */
  0xFB40,   /* U+F9E1 */
  0xFB40,   /* U+F9E2 */
  0xFB40,   /* U+F9E3 */
  0xFB40,   /* U+F9E4 */
  0xFB40,   /* U+F9E5 */
  0xFB40,   /* U+F9E6 */
  0xFB41,   /* U+F9E7 */
  0xFB41,   /* U+F9E8 */
  0xFB41,   /* U+F9E9 */
  0xFB41,   /* U+F9EA */
  0xFB40,   /* U+F9EB */
  0xFB40,   /* U+F9EC */
  0xFB40,   /* U+F9ED */
  0xFB40,   /* U+F9EE */
  0xFB40,   /* U+F9EF */
  0xFB41,   /* U+F9F0 */
  0xFB41,   /* U+F9F1 */
  0xFB41,   /* U+F9F2 */
  0xFB41,   /* U+F9F3 */
  0xFB40,   /* U+F9F4 */
  0xFB40,   /* U+F9F5 */
  0xFB41,   /* U+F9F6 */
  0xFB40,   /* U+F9F7 */
  0xFB40,   /* U+F9F8 */
  0xFB40,   /* U+F9F9 */
  0xFB40,   /* U+F9FA */
  0xFB40,   /* U+F9FB */
  0xFB41,   /* U+F9FC */
  0xFB40,   /* U+F9FD */
  0xFB41,   /* U+F9FE */
  0xFB40,   /* U+F9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+F900 */
  0x0020,   /* U+F901 */
  0x0020,   /* U+F902 */
  0x0020,   /* U+F903 */
  0x0020,   /* U+F904 */
  0x0020,   /* U+F905 */
  0x0020,   /* U+F906 */
  0x0020,   /* U+F907 */
  0x0020,   /* U+F908 */
  0x0020,   /* U+F909 */
  0x0020,   /* U+F90A */
  0x0020,   /* U+F90B */
  0x0020,   /* U+F90C */
  0x0020,   /* U+F90D */
  0x0020,   /* U+F90E */
  0x0020,   /* U+F90F */
  0x0020,   /* U+F910 */
  0x0020,   /* U+F911 */
  0x0020,   /* U+F912 */
  0x0020,   /* U+F913 */
  0x0020,   /* U+F914 */
  0x0020,   /* U+F915 */
  0x0020,   /* U+F916 */
  0x0020,   /* U+F917 */
  0x0020,   /* U+F918 */
  0x0020,   /* U+F919 */
  0x0020,   /* U+F91A */
  0x0020,   /* U+F91B */
  0x0020,   /* U+F91C */
  0x0020,   /* U+F91D */
  0x0020,   /* U+F91E */
  0x0020,   /* U+F91F */
  0x0020,   /* U+F920 */
  0x0020,   /* U+F921 */
  0x0020,   /* U+F922 */
  0x0020,   /* U+F923 */
  0x0020,   /* U+F924 */
  0x0020,   /* U+F925 */
  0x0020,   /* U+F926 */
  0x0020,   /* U+F927 */
  0x0020,   /* U+F928 */
  0x0020,   /* U+F929 */
  0x0020,   /* U+F92A */
  0x0020,   /* U+F92B */
  0x0020,   /* U+F92C */
  0x0020,   /* U+F92D */
  0x0020,   /* U+F92E */
  0x0020,   /* U+F92F */
  0x0020,   /* U+F930 */
  0x0020,   /* U+F931 */
  0x0020,   /* U+F932 */
  0x0020,   /* U+F933 */
  0x0020,   /* U+F934 */
  0x0020,   /* U+F935 */
  0x0020,   /* U+F936 */
  0x0020,   /* U+F937 */
  0x0020,   /* U+F938 */
  0x0020,   /* U+F939 */
  0x0020,   /* U+F93A */
  0x0020,   /* U+F93B */
  0x0020,   /* U+F93C */
  0x0020,   /* U+F93D */
  0x0020,   /* U+F93E */
  0x0020,   /* U+F93F */
  0x0020,   /* U+F940 */
  0x0020,   /* U+F941 */
  0x0020,   /* U+F942 */
  0x0020,   /* U+F943 */
  0x0020,   /* U+F944 */
  0x0020,   /* U+F945 */
  0x0020,   /* U+F946 */
  0x0020,   /* U+F947 */
  0x0020,   /* U+F948 */
  0x0020,   /* U+F949 */
  0x0020,   /* U+F94A */
  0x0020,   /* U+F94B */
  0x0020,   /* U+F94C */
  0x0020,   /* U+F94D */
  0x0020,   /* U+F94E */
  0x0020,   /* U+F94F */
  0x0020,   /* U+F950 */
  0x0020,   /* U+F951 */
  0x0020,   /* U+F952 */
  0x0020,   /* U+F953 */
  0x0020,   /* U+F954 */
  0x0020,   /* U+F955 */
  0x0020,   /* U+F956 */
  0x0020,   /* U+F957 */
  0x0020,   /* U+F958 */
  0x0020,   /* U+F959 */
  0x0020,   /* U+F95A */
  0x0020,   /* U+F95B */
  0x0020,   /* U+F95C */
  0x0020,   /* U+F95D */
  0x0020,   /* U+F95E */
  0x0020,   /* U+F95F */
  0x0020,   /* U+F960 */
  0x0020,   /* U+F961 */
  0x0020,   /* U+F962 */
  0x0020,   /* U+F963 */
  0x0020,   /* U+F964 */
  0x0020,   /* U+F965 */
  0x0020,   /* U+F966 */
  0x0020,   /* U+F967 */
  0x0020,   /* U+F968 */
  0x0020,   /* U+F969 */
  0x0020,   /* U+F96A */
  0x0020,   /* U+F96B */
  0x0020,   /* U+F96C */
  0x0020,   /* U+F96D */
  0x0020,   /* U+F96E */
  0x0020,   /* U+F96F */
  0x0020,   /* U+F970 */
  0x0020,   /* U+F971 */
  0x0020,   /* U+F972 */
  0x0020,   /* U+F973 */
  0x0020,   /* U+F974 */
  0x0020,   /* U+F975 */
  0x0020,   /* U+F976 */
  0x0020,   /* U+F977 */
  0x0020,   /* U+F978 */
  0x0020,   /* U+F979 */
  0x0020,   /* U+F97A */
  0x0020,   /* U+F97B */
  0x0020,   /* U+F97C */
  0x0020,   /* U+F97D */
  0x0020,   /* U+F97E */
  0x0020,   /* U+F97F */
  0x0020,   /* U+F980 */
  0x0020,   /* U+F981 */
  0x0020,   /* U+F982 */
  0x0020,   /* U+F983 */
  0x0020,   /* U+F984 */
  0x0020,   /* U+F985 */
  0x0020,   /* U+F986 */
  0x0020,   /* U+F987 */
  0x0020,   /* U+F988 */
  0x0020,   /* U+F989 */
  0x0020,   /* U+F98A */
  0x0020,   /* U+F98B */
  0x0020,   /* U+F98C */
  0x0020,   /* U+F98D */
  0x0020,   /* U+F98E */
  0x0020,   /* U+F98F */
  0x0020,   /* U+F990 */
  0x0020,   /* U+F991 */
  0x0020,   /* U+F992 */
  0x0020,   /* U+F993 */
  0x0020,   /* U+F994 */
  0x0020,   /* U+F995 */
  0x0020,   /* U+F996 */
  0x0020,   /* U+F997 */
  0x0020,   /* U+F998 */
  0x0020,   /* U+F999 */
  0x0020,   /* U+F99A */
  0x0020,   /* U+F99B */
  0x0020,   /* U+F99C */
  0x0020,   /* U+F99D */
  0x0020,   /* U+F99E */
  0x0020,   /* U+F99F */
  0x0020,   /* U+F9A0 */
  0x0020,   /* U+F9A1 */
  0x0020,   /* U+F9A2 */
  0x0020,   /* U+F9A3 */
  0x0020,   /* U+F9A4 */
  0x0020,   /* U+F9A5 */
  0x0020,   /* U+F9A6 */
  0x0020,   /* U+F9A7 */
  0x0020,   /* U+F9A8 */
  0x0020,   /* U+F9A9 */
  0x0020,   /* U+F9AA */
  0x0020,   /* U+F9AB */
  0x0020,   /* U+F9AC */
  0x0020,   /* U+F9AD */
  0x0020,   /* U+F9AE */
  0x0020,   /* U+F9AF */
  0x0020,   /* U+F9B0 */
  0x0020,   /* U+F9B1 */
  0x0020,   /* U+F9B2 */
  0x0020,   /* U+F9B3 */
  0x0020,   /* U+F9B4 */
  0x0020,   /* U+F9B5 */
  0x0020,   /* U+F9B6 */
  0x0020,   /* U+F9B7 */
  0x0020,   /* U+F9B8 */
  0x0020,   /* U+F9B9 */
  0x0020,   /* U+F9BA */
  0x0020,   /* U+F9BB */
  0x0020,   /* U+F9BC */
  0x0020,   /* U+F9BD */
  0x0020,   /* U+F9BE */
  0x0020,   /* U+F9BF */
  0x0020,   /* U+F9C0 */
  0x0020,   /* U+F9C1 */
  0x0020,   /* U+F9C2 */
  0x0020,   /* U+F9C3 */
  0x0020,   /* U+F9C4 */
  0x0020,   /* U+F9C5 */
  0x0020,   /* U+F9C6 */
  0x0020,   /* U+F9C7 */
  0x0020,   /* U+F9C8 */
  0x0020,   /* U+F9C9 */
  0x0020,   /* U+F9CA */
  0x0020,   /* U+F9CB */
  0x0020,   /* U+F9CC */
  0x0020,   /* U+F9CD */
  0x0020,   /* U+F9CE */
  0x0020,   /* U+F9CF */
  0x0020,   /* U+F9D0 */
  0x0020,   /* U+F9D1 */
  0x0020,   /* U+F9D2 */
  0x0020,   /* U+F9D3 */
  0x0020,   /* U+F9D4 */
  0x0020,   /* U+F9D5 */
  0x0020,   /* U+F9D6 */
  0x0020,   /* U+F9D7 */
  0x0020,   /* U+F9D8 */
  0x0020,   /* U+F9D9 */
  0x0020,   /* U+F9DA */
  0x0020,   /* U+F9DB */
  0x0020,   /* U+F9DC */
  0x0020,   /* U+F9DD */
  0x0020,   /* U+F9DE */
  0x0020,   /* U+F9DF */
  0x0020,   /* U+F9E0 */
  0x0020,   /* U+F9E1 */
  0x0020,   /* U+F9E2 */
  0x0020,   /* U+F9E3 */
  0x0020,   /* U+F9E4 */
  0x0020,   /* U+F9E5 */
  0x0020,   /* U+F9E6 */
  0x0020,   /* U+F9E7 */
  0x0020,   /* U+F9E8 */
  0x0020,   /* U+F9E9 */
  0x0020,   /* U+F9EA */
  0x0020,   /* U+F9EB */
  0x0020,   /* U+F9EC */
  0x0020,   /* U+F9ED */
  0x0020,   /* U+F9EE */
  0x0020,   /* U+F9EF */
  0x0020,   /* U+F9F0 */
  0x0020,   /* U+F9F1 */
  0x0020,   /* U+F9F2 */
  0x0020,   /* U+F9F3 */
  0x0020,   /* U+F9F4 */
  0x0020,   /* U+F9F5 */
  0x0020,   /* U+F9F6 */
  0x0020,   /* U+F9F7 */
  0x0020,   /* U+F9F8 */
  0x0020,   /* U+F9F9 */
  0x0020,   /* U+F9FA */
  0x0020,   /* U+F9FB */
  0x0020,   /* U+F9FC */
  0x0020,   /* U+F9FD */
  0x0020,   /* U+F9FE */
  0x0020,   /* U+F9FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+F900 */
  0x0002,   /* U+F901 */
  0x0002,   /* U+F902 */
  0x0002,   /* U+F903 */
  0x0002,   /* U+F904 */
  0x0002,   /* U+F905 */
  0x0002,   /* U+F906 */
  0x0002,   /* U+F907 */
  0x0002,   /* U+F908 */
  0x0002,   /* U+F909 */
  0x0002,   /* U+F90A */
  0x0002,   /* U+F90B */
  0x0002,   /* U+F90C */
  0x0002,   /* U+F90D */
  0x0002,   /* U+F90E */
  0x0002,   /* U+F90F */
  0x0002,   /* U+F910 */
  0x0002,   /* U+F911 */
  0x0002,   /* U+F912 */
  0x0002,   /* U+F913 */
  0x0002,   /* U+F914 */
  0x0002,   /* U+F915 */
  0x0002,   /* U+F916 */
  0x0002,   /* U+F917 */
  0x0002,   /* U+F918 */
  0x0002,   /* U+F919 */
  0x0002,   /* U+F91A */
  0x0002,   /* U+F91B */
  0x0002,   /* U+F91C */
  0x0002,   /* U+F91D */
  0x0002,   /* U+F91E */
  0x0002,   /* U+F91F */
  0x0002,   /* U+F920 */
  0x0002,   /* U+F921 */
  0x0002,   /* U+F922 */
  0x0002,   /* U+F923 */
  0x0002,   /* U+F924 */
  0x0002,   /* U+F925 */
  0x0002,   /* U+F926 */
  0x0002,   /* U+F927 */
  0x0002,   /* U+F928 */
  0x0002,   /* U+F929 */
  0x0002,   /* U+F92A */
  0x0002,   /* U+F92B */
  0x0002,   /* U+F92C */
  0x0002,   /* U+F92D */
  0x0002,   /* U+F92E */
  0x0002,   /* U+F92F */
  0x0002,   /* U+F930 */
  0x0002,   /* U+F931 */
  0x0002,   /* U+F932 */
  0x0002,   /* U+F933 */
  0x0002,   /* U+F934 */
  0x0002,   /* U+F935 */
  0x0002,   /* U+F936 */
  0x0002,   /* U+F937 */
  0x0002,   /* U+F938 */
  0x0002,   /* U+F939 */
  0x0002,   /* U+F93A */
  0x0002,   /* U+F93B */
  0x0002,   /* U+F93C */
  0x0002,   /* U+F93D */
  0x0002,   /* U+F93E */
  0x0002,   /* U+F93F */
  0x0002,   /* U+F940 */
  0x0002,   /* U+F941 */
  0x0002,   /* U+F942 */
  0x0002,   /* U+F943 */
  0x0002,   /* U+F944 */
  0x0002,   /* U+F945 */
  0x0002,   /* U+F946 */
  0x0002,   /* U+F947 */
  0x0002,   /* U+F948 */
  0x0002,   /* U+F949 */
  0x0002,   /* U+F94A */
  0x0002,   /* U+F94B */
  0x0002,   /* U+F94C */
  0x0002,   /* U+F94D */
  0x0002,   /* U+F94E */
  0x0002,   /* U+F94F */
  0x0002,   /* U+F950 */
  0x0002,   /* U+F951 */
  0x0002,   /* U+F952 */
  0x0002,   /* U+F953 */
  0x0002,   /* U+F954 */
  0x0002,   /* U+F955 */
  0x0002,   /* U+F956 */
  0x0002,   /* U+F957 */
  0x0002,   /* U+F958 */
  0x0002,   /* U+F959 */
  0x0002,   /* U+F95A */
  0x0002,   /* U+F95B */
  0x0002,   /* U+F95C */
  0x0002,   /* U+F95D */
  0x0002,   /* U+F95E */
  0x0002,   /* U+F95F */
  0x0002,   /* U+F960 */
  0x0002,   /* U+F961 */
  0x0002,   /* U+F962 */
  0x0002,   /* U+F963 */
  0x0002,   /* U+F964 */
  0x0002,   /* U+F965 */
  0x0002,   /* U+F966 */
  0x0002,   /* U+F967 */
  0x0002,   /* U+F968 */
  0x0002,   /* U+F969 */
  0x0002,   /* U+F96A */
  0x0002,   /* U+F96B */
  0x0002,   /* U+F96C */
  0x0002,   /* U+F96D */
  0x0002,   /* U+F96E */
  0x0002,   /* U+F96F */
  0x0002,   /* U+F970 */
  0x0002,   /* U+F971 */
  0x0002,   /* U+F972 */
  0x0002,   /* U+F973 */
  0x0002,   /* U+F974 */
  0x0002,   /* U+F975 */
  0x0002,   /* U+F976 */
  0x0002,   /* U+F977 */
  0x0002,   /* U+F978 */
  0x0002,   /* U+F979 */
  0x0002,   /* U+F97A */
  0x0002,   /* U+F97B */
  0x0002,   /* U+F97C */
  0x0002,   /* U+F97D */
  0x0002,   /* U+F97E */
  0x0002,   /* U+F97F */
  0x0002,   /* U+F980 */
  0x0002,   /* U+F981 */
  0x0002,   /* U+F982 */
  0x0002,   /* U+F983 */
  0x0002,   /* U+F984 */
  0x0002,   /* U+F985 */
  0x0002,   /* U+F986 */
  0x0002,   /* U+F987 */
  0x0002,   /* U+F988 */
  0x0002,   /* U+F989 */
  0x0002,   /* U+F98A */
  0x0002,   /* U+F98B */
  0x0002,   /* U+F98C */
  0x0002,   /* U+F98D */
  0x0002,   /* U+F98E */
  0x0002,   /* U+F98F */
  0x0002,   /* U+F990 */
  0x0002,   /* U+F991 */
  0x0002,   /* U+F992 */
  0x0002,   /* U+F993 */
  0x0002,   /* U+F994 */
  0x0002,   /* U+F995 */
  0x0002,   /* U+F996 */
  0x0002,   /* U+F997 */
  0x0002,   /* U+F998 */
  0x0002,   /* U+F999 */
  0x0002,   /* U+F99A */
  0x0002,   /* U+F99B */
  0x0002,   /* U+F99C */
  0x0002,   /* U+F99D */
  0x0002,   /* U+F99E */
  0x0002,   /* U+F99F */
  0x0002,   /* U+F9A0 */
  0x0002,   /* U+F9A1 */
  0x0002,   /* U+F9A2 */
  0x0002,   /* U+F9A3 */
  0x0002,   /* U+F9A4 */
  0x0002,   /* U+F9A5 */
  0x0002,   /* U+F9A6 */
  0x0002,   /* U+F9A7 */
  0x0002,   /* U+F9A8 */
  0x0002,   /* U+F9A9 */
  0x0002,   /* U+F9AA */
  0x0002,   /* U+F9AB */
  0x0002,   /* U+F9AC */
  0x0002,   /* U+F9AD */
  0x0002,   /* U+F9AE */
  0x0002,   /* U+F9AF */
  0x0002,   /* U+F9B0 */
  0x0002,   /* U+F9B1 */
  0x0002,   /* U+F9B2 */
  0x0002,   /* U+F9B3 */
  0x0002,   /* U+F9B4 */
  0x0002,   /* U+F9B5 */
  0x0002,   /* U+F9B6 */
  0x0002,   /* U+F9B7 */
  0x0002,   /* U+F9B8 */
  0x0002,   /* U+F9B9 */
  0x0002,   /* U+F9BA */
  0x0002,   /* U+F9BB */
  0x0002,   /* U+F9BC */
  0x0002,   /* U+F9BD */
  0x0002,   /* U+F9BE */
  0x0002,   /* U+F9BF */
  0x0002,   /* U+F9C0 */
  0x0002,   /* U+F9C1 */
  0x0002,   /* U+F9C2 */
  0x0002,   /* U+F9C3 */
  0x0002,   /* U+F9C4 */
  0x0002,   /* U+F9C5 */
  0x0002,   /* U+F9C6 */
  0x0002,   /* U+F9C7 */
  0x0002,   /* U+F9C8 */
  0x0002,   /* U+F9C9 */
  0x0002,   /* U+F9CA */
  0x0002,   /* U+F9CB */
  0x0002,   /* U+F9CC */
  0x0002,   /* U+F9CD */
  0x0002,   /* U+F9CE */
  0x0002,   /* U+F9CF */
  0x0002,   /* U+F9D0 */
  0x0002,   /* U+F9D1 */
  0x0002,   /* U+F9D2 */
  0x0002,   /* U+F9D3 */
  0x0002,   /* U+F9D4 */
  0x0002,   /* U+F9D5 */
  0x0002,   /* U+F9D6 */
  0x0002,   /* U+F9D7 */
  0x0002,   /* U+F9D8 */
  0x0002,   /* U+F9D9 */
  0x0002,   /* U+F9DA */
  0x0002,   /* U+F9DB */
  0x0002,   /* U+F9DC */
  0x0002,   /* U+F9DD */
  0x0002,   /* U+F9DE */
  0x0002,   /* U+F9DF */
  0x0002,   /* U+F9E0 */
  0x0002,   /* U+F9E1 */
  0x0002,   /* U+F9E2 */
  0x0002,   /* U+F9E3 */
  0x0002,   /* U+F9E4 */
  0x0002,   /* U+F9E5 */
  0x0002,   /* U+F9E6 */
  0x0002,   /* U+F9E7 */
  0x0002,   /* U+F9E8 */
  0x0002,   /* U+F9E9 */
  0x0002,   /* U+F9EA */
  0x0002,   /* U+F9EB */
  0x0002,   /* U+F9EC */
  0x0002,   /* U+F9ED */
  0x0002,   /* U+F9EE */
  0x0002,   /* U+F9EF */
  0x0002,   /* U+F9F0 */
  0x0002,   /* U+F9F1 */
  0x0002,   /* U+F9F2 */
  0x0002,   /* U+F9F3 */
  0x0002,   /* U+F9F4 */
  0x0002,   /* U+F9F5 */
  0x0002,   /* U+F9F6 */
  0x0002,   /* U+F9F7 */
  0x0002,   /* U+F9F8 */
  0x0002,   /* U+F9F9 */
  0x0002,   /* U+F9FA */
  0x0002,   /* U+F9FB */
  0x0002,   /* U+F9FC */
  0x0002,   /* U+F9FD */
  0x0002,   /* U+F9FE */
  0x0002,   /* U+F9FF */

  /* Primary weight 2 for each character. */
  0x8C48,   /* U+F900 */
  0xE6F4,   /* U+F901 */
  0x8ECA,   /* U+F902 */
  0x8CC8,   /* U+F903 */
  0xEED1,   /* U+F904 */
  0xCE32,   /* U+F905 */
  0xD3E5,   /* U+F906 */
  0x9F9C,   /* U+F907 */
  0x9F9C,   /* U+F908 */
  0xD951,   /* U+F909 */
  0x91D1,   /* U+F90A */
  0xD587,   /* U+F90B */
  0xD948,   /* U+F90C */
  0xE1F6,   /* U+F90D */
  0xF669,   /* U+F90E */
  0xFF85,   /* U+F90F */
  0x863F,   /* U+F910 */
  0x87BA,   /* U+F911 */
  0x88F8,   /* U+F912 */
  0x908F,   /* U+F913 */
  0xEA02,   /* U+F914 */
  0xED1B,   /* U+F915 */
  0xF0D9,   /* U+F916 */
  0xF3DE,   /* U+F917 */
  0x843D,   /* U+F918 */
  0x916A,   /* U+F919 */
  0x99F1,   /* U+F91A */
  0xCE82,   /* U+F91B */
  0xD375,   /* U+F91C */
  0xEB04,   /* U+F91D */
  0xF21B,   /* U+F91E */
  0x862D,   /* U+F91F */
  0x9E1E,   /* U+F920 */
  0xDD50,   /* U+F921 */
  0xEFEB,   /* U+F922 */
  0x85CD,   /* U+F923 */
  0x8964,   /* U+F924 */
  0xE2C9,   /* U+F925 */
  0x81D8,   /* U+F926 */
  0x881F,   /* U+F927 */
  0xDECA,   /* U+F928 */
  0xE717,   /* U+F929 */
  0xED6A,   /* U+F92A */
  0xF2FC,   /* U+F92B */
  0x90CE,   /* U+F92C */
  0xCF86,   /* U+F92D */
  0xD1B7,   /* U+F92E */
  0xD2DE,   /* U+F92F */
  0xE4C4,   /* U+F930 */
  0xEAD3,   /* U+F931 */
  0xF210,   /* U+F932 */
  0xF6E7,   /* U+F933 */
  0x8001,   /* U+F934 */
  0x8606,   /* U+F935 */
  0x865C,   /* U+F936 */
  0x8DEF,   /* U+F937 */
  0x9732,   /* U+F938 */
  0x9B6F,   /* U+F939 */
  0x9DFA,   /* U+F93A */
  0xF88C,   /* U+F93B */
  0xF97F,   /* U+F93C */
  0xFDA0,   /* U+F93D */
  0x83C9,   /* U+F93E */
  0x9304,   /* U+F93F */
  0x9E7F,   /* U+F940 */
  0x8AD6,   /* U+F941 */
  0xD8DF,   /* U+F942 */
  0xDF04,   /* U+F943 */
  0xFC60,   /* U+F944 */
  0x807E,   /* U+F945 */
  0xF262,   /* U+F946 */
  0xF8CA,   /* U+F947 */
  0x8CC2,   /* U+F948 */
  0x96F7,   /* U+F949 */
  0xD8D8,   /* U+F94A */
  0xDC62,   /* U+F94B */
  0xEA13,   /* U+F94C */
  0xEDDA,   /* U+F94D */
  0xEF0F,   /* U+F94E */
  0xFD2F,   /* U+F94F */
  0xFE37,   /* U+F950 */
  0x964B,   /* U+F951 */
  0xD2D2,   /* U+F952 */
  0x808B,   /* U+F953 */
  0xD1DC,   /* U+F954 */
  0xD1CC,   /* U+F955 */
  0xFA1C,   /* U+F956 */
  0xFDBE,   /* U+F957 */
  0x83F1,   /* U+F958 */
  0x9675,   /* U+F959 */
  0x8B80,   /* U+F95A */
  0xE2CF,   /* U+F95B */
  0xEA02,   /* U+F95C */
  0x8AFE,   /* U+F95D */
  0xCE39,   /* U+F95E */
  0xDBE7,   /* U+F95F */
  0xE012,   /* U+F960 */
  0xF387,   /* U+F961 */
  0xF570,   /* U+F962 */
  0xD317,   /* U+F963 */
  0xF8FB,   /* U+F964 */
  0xCFBF,   /* U+F965 */
  0xDFA9,   /* U+F966 */
  0xCE0D,   /* U+F967 */
  0xECCC,   /* U+F968 */
  0xE578,   /* U+F969 */
  0xFD22,   /* U+F96A */
  0xD3C3,   /* U+F96B */
  0xD85E,   /* U+F96C */
  0xF701,   /* U+F96D */
  0x8449,   /* U+F96E */
  0x8AAA,   /* U+F96F */
  0xEBBA,   /* U+F970 */
  0x8FB0,   /* U+F971 */
  0xEC88,   /* U+F972 */
  0xE2FE,   /* U+F973 */
  0x82E5,   /* U+F974 */
  0xE3A0,   /* U+F975 */
  0xF565,   /* U+F976 */
  0xCEAE,   /* U+F977 */
  0xD169,   /* U+F978 */
  0xD1C9,   /* U+F979 */
  0xE881,   /* U+F97A */
  0xFCE7,   /* U+F97B */
  0x826F,   /* U+F97C */
  0x8AD2,   /* U+F97D */
  0x91CF,   /* U+F97E */
  0xD2F5,   /* U+F97F */
  0xD442,   /* U+F980 */
  0xD973,   /* U+F981 */
  0xDEEC,   /* U+F982 */
  0xE5C5,   /* U+F983 */
  0xEFFE,   /* U+F984 */
  0xF92A,   /* U+F985 */
  0x95AD,   /* U+F986 */
  0x9A6A,   /* U+F987 */
  0x9E97,   /* U+F988 */
  0x9ECE,   /* U+F989 */
  0xD29B,   /* U+F98A */
  0xE6C6,   /* U+F98B */
  0xEB77,   /* U+F98C */
  0x8F62,   /* U+F98D */
  0xDE74,   /* U+F98E */
  0xE190,   /* U+F98F */
  0xE200,   /* U+F990 */
  0xE49A,   /* U+F991 */
  0xEF23,   /* U+F992 */
  0xF149,   /* U+F993 */
  0xF489,   /* U+F994 */
  0xF9CA,   /* U+F995 */
  0xFDF4,   /* U+F996 */
  0x806F,   /* U+F997 */
  0x8F26,   /* U+F998 */
  0x84EE,   /* U+F999 */
  0x9023,   /* U+F99A */
  0x934A,   /* U+F99B */
  0xD217,   /* U+F99C */
  0xD2A3,   /* U+F99D */
  0xD4BD,   /* U+F99E */
  0xF0C8,   /* U+F99F */
  0x88C2,   /* U+F9A0 */
  0x8AAA,   /* U+F9A1 */
  0xDEC9,   /* U+F9A2 */
  0xDFF5,   /* U+F9A3 */
  0xE37B,   /* U+F9A4 */
  0xEBAE,   /* U+F9A5 */
  0xFC3E,   /* U+F9A6 */
  0xF375,   /* U+F9A7 */
  0xCEE4,   /* U+F9A8 */
  0xD6F9,   /* U+F9A9 */
  0xDBE7,   /* U+F9AA */
  0xDDBA,   /* U+F9AB */
  0xE01C,   /* U+F9AC */
  0xF3B2,   /* U+F9AD */
  0xF469,   /* U+F9AE */
  0xFF9A,   /* U+F9AF */
  0x8046,   /* U+F9B0 */
  0x9234,   /* U+F9B1 */
  0x96F6,   /* U+F9B2 */
  0x9748,   /* U+F9B3 */
  0x9818,   /* U+F9B4 */
  0xCF8B,   /* U+F9B5 */
  0xF9AE,   /* U+F9B6 */
  0x91B4,   /* U+F9B7 */
  0x96B8,   /* U+F9B8 */
  0xE0E1,   /* U+F9B9 */
  0xCE86,   /* U+F9BA */
  0xD0DA,   /* U+F9BB */
  0xDBEE,   /* U+F9BC */
  0xDC3F,   /* U+F9BD */
  0xE599,   /* U+F9BE */
  0xEA02,   /* U+F9BF */
  0xF1CE,   /* U+F9C0 */
  0xF642,   /* U+F9C1 */
  0x84FC,   /* U+F9C2 */
  0x907C,   /* U+F9C3 */
  0x9F8D,   /* U+F9C4 */
  0xE688,   /* U+F9C5 */
  0x962E,   /* U+F9C6 */
  0xD289,   /* U+F9C7 */
  0xE77B,   /* U+F9C8 */
  0xE7F3,   /* U+F9C9 */
  0xED41,   /* U+F9CA */
  0xEE9C,   /* U+F9CB */
  0xF409,   /* U+F9CC */
  0xF559,   /* U+F9CD */
  0xF86B,   /* U+F9CE */
  0xFD10,   /* U+F9CF */
  0x985E,   /* U+F9D0 */
  0xD16D,   /* U+F9D1 */
  0xE22E,   /* U+F9D2 */
  0x9678,   /* U+F9D3 */
  0xD02B,   /* U+F9D4 */
  0xDD19,   /* U+F9D5 */
  0xEDEA,   /* U+F9D6 */
  0x8F2A,   /* U+F9D7 */
  0xDF8B,   /* U+F9D8 */
  0xE144,   /* U+F9D9 */
  0xE817,   /* U+F9DA */
  0xF387,   /* U+F9DB */
  0x9686,   /* U+F9DC */
  0xD229,   /* U+F9DD */
  0xD40F,   /* U+F9DE */
  0xDC65,   /* U+F9DF */
  0xE613,   /* U+F9E0 */
  0xE74E,   /* U+F9E1 */
  0xE8A8,   /* U+F9E2 */
  0xECE5,   /* U+F9E3 */
  0xF406,   /* U+F9E4 */
  0xF5E2,   /* U+F9E5 */
  0xFF79,   /* U+F9E6 */
  0x88CF,   /* U+F9E7 */
  0x88E1,   /* U+F9E8 */
  0x91CC,   /* U+F9E9 */
  0x96E2,   /* U+F9EA */
  0xD33F,   /* U+F9EB */
  0xEEBA,   /* U+F9EC */
  0xD41D,   /* U+F9ED */
  0xF1D0,   /* U+F9EE */
  0xF498,   /* U+F9EF */
  0x85FA,   /* U+F9F0 */
  0x96A3,   /* U+F9F1 */
  0x9C57,   /* U+F9F2 */
  0x9E9F,   /* U+F9F3 */
  0xE797,   /* U+F9F4 */
  0xEDCB,   /* U+F9F5 */
  0x81E8,   /* U+F9F6 */
  0xFACB,   /* U+F9F7 */
  0xFB20,   /* U+F9F8 */
  0xFC92,   /* U+F9F9 */
  0xF2C0,   /* U+F9FA */
  0xF099,   /* U+F9FB */
  0x8B58,   /* U+F9FC */
  0xCEC0,   /* U+F9FD */
  0x8336,   /* U+F9FE */
  0xD23A,   /* U+F9FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+F900 */
  0x0000,   /* U+F901 */
  0x0000,   /* U+F902 */
  0x0000,   /* U+F903 */
  0x0000,   /* U+F904 */
  0x0000,   /* U+F905 */
  0x0000,   /* U+F906 */
  0x0000,   /* U+F907 */
  0x0000,   /* U+F908 */
  0x0000,   /* U+F909 */
  0x0000,   /* U+F90A */
  0x0000,   /* U+F90B */
  0x0000,   /* U+F90C */
  0x0000,   /* U+F90D */
  0x0000,   /* U+F90E */
  0x0000,   /* U+F90F */
  0x0000,   /* U+F910 */
  0x0000,   /* U+F911 */
  0x0000,   /* U+F912 */
  0x0000,   /* U+F913 */
  0x0000,   /* U+F914 */
  0x0000,   /* U+F915 */
  0x0000,   /* U+F916 */
  0x0000,   /* U+F917 */
  0x0000,   /* U+F918 */
  0x0000,   /* U+F919 */
  0x0000,   /* U+F91A */
  0x0000,   /* U+F91B */
  0x0000,   /* U+F91C */
  0x0000,   /* U+F91D */
  0x0000,   /* U+F91E */
  0x0000,   /* U+F91F */
  0x0000,   /* U+F920 */
  0x0000,   /* U+F921 */
  0x0000,   /* U+F922 */
  0x0000,   /* U+F923 */
  0x0000,   /* U+F924 */
  0x0000,   /* U+F925 */
  0x0000,   /* U+F926 */
  0x0000,   /* U+F927 */
  0x0000,   /* U+F928 */
  0x0000,   /* U+F929 */
  0x0000,   /* U+F92A */
  0x0000,   /* U+F92B */
  0x0000,   /* U+F92C */
  0x0000,   /* U+F92D */
  0x0000,   /* U+F92E */
  0x0000,   /* U+F92F */
  0x0000,   /* U+F930 */
  0x0000,   /* U+F931 */
  0x0000,   /* U+F932 */
  0x0000,   /* U+F933 */
  0x0000,   /* U+F934 */
  0x0000,   /* U+F935 */
  0x0000,   /* U+F936 */
  0x0000,   /* U+F937 */
  0x0000,   /* U+F938 */
  0x0000,   /* U+F939 */
  0x0000,   /* U+F93A */
  0x0000,   /* U+F93B */
  0x0000,   /* U+F93C */
  0x0000,   /* U+F93D */
  0x0000,   /* U+F93E */
  0x0000,   /* U+F93F */
  0x0000,   /* U+F940 */
  0x0000,   /* U+F941 */
  0x0000,   /* U+F942 */
  0x0000,   /* U+F943 */
  0x0000,   /* U+F944 */
  0x0000,   /* U+F945 */
  0x0000,   /* U+F946 */
  0x0000,   /* U+F947 */
  0x0000,   /* U+F948 */
  0x0000,   /* U+F949 */
  0x0000,   /* U+F94A */
  0x0000,   /* U+F94B */
  0x0000,   /* U+F94C */
  0x0000,   /* U+F94D */
  0x0000,   /* U+F94E */
  0x0000,   /* U+F94F */
  0x0000,   /* U+F950 */
  0x0000,   /* U+F951 */
  0x0000,   /* U+F952 */
  0x0000,   /* U+F953 */
  0x0000,   /* U+F954 */
  0x0000,   /* U+F955 */
  0x0000,   /* U+F956 */
  0x0000,   /* U+F957 */
  0x0000,   /* U+F958 */
  0x0000,   /* U+F959 */
  0x0000,   /* U+F95A */
  0x0000,   /* U+F95B */
  0x0000,   /* U+F95C */
  0x0000,   /* U+F95D */
  0x0000,   /* U+F95E */
  0x0000,   /* U+F95F */
  0x0000,   /* U+F960 */
  0x0000,   /* U+F961 */
  0x0000,   /* U+F962 */
  0x0000,   /* U+F963 */
  0x0000,   /* U+F964 */
  0x0000,   /* U+F965 */
  0x0000,   /* U+F966 */
  0x0000,   /* U+F967 */
  0x0000,   /* U+F968 */
  0x0000,   /* U+F969 */
  0x0000,   /* U+F96A */
  0x0000,   /* U+F96B */
  0x0000,   /* U+F96C */
  0x0000,   /* U+F96D */
  0x0000,   /* U+F96E */
  0x0000,   /* U+F96F */
  0x0000,   /* U+F970 */
  0x0000,   /* U+F971 */
  0x0000,   /* U+F972 */
  0x0000,   /* U+F973 */
  0x0000,   /* U+F974 */
  0x0000,   /* U+F975 */
  0x0000,   /* U+F976 */
  0x0000,   /* U+F977 */
  0x0000,   /* U+F978 */
  0x0000,   /* U+F979 */
  0x0000,   /* U+F97A */
  0x0000,   /* U+F97B */
  0x0000,   /* U+F97C */
  0x0000,   /* U+F97D */
  0x0000,   /* U+F97E */
  0x0000,   /* U+F97F */
  0x0000,   /* U+F980 */
  0x0000,   /* U+F981 */
  0x0000,   /* U+F982 */
  0x0000,   /* U+F983 */
  0x0000,   /* U+F984 */
  0x0000,   /* U+F985 */
  0x0000,   /* U+F986 */
  0x0000,   /* U+F987 */
  0x0000,   /* U+F988 */
  0x0000,   /* U+F989 */
  0x0000,   /* U+F98A */
  0x0000,   /* U+F98B */
  0x0000,   /* U+F98C */
  0x0000,   /* U+F98D */
  0x0000,   /* U+F98E */
  0x0000,   /* U+F98F */
  0x0000,   /* U+F990 */
  0x0000,   /* U+F991 */
  0x0000,   /* U+F992 */
  0x0000,   /* U+F993 */
  0x0000,   /* U+F994 */
  0x0000,   /* U+F995 */
  0x0000,   /* U+F996 */
  0x0000,   /* U+F997 */
  0x0000,   /* U+F998 */
  0x0000,   /* U+F999 */
  0x0000,   /* U+F99A */
  0x0000,   /* U+F99B */
  0x0000,   /* U+F99C */
  0x0000,   /* U+F99D */
  0x0000,   /* U+F99E */
  0x0000,   /* U+F99F */
  0x0000,   /* U+F9A0 */
  0x0000,   /* U+F9A1 */
  0x0000,   /* U+F9A2 */
  0x0000,   /* U+F9A3 */
  0x0000,   /* U+F9A4 */
  0x0000,   /* U+F9A5 */
  0x0000,   /* U+F9A6 */
  0x0000,   /* U+F9A7 */
  0x0000,   /* U+F9A8 */
  0x0000,   /* U+F9A9 */
  0x0000,   /* U+F9AA */
  0x0000,   /* U+F9AB */
  0x0000,   /* U+F9AC */
  0x0000,   /* U+F9AD */
  0x0000,   /* U+F9AE */
  0x0000,   /* U+F9AF */
  0x0000,   /* U+F9B0 */
  0x0000,   /* U+F9B1 */
  0x0000,   /* U+F9B2 */
  0x0000,   /* U+F9B3 */
  0x0000,   /* U+F9B4 */
  0x0000,   /* U+F9B5 */
  0x0000,   /* U+F9B6 */
  0x0000,   /* U+F9B7 */
  0x0000,   /* U+F9B8 */
  0x0000,   /* U+F9B9 */
  0x0000,   /* U+F9BA */
  0x0000,   /* U+F9BB */
  0x0000,   /* U+F9BC */
  0x0000,   /* U+F9BD */
  0x0000,   /* U+F9BE */
  0x0000,   /* U+F9BF */
  0x0000,   /* U+F9C0 */
  0x0000,   /* U+F9C1 */
  0x0000,   /* U+F9C2 */
  0x0000,   /* U+F9C3 */
  0x0000,   /* U+F9C4 */
  0x0000,   /* U+F9C5 */
  0x0000,   /* U+F9C6 */
  0x0000,   /* U+F9C7 */
  0x0000,   /* U+F9C8 */
  0x0000,   /* U+F9C9 */
  0x0000,   /* U+F9CA */
  0x0000,   /* U+F9CB */
  0x0000,   /* U+F9CC */
  0x0000,   /* U+F9CD */
  0x0000,   /* U+F9CE */
  0x0000,   /* U+F9CF */
  0x0000,   /* U+F9D0 */
  0x0000,   /* U+F9D1 */
  0x0000,   /* U+F9D2 */
  0x0000,   /* U+F9D3 */
  0x0000,   /* U+F9D4 */
  0x0000,   /* U+F9D5 */
  0x0000,   /* U+F9D6 */
  0x0000,   /* U+F9D7 */
  0x0000,   /* U+F9D8 */
  0x0000,   /* U+F9D9 */
  0x0000,   /* U+F9DA */
  0x0000,   /* U+F9DB */
  0x0000,   /* U+F9DC */
  0x0000,   /* U+F9DD */
  0x0000,   /* U+F9DE */
  0x0000,   /* U+F9DF */
  0x0000,   /* U+F9E0 */
  0x0000,   /* U+F9E1 */
  0x0000,   /* U+F9E2 */
  0x0000,   /* U+F9E3 */
  0x0000,   /* U+F9E4 */
  0x0000,   /* U+F9E5 */
  0x0000,   /* U+F9E6 */
  0x0000,   /* U+F9E7 */
  0x0000,   /* U+F9E8 */
  0x0000,   /* U+F9E9 */
  0x0000,   /* U+F9EA */
  0x0000,   /* U+F9EB */
  0x0000,   /* U+F9EC */
  0x0000,   /* U+F9ED */
  0x0000,   /* U+F9EE */
  0x0000,   /* U+F9EF */
  0x0000,   /* U+F9F0 */
  0x0000,   /* U+F9F1 */
  0x0000,   /* U+F9F2 */
  0x0000,   /* U+F9F3 */
  0x0000,   /* U+F9F4 */
  0x0000,   /* U+F9F5 */
  0x0000,   /* U+F9F6 */
  0x0000,   /* U+F9F7 */
  0x0000,   /* U+F9F8 */
  0x0000,   /* U+F9F9 */
  0x0000,   /* U+F9FA */
  0x0000,   /* U+F9FB */
  0x0000,   /* U+F9FC */
  0x0000,   /* U+F9FD */
  0x0000,   /* U+F9FE */
  0x0000,   /* U+F9FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+F900 */
  0x0000,   /* U+F901 */
  0x0000,   /* U+F902 */
  0x0000,   /* U+F903 */
  0x0000,   /* U+F904 */
  0x0000,   /* U+F905 */
  0x0000,   /* U+F906 */
  0x0000,   /* U+F907 */
  0x0000,   /* U+F908 */
  0x0000,   /* U+F909 */
  0x0000,   /* U+F90A */
  0x0000,   /* U+F90B */
  0x0000,   /* U+F90C */
  0x0000,   /* U+F90D */
  0x0000,   /* U+F90E */
  0x0000,   /* U+F90F */
  0x0000,   /* U+F910 */
  0x0000,   /* U+F911 */
  0x0000,   /* U+F912 */
  0x0000,   /* U+F913 */
  0x0000,   /* U+F914 */
  0x0000,   /* U+F915 */
  0x0000,   /* U+F916 */
  0x0000,   /* U+F917 */
  0x0000,   /* U+F918 */
  0x0000,   /* U+F919 */
  0x0000,   /* U+F91A */
  0x0000,   /* U+F91B */
  0x0000,   /* U+F91C */
  0x0000,   /* U+F91D */
  0x0000,   /* U+F91E */
  0x0000,   /* U+F91F */
  0x0000,   /* U+F920 */
  0x0000,   /* U+F921 */
  0x0000,   /* U+F922 */
  0x0000,   /* U+F923 */
  0x0000,   /* U+F924 */
  0x0000,   /* U+F925 */
  0x0000,   /* U+F926 */
  0x0000,   /* U+F927 */
  0x0000,   /* U+F928 */
  0x0000,   /* U+F929 */
  0x0000,   /* U+F92A */
  0x0000,   /* U+F92B */
  0x0000,   /* U+F92C */
  0x0000,   /* U+F92D */
  0x0000,   /* U+F92E */
  0x0000,   /* U+F92F */
  0x0000,   /* U+F930 */
  0x0000,   /* U+F931 */
  0x0000,   /* U+F932 */
  0x0000,   /* U+F933 */
  0x0000,   /* U+F934 */
  0x0000,   /* U+F935 */
  0x0000,   /* U+F936 */
  0x0000,   /* U+F937 */
  0x0000,   /* U+F938 */
  0x0000,   /* U+F939 */
  0x0000,   /* U+F93A */
  0x0000,   /* U+F93B */
  0x0000,   /* U+F93C */
  0x0000,   /* U+F93D */
  0x0000,   /* U+F93E */
  0x0000,   /* U+F93F */
  0x0000,   /* U+F940 */
  0x0000,   /* U+F941 */
  0x0000,   /* U+F942 */
  0x0000,   /* U+F943 */
  0x0000,   /* U+F944 */
  0x0000,   /* U+F945 */
  0x0000,   /* U+F946 */
  0x0000,   /* U+F947 */
  0x0000,   /* U+F948 */
  0x0000,   /* U+F949 */
  0x0000,   /* U+F94A */
  0x0000,   /* U+F94B */
  0x0000,   /* U+F94C */
  0x0000,   /* U+F94D */
  0x0000,   /* U+F94E */
  0x0000,   /* U+F94F */
  0x0000,   /* U+F950 */
  0x0000,   /* U+F951 */
  0x0000,   /* U+F952 */
  0x0000,   /* U+F953 */
  0x0000,   /* U+F954 */
  0x0000,   /* U+F955 */
  0x0000,   /* U+F956 */
  0x0000,   /* U+F957 */
  0x0000,   /* U+F958 */
  0x0000,   /* U+F959 */
  0x0000,   /* U+F95A */
  0x0000,   /* U+F95B */
  0x0000,   /* U+F95C */
  0x0000,   /* U+F95D */
  0x0000,   /* U+F95E */
  0x0000,   /* U+F95F */
  0x0000,   /* U+F960 */
  0x0000,   /* U+F961 */
  0x0000,   /* U+F962 */
  0x0000,   /* U+F963 */
  0x0000,   /* U+F964 */
  0x0000,   /* U+F965 */
  0x0000,   /* U+F966 */
  0x0000,   /* U+F967 */
  0x0000,   /* U+F968 */
  0x0000,   /* U+F969 */
  0x0000,   /* U+F96A */
  0x0000,   /* U+F96B */
  0x0000,   /* U+F96C */
  0x0000,   /* U+F96D */
  0x0000,   /* U+F96E */
  0x0000,   /* U+F96F */
  0x0000,   /* U+F970 */
  0x0000,   /* U+F971 */
  0x0000,   /* U+F972 */
  0x0000,   /* U+F973 */
  0x0000,   /* U+F974 */
  0x0000,   /* U+F975 */
  0x0000,   /* U+F976 */
  0x0000,   /* U+F977 */
  0x0000,   /* U+F978 */
  0x0000,   /* U+F979 */
  0x0000,   /* U+F97A */
  0x0000,   /* U+F97B */
  0x0000,   /* U+F97C */
  0x0000,   /* U+F97D */
  0x0000,   /* U+F97E */
  0x0000,   /* U+F97F */
  0x0000,   /* U+F980 */
  0x0000,   /* U+F981 */
  0x0000,   /* U+F982 */
  0x0000,   /* U+F983 */
  0x0000,   /* U+F984 */
  0x0000,   /* U+F985 */
  0x0000,   /* U+F986 */
  0x0000,   /* U+F987 */
  0x0000,   /* U+F988 */
  0x0000,   /* U+F989 */
  0x0000,   /* U+F98A */
  0x0000,   /* U+F98B */
  0x0000,   /* U+F98C */
  0x0000,   /* U+F98D */
  0x0000,   /* U+F98E */
  0x0000,   /* U+F98F */
  0x0000,   /* U+F990 */
  0x0000,   /* U+F991 */
  0x0000,   /* U+F992 */
  0x0000,   /* U+F993 */
  0x0000,   /* U+F994 */
  0x0000,   /* U+F995 */
  0x0000,   /* U+F996 */
  0x0000,   /* U+F997 */
  0x0000,   /* U+F998 */
  0x0000,   /* U+F999 */
  0x0000,   /* U+F99A */
  0x0000,   /* U+F99B */
  0x0000,   /* U+F99C */
  0x0000,   /* U+F99D */
  0x0000,   /* U+F99E */
  0x0000,   /* U+F99F */
  0x0000,   /* U+F9A0 */
  0x0000,   /* U+F9A1 */
  0x0000,   /* U+F9A2 */
  0x0000,   /* U+F9A3 */
  0x0000,   /* U+F9A4 */
  0x0000,   /* U+F9A5 */
  0x0000,   /* U+F9A6 */
  0x0000,   /* U+F9A7 */
  0x0000,   /* U+F9A8 */
  0x0000,   /* U+F9A9 */
  0x0000,   /* U+F9AA */
  0x0000,   /* U+F9AB */
  0x0000,   /* U+F9AC */
  0x0000,   /* U+F9AD */
  0x0000,   /* U+F9AE */
  0x0000,   /* U+F9AF */
  0x0000,   /* U+F9B0 */
  0x0000,   /* U+F9B1 */
  0x0000,   /* U+F9B2 */
  0x0000,   /* U+F9B3 */
  0x0000,   /* U+F9B4 */
  0x0000,   /* U+F9B5 */
  0x0000,   /* U+F9B6 */
  0x0000,   /* U+F9B7 */
  0x0000,   /* U+F9B8 */
  0x0000,   /* U+F9B9 */
  0x0000,   /* U+F9BA */
  0x0000,   /* U+F9BB */
  0x0000,   /* U+F9BC */
  0x0000,   /* U+F9BD */
  0x0000,   /* U+F9BE */
  0x0000,   /* U+F9BF */
  0x0000,   /* U+F9C0 */
  0x0000,   /* U+F9C1 */
  0x0000,   /* U+F9C2 */
  0x0000,   /* U+F9C3 */
  0x0000,   /* U+F9C4 */
  0x0000,   /* U+F9C5 */
  0x0000,   /* U+F9C6 */
  0x0000,   /* U+F9C7 */
  0x0000,   /* U+F9C8 */
  0x0000,   /* U+F9C9 */
  0x0000,   /* U+F9CA */
  0x0000,   /* U+F9CB */
  0x0000,   /* U+F9CC */
  0x0000,   /* U+F9CD */
  0x0000,   /* U+F9CE */
  0x0000,   /* U+F9CF */
  0x0000,   /* U+F9D0 */
  0x0000,   /* U+F9D1 */
  0x0000,   /* U+F9D2 */
  0x0000,   /* U+F9D3 */
  0x0000,   /* U+F9D4 */
  0x0000,   /* U+F9D5 */
  0x0000,   /* U+F9D6 */
  0x0000,   /* U+F9D7 */
  0x0000,   /* U+F9D8 */
  0x0000,   /* U+F9D9 */
  0x0000,   /* U+F9DA */
  0x0000,   /* U+F9DB */
  0x0000,   /* U+F9DC */
  0x0000,   /* U+F9DD */
  0x0000,   /* U+F9DE */
  0x0000,   /* U+F9DF */
  0x0000,   /* U+F9E0 */
  0x0000,   /* U+F9E1 */
  0x0000,   /* U+F9E2 */
  0x0000,   /* U+F9E3 */
  0x0000,   /* U+F9E4 */
  0x0000,   /* U+F9E5 */
  0x0000,   /* U+F9E6 */
  0x0000,   /* U+F9E7 */
  0x0000,   /* U+F9E8 */
  0x0000,   /* U+F9E9 */
  0x0000,   /* U+F9EA */
  0x0000,   /* U+F9EB */
  0x0000,   /* U+F9EC */
  0x0000,   /* U+F9ED */
  0x0000,   /* U+F9EE */
  0x0000,   /* U+F9EF */
  0x0000,   /* U+F9F0 */
  0x0000,   /* U+F9F1 */
  0x0000,   /* U+F9F2 */
  0x0000,   /* U+F9F3 */
  0x0000,   /* U+F9F4 */
  0x0000,   /* U+F9F5 */
  0x0000,   /* U+F9F6 */
  0x0000,   /* U+F9F7 */
  0x0000,   /* U+F9F8 */
  0x0000,   /* U+F9F9 */
  0x0000,   /* U+F9FA */
  0x0000,   /* U+F9FB */
  0x0000,   /* U+F9FC */
  0x0000,   /* U+F9FD */
  0x0000,   /* U+F9FE */
  0x0000,   /* U+F9FF */
};

uint16 uca900_p0FA[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB40,   /* U+FA00 */
  0xFB40,   /* U+FA01 */
  0xFB40,   /* U+FA02 */
  0xFB40,   /* U+FA03 */
  0xFB40,   /* U+FA04 */
  0xFB40,   /* U+FA05 */
  0xFB40,   /* U+FA06 */
  0xFB41,   /* U+FA07 */
  0xFB41,   /* U+FA08 */
  0xFB41,   /* U+FA09 */
  0xFB41,   /* U+FA0A */
  0xFB40,   /* U+FA0B */
  0xFB40,   /* U+FA0C */
  0xFB40,   /* U+FA0D */
  0xFB41,   /* U+FA0E */
  0xFB41,   /* U+FA0F */
  0xFB40,   /* U+FA10 */
  0xFB41,   /* U+FA11 */
  0xFB40,   /* U+FA12 */
  0xFB41,   /* U+FA13 */
  0xFB41,   /* U+FA14 */
  0xFB40,   /* U+FA15 */
  0xFB40,   /* U+FA16 */
  0xFB40,   /* U+FA17 */
  0xFB40,   /* U+FA18 */
  0xFB40,   /* U+FA19 */
  0xFB40,   /* U+FA1A */
  0xFB40,   /* U+FA1B */
  0xFB41,   /* U+FA1C */
  0xFB40,   /* U+FA1D */
  0xFB40,   /* U+FA1E */
  0xFB41,   /* U+FA1F */
  0xFB41,   /* U+FA20 */
  0xFB41,   /* U+FA21 */
  0xFB41,   /* U+FA22 */
  0xFB41,   /* U+FA23 */
  0xFB41,   /* U+FA24 */
  0xFB41,   /* U+FA25 */
  0xFB41,   /* U+FA26 */
  0xFB41,   /* U+FA27 */
  0xFB41,   /* U+FA28 */
  0xFB41,   /* U+FA29 */
  0xFB41,   /* U+FA2A */
  0xFB41,   /* U+FA2B */
  0xFB41,   /* U+FA2C */
  0xFB41,   /* U+FA2D */
  0xFB41,   /* U+FA2E */
  0xFB41,   /* U+FA2F */
  0xFB40,   /* U+FA30 */
  0xFB40,   /* U+FA31 */
  0xFB40,   /* U+FA32 */
  0xFB40,   /* U+FA33 */
  0xFB40,   /* U+FA34 */
  0xFB40,   /* U+FA35 */
  0xFB40,   /* U+FA36 */
  0xFB40,   /* U+FA37 */
  0xFB40,   /* U+FA38 */
  0xFB40,   /* U+FA39 */
  0xFB40,   /* U+FA3A */
  0xFB40,   /* U+FA3B */
  0xFB40,   /* U+FA3C */
  0xFB40,   /* U+FA3D */
  0xFB40,   /* U+FA3E */
  0xFB40,   /* U+FA3F */
  0xFB40,   /* U+FA40 */
  0xFB40,   /* U+FA41 */
  0xFB40,   /* U+FA42 */
  0xFB40,   /* U+FA43 */
  0xFB40,   /* U+FA44 */
  0xFB40,   /* U+FA45 */
  0xFB40,   /* U+FA46 */
  0xFB40,   /* U+FA47 */
  0xFB40,   /* U+FA48 */
  0xFB40,   /* U+FA49 */
  0xFB40,   /* U+FA4A */
  0xFB40,   /* U+FA4B */
  0xFB40,   /* U+FA4C */
  0xFB40,   /* U+FA4D */
  0xFB40,   /* U+FA4E */
  0xFB40,   /* U+FA4F */
  0xFB40,   /* U+FA50 */
  0xFB40,   /* U+FA51 */
  0xFB40,   /* U+FA52 */
  0xFB40,   /* U+FA53 */
  0xFB40,   /* U+FA54 */
  0xFB40,   /* U+FA55 */
  0xFB40,   /* U+FA56 */
  0xFB40,   /* U+FA57 */
  0xFB40,   /* U+FA58 */
  0xFB40,   /* U+FA59 */
  0xFB40,   /* U+FA5A */
  0xFB41,   /* U+FA5B */
  0xFB41,   /* U+FA5C */
  0xFB41,   /* U+FA5D */
  0xFB41,   /* U+FA5E */
  0xFB41,   /* U+FA5F */
  0xFB41,   /* U+FA60 */
  0xFB41,   /* U+FA61 */
  0xFB41,   /* U+FA62 */
  0xFB41,   /* U+FA63 */
  0xFB41,   /* U+FA64 */
  0xFB41,   /* U+FA65 */
  0xFB41,   /* U+FA66 */
  0xFB41,   /* U+FA67 */
  0xFB41,   /* U+FA68 */
  0xFB41,   /* U+FA69 */
  0xFB41,   /* U+FA6A */
  0xFB40,   /* U+FA6B */
  0xFB84,   /* U+FA6C */
  0xFB41,   /* U+FA6D */
  0xFBC1,   /* U+FA6E */
  0xFBC1,   /* U+FA6F */
  0xFB40,   /* U+FA70 */
  0xFB40,   /* U+FA71 */
  0xFB40,   /* U+FA72 */
  0xFB40,   /* U+FA73 */
  0xFB40,   /* U+FA74 */
  0xFB40,   /* U+FA75 */
  0xFB40,   /* U+FA76 */
  0xFB40,   /* U+FA77 */
  0xFB40,   /* U+FA78 */
  0xFB40,   /* U+FA79 */
  0xFB40,   /* U+FA7A */
  0xFB40,   /* U+FA7B */
  0xFB40,   /* U+FA7C */
  0xFB40,   /* U+FA7D */
  0xFB40,   /* U+FA7E */
  0xFB40,   /* U+FA7F */
  0xFB40,   /* U+FA80 */
  0xFB40,   /* U+FA81 */
  0xFB40,   /* U+FA82 */
  0xFB40,   /* U+FA83 */
  0xFB40,   /* U+FA84 */
  0xFB40,   /* U+FA85 */
  0xFB40,   /* U+FA86 */
  0xFB40,   /* U+FA87 */
  0xFB40,   /* U+FA88 */
  0xFB40,   /* U+FA89 */
  0xFB40,   /* U+FA8A */
  0xFB40,   /* U+FA8B */
  0xFB40,   /* U+FA8C */
  0xFB40,   /* U+FA8D */
  0xFB40,   /* U+FA8E */
  0xFB40,   /* U+FA8F */
  0xFB40,   /* U+FA90 */
  0xFB40,   /* U+FA91 */
  0xFB40,   /* U+FA92 */
  0xFB40,   /* U+FA93 */
  0xFB40,   /* U+FA94 */
  0xFB40,   /* U+FA95 */
  0xFB40,   /* U+FA96 */
  0xFB40,   /* U+FA97 */
  0xFB40,   /* U+FA98 */
  0xFB40,   /* U+FA99 */
  0xFB40,   /* U+FA9A */
  0xFB40,   /* U+FA9B */
  0xFB40,   /* U+FA9C */
  0xFB40,   /* U+FA9D */
  0xFB40,   /* U+FA9E */
  0xFB40,   /* U+FA9F */
  0xFB40,   /* U+FAA0 */
  0xFB40,   /* U+FAA1 */
  0xFB40,   /* U+FAA2 */
  0xFB40,   /* U+FAA3 */
  0xFB40,   /* U+FAA4 */
  0xFB40,   /* U+FAA5 */
  0xFB40,   /* U+FAA6 */
  0xFB40,   /* U+FAA7 */
  0xFB40,   /* U+FAA8 */
  0xFB40,   /* U+FAA9 */
  0xFB40,   /* U+FAAA */
  0xFB40,   /* U+FAAB */
  0xFB40,   /* U+FAAC */
  0xFB40,   /* U+FAAD */
  0xFB40,   /* U+FAAE */
  0xFB40,   /* U+FAAF */
  0xFB40,   /* U+FAB0 */
  0xFB40,   /* U+FAB1 */
  0xFB41,   /* U+FAB2 */
  0xFB41,   /* U+FAB3 */
  0xFB41,   /* U+FAB4 */
  0xFB41,   /* U+FAB5 */
  0xFB41,   /* U+FAB6 */
  0xFB41,   /* U+FAB7 */
  0xFB41,   /* U+FAB8 */
  0xFB41,   /* U+FAB9 */
  0xFB41,   /* U+FABA */
  0xFB41,   /* U+FABB */
  0xFB41,   /* U+FABC */
  0xFB41,   /* U+FABD */
  0xFB41,   /* U+FABE */
  0xFB41,   /* U+FABF */
  0xFB41,   /* U+FAC0 */
  0xFB41,   /* U+FAC1 */
  0xFB41,   /* U+FAC2 */
  0xFB41,   /* U+FAC3 */
  0xFB41,   /* U+FAC4 */
  0xFB41,   /* U+FAC5 */
  0xFB41,   /* U+FAC6 */
  0xFB41,   /* U+FAC7 */
  0xFB41,   /* U+FAC8 */
  0xFB41,   /* U+FAC9 */
  0xFB41,   /* U+FACA */
  0xFB41,   /* U+FACB */
  0xFB41,   /* U+FACC */
  0xFB41,   /* U+FACD */
  0xFB41,   /* U+FACE */
  0xFB84,   /* U+FACF */
  0xFB84,   /* U+FAD0 */
  0xFB84,   /* U+FAD1 */
  0xFB80,   /* U+FAD2 */
  0xFB80,   /* U+FAD3 */
  0xFB80,   /* U+FAD4 */
  0xFB84,   /* U+FAD5 */
  0xFB84,   /* U+FAD6 */
  0xFB84,   /* U+FAD7 */
  0xFB41,   /* U+FAD8 */
  0xFB41,   /* U+FAD9 */
  0xFBC1,   /* U+FADA */
  0xFBC1,   /* U+FADB */
  0xFBC1,   /* U+FADC */
  0xFBC1,   /* U+FADD */
  0xFBC1,   /* U+FADE */
  0xFBC1,   /* U+FADF */
  0xFBC1,   /* U+FAE0 */
  0xFBC1,   /* U+FAE1 */
  0xFBC1,   /* U+FAE2 */
  0xFBC1,   /* U+FAE3 */
  0xFBC1,   /* U+FAE4 */
  0xFBC1,   /* U+FAE5 */
  0xFBC1,   /* U+FAE6 */
  0xFBC1,   /* U+FAE7 */
  0xFBC1,   /* U+FAE8 */
  0xFBC1,   /* U+FAE9 */
  0xFBC1,   /* U+FAEA */
  0xFBC1,   /* U+FAEB */
  0xFBC1,   /* U+FAEC */
  0xFBC1,   /* U+FAED */
  0xFBC1,   /* U+FAEE */
  0xFBC1,   /* U+FAEF */
  0xFBC1,   /* U+FAF0 */
  0xFBC1,   /* U+FAF1 */
  0xFBC1,   /* U+FAF2 */
  0xFBC1,   /* U+FAF3 */
  0xFBC1,   /* U+FAF4 */
  0xFBC1,   /* U+FAF5 */
  0xFBC1,   /* U+FAF6 */
  0xFBC1,   /* U+FAF7 */
  0xFBC1,   /* U+FAF8 */
  0xFBC1,   /* U+FAF9 */
  0xFBC1,   /* U+FAFA */
  0xFBC1,   /* U+FAFB */
  0xFBC1,   /* U+FAFC */
  0xFBC1,   /* U+FAFD */
  0xFBC1,   /* U+FAFE */
  0xFBC1,   /* U+FAFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+FA00 */
  0x0020,   /* U+FA01 */
  0x0020,   /* U+FA02 */
  0x0020,   /* U+FA03 */
  0x0020,   /* U+FA04 */
  0x0020,   /* U+FA05 */
  0x0020,   /* U+FA06 */
  0x0020,   /* U+FA07 */
  0x0020,   /* U+FA08 */
  0x0020,   /* U+FA09 */
  0x0020,   /* U+FA0A */
  0x0020,   /* U+FA0B */
  0x0020,   /* U+FA0C */
  0x0020,   /* U+FA0D */
  0x0020,   /* U+FA0E */
  0x0020,   /* U+FA0F */
  0x0020,   /* U+FA10 */
  0x0020,   /* U+FA11 */
  0x0020,   /* U+FA12 */
  0x0020,   /* U+FA13 */
  0x0020,   /* U+FA14 */
  0x0020,   /* U+FA15 */
  0x0020,   /* U+FA16 */
  0x0020,   /* U+FA17 */
  0x0020,   /* U+FA18 */
  0x0020,   /* U+FA19 */
  0x0020,   /* U+FA1A */
  0x0020,   /* U+FA1B */
  0x0020,   /* U+FA1C */
  0x0020,   /* U+FA1D */
  0x0020,   /* U+FA1E */
  0x0020,   /* U+FA1F */
  0x0020,   /* U+FA20 */
  0x0020,   /* U+FA21 */
  0x0020,   /* U+FA22 */
  0x0020,   /* U+FA23 */
  0x0020,   /* U+FA24 */
  0x0020,   /* U+FA25 */
  0x0020,   /* U+FA26 */
  0x0020,   /* U+FA27 */
  0x0020,   /* U+FA28 */
  0x0020,   /* U+FA29 */
  0x0020,   /* U+FA2A */
  0x0020,   /* U+FA2B */
  0x0020,   /* U+FA2C */
  0x0020,   /* U+FA2D */
  0x0020,   /* U+FA2E */
  0x0020,   /* U+FA2F */
  0x0020,   /* U+FA30 */
  0x0020,   /* U+FA31 */
  0x0020,   /* U+FA32 */
  0x0020,   /* U+FA33 */
  0x0020,   /* U+FA34 */
  0x0020,   /* U+FA35 */
  0x0020,   /* U+FA36 */
  0x0020,   /* U+FA37 */
  0x0020,   /* U+FA38 */
  0x0020,   /* U+FA39 */
  0x0020,   /* U+FA3A */
  0x0020,   /* U+FA3B */
  0x0020,   /* U+FA3C */
  0x0020,   /* U+FA3D */
  0x0020,   /* U+FA3E */
  0x0020,   /* U+FA3F */
  0x0020,   /* U+FA40 */
  0x0020,   /* U+FA41 */
  0x0020,   /* U+FA42 */
  0x0020,   /* U+FA43 */
  0x0020,   /* U+FA44 */
  0x0020,   /* U+FA45 */
  0x0020,   /* U+FA46 */
  0x0020,   /* U+FA47 */
  0x0020,   /* U+FA48 */
  0x0020,   /* U+FA49 */
  0x0020,   /* U+FA4A */
  0x0020,   /* U+FA4B */
  0x0020,   /* U+FA4C */
  0x0020,   /* U+FA4D */
  0x0020,   /* U+FA4E */
  0x0020,   /* U+FA4F */
  0x0020,   /* U+FA50 */
  0x0020,   /* U+FA51 */
  0x0020,   /* U+FA52 */
  0x0020,   /* U+FA53 */
  0x0020,   /* U+FA54 */
  0x0020,   /* U+FA55 */
  0x0020,   /* U+FA56 */
  0x0020,   /* U+FA57 */
  0x0020,   /* U+FA58 */
  0x0020,   /* U+FA59 */
  0x0020,   /* U+FA5A */
  0x0020,   /* U+FA5B */
  0x0020,   /* U+FA5C */
  0x0020,   /* U+FA5D */
  0x0020,   /* U+FA5E */
  0x0020,   /* U+FA5F */
  0x0020,   /* U+FA60 */
  0x0020,   /* U+FA61 */
  0x0020,   /* U+FA62 */
  0x0020,   /* U+FA63 */
  0x0020,   /* U+FA64 */
  0x0020,   /* U+FA65 */
  0x0020,   /* U+FA66 */
  0x0020,   /* U+FA67 */
  0x0020,   /* U+FA68 */
  0x0020,   /* U+FA69 */
  0x0020,   /* U+FA6A */
  0x0020,   /* U+FA6B */
  0x0020,   /* U+FA6C */
  0x0020,   /* U+FA6D */
  0x0020,   /* U+FA6E */
  0x0020,   /* U+FA6F */
  0x0020,   /* U+FA70 */
  0x0020,   /* U+FA71 */
  0x0020,   /* U+FA72 */
  0x0020,   /* U+FA73 */
  0x0020,   /* U+FA74 */
  0x0020,   /* U+FA75 */
  0x0020,   /* U+FA76 */
  0x0020,   /* U+FA77 */
  0x0020,   /* U+FA78 */
  0x0020,   /* U+FA79 */
  0x0020,   /* U+FA7A */
  0x0020,   /* U+FA7B */
  0x0020,   /* U+FA7C */
  0x0020,   /* U+FA7D */
  0x0020,   /* U+FA7E */
  0x0020,   /* U+FA7F */
  0x0020,   /* U+FA80 */
  0x0020,   /* U+FA81 */
  0x0020,   /* U+FA82 */
  0x0020,   /* U+FA83 */
  0x0020,   /* U+FA84 */
  0x0020,   /* U+FA85 */
  0x0020,   /* U+FA86 */
  0x0020,   /* U+FA87 */
  0x0020,   /* U+FA88 */
  0x0020,   /* U+FA89 */
  0x0020,   /* U+FA8A */
  0x0020,   /* U+FA8B */
  0x0020,   /* U+FA8C */
  0x0020,   /* U+FA8D */
  0x0020,   /* U+FA8E */
  0x0020,   /* U+FA8F */
  0x0020,   /* U+FA90 */
  0x0020,   /* U+FA91 */
  0x0020,   /* U+FA92 */
  0x0020,   /* U+FA93 */
  0x0020,   /* U+FA94 */
  0x0020,   /* U+FA95 */
  0x0020,   /* U+FA96 */
  0x0020,   /* U+FA97 */
  0x0020,   /* U+FA98 */
  0x0020,   /* U+FA99 */
  0x0020,   /* U+FA9A */
  0x0020,   /* U+FA9B */
  0x0020,   /* U+FA9C */
  0x0020,   /* U+FA9D */
  0x0020,   /* U+FA9E */
  0x0020,   /* U+FA9F */
  0x0020,   /* U+FAA0 */
  0x0020,   /* U+FAA1 */
  0x0020,   /* U+FAA2 */
  0x0020,   /* U+FAA3 */
  0x0020,   /* U+FAA4 */
  0x0020,   /* U+FAA5 */
  0x0020,   /* U+FAA6 */
  0x0020,   /* U+FAA7 */
  0x0020,   /* U+FAA8 */
  0x0020,   /* U+FAA9 */
  0x0020,   /* U+FAAA */
  0x0020,   /* U+FAAB */
  0x0020,   /* U+FAAC */
  0x0020,   /* U+FAAD */
  0x0020,   /* U+FAAE */
  0x0020,   /* U+FAAF */
  0x0020,   /* U+FAB0 */
  0x0020,   /* U+FAB1 */
  0x0020,   /* U+FAB2 */
  0x0020,   /* U+FAB3 */
  0x0020,   /* U+FAB4 */
  0x0020,   /* U+FAB5 */
  0x0020,   /* U+FAB6 */
  0x0020,   /* U+FAB7 */
  0x0020,   /* U+FAB8 */
  0x0020,   /* U+FAB9 */
  0x0020,   /* U+FABA */
  0x0020,   /* U+FABB */
  0x0020,   /* U+FABC */
  0x0020,   /* U+FABD */
  0x0020,   /* U+FABE */
  0x0020,   /* U+FABF */
  0x0020,   /* U+FAC0 */
  0x0020,   /* U+FAC1 */
  0x0020,   /* U+FAC2 */
  0x0020,   /* U+FAC3 */
  0x0020,   /* U+FAC4 */
  0x0020,   /* U+FAC5 */
  0x0020,   /* U+FAC6 */
  0x0020,   /* U+FAC7 */
  0x0020,   /* U+FAC8 */
  0x0020,   /* U+FAC9 */
  0x0020,   /* U+FACA */
  0x0020,   /* U+FACB */
  0x0020,   /* U+FACC */
  0x0020,   /* U+FACD */
  0x0020,   /* U+FACE */
  0x0020,   /* U+FACF */
  0x0020,   /* U+FAD0 */
  0x0020,   /* U+FAD1 */
  0x0020,   /* U+FAD2 */
  0x0020,   /* U+FAD3 */
  0x0020,   /* U+FAD4 */
  0x0020,   /* U+FAD5 */
  0x0020,   /* U+FAD6 */
  0x0020,   /* U+FAD7 */
  0x0020,   /* U+FAD8 */
  0x0020,   /* U+FAD9 */
  0x0020,   /* U+FADA */
  0x0020,   /* U+FADB */
  0x0020,   /* U+FADC */
  0x0020,   /* U+FADD */
  0x0020,   /* U+FADE */
  0x0020,   /* U+FADF */
  0x0020,   /* U+FAE0 */
  0x0020,   /* U+FAE1 */
  0x0020,   /* U+FAE2 */
  0x0020,   /* U+FAE3 */
  0x0020,   /* U+FAE4 */
  0x0020,   /* U+FAE5 */
  0x0020,   /* U+FAE6 */
  0x0020,   /* U+FAE7 */
  0x0020,   /* U+FAE8 */
  0x0020,   /* U+FAE9 */
  0x0020,   /* U+FAEA */
  0x0020,   /* U+FAEB */
  0x0020,   /* U+FAEC */
  0x0020,   /* U+FAED */
  0x0020,   /* U+FAEE */
  0x0020,   /* U+FAEF */
  0x0020,   /* U+FAF0 */
  0x0020,   /* U+FAF1 */
  0x0020,   /* U+FAF2 */
  0x0020,   /* U+FAF3 */
  0x0020,   /* U+FAF4 */
  0x0020,   /* U+FAF5 */
  0x0020,   /* U+FAF6 */
  0x0020,   /* U+FAF7 */
  0x0020,   /* U+FAF8 */
  0x0020,   /* U+FAF9 */
  0x0020,   /* U+FAFA */
  0x0020,   /* U+FAFB */
  0x0020,   /* U+FAFC */
  0x0020,   /* U+FAFD */
  0x0020,   /* U+FAFE */
  0x0020,   /* U+FAFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+FA00 */
  0x0002,   /* U+FA01 */
  0x0002,   /* U+FA02 */
  0x0002,   /* U+FA03 */
  0x0002,   /* U+FA04 */
  0x0002,   /* U+FA05 */
  0x0002,   /* U+FA06 */
  0x0002,   /* U+FA07 */
  0x0002,   /* U+FA08 */
  0x0002,   /* U+FA09 */
  0x0002,   /* U+FA0A */
  0x0002,   /* U+FA0B */
  0x0002,   /* U+FA0C */
  0x0002,   /* U+FA0D */
  0x0002,   /* U+FA0E */
  0x0002,   /* U+FA0F */
  0x0002,   /* U+FA10 */
  0x0002,   /* U+FA11 */
  0x0002,   /* U+FA12 */
  0x0002,   /* U+FA13 */
  0x0002,   /* U+FA14 */
  0x0002,   /* U+FA15 */
  0x0002,   /* U+FA16 */
  0x0002,   /* U+FA17 */
  0x0002,   /* U+FA18 */
  0x0002,   /* U+FA19 */
  0x0002,   /* U+FA1A */
  0x0002,   /* U+FA1B */
  0x0002,   /* U+FA1C */
  0x0002,   /* U+FA1D */
  0x0002,   /* U+FA1E */
  0x0002,   /* U+FA1F */
  0x0002,   /* U+FA20 */
  0x0002,   /* U+FA21 */
  0x0002,   /* U+FA22 */
  0x0002,   /* U+FA23 */
  0x0002,   /* U+FA24 */
  0x0002,   /* U+FA25 */
  0x0002,   /* U+FA26 */
  0x0002,   /* U+FA27 */
  0x0002,   /* U+FA28 */
  0x0002,   /* U+FA29 */
  0x0002,   /* U+FA2A */
  0x0002,   /* U+FA2B */
  0x0002,   /* U+FA2C */
  0x0002,   /* U+FA2D */
  0x0002,   /* U+FA2E */
  0x0002,   /* U+FA2F */
  0x0002,   /* U+FA30 */
  0x0002,   /* U+FA31 */
  0x0002,   /* U+FA32 */
  0x0002,   /* U+FA33 */
  0x0002,   /* U+FA34 */
  0x0002,   /* U+FA35 */
  0x0002,   /* U+FA36 */
  0x0002,   /* U+FA37 */
  0x0002,   /* U+FA38 */
  0x0002,   /* U+FA39 */
  0x0002,   /* U+FA3A */
  0x0002,   /* U+FA3B */
  0x0002,   /* U+FA3C */
  0x0002,   /* U+FA3D */
  0x0002,   /* U+FA3E */
  0x0002,   /* U+FA3F */
  0x0002,   /* U+FA40 */
  0x0002,   /* U+FA41 */
  0x0002,   /* U+FA42 */
  0x0002,   /* U+FA43 */
  0x0002,   /* U+FA44 */
  0x0002,   /* U+FA45 */
  0x0002,   /* U+FA46 */
  0x0002,   /* U+FA47 */
  0x0002,   /* U+FA48 */
  0x0002,   /* U+FA49 */
  0x0002,   /* U+FA4A */
  0x0002,   /* U+FA4B */
  0x0002,   /* U+FA4C */
  0x0002,   /* U+FA4D */
  0x0002,   /* U+FA4E */
  0x0002,   /* U+FA4F */
  0x0002,   /* U+FA50 */
  0x0002,   /* U+FA51 */
  0x0002,   /* U+FA52 */
  0x0002,   /* U+FA53 */
  0x0002,   /* U+FA54 */
  0x0002,   /* U+FA55 */
  0x0002,   /* U+FA56 */
  0x0002,   /* U+FA57 */
  0x0002,   /* U+FA58 */
  0x0002,   /* U+FA59 */
  0x0002,   /* U+FA5A */
  0x0002,   /* U+FA5B */
  0x0002,   /* U+FA5C */
  0x0002,   /* U+FA5D */
  0x0002,   /* U+FA5E */
  0x0002,   /* U+FA5F */
  0x0002,   /* U+FA60 */
  0x0002,   /* U+FA61 */
  0x0002,   /* U+FA62 */
  0x0002,   /* U+FA63 */
  0x0002,   /* U+FA64 */
  0x0002,   /* U+FA65 */
  0x0002,   /* U+FA66 */
  0x0002,   /* U+FA67 */
  0x0002,   /* U+FA68 */
  0x0002,   /* U+FA69 */
  0x0002,   /* U+FA6A */
  0x0002,   /* U+FA6B */
  0x0002,   /* U+FA6C */
  0x0002,   /* U+FA6D */
  0x0002,   /* U+FA6E */
  0x0002,   /* U+FA6F */
  0x0002,   /* U+FA70 */
  0x0002,   /* U+FA71 */
  0x0002,   /* U+FA72 */
  0x0002,   /* U+FA73 */
  0x0002,   /* U+FA74 */
  0x0002,   /* U+FA75 */
  0x0002,   /* U+FA76 */
  0x0002,   /* U+FA77 */
  0x0002,   /* U+FA78 */
  0x0002,   /* U+FA79 */
  0x0002,   /* U+FA7A */
  0x0002,   /* U+FA7B */
  0x0002,   /* U+FA7C */
  0x0002,   /* U+FA7D */
  0x0002,   /* U+FA7E */
  0x0002,   /* U+FA7F */
  0x0002,   /* U+FA80 */
  0x0002,   /* U+FA81 */
  0x0002,   /* U+FA82 */
  0x0002,   /* U+FA83 */
  0x0002,   /* U+FA84 */
  0x0002,   /* U+FA85 */
  0x0002,   /* U+FA86 */
  0x0002,   /* U+FA87 */
  0x0002,   /* U+FA88 */
  0x0002,   /* U+FA89 */
  0x0002,   /* U+FA8A */
  0x0002,   /* U+FA8B */
  0x0002,   /* U+FA8C */
  0x0002,   /* U+FA8D */
  0x0002,   /* U+FA8E */
  0x0002,   /* U+FA8F */
  0x0002,   /* U+FA90 */
  0x0002,   /* U+FA91 */
  0x0002,   /* U+FA92 */
  0x0002,   /* U+FA93 */
  0x0002,   /* U+FA94 */
  0x0002,   /* U+FA95 */
  0x0002,   /* U+FA96 */
  0x0002,   /* U+FA97 */
  0x0002,   /* U+FA98 */
  0x0002,   /* U+FA99 */
  0x0002,   /* U+FA9A */
  0x0002,   /* U+FA9B */
  0x0002,   /* U+FA9C */
  0x0002,   /* U+FA9D */
  0x0002,   /* U+FA9E */
  0x0002,   /* U+FA9F */
  0x0002,   /* U+FAA0 */
  0x0002,   /* U+FAA1 */
  0x0002,   /* U+FAA2 */
  0x0002,   /* U+FAA3 */
  0x0002,   /* U+FAA4 */
  0x0002,   /* U+FAA5 */
  0x0002,   /* U+FAA6 */
  0x0002,   /* U+FAA7 */
  0x0002,   /* U+FAA8 */
  0x0002,   /* U+FAA9 */
  0x0002,   /* U+FAAA */
  0x0002,   /* U+FAAB */
  0x0002,   /* U+FAAC */
  0x0002,   /* U+FAAD */
  0x0002,   /* U+FAAE */
  0x0002,   /* U+FAAF */
  0x0002,   /* U+FAB0 */
  0x0002,   /* U+FAB1 */
  0x0002,   /* U+FAB2 */
  0x0002,   /* U+FAB3 */
  0x0002,   /* U+FAB4 */
  0x0002,   /* U+FAB5 */
  0x0002,   /* U+FAB6 */
  0x0002,   /* U+FAB7 */
  0x0002,   /* U+FAB8 */
  0x0002,   /* U+FAB9 */
  0x0002,   /* U+FABA */
  0x0002,   /* U+FABB */
  0x0002,   /* U+FABC */
  0x0002,   /* U+FABD */
  0x0002,   /* U+FABE */
  0x0002,   /* U+FABF */
  0x0002,   /* U+FAC0 */
  0x0002,   /* U+FAC1 */
  0x0002,   /* U+FAC2 */
  0x0002,   /* U+FAC3 */
  0x0002,   /* U+FAC4 */
  0x0002,   /* U+FAC5 */
  0x0002,   /* U+FAC6 */
  0x0002,   /* U+FAC7 */
  0x0002,   /* U+FAC8 */
  0x0002,   /* U+FAC9 */
  0x0002,   /* U+FACA */
  0x0002,   /* U+FACB */
  0x0002,   /* U+FACC */
  0x0002,   /* U+FACD */
  0x0002,   /* U+FACE */
  0x0002,   /* U+FACF */
  0x0002,   /* U+FAD0 */
  0x0002,   /* U+FAD1 */
  0x0002,   /* U+FAD2 */
  0x0002,   /* U+FAD3 */
  0x0002,   /* U+FAD4 */
  0x0002,   /* U+FAD5 */
  0x0002,   /* U+FAD6 */
  0x0002,   /* U+FAD7 */
  0x0002,   /* U+FAD8 */
  0x0002,   /* U+FAD9 */
  0x0002,   /* U+FADA */
  0x0002,   /* U+FADB */
  0x0002,   /* U+FADC */
  0x0002,   /* U+FADD */
  0x0002,   /* U+FADE */
  0x0002,   /* U+FADF */
  0x0002,   /* U+FAE0 */
  0x0002,   /* U+FAE1 */
  0x0002,   /* U+FAE2 */
  0x0002,   /* U+FAE3 */
  0x0002,   /* U+FAE4 */
  0x0002,   /* U+FAE5 */
  0x0002,   /* U+FAE6 */
  0x0002,   /* U+FAE7 */
  0x0002,   /* U+FAE8 */
  0x0002,   /* U+FAE9 */
  0x0002,   /* U+FAEA */
  0x0002,   /* U+FAEB */
  0x0002,   /* U+FAEC */
  0x0002,   /* U+FAED */
  0x0002,   /* U+FAEE */
  0x0002,   /* U+FAEF */
  0x0002,   /* U+FAF0 */
  0x0002,   /* U+FAF1 */
  0x0002,   /* U+FAF2 */
  0x0002,   /* U+FAF3 */
  0x0002,   /* U+FAF4 */
  0x0002,   /* U+FAF5 */
  0x0002,   /* U+FAF6 */
  0x0002,   /* U+FAF7 */
  0x0002,   /* U+FAF8 */
  0x0002,   /* U+FAF9 */
  0x0002,   /* U+FAFA */
  0x0002,   /* U+FAFB */
  0x0002,   /* U+FAFC */
  0x0002,   /* U+FAFD */
  0x0002,   /* U+FAFE */
  0x0002,   /* U+FAFF */

  /* Primary weight 2 for each character. */
  0xD207,   /* U+FA00 */
  0xDEA6,   /* U+FA01 */
  0xE2D3,   /* U+FA02 */
  0xFCD6,   /* U+FA03 */
  0xDB85,   /* U+FA04 */
  0xED1E,   /* U+FA05 */
  0xE6B4,   /* U+FA06 */
  0x8F3B,   /* U+FA07 */
  0x884C,   /* U+FA08 */
  0x964D,   /* U+FA09 */
  0x898B,   /* U+FA0A */
  0xDED3,   /* U+FA0B */
  0xD140,   /* U+FA0C */
  0xD5C0,   /* U+FA0D */
  0xFA0E,   /* U+FA0E */
  0xFA0F,   /* U+FA0F */
  0xD85A,   /* U+FA10 */
  0xFA11,   /* U+FA11 */
  0xE674,   /* U+FA12 */
  0xFA13,   /* U+FA13 */
  0xFA14,   /* U+FA14 */
  0xD1DE,   /* U+FA15 */
  0xF32A,   /* U+FA16 */
  0xF6CA,   /* U+FA17 */
  0xF93C,   /* U+FA18 */
  0xF95E,   /* U+FA19 */
  0xF965,   /* U+FA1A */
  0xF98F,   /* U+FA1B */
  0x9756,   /* U+FA1C */
  0xFCBE,   /* U+FA1D */
  0xFFBD,   /* U+FA1E */
  0xFA1F,   /* U+FA1F */
  0x8612,   /* U+FA20 */
  0xFA21,   /* U+FA21 */
  0x8AF8,   /* U+FA22 */
  0xFA23,   /* U+FA23 */
  0xFA24,   /* U+FA24 */
  0x9038,   /* U+FA25 */
  0x90FD,   /* U+FA26 */
  0xFA27,   /* U+FA27 */
  0xFA28,   /* U+FA28 */
  0xFA29,   /* U+FA29 */
  0x98EF,   /* U+FA2A */
  0x98FC,   /* U+FA2B */
  0x9928,   /* U+FA2C */
  0x9DB4,   /* U+FA2D */
  0x90DE,   /* U+FA2E */
  0x96B7,   /* U+FA2F */
  0xCFAE,   /* U+FA30 */
  0xD0E7,   /* U+FA31 */
  0xD14D,   /* U+FA32 */
  0xD2C9,   /* U+FA33 */
  0xD2E4,   /* U+FA34 */
  0xD351,   /* U+FA35 */
  0xD59D,   /* U+FA36 */
  0xD606,   /* U+FA37 */
  0xD668,   /* U+FA38 */
  0xD840,   /* U+FA39 */
  0xD8A8,   /* U+FA3A */
  0xDC64,   /* U+FA3B */
  0xDC6E,   /* U+FA3C */
  0xE094,   /* U+FA3D */
  0xE168,   /* U+FA3E */
  0xE18E,   /* U+FA3F */
  0xE1F2,   /* U+FA40 */
  0xE54F,   /* U+FA41 */
  0xE5E2,   /* U+FA42 */
  0xE691,   /* U+FA43 */
  0xE885,   /* U+FA44 */
  0xED77,   /* U+FA45 */
  0xEE1A,   /* U+FA46 */
  0xEF22,   /* U+FA47 */
  0xF16E,   /* U+FA48 */
  0xF22B,   /* U+FA49 */
  0xF422,   /* U+FA4A */
  0xF891,   /* U+FA4B */
  0xF93E,   /* U+FA4C */
  0xF949,   /* U+FA4D */
  0xF948,   /* U+FA4E */
  0xF950,   /* U+FA4F */
  0xF956,   /* U+FA50 */
  0xF95D,   /* U+FA51 */
  0xF98D,   /* U+FA52 */
  0xF98E,   /* U+FA53 */
  0xFA40,   /* U+FA54 */
  0xFA81,   /* U+FA55 */
  0xFBC0,   /* U+FA56 */
  0xFDF4,   /* U+FA57 */
  0xFE09,   /* U+FA58 */
  0xFE41,   /* U+FA59 */
  0xFF72,   /* U+FA5A */
  0x8005,   /* U+FA5B */
  0x81ED,   /* U+FA5C */
  0x8279,   /* U+FA5D */
  0x8279,   /* U+FA5E */
  0x8457,   /* U+FA5F */
  0x8910,   /* U+FA60 */
  0x8996,   /* U+FA61 */
  0x8B01,   /* U+FA62 */
  0x8B39,   /* U+FA63 */
  0x8CD3,   /* U+FA64 */
  0x8D08,   /* U+FA65 */
  0x8FB6,   /* U+FA66 */
  0x9038,   /* U+FA67 */
  0x96E3,   /* U+FA68 */
  0x97FF,   /* U+FA69 */
  0x983B,   /* U+FA6A */
  0xE075,   /* U+FA6B */
  0xC2EE,   /* U+FA6C */
  0x8218,   /* U+FA6D */
  0xFA6E,   /* U+FA6E */
  0xFA6F,   /* U+FA6F */
  0xCE26,   /* U+FA70 */
  0xD1B5,   /* U+FA71 */
  0xD168,   /* U+FA72 */
  0xCF80,   /* U+FA73 */
  0xD145,   /* U+FA74 */
  0xD180,   /* U+FA75 */
  0xD2C7,   /* U+FA76 */
  0xD2FA,   /* U+FA77 */
  0xD59D,   /* U+FA78 */
  0xD555,   /* U+FA79 */
  0xD599,   /* U+FA7A */
  0xD5E2,   /* U+FA7B */
  0xD85A,   /* U+FA7C */
  0xD8B3,   /* U+FA7D */
  0xD944,   /* U+FA7E */
  0xD954,   /* U+FA7F */
  0xDA62,   /* U+FA80 */
  0xDB28,   /* U+FA81 */
  0xDED2,   /* U+FA82 */
  0xDED9,   /* U+FA83 */
  0xDF69,   /* U+FA84 */
  0xDFAD,   /* U+FA85 */
  0xE0D8,   /* U+FA86 */
  0xE14E,   /* U+FA87 */
  0xE108,   /* U+FA88 */
  0xE18E,   /* U+FA89 */
  0xE160,   /* U+FA8A */
  0xE1F2,   /* U+FA8B */
  0xE234,   /* U+FA8C */
  0xE3C4,   /* U+FA8D */
  0xE41C,   /* U+FA8E */
  0xE452,   /* U+FA8F */
  0xE556,   /* U+FA90 */
  0xE674,   /* U+FA91 */
  0xE717,   /* U+FA92 */
  0xE71B,   /* U+FA93 */
  0xE756,   /* U+FA94 */
  0xEB79,   /* U+FA95 */
  0xEBBA,   /* U+FA96 */
  0xED41,   /* U+FA97 */
  0xEEDB,   /* U+FA98 */
  0xEECB,   /* U+FA99 */
  0xEF22,   /* U+FA9A */
  0xF01E,   /* U+FA9B */
  0xF16E,   /* U+FA9C */
  0xF7A7,   /* U+FA9D */
  0xF235,   /* U+FA9E */
  0xF2AF,   /* U+FA9F */
  0xF32A,   /* U+FAA0 */
  0xF471,   /* U+FAA1 */
  0xF506,   /* U+FAA2 */
  0xF53B,   /* U+FAA3 */
  0xF61D,   /* U+FAA4 */
  0xF61F,   /* U+FAA5 */
  0xF6CA,   /* U+FAA6 */
  0xF6DB,   /* U+FAA7 */
  0xF6F4,   /* U+FAA8 */
  0xF74A,   /* U+FAA9 */
  0xF740,   /* U+FAAA */
  0xF8CC,   /* U+FAAB */
  0xFAB1,   /* U+FAAC */
  0xFBC0,   /* U+FAAD */
  0xFC7B,   /* U+FAAE */
  0xFD5B,   /* U+FAAF */
  0xFDF4,   /* U+FAB0 */
  0xFF3E,   /* U+FAB1 */
  0x8005,   /* U+FAB2 */
  0x8352,   /* U+FAB3 */
  0x83EF,   /* U+FAB4 */
  0x8779,   /* U+FAB5 */
  0x8941,   /* U+FAB6 */
  0x8986,   /* U+FAB7 */
  0x8996,   /* U+FAB8 */
  0x8ABF,   /* U+FAB9 */
  0x8AF8,   /* U+FABA */
  0x8ACB,   /* U+FABB */
  0x8B01,   /* U+FABC */
  0x8AFE,   /* U+FABD */
  0x8AED,   /* U+FABE */
  0x8B39,   /* U+FABF */
  0x8B8A,   /* U+FAC0 */
  0x8D08,   /* U+FAC1 */
  0x8F38,   /* U+FAC2 */
  0x9072,   /* U+FAC3 */
  0x9199,   /* U+FAC4 */
  0x9276,   /* U+FAC5 */
  0x967C,   /* U+FAC6 */
  0x96E3,   /* U+FAC7 */
  0x9756,   /* U+FAC8 */
  0x97DB,   /* U+FAC9 */
  0x97FF,   /* U+FACA */
  0x980B,   /* U+FACB */
  0x983B,   /* U+FACC */
  0x9B12,   /* U+FACD */
  0x9F9C,   /* U+FACE */
  0xA84A,   /* U+FACF */
  0xA844,   /* U+FAD0 */
  0xB3D5,   /* U+FAD1 */
  0xBB9D,   /* U+FAD2 */
  0xC018,   /* U+FAD3 */
  0xC039,   /* U+FAD4 */
  0xD249,   /* U+FAD5 */
  0xDCD0,   /* U+FAD6 */
  0xFED3,   /* U+FAD7 */
  0x9F43,   /* U+FAD8 */
  0x9F8E,   /* U+FAD9 */
  0xFADA,   /* U+FADA */
  0xFADB,   /* U+FADB */
  0xFADC,   /* U+FADC */
  0xFADD,   /* U+FADD */
  0xFADE,   /* U+FADE */
  0xFADF,   /* U+FADF */
  0xFAE0,   /* U+FAE0 */
  0xFAE1,   /* U+FAE1 */
  0xFAE2,   /* U+FAE2 */
  0xFAE3,   /* U+FAE3 */
  0xFAE4,   /* U+FAE4 */
  0xFAE5,   /* U+FAE5 */
  0xFAE6,   /* U+FAE6 */
  0xFAE7,   /* U+FAE7 */
  0xFAE8,   /* U+FAE8 */
  0xFAE9,   /* U+FAE9 */
  0xFAEA,   /* U+FAEA */
  0xFAEB,   /* U+FAEB */
  0xFAEC,   /* U+FAEC */
  0xFAED,   /* U+FAED */
  0xFAEE,   /* U+FAEE */
  0xFAEF,   /* U+FAEF */
  0xFAF0,   /* U+FAF0 */
  0xFAF1,   /* U+FAF1 */
  0xFAF2,   /* U+FAF2 */
  0xFAF3,   /* U+FAF3 */
  0xFAF4,   /* U+FAF4 */
  0xFAF5,   /* U+FAF5 */
  0xFAF6,   /* U+FAF6 */
  0xFAF7,   /* U+FAF7 */
  0xFAF8,   /* U+FAF8 */
  0xFAF9,   /* U+FAF9 */
  0xFAFA,   /* U+FAFA */
  0xFAFB,   /* U+FAFB */
  0xFAFC,   /* U+FAFC */
  0xFAFD,   /* U+FAFD */
  0xFAFE,   /* U+FAFE */
  0xFAFF,   /* U+FAFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+FA00 */
  0x0000,   /* U+FA01 */
  0x0000,   /* U+FA02 */
  0x0000,   /* U+FA03 */
  0x0000,   /* U+FA04 */
  0x0000,   /* U+FA05 */
  0x0000,   /* U+FA06 */
  0x0000,   /* U+FA07 */
  0x0000,   /* U+FA08 */
  0x0000,   /* U+FA09 */
  0x0000,   /* U+FA0A */
  0x0000,   /* U+FA0B */
  0x0000,   /* U+FA0C */
  0x0000,   /* U+FA0D */
  0x0000,   /* U+FA0E */
  0x0000,   /* U+FA0F */
  0x0000,   /* U+FA10 */
  0x0000,   /* U+FA11 */
  0x0000,   /* U+FA12 */
  0x0000,   /* U+FA13 */
  0x0000,   /* U+FA14 */
  0x0000,   /* U+FA15 */
  0x0000,   /* U+FA16 */
  0x0000,   /* U+FA17 */
  0x0000,   /* U+FA18 */
  0x0000,   /* U+FA19 */
  0x0000,   /* U+FA1A */
  0x0000,   /* U+FA1B */
  0x0000,   /* U+FA1C */
  0x0000,   /* U+FA1D */
  0x0000,   /* U+FA1E */
  0x0000,   /* U+FA1F */
  0x0000,   /* U+FA20 */
  0x0000,   /* U+FA21 */
  0x0000,   /* U+FA22 */
  0x0000,   /* U+FA23 */
  0x0000,   /* U+FA24 */
  0x0000,   /* U+FA25 */
  0x0000,   /* U+FA26 */
  0x0000,   /* U+FA27 */
  0x0000,   /* U+FA28 */
  0x0000,   /* U+FA29 */
  0x0000,   /* U+FA2A */
  0x0000,   /* U+FA2B */
  0x0000,   /* U+FA2C */
  0x0000,   /* U+FA2D */
  0x0000,   /* U+FA2E */
  0x0000,   /* U+FA2F */
  0x0000,   /* U+FA30 */
  0x0000,   /* U+FA31 */
  0x0000,   /* U+FA32 */
  0x0000,   /* U+FA33 */
  0x0000,   /* U+FA34 */
  0x0000,   /* U+FA35 */
  0x0000,   /* U+FA36 */
  0x0000,   /* U+FA37 */
  0x0000,   /* U+FA38 */
  0x0000,   /* U+FA39 */
  0x0000,   /* U+FA3A */
  0x0000,   /* U+FA3B */
  0x0000,   /* U+FA3C */
  0x0000,   /* U+FA3D */
  0x0000,   /* U+FA3E */
  0x0000,   /* U+FA3F */
  0x0000,   /* U+FA40 */
  0x0000,   /* U+FA41 */
  0x0000,   /* U+FA42 */
  0x0000,   /* U+FA43 */
  0x0000,   /* U+FA44 */
  0x0000,   /* U+FA45 */
  0x0000,   /* U+FA46 */
  0x0000,   /* U+FA47 */
  0x0000,   /* U+FA48 */
  0x0000,   /* U+FA49 */
  0x0000,   /* U+FA4A */
  0x0000,   /* U+FA4B */
  0x0000,   /* U+FA4C */
  0x0000,   /* U+FA4D */
  0x0000,   /* U+FA4E */
  0x0000,   /* U+FA4F */
  0x0000,   /* U+FA50 */
  0x0000,   /* U+FA51 */
  0x0000,   /* U+FA52 */
  0x0000,   /* U+FA53 */
  0x0000,   /* U+FA54 */
  0x0000,   /* U+FA55 */
  0x0000,   /* U+FA56 */
  0x0000,   /* U+FA57 */
  0x0000,   /* U+FA58 */
  0x0000,   /* U+FA59 */
  0x0000,   /* U+FA5A */
  0x0000,   /* U+FA5B */
  0x0000,   /* U+FA5C */
  0x0000,   /* U+FA5D */
  0x0000,   /* U+FA5E */
  0x0000,   /* U+FA5F */
  0x0000,   /* U+FA60 */
  0x0000,   /* U+FA61 */
  0x0000,   /* U+FA62 */
  0x0000,   /* U+FA63 */
  0x0000,   /* U+FA64 */
  0x0000,   /* U+FA65 */
  0x0000,   /* U+FA66 */
  0x0000,   /* U+FA67 */
  0x0000,   /* U+FA68 */
  0x0000,   /* U+FA69 */
  0x0000,   /* U+FA6A */
  0x0000,   /* U+FA6B */
  0x0000,   /* U+FA6C */
  0x0000,   /* U+FA6D */
  0x0000,   /* U+FA6E */
  0x0000,   /* U+FA6F */
  0x0000,   /* U+FA70 */
  0x0000,   /* U+FA71 */
  0x0000,   /* U+FA72 */
  0x0000,   /* U+FA73 */
  0x0000,   /* U+FA74 */
  0x0000,   /* U+FA75 */
  0x0000,   /* U+FA76 */
  0x0000,   /* U+FA77 */
  0x0000,   /* U+FA78 */
  0x0000,   /* U+FA79 */
  0x0000,   /* U+FA7A */
  0x0000,   /* U+FA7B */
  0x0000,   /* U+FA7C */
  0x0000,   /* U+FA7D */
  0x0000,   /* U+FA7E */
  0x0000,   /* U+FA7F */
  0x0000,   /* U+FA80 */
  0x0000,   /* U+FA81 */
  0x0000,   /* U+FA82 */
  0x0000,   /* U+FA83 */
  0x0000,   /* U+FA84 */
  0x0000,   /* U+FA85 */
  0x0000,   /* U+FA86 */
  0x0000,   /* U+FA87 */
  0x0000,   /* U+FA88 */
  0x0000,   /* U+FA89 */
  0x0000,   /* U+FA8A */
  0x0000,   /* U+FA8B */
  0x0000,   /* U+FA8C */
  0x0000,   /* U+FA8D */
  0x0000,   /* U+FA8E */
  0x0000,   /* U+FA8F */
  0x0000,   /* U+FA90 */
  0x0000,   /* U+FA91 */
  0x0000,   /* U+FA92 */
  0x0000,   /* U+FA93 */
  0x0000,   /* U+FA94 */
  0x0000,   /* U+FA95 */
  0x0000,   /* U+FA96 */
  0x0000,   /* U+FA97 */
  0x0000,   /* U+FA98 */
  0x0000,   /* U+FA99 */
  0x0000,   /* U+FA9A */
  0x0000,   /* U+FA9B */
  0x0000,   /* U+FA9C */
  0x0000,   /* U+FA9D */
  0x0000,   /* U+FA9E */
  0x0000,   /* U+FA9F */
  0x0000,   /* U+FAA0 */
  0x0000,   /* U+FAA1 */
  0x0000,   /* U+FAA2 */
  0x0000,   /* U+FAA3 */
  0x0000,   /* U+FAA4 */
  0x0000,   /* U+FAA5 */
  0x0000,   /* U+FAA6 */
  0x0000,   /* U+FAA7 */
  0x0000,   /* U+FAA8 */
  0x0000,   /* U+FAA9 */
  0x0000,   /* U+FAAA */
  0x0000,   /* U+FAAB */
  0x0000,   /* U+FAAC */
  0x0000,   /* U+FAAD */
  0x0000,   /* U+FAAE */
  0x0000,   /* U+FAAF */
  0x0000,   /* U+FAB0 */
  0x0000,   /* U+FAB1 */
  0x0000,   /* U+FAB2 */
  0x0000,   /* U+FAB3 */
  0x0000,   /* U+FAB4 */
  0x0000,   /* U+FAB5 */
  0x0000,   /* U+FAB6 */
  0x0000,   /* U+FAB7 */
  0x0000,   /* U+FAB8 */
  0x0000,   /* U+FAB9 */
  0x0000,   /* U+FABA */
  0x0000,   /* U+FABB */
  0x0000,   /* U+FABC */
  0x0000,   /* U+FABD */
  0x0000,   /* U+FABE */
  0x0000,   /* U+FABF */
  0x0000,   /* U+FAC0 */
  0x0000,   /* U+FAC1 */
  0x0000,   /* U+FAC2 */
  0x0000,   /* U+FAC3 */
  0x0000,   /* U+FAC4 */
  0x0000,   /* U+FAC5 */
  0x0000,   /* U+FAC6 */
  0x0000,   /* U+FAC7 */
  0x0000,   /* U+FAC8 */
  0x0000,   /* U+FAC9 */
  0x0000,   /* U+FACA */
  0x0000,   /* U+FACB */
  0x0000,   /* U+FACC */
  0x0000,   /* U+FACD */
  0x0000,   /* U+FACE */
  0x0000,   /* U+FACF */
  0x0000,   /* U+FAD0 */
  0x0000,   /* U+FAD1 */
  0x0000,   /* U+FAD2 */
  0x0000,   /* U+FAD3 */
  0x0000,   /* U+FAD4 */
  0x0000,   /* U+FAD5 */
  0x0000,   /* U+FAD6 */
  0x0000,   /* U+FAD7 */
  0x0000,   /* U+FAD8 */
  0x0000,   /* U+FAD9 */
  0x0000,   /* U+FADA */
  0x0000,   /* U+FADB */
  0x0000,   /* U+FADC */
  0x0000,   /* U+FADD */
  0x0000,   /* U+FADE */
  0x0000,   /* U+FADF */
  0x0000,   /* U+FAE0 */
  0x0000,   /* U+FAE1 */
  0x0000,   /* U+FAE2 */
  0x0000,   /* U+FAE3 */
  0x0000,   /* U+FAE4 */
  0x0000,   /* U+FAE5 */
  0x0000,   /* U+FAE6 */
  0x0000,   /* U+FAE7 */
  0x0000,   /* U+FAE8 */
  0x0000,   /* U+FAE9 */
  0x0000,   /* U+FAEA */
  0x0000,   /* U+FAEB */
  0x0000,   /* U+FAEC */
  0x0000,   /* U+FAED */
  0x0000,   /* U+FAEE */
  0x0000,   /* U+FAEF */
  0x0000,   /* U+FAF0 */
  0x0000,   /* U+FAF1 */
  0x0000,   /* U+FAF2 */
  0x0000,   /* U+FAF3 */
  0x0000,   /* U+FAF4 */
  0x0000,   /* U+FAF5 */
  0x0000,   /* U+FAF6 */
  0x0000,   /* U+FAF7 */
  0x0000,   /* U+FAF8 */
  0x0000,   /* U+FAF9 */
  0x0000,   /* U+FAFA */
  0x0000,   /* U+FAFB */
  0x0000,   /* U+FAFC */
  0x0000,   /* U+FAFD */
  0x0000,   /* U+FAFE */
  0x0000,   /* U+FAFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+FA00 */
  0x0000,   /* U+FA01 */
  0x0000,   /* U+FA02 */
  0x0000,   /* U+FA03 */
  0x0000,   /* U+FA04 */
  0x0000,   /* U+FA05 */
  0x0000,   /* U+FA06 */
  0x0000,   /* U+FA07 */
  0x0000,   /* U+FA08 */
  0x0000,   /* U+FA09 */
  0x0000,   /* U+FA0A */
  0x0000,   /* U+FA0B */
  0x0000,   /* U+FA0C */
  0x0000,   /* U+FA0D */
  0x0000,   /* U+FA0E */
  0x0000,   /* U+FA0F */
  0x0000,   /* U+FA10 */
  0x0000,   /* U+FA11 */
  0x0000,   /* U+FA12 */
  0x0000,   /* U+FA13 */
  0x0000,   /* U+FA14 */
  0x0000,   /* U+FA15 */
  0x0000,   /* U+FA16 */
  0x0000,   /* U+FA17 */
  0x0000,   /* U+FA18 */
  0x0000,   /* U+FA19 */
  0x0000,   /* U+FA1A */
  0x0000,   /* U+FA1B */
  0x0000,   /* U+FA1C */
  0x0000,   /* U+FA1D */
  0x0000,   /* U+FA1E */
  0x0000,   /* U+FA1F */
  0x0000,   /* U+FA20 */
  0x0000,   /* U+FA21 */
  0x0000,   /* U+FA22 */
  0x0000,   /* U+FA23 */
  0x0000,   /* U+FA24 */
  0x0000,   /* U+FA25 */
  0x0000,   /* U+FA26 */
  0x0000,   /* U+FA27 */
  0x0000,   /* U+FA28 */
  0x0000,   /* U+FA29 */
  0x0000,   /* U+FA2A */
  0x0000,   /* U+FA2B */
  0x0000,   /* U+FA2C */
  0x0000,   /* U+FA2D */
  0x0000,   /* U+FA2E */
  0x0000,   /* U+FA2F */
  0x0000,   /* U+FA30 */
  0x0000,   /* U+FA31 */
  0x0000,   /* U+FA32 */
  0x0000,   /* U+FA33 */
  0x0000,   /* U+FA34 */
  0x0000,   /* U+FA35 */
  0x0000,   /* U+FA36 */
  0x0000,   /* U+FA37 */
  0x0000,   /* U+FA38 */
  0x0000,   /* U+FA39 */
  0x0000,   /* U+FA3A */
  0x0000,   /* U+FA3B */
  0x0000,   /* U+FA3C */
  0x0000,   /* U+FA3D */
  0x0000,   /* U+FA3E */
  0x0000,   /* U+FA3F */
  0x0000,   /* U+FA40 */
  0x0000,   /* U+FA41 */
  0x0000,   /* U+FA42 */
  0x0000,   /* U+FA43 */
  0x0000,   /* U+FA44 */
  0x0000,   /* U+FA45 */
  0x0000,   /* U+FA46 */
  0x0000,   /* U+FA47 */
  0x0000,   /* U+FA48 */
  0x0000,   /* U+FA49 */
  0x0000,   /* U+FA4A */
  0x0000,   /* U+FA4B */
  0x0000,   /* U+FA4C */
  0x0000,   /* U+FA4D */
  0x0000,   /* U+FA4E */
  0x0000,   /* U+FA4F */
  0x0000,   /* U+FA50 */
  0x0000,   /* U+FA51 */
  0x0000,   /* U+FA52 */
  0x0000,   /* U+FA53 */
  0x0000,   /* U+FA54 */
  0x0000,   /* U+FA55 */
  0x0000,   /* U+FA56 */
  0x0000,   /* U+FA57 */
  0x0000,   /* U+FA58 */
  0x0000,   /* U+FA59 */
  0x0000,   /* U+FA5A */
  0x0000,   /* U+FA5B */
  0x0000,   /* U+FA5C */
  0x0000,   /* U+FA5D */
  0x0000,   /* U+FA5E */
  0x0000,   /* U+FA5F */
  0x0000,   /* U+FA60 */
  0x0000,   /* U+FA61 */
  0x0000,   /* U+FA62 */
  0x0000,   /* U+FA63 */
  0x0000,   /* U+FA64 */
  0x0000,   /* U+FA65 */
  0x0000,   /* U+FA66 */
  0x0000,   /* U+FA67 */
  0x0000,   /* U+FA68 */
  0x0000,   /* U+FA69 */
  0x0000,   /* U+FA6A */
  0x0000,   /* U+FA6B */
  0x0000,   /* U+FA6C */
  0x0000,   /* U+FA6D */
  0x0000,   /* U+FA6E */
  0x0000,   /* U+FA6F */
  0x0000,   /* U+FA70 */
  0x0000,   /* U+FA71 */
  0x0000,   /* U+FA72 */
  0x0000,   /* U+FA73 */
  0x0000,   /* U+FA74 */
  0x0000,   /* U+FA75 */
  0x0000,   /* U+FA76 */
  0x0000,   /* U+FA77 */
  0x0000,   /* U+FA78 */
  0x0000,   /* U+FA79 */
  0x0000,   /* U+FA7A */
  0x0000,   /* U+FA7B */
  0x0000,   /* U+FA7C */
  0x0000,   /* U+FA7D */
  0x0000,   /* U+FA7E */
  0x0000,   /* U+FA7F */
  0x0000,   /* U+FA80 */
  0x0000,   /* U+FA81 */
  0x0000,   /* U+FA82 */
  0x0000,   /* U+FA83 */
  0x0000,   /* U+FA84 */
  0x0000,   /* U+FA85 */
  0x0000,   /* U+FA86 */
  0x0000,   /* U+FA87 */
  0x0000,   /* U+FA88 */
  0x0000,   /* U+FA89 */
  0x0000,   /* U+FA8A */
  0x0000,   /* U+FA8B */
  0x0000,   /* U+FA8C */
  0x0000,   /* U+FA8D */
  0x0000,   /* U+FA8E */
  0x0000,   /* U+FA8F */
  0x0000,   /* U+FA90 */
  0x0000,   /* U+FA91 */
  0x0000,   /* U+FA92 */
  0x0000,   /* U+FA93 */
  0x0000,   /* U+FA94 */
  0x0000,   /* U+FA95 */
  0x0000,   /* U+FA96 */
  0x0000,   /* U+FA97 */
  0x0000,   /* U+FA98 */
  0x0000,   /* U+FA99 */
  0x0000,   /* U+FA9A */
  0x0000,   /* U+FA9B */
  0x0000,   /* U+FA9C */
  0x0000,   /* U+FA9D */
  0x0000,   /* U+FA9E */
  0x0000,   /* U+FA9F */
  0x0000,   /* U+FAA0 */
  0x0000,   /* U+FAA1 */
  0x0000,   /* U+FAA2 */
  0x0000,   /* U+FAA3 */
  0x0000,   /* U+FAA4 */
  0x0000,   /* U+FAA5 */
  0x0000,   /* U+FAA6 */
  0x0000,   /* U+FAA7 */
  0x0000,   /* U+FAA8 */
  0x0000,   /* U+FAA9 */
  0x0000,   /* U+FAAA */
  0x0000,   /* U+FAAB */
  0x0000,   /* U+FAAC */
  0x0000,   /* U+FAAD */
  0x0000,   /* U+FAAE */
  0x0000,   /* U+FAAF */
  0x0000,   /* U+FAB0 */
  0x0000,   /* U+FAB1 */
  0x0000,   /* U+FAB2 */
  0x0000,   /* U+FAB3 */
  0x0000,   /* U+FAB4 */
  0x0000,   /* U+FAB5 */
  0x0000,   /* U+FAB6 */
  0x0000,   /* U+FAB7 */
  0x0000,   /* U+FAB8 */
  0x0000,   /* U+FAB9 */
  0x0000,   /* U+FABA */
  0x0000,   /* U+FABB */
  0x0000,   /* U+FABC */
  0x0000,   /* U+FABD */
  0x0000,   /* U+FABE */
  0x0000,   /* U+FABF */
  0x0000,   /* U+FAC0 */
  0x0000,   /* U+FAC1 */
  0x0000,   /* U+FAC2 */
  0x0000,   /* U+FAC3 */
  0x0000,   /* U+FAC4 */
  0x0000,   /* U+FAC5 */
  0x0000,   /* U+FAC6 */
  0x0000,   /* U+FAC7 */
  0x0000,   /* U+FAC8 */
  0x0000,   /* U+FAC9 */
  0x0000,   /* U+FACA */
  0x0000,   /* U+FACB */
  0x0000,   /* U+FACC */
  0x0000,   /* U+FACD */
  0x0000,   /* U+FACE */
  0x0000,   /* U+FACF */
  0x0000,   /* U+FAD0 */
  0x0000,   /* U+FAD1 */
  0x0000,   /* U+FAD2 */
  0x0000,   /* U+FAD3 */
  0x0000,   /* U+FAD4 */
  0x0000,   /* U+FAD5 */
  0x0000,   /* U+FAD6 */
  0x0000,   /* U+FAD7 */
  0x0000,   /* U+FAD8 */
  0x0000,   /* U+FAD9 */
  0x0000,   /* U+FADA */
  0x0000,   /* U+FADB */
  0x0000,   /* U+FADC */
  0x0000,   /* U+FADD */
  0x0000,   /* U+FADE */
  0x0000,   /* U+FADF */
  0x0000,   /* U+FAE0 */
  0x0000,   /* U+FAE1 */
  0x0000,   /* U+FAE2 */
  0x0000,   /* U+FAE3 */
  0x0000,   /* U+FAE4 */
  0x0000,   /* U+FAE5 */
  0x0000,   /* U+FAE6 */
  0x0000,   /* U+FAE7 */
  0x0000,   /* U+FAE8 */
  0x0000,   /* U+FAE9 */
  0x0000,   /* U+FAEA */
  0x0000,   /* U+FAEB */
  0x0000,   /* U+FAEC */
  0x0000,   /* U+FAED */
  0x0000,   /* U+FAEE */
  0x0000,   /* U+FAEF */
  0x0000,   /* U+FAF0 */
  0x0000,   /* U+FAF1 */
  0x0000,   /* U+FAF2 */
  0x0000,   /* U+FAF3 */
  0x0000,   /* U+FAF4 */
  0x0000,   /* U+FAF5 */
  0x0000,   /* U+FAF6 */
  0x0000,   /* U+FAF7 */
  0x0000,   /* U+FAF8 */
  0x0000,   /* U+FAF9 */
  0x0000,   /* U+FAFA */
  0x0000,   /* U+FAFB */
  0x0000,   /* U+FAFC */
  0x0000,   /* U+FAFD */
  0x0000,   /* U+FAFE */
  0x0000,   /* U+FAFF */
};

uint16 uca900_p0FB[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 3, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 3, 3, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1CE5,   /* U+FB00 */
  0x1CE5,   /* U+FB01 */
  0x1CE5,   /* U+FB02 */
  0x1CE5,   /* U+FB03 */
  0x1CE5,   /* U+FB04 */
  0x1E71,   /* U+FB05 */
  0x1E71,   /* U+FB06 */
  0xFBC1,   /* U+FB07 */
  0xFBC1,   /* U+FB08 */
  0xFBC1,   /* U+FB09 */
  0xFBC1,   /* U+FB0A */
  0xFBC1,   /* U+FB0B */
  0xFBC1,   /* U+FB0C */
  0xFBC1,   /* U+FB0D */
  0xFBC1,   /* U+FB0E */
  0xFBC1,   /* U+FB0F */
  0xFBC1,   /* U+FB10 */
  0xFBC1,   /* U+FB11 */
  0xFBC1,   /* U+FB12 */
  0x22A3,   /* U+FB13 */
  0x22A3,   /* U+FB14 */
  0x22A3,   /* U+FB15 */
  0x22AD,   /* U+FB16 */
  0x22A3,   /* U+FB17 */
  0xFBC1,   /* U+FB18 */
  0xFBC1,   /* U+FB19 */
  0xFBC1,   /* U+FB1A */
  0xFBC1,   /* U+FB1B */
  0xFBC1,   /* U+FB1C */
  0x22C0,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x22C0,   /* U+FB1F */
  0x22C6,   /* U+FB20 */
  0x22B7,   /* U+FB21 */
  0x22BA,   /* U+FB22 */
  0x22BB,   /* U+FB23 */
  0x22C1,   /* U+FB24 */
  0x22C2,   /* U+FB25 */
  0x22C3,   /* U+FB26 */
  0x22CA,   /* U+FB27 */
  0x22CC,   /* U+FB28 */
  0x0616,   /* U+FB29 */
  0x22CB,   /* U+FB2A */
  0x22CB,   /* U+FB2B */
  0x22CB,   /* U+FB2C */
  0x22CB,   /* U+FB2D */
  0x22B7,   /* U+FB2E */
  0x22B7,   /* U+FB2F */
  0x22B7,   /* U+FB30 */
  0x22B8,   /* U+FB31 */
  0x22B9,   /* U+FB32 */
  0x22BA,   /* U+FB33 */
  0x22BB,   /* U+FB34 */
  0x22BC,   /* U+FB35 */
  0x22BD,   /* U+FB36 */
  0xFBC1,   /* U+FB37 */
  0x22BF,   /* U+FB38 */
  0x22C0,   /* U+FB39 */
  0x22C1,   /* U+FB3A */
  0x22C1,   /* U+FB3B */
  0x22C2,   /* U+FB3C */
  0xFBC1,   /* U+FB3D */
  0x22C3,   /* U+FB3E */
  0xFBC1,   /* U+FB3F */
  0x22C4,   /* U+FB40 */
  0x22C5,   /* U+FB41 */
  0xFBC1,   /* U+FB42 */
  0x22C7,   /* U+FB43 */
  0x22C7,   /* U+FB44 */
  0xFBC1,   /* U+FB45 */
  0x22C8,   /* U+FB46 */
  0x22C9,   /* U+FB47 */
  0x22CA,   /* U+FB48 */
  0x22CB,   /* U+FB49 */
  0x22CC,   /* U+FB4A */
  0x22BC,   /* U+FB4B */
  0x22B8,   /* U+FB4C */
  0x22C1,   /* U+FB4D */
  0x22C7,   /* U+FB4E */
  0x22B7,   /* U+FB4F */
  0x2301,   /* U+FB50 */
  0x2301,   /* U+FB51 */
  0x230E,   /* U+FB52 */
  0x230E,   /* U+FB53 */
  0x230E,   /* U+FB54 */
  0x230E,   /* U+FB55 */
  0x230F,   /* U+FB56 */
  0x230F,   /* U+FB57 */
  0x230F,   /* U+FB58 */
  0x230F,   /* U+FB59 */
  0x2310,   /* U+FB5A */
  0x2310,   /* U+FB5B */
  0x2310,   /* U+FB5C */
  0x2310,   /* U+FB5D */
  0x2320,   /* U+FB5E */
  0x2320,   /* U+FB5F */
  0x2320,   /* U+FB60 */
  0x2320,   /* U+FB61 */
  0x2323,   /* U+FB62 */
  0x2323,   /* U+FB63 */
  0x2323,   /* U+FB64 */
  0x2323,   /* U+FB65 */
  0x231F,   /* U+FB66 */
  0x231F,   /* U+FB67 */
  0x231F,   /* U+FB68 */
  0x231F,   /* U+FB69 */
  0x237B,   /* U+FB6A */
  0x237B,   /* U+FB6B */
  0x237B,   /* U+FB6C */
  0x237B,   /* U+FB6D */
  0x237E,   /* U+FB6E */
  0x237E,   /* U+FB6F */
  0x237E,   /* U+FB70 */
  0x237E,   /* U+FB71 */
  0x2327,   /* U+FB72 */
  0x2327,   /* U+FB73 */
  0x2327,   /* U+FB74 */
  0x2327,   /* U+FB75 */
  0x2326,   /* U+FB76 */
  0x2326,   /* U+FB77 */
  0x2326,   /* U+FB78 */
  0x2326,   /* U+FB79 */
  0x2328,   /* U+FB7A */
  0x2328,   /* U+FB7B */
  0x2328,   /* U+FB7C */
  0x2328,   /* U+FB7D */
  0x232A,   /* U+FB7E */
  0x232A,   /* U+FB7F */
  0x232A,   /* U+FB80 */
  0x232A,   /* U+FB81 */
  0x233E,   /* U+FB82 */
  0x233E,   /* U+FB83 */
  0x233D,   /* U+FB84 */
  0x233D,   /* U+FB85 */
  0x2340,   /* U+FB86 */
  0x2340,   /* U+FB87 */
  0x2339,   /* U+FB88 */
  0x2339,   /* U+FB89 */
  0x234F,   /* U+FB8A */
  0x234F,   /* U+FB8B */
  0x2348,   /* U+FB8C */
  0x2348,   /* U+FB8D */
  0x2388,   /* U+FB8E */
  0x2388,   /* U+FB8F */
  0x2388,   /* U+FB90 */
  0x2388,   /* U+FB91 */
  0x2390,   /* U+FB92 */
  0x2390,   /* U+FB93 */
  0x2390,   /* U+FB94 */
  0x2390,   /* U+FB95 */
  0x2395,   /* U+FB96 */
  0x2395,   /* U+FB97 */
  0x2395,   /* U+FB98 */
  0x2395,   /* U+FB99 */
  0x2393,   /* U+FB9A */
  0x2393,   /* U+FB9B */
  0x2393,   /* U+FB9C */
  0x2393,   /* U+FB9D */
  0x23A8,   /* U+FB9E */
  0x23A8,   /* U+FB9F */
  0x23AA,   /* U+FBA0 */
  0x23AA,   /* U+FBA1 */
  0x23AA,   /* U+FBA2 */
  0x23AA,   /* U+FBA3 */
  0x23B6,   /* U+FBA4 */
  0x23B6,   /* U+FBA5 */
  0x23B3,   /* U+FBA6 */
  0x23B3,   /* U+FBA7 */
  0x23B3,   /* U+FBA8 */
  0x23B3,   /* U+FBA9 */
  0x23B2,   /* U+FBAA */
  0x23B2,   /* U+FBAB */
  0x23B2,   /* U+FBAC */
  0x23B2,   /* U+FBAD */
  0x23D4,   /* U+FBAE */
  0x23D4,   /* U+FBAF */
  0x23D4,   /* U+FBB0 */
  0x23D4,   /* U+FBB1 */
  0x0502,   /* U+FBB2 */
  0x0503,   /* U+FBB3 */
  0x0504,   /* U+FBB4 */
  0x0505,   /* U+FBB5 */
  0x0506,   /* U+FBB6 */
  0x0507,   /* U+FBB7 */
  0x0508,   /* U+FBB8 */
  0x0509,   /* U+FBB9 */
  0x050A,   /* U+FBBA */
  0x050B,   /* U+FBBB */
  0x050C,   /* U+FBBC */
  0x050D,   /* U+FBBD */
  0x050E,   /* U+FBBE */
  0x050F,   /* U+FBBF */
  0x0510,   /* U+FBC0 */
  0x0511,   /* U+FBC1 */
  0xFBC1,   /* U+FBC2 */
  0xFBC1,   /* U+FBC3 */
  0xFBC1,   /* U+FBC4 */
  0xFBC1,   /* U+FBC5 */
  0xFBC1,   /* U+FBC6 */
  0xFBC1,   /* U+FBC7 */
  0xFBC1,   /* U+FBC8 */
  0xFBC1,   /* U+FBC9 */
  0xFBC1,   /* U+FBCA */
  0xFBC1,   /* U+FBCB */
  0xFBC1,   /* U+FBCC */
  0xFBC1,   /* U+FBCD */
  0xFBC1,   /* U+FBCE */
  0xFBC1,   /* U+FBCF */
  0xFBC1,   /* U+FBD0 */
  0xFBC1,   /* U+FBD1 */
  0xFBC1,   /* U+FBD2 */
  0x238D,   /* U+FBD3 */
  0x238D,   /* U+FBD4 */
  0x238D,   /* U+FBD5 */
  0x238D,   /* U+FBD6 */
  0x23BB,   /* U+FBD7 */
  0x23BB,   /* U+FBD8 */
  0x23BA,   /* U+FBD9 */
  0x23BA,   /* U+FBDA */
  0x23BC,   /* U+FBDB */
  0x23BC,   /* U+FBDC */
  0x23BB,   /* U+FBDD */
  0x23BF,   /* U+FBDE */
  0x23BF,   /* U+FBDF */
  0x23B9,   /* U+FBE0 */
  0x23B9,   /* U+FBE1 */
  0x23BD,   /* U+FBE2 */
  0x23BD,   /* U+FBE3 */
  0x23CA,   /* U+FBE4 */
  0x23CA,   /* U+FBE5 */
  0x23CA,   /* U+FBE6 */
  0x23CA,   /* U+FBE7 */
  0x23C5,   /* U+FBE8 */
  0x23C5,   /* U+FBE9 */
  0x2307,   /* U+FBEA */
  0x2307,   /* U+FBEB */
  0x2307,   /* U+FBEC */
  0x2307,   /* U+FBED */
  0x2307,   /* U+FBEE */
  0x2307,   /* U+FBEF */
  0x2307,   /* U+FBF0 */
  0x2307,   /* U+FBF1 */
  0x2307,   /* U+FBF2 */
  0x2307,   /* U+FBF3 */
  0x2307,   /* U+FBF4 */
  0x2307,   /* U+FBF5 */
  0x2307,   /* U+FBF6 */
  0x2307,   /* U+FBF7 */
  0x2307,   /* U+FBF8 */
  0x2307,   /* U+FBF9 */
  0x2307,   /* U+FBFA */
  0x2307,   /* U+FBFB */
  0x23C7,   /* U+FBFC */
  0x23C7,   /* U+FBFD */
  0x23C7,   /* U+FBFE */
  0x23C7,   /* U+FBFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+FB00 */
  0x0020,   /* U+FB01 */
  0x0020,   /* U+FB02 */
  0x0020,   /* U+FB03 */
  0x0020,   /* U+FB04 */
  0x0020,   /* U+FB05 */
  0x0020,   /* U+FB06 */
  0x0020,   /* U+FB07 */
  0x0020,   /* U+FB08 */
  0x0020,   /* U+FB09 */
  0x0020,   /* U+FB0A */
  0x0020,   /* U+FB0B */
  0x0020,   /* U+FB0C */
  0x0020,   /* U+FB0D */
  0x0020,   /* U+FB0E */
  0x0020,   /* U+FB0F */
  0x0020,   /* U+FB10 */
  0x0020,   /* U+FB11 */
  0x0020,   /* U+FB12 */
  0x0020,   /* U+FB13 */
  0x0020,   /* U+FB14 */
  0x0020,   /* U+FB15 */
  0x0020,   /* U+FB16 */
  0x0020,   /* U+FB17 */
  0x0020,   /* U+FB18 */
  0x0020,   /* U+FB19 */
  0x0020,   /* U+FB1A */
  0x0020,   /* U+FB1B */
  0x0020,   /* U+FB1C */
  0x0020,   /* U+FB1D */
  0x0061,   /* U+FB1E */
  0x0020,   /* U+FB1F */
  0x0020,   /* U+FB20 */
  0x0020,   /* U+FB21 */
  0x0020,   /* U+FB22 */
  0x0020,   /* U+FB23 */
  0x0020,   /* U+FB24 */
  0x0020,   /* U+FB25 */
  0x0020,   /* U+FB26 */
  0x0020,   /* U+FB27 */
  0x0020,   /* U+FB28 */
  0x0020,   /* U+FB29 */
  0x0020,   /* U+FB2A */
  0x0020,   /* U+FB2B */
  0x0020,   /* U+FB2C */
  0x0020,   /* U+FB2D */
  0x0020,   /* U+FB2E */
  0x0020,   /* U+FB2F */
  0x0020,   /* U+FB30 */
  0x0020,   /* U+FB31 */
  0x0020,   /* U+FB32 */
  0x0020,   /* U+FB33 */
  0x0020,   /* U+FB34 */
  0x0020,   /* U+FB35 */
  0x0020,   /* U+FB36 */
  0x0020,   /* U+FB37 */
  0x0020,   /* U+FB38 */
  0x0020,   /* U+FB39 */
  0x0020,   /* U+FB3A */
  0x0020,   /* U+FB3B */
  0x0020,   /* U+FB3C */
  0x0020,   /* U+FB3D */
  0x0020,   /* U+FB3E */
  0x0020,   /* U+FB3F */
  0x0020,   /* U+FB40 */
  0x0020,   /* U+FB41 */
  0x0020,   /* U+FB42 */
  0x0020,   /* U+FB43 */
  0x0020,   /* U+FB44 */
  0x0020,   /* U+FB45 */
  0x0020,   /* U+FB46 */
  0x0020,   /* U+FB47 */
  0x0020,   /* U+FB48 */
  0x0020,   /* U+FB49 */
  0x0020,   /* U+FB4A */
  0x0020,   /* U+FB4B */
  0x0020,   /* U+FB4C */
  0x0020,   /* U+FB4D */
  0x0020,   /* U+FB4E */
  0x0020,   /* U+FB4F */
  0x0020,   /* U+FB50 */
  0x0020,   /* U+FB51 */
  0x0020,   /* U+FB52 */
  0x0020,   /* U+FB53 */
  0x0020,   /* U+FB54 */
  0x0020,   /* U+FB55 */
  0x0020,   /* U+FB56 */
  0x0020,   /* U+FB57 */
  0x0020,   /* U+FB58 */
  0x0020,   /* U+FB59 */
  0x0020,   /* U+FB5A */
  0x0020,   /* U+FB5B */
  0x0020,   /* U+FB5C */
  0x0020,   /* U+FB5D */
  0x0020,   /* U+FB5E */
  0x0020,   /* U+FB5F */
  0x0020,   /* U+FB60 */
  0x0020,   /* U+FB61 */
  0x0020,   /* U+FB62 */
  0x0020,   /* U+FB63 */
  0x0020,   /* U+FB64 */
  0x0020,   /* U+FB65 */
  0x0020,   /* U+FB66 */
  0x0020,   /* U+FB67 */
  0x0020,   /* U+FB68 */
  0x0020,   /* U+FB69 */
  0x0020,   /* U+FB6A */
  0x0020,   /* U+FB6B */
  0x0020,   /* U+FB6C */
  0x0020,   /* U+FB6D */
  0x0020,   /* U+FB6E */
  0x0020,   /* U+FB6F */
  0x0020,   /* U+FB70 */
  0x0020,   /* U+FB71 */
  0x0020,   /* U+FB72 */
  0x0020,   /* U+FB73 */
  0x0020,   /* U+FB74 */
  0x0020,   /* U+FB75 */
  0x0020,   /* U+FB76 */
  0x0020,   /* U+FB77 */
  0x0020,   /* U+FB78 */
  0x0020,   /* U+FB79 */
  0x0020,   /* U+FB7A */
  0x0020,   /* U+FB7B */
  0x0020,   /* U+FB7C */
  0x0020,   /* U+FB7D */
  0x0020,   /* U+FB7E */
  0x0020,   /* U+FB7F */
  0x0020,   /* U+FB80 */
  0x0020,   /* U+FB81 */
  0x0020,   /* U+FB82 */
  0x0020,   /* U+FB83 */
  0x0020,   /* U+FB84 */
  0x0020,   /* U+FB85 */
  0x0020,   /* U+FB86 */
  0x0020,   /* U+FB87 */
  0x0020,   /* U+FB88 */
  0x0020,   /* U+FB89 */
  0x0020,   /* U+FB8A */
  0x0020,   /* U+FB8B */
  0x0020,   /* U+FB8C */
  0x0020,   /* U+FB8D */
  0x0020,   /* U+FB8E */
  0x0020,   /* U+FB8F */
  0x0020,   /* U+FB90 */
  0x0020,   /* U+FB91 */
  0x0020,   /* U+FB92 */
  0x0020,   /* U+FB93 */
  0x0020,   /* U+FB94 */
  0x0020,   /* U+FB95 */
  0x0020,   /* U+FB96 */
  0x0020,   /* U+FB97 */
  0x0020,   /* U+FB98 */
  0x0020,   /* U+FB99 */
  0x0020,   /* U+FB9A */
  0x0020,   /* U+FB9B */
  0x0020,   /* U+FB9C */
  0x0020,   /* U+FB9D */
  0x0020,   /* U+FB9E */
  0x0020,   /* U+FB9F */
  0x0020,   /* U+FBA0 */
  0x0020,   /* U+FBA1 */
  0x0020,   /* U+FBA2 */
  0x0020,   /* U+FBA3 */
  0x0020,   /* U+FBA4 */
  0x0020,   /* U+FBA5 */
  0x0020,   /* U+FBA6 */
  0x0020,   /* U+FBA7 */
  0x0020,   /* U+FBA8 */
  0x0020,   /* U+FBA9 */
  0x0020,   /* U+FBAA */
  0x0020,   /* U+FBAB */
  0x0020,   /* U+FBAC */
  0x0020,   /* U+FBAD */
  0x0020,   /* U+FBAE */
  0x0020,   /* U+FBAF */
  0x0020,   /* U+FBB0 */
  0x0020,   /* U+FBB1 */
  0x0020,   /* U+FBB2 */
  0x0020,   /* U+FBB3 */
  0x0020,   /* U+FBB4 */
  0x0020,   /* U+FBB5 */
  0x0020,   /* U+FBB6 */
  0x0020,   /* U+FBB7 */
  0x0020,   /* U+FBB8 */
  0x0020,   /* U+FBB9 */
  0x0020,   /* U+FBBA */
  0x0020,   /* U+FBBB */
  0x0020,   /* U+FBBC */
  0x0020,   /* U+FBBD */
  0x0020,   /* U+FBBE */
  0x0020,   /* U+FBBF */
  0x0020,   /* U+FBC0 */
  0x0020,   /* U+FBC1 */
  0x0020,   /* U+FBC2 */
  0x0020,   /* U+FBC3 */
  0x0020,   /* U+FBC4 */
  0x0020,   /* U+FBC5 */
  0x0020,   /* U+FBC6 */
  0x0020,   /* U+FBC7 */
  0x0020,   /* U+FBC8 */
  0x0020,   /* U+FBC9 */
  0x0020,   /* U+FBCA */
  0x0020,   /* U+FBCB */
  0x0020,   /* U+FBCC */
  0x0020,   /* U+FBCD */
  0x0020,   /* U+FBCE */
  0x0020,   /* U+FBCF */
  0x0020,   /* U+FBD0 */
  0x0020,   /* U+FBD1 */
  0x0020,   /* U+FBD2 */
  0x0020,   /* U+FBD3 */
  0x0020,   /* U+FBD4 */
  0x0020,   /* U+FBD5 */
  0x0020,   /* U+FBD6 */
  0x0020,   /* U+FBD7 */
  0x0020,   /* U+FBD8 */
  0x0020,   /* U+FBD9 */
  0x0020,   /* U+FBDA */
  0x0020,   /* U+FBDB */
  0x0020,   /* U+FBDC */
  0x0020,   /* U+FBDD */
  0x0020,   /* U+FBDE */
  0x0020,   /* U+FBDF */
  0x0020,   /* U+FBE0 */
  0x0020,   /* U+FBE1 */
  0x0020,   /* U+FBE2 */
  0x0020,   /* U+FBE3 */
  0x0020,   /* U+FBE4 */
  0x0020,   /* U+FBE5 */
  0x0020,   /* U+FBE6 */
  0x0020,   /* U+FBE7 */
  0x0020,   /* U+FBE8 */
  0x0020,   /* U+FBE9 */
  0x0020,   /* U+FBEA */
  0x0020,   /* U+FBEB */
  0x0020,   /* U+FBEC */
  0x0020,   /* U+FBED */
  0x0020,   /* U+FBEE */
  0x0020,   /* U+FBEF */
  0x0020,   /* U+FBF0 */
  0x0020,   /* U+FBF1 */
  0x0020,   /* U+FBF2 */
  0x0020,   /* U+FBF3 */
  0x0020,   /* U+FBF4 */
  0x0020,   /* U+FBF5 */
  0x0020,   /* U+FBF6 */
  0x0020,   /* U+FBF7 */
  0x0020,   /* U+FBF8 */
  0x0020,   /* U+FBF9 */
  0x0020,   /* U+FBFA */
  0x0020,   /* U+FBFB */
  0x0020,   /* U+FBFC */
  0x0020,   /* U+FBFD */
  0x0020,   /* U+FBFE */
  0x0020,   /* U+FBFF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+FB00 */
  0x0004,   /* U+FB01 */
  0x0004,   /* U+FB02 */
  0x0004,   /* U+FB03 */
  0x0004,   /* U+FB04 */
  0x0004,   /* U+FB05 */
  0x0004,   /* U+FB06 */
  0x0002,   /* U+FB07 */
  0x0002,   /* U+FB08 */
  0x0002,   /* U+FB09 */
  0x0002,   /* U+FB0A */
  0x0002,   /* U+FB0B */
  0x0002,   /* U+FB0C */
  0x0002,   /* U+FB0D */
  0x0002,   /* U+FB0E */
  0x0002,   /* U+FB0F */
  0x0002,   /* U+FB10 */
  0x0002,   /* U+FB11 */
  0x0002,   /* U+FB12 */
  0x0004,   /* U+FB13 */
  0x0004,   /* U+FB14 */
  0x0004,   /* U+FB15 */
  0x0004,   /* U+FB16 */
  0x0004,   /* U+FB17 */
  0x0002,   /* U+FB18 */
  0x0002,   /* U+FB19 */
  0x0002,   /* U+FB1A */
  0x0002,   /* U+FB1B */
  0x0002,   /* U+FB1C */
  0x0002,   /* U+FB1D */
  0x0002,   /* U+FB1E */
  0x0004,   /* U+FB1F */
  0x0005,   /* U+FB20 */
  0x0005,   /* U+FB21 */
  0x0005,   /* U+FB22 */
  0x0005,   /* U+FB23 */
  0x0005,   /* U+FB24 */
  0x0005,   /* U+FB25 */
  0x0005,   /* U+FB26 */
  0x0005,   /* U+FB27 */
  0x0005,   /* U+FB28 */
  0x0005,   /* U+FB29 */
  0x0002,   /* U+FB2A */
  0x0002,   /* U+FB2B */
  0x0002,   /* U+FB2C */
  0x0002,   /* U+FB2D */
  0x0002,   /* U+FB2E */
  0x0002,   /* U+FB2F */
  0x0002,   /* U+FB30 */
  0x0002,   /* U+FB31 */
  0x0002,   /* U+FB32 */
  0x0002,   /* U+FB33 */
  0x0002,   /* U+FB34 */
  0x0002,   /* U+FB35 */
  0x0002,   /* U+FB36 */
  0x0002,   /* U+FB37 */
  0x0002,   /* U+FB38 */
  0x0002,   /* U+FB39 */
  0x0019,   /* U+FB3A */
  0x0002,   /* U+FB3B */
  0x0002,   /* U+FB3C */
  0x0002,   /* U+FB3D */
  0x0002,   /* U+FB3E */
  0x0002,   /* U+FB3F */
  0x0002,   /* U+FB40 */
  0x0002,   /* U+FB41 */
  0x0002,   /* U+FB42 */
  0x0019,   /* U+FB43 */
  0x0002,   /* U+FB44 */
  0x0002,   /* U+FB45 */
  0x0002,   /* U+FB46 */
  0x0002,   /* U+FB47 */
  0x0002,   /* U+FB48 */
  0x0002,   /* U+FB49 */
  0x0002,   /* U+FB4A */
  0x0002,   /* U+FB4B */
  0x0002,   /* U+FB4C */
  0x0002,   /* U+FB4D */
  0x0002,   /* U+FB4E */
  0x0004,   /* U+FB4F */
  0x001A,   /* U+FB50 */
  0x0019,   /* U+FB51 */
  0x001A,   /* U+FB52 */
  0x0019,   /* U+FB53 */
  0x0017,   /* U+FB54 */
  0x0018,   /* U+FB55 */
  0x001A,   /* U+FB56 */
  0x0019,   /* U+FB57 */
  0x0017,   /* U+FB58 */
  0x0018,   /* U+FB59 */
  0x001A,   /* U+FB5A */
  0x0019,   /* U+FB5B */
  0x0017,   /* U+FB5C */
  0x0018,   /* U+FB5D */
  0x001A,   /* U+FB5E */
  0x0019,   /* U+FB5F */
  0x0017,   /* U+FB60 */
  0x0018,   /* U+FB61 */
  0x001A,   /* U+FB62 */
  0x0019,   /* U+FB63 */
  0x0017,   /* U+FB64 */
  0x0018,   /* U+FB65 */
  0x001A,   /* U+FB66 */
  0x0019,   /* U+FB67 */
  0x0017,   /* U+FB68 */
  0x0018,   /* U+FB69 */
  0x001A,   /* U+FB6A */
  0x0019,   /* U+FB6B */
  0x0017,   /* U+FB6C */
  0x0018,   /* U+FB6D */
  0x001A,   /* U+FB6E */
  0x0019,   /* U+FB6F */
  0x0017,   /* U+FB70 */
  0x0018,   /* U+FB71 */
  0x001A,   /* U+FB72 */
  0x0019,   /* U+FB73 */
  0x0017,   /* U+FB74 */
  0x0018,   /* U+FB75 */
  0x001A,   /* U+FB76 */
  0x0019,   /* U+FB77 */
  0x0017,   /* U+FB78 */
  0x0018,   /* U+FB79 */
  0x001A,   /* U+FB7A */
  0x0019,   /* U+FB7B */
  0x0017,   /* U+FB7C */
  0x0018,   /* U+FB7D */
  0x001A,   /* U+FB7E */
  0x0019,   /* U+FB7F */
  0x0017,   /* U+FB80 */
  0x0018,   /* U+FB81 */
  0x001A,   /* U+FB82 */
  0x0019,   /* U+FB83 */
  0x001A,   /* U+FB84 */
  0x0019,   /* U+FB85 */
  0x001A,   /* U+FB86 */
  0x0019,   /* U+FB87 */
  0x001A,   /* U+FB88 */
  0x0019,   /* U+FB89 */
  0x001A,   /* U+FB8A */
  0x0019,   /* U+FB8B */
  0x001A,   /* U+FB8C */
  0x0019,   /* U+FB8D */
  0x001A,   /* U+FB8E */
  0x0019,   /* U+FB8F */
  0x0017,   /* U+FB90 */
  0x0018,   /* U+FB91 */
  0x001A,   /* U+FB92 */
  0x0019,   /* U+FB93 */
  0x0017,   /* U+FB94 */
  0x0018,   /* U+FB95 */
  0x001A,   /* U+FB96 */
  0x0019,   /* U+FB97 */
  0x0017,   /* U+FB98 */
  0x0018,   /* U+FB99 */
  0x001A,   /* U+FB9A */
  0x0019,   /* U+FB9B */
  0x0017,   /* U+FB9C */
  0x0018,   /* U+FB9D */
  0x001A,   /* U+FB9E */
  0x0019,   /* U+FB9F */
  0x001A,   /* U+FBA0 */
  0x0019,   /* U+FBA1 */
  0x0017,   /* U+FBA2 */
  0x0018,   /* U+FBA3 */
  0x001A,   /* U+FBA4 */
  0x0019,   /* U+FBA5 */
  0x001A,   /* U+FBA6 */
  0x0019,   /* U+FBA7 */
  0x0017,   /* U+FBA8 */
  0x0018,   /* U+FBA9 */
  0x001A,   /* U+FBAA */
  0x0019,   /* U+FBAB */
  0x0017,   /* U+FBAC */
  0x0018,   /* U+FBAD */
  0x001A,   /* U+FBAE */
  0x0019,   /* U+FBAF */
  0x001A,   /* U+FBB0 */
  0x0019,   /* U+FBB1 */
  0x0002,   /* U+FBB2 */
  0x0002,   /* U+FBB3 */
  0x0002,   /* U+FBB4 */
  0x0002,   /* U+FBB5 */
  0x0002,   /* U+FBB6 */
  0x0002,   /* U+FBB7 */
  0x0002,   /* U+FBB8 */
  0x0002,   /* U+FBB9 */
  0x0002,   /* U+FBBA */
  0x0002,   /* U+FBBB */
  0x0002,   /* U+FBBC */
  0x0002,   /* U+FBBD */
  0x0002,   /* U+FBBE */
  0x0002,   /* U+FBBF */
  0x0002,   /* U+FBC0 */
  0x0002,   /* U+FBC1 */
  0x0002,   /* U+FBC2 */
  0x0002,   /* U+FBC3 */
  0x0002,   /* U+FBC4 */
  0x0002,   /* U+FBC5 */
  0x0002,   /* U+FBC6 */
  0x0002,   /* U+FBC7 */
  0x0002,   /* U+FBC8 */
  0x0002,   /* U+FBC9 */
  0x0002,   /* U+FBCA */
  0x0002,   /* U+FBCB */
  0x0002,   /* U+FBCC */
  0x0002,   /* U+FBCD */
  0x0002,   /* U+FBCE */
  0x0002,   /* U+FBCF */
  0x0002,   /* U+FBD0 */
  0x0002,   /* U+FBD1 */
  0x0002,   /* U+FBD2 */
  0x001A,   /* U+FBD3 */
  0x0019,   /* U+FBD4 */
  0x0017,   /* U+FBD5 */
  0x0018,   /* U+FBD6 */
  0x001A,   /* U+FBD7 */
  0x0019,   /* U+FBD8 */
  0x001A,   /* U+FBD9 */
  0x0019,   /* U+FBDA */
  0x001A,   /* U+FBDB */
  0x0019,   /* U+FBDC */
  0x001A,   /* U+FBDD */
  0x001A,   /* U+FBDE */
  0x0019,   /* U+FBDF */
  0x001A,   /* U+FBE0 */
  0x0019,   /* U+FBE1 */
  0x001A,   /* U+FBE2 */
  0x0019,   /* U+FBE3 */
  0x001A,   /* U+FBE4 */
  0x0019,   /* U+FBE5 */
  0x0017,   /* U+FBE6 */
  0x0018,   /* U+FBE7 */
  0x0017,   /* U+FBE8 */
  0x0018,   /* U+FBE9 */
  0x001A,   /* U+FBEA */
  0x0019,   /* U+FBEB */
  0x001A,   /* U+FBEC */
  0x0019,   /* U+FBED */
  0x001A,   /* U+FBEE */
  0x0019,   /* U+FBEF */
  0x001A,   /* U+FBF0 */
  0x0019,   /* U+FBF1 */
  0x001A,   /* U+FBF2 */
  0x0019,   /* U+FBF3 */
  0x001A,   /* U+FBF4 */
  0x0019,   /* U+FBF5 */
  0x001A,   /* U+FBF6 */
  0x0019,   /* U+FBF7 */
  0x0017,   /* U+FBF8 */
  0x001A,   /* U+FBF9 */
  0x0019,   /* U+FBFA */
  0x0017,   /* U+FBFB */
  0x001A,   /* U+FBFC */
  0x0019,   /* U+FBFD */
  0x0017,   /* U+FBFE */
  0x0018,   /* U+FBFF */

  /* Primary weight 2 for each character. */
  0x1CE5,   /* U+FB00 */
  0x1D32,   /* U+FB01 */
  0x1D77,   /* U+FB02 */
  0x1CE5,   /* U+FB03 */
  0x1CE5,   /* U+FB04 */
  0x0000,   /* U+FB05 */
  0x1E95,   /* U+FB06 */
  0xFB07,   /* U+FB07 */
  0xFB08,   /* U+FB08 */
  0xFB09,   /* U+FB09 */
  0xFB0A,   /* U+FB0A */
  0xFB0B,   /* U+FB0B */
  0xFB0C,   /* U+FB0C */
  0xFB0D,   /* U+FB0D */
  0xFB0E,   /* U+FB0E */
  0xFB0F,   /* U+FB0F */
  0xFB10,   /* U+FB10 */
  0xFB11,   /* U+FB11 */
  0xFB12,   /* U+FB12 */
  0x22A5,   /* U+FB13 */
  0x2294,   /* U+FB14 */
  0x229A,   /* U+FB15 */
  0x22A5,   /* U+FB16 */
  0x229C,   /* U+FB17 */
  0xFB18,   /* U+FB18 */
  0xFB19,   /* U+FB19 */
  0xFB1A,   /* U+FB1A */
  0xFB1B,   /* U+FB1B */
  0xFB1C,   /* U+FB1C */
  0x0000,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x22C0,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x0000,   /* U+FB2A */
  0x0000,   /* U+FB2B */
  0x0000,   /* U+FB2C */
  0x0000,   /* U+FB2D */
  0x0000,   /* U+FB2E */
  0x0000,   /* U+FB2F */
  0x0000,   /* U+FB30 */
  0x0000,   /* U+FB31 */
  0x0000,   /* U+FB32 */
  0x0000,   /* U+FB33 */
  0x0000,   /* U+FB34 */
  0x0000,   /* U+FB35 */
  0x0000,   /* U+FB36 */
  0xFB37,   /* U+FB37 */
  0x0000,   /* U+FB38 */
  0x0000,   /* U+FB39 */
  0x0000,   /* U+FB3A */
  0x0000,   /* U+FB3B */
  0x0000,   /* U+FB3C */
  0xFB3D,   /* U+FB3D */
  0x0000,   /* U+FB3E */
  0xFB3F,   /* U+FB3F */
  0x0000,   /* U+FB40 */
  0x0000,   /* U+FB41 */
  0xFB42,   /* U+FB42 */
  0x0000,   /* U+FB43 */
  0x0000,   /* U+FB44 */
  0xFB45,   /* U+FB45 */
  0x0000,   /* U+FB46 */
  0x0000,   /* U+FB47 */
  0x0000,   /* U+FB48 */
  0x0000,   /* U+FB49 */
  0x0000,   /* U+FB4A */
  0x0000,   /* U+FB4B */
  0x0000,   /* U+FB4C */
  0x0000,   /* U+FB4D */
  0x0000,   /* U+FB4E */
  0x22C2,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x0000,   /* U+FBA4 */
  0x0000,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x0000,   /* U+FBB0 */
  0x0000,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0xFBC2,   /* U+FBC2 */
  0xFBC3,   /* U+FBC3 */
  0xFBC4,   /* U+FBC4 */
  0xFBC5,   /* U+FBC5 */
  0xFBC6,   /* U+FBC6 */
  0xFBC7,   /* U+FBC7 */
  0xFBC8,   /* U+FBC8 */
  0xFBC9,   /* U+FBC9 */
  0xFBCA,   /* U+FBCA */
  0xFBCB,   /* U+FBCB */
  0xFBCC,   /* U+FBCC */
  0xFBCD,   /* U+FBCD */
  0xFBCE,   /* U+FBCE */
  0xFBCF,   /* U+FBCF */
  0xFBD0,   /* U+FBD0 */
  0xFBD1,   /* U+FBD1 */
  0xFBD2,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x22FD,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x230B,   /* U+FBEA */
  0x230B,   /* U+FBEB */
  0x23B6,   /* U+FBEC */
  0x23B6,   /* U+FBED */
  0x23B7,   /* U+FBEE */
  0x23B7,   /* U+FBEF */
  0x23BB,   /* U+FBF0 */
  0x23BB,   /* U+FBF1 */
  0x23BA,   /* U+FBF2 */
  0x23BA,   /* U+FBF3 */
  0x23BC,   /* U+FBF4 */
  0x23BC,   /* U+FBF5 */
  0x23CA,   /* U+FBF6 */
  0x23CA,   /* U+FBF7 */
  0x23CA,   /* U+FBF8 */
  0x23C5,   /* U+FBF9 */
  0x23C5,   /* U+FBFA */
  0x23C5,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+FB00 */
  0x0020,   /* U+FB01 */
  0x0020,   /* U+FB02 */
  0x0020,   /* U+FB03 */
  0x0020,   /* U+FB04 */
  0x0111,   /* U+FB05 */
  0x0020,   /* U+FB06 */
  0x0000,   /* U+FB07 */
  0x0000,   /* U+FB08 */
  0x0000,   /* U+FB09 */
  0x0000,   /* U+FB0A */
  0x0000,   /* U+FB0B */
  0x0000,   /* U+FB0C */
  0x0000,   /* U+FB0D */
  0x0000,   /* U+FB0E */
  0x0000,   /* U+FB0F */
  0x0000,   /* U+FB10 */
  0x0000,   /* U+FB11 */
  0x0000,   /* U+FB12 */
  0x0020,   /* U+FB13 */
  0x0020,   /* U+FB14 */
  0x0020,   /* U+FB15 */
  0x0020,   /* U+FB16 */
  0x0020,   /* U+FB17 */
  0x0000,   /* U+FB18 */
  0x0000,   /* U+FB19 */
  0x0000,   /* U+FB1A */
  0x0000,   /* U+FB1B */
  0x0000,   /* U+FB1C */
  0x0056,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x0020,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x005E,   /* U+FB2A */
  0x005D,   /* U+FB2B */
  0x005F,   /* U+FB2C */
  0x005F,   /* U+FB2D */
  0x0059,   /* U+FB2E */
  0x005A,   /* U+FB2F */
  0x005F,   /* U+FB30 */
  0x005F,   /* U+FB31 */
  0x005F,   /* U+FB32 */
  0x005F,   /* U+FB33 */
  0x005F,   /* U+FB34 */
  0x005F,   /* U+FB35 */
  0x005F,   /* U+FB36 */
  0x0000,   /* U+FB37 */
  0x005F,   /* U+FB38 */
  0x005F,   /* U+FB39 */
  0x005F,   /* U+FB3A */
  0x005F,   /* U+FB3B */
  0x005F,   /* U+FB3C */
  0x0000,   /* U+FB3D */
  0x005F,   /* U+FB3E */
  0x0000,   /* U+FB3F */
  0x005F,   /* U+FB40 */
  0x005F,   /* U+FB41 */
  0x0000,   /* U+FB42 */
  0x005F,   /* U+FB43 */
  0x005F,   /* U+FB44 */
  0x0000,   /* U+FB45 */
  0x005F,   /* U+FB46 */
  0x005F,   /* U+FB47 */
  0x005F,   /* U+FB48 */
  0x005F,   /* U+FB49 */
  0x005F,   /* U+FB4A */
  0x005B,   /* U+FB4B */
  0x0060,   /* U+FB4C */
  0x0060,   /* U+FB4D */
  0x0060,   /* U+FB4E */
  0x0020,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x0083,   /* U+FBA4 */
  0x0083,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x0083,   /* U+FBB0 */
  0x0083,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0x0000,   /* U+FBC2 */
  0x0000,   /* U+FBC3 */
  0x0000,   /* U+FBC4 */
  0x0000,   /* U+FBC5 */
  0x0000,   /* U+FBC6 */
  0x0000,   /* U+FBC7 */
  0x0000,   /* U+FBC8 */
  0x0000,   /* U+FBC9 */
  0x0000,   /* U+FBCA */
  0x0000,   /* U+FBCB */
  0x0000,   /* U+FBCC */
  0x0000,   /* U+FBCD */
  0x0000,   /* U+FBCE */
  0x0000,   /* U+FBCF */
  0x0000,   /* U+FBD0 */
  0x0000,   /* U+FBD1 */
  0x0000,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x0020,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x0020,   /* U+FBEA */
  0x0020,   /* U+FBEB */
  0x0020,   /* U+FBEC */
  0x0020,   /* U+FBED */
  0x0020,   /* U+FBEE */
  0x0020,   /* U+FBEF */
  0x0020,   /* U+FBF0 */
  0x0020,   /* U+FBF1 */
  0x0020,   /* U+FBF2 */
  0x0020,   /* U+FBF3 */
  0x0020,   /* U+FBF4 */
  0x0020,   /* U+FBF5 */
  0x0020,   /* U+FBF6 */
  0x0020,   /* U+FBF7 */
  0x0020,   /* U+FBF8 */
  0x0020,   /* U+FBF9 */
  0x0020,   /* U+FBFA */
  0x0020,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */

  /* Tertiary weight 2 for each character. */
  0x0004,   /* U+FB00 */
  0x0004,   /* U+FB01 */
  0x0004,   /* U+FB02 */
  0x0004,   /* U+FB03 */
  0x0004,   /* U+FB04 */
  0x0004,   /* U+FB05 */
  0x0004,   /* U+FB06 */
  0x0000,   /* U+FB07 */
  0x0000,   /* U+FB08 */
  0x0000,   /* U+FB09 */
  0x0000,   /* U+FB0A */
  0x0000,   /* U+FB0B */
  0x0000,   /* U+FB0C */
  0x0000,   /* U+FB0D */
  0x0000,   /* U+FB0E */
  0x0000,   /* U+FB0F */
  0x0000,   /* U+FB10 */
  0x0000,   /* U+FB11 */
  0x0000,   /* U+FB12 */
  0x0004,   /* U+FB13 */
  0x0004,   /* U+FB14 */
  0x0004,   /* U+FB15 */
  0x0004,   /* U+FB16 */
  0x0004,   /* U+FB17 */
  0x0000,   /* U+FB18 */
  0x0000,   /* U+FB19 */
  0x0000,   /* U+FB1A */
  0x0000,   /* U+FB1B */
  0x0000,   /* U+FB1C */
  0x0002,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x0004,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x0002,   /* U+FB2A */
  0x0002,   /* U+FB2B */
  0x0002,   /* U+FB2C */
  0x0002,   /* U+FB2D */
  0x0002,   /* U+FB2E */
  0x0002,   /* U+FB2F */
  0x0002,   /* U+FB30 */
  0x0002,   /* U+FB31 */
  0x0002,   /* U+FB32 */
  0x0002,   /* U+FB33 */
  0x0002,   /* U+FB34 */
  0x0002,   /* U+FB35 */
  0x0002,   /* U+FB36 */
  0x0000,   /* U+FB37 */
  0x0002,   /* U+FB38 */
  0x0002,   /* U+FB39 */
  0x0002,   /* U+FB3A */
  0x0002,   /* U+FB3B */
  0x0002,   /* U+FB3C */
  0x0000,   /* U+FB3D */
  0x0002,   /* U+FB3E */
  0x0000,   /* U+FB3F */
  0x0002,   /* U+FB40 */
  0x0002,   /* U+FB41 */
  0x0000,   /* U+FB42 */
  0x0002,   /* U+FB43 */
  0x0002,   /* U+FB44 */
  0x0000,   /* U+FB45 */
  0x0002,   /* U+FB46 */
  0x0002,   /* U+FB47 */
  0x0002,   /* U+FB48 */
  0x0002,   /* U+FB49 */
  0x0002,   /* U+FB4A */
  0x0002,   /* U+FB4B */
  0x0002,   /* U+FB4C */
  0x0002,   /* U+FB4D */
  0x0002,   /* U+FB4E */
  0x0004,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x001A,   /* U+FBA4 */
  0x0019,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x001A,   /* U+FBB0 */
  0x0019,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0x0000,   /* U+FBC2 */
  0x0000,   /* U+FBC3 */
  0x0000,   /* U+FBC4 */
  0x0000,   /* U+FBC5 */
  0x0000,   /* U+FBC6 */
  0x0000,   /* U+FBC7 */
  0x0000,   /* U+FBC8 */
  0x0000,   /* U+FBC9 */
  0x0000,   /* U+FBCA */
  0x0000,   /* U+FBCB */
  0x0000,   /* U+FBCC */
  0x0000,   /* U+FBCD */
  0x0000,   /* U+FBCE */
  0x0000,   /* U+FBCF */
  0x0000,   /* U+FBD0 */
  0x0000,   /* U+FBD1 */
  0x0000,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x001A,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x001A,   /* U+FBEA */
  0x0019,   /* U+FBEB */
  0x001A,   /* U+FBEC */
  0x0019,   /* U+FBED */
  0x001A,   /* U+FBEE */
  0x0019,   /* U+FBEF */
  0x001A,   /* U+FBF0 */
  0x0019,   /* U+FBF1 */
  0x001A,   /* U+FBF2 */
  0x0019,   /* U+FBF3 */
  0x001A,   /* U+FBF4 */
  0x0019,   /* U+FBF5 */
  0x001A,   /* U+FBF6 */
  0x0019,   /* U+FBF7 */
  0x0017,   /* U+FBF8 */
  0x001A,   /* U+FBF9 */
  0x0019,   /* U+FBFA */
  0x0017,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+FB00 */
  0x0000,   /* U+FB01 */
  0x0000,   /* U+FB02 */
  0x1D32,   /* U+FB03 */
  0x1D77,   /* U+FB04 */
  0x1E95,   /* U+FB05 */
  0x0000,   /* U+FB06 */
  0x0000,   /* U+FB07 */
  0x0000,   /* U+FB08 */
  0x0000,   /* U+FB09 */
  0x0000,   /* U+FB0A */
  0x0000,   /* U+FB0B */
  0x0000,   /* U+FB0C */
  0x0000,   /* U+FB0D */
  0x0000,   /* U+FB0E */
  0x0000,   /* U+FB0F */
  0x0000,   /* U+FB10 */
  0x0000,   /* U+FB11 */
  0x0000,   /* U+FB12 */
  0x0000,   /* U+FB13 */
  0x0000,   /* U+FB14 */
  0x0000,   /* U+FB15 */
  0x0000,   /* U+FB16 */
  0x0000,   /* U+FB17 */
  0x0000,   /* U+FB18 */
  0x0000,   /* U+FB19 */
  0x0000,   /* U+FB1A */
  0x0000,   /* U+FB1B */
  0x0000,   /* U+FB1C */
  0x0000,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x0000,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x0000,   /* U+FB2A */
  0x0000,   /* U+FB2B */
  0x0000,   /* U+FB2C */
  0x0000,   /* U+FB2D */
  0x0000,   /* U+FB2E */
  0x0000,   /* U+FB2F */
  0x0000,   /* U+FB30 */
  0x0000,   /* U+FB31 */
  0x0000,   /* U+FB32 */
  0x0000,   /* U+FB33 */
  0x0000,   /* U+FB34 */
  0x0000,   /* U+FB35 */
  0x0000,   /* U+FB36 */
  0x0000,   /* U+FB37 */
  0x0000,   /* U+FB38 */
  0x0000,   /* U+FB39 */
  0x0000,   /* U+FB3A */
  0x0000,   /* U+FB3B */
  0x0000,   /* U+FB3C */
  0x0000,   /* U+FB3D */
  0x0000,   /* U+FB3E */
  0x0000,   /* U+FB3F */
  0x0000,   /* U+FB40 */
  0x0000,   /* U+FB41 */
  0x0000,   /* U+FB42 */
  0x0000,   /* U+FB43 */
  0x0000,   /* U+FB44 */
  0x0000,   /* U+FB45 */
  0x0000,   /* U+FB46 */
  0x0000,   /* U+FB47 */
  0x0000,   /* U+FB48 */
  0x0000,   /* U+FB49 */
  0x0000,   /* U+FB4A */
  0x0000,   /* U+FB4B */
  0x0000,   /* U+FB4C */
  0x0000,   /* U+FB4D */
  0x0000,   /* U+FB4E */
  0x0000,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x0000,   /* U+FBA4 */
  0x0000,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x0000,   /* U+FBB0 */
  0x0000,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0x0000,   /* U+FBC2 */
  0x0000,   /* U+FBC3 */
  0x0000,   /* U+FBC4 */
  0x0000,   /* U+FBC5 */
  0x0000,   /* U+FBC6 */
  0x0000,   /* U+FBC7 */
  0x0000,   /* U+FBC8 */
  0x0000,   /* U+FBC9 */
  0x0000,   /* U+FBCA */
  0x0000,   /* U+FBCB */
  0x0000,   /* U+FBCC */
  0x0000,   /* U+FBCD */
  0x0000,   /* U+FBCE */
  0x0000,   /* U+FBCF */
  0x0000,   /* U+FBD0 */
  0x0000,   /* U+FBD1 */
  0x0000,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x0000,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x0000,   /* U+FBEA */
  0x0000,   /* U+FBEB */
  0x0000,   /* U+FBEC */
  0x0000,   /* U+FBED */
  0x0000,   /* U+FBEE */
  0x0000,   /* U+FBEF */
  0x0000,   /* U+FBF0 */
  0x0000,   /* U+FBF1 */
  0x0000,   /* U+FBF2 */
  0x0000,   /* U+FBF3 */
  0x0000,   /* U+FBF4 */
  0x0000,   /* U+FBF5 */
  0x0000,   /* U+FBF6 */
  0x0000,   /* U+FBF7 */
  0x0000,   /* U+FBF8 */
  0x0000,   /* U+FBF9 */
  0x0000,   /* U+FBFA */
  0x0000,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+FB00 */
  0x0000,   /* U+FB01 */
  0x0000,   /* U+FB02 */
  0x0020,   /* U+FB03 */
  0x0020,   /* U+FB04 */
  0x0020,   /* U+FB05 */
  0x0000,   /* U+FB06 */
  0x0000,   /* U+FB07 */
  0x0000,   /* U+FB08 */
  0x0000,   /* U+FB09 */
  0x0000,   /* U+FB0A */
  0x0000,   /* U+FB0B */
  0x0000,   /* U+FB0C */
  0x0000,   /* U+FB0D */
  0x0000,   /* U+FB0E */
  0x0000,   /* U+FB0F */
  0x0000,   /* U+FB10 */
  0x0000,   /* U+FB11 */
  0x0000,   /* U+FB12 */
  0x0000,   /* U+FB13 */
  0x0000,   /* U+FB14 */
  0x0000,   /* U+FB15 */
  0x0000,   /* U+FB16 */
  0x0000,   /* U+FB17 */
  0x0000,   /* U+FB18 */
  0x0000,   /* U+FB19 */
  0x0000,   /* U+FB1A */
  0x0000,   /* U+FB1B */
  0x0000,   /* U+FB1C */
  0x0000,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x0059,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x0000,   /* U+FB2A */
  0x0000,   /* U+FB2B */
  0x005E,   /* U+FB2C */
  0x005D,   /* U+FB2D */
  0x0000,   /* U+FB2E */
  0x0000,   /* U+FB2F */
  0x0000,   /* U+FB30 */
  0x0000,   /* U+FB31 */
  0x0000,   /* U+FB32 */
  0x0000,   /* U+FB33 */
  0x0000,   /* U+FB34 */
  0x0000,   /* U+FB35 */
  0x0000,   /* U+FB36 */
  0x0000,   /* U+FB37 */
  0x0000,   /* U+FB38 */
  0x0000,   /* U+FB39 */
  0x0000,   /* U+FB3A */
  0x0000,   /* U+FB3B */
  0x0000,   /* U+FB3C */
  0x0000,   /* U+FB3D */
  0x0000,   /* U+FB3E */
  0x0000,   /* U+FB3F */
  0x0000,   /* U+FB40 */
  0x0000,   /* U+FB41 */
  0x0000,   /* U+FB42 */
  0x0000,   /* U+FB43 */
  0x0000,   /* U+FB44 */
  0x0000,   /* U+FB45 */
  0x0000,   /* U+FB46 */
  0x0000,   /* U+FB47 */
  0x0000,   /* U+FB48 */
  0x0000,   /* U+FB49 */
  0x0000,   /* U+FB4A */
  0x0000,   /* U+FB4B */
  0x0000,   /* U+FB4C */
  0x0000,   /* U+FB4D */
  0x0000,   /* U+FB4E */
  0x0000,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x0000,   /* U+FBA4 */
  0x0000,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x0000,   /* U+FBB0 */
  0x0000,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0x0000,   /* U+FBC2 */
  0x0000,   /* U+FBC3 */
  0x0000,   /* U+FBC4 */
  0x0000,   /* U+FBC5 */
  0x0000,   /* U+FBC6 */
  0x0000,   /* U+FBC7 */
  0x0000,   /* U+FBC8 */
  0x0000,   /* U+FBC9 */
  0x0000,   /* U+FBCA */
  0x0000,   /* U+FBCB */
  0x0000,   /* U+FBCC */
  0x0000,   /* U+FBCD */
  0x0000,   /* U+FBCE */
  0x0000,   /* U+FBCF */
  0x0000,   /* U+FBD0 */
  0x0000,   /* U+FBD1 */
  0x0000,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x0000,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x0000,   /* U+FBEA */
  0x0000,   /* U+FBEB */
  0x0000,   /* U+FBEC */
  0x0000,   /* U+FBED */
  0x0000,   /* U+FBEE */
  0x0000,   /* U+FBEF */
  0x0000,   /* U+FBF0 */
  0x0000,   /* U+FBF1 */
  0x0000,   /* U+FBF2 */
  0x0000,   /* U+FBF3 */
  0x0000,   /* U+FBF4 */
  0x0000,   /* U+FBF5 */
  0x0000,   /* U+FBF6 */
  0x0000,   /* U+FBF7 */
  0x0000,   /* U+FBF8 */
  0x0000,   /* U+FBF9 */
  0x0000,   /* U+FBFA */
  0x0000,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+FB00 */
  0x0000,   /* U+FB01 */
  0x0000,   /* U+FB02 */
  0x0004,   /* U+FB03 */
  0x0004,   /* U+FB04 */
  0x0004,   /* U+FB05 */
  0x0000,   /* U+FB06 */
  0x0000,   /* U+FB07 */
  0x0000,   /* U+FB08 */
  0x0000,   /* U+FB09 */
  0x0000,   /* U+FB0A */
  0x0000,   /* U+FB0B */
  0x0000,   /* U+FB0C */
  0x0000,   /* U+FB0D */
  0x0000,   /* U+FB0E */
  0x0000,   /* U+FB0F */
  0x0000,   /* U+FB10 */
  0x0000,   /* U+FB11 */
  0x0000,   /* U+FB12 */
  0x0000,   /* U+FB13 */
  0x0000,   /* U+FB14 */
  0x0000,   /* U+FB15 */
  0x0000,   /* U+FB16 */
  0x0000,   /* U+FB17 */
  0x0000,   /* U+FB18 */
  0x0000,   /* U+FB19 */
  0x0000,   /* U+FB1A */
  0x0000,   /* U+FB1B */
  0x0000,   /* U+FB1C */
  0x0000,   /* U+FB1D */
  0x0000,   /* U+FB1E */
  0x0002,   /* U+FB1F */
  0x0000,   /* U+FB20 */
  0x0000,   /* U+FB21 */
  0x0000,   /* U+FB22 */
  0x0000,   /* U+FB23 */
  0x0000,   /* U+FB24 */
  0x0000,   /* U+FB25 */
  0x0000,   /* U+FB26 */
  0x0000,   /* U+FB27 */
  0x0000,   /* U+FB28 */
  0x0000,   /* U+FB29 */
  0x0000,   /* U+FB2A */
  0x0000,   /* U+FB2B */
  0x0002,   /* U+FB2C */
  0x0002,   /* U+FB2D */
  0x0000,   /* U+FB2E */
  0x0000,   /* U+FB2F */
  0x0000,   /* U+FB30 */
  0x0000,   /* U+FB31 */
  0x0000,   /* U+FB32 */
  0x0000,   /* U+FB33 */
  0x0000,   /* U+FB34 */
  0x0000,   /* U+FB35 */
  0x0000,   /* U+FB36 */
  0x0000,   /* U+FB37 */
  0x0000,   /* U+FB38 */
  0x0000,   /* U+FB39 */
  0x0000,   /* U+FB3A */
  0x0000,   /* U+FB3B */
  0x0000,   /* U+FB3C */
  0x0000,   /* U+FB3D */
  0x0000,   /* U+FB3E */
  0x0000,   /* U+FB3F */
  0x0000,   /* U+FB40 */
  0x0000,   /* U+FB41 */
  0x0000,   /* U+FB42 */
  0x0000,   /* U+FB43 */
  0x0000,   /* U+FB44 */
  0x0000,   /* U+FB45 */
  0x0000,   /* U+FB46 */
  0x0000,   /* U+FB47 */
  0x0000,   /* U+FB48 */
  0x0000,   /* U+FB49 */
  0x0000,   /* U+FB4A */
  0x0000,   /* U+FB4B */
  0x0000,   /* U+FB4C */
  0x0000,   /* U+FB4D */
  0x0000,   /* U+FB4E */
  0x0000,   /* U+FB4F */
  0x0000,   /* U+FB50 */
  0x0000,   /* U+FB51 */
  0x0000,   /* U+FB52 */
  0x0000,   /* U+FB53 */
  0x0000,   /* U+FB54 */
  0x0000,   /* U+FB55 */
  0x0000,   /* U+FB56 */
  0x0000,   /* U+FB57 */
  0x0000,   /* U+FB58 */
  0x0000,   /* U+FB59 */
  0x0000,   /* U+FB5A */
  0x0000,   /* U+FB5B */
  0x0000,   /* U+FB5C */
  0x0000,   /* U+FB5D */
  0x0000,   /* U+FB5E */
  0x0000,   /* U+FB5F */
  0x0000,   /* U+FB60 */
  0x0000,   /* U+FB61 */
  0x0000,   /* U+FB62 */
  0x0000,   /* U+FB63 */
  0x0000,   /* U+FB64 */
  0x0000,   /* U+FB65 */
  0x0000,   /* U+FB66 */
  0x0000,   /* U+FB67 */
  0x0000,   /* U+FB68 */
  0x0000,   /* U+FB69 */
  0x0000,   /* U+FB6A */
  0x0000,   /* U+FB6B */
  0x0000,   /* U+FB6C */
  0x0000,   /* U+FB6D */
  0x0000,   /* U+FB6E */
  0x0000,   /* U+FB6F */
  0x0000,   /* U+FB70 */
  0x0000,   /* U+FB71 */
  0x0000,   /* U+FB72 */
  0x0000,   /* U+FB73 */
  0x0000,   /* U+FB74 */
  0x0000,   /* U+FB75 */
  0x0000,   /* U+FB76 */
  0x0000,   /* U+FB77 */
  0x0000,   /* U+FB78 */
  0x0000,   /* U+FB79 */
  0x0000,   /* U+FB7A */
  0x0000,   /* U+FB7B */
  0x0000,   /* U+FB7C */
  0x0000,   /* U+FB7D */
  0x0000,   /* U+FB7E */
  0x0000,   /* U+FB7F */
  0x0000,   /* U+FB80 */
  0x0000,   /* U+FB81 */
  0x0000,   /* U+FB82 */
  0x0000,   /* U+FB83 */
  0x0000,   /* U+FB84 */
  0x0000,   /* U+FB85 */
  0x0000,   /* U+FB86 */
  0x0000,   /* U+FB87 */
  0x0000,   /* U+FB88 */
  0x0000,   /* U+FB89 */
  0x0000,   /* U+FB8A */
  0x0000,   /* U+FB8B */
  0x0000,   /* U+FB8C */
  0x0000,   /* U+FB8D */
  0x0000,   /* U+FB8E */
  0x0000,   /* U+FB8F */
  0x0000,   /* U+FB90 */
  0x0000,   /* U+FB91 */
  0x0000,   /* U+FB92 */
  0x0000,   /* U+FB93 */
  0x0000,   /* U+FB94 */
  0x0000,   /* U+FB95 */
  0x0000,   /* U+FB96 */
  0x0000,   /* U+FB97 */
  0x0000,   /* U+FB98 */
  0x0000,   /* U+FB99 */
  0x0000,   /* U+FB9A */
  0x0000,   /* U+FB9B */
  0x0000,   /* U+FB9C */
  0x0000,   /* U+FB9D */
  0x0000,   /* U+FB9E */
  0x0000,   /* U+FB9F */
  0x0000,   /* U+FBA0 */
  0x0000,   /* U+FBA1 */
  0x0000,   /* U+FBA2 */
  0x0000,   /* U+FBA3 */
  0x0000,   /* U+FBA4 */
  0x0000,   /* U+FBA5 */
  0x0000,   /* U+FBA6 */
  0x0000,   /* U+FBA7 */
  0x0000,   /* U+FBA8 */
  0x0000,   /* U+FBA9 */
  0x0000,   /* U+FBAA */
  0x0000,   /* U+FBAB */
  0x0000,   /* U+FBAC */
  0x0000,   /* U+FBAD */
  0x0000,   /* U+FBAE */
  0x0000,   /* U+FBAF */
  0x0000,   /* U+FBB0 */
  0x0000,   /* U+FBB1 */
  0x0000,   /* U+FBB2 */
  0x0000,   /* U+FBB3 */
  0x0000,   /* U+FBB4 */
  0x0000,   /* U+FBB5 */
  0x0000,   /* U+FBB6 */
  0x0000,   /* U+FBB7 */
  0x0000,   /* U+FBB8 */
  0x0000,   /* U+FBB9 */
  0x0000,   /* U+FBBA */
  0x0000,   /* U+FBBB */
  0x0000,   /* U+FBBC */
  0x0000,   /* U+FBBD */
  0x0000,   /* U+FBBE */
  0x0000,   /* U+FBBF */
  0x0000,   /* U+FBC0 */
  0x0000,   /* U+FBC1 */
  0x0000,   /* U+FBC2 */
  0x0000,   /* U+FBC3 */
  0x0000,   /* U+FBC4 */
  0x0000,   /* U+FBC5 */
  0x0000,   /* U+FBC6 */
  0x0000,   /* U+FBC7 */
  0x0000,   /* U+FBC8 */
  0x0000,   /* U+FBC9 */
  0x0000,   /* U+FBCA */
  0x0000,   /* U+FBCB */
  0x0000,   /* U+FBCC */
  0x0000,   /* U+FBCD */
  0x0000,   /* U+FBCE */
  0x0000,   /* U+FBCF */
  0x0000,   /* U+FBD0 */
  0x0000,   /* U+FBD1 */
  0x0000,   /* U+FBD2 */
  0x0000,   /* U+FBD3 */
  0x0000,   /* U+FBD4 */
  0x0000,   /* U+FBD5 */
  0x0000,   /* U+FBD6 */
  0x0000,   /* U+FBD7 */
  0x0000,   /* U+FBD8 */
  0x0000,   /* U+FBD9 */
  0x0000,   /* U+FBDA */
  0x0000,   /* U+FBDB */
  0x0000,   /* U+FBDC */
  0x0000,   /* U+FBDD */
  0x0000,   /* U+FBDE */
  0x0000,   /* U+FBDF */
  0x0000,   /* U+FBE0 */
  0x0000,   /* U+FBE1 */
  0x0000,   /* U+FBE2 */
  0x0000,   /* U+FBE3 */
  0x0000,   /* U+FBE4 */
  0x0000,   /* U+FBE5 */
  0x0000,   /* U+FBE6 */
  0x0000,   /* U+FBE7 */
  0x0000,   /* U+FBE8 */
  0x0000,   /* U+FBE9 */
  0x0000,   /* U+FBEA */
  0x0000,   /* U+FBEB */
  0x0000,   /* U+FBEC */
  0x0000,   /* U+FBED */
  0x0000,   /* U+FBEE */
  0x0000,   /* U+FBEF */
  0x0000,   /* U+FBF0 */
  0x0000,   /* U+FBF1 */
  0x0000,   /* U+FBF2 */
  0x0000,   /* U+FBF3 */
  0x0000,   /* U+FBF4 */
  0x0000,   /* U+FBF5 */
  0x0000,   /* U+FBF6 */
  0x0000,   /* U+FBF7 */
  0x0000,   /* U+FBF8 */
  0x0000,   /* U+FBF9 */
  0x0000,   /* U+FBFA */
  0x0000,   /* U+FBFB */
  0x0000,   /* U+FBFC */
  0x0000,   /* U+FBFD */
  0x0000,   /* U+FBFE */
  0x0000,   /* U+FBFF */
};

uint16 uca900_p0FC[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2307,   /* U+FC00 */
  0x2307,   /* U+FC01 */
  0x2307,   /* U+FC02 */
  0x2307,   /* U+FC03 */
  0x2307,   /* U+FC04 */
  0x230D,   /* U+FC05 */
  0x230D,   /* U+FC06 */
  0x230D,   /* U+FC07 */
  0x230D,   /* U+FC08 */
  0x230D,   /* U+FC09 */
  0x230D,   /* U+FC0A */
  0x231D,   /* U+FC0B */
  0x231D,   /* U+FC0C */
  0x231D,   /* U+FC0D */
  0x231D,   /* U+FC0E */
  0x231D,   /* U+FC0F */
  0x231D,   /* U+FC10 */
  0x231E,   /* U+FC11 */
  0x231E,   /* U+FC12 */
  0x231E,   /* U+FC13 */
  0x231E,   /* U+FC14 */
  0x2325,   /* U+FC15 */
  0x2325,   /* U+FC16 */
  0x232C,   /* U+FC17 */
  0x232C,   /* U+FC18 */
  0x232D,   /* U+FC19 */
  0x232D,   /* U+FC1A */
  0x232D,   /* U+FC1B */
  0x2359,   /* U+FC1C */
  0x2359,   /* U+FC1D */
  0x2359,   /* U+FC1E */
  0x2359,   /* U+FC1F */
  0x2364,   /* U+FC20 */
  0x2364,   /* U+FC21 */
  0x2365,   /* U+FC22 */
  0x2365,   /* U+FC23 */
  0x2365,   /* U+FC24 */
  0x2365,   /* U+FC25 */
  0x236A,   /* U+FC26 */
  0x236A,   /* U+FC27 */
  0x236B,   /* U+FC28 */
  0x236E,   /* U+FC29 */
  0x236E,   /* U+FC2A */
  0x236F,   /* U+FC2B */
  0x236F,   /* U+FC2C */
  0x2376,   /* U+FC2D */
  0x2376,   /* U+FC2E */
  0x2376,   /* U+FC2F */
  0x2376,   /* U+FC30 */
  0x2376,   /* U+FC31 */
  0x2376,   /* U+FC32 */
  0x2382,   /* U+FC33 */
  0x2382,   /* U+FC34 */
  0x2382,   /* U+FC35 */
  0x2382,   /* U+FC36 */
  0x2387,   /* U+FC37 */
  0x2387,   /* U+FC38 */
  0x2387,   /* U+FC39 */
  0x2387,   /* U+FC3A */
  0x2387,   /* U+FC3B */
  0x2387,   /* U+FC3C */
  0x2387,   /* U+FC3D */
  0x2387,   /* U+FC3E */
  0x239C,   /* U+FC3F */
  0x239C,   /* U+FC40 */
  0x239C,   /* U+FC41 */
  0x239C,   /* U+FC42 */
  0x239C,   /* U+FC43 */
  0x239C,   /* U+FC44 */
  0x23A3,   /* U+FC45 */
  0x23A3,   /* U+FC46 */
  0x23A3,   /* U+FC47 */
  0x23A3,   /* U+FC48 */
  0x23A3,   /* U+FC49 */
  0x23A3,   /* U+FC4A */
  0x23A7,   /* U+FC4B */
  0x23A7,   /* U+FC4C */
  0x23A7,   /* U+FC4D */
  0x23A7,   /* U+FC4E */
  0x23A7,   /* U+FC4F */
  0x23A7,   /* U+FC50 */
  0x23B1,   /* U+FC51 */
  0x23B1,   /* U+FC52 */
  0x23B1,   /* U+FC53 */
  0x23B1,   /* U+FC54 */
  0x23C6,   /* U+FC55 */
  0x23C6,   /* U+FC56 */
  0x23C6,   /* U+FC57 */
  0x23C6,   /* U+FC58 */
  0x23C6,   /* U+FC59 */
  0x23C6,   /* U+FC5A */
  0x2338,   /* U+FC5B */
  0x2346,   /* U+FC5C */
  0x23C5,   /* U+FC5D */
  0x0000,   /* U+FC5E */
  0x0000,   /* U+FC5F */
  0x0000,   /* U+FC60 */
  0x0000,   /* U+FC61 */
  0x0000,   /* U+FC62 */
  0x0000,   /* U+FC63 */
  0x2307,   /* U+FC64 */
  0x2307,   /* U+FC65 */
  0x2307,   /* U+FC66 */
  0x2307,   /* U+FC67 */
  0x2307,   /* U+FC68 */
  0x2307,   /* U+FC69 */
  0x230D,   /* U+FC6A */
  0x230D,   /* U+FC6B */
  0x230D,   /* U+FC6C */
  0x230D,   /* U+FC6D */
  0x230D,   /* U+FC6E */
  0x230D,   /* U+FC6F */
  0x231D,   /* U+FC70 */
  0x231D,   /* U+FC71 */
  0x231D,   /* U+FC72 */
  0x231D,   /* U+FC73 */
  0x231D,   /* U+FC74 */
  0x231D,   /* U+FC75 */
  0x231E,   /* U+FC76 */
  0x231E,   /* U+FC77 */
  0x231E,   /* U+FC78 */
  0x231E,   /* U+FC79 */
  0x231E,   /* U+FC7A */
  0x231E,   /* U+FC7B */
  0x2376,   /* U+FC7C */
  0x2376,   /* U+FC7D */
  0x2382,   /* U+FC7E */
  0x2382,   /* U+FC7F */
  0x2387,   /* U+FC80 */
  0x2387,   /* U+FC81 */
  0x2387,   /* U+FC82 */
  0x2387,   /* U+FC83 */
  0x2387,   /* U+FC84 */
  0x239C,   /* U+FC85 */
  0x239C,   /* U+FC86 */
  0x239C,   /* U+FC87 */
  0x23A3,   /* U+FC88 */
  0x23A3,   /* U+FC89 */
  0x23A7,   /* U+FC8A */
  0x23A7,   /* U+FC8B */
  0x23A7,   /* U+FC8C */
  0x23A7,   /* U+FC8D */
  0x23A7,   /* U+FC8E */
  0x23A7,   /* U+FC8F */
  0x23C5,   /* U+FC90 */
  0x23C6,   /* U+FC91 */
  0x23C6,   /* U+FC92 */
  0x23C6,   /* U+FC93 */
  0x23C6,   /* U+FC94 */
  0x23C6,   /* U+FC95 */
  0x23C6,   /* U+FC96 */
  0x2307,   /* U+FC97 */
  0x2307,   /* U+FC98 */
  0x2307,   /* U+FC99 */
  0x2307,   /* U+FC9A */
  0x2307,   /* U+FC9B */
  0x230D,   /* U+FC9C */
  0x230D,   /* U+FC9D */
  0x230D,   /* U+FC9E */
  0x230D,   /* U+FC9F */
  0x230D,   /* U+FCA0 */
  0x231D,   /* U+FCA1 */
  0x231D,   /* U+FCA2 */
  0x231D,   /* U+FCA3 */
  0x231D,   /* U+FCA4 */
  0x231D,   /* U+FCA5 */
  0x231E,   /* U+FCA6 */
  0x2325,   /* U+FCA7 */
  0x2325,   /* U+FCA8 */
  0x232C,   /* U+FCA9 */
  0x232C,   /* U+FCAA */
  0x232D,   /* U+FCAB */
  0x232D,   /* U+FCAC */
  0x2359,   /* U+FCAD */
  0x2359,   /* U+FCAE */
  0x2359,   /* U+FCAF */
  0x2359,   /* U+FCB0 */
  0x2364,   /* U+FCB1 */
  0x2364,   /* U+FCB2 */
  0x2364,   /* U+FCB3 */
  0x2365,   /* U+FCB4 */
  0x2365,   /* U+FCB5 */
  0x2365,   /* U+FCB6 */
  0x2365,   /* U+FCB7 */
  0x236A,   /* U+FCB8 */
  0x236B,   /* U+FCB9 */
  0x236E,   /* U+FCBA */
  0x236E,   /* U+FCBB */
  0x236F,   /* U+FCBC */
  0x236F,   /* U+FCBD */
  0x2376,   /* U+FCBE */
  0x2376,   /* U+FCBF */
  0x2376,   /* U+FCC0 */
  0x2376,   /* U+FCC1 */
  0x2382,   /* U+FCC2 */
  0x2382,   /* U+FCC3 */
  0x2387,   /* U+FCC4 */
  0x2387,   /* U+FCC5 */
  0x2387,   /* U+FCC6 */
  0x2387,   /* U+FCC7 */
  0x2387,   /* U+FCC8 */
  0x239C,   /* U+FCC9 */
  0x239C,   /* U+FCCA */
  0x239C,   /* U+FCCB */
  0x239C,   /* U+FCCC */
  0x239C,   /* U+FCCD */
  0x23A3,   /* U+FCCE */
  0x23A3,   /* U+FCCF */
  0x23A3,   /* U+FCD0 */
  0x23A3,   /* U+FCD1 */
  0x23A7,   /* U+FCD2 */
  0x23A7,   /* U+FCD3 */
  0x23A7,   /* U+FCD4 */
  0x23A7,   /* U+FCD5 */
  0x23A7,   /* U+FCD6 */
  0x23B1,   /* U+FCD7 */
  0x23B1,   /* U+FCD8 */
  0x23B1,   /* U+FCD9 */
  0x23C6,   /* U+FCDA */
  0x23C6,   /* U+FCDB */
  0x23C6,   /* U+FCDC */
  0x23C6,   /* U+FCDD */
  0x23C6,   /* U+FCDE */
  0x2307,   /* U+FCDF */
  0x2307,   /* U+FCE0 */
  0x230D,   /* U+FCE1 */
  0x230D,   /* U+FCE2 */
  0x231D,   /* U+FCE3 */
  0x231D,   /* U+FCE4 */
  0x231E,   /* U+FCE5 */
  0x231E,   /* U+FCE6 */
  0x2359,   /* U+FCE7 */
  0x2359,   /* U+FCE8 */
  0x235A,   /* U+FCE9 */
  0x235A,   /* U+FCEA */
  0x2387,   /* U+FCEB */
  0x2387,   /* U+FCEC */
  0x239C,   /* U+FCED */
  0x23A7,   /* U+FCEE */
  0x23A7,   /* U+FCEF */
  0x23C6,   /* U+FCF0 */
  0x23C6,   /* U+FCF1 */
  0x0000,   /* U+FCF2 */
  0x0000,   /* U+FCF3 */
  0x0000,   /* U+FCF4 */
  0x236A,   /* U+FCF5 */
  0x236A,   /* U+FCF6 */
  0x236E,   /* U+FCF7 */
  0x236E,   /* U+FCF8 */
  0x236F,   /* U+FCF9 */
  0x236F,   /* U+FCFA */
  0x2359,   /* U+FCFB */
  0x2359,   /* U+FCFC */
  0x235A,   /* U+FCFD */
  0x235A,   /* U+FCFE */
  0x232C,   /* U+FCFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+FC00 */
  0x0020,   /* U+FC01 */
  0x0020,   /* U+FC02 */
  0x0020,   /* U+FC03 */
  0x0020,   /* U+FC04 */
  0x0020,   /* U+FC05 */
  0x0020,   /* U+FC06 */
  0x0020,   /* U+FC07 */
  0x0020,   /* U+FC08 */
  0x0020,   /* U+FC09 */
  0x0020,   /* U+FC0A */
  0x0020,   /* U+FC0B */
  0x0020,   /* U+FC0C */
  0x0020,   /* U+FC0D */
  0x0020,   /* U+FC0E */
  0x0020,   /* U+FC0F */
  0x0020,   /* U+FC10 */
  0x0020,   /* U+FC11 */
  0x0020,   /* U+FC12 */
  0x0020,   /* U+FC13 */
  0x0020,   /* U+FC14 */
  0x0020,   /* U+FC15 */
  0x0020,   /* U+FC16 */
  0x0020,   /* U+FC17 */
  0x0020,   /* U+FC18 */
  0x0020,   /* U+FC19 */
  0x0020,   /* U+FC1A */
  0x0020,   /* U+FC1B */
  0x0020,   /* U+FC1C */
  0x0020,   /* U+FC1D */
  0x0020,   /* U+FC1E */
  0x0020,   /* U+FC1F */
  0x0020,   /* U+FC20 */
  0x0020,   /* U+FC21 */
  0x0020,   /* U+FC22 */
  0x0020,   /* U+FC23 */
  0x0020,   /* U+FC24 */
  0x0020,   /* U+FC25 */
  0x0020,   /* U+FC26 */
  0x0020,   /* U+FC27 */
  0x0020,   /* U+FC28 */
  0x0020,   /* U+FC29 */
  0x0020,   /* U+FC2A */
  0x0020,   /* U+FC2B */
  0x0020,   /* U+FC2C */
  0x0020,   /* U+FC2D */
  0x0020,   /* U+FC2E */
  0x0020,   /* U+FC2F */
  0x0020,   /* U+FC30 */
  0x0020,   /* U+FC31 */
  0x0020,   /* U+FC32 */
  0x0020,   /* U+FC33 */
  0x0020,   /* U+FC34 */
  0x0020,   /* U+FC35 */
  0x0020,   /* U+FC36 */
  0x0020,   /* U+FC37 */
  0x0020,   /* U+FC38 */
  0x0020,   /* U+FC39 */
  0x0020,   /* U+FC3A */
  0x0020,   /* U+FC3B */
  0x0020,   /* U+FC3C */
  0x0020,   /* U+FC3D */
  0x0020,   /* U+FC3E */
  0x0020,   /* U+FC3F */
  0x0020,   /* U+FC40 */
  0x0020,   /* U+FC41 */
  0x0020,   /* U+FC42 */
  0x0020,   /* U+FC43 */
  0x0020,   /* U+FC44 */
  0x0020,   /* U+FC45 */
  0x0020,   /* U+FC46 */
  0x0020,   /* U+FC47 */
  0x0020,   /* U+FC48 */
  0x0020,   /* U+FC49 */
  0x0020,   /* U+FC4A */
  0x0020,   /* U+FC4B */
  0x0020,   /* U+FC4C */
  0x0020,   /* U+FC4D */
  0x0020,   /* U+FC4E */
  0x0020,   /* U+FC4F */
  0x0020,   /* U+FC50 */
  0x0020,   /* U+FC51 */
  0x0020,   /* U+FC52 */
  0x0020,   /* U+FC53 */
  0x0020,   /* U+FC54 */
  0x0020,   /* U+FC55 */
  0x0020,   /* U+FC56 */
  0x0020,   /* U+FC57 */
  0x0020,   /* U+FC58 */
  0x0020,   /* U+FC59 */
  0x0020,   /* U+FC5A */
  0x0020,   /* U+FC5B */
  0x0020,   /* U+FC5C */
  0x0020,   /* U+FC5D */
  0x0070,   /* U+FC5E */
  0x0073,   /* U+FC5F */
  0x0076,   /* U+FC60 */
  0x007A,   /* U+FC61 */
  0x007D,   /* U+FC62 */
  0x0080,   /* U+FC63 */
  0x0020,   /* U+FC64 */
  0x0020,   /* U+FC65 */
  0x0020,   /* U+FC66 */
  0x0020,   /* U+FC67 */
  0x0020,   /* U+FC68 */
  0x0020,   /* U+FC69 */
  0x0020,   /* U+FC6A */
  0x0020,   /* U+FC6B */
  0x0020,   /* U+FC6C */
  0x0020,   /* U+FC6D */
  0x0020,   /* U+FC6E */
  0x0020,   /* U+FC6F */
  0x0020,   /* U+FC70 */
  0x0020,   /* U+FC71 */
  0x0020,   /* U+FC72 */
  0x0020,   /* U+FC73 */
  0x0020,   /* U+FC74 */
  0x0020,   /* U+FC75 */
  0x0020,   /* U+FC76 */
  0x0020,   /* U+FC77 */
  0x0020,   /* U+FC78 */
  0x0020,   /* U+FC79 */
  0x0020,   /* U+FC7A */
  0x0020,   /* U+FC7B */
  0x0020,   /* U+FC7C */
  0x0020,   /* U+FC7D */
  0x0020,   /* U+FC7E */
  0x0020,   /* U+FC7F */
  0x0020,   /* U+FC80 */
  0x0020,   /* U+FC81 */
  0x0020,   /* U+FC82 */
  0x0020,   /* U+FC83 */
  0x0020,   /* U+FC84 */
  0x0020,   /* U+FC85 */
  0x0020,   /* U+FC86 */
  0x0020,   /* U+FC87 */
  0x0020,   /* U+FC88 */
  0x0020,   /* U+FC89 */
  0x0020,   /* U+FC8A */
  0x0020,   /* U+FC8B */
  0x0020,   /* U+FC8C */
  0x0020,   /* U+FC8D */
  0x0020,   /* U+FC8E */
  0x0020,   /* U+FC8F */
  0x0020,   /* U+FC90 */
  0x0020,   /* U+FC91 */
  0x0020,   /* U+FC92 */
  0x0020,   /* U+FC93 */
  0x0020,   /* U+FC94 */
  0x0020,   /* U+FC95 */
  0x0020,   /* U+FC96 */
  0x0020,   /* U+FC97 */
  0x0020,   /* U+FC98 */
  0x0020,   /* U+FC99 */
  0x0020,   /* U+FC9A */
  0x0020,   /* U+FC9B */
  0x0020,   /* U+FC9C */
  0x0020,   /* U+FC9D */
  0x0020,   /* U+FC9E */
  0x0020,   /* U+FC9F */
  0x0020,   /* U+FCA0 */
  0x0020,   /* U+FCA1 */
  0x0020,   /* U+FCA2 */
  0x0020,   /* U+FCA3 */
  0x0020,   /* U+FCA4 */
  0x0020,   /* U+FCA5 */
  0x0020,   /* U+FCA6 */
  0x0020,   /* U+FCA7 */
  0x0020,   /* U+FCA8 */
  0x0020,   /* U+FCA9 */
  0x0020,   /* U+FCAA */
  0x0020,   /* U+FCAB */
  0x0020,   /* U+FCAC */
  0x0020,   /* U+FCAD */
  0x0020,   /* U+FCAE */
  0x0020,   /* U+FCAF */
  0x0020,   /* U+FCB0 */
  0x0020,   /* U+FCB1 */
  0x0020,   /* U+FCB2 */
  0x0020,   /* U+FCB3 */
  0x0020,   /* U+FCB4 */
  0x0020,   /* U+FCB5 */
  0x0020,   /* U+FCB6 */
  0x0020,   /* U+FCB7 */
  0x0020,   /* U+FCB8 */
  0x0020,   /* U+FCB9 */
  0x0020,   /* U+FCBA */
  0x0020,   /* U+FCBB */
  0x0020,   /* U+FCBC */
  0x0020,   /* U+FCBD */
  0x0020,   /* U+FCBE */
  0x0020,   /* U+FCBF */
  0x0020,   /* U+FCC0 */
  0x0020,   /* U+FCC1 */
  0x0020,   /* U+FCC2 */
  0x0020,   /* U+FCC3 */
  0x0020,   /* U+FCC4 */
  0x0020,   /* U+FCC5 */
  0x0020,   /* U+FCC6 */
  0x0020,   /* U+FCC7 */
  0x0020,   /* U+FCC8 */
  0x0020,   /* U+FCC9 */
  0x0020,   /* U+FCCA */
  0x0020,   /* U+FCCB */
  0x0020,   /* U+FCCC */
  0x0020,   /* U+FCCD */
  0x0020,   /* U+FCCE */
  0x0020,   /* U+FCCF */
  0x0020,   /* U+FCD0 */
  0x0020,   /* U+FCD1 */
  0x0020,   /* U+FCD2 */
  0x0020,   /* U+FCD3 */
  0x0020,   /* U+FCD4 */
  0x0020,   /* U+FCD5 */
  0x0020,   /* U+FCD6 */
  0x0020,   /* U+FCD7 */
  0x0020,   /* U+FCD8 */
  0x0020,   /* U+FCD9 */
  0x0020,   /* U+FCDA */
  0x0020,   /* U+FCDB */
  0x0020,   /* U+FCDC */
  0x0020,   /* U+FCDD */
  0x0020,   /* U+FCDE */
  0x0020,   /* U+FCDF */
  0x0020,   /* U+FCE0 */
  0x0020,   /* U+FCE1 */
  0x0020,   /* U+FCE2 */
  0x0020,   /* U+FCE3 */
  0x0020,   /* U+FCE4 */
  0x0020,   /* U+FCE5 */
  0x0020,   /* U+FCE6 */
  0x0020,   /* U+FCE7 */
  0x0020,   /* U+FCE8 */
  0x0020,   /* U+FCE9 */
  0x0020,   /* U+FCEA */
  0x0020,   /* U+FCEB */
  0x0020,   /* U+FCEC */
  0x0020,   /* U+FCED */
  0x0020,   /* U+FCEE */
  0x0020,   /* U+FCEF */
  0x0020,   /* U+FCF0 */
  0x0020,   /* U+FCF1 */
  0x0076,   /* U+FCF2 */
  0x007A,   /* U+FCF3 */
  0x007D,   /* U+FCF4 */
  0x0020,   /* U+FCF5 */
  0x0020,   /* U+FCF6 */
  0x0020,   /* U+FCF7 */
  0x0020,   /* U+FCF8 */
  0x0020,   /* U+FCF9 */
  0x0020,   /* U+FCFA */
  0x0020,   /* U+FCFB */
  0x0020,   /* U+FCFC */
  0x0020,   /* U+FCFD */
  0x0020,   /* U+FCFE */
  0x0020,   /* U+FCFF */

  /* Tertiary weight 1 for each character. */
  0x001A,   /* U+FC00 */
  0x001A,   /* U+FC01 */
  0x001A,   /* U+FC02 */
  0x001A,   /* U+FC03 */
  0x001A,   /* U+FC04 */
  0x001A,   /* U+FC05 */
  0x001A,   /* U+FC06 */
  0x001A,   /* U+FC07 */
  0x001A,   /* U+FC08 */
  0x001A,   /* U+FC09 */
  0x001A,   /* U+FC0A */
  0x001A,   /* U+FC0B */
  0x001A,   /* U+FC0C */
  0x001A,   /* U+FC0D */
  0x001A,   /* U+FC0E */
  0x001A,   /* U+FC0F */
  0x001A,   /* U+FC10 */
  0x001A,   /* U+FC11 */
  0x001A,   /* U+FC12 */
  0x001A,   /* U+FC13 */
  0x001A,   /* U+FC14 */
  0x001A,   /* U+FC15 */
  0x001A,   /* U+FC16 */
  0x001A,   /* U+FC17 */
  0x001A,   /* U+FC18 */
  0x001A,   /* U+FC19 */
  0x001A,   /* U+FC1A */
  0x001A,   /* U+FC1B */
  0x001A,   /* U+FC1C */
  0x001A,   /* U+FC1D */
  0x001A,   /* U+FC1E */
  0x001A,   /* U+FC1F */
  0x001A,   /* U+FC20 */
  0x001A,   /* U+FC21 */
  0x001A,   /* U+FC22 */
  0x001A,   /* U+FC23 */
  0x001A,   /* U+FC24 */
  0x001A,   /* U+FC25 */
  0x001A,   /* U+FC26 */
  0x001A,   /* U+FC27 */
  0x001A,   /* U+FC28 */
  0x001A,   /* U+FC29 */
  0x001A,   /* U+FC2A */
  0x001A,   /* U+FC2B */
  0x001A,   /* U+FC2C */
  0x001A,   /* U+FC2D */
  0x001A,   /* U+FC2E */
  0x001A,   /* U+FC2F */
  0x001A,   /* U+FC30 */
  0x001A,   /* U+FC31 */
  0x001A,   /* U+FC32 */
  0x001A,   /* U+FC33 */
  0x001A,   /* U+FC34 */
  0x001A,   /* U+FC35 */
  0x001A,   /* U+FC36 */
  0x001A,   /* U+FC37 */
  0x001A,   /* U+FC38 */
  0x001A,   /* U+FC39 */
  0x001A,   /* U+FC3A */
  0x001A,   /* U+FC3B */
  0x001A,   /* U+FC3C */
  0x001A,   /* U+FC3D */
  0x001A,   /* U+FC3E */
  0x001A,   /* U+FC3F */
  0x001A,   /* U+FC40 */
  0x001A,   /* U+FC41 */
  0x001A,   /* U+FC42 */
  0x001A,   /* U+FC43 */
  0x001A,   /* U+FC44 */
  0x001A,   /* U+FC45 */
  0x001A,   /* U+FC46 */
  0x001A,   /* U+FC47 */
  0x001A,   /* U+FC48 */
  0x001A,   /* U+FC49 */
  0x001A,   /* U+FC4A */
  0x001A,   /* U+FC4B */
  0x001A,   /* U+FC4C */
  0x001A,   /* U+FC4D */
  0x001A,   /* U+FC4E */
  0x001A,   /* U+FC4F */
  0x001A,   /* U+FC50 */
  0x001A,   /* U+FC51 */
  0x001A,   /* U+FC52 */
  0x001A,   /* U+FC53 */
  0x001A,   /* U+FC54 */
  0x001A,   /* U+FC55 */
  0x001A,   /* U+FC56 */
  0x001A,   /* U+FC57 */
  0x001A,   /* U+FC58 */
  0x001A,   /* U+FC59 */
  0x001A,   /* U+FC5A */
  0x001A,   /* U+FC5B */
  0x001A,   /* U+FC5C */
  0x001A,   /* U+FC5D */
  0x001A,   /* U+FC5E */
  0x001A,   /* U+FC5F */
  0x001A,   /* U+FC60 */
  0x001A,   /* U+FC61 */
  0x001A,   /* U+FC62 */
  0x001A,   /* U+FC63 */
  0x0019,   /* U+FC64 */
  0x0019,   /* U+FC65 */
  0x0019,   /* U+FC66 */
  0x0019,   /* U+FC67 */
  0x0019,   /* U+FC68 */
  0x0019,   /* U+FC69 */
  0x0019,   /* U+FC6A */
  0x0019,   /* U+FC6B */
  0x0019,   /* U+FC6C */
  0x0019,   /* U+FC6D */
  0x0019,   /* U+FC6E */
  0x0019,   /* U+FC6F */
  0x0019,   /* U+FC70 */
  0x0019,   /* U+FC71 */
  0x0019,   /* U+FC72 */
  0x0019,   /* U+FC73 */
  0x0019,   /* U+FC74 */
  0x0019,   /* U+FC75 */
  0x0019,   /* U+FC76 */
  0x0019,   /* U+FC77 */
  0x0019,   /* U+FC78 */
  0x0019,   /* U+FC79 */
  0x0019,   /* U+FC7A */
  0x0019,   /* U+FC7B */
  0x0019,   /* U+FC7C */
  0x0019,   /* U+FC7D */
  0x0019,   /* U+FC7E */
  0x0019,   /* U+FC7F */
  0x0019,   /* U+FC80 */
  0x0019,   /* U+FC81 */
  0x0019,   /* U+FC82 */
  0x0019,   /* U+FC83 */
  0x0019,   /* U+FC84 */
  0x0019,   /* U+FC85 */
  0x0019,   /* U+FC86 */
  0x0019,   /* U+FC87 */
  0x0019,   /* U+FC88 */
  0x0019,   /* U+FC89 */
  0x0019,   /* U+FC8A */
  0x0019,   /* U+FC8B */
  0x0019,   /* U+FC8C */
  0x0019,   /* U+FC8D */
  0x0019,   /* U+FC8E */
  0x0019,   /* U+FC8F */
  0x0019,   /* U+FC90 */
  0x0019,   /* U+FC91 */
  0x0019,   /* U+FC92 */
  0x0019,   /* U+FC93 */
  0x0019,   /* U+FC94 */
  0x0019,   /* U+FC95 */
  0x0019,   /* U+FC96 */
  0x0017,   /* U+FC97 */
  0x0017,   /* U+FC98 */
  0x0017,   /* U+FC99 */
  0x0017,   /* U+FC9A */
  0x0017,   /* U+FC9B */
  0x0017,   /* U+FC9C */
  0x0017,   /* U+FC9D */
  0x0017,   /* U+FC9E */
  0x0017,   /* U+FC9F */
  0x0017,   /* U+FCA0 */
  0x0017,   /* U+FCA1 */
  0x0017,   /* U+FCA2 */
  0x0017,   /* U+FCA3 */
  0x0017,   /* U+FCA4 */
  0x0017,   /* U+FCA5 */
  0x0017,   /* U+FCA6 */
  0x0017,   /* U+FCA7 */
  0x0017,   /* U+FCA8 */
  0x0017,   /* U+FCA9 */
  0x0017,   /* U+FCAA */
  0x0017,   /* U+FCAB */
  0x0017,   /* U+FCAC */
  0x0017,   /* U+FCAD */
  0x0017,   /* U+FCAE */
  0x0017,   /* U+FCAF */
  0x0017,   /* U+FCB0 */
  0x0017,   /* U+FCB1 */
  0x0017,   /* U+FCB2 */
  0x0017,   /* U+FCB3 */
  0x0017,   /* U+FCB4 */
  0x0017,   /* U+FCB5 */
  0x0017,   /* U+FCB6 */
  0x0017,   /* U+FCB7 */
  0x0017,   /* U+FCB8 */
  0x0017,   /* U+FCB9 */
  0x0017,   /* U+FCBA */
  0x0017,   /* U+FCBB */
  0x0017,   /* U+FCBC */
  0x0017,   /* U+FCBD */
  0x0017,   /* U+FCBE */
  0x0017,   /* U+FCBF */
  0x0017,   /* U+FCC0 */
  0x0017,   /* U+FCC1 */
  0x0017,   /* U+FCC2 */
  0x0017,   /* U+FCC3 */
  0x0017,   /* U+FCC4 */
  0x0017,   /* U+FCC5 */
  0x0017,   /* U+FCC6 */
  0x0017,   /* U+FCC7 */
  0x0017,   /* U+FCC8 */
  0x0017,   /* U+FCC9 */
  0x0017,   /* U+FCCA */
  0x0017,   /* U+FCCB */
  0x0017,   /* U+FCCC */
  0x0017,   /* U+FCCD */
  0x0017,   /* U+FCCE */
  0x0017,   /* U+FCCF */
  0x0017,   /* U+FCD0 */
  0x0017,   /* U+FCD1 */
  0x0017,   /* U+FCD2 */
  0x0017,   /* U+FCD3 */
  0x0017,   /* U+FCD4 */
  0x0017,   /* U+FCD5 */
  0x0017,   /* U+FCD6 */
  0x0017,   /* U+FCD7 */
  0x0017,   /* U+FCD8 */
  0x0017,   /* U+FCD9 */
  0x0017,   /* U+FCDA */
  0x0017,   /* U+FCDB */
  0x0017,   /* U+FCDC */
  0x0017,   /* U+FCDD */
  0x0017,   /* U+FCDE */
  0x0018,   /* U+FCDF */
  0x0018,   /* U+FCE0 */
  0x0018,   /* U+FCE1 */
  0x0018,   /* U+FCE2 */
  0x0018,   /* U+FCE3 */
  0x0018,   /* U+FCE4 */
  0x0018,   /* U+FCE5 */
  0x0018,   /* U+FCE6 */
  0x0018,   /* U+FCE7 */
  0x0018,   /* U+FCE8 */
  0x0018,   /* U+FCE9 */
  0x0018,   /* U+FCEA */
  0x0018,   /* U+FCEB */
  0x0018,   /* U+FCEC */
  0x0018,   /* U+FCED */
  0x0018,   /* U+FCEE */
  0x0018,   /* U+FCEF */
  0x0018,   /* U+FCF0 */
  0x0018,   /* U+FCF1 */
  0x0018,   /* U+FCF2 */
  0x0018,   /* U+FCF3 */
  0x0018,   /* U+FCF4 */
  0x001A,   /* U+FCF5 */
  0x001A,   /* U+FCF6 */
  0x001A,   /* U+FCF7 */
  0x001A,   /* U+FCF8 */
  0x001A,   /* U+FCF9 */
  0x001A,   /* U+FCFA */
  0x001A,   /* U+FCFB */
  0x001A,   /* U+FCFC */
  0x001A,   /* U+FCFD */
  0x001A,   /* U+FCFE */
  0x001A,   /* U+FCFF */

  /* Primary weight 2 for each character. */
  0x2325,   /* U+FC00 */
  0x232C,   /* U+FC01 */
  0x23A3,   /* U+FC02 */
  0x23C5,   /* U+FC03 */
  0x23C6,   /* U+FC04 */
  0x2325,   /* U+FC05 */
  0x232C,   /* U+FC06 */
  0x232D,   /* U+FC07 */
  0x23A3,   /* U+FC08 */
  0x23C5,   /* U+FC09 */
  0x23C6,   /* U+FC0A */
  0x2325,   /* U+FC0B */
  0x232C,   /* U+FC0C */
  0x232D,   /* U+FC0D */
  0x23A3,   /* U+FC0E */
  0x23C5,   /* U+FC0F */
  0x23C6,   /* U+FC10 */
  0x2325,   /* U+FC11 */
  0x23A3,   /* U+FC12 */
  0x23C5,   /* U+FC13 */
  0x23C6,   /* U+FC14 */
  0x232C,   /* U+FC15 */
  0x23A3,   /* U+FC16 */
  0x2325,   /* U+FC17 */
  0x23A3,   /* U+FC18 */
  0x2325,   /* U+FC19 */
  0x232C,   /* U+FC1A */
  0x23A3,   /* U+FC1B */
  0x2325,   /* U+FC1C */
  0x232C,   /* U+FC1D */
  0x232D,   /* U+FC1E */
  0x23A3,   /* U+FC1F */
  0x232C,   /* U+FC20 */
  0x23A3,   /* U+FC21 */
  0x2325,   /* U+FC22 */
  0x232C,   /* U+FC23 */
  0x232D,   /* U+FC24 */
  0x23A3,   /* U+FC25 */
  0x232C,   /* U+FC26 */
  0x23A3,   /* U+FC27 */
  0x23A3,   /* U+FC28 */
  0x2325,   /* U+FC29 */
  0x23A3,   /* U+FC2A */
  0x2325,   /* U+FC2B */
  0x23A3,   /* U+FC2C */
  0x2325,   /* U+FC2D */
  0x232C,   /* U+FC2E */
  0x232D,   /* U+FC2F */
  0x23A3,   /* U+FC30 */
  0x23C5,   /* U+FC31 */
  0x23C6,   /* U+FC32 */
  0x232C,   /* U+FC33 */
  0x23A3,   /* U+FC34 */
  0x23C5,   /* U+FC35 */
  0x23C6,   /* U+FC36 */
  0x230B,   /* U+FC37 */
  0x2325,   /* U+FC38 */
  0x232C,   /* U+FC39 */
  0x232D,   /* U+FC3A */
  0x239C,   /* U+FC3B */
  0x23A3,   /* U+FC3C */
  0x23C5,   /* U+FC3D */
  0x23C6,   /* U+FC3E */
  0x2325,   /* U+FC3F */
  0x232C,   /* U+FC40 */
  0x232D,   /* U+FC41 */
  0x23A3,   /* U+FC42 */
  0x23C5,   /* U+FC43 */
  0x23C6,   /* U+FC44 */
  0x2325,   /* U+FC45 */
  0x232C,   /* U+FC46 */
  0x232D,   /* U+FC47 */
  0x23A3,   /* U+FC48 */
  0x23C5,   /* U+FC49 */
  0x23C6,   /* U+FC4A */
  0x2325,   /* U+FC4B */
  0x232C,   /* U+FC4C */
  0x232D,   /* U+FC4D */
  0x23A3,   /* U+FC4E */
  0x23C5,   /* U+FC4F */
  0x23C6,   /* U+FC50 */
  0x2325,   /* U+FC51 */
  0x23A3,   /* U+FC52 */
  0x23C5,   /* U+FC53 */
  0x23C6,   /* U+FC54 */
  0x2325,   /* U+FC55 */
  0x232C,   /* U+FC56 */
  0x232D,   /* U+FC57 */
  0x23A3,   /* U+FC58 */
  0x23C5,   /* U+FC59 */
  0x23C6,   /* U+FC5A */
  0x0000,   /* U+FC5B */
  0x0000,   /* U+FC5C */
  0x0000,   /* U+FC5D */
  0x0000,   /* U+FC5E */
  0x0000,   /* U+FC5F */
  0x0000,   /* U+FC60 */
  0x0000,   /* U+FC61 */
  0x0000,   /* U+FC62 */
  0x0000,   /* U+FC63 */
  0x2346,   /* U+FC64 */
  0x2347,   /* U+FC65 */
  0x23A3,   /* U+FC66 */
  0x23A7,   /* U+FC67 */
  0x23C5,   /* U+FC68 */
  0x23C6,   /* U+FC69 */
  0x2346,   /* U+FC6A */
  0x2347,   /* U+FC6B */
  0x23A3,   /* U+FC6C */
  0x23A7,   /* U+FC6D */
  0x23C5,   /* U+FC6E */
  0x23C6,   /* U+FC6F */
  0x2346,   /* U+FC70 */
  0x2347,   /* U+FC71 */
  0x23A3,   /* U+FC72 */
  0x23A7,   /* U+FC73 */
  0x23C5,   /* U+FC74 */
  0x23C6,   /* U+FC75 */
  0x2346,   /* U+FC76 */
  0x2347,   /* U+FC77 */
  0x23A3,   /* U+FC78 */
  0x23A7,   /* U+FC79 */
  0x23C5,   /* U+FC7A */
  0x23C6,   /* U+FC7B */
  0x23C5,   /* U+FC7C */
  0x23C6,   /* U+FC7D */
  0x23C5,   /* U+FC7E */
  0x23C6,   /* U+FC7F */
  0x230B,   /* U+FC80 */
  0x239C,   /* U+FC81 */
  0x23A3,   /* U+FC82 */
  0x23C5,   /* U+FC83 */
  0x23C6,   /* U+FC84 */
  0x23A3,   /* U+FC85 */
  0x23C5,   /* U+FC86 */
  0x23C6,   /* U+FC87 */
  0x230B,   /* U+FC88 */
  0x23A3,   /* U+FC89 */
  0x2346,   /* U+FC8A */
  0x2347,   /* U+FC8B */
  0x23A3,   /* U+FC8C */
  0x23A7,   /* U+FC8D */
  0x23C5,   /* U+FC8E */
  0x23C6,   /* U+FC8F */
  0x0000,   /* U+FC90 */
  0x2346,   /* U+FC91 */
  0x2347,   /* U+FC92 */
  0x23A3,   /* U+FC93 */
  0x23A7,   /* U+FC94 */
  0x23C5,   /* U+FC95 */
  0x23C6,   /* U+FC96 */
  0x2325,   /* U+FC97 */
  0x232C,   /* U+FC98 */
  0x232D,   /* U+FC99 */
  0x23A3,   /* U+FC9A */
  0x23B1,   /* U+FC9B */
  0x2325,   /* U+FC9C */
  0x232C,   /* U+FC9D */
  0x232D,   /* U+FC9E */
  0x23A3,   /* U+FC9F */
  0x23B1,   /* U+FCA0 */
  0x2325,   /* U+FCA1 */
  0x232C,   /* U+FCA2 */
  0x232D,   /* U+FCA3 */
  0x23A3,   /* U+FCA4 */
  0x23B1,   /* U+FCA5 */
  0x23A3,   /* U+FCA6 */
  0x232C,   /* U+FCA7 */
  0x23A3,   /* U+FCA8 */
  0x2325,   /* U+FCA9 */
  0x23A3,   /* U+FCAA */
  0x2325,   /* U+FCAB */
  0x23A3,   /* U+FCAC */
  0x2325,   /* U+FCAD */
  0x232C,   /* U+FCAE */
  0x232D,   /* U+FCAF */
  0x23A3,   /* U+FCB0 */
  0x232C,   /* U+FCB1 */
  0x232D,   /* U+FCB2 */
  0x23A3,   /* U+FCB3 */
  0x2325,   /* U+FCB4 */
  0x232C,   /* U+FCB5 */
  0x232D,   /* U+FCB6 */
  0x23A3,   /* U+FCB7 */
  0x232C,   /* U+FCB8 */
  0x23A3,   /* U+FCB9 */
  0x2325,   /* U+FCBA */
  0x23A3,   /* U+FCBB */
  0x2325,   /* U+FCBC */
  0x23A3,   /* U+FCBD */
  0x2325,   /* U+FCBE */
  0x232C,   /* U+FCBF */
  0x232D,   /* U+FCC0 */
  0x23A3,   /* U+FCC1 */
  0x232C,   /* U+FCC2 */
  0x23A3,   /* U+FCC3 */
  0x2325,   /* U+FCC4 */
  0x232C,   /* U+FCC5 */
  0x232D,   /* U+FCC6 */
  0x239C,   /* U+FCC7 */
  0x23A3,   /* U+FCC8 */
  0x2325,   /* U+FCC9 */
  0x232C,   /* U+FCCA */
  0x232D,   /* U+FCCB */
  0x23A3,   /* U+FCCC */
  0x23B1,   /* U+FCCD */
  0x2325,   /* U+FCCE */
  0x232C,   /* U+FCCF */
  0x232D,   /* U+FCD0 */
  0x23A3,   /* U+FCD1 */
  0x2325,   /* U+FCD2 */
  0x232C,   /* U+FCD3 */
  0x232D,   /* U+FCD4 */
  0x23A3,   /* U+FCD5 */
  0x23B1,   /* U+FCD6 */
  0x2325,   /* U+FCD7 */
  0x23A3,   /* U+FCD8 */
  0x0000,   /* U+FCD9 */
  0x2325,   /* U+FCDA */
  0x232C,   /* U+FCDB */
  0x232D,   /* U+FCDC */
  0x23A3,   /* U+FCDD */
  0x23B1,   /* U+FCDE */
  0x23A3,   /* U+FCDF */
  0x23B1,   /* U+FCE0 */
  0x23A3,   /* U+FCE1 */
  0x23B1,   /* U+FCE2 */
  0x23A3,   /* U+FCE3 */
  0x23B1,   /* U+FCE4 */
  0x23A3,   /* U+FCE5 */
  0x23B1,   /* U+FCE6 */
  0x23A3,   /* U+FCE7 */
  0x23B1,   /* U+FCE8 */
  0x23A3,   /* U+FCE9 */
  0x23B1,   /* U+FCEA */
  0x239C,   /* U+FCEB */
  0x23A3,   /* U+FCEC */
  0x23A3,   /* U+FCED */
  0x23A3,   /* U+FCEE */
  0x23B1,   /* U+FCEF */
  0x23A3,   /* U+FCF0 */
  0x23B1,   /* U+FCF1 */
  0x0000,   /* U+FCF2 */
  0x0000,   /* U+FCF3 */
  0x0000,   /* U+FCF4 */
  0x23C5,   /* U+FCF5 */
  0x23C6,   /* U+FCF6 */
  0x23C5,   /* U+FCF7 */
  0x23C6,   /* U+FCF8 */
  0x23C5,   /* U+FCF9 */
  0x23C6,   /* U+FCFA */
  0x23C5,   /* U+FCFB */
  0x23C6,   /* U+FCFC */
  0x23C5,   /* U+FCFD */
  0x23C6,   /* U+FCFE */
  0x23C5,   /* U+FCFF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+FC00 */
  0x0020,   /* U+FC01 */
  0x0020,   /* U+FC02 */
  0x0020,   /* U+FC03 */
  0x0020,   /* U+FC04 */
  0x0020,   /* U+FC05 */
  0x0020,   /* U+FC06 */
  0x0020,   /* U+FC07 */
  0x0020,   /* U+FC08 */
  0x0020,   /* U+FC09 */
  0x0020,   /* U+FC0A */
  0x0020,   /* U+FC0B */
  0x0020,   /* U+FC0C */
  0x0020,   /* U+FC0D */
  0x0020,   /* U+FC0E */
  0x0020,   /* U+FC0F */
  0x0020,   /* U+FC10 */
  0x0020,   /* U+FC11 */
  0x0020,   /* U+FC12 */
  0x0020,   /* U+FC13 */
  0x0020,   /* U+FC14 */
  0x0020,   /* U+FC15 */
  0x0020,   /* U+FC16 */
  0x0020,   /* U+FC17 */
  0x0020,   /* U+FC18 */
  0x0020,   /* U+FC19 */
  0x0020,   /* U+FC1A */
  0x0020,   /* U+FC1B */
  0x0020,   /* U+FC1C */
  0x0020,   /* U+FC1D */
  0x0020,   /* U+FC1E */
  0x0020,   /* U+FC1F */
  0x0020,   /* U+FC20 */
  0x0020,   /* U+FC21 */
  0x0020,   /* U+FC22 */
  0x0020,   /* U+FC23 */
  0x0020,   /* U+FC24 */
  0x0020,   /* U+FC25 */
  0x0020,   /* U+FC26 */
  0x0020,   /* U+FC27 */
  0x0020,   /* U+FC28 */
  0x0020,   /* U+FC29 */
  0x0020,   /* U+FC2A */
  0x0020,   /* U+FC2B */
  0x0020,   /* U+FC2C */
  0x0020,   /* U+FC2D */
  0x0020,   /* U+FC2E */
  0x0020,   /* U+FC2F */
  0x0020,   /* U+FC30 */
  0x0020,   /* U+FC31 */
  0x0020,   /* U+FC32 */
  0x0020,   /* U+FC33 */
  0x0020,   /* U+FC34 */
  0x0020,   /* U+FC35 */
  0x0020,   /* U+FC36 */
  0x0020,   /* U+FC37 */
  0x0020,   /* U+FC38 */
  0x0020,   /* U+FC39 */
  0x0020,   /* U+FC3A */
  0x0020,   /* U+FC3B */
  0x0020,   /* U+FC3C */
  0x0020,   /* U+FC3D */
  0x0020,   /* U+FC3E */
  0x0020,   /* U+FC3F */
  0x0020,   /* U+FC40 */
  0x0020,   /* U+FC41 */
  0x0020,   /* U+FC42 */
  0x0020,   /* U+FC43 */
  0x0020,   /* U+FC44 */
  0x0020,   /* U+FC45 */
  0x0020,   /* U+FC46 */
  0x0020,   /* U+FC47 */
  0x0020,   /* U+FC48 */
  0x0020,   /* U+FC49 */
  0x0020,   /* U+FC4A */
  0x0020,   /* U+FC4B */
  0x0020,   /* U+FC4C */
  0x0020,   /* U+FC4D */
  0x0020,   /* U+FC4E */
  0x0020,   /* U+FC4F */
  0x0020,   /* U+FC50 */
  0x0020,   /* U+FC51 */
  0x0020,   /* U+FC52 */
  0x0020,   /* U+FC53 */
  0x0020,   /* U+FC54 */
  0x0020,   /* U+FC55 */
  0x0020,   /* U+FC56 */
  0x0020,   /* U+FC57 */
  0x0020,   /* U+FC58 */
  0x0020,   /* U+FC59 */
  0x0020,   /* U+FC5A */
  0x0098,   /* U+FC5B */
  0x0098,   /* U+FC5C */
  0x0098,   /* U+FC5D */
  0x0080,   /* U+FC5E */
  0x0080,   /* U+FC5F */
  0x0080,   /* U+FC60 */
  0x0080,   /* U+FC61 */
  0x0080,   /* U+FC62 */
  0x0098,   /* U+FC63 */
  0x0020,   /* U+FC64 */
  0x0020,   /* U+FC65 */
  0x0020,   /* U+FC66 */
  0x0020,   /* U+FC67 */
  0x0020,   /* U+FC68 */
  0x0020,   /* U+FC69 */
  0x0020,   /* U+FC6A */
  0x0020,   /* U+FC6B */
  0x0020,   /* U+FC6C */
  0x0020,   /* U+FC6D */
  0x0020,   /* U+FC6E */
  0x0020,   /* U+FC6F */
  0x0020,   /* U+FC70 */
  0x0020,   /* U+FC71 */
  0x0020,   /* U+FC72 */
  0x0020,   /* U+FC73 */
  0x0020,   /* U+FC74 */
  0x0020,   /* U+FC75 */
  0x0020,   /* U+FC76 */
  0x0020,   /* U+FC77 */
  0x0020,   /* U+FC78 */
  0x0020,   /* U+FC79 */
  0x0020,   /* U+FC7A */
  0x0020,   /* U+FC7B */
  0x0020,   /* U+FC7C */
  0x0020,   /* U+FC7D */
  0x0020,   /* U+FC7E */
  0x0020,   /* U+FC7F */
  0x0020,   /* U+FC80 */
  0x0020,   /* U+FC81 */
  0x0020,   /* U+FC82 */
  0x0020,   /* U+FC83 */
  0x0020,   /* U+FC84 */
  0x0020,   /* U+FC85 */
  0x0020,   /* U+FC86 */
  0x0020,   /* U+FC87 */
  0x0020,   /* U+FC88 */
  0x0020,   /* U+FC89 */
  0x0020,   /* U+FC8A */
  0x0020,   /* U+FC8B */
  0x0020,   /* U+FC8C */
  0x0020,   /* U+FC8D */
  0x0020,   /* U+FC8E */
  0x0020,   /* U+FC8F */
  0x0098,   /* U+FC90 */
  0x0020,   /* U+FC91 */
  0x0020,   /* U+FC92 */
  0x0020,   /* U+FC93 */
  0x0020,   /* U+FC94 */
  0x0020,   /* U+FC95 */
  0x0020,   /* U+FC96 */
  0x0020,   /* U+FC97 */
  0x0020,   /* U+FC98 */
  0x0020,   /* U+FC99 */
  0x0020,   /* U+FC9A */
  0x0020,   /* U+FC9B */
  0x0020,   /* U+FC9C */
  0x0020,   /* U+FC9D */
  0x0020,   /* U+FC9E */
  0x0020,   /* U+FC9F */
  0x0020,   /* U+FCA0 */
  0x0020,   /* U+FCA1 */
  0x0020,   /* U+FCA2 */
  0x0020,   /* U+FCA3 */
  0x0020,   /* U+FCA4 */
  0x0020,   /* U+FCA5 */
  0x0020,   /* U+FCA6 */
  0x0020,   /* U+FCA7 */
  0x0020,   /* U+FCA8 */
  0x0020,   /* U+FCA9 */
  0x0020,   /* U+FCAA */
  0x0020,   /* U+FCAB */
  0x0020,   /* U+FCAC */
  0x0020,   /* U+FCAD */
  0x0020,   /* U+FCAE */
  0x0020,   /* U+FCAF */
  0x0020,   /* U+FCB0 */
  0x0020,   /* U+FCB1 */
  0x0020,   /* U+FCB2 */
  0x0020,   /* U+FCB3 */
  0x0020,   /* U+FCB4 */
  0x0020,   /* U+FCB5 */
  0x0020,   /* U+FCB6 */
  0x0020,   /* U+FCB7 */
  0x0020,   /* U+FCB8 */
  0x0020,   /* U+FCB9 */
  0x0020,   /* U+FCBA */
  0x0020,   /* U+FCBB */
  0x0020,   /* U+FCBC */
  0x0020,   /* U+FCBD */
  0x0020,   /* U+FCBE */
  0x0020,   /* U+FCBF */
  0x0020,   /* U+FCC0 */
  0x0020,   /* U+FCC1 */
  0x0020,   /* U+FCC2 */
  0x0020,   /* U+FCC3 */
  0x0020,   /* U+FCC4 */
  0x0020,   /* U+FCC5 */
  0x0020,   /* U+FCC6 */
  0x0020,   /* U+FCC7 */
  0x0020,   /* U+FCC8 */
  0x0020,   /* U+FCC9 */
  0x0020,   /* U+FCCA */
  0x0020,   /* U+FCCB */
  0x0020,   /* U+FCCC */
  0x0020,   /* U+FCCD */
  0x0020,   /* U+FCCE */
  0x0020,   /* U+FCCF */
  0x0020,   /* U+FCD0 */
  0x0020,   /* U+FCD1 */
  0x0020,   /* U+FCD2 */
  0x0020,   /* U+FCD3 */
  0x0020,   /* U+FCD4 */
  0x0020,   /* U+FCD5 */
  0x0020,   /* U+FCD6 */
  0x0020,   /* U+FCD7 */
  0x0020,   /* U+FCD8 */
  0x0098,   /* U+FCD9 */
  0x0020,   /* U+FCDA */
  0x0020,   /* U+FCDB */
  0x0020,   /* U+FCDC */
  0x0020,   /* U+FCDD */
  0x0020,   /* U+FCDE */
  0x0020,   /* U+FCDF */
  0x0020,   /* U+FCE0 */
  0x0020,   /* U+FCE1 */
  0x0020,   /* U+FCE2 */
  0x0020,   /* U+FCE3 */
  0x0020,   /* U+FCE4 */
  0x0020,   /* U+FCE5 */
  0x0020,   /* U+FCE6 */
  0x0020,   /* U+FCE7 */
  0x0020,   /* U+FCE8 */
  0x0020,   /* U+FCE9 */
  0x0020,   /* U+FCEA */
  0x0020,   /* U+FCEB */
  0x0020,   /* U+FCEC */
  0x0020,   /* U+FCED */
  0x0020,   /* U+FCEE */
  0x0020,   /* U+FCEF */
  0x0020,   /* U+FCF0 */
  0x0020,   /* U+FCF1 */
  0x0080,   /* U+FCF2 */
  0x0080,   /* U+FCF3 */
  0x0080,   /* U+FCF4 */
  0x0020,   /* U+FCF5 */
  0x0020,   /* U+FCF6 */
  0x0020,   /* U+FCF7 */
  0x0020,   /* U+FCF8 */
  0x0020,   /* U+FCF9 */
  0x0020,   /* U+FCFA */
  0x0020,   /* U+FCFB */
  0x0020,   /* U+FCFC */
  0x0020,   /* U+FCFD */
  0x0020,   /* U+FCFE */
  0x0020,   /* U+FCFF */

  /* Tertiary weight 2 for each character. */
  0x001A,   /* U+FC00 */
  0x001A,   /* U+FC01 */
  0x001A,   /* U+FC02 */
  0x001A,   /* U+FC03 */
  0x001A,   /* U+FC04 */
  0x001A,   /* U+FC05 */
  0x001A,   /* U+FC06 */
  0x001A,   /* U+FC07 */
  0x001A,   /* U+FC08 */
  0x001A,   /* U+FC09 */
  0x001A,   /* U+FC0A */
  0x001A,   /* U+FC0B */
  0x001A,   /* U+FC0C */
  0x001A,   /* U+FC0D */
  0x001A,   /* U+FC0E */
  0x001A,   /* U+FC0F */
  0x001A,   /* U+FC10 */
  0x001A,   /* U+FC11 */
  0x001A,   /* U+FC12 */
  0x001A,   /* U+FC13 */
  0x001A,   /* U+FC14 */
  0x001A,   /* U+FC15 */
  0x001A,   /* U+FC16 */
  0x001A,   /* U+FC17 */
  0x001A,   /* U+FC18 */
  0x001A,   /* U+FC19 */
  0x001A,   /* U+FC1A */
  0x001A,   /* U+FC1B */
  0x001A,   /* U+FC1C */
  0x001A,   /* U+FC1D */
  0x001A,   /* U+FC1E */
  0x001A,   /* U+FC1F */
  0x001A,   /* U+FC20 */
  0x001A,   /* U+FC21 */
  0x001A,   /* U+FC22 */
  0x001A,   /* U+FC23 */
  0x001A,   /* U+FC24 */
  0x001A,   /* U+FC25 */
  0x001A,   /* U+FC26 */
  0x001A,   /* U+FC27 */
  0x001A,   /* U+FC28 */
  0x001A,   /* U+FC29 */
  0x001A,   /* U+FC2A */
  0x001A,   /* U+FC2B */
  0x001A,   /* U+FC2C */
  0x001A,   /* U+FC2D */
  0x001A,   /* U+FC2E */
  0x001A,   /* U+FC2F */
  0x001A,   /* U+FC30 */
  0x001A,   /* U+FC31 */
  0x001A,   /* U+FC32 */
  0x001A,   /* U+FC33 */
  0x001A,   /* U+FC34 */
  0x001A,   /* U+FC35 */
  0x001A,   /* U+FC36 */
  0x001A,   /* U+FC37 */
  0x001A,   /* U+FC38 */
  0x001A,   /* U+FC39 */
  0x001A,   /* U+FC3A */
  0x001A,   /* U+FC3B */
  0x001A,   /* U+FC3C */
  0x001A,   /* U+FC3D */
  0x001A,   /* U+FC3E */
  0x001A,   /* U+FC3F */
  0x001A,   /* U+FC40 */
  0x001A,   /* U+FC41 */
  0x001A,   /* U+FC42 */
  0x001A,   /* U+FC43 */
  0x001A,   /* U+FC44 */
  0x001A,   /* U+FC45 */
  0x001A,   /* U+FC46 */
  0x001A,   /* U+FC47 */
  0x001A,   /* U+FC48 */
  0x001A,   /* U+FC49 */
  0x001A,   /* U+FC4A */
  0x001A,   /* U+FC4B */
  0x001A,   /* U+FC4C */
  0x001A,   /* U+FC4D */
  0x001A,   /* U+FC4E */
  0x001A,   /* U+FC4F */
  0x001A,   /* U+FC50 */
  0x001A,   /* U+FC51 */
  0x001A,   /* U+FC52 */
  0x001A,   /* U+FC53 */
  0x001A,   /* U+FC54 */
  0x001A,   /* U+FC55 */
  0x001A,   /* U+FC56 */
  0x001A,   /* U+FC57 */
  0x001A,   /* U+FC58 */
  0x001A,   /* U+FC59 */
  0x001A,   /* U+FC5A */
  0x001A,   /* U+FC5B */
  0x001A,   /* U+FC5C */
  0x001A,   /* U+FC5D */
  0x001A,   /* U+FC5E */
  0x001A,   /* U+FC5F */
  0x001A,   /* U+FC60 */
  0x001A,   /* U+FC61 */
  0x001A,   /* U+FC62 */
  0x001A,   /* U+FC63 */
  0x0019,   /* U+FC64 */
  0x0019,   /* U+FC65 */
  0x0019,   /* U+FC66 */
  0x0019,   /* U+FC67 */
  0x0019,   /* U+FC68 */
  0x0019,   /* U+FC69 */
  0x0019,   /* U+FC6A */
  0x0019,   /* U+FC6B */
  0x0019,   /* U+FC6C */
  0x0019,   /* U+FC6D */
  0x0019,   /* U+FC6E */
  0x0019,   /* U+FC6F */
  0x0019,   /* U+FC70 */
  0x0019,   /* U+FC71 */
  0x0019,   /* U+FC72 */
  0x0019,   /* U+FC73 */
  0x0019,   /* U+FC74 */
  0x0019,   /* U+FC75 */
  0x0019,   /* U+FC76 */
  0x0019,   /* U+FC77 */
  0x0019,   /* U+FC78 */
  0x0019,   /* U+FC79 */
  0x0019,   /* U+FC7A */
  0x0019,   /* U+FC7B */
  0x0019,   /* U+FC7C */
  0x0019,   /* U+FC7D */
  0x0019,   /* U+FC7E */
  0x0019,   /* U+FC7F */
  0x0019,   /* U+FC80 */
  0x0019,   /* U+FC81 */
  0x0019,   /* U+FC82 */
  0x0019,   /* U+FC83 */
  0x0019,   /* U+FC84 */
  0x0019,   /* U+FC85 */
  0x0019,   /* U+FC86 */
  0x0019,   /* U+FC87 */
  0x0019,   /* U+FC88 */
  0x0019,   /* U+FC89 */
  0x0019,   /* U+FC8A */
  0x0019,   /* U+FC8B */
  0x0019,   /* U+FC8C */
  0x0019,   /* U+FC8D */
  0x0019,   /* U+FC8E */
  0x0019,   /* U+FC8F */
  0x0019,   /* U+FC90 */
  0x0019,   /* U+FC91 */
  0x0019,   /* U+FC92 */
  0x0019,   /* U+FC93 */
  0x0019,   /* U+FC94 */
  0x0019,   /* U+FC95 */
  0x0019,   /* U+FC96 */
  0x0017,   /* U+FC97 */
  0x0017,   /* U+FC98 */
  0x0017,   /* U+FC99 */
  0x0017,   /* U+FC9A */
  0x0017,   /* U+FC9B */
  0x0017,   /* U+FC9C */
  0x0017,   /* U+FC9D */
  0x0017,   /* U+FC9E */
  0x0017,   /* U+FC9F */
  0x0017,   /* U+FCA0 */
  0x0017,   /* U+FCA1 */
  0x0017,   /* U+FCA2 */
  0x0017,   /* U+FCA3 */
  0x0017,   /* U+FCA4 */
  0x0017,   /* U+FCA5 */
  0x0017,   /* U+FCA6 */
  0x0017,   /* U+FCA7 */
  0x0017,   /* U+FCA8 */
  0x0017,   /* U+FCA9 */
  0x0017,   /* U+FCAA */
  0x0017,   /* U+FCAB */
  0x0017,   /* U+FCAC */
  0x0017,   /* U+FCAD */
  0x0017,   /* U+FCAE */
  0x0017,   /* U+FCAF */
  0x0017,   /* U+FCB0 */
  0x0017,   /* U+FCB1 */
  0x0017,   /* U+FCB2 */
  0x0017,   /* U+FCB3 */
  0x0017,   /* U+FCB4 */
  0x0017,   /* U+FCB5 */
  0x0017,   /* U+FCB6 */
  0x0017,   /* U+FCB7 */
  0x0017,   /* U+FCB8 */
  0x0017,   /* U+FCB9 */
  0x0017,   /* U+FCBA */
  0x0017,   /* U+FCBB */
  0x0017,   /* U+FCBC */
  0x0017,   /* U+FCBD */
  0x0017,   /* U+FCBE */
  0x0017,   /* U+FCBF */
  0x0017,   /* U+FCC0 */
  0x0017,   /* U+FCC1 */
  0x0017,   /* U+FCC2 */
  0x0017,   /* U+FCC3 */
  0x0017,   /* U+FCC4 */
  0x0017,   /* U+FCC5 */
  0x0017,   /* U+FCC6 */
  0x0017,   /* U+FCC7 */
  0x0017,   /* U+FCC8 */
  0x0017,   /* U+FCC9 */
  0x0017,   /* U+FCCA */
  0x0017,   /* U+FCCB */
  0x0017,   /* U+FCCC */
  0x0017,   /* U+FCCD */
  0x0017,   /* U+FCCE */
  0x0017,   /* U+FCCF */
  0x0017,   /* U+FCD0 */
  0x0017,   /* U+FCD1 */
  0x0017,   /* U+FCD2 */
  0x0017,   /* U+FCD3 */
  0x0017,   /* U+FCD4 */
  0x0017,   /* U+FCD5 */
  0x0017,   /* U+FCD6 */
  0x0017,   /* U+FCD7 */
  0x0017,   /* U+FCD8 */
  0x0017,   /* U+FCD9 */
  0x0017,   /* U+FCDA */
  0x0017,   /* U+FCDB */
  0x0017,   /* U+FCDC */
  0x0017,   /* U+FCDD */
  0x0017,   /* U+FCDE */
  0x0018,   /* U+FCDF */
  0x0018,   /* U+FCE0 */
  0x0018,   /* U+FCE1 */
  0x0018,   /* U+FCE2 */
  0x0018,   /* U+FCE3 */
  0x0018,   /* U+FCE4 */
  0x0018,   /* U+FCE5 */
  0x0018,   /* U+FCE6 */
  0x0018,   /* U+FCE7 */
  0x0018,   /* U+FCE8 */
  0x0018,   /* U+FCE9 */
  0x0018,   /* U+FCEA */
  0x0018,   /* U+FCEB */
  0x0018,   /* U+FCEC */
  0x0018,   /* U+FCED */
  0x0018,   /* U+FCEE */
  0x0018,   /* U+FCEF */
  0x0018,   /* U+FCF0 */
  0x0018,   /* U+FCF1 */
  0x0018,   /* U+FCF2 */
  0x0018,   /* U+FCF3 */
  0x0018,   /* U+FCF4 */
  0x001A,   /* U+FCF5 */
  0x001A,   /* U+FCF6 */
  0x001A,   /* U+FCF7 */
  0x001A,   /* U+FCF8 */
  0x001A,   /* U+FCF9 */
  0x001A,   /* U+FCFA */
  0x001A,   /* U+FCFB */
  0x001A,   /* U+FCFC */
  0x001A,   /* U+FCFD */
  0x001A,   /* U+FCFE */
  0x001A,   /* U+FCFF */
};

uint16 uca900_p0FD[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  3, 3, 4, 4, 4, 4, 4, 4, 4, 3, 8, 8, 4, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0x232C,   /* U+FD00 */
  0x2325,   /* U+FD01 */
  0x2325,   /* U+FD02 */
  0x232D,   /* U+FD03 */
  0x232D,   /* U+FD04 */
  0x2364,   /* U+FD05 */
  0x2364,   /* U+FD06 */
  0x2365,   /* U+FD07 */
  0x2365,   /* U+FD08 */
  0x235A,   /* U+FD09 */
  0x235A,   /* U+FD0A */
  0x235A,   /* U+FD0B */
  0x235A,   /* U+FD0C */
  0x235A,   /* U+FD0D */
  0x2359,   /* U+FD0E */
  0x2364,   /* U+FD0F */
  0x2365,   /* U+FD10 */
  0x236A,   /* U+FD11 */
  0x236A,   /* U+FD12 */
  0x236E,   /* U+FD13 */
  0x236E,   /* U+FD14 */
  0x236F,   /* U+FD15 */
  0x236F,   /* U+FD16 */
  0x2359,   /* U+FD17 */
  0x2359,   /* U+FD18 */
  0x235A,   /* U+FD19 */
  0x235A,   /* U+FD1A */
  0x232C,   /* U+FD1B */
  0x232C,   /* U+FD1C */
  0x2325,   /* U+FD1D */
  0x2325,   /* U+FD1E */
  0x232D,   /* U+FD1F */
  0x232D,   /* U+FD20 */
  0x2364,   /* U+FD21 */
  0x2364,   /* U+FD22 */
  0x2365,   /* U+FD23 */
  0x2365,   /* U+FD24 */
  0x235A,   /* U+FD25 */
  0x235A,   /* U+FD26 */
  0x235A,   /* U+FD27 */
  0x235A,   /* U+FD28 */
  0x235A,   /* U+FD29 */
  0x2359,   /* U+FD2A */
  0x2364,   /* U+FD2B */
  0x2365,   /* U+FD2C */
  0x235A,   /* U+FD2D */
  0x235A,   /* U+FD2E */
  0x235A,   /* U+FD2F */
  0x235A,   /* U+FD30 */
  0x2359,   /* U+FD31 */
  0x235A,   /* U+FD32 */
  0x236A,   /* U+FD33 */
  0x2359,   /* U+FD34 */
  0x2359,   /* U+FD35 */
  0x2359,   /* U+FD36 */
  0x235A,   /* U+FD37 */
  0x235A,   /* U+FD38 */
  0x235A,   /* U+FD39 */
  0x236A,   /* U+FD3A */
  0x236B,   /* U+FD3B */
  0x230B,   /* U+FD3C */
  0x230B,   /* U+FD3D */
  0x0381,   /* U+FD3E */
  0x0382,   /* U+FD3F */
  0xFBC1,   /* U+FD40 */
  0xFBC1,   /* U+FD41 */
  0xFBC1,   /* U+FD42 */
  0xFBC1,   /* U+FD43 */
  0xFBC1,   /* U+FD44 */
  0xFBC1,   /* U+FD45 */
  0xFBC1,   /* U+FD46 */
  0xFBC1,   /* U+FD47 */
  0xFBC1,   /* U+FD48 */
  0xFBC1,   /* U+FD49 */
  0xFBC1,   /* U+FD4A */
  0xFBC1,   /* U+FD4B */
  0xFBC1,   /* U+FD4C */
  0xFBC1,   /* U+FD4D */
  0xFBC1,   /* U+FD4E */
  0xFBC1,   /* U+FD4F */
  0x231D,   /* U+FD50 */
  0x231D,   /* U+FD51 */
  0x231D,   /* U+FD52 */
  0x231D,   /* U+FD53 */
  0x231D,   /* U+FD54 */
  0x231D,   /* U+FD55 */
  0x231D,   /* U+FD56 */
  0x231D,   /* U+FD57 */
  0x2325,   /* U+FD58 */
  0x2325,   /* U+FD59 */
  0x232C,   /* U+FD5A */
  0x232C,   /* U+FD5B */
  0x2359,   /* U+FD5C */
  0x2359,   /* U+FD5D */
  0x2359,   /* U+FD5E */
  0x2359,   /* U+FD5F */
  0x2359,   /* U+FD60 */
  0x2359,   /* U+FD61 */
  0x2359,   /* U+FD62 */
  0x2359,   /* U+FD63 */
  0x2364,   /* U+FD64 */
  0x2364,   /* U+FD65 */
  0x2364,   /* U+FD66 */
  0x235A,   /* U+FD67 */
  0x235A,   /* U+FD68 */
  0x235A,   /* U+FD69 */
  0x235A,   /* U+FD6A */
  0x235A,   /* U+FD6B */
  0x235A,   /* U+FD6C */
  0x235A,   /* U+FD6D */
  0x2365,   /* U+FD6E */
  0x2365,   /* U+FD6F */
  0x2365,   /* U+FD70 */
  0x236A,   /* U+FD71 */
  0x236A,   /* U+FD72 */
  0x236A,   /* U+FD73 */
  0x236A,   /* U+FD74 */
  0x236E,   /* U+FD75 */
  0x236E,   /* U+FD76 */
  0x236E,   /* U+FD77 */
  0x236E,   /* U+FD78 */
  0x236F,   /* U+FD79 */
  0x236F,   /* U+FD7A */
  0x236F,   /* U+FD7B */
  0x2376,   /* U+FD7C */
  0x2376,   /* U+FD7D */
  0x2382,   /* U+FD7E */
  0x2382,   /* U+FD7F */
  0x239C,   /* U+FD80 */
  0x239C,   /* U+FD81 */
  0x239C,   /* U+FD82 */
  0x239C,   /* U+FD83 */
  0x239C,   /* U+FD84 */
  0x239C,   /* U+FD85 */
  0x239C,   /* U+FD86 */
  0x239C,   /* U+FD87 */
  0x239C,   /* U+FD88 */
  0x23A3,   /* U+FD89 */
  0x23A3,   /* U+FD8A */
  0x23A3,   /* U+FD8B */
  0x23A3,   /* U+FD8C */
  0x23A3,   /* U+FD8D */
  0x23A3,   /* U+FD8E */
  0x23A3,   /* U+FD8F */
  0xFBC1,   /* U+FD90 */
  0xFBC1,   /* U+FD91 */
  0x23A3,   /* U+FD92 */
  0x23B1,   /* U+FD93 */
  0x23B1,   /* U+FD94 */
  0x23A7,   /* U+FD95 */
  0x23A7,   /* U+FD96 */
  0x23A7,   /* U+FD97 */
  0x23A7,   /* U+FD98 */
  0x23A7,   /* U+FD99 */
  0x23A7,   /* U+FD9A */
  0x23A7,   /* U+FD9B */
  0x23C6,   /* U+FD9C */
  0x23C6,   /* U+FD9D */
  0x230D,   /* U+FD9E */
  0x231D,   /* U+FD9F */
  0x231D,   /* U+FDA0 */
  0x231D,   /* U+FDA1 */
  0x231D,   /* U+FDA2 */
  0x231D,   /* U+FDA3 */
  0x231D,   /* U+FDA4 */
  0x2325,   /* U+FDA5 */
  0x2325,   /* U+FDA6 */
  0x2325,   /* U+FDA7 */
  0x2359,   /* U+FDA8 */
  0x2364,   /* U+FDA9 */
  0x235A,   /* U+FDAA */
  0x2365,   /* U+FDAB */
  0x239C,   /* U+FDAC */
  0x239C,   /* U+FDAD */
  0x23C6,   /* U+FDAE */
  0x23C6,   /* U+FDAF */
  0x23C6,   /* U+FDB0 */
  0x23A3,   /* U+FDB1 */
  0x2382,   /* U+FDB2 */
  0x23A7,   /* U+FDB3 */
  0x2382,   /* U+FDB4 */
  0x239C,   /* U+FDB5 */
  0x236E,   /* U+FDB6 */
  0x2387,   /* U+FDB7 */
  0x23A7,   /* U+FDB8 */
  0x23A3,   /* U+FDB9 */
  0x239C,   /* U+FDBA */
  0x2387,   /* U+FDBB */
  0x239C,   /* U+FDBC */
  0x23A7,   /* U+FDBD */
  0x2325,   /* U+FDBE */
  0x232C,   /* U+FDBF */
  0x23A3,   /* U+FDC0 */
  0x2376,   /* U+FDC1 */
  0x230D,   /* U+FDC2 */
  0x2387,   /* U+FDC3 */
  0x236E,   /* U+FDC4 */
  0x2364,   /* U+FDC5 */
  0x2359,   /* U+FDC6 */
  0x23A7,   /* U+FDC7 */
  0xFBC1,   /* U+FDC8 */
  0xFBC1,   /* U+FDC9 */
  0xFBC1,   /* U+FDCA */
  0xFBC1,   /* U+FDCB */
  0xFBC1,   /* U+FDCC */
  0xFBC1,   /* U+FDCD */
  0xFBC1,   /* U+FDCE */
  0xFBC1,   /* U+FDCF */
  0xFBC1,   /* U+FDD0 */
  0xFBC1,   /* U+FDD1 */
  0xFBC1,   /* U+FDD2 */
  0xFBC1,   /* U+FDD3 */
  0xFBC1,   /* U+FDD4 */
  0xFBC1,   /* U+FDD5 */
  0xFBC1,   /* U+FDD6 */
  0xFBC1,   /* U+FDD7 */
  0xFBC1,   /* U+FDD8 */
  0xFBC1,   /* U+FDD9 */
  0xFBC1,   /* U+FDDA */
  0xFBC1,   /* U+FDDB */
  0xFBC1,   /* U+FDDC */
  0xFBC1,   /* U+FDDD */
  0xFBC1,   /* U+FDDE */
  0xFBC1,   /* U+FDDF */
  0xFBC1,   /* U+FDE0 */
  0xFBC1,   /* U+FDE1 */
  0xFBC1,   /* U+FDE2 */
  0xFBC1,   /* U+FDE3 */
  0xFBC1,   /* U+FDE4 */
  0xFBC1,   /* U+FDE5 */
  0xFBC1,   /* U+FDE6 */
  0xFBC1,   /* U+FDE7 */
  0xFBC1,   /* U+FDE8 */
  0xFBC1,   /* U+FDE9 */
  0xFBC1,   /* U+FDEA */
  0xFBC1,   /* U+FDEB */
  0xFBC1,   /* U+FDEC */
  0xFBC1,   /* U+FDED */
  0xFBC1,   /* U+FDEE */
  0xFBC1,   /* U+FDEF */
  0x2364,   /* U+FDF0 */
  0x2382,   /* U+FDF1 */
  0x230B,   /* U+FDF2 */
  0x230B,   /* U+FDF3 */
  0x23A3,   /* U+FDF4 */
  0x2364,   /* U+FDF5 */
  0x2346,   /* U+FDF6 */
  0x236E,   /* U+FDF7 */
  0x23B7,   /* U+FDF8 */
  0x2364,   /* U+FDF9 */
  0x2364,   /* U+FDFA */
  0x2325,   /* U+FDFB */
  0x2346,   /* U+FDFC */
  0x0501,   /* U+FDFD */
  0xFBC1,   /* U+FDFE */
  0xFBC1,   /* U+FDFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+FD00 */
  0x0020,   /* U+FD01 */
  0x0020,   /* U+FD02 */
  0x0020,   /* U+FD03 */
  0x0020,   /* U+FD04 */
  0x0020,   /* U+FD05 */
  0x0020,   /* U+FD06 */
  0x0020,   /* U+FD07 */
  0x0020,   /* U+FD08 */
  0x0020,   /* U+FD09 */
  0x0020,   /* U+FD0A */
  0x0020,   /* U+FD0B */
  0x0020,   /* U+FD0C */
  0x0020,   /* U+FD0D */
  0x0020,   /* U+FD0E */
  0x0020,   /* U+FD0F */
  0x0020,   /* U+FD10 */
  0x0020,   /* U+FD11 */
  0x0020,   /* U+FD12 */
  0x0020,   /* U+FD13 */
  0x0020,   /* U+FD14 */
  0x0020,   /* U+FD15 */
  0x0020,   /* U+FD16 */
  0x0020,   /* U+FD17 */
  0x0020,   /* U+FD18 */
  0x0020,   /* U+FD19 */
  0x0020,   /* U+FD1A */
  0x0020,   /* U+FD1B */
  0x0020,   /* U+FD1C */
  0x0020,   /* U+FD1D */
  0x0020,   /* U+FD1E */
  0x0020,   /* U+FD1F */
  0x0020,   /* U+FD20 */
  0x0020,   /* U+FD21 */
  0x0020,   /* U+FD22 */
  0x0020,   /* U+FD23 */
  0x0020,   /* U+FD24 */
  0x0020,   /* U+FD25 */
  0x0020,   /* U+FD26 */
  0x0020,   /* U+FD27 */
  0x0020,   /* U+FD28 */
  0x0020,   /* U+FD29 */
  0x0020,   /* U+FD2A */
  0x0020,   /* U+FD2B */
  0x0020,   /* U+FD2C */
  0x0020,   /* U+FD2D */
  0x0020,   /* U+FD2E */
  0x0020,   /* U+FD2F */
  0x0020,   /* U+FD30 */
  0x0020,   /* U+FD31 */
  0x0020,   /* U+FD32 */
  0x0020,   /* U+FD33 */
  0x0020,   /* U+FD34 */
  0x0020,   /* U+FD35 */
  0x0020,   /* U+FD36 */
  0x0020,   /* U+FD37 */
  0x0020,   /* U+FD38 */
  0x0020,   /* U+FD39 */
  0x0020,   /* U+FD3A */
  0x0020,   /* U+FD3B */
  0x0020,   /* U+FD3C */
  0x0020,   /* U+FD3D */
  0x0020,   /* U+FD3E */
  0x0020,   /* U+FD3F */
  0x0020,   /* U+FD40 */
  0x0020,   /* U+FD41 */
  0x0020,   /* U+FD42 */
  0x0020,   /* U+FD43 */
  0x0020,   /* U+FD44 */
  0x0020,   /* U+FD45 */
  0x0020,   /* U+FD46 */
  0x0020,   /* U+FD47 */
  0x0020,   /* U+FD48 */
  0x0020,   /* U+FD49 */
  0x0020,   /* U+FD4A */
  0x0020,   /* U+FD4B */
  0x0020,   /* U+FD4C */
  0x0020,   /* U+FD4D */
  0x0020,   /* U+FD4E */
  0x0020,   /* U+FD4F */
  0x0020,   /* U+FD50 */
  0x0020,   /* U+FD51 */
  0x0020,   /* U+FD52 */
  0x0020,   /* U+FD53 */
  0x0020,   /* U+FD54 */
  0x0020,   /* U+FD55 */
  0x0020,   /* U+FD56 */
  0x0020,   /* U+FD57 */
  0x0020,   /* U+FD58 */
  0x0020,   /* U+FD59 */
  0x0020,   /* U+FD5A */
  0x0020,   /* U+FD5B */
  0x0020,   /* U+FD5C */
  0x0020,   /* U+FD5D */
  0x0020,   /* U+FD5E */
  0x0020,   /* U+FD5F */
  0x0020,   /* U+FD60 */
  0x0020,   /* U+FD61 */
  0x0020,   /* U+FD62 */
  0x0020,   /* U+FD63 */
  0x0020,   /* U+FD64 */
  0x0020,   /* U+FD65 */
  0x0020,   /* U+FD66 */
  0x0020,   /* U+FD67 */
  0x0020,   /* U+FD68 */
  0x0020,   /* U+FD69 */
  0x0020,   /* U+FD6A */
  0x0020,   /* U+FD6B */
  0x0020,   /* U+FD6C */
  0x0020,   /* U+FD6D */
  0x0020,   /* U+FD6E */
  0x0020,   /* U+FD6F */
  0x0020,   /* U+FD70 */
  0x0020,   /* U+FD71 */
  0x0020,   /* U+FD72 */
  0x0020,   /* U+FD73 */
  0x0020,   /* U+FD74 */
  0x0020,   /* U+FD75 */
  0x0020,   /* U+FD76 */
  0x0020,   /* U+FD77 */
  0x0020,   /* U+FD78 */
  0x0020,   /* U+FD79 */
  0x0020,   /* U+FD7A */
  0x0020,   /* U+FD7B */
  0x0020,   /* U+FD7C */
  0x0020,   /* U+FD7D */
  0x0020,   /* U+FD7E */
  0x0020,   /* U+FD7F */
  0x0020,   /* U+FD80 */
  0x0020,   /* U+FD81 */
  0x0020,   /* U+FD82 */
  0x0020,   /* U+FD83 */
  0x0020,   /* U+FD84 */
  0x0020,   /* U+FD85 */
  0x0020,   /* U+FD86 */
  0x0020,   /* U+FD87 */
  0x0020,   /* U+FD88 */
  0x0020,   /* U+FD89 */
  0x0020,   /* U+FD8A */
  0x0020,   /* U+FD8B */
  0x0020,   /* U+FD8C */
  0x0020,   /* U+FD8D */
  0x0020,   /* U+FD8E */
  0x0020,   /* U+FD8F */
  0x0020,   /* U+FD90 */
  0x0020,   /* U+FD91 */
  0x0020,   /* U+FD92 */
  0x0020,   /* U+FD93 */
  0x0020,   /* U+FD94 */
  0x0020,   /* U+FD95 */
  0x0020,   /* U+FD96 */
  0x0020,   /* U+FD97 */
  0x0020,   /* U+FD98 */
  0x0020,   /* U+FD99 */
  0x0020,   /* U+FD9A */
  0x0020,   /* U+FD9B */
  0x0020,   /* U+FD9C */
  0x0020,   /* U+FD9D */
  0x0020,   /* U+FD9E */
  0x0020,   /* U+FD9F */
  0x0020,   /* U+FDA0 */
  0x0020,   /* U+FDA1 */
  0x0020,   /* U+FDA2 */
  0x0020,   /* U+FDA3 */
  0x0020,   /* U+FDA4 */
  0x0020,   /* U+FDA5 */
  0x0020,   /* U+FDA6 */
  0x0020,   /* U+FDA7 */
  0x0020,   /* U+FDA8 */
  0x0020,   /* U+FDA9 */
  0x0020,   /* U+FDAA */
  0x0020,   /* U+FDAB */
  0x0020,   /* U+FDAC */
  0x0020,   /* U+FDAD */
  0x0020,   /* U+FDAE */
  0x0020,   /* U+FDAF */
  0x0020,   /* U+FDB0 */
  0x0020,   /* U+FDB1 */
  0x0020,   /* U+FDB2 */
  0x0020,   /* U+FDB3 */
  0x0020,   /* U+FDB4 */
  0x0020,   /* U+FDB5 */
  0x0020,   /* U+FDB6 */
  0x0020,   /* U+FDB7 */
  0x0020,   /* U+FDB8 */
  0x0020,   /* U+FDB9 */
  0x0020,   /* U+FDBA */
  0x0020,   /* U+FDBB */
  0x0020,   /* U+FDBC */
  0x0020,   /* U+FDBD */
  0x0020,   /* U+FDBE */
  0x0020,   /* U+FDBF */
  0x0020,   /* U+FDC0 */
  0x0020,   /* U+FDC1 */
  0x0020,   /* U+FDC2 */
  0x0020,   /* U+FDC3 */
  0x0020,   /* U+FDC4 */
  0x0020,   /* U+FDC5 */
  0x0020,   /* U+FDC6 */
  0x0020,   /* U+FDC7 */
  0x0020,   /* U+FDC8 */
  0x0020,   /* U+FDC9 */
  0x0020,   /* U+FDCA */
  0x0020,   /* U+FDCB */
  0x0020,   /* U+FDCC */
  0x0020,   /* U+FDCD */
  0x0020,   /* U+FDCE */
  0x0020,   /* U+FDCF */
  0x0020,   /* U+FDD0 */
  0x0020,   /* U+FDD1 */
  0x0020,   /* U+FDD2 */
  0x0020,   /* U+FDD3 */
  0x0020,   /* U+FDD4 */
  0x0020,   /* U+FDD5 */
  0x0020,   /* U+FDD6 */
  0x0020,   /* U+FDD7 */
  0x0020,   /* U+FDD8 */
  0x0020,   /* U+FDD9 */
  0x0020,   /* U+FDDA */
  0x0020,   /* U+FDDB */
  0x0020,   /* U+FDDC */
  0x0020,   /* U+FDDD */
  0x0020,   /* U+FDDE */
  0x0020,   /* U+FDDF */
  0x0020,   /* U+FDE0 */
  0x0020,   /* U+FDE1 */
  0x0020,   /* U+FDE2 */
  0x0020,   /* U+FDE3 */
  0x0020,   /* U+FDE4 */
  0x0020,   /* U+FDE5 */
  0x0020,   /* U+FDE6 */
  0x0020,   /* U+FDE7 */
  0x0020,   /* U+FDE8 */
  0x0020,   /* U+FDE9 */
  0x0020,   /* U+FDEA */
  0x0020,   /* U+FDEB */
  0x0020,   /* U+FDEC */
  0x0020,   /* U+FDED */
  0x0020,   /* U+FDEE */
  0x0020,   /* U+FDEF */
  0x0020,   /* U+FDF0 */
  0x0020,   /* U+FDF1 */
  0x0020,   /* U+FDF2 */
  0x0020,   /* U+FDF3 */
  0x0020,   /* U+FDF4 */
  0x0020,   /* U+FDF5 */
  0x0020,   /* U+FDF6 */
  0x0020,   /* U+FDF7 */
  0x0020,   /* U+FDF8 */
  0x0020,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0020,   /* U+FDFC */
  0x0020,   /* U+FDFD */
  0x0020,   /* U+FDFE */
  0x0020,   /* U+FDFF */

  /* Tertiary weight 1 for each character. */
  0x001A,   /* U+FD00 */
  0x001A,   /* U+FD01 */
  0x001A,   /* U+FD02 */
  0x001A,   /* U+FD03 */
  0x001A,   /* U+FD04 */
  0x001A,   /* U+FD05 */
  0x001A,   /* U+FD06 */
  0x001A,   /* U+FD07 */
  0x001A,   /* U+FD08 */
  0x001A,   /* U+FD09 */
  0x001A,   /* U+FD0A */
  0x001A,   /* U+FD0B */
  0x001A,   /* U+FD0C */
  0x001A,   /* U+FD0D */
  0x001A,   /* U+FD0E */
  0x001A,   /* U+FD0F */
  0x001A,   /* U+FD10 */
  0x0019,   /* U+FD11 */
  0x0019,   /* U+FD12 */
  0x0019,   /* U+FD13 */
  0x0019,   /* U+FD14 */
  0x0019,   /* U+FD15 */
  0x0019,   /* U+FD16 */
  0x0019,   /* U+FD17 */
  0x0019,   /* U+FD18 */
  0x0019,   /* U+FD19 */
  0x0019,   /* U+FD1A */
  0x0019,   /* U+FD1B */
  0x0019,   /* U+FD1C */
  0x0019,   /* U+FD1D */
  0x0019,   /* U+FD1E */
  0x0019,   /* U+FD1F */
  0x0019,   /* U+FD20 */
  0x0019,   /* U+FD21 */
  0x0019,   /* U+FD22 */
  0x0019,   /* U+FD23 */
  0x0019,   /* U+FD24 */
  0x0019,   /* U+FD25 */
  0x0019,   /* U+FD26 */
  0x0019,   /* U+FD27 */
  0x0019,   /* U+FD28 */
  0x0019,   /* U+FD29 */
  0x0019,   /* U+FD2A */
  0x0019,   /* U+FD2B */
  0x0019,   /* U+FD2C */
  0x0017,   /* U+FD2D */
  0x0017,   /* U+FD2E */
  0x0017,   /* U+FD2F */
  0x0017,   /* U+FD30 */
  0x0017,   /* U+FD31 */
  0x0017,   /* U+FD32 */
  0x0017,   /* U+FD33 */
  0x0018,   /* U+FD34 */
  0x0018,   /* U+FD35 */
  0x0018,   /* U+FD36 */
  0x0018,   /* U+FD37 */
  0x0018,   /* U+FD38 */
  0x0018,   /* U+FD39 */
  0x0018,   /* U+FD3A */
  0x0018,   /* U+FD3B */
  0x0019,   /* U+FD3C */
  0x001A,   /* U+FD3D */
  0x0002,   /* U+FD3E */
  0x0002,   /* U+FD3F */
  0x0002,   /* U+FD40 */
  0x0002,   /* U+FD41 */
  0x0002,   /* U+FD42 */
  0x0002,   /* U+FD43 */
  0x0002,   /* U+FD44 */
  0x0002,   /* U+FD45 */
  0x0002,   /* U+FD46 */
  0x0002,   /* U+FD47 */
  0x0002,   /* U+FD48 */
  0x0002,   /* U+FD49 */
  0x0002,   /* U+FD4A */
  0x0002,   /* U+FD4B */
  0x0002,   /* U+FD4C */
  0x0002,   /* U+FD4D */
  0x0002,   /* U+FD4E */
  0x0002,   /* U+FD4F */
  0x0017,   /* U+FD50 */
  0x0019,   /* U+FD51 */
  0x0017,   /* U+FD52 */
  0x0017,   /* U+FD53 */
  0x0017,   /* U+FD54 */
  0x0017,   /* U+FD55 */
  0x0017,   /* U+FD56 */
  0x0017,   /* U+FD57 */
  0x0019,   /* U+FD58 */
  0x0017,   /* U+FD59 */
  0x0019,   /* U+FD5A */
  0x0019,   /* U+FD5B */
  0x0017,   /* U+FD5C */
  0x0017,   /* U+FD5D */
  0x0019,   /* U+FD5E */
  0x0019,   /* U+FD5F */
  0x0017,   /* U+FD60 */
  0x0017,   /* U+FD61 */
  0x0019,   /* U+FD62 */
  0x0017,   /* U+FD63 */
  0x0019,   /* U+FD64 */
  0x0017,   /* U+FD65 */
  0x0019,   /* U+FD66 */
  0x0019,   /* U+FD67 */
  0x0017,   /* U+FD68 */
  0x0019,   /* U+FD69 */
  0x0019,   /* U+FD6A */
  0x0017,   /* U+FD6B */
  0x0019,   /* U+FD6C */
  0x0017,   /* U+FD6D */
  0x0019,   /* U+FD6E */
  0x0019,   /* U+FD6F */
  0x0017,   /* U+FD70 */
  0x0019,   /* U+FD71 */
  0x0017,   /* U+FD72 */
  0x0017,   /* U+FD73 */
  0x0019,   /* U+FD74 */
  0x0019,   /* U+FD75 */
  0x0019,   /* U+FD76 */
  0x0017,   /* U+FD77 */
  0x0019,   /* U+FD78 */
  0x0019,   /* U+FD79 */
  0x0019,   /* U+FD7A */
  0x0019,   /* U+FD7B */
  0x0019,   /* U+FD7C */
  0x0017,   /* U+FD7D */
  0x0019,   /* U+FD7E */
  0x0019,   /* U+FD7F */
  0x0019,   /* U+FD80 */
  0x0019,   /* U+FD81 */
  0x0019,   /* U+FD82 */
  0x0017,   /* U+FD83 */
  0x0019,   /* U+FD84 */
  0x0019,   /* U+FD85 */
  0x0017,   /* U+FD86 */
  0x0019,   /* U+FD87 */
  0x0017,   /* U+FD88 */
  0x0017,   /* U+FD89 */
  0x0017,   /* U+FD8A */
  0x0019,   /* U+FD8B */
  0x0017,   /* U+FD8C */
  0x0017,   /* U+FD8D */
  0x0017,   /* U+FD8E */
  0x0017,   /* U+FD8F */
  0x0002,   /* U+FD90 */
  0x0002,   /* U+FD91 */
  0x0017,   /* U+FD92 */
  0x0017,   /* U+FD93 */
  0x0017,   /* U+FD94 */
  0x0017,   /* U+FD95 */
  0x0019,   /* U+FD96 */
  0x0019,   /* U+FD97 */
  0x0017,   /* U+FD98 */
  0x0019,   /* U+FD99 */
  0x0019,   /* U+FD9A */
  0x0019,   /* U+FD9B */
  0x0019,   /* U+FD9C */
  0x0017,   /* U+FD9D */
  0x0019,   /* U+FD9E */
  0x0019,   /* U+FD9F */
  0x0019,   /* U+FDA0 */
  0x0019,   /* U+FDA1 */
  0x0019,   /* U+FDA2 */
  0x0019,   /* U+FDA3 */
  0x0019,   /* U+FDA4 */
  0x0019,   /* U+FDA5 */
  0x0019,   /* U+FDA6 */
  0x0019,   /* U+FDA7 */
  0x0019,   /* U+FDA8 */
  0x0019,   /* U+FDA9 */
  0x0019,   /* U+FDAA */
  0x0019,   /* U+FDAB */
  0x0019,   /* U+FDAC */
  0x0019,   /* U+FDAD */
  0x0019,   /* U+FDAE */
  0x0019,   /* U+FDAF */
  0x0019,   /* U+FDB0 */
  0x0019,   /* U+FDB1 */
  0x0019,   /* U+FDB2 */
  0x0019,   /* U+FDB3 */
  0x0017,   /* U+FDB4 */
  0x0017,   /* U+FDB5 */
  0x0019,   /* U+FDB6 */
  0x0019,   /* U+FDB7 */
  0x0017,   /* U+FDB8 */
  0x0019,   /* U+FDB9 */
  0x0017,   /* U+FDBA */
  0x0019,   /* U+FDBB */
  0x0019,   /* U+FDBC */
  0x0019,   /* U+FDBD */
  0x0019,   /* U+FDBE */
  0x0019,   /* U+FDBF */
  0x0019,   /* U+FDC0 */
  0x0019,   /* U+FDC1 */
  0x0019,   /* U+FDC2 */
  0x0017,   /* U+FDC3 */
  0x0017,   /* U+FDC4 */
  0x0017,   /* U+FDC5 */
  0x0019,   /* U+FDC6 */
  0x0019,   /* U+FDC7 */
  0x0002,   /* U+FDC8 */
  0x0002,   /* U+FDC9 */
  0x0002,   /* U+FDCA */
  0x0002,   /* U+FDCB */
  0x0002,   /* U+FDCC */
  0x0002,   /* U+FDCD */
  0x0002,   /* U+FDCE */
  0x0002,   /* U+FDCF */
  0x0002,   /* U+FDD0 */
  0x0002,   /* U+FDD1 */
  0x0002,   /* U+FDD2 */
  0x0002,   /* U+FDD3 */
  0x0002,   /* U+FDD4 */
  0x0002,   /* U+FDD5 */
  0x0002,   /* U+FDD6 */
  0x0002,   /* U+FDD7 */
  0x0002,   /* U+FDD8 */
  0x0002,   /* U+FDD9 */
  0x0002,   /* U+FDDA */
  0x0002,   /* U+FDDB */
  0x0002,   /* U+FDDC */
  0x0002,   /* U+FDDD */
  0x0002,   /* U+FDDE */
  0x0002,   /* U+FDDF */
  0x0002,   /* U+FDE0 */
  0x0002,   /* U+FDE1 */
  0x0002,   /* U+FDE2 */
  0x0002,   /* U+FDE3 */
  0x0002,   /* U+FDE4 */
  0x0002,   /* U+FDE5 */
  0x0002,   /* U+FDE6 */
  0x0002,   /* U+FDE7 */
  0x0002,   /* U+FDE8 */
  0x0002,   /* U+FDE9 */
  0x0002,   /* U+FDEA */
  0x0002,   /* U+FDEB */
  0x0002,   /* U+FDEC */
  0x0002,   /* U+FDED */
  0x0002,   /* U+FDEE */
  0x0002,   /* U+FDEF */
  0x001A,   /* U+FDF0 */
  0x001A,   /* U+FDF1 */
  0x001A,   /* U+FDF2 */
  0x001A,   /* U+FDF3 */
  0x001A,   /* U+FDF4 */
  0x001A,   /* U+FDF5 */
  0x001A,   /* U+FDF6 */
  0x001A,   /* U+FDF7 */
  0x001A,   /* U+FDF8 */
  0x001A,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x001A,   /* U+FDFC */
  0x0002,   /* U+FDFD */
  0x0002,   /* U+FDFE */
  0x0002,   /* U+FDFF */

  /* Primary weight 2 for each character. */
  0x23C6,   /* U+FD00 */
  0x23C5,   /* U+FD01 */
  0x23C6,   /* U+FD02 */
  0x23C5,   /* U+FD03 */
  0x23C6,   /* U+FD04 */
  0x23C5,   /* U+FD05 */
  0x23C6,   /* U+FD06 */
  0x23C5,   /* U+FD07 */
  0x23C6,   /* U+FD08 */
  0x2325,   /* U+FD09 */
  0x232C,   /* U+FD0A */
  0x232D,   /* U+FD0B */
  0x23A3,   /* U+FD0C */
  0x2346,   /* U+FD0D */
  0x2346,   /* U+FD0E */
  0x2346,   /* U+FD0F */
  0x2346,   /* U+FD10 */
  0x23C5,   /* U+FD11 */
  0x23C6,   /* U+FD12 */
  0x23C5,   /* U+FD13 */
  0x23C6,   /* U+FD14 */
  0x23C5,   /* U+FD15 */
  0x23C6,   /* U+FD16 */
  0x23C5,   /* U+FD17 */
  0x23C6,   /* U+FD18 */
  0x23C5,   /* U+FD19 */
  0x23C6,   /* U+FD1A */
  0x23C5,   /* U+FD1B */
  0x23C6,   /* U+FD1C */
  0x23C5,   /* U+FD1D */
  0x23C6,   /* U+FD1E */
  0x23C5,   /* U+FD1F */
  0x23C6,   /* U+FD20 */
  0x23C5,   /* U+FD21 */
  0x23C6,   /* U+FD22 */
  0x23C5,   /* U+FD23 */
  0x23C6,   /* U+FD24 */
  0x2325,   /* U+FD25 */
  0x232C,   /* U+FD26 */
  0x232D,   /* U+FD27 */
  0x23A3,   /* U+FD28 */
  0x2346,   /* U+FD29 */
  0x2346,   /* U+FD2A */
  0x2346,   /* U+FD2B */
  0x2346,   /* U+FD2C */
  0x2325,   /* U+FD2D */
  0x232C,   /* U+FD2E */
  0x232D,   /* U+FD2F */
  0x23A3,   /* U+FD30 */
  0x23B1,   /* U+FD31 */
  0x23B1,   /* U+FD32 */
  0x23A3,   /* U+FD33 */
  0x2325,   /* U+FD34 */
  0x232C,   /* U+FD35 */
  0x232D,   /* U+FD36 */
  0x2325,   /* U+FD37 */
  0x232C,   /* U+FD38 */
  0x232D,   /* U+FD39 */
  0x23A3,   /* U+FD3A */
  0x23A3,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0xFD40,   /* U+FD40 */
  0xFD41,   /* U+FD41 */
  0xFD42,   /* U+FD42 */
  0xFD43,   /* U+FD43 */
  0xFD44,   /* U+FD44 */
  0xFD45,   /* U+FD45 */
  0xFD46,   /* U+FD46 */
  0xFD47,   /* U+FD47 */
  0xFD48,   /* U+FD48 */
  0xFD49,   /* U+FD49 */
  0xFD4A,   /* U+FD4A */
  0xFD4B,   /* U+FD4B */
  0xFD4C,   /* U+FD4C */
  0xFD4D,   /* U+FD4D */
  0xFD4E,   /* U+FD4E */
  0xFD4F,   /* U+FD4F */
  0x2325,   /* U+FD50 */
  0x232C,   /* U+FD51 */
  0x232C,   /* U+FD52 */
  0x232C,   /* U+FD53 */
  0x232D,   /* U+FD54 */
  0x23A3,   /* U+FD55 */
  0x23A3,   /* U+FD56 */
  0x23A3,   /* U+FD57 */
  0x23A3,   /* U+FD58 */
  0x23A3,   /* U+FD59 */
  0x23A3,   /* U+FD5A */
  0x23A3,   /* U+FD5B */
  0x232C,   /* U+FD5C */
  0x2325,   /* U+FD5D */
  0x2325,   /* U+FD5E */
  0x23A3,   /* U+FD5F */
  0x23A3,   /* U+FD60 */
  0x23A3,   /* U+FD61 */
  0x23A3,   /* U+FD62 */
  0x23A3,   /* U+FD63 */
  0x232C,   /* U+FD64 */
  0x232C,   /* U+FD65 */
  0x23A3,   /* U+FD66 */
  0x232C,   /* U+FD67 */
  0x232C,   /* U+FD68 */
  0x2325,   /* U+FD69 */
  0x23A3,   /* U+FD6A */
  0x23A3,   /* U+FD6B */
  0x23A3,   /* U+FD6C */
  0x23A3,   /* U+FD6D */
  0x232C,   /* U+FD6E */
  0x232D,   /* U+FD6F */
  0x232D,   /* U+FD70 */
  0x23A3,   /* U+FD71 */
  0x23A3,   /* U+FD72 */
  0x23A3,   /* U+FD73 */
  0x23A3,   /* U+FD74 */
  0x2325,   /* U+FD75 */
  0x23A3,   /* U+FD76 */
  0x23A3,   /* U+FD77 */
  0x23A3,   /* U+FD78 */
  0x23A3,   /* U+FD79 */
  0x23A3,   /* U+FD7A */
  0x23A3,   /* U+FD7B */
  0x232D,   /* U+FD7C */
  0x232D,   /* U+FD7D */
  0x23A3,   /* U+FD7E */
  0x23A3,   /* U+FD7F */
  0x232C,   /* U+FD80 */
  0x232C,   /* U+FD81 */
  0x232C,   /* U+FD82 */
  0x2325,   /* U+FD83 */
  0x2325,   /* U+FD84 */
  0x232D,   /* U+FD85 */
  0x232D,   /* U+FD86 */
  0x23A3,   /* U+FD87 */
  0x23A3,   /* U+FD88 */
  0x232C,   /* U+FD89 */
  0x232C,   /* U+FD8A */
  0x232C,   /* U+FD8B */
  0x2325,   /* U+FD8C */
  0x2325,   /* U+FD8D */
  0x232D,   /* U+FD8E */
  0x232D,   /* U+FD8F */
  0xFD90,   /* U+FD90 */
  0xFD91,   /* U+FD91 */
  0x2325,   /* U+FD92 */
  0x23A3,   /* U+FD93 */
  0x23A3,   /* U+FD94 */
  0x232C,   /* U+FD95 */
  0x232C,   /* U+FD96 */
  0x2325,   /* U+FD97 */
  0x2325,   /* U+FD98 */
  0x2325,   /* U+FD99 */
  0x23A3,   /* U+FD9A */
  0x23A3,   /* U+FD9B */
  0x23A3,   /* U+FD9C */
  0x23A3,   /* U+FD9D */
  0x232D,   /* U+FD9E */
  0x2325,   /* U+FD9F */
  0x2325,   /* U+FDA0 */
  0x232D,   /* U+FDA1 */
  0x232D,   /* U+FDA2 */
  0x23A3,   /* U+FDA3 */
  0x23A3,   /* U+FDA4 */
  0x23A3,   /* U+FDA5 */
  0x232C,   /* U+FDA6 */
  0x23A3,   /* U+FDA7 */
  0x232D,   /* U+FDA8 */
  0x232C,   /* U+FDA9 */
  0x232C,   /* U+FDAA */
  0x232C,   /* U+FDAB */
  0x2325,   /* U+FDAC */
  0x23A3,   /* U+FDAD */
  0x232C,   /* U+FDAE */
  0x2325,   /* U+FDAF */
  0x23A3,   /* U+FDB0 */
  0x23A3,   /* U+FDB1 */
  0x23A3,   /* U+FDB2 */
  0x232C,   /* U+FDB3 */
  0x23A3,   /* U+FDB4 */
  0x232C,   /* U+FDB5 */
  0x23A3,   /* U+FDB6 */
  0x23A3,   /* U+FDB7 */
  0x2325,   /* U+FDB8 */
  0x232D,   /* U+FDB9 */
  0x2325,   /* U+FDBA */
  0x23A3,   /* U+FDBB */
  0x2325,   /* U+FDBC */
  0x2325,   /* U+FDBD */
  0x232C,   /* U+FDBE */
  0x2325,   /* U+FDBF */
  0x2325,   /* U+FDC0 */
  0x23A3,   /* U+FDC1 */
  0x232C,   /* U+FDC2 */
  0x23A3,   /* U+FDC3 */
  0x2325,   /* U+FDC4 */
  0x23A3,   /* U+FDC5 */
  0x232D,   /* U+FDC6 */
  0x2325,   /* U+FDC7 */
  0xFDC8,   /* U+FDC8 */
  0xFDC9,   /* U+FDC9 */
  0xFDCA,   /* U+FDCA */
  0xFDCB,   /* U+FDCB */
  0xFDCC,   /* U+FDCC */
  0xFDCD,   /* U+FDCD */
  0xFDCE,   /* U+FDCE */
  0xFDCF,   /* U+FDCF */
  0xFDD0,   /* U+FDD0 */
  0xFDD1,   /* U+FDD1 */
  0xFDD2,   /* U+FDD2 */
  0xFDD3,   /* U+FDD3 */
  0xFDD4,   /* U+FDD4 */
  0xFDD5,   /* U+FDD5 */
  0xFDD6,   /* U+FDD6 */
  0xFDD7,   /* U+FDD7 */
  0xFDD8,   /* U+FDD8 */
  0xFDD9,   /* U+FDD9 */
  0xFDDA,   /* U+FDDA */
  0xFDDB,   /* U+FDDB */
  0xFDDC,   /* U+FDDC */
  0xFDDD,   /* U+FDDD */
  0xFDDE,   /* U+FDDE */
  0xFDDF,   /* U+FDDF */
  0xFDE0,   /* U+FDE0 */
  0xFDE1,   /* U+FDE1 */
  0xFDE2,   /* U+FDE2 */
  0xFDE3,   /* U+FDE3 */
  0xFDE4,   /* U+FDE4 */
  0xFDE5,   /* U+FDE5 */
  0xFDE6,   /* U+FDE6 */
  0xFDE7,   /* U+FDE7 */
  0xFDE8,   /* U+FDE8 */
  0xFDE9,   /* U+FDE9 */
  0xFDEA,   /* U+FDEA */
  0xFDEB,   /* U+FDEB */
  0xFDEC,   /* U+FDEC */
  0xFDED,   /* U+FDED */
  0xFDEE,   /* U+FDEE */
  0xFDEF,   /* U+FDEF */
  0x239C,   /* U+FDF0 */
  0x239C,   /* U+FDF1 */
  0x239C,   /* U+FDF2 */
  0x2387,   /* U+FDF3 */
  0x232C,   /* U+FDF4 */
  0x239C,   /* U+FDF5 */
  0x2359,   /* U+FDF6 */
  0x239C,   /* U+FDF7 */
  0x2359,   /* U+FDF8 */
  0x239C,   /* U+FDF9 */
  0x239C,   /* U+FDFA */
  0x239C,   /* U+FDFB */
  0x23C7,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0xFDFE,   /* U+FDFE */
  0xFDFF,   /* U+FDFF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+FD00 */
  0x0020,   /* U+FD01 */
  0x0020,   /* U+FD02 */
  0x0020,   /* U+FD03 */
  0x0020,   /* U+FD04 */
  0x0020,   /* U+FD05 */
  0x0020,   /* U+FD06 */
  0x0020,   /* U+FD07 */
  0x0020,   /* U+FD08 */
  0x0020,   /* U+FD09 */
  0x0020,   /* U+FD0A */
  0x0020,   /* U+FD0B */
  0x0020,   /* U+FD0C */
  0x0020,   /* U+FD0D */
  0x0020,   /* U+FD0E */
  0x0020,   /* U+FD0F */
  0x0020,   /* U+FD10 */
  0x0020,   /* U+FD11 */
  0x0020,   /* U+FD12 */
  0x0020,   /* U+FD13 */
  0x0020,   /* U+FD14 */
  0x0020,   /* U+FD15 */
  0x0020,   /* U+FD16 */
  0x0020,   /* U+FD17 */
  0x0020,   /* U+FD18 */
  0x0020,   /* U+FD19 */
  0x0020,   /* U+FD1A */
  0x0020,   /* U+FD1B */
  0x0020,   /* U+FD1C */
  0x0020,   /* U+FD1D */
  0x0020,   /* U+FD1E */
  0x0020,   /* U+FD1F */
  0x0020,   /* U+FD20 */
  0x0020,   /* U+FD21 */
  0x0020,   /* U+FD22 */
  0x0020,   /* U+FD23 */
  0x0020,   /* U+FD24 */
  0x0020,   /* U+FD25 */
  0x0020,   /* U+FD26 */
  0x0020,   /* U+FD27 */
  0x0020,   /* U+FD28 */
  0x0020,   /* U+FD29 */
  0x0020,   /* U+FD2A */
  0x0020,   /* U+FD2B */
  0x0020,   /* U+FD2C */
  0x0020,   /* U+FD2D */
  0x0020,   /* U+FD2E */
  0x0020,   /* U+FD2F */
  0x0020,   /* U+FD30 */
  0x0020,   /* U+FD31 */
  0x0020,   /* U+FD32 */
  0x0020,   /* U+FD33 */
  0x0020,   /* U+FD34 */
  0x0020,   /* U+FD35 */
  0x0020,   /* U+FD36 */
  0x0020,   /* U+FD37 */
  0x0020,   /* U+FD38 */
  0x0020,   /* U+FD39 */
  0x0020,   /* U+FD3A */
  0x0020,   /* U+FD3B */
  0x006D,   /* U+FD3C */
  0x006D,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0020,   /* U+FD50 */
  0x0020,   /* U+FD51 */
  0x0020,   /* U+FD52 */
  0x0020,   /* U+FD53 */
  0x0020,   /* U+FD54 */
  0x0020,   /* U+FD55 */
  0x0020,   /* U+FD56 */
  0x0020,   /* U+FD57 */
  0x0020,   /* U+FD58 */
  0x0020,   /* U+FD59 */
  0x0020,   /* U+FD5A */
  0x0020,   /* U+FD5B */
  0x0020,   /* U+FD5C */
  0x0020,   /* U+FD5D */
  0x0020,   /* U+FD5E */
  0x0020,   /* U+FD5F */
  0x0020,   /* U+FD60 */
  0x0020,   /* U+FD61 */
  0x0020,   /* U+FD62 */
  0x0020,   /* U+FD63 */
  0x0020,   /* U+FD64 */
  0x0020,   /* U+FD65 */
  0x0020,   /* U+FD66 */
  0x0020,   /* U+FD67 */
  0x0020,   /* U+FD68 */
  0x0020,   /* U+FD69 */
  0x0020,   /* U+FD6A */
  0x0020,   /* U+FD6B */
  0x0020,   /* U+FD6C */
  0x0020,   /* U+FD6D */
  0x0020,   /* U+FD6E */
  0x0020,   /* U+FD6F */
  0x0020,   /* U+FD70 */
  0x0020,   /* U+FD71 */
  0x0020,   /* U+FD72 */
  0x0020,   /* U+FD73 */
  0x0020,   /* U+FD74 */
  0x0020,   /* U+FD75 */
  0x0020,   /* U+FD76 */
  0x0020,   /* U+FD77 */
  0x0020,   /* U+FD78 */
  0x0020,   /* U+FD79 */
  0x0020,   /* U+FD7A */
  0x0020,   /* U+FD7B */
  0x0020,   /* U+FD7C */
  0x0020,   /* U+FD7D */
  0x0020,   /* U+FD7E */
  0x0020,   /* U+FD7F */
  0x0020,   /* U+FD80 */
  0x0020,   /* U+FD81 */
  0x0020,   /* U+FD82 */
  0x0020,   /* U+FD83 */
  0x0020,   /* U+FD84 */
  0x0020,   /* U+FD85 */
  0x0020,   /* U+FD86 */
  0x0020,   /* U+FD87 */
  0x0020,   /* U+FD88 */
  0x0020,   /* U+FD89 */
  0x0020,   /* U+FD8A */
  0x0020,   /* U+FD8B */
  0x0020,   /* U+FD8C */
  0x0020,   /* U+FD8D */
  0x0020,   /* U+FD8E */
  0x0020,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0020,   /* U+FD92 */
  0x0020,   /* U+FD93 */
  0x0020,   /* U+FD94 */
  0x0020,   /* U+FD95 */
  0x0020,   /* U+FD96 */
  0x0020,   /* U+FD97 */
  0x0020,   /* U+FD98 */
  0x0020,   /* U+FD99 */
  0x0020,   /* U+FD9A */
  0x0020,   /* U+FD9B */
  0x0020,   /* U+FD9C */
  0x0020,   /* U+FD9D */
  0x0020,   /* U+FD9E */
  0x0020,   /* U+FD9F */
  0x0020,   /* U+FDA0 */
  0x0020,   /* U+FDA1 */
  0x0020,   /* U+FDA2 */
  0x0020,   /* U+FDA3 */
  0x0020,   /* U+FDA4 */
  0x0020,   /* U+FDA5 */
  0x0020,   /* U+FDA6 */
  0x0020,   /* U+FDA7 */
  0x0020,   /* U+FDA8 */
  0x0020,   /* U+FDA9 */
  0x0020,   /* U+FDAA */
  0x0020,   /* U+FDAB */
  0x0020,   /* U+FDAC */
  0x0020,   /* U+FDAD */
  0x0020,   /* U+FDAE */
  0x0020,   /* U+FDAF */
  0x0020,   /* U+FDB0 */
  0x0020,   /* U+FDB1 */
  0x0020,   /* U+FDB2 */
  0x0020,   /* U+FDB3 */
  0x0020,   /* U+FDB4 */
  0x0020,   /* U+FDB5 */
  0x0020,   /* U+FDB6 */
  0x0020,   /* U+FDB7 */
  0x0020,   /* U+FDB8 */
  0x0020,   /* U+FDB9 */
  0x0020,   /* U+FDBA */
  0x0020,   /* U+FDBB */
  0x0020,   /* U+FDBC */
  0x0020,   /* U+FDBD */
  0x0020,   /* U+FDBE */
  0x0020,   /* U+FDBF */
  0x0020,   /* U+FDC0 */
  0x0020,   /* U+FDC1 */
  0x0020,   /* U+FDC2 */
  0x0020,   /* U+FDC3 */
  0x0020,   /* U+FDC4 */
  0x0020,   /* U+FDC5 */
  0x0020,   /* U+FDC6 */
  0x0020,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0020,   /* U+FDF0 */
  0x0020,   /* U+FDF1 */
  0x0020,   /* U+FDF2 */
  0x0020,   /* U+FDF3 */
  0x0020,   /* U+FDF4 */
  0x0020,   /* U+FDF5 */
  0x0020,   /* U+FDF6 */
  0x0020,   /* U+FDF7 */
  0x0020,   /* U+FDF8 */
  0x0020,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0020,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 2 for each character. */
  0x001A,   /* U+FD00 */
  0x001A,   /* U+FD01 */
  0x001A,   /* U+FD02 */
  0x001A,   /* U+FD03 */
  0x001A,   /* U+FD04 */
  0x001A,   /* U+FD05 */
  0x001A,   /* U+FD06 */
  0x001A,   /* U+FD07 */
  0x001A,   /* U+FD08 */
  0x001A,   /* U+FD09 */
  0x001A,   /* U+FD0A */
  0x001A,   /* U+FD0B */
  0x001A,   /* U+FD0C */
  0x001A,   /* U+FD0D */
  0x001A,   /* U+FD0E */
  0x001A,   /* U+FD0F */
  0x001A,   /* U+FD10 */
  0x0019,   /* U+FD11 */
  0x0019,   /* U+FD12 */
  0x0019,   /* U+FD13 */
  0x0019,   /* U+FD14 */
  0x0019,   /* U+FD15 */
  0x0019,   /* U+FD16 */
  0x0019,   /* U+FD17 */
  0x0019,   /* U+FD18 */
  0x0019,   /* U+FD19 */
  0x0019,   /* U+FD1A */
  0x0019,   /* U+FD1B */
  0x0019,   /* U+FD1C */
  0x0019,   /* U+FD1D */
  0x0019,   /* U+FD1E */
  0x0019,   /* U+FD1F */
  0x0019,   /* U+FD20 */
  0x0019,   /* U+FD21 */
  0x0019,   /* U+FD22 */
  0x0019,   /* U+FD23 */
  0x0019,   /* U+FD24 */
  0x0019,   /* U+FD25 */
  0x0019,   /* U+FD26 */
  0x0019,   /* U+FD27 */
  0x0019,   /* U+FD28 */
  0x0019,   /* U+FD29 */
  0x0019,   /* U+FD2A */
  0x0019,   /* U+FD2B */
  0x0019,   /* U+FD2C */
  0x0017,   /* U+FD2D */
  0x0017,   /* U+FD2E */
  0x0017,   /* U+FD2F */
  0x0017,   /* U+FD30 */
  0x0017,   /* U+FD31 */
  0x0017,   /* U+FD32 */
  0x0017,   /* U+FD33 */
  0x0018,   /* U+FD34 */
  0x0018,   /* U+FD35 */
  0x0018,   /* U+FD36 */
  0x0018,   /* U+FD37 */
  0x0018,   /* U+FD38 */
  0x0018,   /* U+FD39 */
  0x0018,   /* U+FD3A */
  0x0018,   /* U+FD3B */
  0x0019,   /* U+FD3C */
  0x001A,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0017,   /* U+FD50 */
  0x0019,   /* U+FD51 */
  0x0017,   /* U+FD52 */
  0x0017,   /* U+FD53 */
  0x0017,   /* U+FD54 */
  0x0017,   /* U+FD55 */
  0x0017,   /* U+FD56 */
  0x0017,   /* U+FD57 */
  0x0019,   /* U+FD58 */
  0x0017,   /* U+FD59 */
  0x0019,   /* U+FD5A */
  0x0019,   /* U+FD5B */
  0x0017,   /* U+FD5C */
  0x0017,   /* U+FD5D */
  0x0019,   /* U+FD5E */
  0x0019,   /* U+FD5F */
  0x0017,   /* U+FD60 */
  0x0017,   /* U+FD61 */
  0x0019,   /* U+FD62 */
  0x0017,   /* U+FD63 */
  0x0019,   /* U+FD64 */
  0x0017,   /* U+FD65 */
  0x0019,   /* U+FD66 */
  0x0019,   /* U+FD67 */
  0x0017,   /* U+FD68 */
  0x0019,   /* U+FD69 */
  0x0019,   /* U+FD6A */
  0x0017,   /* U+FD6B */
  0x0019,   /* U+FD6C */
  0x0017,   /* U+FD6D */
  0x0019,   /* U+FD6E */
  0x0019,   /* U+FD6F */
  0x0017,   /* U+FD70 */
  0x0019,   /* U+FD71 */
  0x0017,   /* U+FD72 */
  0x0017,   /* U+FD73 */
  0x0019,   /* U+FD74 */
  0x0019,   /* U+FD75 */
  0x0019,   /* U+FD76 */
  0x0017,   /* U+FD77 */
  0x0019,   /* U+FD78 */
  0x0019,   /* U+FD79 */
  0x0019,   /* U+FD7A */
  0x0019,   /* U+FD7B */
  0x0019,   /* U+FD7C */
  0x0017,   /* U+FD7D */
  0x0019,   /* U+FD7E */
  0x0019,   /* U+FD7F */
  0x0019,   /* U+FD80 */
  0x0019,   /* U+FD81 */
  0x0019,   /* U+FD82 */
  0x0017,   /* U+FD83 */
  0x0019,   /* U+FD84 */
  0x0019,   /* U+FD85 */
  0x0017,   /* U+FD86 */
  0x0019,   /* U+FD87 */
  0x0017,   /* U+FD88 */
  0x0017,   /* U+FD89 */
  0x0017,   /* U+FD8A */
  0x0019,   /* U+FD8B */
  0x0017,   /* U+FD8C */
  0x0017,   /* U+FD8D */
  0x0017,   /* U+FD8E */
  0x0017,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0017,   /* U+FD92 */
  0x0017,   /* U+FD93 */
  0x0017,   /* U+FD94 */
  0x0017,   /* U+FD95 */
  0x0019,   /* U+FD96 */
  0x0019,   /* U+FD97 */
  0x0017,   /* U+FD98 */
  0x0019,   /* U+FD99 */
  0x0019,   /* U+FD9A */
  0x0019,   /* U+FD9B */
  0x0019,   /* U+FD9C */
  0x0017,   /* U+FD9D */
  0x0019,   /* U+FD9E */
  0x0019,   /* U+FD9F */
  0x0019,   /* U+FDA0 */
  0x0019,   /* U+FDA1 */
  0x0019,   /* U+FDA2 */
  0x0019,   /* U+FDA3 */
  0x0019,   /* U+FDA4 */
  0x0019,   /* U+FDA5 */
  0x0019,   /* U+FDA6 */
  0x0019,   /* U+FDA7 */
  0x0019,   /* U+FDA8 */
  0x0019,   /* U+FDA9 */
  0x0019,   /* U+FDAA */
  0x0019,   /* U+FDAB */
  0x0019,   /* U+FDAC */
  0x0019,   /* U+FDAD */
  0x0019,   /* U+FDAE */
  0x0019,   /* U+FDAF */
  0x0019,   /* U+FDB0 */
  0x0019,   /* U+FDB1 */
  0x0019,   /* U+FDB2 */
  0x0019,   /* U+FDB3 */
  0x0017,   /* U+FDB4 */
  0x0017,   /* U+FDB5 */
  0x0019,   /* U+FDB6 */
  0x0019,   /* U+FDB7 */
  0x0017,   /* U+FDB8 */
  0x0019,   /* U+FDB9 */
  0x0017,   /* U+FDBA */
  0x0019,   /* U+FDBB */
  0x0019,   /* U+FDBC */
  0x0019,   /* U+FDBD */
  0x0019,   /* U+FDBE */
  0x0019,   /* U+FDBF */
  0x0019,   /* U+FDC0 */
  0x0019,   /* U+FDC1 */
  0x0019,   /* U+FDC2 */
  0x0017,   /* U+FDC3 */
  0x0017,   /* U+FDC4 */
  0x0017,   /* U+FDC5 */
  0x0019,   /* U+FDC6 */
  0x0019,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x001A,   /* U+FDF0 */
  0x001A,   /* U+FDF1 */
  0x001A,   /* U+FDF2 */
  0x001A,   /* U+FDF3 */
  0x001A,   /* U+FDF4 */
  0x001A,   /* U+FDF5 */
  0x001A,   /* U+FDF6 */
  0x001A,   /* U+FDF7 */
  0x001A,   /* U+FDF8 */
  0x001A,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x001A,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x23A3,   /* U+FD50 */
  0x2325,   /* U+FD51 */
  0x2325,   /* U+FD52 */
  0x23A3,   /* U+FD53 */
  0x23A3,   /* U+FD54 */
  0x2325,   /* U+FD55 */
  0x232C,   /* U+FD56 */
  0x232D,   /* U+FD57 */
  0x232C,   /* U+FD58 */
  0x232C,   /* U+FD59 */
  0x23C6,   /* U+FD5A */
  0x23C5,   /* U+FD5B */
  0x2325,   /* U+FD5C */
  0x232C,   /* U+FD5D */
  0x23C5,   /* U+FD5E */
  0x232C,   /* U+FD5F */
  0x232C,   /* U+FD60 */
  0x2325,   /* U+FD61 */
  0x23A3,   /* U+FD62 */
  0x23A3,   /* U+FD63 */
  0x232C,   /* U+FD64 */
  0x232C,   /* U+FD65 */
  0x23A3,   /* U+FD66 */
  0x23A3,   /* U+FD67 */
  0x23A3,   /* U+FD68 */
  0x23C6,   /* U+FD69 */
  0x232D,   /* U+FD6A */
  0x232D,   /* U+FD6B */
  0x23A3,   /* U+FD6C */
  0x23A3,   /* U+FD6D */
  0x23C5,   /* U+FD6E */
  0x23A3,   /* U+FD6F */
  0x23A3,   /* U+FD70 */
  0x232C,   /* U+FD71 */
  0x232C,   /* U+FD72 */
  0x23A3,   /* U+FD73 */
  0x23C6,   /* U+FD74 */
  0x23A3,   /* U+FD75 */
  0x23A3,   /* U+FD76 */
  0x23A3,   /* U+FD77 */
  0x23C5,   /* U+FD78 */
  0x23A3,   /* U+FD79 */
  0x23C6,   /* U+FD7A */
  0x23C5,   /* U+FD7B */
  0x23A3,   /* U+FD7C */
  0x23A3,   /* U+FD7D */
  0x232C,   /* U+FD7E */
  0x23A3,   /* U+FD7F */
  0x23A3,   /* U+FD80 */
  0x23C6,   /* U+FD81 */
  0x23C5,   /* U+FD82 */
  0x2325,   /* U+FD83 */
  0x2325,   /* U+FD84 */
  0x23A3,   /* U+FD85 */
  0x23A3,   /* U+FD86 */
  0x232C,   /* U+FD87 */
  0x232C,   /* U+FD88 */
  0x2325,   /* U+FD89 */
  0x23A3,   /* U+FD8A */
  0x23C6,   /* U+FD8B */
  0x232C,   /* U+FD8C */
  0x23A3,   /* U+FD8D */
  0x2325,   /* U+FD8E */
  0x23A3,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x232D,   /* U+FD92 */
  0x2325,   /* U+FD93 */
  0x23A3,   /* U+FD94 */
  0x23A3,   /* U+FD95 */
  0x23C5,   /* U+FD96 */
  0x23A3,   /* U+FD97 */
  0x23A3,   /* U+FD98 */
  0x23C5,   /* U+FD99 */
  0x23C6,   /* U+FD9A */
  0x23C5,   /* U+FD9B */
  0x23A3,   /* U+FD9C */
  0x23A3,   /* U+FD9D */
  0x23C6,   /* U+FD9E */
  0x23C6,   /* U+FD9F */
  0x23C5,   /* U+FDA0 */
  0x23C6,   /* U+FDA1 */
  0x23C5,   /* U+FDA2 */
  0x23C6,   /* U+FDA3 */
  0x23C5,   /* U+FDA4 */
  0x23C6,   /* U+FDA5 */
  0x23C5,   /* U+FDA6 */
  0x23C5,   /* U+FDA7 */
  0x23C5,   /* U+FDA8 */
  0x23C6,   /* U+FDA9 */
  0x23C6,   /* U+FDAA */
  0x23C6,   /* U+FDAB */
  0x23C6,   /* U+FDAC */
  0x23C6,   /* U+FDAD */
  0x23C6,   /* U+FDAE */
  0x23C6,   /* U+FDAF */
  0x23C6,   /* U+FDB0 */
  0x23C6,   /* U+FDB1 */
  0x23C6,   /* U+FDB2 */
  0x23C6,   /* U+FDB3 */
  0x232C,   /* U+FDB4 */
  0x23A3,   /* U+FDB5 */
  0x23C6,   /* U+FDB6 */
  0x23C6,   /* U+FDB7 */
  0x232C,   /* U+FDB8 */
  0x23C6,   /* U+FDB9 */
  0x23A3,   /* U+FDBA */
  0x23A3,   /* U+FDBB */
  0x23A3,   /* U+FDBC */
  0x232C,   /* U+FDBD */
  0x23C6,   /* U+FDBE */
  0x23C6,   /* U+FDBF */
  0x23C6,   /* U+FDC0 */
  0x23C6,   /* U+FDC1 */
  0x23C6,   /* U+FDC2 */
  0x23A3,   /* U+FDC3 */
  0x23A3,   /* U+FDC4 */
  0x23A3,   /* U+FDC5 */
  0x23C6,   /* U+FDC6 */
  0x23C6,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x23D4,   /* U+FDF0 */
  0x23D4,   /* U+FDF1 */
  0x239C,   /* U+FDF2 */
  0x230D,   /* U+FDF3 */
  0x23A3,   /* U+FDF4 */
  0x236E,   /* U+FDF5 */
  0x23B7,   /* U+FDF6 */
  0x23C6,   /* U+FDF7 */
  0x239C,   /* U+FDF8 */
  0x23C5,   /* U+FDF9 */
  0x23C5,   /* U+FDFA */
  0x0209,   /* U+FDFB */
  0x230B,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0020,   /* U+FD50 */
  0x0020,   /* U+FD51 */
  0x0020,   /* U+FD52 */
  0x0020,   /* U+FD53 */
  0x0020,   /* U+FD54 */
  0x0020,   /* U+FD55 */
  0x0020,   /* U+FD56 */
  0x0020,   /* U+FD57 */
  0x0020,   /* U+FD58 */
  0x0020,   /* U+FD59 */
  0x0020,   /* U+FD5A */
  0x0020,   /* U+FD5B */
  0x0020,   /* U+FD5C */
  0x0020,   /* U+FD5D */
  0x0020,   /* U+FD5E */
  0x0020,   /* U+FD5F */
  0x0020,   /* U+FD60 */
  0x0020,   /* U+FD61 */
  0x0020,   /* U+FD62 */
  0x0020,   /* U+FD63 */
  0x0020,   /* U+FD64 */
  0x0020,   /* U+FD65 */
  0x0020,   /* U+FD66 */
  0x0020,   /* U+FD67 */
  0x0020,   /* U+FD68 */
  0x0020,   /* U+FD69 */
  0x0020,   /* U+FD6A */
  0x0020,   /* U+FD6B */
  0x0020,   /* U+FD6C */
  0x0020,   /* U+FD6D */
  0x0020,   /* U+FD6E */
  0x0020,   /* U+FD6F */
  0x0020,   /* U+FD70 */
  0x0020,   /* U+FD71 */
  0x0020,   /* U+FD72 */
  0x0020,   /* U+FD73 */
  0x0020,   /* U+FD74 */
  0x0020,   /* U+FD75 */
  0x0020,   /* U+FD76 */
  0x0020,   /* U+FD77 */
  0x0020,   /* U+FD78 */
  0x0020,   /* U+FD79 */
  0x0020,   /* U+FD7A */
  0x0020,   /* U+FD7B */
  0x0020,   /* U+FD7C */
  0x0020,   /* U+FD7D */
  0x0020,   /* U+FD7E */
  0x0020,   /* U+FD7F */
  0x0020,   /* U+FD80 */
  0x0020,   /* U+FD81 */
  0x0020,   /* U+FD82 */
  0x0020,   /* U+FD83 */
  0x0020,   /* U+FD84 */
  0x0020,   /* U+FD85 */
  0x0020,   /* U+FD86 */
  0x0020,   /* U+FD87 */
  0x0020,   /* U+FD88 */
  0x0020,   /* U+FD89 */
  0x0020,   /* U+FD8A */
  0x0020,   /* U+FD8B */
  0x0020,   /* U+FD8C */
  0x0020,   /* U+FD8D */
  0x0020,   /* U+FD8E */
  0x0020,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0020,   /* U+FD92 */
  0x0020,   /* U+FD93 */
  0x0020,   /* U+FD94 */
  0x0020,   /* U+FD95 */
  0x0020,   /* U+FD96 */
  0x0020,   /* U+FD97 */
  0x0020,   /* U+FD98 */
  0x0020,   /* U+FD99 */
  0x0020,   /* U+FD9A */
  0x0020,   /* U+FD9B */
  0x0020,   /* U+FD9C */
  0x0020,   /* U+FD9D */
  0x0020,   /* U+FD9E */
  0x0020,   /* U+FD9F */
  0x0020,   /* U+FDA0 */
  0x0020,   /* U+FDA1 */
  0x0020,   /* U+FDA2 */
  0x0020,   /* U+FDA3 */
  0x0020,   /* U+FDA4 */
  0x0020,   /* U+FDA5 */
  0x0020,   /* U+FDA6 */
  0x0020,   /* U+FDA7 */
  0x0020,   /* U+FDA8 */
  0x0020,   /* U+FDA9 */
  0x0020,   /* U+FDAA */
  0x0020,   /* U+FDAB */
  0x0020,   /* U+FDAC */
  0x0020,   /* U+FDAD */
  0x0020,   /* U+FDAE */
  0x0020,   /* U+FDAF */
  0x0020,   /* U+FDB0 */
  0x0020,   /* U+FDB1 */
  0x0020,   /* U+FDB2 */
  0x0020,   /* U+FDB3 */
  0x0020,   /* U+FDB4 */
  0x0020,   /* U+FDB5 */
  0x0020,   /* U+FDB6 */
  0x0020,   /* U+FDB7 */
  0x0020,   /* U+FDB8 */
  0x0020,   /* U+FDB9 */
  0x0020,   /* U+FDBA */
  0x0020,   /* U+FDBB */
  0x0020,   /* U+FDBC */
  0x0020,   /* U+FDBD */
  0x0020,   /* U+FDBE */
  0x0020,   /* U+FDBF */
  0x0020,   /* U+FDC0 */
  0x0020,   /* U+FDC1 */
  0x0020,   /* U+FDC2 */
  0x0020,   /* U+FDC3 */
  0x0020,   /* U+FDC4 */
  0x0020,   /* U+FDC5 */
  0x0020,   /* U+FDC6 */
  0x0020,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0020,   /* U+FDF0 */
  0x0020,   /* U+FDF1 */
  0x0020,   /* U+FDF2 */
  0x0020,   /* U+FDF3 */
  0x0020,   /* U+FDF4 */
  0x0020,   /* U+FDF5 */
  0x0020,   /* U+FDF6 */
  0x0020,   /* U+FDF7 */
  0x0020,   /* U+FDF8 */
  0x0020,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0020,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0017,   /* U+FD50 */
  0x0019,   /* U+FD51 */
  0x0017,   /* U+FD52 */
  0x0017,   /* U+FD53 */
  0x0017,   /* U+FD54 */
  0x0017,   /* U+FD55 */
  0x0017,   /* U+FD56 */
  0x0017,   /* U+FD57 */
  0x0019,   /* U+FD58 */
  0x0017,   /* U+FD59 */
  0x0019,   /* U+FD5A */
  0x0019,   /* U+FD5B */
  0x0017,   /* U+FD5C */
  0x0017,   /* U+FD5D */
  0x0019,   /* U+FD5E */
  0x0019,   /* U+FD5F */
  0x0017,   /* U+FD60 */
  0x0017,   /* U+FD61 */
  0x0019,   /* U+FD62 */
  0x0017,   /* U+FD63 */
  0x0019,   /* U+FD64 */
  0x0017,   /* U+FD65 */
  0x0019,   /* U+FD66 */
  0x0019,   /* U+FD67 */
  0x0017,   /* U+FD68 */
  0x0019,   /* U+FD69 */
  0x0019,   /* U+FD6A */
  0x0017,   /* U+FD6B */
  0x0019,   /* U+FD6C */
  0x0017,   /* U+FD6D */
  0x0019,   /* U+FD6E */
  0x0019,   /* U+FD6F */
  0x0017,   /* U+FD70 */
  0x0019,   /* U+FD71 */
  0x0017,   /* U+FD72 */
  0x0017,   /* U+FD73 */
  0x0019,   /* U+FD74 */
  0x0019,   /* U+FD75 */
  0x0019,   /* U+FD76 */
  0x0017,   /* U+FD77 */
  0x0019,   /* U+FD78 */
  0x0019,   /* U+FD79 */
  0x0019,   /* U+FD7A */
  0x0019,   /* U+FD7B */
  0x0019,   /* U+FD7C */
  0x0017,   /* U+FD7D */
  0x0019,   /* U+FD7E */
  0x0019,   /* U+FD7F */
  0x0019,   /* U+FD80 */
  0x0019,   /* U+FD81 */
  0x0019,   /* U+FD82 */
  0x0017,   /* U+FD83 */
  0x0019,   /* U+FD84 */
  0x0019,   /* U+FD85 */
  0x0017,   /* U+FD86 */
  0x0019,   /* U+FD87 */
  0x0017,   /* U+FD88 */
  0x0017,   /* U+FD89 */
  0x0017,   /* U+FD8A */
  0x0019,   /* U+FD8B */
  0x0017,   /* U+FD8C */
  0x0017,   /* U+FD8D */
  0x0017,   /* U+FD8E */
  0x0017,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0017,   /* U+FD92 */
  0x0017,   /* U+FD93 */
  0x0017,   /* U+FD94 */
  0x0017,   /* U+FD95 */
  0x0019,   /* U+FD96 */
  0x0019,   /* U+FD97 */
  0x0017,   /* U+FD98 */
  0x0019,   /* U+FD99 */
  0x0019,   /* U+FD9A */
  0x0019,   /* U+FD9B */
  0x0019,   /* U+FD9C */
  0x0017,   /* U+FD9D */
  0x0019,   /* U+FD9E */
  0x0019,   /* U+FD9F */
  0x0019,   /* U+FDA0 */
  0x0019,   /* U+FDA1 */
  0x0019,   /* U+FDA2 */
  0x0019,   /* U+FDA3 */
  0x0019,   /* U+FDA4 */
  0x0019,   /* U+FDA5 */
  0x0019,   /* U+FDA6 */
  0x0019,   /* U+FDA7 */
  0x0019,   /* U+FDA8 */
  0x0019,   /* U+FDA9 */
  0x0019,   /* U+FDAA */
  0x0019,   /* U+FDAB */
  0x0019,   /* U+FDAC */
  0x0019,   /* U+FDAD */
  0x0019,   /* U+FDAE */
  0x0019,   /* U+FDAF */
  0x0019,   /* U+FDB0 */
  0x0019,   /* U+FDB1 */
  0x0019,   /* U+FDB2 */
  0x0019,   /* U+FDB3 */
  0x0017,   /* U+FDB4 */
  0x0017,   /* U+FDB5 */
  0x0019,   /* U+FDB6 */
  0x0019,   /* U+FDB7 */
  0x0017,   /* U+FDB8 */
  0x0019,   /* U+FDB9 */
  0x0017,   /* U+FDBA */
  0x0019,   /* U+FDBB */
  0x0019,   /* U+FDBC */
  0x0019,   /* U+FDBD */
  0x0019,   /* U+FDBE */
  0x0019,   /* U+FDBF */
  0x0019,   /* U+FDC0 */
  0x0019,   /* U+FDC1 */
  0x0019,   /* U+FDC2 */
  0x0017,   /* U+FDC3 */
  0x0017,   /* U+FDC4 */
  0x0017,   /* U+FDC5 */
  0x0019,   /* U+FDC6 */
  0x0019,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x001A,   /* U+FDF0 */
  0x001A,   /* U+FDF1 */
  0x001A,   /* U+FDF2 */
  0x001A,   /* U+FDF3 */
  0x001A,   /* U+FDF4 */
  0x001A,   /* U+FDF5 */
  0x001A,   /* U+FDF6 */
  0x001A,   /* U+FDF7 */
  0x001A,   /* U+FDF8 */
  0x001A,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x001A,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x23B1,   /* U+FDF2 */
  0x2346,   /* U+FDF3 */
  0x2337,   /* U+FDF4 */
  0x23A3,   /* U+FDF5 */
  0x239C,   /* U+FDF6 */
  0x23B1,   /* U+FDF7 */
  0x23A3,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0209,   /* U+FDFA */
  0x2325,   /* U+FDFB */
  0x239C,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0020,   /* U+FDF2 */
  0x0020,   /* U+FDF3 */
  0x0020,   /* U+FDF4 */
  0x0020,   /* U+FDF5 */
  0x0020,   /* U+FDF6 */
  0x0020,   /* U+FDF7 */
  0x0020,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0020,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x001A,   /* U+FDF2 */
  0x001A,   /* U+FDF3 */
  0x001A,   /* U+FDF4 */
  0x001A,   /* U+FDF5 */
  0x001A,   /* U+FDF6 */
  0x001A,   /* U+FDF7 */
  0x001A,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x001A,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 5 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x230B,   /* U+FDFA */
  0x239C,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 5 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 5 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 6 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x239C,   /* U+FDFA */
  0x230B,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 6 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 6 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 7 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x239C,   /* U+FDFA */
  0x239C,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 7 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 7 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Primary weight 8 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x23B1,   /* U+FDFA */
  0x23B1,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Secondary weight 8 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x0020,   /* U+FDFA */
  0x0020,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */

  /* Tertiary weight 8 for each character. */
  0x0000,   /* U+FD00 */
  0x0000,   /* U+FD01 */
  0x0000,   /* U+FD02 */
  0x0000,   /* U+FD03 */
  0x0000,   /* U+FD04 */
  0x0000,   /* U+FD05 */
  0x0000,   /* U+FD06 */
  0x0000,   /* U+FD07 */
  0x0000,   /* U+FD08 */
  0x0000,   /* U+FD09 */
  0x0000,   /* U+FD0A */
  0x0000,   /* U+FD0B */
  0x0000,   /* U+FD0C */
  0x0000,   /* U+FD0D */
  0x0000,   /* U+FD0E */
  0x0000,   /* U+FD0F */
  0x0000,   /* U+FD10 */
  0x0000,   /* U+FD11 */
  0x0000,   /* U+FD12 */
  0x0000,   /* U+FD13 */
  0x0000,   /* U+FD14 */
  0x0000,   /* U+FD15 */
  0x0000,   /* U+FD16 */
  0x0000,   /* U+FD17 */
  0x0000,   /* U+FD18 */
  0x0000,   /* U+FD19 */
  0x0000,   /* U+FD1A */
  0x0000,   /* U+FD1B */
  0x0000,   /* U+FD1C */
  0x0000,   /* U+FD1D */
  0x0000,   /* U+FD1E */
  0x0000,   /* U+FD1F */
  0x0000,   /* U+FD20 */
  0x0000,   /* U+FD21 */
  0x0000,   /* U+FD22 */
  0x0000,   /* U+FD23 */
  0x0000,   /* U+FD24 */
  0x0000,   /* U+FD25 */
  0x0000,   /* U+FD26 */
  0x0000,   /* U+FD27 */
  0x0000,   /* U+FD28 */
  0x0000,   /* U+FD29 */
  0x0000,   /* U+FD2A */
  0x0000,   /* U+FD2B */
  0x0000,   /* U+FD2C */
  0x0000,   /* U+FD2D */
  0x0000,   /* U+FD2E */
  0x0000,   /* U+FD2F */
  0x0000,   /* U+FD30 */
  0x0000,   /* U+FD31 */
  0x0000,   /* U+FD32 */
  0x0000,   /* U+FD33 */
  0x0000,   /* U+FD34 */
  0x0000,   /* U+FD35 */
  0x0000,   /* U+FD36 */
  0x0000,   /* U+FD37 */
  0x0000,   /* U+FD38 */
  0x0000,   /* U+FD39 */
  0x0000,   /* U+FD3A */
  0x0000,   /* U+FD3B */
  0x0000,   /* U+FD3C */
  0x0000,   /* U+FD3D */
  0x0000,   /* U+FD3E */
  0x0000,   /* U+FD3F */
  0x0000,   /* U+FD40 */
  0x0000,   /* U+FD41 */
  0x0000,   /* U+FD42 */
  0x0000,   /* U+FD43 */
  0x0000,   /* U+FD44 */
  0x0000,   /* U+FD45 */
  0x0000,   /* U+FD46 */
  0x0000,   /* U+FD47 */
  0x0000,   /* U+FD48 */
  0x0000,   /* U+FD49 */
  0x0000,   /* U+FD4A */
  0x0000,   /* U+FD4B */
  0x0000,   /* U+FD4C */
  0x0000,   /* U+FD4D */
  0x0000,   /* U+FD4E */
  0x0000,   /* U+FD4F */
  0x0000,   /* U+FD50 */
  0x0000,   /* U+FD51 */
  0x0000,   /* U+FD52 */
  0x0000,   /* U+FD53 */
  0x0000,   /* U+FD54 */
  0x0000,   /* U+FD55 */
  0x0000,   /* U+FD56 */
  0x0000,   /* U+FD57 */
  0x0000,   /* U+FD58 */
  0x0000,   /* U+FD59 */
  0x0000,   /* U+FD5A */
  0x0000,   /* U+FD5B */
  0x0000,   /* U+FD5C */
  0x0000,   /* U+FD5D */
  0x0000,   /* U+FD5E */
  0x0000,   /* U+FD5F */
  0x0000,   /* U+FD60 */
  0x0000,   /* U+FD61 */
  0x0000,   /* U+FD62 */
  0x0000,   /* U+FD63 */
  0x0000,   /* U+FD64 */
  0x0000,   /* U+FD65 */
  0x0000,   /* U+FD66 */
  0x0000,   /* U+FD67 */
  0x0000,   /* U+FD68 */
  0x0000,   /* U+FD69 */
  0x0000,   /* U+FD6A */
  0x0000,   /* U+FD6B */
  0x0000,   /* U+FD6C */
  0x0000,   /* U+FD6D */
  0x0000,   /* U+FD6E */
  0x0000,   /* U+FD6F */
  0x0000,   /* U+FD70 */
  0x0000,   /* U+FD71 */
  0x0000,   /* U+FD72 */
  0x0000,   /* U+FD73 */
  0x0000,   /* U+FD74 */
  0x0000,   /* U+FD75 */
  0x0000,   /* U+FD76 */
  0x0000,   /* U+FD77 */
  0x0000,   /* U+FD78 */
  0x0000,   /* U+FD79 */
  0x0000,   /* U+FD7A */
  0x0000,   /* U+FD7B */
  0x0000,   /* U+FD7C */
  0x0000,   /* U+FD7D */
  0x0000,   /* U+FD7E */
  0x0000,   /* U+FD7F */
  0x0000,   /* U+FD80 */
  0x0000,   /* U+FD81 */
  0x0000,   /* U+FD82 */
  0x0000,   /* U+FD83 */
  0x0000,   /* U+FD84 */
  0x0000,   /* U+FD85 */
  0x0000,   /* U+FD86 */
  0x0000,   /* U+FD87 */
  0x0000,   /* U+FD88 */
  0x0000,   /* U+FD89 */
  0x0000,   /* U+FD8A */
  0x0000,   /* U+FD8B */
  0x0000,   /* U+FD8C */
  0x0000,   /* U+FD8D */
  0x0000,   /* U+FD8E */
  0x0000,   /* U+FD8F */
  0x0000,   /* U+FD90 */
  0x0000,   /* U+FD91 */
  0x0000,   /* U+FD92 */
  0x0000,   /* U+FD93 */
  0x0000,   /* U+FD94 */
  0x0000,   /* U+FD95 */
  0x0000,   /* U+FD96 */
  0x0000,   /* U+FD97 */
  0x0000,   /* U+FD98 */
  0x0000,   /* U+FD99 */
  0x0000,   /* U+FD9A */
  0x0000,   /* U+FD9B */
  0x0000,   /* U+FD9C */
  0x0000,   /* U+FD9D */
  0x0000,   /* U+FD9E */
  0x0000,   /* U+FD9F */
  0x0000,   /* U+FDA0 */
  0x0000,   /* U+FDA1 */
  0x0000,   /* U+FDA2 */
  0x0000,   /* U+FDA3 */
  0x0000,   /* U+FDA4 */
  0x0000,   /* U+FDA5 */
  0x0000,   /* U+FDA6 */
  0x0000,   /* U+FDA7 */
  0x0000,   /* U+FDA8 */
  0x0000,   /* U+FDA9 */
  0x0000,   /* U+FDAA */
  0x0000,   /* U+FDAB */
  0x0000,   /* U+FDAC */
  0x0000,   /* U+FDAD */
  0x0000,   /* U+FDAE */
  0x0000,   /* U+FDAF */
  0x0000,   /* U+FDB0 */
  0x0000,   /* U+FDB1 */
  0x0000,   /* U+FDB2 */
  0x0000,   /* U+FDB3 */
  0x0000,   /* U+FDB4 */
  0x0000,   /* U+FDB5 */
  0x0000,   /* U+FDB6 */
  0x0000,   /* U+FDB7 */
  0x0000,   /* U+FDB8 */
  0x0000,   /* U+FDB9 */
  0x0000,   /* U+FDBA */
  0x0000,   /* U+FDBB */
  0x0000,   /* U+FDBC */
  0x0000,   /* U+FDBD */
  0x0000,   /* U+FDBE */
  0x0000,   /* U+FDBF */
  0x0000,   /* U+FDC0 */
  0x0000,   /* U+FDC1 */
  0x0000,   /* U+FDC2 */
  0x0000,   /* U+FDC3 */
  0x0000,   /* U+FDC4 */
  0x0000,   /* U+FDC5 */
  0x0000,   /* U+FDC6 */
  0x0000,   /* U+FDC7 */
  0x0000,   /* U+FDC8 */
  0x0000,   /* U+FDC9 */
  0x0000,   /* U+FDCA */
  0x0000,   /* U+FDCB */
  0x0000,   /* U+FDCC */
  0x0000,   /* U+FDCD */
  0x0000,   /* U+FDCE */
  0x0000,   /* U+FDCF */
  0x0000,   /* U+FDD0 */
  0x0000,   /* U+FDD1 */
  0x0000,   /* U+FDD2 */
  0x0000,   /* U+FDD3 */
  0x0000,   /* U+FDD4 */
  0x0000,   /* U+FDD5 */
  0x0000,   /* U+FDD6 */
  0x0000,   /* U+FDD7 */
  0x0000,   /* U+FDD8 */
  0x0000,   /* U+FDD9 */
  0x0000,   /* U+FDDA */
  0x0000,   /* U+FDDB */
  0x0000,   /* U+FDDC */
  0x0000,   /* U+FDDD */
  0x0000,   /* U+FDDE */
  0x0000,   /* U+FDDF */
  0x0000,   /* U+FDE0 */
  0x0000,   /* U+FDE1 */
  0x0000,   /* U+FDE2 */
  0x0000,   /* U+FDE3 */
  0x0000,   /* U+FDE4 */
  0x0000,   /* U+FDE5 */
  0x0000,   /* U+FDE6 */
  0x0000,   /* U+FDE7 */
  0x0000,   /* U+FDE8 */
  0x0000,   /* U+FDE9 */
  0x0000,   /* U+FDEA */
  0x0000,   /* U+FDEB */
  0x0000,   /* U+FDEC */
  0x0000,   /* U+FDED */
  0x0000,   /* U+FDEE */
  0x0000,   /* U+FDEF */
  0x0000,   /* U+FDF0 */
  0x0000,   /* U+FDF1 */
  0x0000,   /* U+FDF2 */
  0x0000,   /* U+FDF3 */
  0x0000,   /* U+FDF4 */
  0x0000,   /* U+FDF5 */
  0x0000,   /* U+FDF6 */
  0x0000,   /* U+FDF7 */
  0x0000,   /* U+FDF8 */
  0x0000,   /* U+FDF9 */
  0x001A,   /* U+FDFA */
  0x001A,   /* U+FDFB */
  0x0000,   /* U+FDFC */
  0x0000,   /* U+FDFD */
  0x0000,   /* U+FDFE */
  0x0000,   /* U+FDFF */
};

uint16 uca900_p0FE[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0222,   /* U+FE10 */
  0x0231,   /* U+FE11 */
  0x028A,   /* U+FE12 */
  0x0239,   /* U+FE13 */
  0x0234,   /* U+FE14 */
  0x0260,   /* U+FE15 */
  0x0266,   /* U+FE16 */
  0x037B,   /* U+FE17 */
  0x037C,   /* U+FE18 */
  0x0277,   /* U+FE19 */
  0xFBC1,   /* U+FE1A */
  0xFBC1,   /* U+FE1B */
  0xFBC1,   /* U+FE1C */
  0xFBC1,   /* U+FE1D */
  0xFBC1,   /* U+FE1E */
  0xFBC1,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0277,   /* U+FE30 */
  0x0216,   /* U+FE31 */
  0x0215,   /* U+FE32 */
  0x020B,   /* U+FE33 */
  0x020B,   /* U+FE34 */
  0x0317,   /* U+FE35 */
  0x0318,   /* U+FE36 */
  0x031B,   /* U+FE37 */
  0x031C,   /* U+FE38 */
  0x0379,   /* U+FE39 */
  0x037A,   /* U+FE3A */
  0x0377,   /* U+FE3B */
  0x0378,   /* U+FE3C */
  0x0371,   /* U+FE3D */
  0x0372,   /* U+FE3E */
  0x036F,   /* U+FE3F */
  0x0370,   /* U+FE40 */
  0x0373,   /* U+FE41 */
  0x0374,   /* U+FE42 */
  0x0375,   /* U+FE43 */
  0x0376,   /* U+FE44 */
  0x0232,   /* U+FE45 */
  0x0233,   /* U+FE46 */
  0x0319,   /* U+FE47 */
  0x031A,   /* U+FE48 */
  0x020A,   /* U+FE49 */
  0x020A,   /* U+FE4A */
  0x020A,   /* U+FE4B */
  0x020A,   /* U+FE4C */
  0x020B,   /* U+FE4D */
  0x020B,   /* U+FE4E */
  0x020B,   /* U+FE4F */
  0x0222,   /* U+FE50 */
  0x0231,   /* U+FE51 */
  0x0277,   /* U+FE52 */
  0xFBC1,   /* U+FE53 */
  0x0234,   /* U+FE54 */
  0x0239,   /* U+FE55 */
  0x0266,   /* U+FE56 */
  0x0260,   /* U+FE57 */
  0x0216,   /* U+FE58 */
  0x0317,   /* U+FE59 */
  0x0318,   /* U+FE5A */
  0x031B,   /* U+FE5B */
  0x031C,   /* U+FE5C */
  0x0379,   /* U+FE5D */
  0x037A,   /* U+FE5E */
  0x0398,   /* U+FE5F */
  0x0396,   /* U+FE60 */
  0x038F,   /* U+FE61 */
  0x0616,   /* U+FE62 */
  0x020D,   /* U+FE63 */
  0x061A,   /* U+FE64 */
  0x061C,   /* U+FE65 */
  0x061B,   /* U+FE66 */
  0xFBC1,   /* U+FE67 */
  0x0395,   /* U+FE68 */
  0x1C12,   /* U+FE69 */
  0x0399,   /* U+FE6A */
  0x038E,   /* U+FE6B */
  0xFBC1,   /* U+FE6C */
  0xFBC1,   /* U+FE6D */
  0xFBC1,   /* U+FE6E */
  0xFBC1,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0xFBC1,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x22FD,   /* U+FE80 */
  0x22FE,   /* U+FE81 */
  0x22FE,   /* U+FE82 */
  0x22FF,   /* U+FE83 */
  0x22FF,   /* U+FE84 */
  0x2302,   /* U+FE85 */
  0x2302,   /* U+FE86 */
  0x2303,   /* U+FE87 */
  0x2303,   /* U+FE88 */
  0x2307,   /* U+FE89 */
  0x2307,   /* U+FE8A */
  0x2307,   /* U+FE8B */
  0x2307,   /* U+FE8C */
  0x230B,   /* U+FE8D */
  0x230B,   /* U+FE8E */
  0x230D,   /* U+FE8F */
  0x230D,   /* U+FE90 */
  0x230D,   /* U+FE91 */
  0x230D,   /* U+FE92 */
  0x231C,   /* U+FE93 */
  0x231C,   /* U+FE94 */
  0x231D,   /* U+FE95 */
  0x231D,   /* U+FE96 */
  0x231D,   /* U+FE97 */
  0x231D,   /* U+FE98 */
  0x231E,   /* U+FE99 */
  0x231E,   /* U+FE9A */
  0x231E,   /* U+FE9B */
  0x231E,   /* U+FE9C */
  0x2325,   /* U+FE9D */
  0x2325,   /* U+FE9E */
  0x2325,   /* U+FE9F */
  0x2325,   /* U+FEA0 */
  0x232C,   /* U+FEA1 */
  0x232C,   /* U+FEA2 */
  0x232C,   /* U+FEA3 */
  0x232C,   /* U+FEA4 */
  0x232D,   /* U+FEA5 */
  0x232D,   /* U+FEA6 */
  0x232D,   /* U+FEA7 */
  0x232D,   /* U+FEA8 */
  0x2337,   /* U+FEA9 */
  0x2337,   /* U+FEAA */
  0x2338,   /* U+FEAB */
  0x2338,   /* U+FEAC */
  0x2346,   /* U+FEAD */
  0x2346,   /* U+FEAE */
  0x2347,   /* U+FEAF */
  0x2347,   /* U+FEB0 */
  0x2359,   /* U+FEB1 */
  0x2359,   /* U+FEB2 */
  0x2359,   /* U+FEB3 */
  0x2359,   /* U+FEB4 */
  0x235A,   /* U+FEB5 */
  0x235A,   /* U+FEB6 */
  0x235A,   /* U+FEB7 */
  0x235A,   /* U+FEB8 */
  0x2364,   /* U+FEB9 */
  0x2364,   /* U+FEBA */
  0x2364,   /* U+FEBB */
  0x2364,   /* U+FEBC */
  0x2365,   /* U+FEBD */
  0x2365,   /* U+FEBE */
  0x2365,   /* U+FEBF */
  0x2365,   /* U+FEC0 */
  0x236A,   /* U+FEC1 */
  0x236A,   /* U+FEC2 */
  0x236A,   /* U+FEC3 */
  0x236A,   /* U+FEC4 */
  0x236B,   /* U+FEC5 */
  0x236B,   /* U+FEC6 */
  0x236B,   /* U+FEC7 */
  0x236B,   /* U+FEC8 */
  0x236E,   /* U+FEC9 */
  0x236E,   /* U+FECA */
  0x236E,   /* U+FECB */
  0x236E,   /* U+FECC */
  0x236F,   /* U+FECD */
  0x236F,   /* U+FECE */
  0x236F,   /* U+FECF */
  0x236F,   /* U+FED0 */
  0x2376,   /* U+FED1 */
  0x2376,   /* U+FED2 */
  0x2376,   /* U+FED3 */
  0x2376,   /* U+FED4 */
  0x2382,   /* U+FED5 */
  0x2382,   /* U+FED6 */
  0x2382,   /* U+FED7 */
  0x2382,   /* U+FED8 */
  0x2387,   /* U+FED9 */
  0x2387,   /* U+FEDA */
  0x2387,   /* U+FEDB */
  0x2387,   /* U+FEDC */
  0x239C,   /* U+FEDD */
  0x239C,   /* U+FEDE */
  0x239C,   /* U+FEDF */
  0x239C,   /* U+FEE0 */
  0x23A3,   /* U+FEE1 */
  0x23A3,   /* U+FEE2 */
  0x23A3,   /* U+FEE3 */
  0x23A3,   /* U+FEE4 */
  0x23A7,   /* U+FEE5 */
  0x23A7,   /* U+FEE6 */
  0x23A7,   /* U+FEE7 */
  0x23A7,   /* U+FEE8 */
  0x23B1,   /* U+FEE9 */
  0x23B1,   /* U+FEEA */
  0x23B1,   /* U+FEEB */
  0x23B1,   /* U+FEEC */
  0x23B7,   /* U+FEED */
  0x23B7,   /* U+FEEE */
  0x23C5,   /* U+FEEF */
  0x23C5,   /* U+FEF0 */
  0x23C6,   /* U+FEF1 */
  0x23C6,   /* U+FEF2 */
  0x23C6,   /* U+FEF3 */
  0x23C6,   /* U+FEF4 */
  0x239C,   /* U+FEF5 */
  0x239C,   /* U+FEF6 */
  0x239C,   /* U+FEF7 */
  0x239C,   /* U+FEF8 */
  0x239C,   /* U+FEF9 */
  0x239C,   /* U+FEFA */
  0x239C,   /* U+FEFB */
  0x239C,   /* U+FEFC */
  0xFBC1,   /* U+FEFD */
  0xFBC1,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Secondary weight 1 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0020,   /* U+FE10 */
  0x0020,   /* U+FE11 */
  0x0020,   /* U+FE12 */
  0x0020,   /* U+FE13 */
  0x0020,   /* U+FE14 */
  0x0020,   /* U+FE15 */
  0x0020,   /* U+FE16 */
  0x0020,   /* U+FE17 */
  0x0020,   /* U+FE18 */
  0x0020,   /* U+FE19 */
  0x0020,   /* U+FE1A */
  0x0020,   /* U+FE1B */
  0x0020,   /* U+FE1C */
  0x0020,   /* U+FE1D */
  0x0020,   /* U+FE1E */
  0x0020,   /* U+FE1F */
  0x004F,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x004E,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0034,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x004E,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0050,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0020,   /* U+FE30 */
  0x0020,   /* U+FE31 */
  0x0020,   /* U+FE32 */
  0x0020,   /* U+FE33 */
  0x0020,   /* U+FE34 */
  0x0020,   /* U+FE35 */
  0x0020,   /* U+FE36 */
  0x0020,   /* U+FE37 */
  0x0020,   /* U+FE38 */
  0x0020,   /* U+FE39 */
  0x0020,   /* U+FE3A */
  0x0020,   /* U+FE3B */
  0x0020,   /* U+FE3C */
  0x0020,   /* U+FE3D */
  0x0020,   /* U+FE3E */
  0x0020,   /* U+FE3F */
  0x0020,   /* U+FE40 */
  0x0020,   /* U+FE41 */
  0x0020,   /* U+FE42 */
  0x0020,   /* U+FE43 */
  0x0020,   /* U+FE44 */
  0x0020,   /* U+FE45 */
  0x0020,   /* U+FE46 */
  0x0020,   /* U+FE47 */
  0x0020,   /* U+FE48 */
  0x0020,   /* U+FE49 */
  0x0020,   /* U+FE4A */
  0x0020,   /* U+FE4B */
  0x0020,   /* U+FE4C */
  0x0020,   /* U+FE4D */
  0x0020,   /* U+FE4E */
  0x0020,   /* U+FE4F */
  0x0020,   /* U+FE50 */
  0x0020,   /* U+FE51 */
  0x0020,   /* U+FE52 */
  0x0020,   /* U+FE53 */
  0x0020,   /* U+FE54 */
  0x0020,   /* U+FE55 */
  0x0020,   /* U+FE56 */
  0x0020,   /* U+FE57 */
  0x0020,   /* U+FE58 */
  0x0020,   /* U+FE59 */
  0x0020,   /* U+FE5A */
  0x0020,   /* U+FE5B */
  0x0020,   /* U+FE5C */
  0x0020,   /* U+FE5D */
  0x0020,   /* U+FE5E */
  0x0020,   /* U+FE5F */
  0x0020,   /* U+FE60 */
  0x0020,   /* U+FE61 */
  0x0020,   /* U+FE62 */
  0x0020,   /* U+FE63 */
  0x0020,   /* U+FE64 */
  0x0020,   /* U+FE65 */
  0x0020,   /* U+FE66 */
  0x0020,   /* U+FE67 */
  0x0020,   /* U+FE68 */
  0x0020,   /* U+FE69 */
  0x0020,   /* U+FE6A */
  0x0020,   /* U+FE6B */
  0x0020,   /* U+FE6C */
  0x0020,   /* U+FE6D */
  0x0020,   /* U+FE6E */
  0x0020,   /* U+FE6F */
  0x006D,   /* U+FE70 */
  0x006D,   /* U+FE71 */
  0x0070,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0073,   /* U+FE74 */
  0x0020,   /* U+FE75 */
  0x0076,   /* U+FE76 */
  0x0076,   /* U+FE77 */
  0x007A,   /* U+FE78 */
  0x007A,   /* U+FE79 */
  0x007D,   /* U+FE7A */
  0x007D,   /* U+FE7B */
  0x0080,   /* U+FE7C */
  0x0080,   /* U+FE7D */
  0x0081,   /* U+FE7E */
  0x0081,   /* U+FE7F */
  0x0020,   /* U+FE80 */
  0x0020,   /* U+FE81 */
  0x0020,   /* U+FE82 */
  0x0020,   /* U+FE83 */
  0x0020,   /* U+FE84 */
  0x0020,   /* U+FE85 */
  0x0020,   /* U+FE86 */
  0x0020,   /* U+FE87 */
  0x0020,   /* U+FE88 */
  0x0020,   /* U+FE89 */
  0x0020,   /* U+FE8A */
  0x0020,   /* U+FE8B */
  0x0020,   /* U+FE8C */
  0x0020,   /* U+FE8D */
  0x0020,   /* U+FE8E */
  0x0020,   /* U+FE8F */
  0x0020,   /* U+FE90 */
  0x0020,   /* U+FE91 */
  0x0020,   /* U+FE92 */
  0x0020,   /* U+FE93 */
  0x0020,   /* U+FE94 */
  0x0020,   /* U+FE95 */
  0x0020,   /* U+FE96 */
  0x0020,   /* U+FE97 */
  0x0020,   /* U+FE98 */
  0x0020,   /* U+FE99 */
  0x0020,   /* U+FE9A */
  0x0020,   /* U+FE9B */
  0x0020,   /* U+FE9C */
  0x0020,   /* U+FE9D */
  0x0020,   /* U+FE9E */
  0x0020,   /* U+FE9F */
  0x0020,   /* U+FEA0 */
  0x0020,   /* U+FEA1 */
  0x0020,   /* U+FEA2 */
  0x0020,   /* U+FEA3 */
  0x0020,   /* U+FEA4 */
  0x0020,   /* U+FEA5 */
  0x0020,   /* U+FEA6 */
  0x0020,   /* U+FEA7 */
  0x0020,   /* U+FEA8 */
  0x0020,   /* U+FEA9 */
  0x0020,   /* U+FEAA */
  0x0020,   /* U+FEAB */
  0x0020,   /* U+FEAC */
  0x0020,   /* U+FEAD */
  0x0020,   /* U+FEAE */
  0x0020,   /* U+FEAF */
  0x0020,   /* U+FEB0 */
  0x0020,   /* U+FEB1 */
  0x0020,   /* U+FEB2 */
  0x0020,   /* U+FEB3 */
  0x0020,   /* U+FEB4 */
  0x0020,   /* U+FEB5 */
  0x0020,   /* U+FEB6 */
  0x0020,   /* U+FEB7 */
  0x0020,   /* U+FEB8 */
  0x0020,   /* U+FEB9 */
  0x0020,   /* U+FEBA */
  0x0020,   /* U+FEBB */
  0x0020,   /* U+FEBC */
  0x0020,   /* U+FEBD */
  0x0020,   /* U+FEBE */
  0x0020,   /* U+FEBF */
  0x0020,   /* U+FEC0 */
  0x0020,   /* U+FEC1 */
  0x0020,   /* U+FEC2 */
  0x0020,   /* U+FEC3 */
  0x0020,   /* U+FEC4 */
  0x0020,   /* U+FEC5 */
  0x0020,   /* U+FEC6 */
  0x0020,   /* U+FEC7 */
  0x0020,   /* U+FEC8 */
  0x0020,   /* U+FEC9 */
  0x0020,   /* U+FECA */
  0x0020,   /* U+FECB */
  0x0020,   /* U+FECC */
  0x0020,   /* U+FECD */
  0x0020,   /* U+FECE */
  0x0020,   /* U+FECF */
  0x0020,   /* U+FED0 */
  0x0020,   /* U+FED1 */
  0x0020,   /* U+FED2 */
  0x0020,   /* U+FED3 */
  0x0020,   /* U+FED4 */
  0x0020,   /* U+FED5 */
  0x0020,   /* U+FED6 */
  0x0020,   /* U+FED7 */
  0x0020,   /* U+FED8 */
  0x0020,   /* U+FED9 */
  0x0020,   /* U+FEDA */
  0x0020,   /* U+FEDB */
  0x0020,   /* U+FEDC */
  0x0020,   /* U+FEDD */
  0x0020,   /* U+FEDE */
  0x0020,   /* U+FEDF */
  0x0020,   /* U+FEE0 */
  0x0020,   /* U+FEE1 */
  0x0020,   /* U+FEE2 */
  0x0020,   /* U+FEE3 */
  0x0020,   /* U+FEE4 */
  0x0020,   /* U+FEE5 */
  0x0020,   /* U+FEE6 */
  0x0020,   /* U+FEE7 */
  0x0020,   /* U+FEE8 */
  0x0020,   /* U+FEE9 */
  0x0020,   /* U+FEEA */
  0x0020,   /* U+FEEB */
  0x0020,   /* U+FEEC */
  0x0020,   /* U+FEED */
  0x0020,   /* U+FEEE */
  0x0020,   /* U+FEEF */
  0x0020,   /* U+FEF0 */
  0x0020,   /* U+FEF1 */
  0x0020,   /* U+FEF2 */
  0x0020,   /* U+FEF3 */
  0x0020,   /* U+FEF4 */
  0x0020,   /* U+FEF5 */
  0x0020,   /* U+FEF6 */
  0x0020,   /* U+FEF7 */
  0x0020,   /* U+FEF8 */
  0x0020,   /* U+FEF9 */
  0x0020,   /* U+FEFA */
  0x0020,   /* U+FEFB */
  0x0020,   /* U+FEFC */
  0x0020,   /* U+FEFD */
  0x0020,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Tertiary weight 1 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0016,   /* U+FE10 */
  0x0016,   /* U+FE11 */
  0x0016,   /* U+FE12 */
  0x0016,   /* U+FE13 */
  0x0016,   /* U+FE14 */
  0x0016,   /* U+FE15 */
  0x0016,   /* U+FE16 */
  0x0016,   /* U+FE17 */
  0x0016,   /* U+FE18 */
  0x0016,   /* U+FE19 */
  0x0002,   /* U+FE1A */
  0x0002,   /* U+FE1B */
  0x0002,   /* U+FE1C */
  0x0002,   /* U+FE1D */
  0x0002,   /* U+FE1E */
  0x0002,   /* U+FE1F */
  0x0002,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0002,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0002,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0002,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0002,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0016,   /* U+FE30 */
  0x0016,   /* U+FE31 */
  0x0016,   /* U+FE32 */
  0x0016,   /* U+FE33 */
  0x0016,   /* U+FE34 */
  0x0016,   /* U+FE35 */
  0x0016,   /* U+FE36 */
  0x0016,   /* U+FE37 */
  0x0016,   /* U+FE38 */
  0x0016,   /* U+FE39 */
  0x0016,   /* U+FE3A */
  0x0016,   /* U+FE3B */
  0x0016,   /* U+FE3C */
  0x0016,   /* U+FE3D */
  0x0016,   /* U+FE3E */
  0x0016,   /* U+FE3F */
  0x0016,   /* U+FE40 */
  0x0016,   /* U+FE41 */
  0x0016,   /* U+FE42 */
  0x0016,   /* U+FE43 */
  0x0016,   /* U+FE44 */
  0x0002,   /* U+FE45 */
  0x0002,   /* U+FE46 */
  0x0016,   /* U+FE47 */
  0x0016,   /* U+FE48 */
  0x0004,   /* U+FE49 */
  0x0004,   /* U+FE4A */
  0x0004,   /* U+FE4B */
  0x0004,   /* U+FE4C */
  0x0004,   /* U+FE4D */
  0x0004,   /* U+FE4E */
  0x0004,   /* U+FE4F */
  0x000F,   /* U+FE50 */
  0x000F,   /* U+FE51 */
  0x000F,   /* U+FE52 */
  0x0002,   /* U+FE53 */
  0x000F,   /* U+FE54 */
  0x000F,   /* U+FE55 */
  0x000F,   /* U+FE56 */
  0x000F,   /* U+FE57 */
  0x000F,   /* U+FE58 */
  0x000F,   /* U+FE59 */
  0x000F,   /* U+FE5A */
  0x000F,   /* U+FE5B */
  0x000F,   /* U+FE5C */
  0x000F,   /* U+FE5D */
  0x000F,   /* U+FE5E */
  0x000F,   /* U+FE5F */
  0x000F,   /* U+FE60 */
  0x000F,   /* U+FE61 */
  0x000F,   /* U+FE62 */
  0x000F,   /* U+FE63 */
  0x000F,   /* U+FE64 */
  0x000F,   /* U+FE65 */
  0x000F,   /* U+FE66 */
  0x0002,   /* U+FE67 */
  0x000F,   /* U+FE68 */
  0x000F,   /* U+FE69 */
  0x000F,   /* U+FE6A */
  0x000F,   /* U+FE6B */
  0x0002,   /* U+FE6C */
  0x0002,   /* U+FE6D */
  0x0002,   /* U+FE6E */
  0x0002,   /* U+FE6F */
  0x001A,   /* U+FE70 */
  0x0018,   /* U+FE71 */
  0x001A,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x001A,   /* U+FE74 */
  0x0002,   /* U+FE75 */
  0x001A,   /* U+FE76 */
  0x0018,   /* U+FE77 */
  0x001A,   /* U+FE78 */
  0x0018,   /* U+FE79 */
  0x001A,   /* U+FE7A */
  0x0018,   /* U+FE7B */
  0x001A,   /* U+FE7C */
  0x0018,   /* U+FE7D */
  0x001A,   /* U+FE7E */
  0x0018,   /* U+FE7F */
  0x001A,   /* U+FE80 */
  0x001A,   /* U+FE81 */
  0x0019,   /* U+FE82 */
  0x001A,   /* U+FE83 */
  0x0019,   /* U+FE84 */
  0x001A,   /* U+FE85 */
  0x0019,   /* U+FE86 */
  0x001A,   /* U+FE87 */
  0x0019,   /* U+FE88 */
  0x001A,   /* U+FE89 */
  0x0019,   /* U+FE8A */
  0x0017,   /* U+FE8B */
  0x0018,   /* U+FE8C */
  0x001A,   /* U+FE8D */
  0x0019,   /* U+FE8E */
  0x001A,   /* U+FE8F */
  0x0019,   /* U+FE90 */
  0x0017,   /* U+FE91 */
  0x0018,   /* U+FE92 */
  0x001A,   /* U+FE93 */
  0x0019,   /* U+FE94 */
  0x001A,   /* U+FE95 */
  0x0019,   /* U+FE96 */
  0x0017,   /* U+FE97 */
  0x0018,   /* U+FE98 */
  0x001A,   /* U+FE99 */
  0x0019,   /* U+FE9A */
  0x0017,   /* U+FE9B */
  0x0018,   /* U+FE9C */
  0x001A,   /* U+FE9D */
  0x0019,   /* U+FE9E */
  0x0017,   /* U+FE9F */
  0x0018,   /* U+FEA0 */
  0x001A,   /* U+FEA1 */
  0x0019,   /* U+FEA2 */
  0x0017,   /* U+FEA3 */
  0x0018,   /* U+FEA4 */
  0x001A,   /* U+FEA5 */
  0x0019,   /* U+FEA6 */
  0x0017,   /* U+FEA7 */
  0x0018,   /* U+FEA8 */
  0x001A,   /* U+FEA9 */
  0x0019,   /* U+FEAA */
  0x001A,   /* U+FEAB */
  0x0019,   /* U+FEAC */
  0x001A,   /* U+FEAD */
  0x0019,   /* U+FEAE */
  0x001A,   /* U+FEAF */
  0x0019,   /* U+FEB0 */
  0x001A,   /* U+FEB1 */
  0x0019,   /* U+FEB2 */
  0x0017,   /* U+FEB3 */
  0x0018,   /* U+FEB4 */
  0x001A,   /* U+FEB5 */
  0x0019,   /* U+FEB6 */
  0x0017,   /* U+FEB7 */
  0x0018,   /* U+FEB8 */
  0x001A,   /* U+FEB9 */
  0x0019,   /* U+FEBA */
  0x0017,   /* U+FEBB */
  0x0018,   /* U+FEBC */
  0x001A,   /* U+FEBD */
  0x0019,   /* U+FEBE */
  0x0017,   /* U+FEBF */
  0x0018,   /* U+FEC0 */
  0x001A,   /* U+FEC1 */
  0x0019,   /* U+FEC2 */
  0x0017,   /* U+FEC3 */
  0x0018,   /* U+FEC4 */
  0x001A,   /* U+FEC5 */
  0x0019,   /* U+FEC6 */
  0x0017,   /* U+FEC7 */
  0x0018,   /* U+FEC8 */
  0x001A,   /* U+FEC9 */
  0x0019,   /* U+FECA */
  0x0017,   /* U+FECB */
  0x0018,   /* U+FECC */
  0x001A,   /* U+FECD */
  0x0019,   /* U+FECE */
  0x0017,   /* U+FECF */
  0x0018,   /* U+FED0 */
  0x001A,   /* U+FED1 */
  0x0019,   /* U+FED2 */
  0x0017,   /* U+FED3 */
  0x0018,   /* U+FED4 */
  0x001A,   /* U+FED5 */
  0x0019,   /* U+FED6 */
  0x0017,   /* U+FED7 */
  0x0018,   /* U+FED8 */
  0x001A,   /* U+FED9 */
  0x0019,   /* U+FEDA */
  0x0017,   /* U+FEDB */
  0x0018,   /* U+FEDC */
  0x001A,   /* U+FEDD */
  0x0019,   /* U+FEDE */
  0x0017,   /* U+FEDF */
  0x0018,   /* U+FEE0 */
  0x001A,   /* U+FEE1 */
  0x0019,   /* U+FEE2 */
  0x0017,   /* U+FEE3 */
  0x0018,   /* U+FEE4 */
  0x001A,   /* U+FEE5 */
  0x0019,   /* U+FEE6 */
  0x0017,   /* U+FEE7 */
  0x0018,   /* U+FEE8 */
  0x001A,   /* U+FEE9 */
  0x0019,   /* U+FEEA */
  0x0017,   /* U+FEEB */
  0x0018,   /* U+FEEC */
  0x001A,   /* U+FEED */
  0x0019,   /* U+FEEE */
  0x001A,   /* U+FEEF */
  0x0019,   /* U+FEF0 */
  0x001A,   /* U+FEF1 */
  0x0019,   /* U+FEF2 */
  0x0017,   /* U+FEF3 */
  0x0018,   /* U+FEF4 */
  0x001A,   /* U+FEF5 */
  0x0019,   /* U+FEF6 */
  0x001A,   /* U+FEF7 */
  0x0019,   /* U+FEF8 */
  0x001A,   /* U+FEF9 */
  0x0019,   /* U+FEFA */
  0x001A,   /* U+FEFB */
  0x0019,   /* U+FEFC */
  0x0002,   /* U+FEFD */
  0x0002,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0277,   /* U+FE19 */
  0xFE1A,   /* U+FE1A */
  0xFE1B,   /* U+FE1B */
  0xFE1C,   /* U+FE1C */
  0xFE1D,   /* U+FE1D */
  0xFE1E,   /* U+FE1E */
  0xFE1F,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0277,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0xFE53,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0xFE67,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0xFE6C,   /* U+FE6C */
  0xFE6D,   /* U+FE6D */
  0xFE6E,   /* U+FE6E */
  0xFE6F,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0xFE75,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x22FE,   /* U+FEF5 */
  0x22FE,   /* U+FEF6 */
  0x22FF,   /* U+FEF7 */
  0x22FF,   /* U+FEF8 */
  0x2303,   /* U+FEF9 */
  0x2303,   /* U+FEFA */
  0x230B,   /* U+FEFB */
  0x230B,   /* U+FEFC */
  0xFEFD,   /* U+FEFD */
  0xFEFE,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0020,   /* U+FE19 */
  0x0000,   /* U+FE1A */
  0x0000,   /* U+FE1B */
  0x0000,   /* U+FE1C */
  0x0000,   /* U+FE1D */
  0x0000,   /* U+FE1E */
  0x0000,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0020,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0x0000,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0x0000,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0x0000,   /* U+FE6C */
  0x0000,   /* U+FE6D */
  0x0000,   /* U+FE6E */
  0x0000,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0x0000,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x0020,   /* U+FEF5 */
  0x0020,   /* U+FEF6 */
  0x0020,   /* U+FEF7 */
  0x0020,   /* U+FEF8 */
  0x0020,   /* U+FEF9 */
  0x0020,   /* U+FEFA */
  0x0020,   /* U+FEFB */
  0x0020,   /* U+FEFC */
  0x0000,   /* U+FEFD */
  0x0000,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0016,   /* U+FE19 */
  0x0000,   /* U+FE1A */
  0x0000,   /* U+FE1B */
  0x0000,   /* U+FE1C */
  0x0000,   /* U+FE1D */
  0x0000,   /* U+FE1E */
  0x0000,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0016,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0x0000,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0x0000,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0x0000,   /* U+FE6C */
  0x0000,   /* U+FE6D */
  0x0000,   /* U+FE6E */
  0x0000,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0x0000,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x001A,   /* U+FEF5 */
  0x0019,   /* U+FEF6 */
  0x001A,   /* U+FEF7 */
  0x0019,   /* U+FEF8 */
  0x001A,   /* U+FEF9 */
  0x0019,   /* U+FEFA */
  0x001A,   /* U+FEFB */
  0x0019,   /* U+FEFC */
  0x0000,   /* U+FEFD */
  0x0000,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0277,   /* U+FE19 */
  0x0000,   /* U+FE1A */
  0x0000,   /* U+FE1B */
  0x0000,   /* U+FE1C */
  0x0000,   /* U+FE1D */
  0x0000,   /* U+FE1E */
  0x0000,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0000,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0x0000,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0x0000,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0x0000,   /* U+FE6C */
  0x0000,   /* U+FE6D */
  0x0000,   /* U+FE6E */
  0x0000,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0x0000,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x0000,   /* U+FEF5 */
  0x0000,   /* U+FEF6 */
  0x0000,   /* U+FEF7 */
  0x0000,   /* U+FEF8 */
  0x0000,   /* U+FEF9 */
  0x0000,   /* U+FEFA */
  0x0000,   /* U+FEFB */
  0x0000,   /* U+FEFC */
  0x0000,   /* U+FEFD */
  0x0000,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0020,   /* U+FE19 */
  0x0000,   /* U+FE1A */
  0x0000,   /* U+FE1B */
  0x0000,   /* U+FE1C */
  0x0000,   /* U+FE1D */
  0x0000,   /* U+FE1E */
  0x0000,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0000,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0x0000,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0x0000,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0x0000,   /* U+FE6C */
  0x0000,   /* U+FE6D */
  0x0000,   /* U+FE6E */
  0x0000,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0x0000,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x0000,   /* U+FEF5 */
  0x0000,   /* U+FEF6 */
  0x0000,   /* U+FEF7 */
  0x0000,   /* U+FEF8 */
  0x0000,   /* U+FEF9 */
  0x0000,   /* U+FEFA */
  0x0000,   /* U+FEFB */
  0x0000,   /* U+FEFC */
  0x0000,   /* U+FEFD */
  0x0000,   /* U+FEFE */
  0x0000,   /* U+FEFF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+FE00 */
  0x0000,   /* U+FE01 */
  0x0000,   /* U+FE02 */
  0x0000,   /* U+FE03 */
  0x0000,   /* U+FE04 */
  0x0000,   /* U+FE05 */
  0x0000,   /* U+FE06 */
  0x0000,   /* U+FE07 */
  0x0000,   /* U+FE08 */
  0x0000,   /* U+FE09 */
  0x0000,   /* U+FE0A */
  0x0000,   /* U+FE0B */
  0x0000,   /* U+FE0C */
  0x0000,   /* U+FE0D */
  0x0000,   /* U+FE0E */
  0x0000,   /* U+FE0F */
  0x0000,   /* U+FE10 */
  0x0000,   /* U+FE11 */
  0x0000,   /* U+FE12 */
  0x0000,   /* U+FE13 */
  0x0000,   /* U+FE14 */
  0x0000,   /* U+FE15 */
  0x0000,   /* U+FE16 */
  0x0000,   /* U+FE17 */
  0x0000,   /* U+FE18 */
  0x0016,   /* U+FE19 */
  0x0000,   /* U+FE1A */
  0x0000,   /* U+FE1B */
  0x0000,   /* U+FE1C */
  0x0000,   /* U+FE1D */
  0x0000,   /* U+FE1E */
  0x0000,   /* U+FE1F */
  0x0000,   /* U+FE20 */
  0x0000,   /* U+FE21 */
  0x0000,   /* U+FE22 */
  0x0000,   /* U+FE23 */
  0x0000,   /* U+FE24 */
  0x0000,   /* U+FE25 */
  0x0000,   /* U+FE26 */
  0x0000,   /* U+FE27 */
  0x0000,   /* U+FE28 */
  0x0000,   /* U+FE29 */
  0x0000,   /* U+FE2A */
  0x0000,   /* U+FE2B */
  0x0000,   /* U+FE2C */
  0x0000,   /* U+FE2D */
  0x0000,   /* U+FE2E */
  0x0000,   /* U+FE2F */
  0x0000,   /* U+FE30 */
  0x0000,   /* U+FE31 */
  0x0000,   /* U+FE32 */
  0x0000,   /* U+FE33 */
  0x0000,   /* U+FE34 */
  0x0000,   /* U+FE35 */
  0x0000,   /* U+FE36 */
  0x0000,   /* U+FE37 */
  0x0000,   /* U+FE38 */
  0x0000,   /* U+FE39 */
  0x0000,   /* U+FE3A */
  0x0000,   /* U+FE3B */
  0x0000,   /* U+FE3C */
  0x0000,   /* U+FE3D */
  0x0000,   /* U+FE3E */
  0x0000,   /* U+FE3F */
  0x0000,   /* U+FE40 */
  0x0000,   /* U+FE41 */
  0x0000,   /* U+FE42 */
  0x0000,   /* U+FE43 */
  0x0000,   /* U+FE44 */
  0x0000,   /* U+FE45 */
  0x0000,   /* U+FE46 */
  0x0000,   /* U+FE47 */
  0x0000,   /* U+FE48 */
  0x0000,   /* U+FE49 */
  0x0000,   /* U+FE4A */
  0x0000,   /* U+FE4B */
  0x0000,   /* U+FE4C */
  0x0000,   /* U+FE4D */
  0x0000,   /* U+FE4E */
  0x0000,   /* U+FE4F */
  0x0000,   /* U+FE50 */
  0x0000,   /* U+FE51 */
  0x0000,   /* U+FE52 */
  0x0000,   /* U+FE53 */
  0x0000,   /* U+FE54 */
  0x0000,   /* U+FE55 */
  0x0000,   /* U+FE56 */
  0x0000,   /* U+FE57 */
  0x0000,   /* U+FE58 */
  0x0000,   /* U+FE59 */
  0x0000,   /* U+FE5A */
  0x0000,   /* U+FE5B */
  0x0000,   /* U+FE5C */
  0x0000,   /* U+FE5D */
  0x0000,   /* U+FE5E */
  0x0000,   /* U+FE5F */
  0x0000,   /* U+FE60 */
  0x0000,   /* U+FE61 */
  0x0000,   /* U+FE62 */
  0x0000,   /* U+FE63 */
  0x0000,   /* U+FE64 */
  0x0000,   /* U+FE65 */
  0x0000,   /* U+FE66 */
  0x0000,   /* U+FE67 */
  0x0000,   /* U+FE68 */
  0x0000,   /* U+FE69 */
  0x0000,   /* U+FE6A */
  0x0000,   /* U+FE6B */
  0x0000,   /* U+FE6C */
  0x0000,   /* U+FE6D */
  0x0000,   /* U+FE6E */
  0x0000,   /* U+FE6F */
  0x0000,   /* U+FE70 */
  0x0000,   /* U+FE71 */
  0x0000,   /* U+FE72 */
  0x0000,   /* U+FE73 */
  0x0000,   /* U+FE74 */
  0x0000,   /* U+FE75 */
  0x0000,   /* U+FE76 */
  0x0000,   /* U+FE77 */
  0x0000,   /* U+FE78 */
  0x0000,   /* U+FE79 */
  0x0000,   /* U+FE7A */
  0x0000,   /* U+FE7B */
  0x0000,   /* U+FE7C */
  0x0000,   /* U+FE7D */
  0x0000,   /* U+FE7E */
  0x0000,   /* U+FE7F */
  0x0000,   /* U+FE80 */
  0x0000,   /* U+FE81 */
  0x0000,   /* U+FE82 */
  0x0000,   /* U+FE83 */
  0x0000,   /* U+FE84 */
  0x0000,   /* U+FE85 */
  0x0000,   /* U+FE86 */
  0x0000,   /* U+FE87 */
  0x0000,   /* U+FE88 */
  0x0000,   /* U+FE89 */
  0x0000,   /* U+FE8A */
  0x0000,   /* U+FE8B */
  0x0000,   /* U+FE8C */
  0x0000,   /* U+FE8D */
  0x0000,   /* U+FE8E */
  0x0000,   /* U+FE8F */
  0x0000,   /* U+FE90 */
  0x0000,   /* U+FE91 */
  0x0000,   /* U+FE92 */
  0x0000,   /* U+FE93 */
  0x0000,   /* U+FE94 */
  0x0000,   /* U+FE95 */
  0x0000,   /* U+FE96 */
  0x0000,   /* U+FE97 */
  0x0000,   /* U+FE98 */
  0x0000,   /* U+FE99 */
  0x0000,   /* U+FE9A */
  0x0000,   /* U+FE9B */
  0x0000,   /* U+FE9C */
  0x0000,   /* U+FE9D */
  0x0000,   /* U+FE9E */
  0x0000,   /* U+FE9F */
  0x0000,   /* U+FEA0 */
  0x0000,   /* U+FEA1 */
  0x0000,   /* U+FEA2 */
  0x0000,   /* U+FEA3 */
  0x0000,   /* U+FEA4 */
  0x0000,   /* U+FEA5 */
  0x0000,   /* U+FEA6 */
  0x0000,   /* U+FEA7 */
  0x0000,   /* U+FEA8 */
  0x0000,   /* U+FEA9 */
  0x0000,   /* U+FEAA */
  0x0000,   /* U+FEAB */
  0x0000,   /* U+FEAC */
  0x0000,   /* U+FEAD */
  0x0000,   /* U+FEAE */
  0x0000,   /* U+FEAF */
  0x0000,   /* U+FEB0 */
  0x0000,   /* U+FEB1 */
  0x0000,   /* U+FEB2 */
  0x0000,   /* U+FEB3 */
  0x0000,   /* U+FEB4 */
  0x0000,   /* U+FEB5 */
  0x0000,   /* U+FEB6 */
  0x0000,   /* U+FEB7 */
  0x0000,   /* U+FEB8 */
  0x0000,   /* U+FEB9 */
  0x0000,   /* U+FEBA */
  0x0000,   /* U+FEBB */
  0x0000,   /* U+FEBC */
  0x0000,   /* U+FEBD */
  0x0000,   /* U+FEBE */
  0x0000,   /* U+FEBF */
  0x0000,   /* U+FEC0 */
  0x0000,   /* U+FEC1 */
  0x0000,   /* U+FEC2 */
  0x0000,   /* U+FEC3 */
  0x0000,   /* U+FEC4 */
  0x0000,   /* U+FEC5 */
  0x0000,   /* U+FEC6 */
  0x0000,   /* U+FEC7 */
  0x0000,   /* U+FEC8 */
  0x0000,   /* U+FEC9 */
  0x0000,   /* U+FECA */
  0x0000,   /* U+FECB */
  0x0000,   /* U+FECC */
  0x0000,   /* U+FECD */
  0x0000,   /* U+FECE */
  0x0000,   /* U+FECF */
  0x0000,   /* U+FED0 */
  0x0000,   /* U+FED1 */
  0x0000,   /* U+FED2 */
  0x0000,   /* U+FED3 */
  0x0000,   /* U+FED4 */
  0x0000,   /* U+FED5 */
  0x0000,   /* U+FED6 */
  0x0000,   /* U+FED7 */
  0x0000,   /* U+FED8 */
  0x0000,   /* U+FED9 */
  0x0000,   /* U+FEDA */
  0x0000,   /* U+FEDB */
  0x0000,   /* U+FEDC */
  0x0000,   /* U+FEDD */
  0x0000,   /* U+FEDE */
  0x0000,   /* U+FEDF */
  0x0000,   /* U+FEE0 */
  0x0000,   /* U+FEE1 */
  0x0000,   /* U+FEE2 */
  0x0000,   /* U+FEE3 */
  0x0000,   /* U+FEE4 */
  0x0000,   /* U+FEE5 */
  0x0000,   /* U+FEE6 */
  0x0000,   /* U+FEE7 */
  0x0000,   /* U+FEE8 */
  0x0000,   /* U+FEE9 */
  0x0000,   /* U+FEEA */
  0x0000,   /* U+FEEB */
  0x0000,   /* U+FEEC */
  0x0000,   /* U+FEED */
  0x0000,   /* U+FEEE */
  0x0000,   /* U+FEEF */
  0x0000,   /* U+FEF0 */
  0x0000,   /* U+FEF1 */
  0x0000,   /* U+FEF2 */
  0x0000,   /* U+FEF3 */
  0x0000,   /* U+FEF4 */
  0x0000,   /* U+FEF5 */
  0x0000,   /* U+FEF6 */
  0x0000,   /* U+FEF7 */
  0x0000,   /* U+FEF8 */
  0x0000,   /* U+FEF9 */
  0x0000,   /* U+FEFA */
  0x0000,   /* U+FEFB */
  0x0000,   /* U+FEFC */
  0x0000,   /* U+FEFD */
  0x0000,   /* U+FEFE */
  0x0000,   /* U+FEFF */
};

uint16 uca900_p0FF[]= {
  /* Number of CEs for each character. */
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC1,   /* U+FF00 */
  0x0260,   /* U+FF01 */
  0x030C,   /* U+FF02 */
  0x0398,   /* U+FF03 */
  0x1C12,   /* U+FF04 */
  0x0399,   /* U+FF05 */
  0x0396,   /* U+FF06 */
  0x0305,   /* U+FF07 */
  0x0317,   /* U+FF08 */
  0x0318,   /* U+FF09 */
  0x038F,   /* U+FF0A */
  0x0616,   /* U+FF0B */
  0x0222,   /* U+FF0C */
  0x020D,   /* U+FF0D */
  0x0277,   /* U+FF0E */
  0x0394,   /* U+FF0F */
  0x1C3D,   /* U+FF10 */
  0x1C3E,   /* U+FF11 */
  0x1C3F,   /* U+FF12 */
  0x1C40,   /* U+FF13 */
  0x1C41,   /* U+FF14 */
  0x1C42,   /* U+FF15 */
  0x1C43,   /* U+FF16 */
  0x1C44,   /* U+FF17 */
  0x1C45,   /* U+FF18 */
  0x1C46,   /* U+FF19 */
  0x0239,   /* U+FF1A */
  0x0234,   /* U+FF1B */
  0x061A,   /* U+FF1C */
  0x061B,   /* U+FF1D */
  0x061C,   /* U+FF1E */
  0x0266,   /* U+FF1F */
  0x038E,   /* U+FF20 */
  0x1C47,   /* U+FF21 */
  0x1C60,   /* U+FF22 */
  0x1C7A,   /* U+FF23 */
  0x1C8F,   /* U+FF24 */
  0x1CAA,   /* U+FF25 */
  0x1CE5,   /* U+FF26 */
  0x1CF4,   /* U+FF27 */
  0x1D18,   /* U+FF28 */
  0x1D32,   /* U+FF29 */
  0x1D4C,   /* U+FF2A */
  0x1D65,   /* U+FF2B */
  0x1D77,   /* U+FF2C */
  0x1DAA,   /* U+FF2D */
  0x1DB9,   /* U+FF2E */
  0x1DDD,   /* U+FF2F */
  0x1E0C,   /* U+FF30 */
  0x1E21,   /* U+FF31 */
  0x1E33,   /* U+FF32 */
  0x1E71,   /* U+FF33 */
  0x1E95,   /* U+FF34 */
  0x1EB5,   /* U+FF35 */
  0x1EE3,   /* U+FF36 */
  0x1EF5,   /* U+FF37 */
  0x1EFF,   /* U+FF38 */
  0x1F0B,   /* U+FF39 */
  0x1F21,   /* U+FF3A */
  0x0319,   /* U+FF3B */
  0x0395,   /* U+FF3C */
  0x031A,   /* U+FF3D */
  0x0485,   /* U+FF3E */
  0x020B,   /* U+FF3F */
  0x0482,   /* U+FF40 */
  0x1C47,   /* U+FF41 */
  0x1C60,   /* U+FF42 */
  0x1C7A,   /* U+FF43 */
  0x1C8F,   /* U+FF44 */
  0x1CAA,   /* U+FF45 */
  0x1CE5,   /* U+FF46 */
  0x1CF4,   /* U+FF47 */
  0x1D18,   /* U+FF48 */
  0x1D32,   /* U+FF49 */
  0x1D4C,   /* U+FF4A */
  0x1D65,   /* U+FF4B */
  0x1D77,   /* U+FF4C */
  0x1DAA,   /* U+FF4D */
  0x1DB9,   /* U+FF4E */
  0x1DDD,   /* U+FF4F */
  0x1E0C,   /* U+FF50 */
  0x1E21,   /* U+FF51 */
  0x1E33,   /* U+FF52 */
  0x1E71,   /* U+FF53 */
  0x1E95,   /* U+FF54 */
  0x1EB5,   /* U+FF55 */
  0x1EE3,   /* U+FF56 */
  0x1EF5,   /* U+FF57 */
  0x1EFF,   /* U+FF58 */
  0x1F0B,   /* U+FF59 */
  0x1F21,   /* U+FF5A */
  0x031B,   /* U+FF5B */
  0x061E,   /* U+FF5C */
  0x031C,   /* U+FF5D */
  0x0620,   /* U+FF5E */
  0x032D,   /* U+FF5F */
  0x032E,   /* U+FF60 */
  0x028A,   /* U+FF61 */
  0x0373,   /* U+FF62 */
  0x0374,   /* U+FF63 */
  0x0231,   /* U+FF64 */
  0x0221,   /* U+FF65 */
  0x3D8A,   /* U+FF66 */
  0x3D5A,   /* U+FF67 */
  0x3D5B,   /* U+FF68 */
  0x3D5C,   /* U+FF69 */
  0x3D5E,   /* U+FF6A */
  0x3D5F,   /* U+FF6B */
  0x3D7E,   /* U+FF6C */
  0x3D7F,   /* U+FF6D */
  0x3D81,   /* U+FF6E */
  0x3D6C,   /* U+FF6F */
  0x1C0E,   /* U+FF70 */
  0x3D5A,   /* U+FF71 */
  0x3D5B,   /* U+FF72 */
  0x3D5C,   /* U+FF73 */
  0x3D5E,   /* U+FF74 */
  0x3D5F,   /* U+FF75 */
  0x3D60,   /* U+FF76 */
  0x3D61,   /* U+FF77 */
  0x3D62,   /* U+FF78 */
  0x3D63,   /* U+FF79 */
  0x3D64,   /* U+FF7A */
  0x3D65,   /* U+FF7B */
  0x3D66,   /* U+FF7C */
  0x3D67,   /* U+FF7D */
  0x3D68,   /* U+FF7E */
  0x3D69,   /* U+FF7F */
  0x3D6A,   /* U+FF80 */
  0x3D6B,   /* U+FF81 */
  0x3D6C,   /* U+FF82 */
  0x3D6D,   /* U+FF83 */
  0x3D6E,   /* U+FF84 */
  0x3D6F,   /* U+FF85 */
  0x3D70,   /* U+FF86 */
  0x3D71,   /* U+FF87 */
  0x3D72,   /* U+FF88 */
  0x3D73,   /* U+FF89 */
  0x3D74,   /* U+FF8A */
  0x3D75,   /* U+FF8B */
  0x3D76,   /* U+FF8C */
  0x3D77,   /* U+FF8D */
  0x3D78,   /* U+FF8E */
  0x3D79,   /* U+FF8F */
  0x3D7A,   /* U+FF90 */
  0x3D7B,   /* U+FF91 */
  0x3D7C,   /* U+FF92 */
  0x3D7D,   /* U+FF93 */
  0x3D7E,   /* U+FF94 */
  0x3D7F,   /* U+FF95 */
  0x3D81,   /* U+FF96 */
  0x3D82,   /* U+FF97 */
  0x3D83,   /* U+FF98 */
  0x3D84,   /* U+FF99 */
  0x3D85,   /* U+FF9A */
  0x3D86,   /* U+FF9B */
  0x3D87,   /* U+FF9C */
  0x3D8B,   /* U+FF9D */
  0x0000,   /* U+FF9E */
  0x0000,   /* U+FF9F */
  0x3C72,   /* U+FFA0 */
  0x3BF5,   /* U+FFA1 */
  0x3BF6,   /* U+FFA2 */
  0x3CD3,   /* U+FFA3 */
  0x3BF7,   /* U+FFA4 */
  0x3CD5,   /* U+FFA5 */
  0x3CD6,   /* U+FFA6 */
  0x3BF8,   /* U+FFA7 */
  0x3BF9,   /* U+FFA8 */
  0x3BFA,   /* U+FFA9 */
  0x3CD9,   /* U+FFAA */
  0x3CDA,   /* U+FFAB */
  0x3CDB,   /* U+FFAC */
  0x3CDC,   /* U+FFAD */
  0x3CDD,   /* U+FFAE */
  0x3CDE,   /* U+FFAF */
  0x3C0F,   /* U+FFB0 */
  0x3BFB,   /* U+FFB1 */
  0x3BFC,   /* U+FFB2 */
  0x3BFD,   /* U+FFB3 */
  0x3C16,   /* U+FFB4 */
  0x3BFE,   /* U+FFB5 */
  0x3BFF,   /* U+FFB6 */
  0x3C00,   /* U+FFB7 */
  0x3C01,   /* U+FFB8 */
  0x3C02,   /* U+FFB9 */
  0x3C03,   /* U+FFBA */
  0x3C04,   /* U+FFBB */
  0x3C05,   /* U+FFBC */
  0x3C06,   /* U+FFBD */
  0x3C07,   /* U+FFBE */
  0xFBC1,   /* U+FFBF */
  0xFBC1,   /* U+FFC0 */
  0xFBC1,   /* U+FFC1 */
  0x3C73,   /* U+FFC2 */
  0x3C74,   /* U+FFC3 */
  0x3C75,   /* U+FFC4 */
  0x3C76,   /* U+FFC5 */
  0x3C77,   /* U+FFC6 */
  0x3C78,   /* U+FFC7 */
  0xFBC1,   /* U+FFC8 */
  0xFBC1,   /* U+FFC9 */
  0x3C79,   /* U+FFCA */
  0x3C7A,   /* U+FFCB */
  0x3C7B,   /* U+FFCC */
  0x3C7C,   /* U+FFCD */
  0x3C7D,   /* U+FFCE */
  0x3C7E,   /* U+FFCF */
  0xFBC1,   /* U+FFD0 */
  0xFBC1,   /* U+FFD1 */
  0x3C7F,   /* U+FFD2 */
  0x3C80,   /* U+FFD3 */
  0x3C81,   /* U+FFD4 */
  0x3C82,   /* U+FFD5 */
  0x3C83,   /* U+FFD6 */
  0x3C84,   /* U+FFD7 */
  0xFBC1,   /* U+FFD8 */
  0xFBC1,   /* U+FFD9 */
  0x3C85,   /* U+FFDA */
  0x3C86,   /* U+FFDB */
  0x3C87,   /* U+FFDC */
  0xFBC1,   /* U+FFDD */
  0xFBC1,   /* U+FFDE */
  0xFBC1,   /* U+FFDF */
  0x1C11,   /* U+FFE0 */
  0x1C13,   /* U+FFE1 */
  0x061D,   /* U+FFE2 */
  0x0486,   /* U+FFE3 */
  0x061F,   /* U+FFE4 */
  0x1C14,   /* U+FFE5 */
  0x1C27,   /* U+FFE6 */
  0xFBC1,   /* U+FFE7 */
  0x081A,   /* U+FFE8 */
  0x059C,   /* U+FFE9 */
  0x059E,   /* U+FFEA */
  0x059D,   /* U+FFEB */
  0x059F,   /* U+FFEC */
  0x08B8,   /* U+FFED */
  0x08E3,   /* U+FFEE */
  0xFBC1,   /* U+FFEF */
  0xFBC1,   /* U+FFF0 */
  0xFBC1,   /* U+FFF1 */
  0xFBC1,   /* U+FFF2 */
  0xFBC1,   /* U+FFF3 */
  0xFBC1,   /* U+FFF4 */
  0xFBC1,   /* U+FFF5 */
  0xFBC1,   /* U+FFF6 */
  0xFBC1,   /* U+FFF7 */
  0xFBC1,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x1A95,   /* U+FFFC */
  0xFFFD,   /* U+FFFD */
  0xFBC1,   /* U+FFFE */
  0xFBC1,   /* U+FFFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+FF00 */
  0x0020,   /* U+FF01 */
  0x0020,   /* U+FF02 */
  0x0020,   /* U+FF03 */
  0x0020,   /* U+FF04 */
  0x0020,   /* U+FF05 */
  0x0020,   /* U+FF06 */
  0x0020,   /* U+FF07 */
  0x0020,   /* U+FF08 */
  0x0020,   /* U+FF09 */
  0x0020,   /* U+FF0A */
  0x0020,   /* U+FF0B */
  0x0020,   /* U+FF0C */
  0x0020,   /* U+FF0D */
  0x0020,   /* U+FF0E */
  0x0020,   /* U+FF0F */
  0x0020,   /* U+FF10 */
  0x0020,   /* U+FF11 */
  0x0020,   /* U+FF12 */
  0x0020,   /* U+FF13 */
  0x0020,   /* U+FF14 */
  0x0020,   /* U+FF15 */
  0x0020,   /* U+FF16 */
  0x0020,   /* U+FF17 */
  0x0020,   /* U+FF18 */
  0x0020,   /* U+FF19 */
  0x0020,   /* U+FF1A */
  0x0020,   /* U+FF1B */
  0x0020,   /* U+FF1C */
  0x0020,   /* U+FF1D */
  0x0020,   /* U+FF1E */
  0x0020,   /* U+FF1F */
  0x0020,   /* U+FF20 */
  0x0020,   /* U+FF21 */
  0x0020,   /* U+FF22 */
  0x0020,   /* U+FF23 */
  0x0020,   /* U+FF24 */
  0x0020,   /* U+FF25 */
  0x0020,   /* U+FF26 */
  0x0020,   /* U+FF27 */
  0x0020,   /* U+FF28 */
  0x0020,   /* U+FF29 */
  0x0020,   /* U+FF2A */
  0x0020,   /* U+FF2B */
  0x0020,   /* U+FF2C */
  0x0020,   /* U+FF2D */
  0x0020,   /* U+FF2E */
  0x0020,   /* U+FF2F */
  0x0020,   /* U+FF30 */
  0x0020,   /* U+FF31 */
  0x0020,   /* U+FF32 */
  0x0020,   /* U+FF33 */
  0x0020,   /* U+FF34 */
  0x0020,   /* U+FF35 */
  0x0020,   /* U+FF36 */
  0x0020,   /* U+FF37 */
  0x0020,   /* U+FF38 */
  0x0020,   /* U+FF39 */
  0x0020,   /* U+FF3A */
  0x0020,   /* U+FF3B */
  0x0020,   /* U+FF3C */
  0x0020,   /* U+FF3D */
  0x0020,   /* U+FF3E */
  0x0020,   /* U+FF3F */
  0x0020,   /* U+FF40 */
  0x0020,   /* U+FF41 */
  0x0020,   /* U+FF42 */
  0x0020,   /* U+FF43 */
  0x0020,   /* U+FF44 */
  0x0020,   /* U+FF45 */
  0x0020,   /* U+FF46 */
  0x0020,   /* U+FF47 */
  0x0020,   /* U+FF48 */
  0x0020,   /* U+FF49 */
  0x0020,   /* U+FF4A */
  0x0020,   /* U+FF4B */
  0x0020,   /* U+FF4C */
  0x0020,   /* U+FF4D */
  0x0020,   /* U+FF4E */
  0x0020,   /* U+FF4F */
  0x0020,   /* U+FF50 */
  0x0020,   /* U+FF51 */
  0x0020,   /* U+FF52 */
  0x0020,   /* U+FF53 */
  0x0020,   /* U+FF54 */
  0x0020,   /* U+FF55 */
  0x0020,   /* U+FF56 */
  0x0020,   /* U+FF57 */
  0x0020,   /* U+FF58 */
  0x0020,   /* U+FF59 */
  0x0020,   /* U+FF5A */
  0x0020,   /* U+FF5B */
  0x0020,   /* U+FF5C */
  0x0020,   /* U+FF5D */
  0x0020,   /* U+FF5E */
  0x0020,   /* U+FF5F */
  0x0020,   /* U+FF60 */
  0x0020,   /* U+FF61 */
  0x0020,   /* U+FF62 */
  0x0020,   /* U+FF63 */
  0x0020,   /* U+FF64 */
  0x0020,   /* U+FF65 */
  0x0020,   /* U+FF66 */
  0x0020,   /* U+FF67 */
  0x0020,   /* U+FF68 */
  0x0020,   /* U+FF69 */
  0x0020,   /* U+FF6A */
  0x0020,   /* U+FF6B */
  0x0020,   /* U+FF6C */
  0x0020,   /* U+FF6D */
  0x0020,   /* U+FF6E */
  0x0020,   /* U+FF6F */
  0x0020,   /* U+FF70 */
  0x0020,   /* U+FF71 */
  0x0020,   /* U+FF72 */
  0x0020,   /* U+FF73 */
  0x0020,   /* U+FF74 */
  0x0020,   /* U+FF75 */
  0x0020,   /* U+FF76 */
  0x0020,   /* U+FF77 */
  0x0020,   /* U+FF78 */
  0x0020,   /* U+FF79 */
  0x0020,   /* U+FF7A */
  0x0020,   /* U+FF7B */
  0x0020,   /* U+FF7C */
  0x0020,   /* U+FF7D */
  0x0020,   /* U+FF7E */
  0x0020,   /* U+FF7F */
  0x0020,   /* U+FF80 */
  0x0020,   /* U+FF81 */
  0x0020,   /* U+FF82 */
  0x0020,   /* U+FF83 */
  0x0020,   /* U+FF84 */
  0x0020,   /* U+FF85 */
  0x0020,   /* U+FF86 */
  0x0020,   /* U+FF87 */
  0x0020,   /* U+FF88 */
  0x0020,   /* U+FF89 */
  0x0020,   /* U+FF8A */
  0x0020,   /* U+FF8B */
  0x0020,   /* U+FF8C */
  0x0020,   /* U+FF8D */
  0x0020,   /* U+FF8E */
  0x0020,   /* U+FF8F */
  0x0020,   /* U+FF90 */
  0x0020,   /* U+FF91 */
  0x0020,   /* U+FF92 */
  0x0020,   /* U+FF93 */
  0x0020,   /* U+FF94 */
  0x0020,   /* U+FF95 */
  0x0020,   /* U+FF96 */
  0x0020,   /* U+FF97 */
  0x0020,   /* U+FF98 */
  0x0020,   /* U+FF99 */
  0x0020,   /* U+FF9A */
  0x0020,   /* U+FF9B */
  0x0020,   /* U+FF9C */
  0x0020,   /* U+FF9D */
  0x0037,   /* U+FF9E */
  0x0038,   /* U+FF9F */
  0x0020,   /* U+FFA0 */
  0x0020,   /* U+FFA1 */
  0x0020,   /* U+FFA2 */
  0x0020,   /* U+FFA3 */
  0x0020,   /* U+FFA4 */
  0x0020,   /* U+FFA5 */
  0x0020,   /* U+FFA6 */
  0x0020,   /* U+FFA7 */
  0x0020,   /* U+FFA8 */
  0x0020,   /* U+FFA9 */
  0x0020,   /* U+FFAA */
  0x0020,   /* U+FFAB */
  0x0020,   /* U+FFAC */
  0x0020,   /* U+FFAD */
  0x0020,   /* U+FFAE */
  0x0020,   /* U+FFAF */
  0x0020,   /* U+FFB0 */
  0x0020,   /* U+FFB1 */
  0x0020,   /* U+FFB2 */
  0x0020,   /* U+FFB3 */
  0x0020,   /* U+FFB4 */
  0x0020,   /* U+FFB5 */
  0x0020,   /* U+FFB6 */
  0x0020,   /* U+FFB7 */
  0x0020,   /* U+FFB8 */
  0x0020,   /* U+FFB9 */
  0x0020,   /* U+FFBA */
  0x0020,   /* U+FFBB */
  0x0020,   /* U+FFBC */
  0x0020,   /* U+FFBD */
  0x0020,   /* U+FFBE */
  0x0020,   /* U+FFBF */
  0x0020,   /* U+FFC0 */
  0x0020,   /* U+FFC1 */
  0x0020,   /* U+FFC2 */
  0x0020,   /* U+FFC3 */
  0x0020,   /* U+FFC4 */
  0x0020,   /* U+FFC5 */
  0x0020,   /* U+FFC6 */
  0x0020,   /* U+FFC7 */
  0x0020,   /* U+FFC8 */
  0x0020,   /* U+FFC9 */
  0x0020,   /* U+FFCA */
  0x0020,   /* U+FFCB */
  0x0020,   /* U+FFCC */
  0x0020,   /* U+FFCD */
  0x0020,   /* U+FFCE */
  0x0020,   /* U+FFCF */
  0x0020,   /* U+FFD0 */
  0x0020,   /* U+FFD1 */
  0x0020,   /* U+FFD2 */
  0x0020,   /* U+FFD3 */
  0x0020,   /* U+FFD4 */
  0x0020,   /* U+FFD5 */
  0x0020,   /* U+FFD6 */
  0x0020,   /* U+FFD7 */
  0x0020,   /* U+FFD8 */
  0x0020,   /* U+FFD9 */
  0x0020,   /* U+FFDA */
  0x0020,   /* U+FFDB */
  0x0020,   /* U+FFDC */
  0x0020,   /* U+FFDD */
  0x0020,   /* U+FFDE */
  0x0020,   /* U+FFDF */
  0x0020,   /* U+FFE0 */
  0x0020,   /* U+FFE1 */
  0x0020,   /* U+FFE2 */
  0x0020,   /* U+FFE3 */
  0x0020,   /* U+FFE4 */
  0x0020,   /* U+FFE5 */
  0x0020,   /* U+FFE6 */
  0x0020,   /* U+FFE7 */
  0x0020,   /* U+FFE8 */
  0x0020,   /* U+FFE9 */
  0x0020,   /* U+FFEA */
  0x0020,   /* U+FFEB */
  0x0020,   /* U+FFEC */
  0x0020,   /* U+FFED */
  0x0020,   /* U+FFEE */
  0x0020,   /* U+FFEF */
  0x0020,   /* U+FFF0 */
  0x0020,   /* U+FFF1 */
  0x0020,   /* U+FFF2 */
  0x0020,   /* U+FFF3 */
  0x0020,   /* U+FFF4 */
  0x0020,   /* U+FFF5 */
  0x0020,   /* U+FFF6 */
  0x0020,   /* U+FFF7 */
  0x0020,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x0020,   /* U+FFFC */
  0x0020,   /* U+FFFD */
  0x0020,   /* U+FFFE */
  0x0020,   /* U+FFFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+FF00 */
  0x0003,   /* U+FF01 */
  0x0003,   /* U+FF02 */
  0x0003,   /* U+FF03 */
  0x0003,   /* U+FF04 */
  0x0003,   /* U+FF05 */
  0x0003,   /* U+FF06 */
  0x0003,   /* U+FF07 */
  0x0003,   /* U+FF08 */
  0x0003,   /* U+FF09 */
  0x0003,   /* U+FF0A */
  0x0003,   /* U+FF0B */
  0x0003,   /* U+FF0C */
  0x0003,   /* U+FF0D */
  0x0003,   /* U+FF0E */
  0x0003,   /* U+FF0F */
  0x0003,   /* U+FF10 */
  0x0003,   /* U+FF11 */
  0x0003,   /* U+FF12 */
  0x0003,   /* U+FF13 */
  0x0003,   /* U+FF14 */
  0x0003,   /* U+FF15 */
  0x0003,   /* U+FF16 */
  0x0003,   /* U+FF17 */
  0x0003,   /* U+FF18 */
  0x0003,   /* U+FF19 */
  0x0003,   /* U+FF1A */
  0x0003,   /* U+FF1B */
  0x0003,   /* U+FF1C */
  0x0003,   /* U+FF1D */
  0x0003,   /* U+FF1E */
  0x0003,   /* U+FF1F */
  0x0003,   /* U+FF20 */
  0x0009,   /* U+FF21 */
  0x0009,   /* U+FF22 */
  0x0009,   /* U+FF23 */
  0x0009,   /* U+FF24 */
  0x0009,   /* U+FF25 */
  0x0009,   /* U+FF26 */
  0x0009,   /* U+FF27 */
  0x0009,   /* U+FF28 */
  0x0009,   /* U+FF29 */
  0x0009,   /* U+FF2A */
  0x0009,   /* U+FF2B */
  0x0009,   /* U+FF2C */
  0x0009,   /* U+FF2D */
  0x0009,   /* U+FF2E */
  0x0009,   /* U+FF2F */
  0x0009,   /* U+FF30 */
  0x0009,   /* U+FF31 */
  0x0009,   /* U+FF32 */
  0x0009,   /* U+FF33 */
  0x0009,   /* U+FF34 */
  0x0009,   /* U+FF35 */
  0x0009,   /* U+FF36 */
  0x0009,   /* U+FF37 */
  0x0009,   /* U+FF38 */
  0x0009,   /* U+FF39 */
  0x0009,   /* U+FF3A */
  0x0003,   /* U+FF3B */
  0x0003,   /* U+FF3C */
  0x0003,   /* U+FF3D */
  0x0003,   /* U+FF3E */
  0x0003,   /* U+FF3F */
  0x0003,   /* U+FF40 */
  0x0003,   /* U+FF41 */
  0x0003,   /* U+FF42 */
  0x0003,   /* U+FF43 */
  0x0003,   /* U+FF44 */
  0x0003,   /* U+FF45 */
  0x0003,   /* U+FF46 */
  0x0003,   /* U+FF47 */
  0x0003,   /* U+FF48 */
  0x0003,   /* U+FF49 */
  0x0003,   /* U+FF4A */
  0x0003,   /* U+FF4B */
  0x0003,   /* U+FF4C */
  0x0003,   /* U+FF4D */
  0x0003,   /* U+FF4E */
  0x0003,   /* U+FF4F */
  0x0003,   /* U+FF50 */
  0x0003,   /* U+FF51 */
  0x0003,   /* U+FF52 */
  0x0003,   /* U+FF53 */
  0x0003,   /* U+FF54 */
  0x0003,   /* U+FF55 */
  0x0003,   /* U+FF56 */
  0x0003,   /* U+FF57 */
  0x0003,   /* U+FF58 */
  0x0003,   /* U+FF59 */
  0x0003,   /* U+FF5A */
  0x0003,   /* U+FF5B */
  0x0003,   /* U+FF5C */
  0x0003,   /* U+FF5D */
  0x0003,   /* U+FF5E */
  0x0003,   /* U+FF5F */
  0x0003,   /* U+FF60 */
  0x0012,   /* U+FF61 */
  0x0012,   /* U+FF62 */
  0x0012,   /* U+FF63 */
  0x0012,   /* U+FF64 */
  0x0012,   /* U+FF65 */
  0x0012,   /* U+FF66 */
  0x0010,   /* U+FF67 */
  0x0010,   /* U+FF68 */
  0x0010,   /* U+FF69 */
  0x0010,   /* U+FF6A */
  0x0010,   /* U+FF6B */
  0x0010,   /* U+FF6C */
  0x0010,   /* U+FF6D */
  0x0010,   /* U+FF6E */
  0x0010,   /* U+FF6F */
  0x0012,   /* U+FF70 */
  0x0012,   /* U+FF71 */
  0x0012,   /* U+FF72 */
  0x0012,   /* U+FF73 */
  0x0012,   /* U+FF74 */
  0x0012,   /* U+FF75 */
  0x0012,   /* U+FF76 */
  0x0012,   /* U+FF77 */
  0x0012,   /* U+FF78 */
  0x0012,   /* U+FF79 */
  0x0012,   /* U+FF7A */
  0x0012,   /* U+FF7B */
  0x0012,   /* U+FF7C */
  0x0012,   /* U+FF7D */
  0x0012,   /* U+FF7E */
  0x0012,   /* U+FF7F */
  0x0012,   /* U+FF80 */
  0x0012,   /* U+FF81 */
  0x0012,   /* U+FF82 */
  0x0012,   /* U+FF83 */
  0x0012,   /* U+FF84 */
  0x0012,   /* U+FF85 */
  0x0012,   /* U+FF86 */
  0x0012,   /* U+FF87 */
  0x0012,   /* U+FF88 */
  0x0012,   /* U+FF89 */
  0x0012,   /* U+FF8A */
  0x0012,   /* U+FF8B */
  0x0012,   /* U+FF8C */
  0x0012,   /* U+FF8D */
  0x0012,   /* U+FF8E */
  0x0012,   /* U+FF8F */
  0x0012,   /* U+FF90 */
  0x0012,   /* U+FF91 */
  0x0012,   /* U+FF92 */
  0x0012,   /* U+FF93 */
  0x0012,   /* U+FF94 */
  0x0012,   /* U+FF95 */
  0x0012,   /* U+FF96 */
  0x0012,   /* U+FF97 */
  0x0012,   /* U+FF98 */
  0x0012,   /* U+FF99 */
  0x0012,   /* U+FF9A */
  0x0012,   /* U+FF9B */
  0x0012,   /* U+FF9C */
  0x0012,   /* U+FF9D */
  0x0012,   /* U+FF9E */
  0x0012,   /* U+FF9F */
  0x0012,   /* U+FFA0 */
  0x0012,   /* U+FFA1 */
  0x0012,   /* U+FFA2 */
  0x0012,   /* U+FFA3 */
  0x0012,   /* U+FFA4 */
  0x0012,   /* U+FFA5 */
  0x0012,   /* U+FFA6 */
  0x0012,   /* U+FFA7 */
  0x0012,   /* U+FFA8 */
  0x0012,   /* U+FFA9 */
  0x0012,   /* U+FFAA */
  0x0012,   /* U+FFAB */
  0x0012,   /* U+FFAC */
  0x0012,   /* U+FFAD */
  0x0012,   /* U+FFAE */
  0x0012,   /* U+FFAF */
  0x0012,   /* U+FFB0 */
  0x0012,   /* U+FFB1 */
  0x0012,   /* U+FFB2 */
  0x0012,   /* U+FFB3 */
  0x0012,   /* U+FFB4 */
  0x0012,   /* U+FFB5 */
  0x0012,   /* U+FFB6 */
  0x0012,   /* U+FFB7 */
  0x0012,   /* U+FFB8 */
  0x0012,   /* U+FFB9 */
  0x0012,   /* U+FFBA */
  0x0012,   /* U+FFBB */
  0x0012,   /* U+FFBC */
  0x0012,   /* U+FFBD */
  0x0012,   /* U+FFBE */
  0x0002,   /* U+FFBF */
  0x0002,   /* U+FFC0 */
  0x0002,   /* U+FFC1 */
  0x0012,   /* U+FFC2 */
  0x0012,   /* U+FFC3 */
  0x0012,   /* U+FFC4 */
  0x0012,   /* U+FFC5 */
  0x0012,   /* U+FFC6 */
  0x0012,   /* U+FFC7 */
  0x0002,   /* U+FFC8 */
  0x0002,   /* U+FFC9 */
  0x0012,   /* U+FFCA */
  0x0012,   /* U+FFCB */
  0x0012,   /* U+FFCC */
  0x0012,   /* U+FFCD */
  0x0012,   /* U+FFCE */
  0x0012,   /* U+FFCF */
  0x0002,   /* U+FFD0 */
  0x0002,   /* U+FFD1 */
  0x0012,   /* U+FFD2 */
  0x0012,   /* U+FFD3 */
  0x0012,   /* U+FFD4 */
  0x0012,   /* U+FFD5 */
  0x0012,   /* U+FFD6 */
  0x0012,   /* U+FFD7 */
  0x0002,   /* U+FFD8 */
  0x0002,   /* U+FFD9 */
  0x0012,   /* U+FFDA */
  0x0012,   /* U+FFDB */
  0x0012,   /* U+FFDC */
  0x0002,   /* U+FFDD */
  0x0002,   /* U+FFDE */
  0x0002,   /* U+FFDF */
  0x0003,   /* U+FFE0 */
  0x0003,   /* U+FFE1 */
  0x0003,   /* U+FFE2 */
  0x0003,   /* U+FFE3 */
  0x0003,   /* U+FFE4 */
  0x0003,   /* U+FFE5 */
  0x0003,   /* U+FFE6 */
  0x0002,   /* U+FFE7 */
  0x0012,   /* U+FFE8 */
  0x0012,   /* U+FFE9 */
  0x0012,   /* U+FFEA */
  0x0012,   /* U+FFEB */
  0x0012,   /* U+FFEC */
  0x0012,   /* U+FFED */
  0x0012,   /* U+FFEE */
  0x0002,   /* U+FFEF */
  0x0002,   /* U+FFF0 */
  0x0002,   /* U+FFF1 */
  0x0002,   /* U+FFF2 */
  0x0002,   /* U+FFF3 */
  0x0002,   /* U+FFF4 */
  0x0002,   /* U+FFF5 */
  0x0002,   /* U+FFF6 */
  0x0002,   /* U+FFF7 */
  0x0002,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x0002,   /* U+FFFC */
  0x0002,   /* U+FFFD */
  0x0002,   /* U+FFFE */
  0x0002,   /* U+FFFF */

  /* Primary weight 2 for each character. */
  0xFF00,   /* U+FF00 */
  0x0000,   /* U+FF01 */
  0x0000,   /* U+FF02 */
  0x0000,   /* U+FF03 */
  0x0000,   /* U+FF04 */
  0x0000,   /* U+FF05 */
  0x0000,   /* U+FF06 */
  0x0000,   /* U+FF07 */
  0x0000,   /* U+FF08 */
  0x0000,   /* U+FF09 */
  0x0000,   /* U+FF0A */
  0x0000,   /* U+FF0B */
  0x0000,   /* U+FF0C */
  0x0000,   /* U+FF0D */
  0x0000,   /* U+FF0E */
  0x0000,   /* U+FF0F */
  0x0000,   /* U+FF10 */
  0x0000,   /* U+FF11 */
  0x0000,   /* U+FF12 */
  0x0000,   /* U+FF13 */
  0x0000,   /* U+FF14 */
  0x0000,   /* U+FF15 */
  0x0000,   /* U+FF16 */
  0x0000,   /* U+FF17 */
  0x0000,   /* U+FF18 */
  0x0000,   /* U+FF19 */
  0x0000,   /* U+FF1A */
  0x0000,   /* U+FF1B */
  0x0000,   /* U+FF1C */
  0x0000,   /* U+FF1D */
  0x0000,   /* U+FF1E */
  0x0000,   /* U+FF1F */
  0x0000,   /* U+FF20 */
  0x0000,   /* U+FF21 */
  0x0000,   /* U+FF22 */
  0x0000,   /* U+FF23 */
  0x0000,   /* U+FF24 */
  0x0000,   /* U+FF25 */
  0x0000,   /* U+FF26 */
  0x0000,   /* U+FF27 */
  0x0000,   /* U+FF28 */
  0x0000,   /* U+FF29 */
  0x0000,   /* U+FF2A */
  0x0000,   /* U+FF2B */
  0x0000,   /* U+FF2C */
  0x0000,   /* U+FF2D */
  0x0000,   /* U+FF2E */
  0x0000,   /* U+FF2F */
  0x0000,   /* U+FF30 */
  0x0000,   /* U+FF31 */
  0x0000,   /* U+FF32 */
  0x0000,   /* U+FF33 */
  0x0000,   /* U+FF34 */
  0x0000,   /* U+FF35 */
  0x0000,   /* U+FF36 */
  0x0000,   /* U+FF37 */
  0x0000,   /* U+FF38 */
  0x0000,   /* U+FF39 */
  0x0000,   /* U+FF3A */
  0x0000,   /* U+FF3B */
  0x0000,   /* U+FF3C */
  0x0000,   /* U+FF3D */
  0x0000,   /* U+FF3E */
  0x0000,   /* U+FF3F */
  0x0000,   /* U+FF40 */
  0x0000,   /* U+FF41 */
  0x0000,   /* U+FF42 */
  0x0000,   /* U+FF43 */
  0x0000,   /* U+FF44 */
  0x0000,   /* U+FF45 */
  0x0000,   /* U+FF46 */
  0x0000,   /* U+FF47 */
  0x0000,   /* U+FF48 */
  0x0000,   /* U+FF49 */
  0x0000,   /* U+FF4A */
  0x0000,   /* U+FF4B */
  0x0000,   /* U+FF4C */
  0x0000,   /* U+FF4D */
  0x0000,   /* U+FF4E */
  0x0000,   /* U+FF4F */
  0x0000,   /* U+FF50 */
  0x0000,   /* U+FF51 */
  0x0000,   /* U+FF52 */
  0x0000,   /* U+FF53 */
  0x0000,   /* U+FF54 */
  0x0000,   /* U+FF55 */
  0x0000,   /* U+FF56 */
  0x0000,   /* U+FF57 */
  0x0000,   /* U+FF58 */
  0x0000,   /* U+FF59 */
  0x0000,   /* U+FF5A */
  0x0000,   /* U+FF5B */
  0x0000,   /* U+FF5C */
  0x0000,   /* U+FF5D */
  0x0000,   /* U+FF5E */
  0x0000,   /* U+FF5F */
  0x0000,   /* U+FF60 */
  0x0000,   /* U+FF61 */
  0x0000,   /* U+FF62 */
  0x0000,   /* U+FF63 */
  0x0000,   /* U+FF64 */
  0x0000,   /* U+FF65 */
  0x0000,   /* U+FF66 */
  0x0000,   /* U+FF67 */
  0x0000,   /* U+FF68 */
  0x0000,   /* U+FF69 */
  0x0000,   /* U+FF6A */
  0x0000,   /* U+FF6B */
  0x0000,   /* U+FF6C */
  0x0000,   /* U+FF6D */
  0x0000,   /* U+FF6E */
  0x0000,   /* U+FF6F */
  0x0000,   /* U+FF70 */
  0x0000,   /* U+FF71 */
  0x0000,   /* U+FF72 */
  0x0000,   /* U+FF73 */
  0x0000,   /* U+FF74 */
  0x0000,   /* U+FF75 */
  0x0000,   /* U+FF76 */
  0x0000,   /* U+FF77 */
  0x0000,   /* U+FF78 */
  0x0000,   /* U+FF79 */
  0x0000,   /* U+FF7A */
  0x0000,   /* U+FF7B */
  0x0000,   /* U+FF7C */
  0x0000,   /* U+FF7D */
  0x0000,   /* U+FF7E */
  0x0000,   /* U+FF7F */
  0x0000,   /* U+FF80 */
  0x0000,   /* U+FF81 */
  0x0000,   /* U+FF82 */
  0x0000,   /* U+FF83 */
  0x0000,   /* U+FF84 */
  0x0000,   /* U+FF85 */
  0x0000,   /* U+FF86 */
  0x0000,   /* U+FF87 */
  0x0000,   /* U+FF88 */
  0x0000,   /* U+FF89 */
  0x0000,   /* U+FF8A */
  0x0000,   /* U+FF8B */
  0x0000,   /* U+FF8C */
  0x0000,   /* U+FF8D */
  0x0000,   /* U+FF8E */
  0x0000,   /* U+FF8F */
  0x0000,   /* U+FF90 */
  0x0000,   /* U+FF91 */
  0x0000,   /* U+FF92 */
  0x0000,   /* U+FF93 */
  0x0000,   /* U+FF94 */
  0x0000,   /* U+FF95 */
  0x0000,   /* U+FF96 */
  0x0000,   /* U+FF97 */
  0x0000,   /* U+FF98 */
  0x0000,   /* U+FF99 */
  0x0000,   /* U+FF9A */
  0x0000,   /* U+FF9B */
  0x0000,   /* U+FF9C */
  0x0000,   /* U+FF9D */
  0x0000,   /* U+FF9E */
  0x0000,   /* U+FF9F */
  0x0000,   /* U+FFA0 */
  0x0000,   /* U+FFA1 */
  0x0000,   /* U+FFA2 */
  0x0000,   /* U+FFA3 */
  0x0000,   /* U+FFA4 */
  0x0000,   /* U+FFA5 */
  0x0000,   /* U+FFA6 */
  0x0000,   /* U+FFA7 */
  0x0000,   /* U+FFA8 */
  0x0000,   /* U+FFA9 */
  0x0000,   /* U+FFAA */
  0x0000,   /* U+FFAB */
  0x0000,   /* U+FFAC */
  0x0000,   /* U+FFAD */
  0x0000,   /* U+FFAE */
  0x0000,   /* U+FFAF */
  0x0000,   /* U+FFB0 */
  0x0000,   /* U+FFB1 */
  0x0000,   /* U+FFB2 */
  0x0000,   /* U+FFB3 */
  0x0000,   /* U+FFB4 */
  0x0000,   /* U+FFB5 */
  0x0000,   /* U+FFB6 */
  0x0000,   /* U+FFB7 */
  0x0000,   /* U+FFB8 */
  0x0000,   /* U+FFB9 */
  0x0000,   /* U+FFBA */
  0x0000,   /* U+FFBB */
  0x0000,   /* U+FFBC */
  0x0000,   /* U+FFBD */
  0x0000,   /* U+FFBE */
  0xFFBF,   /* U+FFBF */
  0xFFC0,   /* U+FFC0 */
  0xFFC1,   /* U+FFC1 */
  0x0000,   /* U+FFC2 */
  0x0000,   /* U+FFC3 */
  0x0000,   /* U+FFC4 */
  0x0000,   /* U+FFC5 */
  0x0000,   /* U+FFC6 */
  0x0000,   /* U+FFC7 */
  0xFFC8,   /* U+FFC8 */
  0xFFC9,   /* U+FFC9 */
  0x0000,   /* U+FFCA */
  0x0000,   /* U+FFCB */
  0x0000,   /* U+FFCC */
  0x0000,   /* U+FFCD */
  0x0000,   /* U+FFCE */
  0x0000,   /* U+FFCF */
  0xFFD0,   /* U+FFD0 */
  0xFFD1,   /* U+FFD1 */
  0x0000,   /* U+FFD2 */
  0x0000,   /* U+FFD3 */
  0x0000,   /* U+FFD4 */
  0x0000,   /* U+FFD5 */
  0x0000,   /* U+FFD6 */
  0x0000,   /* U+FFD7 */
  0xFFD8,   /* U+FFD8 */
  0xFFD9,   /* U+FFD9 */
  0x0000,   /* U+FFDA */
  0x0000,   /* U+FFDB */
  0x0000,   /* U+FFDC */
  0xFFDD,   /* U+FFDD */
  0xFFDE,   /* U+FFDE */
  0xFFDF,   /* U+FFDF */
  0x0000,   /* U+FFE0 */
  0x0000,   /* U+FFE1 */
  0x0000,   /* U+FFE2 */
  0x0000,   /* U+FFE3 */
  0x0000,   /* U+FFE4 */
  0x0000,   /* U+FFE5 */
  0x0000,   /* U+FFE6 */
  0xFFE7,   /* U+FFE7 */
  0x0000,   /* U+FFE8 */
  0x0000,   /* U+FFE9 */
  0x0000,   /* U+FFEA */
  0x0000,   /* U+FFEB */
  0x0000,   /* U+FFEC */
  0x0000,   /* U+FFED */
  0x0000,   /* U+FFEE */
  0xFFEF,   /* U+FFEF */
  0xFFF0,   /* U+FFF0 */
  0xFFF1,   /* U+FFF1 */
  0xFFF2,   /* U+FFF2 */
  0xFFF3,   /* U+FFF3 */
  0xFFF4,   /* U+FFF4 */
  0xFFF5,   /* U+FFF5 */
  0xFFF6,   /* U+FFF6 */
  0xFFF7,   /* U+FFF7 */
  0xFFF8,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x0000,   /* U+FFFC */
  0x0000,   /* U+FFFD */
  0xFFFE,   /* U+FFFE */
  0xFFFF,   /* U+FFFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+FF00 */
  0x0000,   /* U+FF01 */
  0x0000,   /* U+FF02 */
  0x0000,   /* U+FF03 */
  0x0000,   /* U+FF04 */
  0x0000,   /* U+FF05 */
  0x0000,   /* U+FF06 */
  0x0000,   /* U+FF07 */
  0x0000,   /* U+FF08 */
  0x0000,   /* U+FF09 */
  0x0000,   /* U+FF0A */
  0x0000,   /* U+FF0B */
  0x0000,   /* U+FF0C */
  0x0000,   /* U+FF0D */
  0x0000,   /* U+FF0E */
  0x0000,   /* U+FF0F */
  0x0000,   /* U+FF10 */
  0x0000,   /* U+FF11 */
  0x0000,   /* U+FF12 */
  0x0000,   /* U+FF13 */
  0x0000,   /* U+FF14 */
  0x0000,   /* U+FF15 */
  0x0000,   /* U+FF16 */
  0x0000,   /* U+FF17 */
  0x0000,   /* U+FF18 */
  0x0000,   /* U+FF19 */
  0x0000,   /* U+FF1A */
  0x0000,   /* U+FF1B */
  0x0000,   /* U+FF1C */
  0x0000,   /* U+FF1D */
  0x0000,   /* U+FF1E */
  0x0000,   /* U+FF1F */
  0x0000,   /* U+FF20 */
  0x0000,   /* U+FF21 */
  0x0000,   /* U+FF22 */
  0x0000,   /* U+FF23 */
  0x0000,   /* U+FF24 */
  0x0000,   /* U+FF25 */
  0x0000,   /* U+FF26 */
  0x0000,   /* U+FF27 */
  0x0000,   /* U+FF28 */
  0x0000,   /* U+FF29 */
  0x0000,   /* U+FF2A */
  0x0000,   /* U+FF2B */
  0x0000,   /* U+FF2C */
  0x0000,   /* U+FF2D */
  0x0000,   /* U+FF2E */
  0x0000,   /* U+FF2F */
  0x0000,   /* U+FF30 */
  0x0000,   /* U+FF31 */
  0x0000,   /* U+FF32 */
  0x0000,   /* U+FF33 */
  0x0000,   /* U+FF34 */
  0x0000,   /* U+FF35 */
  0x0000,   /* U+FF36 */
  0x0000,   /* U+FF37 */
  0x0000,   /* U+FF38 */
  0x0000,   /* U+FF39 */
  0x0000,   /* U+FF3A */
  0x0000,   /* U+FF3B */
  0x0000,   /* U+FF3C */
  0x0000,   /* U+FF3D */
  0x0000,   /* U+FF3E */
  0x0000,   /* U+FF3F */
  0x0000,   /* U+FF40 */
  0x0000,   /* U+FF41 */
  0x0000,   /* U+FF42 */
  0x0000,   /* U+FF43 */
  0x0000,   /* U+FF44 */
  0x0000,   /* U+FF45 */
  0x0000,   /* U+FF46 */
  0x0000,   /* U+FF47 */
  0x0000,   /* U+FF48 */
  0x0000,   /* U+FF49 */
  0x0000,   /* U+FF4A */
  0x0000,   /* U+FF4B */
  0x0000,   /* U+FF4C */
  0x0000,   /* U+FF4D */
  0x0000,   /* U+FF4E */
  0x0000,   /* U+FF4F */
  0x0000,   /* U+FF50 */
  0x0000,   /* U+FF51 */
  0x0000,   /* U+FF52 */
  0x0000,   /* U+FF53 */
  0x0000,   /* U+FF54 */
  0x0000,   /* U+FF55 */
  0x0000,   /* U+FF56 */
  0x0000,   /* U+FF57 */
  0x0000,   /* U+FF58 */
  0x0000,   /* U+FF59 */
  0x0000,   /* U+FF5A */
  0x0000,   /* U+FF5B */
  0x0000,   /* U+FF5C */
  0x0000,   /* U+FF5D */
  0x0000,   /* U+FF5E */
  0x0000,   /* U+FF5F */
  0x0000,   /* U+FF60 */
  0x0000,   /* U+FF61 */
  0x0000,   /* U+FF62 */
  0x0000,   /* U+FF63 */
  0x0000,   /* U+FF64 */
  0x0000,   /* U+FF65 */
  0x0000,   /* U+FF66 */
  0x0000,   /* U+FF67 */
  0x0000,   /* U+FF68 */
  0x0000,   /* U+FF69 */
  0x0000,   /* U+FF6A */
  0x0000,   /* U+FF6B */
  0x0000,   /* U+FF6C */
  0x0000,   /* U+FF6D */
  0x0000,   /* U+FF6E */
  0x0000,   /* U+FF6F */
  0x0000,   /* U+FF70 */
  0x0000,   /* U+FF71 */
  0x0000,   /* U+FF72 */
  0x0000,   /* U+FF73 */
  0x0000,   /* U+FF74 */
  0x0000,   /* U+FF75 */
  0x0000,   /* U+FF76 */
  0x0000,   /* U+FF77 */
  0x0000,   /* U+FF78 */
  0x0000,   /* U+FF79 */
  0x0000,   /* U+FF7A */
  0x0000,   /* U+FF7B */
  0x0000,   /* U+FF7C */
  0x0000,   /* U+FF7D */
  0x0000,   /* U+FF7E */
  0x0000,   /* U+FF7F */
  0x0000,   /* U+FF80 */
  0x0000,   /* U+FF81 */
  0x0000,   /* U+FF82 */
  0x0000,   /* U+FF83 */
  0x0000,   /* U+FF84 */
  0x0000,   /* U+FF85 */
  0x0000,   /* U+FF86 */
  0x0000,   /* U+FF87 */
  0x0000,   /* U+FF88 */
  0x0000,   /* U+FF89 */
  0x0000,   /* U+FF8A */
  0x0000,   /* U+FF8B */
  0x0000,   /* U+FF8C */
  0x0000,   /* U+FF8D */
  0x0000,   /* U+FF8E */
  0x0000,   /* U+FF8F */
  0x0000,   /* U+FF90 */
  0x0000,   /* U+FF91 */
  0x0000,   /* U+FF92 */
  0x0000,   /* U+FF93 */
  0x0000,   /* U+FF94 */
  0x0000,   /* U+FF95 */
  0x0000,   /* U+FF96 */
  0x0000,   /* U+FF97 */
  0x0000,   /* U+FF98 */
  0x0000,   /* U+FF99 */
  0x0000,   /* U+FF9A */
  0x0000,   /* U+FF9B */
  0x0000,   /* U+FF9C */
  0x0000,   /* U+FF9D */
  0x0000,   /* U+FF9E */
  0x0000,   /* U+FF9F */
  0x0000,   /* U+FFA0 */
  0x0000,   /* U+FFA1 */
  0x0000,   /* U+FFA2 */
  0x0000,   /* U+FFA3 */
  0x0000,   /* U+FFA4 */
  0x0000,   /* U+FFA5 */
  0x0000,   /* U+FFA6 */
  0x0000,   /* U+FFA7 */
  0x0000,   /* U+FFA8 */
  0x0000,   /* U+FFA9 */
  0x0000,   /* U+FFAA */
  0x0000,   /* U+FFAB */
  0x0000,   /* U+FFAC */
  0x0000,   /* U+FFAD */
  0x0000,   /* U+FFAE */
  0x0000,   /* U+FFAF */
  0x0000,   /* U+FFB0 */
  0x0000,   /* U+FFB1 */
  0x0000,   /* U+FFB2 */
  0x0000,   /* U+FFB3 */
  0x0000,   /* U+FFB4 */
  0x0000,   /* U+FFB5 */
  0x0000,   /* U+FFB6 */
  0x0000,   /* U+FFB7 */
  0x0000,   /* U+FFB8 */
  0x0000,   /* U+FFB9 */
  0x0000,   /* U+FFBA */
  0x0000,   /* U+FFBB */
  0x0000,   /* U+FFBC */
  0x0000,   /* U+FFBD */
  0x0000,   /* U+FFBE */
  0x0000,   /* U+FFBF */
  0x0000,   /* U+FFC0 */
  0x0000,   /* U+FFC1 */
  0x0000,   /* U+FFC2 */
  0x0000,   /* U+FFC3 */
  0x0000,   /* U+FFC4 */
  0x0000,   /* U+FFC5 */
  0x0000,   /* U+FFC6 */
  0x0000,   /* U+FFC7 */
  0x0000,   /* U+FFC8 */
  0x0000,   /* U+FFC9 */
  0x0000,   /* U+FFCA */
  0x0000,   /* U+FFCB */
  0x0000,   /* U+FFCC */
  0x0000,   /* U+FFCD */
  0x0000,   /* U+FFCE */
  0x0000,   /* U+FFCF */
  0x0000,   /* U+FFD0 */
  0x0000,   /* U+FFD1 */
  0x0000,   /* U+FFD2 */
  0x0000,   /* U+FFD3 */
  0x0000,   /* U+FFD4 */
  0x0000,   /* U+FFD5 */
  0x0000,   /* U+FFD6 */
  0x0000,   /* U+FFD7 */
  0x0000,   /* U+FFD8 */
  0x0000,   /* U+FFD9 */
  0x0000,   /* U+FFDA */
  0x0000,   /* U+FFDB */
  0x0000,   /* U+FFDC */
  0x0000,   /* U+FFDD */
  0x0000,   /* U+FFDE */
  0x0000,   /* U+FFDF */
  0x0000,   /* U+FFE0 */
  0x0000,   /* U+FFE1 */
  0x0000,   /* U+FFE2 */
  0x0000,   /* U+FFE3 */
  0x0000,   /* U+FFE4 */
  0x0000,   /* U+FFE5 */
  0x0000,   /* U+FFE6 */
  0x0000,   /* U+FFE7 */
  0x0000,   /* U+FFE8 */
  0x0000,   /* U+FFE9 */
  0x0000,   /* U+FFEA */
  0x0000,   /* U+FFEB */
  0x0000,   /* U+FFEC */
  0x0000,   /* U+FFED */
  0x0000,   /* U+FFEE */
  0x0000,   /* U+FFEF */
  0x0000,   /* U+FFF0 */
  0x0000,   /* U+FFF1 */
  0x0000,   /* U+FFF2 */
  0x0000,   /* U+FFF3 */
  0x0000,   /* U+FFF4 */
  0x0000,   /* U+FFF5 */
  0x0000,   /* U+FFF6 */
  0x0000,   /* U+FFF7 */
  0x0000,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x0000,   /* U+FFFC */
  0x0000,   /* U+FFFD */
  0x0000,   /* U+FFFE */
  0x0000,   /* U+FFFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+FF00 */
  0x0000,   /* U+FF01 */
  0x0000,   /* U+FF02 */
  0x0000,   /* U+FF03 */
  0x0000,   /* U+FF04 */
  0x0000,   /* U+FF05 */
  0x0000,   /* U+FF06 */
  0x0000,   /* U+FF07 */
  0x0000,   /* U+FF08 */
  0x0000,   /* U+FF09 */
  0x0000,   /* U+FF0A */
  0x0000,   /* U+FF0B */
  0x0000,   /* U+FF0C */
  0x0000,   /* U+FF0D */
  0x0000,   /* U+FF0E */
  0x0000,   /* U+FF0F */
  0x0000,   /* U+FF10 */
  0x0000,   /* U+FF11 */
  0x0000,   /* U+FF12 */
  0x0000,   /* U+FF13 */
  0x0000,   /* U+FF14 */
  0x0000,   /* U+FF15 */
  0x0000,   /* U+FF16 */
  0x0000,   /* U+FF17 */
  0x0000,   /* U+FF18 */
  0x0000,   /* U+FF19 */
  0x0000,   /* U+FF1A */
  0x0000,   /* U+FF1B */
  0x0000,   /* U+FF1C */
  0x0000,   /* U+FF1D */
  0x0000,   /* U+FF1E */
  0x0000,   /* U+FF1F */
  0x0000,   /* U+FF20 */
  0x0000,   /* U+FF21 */
  0x0000,   /* U+FF22 */
  0x0000,   /* U+FF23 */
  0x0000,   /* U+FF24 */
  0x0000,   /* U+FF25 */
  0x0000,   /* U+FF26 */
  0x0000,   /* U+FF27 */
  0x0000,   /* U+FF28 */
  0x0000,   /* U+FF29 */
  0x0000,   /* U+FF2A */
  0x0000,   /* U+FF2B */
  0x0000,   /* U+FF2C */
  0x0000,   /* U+FF2D */
  0x0000,   /* U+FF2E */
  0x0000,   /* U+FF2F */
  0x0000,   /* U+FF30 */
  0x0000,   /* U+FF31 */
  0x0000,   /* U+FF32 */
  0x0000,   /* U+FF33 */
  0x0000,   /* U+FF34 */
  0x0000,   /* U+FF35 */
  0x0000,   /* U+FF36 */
  0x0000,   /* U+FF37 */
  0x0000,   /* U+FF38 */
  0x0000,   /* U+FF39 */
  0x0000,   /* U+FF3A */
  0x0000,   /* U+FF3B */
  0x0000,   /* U+FF3C */
  0x0000,   /* U+FF3D */
  0x0000,   /* U+FF3E */
  0x0000,   /* U+FF3F */
  0x0000,   /* U+FF40 */
  0x0000,   /* U+FF41 */
  0x0000,   /* U+FF42 */
  0x0000,   /* U+FF43 */
  0x0000,   /* U+FF44 */
  0x0000,   /* U+FF45 */
  0x0000,   /* U+FF46 */
  0x0000,   /* U+FF47 */
  0x0000,   /* U+FF48 */
  0x0000,   /* U+FF49 */
  0x0000,   /* U+FF4A */
  0x0000,   /* U+FF4B */
  0x0000,   /* U+FF4C */
  0x0000,   /* U+FF4D */
  0x0000,   /* U+FF4E */
  0x0000,   /* U+FF4F */
  0x0000,   /* U+FF50 */
  0x0000,   /* U+FF51 */
  0x0000,   /* U+FF52 */
  0x0000,   /* U+FF53 */
  0x0000,   /* U+FF54 */
  0x0000,   /* U+FF55 */
  0x0000,   /* U+FF56 */
  0x0000,   /* U+FF57 */
  0x0000,   /* U+FF58 */
  0x0000,   /* U+FF59 */
  0x0000,   /* U+FF5A */
  0x0000,   /* U+FF5B */
  0x0000,   /* U+FF5C */
  0x0000,   /* U+FF5D */
  0x0000,   /* U+FF5E */
  0x0000,   /* U+FF5F */
  0x0000,   /* U+FF60 */
  0x0000,   /* U+FF61 */
  0x0000,   /* U+FF62 */
  0x0000,   /* U+FF63 */
  0x0000,   /* U+FF64 */
  0x0000,   /* U+FF65 */
  0x0000,   /* U+FF66 */
  0x0000,   /* U+FF67 */
  0x0000,   /* U+FF68 */
  0x0000,   /* U+FF69 */
  0x0000,   /* U+FF6A */
  0x0000,   /* U+FF6B */
  0x0000,   /* U+FF6C */
  0x0000,   /* U+FF6D */
  0x0000,   /* U+FF6E */
  0x0000,   /* U+FF6F */
  0x0000,   /* U+FF70 */
  0x0000,   /* U+FF71 */
  0x0000,   /* U+FF72 */
  0x0000,   /* U+FF73 */
  0x0000,   /* U+FF74 */
  0x0000,   /* U+FF75 */
  0x0000,   /* U+FF76 */
  0x0000,   /* U+FF77 */
  0x0000,   /* U+FF78 */
  0x0000,   /* U+FF79 */
  0x0000,   /* U+FF7A */
  0x0000,   /* U+FF7B */
  0x0000,   /* U+FF7C */
  0x0000,   /* U+FF7D */
  0x0000,   /* U+FF7E */
  0x0000,   /* U+FF7F */
  0x0000,   /* U+FF80 */
  0x0000,   /* U+FF81 */
  0x0000,   /* U+FF82 */
  0x0000,   /* U+FF83 */
  0x0000,   /* U+FF84 */
  0x0000,   /* U+FF85 */
  0x0000,   /* U+FF86 */
  0x0000,   /* U+FF87 */
  0x0000,   /* U+FF88 */
  0x0000,   /* U+FF89 */
  0x0000,   /* U+FF8A */
  0x0000,   /* U+FF8B */
  0x0000,   /* U+FF8C */
  0x0000,   /* U+FF8D */
  0x0000,   /* U+FF8E */
  0x0000,   /* U+FF8F */
  0x0000,   /* U+FF90 */
  0x0000,   /* U+FF91 */
  0x0000,   /* U+FF92 */
  0x0000,   /* U+FF93 */
  0x0000,   /* U+FF94 */
  0x0000,   /* U+FF95 */
  0x0000,   /* U+FF96 */
  0x0000,   /* U+FF97 */
  0x0000,   /* U+FF98 */
  0x0000,   /* U+FF99 */
  0x0000,   /* U+FF9A */
  0x0000,   /* U+FF9B */
  0x0000,   /* U+FF9C */
  0x0000,   /* U+FF9D */
  0x0000,   /* U+FF9E */
  0x0000,   /* U+FF9F */
  0x0000,   /* U+FFA0 */
  0x0000,   /* U+FFA1 */
  0x0000,   /* U+FFA2 */
  0x0000,   /* U+FFA3 */
  0x0000,   /* U+FFA4 */
  0x0000,   /* U+FFA5 */
  0x0000,   /* U+FFA6 */
  0x0000,   /* U+FFA7 */
  0x0000,   /* U+FFA8 */
  0x0000,   /* U+FFA9 */
  0x0000,   /* U+FFAA */
  0x0000,   /* U+FFAB */
  0x0000,   /* U+FFAC */
  0x0000,   /* U+FFAD */
  0x0000,   /* U+FFAE */
  0x0000,   /* U+FFAF */
  0x0000,   /* U+FFB0 */
  0x0000,   /* U+FFB1 */
  0x0000,   /* U+FFB2 */
  0x0000,   /* U+FFB3 */
  0x0000,   /* U+FFB4 */
  0x0000,   /* U+FFB5 */
  0x0000,   /* U+FFB6 */
  0x0000,   /* U+FFB7 */
  0x0000,   /* U+FFB8 */
  0x0000,   /* U+FFB9 */
  0x0000,   /* U+FFBA */
  0x0000,   /* U+FFBB */
  0x0000,   /* U+FFBC */
  0x0000,   /* U+FFBD */
  0x0000,   /* U+FFBE */
  0x0000,   /* U+FFBF */
  0x0000,   /* U+FFC0 */
  0x0000,   /* U+FFC1 */
  0x0000,   /* U+FFC2 */
  0x0000,   /* U+FFC3 */
  0x0000,   /* U+FFC4 */
  0x0000,   /* U+FFC5 */
  0x0000,   /* U+FFC6 */
  0x0000,   /* U+FFC7 */
  0x0000,   /* U+FFC8 */
  0x0000,   /* U+FFC9 */
  0x0000,   /* U+FFCA */
  0x0000,   /* U+FFCB */
  0x0000,   /* U+FFCC */
  0x0000,   /* U+FFCD */
  0x0000,   /* U+FFCE */
  0x0000,   /* U+FFCF */
  0x0000,   /* U+FFD0 */
  0x0000,   /* U+FFD1 */
  0x0000,   /* U+FFD2 */
  0x0000,   /* U+FFD3 */
  0x0000,   /* U+FFD4 */
  0x0000,   /* U+FFD5 */
  0x0000,   /* U+FFD6 */
  0x0000,   /* U+FFD7 */
  0x0000,   /* U+FFD8 */
  0x0000,   /* U+FFD9 */
  0x0000,   /* U+FFDA */
  0x0000,   /* U+FFDB */
  0x0000,   /* U+FFDC */
  0x0000,   /* U+FFDD */
  0x0000,   /* U+FFDE */
  0x0000,   /* U+FFDF */
  0x0000,   /* U+FFE0 */
  0x0000,   /* U+FFE1 */
  0x0000,   /* U+FFE2 */
  0x0000,   /* U+FFE3 */
  0x0000,   /* U+FFE4 */
  0x0000,   /* U+FFE5 */
  0x0000,   /* U+FFE6 */
  0x0000,   /* U+FFE7 */
  0x0000,   /* U+FFE8 */
  0x0000,   /* U+FFE9 */
  0x0000,   /* U+FFEA */
  0x0000,   /* U+FFEB */
  0x0000,   /* U+FFEC */
  0x0000,   /* U+FFED */
  0x0000,   /* U+FFEE */
  0x0000,   /* U+FFEF */
  0x0000,   /* U+FFF0 */
  0x0000,   /* U+FFF1 */
  0x0000,   /* U+FFF2 */
  0x0000,   /* U+FFF3 */
  0x0000,   /* U+FFF4 */
  0x0000,   /* U+FFF5 */
  0x0000,   /* U+FFF6 */
  0x0000,   /* U+FFF7 */
  0x0000,   /* U+FFF8 */
  0x0000,   /* U+FFF9 */
  0x0000,   /* U+FFFA */
  0x0000,   /* U+FFFB */
  0x0000,   /* U+FFFC */
  0x0000,   /* U+FFFD */
  0x0000,   /* U+FFFE */
  0x0000,   /* U+FFFF */
};

uint16 uca900_p100[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x45E7,   /* U+10000 */
  0x45E8,   /* U+10001 */
  0x45E9,   /* U+10002 */
  0x45EA,   /* U+10003 */
  0x45EB,   /* U+10004 */
  0x45EC,   /* U+10005 */
  0x45ED,   /* U+10006 */
  0x45EE,   /* U+10007 */
  0x45EF,   /* U+10008 */
  0x45F0,   /* U+10009 */
  0x45F1,   /* U+1000A */
  0x45F2,   /* U+1000B */
  0xFBC2,   /* U+1000C */
  0x45F3,   /* U+1000D */
  0x45F4,   /* U+1000E */
  0x45F5,   /* U+1000F */
  0x45F6,   /* U+10010 */
  0x45F7,   /* U+10011 */
  0x45F8,   /* U+10012 */
  0x45F9,   /* U+10013 */
  0x45FA,   /* U+10014 */
  0x45FB,   /* U+10015 */
  0x45FC,   /* U+10016 */
  0x45FD,   /* U+10017 */
  0x45FE,   /* U+10018 */
  0x45FF,   /* U+10019 */
  0x4600,   /* U+1001A */
  0x4601,   /* U+1001B */
  0x4602,   /* U+1001C */
  0x4603,   /* U+1001D */
  0x4604,   /* U+1001E */
  0x4605,   /* U+1001F */
  0x4606,   /* U+10020 */
  0x4607,   /* U+10021 */
  0x4608,   /* U+10022 */
  0x4609,   /* U+10023 */
  0x460A,   /* U+10024 */
  0x460B,   /* U+10025 */
  0x460C,   /* U+10026 */
  0xFBC2,   /* U+10027 */
  0x460D,   /* U+10028 */
  0x460E,   /* U+10029 */
  0x460F,   /* U+1002A */
  0x4610,   /* U+1002B */
  0x4611,   /* U+1002C */
  0x4612,   /* U+1002D */
  0x4613,   /* U+1002E */
  0x4614,   /* U+1002F */
  0x4615,   /* U+10030 */
  0x4616,   /* U+10031 */
  0x4617,   /* U+10032 */
  0x4618,   /* U+10033 */
  0x4619,   /* U+10034 */
  0x461A,   /* U+10035 */
  0x461B,   /* U+10036 */
  0x461C,   /* U+10037 */
  0x461D,   /* U+10038 */
  0x461E,   /* U+10039 */
  0x461F,   /* U+1003A */
  0xFBC2,   /* U+1003B */
  0x4620,   /* U+1003C */
  0x4621,   /* U+1003D */
  0xFBC2,   /* U+1003E */
  0x4622,   /* U+1003F */
  0x4623,   /* U+10040 */
  0x4624,   /* U+10041 */
  0x4625,   /* U+10042 */
  0x4626,   /* U+10043 */
  0x4627,   /* U+10044 */
  0x4628,   /* U+10045 */
  0x4629,   /* U+10046 */
  0x462A,   /* U+10047 */
  0x462B,   /* U+10048 */
  0x462C,   /* U+10049 */
  0x462D,   /* U+1004A */
  0x462E,   /* U+1004B */
  0x462F,   /* U+1004C */
  0x4630,   /* U+1004D */
  0xFBC2,   /* U+1004E */
  0xFBC2,   /* U+1004F */
  0x4631,   /* U+10050 */
  0x4632,   /* U+10051 */
  0x4633,   /* U+10052 */
  0x4634,   /* U+10053 */
  0x4635,   /* U+10054 */
  0x4636,   /* U+10055 */
  0x4637,   /* U+10056 */
  0x4638,   /* U+10057 */
  0x4639,   /* U+10058 */
  0x463A,   /* U+10059 */
  0x463B,   /* U+1005A */
  0x463C,   /* U+1005B */
  0x463D,   /* U+1005C */
  0x463E,   /* U+1005D */
  0xFBC2,   /* U+1005E */
  0xFBC2,   /* U+1005F */
  0xFBC2,   /* U+10060 */
  0xFBC2,   /* U+10061 */
  0xFBC2,   /* U+10062 */
  0xFBC2,   /* U+10063 */
  0xFBC2,   /* U+10064 */
  0xFBC2,   /* U+10065 */
  0xFBC2,   /* U+10066 */
  0xFBC2,   /* U+10067 */
  0xFBC2,   /* U+10068 */
  0xFBC2,   /* U+10069 */
  0xFBC2,   /* U+1006A */
  0xFBC2,   /* U+1006B */
  0xFBC2,   /* U+1006C */
  0xFBC2,   /* U+1006D */
  0xFBC2,   /* U+1006E */
  0xFBC2,   /* U+1006F */
  0xFBC2,   /* U+10070 */
  0xFBC2,   /* U+10071 */
  0xFBC2,   /* U+10072 */
  0xFBC2,   /* U+10073 */
  0xFBC2,   /* U+10074 */
  0xFBC2,   /* U+10075 */
  0xFBC2,   /* U+10076 */
  0xFBC2,   /* U+10077 */
  0xFBC2,   /* U+10078 */
  0xFBC2,   /* U+10079 */
  0xFBC2,   /* U+1007A */
  0xFBC2,   /* U+1007B */
  0xFBC2,   /* U+1007C */
  0xFBC2,   /* U+1007D */
  0xFBC2,   /* U+1007E */
  0xFBC2,   /* U+1007F */
  0x463F,   /* U+10080 */
  0x4640,   /* U+10081 */
  0x4641,   /* U+10082 */
  0x4642,   /* U+10083 */
  0x4643,   /* U+10084 */
  0x4644,   /* U+10085 */
  0x4645,   /* U+10086 */
  0x4646,   /* U+10087 */
  0x4647,   /* U+10088 */
  0x4648,   /* U+10089 */
  0x4649,   /* U+1008A */
  0x464A,   /* U+1008B */
  0x464B,   /* U+1008C */
  0x464C,   /* U+1008D */
  0x464D,   /* U+1008E */
  0x464E,   /* U+1008F */
  0x464F,   /* U+10090 */
  0x4650,   /* U+10091 */
  0x4651,   /* U+10092 */
  0x4652,   /* U+10093 */
  0x4653,   /* U+10094 */
  0x4654,   /* U+10095 */
  0x4655,   /* U+10096 */
  0x4656,   /* U+10097 */
  0x4657,   /* U+10098 */
  0x4658,   /* U+10099 */
  0x4659,   /* U+1009A */
  0x465A,   /* U+1009B */
  0x465B,   /* U+1009C */
  0x465C,   /* U+1009D */
  0x465D,   /* U+1009E */
  0x465E,   /* U+1009F */
  0x465F,   /* U+100A0 */
  0x4660,   /* U+100A1 */
  0x4661,   /* U+100A2 */
  0x4662,   /* U+100A3 */
  0x4663,   /* U+100A4 */
  0x4664,   /* U+100A5 */
  0x4665,   /* U+100A6 */
  0x4666,   /* U+100A7 */
  0x4667,   /* U+100A8 */
  0x4668,   /* U+100A9 */
  0x4669,   /* U+100AA */
  0x466A,   /* U+100AB */
  0x466B,   /* U+100AC */
  0x466C,   /* U+100AD */
  0x466D,   /* U+100AE */
  0x466E,   /* U+100AF */
  0x466F,   /* U+100B0 */
  0x4670,   /* U+100B1 */
  0x4671,   /* U+100B2 */
  0x4672,   /* U+100B3 */
  0x4673,   /* U+100B4 */
  0x4674,   /* U+100B5 */
  0x4675,   /* U+100B6 */
  0x4676,   /* U+100B7 */
  0x4677,   /* U+100B8 */
  0x4678,   /* U+100B9 */
  0x4679,   /* U+100BA */
  0x467A,   /* U+100BB */
  0x467B,   /* U+100BC */
  0x467C,   /* U+100BD */
  0x467D,   /* U+100BE */
  0x467E,   /* U+100BF */
  0x467F,   /* U+100C0 */
  0x4680,   /* U+100C1 */
  0x4681,   /* U+100C2 */
  0x4682,   /* U+100C3 */
  0x4683,   /* U+100C4 */
  0x4684,   /* U+100C5 */
  0x4685,   /* U+100C6 */
  0x4686,   /* U+100C7 */
  0x4687,   /* U+100C8 */
  0x4688,   /* U+100C9 */
  0x4689,   /* U+100CA */
  0x468A,   /* U+100CB */
  0x468B,   /* U+100CC */
  0x468C,   /* U+100CD */
  0x468D,   /* U+100CE */
  0x468E,   /* U+100CF */
  0x468F,   /* U+100D0 */
  0x4690,   /* U+100D1 */
  0x4691,   /* U+100D2 */
  0x4692,   /* U+100D3 */
  0x4693,   /* U+100D4 */
  0x4694,   /* U+100D5 */
  0x4695,   /* U+100D6 */
  0x4696,   /* U+100D7 */
  0x4697,   /* U+100D8 */
  0x4698,   /* U+100D9 */
  0x4699,   /* U+100DA */
  0x469A,   /* U+100DB */
  0x469B,   /* U+100DC */
  0x469C,   /* U+100DD */
  0x469D,   /* U+100DE */
  0x469E,   /* U+100DF */
  0x469F,   /* U+100E0 */
  0x46A0,   /* U+100E1 */
  0x46A1,   /* U+100E2 */
  0x46A2,   /* U+100E3 */
  0x46A3,   /* U+100E4 */
  0x46A4,   /* U+100E5 */
  0x46A5,   /* U+100E6 */
  0x46A6,   /* U+100E7 */
  0x46A7,   /* U+100E8 */
  0x46A8,   /* U+100E9 */
  0x46A9,   /* U+100EA */
  0x46AA,   /* U+100EB */
  0x46AB,   /* U+100EC */
  0x46AC,   /* U+100ED */
  0x46AD,   /* U+100EE */
  0x46AE,   /* U+100EF */
  0x46AF,   /* U+100F0 */
  0x46B0,   /* U+100F1 */
  0x46B1,   /* U+100F2 */
  0x46B2,   /* U+100F3 */
  0x46B3,   /* U+100F4 */
  0x46B4,   /* U+100F5 */
  0x46B5,   /* U+100F6 */
  0x46B6,   /* U+100F7 */
  0x46B7,   /* U+100F8 */
  0x46B8,   /* U+100F9 */
  0x46B9,   /* U+100FA */
  0xFBC2,   /* U+100FB */
  0xFBC2,   /* U+100FC */
  0xFBC2,   /* U+100FD */
  0xFBC2,   /* U+100FE */
  0xFBC2,   /* U+100FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10000 */
  0x0020,   /* U+10001 */
  0x0020,   /* U+10002 */
  0x0020,   /* U+10003 */
  0x0020,   /* U+10004 */
  0x0020,   /* U+10005 */
  0x0020,   /* U+10006 */
  0x0020,   /* U+10007 */
  0x0020,   /* U+10008 */
  0x0020,   /* U+10009 */
  0x0020,   /* U+1000A */
  0x0020,   /* U+1000B */
  0x0020,   /* U+1000C */
  0x0020,   /* U+1000D */
  0x0020,   /* U+1000E */
  0x0020,   /* U+1000F */
  0x0020,   /* U+10010 */
  0x0020,   /* U+10011 */
  0x0020,   /* U+10012 */
  0x0020,   /* U+10013 */
  0x0020,   /* U+10014 */
  0x0020,   /* U+10015 */
  0x0020,   /* U+10016 */
  0x0020,   /* U+10017 */
  0x0020,   /* U+10018 */
  0x0020,   /* U+10019 */
  0x0020,   /* U+1001A */
  0x0020,   /* U+1001B */
  0x0020,   /* U+1001C */
  0x0020,   /* U+1001D */
  0x0020,   /* U+1001E */
  0x0020,   /* U+1001F */
  0x0020,   /* U+10020 */
  0x0020,   /* U+10021 */
  0x0020,   /* U+10022 */
  0x0020,   /* U+10023 */
  0x0020,   /* U+10024 */
  0x0020,   /* U+10025 */
  0x0020,   /* U+10026 */
  0x0020,   /* U+10027 */
  0x0020,   /* U+10028 */
  0x0020,   /* U+10029 */
  0x0020,   /* U+1002A */
  0x0020,   /* U+1002B */
  0x0020,   /* U+1002C */
  0x0020,   /* U+1002D */
  0x0020,   /* U+1002E */
  0x0020,   /* U+1002F */
  0x0020,   /* U+10030 */
  0x0020,   /* U+10031 */
  0x0020,   /* U+10032 */
  0x0020,   /* U+10033 */
  0x0020,   /* U+10034 */
  0x0020,   /* U+10035 */
  0x0020,   /* U+10036 */
  0x0020,   /* U+10037 */
  0x0020,   /* U+10038 */
  0x0020,   /* U+10039 */
  0x0020,   /* U+1003A */
  0x0020,   /* U+1003B */
  0x0020,   /* U+1003C */
  0x0020,   /* U+1003D */
  0x0020,   /* U+1003E */
  0x0020,   /* U+1003F */
  0x0020,   /* U+10040 */
  0x0020,   /* U+10041 */
  0x0020,   /* U+10042 */
  0x0020,   /* U+10043 */
  0x0020,   /* U+10044 */
  0x0020,   /* U+10045 */
  0x0020,   /* U+10046 */
  0x0020,   /* U+10047 */
  0x0020,   /* U+10048 */
  0x0020,   /* U+10049 */
  0x0020,   /* U+1004A */
  0x0020,   /* U+1004B */
  0x0020,   /* U+1004C */
  0x0020,   /* U+1004D */
  0x0020,   /* U+1004E */
  0x0020,   /* U+1004F */
  0x0020,   /* U+10050 */
  0x0020,   /* U+10051 */
  0x0020,   /* U+10052 */
  0x0020,   /* U+10053 */
  0x0020,   /* U+10054 */
  0x0020,   /* U+10055 */
  0x0020,   /* U+10056 */
  0x0020,   /* U+10057 */
  0x0020,   /* U+10058 */
  0x0020,   /* U+10059 */
  0x0020,   /* U+1005A */
  0x0020,   /* U+1005B */
  0x0020,   /* U+1005C */
  0x0020,   /* U+1005D */
  0x0020,   /* U+1005E */
  0x0020,   /* U+1005F */
  0x0020,   /* U+10060 */
  0x0020,   /* U+10061 */
  0x0020,   /* U+10062 */
  0x0020,   /* U+10063 */
  0x0020,   /* U+10064 */
  0x0020,   /* U+10065 */
  0x0020,   /* U+10066 */
  0x0020,   /* U+10067 */
  0x0020,   /* U+10068 */
  0x0020,   /* U+10069 */
  0x0020,   /* U+1006A */
  0x0020,   /* U+1006B */
  0x0020,   /* U+1006C */
  0x0020,   /* U+1006D */
  0x0020,   /* U+1006E */
  0x0020,   /* U+1006F */
  0x0020,   /* U+10070 */
  0x0020,   /* U+10071 */
  0x0020,   /* U+10072 */
  0x0020,   /* U+10073 */
  0x0020,   /* U+10074 */
  0x0020,   /* U+10075 */
  0x0020,   /* U+10076 */
  0x0020,   /* U+10077 */
  0x0020,   /* U+10078 */
  0x0020,   /* U+10079 */
  0x0020,   /* U+1007A */
  0x0020,   /* U+1007B */
  0x0020,   /* U+1007C */
  0x0020,   /* U+1007D */
  0x0020,   /* U+1007E */
  0x0020,   /* U+1007F */
  0x0020,   /* U+10080 */
  0x0020,   /* U+10081 */
  0x0020,   /* U+10082 */
  0x0020,   /* U+10083 */
  0x0020,   /* U+10084 */
  0x0020,   /* U+10085 */
  0x0020,   /* U+10086 */
  0x0020,   /* U+10087 */
  0x0020,   /* U+10088 */
  0x0020,   /* U+10089 */
  0x0020,   /* U+1008A */
  0x0020,   /* U+1008B */
  0x0020,   /* U+1008C */
  0x0020,   /* U+1008D */
  0x0020,   /* U+1008E */
  0x0020,   /* U+1008F */
  0x0020,   /* U+10090 */
  0x0020,   /* U+10091 */
  0x0020,   /* U+10092 */
  0x0020,   /* U+10093 */
  0x0020,   /* U+10094 */
  0x0020,   /* U+10095 */
  0x0020,   /* U+10096 */
  0x0020,   /* U+10097 */
  0x0020,   /* U+10098 */
  0x0020,   /* U+10099 */
  0x0020,   /* U+1009A */
  0x0020,   /* U+1009B */
  0x0020,   /* U+1009C */
  0x0020,   /* U+1009D */
  0x0020,   /* U+1009E */
  0x0020,   /* U+1009F */
  0x0020,   /* U+100A0 */
  0x0020,   /* U+100A1 */
  0x0020,   /* U+100A2 */
  0x0020,   /* U+100A3 */
  0x0020,   /* U+100A4 */
  0x0020,   /* U+100A5 */
  0x0020,   /* U+100A6 */
  0x0020,   /* U+100A7 */
  0x0020,   /* U+100A8 */
  0x0020,   /* U+100A9 */
  0x0020,   /* U+100AA */
  0x0020,   /* U+100AB */
  0x0020,   /* U+100AC */
  0x0020,   /* U+100AD */
  0x0020,   /* U+100AE */
  0x0020,   /* U+100AF */
  0x0020,   /* U+100B0 */
  0x0020,   /* U+100B1 */
  0x0020,   /* U+100B2 */
  0x0020,   /* U+100B3 */
  0x0020,   /* U+100B4 */
  0x0020,   /* U+100B5 */
  0x0020,   /* U+100B6 */
  0x0020,   /* U+100B7 */
  0x0020,   /* U+100B8 */
  0x0020,   /* U+100B9 */
  0x0020,   /* U+100BA */
  0x0020,   /* U+100BB */
  0x0020,   /* U+100BC */
  0x0020,   /* U+100BD */
  0x0020,   /* U+100BE */
  0x0020,   /* U+100BF */
  0x0020,   /* U+100C0 */
  0x0020,   /* U+100C1 */
  0x0020,   /* U+100C2 */
  0x0020,   /* U+100C3 */
  0x0020,   /* U+100C4 */
  0x0020,   /* U+100C5 */
  0x0020,   /* U+100C6 */
  0x0020,   /* U+100C7 */
  0x0020,   /* U+100C8 */
  0x0020,   /* U+100C9 */
  0x0020,   /* U+100CA */
  0x0020,   /* U+100CB */
  0x0020,   /* U+100CC */
  0x0020,   /* U+100CD */
  0x0020,   /* U+100CE */
  0x0020,   /* U+100CF */
  0x0020,   /* U+100D0 */
  0x0020,   /* U+100D1 */
  0x0020,   /* U+100D2 */
  0x0020,   /* U+100D3 */
  0x0020,   /* U+100D4 */
  0x0020,   /* U+100D5 */
  0x0020,   /* U+100D6 */
  0x0020,   /* U+100D7 */
  0x0020,   /* U+100D8 */
  0x0020,   /* U+100D9 */
  0x0020,   /* U+100DA */
  0x0020,   /* U+100DB */
  0x0020,   /* U+100DC */
  0x0020,   /* U+100DD */
  0x0020,   /* U+100DE */
  0x0020,   /* U+100DF */
  0x0020,   /* U+100E0 */
  0x0020,   /* U+100E1 */
  0x0020,   /* U+100E2 */
  0x0020,   /* U+100E3 */
  0x0020,   /* U+100E4 */
  0x0020,   /* U+100E5 */
  0x0020,   /* U+100E6 */
  0x0020,   /* U+100E7 */
  0x0020,   /* U+100E8 */
  0x0020,   /* U+100E9 */
  0x0020,   /* U+100EA */
  0x0020,   /* U+100EB */
  0x0020,   /* U+100EC */
  0x0020,   /* U+100ED */
  0x0020,   /* U+100EE */
  0x0020,   /* U+100EF */
  0x0020,   /* U+100F0 */
  0x0020,   /* U+100F1 */
  0x0020,   /* U+100F2 */
  0x0020,   /* U+100F3 */
  0x0020,   /* U+100F4 */
  0x0020,   /* U+100F5 */
  0x0020,   /* U+100F6 */
  0x0020,   /* U+100F7 */
  0x0020,   /* U+100F8 */
  0x0020,   /* U+100F9 */
  0x0020,   /* U+100FA */
  0x0020,   /* U+100FB */
  0x0020,   /* U+100FC */
  0x0020,   /* U+100FD */
  0x0020,   /* U+100FE */
  0x0020,   /* U+100FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10000 */
  0x0002,   /* U+10001 */
  0x0002,   /* U+10002 */
  0x0002,   /* U+10003 */
  0x0002,   /* U+10004 */
  0x0002,   /* U+10005 */
  0x0002,   /* U+10006 */
  0x0002,   /* U+10007 */
  0x0002,   /* U+10008 */
  0x0002,   /* U+10009 */
  0x0002,   /* U+1000A */
  0x0002,   /* U+1000B */
  0x0002,   /* U+1000C */
  0x0002,   /* U+1000D */
  0x0002,   /* U+1000E */
  0x0002,   /* U+1000F */
  0x0002,   /* U+10010 */
  0x0002,   /* U+10011 */
  0x0002,   /* U+10012 */
  0x0002,   /* U+10013 */
  0x0002,   /* U+10014 */
  0x0002,   /* U+10015 */
  0x0002,   /* U+10016 */
  0x0002,   /* U+10017 */
  0x0002,   /* U+10018 */
  0x0002,   /* U+10019 */
  0x0002,   /* U+1001A */
  0x0002,   /* U+1001B */
  0x0002,   /* U+1001C */
  0x0002,   /* U+1001D */
  0x0002,   /* U+1001E */
  0x0002,   /* U+1001F */
  0x0002,   /* U+10020 */
  0x0002,   /* U+10021 */
  0x0002,   /* U+10022 */
  0x0002,   /* U+10023 */
  0x0002,   /* U+10024 */
  0x0002,   /* U+10025 */
  0x0002,   /* U+10026 */
  0x0002,   /* U+10027 */
  0x0002,   /* U+10028 */
  0x0002,   /* U+10029 */
  0x0002,   /* U+1002A */
  0x0002,   /* U+1002B */
  0x0002,   /* U+1002C */
  0x0002,   /* U+1002D */
  0x0002,   /* U+1002E */
  0x0002,   /* U+1002F */
  0x0002,   /* U+10030 */
  0x0002,   /* U+10031 */
  0x0002,   /* U+10032 */
  0x0002,   /* U+10033 */
  0x0002,   /* U+10034 */
  0x0002,   /* U+10035 */
  0x0002,   /* U+10036 */
  0x0002,   /* U+10037 */
  0x0002,   /* U+10038 */
  0x0002,   /* U+10039 */
  0x0002,   /* U+1003A */
  0x0002,   /* U+1003B */
  0x0002,   /* U+1003C */
  0x0002,   /* U+1003D */
  0x0002,   /* U+1003E */
  0x0002,   /* U+1003F */
  0x0002,   /* U+10040 */
  0x0002,   /* U+10041 */
  0x0002,   /* U+10042 */
  0x0002,   /* U+10043 */
  0x0002,   /* U+10044 */
  0x0002,   /* U+10045 */
  0x0002,   /* U+10046 */
  0x0002,   /* U+10047 */
  0x0002,   /* U+10048 */
  0x0002,   /* U+10049 */
  0x0002,   /* U+1004A */
  0x0002,   /* U+1004B */
  0x0002,   /* U+1004C */
  0x0002,   /* U+1004D */
  0x0002,   /* U+1004E */
  0x0002,   /* U+1004F */
  0x0002,   /* U+10050 */
  0x0002,   /* U+10051 */
  0x0002,   /* U+10052 */
  0x0002,   /* U+10053 */
  0x0002,   /* U+10054 */
  0x0002,   /* U+10055 */
  0x0002,   /* U+10056 */
  0x0002,   /* U+10057 */
  0x0002,   /* U+10058 */
  0x0002,   /* U+10059 */
  0x0002,   /* U+1005A */
  0x0002,   /* U+1005B */
  0x0002,   /* U+1005C */
  0x0002,   /* U+1005D */
  0x0002,   /* U+1005E */
  0x0002,   /* U+1005F */
  0x0002,   /* U+10060 */
  0x0002,   /* U+10061 */
  0x0002,   /* U+10062 */
  0x0002,   /* U+10063 */
  0x0002,   /* U+10064 */
  0x0002,   /* U+10065 */
  0x0002,   /* U+10066 */
  0x0002,   /* U+10067 */
  0x0002,   /* U+10068 */
  0x0002,   /* U+10069 */
  0x0002,   /* U+1006A */
  0x0002,   /* U+1006B */
  0x0002,   /* U+1006C */
  0x0002,   /* U+1006D */
  0x0002,   /* U+1006E */
  0x0002,   /* U+1006F */
  0x0002,   /* U+10070 */
  0x0002,   /* U+10071 */
  0x0002,   /* U+10072 */
  0x0002,   /* U+10073 */
  0x0002,   /* U+10074 */
  0x0002,   /* U+10075 */
  0x0002,   /* U+10076 */
  0x0002,   /* U+10077 */
  0x0002,   /* U+10078 */
  0x0002,   /* U+10079 */
  0x0002,   /* U+1007A */
  0x0002,   /* U+1007B */
  0x0002,   /* U+1007C */
  0x0002,   /* U+1007D */
  0x0002,   /* U+1007E */
  0x0002,   /* U+1007F */
  0x0002,   /* U+10080 */
  0x0002,   /* U+10081 */
  0x0002,   /* U+10082 */
  0x0002,   /* U+10083 */
  0x0002,   /* U+10084 */
  0x0002,   /* U+10085 */
  0x0002,   /* U+10086 */
  0x0002,   /* U+10087 */
  0x0002,   /* U+10088 */
  0x0002,   /* U+10089 */
  0x0002,   /* U+1008A */
  0x0002,   /* U+1008B */
  0x0002,   /* U+1008C */
  0x0002,   /* U+1008D */
  0x0002,   /* U+1008E */
  0x0002,   /* U+1008F */
  0x0002,   /* U+10090 */
  0x0002,   /* U+10091 */
  0x0002,   /* U+10092 */
  0x0002,   /* U+10093 */
  0x0002,   /* U+10094 */
  0x0002,   /* U+10095 */
  0x0002,   /* U+10096 */
  0x0002,   /* U+10097 */
  0x0002,   /* U+10098 */
  0x0002,   /* U+10099 */
  0x0002,   /* U+1009A */
  0x0002,   /* U+1009B */
  0x0002,   /* U+1009C */
  0x0002,   /* U+1009D */
  0x0002,   /* U+1009E */
  0x0002,   /* U+1009F */
  0x0002,   /* U+100A0 */
  0x0002,   /* U+100A1 */
  0x0002,   /* U+100A2 */
  0x0002,   /* U+100A3 */
  0x0002,   /* U+100A4 */
  0x0002,   /* U+100A5 */
  0x0002,   /* U+100A6 */
  0x0002,   /* U+100A7 */
  0x0002,   /* U+100A8 */
  0x0002,   /* U+100A9 */
  0x0002,   /* U+100AA */
  0x0002,   /* U+100AB */
  0x0002,   /* U+100AC */
  0x0002,   /* U+100AD */
  0x0002,   /* U+100AE */
  0x0002,   /* U+100AF */
  0x0002,   /* U+100B0 */
  0x0002,   /* U+100B1 */
  0x0002,   /* U+100B2 */
  0x0002,   /* U+100B3 */
  0x0002,   /* U+100B4 */
  0x0002,   /* U+100B5 */
  0x0002,   /* U+100B6 */
  0x0002,   /* U+100B7 */
  0x0002,   /* U+100B8 */
  0x0002,   /* U+100B9 */
  0x0002,   /* U+100BA */
  0x0002,   /* U+100BB */
  0x0002,   /* U+100BC */
  0x0002,   /* U+100BD */
  0x0002,   /* U+100BE */
  0x0002,   /* U+100BF */
  0x0002,   /* U+100C0 */
  0x0002,   /* U+100C1 */
  0x0002,   /* U+100C2 */
  0x0002,   /* U+100C3 */
  0x0002,   /* U+100C4 */
  0x0002,   /* U+100C5 */
  0x0002,   /* U+100C6 */
  0x0002,   /* U+100C7 */
  0x0002,   /* U+100C8 */
  0x0002,   /* U+100C9 */
  0x0002,   /* U+100CA */
  0x0002,   /* U+100CB */
  0x0002,   /* U+100CC */
  0x0002,   /* U+100CD */
  0x0002,   /* U+100CE */
  0x0002,   /* U+100CF */
  0x0002,   /* U+100D0 */
  0x0002,   /* U+100D1 */
  0x0002,   /* U+100D2 */
  0x0002,   /* U+100D3 */
  0x0002,   /* U+100D4 */
  0x0002,   /* U+100D5 */
  0x0002,   /* U+100D6 */
  0x0002,   /* U+100D7 */
  0x0002,   /* U+100D8 */
  0x0002,   /* U+100D9 */
  0x0002,   /* U+100DA */
  0x0002,   /* U+100DB */
  0x0002,   /* U+100DC */
  0x0002,   /* U+100DD */
  0x0002,   /* U+100DE */
  0x0002,   /* U+100DF */
  0x0002,   /* U+100E0 */
  0x0002,   /* U+100E1 */
  0x0002,   /* U+100E2 */
  0x0002,   /* U+100E3 */
  0x0002,   /* U+100E4 */
  0x0002,   /* U+100E5 */
  0x0002,   /* U+100E6 */
  0x0002,   /* U+100E7 */
  0x0002,   /* U+100E8 */
  0x0002,   /* U+100E9 */
  0x0002,   /* U+100EA */
  0x0002,   /* U+100EB */
  0x0002,   /* U+100EC */
  0x0002,   /* U+100ED */
  0x0002,   /* U+100EE */
  0x0002,   /* U+100EF */
  0x0002,   /* U+100F0 */
  0x0002,   /* U+100F1 */
  0x0002,   /* U+100F2 */
  0x0002,   /* U+100F3 */
  0x0002,   /* U+100F4 */
  0x0002,   /* U+100F5 */
  0x0002,   /* U+100F6 */
  0x0002,   /* U+100F7 */
  0x0002,   /* U+100F8 */
  0x0002,   /* U+100F9 */
  0x0002,   /* U+100FA */
  0x0002,   /* U+100FB */
  0x0002,   /* U+100FC */
  0x0002,   /* U+100FD */
  0x0002,   /* U+100FE */
  0x0002,   /* U+100FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10000 */
  0x0000,   /* U+10001 */
  0x0000,   /* U+10002 */
  0x0000,   /* U+10003 */
  0x0000,   /* U+10004 */
  0x0000,   /* U+10005 */
  0x0000,   /* U+10006 */
  0x0000,   /* U+10007 */
  0x0000,   /* U+10008 */
  0x0000,   /* U+10009 */
  0x0000,   /* U+1000A */
  0x0000,   /* U+1000B */
  0x800C,   /* U+1000C */
  0x0000,   /* U+1000D */
  0x0000,   /* U+1000E */
  0x0000,   /* U+1000F */
  0x0000,   /* U+10010 */
  0x0000,   /* U+10011 */
  0x0000,   /* U+10012 */
  0x0000,   /* U+10013 */
  0x0000,   /* U+10014 */
  0x0000,   /* U+10015 */
  0x0000,   /* U+10016 */
  0x0000,   /* U+10017 */
  0x0000,   /* U+10018 */
  0x0000,   /* U+10019 */
  0x0000,   /* U+1001A */
  0x0000,   /* U+1001B */
  0x0000,   /* U+1001C */
  0x0000,   /* U+1001D */
  0x0000,   /* U+1001E */
  0x0000,   /* U+1001F */
  0x0000,   /* U+10020 */
  0x0000,   /* U+10021 */
  0x0000,   /* U+10022 */
  0x0000,   /* U+10023 */
  0x0000,   /* U+10024 */
  0x0000,   /* U+10025 */
  0x0000,   /* U+10026 */
  0x8027,   /* U+10027 */
  0x0000,   /* U+10028 */
  0x0000,   /* U+10029 */
  0x0000,   /* U+1002A */
  0x0000,   /* U+1002B */
  0x0000,   /* U+1002C */
  0x0000,   /* U+1002D */
  0x0000,   /* U+1002E */
  0x0000,   /* U+1002F */
  0x0000,   /* U+10030 */
  0x0000,   /* U+10031 */
  0x0000,   /* U+10032 */
  0x0000,   /* U+10033 */
  0x0000,   /* U+10034 */
  0x0000,   /* U+10035 */
  0x0000,   /* U+10036 */
  0x0000,   /* U+10037 */
  0x0000,   /* U+10038 */
  0x0000,   /* U+10039 */
  0x0000,   /* U+1003A */
  0x803B,   /* U+1003B */
  0x0000,   /* U+1003C */
  0x0000,   /* U+1003D */
  0x803E,   /* U+1003E */
  0x0000,   /* U+1003F */
  0x0000,   /* U+10040 */
  0x0000,   /* U+10041 */
  0x0000,   /* U+10042 */
  0x0000,   /* U+10043 */
  0x0000,   /* U+10044 */
  0x0000,   /* U+10045 */
  0x0000,   /* U+10046 */
  0x0000,   /* U+10047 */
  0x0000,   /* U+10048 */
  0x0000,   /* U+10049 */
  0x0000,   /* U+1004A */
  0x0000,   /* U+1004B */
  0x0000,   /* U+1004C */
  0x0000,   /* U+1004D */
  0x804E,   /* U+1004E */
  0x804F,   /* U+1004F */
  0x0000,   /* U+10050 */
  0x0000,   /* U+10051 */
  0x0000,   /* U+10052 */
  0x0000,   /* U+10053 */
  0x0000,   /* U+10054 */
  0x0000,   /* U+10055 */
  0x0000,   /* U+10056 */
  0x0000,   /* U+10057 */
  0x0000,   /* U+10058 */
  0x0000,   /* U+10059 */
  0x0000,   /* U+1005A */
  0x0000,   /* U+1005B */
  0x0000,   /* U+1005C */
  0x0000,   /* U+1005D */
  0x805E,   /* U+1005E */
  0x805F,   /* U+1005F */
  0x8060,   /* U+10060 */
  0x8061,   /* U+10061 */
  0x8062,   /* U+10062 */
  0x8063,   /* U+10063 */
  0x8064,   /* U+10064 */
  0x8065,   /* U+10065 */
  0x8066,   /* U+10066 */
  0x8067,   /* U+10067 */
  0x8068,   /* U+10068 */
  0x8069,   /* U+10069 */
  0x806A,   /* U+1006A */
  0x806B,   /* U+1006B */
  0x806C,   /* U+1006C */
  0x806D,   /* U+1006D */
  0x806E,   /* U+1006E */
  0x806F,   /* U+1006F */
  0x8070,   /* U+10070 */
  0x8071,   /* U+10071 */
  0x8072,   /* U+10072 */
  0x8073,   /* U+10073 */
  0x8074,   /* U+10074 */
  0x8075,   /* U+10075 */
  0x8076,   /* U+10076 */
  0x8077,   /* U+10077 */
  0x8078,   /* U+10078 */
  0x8079,   /* U+10079 */
  0x807A,   /* U+1007A */
  0x807B,   /* U+1007B */
  0x807C,   /* U+1007C */
  0x807D,   /* U+1007D */
  0x807E,   /* U+1007E */
  0x807F,   /* U+1007F */
  0x0000,   /* U+10080 */
  0x0000,   /* U+10081 */
  0x0000,   /* U+10082 */
  0x0000,   /* U+10083 */
  0x0000,   /* U+10084 */
  0x0000,   /* U+10085 */
  0x0000,   /* U+10086 */
  0x0000,   /* U+10087 */
  0x0000,   /* U+10088 */
  0x0000,   /* U+10089 */
  0x0000,   /* U+1008A */
  0x0000,   /* U+1008B */
  0x0000,   /* U+1008C */
  0x0000,   /* U+1008D */
  0x0000,   /* U+1008E */
  0x0000,   /* U+1008F */
  0x0000,   /* U+10090 */
  0x0000,   /* U+10091 */
  0x0000,   /* U+10092 */
  0x0000,   /* U+10093 */
  0x0000,   /* U+10094 */
  0x0000,   /* U+10095 */
  0x0000,   /* U+10096 */
  0x0000,   /* U+10097 */
  0x0000,   /* U+10098 */
  0x0000,   /* U+10099 */
  0x0000,   /* U+1009A */
  0x0000,   /* U+1009B */
  0x0000,   /* U+1009C */
  0x0000,   /* U+1009D */
  0x0000,   /* U+1009E */
  0x0000,   /* U+1009F */
  0x0000,   /* U+100A0 */
  0x0000,   /* U+100A1 */
  0x0000,   /* U+100A2 */
  0x0000,   /* U+100A3 */
  0x0000,   /* U+100A4 */
  0x0000,   /* U+100A5 */
  0x0000,   /* U+100A6 */
  0x0000,   /* U+100A7 */
  0x0000,   /* U+100A8 */
  0x0000,   /* U+100A9 */
  0x0000,   /* U+100AA */
  0x0000,   /* U+100AB */
  0x0000,   /* U+100AC */
  0x0000,   /* U+100AD */
  0x0000,   /* U+100AE */
  0x0000,   /* U+100AF */
  0x0000,   /* U+100B0 */
  0x0000,   /* U+100B1 */
  0x0000,   /* U+100B2 */
  0x0000,   /* U+100B3 */
  0x0000,   /* U+100B4 */
  0x0000,   /* U+100B5 */
  0x0000,   /* U+100B6 */
  0x0000,   /* U+100B7 */
  0x0000,   /* U+100B8 */
  0x0000,   /* U+100B9 */
  0x0000,   /* U+100BA */
  0x0000,   /* U+100BB */
  0x0000,   /* U+100BC */
  0x0000,   /* U+100BD */
  0x0000,   /* U+100BE */
  0x0000,   /* U+100BF */
  0x0000,   /* U+100C0 */
  0x0000,   /* U+100C1 */
  0x0000,   /* U+100C2 */
  0x0000,   /* U+100C3 */
  0x0000,   /* U+100C4 */
  0x0000,   /* U+100C5 */
  0x0000,   /* U+100C6 */
  0x0000,   /* U+100C7 */
  0x0000,   /* U+100C8 */
  0x0000,   /* U+100C9 */
  0x0000,   /* U+100CA */
  0x0000,   /* U+100CB */
  0x0000,   /* U+100CC */
  0x0000,   /* U+100CD */
  0x0000,   /* U+100CE */
  0x0000,   /* U+100CF */
  0x0000,   /* U+100D0 */
  0x0000,   /* U+100D1 */
  0x0000,   /* U+100D2 */
  0x0000,   /* U+100D3 */
  0x0000,   /* U+100D4 */
  0x0000,   /* U+100D5 */
  0x0000,   /* U+100D6 */
  0x0000,   /* U+100D7 */
  0x0000,   /* U+100D8 */
  0x0000,   /* U+100D9 */
  0x0000,   /* U+100DA */
  0x0000,   /* U+100DB */
  0x0000,   /* U+100DC */
  0x0000,   /* U+100DD */
  0x0000,   /* U+100DE */
  0x0000,   /* U+100DF */
  0x0000,   /* U+100E0 */
  0x0000,   /* U+100E1 */
  0x0000,   /* U+100E2 */
  0x0000,   /* U+100E3 */
  0x0000,   /* U+100E4 */
  0x0000,   /* U+100E5 */
  0x0000,   /* U+100E6 */
  0x0000,   /* U+100E7 */
  0x0000,   /* U+100E8 */
  0x0000,   /* U+100E9 */
  0x0000,   /* U+100EA */
  0x0000,   /* U+100EB */
  0x0000,   /* U+100EC */
  0x0000,   /* U+100ED */
  0x0000,   /* U+100EE */
  0x0000,   /* U+100EF */
  0x0000,   /* U+100F0 */
  0x0000,   /* U+100F1 */
  0x0000,   /* U+100F2 */
  0x0000,   /* U+100F3 */
  0x0000,   /* U+100F4 */
  0x0000,   /* U+100F5 */
  0x0000,   /* U+100F6 */
  0x0000,   /* U+100F7 */
  0x0000,   /* U+100F8 */
  0x0000,   /* U+100F9 */
  0x0000,   /* U+100FA */
  0x80FB,   /* U+100FB */
  0x80FC,   /* U+100FC */
  0x80FD,   /* U+100FD */
  0x80FE,   /* U+100FE */
  0x80FF,   /* U+100FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10000 */
  0x0000,   /* U+10001 */
  0x0000,   /* U+10002 */
  0x0000,   /* U+10003 */
  0x0000,   /* U+10004 */
  0x0000,   /* U+10005 */
  0x0000,   /* U+10006 */
  0x0000,   /* U+10007 */
  0x0000,   /* U+10008 */
  0x0000,   /* U+10009 */
  0x0000,   /* U+1000A */
  0x0000,   /* U+1000B */
  0x0000,   /* U+1000C */
  0x0000,   /* U+1000D */
  0x0000,   /* U+1000E */
  0x0000,   /* U+1000F */
  0x0000,   /* U+10010 */
  0x0000,   /* U+10011 */
  0x0000,   /* U+10012 */
  0x0000,   /* U+10013 */
  0x0000,   /* U+10014 */
  0x0000,   /* U+10015 */
  0x0000,   /* U+10016 */
  0x0000,   /* U+10017 */
  0x0000,   /* U+10018 */
  0x0000,   /* U+10019 */
  0x0000,   /* U+1001A */
  0x0000,   /* U+1001B */
  0x0000,   /* U+1001C */
  0x0000,   /* U+1001D */
  0x0000,   /* U+1001E */
  0x0000,   /* U+1001F */
  0x0000,   /* U+10020 */
  0x0000,   /* U+10021 */
  0x0000,   /* U+10022 */
  0x0000,   /* U+10023 */
  0x0000,   /* U+10024 */
  0x0000,   /* U+10025 */
  0x0000,   /* U+10026 */
  0x0000,   /* U+10027 */
  0x0000,   /* U+10028 */
  0x0000,   /* U+10029 */
  0x0000,   /* U+1002A */
  0x0000,   /* U+1002B */
  0x0000,   /* U+1002C */
  0x0000,   /* U+1002D */
  0x0000,   /* U+1002E */
  0x0000,   /* U+1002F */
  0x0000,   /* U+10030 */
  0x0000,   /* U+10031 */
  0x0000,   /* U+10032 */
  0x0000,   /* U+10033 */
  0x0000,   /* U+10034 */
  0x0000,   /* U+10035 */
  0x0000,   /* U+10036 */
  0x0000,   /* U+10037 */
  0x0000,   /* U+10038 */
  0x0000,   /* U+10039 */
  0x0000,   /* U+1003A */
  0x0000,   /* U+1003B */
  0x0000,   /* U+1003C */
  0x0000,   /* U+1003D */
  0x0000,   /* U+1003E */
  0x0000,   /* U+1003F */
  0x0000,   /* U+10040 */
  0x0000,   /* U+10041 */
  0x0000,   /* U+10042 */
  0x0000,   /* U+10043 */
  0x0000,   /* U+10044 */
  0x0000,   /* U+10045 */
  0x0000,   /* U+10046 */
  0x0000,   /* U+10047 */
  0x0000,   /* U+10048 */
  0x0000,   /* U+10049 */
  0x0000,   /* U+1004A */
  0x0000,   /* U+1004B */
  0x0000,   /* U+1004C */
  0x0000,   /* U+1004D */
  0x0000,   /* U+1004E */
  0x0000,   /* U+1004F */
  0x0000,   /* U+10050 */
  0x0000,   /* U+10051 */
  0x0000,   /* U+10052 */
  0x0000,   /* U+10053 */
  0x0000,   /* U+10054 */
  0x0000,   /* U+10055 */
  0x0000,   /* U+10056 */
  0x0000,   /* U+10057 */
  0x0000,   /* U+10058 */
  0x0000,   /* U+10059 */
  0x0000,   /* U+1005A */
  0x0000,   /* U+1005B */
  0x0000,   /* U+1005C */
  0x0000,   /* U+1005D */
  0x0000,   /* U+1005E */
  0x0000,   /* U+1005F */
  0x0000,   /* U+10060 */
  0x0000,   /* U+10061 */
  0x0000,   /* U+10062 */
  0x0000,   /* U+10063 */
  0x0000,   /* U+10064 */
  0x0000,   /* U+10065 */
  0x0000,   /* U+10066 */
  0x0000,   /* U+10067 */
  0x0000,   /* U+10068 */
  0x0000,   /* U+10069 */
  0x0000,   /* U+1006A */
  0x0000,   /* U+1006B */
  0x0000,   /* U+1006C */
  0x0000,   /* U+1006D */
  0x0000,   /* U+1006E */
  0x0000,   /* U+1006F */
  0x0000,   /* U+10070 */
  0x0000,   /* U+10071 */
  0x0000,   /* U+10072 */
  0x0000,   /* U+10073 */
  0x0000,   /* U+10074 */
  0x0000,   /* U+10075 */
  0x0000,   /* U+10076 */
  0x0000,   /* U+10077 */
  0x0000,   /* U+10078 */
  0x0000,   /* U+10079 */
  0x0000,   /* U+1007A */
  0x0000,   /* U+1007B */
  0x0000,   /* U+1007C */
  0x0000,   /* U+1007D */
  0x0000,   /* U+1007E */
  0x0000,   /* U+1007F */
  0x0000,   /* U+10080 */
  0x0000,   /* U+10081 */
  0x0000,   /* U+10082 */
  0x0000,   /* U+10083 */
  0x0000,   /* U+10084 */
  0x0000,   /* U+10085 */
  0x0000,   /* U+10086 */
  0x0000,   /* U+10087 */
  0x0000,   /* U+10088 */
  0x0000,   /* U+10089 */
  0x0000,   /* U+1008A */
  0x0000,   /* U+1008B */
  0x0000,   /* U+1008C */
  0x0000,   /* U+1008D */
  0x0000,   /* U+1008E */
  0x0000,   /* U+1008F */
  0x0000,   /* U+10090 */
  0x0000,   /* U+10091 */
  0x0000,   /* U+10092 */
  0x0000,   /* U+10093 */
  0x0000,   /* U+10094 */
  0x0000,   /* U+10095 */
  0x0000,   /* U+10096 */
  0x0000,   /* U+10097 */
  0x0000,   /* U+10098 */
  0x0000,   /* U+10099 */
  0x0000,   /* U+1009A */
  0x0000,   /* U+1009B */
  0x0000,   /* U+1009C */
  0x0000,   /* U+1009D */
  0x0000,   /* U+1009E */
  0x0000,   /* U+1009F */
  0x0000,   /* U+100A0 */
  0x0000,   /* U+100A1 */
  0x0000,   /* U+100A2 */
  0x0000,   /* U+100A3 */
  0x0000,   /* U+100A4 */
  0x0000,   /* U+100A5 */
  0x0000,   /* U+100A6 */
  0x0000,   /* U+100A7 */
  0x0000,   /* U+100A8 */
  0x0000,   /* U+100A9 */
  0x0000,   /* U+100AA */
  0x0000,   /* U+100AB */
  0x0000,   /* U+100AC */
  0x0000,   /* U+100AD */
  0x0000,   /* U+100AE */
  0x0000,   /* U+100AF */
  0x0000,   /* U+100B0 */
  0x0000,   /* U+100B1 */
  0x0000,   /* U+100B2 */
  0x0000,   /* U+100B3 */
  0x0000,   /* U+100B4 */
  0x0000,   /* U+100B5 */
  0x0000,   /* U+100B6 */
  0x0000,   /* U+100B7 */
  0x0000,   /* U+100B8 */
  0x0000,   /* U+100B9 */
  0x0000,   /* U+100BA */
  0x0000,   /* U+100BB */
  0x0000,   /* U+100BC */
  0x0000,   /* U+100BD */
  0x0000,   /* U+100BE */
  0x0000,   /* U+100BF */
  0x0000,   /* U+100C0 */
  0x0000,   /* U+100C1 */
  0x0000,   /* U+100C2 */
  0x0000,   /* U+100C3 */
  0x0000,   /* U+100C4 */
  0x0000,   /* U+100C5 */
  0x0000,   /* U+100C6 */
  0x0000,   /* U+100C7 */
  0x0000,   /* U+100C8 */
  0x0000,   /* U+100C9 */
  0x0000,   /* U+100CA */
  0x0000,   /* U+100CB */
  0x0000,   /* U+100CC */
  0x0000,   /* U+100CD */
  0x0000,   /* U+100CE */
  0x0000,   /* U+100CF */
  0x0000,   /* U+100D0 */
  0x0000,   /* U+100D1 */
  0x0000,   /* U+100D2 */
  0x0000,   /* U+100D3 */
  0x0000,   /* U+100D4 */
  0x0000,   /* U+100D5 */
  0x0000,   /* U+100D6 */
  0x0000,   /* U+100D7 */
  0x0000,   /* U+100D8 */
  0x0000,   /* U+100D9 */
  0x0000,   /* U+100DA */
  0x0000,   /* U+100DB */
  0x0000,   /* U+100DC */
  0x0000,   /* U+100DD */
  0x0000,   /* U+100DE */
  0x0000,   /* U+100DF */
  0x0000,   /* U+100E0 */
  0x0000,   /* U+100E1 */
  0x0000,   /* U+100E2 */
  0x0000,   /* U+100E3 */
  0x0000,   /* U+100E4 */
  0x0000,   /* U+100E5 */
  0x0000,   /* U+100E6 */
  0x0000,   /* U+100E7 */
  0x0000,   /* U+100E8 */
  0x0000,   /* U+100E9 */
  0x0000,   /* U+100EA */
  0x0000,   /* U+100EB */
  0x0000,   /* U+100EC */
  0x0000,   /* U+100ED */
  0x0000,   /* U+100EE */
  0x0000,   /* U+100EF */
  0x0000,   /* U+100F0 */
  0x0000,   /* U+100F1 */
  0x0000,   /* U+100F2 */
  0x0000,   /* U+100F3 */
  0x0000,   /* U+100F4 */
  0x0000,   /* U+100F5 */
  0x0000,   /* U+100F6 */
  0x0000,   /* U+100F7 */
  0x0000,   /* U+100F8 */
  0x0000,   /* U+100F9 */
  0x0000,   /* U+100FA */
  0x0000,   /* U+100FB */
  0x0000,   /* U+100FC */
  0x0000,   /* U+100FD */
  0x0000,   /* U+100FE */
  0x0000,   /* U+100FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10000 */
  0x0000,   /* U+10001 */
  0x0000,   /* U+10002 */
  0x0000,   /* U+10003 */
  0x0000,   /* U+10004 */
  0x0000,   /* U+10005 */
  0x0000,   /* U+10006 */
  0x0000,   /* U+10007 */
  0x0000,   /* U+10008 */
  0x0000,   /* U+10009 */
  0x0000,   /* U+1000A */
  0x0000,   /* U+1000B */
  0x0000,   /* U+1000C */
  0x0000,   /* U+1000D */
  0x0000,   /* U+1000E */
  0x0000,   /* U+1000F */
  0x0000,   /* U+10010 */
  0x0000,   /* U+10011 */
  0x0000,   /* U+10012 */
  0x0000,   /* U+10013 */
  0x0000,   /* U+10014 */
  0x0000,   /* U+10015 */
  0x0000,   /* U+10016 */
  0x0000,   /* U+10017 */
  0x0000,   /* U+10018 */
  0x0000,   /* U+10019 */
  0x0000,   /* U+1001A */
  0x0000,   /* U+1001B */
  0x0000,   /* U+1001C */
  0x0000,   /* U+1001D */
  0x0000,   /* U+1001E */
  0x0000,   /* U+1001F */
  0x0000,   /* U+10020 */
  0x0000,   /* U+10021 */
  0x0000,   /* U+10022 */
  0x0000,   /* U+10023 */
  0x0000,   /* U+10024 */
  0x0000,   /* U+10025 */
  0x0000,   /* U+10026 */
  0x0000,   /* U+10027 */
  0x0000,   /* U+10028 */
  0x0000,   /* U+10029 */
  0x0000,   /* U+1002A */
  0x0000,   /* U+1002B */
  0x0000,   /* U+1002C */
  0x0000,   /* U+1002D */
  0x0000,   /* U+1002E */
  0x0000,   /* U+1002F */
  0x0000,   /* U+10030 */
  0x0000,   /* U+10031 */
  0x0000,   /* U+10032 */
  0x0000,   /* U+10033 */
  0x0000,   /* U+10034 */
  0x0000,   /* U+10035 */
  0x0000,   /* U+10036 */
  0x0000,   /* U+10037 */
  0x0000,   /* U+10038 */
  0x0000,   /* U+10039 */
  0x0000,   /* U+1003A */
  0x0000,   /* U+1003B */
  0x0000,   /* U+1003C */
  0x0000,   /* U+1003D */
  0x0000,   /* U+1003E */
  0x0000,   /* U+1003F */
  0x0000,   /* U+10040 */
  0x0000,   /* U+10041 */
  0x0000,   /* U+10042 */
  0x0000,   /* U+10043 */
  0x0000,   /* U+10044 */
  0x0000,   /* U+10045 */
  0x0000,   /* U+10046 */
  0x0000,   /* U+10047 */
  0x0000,   /* U+10048 */
  0x0000,   /* U+10049 */
  0x0000,   /* U+1004A */
  0x0000,   /* U+1004B */
  0x0000,   /* U+1004C */
  0x0000,   /* U+1004D */
  0x0000,   /* U+1004E */
  0x0000,   /* U+1004F */
  0x0000,   /* U+10050 */
  0x0000,   /* U+10051 */
  0x0000,   /* U+10052 */
  0x0000,   /* U+10053 */
  0x0000,   /* U+10054 */
  0x0000,   /* U+10055 */
  0x0000,   /* U+10056 */
  0x0000,   /* U+10057 */
  0x0000,   /* U+10058 */
  0x0000,   /* U+10059 */
  0x0000,   /* U+1005A */
  0x0000,   /* U+1005B */
  0x0000,   /* U+1005C */
  0x0000,   /* U+1005D */
  0x0000,   /* U+1005E */
  0x0000,   /* U+1005F */
  0x0000,   /* U+10060 */
  0x0000,   /* U+10061 */
  0x0000,   /* U+10062 */
  0x0000,   /* U+10063 */
  0x0000,   /* U+10064 */
  0x0000,   /* U+10065 */
  0x0000,   /* U+10066 */
  0x0000,   /* U+10067 */
  0x0000,   /* U+10068 */
  0x0000,   /* U+10069 */
  0x0000,   /* U+1006A */
  0x0000,   /* U+1006B */
  0x0000,   /* U+1006C */
  0x0000,   /* U+1006D */
  0x0000,   /* U+1006E */
  0x0000,   /* U+1006F */
  0x0000,   /* U+10070 */
  0x0000,   /* U+10071 */
  0x0000,   /* U+10072 */
  0x0000,   /* U+10073 */
  0x0000,   /* U+10074 */
  0x0000,   /* U+10075 */
  0x0000,   /* U+10076 */
  0x0000,   /* U+10077 */
  0x0000,   /* U+10078 */
  0x0000,   /* U+10079 */
  0x0000,   /* U+1007A */
  0x0000,   /* U+1007B */
  0x0000,   /* U+1007C */
  0x0000,   /* U+1007D */
  0x0000,   /* U+1007E */
  0x0000,   /* U+1007F */
  0x0000,   /* U+10080 */
  0x0000,   /* U+10081 */
  0x0000,   /* U+10082 */
  0x0000,   /* U+10083 */
  0x0000,   /* U+10084 */
  0x0000,   /* U+10085 */
  0x0000,   /* U+10086 */
  0x0000,   /* U+10087 */
  0x0000,   /* U+10088 */
  0x0000,   /* U+10089 */
  0x0000,   /* U+1008A */
  0x0000,   /* U+1008B */
  0x0000,   /* U+1008C */
  0x0000,   /* U+1008D */
  0x0000,   /* U+1008E */
  0x0000,   /* U+1008F */
  0x0000,   /* U+10090 */
  0x0000,   /* U+10091 */
  0x0000,   /* U+10092 */
  0x0000,   /* U+10093 */
  0x0000,   /* U+10094 */
  0x0000,   /* U+10095 */
  0x0000,   /* U+10096 */
  0x0000,   /* U+10097 */
  0x0000,   /* U+10098 */
  0x0000,   /* U+10099 */
  0x0000,   /* U+1009A */
  0x0000,   /* U+1009B */
  0x0000,   /* U+1009C */
  0x0000,   /* U+1009D */
  0x0000,   /* U+1009E */
  0x0000,   /* U+1009F */
  0x0000,   /* U+100A0 */
  0x0000,   /* U+100A1 */
  0x0000,   /* U+100A2 */
  0x0000,   /* U+100A3 */
  0x0000,   /* U+100A4 */
  0x0000,   /* U+100A5 */
  0x0000,   /* U+100A6 */
  0x0000,   /* U+100A7 */
  0x0000,   /* U+100A8 */
  0x0000,   /* U+100A9 */
  0x0000,   /* U+100AA */
  0x0000,   /* U+100AB */
  0x0000,   /* U+100AC */
  0x0000,   /* U+100AD */
  0x0000,   /* U+100AE */
  0x0000,   /* U+100AF */
  0x0000,   /* U+100B0 */
  0x0000,   /* U+100B1 */
  0x0000,   /* U+100B2 */
  0x0000,   /* U+100B3 */
  0x0000,   /* U+100B4 */
  0x0000,   /* U+100B5 */
  0x0000,   /* U+100B6 */
  0x0000,   /* U+100B7 */
  0x0000,   /* U+100B8 */
  0x0000,   /* U+100B9 */
  0x0000,   /* U+100BA */
  0x0000,   /* U+100BB */
  0x0000,   /* U+100BC */
  0x0000,   /* U+100BD */
  0x0000,   /* U+100BE */
  0x0000,   /* U+100BF */
  0x0000,   /* U+100C0 */
  0x0000,   /* U+100C1 */
  0x0000,   /* U+100C2 */
  0x0000,   /* U+100C3 */
  0x0000,   /* U+100C4 */
  0x0000,   /* U+100C5 */
  0x0000,   /* U+100C6 */
  0x0000,   /* U+100C7 */
  0x0000,   /* U+100C8 */
  0x0000,   /* U+100C9 */
  0x0000,   /* U+100CA */
  0x0000,   /* U+100CB */
  0x0000,   /* U+100CC */
  0x0000,   /* U+100CD */
  0x0000,   /* U+100CE */
  0x0000,   /* U+100CF */
  0x0000,   /* U+100D0 */
  0x0000,   /* U+100D1 */
  0x0000,   /* U+100D2 */
  0x0000,   /* U+100D3 */
  0x0000,   /* U+100D4 */
  0x0000,   /* U+100D5 */
  0x0000,   /* U+100D6 */
  0x0000,   /* U+100D7 */
  0x0000,   /* U+100D8 */
  0x0000,   /* U+100D9 */
  0x0000,   /* U+100DA */
  0x0000,   /* U+100DB */
  0x0000,   /* U+100DC */
  0x0000,   /* U+100DD */
  0x0000,   /* U+100DE */
  0x0000,   /* U+100DF */
  0x0000,   /* U+100E0 */
  0x0000,   /* U+100E1 */
  0x0000,   /* U+100E2 */
  0x0000,   /* U+100E3 */
  0x0000,   /* U+100E4 */
  0x0000,   /* U+100E5 */
  0x0000,   /* U+100E6 */
  0x0000,   /* U+100E7 */
  0x0000,   /* U+100E8 */
  0x0000,   /* U+100E9 */
  0x0000,   /* U+100EA */
  0x0000,   /* U+100EB */
  0x0000,   /* U+100EC */
  0x0000,   /* U+100ED */
  0x0000,   /* U+100EE */
  0x0000,   /* U+100EF */
  0x0000,   /* U+100F0 */
  0x0000,   /* U+100F1 */
  0x0000,   /* U+100F2 */
  0x0000,   /* U+100F3 */
  0x0000,   /* U+100F4 */
  0x0000,   /* U+100F5 */
  0x0000,   /* U+100F6 */
  0x0000,   /* U+100F7 */
  0x0000,   /* U+100F8 */
  0x0000,   /* U+100F9 */
  0x0000,   /* U+100FA */
  0x0000,   /* U+100FB */
  0x0000,   /* U+100FC */
  0x0000,   /* U+100FD */
  0x0000,   /* U+100FE */
  0x0000,   /* U+100FF */
};

uint16 uca900_p101[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 

  /* Primary weight 1 for each character. */
  0x02FA,   /* U+10100 */
  0x02FB,   /* U+10101 */
  0x02FC,   /* U+10102 */
  0xFBC2,   /* U+10103 */
  0xFBC2,   /* U+10104 */
  0xFBC2,   /* U+10105 */
  0xFBC2,   /* U+10106 */
  0x1C3E,   /* U+10107 */
  0x1C3F,   /* U+10108 */
  0x1C40,   /* U+10109 */
  0x1C41,   /* U+1010A */
  0x1C42,   /* U+1010B */
  0x1C43,   /* U+1010C */
  0x1C44,   /* U+1010D */
  0x1C45,   /* U+1010E */
  0x1C46,   /* U+1010F */
  0x1AE9,   /* U+10110 */
  0x1AEA,   /* U+10111 */
  0x1AEB,   /* U+10112 */
  0x1AEC,   /* U+10113 */
  0x1AED,   /* U+10114 */
  0x1AEE,   /* U+10115 */
  0x1AEF,   /* U+10116 */
  0x1AF0,   /* U+10117 */
  0x1AF1,   /* U+10118 */
  0x1AF2,   /* U+10119 */
  0x1AF3,   /* U+1011A */
  0x1AF4,   /* U+1011B */
  0x1AF5,   /* U+1011C */
  0x1AF6,   /* U+1011D */
  0x1AF7,   /* U+1011E */
  0x1AF8,   /* U+1011F */
  0x1AF9,   /* U+10120 */
  0x1AFA,   /* U+10121 */
  0x1AFB,   /* U+10122 */
  0x1AFC,   /* U+10123 */
  0x1AFD,   /* U+10124 */
  0x1AFE,   /* U+10125 */
  0x1AFF,   /* U+10126 */
  0x1B00,   /* U+10127 */
  0x1B01,   /* U+10128 */
  0x1B02,   /* U+10129 */
  0x1B03,   /* U+1012A */
  0x1B04,   /* U+1012B */
  0x1B05,   /* U+1012C */
  0x1B06,   /* U+1012D */
  0x1B07,   /* U+1012E */
  0x1B08,   /* U+1012F */
  0x1B09,   /* U+10130 */
  0x1B0A,   /* U+10131 */
  0x1B0B,   /* U+10132 */
  0x1B0C,   /* U+10133 */
  0xFBC2,   /* U+10134 */
  0xFBC2,   /* U+10135 */
  0xFBC2,   /* U+10136 */
  0x0F78,   /* U+10137 */
  0x0F79,   /* U+10138 */
  0x0F7A,   /* U+10139 */
  0x0F7B,   /* U+1013A */
  0x0F7C,   /* U+1013B */
  0x0F7D,   /* U+1013C */
  0x0F7E,   /* U+1013D */
  0x0F7F,   /* U+1013E */
  0x0F80,   /* U+1013F */
  0x1B0D,   /* U+10140 */
  0x1B0E,   /* U+10141 */
  0x1C3E,   /* U+10142 */
  0x1C42,   /* U+10143 */
  0x1B0F,   /* U+10144 */
  0x1B10,   /* U+10145 */
  0x1B11,   /* U+10146 */
  0x1B12,   /* U+10147 */
  0x1C42,   /* U+10148 */
  0x1B13,   /* U+10149 */
  0x1B14,   /* U+1014A */
  0x1B15,   /* U+1014B */
  0x1B16,   /* U+1014C */
  0x1B17,   /* U+1014D */
  0x1B18,   /* U+1014E */
  0x1C42,   /* U+1014F */
  0x1B19,   /* U+10150 */
  0x1B1A,   /* U+10151 */
  0x1B1B,   /* U+10152 */
  0x1B1C,   /* U+10153 */
  0x1B1D,   /* U+10154 */
  0x1B1E,   /* U+10155 */
  0x1B1F,   /* U+10156 */
  0x1B20,   /* U+10157 */
  0x1C3E,   /* U+10158 */
  0x1C3E,   /* U+10159 */
  0x1C3E,   /* U+1015A */
  0x1C3F,   /* U+1015B */
  0x1C3F,   /* U+1015C */
  0x1C3F,   /* U+1015D */
  0x1C3F,   /* U+1015E */
  0x1C42,   /* U+1015F */
  0x1B21,   /* U+10160 */
  0x1B22,   /* U+10161 */
  0x1B23,   /* U+10162 */
  0x1B24,   /* U+10163 */
  0x1B25,   /* U+10164 */
  0x1B26,   /* U+10165 */
  0x1B27,   /* U+10166 */
  0x1B28,   /* U+10167 */
  0x1B29,   /* U+10168 */
  0x1B2A,   /* U+10169 */
  0x1B2B,   /* U+1016A */
  0x1B2C,   /* U+1016B */
  0x1B2D,   /* U+1016C */
  0x1B2E,   /* U+1016D */
  0x1B2F,   /* U+1016E */
  0x1B30,   /* U+1016F */
  0x1B31,   /* U+10170 */
  0x1B32,   /* U+10171 */
  0x1B33,   /* U+10172 */
  0x1C42,   /* U+10173 */
  0x1B34,   /* U+10174 */
  0x1B35,   /* U+10175 */
  0x1B36,   /* U+10176 */
  0x1B37,   /* U+10177 */
  0x1B38,   /* U+10178 */
  0x0F81,   /* U+10179 */
  0x0F82,   /* U+1017A */
  0x0F83,   /* U+1017B */
  0x0F84,   /* U+1017C */
  0x0F85,   /* U+1017D */
  0x0F86,   /* U+1017E */
  0x0F87,   /* U+1017F */
  0x0F88,   /* U+10180 */
  0x0F89,   /* U+10181 */
  0x0F8A,   /* U+10182 */
  0x0F8B,   /* U+10183 */
  0x0F8C,   /* U+10184 */
  0x0F8D,   /* U+10185 */
  0x0F8E,   /* U+10186 */
  0x0F8F,   /* U+10187 */
  0x0F90,   /* U+10188 */
  0x0F91,   /* U+10189 */
  0x1C3D,   /* U+1018A */
  0x1B39,   /* U+1018B */
  0x0F92,   /* U+1018C */
  0x0F93,   /* U+1018D */
  0x0F94,   /* U+1018E */
  0xFBC2,   /* U+1018F */
  0x0F95,   /* U+10190 */
  0x0F96,   /* U+10191 */
  0x0F97,   /* U+10192 */
  0x0F98,   /* U+10193 */
  0x0F99,   /* U+10194 */
  0x0F9A,   /* U+10195 */
  0x0F9B,   /* U+10196 */
  0x0F9C,   /* U+10197 */
  0x0F9D,   /* U+10198 */
  0x0F9E,   /* U+10199 */
  0x0F9F,   /* U+1019A */
  0x0FA0,   /* U+1019B */
  0xFBC2,   /* U+1019C */
  0xFBC2,   /* U+1019D */
  0xFBC2,   /* U+1019E */
  0xFBC2,   /* U+1019F */
  0x0FA1,   /* U+101A0 */
  0xFBC2,   /* U+101A1 */
  0xFBC2,   /* U+101A2 */
  0xFBC2,   /* U+101A3 */
  0xFBC2,   /* U+101A4 */
  0xFBC2,   /* U+101A5 */
  0xFBC2,   /* U+101A6 */
  0xFBC2,   /* U+101A7 */
  0xFBC2,   /* U+101A8 */
  0xFBC2,   /* U+101A9 */
  0xFBC2,   /* U+101AA */
  0xFBC2,   /* U+101AB */
  0xFBC2,   /* U+101AC */
  0xFBC2,   /* U+101AD */
  0xFBC2,   /* U+101AE */
  0xFBC2,   /* U+101AF */
  0xFBC2,   /* U+101B0 */
  0xFBC2,   /* U+101B1 */
  0xFBC2,   /* U+101B2 */
  0xFBC2,   /* U+101B3 */
  0xFBC2,   /* U+101B4 */
  0xFBC2,   /* U+101B5 */
  0xFBC2,   /* U+101B6 */
  0xFBC2,   /* U+101B7 */
  0xFBC2,   /* U+101B8 */
  0xFBC2,   /* U+101B9 */
  0xFBC2,   /* U+101BA */
  0xFBC2,   /* U+101BB */
  0xFBC2,   /* U+101BC */
  0xFBC2,   /* U+101BD */
  0xFBC2,   /* U+101BE */
  0xFBC2,   /* U+101BF */
  0xFBC2,   /* U+101C0 */
  0xFBC2,   /* U+101C1 */
  0xFBC2,   /* U+101C2 */
  0xFBC2,   /* U+101C3 */
  0xFBC2,   /* U+101C4 */
  0xFBC2,   /* U+101C5 */
  0xFBC2,   /* U+101C6 */
  0xFBC2,   /* U+101C7 */
  0xFBC2,   /* U+101C8 */
  0xFBC2,   /* U+101C9 */
  0xFBC2,   /* U+101CA */
  0xFBC2,   /* U+101CB */
  0xFBC2,   /* U+101CC */
  0xFBC2,   /* U+101CD */
  0xFBC2,   /* U+101CE */
  0xFBC2,   /* U+101CF */
  0x0FA2,   /* U+101D0 */
  0x0FA3,   /* U+101D1 */
  0x0FA4,   /* U+101D2 */
  0x0FA5,   /* U+101D3 */
  0x0FA6,   /* U+101D4 */
  0x0FA7,   /* U+101D5 */
  0x0FA8,   /* U+101D6 */
  0x0FA9,   /* U+101D7 */
  0x0FAA,   /* U+101D8 */
  0x0FAB,   /* U+101D9 */
  0x0FAC,   /* U+101DA */
  0x0FAD,   /* U+101DB */
  0x0FAE,   /* U+101DC */
  0x0FAF,   /* U+101DD */
  0x0FB0,   /* U+101DE */
  0x0FB1,   /* U+101DF */
  0x0FB2,   /* U+101E0 */
  0x0FB3,   /* U+101E1 */
  0x0FB4,   /* U+101E2 */
  0x0FB5,   /* U+101E3 */
  0x0FB6,   /* U+101E4 */
  0x0FB7,   /* U+101E5 */
  0x0FB8,   /* U+101E6 */
  0x0FB9,   /* U+101E7 */
  0x0FBA,   /* U+101E8 */
  0x0FBB,   /* U+101E9 */
  0x0FBC,   /* U+101EA */
  0x0FBD,   /* U+101EB */
  0x0FBE,   /* U+101EC */
  0x0FBF,   /* U+101ED */
  0x0FC0,   /* U+101EE */
  0x0FC1,   /* U+101EF */
  0x0FC2,   /* U+101F0 */
  0x0FC3,   /* U+101F1 */
  0x0FC4,   /* U+101F2 */
  0x0FC5,   /* U+101F3 */
  0x0FC6,   /* U+101F4 */
  0x0FC7,   /* U+101F5 */
  0x0FC8,   /* U+101F6 */
  0x0FC9,   /* U+101F7 */
  0x0FCA,   /* U+101F8 */
  0x0FCB,   /* U+101F9 */
  0x0FCC,   /* U+101FA */
  0x0FCD,   /* U+101FB */
  0x0FCE,   /* U+101FC */
  0x0000,   /* U+101FD */
  0xFBC2,   /* U+101FE */
  0xFBC2,   /* U+101FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10100 */
  0x0020,   /* U+10101 */
  0x0020,   /* U+10102 */
  0x0020,   /* U+10103 */
  0x0020,   /* U+10104 */
  0x0020,   /* U+10105 */
  0x0020,   /* U+10106 */
  0x0020,   /* U+10107 */
  0x0020,   /* U+10108 */
  0x0020,   /* U+10109 */
  0x0020,   /* U+1010A */
  0x0020,   /* U+1010B */
  0x0020,   /* U+1010C */
  0x0020,   /* U+1010D */
  0x0020,   /* U+1010E */
  0x0020,   /* U+1010F */
  0x0020,   /* U+10110 */
  0x0020,   /* U+10111 */
  0x0020,   /* U+10112 */
  0x0020,   /* U+10113 */
  0x0020,   /* U+10114 */
  0x0020,   /* U+10115 */
  0x0020,   /* U+10116 */
  0x0020,   /* U+10117 */
  0x0020,   /* U+10118 */
  0x0020,   /* U+10119 */
  0x0020,   /* U+1011A */
  0x0020,   /* U+1011B */
  0x0020,   /* U+1011C */
  0x0020,   /* U+1011D */
  0x0020,   /* U+1011E */
  0x0020,   /* U+1011F */
  0x0020,   /* U+10120 */
  0x0020,   /* U+10121 */
  0x0020,   /* U+10122 */
  0x0020,   /* U+10123 */
  0x0020,   /* U+10124 */
  0x0020,   /* U+10125 */
  0x0020,   /* U+10126 */
  0x0020,   /* U+10127 */
  0x0020,   /* U+10128 */
  0x0020,   /* U+10129 */
  0x0020,   /* U+1012A */
  0x0020,   /* U+1012B */
  0x0020,   /* U+1012C */
  0x0020,   /* U+1012D */
  0x0020,   /* U+1012E */
  0x0020,   /* U+1012F */
  0x0020,   /* U+10130 */
  0x0020,   /* U+10131 */
  0x0020,   /* U+10132 */
  0x0020,   /* U+10133 */
  0x0020,   /* U+10134 */
  0x0020,   /* U+10135 */
  0x0020,   /* U+10136 */
  0x0020,   /* U+10137 */
  0x0020,   /* U+10138 */
  0x0020,   /* U+10139 */
  0x0020,   /* U+1013A */
  0x0020,   /* U+1013B */
  0x0020,   /* U+1013C */
  0x0020,   /* U+1013D */
  0x0020,   /* U+1013E */
  0x0020,   /* U+1013F */
  0x0020,   /* U+10140 */
  0x0020,   /* U+10141 */
  0x0020,   /* U+10142 */
  0x0020,   /* U+10143 */
  0x0020,   /* U+10144 */
  0x0020,   /* U+10145 */
  0x0020,   /* U+10146 */
  0x0020,   /* U+10147 */
  0x0020,   /* U+10148 */
  0x0020,   /* U+10149 */
  0x0020,   /* U+1014A */
  0x0020,   /* U+1014B */
  0x0020,   /* U+1014C */
  0x0020,   /* U+1014D */
  0x0020,   /* U+1014E */
  0x0020,   /* U+1014F */
  0x0020,   /* U+10150 */
  0x0020,   /* U+10151 */
  0x0020,   /* U+10152 */
  0x0020,   /* U+10153 */
  0x0020,   /* U+10154 */
  0x0020,   /* U+10155 */
  0x0020,   /* U+10156 */
  0x0020,   /* U+10157 */
  0x0020,   /* U+10158 */
  0x0020,   /* U+10159 */
  0x0020,   /* U+1015A */
  0x0020,   /* U+1015B */
  0x0020,   /* U+1015C */
  0x0020,   /* U+1015D */
  0x0020,   /* U+1015E */
  0x0020,   /* U+1015F */
  0x0020,   /* U+10160 */
  0x0020,   /* U+10161 */
  0x0020,   /* U+10162 */
  0x0020,   /* U+10163 */
  0x0020,   /* U+10164 */
  0x0020,   /* U+10165 */
  0x0020,   /* U+10166 */
  0x0020,   /* U+10167 */
  0x0020,   /* U+10168 */
  0x0020,   /* U+10169 */
  0x0020,   /* U+1016A */
  0x0020,   /* U+1016B */
  0x0020,   /* U+1016C */
  0x0020,   /* U+1016D */
  0x0020,   /* U+1016E */
  0x0020,   /* U+1016F */
  0x0020,   /* U+10170 */
  0x0020,   /* U+10171 */
  0x0020,   /* U+10172 */
  0x0020,   /* U+10173 */
  0x0020,   /* U+10174 */
  0x0020,   /* U+10175 */
  0x0020,   /* U+10176 */
  0x0020,   /* U+10177 */
  0x0020,   /* U+10178 */
  0x0020,   /* U+10179 */
  0x0020,   /* U+1017A */
  0x0020,   /* U+1017B */
  0x0020,   /* U+1017C */
  0x0020,   /* U+1017D */
  0x0020,   /* U+1017E */
  0x0020,   /* U+1017F */
  0x0020,   /* U+10180 */
  0x0020,   /* U+10181 */
  0x0020,   /* U+10182 */
  0x0020,   /* U+10183 */
  0x0020,   /* U+10184 */
  0x0020,   /* U+10185 */
  0x0020,   /* U+10186 */
  0x0020,   /* U+10187 */
  0x0020,   /* U+10188 */
  0x0020,   /* U+10189 */
  0x0020,   /* U+1018A */
  0x0020,   /* U+1018B */
  0x0020,   /* U+1018C */
  0x0020,   /* U+1018D */
  0x0020,   /* U+1018E */
  0x0020,   /* U+1018F */
  0x0020,   /* U+10190 */
  0x0020,   /* U+10191 */
  0x0020,   /* U+10192 */
  0x0020,   /* U+10193 */
  0x0020,   /* U+10194 */
  0x0020,   /* U+10195 */
  0x0020,   /* U+10196 */
  0x0020,   /* U+10197 */
  0x0020,   /* U+10198 */
  0x0020,   /* U+10199 */
  0x0020,   /* U+1019A */
  0x0020,   /* U+1019B */
  0x0020,   /* U+1019C */
  0x0020,   /* U+1019D */
  0x0020,   /* U+1019E */
  0x0020,   /* U+1019F */
  0x0020,   /* U+101A0 */
  0x0020,   /* U+101A1 */
  0x0020,   /* U+101A2 */
  0x0020,   /* U+101A3 */
  0x0020,   /* U+101A4 */
  0x0020,   /* U+101A5 */
  0x0020,   /* U+101A6 */
  0x0020,   /* U+101A7 */
  0x0020,   /* U+101A8 */
  0x0020,   /* U+101A9 */
  0x0020,   /* U+101AA */
  0x0020,   /* U+101AB */
  0x0020,   /* U+101AC */
  0x0020,   /* U+101AD */
  0x0020,   /* U+101AE */
  0x0020,   /* U+101AF */
  0x0020,   /* U+101B0 */
  0x0020,   /* U+101B1 */
  0x0020,   /* U+101B2 */
  0x0020,   /* U+101B3 */
  0x0020,   /* U+101B4 */
  0x0020,   /* U+101B5 */
  0x0020,   /* U+101B6 */
  0x0020,   /* U+101B7 */
  0x0020,   /* U+101B8 */
  0x0020,   /* U+101B9 */
  0x0020,   /* U+101BA */
  0x0020,   /* U+101BB */
  0x0020,   /* U+101BC */
  0x0020,   /* U+101BD */
  0x0020,   /* U+101BE */
  0x0020,   /* U+101BF */
  0x0020,   /* U+101C0 */
  0x0020,   /* U+101C1 */
  0x0020,   /* U+101C2 */
  0x0020,   /* U+101C3 */
  0x0020,   /* U+101C4 */
  0x0020,   /* U+101C5 */
  0x0020,   /* U+101C6 */
  0x0020,   /* U+101C7 */
  0x0020,   /* U+101C8 */
  0x0020,   /* U+101C9 */
  0x0020,   /* U+101CA */
  0x0020,   /* U+101CB */
  0x0020,   /* U+101CC */
  0x0020,   /* U+101CD */
  0x0020,   /* U+101CE */
  0x0020,   /* U+101CF */
  0x0020,   /* U+101D0 */
  0x0020,   /* U+101D1 */
  0x0020,   /* U+101D2 */
  0x0020,   /* U+101D3 */
  0x0020,   /* U+101D4 */
  0x0020,   /* U+101D5 */
  0x0020,   /* U+101D6 */
  0x0020,   /* U+101D7 */
  0x0020,   /* U+101D8 */
  0x0020,   /* U+101D9 */
  0x0020,   /* U+101DA */
  0x0020,   /* U+101DB */
  0x0020,   /* U+101DC */
  0x0020,   /* U+101DD */
  0x0020,   /* U+101DE */
  0x0020,   /* U+101DF */
  0x0020,   /* U+101E0 */
  0x0020,   /* U+101E1 */
  0x0020,   /* U+101E2 */
  0x0020,   /* U+101E3 */
  0x0020,   /* U+101E4 */
  0x0020,   /* U+101E5 */
  0x0020,   /* U+101E6 */
  0x0020,   /* U+101E7 */
  0x0020,   /* U+101E8 */
  0x0020,   /* U+101E9 */
  0x0020,   /* U+101EA */
  0x0020,   /* U+101EB */
  0x0020,   /* U+101EC */
  0x0020,   /* U+101ED */
  0x0020,   /* U+101EE */
  0x0020,   /* U+101EF */
  0x0020,   /* U+101F0 */
  0x0020,   /* U+101F1 */
  0x0020,   /* U+101F2 */
  0x0020,   /* U+101F3 */
  0x0020,   /* U+101F4 */
  0x0020,   /* U+101F5 */
  0x0020,   /* U+101F6 */
  0x0020,   /* U+101F7 */
  0x0020,   /* U+101F8 */
  0x0020,   /* U+101F9 */
  0x0020,   /* U+101FA */
  0x0020,   /* U+101FB */
  0x0020,   /* U+101FC */
  0x010F,   /* U+101FD */
  0x0020,   /* U+101FE */
  0x0020,   /* U+101FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10100 */
  0x0002,   /* U+10101 */
  0x0002,   /* U+10102 */
  0x0002,   /* U+10103 */
  0x0002,   /* U+10104 */
  0x0002,   /* U+10105 */
  0x0002,   /* U+10106 */
  0x0002,   /* U+10107 */
  0x0002,   /* U+10108 */
  0x0002,   /* U+10109 */
  0x0002,   /* U+1010A */
  0x0002,   /* U+1010B */
  0x0002,   /* U+1010C */
  0x0002,   /* U+1010D */
  0x0002,   /* U+1010E */
  0x0002,   /* U+1010F */
  0x0002,   /* U+10110 */
  0x0002,   /* U+10111 */
  0x0002,   /* U+10112 */
  0x0002,   /* U+10113 */
  0x0002,   /* U+10114 */
  0x0002,   /* U+10115 */
  0x0002,   /* U+10116 */
  0x0002,   /* U+10117 */
  0x0002,   /* U+10118 */
  0x0002,   /* U+10119 */
  0x0002,   /* U+1011A */
  0x0002,   /* U+1011B */
  0x0002,   /* U+1011C */
  0x0002,   /* U+1011D */
  0x0002,   /* U+1011E */
  0x0002,   /* U+1011F */
  0x0002,   /* U+10120 */
  0x0002,   /* U+10121 */
  0x0002,   /* U+10122 */
  0x0002,   /* U+10123 */
  0x0002,   /* U+10124 */
  0x0002,   /* U+10125 */
  0x0002,   /* U+10126 */
  0x0002,   /* U+10127 */
  0x0002,   /* U+10128 */
  0x0002,   /* U+10129 */
  0x0002,   /* U+1012A */
  0x0002,   /* U+1012B */
  0x0002,   /* U+1012C */
  0x0002,   /* U+1012D */
  0x0002,   /* U+1012E */
  0x0002,   /* U+1012F */
  0x0002,   /* U+10130 */
  0x0002,   /* U+10131 */
  0x0002,   /* U+10132 */
  0x0002,   /* U+10133 */
  0x0002,   /* U+10134 */
  0x0002,   /* U+10135 */
  0x0002,   /* U+10136 */
  0x0002,   /* U+10137 */
  0x0002,   /* U+10138 */
  0x0002,   /* U+10139 */
  0x0002,   /* U+1013A */
  0x0002,   /* U+1013B */
  0x0002,   /* U+1013C */
  0x0002,   /* U+1013D */
  0x0002,   /* U+1013E */
  0x0002,   /* U+1013F */
  0x0002,   /* U+10140 */
  0x0002,   /* U+10141 */
  0x0002,   /* U+10142 */
  0x0002,   /* U+10143 */
  0x0002,   /* U+10144 */
  0x0002,   /* U+10145 */
  0x0002,   /* U+10146 */
  0x0002,   /* U+10147 */
  0x0002,   /* U+10148 */
  0x0002,   /* U+10149 */
  0x0002,   /* U+1014A */
  0x0002,   /* U+1014B */
  0x0002,   /* U+1014C */
  0x0002,   /* U+1014D */
  0x0002,   /* U+1014E */
  0x0002,   /* U+1014F */
  0x0002,   /* U+10150 */
  0x0002,   /* U+10151 */
  0x0002,   /* U+10152 */
  0x0002,   /* U+10153 */
  0x0002,   /* U+10154 */
  0x0002,   /* U+10155 */
  0x0002,   /* U+10156 */
  0x0002,   /* U+10157 */
  0x0002,   /* U+10158 */
  0x0002,   /* U+10159 */
  0x0002,   /* U+1015A */
  0x0002,   /* U+1015B */
  0x0002,   /* U+1015C */
  0x0002,   /* U+1015D */
  0x0002,   /* U+1015E */
  0x0002,   /* U+1015F */
  0x0002,   /* U+10160 */
  0x0002,   /* U+10161 */
  0x0002,   /* U+10162 */
  0x0002,   /* U+10163 */
  0x0002,   /* U+10164 */
  0x0002,   /* U+10165 */
  0x0002,   /* U+10166 */
  0x0002,   /* U+10167 */
  0x0002,   /* U+10168 */
  0x0002,   /* U+10169 */
  0x0002,   /* U+1016A */
  0x0002,   /* U+1016B */
  0x0002,   /* U+1016C */
  0x0002,   /* U+1016D */
  0x0002,   /* U+1016E */
  0x0002,   /* U+1016F */
  0x0002,   /* U+10170 */
  0x0002,   /* U+10171 */
  0x0002,   /* U+10172 */
  0x0002,   /* U+10173 */
  0x0002,   /* U+10174 */
  0x0002,   /* U+10175 */
  0x0002,   /* U+10176 */
  0x0002,   /* U+10177 */
  0x0002,   /* U+10178 */
  0x0002,   /* U+10179 */
  0x0002,   /* U+1017A */
  0x0002,   /* U+1017B */
  0x0002,   /* U+1017C */
  0x0002,   /* U+1017D */
  0x0002,   /* U+1017E */
  0x0002,   /* U+1017F */
  0x0002,   /* U+10180 */
  0x0002,   /* U+10181 */
  0x0002,   /* U+10182 */
  0x0002,   /* U+10183 */
  0x0002,   /* U+10184 */
  0x0002,   /* U+10185 */
  0x0002,   /* U+10186 */
  0x0002,   /* U+10187 */
  0x0002,   /* U+10188 */
  0x0002,   /* U+10189 */
  0x0002,   /* U+1018A */
  0x0002,   /* U+1018B */
  0x0002,   /* U+1018C */
  0x0002,   /* U+1018D */
  0x0002,   /* U+1018E */
  0x0002,   /* U+1018F */
  0x0002,   /* U+10190 */
  0x0002,   /* U+10191 */
  0x0002,   /* U+10192 */
  0x0002,   /* U+10193 */
  0x0002,   /* U+10194 */
  0x0002,   /* U+10195 */
  0x0002,   /* U+10196 */
  0x0002,   /* U+10197 */
  0x0002,   /* U+10198 */
  0x0002,   /* U+10199 */
  0x0002,   /* U+1019A */
  0x0002,   /* U+1019B */
  0x0002,   /* U+1019C */
  0x0002,   /* U+1019D */
  0x0002,   /* U+1019E */
  0x0002,   /* U+1019F */
  0x0002,   /* U+101A0 */
  0x0002,   /* U+101A1 */
  0x0002,   /* U+101A2 */
  0x0002,   /* U+101A3 */
  0x0002,   /* U+101A4 */
  0x0002,   /* U+101A5 */
  0x0002,   /* U+101A6 */
  0x0002,   /* U+101A7 */
  0x0002,   /* U+101A8 */
  0x0002,   /* U+101A9 */
  0x0002,   /* U+101AA */
  0x0002,   /* U+101AB */
  0x0002,   /* U+101AC */
  0x0002,   /* U+101AD */
  0x0002,   /* U+101AE */
  0x0002,   /* U+101AF */
  0x0002,   /* U+101B0 */
  0x0002,   /* U+101B1 */
  0x0002,   /* U+101B2 */
  0x0002,   /* U+101B3 */
  0x0002,   /* U+101B4 */
  0x0002,   /* U+101B5 */
  0x0002,   /* U+101B6 */
  0x0002,   /* U+101B7 */
  0x0002,   /* U+101B8 */
  0x0002,   /* U+101B9 */
  0x0002,   /* U+101BA */
  0x0002,   /* U+101BB */
  0x0002,   /* U+101BC */
  0x0002,   /* U+101BD */
  0x0002,   /* U+101BE */
  0x0002,   /* U+101BF */
  0x0002,   /* U+101C0 */
  0x0002,   /* U+101C1 */
  0x0002,   /* U+101C2 */
  0x0002,   /* U+101C3 */
  0x0002,   /* U+101C4 */
  0x0002,   /* U+101C5 */
  0x0002,   /* U+101C6 */
  0x0002,   /* U+101C7 */
  0x0002,   /* U+101C8 */
  0x0002,   /* U+101C9 */
  0x0002,   /* U+101CA */
  0x0002,   /* U+101CB */
  0x0002,   /* U+101CC */
  0x0002,   /* U+101CD */
  0x0002,   /* U+101CE */
  0x0002,   /* U+101CF */
  0x0002,   /* U+101D0 */
  0x0002,   /* U+101D1 */
  0x0002,   /* U+101D2 */
  0x0002,   /* U+101D3 */
  0x0002,   /* U+101D4 */
  0x0002,   /* U+101D5 */
  0x0002,   /* U+101D6 */
  0x0002,   /* U+101D7 */
  0x0002,   /* U+101D8 */
  0x0002,   /* U+101D9 */
  0x0002,   /* U+101DA */
  0x0002,   /* U+101DB */
  0x0002,   /* U+101DC */
  0x0002,   /* U+101DD */
  0x0002,   /* U+101DE */
  0x0002,   /* U+101DF */
  0x0002,   /* U+101E0 */
  0x0002,   /* U+101E1 */
  0x0002,   /* U+101E2 */
  0x0002,   /* U+101E3 */
  0x0002,   /* U+101E4 */
  0x0002,   /* U+101E5 */
  0x0002,   /* U+101E6 */
  0x0002,   /* U+101E7 */
  0x0002,   /* U+101E8 */
  0x0002,   /* U+101E9 */
  0x0002,   /* U+101EA */
  0x0002,   /* U+101EB */
  0x0002,   /* U+101EC */
  0x0002,   /* U+101ED */
  0x0002,   /* U+101EE */
  0x0002,   /* U+101EF */
  0x0002,   /* U+101F0 */
  0x0002,   /* U+101F1 */
  0x0002,   /* U+101F2 */
  0x0002,   /* U+101F3 */
  0x0002,   /* U+101F4 */
  0x0002,   /* U+101F5 */
  0x0002,   /* U+101F6 */
  0x0002,   /* U+101F7 */
  0x0002,   /* U+101F8 */
  0x0002,   /* U+101F9 */
  0x0002,   /* U+101FA */
  0x0002,   /* U+101FB */
  0x0002,   /* U+101FC */
  0x0002,   /* U+101FD */
  0x0002,   /* U+101FE */
  0x0002,   /* U+101FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10100 */
  0x0000,   /* U+10101 */
  0x0000,   /* U+10102 */
  0x8103,   /* U+10103 */
  0x8104,   /* U+10104 */
  0x8105,   /* U+10105 */
  0x8106,   /* U+10106 */
  0x0000,   /* U+10107 */
  0x0000,   /* U+10108 */
  0x0000,   /* U+10109 */
  0x0000,   /* U+1010A */
  0x0000,   /* U+1010B */
  0x0000,   /* U+1010C */
  0x0000,   /* U+1010D */
  0x0000,   /* U+1010E */
  0x0000,   /* U+1010F */
  0x0000,   /* U+10110 */
  0x0000,   /* U+10111 */
  0x0000,   /* U+10112 */
  0x0000,   /* U+10113 */
  0x0000,   /* U+10114 */
  0x0000,   /* U+10115 */
  0x0000,   /* U+10116 */
  0x0000,   /* U+10117 */
  0x0000,   /* U+10118 */
  0x0000,   /* U+10119 */
  0x0000,   /* U+1011A */
  0x0000,   /* U+1011B */
  0x0000,   /* U+1011C */
  0x0000,   /* U+1011D */
  0x0000,   /* U+1011E */
  0x0000,   /* U+1011F */
  0x0000,   /* U+10120 */
  0x0000,   /* U+10121 */
  0x0000,   /* U+10122 */
  0x0000,   /* U+10123 */
  0x0000,   /* U+10124 */
  0x0000,   /* U+10125 */
  0x0000,   /* U+10126 */
  0x0000,   /* U+10127 */
  0x0000,   /* U+10128 */
  0x0000,   /* U+10129 */
  0x0000,   /* U+1012A */
  0x0000,   /* U+1012B */
  0x0000,   /* U+1012C */
  0x0000,   /* U+1012D */
  0x0000,   /* U+1012E */
  0x0000,   /* U+1012F */
  0x0000,   /* U+10130 */
  0x0000,   /* U+10131 */
  0x0000,   /* U+10132 */
  0x0000,   /* U+10133 */
  0x8134,   /* U+10134 */
  0x8135,   /* U+10135 */
  0x8136,   /* U+10136 */
  0x0000,   /* U+10137 */
  0x0000,   /* U+10138 */
  0x0000,   /* U+10139 */
  0x0000,   /* U+1013A */
  0x0000,   /* U+1013B */
  0x0000,   /* U+1013C */
  0x0000,   /* U+1013D */
  0x0000,   /* U+1013E */
  0x0000,   /* U+1013F */
  0x0000,   /* U+10140 */
  0x0000,   /* U+10141 */
  0x0000,   /* U+10142 */
  0x0000,   /* U+10143 */
  0x0000,   /* U+10144 */
  0x0000,   /* U+10145 */
  0x0000,   /* U+10146 */
  0x0000,   /* U+10147 */
  0x0000,   /* U+10148 */
  0x0000,   /* U+10149 */
  0x0000,   /* U+1014A */
  0x0000,   /* U+1014B */
  0x0000,   /* U+1014C */
  0x0000,   /* U+1014D */
  0x0000,   /* U+1014E */
  0x0000,   /* U+1014F */
  0x0000,   /* U+10150 */
  0x0000,   /* U+10151 */
  0x0000,   /* U+10152 */
  0x0000,   /* U+10153 */
  0x0000,   /* U+10154 */
  0x0000,   /* U+10155 */
  0x0000,   /* U+10156 */
  0x0000,   /* U+10157 */
  0x0000,   /* U+10158 */
  0x0000,   /* U+10159 */
  0x0000,   /* U+1015A */
  0x0000,   /* U+1015B */
  0x0000,   /* U+1015C */
  0x0000,   /* U+1015D */
  0x0000,   /* U+1015E */
  0x0000,   /* U+1015F */
  0x0000,   /* U+10160 */
  0x0000,   /* U+10161 */
  0x0000,   /* U+10162 */
  0x0000,   /* U+10163 */
  0x0000,   /* U+10164 */
  0x0000,   /* U+10165 */
  0x0000,   /* U+10166 */
  0x0000,   /* U+10167 */
  0x0000,   /* U+10168 */
  0x0000,   /* U+10169 */
  0x0000,   /* U+1016A */
  0x0000,   /* U+1016B */
  0x0000,   /* U+1016C */
  0x0000,   /* U+1016D */
  0x0000,   /* U+1016E */
  0x0000,   /* U+1016F */
  0x0000,   /* U+10170 */
  0x0000,   /* U+10171 */
  0x0000,   /* U+10172 */
  0x0000,   /* U+10173 */
  0x0000,   /* U+10174 */
  0x0000,   /* U+10175 */
  0x0000,   /* U+10176 */
  0x0000,   /* U+10177 */
  0x0000,   /* U+10178 */
  0x0000,   /* U+10179 */
  0x0000,   /* U+1017A */
  0x0000,   /* U+1017B */
  0x0000,   /* U+1017C */
  0x0000,   /* U+1017D */
  0x0000,   /* U+1017E */
  0x0000,   /* U+1017F */
  0x0000,   /* U+10180 */
  0x0000,   /* U+10181 */
  0x0000,   /* U+10182 */
  0x0000,   /* U+10183 */
  0x0000,   /* U+10184 */
  0x0000,   /* U+10185 */
  0x0000,   /* U+10186 */
  0x0000,   /* U+10187 */
  0x0000,   /* U+10188 */
  0x0000,   /* U+10189 */
  0x0000,   /* U+1018A */
  0x0000,   /* U+1018B */
  0x0000,   /* U+1018C */
  0x0000,   /* U+1018D */
  0x0000,   /* U+1018E */
  0x818F,   /* U+1018F */
  0x0000,   /* U+10190 */
  0x0000,   /* U+10191 */
  0x0000,   /* U+10192 */
  0x0000,   /* U+10193 */
  0x0000,   /* U+10194 */
  0x0000,   /* U+10195 */
  0x0000,   /* U+10196 */
  0x0000,   /* U+10197 */
  0x0000,   /* U+10198 */
  0x0000,   /* U+10199 */
  0x0000,   /* U+1019A */
  0x0000,   /* U+1019B */
  0x819C,   /* U+1019C */
  0x819D,   /* U+1019D */
  0x819E,   /* U+1019E */
  0x819F,   /* U+1019F */
  0x0000,   /* U+101A0 */
  0x81A1,   /* U+101A1 */
  0x81A2,   /* U+101A2 */
  0x81A3,   /* U+101A3 */
  0x81A4,   /* U+101A4 */
  0x81A5,   /* U+101A5 */
  0x81A6,   /* U+101A6 */
  0x81A7,   /* U+101A7 */
  0x81A8,   /* U+101A8 */
  0x81A9,   /* U+101A9 */
  0x81AA,   /* U+101AA */
  0x81AB,   /* U+101AB */
  0x81AC,   /* U+101AC */
  0x81AD,   /* U+101AD */
  0x81AE,   /* U+101AE */
  0x81AF,   /* U+101AF */
  0x81B0,   /* U+101B0 */
  0x81B1,   /* U+101B1 */
  0x81B2,   /* U+101B2 */
  0x81B3,   /* U+101B3 */
  0x81B4,   /* U+101B4 */
  0x81B5,   /* U+101B5 */
  0x81B6,   /* U+101B6 */
  0x81B7,   /* U+101B7 */
  0x81B8,   /* U+101B8 */
  0x81B9,   /* U+101B9 */
  0x81BA,   /* U+101BA */
  0x81BB,   /* U+101BB */
  0x81BC,   /* U+101BC */
  0x81BD,   /* U+101BD */
  0x81BE,   /* U+101BE */
  0x81BF,   /* U+101BF */
  0x81C0,   /* U+101C0 */
  0x81C1,   /* U+101C1 */
  0x81C2,   /* U+101C2 */
  0x81C3,   /* U+101C3 */
  0x81C4,   /* U+101C4 */
  0x81C5,   /* U+101C5 */
  0x81C6,   /* U+101C6 */
  0x81C7,   /* U+101C7 */
  0x81C8,   /* U+101C8 */
  0x81C9,   /* U+101C9 */
  0x81CA,   /* U+101CA */
  0x81CB,   /* U+101CB */
  0x81CC,   /* U+101CC */
  0x81CD,   /* U+101CD */
  0x81CE,   /* U+101CE */
  0x81CF,   /* U+101CF */
  0x0000,   /* U+101D0 */
  0x0000,   /* U+101D1 */
  0x0000,   /* U+101D2 */
  0x0000,   /* U+101D3 */
  0x0000,   /* U+101D4 */
  0x0000,   /* U+101D5 */
  0x0000,   /* U+101D6 */
  0x0000,   /* U+101D7 */
  0x0000,   /* U+101D8 */
  0x0000,   /* U+101D9 */
  0x0000,   /* U+101DA */
  0x0000,   /* U+101DB */
  0x0000,   /* U+101DC */
  0x0000,   /* U+101DD */
  0x0000,   /* U+101DE */
  0x0000,   /* U+101DF */
  0x0000,   /* U+101E0 */
  0x0000,   /* U+101E1 */
  0x0000,   /* U+101E2 */
  0x0000,   /* U+101E3 */
  0x0000,   /* U+101E4 */
  0x0000,   /* U+101E5 */
  0x0000,   /* U+101E6 */
  0x0000,   /* U+101E7 */
  0x0000,   /* U+101E8 */
  0x0000,   /* U+101E9 */
  0x0000,   /* U+101EA */
  0x0000,   /* U+101EB */
  0x0000,   /* U+101EC */
  0x0000,   /* U+101ED */
  0x0000,   /* U+101EE */
  0x0000,   /* U+101EF */
  0x0000,   /* U+101F0 */
  0x0000,   /* U+101F1 */
  0x0000,   /* U+101F2 */
  0x0000,   /* U+101F3 */
  0x0000,   /* U+101F4 */
  0x0000,   /* U+101F5 */
  0x0000,   /* U+101F6 */
  0x0000,   /* U+101F7 */
  0x0000,   /* U+101F8 */
  0x0000,   /* U+101F9 */
  0x0000,   /* U+101FA */
  0x0000,   /* U+101FB */
  0x0000,   /* U+101FC */
  0x0000,   /* U+101FD */
  0x81FE,   /* U+101FE */
  0x81FF,   /* U+101FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10100 */
  0x0000,   /* U+10101 */
  0x0000,   /* U+10102 */
  0x0000,   /* U+10103 */
  0x0000,   /* U+10104 */
  0x0000,   /* U+10105 */
  0x0000,   /* U+10106 */
  0x0000,   /* U+10107 */
  0x0000,   /* U+10108 */
  0x0000,   /* U+10109 */
  0x0000,   /* U+1010A */
  0x0000,   /* U+1010B */
  0x0000,   /* U+1010C */
  0x0000,   /* U+1010D */
  0x0000,   /* U+1010E */
  0x0000,   /* U+1010F */
  0x0000,   /* U+10110 */
  0x0000,   /* U+10111 */
  0x0000,   /* U+10112 */
  0x0000,   /* U+10113 */
  0x0000,   /* U+10114 */
  0x0000,   /* U+10115 */
  0x0000,   /* U+10116 */
  0x0000,   /* U+10117 */
  0x0000,   /* U+10118 */
  0x0000,   /* U+10119 */
  0x0000,   /* U+1011A */
  0x0000,   /* U+1011B */
  0x0000,   /* U+1011C */
  0x0000,   /* U+1011D */
  0x0000,   /* U+1011E */
  0x0000,   /* U+1011F */
  0x0000,   /* U+10120 */
  0x0000,   /* U+10121 */
  0x0000,   /* U+10122 */
  0x0000,   /* U+10123 */
  0x0000,   /* U+10124 */
  0x0000,   /* U+10125 */
  0x0000,   /* U+10126 */
  0x0000,   /* U+10127 */
  0x0000,   /* U+10128 */
  0x0000,   /* U+10129 */
  0x0000,   /* U+1012A */
  0x0000,   /* U+1012B */
  0x0000,   /* U+1012C */
  0x0000,   /* U+1012D */
  0x0000,   /* U+1012E */
  0x0000,   /* U+1012F */
  0x0000,   /* U+10130 */
  0x0000,   /* U+10131 */
  0x0000,   /* U+10132 */
  0x0000,   /* U+10133 */
  0x0000,   /* U+10134 */
  0x0000,   /* U+10135 */
  0x0000,   /* U+10136 */
  0x0000,   /* U+10137 */
  0x0000,   /* U+10138 */
  0x0000,   /* U+10139 */
  0x0000,   /* U+1013A */
  0x0000,   /* U+1013B */
  0x0000,   /* U+1013C */
  0x0000,   /* U+1013D */
  0x0000,   /* U+1013E */
  0x0000,   /* U+1013F */
  0x0000,   /* U+10140 */
  0x0000,   /* U+10141 */
  0x0000,   /* U+10142 */
  0x0000,   /* U+10143 */
  0x0000,   /* U+10144 */
  0x0000,   /* U+10145 */
  0x0000,   /* U+10146 */
  0x0000,   /* U+10147 */
  0x0000,   /* U+10148 */
  0x0000,   /* U+10149 */
  0x0000,   /* U+1014A */
  0x0000,   /* U+1014B */
  0x0000,   /* U+1014C */
  0x0000,   /* U+1014D */
  0x0000,   /* U+1014E */
  0x0000,   /* U+1014F */
  0x0000,   /* U+10150 */
  0x0000,   /* U+10151 */
  0x0000,   /* U+10152 */
  0x0000,   /* U+10153 */
  0x0000,   /* U+10154 */
  0x0000,   /* U+10155 */
  0x0000,   /* U+10156 */
  0x0000,   /* U+10157 */
  0x0000,   /* U+10158 */
  0x0000,   /* U+10159 */
  0x0000,   /* U+1015A */
  0x0000,   /* U+1015B */
  0x0000,   /* U+1015C */
  0x0000,   /* U+1015D */
  0x0000,   /* U+1015E */
  0x0000,   /* U+1015F */
  0x0000,   /* U+10160 */
  0x0000,   /* U+10161 */
  0x0000,   /* U+10162 */
  0x0000,   /* U+10163 */
  0x0000,   /* U+10164 */
  0x0000,   /* U+10165 */
  0x0000,   /* U+10166 */
  0x0000,   /* U+10167 */
  0x0000,   /* U+10168 */
  0x0000,   /* U+10169 */
  0x0000,   /* U+1016A */
  0x0000,   /* U+1016B */
  0x0000,   /* U+1016C */
  0x0000,   /* U+1016D */
  0x0000,   /* U+1016E */
  0x0000,   /* U+1016F */
  0x0000,   /* U+10170 */
  0x0000,   /* U+10171 */
  0x0000,   /* U+10172 */
  0x0000,   /* U+10173 */
  0x0000,   /* U+10174 */
  0x0000,   /* U+10175 */
  0x0000,   /* U+10176 */
  0x0000,   /* U+10177 */
  0x0000,   /* U+10178 */
  0x0000,   /* U+10179 */
  0x0000,   /* U+1017A */
  0x0000,   /* U+1017B */
  0x0000,   /* U+1017C */
  0x0000,   /* U+1017D */
  0x0000,   /* U+1017E */
  0x0000,   /* U+1017F */
  0x0000,   /* U+10180 */
  0x0000,   /* U+10181 */
  0x0000,   /* U+10182 */
  0x0000,   /* U+10183 */
  0x0000,   /* U+10184 */
  0x0000,   /* U+10185 */
  0x0000,   /* U+10186 */
  0x0000,   /* U+10187 */
  0x0000,   /* U+10188 */
  0x0000,   /* U+10189 */
  0x0000,   /* U+1018A */
  0x0000,   /* U+1018B */
  0x0000,   /* U+1018C */
  0x0000,   /* U+1018D */
  0x0000,   /* U+1018E */
  0x0000,   /* U+1018F */
  0x0000,   /* U+10190 */
  0x0000,   /* U+10191 */
  0x0000,   /* U+10192 */
  0x0000,   /* U+10193 */
  0x0000,   /* U+10194 */
  0x0000,   /* U+10195 */
  0x0000,   /* U+10196 */
  0x0000,   /* U+10197 */
  0x0000,   /* U+10198 */
  0x0000,   /* U+10199 */
  0x0000,   /* U+1019A */
  0x0000,   /* U+1019B */
  0x0000,   /* U+1019C */
  0x0000,   /* U+1019D */
  0x0000,   /* U+1019E */
  0x0000,   /* U+1019F */
  0x0000,   /* U+101A0 */
  0x0000,   /* U+101A1 */
  0x0000,   /* U+101A2 */
  0x0000,   /* U+101A3 */
  0x0000,   /* U+101A4 */
  0x0000,   /* U+101A5 */
  0x0000,   /* U+101A6 */
  0x0000,   /* U+101A7 */
  0x0000,   /* U+101A8 */
  0x0000,   /* U+101A9 */
  0x0000,   /* U+101AA */
  0x0000,   /* U+101AB */
  0x0000,   /* U+101AC */
  0x0000,   /* U+101AD */
  0x0000,   /* U+101AE */
  0x0000,   /* U+101AF */
  0x0000,   /* U+101B0 */
  0x0000,   /* U+101B1 */
  0x0000,   /* U+101B2 */
  0x0000,   /* U+101B3 */
  0x0000,   /* U+101B4 */
  0x0000,   /* U+101B5 */
  0x0000,   /* U+101B6 */
  0x0000,   /* U+101B7 */
  0x0000,   /* U+101B8 */
  0x0000,   /* U+101B9 */
  0x0000,   /* U+101BA */
  0x0000,   /* U+101BB */
  0x0000,   /* U+101BC */
  0x0000,   /* U+101BD */
  0x0000,   /* U+101BE */
  0x0000,   /* U+101BF */
  0x0000,   /* U+101C0 */
  0x0000,   /* U+101C1 */
  0x0000,   /* U+101C2 */
  0x0000,   /* U+101C3 */
  0x0000,   /* U+101C4 */
  0x0000,   /* U+101C5 */
  0x0000,   /* U+101C6 */
  0x0000,   /* U+101C7 */
  0x0000,   /* U+101C8 */
  0x0000,   /* U+101C9 */
  0x0000,   /* U+101CA */
  0x0000,   /* U+101CB */
  0x0000,   /* U+101CC */
  0x0000,   /* U+101CD */
  0x0000,   /* U+101CE */
  0x0000,   /* U+101CF */
  0x0000,   /* U+101D0 */
  0x0000,   /* U+101D1 */
  0x0000,   /* U+101D2 */
  0x0000,   /* U+101D3 */
  0x0000,   /* U+101D4 */
  0x0000,   /* U+101D5 */
  0x0000,   /* U+101D6 */
  0x0000,   /* U+101D7 */
  0x0000,   /* U+101D8 */
  0x0000,   /* U+101D9 */
  0x0000,   /* U+101DA */
  0x0000,   /* U+101DB */
  0x0000,   /* U+101DC */
  0x0000,   /* U+101DD */
  0x0000,   /* U+101DE */
  0x0000,   /* U+101DF */
  0x0000,   /* U+101E0 */
  0x0000,   /* U+101E1 */
  0x0000,   /* U+101E2 */
  0x0000,   /* U+101E3 */
  0x0000,   /* U+101E4 */
  0x0000,   /* U+101E5 */
  0x0000,   /* U+101E6 */
  0x0000,   /* U+101E7 */
  0x0000,   /* U+101E8 */
  0x0000,   /* U+101E9 */
  0x0000,   /* U+101EA */
  0x0000,   /* U+101EB */
  0x0000,   /* U+101EC */
  0x0000,   /* U+101ED */
  0x0000,   /* U+101EE */
  0x0000,   /* U+101EF */
  0x0000,   /* U+101F0 */
  0x0000,   /* U+101F1 */
  0x0000,   /* U+101F2 */
  0x0000,   /* U+101F3 */
  0x0000,   /* U+101F4 */
  0x0000,   /* U+101F5 */
  0x0000,   /* U+101F6 */
  0x0000,   /* U+101F7 */
  0x0000,   /* U+101F8 */
  0x0000,   /* U+101F9 */
  0x0000,   /* U+101FA */
  0x0000,   /* U+101FB */
  0x0000,   /* U+101FC */
  0x0000,   /* U+101FD */
  0x0000,   /* U+101FE */
  0x0000,   /* U+101FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10100 */
  0x0000,   /* U+10101 */
  0x0000,   /* U+10102 */
  0x0000,   /* U+10103 */
  0x0000,   /* U+10104 */
  0x0000,   /* U+10105 */
  0x0000,   /* U+10106 */
  0x0000,   /* U+10107 */
  0x0000,   /* U+10108 */
  0x0000,   /* U+10109 */
  0x0000,   /* U+1010A */
  0x0000,   /* U+1010B */
  0x0000,   /* U+1010C */
  0x0000,   /* U+1010D */
  0x0000,   /* U+1010E */
  0x0000,   /* U+1010F */
  0x0000,   /* U+10110 */
  0x0000,   /* U+10111 */
  0x0000,   /* U+10112 */
  0x0000,   /* U+10113 */
  0x0000,   /* U+10114 */
  0x0000,   /* U+10115 */
  0x0000,   /* U+10116 */
  0x0000,   /* U+10117 */
  0x0000,   /* U+10118 */
  0x0000,   /* U+10119 */
  0x0000,   /* U+1011A */
  0x0000,   /* U+1011B */
  0x0000,   /* U+1011C */
  0x0000,   /* U+1011D */
  0x0000,   /* U+1011E */
  0x0000,   /* U+1011F */
  0x0000,   /* U+10120 */
  0x0000,   /* U+10121 */
  0x0000,   /* U+10122 */
  0x0000,   /* U+10123 */
  0x0000,   /* U+10124 */
  0x0000,   /* U+10125 */
  0x0000,   /* U+10126 */
  0x0000,   /* U+10127 */
  0x0000,   /* U+10128 */
  0x0000,   /* U+10129 */
  0x0000,   /* U+1012A */
  0x0000,   /* U+1012B */
  0x0000,   /* U+1012C */
  0x0000,   /* U+1012D */
  0x0000,   /* U+1012E */
  0x0000,   /* U+1012F */
  0x0000,   /* U+10130 */
  0x0000,   /* U+10131 */
  0x0000,   /* U+10132 */
  0x0000,   /* U+10133 */
  0x0000,   /* U+10134 */
  0x0000,   /* U+10135 */
  0x0000,   /* U+10136 */
  0x0000,   /* U+10137 */
  0x0000,   /* U+10138 */
  0x0000,   /* U+10139 */
  0x0000,   /* U+1013A */
  0x0000,   /* U+1013B */
  0x0000,   /* U+1013C */
  0x0000,   /* U+1013D */
  0x0000,   /* U+1013E */
  0x0000,   /* U+1013F */
  0x0000,   /* U+10140 */
  0x0000,   /* U+10141 */
  0x0000,   /* U+10142 */
  0x0000,   /* U+10143 */
  0x0000,   /* U+10144 */
  0x0000,   /* U+10145 */
  0x0000,   /* U+10146 */
  0x0000,   /* U+10147 */
  0x0000,   /* U+10148 */
  0x0000,   /* U+10149 */
  0x0000,   /* U+1014A */
  0x0000,   /* U+1014B */
  0x0000,   /* U+1014C */
  0x0000,   /* U+1014D */
  0x0000,   /* U+1014E */
  0x0000,   /* U+1014F */
  0x0000,   /* U+10150 */
  0x0000,   /* U+10151 */
  0x0000,   /* U+10152 */
  0x0000,   /* U+10153 */
  0x0000,   /* U+10154 */
  0x0000,   /* U+10155 */
  0x0000,   /* U+10156 */
  0x0000,   /* U+10157 */
  0x0000,   /* U+10158 */
  0x0000,   /* U+10159 */
  0x0000,   /* U+1015A */
  0x0000,   /* U+1015B */
  0x0000,   /* U+1015C */
  0x0000,   /* U+1015D */
  0x0000,   /* U+1015E */
  0x0000,   /* U+1015F */
  0x0000,   /* U+10160 */
  0x0000,   /* U+10161 */
  0x0000,   /* U+10162 */
  0x0000,   /* U+10163 */
  0x0000,   /* U+10164 */
  0x0000,   /* U+10165 */
  0x0000,   /* U+10166 */
  0x0000,   /* U+10167 */
  0x0000,   /* U+10168 */
  0x0000,   /* U+10169 */
  0x0000,   /* U+1016A */
  0x0000,   /* U+1016B */
  0x0000,   /* U+1016C */
  0x0000,   /* U+1016D */
  0x0000,   /* U+1016E */
  0x0000,   /* U+1016F */
  0x0000,   /* U+10170 */
  0x0000,   /* U+10171 */
  0x0000,   /* U+10172 */
  0x0000,   /* U+10173 */
  0x0000,   /* U+10174 */
  0x0000,   /* U+10175 */
  0x0000,   /* U+10176 */
  0x0000,   /* U+10177 */
  0x0000,   /* U+10178 */
  0x0000,   /* U+10179 */
  0x0000,   /* U+1017A */
  0x0000,   /* U+1017B */
  0x0000,   /* U+1017C */
  0x0000,   /* U+1017D */
  0x0000,   /* U+1017E */
  0x0000,   /* U+1017F */
  0x0000,   /* U+10180 */
  0x0000,   /* U+10181 */
  0x0000,   /* U+10182 */
  0x0000,   /* U+10183 */
  0x0000,   /* U+10184 */
  0x0000,   /* U+10185 */
  0x0000,   /* U+10186 */
  0x0000,   /* U+10187 */
  0x0000,   /* U+10188 */
  0x0000,   /* U+10189 */
  0x0000,   /* U+1018A */
  0x0000,   /* U+1018B */
  0x0000,   /* U+1018C */
  0x0000,   /* U+1018D */
  0x0000,   /* U+1018E */
  0x0000,   /* U+1018F */
  0x0000,   /* U+10190 */
  0x0000,   /* U+10191 */
  0x0000,   /* U+10192 */
  0x0000,   /* U+10193 */
  0x0000,   /* U+10194 */
  0x0000,   /* U+10195 */
  0x0000,   /* U+10196 */
  0x0000,   /* U+10197 */
  0x0000,   /* U+10198 */
  0x0000,   /* U+10199 */
  0x0000,   /* U+1019A */
  0x0000,   /* U+1019B */
  0x0000,   /* U+1019C */
  0x0000,   /* U+1019D */
  0x0000,   /* U+1019E */
  0x0000,   /* U+1019F */
  0x0000,   /* U+101A0 */
  0x0000,   /* U+101A1 */
  0x0000,   /* U+101A2 */
  0x0000,   /* U+101A3 */
  0x0000,   /* U+101A4 */
  0x0000,   /* U+101A5 */
  0x0000,   /* U+101A6 */
  0x0000,   /* U+101A7 */
  0x0000,   /* U+101A8 */
  0x0000,   /* U+101A9 */
  0x0000,   /* U+101AA */
  0x0000,   /* U+101AB */
  0x0000,   /* U+101AC */
  0x0000,   /* U+101AD */
  0x0000,   /* U+101AE */
  0x0000,   /* U+101AF */
  0x0000,   /* U+101B0 */
  0x0000,   /* U+101B1 */
  0x0000,   /* U+101B2 */
  0x0000,   /* U+101B3 */
  0x0000,   /* U+101B4 */
  0x0000,   /* U+101B5 */
  0x0000,   /* U+101B6 */
  0x0000,   /* U+101B7 */
  0x0000,   /* U+101B8 */
  0x0000,   /* U+101B9 */
  0x0000,   /* U+101BA */
  0x0000,   /* U+101BB */
  0x0000,   /* U+101BC */
  0x0000,   /* U+101BD */
  0x0000,   /* U+101BE */
  0x0000,   /* U+101BF */
  0x0000,   /* U+101C0 */
  0x0000,   /* U+101C1 */
  0x0000,   /* U+101C2 */
  0x0000,   /* U+101C3 */
  0x0000,   /* U+101C4 */
  0x0000,   /* U+101C5 */
  0x0000,   /* U+101C6 */
  0x0000,   /* U+101C7 */
  0x0000,   /* U+101C8 */
  0x0000,   /* U+101C9 */
  0x0000,   /* U+101CA */
  0x0000,   /* U+101CB */
  0x0000,   /* U+101CC */
  0x0000,   /* U+101CD */
  0x0000,   /* U+101CE */
  0x0000,   /* U+101CF */
  0x0000,   /* U+101D0 */
  0x0000,   /* U+101D1 */
  0x0000,   /* U+101D2 */
  0x0000,   /* U+101D3 */
  0x0000,   /* U+101D4 */
  0x0000,   /* U+101D5 */
  0x0000,   /* U+101D6 */
  0x0000,   /* U+101D7 */
  0x0000,   /* U+101D8 */
  0x0000,   /* U+101D9 */
  0x0000,   /* U+101DA */
  0x0000,   /* U+101DB */
  0x0000,   /* U+101DC */
  0x0000,   /* U+101DD */
  0x0000,   /* U+101DE */
  0x0000,   /* U+101DF */
  0x0000,   /* U+101E0 */
  0x0000,   /* U+101E1 */
  0x0000,   /* U+101E2 */
  0x0000,   /* U+101E3 */
  0x0000,   /* U+101E4 */
  0x0000,   /* U+101E5 */
  0x0000,   /* U+101E6 */
  0x0000,   /* U+101E7 */
  0x0000,   /* U+101E8 */
  0x0000,   /* U+101E9 */
  0x0000,   /* U+101EA */
  0x0000,   /* U+101EB */
  0x0000,   /* U+101EC */
  0x0000,   /* U+101ED */
  0x0000,   /* U+101EE */
  0x0000,   /* U+101EF */
  0x0000,   /* U+101F0 */
  0x0000,   /* U+101F1 */
  0x0000,   /* U+101F2 */
  0x0000,   /* U+101F3 */
  0x0000,   /* U+101F4 */
  0x0000,   /* U+101F5 */
  0x0000,   /* U+101F6 */
  0x0000,   /* U+101F7 */
  0x0000,   /* U+101F8 */
  0x0000,   /* U+101F9 */
  0x0000,   /* U+101FA */
  0x0000,   /* U+101FB */
  0x0000,   /* U+101FC */
  0x0000,   /* U+101FD */
  0x0000,   /* U+101FE */
  0x0000,   /* U+101FF */
};

uint16 uca900_p102[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC2,   /* U+10200 */
  0xFBC2,   /* U+10201 */
  0xFBC2,   /* U+10202 */
  0xFBC2,   /* U+10203 */
  0xFBC2,   /* U+10204 */
  0xFBC2,   /* U+10205 */
  0xFBC2,   /* U+10206 */
  0xFBC2,   /* U+10207 */
  0xFBC2,   /* U+10208 */
  0xFBC2,   /* U+10209 */
  0xFBC2,   /* U+1020A */
  0xFBC2,   /* U+1020B */
  0xFBC2,   /* U+1020C */
  0xFBC2,   /* U+1020D */
  0xFBC2,   /* U+1020E */
  0xFBC2,   /* U+1020F */
  0xFBC2,   /* U+10210 */
  0xFBC2,   /* U+10211 */
  0xFBC2,   /* U+10212 */
  0xFBC2,   /* U+10213 */
  0xFBC2,   /* U+10214 */
  0xFBC2,   /* U+10215 */
  0xFBC2,   /* U+10216 */
  0xFBC2,   /* U+10217 */
  0xFBC2,   /* U+10218 */
  0xFBC2,   /* U+10219 */
  0xFBC2,   /* U+1021A */
  0xFBC2,   /* U+1021B */
  0xFBC2,   /* U+1021C */
  0xFBC2,   /* U+1021D */
  0xFBC2,   /* U+1021E */
  0xFBC2,   /* U+1021F */
  0xFBC2,   /* U+10220 */
  0xFBC2,   /* U+10221 */
  0xFBC2,   /* U+10222 */
  0xFBC2,   /* U+10223 */
  0xFBC2,   /* U+10224 */
  0xFBC2,   /* U+10225 */
  0xFBC2,   /* U+10226 */
  0xFBC2,   /* U+10227 */
  0xFBC2,   /* U+10228 */
  0xFBC2,   /* U+10229 */
  0xFBC2,   /* U+1022A */
  0xFBC2,   /* U+1022B */
  0xFBC2,   /* U+1022C */
  0xFBC2,   /* U+1022D */
  0xFBC2,   /* U+1022E */
  0xFBC2,   /* U+1022F */
  0xFBC2,   /* U+10230 */
  0xFBC2,   /* U+10231 */
  0xFBC2,   /* U+10232 */
  0xFBC2,   /* U+10233 */
  0xFBC2,   /* U+10234 */
  0xFBC2,   /* U+10235 */
  0xFBC2,   /* U+10236 */
  0xFBC2,   /* U+10237 */
  0xFBC2,   /* U+10238 */
  0xFBC2,   /* U+10239 */
  0xFBC2,   /* U+1023A */
  0xFBC2,   /* U+1023B */
  0xFBC2,   /* U+1023C */
  0xFBC2,   /* U+1023D */
  0xFBC2,   /* U+1023E */
  0xFBC2,   /* U+1023F */
  0xFBC2,   /* U+10240 */
  0xFBC2,   /* U+10241 */
  0xFBC2,   /* U+10242 */
  0xFBC2,   /* U+10243 */
  0xFBC2,   /* U+10244 */
  0xFBC2,   /* U+10245 */
  0xFBC2,   /* U+10246 */
  0xFBC2,   /* U+10247 */
  0xFBC2,   /* U+10248 */
  0xFBC2,   /* U+10249 */
  0xFBC2,   /* U+1024A */
  0xFBC2,   /* U+1024B */
  0xFBC2,   /* U+1024C */
  0xFBC2,   /* U+1024D */
  0xFBC2,   /* U+1024E */
  0xFBC2,   /* U+1024F */
  0xFBC2,   /* U+10250 */
  0xFBC2,   /* U+10251 */
  0xFBC2,   /* U+10252 */
  0xFBC2,   /* U+10253 */
  0xFBC2,   /* U+10254 */
  0xFBC2,   /* U+10255 */
  0xFBC2,   /* U+10256 */
  0xFBC2,   /* U+10257 */
  0xFBC2,   /* U+10258 */
  0xFBC2,   /* U+10259 */
  0xFBC2,   /* U+1025A */
  0xFBC2,   /* U+1025B */
  0xFBC2,   /* U+1025C */
  0xFBC2,   /* U+1025D */
  0xFBC2,   /* U+1025E */
  0xFBC2,   /* U+1025F */
  0xFBC2,   /* U+10260 */
  0xFBC2,   /* U+10261 */
  0xFBC2,   /* U+10262 */
  0xFBC2,   /* U+10263 */
  0xFBC2,   /* U+10264 */
  0xFBC2,   /* U+10265 */
  0xFBC2,   /* U+10266 */
  0xFBC2,   /* U+10267 */
  0xFBC2,   /* U+10268 */
  0xFBC2,   /* U+10269 */
  0xFBC2,   /* U+1026A */
  0xFBC2,   /* U+1026B */
  0xFBC2,   /* U+1026C */
  0xFBC2,   /* U+1026D */
  0xFBC2,   /* U+1026E */
  0xFBC2,   /* U+1026F */
  0xFBC2,   /* U+10270 */
  0xFBC2,   /* U+10271 */
  0xFBC2,   /* U+10272 */
  0xFBC2,   /* U+10273 */
  0xFBC2,   /* U+10274 */
  0xFBC2,   /* U+10275 */
  0xFBC2,   /* U+10276 */
  0xFBC2,   /* U+10277 */
  0xFBC2,   /* U+10278 */
  0xFBC2,   /* U+10279 */
  0xFBC2,   /* U+1027A */
  0xFBC2,   /* U+1027B */
  0xFBC2,   /* U+1027C */
  0xFBC2,   /* U+1027D */
  0xFBC2,   /* U+1027E */
  0xFBC2,   /* U+1027F */
  0x43AF,   /* U+10280 */
  0x43B0,   /* U+10281 */
  0x43B1,   /* U+10282 */
  0x43B2,   /* U+10283 */
  0x43B3,   /* U+10284 */
  0x43B4,   /* U+10285 */
  0x43B5,   /* U+10286 */
  0x43B6,   /* U+10287 */
  0x43B7,   /* U+10288 */
  0x43B8,   /* U+10289 */
  0x43B9,   /* U+1028A */
  0x43BA,   /* U+1028B */
  0x43BB,   /* U+1028C */
  0x43BC,   /* U+1028D */
  0x43BD,   /* U+1028E */
  0x43BE,   /* U+1028F */
  0x43BF,   /* U+10290 */
  0x43C0,   /* U+10291 */
  0x43C1,   /* U+10292 */
  0x43C2,   /* U+10293 */
  0x43C3,   /* U+10294 */
  0x43C4,   /* U+10295 */
  0x43C5,   /* U+10296 */
  0x43C6,   /* U+10297 */
  0x43C7,   /* U+10298 */
  0x43C8,   /* U+10299 */
  0x43C9,   /* U+1029A */
  0x43CA,   /* U+1029B */
  0x43CB,   /* U+1029C */
  0xFBC2,   /* U+1029D */
  0xFBC2,   /* U+1029E */
  0xFBC2,   /* U+1029F */
  0x43CC,   /* U+102A0 */
  0x43CD,   /* U+102A1 */
  0x43CE,   /* U+102A2 */
  0x43CF,   /* U+102A3 */
  0x43D0,   /* U+102A4 */
  0x43D1,   /* U+102A5 */
  0x43D2,   /* U+102A6 */
  0x43D3,   /* U+102A7 */
  0x43D4,   /* U+102A8 */
  0x43D5,   /* U+102A9 */
  0x43D6,   /* U+102AA */
  0x43D7,   /* U+102AB */
  0x43D8,   /* U+102AC */
  0x43D9,   /* U+102AD */
  0x43DA,   /* U+102AE */
  0x43DB,   /* U+102AF */
  0x43DC,   /* U+102B0 */
  0x43DD,   /* U+102B1 */
  0x43DE,   /* U+102B2 */
  0x43DF,   /* U+102B3 */
  0x43E0,   /* U+102B4 */
  0x43E1,   /* U+102B5 */
  0x43E2,   /* U+102B6 */
  0x43E3,   /* U+102B7 */
  0x43E4,   /* U+102B8 */
  0x43E5,   /* U+102B9 */
  0x43E6,   /* U+102BA */
  0x43E7,   /* U+102BB */
  0x43E8,   /* U+102BC */
  0x43E9,   /* U+102BD */
  0x43EA,   /* U+102BE */
  0x43EB,   /* U+102BF */
  0x43EC,   /* U+102C0 */
  0x43ED,   /* U+102C1 */
  0x43EE,   /* U+102C2 */
  0x43EF,   /* U+102C3 */
  0x43F0,   /* U+102C4 */
  0x43F1,   /* U+102C5 */
  0x43F2,   /* U+102C6 */
  0x43F3,   /* U+102C7 */
  0x43F4,   /* U+102C8 */
  0x43F5,   /* U+102C9 */
  0x43F6,   /* U+102CA */
  0x43F7,   /* U+102CB */
  0x43F8,   /* U+102CC */
  0x43F9,   /* U+102CD */
  0x43FA,   /* U+102CE */
  0x43FB,   /* U+102CF */
  0x43FC,   /* U+102D0 */
  0xFBC2,   /* U+102D1 */
  0xFBC2,   /* U+102D2 */
  0xFBC2,   /* U+102D3 */
  0xFBC2,   /* U+102D4 */
  0xFBC2,   /* U+102D5 */
  0xFBC2,   /* U+102D6 */
  0xFBC2,   /* U+102D7 */
  0xFBC2,   /* U+102D8 */
  0xFBC2,   /* U+102D9 */
  0xFBC2,   /* U+102DA */
  0xFBC2,   /* U+102DB */
  0xFBC2,   /* U+102DC */
  0xFBC2,   /* U+102DD */
  0xFBC2,   /* U+102DE */
  0xFBC2,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x1C3E,   /* U+102E1 */
  0x1C3F,   /* U+102E2 */
  0x1C40,   /* U+102E3 */
  0x1C41,   /* U+102E4 */
  0x1C42,   /* U+102E5 */
  0x1C43,   /* U+102E6 */
  0x1C44,   /* U+102E7 */
  0x1C45,   /* U+102E8 */
  0x1C46,   /* U+102E9 */
  0x1B3A,   /* U+102EA */
  0x1B3B,   /* U+102EB */
  0x1B3C,   /* U+102EC */
  0x1B3D,   /* U+102ED */
  0x1B3E,   /* U+102EE */
  0x1B3F,   /* U+102EF */
  0x1B40,   /* U+102F0 */
  0x1B41,   /* U+102F1 */
  0x1B42,   /* U+102F2 */
  0x1B43,   /* U+102F3 */
  0x1B44,   /* U+102F4 */
  0x1B45,   /* U+102F5 */
  0x1B46,   /* U+102F6 */
  0x1B47,   /* U+102F7 */
  0x1B48,   /* U+102F8 */
  0x1B49,   /* U+102F9 */
  0x1B4A,   /* U+102FA */
  0x1B4B,   /* U+102FB */
  0xFBC2,   /* U+102FC */
  0xFBC2,   /* U+102FD */
  0xFBC2,   /* U+102FE */
  0xFBC2,   /* U+102FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10200 */
  0x0020,   /* U+10201 */
  0x0020,   /* U+10202 */
  0x0020,   /* U+10203 */
  0x0020,   /* U+10204 */
  0x0020,   /* U+10205 */
  0x0020,   /* U+10206 */
  0x0020,   /* U+10207 */
  0x0020,   /* U+10208 */
  0x0020,   /* U+10209 */
  0x0020,   /* U+1020A */
  0x0020,   /* U+1020B */
  0x0020,   /* U+1020C */
  0x0020,   /* U+1020D */
  0x0020,   /* U+1020E */
  0x0020,   /* U+1020F */
  0x0020,   /* U+10210 */
  0x0020,   /* U+10211 */
  0x0020,   /* U+10212 */
  0x0020,   /* U+10213 */
  0x0020,   /* U+10214 */
  0x0020,   /* U+10215 */
  0x0020,   /* U+10216 */
  0x0020,   /* U+10217 */
  0x0020,   /* U+10218 */
  0x0020,   /* U+10219 */
  0x0020,   /* U+1021A */
  0x0020,   /* U+1021B */
  0x0020,   /* U+1021C */
  0x0020,   /* U+1021D */
  0x0020,   /* U+1021E */
  0x0020,   /* U+1021F */
  0x0020,   /* U+10220 */
  0x0020,   /* U+10221 */
  0x0020,   /* U+10222 */
  0x0020,   /* U+10223 */
  0x0020,   /* U+10224 */
  0x0020,   /* U+10225 */
  0x0020,   /* U+10226 */
  0x0020,   /* U+10227 */
  0x0020,   /* U+10228 */
  0x0020,   /* U+10229 */
  0x0020,   /* U+1022A */
  0x0020,   /* U+1022B */
  0x0020,   /* U+1022C */
  0x0020,   /* U+1022D */
  0x0020,   /* U+1022E */
  0x0020,   /* U+1022F */
  0x0020,   /* U+10230 */
  0x0020,   /* U+10231 */
  0x0020,   /* U+10232 */
  0x0020,   /* U+10233 */
  0x0020,   /* U+10234 */
  0x0020,   /* U+10235 */
  0x0020,   /* U+10236 */
  0x0020,   /* U+10237 */
  0x0020,   /* U+10238 */
  0x0020,   /* U+10239 */
  0x0020,   /* U+1023A */
  0x0020,   /* U+1023B */
  0x0020,   /* U+1023C */
  0x0020,   /* U+1023D */
  0x0020,   /* U+1023E */
  0x0020,   /* U+1023F */
  0x0020,   /* U+10240 */
  0x0020,   /* U+10241 */
  0x0020,   /* U+10242 */
  0x0020,   /* U+10243 */
  0x0020,   /* U+10244 */
  0x0020,   /* U+10245 */
  0x0020,   /* U+10246 */
  0x0020,   /* U+10247 */
  0x0020,   /* U+10248 */
  0x0020,   /* U+10249 */
  0x0020,   /* U+1024A */
  0x0020,   /* U+1024B */
  0x0020,   /* U+1024C */
  0x0020,   /* U+1024D */
  0x0020,   /* U+1024E */
  0x0020,   /* U+1024F */
  0x0020,   /* U+10250 */
  0x0020,   /* U+10251 */
  0x0020,   /* U+10252 */
  0x0020,   /* U+10253 */
  0x0020,   /* U+10254 */
  0x0020,   /* U+10255 */
  0x0020,   /* U+10256 */
  0x0020,   /* U+10257 */
  0x0020,   /* U+10258 */
  0x0020,   /* U+10259 */
  0x0020,   /* U+1025A */
  0x0020,   /* U+1025B */
  0x0020,   /* U+1025C */
  0x0020,   /* U+1025D */
  0x0020,   /* U+1025E */
  0x0020,   /* U+1025F */
  0x0020,   /* U+10260 */
  0x0020,   /* U+10261 */
  0x0020,   /* U+10262 */
  0x0020,   /* U+10263 */
  0x0020,   /* U+10264 */
  0x0020,   /* U+10265 */
  0x0020,   /* U+10266 */
  0x0020,   /* U+10267 */
  0x0020,   /* U+10268 */
  0x0020,   /* U+10269 */
  0x0020,   /* U+1026A */
  0x0020,   /* U+1026B */
  0x0020,   /* U+1026C */
  0x0020,   /* U+1026D */
  0x0020,   /* U+1026E */
  0x0020,   /* U+1026F */
  0x0020,   /* U+10270 */
  0x0020,   /* U+10271 */
  0x0020,   /* U+10272 */
  0x0020,   /* U+10273 */
  0x0020,   /* U+10274 */
  0x0020,   /* U+10275 */
  0x0020,   /* U+10276 */
  0x0020,   /* U+10277 */
  0x0020,   /* U+10278 */
  0x0020,   /* U+10279 */
  0x0020,   /* U+1027A */
  0x0020,   /* U+1027B */
  0x0020,   /* U+1027C */
  0x0020,   /* U+1027D */
  0x0020,   /* U+1027E */
  0x0020,   /* U+1027F */
  0x0020,   /* U+10280 */
  0x0020,   /* U+10281 */
  0x0020,   /* U+10282 */
  0x0020,   /* U+10283 */
  0x0020,   /* U+10284 */
  0x0020,   /* U+10285 */
  0x0020,   /* U+10286 */
  0x0020,   /* U+10287 */
  0x0020,   /* U+10288 */
  0x0020,   /* U+10289 */
  0x0020,   /* U+1028A */
  0x0020,   /* U+1028B */
  0x0020,   /* U+1028C */
  0x0020,   /* U+1028D */
  0x0020,   /* U+1028E */
  0x0020,   /* U+1028F */
  0x0020,   /* U+10290 */
  0x0020,   /* U+10291 */
  0x0020,   /* U+10292 */
  0x0020,   /* U+10293 */
  0x0020,   /* U+10294 */
  0x0020,   /* U+10295 */
  0x0020,   /* U+10296 */
  0x0020,   /* U+10297 */
  0x0020,   /* U+10298 */
  0x0020,   /* U+10299 */
  0x0020,   /* U+1029A */
  0x0020,   /* U+1029B */
  0x0020,   /* U+1029C */
  0x0020,   /* U+1029D */
  0x0020,   /* U+1029E */
  0x0020,   /* U+1029F */
  0x0020,   /* U+102A0 */
  0x0020,   /* U+102A1 */
  0x0020,   /* U+102A2 */
  0x0020,   /* U+102A3 */
  0x0020,   /* U+102A4 */
  0x0020,   /* U+102A5 */
  0x0020,   /* U+102A6 */
  0x0020,   /* U+102A7 */
  0x0020,   /* U+102A8 */
  0x0020,   /* U+102A9 */
  0x0020,   /* U+102AA */
  0x0020,   /* U+102AB */
  0x0020,   /* U+102AC */
  0x0020,   /* U+102AD */
  0x0020,   /* U+102AE */
  0x0020,   /* U+102AF */
  0x0020,   /* U+102B0 */
  0x0020,   /* U+102B1 */
  0x0020,   /* U+102B2 */
  0x0020,   /* U+102B3 */
  0x0020,   /* U+102B4 */
  0x0020,   /* U+102B5 */
  0x0020,   /* U+102B6 */
  0x0020,   /* U+102B7 */
  0x0020,   /* U+102B8 */
  0x0020,   /* U+102B9 */
  0x0020,   /* U+102BA */
  0x0020,   /* U+102BB */
  0x0020,   /* U+102BC */
  0x0020,   /* U+102BD */
  0x0020,   /* U+102BE */
  0x0020,   /* U+102BF */
  0x0020,   /* U+102C0 */
  0x0020,   /* U+102C1 */
  0x0020,   /* U+102C2 */
  0x0020,   /* U+102C3 */
  0x0020,   /* U+102C4 */
  0x0020,   /* U+102C5 */
  0x0020,   /* U+102C6 */
  0x0020,   /* U+102C7 */
  0x0020,   /* U+102C8 */
  0x0020,   /* U+102C9 */
  0x0020,   /* U+102CA */
  0x0020,   /* U+102CB */
  0x0020,   /* U+102CC */
  0x0020,   /* U+102CD */
  0x0020,   /* U+102CE */
  0x0020,   /* U+102CF */
  0x0020,   /* U+102D0 */
  0x0020,   /* U+102D1 */
  0x0020,   /* U+102D2 */
  0x0020,   /* U+102D3 */
  0x0020,   /* U+102D4 */
  0x0020,   /* U+102D5 */
  0x0020,   /* U+102D6 */
  0x0020,   /* U+102D7 */
  0x0020,   /* U+102D8 */
  0x0020,   /* U+102D9 */
  0x0020,   /* U+102DA */
  0x0020,   /* U+102DB */
  0x0020,   /* U+102DC */
  0x0020,   /* U+102DD */
  0x0020,   /* U+102DE */
  0x0020,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x0020,   /* U+102E1 */
  0x0020,   /* U+102E2 */
  0x0020,   /* U+102E3 */
  0x0020,   /* U+102E4 */
  0x0020,   /* U+102E5 */
  0x0020,   /* U+102E6 */
  0x0020,   /* U+102E7 */
  0x0020,   /* U+102E8 */
  0x0020,   /* U+102E9 */
  0x0020,   /* U+102EA */
  0x0020,   /* U+102EB */
  0x0020,   /* U+102EC */
  0x0020,   /* U+102ED */
  0x0020,   /* U+102EE */
  0x0020,   /* U+102EF */
  0x0020,   /* U+102F0 */
  0x0020,   /* U+102F1 */
  0x0020,   /* U+102F2 */
  0x0020,   /* U+102F3 */
  0x0020,   /* U+102F4 */
  0x0020,   /* U+102F5 */
  0x0020,   /* U+102F6 */
  0x0020,   /* U+102F7 */
  0x0020,   /* U+102F8 */
  0x0020,   /* U+102F9 */
  0x0020,   /* U+102FA */
  0x0020,   /* U+102FB */
  0x0020,   /* U+102FC */
  0x0020,   /* U+102FD */
  0x0020,   /* U+102FE */
  0x0020,   /* U+102FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10200 */
  0x0002,   /* U+10201 */
  0x0002,   /* U+10202 */
  0x0002,   /* U+10203 */
  0x0002,   /* U+10204 */
  0x0002,   /* U+10205 */
  0x0002,   /* U+10206 */
  0x0002,   /* U+10207 */
  0x0002,   /* U+10208 */
  0x0002,   /* U+10209 */
  0x0002,   /* U+1020A */
  0x0002,   /* U+1020B */
  0x0002,   /* U+1020C */
  0x0002,   /* U+1020D */
  0x0002,   /* U+1020E */
  0x0002,   /* U+1020F */
  0x0002,   /* U+10210 */
  0x0002,   /* U+10211 */
  0x0002,   /* U+10212 */
  0x0002,   /* U+10213 */
  0x0002,   /* U+10214 */
  0x0002,   /* U+10215 */
  0x0002,   /* U+10216 */
  0x0002,   /* U+10217 */
  0x0002,   /* U+10218 */
  0x0002,   /* U+10219 */
  0x0002,   /* U+1021A */
  0x0002,   /* U+1021B */
  0x0002,   /* U+1021C */
  0x0002,   /* U+1021D */
  0x0002,   /* U+1021E */
  0x0002,   /* U+1021F */
  0x0002,   /* U+10220 */
  0x0002,   /* U+10221 */
  0x0002,   /* U+10222 */
  0x0002,   /* U+10223 */
  0x0002,   /* U+10224 */
  0x0002,   /* U+10225 */
  0x0002,   /* U+10226 */
  0x0002,   /* U+10227 */
  0x0002,   /* U+10228 */
  0x0002,   /* U+10229 */
  0x0002,   /* U+1022A */
  0x0002,   /* U+1022B */
  0x0002,   /* U+1022C */
  0x0002,   /* U+1022D */
  0x0002,   /* U+1022E */
  0x0002,   /* U+1022F */
  0x0002,   /* U+10230 */
  0x0002,   /* U+10231 */
  0x0002,   /* U+10232 */
  0x0002,   /* U+10233 */
  0x0002,   /* U+10234 */
  0x0002,   /* U+10235 */
  0x0002,   /* U+10236 */
  0x0002,   /* U+10237 */
  0x0002,   /* U+10238 */
  0x0002,   /* U+10239 */
  0x0002,   /* U+1023A */
  0x0002,   /* U+1023B */
  0x0002,   /* U+1023C */
  0x0002,   /* U+1023D */
  0x0002,   /* U+1023E */
  0x0002,   /* U+1023F */
  0x0002,   /* U+10240 */
  0x0002,   /* U+10241 */
  0x0002,   /* U+10242 */
  0x0002,   /* U+10243 */
  0x0002,   /* U+10244 */
  0x0002,   /* U+10245 */
  0x0002,   /* U+10246 */
  0x0002,   /* U+10247 */
  0x0002,   /* U+10248 */
  0x0002,   /* U+10249 */
  0x0002,   /* U+1024A */
  0x0002,   /* U+1024B */
  0x0002,   /* U+1024C */
  0x0002,   /* U+1024D */
  0x0002,   /* U+1024E */
  0x0002,   /* U+1024F */
  0x0002,   /* U+10250 */
  0x0002,   /* U+10251 */
  0x0002,   /* U+10252 */
  0x0002,   /* U+10253 */
  0x0002,   /* U+10254 */
  0x0002,   /* U+10255 */
  0x0002,   /* U+10256 */
  0x0002,   /* U+10257 */
  0x0002,   /* U+10258 */
  0x0002,   /* U+10259 */
  0x0002,   /* U+1025A */
  0x0002,   /* U+1025B */
  0x0002,   /* U+1025C */
  0x0002,   /* U+1025D */
  0x0002,   /* U+1025E */
  0x0002,   /* U+1025F */
  0x0002,   /* U+10260 */
  0x0002,   /* U+10261 */
  0x0002,   /* U+10262 */
  0x0002,   /* U+10263 */
  0x0002,   /* U+10264 */
  0x0002,   /* U+10265 */
  0x0002,   /* U+10266 */
  0x0002,   /* U+10267 */
  0x0002,   /* U+10268 */
  0x0002,   /* U+10269 */
  0x0002,   /* U+1026A */
  0x0002,   /* U+1026B */
  0x0002,   /* U+1026C */
  0x0002,   /* U+1026D */
  0x0002,   /* U+1026E */
  0x0002,   /* U+1026F */
  0x0002,   /* U+10270 */
  0x0002,   /* U+10271 */
  0x0002,   /* U+10272 */
  0x0002,   /* U+10273 */
  0x0002,   /* U+10274 */
  0x0002,   /* U+10275 */
  0x0002,   /* U+10276 */
  0x0002,   /* U+10277 */
  0x0002,   /* U+10278 */
  0x0002,   /* U+10279 */
  0x0002,   /* U+1027A */
  0x0002,   /* U+1027B */
  0x0002,   /* U+1027C */
  0x0002,   /* U+1027D */
  0x0002,   /* U+1027E */
  0x0002,   /* U+1027F */
  0x0002,   /* U+10280 */
  0x0002,   /* U+10281 */
  0x0002,   /* U+10282 */
  0x0002,   /* U+10283 */
  0x0002,   /* U+10284 */
  0x0002,   /* U+10285 */
  0x0002,   /* U+10286 */
  0x0002,   /* U+10287 */
  0x0002,   /* U+10288 */
  0x0002,   /* U+10289 */
  0x0002,   /* U+1028A */
  0x0002,   /* U+1028B */
  0x0002,   /* U+1028C */
  0x0002,   /* U+1028D */
  0x0002,   /* U+1028E */
  0x0002,   /* U+1028F */
  0x0002,   /* U+10290 */
  0x0002,   /* U+10291 */
  0x0002,   /* U+10292 */
  0x0002,   /* U+10293 */
  0x0002,   /* U+10294 */
  0x0002,   /* U+10295 */
  0x0002,   /* U+10296 */
  0x0002,   /* U+10297 */
  0x0002,   /* U+10298 */
  0x0002,   /* U+10299 */
  0x0002,   /* U+1029A */
  0x0002,   /* U+1029B */
  0x0002,   /* U+1029C */
  0x0002,   /* U+1029D */
  0x0002,   /* U+1029E */
  0x0002,   /* U+1029F */
  0x0002,   /* U+102A0 */
  0x0002,   /* U+102A1 */
  0x0002,   /* U+102A2 */
  0x0002,   /* U+102A3 */
  0x0002,   /* U+102A4 */
  0x0002,   /* U+102A5 */
  0x0002,   /* U+102A6 */
  0x0002,   /* U+102A7 */
  0x0002,   /* U+102A8 */
  0x0002,   /* U+102A9 */
  0x0002,   /* U+102AA */
  0x0002,   /* U+102AB */
  0x0002,   /* U+102AC */
  0x0002,   /* U+102AD */
  0x0002,   /* U+102AE */
  0x0002,   /* U+102AF */
  0x0002,   /* U+102B0 */
  0x0002,   /* U+102B1 */
  0x0002,   /* U+102B2 */
  0x0002,   /* U+102B3 */
  0x0002,   /* U+102B4 */
  0x0002,   /* U+102B5 */
  0x0002,   /* U+102B6 */
  0x0002,   /* U+102B7 */
  0x0002,   /* U+102B8 */
  0x0002,   /* U+102B9 */
  0x0002,   /* U+102BA */
  0x0002,   /* U+102BB */
  0x0002,   /* U+102BC */
  0x0002,   /* U+102BD */
  0x0002,   /* U+102BE */
  0x0002,   /* U+102BF */
  0x0002,   /* U+102C0 */
  0x0002,   /* U+102C1 */
  0x0002,   /* U+102C2 */
  0x0002,   /* U+102C3 */
  0x0002,   /* U+102C4 */
  0x0002,   /* U+102C5 */
  0x0002,   /* U+102C6 */
  0x0002,   /* U+102C7 */
  0x0002,   /* U+102C8 */
  0x0002,   /* U+102C9 */
  0x0002,   /* U+102CA */
  0x0002,   /* U+102CB */
  0x0002,   /* U+102CC */
  0x0002,   /* U+102CD */
  0x0002,   /* U+102CE */
  0x0002,   /* U+102CF */
  0x0002,   /* U+102D0 */
  0x0002,   /* U+102D1 */
  0x0002,   /* U+102D2 */
  0x0002,   /* U+102D3 */
  0x0002,   /* U+102D4 */
  0x0002,   /* U+102D5 */
  0x0002,   /* U+102D6 */
  0x0002,   /* U+102D7 */
  0x0002,   /* U+102D8 */
  0x0002,   /* U+102D9 */
  0x0002,   /* U+102DA */
  0x0002,   /* U+102DB */
  0x0002,   /* U+102DC */
  0x0002,   /* U+102DD */
  0x0002,   /* U+102DE */
  0x0002,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x0002,   /* U+102E1 */
  0x0002,   /* U+102E2 */
  0x0002,   /* U+102E3 */
  0x0002,   /* U+102E4 */
  0x0002,   /* U+102E5 */
  0x0002,   /* U+102E6 */
  0x0002,   /* U+102E7 */
  0x0002,   /* U+102E8 */
  0x0002,   /* U+102E9 */
  0x0002,   /* U+102EA */
  0x0002,   /* U+102EB */
  0x0002,   /* U+102EC */
  0x0002,   /* U+102ED */
  0x0002,   /* U+102EE */
  0x0002,   /* U+102EF */
  0x0002,   /* U+102F0 */
  0x0002,   /* U+102F1 */
  0x0002,   /* U+102F2 */
  0x0002,   /* U+102F3 */
  0x0002,   /* U+102F4 */
  0x0002,   /* U+102F5 */
  0x0002,   /* U+102F6 */
  0x0002,   /* U+102F7 */
  0x0002,   /* U+102F8 */
  0x0002,   /* U+102F9 */
  0x0002,   /* U+102FA */
  0x0002,   /* U+102FB */
  0x0002,   /* U+102FC */
  0x0002,   /* U+102FD */
  0x0002,   /* U+102FE */
  0x0002,   /* U+102FF */

  /* Primary weight 2 for each character. */
  0x8200,   /* U+10200 */
  0x8201,   /* U+10201 */
  0x8202,   /* U+10202 */
  0x8203,   /* U+10203 */
  0x8204,   /* U+10204 */
  0x8205,   /* U+10205 */
  0x8206,   /* U+10206 */
  0x8207,   /* U+10207 */
  0x8208,   /* U+10208 */
  0x8209,   /* U+10209 */
  0x820A,   /* U+1020A */
  0x820B,   /* U+1020B */
  0x820C,   /* U+1020C */
  0x820D,   /* U+1020D */
  0x820E,   /* U+1020E */
  0x820F,   /* U+1020F */
  0x8210,   /* U+10210 */
  0x8211,   /* U+10211 */
  0x8212,   /* U+10212 */
  0x8213,   /* U+10213 */
  0x8214,   /* U+10214 */
  0x8215,   /* U+10215 */
  0x8216,   /* U+10216 */
  0x8217,   /* U+10217 */
  0x8218,   /* U+10218 */
  0x8219,   /* U+10219 */
  0x821A,   /* U+1021A */
  0x821B,   /* U+1021B */
  0x821C,   /* U+1021C */
  0x821D,   /* U+1021D */
  0x821E,   /* U+1021E */
  0x821F,   /* U+1021F */
  0x8220,   /* U+10220 */
  0x8221,   /* U+10221 */
  0x8222,   /* U+10222 */
  0x8223,   /* U+10223 */
  0x8224,   /* U+10224 */
  0x8225,   /* U+10225 */
  0x8226,   /* U+10226 */
  0x8227,   /* U+10227 */
  0x8228,   /* U+10228 */
  0x8229,   /* U+10229 */
  0x822A,   /* U+1022A */
  0x822B,   /* U+1022B */
  0x822C,   /* U+1022C */
  0x822D,   /* U+1022D */
  0x822E,   /* U+1022E */
  0x822F,   /* U+1022F */
  0x8230,   /* U+10230 */
  0x8231,   /* U+10231 */
  0x8232,   /* U+10232 */
  0x8233,   /* U+10233 */
  0x8234,   /* U+10234 */
  0x8235,   /* U+10235 */
  0x8236,   /* U+10236 */
  0x8237,   /* U+10237 */
  0x8238,   /* U+10238 */
  0x8239,   /* U+10239 */
  0x823A,   /* U+1023A */
  0x823B,   /* U+1023B */
  0x823C,   /* U+1023C */
  0x823D,   /* U+1023D */
  0x823E,   /* U+1023E */
  0x823F,   /* U+1023F */
  0x8240,   /* U+10240 */
  0x8241,   /* U+10241 */
  0x8242,   /* U+10242 */
  0x8243,   /* U+10243 */
  0x8244,   /* U+10244 */
  0x8245,   /* U+10245 */
  0x8246,   /* U+10246 */
  0x8247,   /* U+10247 */
  0x8248,   /* U+10248 */
  0x8249,   /* U+10249 */
  0x824A,   /* U+1024A */
  0x824B,   /* U+1024B */
  0x824C,   /* U+1024C */
  0x824D,   /* U+1024D */
  0x824E,   /* U+1024E */
  0x824F,   /* U+1024F */
  0x8250,   /* U+10250 */
  0x8251,   /* U+10251 */
  0x8252,   /* U+10252 */
  0x8253,   /* U+10253 */
  0x8254,   /* U+10254 */
  0x8255,   /* U+10255 */
  0x8256,   /* U+10256 */
  0x8257,   /* U+10257 */
  0x8258,   /* U+10258 */
  0x8259,   /* U+10259 */
  0x825A,   /* U+1025A */
  0x825B,   /* U+1025B */
  0x825C,   /* U+1025C */
  0x825D,   /* U+1025D */
  0x825E,   /* U+1025E */
  0x825F,   /* U+1025F */
  0x8260,   /* U+10260 */
  0x8261,   /* U+10261 */
  0x8262,   /* U+10262 */
  0x8263,   /* U+10263 */
  0x8264,   /* U+10264 */
  0x8265,   /* U+10265 */
  0x8266,   /* U+10266 */
  0x8267,   /* U+10267 */
  0x8268,   /* U+10268 */
  0x8269,   /* U+10269 */
  0x826A,   /* U+1026A */
  0x826B,   /* U+1026B */
  0x826C,   /* U+1026C */
  0x826D,   /* U+1026D */
  0x826E,   /* U+1026E */
  0x826F,   /* U+1026F */
  0x8270,   /* U+10270 */
  0x8271,   /* U+10271 */
  0x8272,   /* U+10272 */
  0x8273,   /* U+10273 */
  0x8274,   /* U+10274 */
  0x8275,   /* U+10275 */
  0x8276,   /* U+10276 */
  0x8277,   /* U+10277 */
  0x8278,   /* U+10278 */
  0x8279,   /* U+10279 */
  0x827A,   /* U+1027A */
  0x827B,   /* U+1027B */
  0x827C,   /* U+1027C */
  0x827D,   /* U+1027D */
  0x827E,   /* U+1027E */
  0x827F,   /* U+1027F */
  0x0000,   /* U+10280 */
  0x0000,   /* U+10281 */
  0x0000,   /* U+10282 */
  0x0000,   /* U+10283 */
  0x0000,   /* U+10284 */
  0x0000,   /* U+10285 */
  0x0000,   /* U+10286 */
  0x0000,   /* U+10287 */
  0x0000,   /* U+10288 */
  0x0000,   /* U+10289 */
  0x0000,   /* U+1028A */
  0x0000,   /* U+1028B */
  0x0000,   /* U+1028C */
  0x0000,   /* U+1028D */
  0x0000,   /* U+1028E */
  0x0000,   /* U+1028F */
  0x0000,   /* U+10290 */
  0x0000,   /* U+10291 */
  0x0000,   /* U+10292 */
  0x0000,   /* U+10293 */
  0x0000,   /* U+10294 */
  0x0000,   /* U+10295 */
  0x0000,   /* U+10296 */
  0x0000,   /* U+10297 */
  0x0000,   /* U+10298 */
  0x0000,   /* U+10299 */
  0x0000,   /* U+1029A */
  0x0000,   /* U+1029B */
  0x0000,   /* U+1029C */
  0x829D,   /* U+1029D */
  0x829E,   /* U+1029E */
  0x829F,   /* U+1029F */
  0x0000,   /* U+102A0 */
  0x0000,   /* U+102A1 */
  0x0000,   /* U+102A2 */
  0x0000,   /* U+102A3 */
  0x0000,   /* U+102A4 */
  0x0000,   /* U+102A5 */
  0x0000,   /* U+102A6 */
  0x0000,   /* U+102A7 */
  0x0000,   /* U+102A8 */
  0x0000,   /* U+102A9 */
  0x0000,   /* U+102AA */
  0x0000,   /* U+102AB */
  0x0000,   /* U+102AC */
  0x0000,   /* U+102AD */
  0x0000,   /* U+102AE */
  0x0000,   /* U+102AF */
  0x0000,   /* U+102B0 */
  0x0000,   /* U+102B1 */
  0x0000,   /* U+102B2 */
  0x0000,   /* U+102B3 */
  0x0000,   /* U+102B4 */
  0x0000,   /* U+102B5 */
  0x0000,   /* U+102B6 */
  0x0000,   /* U+102B7 */
  0x0000,   /* U+102B8 */
  0x0000,   /* U+102B9 */
  0x0000,   /* U+102BA */
  0x0000,   /* U+102BB */
  0x0000,   /* U+102BC */
  0x0000,   /* U+102BD */
  0x0000,   /* U+102BE */
  0x0000,   /* U+102BF */
  0x0000,   /* U+102C0 */
  0x0000,   /* U+102C1 */
  0x0000,   /* U+102C2 */
  0x0000,   /* U+102C3 */
  0x0000,   /* U+102C4 */
  0x0000,   /* U+102C5 */
  0x0000,   /* U+102C6 */
  0x0000,   /* U+102C7 */
  0x0000,   /* U+102C8 */
  0x0000,   /* U+102C9 */
  0x0000,   /* U+102CA */
  0x0000,   /* U+102CB */
  0x0000,   /* U+102CC */
  0x0000,   /* U+102CD */
  0x0000,   /* U+102CE */
  0x0000,   /* U+102CF */
  0x0000,   /* U+102D0 */
  0x82D1,   /* U+102D1 */
  0x82D2,   /* U+102D2 */
  0x82D3,   /* U+102D3 */
  0x82D4,   /* U+102D4 */
  0x82D5,   /* U+102D5 */
  0x82D6,   /* U+102D6 */
  0x82D7,   /* U+102D7 */
  0x82D8,   /* U+102D8 */
  0x82D9,   /* U+102D9 */
  0x82DA,   /* U+102DA */
  0x82DB,   /* U+102DB */
  0x82DC,   /* U+102DC */
  0x82DD,   /* U+102DD */
  0x82DE,   /* U+102DE */
  0x82DF,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x0000,   /* U+102E1 */
  0x0000,   /* U+102E2 */
  0x0000,   /* U+102E3 */
  0x0000,   /* U+102E4 */
  0x0000,   /* U+102E5 */
  0x0000,   /* U+102E6 */
  0x0000,   /* U+102E7 */
  0x0000,   /* U+102E8 */
  0x0000,   /* U+102E9 */
  0x0000,   /* U+102EA */
  0x0000,   /* U+102EB */
  0x0000,   /* U+102EC */
  0x0000,   /* U+102ED */
  0x0000,   /* U+102EE */
  0x0000,   /* U+102EF */
  0x0000,   /* U+102F0 */
  0x0000,   /* U+102F1 */
  0x0000,   /* U+102F2 */
  0x0000,   /* U+102F3 */
  0x0000,   /* U+102F4 */
  0x0000,   /* U+102F5 */
  0x0000,   /* U+102F6 */
  0x0000,   /* U+102F7 */
  0x0000,   /* U+102F8 */
  0x0000,   /* U+102F9 */
  0x0000,   /* U+102FA */
  0x0000,   /* U+102FB */
  0x82FC,   /* U+102FC */
  0x82FD,   /* U+102FD */
  0x82FE,   /* U+102FE */
  0x82FF,   /* U+102FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10200 */
  0x0000,   /* U+10201 */
  0x0000,   /* U+10202 */
  0x0000,   /* U+10203 */
  0x0000,   /* U+10204 */
  0x0000,   /* U+10205 */
  0x0000,   /* U+10206 */
  0x0000,   /* U+10207 */
  0x0000,   /* U+10208 */
  0x0000,   /* U+10209 */
  0x0000,   /* U+1020A */
  0x0000,   /* U+1020B */
  0x0000,   /* U+1020C */
  0x0000,   /* U+1020D */
  0x0000,   /* U+1020E */
  0x0000,   /* U+1020F */
  0x0000,   /* U+10210 */
  0x0000,   /* U+10211 */
  0x0000,   /* U+10212 */
  0x0000,   /* U+10213 */
  0x0000,   /* U+10214 */
  0x0000,   /* U+10215 */
  0x0000,   /* U+10216 */
  0x0000,   /* U+10217 */
  0x0000,   /* U+10218 */
  0x0000,   /* U+10219 */
  0x0000,   /* U+1021A */
  0x0000,   /* U+1021B */
  0x0000,   /* U+1021C */
  0x0000,   /* U+1021D */
  0x0000,   /* U+1021E */
  0x0000,   /* U+1021F */
  0x0000,   /* U+10220 */
  0x0000,   /* U+10221 */
  0x0000,   /* U+10222 */
  0x0000,   /* U+10223 */
  0x0000,   /* U+10224 */
  0x0000,   /* U+10225 */
  0x0000,   /* U+10226 */
  0x0000,   /* U+10227 */
  0x0000,   /* U+10228 */
  0x0000,   /* U+10229 */
  0x0000,   /* U+1022A */
  0x0000,   /* U+1022B */
  0x0000,   /* U+1022C */
  0x0000,   /* U+1022D */
  0x0000,   /* U+1022E */
  0x0000,   /* U+1022F */
  0x0000,   /* U+10230 */
  0x0000,   /* U+10231 */
  0x0000,   /* U+10232 */
  0x0000,   /* U+10233 */
  0x0000,   /* U+10234 */
  0x0000,   /* U+10235 */
  0x0000,   /* U+10236 */
  0x0000,   /* U+10237 */
  0x0000,   /* U+10238 */
  0x0000,   /* U+10239 */
  0x0000,   /* U+1023A */
  0x0000,   /* U+1023B */
  0x0000,   /* U+1023C */
  0x0000,   /* U+1023D */
  0x0000,   /* U+1023E */
  0x0000,   /* U+1023F */
  0x0000,   /* U+10240 */
  0x0000,   /* U+10241 */
  0x0000,   /* U+10242 */
  0x0000,   /* U+10243 */
  0x0000,   /* U+10244 */
  0x0000,   /* U+10245 */
  0x0000,   /* U+10246 */
  0x0000,   /* U+10247 */
  0x0000,   /* U+10248 */
  0x0000,   /* U+10249 */
  0x0000,   /* U+1024A */
  0x0000,   /* U+1024B */
  0x0000,   /* U+1024C */
  0x0000,   /* U+1024D */
  0x0000,   /* U+1024E */
  0x0000,   /* U+1024F */
  0x0000,   /* U+10250 */
  0x0000,   /* U+10251 */
  0x0000,   /* U+10252 */
  0x0000,   /* U+10253 */
  0x0000,   /* U+10254 */
  0x0000,   /* U+10255 */
  0x0000,   /* U+10256 */
  0x0000,   /* U+10257 */
  0x0000,   /* U+10258 */
  0x0000,   /* U+10259 */
  0x0000,   /* U+1025A */
  0x0000,   /* U+1025B */
  0x0000,   /* U+1025C */
  0x0000,   /* U+1025D */
  0x0000,   /* U+1025E */
  0x0000,   /* U+1025F */
  0x0000,   /* U+10260 */
  0x0000,   /* U+10261 */
  0x0000,   /* U+10262 */
  0x0000,   /* U+10263 */
  0x0000,   /* U+10264 */
  0x0000,   /* U+10265 */
  0x0000,   /* U+10266 */
  0x0000,   /* U+10267 */
  0x0000,   /* U+10268 */
  0x0000,   /* U+10269 */
  0x0000,   /* U+1026A */
  0x0000,   /* U+1026B */
  0x0000,   /* U+1026C */
  0x0000,   /* U+1026D */
  0x0000,   /* U+1026E */
  0x0000,   /* U+1026F */
  0x0000,   /* U+10270 */
  0x0000,   /* U+10271 */
  0x0000,   /* U+10272 */
  0x0000,   /* U+10273 */
  0x0000,   /* U+10274 */
  0x0000,   /* U+10275 */
  0x0000,   /* U+10276 */
  0x0000,   /* U+10277 */
  0x0000,   /* U+10278 */
  0x0000,   /* U+10279 */
  0x0000,   /* U+1027A */
  0x0000,   /* U+1027B */
  0x0000,   /* U+1027C */
  0x0000,   /* U+1027D */
  0x0000,   /* U+1027E */
  0x0000,   /* U+1027F */
  0x0000,   /* U+10280 */
  0x0000,   /* U+10281 */
  0x0000,   /* U+10282 */
  0x0000,   /* U+10283 */
  0x0000,   /* U+10284 */
  0x0000,   /* U+10285 */
  0x0000,   /* U+10286 */
  0x0000,   /* U+10287 */
  0x0000,   /* U+10288 */
  0x0000,   /* U+10289 */
  0x0000,   /* U+1028A */
  0x0000,   /* U+1028B */
  0x0000,   /* U+1028C */
  0x0000,   /* U+1028D */
  0x0000,   /* U+1028E */
  0x0000,   /* U+1028F */
  0x0000,   /* U+10290 */
  0x0000,   /* U+10291 */
  0x0000,   /* U+10292 */
  0x0000,   /* U+10293 */
  0x0000,   /* U+10294 */
  0x0000,   /* U+10295 */
  0x0000,   /* U+10296 */
  0x0000,   /* U+10297 */
  0x0000,   /* U+10298 */
  0x0000,   /* U+10299 */
  0x0000,   /* U+1029A */
  0x0000,   /* U+1029B */
  0x0000,   /* U+1029C */
  0x0000,   /* U+1029D */
  0x0000,   /* U+1029E */
  0x0000,   /* U+1029F */
  0x0000,   /* U+102A0 */
  0x0000,   /* U+102A1 */
  0x0000,   /* U+102A2 */
  0x0000,   /* U+102A3 */
  0x0000,   /* U+102A4 */
  0x0000,   /* U+102A5 */
  0x0000,   /* U+102A6 */
  0x0000,   /* U+102A7 */
  0x0000,   /* U+102A8 */
  0x0000,   /* U+102A9 */
  0x0000,   /* U+102AA */
  0x0000,   /* U+102AB */
  0x0000,   /* U+102AC */
  0x0000,   /* U+102AD */
  0x0000,   /* U+102AE */
  0x0000,   /* U+102AF */
  0x0000,   /* U+102B0 */
  0x0000,   /* U+102B1 */
  0x0000,   /* U+102B2 */
  0x0000,   /* U+102B3 */
  0x0000,   /* U+102B4 */
  0x0000,   /* U+102B5 */
  0x0000,   /* U+102B6 */
  0x0000,   /* U+102B7 */
  0x0000,   /* U+102B8 */
  0x0000,   /* U+102B9 */
  0x0000,   /* U+102BA */
  0x0000,   /* U+102BB */
  0x0000,   /* U+102BC */
  0x0000,   /* U+102BD */
  0x0000,   /* U+102BE */
  0x0000,   /* U+102BF */
  0x0000,   /* U+102C0 */
  0x0000,   /* U+102C1 */
  0x0000,   /* U+102C2 */
  0x0000,   /* U+102C3 */
  0x0000,   /* U+102C4 */
  0x0000,   /* U+102C5 */
  0x0000,   /* U+102C6 */
  0x0000,   /* U+102C7 */
  0x0000,   /* U+102C8 */
  0x0000,   /* U+102C9 */
  0x0000,   /* U+102CA */
  0x0000,   /* U+102CB */
  0x0000,   /* U+102CC */
  0x0000,   /* U+102CD */
  0x0000,   /* U+102CE */
  0x0000,   /* U+102CF */
  0x0000,   /* U+102D0 */
  0x0000,   /* U+102D1 */
  0x0000,   /* U+102D2 */
  0x0000,   /* U+102D3 */
  0x0000,   /* U+102D4 */
  0x0000,   /* U+102D5 */
  0x0000,   /* U+102D6 */
  0x0000,   /* U+102D7 */
  0x0000,   /* U+102D8 */
  0x0000,   /* U+102D9 */
  0x0000,   /* U+102DA */
  0x0000,   /* U+102DB */
  0x0000,   /* U+102DC */
  0x0000,   /* U+102DD */
  0x0000,   /* U+102DE */
  0x0000,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x0000,   /* U+102E1 */
  0x0000,   /* U+102E2 */
  0x0000,   /* U+102E3 */
  0x0000,   /* U+102E4 */
  0x0000,   /* U+102E5 */
  0x0000,   /* U+102E6 */
  0x0000,   /* U+102E7 */
  0x0000,   /* U+102E8 */
  0x0000,   /* U+102E9 */
  0x0000,   /* U+102EA */
  0x0000,   /* U+102EB */
  0x0000,   /* U+102EC */
  0x0000,   /* U+102ED */
  0x0000,   /* U+102EE */
  0x0000,   /* U+102EF */
  0x0000,   /* U+102F0 */
  0x0000,   /* U+102F1 */
  0x0000,   /* U+102F2 */
  0x0000,   /* U+102F3 */
  0x0000,   /* U+102F4 */
  0x0000,   /* U+102F5 */
  0x0000,   /* U+102F6 */
  0x0000,   /* U+102F7 */
  0x0000,   /* U+102F8 */
  0x0000,   /* U+102F9 */
  0x0000,   /* U+102FA */
  0x0000,   /* U+102FB */
  0x0000,   /* U+102FC */
  0x0000,   /* U+102FD */
  0x0000,   /* U+102FE */
  0x0000,   /* U+102FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10200 */
  0x0000,   /* U+10201 */
  0x0000,   /* U+10202 */
  0x0000,   /* U+10203 */
  0x0000,   /* U+10204 */
  0x0000,   /* U+10205 */
  0x0000,   /* U+10206 */
  0x0000,   /* U+10207 */
  0x0000,   /* U+10208 */
  0x0000,   /* U+10209 */
  0x0000,   /* U+1020A */
  0x0000,   /* U+1020B */
  0x0000,   /* U+1020C */
  0x0000,   /* U+1020D */
  0x0000,   /* U+1020E */
  0x0000,   /* U+1020F */
  0x0000,   /* U+10210 */
  0x0000,   /* U+10211 */
  0x0000,   /* U+10212 */
  0x0000,   /* U+10213 */
  0x0000,   /* U+10214 */
  0x0000,   /* U+10215 */
  0x0000,   /* U+10216 */
  0x0000,   /* U+10217 */
  0x0000,   /* U+10218 */
  0x0000,   /* U+10219 */
  0x0000,   /* U+1021A */
  0x0000,   /* U+1021B */
  0x0000,   /* U+1021C */
  0x0000,   /* U+1021D */
  0x0000,   /* U+1021E */
  0x0000,   /* U+1021F */
  0x0000,   /* U+10220 */
  0x0000,   /* U+10221 */
  0x0000,   /* U+10222 */
  0x0000,   /* U+10223 */
  0x0000,   /* U+10224 */
  0x0000,   /* U+10225 */
  0x0000,   /* U+10226 */
  0x0000,   /* U+10227 */
  0x0000,   /* U+10228 */
  0x0000,   /* U+10229 */
  0x0000,   /* U+1022A */
  0x0000,   /* U+1022B */
  0x0000,   /* U+1022C */
  0x0000,   /* U+1022D */
  0x0000,   /* U+1022E */
  0x0000,   /* U+1022F */
  0x0000,   /* U+10230 */
  0x0000,   /* U+10231 */
  0x0000,   /* U+10232 */
  0x0000,   /* U+10233 */
  0x0000,   /* U+10234 */
  0x0000,   /* U+10235 */
  0x0000,   /* U+10236 */
  0x0000,   /* U+10237 */
  0x0000,   /* U+10238 */
  0x0000,   /* U+10239 */
  0x0000,   /* U+1023A */
  0x0000,   /* U+1023B */
  0x0000,   /* U+1023C */
  0x0000,   /* U+1023D */
  0x0000,   /* U+1023E */
  0x0000,   /* U+1023F */
  0x0000,   /* U+10240 */
  0x0000,   /* U+10241 */
  0x0000,   /* U+10242 */
  0x0000,   /* U+10243 */
  0x0000,   /* U+10244 */
  0x0000,   /* U+10245 */
  0x0000,   /* U+10246 */
  0x0000,   /* U+10247 */
  0x0000,   /* U+10248 */
  0x0000,   /* U+10249 */
  0x0000,   /* U+1024A */
  0x0000,   /* U+1024B */
  0x0000,   /* U+1024C */
  0x0000,   /* U+1024D */
  0x0000,   /* U+1024E */
  0x0000,   /* U+1024F */
  0x0000,   /* U+10250 */
  0x0000,   /* U+10251 */
  0x0000,   /* U+10252 */
  0x0000,   /* U+10253 */
  0x0000,   /* U+10254 */
  0x0000,   /* U+10255 */
  0x0000,   /* U+10256 */
  0x0000,   /* U+10257 */
  0x0000,   /* U+10258 */
  0x0000,   /* U+10259 */
  0x0000,   /* U+1025A */
  0x0000,   /* U+1025B */
  0x0000,   /* U+1025C */
  0x0000,   /* U+1025D */
  0x0000,   /* U+1025E */
  0x0000,   /* U+1025F */
  0x0000,   /* U+10260 */
  0x0000,   /* U+10261 */
  0x0000,   /* U+10262 */
  0x0000,   /* U+10263 */
  0x0000,   /* U+10264 */
  0x0000,   /* U+10265 */
  0x0000,   /* U+10266 */
  0x0000,   /* U+10267 */
  0x0000,   /* U+10268 */
  0x0000,   /* U+10269 */
  0x0000,   /* U+1026A */
  0x0000,   /* U+1026B */
  0x0000,   /* U+1026C */
  0x0000,   /* U+1026D */
  0x0000,   /* U+1026E */
  0x0000,   /* U+1026F */
  0x0000,   /* U+10270 */
  0x0000,   /* U+10271 */
  0x0000,   /* U+10272 */
  0x0000,   /* U+10273 */
  0x0000,   /* U+10274 */
  0x0000,   /* U+10275 */
  0x0000,   /* U+10276 */
  0x0000,   /* U+10277 */
  0x0000,   /* U+10278 */
  0x0000,   /* U+10279 */
  0x0000,   /* U+1027A */
  0x0000,   /* U+1027B */
  0x0000,   /* U+1027C */
  0x0000,   /* U+1027D */
  0x0000,   /* U+1027E */
  0x0000,   /* U+1027F */
  0x0000,   /* U+10280 */
  0x0000,   /* U+10281 */
  0x0000,   /* U+10282 */
  0x0000,   /* U+10283 */
  0x0000,   /* U+10284 */
  0x0000,   /* U+10285 */
  0x0000,   /* U+10286 */
  0x0000,   /* U+10287 */
  0x0000,   /* U+10288 */
  0x0000,   /* U+10289 */
  0x0000,   /* U+1028A */
  0x0000,   /* U+1028B */
  0x0000,   /* U+1028C */
  0x0000,   /* U+1028D */
  0x0000,   /* U+1028E */
  0x0000,   /* U+1028F */
  0x0000,   /* U+10290 */
  0x0000,   /* U+10291 */
  0x0000,   /* U+10292 */
  0x0000,   /* U+10293 */
  0x0000,   /* U+10294 */
  0x0000,   /* U+10295 */
  0x0000,   /* U+10296 */
  0x0000,   /* U+10297 */
  0x0000,   /* U+10298 */
  0x0000,   /* U+10299 */
  0x0000,   /* U+1029A */
  0x0000,   /* U+1029B */
  0x0000,   /* U+1029C */
  0x0000,   /* U+1029D */
  0x0000,   /* U+1029E */
  0x0000,   /* U+1029F */
  0x0000,   /* U+102A0 */
  0x0000,   /* U+102A1 */
  0x0000,   /* U+102A2 */
  0x0000,   /* U+102A3 */
  0x0000,   /* U+102A4 */
  0x0000,   /* U+102A5 */
  0x0000,   /* U+102A6 */
  0x0000,   /* U+102A7 */
  0x0000,   /* U+102A8 */
  0x0000,   /* U+102A9 */
  0x0000,   /* U+102AA */
  0x0000,   /* U+102AB */
  0x0000,   /* U+102AC */
  0x0000,   /* U+102AD */
  0x0000,   /* U+102AE */
  0x0000,   /* U+102AF */
  0x0000,   /* U+102B0 */
  0x0000,   /* U+102B1 */
  0x0000,   /* U+102B2 */
  0x0000,   /* U+102B3 */
  0x0000,   /* U+102B4 */
  0x0000,   /* U+102B5 */
  0x0000,   /* U+102B6 */
  0x0000,   /* U+102B7 */
  0x0000,   /* U+102B8 */
  0x0000,   /* U+102B9 */
  0x0000,   /* U+102BA */
  0x0000,   /* U+102BB */
  0x0000,   /* U+102BC */
  0x0000,   /* U+102BD */
  0x0000,   /* U+102BE */
  0x0000,   /* U+102BF */
  0x0000,   /* U+102C0 */
  0x0000,   /* U+102C1 */
  0x0000,   /* U+102C2 */
  0x0000,   /* U+102C3 */
  0x0000,   /* U+102C4 */
  0x0000,   /* U+102C5 */
  0x0000,   /* U+102C6 */
  0x0000,   /* U+102C7 */
  0x0000,   /* U+102C8 */
  0x0000,   /* U+102C9 */
  0x0000,   /* U+102CA */
  0x0000,   /* U+102CB */
  0x0000,   /* U+102CC */
  0x0000,   /* U+102CD */
  0x0000,   /* U+102CE */
  0x0000,   /* U+102CF */
  0x0000,   /* U+102D0 */
  0x0000,   /* U+102D1 */
  0x0000,   /* U+102D2 */
  0x0000,   /* U+102D3 */
  0x0000,   /* U+102D4 */
  0x0000,   /* U+102D5 */
  0x0000,   /* U+102D6 */
  0x0000,   /* U+102D7 */
  0x0000,   /* U+102D8 */
  0x0000,   /* U+102D9 */
  0x0000,   /* U+102DA */
  0x0000,   /* U+102DB */
  0x0000,   /* U+102DC */
  0x0000,   /* U+102DD */
  0x0000,   /* U+102DE */
  0x0000,   /* U+102DF */
  0x0000,   /* U+102E0 */
  0x0000,   /* U+102E1 */
  0x0000,   /* U+102E2 */
  0x0000,   /* U+102E3 */
  0x0000,   /* U+102E4 */
  0x0000,   /* U+102E5 */
  0x0000,   /* U+102E6 */
  0x0000,   /* U+102E7 */
  0x0000,   /* U+102E8 */
  0x0000,   /* U+102E9 */
  0x0000,   /* U+102EA */
  0x0000,   /* U+102EB */
  0x0000,   /* U+102EC */
  0x0000,   /* U+102ED */
  0x0000,   /* U+102EE */
  0x0000,   /* U+102EF */
  0x0000,   /* U+102F0 */
  0x0000,   /* U+102F1 */
  0x0000,   /* U+102F2 */
  0x0000,   /* U+102F3 */
  0x0000,   /* U+102F4 */
  0x0000,   /* U+102F5 */
  0x0000,   /* U+102F6 */
  0x0000,   /* U+102F7 */
  0x0000,   /* U+102F8 */
  0x0000,   /* U+102F9 */
  0x0000,   /* U+102FA */
  0x0000,   /* U+102FB */
  0x0000,   /* U+102FC */
  0x0000,   /* U+102FD */
  0x0000,   /* U+102FE */
  0x0000,   /* U+102FF */
};

uint16 uca900_p103[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4417,   /* U+10300 */
  0x4418,   /* U+10301 */
  0x4419,   /* U+10302 */
  0x441A,   /* U+10303 */
  0x441B,   /* U+10304 */
  0x441C,   /* U+10305 */
  0x441D,   /* U+10306 */
  0x441E,   /* U+10307 */
  0x441F,   /* U+10308 */
  0x4420,   /* U+10309 */
  0x4421,   /* U+1030A */
  0x4422,   /* U+1030B */
  0x4423,   /* U+1030C */
  0x4424,   /* U+1030D */
  0x4425,   /* U+1030E */
  0x4427,   /* U+1030F */
  0x4428,   /* U+10310 */
  0x4429,   /* U+10311 */
  0x442A,   /* U+10312 */
  0x442B,   /* U+10313 */
  0x442C,   /* U+10314 */
  0x442D,   /* U+10315 */
  0x442E,   /* U+10316 */
  0x442F,   /* U+10317 */
  0x4430,   /* U+10318 */
  0x4431,   /* U+10319 */
  0x4432,   /* U+1031A */
  0x4433,   /* U+1031B */
  0x4434,   /* U+1031C */
  0x4435,   /* U+1031D */
  0x4436,   /* U+1031E */
  0x4426,   /* U+1031F */
  0x1C3E,   /* U+10320 */
  0x1C42,   /* U+10321 */
  0x1AE3,   /* U+10322 */
  0x1AE4,   /* U+10323 */
  0xFBC2,   /* U+10324 */
  0xFBC2,   /* U+10325 */
  0xFBC2,   /* U+10326 */
  0xFBC2,   /* U+10327 */
  0xFBC2,   /* U+10328 */
  0xFBC2,   /* U+10329 */
  0xFBC2,   /* U+1032A */
  0xFBC2,   /* U+1032B */
  0xFBC2,   /* U+1032C */
  0xFBC2,   /* U+1032D */
  0xFBC2,   /* U+1032E */
  0xFBC2,   /* U+1032F */
  0x4437,   /* U+10330 */
  0x4438,   /* U+10331 */
  0x4439,   /* U+10332 */
  0x443A,   /* U+10333 */
  0x443B,   /* U+10334 */
  0x443C,   /* U+10335 */
  0x443D,   /* U+10336 */
  0x443E,   /* U+10337 */
  0x443F,   /* U+10338 */
  0x4440,   /* U+10339 */
  0x4441,   /* U+1033A */
  0x4442,   /* U+1033B */
  0x4443,   /* U+1033C */
  0x4444,   /* U+1033D */
  0x4445,   /* U+1033E */
  0x4446,   /* U+1033F */
  0x4447,   /* U+10340 */
  0x4448,   /* U+10341 */
  0x4449,   /* U+10342 */
  0x444A,   /* U+10343 */
  0x444B,   /* U+10344 */
  0x444C,   /* U+10345 */
  0x444D,   /* U+10346 */
  0x444E,   /* U+10347 */
  0x444F,   /* U+10348 */
  0x4450,   /* U+10349 */
  0x4451,   /* U+1034A */
  0xFBC2,   /* U+1034B */
  0xFBC2,   /* U+1034C */
  0xFBC2,   /* U+1034D */
  0xFBC2,   /* U+1034E */
  0xFBC2,   /* U+1034F */
  0x2214,   /* U+10350 */
  0x2215,   /* U+10351 */
  0x2216,   /* U+10352 */
  0x2217,   /* U+10353 */
  0x2218,   /* U+10354 */
  0x2219,   /* U+10355 */
  0x221A,   /* U+10356 */
  0x221B,   /* U+10357 */
  0x221C,   /* U+10358 */
  0x221D,   /* U+10359 */
  0x221E,   /* U+1035A */
  0x221F,   /* U+1035B */
  0x2220,   /* U+1035C */
  0x2221,   /* U+1035D */
  0x2222,   /* U+1035E */
  0x2223,   /* U+1035F */
  0x2224,   /* U+10360 */
  0x2225,   /* U+10361 */
  0x2226,   /* U+10362 */
  0x2227,   /* U+10363 */
  0x2228,   /* U+10364 */
  0x2229,   /* U+10365 */
  0x222A,   /* U+10366 */
  0x222B,   /* U+10367 */
  0x222C,   /* U+10368 */
  0x222D,   /* U+10369 */
  0x222E,   /* U+1036A */
  0x222F,   /* U+1036B */
  0x2230,   /* U+1036C */
  0x2231,   /* U+1036D */
  0x2232,   /* U+1036E */
  0x2233,   /* U+1036F */
  0x2234,   /* U+10370 */
  0x2235,   /* U+10371 */
  0x2236,   /* U+10372 */
  0x2237,   /* U+10373 */
  0x2238,   /* U+10374 */
  0x2239,   /* U+10375 */
  0x2214,   /* U+10376 */
  0x2217,   /* U+10377 */
  0x221B,   /* U+10378 */
  0x2221,   /* U+10379 */
  0x2225,   /* U+1037A */
  0xFBC2,   /* U+1037B */
  0xFBC2,   /* U+1037C */
  0xFBC2,   /* U+1037D */
  0xFBC2,   /* U+1037E */
  0xFBC2,   /* U+1037F */
  0x496B,   /* U+10380 */
  0x496C,   /* U+10381 */
  0x496D,   /* U+10382 */
  0x496E,   /* U+10383 */
  0x496F,   /* U+10384 */
  0x4970,   /* U+10385 */
  0x4971,   /* U+10386 */
  0x4972,   /* U+10387 */
  0x4973,   /* U+10388 */
  0x4974,   /* U+10389 */
  0x4975,   /* U+1038A */
  0x4976,   /* U+1038B */
  0x4977,   /* U+1038C */
  0x4978,   /* U+1038D */
  0x4979,   /* U+1038E */
  0x497A,   /* U+1038F */
  0x497B,   /* U+10390 */
  0x497C,   /* U+10391 */
  0x497D,   /* U+10392 */
  0x497E,   /* U+10393 */
  0x497F,   /* U+10394 */
  0x4980,   /* U+10395 */
  0x4981,   /* U+10396 */
  0x4982,   /* U+10397 */
  0x4983,   /* U+10398 */
  0x4984,   /* U+10399 */
  0x4985,   /* U+1039A */
  0x4986,   /* U+1039B */
  0x4987,   /* U+1039C */
  0x4988,   /* U+1039D */
  0xFBC2,   /* U+1039E */
  0x02FD,   /* U+1039F */
  0x4989,   /* U+103A0 */
  0x498A,   /* U+103A1 */
  0x498B,   /* U+103A2 */
  0x498C,   /* U+103A3 */
  0x498D,   /* U+103A4 */
  0x498E,   /* U+103A5 */
  0x498F,   /* U+103A6 */
  0x4990,   /* U+103A7 */
  0x4991,   /* U+103A8 */
  0x4992,   /* U+103A9 */
  0x4993,   /* U+103AA */
  0x4994,   /* U+103AB */
  0x4995,   /* U+103AC */
  0x4996,   /* U+103AD */
  0x4997,   /* U+103AE */
  0x4998,   /* U+103AF */
  0x4999,   /* U+103B0 */
  0x499A,   /* U+103B1 */
  0x499B,   /* U+103B2 */
  0x499C,   /* U+103B3 */
  0x499D,   /* U+103B4 */
  0x499E,   /* U+103B5 */
  0x499F,   /* U+103B6 */
  0x49A0,   /* U+103B7 */
  0x49A1,   /* U+103B8 */
  0x49A2,   /* U+103B9 */
  0x49A3,   /* U+103BA */
  0x49A4,   /* U+103BB */
  0x49A5,   /* U+103BC */
  0x49A6,   /* U+103BD */
  0x49A7,   /* U+103BE */
  0x49A8,   /* U+103BF */
  0x49A9,   /* U+103C0 */
  0x49AA,   /* U+103C1 */
  0x49AB,   /* U+103C2 */
  0x49AC,   /* U+103C3 */
  0xFBC2,   /* U+103C4 */
  0xFBC2,   /* U+103C5 */
  0xFBC2,   /* U+103C6 */
  0xFBC2,   /* U+103C7 */
  0x49AD,   /* U+103C8 */
  0x49AE,   /* U+103C9 */
  0x49AF,   /* U+103CA */
  0x49B0,   /* U+103CB */
  0x49B1,   /* U+103CC */
  0x49B2,   /* U+103CD */
  0x49B3,   /* U+103CE */
  0x49B4,   /* U+103CF */
  0x02FE,   /* U+103D0 */
  0x1C3E,   /* U+103D1 */
  0x1C3F,   /* U+103D2 */
  0x1B4C,   /* U+103D3 */
  0x1B4D,   /* U+103D4 */
  0x1B4E,   /* U+103D5 */
  0xFBC2,   /* U+103D6 */
  0xFBC2,   /* U+103D7 */
  0xFBC2,   /* U+103D8 */
  0xFBC2,   /* U+103D9 */
  0xFBC2,   /* U+103DA */
  0xFBC2,   /* U+103DB */
  0xFBC2,   /* U+103DC */
  0xFBC2,   /* U+103DD */
  0xFBC2,   /* U+103DE */
  0xFBC2,   /* U+103DF */
  0xFBC2,   /* U+103E0 */
  0xFBC2,   /* U+103E1 */
  0xFBC2,   /* U+103E2 */
  0xFBC2,   /* U+103E3 */
  0xFBC2,   /* U+103E4 */
  0xFBC2,   /* U+103E5 */
  0xFBC2,   /* U+103E6 */
  0xFBC2,   /* U+103E7 */
  0xFBC2,   /* U+103E8 */
  0xFBC2,   /* U+103E9 */
  0xFBC2,   /* U+103EA */
  0xFBC2,   /* U+103EB */
  0xFBC2,   /* U+103EC */
  0xFBC2,   /* U+103ED */
  0xFBC2,   /* U+103EE */
  0xFBC2,   /* U+103EF */
  0xFBC2,   /* U+103F0 */
  0xFBC2,   /* U+103F1 */
  0xFBC2,   /* U+103F2 */
  0xFBC2,   /* U+103F3 */
  0xFBC2,   /* U+103F4 */
  0xFBC2,   /* U+103F5 */
  0xFBC2,   /* U+103F6 */
  0xFBC2,   /* U+103F7 */
  0xFBC2,   /* U+103F8 */
  0xFBC2,   /* U+103F9 */
  0xFBC2,   /* U+103FA */
  0xFBC2,   /* U+103FB */
  0xFBC2,   /* U+103FC */
  0xFBC2,   /* U+103FD */
  0xFBC2,   /* U+103FE */
  0xFBC2,   /* U+103FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10300 */
  0x0020,   /* U+10301 */
  0x0020,   /* U+10302 */
  0x0020,   /* U+10303 */
  0x0020,   /* U+10304 */
  0x0020,   /* U+10305 */
  0x0020,   /* U+10306 */
  0x0020,   /* U+10307 */
  0x0020,   /* U+10308 */
  0x0020,   /* U+10309 */
  0x0020,   /* U+1030A */
  0x0020,   /* U+1030B */
  0x0020,   /* U+1030C */
  0x0020,   /* U+1030D */
  0x0020,   /* U+1030E */
  0x0020,   /* U+1030F */
  0x0020,   /* U+10310 */
  0x0020,   /* U+10311 */
  0x0020,   /* U+10312 */
  0x0020,   /* U+10313 */
  0x0020,   /* U+10314 */
  0x0020,   /* U+10315 */
  0x0020,   /* U+10316 */
  0x0020,   /* U+10317 */
  0x0020,   /* U+10318 */
  0x0020,   /* U+10319 */
  0x0020,   /* U+1031A */
  0x0020,   /* U+1031B */
  0x0020,   /* U+1031C */
  0x0020,   /* U+1031D */
  0x0020,   /* U+1031E */
  0x0020,   /* U+1031F */
  0x0020,   /* U+10320 */
  0x0020,   /* U+10321 */
  0x0020,   /* U+10322 */
  0x0020,   /* U+10323 */
  0x0020,   /* U+10324 */
  0x0020,   /* U+10325 */
  0x0020,   /* U+10326 */
  0x0020,   /* U+10327 */
  0x0020,   /* U+10328 */
  0x0020,   /* U+10329 */
  0x0020,   /* U+1032A */
  0x0020,   /* U+1032B */
  0x0020,   /* U+1032C */
  0x0020,   /* U+1032D */
  0x0020,   /* U+1032E */
  0x0020,   /* U+1032F */
  0x0020,   /* U+10330 */
  0x0020,   /* U+10331 */
  0x0020,   /* U+10332 */
  0x0020,   /* U+10333 */
  0x0020,   /* U+10334 */
  0x0020,   /* U+10335 */
  0x0020,   /* U+10336 */
  0x0020,   /* U+10337 */
  0x0020,   /* U+10338 */
  0x0020,   /* U+10339 */
  0x0020,   /* U+1033A */
  0x0020,   /* U+1033B */
  0x0020,   /* U+1033C */
  0x0020,   /* U+1033D */
  0x0020,   /* U+1033E */
  0x0020,   /* U+1033F */
  0x0020,   /* U+10340 */
  0x0020,   /* U+10341 */
  0x0020,   /* U+10342 */
  0x0020,   /* U+10343 */
  0x0020,   /* U+10344 */
  0x0020,   /* U+10345 */
  0x0020,   /* U+10346 */
  0x0020,   /* U+10347 */
  0x0020,   /* U+10348 */
  0x0020,   /* U+10349 */
  0x0020,   /* U+1034A */
  0x0020,   /* U+1034B */
  0x0020,   /* U+1034C */
  0x0020,   /* U+1034D */
  0x0020,   /* U+1034E */
  0x0020,   /* U+1034F */
  0x0020,   /* U+10350 */
  0x0020,   /* U+10351 */
  0x0020,   /* U+10352 */
  0x0020,   /* U+10353 */
  0x0020,   /* U+10354 */
  0x0020,   /* U+10355 */
  0x0020,   /* U+10356 */
  0x0020,   /* U+10357 */
  0x0020,   /* U+10358 */
  0x0020,   /* U+10359 */
  0x0020,   /* U+1035A */
  0x0020,   /* U+1035B */
  0x0020,   /* U+1035C */
  0x0020,   /* U+1035D */
  0x0020,   /* U+1035E */
  0x0020,   /* U+1035F */
  0x0020,   /* U+10360 */
  0x0020,   /* U+10361 */
  0x0020,   /* U+10362 */
  0x0020,   /* U+10363 */
  0x0020,   /* U+10364 */
  0x0020,   /* U+10365 */
  0x0020,   /* U+10366 */
  0x0020,   /* U+10367 */
  0x0020,   /* U+10368 */
  0x0020,   /* U+10369 */
  0x0020,   /* U+1036A */
  0x0020,   /* U+1036B */
  0x0020,   /* U+1036C */
  0x0020,   /* U+1036D */
  0x0020,   /* U+1036E */
  0x0020,   /* U+1036F */
  0x0020,   /* U+10370 */
  0x0020,   /* U+10371 */
  0x0020,   /* U+10372 */
  0x0020,   /* U+10373 */
  0x0020,   /* U+10374 */
  0x0020,   /* U+10375 */
  0x0020,   /* U+10376 */
  0x0020,   /* U+10377 */
  0x0020,   /* U+10378 */
  0x0020,   /* U+10379 */
  0x0020,   /* U+1037A */
  0x0020,   /* U+1037B */
  0x0020,   /* U+1037C */
  0x0020,   /* U+1037D */
  0x0020,   /* U+1037E */
  0x0020,   /* U+1037F */
  0x0020,   /* U+10380 */
  0x0020,   /* U+10381 */
  0x0020,   /* U+10382 */
  0x0020,   /* U+10383 */
  0x0020,   /* U+10384 */
  0x0020,   /* U+10385 */
  0x0020,   /* U+10386 */
  0x0020,   /* U+10387 */
  0x0020,   /* U+10388 */
  0x0020,   /* U+10389 */
  0x0020,   /* U+1038A */
  0x0020,   /* U+1038B */
  0x0020,   /* U+1038C */
  0x0020,   /* U+1038D */
  0x0020,   /* U+1038E */
  0x0020,   /* U+1038F */
  0x0020,   /* U+10390 */
  0x0020,   /* U+10391 */
  0x0020,   /* U+10392 */
  0x0020,   /* U+10393 */
  0x0020,   /* U+10394 */
  0x0020,   /* U+10395 */
  0x0020,   /* U+10396 */
  0x0020,   /* U+10397 */
  0x0020,   /* U+10398 */
  0x0020,   /* U+10399 */
  0x0020,   /* U+1039A */
  0x0020,   /* U+1039B */
  0x0020,   /* U+1039C */
  0x0020,   /* U+1039D */
  0x0020,   /* U+1039E */
  0x0020,   /* U+1039F */
  0x0020,   /* U+103A0 */
  0x0020,   /* U+103A1 */
  0x0020,   /* U+103A2 */
  0x0020,   /* U+103A3 */
  0x0020,   /* U+103A4 */
  0x0020,   /* U+103A5 */
  0x0020,   /* U+103A6 */
  0x0020,   /* U+103A7 */
  0x0020,   /* U+103A8 */
  0x0020,   /* U+103A9 */
  0x0020,   /* U+103AA */
  0x0020,   /* U+103AB */
  0x0020,   /* U+103AC */
  0x0020,   /* U+103AD */
  0x0020,   /* U+103AE */
  0x0020,   /* U+103AF */
  0x0020,   /* U+103B0 */
  0x0020,   /* U+103B1 */
  0x0020,   /* U+103B2 */
  0x0020,   /* U+103B3 */
  0x0020,   /* U+103B4 */
  0x0020,   /* U+103B5 */
  0x0020,   /* U+103B6 */
  0x0020,   /* U+103B7 */
  0x0020,   /* U+103B8 */
  0x0020,   /* U+103B9 */
  0x0020,   /* U+103BA */
  0x0020,   /* U+103BB */
  0x0020,   /* U+103BC */
  0x0020,   /* U+103BD */
  0x0020,   /* U+103BE */
  0x0020,   /* U+103BF */
  0x0020,   /* U+103C0 */
  0x0020,   /* U+103C1 */
  0x0020,   /* U+103C2 */
  0x0020,   /* U+103C3 */
  0x0020,   /* U+103C4 */
  0x0020,   /* U+103C5 */
  0x0020,   /* U+103C6 */
  0x0020,   /* U+103C7 */
  0x0020,   /* U+103C8 */
  0x0020,   /* U+103C9 */
  0x0020,   /* U+103CA */
  0x0020,   /* U+103CB */
  0x0020,   /* U+103CC */
  0x0020,   /* U+103CD */
  0x0020,   /* U+103CE */
  0x0020,   /* U+103CF */
  0x0020,   /* U+103D0 */
  0x0020,   /* U+103D1 */
  0x0020,   /* U+103D2 */
  0x0020,   /* U+103D3 */
  0x0020,   /* U+103D4 */
  0x0020,   /* U+103D5 */
  0x0020,   /* U+103D6 */
  0x0020,   /* U+103D7 */
  0x0020,   /* U+103D8 */
  0x0020,   /* U+103D9 */
  0x0020,   /* U+103DA */
  0x0020,   /* U+103DB */
  0x0020,   /* U+103DC */
  0x0020,   /* U+103DD */
  0x0020,   /* U+103DE */
  0x0020,   /* U+103DF */
  0x0020,   /* U+103E0 */
  0x0020,   /* U+103E1 */
  0x0020,   /* U+103E2 */
  0x0020,   /* U+103E3 */
  0x0020,   /* U+103E4 */
  0x0020,   /* U+103E5 */
  0x0020,   /* U+103E6 */
  0x0020,   /* U+103E7 */
  0x0020,   /* U+103E8 */
  0x0020,   /* U+103E9 */
  0x0020,   /* U+103EA */
  0x0020,   /* U+103EB */
  0x0020,   /* U+103EC */
  0x0020,   /* U+103ED */
  0x0020,   /* U+103EE */
  0x0020,   /* U+103EF */
  0x0020,   /* U+103F0 */
  0x0020,   /* U+103F1 */
  0x0020,   /* U+103F2 */
  0x0020,   /* U+103F3 */
  0x0020,   /* U+103F4 */
  0x0020,   /* U+103F5 */
  0x0020,   /* U+103F6 */
  0x0020,   /* U+103F7 */
  0x0020,   /* U+103F8 */
  0x0020,   /* U+103F9 */
  0x0020,   /* U+103FA */
  0x0020,   /* U+103FB */
  0x0020,   /* U+103FC */
  0x0020,   /* U+103FD */
  0x0020,   /* U+103FE */
  0x0020,   /* U+103FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10300 */
  0x0002,   /* U+10301 */
  0x0002,   /* U+10302 */
  0x0002,   /* U+10303 */
  0x0002,   /* U+10304 */
  0x0002,   /* U+10305 */
  0x0002,   /* U+10306 */
  0x0002,   /* U+10307 */
  0x0002,   /* U+10308 */
  0x0002,   /* U+10309 */
  0x0002,   /* U+1030A */
  0x0002,   /* U+1030B */
  0x0002,   /* U+1030C */
  0x0002,   /* U+1030D */
  0x0002,   /* U+1030E */
  0x0002,   /* U+1030F */
  0x0002,   /* U+10310 */
  0x0002,   /* U+10311 */
  0x0002,   /* U+10312 */
  0x0002,   /* U+10313 */
  0x0002,   /* U+10314 */
  0x0002,   /* U+10315 */
  0x0002,   /* U+10316 */
  0x0002,   /* U+10317 */
  0x0002,   /* U+10318 */
  0x0002,   /* U+10319 */
  0x0002,   /* U+1031A */
  0x0002,   /* U+1031B */
  0x0002,   /* U+1031C */
  0x0002,   /* U+1031D */
  0x0002,   /* U+1031E */
  0x0002,   /* U+1031F */
  0x0002,   /* U+10320 */
  0x0002,   /* U+10321 */
  0x0002,   /* U+10322 */
  0x0002,   /* U+10323 */
  0x0002,   /* U+10324 */
  0x0002,   /* U+10325 */
  0x0002,   /* U+10326 */
  0x0002,   /* U+10327 */
  0x0002,   /* U+10328 */
  0x0002,   /* U+10329 */
  0x0002,   /* U+1032A */
  0x0002,   /* U+1032B */
  0x0002,   /* U+1032C */
  0x0002,   /* U+1032D */
  0x0002,   /* U+1032E */
  0x0002,   /* U+1032F */
  0x0002,   /* U+10330 */
  0x0002,   /* U+10331 */
  0x0002,   /* U+10332 */
  0x0002,   /* U+10333 */
  0x0002,   /* U+10334 */
  0x0002,   /* U+10335 */
  0x0002,   /* U+10336 */
  0x0002,   /* U+10337 */
  0x0002,   /* U+10338 */
  0x0002,   /* U+10339 */
  0x0002,   /* U+1033A */
  0x0002,   /* U+1033B */
  0x0002,   /* U+1033C */
  0x0002,   /* U+1033D */
  0x0002,   /* U+1033E */
  0x0002,   /* U+1033F */
  0x0002,   /* U+10340 */
  0x0002,   /* U+10341 */
  0x0002,   /* U+10342 */
  0x0002,   /* U+10343 */
  0x0002,   /* U+10344 */
  0x0002,   /* U+10345 */
  0x0002,   /* U+10346 */
  0x0002,   /* U+10347 */
  0x0002,   /* U+10348 */
  0x0002,   /* U+10349 */
  0x0002,   /* U+1034A */
  0x0002,   /* U+1034B */
  0x0002,   /* U+1034C */
  0x0002,   /* U+1034D */
  0x0002,   /* U+1034E */
  0x0002,   /* U+1034F */
  0x0002,   /* U+10350 */
  0x0002,   /* U+10351 */
  0x0002,   /* U+10352 */
  0x0002,   /* U+10353 */
  0x0002,   /* U+10354 */
  0x0002,   /* U+10355 */
  0x0002,   /* U+10356 */
  0x0002,   /* U+10357 */
  0x0002,   /* U+10358 */
  0x0002,   /* U+10359 */
  0x0002,   /* U+1035A */
  0x0002,   /* U+1035B */
  0x0002,   /* U+1035C */
  0x0002,   /* U+1035D */
  0x0002,   /* U+1035E */
  0x0002,   /* U+1035F */
  0x0002,   /* U+10360 */
  0x0002,   /* U+10361 */
  0x0002,   /* U+10362 */
  0x0002,   /* U+10363 */
  0x0002,   /* U+10364 */
  0x0002,   /* U+10365 */
  0x0002,   /* U+10366 */
  0x0002,   /* U+10367 */
  0x0002,   /* U+10368 */
  0x0002,   /* U+10369 */
  0x0002,   /* U+1036A */
  0x0002,   /* U+1036B */
  0x0002,   /* U+1036C */
  0x0002,   /* U+1036D */
  0x0002,   /* U+1036E */
  0x0002,   /* U+1036F */
  0x0002,   /* U+10370 */
  0x0002,   /* U+10371 */
  0x0002,   /* U+10372 */
  0x0002,   /* U+10373 */
  0x0002,   /* U+10374 */
  0x0002,   /* U+10375 */
  0x0004,   /* U+10376 */
  0x0004,   /* U+10377 */
  0x0004,   /* U+10378 */
  0x0004,   /* U+10379 */
  0x0004,   /* U+1037A */
  0x0002,   /* U+1037B */
  0x0002,   /* U+1037C */
  0x0002,   /* U+1037D */
  0x0002,   /* U+1037E */
  0x0002,   /* U+1037F */
  0x0002,   /* U+10380 */
  0x0002,   /* U+10381 */
  0x0002,   /* U+10382 */
  0x0002,   /* U+10383 */
  0x0002,   /* U+10384 */
  0x0002,   /* U+10385 */
  0x0002,   /* U+10386 */
  0x0002,   /* U+10387 */
  0x0002,   /* U+10388 */
  0x0002,   /* U+10389 */
  0x0002,   /* U+1038A */
  0x0002,   /* U+1038B */
  0x0002,   /* U+1038C */
  0x0002,   /* U+1038D */
  0x0002,   /* U+1038E */
  0x0002,   /* U+1038F */
  0x0002,   /* U+10390 */
  0x0002,   /* U+10391 */
  0x0002,   /* U+10392 */
  0x0002,   /* U+10393 */
  0x0002,   /* U+10394 */
  0x0002,   /* U+10395 */
  0x0002,   /* U+10396 */
  0x0002,   /* U+10397 */
  0x0002,   /* U+10398 */
  0x0002,   /* U+10399 */
  0x0002,   /* U+1039A */
  0x0002,   /* U+1039B */
  0x0002,   /* U+1039C */
  0x0002,   /* U+1039D */
  0x0002,   /* U+1039E */
  0x0002,   /* U+1039F */
  0x0002,   /* U+103A0 */
  0x0002,   /* U+103A1 */
  0x0002,   /* U+103A2 */
  0x0002,   /* U+103A3 */
  0x0002,   /* U+103A4 */
  0x0002,   /* U+103A5 */
  0x0002,   /* U+103A6 */
  0x0002,   /* U+103A7 */
  0x0002,   /* U+103A8 */
  0x0002,   /* U+103A9 */
  0x0002,   /* U+103AA */
  0x0002,   /* U+103AB */
  0x0002,   /* U+103AC */
  0x0002,   /* U+103AD */
  0x0002,   /* U+103AE */
  0x0002,   /* U+103AF */
  0x0002,   /* U+103B0 */
  0x0002,   /* U+103B1 */
  0x0002,   /* U+103B2 */
  0x0002,   /* U+103B3 */
  0x0002,   /* U+103B4 */
  0x0002,   /* U+103B5 */
  0x0002,   /* U+103B6 */
  0x0002,   /* U+103B7 */
  0x0002,   /* U+103B8 */
  0x0002,   /* U+103B9 */
  0x0002,   /* U+103BA */
  0x0002,   /* U+103BB */
  0x0002,   /* U+103BC */
  0x0002,   /* U+103BD */
  0x0002,   /* U+103BE */
  0x0002,   /* U+103BF */
  0x0002,   /* U+103C0 */
  0x0002,   /* U+103C1 */
  0x0002,   /* U+103C2 */
  0x0002,   /* U+103C3 */
  0x0002,   /* U+103C4 */
  0x0002,   /* U+103C5 */
  0x0002,   /* U+103C6 */
  0x0002,   /* U+103C7 */
  0x0002,   /* U+103C8 */
  0x0002,   /* U+103C9 */
  0x0002,   /* U+103CA */
  0x0002,   /* U+103CB */
  0x0002,   /* U+103CC */
  0x0002,   /* U+103CD */
  0x0002,   /* U+103CE */
  0x0002,   /* U+103CF */
  0x0002,   /* U+103D0 */
  0x0002,   /* U+103D1 */
  0x0002,   /* U+103D2 */
  0x0002,   /* U+103D3 */
  0x0002,   /* U+103D4 */
  0x0002,   /* U+103D5 */
  0x0002,   /* U+103D6 */
  0x0002,   /* U+103D7 */
  0x0002,   /* U+103D8 */
  0x0002,   /* U+103D9 */
  0x0002,   /* U+103DA */
  0x0002,   /* U+103DB */
  0x0002,   /* U+103DC */
  0x0002,   /* U+103DD */
  0x0002,   /* U+103DE */
  0x0002,   /* U+103DF */
  0x0002,   /* U+103E0 */
  0x0002,   /* U+103E1 */
  0x0002,   /* U+103E2 */
  0x0002,   /* U+103E3 */
  0x0002,   /* U+103E4 */
  0x0002,   /* U+103E5 */
  0x0002,   /* U+103E6 */
  0x0002,   /* U+103E7 */
  0x0002,   /* U+103E8 */
  0x0002,   /* U+103E9 */
  0x0002,   /* U+103EA */
  0x0002,   /* U+103EB */
  0x0002,   /* U+103EC */
  0x0002,   /* U+103ED */
  0x0002,   /* U+103EE */
  0x0002,   /* U+103EF */
  0x0002,   /* U+103F0 */
  0x0002,   /* U+103F1 */
  0x0002,   /* U+103F2 */
  0x0002,   /* U+103F3 */
  0x0002,   /* U+103F4 */
  0x0002,   /* U+103F5 */
  0x0002,   /* U+103F6 */
  0x0002,   /* U+103F7 */
  0x0002,   /* U+103F8 */
  0x0002,   /* U+103F9 */
  0x0002,   /* U+103FA */
  0x0002,   /* U+103FB */
  0x0002,   /* U+103FC */
  0x0002,   /* U+103FD */
  0x0002,   /* U+103FE */
  0x0002,   /* U+103FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10300 */
  0x0000,   /* U+10301 */
  0x0000,   /* U+10302 */
  0x0000,   /* U+10303 */
  0x0000,   /* U+10304 */
  0x0000,   /* U+10305 */
  0x0000,   /* U+10306 */
  0x0000,   /* U+10307 */
  0x0000,   /* U+10308 */
  0x0000,   /* U+10309 */
  0x0000,   /* U+1030A */
  0x0000,   /* U+1030B */
  0x0000,   /* U+1030C */
  0x0000,   /* U+1030D */
  0x0000,   /* U+1030E */
  0x0000,   /* U+1030F */
  0x0000,   /* U+10310 */
  0x0000,   /* U+10311 */
  0x0000,   /* U+10312 */
  0x0000,   /* U+10313 */
  0x0000,   /* U+10314 */
  0x0000,   /* U+10315 */
  0x0000,   /* U+10316 */
  0x0000,   /* U+10317 */
  0x0000,   /* U+10318 */
  0x0000,   /* U+10319 */
  0x0000,   /* U+1031A */
  0x0000,   /* U+1031B */
  0x0000,   /* U+1031C */
  0x0000,   /* U+1031D */
  0x0000,   /* U+1031E */
  0x0000,   /* U+1031F */
  0x0000,   /* U+10320 */
  0x0000,   /* U+10321 */
  0x0000,   /* U+10322 */
  0x0000,   /* U+10323 */
  0x8324,   /* U+10324 */
  0x8325,   /* U+10325 */
  0x8326,   /* U+10326 */
  0x8327,   /* U+10327 */
  0x8328,   /* U+10328 */
  0x8329,   /* U+10329 */
  0x832A,   /* U+1032A */
  0x832B,   /* U+1032B */
  0x832C,   /* U+1032C */
  0x832D,   /* U+1032D */
  0x832E,   /* U+1032E */
  0x832F,   /* U+1032F */
  0x0000,   /* U+10330 */
  0x0000,   /* U+10331 */
  0x0000,   /* U+10332 */
  0x0000,   /* U+10333 */
  0x0000,   /* U+10334 */
  0x0000,   /* U+10335 */
  0x0000,   /* U+10336 */
  0x0000,   /* U+10337 */
  0x0000,   /* U+10338 */
  0x0000,   /* U+10339 */
  0x0000,   /* U+1033A */
  0x0000,   /* U+1033B */
  0x0000,   /* U+1033C */
  0x0000,   /* U+1033D */
  0x0000,   /* U+1033E */
  0x0000,   /* U+1033F */
  0x0000,   /* U+10340 */
  0x0000,   /* U+10341 */
  0x0000,   /* U+10342 */
  0x0000,   /* U+10343 */
  0x0000,   /* U+10344 */
  0x0000,   /* U+10345 */
  0x0000,   /* U+10346 */
  0x0000,   /* U+10347 */
  0x0000,   /* U+10348 */
  0x0000,   /* U+10349 */
  0x0000,   /* U+1034A */
  0x834B,   /* U+1034B */
  0x834C,   /* U+1034C */
  0x834D,   /* U+1034D */
  0x834E,   /* U+1034E */
  0x834F,   /* U+1034F */
  0x0000,   /* U+10350 */
  0x0000,   /* U+10351 */
  0x0000,   /* U+10352 */
  0x0000,   /* U+10353 */
  0x0000,   /* U+10354 */
  0x0000,   /* U+10355 */
  0x0000,   /* U+10356 */
  0x0000,   /* U+10357 */
  0x0000,   /* U+10358 */
  0x0000,   /* U+10359 */
  0x0000,   /* U+1035A */
  0x0000,   /* U+1035B */
  0x0000,   /* U+1035C */
  0x0000,   /* U+1035D */
  0x0000,   /* U+1035E */
  0x0000,   /* U+1035F */
  0x0000,   /* U+10360 */
  0x0000,   /* U+10361 */
  0x0000,   /* U+10362 */
  0x0000,   /* U+10363 */
  0x0000,   /* U+10364 */
  0x0000,   /* U+10365 */
  0x0000,   /* U+10366 */
  0x0000,   /* U+10367 */
  0x0000,   /* U+10368 */
  0x0000,   /* U+10369 */
  0x0000,   /* U+1036A */
  0x0000,   /* U+1036B */
  0x0000,   /* U+1036C */
  0x0000,   /* U+1036D */
  0x0000,   /* U+1036E */
  0x0000,   /* U+1036F */
  0x0000,   /* U+10370 */
  0x0000,   /* U+10371 */
  0x0000,   /* U+10372 */
  0x0000,   /* U+10373 */
  0x0000,   /* U+10374 */
  0x0000,   /* U+10375 */
  0x0000,   /* U+10376 */
  0x0000,   /* U+10377 */
  0x0000,   /* U+10378 */
  0x0000,   /* U+10379 */
  0x0000,   /* U+1037A */
  0x837B,   /* U+1037B */
  0x837C,   /* U+1037C */
  0x837D,   /* U+1037D */
  0x837E,   /* U+1037E */
  0x837F,   /* U+1037F */
  0x0000,   /* U+10380 */
  0x0000,   /* U+10381 */
  0x0000,   /* U+10382 */
  0x0000,   /* U+10383 */
  0x0000,   /* U+10384 */
  0x0000,   /* U+10385 */
  0x0000,   /* U+10386 */
  0x0000,   /* U+10387 */
  0x0000,   /* U+10388 */
  0x0000,   /* U+10389 */
  0x0000,   /* U+1038A */
  0x0000,   /* U+1038B */
  0x0000,   /* U+1038C */
  0x0000,   /* U+1038D */
  0x0000,   /* U+1038E */
  0x0000,   /* U+1038F */
  0x0000,   /* U+10390 */
  0x0000,   /* U+10391 */
  0x0000,   /* U+10392 */
  0x0000,   /* U+10393 */
  0x0000,   /* U+10394 */
  0x0000,   /* U+10395 */
  0x0000,   /* U+10396 */
  0x0000,   /* U+10397 */
  0x0000,   /* U+10398 */
  0x0000,   /* U+10399 */
  0x0000,   /* U+1039A */
  0x0000,   /* U+1039B */
  0x0000,   /* U+1039C */
  0x0000,   /* U+1039D */
  0x839E,   /* U+1039E */
  0x0000,   /* U+1039F */
  0x0000,   /* U+103A0 */
  0x0000,   /* U+103A1 */
  0x0000,   /* U+103A2 */
  0x0000,   /* U+103A3 */
  0x0000,   /* U+103A4 */
  0x0000,   /* U+103A5 */
  0x0000,   /* U+103A6 */
  0x0000,   /* U+103A7 */
  0x0000,   /* U+103A8 */
  0x0000,   /* U+103A9 */
  0x0000,   /* U+103AA */
  0x0000,   /* U+103AB */
  0x0000,   /* U+103AC */
  0x0000,   /* U+103AD */
  0x0000,   /* U+103AE */
  0x0000,   /* U+103AF */
  0x0000,   /* U+103B0 */
  0x0000,   /* U+103B1 */
  0x0000,   /* U+103B2 */
  0x0000,   /* U+103B3 */
  0x0000,   /* U+103B4 */
  0x0000,   /* U+103B5 */
  0x0000,   /* U+103B6 */
  0x0000,   /* U+103B7 */
  0x0000,   /* U+103B8 */
  0x0000,   /* U+103B9 */
  0x0000,   /* U+103BA */
  0x0000,   /* U+103BB */
  0x0000,   /* U+103BC */
  0x0000,   /* U+103BD */
  0x0000,   /* U+103BE */
  0x0000,   /* U+103BF */
  0x0000,   /* U+103C0 */
  0x0000,   /* U+103C1 */
  0x0000,   /* U+103C2 */
  0x0000,   /* U+103C3 */
  0x83C4,   /* U+103C4 */
  0x83C5,   /* U+103C5 */
  0x83C6,   /* U+103C6 */
  0x83C7,   /* U+103C7 */
  0x0000,   /* U+103C8 */
  0x0000,   /* U+103C9 */
  0x0000,   /* U+103CA */
  0x0000,   /* U+103CB */
  0x0000,   /* U+103CC */
  0x0000,   /* U+103CD */
  0x0000,   /* U+103CE */
  0x0000,   /* U+103CF */
  0x0000,   /* U+103D0 */
  0x0000,   /* U+103D1 */
  0x0000,   /* U+103D2 */
  0x0000,   /* U+103D3 */
  0x0000,   /* U+103D4 */
  0x0000,   /* U+103D5 */
  0x83D6,   /* U+103D6 */
  0x83D7,   /* U+103D7 */
  0x83D8,   /* U+103D8 */
  0x83D9,   /* U+103D9 */
  0x83DA,   /* U+103DA */
  0x83DB,   /* U+103DB */
  0x83DC,   /* U+103DC */
  0x83DD,   /* U+103DD */
  0x83DE,   /* U+103DE */
  0x83DF,   /* U+103DF */
  0x83E0,   /* U+103E0 */
  0x83E1,   /* U+103E1 */
  0x83E2,   /* U+103E2 */
  0x83E3,   /* U+103E3 */
  0x83E4,   /* U+103E4 */
  0x83E5,   /* U+103E5 */
  0x83E6,   /* U+103E6 */
  0x83E7,   /* U+103E7 */
  0x83E8,   /* U+103E8 */
  0x83E9,   /* U+103E9 */
  0x83EA,   /* U+103EA */
  0x83EB,   /* U+103EB */
  0x83EC,   /* U+103EC */
  0x83ED,   /* U+103ED */
  0x83EE,   /* U+103EE */
  0x83EF,   /* U+103EF */
  0x83F0,   /* U+103F0 */
  0x83F1,   /* U+103F1 */
  0x83F2,   /* U+103F2 */
  0x83F3,   /* U+103F3 */
  0x83F4,   /* U+103F4 */
  0x83F5,   /* U+103F5 */
  0x83F6,   /* U+103F6 */
  0x83F7,   /* U+103F7 */
  0x83F8,   /* U+103F8 */
  0x83F9,   /* U+103F9 */
  0x83FA,   /* U+103FA */
  0x83FB,   /* U+103FB */
  0x83FC,   /* U+103FC */
  0x83FD,   /* U+103FD */
  0x83FE,   /* U+103FE */
  0x83FF,   /* U+103FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10300 */
  0x0000,   /* U+10301 */
  0x0000,   /* U+10302 */
  0x0000,   /* U+10303 */
  0x0000,   /* U+10304 */
  0x0000,   /* U+10305 */
  0x0000,   /* U+10306 */
  0x0000,   /* U+10307 */
  0x0000,   /* U+10308 */
  0x0000,   /* U+10309 */
  0x0000,   /* U+1030A */
  0x0000,   /* U+1030B */
  0x0000,   /* U+1030C */
  0x0000,   /* U+1030D */
  0x0000,   /* U+1030E */
  0x0000,   /* U+1030F */
  0x0000,   /* U+10310 */
  0x0000,   /* U+10311 */
  0x0000,   /* U+10312 */
  0x0000,   /* U+10313 */
  0x0000,   /* U+10314 */
  0x0000,   /* U+10315 */
  0x0000,   /* U+10316 */
  0x0000,   /* U+10317 */
  0x0000,   /* U+10318 */
  0x0000,   /* U+10319 */
  0x0000,   /* U+1031A */
  0x0000,   /* U+1031B */
  0x0000,   /* U+1031C */
  0x0000,   /* U+1031D */
  0x0000,   /* U+1031E */
  0x0000,   /* U+1031F */
  0x0000,   /* U+10320 */
  0x0000,   /* U+10321 */
  0x0000,   /* U+10322 */
  0x0000,   /* U+10323 */
  0x0000,   /* U+10324 */
  0x0000,   /* U+10325 */
  0x0000,   /* U+10326 */
  0x0000,   /* U+10327 */
  0x0000,   /* U+10328 */
  0x0000,   /* U+10329 */
  0x0000,   /* U+1032A */
  0x0000,   /* U+1032B */
  0x0000,   /* U+1032C */
  0x0000,   /* U+1032D */
  0x0000,   /* U+1032E */
  0x0000,   /* U+1032F */
  0x0000,   /* U+10330 */
  0x0000,   /* U+10331 */
  0x0000,   /* U+10332 */
  0x0000,   /* U+10333 */
  0x0000,   /* U+10334 */
  0x0000,   /* U+10335 */
  0x0000,   /* U+10336 */
  0x0000,   /* U+10337 */
  0x0000,   /* U+10338 */
  0x0000,   /* U+10339 */
  0x0000,   /* U+1033A */
  0x0000,   /* U+1033B */
  0x0000,   /* U+1033C */
  0x0000,   /* U+1033D */
  0x0000,   /* U+1033E */
  0x0000,   /* U+1033F */
  0x0000,   /* U+10340 */
  0x0000,   /* U+10341 */
  0x0000,   /* U+10342 */
  0x0000,   /* U+10343 */
  0x0000,   /* U+10344 */
  0x0000,   /* U+10345 */
  0x0000,   /* U+10346 */
  0x0000,   /* U+10347 */
  0x0000,   /* U+10348 */
  0x0000,   /* U+10349 */
  0x0000,   /* U+1034A */
  0x0000,   /* U+1034B */
  0x0000,   /* U+1034C */
  0x0000,   /* U+1034D */
  0x0000,   /* U+1034E */
  0x0000,   /* U+1034F */
  0x0000,   /* U+10350 */
  0x0000,   /* U+10351 */
  0x0000,   /* U+10352 */
  0x0000,   /* U+10353 */
  0x0000,   /* U+10354 */
  0x0000,   /* U+10355 */
  0x0000,   /* U+10356 */
  0x0000,   /* U+10357 */
  0x0000,   /* U+10358 */
  0x0000,   /* U+10359 */
  0x0000,   /* U+1035A */
  0x0000,   /* U+1035B */
  0x0000,   /* U+1035C */
  0x0000,   /* U+1035D */
  0x0000,   /* U+1035E */
  0x0000,   /* U+1035F */
  0x0000,   /* U+10360 */
  0x0000,   /* U+10361 */
  0x0000,   /* U+10362 */
  0x0000,   /* U+10363 */
  0x0000,   /* U+10364 */
  0x0000,   /* U+10365 */
  0x0000,   /* U+10366 */
  0x0000,   /* U+10367 */
  0x0000,   /* U+10368 */
  0x0000,   /* U+10369 */
  0x0000,   /* U+1036A */
  0x0000,   /* U+1036B */
  0x0000,   /* U+1036C */
  0x0000,   /* U+1036D */
  0x0000,   /* U+1036E */
  0x0000,   /* U+1036F */
  0x0000,   /* U+10370 */
  0x0000,   /* U+10371 */
  0x0000,   /* U+10372 */
  0x0000,   /* U+10373 */
  0x0000,   /* U+10374 */
  0x0000,   /* U+10375 */
  0x0000,   /* U+10376 */
  0x0000,   /* U+10377 */
  0x0000,   /* U+10378 */
  0x0000,   /* U+10379 */
  0x0000,   /* U+1037A */
  0x0000,   /* U+1037B */
  0x0000,   /* U+1037C */
  0x0000,   /* U+1037D */
  0x0000,   /* U+1037E */
  0x0000,   /* U+1037F */
  0x0000,   /* U+10380 */
  0x0000,   /* U+10381 */
  0x0000,   /* U+10382 */
  0x0000,   /* U+10383 */
  0x0000,   /* U+10384 */
  0x0000,   /* U+10385 */
  0x0000,   /* U+10386 */
  0x0000,   /* U+10387 */
  0x0000,   /* U+10388 */
  0x0000,   /* U+10389 */
  0x0000,   /* U+1038A */
  0x0000,   /* U+1038B */
  0x0000,   /* U+1038C */
  0x0000,   /* U+1038D */
  0x0000,   /* U+1038E */
  0x0000,   /* U+1038F */
  0x0000,   /* U+10390 */
  0x0000,   /* U+10391 */
  0x0000,   /* U+10392 */
  0x0000,   /* U+10393 */
  0x0000,   /* U+10394 */
  0x0000,   /* U+10395 */
  0x0000,   /* U+10396 */
  0x0000,   /* U+10397 */
  0x0000,   /* U+10398 */
  0x0000,   /* U+10399 */
  0x0000,   /* U+1039A */
  0x0000,   /* U+1039B */
  0x0000,   /* U+1039C */
  0x0000,   /* U+1039D */
  0x0000,   /* U+1039E */
  0x0000,   /* U+1039F */
  0x0000,   /* U+103A0 */
  0x0000,   /* U+103A1 */
  0x0000,   /* U+103A2 */
  0x0000,   /* U+103A3 */
  0x0000,   /* U+103A4 */
  0x0000,   /* U+103A5 */
  0x0000,   /* U+103A6 */
  0x0000,   /* U+103A7 */
  0x0000,   /* U+103A8 */
  0x0000,   /* U+103A9 */
  0x0000,   /* U+103AA */
  0x0000,   /* U+103AB */
  0x0000,   /* U+103AC */
  0x0000,   /* U+103AD */
  0x0000,   /* U+103AE */
  0x0000,   /* U+103AF */
  0x0000,   /* U+103B0 */
  0x0000,   /* U+103B1 */
  0x0000,   /* U+103B2 */
  0x0000,   /* U+103B3 */
  0x0000,   /* U+103B4 */
  0x0000,   /* U+103B5 */
  0x0000,   /* U+103B6 */
  0x0000,   /* U+103B7 */
  0x0000,   /* U+103B8 */
  0x0000,   /* U+103B9 */
  0x0000,   /* U+103BA */
  0x0000,   /* U+103BB */
  0x0000,   /* U+103BC */
  0x0000,   /* U+103BD */
  0x0000,   /* U+103BE */
  0x0000,   /* U+103BF */
  0x0000,   /* U+103C0 */
  0x0000,   /* U+103C1 */
  0x0000,   /* U+103C2 */
  0x0000,   /* U+103C3 */
  0x0000,   /* U+103C4 */
  0x0000,   /* U+103C5 */
  0x0000,   /* U+103C6 */
  0x0000,   /* U+103C7 */
  0x0000,   /* U+103C8 */
  0x0000,   /* U+103C9 */
  0x0000,   /* U+103CA */
  0x0000,   /* U+103CB */
  0x0000,   /* U+103CC */
  0x0000,   /* U+103CD */
  0x0000,   /* U+103CE */
  0x0000,   /* U+103CF */
  0x0000,   /* U+103D0 */
  0x0000,   /* U+103D1 */
  0x0000,   /* U+103D2 */
  0x0000,   /* U+103D3 */
  0x0000,   /* U+103D4 */
  0x0000,   /* U+103D5 */
  0x0000,   /* U+103D6 */
  0x0000,   /* U+103D7 */
  0x0000,   /* U+103D8 */
  0x0000,   /* U+103D9 */
  0x0000,   /* U+103DA */
  0x0000,   /* U+103DB */
  0x0000,   /* U+103DC */
  0x0000,   /* U+103DD */
  0x0000,   /* U+103DE */
  0x0000,   /* U+103DF */
  0x0000,   /* U+103E0 */
  0x0000,   /* U+103E1 */
  0x0000,   /* U+103E2 */
  0x0000,   /* U+103E3 */
  0x0000,   /* U+103E4 */
  0x0000,   /* U+103E5 */
  0x0000,   /* U+103E6 */
  0x0000,   /* U+103E7 */
  0x0000,   /* U+103E8 */
  0x0000,   /* U+103E9 */
  0x0000,   /* U+103EA */
  0x0000,   /* U+103EB */
  0x0000,   /* U+103EC */
  0x0000,   /* U+103ED */
  0x0000,   /* U+103EE */
  0x0000,   /* U+103EF */
  0x0000,   /* U+103F0 */
  0x0000,   /* U+103F1 */
  0x0000,   /* U+103F2 */
  0x0000,   /* U+103F3 */
  0x0000,   /* U+103F4 */
  0x0000,   /* U+103F5 */
  0x0000,   /* U+103F6 */
  0x0000,   /* U+103F7 */
  0x0000,   /* U+103F8 */
  0x0000,   /* U+103F9 */
  0x0000,   /* U+103FA */
  0x0000,   /* U+103FB */
  0x0000,   /* U+103FC */
  0x0000,   /* U+103FD */
  0x0000,   /* U+103FE */
  0x0000,   /* U+103FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10300 */
  0x0000,   /* U+10301 */
  0x0000,   /* U+10302 */
  0x0000,   /* U+10303 */
  0x0000,   /* U+10304 */
  0x0000,   /* U+10305 */
  0x0000,   /* U+10306 */
  0x0000,   /* U+10307 */
  0x0000,   /* U+10308 */
  0x0000,   /* U+10309 */
  0x0000,   /* U+1030A */
  0x0000,   /* U+1030B */
  0x0000,   /* U+1030C */
  0x0000,   /* U+1030D */
  0x0000,   /* U+1030E */
  0x0000,   /* U+1030F */
  0x0000,   /* U+10310 */
  0x0000,   /* U+10311 */
  0x0000,   /* U+10312 */
  0x0000,   /* U+10313 */
  0x0000,   /* U+10314 */
  0x0000,   /* U+10315 */
  0x0000,   /* U+10316 */
  0x0000,   /* U+10317 */
  0x0000,   /* U+10318 */
  0x0000,   /* U+10319 */
  0x0000,   /* U+1031A */
  0x0000,   /* U+1031B */
  0x0000,   /* U+1031C */
  0x0000,   /* U+1031D */
  0x0000,   /* U+1031E */
  0x0000,   /* U+1031F */
  0x0000,   /* U+10320 */
  0x0000,   /* U+10321 */
  0x0000,   /* U+10322 */
  0x0000,   /* U+10323 */
  0x0000,   /* U+10324 */
  0x0000,   /* U+10325 */
  0x0000,   /* U+10326 */
  0x0000,   /* U+10327 */
  0x0000,   /* U+10328 */
  0x0000,   /* U+10329 */
  0x0000,   /* U+1032A */
  0x0000,   /* U+1032B */
  0x0000,   /* U+1032C */
  0x0000,   /* U+1032D */
  0x0000,   /* U+1032E */
  0x0000,   /* U+1032F */
  0x0000,   /* U+10330 */
  0x0000,   /* U+10331 */
  0x0000,   /* U+10332 */
  0x0000,   /* U+10333 */
  0x0000,   /* U+10334 */
  0x0000,   /* U+10335 */
  0x0000,   /* U+10336 */
  0x0000,   /* U+10337 */
  0x0000,   /* U+10338 */
  0x0000,   /* U+10339 */
  0x0000,   /* U+1033A */
  0x0000,   /* U+1033B */
  0x0000,   /* U+1033C */
  0x0000,   /* U+1033D */
  0x0000,   /* U+1033E */
  0x0000,   /* U+1033F */
  0x0000,   /* U+10340 */
  0x0000,   /* U+10341 */
  0x0000,   /* U+10342 */
  0x0000,   /* U+10343 */
  0x0000,   /* U+10344 */
  0x0000,   /* U+10345 */
  0x0000,   /* U+10346 */
  0x0000,   /* U+10347 */
  0x0000,   /* U+10348 */
  0x0000,   /* U+10349 */
  0x0000,   /* U+1034A */
  0x0000,   /* U+1034B */
  0x0000,   /* U+1034C */
  0x0000,   /* U+1034D */
  0x0000,   /* U+1034E */
  0x0000,   /* U+1034F */
  0x0000,   /* U+10350 */
  0x0000,   /* U+10351 */
  0x0000,   /* U+10352 */
  0x0000,   /* U+10353 */
  0x0000,   /* U+10354 */
  0x0000,   /* U+10355 */
  0x0000,   /* U+10356 */
  0x0000,   /* U+10357 */
  0x0000,   /* U+10358 */
  0x0000,   /* U+10359 */
  0x0000,   /* U+1035A */
  0x0000,   /* U+1035B */
  0x0000,   /* U+1035C */
  0x0000,   /* U+1035D */
  0x0000,   /* U+1035E */
  0x0000,   /* U+1035F */
  0x0000,   /* U+10360 */
  0x0000,   /* U+10361 */
  0x0000,   /* U+10362 */
  0x0000,   /* U+10363 */
  0x0000,   /* U+10364 */
  0x0000,   /* U+10365 */
  0x0000,   /* U+10366 */
  0x0000,   /* U+10367 */
  0x0000,   /* U+10368 */
  0x0000,   /* U+10369 */
  0x0000,   /* U+1036A */
  0x0000,   /* U+1036B */
  0x0000,   /* U+1036C */
  0x0000,   /* U+1036D */
  0x0000,   /* U+1036E */
  0x0000,   /* U+1036F */
  0x0000,   /* U+10370 */
  0x0000,   /* U+10371 */
  0x0000,   /* U+10372 */
  0x0000,   /* U+10373 */
  0x0000,   /* U+10374 */
  0x0000,   /* U+10375 */
  0x0000,   /* U+10376 */
  0x0000,   /* U+10377 */
  0x0000,   /* U+10378 */
  0x0000,   /* U+10379 */
  0x0000,   /* U+1037A */
  0x0000,   /* U+1037B */
  0x0000,   /* U+1037C */
  0x0000,   /* U+1037D */
  0x0000,   /* U+1037E */
  0x0000,   /* U+1037F */
  0x0000,   /* U+10380 */
  0x0000,   /* U+10381 */
  0x0000,   /* U+10382 */
  0x0000,   /* U+10383 */
  0x0000,   /* U+10384 */
  0x0000,   /* U+10385 */
  0x0000,   /* U+10386 */
  0x0000,   /* U+10387 */
  0x0000,   /* U+10388 */
  0x0000,   /* U+10389 */
  0x0000,   /* U+1038A */
  0x0000,   /* U+1038B */
  0x0000,   /* U+1038C */
  0x0000,   /* U+1038D */
  0x0000,   /* U+1038E */
  0x0000,   /* U+1038F */
  0x0000,   /* U+10390 */
  0x0000,   /* U+10391 */
  0x0000,   /* U+10392 */
  0x0000,   /* U+10393 */
  0x0000,   /* U+10394 */
  0x0000,   /* U+10395 */
  0x0000,   /* U+10396 */
  0x0000,   /* U+10397 */
  0x0000,   /* U+10398 */
  0x0000,   /* U+10399 */
  0x0000,   /* U+1039A */
  0x0000,   /* U+1039B */
  0x0000,   /* U+1039C */
  0x0000,   /* U+1039D */
  0x0000,   /* U+1039E */
  0x0000,   /* U+1039F */
  0x0000,   /* U+103A0 */
  0x0000,   /* U+103A1 */
  0x0000,   /* U+103A2 */
  0x0000,   /* U+103A3 */
  0x0000,   /* U+103A4 */
  0x0000,   /* U+103A5 */
  0x0000,   /* U+103A6 */
  0x0000,   /* U+103A7 */
  0x0000,   /* U+103A8 */
  0x0000,   /* U+103A9 */
  0x0000,   /* U+103AA */
  0x0000,   /* U+103AB */
  0x0000,   /* U+103AC */
  0x0000,   /* U+103AD */
  0x0000,   /* U+103AE */
  0x0000,   /* U+103AF */
  0x0000,   /* U+103B0 */
  0x0000,   /* U+103B1 */
  0x0000,   /* U+103B2 */
  0x0000,   /* U+103B3 */
  0x0000,   /* U+103B4 */
  0x0000,   /* U+103B5 */
  0x0000,   /* U+103B6 */
  0x0000,   /* U+103B7 */
  0x0000,   /* U+103B8 */
  0x0000,   /* U+103B9 */
  0x0000,   /* U+103BA */
  0x0000,   /* U+103BB */
  0x0000,   /* U+103BC */
  0x0000,   /* U+103BD */
  0x0000,   /* U+103BE */
  0x0000,   /* U+103BF */
  0x0000,   /* U+103C0 */
  0x0000,   /* U+103C1 */
  0x0000,   /* U+103C2 */
  0x0000,   /* U+103C3 */
  0x0000,   /* U+103C4 */
  0x0000,   /* U+103C5 */
  0x0000,   /* U+103C6 */
  0x0000,   /* U+103C7 */
  0x0000,   /* U+103C8 */
  0x0000,   /* U+103C9 */
  0x0000,   /* U+103CA */
  0x0000,   /* U+103CB */
  0x0000,   /* U+103CC */
  0x0000,   /* U+103CD */
  0x0000,   /* U+103CE */
  0x0000,   /* U+103CF */
  0x0000,   /* U+103D0 */
  0x0000,   /* U+103D1 */
  0x0000,   /* U+103D2 */
  0x0000,   /* U+103D3 */
  0x0000,   /* U+103D4 */
  0x0000,   /* U+103D5 */
  0x0000,   /* U+103D6 */
  0x0000,   /* U+103D7 */
  0x0000,   /* U+103D8 */
  0x0000,   /* U+103D9 */
  0x0000,   /* U+103DA */
  0x0000,   /* U+103DB */
  0x0000,   /* U+103DC */
  0x0000,   /* U+103DD */
  0x0000,   /* U+103DE */
  0x0000,   /* U+103DF */
  0x0000,   /* U+103E0 */
  0x0000,   /* U+103E1 */
  0x0000,   /* U+103E2 */
  0x0000,   /* U+103E3 */
  0x0000,   /* U+103E4 */
  0x0000,   /* U+103E5 */
  0x0000,   /* U+103E6 */
  0x0000,   /* U+103E7 */
  0x0000,   /* U+103E8 */
  0x0000,   /* U+103E9 */
  0x0000,   /* U+103EA */
  0x0000,   /* U+103EB */
  0x0000,   /* U+103EC */
  0x0000,   /* U+103ED */
  0x0000,   /* U+103EE */
  0x0000,   /* U+103EF */
  0x0000,   /* U+103F0 */
  0x0000,   /* U+103F1 */
  0x0000,   /* U+103F2 */
  0x0000,   /* U+103F3 */
  0x0000,   /* U+103F4 */
  0x0000,   /* U+103F5 */
  0x0000,   /* U+103F6 */
  0x0000,   /* U+103F7 */
  0x0000,   /* U+103F8 */
  0x0000,   /* U+103F9 */
  0x0000,   /* U+103FA */
  0x0000,   /* U+103FB */
  0x0000,   /* U+103FC */
  0x0000,   /* U+103FD */
  0x0000,   /* U+103FE */
  0x0000,   /* U+103FF */
};

uint16 uca900_p104[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4452,   /* U+10400 */
  0x4453,   /* U+10401 */
  0x4454,   /* U+10402 */
  0x4455,   /* U+10403 */
  0x4456,   /* U+10404 */
  0x4457,   /* U+10405 */
  0x4458,   /* U+10406 */
  0x4459,   /* U+10407 */
  0x445A,   /* U+10408 */
  0x445B,   /* U+10409 */
  0x445C,   /* U+1040A */
  0x445D,   /* U+1040B */
  0x445E,   /* U+1040C */
  0x445F,   /* U+1040D */
  0x4460,   /* U+1040E */
  0x4461,   /* U+1040F */
  0x4462,   /* U+10410 */
  0x4463,   /* U+10411 */
  0x4464,   /* U+10412 */
  0x4465,   /* U+10413 */
  0x4466,   /* U+10414 */
  0x4467,   /* U+10415 */
  0x4468,   /* U+10416 */
  0x4469,   /* U+10417 */
  0x446A,   /* U+10418 */
  0x446B,   /* U+10419 */
  0x446C,   /* U+1041A */
  0x446D,   /* U+1041B */
  0x446E,   /* U+1041C */
  0x446F,   /* U+1041D */
  0x4470,   /* U+1041E */
  0x4471,   /* U+1041F */
  0x4472,   /* U+10420 */
  0x4473,   /* U+10421 */
  0x4474,   /* U+10422 */
  0x4475,   /* U+10423 */
  0x4476,   /* U+10424 */
  0x4477,   /* U+10425 */
  0x4478,   /* U+10426 */
  0x4479,   /* U+10427 */
  0x4452,   /* U+10428 */
  0x4453,   /* U+10429 */
  0x4454,   /* U+1042A */
  0x4455,   /* U+1042B */
  0x4456,   /* U+1042C */
  0x4457,   /* U+1042D */
  0x4458,   /* U+1042E */
  0x4459,   /* U+1042F */
  0x445A,   /* U+10430 */
  0x445B,   /* U+10431 */
  0x445C,   /* U+10432 */
  0x445D,   /* U+10433 */
  0x445E,   /* U+10434 */
  0x445F,   /* U+10435 */
  0x4460,   /* U+10436 */
  0x4461,   /* U+10437 */
  0x4462,   /* U+10438 */
  0x4463,   /* U+10439 */
  0x4464,   /* U+1043A */
  0x4465,   /* U+1043B */
  0x4466,   /* U+1043C */
  0x4467,   /* U+1043D */
  0x4468,   /* U+1043E */
  0x4469,   /* U+1043F */
  0x446A,   /* U+10440 */
  0x446B,   /* U+10441 */
  0x446C,   /* U+10442 */
  0x446D,   /* U+10443 */
  0x446E,   /* U+10444 */
  0x446F,   /* U+10445 */
  0x4470,   /* U+10446 */
  0x4471,   /* U+10447 */
  0x4472,   /* U+10448 */
  0x4473,   /* U+10449 */
  0x4474,   /* U+1044A */
  0x4475,   /* U+1044B */
  0x4476,   /* U+1044C */
  0x4477,   /* U+1044D */
  0x4478,   /* U+1044E */
  0x4479,   /* U+1044F */
  0x447A,   /* U+10450 */
  0x447B,   /* U+10451 */
  0x447C,   /* U+10452 */
  0x447D,   /* U+10453 */
  0x447E,   /* U+10454 */
  0x447F,   /* U+10455 */
  0x4480,   /* U+10456 */
  0x4481,   /* U+10457 */
  0x4482,   /* U+10458 */
  0x4483,   /* U+10459 */
  0x4484,   /* U+1045A */
  0x4485,   /* U+1045B */
  0x4486,   /* U+1045C */
  0x4487,   /* U+1045D */
  0x4488,   /* U+1045E */
  0x4489,   /* U+1045F */
  0x448A,   /* U+10460 */
  0x448B,   /* U+10461 */
  0x448C,   /* U+10462 */
  0x448D,   /* U+10463 */
  0x448E,   /* U+10464 */
  0x448F,   /* U+10465 */
  0x4490,   /* U+10466 */
  0x4491,   /* U+10467 */
  0x4492,   /* U+10468 */
  0x4493,   /* U+10469 */
  0x4494,   /* U+1046A */
  0x4495,   /* U+1046B */
  0x4496,   /* U+1046C */
  0x4497,   /* U+1046D */
  0x4498,   /* U+1046E */
  0x4499,   /* U+1046F */
  0x449A,   /* U+10470 */
  0x449B,   /* U+10471 */
  0x449C,   /* U+10472 */
  0x449D,   /* U+10473 */
  0x449E,   /* U+10474 */
  0x449F,   /* U+10475 */
  0x44A0,   /* U+10476 */
  0x44A1,   /* U+10477 */
  0x44A2,   /* U+10478 */
  0x44A3,   /* U+10479 */
  0x44A4,   /* U+1047A */
  0x44A5,   /* U+1047B */
  0x44A6,   /* U+1047C */
  0x44A7,   /* U+1047D */
  0x44A8,   /* U+1047E */
  0x44A9,   /* U+1047F */
  0x4535,   /* U+10480 */
  0x4536,   /* U+10481 */
  0x4537,   /* U+10482 */
  0x4538,   /* U+10483 */
  0x4539,   /* U+10484 */
  0x453A,   /* U+10485 */
  0x453B,   /* U+10486 */
  0x453C,   /* U+10487 */
  0x453D,   /* U+10488 */
  0x453E,   /* U+10489 */
  0x453F,   /* U+1048A */
  0x4540,   /* U+1048B */
  0x4541,   /* U+1048C */
  0x4542,   /* U+1048D */
  0x4543,   /* U+1048E */
  0x4544,   /* U+1048F */
  0x4545,   /* U+10490 */
  0x4546,   /* U+10491 */
  0x4547,   /* U+10492 */
  0x4548,   /* U+10493 */
  0x4549,   /* U+10494 */
  0x454A,   /* U+10495 */
  0x454B,   /* U+10496 */
  0x454C,   /* U+10497 */
  0x454D,   /* U+10498 */
  0x454E,   /* U+10499 */
  0x454F,   /* U+1049A */
  0x4550,   /* U+1049B */
  0x4551,   /* U+1049C */
  0x4552,   /* U+1049D */
  0xFBC2,   /* U+1049E */
  0xFBC2,   /* U+1049F */
  0x1C3D,   /* U+104A0 */
  0x1C3E,   /* U+104A1 */
  0x1C3F,   /* U+104A2 */
  0x1C40,   /* U+104A3 */
  0x1C41,   /* U+104A4 */
  0x1C42,   /* U+104A5 */
  0x1C43,   /* U+104A6 */
  0x1C44,   /* U+104A7 */
  0x1C45,   /* U+104A8 */
  0x1C46,   /* U+104A9 */
  0xFBC2,   /* U+104AA */
  0xFBC2,   /* U+104AB */
  0xFBC2,   /* U+104AC */
  0xFBC2,   /* U+104AD */
  0xFBC2,   /* U+104AE */
  0xFBC2,   /* U+104AF */
  0x33D5,   /* U+104B0 */
  0x33D6,   /* U+104B1 */
  0x33D7,   /* U+104B2 */
  0x33D8,   /* U+104B3 */
  0x33D9,   /* U+104B4 */
  0x33DA,   /* U+104B5 */
  0x33DB,   /* U+104B6 */
  0x33DC,   /* U+104B7 */
  0x33DD,   /* U+104B8 */
  0x33DE,   /* U+104B9 */
  0x33DF,   /* U+104BA */
  0x33E0,   /* U+104BB */
  0x33E1,   /* U+104BC */
  0x33E2,   /* U+104BD */
  0x33E3,   /* U+104BE */
  0x33E4,   /* U+104BF */
  0x33E5,   /* U+104C0 */
  0x33E6,   /* U+104C1 */
  0x33E7,   /* U+104C2 */
  0x33E8,   /* U+104C3 */
  0x33E9,   /* U+104C4 */
  0x33EA,   /* U+104C5 */
  0x33EB,   /* U+104C6 */
  0x33EC,   /* U+104C7 */
  0x33ED,   /* U+104C8 */
  0x33EE,   /* U+104C9 */
  0x33EF,   /* U+104CA */
  0x33F0,   /* U+104CB */
  0x33F1,   /* U+104CC */
  0x33F2,   /* U+104CD */
  0x33F3,   /* U+104CE */
  0x33F4,   /* U+104CF */
  0x33F5,   /* U+104D0 */
  0x33F6,   /* U+104D1 */
  0x33F7,   /* U+104D2 */
  0x33F8,   /* U+104D3 */
  0xFBC2,   /* U+104D4 */
  0xFBC2,   /* U+104D5 */
  0xFBC2,   /* U+104D6 */
  0xFBC2,   /* U+104D7 */
  0x33D5,   /* U+104D8 */
  0x33D6,   /* U+104D9 */
  0x33D7,   /* U+104DA */
  0x33D8,   /* U+104DB */
  0x33D9,   /* U+104DC */
  0x33DA,   /* U+104DD */
  0x33DB,   /* U+104DE */
  0x33DC,   /* U+104DF */
  0x33DD,   /* U+104E0 */
  0x33DE,   /* U+104E1 */
  0x33DF,   /* U+104E2 */
  0x33E0,   /* U+104E3 */
  0x33E1,   /* U+104E4 */
  0x33E2,   /* U+104E5 */
  0x33E3,   /* U+104E6 */
  0x33E4,   /* U+104E7 */
  0x33E5,   /* U+104E8 */
  0x33E6,   /* U+104E9 */
  0x33E7,   /* U+104EA */
  0x33E8,   /* U+104EB */
  0x33E9,   /* U+104EC */
  0x33EA,   /* U+104ED */
  0x33EB,   /* U+104EE */
  0x33EC,   /* U+104EF */
  0x33ED,   /* U+104F0 */
  0x33EE,   /* U+104F1 */
  0x33EF,   /* U+104F2 */
  0x33F0,   /* U+104F3 */
  0x33F1,   /* U+104F4 */
  0x33F2,   /* U+104F5 */
  0x33F3,   /* U+104F6 */
  0x33F4,   /* U+104F7 */
  0x33F5,   /* U+104F8 */
  0x33F6,   /* U+104F9 */
  0x33F7,   /* U+104FA */
  0x33F8,   /* U+104FB */
  0xFBC2,   /* U+104FC */
  0xFBC2,   /* U+104FD */
  0xFBC2,   /* U+104FE */
  0xFBC2,   /* U+104FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10400 */
  0x0020,   /* U+10401 */
  0x0020,   /* U+10402 */
  0x0020,   /* U+10403 */
  0x0020,   /* U+10404 */
  0x0020,   /* U+10405 */
  0x0020,   /* U+10406 */
  0x0020,   /* U+10407 */
  0x0020,   /* U+10408 */
  0x0020,   /* U+10409 */
  0x0020,   /* U+1040A */
  0x0020,   /* U+1040B */
  0x0020,   /* U+1040C */
  0x0020,   /* U+1040D */
  0x0020,   /* U+1040E */
  0x0020,   /* U+1040F */
  0x0020,   /* U+10410 */
  0x0020,   /* U+10411 */
  0x0020,   /* U+10412 */
  0x0020,   /* U+10413 */
  0x0020,   /* U+10414 */
  0x0020,   /* U+10415 */
  0x0020,   /* U+10416 */
  0x0020,   /* U+10417 */
  0x0020,   /* U+10418 */
  0x0020,   /* U+10419 */
  0x0020,   /* U+1041A */
  0x0020,   /* U+1041B */
  0x0020,   /* U+1041C */
  0x0020,   /* U+1041D */
  0x0020,   /* U+1041E */
  0x0020,   /* U+1041F */
  0x0020,   /* U+10420 */
  0x0020,   /* U+10421 */
  0x0020,   /* U+10422 */
  0x0020,   /* U+10423 */
  0x0020,   /* U+10424 */
  0x0020,   /* U+10425 */
  0x0020,   /* U+10426 */
  0x0020,   /* U+10427 */
  0x0020,   /* U+10428 */
  0x0020,   /* U+10429 */
  0x0020,   /* U+1042A */
  0x0020,   /* U+1042B */
  0x0020,   /* U+1042C */
  0x0020,   /* U+1042D */
  0x0020,   /* U+1042E */
  0x0020,   /* U+1042F */
  0x0020,   /* U+10430 */
  0x0020,   /* U+10431 */
  0x0020,   /* U+10432 */
  0x0020,   /* U+10433 */
  0x0020,   /* U+10434 */
  0x0020,   /* U+10435 */
  0x0020,   /* U+10436 */
  0x0020,   /* U+10437 */
  0x0020,   /* U+10438 */
  0x0020,   /* U+10439 */
  0x0020,   /* U+1043A */
  0x0020,   /* U+1043B */
  0x0020,   /* U+1043C */
  0x0020,   /* U+1043D */
  0x0020,   /* U+1043E */
  0x0020,   /* U+1043F */
  0x0020,   /* U+10440 */
  0x0020,   /* U+10441 */
  0x0020,   /* U+10442 */
  0x0020,   /* U+10443 */
  0x0020,   /* U+10444 */
  0x0020,   /* U+10445 */
  0x0020,   /* U+10446 */
  0x0020,   /* U+10447 */
  0x0020,   /* U+10448 */
  0x0020,   /* U+10449 */
  0x0020,   /* U+1044A */
  0x0020,   /* U+1044B */
  0x0020,   /* U+1044C */
  0x0020,   /* U+1044D */
  0x0020,   /* U+1044E */
  0x0020,   /* U+1044F */
  0x0020,   /* U+10450 */
  0x0020,   /* U+10451 */
  0x0020,   /* U+10452 */
  0x0020,   /* U+10453 */
  0x0020,   /* U+10454 */
  0x0020,   /* U+10455 */
  0x0020,   /* U+10456 */
  0x0020,   /* U+10457 */
  0x0020,   /* U+10458 */
  0x0020,   /* U+10459 */
  0x0020,   /* U+1045A */
  0x0020,   /* U+1045B */
  0x0020,   /* U+1045C */
  0x0020,   /* U+1045D */
  0x0020,   /* U+1045E */
  0x0020,   /* U+1045F */
  0x0020,   /* U+10460 */
  0x0020,   /* U+10461 */
  0x0020,   /* U+10462 */
  0x0020,   /* U+10463 */
  0x0020,   /* U+10464 */
  0x0020,   /* U+10465 */
  0x0020,   /* U+10466 */
  0x0020,   /* U+10467 */
  0x0020,   /* U+10468 */
  0x0020,   /* U+10469 */
  0x0020,   /* U+1046A */
  0x0020,   /* U+1046B */
  0x0020,   /* U+1046C */
  0x0020,   /* U+1046D */
  0x0020,   /* U+1046E */
  0x0020,   /* U+1046F */
  0x0020,   /* U+10470 */
  0x0020,   /* U+10471 */
  0x0020,   /* U+10472 */
  0x0020,   /* U+10473 */
  0x0020,   /* U+10474 */
  0x0020,   /* U+10475 */
  0x0020,   /* U+10476 */
  0x0020,   /* U+10477 */
  0x0020,   /* U+10478 */
  0x0020,   /* U+10479 */
  0x0020,   /* U+1047A */
  0x0020,   /* U+1047B */
  0x0020,   /* U+1047C */
  0x0020,   /* U+1047D */
  0x0020,   /* U+1047E */
  0x0020,   /* U+1047F */
  0x0020,   /* U+10480 */
  0x0020,   /* U+10481 */
  0x0020,   /* U+10482 */
  0x0020,   /* U+10483 */
  0x0020,   /* U+10484 */
  0x0020,   /* U+10485 */
  0x0020,   /* U+10486 */
  0x0020,   /* U+10487 */
  0x0020,   /* U+10488 */
  0x0020,   /* U+10489 */
  0x0020,   /* U+1048A */
  0x0020,   /* U+1048B */
  0x0020,   /* U+1048C */
  0x0020,   /* U+1048D */
  0x0020,   /* U+1048E */
  0x0020,   /* U+1048F */
  0x0020,   /* U+10490 */
  0x0020,   /* U+10491 */
  0x0020,   /* U+10492 */
  0x0020,   /* U+10493 */
  0x0020,   /* U+10494 */
  0x0020,   /* U+10495 */
  0x0020,   /* U+10496 */
  0x0020,   /* U+10497 */
  0x0020,   /* U+10498 */
  0x0020,   /* U+10499 */
  0x0020,   /* U+1049A */
  0x0020,   /* U+1049B */
  0x0020,   /* U+1049C */
  0x0020,   /* U+1049D */
  0x0020,   /* U+1049E */
  0x0020,   /* U+1049F */
  0x0020,   /* U+104A0 */
  0x0020,   /* U+104A1 */
  0x0020,   /* U+104A2 */
  0x0020,   /* U+104A3 */
  0x0020,   /* U+104A4 */
  0x0020,   /* U+104A5 */
  0x0020,   /* U+104A6 */
  0x0020,   /* U+104A7 */
  0x0020,   /* U+104A8 */
  0x0020,   /* U+104A9 */
  0x0020,   /* U+104AA */
  0x0020,   /* U+104AB */
  0x0020,   /* U+104AC */
  0x0020,   /* U+104AD */
  0x0020,   /* U+104AE */
  0x0020,   /* U+104AF */
  0x0020,   /* U+104B0 */
  0x0020,   /* U+104B1 */
  0x0020,   /* U+104B2 */
  0x0020,   /* U+104B3 */
  0x0020,   /* U+104B4 */
  0x0020,   /* U+104B5 */
  0x0020,   /* U+104B6 */
  0x0020,   /* U+104B7 */
  0x0020,   /* U+104B8 */
  0x0020,   /* U+104B9 */
  0x0020,   /* U+104BA */
  0x0020,   /* U+104BB */
  0x0020,   /* U+104BC */
  0x0020,   /* U+104BD */
  0x0020,   /* U+104BE */
  0x0020,   /* U+104BF */
  0x0020,   /* U+104C0 */
  0x0020,   /* U+104C1 */
  0x0020,   /* U+104C2 */
  0x0020,   /* U+104C3 */
  0x0020,   /* U+104C4 */
  0x0020,   /* U+104C5 */
  0x0020,   /* U+104C6 */
  0x0020,   /* U+104C7 */
  0x0020,   /* U+104C8 */
  0x0020,   /* U+104C9 */
  0x0020,   /* U+104CA */
  0x0020,   /* U+104CB */
  0x0020,   /* U+104CC */
  0x0020,   /* U+104CD */
  0x0020,   /* U+104CE */
  0x0020,   /* U+104CF */
  0x0020,   /* U+104D0 */
  0x0020,   /* U+104D1 */
  0x0020,   /* U+104D2 */
  0x0020,   /* U+104D3 */
  0x0020,   /* U+104D4 */
  0x0020,   /* U+104D5 */
  0x0020,   /* U+104D6 */
  0x0020,   /* U+104D7 */
  0x0020,   /* U+104D8 */
  0x0020,   /* U+104D9 */
  0x0020,   /* U+104DA */
  0x0020,   /* U+104DB */
  0x0020,   /* U+104DC */
  0x0020,   /* U+104DD */
  0x0020,   /* U+104DE */
  0x0020,   /* U+104DF */
  0x0020,   /* U+104E0 */
  0x0020,   /* U+104E1 */
  0x0020,   /* U+104E2 */
  0x0020,   /* U+104E3 */
  0x0020,   /* U+104E4 */
  0x0020,   /* U+104E5 */
  0x0020,   /* U+104E6 */
  0x0020,   /* U+104E7 */
  0x0020,   /* U+104E8 */
  0x0020,   /* U+104E9 */
  0x0020,   /* U+104EA */
  0x0020,   /* U+104EB */
  0x0020,   /* U+104EC */
  0x0020,   /* U+104ED */
  0x0020,   /* U+104EE */
  0x0020,   /* U+104EF */
  0x0020,   /* U+104F0 */
  0x0020,   /* U+104F1 */
  0x0020,   /* U+104F2 */
  0x0020,   /* U+104F3 */
  0x0020,   /* U+104F4 */
  0x0020,   /* U+104F5 */
  0x0020,   /* U+104F6 */
  0x0020,   /* U+104F7 */
  0x0020,   /* U+104F8 */
  0x0020,   /* U+104F9 */
  0x0020,   /* U+104FA */
  0x0020,   /* U+104FB */
  0x0020,   /* U+104FC */
  0x0020,   /* U+104FD */
  0x0020,   /* U+104FE */
  0x0020,   /* U+104FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+10400 */
  0x0008,   /* U+10401 */
  0x0008,   /* U+10402 */
  0x0008,   /* U+10403 */
  0x0008,   /* U+10404 */
  0x0008,   /* U+10405 */
  0x0008,   /* U+10406 */
  0x0008,   /* U+10407 */
  0x0008,   /* U+10408 */
  0x0008,   /* U+10409 */
  0x0008,   /* U+1040A */
  0x0008,   /* U+1040B */
  0x0008,   /* U+1040C */
  0x0008,   /* U+1040D */
  0x0008,   /* U+1040E */
  0x0008,   /* U+1040F */
  0x0008,   /* U+10410 */
  0x0008,   /* U+10411 */
  0x0008,   /* U+10412 */
  0x0008,   /* U+10413 */
  0x0008,   /* U+10414 */
  0x0008,   /* U+10415 */
  0x0008,   /* U+10416 */
  0x0008,   /* U+10417 */
  0x0008,   /* U+10418 */
  0x0008,   /* U+10419 */
  0x0008,   /* U+1041A */
  0x0008,   /* U+1041B */
  0x0008,   /* U+1041C */
  0x0008,   /* U+1041D */
  0x0008,   /* U+1041E */
  0x0008,   /* U+1041F */
  0x0008,   /* U+10420 */
  0x0008,   /* U+10421 */
  0x0008,   /* U+10422 */
  0x0008,   /* U+10423 */
  0x0008,   /* U+10424 */
  0x0008,   /* U+10425 */
  0x0008,   /* U+10426 */
  0x0008,   /* U+10427 */
  0x0002,   /* U+10428 */
  0x0002,   /* U+10429 */
  0x0002,   /* U+1042A */
  0x0002,   /* U+1042B */
  0x0002,   /* U+1042C */
  0x0002,   /* U+1042D */
  0x0002,   /* U+1042E */
  0x0002,   /* U+1042F */
  0x0002,   /* U+10430 */
  0x0002,   /* U+10431 */
  0x0002,   /* U+10432 */
  0x0002,   /* U+10433 */
  0x0002,   /* U+10434 */
  0x0002,   /* U+10435 */
  0x0002,   /* U+10436 */
  0x0002,   /* U+10437 */
  0x0002,   /* U+10438 */
  0x0002,   /* U+10439 */
  0x0002,   /* U+1043A */
  0x0002,   /* U+1043B */
  0x0002,   /* U+1043C */
  0x0002,   /* U+1043D */
  0x0002,   /* U+1043E */
  0x0002,   /* U+1043F */
  0x0002,   /* U+10440 */
  0x0002,   /* U+10441 */
  0x0002,   /* U+10442 */
  0x0002,   /* U+10443 */
  0x0002,   /* U+10444 */
  0x0002,   /* U+10445 */
  0x0002,   /* U+10446 */
  0x0002,   /* U+10447 */
  0x0002,   /* U+10448 */
  0x0002,   /* U+10449 */
  0x0002,   /* U+1044A */
  0x0002,   /* U+1044B */
  0x0002,   /* U+1044C */
  0x0002,   /* U+1044D */
  0x0002,   /* U+1044E */
  0x0002,   /* U+1044F */
  0x0002,   /* U+10450 */
  0x0002,   /* U+10451 */
  0x0002,   /* U+10452 */
  0x0002,   /* U+10453 */
  0x0002,   /* U+10454 */
  0x0002,   /* U+10455 */
  0x0002,   /* U+10456 */
  0x0002,   /* U+10457 */
  0x0002,   /* U+10458 */
  0x0002,   /* U+10459 */
  0x0002,   /* U+1045A */
  0x0002,   /* U+1045B */
  0x0002,   /* U+1045C */
  0x0002,   /* U+1045D */
  0x0002,   /* U+1045E */
  0x0002,   /* U+1045F */
  0x0002,   /* U+10460 */
  0x0002,   /* U+10461 */
  0x0002,   /* U+10462 */
  0x0002,   /* U+10463 */
  0x0002,   /* U+10464 */
  0x0002,   /* U+10465 */
  0x0002,   /* U+10466 */
  0x0002,   /* U+10467 */
  0x0002,   /* U+10468 */
  0x0002,   /* U+10469 */
  0x0002,   /* U+1046A */
  0x0002,   /* U+1046B */
  0x0002,   /* U+1046C */
  0x0002,   /* U+1046D */
  0x0002,   /* U+1046E */
  0x0002,   /* U+1046F */
  0x0002,   /* U+10470 */
  0x0002,   /* U+10471 */
  0x0002,   /* U+10472 */
  0x0002,   /* U+10473 */
  0x0002,   /* U+10474 */
  0x0002,   /* U+10475 */
  0x0002,   /* U+10476 */
  0x0002,   /* U+10477 */
  0x0002,   /* U+10478 */
  0x0002,   /* U+10479 */
  0x0002,   /* U+1047A */
  0x0002,   /* U+1047B */
  0x0002,   /* U+1047C */
  0x0002,   /* U+1047D */
  0x0002,   /* U+1047E */
  0x0002,   /* U+1047F */
  0x0002,   /* U+10480 */
  0x0002,   /* U+10481 */
  0x0002,   /* U+10482 */
  0x0002,   /* U+10483 */
  0x0002,   /* U+10484 */
  0x0002,   /* U+10485 */
  0x0002,   /* U+10486 */
  0x0002,   /* U+10487 */
  0x0002,   /* U+10488 */
  0x0002,   /* U+10489 */
  0x0002,   /* U+1048A */
  0x0002,   /* U+1048B */
  0x0002,   /* U+1048C */
  0x0002,   /* U+1048D */
  0x0002,   /* U+1048E */
  0x0002,   /* U+1048F */
  0x0002,   /* U+10490 */
  0x0002,   /* U+10491 */
  0x0002,   /* U+10492 */
  0x0002,   /* U+10493 */
  0x0002,   /* U+10494 */
  0x0002,   /* U+10495 */
  0x0002,   /* U+10496 */
  0x0002,   /* U+10497 */
  0x0002,   /* U+10498 */
  0x0002,   /* U+10499 */
  0x0002,   /* U+1049A */
  0x0002,   /* U+1049B */
  0x0002,   /* U+1049C */
  0x0002,   /* U+1049D */
  0x0002,   /* U+1049E */
  0x0002,   /* U+1049F */
  0x0002,   /* U+104A0 */
  0x0002,   /* U+104A1 */
  0x0002,   /* U+104A2 */
  0x0002,   /* U+104A3 */
  0x0002,   /* U+104A4 */
  0x0002,   /* U+104A5 */
  0x0002,   /* U+104A6 */
  0x0002,   /* U+104A7 */
  0x0002,   /* U+104A8 */
  0x0002,   /* U+104A9 */
  0x0002,   /* U+104AA */
  0x0002,   /* U+104AB */
  0x0002,   /* U+104AC */
  0x0002,   /* U+104AD */
  0x0002,   /* U+104AE */
  0x0002,   /* U+104AF */
  0x0008,   /* U+104B0 */
  0x0008,   /* U+104B1 */
  0x0008,   /* U+104B2 */
  0x0008,   /* U+104B3 */
  0x0008,   /* U+104B4 */
  0x0008,   /* U+104B5 */
  0x0008,   /* U+104B6 */
  0x0008,   /* U+104B7 */
  0x0008,   /* U+104B8 */
  0x0008,   /* U+104B9 */
  0x0008,   /* U+104BA */
  0x0008,   /* U+104BB */
  0x0008,   /* U+104BC */
  0x0008,   /* U+104BD */
  0x0008,   /* U+104BE */
  0x0008,   /* U+104BF */
  0x0008,   /* U+104C0 */
  0x0008,   /* U+104C1 */
  0x0008,   /* U+104C2 */
  0x0008,   /* U+104C3 */
  0x0008,   /* U+104C4 */
  0x0008,   /* U+104C5 */
  0x0008,   /* U+104C6 */
  0x0008,   /* U+104C7 */
  0x0008,   /* U+104C8 */
  0x0008,   /* U+104C9 */
  0x0008,   /* U+104CA */
  0x0008,   /* U+104CB */
  0x0008,   /* U+104CC */
  0x0008,   /* U+104CD */
  0x0008,   /* U+104CE */
  0x0008,   /* U+104CF */
  0x0008,   /* U+104D0 */
  0x0008,   /* U+104D1 */
  0x0008,   /* U+104D2 */
  0x0008,   /* U+104D3 */
  0x0002,   /* U+104D4 */
  0x0002,   /* U+104D5 */
  0x0002,   /* U+104D6 */
  0x0002,   /* U+104D7 */
  0x0002,   /* U+104D8 */
  0x0002,   /* U+104D9 */
  0x0002,   /* U+104DA */
  0x0002,   /* U+104DB */
  0x0002,   /* U+104DC */
  0x0002,   /* U+104DD */
  0x0002,   /* U+104DE */
  0x0002,   /* U+104DF */
  0x0002,   /* U+104E0 */
  0x0002,   /* U+104E1 */
  0x0002,   /* U+104E2 */
  0x0002,   /* U+104E3 */
  0x0002,   /* U+104E4 */
  0x0002,   /* U+104E5 */
  0x0002,   /* U+104E6 */
  0x0002,   /* U+104E7 */
  0x0002,   /* U+104E8 */
  0x0002,   /* U+104E9 */
  0x0002,   /* U+104EA */
  0x0002,   /* U+104EB */
  0x0002,   /* U+104EC */
  0x0002,   /* U+104ED */
  0x0002,   /* U+104EE */
  0x0002,   /* U+104EF */
  0x0002,   /* U+104F0 */
  0x0002,   /* U+104F1 */
  0x0002,   /* U+104F2 */
  0x0002,   /* U+104F3 */
  0x0002,   /* U+104F4 */
  0x0002,   /* U+104F5 */
  0x0002,   /* U+104F6 */
  0x0002,   /* U+104F7 */
  0x0002,   /* U+104F8 */
  0x0002,   /* U+104F9 */
  0x0002,   /* U+104FA */
  0x0002,   /* U+104FB */
  0x0002,   /* U+104FC */
  0x0002,   /* U+104FD */
  0x0002,   /* U+104FE */
  0x0002,   /* U+104FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10400 */
  0x0000,   /* U+10401 */
  0x0000,   /* U+10402 */
  0x0000,   /* U+10403 */
  0x0000,   /* U+10404 */
  0x0000,   /* U+10405 */
  0x0000,   /* U+10406 */
  0x0000,   /* U+10407 */
  0x0000,   /* U+10408 */
  0x0000,   /* U+10409 */
  0x0000,   /* U+1040A */
  0x0000,   /* U+1040B */
  0x0000,   /* U+1040C */
  0x0000,   /* U+1040D */
  0x0000,   /* U+1040E */
  0x0000,   /* U+1040F */
  0x0000,   /* U+10410 */
  0x0000,   /* U+10411 */
  0x0000,   /* U+10412 */
  0x0000,   /* U+10413 */
  0x0000,   /* U+10414 */
  0x0000,   /* U+10415 */
  0x0000,   /* U+10416 */
  0x0000,   /* U+10417 */
  0x0000,   /* U+10418 */
  0x0000,   /* U+10419 */
  0x0000,   /* U+1041A */
  0x0000,   /* U+1041B */
  0x0000,   /* U+1041C */
  0x0000,   /* U+1041D */
  0x0000,   /* U+1041E */
  0x0000,   /* U+1041F */
  0x0000,   /* U+10420 */
  0x0000,   /* U+10421 */
  0x0000,   /* U+10422 */
  0x0000,   /* U+10423 */
  0x0000,   /* U+10424 */
  0x0000,   /* U+10425 */
  0x0000,   /* U+10426 */
  0x0000,   /* U+10427 */
  0x0000,   /* U+10428 */
  0x0000,   /* U+10429 */
  0x0000,   /* U+1042A */
  0x0000,   /* U+1042B */
  0x0000,   /* U+1042C */
  0x0000,   /* U+1042D */
  0x0000,   /* U+1042E */
  0x0000,   /* U+1042F */
  0x0000,   /* U+10430 */
  0x0000,   /* U+10431 */
  0x0000,   /* U+10432 */
  0x0000,   /* U+10433 */
  0x0000,   /* U+10434 */
  0x0000,   /* U+10435 */
  0x0000,   /* U+10436 */
  0x0000,   /* U+10437 */
  0x0000,   /* U+10438 */
  0x0000,   /* U+10439 */
  0x0000,   /* U+1043A */
  0x0000,   /* U+1043B */
  0x0000,   /* U+1043C */
  0x0000,   /* U+1043D */
  0x0000,   /* U+1043E */
  0x0000,   /* U+1043F */
  0x0000,   /* U+10440 */
  0x0000,   /* U+10441 */
  0x0000,   /* U+10442 */
  0x0000,   /* U+10443 */
  0x0000,   /* U+10444 */
  0x0000,   /* U+10445 */
  0x0000,   /* U+10446 */
  0x0000,   /* U+10447 */
  0x0000,   /* U+10448 */
  0x0000,   /* U+10449 */
  0x0000,   /* U+1044A */
  0x0000,   /* U+1044B */
  0x0000,   /* U+1044C */
  0x0000,   /* U+1044D */
  0x0000,   /* U+1044E */
  0x0000,   /* U+1044F */
  0x0000,   /* U+10450 */
  0x0000,   /* U+10451 */
  0x0000,   /* U+10452 */
  0x0000,   /* U+10453 */
  0x0000,   /* U+10454 */
  0x0000,   /* U+10455 */
  0x0000,   /* U+10456 */
  0x0000,   /* U+10457 */
  0x0000,   /* U+10458 */
  0x0000,   /* U+10459 */
  0x0000,   /* U+1045A */
  0x0000,   /* U+1045B */
  0x0000,   /* U+1045C */
  0x0000,   /* U+1045D */
  0x0000,   /* U+1045E */
  0x0000,   /* U+1045F */
  0x0000,   /* U+10460 */
  0x0000,   /* U+10461 */
  0x0000,   /* U+10462 */
  0x0000,   /* U+10463 */
  0x0000,   /* U+10464 */
  0x0000,   /* U+10465 */
  0x0000,   /* U+10466 */
  0x0000,   /* U+10467 */
  0x0000,   /* U+10468 */
  0x0000,   /* U+10469 */
  0x0000,   /* U+1046A */
  0x0000,   /* U+1046B */
  0x0000,   /* U+1046C */
  0x0000,   /* U+1046D */
  0x0000,   /* U+1046E */
  0x0000,   /* U+1046F */
  0x0000,   /* U+10470 */
  0x0000,   /* U+10471 */
  0x0000,   /* U+10472 */
  0x0000,   /* U+10473 */
  0x0000,   /* U+10474 */
  0x0000,   /* U+10475 */
  0x0000,   /* U+10476 */
  0x0000,   /* U+10477 */
  0x0000,   /* U+10478 */
  0x0000,   /* U+10479 */
  0x0000,   /* U+1047A */
  0x0000,   /* U+1047B */
  0x0000,   /* U+1047C */
  0x0000,   /* U+1047D */
  0x0000,   /* U+1047E */
  0x0000,   /* U+1047F */
  0x0000,   /* U+10480 */
  0x0000,   /* U+10481 */
  0x0000,   /* U+10482 */
  0x0000,   /* U+10483 */
  0x0000,   /* U+10484 */
  0x0000,   /* U+10485 */
  0x0000,   /* U+10486 */
  0x0000,   /* U+10487 */
  0x0000,   /* U+10488 */
  0x0000,   /* U+10489 */
  0x0000,   /* U+1048A */
  0x0000,   /* U+1048B */
  0x0000,   /* U+1048C */
  0x0000,   /* U+1048D */
  0x0000,   /* U+1048E */
  0x0000,   /* U+1048F */
  0x0000,   /* U+10490 */
  0x0000,   /* U+10491 */
  0x0000,   /* U+10492 */
  0x0000,   /* U+10493 */
  0x0000,   /* U+10494 */
  0x0000,   /* U+10495 */
  0x0000,   /* U+10496 */
  0x0000,   /* U+10497 */
  0x0000,   /* U+10498 */
  0x0000,   /* U+10499 */
  0x0000,   /* U+1049A */
  0x0000,   /* U+1049B */
  0x0000,   /* U+1049C */
  0x0000,   /* U+1049D */
  0x849E,   /* U+1049E */
  0x849F,   /* U+1049F */
  0x0000,   /* U+104A0 */
  0x0000,   /* U+104A1 */
  0x0000,   /* U+104A2 */
  0x0000,   /* U+104A3 */
  0x0000,   /* U+104A4 */
  0x0000,   /* U+104A5 */
  0x0000,   /* U+104A6 */
  0x0000,   /* U+104A7 */
  0x0000,   /* U+104A8 */
  0x0000,   /* U+104A9 */
  0x84AA,   /* U+104AA */
  0x84AB,   /* U+104AB */
  0x84AC,   /* U+104AC */
  0x84AD,   /* U+104AD */
  0x84AE,   /* U+104AE */
  0x84AF,   /* U+104AF */
  0x0000,   /* U+104B0 */
  0x0000,   /* U+104B1 */
  0x0000,   /* U+104B2 */
  0x0000,   /* U+104B3 */
  0x0000,   /* U+104B4 */
  0x0000,   /* U+104B5 */
  0x0000,   /* U+104B6 */
  0x0000,   /* U+104B7 */
  0x0000,   /* U+104B8 */
  0x0000,   /* U+104B9 */
  0x0000,   /* U+104BA */
  0x0000,   /* U+104BB */
  0x0000,   /* U+104BC */
  0x0000,   /* U+104BD */
  0x0000,   /* U+104BE */
  0x0000,   /* U+104BF */
  0x0000,   /* U+104C0 */
  0x0000,   /* U+104C1 */
  0x0000,   /* U+104C2 */
  0x0000,   /* U+104C3 */
  0x0000,   /* U+104C4 */
  0x0000,   /* U+104C5 */
  0x0000,   /* U+104C6 */
  0x0000,   /* U+104C7 */
  0x0000,   /* U+104C8 */
  0x0000,   /* U+104C9 */
  0x0000,   /* U+104CA */
  0x0000,   /* U+104CB */
  0x0000,   /* U+104CC */
  0x0000,   /* U+104CD */
  0x0000,   /* U+104CE */
  0x0000,   /* U+104CF */
  0x0000,   /* U+104D0 */
  0x0000,   /* U+104D1 */
  0x0000,   /* U+104D2 */
  0x0000,   /* U+104D3 */
  0x84D4,   /* U+104D4 */
  0x84D5,   /* U+104D5 */
  0x84D6,   /* U+104D6 */
  0x84D7,   /* U+104D7 */
  0x0000,   /* U+104D8 */
  0x0000,   /* U+104D9 */
  0x0000,   /* U+104DA */
  0x0000,   /* U+104DB */
  0x0000,   /* U+104DC */
  0x0000,   /* U+104DD */
  0x0000,   /* U+104DE */
  0x0000,   /* U+104DF */
  0x0000,   /* U+104E0 */
  0x0000,   /* U+104E1 */
  0x0000,   /* U+104E2 */
  0x0000,   /* U+104E3 */
  0x0000,   /* U+104E4 */
  0x0000,   /* U+104E5 */
  0x0000,   /* U+104E6 */
  0x0000,   /* U+104E7 */
  0x0000,   /* U+104E8 */
  0x0000,   /* U+104E9 */
  0x0000,   /* U+104EA */
  0x0000,   /* U+104EB */
  0x0000,   /* U+104EC */
  0x0000,   /* U+104ED */
  0x0000,   /* U+104EE */
  0x0000,   /* U+104EF */
  0x0000,   /* U+104F0 */
  0x0000,   /* U+104F1 */
  0x0000,   /* U+104F2 */
  0x0000,   /* U+104F3 */
  0x0000,   /* U+104F4 */
  0x0000,   /* U+104F5 */
  0x0000,   /* U+104F6 */
  0x0000,   /* U+104F7 */
  0x0000,   /* U+104F8 */
  0x0000,   /* U+104F9 */
  0x0000,   /* U+104FA */
  0x0000,   /* U+104FB */
  0x84FC,   /* U+104FC */
  0x84FD,   /* U+104FD */
  0x84FE,   /* U+104FE */
  0x84FF,   /* U+104FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10400 */
  0x0000,   /* U+10401 */
  0x0000,   /* U+10402 */
  0x0000,   /* U+10403 */
  0x0000,   /* U+10404 */
  0x0000,   /* U+10405 */
  0x0000,   /* U+10406 */
  0x0000,   /* U+10407 */
  0x0000,   /* U+10408 */
  0x0000,   /* U+10409 */
  0x0000,   /* U+1040A */
  0x0000,   /* U+1040B */
  0x0000,   /* U+1040C */
  0x0000,   /* U+1040D */
  0x0000,   /* U+1040E */
  0x0000,   /* U+1040F */
  0x0000,   /* U+10410 */
  0x0000,   /* U+10411 */
  0x0000,   /* U+10412 */
  0x0000,   /* U+10413 */
  0x0000,   /* U+10414 */
  0x0000,   /* U+10415 */
  0x0000,   /* U+10416 */
  0x0000,   /* U+10417 */
  0x0000,   /* U+10418 */
  0x0000,   /* U+10419 */
  0x0000,   /* U+1041A */
  0x0000,   /* U+1041B */
  0x0000,   /* U+1041C */
  0x0000,   /* U+1041D */
  0x0000,   /* U+1041E */
  0x0000,   /* U+1041F */
  0x0000,   /* U+10420 */
  0x0000,   /* U+10421 */
  0x0000,   /* U+10422 */
  0x0000,   /* U+10423 */
  0x0000,   /* U+10424 */
  0x0000,   /* U+10425 */
  0x0000,   /* U+10426 */
  0x0000,   /* U+10427 */
  0x0000,   /* U+10428 */
  0x0000,   /* U+10429 */
  0x0000,   /* U+1042A */
  0x0000,   /* U+1042B */
  0x0000,   /* U+1042C */
  0x0000,   /* U+1042D */
  0x0000,   /* U+1042E */
  0x0000,   /* U+1042F */
  0x0000,   /* U+10430 */
  0x0000,   /* U+10431 */
  0x0000,   /* U+10432 */
  0x0000,   /* U+10433 */
  0x0000,   /* U+10434 */
  0x0000,   /* U+10435 */
  0x0000,   /* U+10436 */
  0x0000,   /* U+10437 */
  0x0000,   /* U+10438 */
  0x0000,   /* U+10439 */
  0x0000,   /* U+1043A */
  0x0000,   /* U+1043B */
  0x0000,   /* U+1043C */
  0x0000,   /* U+1043D */
  0x0000,   /* U+1043E */
  0x0000,   /* U+1043F */
  0x0000,   /* U+10440 */
  0x0000,   /* U+10441 */
  0x0000,   /* U+10442 */
  0x0000,   /* U+10443 */
  0x0000,   /* U+10444 */
  0x0000,   /* U+10445 */
  0x0000,   /* U+10446 */
  0x0000,   /* U+10447 */
  0x0000,   /* U+10448 */
  0x0000,   /* U+10449 */
  0x0000,   /* U+1044A */
  0x0000,   /* U+1044B */
  0x0000,   /* U+1044C */
  0x0000,   /* U+1044D */
  0x0000,   /* U+1044E */
  0x0000,   /* U+1044F */
  0x0000,   /* U+10450 */
  0x0000,   /* U+10451 */
  0x0000,   /* U+10452 */
  0x0000,   /* U+10453 */
  0x0000,   /* U+10454 */
  0x0000,   /* U+10455 */
  0x0000,   /* U+10456 */
  0x0000,   /* U+10457 */
  0x0000,   /* U+10458 */
  0x0000,   /* U+10459 */
  0x0000,   /* U+1045A */
  0x0000,   /* U+1045B */
  0x0000,   /* U+1045C */
  0x0000,   /* U+1045D */
  0x0000,   /* U+1045E */
  0x0000,   /* U+1045F */
  0x0000,   /* U+10460 */
  0x0000,   /* U+10461 */
  0x0000,   /* U+10462 */
  0x0000,   /* U+10463 */
  0x0000,   /* U+10464 */
  0x0000,   /* U+10465 */
  0x0000,   /* U+10466 */
  0x0000,   /* U+10467 */
  0x0000,   /* U+10468 */
  0x0000,   /* U+10469 */
  0x0000,   /* U+1046A */
  0x0000,   /* U+1046B */
  0x0000,   /* U+1046C */
  0x0000,   /* U+1046D */
  0x0000,   /* U+1046E */
  0x0000,   /* U+1046F */
  0x0000,   /* U+10470 */
  0x0000,   /* U+10471 */
  0x0000,   /* U+10472 */
  0x0000,   /* U+10473 */
  0x0000,   /* U+10474 */
  0x0000,   /* U+10475 */
  0x0000,   /* U+10476 */
  0x0000,   /* U+10477 */
  0x0000,   /* U+10478 */
  0x0000,   /* U+10479 */
  0x0000,   /* U+1047A */
  0x0000,   /* U+1047B */
  0x0000,   /* U+1047C */
  0x0000,   /* U+1047D */
  0x0000,   /* U+1047E */
  0x0000,   /* U+1047F */
  0x0000,   /* U+10480 */
  0x0000,   /* U+10481 */
  0x0000,   /* U+10482 */
  0x0000,   /* U+10483 */
  0x0000,   /* U+10484 */
  0x0000,   /* U+10485 */
  0x0000,   /* U+10486 */
  0x0000,   /* U+10487 */
  0x0000,   /* U+10488 */
  0x0000,   /* U+10489 */
  0x0000,   /* U+1048A */
  0x0000,   /* U+1048B */
  0x0000,   /* U+1048C */
  0x0000,   /* U+1048D */
  0x0000,   /* U+1048E */
  0x0000,   /* U+1048F */
  0x0000,   /* U+10490 */
  0x0000,   /* U+10491 */
  0x0000,   /* U+10492 */
  0x0000,   /* U+10493 */
  0x0000,   /* U+10494 */
  0x0000,   /* U+10495 */
  0x0000,   /* U+10496 */
  0x0000,   /* U+10497 */
  0x0000,   /* U+10498 */
  0x0000,   /* U+10499 */
  0x0000,   /* U+1049A */
  0x0000,   /* U+1049B */
  0x0000,   /* U+1049C */
  0x0000,   /* U+1049D */
  0x0000,   /* U+1049E */
  0x0000,   /* U+1049F */
  0x0000,   /* U+104A0 */
  0x0000,   /* U+104A1 */
  0x0000,   /* U+104A2 */
  0x0000,   /* U+104A3 */
  0x0000,   /* U+104A4 */
  0x0000,   /* U+104A5 */
  0x0000,   /* U+104A6 */
  0x0000,   /* U+104A7 */
  0x0000,   /* U+104A8 */
  0x0000,   /* U+104A9 */
  0x0000,   /* U+104AA */
  0x0000,   /* U+104AB */
  0x0000,   /* U+104AC */
  0x0000,   /* U+104AD */
  0x0000,   /* U+104AE */
  0x0000,   /* U+104AF */
  0x0000,   /* U+104B0 */
  0x0000,   /* U+104B1 */
  0x0000,   /* U+104B2 */
  0x0000,   /* U+104B3 */
  0x0000,   /* U+104B4 */
  0x0000,   /* U+104B5 */
  0x0000,   /* U+104B6 */
  0x0000,   /* U+104B7 */
  0x0000,   /* U+104B8 */
  0x0000,   /* U+104B9 */
  0x0000,   /* U+104BA */
  0x0000,   /* U+104BB */
  0x0000,   /* U+104BC */
  0x0000,   /* U+104BD */
  0x0000,   /* U+104BE */
  0x0000,   /* U+104BF */
  0x0000,   /* U+104C0 */
  0x0000,   /* U+104C1 */
  0x0000,   /* U+104C2 */
  0x0000,   /* U+104C3 */
  0x0000,   /* U+104C4 */
  0x0000,   /* U+104C5 */
  0x0000,   /* U+104C6 */
  0x0000,   /* U+104C7 */
  0x0000,   /* U+104C8 */
  0x0000,   /* U+104C9 */
  0x0000,   /* U+104CA */
  0x0000,   /* U+104CB */
  0x0000,   /* U+104CC */
  0x0000,   /* U+104CD */
  0x0000,   /* U+104CE */
  0x0000,   /* U+104CF */
  0x0000,   /* U+104D0 */
  0x0000,   /* U+104D1 */
  0x0000,   /* U+104D2 */
  0x0000,   /* U+104D3 */
  0x0000,   /* U+104D4 */
  0x0000,   /* U+104D5 */
  0x0000,   /* U+104D6 */
  0x0000,   /* U+104D7 */
  0x0000,   /* U+104D8 */
  0x0000,   /* U+104D9 */
  0x0000,   /* U+104DA */
  0x0000,   /* U+104DB */
  0x0000,   /* U+104DC */
  0x0000,   /* U+104DD */
  0x0000,   /* U+104DE */
  0x0000,   /* U+104DF */
  0x0000,   /* U+104E0 */
  0x0000,   /* U+104E1 */
  0x0000,   /* U+104E2 */
  0x0000,   /* U+104E3 */
  0x0000,   /* U+104E4 */
  0x0000,   /* U+104E5 */
  0x0000,   /* U+104E6 */
  0x0000,   /* U+104E7 */
  0x0000,   /* U+104E8 */
  0x0000,   /* U+104E9 */
  0x0000,   /* U+104EA */
  0x0000,   /* U+104EB */
  0x0000,   /* U+104EC */
  0x0000,   /* U+104ED */
  0x0000,   /* U+104EE */
  0x0000,   /* U+104EF */
  0x0000,   /* U+104F0 */
  0x0000,   /* U+104F1 */
  0x0000,   /* U+104F2 */
  0x0000,   /* U+104F3 */
  0x0000,   /* U+104F4 */
  0x0000,   /* U+104F5 */
  0x0000,   /* U+104F6 */
  0x0000,   /* U+104F7 */
  0x0000,   /* U+104F8 */
  0x0000,   /* U+104F9 */
  0x0000,   /* U+104FA */
  0x0000,   /* U+104FB */
  0x0000,   /* U+104FC */
  0x0000,   /* U+104FD */
  0x0000,   /* U+104FE */
  0x0000,   /* U+104FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10400 */
  0x0000,   /* U+10401 */
  0x0000,   /* U+10402 */
  0x0000,   /* U+10403 */
  0x0000,   /* U+10404 */
  0x0000,   /* U+10405 */
  0x0000,   /* U+10406 */
  0x0000,   /* U+10407 */
  0x0000,   /* U+10408 */
  0x0000,   /* U+10409 */
  0x0000,   /* U+1040A */
  0x0000,   /* U+1040B */
  0x0000,   /* U+1040C */
  0x0000,   /* U+1040D */
  0x0000,   /* U+1040E */
  0x0000,   /* U+1040F */
  0x0000,   /* U+10410 */
  0x0000,   /* U+10411 */
  0x0000,   /* U+10412 */
  0x0000,   /* U+10413 */
  0x0000,   /* U+10414 */
  0x0000,   /* U+10415 */
  0x0000,   /* U+10416 */
  0x0000,   /* U+10417 */
  0x0000,   /* U+10418 */
  0x0000,   /* U+10419 */
  0x0000,   /* U+1041A */
  0x0000,   /* U+1041B */
  0x0000,   /* U+1041C */
  0x0000,   /* U+1041D */
  0x0000,   /* U+1041E */
  0x0000,   /* U+1041F */
  0x0000,   /* U+10420 */
  0x0000,   /* U+10421 */
  0x0000,   /* U+10422 */
  0x0000,   /* U+10423 */
  0x0000,   /* U+10424 */
  0x0000,   /* U+10425 */
  0x0000,   /* U+10426 */
  0x0000,   /* U+10427 */
  0x0000,   /* U+10428 */
  0x0000,   /* U+10429 */
  0x0000,   /* U+1042A */
  0x0000,   /* U+1042B */
  0x0000,   /* U+1042C */
  0x0000,   /* U+1042D */
  0x0000,   /* U+1042E */
  0x0000,   /* U+1042F */
  0x0000,   /* U+10430 */
  0x0000,   /* U+10431 */
  0x0000,   /* U+10432 */
  0x0000,   /* U+10433 */
  0x0000,   /* U+10434 */
  0x0000,   /* U+10435 */
  0x0000,   /* U+10436 */
  0x0000,   /* U+10437 */
  0x0000,   /* U+10438 */
  0x0000,   /* U+10439 */
  0x0000,   /* U+1043A */
  0x0000,   /* U+1043B */
  0x0000,   /* U+1043C */
  0x0000,   /* U+1043D */
  0x0000,   /* U+1043E */
  0x0000,   /* U+1043F */
  0x0000,   /* U+10440 */
  0x0000,   /* U+10441 */
  0x0000,   /* U+10442 */
  0x0000,   /* U+10443 */
  0x0000,   /* U+10444 */
  0x0000,   /* U+10445 */
  0x0000,   /* U+10446 */
  0x0000,   /* U+10447 */
  0x0000,   /* U+10448 */
  0x0000,   /* U+10449 */
  0x0000,   /* U+1044A */
  0x0000,   /* U+1044B */
  0x0000,   /* U+1044C */
  0x0000,   /* U+1044D */
  0x0000,   /* U+1044E */
  0x0000,   /* U+1044F */
  0x0000,   /* U+10450 */
  0x0000,   /* U+10451 */
  0x0000,   /* U+10452 */
  0x0000,   /* U+10453 */
  0x0000,   /* U+10454 */
  0x0000,   /* U+10455 */
  0x0000,   /* U+10456 */
  0x0000,   /* U+10457 */
  0x0000,   /* U+10458 */
  0x0000,   /* U+10459 */
  0x0000,   /* U+1045A */
  0x0000,   /* U+1045B */
  0x0000,   /* U+1045C */
  0x0000,   /* U+1045D */
  0x0000,   /* U+1045E */
  0x0000,   /* U+1045F */
  0x0000,   /* U+10460 */
  0x0000,   /* U+10461 */
  0x0000,   /* U+10462 */
  0x0000,   /* U+10463 */
  0x0000,   /* U+10464 */
  0x0000,   /* U+10465 */
  0x0000,   /* U+10466 */
  0x0000,   /* U+10467 */
  0x0000,   /* U+10468 */
  0x0000,   /* U+10469 */
  0x0000,   /* U+1046A */
  0x0000,   /* U+1046B */
  0x0000,   /* U+1046C */
  0x0000,   /* U+1046D */
  0x0000,   /* U+1046E */
  0x0000,   /* U+1046F */
  0x0000,   /* U+10470 */
  0x0000,   /* U+10471 */
  0x0000,   /* U+10472 */
  0x0000,   /* U+10473 */
  0x0000,   /* U+10474 */
  0x0000,   /* U+10475 */
  0x0000,   /* U+10476 */
  0x0000,   /* U+10477 */
  0x0000,   /* U+10478 */
  0x0000,   /* U+10479 */
  0x0000,   /* U+1047A */
  0x0000,   /* U+1047B */
  0x0000,   /* U+1047C */
  0x0000,   /* U+1047D */
  0x0000,   /* U+1047E */
  0x0000,   /* U+1047F */
  0x0000,   /* U+10480 */
  0x0000,   /* U+10481 */
  0x0000,   /* U+10482 */
  0x0000,   /* U+10483 */
  0x0000,   /* U+10484 */
  0x0000,   /* U+10485 */
  0x0000,   /* U+10486 */
  0x0000,   /* U+10487 */
  0x0000,   /* U+10488 */
  0x0000,   /* U+10489 */
  0x0000,   /* U+1048A */
  0x0000,   /* U+1048B */
  0x0000,   /* U+1048C */
  0x0000,   /* U+1048D */
  0x0000,   /* U+1048E */
  0x0000,   /* U+1048F */
  0x0000,   /* U+10490 */
  0x0000,   /* U+10491 */
  0x0000,   /* U+10492 */
  0x0000,   /* U+10493 */
  0x0000,   /* U+10494 */
  0x0000,   /* U+10495 */
  0x0000,   /* U+10496 */
  0x0000,   /* U+10497 */
  0x0000,   /* U+10498 */
  0x0000,   /* U+10499 */
  0x0000,   /* U+1049A */
  0x0000,   /* U+1049B */
  0x0000,   /* U+1049C */
  0x0000,   /* U+1049D */
  0x0000,   /* U+1049E */
  0x0000,   /* U+1049F */
  0x0000,   /* U+104A0 */
  0x0000,   /* U+104A1 */
  0x0000,   /* U+104A2 */
  0x0000,   /* U+104A3 */
  0x0000,   /* U+104A4 */
  0x0000,   /* U+104A5 */
  0x0000,   /* U+104A6 */
  0x0000,   /* U+104A7 */
  0x0000,   /* U+104A8 */
  0x0000,   /* U+104A9 */
  0x0000,   /* U+104AA */
  0x0000,   /* U+104AB */
  0x0000,   /* U+104AC */
  0x0000,   /* U+104AD */
  0x0000,   /* U+104AE */
  0x0000,   /* U+104AF */
  0x0000,   /* U+104B0 */
  0x0000,   /* U+104B1 */
  0x0000,   /* U+104B2 */
  0x0000,   /* U+104B3 */
  0x0000,   /* U+104B4 */
  0x0000,   /* U+104B5 */
  0x0000,   /* U+104B6 */
  0x0000,   /* U+104B7 */
  0x0000,   /* U+104B8 */
  0x0000,   /* U+104B9 */
  0x0000,   /* U+104BA */
  0x0000,   /* U+104BB */
  0x0000,   /* U+104BC */
  0x0000,   /* U+104BD */
  0x0000,   /* U+104BE */
  0x0000,   /* U+104BF */
  0x0000,   /* U+104C0 */
  0x0000,   /* U+104C1 */
  0x0000,   /* U+104C2 */
  0x0000,   /* U+104C3 */
  0x0000,   /* U+104C4 */
  0x0000,   /* U+104C5 */
  0x0000,   /* U+104C6 */
  0x0000,   /* U+104C7 */
  0x0000,   /* U+104C8 */
  0x0000,   /* U+104C9 */
  0x0000,   /* U+104CA */
  0x0000,   /* U+104CB */
  0x0000,   /* U+104CC */
  0x0000,   /* U+104CD */
  0x0000,   /* U+104CE */
  0x0000,   /* U+104CF */
  0x0000,   /* U+104D0 */
  0x0000,   /* U+104D1 */
  0x0000,   /* U+104D2 */
  0x0000,   /* U+104D3 */
  0x0000,   /* U+104D4 */
  0x0000,   /* U+104D5 */
  0x0000,   /* U+104D6 */
  0x0000,   /* U+104D7 */
  0x0000,   /* U+104D8 */
  0x0000,   /* U+104D9 */
  0x0000,   /* U+104DA */
  0x0000,   /* U+104DB */
  0x0000,   /* U+104DC */
  0x0000,   /* U+104DD */
  0x0000,   /* U+104DE */
  0x0000,   /* U+104DF */
  0x0000,   /* U+104E0 */
  0x0000,   /* U+104E1 */
  0x0000,   /* U+104E2 */
  0x0000,   /* U+104E3 */
  0x0000,   /* U+104E4 */
  0x0000,   /* U+104E5 */
  0x0000,   /* U+104E6 */
  0x0000,   /* U+104E7 */
  0x0000,   /* U+104E8 */
  0x0000,   /* U+104E9 */
  0x0000,   /* U+104EA */
  0x0000,   /* U+104EB */
  0x0000,   /* U+104EC */
  0x0000,   /* U+104ED */
  0x0000,   /* U+104EE */
  0x0000,   /* U+104EF */
  0x0000,   /* U+104F0 */
  0x0000,   /* U+104F1 */
  0x0000,   /* U+104F2 */
  0x0000,   /* U+104F3 */
  0x0000,   /* U+104F4 */
  0x0000,   /* U+104F5 */
  0x0000,   /* U+104F6 */
  0x0000,   /* U+104F7 */
  0x0000,   /* U+104F8 */
  0x0000,   /* U+104F9 */
  0x0000,   /* U+104FA */
  0x0000,   /* U+104FB */
  0x0000,   /* U+104FC */
  0x0000,   /* U+104FD */
  0x0000,   /* U+104FE */
  0x0000,   /* U+104FF */
};

uint16 uca900_p105[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4553,   /* U+10500 */
  0x4554,   /* U+10501 */
  0x4555,   /* U+10502 */
  0x4556,   /* U+10503 */
  0x4557,   /* U+10504 */
  0x4558,   /* U+10505 */
  0x4559,   /* U+10506 */
  0x455A,   /* U+10507 */
  0x455B,   /* U+10508 */
  0x455C,   /* U+10509 */
  0x455D,   /* U+1050A */
  0x455E,   /* U+1050B */
  0x455F,   /* U+1050C */
  0x4560,   /* U+1050D */
  0x4561,   /* U+1050E */
  0x4562,   /* U+1050F */
  0x4563,   /* U+10510 */
  0x4564,   /* U+10511 */
  0x4565,   /* U+10512 */
  0x4566,   /* U+10513 */
  0x4567,   /* U+10514 */
  0x4568,   /* U+10515 */
  0x4569,   /* U+10516 */
  0x456A,   /* U+10517 */
  0x456B,   /* U+10518 */
  0x456C,   /* U+10519 */
  0x456D,   /* U+1051A */
  0x456E,   /* U+1051B */
  0x456F,   /* U+1051C */
  0x4570,   /* U+1051D */
  0x4571,   /* U+1051E */
  0x4572,   /* U+1051F */
  0x4573,   /* U+10520 */
  0x4574,   /* U+10521 */
  0x4575,   /* U+10522 */
  0x4576,   /* U+10523 */
  0x4577,   /* U+10524 */
  0x4578,   /* U+10525 */
  0x4579,   /* U+10526 */
  0x457A,   /* U+10527 */
  0xFBC2,   /* U+10528 */
  0xFBC2,   /* U+10529 */
  0xFBC2,   /* U+1052A */
  0xFBC2,   /* U+1052B */
  0xFBC2,   /* U+1052C */
  0xFBC2,   /* U+1052D */
  0xFBC2,   /* U+1052E */
  0xFBC2,   /* U+1052F */
  0x457B,   /* U+10530 */
  0x457C,   /* U+10531 */
  0x457D,   /* U+10532 */
  0x457E,   /* U+10533 */
  0x457F,   /* U+10534 */
  0x4580,   /* U+10535 */
  0x4581,   /* U+10536 */
  0x4582,   /* U+10537 */
  0x4583,   /* U+10538 */
  0x4584,   /* U+10539 */
  0x4585,   /* U+1053A */
  0x4586,   /* U+1053B */
  0x4587,   /* U+1053C */
  0x4588,   /* U+1053D */
  0x4589,   /* U+1053E */
  0x458A,   /* U+1053F */
  0x458B,   /* U+10540 */
  0x458C,   /* U+10541 */
  0x458D,   /* U+10542 */
  0x458E,   /* U+10543 */
  0x458F,   /* U+10544 */
  0x4590,   /* U+10545 */
  0x4591,   /* U+10546 */
  0x4592,   /* U+10547 */
  0x4593,   /* U+10548 */
  0x4594,   /* U+10549 */
  0x4595,   /* U+1054A */
  0x4596,   /* U+1054B */
  0x4597,   /* U+1054C */
  0x4598,   /* U+1054D */
  0x4599,   /* U+1054E */
  0x459A,   /* U+1054F */
  0x459B,   /* U+10550 */
  0x459C,   /* U+10551 */
  0x459D,   /* U+10552 */
  0x459E,   /* U+10553 */
  0x459F,   /* U+10554 */
  0x45A0,   /* U+10555 */
  0x45A1,   /* U+10556 */
  0x45A2,   /* U+10557 */
  0x45A3,   /* U+10558 */
  0x45A4,   /* U+10559 */
  0x45A5,   /* U+1055A */
  0x45A6,   /* U+1055B */
  0x45A7,   /* U+1055C */
  0x45A8,   /* U+1055D */
  0x45A9,   /* U+1055E */
  0x45AA,   /* U+1055F */
  0x45AB,   /* U+10560 */
  0x45AC,   /* U+10561 */
  0x45AD,   /* U+10562 */
  0x45AE,   /* U+10563 */
  0xFBC2,   /* U+10564 */
  0xFBC2,   /* U+10565 */
  0xFBC2,   /* U+10566 */
  0xFBC2,   /* U+10567 */
  0xFBC2,   /* U+10568 */
  0xFBC2,   /* U+10569 */
  0xFBC2,   /* U+1056A */
  0xFBC2,   /* U+1056B */
  0xFBC2,   /* U+1056C */
  0xFBC2,   /* U+1056D */
  0xFBC2,   /* U+1056E */
  0x0432,   /* U+1056F */
  0xFBC2,   /* U+10570 */
  0xFBC2,   /* U+10571 */
  0xFBC2,   /* U+10572 */
  0xFBC2,   /* U+10573 */
  0xFBC2,   /* U+10574 */
  0xFBC2,   /* U+10575 */
  0xFBC2,   /* U+10576 */
  0xFBC2,   /* U+10577 */
  0xFBC2,   /* U+10578 */
  0xFBC2,   /* U+10579 */
  0xFBC2,   /* U+1057A */
  0xFBC2,   /* U+1057B */
  0xFBC2,   /* U+1057C */
  0xFBC2,   /* U+1057D */
  0xFBC2,   /* U+1057E */
  0xFBC2,   /* U+1057F */
  0xFBC2,   /* U+10580 */
  0xFBC2,   /* U+10581 */
  0xFBC2,   /* U+10582 */
  0xFBC2,   /* U+10583 */
  0xFBC2,   /* U+10584 */
  0xFBC2,   /* U+10585 */
  0xFBC2,   /* U+10586 */
  0xFBC2,   /* U+10587 */
  0xFBC2,   /* U+10588 */
  0xFBC2,   /* U+10589 */
  0xFBC2,   /* U+1058A */
  0xFBC2,   /* U+1058B */
  0xFBC2,   /* U+1058C */
  0xFBC2,   /* U+1058D */
  0xFBC2,   /* U+1058E */
  0xFBC2,   /* U+1058F */
  0xFBC2,   /* U+10590 */
  0xFBC2,   /* U+10591 */
  0xFBC2,   /* U+10592 */
  0xFBC2,   /* U+10593 */
  0xFBC2,   /* U+10594 */
  0xFBC2,   /* U+10595 */
  0xFBC2,   /* U+10596 */
  0xFBC2,   /* U+10597 */
  0xFBC2,   /* U+10598 */
  0xFBC2,   /* U+10599 */
  0xFBC2,   /* U+1059A */
  0xFBC2,   /* U+1059B */
  0xFBC2,   /* U+1059C */
  0xFBC2,   /* U+1059D */
  0xFBC2,   /* U+1059E */
  0xFBC2,   /* U+1059F */
  0xFBC2,   /* U+105A0 */
  0xFBC2,   /* U+105A1 */
  0xFBC2,   /* U+105A2 */
  0xFBC2,   /* U+105A3 */
  0xFBC2,   /* U+105A4 */
  0xFBC2,   /* U+105A5 */
  0xFBC2,   /* U+105A6 */
  0xFBC2,   /* U+105A7 */
  0xFBC2,   /* U+105A8 */
  0xFBC2,   /* U+105A9 */
  0xFBC2,   /* U+105AA */
  0xFBC2,   /* U+105AB */
  0xFBC2,   /* U+105AC */
  0xFBC2,   /* U+105AD */
  0xFBC2,   /* U+105AE */
  0xFBC2,   /* U+105AF */
  0xFBC2,   /* U+105B0 */
  0xFBC2,   /* U+105B1 */
  0xFBC2,   /* U+105B2 */
  0xFBC2,   /* U+105B3 */
  0xFBC2,   /* U+105B4 */
  0xFBC2,   /* U+105B5 */
  0xFBC2,   /* U+105B6 */
  0xFBC2,   /* U+105B7 */
  0xFBC2,   /* U+105B8 */
  0xFBC2,   /* U+105B9 */
  0xFBC2,   /* U+105BA */
  0xFBC2,   /* U+105BB */
  0xFBC2,   /* U+105BC */
  0xFBC2,   /* U+105BD */
  0xFBC2,   /* U+105BE */
  0xFBC2,   /* U+105BF */
  0xFBC2,   /* U+105C0 */
  0xFBC2,   /* U+105C1 */
  0xFBC2,   /* U+105C2 */
  0xFBC2,   /* U+105C3 */
  0xFBC2,   /* U+105C4 */
  0xFBC2,   /* U+105C5 */
  0xFBC2,   /* U+105C6 */
  0xFBC2,   /* U+105C7 */
  0xFBC2,   /* U+105C8 */
  0xFBC2,   /* U+105C9 */
  0xFBC2,   /* U+105CA */
  0xFBC2,   /* U+105CB */
  0xFBC2,   /* U+105CC */
  0xFBC2,   /* U+105CD */
  0xFBC2,   /* U+105CE */
  0xFBC2,   /* U+105CF */
  0xFBC2,   /* U+105D0 */
  0xFBC2,   /* U+105D1 */
  0xFBC2,   /* U+105D2 */
  0xFBC2,   /* U+105D3 */
  0xFBC2,   /* U+105D4 */
  0xFBC2,   /* U+105D5 */
  0xFBC2,   /* U+105D6 */
  0xFBC2,   /* U+105D7 */
  0xFBC2,   /* U+105D8 */
  0xFBC2,   /* U+105D9 */
  0xFBC2,   /* U+105DA */
  0xFBC2,   /* U+105DB */
  0xFBC2,   /* U+105DC */
  0xFBC2,   /* U+105DD */
  0xFBC2,   /* U+105DE */
  0xFBC2,   /* U+105DF */
  0xFBC2,   /* U+105E0 */
  0xFBC2,   /* U+105E1 */
  0xFBC2,   /* U+105E2 */
  0xFBC2,   /* U+105E3 */
  0xFBC2,   /* U+105E4 */
  0xFBC2,   /* U+105E5 */
  0xFBC2,   /* U+105E6 */
  0xFBC2,   /* U+105E7 */
  0xFBC2,   /* U+105E8 */
  0xFBC2,   /* U+105E9 */
  0xFBC2,   /* U+105EA */
  0xFBC2,   /* U+105EB */
  0xFBC2,   /* U+105EC */
  0xFBC2,   /* U+105ED */
  0xFBC2,   /* U+105EE */
  0xFBC2,   /* U+105EF */
  0xFBC2,   /* U+105F0 */
  0xFBC2,   /* U+105F1 */
  0xFBC2,   /* U+105F2 */
  0xFBC2,   /* U+105F3 */
  0xFBC2,   /* U+105F4 */
  0xFBC2,   /* U+105F5 */
  0xFBC2,   /* U+105F6 */
  0xFBC2,   /* U+105F7 */
  0xFBC2,   /* U+105F8 */
  0xFBC2,   /* U+105F9 */
  0xFBC2,   /* U+105FA */
  0xFBC2,   /* U+105FB */
  0xFBC2,   /* U+105FC */
  0xFBC2,   /* U+105FD */
  0xFBC2,   /* U+105FE */
  0xFBC2,   /* U+105FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10500 */
  0x0020,   /* U+10501 */
  0x0020,   /* U+10502 */
  0x0020,   /* U+10503 */
  0x0020,   /* U+10504 */
  0x0020,   /* U+10505 */
  0x0020,   /* U+10506 */
  0x0020,   /* U+10507 */
  0x0020,   /* U+10508 */
  0x0020,   /* U+10509 */
  0x0020,   /* U+1050A */
  0x0020,   /* U+1050B */
  0x0020,   /* U+1050C */
  0x0020,   /* U+1050D */
  0x0020,   /* U+1050E */
  0x0020,   /* U+1050F */
  0x0020,   /* U+10510 */
  0x0020,   /* U+10511 */
  0x0020,   /* U+10512 */
  0x0020,   /* U+10513 */
  0x0020,   /* U+10514 */
  0x0020,   /* U+10515 */
  0x0020,   /* U+10516 */
  0x0020,   /* U+10517 */
  0x0020,   /* U+10518 */
  0x0020,   /* U+10519 */
  0x0020,   /* U+1051A */
  0x0020,   /* U+1051B */
  0x0020,   /* U+1051C */
  0x0020,   /* U+1051D */
  0x0020,   /* U+1051E */
  0x0020,   /* U+1051F */
  0x0020,   /* U+10520 */
  0x0020,   /* U+10521 */
  0x0020,   /* U+10522 */
  0x0020,   /* U+10523 */
  0x0020,   /* U+10524 */
  0x0020,   /* U+10525 */
  0x0020,   /* U+10526 */
  0x0020,   /* U+10527 */
  0x0020,   /* U+10528 */
  0x0020,   /* U+10529 */
  0x0020,   /* U+1052A */
  0x0020,   /* U+1052B */
  0x0020,   /* U+1052C */
  0x0020,   /* U+1052D */
  0x0020,   /* U+1052E */
  0x0020,   /* U+1052F */
  0x0020,   /* U+10530 */
  0x0020,   /* U+10531 */
  0x0020,   /* U+10532 */
  0x0020,   /* U+10533 */
  0x0020,   /* U+10534 */
  0x0020,   /* U+10535 */
  0x0020,   /* U+10536 */
  0x0020,   /* U+10537 */
  0x0020,   /* U+10538 */
  0x0020,   /* U+10539 */
  0x0020,   /* U+1053A */
  0x0020,   /* U+1053B */
  0x0020,   /* U+1053C */
  0x0020,   /* U+1053D */
  0x0020,   /* U+1053E */
  0x0020,   /* U+1053F */
  0x0020,   /* U+10540 */
  0x0020,   /* U+10541 */
  0x0020,   /* U+10542 */
  0x0020,   /* U+10543 */
  0x0020,   /* U+10544 */
  0x0020,   /* U+10545 */
  0x0020,   /* U+10546 */
  0x0020,   /* U+10547 */
  0x0020,   /* U+10548 */
  0x0020,   /* U+10549 */
  0x0020,   /* U+1054A */
  0x0020,   /* U+1054B */
  0x0020,   /* U+1054C */
  0x0020,   /* U+1054D */
  0x0020,   /* U+1054E */
  0x0020,   /* U+1054F */
  0x0020,   /* U+10550 */
  0x0020,   /* U+10551 */
  0x0020,   /* U+10552 */
  0x0020,   /* U+10553 */
  0x0020,   /* U+10554 */
  0x0020,   /* U+10555 */
  0x0020,   /* U+10556 */
  0x0020,   /* U+10557 */
  0x0020,   /* U+10558 */
  0x0020,   /* U+10559 */
  0x0020,   /* U+1055A */
  0x0020,   /* U+1055B */
  0x0020,   /* U+1055C */
  0x0020,   /* U+1055D */
  0x0020,   /* U+1055E */
  0x0020,   /* U+1055F */
  0x0020,   /* U+10560 */
  0x0020,   /* U+10561 */
  0x0020,   /* U+10562 */
  0x0020,   /* U+10563 */
  0x0020,   /* U+10564 */
  0x0020,   /* U+10565 */
  0x0020,   /* U+10566 */
  0x0020,   /* U+10567 */
  0x0020,   /* U+10568 */
  0x0020,   /* U+10569 */
  0x0020,   /* U+1056A */
  0x0020,   /* U+1056B */
  0x0020,   /* U+1056C */
  0x0020,   /* U+1056D */
  0x0020,   /* U+1056E */
  0x0020,   /* U+1056F */
  0x0020,   /* U+10570 */
  0x0020,   /* U+10571 */
  0x0020,   /* U+10572 */
  0x0020,   /* U+10573 */
  0x0020,   /* U+10574 */
  0x0020,   /* U+10575 */
  0x0020,   /* U+10576 */
  0x0020,   /* U+10577 */
  0x0020,   /* U+10578 */
  0x0020,   /* U+10579 */
  0x0020,   /* U+1057A */
  0x0020,   /* U+1057B */
  0x0020,   /* U+1057C */
  0x0020,   /* U+1057D */
  0x0020,   /* U+1057E */
  0x0020,   /* U+1057F */
  0x0020,   /* U+10580 */
  0x0020,   /* U+10581 */
  0x0020,   /* U+10582 */
  0x0020,   /* U+10583 */
  0x0020,   /* U+10584 */
  0x0020,   /* U+10585 */
  0x0020,   /* U+10586 */
  0x0020,   /* U+10587 */
  0x0020,   /* U+10588 */
  0x0020,   /* U+10589 */
  0x0020,   /* U+1058A */
  0x0020,   /* U+1058B */
  0x0020,   /* U+1058C */
  0x0020,   /* U+1058D */
  0x0020,   /* U+1058E */
  0x0020,   /* U+1058F */
  0x0020,   /* U+10590 */
  0x0020,   /* U+10591 */
  0x0020,   /* U+10592 */
  0x0020,   /* U+10593 */
  0x0020,   /* U+10594 */
  0x0020,   /* U+10595 */
  0x0020,   /* U+10596 */
  0x0020,   /* U+10597 */
  0x0020,   /* U+10598 */
  0x0020,   /* U+10599 */
  0x0020,   /* U+1059A */
  0x0020,   /* U+1059B */
  0x0020,   /* U+1059C */
  0x0020,   /* U+1059D */
  0x0020,   /* U+1059E */
  0x0020,   /* U+1059F */
  0x0020,   /* U+105A0 */
  0x0020,   /* U+105A1 */
  0x0020,   /* U+105A2 */
  0x0020,   /* U+105A3 */
  0x0020,   /* U+105A4 */
  0x0020,   /* U+105A5 */
  0x0020,   /* U+105A6 */
  0x0020,   /* U+105A7 */
  0x0020,   /* U+105A8 */
  0x0020,   /* U+105A9 */
  0x0020,   /* U+105AA */
  0x0020,   /* U+105AB */
  0x0020,   /* U+105AC */
  0x0020,   /* U+105AD */
  0x0020,   /* U+105AE */
  0x0020,   /* U+105AF */
  0x0020,   /* U+105B0 */
  0x0020,   /* U+105B1 */
  0x0020,   /* U+105B2 */
  0x0020,   /* U+105B3 */
  0x0020,   /* U+105B4 */
  0x0020,   /* U+105B5 */
  0x0020,   /* U+105B6 */
  0x0020,   /* U+105B7 */
  0x0020,   /* U+105B8 */
  0x0020,   /* U+105B9 */
  0x0020,   /* U+105BA */
  0x0020,   /* U+105BB */
  0x0020,   /* U+105BC */
  0x0020,   /* U+105BD */
  0x0020,   /* U+105BE */
  0x0020,   /* U+105BF */
  0x0020,   /* U+105C0 */
  0x0020,   /* U+105C1 */
  0x0020,   /* U+105C2 */
  0x0020,   /* U+105C3 */
  0x0020,   /* U+105C4 */
  0x0020,   /* U+105C5 */
  0x0020,   /* U+105C6 */
  0x0020,   /* U+105C7 */
  0x0020,   /* U+105C8 */
  0x0020,   /* U+105C9 */
  0x0020,   /* U+105CA */
  0x0020,   /* U+105CB */
  0x0020,   /* U+105CC */
  0x0020,   /* U+105CD */
  0x0020,   /* U+105CE */
  0x0020,   /* U+105CF */
  0x0020,   /* U+105D0 */
  0x0020,   /* U+105D1 */
  0x0020,   /* U+105D2 */
  0x0020,   /* U+105D3 */
  0x0020,   /* U+105D4 */
  0x0020,   /* U+105D5 */
  0x0020,   /* U+105D6 */
  0x0020,   /* U+105D7 */
  0x0020,   /* U+105D8 */
  0x0020,   /* U+105D9 */
  0x0020,   /* U+105DA */
  0x0020,   /* U+105DB */
  0x0020,   /* U+105DC */
  0x0020,   /* U+105DD */
  0x0020,   /* U+105DE */
  0x0020,   /* U+105DF */
  0x0020,   /* U+105E0 */
  0x0020,   /* U+105E1 */
  0x0020,   /* U+105E2 */
  0x0020,   /* U+105E3 */
  0x0020,   /* U+105E4 */
  0x0020,   /* U+105E5 */
  0x0020,   /* U+105E6 */
  0x0020,   /* U+105E7 */
  0x0020,   /* U+105E8 */
  0x0020,   /* U+105E9 */
  0x0020,   /* U+105EA */
  0x0020,   /* U+105EB */
  0x0020,   /* U+105EC */
  0x0020,   /* U+105ED */
  0x0020,   /* U+105EE */
  0x0020,   /* U+105EF */
  0x0020,   /* U+105F0 */
  0x0020,   /* U+105F1 */
  0x0020,   /* U+105F2 */
  0x0020,   /* U+105F3 */
  0x0020,   /* U+105F4 */
  0x0020,   /* U+105F5 */
  0x0020,   /* U+105F6 */
  0x0020,   /* U+105F7 */
  0x0020,   /* U+105F8 */
  0x0020,   /* U+105F9 */
  0x0020,   /* U+105FA */
  0x0020,   /* U+105FB */
  0x0020,   /* U+105FC */
  0x0020,   /* U+105FD */
  0x0020,   /* U+105FE */
  0x0020,   /* U+105FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10500 */
  0x0002,   /* U+10501 */
  0x0002,   /* U+10502 */
  0x0002,   /* U+10503 */
  0x0002,   /* U+10504 */
  0x0002,   /* U+10505 */
  0x0002,   /* U+10506 */
  0x0002,   /* U+10507 */
  0x0002,   /* U+10508 */
  0x0002,   /* U+10509 */
  0x0002,   /* U+1050A */
  0x0002,   /* U+1050B */
  0x0002,   /* U+1050C */
  0x0002,   /* U+1050D */
  0x0002,   /* U+1050E */
  0x0002,   /* U+1050F */
  0x0002,   /* U+10510 */
  0x0002,   /* U+10511 */
  0x0002,   /* U+10512 */
  0x0002,   /* U+10513 */
  0x0002,   /* U+10514 */
  0x0002,   /* U+10515 */
  0x0002,   /* U+10516 */
  0x0002,   /* U+10517 */
  0x0002,   /* U+10518 */
  0x0002,   /* U+10519 */
  0x0002,   /* U+1051A */
  0x0002,   /* U+1051B */
  0x0002,   /* U+1051C */
  0x0002,   /* U+1051D */
  0x0002,   /* U+1051E */
  0x0002,   /* U+1051F */
  0x0002,   /* U+10520 */
  0x0002,   /* U+10521 */
  0x0002,   /* U+10522 */
  0x0002,   /* U+10523 */
  0x0002,   /* U+10524 */
  0x0002,   /* U+10525 */
  0x0002,   /* U+10526 */
  0x0002,   /* U+10527 */
  0x0002,   /* U+10528 */
  0x0002,   /* U+10529 */
  0x0002,   /* U+1052A */
  0x0002,   /* U+1052B */
  0x0002,   /* U+1052C */
  0x0002,   /* U+1052D */
  0x0002,   /* U+1052E */
  0x0002,   /* U+1052F */
  0x0002,   /* U+10530 */
  0x0002,   /* U+10531 */
  0x0002,   /* U+10532 */
  0x0002,   /* U+10533 */
  0x0002,   /* U+10534 */
  0x0002,   /* U+10535 */
  0x0002,   /* U+10536 */
  0x0002,   /* U+10537 */
  0x0002,   /* U+10538 */
  0x0002,   /* U+10539 */
  0x0002,   /* U+1053A */
  0x0002,   /* U+1053B */
  0x0002,   /* U+1053C */
  0x0002,   /* U+1053D */
  0x0002,   /* U+1053E */
  0x0002,   /* U+1053F */
  0x0002,   /* U+10540 */
  0x0002,   /* U+10541 */
  0x0002,   /* U+10542 */
  0x0002,   /* U+10543 */
  0x0002,   /* U+10544 */
  0x0002,   /* U+10545 */
  0x0002,   /* U+10546 */
  0x0002,   /* U+10547 */
  0x0002,   /* U+10548 */
  0x0002,   /* U+10549 */
  0x0002,   /* U+1054A */
  0x0002,   /* U+1054B */
  0x0002,   /* U+1054C */
  0x0002,   /* U+1054D */
  0x0002,   /* U+1054E */
  0x0002,   /* U+1054F */
  0x0002,   /* U+10550 */
  0x0002,   /* U+10551 */
  0x0002,   /* U+10552 */
  0x0002,   /* U+10553 */
  0x0002,   /* U+10554 */
  0x0002,   /* U+10555 */
  0x0002,   /* U+10556 */
  0x0002,   /* U+10557 */
  0x0002,   /* U+10558 */
  0x0002,   /* U+10559 */
  0x0002,   /* U+1055A */
  0x0002,   /* U+1055B */
  0x0002,   /* U+1055C */
  0x0002,   /* U+1055D */
  0x0002,   /* U+1055E */
  0x0002,   /* U+1055F */
  0x0002,   /* U+10560 */
  0x0002,   /* U+10561 */
  0x0002,   /* U+10562 */
  0x0002,   /* U+10563 */
  0x0002,   /* U+10564 */
  0x0002,   /* U+10565 */
  0x0002,   /* U+10566 */
  0x0002,   /* U+10567 */
  0x0002,   /* U+10568 */
  0x0002,   /* U+10569 */
  0x0002,   /* U+1056A */
  0x0002,   /* U+1056B */
  0x0002,   /* U+1056C */
  0x0002,   /* U+1056D */
  0x0002,   /* U+1056E */
  0x0002,   /* U+1056F */
  0x0002,   /* U+10570 */
  0x0002,   /* U+10571 */
  0x0002,   /* U+10572 */
  0x0002,   /* U+10573 */
  0x0002,   /* U+10574 */
  0x0002,   /* U+10575 */
  0x0002,   /* U+10576 */
  0x0002,   /* U+10577 */
  0x0002,   /* U+10578 */
  0x0002,   /* U+10579 */
  0x0002,   /* U+1057A */
  0x0002,   /* U+1057B */
  0x0002,   /* U+1057C */
  0x0002,   /* U+1057D */
  0x0002,   /* U+1057E */
  0x0002,   /* U+1057F */
  0x0002,   /* U+10580 */
  0x0002,   /* U+10581 */
  0x0002,   /* U+10582 */
  0x0002,   /* U+10583 */
  0x0002,   /* U+10584 */
  0x0002,   /* U+10585 */
  0x0002,   /* U+10586 */
  0x0002,   /* U+10587 */
  0x0002,   /* U+10588 */
  0x0002,   /* U+10589 */
  0x0002,   /* U+1058A */
  0x0002,   /* U+1058B */
  0x0002,   /* U+1058C */
  0x0002,   /* U+1058D */
  0x0002,   /* U+1058E */
  0x0002,   /* U+1058F */
  0x0002,   /* U+10590 */
  0x0002,   /* U+10591 */
  0x0002,   /* U+10592 */
  0x0002,   /* U+10593 */
  0x0002,   /* U+10594 */
  0x0002,   /* U+10595 */
  0x0002,   /* U+10596 */
  0x0002,   /* U+10597 */
  0x0002,   /* U+10598 */
  0x0002,   /* U+10599 */
  0x0002,   /* U+1059A */
  0x0002,   /* U+1059B */
  0x0002,   /* U+1059C */
  0x0002,   /* U+1059D */
  0x0002,   /* U+1059E */
  0x0002,   /* U+1059F */
  0x0002,   /* U+105A0 */
  0x0002,   /* U+105A1 */
  0x0002,   /* U+105A2 */
  0x0002,   /* U+105A3 */
  0x0002,   /* U+105A4 */
  0x0002,   /* U+105A5 */
  0x0002,   /* U+105A6 */
  0x0002,   /* U+105A7 */
  0x0002,   /* U+105A8 */
  0x0002,   /* U+105A9 */
  0x0002,   /* U+105AA */
  0x0002,   /* U+105AB */
  0x0002,   /* U+105AC */
  0x0002,   /* U+105AD */
  0x0002,   /* U+105AE */
  0x0002,   /* U+105AF */
  0x0002,   /* U+105B0 */
  0x0002,   /* U+105B1 */
  0x0002,   /* U+105B2 */
  0x0002,   /* U+105B3 */
  0x0002,   /* U+105B4 */
  0x0002,   /* U+105B5 */
  0x0002,   /* U+105B6 */
  0x0002,   /* U+105B7 */
  0x0002,   /* U+105B8 */
  0x0002,   /* U+105B9 */
  0x0002,   /* U+105BA */
  0x0002,   /* U+105BB */
  0x0002,   /* U+105BC */
  0x0002,   /* U+105BD */
  0x0002,   /* U+105BE */
  0x0002,   /* U+105BF */
  0x0002,   /* U+105C0 */
  0x0002,   /* U+105C1 */
  0x0002,   /* U+105C2 */
  0x0002,   /* U+105C3 */
  0x0002,   /* U+105C4 */
  0x0002,   /* U+105C5 */
  0x0002,   /* U+105C6 */
  0x0002,   /* U+105C7 */
  0x0002,   /* U+105C8 */
  0x0002,   /* U+105C9 */
  0x0002,   /* U+105CA */
  0x0002,   /* U+105CB */
  0x0002,   /* U+105CC */
  0x0002,   /* U+105CD */
  0x0002,   /* U+105CE */
  0x0002,   /* U+105CF */
  0x0002,   /* U+105D0 */
  0x0002,   /* U+105D1 */
  0x0002,   /* U+105D2 */
  0x0002,   /* U+105D3 */
  0x0002,   /* U+105D4 */
  0x0002,   /* U+105D5 */
  0x0002,   /* U+105D6 */
  0x0002,   /* U+105D7 */
  0x0002,   /* U+105D8 */
  0x0002,   /* U+105D9 */
  0x0002,   /* U+105DA */
  0x0002,   /* U+105DB */
  0x0002,   /* U+105DC */
  0x0002,   /* U+105DD */
  0x0002,   /* U+105DE */
  0x0002,   /* U+105DF */
  0x0002,   /* U+105E0 */
  0x0002,   /* U+105E1 */
  0x0002,   /* U+105E2 */
  0x0002,   /* U+105E3 */
  0x0002,   /* U+105E4 */
  0x0002,   /* U+105E5 */
  0x0002,   /* U+105E6 */
  0x0002,   /* U+105E7 */
  0x0002,   /* U+105E8 */
  0x0002,   /* U+105E9 */
  0x0002,   /* U+105EA */
  0x0002,   /* U+105EB */
  0x0002,   /* U+105EC */
  0x0002,   /* U+105ED */
  0x0002,   /* U+105EE */
  0x0002,   /* U+105EF */
  0x0002,   /* U+105F0 */
  0x0002,   /* U+105F1 */
  0x0002,   /* U+105F2 */
  0x0002,   /* U+105F3 */
  0x0002,   /* U+105F4 */
  0x0002,   /* U+105F5 */
  0x0002,   /* U+105F6 */
  0x0002,   /* U+105F7 */
  0x0002,   /* U+105F8 */
  0x0002,   /* U+105F9 */
  0x0002,   /* U+105FA */
  0x0002,   /* U+105FB */
  0x0002,   /* U+105FC */
  0x0002,   /* U+105FD */
  0x0002,   /* U+105FE */
  0x0002,   /* U+105FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10500 */
  0x0000,   /* U+10501 */
  0x0000,   /* U+10502 */
  0x0000,   /* U+10503 */
  0x0000,   /* U+10504 */
  0x0000,   /* U+10505 */
  0x0000,   /* U+10506 */
  0x0000,   /* U+10507 */
  0x0000,   /* U+10508 */
  0x0000,   /* U+10509 */
  0x0000,   /* U+1050A */
  0x0000,   /* U+1050B */
  0x0000,   /* U+1050C */
  0x0000,   /* U+1050D */
  0x0000,   /* U+1050E */
  0x0000,   /* U+1050F */
  0x0000,   /* U+10510 */
  0x0000,   /* U+10511 */
  0x0000,   /* U+10512 */
  0x0000,   /* U+10513 */
  0x0000,   /* U+10514 */
  0x0000,   /* U+10515 */
  0x0000,   /* U+10516 */
  0x0000,   /* U+10517 */
  0x0000,   /* U+10518 */
  0x0000,   /* U+10519 */
  0x0000,   /* U+1051A */
  0x0000,   /* U+1051B */
  0x0000,   /* U+1051C */
  0x0000,   /* U+1051D */
  0x0000,   /* U+1051E */
  0x0000,   /* U+1051F */
  0x0000,   /* U+10520 */
  0x0000,   /* U+10521 */
  0x0000,   /* U+10522 */
  0x0000,   /* U+10523 */
  0x0000,   /* U+10524 */
  0x0000,   /* U+10525 */
  0x0000,   /* U+10526 */
  0x0000,   /* U+10527 */
  0x8528,   /* U+10528 */
  0x8529,   /* U+10529 */
  0x852A,   /* U+1052A */
  0x852B,   /* U+1052B */
  0x852C,   /* U+1052C */
  0x852D,   /* U+1052D */
  0x852E,   /* U+1052E */
  0x852F,   /* U+1052F */
  0x0000,   /* U+10530 */
  0x0000,   /* U+10531 */
  0x0000,   /* U+10532 */
  0x0000,   /* U+10533 */
  0x0000,   /* U+10534 */
  0x0000,   /* U+10535 */
  0x0000,   /* U+10536 */
  0x0000,   /* U+10537 */
  0x0000,   /* U+10538 */
  0x0000,   /* U+10539 */
  0x0000,   /* U+1053A */
  0x0000,   /* U+1053B */
  0x0000,   /* U+1053C */
  0x0000,   /* U+1053D */
  0x0000,   /* U+1053E */
  0x0000,   /* U+1053F */
  0x0000,   /* U+10540 */
  0x0000,   /* U+10541 */
  0x0000,   /* U+10542 */
  0x0000,   /* U+10543 */
  0x0000,   /* U+10544 */
  0x0000,   /* U+10545 */
  0x0000,   /* U+10546 */
  0x0000,   /* U+10547 */
  0x0000,   /* U+10548 */
  0x0000,   /* U+10549 */
  0x0000,   /* U+1054A */
  0x0000,   /* U+1054B */
  0x0000,   /* U+1054C */
  0x0000,   /* U+1054D */
  0x0000,   /* U+1054E */
  0x0000,   /* U+1054F */
  0x0000,   /* U+10550 */
  0x0000,   /* U+10551 */
  0x0000,   /* U+10552 */
  0x0000,   /* U+10553 */
  0x0000,   /* U+10554 */
  0x0000,   /* U+10555 */
  0x0000,   /* U+10556 */
  0x0000,   /* U+10557 */
  0x0000,   /* U+10558 */
  0x0000,   /* U+10559 */
  0x0000,   /* U+1055A */
  0x0000,   /* U+1055B */
  0x0000,   /* U+1055C */
  0x0000,   /* U+1055D */
  0x0000,   /* U+1055E */
  0x0000,   /* U+1055F */
  0x0000,   /* U+10560 */
  0x0000,   /* U+10561 */
  0x0000,   /* U+10562 */
  0x0000,   /* U+10563 */
  0x8564,   /* U+10564 */
  0x8565,   /* U+10565 */
  0x8566,   /* U+10566 */
  0x8567,   /* U+10567 */
  0x8568,   /* U+10568 */
  0x8569,   /* U+10569 */
  0x856A,   /* U+1056A */
  0x856B,   /* U+1056B */
  0x856C,   /* U+1056C */
  0x856D,   /* U+1056D */
  0x856E,   /* U+1056E */
  0x0000,   /* U+1056F */
  0x8570,   /* U+10570 */
  0x8571,   /* U+10571 */
  0x8572,   /* U+10572 */
  0x8573,   /* U+10573 */
  0x8574,   /* U+10574 */
  0x8575,   /* U+10575 */
  0x8576,   /* U+10576 */
  0x8577,   /* U+10577 */
  0x8578,   /* U+10578 */
  0x8579,   /* U+10579 */
  0x857A,   /* U+1057A */
  0x857B,   /* U+1057B */
  0x857C,   /* U+1057C */
  0x857D,   /* U+1057D */
  0x857E,   /* U+1057E */
  0x857F,   /* U+1057F */
  0x8580,   /* U+10580 */
  0x8581,   /* U+10581 */
  0x8582,   /* U+10582 */
  0x8583,   /* U+10583 */
  0x8584,   /* U+10584 */
  0x8585,   /* U+10585 */
  0x8586,   /* U+10586 */
  0x8587,   /* U+10587 */
  0x8588,   /* U+10588 */
  0x8589,   /* U+10589 */
  0x858A,   /* U+1058A */
  0x858B,   /* U+1058B */
  0x858C,   /* U+1058C */
  0x858D,   /* U+1058D */
  0x858E,   /* U+1058E */
  0x858F,   /* U+1058F */
  0x8590,   /* U+10590 */
  0x8591,   /* U+10591 */
  0x8592,   /* U+10592 */
  0x8593,   /* U+10593 */
  0x8594,   /* U+10594 */
  0x8595,   /* U+10595 */
  0x8596,   /* U+10596 */
  0x8597,   /* U+10597 */
  0x8598,   /* U+10598 */
  0x8599,   /* U+10599 */
  0x859A,   /* U+1059A */
  0x859B,   /* U+1059B */
  0x859C,   /* U+1059C */
  0x859D,   /* U+1059D */
  0x859E,   /* U+1059E */
  0x859F,   /* U+1059F */
  0x85A0,   /* U+105A0 */
  0x85A1,   /* U+105A1 */
  0x85A2,   /* U+105A2 */
  0x85A3,   /* U+105A3 */
  0x85A4,   /* U+105A4 */
  0x85A5,   /* U+105A5 */
  0x85A6,   /* U+105A6 */
  0x85A7,   /* U+105A7 */
  0x85A8,   /* U+105A8 */
  0x85A9,   /* U+105A9 */
  0x85AA,   /* U+105AA */
  0x85AB,   /* U+105AB */
  0x85AC,   /* U+105AC */
  0x85AD,   /* U+105AD */
  0x85AE,   /* U+105AE */
  0x85AF,   /* U+105AF */
  0x85B0,   /* U+105B0 */
  0x85B1,   /* U+105B1 */
  0x85B2,   /* U+105B2 */
  0x85B3,   /* U+105B3 */
  0x85B4,   /* U+105B4 */
  0x85B5,   /* U+105B5 */
  0x85B6,   /* U+105B6 */
  0x85B7,   /* U+105B7 */
  0x85B8,   /* U+105B8 */
  0x85B9,   /* U+105B9 */
  0x85BA,   /* U+105BA */
  0x85BB,   /* U+105BB */
  0x85BC,   /* U+105BC */
  0x85BD,   /* U+105BD */
  0x85BE,   /* U+105BE */
  0x85BF,   /* U+105BF */
  0x85C0,   /* U+105C0 */
  0x85C1,   /* U+105C1 */
  0x85C2,   /* U+105C2 */
  0x85C3,   /* U+105C3 */
  0x85C4,   /* U+105C4 */
  0x85C5,   /* U+105C5 */
  0x85C6,   /* U+105C6 */
  0x85C7,   /* U+105C7 */
  0x85C8,   /* U+105C8 */
  0x85C9,   /* U+105C9 */
  0x85CA,   /* U+105CA */
  0x85CB,   /* U+105CB */
  0x85CC,   /* U+105CC */
  0x85CD,   /* U+105CD */
  0x85CE,   /* U+105CE */
  0x85CF,   /* U+105CF */
  0x85D0,   /* U+105D0 */
  0x85D1,   /* U+105D1 */
  0x85D2,   /* U+105D2 */
  0x85D3,   /* U+105D3 */
  0x85D4,   /* U+105D4 */
  0x85D5,   /* U+105D5 */
  0x85D6,   /* U+105D6 */
  0x85D7,   /* U+105D7 */
  0x85D8,   /* U+105D8 */
  0x85D9,   /* U+105D9 */
  0x85DA,   /* U+105DA */
  0x85DB,   /* U+105DB */
  0x85DC,   /* U+105DC */
  0x85DD,   /* U+105DD */
  0x85DE,   /* U+105DE */
  0x85DF,   /* U+105DF */
  0x85E0,   /* U+105E0 */
  0x85E1,   /* U+105E1 */
  0x85E2,   /* U+105E2 */
  0x85E3,   /* U+105E3 */
  0x85E4,   /* U+105E4 */
  0x85E5,   /* U+105E5 */
  0x85E6,   /* U+105E6 */
  0x85E7,   /* U+105E7 */
  0x85E8,   /* U+105E8 */
  0x85E9,   /* U+105E9 */
  0x85EA,   /* U+105EA */
  0x85EB,   /* U+105EB */
  0x85EC,   /* U+105EC */
  0x85ED,   /* U+105ED */
  0x85EE,   /* U+105EE */
  0x85EF,   /* U+105EF */
  0x85F0,   /* U+105F0 */
  0x85F1,   /* U+105F1 */
  0x85F2,   /* U+105F2 */
  0x85F3,   /* U+105F3 */
  0x85F4,   /* U+105F4 */
  0x85F5,   /* U+105F5 */
  0x85F6,   /* U+105F6 */
  0x85F7,   /* U+105F7 */
  0x85F8,   /* U+105F8 */
  0x85F9,   /* U+105F9 */
  0x85FA,   /* U+105FA */
  0x85FB,   /* U+105FB */
  0x85FC,   /* U+105FC */
  0x85FD,   /* U+105FD */
  0x85FE,   /* U+105FE */
  0x85FF,   /* U+105FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10500 */
  0x0000,   /* U+10501 */
  0x0000,   /* U+10502 */
  0x0000,   /* U+10503 */
  0x0000,   /* U+10504 */
  0x0000,   /* U+10505 */
  0x0000,   /* U+10506 */
  0x0000,   /* U+10507 */
  0x0000,   /* U+10508 */
  0x0000,   /* U+10509 */
  0x0000,   /* U+1050A */
  0x0000,   /* U+1050B */
  0x0000,   /* U+1050C */
  0x0000,   /* U+1050D */
  0x0000,   /* U+1050E */
  0x0000,   /* U+1050F */
  0x0000,   /* U+10510 */
  0x0000,   /* U+10511 */
  0x0000,   /* U+10512 */
  0x0000,   /* U+10513 */
  0x0000,   /* U+10514 */
  0x0000,   /* U+10515 */
  0x0000,   /* U+10516 */
  0x0000,   /* U+10517 */
  0x0000,   /* U+10518 */
  0x0000,   /* U+10519 */
  0x0000,   /* U+1051A */
  0x0000,   /* U+1051B */
  0x0000,   /* U+1051C */
  0x0000,   /* U+1051D */
  0x0000,   /* U+1051E */
  0x0000,   /* U+1051F */
  0x0000,   /* U+10520 */
  0x0000,   /* U+10521 */
  0x0000,   /* U+10522 */
  0x0000,   /* U+10523 */
  0x0000,   /* U+10524 */
  0x0000,   /* U+10525 */
  0x0000,   /* U+10526 */
  0x0000,   /* U+10527 */
  0x0000,   /* U+10528 */
  0x0000,   /* U+10529 */
  0x0000,   /* U+1052A */
  0x0000,   /* U+1052B */
  0x0000,   /* U+1052C */
  0x0000,   /* U+1052D */
  0x0000,   /* U+1052E */
  0x0000,   /* U+1052F */
  0x0000,   /* U+10530 */
  0x0000,   /* U+10531 */
  0x0000,   /* U+10532 */
  0x0000,   /* U+10533 */
  0x0000,   /* U+10534 */
  0x0000,   /* U+10535 */
  0x0000,   /* U+10536 */
  0x0000,   /* U+10537 */
  0x0000,   /* U+10538 */
  0x0000,   /* U+10539 */
  0x0000,   /* U+1053A */
  0x0000,   /* U+1053B */
  0x0000,   /* U+1053C */
  0x0000,   /* U+1053D */
  0x0000,   /* U+1053E */
  0x0000,   /* U+1053F */
  0x0000,   /* U+10540 */
  0x0000,   /* U+10541 */
  0x0000,   /* U+10542 */
  0x0000,   /* U+10543 */
  0x0000,   /* U+10544 */
  0x0000,   /* U+10545 */
  0x0000,   /* U+10546 */
  0x0000,   /* U+10547 */
  0x0000,   /* U+10548 */
  0x0000,   /* U+10549 */
  0x0000,   /* U+1054A */
  0x0000,   /* U+1054B */
  0x0000,   /* U+1054C */
  0x0000,   /* U+1054D */
  0x0000,   /* U+1054E */
  0x0000,   /* U+1054F */
  0x0000,   /* U+10550 */
  0x0000,   /* U+10551 */
  0x0000,   /* U+10552 */
  0x0000,   /* U+10553 */
  0x0000,   /* U+10554 */
  0x0000,   /* U+10555 */
  0x0000,   /* U+10556 */
  0x0000,   /* U+10557 */
  0x0000,   /* U+10558 */
  0x0000,   /* U+10559 */
  0x0000,   /* U+1055A */
  0x0000,   /* U+1055B */
  0x0000,   /* U+1055C */
  0x0000,   /* U+1055D */
  0x0000,   /* U+1055E */
  0x0000,   /* U+1055F */
  0x0000,   /* U+10560 */
  0x0000,   /* U+10561 */
  0x0000,   /* U+10562 */
  0x0000,   /* U+10563 */
  0x0000,   /* U+10564 */
  0x0000,   /* U+10565 */
  0x0000,   /* U+10566 */
  0x0000,   /* U+10567 */
  0x0000,   /* U+10568 */
  0x0000,   /* U+10569 */
  0x0000,   /* U+1056A */
  0x0000,   /* U+1056B */
  0x0000,   /* U+1056C */
  0x0000,   /* U+1056D */
  0x0000,   /* U+1056E */
  0x0000,   /* U+1056F */
  0x0000,   /* U+10570 */
  0x0000,   /* U+10571 */
  0x0000,   /* U+10572 */
  0x0000,   /* U+10573 */
  0x0000,   /* U+10574 */
  0x0000,   /* U+10575 */
  0x0000,   /* U+10576 */
  0x0000,   /* U+10577 */
  0x0000,   /* U+10578 */
  0x0000,   /* U+10579 */
  0x0000,   /* U+1057A */
  0x0000,   /* U+1057B */
  0x0000,   /* U+1057C */
  0x0000,   /* U+1057D */
  0x0000,   /* U+1057E */
  0x0000,   /* U+1057F */
  0x0000,   /* U+10580 */
  0x0000,   /* U+10581 */
  0x0000,   /* U+10582 */
  0x0000,   /* U+10583 */
  0x0000,   /* U+10584 */
  0x0000,   /* U+10585 */
  0x0000,   /* U+10586 */
  0x0000,   /* U+10587 */
  0x0000,   /* U+10588 */
  0x0000,   /* U+10589 */
  0x0000,   /* U+1058A */
  0x0000,   /* U+1058B */
  0x0000,   /* U+1058C */
  0x0000,   /* U+1058D */
  0x0000,   /* U+1058E */
  0x0000,   /* U+1058F */
  0x0000,   /* U+10590 */
  0x0000,   /* U+10591 */
  0x0000,   /* U+10592 */
  0x0000,   /* U+10593 */
  0x0000,   /* U+10594 */
  0x0000,   /* U+10595 */
  0x0000,   /* U+10596 */
  0x0000,   /* U+10597 */
  0x0000,   /* U+10598 */
  0x0000,   /* U+10599 */
  0x0000,   /* U+1059A */
  0x0000,   /* U+1059B */
  0x0000,   /* U+1059C */
  0x0000,   /* U+1059D */
  0x0000,   /* U+1059E */
  0x0000,   /* U+1059F */
  0x0000,   /* U+105A0 */
  0x0000,   /* U+105A1 */
  0x0000,   /* U+105A2 */
  0x0000,   /* U+105A3 */
  0x0000,   /* U+105A4 */
  0x0000,   /* U+105A5 */
  0x0000,   /* U+105A6 */
  0x0000,   /* U+105A7 */
  0x0000,   /* U+105A8 */
  0x0000,   /* U+105A9 */
  0x0000,   /* U+105AA */
  0x0000,   /* U+105AB */
  0x0000,   /* U+105AC */
  0x0000,   /* U+105AD */
  0x0000,   /* U+105AE */
  0x0000,   /* U+105AF */
  0x0000,   /* U+105B0 */
  0x0000,   /* U+105B1 */
  0x0000,   /* U+105B2 */
  0x0000,   /* U+105B3 */
  0x0000,   /* U+105B4 */
  0x0000,   /* U+105B5 */
  0x0000,   /* U+105B6 */
  0x0000,   /* U+105B7 */
  0x0000,   /* U+105B8 */
  0x0000,   /* U+105B9 */
  0x0000,   /* U+105BA */
  0x0000,   /* U+105BB */
  0x0000,   /* U+105BC */
  0x0000,   /* U+105BD */
  0x0000,   /* U+105BE */
  0x0000,   /* U+105BF */
  0x0000,   /* U+105C0 */
  0x0000,   /* U+105C1 */
  0x0000,   /* U+105C2 */
  0x0000,   /* U+105C3 */
  0x0000,   /* U+105C4 */
  0x0000,   /* U+105C5 */
  0x0000,   /* U+105C6 */
  0x0000,   /* U+105C7 */
  0x0000,   /* U+105C8 */
  0x0000,   /* U+105C9 */
  0x0000,   /* U+105CA */
  0x0000,   /* U+105CB */
  0x0000,   /* U+105CC */
  0x0000,   /* U+105CD */
  0x0000,   /* U+105CE */
  0x0000,   /* U+105CF */
  0x0000,   /* U+105D0 */
  0x0000,   /* U+105D1 */
  0x0000,   /* U+105D2 */
  0x0000,   /* U+105D3 */
  0x0000,   /* U+105D4 */
  0x0000,   /* U+105D5 */
  0x0000,   /* U+105D6 */
  0x0000,   /* U+105D7 */
  0x0000,   /* U+105D8 */
  0x0000,   /* U+105D9 */
  0x0000,   /* U+105DA */
  0x0000,   /* U+105DB */
  0x0000,   /* U+105DC */
  0x0000,   /* U+105DD */
  0x0000,   /* U+105DE */
  0x0000,   /* U+105DF */
  0x0000,   /* U+105E0 */
  0x0000,   /* U+105E1 */
  0x0000,   /* U+105E2 */
  0x0000,   /* U+105E3 */
  0x0000,   /* U+105E4 */
  0x0000,   /* U+105E5 */
  0x0000,   /* U+105E6 */
  0x0000,   /* U+105E7 */
  0x0000,   /* U+105E8 */
  0x0000,   /* U+105E9 */
  0x0000,   /* U+105EA */
  0x0000,   /* U+105EB */
  0x0000,   /* U+105EC */
  0x0000,   /* U+105ED */
  0x0000,   /* U+105EE */
  0x0000,   /* U+105EF */
  0x0000,   /* U+105F0 */
  0x0000,   /* U+105F1 */
  0x0000,   /* U+105F2 */
  0x0000,   /* U+105F3 */
  0x0000,   /* U+105F4 */
  0x0000,   /* U+105F5 */
  0x0000,   /* U+105F6 */
  0x0000,   /* U+105F7 */
  0x0000,   /* U+105F8 */
  0x0000,   /* U+105F9 */
  0x0000,   /* U+105FA */
  0x0000,   /* U+105FB */
  0x0000,   /* U+105FC */
  0x0000,   /* U+105FD */
  0x0000,   /* U+105FE */
  0x0000,   /* U+105FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10500 */
  0x0000,   /* U+10501 */
  0x0000,   /* U+10502 */
  0x0000,   /* U+10503 */
  0x0000,   /* U+10504 */
  0x0000,   /* U+10505 */
  0x0000,   /* U+10506 */
  0x0000,   /* U+10507 */
  0x0000,   /* U+10508 */
  0x0000,   /* U+10509 */
  0x0000,   /* U+1050A */
  0x0000,   /* U+1050B */
  0x0000,   /* U+1050C */
  0x0000,   /* U+1050D */
  0x0000,   /* U+1050E */
  0x0000,   /* U+1050F */
  0x0000,   /* U+10510 */
  0x0000,   /* U+10511 */
  0x0000,   /* U+10512 */
  0x0000,   /* U+10513 */
  0x0000,   /* U+10514 */
  0x0000,   /* U+10515 */
  0x0000,   /* U+10516 */
  0x0000,   /* U+10517 */
  0x0000,   /* U+10518 */
  0x0000,   /* U+10519 */
  0x0000,   /* U+1051A */
  0x0000,   /* U+1051B */
  0x0000,   /* U+1051C */
  0x0000,   /* U+1051D */
  0x0000,   /* U+1051E */
  0x0000,   /* U+1051F */
  0x0000,   /* U+10520 */
  0x0000,   /* U+10521 */
  0x0000,   /* U+10522 */
  0x0000,   /* U+10523 */
  0x0000,   /* U+10524 */
  0x0000,   /* U+10525 */
  0x0000,   /* U+10526 */
  0x0000,   /* U+10527 */
  0x0000,   /* U+10528 */
  0x0000,   /* U+10529 */
  0x0000,   /* U+1052A */
  0x0000,   /* U+1052B */
  0x0000,   /* U+1052C */
  0x0000,   /* U+1052D */
  0x0000,   /* U+1052E */
  0x0000,   /* U+1052F */
  0x0000,   /* U+10530 */
  0x0000,   /* U+10531 */
  0x0000,   /* U+10532 */
  0x0000,   /* U+10533 */
  0x0000,   /* U+10534 */
  0x0000,   /* U+10535 */
  0x0000,   /* U+10536 */
  0x0000,   /* U+10537 */
  0x0000,   /* U+10538 */
  0x0000,   /* U+10539 */
  0x0000,   /* U+1053A */
  0x0000,   /* U+1053B */
  0x0000,   /* U+1053C */
  0x0000,   /* U+1053D */
  0x0000,   /* U+1053E */
  0x0000,   /* U+1053F */
  0x0000,   /* U+10540 */
  0x0000,   /* U+10541 */
  0x0000,   /* U+10542 */
  0x0000,   /* U+10543 */
  0x0000,   /* U+10544 */
  0x0000,   /* U+10545 */
  0x0000,   /* U+10546 */
  0x0000,   /* U+10547 */
  0x0000,   /* U+10548 */
  0x0000,   /* U+10549 */
  0x0000,   /* U+1054A */
  0x0000,   /* U+1054B */
  0x0000,   /* U+1054C */
  0x0000,   /* U+1054D */
  0x0000,   /* U+1054E */
  0x0000,   /* U+1054F */
  0x0000,   /* U+10550 */
  0x0000,   /* U+10551 */
  0x0000,   /* U+10552 */
  0x0000,   /* U+10553 */
  0x0000,   /* U+10554 */
  0x0000,   /* U+10555 */
  0x0000,   /* U+10556 */
  0x0000,   /* U+10557 */
  0x0000,   /* U+10558 */
  0x0000,   /* U+10559 */
  0x0000,   /* U+1055A */
  0x0000,   /* U+1055B */
  0x0000,   /* U+1055C */
  0x0000,   /* U+1055D */
  0x0000,   /* U+1055E */
  0x0000,   /* U+1055F */
  0x0000,   /* U+10560 */
  0x0000,   /* U+10561 */
  0x0000,   /* U+10562 */
  0x0000,   /* U+10563 */
  0x0000,   /* U+10564 */
  0x0000,   /* U+10565 */
  0x0000,   /* U+10566 */
  0x0000,   /* U+10567 */
  0x0000,   /* U+10568 */
  0x0000,   /* U+10569 */
  0x0000,   /* U+1056A */
  0x0000,   /* U+1056B */
  0x0000,   /* U+1056C */
  0x0000,   /* U+1056D */
  0x0000,   /* U+1056E */
  0x0000,   /* U+1056F */
  0x0000,   /* U+10570 */
  0x0000,   /* U+10571 */
  0x0000,   /* U+10572 */
  0x0000,   /* U+10573 */
  0x0000,   /* U+10574 */
  0x0000,   /* U+10575 */
  0x0000,   /* U+10576 */
  0x0000,   /* U+10577 */
  0x0000,   /* U+10578 */
  0x0000,   /* U+10579 */
  0x0000,   /* U+1057A */
  0x0000,   /* U+1057B */
  0x0000,   /* U+1057C */
  0x0000,   /* U+1057D */
  0x0000,   /* U+1057E */
  0x0000,   /* U+1057F */
  0x0000,   /* U+10580 */
  0x0000,   /* U+10581 */
  0x0000,   /* U+10582 */
  0x0000,   /* U+10583 */
  0x0000,   /* U+10584 */
  0x0000,   /* U+10585 */
  0x0000,   /* U+10586 */
  0x0000,   /* U+10587 */
  0x0000,   /* U+10588 */
  0x0000,   /* U+10589 */
  0x0000,   /* U+1058A */
  0x0000,   /* U+1058B */
  0x0000,   /* U+1058C */
  0x0000,   /* U+1058D */
  0x0000,   /* U+1058E */
  0x0000,   /* U+1058F */
  0x0000,   /* U+10590 */
  0x0000,   /* U+10591 */
  0x0000,   /* U+10592 */
  0x0000,   /* U+10593 */
  0x0000,   /* U+10594 */
  0x0000,   /* U+10595 */
  0x0000,   /* U+10596 */
  0x0000,   /* U+10597 */
  0x0000,   /* U+10598 */
  0x0000,   /* U+10599 */
  0x0000,   /* U+1059A */
  0x0000,   /* U+1059B */
  0x0000,   /* U+1059C */
  0x0000,   /* U+1059D */
  0x0000,   /* U+1059E */
  0x0000,   /* U+1059F */
  0x0000,   /* U+105A0 */
  0x0000,   /* U+105A1 */
  0x0000,   /* U+105A2 */
  0x0000,   /* U+105A3 */
  0x0000,   /* U+105A4 */
  0x0000,   /* U+105A5 */
  0x0000,   /* U+105A6 */
  0x0000,   /* U+105A7 */
  0x0000,   /* U+105A8 */
  0x0000,   /* U+105A9 */
  0x0000,   /* U+105AA */
  0x0000,   /* U+105AB */
  0x0000,   /* U+105AC */
  0x0000,   /* U+105AD */
  0x0000,   /* U+105AE */
  0x0000,   /* U+105AF */
  0x0000,   /* U+105B0 */
  0x0000,   /* U+105B1 */
  0x0000,   /* U+105B2 */
  0x0000,   /* U+105B3 */
  0x0000,   /* U+105B4 */
  0x0000,   /* U+105B5 */
  0x0000,   /* U+105B6 */
  0x0000,   /* U+105B7 */
  0x0000,   /* U+105B8 */
  0x0000,   /* U+105B9 */
  0x0000,   /* U+105BA */
  0x0000,   /* U+105BB */
  0x0000,   /* U+105BC */
  0x0000,   /* U+105BD */
  0x0000,   /* U+105BE */
  0x0000,   /* U+105BF */
  0x0000,   /* U+105C0 */
  0x0000,   /* U+105C1 */
  0x0000,   /* U+105C2 */
  0x0000,   /* U+105C3 */
  0x0000,   /* U+105C4 */
  0x0000,   /* U+105C5 */
  0x0000,   /* U+105C6 */
  0x0000,   /* U+105C7 */
  0x0000,   /* U+105C8 */
  0x0000,   /* U+105C9 */
  0x0000,   /* U+105CA */
  0x0000,   /* U+105CB */
  0x0000,   /* U+105CC */
  0x0000,   /* U+105CD */
  0x0000,   /* U+105CE */
  0x0000,   /* U+105CF */
  0x0000,   /* U+105D0 */
  0x0000,   /* U+105D1 */
  0x0000,   /* U+105D2 */
  0x0000,   /* U+105D3 */
  0x0000,   /* U+105D4 */
  0x0000,   /* U+105D5 */
  0x0000,   /* U+105D6 */
  0x0000,   /* U+105D7 */
  0x0000,   /* U+105D8 */
  0x0000,   /* U+105D9 */
  0x0000,   /* U+105DA */
  0x0000,   /* U+105DB */
  0x0000,   /* U+105DC */
  0x0000,   /* U+105DD */
  0x0000,   /* U+105DE */
  0x0000,   /* U+105DF */
  0x0000,   /* U+105E0 */
  0x0000,   /* U+105E1 */
  0x0000,   /* U+105E2 */
  0x0000,   /* U+105E3 */
  0x0000,   /* U+105E4 */
  0x0000,   /* U+105E5 */
  0x0000,   /* U+105E6 */
  0x0000,   /* U+105E7 */
  0x0000,   /* U+105E8 */
  0x0000,   /* U+105E9 */
  0x0000,   /* U+105EA */
  0x0000,   /* U+105EB */
  0x0000,   /* U+105EC */
  0x0000,   /* U+105ED */
  0x0000,   /* U+105EE */
  0x0000,   /* U+105EF */
  0x0000,   /* U+105F0 */
  0x0000,   /* U+105F1 */
  0x0000,   /* U+105F2 */
  0x0000,   /* U+105F3 */
  0x0000,   /* U+105F4 */
  0x0000,   /* U+105F5 */
  0x0000,   /* U+105F6 */
  0x0000,   /* U+105F7 */
  0x0000,   /* U+105F8 */
  0x0000,   /* U+105F9 */
  0x0000,   /* U+105FA */
  0x0000,   /* U+105FB */
  0x0000,   /* U+105FC */
  0x0000,   /* U+105FD */
  0x0000,   /* U+105FE */
  0x0000,   /* U+105FF */
};

uint16 uca900_p106[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x46BA,   /* U+10600 */
  0x46BB,   /* U+10601 */
  0x46BC,   /* U+10602 */
  0x46BD,   /* U+10603 */
  0x46BE,   /* U+10604 */
  0x46BF,   /* U+10605 */
  0x46C0,   /* U+10606 */
  0x46C1,   /* U+10607 */
  0x46C2,   /* U+10608 */
  0x46C3,   /* U+10609 */
  0x46C4,   /* U+1060A */
  0x46C5,   /* U+1060B */
  0x46C6,   /* U+1060C */
  0x46C7,   /* U+1060D */
  0x46C8,   /* U+1060E */
  0x46C9,   /* U+1060F */
  0x46CA,   /* U+10610 */
  0x46CB,   /* U+10611 */
  0x46CC,   /* U+10612 */
  0x46CD,   /* U+10613 */
  0x46CE,   /* U+10614 */
  0x46CF,   /* U+10615 */
  0x46D0,   /* U+10616 */
  0x46D1,   /* U+10617 */
  0x46D2,   /* U+10618 */
  0x46D3,   /* U+10619 */
  0x46D4,   /* U+1061A */
  0x46D5,   /* U+1061B */
  0x46D6,   /* U+1061C */
  0x46D7,   /* U+1061D */
  0x46D8,   /* U+1061E */
  0x46D9,   /* U+1061F */
  0x46DA,   /* U+10620 */
  0x46DB,   /* U+10621 */
  0x46DC,   /* U+10622 */
  0x46DD,   /* U+10623 */
  0x46DE,   /* U+10624 */
  0x46DF,   /* U+10625 */
  0x46E0,   /* U+10626 */
  0x46E1,   /* U+10627 */
  0x46E2,   /* U+10628 */
  0x46E3,   /* U+10629 */
  0x46E4,   /* U+1062A */
  0x46E5,   /* U+1062B */
  0x46E6,   /* U+1062C */
  0x46E7,   /* U+1062D */
  0x46E8,   /* U+1062E */
  0x46E9,   /* U+1062F */
  0x46EA,   /* U+10630 */
  0x46EB,   /* U+10631 */
  0x46EC,   /* U+10632 */
  0x46ED,   /* U+10633 */
  0x46EE,   /* U+10634 */
  0x46EF,   /* U+10635 */
  0x46F0,   /* U+10636 */
  0x46F1,   /* U+10637 */
  0x46F2,   /* U+10638 */
  0x46F3,   /* U+10639 */
  0x46F4,   /* U+1063A */
  0x46F5,   /* U+1063B */
  0x46F6,   /* U+1063C */
  0x46F7,   /* U+1063D */
  0x46F8,   /* U+1063E */
  0x46F9,   /* U+1063F */
  0x46FA,   /* U+10640 */
  0x46FB,   /* U+10641 */
  0x46FC,   /* U+10642 */
  0x46FD,   /* U+10643 */
  0x46FE,   /* U+10644 */
  0x46FF,   /* U+10645 */
  0x4700,   /* U+10646 */
  0x4701,   /* U+10647 */
  0x4702,   /* U+10648 */
  0x4703,   /* U+10649 */
  0x4704,   /* U+1064A */
  0x4705,   /* U+1064B */
  0x4706,   /* U+1064C */
  0x4707,   /* U+1064D */
  0x4708,   /* U+1064E */
  0x4709,   /* U+1064F */
  0x470A,   /* U+10650 */
  0x470B,   /* U+10651 */
  0x470C,   /* U+10652 */
  0x470D,   /* U+10653 */
  0x470E,   /* U+10654 */
  0x470F,   /* U+10655 */
  0x4710,   /* U+10656 */
  0x4711,   /* U+10657 */
  0x4712,   /* U+10658 */
  0x4713,   /* U+10659 */
  0x4714,   /* U+1065A */
  0x4715,   /* U+1065B */
  0x4716,   /* U+1065C */
  0x4717,   /* U+1065D */
  0x4718,   /* U+1065E */
  0x4719,   /* U+1065F */
  0x471A,   /* U+10660 */
  0x471B,   /* U+10661 */
  0x471C,   /* U+10662 */
  0x471D,   /* U+10663 */
  0x471E,   /* U+10664 */
  0x471F,   /* U+10665 */
  0x4720,   /* U+10666 */
  0x4721,   /* U+10667 */
  0x4722,   /* U+10668 */
  0x4723,   /* U+10669 */
  0x4724,   /* U+1066A */
  0x4725,   /* U+1066B */
  0x4726,   /* U+1066C */
  0x4727,   /* U+1066D */
  0x4728,   /* U+1066E */
  0x4729,   /* U+1066F */
  0x472A,   /* U+10670 */
  0x472B,   /* U+10671 */
  0x472C,   /* U+10672 */
  0x472D,   /* U+10673 */
  0x472E,   /* U+10674 */
  0x472F,   /* U+10675 */
  0x4730,   /* U+10676 */
  0x4731,   /* U+10677 */
  0x4732,   /* U+10678 */
  0x4733,   /* U+10679 */
  0x4734,   /* U+1067A */
  0x4735,   /* U+1067B */
  0x4736,   /* U+1067C */
  0x4737,   /* U+1067D */
  0x4738,   /* U+1067E */
  0x4739,   /* U+1067F */
  0x473A,   /* U+10680 */
  0x473B,   /* U+10681 */
  0x473C,   /* U+10682 */
  0x473D,   /* U+10683 */
  0x473E,   /* U+10684 */
  0x473F,   /* U+10685 */
  0x4740,   /* U+10686 */
  0x4741,   /* U+10687 */
  0x4742,   /* U+10688 */
  0x4743,   /* U+10689 */
  0x4744,   /* U+1068A */
  0x4745,   /* U+1068B */
  0x4746,   /* U+1068C */
  0x4747,   /* U+1068D */
  0x4748,   /* U+1068E */
  0x4749,   /* U+1068F */
  0x474A,   /* U+10690 */
  0x474B,   /* U+10691 */
  0x474C,   /* U+10692 */
  0x474D,   /* U+10693 */
  0x474E,   /* U+10694 */
  0x474F,   /* U+10695 */
  0x4750,   /* U+10696 */
  0x4751,   /* U+10697 */
  0x4752,   /* U+10698 */
  0x4753,   /* U+10699 */
  0x4754,   /* U+1069A */
  0x4755,   /* U+1069B */
  0x4756,   /* U+1069C */
  0x4757,   /* U+1069D */
  0x4758,   /* U+1069E */
  0x4759,   /* U+1069F */
  0x475A,   /* U+106A0 */
  0x475B,   /* U+106A1 */
  0x475C,   /* U+106A2 */
  0x475D,   /* U+106A3 */
  0x475E,   /* U+106A4 */
  0x475F,   /* U+106A5 */
  0x4760,   /* U+106A6 */
  0x4761,   /* U+106A7 */
  0x4762,   /* U+106A8 */
  0x4763,   /* U+106A9 */
  0x4764,   /* U+106AA */
  0x4765,   /* U+106AB */
  0x4766,   /* U+106AC */
  0x4767,   /* U+106AD */
  0x4768,   /* U+106AE */
  0x4769,   /* U+106AF */
  0x476A,   /* U+106B0 */
  0x476B,   /* U+106B1 */
  0x476C,   /* U+106B2 */
  0x476D,   /* U+106B3 */
  0x476E,   /* U+106B4 */
  0x476F,   /* U+106B5 */
  0x4770,   /* U+106B6 */
  0x4771,   /* U+106B7 */
  0x4772,   /* U+106B8 */
  0x4773,   /* U+106B9 */
  0x4774,   /* U+106BA */
  0x4775,   /* U+106BB */
  0x4776,   /* U+106BC */
  0x4777,   /* U+106BD */
  0x4778,   /* U+106BE */
  0x4779,   /* U+106BF */
  0x477A,   /* U+106C0 */
  0x477B,   /* U+106C1 */
  0x477C,   /* U+106C2 */
  0x477D,   /* U+106C3 */
  0x477E,   /* U+106C4 */
  0x477F,   /* U+106C5 */
  0x4780,   /* U+106C6 */
  0x4781,   /* U+106C7 */
  0x4782,   /* U+106C8 */
  0x4783,   /* U+106C9 */
  0x4784,   /* U+106CA */
  0x4785,   /* U+106CB */
  0x4786,   /* U+106CC */
  0x4787,   /* U+106CD */
  0x4788,   /* U+106CE */
  0x4789,   /* U+106CF */
  0x478A,   /* U+106D0 */
  0x478B,   /* U+106D1 */
  0x478C,   /* U+106D2 */
  0x478D,   /* U+106D3 */
  0x478E,   /* U+106D4 */
  0x478F,   /* U+106D5 */
  0x4790,   /* U+106D6 */
  0x4791,   /* U+106D7 */
  0x4792,   /* U+106D8 */
  0x4793,   /* U+106D9 */
  0x4794,   /* U+106DA */
  0x4795,   /* U+106DB */
  0x4796,   /* U+106DC */
  0x4797,   /* U+106DD */
  0x4798,   /* U+106DE */
  0x4799,   /* U+106DF */
  0x479A,   /* U+106E0 */
  0x479B,   /* U+106E1 */
  0x479C,   /* U+106E2 */
  0x479D,   /* U+106E3 */
  0x479E,   /* U+106E4 */
  0x479F,   /* U+106E5 */
  0x47A0,   /* U+106E6 */
  0x47A1,   /* U+106E7 */
  0x47A2,   /* U+106E8 */
  0x47A3,   /* U+106E9 */
  0x47A4,   /* U+106EA */
  0x47A5,   /* U+106EB */
  0x47A6,   /* U+106EC */
  0x47A7,   /* U+106ED */
  0x47A8,   /* U+106EE */
  0x47A9,   /* U+106EF */
  0x47AA,   /* U+106F0 */
  0x47AB,   /* U+106F1 */
  0x47AC,   /* U+106F2 */
  0x47AD,   /* U+106F3 */
  0x47AE,   /* U+106F4 */
  0x47AF,   /* U+106F5 */
  0x47B0,   /* U+106F6 */
  0x47B1,   /* U+106F7 */
  0x47B2,   /* U+106F8 */
  0x47B3,   /* U+106F9 */
  0x47B4,   /* U+106FA */
  0x47B5,   /* U+106FB */
  0x47B6,   /* U+106FC */
  0x47B7,   /* U+106FD */
  0x47B8,   /* U+106FE */
  0x47B9,   /* U+106FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10600 */
  0x0020,   /* U+10601 */
  0x0020,   /* U+10602 */
  0x0020,   /* U+10603 */
  0x0020,   /* U+10604 */
  0x0020,   /* U+10605 */
  0x0020,   /* U+10606 */
  0x0020,   /* U+10607 */
  0x0020,   /* U+10608 */
  0x0020,   /* U+10609 */
  0x0020,   /* U+1060A */
  0x0020,   /* U+1060B */
  0x0020,   /* U+1060C */
  0x0020,   /* U+1060D */
  0x0020,   /* U+1060E */
  0x0020,   /* U+1060F */
  0x0020,   /* U+10610 */
  0x0020,   /* U+10611 */
  0x0020,   /* U+10612 */
  0x0020,   /* U+10613 */
  0x0020,   /* U+10614 */
  0x0020,   /* U+10615 */
  0x0020,   /* U+10616 */
  0x0020,   /* U+10617 */
  0x0020,   /* U+10618 */
  0x0020,   /* U+10619 */
  0x0020,   /* U+1061A */
  0x0020,   /* U+1061B */
  0x0020,   /* U+1061C */
  0x0020,   /* U+1061D */
  0x0020,   /* U+1061E */
  0x0020,   /* U+1061F */
  0x0020,   /* U+10620 */
  0x0020,   /* U+10621 */
  0x0020,   /* U+10622 */
  0x0020,   /* U+10623 */
  0x0020,   /* U+10624 */
  0x0020,   /* U+10625 */
  0x0020,   /* U+10626 */
  0x0020,   /* U+10627 */
  0x0020,   /* U+10628 */
  0x0020,   /* U+10629 */
  0x0020,   /* U+1062A */
  0x0020,   /* U+1062B */
  0x0020,   /* U+1062C */
  0x0020,   /* U+1062D */
  0x0020,   /* U+1062E */
  0x0020,   /* U+1062F */
  0x0020,   /* U+10630 */
  0x0020,   /* U+10631 */
  0x0020,   /* U+10632 */
  0x0020,   /* U+10633 */
  0x0020,   /* U+10634 */
  0x0020,   /* U+10635 */
  0x0020,   /* U+10636 */
  0x0020,   /* U+10637 */
  0x0020,   /* U+10638 */
  0x0020,   /* U+10639 */
  0x0020,   /* U+1063A */
  0x0020,   /* U+1063B */
  0x0020,   /* U+1063C */
  0x0020,   /* U+1063D */
  0x0020,   /* U+1063E */
  0x0020,   /* U+1063F */
  0x0020,   /* U+10640 */
  0x0020,   /* U+10641 */
  0x0020,   /* U+10642 */
  0x0020,   /* U+10643 */
  0x0020,   /* U+10644 */
  0x0020,   /* U+10645 */
  0x0020,   /* U+10646 */
  0x0020,   /* U+10647 */
  0x0020,   /* U+10648 */
  0x0020,   /* U+10649 */
  0x0020,   /* U+1064A */
  0x0020,   /* U+1064B */
  0x0020,   /* U+1064C */
  0x0020,   /* U+1064D */
  0x0020,   /* U+1064E */
  0x0020,   /* U+1064F */
  0x0020,   /* U+10650 */
  0x0020,   /* U+10651 */
  0x0020,   /* U+10652 */
  0x0020,   /* U+10653 */
  0x0020,   /* U+10654 */
  0x0020,   /* U+10655 */
  0x0020,   /* U+10656 */
  0x0020,   /* U+10657 */
  0x0020,   /* U+10658 */
  0x0020,   /* U+10659 */
  0x0020,   /* U+1065A */
  0x0020,   /* U+1065B */
  0x0020,   /* U+1065C */
  0x0020,   /* U+1065D */
  0x0020,   /* U+1065E */
  0x0020,   /* U+1065F */
  0x0020,   /* U+10660 */
  0x0020,   /* U+10661 */
  0x0020,   /* U+10662 */
  0x0020,   /* U+10663 */
  0x0020,   /* U+10664 */
  0x0020,   /* U+10665 */
  0x0020,   /* U+10666 */
  0x0020,   /* U+10667 */
  0x0020,   /* U+10668 */
  0x0020,   /* U+10669 */
  0x0020,   /* U+1066A */
  0x0020,   /* U+1066B */
  0x0020,   /* U+1066C */
  0x0020,   /* U+1066D */
  0x0020,   /* U+1066E */
  0x0020,   /* U+1066F */
  0x0020,   /* U+10670 */
  0x0020,   /* U+10671 */
  0x0020,   /* U+10672 */
  0x0020,   /* U+10673 */
  0x0020,   /* U+10674 */
  0x0020,   /* U+10675 */
  0x0020,   /* U+10676 */
  0x0020,   /* U+10677 */
  0x0020,   /* U+10678 */
  0x0020,   /* U+10679 */
  0x0020,   /* U+1067A */
  0x0020,   /* U+1067B */
  0x0020,   /* U+1067C */
  0x0020,   /* U+1067D */
  0x0020,   /* U+1067E */
  0x0020,   /* U+1067F */
  0x0020,   /* U+10680 */
  0x0020,   /* U+10681 */
  0x0020,   /* U+10682 */
  0x0020,   /* U+10683 */
  0x0020,   /* U+10684 */
  0x0020,   /* U+10685 */
  0x0020,   /* U+10686 */
  0x0020,   /* U+10687 */
  0x0020,   /* U+10688 */
  0x0020,   /* U+10689 */
  0x0020,   /* U+1068A */
  0x0020,   /* U+1068B */
  0x0020,   /* U+1068C */
  0x0020,   /* U+1068D */
  0x0020,   /* U+1068E */
  0x0020,   /* U+1068F */
  0x0020,   /* U+10690 */
  0x0020,   /* U+10691 */
  0x0020,   /* U+10692 */
  0x0020,   /* U+10693 */
  0x0020,   /* U+10694 */
  0x0020,   /* U+10695 */
  0x0020,   /* U+10696 */
  0x0020,   /* U+10697 */
  0x0020,   /* U+10698 */
  0x0020,   /* U+10699 */
  0x0020,   /* U+1069A */
  0x0020,   /* U+1069B */
  0x0020,   /* U+1069C */
  0x0020,   /* U+1069D */
  0x0020,   /* U+1069E */
  0x0020,   /* U+1069F */
  0x0020,   /* U+106A0 */
  0x0020,   /* U+106A1 */
  0x0020,   /* U+106A2 */
  0x0020,   /* U+106A3 */
  0x0020,   /* U+106A4 */
  0x0020,   /* U+106A5 */
  0x0020,   /* U+106A6 */
  0x0020,   /* U+106A7 */
  0x0020,   /* U+106A8 */
  0x0020,   /* U+106A9 */
  0x0020,   /* U+106AA */
  0x0020,   /* U+106AB */
  0x0020,   /* U+106AC */
  0x0020,   /* U+106AD */
  0x0020,   /* U+106AE */
  0x0020,   /* U+106AF */
  0x0020,   /* U+106B0 */
  0x0020,   /* U+106B1 */
  0x0020,   /* U+106B2 */
  0x0020,   /* U+106B3 */
  0x0020,   /* U+106B4 */
  0x0020,   /* U+106B5 */
  0x0020,   /* U+106B6 */
  0x0020,   /* U+106B7 */
  0x0020,   /* U+106B8 */
  0x0020,   /* U+106B9 */
  0x0020,   /* U+106BA */
  0x0020,   /* U+106BB */
  0x0020,   /* U+106BC */
  0x0020,   /* U+106BD */
  0x0020,   /* U+106BE */
  0x0020,   /* U+106BF */
  0x0020,   /* U+106C0 */
  0x0020,   /* U+106C1 */
  0x0020,   /* U+106C2 */
  0x0020,   /* U+106C3 */
  0x0020,   /* U+106C4 */
  0x0020,   /* U+106C5 */
  0x0020,   /* U+106C6 */
  0x0020,   /* U+106C7 */
  0x0020,   /* U+106C8 */
  0x0020,   /* U+106C9 */
  0x0020,   /* U+106CA */
  0x0020,   /* U+106CB */
  0x0020,   /* U+106CC */
  0x0020,   /* U+106CD */
  0x0020,   /* U+106CE */
  0x0020,   /* U+106CF */
  0x0020,   /* U+106D0 */
  0x0020,   /* U+106D1 */
  0x0020,   /* U+106D2 */
  0x0020,   /* U+106D3 */
  0x0020,   /* U+106D4 */
  0x0020,   /* U+106D5 */
  0x0020,   /* U+106D6 */
  0x0020,   /* U+106D7 */
  0x0020,   /* U+106D8 */
  0x0020,   /* U+106D9 */
  0x0020,   /* U+106DA */
  0x0020,   /* U+106DB */
  0x0020,   /* U+106DC */
  0x0020,   /* U+106DD */
  0x0020,   /* U+106DE */
  0x0020,   /* U+106DF */
  0x0020,   /* U+106E0 */
  0x0020,   /* U+106E1 */
  0x0020,   /* U+106E2 */
  0x0020,   /* U+106E3 */
  0x0020,   /* U+106E4 */
  0x0020,   /* U+106E5 */
  0x0020,   /* U+106E6 */
  0x0020,   /* U+106E7 */
  0x0020,   /* U+106E8 */
  0x0020,   /* U+106E9 */
  0x0020,   /* U+106EA */
  0x0020,   /* U+106EB */
  0x0020,   /* U+106EC */
  0x0020,   /* U+106ED */
  0x0020,   /* U+106EE */
  0x0020,   /* U+106EF */
  0x0020,   /* U+106F0 */
  0x0020,   /* U+106F1 */
  0x0020,   /* U+106F2 */
  0x0020,   /* U+106F3 */
  0x0020,   /* U+106F4 */
  0x0020,   /* U+106F5 */
  0x0020,   /* U+106F6 */
  0x0020,   /* U+106F7 */
  0x0020,   /* U+106F8 */
  0x0020,   /* U+106F9 */
  0x0020,   /* U+106FA */
  0x0020,   /* U+106FB */
  0x0020,   /* U+106FC */
  0x0020,   /* U+106FD */
  0x0020,   /* U+106FE */
  0x0020,   /* U+106FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10600 */
  0x0002,   /* U+10601 */
  0x0002,   /* U+10602 */
  0x0002,   /* U+10603 */
  0x0002,   /* U+10604 */
  0x0002,   /* U+10605 */
  0x0002,   /* U+10606 */
  0x0002,   /* U+10607 */
  0x0002,   /* U+10608 */
  0x0002,   /* U+10609 */
  0x0002,   /* U+1060A */
  0x0002,   /* U+1060B */
  0x0002,   /* U+1060C */
  0x0002,   /* U+1060D */
  0x0002,   /* U+1060E */
  0x0002,   /* U+1060F */
  0x0002,   /* U+10610 */
  0x0002,   /* U+10611 */
  0x0002,   /* U+10612 */
  0x0002,   /* U+10613 */
  0x0002,   /* U+10614 */
  0x0002,   /* U+10615 */
  0x0002,   /* U+10616 */
  0x0002,   /* U+10617 */
  0x0002,   /* U+10618 */
  0x0002,   /* U+10619 */
  0x0002,   /* U+1061A */
  0x0002,   /* U+1061B */
  0x0002,   /* U+1061C */
  0x0002,   /* U+1061D */
  0x0002,   /* U+1061E */
  0x0002,   /* U+1061F */
  0x0002,   /* U+10620 */
  0x0002,   /* U+10621 */
  0x0002,   /* U+10622 */
  0x0002,   /* U+10623 */
  0x0002,   /* U+10624 */
  0x0002,   /* U+10625 */
  0x0002,   /* U+10626 */
  0x0002,   /* U+10627 */
  0x0002,   /* U+10628 */
  0x0002,   /* U+10629 */
  0x0002,   /* U+1062A */
  0x0002,   /* U+1062B */
  0x0002,   /* U+1062C */
  0x0002,   /* U+1062D */
  0x0002,   /* U+1062E */
  0x0002,   /* U+1062F */
  0x0002,   /* U+10630 */
  0x0002,   /* U+10631 */
  0x0002,   /* U+10632 */
  0x0002,   /* U+10633 */
  0x0002,   /* U+10634 */
  0x0002,   /* U+10635 */
  0x0002,   /* U+10636 */
  0x0002,   /* U+10637 */
  0x0002,   /* U+10638 */
  0x0002,   /* U+10639 */
  0x0002,   /* U+1063A */
  0x0002,   /* U+1063B */
  0x0002,   /* U+1063C */
  0x0002,   /* U+1063D */
  0x0002,   /* U+1063E */
  0x0002,   /* U+1063F */
  0x0002,   /* U+10640 */
  0x0002,   /* U+10641 */
  0x0002,   /* U+10642 */
  0x0002,   /* U+10643 */
  0x0002,   /* U+10644 */
  0x0002,   /* U+10645 */
  0x0002,   /* U+10646 */
  0x0002,   /* U+10647 */
  0x0002,   /* U+10648 */
  0x0002,   /* U+10649 */
  0x0002,   /* U+1064A */
  0x0002,   /* U+1064B */
  0x0002,   /* U+1064C */
  0x0002,   /* U+1064D */
  0x0002,   /* U+1064E */
  0x0002,   /* U+1064F */
  0x0002,   /* U+10650 */
  0x0002,   /* U+10651 */
  0x0002,   /* U+10652 */
  0x0002,   /* U+10653 */
  0x0002,   /* U+10654 */
  0x0002,   /* U+10655 */
  0x0002,   /* U+10656 */
  0x0002,   /* U+10657 */
  0x0002,   /* U+10658 */
  0x0002,   /* U+10659 */
  0x0002,   /* U+1065A */
  0x0002,   /* U+1065B */
  0x0002,   /* U+1065C */
  0x0002,   /* U+1065D */
  0x0002,   /* U+1065E */
  0x0002,   /* U+1065F */
  0x0002,   /* U+10660 */
  0x0002,   /* U+10661 */
  0x0002,   /* U+10662 */
  0x0002,   /* U+10663 */
  0x0002,   /* U+10664 */
  0x0002,   /* U+10665 */
  0x0002,   /* U+10666 */
  0x0002,   /* U+10667 */
  0x0002,   /* U+10668 */
  0x0002,   /* U+10669 */
  0x0002,   /* U+1066A */
  0x0002,   /* U+1066B */
  0x0002,   /* U+1066C */
  0x0002,   /* U+1066D */
  0x0002,   /* U+1066E */
  0x0002,   /* U+1066F */
  0x0002,   /* U+10670 */
  0x0002,   /* U+10671 */
  0x0002,   /* U+10672 */
  0x0002,   /* U+10673 */
  0x0002,   /* U+10674 */
  0x0002,   /* U+10675 */
  0x0002,   /* U+10676 */
  0x0002,   /* U+10677 */
  0x0002,   /* U+10678 */
  0x0002,   /* U+10679 */
  0x0002,   /* U+1067A */
  0x0002,   /* U+1067B */
  0x0002,   /* U+1067C */
  0x0002,   /* U+1067D */
  0x0002,   /* U+1067E */
  0x0002,   /* U+1067F */
  0x0002,   /* U+10680 */
  0x0002,   /* U+10681 */
  0x0002,   /* U+10682 */
  0x0002,   /* U+10683 */
  0x0002,   /* U+10684 */
  0x0002,   /* U+10685 */
  0x0002,   /* U+10686 */
  0x0002,   /* U+10687 */
  0x0002,   /* U+10688 */
  0x0002,   /* U+10689 */
  0x0002,   /* U+1068A */
  0x0002,   /* U+1068B */
  0x0002,   /* U+1068C */
  0x0002,   /* U+1068D */
  0x0002,   /* U+1068E */
  0x0002,   /* U+1068F */
  0x0002,   /* U+10690 */
  0x0002,   /* U+10691 */
  0x0002,   /* U+10692 */
  0x0002,   /* U+10693 */
  0x0002,   /* U+10694 */
  0x0002,   /* U+10695 */
  0x0002,   /* U+10696 */
  0x0002,   /* U+10697 */
  0x0002,   /* U+10698 */
  0x0002,   /* U+10699 */
  0x0002,   /* U+1069A */
  0x0002,   /* U+1069B */
  0x0002,   /* U+1069C */
  0x0002,   /* U+1069D */
  0x0002,   /* U+1069E */
  0x0002,   /* U+1069F */
  0x0002,   /* U+106A0 */
  0x0002,   /* U+106A1 */
  0x0002,   /* U+106A2 */
  0x0002,   /* U+106A3 */
  0x0002,   /* U+106A4 */
  0x0002,   /* U+106A5 */
  0x0002,   /* U+106A6 */
  0x0002,   /* U+106A7 */
  0x0002,   /* U+106A8 */
  0x0002,   /* U+106A9 */
  0x0002,   /* U+106AA */
  0x0002,   /* U+106AB */
  0x0002,   /* U+106AC */
  0x0002,   /* U+106AD */
  0x0002,   /* U+106AE */
  0x0002,   /* U+106AF */
  0x0002,   /* U+106B0 */
  0x0002,   /* U+106B1 */
  0x0002,   /* U+106B2 */
  0x0002,   /* U+106B3 */
  0x0002,   /* U+106B4 */
  0x0002,   /* U+106B5 */
  0x0002,   /* U+106B6 */
  0x0002,   /* U+106B7 */
  0x0002,   /* U+106B8 */
  0x0002,   /* U+106B9 */
  0x0002,   /* U+106BA */
  0x0002,   /* U+106BB */
  0x0002,   /* U+106BC */
  0x0002,   /* U+106BD */
  0x0002,   /* U+106BE */
  0x0002,   /* U+106BF */
  0x0002,   /* U+106C0 */
  0x0002,   /* U+106C1 */
  0x0002,   /* U+106C2 */
  0x0002,   /* U+106C3 */
  0x0002,   /* U+106C4 */
  0x0002,   /* U+106C5 */
  0x0002,   /* U+106C6 */
  0x0002,   /* U+106C7 */
  0x0002,   /* U+106C8 */
  0x0002,   /* U+106C9 */
  0x0002,   /* U+106CA */
  0x0002,   /* U+106CB */
  0x0002,   /* U+106CC */
  0x0002,   /* U+106CD */
  0x0002,   /* U+106CE */
  0x0002,   /* U+106CF */
  0x0002,   /* U+106D0 */
  0x0002,   /* U+106D1 */
  0x0002,   /* U+106D2 */
  0x0002,   /* U+106D3 */
  0x0002,   /* U+106D4 */
  0x0002,   /* U+106D5 */
  0x0002,   /* U+106D6 */
  0x0002,   /* U+106D7 */
  0x0002,   /* U+106D8 */
  0x0002,   /* U+106D9 */
  0x0002,   /* U+106DA */
  0x0002,   /* U+106DB */
  0x0002,   /* U+106DC */
  0x0002,   /* U+106DD */
  0x0002,   /* U+106DE */
  0x0002,   /* U+106DF */
  0x0002,   /* U+106E0 */
  0x0002,   /* U+106E1 */
  0x0002,   /* U+106E2 */
  0x0002,   /* U+106E3 */
  0x0002,   /* U+106E4 */
  0x0002,   /* U+106E5 */
  0x0002,   /* U+106E6 */
  0x0002,   /* U+106E7 */
  0x0002,   /* U+106E8 */
  0x0002,   /* U+106E9 */
  0x0002,   /* U+106EA */
  0x0002,   /* U+106EB */
  0x0002,   /* U+106EC */
  0x0002,   /* U+106ED */
  0x0002,   /* U+106EE */
  0x0002,   /* U+106EF */
  0x0002,   /* U+106F0 */
  0x0002,   /* U+106F1 */
  0x0002,   /* U+106F2 */
  0x0002,   /* U+106F3 */
  0x0002,   /* U+106F4 */
  0x0002,   /* U+106F5 */
  0x0002,   /* U+106F6 */
  0x0002,   /* U+106F7 */
  0x0002,   /* U+106F8 */
  0x0002,   /* U+106F9 */
  0x0002,   /* U+106FA */
  0x0002,   /* U+106FB */
  0x0002,   /* U+106FC */
  0x0002,   /* U+106FD */
  0x0002,   /* U+106FE */
  0x0002,   /* U+106FF */
};

uint16 uca900_p107[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x47BA,   /* U+10700 */
  0x47BB,   /* U+10701 */
  0x47BC,   /* U+10702 */
  0x47BD,   /* U+10703 */
  0x47BE,   /* U+10704 */
  0x47BF,   /* U+10705 */
  0x47C0,   /* U+10706 */
  0x47C1,   /* U+10707 */
  0x47C2,   /* U+10708 */
  0x47C3,   /* U+10709 */
  0x47C4,   /* U+1070A */
  0x47C5,   /* U+1070B */
  0x47C6,   /* U+1070C */
  0x47C7,   /* U+1070D */
  0x47C8,   /* U+1070E */
  0x47C9,   /* U+1070F */
  0x47CA,   /* U+10710 */
  0x47CB,   /* U+10711 */
  0x47CC,   /* U+10712 */
  0x47CD,   /* U+10713 */
  0x47CE,   /* U+10714 */
  0x47CF,   /* U+10715 */
  0x47D0,   /* U+10716 */
  0x47D1,   /* U+10717 */
  0x47D2,   /* U+10718 */
  0x47D3,   /* U+10719 */
  0x47D4,   /* U+1071A */
  0x47D5,   /* U+1071B */
  0x47D6,   /* U+1071C */
  0x47D7,   /* U+1071D */
  0x47D8,   /* U+1071E */
  0x47D9,   /* U+1071F */
  0x47DA,   /* U+10720 */
  0x47DB,   /* U+10721 */
  0x47DC,   /* U+10722 */
  0x47DD,   /* U+10723 */
  0x47DE,   /* U+10724 */
  0x47DF,   /* U+10725 */
  0x47E0,   /* U+10726 */
  0x47E1,   /* U+10727 */
  0x47E2,   /* U+10728 */
  0x47E3,   /* U+10729 */
  0x47E4,   /* U+1072A */
  0x47E5,   /* U+1072B */
  0x47E6,   /* U+1072C */
  0x47E7,   /* U+1072D */
  0x47E8,   /* U+1072E */
  0x47E9,   /* U+1072F */
  0x47EA,   /* U+10730 */
  0x47EB,   /* U+10731 */
  0x47EC,   /* U+10732 */
  0x47ED,   /* U+10733 */
  0x47EE,   /* U+10734 */
  0x47EF,   /* U+10735 */
  0x47F0,   /* U+10736 */
  0xFBC2,   /* U+10737 */
  0xFBC2,   /* U+10738 */
  0xFBC2,   /* U+10739 */
  0xFBC2,   /* U+1073A */
  0xFBC2,   /* U+1073B */
  0xFBC2,   /* U+1073C */
  0xFBC2,   /* U+1073D */
  0xFBC2,   /* U+1073E */
  0xFBC2,   /* U+1073F */
  0x47F1,   /* U+10740 */
  0x47F2,   /* U+10741 */
  0x47F3,   /* U+10742 */
  0x47F4,   /* U+10743 */
  0x47F5,   /* U+10744 */
  0x47F6,   /* U+10745 */
  0x47F7,   /* U+10746 */
  0x47F8,   /* U+10747 */
  0x47F9,   /* U+10748 */
  0x47FA,   /* U+10749 */
  0x47FB,   /* U+1074A */
  0x47FC,   /* U+1074B */
  0x47FD,   /* U+1074C */
  0x47FE,   /* U+1074D */
  0x47FF,   /* U+1074E */
  0x4800,   /* U+1074F */
  0x4801,   /* U+10750 */
  0x4802,   /* U+10751 */
  0x4803,   /* U+10752 */
  0x4804,   /* U+10753 */
  0x4805,   /* U+10754 */
  0x4806,   /* U+10755 */
  0xFBC2,   /* U+10756 */
  0xFBC2,   /* U+10757 */
  0xFBC2,   /* U+10758 */
  0xFBC2,   /* U+10759 */
  0xFBC2,   /* U+1075A */
  0xFBC2,   /* U+1075B */
  0xFBC2,   /* U+1075C */
  0xFBC2,   /* U+1075D */
  0xFBC2,   /* U+1075E */
  0xFBC2,   /* U+1075F */
  0x4807,   /* U+10760 */
  0x4808,   /* U+10761 */
  0x4809,   /* U+10762 */
  0x480A,   /* U+10763 */
  0x480B,   /* U+10764 */
  0x480C,   /* U+10765 */
  0x480D,   /* U+10766 */
  0x480E,   /* U+10767 */
  0xFBC2,   /* U+10768 */
  0xFBC2,   /* U+10769 */
  0xFBC2,   /* U+1076A */
  0xFBC2,   /* U+1076B */
  0xFBC2,   /* U+1076C */
  0xFBC2,   /* U+1076D */
  0xFBC2,   /* U+1076E */
  0xFBC2,   /* U+1076F */
  0xFBC2,   /* U+10770 */
  0xFBC2,   /* U+10771 */
  0xFBC2,   /* U+10772 */
  0xFBC2,   /* U+10773 */
  0xFBC2,   /* U+10774 */
  0xFBC2,   /* U+10775 */
  0xFBC2,   /* U+10776 */
  0xFBC2,   /* U+10777 */
  0xFBC2,   /* U+10778 */
  0xFBC2,   /* U+10779 */
  0xFBC2,   /* U+1077A */
  0xFBC2,   /* U+1077B */
  0xFBC2,   /* U+1077C */
  0xFBC2,   /* U+1077D */
  0xFBC2,   /* U+1077E */
  0xFBC2,   /* U+1077F */
  0xFBC2,   /* U+10780 */
  0xFBC2,   /* U+10781 */
  0xFBC2,   /* U+10782 */
  0xFBC2,   /* U+10783 */
  0xFBC2,   /* U+10784 */
  0xFBC2,   /* U+10785 */
  0xFBC2,   /* U+10786 */
  0xFBC2,   /* U+10787 */
  0xFBC2,   /* U+10788 */
  0xFBC2,   /* U+10789 */
  0xFBC2,   /* U+1078A */
  0xFBC2,   /* U+1078B */
  0xFBC2,   /* U+1078C */
  0xFBC2,   /* U+1078D */
  0xFBC2,   /* U+1078E */
  0xFBC2,   /* U+1078F */
  0xFBC2,   /* U+10790 */
  0xFBC2,   /* U+10791 */
  0xFBC2,   /* U+10792 */
  0xFBC2,   /* U+10793 */
  0xFBC2,   /* U+10794 */
  0xFBC2,   /* U+10795 */
  0xFBC2,   /* U+10796 */
  0xFBC2,   /* U+10797 */
  0xFBC2,   /* U+10798 */
  0xFBC2,   /* U+10799 */
  0xFBC2,   /* U+1079A */
  0xFBC2,   /* U+1079B */
  0xFBC2,   /* U+1079C */
  0xFBC2,   /* U+1079D */
  0xFBC2,   /* U+1079E */
  0xFBC2,   /* U+1079F */
  0xFBC2,   /* U+107A0 */
  0xFBC2,   /* U+107A1 */
  0xFBC2,   /* U+107A2 */
  0xFBC2,   /* U+107A3 */
  0xFBC2,   /* U+107A4 */
  0xFBC2,   /* U+107A5 */
  0xFBC2,   /* U+107A6 */
  0xFBC2,   /* U+107A7 */
  0xFBC2,   /* U+107A8 */
  0xFBC2,   /* U+107A9 */
  0xFBC2,   /* U+107AA */
  0xFBC2,   /* U+107AB */
  0xFBC2,   /* U+107AC */
  0xFBC2,   /* U+107AD */
  0xFBC2,   /* U+107AE */
  0xFBC2,   /* U+107AF */
  0xFBC2,   /* U+107B0 */
  0xFBC2,   /* U+107B1 */
  0xFBC2,   /* U+107B2 */
  0xFBC2,   /* U+107B3 */
  0xFBC2,   /* U+107B4 */
  0xFBC2,   /* U+107B5 */
  0xFBC2,   /* U+107B6 */
  0xFBC2,   /* U+107B7 */
  0xFBC2,   /* U+107B8 */
  0xFBC2,   /* U+107B9 */
  0xFBC2,   /* U+107BA */
  0xFBC2,   /* U+107BB */
  0xFBC2,   /* U+107BC */
  0xFBC2,   /* U+107BD */
  0xFBC2,   /* U+107BE */
  0xFBC2,   /* U+107BF */
  0xFBC2,   /* U+107C0 */
  0xFBC2,   /* U+107C1 */
  0xFBC2,   /* U+107C2 */
  0xFBC2,   /* U+107C3 */
  0xFBC2,   /* U+107C4 */
  0xFBC2,   /* U+107C5 */
  0xFBC2,   /* U+107C6 */
  0xFBC2,   /* U+107C7 */
  0xFBC2,   /* U+107C8 */
  0xFBC2,   /* U+107C9 */
  0xFBC2,   /* U+107CA */
  0xFBC2,   /* U+107CB */
  0xFBC2,   /* U+107CC */
  0xFBC2,   /* U+107CD */
  0xFBC2,   /* U+107CE */
  0xFBC2,   /* U+107CF */
  0xFBC2,   /* U+107D0 */
  0xFBC2,   /* U+107D1 */
  0xFBC2,   /* U+107D2 */
  0xFBC2,   /* U+107D3 */
  0xFBC2,   /* U+107D4 */
  0xFBC2,   /* U+107D5 */
  0xFBC2,   /* U+107D6 */
  0xFBC2,   /* U+107D7 */
  0xFBC2,   /* U+107D8 */
  0xFBC2,   /* U+107D9 */
  0xFBC2,   /* U+107DA */
  0xFBC2,   /* U+107DB */
  0xFBC2,   /* U+107DC */
  0xFBC2,   /* U+107DD */
  0xFBC2,   /* U+107DE */
  0xFBC2,   /* U+107DF */
  0xFBC2,   /* U+107E0 */
  0xFBC2,   /* U+107E1 */
  0xFBC2,   /* U+107E2 */
  0xFBC2,   /* U+107E3 */
  0xFBC2,   /* U+107E4 */
  0xFBC2,   /* U+107E5 */
  0xFBC2,   /* U+107E6 */
  0xFBC2,   /* U+107E7 */
  0xFBC2,   /* U+107E8 */
  0xFBC2,   /* U+107E9 */
  0xFBC2,   /* U+107EA */
  0xFBC2,   /* U+107EB */
  0xFBC2,   /* U+107EC */
  0xFBC2,   /* U+107ED */
  0xFBC2,   /* U+107EE */
  0xFBC2,   /* U+107EF */
  0xFBC2,   /* U+107F0 */
  0xFBC2,   /* U+107F1 */
  0xFBC2,   /* U+107F2 */
  0xFBC2,   /* U+107F3 */
  0xFBC2,   /* U+107F4 */
  0xFBC2,   /* U+107F5 */
  0xFBC2,   /* U+107F6 */
  0xFBC2,   /* U+107F7 */
  0xFBC2,   /* U+107F8 */
  0xFBC2,   /* U+107F9 */
  0xFBC2,   /* U+107FA */
  0xFBC2,   /* U+107FB */
  0xFBC2,   /* U+107FC */
  0xFBC2,   /* U+107FD */
  0xFBC2,   /* U+107FE */
  0xFBC2,   /* U+107FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10700 */
  0x0020,   /* U+10701 */
  0x0020,   /* U+10702 */
  0x0020,   /* U+10703 */
  0x0020,   /* U+10704 */
  0x0020,   /* U+10705 */
  0x0020,   /* U+10706 */
  0x0020,   /* U+10707 */
  0x0020,   /* U+10708 */
  0x0020,   /* U+10709 */
  0x0020,   /* U+1070A */
  0x0020,   /* U+1070B */
  0x0020,   /* U+1070C */
  0x0020,   /* U+1070D */
  0x0020,   /* U+1070E */
  0x0020,   /* U+1070F */
  0x0020,   /* U+10710 */
  0x0020,   /* U+10711 */
  0x0020,   /* U+10712 */
  0x0020,   /* U+10713 */
  0x0020,   /* U+10714 */
  0x0020,   /* U+10715 */
  0x0020,   /* U+10716 */
  0x0020,   /* U+10717 */
  0x0020,   /* U+10718 */
  0x0020,   /* U+10719 */
  0x0020,   /* U+1071A */
  0x0020,   /* U+1071B */
  0x0020,   /* U+1071C */
  0x0020,   /* U+1071D */
  0x0020,   /* U+1071E */
  0x0020,   /* U+1071F */
  0x0020,   /* U+10720 */
  0x0020,   /* U+10721 */
  0x0020,   /* U+10722 */
  0x0020,   /* U+10723 */
  0x0020,   /* U+10724 */
  0x0020,   /* U+10725 */
  0x0020,   /* U+10726 */
  0x0020,   /* U+10727 */
  0x0020,   /* U+10728 */
  0x0020,   /* U+10729 */
  0x0020,   /* U+1072A */
  0x0020,   /* U+1072B */
  0x0020,   /* U+1072C */
  0x0020,   /* U+1072D */
  0x0020,   /* U+1072E */
  0x0020,   /* U+1072F */
  0x0020,   /* U+10730 */
  0x0020,   /* U+10731 */
  0x0020,   /* U+10732 */
  0x0020,   /* U+10733 */
  0x0020,   /* U+10734 */
  0x0020,   /* U+10735 */
  0x0020,   /* U+10736 */
  0x0020,   /* U+10737 */
  0x0020,   /* U+10738 */
  0x0020,   /* U+10739 */
  0x0020,   /* U+1073A */
  0x0020,   /* U+1073B */
  0x0020,   /* U+1073C */
  0x0020,   /* U+1073D */
  0x0020,   /* U+1073E */
  0x0020,   /* U+1073F */
  0x0020,   /* U+10740 */
  0x0020,   /* U+10741 */
  0x0020,   /* U+10742 */
  0x0020,   /* U+10743 */
  0x0020,   /* U+10744 */
  0x0020,   /* U+10745 */
  0x0020,   /* U+10746 */
  0x0020,   /* U+10747 */
  0x0020,   /* U+10748 */
  0x0020,   /* U+10749 */
  0x0020,   /* U+1074A */
  0x0020,   /* U+1074B */
  0x0020,   /* U+1074C */
  0x0020,   /* U+1074D */
  0x0020,   /* U+1074E */
  0x0020,   /* U+1074F */
  0x0020,   /* U+10750 */
  0x0020,   /* U+10751 */
  0x0020,   /* U+10752 */
  0x0020,   /* U+10753 */
  0x0020,   /* U+10754 */
  0x0020,   /* U+10755 */
  0x0020,   /* U+10756 */
  0x0020,   /* U+10757 */
  0x0020,   /* U+10758 */
  0x0020,   /* U+10759 */
  0x0020,   /* U+1075A */
  0x0020,   /* U+1075B */
  0x0020,   /* U+1075C */
  0x0020,   /* U+1075D */
  0x0020,   /* U+1075E */
  0x0020,   /* U+1075F */
  0x0020,   /* U+10760 */
  0x0020,   /* U+10761 */
  0x0020,   /* U+10762 */
  0x0020,   /* U+10763 */
  0x0020,   /* U+10764 */
  0x0020,   /* U+10765 */
  0x0020,   /* U+10766 */
  0x0020,   /* U+10767 */
  0x0020,   /* U+10768 */
  0x0020,   /* U+10769 */
  0x0020,   /* U+1076A */
  0x0020,   /* U+1076B */
  0x0020,   /* U+1076C */
  0x0020,   /* U+1076D */
  0x0020,   /* U+1076E */
  0x0020,   /* U+1076F */
  0x0020,   /* U+10770 */
  0x0020,   /* U+10771 */
  0x0020,   /* U+10772 */
  0x0020,   /* U+10773 */
  0x0020,   /* U+10774 */
  0x0020,   /* U+10775 */
  0x0020,   /* U+10776 */
  0x0020,   /* U+10777 */
  0x0020,   /* U+10778 */
  0x0020,   /* U+10779 */
  0x0020,   /* U+1077A */
  0x0020,   /* U+1077B */
  0x0020,   /* U+1077C */
  0x0020,   /* U+1077D */
  0x0020,   /* U+1077E */
  0x0020,   /* U+1077F */
  0x0020,   /* U+10780 */
  0x0020,   /* U+10781 */
  0x0020,   /* U+10782 */
  0x0020,   /* U+10783 */
  0x0020,   /* U+10784 */
  0x0020,   /* U+10785 */
  0x0020,   /* U+10786 */
  0x0020,   /* U+10787 */
  0x0020,   /* U+10788 */
  0x0020,   /* U+10789 */
  0x0020,   /* U+1078A */
  0x0020,   /* U+1078B */
  0x0020,   /* U+1078C */
  0x0020,   /* U+1078D */
  0x0020,   /* U+1078E */
  0x0020,   /* U+1078F */
  0x0020,   /* U+10790 */
  0x0020,   /* U+10791 */
  0x0020,   /* U+10792 */
  0x0020,   /* U+10793 */
  0x0020,   /* U+10794 */
  0x0020,   /* U+10795 */
  0x0020,   /* U+10796 */
  0x0020,   /* U+10797 */
  0x0020,   /* U+10798 */
  0x0020,   /* U+10799 */
  0x0020,   /* U+1079A */
  0x0020,   /* U+1079B */
  0x0020,   /* U+1079C */
  0x0020,   /* U+1079D */
  0x0020,   /* U+1079E */
  0x0020,   /* U+1079F */
  0x0020,   /* U+107A0 */
  0x0020,   /* U+107A1 */
  0x0020,   /* U+107A2 */
  0x0020,   /* U+107A3 */
  0x0020,   /* U+107A4 */
  0x0020,   /* U+107A5 */
  0x0020,   /* U+107A6 */
  0x0020,   /* U+107A7 */
  0x0020,   /* U+107A8 */
  0x0020,   /* U+107A9 */
  0x0020,   /* U+107AA */
  0x0020,   /* U+107AB */
  0x0020,   /* U+107AC */
  0x0020,   /* U+107AD */
  0x0020,   /* U+107AE */
  0x0020,   /* U+107AF */
  0x0020,   /* U+107B0 */
  0x0020,   /* U+107B1 */
  0x0020,   /* U+107B2 */
  0x0020,   /* U+107B3 */
  0x0020,   /* U+107B4 */
  0x0020,   /* U+107B5 */
  0x0020,   /* U+107B6 */
  0x0020,   /* U+107B7 */
  0x0020,   /* U+107B8 */
  0x0020,   /* U+107B9 */
  0x0020,   /* U+107BA */
  0x0020,   /* U+107BB */
  0x0020,   /* U+107BC */
  0x0020,   /* U+107BD */
  0x0020,   /* U+107BE */
  0x0020,   /* U+107BF */
  0x0020,   /* U+107C0 */
  0x0020,   /* U+107C1 */
  0x0020,   /* U+107C2 */
  0x0020,   /* U+107C3 */
  0x0020,   /* U+107C4 */
  0x0020,   /* U+107C5 */
  0x0020,   /* U+107C6 */
  0x0020,   /* U+107C7 */
  0x0020,   /* U+107C8 */
  0x0020,   /* U+107C9 */
  0x0020,   /* U+107CA */
  0x0020,   /* U+107CB */
  0x0020,   /* U+107CC */
  0x0020,   /* U+107CD */
  0x0020,   /* U+107CE */
  0x0020,   /* U+107CF */
  0x0020,   /* U+107D0 */
  0x0020,   /* U+107D1 */
  0x0020,   /* U+107D2 */
  0x0020,   /* U+107D3 */
  0x0020,   /* U+107D4 */
  0x0020,   /* U+107D5 */
  0x0020,   /* U+107D6 */
  0x0020,   /* U+107D7 */
  0x0020,   /* U+107D8 */
  0x0020,   /* U+107D9 */
  0x0020,   /* U+107DA */
  0x0020,   /* U+107DB */
  0x0020,   /* U+107DC */
  0x0020,   /* U+107DD */
  0x0020,   /* U+107DE */
  0x0020,   /* U+107DF */
  0x0020,   /* U+107E0 */
  0x0020,   /* U+107E1 */
  0x0020,   /* U+107E2 */
  0x0020,   /* U+107E3 */
  0x0020,   /* U+107E4 */
  0x0020,   /* U+107E5 */
  0x0020,   /* U+107E6 */
  0x0020,   /* U+107E7 */
  0x0020,   /* U+107E8 */
  0x0020,   /* U+107E9 */
  0x0020,   /* U+107EA */
  0x0020,   /* U+107EB */
  0x0020,   /* U+107EC */
  0x0020,   /* U+107ED */
  0x0020,   /* U+107EE */
  0x0020,   /* U+107EF */
  0x0020,   /* U+107F0 */
  0x0020,   /* U+107F1 */
  0x0020,   /* U+107F2 */
  0x0020,   /* U+107F3 */
  0x0020,   /* U+107F4 */
  0x0020,   /* U+107F5 */
  0x0020,   /* U+107F6 */
  0x0020,   /* U+107F7 */
  0x0020,   /* U+107F8 */
  0x0020,   /* U+107F9 */
  0x0020,   /* U+107FA */
  0x0020,   /* U+107FB */
  0x0020,   /* U+107FC */
  0x0020,   /* U+107FD */
  0x0020,   /* U+107FE */
  0x0020,   /* U+107FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10700 */
  0x0002,   /* U+10701 */
  0x0002,   /* U+10702 */
  0x0002,   /* U+10703 */
  0x0002,   /* U+10704 */
  0x0002,   /* U+10705 */
  0x0002,   /* U+10706 */
  0x0002,   /* U+10707 */
  0x0002,   /* U+10708 */
  0x0002,   /* U+10709 */
  0x0002,   /* U+1070A */
  0x0002,   /* U+1070B */
  0x0002,   /* U+1070C */
  0x0002,   /* U+1070D */
  0x0002,   /* U+1070E */
  0x0002,   /* U+1070F */
  0x0002,   /* U+10710 */
  0x0002,   /* U+10711 */
  0x0002,   /* U+10712 */
  0x0002,   /* U+10713 */
  0x0002,   /* U+10714 */
  0x0002,   /* U+10715 */
  0x0002,   /* U+10716 */
  0x0002,   /* U+10717 */
  0x0002,   /* U+10718 */
  0x0002,   /* U+10719 */
  0x0002,   /* U+1071A */
  0x0002,   /* U+1071B */
  0x0002,   /* U+1071C */
  0x0002,   /* U+1071D */
  0x0002,   /* U+1071E */
  0x0002,   /* U+1071F */
  0x0002,   /* U+10720 */
  0x0002,   /* U+10721 */
  0x0002,   /* U+10722 */
  0x0002,   /* U+10723 */
  0x0002,   /* U+10724 */
  0x0002,   /* U+10725 */
  0x0002,   /* U+10726 */
  0x0002,   /* U+10727 */
  0x0002,   /* U+10728 */
  0x0002,   /* U+10729 */
  0x0002,   /* U+1072A */
  0x0002,   /* U+1072B */
  0x0002,   /* U+1072C */
  0x0002,   /* U+1072D */
  0x0002,   /* U+1072E */
  0x0002,   /* U+1072F */
  0x0002,   /* U+10730 */
  0x0002,   /* U+10731 */
  0x0002,   /* U+10732 */
  0x0002,   /* U+10733 */
  0x0002,   /* U+10734 */
  0x0002,   /* U+10735 */
  0x0002,   /* U+10736 */
  0x0002,   /* U+10737 */
  0x0002,   /* U+10738 */
  0x0002,   /* U+10739 */
  0x0002,   /* U+1073A */
  0x0002,   /* U+1073B */
  0x0002,   /* U+1073C */
  0x0002,   /* U+1073D */
  0x0002,   /* U+1073E */
  0x0002,   /* U+1073F */
  0x0002,   /* U+10740 */
  0x0002,   /* U+10741 */
  0x0002,   /* U+10742 */
  0x0002,   /* U+10743 */
  0x0002,   /* U+10744 */
  0x0002,   /* U+10745 */
  0x0002,   /* U+10746 */
  0x0002,   /* U+10747 */
  0x0002,   /* U+10748 */
  0x0002,   /* U+10749 */
  0x0002,   /* U+1074A */
  0x0002,   /* U+1074B */
  0x0002,   /* U+1074C */
  0x0002,   /* U+1074D */
  0x0002,   /* U+1074E */
  0x0002,   /* U+1074F */
  0x0002,   /* U+10750 */
  0x0002,   /* U+10751 */
  0x0002,   /* U+10752 */
  0x0002,   /* U+10753 */
  0x0002,   /* U+10754 */
  0x0002,   /* U+10755 */
  0x0002,   /* U+10756 */
  0x0002,   /* U+10757 */
  0x0002,   /* U+10758 */
  0x0002,   /* U+10759 */
  0x0002,   /* U+1075A */
  0x0002,   /* U+1075B */
  0x0002,   /* U+1075C */
  0x0002,   /* U+1075D */
  0x0002,   /* U+1075E */
  0x0002,   /* U+1075F */
  0x0002,   /* U+10760 */
  0x0002,   /* U+10761 */
  0x0002,   /* U+10762 */
  0x0002,   /* U+10763 */
  0x0002,   /* U+10764 */
  0x0002,   /* U+10765 */
  0x0002,   /* U+10766 */
  0x0002,   /* U+10767 */
  0x0002,   /* U+10768 */
  0x0002,   /* U+10769 */
  0x0002,   /* U+1076A */
  0x0002,   /* U+1076B */
  0x0002,   /* U+1076C */
  0x0002,   /* U+1076D */
  0x0002,   /* U+1076E */
  0x0002,   /* U+1076F */
  0x0002,   /* U+10770 */
  0x0002,   /* U+10771 */
  0x0002,   /* U+10772 */
  0x0002,   /* U+10773 */
  0x0002,   /* U+10774 */
  0x0002,   /* U+10775 */
  0x0002,   /* U+10776 */
  0x0002,   /* U+10777 */
  0x0002,   /* U+10778 */
  0x0002,   /* U+10779 */
  0x0002,   /* U+1077A */
  0x0002,   /* U+1077B */
  0x0002,   /* U+1077C */
  0x0002,   /* U+1077D */
  0x0002,   /* U+1077E */
  0x0002,   /* U+1077F */
  0x0002,   /* U+10780 */
  0x0002,   /* U+10781 */
  0x0002,   /* U+10782 */
  0x0002,   /* U+10783 */
  0x0002,   /* U+10784 */
  0x0002,   /* U+10785 */
  0x0002,   /* U+10786 */
  0x0002,   /* U+10787 */
  0x0002,   /* U+10788 */
  0x0002,   /* U+10789 */
  0x0002,   /* U+1078A */
  0x0002,   /* U+1078B */
  0x0002,   /* U+1078C */
  0x0002,   /* U+1078D */
  0x0002,   /* U+1078E */
  0x0002,   /* U+1078F */
  0x0002,   /* U+10790 */
  0x0002,   /* U+10791 */
  0x0002,   /* U+10792 */
  0x0002,   /* U+10793 */
  0x0002,   /* U+10794 */
  0x0002,   /* U+10795 */
  0x0002,   /* U+10796 */
  0x0002,   /* U+10797 */
  0x0002,   /* U+10798 */
  0x0002,   /* U+10799 */
  0x0002,   /* U+1079A */
  0x0002,   /* U+1079B */
  0x0002,   /* U+1079C */
  0x0002,   /* U+1079D */
  0x0002,   /* U+1079E */
  0x0002,   /* U+1079F */
  0x0002,   /* U+107A0 */
  0x0002,   /* U+107A1 */
  0x0002,   /* U+107A2 */
  0x0002,   /* U+107A3 */
  0x0002,   /* U+107A4 */
  0x0002,   /* U+107A5 */
  0x0002,   /* U+107A6 */
  0x0002,   /* U+107A7 */
  0x0002,   /* U+107A8 */
  0x0002,   /* U+107A9 */
  0x0002,   /* U+107AA */
  0x0002,   /* U+107AB */
  0x0002,   /* U+107AC */
  0x0002,   /* U+107AD */
  0x0002,   /* U+107AE */
  0x0002,   /* U+107AF */
  0x0002,   /* U+107B0 */
  0x0002,   /* U+107B1 */
  0x0002,   /* U+107B2 */
  0x0002,   /* U+107B3 */
  0x0002,   /* U+107B4 */
  0x0002,   /* U+107B5 */
  0x0002,   /* U+107B6 */
  0x0002,   /* U+107B7 */
  0x0002,   /* U+107B8 */
  0x0002,   /* U+107B9 */
  0x0002,   /* U+107BA */
  0x0002,   /* U+107BB */
  0x0002,   /* U+107BC */
  0x0002,   /* U+107BD */
  0x0002,   /* U+107BE */
  0x0002,   /* U+107BF */
  0x0002,   /* U+107C0 */
  0x0002,   /* U+107C1 */
  0x0002,   /* U+107C2 */
  0x0002,   /* U+107C3 */
  0x0002,   /* U+107C4 */
  0x0002,   /* U+107C5 */
  0x0002,   /* U+107C6 */
  0x0002,   /* U+107C7 */
  0x0002,   /* U+107C8 */
  0x0002,   /* U+107C9 */
  0x0002,   /* U+107CA */
  0x0002,   /* U+107CB */
  0x0002,   /* U+107CC */
  0x0002,   /* U+107CD */
  0x0002,   /* U+107CE */
  0x0002,   /* U+107CF */
  0x0002,   /* U+107D0 */
  0x0002,   /* U+107D1 */
  0x0002,   /* U+107D2 */
  0x0002,   /* U+107D3 */
  0x0002,   /* U+107D4 */
  0x0002,   /* U+107D5 */
  0x0002,   /* U+107D6 */
  0x0002,   /* U+107D7 */
  0x0002,   /* U+107D8 */
  0x0002,   /* U+107D9 */
  0x0002,   /* U+107DA */
  0x0002,   /* U+107DB */
  0x0002,   /* U+107DC */
  0x0002,   /* U+107DD */
  0x0002,   /* U+107DE */
  0x0002,   /* U+107DF */
  0x0002,   /* U+107E0 */
  0x0002,   /* U+107E1 */
  0x0002,   /* U+107E2 */
  0x0002,   /* U+107E3 */
  0x0002,   /* U+107E4 */
  0x0002,   /* U+107E5 */
  0x0002,   /* U+107E6 */
  0x0002,   /* U+107E7 */
  0x0002,   /* U+107E8 */
  0x0002,   /* U+107E9 */
  0x0002,   /* U+107EA */
  0x0002,   /* U+107EB */
  0x0002,   /* U+107EC */
  0x0002,   /* U+107ED */
  0x0002,   /* U+107EE */
  0x0002,   /* U+107EF */
  0x0002,   /* U+107F0 */
  0x0002,   /* U+107F1 */
  0x0002,   /* U+107F2 */
  0x0002,   /* U+107F3 */
  0x0002,   /* U+107F4 */
  0x0002,   /* U+107F5 */
  0x0002,   /* U+107F6 */
  0x0002,   /* U+107F7 */
  0x0002,   /* U+107F8 */
  0x0002,   /* U+107F9 */
  0x0002,   /* U+107FA */
  0x0002,   /* U+107FB */
  0x0002,   /* U+107FC */
  0x0002,   /* U+107FD */
  0x0002,   /* U+107FE */
  0x0002,   /* U+107FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10700 */
  0x0000,   /* U+10701 */
  0x0000,   /* U+10702 */
  0x0000,   /* U+10703 */
  0x0000,   /* U+10704 */
  0x0000,   /* U+10705 */
  0x0000,   /* U+10706 */
  0x0000,   /* U+10707 */
  0x0000,   /* U+10708 */
  0x0000,   /* U+10709 */
  0x0000,   /* U+1070A */
  0x0000,   /* U+1070B */
  0x0000,   /* U+1070C */
  0x0000,   /* U+1070D */
  0x0000,   /* U+1070E */
  0x0000,   /* U+1070F */
  0x0000,   /* U+10710 */
  0x0000,   /* U+10711 */
  0x0000,   /* U+10712 */
  0x0000,   /* U+10713 */
  0x0000,   /* U+10714 */
  0x0000,   /* U+10715 */
  0x0000,   /* U+10716 */
  0x0000,   /* U+10717 */
  0x0000,   /* U+10718 */
  0x0000,   /* U+10719 */
  0x0000,   /* U+1071A */
  0x0000,   /* U+1071B */
  0x0000,   /* U+1071C */
  0x0000,   /* U+1071D */
  0x0000,   /* U+1071E */
  0x0000,   /* U+1071F */
  0x0000,   /* U+10720 */
  0x0000,   /* U+10721 */
  0x0000,   /* U+10722 */
  0x0000,   /* U+10723 */
  0x0000,   /* U+10724 */
  0x0000,   /* U+10725 */
  0x0000,   /* U+10726 */
  0x0000,   /* U+10727 */
  0x0000,   /* U+10728 */
  0x0000,   /* U+10729 */
  0x0000,   /* U+1072A */
  0x0000,   /* U+1072B */
  0x0000,   /* U+1072C */
  0x0000,   /* U+1072D */
  0x0000,   /* U+1072E */
  0x0000,   /* U+1072F */
  0x0000,   /* U+10730 */
  0x0000,   /* U+10731 */
  0x0000,   /* U+10732 */
  0x0000,   /* U+10733 */
  0x0000,   /* U+10734 */
  0x0000,   /* U+10735 */
  0x0000,   /* U+10736 */
  0x8737,   /* U+10737 */
  0x8738,   /* U+10738 */
  0x8739,   /* U+10739 */
  0x873A,   /* U+1073A */
  0x873B,   /* U+1073B */
  0x873C,   /* U+1073C */
  0x873D,   /* U+1073D */
  0x873E,   /* U+1073E */
  0x873F,   /* U+1073F */
  0x0000,   /* U+10740 */
  0x0000,   /* U+10741 */
  0x0000,   /* U+10742 */
  0x0000,   /* U+10743 */
  0x0000,   /* U+10744 */
  0x0000,   /* U+10745 */
  0x0000,   /* U+10746 */
  0x0000,   /* U+10747 */
  0x0000,   /* U+10748 */
  0x0000,   /* U+10749 */
  0x0000,   /* U+1074A */
  0x0000,   /* U+1074B */
  0x0000,   /* U+1074C */
  0x0000,   /* U+1074D */
  0x0000,   /* U+1074E */
  0x0000,   /* U+1074F */
  0x0000,   /* U+10750 */
  0x0000,   /* U+10751 */
  0x0000,   /* U+10752 */
  0x0000,   /* U+10753 */
  0x0000,   /* U+10754 */
  0x0000,   /* U+10755 */
  0x8756,   /* U+10756 */
  0x8757,   /* U+10757 */
  0x8758,   /* U+10758 */
  0x8759,   /* U+10759 */
  0x875A,   /* U+1075A */
  0x875B,   /* U+1075B */
  0x875C,   /* U+1075C */
  0x875D,   /* U+1075D */
  0x875E,   /* U+1075E */
  0x875F,   /* U+1075F */
  0x0000,   /* U+10760 */
  0x0000,   /* U+10761 */
  0x0000,   /* U+10762 */
  0x0000,   /* U+10763 */
  0x0000,   /* U+10764 */
  0x0000,   /* U+10765 */
  0x0000,   /* U+10766 */
  0x0000,   /* U+10767 */
  0x8768,   /* U+10768 */
  0x8769,   /* U+10769 */
  0x876A,   /* U+1076A */
  0x876B,   /* U+1076B */
  0x876C,   /* U+1076C */
  0x876D,   /* U+1076D */
  0x876E,   /* U+1076E */
  0x876F,   /* U+1076F */
  0x8770,   /* U+10770 */
  0x8771,   /* U+10771 */
  0x8772,   /* U+10772 */
  0x8773,   /* U+10773 */
  0x8774,   /* U+10774 */
  0x8775,   /* U+10775 */
  0x8776,   /* U+10776 */
  0x8777,   /* U+10777 */
  0x8778,   /* U+10778 */
  0x8779,   /* U+10779 */
  0x877A,   /* U+1077A */
  0x877B,   /* U+1077B */
  0x877C,   /* U+1077C */
  0x877D,   /* U+1077D */
  0x877E,   /* U+1077E */
  0x877F,   /* U+1077F */
  0x8780,   /* U+10780 */
  0x8781,   /* U+10781 */
  0x8782,   /* U+10782 */
  0x8783,   /* U+10783 */
  0x8784,   /* U+10784 */
  0x8785,   /* U+10785 */
  0x8786,   /* U+10786 */
  0x8787,   /* U+10787 */
  0x8788,   /* U+10788 */
  0x8789,   /* U+10789 */
  0x878A,   /* U+1078A */
  0x878B,   /* U+1078B */
  0x878C,   /* U+1078C */
  0x878D,   /* U+1078D */
  0x878E,   /* U+1078E */
  0x878F,   /* U+1078F */
  0x8790,   /* U+10790 */
  0x8791,   /* U+10791 */
  0x8792,   /* U+10792 */
  0x8793,   /* U+10793 */
  0x8794,   /* U+10794 */
  0x8795,   /* U+10795 */
  0x8796,   /* U+10796 */
  0x8797,   /* U+10797 */
  0x8798,   /* U+10798 */
  0x8799,   /* U+10799 */
  0x879A,   /* U+1079A */
  0x879B,   /* U+1079B */
  0x879C,   /* U+1079C */
  0x879D,   /* U+1079D */
  0x879E,   /* U+1079E */
  0x879F,   /* U+1079F */
  0x87A0,   /* U+107A0 */
  0x87A1,   /* U+107A1 */
  0x87A2,   /* U+107A2 */
  0x87A3,   /* U+107A3 */
  0x87A4,   /* U+107A4 */
  0x87A5,   /* U+107A5 */
  0x87A6,   /* U+107A6 */
  0x87A7,   /* U+107A7 */
  0x87A8,   /* U+107A8 */
  0x87A9,   /* U+107A9 */
  0x87AA,   /* U+107AA */
  0x87AB,   /* U+107AB */
  0x87AC,   /* U+107AC */
  0x87AD,   /* U+107AD */
  0x87AE,   /* U+107AE */
  0x87AF,   /* U+107AF */
  0x87B0,   /* U+107B0 */
  0x87B1,   /* U+107B1 */
  0x87B2,   /* U+107B2 */
  0x87B3,   /* U+107B3 */
  0x87B4,   /* U+107B4 */
  0x87B5,   /* U+107B5 */
  0x87B6,   /* U+107B6 */
  0x87B7,   /* U+107B7 */
  0x87B8,   /* U+107B8 */
  0x87B9,   /* U+107B9 */
  0x87BA,   /* U+107BA */
  0x87BB,   /* U+107BB */
  0x87BC,   /* U+107BC */
  0x87BD,   /* U+107BD */
  0x87BE,   /* U+107BE */
  0x87BF,   /* U+107BF */
  0x87C0,   /* U+107C0 */
  0x87C1,   /* U+107C1 */
  0x87C2,   /* U+107C2 */
  0x87C3,   /* U+107C3 */
  0x87C4,   /* U+107C4 */
  0x87C5,   /* U+107C5 */
  0x87C6,   /* U+107C6 */
  0x87C7,   /* U+107C7 */
  0x87C8,   /* U+107C8 */
  0x87C9,   /* U+107C9 */
  0x87CA,   /* U+107CA */
  0x87CB,   /* U+107CB */
  0x87CC,   /* U+107CC */
  0x87CD,   /* U+107CD */
  0x87CE,   /* U+107CE */
  0x87CF,   /* U+107CF */
  0x87D0,   /* U+107D0 */
  0x87D1,   /* U+107D1 */
  0x87D2,   /* U+107D2 */
  0x87D3,   /* U+107D3 */
  0x87D4,   /* U+107D4 */
  0x87D5,   /* U+107D5 */
  0x87D6,   /* U+107D6 */
  0x87D7,   /* U+107D7 */
  0x87D8,   /* U+107D8 */
  0x87D9,   /* U+107D9 */
  0x87DA,   /* U+107DA */
  0x87DB,   /* U+107DB */
  0x87DC,   /* U+107DC */
  0x87DD,   /* U+107DD */
  0x87DE,   /* U+107DE */
  0x87DF,   /* U+107DF */
  0x87E0,   /* U+107E0 */
  0x87E1,   /* U+107E1 */
  0x87E2,   /* U+107E2 */
  0x87E3,   /* U+107E3 */
  0x87E4,   /* U+107E4 */
  0x87E5,   /* U+107E5 */
  0x87E6,   /* U+107E6 */
  0x87E7,   /* U+107E7 */
  0x87E8,   /* U+107E8 */
  0x87E9,   /* U+107E9 */
  0x87EA,   /* U+107EA */
  0x87EB,   /* U+107EB */
  0x87EC,   /* U+107EC */
  0x87ED,   /* U+107ED */
  0x87EE,   /* U+107EE */
  0x87EF,   /* U+107EF */
  0x87F0,   /* U+107F0 */
  0x87F1,   /* U+107F1 */
  0x87F2,   /* U+107F2 */
  0x87F3,   /* U+107F3 */
  0x87F4,   /* U+107F4 */
  0x87F5,   /* U+107F5 */
  0x87F6,   /* U+107F6 */
  0x87F7,   /* U+107F7 */
  0x87F8,   /* U+107F8 */
  0x87F9,   /* U+107F9 */
  0x87FA,   /* U+107FA */
  0x87FB,   /* U+107FB */
  0x87FC,   /* U+107FC */
  0x87FD,   /* U+107FD */
  0x87FE,   /* U+107FE */
  0x87FF,   /* U+107FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10700 */
  0x0000,   /* U+10701 */
  0x0000,   /* U+10702 */
  0x0000,   /* U+10703 */
  0x0000,   /* U+10704 */
  0x0000,   /* U+10705 */
  0x0000,   /* U+10706 */
  0x0000,   /* U+10707 */
  0x0000,   /* U+10708 */
  0x0000,   /* U+10709 */
  0x0000,   /* U+1070A */
  0x0000,   /* U+1070B */
  0x0000,   /* U+1070C */
  0x0000,   /* U+1070D */
  0x0000,   /* U+1070E */
  0x0000,   /* U+1070F */
  0x0000,   /* U+10710 */
  0x0000,   /* U+10711 */
  0x0000,   /* U+10712 */
  0x0000,   /* U+10713 */
  0x0000,   /* U+10714 */
  0x0000,   /* U+10715 */
  0x0000,   /* U+10716 */
  0x0000,   /* U+10717 */
  0x0000,   /* U+10718 */
  0x0000,   /* U+10719 */
  0x0000,   /* U+1071A */
  0x0000,   /* U+1071B */
  0x0000,   /* U+1071C */
  0x0000,   /* U+1071D */
  0x0000,   /* U+1071E */
  0x0000,   /* U+1071F */
  0x0000,   /* U+10720 */
  0x0000,   /* U+10721 */
  0x0000,   /* U+10722 */
  0x0000,   /* U+10723 */
  0x0000,   /* U+10724 */
  0x0000,   /* U+10725 */
  0x0000,   /* U+10726 */
  0x0000,   /* U+10727 */
  0x0000,   /* U+10728 */
  0x0000,   /* U+10729 */
  0x0000,   /* U+1072A */
  0x0000,   /* U+1072B */
  0x0000,   /* U+1072C */
  0x0000,   /* U+1072D */
  0x0000,   /* U+1072E */
  0x0000,   /* U+1072F */
  0x0000,   /* U+10730 */
  0x0000,   /* U+10731 */
  0x0000,   /* U+10732 */
  0x0000,   /* U+10733 */
  0x0000,   /* U+10734 */
  0x0000,   /* U+10735 */
  0x0000,   /* U+10736 */
  0x0000,   /* U+10737 */
  0x0000,   /* U+10738 */
  0x0000,   /* U+10739 */
  0x0000,   /* U+1073A */
  0x0000,   /* U+1073B */
  0x0000,   /* U+1073C */
  0x0000,   /* U+1073D */
  0x0000,   /* U+1073E */
  0x0000,   /* U+1073F */
  0x0000,   /* U+10740 */
  0x0000,   /* U+10741 */
  0x0000,   /* U+10742 */
  0x0000,   /* U+10743 */
  0x0000,   /* U+10744 */
  0x0000,   /* U+10745 */
  0x0000,   /* U+10746 */
  0x0000,   /* U+10747 */
  0x0000,   /* U+10748 */
  0x0000,   /* U+10749 */
  0x0000,   /* U+1074A */
  0x0000,   /* U+1074B */
  0x0000,   /* U+1074C */
  0x0000,   /* U+1074D */
  0x0000,   /* U+1074E */
  0x0000,   /* U+1074F */
  0x0000,   /* U+10750 */
  0x0000,   /* U+10751 */
  0x0000,   /* U+10752 */
  0x0000,   /* U+10753 */
  0x0000,   /* U+10754 */
  0x0000,   /* U+10755 */
  0x0000,   /* U+10756 */
  0x0000,   /* U+10757 */
  0x0000,   /* U+10758 */
  0x0000,   /* U+10759 */
  0x0000,   /* U+1075A */
  0x0000,   /* U+1075B */
  0x0000,   /* U+1075C */
  0x0000,   /* U+1075D */
  0x0000,   /* U+1075E */
  0x0000,   /* U+1075F */
  0x0000,   /* U+10760 */
  0x0000,   /* U+10761 */
  0x0000,   /* U+10762 */
  0x0000,   /* U+10763 */
  0x0000,   /* U+10764 */
  0x0000,   /* U+10765 */
  0x0000,   /* U+10766 */
  0x0000,   /* U+10767 */
  0x0000,   /* U+10768 */
  0x0000,   /* U+10769 */
  0x0000,   /* U+1076A */
  0x0000,   /* U+1076B */
  0x0000,   /* U+1076C */
  0x0000,   /* U+1076D */
  0x0000,   /* U+1076E */
  0x0000,   /* U+1076F */
  0x0000,   /* U+10770 */
  0x0000,   /* U+10771 */
  0x0000,   /* U+10772 */
  0x0000,   /* U+10773 */
  0x0000,   /* U+10774 */
  0x0000,   /* U+10775 */
  0x0000,   /* U+10776 */
  0x0000,   /* U+10777 */
  0x0000,   /* U+10778 */
  0x0000,   /* U+10779 */
  0x0000,   /* U+1077A */
  0x0000,   /* U+1077B */
  0x0000,   /* U+1077C */
  0x0000,   /* U+1077D */
  0x0000,   /* U+1077E */
  0x0000,   /* U+1077F */
  0x0000,   /* U+10780 */
  0x0000,   /* U+10781 */
  0x0000,   /* U+10782 */
  0x0000,   /* U+10783 */
  0x0000,   /* U+10784 */
  0x0000,   /* U+10785 */
  0x0000,   /* U+10786 */
  0x0000,   /* U+10787 */
  0x0000,   /* U+10788 */
  0x0000,   /* U+10789 */
  0x0000,   /* U+1078A */
  0x0000,   /* U+1078B */
  0x0000,   /* U+1078C */
  0x0000,   /* U+1078D */
  0x0000,   /* U+1078E */
  0x0000,   /* U+1078F */
  0x0000,   /* U+10790 */
  0x0000,   /* U+10791 */
  0x0000,   /* U+10792 */
  0x0000,   /* U+10793 */
  0x0000,   /* U+10794 */
  0x0000,   /* U+10795 */
  0x0000,   /* U+10796 */
  0x0000,   /* U+10797 */
  0x0000,   /* U+10798 */
  0x0000,   /* U+10799 */
  0x0000,   /* U+1079A */
  0x0000,   /* U+1079B */
  0x0000,   /* U+1079C */
  0x0000,   /* U+1079D */
  0x0000,   /* U+1079E */
  0x0000,   /* U+1079F */
  0x0000,   /* U+107A0 */
  0x0000,   /* U+107A1 */
  0x0000,   /* U+107A2 */
  0x0000,   /* U+107A3 */
  0x0000,   /* U+107A4 */
  0x0000,   /* U+107A5 */
  0x0000,   /* U+107A6 */
  0x0000,   /* U+107A7 */
  0x0000,   /* U+107A8 */
  0x0000,   /* U+107A9 */
  0x0000,   /* U+107AA */
  0x0000,   /* U+107AB */
  0x0000,   /* U+107AC */
  0x0000,   /* U+107AD */
  0x0000,   /* U+107AE */
  0x0000,   /* U+107AF */
  0x0000,   /* U+107B0 */
  0x0000,   /* U+107B1 */
  0x0000,   /* U+107B2 */
  0x0000,   /* U+107B3 */
  0x0000,   /* U+107B4 */
  0x0000,   /* U+107B5 */
  0x0000,   /* U+107B6 */
  0x0000,   /* U+107B7 */
  0x0000,   /* U+107B8 */
  0x0000,   /* U+107B9 */
  0x0000,   /* U+107BA */
  0x0000,   /* U+107BB */
  0x0000,   /* U+107BC */
  0x0000,   /* U+107BD */
  0x0000,   /* U+107BE */
  0x0000,   /* U+107BF */
  0x0000,   /* U+107C0 */
  0x0000,   /* U+107C1 */
  0x0000,   /* U+107C2 */
  0x0000,   /* U+107C3 */
  0x0000,   /* U+107C4 */
  0x0000,   /* U+107C5 */
  0x0000,   /* U+107C6 */
  0x0000,   /* U+107C7 */
  0x0000,   /* U+107C8 */
  0x0000,   /* U+107C9 */
  0x0000,   /* U+107CA */
  0x0000,   /* U+107CB */
  0x0000,   /* U+107CC */
  0x0000,   /* U+107CD */
  0x0000,   /* U+107CE */
  0x0000,   /* U+107CF */
  0x0000,   /* U+107D0 */
  0x0000,   /* U+107D1 */
  0x0000,   /* U+107D2 */
  0x0000,   /* U+107D3 */
  0x0000,   /* U+107D4 */
  0x0000,   /* U+107D5 */
  0x0000,   /* U+107D6 */
  0x0000,   /* U+107D7 */
  0x0000,   /* U+107D8 */
  0x0000,   /* U+107D9 */
  0x0000,   /* U+107DA */
  0x0000,   /* U+107DB */
  0x0000,   /* U+107DC */
  0x0000,   /* U+107DD */
  0x0000,   /* U+107DE */
  0x0000,   /* U+107DF */
  0x0000,   /* U+107E0 */
  0x0000,   /* U+107E1 */
  0x0000,   /* U+107E2 */
  0x0000,   /* U+107E3 */
  0x0000,   /* U+107E4 */
  0x0000,   /* U+107E5 */
  0x0000,   /* U+107E6 */
  0x0000,   /* U+107E7 */
  0x0000,   /* U+107E8 */
  0x0000,   /* U+107E9 */
  0x0000,   /* U+107EA */
  0x0000,   /* U+107EB */
  0x0000,   /* U+107EC */
  0x0000,   /* U+107ED */
  0x0000,   /* U+107EE */
  0x0000,   /* U+107EF */
  0x0000,   /* U+107F0 */
  0x0000,   /* U+107F1 */
  0x0000,   /* U+107F2 */
  0x0000,   /* U+107F3 */
  0x0000,   /* U+107F4 */
  0x0000,   /* U+107F5 */
  0x0000,   /* U+107F6 */
  0x0000,   /* U+107F7 */
  0x0000,   /* U+107F8 */
  0x0000,   /* U+107F9 */
  0x0000,   /* U+107FA */
  0x0000,   /* U+107FB */
  0x0000,   /* U+107FC */
  0x0000,   /* U+107FD */
  0x0000,   /* U+107FE */
  0x0000,   /* U+107FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10700 */
  0x0000,   /* U+10701 */
  0x0000,   /* U+10702 */
  0x0000,   /* U+10703 */
  0x0000,   /* U+10704 */
  0x0000,   /* U+10705 */
  0x0000,   /* U+10706 */
  0x0000,   /* U+10707 */
  0x0000,   /* U+10708 */
  0x0000,   /* U+10709 */
  0x0000,   /* U+1070A */
  0x0000,   /* U+1070B */
  0x0000,   /* U+1070C */
  0x0000,   /* U+1070D */
  0x0000,   /* U+1070E */
  0x0000,   /* U+1070F */
  0x0000,   /* U+10710 */
  0x0000,   /* U+10711 */
  0x0000,   /* U+10712 */
  0x0000,   /* U+10713 */
  0x0000,   /* U+10714 */
  0x0000,   /* U+10715 */
  0x0000,   /* U+10716 */
  0x0000,   /* U+10717 */
  0x0000,   /* U+10718 */
  0x0000,   /* U+10719 */
  0x0000,   /* U+1071A */
  0x0000,   /* U+1071B */
  0x0000,   /* U+1071C */
  0x0000,   /* U+1071D */
  0x0000,   /* U+1071E */
  0x0000,   /* U+1071F */
  0x0000,   /* U+10720 */
  0x0000,   /* U+10721 */
  0x0000,   /* U+10722 */
  0x0000,   /* U+10723 */
  0x0000,   /* U+10724 */
  0x0000,   /* U+10725 */
  0x0000,   /* U+10726 */
  0x0000,   /* U+10727 */
  0x0000,   /* U+10728 */
  0x0000,   /* U+10729 */
  0x0000,   /* U+1072A */
  0x0000,   /* U+1072B */
  0x0000,   /* U+1072C */
  0x0000,   /* U+1072D */
  0x0000,   /* U+1072E */
  0x0000,   /* U+1072F */
  0x0000,   /* U+10730 */
  0x0000,   /* U+10731 */
  0x0000,   /* U+10732 */
  0x0000,   /* U+10733 */
  0x0000,   /* U+10734 */
  0x0000,   /* U+10735 */
  0x0000,   /* U+10736 */
  0x0000,   /* U+10737 */
  0x0000,   /* U+10738 */
  0x0000,   /* U+10739 */
  0x0000,   /* U+1073A */
  0x0000,   /* U+1073B */
  0x0000,   /* U+1073C */
  0x0000,   /* U+1073D */
  0x0000,   /* U+1073E */
  0x0000,   /* U+1073F */
  0x0000,   /* U+10740 */
  0x0000,   /* U+10741 */
  0x0000,   /* U+10742 */
  0x0000,   /* U+10743 */
  0x0000,   /* U+10744 */
  0x0000,   /* U+10745 */
  0x0000,   /* U+10746 */
  0x0000,   /* U+10747 */
  0x0000,   /* U+10748 */
  0x0000,   /* U+10749 */
  0x0000,   /* U+1074A */
  0x0000,   /* U+1074B */
  0x0000,   /* U+1074C */
  0x0000,   /* U+1074D */
  0x0000,   /* U+1074E */
  0x0000,   /* U+1074F */
  0x0000,   /* U+10750 */
  0x0000,   /* U+10751 */
  0x0000,   /* U+10752 */
  0x0000,   /* U+10753 */
  0x0000,   /* U+10754 */
  0x0000,   /* U+10755 */
  0x0000,   /* U+10756 */
  0x0000,   /* U+10757 */
  0x0000,   /* U+10758 */
  0x0000,   /* U+10759 */
  0x0000,   /* U+1075A */
  0x0000,   /* U+1075B */
  0x0000,   /* U+1075C */
  0x0000,   /* U+1075D */
  0x0000,   /* U+1075E */
  0x0000,   /* U+1075F */
  0x0000,   /* U+10760 */
  0x0000,   /* U+10761 */
  0x0000,   /* U+10762 */
  0x0000,   /* U+10763 */
  0x0000,   /* U+10764 */
  0x0000,   /* U+10765 */
  0x0000,   /* U+10766 */
  0x0000,   /* U+10767 */
  0x0000,   /* U+10768 */
  0x0000,   /* U+10769 */
  0x0000,   /* U+1076A */
  0x0000,   /* U+1076B */
  0x0000,   /* U+1076C */
  0x0000,   /* U+1076D */
  0x0000,   /* U+1076E */
  0x0000,   /* U+1076F */
  0x0000,   /* U+10770 */
  0x0000,   /* U+10771 */
  0x0000,   /* U+10772 */
  0x0000,   /* U+10773 */
  0x0000,   /* U+10774 */
  0x0000,   /* U+10775 */
  0x0000,   /* U+10776 */
  0x0000,   /* U+10777 */
  0x0000,   /* U+10778 */
  0x0000,   /* U+10779 */
  0x0000,   /* U+1077A */
  0x0000,   /* U+1077B */
  0x0000,   /* U+1077C */
  0x0000,   /* U+1077D */
  0x0000,   /* U+1077E */
  0x0000,   /* U+1077F */
  0x0000,   /* U+10780 */
  0x0000,   /* U+10781 */
  0x0000,   /* U+10782 */
  0x0000,   /* U+10783 */
  0x0000,   /* U+10784 */
  0x0000,   /* U+10785 */
  0x0000,   /* U+10786 */
  0x0000,   /* U+10787 */
  0x0000,   /* U+10788 */
  0x0000,   /* U+10789 */
  0x0000,   /* U+1078A */
  0x0000,   /* U+1078B */
  0x0000,   /* U+1078C */
  0x0000,   /* U+1078D */
  0x0000,   /* U+1078E */
  0x0000,   /* U+1078F */
  0x0000,   /* U+10790 */
  0x0000,   /* U+10791 */
  0x0000,   /* U+10792 */
  0x0000,   /* U+10793 */
  0x0000,   /* U+10794 */
  0x0000,   /* U+10795 */
  0x0000,   /* U+10796 */
  0x0000,   /* U+10797 */
  0x0000,   /* U+10798 */
  0x0000,   /* U+10799 */
  0x0000,   /* U+1079A */
  0x0000,   /* U+1079B */
  0x0000,   /* U+1079C */
  0x0000,   /* U+1079D */
  0x0000,   /* U+1079E */
  0x0000,   /* U+1079F */
  0x0000,   /* U+107A0 */
  0x0000,   /* U+107A1 */
  0x0000,   /* U+107A2 */
  0x0000,   /* U+107A3 */
  0x0000,   /* U+107A4 */
  0x0000,   /* U+107A5 */
  0x0000,   /* U+107A6 */
  0x0000,   /* U+107A7 */
  0x0000,   /* U+107A8 */
  0x0000,   /* U+107A9 */
  0x0000,   /* U+107AA */
  0x0000,   /* U+107AB */
  0x0000,   /* U+107AC */
  0x0000,   /* U+107AD */
  0x0000,   /* U+107AE */
  0x0000,   /* U+107AF */
  0x0000,   /* U+107B0 */
  0x0000,   /* U+107B1 */
  0x0000,   /* U+107B2 */
  0x0000,   /* U+107B3 */
  0x0000,   /* U+107B4 */
  0x0000,   /* U+107B5 */
  0x0000,   /* U+107B6 */
  0x0000,   /* U+107B7 */
  0x0000,   /* U+107B8 */
  0x0000,   /* U+107B9 */
  0x0000,   /* U+107BA */
  0x0000,   /* U+107BB */
  0x0000,   /* U+107BC */
  0x0000,   /* U+107BD */
  0x0000,   /* U+107BE */
  0x0000,   /* U+107BF */
  0x0000,   /* U+107C0 */
  0x0000,   /* U+107C1 */
  0x0000,   /* U+107C2 */
  0x0000,   /* U+107C3 */
  0x0000,   /* U+107C4 */
  0x0000,   /* U+107C5 */
  0x0000,   /* U+107C6 */
  0x0000,   /* U+107C7 */
  0x0000,   /* U+107C8 */
  0x0000,   /* U+107C9 */
  0x0000,   /* U+107CA */
  0x0000,   /* U+107CB */
  0x0000,   /* U+107CC */
  0x0000,   /* U+107CD */
  0x0000,   /* U+107CE */
  0x0000,   /* U+107CF */
  0x0000,   /* U+107D0 */
  0x0000,   /* U+107D1 */
  0x0000,   /* U+107D2 */
  0x0000,   /* U+107D3 */
  0x0000,   /* U+107D4 */
  0x0000,   /* U+107D5 */
  0x0000,   /* U+107D6 */
  0x0000,   /* U+107D7 */
  0x0000,   /* U+107D8 */
  0x0000,   /* U+107D9 */
  0x0000,   /* U+107DA */
  0x0000,   /* U+107DB */
  0x0000,   /* U+107DC */
  0x0000,   /* U+107DD */
  0x0000,   /* U+107DE */
  0x0000,   /* U+107DF */
  0x0000,   /* U+107E0 */
  0x0000,   /* U+107E1 */
  0x0000,   /* U+107E2 */
  0x0000,   /* U+107E3 */
  0x0000,   /* U+107E4 */
  0x0000,   /* U+107E5 */
  0x0000,   /* U+107E6 */
  0x0000,   /* U+107E7 */
  0x0000,   /* U+107E8 */
  0x0000,   /* U+107E9 */
  0x0000,   /* U+107EA */
  0x0000,   /* U+107EB */
  0x0000,   /* U+107EC */
  0x0000,   /* U+107ED */
  0x0000,   /* U+107EE */
  0x0000,   /* U+107EF */
  0x0000,   /* U+107F0 */
  0x0000,   /* U+107F1 */
  0x0000,   /* U+107F2 */
  0x0000,   /* U+107F3 */
  0x0000,   /* U+107F4 */
  0x0000,   /* U+107F5 */
  0x0000,   /* U+107F6 */
  0x0000,   /* U+107F7 */
  0x0000,   /* U+107F8 */
  0x0000,   /* U+107F9 */
  0x0000,   /* U+107FA */
  0x0000,   /* U+107FB */
  0x0000,   /* U+107FC */
  0x0000,   /* U+107FD */
  0x0000,   /* U+107FE */
  0x0000,   /* U+107FF */
};

uint16 uca900_p108[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 2, 2, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 2, 2, 1, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x480F,   /* U+10800 */
  0x4810,   /* U+10801 */
  0x4811,   /* U+10802 */
  0x4812,   /* U+10803 */
  0x4813,   /* U+10804 */
  0x4814,   /* U+10805 */
  0xFBC2,   /* U+10806 */
  0xFBC2,   /* U+10807 */
  0x4815,   /* U+10808 */
  0xFBC2,   /* U+10809 */
  0x4816,   /* U+1080A */
  0x4817,   /* U+1080B */
  0x4818,   /* U+1080C */
  0x4819,   /* U+1080D */
  0x481A,   /* U+1080E */
  0x481B,   /* U+1080F */
  0x481C,   /* U+10810 */
  0x481D,   /* U+10811 */
  0x481E,   /* U+10812 */
  0x481F,   /* U+10813 */
  0x4820,   /* U+10814 */
  0x4821,   /* U+10815 */
  0x4822,   /* U+10816 */
  0x4823,   /* U+10817 */
  0x4824,   /* U+10818 */
  0x4825,   /* U+10819 */
  0x4826,   /* U+1081A */
  0x4827,   /* U+1081B */
  0x4828,   /* U+1081C */
  0x4829,   /* U+1081D */
  0x482A,   /* U+1081E */
  0x482B,   /* U+1081F */
  0x482C,   /* U+10820 */
  0x482D,   /* U+10821 */
  0x482E,   /* U+10822 */
  0x482F,   /* U+10823 */
  0x4830,   /* U+10824 */
  0x4831,   /* U+10825 */
  0x4832,   /* U+10826 */
  0x4833,   /* U+10827 */
  0x4834,   /* U+10828 */
  0x4835,   /* U+10829 */
  0x4836,   /* U+1082A */
  0x4837,   /* U+1082B */
  0x4838,   /* U+1082C */
  0x4839,   /* U+1082D */
  0x483A,   /* U+1082E */
  0x483B,   /* U+1082F */
  0x483C,   /* U+10830 */
  0x483D,   /* U+10831 */
  0x483E,   /* U+10832 */
  0x483F,   /* U+10833 */
  0x4840,   /* U+10834 */
  0x4841,   /* U+10835 */
  0xFBC2,   /* U+10836 */
  0x4842,   /* U+10837 */
  0x4843,   /* U+10838 */
  0xFBC2,   /* U+10839 */
  0xFBC2,   /* U+1083A */
  0xFBC2,   /* U+1083B */
  0x4844,   /* U+1083C */
  0xFBC2,   /* U+1083D */
  0xFBC2,   /* U+1083E */
  0x4845,   /* U+1083F */
  0x48F6,   /* U+10840 */
  0x48F7,   /* U+10841 */
  0x48F8,   /* U+10842 */
  0x48F9,   /* U+10843 */
  0x48FA,   /* U+10844 */
  0x48FB,   /* U+10845 */
  0x48FC,   /* U+10846 */
  0x48FD,   /* U+10847 */
  0x48FE,   /* U+10848 */
  0x48FF,   /* U+10849 */
  0x4900,   /* U+1084A */
  0x4901,   /* U+1084B */
  0x4902,   /* U+1084C */
  0x4903,   /* U+1084D */
  0x4904,   /* U+1084E */
  0x4905,   /* U+1084F */
  0x4906,   /* U+10850 */
  0x4907,   /* U+10851 */
  0x4908,   /* U+10852 */
  0x4909,   /* U+10853 */
  0x490A,   /* U+10854 */
  0x490B,   /* U+10855 */
  0xFBC2,   /* U+10856 */
  0x02D9,   /* U+10857 */
  0x1C3E,   /* U+10858 */
  0x1C3F,   /* U+10859 */
  0x1C40,   /* U+1085A */
  0x1B5E,   /* U+1085B */
  0x1B5F,   /* U+1085C */
  0x1B60,   /* U+1085D */
  0x1B61,   /* U+1085E */
  0x1B62,   /* U+1085F */
  0x48B5,   /* U+10860 */
  0x48B6,   /* U+10861 */
  0x48B7,   /* U+10862 */
  0x48B8,   /* U+10863 */
  0x48B9,   /* U+10864 */
  0x48BA,   /* U+10865 */
  0x48BB,   /* U+10866 */
  0x48BC,   /* U+10867 */
  0x48BD,   /* U+10868 */
  0x48BE,   /* U+10869 */
  0x48BF,   /* U+1086A */
  0x48C0,   /* U+1086B */
  0x48C1,   /* U+1086C */
  0x48C2,   /* U+1086D */
  0x48C2,   /* U+1086E */
  0x48C3,   /* U+1086F */
  0x48C4,   /* U+10870 */
  0x48C5,   /* U+10871 */
  0x48C6,   /* U+10872 */
  0x48C7,   /* U+10873 */
  0x48C8,   /* U+10874 */
  0x48C9,   /* U+10875 */
  0x48CA,   /* U+10876 */
  0x0FCF,   /* U+10877 */
  0x0FD0,   /* U+10878 */
  0x1C3E,   /* U+10879 */
  0x1C3F,   /* U+1087A */
  0x1C40,   /* U+1087B */
  0x1C41,   /* U+1087C */
  0x1C42,   /* U+1087D */
  0x1B4F,   /* U+1087E */
  0x1B50,   /* U+1087F */
  0x48CB,   /* U+10880 */
  0x48CB,   /* U+10881 */
  0x48CC,   /* U+10882 */
  0x48CC,   /* U+10883 */
  0x48CD,   /* U+10884 */
  0x48CE,   /* U+10885 */
  0x48CF,   /* U+10886 */
  0x48CF,   /* U+10887 */
  0x48D0,   /* U+10888 */
  0x48D1,   /* U+10889 */
  0x48D2,   /* U+1088A */
  0x48D3,   /* U+1088B */
  0x48D4,   /* U+1088C */
  0x48D4,   /* U+1088D */
  0x48D5,   /* U+1088E */
  0x48D5,   /* U+1088F */
  0x48D6,   /* U+10890 */
  0x48D6,   /* U+10891 */
  0x48D7,   /* U+10892 */
  0x48D7,   /* U+10893 */
  0x48D8,   /* U+10894 */
  0x48D8,   /* U+10895 */
  0x48D9,   /* U+10896 */
  0x48DA,   /* U+10897 */
  0x48DB,   /* U+10898 */
  0x48DC,   /* U+10899 */
  0x48DD,   /* U+1089A */
  0x48DE,   /* U+1089B */
  0x48DF,   /* U+1089C */
  0x48DF,   /* U+1089D */
  0x48E0,   /* U+1089E */
  0xFBC2,   /* U+1089F */
  0xFBC2,   /* U+108A0 */
  0xFBC2,   /* U+108A1 */
  0xFBC2,   /* U+108A2 */
  0xFBC2,   /* U+108A3 */
  0xFBC2,   /* U+108A4 */
  0xFBC2,   /* U+108A5 */
  0xFBC2,   /* U+108A6 */
  0x1C3E,   /* U+108A7 */
  0x1C3F,   /* U+108A8 */
  0x1C40,   /* U+108A9 */
  0x1C41,   /* U+108AA */
  0x1C41,   /* U+108AB */
  0x1C42,   /* U+108AC */
  0x1B51,   /* U+108AD */
  0x1B52,   /* U+108AE */
  0x1B53,   /* U+108AF */
  0xFBC2,   /* U+108B0 */
  0xFBC2,   /* U+108B1 */
  0xFBC2,   /* U+108B2 */
  0xFBC2,   /* U+108B3 */
  0xFBC2,   /* U+108B4 */
  0xFBC2,   /* U+108B5 */
  0xFBC2,   /* U+108B6 */
  0xFBC2,   /* U+108B7 */
  0xFBC2,   /* U+108B8 */
  0xFBC2,   /* U+108B9 */
  0xFBC2,   /* U+108BA */
  0xFBC2,   /* U+108BB */
  0xFBC2,   /* U+108BC */
  0xFBC2,   /* U+108BD */
  0xFBC2,   /* U+108BE */
  0xFBC2,   /* U+108BF */
  0xFBC2,   /* U+108C0 */
  0xFBC2,   /* U+108C1 */
  0xFBC2,   /* U+108C2 */
  0xFBC2,   /* U+108C3 */
  0xFBC2,   /* U+108C4 */
  0xFBC2,   /* U+108C5 */
  0xFBC2,   /* U+108C6 */
  0xFBC2,   /* U+108C7 */
  0xFBC2,   /* U+108C8 */
  0xFBC2,   /* U+108C9 */
  0xFBC2,   /* U+108CA */
  0xFBC2,   /* U+108CB */
  0xFBC2,   /* U+108CC */
  0xFBC2,   /* U+108CD */
  0xFBC2,   /* U+108CE */
  0xFBC2,   /* U+108CF */
  0xFBC2,   /* U+108D0 */
  0xFBC2,   /* U+108D1 */
  0xFBC2,   /* U+108D2 */
  0xFBC2,   /* U+108D3 */
  0xFBC2,   /* U+108D4 */
  0xFBC2,   /* U+108D5 */
  0xFBC2,   /* U+108D6 */
  0xFBC2,   /* U+108D7 */
  0xFBC2,   /* U+108D8 */
  0xFBC2,   /* U+108D9 */
  0xFBC2,   /* U+108DA */
  0xFBC2,   /* U+108DB */
  0xFBC2,   /* U+108DC */
  0xFBC2,   /* U+108DD */
  0xFBC2,   /* U+108DE */
  0xFBC2,   /* U+108DF */
  0x48E1,   /* U+108E0 */
  0x48E2,   /* U+108E1 */
  0x48E3,   /* U+108E2 */
  0x48E4,   /* U+108E3 */
  0x48E5,   /* U+108E4 */
  0x48E6,   /* U+108E5 */
  0x48E7,   /* U+108E6 */
  0x48E8,   /* U+108E7 */
  0x48E9,   /* U+108E8 */
  0x48EA,   /* U+108E9 */
  0x48EB,   /* U+108EA */
  0x48EC,   /* U+108EB */
  0x48ED,   /* U+108EC */
  0x48EE,   /* U+108ED */
  0x48EF,   /* U+108EE */
  0x48F0,   /* U+108EF */
  0x48F1,   /* U+108F0 */
  0x48F2,   /* U+108F1 */
  0x48F3,   /* U+108F2 */
  0xFBC2,   /* U+108F3 */
  0x48F4,   /* U+108F4 */
  0x48F5,   /* U+108F5 */
  0xFBC2,   /* U+108F6 */
  0xFBC2,   /* U+108F7 */
  0xFBC2,   /* U+108F8 */
  0xFBC2,   /* U+108F9 */
  0xFBC2,   /* U+108FA */
  0x1C3E,   /* U+108FB */
  0x1C42,   /* U+108FC */
  0x1B54,   /* U+108FD */
  0x1B55,   /* U+108FE */
  0x1B56,   /* U+108FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10800 */
  0x0020,   /* U+10801 */
  0x0020,   /* U+10802 */
  0x0020,   /* U+10803 */
  0x0020,   /* U+10804 */
  0x0020,   /* U+10805 */
  0x0020,   /* U+10806 */
  0x0020,   /* U+10807 */
  0x0020,   /* U+10808 */
  0x0020,   /* U+10809 */
  0x0020,   /* U+1080A */
  0x0020,   /* U+1080B */
  0x0020,   /* U+1080C */
  0x0020,   /* U+1080D */
  0x0020,   /* U+1080E */
  0x0020,   /* U+1080F */
  0x0020,   /* U+10810 */
  0x0020,   /* U+10811 */
  0x0020,   /* U+10812 */
  0x0020,   /* U+10813 */
  0x0020,   /* U+10814 */
  0x0020,   /* U+10815 */
  0x0020,   /* U+10816 */
  0x0020,   /* U+10817 */
  0x0020,   /* U+10818 */
  0x0020,   /* U+10819 */
  0x0020,   /* U+1081A */
  0x0020,   /* U+1081B */
  0x0020,   /* U+1081C */
  0x0020,   /* U+1081D */
  0x0020,   /* U+1081E */
  0x0020,   /* U+1081F */
  0x0020,   /* U+10820 */
  0x0020,   /* U+10821 */
  0x0020,   /* U+10822 */
  0x0020,   /* U+10823 */
  0x0020,   /* U+10824 */
  0x0020,   /* U+10825 */
  0x0020,   /* U+10826 */
  0x0020,   /* U+10827 */
  0x0020,   /* U+10828 */
  0x0020,   /* U+10829 */
  0x0020,   /* U+1082A */
  0x0020,   /* U+1082B */
  0x0020,   /* U+1082C */
  0x0020,   /* U+1082D */
  0x0020,   /* U+1082E */
  0x0020,   /* U+1082F */
  0x0020,   /* U+10830 */
  0x0020,   /* U+10831 */
  0x0020,   /* U+10832 */
  0x0020,   /* U+10833 */
  0x0020,   /* U+10834 */
  0x0020,   /* U+10835 */
  0x0020,   /* U+10836 */
  0x0020,   /* U+10837 */
  0x0020,   /* U+10838 */
  0x0020,   /* U+10839 */
  0x0020,   /* U+1083A */
  0x0020,   /* U+1083B */
  0x0020,   /* U+1083C */
  0x0020,   /* U+1083D */
  0x0020,   /* U+1083E */
  0x0020,   /* U+1083F */
  0x0020,   /* U+10840 */
  0x0020,   /* U+10841 */
  0x0020,   /* U+10842 */
  0x0020,   /* U+10843 */
  0x0020,   /* U+10844 */
  0x0020,   /* U+10845 */
  0x0020,   /* U+10846 */
  0x0020,   /* U+10847 */
  0x0020,   /* U+10848 */
  0x0020,   /* U+10849 */
  0x0020,   /* U+1084A */
  0x0020,   /* U+1084B */
  0x0020,   /* U+1084C */
  0x0020,   /* U+1084D */
  0x0020,   /* U+1084E */
  0x0020,   /* U+1084F */
  0x0020,   /* U+10850 */
  0x0020,   /* U+10851 */
  0x0020,   /* U+10852 */
  0x0020,   /* U+10853 */
  0x0020,   /* U+10854 */
  0x0020,   /* U+10855 */
  0x0020,   /* U+10856 */
  0x0020,   /* U+10857 */
  0x0020,   /* U+10858 */
  0x0020,   /* U+10859 */
  0x0020,   /* U+1085A */
  0x0020,   /* U+1085B */
  0x0020,   /* U+1085C */
  0x0020,   /* U+1085D */
  0x0020,   /* U+1085E */
  0x0020,   /* U+1085F */
  0x0020,   /* U+10860 */
  0x0020,   /* U+10861 */
  0x0020,   /* U+10862 */
  0x0020,   /* U+10863 */
  0x0020,   /* U+10864 */
  0x0020,   /* U+10865 */
  0x0020,   /* U+10866 */
  0x0020,   /* U+10867 */
  0x0020,   /* U+10868 */
  0x0020,   /* U+10869 */
  0x0020,   /* U+1086A */
  0x0020,   /* U+1086B */
  0x0020,   /* U+1086C */
  0x0020,   /* U+1086D */
  0x0020,   /* U+1086E */
  0x0020,   /* U+1086F */
  0x0020,   /* U+10870 */
  0x0020,   /* U+10871 */
  0x0020,   /* U+10872 */
  0x0020,   /* U+10873 */
  0x0020,   /* U+10874 */
  0x0020,   /* U+10875 */
  0x0020,   /* U+10876 */
  0x0020,   /* U+10877 */
  0x0020,   /* U+10878 */
  0x0020,   /* U+10879 */
  0x0020,   /* U+1087A */
  0x0020,   /* U+1087B */
  0x0020,   /* U+1087C */
  0x0020,   /* U+1087D */
  0x0020,   /* U+1087E */
  0x0020,   /* U+1087F */
  0x0020,   /* U+10880 */
  0x0020,   /* U+10881 */
  0x0020,   /* U+10882 */
  0x0020,   /* U+10883 */
  0x0020,   /* U+10884 */
  0x0020,   /* U+10885 */
  0x0020,   /* U+10886 */
  0x0020,   /* U+10887 */
  0x0020,   /* U+10888 */
  0x0020,   /* U+10889 */
  0x0020,   /* U+1088A */
  0x0020,   /* U+1088B */
  0x0020,   /* U+1088C */
  0x0020,   /* U+1088D */
  0x0020,   /* U+1088E */
  0x0020,   /* U+1088F */
  0x0020,   /* U+10890 */
  0x0020,   /* U+10891 */
  0x0020,   /* U+10892 */
  0x0020,   /* U+10893 */
  0x0020,   /* U+10894 */
  0x0020,   /* U+10895 */
  0x0020,   /* U+10896 */
  0x0020,   /* U+10897 */
  0x0020,   /* U+10898 */
  0x0020,   /* U+10899 */
  0x0020,   /* U+1089A */
  0x0020,   /* U+1089B */
  0x0020,   /* U+1089C */
  0x0020,   /* U+1089D */
  0x0020,   /* U+1089E */
  0x0020,   /* U+1089F */
  0x0020,   /* U+108A0 */
  0x0020,   /* U+108A1 */
  0x0020,   /* U+108A2 */
  0x0020,   /* U+108A3 */
  0x0020,   /* U+108A4 */
  0x0020,   /* U+108A5 */
  0x0020,   /* U+108A6 */
  0x0020,   /* U+108A7 */
  0x0020,   /* U+108A8 */
  0x0020,   /* U+108A9 */
  0x0020,   /* U+108AA */
  0x0020,   /* U+108AB */
  0x0020,   /* U+108AC */
  0x0020,   /* U+108AD */
  0x0020,   /* U+108AE */
  0x0020,   /* U+108AF */
  0x0020,   /* U+108B0 */
  0x0020,   /* U+108B1 */
  0x0020,   /* U+108B2 */
  0x0020,   /* U+108B3 */
  0x0020,   /* U+108B4 */
  0x0020,   /* U+108B5 */
  0x0020,   /* U+108B6 */
  0x0020,   /* U+108B7 */
  0x0020,   /* U+108B8 */
  0x0020,   /* U+108B9 */
  0x0020,   /* U+108BA */
  0x0020,   /* U+108BB */
  0x0020,   /* U+108BC */
  0x0020,   /* U+108BD */
  0x0020,   /* U+108BE */
  0x0020,   /* U+108BF */
  0x0020,   /* U+108C0 */
  0x0020,   /* U+108C1 */
  0x0020,   /* U+108C2 */
  0x0020,   /* U+108C3 */
  0x0020,   /* U+108C4 */
  0x0020,   /* U+108C5 */
  0x0020,   /* U+108C6 */
  0x0020,   /* U+108C7 */
  0x0020,   /* U+108C8 */
  0x0020,   /* U+108C9 */
  0x0020,   /* U+108CA */
  0x0020,   /* U+108CB */
  0x0020,   /* U+108CC */
  0x0020,   /* U+108CD */
  0x0020,   /* U+108CE */
  0x0020,   /* U+108CF */
  0x0020,   /* U+108D0 */
  0x0020,   /* U+108D1 */
  0x0020,   /* U+108D2 */
  0x0020,   /* U+108D3 */
  0x0020,   /* U+108D4 */
  0x0020,   /* U+108D5 */
  0x0020,   /* U+108D6 */
  0x0020,   /* U+108D7 */
  0x0020,   /* U+108D8 */
  0x0020,   /* U+108D9 */
  0x0020,   /* U+108DA */
  0x0020,   /* U+108DB */
  0x0020,   /* U+108DC */
  0x0020,   /* U+108DD */
  0x0020,   /* U+108DE */
  0x0020,   /* U+108DF */
  0x0020,   /* U+108E0 */
  0x0020,   /* U+108E1 */
  0x0020,   /* U+108E2 */
  0x0020,   /* U+108E3 */
  0x0020,   /* U+108E4 */
  0x0020,   /* U+108E5 */
  0x0020,   /* U+108E6 */
  0x0020,   /* U+108E7 */
  0x0020,   /* U+108E8 */
  0x0020,   /* U+108E9 */
  0x0020,   /* U+108EA */
  0x0020,   /* U+108EB */
  0x0020,   /* U+108EC */
  0x0020,   /* U+108ED */
  0x0020,   /* U+108EE */
  0x0020,   /* U+108EF */
  0x0020,   /* U+108F0 */
  0x0020,   /* U+108F1 */
  0x0020,   /* U+108F2 */
  0x0020,   /* U+108F3 */
  0x0020,   /* U+108F4 */
  0x0020,   /* U+108F5 */
  0x0020,   /* U+108F6 */
  0x0020,   /* U+108F7 */
  0x0020,   /* U+108F8 */
  0x0020,   /* U+108F9 */
  0x0020,   /* U+108FA */
  0x0020,   /* U+108FB */
  0x0020,   /* U+108FC */
  0x0020,   /* U+108FD */
  0x0020,   /* U+108FE */
  0x0020,   /* U+108FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10800 */
  0x0002,   /* U+10801 */
  0x0002,   /* U+10802 */
  0x0002,   /* U+10803 */
  0x0002,   /* U+10804 */
  0x0002,   /* U+10805 */
  0x0002,   /* U+10806 */
  0x0002,   /* U+10807 */
  0x0002,   /* U+10808 */
  0x0002,   /* U+10809 */
  0x0002,   /* U+1080A */
  0x0002,   /* U+1080B */
  0x0002,   /* U+1080C */
  0x0002,   /* U+1080D */
  0x0002,   /* U+1080E */
  0x0002,   /* U+1080F */
  0x0002,   /* U+10810 */
  0x0002,   /* U+10811 */
  0x0002,   /* U+10812 */
  0x0002,   /* U+10813 */
  0x0002,   /* U+10814 */
  0x0002,   /* U+10815 */
  0x0002,   /* U+10816 */
  0x0002,   /* U+10817 */
  0x0002,   /* U+10818 */
  0x0002,   /* U+10819 */
  0x0002,   /* U+1081A */
  0x0002,   /* U+1081B */
  0x0002,   /* U+1081C */
  0x0002,   /* U+1081D */
  0x0002,   /* U+1081E */
  0x0002,   /* U+1081F */
  0x0002,   /* U+10820 */
  0x0002,   /* U+10821 */
  0x0002,   /* U+10822 */
  0x0002,   /* U+10823 */
  0x0002,   /* U+10824 */
  0x0002,   /* U+10825 */
  0x0002,   /* U+10826 */
  0x0002,   /* U+10827 */
  0x0002,   /* U+10828 */
  0x0002,   /* U+10829 */
  0x0002,   /* U+1082A */
  0x0002,   /* U+1082B */
  0x0002,   /* U+1082C */
  0x0002,   /* U+1082D */
  0x0002,   /* U+1082E */
  0x0002,   /* U+1082F */
  0x0002,   /* U+10830 */
  0x0002,   /* U+10831 */
  0x0002,   /* U+10832 */
  0x0002,   /* U+10833 */
  0x0002,   /* U+10834 */
  0x0002,   /* U+10835 */
  0x0002,   /* U+10836 */
  0x0002,   /* U+10837 */
  0x0002,   /* U+10838 */
  0x0002,   /* U+10839 */
  0x0002,   /* U+1083A */
  0x0002,   /* U+1083B */
  0x0002,   /* U+1083C */
  0x0002,   /* U+1083D */
  0x0002,   /* U+1083E */
  0x0002,   /* U+1083F */
  0x0002,   /* U+10840 */
  0x0002,   /* U+10841 */
  0x0002,   /* U+10842 */
  0x0002,   /* U+10843 */
  0x0002,   /* U+10844 */
  0x0002,   /* U+10845 */
  0x0002,   /* U+10846 */
  0x0002,   /* U+10847 */
  0x0002,   /* U+10848 */
  0x0002,   /* U+10849 */
  0x0002,   /* U+1084A */
  0x0002,   /* U+1084B */
  0x0002,   /* U+1084C */
  0x0002,   /* U+1084D */
  0x0002,   /* U+1084E */
  0x0002,   /* U+1084F */
  0x0002,   /* U+10850 */
  0x0002,   /* U+10851 */
  0x0002,   /* U+10852 */
  0x0002,   /* U+10853 */
  0x0002,   /* U+10854 */
  0x0002,   /* U+10855 */
  0x0002,   /* U+10856 */
  0x0002,   /* U+10857 */
  0x0002,   /* U+10858 */
  0x0002,   /* U+10859 */
  0x0002,   /* U+1085A */
  0x0002,   /* U+1085B */
  0x0002,   /* U+1085C */
  0x0002,   /* U+1085D */
  0x0002,   /* U+1085E */
  0x0002,   /* U+1085F */
  0x0002,   /* U+10860 */
  0x0002,   /* U+10861 */
  0x0002,   /* U+10862 */
  0x0002,   /* U+10863 */
  0x0002,   /* U+10864 */
  0x0002,   /* U+10865 */
  0x0002,   /* U+10866 */
  0x0002,   /* U+10867 */
  0x0002,   /* U+10868 */
  0x0002,   /* U+10869 */
  0x0002,   /* U+1086A */
  0x0002,   /* U+1086B */
  0x0002,   /* U+1086C */
  0x0019,   /* U+1086D */
  0x0002,   /* U+1086E */
  0x0002,   /* U+1086F */
  0x0002,   /* U+10870 */
  0x0002,   /* U+10871 */
  0x0002,   /* U+10872 */
  0x0002,   /* U+10873 */
  0x0002,   /* U+10874 */
  0x0002,   /* U+10875 */
  0x0002,   /* U+10876 */
  0x0002,   /* U+10877 */
  0x0002,   /* U+10878 */
  0x0002,   /* U+10879 */
  0x0002,   /* U+1087A */
  0x0002,   /* U+1087B */
  0x0002,   /* U+1087C */
  0x0002,   /* U+1087D */
  0x0002,   /* U+1087E */
  0x0002,   /* U+1087F */
  0x0019,   /* U+10880 */
  0x0002,   /* U+10881 */
  0x0019,   /* U+10882 */
  0x0002,   /* U+10883 */
  0x0002,   /* U+10884 */
  0x0002,   /* U+10885 */
  0x0019,   /* U+10886 */
  0x0002,   /* U+10887 */
  0x0002,   /* U+10888 */
  0x0002,   /* U+10889 */
  0x0002,   /* U+1088A */
  0x0002,   /* U+1088B */
  0x0019,   /* U+1088C */
  0x0002,   /* U+1088D */
  0x0019,   /* U+1088E */
  0x0002,   /* U+1088F */
  0x0019,   /* U+10890 */
  0x0002,   /* U+10891 */
  0x0019,   /* U+10892 */
  0x0002,   /* U+10893 */
  0x0019,   /* U+10894 */
  0x0002,   /* U+10895 */
  0x0002,   /* U+10896 */
  0x0002,   /* U+10897 */
  0x0002,   /* U+10898 */
  0x0002,   /* U+10899 */
  0x0002,   /* U+1089A */
  0x0002,   /* U+1089B */
  0x0019,   /* U+1089C */
  0x0002,   /* U+1089D */
  0x0002,   /* U+1089E */
  0x0002,   /* U+1089F */
  0x0002,   /* U+108A0 */
  0x0002,   /* U+108A1 */
  0x0002,   /* U+108A2 */
  0x0002,   /* U+108A3 */
  0x0002,   /* U+108A4 */
  0x0002,   /* U+108A5 */
  0x0002,   /* U+108A6 */
  0x0002,   /* U+108A7 */
  0x0002,   /* U+108A8 */
  0x0002,   /* U+108A9 */
  0x0002,   /* U+108AA */
  0x0002,   /* U+108AB */
  0x0002,   /* U+108AC */
  0x0002,   /* U+108AD */
  0x0002,   /* U+108AE */
  0x0002,   /* U+108AF */
  0x0002,   /* U+108B0 */
  0x0002,   /* U+108B1 */
  0x0002,   /* U+108B2 */
  0x0002,   /* U+108B3 */
  0x0002,   /* U+108B4 */
  0x0002,   /* U+108B5 */
  0x0002,   /* U+108B6 */
  0x0002,   /* U+108B7 */
  0x0002,   /* U+108B8 */
  0x0002,   /* U+108B9 */
  0x0002,   /* U+108BA */
  0x0002,   /* U+108BB */
  0x0002,   /* U+108BC */
  0x0002,   /* U+108BD */
  0x0002,   /* U+108BE */
  0x0002,   /* U+108BF */
  0x0002,   /* U+108C0 */
  0x0002,   /* U+108C1 */
  0x0002,   /* U+108C2 */
  0x0002,   /* U+108C3 */
  0x0002,   /* U+108C4 */
  0x0002,   /* U+108C5 */
  0x0002,   /* U+108C6 */
  0x0002,   /* U+108C7 */
  0x0002,   /* U+108C8 */
  0x0002,   /* U+108C9 */
  0x0002,   /* U+108CA */
  0x0002,   /* U+108CB */
  0x0002,   /* U+108CC */
  0x0002,   /* U+108CD */
  0x0002,   /* U+108CE */
  0x0002,   /* U+108CF */
  0x0002,   /* U+108D0 */
  0x0002,   /* U+108D1 */
  0x0002,   /* U+108D2 */
  0x0002,   /* U+108D3 */
  0x0002,   /* U+108D4 */
  0x0002,   /* U+108D5 */
  0x0002,   /* U+108D6 */
  0x0002,   /* U+108D7 */
  0x0002,   /* U+108D8 */
  0x0002,   /* U+108D9 */
  0x0002,   /* U+108DA */
  0x0002,   /* U+108DB */
  0x0002,   /* U+108DC */
  0x0002,   /* U+108DD */
  0x0002,   /* U+108DE */
  0x0002,   /* U+108DF */
  0x0002,   /* U+108E0 */
  0x0002,   /* U+108E1 */
  0x0002,   /* U+108E2 */
  0x0002,   /* U+108E3 */
  0x0002,   /* U+108E4 */
  0x0002,   /* U+108E5 */
  0x0002,   /* U+108E6 */
  0x0002,   /* U+108E7 */
  0x0002,   /* U+108E8 */
  0x0002,   /* U+108E9 */
  0x0002,   /* U+108EA */
  0x0002,   /* U+108EB */
  0x0002,   /* U+108EC */
  0x0002,   /* U+108ED */
  0x0002,   /* U+108EE */
  0x0002,   /* U+108EF */
  0x0002,   /* U+108F0 */
  0x0002,   /* U+108F1 */
  0x0002,   /* U+108F2 */
  0x0002,   /* U+108F3 */
  0x0002,   /* U+108F4 */
  0x0002,   /* U+108F5 */
  0x0002,   /* U+108F6 */
  0x0002,   /* U+108F7 */
  0x0002,   /* U+108F8 */
  0x0002,   /* U+108F9 */
  0x0002,   /* U+108FA */
  0x0002,   /* U+108FB */
  0x0002,   /* U+108FC */
  0x0002,   /* U+108FD */
  0x0002,   /* U+108FE */
  0x0002,   /* U+108FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10800 */
  0x0000,   /* U+10801 */
  0x0000,   /* U+10802 */
  0x0000,   /* U+10803 */
  0x0000,   /* U+10804 */
  0x0000,   /* U+10805 */
  0x8806,   /* U+10806 */
  0x8807,   /* U+10807 */
  0x0000,   /* U+10808 */
  0x8809,   /* U+10809 */
  0x0000,   /* U+1080A */
  0x0000,   /* U+1080B */
  0x0000,   /* U+1080C */
  0x0000,   /* U+1080D */
  0x0000,   /* U+1080E */
  0x0000,   /* U+1080F */
  0x0000,   /* U+10810 */
  0x0000,   /* U+10811 */
  0x0000,   /* U+10812 */
  0x0000,   /* U+10813 */
  0x0000,   /* U+10814 */
  0x0000,   /* U+10815 */
  0x0000,   /* U+10816 */
  0x0000,   /* U+10817 */
  0x0000,   /* U+10818 */
  0x0000,   /* U+10819 */
  0x0000,   /* U+1081A */
  0x0000,   /* U+1081B */
  0x0000,   /* U+1081C */
  0x0000,   /* U+1081D */
  0x0000,   /* U+1081E */
  0x0000,   /* U+1081F */
  0x0000,   /* U+10820 */
  0x0000,   /* U+10821 */
  0x0000,   /* U+10822 */
  0x0000,   /* U+10823 */
  0x0000,   /* U+10824 */
  0x0000,   /* U+10825 */
  0x0000,   /* U+10826 */
  0x0000,   /* U+10827 */
  0x0000,   /* U+10828 */
  0x0000,   /* U+10829 */
  0x0000,   /* U+1082A */
  0x0000,   /* U+1082B */
  0x0000,   /* U+1082C */
  0x0000,   /* U+1082D */
  0x0000,   /* U+1082E */
  0x0000,   /* U+1082F */
  0x0000,   /* U+10830 */
  0x0000,   /* U+10831 */
  0x0000,   /* U+10832 */
  0x0000,   /* U+10833 */
  0x0000,   /* U+10834 */
  0x0000,   /* U+10835 */
  0x8836,   /* U+10836 */
  0x0000,   /* U+10837 */
  0x0000,   /* U+10838 */
  0x8839,   /* U+10839 */
  0x883A,   /* U+1083A */
  0x883B,   /* U+1083B */
  0x0000,   /* U+1083C */
  0x883D,   /* U+1083D */
  0x883E,   /* U+1083E */
  0x0000,   /* U+1083F */
  0x0000,   /* U+10840 */
  0x0000,   /* U+10841 */
  0x0000,   /* U+10842 */
  0x0000,   /* U+10843 */
  0x0000,   /* U+10844 */
  0x0000,   /* U+10845 */
  0x0000,   /* U+10846 */
  0x0000,   /* U+10847 */
  0x0000,   /* U+10848 */
  0x0000,   /* U+10849 */
  0x0000,   /* U+1084A */
  0x0000,   /* U+1084B */
  0x0000,   /* U+1084C */
  0x0000,   /* U+1084D */
  0x0000,   /* U+1084E */
  0x0000,   /* U+1084F */
  0x0000,   /* U+10850 */
  0x0000,   /* U+10851 */
  0x0000,   /* U+10852 */
  0x0000,   /* U+10853 */
  0x0000,   /* U+10854 */
  0x0000,   /* U+10855 */
  0x8856,   /* U+10856 */
  0x0000,   /* U+10857 */
  0x0000,   /* U+10858 */
  0x0000,   /* U+10859 */
  0x0000,   /* U+1085A */
  0x0000,   /* U+1085B */
  0x0000,   /* U+1085C */
  0x0000,   /* U+1085D */
  0x0000,   /* U+1085E */
  0x0000,   /* U+1085F */
  0x0000,   /* U+10860 */
  0x0000,   /* U+10861 */
  0x0000,   /* U+10862 */
  0x0000,   /* U+10863 */
  0x0000,   /* U+10864 */
  0x0000,   /* U+10865 */
  0x0000,   /* U+10866 */
  0x0000,   /* U+10867 */
  0x0000,   /* U+10868 */
  0x0000,   /* U+10869 */
  0x0000,   /* U+1086A */
  0x0000,   /* U+1086B */
  0x0000,   /* U+1086C */
  0x0000,   /* U+1086D */
  0x0000,   /* U+1086E */
  0x0000,   /* U+1086F */
  0x0000,   /* U+10870 */
  0x0000,   /* U+10871 */
  0x0000,   /* U+10872 */
  0x0000,   /* U+10873 */
  0x0000,   /* U+10874 */
  0x0000,   /* U+10875 */
  0x0000,   /* U+10876 */
  0x0000,   /* U+10877 */
  0x0000,   /* U+10878 */
  0x0000,   /* U+10879 */
  0x0000,   /* U+1087A */
  0x0000,   /* U+1087B */
  0x0000,   /* U+1087C */
  0x0000,   /* U+1087D */
  0x0000,   /* U+1087E */
  0x0000,   /* U+1087F */
  0x0000,   /* U+10880 */
  0x0000,   /* U+10881 */
  0x0000,   /* U+10882 */
  0x0000,   /* U+10883 */
  0x0000,   /* U+10884 */
  0x0000,   /* U+10885 */
  0x0000,   /* U+10886 */
  0x0000,   /* U+10887 */
  0x0000,   /* U+10888 */
  0x0000,   /* U+10889 */
  0x0000,   /* U+1088A */
  0x0000,   /* U+1088B */
  0x0000,   /* U+1088C */
  0x0000,   /* U+1088D */
  0x0000,   /* U+1088E */
  0x0000,   /* U+1088F */
  0x0000,   /* U+10890 */
  0x0000,   /* U+10891 */
  0x0000,   /* U+10892 */
  0x0000,   /* U+10893 */
  0x0000,   /* U+10894 */
  0x0000,   /* U+10895 */
  0x0000,   /* U+10896 */
  0x0000,   /* U+10897 */
  0x0000,   /* U+10898 */
  0x0000,   /* U+10899 */
  0x0000,   /* U+1089A */
  0x0000,   /* U+1089B */
  0x0000,   /* U+1089C */
  0x0000,   /* U+1089D */
  0x0000,   /* U+1089E */
  0x889F,   /* U+1089F */
  0x88A0,   /* U+108A0 */
  0x88A1,   /* U+108A1 */
  0x88A2,   /* U+108A2 */
  0x88A3,   /* U+108A3 */
  0x88A4,   /* U+108A4 */
  0x88A5,   /* U+108A5 */
  0x88A6,   /* U+108A6 */
  0x0000,   /* U+108A7 */
  0x0000,   /* U+108A8 */
  0x0000,   /* U+108A9 */
  0x0000,   /* U+108AA */
  0x0000,   /* U+108AB */
  0x0000,   /* U+108AC */
  0x0000,   /* U+108AD */
  0x0000,   /* U+108AE */
  0x0000,   /* U+108AF */
  0x88B0,   /* U+108B0 */
  0x88B1,   /* U+108B1 */
  0x88B2,   /* U+108B2 */
  0x88B3,   /* U+108B3 */
  0x88B4,   /* U+108B4 */
  0x88B5,   /* U+108B5 */
  0x88B6,   /* U+108B6 */
  0x88B7,   /* U+108B7 */
  0x88B8,   /* U+108B8 */
  0x88B9,   /* U+108B9 */
  0x88BA,   /* U+108BA */
  0x88BB,   /* U+108BB */
  0x88BC,   /* U+108BC */
  0x88BD,   /* U+108BD */
  0x88BE,   /* U+108BE */
  0x88BF,   /* U+108BF */
  0x88C0,   /* U+108C0 */
  0x88C1,   /* U+108C1 */
  0x88C2,   /* U+108C2 */
  0x88C3,   /* U+108C3 */
  0x88C4,   /* U+108C4 */
  0x88C5,   /* U+108C5 */
  0x88C6,   /* U+108C6 */
  0x88C7,   /* U+108C7 */
  0x88C8,   /* U+108C8 */
  0x88C9,   /* U+108C9 */
  0x88CA,   /* U+108CA */
  0x88CB,   /* U+108CB */
  0x88CC,   /* U+108CC */
  0x88CD,   /* U+108CD */
  0x88CE,   /* U+108CE */
  0x88CF,   /* U+108CF */
  0x88D0,   /* U+108D0 */
  0x88D1,   /* U+108D1 */
  0x88D2,   /* U+108D2 */
  0x88D3,   /* U+108D3 */
  0x88D4,   /* U+108D4 */
  0x88D5,   /* U+108D5 */
  0x88D6,   /* U+108D6 */
  0x88D7,   /* U+108D7 */
  0x88D8,   /* U+108D8 */
  0x88D9,   /* U+108D9 */
  0x88DA,   /* U+108DA */
  0x88DB,   /* U+108DB */
  0x88DC,   /* U+108DC */
  0x88DD,   /* U+108DD */
  0x88DE,   /* U+108DE */
  0x88DF,   /* U+108DF */
  0x0000,   /* U+108E0 */
  0x0000,   /* U+108E1 */
  0x0000,   /* U+108E2 */
  0x0000,   /* U+108E3 */
  0x0000,   /* U+108E4 */
  0x0000,   /* U+108E5 */
  0x0000,   /* U+108E6 */
  0x0000,   /* U+108E7 */
  0x0000,   /* U+108E8 */
  0x0000,   /* U+108E9 */
  0x0000,   /* U+108EA */
  0x0000,   /* U+108EB */
  0x0000,   /* U+108EC */
  0x0000,   /* U+108ED */
  0x0000,   /* U+108EE */
  0x0000,   /* U+108EF */
  0x0000,   /* U+108F0 */
  0x0000,   /* U+108F1 */
  0x0000,   /* U+108F2 */
  0x88F3,   /* U+108F3 */
  0x0000,   /* U+108F4 */
  0x0000,   /* U+108F5 */
  0x88F6,   /* U+108F6 */
  0x88F7,   /* U+108F7 */
  0x88F8,   /* U+108F8 */
  0x88F9,   /* U+108F9 */
  0x88FA,   /* U+108FA */
  0x0000,   /* U+108FB */
  0x0000,   /* U+108FC */
  0x0000,   /* U+108FD */
  0x0000,   /* U+108FE */
  0x0000,   /* U+108FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10800 */
  0x0000,   /* U+10801 */
  0x0000,   /* U+10802 */
  0x0000,   /* U+10803 */
  0x0000,   /* U+10804 */
  0x0000,   /* U+10805 */
  0x0000,   /* U+10806 */
  0x0000,   /* U+10807 */
  0x0000,   /* U+10808 */
  0x0000,   /* U+10809 */
  0x0000,   /* U+1080A */
  0x0000,   /* U+1080B */
  0x0000,   /* U+1080C */
  0x0000,   /* U+1080D */
  0x0000,   /* U+1080E */
  0x0000,   /* U+1080F */
  0x0000,   /* U+10810 */
  0x0000,   /* U+10811 */
  0x0000,   /* U+10812 */
  0x0000,   /* U+10813 */
  0x0000,   /* U+10814 */
  0x0000,   /* U+10815 */
  0x0000,   /* U+10816 */
  0x0000,   /* U+10817 */
  0x0000,   /* U+10818 */
  0x0000,   /* U+10819 */
  0x0000,   /* U+1081A */
  0x0000,   /* U+1081B */
  0x0000,   /* U+1081C */
  0x0000,   /* U+1081D */
  0x0000,   /* U+1081E */
  0x0000,   /* U+1081F */
  0x0000,   /* U+10820 */
  0x0000,   /* U+10821 */
  0x0000,   /* U+10822 */
  0x0000,   /* U+10823 */
  0x0000,   /* U+10824 */
  0x0000,   /* U+10825 */
  0x0000,   /* U+10826 */
  0x0000,   /* U+10827 */
  0x0000,   /* U+10828 */
  0x0000,   /* U+10829 */
  0x0000,   /* U+1082A */
  0x0000,   /* U+1082B */
  0x0000,   /* U+1082C */
  0x0000,   /* U+1082D */
  0x0000,   /* U+1082E */
  0x0000,   /* U+1082F */
  0x0000,   /* U+10830 */
  0x0000,   /* U+10831 */
  0x0000,   /* U+10832 */
  0x0000,   /* U+10833 */
  0x0000,   /* U+10834 */
  0x0000,   /* U+10835 */
  0x0000,   /* U+10836 */
  0x0000,   /* U+10837 */
  0x0000,   /* U+10838 */
  0x0000,   /* U+10839 */
  0x0000,   /* U+1083A */
  0x0000,   /* U+1083B */
  0x0000,   /* U+1083C */
  0x0000,   /* U+1083D */
  0x0000,   /* U+1083E */
  0x0000,   /* U+1083F */
  0x0000,   /* U+10840 */
  0x0000,   /* U+10841 */
  0x0000,   /* U+10842 */
  0x0000,   /* U+10843 */
  0x0000,   /* U+10844 */
  0x0000,   /* U+10845 */
  0x0000,   /* U+10846 */
  0x0000,   /* U+10847 */
  0x0000,   /* U+10848 */
  0x0000,   /* U+10849 */
  0x0000,   /* U+1084A */
  0x0000,   /* U+1084B */
  0x0000,   /* U+1084C */
  0x0000,   /* U+1084D */
  0x0000,   /* U+1084E */
  0x0000,   /* U+1084F */
  0x0000,   /* U+10850 */
  0x0000,   /* U+10851 */
  0x0000,   /* U+10852 */
  0x0000,   /* U+10853 */
  0x0000,   /* U+10854 */
  0x0000,   /* U+10855 */
  0x0000,   /* U+10856 */
  0x0000,   /* U+10857 */
  0x0000,   /* U+10858 */
  0x0000,   /* U+10859 */
  0x0000,   /* U+1085A */
  0x0000,   /* U+1085B */
  0x0000,   /* U+1085C */
  0x0000,   /* U+1085D */
  0x0000,   /* U+1085E */
  0x0000,   /* U+1085F */
  0x0000,   /* U+10860 */
  0x0000,   /* U+10861 */
  0x0000,   /* U+10862 */
  0x0000,   /* U+10863 */
  0x0000,   /* U+10864 */
  0x0000,   /* U+10865 */
  0x0000,   /* U+10866 */
  0x0000,   /* U+10867 */
  0x0000,   /* U+10868 */
  0x0000,   /* U+10869 */
  0x0000,   /* U+1086A */
  0x0000,   /* U+1086B */
  0x0000,   /* U+1086C */
  0x0000,   /* U+1086D */
  0x0000,   /* U+1086E */
  0x0000,   /* U+1086F */
  0x0000,   /* U+10870 */
  0x0000,   /* U+10871 */
  0x0000,   /* U+10872 */
  0x0000,   /* U+10873 */
  0x0000,   /* U+10874 */
  0x0000,   /* U+10875 */
  0x0000,   /* U+10876 */
  0x0000,   /* U+10877 */
  0x0000,   /* U+10878 */
  0x0000,   /* U+10879 */
  0x0000,   /* U+1087A */
  0x0000,   /* U+1087B */
  0x0000,   /* U+1087C */
  0x0000,   /* U+1087D */
  0x0000,   /* U+1087E */
  0x0000,   /* U+1087F */
  0x0000,   /* U+10880 */
  0x0000,   /* U+10881 */
  0x0000,   /* U+10882 */
  0x0000,   /* U+10883 */
  0x0000,   /* U+10884 */
  0x0000,   /* U+10885 */
  0x0000,   /* U+10886 */
  0x0000,   /* U+10887 */
  0x0000,   /* U+10888 */
  0x0000,   /* U+10889 */
  0x0000,   /* U+1088A */
  0x0000,   /* U+1088B */
  0x0000,   /* U+1088C */
  0x0000,   /* U+1088D */
  0x0000,   /* U+1088E */
  0x0000,   /* U+1088F */
  0x0000,   /* U+10890 */
  0x0000,   /* U+10891 */
  0x0000,   /* U+10892 */
  0x0000,   /* U+10893 */
  0x0000,   /* U+10894 */
  0x0000,   /* U+10895 */
  0x0000,   /* U+10896 */
  0x0000,   /* U+10897 */
  0x0000,   /* U+10898 */
  0x0000,   /* U+10899 */
  0x0000,   /* U+1089A */
  0x0000,   /* U+1089B */
  0x0000,   /* U+1089C */
  0x0000,   /* U+1089D */
  0x0000,   /* U+1089E */
  0x0000,   /* U+1089F */
  0x0000,   /* U+108A0 */
  0x0000,   /* U+108A1 */
  0x0000,   /* U+108A2 */
  0x0000,   /* U+108A3 */
  0x0000,   /* U+108A4 */
  0x0000,   /* U+108A5 */
  0x0000,   /* U+108A6 */
  0x0000,   /* U+108A7 */
  0x0000,   /* U+108A8 */
  0x0000,   /* U+108A9 */
  0x0000,   /* U+108AA */
  0x0000,   /* U+108AB */
  0x0000,   /* U+108AC */
  0x0000,   /* U+108AD */
  0x0000,   /* U+108AE */
  0x0000,   /* U+108AF */
  0x0000,   /* U+108B0 */
  0x0000,   /* U+108B1 */
  0x0000,   /* U+108B2 */
  0x0000,   /* U+108B3 */
  0x0000,   /* U+108B4 */
  0x0000,   /* U+108B5 */
  0x0000,   /* U+108B6 */
  0x0000,   /* U+108B7 */
  0x0000,   /* U+108B8 */
  0x0000,   /* U+108B9 */
  0x0000,   /* U+108BA */
  0x0000,   /* U+108BB */
  0x0000,   /* U+108BC */
  0x0000,   /* U+108BD */
  0x0000,   /* U+108BE */
  0x0000,   /* U+108BF */
  0x0000,   /* U+108C0 */
  0x0000,   /* U+108C1 */
  0x0000,   /* U+108C2 */
  0x0000,   /* U+108C3 */
  0x0000,   /* U+108C4 */
  0x0000,   /* U+108C5 */
  0x0000,   /* U+108C6 */
  0x0000,   /* U+108C7 */
  0x0000,   /* U+108C8 */
  0x0000,   /* U+108C9 */
  0x0000,   /* U+108CA */
  0x0000,   /* U+108CB */
  0x0000,   /* U+108CC */
  0x0000,   /* U+108CD */
  0x0000,   /* U+108CE */
  0x0000,   /* U+108CF */
  0x0000,   /* U+108D0 */
  0x0000,   /* U+108D1 */
  0x0000,   /* U+108D2 */
  0x0000,   /* U+108D3 */
  0x0000,   /* U+108D4 */
  0x0000,   /* U+108D5 */
  0x0000,   /* U+108D6 */
  0x0000,   /* U+108D7 */
  0x0000,   /* U+108D8 */
  0x0000,   /* U+108D9 */
  0x0000,   /* U+108DA */
  0x0000,   /* U+108DB */
  0x0000,   /* U+108DC */
  0x0000,   /* U+108DD */
  0x0000,   /* U+108DE */
  0x0000,   /* U+108DF */
  0x0000,   /* U+108E0 */
  0x0000,   /* U+108E1 */
  0x0000,   /* U+108E2 */
  0x0000,   /* U+108E3 */
  0x0000,   /* U+108E4 */
  0x0000,   /* U+108E5 */
  0x0000,   /* U+108E6 */
  0x0000,   /* U+108E7 */
  0x0000,   /* U+108E8 */
  0x0000,   /* U+108E9 */
  0x0000,   /* U+108EA */
  0x0000,   /* U+108EB */
  0x0000,   /* U+108EC */
  0x0000,   /* U+108ED */
  0x0000,   /* U+108EE */
  0x0000,   /* U+108EF */
  0x0000,   /* U+108F0 */
  0x0000,   /* U+108F1 */
  0x0000,   /* U+108F2 */
  0x0000,   /* U+108F3 */
  0x0000,   /* U+108F4 */
  0x0000,   /* U+108F5 */
  0x0000,   /* U+108F6 */
  0x0000,   /* U+108F7 */
  0x0000,   /* U+108F8 */
  0x0000,   /* U+108F9 */
  0x0000,   /* U+108FA */
  0x0000,   /* U+108FB */
  0x0000,   /* U+108FC */
  0x0000,   /* U+108FD */
  0x0000,   /* U+108FE */
  0x0000,   /* U+108FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10800 */
  0x0000,   /* U+10801 */
  0x0000,   /* U+10802 */
  0x0000,   /* U+10803 */
  0x0000,   /* U+10804 */
  0x0000,   /* U+10805 */
  0x0000,   /* U+10806 */
  0x0000,   /* U+10807 */
  0x0000,   /* U+10808 */
  0x0000,   /* U+10809 */
  0x0000,   /* U+1080A */
  0x0000,   /* U+1080B */
  0x0000,   /* U+1080C */
  0x0000,   /* U+1080D */
  0x0000,   /* U+1080E */
  0x0000,   /* U+1080F */
  0x0000,   /* U+10810 */
  0x0000,   /* U+10811 */
  0x0000,   /* U+10812 */
  0x0000,   /* U+10813 */
  0x0000,   /* U+10814 */
  0x0000,   /* U+10815 */
  0x0000,   /* U+10816 */
  0x0000,   /* U+10817 */
  0x0000,   /* U+10818 */
  0x0000,   /* U+10819 */
  0x0000,   /* U+1081A */
  0x0000,   /* U+1081B */
  0x0000,   /* U+1081C */
  0x0000,   /* U+1081D */
  0x0000,   /* U+1081E */
  0x0000,   /* U+1081F */
  0x0000,   /* U+10820 */
  0x0000,   /* U+10821 */
  0x0000,   /* U+10822 */
  0x0000,   /* U+10823 */
  0x0000,   /* U+10824 */
  0x0000,   /* U+10825 */
  0x0000,   /* U+10826 */
  0x0000,   /* U+10827 */
  0x0000,   /* U+10828 */
  0x0000,   /* U+10829 */
  0x0000,   /* U+1082A */
  0x0000,   /* U+1082B */
  0x0000,   /* U+1082C */
  0x0000,   /* U+1082D */
  0x0000,   /* U+1082E */
  0x0000,   /* U+1082F */
  0x0000,   /* U+10830 */
  0x0000,   /* U+10831 */
  0x0000,   /* U+10832 */
  0x0000,   /* U+10833 */
  0x0000,   /* U+10834 */
  0x0000,   /* U+10835 */
  0x0000,   /* U+10836 */
  0x0000,   /* U+10837 */
  0x0000,   /* U+10838 */
  0x0000,   /* U+10839 */
  0x0000,   /* U+1083A */
  0x0000,   /* U+1083B */
  0x0000,   /* U+1083C */
  0x0000,   /* U+1083D */
  0x0000,   /* U+1083E */
  0x0000,   /* U+1083F */
  0x0000,   /* U+10840 */
  0x0000,   /* U+10841 */
  0x0000,   /* U+10842 */
  0x0000,   /* U+10843 */
  0x0000,   /* U+10844 */
  0x0000,   /* U+10845 */
  0x0000,   /* U+10846 */
  0x0000,   /* U+10847 */
  0x0000,   /* U+10848 */
  0x0000,   /* U+10849 */
  0x0000,   /* U+1084A */
  0x0000,   /* U+1084B */
  0x0000,   /* U+1084C */
  0x0000,   /* U+1084D */
  0x0000,   /* U+1084E */
  0x0000,   /* U+1084F */
  0x0000,   /* U+10850 */
  0x0000,   /* U+10851 */
  0x0000,   /* U+10852 */
  0x0000,   /* U+10853 */
  0x0000,   /* U+10854 */
  0x0000,   /* U+10855 */
  0x0000,   /* U+10856 */
  0x0000,   /* U+10857 */
  0x0000,   /* U+10858 */
  0x0000,   /* U+10859 */
  0x0000,   /* U+1085A */
  0x0000,   /* U+1085B */
  0x0000,   /* U+1085C */
  0x0000,   /* U+1085D */
  0x0000,   /* U+1085E */
  0x0000,   /* U+1085F */
  0x0000,   /* U+10860 */
  0x0000,   /* U+10861 */
  0x0000,   /* U+10862 */
  0x0000,   /* U+10863 */
  0x0000,   /* U+10864 */
  0x0000,   /* U+10865 */
  0x0000,   /* U+10866 */
  0x0000,   /* U+10867 */
  0x0000,   /* U+10868 */
  0x0000,   /* U+10869 */
  0x0000,   /* U+1086A */
  0x0000,   /* U+1086B */
  0x0000,   /* U+1086C */
  0x0000,   /* U+1086D */
  0x0000,   /* U+1086E */
  0x0000,   /* U+1086F */
  0x0000,   /* U+10870 */
  0x0000,   /* U+10871 */
  0x0000,   /* U+10872 */
  0x0000,   /* U+10873 */
  0x0000,   /* U+10874 */
  0x0000,   /* U+10875 */
  0x0000,   /* U+10876 */
  0x0000,   /* U+10877 */
  0x0000,   /* U+10878 */
  0x0000,   /* U+10879 */
  0x0000,   /* U+1087A */
  0x0000,   /* U+1087B */
  0x0000,   /* U+1087C */
  0x0000,   /* U+1087D */
  0x0000,   /* U+1087E */
  0x0000,   /* U+1087F */
  0x0000,   /* U+10880 */
  0x0000,   /* U+10881 */
  0x0000,   /* U+10882 */
  0x0000,   /* U+10883 */
  0x0000,   /* U+10884 */
  0x0000,   /* U+10885 */
  0x0000,   /* U+10886 */
  0x0000,   /* U+10887 */
  0x0000,   /* U+10888 */
  0x0000,   /* U+10889 */
  0x0000,   /* U+1088A */
  0x0000,   /* U+1088B */
  0x0000,   /* U+1088C */
  0x0000,   /* U+1088D */
  0x0000,   /* U+1088E */
  0x0000,   /* U+1088F */
  0x0000,   /* U+10890 */
  0x0000,   /* U+10891 */
  0x0000,   /* U+10892 */
  0x0000,   /* U+10893 */
  0x0000,   /* U+10894 */
  0x0000,   /* U+10895 */
  0x0000,   /* U+10896 */
  0x0000,   /* U+10897 */
  0x0000,   /* U+10898 */
  0x0000,   /* U+10899 */
  0x0000,   /* U+1089A */
  0x0000,   /* U+1089B */
  0x0000,   /* U+1089C */
  0x0000,   /* U+1089D */
  0x0000,   /* U+1089E */
  0x0000,   /* U+1089F */
  0x0000,   /* U+108A0 */
  0x0000,   /* U+108A1 */
  0x0000,   /* U+108A2 */
  0x0000,   /* U+108A3 */
  0x0000,   /* U+108A4 */
  0x0000,   /* U+108A5 */
  0x0000,   /* U+108A6 */
  0x0000,   /* U+108A7 */
  0x0000,   /* U+108A8 */
  0x0000,   /* U+108A9 */
  0x0000,   /* U+108AA */
  0x0000,   /* U+108AB */
  0x0000,   /* U+108AC */
  0x0000,   /* U+108AD */
  0x0000,   /* U+108AE */
  0x0000,   /* U+108AF */
  0x0000,   /* U+108B0 */
  0x0000,   /* U+108B1 */
  0x0000,   /* U+108B2 */
  0x0000,   /* U+108B3 */
  0x0000,   /* U+108B4 */
  0x0000,   /* U+108B5 */
  0x0000,   /* U+108B6 */
  0x0000,   /* U+108B7 */
  0x0000,   /* U+108B8 */
  0x0000,   /* U+108B9 */
  0x0000,   /* U+108BA */
  0x0000,   /* U+108BB */
  0x0000,   /* U+108BC */
  0x0000,   /* U+108BD */
  0x0000,   /* U+108BE */
  0x0000,   /* U+108BF */
  0x0000,   /* U+108C0 */
  0x0000,   /* U+108C1 */
  0x0000,   /* U+108C2 */
  0x0000,   /* U+108C3 */
  0x0000,   /* U+108C4 */
  0x0000,   /* U+108C5 */
  0x0000,   /* U+108C6 */
  0x0000,   /* U+108C7 */
  0x0000,   /* U+108C8 */
  0x0000,   /* U+108C9 */
  0x0000,   /* U+108CA */
  0x0000,   /* U+108CB */
  0x0000,   /* U+108CC */
  0x0000,   /* U+108CD */
  0x0000,   /* U+108CE */
  0x0000,   /* U+108CF */
  0x0000,   /* U+108D0 */
  0x0000,   /* U+108D1 */
  0x0000,   /* U+108D2 */
  0x0000,   /* U+108D3 */
  0x0000,   /* U+108D4 */
  0x0000,   /* U+108D5 */
  0x0000,   /* U+108D6 */
  0x0000,   /* U+108D7 */
  0x0000,   /* U+108D8 */
  0x0000,   /* U+108D9 */
  0x0000,   /* U+108DA */
  0x0000,   /* U+108DB */
  0x0000,   /* U+108DC */
  0x0000,   /* U+108DD */
  0x0000,   /* U+108DE */
  0x0000,   /* U+108DF */
  0x0000,   /* U+108E0 */
  0x0000,   /* U+108E1 */
  0x0000,   /* U+108E2 */
  0x0000,   /* U+108E3 */
  0x0000,   /* U+108E4 */
  0x0000,   /* U+108E5 */
  0x0000,   /* U+108E6 */
  0x0000,   /* U+108E7 */
  0x0000,   /* U+108E8 */
  0x0000,   /* U+108E9 */
  0x0000,   /* U+108EA */
  0x0000,   /* U+108EB */
  0x0000,   /* U+108EC */
  0x0000,   /* U+108ED */
  0x0000,   /* U+108EE */
  0x0000,   /* U+108EF */
  0x0000,   /* U+108F0 */
  0x0000,   /* U+108F1 */
  0x0000,   /* U+108F2 */
  0x0000,   /* U+108F3 */
  0x0000,   /* U+108F4 */
  0x0000,   /* U+108F5 */
  0x0000,   /* U+108F6 */
  0x0000,   /* U+108F7 */
  0x0000,   /* U+108F8 */
  0x0000,   /* U+108F9 */
  0x0000,   /* U+108FA */
  0x0000,   /* U+108FB */
  0x0000,   /* U+108FC */
  0x0000,   /* U+108FD */
  0x0000,   /* U+108FE */
  0x0000,   /* U+108FF */
};

uint16 uca900_p109[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x22CD,   /* U+10900 */
  0x22CE,   /* U+10901 */
  0x22CF,   /* U+10902 */
  0x22D0,   /* U+10903 */
  0x22D1,   /* U+10904 */
  0x22D2,   /* U+10905 */
  0x22D3,   /* U+10906 */
  0x22D4,   /* U+10907 */
  0x22D5,   /* U+10908 */
  0x22D6,   /* U+10909 */
  0x22D7,   /* U+1090A */
  0x22D8,   /* U+1090B */
  0x22D9,   /* U+1090C */
  0x22DA,   /* U+1090D */
  0x22DB,   /* U+1090E */
  0x22DC,   /* U+1090F */
  0x22DD,   /* U+10910 */
  0x22DE,   /* U+10911 */
  0x22DF,   /* U+10912 */
  0x22E0,   /* U+10913 */
  0x22E1,   /* U+10914 */
  0x22E2,   /* U+10915 */
  0x1C3E,   /* U+10916 */
  0x1B5B,   /* U+10917 */
  0x1B5C,   /* U+10918 */
  0x1B5D,   /* U+10919 */
  0x1C3F,   /* U+1091A */
  0x1C40,   /* U+1091B */
  0xFBC2,   /* U+1091C */
  0xFBC2,   /* U+1091D */
  0xFBC2,   /* U+1091E */
  0x02FF,   /* U+1091F */
  0x43FD,   /* U+10920 */
  0x43FE,   /* U+10921 */
  0x43FF,   /* U+10922 */
  0x4400,   /* U+10923 */
  0x4401,   /* U+10924 */
  0x4402,   /* U+10925 */
  0x4403,   /* U+10926 */
  0x4404,   /* U+10927 */
  0x4405,   /* U+10928 */
  0x4406,   /* U+10929 */
  0x4407,   /* U+1092A */
  0x4408,   /* U+1092B */
  0x4409,   /* U+1092C */
  0x440A,   /* U+1092D */
  0x440B,   /* U+1092E */
  0x440C,   /* U+1092F */
  0x440D,   /* U+10930 */
  0x440E,   /* U+10931 */
  0x440F,   /* U+10932 */
  0x4410,   /* U+10933 */
  0x4411,   /* U+10934 */
  0x4412,   /* U+10935 */
  0x4413,   /* U+10936 */
  0x4414,   /* U+10937 */
  0x4415,   /* U+10938 */
  0x4416,   /* U+10939 */
  0xFBC2,   /* U+1093A */
  0xFBC2,   /* U+1093B */
  0xFBC2,   /* U+1093C */
  0xFBC2,   /* U+1093D */
  0xFBC2,   /* U+1093E */
  0x02F9,   /* U+1093F */
  0xFBC2,   /* U+10940 */
  0xFBC2,   /* U+10941 */
  0xFBC2,   /* U+10942 */
  0xFBC2,   /* U+10943 */
  0xFBC2,   /* U+10944 */
  0xFBC2,   /* U+10945 */
  0xFBC2,   /* U+10946 */
  0xFBC2,   /* U+10947 */
  0xFBC2,   /* U+10948 */
  0xFBC2,   /* U+10949 */
  0xFBC2,   /* U+1094A */
  0xFBC2,   /* U+1094B */
  0xFBC2,   /* U+1094C */
  0xFBC2,   /* U+1094D */
  0xFBC2,   /* U+1094E */
  0xFBC2,   /* U+1094F */
  0xFBC2,   /* U+10950 */
  0xFBC2,   /* U+10951 */
  0xFBC2,   /* U+10952 */
  0xFBC2,   /* U+10953 */
  0xFBC2,   /* U+10954 */
  0xFBC2,   /* U+10955 */
  0xFBC2,   /* U+10956 */
  0xFBC2,   /* U+10957 */
  0xFBC2,   /* U+10958 */
  0xFBC2,   /* U+10959 */
  0xFBC2,   /* U+1095A */
  0xFBC2,   /* U+1095B */
  0xFBC2,   /* U+1095C */
  0xFBC2,   /* U+1095D */
  0xFBC2,   /* U+1095E */
  0xFBC2,   /* U+1095F */
  0xFBC2,   /* U+10960 */
  0xFBC2,   /* U+10961 */
  0xFBC2,   /* U+10962 */
  0xFBC2,   /* U+10963 */
  0xFBC2,   /* U+10964 */
  0xFBC2,   /* U+10965 */
  0xFBC2,   /* U+10966 */
  0xFBC2,   /* U+10967 */
  0xFBC2,   /* U+10968 */
  0xFBC2,   /* U+10969 */
  0xFBC2,   /* U+1096A */
  0xFBC2,   /* U+1096B */
  0xFBC2,   /* U+1096C */
  0xFBC2,   /* U+1096D */
  0xFBC2,   /* U+1096E */
  0xFBC2,   /* U+1096F */
  0xFBC2,   /* U+10970 */
  0xFBC2,   /* U+10971 */
  0xFBC2,   /* U+10972 */
  0xFBC2,   /* U+10973 */
  0xFBC2,   /* U+10974 */
  0xFBC2,   /* U+10975 */
  0xFBC2,   /* U+10976 */
  0xFBC2,   /* U+10977 */
  0xFBC2,   /* U+10978 */
  0xFBC2,   /* U+10979 */
  0xFBC2,   /* U+1097A */
  0xFBC2,   /* U+1097B */
  0xFBC2,   /* U+1097C */
  0xFBC2,   /* U+1097D */
  0xFBC2,   /* U+1097E */
  0xFBC2,   /* U+1097F */
  0x5242,   /* U+10980 */
  0x5243,   /* U+10981 */
  0x5244,   /* U+10982 */
  0x5245,   /* U+10983 */
  0x5246,   /* U+10984 */
  0x5247,   /* U+10985 */
  0x5248,   /* U+10986 */
  0x5248,   /* U+10987 */
  0x5249,   /* U+10988 */
  0x524A,   /* U+10989 */
  0x524B,   /* U+1098A */
  0x524B,   /* U+1098B */
  0x524C,   /* U+1098C */
  0x524C,   /* U+1098D */
  0x524D,   /* U+1098E */
  0x524D,   /* U+1098F */
  0x524E,   /* U+10990 */
  0x524F,   /* U+10991 */
  0x5250,   /* U+10992 */
  0x5251,   /* U+10993 */
  0x5251,   /* U+10994 */
  0x5252,   /* U+10995 */
  0x5253,   /* U+10996 */
  0x5254,   /* U+10997 */
  0x5255,   /* U+10998 */
  0x5255,   /* U+10999 */
  0x5256,   /* U+1099A */
  0x5256,   /* U+1099B */
  0x5257,   /* U+1099C */
  0x5258,   /* U+1099D */
  0x525B,   /* U+1099E */
  0x525C,   /* U+1099F */
  0x5242,   /* U+109A0 */
  0x5243,   /* U+109A1 */
  0x5244,   /* U+109A2 */
  0x5245,   /* U+109A3 */
  0x5246,   /* U+109A4 */
  0x5247,   /* U+109A5 */
  0x5248,   /* U+109A6 */
  0x5249,   /* U+109A7 */
  0x524A,   /* U+109A8 */
  0x524B,   /* U+109A9 */
  0x524C,   /* U+109AA */
  0x524D,   /* U+109AB */
  0x524E,   /* U+109AC */
  0x524F,   /* U+109AD */
  0x5250,   /* U+109AE */
  0x5251,   /* U+109AF */
  0x5251,   /* U+109B0 */
  0x5252,   /* U+109B1 */
  0x5253,   /* U+109B2 */
  0x5254,   /* U+109B3 */
  0x5255,   /* U+109B4 */
  0x5256,   /* U+109B5 */
  0x5257,   /* U+109B6 */
  0x5258,   /* U+109B7 */
  0xFBC2,   /* U+109B8 */
  0xFBC2,   /* U+109B9 */
  0xFBC2,   /* U+109BA */
  0xFBC2,   /* U+109BB */
  0x1BDD,   /* U+109BC */
  0x1BD2,   /* U+109BD */
  0x5259,   /* U+109BE */
  0x525A,   /* U+109BF */
  0x1C3E,   /* U+109C0 */
  0x1C3F,   /* U+109C1 */
  0x1C40,   /* U+109C2 */
  0x1C41,   /* U+109C3 */
  0x1C42,   /* U+109C4 */
  0x1C43,   /* U+109C5 */
  0x1C44,   /* U+109C6 */
  0x1C45,   /* U+109C7 */
  0x1C46,   /* U+109C8 */
  0x1BA7,   /* U+109C9 */
  0x1BA8,   /* U+109CA */
  0x1BA9,   /* U+109CB */
  0x1BAA,   /* U+109CC */
  0x1BAB,   /* U+109CD */
  0x1BAC,   /* U+109CE */
  0x1BAD,   /* U+109CF */
  0xFBC2,   /* U+109D0 */
  0xFBC2,   /* U+109D1 */
  0x1BAE,   /* U+109D2 */
  0x1BAF,   /* U+109D3 */
  0x1BB0,   /* U+109D4 */
  0x1BB1,   /* U+109D5 */
  0x1BB2,   /* U+109D6 */
  0x1BB3,   /* U+109D7 */
  0x1BB4,   /* U+109D8 */
  0x1BB5,   /* U+109D9 */
  0x1BB6,   /* U+109DA */
  0x1BB7,   /* U+109DB */
  0x1BB8,   /* U+109DC */
  0x1BB9,   /* U+109DD */
  0x1BBA,   /* U+109DE */
  0x1BBB,   /* U+109DF */
  0x1BBC,   /* U+109E0 */
  0x1BBD,   /* U+109E1 */
  0x1BBE,   /* U+109E2 */
  0x1BBF,   /* U+109E3 */
  0x1BC0,   /* U+109E4 */
  0x1BC1,   /* U+109E5 */
  0x1BC2,   /* U+109E6 */
  0x1BC3,   /* U+109E7 */
  0x1BC4,   /* U+109E8 */
  0x1BC5,   /* U+109E9 */
  0x1BC6,   /* U+109EA */
  0x1BC7,   /* U+109EB */
  0x1BC8,   /* U+109EC */
  0x1BC9,   /* U+109ED */
  0x1BCA,   /* U+109EE */
  0x1BCB,   /* U+109EF */
  0x1BCC,   /* U+109F0 */
  0x1BCD,   /* U+109F1 */
  0x1BCE,   /* U+109F2 */
  0x1BCF,   /* U+109F3 */
  0x1BD0,   /* U+109F4 */
  0x1BD1,   /* U+109F5 */
  0x1BD3,   /* U+109F6 */
  0x1BD4,   /* U+109F7 */
  0x1BD5,   /* U+109F8 */
  0x1BD6,   /* U+109F9 */
  0x1BD7,   /* U+109FA */
  0x1BD8,   /* U+109FB */
  0x1BD9,   /* U+109FC */
  0x1BDA,   /* U+109FD */
  0x1BDB,   /* U+109FE */
  0x1BDC,   /* U+109FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10900 */
  0x0020,   /* U+10901 */
  0x0020,   /* U+10902 */
  0x0020,   /* U+10903 */
  0x0020,   /* U+10904 */
  0x0020,   /* U+10905 */
  0x0020,   /* U+10906 */
  0x0020,   /* U+10907 */
  0x0020,   /* U+10908 */
  0x0020,   /* U+10909 */
  0x0020,   /* U+1090A */
  0x0020,   /* U+1090B */
  0x0020,   /* U+1090C */
  0x0020,   /* U+1090D */
  0x0020,   /* U+1090E */
  0x0020,   /* U+1090F */
  0x0020,   /* U+10910 */
  0x0020,   /* U+10911 */
  0x0020,   /* U+10912 */
  0x0020,   /* U+10913 */
  0x0020,   /* U+10914 */
  0x0020,   /* U+10915 */
  0x0020,   /* U+10916 */
  0x0020,   /* U+10917 */
  0x0020,   /* U+10918 */
  0x0020,   /* U+10919 */
  0x0020,   /* U+1091A */
  0x0020,   /* U+1091B */
  0x0020,   /* U+1091C */
  0x0020,   /* U+1091D */
  0x0020,   /* U+1091E */
  0x0020,   /* U+1091F */
  0x0020,   /* U+10920 */
  0x0020,   /* U+10921 */
  0x0020,   /* U+10922 */
  0x0020,   /* U+10923 */
  0x0020,   /* U+10924 */
  0x0020,   /* U+10925 */
  0x0020,   /* U+10926 */
  0x0020,   /* U+10927 */
  0x0020,   /* U+10928 */
  0x0020,   /* U+10929 */
  0x0020,   /* U+1092A */
  0x0020,   /* U+1092B */
  0x0020,   /* U+1092C */
  0x0020,   /* U+1092D */
  0x0020,   /* U+1092E */
  0x0020,   /* U+1092F */
  0x0020,   /* U+10930 */
  0x0020,   /* U+10931 */
  0x0020,   /* U+10932 */
  0x0020,   /* U+10933 */
  0x0020,   /* U+10934 */
  0x0020,   /* U+10935 */
  0x0020,   /* U+10936 */
  0x0020,   /* U+10937 */
  0x0020,   /* U+10938 */
  0x0020,   /* U+10939 */
  0x0020,   /* U+1093A */
  0x0020,   /* U+1093B */
  0x0020,   /* U+1093C */
  0x0020,   /* U+1093D */
  0x0020,   /* U+1093E */
  0x0020,   /* U+1093F */
  0x0020,   /* U+10940 */
  0x0020,   /* U+10941 */
  0x0020,   /* U+10942 */
  0x0020,   /* U+10943 */
  0x0020,   /* U+10944 */
  0x0020,   /* U+10945 */
  0x0020,   /* U+10946 */
  0x0020,   /* U+10947 */
  0x0020,   /* U+10948 */
  0x0020,   /* U+10949 */
  0x0020,   /* U+1094A */
  0x0020,   /* U+1094B */
  0x0020,   /* U+1094C */
  0x0020,   /* U+1094D */
  0x0020,   /* U+1094E */
  0x0020,   /* U+1094F */
  0x0020,   /* U+10950 */
  0x0020,   /* U+10951 */
  0x0020,   /* U+10952 */
  0x0020,   /* U+10953 */
  0x0020,   /* U+10954 */
  0x0020,   /* U+10955 */
  0x0020,   /* U+10956 */
  0x0020,   /* U+10957 */
  0x0020,   /* U+10958 */
  0x0020,   /* U+10959 */
  0x0020,   /* U+1095A */
  0x0020,   /* U+1095B */
  0x0020,   /* U+1095C */
  0x0020,   /* U+1095D */
  0x0020,   /* U+1095E */
  0x0020,   /* U+1095F */
  0x0020,   /* U+10960 */
  0x0020,   /* U+10961 */
  0x0020,   /* U+10962 */
  0x0020,   /* U+10963 */
  0x0020,   /* U+10964 */
  0x0020,   /* U+10965 */
  0x0020,   /* U+10966 */
  0x0020,   /* U+10967 */
  0x0020,   /* U+10968 */
  0x0020,   /* U+10969 */
  0x0020,   /* U+1096A */
  0x0020,   /* U+1096B */
  0x0020,   /* U+1096C */
  0x0020,   /* U+1096D */
  0x0020,   /* U+1096E */
  0x0020,   /* U+1096F */
  0x0020,   /* U+10970 */
  0x0020,   /* U+10971 */
  0x0020,   /* U+10972 */
  0x0020,   /* U+10973 */
  0x0020,   /* U+10974 */
  0x0020,   /* U+10975 */
  0x0020,   /* U+10976 */
  0x0020,   /* U+10977 */
  0x0020,   /* U+10978 */
  0x0020,   /* U+10979 */
  0x0020,   /* U+1097A */
  0x0020,   /* U+1097B */
  0x0020,   /* U+1097C */
  0x0020,   /* U+1097D */
  0x0020,   /* U+1097E */
  0x0020,   /* U+1097F */
  0x0020,   /* U+10980 */
  0x0020,   /* U+10981 */
  0x0020,   /* U+10982 */
  0x0020,   /* U+10983 */
  0x0020,   /* U+10984 */
  0x0020,   /* U+10985 */
  0x0020,   /* U+10986 */
  0x0020,   /* U+10987 */
  0x0020,   /* U+10988 */
  0x0020,   /* U+10989 */
  0x0020,   /* U+1098A */
  0x0020,   /* U+1098B */
  0x0020,   /* U+1098C */
  0x0020,   /* U+1098D */
  0x0020,   /* U+1098E */
  0x0020,   /* U+1098F */
  0x0020,   /* U+10990 */
  0x0020,   /* U+10991 */
  0x0020,   /* U+10992 */
  0x0020,   /* U+10993 */
  0x0020,   /* U+10994 */
  0x0020,   /* U+10995 */
  0x0020,   /* U+10996 */
  0x0020,   /* U+10997 */
  0x0020,   /* U+10998 */
  0x0020,   /* U+10999 */
  0x0020,   /* U+1099A */
  0x0020,   /* U+1099B */
  0x0020,   /* U+1099C */
  0x0020,   /* U+1099D */
  0x0020,   /* U+1099E */
  0x0020,   /* U+1099F */
  0x0020,   /* U+109A0 */
  0x0020,   /* U+109A1 */
  0x0020,   /* U+109A2 */
  0x0020,   /* U+109A3 */
  0x0020,   /* U+109A4 */
  0x0020,   /* U+109A5 */
  0x0020,   /* U+109A6 */
  0x0020,   /* U+109A7 */
  0x0020,   /* U+109A8 */
  0x0020,   /* U+109A9 */
  0x0020,   /* U+109AA */
  0x0020,   /* U+109AB */
  0x0020,   /* U+109AC */
  0x0020,   /* U+109AD */
  0x0020,   /* U+109AE */
  0x0020,   /* U+109AF */
  0x0020,   /* U+109B0 */
  0x0020,   /* U+109B1 */
  0x0020,   /* U+109B2 */
  0x0020,   /* U+109B3 */
  0x0020,   /* U+109B4 */
  0x0020,   /* U+109B5 */
  0x0020,   /* U+109B6 */
  0x0020,   /* U+109B7 */
  0x0020,   /* U+109B8 */
  0x0020,   /* U+109B9 */
  0x0020,   /* U+109BA */
  0x0020,   /* U+109BB */
  0x0020,   /* U+109BC */
  0x0020,   /* U+109BD */
  0x0020,   /* U+109BE */
  0x0020,   /* U+109BF */
  0x0020,   /* U+109C0 */
  0x0020,   /* U+109C1 */
  0x0020,   /* U+109C2 */
  0x0020,   /* U+109C3 */
  0x0020,   /* U+109C4 */
  0x0020,   /* U+109C5 */
  0x0020,   /* U+109C6 */
  0x0020,   /* U+109C7 */
  0x0020,   /* U+109C8 */
  0x0020,   /* U+109C9 */
  0x0020,   /* U+109CA */
  0x0020,   /* U+109CB */
  0x0020,   /* U+109CC */
  0x0020,   /* U+109CD */
  0x0020,   /* U+109CE */
  0x0020,   /* U+109CF */
  0x0020,   /* U+109D0 */
  0x0020,   /* U+109D1 */
  0x0020,   /* U+109D2 */
  0x0020,   /* U+109D3 */
  0x0020,   /* U+109D4 */
  0x0020,   /* U+109D5 */
  0x0020,   /* U+109D6 */
  0x0020,   /* U+109D7 */
  0x0020,   /* U+109D8 */
  0x0020,   /* U+109D9 */
  0x0020,   /* U+109DA */
  0x0020,   /* U+109DB */
  0x0020,   /* U+109DC */
  0x0020,   /* U+109DD */
  0x0020,   /* U+109DE */
  0x0020,   /* U+109DF */
  0x0020,   /* U+109E0 */
  0x0020,   /* U+109E1 */
  0x0020,   /* U+109E2 */
  0x0020,   /* U+109E3 */
  0x0020,   /* U+109E4 */
  0x0020,   /* U+109E5 */
  0x0020,   /* U+109E6 */
  0x0020,   /* U+109E7 */
  0x0020,   /* U+109E8 */
  0x0020,   /* U+109E9 */
  0x0020,   /* U+109EA */
  0x0020,   /* U+109EB */
  0x0020,   /* U+109EC */
  0x0020,   /* U+109ED */
  0x0020,   /* U+109EE */
  0x0020,   /* U+109EF */
  0x0020,   /* U+109F0 */
  0x0020,   /* U+109F1 */
  0x0020,   /* U+109F2 */
  0x0020,   /* U+109F3 */
  0x0020,   /* U+109F4 */
  0x0020,   /* U+109F5 */
  0x0020,   /* U+109F6 */
  0x0020,   /* U+109F7 */
  0x0020,   /* U+109F8 */
  0x0020,   /* U+109F9 */
  0x0020,   /* U+109FA */
  0x0020,   /* U+109FB */
  0x0020,   /* U+109FC */
  0x0020,   /* U+109FD */
  0x0020,   /* U+109FE */
  0x0020,   /* U+109FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10900 */
  0x0002,   /* U+10901 */
  0x0002,   /* U+10902 */
  0x0002,   /* U+10903 */
  0x0002,   /* U+10904 */
  0x0002,   /* U+10905 */
  0x0002,   /* U+10906 */
  0x0002,   /* U+10907 */
  0x0002,   /* U+10908 */
  0x0002,   /* U+10909 */
  0x0002,   /* U+1090A */
  0x0002,   /* U+1090B */
  0x0002,   /* U+1090C */
  0x0002,   /* U+1090D */
  0x0002,   /* U+1090E */
  0x0002,   /* U+1090F */
  0x0002,   /* U+10910 */
  0x0002,   /* U+10911 */
  0x0002,   /* U+10912 */
  0x0002,   /* U+10913 */
  0x0002,   /* U+10914 */
  0x0002,   /* U+10915 */
  0x0002,   /* U+10916 */
  0x0002,   /* U+10917 */
  0x0002,   /* U+10918 */
  0x0002,   /* U+10919 */
  0x0002,   /* U+1091A */
  0x0002,   /* U+1091B */
  0x0002,   /* U+1091C */
  0x0002,   /* U+1091D */
  0x0002,   /* U+1091E */
  0x0002,   /* U+1091F */
  0x0002,   /* U+10920 */
  0x0002,   /* U+10921 */
  0x0002,   /* U+10922 */
  0x0002,   /* U+10923 */
  0x0002,   /* U+10924 */
  0x0002,   /* U+10925 */
  0x0002,   /* U+10926 */
  0x0002,   /* U+10927 */
  0x0002,   /* U+10928 */
  0x0002,   /* U+10929 */
  0x0002,   /* U+1092A */
  0x0002,   /* U+1092B */
  0x0002,   /* U+1092C */
  0x0002,   /* U+1092D */
  0x0002,   /* U+1092E */
  0x0002,   /* U+1092F */
  0x0002,   /* U+10930 */
  0x0002,   /* U+10931 */
  0x0002,   /* U+10932 */
  0x0002,   /* U+10933 */
  0x0002,   /* U+10934 */
  0x0002,   /* U+10935 */
  0x0002,   /* U+10936 */
  0x0002,   /* U+10937 */
  0x0002,   /* U+10938 */
  0x0002,   /* U+10939 */
  0x0002,   /* U+1093A */
  0x0002,   /* U+1093B */
  0x0002,   /* U+1093C */
  0x0002,   /* U+1093D */
  0x0002,   /* U+1093E */
  0x0002,   /* U+1093F */
  0x0002,   /* U+10940 */
  0x0002,   /* U+10941 */
  0x0002,   /* U+10942 */
  0x0002,   /* U+10943 */
  0x0002,   /* U+10944 */
  0x0002,   /* U+10945 */
  0x0002,   /* U+10946 */
  0x0002,   /* U+10947 */
  0x0002,   /* U+10948 */
  0x0002,   /* U+10949 */
  0x0002,   /* U+1094A */
  0x0002,   /* U+1094B */
  0x0002,   /* U+1094C */
  0x0002,   /* U+1094D */
  0x0002,   /* U+1094E */
  0x0002,   /* U+1094F */
  0x0002,   /* U+10950 */
  0x0002,   /* U+10951 */
  0x0002,   /* U+10952 */
  0x0002,   /* U+10953 */
  0x0002,   /* U+10954 */
  0x0002,   /* U+10955 */
  0x0002,   /* U+10956 */
  0x0002,   /* U+10957 */
  0x0002,   /* U+10958 */
  0x0002,   /* U+10959 */
  0x0002,   /* U+1095A */
  0x0002,   /* U+1095B */
  0x0002,   /* U+1095C */
  0x0002,   /* U+1095D */
  0x0002,   /* U+1095E */
  0x0002,   /* U+1095F */
  0x0002,   /* U+10960 */
  0x0002,   /* U+10961 */
  0x0002,   /* U+10962 */
  0x0002,   /* U+10963 */
  0x0002,   /* U+10964 */
  0x0002,   /* U+10965 */
  0x0002,   /* U+10966 */
  0x0002,   /* U+10967 */
  0x0002,   /* U+10968 */
  0x0002,   /* U+10969 */
  0x0002,   /* U+1096A */
  0x0002,   /* U+1096B */
  0x0002,   /* U+1096C */
  0x0002,   /* U+1096D */
  0x0002,   /* U+1096E */
  0x0002,   /* U+1096F */
  0x0002,   /* U+10970 */
  0x0002,   /* U+10971 */
  0x0002,   /* U+10972 */
  0x0002,   /* U+10973 */
  0x0002,   /* U+10974 */
  0x0002,   /* U+10975 */
  0x0002,   /* U+10976 */
  0x0002,   /* U+10977 */
  0x0002,   /* U+10978 */
  0x0002,   /* U+10979 */
  0x0002,   /* U+1097A */
  0x0002,   /* U+1097B */
  0x0002,   /* U+1097C */
  0x0002,   /* U+1097D */
  0x0002,   /* U+1097E */
  0x0002,   /* U+1097F */
  0x0004,   /* U+10980 */
  0x0004,   /* U+10981 */
  0x0004,   /* U+10982 */
  0x0004,   /* U+10983 */
  0x0004,   /* U+10984 */
  0x0004,   /* U+10985 */
  0x0004,   /* U+10986 */
  0x0004,   /* U+10987 */
  0x0004,   /* U+10988 */
  0x0004,   /* U+10989 */
  0x0004,   /* U+1098A */
  0x0004,   /* U+1098B */
  0x0004,   /* U+1098C */
  0x0004,   /* U+1098D */
  0x0004,   /* U+1098E */
  0x0004,   /* U+1098F */
  0x0004,   /* U+10990 */
  0x0004,   /* U+10991 */
  0x0004,   /* U+10992 */
  0x0004,   /* U+10993 */
  0x0004,   /* U+10994 */
  0x0004,   /* U+10995 */
  0x0004,   /* U+10996 */
  0x0004,   /* U+10997 */
  0x0004,   /* U+10998 */
  0x0004,   /* U+10999 */
  0x0004,   /* U+1099A */
  0x0004,   /* U+1099B */
  0x0004,   /* U+1099C */
  0x0004,   /* U+1099D */
  0x0002,   /* U+1099E */
  0x0002,   /* U+1099F */
  0x0002,   /* U+109A0 */
  0x0002,   /* U+109A1 */
  0x0002,   /* U+109A2 */
  0x0002,   /* U+109A3 */
  0x0002,   /* U+109A4 */
  0x0002,   /* U+109A5 */
  0x0002,   /* U+109A6 */
  0x0002,   /* U+109A7 */
  0x0002,   /* U+109A8 */
  0x0002,   /* U+109A9 */
  0x0002,   /* U+109AA */
  0x0002,   /* U+109AB */
  0x0002,   /* U+109AC */
  0x0002,   /* U+109AD */
  0x0002,   /* U+109AE */
  0x0002,   /* U+109AF */
  0x0004,   /* U+109B0 */
  0x0002,   /* U+109B1 */
  0x0002,   /* U+109B2 */
  0x0002,   /* U+109B3 */
  0x0002,   /* U+109B4 */
  0x0002,   /* U+109B5 */
  0x0002,   /* U+109B6 */
  0x0002,   /* U+109B7 */
  0x0002,   /* U+109B8 */
  0x0002,   /* U+109B9 */
  0x0002,   /* U+109BA */
  0x0002,   /* U+109BB */
  0x0002,   /* U+109BC */
  0x0002,   /* U+109BD */
  0x0002,   /* U+109BE */
  0x0002,   /* U+109BF */
  0x0002,   /* U+109C0 */
  0x0002,   /* U+109C1 */
  0x0002,   /* U+109C2 */
  0x0002,   /* U+109C3 */
  0x0002,   /* U+109C4 */
  0x0002,   /* U+109C5 */
  0x0002,   /* U+109C6 */
  0x0002,   /* U+109C7 */
  0x0002,   /* U+109C8 */
  0x0002,   /* U+109C9 */
  0x0002,   /* U+109CA */
  0x0002,   /* U+109CB */
  0x0002,   /* U+109CC */
  0x0002,   /* U+109CD */
  0x0002,   /* U+109CE */
  0x0002,   /* U+109CF */
  0x0002,   /* U+109D0 */
  0x0002,   /* U+109D1 */
  0x0002,   /* U+109D2 */
  0x0002,   /* U+109D3 */
  0x0002,   /* U+109D4 */
  0x0002,   /* U+109D5 */
  0x0002,   /* U+109D6 */
  0x0002,   /* U+109D7 */
  0x0002,   /* U+109D8 */
  0x0002,   /* U+109D9 */
  0x0002,   /* U+109DA */
  0x0002,   /* U+109DB */
  0x0002,   /* U+109DC */
  0x0002,   /* U+109DD */
  0x0002,   /* U+109DE */
  0x0002,   /* U+109DF */
  0x0002,   /* U+109E0 */
  0x0002,   /* U+109E1 */
  0x0002,   /* U+109E2 */
  0x0002,   /* U+109E3 */
  0x0002,   /* U+109E4 */
  0x0002,   /* U+109E5 */
  0x0002,   /* U+109E6 */
  0x0002,   /* U+109E7 */
  0x0002,   /* U+109E8 */
  0x0002,   /* U+109E9 */
  0x0002,   /* U+109EA */
  0x0002,   /* U+109EB */
  0x0002,   /* U+109EC */
  0x0002,   /* U+109ED */
  0x0002,   /* U+109EE */
  0x0002,   /* U+109EF */
  0x0002,   /* U+109F0 */
  0x0002,   /* U+109F1 */
  0x0002,   /* U+109F2 */
  0x0002,   /* U+109F3 */
  0x0002,   /* U+109F4 */
  0x0002,   /* U+109F5 */
  0x0002,   /* U+109F6 */
  0x0002,   /* U+109F7 */
  0x0002,   /* U+109F8 */
  0x0002,   /* U+109F9 */
  0x0002,   /* U+109FA */
  0x0002,   /* U+109FB */
  0x0002,   /* U+109FC */
  0x0002,   /* U+109FD */
  0x0002,   /* U+109FE */
  0x0002,   /* U+109FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10900 */
  0x0000,   /* U+10901 */
  0x0000,   /* U+10902 */
  0x0000,   /* U+10903 */
  0x0000,   /* U+10904 */
  0x0000,   /* U+10905 */
  0x0000,   /* U+10906 */
  0x0000,   /* U+10907 */
  0x0000,   /* U+10908 */
  0x0000,   /* U+10909 */
  0x0000,   /* U+1090A */
  0x0000,   /* U+1090B */
  0x0000,   /* U+1090C */
  0x0000,   /* U+1090D */
  0x0000,   /* U+1090E */
  0x0000,   /* U+1090F */
  0x0000,   /* U+10910 */
  0x0000,   /* U+10911 */
  0x0000,   /* U+10912 */
  0x0000,   /* U+10913 */
  0x0000,   /* U+10914 */
  0x0000,   /* U+10915 */
  0x0000,   /* U+10916 */
  0x0000,   /* U+10917 */
  0x0000,   /* U+10918 */
  0x0000,   /* U+10919 */
  0x0000,   /* U+1091A */
  0x0000,   /* U+1091B */
  0x891C,   /* U+1091C */
  0x891D,   /* U+1091D */
  0x891E,   /* U+1091E */
  0x0000,   /* U+1091F */
  0x0000,   /* U+10920 */
  0x0000,   /* U+10921 */
  0x0000,   /* U+10922 */
  0x0000,   /* U+10923 */
  0x0000,   /* U+10924 */
  0x0000,   /* U+10925 */
  0x0000,   /* U+10926 */
  0x0000,   /* U+10927 */
  0x0000,   /* U+10928 */
  0x0000,   /* U+10929 */
  0x0000,   /* U+1092A */
  0x0000,   /* U+1092B */
  0x0000,   /* U+1092C */
  0x0000,   /* U+1092D */
  0x0000,   /* U+1092E */
  0x0000,   /* U+1092F */
  0x0000,   /* U+10930 */
  0x0000,   /* U+10931 */
  0x0000,   /* U+10932 */
  0x0000,   /* U+10933 */
  0x0000,   /* U+10934 */
  0x0000,   /* U+10935 */
  0x0000,   /* U+10936 */
  0x0000,   /* U+10937 */
  0x0000,   /* U+10938 */
  0x0000,   /* U+10939 */
  0x893A,   /* U+1093A */
  0x893B,   /* U+1093B */
  0x893C,   /* U+1093C */
  0x893D,   /* U+1093D */
  0x893E,   /* U+1093E */
  0x0000,   /* U+1093F */
  0x8940,   /* U+10940 */
  0x8941,   /* U+10941 */
  0x8942,   /* U+10942 */
  0x8943,   /* U+10943 */
  0x8944,   /* U+10944 */
  0x8945,   /* U+10945 */
  0x8946,   /* U+10946 */
  0x8947,   /* U+10947 */
  0x8948,   /* U+10948 */
  0x8949,   /* U+10949 */
  0x894A,   /* U+1094A */
  0x894B,   /* U+1094B */
  0x894C,   /* U+1094C */
  0x894D,   /* U+1094D */
  0x894E,   /* U+1094E */
  0x894F,   /* U+1094F */
  0x8950,   /* U+10950 */
  0x8951,   /* U+10951 */
  0x8952,   /* U+10952 */
  0x8953,   /* U+10953 */
  0x8954,   /* U+10954 */
  0x8955,   /* U+10955 */
  0x8956,   /* U+10956 */
  0x8957,   /* U+10957 */
  0x8958,   /* U+10958 */
  0x8959,   /* U+10959 */
  0x895A,   /* U+1095A */
  0x895B,   /* U+1095B */
  0x895C,   /* U+1095C */
  0x895D,   /* U+1095D */
  0x895E,   /* U+1095E */
  0x895F,   /* U+1095F */
  0x8960,   /* U+10960 */
  0x8961,   /* U+10961 */
  0x8962,   /* U+10962 */
  0x8963,   /* U+10963 */
  0x8964,   /* U+10964 */
  0x8965,   /* U+10965 */
  0x8966,   /* U+10966 */
  0x8967,   /* U+10967 */
  0x8968,   /* U+10968 */
  0x8969,   /* U+10969 */
  0x896A,   /* U+1096A */
  0x896B,   /* U+1096B */
  0x896C,   /* U+1096C */
  0x896D,   /* U+1096D */
  0x896E,   /* U+1096E */
  0x896F,   /* U+1096F */
  0x8970,   /* U+10970 */
  0x8971,   /* U+10971 */
  0x8972,   /* U+10972 */
  0x8973,   /* U+10973 */
  0x8974,   /* U+10974 */
  0x8975,   /* U+10975 */
  0x8976,   /* U+10976 */
  0x8977,   /* U+10977 */
  0x8978,   /* U+10978 */
  0x8979,   /* U+10979 */
  0x897A,   /* U+1097A */
  0x897B,   /* U+1097B */
  0x897C,   /* U+1097C */
  0x897D,   /* U+1097D */
  0x897E,   /* U+1097E */
  0x897F,   /* U+1097F */
  0x0000,   /* U+10980 */
  0x0000,   /* U+10981 */
  0x0000,   /* U+10982 */
  0x0000,   /* U+10983 */
  0x0000,   /* U+10984 */
  0x0000,   /* U+10985 */
  0x0000,   /* U+10986 */
  0x0000,   /* U+10987 */
  0x0000,   /* U+10988 */
  0x0000,   /* U+10989 */
  0x0000,   /* U+1098A */
  0x0000,   /* U+1098B */
  0x0000,   /* U+1098C */
  0x0000,   /* U+1098D */
  0x0000,   /* U+1098E */
  0x0000,   /* U+1098F */
  0x0000,   /* U+10990 */
  0x0000,   /* U+10991 */
  0x0000,   /* U+10992 */
  0x0000,   /* U+10993 */
  0x0000,   /* U+10994 */
  0x0000,   /* U+10995 */
  0x0000,   /* U+10996 */
  0x0000,   /* U+10997 */
  0x0000,   /* U+10998 */
  0x0000,   /* U+10999 */
  0x0000,   /* U+1099A */
  0x0000,   /* U+1099B */
  0x0000,   /* U+1099C */
  0x0000,   /* U+1099D */
  0x0000,   /* U+1099E */
  0x0000,   /* U+1099F */
  0x0000,   /* U+109A0 */
  0x0000,   /* U+109A1 */
  0x0000,   /* U+109A2 */
  0x0000,   /* U+109A3 */
  0x0000,   /* U+109A4 */
  0x0000,   /* U+109A5 */
  0x0000,   /* U+109A6 */
  0x0000,   /* U+109A7 */
  0x0000,   /* U+109A8 */
  0x0000,   /* U+109A9 */
  0x0000,   /* U+109AA */
  0x0000,   /* U+109AB */
  0x0000,   /* U+109AC */
  0x0000,   /* U+109AD */
  0x0000,   /* U+109AE */
  0x0000,   /* U+109AF */
  0x0000,   /* U+109B0 */
  0x0000,   /* U+109B1 */
  0x0000,   /* U+109B2 */
  0x0000,   /* U+109B3 */
  0x0000,   /* U+109B4 */
  0x0000,   /* U+109B5 */
  0x0000,   /* U+109B6 */
  0x0000,   /* U+109B7 */
  0x89B8,   /* U+109B8 */
  0x89B9,   /* U+109B9 */
  0x89BA,   /* U+109BA */
  0x89BB,   /* U+109BB */
  0x0000,   /* U+109BC */
  0x0000,   /* U+109BD */
  0x0000,   /* U+109BE */
  0x0000,   /* U+109BF */
  0x0000,   /* U+109C0 */
  0x0000,   /* U+109C1 */
  0x0000,   /* U+109C2 */
  0x0000,   /* U+109C3 */
  0x0000,   /* U+109C4 */
  0x0000,   /* U+109C5 */
  0x0000,   /* U+109C6 */
  0x0000,   /* U+109C7 */
  0x0000,   /* U+109C8 */
  0x0000,   /* U+109C9 */
  0x0000,   /* U+109CA */
  0x0000,   /* U+109CB */
  0x0000,   /* U+109CC */
  0x0000,   /* U+109CD */
  0x0000,   /* U+109CE */
  0x0000,   /* U+109CF */
  0x89D0,   /* U+109D0 */
  0x89D1,   /* U+109D1 */
  0x0000,   /* U+109D2 */
  0x0000,   /* U+109D3 */
  0x0000,   /* U+109D4 */
  0x0000,   /* U+109D5 */
  0x0000,   /* U+109D6 */
  0x0000,   /* U+109D7 */
  0x0000,   /* U+109D8 */
  0x0000,   /* U+109D9 */
  0x0000,   /* U+109DA */
  0x0000,   /* U+109DB */
  0x0000,   /* U+109DC */
  0x0000,   /* U+109DD */
  0x0000,   /* U+109DE */
  0x0000,   /* U+109DF */
  0x0000,   /* U+109E0 */
  0x0000,   /* U+109E1 */
  0x0000,   /* U+109E2 */
  0x0000,   /* U+109E3 */
  0x0000,   /* U+109E4 */
  0x0000,   /* U+109E5 */
  0x0000,   /* U+109E6 */
  0x0000,   /* U+109E7 */
  0x0000,   /* U+109E8 */
  0x0000,   /* U+109E9 */
  0x0000,   /* U+109EA */
  0x0000,   /* U+109EB */
  0x0000,   /* U+109EC */
  0x0000,   /* U+109ED */
  0x0000,   /* U+109EE */
  0x0000,   /* U+109EF */
  0x0000,   /* U+109F0 */
  0x0000,   /* U+109F1 */
  0x0000,   /* U+109F2 */
  0x0000,   /* U+109F3 */
  0x0000,   /* U+109F4 */
  0x0000,   /* U+109F5 */
  0x0000,   /* U+109F6 */
  0x0000,   /* U+109F7 */
  0x0000,   /* U+109F8 */
  0x0000,   /* U+109F9 */
  0x0000,   /* U+109FA */
  0x0000,   /* U+109FB */
  0x0000,   /* U+109FC */
  0x0000,   /* U+109FD */
  0x0000,   /* U+109FE */
  0x0000,   /* U+109FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10900 */
  0x0000,   /* U+10901 */
  0x0000,   /* U+10902 */
  0x0000,   /* U+10903 */
  0x0000,   /* U+10904 */
  0x0000,   /* U+10905 */
  0x0000,   /* U+10906 */
  0x0000,   /* U+10907 */
  0x0000,   /* U+10908 */
  0x0000,   /* U+10909 */
  0x0000,   /* U+1090A */
  0x0000,   /* U+1090B */
  0x0000,   /* U+1090C */
  0x0000,   /* U+1090D */
  0x0000,   /* U+1090E */
  0x0000,   /* U+1090F */
  0x0000,   /* U+10910 */
  0x0000,   /* U+10911 */
  0x0000,   /* U+10912 */
  0x0000,   /* U+10913 */
  0x0000,   /* U+10914 */
  0x0000,   /* U+10915 */
  0x0000,   /* U+10916 */
  0x0000,   /* U+10917 */
  0x0000,   /* U+10918 */
  0x0000,   /* U+10919 */
  0x0000,   /* U+1091A */
  0x0000,   /* U+1091B */
  0x0000,   /* U+1091C */
  0x0000,   /* U+1091D */
  0x0000,   /* U+1091E */
  0x0000,   /* U+1091F */
  0x0000,   /* U+10920 */
  0x0000,   /* U+10921 */
  0x0000,   /* U+10922 */
  0x0000,   /* U+10923 */
  0x0000,   /* U+10924 */
  0x0000,   /* U+10925 */
  0x0000,   /* U+10926 */
  0x0000,   /* U+10927 */
  0x0000,   /* U+10928 */
  0x0000,   /* U+10929 */
  0x0000,   /* U+1092A */
  0x0000,   /* U+1092B */
  0x0000,   /* U+1092C */
  0x0000,   /* U+1092D */
  0x0000,   /* U+1092E */
  0x0000,   /* U+1092F */
  0x0000,   /* U+10930 */
  0x0000,   /* U+10931 */
  0x0000,   /* U+10932 */
  0x0000,   /* U+10933 */
  0x0000,   /* U+10934 */
  0x0000,   /* U+10935 */
  0x0000,   /* U+10936 */
  0x0000,   /* U+10937 */
  0x0000,   /* U+10938 */
  0x0000,   /* U+10939 */
  0x0000,   /* U+1093A */
  0x0000,   /* U+1093B */
  0x0000,   /* U+1093C */
  0x0000,   /* U+1093D */
  0x0000,   /* U+1093E */
  0x0000,   /* U+1093F */
  0x0000,   /* U+10940 */
  0x0000,   /* U+10941 */
  0x0000,   /* U+10942 */
  0x0000,   /* U+10943 */
  0x0000,   /* U+10944 */
  0x0000,   /* U+10945 */
  0x0000,   /* U+10946 */
  0x0000,   /* U+10947 */
  0x0000,   /* U+10948 */
  0x0000,   /* U+10949 */
  0x0000,   /* U+1094A */
  0x0000,   /* U+1094B */
  0x0000,   /* U+1094C */
  0x0000,   /* U+1094D */
  0x0000,   /* U+1094E */
  0x0000,   /* U+1094F */
  0x0000,   /* U+10950 */
  0x0000,   /* U+10951 */
  0x0000,   /* U+10952 */
  0x0000,   /* U+10953 */
  0x0000,   /* U+10954 */
  0x0000,   /* U+10955 */
  0x0000,   /* U+10956 */
  0x0000,   /* U+10957 */
  0x0000,   /* U+10958 */
  0x0000,   /* U+10959 */
  0x0000,   /* U+1095A */
  0x0000,   /* U+1095B */
  0x0000,   /* U+1095C */
  0x0000,   /* U+1095D */
  0x0000,   /* U+1095E */
  0x0000,   /* U+1095F */
  0x0000,   /* U+10960 */
  0x0000,   /* U+10961 */
  0x0000,   /* U+10962 */
  0x0000,   /* U+10963 */
  0x0000,   /* U+10964 */
  0x0000,   /* U+10965 */
  0x0000,   /* U+10966 */
  0x0000,   /* U+10967 */
  0x0000,   /* U+10968 */
  0x0000,   /* U+10969 */
  0x0000,   /* U+1096A */
  0x0000,   /* U+1096B */
  0x0000,   /* U+1096C */
  0x0000,   /* U+1096D */
  0x0000,   /* U+1096E */
  0x0000,   /* U+1096F */
  0x0000,   /* U+10970 */
  0x0000,   /* U+10971 */
  0x0000,   /* U+10972 */
  0x0000,   /* U+10973 */
  0x0000,   /* U+10974 */
  0x0000,   /* U+10975 */
  0x0000,   /* U+10976 */
  0x0000,   /* U+10977 */
  0x0000,   /* U+10978 */
  0x0000,   /* U+10979 */
  0x0000,   /* U+1097A */
  0x0000,   /* U+1097B */
  0x0000,   /* U+1097C */
  0x0000,   /* U+1097D */
  0x0000,   /* U+1097E */
  0x0000,   /* U+1097F */
  0x0110,   /* U+10980 */
  0x0110,   /* U+10981 */
  0x0110,   /* U+10982 */
  0x0110,   /* U+10983 */
  0x0110,   /* U+10984 */
  0x0110,   /* U+10985 */
  0x0110,   /* U+10986 */
  0x0111,   /* U+10987 */
  0x0110,   /* U+10988 */
  0x0110,   /* U+10989 */
  0x0110,   /* U+1098A */
  0x0111,   /* U+1098B */
  0x0110,   /* U+1098C */
  0x0111,   /* U+1098D */
  0x0110,   /* U+1098E */
  0x0111,   /* U+1098F */
  0x0110,   /* U+10990 */
  0x0110,   /* U+10991 */
  0x0110,   /* U+10992 */
  0x0111,   /* U+10993 */
  0x0112,   /* U+10994 */
  0x0110,   /* U+10995 */
  0x0110,   /* U+10996 */
  0x0110,   /* U+10997 */
  0x0110,   /* U+10998 */
  0x0111,   /* U+10999 */
  0x0110,   /* U+1099A */
  0x0111,   /* U+1099B */
  0x0110,   /* U+1099C */
  0x0110,   /* U+1099D */
  0x0000,   /* U+1099E */
  0x0000,   /* U+1099F */
  0x0000,   /* U+109A0 */
  0x0000,   /* U+109A1 */
  0x0000,   /* U+109A2 */
  0x0000,   /* U+109A3 */
  0x0000,   /* U+109A4 */
  0x0000,   /* U+109A5 */
  0x0000,   /* U+109A6 */
  0x0000,   /* U+109A7 */
  0x0000,   /* U+109A8 */
  0x0000,   /* U+109A9 */
  0x0000,   /* U+109AA */
  0x0000,   /* U+109AB */
  0x0000,   /* U+109AC */
  0x0000,   /* U+109AD */
  0x0000,   /* U+109AE */
  0x0000,   /* U+109AF */
  0x0110,   /* U+109B0 */
  0x0000,   /* U+109B1 */
  0x0000,   /* U+109B2 */
  0x0000,   /* U+109B3 */
  0x0000,   /* U+109B4 */
  0x0000,   /* U+109B5 */
  0x0000,   /* U+109B6 */
  0x0000,   /* U+109B7 */
  0x0000,   /* U+109B8 */
  0x0000,   /* U+109B9 */
  0x0000,   /* U+109BA */
  0x0000,   /* U+109BB */
  0x0000,   /* U+109BC */
  0x0000,   /* U+109BD */
  0x0000,   /* U+109BE */
  0x0000,   /* U+109BF */
  0x0000,   /* U+109C0 */
  0x0000,   /* U+109C1 */
  0x0000,   /* U+109C2 */
  0x0000,   /* U+109C3 */
  0x0000,   /* U+109C4 */
  0x0000,   /* U+109C5 */
  0x0000,   /* U+109C6 */
  0x0000,   /* U+109C7 */
  0x0000,   /* U+109C8 */
  0x0000,   /* U+109C9 */
  0x0000,   /* U+109CA */
  0x0000,   /* U+109CB */
  0x0000,   /* U+109CC */
  0x0000,   /* U+109CD */
  0x0000,   /* U+109CE */
  0x0000,   /* U+109CF */
  0x0000,   /* U+109D0 */
  0x0000,   /* U+109D1 */
  0x0000,   /* U+109D2 */
  0x0000,   /* U+109D3 */
  0x0000,   /* U+109D4 */
  0x0000,   /* U+109D5 */
  0x0000,   /* U+109D6 */
  0x0000,   /* U+109D7 */
  0x0000,   /* U+109D8 */
  0x0000,   /* U+109D9 */
  0x0000,   /* U+109DA */
  0x0000,   /* U+109DB */
  0x0000,   /* U+109DC */
  0x0000,   /* U+109DD */
  0x0000,   /* U+109DE */
  0x0000,   /* U+109DF */
  0x0000,   /* U+109E0 */
  0x0000,   /* U+109E1 */
  0x0000,   /* U+109E2 */
  0x0000,   /* U+109E3 */
  0x0000,   /* U+109E4 */
  0x0000,   /* U+109E5 */
  0x0000,   /* U+109E6 */
  0x0000,   /* U+109E7 */
  0x0000,   /* U+109E8 */
  0x0000,   /* U+109E9 */
  0x0000,   /* U+109EA */
  0x0000,   /* U+109EB */
  0x0000,   /* U+109EC */
  0x0000,   /* U+109ED */
  0x0000,   /* U+109EE */
  0x0000,   /* U+109EF */
  0x0000,   /* U+109F0 */
  0x0000,   /* U+109F1 */
  0x0000,   /* U+109F2 */
  0x0000,   /* U+109F3 */
  0x0000,   /* U+109F4 */
  0x0000,   /* U+109F5 */
  0x0000,   /* U+109F6 */
  0x0000,   /* U+109F7 */
  0x0000,   /* U+109F8 */
  0x0000,   /* U+109F9 */
  0x0000,   /* U+109FA */
  0x0000,   /* U+109FB */
  0x0000,   /* U+109FC */
  0x0000,   /* U+109FD */
  0x0000,   /* U+109FE */
  0x0000,   /* U+109FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10900 */
  0x0000,   /* U+10901 */
  0x0000,   /* U+10902 */
  0x0000,   /* U+10903 */
  0x0000,   /* U+10904 */
  0x0000,   /* U+10905 */
  0x0000,   /* U+10906 */
  0x0000,   /* U+10907 */
  0x0000,   /* U+10908 */
  0x0000,   /* U+10909 */
  0x0000,   /* U+1090A */
  0x0000,   /* U+1090B */
  0x0000,   /* U+1090C */
  0x0000,   /* U+1090D */
  0x0000,   /* U+1090E */
  0x0000,   /* U+1090F */
  0x0000,   /* U+10910 */
  0x0000,   /* U+10911 */
  0x0000,   /* U+10912 */
  0x0000,   /* U+10913 */
  0x0000,   /* U+10914 */
  0x0000,   /* U+10915 */
  0x0000,   /* U+10916 */
  0x0000,   /* U+10917 */
  0x0000,   /* U+10918 */
  0x0000,   /* U+10919 */
  0x0000,   /* U+1091A */
  0x0000,   /* U+1091B */
  0x0000,   /* U+1091C */
  0x0000,   /* U+1091D */
  0x0000,   /* U+1091E */
  0x0000,   /* U+1091F */
  0x0000,   /* U+10920 */
  0x0000,   /* U+10921 */
  0x0000,   /* U+10922 */
  0x0000,   /* U+10923 */
  0x0000,   /* U+10924 */
  0x0000,   /* U+10925 */
  0x0000,   /* U+10926 */
  0x0000,   /* U+10927 */
  0x0000,   /* U+10928 */
  0x0000,   /* U+10929 */
  0x0000,   /* U+1092A */
  0x0000,   /* U+1092B */
  0x0000,   /* U+1092C */
  0x0000,   /* U+1092D */
  0x0000,   /* U+1092E */
  0x0000,   /* U+1092F */
  0x0000,   /* U+10930 */
  0x0000,   /* U+10931 */
  0x0000,   /* U+10932 */
  0x0000,   /* U+10933 */
  0x0000,   /* U+10934 */
  0x0000,   /* U+10935 */
  0x0000,   /* U+10936 */
  0x0000,   /* U+10937 */
  0x0000,   /* U+10938 */
  0x0000,   /* U+10939 */
  0x0000,   /* U+1093A */
  0x0000,   /* U+1093B */
  0x0000,   /* U+1093C */
  0x0000,   /* U+1093D */
  0x0000,   /* U+1093E */
  0x0000,   /* U+1093F */
  0x0000,   /* U+10940 */
  0x0000,   /* U+10941 */
  0x0000,   /* U+10942 */
  0x0000,   /* U+10943 */
  0x0000,   /* U+10944 */
  0x0000,   /* U+10945 */
  0x0000,   /* U+10946 */
  0x0000,   /* U+10947 */
  0x0000,   /* U+10948 */
  0x0000,   /* U+10949 */
  0x0000,   /* U+1094A */
  0x0000,   /* U+1094B */
  0x0000,   /* U+1094C */
  0x0000,   /* U+1094D */
  0x0000,   /* U+1094E */
  0x0000,   /* U+1094F */
  0x0000,   /* U+10950 */
  0x0000,   /* U+10951 */
  0x0000,   /* U+10952 */
  0x0000,   /* U+10953 */
  0x0000,   /* U+10954 */
  0x0000,   /* U+10955 */
  0x0000,   /* U+10956 */
  0x0000,   /* U+10957 */
  0x0000,   /* U+10958 */
  0x0000,   /* U+10959 */
  0x0000,   /* U+1095A */
  0x0000,   /* U+1095B */
  0x0000,   /* U+1095C */
  0x0000,   /* U+1095D */
  0x0000,   /* U+1095E */
  0x0000,   /* U+1095F */
  0x0000,   /* U+10960 */
  0x0000,   /* U+10961 */
  0x0000,   /* U+10962 */
  0x0000,   /* U+10963 */
  0x0000,   /* U+10964 */
  0x0000,   /* U+10965 */
  0x0000,   /* U+10966 */
  0x0000,   /* U+10967 */
  0x0000,   /* U+10968 */
  0x0000,   /* U+10969 */
  0x0000,   /* U+1096A */
  0x0000,   /* U+1096B */
  0x0000,   /* U+1096C */
  0x0000,   /* U+1096D */
  0x0000,   /* U+1096E */
  0x0000,   /* U+1096F */
  0x0000,   /* U+10970 */
  0x0000,   /* U+10971 */
  0x0000,   /* U+10972 */
  0x0000,   /* U+10973 */
  0x0000,   /* U+10974 */
  0x0000,   /* U+10975 */
  0x0000,   /* U+10976 */
  0x0000,   /* U+10977 */
  0x0000,   /* U+10978 */
  0x0000,   /* U+10979 */
  0x0000,   /* U+1097A */
  0x0000,   /* U+1097B */
  0x0000,   /* U+1097C */
  0x0000,   /* U+1097D */
  0x0000,   /* U+1097E */
  0x0000,   /* U+1097F */
  0x0004,   /* U+10980 */
  0x0004,   /* U+10981 */
  0x0004,   /* U+10982 */
  0x0004,   /* U+10983 */
  0x0004,   /* U+10984 */
  0x0004,   /* U+10985 */
  0x0004,   /* U+10986 */
  0x0004,   /* U+10987 */
  0x0004,   /* U+10988 */
  0x0004,   /* U+10989 */
  0x0004,   /* U+1098A */
  0x0004,   /* U+1098B */
  0x0004,   /* U+1098C */
  0x0004,   /* U+1098D */
  0x0004,   /* U+1098E */
  0x0004,   /* U+1098F */
  0x0004,   /* U+10990 */
  0x0004,   /* U+10991 */
  0x0004,   /* U+10992 */
  0x0004,   /* U+10993 */
  0x0004,   /* U+10994 */
  0x0004,   /* U+10995 */
  0x0004,   /* U+10996 */
  0x0004,   /* U+10997 */
  0x0004,   /* U+10998 */
  0x0004,   /* U+10999 */
  0x0004,   /* U+1099A */
  0x0004,   /* U+1099B */
  0x0004,   /* U+1099C */
  0x0004,   /* U+1099D */
  0x0000,   /* U+1099E */
  0x0000,   /* U+1099F */
  0x0000,   /* U+109A0 */
  0x0000,   /* U+109A1 */
  0x0000,   /* U+109A2 */
  0x0000,   /* U+109A3 */
  0x0000,   /* U+109A4 */
  0x0000,   /* U+109A5 */
  0x0000,   /* U+109A6 */
  0x0000,   /* U+109A7 */
  0x0000,   /* U+109A8 */
  0x0000,   /* U+109A9 */
  0x0000,   /* U+109AA */
  0x0000,   /* U+109AB */
  0x0000,   /* U+109AC */
  0x0000,   /* U+109AD */
  0x0000,   /* U+109AE */
  0x0000,   /* U+109AF */
  0x0004,   /* U+109B0 */
  0x0000,   /* U+109B1 */
  0x0000,   /* U+109B2 */
  0x0000,   /* U+109B3 */
  0x0000,   /* U+109B4 */
  0x0000,   /* U+109B5 */
  0x0000,   /* U+109B6 */
  0x0000,   /* U+109B7 */
  0x0000,   /* U+109B8 */
  0x0000,   /* U+109B9 */
  0x0000,   /* U+109BA */
  0x0000,   /* U+109BB */
  0x0000,   /* U+109BC */
  0x0000,   /* U+109BD */
  0x0000,   /* U+109BE */
  0x0000,   /* U+109BF */
  0x0000,   /* U+109C0 */
  0x0000,   /* U+109C1 */
  0x0000,   /* U+109C2 */
  0x0000,   /* U+109C3 */
  0x0000,   /* U+109C4 */
  0x0000,   /* U+109C5 */
  0x0000,   /* U+109C6 */
  0x0000,   /* U+109C7 */
  0x0000,   /* U+109C8 */
  0x0000,   /* U+109C9 */
  0x0000,   /* U+109CA */
  0x0000,   /* U+109CB */
  0x0000,   /* U+109CC */
  0x0000,   /* U+109CD */
  0x0000,   /* U+109CE */
  0x0000,   /* U+109CF */
  0x0000,   /* U+109D0 */
  0x0000,   /* U+109D1 */
  0x0000,   /* U+109D2 */
  0x0000,   /* U+109D3 */
  0x0000,   /* U+109D4 */
  0x0000,   /* U+109D5 */
  0x0000,   /* U+109D6 */
  0x0000,   /* U+109D7 */
  0x0000,   /* U+109D8 */
  0x0000,   /* U+109D9 */
  0x0000,   /* U+109DA */
  0x0000,   /* U+109DB */
  0x0000,   /* U+109DC */
  0x0000,   /* U+109DD */
  0x0000,   /* U+109DE */
  0x0000,   /* U+109DF */
  0x0000,   /* U+109E0 */
  0x0000,   /* U+109E1 */
  0x0000,   /* U+109E2 */
  0x0000,   /* U+109E3 */
  0x0000,   /* U+109E4 */
  0x0000,   /* U+109E5 */
  0x0000,   /* U+109E6 */
  0x0000,   /* U+109E7 */
  0x0000,   /* U+109E8 */
  0x0000,   /* U+109E9 */
  0x0000,   /* U+109EA */
  0x0000,   /* U+109EB */
  0x0000,   /* U+109EC */
  0x0000,   /* U+109ED */
  0x0000,   /* U+109EE */
  0x0000,   /* U+109EF */
  0x0000,   /* U+109F0 */
  0x0000,   /* U+109F1 */
  0x0000,   /* U+109F2 */
  0x0000,   /* U+109F3 */
  0x0000,   /* U+109F4 */
  0x0000,   /* U+109F5 */
  0x0000,   /* U+109F6 */
  0x0000,   /* U+109F7 */
  0x0000,   /* U+109F8 */
  0x0000,   /* U+109F9 */
  0x0000,   /* U+109FA */
  0x0000,   /* U+109FB */
  0x0000,   /* U+109FC */
  0x0000,   /* U+109FD */
  0x0000,   /* U+109FE */
  0x0000,   /* U+109FF */
};

uint16 uca900_p10A[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 2, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2D0D,   /* U+10A00 */
  0x2D0E,   /* U+10A01 */
  0x2D0F,   /* U+10A02 */
  0x2D10,   /* U+10A03 */
  0xFBC2,   /* U+10A04 */
  0x2D11,   /* U+10A05 */
  0x2D12,   /* U+10A06 */
  0xFBC2,   /* U+10A07 */
  0xFBC2,   /* U+10A08 */
  0xFBC2,   /* U+10A09 */
  0xFBC2,   /* U+10A0A */
  0xFBC2,   /* U+10A0B */
  0x2D13,   /* U+10A0C */
  0x0000,   /* U+10A0D */
  0x0000,   /* U+10A0E */
  0x0000,   /* U+10A0F */
  0x2D14,   /* U+10A10 */
  0x2D15,   /* U+10A11 */
  0x2D16,   /* U+10A12 */
  0x2D17,   /* U+10A13 */
  0xFBC2,   /* U+10A14 */
  0x2D18,   /* U+10A15 */
  0x2D19,   /* U+10A16 */
  0x2D1A,   /* U+10A17 */
  0xFBC2,   /* U+10A18 */
  0x2D1B,   /* U+10A19 */
  0x2D1C,   /* U+10A1A */
  0x2D1D,   /* U+10A1B */
  0x2D1E,   /* U+10A1C */
  0x2D1F,   /* U+10A1D */
  0x2D20,   /* U+10A1E */
  0x2D21,   /* U+10A1F */
  0x2D22,   /* U+10A20 */
  0x2D23,   /* U+10A21 */
  0x2D24,   /* U+10A22 */
  0x2D25,   /* U+10A23 */
  0x2D26,   /* U+10A24 */
  0x2D27,   /* U+10A25 */
  0x2D28,   /* U+10A26 */
  0x2D29,   /* U+10A27 */
  0x2D2A,   /* U+10A28 */
  0x2D2B,   /* U+10A29 */
  0x2D2C,   /* U+10A2A */
  0x2D2D,   /* U+10A2B */
  0x2D2E,   /* U+10A2C */
  0x2D2F,   /* U+10A2D */
  0x2D30,   /* U+10A2E */
  0x2D31,   /* U+10A2F */
  0x2D32,   /* U+10A30 */
  0x2D33,   /* U+10A31 */
  0x2D34,   /* U+10A32 */
  0x2D35,   /* U+10A33 */
  0xFBC2,   /* U+10A34 */
  0xFBC2,   /* U+10A35 */
  0xFBC2,   /* U+10A36 */
  0xFBC2,   /* U+10A37 */
  0x0000,   /* U+10A38 */
  0x0000,   /* U+10A39 */
  0x0000,   /* U+10A3A */
  0xFBC2,   /* U+10A3B */
  0xFBC2,   /* U+10A3C */
  0xFBC2,   /* U+10A3D */
  0xFBC2,   /* U+10A3E */
  0x2D36,   /* U+10A3F */
  0x1C3E,   /* U+10A40 */
  0x1C3F,   /* U+10A41 */
  0x1C40,   /* U+10A42 */
  0x1C41,   /* U+10A43 */
  0x1B7E,   /* U+10A44 */
  0x1B7F,   /* U+10A45 */
  0x1B80,   /* U+10A46 */
  0x1B81,   /* U+10A47 */
  0xFBC2,   /* U+10A48 */
  0xFBC2,   /* U+10A49 */
  0xFBC2,   /* U+10A4A */
  0xFBC2,   /* U+10A4B */
  0xFBC2,   /* U+10A4C */
  0xFBC2,   /* U+10A4D */
  0xFBC2,   /* U+10A4E */
  0xFBC2,   /* U+10A4F */
  0x0438,   /* U+10A50 */
  0x0439,   /* U+10A51 */
  0x043A,   /* U+10A52 */
  0x043B,   /* U+10A53 */
  0x043C,   /* U+10A54 */
  0x043D,   /* U+10A55 */
  0x02AA,   /* U+10A56 */
  0x02AB,   /* U+10A57 */
  0x043E,   /* U+10A58 */
  0xFBC2,   /* U+10A59 */
  0xFBC2,   /* U+10A5A */
  0xFBC2,   /* U+10A5B */
  0xFBC2,   /* U+10A5C */
  0xFBC2,   /* U+10A5D */
  0xFBC2,   /* U+10A5E */
  0xFBC2,   /* U+10A5F */
  0x4846,   /* U+10A60 */
  0x4847,   /* U+10A61 */
  0x4848,   /* U+10A62 */
  0x4849,   /* U+10A63 */
  0x484A,   /* U+10A64 */
  0x484B,   /* U+10A65 */
  0x484C,   /* U+10A66 */
  0x484D,   /* U+10A67 */
  0x484E,   /* U+10A68 */
  0x484F,   /* U+10A69 */
  0x4850,   /* U+10A6A */
  0x4851,   /* U+10A6B */
  0x4852,   /* U+10A6C */
  0x4853,   /* U+10A6D */
  0x4854,   /* U+10A6E */
  0x4855,   /* U+10A6F */
  0x4856,   /* U+10A70 */
  0x4857,   /* U+10A71 */
  0x4858,   /* U+10A72 */
  0x4859,   /* U+10A73 */
  0x485A,   /* U+10A74 */
  0x485B,   /* U+10A75 */
  0x485C,   /* U+10A76 */
  0x485D,   /* U+10A77 */
  0x485E,   /* U+10A78 */
  0x485F,   /* U+10A79 */
  0x4860,   /* U+10A7A */
  0x4861,   /* U+10A7B */
  0x4862,   /* U+10A7C */
  0x1C3E,   /* U+10A7D */
  0x1B57,   /* U+10A7E */
  0x1B58,   /* U+10A7F */
  0x4863,   /* U+10A80 */
  0x4864,   /* U+10A81 */
  0x4865,   /* U+10A82 */
  0x4866,   /* U+10A83 */
  0x4867,   /* U+10A84 */
  0x4868,   /* U+10A85 */
  0x4869,   /* U+10A86 */
  0x486A,   /* U+10A87 */
  0x486B,   /* U+10A88 */
  0x486C,   /* U+10A89 */
  0x486D,   /* U+10A8A */
  0x486E,   /* U+10A8B */
  0x486F,   /* U+10A8C */
  0x4870,   /* U+10A8D */
  0x4871,   /* U+10A8E */
  0x4872,   /* U+10A8F */
  0x4873,   /* U+10A90 */
  0x4874,   /* U+10A91 */
  0x4875,   /* U+10A92 */
  0x4876,   /* U+10A93 */
  0x4877,   /* U+10A94 */
  0x4878,   /* U+10A95 */
  0x4879,   /* U+10A96 */
  0x487A,   /* U+10A97 */
  0x487B,   /* U+10A98 */
  0x487C,   /* U+10A99 */
  0x487D,   /* U+10A9A */
  0x487E,   /* U+10A9B */
  0x487F,   /* U+10A9C */
  0x1C3E,   /* U+10A9D */
  0x1B59,   /* U+10A9E */
  0x1B5A,   /* U+10A9F */
  0xFBC2,   /* U+10AA0 */
  0xFBC2,   /* U+10AA1 */
  0xFBC2,   /* U+10AA2 */
  0xFBC2,   /* U+10AA3 */
  0xFBC2,   /* U+10AA4 */
  0xFBC2,   /* U+10AA5 */
  0xFBC2,   /* U+10AA6 */
  0xFBC2,   /* U+10AA7 */
  0xFBC2,   /* U+10AA8 */
  0xFBC2,   /* U+10AA9 */
  0xFBC2,   /* U+10AAA */
  0xFBC2,   /* U+10AAB */
  0xFBC2,   /* U+10AAC */
  0xFBC2,   /* U+10AAD */
  0xFBC2,   /* U+10AAE */
  0xFBC2,   /* U+10AAF */
  0xFBC2,   /* U+10AB0 */
  0xFBC2,   /* U+10AB1 */
  0xFBC2,   /* U+10AB2 */
  0xFBC2,   /* U+10AB3 */
  0xFBC2,   /* U+10AB4 */
  0xFBC2,   /* U+10AB5 */
  0xFBC2,   /* U+10AB6 */
  0xFBC2,   /* U+10AB7 */
  0xFBC2,   /* U+10AB8 */
  0xFBC2,   /* U+10AB9 */
  0xFBC2,   /* U+10ABA */
  0xFBC2,   /* U+10ABB */
  0xFBC2,   /* U+10ABC */
  0xFBC2,   /* U+10ABD */
  0xFBC2,   /* U+10ABE */
  0xFBC2,   /* U+10ABF */
  0x4947,   /* U+10AC0 */
  0x4948,   /* U+10AC1 */
  0x4949,   /* U+10AC2 */
  0x494A,   /* U+10AC3 */
  0x494B,   /* U+10AC4 */
  0x494C,   /* U+10AC5 */
  0x494D,   /* U+10AC6 */
  0x494E,   /* U+10AC7 */
  0x494E,   /* U+10AC8 */
  0x494F,   /* U+10AC9 */
  0x4950,   /* U+10ACA */
  0x4951,   /* U+10ACB */
  0x4952,   /* U+10ACC */
  0x4953,   /* U+10ACD */
  0x4954,   /* U+10ACE */
  0x4955,   /* U+10ACF */
  0x4956,   /* U+10AD0 */
  0x4957,   /* U+10AD1 */
  0x4958,   /* U+10AD2 */
  0x4959,   /* U+10AD3 */
  0x495A,   /* U+10AD4 */
  0x495B,   /* U+10AD5 */
  0x495C,   /* U+10AD6 */
  0x495D,   /* U+10AD7 */
  0x495E,   /* U+10AD8 */
  0x495F,   /* U+10AD9 */
  0x4960,   /* U+10ADA */
  0x4961,   /* U+10ADB */
  0x4962,   /* U+10ADC */
  0x4963,   /* U+10ADD */
  0x4964,   /* U+10ADE */
  0x4965,   /* U+10ADF */
  0x4966,   /* U+10AE0 */
  0x4967,   /* U+10AE1 */
  0x4968,   /* U+10AE2 */
  0x4969,   /* U+10AE3 */
  0x496A,   /* U+10AE4 */
  0x0000,   /* U+10AE5 */
  0x0000,   /* U+10AE6 */
  0xFBC2,   /* U+10AE7 */
  0xFBC2,   /* U+10AE8 */
  0xFBC2,   /* U+10AE9 */
  0xFBC2,   /* U+10AEA */
  0x1C3E,   /* U+10AEB */
  0x1C42,   /* U+10AEC */
  0x1B63,   /* U+10AED */
  0x1B64,   /* U+10AEE */
  0x1B65,   /* U+10AEF */
  0x0443,   /* U+10AF0 */
  0x0444,   /* U+10AF1 */
  0x0445,   /* U+10AF2 */
  0x0446,   /* U+10AF3 */
  0x0447,   /* U+10AF4 */
  0x0448,   /* U+10AF5 */
  0x0449,   /* U+10AF6 */
  0xFBC2,   /* U+10AF7 */
  0xFBC2,   /* U+10AF8 */
  0xFBC2,   /* U+10AF9 */
  0xFBC2,   /* U+10AFA */
  0xFBC2,   /* U+10AFB */
  0xFBC2,   /* U+10AFC */
  0xFBC2,   /* U+10AFD */
  0xFBC2,   /* U+10AFE */
  0xFBC2,   /* U+10AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10A00 */
  0x0020,   /* U+10A01 */
  0x0020,   /* U+10A02 */
  0x0020,   /* U+10A03 */
  0x0020,   /* U+10A04 */
  0x0020,   /* U+10A05 */
  0x0020,   /* U+10A06 */
  0x0020,   /* U+10A07 */
  0x0020,   /* U+10A08 */
  0x0020,   /* U+10A09 */
  0x0020,   /* U+10A0A */
  0x0020,   /* U+10A0B */
  0x0020,   /* U+10A0C */
  0x0034,   /* U+10A0D */
  0x00C4,   /* U+10A0E */
  0x00C5,   /* U+10A0F */
  0x0020,   /* U+10A10 */
  0x0020,   /* U+10A11 */
  0x0020,   /* U+10A12 */
  0x0020,   /* U+10A13 */
  0x0020,   /* U+10A14 */
  0x0020,   /* U+10A15 */
  0x0020,   /* U+10A16 */
  0x0020,   /* U+10A17 */
  0x0020,   /* U+10A18 */
  0x0020,   /* U+10A19 */
  0x0020,   /* U+10A1A */
  0x0020,   /* U+10A1B */
  0x0020,   /* U+10A1C */
  0x0020,   /* U+10A1D */
  0x0020,   /* U+10A1E */
  0x0020,   /* U+10A1F */
  0x0020,   /* U+10A20 */
  0x0020,   /* U+10A21 */
  0x0020,   /* U+10A22 */
  0x0020,   /* U+10A23 */
  0x0020,   /* U+10A24 */
  0x0020,   /* U+10A25 */
  0x0020,   /* U+10A26 */
  0x0020,   /* U+10A27 */
  0x0020,   /* U+10A28 */
  0x0020,   /* U+10A29 */
  0x0020,   /* U+10A2A */
  0x0020,   /* U+10A2B */
  0x0020,   /* U+10A2C */
  0x0020,   /* U+10A2D */
  0x0020,   /* U+10A2E */
  0x0020,   /* U+10A2F */
  0x0020,   /* U+10A30 */
  0x0020,   /* U+10A31 */
  0x0020,   /* U+10A32 */
  0x0020,   /* U+10A33 */
  0x0020,   /* U+10A34 */
  0x0020,   /* U+10A35 */
  0x0020,   /* U+10A36 */
  0x0020,   /* U+10A37 */
  0x00CC,   /* U+10A38 */
  0x00CD,   /* U+10A39 */
  0x00CE,   /* U+10A3A */
  0x0020,   /* U+10A3B */
  0x0020,   /* U+10A3C */
  0x0020,   /* U+10A3D */
  0x0020,   /* U+10A3E */
  0x0020,   /* U+10A3F */
  0x0020,   /* U+10A40 */
  0x0020,   /* U+10A41 */
  0x0020,   /* U+10A42 */
  0x0020,   /* U+10A43 */
  0x0020,   /* U+10A44 */
  0x0020,   /* U+10A45 */
  0x0020,   /* U+10A46 */
  0x0020,   /* U+10A47 */
  0x0020,   /* U+10A48 */
  0x0020,   /* U+10A49 */
  0x0020,   /* U+10A4A */
  0x0020,   /* U+10A4B */
  0x0020,   /* U+10A4C */
  0x0020,   /* U+10A4D */
  0x0020,   /* U+10A4E */
  0x0020,   /* U+10A4F */
  0x0020,   /* U+10A50 */
  0x0020,   /* U+10A51 */
  0x0020,   /* U+10A52 */
  0x0020,   /* U+10A53 */
  0x0020,   /* U+10A54 */
  0x0020,   /* U+10A55 */
  0x0020,   /* U+10A56 */
  0x0020,   /* U+10A57 */
  0x0020,   /* U+10A58 */
  0x0020,   /* U+10A59 */
  0x0020,   /* U+10A5A */
  0x0020,   /* U+10A5B */
  0x0020,   /* U+10A5C */
  0x0020,   /* U+10A5D */
  0x0020,   /* U+10A5E */
  0x0020,   /* U+10A5F */
  0x0020,   /* U+10A60 */
  0x0020,   /* U+10A61 */
  0x0020,   /* U+10A62 */
  0x0020,   /* U+10A63 */
  0x0020,   /* U+10A64 */
  0x0020,   /* U+10A65 */
  0x0020,   /* U+10A66 */
  0x0020,   /* U+10A67 */
  0x0020,   /* U+10A68 */
  0x0020,   /* U+10A69 */
  0x0020,   /* U+10A6A */
  0x0020,   /* U+10A6B */
  0x0020,   /* U+10A6C */
  0x0020,   /* U+10A6D */
  0x0020,   /* U+10A6E */
  0x0020,   /* U+10A6F */
  0x0020,   /* U+10A70 */
  0x0020,   /* U+10A71 */
  0x0020,   /* U+10A72 */
  0x0020,   /* U+10A73 */
  0x0020,   /* U+10A74 */
  0x0020,   /* U+10A75 */
  0x0020,   /* U+10A76 */
  0x0020,   /* U+10A77 */
  0x0020,   /* U+10A78 */
  0x0020,   /* U+10A79 */
  0x0020,   /* U+10A7A */
  0x0020,   /* U+10A7B */
  0x0020,   /* U+10A7C */
  0x0020,   /* U+10A7D */
  0x0020,   /* U+10A7E */
  0x0020,   /* U+10A7F */
  0x0020,   /* U+10A80 */
  0x0020,   /* U+10A81 */
  0x0020,   /* U+10A82 */
  0x0020,   /* U+10A83 */
  0x0020,   /* U+10A84 */
  0x0020,   /* U+10A85 */
  0x0020,   /* U+10A86 */
  0x0020,   /* U+10A87 */
  0x0020,   /* U+10A88 */
  0x0020,   /* U+10A89 */
  0x0020,   /* U+10A8A */
  0x0020,   /* U+10A8B */
  0x0020,   /* U+10A8C */
  0x0020,   /* U+10A8D */
  0x0020,   /* U+10A8E */
  0x0020,   /* U+10A8F */
  0x0020,   /* U+10A90 */
  0x0020,   /* U+10A91 */
  0x0020,   /* U+10A92 */
  0x0020,   /* U+10A93 */
  0x0020,   /* U+10A94 */
  0x0020,   /* U+10A95 */
  0x0020,   /* U+10A96 */
  0x0020,   /* U+10A97 */
  0x0020,   /* U+10A98 */
  0x0020,   /* U+10A99 */
  0x0020,   /* U+10A9A */
  0x0020,   /* U+10A9B */
  0x0020,   /* U+10A9C */
  0x0020,   /* U+10A9D */
  0x0020,   /* U+10A9E */
  0x0020,   /* U+10A9F */
  0x0020,   /* U+10AA0 */
  0x0020,   /* U+10AA1 */
  0x0020,   /* U+10AA2 */
  0x0020,   /* U+10AA3 */
  0x0020,   /* U+10AA4 */
  0x0020,   /* U+10AA5 */
  0x0020,   /* U+10AA6 */
  0x0020,   /* U+10AA7 */
  0x0020,   /* U+10AA8 */
  0x0020,   /* U+10AA9 */
  0x0020,   /* U+10AAA */
  0x0020,   /* U+10AAB */
  0x0020,   /* U+10AAC */
  0x0020,   /* U+10AAD */
  0x0020,   /* U+10AAE */
  0x0020,   /* U+10AAF */
  0x0020,   /* U+10AB0 */
  0x0020,   /* U+10AB1 */
  0x0020,   /* U+10AB2 */
  0x0020,   /* U+10AB3 */
  0x0020,   /* U+10AB4 */
  0x0020,   /* U+10AB5 */
  0x0020,   /* U+10AB6 */
  0x0020,   /* U+10AB7 */
  0x0020,   /* U+10AB8 */
  0x0020,   /* U+10AB9 */
  0x0020,   /* U+10ABA */
  0x0020,   /* U+10ABB */
  0x0020,   /* U+10ABC */
  0x0020,   /* U+10ABD */
  0x0020,   /* U+10ABE */
  0x0020,   /* U+10ABF */
  0x0020,   /* U+10AC0 */
  0x0020,   /* U+10AC1 */
  0x0020,   /* U+10AC2 */
  0x0020,   /* U+10AC3 */
  0x0020,   /* U+10AC4 */
  0x0020,   /* U+10AC5 */
  0x0020,   /* U+10AC6 */
  0x0020,   /* U+10AC7 */
  0x0020,   /* U+10AC8 */
  0x0020,   /* U+10AC9 */
  0x0020,   /* U+10ACA */
  0x0020,   /* U+10ACB */
  0x0020,   /* U+10ACC */
  0x0020,   /* U+10ACD */
  0x0020,   /* U+10ACE */
  0x0020,   /* U+10ACF */
  0x0020,   /* U+10AD0 */
  0x0020,   /* U+10AD1 */
  0x0020,   /* U+10AD2 */
  0x0020,   /* U+10AD3 */
  0x0020,   /* U+10AD4 */
  0x0020,   /* U+10AD5 */
  0x0020,   /* U+10AD6 */
  0x0020,   /* U+10AD7 */
  0x0020,   /* U+10AD8 */
  0x0020,   /* U+10AD9 */
  0x0020,   /* U+10ADA */
  0x0020,   /* U+10ADB */
  0x0020,   /* U+10ADC */
  0x0020,   /* U+10ADD */
  0x0020,   /* U+10ADE */
  0x0020,   /* U+10ADF */
  0x0020,   /* U+10AE0 */
  0x0020,   /* U+10AE1 */
  0x0020,   /* U+10AE2 */
  0x0020,   /* U+10AE3 */
  0x0020,   /* U+10AE4 */
  0x0033,   /* U+10AE5 */
  0x0034,   /* U+10AE6 */
  0x0020,   /* U+10AE7 */
  0x0020,   /* U+10AE8 */
  0x0020,   /* U+10AE9 */
  0x0020,   /* U+10AEA */
  0x0020,   /* U+10AEB */
  0x0020,   /* U+10AEC */
  0x0020,   /* U+10AED */
  0x0020,   /* U+10AEE */
  0x0020,   /* U+10AEF */
  0x0020,   /* U+10AF0 */
  0x0020,   /* U+10AF1 */
  0x0020,   /* U+10AF2 */
  0x0020,   /* U+10AF3 */
  0x0020,   /* U+10AF4 */
  0x0020,   /* U+10AF5 */
  0x0020,   /* U+10AF6 */
  0x0020,   /* U+10AF7 */
  0x0020,   /* U+10AF8 */
  0x0020,   /* U+10AF9 */
  0x0020,   /* U+10AFA */
  0x0020,   /* U+10AFB */
  0x0020,   /* U+10AFC */
  0x0020,   /* U+10AFD */
  0x0020,   /* U+10AFE */
  0x0020,   /* U+10AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10A00 */
  0x0002,   /* U+10A01 */
  0x0002,   /* U+10A02 */
  0x0002,   /* U+10A03 */
  0x0002,   /* U+10A04 */
  0x0002,   /* U+10A05 */
  0x0002,   /* U+10A06 */
  0x0002,   /* U+10A07 */
  0x0002,   /* U+10A08 */
  0x0002,   /* U+10A09 */
  0x0002,   /* U+10A0A */
  0x0002,   /* U+10A0B */
  0x0002,   /* U+10A0C */
  0x0002,   /* U+10A0D */
  0x0002,   /* U+10A0E */
  0x0002,   /* U+10A0F */
  0x0002,   /* U+10A10 */
  0x0002,   /* U+10A11 */
  0x0002,   /* U+10A12 */
  0x0002,   /* U+10A13 */
  0x0002,   /* U+10A14 */
  0x0002,   /* U+10A15 */
  0x0002,   /* U+10A16 */
  0x0002,   /* U+10A17 */
  0x0002,   /* U+10A18 */
  0x0002,   /* U+10A19 */
  0x0002,   /* U+10A1A */
  0x0002,   /* U+10A1B */
  0x0002,   /* U+10A1C */
  0x0002,   /* U+10A1D */
  0x0002,   /* U+10A1E */
  0x0002,   /* U+10A1F */
  0x0002,   /* U+10A20 */
  0x0002,   /* U+10A21 */
  0x0002,   /* U+10A22 */
  0x0002,   /* U+10A23 */
  0x0002,   /* U+10A24 */
  0x0002,   /* U+10A25 */
  0x0002,   /* U+10A26 */
  0x0002,   /* U+10A27 */
  0x0002,   /* U+10A28 */
  0x0002,   /* U+10A29 */
  0x0002,   /* U+10A2A */
  0x0002,   /* U+10A2B */
  0x0002,   /* U+10A2C */
  0x0002,   /* U+10A2D */
  0x0002,   /* U+10A2E */
  0x0002,   /* U+10A2F */
  0x0002,   /* U+10A30 */
  0x0002,   /* U+10A31 */
  0x0002,   /* U+10A32 */
  0x0002,   /* U+10A33 */
  0x0002,   /* U+10A34 */
  0x0002,   /* U+10A35 */
  0x0002,   /* U+10A36 */
  0x0002,   /* U+10A37 */
  0x0002,   /* U+10A38 */
  0x0002,   /* U+10A39 */
  0x0002,   /* U+10A3A */
  0x0002,   /* U+10A3B */
  0x0002,   /* U+10A3C */
  0x0002,   /* U+10A3D */
  0x0002,   /* U+10A3E */
  0x0002,   /* U+10A3F */
  0x0002,   /* U+10A40 */
  0x0002,   /* U+10A41 */
  0x0002,   /* U+10A42 */
  0x0002,   /* U+10A43 */
  0x0002,   /* U+10A44 */
  0x0002,   /* U+10A45 */
  0x0002,   /* U+10A46 */
  0x0002,   /* U+10A47 */
  0x0002,   /* U+10A48 */
  0x0002,   /* U+10A49 */
  0x0002,   /* U+10A4A */
  0x0002,   /* U+10A4B */
  0x0002,   /* U+10A4C */
  0x0002,   /* U+10A4D */
  0x0002,   /* U+10A4E */
  0x0002,   /* U+10A4F */
  0x0002,   /* U+10A50 */
  0x0002,   /* U+10A51 */
  0x0002,   /* U+10A52 */
  0x0002,   /* U+10A53 */
  0x0002,   /* U+10A54 */
  0x0002,   /* U+10A55 */
  0x0002,   /* U+10A56 */
  0x0002,   /* U+10A57 */
  0x0002,   /* U+10A58 */
  0x0002,   /* U+10A59 */
  0x0002,   /* U+10A5A */
  0x0002,   /* U+10A5B */
  0x0002,   /* U+10A5C */
  0x0002,   /* U+10A5D */
  0x0002,   /* U+10A5E */
  0x0002,   /* U+10A5F */
  0x0002,   /* U+10A60 */
  0x0002,   /* U+10A61 */
  0x0002,   /* U+10A62 */
  0x0002,   /* U+10A63 */
  0x0002,   /* U+10A64 */
  0x0002,   /* U+10A65 */
  0x0002,   /* U+10A66 */
  0x0002,   /* U+10A67 */
  0x0002,   /* U+10A68 */
  0x0002,   /* U+10A69 */
  0x0002,   /* U+10A6A */
  0x0002,   /* U+10A6B */
  0x0002,   /* U+10A6C */
  0x0002,   /* U+10A6D */
  0x0002,   /* U+10A6E */
  0x0002,   /* U+10A6F */
  0x0002,   /* U+10A70 */
  0x0002,   /* U+10A71 */
  0x0002,   /* U+10A72 */
  0x0002,   /* U+10A73 */
  0x0002,   /* U+10A74 */
  0x0002,   /* U+10A75 */
  0x0002,   /* U+10A76 */
  0x0002,   /* U+10A77 */
  0x0002,   /* U+10A78 */
  0x0002,   /* U+10A79 */
  0x0002,   /* U+10A7A */
  0x0002,   /* U+10A7B */
  0x0002,   /* U+10A7C */
  0x0002,   /* U+10A7D */
  0x0002,   /* U+10A7E */
  0x0002,   /* U+10A7F */
  0x0002,   /* U+10A80 */
  0x0002,   /* U+10A81 */
  0x0002,   /* U+10A82 */
  0x0002,   /* U+10A83 */
  0x0002,   /* U+10A84 */
  0x0002,   /* U+10A85 */
  0x0002,   /* U+10A86 */
  0x0002,   /* U+10A87 */
  0x0002,   /* U+10A88 */
  0x0002,   /* U+10A89 */
  0x0002,   /* U+10A8A */
  0x0002,   /* U+10A8B */
  0x0002,   /* U+10A8C */
  0x0002,   /* U+10A8D */
  0x0002,   /* U+10A8E */
  0x0002,   /* U+10A8F */
  0x0002,   /* U+10A90 */
  0x0002,   /* U+10A91 */
  0x0002,   /* U+10A92 */
  0x0002,   /* U+10A93 */
  0x0002,   /* U+10A94 */
  0x0002,   /* U+10A95 */
  0x0002,   /* U+10A96 */
  0x0002,   /* U+10A97 */
  0x0002,   /* U+10A98 */
  0x0002,   /* U+10A99 */
  0x0002,   /* U+10A9A */
  0x0002,   /* U+10A9B */
  0x0002,   /* U+10A9C */
  0x0002,   /* U+10A9D */
  0x0002,   /* U+10A9E */
  0x0002,   /* U+10A9F */
  0x0002,   /* U+10AA0 */
  0x0002,   /* U+10AA1 */
  0x0002,   /* U+10AA2 */
  0x0002,   /* U+10AA3 */
  0x0002,   /* U+10AA4 */
  0x0002,   /* U+10AA5 */
  0x0002,   /* U+10AA6 */
  0x0002,   /* U+10AA7 */
  0x0002,   /* U+10AA8 */
  0x0002,   /* U+10AA9 */
  0x0002,   /* U+10AAA */
  0x0002,   /* U+10AAB */
  0x0002,   /* U+10AAC */
  0x0002,   /* U+10AAD */
  0x0002,   /* U+10AAE */
  0x0002,   /* U+10AAF */
  0x0002,   /* U+10AB0 */
  0x0002,   /* U+10AB1 */
  0x0002,   /* U+10AB2 */
  0x0002,   /* U+10AB3 */
  0x0002,   /* U+10AB4 */
  0x0002,   /* U+10AB5 */
  0x0002,   /* U+10AB6 */
  0x0002,   /* U+10AB7 */
  0x0002,   /* U+10AB8 */
  0x0002,   /* U+10AB9 */
  0x0002,   /* U+10ABA */
  0x0002,   /* U+10ABB */
  0x0002,   /* U+10ABC */
  0x0002,   /* U+10ABD */
  0x0002,   /* U+10ABE */
  0x0002,   /* U+10ABF */
  0x0002,   /* U+10AC0 */
  0x0002,   /* U+10AC1 */
  0x0002,   /* U+10AC2 */
  0x0002,   /* U+10AC3 */
  0x0002,   /* U+10AC4 */
  0x0002,   /* U+10AC5 */
  0x0002,   /* U+10AC6 */
  0x0002,   /* U+10AC7 */
  0x0004,   /* U+10AC8 */
  0x0002,   /* U+10AC9 */
  0x0002,   /* U+10ACA */
  0x0002,   /* U+10ACB */
  0x0002,   /* U+10ACC */
  0x0002,   /* U+10ACD */
  0x0002,   /* U+10ACE */
  0x0002,   /* U+10ACF */
  0x0002,   /* U+10AD0 */
  0x0002,   /* U+10AD1 */
  0x0002,   /* U+10AD2 */
  0x0002,   /* U+10AD3 */
  0x0002,   /* U+10AD4 */
  0x0002,   /* U+10AD5 */
  0x0002,   /* U+10AD6 */
  0x0002,   /* U+10AD7 */
  0x0002,   /* U+10AD8 */
  0x0002,   /* U+10AD9 */
  0x0002,   /* U+10ADA */
  0x0002,   /* U+10ADB */
  0x0002,   /* U+10ADC */
  0x0002,   /* U+10ADD */
  0x0002,   /* U+10ADE */
  0x0002,   /* U+10ADF */
  0x0002,   /* U+10AE0 */
  0x0002,   /* U+10AE1 */
  0x0002,   /* U+10AE2 */
  0x0002,   /* U+10AE3 */
  0x0002,   /* U+10AE4 */
  0x0002,   /* U+10AE5 */
  0x0002,   /* U+10AE6 */
  0x0002,   /* U+10AE7 */
  0x0002,   /* U+10AE8 */
  0x0002,   /* U+10AE9 */
  0x0002,   /* U+10AEA */
  0x0002,   /* U+10AEB */
  0x0002,   /* U+10AEC */
  0x0002,   /* U+10AED */
  0x0002,   /* U+10AEE */
  0x0002,   /* U+10AEF */
  0x0002,   /* U+10AF0 */
  0x0002,   /* U+10AF1 */
  0x0002,   /* U+10AF2 */
  0x0002,   /* U+10AF3 */
  0x0002,   /* U+10AF4 */
  0x0002,   /* U+10AF5 */
  0x0002,   /* U+10AF6 */
  0x0002,   /* U+10AF7 */
  0x0002,   /* U+10AF8 */
  0x0002,   /* U+10AF9 */
  0x0002,   /* U+10AFA */
  0x0002,   /* U+10AFB */
  0x0002,   /* U+10AFC */
  0x0002,   /* U+10AFD */
  0x0002,   /* U+10AFE */
  0x0002,   /* U+10AFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10A00 */
  0x0000,   /* U+10A01 */
  0x0000,   /* U+10A02 */
  0x0000,   /* U+10A03 */
  0x8A04,   /* U+10A04 */
  0x0000,   /* U+10A05 */
  0x0000,   /* U+10A06 */
  0x8A07,   /* U+10A07 */
  0x8A08,   /* U+10A08 */
  0x8A09,   /* U+10A09 */
  0x8A0A,   /* U+10A0A */
  0x8A0B,   /* U+10A0B */
  0x0000,   /* U+10A0C */
  0x0000,   /* U+10A0D */
  0x0000,   /* U+10A0E */
  0x0000,   /* U+10A0F */
  0x0000,   /* U+10A10 */
  0x0000,   /* U+10A11 */
  0x0000,   /* U+10A12 */
  0x0000,   /* U+10A13 */
  0x8A14,   /* U+10A14 */
  0x0000,   /* U+10A15 */
  0x0000,   /* U+10A16 */
  0x0000,   /* U+10A17 */
  0x8A18,   /* U+10A18 */
  0x0000,   /* U+10A19 */
  0x0000,   /* U+10A1A */
  0x0000,   /* U+10A1B */
  0x0000,   /* U+10A1C */
  0x0000,   /* U+10A1D */
  0x0000,   /* U+10A1E */
  0x0000,   /* U+10A1F */
  0x0000,   /* U+10A20 */
  0x0000,   /* U+10A21 */
  0x0000,   /* U+10A22 */
  0x0000,   /* U+10A23 */
  0x0000,   /* U+10A24 */
  0x0000,   /* U+10A25 */
  0x0000,   /* U+10A26 */
  0x0000,   /* U+10A27 */
  0x0000,   /* U+10A28 */
  0x0000,   /* U+10A29 */
  0x0000,   /* U+10A2A */
  0x0000,   /* U+10A2B */
  0x0000,   /* U+10A2C */
  0x0000,   /* U+10A2D */
  0x0000,   /* U+10A2E */
  0x0000,   /* U+10A2F */
  0x0000,   /* U+10A30 */
  0x0000,   /* U+10A31 */
  0x0000,   /* U+10A32 */
  0x0000,   /* U+10A33 */
  0x8A34,   /* U+10A34 */
  0x8A35,   /* U+10A35 */
  0x8A36,   /* U+10A36 */
  0x8A37,   /* U+10A37 */
  0x0000,   /* U+10A38 */
  0x0000,   /* U+10A39 */
  0x0000,   /* U+10A3A */
  0x8A3B,   /* U+10A3B */
  0x8A3C,   /* U+10A3C */
  0x8A3D,   /* U+10A3D */
  0x8A3E,   /* U+10A3E */
  0x0000,   /* U+10A3F */
  0x0000,   /* U+10A40 */
  0x0000,   /* U+10A41 */
  0x0000,   /* U+10A42 */
  0x0000,   /* U+10A43 */
  0x0000,   /* U+10A44 */
  0x0000,   /* U+10A45 */
  0x0000,   /* U+10A46 */
  0x0000,   /* U+10A47 */
  0x8A48,   /* U+10A48 */
  0x8A49,   /* U+10A49 */
  0x8A4A,   /* U+10A4A */
  0x8A4B,   /* U+10A4B */
  0x8A4C,   /* U+10A4C */
  0x8A4D,   /* U+10A4D */
  0x8A4E,   /* U+10A4E */
  0x8A4F,   /* U+10A4F */
  0x0000,   /* U+10A50 */
  0x0000,   /* U+10A51 */
  0x0000,   /* U+10A52 */
  0x0000,   /* U+10A53 */
  0x0000,   /* U+10A54 */
  0x0000,   /* U+10A55 */
  0x0000,   /* U+10A56 */
  0x0000,   /* U+10A57 */
  0x0000,   /* U+10A58 */
  0x8A59,   /* U+10A59 */
  0x8A5A,   /* U+10A5A */
  0x8A5B,   /* U+10A5B */
  0x8A5C,   /* U+10A5C */
  0x8A5D,   /* U+10A5D */
  0x8A5E,   /* U+10A5E */
  0x8A5F,   /* U+10A5F */
  0x0000,   /* U+10A60 */
  0x0000,   /* U+10A61 */
  0x0000,   /* U+10A62 */
  0x0000,   /* U+10A63 */
  0x0000,   /* U+10A64 */
  0x0000,   /* U+10A65 */
  0x0000,   /* U+10A66 */
  0x0000,   /* U+10A67 */
  0x0000,   /* U+10A68 */
  0x0000,   /* U+10A69 */
  0x0000,   /* U+10A6A */
  0x0000,   /* U+10A6B */
  0x0000,   /* U+10A6C */
  0x0000,   /* U+10A6D */
  0x0000,   /* U+10A6E */
  0x0000,   /* U+10A6F */
  0x0000,   /* U+10A70 */
  0x0000,   /* U+10A71 */
  0x0000,   /* U+10A72 */
  0x0000,   /* U+10A73 */
  0x0000,   /* U+10A74 */
  0x0000,   /* U+10A75 */
  0x0000,   /* U+10A76 */
  0x0000,   /* U+10A77 */
  0x0000,   /* U+10A78 */
  0x0000,   /* U+10A79 */
  0x0000,   /* U+10A7A */
  0x0000,   /* U+10A7B */
  0x0000,   /* U+10A7C */
  0x0000,   /* U+10A7D */
  0x0000,   /* U+10A7E */
  0x0000,   /* U+10A7F */
  0x0000,   /* U+10A80 */
  0x0000,   /* U+10A81 */
  0x0000,   /* U+10A82 */
  0x0000,   /* U+10A83 */
  0x0000,   /* U+10A84 */
  0x0000,   /* U+10A85 */
  0x0000,   /* U+10A86 */
  0x0000,   /* U+10A87 */
  0x0000,   /* U+10A88 */
  0x0000,   /* U+10A89 */
  0x0000,   /* U+10A8A */
  0x0000,   /* U+10A8B */
  0x0000,   /* U+10A8C */
  0x0000,   /* U+10A8D */
  0x0000,   /* U+10A8E */
  0x0000,   /* U+10A8F */
  0x0000,   /* U+10A90 */
  0x0000,   /* U+10A91 */
  0x0000,   /* U+10A92 */
  0x0000,   /* U+10A93 */
  0x0000,   /* U+10A94 */
  0x0000,   /* U+10A95 */
  0x0000,   /* U+10A96 */
  0x0000,   /* U+10A97 */
  0x0000,   /* U+10A98 */
  0x0000,   /* U+10A99 */
  0x0000,   /* U+10A9A */
  0x0000,   /* U+10A9B */
  0x0000,   /* U+10A9C */
  0x0000,   /* U+10A9D */
  0x0000,   /* U+10A9E */
  0x0000,   /* U+10A9F */
  0x8AA0,   /* U+10AA0 */
  0x8AA1,   /* U+10AA1 */
  0x8AA2,   /* U+10AA2 */
  0x8AA3,   /* U+10AA3 */
  0x8AA4,   /* U+10AA4 */
  0x8AA5,   /* U+10AA5 */
  0x8AA6,   /* U+10AA6 */
  0x8AA7,   /* U+10AA7 */
  0x8AA8,   /* U+10AA8 */
  0x8AA9,   /* U+10AA9 */
  0x8AAA,   /* U+10AAA */
  0x8AAB,   /* U+10AAB */
  0x8AAC,   /* U+10AAC */
  0x8AAD,   /* U+10AAD */
  0x8AAE,   /* U+10AAE */
  0x8AAF,   /* U+10AAF */
  0x8AB0,   /* U+10AB0 */
  0x8AB1,   /* U+10AB1 */
  0x8AB2,   /* U+10AB2 */
  0x8AB3,   /* U+10AB3 */
  0x8AB4,   /* U+10AB4 */
  0x8AB5,   /* U+10AB5 */
  0x8AB6,   /* U+10AB6 */
  0x8AB7,   /* U+10AB7 */
  0x8AB8,   /* U+10AB8 */
  0x8AB9,   /* U+10AB9 */
  0x8ABA,   /* U+10ABA */
  0x8ABB,   /* U+10ABB */
  0x8ABC,   /* U+10ABC */
  0x8ABD,   /* U+10ABD */
  0x8ABE,   /* U+10ABE */
  0x8ABF,   /* U+10ABF */
  0x0000,   /* U+10AC0 */
  0x0000,   /* U+10AC1 */
  0x0000,   /* U+10AC2 */
  0x0000,   /* U+10AC3 */
  0x0000,   /* U+10AC4 */
  0x0000,   /* U+10AC5 */
  0x0000,   /* U+10AC6 */
  0x0000,   /* U+10AC7 */
  0x0000,   /* U+10AC8 */
  0x0000,   /* U+10AC9 */
  0x0000,   /* U+10ACA */
  0x0000,   /* U+10ACB */
  0x0000,   /* U+10ACC */
  0x0000,   /* U+10ACD */
  0x0000,   /* U+10ACE */
  0x0000,   /* U+10ACF */
  0x0000,   /* U+10AD0 */
  0x0000,   /* U+10AD1 */
  0x0000,   /* U+10AD2 */
  0x0000,   /* U+10AD3 */
  0x0000,   /* U+10AD4 */
  0x0000,   /* U+10AD5 */
  0x0000,   /* U+10AD6 */
  0x0000,   /* U+10AD7 */
  0x0000,   /* U+10AD8 */
  0x0000,   /* U+10AD9 */
  0x0000,   /* U+10ADA */
  0x0000,   /* U+10ADB */
  0x0000,   /* U+10ADC */
  0x0000,   /* U+10ADD */
  0x0000,   /* U+10ADE */
  0x0000,   /* U+10ADF */
  0x0000,   /* U+10AE0 */
  0x0000,   /* U+10AE1 */
  0x0000,   /* U+10AE2 */
  0x0000,   /* U+10AE3 */
  0x0000,   /* U+10AE4 */
  0x0000,   /* U+10AE5 */
  0x0000,   /* U+10AE6 */
  0x8AE7,   /* U+10AE7 */
  0x8AE8,   /* U+10AE8 */
  0x8AE9,   /* U+10AE9 */
  0x8AEA,   /* U+10AEA */
  0x0000,   /* U+10AEB */
  0x0000,   /* U+10AEC */
  0x0000,   /* U+10AED */
  0x0000,   /* U+10AEE */
  0x0000,   /* U+10AEF */
  0x0000,   /* U+10AF0 */
  0x0000,   /* U+10AF1 */
  0x0000,   /* U+10AF2 */
  0x0000,   /* U+10AF3 */
  0x0000,   /* U+10AF4 */
  0x0000,   /* U+10AF5 */
  0x0000,   /* U+10AF6 */
  0x8AF7,   /* U+10AF7 */
  0x8AF8,   /* U+10AF8 */
  0x8AF9,   /* U+10AF9 */
  0x8AFA,   /* U+10AFA */
  0x8AFB,   /* U+10AFB */
  0x8AFC,   /* U+10AFC */
  0x8AFD,   /* U+10AFD */
  0x8AFE,   /* U+10AFE */
  0x8AFF,   /* U+10AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10A00 */
  0x0000,   /* U+10A01 */
  0x0000,   /* U+10A02 */
  0x0000,   /* U+10A03 */
  0x0000,   /* U+10A04 */
  0x0000,   /* U+10A05 */
  0x0000,   /* U+10A06 */
  0x0000,   /* U+10A07 */
  0x0000,   /* U+10A08 */
  0x0000,   /* U+10A09 */
  0x0000,   /* U+10A0A */
  0x0000,   /* U+10A0B */
  0x0000,   /* U+10A0C */
  0x0000,   /* U+10A0D */
  0x0000,   /* U+10A0E */
  0x0000,   /* U+10A0F */
  0x0000,   /* U+10A10 */
  0x0000,   /* U+10A11 */
  0x0000,   /* U+10A12 */
  0x0000,   /* U+10A13 */
  0x0000,   /* U+10A14 */
  0x0000,   /* U+10A15 */
  0x0000,   /* U+10A16 */
  0x0000,   /* U+10A17 */
  0x0000,   /* U+10A18 */
  0x0000,   /* U+10A19 */
  0x0000,   /* U+10A1A */
  0x0000,   /* U+10A1B */
  0x0000,   /* U+10A1C */
  0x0000,   /* U+10A1D */
  0x0000,   /* U+10A1E */
  0x0000,   /* U+10A1F */
  0x0000,   /* U+10A20 */
  0x0000,   /* U+10A21 */
  0x0000,   /* U+10A22 */
  0x0000,   /* U+10A23 */
  0x0000,   /* U+10A24 */
  0x0000,   /* U+10A25 */
  0x0000,   /* U+10A26 */
  0x0000,   /* U+10A27 */
  0x0000,   /* U+10A28 */
  0x0000,   /* U+10A29 */
  0x0000,   /* U+10A2A */
  0x0000,   /* U+10A2B */
  0x0000,   /* U+10A2C */
  0x0000,   /* U+10A2D */
  0x0000,   /* U+10A2E */
  0x0000,   /* U+10A2F */
  0x0000,   /* U+10A30 */
  0x0000,   /* U+10A31 */
  0x0000,   /* U+10A32 */
  0x0000,   /* U+10A33 */
  0x0000,   /* U+10A34 */
  0x0000,   /* U+10A35 */
  0x0000,   /* U+10A36 */
  0x0000,   /* U+10A37 */
  0x0000,   /* U+10A38 */
  0x0000,   /* U+10A39 */
  0x0000,   /* U+10A3A */
  0x0000,   /* U+10A3B */
  0x0000,   /* U+10A3C */
  0x0000,   /* U+10A3D */
  0x0000,   /* U+10A3E */
  0x0000,   /* U+10A3F */
  0x0000,   /* U+10A40 */
  0x0000,   /* U+10A41 */
  0x0000,   /* U+10A42 */
  0x0000,   /* U+10A43 */
  0x0000,   /* U+10A44 */
  0x0000,   /* U+10A45 */
  0x0000,   /* U+10A46 */
  0x0000,   /* U+10A47 */
  0x0000,   /* U+10A48 */
  0x0000,   /* U+10A49 */
  0x0000,   /* U+10A4A */
  0x0000,   /* U+10A4B */
  0x0000,   /* U+10A4C */
  0x0000,   /* U+10A4D */
  0x0000,   /* U+10A4E */
  0x0000,   /* U+10A4F */
  0x0000,   /* U+10A50 */
  0x0000,   /* U+10A51 */
  0x0000,   /* U+10A52 */
  0x0000,   /* U+10A53 */
  0x0000,   /* U+10A54 */
  0x0000,   /* U+10A55 */
  0x0000,   /* U+10A56 */
  0x0000,   /* U+10A57 */
  0x0000,   /* U+10A58 */
  0x0000,   /* U+10A59 */
  0x0000,   /* U+10A5A */
  0x0000,   /* U+10A5B */
  0x0000,   /* U+10A5C */
  0x0000,   /* U+10A5D */
  0x0000,   /* U+10A5E */
  0x0000,   /* U+10A5F */
  0x0000,   /* U+10A60 */
  0x0000,   /* U+10A61 */
  0x0000,   /* U+10A62 */
  0x0000,   /* U+10A63 */
  0x0000,   /* U+10A64 */
  0x0000,   /* U+10A65 */
  0x0000,   /* U+10A66 */
  0x0000,   /* U+10A67 */
  0x0000,   /* U+10A68 */
  0x0000,   /* U+10A69 */
  0x0000,   /* U+10A6A */
  0x0000,   /* U+10A6B */
  0x0000,   /* U+10A6C */
  0x0000,   /* U+10A6D */
  0x0000,   /* U+10A6E */
  0x0000,   /* U+10A6F */
  0x0000,   /* U+10A70 */
  0x0000,   /* U+10A71 */
  0x0000,   /* U+10A72 */
  0x0000,   /* U+10A73 */
  0x0000,   /* U+10A74 */
  0x0000,   /* U+10A75 */
  0x0000,   /* U+10A76 */
  0x0000,   /* U+10A77 */
  0x0000,   /* U+10A78 */
  0x0000,   /* U+10A79 */
  0x0000,   /* U+10A7A */
  0x0000,   /* U+10A7B */
  0x0000,   /* U+10A7C */
  0x0000,   /* U+10A7D */
  0x0000,   /* U+10A7E */
  0x0000,   /* U+10A7F */
  0x0000,   /* U+10A80 */
  0x0000,   /* U+10A81 */
  0x0000,   /* U+10A82 */
  0x0000,   /* U+10A83 */
  0x0000,   /* U+10A84 */
  0x0000,   /* U+10A85 */
  0x0000,   /* U+10A86 */
  0x0000,   /* U+10A87 */
  0x0000,   /* U+10A88 */
  0x0000,   /* U+10A89 */
  0x0000,   /* U+10A8A */
  0x0000,   /* U+10A8B */
  0x0000,   /* U+10A8C */
  0x0000,   /* U+10A8D */
  0x0000,   /* U+10A8E */
  0x0000,   /* U+10A8F */
  0x0000,   /* U+10A90 */
  0x0000,   /* U+10A91 */
  0x0000,   /* U+10A92 */
  0x0000,   /* U+10A93 */
  0x0000,   /* U+10A94 */
  0x0000,   /* U+10A95 */
  0x0000,   /* U+10A96 */
  0x0000,   /* U+10A97 */
  0x0000,   /* U+10A98 */
  0x0000,   /* U+10A99 */
  0x0000,   /* U+10A9A */
  0x0000,   /* U+10A9B */
  0x0000,   /* U+10A9C */
  0x0000,   /* U+10A9D */
  0x0000,   /* U+10A9E */
  0x0000,   /* U+10A9F */
  0x0000,   /* U+10AA0 */
  0x0000,   /* U+10AA1 */
  0x0000,   /* U+10AA2 */
  0x0000,   /* U+10AA3 */
  0x0000,   /* U+10AA4 */
  0x0000,   /* U+10AA5 */
  0x0000,   /* U+10AA6 */
  0x0000,   /* U+10AA7 */
  0x0000,   /* U+10AA8 */
  0x0000,   /* U+10AA9 */
  0x0000,   /* U+10AAA */
  0x0000,   /* U+10AAB */
  0x0000,   /* U+10AAC */
  0x0000,   /* U+10AAD */
  0x0000,   /* U+10AAE */
  0x0000,   /* U+10AAF */
  0x0000,   /* U+10AB0 */
  0x0000,   /* U+10AB1 */
  0x0000,   /* U+10AB2 */
  0x0000,   /* U+10AB3 */
  0x0000,   /* U+10AB4 */
  0x0000,   /* U+10AB5 */
  0x0000,   /* U+10AB6 */
  0x0000,   /* U+10AB7 */
  0x0000,   /* U+10AB8 */
  0x0000,   /* U+10AB9 */
  0x0000,   /* U+10ABA */
  0x0000,   /* U+10ABB */
  0x0000,   /* U+10ABC */
  0x0000,   /* U+10ABD */
  0x0000,   /* U+10ABE */
  0x0000,   /* U+10ABF */
  0x0000,   /* U+10AC0 */
  0x0000,   /* U+10AC1 */
  0x0000,   /* U+10AC2 */
  0x0000,   /* U+10AC3 */
  0x0000,   /* U+10AC4 */
  0x0000,   /* U+10AC5 */
  0x0000,   /* U+10AC6 */
  0x0000,   /* U+10AC7 */
  0x0111,   /* U+10AC8 */
  0x0000,   /* U+10AC9 */
  0x0000,   /* U+10ACA */
  0x0000,   /* U+10ACB */
  0x0000,   /* U+10ACC */
  0x0000,   /* U+10ACD */
  0x0000,   /* U+10ACE */
  0x0000,   /* U+10ACF */
  0x0000,   /* U+10AD0 */
  0x0000,   /* U+10AD1 */
  0x0000,   /* U+10AD2 */
  0x0000,   /* U+10AD3 */
  0x0000,   /* U+10AD4 */
  0x0000,   /* U+10AD5 */
  0x0000,   /* U+10AD6 */
  0x0000,   /* U+10AD7 */
  0x0000,   /* U+10AD8 */
  0x0000,   /* U+10AD9 */
  0x0000,   /* U+10ADA */
  0x0000,   /* U+10ADB */
  0x0000,   /* U+10ADC */
  0x0000,   /* U+10ADD */
  0x0000,   /* U+10ADE */
  0x0000,   /* U+10ADF */
  0x0000,   /* U+10AE0 */
  0x0000,   /* U+10AE1 */
  0x0000,   /* U+10AE2 */
  0x0000,   /* U+10AE3 */
  0x0000,   /* U+10AE4 */
  0x0000,   /* U+10AE5 */
  0x0000,   /* U+10AE6 */
  0x0000,   /* U+10AE7 */
  0x0000,   /* U+10AE8 */
  0x0000,   /* U+10AE9 */
  0x0000,   /* U+10AEA */
  0x0000,   /* U+10AEB */
  0x0000,   /* U+10AEC */
  0x0000,   /* U+10AED */
  0x0000,   /* U+10AEE */
  0x0000,   /* U+10AEF */
  0x0000,   /* U+10AF0 */
  0x0000,   /* U+10AF1 */
  0x0000,   /* U+10AF2 */
  0x0000,   /* U+10AF3 */
  0x0000,   /* U+10AF4 */
  0x0000,   /* U+10AF5 */
  0x0000,   /* U+10AF6 */
  0x0000,   /* U+10AF7 */
  0x0000,   /* U+10AF8 */
  0x0000,   /* U+10AF9 */
  0x0000,   /* U+10AFA */
  0x0000,   /* U+10AFB */
  0x0000,   /* U+10AFC */
  0x0000,   /* U+10AFD */
  0x0000,   /* U+10AFE */
  0x0000,   /* U+10AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10A00 */
  0x0000,   /* U+10A01 */
  0x0000,   /* U+10A02 */
  0x0000,   /* U+10A03 */
  0x0000,   /* U+10A04 */
  0x0000,   /* U+10A05 */
  0x0000,   /* U+10A06 */
  0x0000,   /* U+10A07 */
  0x0000,   /* U+10A08 */
  0x0000,   /* U+10A09 */
  0x0000,   /* U+10A0A */
  0x0000,   /* U+10A0B */
  0x0000,   /* U+10A0C */
  0x0000,   /* U+10A0D */
  0x0000,   /* U+10A0E */
  0x0000,   /* U+10A0F */
  0x0000,   /* U+10A10 */
  0x0000,   /* U+10A11 */
  0x0000,   /* U+10A12 */
  0x0000,   /* U+10A13 */
  0x0000,   /* U+10A14 */
  0x0000,   /* U+10A15 */
  0x0000,   /* U+10A16 */
  0x0000,   /* U+10A17 */
  0x0000,   /* U+10A18 */
  0x0000,   /* U+10A19 */
  0x0000,   /* U+10A1A */
  0x0000,   /* U+10A1B */
  0x0000,   /* U+10A1C */
  0x0000,   /* U+10A1D */
  0x0000,   /* U+10A1E */
  0x0000,   /* U+10A1F */
  0x0000,   /* U+10A20 */
  0x0000,   /* U+10A21 */
  0x0000,   /* U+10A22 */
  0x0000,   /* U+10A23 */
  0x0000,   /* U+10A24 */
  0x0000,   /* U+10A25 */
  0x0000,   /* U+10A26 */
  0x0000,   /* U+10A27 */
  0x0000,   /* U+10A28 */
  0x0000,   /* U+10A29 */
  0x0000,   /* U+10A2A */
  0x0000,   /* U+10A2B */
  0x0000,   /* U+10A2C */
  0x0000,   /* U+10A2D */
  0x0000,   /* U+10A2E */
  0x0000,   /* U+10A2F */
  0x0000,   /* U+10A30 */
  0x0000,   /* U+10A31 */
  0x0000,   /* U+10A32 */
  0x0000,   /* U+10A33 */
  0x0000,   /* U+10A34 */
  0x0000,   /* U+10A35 */
  0x0000,   /* U+10A36 */
  0x0000,   /* U+10A37 */
  0x0000,   /* U+10A38 */
  0x0000,   /* U+10A39 */
  0x0000,   /* U+10A3A */
  0x0000,   /* U+10A3B */
  0x0000,   /* U+10A3C */
  0x0000,   /* U+10A3D */
  0x0000,   /* U+10A3E */
  0x0000,   /* U+10A3F */
  0x0000,   /* U+10A40 */
  0x0000,   /* U+10A41 */
  0x0000,   /* U+10A42 */
  0x0000,   /* U+10A43 */
  0x0000,   /* U+10A44 */
  0x0000,   /* U+10A45 */
  0x0000,   /* U+10A46 */
  0x0000,   /* U+10A47 */
  0x0000,   /* U+10A48 */
  0x0000,   /* U+10A49 */
  0x0000,   /* U+10A4A */
  0x0000,   /* U+10A4B */
  0x0000,   /* U+10A4C */
  0x0000,   /* U+10A4D */
  0x0000,   /* U+10A4E */
  0x0000,   /* U+10A4F */
  0x0000,   /* U+10A50 */
  0x0000,   /* U+10A51 */
  0x0000,   /* U+10A52 */
  0x0000,   /* U+10A53 */
  0x0000,   /* U+10A54 */
  0x0000,   /* U+10A55 */
  0x0000,   /* U+10A56 */
  0x0000,   /* U+10A57 */
  0x0000,   /* U+10A58 */
  0x0000,   /* U+10A59 */
  0x0000,   /* U+10A5A */
  0x0000,   /* U+10A5B */
  0x0000,   /* U+10A5C */
  0x0000,   /* U+10A5D */
  0x0000,   /* U+10A5E */
  0x0000,   /* U+10A5F */
  0x0000,   /* U+10A60 */
  0x0000,   /* U+10A61 */
  0x0000,   /* U+10A62 */
  0x0000,   /* U+10A63 */
  0x0000,   /* U+10A64 */
  0x0000,   /* U+10A65 */
  0x0000,   /* U+10A66 */
  0x0000,   /* U+10A67 */
  0x0000,   /* U+10A68 */
  0x0000,   /* U+10A69 */
  0x0000,   /* U+10A6A */
  0x0000,   /* U+10A6B */
  0x0000,   /* U+10A6C */
  0x0000,   /* U+10A6D */
  0x0000,   /* U+10A6E */
  0x0000,   /* U+10A6F */
  0x0000,   /* U+10A70 */
  0x0000,   /* U+10A71 */
  0x0000,   /* U+10A72 */
  0x0000,   /* U+10A73 */
  0x0000,   /* U+10A74 */
  0x0000,   /* U+10A75 */
  0x0000,   /* U+10A76 */
  0x0000,   /* U+10A77 */
  0x0000,   /* U+10A78 */
  0x0000,   /* U+10A79 */
  0x0000,   /* U+10A7A */
  0x0000,   /* U+10A7B */
  0x0000,   /* U+10A7C */
  0x0000,   /* U+10A7D */
  0x0000,   /* U+10A7E */
  0x0000,   /* U+10A7F */
  0x0000,   /* U+10A80 */
  0x0000,   /* U+10A81 */
  0x0000,   /* U+10A82 */
  0x0000,   /* U+10A83 */
  0x0000,   /* U+10A84 */
  0x0000,   /* U+10A85 */
  0x0000,   /* U+10A86 */
  0x0000,   /* U+10A87 */
  0x0000,   /* U+10A88 */
  0x0000,   /* U+10A89 */
  0x0000,   /* U+10A8A */
  0x0000,   /* U+10A8B */
  0x0000,   /* U+10A8C */
  0x0000,   /* U+10A8D */
  0x0000,   /* U+10A8E */
  0x0000,   /* U+10A8F */
  0x0000,   /* U+10A90 */
  0x0000,   /* U+10A91 */
  0x0000,   /* U+10A92 */
  0x0000,   /* U+10A93 */
  0x0000,   /* U+10A94 */
  0x0000,   /* U+10A95 */
  0x0000,   /* U+10A96 */
  0x0000,   /* U+10A97 */
  0x0000,   /* U+10A98 */
  0x0000,   /* U+10A99 */
  0x0000,   /* U+10A9A */
  0x0000,   /* U+10A9B */
  0x0000,   /* U+10A9C */
  0x0000,   /* U+10A9D */
  0x0000,   /* U+10A9E */
  0x0000,   /* U+10A9F */
  0x0000,   /* U+10AA0 */
  0x0000,   /* U+10AA1 */
  0x0000,   /* U+10AA2 */
  0x0000,   /* U+10AA3 */
  0x0000,   /* U+10AA4 */
  0x0000,   /* U+10AA5 */
  0x0000,   /* U+10AA6 */
  0x0000,   /* U+10AA7 */
  0x0000,   /* U+10AA8 */
  0x0000,   /* U+10AA9 */
  0x0000,   /* U+10AAA */
  0x0000,   /* U+10AAB */
  0x0000,   /* U+10AAC */
  0x0000,   /* U+10AAD */
  0x0000,   /* U+10AAE */
  0x0000,   /* U+10AAF */
  0x0000,   /* U+10AB0 */
  0x0000,   /* U+10AB1 */
  0x0000,   /* U+10AB2 */
  0x0000,   /* U+10AB3 */
  0x0000,   /* U+10AB4 */
  0x0000,   /* U+10AB5 */
  0x0000,   /* U+10AB6 */
  0x0000,   /* U+10AB7 */
  0x0000,   /* U+10AB8 */
  0x0000,   /* U+10AB9 */
  0x0000,   /* U+10ABA */
  0x0000,   /* U+10ABB */
  0x0000,   /* U+10ABC */
  0x0000,   /* U+10ABD */
  0x0000,   /* U+10ABE */
  0x0000,   /* U+10ABF */
  0x0000,   /* U+10AC0 */
  0x0000,   /* U+10AC1 */
  0x0000,   /* U+10AC2 */
  0x0000,   /* U+10AC3 */
  0x0000,   /* U+10AC4 */
  0x0000,   /* U+10AC5 */
  0x0000,   /* U+10AC6 */
  0x0000,   /* U+10AC7 */
  0x0004,   /* U+10AC8 */
  0x0000,   /* U+10AC9 */
  0x0000,   /* U+10ACA */
  0x0000,   /* U+10ACB */
  0x0000,   /* U+10ACC */
  0x0000,   /* U+10ACD */
  0x0000,   /* U+10ACE */
  0x0000,   /* U+10ACF */
  0x0000,   /* U+10AD0 */
  0x0000,   /* U+10AD1 */
  0x0000,   /* U+10AD2 */
  0x0000,   /* U+10AD3 */
  0x0000,   /* U+10AD4 */
  0x0000,   /* U+10AD5 */
  0x0000,   /* U+10AD6 */
  0x0000,   /* U+10AD7 */
  0x0000,   /* U+10AD8 */
  0x0000,   /* U+10AD9 */
  0x0000,   /* U+10ADA */
  0x0000,   /* U+10ADB */
  0x0000,   /* U+10ADC */
  0x0000,   /* U+10ADD */
  0x0000,   /* U+10ADE */
  0x0000,   /* U+10ADF */
  0x0000,   /* U+10AE0 */
  0x0000,   /* U+10AE1 */
  0x0000,   /* U+10AE2 */
  0x0000,   /* U+10AE3 */
  0x0000,   /* U+10AE4 */
  0x0000,   /* U+10AE5 */
  0x0000,   /* U+10AE6 */
  0x0000,   /* U+10AE7 */
  0x0000,   /* U+10AE8 */
  0x0000,   /* U+10AE9 */
  0x0000,   /* U+10AEA */
  0x0000,   /* U+10AEB */
  0x0000,   /* U+10AEC */
  0x0000,   /* U+10AED */
  0x0000,   /* U+10AEE */
  0x0000,   /* U+10AEF */
  0x0000,   /* U+10AF0 */
  0x0000,   /* U+10AF1 */
  0x0000,   /* U+10AF2 */
  0x0000,   /* U+10AF3 */
  0x0000,   /* U+10AF4 */
  0x0000,   /* U+10AF5 */
  0x0000,   /* U+10AF6 */
  0x0000,   /* U+10AF7 */
  0x0000,   /* U+10AF8 */
  0x0000,   /* U+10AF9 */
  0x0000,   /* U+10AFA */
  0x0000,   /* U+10AFB */
  0x0000,   /* U+10AFC */
  0x0000,   /* U+10AFD */
  0x0000,   /* U+10AFE */
  0x0000,   /* U+10AFF */
};

uint16 uca900_p10B[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4880,   /* U+10B00 */
  0x4881,   /* U+10B01 */
  0x4882,   /* U+10B02 */
  0x4883,   /* U+10B03 */
  0x4884,   /* U+10B04 */
  0x4885,   /* U+10B05 */
  0x4886,   /* U+10B06 */
  0x4887,   /* U+10B07 */
  0x4888,   /* U+10B08 */
  0x4889,   /* U+10B09 */
  0x488A,   /* U+10B0A */
  0x488B,   /* U+10B0B */
  0x488C,   /* U+10B0C */
  0x488D,   /* U+10B0D */
  0x488E,   /* U+10B0E */
  0x488F,   /* U+10B0F */
  0x4890,   /* U+10B10 */
  0x4891,   /* U+10B11 */
  0x4892,   /* U+10B12 */
  0x4893,   /* U+10B13 */
  0x4894,   /* U+10B14 */
  0x4895,   /* U+10B15 */
  0x4896,   /* U+10B16 */
  0x4897,   /* U+10B17 */
  0x4898,   /* U+10B18 */
  0x4899,   /* U+10B19 */
  0x489A,   /* U+10B1A */
  0x489B,   /* U+10B1B */
  0x489C,   /* U+10B1C */
  0x489D,   /* U+10B1D */
  0x489E,   /* U+10B1E */
  0x489F,   /* U+10B1F */
  0x48A0,   /* U+10B20 */
  0x48A1,   /* U+10B21 */
  0x48A2,   /* U+10B22 */
  0x48A3,   /* U+10B23 */
  0x48A4,   /* U+10B24 */
  0x48A5,   /* U+10B25 */
  0x48A6,   /* U+10B26 */
  0x48A7,   /* U+10B27 */
  0x48A8,   /* U+10B28 */
  0x48A9,   /* U+10B29 */
  0x48AA,   /* U+10B2A */
  0x48AB,   /* U+10B2B */
  0x48AC,   /* U+10B2C */
  0x48AD,   /* U+10B2D */
  0x48AD,   /* U+10B2E */
  0x48AE,   /* U+10B2F */
  0x48AF,   /* U+10B30 */
  0x48B0,   /* U+10B31 */
  0x48B1,   /* U+10B32 */
  0x48B2,   /* U+10B33 */
  0x48B3,   /* U+10B34 */
  0x48B4,   /* U+10B35 */
  0xFBC2,   /* U+10B36 */
  0xFBC2,   /* U+10B37 */
  0xFBC2,   /* U+10B38 */
  0x0442,   /* U+10B39 */
  0x02DA,   /* U+10B3A */
  0x02DB,   /* U+10B3B */
  0x02DC,   /* U+10B3C */
  0x02DD,   /* U+10B3D */
  0x02DE,   /* U+10B3E */
  0x02DF,   /* U+10B3F */
  0x490C,   /* U+10B40 */
  0x490D,   /* U+10B41 */
  0x490E,   /* U+10B42 */
  0x490F,   /* U+10B43 */
  0x4910,   /* U+10B44 */
  0x4911,   /* U+10B45 */
  0x4912,   /* U+10B46 */
  0x4913,   /* U+10B47 */
  0x4914,   /* U+10B48 */
  0x4915,   /* U+10B49 */
  0x4916,   /* U+10B4A */
  0x4917,   /* U+10B4B */
  0x4918,   /* U+10B4C */
  0x4919,   /* U+10B4D */
  0x491A,   /* U+10B4E */
  0x491B,   /* U+10B4F */
  0x491C,   /* U+10B50 */
  0x491D,   /* U+10B51 */
  0x491E,   /* U+10B52 */
  0x491F,   /* U+10B53 */
  0x4920,   /* U+10B54 */
  0x4921,   /* U+10B55 */
  0xFBC2,   /* U+10B56 */
  0xFBC2,   /* U+10B57 */
  0x1C3E,   /* U+10B58 */
  0x1C3F,   /* U+10B59 */
  0x1C40,   /* U+10B5A */
  0x1C41,   /* U+10B5B */
  0x1B66,   /* U+10B5C */
  0x1B67,   /* U+10B5D */
  0x1B68,   /* U+10B5E */
  0x1B69,   /* U+10B5F */
  0x4922,   /* U+10B60 */
  0x4923,   /* U+10B61 */
  0x4924,   /* U+10B62 */
  0x4925,   /* U+10B63 */
  0x4926,   /* U+10B64 */
  0x4927,   /* U+10B65 */
  0x4928,   /* U+10B66 */
  0x4929,   /* U+10B67 */
  0x492A,   /* U+10B68 */
  0x492B,   /* U+10B69 */
  0x492C,   /* U+10B6A */
  0x492D,   /* U+10B6B */
  0x492E,   /* U+10B6C */
  0x492F,   /* U+10B6D */
  0x4930,   /* U+10B6E */
  0x4931,   /* U+10B6F */
  0x4932,   /* U+10B70 */
  0x4933,   /* U+10B71 */
  0x4934,   /* U+10B72 */
  0xFBC2,   /* U+10B73 */
  0xFBC2,   /* U+10B74 */
  0xFBC2,   /* U+10B75 */
  0xFBC2,   /* U+10B76 */
  0xFBC2,   /* U+10B77 */
  0x1C3E,   /* U+10B78 */
  0x1C3F,   /* U+10B79 */
  0x1C40,   /* U+10B7A */
  0x1C41,   /* U+10B7B */
  0x1B6A,   /* U+10B7C */
  0x1B6B,   /* U+10B7D */
  0x1B6C,   /* U+10B7E */
  0x1B6D,   /* U+10B7F */
  0x4935,   /* U+10B80 */
  0x4936,   /* U+10B81 */
  0x4937,   /* U+10B82 */
  0x4938,   /* U+10B83 */
  0x4939,   /* U+10B84 */
  0x493A,   /* U+10B85 */
  0x493B,   /* U+10B86 */
  0x493C,   /* U+10B87 */
  0x493D,   /* U+10B88 */
  0x493E,   /* U+10B89 */
  0x493F,   /* U+10B8A */
  0x4940,   /* U+10B8B */
  0x4941,   /* U+10B8C */
  0x4942,   /* U+10B8D */
  0x4943,   /* U+10B8E */
  0x4944,   /* U+10B8F */
  0x4945,   /* U+10B90 */
  0x4946,   /* U+10B91 */
  0xFBC2,   /* U+10B92 */
  0xFBC2,   /* U+10B93 */
  0xFBC2,   /* U+10B94 */
  0xFBC2,   /* U+10B95 */
  0xFBC2,   /* U+10B96 */
  0xFBC2,   /* U+10B97 */
  0xFBC2,   /* U+10B98 */
  0x044A,   /* U+10B99 */
  0x044B,   /* U+10B9A */
  0x044C,   /* U+10B9B */
  0x044D,   /* U+10B9C */
  0xFBC2,   /* U+10B9D */
  0xFBC2,   /* U+10B9E */
  0xFBC2,   /* U+10B9F */
  0xFBC2,   /* U+10BA0 */
  0xFBC2,   /* U+10BA1 */
  0xFBC2,   /* U+10BA2 */
  0xFBC2,   /* U+10BA3 */
  0xFBC2,   /* U+10BA4 */
  0xFBC2,   /* U+10BA5 */
  0xFBC2,   /* U+10BA6 */
  0xFBC2,   /* U+10BA7 */
  0xFBC2,   /* U+10BA8 */
  0x1C3E,   /* U+10BA9 */
  0x1C3F,   /* U+10BAA */
  0x1C40,   /* U+10BAB */
  0x1C41,   /* U+10BAC */
  0x1B6E,   /* U+10BAD */
  0x1B6F,   /* U+10BAE */
  0x1B70,   /* U+10BAF */
  0xFBC2,   /* U+10BB0 */
  0xFBC2,   /* U+10BB1 */
  0xFBC2,   /* U+10BB2 */
  0xFBC2,   /* U+10BB3 */
  0xFBC2,   /* U+10BB4 */
  0xFBC2,   /* U+10BB5 */
  0xFBC2,   /* U+10BB6 */
  0xFBC2,   /* U+10BB7 */
  0xFBC2,   /* U+10BB8 */
  0xFBC2,   /* U+10BB9 */
  0xFBC2,   /* U+10BBA */
  0xFBC2,   /* U+10BBB */
  0xFBC2,   /* U+10BBC */
  0xFBC2,   /* U+10BBD */
  0xFBC2,   /* U+10BBE */
  0xFBC2,   /* U+10BBF */
  0xFBC2,   /* U+10BC0 */
  0xFBC2,   /* U+10BC1 */
  0xFBC2,   /* U+10BC2 */
  0xFBC2,   /* U+10BC3 */
  0xFBC2,   /* U+10BC4 */
  0xFBC2,   /* U+10BC5 */
  0xFBC2,   /* U+10BC6 */
  0xFBC2,   /* U+10BC7 */
  0xFBC2,   /* U+10BC8 */
  0xFBC2,   /* U+10BC9 */
  0xFBC2,   /* U+10BCA */
  0xFBC2,   /* U+10BCB */
  0xFBC2,   /* U+10BCC */
  0xFBC2,   /* U+10BCD */
  0xFBC2,   /* U+10BCE */
  0xFBC2,   /* U+10BCF */
  0xFBC2,   /* U+10BD0 */
  0xFBC2,   /* U+10BD1 */
  0xFBC2,   /* U+10BD2 */
  0xFBC2,   /* U+10BD3 */
  0xFBC2,   /* U+10BD4 */
  0xFBC2,   /* U+10BD5 */
  0xFBC2,   /* U+10BD6 */
  0xFBC2,   /* U+10BD7 */
  0xFBC2,   /* U+10BD8 */
  0xFBC2,   /* U+10BD9 */
  0xFBC2,   /* U+10BDA */
  0xFBC2,   /* U+10BDB */
  0xFBC2,   /* U+10BDC */
  0xFBC2,   /* U+10BDD */
  0xFBC2,   /* U+10BDE */
  0xFBC2,   /* U+10BDF */
  0xFBC2,   /* U+10BE0 */
  0xFBC2,   /* U+10BE1 */
  0xFBC2,   /* U+10BE2 */
  0xFBC2,   /* U+10BE3 */
  0xFBC2,   /* U+10BE4 */
  0xFBC2,   /* U+10BE5 */
  0xFBC2,   /* U+10BE6 */
  0xFBC2,   /* U+10BE7 */
  0xFBC2,   /* U+10BE8 */
  0xFBC2,   /* U+10BE9 */
  0xFBC2,   /* U+10BEA */
  0xFBC2,   /* U+10BEB */
  0xFBC2,   /* U+10BEC */
  0xFBC2,   /* U+10BED */
  0xFBC2,   /* U+10BEE */
  0xFBC2,   /* U+10BEF */
  0xFBC2,   /* U+10BF0 */
  0xFBC2,   /* U+10BF1 */
  0xFBC2,   /* U+10BF2 */
  0xFBC2,   /* U+10BF3 */
  0xFBC2,   /* U+10BF4 */
  0xFBC2,   /* U+10BF5 */
  0xFBC2,   /* U+10BF6 */
  0xFBC2,   /* U+10BF7 */
  0xFBC2,   /* U+10BF8 */
  0xFBC2,   /* U+10BF9 */
  0xFBC2,   /* U+10BFA */
  0xFBC2,   /* U+10BFB */
  0xFBC2,   /* U+10BFC */
  0xFBC2,   /* U+10BFD */
  0xFBC2,   /* U+10BFE */
  0xFBC2,   /* U+10BFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10B00 */
  0x0020,   /* U+10B01 */
  0x0020,   /* U+10B02 */
  0x0020,   /* U+10B03 */
  0x0020,   /* U+10B04 */
  0x0020,   /* U+10B05 */
  0x0020,   /* U+10B06 */
  0x0020,   /* U+10B07 */
  0x0020,   /* U+10B08 */
  0x0020,   /* U+10B09 */
  0x0020,   /* U+10B0A */
  0x0020,   /* U+10B0B */
  0x0020,   /* U+10B0C */
  0x0020,   /* U+10B0D */
  0x0020,   /* U+10B0E */
  0x0020,   /* U+10B0F */
  0x0020,   /* U+10B10 */
  0x0020,   /* U+10B11 */
  0x0020,   /* U+10B12 */
  0x0020,   /* U+10B13 */
  0x0020,   /* U+10B14 */
  0x0020,   /* U+10B15 */
  0x0020,   /* U+10B16 */
  0x0020,   /* U+10B17 */
  0x0020,   /* U+10B18 */
  0x0020,   /* U+10B19 */
  0x0020,   /* U+10B1A */
  0x0020,   /* U+10B1B */
  0x0020,   /* U+10B1C */
  0x0020,   /* U+10B1D */
  0x0020,   /* U+10B1E */
  0x0020,   /* U+10B1F */
  0x0020,   /* U+10B20 */
  0x0020,   /* U+10B21 */
  0x0020,   /* U+10B22 */
  0x0020,   /* U+10B23 */
  0x0020,   /* U+10B24 */
  0x0020,   /* U+10B25 */
  0x0020,   /* U+10B26 */
  0x0020,   /* U+10B27 */
  0x0020,   /* U+10B28 */
  0x0020,   /* U+10B29 */
  0x0020,   /* U+10B2A */
  0x0020,   /* U+10B2B */
  0x0020,   /* U+10B2C */
  0x0020,   /* U+10B2D */
  0x0020,   /* U+10B2E */
  0x0020,   /* U+10B2F */
  0x0020,   /* U+10B30 */
  0x0020,   /* U+10B31 */
  0x0020,   /* U+10B32 */
  0x0020,   /* U+10B33 */
  0x0020,   /* U+10B34 */
  0x0020,   /* U+10B35 */
  0x0020,   /* U+10B36 */
  0x0020,   /* U+10B37 */
  0x0020,   /* U+10B38 */
  0x0020,   /* U+10B39 */
  0x0020,   /* U+10B3A */
  0x0020,   /* U+10B3B */
  0x0020,   /* U+10B3C */
  0x0020,   /* U+10B3D */
  0x0020,   /* U+10B3E */
  0x0020,   /* U+10B3F */
  0x0020,   /* U+10B40 */
  0x0020,   /* U+10B41 */
  0x0020,   /* U+10B42 */
  0x0020,   /* U+10B43 */
  0x0020,   /* U+10B44 */
  0x0020,   /* U+10B45 */
  0x0020,   /* U+10B46 */
  0x0020,   /* U+10B47 */
  0x0020,   /* U+10B48 */
  0x0020,   /* U+10B49 */
  0x0020,   /* U+10B4A */
  0x0020,   /* U+10B4B */
  0x0020,   /* U+10B4C */
  0x0020,   /* U+10B4D */
  0x0020,   /* U+10B4E */
  0x0020,   /* U+10B4F */
  0x0020,   /* U+10B50 */
  0x0020,   /* U+10B51 */
  0x0020,   /* U+10B52 */
  0x0020,   /* U+10B53 */
  0x0020,   /* U+10B54 */
  0x0020,   /* U+10B55 */
  0x0020,   /* U+10B56 */
  0x0020,   /* U+10B57 */
  0x0020,   /* U+10B58 */
  0x0020,   /* U+10B59 */
  0x0020,   /* U+10B5A */
  0x0020,   /* U+10B5B */
  0x0020,   /* U+10B5C */
  0x0020,   /* U+10B5D */
  0x0020,   /* U+10B5E */
  0x0020,   /* U+10B5F */
  0x0020,   /* U+10B60 */
  0x0020,   /* U+10B61 */
  0x0020,   /* U+10B62 */
  0x0020,   /* U+10B63 */
  0x0020,   /* U+10B64 */
  0x0020,   /* U+10B65 */
  0x0020,   /* U+10B66 */
  0x0020,   /* U+10B67 */
  0x0020,   /* U+10B68 */
  0x0020,   /* U+10B69 */
  0x0020,   /* U+10B6A */
  0x0020,   /* U+10B6B */
  0x0020,   /* U+10B6C */
  0x0020,   /* U+10B6D */
  0x0020,   /* U+10B6E */
  0x0020,   /* U+10B6F */
  0x0020,   /* U+10B70 */
  0x0020,   /* U+10B71 */
  0x0020,   /* U+10B72 */
  0x0020,   /* U+10B73 */
  0x0020,   /* U+10B74 */
  0x0020,   /* U+10B75 */
  0x0020,   /* U+10B76 */
  0x0020,   /* U+10B77 */
  0x0020,   /* U+10B78 */
  0x0020,   /* U+10B79 */
  0x0020,   /* U+10B7A */
  0x0020,   /* U+10B7B */
  0x0020,   /* U+10B7C */
  0x0020,   /* U+10B7D */
  0x0020,   /* U+10B7E */
  0x0020,   /* U+10B7F */
  0x0020,   /* U+10B80 */
  0x0020,   /* U+10B81 */
  0x0020,   /* U+10B82 */
  0x0020,   /* U+10B83 */
  0x0020,   /* U+10B84 */
  0x0020,   /* U+10B85 */
  0x0020,   /* U+10B86 */
  0x0020,   /* U+10B87 */
  0x0020,   /* U+10B88 */
  0x0020,   /* U+10B89 */
  0x0020,   /* U+10B8A */
  0x0020,   /* U+10B8B */
  0x0020,   /* U+10B8C */
  0x0020,   /* U+10B8D */
  0x0020,   /* U+10B8E */
  0x0020,   /* U+10B8F */
  0x0020,   /* U+10B90 */
  0x0020,   /* U+10B91 */
  0x0020,   /* U+10B92 */
  0x0020,   /* U+10B93 */
  0x0020,   /* U+10B94 */
  0x0020,   /* U+10B95 */
  0x0020,   /* U+10B96 */
  0x0020,   /* U+10B97 */
  0x0020,   /* U+10B98 */
  0x0020,   /* U+10B99 */
  0x0020,   /* U+10B9A */
  0x0020,   /* U+10B9B */
  0x0020,   /* U+10B9C */
  0x0020,   /* U+10B9D */
  0x0020,   /* U+10B9E */
  0x0020,   /* U+10B9F */
  0x0020,   /* U+10BA0 */
  0x0020,   /* U+10BA1 */
  0x0020,   /* U+10BA2 */
  0x0020,   /* U+10BA3 */
  0x0020,   /* U+10BA4 */
  0x0020,   /* U+10BA5 */
  0x0020,   /* U+10BA6 */
  0x0020,   /* U+10BA7 */
  0x0020,   /* U+10BA8 */
  0x0020,   /* U+10BA9 */
  0x0020,   /* U+10BAA */
  0x0020,   /* U+10BAB */
  0x0020,   /* U+10BAC */
  0x0020,   /* U+10BAD */
  0x0020,   /* U+10BAE */
  0x0020,   /* U+10BAF */
  0x0020,   /* U+10BB0 */
  0x0020,   /* U+10BB1 */
  0x0020,   /* U+10BB2 */
  0x0020,   /* U+10BB3 */
  0x0020,   /* U+10BB4 */
  0x0020,   /* U+10BB5 */
  0x0020,   /* U+10BB6 */
  0x0020,   /* U+10BB7 */
  0x0020,   /* U+10BB8 */
  0x0020,   /* U+10BB9 */
  0x0020,   /* U+10BBA */
  0x0020,   /* U+10BBB */
  0x0020,   /* U+10BBC */
  0x0020,   /* U+10BBD */
  0x0020,   /* U+10BBE */
  0x0020,   /* U+10BBF */
  0x0020,   /* U+10BC0 */
  0x0020,   /* U+10BC1 */
  0x0020,   /* U+10BC2 */
  0x0020,   /* U+10BC3 */
  0x0020,   /* U+10BC4 */
  0x0020,   /* U+10BC5 */
  0x0020,   /* U+10BC6 */
  0x0020,   /* U+10BC7 */
  0x0020,   /* U+10BC8 */
  0x0020,   /* U+10BC9 */
  0x0020,   /* U+10BCA */
  0x0020,   /* U+10BCB */
  0x0020,   /* U+10BCC */
  0x0020,   /* U+10BCD */
  0x0020,   /* U+10BCE */
  0x0020,   /* U+10BCF */
  0x0020,   /* U+10BD0 */
  0x0020,   /* U+10BD1 */
  0x0020,   /* U+10BD2 */
  0x0020,   /* U+10BD3 */
  0x0020,   /* U+10BD4 */
  0x0020,   /* U+10BD5 */
  0x0020,   /* U+10BD6 */
  0x0020,   /* U+10BD7 */
  0x0020,   /* U+10BD8 */
  0x0020,   /* U+10BD9 */
  0x0020,   /* U+10BDA */
  0x0020,   /* U+10BDB */
  0x0020,   /* U+10BDC */
  0x0020,   /* U+10BDD */
  0x0020,   /* U+10BDE */
  0x0020,   /* U+10BDF */
  0x0020,   /* U+10BE0 */
  0x0020,   /* U+10BE1 */
  0x0020,   /* U+10BE2 */
  0x0020,   /* U+10BE3 */
  0x0020,   /* U+10BE4 */
  0x0020,   /* U+10BE5 */
  0x0020,   /* U+10BE6 */
  0x0020,   /* U+10BE7 */
  0x0020,   /* U+10BE8 */
  0x0020,   /* U+10BE9 */
  0x0020,   /* U+10BEA */
  0x0020,   /* U+10BEB */
  0x0020,   /* U+10BEC */
  0x0020,   /* U+10BED */
  0x0020,   /* U+10BEE */
  0x0020,   /* U+10BEF */
  0x0020,   /* U+10BF0 */
  0x0020,   /* U+10BF1 */
  0x0020,   /* U+10BF2 */
  0x0020,   /* U+10BF3 */
  0x0020,   /* U+10BF4 */
  0x0020,   /* U+10BF5 */
  0x0020,   /* U+10BF6 */
  0x0020,   /* U+10BF7 */
  0x0020,   /* U+10BF8 */
  0x0020,   /* U+10BF9 */
  0x0020,   /* U+10BFA */
  0x0020,   /* U+10BFB */
  0x0020,   /* U+10BFC */
  0x0020,   /* U+10BFD */
  0x0020,   /* U+10BFE */
  0x0020,   /* U+10BFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10B00 */
  0x0002,   /* U+10B01 */
  0x0002,   /* U+10B02 */
  0x0002,   /* U+10B03 */
  0x0002,   /* U+10B04 */
  0x0002,   /* U+10B05 */
  0x0002,   /* U+10B06 */
  0x0002,   /* U+10B07 */
  0x0002,   /* U+10B08 */
  0x0002,   /* U+10B09 */
  0x0002,   /* U+10B0A */
  0x0002,   /* U+10B0B */
  0x0002,   /* U+10B0C */
  0x0002,   /* U+10B0D */
  0x0002,   /* U+10B0E */
  0x0002,   /* U+10B0F */
  0x0002,   /* U+10B10 */
  0x0002,   /* U+10B11 */
  0x0002,   /* U+10B12 */
  0x0002,   /* U+10B13 */
  0x0002,   /* U+10B14 */
  0x0002,   /* U+10B15 */
  0x0002,   /* U+10B16 */
  0x0002,   /* U+10B17 */
  0x0002,   /* U+10B18 */
  0x0002,   /* U+10B19 */
  0x0002,   /* U+10B1A */
  0x0002,   /* U+10B1B */
  0x0002,   /* U+10B1C */
  0x0002,   /* U+10B1D */
  0x0002,   /* U+10B1E */
  0x0002,   /* U+10B1F */
  0x0002,   /* U+10B20 */
  0x0002,   /* U+10B21 */
  0x0002,   /* U+10B22 */
  0x0002,   /* U+10B23 */
  0x0002,   /* U+10B24 */
  0x0002,   /* U+10B25 */
  0x0002,   /* U+10B26 */
  0x0002,   /* U+10B27 */
  0x0002,   /* U+10B28 */
  0x0002,   /* U+10B29 */
  0x0002,   /* U+10B2A */
  0x0002,   /* U+10B2B */
  0x0002,   /* U+10B2C */
  0x0002,   /* U+10B2D */
  0x0004,   /* U+10B2E */
  0x0002,   /* U+10B2F */
  0x0002,   /* U+10B30 */
  0x0002,   /* U+10B31 */
  0x0002,   /* U+10B32 */
  0x0002,   /* U+10B33 */
  0x0002,   /* U+10B34 */
  0x0002,   /* U+10B35 */
  0x0002,   /* U+10B36 */
  0x0002,   /* U+10B37 */
  0x0002,   /* U+10B38 */
  0x0002,   /* U+10B39 */
  0x0002,   /* U+10B3A */
  0x0002,   /* U+10B3B */
  0x0002,   /* U+10B3C */
  0x0002,   /* U+10B3D */
  0x0002,   /* U+10B3E */
  0x0002,   /* U+10B3F */
  0x0002,   /* U+10B40 */
  0x0002,   /* U+10B41 */
  0x0002,   /* U+10B42 */
  0x0002,   /* U+10B43 */
  0x0002,   /* U+10B44 */
  0x0002,   /* U+10B45 */
  0x0002,   /* U+10B46 */
  0x0002,   /* U+10B47 */
  0x0002,   /* U+10B48 */
  0x0002,   /* U+10B49 */
  0x0002,   /* U+10B4A */
  0x0002,   /* U+10B4B */
  0x0002,   /* U+10B4C */
  0x0002,   /* U+10B4D */
  0x0002,   /* U+10B4E */
  0x0002,   /* U+10B4F */
  0x0002,   /* U+10B50 */
  0x0002,   /* U+10B51 */
  0x0002,   /* U+10B52 */
  0x0002,   /* U+10B53 */
  0x0002,   /* U+10B54 */
  0x0002,   /* U+10B55 */
  0x0002,   /* U+10B56 */
  0x0002,   /* U+10B57 */
  0x0002,   /* U+10B58 */
  0x0002,   /* U+10B59 */
  0x0002,   /* U+10B5A */
  0x0002,   /* U+10B5B */
  0x0002,   /* U+10B5C */
  0x0002,   /* U+10B5D */
  0x0002,   /* U+10B5E */
  0x0002,   /* U+10B5F */
  0x0002,   /* U+10B60 */
  0x0002,   /* U+10B61 */
  0x0002,   /* U+10B62 */
  0x0002,   /* U+10B63 */
  0x0002,   /* U+10B64 */
  0x0002,   /* U+10B65 */
  0x0002,   /* U+10B66 */
  0x0002,   /* U+10B67 */
  0x0002,   /* U+10B68 */
  0x0002,   /* U+10B69 */
  0x0002,   /* U+10B6A */
  0x0002,   /* U+10B6B */
  0x0002,   /* U+10B6C */
  0x0002,   /* U+10B6D */
  0x0002,   /* U+10B6E */
  0x0002,   /* U+10B6F */
  0x0002,   /* U+10B70 */
  0x0002,   /* U+10B71 */
  0x0002,   /* U+10B72 */
  0x0002,   /* U+10B73 */
  0x0002,   /* U+10B74 */
  0x0002,   /* U+10B75 */
  0x0002,   /* U+10B76 */
  0x0002,   /* U+10B77 */
  0x0002,   /* U+10B78 */
  0x0002,   /* U+10B79 */
  0x0002,   /* U+10B7A */
  0x0002,   /* U+10B7B */
  0x0002,   /* U+10B7C */
  0x0002,   /* U+10B7D */
  0x0002,   /* U+10B7E */
  0x0002,   /* U+10B7F */
  0x0002,   /* U+10B80 */
  0x0002,   /* U+10B81 */
  0x0002,   /* U+10B82 */
  0x0002,   /* U+10B83 */
  0x0002,   /* U+10B84 */
  0x0002,   /* U+10B85 */
  0x0002,   /* U+10B86 */
  0x0002,   /* U+10B87 */
  0x0002,   /* U+10B88 */
  0x0002,   /* U+10B89 */
  0x0002,   /* U+10B8A */
  0x0002,   /* U+10B8B */
  0x0002,   /* U+10B8C */
  0x0002,   /* U+10B8D */
  0x0002,   /* U+10B8E */
  0x0002,   /* U+10B8F */
  0x0002,   /* U+10B90 */
  0x0002,   /* U+10B91 */
  0x0002,   /* U+10B92 */
  0x0002,   /* U+10B93 */
  0x0002,   /* U+10B94 */
  0x0002,   /* U+10B95 */
  0x0002,   /* U+10B96 */
  0x0002,   /* U+10B97 */
  0x0002,   /* U+10B98 */
  0x0002,   /* U+10B99 */
  0x0002,   /* U+10B9A */
  0x0002,   /* U+10B9B */
  0x0002,   /* U+10B9C */
  0x0002,   /* U+10B9D */
  0x0002,   /* U+10B9E */
  0x0002,   /* U+10B9F */
  0x0002,   /* U+10BA0 */
  0x0002,   /* U+10BA1 */
  0x0002,   /* U+10BA2 */
  0x0002,   /* U+10BA3 */
  0x0002,   /* U+10BA4 */
  0x0002,   /* U+10BA5 */
  0x0002,   /* U+10BA6 */
  0x0002,   /* U+10BA7 */
  0x0002,   /* U+10BA8 */
  0x0002,   /* U+10BA9 */
  0x0002,   /* U+10BAA */
  0x0002,   /* U+10BAB */
  0x0002,   /* U+10BAC */
  0x0002,   /* U+10BAD */
  0x0002,   /* U+10BAE */
  0x0002,   /* U+10BAF */
  0x0002,   /* U+10BB0 */
  0x0002,   /* U+10BB1 */
  0x0002,   /* U+10BB2 */
  0x0002,   /* U+10BB3 */
  0x0002,   /* U+10BB4 */
  0x0002,   /* U+10BB5 */
  0x0002,   /* U+10BB6 */
  0x0002,   /* U+10BB7 */
  0x0002,   /* U+10BB8 */
  0x0002,   /* U+10BB9 */
  0x0002,   /* U+10BBA */
  0x0002,   /* U+10BBB */
  0x0002,   /* U+10BBC */
  0x0002,   /* U+10BBD */
  0x0002,   /* U+10BBE */
  0x0002,   /* U+10BBF */
  0x0002,   /* U+10BC0 */
  0x0002,   /* U+10BC1 */
  0x0002,   /* U+10BC2 */
  0x0002,   /* U+10BC3 */
  0x0002,   /* U+10BC4 */
  0x0002,   /* U+10BC5 */
  0x0002,   /* U+10BC6 */
  0x0002,   /* U+10BC7 */
  0x0002,   /* U+10BC8 */
  0x0002,   /* U+10BC9 */
  0x0002,   /* U+10BCA */
  0x0002,   /* U+10BCB */
  0x0002,   /* U+10BCC */
  0x0002,   /* U+10BCD */
  0x0002,   /* U+10BCE */
  0x0002,   /* U+10BCF */
  0x0002,   /* U+10BD0 */
  0x0002,   /* U+10BD1 */
  0x0002,   /* U+10BD2 */
  0x0002,   /* U+10BD3 */
  0x0002,   /* U+10BD4 */
  0x0002,   /* U+10BD5 */
  0x0002,   /* U+10BD6 */
  0x0002,   /* U+10BD7 */
  0x0002,   /* U+10BD8 */
  0x0002,   /* U+10BD9 */
  0x0002,   /* U+10BDA */
  0x0002,   /* U+10BDB */
  0x0002,   /* U+10BDC */
  0x0002,   /* U+10BDD */
  0x0002,   /* U+10BDE */
  0x0002,   /* U+10BDF */
  0x0002,   /* U+10BE0 */
  0x0002,   /* U+10BE1 */
  0x0002,   /* U+10BE2 */
  0x0002,   /* U+10BE3 */
  0x0002,   /* U+10BE4 */
  0x0002,   /* U+10BE5 */
  0x0002,   /* U+10BE6 */
  0x0002,   /* U+10BE7 */
  0x0002,   /* U+10BE8 */
  0x0002,   /* U+10BE9 */
  0x0002,   /* U+10BEA */
  0x0002,   /* U+10BEB */
  0x0002,   /* U+10BEC */
  0x0002,   /* U+10BED */
  0x0002,   /* U+10BEE */
  0x0002,   /* U+10BEF */
  0x0002,   /* U+10BF0 */
  0x0002,   /* U+10BF1 */
  0x0002,   /* U+10BF2 */
  0x0002,   /* U+10BF3 */
  0x0002,   /* U+10BF4 */
  0x0002,   /* U+10BF5 */
  0x0002,   /* U+10BF6 */
  0x0002,   /* U+10BF7 */
  0x0002,   /* U+10BF8 */
  0x0002,   /* U+10BF9 */
  0x0002,   /* U+10BFA */
  0x0002,   /* U+10BFB */
  0x0002,   /* U+10BFC */
  0x0002,   /* U+10BFD */
  0x0002,   /* U+10BFE */
  0x0002,   /* U+10BFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10B00 */
  0x0000,   /* U+10B01 */
  0x0000,   /* U+10B02 */
  0x0000,   /* U+10B03 */
  0x0000,   /* U+10B04 */
  0x0000,   /* U+10B05 */
  0x0000,   /* U+10B06 */
  0x0000,   /* U+10B07 */
  0x0000,   /* U+10B08 */
  0x0000,   /* U+10B09 */
  0x0000,   /* U+10B0A */
  0x0000,   /* U+10B0B */
  0x0000,   /* U+10B0C */
  0x0000,   /* U+10B0D */
  0x0000,   /* U+10B0E */
  0x0000,   /* U+10B0F */
  0x0000,   /* U+10B10 */
  0x0000,   /* U+10B11 */
  0x0000,   /* U+10B12 */
  0x0000,   /* U+10B13 */
  0x0000,   /* U+10B14 */
  0x0000,   /* U+10B15 */
  0x0000,   /* U+10B16 */
  0x0000,   /* U+10B17 */
  0x0000,   /* U+10B18 */
  0x0000,   /* U+10B19 */
  0x0000,   /* U+10B1A */
  0x0000,   /* U+10B1B */
  0x0000,   /* U+10B1C */
  0x0000,   /* U+10B1D */
  0x0000,   /* U+10B1E */
  0x0000,   /* U+10B1F */
  0x0000,   /* U+10B20 */
  0x0000,   /* U+10B21 */
  0x0000,   /* U+10B22 */
  0x0000,   /* U+10B23 */
  0x0000,   /* U+10B24 */
  0x0000,   /* U+10B25 */
  0x0000,   /* U+10B26 */
  0x0000,   /* U+10B27 */
  0x0000,   /* U+10B28 */
  0x0000,   /* U+10B29 */
  0x0000,   /* U+10B2A */
  0x0000,   /* U+10B2B */
  0x0000,   /* U+10B2C */
  0x0000,   /* U+10B2D */
  0x0000,   /* U+10B2E */
  0x0000,   /* U+10B2F */
  0x0000,   /* U+10B30 */
  0x0000,   /* U+10B31 */
  0x0000,   /* U+10B32 */
  0x0000,   /* U+10B33 */
  0x0000,   /* U+10B34 */
  0x0000,   /* U+10B35 */
  0x8B36,   /* U+10B36 */
  0x8B37,   /* U+10B37 */
  0x8B38,   /* U+10B38 */
  0x0000,   /* U+10B39 */
  0x0000,   /* U+10B3A */
  0x0000,   /* U+10B3B */
  0x0000,   /* U+10B3C */
  0x0000,   /* U+10B3D */
  0x0000,   /* U+10B3E */
  0x0000,   /* U+10B3F */
  0x0000,   /* U+10B40 */
  0x0000,   /* U+10B41 */
  0x0000,   /* U+10B42 */
  0x0000,   /* U+10B43 */
  0x0000,   /* U+10B44 */
  0x0000,   /* U+10B45 */
  0x0000,   /* U+10B46 */
  0x0000,   /* U+10B47 */
  0x0000,   /* U+10B48 */
  0x0000,   /* U+10B49 */
  0x0000,   /* U+10B4A */
  0x0000,   /* U+10B4B */
  0x0000,   /* U+10B4C */
  0x0000,   /* U+10B4D */
  0x0000,   /* U+10B4E */
  0x0000,   /* U+10B4F */
  0x0000,   /* U+10B50 */
  0x0000,   /* U+10B51 */
  0x0000,   /* U+10B52 */
  0x0000,   /* U+10B53 */
  0x0000,   /* U+10B54 */
  0x0000,   /* U+10B55 */
  0x8B56,   /* U+10B56 */
  0x8B57,   /* U+10B57 */
  0x0000,   /* U+10B58 */
  0x0000,   /* U+10B59 */
  0x0000,   /* U+10B5A */
  0x0000,   /* U+10B5B */
  0x0000,   /* U+10B5C */
  0x0000,   /* U+10B5D */
  0x0000,   /* U+10B5E */
  0x0000,   /* U+10B5F */
  0x0000,   /* U+10B60 */
  0x0000,   /* U+10B61 */
  0x0000,   /* U+10B62 */
  0x0000,   /* U+10B63 */
  0x0000,   /* U+10B64 */
  0x0000,   /* U+10B65 */
  0x0000,   /* U+10B66 */
  0x0000,   /* U+10B67 */
  0x0000,   /* U+10B68 */
  0x0000,   /* U+10B69 */
  0x0000,   /* U+10B6A */
  0x0000,   /* U+10B6B */
  0x0000,   /* U+10B6C */
  0x0000,   /* U+10B6D */
  0x0000,   /* U+10B6E */
  0x0000,   /* U+10B6F */
  0x0000,   /* U+10B70 */
  0x0000,   /* U+10B71 */
  0x0000,   /* U+10B72 */
  0x8B73,   /* U+10B73 */
  0x8B74,   /* U+10B74 */
  0x8B75,   /* U+10B75 */
  0x8B76,   /* U+10B76 */
  0x8B77,   /* U+10B77 */
  0x0000,   /* U+10B78 */
  0x0000,   /* U+10B79 */
  0x0000,   /* U+10B7A */
  0x0000,   /* U+10B7B */
  0x0000,   /* U+10B7C */
  0x0000,   /* U+10B7D */
  0x0000,   /* U+10B7E */
  0x0000,   /* U+10B7F */
  0x0000,   /* U+10B80 */
  0x0000,   /* U+10B81 */
  0x0000,   /* U+10B82 */
  0x0000,   /* U+10B83 */
  0x0000,   /* U+10B84 */
  0x0000,   /* U+10B85 */
  0x0000,   /* U+10B86 */
  0x0000,   /* U+10B87 */
  0x0000,   /* U+10B88 */
  0x0000,   /* U+10B89 */
  0x0000,   /* U+10B8A */
  0x0000,   /* U+10B8B */
  0x0000,   /* U+10B8C */
  0x0000,   /* U+10B8D */
  0x0000,   /* U+10B8E */
  0x0000,   /* U+10B8F */
  0x0000,   /* U+10B90 */
  0x0000,   /* U+10B91 */
  0x8B92,   /* U+10B92 */
  0x8B93,   /* U+10B93 */
  0x8B94,   /* U+10B94 */
  0x8B95,   /* U+10B95 */
  0x8B96,   /* U+10B96 */
  0x8B97,   /* U+10B97 */
  0x8B98,   /* U+10B98 */
  0x0000,   /* U+10B99 */
  0x0000,   /* U+10B9A */
  0x0000,   /* U+10B9B */
  0x0000,   /* U+10B9C */
  0x8B9D,   /* U+10B9D */
  0x8B9E,   /* U+10B9E */
  0x8B9F,   /* U+10B9F */
  0x8BA0,   /* U+10BA0 */
  0x8BA1,   /* U+10BA1 */
  0x8BA2,   /* U+10BA2 */
  0x8BA3,   /* U+10BA3 */
  0x8BA4,   /* U+10BA4 */
  0x8BA5,   /* U+10BA5 */
  0x8BA6,   /* U+10BA6 */
  0x8BA7,   /* U+10BA7 */
  0x8BA8,   /* U+10BA8 */
  0x0000,   /* U+10BA9 */
  0x0000,   /* U+10BAA */
  0x0000,   /* U+10BAB */
  0x0000,   /* U+10BAC */
  0x0000,   /* U+10BAD */
  0x0000,   /* U+10BAE */
  0x0000,   /* U+10BAF */
  0x8BB0,   /* U+10BB0 */
  0x8BB1,   /* U+10BB1 */
  0x8BB2,   /* U+10BB2 */
  0x8BB3,   /* U+10BB3 */
  0x8BB4,   /* U+10BB4 */
  0x8BB5,   /* U+10BB5 */
  0x8BB6,   /* U+10BB6 */
  0x8BB7,   /* U+10BB7 */
  0x8BB8,   /* U+10BB8 */
  0x8BB9,   /* U+10BB9 */
  0x8BBA,   /* U+10BBA */
  0x8BBB,   /* U+10BBB */
  0x8BBC,   /* U+10BBC */
  0x8BBD,   /* U+10BBD */
  0x8BBE,   /* U+10BBE */
  0x8BBF,   /* U+10BBF */
  0x8BC0,   /* U+10BC0 */
  0x8BC1,   /* U+10BC1 */
  0x8BC2,   /* U+10BC2 */
  0x8BC3,   /* U+10BC3 */
  0x8BC4,   /* U+10BC4 */
  0x8BC5,   /* U+10BC5 */
  0x8BC6,   /* U+10BC6 */
  0x8BC7,   /* U+10BC7 */
  0x8BC8,   /* U+10BC8 */
  0x8BC9,   /* U+10BC9 */
  0x8BCA,   /* U+10BCA */
  0x8BCB,   /* U+10BCB */
  0x8BCC,   /* U+10BCC */
  0x8BCD,   /* U+10BCD */
  0x8BCE,   /* U+10BCE */
  0x8BCF,   /* U+10BCF */
  0x8BD0,   /* U+10BD0 */
  0x8BD1,   /* U+10BD1 */
  0x8BD2,   /* U+10BD2 */
  0x8BD3,   /* U+10BD3 */
  0x8BD4,   /* U+10BD4 */
  0x8BD5,   /* U+10BD5 */
  0x8BD6,   /* U+10BD6 */
  0x8BD7,   /* U+10BD7 */
  0x8BD8,   /* U+10BD8 */
  0x8BD9,   /* U+10BD9 */
  0x8BDA,   /* U+10BDA */
  0x8BDB,   /* U+10BDB */
  0x8BDC,   /* U+10BDC */
  0x8BDD,   /* U+10BDD */
  0x8BDE,   /* U+10BDE */
  0x8BDF,   /* U+10BDF */
  0x8BE0,   /* U+10BE0 */
  0x8BE1,   /* U+10BE1 */
  0x8BE2,   /* U+10BE2 */
  0x8BE3,   /* U+10BE3 */
  0x8BE4,   /* U+10BE4 */
  0x8BE5,   /* U+10BE5 */
  0x8BE6,   /* U+10BE6 */
  0x8BE7,   /* U+10BE7 */
  0x8BE8,   /* U+10BE8 */
  0x8BE9,   /* U+10BE9 */
  0x8BEA,   /* U+10BEA */
  0x8BEB,   /* U+10BEB */
  0x8BEC,   /* U+10BEC */
  0x8BED,   /* U+10BED */
  0x8BEE,   /* U+10BEE */
  0x8BEF,   /* U+10BEF */
  0x8BF0,   /* U+10BF0 */
  0x8BF1,   /* U+10BF1 */
  0x8BF2,   /* U+10BF2 */
  0x8BF3,   /* U+10BF3 */
  0x8BF4,   /* U+10BF4 */
  0x8BF5,   /* U+10BF5 */
  0x8BF6,   /* U+10BF6 */
  0x8BF7,   /* U+10BF7 */
  0x8BF8,   /* U+10BF8 */
  0x8BF9,   /* U+10BF9 */
  0x8BFA,   /* U+10BFA */
  0x8BFB,   /* U+10BFB */
  0x8BFC,   /* U+10BFC */
  0x8BFD,   /* U+10BFD */
  0x8BFE,   /* U+10BFE */
  0x8BFF,   /* U+10BFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10B00 */
  0x0000,   /* U+10B01 */
  0x0000,   /* U+10B02 */
  0x0000,   /* U+10B03 */
  0x0000,   /* U+10B04 */
  0x0000,   /* U+10B05 */
  0x0000,   /* U+10B06 */
  0x0000,   /* U+10B07 */
  0x0000,   /* U+10B08 */
  0x0000,   /* U+10B09 */
  0x0000,   /* U+10B0A */
  0x0000,   /* U+10B0B */
  0x0000,   /* U+10B0C */
  0x0000,   /* U+10B0D */
  0x0000,   /* U+10B0E */
  0x0000,   /* U+10B0F */
  0x0000,   /* U+10B10 */
  0x0000,   /* U+10B11 */
  0x0000,   /* U+10B12 */
  0x0000,   /* U+10B13 */
  0x0000,   /* U+10B14 */
  0x0000,   /* U+10B15 */
  0x0000,   /* U+10B16 */
  0x0000,   /* U+10B17 */
  0x0000,   /* U+10B18 */
  0x0000,   /* U+10B19 */
  0x0000,   /* U+10B1A */
  0x0000,   /* U+10B1B */
  0x0000,   /* U+10B1C */
  0x0000,   /* U+10B1D */
  0x0000,   /* U+10B1E */
  0x0000,   /* U+10B1F */
  0x0000,   /* U+10B20 */
  0x0000,   /* U+10B21 */
  0x0000,   /* U+10B22 */
  0x0000,   /* U+10B23 */
  0x0000,   /* U+10B24 */
  0x0000,   /* U+10B25 */
  0x0000,   /* U+10B26 */
  0x0000,   /* U+10B27 */
  0x0000,   /* U+10B28 */
  0x0000,   /* U+10B29 */
  0x0000,   /* U+10B2A */
  0x0000,   /* U+10B2B */
  0x0000,   /* U+10B2C */
  0x0000,   /* U+10B2D */
  0x0110,   /* U+10B2E */
  0x0000,   /* U+10B2F */
  0x0000,   /* U+10B30 */
  0x0000,   /* U+10B31 */
  0x0000,   /* U+10B32 */
  0x0000,   /* U+10B33 */
  0x0000,   /* U+10B34 */
  0x0000,   /* U+10B35 */
  0x0000,   /* U+10B36 */
  0x0000,   /* U+10B37 */
  0x0000,   /* U+10B38 */
  0x0000,   /* U+10B39 */
  0x0000,   /* U+10B3A */
  0x0000,   /* U+10B3B */
  0x0000,   /* U+10B3C */
  0x0000,   /* U+10B3D */
  0x0000,   /* U+10B3E */
  0x0000,   /* U+10B3F */
  0x0000,   /* U+10B40 */
  0x0000,   /* U+10B41 */
  0x0000,   /* U+10B42 */
  0x0000,   /* U+10B43 */
  0x0000,   /* U+10B44 */
  0x0000,   /* U+10B45 */
  0x0000,   /* U+10B46 */
  0x0000,   /* U+10B47 */
  0x0000,   /* U+10B48 */
  0x0000,   /* U+10B49 */
  0x0000,   /* U+10B4A */
  0x0000,   /* U+10B4B */
  0x0000,   /* U+10B4C */
  0x0000,   /* U+10B4D */
  0x0000,   /* U+10B4E */
  0x0000,   /* U+10B4F */
  0x0000,   /* U+10B50 */
  0x0000,   /* U+10B51 */
  0x0000,   /* U+10B52 */
  0x0000,   /* U+10B53 */
  0x0000,   /* U+10B54 */
  0x0000,   /* U+10B55 */
  0x0000,   /* U+10B56 */
  0x0000,   /* U+10B57 */
  0x0000,   /* U+10B58 */
  0x0000,   /* U+10B59 */
  0x0000,   /* U+10B5A */
  0x0000,   /* U+10B5B */
  0x0000,   /* U+10B5C */
  0x0000,   /* U+10B5D */
  0x0000,   /* U+10B5E */
  0x0000,   /* U+10B5F */
  0x0000,   /* U+10B60 */
  0x0000,   /* U+10B61 */
  0x0000,   /* U+10B62 */
  0x0000,   /* U+10B63 */
  0x0000,   /* U+10B64 */
  0x0000,   /* U+10B65 */
  0x0000,   /* U+10B66 */
  0x0000,   /* U+10B67 */
  0x0000,   /* U+10B68 */
  0x0000,   /* U+10B69 */
  0x0000,   /* U+10B6A */
  0x0000,   /* U+10B6B */
  0x0000,   /* U+10B6C */
  0x0000,   /* U+10B6D */
  0x0000,   /* U+10B6E */
  0x0000,   /* U+10B6F */
  0x0000,   /* U+10B70 */
  0x0000,   /* U+10B71 */
  0x0000,   /* U+10B72 */
  0x0000,   /* U+10B73 */
  0x0000,   /* U+10B74 */
  0x0000,   /* U+10B75 */
  0x0000,   /* U+10B76 */
  0x0000,   /* U+10B77 */
  0x0000,   /* U+10B78 */
  0x0000,   /* U+10B79 */
  0x0000,   /* U+10B7A */
  0x0000,   /* U+10B7B */
  0x0000,   /* U+10B7C */
  0x0000,   /* U+10B7D */
  0x0000,   /* U+10B7E */
  0x0000,   /* U+10B7F */
  0x0000,   /* U+10B80 */
  0x0000,   /* U+10B81 */
  0x0000,   /* U+10B82 */
  0x0000,   /* U+10B83 */
  0x0000,   /* U+10B84 */
  0x0000,   /* U+10B85 */
  0x0000,   /* U+10B86 */
  0x0000,   /* U+10B87 */
  0x0000,   /* U+10B88 */
  0x0000,   /* U+10B89 */
  0x0000,   /* U+10B8A */
  0x0000,   /* U+10B8B */
  0x0000,   /* U+10B8C */
  0x0000,   /* U+10B8D */
  0x0000,   /* U+10B8E */
  0x0000,   /* U+10B8F */
  0x0000,   /* U+10B90 */
  0x0000,   /* U+10B91 */
  0x0000,   /* U+10B92 */
  0x0000,   /* U+10B93 */
  0x0000,   /* U+10B94 */
  0x0000,   /* U+10B95 */
  0x0000,   /* U+10B96 */
  0x0000,   /* U+10B97 */
  0x0000,   /* U+10B98 */
  0x0000,   /* U+10B99 */
  0x0000,   /* U+10B9A */
  0x0000,   /* U+10B9B */
  0x0000,   /* U+10B9C */
  0x0000,   /* U+10B9D */
  0x0000,   /* U+10B9E */
  0x0000,   /* U+10B9F */
  0x0000,   /* U+10BA0 */
  0x0000,   /* U+10BA1 */
  0x0000,   /* U+10BA2 */
  0x0000,   /* U+10BA3 */
  0x0000,   /* U+10BA4 */
  0x0000,   /* U+10BA5 */
  0x0000,   /* U+10BA6 */
  0x0000,   /* U+10BA7 */
  0x0000,   /* U+10BA8 */
  0x0000,   /* U+10BA9 */
  0x0000,   /* U+10BAA */
  0x0000,   /* U+10BAB */
  0x0000,   /* U+10BAC */
  0x0000,   /* U+10BAD */
  0x0000,   /* U+10BAE */
  0x0000,   /* U+10BAF */
  0x0000,   /* U+10BB0 */
  0x0000,   /* U+10BB1 */
  0x0000,   /* U+10BB2 */
  0x0000,   /* U+10BB3 */
  0x0000,   /* U+10BB4 */
  0x0000,   /* U+10BB5 */
  0x0000,   /* U+10BB6 */
  0x0000,   /* U+10BB7 */
  0x0000,   /* U+10BB8 */
  0x0000,   /* U+10BB9 */
  0x0000,   /* U+10BBA */
  0x0000,   /* U+10BBB */
  0x0000,   /* U+10BBC */
  0x0000,   /* U+10BBD */
  0x0000,   /* U+10BBE */
  0x0000,   /* U+10BBF */
  0x0000,   /* U+10BC0 */
  0x0000,   /* U+10BC1 */
  0x0000,   /* U+10BC2 */
  0x0000,   /* U+10BC3 */
  0x0000,   /* U+10BC4 */
  0x0000,   /* U+10BC5 */
  0x0000,   /* U+10BC6 */
  0x0000,   /* U+10BC7 */
  0x0000,   /* U+10BC8 */
  0x0000,   /* U+10BC9 */
  0x0000,   /* U+10BCA */
  0x0000,   /* U+10BCB */
  0x0000,   /* U+10BCC */
  0x0000,   /* U+10BCD */
  0x0000,   /* U+10BCE */
  0x0000,   /* U+10BCF */
  0x0000,   /* U+10BD0 */
  0x0000,   /* U+10BD1 */
  0x0000,   /* U+10BD2 */
  0x0000,   /* U+10BD3 */
  0x0000,   /* U+10BD4 */
  0x0000,   /* U+10BD5 */
  0x0000,   /* U+10BD6 */
  0x0000,   /* U+10BD7 */
  0x0000,   /* U+10BD8 */
  0x0000,   /* U+10BD9 */
  0x0000,   /* U+10BDA */
  0x0000,   /* U+10BDB */
  0x0000,   /* U+10BDC */
  0x0000,   /* U+10BDD */
  0x0000,   /* U+10BDE */
  0x0000,   /* U+10BDF */
  0x0000,   /* U+10BE0 */
  0x0000,   /* U+10BE1 */
  0x0000,   /* U+10BE2 */
  0x0000,   /* U+10BE3 */
  0x0000,   /* U+10BE4 */
  0x0000,   /* U+10BE5 */
  0x0000,   /* U+10BE6 */
  0x0000,   /* U+10BE7 */
  0x0000,   /* U+10BE8 */
  0x0000,   /* U+10BE9 */
  0x0000,   /* U+10BEA */
  0x0000,   /* U+10BEB */
  0x0000,   /* U+10BEC */
  0x0000,   /* U+10BED */
  0x0000,   /* U+10BEE */
  0x0000,   /* U+10BEF */
  0x0000,   /* U+10BF0 */
  0x0000,   /* U+10BF1 */
  0x0000,   /* U+10BF2 */
  0x0000,   /* U+10BF3 */
  0x0000,   /* U+10BF4 */
  0x0000,   /* U+10BF5 */
  0x0000,   /* U+10BF6 */
  0x0000,   /* U+10BF7 */
  0x0000,   /* U+10BF8 */
  0x0000,   /* U+10BF9 */
  0x0000,   /* U+10BFA */
  0x0000,   /* U+10BFB */
  0x0000,   /* U+10BFC */
  0x0000,   /* U+10BFD */
  0x0000,   /* U+10BFE */
  0x0000,   /* U+10BFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10B00 */
  0x0000,   /* U+10B01 */
  0x0000,   /* U+10B02 */
  0x0000,   /* U+10B03 */
  0x0000,   /* U+10B04 */
  0x0000,   /* U+10B05 */
  0x0000,   /* U+10B06 */
  0x0000,   /* U+10B07 */
  0x0000,   /* U+10B08 */
  0x0000,   /* U+10B09 */
  0x0000,   /* U+10B0A */
  0x0000,   /* U+10B0B */
  0x0000,   /* U+10B0C */
  0x0000,   /* U+10B0D */
  0x0000,   /* U+10B0E */
  0x0000,   /* U+10B0F */
  0x0000,   /* U+10B10 */
  0x0000,   /* U+10B11 */
  0x0000,   /* U+10B12 */
  0x0000,   /* U+10B13 */
  0x0000,   /* U+10B14 */
  0x0000,   /* U+10B15 */
  0x0000,   /* U+10B16 */
  0x0000,   /* U+10B17 */
  0x0000,   /* U+10B18 */
  0x0000,   /* U+10B19 */
  0x0000,   /* U+10B1A */
  0x0000,   /* U+10B1B */
  0x0000,   /* U+10B1C */
  0x0000,   /* U+10B1D */
  0x0000,   /* U+10B1E */
  0x0000,   /* U+10B1F */
  0x0000,   /* U+10B20 */
  0x0000,   /* U+10B21 */
  0x0000,   /* U+10B22 */
  0x0000,   /* U+10B23 */
  0x0000,   /* U+10B24 */
  0x0000,   /* U+10B25 */
  0x0000,   /* U+10B26 */
  0x0000,   /* U+10B27 */
  0x0000,   /* U+10B28 */
  0x0000,   /* U+10B29 */
  0x0000,   /* U+10B2A */
  0x0000,   /* U+10B2B */
  0x0000,   /* U+10B2C */
  0x0000,   /* U+10B2D */
  0x0004,   /* U+10B2E */
  0x0000,   /* U+10B2F */
  0x0000,   /* U+10B30 */
  0x0000,   /* U+10B31 */
  0x0000,   /* U+10B32 */
  0x0000,   /* U+10B33 */
  0x0000,   /* U+10B34 */
  0x0000,   /* U+10B35 */
  0x0000,   /* U+10B36 */
  0x0000,   /* U+10B37 */
  0x0000,   /* U+10B38 */
  0x0000,   /* U+10B39 */
  0x0000,   /* U+10B3A */
  0x0000,   /* U+10B3B */
  0x0000,   /* U+10B3C */
  0x0000,   /* U+10B3D */
  0x0000,   /* U+10B3E */
  0x0000,   /* U+10B3F */
  0x0000,   /* U+10B40 */
  0x0000,   /* U+10B41 */
  0x0000,   /* U+10B42 */
  0x0000,   /* U+10B43 */
  0x0000,   /* U+10B44 */
  0x0000,   /* U+10B45 */
  0x0000,   /* U+10B46 */
  0x0000,   /* U+10B47 */
  0x0000,   /* U+10B48 */
  0x0000,   /* U+10B49 */
  0x0000,   /* U+10B4A */
  0x0000,   /* U+10B4B */
  0x0000,   /* U+10B4C */
  0x0000,   /* U+10B4D */
  0x0000,   /* U+10B4E */
  0x0000,   /* U+10B4F */
  0x0000,   /* U+10B50 */
  0x0000,   /* U+10B51 */
  0x0000,   /* U+10B52 */
  0x0000,   /* U+10B53 */
  0x0000,   /* U+10B54 */
  0x0000,   /* U+10B55 */
  0x0000,   /* U+10B56 */
  0x0000,   /* U+10B57 */
  0x0000,   /* U+10B58 */
  0x0000,   /* U+10B59 */
  0x0000,   /* U+10B5A */
  0x0000,   /* U+10B5B */
  0x0000,   /* U+10B5C */
  0x0000,   /* U+10B5D */
  0x0000,   /* U+10B5E */
  0x0000,   /* U+10B5F */
  0x0000,   /* U+10B60 */
  0x0000,   /* U+10B61 */
  0x0000,   /* U+10B62 */
  0x0000,   /* U+10B63 */
  0x0000,   /* U+10B64 */
  0x0000,   /* U+10B65 */
  0x0000,   /* U+10B66 */
  0x0000,   /* U+10B67 */
  0x0000,   /* U+10B68 */
  0x0000,   /* U+10B69 */
  0x0000,   /* U+10B6A */
  0x0000,   /* U+10B6B */
  0x0000,   /* U+10B6C */
  0x0000,   /* U+10B6D */
  0x0000,   /* U+10B6E */
  0x0000,   /* U+10B6F */
  0x0000,   /* U+10B70 */
  0x0000,   /* U+10B71 */
  0x0000,   /* U+10B72 */
  0x0000,   /* U+10B73 */
  0x0000,   /* U+10B74 */
  0x0000,   /* U+10B75 */
  0x0000,   /* U+10B76 */
  0x0000,   /* U+10B77 */
  0x0000,   /* U+10B78 */
  0x0000,   /* U+10B79 */
  0x0000,   /* U+10B7A */
  0x0000,   /* U+10B7B */
  0x0000,   /* U+10B7C */
  0x0000,   /* U+10B7D */
  0x0000,   /* U+10B7E */
  0x0000,   /* U+10B7F */
  0x0000,   /* U+10B80 */
  0x0000,   /* U+10B81 */
  0x0000,   /* U+10B82 */
  0x0000,   /* U+10B83 */
  0x0000,   /* U+10B84 */
  0x0000,   /* U+10B85 */
  0x0000,   /* U+10B86 */
  0x0000,   /* U+10B87 */
  0x0000,   /* U+10B88 */
  0x0000,   /* U+10B89 */
  0x0000,   /* U+10B8A */
  0x0000,   /* U+10B8B */
  0x0000,   /* U+10B8C */
  0x0000,   /* U+10B8D */
  0x0000,   /* U+10B8E */
  0x0000,   /* U+10B8F */
  0x0000,   /* U+10B90 */
  0x0000,   /* U+10B91 */
  0x0000,   /* U+10B92 */
  0x0000,   /* U+10B93 */
  0x0000,   /* U+10B94 */
  0x0000,   /* U+10B95 */
  0x0000,   /* U+10B96 */
  0x0000,   /* U+10B97 */
  0x0000,   /* U+10B98 */
  0x0000,   /* U+10B99 */
  0x0000,   /* U+10B9A */
  0x0000,   /* U+10B9B */
  0x0000,   /* U+10B9C */
  0x0000,   /* U+10B9D */
  0x0000,   /* U+10B9E */
  0x0000,   /* U+10B9F */
  0x0000,   /* U+10BA0 */
  0x0000,   /* U+10BA1 */
  0x0000,   /* U+10BA2 */
  0x0000,   /* U+10BA3 */
  0x0000,   /* U+10BA4 */
  0x0000,   /* U+10BA5 */
  0x0000,   /* U+10BA6 */
  0x0000,   /* U+10BA7 */
  0x0000,   /* U+10BA8 */
  0x0000,   /* U+10BA9 */
  0x0000,   /* U+10BAA */
  0x0000,   /* U+10BAB */
  0x0000,   /* U+10BAC */
  0x0000,   /* U+10BAD */
  0x0000,   /* U+10BAE */
  0x0000,   /* U+10BAF */
  0x0000,   /* U+10BB0 */
  0x0000,   /* U+10BB1 */
  0x0000,   /* U+10BB2 */
  0x0000,   /* U+10BB3 */
  0x0000,   /* U+10BB4 */
  0x0000,   /* U+10BB5 */
  0x0000,   /* U+10BB6 */
  0x0000,   /* U+10BB7 */
  0x0000,   /* U+10BB8 */
  0x0000,   /* U+10BB9 */
  0x0000,   /* U+10BBA */
  0x0000,   /* U+10BBB */
  0x0000,   /* U+10BBC */
  0x0000,   /* U+10BBD */
  0x0000,   /* U+10BBE */
  0x0000,   /* U+10BBF */
  0x0000,   /* U+10BC0 */
  0x0000,   /* U+10BC1 */
  0x0000,   /* U+10BC2 */
  0x0000,   /* U+10BC3 */
  0x0000,   /* U+10BC4 */
  0x0000,   /* U+10BC5 */
  0x0000,   /* U+10BC6 */
  0x0000,   /* U+10BC7 */
  0x0000,   /* U+10BC8 */
  0x0000,   /* U+10BC9 */
  0x0000,   /* U+10BCA */
  0x0000,   /* U+10BCB */
  0x0000,   /* U+10BCC */
  0x0000,   /* U+10BCD */
  0x0000,   /* U+10BCE */
  0x0000,   /* U+10BCF */
  0x0000,   /* U+10BD0 */
  0x0000,   /* U+10BD1 */
  0x0000,   /* U+10BD2 */
  0x0000,   /* U+10BD3 */
  0x0000,   /* U+10BD4 */
  0x0000,   /* U+10BD5 */
  0x0000,   /* U+10BD6 */
  0x0000,   /* U+10BD7 */
  0x0000,   /* U+10BD8 */
  0x0000,   /* U+10BD9 */
  0x0000,   /* U+10BDA */
  0x0000,   /* U+10BDB */
  0x0000,   /* U+10BDC */
  0x0000,   /* U+10BDD */
  0x0000,   /* U+10BDE */
  0x0000,   /* U+10BDF */
  0x0000,   /* U+10BE0 */
  0x0000,   /* U+10BE1 */
  0x0000,   /* U+10BE2 */
  0x0000,   /* U+10BE3 */
  0x0000,   /* U+10BE4 */
  0x0000,   /* U+10BE5 */
  0x0000,   /* U+10BE6 */
  0x0000,   /* U+10BE7 */
  0x0000,   /* U+10BE8 */
  0x0000,   /* U+10BE9 */
  0x0000,   /* U+10BEA */
  0x0000,   /* U+10BEB */
  0x0000,   /* U+10BEC */
  0x0000,   /* U+10BED */
  0x0000,   /* U+10BEE */
  0x0000,   /* U+10BEF */
  0x0000,   /* U+10BF0 */
  0x0000,   /* U+10BF1 */
  0x0000,   /* U+10BF2 */
  0x0000,   /* U+10BF3 */
  0x0000,   /* U+10BF4 */
  0x0000,   /* U+10BF5 */
  0x0000,   /* U+10BF6 */
  0x0000,   /* U+10BF7 */
  0x0000,   /* U+10BF8 */
  0x0000,   /* U+10BF9 */
  0x0000,   /* U+10BFA */
  0x0000,   /* U+10BFB */
  0x0000,   /* U+10BFC */
  0x0000,   /* U+10BFD */
  0x0000,   /* U+10BFE */
  0x0000,   /* U+10BFF */
};

uint16 uca900_p10C[]= {
  /* Number of CEs for each character. */
  1, 2, 1, 1, 2, 1, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 
  2, 1, 2, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 
  1, 1, 1, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 1, 2, 1, 
  1, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 1, 1, 1, 
  2, 1, 2, 1, 2, 1, 2, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 
  1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x372D,   /* U+10C00 */
  0x372D,   /* U+10C01 */
  0x372E,   /* U+10C02 */
  0x372F,   /* U+10C03 */
  0x372F,   /* U+10C04 */
  0x3730,   /* U+10C05 */
  0x3731,   /* U+10C06 */
  0x3732,   /* U+10C07 */
  0x3732,   /* U+10C08 */
  0x3733,   /* U+10C09 */
  0x3733,   /* U+10C0A */
  0x3734,   /* U+10C0B */
  0x3734,   /* U+10C0C */
  0x3735,   /* U+10C0D */
  0x3735,   /* U+10C0E */
  0x3736,   /* U+10C0F */
  0x3736,   /* U+10C10 */
  0x3737,   /* U+10C11 */
  0x3737,   /* U+10C12 */
  0x3738,   /* U+10C13 */
  0x3739,   /* U+10C14 */
  0x3739,   /* U+10C15 */
  0x373A,   /* U+10C16 */
  0x373A,   /* U+10C17 */
  0x373B,   /* U+10C18 */
  0x373B,   /* U+10C19 */
  0x373C,   /* U+10C1A */
  0x373C,   /* U+10C1B */
  0x373D,   /* U+10C1C */
  0x373D,   /* U+10C1D */
  0x373E,   /* U+10C1E */
  0x373E,   /* U+10C1F */
  0x373F,   /* U+10C20 */
  0x3740,   /* U+10C21 */
  0x3741,   /* U+10C22 */
  0x3742,   /* U+10C23 */
  0x3743,   /* U+10C24 */
  0x3743,   /* U+10C25 */
  0x3744,   /* U+10C26 */
  0x3744,   /* U+10C27 */
  0x3745,   /* U+10C28 */
  0x3745,   /* U+10C29 */
  0x3746,   /* U+10C2A */
  0x3746,   /* U+10C2B */
  0x3747,   /* U+10C2C */
  0x3748,   /* U+10C2D */
  0x3748,   /* U+10C2E */
  0x3749,   /* U+10C2F */
  0x374A,   /* U+10C30 */
  0x374B,   /* U+10C31 */
  0x374C,   /* U+10C32 */
  0x374C,   /* U+10C33 */
  0x374D,   /* U+10C34 */
  0x374D,   /* U+10C35 */
  0x374E,   /* U+10C36 */
  0x374E,   /* U+10C37 */
  0x374F,   /* U+10C38 */
  0x374F,   /* U+10C39 */
  0x3750,   /* U+10C3A */
  0x3750,   /* U+10C3B */
  0x3751,   /* U+10C3C */
  0x3752,   /* U+10C3D */
  0x3753,   /* U+10C3E */
  0x3754,   /* U+10C3F */
  0x3754,   /* U+10C40 */
  0x3755,   /* U+10C41 */
  0x3755,   /* U+10C42 */
  0x3756,   /* U+10C43 */
  0x3756,   /* U+10C44 */
  0x3757,   /* U+10C45 */
  0x3757,   /* U+10C46 */
  0x3758,   /* U+10C47 */
  0x3759,   /* U+10C48 */
  0xFBC2,   /* U+10C49 */
  0xFBC2,   /* U+10C4A */
  0xFBC2,   /* U+10C4B */
  0xFBC2,   /* U+10C4C */
  0xFBC2,   /* U+10C4D */
  0xFBC2,   /* U+10C4E */
  0xFBC2,   /* U+10C4F */
  0xFBC2,   /* U+10C50 */
  0xFBC2,   /* U+10C51 */
  0xFBC2,   /* U+10C52 */
  0xFBC2,   /* U+10C53 */
  0xFBC2,   /* U+10C54 */
  0xFBC2,   /* U+10C55 */
  0xFBC2,   /* U+10C56 */
  0xFBC2,   /* U+10C57 */
  0xFBC2,   /* U+10C58 */
  0xFBC2,   /* U+10C59 */
  0xFBC2,   /* U+10C5A */
  0xFBC2,   /* U+10C5B */
  0xFBC2,   /* U+10C5C */
  0xFBC2,   /* U+10C5D */
  0xFBC2,   /* U+10C5E */
  0xFBC2,   /* U+10C5F */
  0xFBC2,   /* U+10C60 */
  0xFBC2,   /* U+10C61 */
  0xFBC2,   /* U+10C62 */
  0xFBC2,   /* U+10C63 */
  0xFBC2,   /* U+10C64 */
  0xFBC2,   /* U+10C65 */
  0xFBC2,   /* U+10C66 */
  0xFBC2,   /* U+10C67 */
  0xFBC2,   /* U+10C68 */
  0xFBC2,   /* U+10C69 */
  0xFBC2,   /* U+10C6A */
  0xFBC2,   /* U+10C6B */
  0xFBC2,   /* U+10C6C */
  0xFBC2,   /* U+10C6D */
  0xFBC2,   /* U+10C6E */
  0xFBC2,   /* U+10C6F */
  0xFBC2,   /* U+10C70 */
  0xFBC2,   /* U+10C71 */
  0xFBC2,   /* U+10C72 */
  0xFBC2,   /* U+10C73 */
  0xFBC2,   /* U+10C74 */
  0xFBC2,   /* U+10C75 */
  0xFBC2,   /* U+10C76 */
  0xFBC2,   /* U+10C77 */
  0xFBC2,   /* U+10C78 */
  0xFBC2,   /* U+10C79 */
  0xFBC2,   /* U+10C7A */
  0xFBC2,   /* U+10C7B */
  0xFBC2,   /* U+10C7C */
  0xFBC2,   /* U+10C7D */
  0xFBC2,   /* U+10C7E */
  0xFBC2,   /* U+10C7F */
  0x3704,   /* U+10C80 */
  0x3704,   /* U+10C81 */
  0x3705,   /* U+10C82 */
  0x3706,   /* U+10C83 */
  0x3707,   /* U+10C84 */
  0x3708,   /* U+10C85 */
  0x3709,   /* U+10C86 */
  0x370A,   /* U+10C87 */
  0x370B,   /* U+10C88 */
  0x370C,   /* U+10C89 */
  0x370C,   /* U+10C8A */
  0x370C,   /* U+10C8B */
  0x370D,   /* U+10C8C */
  0x370E,   /* U+10C8D */
  0x370F,   /* U+10C8E */
  0x3710,   /* U+10C8F */
  0x3711,   /* U+10C90 */
  0x3711,   /* U+10C91 */
  0x3712,   /* U+10C92 */
  0x3713,   /* U+10C93 */
  0x3714,   /* U+10C94 */
  0x3715,   /* U+10C95 */
  0x3716,   /* U+10C96 */
  0x3717,   /* U+10C97 */
  0x3718,   /* U+10C98 */
  0x3719,   /* U+10C99 */
  0x371A,   /* U+10C9A */
  0x371B,   /* U+10C9B */
  0x371B,   /* U+10C9C */
  0x371C,   /* U+10C9D */
  0x371C,   /* U+10C9E */
  0x371C,   /* U+10C9F */
  0x371D,   /* U+10CA0 */
  0x371E,   /* U+10CA1 */
  0x371F,   /* U+10CA2 */
  0x371F,   /* U+10CA3 */
  0x3720,   /* U+10CA4 */
  0x3721,   /* U+10CA5 */
  0x3722,   /* U+10CA6 */
  0x3723,   /* U+10CA7 */
  0x3724,   /* U+10CA8 */
  0x3725,   /* U+10CA9 */
  0x3726,   /* U+10CAA */
  0x3726,   /* U+10CAB */
  0x3727,   /* U+10CAC */
  0x3727,   /* U+10CAD */
  0x3728,   /* U+10CAE */
  0x3729,   /* U+10CAF */
  0x372A,   /* U+10CB0 */
  0x372B,   /* U+10CB1 */
  0x372C,   /* U+10CB2 */
  0xFBC2,   /* U+10CB3 */
  0xFBC2,   /* U+10CB4 */
  0xFBC2,   /* U+10CB5 */
  0xFBC2,   /* U+10CB6 */
  0xFBC2,   /* U+10CB7 */
  0xFBC2,   /* U+10CB8 */
  0xFBC2,   /* U+10CB9 */
  0xFBC2,   /* U+10CBA */
  0xFBC2,   /* U+10CBB */
  0xFBC2,   /* U+10CBC */
  0xFBC2,   /* U+10CBD */
  0xFBC2,   /* U+10CBE */
  0xFBC2,   /* U+10CBF */
  0x3704,   /* U+10CC0 */
  0x3704,   /* U+10CC1 */
  0x3705,   /* U+10CC2 */
  0x3706,   /* U+10CC3 */
  0x3707,   /* U+10CC4 */
  0x3708,   /* U+10CC5 */
  0x3709,   /* U+10CC6 */
  0x370A,   /* U+10CC7 */
  0x370B,   /* U+10CC8 */
  0x370C,   /* U+10CC9 */
  0x370C,   /* U+10CCA */
  0x370C,   /* U+10CCB */
  0x370D,   /* U+10CCC */
  0x370E,   /* U+10CCD */
  0x370F,   /* U+10CCE */
  0x3710,   /* U+10CCF */
  0x3711,   /* U+10CD0 */
  0x3711,   /* U+10CD1 */
  0x3712,   /* U+10CD2 */
  0x3713,   /* U+10CD3 */
  0x3714,   /* U+10CD4 */
  0x3715,   /* U+10CD5 */
  0x3716,   /* U+10CD6 */
  0x3717,   /* U+10CD7 */
  0x3718,   /* U+10CD8 */
  0x3719,   /* U+10CD9 */
  0x371A,   /* U+10CDA */
  0x371B,   /* U+10CDB */
  0x371B,   /* U+10CDC */
  0x371C,   /* U+10CDD */
  0x371C,   /* U+10CDE */
  0x371C,   /* U+10CDF */
  0x371D,   /* U+10CE0 */
  0x371E,   /* U+10CE1 */
  0x371F,   /* U+10CE2 */
  0x371F,   /* U+10CE3 */
  0x3720,   /* U+10CE4 */
  0x3721,   /* U+10CE5 */
  0x3722,   /* U+10CE6 */
  0x3723,   /* U+10CE7 */
  0x3724,   /* U+10CE8 */
  0x3725,   /* U+10CE9 */
  0x3726,   /* U+10CEA */
  0x3726,   /* U+10CEB */
  0x3727,   /* U+10CEC */
  0x3727,   /* U+10CED */
  0x3728,   /* U+10CEE */
  0x3729,   /* U+10CEF */
  0x372A,   /* U+10CF0 */
  0x372B,   /* U+10CF1 */
  0x372C,   /* U+10CF2 */
  0xFBC2,   /* U+10CF3 */
  0xFBC2,   /* U+10CF4 */
  0xFBC2,   /* U+10CF5 */
  0xFBC2,   /* U+10CF6 */
  0xFBC2,   /* U+10CF7 */
  0xFBC2,   /* U+10CF8 */
  0xFBC2,   /* U+10CF9 */
  0x1C3E,   /* U+10CFA */
  0x1C42,   /* U+10CFB */
  0x1AE5,   /* U+10CFC */
  0x1AE6,   /* U+10CFD */
  0x1AE7,   /* U+10CFE */
  0x1AE8,   /* U+10CFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10C00 */
  0x0020,   /* U+10C01 */
  0x0020,   /* U+10C02 */
  0x0020,   /* U+10C03 */
  0x0020,   /* U+10C04 */
  0x0020,   /* U+10C05 */
  0x0020,   /* U+10C06 */
  0x0020,   /* U+10C07 */
  0x0020,   /* U+10C08 */
  0x0020,   /* U+10C09 */
  0x0020,   /* U+10C0A */
  0x0020,   /* U+10C0B */
  0x0020,   /* U+10C0C */
  0x0020,   /* U+10C0D */
  0x0020,   /* U+10C0E */
  0x0020,   /* U+10C0F */
  0x0020,   /* U+10C10 */
  0x0020,   /* U+10C11 */
  0x0020,   /* U+10C12 */
  0x0020,   /* U+10C13 */
  0x0020,   /* U+10C14 */
  0x0020,   /* U+10C15 */
  0x0020,   /* U+10C16 */
  0x0020,   /* U+10C17 */
  0x0020,   /* U+10C18 */
  0x0020,   /* U+10C19 */
  0x0020,   /* U+10C1A */
  0x0020,   /* U+10C1B */
  0x0020,   /* U+10C1C */
  0x0020,   /* U+10C1D */
  0x0020,   /* U+10C1E */
  0x0020,   /* U+10C1F */
  0x0020,   /* U+10C20 */
  0x0020,   /* U+10C21 */
  0x0020,   /* U+10C22 */
  0x0020,   /* U+10C23 */
  0x0020,   /* U+10C24 */
  0x0020,   /* U+10C25 */
  0x0020,   /* U+10C26 */
  0x0020,   /* U+10C27 */
  0x0020,   /* U+10C28 */
  0x0020,   /* U+10C29 */
  0x0020,   /* U+10C2A */
  0x0020,   /* U+10C2B */
  0x0020,   /* U+10C2C */
  0x0020,   /* U+10C2D */
  0x0020,   /* U+10C2E */
  0x0020,   /* U+10C2F */
  0x0020,   /* U+10C30 */
  0x0020,   /* U+10C31 */
  0x0020,   /* U+10C32 */
  0x0020,   /* U+10C33 */
  0x0020,   /* U+10C34 */
  0x0020,   /* U+10C35 */
  0x0020,   /* U+10C36 */
  0x0020,   /* U+10C37 */
  0x0020,   /* U+10C38 */
  0x0020,   /* U+10C39 */
  0x0020,   /* U+10C3A */
  0x0020,   /* U+10C3B */
  0x0020,   /* U+10C3C */
  0x0020,   /* U+10C3D */
  0x0020,   /* U+10C3E */
  0x0020,   /* U+10C3F */
  0x0020,   /* U+10C40 */
  0x0020,   /* U+10C41 */
  0x0020,   /* U+10C42 */
  0x0020,   /* U+10C43 */
  0x0020,   /* U+10C44 */
  0x0020,   /* U+10C45 */
  0x0020,   /* U+10C46 */
  0x0020,   /* U+10C47 */
  0x0020,   /* U+10C48 */
  0x0020,   /* U+10C49 */
  0x0020,   /* U+10C4A */
  0x0020,   /* U+10C4B */
  0x0020,   /* U+10C4C */
  0x0020,   /* U+10C4D */
  0x0020,   /* U+10C4E */
  0x0020,   /* U+10C4F */
  0x0020,   /* U+10C50 */
  0x0020,   /* U+10C51 */
  0x0020,   /* U+10C52 */
  0x0020,   /* U+10C53 */
  0x0020,   /* U+10C54 */
  0x0020,   /* U+10C55 */
  0x0020,   /* U+10C56 */
  0x0020,   /* U+10C57 */
  0x0020,   /* U+10C58 */
  0x0020,   /* U+10C59 */
  0x0020,   /* U+10C5A */
  0x0020,   /* U+10C5B */
  0x0020,   /* U+10C5C */
  0x0020,   /* U+10C5D */
  0x0020,   /* U+10C5E */
  0x0020,   /* U+10C5F */
  0x0020,   /* U+10C60 */
  0x0020,   /* U+10C61 */
  0x0020,   /* U+10C62 */
  0x0020,   /* U+10C63 */
  0x0020,   /* U+10C64 */
  0x0020,   /* U+10C65 */
  0x0020,   /* U+10C66 */
  0x0020,   /* U+10C67 */
  0x0020,   /* U+10C68 */
  0x0020,   /* U+10C69 */
  0x0020,   /* U+10C6A */
  0x0020,   /* U+10C6B */
  0x0020,   /* U+10C6C */
  0x0020,   /* U+10C6D */
  0x0020,   /* U+10C6E */
  0x0020,   /* U+10C6F */
  0x0020,   /* U+10C70 */
  0x0020,   /* U+10C71 */
  0x0020,   /* U+10C72 */
  0x0020,   /* U+10C73 */
  0x0020,   /* U+10C74 */
  0x0020,   /* U+10C75 */
  0x0020,   /* U+10C76 */
  0x0020,   /* U+10C77 */
  0x0020,   /* U+10C78 */
  0x0020,   /* U+10C79 */
  0x0020,   /* U+10C7A */
  0x0020,   /* U+10C7B */
  0x0020,   /* U+10C7C */
  0x0020,   /* U+10C7D */
  0x0020,   /* U+10C7E */
  0x0020,   /* U+10C7F */
  0x0020,   /* U+10C80 */
  0x0020,   /* U+10C81 */
  0x0020,   /* U+10C82 */
  0x0020,   /* U+10C83 */
  0x0020,   /* U+10C84 */
  0x0020,   /* U+10C85 */
  0x0020,   /* U+10C86 */
  0x0020,   /* U+10C87 */
  0x0020,   /* U+10C88 */
  0x0020,   /* U+10C89 */
  0x0020,   /* U+10C8A */
  0x0020,   /* U+10C8B */
  0x0020,   /* U+10C8C */
  0x0020,   /* U+10C8D */
  0x0020,   /* U+10C8E */
  0x0020,   /* U+10C8F */
  0x0020,   /* U+10C90 */
  0x0020,   /* U+10C91 */
  0x0020,   /* U+10C92 */
  0x0020,   /* U+10C93 */
  0x0020,   /* U+10C94 */
  0x0020,   /* U+10C95 */
  0x0020,   /* U+10C96 */
  0x0020,   /* U+10C97 */
  0x0020,   /* U+10C98 */
  0x0020,   /* U+10C99 */
  0x0020,   /* U+10C9A */
  0x0020,   /* U+10C9B */
  0x0020,   /* U+10C9C */
  0x0020,   /* U+10C9D */
  0x0020,   /* U+10C9E */
  0x0020,   /* U+10C9F */
  0x0020,   /* U+10CA0 */
  0x0020,   /* U+10CA1 */
  0x0020,   /* U+10CA2 */
  0x0020,   /* U+10CA3 */
  0x0020,   /* U+10CA4 */
  0x0020,   /* U+10CA5 */
  0x0020,   /* U+10CA6 */
  0x0020,   /* U+10CA7 */
  0x0020,   /* U+10CA8 */
  0x0020,   /* U+10CA9 */
  0x0020,   /* U+10CAA */
  0x0020,   /* U+10CAB */
  0x0020,   /* U+10CAC */
  0x0020,   /* U+10CAD */
  0x0020,   /* U+10CAE */
  0x0020,   /* U+10CAF */
  0x0020,   /* U+10CB0 */
  0x0020,   /* U+10CB1 */
  0x0020,   /* U+10CB2 */
  0x0020,   /* U+10CB3 */
  0x0020,   /* U+10CB4 */
  0x0020,   /* U+10CB5 */
  0x0020,   /* U+10CB6 */
  0x0020,   /* U+10CB7 */
  0x0020,   /* U+10CB8 */
  0x0020,   /* U+10CB9 */
  0x0020,   /* U+10CBA */
  0x0020,   /* U+10CBB */
  0x0020,   /* U+10CBC */
  0x0020,   /* U+10CBD */
  0x0020,   /* U+10CBE */
  0x0020,   /* U+10CBF */
  0x0020,   /* U+10CC0 */
  0x0020,   /* U+10CC1 */
  0x0020,   /* U+10CC2 */
  0x0020,   /* U+10CC3 */
  0x0020,   /* U+10CC4 */
  0x0020,   /* U+10CC5 */
  0x0020,   /* U+10CC6 */
  0x0020,   /* U+10CC7 */
  0x0020,   /* U+10CC8 */
  0x0020,   /* U+10CC9 */
  0x0020,   /* U+10CCA */
  0x0020,   /* U+10CCB */
  0x0020,   /* U+10CCC */
  0x0020,   /* U+10CCD */
  0x0020,   /* U+10CCE */
  0x0020,   /* U+10CCF */
  0x0020,   /* U+10CD0 */
  0x0020,   /* U+10CD1 */
  0x0020,   /* U+10CD2 */
  0x0020,   /* U+10CD3 */
  0x0020,   /* U+10CD4 */
  0x0020,   /* U+10CD5 */
  0x0020,   /* U+10CD6 */
  0x0020,   /* U+10CD7 */
  0x0020,   /* U+10CD8 */
  0x0020,   /* U+10CD9 */
  0x0020,   /* U+10CDA */
  0x0020,   /* U+10CDB */
  0x0020,   /* U+10CDC */
  0x0020,   /* U+10CDD */
  0x0020,   /* U+10CDE */
  0x0020,   /* U+10CDF */
  0x0020,   /* U+10CE0 */
  0x0020,   /* U+10CE1 */
  0x0020,   /* U+10CE2 */
  0x0020,   /* U+10CE3 */
  0x0020,   /* U+10CE4 */
  0x0020,   /* U+10CE5 */
  0x0020,   /* U+10CE6 */
  0x0020,   /* U+10CE7 */
  0x0020,   /* U+10CE8 */
  0x0020,   /* U+10CE9 */
  0x0020,   /* U+10CEA */
  0x0020,   /* U+10CEB */
  0x0020,   /* U+10CEC */
  0x0020,   /* U+10CED */
  0x0020,   /* U+10CEE */
  0x0020,   /* U+10CEF */
  0x0020,   /* U+10CF0 */
  0x0020,   /* U+10CF1 */
  0x0020,   /* U+10CF2 */
  0x0020,   /* U+10CF3 */
  0x0020,   /* U+10CF4 */
  0x0020,   /* U+10CF5 */
  0x0020,   /* U+10CF6 */
  0x0020,   /* U+10CF7 */
  0x0020,   /* U+10CF8 */
  0x0020,   /* U+10CF9 */
  0x0020,   /* U+10CFA */
  0x0020,   /* U+10CFB */
  0x0020,   /* U+10CFC */
  0x0020,   /* U+10CFD */
  0x0020,   /* U+10CFE */
  0x0020,   /* U+10CFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10C00 */
  0x0004,   /* U+10C01 */
  0x0002,   /* U+10C02 */
  0x0002,   /* U+10C03 */
  0x0004,   /* U+10C04 */
  0x0002,   /* U+10C05 */
  0x0002,   /* U+10C06 */
  0x0002,   /* U+10C07 */
  0x0004,   /* U+10C08 */
  0x0002,   /* U+10C09 */
  0x0004,   /* U+10C0A */
  0x0002,   /* U+10C0B */
  0x0004,   /* U+10C0C */
  0x0002,   /* U+10C0D */
  0x0004,   /* U+10C0E */
  0x0002,   /* U+10C0F */
  0x0004,   /* U+10C10 */
  0x0002,   /* U+10C11 */
  0x0004,   /* U+10C12 */
  0x0002,   /* U+10C13 */
  0x0002,   /* U+10C14 */
  0x0004,   /* U+10C15 */
  0x0002,   /* U+10C16 */
  0x0004,   /* U+10C17 */
  0x0002,   /* U+10C18 */
  0x0004,   /* U+10C19 */
  0x0002,   /* U+10C1A */
  0x0004,   /* U+10C1B */
  0x0002,   /* U+10C1C */
  0x0004,   /* U+10C1D */
  0x0002,   /* U+10C1E */
  0x0004,   /* U+10C1F */
  0x0002,   /* U+10C20 */
  0x0002,   /* U+10C21 */
  0x0002,   /* U+10C22 */
  0x0002,   /* U+10C23 */
  0x0002,   /* U+10C24 */
  0x0004,   /* U+10C25 */
  0x0002,   /* U+10C26 */
  0x0004,   /* U+10C27 */
  0x0002,   /* U+10C28 */
  0x0004,   /* U+10C29 */
  0x0002,   /* U+10C2A */
  0x0004,   /* U+10C2B */
  0x0002,   /* U+10C2C */
  0x0002,   /* U+10C2D */
  0x0004,   /* U+10C2E */
  0x0002,   /* U+10C2F */
  0x0002,   /* U+10C30 */
  0x0002,   /* U+10C31 */
  0x0002,   /* U+10C32 */
  0x0004,   /* U+10C33 */
  0x0002,   /* U+10C34 */
  0x0004,   /* U+10C35 */
  0x0002,   /* U+10C36 */
  0x0004,   /* U+10C37 */
  0x0002,   /* U+10C38 */
  0x0004,   /* U+10C39 */
  0x0002,   /* U+10C3A */
  0x0004,   /* U+10C3B */
  0x0002,   /* U+10C3C */
  0x0002,   /* U+10C3D */
  0x0002,   /* U+10C3E */
  0x0002,   /* U+10C3F */
  0x0004,   /* U+10C40 */
  0x0002,   /* U+10C41 */
  0x0004,   /* U+10C42 */
  0x0002,   /* U+10C43 */
  0x0004,   /* U+10C44 */
  0x0002,   /* U+10C45 */
  0x0004,   /* U+10C46 */
  0x0002,   /* U+10C47 */
  0x0002,   /* U+10C48 */
  0x0002,   /* U+10C49 */
  0x0002,   /* U+10C4A */
  0x0002,   /* U+10C4B */
  0x0002,   /* U+10C4C */
  0x0002,   /* U+10C4D */
  0x0002,   /* U+10C4E */
  0x0002,   /* U+10C4F */
  0x0002,   /* U+10C50 */
  0x0002,   /* U+10C51 */
  0x0002,   /* U+10C52 */
  0x0002,   /* U+10C53 */
  0x0002,   /* U+10C54 */
  0x0002,   /* U+10C55 */
  0x0002,   /* U+10C56 */
  0x0002,   /* U+10C57 */
  0x0002,   /* U+10C58 */
  0x0002,   /* U+10C59 */
  0x0002,   /* U+10C5A */
  0x0002,   /* U+10C5B */
  0x0002,   /* U+10C5C */
  0x0002,   /* U+10C5D */
  0x0002,   /* U+10C5E */
  0x0002,   /* U+10C5F */
  0x0002,   /* U+10C60 */
  0x0002,   /* U+10C61 */
  0x0002,   /* U+10C62 */
  0x0002,   /* U+10C63 */
  0x0002,   /* U+10C64 */
  0x0002,   /* U+10C65 */
  0x0002,   /* U+10C66 */
  0x0002,   /* U+10C67 */
  0x0002,   /* U+10C68 */
  0x0002,   /* U+10C69 */
  0x0002,   /* U+10C6A */
  0x0002,   /* U+10C6B */
  0x0002,   /* U+10C6C */
  0x0002,   /* U+10C6D */
  0x0002,   /* U+10C6E */
  0x0002,   /* U+10C6F */
  0x0002,   /* U+10C70 */
  0x0002,   /* U+10C71 */
  0x0002,   /* U+10C72 */
  0x0002,   /* U+10C73 */
  0x0002,   /* U+10C74 */
  0x0002,   /* U+10C75 */
  0x0002,   /* U+10C76 */
  0x0002,   /* U+10C77 */
  0x0002,   /* U+10C78 */
  0x0002,   /* U+10C79 */
  0x0002,   /* U+10C7A */
  0x0002,   /* U+10C7B */
  0x0002,   /* U+10C7C */
  0x0002,   /* U+10C7D */
  0x0002,   /* U+10C7E */
  0x0002,   /* U+10C7F */
  0x0008,   /* U+10C80 */
  0x000A,   /* U+10C81 */
  0x0008,   /* U+10C82 */
  0x0008,   /* U+10C83 */
  0x0008,   /* U+10C84 */
  0x0008,   /* U+10C85 */
  0x0008,   /* U+10C86 */
  0x0008,   /* U+10C87 */
  0x0008,   /* U+10C88 */
  0x0008,   /* U+10C89 */
  0x000A,   /* U+10C8A */
  0x000A,   /* U+10C8B */
  0x0008,   /* U+10C8C */
  0x0008,   /* U+10C8D */
  0x0008,   /* U+10C8E */
  0x0008,   /* U+10C8F */
  0x0008,   /* U+10C90 */
  0x000A,   /* U+10C91 */
  0x0008,   /* U+10C92 */
  0x0008,   /* U+10C93 */
  0x0008,   /* U+10C94 */
  0x0008,   /* U+10C95 */
  0x0008,   /* U+10C96 */
  0x0008,   /* U+10C97 */
  0x0008,   /* U+10C98 */
  0x0008,   /* U+10C99 */
  0x0008,   /* U+10C9A */
  0x0008,   /* U+10C9B */
  0x000A,   /* U+10C9C */
  0x0008,   /* U+10C9D */
  0x000A,   /* U+10C9E */
  0x000A,   /* U+10C9F */
  0x0008,   /* U+10CA0 */
  0x0008,   /* U+10CA1 */
  0x0008,   /* U+10CA2 */
  0x000A,   /* U+10CA3 */
  0x0008,   /* U+10CA4 */
  0x0008,   /* U+10CA5 */
  0x0008,   /* U+10CA6 */
  0x0008,   /* U+10CA7 */
  0x0008,   /* U+10CA8 */
  0x0008,   /* U+10CA9 */
  0x0008,   /* U+10CAA */
  0x000A,   /* U+10CAB */
  0x0008,   /* U+10CAC */
  0x000A,   /* U+10CAD */
  0x0008,   /* U+10CAE */
  0x0008,   /* U+10CAF */
  0x0008,   /* U+10CB0 */
  0x0008,   /* U+10CB1 */
  0x0008,   /* U+10CB2 */
  0x0002,   /* U+10CB3 */
  0x0002,   /* U+10CB4 */
  0x0002,   /* U+10CB5 */
  0x0002,   /* U+10CB6 */
  0x0002,   /* U+10CB7 */
  0x0002,   /* U+10CB8 */
  0x0002,   /* U+10CB9 */
  0x0002,   /* U+10CBA */
  0x0002,   /* U+10CBB */
  0x0002,   /* U+10CBC */
  0x0002,   /* U+10CBD */
  0x0002,   /* U+10CBE */
  0x0002,   /* U+10CBF */
  0x0002,   /* U+10CC0 */
  0x0004,   /* U+10CC1 */
  0x0002,   /* U+10CC2 */
  0x0002,   /* U+10CC3 */
  0x0002,   /* U+10CC4 */
  0x0002,   /* U+10CC5 */
  0x0002,   /* U+10CC6 */
  0x0002,   /* U+10CC7 */
  0x0002,   /* U+10CC8 */
  0x0002,   /* U+10CC9 */
  0x0004,   /* U+10CCA */
  0x0004,   /* U+10CCB */
  0x0002,   /* U+10CCC */
  0x0002,   /* U+10CCD */
  0x0002,   /* U+10CCE */
  0x0002,   /* U+10CCF */
  0x0002,   /* U+10CD0 */
  0x0004,   /* U+10CD1 */
  0x0002,   /* U+10CD2 */
  0x0002,   /* U+10CD3 */
  0x0002,   /* U+10CD4 */
  0x0002,   /* U+10CD5 */
  0x0002,   /* U+10CD6 */
  0x0002,   /* U+10CD7 */
  0x0002,   /* U+10CD8 */
  0x0002,   /* U+10CD9 */
  0x0002,   /* U+10CDA */
  0x0002,   /* U+10CDB */
  0x0004,   /* U+10CDC */
  0x0002,   /* U+10CDD */
  0x0004,   /* U+10CDE */
  0x0004,   /* U+10CDF */
  0x0002,   /* U+10CE0 */
  0x0002,   /* U+10CE1 */
  0x0002,   /* U+10CE2 */
  0x0004,   /* U+10CE3 */
  0x0002,   /* U+10CE4 */
  0x0002,   /* U+10CE5 */
  0x0002,   /* U+10CE6 */
  0x0002,   /* U+10CE7 */
  0x0002,   /* U+10CE8 */
  0x0002,   /* U+10CE9 */
  0x0002,   /* U+10CEA */
  0x0004,   /* U+10CEB */
  0x0002,   /* U+10CEC */
  0x0004,   /* U+10CED */
  0x0002,   /* U+10CEE */
  0x0002,   /* U+10CEF */
  0x0002,   /* U+10CF0 */
  0x0002,   /* U+10CF1 */
  0x0002,   /* U+10CF2 */
  0x0002,   /* U+10CF3 */
  0x0002,   /* U+10CF4 */
  0x0002,   /* U+10CF5 */
  0x0002,   /* U+10CF6 */
  0x0002,   /* U+10CF7 */
  0x0002,   /* U+10CF8 */
  0x0002,   /* U+10CF9 */
  0x0002,   /* U+10CFA */
  0x0002,   /* U+10CFB */
  0x0002,   /* U+10CFC */
  0x0002,   /* U+10CFD */
  0x0002,   /* U+10CFE */
  0x0002,   /* U+10CFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+10C00 */
  0x0000,   /* U+10C01 */
  0x0000,   /* U+10C02 */
  0x0000,   /* U+10C03 */
  0x0000,   /* U+10C04 */
  0x0000,   /* U+10C05 */
  0x0000,   /* U+10C06 */
  0x0000,   /* U+10C07 */
  0x0000,   /* U+10C08 */
  0x0000,   /* U+10C09 */
  0x0000,   /* U+10C0A */
  0x0000,   /* U+10C0B */
  0x0000,   /* U+10C0C */
  0x0000,   /* U+10C0D */
  0x0000,   /* U+10C0E */
  0x0000,   /* U+10C0F */
  0x0000,   /* U+10C10 */
  0x0000,   /* U+10C11 */
  0x0000,   /* U+10C12 */
  0x0000,   /* U+10C13 */
  0x0000,   /* U+10C14 */
  0x0000,   /* U+10C15 */
  0x0000,   /* U+10C16 */
  0x0000,   /* U+10C17 */
  0x0000,   /* U+10C18 */
  0x0000,   /* U+10C19 */
  0x0000,   /* U+10C1A */
  0x0000,   /* U+10C1B */
  0x0000,   /* U+10C1C */
  0x0000,   /* U+10C1D */
  0x0000,   /* U+10C1E */
  0x0000,   /* U+10C1F */
  0x0000,   /* U+10C20 */
  0x0000,   /* U+10C21 */
  0x0000,   /* U+10C22 */
  0x0000,   /* U+10C23 */
  0x0000,   /* U+10C24 */
  0x0000,   /* U+10C25 */
  0x0000,   /* U+10C26 */
  0x0000,   /* U+10C27 */
  0x0000,   /* U+10C28 */
  0x0000,   /* U+10C29 */
  0x0000,   /* U+10C2A */
  0x0000,   /* U+10C2B */
  0x0000,   /* U+10C2C */
  0x0000,   /* U+10C2D */
  0x0000,   /* U+10C2E */
  0x0000,   /* U+10C2F */
  0x0000,   /* U+10C30 */
  0x0000,   /* U+10C31 */
  0x0000,   /* U+10C32 */
  0x0000,   /* U+10C33 */
  0x0000,   /* U+10C34 */
  0x0000,   /* U+10C35 */
  0x0000,   /* U+10C36 */
  0x0000,   /* U+10C37 */
  0x0000,   /* U+10C38 */
  0x0000,   /* U+10C39 */
  0x0000,   /* U+10C3A */
  0x0000,   /* U+10C3B */
  0x0000,   /* U+10C3C */
  0x0000,   /* U+10C3D */
  0x0000,   /* U+10C3E */
  0x0000,   /* U+10C3F */
  0x0000,   /* U+10C40 */
  0x0000,   /* U+10C41 */
  0x0000,   /* U+10C42 */
  0x0000,   /* U+10C43 */
  0x0000,   /* U+10C44 */
  0x0000,   /* U+10C45 */
  0x0000,   /* U+10C46 */
  0x0000,   /* U+10C47 */
  0x0000,   /* U+10C48 */
  0x8C49,   /* U+10C49 */
  0x8C4A,   /* U+10C4A */
  0x8C4B,   /* U+10C4B */
  0x8C4C,   /* U+10C4C */
  0x8C4D,   /* U+10C4D */
  0x8C4E,   /* U+10C4E */
  0x8C4F,   /* U+10C4F */
  0x8C50,   /* U+10C50 */
  0x8C51,   /* U+10C51 */
  0x8C52,   /* U+10C52 */
  0x8C53,   /* U+10C53 */
  0x8C54,   /* U+10C54 */
  0x8C55,   /* U+10C55 */
  0x8C56,   /* U+10C56 */
  0x8C57,   /* U+10C57 */
  0x8C58,   /* U+10C58 */
  0x8C59,   /* U+10C59 */
  0x8C5A,   /* U+10C5A */
  0x8C5B,   /* U+10C5B */
  0x8C5C,   /* U+10C5C */
  0x8C5D,   /* U+10C5D */
  0x8C5E,   /* U+10C5E */
  0x8C5F,   /* U+10C5F */
  0x8C60,   /* U+10C60 */
  0x8C61,   /* U+10C61 */
  0x8C62,   /* U+10C62 */
  0x8C63,   /* U+10C63 */
  0x8C64,   /* U+10C64 */
  0x8C65,   /* U+10C65 */
  0x8C66,   /* U+10C66 */
  0x8C67,   /* U+10C67 */
  0x8C68,   /* U+10C68 */
  0x8C69,   /* U+10C69 */
  0x8C6A,   /* U+10C6A */
  0x8C6B,   /* U+10C6B */
  0x8C6C,   /* U+10C6C */
  0x8C6D,   /* U+10C6D */
  0x8C6E,   /* U+10C6E */
  0x8C6F,   /* U+10C6F */
  0x8C70,   /* U+10C70 */
  0x8C71,   /* U+10C71 */
  0x8C72,   /* U+10C72 */
  0x8C73,   /* U+10C73 */
  0x8C74,   /* U+10C74 */
  0x8C75,   /* U+10C75 */
  0x8C76,   /* U+10C76 */
  0x8C77,   /* U+10C77 */
  0x8C78,   /* U+10C78 */
  0x8C79,   /* U+10C79 */
  0x8C7A,   /* U+10C7A */
  0x8C7B,   /* U+10C7B */
  0x8C7C,   /* U+10C7C */
  0x8C7D,   /* U+10C7D */
  0x8C7E,   /* U+10C7E */
  0x8C7F,   /* U+10C7F */
  0x0000,   /* U+10C80 */
  0x0000,   /* U+10C81 */
  0x0000,   /* U+10C82 */
  0x0000,   /* U+10C83 */
  0x0000,   /* U+10C84 */
  0x0000,   /* U+10C85 */
  0x0000,   /* U+10C86 */
  0x0000,   /* U+10C87 */
  0x0000,   /* U+10C88 */
  0x0000,   /* U+10C89 */
  0x0000,   /* U+10C8A */
  0x0000,   /* U+10C8B */
  0x0000,   /* U+10C8C */
  0x0000,   /* U+10C8D */
  0x0000,   /* U+10C8E */
  0x0000,   /* U+10C8F */
  0x0000,   /* U+10C90 */
  0x0000,   /* U+10C91 */
  0x0000,   /* U+10C92 */
  0x0000,   /* U+10C93 */
  0x0000,   /* U+10C94 */
  0x0000,   /* U+10C95 */
  0x0000,   /* U+10C96 */
  0x0000,   /* U+10C97 */
  0x0000,   /* U+10C98 */
  0x0000,   /* U+10C99 */
  0x0000,   /* U+10C9A */
  0x0000,   /* U+10C9B */
  0x0000,   /* U+10C9C */
  0x0000,   /* U+10C9D */
  0x0000,   /* U+10C9E */
  0x0000,   /* U+10C9F */
  0x0000,   /* U+10CA0 */
  0x0000,   /* U+10CA1 */
  0x0000,   /* U+10CA2 */
  0x0000,   /* U+10CA3 */
  0x0000,   /* U+10CA4 */
  0x0000,   /* U+10CA5 */
  0x0000,   /* U+10CA6 */
  0x0000,   /* U+10CA7 */
  0x0000,   /* U+10CA8 */
  0x0000,   /* U+10CA9 */
  0x0000,   /* U+10CAA */
  0x0000,   /* U+10CAB */
  0x0000,   /* U+10CAC */
  0x0000,   /* U+10CAD */
  0x0000,   /* U+10CAE */
  0x0000,   /* U+10CAF */
  0x0000,   /* U+10CB0 */
  0x0000,   /* U+10CB1 */
  0x0000,   /* U+10CB2 */
  0x8CB3,   /* U+10CB3 */
  0x8CB4,   /* U+10CB4 */
  0x8CB5,   /* U+10CB5 */
  0x8CB6,   /* U+10CB6 */
  0x8CB7,   /* U+10CB7 */
  0x8CB8,   /* U+10CB8 */
  0x8CB9,   /* U+10CB9 */
  0x8CBA,   /* U+10CBA */
  0x8CBB,   /* U+10CBB */
  0x8CBC,   /* U+10CBC */
  0x8CBD,   /* U+10CBD */
  0x8CBE,   /* U+10CBE */
  0x8CBF,   /* U+10CBF */
  0x0000,   /* U+10CC0 */
  0x0000,   /* U+10CC1 */
  0x0000,   /* U+10CC2 */
  0x0000,   /* U+10CC3 */
  0x0000,   /* U+10CC4 */
  0x0000,   /* U+10CC5 */
  0x0000,   /* U+10CC6 */
  0x0000,   /* U+10CC7 */
  0x0000,   /* U+10CC8 */
  0x0000,   /* U+10CC9 */
  0x0000,   /* U+10CCA */
  0x0000,   /* U+10CCB */
  0x0000,   /* U+10CCC */
  0x0000,   /* U+10CCD */
  0x0000,   /* U+10CCE */
  0x0000,   /* U+10CCF */
  0x0000,   /* U+10CD0 */
  0x0000,   /* U+10CD1 */
  0x0000,   /* U+10CD2 */
  0x0000,   /* U+10CD3 */
  0x0000,   /* U+10CD4 */
  0x0000,   /* U+10CD5 */
  0x0000,   /* U+10CD6 */
  0x0000,   /* U+10CD7 */
  0x0000,   /* U+10CD8 */
  0x0000,   /* U+10CD9 */
  0x0000,   /* U+10CDA */
  0x0000,   /* U+10CDB */
  0x0000,   /* U+10CDC */
  0x0000,   /* U+10CDD */
  0x0000,   /* U+10CDE */
  0x0000,   /* U+10CDF */
  0x0000,   /* U+10CE0 */
  0x0000,   /* U+10CE1 */
  0x0000,   /* U+10CE2 */
  0x0000,   /* U+10CE3 */
  0x0000,   /* U+10CE4 */
  0x0000,   /* U+10CE5 */
  0x0000,   /* U+10CE6 */
  0x0000,   /* U+10CE7 */
  0x0000,   /* U+10CE8 */
  0x0000,   /* U+10CE9 */
  0x0000,   /* U+10CEA */
  0x0000,   /* U+10CEB */
  0x0000,   /* U+10CEC */
  0x0000,   /* U+10CED */
  0x0000,   /* U+10CEE */
  0x0000,   /* U+10CEF */
  0x0000,   /* U+10CF0 */
  0x0000,   /* U+10CF1 */
  0x0000,   /* U+10CF2 */
  0x8CF3,   /* U+10CF3 */
  0x8CF4,   /* U+10CF4 */
  0x8CF5,   /* U+10CF5 */
  0x8CF6,   /* U+10CF6 */
  0x8CF7,   /* U+10CF7 */
  0x8CF8,   /* U+10CF8 */
  0x8CF9,   /* U+10CF9 */
  0x0000,   /* U+10CFA */
  0x0000,   /* U+10CFB */
  0x0000,   /* U+10CFC */
  0x0000,   /* U+10CFD */
  0x0000,   /* U+10CFE */
  0x0000,   /* U+10CFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10C00 */
  0x0110,   /* U+10C01 */
  0x0000,   /* U+10C02 */
  0x0000,   /* U+10C03 */
  0x0110,   /* U+10C04 */
  0x0000,   /* U+10C05 */
  0x0000,   /* U+10C06 */
  0x0000,   /* U+10C07 */
  0x0110,   /* U+10C08 */
  0x0000,   /* U+10C09 */
  0x0110,   /* U+10C0A */
  0x0000,   /* U+10C0B */
  0x0110,   /* U+10C0C */
  0x0000,   /* U+10C0D */
  0x0110,   /* U+10C0E */
  0x0000,   /* U+10C0F */
  0x0110,   /* U+10C10 */
  0x0000,   /* U+10C11 */
  0x0110,   /* U+10C12 */
  0x0000,   /* U+10C13 */
  0x0000,   /* U+10C14 */
  0x0110,   /* U+10C15 */
  0x0000,   /* U+10C16 */
  0x0110,   /* U+10C17 */
  0x0000,   /* U+10C18 */
  0x0110,   /* U+10C19 */
  0x0000,   /* U+10C1A */
  0x0110,   /* U+10C1B */
  0x0000,   /* U+10C1C */
  0x0110,   /* U+10C1D */
  0x0000,   /* U+10C1E */
  0x0110,   /* U+10C1F */
  0x0000,   /* U+10C20 */
  0x0000,   /* U+10C21 */
  0x0000,   /* U+10C22 */
  0x0000,   /* U+10C23 */
  0x0000,   /* U+10C24 */
  0x0110,   /* U+10C25 */
  0x0000,   /* U+10C26 */
  0x0110,   /* U+10C27 */
  0x0000,   /* U+10C28 */
  0x0110,   /* U+10C29 */
  0x0000,   /* U+10C2A */
  0x0110,   /* U+10C2B */
  0x0000,   /* U+10C2C */
  0x0000,   /* U+10C2D */
  0x0110,   /* U+10C2E */
  0x0000,   /* U+10C2F */
  0x0000,   /* U+10C30 */
  0x0000,   /* U+10C31 */
  0x0000,   /* U+10C32 */
  0x0110,   /* U+10C33 */
  0x0000,   /* U+10C34 */
  0x0110,   /* U+10C35 */
  0x0000,   /* U+10C36 */
  0x0110,   /* U+10C37 */
  0x0000,   /* U+10C38 */
  0x0110,   /* U+10C39 */
  0x0000,   /* U+10C3A */
  0x0110,   /* U+10C3B */
  0x0000,   /* U+10C3C */
  0x0000,   /* U+10C3D */
  0x0000,   /* U+10C3E */
  0x0000,   /* U+10C3F */
  0x0110,   /* U+10C40 */
  0x0000,   /* U+10C41 */
  0x0110,   /* U+10C42 */
  0x0000,   /* U+10C43 */
  0x0110,   /* U+10C44 */
  0x0000,   /* U+10C45 */
  0x0110,   /* U+10C46 */
  0x0000,   /* U+10C47 */
  0x0000,   /* U+10C48 */
  0x0000,   /* U+10C49 */
  0x0000,   /* U+10C4A */
  0x0000,   /* U+10C4B */
  0x0000,   /* U+10C4C */
  0x0000,   /* U+10C4D */
  0x0000,   /* U+10C4E */
  0x0000,   /* U+10C4F */
  0x0000,   /* U+10C50 */
  0x0000,   /* U+10C51 */
  0x0000,   /* U+10C52 */
  0x0000,   /* U+10C53 */
  0x0000,   /* U+10C54 */
  0x0000,   /* U+10C55 */
  0x0000,   /* U+10C56 */
  0x0000,   /* U+10C57 */
  0x0000,   /* U+10C58 */
  0x0000,   /* U+10C59 */
  0x0000,   /* U+10C5A */
  0x0000,   /* U+10C5B */
  0x0000,   /* U+10C5C */
  0x0000,   /* U+10C5D */
  0x0000,   /* U+10C5E */
  0x0000,   /* U+10C5F */
  0x0000,   /* U+10C60 */
  0x0000,   /* U+10C61 */
  0x0000,   /* U+10C62 */
  0x0000,   /* U+10C63 */
  0x0000,   /* U+10C64 */
  0x0000,   /* U+10C65 */
  0x0000,   /* U+10C66 */
  0x0000,   /* U+10C67 */
  0x0000,   /* U+10C68 */
  0x0000,   /* U+10C69 */
  0x0000,   /* U+10C6A */
  0x0000,   /* U+10C6B */
  0x0000,   /* U+10C6C */
  0x0000,   /* U+10C6D */
  0x0000,   /* U+10C6E */
  0x0000,   /* U+10C6F */
  0x0000,   /* U+10C70 */
  0x0000,   /* U+10C71 */
  0x0000,   /* U+10C72 */
  0x0000,   /* U+10C73 */
  0x0000,   /* U+10C74 */
  0x0000,   /* U+10C75 */
  0x0000,   /* U+10C76 */
  0x0000,   /* U+10C77 */
  0x0000,   /* U+10C78 */
  0x0000,   /* U+10C79 */
  0x0000,   /* U+10C7A */
  0x0000,   /* U+10C7B */
  0x0000,   /* U+10C7C */
  0x0000,   /* U+10C7D */
  0x0000,   /* U+10C7E */
  0x0000,   /* U+10C7F */
  0x0000,   /* U+10C80 */
  0x0110,   /* U+10C81 */
  0x0000,   /* U+10C82 */
  0x0000,   /* U+10C83 */
  0x0000,   /* U+10C84 */
  0x0000,   /* U+10C85 */
  0x0000,   /* U+10C86 */
  0x0000,   /* U+10C87 */
  0x0000,   /* U+10C88 */
  0x0000,   /* U+10C89 */
  0x0110,   /* U+10C8A */
  0x0111,   /* U+10C8B */
  0x0000,   /* U+10C8C */
  0x0000,   /* U+10C8D */
  0x0000,   /* U+10C8E */
  0x0000,   /* U+10C8F */
  0x0000,   /* U+10C90 */
  0x0110,   /* U+10C91 */
  0x0000,   /* U+10C92 */
  0x0000,   /* U+10C93 */
  0x0000,   /* U+10C94 */
  0x0000,   /* U+10C95 */
  0x0000,   /* U+10C96 */
  0x0000,   /* U+10C97 */
  0x0000,   /* U+10C98 */
  0x0000,   /* U+10C99 */
  0x0000,   /* U+10C9A */
  0x0000,   /* U+10C9B */
  0x0110,   /* U+10C9C */
  0x0000,   /* U+10C9D */
  0x0110,   /* U+10C9E */
  0x0111,   /* U+10C9F */
  0x0000,   /* U+10CA0 */
  0x0000,   /* U+10CA1 */
  0x0000,   /* U+10CA2 */
  0x0110,   /* U+10CA3 */
  0x0000,   /* U+10CA4 */
  0x0000,   /* U+10CA5 */
  0x0000,   /* U+10CA6 */
  0x0000,   /* U+10CA7 */
  0x0000,   /* U+10CA8 */
  0x0000,   /* U+10CA9 */
  0x0000,   /* U+10CAA */
  0x0110,   /* U+10CAB */
  0x0000,   /* U+10CAC */
  0x0110,   /* U+10CAD */
  0x0000,   /* U+10CAE */
  0x0000,   /* U+10CAF */
  0x0000,   /* U+10CB0 */
  0x0000,   /* U+10CB1 */
  0x0000,   /* U+10CB2 */
  0x0000,   /* U+10CB3 */
  0x0000,   /* U+10CB4 */
  0x0000,   /* U+10CB5 */
  0x0000,   /* U+10CB6 */
  0x0000,   /* U+10CB7 */
  0x0000,   /* U+10CB8 */
  0x0000,   /* U+10CB9 */
  0x0000,   /* U+10CBA */
  0x0000,   /* U+10CBB */
  0x0000,   /* U+10CBC */
  0x0000,   /* U+10CBD */
  0x0000,   /* U+10CBE */
  0x0000,   /* U+10CBF */
  0x0000,   /* U+10CC0 */
  0x0110,   /* U+10CC1 */
  0x0000,   /* U+10CC2 */
  0x0000,   /* U+10CC3 */
  0x0000,   /* U+10CC4 */
  0x0000,   /* U+10CC5 */
  0x0000,   /* U+10CC6 */
  0x0000,   /* U+10CC7 */
  0x0000,   /* U+10CC8 */
  0x0000,   /* U+10CC9 */
  0x0110,   /* U+10CCA */
  0x0111,   /* U+10CCB */
  0x0000,   /* U+10CCC */
  0x0000,   /* U+10CCD */
  0x0000,   /* U+10CCE */
  0x0000,   /* U+10CCF */
  0x0000,   /* U+10CD0 */
  0x0110,   /* U+10CD1 */
  0x0000,   /* U+10CD2 */
  0x0000,   /* U+10CD3 */
  0x0000,   /* U+10CD4 */
  0x0000,   /* U+10CD5 */
  0x0000,   /* U+10CD6 */
  0x0000,   /* U+10CD7 */
  0x0000,   /* U+10CD8 */
  0x0000,   /* U+10CD9 */
  0x0000,   /* U+10CDA */
  0x0000,   /* U+10CDB */
  0x0110,   /* U+10CDC */
  0x0000,   /* U+10CDD */
  0x0110,   /* U+10CDE */
  0x0111,   /* U+10CDF */
  0x0000,   /* U+10CE0 */
  0x0000,   /* U+10CE1 */
  0x0000,   /* U+10CE2 */
  0x0110,   /* U+10CE3 */
  0x0000,   /* U+10CE4 */
  0x0000,   /* U+10CE5 */
  0x0000,   /* U+10CE6 */
  0x0000,   /* U+10CE7 */
  0x0000,   /* U+10CE8 */
  0x0000,   /* U+10CE9 */
  0x0000,   /* U+10CEA */
  0x0110,   /* U+10CEB */
  0x0000,   /* U+10CEC */
  0x0110,   /* U+10CED */
  0x0000,   /* U+10CEE */
  0x0000,   /* U+10CEF */
  0x0000,   /* U+10CF0 */
  0x0000,   /* U+10CF1 */
  0x0000,   /* U+10CF2 */
  0x0000,   /* U+10CF3 */
  0x0000,   /* U+10CF4 */
  0x0000,   /* U+10CF5 */
  0x0000,   /* U+10CF6 */
  0x0000,   /* U+10CF7 */
  0x0000,   /* U+10CF8 */
  0x0000,   /* U+10CF9 */
  0x0000,   /* U+10CFA */
  0x0000,   /* U+10CFB */
  0x0000,   /* U+10CFC */
  0x0000,   /* U+10CFD */
  0x0000,   /* U+10CFE */
  0x0000,   /* U+10CFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10C00 */
  0x0004,   /* U+10C01 */
  0x0000,   /* U+10C02 */
  0x0000,   /* U+10C03 */
  0x0004,   /* U+10C04 */
  0x0000,   /* U+10C05 */
  0x0000,   /* U+10C06 */
  0x0000,   /* U+10C07 */
  0x0004,   /* U+10C08 */
  0x0000,   /* U+10C09 */
  0x0004,   /* U+10C0A */
  0x0000,   /* U+10C0B */
  0x0004,   /* U+10C0C */
  0x0000,   /* U+10C0D */
  0x0004,   /* U+10C0E */
  0x0000,   /* U+10C0F */
  0x0004,   /* U+10C10 */
  0x0000,   /* U+10C11 */
  0x0004,   /* U+10C12 */
  0x0000,   /* U+10C13 */
  0x0000,   /* U+10C14 */
  0x0004,   /* U+10C15 */
  0x0000,   /* U+10C16 */
  0x0004,   /* U+10C17 */
  0x0000,   /* U+10C18 */
  0x0004,   /* U+10C19 */
  0x0000,   /* U+10C1A */
  0x0004,   /* U+10C1B */
  0x0000,   /* U+10C1C */
  0x0004,   /* U+10C1D */
  0x0000,   /* U+10C1E */
  0x0004,   /* U+10C1F */
  0x0000,   /* U+10C20 */
  0x0000,   /* U+10C21 */
  0x0000,   /* U+10C22 */
  0x0000,   /* U+10C23 */
  0x0000,   /* U+10C24 */
  0x0004,   /* U+10C25 */
  0x0000,   /* U+10C26 */
  0x0004,   /* U+10C27 */
  0x0000,   /* U+10C28 */
  0x0004,   /* U+10C29 */
  0x0000,   /* U+10C2A */
  0x0004,   /* U+10C2B */
  0x0000,   /* U+10C2C */
  0x0000,   /* U+10C2D */
  0x0004,   /* U+10C2E */
  0x0000,   /* U+10C2F */
  0x0000,   /* U+10C30 */
  0x0000,   /* U+10C31 */
  0x0000,   /* U+10C32 */
  0x0004,   /* U+10C33 */
  0x0000,   /* U+10C34 */
  0x0004,   /* U+10C35 */
  0x0000,   /* U+10C36 */
  0x0004,   /* U+10C37 */
  0x0000,   /* U+10C38 */
  0x0004,   /* U+10C39 */
  0x0000,   /* U+10C3A */
  0x0004,   /* U+10C3B */
  0x0000,   /* U+10C3C */
  0x0000,   /* U+10C3D */
  0x0000,   /* U+10C3E */
  0x0000,   /* U+10C3F */
  0x0004,   /* U+10C40 */
  0x0000,   /* U+10C41 */
  0x0004,   /* U+10C42 */
  0x0000,   /* U+10C43 */
  0x0004,   /* U+10C44 */
  0x0000,   /* U+10C45 */
  0x0004,   /* U+10C46 */
  0x0000,   /* U+10C47 */
  0x0000,   /* U+10C48 */
  0x0000,   /* U+10C49 */
  0x0000,   /* U+10C4A */
  0x0000,   /* U+10C4B */
  0x0000,   /* U+10C4C */
  0x0000,   /* U+10C4D */
  0x0000,   /* U+10C4E */
  0x0000,   /* U+10C4F */
  0x0000,   /* U+10C50 */
  0x0000,   /* U+10C51 */
  0x0000,   /* U+10C52 */
  0x0000,   /* U+10C53 */
  0x0000,   /* U+10C54 */
  0x0000,   /* U+10C55 */
  0x0000,   /* U+10C56 */
  0x0000,   /* U+10C57 */
  0x0000,   /* U+10C58 */
  0x0000,   /* U+10C59 */
  0x0000,   /* U+10C5A */
  0x0000,   /* U+10C5B */
  0x0000,   /* U+10C5C */
  0x0000,   /* U+10C5D */
  0x0000,   /* U+10C5E */
  0x0000,   /* U+10C5F */
  0x0000,   /* U+10C60 */
  0x0000,   /* U+10C61 */
  0x0000,   /* U+10C62 */
  0x0000,   /* U+10C63 */
  0x0000,   /* U+10C64 */
  0x0000,   /* U+10C65 */
  0x0000,   /* U+10C66 */
  0x0000,   /* U+10C67 */
  0x0000,   /* U+10C68 */
  0x0000,   /* U+10C69 */
  0x0000,   /* U+10C6A */
  0x0000,   /* U+10C6B */
  0x0000,   /* U+10C6C */
  0x0000,   /* U+10C6D */
  0x0000,   /* U+10C6E */
  0x0000,   /* U+10C6F */
  0x0000,   /* U+10C70 */
  0x0000,   /* U+10C71 */
  0x0000,   /* U+10C72 */
  0x0000,   /* U+10C73 */
  0x0000,   /* U+10C74 */
  0x0000,   /* U+10C75 */
  0x0000,   /* U+10C76 */
  0x0000,   /* U+10C77 */
  0x0000,   /* U+10C78 */
  0x0000,   /* U+10C79 */
  0x0000,   /* U+10C7A */
  0x0000,   /* U+10C7B */
  0x0000,   /* U+10C7C */
  0x0000,   /* U+10C7D */
  0x0000,   /* U+10C7E */
  0x0000,   /* U+10C7F */
  0x0000,   /* U+10C80 */
  0x0004,   /* U+10C81 */
  0x0000,   /* U+10C82 */
  0x0000,   /* U+10C83 */
  0x0000,   /* U+10C84 */
  0x0000,   /* U+10C85 */
  0x0000,   /* U+10C86 */
  0x0000,   /* U+10C87 */
  0x0000,   /* U+10C88 */
  0x0000,   /* U+10C89 */
  0x0004,   /* U+10C8A */
  0x0004,   /* U+10C8B */
  0x0000,   /* U+10C8C */
  0x0000,   /* U+10C8D */
  0x0000,   /* U+10C8E */
  0x0000,   /* U+10C8F */
  0x0000,   /* U+10C90 */
  0x0004,   /* U+10C91 */
  0x0000,   /* U+10C92 */
  0x0000,   /* U+10C93 */
  0x0000,   /* U+10C94 */
  0x0000,   /* U+10C95 */
  0x0000,   /* U+10C96 */
  0x0000,   /* U+10C97 */
  0x0000,   /* U+10C98 */
  0x0000,   /* U+10C99 */
  0x0000,   /* U+10C9A */
  0x0000,   /* U+10C9B */
  0x0004,   /* U+10C9C */
  0x0000,   /* U+10C9D */
  0x0004,   /* U+10C9E */
  0x0004,   /* U+10C9F */
  0x0000,   /* U+10CA0 */
  0x0000,   /* U+10CA1 */
  0x0000,   /* U+10CA2 */
  0x0004,   /* U+10CA3 */
  0x0000,   /* U+10CA4 */
  0x0000,   /* U+10CA5 */
  0x0000,   /* U+10CA6 */
  0x0000,   /* U+10CA7 */
  0x0000,   /* U+10CA8 */
  0x0000,   /* U+10CA9 */
  0x0000,   /* U+10CAA */
  0x0004,   /* U+10CAB */
  0x0000,   /* U+10CAC */
  0x0004,   /* U+10CAD */
  0x0000,   /* U+10CAE */
  0x0000,   /* U+10CAF */
  0x0000,   /* U+10CB0 */
  0x0000,   /* U+10CB1 */
  0x0000,   /* U+10CB2 */
  0x0000,   /* U+10CB3 */
  0x0000,   /* U+10CB4 */
  0x0000,   /* U+10CB5 */
  0x0000,   /* U+10CB6 */
  0x0000,   /* U+10CB7 */
  0x0000,   /* U+10CB8 */
  0x0000,   /* U+10CB9 */
  0x0000,   /* U+10CBA */
  0x0000,   /* U+10CBB */
  0x0000,   /* U+10CBC */
  0x0000,   /* U+10CBD */
  0x0000,   /* U+10CBE */
  0x0000,   /* U+10CBF */
  0x0000,   /* U+10CC0 */
  0x0004,   /* U+10CC1 */
  0x0000,   /* U+10CC2 */
  0x0000,   /* U+10CC3 */
  0x0000,   /* U+10CC4 */
  0x0000,   /* U+10CC5 */
  0x0000,   /* U+10CC6 */
  0x0000,   /* U+10CC7 */
  0x0000,   /* U+10CC8 */
  0x0000,   /* U+10CC9 */
  0x0004,   /* U+10CCA */
  0x0004,   /* U+10CCB */
  0x0000,   /* U+10CCC */
  0x0000,   /* U+10CCD */
  0x0000,   /* U+10CCE */
  0x0000,   /* U+10CCF */
  0x0000,   /* U+10CD0 */
  0x0004,   /* U+10CD1 */
  0x0000,   /* U+10CD2 */
  0x0000,   /* U+10CD3 */
  0x0000,   /* U+10CD4 */
  0x0000,   /* U+10CD5 */
  0x0000,   /* U+10CD6 */
  0x0000,   /* U+10CD7 */
  0x0000,   /* U+10CD8 */
  0x0000,   /* U+10CD9 */
  0x0000,   /* U+10CDA */
  0x0000,   /* U+10CDB */
  0x0004,   /* U+10CDC */
  0x0000,   /* U+10CDD */
  0x0004,   /* U+10CDE */
  0x0004,   /* U+10CDF */
  0x0000,   /* U+10CE0 */
  0x0000,   /* U+10CE1 */
  0x0000,   /* U+10CE2 */
  0x0004,   /* U+10CE3 */
  0x0000,   /* U+10CE4 */
  0x0000,   /* U+10CE5 */
  0x0000,   /* U+10CE6 */
  0x0000,   /* U+10CE7 */
  0x0000,   /* U+10CE8 */
  0x0000,   /* U+10CE9 */
  0x0000,   /* U+10CEA */
  0x0004,   /* U+10CEB */
  0x0000,   /* U+10CEC */
  0x0004,   /* U+10CED */
  0x0000,   /* U+10CEE */
  0x0000,   /* U+10CEF */
  0x0000,   /* U+10CF0 */
  0x0000,   /* U+10CF1 */
  0x0000,   /* U+10CF2 */
  0x0000,   /* U+10CF3 */
  0x0000,   /* U+10CF4 */
  0x0000,   /* U+10CF5 */
  0x0000,   /* U+10CF6 */
  0x0000,   /* U+10CF7 */
  0x0000,   /* U+10CF8 */
  0x0000,   /* U+10CF9 */
  0x0000,   /* U+10CFA */
  0x0000,   /* U+10CFB */
  0x0000,   /* U+10CFC */
  0x0000,   /* U+10CFD */
  0x0000,   /* U+10CFE */
  0x0000,   /* U+10CFF */
};

uint16 uca900_p10E[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC2,   /* U+10E00 */
  0xFBC2,   /* U+10E01 */
  0xFBC2,   /* U+10E02 */
  0xFBC2,   /* U+10E03 */
  0xFBC2,   /* U+10E04 */
  0xFBC2,   /* U+10E05 */
  0xFBC2,   /* U+10E06 */
  0xFBC2,   /* U+10E07 */
  0xFBC2,   /* U+10E08 */
  0xFBC2,   /* U+10E09 */
  0xFBC2,   /* U+10E0A */
  0xFBC2,   /* U+10E0B */
  0xFBC2,   /* U+10E0C */
  0xFBC2,   /* U+10E0D */
  0xFBC2,   /* U+10E0E */
  0xFBC2,   /* U+10E0F */
  0xFBC2,   /* U+10E10 */
  0xFBC2,   /* U+10E11 */
  0xFBC2,   /* U+10E12 */
  0xFBC2,   /* U+10E13 */
  0xFBC2,   /* U+10E14 */
  0xFBC2,   /* U+10E15 */
  0xFBC2,   /* U+10E16 */
  0xFBC2,   /* U+10E17 */
  0xFBC2,   /* U+10E18 */
  0xFBC2,   /* U+10E19 */
  0xFBC2,   /* U+10E1A */
  0xFBC2,   /* U+10E1B */
  0xFBC2,   /* U+10E1C */
  0xFBC2,   /* U+10E1D */
  0xFBC2,   /* U+10E1E */
  0xFBC2,   /* U+10E1F */
  0xFBC2,   /* U+10E20 */
  0xFBC2,   /* U+10E21 */
  0xFBC2,   /* U+10E22 */
  0xFBC2,   /* U+10E23 */
  0xFBC2,   /* U+10E24 */
  0xFBC2,   /* U+10E25 */
  0xFBC2,   /* U+10E26 */
  0xFBC2,   /* U+10E27 */
  0xFBC2,   /* U+10E28 */
  0xFBC2,   /* U+10E29 */
  0xFBC2,   /* U+10E2A */
  0xFBC2,   /* U+10E2B */
  0xFBC2,   /* U+10E2C */
  0xFBC2,   /* U+10E2D */
  0xFBC2,   /* U+10E2E */
  0xFBC2,   /* U+10E2F */
  0xFBC2,   /* U+10E30 */
  0xFBC2,   /* U+10E31 */
  0xFBC2,   /* U+10E32 */
  0xFBC2,   /* U+10E33 */
  0xFBC2,   /* U+10E34 */
  0xFBC2,   /* U+10E35 */
  0xFBC2,   /* U+10E36 */
  0xFBC2,   /* U+10E37 */
  0xFBC2,   /* U+10E38 */
  0xFBC2,   /* U+10E39 */
  0xFBC2,   /* U+10E3A */
  0xFBC2,   /* U+10E3B */
  0xFBC2,   /* U+10E3C */
  0xFBC2,   /* U+10E3D */
  0xFBC2,   /* U+10E3E */
  0xFBC2,   /* U+10E3F */
  0xFBC2,   /* U+10E40 */
  0xFBC2,   /* U+10E41 */
  0xFBC2,   /* U+10E42 */
  0xFBC2,   /* U+10E43 */
  0xFBC2,   /* U+10E44 */
  0xFBC2,   /* U+10E45 */
  0xFBC2,   /* U+10E46 */
  0xFBC2,   /* U+10E47 */
  0xFBC2,   /* U+10E48 */
  0xFBC2,   /* U+10E49 */
  0xFBC2,   /* U+10E4A */
  0xFBC2,   /* U+10E4B */
  0xFBC2,   /* U+10E4C */
  0xFBC2,   /* U+10E4D */
  0xFBC2,   /* U+10E4E */
  0xFBC2,   /* U+10E4F */
  0xFBC2,   /* U+10E50 */
  0xFBC2,   /* U+10E51 */
  0xFBC2,   /* U+10E52 */
  0xFBC2,   /* U+10E53 */
  0xFBC2,   /* U+10E54 */
  0xFBC2,   /* U+10E55 */
  0xFBC2,   /* U+10E56 */
  0xFBC2,   /* U+10E57 */
  0xFBC2,   /* U+10E58 */
  0xFBC2,   /* U+10E59 */
  0xFBC2,   /* U+10E5A */
  0xFBC2,   /* U+10E5B */
  0xFBC2,   /* U+10E5C */
  0xFBC2,   /* U+10E5D */
  0xFBC2,   /* U+10E5E */
  0xFBC2,   /* U+10E5F */
  0x1C3E,   /* U+10E60 */
  0x1C3F,   /* U+10E61 */
  0x1C40,   /* U+10E62 */
  0x1C41,   /* U+10E63 */
  0x1C42,   /* U+10E64 */
  0x1C43,   /* U+10E65 */
  0x1C44,   /* U+10E66 */
  0x1C45,   /* U+10E67 */
  0x1C46,   /* U+10E68 */
  0x1ACC,   /* U+10E69 */
  0x1ACD,   /* U+10E6A */
  0x1ACE,   /* U+10E6B */
  0x1ACF,   /* U+10E6C */
  0x1AD0,   /* U+10E6D */
  0x1AD1,   /* U+10E6E */
  0x1AD2,   /* U+10E6F */
  0x1AD3,   /* U+10E70 */
  0x1AD4,   /* U+10E71 */
  0x1AD5,   /* U+10E72 */
  0x1AD6,   /* U+10E73 */
  0x1AD7,   /* U+10E74 */
  0x1AD8,   /* U+10E75 */
  0x1AD9,   /* U+10E76 */
  0x1ADA,   /* U+10E77 */
  0x1ADB,   /* U+10E78 */
  0x1ADC,   /* U+10E79 */
  0x1ADD,   /* U+10E7A */
  0x1ADE,   /* U+10E7B */
  0x1ADF,   /* U+10E7C */
  0x1AE0,   /* U+10E7D */
  0x1AE1,   /* U+10E7E */
  0xFBC2,   /* U+10E7F */
  0xFBC2,   /* U+10E80 */
  0xFBC2,   /* U+10E81 */
  0xFBC2,   /* U+10E82 */
  0xFBC2,   /* U+10E83 */
  0xFBC2,   /* U+10E84 */
  0xFBC2,   /* U+10E85 */
  0xFBC2,   /* U+10E86 */
  0xFBC2,   /* U+10E87 */
  0xFBC2,   /* U+10E88 */
  0xFBC2,   /* U+10E89 */
  0xFBC2,   /* U+10E8A */
  0xFBC2,   /* U+10E8B */
  0xFBC2,   /* U+10E8C */
  0xFBC2,   /* U+10E8D */
  0xFBC2,   /* U+10E8E */
  0xFBC2,   /* U+10E8F */
  0xFBC2,   /* U+10E90 */
  0xFBC2,   /* U+10E91 */
  0xFBC2,   /* U+10E92 */
  0xFBC2,   /* U+10E93 */
  0xFBC2,   /* U+10E94 */
  0xFBC2,   /* U+10E95 */
  0xFBC2,   /* U+10E96 */
  0xFBC2,   /* U+10E97 */
  0xFBC2,   /* U+10E98 */
  0xFBC2,   /* U+10E99 */
  0xFBC2,   /* U+10E9A */
  0xFBC2,   /* U+10E9B */
  0xFBC2,   /* U+10E9C */
  0xFBC2,   /* U+10E9D */
  0xFBC2,   /* U+10E9E */
  0xFBC2,   /* U+10E9F */
  0xFBC2,   /* U+10EA0 */
  0xFBC2,   /* U+10EA1 */
  0xFBC2,   /* U+10EA2 */
  0xFBC2,   /* U+10EA3 */
  0xFBC2,   /* U+10EA4 */
  0xFBC2,   /* U+10EA5 */
  0xFBC2,   /* U+10EA6 */
  0xFBC2,   /* U+10EA7 */
  0xFBC2,   /* U+10EA8 */
  0xFBC2,   /* U+10EA9 */
  0xFBC2,   /* U+10EAA */
  0xFBC2,   /* U+10EAB */
  0xFBC2,   /* U+10EAC */
  0xFBC2,   /* U+10EAD */
  0xFBC2,   /* U+10EAE */
  0xFBC2,   /* U+10EAF */
  0xFBC2,   /* U+10EB0 */
  0xFBC2,   /* U+10EB1 */
  0xFBC2,   /* U+10EB2 */
  0xFBC2,   /* U+10EB3 */
  0xFBC2,   /* U+10EB4 */
  0xFBC2,   /* U+10EB5 */
  0xFBC2,   /* U+10EB6 */
  0xFBC2,   /* U+10EB7 */
  0xFBC2,   /* U+10EB8 */
  0xFBC2,   /* U+10EB9 */
  0xFBC2,   /* U+10EBA */
  0xFBC2,   /* U+10EBB */
  0xFBC2,   /* U+10EBC */
  0xFBC2,   /* U+10EBD */
  0xFBC2,   /* U+10EBE */
  0xFBC2,   /* U+10EBF */
  0xFBC2,   /* U+10EC0 */
  0xFBC2,   /* U+10EC1 */
  0xFBC2,   /* U+10EC2 */
  0xFBC2,   /* U+10EC3 */
  0xFBC2,   /* U+10EC4 */
  0xFBC2,   /* U+10EC5 */
  0xFBC2,   /* U+10EC6 */
  0xFBC2,   /* U+10EC7 */
  0xFBC2,   /* U+10EC8 */
  0xFBC2,   /* U+10EC9 */
  0xFBC2,   /* U+10ECA */
  0xFBC2,   /* U+10ECB */
  0xFBC2,   /* U+10ECC */
  0xFBC2,   /* U+10ECD */
  0xFBC2,   /* U+10ECE */
  0xFBC2,   /* U+10ECF */
  0xFBC2,   /* U+10ED0 */
  0xFBC2,   /* U+10ED1 */
  0xFBC2,   /* U+10ED2 */
  0xFBC2,   /* U+10ED3 */
  0xFBC2,   /* U+10ED4 */
  0xFBC2,   /* U+10ED5 */
  0xFBC2,   /* U+10ED6 */
  0xFBC2,   /* U+10ED7 */
  0xFBC2,   /* U+10ED8 */
  0xFBC2,   /* U+10ED9 */
  0xFBC2,   /* U+10EDA */
  0xFBC2,   /* U+10EDB */
  0xFBC2,   /* U+10EDC */
  0xFBC2,   /* U+10EDD */
  0xFBC2,   /* U+10EDE */
  0xFBC2,   /* U+10EDF */
  0xFBC2,   /* U+10EE0 */
  0xFBC2,   /* U+10EE1 */
  0xFBC2,   /* U+10EE2 */
  0xFBC2,   /* U+10EE3 */
  0xFBC2,   /* U+10EE4 */
  0xFBC2,   /* U+10EE5 */
  0xFBC2,   /* U+10EE6 */
  0xFBC2,   /* U+10EE7 */
  0xFBC2,   /* U+10EE8 */
  0xFBC2,   /* U+10EE9 */
  0xFBC2,   /* U+10EEA */
  0xFBC2,   /* U+10EEB */
  0xFBC2,   /* U+10EEC */
  0xFBC2,   /* U+10EED */
  0xFBC2,   /* U+10EEE */
  0xFBC2,   /* U+10EEF */
  0xFBC2,   /* U+10EF0 */
  0xFBC2,   /* U+10EF1 */
  0xFBC2,   /* U+10EF2 */
  0xFBC2,   /* U+10EF3 */
  0xFBC2,   /* U+10EF4 */
  0xFBC2,   /* U+10EF5 */
  0xFBC2,   /* U+10EF6 */
  0xFBC2,   /* U+10EF7 */
  0xFBC2,   /* U+10EF8 */
  0xFBC2,   /* U+10EF9 */
  0xFBC2,   /* U+10EFA */
  0xFBC2,   /* U+10EFB */
  0xFBC2,   /* U+10EFC */
  0xFBC2,   /* U+10EFD */
  0xFBC2,   /* U+10EFE */
  0xFBC2,   /* U+10EFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+10E00 */
  0x0020,   /* U+10E01 */
  0x0020,   /* U+10E02 */
  0x0020,   /* U+10E03 */
  0x0020,   /* U+10E04 */
  0x0020,   /* U+10E05 */
  0x0020,   /* U+10E06 */
  0x0020,   /* U+10E07 */
  0x0020,   /* U+10E08 */
  0x0020,   /* U+10E09 */
  0x0020,   /* U+10E0A */
  0x0020,   /* U+10E0B */
  0x0020,   /* U+10E0C */
  0x0020,   /* U+10E0D */
  0x0020,   /* U+10E0E */
  0x0020,   /* U+10E0F */
  0x0020,   /* U+10E10 */
  0x0020,   /* U+10E11 */
  0x0020,   /* U+10E12 */
  0x0020,   /* U+10E13 */
  0x0020,   /* U+10E14 */
  0x0020,   /* U+10E15 */
  0x0020,   /* U+10E16 */
  0x0020,   /* U+10E17 */
  0x0020,   /* U+10E18 */
  0x0020,   /* U+10E19 */
  0x0020,   /* U+10E1A */
  0x0020,   /* U+10E1B */
  0x0020,   /* U+10E1C */
  0x0020,   /* U+10E1D */
  0x0020,   /* U+10E1E */
  0x0020,   /* U+10E1F */
  0x0020,   /* U+10E20 */
  0x0020,   /* U+10E21 */
  0x0020,   /* U+10E22 */
  0x0020,   /* U+10E23 */
  0x0020,   /* U+10E24 */
  0x0020,   /* U+10E25 */
  0x0020,   /* U+10E26 */
  0x0020,   /* U+10E27 */
  0x0020,   /* U+10E28 */
  0x0020,   /* U+10E29 */
  0x0020,   /* U+10E2A */
  0x0020,   /* U+10E2B */
  0x0020,   /* U+10E2C */
  0x0020,   /* U+10E2D */
  0x0020,   /* U+10E2E */
  0x0020,   /* U+10E2F */
  0x0020,   /* U+10E30 */
  0x0020,   /* U+10E31 */
  0x0020,   /* U+10E32 */
  0x0020,   /* U+10E33 */
  0x0020,   /* U+10E34 */
  0x0020,   /* U+10E35 */
  0x0020,   /* U+10E36 */
  0x0020,   /* U+10E37 */
  0x0020,   /* U+10E38 */
  0x0020,   /* U+10E39 */
  0x0020,   /* U+10E3A */
  0x0020,   /* U+10E3B */
  0x0020,   /* U+10E3C */
  0x0020,   /* U+10E3D */
  0x0020,   /* U+10E3E */
  0x0020,   /* U+10E3F */
  0x0020,   /* U+10E40 */
  0x0020,   /* U+10E41 */
  0x0020,   /* U+10E42 */
  0x0020,   /* U+10E43 */
  0x0020,   /* U+10E44 */
  0x0020,   /* U+10E45 */
  0x0020,   /* U+10E46 */
  0x0020,   /* U+10E47 */
  0x0020,   /* U+10E48 */
  0x0020,   /* U+10E49 */
  0x0020,   /* U+10E4A */
  0x0020,   /* U+10E4B */
  0x0020,   /* U+10E4C */
  0x0020,   /* U+10E4D */
  0x0020,   /* U+10E4E */
  0x0020,   /* U+10E4F */
  0x0020,   /* U+10E50 */
  0x0020,   /* U+10E51 */
  0x0020,   /* U+10E52 */
  0x0020,   /* U+10E53 */
  0x0020,   /* U+10E54 */
  0x0020,   /* U+10E55 */
  0x0020,   /* U+10E56 */
  0x0020,   /* U+10E57 */
  0x0020,   /* U+10E58 */
  0x0020,   /* U+10E59 */
  0x0020,   /* U+10E5A */
  0x0020,   /* U+10E5B */
  0x0020,   /* U+10E5C */
  0x0020,   /* U+10E5D */
  0x0020,   /* U+10E5E */
  0x0020,   /* U+10E5F */
  0x0020,   /* U+10E60 */
  0x0020,   /* U+10E61 */
  0x0020,   /* U+10E62 */
  0x0020,   /* U+10E63 */
  0x0020,   /* U+10E64 */
  0x0020,   /* U+10E65 */
  0x0020,   /* U+10E66 */
  0x0020,   /* U+10E67 */
  0x0020,   /* U+10E68 */
  0x0020,   /* U+10E69 */
  0x0020,   /* U+10E6A */
  0x0020,   /* U+10E6B */
  0x0020,   /* U+10E6C */
  0x0020,   /* U+10E6D */
  0x0020,   /* U+10E6E */
  0x0020,   /* U+10E6F */
  0x0020,   /* U+10E70 */
  0x0020,   /* U+10E71 */
  0x0020,   /* U+10E72 */
  0x0020,   /* U+10E73 */
  0x0020,   /* U+10E74 */
  0x0020,   /* U+10E75 */
  0x0020,   /* U+10E76 */
  0x0020,   /* U+10E77 */
  0x0020,   /* U+10E78 */
  0x0020,   /* U+10E79 */
  0x0020,   /* U+10E7A */
  0x0020,   /* U+10E7B */
  0x0020,   /* U+10E7C */
  0x0020,   /* U+10E7D */
  0x0020,   /* U+10E7E */
  0x0020,   /* U+10E7F */
  0x0020,   /* U+10E80 */
  0x0020,   /* U+10E81 */
  0x0020,   /* U+10E82 */
  0x0020,   /* U+10E83 */
  0x0020,   /* U+10E84 */
  0x0020,   /* U+10E85 */
  0x0020,   /* U+10E86 */
  0x0020,   /* U+10E87 */
  0x0020,   /* U+10E88 */
  0x0020,   /* U+10E89 */
  0x0020,   /* U+10E8A */
  0x0020,   /* U+10E8B */
  0x0020,   /* U+10E8C */
  0x0020,   /* U+10E8D */
  0x0020,   /* U+10E8E */
  0x0020,   /* U+10E8F */
  0x0020,   /* U+10E90 */
  0x0020,   /* U+10E91 */
  0x0020,   /* U+10E92 */
  0x0020,   /* U+10E93 */
  0x0020,   /* U+10E94 */
  0x0020,   /* U+10E95 */
  0x0020,   /* U+10E96 */
  0x0020,   /* U+10E97 */
  0x0020,   /* U+10E98 */
  0x0020,   /* U+10E99 */
  0x0020,   /* U+10E9A */
  0x0020,   /* U+10E9B */
  0x0020,   /* U+10E9C */
  0x0020,   /* U+10E9D */
  0x0020,   /* U+10E9E */
  0x0020,   /* U+10E9F */
  0x0020,   /* U+10EA0 */
  0x0020,   /* U+10EA1 */
  0x0020,   /* U+10EA2 */
  0x0020,   /* U+10EA3 */
  0x0020,   /* U+10EA4 */
  0x0020,   /* U+10EA5 */
  0x0020,   /* U+10EA6 */
  0x0020,   /* U+10EA7 */
  0x0020,   /* U+10EA8 */
  0x0020,   /* U+10EA9 */
  0x0020,   /* U+10EAA */
  0x0020,   /* U+10EAB */
  0x0020,   /* U+10EAC */
  0x0020,   /* U+10EAD */
  0x0020,   /* U+10EAE */
  0x0020,   /* U+10EAF */
  0x0020,   /* U+10EB0 */
  0x0020,   /* U+10EB1 */
  0x0020,   /* U+10EB2 */
  0x0020,   /* U+10EB3 */
  0x0020,   /* U+10EB4 */
  0x0020,   /* U+10EB5 */
  0x0020,   /* U+10EB6 */
  0x0020,   /* U+10EB7 */
  0x0020,   /* U+10EB8 */
  0x0020,   /* U+10EB9 */
  0x0020,   /* U+10EBA */
  0x0020,   /* U+10EBB */
  0x0020,   /* U+10EBC */
  0x0020,   /* U+10EBD */
  0x0020,   /* U+10EBE */
  0x0020,   /* U+10EBF */
  0x0020,   /* U+10EC0 */
  0x0020,   /* U+10EC1 */
  0x0020,   /* U+10EC2 */
  0x0020,   /* U+10EC3 */
  0x0020,   /* U+10EC4 */
  0x0020,   /* U+10EC5 */
  0x0020,   /* U+10EC6 */
  0x0020,   /* U+10EC7 */
  0x0020,   /* U+10EC8 */
  0x0020,   /* U+10EC9 */
  0x0020,   /* U+10ECA */
  0x0020,   /* U+10ECB */
  0x0020,   /* U+10ECC */
  0x0020,   /* U+10ECD */
  0x0020,   /* U+10ECE */
  0x0020,   /* U+10ECF */
  0x0020,   /* U+10ED0 */
  0x0020,   /* U+10ED1 */
  0x0020,   /* U+10ED2 */
  0x0020,   /* U+10ED3 */
  0x0020,   /* U+10ED4 */
  0x0020,   /* U+10ED5 */
  0x0020,   /* U+10ED6 */
  0x0020,   /* U+10ED7 */
  0x0020,   /* U+10ED8 */
  0x0020,   /* U+10ED9 */
  0x0020,   /* U+10EDA */
  0x0020,   /* U+10EDB */
  0x0020,   /* U+10EDC */
  0x0020,   /* U+10EDD */
  0x0020,   /* U+10EDE */
  0x0020,   /* U+10EDF */
  0x0020,   /* U+10EE0 */
  0x0020,   /* U+10EE1 */
  0x0020,   /* U+10EE2 */
  0x0020,   /* U+10EE3 */
  0x0020,   /* U+10EE4 */
  0x0020,   /* U+10EE5 */
  0x0020,   /* U+10EE6 */
  0x0020,   /* U+10EE7 */
  0x0020,   /* U+10EE8 */
  0x0020,   /* U+10EE9 */
  0x0020,   /* U+10EEA */
  0x0020,   /* U+10EEB */
  0x0020,   /* U+10EEC */
  0x0020,   /* U+10EED */
  0x0020,   /* U+10EEE */
  0x0020,   /* U+10EEF */
  0x0020,   /* U+10EF0 */
  0x0020,   /* U+10EF1 */
  0x0020,   /* U+10EF2 */
  0x0020,   /* U+10EF3 */
  0x0020,   /* U+10EF4 */
  0x0020,   /* U+10EF5 */
  0x0020,   /* U+10EF6 */
  0x0020,   /* U+10EF7 */
  0x0020,   /* U+10EF8 */
  0x0020,   /* U+10EF9 */
  0x0020,   /* U+10EFA */
  0x0020,   /* U+10EFB */
  0x0020,   /* U+10EFC */
  0x0020,   /* U+10EFD */
  0x0020,   /* U+10EFE */
  0x0020,   /* U+10EFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+10E00 */
  0x0002,   /* U+10E01 */
  0x0002,   /* U+10E02 */
  0x0002,   /* U+10E03 */
  0x0002,   /* U+10E04 */
  0x0002,   /* U+10E05 */
  0x0002,   /* U+10E06 */
  0x0002,   /* U+10E07 */
  0x0002,   /* U+10E08 */
  0x0002,   /* U+10E09 */
  0x0002,   /* U+10E0A */
  0x0002,   /* U+10E0B */
  0x0002,   /* U+10E0C */
  0x0002,   /* U+10E0D */
  0x0002,   /* U+10E0E */
  0x0002,   /* U+10E0F */
  0x0002,   /* U+10E10 */
  0x0002,   /* U+10E11 */
  0x0002,   /* U+10E12 */
  0x0002,   /* U+10E13 */
  0x0002,   /* U+10E14 */
  0x0002,   /* U+10E15 */
  0x0002,   /* U+10E16 */
  0x0002,   /* U+10E17 */
  0x0002,   /* U+10E18 */
  0x0002,   /* U+10E19 */
  0x0002,   /* U+10E1A */
  0x0002,   /* U+10E1B */
  0x0002,   /* U+10E1C */
  0x0002,   /* U+10E1D */
  0x0002,   /* U+10E1E */
  0x0002,   /* U+10E1F */
  0x0002,   /* U+10E20 */
  0x0002,   /* U+10E21 */
  0x0002,   /* U+10E22 */
  0x0002,   /* U+10E23 */
  0x0002,   /* U+10E24 */
  0x0002,   /* U+10E25 */
  0x0002,   /* U+10E26 */
  0x0002,   /* U+10E27 */
  0x0002,   /* U+10E28 */
  0x0002,   /* U+10E29 */
  0x0002,   /* U+10E2A */
  0x0002,   /* U+10E2B */
  0x0002,   /* U+10E2C */
  0x0002,   /* U+10E2D */
  0x0002,   /* U+10E2E */
  0x0002,   /* U+10E2F */
  0x0002,   /* U+10E30 */
  0x0002,   /* U+10E31 */
  0x0002,   /* U+10E32 */
  0x0002,   /* U+10E33 */
  0x0002,   /* U+10E34 */
  0x0002,   /* U+10E35 */
  0x0002,   /* U+10E36 */
  0x0002,   /* U+10E37 */
  0x0002,   /* U+10E38 */
  0x0002,   /* U+10E39 */
  0x0002,   /* U+10E3A */
  0x0002,   /* U+10E3B */
  0x0002,   /* U+10E3C */
  0x0002,   /* U+10E3D */
  0x0002,   /* U+10E3E */
  0x0002,   /* U+10E3F */
  0x0002,   /* U+10E40 */
  0x0002,   /* U+10E41 */
  0x0002,   /* U+10E42 */
  0x0002,   /* U+10E43 */
  0x0002,   /* U+10E44 */
  0x0002,   /* U+10E45 */
  0x0002,   /* U+10E46 */
  0x0002,   /* U+10E47 */
  0x0002,   /* U+10E48 */
  0x0002,   /* U+10E49 */
  0x0002,   /* U+10E4A */
  0x0002,   /* U+10E4B */
  0x0002,   /* U+10E4C */
  0x0002,   /* U+10E4D */
  0x0002,   /* U+10E4E */
  0x0002,   /* U+10E4F */
  0x0002,   /* U+10E50 */
  0x0002,   /* U+10E51 */
  0x0002,   /* U+10E52 */
  0x0002,   /* U+10E53 */
  0x0002,   /* U+10E54 */
  0x0002,   /* U+10E55 */
  0x0002,   /* U+10E56 */
  0x0002,   /* U+10E57 */
  0x0002,   /* U+10E58 */
  0x0002,   /* U+10E59 */
  0x0002,   /* U+10E5A */
  0x0002,   /* U+10E5B */
  0x0002,   /* U+10E5C */
  0x0002,   /* U+10E5D */
  0x0002,   /* U+10E5E */
  0x0002,   /* U+10E5F */
  0x0002,   /* U+10E60 */
  0x0002,   /* U+10E61 */
  0x0002,   /* U+10E62 */
  0x0002,   /* U+10E63 */
  0x0002,   /* U+10E64 */
  0x0002,   /* U+10E65 */
  0x0002,   /* U+10E66 */
  0x0002,   /* U+10E67 */
  0x0002,   /* U+10E68 */
  0x0002,   /* U+10E69 */
  0x0002,   /* U+10E6A */
  0x0002,   /* U+10E6B */
  0x0002,   /* U+10E6C */
  0x0002,   /* U+10E6D */
  0x0002,   /* U+10E6E */
  0x0002,   /* U+10E6F */
  0x0002,   /* U+10E70 */
  0x0002,   /* U+10E71 */
  0x0002,   /* U+10E72 */
  0x0002,   /* U+10E73 */
  0x0002,   /* U+10E74 */
  0x0002,   /* U+10E75 */
  0x0002,   /* U+10E76 */
  0x0002,   /* U+10E77 */
  0x0002,   /* U+10E78 */
  0x0002,   /* U+10E79 */
  0x0002,   /* U+10E7A */
  0x0002,   /* U+10E7B */
  0x0002,   /* U+10E7C */
  0x0002,   /* U+10E7D */
  0x0002,   /* U+10E7E */
  0x0002,   /* U+10E7F */
  0x0002,   /* U+10E80 */
  0x0002,   /* U+10E81 */
  0x0002,   /* U+10E82 */
  0x0002,   /* U+10E83 */
  0x0002,   /* U+10E84 */
  0x0002,   /* U+10E85 */
  0x0002,   /* U+10E86 */
  0x0002,   /* U+10E87 */
  0x0002,   /* U+10E88 */
  0x0002,   /* U+10E89 */
  0x0002,   /* U+10E8A */
  0x0002,   /* U+10E8B */
  0x0002,   /* U+10E8C */
  0x0002,   /* U+10E8D */
  0x0002,   /* U+10E8E */
  0x0002,   /* U+10E8F */
  0x0002,   /* U+10E90 */
  0x0002,   /* U+10E91 */
  0x0002,   /* U+10E92 */
  0x0002,   /* U+10E93 */
  0x0002,   /* U+10E94 */
  0x0002,   /* U+10E95 */
  0x0002,   /* U+10E96 */
  0x0002,   /* U+10E97 */
  0x0002,   /* U+10E98 */
  0x0002,   /* U+10E99 */
  0x0002,   /* U+10E9A */
  0x0002,   /* U+10E9B */
  0x0002,   /* U+10E9C */
  0x0002,   /* U+10E9D */
  0x0002,   /* U+10E9E */
  0x0002,   /* U+10E9F */
  0x0002,   /* U+10EA0 */
  0x0002,   /* U+10EA1 */
  0x0002,   /* U+10EA2 */
  0x0002,   /* U+10EA3 */
  0x0002,   /* U+10EA4 */
  0x0002,   /* U+10EA5 */
  0x0002,   /* U+10EA6 */
  0x0002,   /* U+10EA7 */
  0x0002,   /* U+10EA8 */
  0x0002,   /* U+10EA9 */
  0x0002,   /* U+10EAA */
  0x0002,   /* U+10EAB */
  0x0002,   /* U+10EAC */
  0x0002,   /* U+10EAD */
  0x0002,   /* U+10EAE */
  0x0002,   /* U+10EAF */
  0x0002,   /* U+10EB0 */
  0x0002,   /* U+10EB1 */
  0x0002,   /* U+10EB2 */
  0x0002,   /* U+10EB3 */
  0x0002,   /* U+10EB4 */
  0x0002,   /* U+10EB5 */
  0x0002,   /* U+10EB6 */
  0x0002,   /* U+10EB7 */
  0x0002,   /* U+10EB8 */
  0x0002,   /* U+10EB9 */
  0x0002,   /* U+10EBA */
  0x0002,   /* U+10EBB */
  0x0002,   /* U+10EBC */
  0x0002,   /* U+10EBD */
  0x0002,   /* U+10EBE */
  0x0002,   /* U+10EBF */
  0x0002,   /* U+10EC0 */
  0x0002,   /* U+10EC1 */
  0x0002,   /* U+10EC2 */
  0x0002,   /* U+10EC3 */
  0x0002,   /* U+10EC4 */
  0x0002,   /* U+10EC5 */
  0x0002,   /* U+10EC6 */
  0x0002,   /* U+10EC7 */
  0x0002,   /* U+10EC8 */
  0x0002,   /* U+10EC9 */
  0x0002,   /* U+10ECA */
  0x0002,   /* U+10ECB */
  0x0002,   /* U+10ECC */
  0x0002,   /* U+10ECD */
  0x0002,   /* U+10ECE */
  0x0002,   /* U+10ECF */
  0x0002,   /* U+10ED0 */
  0x0002,   /* U+10ED1 */
  0x0002,   /* U+10ED2 */
  0x0002,   /* U+10ED3 */
  0x0002,   /* U+10ED4 */
  0x0002,   /* U+10ED5 */
  0x0002,   /* U+10ED6 */
  0x0002,   /* U+10ED7 */
  0x0002,   /* U+10ED8 */
  0x0002,   /* U+10ED9 */
  0x0002,   /* U+10EDA */
  0x0002,   /* U+10EDB */
  0x0002,   /* U+10EDC */
  0x0002,   /* U+10EDD */
  0x0002,   /* U+10EDE */
  0x0002,   /* U+10EDF */
  0x0002,   /* U+10EE0 */
  0x0002,   /* U+10EE1 */
  0x0002,   /* U+10EE2 */
  0x0002,   /* U+10EE3 */
  0x0002,   /* U+10EE4 */
  0x0002,   /* U+10EE5 */
  0x0002,   /* U+10EE6 */
  0x0002,   /* U+10EE7 */
  0x0002,   /* U+10EE8 */
  0x0002,   /* U+10EE9 */
  0x0002,   /* U+10EEA */
  0x0002,   /* U+10EEB */
  0x0002,   /* U+10EEC */
  0x0002,   /* U+10EED */
  0x0002,   /* U+10EEE */
  0x0002,   /* U+10EEF */
  0x0002,   /* U+10EF0 */
  0x0002,   /* U+10EF1 */
  0x0002,   /* U+10EF2 */
  0x0002,   /* U+10EF3 */
  0x0002,   /* U+10EF4 */
  0x0002,   /* U+10EF5 */
  0x0002,   /* U+10EF6 */
  0x0002,   /* U+10EF7 */
  0x0002,   /* U+10EF8 */
  0x0002,   /* U+10EF9 */
  0x0002,   /* U+10EFA */
  0x0002,   /* U+10EFB */
  0x0002,   /* U+10EFC */
  0x0002,   /* U+10EFD */
  0x0002,   /* U+10EFE */
  0x0002,   /* U+10EFF */

  /* Primary weight 2 for each character. */
  0x8E00,   /* U+10E00 */
  0x8E01,   /* U+10E01 */
  0x8E02,   /* U+10E02 */
  0x8E03,   /* U+10E03 */
  0x8E04,   /* U+10E04 */
  0x8E05,   /* U+10E05 */
  0x8E06,   /* U+10E06 */
  0x8E07,   /* U+10E07 */
  0x8E08,   /* U+10E08 */
  0x8E09,   /* U+10E09 */
  0x8E0A,   /* U+10E0A */
  0x8E0B,   /* U+10E0B */
  0x8E0C,   /* U+10E0C */
  0x8E0D,   /* U+10E0D */
  0x8E0E,   /* U+10E0E */
  0x8E0F,   /* U+10E0F */
  0x8E10,   /* U+10E10 */
  0x8E11,   /* U+10E11 */
  0x8E12,   /* U+10E12 */
  0x8E13,   /* U+10E13 */
  0x8E14,   /* U+10E14 */
  0x8E15,   /* U+10E15 */
  0x8E16,   /* U+10E16 */
  0x8E17,   /* U+10E17 */
  0x8E18,   /* U+10E18 */
  0x8E19,   /* U+10E19 */
  0x8E1A,   /* U+10E1A */
  0x8E1B,   /* U+10E1B */
  0x8E1C,   /* U+10E1C */
  0x8E1D,   /* U+10E1D */
  0x8E1E,   /* U+10E1E */
  0x8E1F,   /* U+10E1F */
  0x8E20,   /* U+10E20 */
  0x8E21,   /* U+10E21 */
  0x8E22,   /* U+10E22 */
  0x8E23,   /* U+10E23 */
  0x8E24,   /* U+10E24 */
  0x8E25,   /* U+10E25 */
  0x8E26,   /* U+10E26 */
  0x8E27,   /* U+10E27 */
  0x8E28,   /* U+10E28 */
  0x8E29,   /* U+10E29 */
  0x8E2A,   /* U+10E2A */
  0x8E2B,   /* U+10E2B */
  0x8E2C,   /* U+10E2C */
  0x8E2D,   /* U+10E2D */
  0x8E2E,   /* U+10E2E */
  0x8E2F,   /* U+10E2F */
  0x8E30,   /* U+10E30 */
  0x8E31,   /* U+10E31 */
  0x8E32,   /* U+10E32 */
  0x8E33,   /* U+10E33 */
  0x8E34,   /* U+10E34 */
  0x8E35,   /* U+10E35 */
  0x8E36,   /* U+10E36 */
  0x8E37,   /* U+10E37 */
  0x8E38,   /* U+10E38 */
  0x8E39,   /* U+10E39 */
  0x8E3A,   /* U+10E3A */
  0x8E3B,   /* U+10E3B */
  0x8E3C,   /* U+10E3C */
  0x8E3D,   /* U+10E3D */
  0x8E3E,   /* U+10E3E */
  0x8E3F,   /* U+10E3F */
  0x8E40,   /* U+10E40 */
  0x8E41,   /* U+10E41 */
  0x8E42,   /* U+10E42 */
  0x8E43,   /* U+10E43 */
  0x8E44,   /* U+10E44 */
  0x8E45,   /* U+10E45 */
  0x8E46,   /* U+10E46 */
  0x8E47,   /* U+10E47 */
  0x8E48,   /* U+10E48 */
  0x8E49,   /* U+10E49 */
  0x8E4A,   /* U+10E4A */
  0x8E4B,   /* U+10E4B */
  0x8E4C,   /* U+10E4C */
  0x8E4D,   /* U+10E4D */
  0x8E4E,   /* U+10E4E */
  0x8E4F,   /* U+10E4F */
  0x8E50,   /* U+10E50 */
  0x8E51,   /* U+10E51 */
  0x8E52,   /* U+10E52 */
  0x8E53,   /* U+10E53 */
  0x8E54,   /* U+10E54 */
  0x8E55,   /* U+10E55 */
  0x8E56,   /* U+10E56 */
  0x8E57,   /* U+10E57 */
  0x8E58,   /* U+10E58 */
  0x8E59,   /* U+10E59 */
  0x8E5A,   /* U+10E5A */
  0x8E5B,   /* U+10E5B */
  0x8E5C,   /* U+10E5C */
  0x8E5D,   /* U+10E5D */
  0x8E5E,   /* U+10E5E */
  0x8E5F,   /* U+10E5F */
  0x0000,   /* U+10E60 */
  0x0000,   /* U+10E61 */
  0x0000,   /* U+10E62 */
  0x0000,   /* U+10E63 */
  0x0000,   /* U+10E64 */
  0x0000,   /* U+10E65 */
  0x0000,   /* U+10E66 */
  0x0000,   /* U+10E67 */
  0x0000,   /* U+10E68 */
  0x0000,   /* U+10E69 */
  0x0000,   /* U+10E6A */
  0x0000,   /* U+10E6B */
  0x0000,   /* U+10E6C */
  0x0000,   /* U+10E6D */
  0x0000,   /* U+10E6E */
  0x0000,   /* U+10E6F */
  0x0000,   /* U+10E70 */
  0x0000,   /* U+10E71 */
  0x0000,   /* U+10E72 */
  0x0000,   /* U+10E73 */
  0x0000,   /* U+10E74 */
  0x0000,   /* U+10E75 */
  0x0000,   /* U+10E76 */
  0x0000,   /* U+10E77 */
  0x0000,   /* U+10E78 */
  0x0000,   /* U+10E79 */
  0x0000,   /* U+10E7A */
  0x0000,   /* U+10E7B */
  0x0000,   /* U+10E7C */
  0x0000,   /* U+10E7D */
  0x0000,   /* U+10E7E */
  0x8E7F,   /* U+10E7F */
  0x8E80,   /* U+10E80 */
  0x8E81,   /* U+10E81 */
  0x8E82,   /* U+10E82 */
  0x8E83,   /* U+10E83 */
  0x8E84,   /* U+10E84 */
  0x8E85,   /* U+10E85 */
  0x8E86,   /* U+10E86 */
  0x8E87,   /* U+10E87 */
  0x8E88,   /* U+10E88 */
  0x8E89,   /* U+10E89 */
  0x8E8A,   /* U+10E8A */
  0x8E8B,   /* U+10E8B */
  0x8E8C,   /* U+10E8C */
  0x8E8D,   /* U+10E8D */
  0x8E8E,   /* U+10E8E */
  0x8E8F,   /* U+10E8F */
  0x8E90,   /* U+10E90 */
  0x8E91,   /* U+10E91 */
  0x8E92,   /* U+10E92 */
  0x8E93,   /* U+10E93 */
  0x8E94,   /* U+10E94 */
  0x8E95,   /* U+10E95 */
  0x8E96,   /* U+10E96 */
  0x8E97,   /* U+10E97 */
  0x8E98,   /* U+10E98 */
  0x8E99,   /* U+10E99 */
  0x8E9A,   /* U+10E9A */
  0x8E9B,   /* U+10E9B */
  0x8E9C,   /* U+10E9C */
  0x8E9D,   /* U+10E9D */
  0x8E9E,   /* U+10E9E */
  0x8E9F,   /* U+10E9F */
  0x8EA0,   /* U+10EA0 */
  0x8EA1,   /* U+10EA1 */
  0x8EA2,   /* U+10EA2 */
  0x8EA3,   /* U+10EA3 */
  0x8EA4,   /* U+10EA4 */
  0x8EA5,   /* U+10EA5 */
  0x8EA6,   /* U+10EA6 */
  0x8EA7,   /* U+10EA7 */
  0x8EA8,   /* U+10EA8 */
  0x8EA9,   /* U+10EA9 */
  0x8EAA,   /* U+10EAA */
  0x8EAB,   /* U+10EAB */
  0x8EAC,   /* U+10EAC */
  0x8EAD,   /* U+10EAD */
  0x8EAE,   /* U+10EAE */
  0x8EAF,   /* U+10EAF */
  0x8EB0,   /* U+10EB0 */
  0x8EB1,   /* U+10EB1 */
  0x8EB2,   /* U+10EB2 */
  0x8EB3,   /* U+10EB3 */
  0x8EB4,   /* U+10EB4 */
  0x8EB5,   /* U+10EB5 */
  0x8EB6,   /* U+10EB6 */
  0x8EB7,   /* U+10EB7 */
  0x8EB8,   /* U+10EB8 */
  0x8EB9,   /* U+10EB9 */
  0x8EBA,   /* U+10EBA */
  0x8EBB,   /* U+10EBB */
  0x8EBC,   /* U+10EBC */
  0x8EBD,   /* U+10EBD */
  0x8EBE,   /* U+10EBE */
  0x8EBF,   /* U+10EBF */
  0x8EC0,   /* U+10EC0 */
  0x8EC1,   /* U+10EC1 */
  0x8EC2,   /* U+10EC2 */
  0x8EC3,   /* U+10EC3 */
  0x8EC4,   /* U+10EC4 */
  0x8EC5,   /* U+10EC5 */
  0x8EC6,   /* U+10EC6 */
  0x8EC7,   /* U+10EC7 */
  0x8EC8,   /* U+10EC8 */
  0x8EC9,   /* U+10EC9 */
  0x8ECA,   /* U+10ECA */
  0x8ECB,   /* U+10ECB */
  0x8ECC,   /* U+10ECC */
  0x8ECD,   /* U+10ECD */
  0x8ECE,   /* U+10ECE */
  0x8ECF,   /* U+10ECF */
  0x8ED0,   /* U+10ED0 */
  0x8ED1,   /* U+10ED1 */
  0x8ED2,   /* U+10ED2 */
  0x8ED3,   /* U+10ED3 */
  0x8ED4,   /* U+10ED4 */
  0x8ED5,   /* U+10ED5 */
  0x8ED6,   /* U+10ED6 */
  0x8ED7,   /* U+10ED7 */
  0x8ED8,   /* U+10ED8 */
  0x8ED9,   /* U+10ED9 */
  0x8EDA,   /* U+10EDA */
  0x8EDB,   /* U+10EDB */
  0x8EDC,   /* U+10EDC */
  0x8EDD,   /* U+10EDD */
  0x8EDE,   /* U+10EDE */
  0x8EDF,   /* U+10EDF */
  0x8EE0,   /* U+10EE0 */
  0x8EE1,   /* U+10EE1 */
  0x8EE2,   /* U+10EE2 */
  0x8EE3,   /* U+10EE3 */
  0x8EE4,   /* U+10EE4 */
  0x8EE5,   /* U+10EE5 */
  0x8EE6,   /* U+10EE6 */
  0x8EE7,   /* U+10EE7 */
  0x8EE8,   /* U+10EE8 */
  0x8EE9,   /* U+10EE9 */
  0x8EEA,   /* U+10EEA */
  0x8EEB,   /* U+10EEB */
  0x8EEC,   /* U+10EEC */
  0x8EED,   /* U+10EED */
  0x8EEE,   /* U+10EEE */
  0x8EEF,   /* U+10EEF */
  0x8EF0,   /* U+10EF0 */
  0x8EF1,   /* U+10EF1 */
  0x8EF2,   /* U+10EF2 */
  0x8EF3,   /* U+10EF3 */
  0x8EF4,   /* U+10EF4 */
  0x8EF5,   /* U+10EF5 */
  0x8EF6,   /* U+10EF6 */
  0x8EF7,   /* U+10EF7 */
  0x8EF8,   /* U+10EF8 */
  0x8EF9,   /* U+10EF9 */
  0x8EFA,   /* U+10EFA */
  0x8EFB,   /* U+10EFB */
  0x8EFC,   /* U+10EFC */
  0x8EFD,   /* U+10EFD */
  0x8EFE,   /* U+10EFE */
  0x8EFF,   /* U+10EFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+10E00 */
  0x0000,   /* U+10E01 */
  0x0000,   /* U+10E02 */
  0x0000,   /* U+10E03 */
  0x0000,   /* U+10E04 */
  0x0000,   /* U+10E05 */
  0x0000,   /* U+10E06 */
  0x0000,   /* U+10E07 */
  0x0000,   /* U+10E08 */
  0x0000,   /* U+10E09 */
  0x0000,   /* U+10E0A */
  0x0000,   /* U+10E0B */
  0x0000,   /* U+10E0C */
  0x0000,   /* U+10E0D */
  0x0000,   /* U+10E0E */
  0x0000,   /* U+10E0F */
  0x0000,   /* U+10E10 */
  0x0000,   /* U+10E11 */
  0x0000,   /* U+10E12 */
  0x0000,   /* U+10E13 */
  0x0000,   /* U+10E14 */
  0x0000,   /* U+10E15 */
  0x0000,   /* U+10E16 */
  0x0000,   /* U+10E17 */
  0x0000,   /* U+10E18 */
  0x0000,   /* U+10E19 */
  0x0000,   /* U+10E1A */
  0x0000,   /* U+10E1B */
  0x0000,   /* U+10E1C */
  0x0000,   /* U+10E1D */
  0x0000,   /* U+10E1E */
  0x0000,   /* U+10E1F */
  0x0000,   /* U+10E20 */
  0x0000,   /* U+10E21 */
  0x0000,   /* U+10E22 */
  0x0000,   /* U+10E23 */
  0x0000,   /* U+10E24 */
  0x0000,   /* U+10E25 */
  0x0000,   /* U+10E26 */
  0x0000,   /* U+10E27 */
  0x0000,   /* U+10E28 */
  0x0000,   /* U+10E29 */
  0x0000,   /* U+10E2A */
  0x0000,   /* U+10E2B */
  0x0000,   /* U+10E2C */
  0x0000,   /* U+10E2D */
  0x0000,   /* U+10E2E */
  0x0000,   /* U+10E2F */
  0x0000,   /* U+10E30 */
  0x0000,   /* U+10E31 */
  0x0000,   /* U+10E32 */
  0x0000,   /* U+10E33 */
  0x0000,   /* U+10E34 */
  0x0000,   /* U+10E35 */
  0x0000,   /* U+10E36 */
  0x0000,   /* U+10E37 */
  0x0000,   /* U+10E38 */
  0x0000,   /* U+10E39 */
  0x0000,   /* U+10E3A */
  0x0000,   /* U+10E3B */
  0x0000,   /* U+10E3C */
  0x0000,   /* U+10E3D */
  0x0000,   /* U+10E3E */
  0x0000,   /* U+10E3F */
  0x0000,   /* U+10E40 */
  0x0000,   /* U+10E41 */
  0x0000,   /* U+10E42 */
  0x0000,   /* U+10E43 */
  0x0000,   /* U+10E44 */
  0x0000,   /* U+10E45 */
  0x0000,   /* U+10E46 */
  0x0000,   /* U+10E47 */
  0x0000,   /* U+10E48 */
  0x0000,   /* U+10E49 */
  0x0000,   /* U+10E4A */
  0x0000,   /* U+10E4B */
  0x0000,   /* U+10E4C */
  0x0000,   /* U+10E4D */
  0x0000,   /* U+10E4E */
  0x0000,   /* U+10E4F */
  0x0000,   /* U+10E50 */
  0x0000,   /* U+10E51 */
  0x0000,   /* U+10E52 */
  0x0000,   /* U+10E53 */
  0x0000,   /* U+10E54 */
  0x0000,   /* U+10E55 */
  0x0000,   /* U+10E56 */
  0x0000,   /* U+10E57 */
  0x0000,   /* U+10E58 */
  0x0000,   /* U+10E59 */
  0x0000,   /* U+10E5A */
  0x0000,   /* U+10E5B */
  0x0000,   /* U+10E5C */
  0x0000,   /* U+10E5D */
  0x0000,   /* U+10E5E */
  0x0000,   /* U+10E5F */
  0x0000,   /* U+10E60 */
  0x0000,   /* U+10E61 */
  0x0000,   /* U+10E62 */
  0x0000,   /* U+10E63 */
  0x0000,   /* U+10E64 */
  0x0000,   /* U+10E65 */
  0x0000,   /* U+10E66 */
  0x0000,   /* U+10E67 */
  0x0000,   /* U+10E68 */
  0x0000,   /* U+10E69 */
  0x0000,   /* U+10E6A */
  0x0000,   /* U+10E6B */
  0x0000,   /* U+10E6C */
  0x0000,   /* U+10E6D */
  0x0000,   /* U+10E6E */
  0x0000,   /* U+10E6F */
  0x0000,   /* U+10E70 */
  0x0000,   /* U+10E71 */
  0x0000,   /* U+10E72 */
  0x0000,   /* U+10E73 */
  0x0000,   /* U+10E74 */
  0x0000,   /* U+10E75 */
  0x0000,   /* U+10E76 */
  0x0000,   /* U+10E77 */
  0x0000,   /* U+10E78 */
  0x0000,   /* U+10E79 */
  0x0000,   /* U+10E7A */
  0x0000,   /* U+10E7B */
  0x0000,   /* U+10E7C */
  0x0000,   /* U+10E7D */
  0x0000,   /* U+10E7E */
  0x0000,   /* U+10E7F */
  0x0000,   /* U+10E80 */
  0x0000,   /* U+10E81 */
  0x0000,   /* U+10E82 */
  0x0000,   /* U+10E83 */
  0x0000,   /* U+10E84 */
  0x0000,   /* U+10E85 */
  0x0000,   /* U+10E86 */
  0x0000,   /* U+10E87 */
  0x0000,   /* U+10E88 */
  0x0000,   /* U+10E89 */
  0x0000,   /* U+10E8A */
  0x0000,   /* U+10E8B */
  0x0000,   /* U+10E8C */
  0x0000,   /* U+10E8D */
  0x0000,   /* U+10E8E */
  0x0000,   /* U+10E8F */
  0x0000,   /* U+10E90 */
  0x0000,   /* U+10E91 */
  0x0000,   /* U+10E92 */
  0x0000,   /* U+10E93 */
  0x0000,   /* U+10E94 */
  0x0000,   /* U+10E95 */
  0x0000,   /* U+10E96 */
  0x0000,   /* U+10E97 */
  0x0000,   /* U+10E98 */
  0x0000,   /* U+10E99 */
  0x0000,   /* U+10E9A */
  0x0000,   /* U+10E9B */
  0x0000,   /* U+10E9C */
  0x0000,   /* U+10E9D */
  0x0000,   /* U+10E9E */
  0x0000,   /* U+10E9F */
  0x0000,   /* U+10EA0 */
  0x0000,   /* U+10EA1 */
  0x0000,   /* U+10EA2 */
  0x0000,   /* U+10EA3 */
  0x0000,   /* U+10EA4 */
  0x0000,   /* U+10EA5 */
  0x0000,   /* U+10EA6 */
  0x0000,   /* U+10EA7 */
  0x0000,   /* U+10EA8 */
  0x0000,   /* U+10EA9 */
  0x0000,   /* U+10EAA */
  0x0000,   /* U+10EAB */
  0x0000,   /* U+10EAC */
  0x0000,   /* U+10EAD */
  0x0000,   /* U+10EAE */
  0x0000,   /* U+10EAF */
  0x0000,   /* U+10EB0 */
  0x0000,   /* U+10EB1 */
  0x0000,   /* U+10EB2 */
  0x0000,   /* U+10EB3 */
  0x0000,   /* U+10EB4 */
  0x0000,   /* U+10EB5 */
  0x0000,   /* U+10EB6 */
  0x0000,   /* U+10EB7 */
  0x0000,   /* U+10EB8 */
  0x0000,   /* U+10EB9 */
  0x0000,   /* U+10EBA */
  0x0000,   /* U+10EBB */
  0x0000,   /* U+10EBC */
  0x0000,   /* U+10EBD */
  0x0000,   /* U+10EBE */
  0x0000,   /* U+10EBF */
  0x0000,   /* U+10EC0 */
  0x0000,   /* U+10EC1 */
  0x0000,   /* U+10EC2 */
  0x0000,   /* U+10EC3 */
  0x0000,   /* U+10EC4 */
  0x0000,   /* U+10EC5 */
  0x0000,   /* U+10EC6 */
  0x0000,   /* U+10EC7 */
  0x0000,   /* U+10EC8 */
  0x0000,   /* U+10EC9 */
  0x0000,   /* U+10ECA */
  0x0000,   /* U+10ECB */
  0x0000,   /* U+10ECC */
  0x0000,   /* U+10ECD */
  0x0000,   /* U+10ECE */
  0x0000,   /* U+10ECF */
  0x0000,   /* U+10ED0 */
  0x0000,   /* U+10ED1 */
  0x0000,   /* U+10ED2 */
  0x0000,   /* U+10ED3 */
  0x0000,   /* U+10ED4 */
  0x0000,   /* U+10ED5 */
  0x0000,   /* U+10ED6 */
  0x0000,   /* U+10ED7 */
  0x0000,   /* U+10ED8 */
  0x0000,   /* U+10ED9 */
  0x0000,   /* U+10EDA */
  0x0000,   /* U+10EDB */
  0x0000,   /* U+10EDC */
  0x0000,   /* U+10EDD */
  0x0000,   /* U+10EDE */
  0x0000,   /* U+10EDF */
  0x0000,   /* U+10EE0 */
  0x0000,   /* U+10EE1 */
  0x0000,   /* U+10EE2 */
  0x0000,   /* U+10EE3 */
  0x0000,   /* U+10EE4 */
  0x0000,   /* U+10EE5 */
  0x0000,   /* U+10EE6 */
  0x0000,   /* U+10EE7 */
  0x0000,   /* U+10EE8 */
  0x0000,   /* U+10EE9 */
  0x0000,   /* U+10EEA */
  0x0000,   /* U+10EEB */
  0x0000,   /* U+10EEC */
  0x0000,   /* U+10EED */
  0x0000,   /* U+10EEE */
  0x0000,   /* U+10EEF */
  0x0000,   /* U+10EF0 */
  0x0000,   /* U+10EF1 */
  0x0000,   /* U+10EF2 */
  0x0000,   /* U+10EF3 */
  0x0000,   /* U+10EF4 */
  0x0000,   /* U+10EF5 */
  0x0000,   /* U+10EF6 */
  0x0000,   /* U+10EF7 */
  0x0000,   /* U+10EF8 */
  0x0000,   /* U+10EF9 */
  0x0000,   /* U+10EFA */
  0x0000,   /* U+10EFB */
  0x0000,   /* U+10EFC */
  0x0000,   /* U+10EFD */
  0x0000,   /* U+10EFE */
  0x0000,   /* U+10EFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+10E00 */
  0x0000,   /* U+10E01 */
  0x0000,   /* U+10E02 */
  0x0000,   /* U+10E03 */
  0x0000,   /* U+10E04 */
  0x0000,   /* U+10E05 */
  0x0000,   /* U+10E06 */
  0x0000,   /* U+10E07 */
  0x0000,   /* U+10E08 */
  0x0000,   /* U+10E09 */
  0x0000,   /* U+10E0A */
  0x0000,   /* U+10E0B */
  0x0000,   /* U+10E0C */
  0x0000,   /* U+10E0D */
  0x0000,   /* U+10E0E */
  0x0000,   /* U+10E0F */
  0x0000,   /* U+10E10 */
  0x0000,   /* U+10E11 */
  0x0000,   /* U+10E12 */
  0x0000,   /* U+10E13 */
  0x0000,   /* U+10E14 */
  0x0000,   /* U+10E15 */
  0x0000,   /* U+10E16 */
  0x0000,   /* U+10E17 */
  0x0000,   /* U+10E18 */
  0x0000,   /* U+10E19 */
  0x0000,   /* U+10E1A */
  0x0000,   /* U+10E1B */
  0x0000,   /* U+10E1C */
  0x0000,   /* U+10E1D */
  0x0000,   /* U+10E1E */
  0x0000,   /* U+10E1F */
  0x0000,   /* U+10E20 */
  0x0000,   /* U+10E21 */
  0x0000,   /* U+10E22 */
  0x0000,   /* U+10E23 */
  0x0000,   /* U+10E24 */
  0x0000,   /* U+10E25 */
  0x0000,   /* U+10E26 */
  0x0000,   /* U+10E27 */
  0x0000,   /* U+10E28 */
  0x0000,   /* U+10E29 */
  0x0000,   /* U+10E2A */
  0x0000,   /* U+10E2B */
  0x0000,   /* U+10E2C */
  0x0000,   /* U+10E2D */
  0x0000,   /* U+10E2E */
  0x0000,   /* U+10E2F */
  0x0000,   /* U+10E30 */
  0x0000,   /* U+10E31 */
  0x0000,   /* U+10E32 */
  0x0000,   /* U+10E33 */
  0x0000,   /* U+10E34 */
  0x0000,   /* U+10E35 */
  0x0000,   /* U+10E36 */
  0x0000,   /* U+10E37 */
  0x0000,   /* U+10E38 */
  0x0000,   /* U+10E39 */
  0x0000,   /* U+10E3A */
  0x0000,   /* U+10E3B */
  0x0000,   /* U+10E3C */
  0x0000,   /* U+10E3D */
  0x0000,   /* U+10E3E */
  0x0000,   /* U+10E3F */
  0x0000,   /* U+10E40 */
  0x0000,   /* U+10E41 */
  0x0000,   /* U+10E42 */
  0x0000,   /* U+10E43 */
  0x0000,   /* U+10E44 */
  0x0000,   /* U+10E45 */
  0x0000,   /* U+10E46 */
  0x0000,   /* U+10E47 */
  0x0000,   /* U+10E48 */
  0x0000,   /* U+10E49 */
  0x0000,   /* U+10E4A */
  0x0000,   /* U+10E4B */
  0x0000,   /* U+10E4C */
  0x0000,   /* U+10E4D */
  0x0000,   /* U+10E4E */
  0x0000,   /* U+10E4F */
  0x0000,   /* U+10E50 */
  0x0000,   /* U+10E51 */
  0x0000,   /* U+10E52 */
  0x0000,   /* U+10E53 */
  0x0000,   /* U+10E54 */
  0x0000,   /* U+10E55 */
  0x0000,   /* U+10E56 */
  0x0000,   /* U+10E57 */
  0x0000,   /* U+10E58 */
  0x0000,   /* U+10E59 */
  0x0000,   /* U+10E5A */
  0x0000,   /* U+10E5B */
  0x0000,   /* U+10E5C */
  0x0000,   /* U+10E5D */
  0x0000,   /* U+10E5E */
  0x0000,   /* U+10E5F */
  0x0000,   /* U+10E60 */
  0x0000,   /* U+10E61 */
  0x0000,   /* U+10E62 */
  0x0000,   /* U+10E63 */
  0x0000,   /* U+10E64 */
  0x0000,   /* U+10E65 */
  0x0000,   /* U+10E66 */
  0x0000,   /* U+10E67 */
  0x0000,   /* U+10E68 */
  0x0000,   /* U+10E69 */
  0x0000,   /* U+10E6A */
  0x0000,   /* U+10E6B */
  0x0000,   /* U+10E6C */
  0x0000,   /* U+10E6D */
  0x0000,   /* U+10E6E */
  0x0000,   /* U+10E6F */
  0x0000,   /* U+10E70 */
  0x0000,   /* U+10E71 */
  0x0000,   /* U+10E72 */
  0x0000,   /* U+10E73 */
  0x0000,   /* U+10E74 */
  0x0000,   /* U+10E75 */
  0x0000,   /* U+10E76 */
  0x0000,   /* U+10E77 */
  0x0000,   /* U+10E78 */
  0x0000,   /* U+10E79 */
  0x0000,   /* U+10E7A */
  0x0000,   /* U+10E7B */
  0x0000,   /* U+10E7C */
  0x0000,   /* U+10E7D */
  0x0000,   /* U+10E7E */
  0x0000,   /* U+10E7F */
  0x0000,   /* U+10E80 */
  0x0000,   /* U+10E81 */
  0x0000,   /* U+10E82 */
  0x0000,   /* U+10E83 */
  0x0000,   /* U+10E84 */
  0x0000,   /* U+10E85 */
  0x0000,   /* U+10E86 */
  0x0000,   /* U+10E87 */
  0x0000,   /* U+10E88 */
  0x0000,   /* U+10E89 */
  0x0000,   /* U+10E8A */
  0x0000,   /* U+10E8B */
  0x0000,   /* U+10E8C */
  0x0000,   /* U+10E8D */
  0x0000,   /* U+10E8E */
  0x0000,   /* U+10E8F */
  0x0000,   /* U+10E90 */
  0x0000,   /* U+10E91 */
  0x0000,   /* U+10E92 */
  0x0000,   /* U+10E93 */
  0x0000,   /* U+10E94 */
  0x0000,   /* U+10E95 */
  0x0000,   /* U+10E96 */
  0x0000,   /* U+10E97 */
  0x0000,   /* U+10E98 */
  0x0000,   /* U+10E99 */
  0x0000,   /* U+10E9A */
  0x0000,   /* U+10E9B */
  0x0000,   /* U+10E9C */
  0x0000,   /* U+10E9D */
  0x0000,   /* U+10E9E */
  0x0000,   /* U+10E9F */
  0x0000,   /* U+10EA0 */
  0x0000,   /* U+10EA1 */
  0x0000,   /* U+10EA2 */
  0x0000,   /* U+10EA3 */
  0x0000,   /* U+10EA4 */
  0x0000,   /* U+10EA5 */
  0x0000,   /* U+10EA6 */
  0x0000,   /* U+10EA7 */
  0x0000,   /* U+10EA8 */
  0x0000,   /* U+10EA9 */
  0x0000,   /* U+10EAA */
  0x0000,   /* U+10EAB */
  0x0000,   /* U+10EAC */
  0x0000,   /* U+10EAD */
  0x0000,   /* U+10EAE */
  0x0000,   /* U+10EAF */
  0x0000,   /* U+10EB0 */
  0x0000,   /* U+10EB1 */
  0x0000,   /* U+10EB2 */
  0x0000,   /* U+10EB3 */
  0x0000,   /* U+10EB4 */
  0x0000,   /* U+10EB5 */
  0x0000,   /* U+10EB6 */
  0x0000,   /* U+10EB7 */
  0x0000,   /* U+10EB8 */
  0x0000,   /* U+10EB9 */
  0x0000,   /* U+10EBA */
  0x0000,   /* U+10EBB */
  0x0000,   /* U+10EBC */
  0x0000,   /* U+10EBD */
  0x0000,   /* U+10EBE */
  0x0000,   /* U+10EBF */
  0x0000,   /* U+10EC0 */
  0x0000,   /* U+10EC1 */
  0x0000,   /* U+10EC2 */
  0x0000,   /* U+10EC3 */
  0x0000,   /* U+10EC4 */
  0x0000,   /* U+10EC5 */
  0x0000,   /* U+10EC6 */
  0x0000,   /* U+10EC7 */
  0x0000,   /* U+10EC8 */
  0x0000,   /* U+10EC9 */
  0x0000,   /* U+10ECA */
  0x0000,   /* U+10ECB */
  0x0000,   /* U+10ECC */
  0x0000,   /* U+10ECD */
  0x0000,   /* U+10ECE */
  0x0000,   /* U+10ECF */
  0x0000,   /* U+10ED0 */
  0x0000,   /* U+10ED1 */
  0x0000,   /* U+10ED2 */
  0x0000,   /* U+10ED3 */
  0x0000,   /* U+10ED4 */
  0x0000,   /* U+10ED5 */
  0x0000,   /* U+10ED6 */
  0x0000,   /* U+10ED7 */
  0x0000,   /* U+10ED8 */
  0x0000,   /* U+10ED9 */
  0x0000,   /* U+10EDA */
  0x0000,   /* U+10EDB */
  0x0000,   /* U+10EDC */
  0x0000,   /* U+10EDD */
  0x0000,   /* U+10EDE */
  0x0000,   /* U+10EDF */
  0x0000,   /* U+10EE0 */
  0x0000,   /* U+10EE1 */
  0x0000,   /* U+10EE2 */
  0x0000,   /* U+10EE3 */
  0x0000,   /* U+10EE4 */
  0x0000,   /* U+10EE5 */
  0x0000,   /* U+10EE6 */
  0x0000,   /* U+10EE7 */
  0x0000,   /* U+10EE8 */
  0x0000,   /* U+10EE9 */
  0x0000,   /* U+10EEA */
  0x0000,   /* U+10EEB */
  0x0000,   /* U+10EEC */
  0x0000,   /* U+10EED */
  0x0000,   /* U+10EEE */
  0x0000,   /* U+10EEF */
  0x0000,   /* U+10EF0 */
  0x0000,   /* U+10EF1 */
  0x0000,   /* U+10EF2 */
  0x0000,   /* U+10EF3 */
  0x0000,   /* U+10EF4 */
  0x0000,   /* U+10EF5 */
  0x0000,   /* U+10EF6 */
  0x0000,   /* U+10EF7 */
  0x0000,   /* U+10EF8 */
  0x0000,   /* U+10EF9 */
  0x0000,   /* U+10EFA */
  0x0000,   /* U+10EFB */
  0x0000,   /* U+10EFC */
  0x0000,   /* U+10EFD */
  0x0000,   /* U+10EFE */
  0x0000,   /* U+10EFF */
};

uint16 uca900_p110[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+11000 */
  0x0000,   /* U+11001 */
  0x0000,   /* U+11002 */
  0x2CF7,   /* U+11003 */
  0x2CF8,   /* U+11004 */
  0x2CC8,   /* U+11005 */
  0x2CC9,   /* U+11006 */
  0x2CCA,   /* U+11007 */
  0x2CCB,   /* U+11008 */
  0x2CCC,   /* U+11009 */
  0x2CCD,   /* U+1100A */
  0x2CCE,   /* U+1100B */
  0x2CCF,   /* U+1100C */
  0x2CD0,   /* U+1100D */
  0x2CD1,   /* U+1100E */
  0x2CD2,   /* U+1100F */
  0x2CD3,   /* U+11010 */
  0x2CD4,   /* U+11011 */
  0x2CD5,   /* U+11012 */
  0x2CD6,   /* U+11013 */
  0x2CD7,   /* U+11014 */
  0x2CD8,   /* U+11015 */
  0x2CD9,   /* U+11016 */
  0x2CDA,   /* U+11017 */
  0x2CDB,   /* U+11018 */
  0x2CDC,   /* U+11019 */
  0x2CDD,   /* U+1101A */
  0x2CDE,   /* U+1101B */
  0x2CDF,   /* U+1101C */
  0x2CE0,   /* U+1101D */
  0x2CE1,   /* U+1101E */
  0x2CE2,   /* U+1101F */
  0x2CE3,   /* U+11020 */
  0x2CE4,   /* U+11021 */
  0x2CE5,   /* U+11022 */
  0x2CE6,   /* U+11023 */
  0x2CE7,   /* U+11024 */
  0x2CE8,   /* U+11025 */
  0x2CE9,   /* U+11026 */
  0x2CEA,   /* U+11027 */
  0x2CEB,   /* U+11028 */
  0x2CEC,   /* U+11029 */
  0x2CED,   /* U+1102A */
  0x2CEE,   /* U+1102B */
  0x2CEF,   /* U+1102C */
  0x2CF0,   /* U+1102D */
  0x2CF1,   /* U+1102E */
  0x2CF2,   /* U+1102F */
  0x2CF3,   /* U+11030 */
  0x2CF4,   /* U+11031 */
  0x2CF5,   /* U+11032 */
  0x2CF6,   /* U+11033 */
  0x2CF9,   /* U+11034 */
  0x2CFA,   /* U+11035 */
  0x2CFB,   /* U+11036 */
  0x2CFC,   /* U+11037 */
  0x2CFD,   /* U+11038 */
  0x2CFE,   /* U+11039 */
  0x2CFF,   /* U+1103A */
  0x2D00,   /* U+1103B */
  0x2D01,   /* U+1103C */
  0x2D02,   /* U+1103D */
  0x2D03,   /* U+1103E */
  0x2D04,   /* U+1103F */
  0x2D05,   /* U+11040 */
  0x2D06,   /* U+11041 */
  0x2D07,   /* U+11042 */
  0x2D08,   /* U+11043 */
  0x2D09,   /* U+11044 */
  0x2D0A,   /* U+11045 */
  0x2D0B,   /* U+11046 */
  0x02AC,   /* U+11047 */
  0x02AD,   /* U+11048 */
  0x0433,   /* U+11049 */
  0x0434,   /* U+1104A */
  0x0435,   /* U+1104B */
  0x0436,   /* U+1104C */
  0x0437,   /* U+1104D */
  0xFBC2,   /* U+1104E */
  0xFBC2,   /* U+1104F */
  0xFBC2,   /* U+11050 */
  0xFBC2,   /* U+11051 */
  0x1C3E,   /* U+11052 */
  0x1C3F,   /* U+11053 */
  0x1C40,   /* U+11054 */
  0x1C41,   /* U+11055 */
  0x1C42,   /* U+11056 */
  0x1C43,   /* U+11057 */
  0x1C44,   /* U+11058 */
  0x1C45,   /* U+11059 */
  0x1C46,   /* U+1105A */
  0x1B73,   /* U+1105B */
  0x1B74,   /* U+1105C */
  0x1B75,   /* U+1105D */
  0x1B76,   /* U+1105E */
  0x1B77,   /* U+1105F */
  0x1B78,   /* U+11060 */
  0x1B79,   /* U+11061 */
  0x1B7A,   /* U+11062 */
  0x1B7B,   /* U+11063 */
  0x1B7C,   /* U+11064 */
  0x1B7D,   /* U+11065 */
  0x1C3D,   /* U+11066 */
  0x1C3E,   /* U+11067 */
  0x1C3F,   /* U+11068 */
  0x1C40,   /* U+11069 */
  0x1C41,   /* U+1106A */
  0x1C42,   /* U+1106B */
  0x1C43,   /* U+1106C */
  0x1C44,   /* U+1106D */
  0x1C45,   /* U+1106E */
  0x1C46,   /* U+1106F */
  0xFBC2,   /* U+11070 */
  0xFBC2,   /* U+11071 */
  0xFBC2,   /* U+11072 */
  0xFBC2,   /* U+11073 */
  0xFBC2,   /* U+11074 */
  0xFBC2,   /* U+11075 */
  0xFBC2,   /* U+11076 */
  0xFBC2,   /* U+11077 */
  0xFBC2,   /* U+11078 */
  0xFBC2,   /* U+11079 */
  0xFBC2,   /* U+1107A */
  0xFBC2,   /* U+1107B */
  0xFBC2,   /* U+1107C */
  0xFBC2,   /* U+1107D */
  0xFBC2,   /* U+1107E */
  0x2D0C,   /* U+1107F */
  0x0000,   /* U+11080 */
  0x0000,   /* U+11081 */
  0x0000,   /* U+11082 */
  0x29C6,   /* U+11083 */
  0x29C7,   /* U+11084 */
  0x29C8,   /* U+11085 */
  0x29C9,   /* U+11086 */
  0x29CA,   /* U+11087 */
  0x29CB,   /* U+11088 */
  0x29CC,   /* U+11089 */
  0x29CD,   /* U+1108A */
  0x29CE,   /* U+1108B */
  0x29CF,   /* U+1108C */
  0x29D0,   /* U+1108D */
  0x29D1,   /* U+1108E */
  0x29D2,   /* U+1108F */
  0x29D3,   /* U+11090 */
  0x29D4,   /* U+11091 */
  0x29D5,   /* U+11092 */
  0x29D6,   /* U+11093 */
  0x29D7,   /* U+11094 */
  0x29D8,   /* U+11095 */
  0x29D9,   /* U+11096 */
  0x29DA,   /* U+11097 */
  0x29DB,   /* U+11098 */
  0x29DC,   /* U+11099 */
  0x29DC,   /* U+1109A */
  0x29DD,   /* U+1109B */
  0x29DD,   /* U+1109C */
  0x29DE,   /* U+1109D */
  0x29DF,   /* U+1109E */
  0x29E0,   /* U+1109F */
  0x29E1,   /* U+110A0 */
  0x29E2,   /* U+110A1 */
  0x29E3,   /* U+110A2 */
  0x29E4,   /* U+110A3 */
  0x29E5,   /* U+110A4 */
  0x29E6,   /* U+110A5 */
  0x29E7,   /* U+110A6 */
  0x29E8,   /* U+110A7 */
  0x29E9,   /* U+110A8 */
  0x29EA,   /* U+110A9 */
  0x29EB,   /* U+110AA */
  0x29E6,   /* U+110AB */
  0x29EC,   /* U+110AC */
  0x29ED,   /* U+110AD */
  0x29EE,   /* U+110AE */
  0x29EF,   /* U+110AF */
  0x29F0,   /* U+110B0 */
  0x29F1,   /* U+110B1 */
  0x29F2,   /* U+110B2 */
  0x29F3,   /* U+110B3 */
  0x29F4,   /* U+110B4 */
  0x29F5,   /* U+110B5 */
  0x29F6,   /* U+110B6 */
  0x29F7,   /* U+110B7 */
  0x29F8,   /* U+110B8 */
  0x29F9,   /* U+110B9 */
  0x0000,   /* U+110BA */
  0x044E,   /* U+110BB */
  0x044F,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x02E0,   /* U+110BE */
  0x02E1,   /* U+110BF */
  0x02AE,   /* U+110C0 */
  0x02AF,   /* U+110C1 */
  0xFBC2,   /* U+110C2 */
  0xFBC2,   /* U+110C3 */
  0xFBC2,   /* U+110C4 */
  0xFBC2,   /* U+110C5 */
  0xFBC2,   /* U+110C6 */
  0xFBC2,   /* U+110C7 */
  0xFBC2,   /* U+110C8 */
  0xFBC2,   /* U+110C9 */
  0xFBC2,   /* U+110CA */
  0xFBC2,   /* U+110CB */
  0xFBC2,   /* U+110CC */
  0xFBC2,   /* U+110CD */
  0xFBC2,   /* U+110CE */
  0xFBC2,   /* U+110CF */
  0x45AF,   /* U+110D0 */
  0x45B0,   /* U+110D1 */
  0x45B1,   /* U+110D2 */
  0x45B2,   /* U+110D3 */
  0x45B3,   /* U+110D4 */
  0x45B4,   /* U+110D5 */
  0x45B5,   /* U+110D6 */
  0x45B6,   /* U+110D7 */
  0x45B7,   /* U+110D8 */
  0x45B8,   /* U+110D9 */
  0x45B9,   /* U+110DA */
  0x45BA,   /* U+110DB */
  0x45BB,   /* U+110DC */
  0x45BC,   /* U+110DD */
  0x45BD,   /* U+110DE */
  0x45BE,   /* U+110DF */
  0x45BF,   /* U+110E0 */
  0x45C0,   /* U+110E1 */
  0x45C1,   /* U+110E2 */
  0x45C2,   /* U+110E3 */
  0x45C3,   /* U+110E4 */
  0x45C4,   /* U+110E5 */
  0x45C5,   /* U+110E6 */
  0x45C6,   /* U+110E7 */
  0x45C7,   /* U+110E8 */
  0xFBC2,   /* U+110E9 */
  0xFBC2,   /* U+110EA */
  0xFBC2,   /* U+110EB */
  0xFBC2,   /* U+110EC */
  0xFBC2,   /* U+110ED */
  0xFBC2,   /* U+110EE */
  0xFBC2,   /* U+110EF */
  0x1C3D,   /* U+110F0 */
  0x1C3E,   /* U+110F1 */
  0x1C3F,   /* U+110F2 */
  0x1C40,   /* U+110F3 */
  0x1C41,   /* U+110F4 */
  0x1C42,   /* U+110F5 */
  0x1C43,   /* U+110F6 */
  0x1C44,   /* U+110F7 */
  0x1C45,   /* U+110F8 */
  0x1C46,   /* U+110F9 */
  0xFBC2,   /* U+110FA */
  0xFBC2,   /* U+110FB */
  0xFBC2,   /* U+110FC */
  0xFBC2,   /* U+110FD */
  0xFBC2,   /* U+110FE */
  0xFBC2,   /* U+110FF */

  /* Secondary weight 1 for each character. */
  0x00C3,   /* U+11000 */
  0x00C4,   /* U+11001 */
  0x00C5,   /* U+11002 */
  0x0020,   /* U+11003 */
  0x0020,   /* U+11004 */
  0x0020,   /* U+11005 */
  0x0020,   /* U+11006 */
  0x0020,   /* U+11007 */
  0x0020,   /* U+11008 */
  0x0020,   /* U+11009 */
  0x0020,   /* U+1100A */
  0x0020,   /* U+1100B */
  0x0020,   /* U+1100C */
  0x0020,   /* U+1100D */
  0x0020,   /* U+1100E */
  0x0020,   /* U+1100F */
  0x0020,   /* U+11010 */
  0x0020,   /* U+11011 */
  0x0020,   /* U+11012 */
  0x0020,   /* U+11013 */
  0x0020,   /* U+11014 */
  0x0020,   /* U+11015 */
  0x0020,   /* U+11016 */
  0x0020,   /* U+11017 */
  0x0020,   /* U+11018 */
  0x0020,   /* U+11019 */
  0x0020,   /* U+1101A */
  0x0020,   /* U+1101B */
  0x0020,   /* U+1101C */
  0x0020,   /* U+1101D */
  0x0020,   /* U+1101E */
  0x0020,   /* U+1101F */
  0x0020,   /* U+11020 */
  0x0020,   /* U+11021 */
  0x0020,   /* U+11022 */
  0x0020,   /* U+11023 */
  0x0020,   /* U+11024 */
  0x0020,   /* U+11025 */
  0x0020,   /* U+11026 */
  0x0020,   /* U+11027 */
  0x0020,   /* U+11028 */
  0x0020,   /* U+11029 */
  0x0020,   /* U+1102A */
  0x0020,   /* U+1102B */
  0x0020,   /* U+1102C */
  0x0020,   /* U+1102D */
  0x0020,   /* U+1102E */
  0x0020,   /* U+1102F */
  0x0020,   /* U+11030 */
  0x0020,   /* U+11031 */
  0x0020,   /* U+11032 */
  0x0020,   /* U+11033 */
  0x0020,   /* U+11034 */
  0x0020,   /* U+11035 */
  0x0020,   /* U+11036 */
  0x0020,   /* U+11037 */
  0x0020,   /* U+11038 */
  0x0020,   /* U+11039 */
  0x0020,   /* U+1103A */
  0x0020,   /* U+1103B */
  0x0020,   /* U+1103C */
  0x0020,   /* U+1103D */
  0x0020,   /* U+1103E */
  0x0020,   /* U+1103F */
  0x0020,   /* U+11040 */
  0x0020,   /* U+11041 */
  0x0020,   /* U+11042 */
  0x0020,   /* U+11043 */
  0x0020,   /* U+11044 */
  0x0020,   /* U+11045 */
  0x0020,   /* U+11046 */
  0x0020,   /* U+11047 */
  0x0020,   /* U+11048 */
  0x0020,   /* U+11049 */
  0x0020,   /* U+1104A */
  0x0020,   /* U+1104B */
  0x0020,   /* U+1104C */
  0x0020,   /* U+1104D */
  0x0020,   /* U+1104E */
  0x0020,   /* U+1104F */
  0x0020,   /* U+11050 */
  0x0020,   /* U+11051 */
  0x0020,   /* U+11052 */
  0x0020,   /* U+11053 */
  0x0020,   /* U+11054 */
  0x0020,   /* U+11055 */
  0x0020,   /* U+11056 */
  0x0020,   /* U+11057 */
  0x0020,   /* U+11058 */
  0x0020,   /* U+11059 */
  0x0020,   /* U+1105A */
  0x0020,   /* U+1105B */
  0x0020,   /* U+1105C */
  0x0020,   /* U+1105D */
  0x0020,   /* U+1105E */
  0x0020,   /* U+1105F */
  0x0020,   /* U+11060 */
  0x0020,   /* U+11061 */
  0x0020,   /* U+11062 */
  0x0020,   /* U+11063 */
  0x0020,   /* U+11064 */
  0x0020,   /* U+11065 */
  0x0020,   /* U+11066 */
  0x0020,   /* U+11067 */
  0x0020,   /* U+11068 */
  0x0020,   /* U+11069 */
  0x0020,   /* U+1106A */
  0x0020,   /* U+1106B */
  0x0020,   /* U+1106C */
  0x0020,   /* U+1106D */
  0x0020,   /* U+1106E */
  0x0020,   /* U+1106F */
  0x0020,   /* U+11070 */
  0x0020,   /* U+11071 */
  0x0020,   /* U+11072 */
  0x0020,   /* U+11073 */
  0x0020,   /* U+11074 */
  0x0020,   /* U+11075 */
  0x0020,   /* U+11076 */
  0x0020,   /* U+11077 */
  0x0020,   /* U+11078 */
  0x0020,   /* U+11079 */
  0x0020,   /* U+1107A */
  0x0020,   /* U+1107B */
  0x0020,   /* U+1107C */
  0x0020,   /* U+1107D */
  0x0020,   /* U+1107E */
  0x0020,   /* U+1107F */
  0x00C3,   /* U+11080 */
  0x00C4,   /* U+11081 */
  0x00C5,   /* U+11082 */
  0x0020,   /* U+11083 */
  0x0020,   /* U+11084 */
  0x0020,   /* U+11085 */
  0x0020,   /* U+11086 */
  0x0020,   /* U+11087 */
  0x0020,   /* U+11088 */
  0x0020,   /* U+11089 */
  0x0020,   /* U+1108A */
  0x0020,   /* U+1108B */
  0x0020,   /* U+1108C */
  0x0020,   /* U+1108D */
  0x0020,   /* U+1108E */
  0x0020,   /* U+1108F */
  0x0020,   /* U+11090 */
  0x0020,   /* U+11091 */
  0x0020,   /* U+11092 */
  0x0020,   /* U+11093 */
  0x0020,   /* U+11094 */
  0x0020,   /* U+11095 */
  0x0020,   /* U+11096 */
  0x0020,   /* U+11097 */
  0x0020,   /* U+11098 */
  0x0020,   /* U+11099 */
  0x0020,   /* U+1109A */
  0x0020,   /* U+1109B */
  0x0020,   /* U+1109C */
  0x0020,   /* U+1109D */
  0x0020,   /* U+1109E */
  0x0020,   /* U+1109F */
  0x0020,   /* U+110A0 */
  0x0020,   /* U+110A1 */
  0x0020,   /* U+110A2 */
  0x0020,   /* U+110A3 */
  0x0020,   /* U+110A4 */
  0x0020,   /* U+110A5 */
  0x0020,   /* U+110A6 */
  0x0020,   /* U+110A7 */
  0x0020,   /* U+110A8 */
  0x0020,   /* U+110A9 */
  0x0020,   /* U+110AA */
  0x0020,   /* U+110AB */
  0x0020,   /* U+110AC */
  0x0020,   /* U+110AD */
  0x0020,   /* U+110AE */
  0x0020,   /* U+110AF */
  0x0020,   /* U+110B0 */
  0x0020,   /* U+110B1 */
  0x0020,   /* U+110B2 */
  0x0020,   /* U+110B3 */
  0x0020,   /* U+110B4 */
  0x0020,   /* U+110B5 */
  0x0020,   /* U+110B6 */
  0x0020,   /* U+110B7 */
  0x0020,   /* U+110B8 */
  0x0020,   /* U+110B9 */
  0x00C2,   /* U+110BA */
  0x0020,   /* U+110BB */
  0x0020,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x0020,   /* U+110BE */
  0x0020,   /* U+110BF */
  0x0020,   /* U+110C0 */
  0x0020,   /* U+110C1 */
  0x0020,   /* U+110C2 */
  0x0020,   /* U+110C3 */
  0x0020,   /* U+110C4 */
  0x0020,   /* U+110C5 */
  0x0020,   /* U+110C6 */
  0x0020,   /* U+110C7 */
  0x0020,   /* U+110C8 */
  0x0020,   /* U+110C9 */
  0x0020,   /* U+110CA */
  0x0020,   /* U+110CB */
  0x0020,   /* U+110CC */
  0x0020,   /* U+110CD */
  0x0020,   /* U+110CE */
  0x0020,   /* U+110CF */
  0x0020,   /* U+110D0 */
  0x0020,   /* U+110D1 */
  0x0020,   /* U+110D2 */
  0x0020,   /* U+110D3 */
  0x0020,   /* U+110D4 */
  0x0020,   /* U+110D5 */
  0x0020,   /* U+110D6 */
  0x0020,   /* U+110D7 */
  0x0020,   /* U+110D8 */
  0x0020,   /* U+110D9 */
  0x0020,   /* U+110DA */
  0x0020,   /* U+110DB */
  0x0020,   /* U+110DC */
  0x0020,   /* U+110DD */
  0x0020,   /* U+110DE */
  0x0020,   /* U+110DF */
  0x0020,   /* U+110E0 */
  0x0020,   /* U+110E1 */
  0x0020,   /* U+110E2 */
  0x0020,   /* U+110E3 */
  0x0020,   /* U+110E4 */
  0x0020,   /* U+110E5 */
  0x0020,   /* U+110E6 */
  0x0020,   /* U+110E7 */
  0x0020,   /* U+110E8 */
  0x0020,   /* U+110E9 */
  0x0020,   /* U+110EA */
  0x0020,   /* U+110EB */
  0x0020,   /* U+110EC */
  0x0020,   /* U+110ED */
  0x0020,   /* U+110EE */
  0x0020,   /* U+110EF */
  0x0020,   /* U+110F0 */
  0x0020,   /* U+110F1 */
  0x0020,   /* U+110F2 */
  0x0020,   /* U+110F3 */
  0x0020,   /* U+110F4 */
  0x0020,   /* U+110F5 */
  0x0020,   /* U+110F6 */
  0x0020,   /* U+110F7 */
  0x0020,   /* U+110F8 */
  0x0020,   /* U+110F9 */
  0x0020,   /* U+110FA */
  0x0020,   /* U+110FB */
  0x0020,   /* U+110FC */
  0x0020,   /* U+110FD */
  0x0020,   /* U+110FE */
  0x0020,   /* U+110FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11000 */
  0x0002,   /* U+11001 */
  0x0002,   /* U+11002 */
  0x0002,   /* U+11003 */
  0x0002,   /* U+11004 */
  0x0002,   /* U+11005 */
  0x0002,   /* U+11006 */
  0x0002,   /* U+11007 */
  0x0002,   /* U+11008 */
  0x0002,   /* U+11009 */
  0x0002,   /* U+1100A */
  0x0002,   /* U+1100B */
  0x0002,   /* U+1100C */
  0x0002,   /* U+1100D */
  0x0002,   /* U+1100E */
  0x0002,   /* U+1100F */
  0x0002,   /* U+11010 */
  0x0002,   /* U+11011 */
  0x0002,   /* U+11012 */
  0x0002,   /* U+11013 */
  0x0002,   /* U+11014 */
  0x0002,   /* U+11015 */
  0x0002,   /* U+11016 */
  0x0002,   /* U+11017 */
  0x0002,   /* U+11018 */
  0x0002,   /* U+11019 */
  0x0002,   /* U+1101A */
  0x0002,   /* U+1101B */
  0x0002,   /* U+1101C */
  0x0002,   /* U+1101D */
  0x0002,   /* U+1101E */
  0x0002,   /* U+1101F */
  0x0002,   /* U+11020 */
  0x0002,   /* U+11021 */
  0x0002,   /* U+11022 */
  0x0002,   /* U+11023 */
  0x0002,   /* U+11024 */
  0x0002,   /* U+11025 */
  0x0002,   /* U+11026 */
  0x0002,   /* U+11027 */
  0x0002,   /* U+11028 */
  0x0002,   /* U+11029 */
  0x0002,   /* U+1102A */
  0x0002,   /* U+1102B */
  0x0002,   /* U+1102C */
  0x0002,   /* U+1102D */
  0x0002,   /* U+1102E */
  0x0002,   /* U+1102F */
  0x0002,   /* U+11030 */
  0x0002,   /* U+11031 */
  0x0002,   /* U+11032 */
  0x0002,   /* U+11033 */
  0x0002,   /* U+11034 */
  0x0002,   /* U+11035 */
  0x0002,   /* U+11036 */
  0x0002,   /* U+11037 */
  0x0002,   /* U+11038 */
  0x0002,   /* U+11039 */
  0x0002,   /* U+1103A */
  0x0002,   /* U+1103B */
  0x0002,   /* U+1103C */
  0x0002,   /* U+1103D */
  0x0002,   /* U+1103E */
  0x0002,   /* U+1103F */
  0x0002,   /* U+11040 */
  0x0002,   /* U+11041 */
  0x0002,   /* U+11042 */
  0x0002,   /* U+11043 */
  0x0002,   /* U+11044 */
  0x0002,   /* U+11045 */
  0x0002,   /* U+11046 */
  0x0002,   /* U+11047 */
  0x0002,   /* U+11048 */
  0x0002,   /* U+11049 */
  0x0002,   /* U+1104A */
  0x0002,   /* U+1104B */
  0x0002,   /* U+1104C */
  0x0002,   /* U+1104D */
  0x0002,   /* U+1104E */
  0x0002,   /* U+1104F */
  0x0002,   /* U+11050 */
  0x0002,   /* U+11051 */
  0x0002,   /* U+11052 */
  0x0002,   /* U+11053 */
  0x0002,   /* U+11054 */
  0x0002,   /* U+11055 */
  0x0002,   /* U+11056 */
  0x0002,   /* U+11057 */
  0x0002,   /* U+11058 */
  0x0002,   /* U+11059 */
  0x0002,   /* U+1105A */
  0x0002,   /* U+1105B */
  0x0002,   /* U+1105C */
  0x0002,   /* U+1105D */
  0x0002,   /* U+1105E */
  0x0002,   /* U+1105F */
  0x0002,   /* U+11060 */
  0x0002,   /* U+11061 */
  0x0002,   /* U+11062 */
  0x0002,   /* U+11063 */
  0x0002,   /* U+11064 */
  0x0002,   /* U+11065 */
  0x0002,   /* U+11066 */
  0x0002,   /* U+11067 */
  0x0002,   /* U+11068 */
  0x0002,   /* U+11069 */
  0x0002,   /* U+1106A */
  0x0002,   /* U+1106B */
  0x0002,   /* U+1106C */
  0x0002,   /* U+1106D */
  0x0002,   /* U+1106E */
  0x0002,   /* U+1106F */
  0x0002,   /* U+11070 */
  0x0002,   /* U+11071 */
  0x0002,   /* U+11072 */
  0x0002,   /* U+11073 */
  0x0002,   /* U+11074 */
  0x0002,   /* U+11075 */
  0x0002,   /* U+11076 */
  0x0002,   /* U+11077 */
  0x0002,   /* U+11078 */
  0x0002,   /* U+11079 */
  0x0002,   /* U+1107A */
  0x0002,   /* U+1107B */
  0x0002,   /* U+1107C */
  0x0002,   /* U+1107D */
  0x0002,   /* U+1107E */
  0x0002,   /* U+1107F */
  0x0002,   /* U+11080 */
  0x0002,   /* U+11081 */
  0x0002,   /* U+11082 */
  0x0002,   /* U+11083 */
  0x0002,   /* U+11084 */
  0x0002,   /* U+11085 */
  0x0002,   /* U+11086 */
  0x0002,   /* U+11087 */
  0x0002,   /* U+11088 */
  0x0002,   /* U+11089 */
  0x0002,   /* U+1108A */
  0x0002,   /* U+1108B */
  0x0002,   /* U+1108C */
  0x0002,   /* U+1108D */
  0x0002,   /* U+1108E */
  0x0002,   /* U+1108F */
  0x0002,   /* U+11090 */
  0x0002,   /* U+11091 */
  0x0002,   /* U+11092 */
  0x0002,   /* U+11093 */
  0x0002,   /* U+11094 */
  0x0002,   /* U+11095 */
  0x0002,   /* U+11096 */
  0x0002,   /* U+11097 */
  0x0002,   /* U+11098 */
  0x0002,   /* U+11099 */
  0x0002,   /* U+1109A */
  0x0002,   /* U+1109B */
  0x0002,   /* U+1109C */
  0x0002,   /* U+1109D */
  0x0002,   /* U+1109E */
  0x0002,   /* U+1109F */
  0x0002,   /* U+110A0 */
  0x0002,   /* U+110A1 */
  0x0002,   /* U+110A2 */
  0x0002,   /* U+110A3 */
  0x0002,   /* U+110A4 */
  0x0002,   /* U+110A5 */
  0x0002,   /* U+110A6 */
  0x0002,   /* U+110A7 */
  0x0002,   /* U+110A8 */
  0x0002,   /* U+110A9 */
  0x0002,   /* U+110AA */
  0x0002,   /* U+110AB */
  0x0002,   /* U+110AC */
  0x0002,   /* U+110AD */
  0x0002,   /* U+110AE */
  0x0002,   /* U+110AF */
  0x0002,   /* U+110B0 */
  0x0002,   /* U+110B1 */
  0x0002,   /* U+110B2 */
  0x0002,   /* U+110B3 */
  0x0002,   /* U+110B4 */
  0x0002,   /* U+110B5 */
  0x0002,   /* U+110B6 */
  0x0002,   /* U+110B7 */
  0x0002,   /* U+110B8 */
  0x0002,   /* U+110B9 */
  0x0002,   /* U+110BA */
  0x0002,   /* U+110BB */
  0x0002,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x0002,   /* U+110BE */
  0x0002,   /* U+110BF */
  0x0002,   /* U+110C0 */
  0x0002,   /* U+110C1 */
  0x0002,   /* U+110C2 */
  0x0002,   /* U+110C3 */
  0x0002,   /* U+110C4 */
  0x0002,   /* U+110C5 */
  0x0002,   /* U+110C6 */
  0x0002,   /* U+110C7 */
  0x0002,   /* U+110C8 */
  0x0002,   /* U+110C9 */
  0x0002,   /* U+110CA */
  0x0002,   /* U+110CB */
  0x0002,   /* U+110CC */
  0x0002,   /* U+110CD */
  0x0002,   /* U+110CE */
  0x0002,   /* U+110CF */
  0x0002,   /* U+110D0 */
  0x0002,   /* U+110D1 */
  0x0002,   /* U+110D2 */
  0x0002,   /* U+110D3 */
  0x0002,   /* U+110D4 */
  0x0002,   /* U+110D5 */
  0x0002,   /* U+110D6 */
  0x0002,   /* U+110D7 */
  0x0002,   /* U+110D8 */
  0x0002,   /* U+110D9 */
  0x0002,   /* U+110DA */
  0x0002,   /* U+110DB */
  0x0002,   /* U+110DC */
  0x0002,   /* U+110DD */
  0x0002,   /* U+110DE */
  0x0002,   /* U+110DF */
  0x0002,   /* U+110E0 */
  0x0002,   /* U+110E1 */
  0x0002,   /* U+110E2 */
  0x0002,   /* U+110E3 */
  0x0002,   /* U+110E4 */
  0x0002,   /* U+110E5 */
  0x0002,   /* U+110E6 */
  0x0002,   /* U+110E7 */
  0x0002,   /* U+110E8 */
  0x0002,   /* U+110E9 */
  0x0002,   /* U+110EA */
  0x0002,   /* U+110EB */
  0x0002,   /* U+110EC */
  0x0002,   /* U+110ED */
  0x0002,   /* U+110EE */
  0x0002,   /* U+110EF */
  0x0002,   /* U+110F0 */
  0x0002,   /* U+110F1 */
  0x0002,   /* U+110F2 */
  0x0002,   /* U+110F3 */
  0x0002,   /* U+110F4 */
  0x0002,   /* U+110F5 */
  0x0002,   /* U+110F6 */
  0x0002,   /* U+110F7 */
  0x0002,   /* U+110F8 */
  0x0002,   /* U+110F9 */
  0x0002,   /* U+110FA */
  0x0002,   /* U+110FB */
  0x0002,   /* U+110FC */
  0x0002,   /* U+110FD */
  0x0002,   /* U+110FE */
  0x0002,   /* U+110FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11000 */
  0x0000,   /* U+11001 */
  0x0000,   /* U+11002 */
  0x0000,   /* U+11003 */
  0x0000,   /* U+11004 */
  0x0000,   /* U+11005 */
  0x0000,   /* U+11006 */
  0x0000,   /* U+11007 */
  0x0000,   /* U+11008 */
  0x0000,   /* U+11009 */
  0x0000,   /* U+1100A */
  0x0000,   /* U+1100B */
  0x0000,   /* U+1100C */
  0x0000,   /* U+1100D */
  0x0000,   /* U+1100E */
  0x0000,   /* U+1100F */
  0x0000,   /* U+11010 */
  0x0000,   /* U+11011 */
  0x0000,   /* U+11012 */
  0x0000,   /* U+11013 */
  0x0000,   /* U+11014 */
  0x0000,   /* U+11015 */
  0x0000,   /* U+11016 */
  0x0000,   /* U+11017 */
  0x0000,   /* U+11018 */
  0x0000,   /* U+11019 */
  0x0000,   /* U+1101A */
  0x0000,   /* U+1101B */
  0x0000,   /* U+1101C */
  0x0000,   /* U+1101D */
  0x0000,   /* U+1101E */
  0x0000,   /* U+1101F */
  0x0000,   /* U+11020 */
  0x0000,   /* U+11021 */
  0x0000,   /* U+11022 */
  0x0000,   /* U+11023 */
  0x0000,   /* U+11024 */
  0x0000,   /* U+11025 */
  0x0000,   /* U+11026 */
  0x0000,   /* U+11027 */
  0x0000,   /* U+11028 */
  0x0000,   /* U+11029 */
  0x0000,   /* U+1102A */
  0x0000,   /* U+1102B */
  0x0000,   /* U+1102C */
  0x0000,   /* U+1102D */
  0x0000,   /* U+1102E */
  0x0000,   /* U+1102F */
  0x0000,   /* U+11030 */
  0x0000,   /* U+11031 */
  0x0000,   /* U+11032 */
  0x0000,   /* U+11033 */
  0x0000,   /* U+11034 */
  0x0000,   /* U+11035 */
  0x0000,   /* U+11036 */
  0x0000,   /* U+11037 */
  0x0000,   /* U+11038 */
  0x0000,   /* U+11039 */
  0x0000,   /* U+1103A */
  0x0000,   /* U+1103B */
  0x0000,   /* U+1103C */
  0x0000,   /* U+1103D */
  0x0000,   /* U+1103E */
  0x0000,   /* U+1103F */
  0x0000,   /* U+11040 */
  0x0000,   /* U+11041 */
  0x0000,   /* U+11042 */
  0x0000,   /* U+11043 */
  0x0000,   /* U+11044 */
  0x0000,   /* U+11045 */
  0x0000,   /* U+11046 */
  0x0000,   /* U+11047 */
  0x0000,   /* U+11048 */
  0x0000,   /* U+11049 */
  0x0000,   /* U+1104A */
  0x0000,   /* U+1104B */
  0x0000,   /* U+1104C */
  0x0000,   /* U+1104D */
  0x904E,   /* U+1104E */
  0x904F,   /* U+1104F */
  0x9050,   /* U+11050 */
  0x9051,   /* U+11051 */
  0x0000,   /* U+11052 */
  0x0000,   /* U+11053 */
  0x0000,   /* U+11054 */
  0x0000,   /* U+11055 */
  0x0000,   /* U+11056 */
  0x0000,   /* U+11057 */
  0x0000,   /* U+11058 */
  0x0000,   /* U+11059 */
  0x0000,   /* U+1105A */
  0x0000,   /* U+1105B */
  0x0000,   /* U+1105C */
  0x0000,   /* U+1105D */
  0x0000,   /* U+1105E */
  0x0000,   /* U+1105F */
  0x0000,   /* U+11060 */
  0x0000,   /* U+11061 */
  0x0000,   /* U+11062 */
  0x0000,   /* U+11063 */
  0x0000,   /* U+11064 */
  0x0000,   /* U+11065 */
  0x0000,   /* U+11066 */
  0x0000,   /* U+11067 */
  0x0000,   /* U+11068 */
  0x0000,   /* U+11069 */
  0x0000,   /* U+1106A */
  0x0000,   /* U+1106B */
  0x0000,   /* U+1106C */
  0x0000,   /* U+1106D */
  0x0000,   /* U+1106E */
  0x0000,   /* U+1106F */
  0x9070,   /* U+11070 */
  0x9071,   /* U+11071 */
  0x9072,   /* U+11072 */
  0x9073,   /* U+11073 */
  0x9074,   /* U+11074 */
  0x9075,   /* U+11075 */
  0x9076,   /* U+11076 */
  0x9077,   /* U+11077 */
  0x9078,   /* U+11078 */
  0x9079,   /* U+11079 */
  0x907A,   /* U+1107A */
  0x907B,   /* U+1107B */
  0x907C,   /* U+1107C */
  0x907D,   /* U+1107D */
  0x907E,   /* U+1107E */
  0x0000,   /* U+1107F */
  0x0000,   /* U+11080 */
  0x0000,   /* U+11081 */
  0x0000,   /* U+11082 */
  0x0000,   /* U+11083 */
  0x0000,   /* U+11084 */
  0x0000,   /* U+11085 */
  0x0000,   /* U+11086 */
  0x0000,   /* U+11087 */
  0x0000,   /* U+11088 */
  0x0000,   /* U+11089 */
  0x0000,   /* U+1108A */
  0x0000,   /* U+1108B */
  0x0000,   /* U+1108C */
  0x0000,   /* U+1108D */
  0x0000,   /* U+1108E */
  0x0000,   /* U+1108F */
  0x0000,   /* U+11090 */
  0x0000,   /* U+11091 */
  0x0000,   /* U+11092 */
  0x0000,   /* U+11093 */
  0x0000,   /* U+11094 */
  0x0000,   /* U+11095 */
  0x0000,   /* U+11096 */
  0x0000,   /* U+11097 */
  0x0000,   /* U+11098 */
  0x0000,   /* U+11099 */
  0x0000,   /* U+1109A */
  0x0000,   /* U+1109B */
  0x0000,   /* U+1109C */
  0x0000,   /* U+1109D */
  0x0000,   /* U+1109E */
  0x0000,   /* U+1109F */
  0x0000,   /* U+110A0 */
  0x0000,   /* U+110A1 */
  0x0000,   /* U+110A2 */
  0x0000,   /* U+110A3 */
  0x0000,   /* U+110A4 */
  0x0000,   /* U+110A5 */
  0x0000,   /* U+110A6 */
  0x0000,   /* U+110A7 */
  0x0000,   /* U+110A8 */
  0x0000,   /* U+110A9 */
  0x0000,   /* U+110AA */
  0x0000,   /* U+110AB */
  0x0000,   /* U+110AC */
  0x0000,   /* U+110AD */
  0x0000,   /* U+110AE */
  0x0000,   /* U+110AF */
  0x0000,   /* U+110B0 */
  0x0000,   /* U+110B1 */
  0x0000,   /* U+110B2 */
  0x0000,   /* U+110B3 */
  0x0000,   /* U+110B4 */
  0x0000,   /* U+110B5 */
  0x0000,   /* U+110B6 */
  0x0000,   /* U+110B7 */
  0x0000,   /* U+110B8 */
  0x0000,   /* U+110B9 */
  0x0000,   /* U+110BA */
  0x0000,   /* U+110BB */
  0x0000,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x0000,   /* U+110BE */
  0x0000,   /* U+110BF */
  0x0000,   /* U+110C0 */
  0x0000,   /* U+110C1 */
  0x90C2,   /* U+110C2 */
  0x90C3,   /* U+110C3 */
  0x90C4,   /* U+110C4 */
  0x90C5,   /* U+110C5 */
  0x90C6,   /* U+110C6 */
  0x90C7,   /* U+110C7 */
  0x90C8,   /* U+110C8 */
  0x90C9,   /* U+110C9 */
  0x90CA,   /* U+110CA */
  0x90CB,   /* U+110CB */
  0x90CC,   /* U+110CC */
  0x90CD,   /* U+110CD */
  0x90CE,   /* U+110CE */
  0x90CF,   /* U+110CF */
  0x0000,   /* U+110D0 */
  0x0000,   /* U+110D1 */
  0x0000,   /* U+110D2 */
  0x0000,   /* U+110D3 */
  0x0000,   /* U+110D4 */
  0x0000,   /* U+110D5 */
  0x0000,   /* U+110D6 */
  0x0000,   /* U+110D7 */
  0x0000,   /* U+110D8 */
  0x0000,   /* U+110D9 */
  0x0000,   /* U+110DA */
  0x0000,   /* U+110DB */
  0x0000,   /* U+110DC */
  0x0000,   /* U+110DD */
  0x0000,   /* U+110DE */
  0x0000,   /* U+110DF */
  0x0000,   /* U+110E0 */
  0x0000,   /* U+110E1 */
  0x0000,   /* U+110E2 */
  0x0000,   /* U+110E3 */
  0x0000,   /* U+110E4 */
  0x0000,   /* U+110E5 */
  0x0000,   /* U+110E6 */
  0x0000,   /* U+110E7 */
  0x0000,   /* U+110E8 */
  0x90E9,   /* U+110E9 */
  0x90EA,   /* U+110EA */
  0x90EB,   /* U+110EB */
  0x90EC,   /* U+110EC */
  0x90ED,   /* U+110ED */
  0x90EE,   /* U+110EE */
  0x90EF,   /* U+110EF */
  0x0000,   /* U+110F0 */
  0x0000,   /* U+110F1 */
  0x0000,   /* U+110F2 */
  0x0000,   /* U+110F3 */
  0x0000,   /* U+110F4 */
  0x0000,   /* U+110F5 */
  0x0000,   /* U+110F6 */
  0x0000,   /* U+110F7 */
  0x0000,   /* U+110F8 */
  0x0000,   /* U+110F9 */
  0x90FA,   /* U+110FA */
  0x90FB,   /* U+110FB */
  0x90FC,   /* U+110FC */
  0x90FD,   /* U+110FD */
  0x90FE,   /* U+110FE */
  0x90FF,   /* U+110FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11000 */
  0x0000,   /* U+11001 */
  0x0000,   /* U+11002 */
  0x0000,   /* U+11003 */
  0x0000,   /* U+11004 */
  0x0000,   /* U+11005 */
  0x0000,   /* U+11006 */
  0x0000,   /* U+11007 */
  0x0000,   /* U+11008 */
  0x0000,   /* U+11009 */
  0x0000,   /* U+1100A */
  0x0000,   /* U+1100B */
  0x0000,   /* U+1100C */
  0x0000,   /* U+1100D */
  0x0000,   /* U+1100E */
  0x0000,   /* U+1100F */
  0x0000,   /* U+11010 */
  0x0000,   /* U+11011 */
  0x0000,   /* U+11012 */
  0x0000,   /* U+11013 */
  0x0000,   /* U+11014 */
  0x0000,   /* U+11015 */
  0x0000,   /* U+11016 */
  0x0000,   /* U+11017 */
  0x0000,   /* U+11018 */
  0x0000,   /* U+11019 */
  0x0000,   /* U+1101A */
  0x0000,   /* U+1101B */
  0x0000,   /* U+1101C */
  0x0000,   /* U+1101D */
  0x0000,   /* U+1101E */
  0x0000,   /* U+1101F */
  0x0000,   /* U+11020 */
  0x0000,   /* U+11021 */
  0x0000,   /* U+11022 */
  0x0000,   /* U+11023 */
  0x0000,   /* U+11024 */
  0x0000,   /* U+11025 */
  0x0000,   /* U+11026 */
  0x0000,   /* U+11027 */
  0x0000,   /* U+11028 */
  0x0000,   /* U+11029 */
  0x0000,   /* U+1102A */
  0x0000,   /* U+1102B */
  0x0000,   /* U+1102C */
  0x0000,   /* U+1102D */
  0x0000,   /* U+1102E */
  0x0000,   /* U+1102F */
  0x0000,   /* U+11030 */
  0x0000,   /* U+11031 */
  0x0000,   /* U+11032 */
  0x0000,   /* U+11033 */
  0x0000,   /* U+11034 */
  0x0000,   /* U+11035 */
  0x0000,   /* U+11036 */
  0x0000,   /* U+11037 */
  0x0000,   /* U+11038 */
  0x0000,   /* U+11039 */
  0x0000,   /* U+1103A */
  0x0000,   /* U+1103B */
  0x0000,   /* U+1103C */
  0x0000,   /* U+1103D */
  0x0000,   /* U+1103E */
  0x0000,   /* U+1103F */
  0x0000,   /* U+11040 */
  0x0000,   /* U+11041 */
  0x0000,   /* U+11042 */
  0x0000,   /* U+11043 */
  0x0000,   /* U+11044 */
  0x0000,   /* U+11045 */
  0x0000,   /* U+11046 */
  0x0000,   /* U+11047 */
  0x0000,   /* U+11048 */
  0x0000,   /* U+11049 */
  0x0000,   /* U+1104A */
  0x0000,   /* U+1104B */
  0x0000,   /* U+1104C */
  0x0000,   /* U+1104D */
  0x0000,   /* U+1104E */
  0x0000,   /* U+1104F */
  0x0000,   /* U+11050 */
  0x0000,   /* U+11051 */
  0x0000,   /* U+11052 */
  0x0000,   /* U+11053 */
  0x0000,   /* U+11054 */
  0x0000,   /* U+11055 */
  0x0000,   /* U+11056 */
  0x0000,   /* U+11057 */
  0x0000,   /* U+11058 */
  0x0000,   /* U+11059 */
  0x0000,   /* U+1105A */
  0x0000,   /* U+1105B */
  0x0000,   /* U+1105C */
  0x0000,   /* U+1105D */
  0x0000,   /* U+1105E */
  0x0000,   /* U+1105F */
  0x0000,   /* U+11060 */
  0x0000,   /* U+11061 */
  0x0000,   /* U+11062 */
  0x0000,   /* U+11063 */
  0x0000,   /* U+11064 */
  0x0000,   /* U+11065 */
  0x0000,   /* U+11066 */
  0x0000,   /* U+11067 */
  0x0000,   /* U+11068 */
  0x0000,   /* U+11069 */
  0x0000,   /* U+1106A */
  0x0000,   /* U+1106B */
  0x0000,   /* U+1106C */
  0x0000,   /* U+1106D */
  0x0000,   /* U+1106E */
  0x0000,   /* U+1106F */
  0x0000,   /* U+11070 */
  0x0000,   /* U+11071 */
  0x0000,   /* U+11072 */
  0x0000,   /* U+11073 */
  0x0000,   /* U+11074 */
  0x0000,   /* U+11075 */
  0x0000,   /* U+11076 */
  0x0000,   /* U+11077 */
  0x0000,   /* U+11078 */
  0x0000,   /* U+11079 */
  0x0000,   /* U+1107A */
  0x0000,   /* U+1107B */
  0x0000,   /* U+1107C */
  0x0000,   /* U+1107D */
  0x0000,   /* U+1107E */
  0x0000,   /* U+1107F */
  0x0000,   /* U+11080 */
  0x0000,   /* U+11081 */
  0x0000,   /* U+11082 */
  0x0000,   /* U+11083 */
  0x0000,   /* U+11084 */
  0x0000,   /* U+11085 */
  0x0000,   /* U+11086 */
  0x0000,   /* U+11087 */
  0x0000,   /* U+11088 */
  0x0000,   /* U+11089 */
  0x0000,   /* U+1108A */
  0x0000,   /* U+1108B */
  0x0000,   /* U+1108C */
  0x0000,   /* U+1108D */
  0x0000,   /* U+1108E */
  0x0000,   /* U+1108F */
  0x0000,   /* U+11090 */
  0x0000,   /* U+11091 */
  0x0000,   /* U+11092 */
  0x0000,   /* U+11093 */
  0x0000,   /* U+11094 */
  0x0000,   /* U+11095 */
  0x0000,   /* U+11096 */
  0x0000,   /* U+11097 */
  0x0000,   /* U+11098 */
  0x0000,   /* U+11099 */
  0x00C2,   /* U+1109A */
  0x0000,   /* U+1109B */
  0x00C2,   /* U+1109C */
  0x0000,   /* U+1109D */
  0x0000,   /* U+1109E */
  0x0000,   /* U+1109F */
  0x0000,   /* U+110A0 */
  0x0000,   /* U+110A1 */
  0x0000,   /* U+110A2 */
  0x0000,   /* U+110A3 */
  0x0000,   /* U+110A4 */
  0x0000,   /* U+110A5 */
  0x0000,   /* U+110A6 */
  0x0000,   /* U+110A7 */
  0x0000,   /* U+110A8 */
  0x0000,   /* U+110A9 */
  0x0000,   /* U+110AA */
  0x00C2,   /* U+110AB */
  0x0000,   /* U+110AC */
  0x0000,   /* U+110AD */
  0x0000,   /* U+110AE */
  0x0000,   /* U+110AF */
  0x0000,   /* U+110B0 */
  0x0000,   /* U+110B1 */
  0x0000,   /* U+110B2 */
  0x0000,   /* U+110B3 */
  0x0000,   /* U+110B4 */
  0x0000,   /* U+110B5 */
  0x0000,   /* U+110B6 */
  0x0000,   /* U+110B7 */
  0x0000,   /* U+110B8 */
  0x0000,   /* U+110B9 */
  0x0000,   /* U+110BA */
  0x0000,   /* U+110BB */
  0x0000,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x0000,   /* U+110BE */
  0x0000,   /* U+110BF */
  0x0000,   /* U+110C0 */
  0x0000,   /* U+110C1 */
  0x0000,   /* U+110C2 */
  0x0000,   /* U+110C3 */
  0x0000,   /* U+110C4 */
  0x0000,   /* U+110C5 */
  0x0000,   /* U+110C6 */
  0x0000,   /* U+110C7 */
  0x0000,   /* U+110C8 */
  0x0000,   /* U+110C9 */
  0x0000,   /* U+110CA */
  0x0000,   /* U+110CB */
  0x0000,   /* U+110CC */
  0x0000,   /* U+110CD */
  0x0000,   /* U+110CE */
  0x0000,   /* U+110CF */
  0x0000,   /* U+110D0 */
  0x0000,   /* U+110D1 */
  0x0000,   /* U+110D2 */
  0x0000,   /* U+110D3 */
  0x0000,   /* U+110D4 */
  0x0000,   /* U+110D5 */
  0x0000,   /* U+110D6 */
  0x0000,   /* U+110D7 */
  0x0000,   /* U+110D8 */
  0x0000,   /* U+110D9 */
  0x0000,   /* U+110DA */
  0x0000,   /* U+110DB */
  0x0000,   /* U+110DC */
  0x0000,   /* U+110DD */
  0x0000,   /* U+110DE */
  0x0000,   /* U+110DF */
  0x0000,   /* U+110E0 */
  0x0000,   /* U+110E1 */
  0x0000,   /* U+110E2 */
  0x0000,   /* U+110E3 */
  0x0000,   /* U+110E4 */
  0x0000,   /* U+110E5 */
  0x0000,   /* U+110E6 */
  0x0000,   /* U+110E7 */
  0x0000,   /* U+110E8 */
  0x0000,   /* U+110E9 */
  0x0000,   /* U+110EA */
  0x0000,   /* U+110EB */
  0x0000,   /* U+110EC */
  0x0000,   /* U+110ED */
  0x0000,   /* U+110EE */
  0x0000,   /* U+110EF */
  0x0000,   /* U+110F0 */
  0x0000,   /* U+110F1 */
  0x0000,   /* U+110F2 */
  0x0000,   /* U+110F3 */
  0x0000,   /* U+110F4 */
  0x0000,   /* U+110F5 */
  0x0000,   /* U+110F6 */
  0x0000,   /* U+110F7 */
  0x0000,   /* U+110F8 */
  0x0000,   /* U+110F9 */
  0x0000,   /* U+110FA */
  0x0000,   /* U+110FB */
  0x0000,   /* U+110FC */
  0x0000,   /* U+110FD */
  0x0000,   /* U+110FE */
  0x0000,   /* U+110FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11000 */
  0x0000,   /* U+11001 */
  0x0000,   /* U+11002 */
  0x0000,   /* U+11003 */
  0x0000,   /* U+11004 */
  0x0000,   /* U+11005 */
  0x0000,   /* U+11006 */
  0x0000,   /* U+11007 */
  0x0000,   /* U+11008 */
  0x0000,   /* U+11009 */
  0x0000,   /* U+1100A */
  0x0000,   /* U+1100B */
  0x0000,   /* U+1100C */
  0x0000,   /* U+1100D */
  0x0000,   /* U+1100E */
  0x0000,   /* U+1100F */
  0x0000,   /* U+11010 */
  0x0000,   /* U+11011 */
  0x0000,   /* U+11012 */
  0x0000,   /* U+11013 */
  0x0000,   /* U+11014 */
  0x0000,   /* U+11015 */
  0x0000,   /* U+11016 */
  0x0000,   /* U+11017 */
  0x0000,   /* U+11018 */
  0x0000,   /* U+11019 */
  0x0000,   /* U+1101A */
  0x0000,   /* U+1101B */
  0x0000,   /* U+1101C */
  0x0000,   /* U+1101D */
  0x0000,   /* U+1101E */
  0x0000,   /* U+1101F */
  0x0000,   /* U+11020 */
  0x0000,   /* U+11021 */
  0x0000,   /* U+11022 */
  0x0000,   /* U+11023 */
  0x0000,   /* U+11024 */
  0x0000,   /* U+11025 */
  0x0000,   /* U+11026 */
  0x0000,   /* U+11027 */
  0x0000,   /* U+11028 */
  0x0000,   /* U+11029 */
  0x0000,   /* U+1102A */
  0x0000,   /* U+1102B */
  0x0000,   /* U+1102C */
  0x0000,   /* U+1102D */
  0x0000,   /* U+1102E */
  0x0000,   /* U+1102F */
  0x0000,   /* U+11030 */
  0x0000,   /* U+11031 */
  0x0000,   /* U+11032 */
  0x0000,   /* U+11033 */
  0x0000,   /* U+11034 */
  0x0000,   /* U+11035 */
  0x0000,   /* U+11036 */
  0x0000,   /* U+11037 */
  0x0000,   /* U+11038 */
  0x0000,   /* U+11039 */
  0x0000,   /* U+1103A */
  0x0000,   /* U+1103B */
  0x0000,   /* U+1103C */
  0x0000,   /* U+1103D */
  0x0000,   /* U+1103E */
  0x0000,   /* U+1103F */
  0x0000,   /* U+11040 */
  0x0000,   /* U+11041 */
  0x0000,   /* U+11042 */
  0x0000,   /* U+11043 */
  0x0000,   /* U+11044 */
  0x0000,   /* U+11045 */
  0x0000,   /* U+11046 */
  0x0000,   /* U+11047 */
  0x0000,   /* U+11048 */
  0x0000,   /* U+11049 */
  0x0000,   /* U+1104A */
  0x0000,   /* U+1104B */
  0x0000,   /* U+1104C */
  0x0000,   /* U+1104D */
  0x0000,   /* U+1104E */
  0x0000,   /* U+1104F */
  0x0000,   /* U+11050 */
  0x0000,   /* U+11051 */
  0x0000,   /* U+11052 */
  0x0000,   /* U+11053 */
  0x0000,   /* U+11054 */
  0x0000,   /* U+11055 */
  0x0000,   /* U+11056 */
  0x0000,   /* U+11057 */
  0x0000,   /* U+11058 */
  0x0000,   /* U+11059 */
  0x0000,   /* U+1105A */
  0x0000,   /* U+1105B */
  0x0000,   /* U+1105C */
  0x0000,   /* U+1105D */
  0x0000,   /* U+1105E */
  0x0000,   /* U+1105F */
  0x0000,   /* U+11060 */
  0x0000,   /* U+11061 */
  0x0000,   /* U+11062 */
  0x0000,   /* U+11063 */
  0x0000,   /* U+11064 */
  0x0000,   /* U+11065 */
  0x0000,   /* U+11066 */
  0x0000,   /* U+11067 */
  0x0000,   /* U+11068 */
  0x0000,   /* U+11069 */
  0x0000,   /* U+1106A */
  0x0000,   /* U+1106B */
  0x0000,   /* U+1106C */
  0x0000,   /* U+1106D */
  0x0000,   /* U+1106E */
  0x0000,   /* U+1106F */
  0x0000,   /* U+11070 */
  0x0000,   /* U+11071 */
  0x0000,   /* U+11072 */
  0x0000,   /* U+11073 */
  0x0000,   /* U+11074 */
  0x0000,   /* U+11075 */
  0x0000,   /* U+11076 */
  0x0000,   /* U+11077 */
  0x0000,   /* U+11078 */
  0x0000,   /* U+11079 */
  0x0000,   /* U+1107A */
  0x0000,   /* U+1107B */
  0x0000,   /* U+1107C */
  0x0000,   /* U+1107D */
  0x0000,   /* U+1107E */
  0x0000,   /* U+1107F */
  0x0000,   /* U+11080 */
  0x0000,   /* U+11081 */
  0x0000,   /* U+11082 */
  0x0000,   /* U+11083 */
  0x0000,   /* U+11084 */
  0x0000,   /* U+11085 */
  0x0000,   /* U+11086 */
  0x0000,   /* U+11087 */
  0x0000,   /* U+11088 */
  0x0000,   /* U+11089 */
  0x0000,   /* U+1108A */
  0x0000,   /* U+1108B */
  0x0000,   /* U+1108C */
  0x0000,   /* U+1108D */
  0x0000,   /* U+1108E */
  0x0000,   /* U+1108F */
  0x0000,   /* U+11090 */
  0x0000,   /* U+11091 */
  0x0000,   /* U+11092 */
  0x0000,   /* U+11093 */
  0x0000,   /* U+11094 */
  0x0000,   /* U+11095 */
  0x0000,   /* U+11096 */
  0x0000,   /* U+11097 */
  0x0000,   /* U+11098 */
  0x0000,   /* U+11099 */
  0x0002,   /* U+1109A */
  0x0000,   /* U+1109B */
  0x0002,   /* U+1109C */
  0x0000,   /* U+1109D */
  0x0000,   /* U+1109E */
  0x0000,   /* U+1109F */
  0x0000,   /* U+110A0 */
  0x0000,   /* U+110A1 */
  0x0000,   /* U+110A2 */
  0x0000,   /* U+110A3 */
  0x0000,   /* U+110A4 */
  0x0000,   /* U+110A5 */
  0x0000,   /* U+110A6 */
  0x0000,   /* U+110A7 */
  0x0000,   /* U+110A8 */
  0x0000,   /* U+110A9 */
  0x0000,   /* U+110AA */
  0x0002,   /* U+110AB */
  0x0000,   /* U+110AC */
  0x0000,   /* U+110AD */
  0x0000,   /* U+110AE */
  0x0000,   /* U+110AF */
  0x0000,   /* U+110B0 */
  0x0000,   /* U+110B1 */
  0x0000,   /* U+110B2 */
  0x0000,   /* U+110B3 */
  0x0000,   /* U+110B4 */
  0x0000,   /* U+110B5 */
  0x0000,   /* U+110B6 */
  0x0000,   /* U+110B7 */
  0x0000,   /* U+110B8 */
  0x0000,   /* U+110B9 */
  0x0000,   /* U+110BA */
  0x0000,   /* U+110BB */
  0x0000,   /* U+110BC */
  0x0000,   /* U+110BD */
  0x0000,   /* U+110BE */
  0x0000,   /* U+110BF */
  0x0000,   /* U+110C0 */
  0x0000,   /* U+110C1 */
  0x0000,   /* U+110C2 */
  0x0000,   /* U+110C3 */
  0x0000,   /* U+110C4 */
  0x0000,   /* U+110C5 */
  0x0000,   /* U+110C6 */
  0x0000,   /* U+110C7 */
  0x0000,   /* U+110C8 */
  0x0000,   /* U+110C9 */
  0x0000,   /* U+110CA */
  0x0000,   /* U+110CB */
  0x0000,   /* U+110CC */
  0x0000,   /* U+110CD */
  0x0000,   /* U+110CE */
  0x0000,   /* U+110CF */
  0x0000,   /* U+110D0 */
  0x0000,   /* U+110D1 */
  0x0000,   /* U+110D2 */
  0x0000,   /* U+110D3 */
  0x0000,   /* U+110D4 */
  0x0000,   /* U+110D5 */
  0x0000,   /* U+110D6 */
  0x0000,   /* U+110D7 */
  0x0000,   /* U+110D8 */
  0x0000,   /* U+110D9 */
  0x0000,   /* U+110DA */
  0x0000,   /* U+110DB */
  0x0000,   /* U+110DC */
  0x0000,   /* U+110DD */
  0x0000,   /* U+110DE */
  0x0000,   /* U+110DF */
  0x0000,   /* U+110E0 */
  0x0000,   /* U+110E1 */
  0x0000,   /* U+110E2 */
  0x0000,   /* U+110E3 */
  0x0000,   /* U+110E4 */
  0x0000,   /* U+110E5 */
  0x0000,   /* U+110E6 */
  0x0000,   /* U+110E7 */
  0x0000,   /* U+110E8 */
  0x0000,   /* U+110E9 */
  0x0000,   /* U+110EA */
  0x0000,   /* U+110EB */
  0x0000,   /* U+110EC */
  0x0000,   /* U+110ED */
  0x0000,   /* U+110EE */
  0x0000,   /* U+110EF */
  0x0000,   /* U+110F0 */
  0x0000,   /* U+110F1 */
  0x0000,   /* U+110F2 */
  0x0000,   /* U+110F3 */
  0x0000,   /* U+110F4 */
  0x0000,   /* U+110F5 */
  0x0000,   /* U+110F6 */
  0x0000,   /* U+110F7 */
  0x0000,   /* U+110F8 */
  0x0000,   /* U+110F9 */
  0x0000,   /* U+110FA */
  0x0000,   /* U+110FB */
  0x0000,   /* U+110FC */
  0x0000,   /* U+110FD */
  0x0000,   /* U+110FE */
  0x0000,   /* U+110FF */
};

uint16 uca900_p111[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+11100 */
  0x0000,   /* U+11101 */
  0x0000,   /* U+11102 */
  0x30E4,   /* U+11103 */
  0x30E5,   /* U+11104 */
  0x30E6,   /* U+11105 */
  0x30E7,   /* U+11106 */
  0x30E8,   /* U+11107 */
  0x30E9,   /* U+11108 */
  0x30EA,   /* U+11109 */
  0x30EB,   /* U+1110A */
  0x30EC,   /* U+1110B */
  0x30ED,   /* U+1110C */
  0x30EE,   /* U+1110D */
  0x30EF,   /* U+1110E */
  0x30F0,   /* U+1110F */
  0x30F1,   /* U+11110 */
  0x30F2,   /* U+11111 */
  0x30F3,   /* U+11112 */
  0x30F4,   /* U+11113 */
  0x30F5,   /* U+11114 */
  0x30F6,   /* U+11115 */
  0x30F7,   /* U+11116 */
  0x30F8,   /* U+11117 */
  0x30F9,   /* U+11118 */
  0x30FA,   /* U+11119 */
  0x30FB,   /* U+1111A */
  0x30FC,   /* U+1111B */
  0x30FD,   /* U+1111C */
  0x30FE,   /* U+1111D */
  0x30FF,   /* U+1111E */
  0x3100,   /* U+1111F */
  0x3101,   /* U+11120 */
  0x3102,   /* U+11121 */
  0x3103,   /* U+11122 */
  0x3104,   /* U+11123 */
  0x3105,   /* U+11124 */
  0x3106,   /* U+11125 */
  0x3107,   /* U+11126 */
  0x3108,   /* U+11127 */
  0x3109,   /* U+11128 */
  0x310A,   /* U+11129 */
  0x310B,   /* U+1112A */
  0x310C,   /* U+1112B */
  0x310D,   /* U+1112C */
  0x310E,   /* U+1112D */
  0x310F,   /* U+1112E */
  0x3110,   /* U+1112F */
  0x3111,   /* U+11130 */
  0x3112,   /* U+11131 */
  0x3113,   /* U+11132 */
  0x3114,   /* U+11133 */
  0x3115,   /* U+11134 */
  0xFBC2,   /* U+11135 */
  0x1C3D,   /* U+11136 */
  0x1C3E,   /* U+11137 */
  0x1C3F,   /* U+11138 */
  0x1C40,   /* U+11139 */
  0x1C41,   /* U+1113A */
  0x1C42,   /* U+1113B */
  0x1C43,   /* U+1113C */
  0x1C44,   /* U+1113D */
  0x1C45,   /* U+1113E */
  0x1C46,   /* U+1113F */
  0x02E2,   /* U+11140 */
  0x02B0,   /* U+11141 */
  0x02B1,   /* U+11142 */
  0x0273,   /* U+11143 */
  0xFBC2,   /* U+11144 */
  0xFBC2,   /* U+11145 */
  0xFBC2,   /* U+11146 */
  0xFBC2,   /* U+11147 */
  0xFBC2,   /* U+11148 */
  0xFBC2,   /* U+11149 */
  0xFBC2,   /* U+1114A */
  0xFBC2,   /* U+1114B */
  0xFBC2,   /* U+1114C */
  0xFBC2,   /* U+1114D */
  0xFBC2,   /* U+1114E */
  0xFBC2,   /* U+1114F */
  0x29FA,   /* U+11150 */
  0x29FB,   /* U+11151 */
  0x29FC,   /* U+11152 */
  0x29FD,   /* U+11153 */
  0x29FE,   /* U+11154 */
  0x29FF,   /* U+11155 */
  0x2A00,   /* U+11156 */
  0x2A01,   /* U+11157 */
  0x2A02,   /* U+11158 */
  0x2A03,   /* U+11159 */
  0x2A04,   /* U+1115A */
  0x2A05,   /* U+1115B */
  0x2A06,   /* U+1115C */
  0x2A07,   /* U+1115D */
  0x2A08,   /* U+1115E */
  0x2A09,   /* U+1115F */
  0x2A0A,   /* U+11160 */
  0x2A0B,   /* U+11161 */
  0x2A0C,   /* U+11162 */
  0x2A0D,   /* U+11163 */
  0x2A0E,   /* U+11164 */
  0x2A0F,   /* U+11165 */
  0x2A10,   /* U+11166 */
  0x2A11,   /* U+11167 */
  0x2A12,   /* U+11168 */
  0x2A13,   /* U+11169 */
  0x2A14,   /* U+1116A */
  0x2A15,   /* U+1116B */
  0x2A16,   /* U+1116C */
  0x2A17,   /* U+1116D */
  0x2A18,   /* U+1116E */
  0x2A19,   /* U+1116F */
  0x2A1B,   /* U+11170 */
  0x2A1C,   /* U+11171 */
  0x2A1D,   /* U+11172 */
  0x0000,   /* U+11173 */
  0x0450,   /* U+11174 */
  0x0451,   /* U+11175 */
  0x2A1A,   /* U+11176 */
  0xFBC2,   /* U+11177 */
  0xFBC2,   /* U+11178 */
  0xFBC2,   /* U+11179 */
  0xFBC2,   /* U+1117A */
  0xFBC2,   /* U+1117B */
  0xFBC2,   /* U+1117C */
  0xFBC2,   /* U+1117D */
  0xFBC2,   /* U+1117E */
  0xFBC2,   /* U+1117F */
  0x0000,   /* U+11180 */
  0x0000,   /* U+11181 */
  0x0000,   /* U+11182 */
  0x2A20,   /* U+11183 */
  0x2A21,   /* U+11184 */
  0x2A22,   /* U+11185 */
  0x2A23,   /* U+11186 */
  0x2A24,   /* U+11187 */
  0x2A25,   /* U+11188 */
  0x2A26,   /* U+11189 */
  0x2A27,   /* U+1118A */
  0x2A28,   /* U+1118B */
  0x2A29,   /* U+1118C */
  0x2A2A,   /* U+1118D */
  0x2A2B,   /* U+1118E */
  0x2A2C,   /* U+1118F */
  0x2A2D,   /* U+11190 */
  0x2A2E,   /* U+11191 */
  0x2A2F,   /* U+11192 */
  0x2A30,   /* U+11193 */
  0x2A31,   /* U+11194 */
  0x2A32,   /* U+11195 */
  0x2A33,   /* U+11196 */
  0x2A34,   /* U+11197 */
  0x2A35,   /* U+11198 */
  0x2A36,   /* U+11199 */
  0x2A37,   /* U+1119A */
  0x2A38,   /* U+1119B */
  0x2A39,   /* U+1119C */
  0x2A3A,   /* U+1119D */
  0x2A3B,   /* U+1119E */
  0x2A3C,   /* U+1119F */
  0x2A3D,   /* U+111A0 */
  0x2A3E,   /* U+111A1 */
  0x2A3F,   /* U+111A2 */
  0x2A40,   /* U+111A3 */
  0x2A41,   /* U+111A4 */
  0x2A42,   /* U+111A5 */
  0x2A43,   /* U+111A6 */
  0x2A44,   /* U+111A7 */
  0x2A45,   /* U+111A8 */
  0x2A46,   /* U+111A9 */
  0x2A47,   /* U+111AA */
  0x2A48,   /* U+111AB */
  0x2A49,   /* U+111AC */
  0x2A4A,   /* U+111AD */
  0x2A4B,   /* U+111AE */
  0x2A4C,   /* U+111AF */
  0x2A4D,   /* U+111B0 */
  0x2A4E,   /* U+111B1 */
  0x2A4F,   /* U+111B2 */
  0x2A54,   /* U+111B3 */
  0x2A55,   /* U+111B4 */
  0x2A56,   /* U+111B5 */
  0x2A57,   /* U+111B6 */
  0x2A58,   /* U+111B7 */
  0x2A59,   /* U+111B8 */
  0x2A5A,   /* U+111B9 */
  0x2A5B,   /* U+111BA */
  0x2A5C,   /* U+111BB */
  0x2A5D,   /* U+111BC */
  0x2A5E,   /* U+111BD */
  0x2A5F,   /* U+111BE */
  0x2A60,   /* U+111BF */
  0x2A61,   /* U+111C0 */
  0x2A50,   /* U+111C1 */
  0x2A51,   /* U+111C2 */
  0x2A52,   /* U+111C3 */
  0x2A1E,   /* U+111C4 */
  0x02B2,   /* U+111C5 */
  0x02B3,   /* U+111C6 */
  0x0453,   /* U+111C7 */
  0x02E3,   /* U+111C8 */
  0x0454,   /* U+111C9 */
  0x0000,   /* U+111CA */
  0x0000,   /* U+111CB */
  0x0000,   /* U+111CC */
  0x0452,   /* U+111CD */
  0xFBC2,   /* U+111CE */
  0xFBC2,   /* U+111CF */
  0x1C3D,   /* U+111D0 */
  0x1C3E,   /* U+111D1 */
  0x1C3F,   /* U+111D2 */
  0x1C40,   /* U+111D3 */
  0x1C41,   /* U+111D4 */
  0x1C42,   /* U+111D5 */
  0x1C43,   /* U+111D6 */
  0x1C44,   /* U+111D7 */
  0x1C45,   /* U+111D8 */
  0x1C46,   /* U+111D9 */
  0x2A1F,   /* U+111DA */
  0x0455,   /* U+111DB */
  0x2A53,   /* U+111DC */
  0x0456,   /* U+111DD */
  0x02E4,   /* U+111DE */
  0x02E5,   /* U+111DF */
  0xFBC2,   /* U+111E0 */
  0x1C3E,   /* U+111E1 */
  0x1C3F,   /* U+111E2 */
  0x1C40,   /* U+111E3 */
  0x1C41,   /* U+111E4 */
  0x1C42,   /* U+111E5 */
  0x1C43,   /* U+111E6 */
  0x1C44,   /* U+111E7 */
  0x1C45,   /* U+111E8 */
  0x1C46,   /* U+111E9 */
  0x1B8C,   /* U+111EA */
  0x1B8D,   /* U+111EB */
  0x1B8E,   /* U+111EC */
  0x1B8F,   /* U+111ED */
  0x1B90,   /* U+111EE */
  0x1B91,   /* U+111EF */
  0x1B92,   /* U+111F0 */
  0x1B93,   /* U+111F1 */
  0x1B94,   /* U+111F2 */
  0x1B95,   /* U+111F3 */
  0x1B96,   /* U+111F4 */
  0xFBC2,   /* U+111F5 */
  0xFBC2,   /* U+111F6 */
  0xFBC2,   /* U+111F7 */
  0xFBC2,   /* U+111F8 */
  0xFBC2,   /* U+111F9 */
  0xFBC2,   /* U+111FA */
  0xFBC2,   /* U+111FB */
  0xFBC2,   /* U+111FC */
  0xFBC2,   /* U+111FD */
  0xFBC2,   /* U+111FE */
  0xFBC2,   /* U+111FF */

  /* Secondary weight 1 for each character. */
  0x00C3,   /* U+11100 */
  0x00C4,   /* U+11101 */
  0x00C5,   /* U+11102 */
  0x0020,   /* U+11103 */
  0x0020,   /* U+11104 */
  0x0020,   /* U+11105 */
  0x0020,   /* U+11106 */
  0x0020,   /* U+11107 */
  0x0020,   /* U+11108 */
  0x0020,   /* U+11109 */
  0x0020,   /* U+1110A */
  0x0020,   /* U+1110B */
  0x0020,   /* U+1110C */
  0x0020,   /* U+1110D */
  0x0020,   /* U+1110E */
  0x0020,   /* U+1110F */
  0x0020,   /* U+11110 */
  0x0020,   /* U+11111 */
  0x0020,   /* U+11112 */
  0x0020,   /* U+11113 */
  0x0020,   /* U+11114 */
  0x0020,   /* U+11115 */
  0x0020,   /* U+11116 */
  0x0020,   /* U+11117 */
  0x0020,   /* U+11118 */
  0x0020,   /* U+11119 */
  0x0020,   /* U+1111A */
  0x0020,   /* U+1111B */
  0x0020,   /* U+1111C */
  0x0020,   /* U+1111D */
  0x0020,   /* U+1111E */
  0x0020,   /* U+1111F */
  0x0020,   /* U+11120 */
  0x0020,   /* U+11121 */
  0x0020,   /* U+11122 */
  0x0020,   /* U+11123 */
  0x0020,   /* U+11124 */
  0x0020,   /* U+11125 */
  0x0020,   /* U+11126 */
  0x0020,   /* U+11127 */
  0x0020,   /* U+11128 */
  0x0020,   /* U+11129 */
  0x0020,   /* U+1112A */
  0x0020,   /* U+1112B */
  0x0020,   /* U+1112C */
  0x0020,   /* U+1112D */
  0x0020,   /* U+1112E */
  0x0020,   /* U+1112F */
  0x0020,   /* U+11130 */
  0x0020,   /* U+11131 */
  0x0020,   /* U+11132 */
  0x0020,   /* U+11133 */
  0x0020,   /* U+11134 */
  0x0020,   /* U+11135 */
  0x0020,   /* U+11136 */
  0x0020,   /* U+11137 */
  0x0020,   /* U+11138 */
  0x0020,   /* U+11139 */
  0x0020,   /* U+1113A */
  0x0020,   /* U+1113B */
  0x0020,   /* U+1113C */
  0x0020,   /* U+1113D */
  0x0020,   /* U+1113E */
  0x0020,   /* U+1113F */
  0x0020,   /* U+11140 */
  0x0020,   /* U+11141 */
  0x0020,   /* U+11142 */
  0x0020,   /* U+11143 */
  0x0020,   /* U+11144 */
  0x0020,   /* U+11145 */
  0x0020,   /* U+11146 */
  0x0020,   /* U+11147 */
  0x0020,   /* U+11148 */
  0x0020,   /* U+11149 */
  0x0020,   /* U+1114A */
  0x0020,   /* U+1114B */
  0x0020,   /* U+1114C */
  0x0020,   /* U+1114D */
  0x0020,   /* U+1114E */
  0x0020,   /* U+1114F */
  0x0020,   /* U+11150 */
  0x0020,   /* U+11151 */
  0x0020,   /* U+11152 */
  0x0020,   /* U+11153 */
  0x0020,   /* U+11154 */
  0x0020,   /* U+11155 */
  0x0020,   /* U+11156 */
  0x0020,   /* U+11157 */
  0x0020,   /* U+11158 */
  0x0020,   /* U+11159 */
  0x0020,   /* U+1115A */
  0x0020,   /* U+1115B */
  0x0020,   /* U+1115C */
  0x0020,   /* U+1115D */
  0x0020,   /* U+1115E */
  0x0020,   /* U+1115F */
  0x0020,   /* U+11160 */
  0x0020,   /* U+11161 */
  0x0020,   /* U+11162 */
  0x0020,   /* U+11163 */
  0x0020,   /* U+11164 */
  0x0020,   /* U+11165 */
  0x0020,   /* U+11166 */
  0x0020,   /* U+11167 */
  0x0020,   /* U+11168 */
  0x0020,   /* U+11169 */
  0x0020,   /* U+1116A */
  0x0020,   /* U+1116B */
  0x0020,   /* U+1116C */
  0x0020,   /* U+1116D */
  0x0020,   /* U+1116E */
  0x0020,   /* U+1116F */
  0x0020,   /* U+11170 */
  0x0020,   /* U+11171 */
  0x0020,   /* U+11172 */
  0x00C2,   /* U+11173 */
  0x0020,   /* U+11174 */
  0x0020,   /* U+11175 */
  0x0020,   /* U+11176 */
  0x0020,   /* U+11177 */
  0x0020,   /* U+11178 */
  0x0020,   /* U+11179 */
  0x0020,   /* U+1117A */
  0x0020,   /* U+1117B */
  0x0020,   /* U+1117C */
  0x0020,   /* U+1117D */
  0x0020,   /* U+1117E */
  0x0020,   /* U+1117F */
  0x00C3,   /* U+11180 */
  0x00C4,   /* U+11181 */
  0x00C5,   /* U+11182 */
  0x0020,   /* U+11183 */
  0x0020,   /* U+11184 */
  0x0020,   /* U+11185 */
  0x0020,   /* U+11186 */
  0x0020,   /* U+11187 */
  0x0020,   /* U+11188 */
  0x0020,   /* U+11189 */
  0x0020,   /* U+1118A */
  0x0020,   /* U+1118B */
  0x0020,   /* U+1118C */
  0x0020,   /* U+1118D */
  0x0020,   /* U+1118E */
  0x0020,   /* U+1118F */
  0x0020,   /* U+11190 */
  0x0020,   /* U+11191 */
  0x0020,   /* U+11192 */
  0x0020,   /* U+11193 */
  0x0020,   /* U+11194 */
  0x0020,   /* U+11195 */
  0x0020,   /* U+11196 */
  0x0020,   /* U+11197 */
  0x0020,   /* U+11198 */
  0x0020,   /* U+11199 */
  0x0020,   /* U+1119A */
  0x0020,   /* U+1119B */
  0x0020,   /* U+1119C */
  0x0020,   /* U+1119D */
  0x0020,   /* U+1119E */
  0x0020,   /* U+1119F */
  0x0020,   /* U+111A0 */
  0x0020,   /* U+111A1 */
  0x0020,   /* U+111A2 */
  0x0020,   /* U+111A3 */
  0x0020,   /* U+111A4 */
  0x0020,   /* U+111A5 */
  0x0020,   /* U+111A6 */
  0x0020,   /* U+111A7 */
  0x0020,   /* U+111A8 */
  0x0020,   /* U+111A9 */
  0x0020,   /* U+111AA */
  0x0020,   /* U+111AB */
  0x0020,   /* U+111AC */
  0x0020,   /* U+111AD */
  0x0020,   /* U+111AE */
  0x0020,   /* U+111AF */
  0x0020,   /* U+111B0 */
  0x0020,   /* U+111B1 */
  0x0020,   /* U+111B2 */
  0x0020,   /* U+111B3 */
  0x0020,   /* U+111B4 */
  0x0020,   /* U+111B5 */
  0x0020,   /* U+111B6 */
  0x0020,   /* U+111B7 */
  0x0020,   /* U+111B8 */
  0x0020,   /* U+111B9 */
  0x0020,   /* U+111BA */
  0x0020,   /* U+111BB */
  0x0020,   /* U+111BC */
  0x0020,   /* U+111BD */
  0x0020,   /* U+111BE */
  0x0020,   /* U+111BF */
  0x0020,   /* U+111C0 */
  0x0020,   /* U+111C1 */
  0x0020,   /* U+111C2 */
  0x0020,   /* U+111C3 */
  0x0020,   /* U+111C4 */
  0x0020,   /* U+111C5 */
  0x0020,   /* U+111C6 */
  0x0020,   /* U+111C7 */
  0x0020,   /* U+111C8 */
  0x0020,   /* U+111C9 */
  0x00C2,   /* U+111CA */
  0x00CF,   /* U+111CB */
  0x00D0,   /* U+111CC */
  0x0020,   /* U+111CD */
  0x0020,   /* U+111CE */
  0x0020,   /* U+111CF */
  0x0020,   /* U+111D0 */
  0x0020,   /* U+111D1 */
  0x0020,   /* U+111D2 */
  0x0020,   /* U+111D3 */
  0x0020,   /* U+111D4 */
  0x0020,   /* U+111D5 */
  0x0020,   /* U+111D6 */
  0x0020,   /* U+111D7 */
  0x0020,   /* U+111D8 */
  0x0020,   /* U+111D9 */
  0x0020,   /* U+111DA */
  0x0020,   /* U+111DB */
  0x0020,   /* U+111DC */
  0x0020,   /* U+111DD */
  0x0020,   /* U+111DE */
  0x0020,   /* U+111DF */
  0x0020,   /* U+111E0 */
  0x0020,   /* U+111E1 */
  0x0020,   /* U+111E2 */
  0x0020,   /* U+111E3 */
  0x0020,   /* U+111E4 */
  0x0020,   /* U+111E5 */
  0x0020,   /* U+111E6 */
  0x0020,   /* U+111E7 */
  0x0020,   /* U+111E8 */
  0x0020,   /* U+111E9 */
  0x0020,   /* U+111EA */
  0x0020,   /* U+111EB */
  0x0020,   /* U+111EC */
  0x0020,   /* U+111ED */
  0x0020,   /* U+111EE */
  0x0020,   /* U+111EF */
  0x0020,   /* U+111F0 */
  0x0020,   /* U+111F1 */
  0x0020,   /* U+111F2 */
  0x0020,   /* U+111F3 */
  0x0020,   /* U+111F4 */
  0x0020,   /* U+111F5 */
  0x0020,   /* U+111F6 */
  0x0020,   /* U+111F7 */
  0x0020,   /* U+111F8 */
  0x0020,   /* U+111F9 */
  0x0020,   /* U+111FA */
  0x0020,   /* U+111FB */
  0x0020,   /* U+111FC */
  0x0020,   /* U+111FD */
  0x0020,   /* U+111FE */
  0x0020,   /* U+111FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11100 */
  0x0002,   /* U+11101 */
  0x0002,   /* U+11102 */
  0x0002,   /* U+11103 */
  0x0002,   /* U+11104 */
  0x0002,   /* U+11105 */
  0x0002,   /* U+11106 */
  0x0002,   /* U+11107 */
  0x0002,   /* U+11108 */
  0x0002,   /* U+11109 */
  0x0002,   /* U+1110A */
  0x0002,   /* U+1110B */
  0x0002,   /* U+1110C */
  0x0002,   /* U+1110D */
  0x0002,   /* U+1110E */
  0x0002,   /* U+1110F */
  0x0002,   /* U+11110 */
  0x0002,   /* U+11111 */
  0x0002,   /* U+11112 */
  0x0002,   /* U+11113 */
  0x0002,   /* U+11114 */
  0x0002,   /* U+11115 */
  0x0002,   /* U+11116 */
  0x0002,   /* U+11117 */
  0x0002,   /* U+11118 */
  0x0002,   /* U+11119 */
  0x0002,   /* U+1111A */
  0x0002,   /* U+1111B */
  0x0002,   /* U+1111C */
  0x0002,   /* U+1111D */
  0x0002,   /* U+1111E */
  0x0002,   /* U+1111F */
  0x0002,   /* U+11120 */
  0x0002,   /* U+11121 */
  0x0002,   /* U+11122 */
  0x0002,   /* U+11123 */
  0x0002,   /* U+11124 */
  0x0002,   /* U+11125 */
  0x0002,   /* U+11126 */
  0x0002,   /* U+11127 */
  0x0002,   /* U+11128 */
  0x0002,   /* U+11129 */
  0x0002,   /* U+1112A */
  0x0002,   /* U+1112B */
  0x0002,   /* U+1112C */
  0x0002,   /* U+1112D */
  0x0002,   /* U+1112E */
  0x0002,   /* U+1112F */
  0x0002,   /* U+11130 */
  0x0002,   /* U+11131 */
  0x0002,   /* U+11132 */
  0x0002,   /* U+11133 */
  0x0002,   /* U+11134 */
  0x0002,   /* U+11135 */
  0x0002,   /* U+11136 */
  0x0002,   /* U+11137 */
  0x0002,   /* U+11138 */
  0x0002,   /* U+11139 */
  0x0002,   /* U+1113A */
  0x0002,   /* U+1113B */
  0x0002,   /* U+1113C */
  0x0002,   /* U+1113D */
  0x0002,   /* U+1113E */
  0x0002,   /* U+1113F */
  0x0002,   /* U+11140 */
  0x0002,   /* U+11141 */
  0x0002,   /* U+11142 */
  0x0002,   /* U+11143 */
  0x0002,   /* U+11144 */
  0x0002,   /* U+11145 */
  0x0002,   /* U+11146 */
  0x0002,   /* U+11147 */
  0x0002,   /* U+11148 */
  0x0002,   /* U+11149 */
  0x0002,   /* U+1114A */
  0x0002,   /* U+1114B */
  0x0002,   /* U+1114C */
  0x0002,   /* U+1114D */
  0x0002,   /* U+1114E */
  0x0002,   /* U+1114F */
  0x0002,   /* U+11150 */
  0x0002,   /* U+11151 */
  0x0002,   /* U+11152 */
  0x0002,   /* U+11153 */
  0x0002,   /* U+11154 */
  0x0002,   /* U+11155 */
  0x0002,   /* U+11156 */
  0x0002,   /* U+11157 */
  0x0002,   /* U+11158 */
  0x0002,   /* U+11159 */
  0x0002,   /* U+1115A */
  0x0002,   /* U+1115B */
  0x0002,   /* U+1115C */
  0x0002,   /* U+1115D */
  0x0002,   /* U+1115E */
  0x0002,   /* U+1115F */
  0x0002,   /* U+11160 */
  0x0002,   /* U+11161 */
  0x0002,   /* U+11162 */
  0x0002,   /* U+11163 */
  0x0002,   /* U+11164 */
  0x0002,   /* U+11165 */
  0x0002,   /* U+11166 */
  0x0002,   /* U+11167 */
  0x0002,   /* U+11168 */
  0x0002,   /* U+11169 */
  0x0002,   /* U+1116A */
  0x0002,   /* U+1116B */
  0x0002,   /* U+1116C */
  0x0002,   /* U+1116D */
  0x0002,   /* U+1116E */
  0x0002,   /* U+1116F */
  0x0002,   /* U+11170 */
  0x0002,   /* U+11171 */
  0x0002,   /* U+11172 */
  0x0002,   /* U+11173 */
  0x0002,   /* U+11174 */
  0x0002,   /* U+11175 */
  0x0002,   /* U+11176 */
  0x0002,   /* U+11177 */
  0x0002,   /* U+11178 */
  0x0002,   /* U+11179 */
  0x0002,   /* U+1117A */
  0x0002,   /* U+1117B */
  0x0002,   /* U+1117C */
  0x0002,   /* U+1117D */
  0x0002,   /* U+1117E */
  0x0002,   /* U+1117F */
  0x0002,   /* U+11180 */
  0x0002,   /* U+11181 */
  0x0002,   /* U+11182 */
  0x0002,   /* U+11183 */
  0x0002,   /* U+11184 */
  0x0002,   /* U+11185 */
  0x0002,   /* U+11186 */
  0x0002,   /* U+11187 */
  0x0002,   /* U+11188 */
  0x0002,   /* U+11189 */
  0x0002,   /* U+1118A */
  0x0002,   /* U+1118B */
  0x0002,   /* U+1118C */
  0x0002,   /* U+1118D */
  0x0002,   /* U+1118E */
  0x0002,   /* U+1118F */
  0x0002,   /* U+11190 */
  0x0002,   /* U+11191 */
  0x0002,   /* U+11192 */
  0x0002,   /* U+11193 */
  0x0002,   /* U+11194 */
  0x0002,   /* U+11195 */
  0x0002,   /* U+11196 */
  0x0002,   /* U+11197 */
  0x0002,   /* U+11198 */
  0x0002,   /* U+11199 */
  0x0002,   /* U+1119A */
  0x0002,   /* U+1119B */
  0x0002,   /* U+1119C */
  0x0002,   /* U+1119D */
  0x0002,   /* U+1119E */
  0x0002,   /* U+1119F */
  0x0002,   /* U+111A0 */
  0x0002,   /* U+111A1 */
  0x0002,   /* U+111A2 */
  0x0002,   /* U+111A3 */
  0x0002,   /* U+111A4 */
  0x0002,   /* U+111A5 */
  0x0002,   /* U+111A6 */
  0x0002,   /* U+111A7 */
  0x0002,   /* U+111A8 */
  0x0002,   /* U+111A9 */
  0x0002,   /* U+111AA */
  0x0002,   /* U+111AB */
  0x0002,   /* U+111AC */
  0x0002,   /* U+111AD */
  0x0002,   /* U+111AE */
  0x0002,   /* U+111AF */
  0x0002,   /* U+111B0 */
  0x0002,   /* U+111B1 */
  0x0002,   /* U+111B2 */
  0x0002,   /* U+111B3 */
  0x0002,   /* U+111B4 */
  0x0002,   /* U+111B5 */
  0x0002,   /* U+111B6 */
  0x0002,   /* U+111B7 */
  0x0002,   /* U+111B8 */
  0x0002,   /* U+111B9 */
  0x0002,   /* U+111BA */
  0x0002,   /* U+111BB */
  0x0002,   /* U+111BC */
  0x0002,   /* U+111BD */
  0x0002,   /* U+111BE */
  0x0002,   /* U+111BF */
  0x0002,   /* U+111C0 */
  0x0002,   /* U+111C1 */
  0x0002,   /* U+111C2 */
  0x0002,   /* U+111C3 */
  0x0002,   /* U+111C4 */
  0x0002,   /* U+111C5 */
  0x0002,   /* U+111C6 */
  0x0002,   /* U+111C7 */
  0x0002,   /* U+111C8 */
  0x0002,   /* U+111C9 */
  0x0002,   /* U+111CA */
  0x0002,   /* U+111CB */
  0x0002,   /* U+111CC */
  0x0002,   /* U+111CD */
  0x0002,   /* U+111CE */
  0x0002,   /* U+111CF */
  0x0002,   /* U+111D0 */
  0x0002,   /* U+111D1 */
  0x0002,   /* U+111D2 */
  0x0002,   /* U+111D3 */
  0x0002,   /* U+111D4 */
  0x0002,   /* U+111D5 */
  0x0002,   /* U+111D6 */
  0x0002,   /* U+111D7 */
  0x0002,   /* U+111D8 */
  0x0002,   /* U+111D9 */
  0x0002,   /* U+111DA */
  0x0002,   /* U+111DB */
  0x0002,   /* U+111DC */
  0x0002,   /* U+111DD */
  0x0002,   /* U+111DE */
  0x0002,   /* U+111DF */
  0x0002,   /* U+111E0 */
  0x0002,   /* U+111E1 */
  0x0002,   /* U+111E2 */
  0x0002,   /* U+111E3 */
  0x0002,   /* U+111E4 */
  0x0002,   /* U+111E5 */
  0x0002,   /* U+111E6 */
  0x0002,   /* U+111E7 */
  0x0002,   /* U+111E8 */
  0x0002,   /* U+111E9 */
  0x0002,   /* U+111EA */
  0x0002,   /* U+111EB */
  0x0002,   /* U+111EC */
  0x0002,   /* U+111ED */
  0x0002,   /* U+111EE */
  0x0002,   /* U+111EF */
  0x0002,   /* U+111F0 */
  0x0002,   /* U+111F1 */
  0x0002,   /* U+111F2 */
  0x0002,   /* U+111F3 */
  0x0002,   /* U+111F4 */
  0x0002,   /* U+111F5 */
  0x0002,   /* U+111F6 */
  0x0002,   /* U+111F7 */
  0x0002,   /* U+111F8 */
  0x0002,   /* U+111F9 */
  0x0002,   /* U+111FA */
  0x0002,   /* U+111FB */
  0x0002,   /* U+111FC */
  0x0002,   /* U+111FD */
  0x0002,   /* U+111FE */
  0x0002,   /* U+111FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11100 */
  0x0000,   /* U+11101 */
  0x0000,   /* U+11102 */
  0x0000,   /* U+11103 */
  0x0000,   /* U+11104 */
  0x0000,   /* U+11105 */
  0x0000,   /* U+11106 */
  0x0000,   /* U+11107 */
  0x0000,   /* U+11108 */
  0x0000,   /* U+11109 */
  0x0000,   /* U+1110A */
  0x0000,   /* U+1110B */
  0x0000,   /* U+1110C */
  0x0000,   /* U+1110D */
  0x0000,   /* U+1110E */
  0x0000,   /* U+1110F */
  0x0000,   /* U+11110 */
  0x0000,   /* U+11111 */
  0x0000,   /* U+11112 */
  0x0000,   /* U+11113 */
  0x0000,   /* U+11114 */
  0x0000,   /* U+11115 */
  0x0000,   /* U+11116 */
  0x0000,   /* U+11117 */
  0x0000,   /* U+11118 */
  0x0000,   /* U+11119 */
  0x0000,   /* U+1111A */
  0x0000,   /* U+1111B */
  0x0000,   /* U+1111C */
  0x0000,   /* U+1111D */
  0x0000,   /* U+1111E */
  0x0000,   /* U+1111F */
  0x0000,   /* U+11120 */
  0x0000,   /* U+11121 */
  0x0000,   /* U+11122 */
  0x0000,   /* U+11123 */
  0x0000,   /* U+11124 */
  0x0000,   /* U+11125 */
  0x0000,   /* U+11126 */
  0x0000,   /* U+11127 */
  0x0000,   /* U+11128 */
  0x0000,   /* U+11129 */
  0x0000,   /* U+1112A */
  0x0000,   /* U+1112B */
  0x0000,   /* U+1112C */
  0x0000,   /* U+1112D */
  0x0000,   /* U+1112E */
  0x0000,   /* U+1112F */
  0x0000,   /* U+11130 */
  0x0000,   /* U+11131 */
  0x0000,   /* U+11132 */
  0x0000,   /* U+11133 */
  0x0000,   /* U+11134 */
  0x9135,   /* U+11135 */
  0x0000,   /* U+11136 */
  0x0000,   /* U+11137 */
  0x0000,   /* U+11138 */
  0x0000,   /* U+11139 */
  0x0000,   /* U+1113A */
  0x0000,   /* U+1113B */
  0x0000,   /* U+1113C */
  0x0000,   /* U+1113D */
  0x0000,   /* U+1113E */
  0x0000,   /* U+1113F */
  0x0000,   /* U+11140 */
  0x0000,   /* U+11141 */
  0x0000,   /* U+11142 */
  0x0000,   /* U+11143 */
  0x9144,   /* U+11144 */
  0x9145,   /* U+11145 */
  0x9146,   /* U+11146 */
  0x9147,   /* U+11147 */
  0x9148,   /* U+11148 */
  0x9149,   /* U+11149 */
  0x914A,   /* U+1114A */
  0x914B,   /* U+1114B */
  0x914C,   /* U+1114C */
  0x914D,   /* U+1114D */
  0x914E,   /* U+1114E */
  0x914F,   /* U+1114F */
  0x0000,   /* U+11150 */
  0x0000,   /* U+11151 */
  0x0000,   /* U+11152 */
  0x0000,   /* U+11153 */
  0x0000,   /* U+11154 */
  0x0000,   /* U+11155 */
  0x0000,   /* U+11156 */
  0x0000,   /* U+11157 */
  0x0000,   /* U+11158 */
  0x0000,   /* U+11159 */
  0x0000,   /* U+1115A */
  0x0000,   /* U+1115B */
  0x0000,   /* U+1115C */
  0x0000,   /* U+1115D */
  0x0000,   /* U+1115E */
  0x0000,   /* U+1115F */
  0x0000,   /* U+11160 */
  0x0000,   /* U+11161 */
  0x0000,   /* U+11162 */
  0x0000,   /* U+11163 */
  0x0000,   /* U+11164 */
  0x0000,   /* U+11165 */
  0x0000,   /* U+11166 */
  0x0000,   /* U+11167 */
  0x0000,   /* U+11168 */
  0x0000,   /* U+11169 */
  0x0000,   /* U+1116A */
  0x0000,   /* U+1116B */
  0x0000,   /* U+1116C */
  0x0000,   /* U+1116D */
  0x0000,   /* U+1116E */
  0x0000,   /* U+1116F */
  0x0000,   /* U+11170 */
  0x0000,   /* U+11171 */
  0x0000,   /* U+11172 */
  0x0000,   /* U+11173 */
  0x0000,   /* U+11174 */
  0x0000,   /* U+11175 */
  0x0000,   /* U+11176 */
  0x9177,   /* U+11177 */
  0x9178,   /* U+11178 */
  0x9179,   /* U+11179 */
  0x917A,   /* U+1117A */
  0x917B,   /* U+1117B */
  0x917C,   /* U+1117C */
  0x917D,   /* U+1117D */
  0x917E,   /* U+1117E */
  0x917F,   /* U+1117F */
  0x0000,   /* U+11180 */
  0x0000,   /* U+11181 */
  0x0000,   /* U+11182 */
  0x0000,   /* U+11183 */
  0x0000,   /* U+11184 */
  0x0000,   /* U+11185 */
  0x0000,   /* U+11186 */
  0x0000,   /* U+11187 */
  0x0000,   /* U+11188 */
  0x0000,   /* U+11189 */
  0x0000,   /* U+1118A */
  0x0000,   /* U+1118B */
  0x0000,   /* U+1118C */
  0x0000,   /* U+1118D */
  0x0000,   /* U+1118E */
  0x0000,   /* U+1118F */
  0x0000,   /* U+11190 */
  0x0000,   /* U+11191 */
  0x0000,   /* U+11192 */
  0x0000,   /* U+11193 */
  0x0000,   /* U+11194 */
  0x0000,   /* U+11195 */
  0x0000,   /* U+11196 */
  0x0000,   /* U+11197 */
  0x0000,   /* U+11198 */
  0x0000,   /* U+11199 */
  0x0000,   /* U+1119A */
  0x0000,   /* U+1119B */
  0x0000,   /* U+1119C */
  0x0000,   /* U+1119D */
  0x0000,   /* U+1119E */
  0x0000,   /* U+1119F */
  0x0000,   /* U+111A0 */
  0x0000,   /* U+111A1 */
  0x0000,   /* U+111A2 */
  0x0000,   /* U+111A3 */
  0x0000,   /* U+111A4 */
  0x0000,   /* U+111A5 */
  0x0000,   /* U+111A6 */
  0x0000,   /* U+111A7 */
  0x0000,   /* U+111A8 */
  0x0000,   /* U+111A9 */
  0x0000,   /* U+111AA */
  0x0000,   /* U+111AB */
  0x0000,   /* U+111AC */
  0x0000,   /* U+111AD */
  0x0000,   /* U+111AE */
  0x0000,   /* U+111AF */
  0x0000,   /* U+111B0 */
  0x0000,   /* U+111B1 */
  0x0000,   /* U+111B2 */
  0x0000,   /* U+111B3 */
  0x0000,   /* U+111B4 */
  0x0000,   /* U+111B5 */
  0x0000,   /* U+111B6 */
  0x0000,   /* U+111B7 */
  0x0000,   /* U+111B8 */
  0x0000,   /* U+111B9 */
  0x0000,   /* U+111BA */
  0x0000,   /* U+111BB */
  0x0000,   /* U+111BC */
  0x0000,   /* U+111BD */
  0x0000,   /* U+111BE */
  0x0000,   /* U+111BF */
  0x0000,   /* U+111C0 */
  0x0000,   /* U+111C1 */
  0x0000,   /* U+111C2 */
  0x0000,   /* U+111C3 */
  0x0000,   /* U+111C4 */
  0x0000,   /* U+111C5 */
  0x0000,   /* U+111C6 */
  0x0000,   /* U+111C7 */
  0x0000,   /* U+111C8 */
  0x0000,   /* U+111C9 */
  0x0000,   /* U+111CA */
  0x0000,   /* U+111CB */
  0x0000,   /* U+111CC */
  0x0000,   /* U+111CD */
  0x91CE,   /* U+111CE */
  0x91CF,   /* U+111CF */
  0x0000,   /* U+111D0 */
  0x0000,   /* U+111D1 */
  0x0000,   /* U+111D2 */
  0x0000,   /* U+111D3 */
  0x0000,   /* U+111D4 */
  0x0000,   /* U+111D5 */
  0x0000,   /* U+111D6 */
  0x0000,   /* U+111D7 */
  0x0000,   /* U+111D8 */
  0x0000,   /* U+111D9 */
  0x0000,   /* U+111DA */
  0x0000,   /* U+111DB */
  0x0000,   /* U+111DC */
  0x0000,   /* U+111DD */
  0x0000,   /* U+111DE */
  0x0000,   /* U+111DF */
  0x91E0,   /* U+111E0 */
  0x0000,   /* U+111E1 */
  0x0000,   /* U+111E2 */
  0x0000,   /* U+111E3 */
  0x0000,   /* U+111E4 */
  0x0000,   /* U+111E5 */
  0x0000,   /* U+111E6 */
  0x0000,   /* U+111E7 */
  0x0000,   /* U+111E8 */
  0x0000,   /* U+111E9 */
  0x0000,   /* U+111EA */
  0x0000,   /* U+111EB */
  0x0000,   /* U+111EC */
  0x0000,   /* U+111ED */
  0x0000,   /* U+111EE */
  0x0000,   /* U+111EF */
  0x0000,   /* U+111F0 */
  0x0000,   /* U+111F1 */
  0x0000,   /* U+111F2 */
  0x0000,   /* U+111F3 */
  0x0000,   /* U+111F4 */
  0x91F5,   /* U+111F5 */
  0x91F6,   /* U+111F6 */
  0x91F7,   /* U+111F7 */
  0x91F8,   /* U+111F8 */
  0x91F9,   /* U+111F9 */
  0x91FA,   /* U+111FA */
  0x91FB,   /* U+111FB */
  0x91FC,   /* U+111FC */
  0x91FD,   /* U+111FD */
  0x91FE,   /* U+111FE */
  0x91FF,   /* U+111FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11100 */
  0x0000,   /* U+11101 */
  0x0000,   /* U+11102 */
  0x0000,   /* U+11103 */
  0x0000,   /* U+11104 */
  0x0000,   /* U+11105 */
  0x0000,   /* U+11106 */
  0x0000,   /* U+11107 */
  0x0000,   /* U+11108 */
  0x0000,   /* U+11109 */
  0x0000,   /* U+1110A */
  0x0000,   /* U+1110B */
  0x0000,   /* U+1110C */
  0x0000,   /* U+1110D */
  0x0000,   /* U+1110E */
  0x0000,   /* U+1110F */
  0x0000,   /* U+11110 */
  0x0000,   /* U+11111 */
  0x0000,   /* U+11112 */
  0x0000,   /* U+11113 */
  0x0000,   /* U+11114 */
  0x0000,   /* U+11115 */
  0x0000,   /* U+11116 */
  0x0000,   /* U+11117 */
  0x0000,   /* U+11118 */
  0x0000,   /* U+11119 */
  0x0000,   /* U+1111A */
  0x0000,   /* U+1111B */
  0x0000,   /* U+1111C */
  0x0000,   /* U+1111D */
  0x0000,   /* U+1111E */
  0x0000,   /* U+1111F */
  0x0000,   /* U+11120 */
  0x0000,   /* U+11121 */
  0x0000,   /* U+11122 */
  0x0000,   /* U+11123 */
  0x0000,   /* U+11124 */
  0x0000,   /* U+11125 */
  0x0000,   /* U+11126 */
  0x0000,   /* U+11127 */
  0x0000,   /* U+11128 */
  0x0000,   /* U+11129 */
  0x0000,   /* U+1112A */
  0x0000,   /* U+1112B */
  0x0000,   /* U+1112C */
  0x0000,   /* U+1112D */
  0x0000,   /* U+1112E */
  0x0000,   /* U+1112F */
  0x0000,   /* U+11130 */
  0x0000,   /* U+11131 */
  0x0000,   /* U+11132 */
  0x0000,   /* U+11133 */
  0x0000,   /* U+11134 */
  0x0000,   /* U+11135 */
  0x0000,   /* U+11136 */
  0x0000,   /* U+11137 */
  0x0000,   /* U+11138 */
  0x0000,   /* U+11139 */
  0x0000,   /* U+1113A */
  0x0000,   /* U+1113B */
  0x0000,   /* U+1113C */
  0x0000,   /* U+1113D */
  0x0000,   /* U+1113E */
  0x0000,   /* U+1113F */
  0x0000,   /* U+11140 */
  0x0000,   /* U+11141 */
  0x0000,   /* U+11142 */
  0x0000,   /* U+11143 */
  0x0000,   /* U+11144 */
  0x0000,   /* U+11145 */
  0x0000,   /* U+11146 */
  0x0000,   /* U+11147 */
  0x0000,   /* U+11148 */
  0x0000,   /* U+11149 */
  0x0000,   /* U+1114A */
  0x0000,   /* U+1114B */
  0x0000,   /* U+1114C */
  0x0000,   /* U+1114D */
  0x0000,   /* U+1114E */
  0x0000,   /* U+1114F */
  0x0000,   /* U+11150 */
  0x0000,   /* U+11151 */
  0x0000,   /* U+11152 */
  0x0000,   /* U+11153 */
  0x0000,   /* U+11154 */
  0x0000,   /* U+11155 */
  0x0000,   /* U+11156 */
  0x0000,   /* U+11157 */
  0x0000,   /* U+11158 */
  0x0000,   /* U+11159 */
  0x0000,   /* U+1115A */
  0x0000,   /* U+1115B */
  0x0000,   /* U+1115C */
  0x0000,   /* U+1115D */
  0x0000,   /* U+1115E */
  0x0000,   /* U+1115F */
  0x0000,   /* U+11160 */
  0x0000,   /* U+11161 */
  0x0000,   /* U+11162 */
  0x0000,   /* U+11163 */
  0x0000,   /* U+11164 */
  0x0000,   /* U+11165 */
  0x0000,   /* U+11166 */
  0x0000,   /* U+11167 */
  0x0000,   /* U+11168 */
  0x0000,   /* U+11169 */
  0x0000,   /* U+1116A */
  0x0000,   /* U+1116B */
  0x0000,   /* U+1116C */
  0x0000,   /* U+1116D */
  0x0000,   /* U+1116E */
  0x0000,   /* U+1116F */
  0x0000,   /* U+11170 */
  0x0000,   /* U+11171 */
  0x0000,   /* U+11172 */
  0x0000,   /* U+11173 */
  0x0000,   /* U+11174 */
  0x0000,   /* U+11175 */
  0x0000,   /* U+11176 */
  0x0000,   /* U+11177 */
  0x0000,   /* U+11178 */
  0x0000,   /* U+11179 */
  0x0000,   /* U+1117A */
  0x0000,   /* U+1117B */
  0x0000,   /* U+1117C */
  0x0000,   /* U+1117D */
  0x0000,   /* U+1117E */
  0x0000,   /* U+1117F */
  0x0000,   /* U+11180 */
  0x0000,   /* U+11181 */
  0x0000,   /* U+11182 */
  0x0000,   /* U+11183 */
  0x0000,   /* U+11184 */
  0x0000,   /* U+11185 */
  0x0000,   /* U+11186 */
  0x0000,   /* U+11187 */
  0x0000,   /* U+11188 */
  0x0000,   /* U+11189 */
  0x0000,   /* U+1118A */
  0x0000,   /* U+1118B */
  0x0000,   /* U+1118C */
  0x0000,   /* U+1118D */
  0x0000,   /* U+1118E */
  0x0000,   /* U+1118F */
  0x0000,   /* U+11190 */
  0x0000,   /* U+11191 */
  0x0000,   /* U+11192 */
  0x0000,   /* U+11193 */
  0x0000,   /* U+11194 */
  0x0000,   /* U+11195 */
  0x0000,   /* U+11196 */
  0x0000,   /* U+11197 */
  0x0000,   /* U+11198 */
  0x0000,   /* U+11199 */
  0x0000,   /* U+1119A */
  0x0000,   /* U+1119B */
  0x0000,   /* U+1119C */
  0x0000,   /* U+1119D */
  0x0000,   /* U+1119E */
  0x0000,   /* U+1119F */
  0x0000,   /* U+111A0 */
  0x0000,   /* U+111A1 */
  0x0000,   /* U+111A2 */
  0x0000,   /* U+111A3 */
  0x0000,   /* U+111A4 */
  0x0000,   /* U+111A5 */
  0x0000,   /* U+111A6 */
  0x0000,   /* U+111A7 */
  0x0000,   /* U+111A8 */
  0x0000,   /* U+111A9 */
  0x0000,   /* U+111AA */
  0x0000,   /* U+111AB */
  0x0000,   /* U+111AC */
  0x0000,   /* U+111AD */
  0x0000,   /* U+111AE */
  0x0000,   /* U+111AF */
  0x0000,   /* U+111B0 */
  0x0000,   /* U+111B1 */
  0x0000,   /* U+111B2 */
  0x0000,   /* U+111B3 */
  0x0000,   /* U+111B4 */
  0x0000,   /* U+111B5 */
  0x0000,   /* U+111B6 */
  0x0000,   /* U+111B7 */
  0x0000,   /* U+111B8 */
  0x0000,   /* U+111B9 */
  0x0000,   /* U+111BA */
  0x0000,   /* U+111BB */
  0x0000,   /* U+111BC */
  0x0000,   /* U+111BD */
  0x0000,   /* U+111BE */
  0x0000,   /* U+111BF */
  0x0000,   /* U+111C0 */
  0x0000,   /* U+111C1 */
  0x0000,   /* U+111C2 */
  0x0000,   /* U+111C3 */
  0x0000,   /* U+111C4 */
  0x0000,   /* U+111C5 */
  0x0000,   /* U+111C6 */
  0x0000,   /* U+111C7 */
  0x0000,   /* U+111C8 */
  0x0000,   /* U+111C9 */
  0x0000,   /* U+111CA */
  0x0000,   /* U+111CB */
  0x0000,   /* U+111CC */
  0x0000,   /* U+111CD */
  0x0000,   /* U+111CE */
  0x0000,   /* U+111CF */
  0x0000,   /* U+111D0 */
  0x0000,   /* U+111D1 */
  0x0000,   /* U+111D2 */
  0x0000,   /* U+111D3 */
  0x0000,   /* U+111D4 */
  0x0000,   /* U+111D5 */
  0x0000,   /* U+111D6 */
  0x0000,   /* U+111D7 */
  0x0000,   /* U+111D8 */
  0x0000,   /* U+111D9 */
  0x0000,   /* U+111DA */
  0x0000,   /* U+111DB */
  0x0000,   /* U+111DC */
  0x0000,   /* U+111DD */
  0x0000,   /* U+111DE */
  0x0000,   /* U+111DF */
  0x0000,   /* U+111E0 */
  0x0000,   /* U+111E1 */
  0x0000,   /* U+111E2 */
  0x0000,   /* U+111E3 */
  0x0000,   /* U+111E4 */
  0x0000,   /* U+111E5 */
  0x0000,   /* U+111E6 */
  0x0000,   /* U+111E7 */
  0x0000,   /* U+111E8 */
  0x0000,   /* U+111E9 */
  0x0000,   /* U+111EA */
  0x0000,   /* U+111EB */
  0x0000,   /* U+111EC */
  0x0000,   /* U+111ED */
  0x0000,   /* U+111EE */
  0x0000,   /* U+111EF */
  0x0000,   /* U+111F0 */
  0x0000,   /* U+111F1 */
  0x0000,   /* U+111F2 */
  0x0000,   /* U+111F3 */
  0x0000,   /* U+111F4 */
  0x0000,   /* U+111F5 */
  0x0000,   /* U+111F6 */
  0x0000,   /* U+111F7 */
  0x0000,   /* U+111F8 */
  0x0000,   /* U+111F9 */
  0x0000,   /* U+111FA */
  0x0000,   /* U+111FB */
  0x0000,   /* U+111FC */
  0x0000,   /* U+111FD */
  0x0000,   /* U+111FE */
  0x0000,   /* U+111FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11100 */
  0x0000,   /* U+11101 */
  0x0000,   /* U+11102 */
  0x0000,   /* U+11103 */
  0x0000,   /* U+11104 */
  0x0000,   /* U+11105 */
  0x0000,   /* U+11106 */
  0x0000,   /* U+11107 */
  0x0000,   /* U+11108 */
  0x0000,   /* U+11109 */
  0x0000,   /* U+1110A */
  0x0000,   /* U+1110B */
  0x0000,   /* U+1110C */
  0x0000,   /* U+1110D */
  0x0000,   /* U+1110E */
  0x0000,   /* U+1110F */
  0x0000,   /* U+11110 */
  0x0000,   /* U+11111 */
  0x0000,   /* U+11112 */
  0x0000,   /* U+11113 */
  0x0000,   /* U+11114 */
  0x0000,   /* U+11115 */
  0x0000,   /* U+11116 */
  0x0000,   /* U+11117 */
  0x0000,   /* U+11118 */
  0x0000,   /* U+11119 */
  0x0000,   /* U+1111A */
  0x0000,   /* U+1111B */
  0x0000,   /* U+1111C */
  0x0000,   /* U+1111D */
  0x0000,   /* U+1111E */
  0x0000,   /* U+1111F */
  0x0000,   /* U+11120 */
  0x0000,   /* U+11121 */
  0x0000,   /* U+11122 */
  0x0000,   /* U+11123 */
  0x0000,   /* U+11124 */
  0x0000,   /* U+11125 */
  0x0000,   /* U+11126 */
  0x0000,   /* U+11127 */
  0x0000,   /* U+11128 */
  0x0000,   /* U+11129 */
  0x0000,   /* U+1112A */
  0x0000,   /* U+1112B */
  0x0000,   /* U+1112C */
  0x0000,   /* U+1112D */
  0x0000,   /* U+1112E */
  0x0000,   /* U+1112F */
  0x0000,   /* U+11130 */
  0x0000,   /* U+11131 */
  0x0000,   /* U+11132 */
  0x0000,   /* U+11133 */
  0x0000,   /* U+11134 */
  0x0000,   /* U+11135 */
  0x0000,   /* U+11136 */
  0x0000,   /* U+11137 */
  0x0000,   /* U+11138 */
  0x0000,   /* U+11139 */
  0x0000,   /* U+1113A */
  0x0000,   /* U+1113B */
  0x0000,   /* U+1113C */
  0x0000,   /* U+1113D */
  0x0000,   /* U+1113E */
  0x0000,   /* U+1113F */
  0x0000,   /* U+11140 */
  0x0000,   /* U+11141 */
  0x0000,   /* U+11142 */
  0x0000,   /* U+11143 */
  0x0000,   /* U+11144 */
  0x0000,   /* U+11145 */
  0x0000,   /* U+11146 */
  0x0000,   /* U+11147 */
  0x0000,   /* U+11148 */
  0x0000,   /* U+11149 */
  0x0000,   /* U+1114A */
  0x0000,   /* U+1114B */
  0x0000,   /* U+1114C */
  0x0000,   /* U+1114D */
  0x0000,   /* U+1114E */
  0x0000,   /* U+1114F */
  0x0000,   /* U+11150 */
  0x0000,   /* U+11151 */
  0x0000,   /* U+11152 */
  0x0000,   /* U+11153 */
  0x0000,   /* U+11154 */
  0x0000,   /* U+11155 */
  0x0000,   /* U+11156 */
  0x0000,   /* U+11157 */
  0x0000,   /* U+11158 */
  0x0000,   /* U+11159 */
  0x0000,   /* U+1115A */
  0x0000,   /* U+1115B */
  0x0000,   /* U+1115C */
  0x0000,   /* U+1115D */
  0x0000,   /* U+1115E */
  0x0000,   /* U+1115F */
  0x0000,   /* U+11160 */
  0x0000,   /* U+11161 */
  0x0000,   /* U+11162 */
  0x0000,   /* U+11163 */
  0x0000,   /* U+11164 */
  0x0000,   /* U+11165 */
  0x0000,   /* U+11166 */
  0x0000,   /* U+11167 */
  0x0000,   /* U+11168 */
  0x0000,   /* U+11169 */
  0x0000,   /* U+1116A */
  0x0000,   /* U+1116B */
  0x0000,   /* U+1116C */
  0x0000,   /* U+1116D */
  0x0000,   /* U+1116E */
  0x0000,   /* U+1116F */
  0x0000,   /* U+11170 */
  0x0000,   /* U+11171 */
  0x0000,   /* U+11172 */
  0x0000,   /* U+11173 */
  0x0000,   /* U+11174 */
  0x0000,   /* U+11175 */
  0x0000,   /* U+11176 */
  0x0000,   /* U+11177 */
  0x0000,   /* U+11178 */
  0x0000,   /* U+11179 */
  0x0000,   /* U+1117A */
  0x0000,   /* U+1117B */
  0x0000,   /* U+1117C */
  0x0000,   /* U+1117D */
  0x0000,   /* U+1117E */
  0x0000,   /* U+1117F */
  0x0000,   /* U+11180 */
  0x0000,   /* U+11181 */
  0x0000,   /* U+11182 */
  0x0000,   /* U+11183 */
  0x0000,   /* U+11184 */
  0x0000,   /* U+11185 */
  0x0000,   /* U+11186 */
  0x0000,   /* U+11187 */
  0x0000,   /* U+11188 */
  0x0000,   /* U+11189 */
  0x0000,   /* U+1118A */
  0x0000,   /* U+1118B */
  0x0000,   /* U+1118C */
  0x0000,   /* U+1118D */
  0x0000,   /* U+1118E */
  0x0000,   /* U+1118F */
  0x0000,   /* U+11190 */
  0x0000,   /* U+11191 */
  0x0000,   /* U+11192 */
  0x0000,   /* U+11193 */
  0x0000,   /* U+11194 */
  0x0000,   /* U+11195 */
  0x0000,   /* U+11196 */
  0x0000,   /* U+11197 */
  0x0000,   /* U+11198 */
  0x0000,   /* U+11199 */
  0x0000,   /* U+1119A */
  0x0000,   /* U+1119B */
  0x0000,   /* U+1119C */
  0x0000,   /* U+1119D */
  0x0000,   /* U+1119E */
  0x0000,   /* U+1119F */
  0x0000,   /* U+111A0 */
  0x0000,   /* U+111A1 */
  0x0000,   /* U+111A2 */
  0x0000,   /* U+111A3 */
  0x0000,   /* U+111A4 */
  0x0000,   /* U+111A5 */
  0x0000,   /* U+111A6 */
  0x0000,   /* U+111A7 */
  0x0000,   /* U+111A8 */
  0x0000,   /* U+111A9 */
  0x0000,   /* U+111AA */
  0x0000,   /* U+111AB */
  0x0000,   /* U+111AC */
  0x0000,   /* U+111AD */
  0x0000,   /* U+111AE */
  0x0000,   /* U+111AF */
  0x0000,   /* U+111B0 */
  0x0000,   /* U+111B1 */
  0x0000,   /* U+111B2 */
  0x0000,   /* U+111B3 */
  0x0000,   /* U+111B4 */
  0x0000,   /* U+111B5 */
  0x0000,   /* U+111B6 */
  0x0000,   /* U+111B7 */
  0x0000,   /* U+111B8 */
  0x0000,   /* U+111B9 */
  0x0000,   /* U+111BA */
  0x0000,   /* U+111BB */
  0x0000,   /* U+111BC */
  0x0000,   /* U+111BD */
  0x0000,   /* U+111BE */
  0x0000,   /* U+111BF */
  0x0000,   /* U+111C0 */
  0x0000,   /* U+111C1 */
  0x0000,   /* U+111C2 */
  0x0000,   /* U+111C3 */
  0x0000,   /* U+111C4 */
  0x0000,   /* U+111C5 */
  0x0000,   /* U+111C6 */
  0x0000,   /* U+111C7 */
  0x0000,   /* U+111C8 */
  0x0000,   /* U+111C9 */
  0x0000,   /* U+111CA */
  0x0000,   /* U+111CB */
  0x0000,   /* U+111CC */
  0x0000,   /* U+111CD */
  0x0000,   /* U+111CE */
  0x0000,   /* U+111CF */
  0x0000,   /* U+111D0 */
  0x0000,   /* U+111D1 */
  0x0000,   /* U+111D2 */
  0x0000,   /* U+111D3 */
  0x0000,   /* U+111D4 */
  0x0000,   /* U+111D5 */
  0x0000,   /* U+111D6 */
  0x0000,   /* U+111D7 */
  0x0000,   /* U+111D8 */
  0x0000,   /* U+111D9 */
  0x0000,   /* U+111DA */
  0x0000,   /* U+111DB */
  0x0000,   /* U+111DC */
  0x0000,   /* U+111DD */
  0x0000,   /* U+111DE */
  0x0000,   /* U+111DF */
  0x0000,   /* U+111E0 */
  0x0000,   /* U+111E1 */
  0x0000,   /* U+111E2 */
  0x0000,   /* U+111E3 */
  0x0000,   /* U+111E4 */
  0x0000,   /* U+111E5 */
  0x0000,   /* U+111E6 */
  0x0000,   /* U+111E7 */
  0x0000,   /* U+111E8 */
  0x0000,   /* U+111E9 */
  0x0000,   /* U+111EA */
  0x0000,   /* U+111EB */
  0x0000,   /* U+111EC */
  0x0000,   /* U+111ED */
  0x0000,   /* U+111EE */
  0x0000,   /* U+111EF */
  0x0000,   /* U+111F0 */
  0x0000,   /* U+111F1 */
  0x0000,   /* U+111F2 */
  0x0000,   /* U+111F3 */
  0x0000,   /* U+111F4 */
  0x0000,   /* U+111F5 */
  0x0000,   /* U+111F6 */
  0x0000,   /* U+111F7 */
  0x0000,   /* U+111F8 */
  0x0000,   /* U+111F9 */
  0x0000,   /* U+111FA */
  0x0000,   /* U+111FB */
  0x0000,   /* U+111FC */
  0x0000,   /* U+111FD */
  0x0000,   /* U+111FE */
  0x0000,   /* U+111FF */
};

uint16 uca900_p112[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2A62,   /* U+11200 */
  0x2A63,   /* U+11201 */
  0x2A64,   /* U+11202 */
  0x2A65,   /* U+11203 */
  0x2A66,   /* U+11204 */
  0x2A67,   /* U+11205 */
  0x2A68,   /* U+11206 */
  0x2A69,   /* U+11207 */
  0x2A6A,   /* U+11208 */
  0x2A6B,   /* U+11209 */
  0x2A6C,   /* U+1120A */
  0x2A6D,   /* U+1120B */
  0x2A6E,   /* U+1120C */
  0x2A6F,   /* U+1120D */
  0x2A70,   /* U+1120E */
  0x2A71,   /* U+1120F */
  0x2A72,   /* U+11210 */
  0x2A73,   /* U+11211 */
  0xFBC2,   /* U+11212 */
  0x2A74,   /* U+11213 */
  0x2A75,   /* U+11214 */
  0x2A76,   /* U+11215 */
  0x2A77,   /* U+11216 */
  0x2A78,   /* U+11217 */
  0x2A79,   /* U+11218 */
  0x2A7A,   /* U+11219 */
  0x2A7B,   /* U+1121A */
  0x2A7C,   /* U+1121B */
  0x2A7D,   /* U+1121C */
  0x2A7E,   /* U+1121D */
  0x2A7F,   /* U+1121E */
  0x2A80,   /* U+1121F */
  0x2A81,   /* U+11220 */
  0x2A82,   /* U+11221 */
  0x2A83,   /* U+11222 */
  0x2A84,   /* U+11223 */
  0x2A85,   /* U+11224 */
  0x2A86,   /* U+11225 */
  0x2A87,   /* U+11226 */
  0x2A88,   /* U+11227 */
  0x2A89,   /* U+11228 */
  0x2A8A,   /* U+11229 */
  0x2A8B,   /* U+1122A */
  0x2A8C,   /* U+1122B */
  0x2A8D,   /* U+1122C */
  0x2A8E,   /* U+1122D */
  0x2A8F,   /* U+1122E */
  0x2A90,   /* U+1122F */
  0x2A91,   /* U+11230 */
  0x2A92,   /* U+11231 */
  0x2A93,   /* U+11232 */
  0x2A94,   /* U+11233 */
  0x0000,   /* U+11234 */
  0x2A95,   /* U+11235 */
  0x0000,   /* U+11236 */
  0x0000,   /* U+11237 */
  0x02B4,   /* U+11238 */
  0x02B5,   /* U+11239 */
  0x0457,   /* U+1123A */
  0x0458,   /* U+1123B */
  0x0459,   /* U+1123C */
  0x045A,   /* U+1123D */
  0x0000,   /* U+1123E */
  0xFBC2,   /* U+1123F */
  0xFBC2,   /* U+11240 */
  0xFBC2,   /* U+11241 */
  0xFBC2,   /* U+11242 */
  0xFBC2,   /* U+11243 */
  0xFBC2,   /* U+11244 */
  0xFBC2,   /* U+11245 */
  0xFBC2,   /* U+11246 */
  0xFBC2,   /* U+11247 */
  0xFBC2,   /* U+11248 */
  0xFBC2,   /* U+11249 */
  0xFBC2,   /* U+1124A */
  0xFBC2,   /* U+1124B */
  0xFBC2,   /* U+1124C */
  0xFBC2,   /* U+1124D */
  0xFBC2,   /* U+1124E */
  0xFBC2,   /* U+1124F */
  0xFBC2,   /* U+11250 */
  0xFBC2,   /* U+11251 */
  0xFBC2,   /* U+11252 */
  0xFBC2,   /* U+11253 */
  0xFBC2,   /* U+11254 */
  0xFBC2,   /* U+11255 */
  0xFBC2,   /* U+11256 */
  0xFBC2,   /* U+11257 */
  0xFBC2,   /* U+11258 */
  0xFBC2,   /* U+11259 */
  0xFBC2,   /* U+1125A */
  0xFBC2,   /* U+1125B */
  0xFBC2,   /* U+1125C */
  0xFBC2,   /* U+1125D */
  0xFBC2,   /* U+1125E */
  0xFBC2,   /* U+1125F */
  0xFBC2,   /* U+11260 */
  0xFBC2,   /* U+11261 */
  0xFBC2,   /* U+11262 */
  0xFBC2,   /* U+11263 */
  0xFBC2,   /* U+11264 */
  0xFBC2,   /* U+11265 */
  0xFBC2,   /* U+11266 */
  0xFBC2,   /* U+11267 */
  0xFBC2,   /* U+11268 */
  0xFBC2,   /* U+11269 */
  0xFBC2,   /* U+1126A */
  0xFBC2,   /* U+1126B */
  0xFBC2,   /* U+1126C */
  0xFBC2,   /* U+1126D */
  0xFBC2,   /* U+1126E */
  0xFBC2,   /* U+1126F */
  0xFBC2,   /* U+11270 */
  0xFBC2,   /* U+11271 */
  0xFBC2,   /* U+11272 */
  0xFBC2,   /* U+11273 */
  0xFBC2,   /* U+11274 */
  0xFBC2,   /* U+11275 */
  0xFBC2,   /* U+11276 */
  0xFBC2,   /* U+11277 */
  0xFBC2,   /* U+11278 */
  0xFBC2,   /* U+11279 */
  0xFBC2,   /* U+1127A */
  0xFBC2,   /* U+1127B */
  0xFBC2,   /* U+1127C */
  0xFBC2,   /* U+1127D */
  0xFBC2,   /* U+1127E */
  0xFBC2,   /* U+1127F */
  0x2ACF,   /* U+11280 */
  0x2AD0,   /* U+11281 */
  0x2AD1,   /* U+11282 */
  0x2AD2,   /* U+11283 */
  0x2AD5,   /* U+11284 */
  0x2AD6,   /* U+11285 */
  0x2AD7,   /* U+11286 */
  0xFBC2,   /* U+11287 */
  0x2AD8,   /* U+11288 */
  0xFBC2,   /* U+11289 */
  0x2AD9,   /* U+1128A */
  0x2ADA,   /* U+1128B */
  0x2ADB,   /* U+1128C */
  0x2ADC,   /* U+1128D */
  0xFBC2,   /* U+1128E */
  0x2ADD,   /* U+1128F */
  0x2ADE,   /* U+11290 */
  0x2ADF,   /* U+11291 */
  0x2AE0,   /* U+11292 */
  0x2AE1,   /* U+11293 */
  0x2AE2,   /* U+11294 */
  0x2AE3,   /* U+11295 */
  0x2AE4,   /* U+11296 */
  0x2AE5,   /* U+11297 */
  0x2AE6,   /* U+11298 */
  0x2AE7,   /* U+11299 */
  0x2AE8,   /* U+1129A */
  0x2AE9,   /* U+1129B */
  0x2AEA,   /* U+1129C */
  0x2AEB,   /* U+1129D */
  0xFBC2,   /* U+1129E */
  0x2AEC,   /* U+1129F */
  0x2AED,   /* U+112A0 */
  0x2AEE,   /* U+112A1 */
  0x2AEF,   /* U+112A2 */
  0x2AF0,   /* U+112A3 */
  0x2AF1,   /* U+112A4 */
  0x2AD3,   /* U+112A5 */
  0x2AD4,   /* U+112A6 */
  0x2AF2,   /* U+112A7 */
  0x2AF3,   /* U+112A8 */
  0x02E6,   /* U+112A9 */
  0xFBC2,   /* U+112AA */
  0xFBC2,   /* U+112AB */
  0xFBC2,   /* U+112AC */
  0xFBC2,   /* U+112AD */
  0xFBC2,   /* U+112AE */
  0xFBC2,   /* U+112AF */
  0x2A96,   /* U+112B0 */
  0x2A97,   /* U+112B1 */
  0x2A98,   /* U+112B2 */
  0x2A99,   /* U+112B3 */
  0x2A9A,   /* U+112B4 */
  0x2A9B,   /* U+112B5 */
  0x2A9C,   /* U+112B6 */
  0x2A9D,   /* U+112B7 */
  0x2A9E,   /* U+112B8 */
  0x2A9F,   /* U+112B9 */
  0x2AA0,   /* U+112BA */
  0x2AA1,   /* U+112BB */
  0x2AA2,   /* U+112BC */
  0x2AA3,   /* U+112BD */
  0x2AA4,   /* U+112BE */
  0x2AA5,   /* U+112BF */
  0x2AA6,   /* U+112C0 */
  0x2AA7,   /* U+112C1 */
  0x2AA8,   /* U+112C2 */
  0x2AA9,   /* U+112C3 */
  0x2AAA,   /* U+112C4 */
  0x2AAB,   /* U+112C5 */
  0x2AAC,   /* U+112C6 */
  0x2AAD,   /* U+112C7 */
  0x2AAE,   /* U+112C8 */
  0x2AAF,   /* U+112C9 */
  0x2AB0,   /* U+112CA */
  0x2AB1,   /* U+112CB */
  0x2AB2,   /* U+112CC */
  0x2AB3,   /* U+112CD */
  0x2AB4,   /* U+112CE */
  0x2AB5,   /* U+112CF */
  0x2AB6,   /* U+112D0 */
  0x2AB7,   /* U+112D1 */
  0x2AB8,   /* U+112D2 */
  0x2AB9,   /* U+112D3 */
  0x2ABA,   /* U+112D4 */
  0x2ABB,   /* U+112D5 */
  0x2ABC,   /* U+112D6 */
  0x2ABD,   /* U+112D7 */
  0x2ABE,   /* U+112D8 */
  0x2ABF,   /* U+112D9 */
  0x2AC0,   /* U+112DA */
  0x2AC1,   /* U+112DB */
  0x2AC2,   /* U+112DC */
  0x2AC3,   /* U+112DD */
  0x2AC4,   /* U+112DE */
  0x0000,   /* U+112DF */
  0x2AC5,   /* U+112E0 */
  0x2AC6,   /* U+112E1 */
  0x2AC7,   /* U+112E2 */
  0x2AC8,   /* U+112E3 */
  0x2AC9,   /* U+112E4 */
  0x2ACA,   /* U+112E5 */
  0x2ACB,   /* U+112E6 */
  0x2ACC,   /* U+112E7 */
  0x2ACD,   /* U+112E8 */
  0x0000,   /* U+112E9 */
  0x2ACE,   /* U+112EA */
  0xFBC2,   /* U+112EB */
  0xFBC2,   /* U+112EC */
  0xFBC2,   /* U+112ED */
  0xFBC2,   /* U+112EE */
  0xFBC2,   /* U+112EF */
  0x1C3D,   /* U+112F0 */
  0x1C3E,   /* U+112F1 */
  0x1C3F,   /* U+112F2 */
  0x1C40,   /* U+112F3 */
  0x1C41,   /* U+112F4 */
  0x1C42,   /* U+112F5 */
  0x1C43,   /* U+112F6 */
  0x1C44,   /* U+112F7 */
  0x1C45,   /* U+112F8 */
  0x1C46,   /* U+112F9 */
  0xFBC2,   /* U+112FA */
  0xFBC2,   /* U+112FB */
  0xFBC2,   /* U+112FC */
  0xFBC2,   /* U+112FD */
  0xFBC2,   /* U+112FE */
  0xFBC2,   /* U+112FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11200 */
  0x0020,   /* U+11201 */
  0x0020,   /* U+11202 */
  0x0020,   /* U+11203 */
  0x0020,   /* U+11204 */
  0x0020,   /* U+11205 */
  0x0020,   /* U+11206 */
  0x0020,   /* U+11207 */
  0x0020,   /* U+11208 */
  0x0020,   /* U+11209 */
  0x0020,   /* U+1120A */
  0x0020,   /* U+1120B */
  0x0020,   /* U+1120C */
  0x0020,   /* U+1120D */
  0x0020,   /* U+1120E */
  0x0020,   /* U+1120F */
  0x0020,   /* U+11210 */
  0x0020,   /* U+11211 */
  0x0020,   /* U+11212 */
  0x0020,   /* U+11213 */
  0x0020,   /* U+11214 */
  0x0020,   /* U+11215 */
  0x0020,   /* U+11216 */
  0x0020,   /* U+11217 */
  0x0020,   /* U+11218 */
  0x0020,   /* U+11219 */
  0x0020,   /* U+1121A */
  0x0020,   /* U+1121B */
  0x0020,   /* U+1121C */
  0x0020,   /* U+1121D */
  0x0020,   /* U+1121E */
  0x0020,   /* U+1121F */
  0x0020,   /* U+11220 */
  0x0020,   /* U+11221 */
  0x0020,   /* U+11222 */
  0x0020,   /* U+11223 */
  0x0020,   /* U+11224 */
  0x0020,   /* U+11225 */
  0x0020,   /* U+11226 */
  0x0020,   /* U+11227 */
  0x0020,   /* U+11228 */
  0x0020,   /* U+11229 */
  0x0020,   /* U+1122A */
  0x0020,   /* U+1122B */
  0x0020,   /* U+1122C */
  0x0020,   /* U+1122D */
  0x0020,   /* U+1122E */
  0x0020,   /* U+1122F */
  0x0020,   /* U+11230 */
  0x0020,   /* U+11231 */
  0x0020,   /* U+11232 */
  0x0020,   /* U+11233 */
  0x00C4,   /* U+11234 */
  0x0020,   /* U+11235 */
  0x00C2,   /* U+11236 */
  0x0080,   /* U+11237 */
  0x0020,   /* U+11238 */
  0x0020,   /* U+11239 */
  0x0020,   /* U+1123A */
  0x0020,   /* U+1123B */
  0x0020,   /* U+1123C */
  0x0020,   /* U+1123D */
  0x0081,   /* U+1123E */
  0x0020,   /* U+1123F */
  0x0020,   /* U+11240 */
  0x0020,   /* U+11241 */
  0x0020,   /* U+11242 */
  0x0020,   /* U+11243 */
  0x0020,   /* U+11244 */
  0x0020,   /* U+11245 */
  0x0020,   /* U+11246 */
  0x0020,   /* U+11247 */
  0x0020,   /* U+11248 */
  0x0020,   /* U+11249 */
  0x0020,   /* U+1124A */
  0x0020,   /* U+1124B */
  0x0020,   /* U+1124C */
  0x0020,   /* U+1124D */
  0x0020,   /* U+1124E */
  0x0020,   /* U+1124F */
  0x0020,   /* U+11250 */
  0x0020,   /* U+11251 */
  0x0020,   /* U+11252 */
  0x0020,   /* U+11253 */
  0x0020,   /* U+11254 */
  0x0020,   /* U+11255 */
  0x0020,   /* U+11256 */
  0x0020,   /* U+11257 */
  0x0020,   /* U+11258 */
  0x0020,   /* U+11259 */
  0x0020,   /* U+1125A */
  0x0020,   /* U+1125B */
  0x0020,   /* U+1125C */
  0x0020,   /* U+1125D */
  0x0020,   /* U+1125E */
  0x0020,   /* U+1125F */
  0x0020,   /* U+11260 */
  0x0020,   /* U+11261 */
  0x0020,   /* U+11262 */
  0x0020,   /* U+11263 */
  0x0020,   /* U+11264 */
  0x0020,   /* U+11265 */
  0x0020,   /* U+11266 */
  0x0020,   /* U+11267 */
  0x0020,   /* U+11268 */
  0x0020,   /* U+11269 */
  0x0020,   /* U+1126A */
  0x0020,   /* U+1126B */
  0x0020,   /* U+1126C */
  0x0020,   /* U+1126D */
  0x0020,   /* U+1126E */
  0x0020,   /* U+1126F */
  0x0020,   /* U+11270 */
  0x0020,   /* U+11271 */
  0x0020,   /* U+11272 */
  0x0020,   /* U+11273 */
  0x0020,   /* U+11274 */
  0x0020,   /* U+11275 */
  0x0020,   /* U+11276 */
  0x0020,   /* U+11277 */
  0x0020,   /* U+11278 */
  0x0020,   /* U+11279 */
  0x0020,   /* U+1127A */
  0x0020,   /* U+1127B */
  0x0020,   /* U+1127C */
  0x0020,   /* U+1127D */
  0x0020,   /* U+1127E */
  0x0020,   /* U+1127F */
  0x0020,   /* U+11280 */
  0x0020,   /* U+11281 */
  0x0020,   /* U+11282 */
  0x0020,   /* U+11283 */
  0x0020,   /* U+11284 */
  0x0020,   /* U+11285 */
  0x0020,   /* U+11286 */
  0x0020,   /* U+11287 */
  0x0020,   /* U+11288 */
  0x0020,   /* U+11289 */
  0x0020,   /* U+1128A */
  0x0020,   /* U+1128B */
  0x0020,   /* U+1128C */
  0x0020,   /* U+1128D */
  0x0020,   /* U+1128E */
  0x0020,   /* U+1128F */
  0x0020,   /* U+11290 */
  0x0020,   /* U+11291 */
  0x0020,   /* U+11292 */
  0x0020,   /* U+11293 */
  0x0020,   /* U+11294 */
  0x0020,   /* U+11295 */
  0x0020,   /* U+11296 */
  0x0020,   /* U+11297 */
  0x0020,   /* U+11298 */
  0x0020,   /* U+11299 */
  0x0020,   /* U+1129A */
  0x0020,   /* U+1129B */
  0x0020,   /* U+1129C */
  0x0020,   /* U+1129D */
  0x0020,   /* U+1129E */
  0x0020,   /* U+1129F */
  0x0020,   /* U+112A0 */
  0x0020,   /* U+112A1 */
  0x0020,   /* U+112A2 */
  0x0020,   /* U+112A3 */
  0x0020,   /* U+112A4 */
  0x0020,   /* U+112A5 */
  0x0020,   /* U+112A6 */
  0x0020,   /* U+112A7 */
  0x0020,   /* U+112A8 */
  0x0020,   /* U+112A9 */
  0x0020,   /* U+112AA */
  0x0020,   /* U+112AB */
  0x0020,   /* U+112AC */
  0x0020,   /* U+112AD */
  0x0020,   /* U+112AE */
  0x0020,   /* U+112AF */
  0x0020,   /* U+112B0 */
  0x0020,   /* U+112B1 */
  0x0020,   /* U+112B2 */
  0x0020,   /* U+112B3 */
  0x0020,   /* U+112B4 */
  0x0020,   /* U+112B5 */
  0x0020,   /* U+112B6 */
  0x0020,   /* U+112B7 */
  0x0020,   /* U+112B8 */
  0x0020,   /* U+112B9 */
  0x0020,   /* U+112BA */
  0x0020,   /* U+112BB */
  0x0020,   /* U+112BC */
  0x0020,   /* U+112BD */
  0x0020,   /* U+112BE */
  0x0020,   /* U+112BF */
  0x0020,   /* U+112C0 */
  0x0020,   /* U+112C1 */
  0x0020,   /* U+112C2 */
  0x0020,   /* U+112C3 */
  0x0020,   /* U+112C4 */
  0x0020,   /* U+112C5 */
  0x0020,   /* U+112C6 */
  0x0020,   /* U+112C7 */
  0x0020,   /* U+112C8 */
  0x0020,   /* U+112C9 */
  0x0020,   /* U+112CA */
  0x0020,   /* U+112CB */
  0x0020,   /* U+112CC */
  0x0020,   /* U+112CD */
  0x0020,   /* U+112CE */
  0x0020,   /* U+112CF */
  0x0020,   /* U+112D0 */
  0x0020,   /* U+112D1 */
  0x0020,   /* U+112D2 */
  0x0020,   /* U+112D3 */
  0x0020,   /* U+112D4 */
  0x0020,   /* U+112D5 */
  0x0020,   /* U+112D6 */
  0x0020,   /* U+112D7 */
  0x0020,   /* U+112D8 */
  0x0020,   /* U+112D9 */
  0x0020,   /* U+112DA */
  0x0020,   /* U+112DB */
  0x0020,   /* U+112DC */
  0x0020,   /* U+112DD */
  0x0020,   /* U+112DE */
  0x00C4,   /* U+112DF */
  0x0020,   /* U+112E0 */
  0x0020,   /* U+112E1 */
  0x0020,   /* U+112E2 */
  0x0020,   /* U+112E3 */
  0x0020,   /* U+112E4 */
  0x0020,   /* U+112E5 */
  0x0020,   /* U+112E6 */
  0x0020,   /* U+112E7 */
  0x0020,   /* U+112E8 */
  0x00C2,   /* U+112E9 */
  0x0020,   /* U+112EA */
  0x0020,   /* U+112EB */
  0x0020,   /* U+112EC */
  0x0020,   /* U+112ED */
  0x0020,   /* U+112EE */
  0x0020,   /* U+112EF */
  0x0020,   /* U+112F0 */
  0x0020,   /* U+112F1 */
  0x0020,   /* U+112F2 */
  0x0020,   /* U+112F3 */
  0x0020,   /* U+112F4 */
  0x0020,   /* U+112F5 */
  0x0020,   /* U+112F6 */
  0x0020,   /* U+112F7 */
  0x0020,   /* U+112F8 */
  0x0020,   /* U+112F9 */
  0x0020,   /* U+112FA */
  0x0020,   /* U+112FB */
  0x0020,   /* U+112FC */
  0x0020,   /* U+112FD */
  0x0020,   /* U+112FE */
  0x0020,   /* U+112FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11200 */
  0x0002,   /* U+11201 */
  0x0002,   /* U+11202 */
  0x0002,   /* U+11203 */
  0x0002,   /* U+11204 */
  0x0002,   /* U+11205 */
  0x0002,   /* U+11206 */
  0x0002,   /* U+11207 */
  0x0002,   /* U+11208 */
  0x0002,   /* U+11209 */
  0x0002,   /* U+1120A */
  0x0002,   /* U+1120B */
  0x0002,   /* U+1120C */
  0x0002,   /* U+1120D */
  0x0002,   /* U+1120E */
  0x0002,   /* U+1120F */
  0x0002,   /* U+11210 */
  0x0002,   /* U+11211 */
  0x0002,   /* U+11212 */
  0x0002,   /* U+11213 */
  0x0002,   /* U+11214 */
  0x0002,   /* U+11215 */
  0x0002,   /* U+11216 */
  0x0002,   /* U+11217 */
  0x0002,   /* U+11218 */
  0x0002,   /* U+11219 */
  0x0002,   /* U+1121A */
  0x0002,   /* U+1121B */
  0x0002,   /* U+1121C */
  0x0002,   /* U+1121D */
  0x0002,   /* U+1121E */
  0x0002,   /* U+1121F */
  0x0002,   /* U+11220 */
  0x0002,   /* U+11221 */
  0x0002,   /* U+11222 */
  0x0002,   /* U+11223 */
  0x0002,   /* U+11224 */
  0x0002,   /* U+11225 */
  0x0002,   /* U+11226 */
  0x0002,   /* U+11227 */
  0x0002,   /* U+11228 */
  0x0002,   /* U+11229 */
  0x0002,   /* U+1122A */
  0x0002,   /* U+1122B */
  0x0002,   /* U+1122C */
  0x0002,   /* U+1122D */
  0x0002,   /* U+1122E */
  0x0002,   /* U+1122F */
  0x0002,   /* U+11230 */
  0x0002,   /* U+11231 */
  0x0002,   /* U+11232 */
  0x0002,   /* U+11233 */
  0x0002,   /* U+11234 */
  0x0002,   /* U+11235 */
  0x0002,   /* U+11236 */
  0x0002,   /* U+11237 */
  0x0002,   /* U+11238 */
  0x0002,   /* U+11239 */
  0x0002,   /* U+1123A */
  0x0002,   /* U+1123B */
  0x0002,   /* U+1123C */
  0x0002,   /* U+1123D */
  0x0002,   /* U+1123E */
  0x0002,   /* U+1123F */
  0x0002,   /* U+11240 */
  0x0002,   /* U+11241 */
  0x0002,   /* U+11242 */
  0x0002,   /* U+11243 */
  0x0002,   /* U+11244 */
  0x0002,   /* U+11245 */
  0x0002,   /* U+11246 */
  0x0002,   /* U+11247 */
  0x0002,   /* U+11248 */
  0x0002,   /* U+11249 */
  0x0002,   /* U+1124A */
  0x0002,   /* U+1124B */
  0x0002,   /* U+1124C */
  0x0002,   /* U+1124D */
  0x0002,   /* U+1124E */
  0x0002,   /* U+1124F */
  0x0002,   /* U+11250 */
  0x0002,   /* U+11251 */
  0x0002,   /* U+11252 */
  0x0002,   /* U+11253 */
  0x0002,   /* U+11254 */
  0x0002,   /* U+11255 */
  0x0002,   /* U+11256 */
  0x0002,   /* U+11257 */
  0x0002,   /* U+11258 */
  0x0002,   /* U+11259 */
  0x0002,   /* U+1125A */
  0x0002,   /* U+1125B */
  0x0002,   /* U+1125C */
  0x0002,   /* U+1125D */
  0x0002,   /* U+1125E */
  0x0002,   /* U+1125F */
  0x0002,   /* U+11260 */
  0x0002,   /* U+11261 */
  0x0002,   /* U+11262 */
  0x0002,   /* U+11263 */
  0x0002,   /* U+11264 */
  0x0002,   /* U+11265 */
  0x0002,   /* U+11266 */
  0x0002,   /* U+11267 */
  0x0002,   /* U+11268 */
  0x0002,   /* U+11269 */
  0x0002,   /* U+1126A */
  0x0002,   /* U+1126B */
  0x0002,   /* U+1126C */
  0x0002,   /* U+1126D */
  0x0002,   /* U+1126E */
  0x0002,   /* U+1126F */
  0x0002,   /* U+11270 */
  0x0002,   /* U+11271 */
  0x0002,   /* U+11272 */
  0x0002,   /* U+11273 */
  0x0002,   /* U+11274 */
  0x0002,   /* U+11275 */
  0x0002,   /* U+11276 */
  0x0002,   /* U+11277 */
  0x0002,   /* U+11278 */
  0x0002,   /* U+11279 */
  0x0002,   /* U+1127A */
  0x0002,   /* U+1127B */
  0x0002,   /* U+1127C */
  0x0002,   /* U+1127D */
  0x0002,   /* U+1127E */
  0x0002,   /* U+1127F */
  0x0002,   /* U+11280 */
  0x0002,   /* U+11281 */
  0x0002,   /* U+11282 */
  0x0002,   /* U+11283 */
  0x0002,   /* U+11284 */
  0x0002,   /* U+11285 */
  0x0002,   /* U+11286 */
  0x0002,   /* U+11287 */
  0x0002,   /* U+11288 */
  0x0002,   /* U+11289 */
  0x0002,   /* U+1128A */
  0x0002,   /* U+1128B */
  0x0002,   /* U+1128C */
  0x0002,   /* U+1128D */
  0x0002,   /* U+1128E */
  0x0002,   /* U+1128F */
  0x0002,   /* U+11290 */
  0x0002,   /* U+11291 */
  0x0002,   /* U+11292 */
  0x0002,   /* U+11293 */
  0x0002,   /* U+11294 */
  0x0002,   /* U+11295 */
  0x0002,   /* U+11296 */
  0x0002,   /* U+11297 */
  0x0002,   /* U+11298 */
  0x0002,   /* U+11299 */
  0x0002,   /* U+1129A */
  0x0002,   /* U+1129B */
  0x0002,   /* U+1129C */
  0x0002,   /* U+1129D */
  0x0002,   /* U+1129E */
  0x0002,   /* U+1129F */
  0x0002,   /* U+112A0 */
  0x0002,   /* U+112A1 */
  0x0002,   /* U+112A2 */
  0x0002,   /* U+112A3 */
  0x0002,   /* U+112A4 */
  0x0002,   /* U+112A5 */
  0x0002,   /* U+112A6 */
  0x0002,   /* U+112A7 */
  0x0002,   /* U+112A8 */
  0x0002,   /* U+112A9 */
  0x0002,   /* U+112AA */
  0x0002,   /* U+112AB */
  0x0002,   /* U+112AC */
  0x0002,   /* U+112AD */
  0x0002,   /* U+112AE */
  0x0002,   /* U+112AF */
  0x0002,   /* U+112B0 */
  0x0002,   /* U+112B1 */
  0x0002,   /* U+112B2 */
  0x0002,   /* U+112B3 */
  0x0002,   /* U+112B4 */
  0x0002,   /* U+112B5 */
  0x0002,   /* U+112B6 */
  0x0002,   /* U+112B7 */
  0x0002,   /* U+112B8 */
  0x0002,   /* U+112B9 */
  0x0002,   /* U+112BA */
  0x0002,   /* U+112BB */
  0x0002,   /* U+112BC */
  0x0002,   /* U+112BD */
  0x0002,   /* U+112BE */
  0x0002,   /* U+112BF */
  0x0002,   /* U+112C0 */
  0x0002,   /* U+112C1 */
  0x0002,   /* U+112C2 */
  0x0002,   /* U+112C3 */
  0x0002,   /* U+112C4 */
  0x0002,   /* U+112C5 */
  0x0002,   /* U+112C6 */
  0x0002,   /* U+112C7 */
  0x0002,   /* U+112C8 */
  0x0002,   /* U+112C9 */
  0x0002,   /* U+112CA */
  0x0002,   /* U+112CB */
  0x0002,   /* U+112CC */
  0x0002,   /* U+112CD */
  0x0002,   /* U+112CE */
  0x0002,   /* U+112CF */
  0x0002,   /* U+112D0 */
  0x0002,   /* U+112D1 */
  0x0002,   /* U+112D2 */
  0x0002,   /* U+112D3 */
  0x0002,   /* U+112D4 */
  0x0002,   /* U+112D5 */
  0x0002,   /* U+112D6 */
  0x0002,   /* U+112D7 */
  0x0002,   /* U+112D8 */
  0x0002,   /* U+112D9 */
  0x0002,   /* U+112DA */
  0x0002,   /* U+112DB */
  0x0002,   /* U+112DC */
  0x0002,   /* U+112DD */
  0x0002,   /* U+112DE */
  0x0002,   /* U+112DF */
  0x0002,   /* U+112E0 */
  0x0002,   /* U+112E1 */
  0x0002,   /* U+112E2 */
  0x0002,   /* U+112E3 */
  0x0002,   /* U+112E4 */
  0x0002,   /* U+112E5 */
  0x0002,   /* U+112E6 */
  0x0002,   /* U+112E7 */
  0x0002,   /* U+112E8 */
  0x0002,   /* U+112E9 */
  0x0002,   /* U+112EA */
  0x0002,   /* U+112EB */
  0x0002,   /* U+112EC */
  0x0002,   /* U+112ED */
  0x0002,   /* U+112EE */
  0x0002,   /* U+112EF */
  0x0002,   /* U+112F0 */
  0x0002,   /* U+112F1 */
  0x0002,   /* U+112F2 */
  0x0002,   /* U+112F3 */
  0x0002,   /* U+112F4 */
  0x0002,   /* U+112F5 */
  0x0002,   /* U+112F6 */
  0x0002,   /* U+112F7 */
  0x0002,   /* U+112F8 */
  0x0002,   /* U+112F9 */
  0x0002,   /* U+112FA */
  0x0002,   /* U+112FB */
  0x0002,   /* U+112FC */
  0x0002,   /* U+112FD */
  0x0002,   /* U+112FE */
  0x0002,   /* U+112FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11200 */
  0x0000,   /* U+11201 */
  0x0000,   /* U+11202 */
  0x0000,   /* U+11203 */
  0x0000,   /* U+11204 */
  0x0000,   /* U+11205 */
  0x0000,   /* U+11206 */
  0x0000,   /* U+11207 */
  0x0000,   /* U+11208 */
  0x0000,   /* U+11209 */
  0x0000,   /* U+1120A */
  0x0000,   /* U+1120B */
  0x0000,   /* U+1120C */
  0x0000,   /* U+1120D */
  0x0000,   /* U+1120E */
  0x0000,   /* U+1120F */
  0x0000,   /* U+11210 */
  0x0000,   /* U+11211 */
  0x9212,   /* U+11212 */
  0x0000,   /* U+11213 */
  0x0000,   /* U+11214 */
  0x0000,   /* U+11215 */
  0x0000,   /* U+11216 */
  0x0000,   /* U+11217 */
  0x0000,   /* U+11218 */
  0x0000,   /* U+11219 */
  0x0000,   /* U+1121A */
  0x0000,   /* U+1121B */
  0x0000,   /* U+1121C */
  0x0000,   /* U+1121D */
  0x0000,   /* U+1121E */
  0x0000,   /* U+1121F */
  0x0000,   /* U+11220 */
  0x0000,   /* U+11221 */
  0x0000,   /* U+11222 */
  0x0000,   /* U+11223 */
  0x0000,   /* U+11224 */
  0x0000,   /* U+11225 */
  0x0000,   /* U+11226 */
  0x0000,   /* U+11227 */
  0x0000,   /* U+11228 */
  0x0000,   /* U+11229 */
  0x0000,   /* U+1122A */
  0x0000,   /* U+1122B */
  0x0000,   /* U+1122C */
  0x0000,   /* U+1122D */
  0x0000,   /* U+1122E */
  0x0000,   /* U+1122F */
  0x0000,   /* U+11230 */
  0x0000,   /* U+11231 */
  0x0000,   /* U+11232 */
  0x0000,   /* U+11233 */
  0x0000,   /* U+11234 */
  0x0000,   /* U+11235 */
  0x0000,   /* U+11236 */
  0x0000,   /* U+11237 */
  0x0000,   /* U+11238 */
  0x0000,   /* U+11239 */
  0x0000,   /* U+1123A */
  0x0000,   /* U+1123B */
  0x0000,   /* U+1123C */
  0x0000,   /* U+1123D */
  0x0000,   /* U+1123E */
  0x923F,   /* U+1123F */
  0x9240,   /* U+11240 */
  0x9241,   /* U+11241 */
  0x9242,   /* U+11242 */
  0x9243,   /* U+11243 */
  0x9244,   /* U+11244 */
  0x9245,   /* U+11245 */
  0x9246,   /* U+11246 */
  0x9247,   /* U+11247 */
  0x9248,   /* U+11248 */
  0x9249,   /* U+11249 */
  0x924A,   /* U+1124A */
  0x924B,   /* U+1124B */
  0x924C,   /* U+1124C */
  0x924D,   /* U+1124D */
  0x924E,   /* U+1124E */
  0x924F,   /* U+1124F */
  0x9250,   /* U+11250 */
  0x9251,   /* U+11251 */
  0x9252,   /* U+11252 */
  0x9253,   /* U+11253 */
  0x9254,   /* U+11254 */
  0x9255,   /* U+11255 */
  0x9256,   /* U+11256 */
  0x9257,   /* U+11257 */
  0x9258,   /* U+11258 */
  0x9259,   /* U+11259 */
  0x925A,   /* U+1125A */
  0x925B,   /* U+1125B */
  0x925C,   /* U+1125C */
  0x925D,   /* U+1125D */
  0x925E,   /* U+1125E */
  0x925F,   /* U+1125F */
  0x9260,   /* U+11260 */
  0x9261,   /* U+11261 */
  0x9262,   /* U+11262 */
  0x9263,   /* U+11263 */
  0x9264,   /* U+11264 */
  0x9265,   /* U+11265 */
  0x9266,   /* U+11266 */
  0x9267,   /* U+11267 */
  0x9268,   /* U+11268 */
  0x9269,   /* U+11269 */
  0x926A,   /* U+1126A */
  0x926B,   /* U+1126B */
  0x926C,   /* U+1126C */
  0x926D,   /* U+1126D */
  0x926E,   /* U+1126E */
  0x926F,   /* U+1126F */
  0x9270,   /* U+11270 */
  0x9271,   /* U+11271 */
  0x9272,   /* U+11272 */
  0x9273,   /* U+11273 */
  0x9274,   /* U+11274 */
  0x9275,   /* U+11275 */
  0x9276,   /* U+11276 */
  0x9277,   /* U+11277 */
  0x9278,   /* U+11278 */
  0x9279,   /* U+11279 */
  0x927A,   /* U+1127A */
  0x927B,   /* U+1127B */
  0x927C,   /* U+1127C */
  0x927D,   /* U+1127D */
  0x927E,   /* U+1127E */
  0x927F,   /* U+1127F */
  0x0000,   /* U+11280 */
  0x0000,   /* U+11281 */
  0x0000,   /* U+11282 */
  0x0000,   /* U+11283 */
  0x0000,   /* U+11284 */
  0x0000,   /* U+11285 */
  0x0000,   /* U+11286 */
  0x9287,   /* U+11287 */
  0x0000,   /* U+11288 */
  0x9289,   /* U+11289 */
  0x0000,   /* U+1128A */
  0x0000,   /* U+1128B */
  0x0000,   /* U+1128C */
  0x0000,   /* U+1128D */
  0x928E,   /* U+1128E */
  0x0000,   /* U+1128F */
  0x0000,   /* U+11290 */
  0x0000,   /* U+11291 */
  0x0000,   /* U+11292 */
  0x0000,   /* U+11293 */
  0x0000,   /* U+11294 */
  0x0000,   /* U+11295 */
  0x0000,   /* U+11296 */
  0x0000,   /* U+11297 */
  0x0000,   /* U+11298 */
  0x0000,   /* U+11299 */
  0x0000,   /* U+1129A */
  0x0000,   /* U+1129B */
  0x0000,   /* U+1129C */
  0x0000,   /* U+1129D */
  0x929E,   /* U+1129E */
  0x0000,   /* U+1129F */
  0x0000,   /* U+112A0 */
  0x0000,   /* U+112A1 */
  0x0000,   /* U+112A2 */
  0x0000,   /* U+112A3 */
  0x0000,   /* U+112A4 */
  0x0000,   /* U+112A5 */
  0x0000,   /* U+112A6 */
  0x0000,   /* U+112A7 */
  0x0000,   /* U+112A8 */
  0x0000,   /* U+112A9 */
  0x92AA,   /* U+112AA */
  0x92AB,   /* U+112AB */
  0x92AC,   /* U+112AC */
  0x92AD,   /* U+112AD */
  0x92AE,   /* U+112AE */
  0x92AF,   /* U+112AF */
  0x0000,   /* U+112B0 */
  0x0000,   /* U+112B1 */
  0x0000,   /* U+112B2 */
  0x0000,   /* U+112B3 */
  0x0000,   /* U+112B4 */
  0x0000,   /* U+112B5 */
  0x0000,   /* U+112B6 */
  0x0000,   /* U+112B7 */
  0x0000,   /* U+112B8 */
  0x0000,   /* U+112B9 */
  0x0000,   /* U+112BA */
  0x0000,   /* U+112BB */
  0x0000,   /* U+112BC */
  0x0000,   /* U+112BD */
  0x0000,   /* U+112BE */
  0x0000,   /* U+112BF */
  0x0000,   /* U+112C0 */
  0x0000,   /* U+112C1 */
  0x0000,   /* U+112C2 */
  0x0000,   /* U+112C3 */
  0x0000,   /* U+112C4 */
  0x0000,   /* U+112C5 */
  0x0000,   /* U+112C6 */
  0x0000,   /* U+112C7 */
  0x0000,   /* U+112C8 */
  0x0000,   /* U+112C9 */
  0x0000,   /* U+112CA */
  0x0000,   /* U+112CB */
  0x0000,   /* U+112CC */
  0x0000,   /* U+112CD */
  0x0000,   /* U+112CE */
  0x0000,   /* U+112CF */
  0x0000,   /* U+112D0 */
  0x0000,   /* U+112D1 */
  0x0000,   /* U+112D2 */
  0x0000,   /* U+112D3 */
  0x0000,   /* U+112D4 */
  0x0000,   /* U+112D5 */
  0x0000,   /* U+112D6 */
  0x0000,   /* U+112D7 */
  0x0000,   /* U+112D8 */
  0x0000,   /* U+112D9 */
  0x0000,   /* U+112DA */
  0x0000,   /* U+112DB */
  0x0000,   /* U+112DC */
  0x0000,   /* U+112DD */
  0x0000,   /* U+112DE */
  0x0000,   /* U+112DF */
  0x0000,   /* U+112E0 */
  0x0000,   /* U+112E1 */
  0x0000,   /* U+112E2 */
  0x0000,   /* U+112E3 */
  0x0000,   /* U+112E4 */
  0x0000,   /* U+112E5 */
  0x0000,   /* U+112E6 */
  0x0000,   /* U+112E7 */
  0x0000,   /* U+112E8 */
  0x0000,   /* U+112E9 */
  0x0000,   /* U+112EA */
  0x92EB,   /* U+112EB */
  0x92EC,   /* U+112EC */
  0x92ED,   /* U+112ED */
  0x92EE,   /* U+112EE */
  0x92EF,   /* U+112EF */
  0x0000,   /* U+112F0 */
  0x0000,   /* U+112F1 */
  0x0000,   /* U+112F2 */
  0x0000,   /* U+112F3 */
  0x0000,   /* U+112F4 */
  0x0000,   /* U+112F5 */
  0x0000,   /* U+112F6 */
  0x0000,   /* U+112F7 */
  0x0000,   /* U+112F8 */
  0x0000,   /* U+112F9 */
  0x92FA,   /* U+112FA */
  0x92FB,   /* U+112FB */
  0x92FC,   /* U+112FC */
  0x92FD,   /* U+112FD */
  0x92FE,   /* U+112FE */
  0x92FF,   /* U+112FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11200 */
  0x0000,   /* U+11201 */
  0x0000,   /* U+11202 */
  0x0000,   /* U+11203 */
  0x0000,   /* U+11204 */
  0x0000,   /* U+11205 */
  0x0000,   /* U+11206 */
  0x0000,   /* U+11207 */
  0x0000,   /* U+11208 */
  0x0000,   /* U+11209 */
  0x0000,   /* U+1120A */
  0x0000,   /* U+1120B */
  0x0000,   /* U+1120C */
  0x0000,   /* U+1120D */
  0x0000,   /* U+1120E */
  0x0000,   /* U+1120F */
  0x0000,   /* U+11210 */
  0x0000,   /* U+11211 */
  0x0000,   /* U+11212 */
  0x0000,   /* U+11213 */
  0x0000,   /* U+11214 */
  0x0000,   /* U+11215 */
  0x0000,   /* U+11216 */
  0x0000,   /* U+11217 */
  0x0000,   /* U+11218 */
  0x0000,   /* U+11219 */
  0x0000,   /* U+1121A */
  0x0000,   /* U+1121B */
  0x0000,   /* U+1121C */
  0x0000,   /* U+1121D */
  0x0000,   /* U+1121E */
  0x0000,   /* U+1121F */
  0x0000,   /* U+11220 */
  0x0000,   /* U+11221 */
  0x0000,   /* U+11222 */
  0x0000,   /* U+11223 */
  0x0000,   /* U+11224 */
  0x0000,   /* U+11225 */
  0x0000,   /* U+11226 */
  0x0000,   /* U+11227 */
  0x0000,   /* U+11228 */
  0x0000,   /* U+11229 */
  0x0000,   /* U+1122A */
  0x0000,   /* U+1122B */
  0x0000,   /* U+1122C */
  0x0000,   /* U+1122D */
  0x0000,   /* U+1122E */
  0x0000,   /* U+1122F */
  0x0000,   /* U+11230 */
  0x0000,   /* U+11231 */
  0x0000,   /* U+11232 */
  0x0000,   /* U+11233 */
  0x0000,   /* U+11234 */
  0x0000,   /* U+11235 */
  0x0000,   /* U+11236 */
  0x0000,   /* U+11237 */
  0x0000,   /* U+11238 */
  0x0000,   /* U+11239 */
  0x0000,   /* U+1123A */
  0x0000,   /* U+1123B */
  0x0000,   /* U+1123C */
  0x0000,   /* U+1123D */
  0x0000,   /* U+1123E */
  0x0000,   /* U+1123F */
  0x0000,   /* U+11240 */
  0x0000,   /* U+11241 */
  0x0000,   /* U+11242 */
  0x0000,   /* U+11243 */
  0x0000,   /* U+11244 */
  0x0000,   /* U+11245 */
  0x0000,   /* U+11246 */
  0x0000,   /* U+11247 */
  0x0000,   /* U+11248 */
  0x0000,   /* U+11249 */
  0x0000,   /* U+1124A */
  0x0000,   /* U+1124B */
  0x0000,   /* U+1124C */
  0x0000,   /* U+1124D */
  0x0000,   /* U+1124E */
  0x0000,   /* U+1124F */
  0x0000,   /* U+11250 */
  0x0000,   /* U+11251 */
  0x0000,   /* U+11252 */
  0x0000,   /* U+11253 */
  0x0000,   /* U+11254 */
  0x0000,   /* U+11255 */
  0x0000,   /* U+11256 */
  0x0000,   /* U+11257 */
  0x0000,   /* U+11258 */
  0x0000,   /* U+11259 */
  0x0000,   /* U+1125A */
  0x0000,   /* U+1125B */
  0x0000,   /* U+1125C */
  0x0000,   /* U+1125D */
  0x0000,   /* U+1125E */
  0x0000,   /* U+1125F */
  0x0000,   /* U+11260 */
  0x0000,   /* U+11261 */
  0x0000,   /* U+11262 */
  0x0000,   /* U+11263 */
  0x0000,   /* U+11264 */
  0x0000,   /* U+11265 */
  0x0000,   /* U+11266 */
  0x0000,   /* U+11267 */
  0x0000,   /* U+11268 */
  0x0000,   /* U+11269 */
  0x0000,   /* U+1126A */
  0x0000,   /* U+1126B */
  0x0000,   /* U+1126C */
  0x0000,   /* U+1126D */
  0x0000,   /* U+1126E */
  0x0000,   /* U+1126F */
  0x0000,   /* U+11270 */
  0x0000,   /* U+11271 */
  0x0000,   /* U+11272 */
  0x0000,   /* U+11273 */
  0x0000,   /* U+11274 */
  0x0000,   /* U+11275 */
  0x0000,   /* U+11276 */
  0x0000,   /* U+11277 */
  0x0000,   /* U+11278 */
  0x0000,   /* U+11279 */
  0x0000,   /* U+1127A */
  0x0000,   /* U+1127B */
  0x0000,   /* U+1127C */
  0x0000,   /* U+1127D */
  0x0000,   /* U+1127E */
  0x0000,   /* U+1127F */
  0x0000,   /* U+11280 */
  0x0000,   /* U+11281 */
  0x0000,   /* U+11282 */
  0x0000,   /* U+11283 */
  0x0000,   /* U+11284 */
  0x0000,   /* U+11285 */
  0x0000,   /* U+11286 */
  0x0000,   /* U+11287 */
  0x0000,   /* U+11288 */
  0x0000,   /* U+11289 */
  0x0000,   /* U+1128A */
  0x0000,   /* U+1128B */
  0x0000,   /* U+1128C */
  0x0000,   /* U+1128D */
  0x0000,   /* U+1128E */
  0x0000,   /* U+1128F */
  0x0000,   /* U+11290 */
  0x0000,   /* U+11291 */
  0x0000,   /* U+11292 */
  0x0000,   /* U+11293 */
  0x0000,   /* U+11294 */
  0x0000,   /* U+11295 */
  0x0000,   /* U+11296 */
  0x0000,   /* U+11297 */
  0x0000,   /* U+11298 */
  0x0000,   /* U+11299 */
  0x0000,   /* U+1129A */
  0x0000,   /* U+1129B */
  0x0000,   /* U+1129C */
  0x0000,   /* U+1129D */
  0x0000,   /* U+1129E */
  0x0000,   /* U+1129F */
  0x0000,   /* U+112A0 */
  0x0000,   /* U+112A1 */
  0x0000,   /* U+112A2 */
  0x0000,   /* U+112A3 */
  0x0000,   /* U+112A4 */
  0x0000,   /* U+112A5 */
  0x0000,   /* U+112A6 */
  0x0000,   /* U+112A7 */
  0x0000,   /* U+112A8 */
  0x0000,   /* U+112A9 */
  0x0000,   /* U+112AA */
  0x0000,   /* U+112AB */
  0x0000,   /* U+112AC */
  0x0000,   /* U+112AD */
  0x0000,   /* U+112AE */
  0x0000,   /* U+112AF */
  0x0000,   /* U+112B0 */
  0x0000,   /* U+112B1 */
  0x0000,   /* U+112B2 */
  0x0000,   /* U+112B3 */
  0x0000,   /* U+112B4 */
  0x0000,   /* U+112B5 */
  0x0000,   /* U+112B6 */
  0x0000,   /* U+112B7 */
  0x0000,   /* U+112B8 */
  0x0000,   /* U+112B9 */
  0x0000,   /* U+112BA */
  0x0000,   /* U+112BB */
  0x0000,   /* U+112BC */
  0x0000,   /* U+112BD */
  0x0000,   /* U+112BE */
  0x0000,   /* U+112BF */
  0x0000,   /* U+112C0 */
  0x0000,   /* U+112C1 */
  0x0000,   /* U+112C2 */
  0x0000,   /* U+112C3 */
  0x0000,   /* U+112C4 */
  0x0000,   /* U+112C5 */
  0x0000,   /* U+112C6 */
  0x0000,   /* U+112C7 */
  0x0000,   /* U+112C8 */
  0x0000,   /* U+112C9 */
  0x0000,   /* U+112CA */
  0x0000,   /* U+112CB */
  0x0000,   /* U+112CC */
  0x0000,   /* U+112CD */
  0x0000,   /* U+112CE */
  0x0000,   /* U+112CF */
  0x0000,   /* U+112D0 */
  0x0000,   /* U+112D1 */
  0x0000,   /* U+112D2 */
  0x0000,   /* U+112D3 */
  0x0000,   /* U+112D4 */
  0x0000,   /* U+112D5 */
  0x0000,   /* U+112D6 */
  0x0000,   /* U+112D7 */
  0x0000,   /* U+112D8 */
  0x0000,   /* U+112D9 */
  0x0000,   /* U+112DA */
  0x0000,   /* U+112DB */
  0x0000,   /* U+112DC */
  0x0000,   /* U+112DD */
  0x0000,   /* U+112DE */
  0x0000,   /* U+112DF */
  0x0000,   /* U+112E0 */
  0x0000,   /* U+112E1 */
  0x0000,   /* U+112E2 */
  0x0000,   /* U+112E3 */
  0x0000,   /* U+112E4 */
  0x0000,   /* U+112E5 */
  0x0000,   /* U+112E6 */
  0x0000,   /* U+112E7 */
  0x0000,   /* U+112E8 */
  0x0000,   /* U+112E9 */
  0x0000,   /* U+112EA */
  0x0000,   /* U+112EB */
  0x0000,   /* U+112EC */
  0x0000,   /* U+112ED */
  0x0000,   /* U+112EE */
  0x0000,   /* U+112EF */
  0x0000,   /* U+112F0 */
  0x0000,   /* U+112F1 */
  0x0000,   /* U+112F2 */
  0x0000,   /* U+112F3 */
  0x0000,   /* U+112F4 */
  0x0000,   /* U+112F5 */
  0x0000,   /* U+112F6 */
  0x0000,   /* U+112F7 */
  0x0000,   /* U+112F8 */
  0x0000,   /* U+112F9 */
  0x0000,   /* U+112FA */
  0x0000,   /* U+112FB */
  0x0000,   /* U+112FC */
  0x0000,   /* U+112FD */
  0x0000,   /* U+112FE */
  0x0000,   /* U+112FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11200 */
  0x0000,   /* U+11201 */
  0x0000,   /* U+11202 */
  0x0000,   /* U+11203 */
  0x0000,   /* U+11204 */
  0x0000,   /* U+11205 */
  0x0000,   /* U+11206 */
  0x0000,   /* U+11207 */
  0x0000,   /* U+11208 */
  0x0000,   /* U+11209 */
  0x0000,   /* U+1120A */
  0x0000,   /* U+1120B */
  0x0000,   /* U+1120C */
  0x0000,   /* U+1120D */
  0x0000,   /* U+1120E */
  0x0000,   /* U+1120F */
  0x0000,   /* U+11210 */
  0x0000,   /* U+11211 */
  0x0000,   /* U+11212 */
  0x0000,   /* U+11213 */
  0x0000,   /* U+11214 */
  0x0000,   /* U+11215 */
  0x0000,   /* U+11216 */
  0x0000,   /* U+11217 */
  0x0000,   /* U+11218 */
  0x0000,   /* U+11219 */
  0x0000,   /* U+1121A */
  0x0000,   /* U+1121B */
  0x0000,   /* U+1121C */
  0x0000,   /* U+1121D */
  0x0000,   /* U+1121E */
  0x0000,   /* U+1121F */
  0x0000,   /* U+11220 */
  0x0000,   /* U+11221 */
  0x0000,   /* U+11222 */
  0x0000,   /* U+11223 */
  0x0000,   /* U+11224 */
  0x0000,   /* U+11225 */
  0x0000,   /* U+11226 */
  0x0000,   /* U+11227 */
  0x0000,   /* U+11228 */
  0x0000,   /* U+11229 */
  0x0000,   /* U+1122A */
  0x0000,   /* U+1122B */
  0x0000,   /* U+1122C */
  0x0000,   /* U+1122D */
  0x0000,   /* U+1122E */
  0x0000,   /* U+1122F */
  0x0000,   /* U+11230 */
  0x0000,   /* U+11231 */
  0x0000,   /* U+11232 */
  0x0000,   /* U+11233 */
  0x0000,   /* U+11234 */
  0x0000,   /* U+11235 */
  0x0000,   /* U+11236 */
  0x0000,   /* U+11237 */
  0x0000,   /* U+11238 */
  0x0000,   /* U+11239 */
  0x0000,   /* U+1123A */
  0x0000,   /* U+1123B */
  0x0000,   /* U+1123C */
  0x0000,   /* U+1123D */
  0x0000,   /* U+1123E */
  0x0000,   /* U+1123F */
  0x0000,   /* U+11240 */
  0x0000,   /* U+11241 */
  0x0000,   /* U+11242 */
  0x0000,   /* U+11243 */
  0x0000,   /* U+11244 */
  0x0000,   /* U+11245 */
  0x0000,   /* U+11246 */
  0x0000,   /* U+11247 */
  0x0000,   /* U+11248 */
  0x0000,   /* U+11249 */
  0x0000,   /* U+1124A */
  0x0000,   /* U+1124B */
  0x0000,   /* U+1124C */
  0x0000,   /* U+1124D */
  0x0000,   /* U+1124E */
  0x0000,   /* U+1124F */
  0x0000,   /* U+11250 */
  0x0000,   /* U+11251 */
  0x0000,   /* U+11252 */
  0x0000,   /* U+11253 */
  0x0000,   /* U+11254 */
  0x0000,   /* U+11255 */
  0x0000,   /* U+11256 */
  0x0000,   /* U+11257 */
  0x0000,   /* U+11258 */
  0x0000,   /* U+11259 */
  0x0000,   /* U+1125A */
  0x0000,   /* U+1125B */
  0x0000,   /* U+1125C */
  0x0000,   /* U+1125D */
  0x0000,   /* U+1125E */
  0x0000,   /* U+1125F */
  0x0000,   /* U+11260 */
  0x0000,   /* U+11261 */
  0x0000,   /* U+11262 */
  0x0000,   /* U+11263 */
  0x0000,   /* U+11264 */
  0x0000,   /* U+11265 */
  0x0000,   /* U+11266 */
  0x0000,   /* U+11267 */
  0x0000,   /* U+11268 */
  0x0000,   /* U+11269 */
  0x0000,   /* U+1126A */
  0x0000,   /* U+1126B */
  0x0000,   /* U+1126C */
  0x0000,   /* U+1126D */
  0x0000,   /* U+1126E */
  0x0000,   /* U+1126F */
  0x0000,   /* U+11270 */
  0x0000,   /* U+11271 */
  0x0000,   /* U+11272 */
  0x0000,   /* U+11273 */
  0x0000,   /* U+11274 */
  0x0000,   /* U+11275 */
  0x0000,   /* U+11276 */
  0x0000,   /* U+11277 */
  0x0000,   /* U+11278 */
  0x0000,   /* U+11279 */
  0x0000,   /* U+1127A */
  0x0000,   /* U+1127B */
  0x0000,   /* U+1127C */
  0x0000,   /* U+1127D */
  0x0000,   /* U+1127E */
  0x0000,   /* U+1127F */
  0x0000,   /* U+11280 */
  0x0000,   /* U+11281 */
  0x0000,   /* U+11282 */
  0x0000,   /* U+11283 */
  0x0000,   /* U+11284 */
  0x0000,   /* U+11285 */
  0x0000,   /* U+11286 */
  0x0000,   /* U+11287 */
  0x0000,   /* U+11288 */
  0x0000,   /* U+11289 */
  0x0000,   /* U+1128A */
  0x0000,   /* U+1128B */
  0x0000,   /* U+1128C */
  0x0000,   /* U+1128D */
  0x0000,   /* U+1128E */
  0x0000,   /* U+1128F */
  0x0000,   /* U+11290 */
  0x0000,   /* U+11291 */
  0x0000,   /* U+11292 */
  0x0000,   /* U+11293 */
  0x0000,   /* U+11294 */
  0x0000,   /* U+11295 */
  0x0000,   /* U+11296 */
  0x0000,   /* U+11297 */
  0x0000,   /* U+11298 */
  0x0000,   /* U+11299 */
  0x0000,   /* U+1129A */
  0x0000,   /* U+1129B */
  0x0000,   /* U+1129C */
  0x0000,   /* U+1129D */
  0x0000,   /* U+1129E */
  0x0000,   /* U+1129F */
  0x0000,   /* U+112A0 */
  0x0000,   /* U+112A1 */
  0x0000,   /* U+112A2 */
  0x0000,   /* U+112A3 */
  0x0000,   /* U+112A4 */
  0x0000,   /* U+112A5 */
  0x0000,   /* U+112A6 */
  0x0000,   /* U+112A7 */
  0x0000,   /* U+112A8 */
  0x0000,   /* U+112A9 */
  0x0000,   /* U+112AA */
  0x0000,   /* U+112AB */
  0x0000,   /* U+112AC */
  0x0000,   /* U+112AD */
  0x0000,   /* U+112AE */
  0x0000,   /* U+112AF */
  0x0000,   /* U+112B0 */
  0x0000,   /* U+112B1 */
  0x0000,   /* U+112B2 */
  0x0000,   /* U+112B3 */
  0x0000,   /* U+112B4 */
  0x0000,   /* U+112B5 */
  0x0000,   /* U+112B6 */
  0x0000,   /* U+112B7 */
  0x0000,   /* U+112B8 */
  0x0000,   /* U+112B9 */
  0x0000,   /* U+112BA */
  0x0000,   /* U+112BB */
  0x0000,   /* U+112BC */
  0x0000,   /* U+112BD */
  0x0000,   /* U+112BE */
  0x0000,   /* U+112BF */
  0x0000,   /* U+112C0 */
  0x0000,   /* U+112C1 */
  0x0000,   /* U+112C2 */
  0x0000,   /* U+112C3 */
  0x0000,   /* U+112C4 */
  0x0000,   /* U+112C5 */
  0x0000,   /* U+112C6 */
  0x0000,   /* U+112C7 */
  0x0000,   /* U+112C8 */
  0x0000,   /* U+112C9 */
  0x0000,   /* U+112CA */
  0x0000,   /* U+112CB */
  0x0000,   /* U+112CC */
  0x0000,   /* U+112CD */
  0x0000,   /* U+112CE */
  0x0000,   /* U+112CF */
  0x0000,   /* U+112D0 */
  0x0000,   /* U+112D1 */
  0x0000,   /* U+112D2 */
  0x0000,   /* U+112D3 */
  0x0000,   /* U+112D4 */
  0x0000,   /* U+112D5 */
  0x0000,   /* U+112D6 */
  0x0000,   /* U+112D7 */
  0x0000,   /* U+112D8 */
  0x0000,   /* U+112D9 */
  0x0000,   /* U+112DA */
  0x0000,   /* U+112DB */
  0x0000,   /* U+112DC */
  0x0000,   /* U+112DD */
  0x0000,   /* U+112DE */
  0x0000,   /* U+112DF */
  0x0000,   /* U+112E0 */
  0x0000,   /* U+112E1 */
  0x0000,   /* U+112E2 */
  0x0000,   /* U+112E3 */
  0x0000,   /* U+112E4 */
  0x0000,   /* U+112E5 */
  0x0000,   /* U+112E6 */
  0x0000,   /* U+112E7 */
  0x0000,   /* U+112E8 */
  0x0000,   /* U+112E9 */
  0x0000,   /* U+112EA */
  0x0000,   /* U+112EB */
  0x0000,   /* U+112EC */
  0x0000,   /* U+112ED */
  0x0000,   /* U+112EE */
  0x0000,   /* U+112EF */
  0x0000,   /* U+112F0 */
  0x0000,   /* U+112F1 */
  0x0000,   /* U+112F2 */
  0x0000,   /* U+112F3 */
  0x0000,   /* U+112F4 */
  0x0000,   /* U+112F5 */
  0x0000,   /* U+112F6 */
  0x0000,   /* U+112F7 */
  0x0000,   /* U+112F8 */
  0x0000,   /* U+112F9 */
  0x0000,   /* U+112FA */
  0x0000,   /* U+112FB */
  0x0000,   /* U+112FC */
  0x0000,   /* U+112FD */
  0x0000,   /* U+112FE */
  0x0000,   /* U+112FF */
};

uint16 uca900_p113[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+11300 */
  0x0000,   /* U+11301 */
  0x0000,   /* U+11302 */
  0x0000,   /* U+11303 */
  0xFBC2,   /* U+11304 */
  0x2AF5,   /* U+11305 */
  0x2AF6,   /* U+11306 */
  0x2AF7,   /* U+11307 */
  0x2AF8,   /* U+11308 */
  0x2AF9,   /* U+11309 */
  0x2AFA,   /* U+1130A */
  0x2AFB,   /* U+1130B */
  0x2AFD,   /* U+1130C */
  0xFBC2,   /* U+1130D */
  0xFBC2,   /* U+1130E */
  0x2AFF,   /* U+1130F */
  0x2B00,   /* U+11310 */
  0xFBC2,   /* U+11311 */
  0xFBC2,   /* U+11312 */
  0x2B01,   /* U+11313 */
  0x2B02,   /* U+11314 */
  0x2B03,   /* U+11315 */
  0x2B04,   /* U+11316 */
  0x2B05,   /* U+11317 */
  0x2B06,   /* U+11318 */
  0x2B07,   /* U+11319 */
  0x2B08,   /* U+1131A */
  0x2B09,   /* U+1131B */
  0x2B0A,   /* U+1131C */
  0x2B0B,   /* U+1131D */
  0x2B0C,   /* U+1131E */
  0x2B0D,   /* U+1131F */
  0x2B0E,   /* U+11320 */
  0x2B0F,   /* U+11321 */
  0x2B10,   /* U+11322 */
  0x2B11,   /* U+11323 */
  0x2B12,   /* U+11324 */
  0x2B13,   /* U+11325 */
  0x2B14,   /* U+11326 */
  0x2B15,   /* U+11327 */
  0x2B16,   /* U+11328 */
  0xFBC2,   /* U+11329 */
  0x2B17,   /* U+1132A */
  0x2B18,   /* U+1132B */
  0x2B19,   /* U+1132C */
  0x2B1A,   /* U+1132D */
  0x2B1B,   /* U+1132E */
  0x2B1C,   /* U+1132F */
  0x2B1D,   /* U+11330 */
  0xFBC2,   /* U+11331 */
  0x2B1E,   /* U+11332 */
  0x2B1F,   /* U+11333 */
  0xFBC2,   /* U+11334 */
  0x2B20,   /* U+11335 */
  0x2B21,   /* U+11336 */
  0x2B22,   /* U+11337 */
  0x2B23,   /* U+11338 */
  0x2B24,   /* U+11339 */
  0xFBC2,   /* U+1133A */
  0xFBC2,   /* U+1133B */
  0x0000,   /* U+1133C */
  0x2B25,   /* U+1133D */
  0x2B28,   /* U+1133E */
  0x2B29,   /* U+1133F */
  0x2B2A,   /* U+11340 */
  0x2B2B,   /* U+11341 */
  0x2B2C,   /* U+11342 */
  0x2B2D,   /* U+11343 */
  0x2B2E,   /* U+11344 */
  0xFBC2,   /* U+11345 */
  0xFBC2,   /* U+11346 */
  0x2B31,   /* U+11347 */
  0x2B32,   /* U+11348 */
  0xFBC2,   /* U+11349 */
  0xFBC2,   /* U+1134A */
  0x2B33,   /* U+1134B */
  0x2B34,   /* U+1134C */
  0x2B35,   /* U+1134D */
  0xFBC2,   /* U+1134E */
  0xFBC2,   /* U+1134F */
  0x2AF4,   /* U+11350 */
  0xFBC2,   /* U+11351 */
  0xFBC2,   /* U+11352 */
  0xFBC2,   /* U+11353 */
  0xFBC2,   /* U+11354 */
  0xFBC2,   /* U+11355 */
  0xFBC2,   /* U+11356 */
  0x2B36,   /* U+11357 */
  0xFBC2,   /* U+11358 */
  0xFBC2,   /* U+11359 */
  0xFBC2,   /* U+1135A */
  0xFBC2,   /* U+1135B */
  0xFBC2,   /* U+1135C */
  0x2B37,   /* U+1135D */
  0x2B26,   /* U+1135E */
  0x2B27,   /* U+1135F */
  0x2AFC,   /* U+11360 */
  0x2AFE,   /* U+11361 */
  0x2B2F,   /* U+11362 */
  0x2B30,   /* U+11363 */
  0xFBC2,   /* U+11364 */
  0xFBC2,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0xFBC2,   /* U+1136D */
  0xFBC2,   /* U+1136E */
  0xFBC2,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0xFBC2,   /* U+11375 */
  0xFBC2,   /* U+11376 */
  0xFBC2,   /* U+11377 */
  0xFBC2,   /* U+11378 */
  0xFBC2,   /* U+11379 */
  0xFBC2,   /* U+1137A */
  0xFBC2,   /* U+1137B */
  0xFBC2,   /* U+1137C */
  0xFBC2,   /* U+1137D */
  0xFBC2,   /* U+1137E */
  0xFBC2,   /* U+1137F */
  0xFBC2,   /* U+11380 */
  0xFBC2,   /* U+11381 */
  0xFBC2,   /* U+11382 */
  0xFBC2,   /* U+11383 */
  0xFBC2,   /* U+11384 */
  0xFBC2,   /* U+11385 */
  0xFBC2,   /* U+11386 */
  0xFBC2,   /* U+11387 */
  0xFBC2,   /* U+11388 */
  0xFBC2,   /* U+11389 */
  0xFBC2,   /* U+1138A */
  0xFBC2,   /* U+1138B */
  0xFBC2,   /* U+1138C */
  0xFBC2,   /* U+1138D */
  0xFBC2,   /* U+1138E */
  0xFBC2,   /* U+1138F */
  0xFBC2,   /* U+11390 */
  0xFBC2,   /* U+11391 */
  0xFBC2,   /* U+11392 */
  0xFBC2,   /* U+11393 */
  0xFBC2,   /* U+11394 */
  0xFBC2,   /* U+11395 */
  0xFBC2,   /* U+11396 */
  0xFBC2,   /* U+11397 */
  0xFBC2,   /* U+11398 */
  0xFBC2,   /* U+11399 */
  0xFBC2,   /* U+1139A */
  0xFBC2,   /* U+1139B */
  0xFBC2,   /* U+1139C */
  0xFBC2,   /* U+1139D */
  0xFBC2,   /* U+1139E */
  0xFBC2,   /* U+1139F */
  0xFBC2,   /* U+113A0 */
  0xFBC2,   /* U+113A1 */
  0xFBC2,   /* U+113A2 */
  0xFBC2,   /* U+113A3 */
  0xFBC2,   /* U+113A4 */
  0xFBC2,   /* U+113A5 */
  0xFBC2,   /* U+113A6 */
  0xFBC2,   /* U+113A7 */
  0xFBC2,   /* U+113A8 */
  0xFBC2,   /* U+113A9 */
  0xFBC2,   /* U+113AA */
  0xFBC2,   /* U+113AB */
  0xFBC2,   /* U+113AC */
  0xFBC2,   /* U+113AD */
  0xFBC2,   /* U+113AE */
  0xFBC2,   /* U+113AF */
  0xFBC2,   /* U+113B0 */
  0xFBC2,   /* U+113B1 */
  0xFBC2,   /* U+113B2 */
  0xFBC2,   /* U+113B3 */
  0xFBC2,   /* U+113B4 */
  0xFBC2,   /* U+113B5 */
  0xFBC2,   /* U+113B6 */
  0xFBC2,   /* U+113B7 */
  0xFBC2,   /* U+113B8 */
  0xFBC2,   /* U+113B9 */
  0xFBC2,   /* U+113BA */
  0xFBC2,   /* U+113BB */
  0xFBC2,   /* U+113BC */
  0xFBC2,   /* U+113BD */
  0xFBC2,   /* U+113BE */
  0xFBC2,   /* U+113BF */
  0xFBC2,   /* U+113C0 */
  0xFBC2,   /* U+113C1 */
  0xFBC2,   /* U+113C2 */
  0xFBC2,   /* U+113C3 */
  0xFBC2,   /* U+113C4 */
  0xFBC2,   /* U+113C5 */
  0xFBC2,   /* U+113C6 */
  0xFBC2,   /* U+113C7 */
  0xFBC2,   /* U+113C8 */
  0xFBC2,   /* U+113C9 */
  0xFBC2,   /* U+113CA */
  0xFBC2,   /* U+113CB */
  0xFBC2,   /* U+113CC */
  0xFBC2,   /* U+113CD */
  0xFBC2,   /* U+113CE */
  0xFBC2,   /* U+113CF */
  0xFBC2,   /* U+113D0 */
  0xFBC2,   /* U+113D1 */
  0xFBC2,   /* U+113D2 */
  0xFBC2,   /* U+113D3 */
  0xFBC2,   /* U+113D4 */
  0xFBC2,   /* U+113D5 */
  0xFBC2,   /* U+113D6 */
  0xFBC2,   /* U+113D7 */
  0xFBC2,   /* U+113D8 */
  0xFBC2,   /* U+113D9 */
  0xFBC2,   /* U+113DA */
  0xFBC2,   /* U+113DB */
  0xFBC2,   /* U+113DC */
  0xFBC2,   /* U+113DD */
  0xFBC2,   /* U+113DE */
  0xFBC2,   /* U+113DF */
  0xFBC2,   /* U+113E0 */
  0xFBC2,   /* U+113E1 */
  0xFBC2,   /* U+113E2 */
  0xFBC2,   /* U+113E3 */
  0xFBC2,   /* U+113E4 */
  0xFBC2,   /* U+113E5 */
  0xFBC2,   /* U+113E6 */
  0xFBC2,   /* U+113E7 */
  0xFBC2,   /* U+113E8 */
  0xFBC2,   /* U+113E9 */
  0xFBC2,   /* U+113EA */
  0xFBC2,   /* U+113EB */
  0xFBC2,   /* U+113EC */
  0xFBC2,   /* U+113ED */
  0xFBC2,   /* U+113EE */
  0xFBC2,   /* U+113EF */
  0xFBC2,   /* U+113F0 */
  0xFBC2,   /* U+113F1 */
  0xFBC2,   /* U+113F2 */
  0xFBC2,   /* U+113F3 */
  0xFBC2,   /* U+113F4 */
  0xFBC2,   /* U+113F5 */
  0xFBC2,   /* U+113F6 */
  0xFBC2,   /* U+113F7 */
  0xFBC2,   /* U+113F8 */
  0xFBC2,   /* U+113F9 */
  0xFBC2,   /* U+113FA */
  0xFBC2,   /* U+113FB */
  0xFBC2,   /* U+113FC */
  0xFBC2,   /* U+113FD */
  0xFBC2,   /* U+113FE */
  0xFBC2,   /* U+113FF */

  /* Secondary weight 1 for each character. */
  0x00C4,   /* U+11300 */
  0x00C3,   /* U+11301 */
  0x00C4,   /* U+11302 */
  0x00C5,   /* U+11303 */
  0x0020,   /* U+11304 */
  0x0020,   /* U+11305 */
  0x0020,   /* U+11306 */
  0x0020,   /* U+11307 */
  0x0020,   /* U+11308 */
  0x0020,   /* U+11309 */
  0x0020,   /* U+1130A */
  0x0020,   /* U+1130B */
  0x0020,   /* U+1130C */
  0x0020,   /* U+1130D */
  0x0020,   /* U+1130E */
  0x0020,   /* U+1130F */
  0x0020,   /* U+11310 */
  0x0020,   /* U+11311 */
  0x0020,   /* U+11312 */
  0x0020,   /* U+11313 */
  0x0020,   /* U+11314 */
  0x0020,   /* U+11315 */
  0x0020,   /* U+11316 */
  0x0020,   /* U+11317 */
  0x0020,   /* U+11318 */
  0x0020,   /* U+11319 */
  0x0020,   /* U+1131A */
  0x0020,   /* U+1131B */
  0x0020,   /* U+1131C */
  0x0020,   /* U+1131D */
  0x0020,   /* U+1131E */
  0x0020,   /* U+1131F */
  0x0020,   /* U+11320 */
  0x0020,   /* U+11321 */
  0x0020,   /* U+11322 */
  0x0020,   /* U+11323 */
  0x0020,   /* U+11324 */
  0x0020,   /* U+11325 */
  0x0020,   /* U+11326 */
  0x0020,   /* U+11327 */
  0x0020,   /* U+11328 */
  0x0020,   /* U+11329 */
  0x0020,   /* U+1132A */
  0x0020,   /* U+1132B */
  0x0020,   /* U+1132C */
  0x0020,   /* U+1132D */
  0x0020,   /* U+1132E */
  0x0020,   /* U+1132F */
  0x0020,   /* U+11330 */
  0x0020,   /* U+11331 */
  0x0020,   /* U+11332 */
  0x0020,   /* U+11333 */
  0x0020,   /* U+11334 */
  0x0020,   /* U+11335 */
  0x0020,   /* U+11336 */
  0x0020,   /* U+11337 */
  0x0020,   /* U+11338 */
  0x0020,   /* U+11339 */
  0x0020,   /* U+1133A */
  0x0020,   /* U+1133B */
  0x00C2,   /* U+1133C */
  0x0020,   /* U+1133D */
  0x0020,   /* U+1133E */
  0x0020,   /* U+1133F */
  0x0020,   /* U+11340 */
  0x0020,   /* U+11341 */
  0x0020,   /* U+11342 */
  0x0020,   /* U+11343 */
  0x0020,   /* U+11344 */
  0x0020,   /* U+11345 */
  0x0020,   /* U+11346 */
  0x0020,   /* U+11347 */
  0x0020,   /* U+11348 */
  0x0020,   /* U+11349 */
  0x0020,   /* U+1134A */
  0x0020,   /* U+1134B */
  0x0020,   /* U+1134C */
  0x0020,   /* U+1134D */
  0x0020,   /* U+1134E */
  0x0020,   /* U+1134F */
  0x0020,   /* U+11350 */
  0x0020,   /* U+11351 */
  0x0020,   /* U+11352 */
  0x0020,   /* U+11353 */
  0x0020,   /* U+11354 */
  0x0020,   /* U+11355 */
  0x0020,   /* U+11356 */
  0x0020,   /* U+11357 */
  0x0020,   /* U+11358 */
  0x0020,   /* U+11359 */
  0x0020,   /* U+1135A */
  0x0020,   /* U+1135B */
  0x0020,   /* U+1135C */
  0x0020,   /* U+1135D */
  0x0020,   /* U+1135E */
  0x0020,   /* U+1135F */
  0x0020,   /* U+11360 */
  0x0020,   /* U+11361 */
  0x0020,   /* U+11362 */
  0x0020,   /* U+11363 */
  0x0020,   /* U+11364 */
  0x0020,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0x0020,   /* U+1136D */
  0x0020,   /* U+1136E */
  0x0020,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0x0020,   /* U+11375 */
  0x0020,   /* U+11376 */
  0x0020,   /* U+11377 */
  0x0020,   /* U+11378 */
  0x0020,   /* U+11379 */
  0x0020,   /* U+1137A */
  0x0020,   /* U+1137B */
  0x0020,   /* U+1137C */
  0x0020,   /* U+1137D */
  0x0020,   /* U+1137E */
  0x0020,   /* U+1137F */
  0x0020,   /* U+11380 */
  0x0020,   /* U+11381 */
  0x0020,   /* U+11382 */
  0x0020,   /* U+11383 */
  0x0020,   /* U+11384 */
  0x0020,   /* U+11385 */
  0x0020,   /* U+11386 */
  0x0020,   /* U+11387 */
  0x0020,   /* U+11388 */
  0x0020,   /* U+11389 */
  0x0020,   /* U+1138A */
  0x0020,   /* U+1138B */
  0x0020,   /* U+1138C */
  0x0020,   /* U+1138D */
  0x0020,   /* U+1138E */
  0x0020,   /* U+1138F */
  0x0020,   /* U+11390 */
  0x0020,   /* U+11391 */
  0x0020,   /* U+11392 */
  0x0020,   /* U+11393 */
  0x0020,   /* U+11394 */
  0x0020,   /* U+11395 */
  0x0020,   /* U+11396 */
  0x0020,   /* U+11397 */
  0x0020,   /* U+11398 */
  0x0020,   /* U+11399 */
  0x0020,   /* U+1139A */
  0x0020,   /* U+1139B */
  0x0020,   /* U+1139C */
  0x0020,   /* U+1139D */
  0x0020,   /* U+1139E */
  0x0020,   /* U+1139F */
  0x0020,   /* U+113A0 */
  0x0020,   /* U+113A1 */
  0x0020,   /* U+113A2 */
  0x0020,   /* U+113A3 */
  0x0020,   /* U+113A4 */
  0x0020,   /* U+113A5 */
  0x0020,   /* U+113A6 */
  0x0020,   /* U+113A7 */
  0x0020,   /* U+113A8 */
  0x0020,   /* U+113A9 */
  0x0020,   /* U+113AA */
  0x0020,   /* U+113AB */
  0x0020,   /* U+113AC */
  0x0020,   /* U+113AD */
  0x0020,   /* U+113AE */
  0x0020,   /* U+113AF */
  0x0020,   /* U+113B0 */
  0x0020,   /* U+113B1 */
  0x0020,   /* U+113B2 */
  0x0020,   /* U+113B3 */
  0x0020,   /* U+113B4 */
  0x0020,   /* U+113B5 */
  0x0020,   /* U+113B6 */
  0x0020,   /* U+113B7 */
  0x0020,   /* U+113B8 */
  0x0020,   /* U+113B9 */
  0x0020,   /* U+113BA */
  0x0020,   /* U+113BB */
  0x0020,   /* U+113BC */
  0x0020,   /* U+113BD */
  0x0020,   /* U+113BE */
  0x0020,   /* U+113BF */
  0x0020,   /* U+113C0 */
  0x0020,   /* U+113C1 */
  0x0020,   /* U+113C2 */
  0x0020,   /* U+113C3 */
  0x0020,   /* U+113C4 */
  0x0020,   /* U+113C5 */
  0x0020,   /* U+113C6 */
  0x0020,   /* U+113C7 */
  0x0020,   /* U+113C8 */
  0x0020,   /* U+113C9 */
  0x0020,   /* U+113CA */
  0x0020,   /* U+113CB */
  0x0020,   /* U+113CC */
  0x0020,   /* U+113CD */
  0x0020,   /* U+113CE */
  0x0020,   /* U+113CF */
  0x0020,   /* U+113D0 */
  0x0020,   /* U+113D1 */
  0x0020,   /* U+113D2 */
  0x0020,   /* U+113D3 */
  0x0020,   /* U+113D4 */
  0x0020,   /* U+113D5 */
  0x0020,   /* U+113D6 */
  0x0020,   /* U+113D7 */
  0x0020,   /* U+113D8 */
  0x0020,   /* U+113D9 */
  0x0020,   /* U+113DA */
  0x0020,   /* U+113DB */
  0x0020,   /* U+113DC */
  0x0020,   /* U+113DD */
  0x0020,   /* U+113DE */
  0x0020,   /* U+113DF */
  0x0020,   /* U+113E0 */
  0x0020,   /* U+113E1 */
  0x0020,   /* U+113E2 */
  0x0020,   /* U+113E3 */
  0x0020,   /* U+113E4 */
  0x0020,   /* U+113E5 */
  0x0020,   /* U+113E6 */
  0x0020,   /* U+113E7 */
  0x0020,   /* U+113E8 */
  0x0020,   /* U+113E9 */
  0x0020,   /* U+113EA */
  0x0020,   /* U+113EB */
  0x0020,   /* U+113EC */
  0x0020,   /* U+113ED */
  0x0020,   /* U+113EE */
  0x0020,   /* U+113EF */
  0x0020,   /* U+113F0 */
  0x0020,   /* U+113F1 */
  0x0020,   /* U+113F2 */
  0x0020,   /* U+113F3 */
  0x0020,   /* U+113F4 */
  0x0020,   /* U+113F5 */
  0x0020,   /* U+113F6 */
  0x0020,   /* U+113F7 */
  0x0020,   /* U+113F8 */
  0x0020,   /* U+113F9 */
  0x0020,   /* U+113FA */
  0x0020,   /* U+113FB */
  0x0020,   /* U+113FC */
  0x0020,   /* U+113FD */
  0x0020,   /* U+113FE */
  0x0020,   /* U+113FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11300 */
  0x0002,   /* U+11301 */
  0x0002,   /* U+11302 */
  0x0002,   /* U+11303 */
  0x0002,   /* U+11304 */
  0x0002,   /* U+11305 */
  0x0002,   /* U+11306 */
  0x0002,   /* U+11307 */
  0x0002,   /* U+11308 */
  0x0002,   /* U+11309 */
  0x0002,   /* U+1130A */
  0x0002,   /* U+1130B */
  0x0002,   /* U+1130C */
  0x0002,   /* U+1130D */
  0x0002,   /* U+1130E */
  0x0002,   /* U+1130F */
  0x0002,   /* U+11310 */
  0x0002,   /* U+11311 */
  0x0002,   /* U+11312 */
  0x0002,   /* U+11313 */
  0x0002,   /* U+11314 */
  0x0002,   /* U+11315 */
  0x0002,   /* U+11316 */
  0x0002,   /* U+11317 */
  0x0002,   /* U+11318 */
  0x0002,   /* U+11319 */
  0x0002,   /* U+1131A */
  0x0002,   /* U+1131B */
  0x0002,   /* U+1131C */
  0x0002,   /* U+1131D */
  0x0002,   /* U+1131E */
  0x0002,   /* U+1131F */
  0x0002,   /* U+11320 */
  0x0002,   /* U+11321 */
  0x0002,   /* U+11322 */
  0x0002,   /* U+11323 */
  0x0002,   /* U+11324 */
  0x0002,   /* U+11325 */
  0x0002,   /* U+11326 */
  0x0002,   /* U+11327 */
  0x0002,   /* U+11328 */
  0x0002,   /* U+11329 */
  0x0002,   /* U+1132A */
  0x0002,   /* U+1132B */
  0x0002,   /* U+1132C */
  0x0002,   /* U+1132D */
  0x0002,   /* U+1132E */
  0x0002,   /* U+1132F */
  0x0002,   /* U+11330 */
  0x0002,   /* U+11331 */
  0x0002,   /* U+11332 */
  0x0002,   /* U+11333 */
  0x0002,   /* U+11334 */
  0x0002,   /* U+11335 */
  0x0002,   /* U+11336 */
  0x0002,   /* U+11337 */
  0x0002,   /* U+11338 */
  0x0002,   /* U+11339 */
  0x0002,   /* U+1133A */
  0x0002,   /* U+1133B */
  0x0002,   /* U+1133C */
  0x0002,   /* U+1133D */
  0x0002,   /* U+1133E */
  0x0002,   /* U+1133F */
  0x0002,   /* U+11340 */
  0x0002,   /* U+11341 */
  0x0002,   /* U+11342 */
  0x0002,   /* U+11343 */
  0x0002,   /* U+11344 */
  0x0002,   /* U+11345 */
  0x0002,   /* U+11346 */
  0x0002,   /* U+11347 */
  0x0002,   /* U+11348 */
  0x0002,   /* U+11349 */
  0x0002,   /* U+1134A */
  0x0002,   /* U+1134B */
  0x0002,   /* U+1134C */
  0x0002,   /* U+1134D */
  0x0002,   /* U+1134E */
  0x0002,   /* U+1134F */
  0x0002,   /* U+11350 */
  0x0002,   /* U+11351 */
  0x0002,   /* U+11352 */
  0x0002,   /* U+11353 */
  0x0002,   /* U+11354 */
  0x0002,   /* U+11355 */
  0x0002,   /* U+11356 */
  0x0002,   /* U+11357 */
  0x0002,   /* U+11358 */
  0x0002,   /* U+11359 */
  0x0002,   /* U+1135A */
  0x0002,   /* U+1135B */
  0x0002,   /* U+1135C */
  0x0002,   /* U+1135D */
  0x0002,   /* U+1135E */
  0x0002,   /* U+1135F */
  0x0002,   /* U+11360 */
  0x0002,   /* U+11361 */
  0x0002,   /* U+11362 */
  0x0002,   /* U+11363 */
  0x0002,   /* U+11364 */
  0x0002,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0x0002,   /* U+1136D */
  0x0002,   /* U+1136E */
  0x0002,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0x0002,   /* U+11375 */
  0x0002,   /* U+11376 */
  0x0002,   /* U+11377 */
  0x0002,   /* U+11378 */
  0x0002,   /* U+11379 */
  0x0002,   /* U+1137A */
  0x0002,   /* U+1137B */
  0x0002,   /* U+1137C */
  0x0002,   /* U+1137D */
  0x0002,   /* U+1137E */
  0x0002,   /* U+1137F */
  0x0002,   /* U+11380 */
  0x0002,   /* U+11381 */
  0x0002,   /* U+11382 */
  0x0002,   /* U+11383 */
  0x0002,   /* U+11384 */
  0x0002,   /* U+11385 */
  0x0002,   /* U+11386 */
  0x0002,   /* U+11387 */
  0x0002,   /* U+11388 */
  0x0002,   /* U+11389 */
  0x0002,   /* U+1138A */
  0x0002,   /* U+1138B */
  0x0002,   /* U+1138C */
  0x0002,   /* U+1138D */
  0x0002,   /* U+1138E */
  0x0002,   /* U+1138F */
  0x0002,   /* U+11390 */
  0x0002,   /* U+11391 */
  0x0002,   /* U+11392 */
  0x0002,   /* U+11393 */
  0x0002,   /* U+11394 */
  0x0002,   /* U+11395 */
  0x0002,   /* U+11396 */
  0x0002,   /* U+11397 */
  0x0002,   /* U+11398 */
  0x0002,   /* U+11399 */
  0x0002,   /* U+1139A */
  0x0002,   /* U+1139B */
  0x0002,   /* U+1139C */
  0x0002,   /* U+1139D */
  0x0002,   /* U+1139E */
  0x0002,   /* U+1139F */
  0x0002,   /* U+113A0 */
  0x0002,   /* U+113A1 */
  0x0002,   /* U+113A2 */
  0x0002,   /* U+113A3 */
  0x0002,   /* U+113A4 */
  0x0002,   /* U+113A5 */
  0x0002,   /* U+113A6 */
  0x0002,   /* U+113A7 */
  0x0002,   /* U+113A8 */
  0x0002,   /* U+113A9 */
  0x0002,   /* U+113AA */
  0x0002,   /* U+113AB */
  0x0002,   /* U+113AC */
  0x0002,   /* U+113AD */
  0x0002,   /* U+113AE */
  0x0002,   /* U+113AF */
  0x0002,   /* U+113B0 */
  0x0002,   /* U+113B1 */
  0x0002,   /* U+113B2 */
  0x0002,   /* U+113B3 */
  0x0002,   /* U+113B4 */
  0x0002,   /* U+113B5 */
  0x0002,   /* U+113B6 */
  0x0002,   /* U+113B7 */
  0x0002,   /* U+113B8 */
  0x0002,   /* U+113B9 */
  0x0002,   /* U+113BA */
  0x0002,   /* U+113BB */
  0x0002,   /* U+113BC */
  0x0002,   /* U+113BD */
  0x0002,   /* U+113BE */
  0x0002,   /* U+113BF */
  0x0002,   /* U+113C0 */
  0x0002,   /* U+113C1 */
  0x0002,   /* U+113C2 */
  0x0002,   /* U+113C3 */
  0x0002,   /* U+113C4 */
  0x0002,   /* U+113C5 */
  0x0002,   /* U+113C6 */
  0x0002,   /* U+113C7 */
  0x0002,   /* U+113C8 */
  0x0002,   /* U+113C9 */
  0x0002,   /* U+113CA */
  0x0002,   /* U+113CB */
  0x0002,   /* U+113CC */
  0x0002,   /* U+113CD */
  0x0002,   /* U+113CE */
  0x0002,   /* U+113CF */
  0x0002,   /* U+113D0 */
  0x0002,   /* U+113D1 */
  0x0002,   /* U+113D2 */
  0x0002,   /* U+113D3 */
  0x0002,   /* U+113D4 */
  0x0002,   /* U+113D5 */
  0x0002,   /* U+113D6 */
  0x0002,   /* U+113D7 */
  0x0002,   /* U+113D8 */
  0x0002,   /* U+113D9 */
  0x0002,   /* U+113DA */
  0x0002,   /* U+113DB */
  0x0002,   /* U+113DC */
  0x0002,   /* U+113DD */
  0x0002,   /* U+113DE */
  0x0002,   /* U+113DF */
  0x0002,   /* U+113E0 */
  0x0002,   /* U+113E1 */
  0x0002,   /* U+113E2 */
  0x0002,   /* U+113E3 */
  0x0002,   /* U+113E4 */
  0x0002,   /* U+113E5 */
  0x0002,   /* U+113E6 */
  0x0002,   /* U+113E7 */
  0x0002,   /* U+113E8 */
  0x0002,   /* U+113E9 */
  0x0002,   /* U+113EA */
  0x0002,   /* U+113EB */
  0x0002,   /* U+113EC */
  0x0002,   /* U+113ED */
  0x0002,   /* U+113EE */
  0x0002,   /* U+113EF */
  0x0002,   /* U+113F0 */
  0x0002,   /* U+113F1 */
  0x0002,   /* U+113F2 */
  0x0002,   /* U+113F3 */
  0x0002,   /* U+113F4 */
  0x0002,   /* U+113F5 */
  0x0002,   /* U+113F6 */
  0x0002,   /* U+113F7 */
  0x0002,   /* U+113F8 */
  0x0002,   /* U+113F9 */
  0x0002,   /* U+113FA */
  0x0002,   /* U+113FB */
  0x0002,   /* U+113FC */
  0x0002,   /* U+113FD */
  0x0002,   /* U+113FE */
  0x0002,   /* U+113FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11300 */
  0x0000,   /* U+11301 */
  0x0000,   /* U+11302 */
  0x0000,   /* U+11303 */
  0x9304,   /* U+11304 */
  0x0000,   /* U+11305 */
  0x0000,   /* U+11306 */
  0x0000,   /* U+11307 */
  0x0000,   /* U+11308 */
  0x0000,   /* U+11309 */
  0x0000,   /* U+1130A */
  0x0000,   /* U+1130B */
  0x0000,   /* U+1130C */
  0x930D,   /* U+1130D */
  0x930E,   /* U+1130E */
  0x0000,   /* U+1130F */
  0x0000,   /* U+11310 */
  0x9311,   /* U+11311 */
  0x9312,   /* U+11312 */
  0x0000,   /* U+11313 */
  0x0000,   /* U+11314 */
  0x0000,   /* U+11315 */
  0x0000,   /* U+11316 */
  0x0000,   /* U+11317 */
  0x0000,   /* U+11318 */
  0x0000,   /* U+11319 */
  0x0000,   /* U+1131A */
  0x0000,   /* U+1131B */
  0x0000,   /* U+1131C */
  0x0000,   /* U+1131D */
  0x0000,   /* U+1131E */
  0x0000,   /* U+1131F */
  0x0000,   /* U+11320 */
  0x0000,   /* U+11321 */
  0x0000,   /* U+11322 */
  0x0000,   /* U+11323 */
  0x0000,   /* U+11324 */
  0x0000,   /* U+11325 */
  0x0000,   /* U+11326 */
  0x0000,   /* U+11327 */
  0x0000,   /* U+11328 */
  0x9329,   /* U+11329 */
  0x0000,   /* U+1132A */
  0x0000,   /* U+1132B */
  0x0000,   /* U+1132C */
  0x0000,   /* U+1132D */
  0x0000,   /* U+1132E */
  0x0000,   /* U+1132F */
  0x0000,   /* U+11330 */
  0x9331,   /* U+11331 */
  0x0000,   /* U+11332 */
  0x0000,   /* U+11333 */
  0x9334,   /* U+11334 */
  0x0000,   /* U+11335 */
  0x0000,   /* U+11336 */
  0x0000,   /* U+11337 */
  0x0000,   /* U+11338 */
  0x0000,   /* U+11339 */
  0x933A,   /* U+1133A */
  0x933B,   /* U+1133B */
  0x0000,   /* U+1133C */
  0x0000,   /* U+1133D */
  0x0000,   /* U+1133E */
  0x0000,   /* U+1133F */
  0x0000,   /* U+11340 */
  0x0000,   /* U+11341 */
  0x0000,   /* U+11342 */
  0x0000,   /* U+11343 */
  0x0000,   /* U+11344 */
  0x9345,   /* U+11345 */
  0x9346,   /* U+11346 */
  0x0000,   /* U+11347 */
  0x0000,   /* U+11348 */
  0x9349,   /* U+11349 */
  0x934A,   /* U+1134A */
  0x0000,   /* U+1134B */
  0x0000,   /* U+1134C */
  0x0000,   /* U+1134D */
  0x934E,   /* U+1134E */
  0x934F,   /* U+1134F */
  0x0000,   /* U+11350 */
  0x9351,   /* U+11351 */
  0x9352,   /* U+11352 */
  0x9353,   /* U+11353 */
  0x9354,   /* U+11354 */
  0x9355,   /* U+11355 */
  0x9356,   /* U+11356 */
  0x0000,   /* U+11357 */
  0x9358,   /* U+11358 */
  0x9359,   /* U+11359 */
  0x935A,   /* U+1135A */
  0x935B,   /* U+1135B */
  0x935C,   /* U+1135C */
  0x0000,   /* U+1135D */
  0x0000,   /* U+1135E */
  0x0000,   /* U+1135F */
  0x0000,   /* U+11360 */
  0x0000,   /* U+11361 */
  0x0000,   /* U+11362 */
  0x0000,   /* U+11363 */
  0x9364,   /* U+11364 */
  0x9365,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0x936D,   /* U+1136D */
  0x936E,   /* U+1136E */
  0x936F,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0x9375,   /* U+11375 */
  0x9376,   /* U+11376 */
  0x9377,   /* U+11377 */
  0x9378,   /* U+11378 */
  0x9379,   /* U+11379 */
  0x937A,   /* U+1137A */
  0x937B,   /* U+1137B */
  0x937C,   /* U+1137C */
  0x937D,   /* U+1137D */
  0x937E,   /* U+1137E */
  0x937F,   /* U+1137F */
  0x9380,   /* U+11380 */
  0x9381,   /* U+11381 */
  0x9382,   /* U+11382 */
  0x9383,   /* U+11383 */
  0x9384,   /* U+11384 */
  0x9385,   /* U+11385 */
  0x9386,   /* U+11386 */
  0x9387,   /* U+11387 */
  0x9388,   /* U+11388 */
  0x9389,   /* U+11389 */
  0x938A,   /* U+1138A */
  0x938B,   /* U+1138B */
  0x938C,   /* U+1138C */
  0x938D,   /* U+1138D */
  0x938E,   /* U+1138E */
  0x938F,   /* U+1138F */
  0x9390,   /* U+11390 */
  0x9391,   /* U+11391 */
  0x9392,   /* U+11392 */
  0x9393,   /* U+11393 */
  0x9394,   /* U+11394 */
  0x9395,   /* U+11395 */
  0x9396,   /* U+11396 */
  0x9397,   /* U+11397 */
  0x9398,   /* U+11398 */
  0x9399,   /* U+11399 */
  0x939A,   /* U+1139A */
  0x939B,   /* U+1139B */
  0x939C,   /* U+1139C */
  0x939D,   /* U+1139D */
  0x939E,   /* U+1139E */
  0x939F,   /* U+1139F */
  0x93A0,   /* U+113A0 */
  0x93A1,   /* U+113A1 */
  0x93A2,   /* U+113A2 */
  0x93A3,   /* U+113A3 */
  0x93A4,   /* U+113A4 */
  0x93A5,   /* U+113A5 */
  0x93A6,   /* U+113A6 */
  0x93A7,   /* U+113A7 */
  0x93A8,   /* U+113A8 */
  0x93A9,   /* U+113A9 */
  0x93AA,   /* U+113AA */
  0x93AB,   /* U+113AB */
  0x93AC,   /* U+113AC */
  0x93AD,   /* U+113AD */
  0x93AE,   /* U+113AE */
  0x93AF,   /* U+113AF */
  0x93B0,   /* U+113B0 */
  0x93B1,   /* U+113B1 */
  0x93B2,   /* U+113B2 */
  0x93B3,   /* U+113B3 */
  0x93B4,   /* U+113B4 */
  0x93B5,   /* U+113B5 */
  0x93B6,   /* U+113B6 */
  0x93B7,   /* U+113B7 */
  0x93B8,   /* U+113B8 */
  0x93B9,   /* U+113B9 */
  0x93BA,   /* U+113BA */
  0x93BB,   /* U+113BB */
  0x93BC,   /* U+113BC */
  0x93BD,   /* U+113BD */
  0x93BE,   /* U+113BE */
  0x93BF,   /* U+113BF */
  0x93C0,   /* U+113C0 */
  0x93C1,   /* U+113C1 */
  0x93C2,   /* U+113C2 */
  0x93C3,   /* U+113C3 */
  0x93C4,   /* U+113C4 */
  0x93C5,   /* U+113C5 */
  0x93C6,   /* U+113C6 */
  0x93C7,   /* U+113C7 */
  0x93C8,   /* U+113C8 */
  0x93C9,   /* U+113C9 */
  0x93CA,   /* U+113CA */
  0x93CB,   /* U+113CB */
  0x93CC,   /* U+113CC */
  0x93CD,   /* U+113CD */
  0x93CE,   /* U+113CE */
  0x93CF,   /* U+113CF */
  0x93D0,   /* U+113D0 */
  0x93D1,   /* U+113D1 */
  0x93D2,   /* U+113D2 */
  0x93D3,   /* U+113D3 */
  0x93D4,   /* U+113D4 */
  0x93D5,   /* U+113D5 */
  0x93D6,   /* U+113D6 */
  0x93D7,   /* U+113D7 */
  0x93D8,   /* U+113D8 */
  0x93D9,   /* U+113D9 */
  0x93DA,   /* U+113DA */
  0x93DB,   /* U+113DB */
  0x93DC,   /* U+113DC */
  0x93DD,   /* U+113DD */
  0x93DE,   /* U+113DE */
  0x93DF,   /* U+113DF */
  0x93E0,   /* U+113E0 */
  0x93E1,   /* U+113E1 */
  0x93E2,   /* U+113E2 */
  0x93E3,   /* U+113E3 */
  0x93E4,   /* U+113E4 */
  0x93E5,   /* U+113E5 */
  0x93E6,   /* U+113E6 */
  0x93E7,   /* U+113E7 */
  0x93E8,   /* U+113E8 */
  0x93E9,   /* U+113E9 */
  0x93EA,   /* U+113EA */
  0x93EB,   /* U+113EB */
  0x93EC,   /* U+113EC */
  0x93ED,   /* U+113ED */
  0x93EE,   /* U+113EE */
  0x93EF,   /* U+113EF */
  0x93F0,   /* U+113F0 */
  0x93F1,   /* U+113F1 */
  0x93F2,   /* U+113F2 */
  0x93F3,   /* U+113F3 */
  0x93F4,   /* U+113F4 */
  0x93F5,   /* U+113F5 */
  0x93F6,   /* U+113F6 */
  0x93F7,   /* U+113F7 */
  0x93F8,   /* U+113F8 */
  0x93F9,   /* U+113F9 */
  0x93FA,   /* U+113FA */
  0x93FB,   /* U+113FB */
  0x93FC,   /* U+113FC */
  0x93FD,   /* U+113FD */
  0x93FE,   /* U+113FE */
  0x93FF,   /* U+113FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11300 */
  0x0000,   /* U+11301 */
  0x0000,   /* U+11302 */
  0x0000,   /* U+11303 */
  0x0000,   /* U+11304 */
  0x0000,   /* U+11305 */
  0x0000,   /* U+11306 */
  0x0000,   /* U+11307 */
  0x0000,   /* U+11308 */
  0x0000,   /* U+11309 */
  0x0000,   /* U+1130A */
  0x0000,   /* U+1130B */
  0x0000,   /* U+1130C */
  0x0000,   /* U+1130D */
  0x0000,   /* U+1130E */
  0x0000,   /* U+1130F */
  0x0000,   /* U+11310 */
  0x0000,   /* U+11311 */
  0x0000,   /* U+11312 */
  0x0000,   /* U+11313 */
  0x0000,   /* U+11314 */
  0x0000,   /* U+11315 */
  0x0000,   /* U+11316 */
  0x0000,   /* U+11317 */
  0x0000,   /* U+11318 */
  0x0000,   /* U+11319 */
  0x0000,   /* U+1131A */
  0x0000,   /* U+1131B */
  0x0000,   /* U+1131C */
  0x0000,   /* U+1131D */
  0x0000,   /* U+1131E */
  0x0000,   /* U+1131F */
  0x0000,   /* U+11320 */
  0x0000,   /* U+11321 */
  0x0000,   /* U+11322 */
  0x0000,   /* U+11323 */
  0x0000,   /* U+11324 */
  0x0000,   /* U+11325 */
  0x0000,   /* U+11326 */
  0x0000,   /* U+11327 */
  0x0000,   /* U+11328 */
  0x0000,   /* U+11329 */
  0x0000,   /* U+1132A */
  0x0000,   /* U+1132B */
  0x0000,   /* U+1132C */
  0x0000,   /* U+1132D */
  0x0000,   /* U+1132E */
  0x0000,   /* U+1132F */
  0x0000,   /* U+11330 */
  0x0000,   /* U+11331 */
  0x0000,   /* U+11332 */
  0x0000,   /* U+11333 */
  0x0000,   /* U+11334 */
  0x0000,   /* U+11335 */
  0x0000,   /* U+11336 */
  0x0000,   /* U+11337 */
  0x0000,   /* U+11338 */
  0x0000,   /* U+11339 */
  0x0000,   /* U+1133A */
  0x0000,   /* U+1133B */
  0x0000,   /* U+1133C */
  0x0000,   /* U+1133D */
  0x0000,   /* U+1133E */
  0x0000,   /* U+1133F */
  0x0000,   /* U+11340 */
  0x0000,   /* U+11341 */
  0x0000,   /* U+11342 */
  0x0000,   /* U+11343 */
  0x0000,   /* U+11344 */
  0x0000,   /* U+11345 */
  0x0000,   /* U+11346 */
  0x0000,   /* U+11347 */
  0x0000,   /* U+11348 */
  0x0000,   /* U+11349 */
  0x0000,   /* U+1134A */
  0x0000,   /* U+1134B */
  0x0000,   /* U+1134C */
  0x0000,   /* U+1134D */
  0x0000,   /* U+1134E */
  0x0000,   /* U+1134F */
  0x0000,   /* U+11350 */
  0x0000,   /* U+11351 */
  0x0000,   /* U+11352 */
  0x0000,   /* U+11353 */
  0x0000,   /* U+11354 */
  0x0000,   /* U+11355 */
  0x0000,   /* U+11356 */
  0x0000,   /* U+11357 */
  0x0000,   /* U+11358 */
  0x0000,   /* U+11359 */
  0x0000,   /* U+1135A */
  0x0000,   /* U+1135B */
  0x0000,   /* U+1135C */
  0x0000,   /* U+1135D */
  0x0000,   /* U+1135E */
  0x0000,   /* U+1135F */
  0x0000,   /* U+11360 */
  0x0000,   /* U+11361 */
  0x0000,   /* U+11362 */
  0x0000,   /* U+11363 */
  0x0000,   /* U+11364 */
  0x0000,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0x0000,   /* U+1136D */
  0x0000,   /* U+1136E */
  0x0000,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0x0000,   /* U+11375 */
  0x0000,   /* U+11376 */
  0x0000,   /* U+11377 */
  0x0000,   /* U+11378 */
  0x0000,   /* U+11379 */
  0x0000,   /* U+1137A */
  0x0000,   /* U+1137B */
  0x0000,   /* U+1137C */
  0x0000,   /* U+1137D */
  0x0000,   /* U+1137E */
  0x0000,   /* U+1137F */
  0x0000,   /* U+11380 */
  0x0000,   /* U+11381 */
  0x0000,   /* U+11382 */
  0x0000,   /* U+11383 */
  0x0000,   /* U+11384 */
  0x0000,   /* U+11385 */
  0x0000,   /* U+11386 */
  0x0000,   /* U+11387 */
  0x0000,   /* U+11388 */
  0x0000,   /* U+11389 */
  0x0000,   /* U+1138A */
  0x0000,   /* U+1138B */
  0x0000,   /* U+1138C */
  0x0000,   /* U+1138D */
  0x0000,   /* U+1138E */
  0x0000,   /* U+1138F */
  0x0000,   /* U+11390 */
  0x0000,   /* U+11391 */
  0x0000,   /* U+11392 */
  0x0000,   /* U+11393 */
  0x0000,   /* U+11394 */
  0x0000,   /* U+11395 */
  0x0000,   /* U+11396 */
  0x0000,   /* U+11397 */
  0x0000,   /* U+11398 */
  0x0000,   /* U+11399 */
  0x0000,   /* U+1139A */
  0x0000,   /* U+1139B */
  0x0000,   /* U+1139C */
  0x0000,   /* U+1139D */
  0x0000,   /* U+1139E */
  0x0000,   /* U+1139F */
  0x0000,   /* U+113A0 */
  0x0000,   /* U+113A1 */
  0x0000,   /* U+113A2 */
  0x0000,   /* U+113A3 */
  0x0000,   /* U+113A4 */
  0x0000,   /* U+113A5 */
  0x0000,   /* U+113A6 */
  0x0000,   /* U+113A7 */
  0x0000,   /* U+113A8 */
  0x0000,   /* U+113A9 */
  0x0000,   /* U+113AA */
  0x0000,   /* U+113AB */
  0x0000,   /* U+113AC */
  0x0000,   /* U+113AD */
  0x0000,   /* U+113AE */
  0x0000,   /* U+113AF */
  0x0000,   /* U+113B0 */
  0x0000,   /* U+113B1 */
  0x0000,   /* U+113B2 */
  0x0000,   /* U+113B3 */
  0x0000,   /* U+113B4 */
  0x0000,   /* U+113B5 */
  0x0000,   /* U+113B6 */
  0x0000,   /* U+113B7 */
  0x0000,   /* U+113B8 */
  0x0000,   /* U+113B9 */
  0x0000,   /* U+113BA */
  0x0000,   /* U+113BB */
  0x0000,   /* U+113BC */
  0x0000,   /* U+113BD */
  0x0000,   /* U+113BE */
  0x0000,   /* U+113BF */
  0x0000,   /* U+113C0 */
  0x0000,   /* U+113C1 */
  0x0000,   /* U+113C2 */
  0x0000,   /* U+113C3 */
  0x0000,   /* U+113C4 */
  0x0000,   /* U+113C5 */
  0x0000,   /* U+113C6 */
  0x0000,   /* U+113C7 */
  0x0000,   /* U+113C8 */
  0x0000,   /* U+113C9 */
  0x0000,   /* U+113CA */
  0x0000,   /* U+113CB */
  0x0000,   /* U+113CC */
  0x0000,   /* U+113CD */
  0x0000,   /* U+113CE */
  0x0000,   /* U+113CF */
  0x0000,   /* U+113D0 */
  0x0000,   /* U+113D1 */
  0x0000,   /* U+113D2 */
  0x0000,   /* U+113D3 */
  0x0000,   /* U+113D4 */
  0x0000,   /* U+113D5 */
  0x0000,   /* U+113D6 */
  0x0000,   /* U+113D7 */
  0x0000,   /* U+113D8 */
  0x0000,   /* U+113D9 */
  0x0000,   /* U+113DA */
  0x0000,   /* U+113DB */
  0x0000,   /* U+113DC */
  0x0000,   /* U+113DD */
  0x0000,   /* U+113DE */
  0x0000,   /* U+113DF */
  0x0000,   /* U+113E0 */
  0x0000,   /* U+113E1 */
  0x0000,   /* U+113E2 */
  0x0000,   /* U+113E3 */
  0x0000,   /* U+113E4 */
  0x0000,   /* U+113E5 */
  0x0000,   /* U+113E6 */
  0x0000,   /* U+113E7 */
  0x0000,   /* U+113E8 */
  0x0000,   /* U+113E9 */
  0x0000,   /* U+113EA */
  0x0000,   /* U+113EB */
  0x0000,   /* U+113EC */
  0x0000,   /* U+113ED */
  0x0000,   /* U+113EE */
  0x0000,   /* U+113EF */
  0x0000,   /* U+113F0 */
  0x0000,   /* U+113F1 */
  0x0000,   /* U+113F2 */
  0x0000,   /* U+113F3 */
  0x0000,   /* U+113F4 */
  0x0000,   /* U+113F5 */
  0x0000,   /* U+113F6 */
  0x0000,   /* U+113F7 */
  0x0000,   /* U+113F8 */
  0x0000,   /* U+113F9 */
  0x0000,   /* U+113FA */
  0x0000,   /* U+113FB */
  0x0000,   /* U+113FC */
  0x0000,   /* U+113FD */
  0x0000,   /* U+113FE */
  0x0000,   /* U+113FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11300 */
  0x0000,   /* U+11301 */
  0x0000,   /* U+11302 */
  0x0000,   /* U+11303 */
  0x0000,   /* U+11304 */
  0x0000,   /* U+11305 */
  0x0000,   /* U+11306 */
  0x0000,   /* U+11307 */
  0x0000,   /* U+11308 */
  0x0000,   /* U+11309 */
  0x0000,   /* U+1130A */
  0x0000,   /* U+1130B */
  0x0000,   /* U+1130C */
  0x0000,   /* U+1130D */
  0x0000,   /* U+1130E */
  0x0000,   /* U+1130F */
  0x0000,   /* U+11310 */
  0x0000,   /* U+11311 */
  0x0000,   /* U+11312 */
  0x0000,   /* U+11313 */
  0x0000,   /* U+11314 */
  0x0000,   /* U+11315 */
  0x0000,   /* U+11316 */
  0x0000,   /* U+11317 */
  0x0000,   /* U+11318 */
  0x0000,   /* U+11319 */
  0x0000,   /* U+1131A */
  0x0000,   /* U+1131B */
  0x0000,   /* U+1131C */
  0x0000,   /* U+1131D */
  0x0000,   /* U+1131E */
  0x0000,   /* U+1131F */
  0x0000,   /* U+11320 */
  0x0000,   /* U+11321 */
  0x0000,   /* U+11322 */
  0x0000,   /* U+11323 */
  0x0000,   /* U+11324 */
  0x0000,   /* U+11325 */
  0x0000,   /* U+11326 */
  0x0000,   /* U+11327 */
  0x0000,   /* U+11328 */
  0x0000,   /* U+11329 */
  0x0000,   /* U+1132A */
  0x0000,   /* U+1132B */
  0x0000,   /* U+1132C */
  0x0000,   /* U+1132D */
  0x0000,   /* U+1132E */
  0x0000,   /* U+1132F */
  0x0000,   /* U+11330 */
  0x0000,   /* U+11331 */
  0x0000,   /* U+11332 */
  0x0000,   /* U+11333 */
  0x0000,   /* U+11334 */
  0x0000,   /* U+11335 */
  0x0000,   /* U+11336 */
  0x0000,   /* U+11337 */
  0x0000,   /* U+11338 */
  0x0000,   /* U+11339 */
  0x0000,   /* U+1133A */
  0x0000,   /* U+1133B */
  0x0000,   /* U+1133C */
  0x0000,   /* U+1133D */
  0x0000,   /* U+1133E */
  0x0000,   /* U+1133F */
  0x0000,   /* U+11340 */
  0x0000,   /* U+11341 */
  0x0000,   /* U+11342 */
  0x0000,   /* U+11343 */
  0x0000,   /* U+11344 */
  0x0000,   /* U+11345 */
  0x0000,   /* U+11346 */
  0x0000,   /* U+11347 */
  0x0000,   /* U+11348 */
  0x0000,   /* U+11349 */
  0x0000,   /* U+1134A */
  0x0000,   /* U+1134B */
  0x0000,   /* U+1134C */
  0x0000,   /* U+1134D */
  0x0000,   /* U+1134E */
  0x0000,   /* U+1134F */
  0x0000,   /* U+11350 */
  0x0000,   /* U+11351 */
  0x0000,   /* U+11352 */
  0x0000,   /* U+11353 */
  0x0000,   /* U+11354 */
  0x0000,   /* U+11355 */
  0x0000,   /* U+11356 */
  0x0000,   /* U+11357 */
  0x0000,   /* U+11358 */
  0x0000,   /* U+11359 */
  0x0000,   /* U+1135A */
  0x0000,   /* U+1135B */
  0x0000,   /* U+1135C */
  0x0000,   /* U+1135D */
  0x0000,   /* U+1135E */
  0x0000,   /* U+1135F */
  0x0000,   /* U+11360 */
  0x0000,   /* U+11361 */
  0x0000,   /* U+11362 */
  0x0000,   /* U+11363 */
  0x0000,   /* U+11364 */
  0x0000,   /* U+11365 */
  0x0000,   /* U+11366 */
  0x0000,   /* U+11367 */
  0x0000,   /* U+11368 */
  0x0000,   /* U+11369 */
  0x0000,   /* U+1136A */
  0x0000,   /* U+1136B */
  0x0000,   /* U+1136C */
  0x0000,   /* U+1136D */
  0x0000,   /* U+1136E */
  0x0000,   /* U+1136F */
  0x0000,   /* U+11370 */
  0x0000,   /* U+11371 */
  0x0000,   /* U+11372 */
  0x0000,   /* U+11373 */
  0x0000,   /* U+11374 */
  0x0000,   /* U+11375 */
  0x0000,   /* U+11376 */
  0x0000,   /* U+11377 */
  0x0000,   /* U+11378 */
  0x0000,   /* U+11379 */
  0x0000,   /* U+1137A */
  0x0000,   /* U+1137B */
  0x0000,   /* U+1137C */
  0x0000,   /* U+1137D */
  0x0000,   /* U+1137E */
  0x0000,   /* U+1137F */
  0x0000,   /* U+11380 */
  0x0000,   /* U+11381 */
  0x0000,   /* U+11382 */
  0x0000,   /* U+11383 */
  0x0000,   /* U+11384 */
  0x0000,   /* U+11385 */
  0x0000,   /* U+11386 */
  0x0000,   /* U+11387 */
  0x0000,   /* U+11388 */
  0x0000,   /* U+11389 */
  0x0000,   /* U+1138A */
  0x0000,   /* U+1138B */
  0x0000,   /* U+1138C */
  0x0000,   /* U+1138D */
  0x0000,   /* U+1138E */
  0x0000,   /* U+1138F */
  0x0000,   /* U+11390 */
  0x0000,   /* U+11391 */
  0x0000,   /* U+11392 */
  0x0000,   /* U+11393 */
  0x0000,   /* U+11394 */
  0x0000,   /* U+11395 */
  0x0000,   /* U+11396 */
  0x0000,   /* U+11397 */
  0x0000,   /* U+11398 */
  0x0000,   /* U+11399 */
  0x0000,   /* U+1139A */
  0x0000,   /* U+1139B */
  0x0000,   /* U+1139C */
  0x0000,   /* U+1139D */
  0x0000,   /* U+1139E */
  0x0000,   /* U+1139F */
  0x0000,   /* U+113A0 */
  0x0000,   /* U+113A1 */
  0x0000,   /* U+113A2 */
  0x0000,   /* U+113A3 */
  0x0000,   /* U+113A4 */
  0x0000,   /* U+113A5 */
  0x0000,   /* U+113A6 */
  0x0000,   /* U+113A7 */
  0x0000,   /* U+113A8 */
  0x0000,   /* U+113A9 */
  0x0000,   /* U+113AA */
  0x0000,   /* U+113AB */
  0x0000,   /* U+113AC */
  0x0000,   /* U+113AD */
  0x0000,   /* U+113AE */
  0x0000,   /* U+113AF */
  0x0000,   /* U+113B0 */
  0x0000,   /* U+113B1 */
  0x0000,   /* U+113B2 */
  0x0000,   /* U+113B3 */
  0x0000,   /* U+113B4 */
  0x0000,   /* U+113B5 */
  0x0000,   /* U+113B6 */
  0x0000,   /* U+113B7 */
  0x0000,   /* U+113B8 */
  0x0000,   /* U+113B9 */
  0x0000,   /* U+113BA */
  0x0000,   /* U+113BB */
  0x0000,   /* U+113BC */
  0x0000,   /* U+113BD */
  0x0000,   /* U+113BE */
  0x0000,   /* U+113BF */
  0x0000,   /* U+113C0 */
  0x0000,   /* U+113C1 */
  0x0000,   /* U+113C2 */
  0x0000,   /* U+113C3 */
  0x0000,   /* U+113C4 */
  0x0000,   /* U+113C5 */
  0x0000,   /* U+113C6 */
  0x0000,   /* U+113C7 */
  0x0000,   /* U+113C8 */
  0x0000,   /* U+113C9 */
  0x0000,   /* U+113CA */
  0x0000,   /* U+113CB */
  0x0000,   /* U+113CC */
  0x0000,   /* U+113CD */
  0x0000,   /* U+113CE */
  0x0000,   /* U+113CF */
  0x0000,   /* U+113D0 */
  0x0000,   /* U+113D1 */
  0x0000,   /* U+113D2 */
  0x0000,   /* U+113D3 */
  0x0000,   /* U+113D4 */
  0x0000,   /* U+113D5 */
  0x0000,   /* U+113D6 */
  0x0000,   /* U+113D7 */
  0x0000,   /* U+113D8 */
  0x0000,   /* U+113D9 */
  0x0000,   /* U+113DA */
  0x0000,   /* U+113DB */
  0x0000,   /* U+113DC */
  0x0000,   /* U+113DD */
  0x0000,   /* U+113DE */
  0x0000,   /* U+113DF */
  0x0000,   /* U+113E0 */
  0x0000,   /* U+113E1 */
  0x0000,   /* U+113E2 */
  0x0000,   /* U+113E3 */
  0x0000,   /* U+113E4 */
  0x0000,   /* U+113E5 */
  0x0000,   /* U+113E6 */
  0x0000,   /* U+113E7 */
  0x0000,   /* U+113E8 */
  0x0000,   /* U+113E9 */
  0x0000,   /* U+113EA */
  0x0000,   /* U+113EB */
  0x0000,   /* U+113EC */
  0x0000,   /* U+113ED */
  0x0000,   /* U+113EE */
  0x0000,   /* U+113EF */
  0x0000,   /* U+113F0 */
  0x0000,   /* U+113F1 */
  0x0000,   /* U+113F2 */
  0x0000,   /* U+113F3 */
  0x0000,   /* U+113F4 */
  0x0000,   /* U+113F5 */
  0x0000,   /* U+113F6 */
  0x0000,   /* U+113F7 */
  0x0000,   /* U+113F8 */
  0x0000,   /* U+113F9 */
  0x0000,   /* U+113FA */
  0x0000,   /* U+113FB */
  0x0000,   /* U+113FC */
  0x0000,   /* U+113FD */
  0x0000,   /* U+113FE */
  0x0000,   /* U+113FF */
};

uint16 uca900_p114[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2B3A,   /* U+11400 */
  0x2B3B,   /* U+11401 */
  0x2B3C,   /* U+11402 */
  0x2B3D,   /* U+11403 */
  0x2B3E,   /* U+11404 */
  0x2B3F,   /* U+11405 */
  0x2B40,   /* U+11406 */
  0x2B41,   /* U+11407 */
  0x2B42,   /* U+11408 */
  0x2B43,   /* U+11409 */
  0x2B44,   /* U+1140A */
  0x2B45,   /* U+1140B */
  0x2B46,   /* U+1140C */
  0x2B47,   /* U+1140D */
  0x2B48,   /* U+1140E */
  0x2B49,   /* U+1140F */
  0x2B4A,   /* U+11410 */
  0x2B4B,   /* U+11411 */
  0x2B4C,   /* U+11412 */
  0x2B4D,   /* U+11413 */
  0x2B4E,   /* U+11414 */
  0x2B4F,   /* U+11415 */
  0x2B50,   /* U+11416 */
  0x2B51,   /* U+11417 */
  0x2B52,   /* U+11418 */
  0x2B53,   /* U+11419 */
  0x2B54,   /* U+1141A */
  0x2B55,   /* U+1141B */
  0x2B56,   /* U+1141C */
  0x2B57,   /* U+1141D */
  0x2B58,   /* U+1141E */
  0x2B59,   /* U+1141F */
  0x2B5A,   /* U+11420 */
  0x2B5B,   /* U+11421 */
  0x2B5C,   /* U+11422 */
  0x2B5D,   /* U+11423 */
  0x2B5E,   /* U+11424 */
  0x2B5F,   /* U+11425 */
  0x2B60,   /* U+11426 */
  0x2B61,   /* U+11427 */
  0x2B62,   /* U+11428 */
  0x2B63,   /* U+11429 */
  0x2B64,   /* U+1142A */
  0x2B65,   /* U+1142B */
  0x2B66,   /* U+1142C */
  0x2B67,   /* U+1142D */
  0x2B68,   /* U+1142E */
  0x2B69,   /* U+1142F */
  0x2B6A,   /* U+11430 */
  0x2B6B,   /* U+11431 */
  0x2B6C,   /* U+11432 */
  0x2B6D,   /* U+11433 */
  0x2B6E,   /* U+11434 */
  0x2B71,   /* U+11435 */
  0x2B72,   /* U+11436 */
  0x2B73,   /* U+11437 */
  0x2B74,   /* U+11438 */
  0x2B75,   /* U+11439 */
  0x2B76,   /* U+1143A */
  0x2B77,   /* U+1143B */
  0x2B78,   /* U+1143C */
  0x2B79,   /* U+1143D */
  0x2B7A,   /* U+1143E */
  0x2B7B,   /* U+1143F */
  0x2B7C,   /* U+11440 */
  0x2B7D,   /* U+11441 */
  0x2B7E,   /* U+11442 */
  0x0000,   /* U+11443 */
  0x0000,   /* U+11444 */
  0x0000,   /* U+11445 */
  0x0000,   /* U+11446 */
  0x2B6F,   /* U+11447 */
  0x2B70,   /* U+11448 */
  0x2B38,   /* U+11449 */
  0x2B39,   /* U+1144A */
  0x02B6,   /* U+1144B */
  0x02B7,   /* U+1144C */
  0x045B,   /* U+1144D */
  0x045C,   /* U+1144E */
  0x045D,   /* U+1144F */
  0x1C3D,   /* U+11450 */
  0x1C3E,   /* U+11451 */
  0x1C3F,   /* U+11452 */
  0x1C40,   /* U+11453 */
  0x1C41,   /* U+11454 */
  0x1C42,   /* U+11455 */
  0x1C43,   /* U+11456 */
  0x1C44,   /* U+11457 */
  0x1C45,   /* U+11458 */
  0x1C46,   /* U+11459 */
  0xFBC2,   /* U+1145A */
  0x045E,   /* U+1145B */
  0xFBC2,   /* U+1145C */
  0x045F,   /* U+1145D */
  0xFBC2,   /* U+1145E */
  0xFBC2,   /* U+1145F */
  0xFBC2,   /* U+11460 */
  0xFBC2,   /* U+11461 */
  0xFBC2,   /* U+11462 */
  0xFBC2,   /* U+11463 */
  0xFBC2,   /* U+11464 */
  0xFBC2,   /* U+11465 */
  0xFBC2,   /* U+11466 */
  0xFBC2,   /* U+11467 */
  0xFBC2,   /* U+11468 */
  0xFBC2,   /* U+11469 */
  0xFBC2,   /* U+1146A */
  0xFBC2,   /* U+1146B */
  0xFBC2,   /* U+1146C */
  0xFBC2,   /* U+1146D */
  0xFBC2,   /* U+1146E */
  0xFBC2,   /* U+1146F */
  0xFBC2,   /* U+11470 */
  0xFBC2,   /* U+11471 */
  0xFBC2,   /* U+11472 */
  0xFBC2,   /* U+11473 */
  0xFBC2,   /* U+11474 */
  0xFBC2,   /* U+11475 */
  0xFBC2,   /* U+11476 */
  0xFBC2,   /* U+11477 */
  0xFBC2,   /* U+11478 */
  0xFBC2,   /* U+11479 */
  0xFBC2,   /* U+1147A */
  0xFBC2,   /* U+1147B */
  0xFBC2,   /* U+1147C */
  0xFBC2,   /* U+1147D */
  0xFBC2,   /* U+1147E */
  0xFBC2,   /* U+1147F */
  0x2B80,   /* U+11480 */
  0x2B81,   /* U+11481 */
  0x2B82,   /* U+11482 */
  0x2B83,   /* U+11483 */
  0x2B84,   /* U+11484 */
  0x2B85,   /* U+11485 */
  0x2B86,   /* U+11486 */
  0x2B87,   /* U+11487 */
  0x2B88,   /* U+11488 */
  0x2B89,   /* U+11489 */
  0x2B8A,   /* U+1148A */
  0x2B8B,   /* U+1148B */
  0x2B8C,   /* U+1148C */
  0x2B8D,   /* U+1148D */
  0x2B8E,   /* U+1148E */
  0x2B8F,   /* U+1148F */
  0x2B90,   /* U+11490 */
  0x2B91,   /* U+11491 */
  0x2B92,   /* U+11492 */
  0x2B93,   /* U+11493 */
  0x2B94,   /* U+11494 */
  0x2B95,   /* U+11495 */
  0x2B96,   /* U+11496 */
  0x2B97,   /* U+11497 */
  0x2B98,   /* U+11498 */
  0x2B99,   /* U+11499 */
  0x2B9A,   /* U+1149A */
  0x2B9B,   /* U+1149B */
  0x2B9C,   /* U+1149C */
  0x2B9D,   /* U+1149D */
  0x2B9E,   /* U+1149E */
  0x2B9F,   /* U+1149F */
  0x2BA0,   /* U+114A0 */
  0x2BA1,   /* U+114A1 */
  0x2BA2,   /* U+114A2 */
  0x2BA3,   /* U+114A3 */
  0x2BA4,   /* U+114A4 */
  0x2BA5,   /* U+114A5 */
  0x2BA6,   /* U+114A6 */
  0x2BA7,   /* U+114A7 */
  0x2BA8,   /* U+114A8 */
  0x2BA9,   /* U+114A9 */
  0x2BAA,   /* U+114AA */
  0x2BAB,   /* U+114AB */
  0x2BAC,   /* U+114AC */
  0x2BAD,   /* U+114AD */
  0x2BAE,   /* U+114AE */
  0x2BAF,   /* U+114AF */
  0x2BB2,   /* U+114B0 */
  0x2BB3,   /* U+114B1 */
  0x2BB4,   /* U+114B2 */
  0x2BB5,   /* U+114B3 */
  0x2BB6,   /* U+114B4 */
  0x2BB7,   /* U+114B5 */
  0x2BB8,   /* U+114B6 */
  0x2BB9,   /* U+114B7 */
  0x2BBA,   /* U+114B8 */
  0x2BBB,   /* U+114B9 */
  0x2BBC,   /* U+114BA */
  0x2BBD,   /* U+114BB */
  0x2BBE,   /* U+114BC */
  0x2BBF,   /* U+114BD */
  0x2BC0,   /* U+114BE */
  0x0000,   /* U+114BF */
  0x0000,   /* U+114C0 */
  0x0000,   /* U+114C1 */
  0x2BC1,   /* U+114C2 */
  0x0000,   /* U+114C3 */
  0x2BB0,   /* U+114C4 */
  0x2BB1,   /* U+114C5 */
  0x0460,   /* U+114C6 */
  0x2B7F,   /* U+114C7 */
  0xFBC2,   /* U+114C8 */
  0xFBC2,   /* U+114C9 */
  0xFBC2,   /* U+114CA */
  0xFBC2,   /* U+114CB */
  0xFBC2,   /* U+114CC */
  0xFBC2,   /* U+114CD */
  0xFBC2,   /* U+114CE */
  0xFBC2,   /* U+114CF */
  0x1C3D,   /* U+114D0 */
  0x1C3E,   /* U+114D1 */
  0x1C3F,   /* U+114D2 */
  0x1C40,   /* U+114D3 */
  0x1C41,   /* U+114D4 */
  0x1C42,   /* U+114D5 */
  0x1C43,   /* U+114D6 */
  0x1C44,   /* U+114D7 */
  0x1C45,   /* U+114D8 */
  0x1C46,   /* U+114D9 */
  0xFBC2,   /* U+114DA */
  0xFBC2,   /* U+114DB */
  0xFBC2,   /* U+114DC */
  0xFBC2,   /* U+114DD */
  0xFBC2,   /* U+114DE */
  0xFBC2,   /* U+114DF */
  0xFBC2,   /* U+114E0 */
  0xFBC2,   /* U+114E1 */
  0xFBC2,   /* U+114E2 */
  0xFBC2,   /* U+114E3 */
  0xFBC2,   /* U+114E4 */
  0xFBC2,   /* U+114E5 */
  0xFBC2,   /* U+114E6 */
  0xFBC2,   /* U+114E7 */
  0xFBC2,   /* U+114E8 */
  0xFBC2,   /* U+114E9 */
  0xFBC2,   /* U+114EA */
  0xFBC2,   /* U+114EB */
  0xFBC2,   /* U+114EC */
  0xFBC2,   /* U+114ED */
  0xFBC2,   /* U+114EE */
  0xFBC2,   /* U+114EF */
  0xFBC2,   /* U+114F0 */
  0xFBC2,   /* U+114F1 */
  0xFBC2,   /* U+114F2 */
  0xFBC2,   /* U+114F3 */
  0xFBC2,   /* U+114F4 */
  0xFBC2,   /* U+114F5 */
  0xFBC2,   /* U+114F6 */
  0xFBC2,   /* U+114F7 */
  0xFBC2,   /* U+114F8 */
  0xFBC2,   /* U+114F9 */
  0xFBC2,   /* U+114FA */
  0xFBC2,   /* U+114FB */
  0xFBC2,   /* U+114FC */
  0xFBC2,   /* U+114FD */
  0xFBC2,   /* U+114FE */
  0xFBC2,   /* U+114FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11400 */
  0x0020,   /* U+11401 */
  0x0020,   /* U+11402 */
  0x0020,   /* U+11403 */
  0x0020,   /* U+11404 */
  0x0020,   /* U+11405 */
  0x0020,   /* U+11406 */
  0x0020,   /* U+11407 */
  0x0020,   /* U+11408 */
  0x0020,   /* U+11409 */
  0x0020,   /* U+1140A */
  0x0020,   /* U+1140B */
  0x0020,   /* U+1140C */
  0x0020,   /* U+1140D */
  0x0020,   /* U+1140E */
  0x0020,   /* U+1140F */
  0x0020,   /* U+11410 */
  0x0020,   /* U+11411 */
  0x0020,   /* U+11412 */
  0x0020,   /* U+11413 */
  0x0020,   /* U+11414 */
  0x0020,   /* U+11415 */
  0x0020,   /* U+11416 */
  0x0020,   /* U+11417 */
  0x0020,   /* U+11418 */
  0x0020,   /* U+11419 */
  0x0020,   /* U+1141A */
  0x0020,   /* U+1141B */
  0x0020,   /* U+1141C */
  0x0020,   /* U+1141D */
  0x0020,   /* U+1141E */
  0x0020,   /* U+1141F */
  0x0020,   /* U+11420 */
  0x0020,   /* U+11421 */
  0x0020,   /* U+11422 */
  0x0020,   /* U+11423 */
  0x0020,   /* U+11424 */
  0x0020,   /* U+11425 */
  0x0020,   /* U+11426 */
  0x0020,   /* U+11427 */
  0x0020,   /* U+11428 */
  0x0020,   /* U+11429 */
  0x0020,   /* U+1142A */
  0x0020,   /* U+1142B */
  0x0020,   /* U+1142C */
  0x0020,   /* U+1142D */
  0x0020,   /* U+1142E */
  0x0020,   /* U+1142F */
  0x0020,   /* U+11430 */
  0x0020,   /* U+11431 */
  0x0020,   /* U+11432 */
  0x0020,   /* U+11433 */
  0x0020,   /* U+11434 */
  0x0020,   /* U+11435 */
  0x0020,   /* U+11436 */
  0x0020,   /* U+11437 */
  0x0020,   /* U+11438 */
  0x0020,   /* U+11439 */
  0x0020,   /* U+1143A */
  0x0020,   /* U+1143B */
  0x0020,   /* U+1143C */
  0x0020,   /* U+1143D */
  0x0020,   /* U+1143E */
  0x0020,   /* U+1143F */
  0x0020,   /* U+11440 */
  0x0020,   /* U+11441 */
  0x0020,   /* U+11442 */
  0x00C3,   /* U+11443 */
  0x00C4,   /* U+11444 */
  0x00C5,   /* U+11445 */
  0x00C2,   /* U+11446 */
  0x0020,   /* U+11447 */
  0x0020,   /* U+11448 */
  0x0020,   /* U+11449 */
  0x0020,   /* U+1144A */
  0x0020,   /* U+1144B */
  0x0020,   /* U+1144C */
  0x0020,   /* U+1144D */
  0x0020,   /* U+1144E */
  0x0020,   /* U+1144F */
  0x0020,   /* U+11450 */
  0x0020,   /* U+11451 */
  0x0020,   /* U+11452 */
  0x0020,   /* U+11453 */
  0x0020,   /* U+11454 */
  0x0020,   /* U+11455 */
  0x0020,   /* U+11456 */
  0x0020,   /* U+11457 */
  0x0020,   /* U+11458 */
  0x0020,   /* U+11459 */
  0x0020,   /* U+1145A */
  0x0020,   /* U+1145B */
  0x0020,   /* U+1145C */
  0x0020,   /* U+1145D */
  0x0020,   /* U+1145E */
  0x0020,   /* U+1145F */
  0x0020,   /* U+11460 */
  0x0020,   /* U+11461 */
  0x0020,   /* U+11462 */
  0x0020,   /* U+11463 */
  0x0020,   /* U+11464 */
  0x0020,   /* U+11465 */
  0x0020,   /* U+11466 */
  0x0020,   /* U+11467 */
  0x0020,   /* U+11468 */
  0x0020,   /* U+11469 */
  0x0020,   /* U+1146A */
  0x0020,   /* U+1146B */
  0x0020,   /* U+1146C */
  0x0020,   /* U+1146D */
  0x0020,   /* U+1146E */
  0x0020,   /* U+1146F */
  0x0020,   /* U+11470 */
  0x0020,   /* U+11471 */
  0x0020,   /* U+11472 */
  0x0020,   /* U+11473 */
  0x0020,   /* U+11474 */
  0x0020,   /* U+11475 */
  0x0020,   /* U+11476 */
  0x0020,   /* U+11477 */
  0x0020,   /* U+11478 */
  0x0020,   /* U+11479 */
  0x0020,   /* U+1147A */
  0x0020,   /* U+1147B */
  0x0020,   /* U+1147C */
  0x0020,   /* U+1147D */
  0x0020,   /* U+1147E */
  0x0020,   /* U+1147F */
  0x0020,   /* U+11480 */
  0x0020,   /* U+11481 */
  0x0020,   /* U+11482 */
  0x0020,   /* U+11483 */
  0x0020,   /* U+11484 */
  0x0020,   /* U+11485 */
  0x0020,   /* U+11486 */
  0x0020,   /* U+11487 */
  0x0020,   /* U+11488 */
  0x0020,   /* U+11489 */
  0x0020,   /* U+1148A */
  0x0020,   /* U+1148B */
  0x0020,   /* U+1148C */
  0x0020,   /* U+1148D */
  0x0020,   /* U+1148E */
  0x0020,   /* U+1148F */
  0x0020,   /* U+11490 */
  0x0020,   /* U+11491 */
  0x0020,   /* U+11492 */
  0x0020,   /* U+11493 */
  0x0020,   /* U+11494 */
  0x0020,   /* U+11495 */
  0x0020,   /* U+11496 */
  0x0020,   /* U+11497 */
  0x0020,   /* U+11498 */
  0x0020,   /* U+11499 */
  0x0020,   /* U+1149A */
  0x0020,   /* U+1149B */
  0x0020,   /* U+1149C */
  0x0020,   /* U+1149D */
  0x0020,   /* U+1149E */
  0x0020,   /* U+1149F */
  0x0020,   /* U+114A0 */
  0x0020,   /* U+114A1 */
  0x0020,   /* U+114A2 */
  0x0020,   /* U+114A3 */
  0x0020,   /* U+114A4 */
  0x0020,   /* U+114A5 */
  0x0020,   /* U+114A6 */
  0x0020,   /* U+114A7 */
  0x0020,   /* U+114A8 */
  0x0020,   /* U+114A9 */
  0x0020,   /* U+114AA */
  0x0020,   /* U+114AB */
  0x0020,   /* U+114AC */
  0x0020,   /* U+114AD */
  0x0020,   /* U+114AE */
  0x0020,   /* U+114AF */
  0x0020,   /* U+114B0 */
  0x0020,   /* U+114B1 */
  0x0020,   /* U+114B2 */
  0x0020,   /* U+114B3 */
  0x0020,   /* U+114B4 */
  0x0020,   /* U+114B5 */
  0x0020,   /* U+114B6 */
  0x0020,   /* U+114B7 */
  0x0020,   /* U+114B8 */
  0x0020,   /* U+114B9 */
  0x0020,   /* U+114BA */
  0x0020,   /* U+114BB */
  0x0020,   /* U+114BC */
  0x0020,   /* U+114BD */
  0x0020,   /* U+114BE */
  0x00C3,   /* U+114BF */
  0x00C4,   /* U+114C0 */
  0x00C5,   /* U+114C1 */
  0x0020,   /* U+114C2 */
  0x00C2,   /* U+114C3 */
  0x0020,   /* U+114C4 */
  0x0020,   /* U+114C5 */
  0x0020,   /* U+114C6 */
  0x0020,   /* U+114C7 */
  0x0020,   /* U+114C8 */
  0x0020,   /* U+114C9 */
  0x0020,   /* U+114CA */
  0x0020,   /* U+114CB */
  0x0020,   /* U+114CC */
  0x0020,   /* U+114CD */
  0x0020,   /* U+114CE */
  0x0020,   /* U+114CF */
  0x0020,   /* U+114D0 */
  0x0020,   /* U+114D1 */
  0x0020,   /* U+114D2 */
  0x0020,   /* U+114D3 */
  0x0020,   /* U+114D4 */
  0x0020,   /* U+114D5 */
  0x0020,   /* U+114D6 */
  0x0020,   /* U+114D7 */
  0x0020,   /* U+114D8 */
  0x0020,   /* U+114D9 */
  0x0020,   /* U+114DA */
  0x0020,   /* U+114DB */
  0x0020,   /* U+114DC */
  0x0020,   /* U+114DD */
  0x0020,   /* U+114DE */
  0x0020,   /* U+114DF */
  0x0020,   /* U+114E0 */
  0x0020,   /* U+114E1 */
  0x0020,   /* U+114E2 */
  0x0020,   /* U+114E3 */
  0x0020,   /* U+114E4 */
  0x0020,   /* U+114E5 */
  0x0020,   /* U+114E6 */
  0x0020,   /* U+114E7 */
  0x0020,   /* U+114E8 */
  0x0020,   /* U+114E9 */
  0x0020,   /* U+114EA */
  0x0020,   /* U+114EB */
  0x0020,   /* U+114EC */
  0x0020,   /* U+114ED */
  0x0020,   /* U+114EE */
  0x0020,   /* U+114EF */
  0x0020,   /* U+114F0 */
  0x0020,   /* U+114F1 */
  0x0020,   /* U+114F2 */
  0x0020,   /* U+114F3 */
  0x0020,   /* U+114F4 */
  0x0020,   /* U+114F5 */
  0x0020,   /* U+114F6 */
  0x0020,   /* U+114F7 */
  0x0020,   /* U+114F8 */
  0x0020,   /* U+114F9 */
  0x0020,   /* U+114FA */
  0x0020,   /* U+114FB */
  0x0020,   /* U+114FC */
  0x0020,   /* U+114FD */
  0x0020,   /* U+114FE */
  0x0020,   /* U+114FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11400 */
  0x0002,   /* U+11401 */
  0x0002,   /* U+11402 */
  0x0002,   /* U+11403 */
  0x0002,   /* U+11404 */
  0x0002,   /* U+11405 */
  0x0002,   /* U+11406 */
  0x0002,   /* U+11407 */
  0x0002,   /* U+11408 */
  0x0002,   /* U+11409 */
  0x0002,   /* U+1140A */
  0x0002,   /* U+1140B */
  0x0002,   /* U+1140C */
  0x0002,   /* U+1140D */
  0x0002,   /* U+1140E */
  0x0002,   /* U+1140F */
  0x0002,   /* U+11410 */
  0x0002,   /* U+11411 */
  0x0002,   /* U+11412 */
  0x0002,   /* U+11413 */
  0x0002,   /* U+11414 */
  0x0002,   /* U+11415 */
  0x0002,   /* U+11416 */
  0x0002,   /* U+11417 */
  0x0002,   /* U+11418 */
  0x0002,   /* U+11419 */
  0x0002,   /* U+1141A */
  0x0002,   /* U+1141B */
  0x0002,   /* U+1141C */
  0x0002,   /* U+1141D */
  0x0002,   /* U+1141E */
  0x0002,   /* U+1141F */
  0x0002,   /* U+11420 */
  0x0002,   /* U+11421 */
  0x0002,   /* U+11422 */
  0x0002,   /* U+11423 */
  0x0002,   /* U+11424 */
  0x0002,   /* U+11425 */
  0x0002,   /* U+11426 */
  0x0002,   /* U+11427 */
  0x0002,   /* U+11428 */
  0x0002,   /* U+11429 */
  0x0002,   /* U+1142A */
  0x0002,   /* U+1142B */
  0x0002,   /* U+1142C */
  0x0002,   /* U+1142D */
  0x0002,   /* U+1142E */
  0x0002,   /* U+1142F */
  0x0002,   /* U+11430 */
  0x0002,   /* U+11431 */
  0x0002,   /* U+11432 */
  0x0002,   /* U+11433 */
  0x0002,   /* U+11434 */
  0x0002,   /* U+11435 */
  0x0002,   /* U+11436 */
  0x0002,   /* U+11437 */
  0x0002,   /* U+11438 */
  0x0002,   /* U+11439 */
  0x0002,   /* U+1143A */
  0x0002,   /* U+1143B */
  0x0002,   /* U+1143C */
  0x0002,   /* U+1143D */
  0x0002,   /* U+1143E */
  0x0002,   /* U+1143F */
  0x0002,   /* U+11440 */
  0x0002,   /* U+11441 */
  0x0002,   /* U+11442 */
  0x0002,   /* U+11443 */
  0x0002,   /* U+11444 */
  0x0002,   /* U+11445 */
  0x0002,   /* U+11446 */
  0x0002,   /* U+11447 */
  0x0002,   /* U+11448 */
  0x0002,   /* U+11449 */
  0x0002,   /* U+1144A */
  0x0002,   /* U+1144B */
  0x0002,   /* U+1144C */
  0x0002,   /* U+1144D */
  0x0002,   /* U+1144E */
  0x0002,   /* U+1144F */
  0x0002,   /* U+11450 */
  0x0002,   /* U+11451 */
  0x0002,   /* U+11452 */
  0x0002,   /* U+11453 */
  0x0002,   /* U+11454 */
  0x0002,   /* U+11455 */
  0x0002,   /* U+11456 */
  0x0002,   /* U+11457 */
  0x0002,   /* U+11458 */
  0x0002,   /* U+11459 */
  0x0002,   /* U+1145A */
  0x0002,   /* U+1145B */
  0x0002,   /* U+1145C */
  0x0002,   /* U+1145D */
  0x0002,   /* U+1145E */
  0x0002,   /* U+1145F */
  0x0002,   /* U+11460 */
  0x0002,   /* U+11461 */
  0x0002,   /* U+11462 */
  0x0002,   /* U+11463 */
  0x0002,   /* U+11464 */
  0x0002,   /* U+11465 */
  0x0002,   /* U+11466 */
  0x0002,   /* U+11467 */
  0x0002,   /* U+11468 */
  0x0002,   /* U+11469 */
  0x0002,   /* U+1146A */
  0x0002,   /* U+1146B */
  0x0002,   /* U+1146C */
  0x0002,   /* U+1146D */
  0x0002,   /* U+1146E */
  0x0002,   /* U+1146F */
  0x0002,   /* U+11470 */
  0x0002,   /* U+11471 */
  0x0002,   /* U+11472 */
  0x0002,   /* U+11473 */
  0x0002,   /* U+11474 */
  0x0002,   /* U+11475 */
  0x0002,   /* U+11476 */
  0x0002,   /* U+11477 */
  0x0002,   /* U+11478 */
  0x0002,   /* U+11479 */
  0x0002,   /* U+1147A */
  0x0002,   /* U+1147B */
  0x0002,   /* U+1147C */
  0x0002,   /* U+1147D */
  0x0002,   /* U+1147E */
  0x0002,   /* U+1147F */
  0x0002,   /* U+11480 */
  0x0002,   /* U+11481 */
  0x0002,   /* U+11482 */
  0x0002,   /* U+11483 */
  0x0002,   /* U+11484 */
  0x0002,   /* U+11485 */
  0x0002,   /* U+11486 */
  0x0002,   /* U+11487 */
  0x0002,   /* U+11488 */
  0x0002,   /* U+11489 */
  0x0002,   /* U+1148A */
  0x0002,   /* U+1148B */
  0x0002,   /* U+1148C */
  0x0002,   /* U+1148D */
  0x0002,   /* U+1148E */
  0x0002,   /* U+1148F */
  0x0002,   /* U+11490 */
  0x0002,   /* U+11491 */
  0x0002,   /* U+11492 */
  0x0002,   /* U+11493 */
  0x0002,   /* U+11494 */
  0x0002,   /* U+11495 */
  0x0002,   /* U+11496 */
  0x0002,   /* U+11497 */
  0x0002,   /* U+11498 */
  0x0002,   /* U+11499 */
  0x0002,   /* U+1149A */
  0x0002,   /* U+1149B */
  0x0002,   /* U+1149C */
  0x0002,   /* U+1149D */
  0x0002,   /* U+1149E */
  0x0002,   /* U+1149F */
  0x0002,   /* U+114A0 */
  0x0002,   /* U+114A1 */
  0x0002,   /* U+114A2 */
  0x0002,   /* U+114A3 */
  0x0002,   /* U+114A4 */
  0x0002,   /* U+114A5 */
  0x0002,   /* U+114A6 */
  0x0002,   /* U+114A7 */
  0x0002,   /* U+114A8 */
  0x0002,   /* U+114A9 */
  0x0002,   /* U+114AA */
  0x0002,   /* U+114AB */
  0x0002,   /* U+114AC */
  0x0002,   /* U+114AD */
  0x0002,   /* U+114AE */
  0x0002,   /* U+114AF */
  0x0002,   /* U+114B0 */
  0x0002,   /* U+114B1 */
  0x0002,   /* U+114B2 */
  0x0002,   /* U+114B3 */
  0x0002,   /* U+114B4 */
  0x0002,   /* U+114B5 */
  0x0002,   /* U+114B6 */
  0x0002,   /* U+114B7 */
  0x0002,   /* U+114B8 */
  0x0002,   /* U+114B9 */
  0x0002,   /* U+114BA */
  0x0002,   /* U+114BB */
  0x0002,   /* U+114BC */
  0x0002,   /* U+114BD */
  0x0002,   /* U+114BE */
  0x0002,   /* U+114BF */
  0x0002,   /* U+114C0 */
  0x0002,   /* U+114C1 */
  0x0002,   /* U+114C2 */
  0x0002,   /* U+114C3 */
  0x0002,   /* U+114C4 */
  0x0002,   /* U+114C5 */
  0x0002,   /* U+114C6 */
  0x0002,   /* U+114C7 */
  0x0002,   /* U+114C8 */
  0x0002,   /* U+114C9 */
  0x0002,   /* U+114CA */
  0x0002,   /* U+114CB */
  0x0002,   /* U+114CC */
  0x0002,   /* U+114CD */
  0x0002,   /* U+114CE */
  0x0002,   /* U+114CF */
  0x0002,   /* U+114D0 */
  0x0002,   /* U+114D1 */
  0x0002,   /* U+114D2 */
  0x0002,   /* U+114D3 */
  0x0002,   /* U+114D4 */
  0x0002,   /* U+114D5 */
  0x0002,   /* U+114D6 */
  0x0002,   /* U+114D7 */
  0x0002,   /* U+114D8 */
  0x0002,   /* U+114D9 */
  0x0002,   /* U+114DA */
  0x0002,   /* U+114DB */
  0x0002,   /* U+114DC */
  0x0002,   /* U+114DD */
  0x0002,   /* U+114DE */
  0x0002,   /* U+114DF */
  0x0002,   /* U+114E0 */
  0x0002,   /* U+114E1 */
  0x0002,   /* U+114E2 */
  0x0002,   /* U+114E3 */
  0x0002,   /* U+114E4 */
  0x0002,   /* U+114E5 */
  0x0002,   /* U+114E6 */
  0x0002,   /* U+114E7 */
  0x0002,   /* U+114E8 */
  0x0002,   /* U+114E9 */
  0x0002,   /* U+114EA */
  0x0002,   /* U+114EB */
  0x0002,   /* U+114EC */
  0x0002,   /* U+114ED */
  0x0002,   /* U+114EE */
  0x0002,   /* U+114EF */
  0x0002,   /* U+114F0 */
  0x0002,   /* U+114F1 */
  0x0002,   /* U+114F2 */
  0x0002,   /* U+114F3 */
  0x0002,   /* U+114F4 */
  0x0002,   /* U+114F5 */
  0x0002,   /* U+114F6 */
  0x0002,   /* U+114F7 */
  0x0002,   /* U+114F8 */
  0x0002,   /* U+114F9 */
  0x0002,   /* U+114FA */
  0x0002,   /* U+114FB */
  0x0002,   /* U+114FC */
  0x0002,   /* U+114FD */
  0x0002,   /* U+114FE */
  0x0002,   /* U+114FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11400 */
  0x0000,   /* U+11401 */
  0x0000,   /* U+11402 */
  0x0000,   /* U+11403 */
  0x0000,   /* U+11404 */
  0x0000,   /* U+11405 */
  0x0000,   /* U+11406 */
  0x0000,   /* U+11407 */
  0x0000,   /* U+11408 */
  0x0000,   /* U+11409 */
  0x0000,   /* U+1140A */
  0x0000,   /* U+1140B */
  0x0000,   /* U+1140C */
  0x0000,   /* U+1140D */
  0x0000,   /* U+1140E */
  0x0000,   /* U+1140F */
  0x0000,   /* U+11410 */
  0x0000,   /* U+11411 */
  0x0000,   /* U+11412 */
  0x0000,   /* U+11413 */
  0x0000,   /* U+11414 */
  0x0000,   /* U+11415 */
  0x0000,   /* U+11416 */
  0x0000,   /* U+11417 */
  0x0000,   /* U+11418 */
  0x0000,   /* U+11419 */
  0x0000,   /* U+1141A */
  0x0000,   /* U+1141B */
  0x0000,   /* U+1141C */
  0x0000,   /* U+1141D */
  0x0000,   /* U+1141E */
  0x0000,   /* U+1141F */
  0x0000,   /* U+11420 */
  0x0000,   /* U+11421 */
  0x0000,   /* U+11422 */
  0x0000,   /* U+11423 */
  0x0000,   /* U+11424 */
  0x0000,   /* U+11425 */
  0x0000,   /* U+11426 */
  0x0000,   /* U+11427 */
  0x0000,   /* U+11428 */
  0x0000,   /* U+11429 */
  0x0000,   /* U+1142A */
  0x0000,   /* U+1142B */
  0x0000,   /* U+1142C */
  0x0000,   /* U+1142D */
  0x0000,   /* U+1142E */
  0x0000,   /* U+1142F */
  0x0000,   /* U+11430 */
  0x0000,   /* U+11431 */
  0x0000,   /* U+11432 */
  0x0000,   /* U+11433 */
  0x0000,   /* U+11434 */
  0x0000,   /* U+11435 */
  0x0000,   /* U+11436 */
  0x0000,   /* U+11437 */
  0x0000,   /* U+11438 */
  0x0000,   /* U+11439 */
  0x0000,   /* U+1143A */
  0x0000,   /* U+1143B */
  0x0000,   /* U+1143C */
  0x0000,   /* U+1143D */
  0x0000,   /* U+1143E */
  0x0000,   /* U+1143F */
  0x0000,   /* U+11440 */
  0x0000,   /* U+11441 */
  0x0000,   /* U+11442 */
  0x0000,   /* U+11443 */
  0x0000,   /* U+11444 */
  0x0000,   /* U+11445 */
  0x0000,   /* U+11446 */
  0x0000,   /* U+11447 */
  0x0000,   /* U+11448 */
  0x0000,   /* U+11449 */
  0x0000,   /* U+1144A */
  0x0000,   /* U+1144B */
  0x0000,   /* U+1144C */
  0x0000,   /* U+1144D */
  0x0000,   /* U+1144E */
  0x0000,   /* U+1144F */
  0x0000,   /* U+11450 */
  0x0000,   /* U+11451 */
  0x0000,   /* U+11452 */
  0x0000,   /* U+11453 */
  0x0000,   /* U+11454 */
  0x0000,   /* U+11455 */
  0x0000,   /* U+11456 */
  0x0000,   /* U+11457 */
  0x0000,   /* U+11458 */
  0x0000,   /* U+11459 */
  0x945A,   /* U+1145A */
  0x0000,   /* U+1145B */
  0x945C,   /* U+1145C */
  0x0000,   /* U+1145D */
  0x945E,   /* U+1145E */
  0x945F,   /* U+1145F */
  0x9460,   /* U+11460 */
  0x9461,   /* U+11461 */
  0x9462,   /* U+11462 */
  0x9463,   /* U+11463 */
  0x9464,   /* U+11464 */
  0x9465,   /* U+11465 */
  0x9466,   /* U+11466 */
  0x9467,   /* U+11467 */
  0x9468,   /* U+11468 */
  0x9469,   /* U+11469 */
  0x946A,   /* U+1146A */
  0x946B,   /* U+1146B */
  0x946C,   /* U+1146C */
  0x946D,   /* U+1146D */
  0x946E,   /* U+1146E */
  0x946F,   /* U+1146F */
  0x9470,   /* U+11470 */
  0x9471,   /* U+11471 */
  0x9472,   /* U+11472 */
  0x9473,   /* U+11473 */
  0x9474,   /* U+11474 */
  0x9475,   /* U+11475 */
  0x9476,   /* U+11476 */
  0x9477,   /* U+11477 */
  0x9478,   /* U+11478 */
  0x9479,   /* U+11479 */
  0x947A,   /* U+1147A */
  0x947B,   /* U+1147B */
  0x947C,   /* U+1147C */
  0x947D,   /* U+1147D */
  0x947E,   /* U+1147E */
  0x947F,   /* U+1147F */
  0x0000,   /* U+11480 */
  0x0000,   /* U+11481 */
  0x0000,   /* U+11482 */
  0x0000,   /* U+11483 */
  0x0000,   /* U+11484 */
  0x0000,   /* U+11485 */
  0x0000,   /* U+11486 */
  0x0000,   /* U+11487 */
  0x0000,   /* U+11488 */
  0x0000,   /* U+11489 */
  0x0000,   /* U+1148A */
  0x0000,   /* U+1148B */
  0x0000,   /* U+1148C */
  0x0000,   /* U+1148D */
  0x0000,   /* U+1148E */
  0x0000,   /* U+1148F */
  0x0000,   /* U+11490 */
  0x0000,   /* U+11491 */
  0x0000,   /* U+11492 */
  0x0000,   /* U+11493 */
  0x0000,   /* U+11494 */
  0x0000,   /* U+11495 */
  0x0000,   /* U+11496 */
  0x0000,   /* U+11497 */
  0x0000,   /* U+11498 */
  0x0000,   /* U+11499 */
  0x0000,   /* U+1149A */
  0x0000,   /* U+1149B */
  0x0000,   /* U+1149C */
  0x0000,   /* U+1149D */
  0x0000,   /* U+1149E */
  0x0000,   /* U+1149F */
  0x0000,   /* U+114A0 */
  0x0000,   /* U+114A1 */
  0x0000,   /* U+114A2 */
  0x0000,   /* U+114A3 */
  0x0000,   /* U+114A4 */
  0x0000,   /* U+114A5 */
  0x0000,   /* U+114A6 */
  0x0000,   /* U+114A7 */
  0x0000,   /* U+114A8 */
  0x0000,   /* U+114A9 */
  0x0000,   /* U+114AA */
  0x0000,   /* U+114AB */
  0x0000,   /* U+114AC */
  0x0000,   /* U+114AD */
  0x0000,   /* U+114AE */
  0x0000,   /* U+114AF */
  0x0000,   /* U+114B0 */
  0x0000,   /* U+114B1 */
  0x0000,   /* U+114B2 */
  0x0000,   /* U+114B3 */
  0x0000,   /* U+114B4 */
  0x0000,   /* U+114B5 */
  0x0000,   /* U+114B6 */
  0x0000,   /* U+114B7 */
  0x0000,   /* U+114B8 */
  0x0000,   /* U+114B9 */
  0x0000,   /* U+114BA */
  0x0000,   /* U+114BB */
  0x0000,   /* U+114BC */
  0x0000,   /* U+114BD */
  0x0000,   /* U+114BE */
  0x0000,   /* U+114BF */
  0x0000,   /* U+114C0 */
  0x0000,   /* U+114C1 */
  0x0000,   /* U+114C2 */
  0x0000,   /* U+114C3 */
  0x0000,   /* U+114C4 */
  0x0000,   /* U+114C5 */
  0x0000,   /* U+114C6 */
  0x0000,   /* U+114C7 */
  0x94C8,   /* U+114C8 */
  0x94C9,   /* U+114C9 */
  0x94CA,   /* U+114CA */
  0x94CB,   /* U+114CB */
  0x94CC,   /* U+114CC */
  0x94CD,   /* U+114CD */
  0x94CE,   /* U+114CE */
  0x94CF,   /* U+114CF */
  0x0000,   /* U+114D0 */
  0x0000,   /* U+114D1 */
  0x0000,   /* U+114D2 */
  0x0000,   /* U+114D3 */
  0x0000,   /* U+114D4 */
  0x0000,   /* U+114D5 */
  0x0000,   /* U+114D6 */
  0x0000,   /* U+114D7 */
  0x0000,   /* U+114D8 */
  0x0000,   /* U+114D9 */
  0x94DA,   /* U+114DA */
  0x94DB,   /* U+114DB */
  0x94DC,   /* U+114DC */
  0x94DD,   /* U+114DD */
  0x94DE,   /* U+114DE */
  0x94DF,   /* U+114DF */
  0x94E0,   /* U+114E0 */
  0x94E1,   /* U+114E1 */
  0x94E2,   /* U+114E2 */
  0x94E3,   /* U+114E3 */
  0x94E4,   /* U+114E4 */
  0x94E5,   /* U+114E5 */
  0x94E6,   /* U+114E6 */
  0x94E7,   /* U+114E7 */
  0x94E8,   /* U+114E8 */
  0x94E9,   /* U+114E9 */
  0x94EA,   /* U+114EA */
  0x94EB,   /* U+114EB */
  0x94EC,   /* U+114EC */
  0x94ED,   /* U+114ED */
  0x94EE,   /* U+114EE */
  0x94EF,   /* U+114EF */
  0x94F0,   /* U+114F0 */
  0x94F1,   /* U+114F1 */
  0x94F2,   /* U+114F2 */
  0x94F3,   /* U+114F3 */
  0x94F4,   /* U+114F4 */
  0x94F5,   /* U+114F5 */
  0x94F6,   /* U+114F6 */
  0x94F7,   /* U+114F7 */
  0x94F8,   /* U+114F8 */
  0x94F9,   /* U+114F9 */
  0x94FA,   /* U+114FA */
  0x94FB,   /* U+114FB */
  0x94FC,   /* U+114FC */
  0x94FD,   /* U+114FD */
  0x94FE,   /* U+114FE */
  0x94FF,   /* U+114FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11400 */
  0x0000,   /* U+11401 */
  0x0000,   /* U+11402 */
  0x0000,   /* U+11403 */
  0x0000,   /* U+11404 */
  0x0000,   /* U+11405 */
  0x0000,   /* U+11406 */
  0x0000,   /* U+11407 */
  0x0000,   /* U+11408 */
  0x0000,   /* U+11409 */
  0x0000,   /* U+1140A */
  0x0000,   /* U+1140B */
  0x0000,   /* U+1140C */
  0x0000,   /* U+1140D */
  0x0000,   /* U+1140E */
  0x0000,   /* U+1140F */
  0x0000,   /* U+11410 */
  0x0000,   /* U+11411 */
  0x0000,   /* U+11412 */
  0x0000,   /* U+11413 */
  0x0000,   /* U+11414 */
  0x0000,   /* U+11415 */
  0x0000,   /* U+11416 */
  0x0000,   /* U+11417 */
  0x0000,   /* U+11418 */
  0x0000,   /* U+11419 */
  0x0000,   /* U+1141A */
  0x0000,   /* U+1141B */
  0x0000,   /* U+1141C */
  0x0000,   /* U+1141D */
  0x0000,   /* U+1141E */
  0x0000,   /* U+1141F */
  0x0000,   /* U+11420 */
  0x0000,   /* U+11421 */
  0x0000,   /* U+11422 */
  0x0000,   /* U+11423 */
  0x0000,   /* U+11424 */
  0x0000,   /* U+11425 */
  0x0000,   /* U+11426 */
  0x0000,   /* U+11427 */
  0x0000,   /* U+11428 */
  0x0000,   /* U+11429 */
  0x0000,   /* U+1142A */
  0x0000,   /* U+1142B */
  0x0000,   /* U+1142C */
  0x0000,   /* U+1142D */
  0x0000,   /* U+1142E */
  0x0000,   /* U+1142F */
  0x0000,   /* U+11430 */
  0x0000,   /* U+11431 */
  0x0000,   /* U+11432 */
  0x0000,   /* U+11433 */
  0x0000,   /* U+11434 */
  0x0000,   /* U+11435 */
  0x0000,   /* U+11436 */
  0x0000,   /* U+11437 */
  0x0000,   /* U+11438 */
  0x0000,   /* U+11439 */
  0x0000,   /* U+1143A */
  0x0000,   /* U+1143B */
  0x0000,   /* U+1143C */
  0x0000,   /* U+1143D */
  0x0000,   /* U+1143E */
  0x0000,   /* U+1143F */
  0x0000,   /* U+11440 */
  0x0000,   /* U+11441 */
  0x0000,   /* U+11442 */
  0x0000,   /* U+11443 */
  0x0000,   /* U+11444 */
  0x0000,   /* U+11445 */
  0x0000,   /* U+11446 */
  0x0000,   /* U+11447 */
  0x0000,   /* U+11448 */
  0x0000,   /* U+11449 */
  0x0000,   /* U+1144A */
  0x0000,   /* U+1144B */
  0x0000,   /* U+1144C */
  0x0000,   /* U+1144D */
  0x0000,   /* U+1144E */
  0x0000,   /* U+1144F */
  0x0000,   /* U+11450 */
  0x0000,   /* U+11451 */
  0x0000,   /* U+11452 */
  0x0000,   /* U+11453 */
  0x0000,   /* U+11454 */
  0x0000,   /* U+11455 */
  0x0000,   /* U+11456 */
  0x0000,   /* U+11457 */
  0x0000,   /* U+11458 */
  0x0000,   /* U+11459 */
  0x0000,   /* U+1145A */
  0x0000,   /* U+1145B */
  0x0000,   /* U+1145C */
  0x0000,   /* U+1145D */
  0x0000,   /* U+1145E */
  0x0000,   /* U+1145F */
  0x0000,   /* U+11460 */
  0x0000,   /* U+11461 */
  0x0000,   /* U+11462 */
  0x0000,   /* U+11463 */
  0x0000,   /* U+11464 */
  0x0000,   /* U+11465 */
  0x0000,   /* U+11466 */
  0x0000,   /* U+11467 */
  0x0000,   /* U+11468 */
  0x0000,   /* U+11469 */
  0x0000,   /* U+1146A */
  0x0000,   /* U+1146B */
  0x0000,   /* U+1146C */
  0x0000,   /* U+1146D */
  0x0000,   /* U+1146E */
  0x0000,   /* U+1146F */
  0x0000,   /* U+11470 */
  0x0000,   /* U+11471 */
  0x0000,   /* U+11472 */
  0x0000,   /* U+11473 */
  0x0000,   /* U+11474 */
  0x0000,   /* U+11475 */
  0x0000,   /* U+11476 */
  0x0000,   /* U+11477 */
  0x0000,   /* U+11478 */
  0x0000,   /* U+11479 */
  0x0000,   /* U+1147A */
  0x0000,   /* U+1147B */
  0x0000,   /* U+1147C */
  0x0000,   /* U+1147D */
  0x0000,   /* U+1147E */
  0x0000,   /* U+1147F */
  0x0000,   /* U+11480 */
  0x0000,   /* U+11481 */
  0x0000,   /* U+11482 */
  0x0000,   /* U+11483 */
  0x0000,   /* U+11484 */
  0x0000,   /* U+11485 */
  0x0000,   /* U+11486 */
  0x0000,   /* U+11487 */
  0x0000,   /* U+11488 */
  0x0000,   /* U+11489 */
  0x0000,   /* U+1148A */
  0x0000,   /* U+1148B */
  0x0000,   /* U+1148C */
  0x0000,   /* U+1148D */
  0x0000,   /* U+1148E */
  0x0000,   /* U+1148F */
  0x0000,   /* U+11490 */
  0x0000,   /* U+11491 */
  0x0000,   /* U+11492 */
  0x0000,   /* U+11493 */
  0x0000,   /* U+11494 */
  0x0000,   /* U+11495 */
  0x0000,   /* U+11496 */
  0x0000,   /* U+11497 */
  0x0000,   /* U+11498 */
  0x0000,   /* U+11499 */
  0x0000,   /* U+1149A */
  0x0000,   /* U+1149B */
  0x0000,   /* U+1149C */
  0x0000,   /* U+1149D */
  0x0000,   /* U+1149E */
  0x0000,   /* U+1149F */
  0x0000,   /* U+114A0 */
  0x0000,   /* U+114A1 */
  0x0000,   /* U+114A2 */
  0x0000,   /* U+114A3 */
  0x0000,   /* U+114A4 */
  0x0000,   /* U+114A5 */
  0x0000,   /* U+114A6 */
  0x0000,   /* U+114A7 */
  0x0000,   /* U+114A8 */
  0x0000,   /* U+114A9 */
  0x0000,   /* U+114AA */
  0x0000,   /* U+114AB */
  0x0000,   /* U+114AC */
  0x0000,   /* U+114AD */
  0x0000,   /* U+114AE */
  0x0000,   /* U+114AF */
  0x0000,   /* U+114B0 */
  0x0000,   /* U+114B1 */
  0x0000,   /* U+114B2 */
  0x0000,   /* U+114B3 */
  0x0000,   /* U+114B4 */
  0x0000,   /* U+114B5 */
  0x0000,   /* U+114B6 */
  0x0000,   /* U+114B7 */
  0x0000,   /* U+114B8 */
  0x0000,   /* U+114B9 */
  0x0000,   /* U+114BA */
  0x0000,   /* U+114BB */
  0x0000,   /* U+114BC */
  0x0000,   /* U+114BD */
  0x0000,   /* U+114BE */
  0x0000,   /* U+114BF */
  0x0000,   /* U+114C0 */
  0x0000,   /* U+114C1 */
  0x0000,   /* U+114C2 */
  0x0000,   /* U+114C3 */
  0x0000,   /* U+114C4 */
  0x0000,   /* U+114C5 */
  0x0000,   /* U+114C6 */
  0x0000,   /* U+114C7 */
  0x0000,   /* U+114C8 */
  0x0000,   /* U+114C9 */
  0x0000,   /* U+114CA */
  0x0000,   /* U+114CB */
  0x0000,   /* U+114CC */
  0x0000,   /* U+114CD */
  0x0000,   /* U+114CE */
  0x0000,   /* U+114CF */
  0x0000,   /* U+114D0 */
  0x0000,   /* U+114D1 */
  0x0000,   /* U+114D2 */
  0x0000,   /* U+114D3 */
  0x0000,   /* U+114D4 */
  0x0000,   /* U+114D5 */
  0x0000,   /* U+114D6 */
  0x0000,   /* U+114D7 */
  0x0000,   /* U+114D8 */
  0x0000,   /* U+114D9 */
  0x0000,   /* U+114DA */
  0x0000,   /* U+114DB */
  0x0000,   /* U+114DC */
  0x0000,   /* U+114DD */
  0x0000,   /* U+114DE */
  0x0000,   /* U+114DF */
  0x0000,   /* U+114E0 */
  0x0000,   /* U+114E1 */
  0x0000,   /* U+114E2 */
  0x0000,   /* U+114E3 */
  0x0000,   /* U+114E4 */
  0x0000,   /* U+114E5 */
  0x0000,   /* U+114E6 */
  0x0000,   /* U+114E7 */
  0x0000,   /* U+114E8 */
  0x0000,   /* U+114E9 */
  0x0000,   /* U+114EA */
  0x0000,   /* U+114EB */
  0x0000,   /* U+114EC */
  0x0000,   /* U+114ED */
  0x0000,   /* U+114EE */
  0x0000,   /* U+114EF */
  0x0000,   /* U+114F0 */
  0x0000,   /* U+114F1 */
  0x0000,   /* U+114F2 */
  0x0000,   /* U+114F3 */
  0x0000,   /* U+114F4 */
  0x0000,   /* U+114F5 */
  0x0000,   /* U+114F6 */
  0x0000,   /* U+114F7 */
  0x0000,   /* U+114F8 */
  0x0000,   /* U+114F9 */
  0x0000,   /* U+114FA */
  0x0000,   /* U+114FB */
  0x0000,   /* U+114FC */
  0x0000,   /* U+114FD */
  0x0000,   /* U+114FE */
  0x0000,   /* U+114FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11400 */
  0x0000,   /* U+11401 */
  0x0000,   /* U+11402 */
  0x0000,   /* U+11403 */
  0x0000,   /* U+11404 */
  0x0000,   /* U+11405 */
  0x0000,   /* U+11406 */
  0x0000,   /* U+11407 */
  0x0000,   /* U+11408 */
  0x0000,   /* U+11409 */
  0x0000,   /* U+1140A */
  0x0000,   /* U+1140B */
  0x0000,   /* U+1140C */
  0x0000,   /* U+1140D */
  0x0000,   /* U+1140E */
  0x0000,   /* U+1140F */
  0x0000,   /* U+11410 */
  0x0000,   /* U+11411 */
  0x0000,   /* U+11412 */
  0x0000,   /* U+11413 */
  0x0000,   /* U+11414 */
  0x0000,   /* U+11415 */
  0x0000,   /* U+11416 */
  0x0000,   /* U+11417 */
  0x0000,   /* U+11418 */
  0x0000,   /* U+11419 */
  0x0000,   /* U+1141A */
  0x0000,   /* U+1141B */
  0x0000,   /* U+1141C */
  0x0000,   /* U+1141D */
  0x0000,   /* U+1141E */
  0x0000,   /* U+1141F */
  0x0000,   /* U+11420 */
  0x0000,   /* U+11421 */
  0x0000,   /* U+11422 */
  0x0000,   /* U+11423 */
  0x0000,   /* U+11424 */
  0x0000,   /* U+11425 */
  0x0000,   /* U+11426 */
  0x0000,   /* U+11427 */
  0x0000,   /* U+11428 */
  0x0000,   /* U+11429 */
  0x0000,   /* U+1142A */
  0x0000,   /* U+1142B */
  0x0000,   /* U+1142C */
  0x0000,   /* U+1142D */
  0x0000,   /* U+1142E */
  0x0000,   /* U+1142F */
  0x0000,   /* U+11430 */
  0x0000,   /* U+11431 */
  0x0000,   /* U+11432 */
  0x0000,   /* U+11433 */
  0x0000,   /* U+11434 */
  0x0000,   /* U+11435 */
  0x0000,   /* U+11436 */
  0x0000,   /* U+11437 */
  0x0000,   /* U+11438 */
  0x0000,   /* U+11439 */
  0x0000,   /* U+1143A */
  0x0000,   /* U+1143B */
  0x0000,   /* U+1143C */
  0x0000,   /* U+1143D */
  0x0000,   /* U+1143E */
  0x0000,   /* U+1143F */
  0x0000,   /* U+11440 */
  0x0000,   /* U+11441 */
  0x0000,   /* U+11442 */
  0x0000,   /* U+11443 */
  0x0000,   /* U+11444 */
  0x0000,   /* U+11445 */
  0x0000,   /* U+11446 */
  0x0000,   /* U+11447 */
  0x0000,   /* U+11448 */
  0x0000,   /* U+11449 */
  0x0000,   /* U+1144A */
  0x0000,   /* U+1144B */
  0x0000,   /* U+1144C */
  0x0000,   /* U+1144D */
  0x0000,   /* U+1144E */
  0x0000,   /* U+1144F */
  0x0000,   /* U+11450 */
  0x0000,   /* U+11451 */
  0x0000,   /* U+11452 */
  0x0000,   /* U+11453 */
  0x0000,   /* U+11454 */
  0x0000,   /* U+11455 */
  0x0000,   /* U+11456 */
  0x0000,   /* U+11457 */
  0x0000,   /* U+11458 */
  0x0000,   /* U+11459 */
  0x0000,   /* U+1145A */
  0x0000,   /* U+1145B */
  0x0000,   /* U+1145C */
  0x0000,   /* U+1145D */
  0x0000,   /* U+1145E */
  0x0000,   /* U+1145F */
  0x0000,   /* U+11460 */
  0x0000,   /* U+11461 */
  0x0000,   /* U+11462 */
  0x0000,   /* U+11463 */
  0x0000,   /* U+11464 */
  0x0000,   /* U+11465 */
  0x0000,   /* U+11466 */
  0x0000,   /* U+11467 */
  0x0000,   /* U+11468 */
  0x0000,   /* U+11469 */
  0x0000,   /* U+1146A */
  0x0000,   /* U+1146B */
  0x0000,   /* U+1146C */
  0x0000,   /* U+1146D */
  0x0000,   /* U+1146E */
  0x0000,   /* U+1146F */
  0x0000,   /* U+11470 */
  0x0000,   /* U+11471 */
  0x0000,   /* U+11472 */
  0x0000,   /* U+11473 */
  0x0000,   /* U+11474 */
  0x0000,   /* U+11475 */
  0x0000,   /* U+11476 */
  0x0000,   /* U+11477 */
  0x0000,   /* U+11478 */
  0x0000,   /* U+11479 */
  0x0000,   /* U+1147A */
  0x0000,   /* U+1147B */
  0x0000,   /* U+1147C */
  0x0000,   /* U+1147D */
  0x0000,   /* U+1147E */
  0x0000,   /* U+1147F */
  0x0000,   /* U+11480 */
  0x0000,   /* U+11481 */
  0x0000,   /* U+11482 */
  0x0000,   /* U+11483 */
  0x0000,   /* U+11484 */
  0x0000,   /* U+11485 */
  0x0000,   /* U+11486 */
  0x0000,   /* U+11487 */
  0x0000,   /* U+11488 */
  0x0000,   /* U+11489 */
  0x0000,   /* U+1148A */
  0x0000,   /* U+1148B */
  0x0000,   /* U+1148C */
  0x0000,   /* U+1148D */
  0x0000,   /* U+1148E */
  0x0000,   /* U+1148F */
  0x0000,   /* U+11490 */
  0x0000,   /* U+11491 */
  0x0000,   /* U+11492 */
  0x0000,   /* U+11493 */
  0x0000,   /* U+11494 */
  0x0000,   /* U+11495 */
  0x0000,   /* U+11496 */
  0x0000,   /* U+11497 */
  0x0000,   /* U+11498 */
  0x0000,   /* U+11499 */
  0x0000,   /* U+1149A */
  0x0000,   /* U+1149B */
  0x0000,   /* U+1149C */
  0x0000,   /* U+1149D */
  0x0000,   /* U+1149E */
  0x0000,   /* U+1149F */
  0x0000,   /* U+114A0 */
  0x0000,   /* U+114A1 */
  0x0000,   /* U+114A2 */
  0x0000,   /* U+114A3 */
  0x0000,   /* U+114A4 */
  0x0000,   /* U+114A5 */
  0x0000,   /* U+114A6 */
  0x0000,   /* U+114A7 */
  0x0000,   /* U+114A8 */
  0x0000,   /* U+114A9 */
  0x0000,   /* U+114AA */
  0x0000,   /* U+114AB */
  0x0000,   /* U+114AC */
  0x0000,   /* U+114AD */
  0x0000,   /* U+114AE */
  0x0000,   /* U+114AF */
  0x0000,   /* U+114B0 */
  0x0000,   /* U+114B1 */
  0x0000,   /* U+114B2 */
  0x0000,   /* U+114B3 */
  0x0000,   /* U+114B4 */
  0x0000,   /* U+114B5 */
  0x0000,   /* U+114B6 */
  0x0000,   /* U+114B7 */
  0x0000,   /* U+114B8 */
  0x0000,   /* U+114B9 */
  0x0000,   /* U+114BA */
  0x0000,   /* U+114BB */
  0x0000,   /* U+114BC */
  0x0000,   /* U+114BD */
  0x0000,   /* U+114BE */
  0x0000,   /* U+114BF */
  0x0000,   /* U+114C0 */
  0x0000,   /* U+114C1 */
  0x0000,   /* U+114C2 */
  0x0000,   /* U+114C3 */
  0x0000,   /* U+114C4 */
  0x0000,   /* U+114C5 */
  0x0000,   /* U+114C6 */
  0x0000,   /* U+114C7 */
  0x0000,   /* U+114C8 */
  0x0000,   /* U+114C9 */
  0x0000,   /* U+114CA */
  0x0000,   /* U+114CB */
  0x0000,   /* U+114CC */
  0x0000,   /* U+114CD */
  0x0000,   /* U+114CE */
  0x0000,   /* U+114CF */
  0x0000,   /* U+114D0 */
  0x0000,   /* U+114D1 */
  0x0000,   /* U+114D2 */
  0x0000,   /* U+114D3 */
  0x0000,   /* U+114D4 */
  0x0000,   /* U+114D5 */
  0x0000,   /* U+114D6 */
  0x0000,   /* U+114D7 */
  0x0000,   /* U+114D8 */
  0x0000,   /* U+114D9 */
  0x0000,   /* U+114DA */
  0x0000,   /* U+114DB */
  0x0000,   /* U+114DC */
  0x0000,   /* U+114DD */
  0x0000,   /* U+114DE */
  0x0000,   /* U+114DF */
  0x0000,   /* U+114E0 */
  0x0000,   /* U+114E1 */
  0x0000,   /* U+114E2 */
  0x0000,   /* U+114E3 */
  0x0000,   /* U+114E4 */
  0x0000,   /* U+114E5 */
  0x0000,   /* U+114E6 */
  0x0000,   /* U+114E7 */
  0x0000,   /* U+114E8 */
  0x0000,   /* U+114E9 */
  0x0000,   /* U+114EA */
  0x0000,   /* U+114EB */
  0x0000,   /* U+114EC */
  0x0000,   /* U+114ED */
  0x0000,   /* U+114EE */
  0x0000,   /* U+114EF */
  0x0000,   /* U+114F0 */
  0x0000,   /* U+114F1 */
  0x0000,   /* U+114F2 */
  0x0000,   /* U+114F3 */
  0x0000,   /* U+114F4 */
  0x0000,   /* U+114F5 */
  0x0000,   /* U+114F6 */
  0x0000,   /* U+114F7 */
  0x0000,   /* U+114F8 */
  0x0000,   /* U+114F9 */
  0x0000,   /* U+114FA */
  0x0000,   /* U+114FB */
  0x0000,   /* U+114FC */
  0x0000,   /* U+114FD */
  0x0000,   /* U+114FE */
  0x0000,   /* U+114FF */
};

uint16 uca900_p115[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC2,   /* U+11500 */
  0xFBC2,   /* U+11501 */
  0xFBC2,   /* U+11502 */
  0xFBC2,   /* U+11503 */
  0xFBC2,   /* U+11504 */
  0xFBC2,   /* U+11505 */
  0xFBC2,   /* U+11506 */
  0xFBC2,   /* U+11507 */
  0xFBC2,   /* U+11508 */
  0xFBC2,   /* U+11509 */
  0xFBC2,   /* U+1150A */
  0xFBC2,   /* U+1150B */
  0xFBC2,   /* U+1150C */
  0xFBC2,   /* U+1150D */
  0xFBC2,   /* U+1150E */
  0xFBC2,   /* U+1150F */
  0xFBC2,   /* U+11510 */
  0xFBC2,   /* U+11511 */
  0xFBC2,   /* U+11512 */
  0xFBC2,   /* U+11513 */
  0xFBC2,   /* U+11514 */
  0xFBC2,   /* U+11515 */
  0xFBC2,   /* U+11516 */
  0xFBC2,   /* U+11517 */
  0xFBC2,   /* U+11518 */
  0xFBC2,   /* U+11519 */
  0xFBC2,   /* U+1151A */
  0xFBC2,   /* U+1151B */
  0xFBC2,   /* U+1151C */
  0xFBC2,   /* U+1151D */
  0xFBC2,   /* U+1151E */
  0xFBC2,   /* U+1151F */
  0xFBC2,   /* U+11520 */
  0xFBC2,   /* U+11521 */
  0xFBC2,   /* U+11522 */
  0xFBC2,   /* U+11523 */
  0xFBC2,   /* U+11524 */
  0xFBC2,   /* U+11525 */
  0xFBC2,   /* U+11526 */
  0xFBC2,   /* U+11527 */
  0xFBC2,   /* U+11528 */
  0xFBC2,   /* U+11529 */
  0xFBC2,   /* U+1152A */
  0xFBC2,   /* U+1152B */
  0xFBC2,   /* U+1152C */
  0xFBC2,   /* U+1152D */
  0xFBC2,   /* U+1152E */
  0xFBC2,   /* U+1152F */
  0xFBC2,   /* U+11530 */
  0xFBC2,   /* U+11531 */
  0xFBC2,   /* U+11532 */
  0xFBC2,   /* U+11533 */
  0xFBC2,   /* U+11534 */
  0xFBC2,   /* U+11535 */
  0xFBC2,   /* U+11536 */
  0xFBC2,   /* U+11537 */
  0xFBC2,   /* U+11538 */
  0xFBC2,   /* U+11539 */
  0xFBC2,   /* U+1153A */
  0xFBC2,   /* U+1153B */
  0xFBC2,   /* U+1153C */
  0xFBC2,   /* U+1153D */
  0xFBC2,   /* U+1153E */
  0xFBC2,   /* U+1153F */
  0xFBC2,   /* U+11540 */
  0xFBC2,   /* U+11541 */
  0xFBC2,   /* U+11542 */
  0xFBC2,   /* U+11543 */
  0xFBC2,   /* U+11544 */
  0xFBC2,   /* U+11545 */
  0xFBC2,   /* U+11546 */
  0xFBC2,   /* U+11547 */
  0xFBC2,   /* U+11548 */
  0xFBC2,   /* U+11549 */
  0xFBC2,   /* U+1154A */
  0xFBC2,   /* U+1154B */
  0xFBC2,   /* U+1154C */
  0xFBC2,   /* U+1154D */
  0xFBC2,   /* U+1154E */
  0xFBC2,   /* U+1154F */
  0xFBC2,   /* U+11550 */
  0xFBC2,   /* U+11551 */
  0xFBC2,   /* U+11552 */
  0xFBC2,   /* U+11553 */
  0xFBC2,   /* U+11554 */
  0xFBC2,   /* U+11555 */
  0xFBC2,   /* U+11556 */
  0xFBC2,   /* U+11557 */
  0xFBC2,   /* U+11558 */
  0xFBC2,   /* U+11559 */
  0xFBC2,   /* U+1155A */
  0xFBC2,   /* U+1155B */
  0xFBC2,   /* U+1155C */
  0xFBC2,   /* U+1155D */
  0xFBC2,   /* U+1155E */
  0xFBC2,   /* U+1155F */
  0xFBC2,   /* U+11560 */
  0xFBC2,   /* U+11561 */
  0xFBC2,   /* U+11562 */
  0xFBC2,   /* U+11563 */
  0xFBC2,   /* U+11564 */
  0xFBC2,   /* U+11565 */
  0xFBC2,   /* U+11566 */
  0xFBC2,   /* U+11567 */
  0xFBC2,   /* U+11568 */
  0xFBC2,   /* U+11569 */
  0xFBC2,   /* U+1156A */
  0xFBC2,   /* U+1156B */
  0xFBC2,   /* U+1156C */
  0xFBC2,   /* U+1156D */
  0xFBC2,   /* U+1156E */
  0xFBC2,   /* U+1156F */
  0xFBC2,   /* U+11570 */
  0xFBC2,   /* U+11571 */
  0xFBC2,   /* U+11572 */
  0xFBC2,   /* U+11573 */
  0xFBC2,   /* U+11574 */
  0xFBC2,   /* U+11575 */
  0xFBC2,   /* U+11576 */
  0xFBC2,   /* U+11577 */
  0xFBC2,   /* U+11578 */
  0xFBC2,   /* U+11579 */
  0xFBC2,   /* U+1157A */
  0xFBC2,   /* U+1157B */
  0xFBC2,   /* U+1157C */
  0xFBC2,   /* U+1157D */
  0xFBC2,   /* U+1157E */
  0xFBC2,   /* U+1157F */
  0x2BC2,   /* U+11580 */
  0x2BC3,   /* U+11581 */
  0x2BC4,   /* U+11582 */
  0x2BC5,   /* U+11583 */
  0x2BC6,   /* U+11584 */
  0x2BC7,   /* U+11585 */
  0x2BC8,   /* U+11586 */
  0x2BC9,   /* U+11587 */
  0x2BCA,   /* U+11588 */
  0x2BCB,   /* U+11589 */
  0x2BCC,   /* U+1158A */
  0x2BCD,   /* U+1158B */
  0x2BCE,   /* U+1158C */
  0x2BCF,   /* U+1158D */
  0x2BD0,   /* U+1158E */
  0x2BD1,   /* U+1158F */
  0x2BD2,   /* U+11590 */
  0x2BD3,   /* U+11591 */
  0x2BD4,   /* U+11592 */
  0x2BD5,   /* U+11593 */
  0x2BD6,   /* U+11594 */
  0x2BD7,   /* U+11595 */
  0x2BD8,   /* U+11596 */
  0x2BD9,   /* U+11597 */
  0x2BDA,   /* U+11598 */
  0x2BDB,   /* U+11599 */
  0x2BDC,   /* U+1159A */
  0x2BDD,   /* U+1159B */
  0x2BDE,   /* U+1159C */
  0x2BDF,   /* U+1159D */
  0x2BE0,   /* U+1159E */
  0x2BE1,   /* U+1159F */
  0x2BE2,   /* U+115A0 */
  0x2BE3,   /* U+115A1 */
  0x2BE4,   /* U+115A2 */
  0x2BE5,   /* U+115A3 */
  0x2BE6,   /* U+115A4 */
  0x2BE7,   /* U+115A5 */
  0x2BE8,   /* U+115A6 */
  0x2BE9,   /* U+115A7 */
  0x2BEA,   /* U+115A8 */
  0x2BEB,   /* U+115A9 */
  0x2BEC,   /* U+115AA */
  0x2BED,   /* U+115AB */
  0x2BEE,   /* U+115AC */
  0x2BEF,   /* U+115AD */
  0x2BF0,   /* U+115AE */
  0x2BF1,   /* U+115AF */
  0x2BF2,   /* U+115B0 */
  0x2BF3,   /* U+115B1 */
  0x2BF4,   /* U+115B2 */
  0x2BF5,   /* U+115B3 */
  0x2BF6,   /* U+115B4 */
  0x2BF7,   /* U+115B5 */
  0xFBC2,   /* U+115B6 */
  0xFBC2,   /* U+115B7 */
  0x2BF8,   /* U+115B8 */
  0x2BF9,   /* U+115B9 */
  0x2BFA,   /* U+115BA */
  0x2BFB,   /* U+115BB */
  0x0000,   /* U+115BC */
  0x0000,   /* U+115BD */
  0x0000,   /* U+115BE */
  0x2BFC,   /* U+115BF */
  0x0000,   /* U+115C0 */
  0x0461,   /* U+115C1 */
  0x02B8,   /* U+115C2 */
  0x02B9,   /* U+115C3 */
  0x0462,   /* U+115C4 */
  0x0463,   /* U+115C5 */
  0x0464,   /* U+115C6 */
  0x0465,   /* U+115C7 */
  0x0466,   /* U+115C8 */
  0x0467,   /* U+115C9 */
  0x0468,   /* U+115CA */
  0x0469,   /* U+115CB */
  0x046A,   /* U+115CC */
  0x046B,   /* U+115CD */
  0x046C,   /* U+115CE */
  0x046D,   /* U+115CF */
  0x046E,   /* U+115D0 */
  0x046F,   /* U+115D1 */
  0x0470,   /* U+115D2 */
  0x0471,   /* U+115D3 */
  0x0472,   /* U+115D4 */
  0x0473,   /* U+115D5 */
  0x0474,   /* U+115D6 */
  0x0475,   /* U+115D7 */
  0x2BC4,   /* U+115D8 */
  0x2BC4,   /* U+115D9 */
  0x2BC5,   /* U+115DA */
  0x2BC6,   /* U+115DB */
  0x2BF4,   /* U+115DC */
  0x2BF5,   /* U+115DD */
  0xFBC2,   /* U+115DE */
  0xFBC2,   /* U+115DF */
  0xFBC2,   /* U+115E0 */
  0xFBC2,   /* U+115E1 */
  0xFBC2,   /* U+115E2 */
  0xFBC2,   /* U+115E3 */
  0xFBC2,   /* U+115E4 */
  0xFBC2,   /* U+115E5 */
  0xFBC2,   /* U+115E6 */
  0xFBC2,   /* U+115E7 */
  0xFBC2,   /* U+115E8 */
  0xFBC2,   /* U+115E9 */
  0xFBC2,   /* U+115EA */
  0xFBC2,   /* U+115EB */
  0xFBC2,   /* U+115EC */
  0xFBC2,   /* U+115ED */
  0xFBC2,   /* U+115EE */
  0xFBC2,   /* U+115EF */
  0xFBC2,   /* U+115F0 */
  0xFBC2,   /* U+115F1 */
  0xFBC2,   /* U+115F2 */
  0xFBC2,   /* U+115F3 */
  0xFBC2,   /* U+115F4 */
  0xFBC2,   /* U+115F5 */
  0xFBC2,   /* U+115F6 */
  0xFBC2,   /* U+115F7 */
  0xFBC2,   /* U+115F8 */
  0xFBC2,   /* U+115F9 */
  0xFBC2,   /* U+115FA */
  0xFBC2,   /* U+115FB */
  0xFBC2,   /* U+115FC */
  0xFBC2,   /* U+115FD */
  0xFBC2,   /* U+115FE */
  0xFBC2,   /* U+115FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11500 */
  0x0020,   /* U+11501 */
  0x0020,   /* U+11502 */
  0x0020,   /* U+11503 */
  0x0020,   /* U+11504 */
  0x0020,   /* U+11505 */
  0x0020,   /* U+11506 */
  0x0020,   /* U+11507 */
  0x0020,   /* U+11508 */
  0x0020,   /* U+11509 */
  0x0020,   /* U+1150A */
  0x0020,   /* U+1150B */
  0x0020,   /* U+1150C */
  0x0020,   /* U+1150D */
  0x0020,   /* U+1150E */
  0x0020,   /* U+1150F */
  0x0020,   /* U+11510 */
  0x0020,   /* U+11511 */
  0x0020,   /* U+11512 */
  0x0020,   /* U+11513 */
  0x0020,   /* U+11514 */
  0x0020,   /* U+11515 */
  0x0020,   /* U+11516 */
  0x0020,   /* U+11517 */
  0x0020,   /* U+11518 */
  0x0020,   /* U+11519 */
  0x0020,   /* U+1151A */
  0x0020,   /* U+1151B */
  0x0020,   /* U+1151C */
  0x0020,   /* U+1151D */
  0x0020,   /* U+1151E */
  0x0020,   /* U+1151F */
  0x0020,   /* U+11520 */
  0x0020,   /* U+11521 */
  0x0020,   /* U+11522 */
  0x0020,   /* U+11523 */
  0x0020,   /* U+11524 */
  0x0020,   /* U+11525 */
  0x0020,   /* U+11526 */
  0x0020,   /* U+11527 */
  0x0020,   /* U+11528 */
  0x0020,   /* U+11529 */
  0x0020,   /* U+1152A */
  0x0020,   /* U+1152B */
  0x0020,   /* U+1152C */
  0x0020,   /* U+1152D */
  0x0020,   /* U+1152E */
  0x0020,   /* U+1152F */
  0x0020,   /* U+11530 */
  0x0020,   /* U+11531 */
  0x0020,   /* U+11532 */
  0x0020,   /* U+11533 */
  0x0020,   /* U+11534 */
  0x0020,   /* U+11535 */
  0x0020,   /* U+11536 */
  0x0020,   /* U+11537 */
  0x0020,   /* U+11538 */
  0x0020,   /* U+11539 */
  0x0020,   /* U+1153A */
  0x0020,   /* U+1153B */
  0x0020,   /* U+1153C */
  0x0020,   /* U+1153D */
  0x0020,   /* U+1153E */
  0x0020,   /* U+1153F */
  0x0020,   /* U+11540 */
  0x0020,   /* U+11541 */
  0x0020,   /* U+11542 */
  0x0020,   /* U+11543 */
  0x0020,   /* U+11544 */
  0x0020,   /* U+11545 */
  0x0020,   /* U+11546 */
  0x0020,   /* U+11547 */
  0x0020,   /* U+11548 */
  0x0020,   /* U+11549 */
  0x0020,   /* U+1154A */
  0x0020,   /* U+1154B */
  0x0020,   /* U+1154C */
  0x0020,   /* U+1154D */
  0x0020,   /* U+1154E */
  0x0020,   /* U+1154F */
  0x0020,   /* U+11550 */
  0x0020,   /* U+11551 */
  0x0020,   /* U+11552 */
  0x0020,   /* U+11553 */
  0x0020,   /* U+11554 */
  0x0020,   /* U+11555 */
  0x0020,   /* U+11556 */
  0x0020,   /* U+11557 */
  0x0020,   /* U+11558 */
  0x0020,   /* U+11559 */
  0x0020,   /* U+1155A */
  0x0020,   /* U+1155B */
  0x0020,   /* U+1155C */
  0x0020,   /* U+1155D */
  0x0020,   /* U+1155E */
  0x0020,   /* U+1155F */
  0x0020,   /* U+11560 */
  0x0020,   /* U+11561 */
  0x0020,   /* U+11562 */
  0x0020,   /* U+11563 */
  0x0020,   /* U+11564 */
  0x0020,   /* U+11565 */
  0x0020,   /* U+11566 */
  0x0020,   /* U+11567 */
  0x0020,   /* U+11568 */
  0x0020,   /* U+11569 */
  0x0020,   /* U+1156A */
  0x0020,   /* U+1156B */
  0x0020,   /* U+1156C */
  0x0020,   /* U+1156D */
  0x0020,   /* U+1156E */
  0x0020,   /* U+1156F */
  0x0020,   /* U+11570 */
  0x0020,   /* U+11571 */
  0x0020,   /* U+11572 */
  0x0020,   /* U+11573 */
  0x0020,   /* U+11574 */
  0x0020,   /* U+11575 */
  0x0020,   /* U+11576 */
  0x0020,   /* U+11577 */
  0x0020,   /* U+11578 */
  0x0020,   /* U+11579 */
  0x0020,   /* U+1157A */
  0x0020,   /* U+1157B */
  0x0020,   /* U+1157C */
  0x0020,   /* U+1157D */
  0x0020,   /* U+1157E */
  0x0020,   /* U+1157F */
  0x0020,   /* U+11580 */
  0x0020,   /* U+11581 */
  0x0020,   /* U+11582 */
  0x0020,   /* U+11583 */
  0x0020,   /* U+11584 */
  0x0020,   /* U+11585 */
  0x0020,   /* U+11586 */
  0x0020,   /* U+11587 */
  0x0020,   /* U+11588 */
  0x0020,   /* U+11589 */
  0x0020,   /* U+1158A */
  0x0020,   /* U+1158B */
  0x0020,   /* U+1158C */
  0x0020,   /* U+1158D */
  0x0020,   /* U+1158E */
  0x0020,   /* U+1158F */
  0x0020,   /* U+11590 */
  0x0020,   /* U+11591 */
  0x0020,   /* U+11592 */
  0x0020,   /* U+11593 */
  0x0020,   /* U+11594 */
  0x0020,   /* U+11595 */
  0x0020,   /* U+11596 */
  0x0020,   /* U+11597 */
  0x0020,   /* U+11598 */
  0x0020,   /* U+11599 */
  0x0020,   /* U+1159A */
  0x0020,   /* U+1159B */
  0x0020,   /* U+1159C */
  0x0020,   /* U+1159D */
  0x0020,   /* U+1159E */
  0x0020,   /* U+1159F */
  0x0020,   /* U+115A0 */
  0x0020,   /* U+115A1 */
  0x0020,   /* U+115A2 */
  0x0020,   /* U+115A3 */
  0x0020,   /* U+115A4 */
  0x0020,   /* U+115A5 */
  0x0020,   /* U+115A6 */
  0x0020,   /* U+115A7 */
  0x0020,   /* U+115A8 */
  0x0020,   /* U+115A9 */
  0x0020,   /* U+115AA */
  0x0020,   /* U+115AB */
  0x0020,   /* U+115AC */
  0x0020,   /* U+115AD */
  0x0020,   /* U+115AE */
  0x0020,   /* U+115AF */
  0x0020,   /* U+115B0 */
  0x0020,   /* U+115B1 */
  0x0020,   /* U+115B2 */
  0x0020,   /* U+115B3 */
  0x0020,   /* U+115B4 */
  0x0020,   /* U+115B5 */
  0x0020,   /* U+115B6 */
  0x0020,   /* U+115B7 */
  0x0020,   /* U+115B8 */
  0x0020,   /* U+115B9 */
  0x0020,   /* U+115BA */
  0x0020,   /* U+115BB */
  0x00C3,   /* U+115BC */
  0x00C4,   /* U+115BD */
  0x00C5,   /* U+115BE */
  0x0020,   /* U+115BF */
  0x00C2,   /* U+115C0 */
  0x0020,   /* U+115C1 */
  0x0020,   /* U+115C2 */
  0x0020,   /* U+115C3 */
  0x0020,   /* U+115C4 */
  0x0020,   /* U+115C5 */
  0x0020,   /* U+115C6 */
  0x0020,   /* U+115C7 */
  0x0020,   /* U+115C8 */
  0x0020,   /* U+115C9 */
  0x0020,   /* U+115CA */
  0x0020,   /* U+115CB */
  0x0020,   /* U+115CC */
  0x0020,   /* U+115CD */
  0x0020,   /* U+115CE */
  0x0020,   /* U+115CF */
  0x0020,   /* U+115D0 */
  0x0020,   /* U+115D1 */
  0x0020,   /* U+115D2 */
  0x0020,   /* U+115D3 */
  0x0020,   /* U+115D4 */
  0x0020,   /* U+115D5 */
  0x0020,   /* U+115D6 */
  0x0020,   /* U+115D7 */
  0x0020,   /* U+115D8 */
  0x0020,   /* U+115D9 */
  0x0020,   /* U+115DA */
  0x0020,   /* U+115DB */
  0x0020,   /* U+115DC */
  0x0020,   /* U+115DD */
  0x0020,   /* U+115DE */
  0x0020,   /* U+115DF */
  0x0020,   /* U+115E0 */
  0x0020,   /* U+115E1 */
  0x0020,   /* U+115E2 */
  0x0020,   /* U+115E3 */
  0x0020,   /* U+115E4 */
  0x0020,   /* U+115E5 */
  0x0020,   /* U+115E6 */
  0x0020,   /* U+115E7 */
  0x0020,   /* U+115E8 */
  0x0020,   /* U+115E9 */
  0x0020,   /* U+115EA */
  0x0020,   /* U+115EB */
  0x0020,   /* U+115EC */
  0x0020,   /* U+115ED */
  0x0020,   /* U+115EE */
  0x0020,   /* U+115EF */
  0x0020,   /* U+115F0 */
  0x0020,   /* U+115F1 */
  0x0020,   /* U+115F2 */
  0x0020,   /* U+115F3 */
  0x0020,   /* U+115F4 */
  0x0020,   /* U+115F5 */
  0x0020,   /* U+115F6 */
  0x0020,   /* U+115F7 */
  0x0020,   /* U+115F8 */
  0x0020,   /* U+115F9 */
  0x0020,   /* U+115FA */
  0x0020,   /* U+115FB */
  0x0020,   /* U+115FC */
  0x0020,   /* U+115FD */
  0x0020,   /* U+115FE */
  0x0020,   /* U+115FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11500 */
  0x0002,   /* U+11501 */
  0x0002,   /* U+11502 */
  0x0002,   /* U+11503 */
  0x0002,   /* U+11504 */
  0x0002,   /* U+11505 */
  0x0002,   /* U+11506 */
  0x0002,   /* U+11507 */
  0x0002,   /* U+11508 */
  0x0002,   /* U+11509 */
  0x0002,   /* U+1150A */
  0x0002,   /* U+1150B */
  0x0002,   /* U+1150C */
  0x0002,   /* U+1150D */
  0x0002,   /* U+1150E */
  0x0002,   /* U+1150F */
  0x0002,   /* U+11510 */
  0x0002,   /* U+11511 */
  0x0002,   /* U+11512 */
  0x0002,   /* U+11513 */
  0x0002,   /* U+11514 */
  0x0002,   /* U+11515 */
  0x0002,   /* U+11516 */
  0x0002,   /* U+11517 */
  0x0002,   /* U+11518 */
  0x0002,   /* U+11519 */
  0x0002,   /* U+1151A */
  0x0002,   /* U+1151B */
  0x0002,   /* U+1151C */
  0x0002,   /* U+1151D */
  0x0002,   /* U+1151E */
  0x0002,   /* U+1151F */
  0x0002,   /* U+11520 */
  0x0002,   /* U+11521 */
  0x0002,   /* U+11522 */
  0x0002,   /* U+11523 */
  0x0002,   /* U+11524 */
  0x0002,   /* U+11525 */
  0x0002,   /* U+11526 */
  0x0002,   /* U+11527 */
  0x0002,   /* U+11528 */
  0x0002,   /* U+11529 */
  0x0002,   /* U+1152A */
  0x0002,   /* U+1152B */
  0x0002,   /* U+1152C */
  0x0002,   /* U+1152D */
  0x0002,   /* U+1152E */
  0x0002,   /* U+1152F */
  0x0002,   /* U+11530 */
  0x0002,   /* U+11531 */
  0x0002,   /* U+11532 */
  0x0002,   /* U+11533 */
  0x0002,   /* U+11534 */
  0x0002,   /* U+11535 */
  0x0002,   /* U+11536 */
  0x0002,   /* U+11537 */
  0x0002,   /* U+11538 */
  0x0002,   /* U+11539 */
  0x0002,   /* U+1153A */
  0x0002,   /* U+1153B */
  0x0002,   /* U+1153C */
  0x0002,   /* U+1153D */
  0x0002,   /* U+1153E */
  0x0002,   /* U+1153F */
  0x0002,   /* U+11540 */
  0x0002,   /* U+11541 */
  0x0002,   /* U+11542 */
  0x0002,   /* U+11543 */
  0x0002,   /* U+11544 */
  0x0002,   /* U+11545 */
  0x0002,   /* U+11546 */
  0x0002,   /* U+11547 */
  0x0002,   /* U+11548 */
  0x0002,   /* U+11549 */
  0x0002,   /* U+1154A */
  0x0002,   /* U+1154B */
  0x0002,   /* U+1154C */
  0x0002,   /* U+1154D */
  0x0002,   /* U+1154E */
  0x0002,   /* U+1154F */
  0x0002,   /* U+11550 */
  0x0002,   /* U+11551 */
  0x0002,   /* U+11552 */
  0x0002,   /* U+11553 */
  0x0002,   /* U+11554 */
  0x0002,   /* U+11555 */
  0x0002,   /* U+11556 */
  0x0002,   /* U+11557 */
  0x0002,   /* U+11558 */
  0x0002,   /* U+11559 */
  0x0002,   /* U+1155A */
  0x0002,   /* U+1155B */
  0x0002,   /* U+1155C */
  0x0002,   /* U+1155D */
  0x0002,   /* U+1155E */
  0x0002,   /* U+1155F */
  0x0002,   /* U+11560 */
  0x0002,   /* U+11561 */
  0x0002,   /* U+11562 */
  0x0002,   /* U+11563 */
  0x0002,   /* U+11564 */
  0x0002,   /* U+11565 */
  0x0002,   /* U+11566 */
  0x0002,   /* U+11567 */
  0x0002,   /* U+11568 */
  0x0002,   /* U+11569 */
  0x0002,   /* U+1156A */
  0x0002,   /* U+1156B */
  0x0002,   /* U+1156C */
  0x0002,   /* U+1156D */
  0x0002,   /* U+1156E */
  0x0002,   /* U+1156F */
  0x0002,   /* U+11570 */
  0x0002,   /* U+11571 */
  0x0002,   /* U+11572 */
  0x0002,   /* U+11573 */
  0x0002,   /* U+11574 */
  0x0002,   /* U+11575 */
  0x0002,   /* U+11576 */
  0x0002,   /* U+11577 */
  0x0002,   /* U+11578 */
  0x0002,   /* U+11579 */
  0x0002,   /* U+1157A */
  0x0002,   /* U+1157B */
  0x0002,   /* U+1157C */
  0x0002,   /* U+1157D */
  0x0002,   /* U+1157E */
  0x0002,   /* U+1157F */
  0x0002,   /* U+11580 */
  0x0002,   /* U+11581 */
  0x0002,   /* U+11582 */
  0x0002,   /* U+11583 */
  0x0002,   /* U+11584 */
  0x0002,   /* U+11585 */
  0x0002,   /* U+11586 */
  0x0002,   /* U+11587 */
  0x0002,   /* U+11588 */
  0x0002,   /* U+11589 */
  0x0002,   /* U+1158A */
  0x0002,   /* U+1158B */
  0x0002,   /* U+1158C */
  0x0002,   /* U+1158D */
  0x0002,   /* U+1158E */
  0x0002,   /* U+1158F */
  0x0002,   /* U+11590 */
  0x0002,   /* U+11591 */
  0x0002,   /* U+11592 */
  0x0002,   /* U+11593 */
  0x0002,   /* U+11594 */
  0x0002,   /* U+11595 */
  0x0002,   /* U+11596 */
  0x0002,   /* U+11597 */
  0x0002,   /* U+11598 */
  0x0002,   /* U+11599 */
  0x0002,   /* U+1159A */
  0x0002,   /* U+1159B */
  0x0002,   /* U+1159C */
  0x0002,   /* U+1159D */
  0x0002,   /* U+1159E */
  0x0002,   /* U+1159F */
  0x0002,   /* U+115A0 */
  0x0002,   /* U+115A1 */
  0x0002,   /* U+115A2 */
  0x0002,   /* U+115A3 */
  0x0002,   /* U+115A4 */
  0x0002,   /* U+115A5 */
  0x0002,   /* U+115A6 */
  0x0002,   /* U+115A7 */
  0x0002,   /* U+115A8 */
  0x0002,   /* U+115A9 */
  0x0002,   /* U+115AA */
  0x0002,   /* U+115AB */
  0x0002,   /* U+115AC */
  0x0002,   /* U+115AD */
  0x0002,   /* U+115AE */
  0x0002,   /* U+115AF */
  0x0002,   /* U+115B0 */
  0x0002,   /* U+115B1 */
  0x0002,   /* U+115B2 */
  0x0002,   /* U+115B3 */
  0x0002,   /* U+115B4 */
  0x0002,   /* U+115B5 */
  0x0002,   /* U+115B6 */
  0x0002,   /* U+115B7 */
  0x0002,   /* U+115B8 */
  0x0002,   /* U+115B9 */
  0x0002,   /* U+115BA */
  0x0002,   /* U+115BB */
  0x0002,   /* U+115BC */
  0x0002,   /* U+115BD */
  0x0002,   /* U+115BE */
  0x0002,   /* U+115BF */
  0x0002,   /* U+115C0 */
  0x0002,   /* U+115C1 */
  0x0002,   /* U+115C2 */
  0x0002,   /* U+115C3 */
  0x0002,   /* U+115C4 */
  0x0002,   /* U+115C5 */
  0x0002,   /* U+115C6 */
  0x0002,   /* U+115C7 */
  0x0002,   /* U+115C8 */
  0x0002,   /* U+115C9 */
  0x0002,   /* U+115CA */
  0x0002,   /* U+115CB */
  0x0002,   /* U+115CC */
  0x0002,   /* U+115CD */
  0x0002,   /* U+115CE */
  0x0002,   /* U+115CF */
  0x0002,   /* U+115D0 */
  0x0002,   /* U+115D1 */
  0x0002,   /* U+115D2 */
  0x0002,   /* U+115D3 */
  0x0002,   /* U+115D4 */
  0x0002,   /* U+115D5 */
  0x0002,   /* U+115D6 */
  0x0002,   /* U+115D7 */
  0x0004,   /* U+115D8 */
  0x0004,   /* U+115D9 */
  0x0004,   /* U+115DA */
  0x0004,   /* U+115DB */
  0x0004,   /* U+115DC */
  0x0004,   /* U+115DD */
  0x0002,   /* U+115DE */
  0x0002,   /* U+115DF */
  0x0002,   /* U+115E0 */
  0x0002,   /* U+115E1 */
  0x0002,   /* U+115E2 */
  0x0002,   /* U+115E3 */
  0x0002,   /* U+115E4 */
  0x0002,   /* U+115E5 */
  0x0002,   /* U+115E6 */
  0x0002,   /* U+115E7 */
  0x0002,   /* U+115E8 */
  0x0002,   /* U+115E9 */
  0x0002,   /* U+115EA */
  0x0002,   /* U+115EB */
  0x0002,   /* U+115EC */
  0x0002,   /* U+115ED */
  0x0002,   /* U+115EE */
  0x0002,   /* U+115EF */
  0x0002,   /* U+115F0 */
  0x0002,   /* U+115F1 */
  0x0002,   /* U+115F2 */
  0x0002,   /* U+115F3 */
  0x0002,   /* U+115F4 */
  0x0002,   /* U+115F5 */
  0x0002,   /* U+115F6 */
  0x0002,   /* U+115F7 */
  0x0002,   /* U+115F8 */
  0x0002,   /* U+115F9 */
  0x0002,   /* U+115FA */
  0x0002,   /* U+115FB */
  0x0002,   /* U+115FC */
  0x0002,   /* U+115FD */
  0x0002,   /* U+115FE */
  0x0002,   /* U+115FF */

  /* Primary weight 2 for each character. */
  0x9500,   /* U+11500 */
  0x9501,   /* U+11501 */
  0x9502,   /* U+11502 */
  0x9503,   /* U+11503 */
  0x9504,   /* U+11504 */
  0x9505,   /* U+11505 */
  0x9506,   /* U+11506 */
  0x9507,   /* U+11507 */
  0x9508,   /* U+11508 */
  0x9509,   /* U+11509 */
  0x950A,   /* U+1150A */
  0x950B,   /* U+1150B */
  0x950C,   /* U+1150C */
  0x950D,   /* U+1150D */
  0x950E,   /* U+1150E */
  0x950F,   /* U+1150F */
  0x9510,   /* U+11510 */
  0x9511,   /* U+11511 */
  0x9512,   /* U+11512 */
  0x9513,   /* U+11513 */
  0x9514,   /* U+11514 */
  0x9515,   /* U+11515 */
  0x9516,   /* U+11516 */
  0x9517,   /* U+11517 */
  0x9518,   /* U+11518 */
  0x9519,   /* U+11519 */
  0x951A,   /* U+1151A */
  0x951B,   /* U+1151B */
  0x951C,   /* U+1151C */
  0x951D,   /* U+1151D */
  0x951E,   /* U+1151E */
  0x951F,   /* U+1151F */
  0x9520,   /* U+11520 */
  0x9521,   /* U+11521 */
  0x9522,   /* U+11522 */
  0x9523,   /* U+11523 */
  0x9524,   /* U+11524 */
  0x9525,   /* U+11525 */
  0x9526,   /* U+11526 */
  0x9527,   /* U+11527 */
  0x9528,   /* U+11528 */
  0x9529,   /* U+11529 */
  0x952A,   /* U+1152A */
  0x952B,   /* U+1152B */
  0x952C,   /* U+1152C */
  0x952D,   /* U+1152D */
  0x952E,   /* U+1152E */
  0x952F,   /* U+1152F */
  0x9530,   /* U+11530 */
  0x9531,   /* U+11531 */
  0x9532,   /* U+11532 */
  0x9533,   /* U+11533 */
  0x9534,   /* U+11534 */
  0x9535,   /* U+11535 */
  0x9536,   /* U+11536 */
  0x9537,   /* U+11537 */
  0x9538,   /* U+11538 */
  0x9539,   /* U+11539 */
  0x953A,   /* U+1153A */
  0x953B,   /* U+1153B */
  0x953C,   /* U+1153C */
  0x953D,   /* U+1153D */
  0x953E,   /* U+1153E */
  0x953F,   /* U+1153F */
  0x9540,   /* U+11540 */
  0x9541,   /* U+11541 */
  0x9542,   /* U+11542 */
  0x9543,   /* U+11543 */
  0x9544,   /* U+11544 */
  0x9545,   /* U+11545 */
  0x9546,   /* U+11546 */
  0x9547,   /* U+11547 */
  0x9548,   /* U+11548 */
  0x9549,   /* U+11549 */
  0x954A,   /* U+1154A */
  0x954B,   /* U+1154B */
  0x954C,   /* U+1154C */
  0x954D,   /* U+1154D */
  0x954E,   /* U+1154E */
  0x954F,   /* U+1154F */
  0x9550,   /* U+11550 */
  0x9551,   /* U+11551 */
  0x9552,   /* U+11552 */
  0x9553,   /* U+11553 */
  0x9554,   /* U+11554 */
  0x9555,   /* U+11555 */
  0x9556,   /* U+11556 */
  0x9557,   /* U+11557 */
  0x9558,   /* U+11558 */
  0x9559,   /* U+11559 */
  0x955A,   /* U+1155A */
  0x955B,   /* U+1155B */
  0x955C,   /* U+1155C */
  0x955D,   /* U+1155D */
  0x955E,   /* U+1155E */
  0x955F,   /* U+1155F */
  0x9560,   /* U+11560 */
  0x9561,   /* U+11561 */
  0x9562,   /* U+11562 */
  0x9563,   /* U+11563 */
  0x9564,   /* U+11564 */
  0x9565,   /* U+11565 */
  0x9566,   /* U+11566 */
  0x9567,   /* U+11567 */
  0x9568,   /* U+11568 */
  0x9569,   /* U+11569 */
  0x956A,   /* U+1156A */
  0x956B,   /* U+1156B */
  0x956C,   /* U+1156C */
  0x956D,   /* U+1156D */
  0x956E,   /* U+1156E */
  0x956F,   /* U+1156F */
  0x9570,   /* U+11570 */
  0x9571,   /* U+11571 */
  0x9572,   /* U+11572 */
  0x9573,   /* U+11573 */
  0x9574,   /* U+11574 */
  0x9575,   /* U+11575 */
  0x9576,   /* U+11576 */
  0x9577,   /* U+11577 */
  0x9578,   /* U+11578 */
  0x9579,   /* U+11579 */
  0x957A,   /* U+1157A */
  0x957B,   /* U+1157B */
  0x957C,   /* U+1157C */
  0x957D,   /* U+1157D */
  0x957E,   /* U+1157E */
  0x957F,   /* U+1157F */
  0x0000,   /* U+11580 */
  0x0000,   /* U+11581 */
  0x0000,   /* U+11582 */
  0x0000,   /* U+11583 */
  0x0000,   /* U+11584 */
  0x0000,   /* U+11585 */
  0x0000,   /* U+11586 */
  0x0000,   /* U+11587 */
  0x0000,   /* U+11588 */
  0x0000,   /* U+11589 */
  0x0000,   /* U+1158A */
  0x0000,   /* U+1158B */
  0x0000,   /* U+1158C */
  0x0000,   /* U+1158D */
  0x0000,   /* U+1158E */
  0x0000,   /* U+1158F */
  0x0000,   /* U+11590 */
  0x0000,   /* U+11591 */
  0x0000,   /* U+11592 */
  0x0000,   /* U+11593 */
  0x0000,   /* U+11594 */
  0x0000,   /* U+11595 */
  0x0000,   /* U+11596 */
  0x0000,   /* U+11597 */
  0x0000,   /* U+11598 */
  0x0000,   /* U+11599 */
  0x0000,   /* U+1159A */
  0x0000,   /* U+1159B */
  0x0000,   /* U+1159C */
  0x0000,   /* U+1159D */
  0x0000,   /* U+1159E */
  0x0000,   /* U+1159F */
  0x0000,   /* U+115A0 */
  0x0000,   /* U+115A1 */
  0x0000,   /* U+115A2 */
  0x0000,   /* U+115A3 */
  0x0000,   /* U+115A4 */
  0x0000,   /* U+115A5 */
  0x0000,   /* U+115A6 */
  0x0000,   /* U+115A7 */
  0x0000,   /* U+115A8 */
  0x0000,   /* U+115A9 */
  0x0000,   /* U+115AA */
  0x0000,   /* U+115AB */
  0x0000,   /* U+115AC */
  0x0000,   /* U+115AD */
  0x0000,   /* U+115AE */
  0x0000,   /* U+115AF */
  0x0000,   /* U+115B0 */
  0x0000,   /* U+115B1 */
  0x0000,   /* U+115B2 */
  0x0000,   /* U+115B3 */
  0x0000,   /* U+115B4 */
  0x0000,   /* U+115B5 */
  0x95B6,   /* U+115B6 */
  0x95B7,   /* U+115B7 */
  0x0000,   /* U+115B8 */
  0x0000,   /* U+115B9 */
  0x0000,   /* U+115BA */
  0x0000,   /* U+115BB */
  0x0000,   /* U+115BC */
  0x0000,   /* U+115BD */
  0x0000,   /* U+115BE */
  0x0000,   /* U+115BF */
  0x0000,   /* U+115C0 */
  0x0000,   /* U+115C1 */
  0x0000,   /* U+115C2 */
  0x0000,   /* U+115C3 */
  0x0000,   /* U+115C4 */
  0x0000,   /* U+115C5 */
  0x0000,   /* U+115C6 */
  0x0000,   /* U+115C7 */
  0x0000,   /* U+115C8 */
  0x0000,   /* U+115C9 */
  0x0000,   /* U+115CA */
  0x0000,   /* U+115CB */
  0x0000,   /* U+115CC */
  0x0000,   /* U+115CD */
  0x0000,   /* U+115CE */
  0x0000,   /* U+115CF */
  0x0000,   /* U+115D0 */
  0x0000,   /* U+115D1 */
  0x0000,   /* U+115D2 */
  0x0000,   /* U+115D3 */
  0x0000,   /* U+115D4 */
  0x0000,   /* U+115D5 */
  0x0000,   /* U+115D6 */
  0x0000,   /* U+115D7 */
  0x0000,   /* U+115D8 */
  0x0000,   /* U+115D9 */
  0x0000,   /* U+115DA */
  0x0000,   /* U+115DB */
  0x0000,   /* U+115DC */
  0x0000,   /* U+115DD */
  0x95DE,   /* U+115DE */
  0x95DF,   /* U+115DF */
  0x95E0,   /* U+115E0 */
  0x95E1,   /* U+115E1 */
  0x95E2,   /* U+115E2 */
  0x95E3,   /* U+115E3 */
  0x95E4,   /* U+115E4 */
  0x95E5,   /* U+115E5 */
  0x95E6,   /* U+115E6 */
  0x95E7,   /* U+115E7 */
  0x95E8,   /* U+115E8 */
  0x95E9,   /* U+115E9 */
  0x95EA,   /* U+115EA */
  0x95EB,   /* U+115EB */
  0x95EC,   /* U+115EC */
  0x95ED,   /* U+115ED */
  0x95EE,   /* U+115EE */
  0x95EF,   /* U+115EF */
  0x95F0,   /* U+115F0 */
  0x95F1,   /* U+115F1 */
  0x95F2,   /* U+115F2 */
  0x95F3,   /* U+115F3 */
  0x95F4,   /* U+115F4 */
  0x95F5,   /* U+115F5 */
  0x95F6,   /* U+115F6 */
  0x95F7,   /* U+115F7 */
  0x95F8,   /* U+115F8 */
  0x95F9,   /* U+115F9 */
  0x95FA,   /* U+115FA */
  0x95FB,   /* U+115FB */
  0x95FC,   /* U+115FC */
  0x95FD,   /* U+115FD */
  0x95FE,   /* U+115FE */
  0x95FF,   /* U+115FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11500 */
  0x0000,   /* U+11501 */
  0x0000,   /* U+11502 */
  0x0000,   /* U+11503 */
  0x0000,   /* U+11504 */
  0x0000,   /* U+11505 */
  0x0000,   /* U+11506 */
  0x0000,   /* U+11507 */
  0x0000,   /* U+11508 */
  0x0000,   /* U+11509 */
  0x0000,   /* U+1150A */
  0x0000,   /* U+1150B */
  0x0000,   /* U+1150C */
  0x0000,   /* U+1150D */
  0x0000,   /* U+1150E */
  0x0000,   /* U+1150F */
  0x0000,   /* U+11510 */
  0x0000,   /* U+11511 */
  0x0000,   /* U+11512 */
  0x0000,   /* U+11513 */
  0x0000,   /* U+11514 */
  0x0000,   /* U+11515 */
  0x0000,   /* U+11516 */
  0x0000,   /* U+11517 */
  0x0000,   /* U+11518 */
  0x0000,   /* U+11519 */
  0x0000,   /* U+1151A */
  0x0000,   /* U+1151B */
  0x0000,   /* U+1151C */
  0x0000,   /* U+1151D */
  0x0000,   /* U+1151E */
  0x0000,   /* U+1151F */
  0x0000,   /* U+11520 */
  0x0000,   /* U+11521 */
  0x0000,   /* U+11522 */
  0x0000,   /* U+11523 */
  0x0000,   /* U+11524 */
  0x0000,   /* U+11525 */
  0x0000,   /* U+11526 */
  0x0000,   /* U+11527 */
  0x0000,   /* U+11528 */
  0x0000,   /* U+11529 */
  0x0000,   /* U+1152A */
  0x0000,   /* U+1152B */
  0x0000,   /* U+1152C */
  0x0000,   /* U+1152D */
  0x0000,   /* U+1152E */
  0x0000,   /* U+1152F */
  0x0000,   /* U+11530 */
  0x0000,   /* U+11531 */
  0x0000,   /* U+11532 */
  0x0000,   /* U+11533 */
  0x0000,   /* U+11534 */
  0x0000,   /* U+11535 */
  0x0000,   /* U+11536 */
  0x0000,   /* U+11537 */
  0x0000,   /* U+11538 */
  0x0000,   /* U+11539 */
  0x0000,   /* U+1153A */
  0x0000,   /* U+1153B */
  0x0000,   /* U+1153C */
  0x0000,   /* U+1153D */
  0x0000,   /* U+1153E */
  0x0000,   /* U+1153F */
  0x0000,   /* U+11540 */
  0x0000,   /* U+11541 */
  0x0000,   /* U+11542 */
  0x0000,   /* U+11543 */
  0x0000,   /* U+11544 */
  0x0000,   /* U+11545 */
  0x0000,   /* U+11546 */
  0x0000,   /* U+11547 */
  0x0000,   /* U+11548 */
  0x0000,   /* U+11549 */
  0x0000,   /* U+1154A */
  0x0000,   /* U+1154B */
  0x0000,   /* U+1154C */
  0x0000,   /* U+1154D */
  0x0000,   /* U+1154E */
  0x0000,   /* U+1154F */
  0x0000,   /* U+11550 */
  0x0000,   /* U+11551 */
  0x0000,   /* U+11552 */
  0x0000,   /* U+11553 */
  0x0000,   /* U+11554 */
  0x0000,   /* U+11555 */
  0x0000,   /* U+11556 */
  0x0000,   /* U+11557 */
  0x0000,   /* U+11558 */
  0x0000,   /* U+11559 */
  0x0000,   /* U+1155A */
  0x0000,   /* U+1155B */
  0x0000,   /* U+1155C */
  0x0000,   /* U+1155D */
  0x0000,   /* U+1155E */
  0x0000,   /* U+1155F */
  0x0000,   /* U+11560 */
  0x0000,   /* U+11561 */
  0x0000,   /* U+11562 */
  0x0000,   /* U+11563 */
  0x0000,   /* U+11564 */
  0x0000,   /* U+11565 */
  0x0000,   /* U+11566 */
  0x0000,   /* U+11567 */
  0x0000,   /* U+11568 */
  0x0000,   /* U+11569 */
  0x0000,   /* U+1156A */
  0x0000,   /* U+1156B */
  0x0000,   /* U+1156C */
  0x0000,   /* U+1156D */
  0x0000,   /* U+1156E */
  0x0000,   /* U+1156F */
  0x0000,   /* U+11570 */
  0x0000,   /* U+11571 */
  0x0000,   /* U+11572 */
  0x0000,   /* U+11573 */
  0x0000,   /* U+11574 */
  0x0000,   /* U+11575 */
  0x0000,   /* U+11576 */
  0x0000,   /* U+11577 */
  0x0000,   /* U+11578 */
  0x0000,   /* U+11579 */
  0x0000,   /* U+1157A */
  0x0000,   /* U+1157B */
  0x0000,   /* U+1157C */
  0x0000,   /* U+1157D */
  0x0000,   /* U+1157E */
  0x0000,   /* U+1157F */
  0x0000,   /* U+11580 */
  0x0000,   /* U+11581 */
  0x0000,   /* U+11582 */
  0x0000,   /* U+11583 */
  0x0000,   /* U+11584 */
  0x0000,   /* U+11585 */
  0x0000,   /* U+11586 */
  0x0000,   /* U+11587 */
  0x0000,   /* U+11588 */
  0x0000,   /* U+11589 */
  0x0000,   /* U+1158A */
  0x0000,   /* U+1158B */
  0x0000,   /* U+1158C */
  0x0000,   /* U+1158D */
  0x0000,   /* U+1158E */
  0x0000,   /* U+1158F */
  0x0000,   /* U+11590 */
  0x0000,   /* U+11591 */
  0x0000,   /* U+11592 */
  0x0000,   /* U+11593 */
  0x0000,   /* U+11594 */
  0x0000,   /* U+11595 */
  0x0000,   /* U+11596 */
  0x0000,   /* U+11597 */
  0x0000,   /* U+11598 */
  0x0000,   /* U+11599 */
  0x0000,   /* U+1159A */
  0x0000,   /* U+1159B */
  0x0000,   /* U+1159C */
  0x0000,   /* U+1159D */
  0x0000,   /* U+1159E */
  0x0000,   /* U+1159F */
  0x0000,   /* U+115A0 */
  0x0000,   /* U+115A1 */
  0x0000,   /* U+115A2 */
  0x0000,   /* U+115A3 */
  0x0000,   /* U+115A4 */
  0x0000,   /* U+115A5 */
  0x0000,   /* U+115A6 */
  0x0000,   /* U+115A7 */
  0x0000,   /* U+115A8 */
  0x0000,   /* U+115A9 */
  0x0000,   /* U+115AA */
  0x0000,   /* U+115AB */
  0x0000,   /* U+115AC */
  0x0000,   /* U+115AD */
  0x0000,   /* U+115AE */
  0x0000,   /* U+115AF */
  0x0000,   /* U+115B0 */
  0x0000,   /* U+115B1 */
  0x0000,   /* U+115B2 */
  0x0000,   /* U+115B3 */
  0x0000,   /* U+115B4 */
  0x0000,   /* U+115B5 */
  0x0000,   /* U+115B6 */
  0x0000,   /* U+115B7 */
  0x0000,   /* U+115B8 */
  0x0000,   /* U+115B9 */
  0x0000,   /* U+115BA */
  0x0000,   /* U+115BB */
  0x0000,   /* U+115BC */
  0x0000,   /* U+115BD */
  0x0000,   /* U+115BE */
  0x0000,   /* U+115BF */
  0x0000,   /* U+115C0 */
  0x0000,   /* U+115C1 */
  0x0000,   /* U+115C2 */
  0x0000,   /* U+115C3 */
  0x0000,   /* U+115C4 */
  0x0000,   /* U+115C5 */
  0x0000,   /* U+115C6 */
  0x0000,   /* U+115C7 */
  0x0000,   /* U+115C8 */
  0x0000,   /* U+115C9 */
  0x0000,   /* U+115CA */
  0x0000,   /* U+115CB */
  0x0000,   /* U+115CC */
  0x0000,   /* U+115CD */
  0x0000,   /* U+115CE */
  0x0000,   /* U+115CF */
  0x0000,   /* U+115D0 */
  0x0000,   /* U+115D1 */
  0x0000,   /* U+115D2 */
  0x0000,   /* U+115D3 */
  0x0000,   /* U+115D4 */
  0x0000,   /* U+115D5 */
  0x0000,   /* U+115D6 */
  0x0000,   /* U+115D7 */
  0x0110,   /* U+115D8 */
  0x0111,   /* U+115D9 */
  0x0110,   /* U+115DA */
  0x0110,   /* U+115DB */
  0x0110,   /* U+115DC */
  0x0110,   /* U+115DD */
  0x0000,   /* U+115DE */
  0x0000,   /* U+115DF */
  0x0000,   /* U+115E0 */
  0x0000,   /* U+115E1 */
  0x0000,   /* U+115E2 */
  0x0000,   /* U+115E3 */
  0x0000,   /* U+115E4 */
  0x0000,   /* U+115E5 */
  0x0000,   /* U+115E6 */
  0x0000,   /* U+115E7 */
  0x0000,   /* U+115E8 */
  0x0000,   /* U+115E9 */
  0x0000,   /* U+115EA */
  0x0000,   /* U+115EB */
  0x0000,   /* U+115EC */
  0x0000,   /* U+115ED */
  0x0000,   /* U+115EE */
  0x0000,   /* U+115EF */
  0x0000,   /* U+115F0 */
  0x0000,   /* U+115F1 */
  0x0000,   /* U+115F2 */
  0x0000,   /* U+115F3 */
  0x0000,   /* U+115F4 */
  0x0000,   /* U+115F5 */
  0x0000,   /* U+115F6 */
  0x0000,   /* U+115F7 */
  0x0000,   /* U+115F8 */
  0x0000,   /* U+115F9 */
  0x0000,   /* U+115FA */
  0x0000,   /* U+115FB */
  0x0000,   /* U+115FC */
  0x0000,   /* U+115FD */
  0x0000,   /* U+115FE */
  0x0000,   /* U+115FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11500 */
  0x0000,   /* U+11501 */
  0x0000,   /* U+11502 */
  0x0000,   /* U+11503 */
  0x0000,   /* U+11504 */
  0x0000,   /* U+11505 */
  0x0000,   /* U+11506 */
  0x0000,   /* U+11507 */
  0x0000,   /* U+11508 */
  0x0000,   /* U+11509 */
  0x0000,   /* U+1150A */
  0x0000,   /* U+1150B */
  0x0000,   /* U+1150C */
  0x0000,   /* U+1150D */
  0x0000,   /* U+1150E */
  0x0000,   /* U+1150F */
  0x0000,   /* U+11510 */
  0x0000,   /* U+11511 */
  0x0000,   /* U+11512 */
  0x0000,   /* U+11513 */
  0x0000,   /* U+11514 */
  0x0000,   /* U+11515 */
  0x0000,   /* U+11516 */
  0x0000,   /* U+11517 */
  0x0000,   /* U+11518 */
  0x0000,   /* U+11519 */
  0x0000,   /* U+1151A */
  0x0000,   /* U+1151B */
  0x0000,   /* U+1151C */
  0x0000,   /* U+1151D */
  0x0000,   /* U+1151E */
  0x0000,   /* U+1151F */
  0x0000,   /* U+11520 */
  0x0000,   /* U+11521 */
  0x0000,   /* U+11522 */
  0x0000,   /* U+11523 */
  0x0000,   /* U+11524 */
  0x0000,   /* U+11525 */
  0x0000,   /* U+11526 */
  0x0000,   /* U+11527 */
  0x0000,   /* U+11528 */
  0x0000,   /* U+11529 */
  0x0000,   /* U+1152A */
  0x0000,   /* U+1152B */
  0x0000,   /* U+1152C */
  0x0000,   /* U+1152D */
  0x0000,   /* U+1152E */
  0x0000,   /* U+1152F */
  0x0000,   /* U+11530 */
  0x0000,   /* U+11531 */
  0x0000,   /* U+11532 */
  0x0000,   /* U+11533 */
  0x0000,   /* U+11534 */
  0x0000,   /* U+11535 */
  0x0000,   /* U+11536 */
  0x0000,   /* U+11537 */
  0x0000,   /* U+11538 */
  0x0000,   /* U+11539 */
  0x0000,   /* U+1153A */
  0x0000,   /* U+1153B */
  0x0000,   /* U+1153C */
  0x0000,   /* U+1153D */
  0x0000,   /* U+1153E */
  0x0000,   /* U+1153F */
  0x0000,   /* U+11540 */
  0x0000,   /* U+11541 */
  0x0000,   /* U+11542 */
  0x0000,   /* U+11543 */
  0x0000,   /* U+11544 */
  0x0000,   /* U+11545 */
  0x0000,   /* U+11546 */
  0x0000,   /* U+11547 */
  0x0000,   /* U+11548 */
  0x0000,   /* U+11549 */
  0x0000,   /* U+1154A */
  0x0000,   /* U+1154B */
  0x0000,   /* U+1154C */
  0x0000,   /* U+1154D */
  0x0000,   /* U+1154E */
  0x0000,   /* U+1154F */
  0x0000,   /* U+11550 */
  0x0000,   /* U+11551 */
  0x0000,   /* U+11552 */
  0x0000,   /* U+11553 */
  0x0000,   /* U+11554 */
  0x0000,   /* U+11555 */
  0x0000,   /* U+11556 */
  0x0000,   /* U+11557 */
  0x0000,   /* U+11558 */
  0x0000,   /* U+11559 */
  0x0000,   /* U+1155A */
  0x0000,   /* U+1155B */
  0x0000,   /* U+1155C */
  0x0000,   /* U+1155D */
  0x0000,   /* U+1155E */
  0x0000,   /* U+1155F */
  0x0000,   /* U+11560 */
  0x0000,   /* U+11561 */
  0x0000,   /* U+11562 */
  0x0000,   /* U+11563 */
  0x0000,   /* U+11564 */
  0x0000,   /* U+11565 */
  0x0000,   /* U+11566 */
  0x0000,   /* U+11567 */
  0x0000,   /* U+11568 */
  0x0000,   /* U+11569 */
  0x0000,   /* U+1156A */
  0x0000,   /* U+1156B */
  0x0000,   /* U+1156C */
  0x0000,   /* U+1156D */
  0x0000,   /* U+1156E */
  0x0000,   /* U+1156F */
  0x0000,   /* U+11570 */
  0x0000,   /* U+11571 */
  0x0000,   /* U+11572 */
  0x0000,   /* U+11573 */
  0x0000,   /* U+11574 */
  0x0000,   /* U+11575 */
  0x0000,   /* U+11576 */
  0x0000,   /* U+11577 */
  0x0000,   /* U+11578 */
  0x0000,   /* U+11579 */
  0x0000,   /* U+1157A */
  0x0000,   /* U+1157B */
  0x0000,   /* U+1157C */
  0x0000,   /* U+1157D */
  0x0000,   /* U+1157E */
  0x0000,   /* U+1157F */
  0x0000,   /* U+11580 */
  0x0000,   /* U+11581 */
  0x0000,   /* U+11582 */
  0x0000,   /* U+11583 */
  0x0000,   /* U+11584 */
  0x0000,   /* U+11585 */
  0x0000,   /* U+11586 */
  0x0000,   /* U+11587 */
  0x0000,   /* U+11588 */
  0x0000,   /* U+11589 */
  0x0000,   /* U+1158A */
  0x0000,   /* U+1158B */
  0x0000,   /* U+1158C */
  0x0000,   /* U+1158D */
  0x0000,   /* U+1158E */
  0x0000,   /* U+1158F */
  0x0000,   /* U+11590 */
  0x0000,   /* U+11591 */
  0x0000,   /* U+11592 */
  0x0000,   /* U+11593 */
  0x0000,   /* U+11594 */
  0x0000,   /* U+11595 */
  0x0000,   /* U+11596 */
  0x0000,   /* U+11597 */
  0x0000,   /* U+11598 */
  0x0000,   /* U+11599 */
  0x0000,   /* U+1159A */
  0x0000,   /* U+1159B */
  0x0000,   /* U+1159C */
  0x0000,   /* U+1159D */
  0x0000,   /* U+1159E */
  0x0000,   /* U+1159F */
  0x0000,   /* U+115A0 */
  0x0000,   /* U+115A1 */
  0x0000,   /* U+115A2 */
  0x0000,   /* U+115A3 */
  0x0000,   /* U+115A4 */
  0x0000,   /* U+115A5 */
  0x0000,   /* U+115A6 */
  0x0000,   /* U+115A7 */
  0x0000,   /* U+115A8 */
  0x0000,   /* U+115A9 */
  0x0000,   /* U+115AA */
  0x0000,   /* U+115AB */
  0x0000,   /* U+115AC */
  0x0000,   /* U+115AD */
  0x0000,   /* U+115AE */
  0x0000,   /* U+115AF */
  0x0000,   /* U+115B0 */
  0x0000,   /* U+115B1 */
  0x0000,   /* U+115B2 */
  0x0000,   /* U+115B3 */
  0x0000,   /* U+115B4 */
  0x0000,   /* U+115B5 */
  0x0000,   /* U+115B6 */
  0x0000,   /* U+115B7 */
  0x0000,   /* U+115B8 */
  0x0000,   /* U+115B9 */
  0x0000,   /* U+115BA */
  0x0000,   /* U+115BB */
  0x0000,   /* U+115BC */
  0x0000,   /* U+115BD */
  0x0000,   /* U+115BE */
  0x0000,   /* U+115BF */
  0x0000,   /* U+115C0 */
  0x0000,   /* U+115C1 */
  0x0000,   /* U+115C2 */
  0x0000,   /* U+115C3 */
  0x0000,   /* U+115C4 */
  0x0000,   /* U+115C5 */
  0x0000,   /* U+115C6 */
  0x0000,   /* U+115C7 */
  0x0000,   /* U+115C8 */
  0x0000,   /* U+115C9 */
  0x0000,   /* U+115CA */
  0x0000,   /* U+115CB */
  0x0000,   /* U+115CC */
  0x0000,   /* U+115CD */
  0x0000,   /* U+115CE */
  0x0000,   /* U+115CF */
  0x0000,   /* U+115D0 */
  0x0000,   /* U+115D1 */
  0x0000,   /* U+115D2 */
  0x0000,   /* U+115D3 */
  0x0000,   /* U+115D4 */
  0x0000,   /* U+115D5 */
  0x0000,   /* U+115D6 */
  0x0000,   /* U+115D7 */
  0x0004,   /* U+115D8 */
  0x0004,   /* U+115D9 */
  0x0004,   /* U+115DA */
  0x0004,   /* U+115DB */
  0x0004,   /* U+115DC */
  0x0004,   /* U+115DD */
  0x0000,   /* U+115DE */
  0x0000,   /* U+115DF */
  0x0000,   /* U+115E0 */
  0x0000,   /* U+115E1 */
  0x0000,   /* U+115E2 */
  0x0000,   /* U+115E3 */
  0x0000,   /* U+115E4 */
  0x0000,   /* U+115E5 */
  0x0000,   /* U+115E6 */
  0x0000,   /* U+115E7 */
  0x0000,   /* U+115E8 */
  0x0000,   /* U+115E9 */
  0x0000,   /* U+115EA */
  0x0000,   /* U+115EB */
  0x0000,   /* U+115EC */
  0x0000,   /* U+115ED */
  0x0000,   /* U+115EE */
  0x0000,   /* U+115EF */
  0x0000,   /* U+115F0 */
  0x0000,   /* U+115F1 */
  0x0000,   /* U+115F2 */
  0x0000,   /* U+115F3 */
  0x0000,   /* U+115F4 */
  0x0000,   /* U+115F5 */
  0x0000,   /* U+115F6 */
  0x0000,   /* U+115F7 */
  0x0000,   /* U+115F8 */
  0x0000,   /* U+115F9 */
  0x0000,   /* U+115FA */
  0x0000,   /* U+115FB */
  0x0000,   /* U+115FC */
  0x0000,   /* U+115FD */
  0x0000,   /* U+115FE */
  0x0000,   /* U+115FF */
};

uint16 uca900_p116[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2BFD,   /* U+11600 */
  0x2BFE,   /* U+11601 */
  0x2BFF,   /* U+11602 */
  0x2C00,   /* U+11603 */
  0x2C01,   /* U+11604 */
  0x2C02,   /* U+11605 */
  0x2C03,   /* U+11606 */
  0x2C04,   /* U+11607 */
  0x2C05,   /* U+11608 */
  0x2C06,   /* U+11609 */
  0x2C07,   /* U+1160A */
  0x2C08,   /* U+1160B */
  0x2C09,   /* U+1160C */
  0x2C0A,   /* U+1160D */
  0x2C0B,   /* U+1160E */
  0x2C0C,   /* U+1160F */
  0x2C0D,   /* U+11610 */
  0x2C0E,   /* U+11611 */
  0x2C0F,   /* U+11612 */
  0x2C10,   /* U+11613 */
  0x2C11,   /* U+11614 */
  0x2C12,   /* U+11615 */
  0x2C13,   /* U+11616 */
  0x2C14,   /* U+11617 */
  0x2C15,   /* U+11618 */
  0x2C16,   /* U+11619 */
  0x2C17,   /* U+1161A */
  0x2C18,   /* U+1161B */
  0x2C19,   /* U+1161C */
  0x2C1A,   /* U+1161D */
  0x2C1B,   /* U+1161E */
  0x2C1C,   /* U+1161F */
  0x2C1D,   /* U+11620 */
  0x2C1E,   /* U+11621 */
  0x2C1F,   /* U+11622 */
  0x2C20,   /* U+11623 */
  0x2C21,   /* U+11624 */
  0x2C22,   /* U+11625 */
  0x2C23,   /* U+11626 */
  0x2C24,   /* U+11627 */
  0x2C25,   /* U+11628 */
  0x2C26,   /* U+11629 */
  0x2C27,   /* U+1162A */
  0x2C28,   /* U+1162B */
  0x2C29,   /* U+1162C */
  0x2C2A,   /* U+1162D */
  0x2C2B,   /* U+1162E */
  0x2C2C,   /* U+1162F */
  0x2C2D,   /* U+11630 */
  0x2C2E,   /* U+11631 */
  0x2C2F,   /* U+11632 */
  0x2C30,   /* U+11633 */
  0x2C31,   /* U+11634 */
  0x2C32,   /* U+11635 */
  0x2C33,   /* U+11636 */
  0x2C34,   /* U+11637 */
  0x2C35,   /* U+11638 */
  0x2C36,   /* U+11639 */
  0x2C37,   /* U+1163A */
  0x2C38,   /* U+1163B */
  0x2C39,   /* U+1163C */
  0x0000,   /* U+1163D */
  0x0000,   /* U+1163E */
  0x2C3A,   /* U+1163F */
  0x0000,   /* U+11640 */
  0x02BA,   /* U+11641 */
  0x02BB,   /* U+11642 */
  0x0476,   /* U+11643 */
  0x2C3B,   /* U+11644 */
  0xFBC2,   /* U+11645 */
  0xFBC2,   /* U+11646 */
  0xFBC2,   /* U+11647 */
  0xFBC2,   /* U+11648 */
  0xFBC2,   /* U+11649 */
  0xFBC2,   /* U+1164A */
  0xFBC2,   /* U+1164B */
  0xFBC2,   /* U+1164C */
  0xFBC2,   /* U+1164D */
  0xFBC2,   /* U+1164E */
  0xFBC2,   /* U+1164F */
  0x1C3D,   /* U+11650 */
  0x1C3E,   /* U+11651 */
  0x1C3F,   /* U+11652 */
  0x1C40,   /* U+11653 */
  0x1C41,   /* U+11654 */
  0x1C42,   /* U+11655 */
  0x1C43,   /* U+11656 */
  0x1C44,   /* U+11657 */
  0x1C45,   /* U+11658 */
  0x1C46,   /* U+11659 */
  0xFBC2,   /* U+1165A */
  0xFBC2,   /* U+1165B */
  0xFBC2,   /* U+1165C */
  0xFBC2,   /* U+1165D */
  0xFBC2,   /* U+1165E */
  0xFBC2,   /* U+1165F */
  0x03DA,   /* U+11660 */
  0x03DB,   /* U+11661 */
  0x03DC,   /* U+11662 */
  0x03DD,   /* U+11663 */
  0x03DE,   /* U+11664 */
  0x03DF,   /* U+11665 */
  0x03E0,   /* U+11666 */
  0x03E1,   /* U+11667 */
  0x03E2,   /* U+11668 */
  0x03E3,   /* U+11669 */
  0x03E4,   /* U+1166A */
  0x03E5,   /* U+1166B */
  0x03E6,   /* U+1166C */
  0xFBC2,   /* U+1166D */
  0xFBC2,   /* U+1166E */
  0xFBC2,   /* U+1166F */
  0xFBC2,   /* U+11670 */
  0xFBC2,   /* U+11671 */
  0xFBC2,   /* U+11672 */
  0xFBC2,   /* U+11673 */
  0xFBC2,   /* U+11674 */
  0xFBC2,   /* U+11675 */
  0xFBC2,   /* U+11676 */
  0xFBC2,   /* U+11677 */
  0xFBC2,   /* U+11678 */
  0xFBC2,   /* U+11679 */
  0xFBC2,   /* U+1167A */
  0xFBC2,   /* U+1167B */
  0xFBC2,   /* U+1167C */
  0xFBC2,   /* U+1167D */
  0xFBC2,   /* U+1167E */
  0xFBC2,   /* U+1167F */
  0x2C3C,   /* U+11680 */
  0x2C3D,   /* U+11681 */
  0x2C3E,   /* U+11682 */
  0x2C3F,   /* U+11683 */
  0x2C40,   /* U+11684 */
  0x2C41,   /* U+11685 */
  0x2C42,   /* U+11686 */
  0x2C43,   /* U+11687 */
  0x2C44,   /* U+11688 */
  0x2C45,   /* U+11689 */
  0x2C49,   /* U+1168A */
  0x2C4A,   /* U+1168B */
  0x2C4B,   /* U+1168C */
  0x2C4C,   /* U+1168D */
  0x2C4D,   /* U+1168E */
  0x2C4E,   /* U+1168F */
  0x2C4F,   /* U+11690 */
  0x2C50,   /* U+11691 */
  0x2C51,   /* U+11692 */
  0x2C52,   /* U+11693 */
  0x2C53,   /* U+11694 */
  0x2C54,   /* U+11695 */
  0x2C55,   /* U+11696 */
  0x2C56,   /* U+11697 */
  0x2C57,   /* U+11698 */
  0x2C58,   /* U+11699 */
  0x2C59,   /* U+1169A */
  0x2C5A,   /* U+1169B */
  0x2C5B,   /* U+1169C */
  0x2C5C,   /* U+1169D */
  0x2C5D,   /* U+1169E */
  0x2C5E,   /* U+1169F */
  0x2C5F,   /* U+116A0 */
  0x2C60,   /* U+116A1 */
  0x2C61,   /* U+116A2 */
  0x2C62,   /* U+116A3 */
  0x2C63,   /* U+116A4 */
  0x2C64,   /* U+116A5 */
  0x2C65,   /* U+116A6 */
  0x2C47,   /* U+116A7 */
  0x2C46,   /* U+116A8 */
  0x2C48,   /* U+116A9 */
  0x2C66,   /* U+116AA */
  0x0000,   /* U+116AB */
  0x0000,   /* U+116AC */
  0x2C67,   /* U+116AD */
  0x2C68,   /* U+116AE */
  0x2C69,   /* U+116AF */
  0x2C6A,   /* U+116B0 */
  0x2C6B,   /* U+116B1 */
  0x2C6C,   /* U+116B2 */
  0x2C6D,   /* U+116B3 */
  0x2C6E,   /* U+116B4 */
  0x2C6F,   /* U+116B5 */
  0x2C70,   /* U+116B6 */
  0x0000,   /* U+116B7 */
  0xFBC2,   /* U+116B8 */
  0xFBC2,   /* U+116B9 */
  0xFBC2,   /* U+116BA */
  0xFBC2,   /* U+116BB */
  0xFBC2,   /* U+116BC */
  0xFBC2,   /* U+116BD */
  0xFBC2,   /* U+116BE */
  0xFBC2,   /* U+116BF */
  0x1C3D,   /* U+116C0 */
  0x1C3E,   /* U+116C1 */
  0x1C3F,   /* U+116C2 */
  0x1C40,   /* U+116C3 */
  0x1C41,   /* U+116C4 */
  0x1C42,   /* U+116C5 */
  0x1C43,   /* U+116C6 */
  0x1C44,   /* U+116C7 */
  0x1C45,   /* U+116C8 */
  0x1C46,   /* U+116C9 */
  0xFBC2,   /* U+116CA */
  0xFBC2,   /* U+116CB */
  0xFBC2,   /* U+116CC */
  0xFBC2,   /* U+116CD */
  0xFBC2,   /* U+116CE */
  0xFBC2,   /* U+116CF */
  0xFBC2,   /* U+116D0 */
  0xFBC2,   /* U+116D1 */
  0xFBC2,   /* U+116D2 */
  0xFBC2,   /* U+116D3 */
  0xFBC2,   /* U+116D4 */
  0xFBC2,   /* U+116D5 */
  0xFBC2,   /* U+116D6 */
  0xFBC2,   /* U+116D7 */
  0xFBC2,   /* U+116D8 */
  0xFBC2,   /* U+116D9 */
  0xFBC2,   /* U+116DA */
  0xFBC2,   /* U+116DB */
  0xFBC2,   /* U+116DC */
  0xFBC2,   /* U+116DD */
  0xFBC2,   /* U+116DE */
  0xFBC2,   /* U+116DF */
  0xFBC2,   /* U+116E0 */
  0xFBC2,   /* U+116E1 */
  0xFBC2,   /* U+116E2 */
  0xFBC2,   /* U+116E3 */
  0xFBC2,   /* U+116E4 */
  0xFBC2,   /* U+116E5 */
  0xFBC2,   /* U+116E6 */
  0xFBC2,   /* U+116E7 */
  0xFBC2,   /* U+116E8 */
  0xFBC2,   /* U+116E9 */
  0xFBC2,   /* U+116EA */
  0xFBC2,   /* U+116EB */
  0xFBC2,   /* U+116EC */
  0xFBC2,   /* U+116ED */
  0xFBC2,   /* U+116EE */
  0xFBC2,   /* U+116EF */
  0xFBC2,   /* U+116F0 */
  0xFBC2,   /* U+116F1 */
  0xFBC2,   /* U+116F2 */
  0xFBC2,   /* U+116F3 */
  0xFBC2,   /* U+116F4 */
  0xFBC2,   /* U+116F5 */
  0xFBC2,   /* U+116F6 */
  0xFBC2,   /* U+116F7 */
  0xFBC2,   /* U+116F8 */
  0xFBC2,   /* U+116F9 */
  0xFBC2,   /* U+116FA */
  0xFBC2,   /* U+116FB */
  0xFBC2,   /* U+116FC */
  0xFBC2,   /* U+116FD */
  0xFBC2,   /* U+116FE */
  0xFBC2,   /* U+116FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11600 */
  0x0020,   /* U+11601 */
  0x0020,   /* U+11602 */
  0x0020,   /* U+11603 */
  0x0020,   /* U+11604 */
  0x0020,   /* U+11605 */
  0x0020,   /* U+11606 */
  0x0020,   /* U+11607 */
  0x0020,   /* U+11608 */
  0x0020,   /* U+11609 */
  0x0020,   /* U+1160A */
  0x0020,   /* U+1160B */
  0x0020,   /* U+1160C */
  0x0020,   /* U+1160D */
  0x0020,   /* U+1160E */
  0x0020,   /* U+1160F */
  0x0020,   /* U+11610 */
  0x0020,   /* U+11611 */
  0x0020,   /* U+11612 */
  0x0020,   /* U+11613 */
  0x0020,   /* U+11614 */
  0x0020,   /* U+11615 */
  0x0020,   /* U+11616 */
  0x0020,   /* U+11617 */
  0x0020,   /* U+11618 */
  0x0020,   /* U+11619 */
  0x0020,   /* U+1161A */
  0x0020,   /* U+1161B */
  0x0020,   /* U+1161C */
  0x0020,   /* U+1161D */
  0x0020,   /* U+1161E */
  0x0020,   /* U+1161F */
  0x0020,   /* U+11620 */
  0x0020,   /* U+11621 */
  0x0020,   /* U+11622 */
  0x0020,   /* U+11623 */
  0x0020,   /* U+11624 */
  0x0020,   /* U+11625 */
  0x0020,   /* U+11626 */
  0x0020,   /* U+11627 */
  0x0020,   /* U+11628 */
  0x0020,   /* U+11629 */
  0x0020,   /* U+1162A */
  0x0020,   /* U+1162B */
  0x0020,   /* U+1162C */
  0x0020,   /* U+1162D */
  0x0020,   /* U+1162E */
  0x0020,   /* U+1162F */
  0x0020,   /* U+11630 */
  0x0020,   /* U+11631 */
  0x0020,   /* U+11632 */
  0x0020,   /* U+11633 */
  0x0020,   /* U+11634 */
  0x0020,   /* U+11635 */
  0x0020,   /* U+11636 */
  0x0020,   /* U+11637 */
  0x0020,   /* U+11638 */
  0x0020,   /* U+11639 */
  0x0020,   /* U+1163A */
  0x0020,   /* U+1163B */
  0x0020,   /* U+1163C */
  0x00C4,   /* U+1163D */
  0x00C5,   /* U+1163E */
  0x0020,   /* U+1163F */
  0x00C3,   /* U+11640 */
  0x0020,   /* U+11641 */
  0x0020,   /* U+11642 */
  0x0020,   /* U+11643 */
  0x0020,   /* U+11644 */
  0x0020,   /* U+11645 */
  0x0020,   /* U+11646 */
  0x0020,   /* U+11647 */
  0x0020,   /* U+11648 */
  0x0020,   /* U+11649 */
  0x0020,   /* U+1164A */
  0x0020,   /* U+1164B */
  0x0020,   /* U+1164C */
  0x0020,   /* U+1164D */
  0x0020,   /* U+1164E */
  0x0020,   /* U+1164F */
  0x0020,   /* U+11650 */
  0x0020,   /* U+11651 */
  0x0020,   /* U+11652 */
  0x0020,   /* U+11653 */
  0x0020,   /* U+11654 */
  0x0020,   /* U+11655 */
  0x0020,   /* U+11656 */
  0x0020,   /* U+11657 */
  0x0020,   /* U+11658 */
  0x0020,   /* U+11659 */
  0x0020,   /* U+1165A */
  0x0020,   /* U+1165B */
  0x0020,   /* U+1165C */
  0x0020,   /* U+1165D */
  0x0020,   /* U+1165E */
  0x0020,   /* U+1165F */
  0x0020,   /* U+11660 */
  0x0020,   /* U+11661 */
  0x0020,   /* U+11662 */
  0x0020,   /* U+11663 */
  0x0020,   /* U+11664 */
  0x0020,   /* U+11665 */
  0x0020,   /* U+11666 */
  0x0020,   /* U+11667 */
  0x0020,   /* U+11668 */
  0x0020,   /* U+11669 */
  0x0020,   /* U+1166A */
  0x0020,   /* U+1166B */
  0x0020,   /* U+1166C */
  0x0020,   /* U+1166D */
  0x0020,   /* U+1166E */
  0x0020,   /* U+1166F */
  0x0020,   /* U+11670 */
  0x0020,   /* U+11671 */
  0x0020,   /* U+11672 */
  0x0020,   /* U+11673 */
  0x0020,   /* U+11674 */
  0x0020,   /* U+11675 */
  0x0020,   /* U+11676 */
  0x0020,   /* U+11677 */
  0x0020,   /* U+11678 */
  0x0020,   /* U+11679 */
  0x0020,   /* U+1167A */
  0x0020,   /* U+1167B */
  0x0020,   /* U+1167C */
  0x0020,   /* U+1167D */
  0x0020,   /* U+1167E */
  0x0020,   /* U+1167F */
  0x0020,   /* U+11680 */
  0x0020,   /* U+11681 */
  0x0020,   /* U+11682 */
  0x0020,   /* U+11683 */
  0x0020,   /* U+11684 */
  0x0020,   /* U+11685 */
  0x0020,   /* U+11686 */
  0x0020,   /* U+11687 */
  0x0020,   /* U+11688 */
  0x0020,   /* U+11689 */
  0x0020,   /* U+1168A */
  0x0020,   /* U+1168B */
  0x0020,   /* U+1168C */
  0x0020,   /* U+1168D */
  0x0020,   /* U+1168E */
  0x0020,   /* U+1168F */
  0x0020,   /* U+11690 */
  0x0020,   /* U+11691 */
  0x0020,   /* U+11692 */
  0x0020,   /* U+11693 */
  0x0020,   /* U+11694 */
  0x0020,   /* U+11695 */
  0x0020,   /* U+11696 */
  0x0020,   /* U+11697 */
  0x0020,   /* U+11698 */
  0x0020,   /* U+11699 */
  0x0020,   /* U+1169A */
  0x0020,   /* U+1169B */
  0x0020,   /* U+1169C */
  0x0020,   /* U+1169D */
  0x0020,   /* U+1169E */
  0x0020,   /* U+1169F */
  0x0020,   /* U+116A0 */
  0x0020,   /* U+116A1 */
  0x0020,   /* U+116A2 */
  0x0020,   /* U+116A3 */
  0x0020,   /* U+116A4 */
  0x0020,   /* U+116A5 */
  0x0020,   /* U+116A6 */
  0x0020,   /* U+116A7 */
  0x0020,   /* U+116A8 */
  0x0020,   /* U+116A9 */
  0x0020,   /* U+116AA */
  0x00C4,   /* U+116AB */
  0x00C5,   /* U+116AC */
  0x0020,   /* U+116AD */
  0x0020,   /* U+116AE */
  0x0020,   /* U+116AF */
  0x0020,   /* U+116B0 */
  0x0020,   /* U+116B1 */
  0x0020,   /* U+116B2 */
  0x0020,   /* U+116B3 */
  0x0020,   /* U+116B4 */
  0x0020,   /* U+116B5 */
  0x0020,   /* U+116B6 */
  0x00C2,   /* U+116B7 */
  0x0020,   /* U+116B8 */
  0x0020,   /* U+116B9 */
  0x0020,   /* U+116BA */
  0x0020,   /* U+116BB */
  0x0020,   /* U+116BC */
  0x0020,   /* U+116BD */
  0x0020,   /* U+116BE */
  0x0020,   /* U+116BF */
  0x0020,   /* U+116C0 */
  0x0020,   /* U+116C1 */
  0x0020,   /* U+116C2 */
  0x0020,   /* U+116C3 */
  0x0020,   /* U+116C4 */
  0x0020,   /* U+116C5 */
  0x0020,   /* U+116C6 */
  0x0020,   /* U+116C7 */
  0x0020,   /* U+116C8 */
  0x0020,   /* U+116C9 */
  0x0020,   /* U+116CA */
  0x0020,   /* U+116CB */
  0x0020,   /* U+116CC */
  0x0020,   /* U+116CD */
  0x0020,   /* U+116CE */
  0x0020,   /* U+116CF */
  0x0020,   /* U+116D0 */
  0x0020,   /* U+116D1 */
  0x0020,   /* U+116D2 */
  0x0020,   /* U+116D3 */
  0x0020,   /* U+116D4 */
  0x0020,   /* U+116D5 */
  0x0020,   /* U+116D6 */
  0x0020,   /* U+116D7 */
  0x0020,   /* U+116D8 */
  0x0020,   /* U+116D9 */
  0x0020,   /* U+116DA */
  0x0020,   /* U+116DB */
  0x0020,   /* U+116DC */
  0x0020,   /* U+116DD */
  0x0020,   /* U+116DE */
  0x0020,   /* U+116DF */
  0x0020,   /* U+116E0 */
  0x0020,   /* U+116E1 */
  0x0020,   /* U+116E2 */
  0x0020,   /* U+116E3 */
  0x0020,   /* U+116E4 */
  0x0020,   /* U+116E5 */
  0x0020,   /* U+116E6 */
  0x0020,   /* U+116E7 */
  0x0020,   /* U+116E8 */
  0x0020,   /* U+116E9 */
  0x0020,   /* U+116EA */
  0x0020,   /* U+116EB */
  0x0020,   /* U+116EC */
  0x0020,   /* U+116ED */
  0x0020,   /* U+116EE */
  0x0020,   /* U+116EF */
  0x0020,   /* U+116F0 */
  0x0020,   /* U+116F1 */
  0x0020,   /* U+116F2 */
  0x0020,   /* U+116F3 */
  0x0020,   /* U+116F4 */
  0x0020,   /* U+116F5 */
  0x0020,   /* U+116F6 */
  0x0020,   /* U+116F7 */
  0x0020,   /* U+116F8 */
  0x0020,   /* U+116F9 */
  0x0020,   /* U+116FA */
  0x0020,   /* U+116FB */
  0x0020,   /* U+116FC */
  0x0020,   /* U+116FD */
  0x0020,   /* U+116FE */
  0x0020,   /* U+116FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11600 */
  0x0002,   /* U+11601 */
  0x0002,   /* U+11602 */
  0x0002,   /* U+11603 */
  0x0002,   /* U+11604 */
  0x0002,   /* U+11605 */
  0x0002,   /* U+11606 */
  0x0002,   /* U+11607 */
  0x0002,   /* U+11608 */
  0x0002,   /* U+11609 */
  0x0002,   /* U+1160A */
  0x0002,   /* U+1160B */
  0x0002,   /* U+1160C */
  0x0002,   /* U+1160D */
  0x0002,   /* U+1160E */
  0x0002,   /* U+1160F */
  0x0002,   /* U+11610 */
  0x0002,   /* U+11611 */
  0x0002,   /* U+11612 */
  0x0002,   /* U+11613 */
  0x0002,   /* U+11614 */
  0x0002,   /* U+11615 */
  0x0002,   /* U+11616 */
  0x0002,   /* U+11617 */
  0x0002,   /* U+11618 */
  0x0002,   /* U+11619 */
  0x0002,   /* U+1161A */
  0x0002,   /* U+1161B */
  0x0002,   /* U+1161C */
  0x0002,   /* U+1161D */
  0x0002,   /* U+1161E */
  0x0002,   /* U+1161F */
  0x0002,   /* U+11620 */
  0x0002,   /* U+11621 */
  0x0002,   /* U+11622 */
  0x0002,   /* U+11623 */
  0x0002,   /* U+11624 */
  0x0002,   /* U+11625 */
  0x0002,   /* U+11626 */
  0x0002,   /* U+11627 */
  0x0002,   /* U+11628 */
  0x0002,   /* U+11629 */
  0x0002,   /* U+1162A */
  0x0002,   /* U+1162B */
  0x0002,   /* U+1162C */
  0x0002,   /* U+1162D */
  0x0002,   /* U+1162E */
  0x0002,   /* U+1162F */
  0x0002,   /* U+11630 */
  0x0002,   /* U+11631 */
  0x0002,   /* U+11632 */
  0x0002,   /* U+11633 */
  0x0002,   /* U+11634 */
  0x0002,   /* U+11635 */
  0x0002,   /* U+11636 */
  0x0002,   /* U+11637 */
  0x0002,   /* U+11638 */
  0x0002,   /* U+11639 */
  0x0002,   /* U+1163A */
  0x0002,   /* U+1163B */
  0x0002,   /* U+1163C */
  0x0002,   /* U+1163D */
  0x0002,   /* U+1163E */
  0x0002,   /* U+1163F */
  0x0002,   /* U+11640 */
  0x0002,   /* U+11641 */
  0x0002,   /* U+11642 */
  0x0002,   /* U+11643 */
  0x0002,   /* U+11644 */
  0x0002,   /* U+11645 */
  0x0002,   /* U+11646 */
  0x0002,   /* U+11647 */
  0x0002,   /* U+11648 */
  0x0002,   /* U+11649 */
  0x0002,   /* U+1164A */
  0x0002,   /* U+1164B */
  0x0002,   /* U+1164C */
  0x0002,   /* U+1164D */
  0x0002,   /* U+1164E */
  0x0002,   /* U+1164F */
  0x0002,   /* U+11650 */
  0x0002,   /* U+11651 */
  0x0002,   /* U+11652 */
  0x0002,   /* U+11653 */
  0x0002,   /* U+11654 */
  0x0002,   /* U+11655 */
  0x0002,   /* U+11656 */
  0x0002,   /* U+11657 */
  0x0002,   /* U+11658 */
  0x0002,   /* U+11659 */
  0x0002,   /* U+1165A */
  0x0002,   /* U+1165B */
  0x0002,   /* U+1165C */
  0x0002,   /* U+1165D */
  0x0002,   /* U+1165E */
  0x0002,   /* U+1165F */
  0x0002,   /* U+11660 */
  0x0002,   /* U+11661 */
  0x0002,   /* U+11662 */
  0x0002,   /* U+11663 */
  0x0002,   /* U+11664 */
  0x0002,   /* U+11665 */
  0x0002,   /* U+11666 */
  0x0002,   /* U+11667 */
  0x0002,   /* U+11668 */
  0x0002,   /* U+11669 */
  0x0002,   /* U+1166A */
  0x0002,   /* U+1166B */
  0x0002,   /* U+1166C */
  0x0002,   /* U+1166D */
  0x0002,   /* U+1166E */
  0x0002,   /* U+1166F */
  0x0002,   /* U+11670 */
  0x0002,   /* U+11671 */
  0x0002,   /* U+11672 */
  0x0002,   /* U+11673 */
  0x0002,   /* U+11674 */
  0x0002,   /* U+11675 */
  0x0002,   /* U+11676 */
  0x0002,   /* U+11677 */
  0x0002,   /* U+11678 */
  0x0002,   /* U+11679 */
  0x0002,   /* U+1167A */
  0x0002,   /* U+1167B */
  0x0002,   /* U+1167C */
  0x0002,   /* U+1167D */
  0x0002,   /* U+1167E */
  0x0002,   /* U+1167F */
  0x0002,   /* U+11680 */
  0x0002,   /* U+11681 */
  0x0002,   /* U+11682 */
  0x0002,   /* U+11683 */
  0x0002,   /* U+11684 */
  0x0002,   /* U+11685 */
  0x0002,   /* U+11686 */
  0x0002,   /* U+11687 */
  0x0002,   /* U+11688 */
  0x0002,   /* U+11689 */
  0x0002,   /* U+1168A */
  0x0002,   /* U+1168B */
  0x0002,   /* U+1168C */
  0x0002,   /* U+1168D */
  0x0002,   /* U+1168E */
  0x0002,   /* U+1168F */
  0x0002,   /* U+11690 */
  0x0002,   /* U+11691 */
  0x0002,   /* U+11692 */
  0x0002,   /* U+11693 */
  0x0002,   /* U+11694 */
  0x0002,   /* U+11695 */
  0x0002,   /* U+11696 */
  0x0002,   /* U+11697 */
  0x0002,   /* U+11698 */
  0x0002,   /* U+11699 */
  0x0002,   /* U+1169A */
  0x0002,   /* U+1169B */
  0x0002,   /* U+1169C */
  0x0002,   /* U+1169D */
  0x0002,   /* U+1169E */
  0x0002,   /* U+1169F */
  0x0002,   /* U+116A0 */
  0x0002,   /* U+116A1 */
  0x0002,   /* U+116A2 */
  0x0002,   /* U+116A3 */
  0x0002,   /* U+116A4 */
  0x0002,   /* U+116A5 */
  0x0002,   /* U+116A6 */
  0x0002,   /* U+116A7 */
  0x0002,   /* U+116A8 */
  0x0002,   /* U+116A9 */
  0x0002,   /* U+116AA */
  0x0002,   /* U+116AB */
  0x0002,   /* U+116AC */
  0x0002,   /* U+116AD */
  0x0002,   /* U+116AE */
  0x0002,   /* U+116AF */
  0x0002,   /* U+116B0 */
  0x0002,   /* U+116B1 */
  0x0002,   /* U+116B2 */
  0x0002,   /* U+116B3 */
  0x0002,   /* U+116B4 */
  0x0002,   /* U+116B5 */
  0x0002,   /* U+116B6 */
  0x0002,   /* U+116B7 */
  0x0002,   /* U+116B8 */
  0x0002,   /* U+116B9 */
  0x0002,   /* U+116BA */
  0x0002,   /* U+116BB */
  0x0002,   /* U+116BC */
  0x0002,   /* U+116BD */
  0x0002,   /* U+116BE */
  0x0002,   /* U+116BF */
  0x0002,   /* U+116C0 */
  0x0002,   /* U+116C1 */
  0x0002,   /* U+116C2 */
  0x0002,   /* U+116C3 */
  0x0002,   /* U+116C4 */
  0x0002,   /* U+116C5 */
  0x0002,   /* U+116C6 */
  0x0002,   /* U+116C7 */
  0x0002,   /* U+116C8 */
  0x0002,   /* U+116C9 */
  0x0002,   /* U+116CA */
  0x0002,   /* U+116CB */
  0x0002,   /* U+116CC */
  0x0002,   /* U+116CD */
  0x0002,   /* U+116CE */
  0x0002,   /* U+116CF */
  0x0002,   /* U+116D0 */
  0x0002,   /* U+116D1 */
  0x0002,   /* U+116D2 */
  0x0002,   /* U+116D3 */
  0x0002,   /* U+116D4 */
  0x0002,   /* U+116D5 */
  0x0002,   /* U+116D6 */
  0x0002,   /* U+116D7 */
  0x0002,   /* U+116D8 */
  0x0002,   /* U+116D9 */
  0x0002,   /* U+116DA */
  0x0002,   /* U+116DB */
  0x0002,   /* U+116DC */
  0x0002,   /* U+116DD */
  0x0002,   /* U+116DE */
  0x0002,   /* U+116DF */
  0x0002,   /* U+116E0 */
  0x0002,   /* U+116E1 */
  0x0002,   /* U+116E2 */
  0x0002,   /* U+116E3 */
  0x0002,   /* U+116E4 */
  0x0002,   /* U+116E5 */
  0x0002,   /* U+116E6 */
  0x0002,   /* U+116E7 */
  0x0002,   /* U+116E8 */
  0x0002,   /* U+116E9 */
  0x0002,   /* U+116EA */
  0x0002,   /* U+116EB */
  0x0002,   /* U+116EC */
  0x0002,   /* U+116ED */
  0x0002,   /* U+116EE */
  0x0002,   /* U+116EF */
  0x0002,   /* U+116F0 */
  0x0002,   /* U+116F1 */
  0x0002,   /* U+116F2 */
  0x0002,   /* U+116F3 */
  0x0002,   /* U+116F4 */
  0x0002,   /* U+116F5 */
  0x0002,   /* U+116F6 */
  0x0002,   /* U+116F7 */
  0x0002,   /* U+116F8 */
  0x0002,   /* U+116F9 */
  0x0002,   /* U+116FA */
  0x0002,   /* U+116FB */
  0x0002,   /* U+116FC */
  0x0002,   /* U+116FD */
  0x0002,   /* U+116FE */
  0x0002,   /* U+116FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11600 */
  0x0000,   /* U+11601 */
  0x0000,   /* U+11602 */
  0x0000,   /* U+11603 */
  0x0000,   /* U+11604 */
  0x0000,   /* U+11605 */
  0x0000,   /* U+11606 */
  0x0000,   /* U+11607 */
  0x0000,   /* U+11608 */
  0x0000,   /* U+11609 */
  0x0000,   /* U+1160A */
  0x0000,   /* U+1160B */
  0x0000,   /* U+1160C */
  0x0000,   /* U+1160D */
  0x0000,   /* U+1160E */
  0x0000,   /* U+1160F */
  0x0000,   /* U+11610 */
  0x0000,   /* U+11611 */
  0x0000,   /* U+11612 */
  0x0000,   /* U+11613 */
  0x0000,   /* U+11614 */
  0x0000,   /* U+11615 */
  0x0000,   /* U+11616 */
  0x0000,   /* U+11617 */
  0x0000,   /* U+11618 */
  0x0000,   /* U+11619 */
  0x0000,   /* U+1161A */
  0x0000,   /* U+1161B */
  0x0000,   /* U+1161C */
  0x0000,   /* U+1161D */
  0x0000,   /* U+1161E */
  0x0000,   /* U+1161F */
  0x0000,   /* U+11620 */
  0x0000,   /* U+11621 */
  0x0000,   /* U+11622 */
  0x0000,   /* U+11623 */
  0x0000,   /* U+11624 */
  0x0000,   /* U+11625 */
  0x0000,   /* U+11626 */
  0x0000,   /* U+11627 */
  0x0000,   /* U+11628 */
  0x0000,   /* U+11629 */
  0x0000,   /* U+1162A */
  0x0000,   /* U+1162B */
  0x0000,   /* U+1162C */
  0x0000,   /* U+1162D */
  0x0000,   /* U+1162E */
  0x0000,   /* U+1162F */
  0x0000,   /* U+11630 */
  0x0000,   /* U+11631 */
  0x0000,   /* U+11632 */
  0x0000,   /* U+11633 */
  0x0000,   /* U+11634 */
  0x0000,   /* U+11635 */
  0x0000,   /* U+11636 */
  0x0000,   /* U+11637 */
  0x0000,   /* U+11638 */
  0x0000,   /* U+11639 */
  0x0000,   /* U+1163A */
  0x0000,   /* U+1163B */
  0x0000,   /* U+1163C */
  0x0000,   /* U+1163D */
  0x0000,   /* U+1163E */
  0x0000,   /* U+1163F */
  0x0000,   /* U+11640 */
  0x0000,   /* U+11641 */
  0x0000,   /* U+11642 */
  0x0000,   /* U+11643 */
  0x0000,   /* U+11644 */
  0x9645,   /* U+11645 */
  0x9646,   /* U+11646 */
  0x9647,   /* U+11647 */
  0x9648,   /* U+11648 */
  0x9649,   /* U+11649 */
  0x964A,   /* U+1164A */
  0x964B,   /* U+1164B */
  0x964C,   /* U+1164C */
  0x964D,   /* U+1164D */
  0x964E,   /* U+1164E */
  0x964F,   /* U+1164F */
  0x0000,   /* U+11650 */
  0x0000,   /* U+11651 */
  0x0000,   /* U+11652 */
  0x0000,   /* U+11653 */
  0x0000,   /* U+11654 */
  0x0000,   /* U+11655 */
  0x0000,   /* U+11656 */
  0x0000,   /* U+11657 */
  0x0000,   /* U+11658 */
  0x0000,   /* U+11659 */
  0x965A,   /* U+1165A */
  0x965B,   /* U+1165B */
  0x965C,   /* U+1165C */
  0x965D,   /* U+1165D */
  0x965E,   /* U+1165E */
  0x965F,   /* U+1165F */
  0x0000,   /* U+11660 */
  0x0000,   /* U+11661 */
  0x0000,   /* U+11662 */
  0x0000,   /* U+11663 */
  0x0000,   /* U+11664 */
  0x0000,   /* U+11665 */
  0x0000,   /* U+11666 */
  0x0000,   /* U+11667 */
  0x0000,   /* U+11668 */
  0x0000,   /* U+11669 */
  0x0000,   /* U+1166A */
  0x0000,   /* U+1166B */
  0x0000,   /* U+1166C */
  0x966D,   /* U+1166D */
  0x966E,   /* U+1166E */
  0x966F,   /* U+1166F */
  0x9670,   /* U+11670 */
  0x9671,   /* U+11671 */
  0x9672,   /* U+11672 */
  0x9673,   /* U+11673 */
  0x9674,   /* U+11674 */
  0x9675,   /* U+11675 */
  0x9676,   /* U+11676 */
  0x9677,   /* U+11677 */
  0x9678,   /* U+11678 */
  0x9679,   /* U+11679 */
  0x967A,   /* U+1167A */
  0x967B,   /* U+1167B */
  0x967C,   /* U+1167C */
  0x967D,   /* U+1167D */
  0x967E,   /* U+1167E */
  0x967F,   /* U+1167F */
  0x0000,   /* U+11680 */
  0x0000,   /* U+11681 */
  0x0000,   /* U+11682 */
  0x0000,   /* U+11683 */
  0x0000,   /* U+11684 */
  0x0000,   /* U+11685 */
  0x0000,   /* U+11686 */
  0x0000,   /* U+11687 */
  0x0000,   /* U+11688 */
  0x0000,   /* U+11689 */
  0x0000,   /* U+1168A */
  0x0000,   /* U+1168B */
  0x0000,   /* U+1168C */
  0x0000,   /* U+1168D */
  0x0000,   /* U+1168E */
  0x0000,   /* U+1168F */
  0x0000,   /* U+11690 */
  0x0000,   /* U+11691 */
  0x0000,   /* U+11692 */
  0x0000,   /* U+11693 */
  0x0000,   /* U+11694 */
  0x0000,   /* U+11695 */
  0x0000,   /* U+11696 */
  0x0000,   /* U+11697 */
  0x0000,   /* U+11698 */
  0x0000,   /* U+11699 */
  0x0000,   /* U+1169A */
  0x0000,   /* U+1169B */
  0x0000,   /* U+1169C */
  0x0000,   /* U+1169D */
  0x0000,   /* U+1169E */
  0x0000,   /* U+1169F */
  0x0000,   /* U+116A0 */
  0x0000,   /* U+116A1 */
  0x0000,   /* U+116A2 */
  0x0000,   /* U+116A3 */
  0x0000,   /* U+116A4 */
  0x0000,   /* U+116A5 */
  0x0000,   /* U+116A6 */
  0x0000,   /* U+116A7 */
  0x0000,   /* U+116A8 */
  0x0000,   /* U+116A9 */
  0x0000,   /* U+116AA */
  0x0000,   /* U+116AB */
  0x0000,   /* U+116AC */
  0x0000,   /* U+116AD */
  0x0000,   /* U+116AE */
  0x0000,   /* U+116AF */
  0x0000,   /* U+116B0 */
  0x0000,   /* U+116B1 */
  0x0000,   /* U+116B2 */
  0x0000,   /* U+116B3 */
  0x0000,   /* U+116B4 */
  0x0000,   /* U+116B5 */
  0x0000,   /* U+116B6 */
  0x0000,   /* U+116B7 */
  0x96B8,   /* U+116B8 */
  0x96B9,   /* U+116B9 */
  0x96BA,   /* U+116BA */
  0x96BB,   /* U+116BB */
  0x96BC,   /* U+116BC */
  0x96BD,   /* U+116BD */
  0x96BE,   /* U+116BE */
  0x96BF,   /* U+116BF */
  0x0000,   /* U+116C0 */
  0x0000,   /* U+116C1 */
  0x0000,   /* U+116C2 */
  0x0000,   /* U+116C3 */
  0x0000,   /* U+116C4 */
  0x0000,   /* U+116C5 */
  0x0000,   /* U+116C6 */
  0x0000,   /* U+116C7 */
  0x0000,   /* U+116C8 */
  0x0000,   /* U+116C9 */
  0x96CA,   /* U+116CA */
  0x96CB,   /* U+116CB */
  0x96CC,   /* U+116CC */
  0x96CD,   /* U+116CD */
  0x96CE,   /* U+116CE */
  0x96CF,   /* U+116CF */
  0x96D0,   /* U+116D0 */
  0x96D1,   /* U+116D1 */
  0x96D2,   /* U+116D2 */
  0x96D3,   /* U+116D3 */
  0x96D4,   /* U+116D4 */
  0x96D5,   /* U+116D5 */
  0x96D6,   /* U+116D6 */
  0x96D7,   /* U+116D7 */
  0x96D8,   /* U+116D8 */
  0x96D9,   /* U+116D9 */
  0x96DA,   /* U+116DA */
  0x96DB,   /* U+116DB */
  0x96DC,   /* U+116DC */
  0x96DD,   /* U+116DD */
  0x96DE,   /* U+116DE */
  0x96DF,   /* U+116DF */
  0x96E0,   /* U+116E0 */
  0x96E1,   /* U+116E1 */
  0x96E2,   /* U+116E2 */
  0x96E3,   /* U+116E3 */
  0x96E4,   /* U+116E4 */
  0x96E5,   /* U+116E5 */
  0x96E6,   /* U+116E6 */
  0x96E7,   /* U+116E7 */
  0x96E8,   /* U+116E8 */
  0x96E9,   /* U+116E9 */
  0x96EA,   /* U+116EA */
  0x96EB,   /* U+116EB */
  0x96EC,   /* U+116EC */
  0x96ED,   /* U+116ED */
  0x96EE,   /* U+116EE */
  0x96EF,   /* U+116EF */
  0x96F0,   /* U+116F0 */
  0x96F1,   /* U+116F1 */
  0x96F2,   /* U+116F2 */
  0x96F3,   /* U+116F3 */
  0x96F4,   /* U+116F4 */
  0x96F5,   /* U+116F5 */
  0x96F6,   /* U+116F6 */
  0x96F7,   /* U+116F7 */
  0x96F8,   /* U+116F8 */
  0x96F9,   /* U+116F9 */
  0x96FA,   /* U+116FA */
  0x96FB,   /* U+116FB */
  0x96FC,   /* U+116FC */
  0x96FD,   /* U+116FD */
  0x96FE,   /* U+116FE */
  0x96FF,   /* U+116FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11600 */
  0x0000,   /* U+11601 */
  0x0000,   /* U+11602 */
  0x0000,   /* U+11603 */
  0x0000,   /* U+11604 */
  0x0000,   /* U+11605 */
  0x0000,   /* U+11606 */
  0x0000,   /* U+11607 */
  0x0000,   /* U+11608 */
  0x0000,   /* U+11609 */
  0x0000,   /* U+1160A */
  0x0000,   /* U+1160B */
  0x0000,   /* U+1160C */
  0x0000,   /* U+1160D */
  0x0000,   /* U+1160E */
  0x0000,   /* U+1160F */
  0x0000,   /* U+11610 */
  0x0000,   /* U+11611 */
  0x0000,   /* U+11612 */
  0x0000,   /* U+11613 */
  0x0000,   /* U+11614 */
  0x0000,   /* U+11615 */
  0x0000,   /* U+11616 */
  0x0000,   /* U+11617 */
  0x0000,   /* U+11618 */
  0x0000,   /* U+11619 */
  0x0000,   /* U+1161A */
  0x0000,   /* U+1161B */
  0x0000,   /* U+1161C */
  0x0000,   /* U+1161D */
  0x0000,   /* U+1161E */
  0x0000,   /* U+1161F */
  0x0000,   /* U+11620 */
  0x0000,   /* U+11621 */
  0x0000,   /* U+11622 */
  0x0000,   /* U+11623 */
  0x0000,   /* U+11624 */
  0x0000,   /* U+11625 */
  0x0000,   /* U+11626 */
  0x0000,   /* U+11627 */
  0x0000,   /* U+11628 */
  0x0000,   /* U+11629 */
  0x0000,   /* U+1162A */
  0x0000,   /* U+1162B */
  0x0000,   /* U+1162C */
  0x0000,   /* U+1162D */
  0x0000,   /* U+1162E */
  0x0000,   /* U+1162F */
  0x0000,   /* U+11630 */
  0x0000,   /* U+11631 */
  0x0000,   /* U+11632 */
  0x0000,   /* U+11633 */
  0x0000,   /* U+11634 */
  0x0000,   /* U+11635 */
  0x0000,   /* U+11636 */
  0x0000,   /* U+11637 */
  0x0000,   /* U+11638 */
  0x0000,   /* U+11639 */
  0x0000,   /* U+1163A */
  0x0000,   /* U+1163B */
  0x0000,   /* U+1163C */
  0x0000,   /* U+1163D */
  0x0000,   /* U+1163E */
  0x0000,   /* U+1163F */
  0x0000,   /* U+11640 */
  0x0000,   /* U+11641 */
  0x0000,   /* U+11642 */
  0x0000,   /* U+11643 */
  0x0000,   /* U+11644 */
  0x0000,   /* U+11645 */
  0x0000,   /* U+11646 */
  0x0000,   /* U+11647 */
  0x0000,   /* U+11648 */
  0x0000,   /* U+11649 */
  0x0000,   /* U+1164A */
  0x0000,   /* U+1164B */
  0x0000,   /* U+1164C */
  0x0000,   /* U+1164D */
  0x0000,   /* U+1164E */
  0x0000,   /* U+1164F */
  0x0000,   /* U+11650 */
  0x0000,   /* U+11651 */
  0x0000,   /* U+11652 */
  0x0000,   /* U+11653 */
  0x0000,   /* U+11654 */
  0x0000,   /* U+11655 */
  0x0000,   /* U+11656 */
  0x0000,   /* U+11657 */
  0x0000,   /* U+11658 */
  0x0000,   /* U+11659 */
  0x0000,   /* U+1165A */
  0x0000,   /* U+1165B */
  0x0000,   /* U+1165C */
  0x0000,   /* U+1165D */
  0x0000,   /* U+1165E */
  0x0000,   /* U+1165F */
  0x0000,   /* U+11660 */
  0x0000,   /* U+11661 */
  0x0000,   /* U+11662 */
  0x0000,   /* U+11663 */
  0x0000,   /* U+11664 */
  0x0000,   /* U+11665 */
  0x0000,   /* U+11666 */
  0x0000,   /* U+11667 */
  0x0000,   /* U+11668 */
  0x0000,   /* U+11669 */
  0x0000,   /* U+1166A */
  0x0000,   /* U+1166B */
  0x0000,   /* U+1166C */
  0x0000,   /* U+1166D */
  0x0000,   /* U+1166E */
  0x0000,   /* U+1166F */
  0x0000,   /* U+11670 */
  0x0000,   /* U+11671 */
  0x0000,   /* U+11672 */
  0x0000,   /* U+11673 */
  0x0000,   /* U+11674 */
  0x0000,   /* U+11675 */
  0x0000,   /* U+11676 */
  0x0000,   /* U+11677 */
  0x0000,   /* U+11678 */
  0x0000,   /* U+11679 */
  0x0000,   /* U+1167A */
  0x0000,   /* U+1167B */
  0x0000,   /* U+1167C */
  0x0000,   /* U+1167D */
  0x0000,   /* U+1167E */
  0x0000,   /* U+1167F */
  0x0000,   /* U+11680 */
  0x0000,   /* U+11681 */
  0x0000,   /* U+11682 */
  0x0000,   /* U+11683 */
  0x0000,   /* U+11684 */
  0x0000,   /* U+11685 */
  0x0000,   /* U+11686 */
  0x0000,   /* U+11687 */
  0x0000,   /* U+11688 */
  0x0000,   /* U+11689 */
  0x0000,   /* U+1168A */
  0x0000,   /* U+1168B */
  0x0000,   /* U+1168C */
  0x0000,   /* U+1168D */
  0x0000,   /* U+1168E */
  0x0000,   /* U+1168F */
  0x0000,   /* U+11690 */
  0x0000,   /* U+11691 */
  0x0000,   /* U+11692 */
  0x0000,   /* U+11693 */
  0x0000,   /* U+11694 */
  0x0000,   /* U+11695 */
  0x0000,   /* U+11696 */
  0x0000,   /* U+11697 */
  0x0000,   /* U+11698 */
  0x0000,   /* U+11699 */
  0x0000,   /* U+1169A */
  0x0000,   /* U+1169B */
  0x0000,   /* U+1169C */
  0x0000,   /* U+1169D */
  0x0000,   /* U+1169E */
  0x0000,   /* U+1169F */
  0x0000,   /* U+116A0 */
  0x0000,   /* U+116A1 */
  0x0000,   /* U+116A2 */
  0x0000,   /* U+116A3 */
  0x0000,   /* U+116A4 */
  0x0000,   /* U+116A5 */
  0x0000,   /* U+116A6 */
  0x0000,   /* U+116A7 */
  0x0000,   /* U+116A8 */
  0x0000,   /* U+116A9 */
  0x0000,   /* U+116AA */
  0x0000,   /* U+116AB */
  0x0000,   /* U+116AC */
  0x0000,   /* U+116AD */
  0x0000,   /* U+116AE */
  0x0000,   /* U+116AF */
  0x0000,   /* U+116B0 */
  0x0000,   /* U+116B1 */
  0x0000,   /* U+116B2 */
  0x0000,   /* U+116B3 */
  0x0000,   /* U+116B4 */
  0x0000,   /* U+116B5 */
  0x0000,   /* U+116B6 */
  0x0000,   /* U+116B7 */
  0x0000,   /* U+116B8 */
  0x0000,   /* U+116B9 */
  0x0000,   /* U+116BA */
  0x0000,   /* U+116BB */
  0x0000,   /* U+116BC */
  0x0000,   /* U+116BD */
  0x0000,   /* U+116BE */
  0x0000,   /* U+116BF */
  0x0000,   /* U+116C0 */
  0x0000,   /* U+116C1 */
  0x0000,   /* U+116C2 */
  0x0000,   /* U+116C3 */
  0x0000,   /* U+116C4 */
  0x0000,   /* U+116C5 */
  0x0000,   /* U+116C6 */
  0x0000,   /* U+116C7 */
  0x0000,   /* U+116C8 */
  0x0000,   /* U+116C9 */
  0x0000,   /* U+116CA */
  0x0000,   /* U+116CB */
  0x0000,   /* U+116CC */
  0x0000,   /* U+116CD */
  0x0000,   /* U+116CE */
  0x0000,   /* U+116CF */
  0x0000,   /* U+116D0 */
  0x0000,   /* U+116D1 */
  0x0000,   /* U+116D2 */
  0x0000,   /* U+116D3 */
  0x0000,   /* U+116D4 */
  0x0000,   /* U+116D5 */
  0x0000,   /* U+116D6 */
  0x0000,   /* U+116D7 */
  0x0000,   /* U+116D8 */
  0x0000,   /* U+116D9 */
  0x0000,   /* U+116DA */
  0x0000,   /* U+116DB */
  0x0000,   /* U+116DC */
  0x0000,   /* U+116DD */
  0x0000,   /* U+116DE */
  0x0000,   /* U+116DF */
  0x0000,   /* U+116E0 */
  0x0000,   /* U+116E1 */
  0x0000,   /* U+116E2 */
  0x0000,   /* U+116E3 */
  0x0000,   /* U+116E4 */
  0x0000,   /* U+116E5 */
  0x0000,   /* U+116E6 */
  0x0000,   /* U+116E7 */
  0x0000,   /* U+116E8 */
  0x0000,   /* U+116E9 */
  0x0000,   /* U+116EA */
  0x0000,   /* U+116EB */
  0x0000,   /* U+116EC */
  0x0000,   /* U+116ED */
  0x0000,   /* U+116EE */
  0x0000,   /* U+116EF */
  0x0000,   /* U+116F0 */
  0x0000,   /* U+116F1 */
  0x0000,   /* U+116F2 */
  0x0000,   /* U+116F3 */
  0x0000,   /* U+116F4 */
  0x0000,   /* U+116F5 */
  0x0000,   /* U+116F6 */
  0x0000,   /* U+116F7 */
  0x0000,   /* U+116F8 */
  0x0000,   /* U+116F9 */
  0x0000,   /* U+116FA */
  0x0000,   /* U+116FB */
  0x0000,   /* U+116FC */
  0x0000,   /* U+116FD */
  0x0000,   /* U+116FE */
  0x0000,   /* U+116FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11600 */
  0x0000,   /* U+11601 */
  0x0000,   /* U+11602 */
  0x0000,   /* U+11603 */
  0x0000,   /* U+11604 */
  0x0000,   /* U+11605 */
  0x0000,   /* U+11606 */
  0x0000,   /* U+11607 */
  0x0000,   /* U+11608 */
  0x0000,   /* U+11609 */
  0x0000,   /* U+1160A */
  0x0000,   /* U+1160B */
  0x0000,   /* U+1160C */
  0x0000,   /* U+1160D */
  0x0000,   /* U+1160E */
  0x0000,   /* U+1160F */
  0x0000,   /* U+11610 */
  0x0000,   /* U+11611 */
  0x0000,   /* U+11612 */
  0x0000,   /* U+11613 */
  0x0000,   /* U+11614 */
  0x0000,   /* U+11615 */
  0x0000,   /* U+11616 */
  0x0000,   /* U+11617 */
  0x0000,   /* U+11618 */
  0x0000,   /* U+11619 */
  0x0000,   /* U+1161A */
  0x0000,   /* U+1161B */
  0x0000,   /* U+1161C */
  0x0000,   /* U+1161D */
  0x0000,   /* U+1161E */
  0x0000,   /* U+1161F */
  0x0000,   /* U+11620 */
  0x0000,   /* U+11621 */
  0x0000,   /* U+11622 */
  0x0000,   /* U+11623 */
  0x0000,   /* U+11624 */
  0x0000,   /* U+11625 */
  0x0000,   /* U+11626 */
  0x0000,   /* U+11627 */
  0x0000,   /* U+11628 */
  0x0000,   /* U+11629 */
  0x0000,   /* U+1162A */
  0x0000,   /* U+1162B */
  0x0000,   /* U+1162C */
  0x0000,   /* U+1162D */
  0x0000,   /* U+1162E */
  0x0000,   /* U+1162F */
  0x0000,   /* U+11630 */
  0x0000,   /* U+11631 */
  0x0000,   /* U+11632 */
  0x0000,   /* U+11633 */
  0x0000,   /* U+11634 */
  0x0000,   /* U+11635 */
  0x0000,   /* U+11636 */
  0x0000,   /* U+11637 */
  0x0000,   /* U+11638 */
  0x0000,   /* U+11639 */
  0x0000,   /* U+1163A */
  0x0000,   /* U+1163B */
  0x0000,   /* U+1163C */
  0x0000,   /* U+1163D */
  0x0000,   /* U+1163E */
  0x0000,   /* U+1163F */
  0x0000,   /* U+11640 */
  0x0000,   /* U+11641 */
  0x0000,   /* U+11642 */
  0x0000,   /* U+11643 */
  0x0000,   /* U+11644 */
  0x0000,   /* U+11645 */
  0x0000,   /* U+11646 */
  0x0000,   /* U+11647 */
  0x0000,   /* U+11648 */
  0x0000,   /* U+11649 */
  0x0000,   /* U+1164A */
  0x0000,   /* U+1164B */
  0x0000,   /* U+1164C */
  0x0000,   /* U+1164D */
  0x0000,   /* U+1164E */
  0x0000,   /* U+1164F */
  0x0000,   /* U+11650 */
  0x0000,   /* U+11651 */
  0x0000,   /* U+11652 */
  0x0000,   /* U+11653 */
  0x0000,   /* U+11654 */
  0x0000,   /* U+11655 */
  0x0000,   /* U+11656 */
  0x0000,   /* U+11657 */
  0x0000,   /* U+11658 */
  0x0000,   /* U+11659 */
  0x0000,   /* U+1165A */
  0x0000,   /* U+1165B */
  0x0000,   /* U+1165C */
  0x0000,   /* U+1165D */
  0x0000,   /* U+1165E */
  0x0000,   /* U+1165F */
  0x0000,   /* U+11660 */
  0x0000,   /* U+11661 */
  0x0000,   /* U+11662 */
  0x0000,   /* U+11663 */
  0x0000,   /* U+11664 */
  0x0000,   /* U+11665 */
  0x0000,   /* U+11666 */
  0x0000,   /* U+11667 */
  0x0000,   /* U+11668 */
  0x0000,   /* U+11669 */
  0x0000,   /* U+1166A */
  0x0000,   /* U+1166B */
  0x0000,   /* U+1166C */
  0x0000,   /* U+1166D */
  0x0000,   /* U+1166E */
  0x0000,   /* U+1166F */
  0x0000,   /* U+11670 */
  0x0000,   /* U+11671 */
  0x0000,   /* U+11672 */
  0x0000,   /* U+11673 */
  0x0000,   /* U+11674 */
  0x0000,   /* U+11675 */
  0x0000,   /* U+11676 */
  0x0000,   /* U+11677 */
  0x0000,   /* U+11678 */
  0x0000,   /* U+11679 */
  0x0000,   /* U+1167A */
  0x0000,   /* U+1167B */
  0x0000,   /* U+1167C */
  0x0000,   /* U+1167D */
  0x0000,   /* U+1167E */
  0x0000,   /* U+1167F */
  0x0000,   /* U+11680 */
  0x0000,   /* U+11681 */
  0x0000,   /* U+11682 */
  0x0000,   /* U+11683 */
  0x0000,   /* U+11684 */
  0x0000,   /* U+11685 */
  0x0000,   /* U+11686 */
  0x0000,   /* U+11687 */
  0x0000,   /* U+11688 */
  0x0000,   /* U+11689 */
  0x0000,   /* U+1168A */
  0x0000,   /* U+1168B */
  0x0000,   /* U+1168C */
  0x0000,   /* U+1168D */
  0x0000,   /* U+1168E */
  0x0000,   /* U+1168F */
  0x0000,   /* U+11690 */
  0x0000,   /* U+11691 */
  0x0000,   /* U+11692 */
  0x0000,   /* U+11693 */
  0x0000,   /* U+11694 */
  0x0000,   /* U+11695 */
  0x0000,   /* U+11696 */
  0x0000,   /* U+11697 */
  0x0000,   /* U+11698 */
  0x0000,   /* U+11699 */
  0x0000,   /* U+1169A */
  0x0000,   /* U+1169B */
  0x0000,   /* U+1169C */
  0x0000,   /* U+1169D */
  0x0000,   /* U+1169E */
  0x0000,   /* U+1169F */
  0x0000,   /* U+116A0 */
  0x0000,   /* U+116A1 */
  0x0000,   /* U+116A2 */
  0x0000,   /* U+116A3 */
  0x0000,   /* U+116A4 */
  0x0000,   /* U+116A5 */
  0x0000,   /* U+116A6 */
  0x0000,   /* U+116A7 */
  0x0000,   /* U+116A8 */
  0x0000,   /* U+116A9 */
  0x0000,   /* U+116AA */
  0x0000,   /* U+116AB */
  0x0000,   /* U+116AC */
  0x0000,   /* U+116AD */
  0x0000,   /* U+116AE */
  0x0000,   /* U+116AF */
  0x0000,   /* U+116B0 */
  0x0000,   /* U+116B1 */
  0x0000,   /* U+116B2 */
  0x0000,   /* U+116B3 */
  0x0000,   /* U+116B4 */
  0x0000,   /* U+116B5 */
  0x0000,   /* U+116B6 */
  0x0000,   /* U+116B7 */
  0x0000,   /* U+116B8 */
  0x0000,   /* U+116B9 */
  0x0000,   /* U+116BA */
  0x0000,   /* U+116BB */
  0x0000,   /* U+116BC */
  0x0000,   /* U+116BD */
  0x0000,   /* U+116BE */
  0x0000,   /* U+116BF */
  0x0000,   /* U+116C0 */
  0x0000,   /* U+116C1 */
  0x0000,   /* U+116C2 */
  0x0000,   /* U+116C3 */
  0x0000,   /* U+116C4 */
  0x0000,   /* U+116C5 */
  0x0000,   /* U+116C6 */
  0x0000,   /* U+116C7 */
  0x0000,   /* U+116C8 */
  0x0000,   /* U+116C9 */
  0x0000,   /* U+116CA */
  0x0000,   /* U+116CB */
  0x0000,   /* U+116CC */
  0x0000,   /* U+116CD */
  0x0000,   /* U+116CE */
  0x0000,   /* U+116CF */
  0x0000,   /* U+116D0 */
  0x0000,   /* U+116D1 */
  0x0000,   /* U+116D2 */
  0x0000,   /* U+116D3 */
  0x0000,   /* U+116D4 */
  0x0000,   /* U+116D5 */
  0x0000,   /* U+116D6 */
  0x0000,   /* U+116D7 */
  0x0000,   /* U+116D8 */
  0x0000,   /* U+116D9 */
  0x0000,   /* U+116DA */
  0x0000,   /* U+116DB */
  0x0000,   /* U+116DC */
  0x0000,   /* U+116DD */
  0x0000,   /* U+116DE */
  0x0000,   /* U+116DF */
  0x0000,   /* U+116E0 */
  0x0000,   /* U+116E1 */
  0x0000,   /* U+116E2 */
  0x0000,   /* U+116E3 */
  0x0000,   /* U+116E4 */
  0x0000,   /* U+116E5 */
  0x0000,   /* U+116E6 */
  0x0000,   /* U+116E7 */
  0x0000,   /* U+116E8 */
  0x0000,   /* U+116E9 */
  0x0000,   /* U+116EA */
  0x0000,   /* U+116EB */
  0x0000,   /* U+116EC */
  0x0000,   /* U+116ED */
  0x0000,   /* U+116EE */
  0x0000,   /* U+116EF */
  0x0000,   /* U+116F0 */
  0x0000,   /* U+116F1 */
  0x0000,   /* U+116F2 */
  0x0000,   /* U+116F3 */
  0x0000,   /* U+116F4 */
  0x0000,   /* U+116F5 */
  0x0000,   /* U+116F6 */
  0x0000,   /* U+116F7 */
  0x0000,   /* U+116F8 */
  0x0000,   /* U+116F9 */
  0x0000,   /* U+116FA */
  0x0000,   /* U+116FB */
  0x0000,   /* U+116FC */
  0x0000,   /* U+116FD */
  0x0000,   /* U+116FE */
  0x0000,   /* U+116FF */
};

uint16 uca900_p117[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2C71,   /* U+11700 */
  0x2C72,   /* U+11701 */
  0x2C73,   /* U+11702 */
  0x2C74,   /* U+11703 */
  0x2C75,   /* U+11704 */
  0x2C75,   /* U+11705 */
  0x2C76,   /* U+11706 */
  0x2C77,   /* U+11707 */
  0x2C78,   /* U+11708 */
  0x2C79,   /* U+11709 */
  0x2C7A,   /* U+1170A */
  0x2C7B,   /* U+1170B */
  0x2C7C,   /* U+1170C */
  0x2C7D,   /* U+1170D */
  0x2C7E,   /* U+1170E */
  0x2C7F,   /* U+1170F */
  0x2C80,   /* U+11710 */
  0x2C81,   /* U+11711 */
  0x2C82,   /* U+11712 */
  0x2C83,   /* U+11713 */
  0x2C84,   /* U+11714 */
  0x2C85,   /* U+11715 */
  0x2C85,   /* U+11716 */
  0x2C86,   /* U+11717 */
  0x2C87,   /* U+11718 */
  0x2C88,   /* U+11719 */
  0xFBC2,   /* U+1171A */
  0xFBC2,   /* U+1171B */
  0xFBC2,   /* U+1171C */
  0x2C95,   /* U+1171D */
  0x2C96,   /* U+1171E */
  0x2C97,   /* U+1171F */
  0x2C89,   /* U+11720 */
  0x2C8A,   /* U+11721 */
  0x2C8B,   /* U+11722 */
  0x2C8C,   /* U+11723 */
  0x2C8D,   /* U+11724 */
  0x2C8E,   /* U+11725 */
  0x2C8F,   /* U+11726 */
  0x2C90,   /* U+11727 */
  0x2C91,   /* U+11728 */
  0x2C92,   /* U+11729 */
  0x2C93,   /* U+1172A */
  0x2C94,   /* U+1172B */
  0xFBC2,   /* U+1172C */
  0xFBC2,   /* U+1172D */
  0xFBC2,   /* U+1172E */
  0xFBC2,   /* U+1172F */
  0x1C3D,   /* U+11730 */
  0x1C3E,   /* U+11731 */
  0x1C3F,   /* U+11732 */
  0x1C40,   /* U+11733 */
  0x1C41,   /* U+11734 */
  0x1C42,   /* U+11735 */
  0x1C43,   /* U+11736 */
  0x1C44,   /* U+11737 */
  0x1C45,   /* U+11738 */
  0x1C46,   /* U+11739 */
  0x1B71,   /* U+1173A */
  0x1B72,   /* U+1173B */
  0x02BC,   /* U+1173C */
  0x02BD,   /* U+1173D */
  0x02E7,   /* U+1173E */
  0x0550,   /* U+1173F */
  0xFBC2,   /* U+11740 */
  0xFBC2,   /* U+11741 */
  0xFBC2,   /* U+11742 */
  0xFBC2,   /* U+11743 */
  0xFBC2,   /* U+11744 */
  0xFBC2,   /* U+11745 */
  0xFBC2,   /* U+11746 */
  0xFBC2,   /* U+11747 */
  0xFBC2,   /* U+11748 */
  0xFBC2,   /* U+11749 */
  0xFBC2,   /* U+1174A */
  0xFBC2,   /* U+1174B */
  0xFBC2,   /* U+1174C */
  0xFBC2,   /* U+1174D */
  0xFBC2,   /* U+1174E */
  0xFBC2,   /* U+1174F */
  0xFBC2,   /* U+11750 */
  0xFBC2,   /* U+11751 */
  0xFBC2,   /* U+11752 */
  0xFBC2,   /* U+11753 */
  0xFBC2,   /* U+11754 */
  0xFBC2,   /* U+11755 */
  0xFBC2,   /* U+11756 */
  0xFBC2,   /* U+11757 */
  0xFBC2,   /* U+11758 */
  0xFBC2,   /* U+11759 */
  0xFBC2,   /* U+1175A */
  0xFBC2,   /* U+1175B */
  0xFBC2,   /* U+1175C */
  0xFBC2,   /* U+1175D */
  0xFBC2,   /* U+1175E */
  0xFBC2,   /* U+1175F */
  0xFBC2,   /* U+11760 */
  0xFBC2,   /* U+11761 */
  0xFBC2,   /* U+11762 */
  0xFBC2,   /* U+11763 */
  0xFBC2,   /* U+11764 */
  0xFBC2,   /* U+11765 */
  0xFBC2,   /* U+11766 */
  0xFBC2,   /* U+11767 */
  0xFBC2,   /* U+11768 */
  0xFBC2,   /* U+11769 */
  0xFBC2,   /* U+1176A */
  0xFBC2,   /* U+1176B */
  0xFBC2,   /* U+1176C */
  0xFBC2,   /* U+1176D */
  0xFBC2,   /* U+1176E */
  0xFBC2,   /* U+1176F */
  0xFBC2,   /* U+11770 */
  0xFBC2,   /* U+11771 */
  0xFBC2,   /* U+11772 */
  0xFBC2,   /* U+11773 */
  0xFBC2,   /* U+11774 */
  0xFBC2,   /* U+11775 */
  0xFBC2,   /* U+11776 */
  0xFBC2,   /* U+11777 */
  0xFBC2,   /* U+11778 */
  0xFBC2,   /* U+11779 */
  0xFBC2,   /* U+1177A */
  0xFBC2,   /* U+1177B */
  0xFBC2,   /* U+1177C */
  0xFBC2,   /* U+1177D */
  0xFBC2,   /* U+1177E */
  0xFBC2,   /* U+1177F */
  0xFBC2,   /* U+11780 */
  0xFBC2,   /* U+11781 */
  0xFBC2,   /* U+11782 */
  0xFBC2,   /* U+11783 */
  0xFBC2,   /* U+11784 */
  0xFBC2,   /* U+11785 */
  0xFBC2,   /* U+11786 */
  0xFBC2,   /* U+11787 */
  0xFBC2,   /* U+11788 */
  0xFBC2,   /* U+11789 */
  0xFBC2,   /* U+1178A */
  0xFBC2,   /* U+1178B */
  0xFBC2,   /* U+1178C */
  0xFBC2,   /* U+1178D */
  0xFBC2,   /* U+1178E */
  0xFBC2,   /* U+1178F */
  0xFBC2,   /* U+11790 */
  0xFBC2,   /* U+11791 */
  0xFBC2,   /* U+11792 */
  0xFBC2,   /* U+11793 */
  0xFBC2,   /* U+11794 */
  0xFBC2,   /* U+11795 */
  0xFBC2,   /* U+11796 */
  0xFBC2,   /* U+11797 */
  0xFBC2,   /* U+11798 */
  0xFBC2,   /* U+11799 */
  0xFBC2,   /* U+1179A */
  0xFBC2,   /* U+1179B */
  0xFBC2,   /* U+1179C */
  0xFBC2,   /* U+1179D */
  0xFBC2,   /* U+1179E */
  0xFBC2,   /* U+1179F */
  0xFBC2,   /* U+117A0 */
  0xFBC2,   /* U+117A1 */
  0xFBC2,   /* U+117A2 */
  0xFBC2,   /* U+117A3 */
  0xFBC2,   /* U+117A4 */
  0xFBC2,   /* U+117A5 */
  0xFBC2,   /* U+117A6 */
  0xFBC2,   /* U+117A7 */
  0xFBC2,   /* U+117A8 */
  0xFBC2,   /* U+117A9 */
  0xFBC2,   /* U+117AA */
  0xFBC2,   /* U+117AB */
  0xFBC2,   /* U+117AC */
  0xFBC2,   /* U+117AD */
  0xFBC2,   /* U+117AE */
  0xFBC2,   /* U+117AF */
  0xFBC2,   /* U+117B0 */
  0xFBC2,   /* U+117B1 */
  0xFBC2,   /* U+117B2 */
  0xFBC2,   /* U+117B3 */
  0xFBC2,   /* U+117B4 */
  0xFBC2,   /* U+117B5 */
  0xFBC2,   /* U+117B6 */
  0xFBC2,   /* U+117B7 */
  0xFBC2,   /* U+117B8 */
  0xFBC2,   /* U+117B9 */
  0xFBC2,   /* U+117BA */
  0xFBC2,   /* U+117BB */
  0xFBC2,   /* U+117BC */
  0xFBC2,   /* U+117BD */
  0xFBC2,   /* U+117BE */
  0xFBC2,   /* U+117BF */
  0xFBC2,   /* U+117C0 */
  0xFBC2,   /* U+117C1 */
  0xFBC2,   /* U+117C2 */
  0xFBC2,   /* U+117C3 */
  0xFBC2,   /* U+117C4 */
  0xFBC2,   /* U+117C5 */
  0xFBC2,   /* U+117C6 */
  0xFBC2,   /* U+117C7 */
  0xFBC2,   /* U+117C8 */
  0xFBC2,   /* U+117C9 */
  0xFBC2,   /* U+117CA */
  0xFBC2,   /* U+117CB */
  0xFBC2,   /* U+117CC */
  0xFBC2,   /* U+117CD */
  0xFBC2,   /* U+117CE */
  0xFBC2,   /* U+117CF */
  0xFBC2,   /* U+117D0 */
  0xFBC2,   /* U+117D1 */
  0xFBC2,   /* U+117D2 */
  0xFBC2,   /* U+117D3 */
  0xFBC2,   /* U+117D4 */
  0xFBC2,   /* U+117D5 */
  0xFBC2,   /* U+117D6 */
  0xFBC2,   /* U+117D7 */
  0xFBC2,   /* U+117D8 */
  0xFBC2,   /* U+117D9 */
  0xFBC2,   /* U+117DA */
  0xFBC2,   /* U+117DB */
  0xFBC2,   /* U+117DC */
  0xFBC2,   /* U+117DD */
  0xFBC2,   /* U+117DE */
  0xFBC2,   /* U+117DF */
  0xFBC2,   /* U+117E0 */
  0xFBC2,   /* U+117E1 */
  0xFBC2,   /* U+117E2 */
  0xFBC2,   /* U+117E3 */
  0xFBC2,   /* U+117E4 */
  0xFBC2,   /* U+117E5 */
  0xFBC2,   /* U+117E6 */
  0xFBC2,   /* U+117E7 */
  0xFBC2,   /* U+117E8 */
  0xFBC2,   /* U+117E9 */
  0xFBC2,   /* U+117EA */
  0xFBC2,   /* U+117EB */
  0xFBC2,   /* U+117EC */
  0xFBC2,   /* U+117ED */
  0xFBC2,   /* U+117EE */
  0xFBC2,   /* U+117EF */
  0xFBC2,   /* U+117F0 */
  0xFBC2,   /* U+117F1 */
  0xFBC2,   /* U+117F2 */
  0xFBC2,   /* U+117F3 */
  0xFBC2,   /* U+117F4 */
  0xFBC2,   /* U+117F5 */
  0xFBC2,   /* U+117F6 */
  0xFBC2,   /* U+117F7 */
  0xFBC2,   /* U+117F8 */
  0xFBC2,   /* U+117F9 */
  0xFBC2,   /* U+117FA */
  0xFBC2,   /* U+117FB */
  0xFBC2,   /* U+117FC */
  0xFBC2,   /* U+117FD */
  0xFBC2,   /* U+117FE */
  0xFBC2,   /* U+117FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11700 */
  0x0020,   /* U+11701 */
  0x0020,   /* U+11702 */
  0x0020,   /* U+11703 */
  0x0020,   /* U+11704 */
  0x0020,   /* U+11705 */
  0x0020,   /* U+11706 */
  0x0020,   /* U+11707 */
  0x0020,   /* U+11708 */
  0x0020,   /* U+11709 */
  0x0020,   /* U+1170A */
  0x0020,   /* U+1170B */
  0x0020,   /* U+1170C */
  0x0020,   /* U+1170D */
  0x0020,   /* U+1170E */
  0x0020,   /* U+1170F */
  0x0020,   /* U+11710 */
  0x0020,   /* U+11711 */
  0x0020,   /* U+11712 */
  0x0020,   /* U+11713 */
  0x0020,   /* U+11714 */
  0x0020,   /* U+11715 */
  0x0020,   /* U+11716 */
  0x0020,   /* U+11717 */
  0x0020,   /* U+11718 */
  0x0020,   /* U+11719 */
  0x0020,   /* U+1171A */
  0x0020,   /* U+1171B */
  0x0020,   /* U+1171C */
  0x0020,   /* U+1171D */
  0x0020,   /* U+1171E */
  0x0020,   /* U+1171F */
  0x0020,   /* U+11720 */
  0x0020,   /* U+11721 */
  0x0020,   /* U+11722 */
  0x0020,   /* U+11723 */
  0x0020,   /* U+11724 */
  0x0020,   /* U+11725 */
  0x0020,   /* U+11726 */
  0x0020,   /* U+11727 */
  0x0020,   /* U+11728 */
  0x0020,   /* U+11729 */
  0x0020,   /* U+1172A */
  0x0020,   /* U+1172B */
  0x0020,   /* U+1172C */
  0x0020,   /* U+1172D */
  0x0020,   /* U+1172E */
  0x0020,   /* U+1172F */
  0x0020,   /* U+11730 */
  0x0020,   /* U+11731 */
  0x0020,   /* U+11732 */
  0x0020,   /* U+11733 */
  0x0020,   /* U+11734 */
  0x0020,   /* U+11735 */
  0x0020,   /* U+11736 */
  0x0020,   /* U+11737 */
  0x0020,   /* U+11738 */
  0x0020,   /* U+11739 */
  0x0020,   /* U+1173A */
  0x0020,   /* U+1173B */
  0x0020,   /* U+1173C */
  0x0020,   /* U+1173D */
  0x0020,   /* U+1173E */
  0x0020,   /* U+1173F */
  0x0020,   /* U+11740 */
  0x0020,   /* U+11741 */
  0x0020,   /* U+11742 */
  0x0020,   /* U+11743 */
  0x0020,   /* U+11744 */
  0x0020,   /* U+11745 */
  0x0020,   /* U+11746 */
  0x0020,   /* U+11747 */
  0x0020,   /* U+11748 */
  0x0020,   /* U+11749 */
  0x0020,   /* U+1174A */
  0x0020,   /* U+1174B */
  0x0020,   /* U+1174C */
  0x0020,   /* U+1174D */
  0x0020,   /* U+1174E */
  0x0020,   /* U+1174F */
  0x0020,   /* U+11750 */
  0x0020,   /* U+11751 */
  0x0020,   /* U+11752 */
  0x0020,   /* U+11753 */
  0x0020,   /* U+11754 */
  0x0020,   /* U+11755 */
  0x0020,   /* U+11756 */
  0x0020,   /* U+11757 */
  0x0020,   /* U+11758 */
  0x0020,   /* U+11759 */
  0x0020,   /* U+1175A */
  0x0020,   /* U+1175B */
  0x0020,   /* U+1175C */
  0x0020,   /* U+1175D */
  0x0020,   /* U+1175E */
  0x0020,   /* U+1175F */
  0x0020,   /* U+11760 */
  0x0020,   /* U+11761 */
  0x0020,   /* U+11762 */
  0x0020,   /* U+11763 */
  0x0020,   /* U+11764 */
  0x0020,   /* U+11765 */
  0x0020,   /* U+11766 */
  0x0020,   /* U+11767 */
  0x0020,   /* U+11768 */
  0x0020,   /* U+11769 */
  0x0020,   /* U+1176A */
  0x0020,   /* U+1176B */
  0x0020,   /* U+1176C */
  0x0020,   /* U+1176D */
  0x0020,   /* U+1176E */
  0x0020,   /* U+1176F */
  0x0020,   /* U+11770 */
  0x0020,   /* U+11771 */
  0x0020,   /* U+11772 */
  0x0020,   /* U+11773 */
  0x0020,   /* U+11774 */
  0x0020,   /* U+11775 */
  0x0020,   /* U+11776 */
  0x0020,   /* U+11777 */
  0x0020,   /* U+11778 */
  0x0020,   /* U+11779 */
  0x0020,   /* U+1177A */
  0x0020,   /* U+1177B */
  0x0020,   /* U+1177C */
  0x0020,   /* U+1177D */
  0x0020,   /* U+1177E */
  0x0020,   /* U+1177F */
  0x0020,   /* U+11780 */
  0x0020,   /* U+11781 */
  0x0020,   /* U+11782 */
  0x0020,   /* U+11783 */
  0x0020,   /* U+11784 */
  0x0020,   /* U+11785 */
  0x0020,   /* U+11786 */
  0x0020,   /* U+11787 */
  0x0020,   /* U+11788 */
  0x0020,   /* U+11789 */
  0x0020,   /* U+1178A */
  0x0020,   /* U+1178B */
  0x0020,   /* U+1178C */
  0x0020,   /* U+1178D */
  0x0020,   /* U+1178E */
  0x0020,   /* U+1178F */
  0x0020,   /* U+11790 */
  0x0020,   /* U+11791 */
  0x0020,   /* U+11792 */
  0x0020,   /* U+11793 */
  0x0020,   /* U+11794 */
  0x0020,   /* U+11795 */
  0x0020,   /* U+11796 */
  0x0020,   /* U+11797 */
  0x0020,   /* U+11798 */
  0x0020,   /* U+11799 */
  0x0020,   /* U+1179A */
  0x0020,   /* U+1179B */
  0x0020,   /* U+1179C */
  0x0020,   /* U+1179D */
  0x0020,   /* U+1179E */
  0x0020,   /* U+1179F */
  0x0020,   /* U+117A0 */
  0x0020,   /* U+117A1 */
  0x0020,   /* U+117A2 */
  0x0020,   /* U+117A3 */
  0x0020,   /* U+117A4 */
  0x0020,   /* U+117A5 */
  0x0020,   /* U+117A6 */
  0x0020,   /* U+117A7 */
  0x0020,   /* U+117A8 */
  0x0020,   /* U+117A9 */
  0x0020,   /* U+117AA */
  0x0020,   /* U+117AB */
  0x0020,   /* U+117AC */
  0x0020,   /* U+117AD */
  0x0020,   /* U+117AE */
  0x0020,   /* U+117AF */
  0x0020,   /* U+117B0 */
  0x0020,   /* U+117B1 */
  0x0020,   /* U+117B2 */
  0x0020,   /* U+117B3 */
  0x0020,   /* U+117B4 */
  0x0020,   /* U+117B5 */
  0x0020,   /* U+117B6 */
  0x0020,   /* U+117B7 */
  0x0020,   /* U+117B8 */
  0x0020,   /* U+117B9 */
  0x0020,   /* U+117BA */
  0x0020,   /* U+117BB */
  0x0020,   /* U+117BC */
  0x0020,   /* U+117BD */
  0x0020,   /* U+117BE */
  0x0020,   /* U+117BF */
  0x0020,   /* U+117C0 */
  0x0020,   /* U+117C1 */
  0x0020,   /* U+117C2 */
  0x0020,   /* U+117C3 */
  0x0020,   /* U+117C4 */
  0x0020,   /* U+117C5 */
  0x0020,   /* U+117C6 */
  0x0020,   /* U+117C7 */
  0x0020,   /* U+117C8 */
  0x0020,   /* U+117C9 */
  0x0020,   /* U+117CA */
  0x0020,   /* U+117CB */
  0x0020,   /* U+117CC */
  0x0020,   /* U+117CD */
  0x0020,   /* U+117CE */
  0x0020,   /* U+117CF */
  0x0020,   /* U+117D0 */
  0x0020,   /* U+117D1 */
  0x0020,   /* U+117D2 */
  0x0020,   /* U+117D3 */
  0x0020,   /* U+117D4 */
  0x0020,   /* U+117D5 */
  0x0020,   /* U+117D6 */
  0x0020,   /* U+117D7 */
  0x0020,   /* U+117D8 */
  0x0020,   /* U+117D9 */
  0x0020,   /* U+117DA */
  0x0020,   /* U+117DB */
  0x0020,   /* U+117DC */
  0x0020,   /* U+117DD */
  0x0020,   /* U+117DE */
  0x0020,   /* U+117DF */
  0x0020,   /* U+117E0 */
  0x0020,   /* U+117E1 */
  0x0020,   /* U+117E2 */
  0x0020,   /* U+117E3 */
  0x0020,   /* U+117E4 */
  0x0020,   /* U+117E5 */
  0x0020,   /* U+117E6 */
  0x0020,   /* U+117E7 */
  0x0020,   /* U+117E8 */
  0x0020,   /* U+117E9 */
  0x0020,   /* U+117EA */
  0x0020,   /* U+117EB */
  0x0020,   /* U+117EC */
  0x0020,   /* U+117ED */
  0x0020,   /* U+117EE */
  0x0020,   /* U+117EF */
  0x0020,   /* U+117F0 */
  0x0020,   /* U+117F1 */
  0x0020,   /* U+117F2 */
  0x0020,   /* U+117F3 */
  0x0020,   /* U+117F4 */
  0x0020,   /* U+117F5 */
  0x0020,   /* U+117F6 */
  0x0020,   /* U+117F7 */
  0x0020,   /* U+117F8 */
  0x0020,   /* U+117F9 */
  0x0020,   /* U+117FA */
  0x0020,   /* U+117FB */
  0x0020,   /* U+117FC */
  0x0020,   /* U+117FD */
  0x0020,   /* U+117FE */
  0x0020,   /* U+117FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11700 */
  0x0002,   /* U+11701 */
  0x0002,   /* U+11702 */
  0x0002,   /* U+11703 */
  0x0002,   /* U+11704 */
  0x0004,   /* U+11705 */
  0x0002,   /* U+11706 */
  0x0002,   /* U+11707 */
  0x0002,   /* U+11708 */
  0x0002,   /* U+11709 */
  0x0002,   /* U+1170A */
  0x0002,   /* U+1170B */
  0x0002,   /* U+1170C */
  0x0002,   /* U+1170D */
  0x0002,   /* U+1170E */
  0x0002,   /* U+1170F */
  0x0002,   /* U+11710 */
  0x0002,   /* U+11711 */
  0x0002,   /* U+11712 */
  0x0002,   /* U+11713 */
  0x0002,   /* U+11714 */
  0x0002,   /* U+11715 */
  0x0004,   /* U+11716 */
  0x0002,   /* U+11717 */
  0x0002,   /* U+11718 */
  0x0002,   /* U+11719 */
  0x0002,   /* U+1171A */
  0x0002,   /* U+1171B */
  0x0002,   /* U+1171C */
  0x0002,   /* U+1171D */
  0x0002,   /* U+1171E */
  0x0002,   /* U+1171F */
  0x0002,   /* U+11720 */
  0x0002,   /* U+11721 */
  0x0002,   /* U+11722 */
  0x0002,   /* U+11723 */
  0x0002,   /* U+11724 */
  0x0002,   /* U+11725 */
  0x0002,   /* U+11726 */
  0x0002,   /* U+11727 */
  0x0002,   /* U+11728 */
  0x0002,   /* U+11729 */
  0x0002,   /* U+1172A */
  0x0002,   /* U+1172B */
  0x0002,   /* U+1172C */
  0x0002,   /* U+1172D */
  0x0002,   /* U+1172E */
  0x0002,   /* U+1172F */
  0x0002,   /* U+11730 */
  0x0002,   /* U+11731 */
  0x0002,   /* U+11732 */
  0x0002,   /* U+11733 */
  0x0002,   /* U+11734 */
  0x0002,   /* U+11735 */
  0x0002,   /* U+11736 */
  0x0002,   /* U+11737 */
  0x0002,   /* U+11738 */
  0x0002,   /* U+11739 */
  0x0002,   /* U+1173A */
  0x0002,   /* U+1173B */
  0x0002,   /* U+1173C */
  0x0002,   /* U+1173D */
  0x0002,   /* U+1173E */
  0x0002,   /* U+1173F */
  0x0002,   /* U+11740 */
  0x0002,   /* U+11741 */
  0x0002,   /* U+11742 */
  0x0002,   /* U+11743 */
  0x0002,   /* U+11744 */
  0x0002,   /* U+11745 */
  0x0002,   /* U+11746 */
  0x0002,   /* U+11747 */
  0x0002,   /* U+11748 */
  0x0002,   /* U+11749 */
  0x0002,   /* U+1174A */
  0x0002,   /* U+1174B */
  0x0002,   /* U+1174C */
  0x0002,   /* U+1174D */
  0x0002,   /* U+1174E */
  0x0002,   /* U+1174F */
  0x0002,   /* U+11750 */
  0x0002,   /* U+11751 */
  0x0002,   /* U+11752 */
  0x0002,   /* U+11753 */
  0x0002,   /* U+11754 */
  0x0002,   /* U+11755 */
  0x0002,   /* U+11756 */
  0x0002,   /* U+11757 */
  0x0002,   /* U+11758 */
  0x0002,   /* U+11759 */
  0x0002,   /* U+1175A */
  0x0002,   /* U+1175B */
  0x0002,   /* U+1175C */
  0x0002,   /* U+1175D */
  0x0002,   /* U+1175E */
  0x0002,   /* U+1175F */
  0x0002,   /* U+11760 */
  0x0002,   /* U+11761 */
  0x0002,   /* U+11762 */
  0x0002,   /* U+11763 */
  0x0002,   /* U+11764 */
  0x0002,   /* U+11765 */
  0x0002,   /* U+11766 */
  0x0002,   /* U+11767 */
  0x0002,   /* U+11768 */
  0x0002,   /* U+11769 */
  0x0002,   /* U+1176A */
  0x0002,   /* U+1176B */
  0x0002,   /* U+1176C */
  0x0002,   /* U+1176D */
  0x0002,   /* U+1176E */
  0x0002,   /* U+1176F */
  0x0002,   /* U+11770 */
  0x0002,   /* U+11771 */
  0x0002,   /* U+11772 */
  0x0002,   /* U+11773 */
  0x0002,   /* U+11774 */
  0x0002,   /* U+11775 */
  0x0002,   /* U+11776 */
  0x0002,   /* U+11777 */
  0x0002,   /* U+11778 */
  0x0002,   /* U+11779 */
  0x0002,   /* U+1177A */
  0x0002,   /* U+1177B */
  0x0002,   /* U+1177C */
  0x0002,   /* U+1177D */
  0x0002,   /* U+1177E */
  0x0002,   /* U+1177F */
  0x0002,   /* U+11780 */
  0x0002,   /* U+11781 */
  0x0002,   /* U+11782 */
  0x0002,   /* U+11783 */
  0x0002,   /* U+11784 */
  0x0002,   /* U+11785 */
  0x0002,   /* U+11786 */
  0x0002,   /* U+11787 */
  0x0002,   /* U+11788 */
  0x0002,   /* U+11789 */
  0x0002,   /* U+1178A */
  0x0002,   /* U+1178B */
  0x0002,   /* U+1178C */
  0x0002,   /* U+1178D */
  0x0002,   /* U+1178E */
  0x0002,   /* U+1178F */
  0x0002,   /* U+11790 */
  0x0002,   /* U+11791 */
  0x0002,   /* U+11792 */
  0x0002,   /* U+11793 */
  0x0002,   /* U+11794 */
  0x0002,   /* U+11795 */
  0x0002,   /* U+11796 */
  0x0002,   /* U+11797 */
  0x0002,   /* U+11798 */
  0x0002,   /* U+11799 */
  0x0002,   /* U+1179A */
  0x0002,   /* U+1179B */
  0x0002,   /* U+1179C */
  0x0002,   /* U+1179D */
  0x0002,   /* U+1179E */
  0x0002,   /* U+1179F */
  0x0002,   /* U+117A0 */
  0x0002,   /* U+117A1 */
  0x0002,   /* U+117A2 */
  0x0002,   /* U+117A3 */
  0x0002,   /* U+117A4 */
  0x0002,   /* U+117A5 */
  0x0002,   /* U+117A6 */
  0x0002,   /* U+117A7 */
  0x0002,   /* U+117A8 */
  0x0002,   /* U+117A9 */
  0x0002,   /* U+117AA */
  0x0002,   /* U+117AB */
  0x0002,   /* U+117AC */
  0x0002,   /* U+117AD */
  0x0002,   /* U+117AE */
  0x0002,   /* U+117AF */
  0x0002,   /* U+117B0 */
  0x0002,   /* U+117B1 */
  0x0002,   /* U+117B2 */
  0x0002,   /* U+117B3 */
  0x0002,   /* U+117B4 */
  0x0002,   /* U+117B5 */
  0x0002,   /* U+117B6 */
  0x0002,   /* U+117B7 */
  0x0002,   /* U+117B8 */
  0x0002,   /* U+117B9 */
  0x0002,   /* U+117BA */
  0x0002,   /* U+117BB */
  0x0002,   /* U+117BC */
  0x0002,   /* U+117BD */
  0x0002,   /* U+117BE */
  0x0002,   /* U+117BF */
  0x0002,   /* U+117C0 */
  0x0002,   /* U+117C1 */
  0x0002,   /* U+117C2 */
  0x0002,   /* U+117C3 */
  0x0002,   /* U+117C4 */
  0x0002,   /* U+117C5 */
  0x0002,   /* U+117C6 */
  0x0002,   /* U+117C7 */
  0x0002,   /* U+117C8 */
  0x0002,   /* U+117C9 */
  0x0002,   /* U+117CA */
  0x0002,   /* U+117CB */
  0x0002,   /* U+117CC */
  0x0002,   /* U+117CD */
  0x0002,   /* U+117CE */
  0x0002,   /* U+117CF */
  0x0002,   /* U+117D0 */
  0x0002,   /* U+117D1 */
  0x0002,   /* U+117D2 */
  0x0002,   /* U+117D3 */
  0x0002,   /* U+117D4 */
  0x0002,   /* U+117D5 */
  0x0002,   /* U+117D6 */
  0x0002,   /* U+117D7 */
  0x0002,   /* U+117D8 */
  0x0002,   /* U+117D9 */
  0x0002,   /* U+117DA */
  0x0002,   /* U+117DB */
  0x0002,   /* U+117DC */
  0x0002,   /* U+117DD */
  0x0002,   /* U+117DE */
  0x0002,   /* U+117DF */
  0x0002,   /* U+117E0 */
  0x0002,   /* U+117E1 */
  0x0002,   /* U+117E2 */
  0x0002,   /* U+117E3 */
  0x0002,   /* U+117E4 */
  0x0002,   /* U+117E5 */
  0x0002,   /* U+117E6 */
  0x0002,   /* U+117E7 */
  0x0002,   /* U+117E8 */
  0x0002,   /* U+117E9 */
  0x0002,   /* U+117EA */
  0x0002,   /* U+117EB */
  0x0002,   /* U+117EC */
  0x0002,   /* U+117ED */
  0x0002,   /* U+117EE */
  0x0002,   /* U+117EF */
  0x0002,   /* U+117F0 */
  0x0002,   /* U+117F1 */
  0x0002,   /* U+117F2 */
  0x0002,   /* U+117F3 */
  0x0002,   /* U+117F4 */
  0x0002,   /* U+117F5 */
  0x0002,   /* U+117F6 */
  0x0002,   /* U+117F7 */
  0x0002,   /* U+117F8 */
  0x0002,   /* U+117F9 */
  0x0002,   /* U+117FA */
  0x0002,   /* U+117FB */
  0x0002,   /* U+117FC */
  0x0002,   /* U+117FD */
  0x0002,   /* U+117FE */
  0x0002,   /* U+117FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11700 */
  0x0000,   /* U+11701 */
  0x0000,   /* U+11702 */
  0x0000,   /* U+11703 */
  0x0000,   /* U+11704 */
  0x0000,   /* U+11705 */
  0x0000,   /* U+11706 */
  0x0000,   /* U+11707 */
  0x0000,   /* U+11708 */
  0x0000,   /* U+11709 */
  0x0000,   /* U+1170A */
  0x0000,   /* U+1170B */
  0x0000,   /* U+1170C */
  0x0000,   /* U+1170D */
  0x0000,   /* U+1170E */
  0x0000,   /* U+1170F */
  0x0000,   /* U+11710 */
  0x0000,   /* U+11711 */
  0x0000,   /* U+11712 */
  0x0000,   /* U+11713 */
  0x0000,   /* U+11714 */
  0x0000,   /* U+11715 */
  0x0000,   /* U+11716 */
  0x0000,   /* U+11717 */
  0x0000,   /* U+11718 */
  0x0000,   /* U+11719 */
  0x971A,   /* U+1171A */
  0x971B,   /* U+1171B */
  0x971C,   /* U+1171C */
  0x0000,   /* U+1171D */
  0x0000,   /* U+1171E */
  0x0000,   /* U+1171F */
  0x0000,   /* U+11720 */
  0x0000,   /* U+11721 */
  0x0000,   /* U+11722 */
  0x0000,   /* U+11723 */
  0x0000,   /* U+11724 */
  0x0000,   /* U+11725 */
  0x0000,   /* U+11726 */
  0x0000,   /* U+11727 */
  0x0000,   /* U+11728 */
  0x0000,   /* U+11729 */
  0x0000,   /* U+1172A */
  0x0000,   /* U+1172B */
  0x972C,   /* U+1172C */
  0x972D,   /* U+1172D */
  0x972E,   /* U+1172E */
  0x972F,   /* U+1172F */
  0x0000,   /* U+11730 */
  0x0000,   /* U+11731 */
  0x0000,   /* U+11732 */
  0x0000,   /* U+11733 */
  0x0000,   /* U+11734 */
  0x0000,   /* U+11735 */
  0x0000,   /* U+11736 */
  0x0000,   /* U+11737 */
  0x0000,   /* U+11738 */
  0x0000,   /* U+11739 */
  0x0000,   /* U+1173A */
  0x0000,   /* U+1173B */
  0x0000,   /* U+1173C */
  0x0000,   /* U+1173D */
  0x0000,   /* U+1173E */
  0x0000,   /* U+1173F */
  0x9740,   /* U+11740 */
  0x9741,   /* U+11741 */
  0x9742,   /* U+11742 */
  0x9743,   /* U+11743 */
  0x9744,   /* U+11744 */
  0x9745,   /* U+11745 */
  0x9746,   /* U+11746 */
  0x9747,   /* U+11747 */
  0x9748,   /* U+11748 */
  0x9749,   /* U+11749 */
  0x974A,   /* U+1174A */
  0x974B,   /* U+1174B */
  0x974C,   /* U+1174C */
  0x974D,   /* U+1174D */
  0x974E,   /* U+1174E */
  0x974F,   /* U+1174F */
  0x9750,   /* U+11750 */
  0x9751,   /* U+11751 */
  0x9752,   /* U+11752 */
  0x9753,   /* U+11753 */
  0x9754,   /* U+11754 */
  0x9755,   /* U+11755 */
  0x9756,   /* U+11756 */
  0x9757,   /* U+11757 */
  0x9758,   /* U+11758 */
  0x9759,   /* U+11759 */
  0x975A,   /* U+1175A */
  0x975B,   /* U+1175B */
  0x975C,   /* U+1175C */
  0x975D,   /* U+1175D */
  0x975E,   /* U+1175E */
  0x975F,   /* U+1175F */
  0x9760,   /* U+11760 */
  0x9761,   /* U+11761 */
  0x9762,   /* U+11762 */
  0x9763,   /* U+11763 */
  0x9764,   /* U+11764 */
  0x9765,   /* U+11765 */
  0x9766,   /* U+11766 */
  0x9767,   /* U+11767 */
  0x9768,   /* U+11768 */
  0x9769,   /* U+11769 */
  0x976A,   /* U+1176A */
  0x976B,   /* U+1176B */
  0x976C,   /* U+1176C */
  0x976D,   /* U+1176D */
  0x976E,   /* U+1176E */
  0x976F,   /* U+1176F */
  0x9770,   /* U+11770 */
  0x9771,   /* U+11771 */
  0x9772,   /* U+11772 */
  0x9773,   /* U+11773 */
  0x9774,   /* U+11774 */
  0x9775,   /* U+11775 */
  0x9776,   /* U+11776 */
  0x9777,   /* U+11777 */
  0x9778,   /* U+11778 */
  0x9779,   /* U+11779 */
  0x977A,   /* U+1177A */
  0x977B,   /* U+1177B */
  0x977C,   /* U+1177C */
  0x977D,   /* U+1177D */
  0x977E,   /* U+1177E */
  0x977F,   /* U+1177F */
  0x9780,   /* U+11780 */
  0x9781,   /* U+11781 */
  0x9782,   /* U+11782 */
  0x9783,   /* U+11783 */
  0x9784,   /* U+11784 */
  0x9785,   /* U+11785 */
  0x9786,   /* U+11786 */
  0x9787,   /* U+11787 */
  0x9788,   /* U+11788 */
  0x9789,   /* U+11789 */
  0x978A,   /* U+1178A */
  0x978B,   /* U+1178B */
  0x978C,   /* U+1178C */
  0x978D,   /* U+1178D */
  0x978E,   /* U+1178E */
  0x978F,   /* U+1178F */
  0x9790,   /* U+11790 */
  0x9791,   /* U+11791 */
  0x9792,   /* U+11792 */
  0x9793,   /* U+11793 */
  0x9794,   /* U+11794 */
  0x9795,   /* U+11795 */
  0x9796,   /* U+11796 */
  0x9797,   /* U+11797 */
  0x9798,   /* U+11798 */
  0x9799,   /* U+11799 */
  0x979A,   /* U+1179A */
  0x979B,   /* U+1179B */
  0x979C,   /* U+1179C */
  0x979D,   /* U+1179D */
  0x979E,   /* U+1179E */
  0x979F,   /* U+1179F */
  0x97A0,   /* U+117A0 */
  0x97A1,   /* U+117A1 */
  0x97A2,   /* U+117A2 */
  0x97A3,   /* U+117A3 */
  0x97A4,   /* U+117A4 */
  0x97A5,   /* U+117A5 */
  0x97A6,   /* U+117A6 */
  0x97A7,   /* U+117A7 */
  0x97A8,   /* U+117A8 */
  0x97A9,   /* U+117A9 */
  0x97AA,   /* U+117AA */
  0x97AB,   /* U+117AB */
  0x97AC,   /* U+117AC */
  0x97AD,   /* U+117AD */
  0x97AE,   /* U+117AE */
  0x97AF,   /* U+117AF */
  0x97B0,   /* U+117B0 */
  0x97B1,   /* U+117B1 */
  0x97B2,   /* U+117B2 */
  0x97B3,   /* U+117B3 */
  0x97B4,   /* U+117B4 */
  0x97B5,   /* U+117B5 */
  0x97B6,   /* U+117B6 */
  0x97B7,   /* U+117B7 */
  0x97B8,   /* U+117B8 */
  0x97B9,   /* U+117B9 */
  0x97BA,   /* U+117BA */
  0x97BB,   /* U+117BB */
  0x97BC,   /* U+117BC */
  0x97BD,   /* U+117BD */
  0x97BE,   /* U+117BE */
  0x97BF,   /* U+117BF */
  0x97C0,   /* U+117C0 */
  0x97C1,   /* U+117C1 */
  0x97C2,   /* U+117C2 */
  0x97C3,   /* U+117C3 */
  0x97C4,   /* U+117C4 */
  0x97C5,   /* U+117C5 */
  0x97C6,   /* U+117C6 */
  0x97C7,   /* U+117C7 */
  0x97C8,   /* U+117C8 */
  0x97C9,   /* U+117C9 */
  0x97CA,   /* U+117CA */
  0x97CB,   /* U+117CB */
  0x97CC,   /* U+117CC */
  0x97CD,   /* U+117CD */
  0x97CE,   /* U+117CE */
  0x97CF,   /* U+117CF */
  0x97D0,   /* U+117D0 */
  0x97D1,   /* U+117D1 */
  0x97D2,   /* U+117D2 */
  0x97D3,   /* U+117D3 */
  0x97D4,   /* U+117D4 */
  0x97D5,   /* U+117D5 */
  0x97D6,   /* U+117D6 */
  0x97D7,   /* U+117D7 */
  0x97D8,   /* U+117D8 */
  0x97D9,   /* U+117D9 */
  0x97DA,   /* U+117DA */
  0x97DB,   /* U+117DB */
  0x97DC,   /* U+117DC */
  0x97DD,   /* U+117DD */
  0x97DE,   /* U+117DE */
  0x97DF,   /* U+117DF */
  0x97E0,   /* U+117E0 */
  0x97E1,   /* U+117E1 */
  0x97E2,   /* U+117E2 */
  0x97E3,   /* U+117E3 */
  0x97E4,   /* U+117E4 */
  0x97E5,   /* U+117E5 */
  0x97E6,   /* U+117E6 */
  0x97E7,   /* U+117E7 */
  0x97E8,   /* U+117E8 */
  0x97E9,   /* U+117E9 */
  0x97EA,   /* U+117EA */
  0x97EB,   /* U+117EB */
  0x97EC,   /* U+117EC */
  0x97ED,   /* U+117ED */
  0x97EE,   /* U+117EE */
  0x97EF,   /* U+117EF */
  0x97F0,   /* U+117F0 */
  0x97F1,   /* U+117F1 */
  0x97F2,   /* U+117F2 */
  0x97F3,   /* U+117F3 */
  0x97F4,   /* U+117F4 */
  0x97F5,   /* U+117F5 */
  0x97F6,   /* U+117F6 */
  0x97F7,   /* U+117F7 */
  0x97F8,   /* U+117F8 */
  0x97F9,   /* U+117F9 */
  0x97FA,   /* U+117FA */
  0x97FB,   /* U+117FB */
  0x97FC,   /* U+117FC */
  0x97FD,   /* U+117FD */
  0x97FE,   /* U+117FE */
  0x97FF,   /* U+117FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11700 */
  0x0000,   /* U+11701 */
  0x0000,   /* U+11702 */
  0x0000,   /* U+11703 */
  0x0000,   /* U+11704 */
  0x0110,   /* U+11705 */
  0x0000,   /* U+11706 */
  0x0000,   /* U+11707 */
  0x0000,   /* U+11708 */
  0x0000,   /* U+11709 */
  0x0000,   /* U+1170A */
  0x0000,   /* U+1170B */
  0x0000,   /* U+1170C */
  0x0000,   /* U+1170D */
  0x0000,   /* U+1170E */
  0x0000,   /* U+1170F */
  0x0000,   /* U+11710 */
  0x0000,   /* U+11711 */
  0x0000,   /* U+11712 */
  0x0000,   /* U+11713 */
  0x0000,   /* U+11714 */
  0x0000,   /* U+11715 */
  0x0110,   /* U+11716 */
  0x0000,   /* U+11717 */
  0x0000,   /* U+11718 */
  0x0000,   /* U+11719 */
  0x0000,   /* U+1171A */
  0x0000,   /* U+1171B */
  0x0000,   /* U+1171C */
  0x0000,   /* U+1171D */
  0x0000,   /* U+1171E */
  0x0000,   /* U+1171F */
  0x0000,   /* U+11720 */
  0x0000,   /* U+11721 */
  0x0000,   /* U+11722 */
  0x0000,   /* U+11723 */
  0x0000,   /* U+11724 */
  0x0000,   /* U+11725 */
  0x0000,   /* U+11726 */
  0x0000,   /* U+11727 */
  0x0000,   /* U+11728 */
  0x0000,   /* U+11729 */
  0x0000,   /* U+1172A */
  0x0000,   /* U+1172B */
  0x0000,   /* U+1172C */
  0x0000,   /* U+1172D */
  0x0000,   /* U+1172E */
  0x0000,   /* U+1172F */
  0x0000,   /* U+11730 */
  0x0000,   /* U+11731 */
  0x0000,   /* U+11732 */
  0x0000,   /* U+11733 */
  0x0000,   /* U+11734 */
  0x0000,   /* U+11735 */
  0x0000,   /* U+11736 */
  0x0000,   /* U+11737 */
  0x0000,   /* U+11738 */
  0x0000,   /* U+11739 */
  0x0000,   /* U+1173A */
  0x0000,   /* U+1173B */
  0x0000,   /* U+1173C */
  0x0000,   /* U+1173D */
  0x0000,   /* U+1173E */
  0x0000,   /* U+1173F */
  0x0000,   /* U+11740 */
  0x0000,   /* U+11741 */
  0x0000,   /* U+11742 */
  0x0000,   /* U+11743 */
  0x0000,   /* U+11744 */
  0x0000,   /* U+11745 */
  0x0000,   /* U+11746 */
  0x0000,   /* U+11747 */
  0x0000,   /* U+11748 */
  0x0000,   /* U+11749 */
  0x0000,   /* U+1174A */
  0x0000,   /* U+1174B */
  0x0000,   /* U+1174C */
  0x0000,   /* U+1174D */
  0x0000,   /* U+1174E */
  0x0000,   /* U+1174F */
  0x0000,   /* U+11750 */
  0x0000,   /* U+11751 */
  0x0000,   /* U+11752 */
  0x0000,   /* U+11753 */
  0x0000,   /* U+11754 */
  0x0000,   /* U+11755 */
  0x0000,   /* U+11756 */
  0x0000,   /* U+11757 */
  0x0000,   /* U+11758 */
  0x0000,   /* U+11759 */
  0x0000,   /* U+1175A */
  0x0000,   /* U+1175B */
  0x0000,   /* U+1175C */
  0x0000,   /* U+1175D */
  0x0000,   /* U+1175E */
  0x0000,   /* U+1175F */
  0x0000,   /* U+11760 */
  0x0000,   /* U+11761 */
  0x0000,   /* U+11762 */
  0x0000,   /* U+11763 */
  0x0000,   /* U+11764 */
  0x0000,   /* U+11765 */
  0x0000,   /* U+11766 */
  0x0000,   /* U+11767 */
  0x0000,   /* U+11768 */
  0x0000,   /* U+11769 */
  0x0000,   /* U+1176A */
  0x0000,   /* U+1176B */
  0x0000,   /* U+1176C */
  0x0000,   /* U+1176D */
  0x0000,   /* U+1176E */
  0x0000,   /* U+1176F */
  0x0000,   /* U+11770 */
  0x0000,   /* U+11771 */
  0x0000,   /* U+11772 */
  0x0000,   /* U+11773 */
  0x0000,   /* U+11774 */
  0x0000,   /* U+11775 */
  0x0000,   /* U+11776 */
  0x0000,   /* U+11777 */
  0x0000,   /* U+11778 */
  0x0000,   /* U+11779 */
  0x0000,   /* U+1177A */
  0x0000,   /* U+1177B */
  0x0000,   /* U+1177C */
  0x0000,   /* U+1177D */
  0x0000,   /* U+1177E */
  0x0000,   /* U+1177F */
  0x0000,   /* U+11780 */
  0x0000,   /* U+11781 */
  0x0000,   /* U+11782 */
  0x0000,   /* U+11783 */
  0x0000,   /* U+11784 */
  0x0000,   /* U+11785 */
  0x0000,   /* U+11786 */
  0x0000,   /* U+11787 */
  0x0000,   /* U+11788 */
  0x0000,   /* U+11789 */
  0x0000,   /* U+1178A */
  0x0000,   /* U+1178B */
  0x0000,   /* U+1178C */
  0x0000,   /* U+1178D */
  0x0000,   /* U+1178E */
  0x0000,   /* U+1178F */
  0x0000,   /* U+11790 */
  0x0000,   /* U+11791 */
  0x0000,   /* U+11792 */
  0x0000,   /* U+11793 */
  0x0000,   /* U+11794 */
  0x0000,   /* U+11795 */
  0x0000,   /* U+11796 */
  0x0000,   /* U+11797 */
  0x0000,   /* U+11798 */
  0x0000,   /* U+11799 */
  0x0000,   /* U+1179A */
  0x0000,   /* U+1179B */
  0x0000,   /* U+1179C */
  0x0000,   /* U+1179D */
  0x0000,   /* U+1179E */
  0x0000,   /* U+1179F */
  0x0000,   /* U+117A0 */
  0x0000,   /* U+117A1 */
  0x0000,   /* U+117A2 */
  0x0000,   /* U+117A3 */
  0x0000,   /* U+117A4 */
  0x0000,   /* U+117A5 */
  0x0000,   /* U+117A6 */
  0x0000,   /* U+117A7 */
  0x0000,   /* U+117A8 */
  0x0000,   /* U+117A9 */
  0x0000,   /* U+117AA */
  0x0000,   /* U+117AB */
  0x0000,   /* U+117AC */
  0x0000,   /* U+117AD */
  0x0000,   /* U+117AE */
  0x0000,   /* U+117AF */
  0x0000,   /* U+117B0 */
  0x0000,   /* U+117B1 */
  0x0000,   /* U+117B2 */
  0x0000,   /* U+117B3 */
  0x0000,   /* U+117B4 */
  0x0000,   /* U+117B5 */
  0x0000,   /* U+117B6 */
  0x0000,   /* U+117B7 */
  0x0000,   /* U+117B8 */
  0x0000,   /* U+117B9 */
  0x0000,   /* U+117BA */
  0x0000,   /* U+117BB */
  0x0000,   /* U+117BC */
  0x0000,   /* U+117BD */
  0x0000,   /* U+117BE */
  0x0000,   /* U+117BF */
  0x0000,   /* U+117C0 */
  0x0000,   /* U+117C1 */
  0x0000,   /* U+117C2 */
  0x0000,   /* U+117C3 */
  0x0000,   /* U+117C4 */
  0x0000,   /* U+117C5 */
  0x0000,   /* U+117C6 */
  0x0000,   /* U+117C7 */
  0x0000,   /* U+117C8 */
  0x0000,   /* U+117C9 */
  0x0000,   /* U+117CA */
  0x0000,   /* U+117CB */
  0x0000,   /* U+117CC */
  0x0000,   /* U+117CD */
  0x0000,   /* U+117CE */
  0x0000,   /* U+117CF */
  0x0000,   /* U+117D0 */
  0x0000,   /* U+117D1 */
  0x0000,   /* U+117D2 */
  0x0000,   /* U+117D3 */
  0x0000,   /* U+117D4 */
  0x0000,   /* U+117D5 */
  0x0000,   /* U+117D6 */
  0x0000,   /* U+117D7 */
  0x0000,   /* U+117D8 */
  0x0000,   /* U+117D9 */
  0x0000,   /* U+117DA */
  0x0000,   /* U+117DB */
  0x0000,   /* U+117DC */
  0x0000,   /* U+117DD */
  0x0000,   /* U+117DE */
  0x0000,   /* U+117DF */
  0x0000,   /* U+117E0 */
  0x0000,   /* U+117E1 */
  0x0000,   /* U+117E2 */
  0x0000,   /* U+117E3 */
  0x0000,   /* U+117E4 */
  0x0000,   /* U+117E5 */
  0x0000,   /* U+117E6 */
  0x0000,   /* U+117E7 */
  0x0000,   /* U+117E8 */
  0x0000,   /* U+117E9 */
  0x0000,   /* U+117EA */
  0x0000,   /* U+117EB */
  0x0000,   /* U+117EC */
  0x0000,   /* U+117ED */
  0x0000,   /* U+117EE */
  0x0000,   /* U+117EF */
  0x0000,   /* U+117F0 */
  0x0000,   /* U+117F1 */
  0x0000,   /* U+117F2 */
  0x0000,   /* U+117F3 */
  0x0000,   /* U+117F4 */
  0x0000,   /* U+117F5 */
  0x0000,   /* U+117F6 */
  0x0000,   /* U+117F7 */
  0x0000,   /* U+117F8 */
  0x0000,   /* U+117F9 */
  0x0000,   /* U+117FA */
  0x0000,   /* U+117FB */
  0x0000,   /* U+117FC */
  0x0000,   /* U+117FD */
  0x0000,   /* U+117FE */
  0x0000,   /* U+117FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11700 */
  0x0000,   /* U+11701 */
  0x0000,   /* U+11702 */
  0x0000,   /* U+11703 */
  0x0000,   /* U+11704 */
  0x0004,   /* U+11705 */
  0x0000,   /* U+11706 */
  0x0000,   /* U+11707 */
  0x0000,   /* U+11708 */
  0x0000,   /* U+11709 */
  0x0000,   /* U+1170A */
  0x0000,   /* U+1170B */
  0x0000,   /* U+1170C */
  0x0000,   /* U+1170D */
  0x0000,   /* U+1170E */
  0x0000,   /* U+1170F */
  0x0000,   /* U+11710 */
  0x0000,   /* U+11711 */
  0x0000,   /* U+11712 */
  0x0000,   /* U+11713 */
  0x0000,   /* U+11714 */
  0x0000,   /* U+11715 */
  0x0004,   /* U+11716 */
  0x0000,   /* U+11717 */
  0x0000,   /* U+11718 */
  0x0000,   /* U+11719 */
  0x0000,   /* U+1171A */
  0x0000,   /* U+1171B */
  0x0000,   /* U+1171C */
  0x0000,   /* U+1171D */
  0x0000,   /* U+1171E */
  0x0000,   /* U+1171F */
  0x0000,   /* U+11720 */
  0x0000,   /* U+11721 */
  0x0000,   /* U+11722 */
  0x0000,   /* U+11723 */
  0x0000,   /* U+11724 */
  0x0000,   /* U+11725 */
  0x0000,   /* U+11726 */
  0x0000,   /* U+11727 */
  0x0000,   /* U+11728 */
  0x0000,   /* U+11729 */
  0x0000,   /* U+1172A */
  0x0000,   /* U+1172B */
  0x0000,   /* U+1172C */
  0x0000,   /* U+1172D */
  0x0000,   /* U+1172E */
  0x0000,   /* U+1172F */
  0x0000,   /* U+11730 */
  0x0000,   /* U+11731 */
  0x0000,   /* U+11732 */
  0x0000,   /* U+11733 */
  0x0000,   /* U+11734 */
  0x0000,   /* U+11735 */
  0x0000,   /* U+11736 */
  0x0000,   /* U+11737 */
  0x0000,   /* U+11738 */
  0x0000,   /* U+11739 */
  0x0000,   /* U+1173A */
  0x0000,   /* U+1173B */
  0x0000,   /* U+1173C */
  0x0000,   /* U+1173D */
  0x0000,   /* U+1173E */
  0x0000,   /* U+1173F */
  0x0000,   /* U+11740 */
  0x0000,   /* U+11741 */
  0x0000,   /* U+11742 */
  0x0000,   /* U+11743 */
  0x0000,   /* U+11744 */
  0x0000,   /* U+11745 */
  0x0000,   /* U+11746 */
  0x0000,   /* U+11747 */
  0x0000,   /* U+11748 */
  0x0000,   /* U+11749 */
  0x0000,   /* U+1174A */
  0x0000,   /* U+1174B */
  0x0000,   /* U+1174C */
  0x0000,   /* U+1174D */
  0x0000,   /* U+1174E */
  0x0000,   /* U+1174F */
  0x0000,   /* U+11750 */
  0x0000,   /* U+11751 */
  0x0000,   /* U+11752 */
  0x0000,   /* U+11753 */
  0x0000,   /* U+11754 */
  0x0000,   /* U+11755 */
  0x0000,   /* U+11756 */
  0x0000,   /* U+11757 */
  0x0000,   /* U+11758 */
  0x0000,   /* U+11759 */
  0x0000,   /* U+1175A */
  0x0000,   /* U+1175B */
  0x0000,   /* U+1175C */
  0x0000,   /* U+1175D */
  0x0000,   /* U+1175E */
  0x0000,   /* U+1175F */
  0x0000,   /* U+11760 */
  0x0000,   /* U+11761 */
  0x0000,   /* U+11762 */
  0x0000,   /* U+11763 */
  0x0000,   /* U+11764 */
  0x0000,   /* U+11765 */
  0x0000,   /* U+11766 */
  0x0000,   /* U+11767 */
  0x0000,   /* U+11768 */
  0x0000,   /* U+11769 */
  0x0000,   /* U+1176A */
  0x0000,   /* U+1176B */
  0x0000,   /* U+1176C */
  0x0000,   /* U+1176D */
  0x0000,   /* U+1176E */
  0x0000,   /* U+1176F */
  0x0000,   /* U+11770 */
  0x0000,   /* U+11771 */
  0x0000,   /* U+11772 */
  0x0000,   /* U+11773 */
  0x0000,   /* U+11774 */
  0x0000,   /* U+11775 */
  0x0000,   /* U+11776 */
  0x0000,   /* U+11777 */
  0x0000,   /* U+11778 */
  0x0000,   /* U+11779 */
  0x0000,   /* U+1177A */
  0x0000,   /* U+1177B */
  0x0000,   /* U+1177C */
  0x0000,   /* U+1177D */
  0x0000,   /* U+1177E */
  0x0000,   /* U+1177F */
  0x0000,   /* U+11780 */
  0x0000,   /* U+11781 */
  0x0000,   /* U+11782 */
  0x0000,   /* U+11783 */
  0x0000,   /* U+11784 */
  0x0000,   /* U+11785 */
  0x0000,   /* U+11786 */
  0x0000,   /* U+11787 */
  0x0000,   /* U+11788 */
  0x0000,   /* U+11789 */
  0x0000,   /* U+1178A */
  0x0000,   /* U+1178B */
  0x0000,   /* U+1178C */
  0x0000,   /* U+1178D */
  0x0000,   /* U+1178E */
  0x0000,   /* U+1178F */
  0x0000,   /* U+11790 */
  0x0000,   /* U+11791 */
  0x0000,   /* U+11792 */
  0x0000,   /* U+11793 */
  0x0000,   /* U+11794 */
  0x0000,   /* U+11795 */
  0x0000,   /* U+11796 */
  0x0000,   /* U+11797 */
  0x0000,   /* U+11798 */
  0x0000,   /* U+11799 */
  0x0000,   /* U+1179A */
  0x0000,   /* U+1179B */
  0x0000,   /* U+1179C */
  0x0000,   /* U+1179D */
  0x0000,   /* U+1179E */
  0x0000,   /* U+1179F */
  0x0000,   /* U+117A0 */
  0x0000,   /* U+117A1 */
  0x0000,   /* U+117A2 */
  0x0000,   /* U+117A3 */
  0x0000,   /* U+117A4 */
  0x0000,   /* U+117A5 */
  0x0000,   /* U+117A6 */
  0x0000,   /* U+117A7 */
  0x0000,   /* U+117A8 */
  0x0000,   /* U+117A9 */
  0x0000,   /* U+117AA */
  0x0000,   /* U+117AB */
  0x0000,   /* U+117AC */
  0x0000,   /* U+117AD */
  0x0000,   /* U+117AE */
  0x0000,   /* U+117AF */
  0x0000,   /* U+117B0 */
  0x0000,   /* U+117B1 */
  0x0000,   /* U+117B2 */
  0x0000,   /* U+117B3 */
  0x0000,   /* U+117B4 */
  0x0000,   /* U+117B5 */
  0x0000,   /* U+117B6 */
  0x0000,   /* U+117B7 */
  0x0000,   /* U+117B8 */
  0x0000,   /* U+117B9 */
  0x0000,   /* U+117BA */
  0x0000,   /* U+117BB */
  0x0000,   /* U+117BC */
  0x0000,   /* U+117BD */
  0x0000,   /* U+117BE */
  0x0000,   /* U+117BF */
  0x0000,   /* U+117C0 */
  0x0000,   /* U+117C1 */
  0x0000,   /* U+117C2 */
  0x0000,   /* U+117C3 */
  0x0000,   /* U+117C4 */
  0x0000,   /* U+117C5 */
  0x0000,   /* U+117C6 */
  0x0000,   /* U+117C7 */
  0x0000,   /* U+117C8 */
  0x0000,   /* U+117C9 */
  0x0000,   /* U+117CA */
  0x0000,   /* U+117CB */
  0x0000,   /* U+117CC */
  0x0000,   /* U+117CD */
  0x0000,   /* U+117CE */
  0x0000,   /* U+117CF */
  0x0000,   /* U+117D0 */
  0x0000,   /* U+117D1 */
  0x0000,   /* U+117D2 */
  0x0000,   /* U+117D3 */
  0x0000,   /* U+117D4 */
  0x0000,   /* U+117D5 */
  0x0000,   /* U+117D6 */
  0x0000,   /* U+117D7 */
  0x0000,   /* U+117D8 */
  0x0000,   /* U+117D9 */
  0x0000,   /* U+117DA */
  0x0000,   /* U+117DB */
  0x0000,   /* U+117DC */
  0x0000,   /* U+117DD */
  0x0000,   /* U+117DE */
  0x0000,   /* U+117DF */
  0x0000,   /* U+117E0 */
  0x0000,   /* U+117E1 */
  0x0000,   /* U+117E2 */
  0x0000,   /* U+117E3 */
  0x0000,   /* U+117E4 */
  0x0000,   /* U+117E5 */
  0x0000,   /* U+117E6 */
  0x0000,   /* U+117E7 */
  0x0000,   /* U+117E8 */
  0x0000,   /* U+117E9 */
  0x0000,   /* U+117EA */
  0x0000,   /* U+117EB */
  0x0000,   /* U+117EC */
  0x0000,   /* U+117ED */
  0x0000,   /* U+117EE */
  0x0000,   /* U+117EF */
  0x0000,   /* U+117F0 */
  0x0000,   /* U+117F1 */
  0x0000,   /* U+117F2 */
  0x0000,   /* U+117F3 */
  0x0000,   /* U+117F4 */
  0x0000,   /* U+117F5 */
  0x0000,   /* U+117F6 */
  0x0000,   /* U+117F7 */
  0x0000,   /* U+117F8 */
  0x0000,   /* U+117F9 */
  0x0000,   /* U+117FA */
  0x0000,   /* U+117FB */
  0x0000,   /* U+117FC */
  0x0000,   /* U+117FD */
  0x0000,   /* U+117FE */
  0x0000,   /* U+117FF */
};

uint16 uca900_p118[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 

  /* Primary weight 1 for each character. */
  0xFBC2,   /* U+11800 */
  0xFBC2,   /* U+11801 */
  0xFBC2,   /* U+11802 */
  0xFBC2,   /* U+11803 */
  0xFBC2,   /* U+11804 */
  0xFBC2,   /* U+11805 */
  0xFBC2,   /* U+11806 */
  0xFBC2,   /* U+11807 */
  0xFBC2,   /* U+11808 */
  0xFBC2,   /* U+11809 */
  0xFBC2,   /* U+1180A */
  0xFBC2,   /* U+1180B */
  0xFBC2,   /* U+1180C */
  0xFBC2,   /* U+1180D */
  0xFBC2,   /* U+1180E */
  0xFBC2,   /* U+1180F */
  0xFBC2,   /* U+11810 */
  0xFBC2,   /* U+11811 */
  0xFBC2,   /* U+11812 */
  0xFBC2,   /* U+11813 */
  0xFBC2,   /* U+11814 */
  0xFBC2,   /* U+11815 */
  0xFBC2,   /* U+11816 */
  0xFBC2,   /* U+11817 */
  0xFBC2,   /* U+11818 */
  0xFBC2,   /* U+11819 */
  0xFBC2,   /* U+1181A */
  0xFBC2,   /* U+1181B */
  0xFBC2,   /* U+1181C */
  0xFBC2,   /* U+1181D */
  0xFBC2,   /* U+1181E */
  0xFBC2,   /* U+1181F */
  0xFBC2,   /* U+11820 */
  0xFBC2,   /* U+11821 */
  0xFBC2,   /* U+11822 */
  0xFBC2,   /* U+11823 */
  0xFBC2,   /* U+11824 */
  0xFBC2,   /* U+11825 */
  0xFBC2,   /* U+11826 */
  0xFBC2,   /* U+11827 */
  0xFBC2,   /* U+11828 */
  0xFBC2,   /* U+11829 */
  0xFBC2,   /* U+1182A */
  0xFBC2,   /* U+1182B */
  0xFBC2,   /* U+1182C */
  0xFBC2,   /* U+1182D */
  0xFBC2,   /* U+1182E */
  0xFBC2,   /* U+1182F */
  0xFBC2,   /* U+11830 */
  0xFBC2,   /* U+11831 */
  0xFBC2,   /* U+11832 */
  0xFBC2,   /* U+11833 */
  0xFBC2,   /* U+11834 */
  0xFBC2,   /* U+11835 */
  0xFBC2,   /* U+11836 */
  0xFBC2,   /* U+11837 */
  0xFBC2,   /* U+11838 */
  0xFBC2,   /* U+11839 */
  0xFBC2,   /* U+1183A */
  0xFBC2,   /* U+1183B */
  0xFBC2,   /* U+1183C */
  0xFBC2,   /* U+1183D */
  0xFBC2,   /* U+1183E */
  0xFBC2,   /* U+1183F */
  0xFBC2,   /* U+11840 */
  0xFBC2,   /* U+11841 */
  0xFBC2,   /* U+11842 */
  0xFBC2,   /* U+11843 */
  0xFBC2,   /* U+11844 */
  0xFBC2,   /* U+11845 */
  0xFBC2,   /* U+11846 */
  0xFBC2,   /* U+11847 */
  0xFBC2,   /* U+11848 */
  0xFBC2,   /* U+11849 */
  0xFBC2,   /* U+1184A */
  0xFBC2,   /* U+1184B */
  0xFBC2,   /* U+1184C */
  0xFBC2,   /* U+1184D */
  0xFBC2,   /* U+1184E */
  0xFBC2,   /* U+1184F */
  0xFBC2,   /* U+11850 */
  0xFBC2,   /* U+11851 */
  0xFBC2,   /* U+11852 */
  0xFBC2,   /* U+11853 */
  0xFBC2,   /* U+11854 */
  0xFBC2,   /* U+11855 */
  0xFBC2,   /* U+11856 */
  0xFBC2,   /* U+11857 */
  0xFBC2,   /* U+11858 */
  0xFBC2,   /* U+11859 */
  0xFBC2,   /* U+1185A */
  0xFBC2,   /* U+1185B */
  0xFBC2,   /* U+1185C */
  0xFBC2,   /* U+1185D */
  0xFBC2,   /* U+1185E */
  0xFBC2,   /* U+1185F */
  0xFBC2,   /* U+11860 */
  0xFBC2,   /* U+11861 */
  0xFBC2,   /* U+11862 */
  0xFBC2,   /* U+11863 */
  0xFBC2,   /* U+11864 */
  0xFBC2,   /* U+11865 */
  0xFBC2,   /* U+11866 */
  0xFBC2,   /* U+11867 */
  0xFBC2,   /* U+11868 */
  0xFBC2,   /* U+11869 */
  0xFBC2,   /* U+1186A */
  0xFBC2,   /* U+1186B */
  0xFBC2,   /* U+1186C */
  0xFBC2,   /* U+1186D */
  0xFBC2,   /* U+1186E */
  0xFBC2,   /* U+1186F */
  0xFBC2,   /* U+11870 */
  0xFBC2,   /* U+11871 */
  0xFBC2,   /* U+11872 */
  0xFBC2,   /* U+11873 */
  0xFBC2,   /* U+11874 */
  0xFBC2,   /* U+11875 */
  0xFBC2,   /* U+11876 */
  0xFBC2,   /* U+11877 */
  0xFBC2,   /* U+11878 */
  0xFBC2,   /* U+11879 */
  0xFBC2,   /* U+1187A */
  0xFBC2,   /* U+1187B */
  0xFBC2,   /* U+1187C */
  0xFBC2,   /* U+1187D */
  0xFBC2,   /* U+1187E */
  0xFBC2,   /* U+1187F */
  0xFBC2,   /* U+11880 */
  0xFBC2,   /* U+11881 */
  0xFBC2,   /* U+11882 */
  0xFBC2,   /* U+11883 */
  0xFBC2,   /* U+11884 */
  0xFBC2,   /* U+11885 */
  0xFBC2,   /* U+11886 */
  0xFBC2,   /* U+11887 */
  0xFBC2,   /* U+11888 */
  0xFBC2,   /* U+11889 */
  0xFBC2,   /* U+1188A */
  0xFBC2,   /* U+1188B */
  0xFBC2,   /* U+1188C */
  0xFBC2,   /* U+1188D */
  0xFBC2,   /* U+1188E */
  0xFBC2,   /* U+1188F */
  0xFBC2,   /* U+11890 */
  0xFBC2,   /* U+11891 */
  0xFBC2,   /* U+11892 */
  0xFBC2,   /* U+11893 */
  0xFBC2,   /* U+11894 */
  0xFBC2,   /* U+11895 */
  0xFBC2,   /* U+11896 */
  0xFBC2,   /* U+11897 */
  0xFBC2,   /* U+11898 */
  0xFBC2,   /* U+11899 */
  0xFBC2,   /* U+1189A */
  0xFBC2,   /* U+1189B */
  0xFBC2,   /* U+1189C */
  0xFBC2,   /* U+1189D */
  0xFBC2,   /* U+1189E */
  0xFBC2,   /* U+1189F */
  0x42FC,   /* U+118A0 */
  0x42FD,   /* U+118A1 */
  0x42FE,   /* U+118A2 */
  0x42FF,   /* U+118A3 */
  0x4300,   /* U+118A4 */
  0x4301,   /* U+118A5 */
  0x4302,   /* U+118A6 */
  0x4303,   /* U+118A7 */
  0x4304,   /* U+118A8 */
  0x4305,   /* U+118A9 */
  0x4306,   /* U+118AA */
  0x4307,   /* U+118AB */
  0x4308,   /* U+118AC */
  0x4309,   /* U+118AD */
  0x430A,   /* U+118AE */
  0x430B,   /* U+118AF */
  0x430C,   /* U+118B0 */
  0x430D,   /* U+118B1 */
  0x430E,   /* U+118B2 */
  0x430F,   /* U+118B3 */
  0x4310,   /* U+118B4 */
  0x4311,   /* U+118B5 */
  0x4312,   /* U+118B6 */
  0x4313,   /* U+118B7 */
  0x4314,   /* U+118B8 */
  0x4315,   /* U+118B9 */
  0x4316,   /* U+118BA */
  0x4317,   /* U+118BB */
  0x4318,   /* U+118BC */
  0x4319,   /* U+118BD */
  0x431A,   /* U+118BE */
  0x431B,   /* U+118BF */
  0x42FC,   /* U+118C0 */
  0x42FD,   /* U+118C1 */
  0x42FE,   /* U+118C2 */
  0x42FF,   /* U+118C3 */
  0x4300,   /* U+118C4 */
  0x4301,   /* U+118C5 */
  0x4302,   /* U+118C6 */
  0x4303,   /* U+118C7 */
  0x4304,   /* U+118C8 */
  0x4305,   /* U+118C9 */
  0x4306,   /* U+118CA */
  0x4307,   /* U+118CB */
  0x4308,   /* U+118CC */
  0x4309,   /* U+118CD */
  0x430A,   /* U+118CE */
  0x430B,   /* U+118CF */
  0x430C,   /* U+118D0 */
  0x430D,   /* U+118D1 */
  0x430E,   /* U+118D2 */
  0x430F,   /* U+118D3 */
  0x4310,   /* U+118D4 */
  0x4311,   /* U+118D5 */
  0x4312,   /* U+118D6 */
  0x4313,   /* U+118D7 */
  0x4314,   /* U+118D8 */
  0x4315,   /* U+118D9 */
  0x4316,   /* U+118DA */
  0x4317,   /* U+118DB */
  0x4318,   /* U+118DC */
  0x4319,   /* U+118DD */
  0x431A,   /* U+118DE */
  0x431B,   /* U+118DF */
  0x1C3D,   /* U+118E0 */
  0x1C3E,   /* U+118E1 */
  0x1C3F,   /* U+118E2 */
  0x1C40,   /* U+118E3 */
  0x1C41,   /* U+118E4 */
  0x1C42,   /* U+118E5 */
  0x1C43,   /* U+118E6 */
  0x1C44,   /* U+118E7 */
  0x1C45,   /* U+118E8 */
  0x1C46,   /* U+118E9 */
  0x1B97,   /* U+118EA */
  0x1B98,   /* U+118EB */
  0x1B99,   /* U+118EC */
  0x1B9A,   /* U+118ED */
  0x1B9B,   /* U+118EE */
  0x1B9C,   /* U+118EF */
  0x1B9D,   /* U+118F0 */
  0x1B9E,   /* U+118F1 */
  0x1B9F,   /* U+118F2 */
  0xFBC2,   /* U+118F3 */
  0xFBC2,   /* U+118F4 */
  0xFBC2,   /* U+118F5 */
  0xFBC2,   /* U+118F6 */
  0xFBC2,   /* U+118F7 */
  0xFBC2,   /* U+118F8 */
  0xFBC2,   /* U+118F9 */
  0xFBC2,   /* U+118FA */
  0xFBC2,   /* U+118FB */
  0xFBC2,   /* U+118FC */
  0xFBC2,   /* U+118FD */
  0xFBC2,   /* U+118FE */
  0x42FB,   /* U+118FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11800 */
  0x0020,   /* U+11801 */
  0x0020,   /* U+11802 */
  0x0020,   /* U+11803 */
  0x0020,   /* U+11804 */
  0x0020,   /* U+11805 */
  0x0020,   /* U+11806 */
  0x0020,   /* U+11807 */
  0x0020,   /* U+11808 */
  0x0020,   /* U+11809 */
  0x0020,   /* U+1180A */
  0x0020,   /* U+1180B */
  0x0020,   /* U+1180C */
  0x0020,   /* U+1180D */
  0x0020,   /* U+1180E */
  0x0020,   /* U+1180F */
  0x0020,   /* U+11810 */
  0x0020,   /* U+11811 */
  0x0020,   /* U+11812 */
  0x0020,   /* U+11813 */
  0x0020,   /* U+11814 */
  0x0020,   /* U+11815 */
  0x0020,   /* U+11816 */
  0x0020,   /* U+11817 */
  0x0020,   /* U+11818 */
  0x0020,   /* U+11819 */
  0x0020,   /* U+1181A */
  0x0020,   /* U+1181B */
  0x0020,   /* U+1181C */
  0x0020,   /* U+1181D */
  0x0020,   /* U+1181E */
  0x0020,   /* U+1181F */
  0x0020,   /* U+11820 */
  0x0020,   /* U+11821 */
  0x0020,   /* U+11822 */
  0x0020,   /* U+11823 */
  0x0020,   /* U+11824 */
  0x0020,   /* U+11825 */
  0x0020,   /* U+11826 */
  0x0020,   /* U+11827 */
  0x0020,   /* U+11828 */
  0x0020,   /* U+11829 */
  0x0020,   /* U+1182A */
  0x0020,   /* U+1182B */
  0x0020,   /* U+1182C */
  0x0020,   /* U+1182D */
  0x0020,   /* U+1182E */
  0x0020,   /* U+1182F */
  0x0020,   /* U+11830 */
  0x0020,   /* U+11831 */
  0x0020,   /* U+11832 */
  0x0020,   /* U+11833 */
  0x0020,   /* U+11834 */
  0x0020,   /* U+11835 */
  0x0020,   /* U+11836 */
  0x0020,   /* U+11837 */
  0x0020,   /* U+11838 */
  0x0020,   /* U+11839 */
  0x0020,   /* U+1183A */
  0x0020,   /* U+1183B */
  0x0020,   /* U+1183C */
  0x0020,   /* U+1183D */
  0x0020,   /* U+1183E */
  0x0020,   /* U+1183F */
  0x0020,   /* U+11840 */
  0x0020,   /* U+11841 */
  0x0020,   /* U+11842 */
  0x0020,   /* U+11843 */
  0x0020,   /* U+11844 */
  0x0020,   /* U+11845 */
  0x0020,   /* U+11846 */
  0x0020,   /* U+11847 */
  0x0020,   /* U+11848 */
  0x0020,   /* U+11849 */
  0x0020,   /* U+1184A */
  0x0020,   /* U+1184B */
  0x0020,   /* U+1184C */
  0x0020,   /* U+1184D */
  0x0020,   /* U+1184E */
  0x0020,   /* U+1184F */
  0x0020,   /* U+11850 */
  0x0020,   /* U+11851 */
  0x0020,   /* U+11852 */
  0x0020,   /* U+11853 */
  0x0020,   /* U+11854 */
  0x0020,   /* U+11855 */
  0x0020,   /* U+11856 */
  0x0020,   /* U+11857 */
  0x0020,   /* U+11858 */
  0x0020,   /* U+11859 */
  0x0020,   /* U+1185A */
  0x0020,   /* U+1185B */
  0x0020,   /* U+1185C */
  0x0020,   /* U+1185D */
  0x0020,   /* U+1185E */
  0x0020,   /* U+1185F */
  0x0020,   /* U+11860 */
  0x0020,   /* U+11861 */
  0x0020,   /* U+11862 */
  0x0020,   /* U+11863 */
  0x0020,   /* U+11864 */
  0x0020,   /* U+11865 */
  0x0020,   /* U+11866 */
  0x0020,   /* U+11867 */
  0x0020,   /* U+11868 */
  0x0020,   /* U+11869 */
  0x0020,   /* U+1186A */
  0x0020,   /* U+1186B */
  0x0020,   /* U+1186C */
  0x0020,   /* U+1186D */
  0x0020,   /* U+1186E */
  0x0020,   /* U+1186F */
  0x0020,   /* U+11870 */
  0x0020,   /* U+11871 */
  0x0020,   /* U+11872 */
  0x0020,   /* U+11873 */
  0x0020,   /* U+11874 */
  0x0020,   /* U+11875 */
  0x0020,   /* U+11876 */
  0x0020,   /* U+11877 */
  0x0020,   /* U+11878 */
  0x0020,   /* U+11879 */
  0x0020,   /* U+1187A */
  0x0020,   /* U+1187B */
  0x0020,   /* U+1187C */
  0x0020,   /* U+1187D */
  0x0020,   /* U+1187E */
  0x0020,   /* U+1187F */
  0x0020,   /* U+11880 */
  0x0020,   /* U+11881 */
  0x0020,   /* U+11882 */
  0x0020,   /* U+11883 */
  0x0020,   /* U+11884 */
  0x0020,   /* U+11885 */
  0x0020,   /* U+11886 */
  0x0020,   /* U+11887 */
  0x0020,   /* U+11888 */
  0x0020,   /* U+11889 */
  0x0020,   /* U+1188A */
  0x0020,   /* U+1188B */
  0x0020,   /* U+1188C */
  0x0020,   /* U+1188D */
  0x0020,   /* U+1188E */
  0x0020,   /* U+1188F */
  0x0020,   /* U+11890 */
  0x0020,   /* U+11891 */
  0x0020,   /* U+11892 */
  0x0020,   /* U+11893 */
  0x0020,   /* U+11894 */
  0x0020,   /* U+11895 */
  0x0020,   /* U+11896 */
  0x0020,   /* U+11897 */
  0x0020,   /* U+11898 */
  0x0020,   /* U+11899 */
  0x0020,   /* U+1189A */
  0x0020,   /* U+1189B */
  0x0020,   /* U+1189C */
  0x0020,   /* U+1189D */
  0x0020,   /* U+1189E */
  0x0020,   /* U+1189F */
  0x0020,   /* U+118A0 */
  0x0020,   /* U+118A1 */
  0x0020,   /* U+118A2 */
  0x0020,   /* U+118A3 */
  0x0020,   /* U+118A4 */
  0x0020,   /* U+118A5 */
  0x0020,   /* U+118A6 */
  0x0020,   /* U+118A7 */
  0x0020,   /* U+118A8 */
  0x0020,   /* U+118A9 */
  0x0020,   /* U+118AA */
  0x0020,   /* U+118AB */
  0x0020,   /* U+118AC */
  0x0020,   /* U+118AD */
  0x0020,   /* U+118AE */
  0x0020,   /* U+118AF */
  0x0020,   /* U+118B0 */
  0x0020,   /* U+118B1 */
  0x0020,   /* U+118B2 */
  0x0020,   /* U+118B3 */
  0x0020,   /* U+118B4 */
  0x0020,   /* U+118B5 */
  0x0020,   /* U+118B6 */
  0x0020,   /* U+118B7 */
  0x0020,   /* U+118B8 */
  0x0020,   /* U+118B9 */
  0x0020,   /* U+118BA */
  0x0020,   /* U+118BB */
  0x0020,   /* U+118BC */
  0x0020,   /* U+118BD */
  0x0020,   /* U+118BE */
  0x0020,   /* U+118BF */
  0x0020,   /* U+118C0 */
  0x0020,   /* U+118C1 */
  0x0020,   /* U+118C2 */
  0x0020,   /* U+118C3 */
  0x0020,   /* U+118C4 */
  0x0020,   /* U+118C5 */
  0x0020,   /* U+118C6 */
  0x0020,   /* U+118C7 */
  0x0020,   /* U+118C8 */
  0x0020,   /* U+118C9 */
  0x0020,   /* U+118CA */
  0x0020,   /* U+118CB */
  0x0020,   /* U+118CC */
  0x0020,   /* U+118CD */
  0x0020,   /* U+118CE */
  0x0020,   /* U+118CF */
  0x0020,   /* U+118D0 */
  0x0020,   /* U+118D1 */
  0x0020,   /* U+118D2 */
  0x0020,   /* U+118D3 */
  0x0020,   /* U+118D4 */
  0x0020,   /* U+118D5 */
  0x0020,   /* U+118D6 */
  0x0020,   /* U+118D7 */
  0x0020,   /* U+118D8 */
  0x0020,   /* U+118D9 */
  0x0020,   /* U+118DA */
  0x0020,   /* U+118DB */
  0x0020,   /* U+118DC */
  0x0020,   /* U+118DD */
  0x0020,   /* U+118DE */
  0x0020,   /* U+118DF */
  0x0020,   /* U+118E0 */
  0x0020,   /* U+118E1 */
  0x0020,   /* U+118E2 */
  0x0020,   /* U+118E3 */
  0x0020,   /* U+118E4 */
  0x0020,   /* U+118E5 */
  0x0020,   /* U+118E6 */
  0x0020,   /* U+118E7 */
  0x0020,   /* U+118E8 */
  0x0020,   /* U+118E9 */
  0x0020,   /* U+118EA */
  0x0020,   /* U+118EB */
  0x0020,   /* U+118EC */
  0x0020,   /* U+118ED */
  0x0020,   /* U+118EE */
  0x0020,   /* U+118EF */
  0x0020,   /* U+118F0 */
  0x0020,   /* U+118F1 */
  0x0020,   /* U+118F2 */
  0x0020,   /* U+118F3 */
  0x0020,   /* U+118F4 */
  0x0020,   /* U+118F5 */
  0x0020,   /* U+118F6 */
  0x0020,   /* U+118F7 */
  0x0020,   /* U+118F8 */
  0x0020,   /* U+118F9 */
  0x0020,   /* U+118FA */
  0x0020,   /* U+118FB */
  0x0020,   /* U+118FC */
  0x0020,   /* U+118FD */
  0x0020,   /* U+118FE */
  0x0020,   /* U+118FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11800 */
  0x0002,   /* U+11801 */
  0x0002,   /* U+11802 */
  0x0002,   /* U+11803 */
  0x0002,   /* U+11804 */
  0x0002,   /* U+11805 */
  0x0002,   /* U+11806 */
  0x0002,   /* U+11807 */
  0x0002,   /* U+11808 */
  0x0002,   /* U+11809 */
  0x0002,   /* U+1180A */
  0x0002,   /* U+1180B */
  0x0002,   /* U+1180C */
  0x0002,   /* U+1180D */
  0x0002,   /* U+1180E */
  0x0002,   /* U+1180F */
  0x0002,   /* U+11810 */
  0x0002,   /* U+11811 */
  0x0002,   /* U+11812 */
  0x0002,   /* U+11813 */
  0x0002,   /* U+11814 */
  0x0002,   /* U+11815 */
  0x0002,   /* U+11816 */
  0x0002,   /* U+11817 */
  0x0002,   /* U+11818 */
  0x0002,   /* U+11819 */
  0x0002,   /* U+1181A */
  0x0002,   /* U+1181B */
  0x0002,   /* U+1181C */
  0x0002,   /* U+1181D */
  0x0002,   /* U+1181E */
  0x0002,   /* U+1181F */
  0x0002,   /* U+11820 */
  0x0002,   /* U+11821 */
  0x0002,   /* U+11822 */
  0x0002,   /* U+11823 */
  0x0002,   /* U+11824 */
  0x0002,   /* U+11825 */
  0x0002,   /* U+11826 */
  0x0002,   /* U+11827 */
  0x0002,   /* U+11828 */
  0x0002,   /* U+11829 */
  0x0002,   /* U+1182A */
  0x0002,   /* U+1182B */
  0x0002,   /* U+1182C */
  0x0002,   /* U+1182D */
  0x0002,   /* U+1182E */
  0x0002,   /* U+1182F */
  0x0002,   /* U+11830 */
  0x0002,   /* U+11831 */
  0x0002,   /* U+11832 */
  0x0002,   /* U+11833 */
  0x0002,   /* U+11834 */
  0x0002,   /* U+11835 */
  0x0002,   /* U+11836 */
  0x0002,   /* U+11837 */
  0x0002,   /* U+11838 */
  0x0002,   /* U+11839 */
  0x0002,   /* U+1183A */
  0x0002,   /* U+1183B */
  0x0002,   /* U+1183C */
  0x0002,   /* U+1183D */
  0x0002,   /* U+1183E */
  0x0002,   /* U+1183F */
  0x0002,   /* U+11840 */
  0x0002,   /* U+11841 */
  0x0002,   /* U+11842 */
  0x0002,   /* U+11843 */
  0x0002,   /* U+11844 */
  0x0002,   /* U+11845 */
  0x0002,   /* U+11846 */
  0x0002,   /* U+11847 */
  0x0002,   /* U+11848 */
  0x0002,   /* U+11849 */
  0x0002,   /* U+1184A */
  0x0002,   /* U+1184B */
  0x0002,   /* U+1184C */
  0x0002,   /* U+1184D */
  0x0002,   /* U+1184E */
  0x0002,   /* U+1184F */
  0x0002,   /* U+11850 */
  0x0002,   /* U+11851 */
  0x0002,   /* U+11852 */
  0x0002,   /* U+11853 */
  0x0002,   /* U+11854 */
  0x0002,   /* U+11855 */
  0x0002,   /* U+11856 */
  0x0002,   /* U+11857 */
  0x0002,   /* U+11858 */
  0x0002,   /* U+11859 */
  0x0002,   /* U+1185A */
  0x0002,   /* U+1185B */
  0x0002,   /* U+1185C */
  0x0002,   /* U+1185D */
  0x0002,   /* U+1185E */
  0x0002,   /* U+1185F */
  0x0002,   /* U+11860 */
  0x0002,   /* U+11861 */
  0x0002,   /* U+11862 */
  0x0002,   /* U+11863 */
  0x0002,   /* U+11864 */
  0x0002,   /* U+11865 */
  0x0002,   /* U+11866 */
  0x0002,   /* U+11867 */
  0x0002,   /* U+11868 */
  0x0002,   /* U+11869 */
  0x0002,   /* U+1186A */
  0x0002,   /* U+1186B */
  0x0002,   /* U+1186C */
  0x0002,   /* U+1186D */
  0x0002,   /* U+1186E */
  0x0002,   /* U+1186F */
  0x0002,   /* U+11870 */
  0x0002,   /* U+11871 */
  0x0002,   /* U+11872 */
  0x0002,   /* U+11873 */
  0x0002,   /* U+11874 */
  0x0002,   /* U+11875 */
  0x0002,   /* U+11876 */
  0x0002,   /* U+11877 */
  0x0002,   /* U+11878 */
  0x0002,   /* U+11879 */
  0x0002,   /* U+1187A */
  0x0002,   /* U+1187B */
  0x0002,   /* U+1187C */
  0x0002,   /* U+1187D */
  0x0002,   /* U+1187E */
  0x0002,   /* U+1187F */
  0x0002,   /* U+11880 */
  0x0002,   /* U+11881 */
  0x0002,   /* U+11882 */
  0x0002,   /* U+11883 */
  0x0002,   /* U+11884 */
  0x0002,   /* U+11885 */
  0x0002,   /* U+11886 */
  0x0002,   /* U+11887 */
  0x0002,   /* U+11888 */
  0x0002,   /* U+11889 */
  0x0002,   /* U+1188A */
  0x0002,   /* U+1188B */
  0x0002,   /* U+1188C */
  0x0002,   /* U+1188D */
  0x0002,   /* U+1188E */
  0x0002,   /* U+1188F */
  0x0002,   /* U+11890 */
  0x0002,   /* U+11891 */
  0x0002,   /* U+11892 */
  0x0002,   /* U+11893 */
  0x0002,   /* U+11894 */
  0x0002,   /* U+11895 */
  0x0002,   /* U+11896 */
  0x0002,   /* U+11897 */
  0x0002,   /* U+11898 */
  0x0002,   /* U+11899 */
  0x0002,   /* U+1189A */
  0x0002,   /* U+1189B */
  0x0002,   /* U+1189C */
  0x0002,   /* U+1189D */
  0x0002,   /* U+1189E */
  0x0002,   /* U+1189F */
  0x0008,   /* U+118A0 */
  0x0008,   /* U+118A1 */
  0x0008,   /* U+118A2 */
  0x0008,   /* U+118A3 */
  0x0008,   /* U+118A4 */
  0x0008,   /* U+118A5 */
  0x0008,   /* U+118A6 */
  0x0008,   /* U+118A7 */
  0x0008,   /* U+118A8 */
  0x0008,   /* U+118A9 */
  0x0008,   /* U+118AA */
  0x0008,   /* U+118AB */
  0x0008,   /* U+118AC */
  0x0008,   /* U+118AD */
  0x0008,   /* U+118AE */
  0x0008,   /* U+118AF */
  0x0008,   /* U+118B0 */
  0x0008,   /* U+118B1 */
  0x0008,   /* U+118B2 */
  0x0008,   /* U+118B3 */
  0x0008,   /* U+118B4 */
  0x0008,   /* U+118B5 */
  0x0008,   /* U+118B6 */
  0x0008,   /* U+118B7 */
  0x0008,   /* U+118B8 */
  0x0008,   /* U+118B9 */
  0x0008,   /* U+118BA */
  0x0008,   /* U+118BB */
  0x0008,   /* U+118BC */
  0x0008,   /* U+118BD */
  0x0008,   /* U+118BE */
  0x0008,   /* U+118BF */
  0x0002,   /* U+118C0 */
  0x0002,   /* U+118C1 */
  0x0002,   /* U+118C2 */
  0x0002,   /* U+118C3 */
  0x0002,   /* U+118C4 */
  0x0002,   /* U+118C5 */
  0x0002,   /* U+118C6 */
  0x0002,   /* U+118C7 */
  0x0002,   /* U+118C8 */
  0x0002,   /* U+118C9 */
  0x0002,   /* U+118CA */
  0x0002,   /* U+118CB */
  0x0002,   /* U+118CC */
  0x0002,   /* U+118CD */
  0x0002,   /* U+118CE */
  0x0002,   /* U+118CF */
  0x0002,   /* U+118D0 */
  0x0002,   /* U+118D1 */
  0x0002,   /* U+118D2 */
  0x0002,   /* U+118D3 */
  0x0002,   /* U+118D4 */
  0x0002,   /* U+118D5 */
  0x0002,   /* U+118D6 */
  0x0002,   /* U+118D7 */
  0x0002,   /* U+118D8 */
  0x0002,   /* U+118D9 */
  0x0002,   /* U+118DA */
  0x0002,   /* U+118DB */
  0x0002,   /* U+118DC */
  0x0002,   /* U+118DD */
  0x0002,   /* U+118DE */
  0x0002,   /* U+118DF */
  0x0002,   /* U+118E0 */
  0x0002,   /* U+118E1 */
  0x0002,   /* U+118E2 */
  0x0002,   /* U+118E3 */
  0x0002,   /* U+118E4 */
  0x0002,   /* U+118E5 */
  0x0002,   /* U+118E6 */
  0x0002,   /* U+118E7 */
  0x0002,   /* U+118E8 */
  0x0002,   /* U+118E9 */
  0x0002,   /* U+118EA */
  0x0002,   /* U+118EB */
  0x0002,   /* U+118EC */
  0x0002,   /* U+118ED */
  0x0002,   /* U+118EE */
  0x0002,   /* U+118EF */
  0x0002,   /* U+118F0 */
  0x0002,   /* U+118F1 */
  0x0002,   /* U+118F2 */
  0x0002,   /* U+118F3 */
  0x0002,   /* U+118F4 */
  0x0002,   /* U+118F5 */
  0x0002,   /* U+118F6 */
  0x0002,   /* U+118F7 */
  0x0002,   /* U+118F8 */
  0x0002,   /* U+118F9 */
  0x0002,   /* U+118FA */
  0x0002,   /* U+118FB */
  0x0002,   /* U+118FC */
  0x0002,   /* U+118FD */
  0x0002,   /* U+118FE */
  0x0002,   /* U+118FF */

  /* Primary weight 2 for each character. */
  0x9800,   /* U+11800 */
  0x9801,   /* U+11801 */
  0x9802,   /* U+11802 */
  0x9803,   /* U+11803 */
  0x9804,   /* U+11804 */
  0x9805,   /* U+11805 */
  0x9806,   /* U+11806 */
  0x9807,   /* U+11807 */
  0x9808,   /* U+11808 */
  0x9809,   /* U+11809 */
  0x980A,   /* U+1180A */
  0x980B,   /* U+1180B */
  0x980C,   /* U+1180C */
  0x980D,   /* U+1180D */
  0x980E,   /* U+1180E */
  0x980F,   /* U+1180F */
  0x9810,   /* U+11810 */
  0x9811,   /* U+11811 */
  0x9812,   /* U+11812 */
  0x9813,   /* U+11813 */
  0x9814,   /* U+11814 */
  0x9815,   /* U+11815 */
  0x9816,   /* U+11816 */
  0x9817,   /* U+11817 */
  0x9818,   /* U+11818 */
  0x9819,   /* U+11819 */
  0x981A,   /* U+1181A */
  0x981B,   /* U+1181B */
  0x981C,   /* U+1181C */
  0x981D,   /* U+1181D */
  0x981E,   /* U+1181E */
  0x981F,   /* U+1181F */
  0x9820,   /* U+11820 */
  0x9821,   /* U+11821 */
  0x9822,   /* U+11822 */
  0x9823,   /* U+11823 */
  0x9824,   /* U+11824 */
  0x9825,   /* U+11825 */
  0x9826,   /* U+11826 */
  0x9827,   /* U+11827 */
  0x9828,   /* U+11828 */
  0x9829,   /* U+11829 */
  0x982A,   /* U+1182A */
  0x982B,   /* U+1182B */
  0x982C,   /* U+1182C */
  0x982D,   /* U+1182D */
  0x982E,   /* U+1182E */
  0x982F,   /* U+1182F */
  0x9830,   /* U+11830 */
  0x9831,   /* U+11831 */
  0x9832,   /* U+11832 */
  0x9833,   /* U+11833 */
  0x9834,   /* U+11834 */
  0x9835,   /* U+11835 */
  0x9836,   /* U+11836 */
  0x9837,   /* U+11837 */
  0x9838,   /* U+11838 */
  0x9839,   /* U+11839 */
  0x983A,   /* U+1183A */
  0x983B,   /* U+1183B */
  0x983C,   /* U+1183C */
  0x983D,   /* U+1183D */
  0x983E,   /* U+1183E */
  0x983F,   /* U+1183F */
  0x9840,   /* U+11840 */
  0x9841,   /* U+11841 */
  0x9842,   /* U+11842 */
  0x9843,   /* U+11843 */
  0x9844,   /* U+11844 */
  0x9845,   /* U+11845 */
  0x9846,   /* U+11846 */
  0x9847,   /* U+11847 */
  0x9848,   /* U+11848 */
  0x9849,   /* U+11849 */
  0x984A,   /* U+1184A */
  0x984B,   /* U+1184B */
  0x984C,   /* U+1184C */
  0x984D,   /* U+1184D */
  0x984E,   /* U+1184E */
  0x984F,   /* U+1184F */
  0x9850,   /* U+11850 */
  0x9851,   /* U+11851 */
  0x9852,   /* U+11852 */
  0x9853,   /* U+11853 */
  0x9854,   /* U+11854 */
  0x9855,   /* U+11855 */
  0x9856,   /* U+11856 */
  0x9857,   /* U+11857 */
  0x9858,   /* U+11858 */
  0x9859,   /* U+11859 */
  0x985A,   /* U+1185A */
  0x985B,   /* U+1185B */
  0x985C,   /* U+1185C */
  0x985D,   /* U+1185D */
  0x985E,   /* U+1185E */
  0x985F,   /* U+1185F */
  0x9860,   /* U+11860 */
  0x9861,   /* U+11861 */
  0x9862,   /* U+11862 */
  0x9863,   /* U+11863 */
  0x9864,   /* U+11864 */
  0x9865,   /* U+11865 */
  0x9866,   /* U+11866 */
  0x9867,   /* U+11867 */
  0x9868,   /* U+11868 */
  0x9869,   /* U+11869 */
  0x986A,   /* U+1186A */
  0x986B,   /* U+1186B */
  0x986C,   /* U+1186C */
  0x986D,   /* U+1186D */
  0x986E,   /* U+1186E */
  0x986F,   /* U+1186F */
  0x9870,   /* U+11870 */
  0x9871,   /* U+11871 */
  0x9872,   /* U+11872 */
  0x9873,   /* U+11873 */
  0x9874,   /* U+11874 */
  0x9875,   /* U+11875 */
  0x9876,   /* U+11876 */
  0x9877,   /* U+11877 */
  0x9878,   /* U+11878 */
  0x9879,   /* U+11879 */
  0x987A,   /* U+1187A */
  0x987B,   /* U+1187B */
  0x987C,   /* U+1187C */
  0x987D,   /* U+1187D */
  0x987E,   /* U+1187E */
  0x987F,   /* U+1187F */
  0x9880,   /* U+11880 */
  0x9881,   /* U+11881 */
  0x9882,   /* U+11882 */
  0x9883,   /* U+11883 */
  0x9884,   /* U+11884 */
  0x9885,   /* U+11885 */
  0x9886,   /* U+11886 */
  0x9887,   /* U+11887 */
  0x9888,   /* U+11888 */
  0x9889,   /* U+11889 */
  0x988A,   /* U+1188A */
  0x988B,   /* U+1188B */
  0x988C,   /* U+1188C */
  0x988D,   /* U+1188D */
  0x988E,   /* U+1188E */
  0x988F,   /* U+1188F */
  0x9890,   /* U+11890 */
  0x9891,   /* U+11891 */
  0x9892,   /* U+11892 */
  0x9893,   /* U+11893 */
  0x9894,   /* U+11894 */
  0x9895,   /* U+11895 */
  0x9896,   /* U+11896 */
  0x9897,   /* U+11897 */
  0x9898,   /* U+11898 */
  0x9899,   /* U+11899 */
  0x989A,   /* U+1189A */
  0x989B,   /* U+1189B */
  0x989C,   /* U+1189C */
  0x989D,   /* U+1189D */
  0x989E,   /* U+1189E */
  0x989F,   /* U+1189F */
  0x0000,   /* U+118A0 */
  0x0000,   /* U+118A1 */
  0x0000,   /* U+118A2 */
  0x0000,   /* U+118A3 */
  0x0000,   /* U+118A4 */
  0x0000,   /* U+118A5 */
  0x0000,   /* U+118A6 */
  0x0000,   /* U+118A7 */
  0x0000,   /* U+118A8 */
  0x0000,   /* U+118A9 */
  0x0000,   /* U+118AA */
  0x0000,   /* U+118AB */
  0x0000,   /* U+118AC */
  0x0000,   /* U+118AD */
  0x0000,   /* U+118AE */
  0x0000,   /* U+118AF */
  0x0000,   /* U+118B0 */
  0x0000,   /* U+118B1 */
  0x0000,   /* U+118B2 */
  0x0000,   /* U+118B3 */
  0x0000,   /* U+118B4 */
  0x0000,   /* U+118B5 */
  0x0000,   /* U+118B6 */
  0x0000,   /* U+118B7 */
  0x0000,   /* U+118B8 */
  0x0000,   /* U+118B9 */
  0x0000,   /* U+118BA */
  0x0000,   /* U+118BB */
  0x0000,   /* U+118BC */
  0x0000,   /* U+118BD */
  0x0000,   /* U+118BE */
  0x0000,   /* U+118BF */
  0x0000,   /* U+118C0 */
  0x0000,   /* U+118C1 */
  0x0000,   /* U+118C2 */
  0x0000,   /* U+118C3 */
  0x0000,   /* U+118C4 */
  0x0000,   /* U+118C5 */
  0x0000,   /* U+118C6 */
  0x0000,   /* U+118C7 */
  0x0000,   /* U+118C8 */
  0x0000,   /* U+118C9 */
  0x0000,   /* U+118CA */
  0x0000,   /* U+118CB */
  0x0000,   /* U+118CC */
  0x0000,   /* U+118CD */
  0x0000,   /* U+118CE */
  0x0000,   /* U+118CF */
  0x0000,   /* U+118D0 */
  0x0000,   /* U+118D1 */
  0x0000,   /* U+118D2 */
  0x0000,   /* U+118D3 */
  0x0000,   /* U+118D4 */
  0x0000,   /* U+118D5 */
  0x0000,   /* U+118D6 */
  0x0000,   /* U+118D7 */
  0x0000,   /* U+118D8 */
  0x0000,   /* U+118D9 */
  0x0000,   /* U+118DA */
  0x0000,   /* U+118DB */
  0x0000,   /* U+118DC */
  0x0000,   /* U+118DD */
  0x0000,   /* U+118DE */
  0x0000,   /* U+118DF */
  0x0000,   /* U+118E0 */
  0x0000,   /* U+118E1 */
  0x0000,   /* U+118E2 */
  0x0000,   /* U+118E3 */
  0x0000,   /* U+118E4 */
  0x0000,   /* U+118E5 */
  0x0000,   /* U+118E6 */
  0x0000,   /* U+118E7 */
  0x0000,   /* U+118E8 */
  0x0000,   /* U+118E9 */
  0x0000,   /* U+118EA */
  0x0000,   /* U+118EB */
  0x0000,   /* U+118EC */
  0x0000,   /* U+118ED */
  0x0000,   /* U+118EE */
  0x0000,   /* U+118EF */
  0x0000,   /* U+118F0 */
  0x0000,   /* U+118F1 */
  0x0000,   /* U+118F2 */
  0x98F3,   /* U+118F3 */
  0x98F4,   /* U+118F4 */
  0x98F5,   /* U+118F5 */
  0x98F6,   /* U+118F6 */
  0x98F7,   /* U+118F7 */
  0x98F8,   /* U+118F8 */
  0x98F9,   /* U+118F9 */
  0x98FA,   /* U+118FA */
  0x98FB,   /* U+118FB */
  0x98FC,   /* U+118FC */
  0x98FD,   /* U+118FD */
  0x98FE,   /* U+118FE */
  0x0000,   /* U+118FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11800 */
  0x0000,   /* U+11801 */
  0x0000,   /* U+11802 */
  0x0000,   /* U+11803 */
  0x0000,   /* U+11804 */
  0x0000,   /* U+11805 */
  0x0000,   /* U+11806 */
  0x0000,   /* U+11807 */
  0x0000,   /* U+11808 */
  0x0000,   /* U+11809 */
  0x0000,   /* U+1180A */
  0x0000,   /* U+1180B */
  0x0000,   /* U+1180C */
  0x0000,   /* U+1180D */
  0x0000,   /* U+1180E */
  0x0000,   /* U+1180F */
  0x0000,   /* U+11810 */
  0x0000,   /* U+11811 */
  0x0000,   /* U+11812 */
  0x0000,   /* U+11813 */
  0x0000,   /* U+11814 */
  0x0000,   /* U+11815 */
  0x0000,   /* U+11816 */
  0x0000,   /* U+11817 */
  0x0000,   /* U+11818 */
  0x0000,   /* U+11819 */
  0x0000,   /* U+1181A */
  0x0000,   /* U+1181B */
  0x0000,   /* U+1181C */
  0x0000,   /* U+1181D */
  0x0000,   /* U+1181E */
  0x0000,   /* U+1181F */
  0x0000,   /* U+11820 */
  0x0000,   /* U+11821 */
  0x0000,   /* U+11822 */
  0x0000,   /* U+11823 */
  0x0000,   /* U+11824 */
  0x0000,   /* U+11825 */
  0x0000,   /* U+11826 */
  0x0000,   /* U+11827 */
  0x0000,   /* U+11828 */
  0x0000,   /* U+11829 */
  0x0000,   /* U+1182A */
  0x0000,   /* U+1182B */
  0x0000,   /* U+1182C */
  0x0000,   /* U+1182D */
  0x0000,   /* U+1182E */
  0x0000,   /* U+1182F */
  0x0000,   /* U+11830 */
  0x0000,   /* U+11831 */
  0x0000,   /* U+11832 */
  0x0000,   /* U+11833 */
  0x0000,   /* U+11834 */
  0x0000,   /* U+11835 */
  0x0000,   /* U+11836 */
  0x0000,   /* U+11837 */
  0x0000,   /* U+11838 */
  0x0000,   /* U+11839 */
  0x0000,   /* U+1183A */
  0x0000,   /* U+1183B */
  0x0000,   /* U+1183C */
  0x0000,   /* U+1183D */
  0x0000,   /* U+1183E */
  0x0000,   /* U+1183F */
  0x0000,   /* U+11840 */
  0x0000,   /* U+11841 */
  0x0000,   /* U+11842 */
  0x0000,   /* U+11843 */
  0x0000,   /* U+11844 */
  0x0000,   /* U+11845 */
  0x0000,   /* U+11846 */
  0x0000,   /* U+11847 */
  0x0000,   /* U+11848 */
  0x0000,   /* U+11849 */
  0x0000,   /* U+1184A */
  0x0000,   /* U+1184B */
  0x0000,   /* U+1184C */
  0x0000,   /* U+1184D */
  0x0000,   /* U+1184E */
  0x0000,   /* U+1184F */
  0x0000,   /* U+11850 */
  0x0000,   /* U+11851 */
  0x0000,   /* U+11852 */
  0x0000,   /* U+11853 */
  0x0000,   /* U+11854 */
  0x0000,   /* U+11855 */
  0x0000,   /* U+11856 */
  0x0000,   /* U+11857 */
  0x0000,   /* U+11858 */
  0x0000,   /* U+11859 */
  0x0000,   /* U+1185A */
  0x0000,   /* U+1185B */
  0x0000,   /* U+1185C */
  0x0000,   /* U+1185D */
  0x0000,   /* U+1185E */
  0x0000,   /* U+1185F */
  0x0000,   /* U+11860 */
  0x0000,   /* U+11861 */
  0x0000,   /* U+11862 */
  0x0000,   /* U+11863 */
  0x0000,   /* U+11864 */
  0x0000,   /* U+11865 */
  0x0000,   /* U+11866 */
  0x0000,   /* U+11867 */
  0x0000,   /* U+11868 */
  0x0000,   /* U+11869 */
  0x0000,   /* U+1186A */
  0x0000,   /* U+1186B */
  0x0000,   /* U+1186C */
  0x0000,   /* U+1186D */
  0x0000,   /* U+1186E */
  0x0000,   /* U+1186F */
  0x0000,   /* U+11870 */
  0x0000,   /* U+11871 */
  0x0000,   /* U+11872 */
  0x0000,   /* U+11873 */
  0x0000,   /* U+11874 */
  0x0000,   /* U+11875 */
  0x0000,   /* U+11876 */
  0x0000,   /* U+11877 */
  0x0000,   /* U+11878 */
  0x0000,   /* U+11879 */
  0x0000,   /* U+1187A */
  0x0000,   /* U+1187B */
  0x0000,   /* U+1187C */
  0x0000,   /* U+1187D */
  0x0000,   /* U+1187E */
  0x0000,   /* U+1187F */
  0x0000,   /* U+11880 */
  0x0000,   /* U+11881 */
  0x0000,   /* U+11882 */
  0x0000,   /* U+11883 */
  0x0000,   /* U+11884 */
  0x0000,   /* U+11885 */
  0x0000,   /* U+11886 */
  0x0000,   /* U+11887 */
  0x0000,   /* U+11888 */
  0x0000,   /* U+11889 */
  0x0000,   /* U+1188A */
  0x0000,   /* U+1188B */
  0x0000,   /* U+1188C */
  0x0000,   /* U+1188D */
  0x0000,   /* U+1188E */
  0x0000,   /* U+1188F */
  0x0000,   /* U+11890 */
  0x0000,   /* U+11891 */
  0x0000,   /* U+11892 */
  0x0000,   /* U+11893 */
  0x0000,   /* U+11894 */
  0x0000,   /* U+11895 */
  0x0000,   /* U+11896 */
  0x0000,   /* U+11897 */
  0x0000,   /* U+11898 */
  0x0000,   /* U+11899 */
  0x0000,   /* U+1189A */
  0x0000,   /* U+1189B */
  0x0000,   /* U+1189C */
  0x0000,   /* U+1189D */
  0x0000,   /* U+1189E */
  0x0000,   /* U+1189F */
  0x0000,   /* U+118A0 */
  0x0000,   /* U+118A1 */
  0x0000,   /* U+118A2 */
  0x0000,   /* U+118A3 */
  0x0000,   /* U+118A4 */
  0x0000,   /* U+118A5 */
  0x0000,   /* U+118A6 */
  0x0000,   /* U+118A7 */
  0x0000,   /* U+118A8 */
  0x0000,   /* U+118A9 */
  0x0000,   /* U+118AA */
  0x0000,   /* U+118AB */
  0x0000,   /* U+118AC */
  0x0000,   /* U+118AD */
  0x0000,   /* U+118AE */
  0x0000,   /* U+118AF */
  0x0000,   /* U+118B0 */
  0x0000,   /* U+118B1 */
  0x0000,   /* U+118B2 */
  0x0000,   /* U+118B3 */
  0x0000,   /* U+118B4 */
  0x0000,   /* U+118B5 */
  0x0000,   /* U+118B6 */
  0x0000,   /* U+118B7 */
  0x0000,   /* U+118B8 */
  0x0000,   /* U+118B9 */
  0x0000,   /* U+118BA */
  0x0000,   /* U+118BB */
  0x0000,   /* U+118BC */
  0x0000,   /* U+118BD */
  0x0000,   /* U+118BE */
  0x0000,   /* U+118BF */
  0x0000,   /* U+118C0 */
  0x0000,   /* U+118C1 */
  0x0000,   /* U+118C2 */
  0x0000,   /* U+118C3 */
  0x0000,   /* U+118C4 */
  0x0000,   /* U+118C5 */
  0x0000,   /* U+118C6 */
  0x0000,   /* U+118C7 */
  0x0000,   /* U+118C8 */
  0x0000,   /* U+118C9 */
  0x0000,   /* U+118CA */
  0x0000,   /* U+118CB */
  0x0000,   /* U+118CC */
  0x0000,   /* U+118CD */
  0x0000,   /* U+118CE */
  0x0000,   /* U+118CF */
  0x0000,   /* U+118D0 */
  0x0000,   /* U+118D1 */
  0x0000,   /* U+118D2 */
  0x0000,   /* U+118D3 */
  0x0000,   /* U+118D4 */
  0x0000,   /* U+118D5 */
  0x0000,   /* U+118D6 */
  0x0000,   /* U+118D7 */
  0x0000,   /* U+118D8 */
  0x0000,   /* U+118D9 */
  0x0000,   /* U+118DA */
  0x0000,   /* U+118DB */
  0x0000,   /* U+118DC */
  0x0000,   /* U+118DD */
  0x0000,   /* U+118DE */
  0x0000,   /* U+118DF */
  0x0000,   /* U+118E0 */
  0x0000,   /* U+118E1 */
  0x0000,   /* U+118E2 */
  0x0000,   /* U+118E3 */
  0x0000,   /* U+118E4 */
  0x0000,   /* U+118E5 */
  0x0000,   /* U+118E6 */
  0x0000,   /* U+118E7 */
  0x0000,   /* U+118E8 */
  0x0000,   /* U+118E9 */
  0x0000,   /* U+118EA */
  0x0000,   /* U+118EB */
  0x0000,   /* U+118EC */
  0x0000,   /* U+118ED */
  0x0000,   /* U+118EE */
  0x0000,   /* U+118EF */
  0x0000,   /* U+118F0 */
  0x0000,   /* U+118F1 */
  0x0000,   /* U+118F2 */
  0x0000,   /* U+118F3 */
  0x0000,   /* U+118F4 */
  0x0000,   /* U+118F5 */
  0x0000,   /* U+118F6 */
  0x0000,   /* U+118F7 */
  0x0000,   /* U+118F8 */
  0x0000,   /* U+118F9 */
  0x0000,   /* U+118FA */
  0x0000,   /* U+118FB */
  0x0000,   /* U+118FC */
  0x0000,   /* U+118FD */
  0x0000,   /* U+118FE */
  0x0000,   /* U+118FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11800 */
  0x0000,   /* U+11801 */
  0x0000,   /* U+11802 */
  0x0000,   /* U+11803 */
  0x0000,   /* U+11804 */
  0x0000,   /* U+11805 */
  0x0000,   /* U+11806 */
  0x0000,   /* U+11807 */
  0x0000,   /* U+11808 */
  0x0000,   /* U+11809 */
  0x0000,   /* U+1180A */
  0x0000,   /* U+1180B */
  0x0000,   /* U+1180C */
  0x0000,   /* U+1180D */
  0x0000,   /* U+1180E */
  0x0000,   /* U+1180F */
  0x0000,   /* U+11810 */
  0x0000,   /* U+11811 */
  0x0000,   /* U+11812 */
  0x0000,   /* U+11813 */
  0x0000,   /* U+11814 */
  0x0000,   /* U+11815 */
  0x0000,   /* U+11816 */
  0x0000,   /* U+11817 */
  0x0000,   /* U+11818 */
  0x0000,   /* U+11819 */
  0x0000,   /* U+1181A */
  0x0000,   /* U+1181B */
  0x0000,   /* U+1181C */
  0x0000,   /* U+1181D */
  0x0000,   /* U+1181E */
  0x0000,   /* U+1181F */
  0x0000,   /* U+11820 */
  0x0000,   /* U+11821 */
  0x0000,   /* U+11822 */
  0x0000,   /* U+11823 */
  0x0000,   /* U+11824 */
  0x0000,   /* U+11825 */
  0x0000,   /* U+11826 */
  0x0000,   /* U+11827 */
  0x0000,   /* U+11828 */
  0x0000,   /* U+11829 */
  0x0000,   /* U+1182A */
  0x0000,   /* U+1182B */
  0x0000,   /* U+1182C */
  0x0000,   /* U+1182D */
  0x0000,   /* U+1182E */
  0x0000,   /* U+1182F */
  0x0000,   /* U+11830 */
  0x0000,   /* U+11831 */
  0x0000,   /* U+11832 */
  0x0000,   /* U+11833 */
  0x0000,   /* U+11834 */
  0x0000,   /* U+11835 */
  0x0000,   /* U+11836 */
  0x0000,   /* U+11837 */
  0x0000,   /* U+11838 */
  0x0000,   /* U+11839 */
  0x0000,   /* U+1183A */
  0x0000,   /* U+1183B */
  0x0000,   /* U+1183C */
  0x0000,   /* U+1183D */
  0x0000,   /* U+1183E */
  0x0000,   /* U+1183F */
  0x0000,   /* U+11840 */
  0x0000,   /* U+11841 */
  0x0000,   /* U+11842 */
  0x0000,   /* U+11843 */
  0x0000,   /* U+11844 */
  0x0000,   /* U+11845 */
  0x0000,   /* U+11846 */
  0x0000,   /* U+11847 */
  0x0000,   /* U+11848 */
  0x0000,   /* U+11849 */
  0x0000,   /* U+1184A */
  0x0000,   /* U+1184B */
  0x0000,   /* U+1184C */
  0x0000,   /* U+1184D */
  0x0000,   /* U+1184E */
  0x0000,   /* U+1184F */
  0x0000,   /* U+11850 */
  0x0000,   /* U+11851 */
  0x0000,   /* U+11852 */
  0x0000,   /* U+11853 */
  0x0000,   /* U+11854 */
  0x0000,   /* U+11855 */
  0x0000,   /* U+11856 */
  0x0000,   /* U+11857 */
  0x0000,   /* U+11858 */
  0x0000,   /* U+11859 */
  0x0000,   /* U+1185A */
  0x0000,   /* U+1185B */
  0x0000,   /* U+1185C */
  0x0000,   /* U+1185D */
  0x0000,   /* U+1185E */
  0x0000,   /* U+1185F */
  0x0000,   /* U+11860 */
  0x0000,   /* U+11861 */
  0x0000,   /* U+11862 */
  0x0000,   /* U+11863 */
  0x0000,   /* U+11864 */
  0x0000,   /* U+11865 */
  0x0000,   /* U+11866 */
  0x0000,   /* U+11867 */
  0x0000,   /* U+11868 */
  0x0000,   /* U+11869 */
  0x0000,   /* U+1186A */
  0x0000,   /* U+1186B */
  0x0000,   /* U+1186C */
  0x0000,   /* U+1186D */
  0x0000,   /* U+1186E */
  0x0000,   /* U+1186F */
  0x0000,   /* U+11870 */
  0x0000,   /* U+11871 */
  0x0000,   /* U+11872 */
  0x0000,   /* U+11873 */
  0x0000,   /* U+11874 */
  0x0000,   /* U+11875 */
  0x0000,   /* U+11876 */
  0x0000,   /* U+11877 */
  0x0000,   /* U+11878 */
  0x0000,   /* U+11879 */
  0x0000,   /* U+1187A */
  0x0000,   /* U+1187B */
  0x0000,   /* U+1187C */
  0x0000,   /* U+1187D */
  0x0000,   /* U+1187E */
  0x0000,   /* U+1187F */
  0x0000,   /* U+11880 */
  0x0000,   /* U+11881 */
  0x0000,   /* U+11882 */
  0x0000,   /* U+11883 */
  0x0000,   /* U+11884 */
  0x0000,   /* U+11885 */
  0x0000,   /* U+11886 */
  0x0000,   /* U+11887 */
  0x0000,   /* U+11888 */
  0x0000,   /* U+11889 */
  0x0000,   /* U+1188A */
  0x0000,   /* U+1188B */
  0x0000,   /* U+1188C */
  0x0000,   /* U+1188D */
  0x0000,   /* U+1188E */
  0x0000,   /* U+1188F */
  0x0000,   /* U+11890 */
  0x0000,   /* U+11891 */
  0x0000,   /* U+11892 */
  0x0000,   /* U+11893 */
  0x0000,   /* U+11894 */
  0x0000,   /* U+11895 */
  0x0000,   /* U+11896 */
  0x0000,   /* U+11897 */
  0x0000,   /* U+11898 */
  0x0000,   /* U+11899 */
  0x0000,   /* U+1189A */
  0x0000,   /* U+1189B */
  0x0000,   /* U+1189C */
  0x0000,   /* U+1189D */
  0x0000,   /* U+1189E */
  0x0000,   /* U+1189F */
  0x0000,   /* U+118A0 */
  0x0000,   /* U+118A1 */
  0x0000,   /* U+118A2 */
  0x0000,   /* U+118A3 */
  0x0000,   /* U+118A4 */
  0x0000,   /* U+118A5 */
  0x0000,   /* U+118A6 */
  0x0000,   /* U+118A7 */
  0x0000,   /* U+118A8 */
  0x0000,   /* U+118A9 */
  0x0000,   /* U+118AA */
  0x0000,   /* U+118AB */
  0x0000,   /* U+118AC */
  0x0000,   /* U+118AD */
  0x0000,   /* U+118AE */
  0x0000,   /* U+118AF */
  0x0000,   /* U+118B0 */
  0x0000,   /* U+118B1 */
  0x0000,   /* U+118B2 */
  0x0000,   /* U+118B3 */
  0x0000,   /* U+118B4 */
  0x0000,   /* U+118B5 */
  0x0000,   /* U+118B6 */
  0x0000,   /* U+118B7 */
  0x0000,   /* U+118B8 */
  0x0000,   /* U+118B9 */
  0x0000,   /* U+118BA */
  0x0000,   /* U+118BB */
  0x0000,   /* U+118BC */
  0x0000,   /* U+118BD */
  0x0000,   /* U+118BE */
  0x0000,   /* U+118BF */
  0x0000,   /* U+118C0 */
  0x0000,   /* U+118C1 */
  0x0000,   /* U+118C2 */
  0x0000,   /* U+118C3 */
  0x0000,   /* U+118C4 */
  0x0000,   /* U+118C5 */
  0x0000,   /* U+118C6 */
  0x0000,   /* U+118C7 */
  0x0000,   /* U+118C8 */
  0x0000,   /* U+118C9 */
  0x0000,   /* U+118CA */
  0x0000,   /* U+118CB */
  0x0000,   /* U+118CC */
  0x0000,   /* U+118CD */
  0x0000,   /* U+118CE */
  0x0000,   /* U+118CF */
  0x0000,   /* U+118D0 */
  0x0000,   /* U+118D1 */
  0x0000,   /* U+118D2 */
  0x0000,   /* U+118D3 */
  0x0000,   /* U+118D4 */
  0x0000,   /* U+118D5 */
  0x0000,   /* U+118D6 */
  0x0000,   /* U+118D7 */
  0x0000,   /* U+118D8 */
  0x0000,   /* U+118D9 */
  0x0000,   /* U+118DA */
  0x0000,   /* U+118DB */
  0x0000,   /* U+118DC */
  0x0000,   /* U+118DD */
  0x0000,   /* U+118DE */
  0x0000,   /* U+118DF */
  0x0000,   /* U+118E0 */
  0x0000,   /* U+118E1 */
  0x0000,   /* U+118E2 */
  0x0000,   /* U+118E3 */
  0x0000,   /* U+118E4 */
  0x0000,   /* U+118E5 */
  0x0000,   /* U+118E6 */
  0x0000,   /* U+118E7 */
  0x0000,   /* U+118E8 */
  0x0000,   /* U+118E9 */
  0x0000,   /* U+118EA */
  0x0000,   /* U+118EB */
  0x0000,   /* U+118EC */
  0x0000,   /* U+118ED */
  0x0000,   /* U+118EE */
  0x0000,   /* U+118EF */
  0x0000,   /* U+118F0 */
  0x0000,   /* U+118F1 */
  0x0000,   /* U+118F2 */
  0x0000,   /* U+118F3 */
  0x0000,   /* U+118F4 */
  0x0000,   /* U+118F5 */
  0x0000,   /* U+118F6 */
  0x0000,   /* U+118F7 */
  0x0000,   /* U+118F8 */
  0x0000,   /* U+118F9 */
  0x0000,   /* U+118FA */
  0x0000,   /* U+118FB */
  0x0000,   /* U+118FC */
  0x0000,   /* U+118FD */
  0x0000,   /* U+118FE */
  0x0000,   /* U+118FF */
};

uint16 uca900_p11A[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC2,   /* U+11A00 */
  0xFBC2,   /* U+11A01 */
  0xFBC2,   /* U+11A02 */
  0xFBC2,   /* U+11A03 */
  0xFBC2,   /* U+11A04 */
  0xFBC2,   /* U+11A05 */
  0xFBC2,   /* U+11A06 */
  0xFBC2,   /* U+11A07 */
  0xFBC2,   /* U+11A08 */
  0xFBC2,   /* U+11A09 */
  0xFBC2,   /* U+11A0A */
  0xFBC2,   /* U+11A0B */
  0xFBC2,   /* U+11A0C */
  0xFBC2,   /* U+11A0D */
  0xFBC2,   /* U+11A0E */
  0xFBC2,   /* U+11A0F */
  0xFBC2,   /* U+11A10 */
  0xFBC2,   /* U+11A11 */
  0xFBC2,   /* U+11A12 */
  0xFBC2,   /* U+11A13 */
  0xFBC2,   /* U+11A14 */
  0xFBC2,   /* U+11A15 */
  0xFBC2,   /* U+11A16 */
  0xFBC2,   /* U+11A17 */
  0xFBC2,   /* U+11A18 */
  0xFBC2,   /* U+11A19 */
  0xFBC2,   /* U+11A1A */
  0xFBC2,   /* U+11A1B */
  0xFBC2,   /* U+11A1C */
  0xFBC2,   /* U+11A1D */
  0xFBC2,   /* U+11A1E */
  0xFBC2,   /* U+11A1F */
  0xFBC2,   /* U+11A20 */
  0xFBC2,   /* U+11A21 */
  0xFBC2,   /* U+11A22 */
  0xFBC2,   /* U+11A23 */
  0xFBC2,   /* U+11A24 */
  0xFBC2,   /* U+11A25 */
  0xFBC2,   /* U+11A26 */
  0xFBC2,   /* U+11A27 */
  0xFBC2,   /* U+11A28 */
  0xFBC2,   /* U+11A29 */
  0xFBC2,   /* U+11A2A */
  0xFBC2,   /* U+11A2B */
  0xFBC2,   /* U+11A2C */
  0xFBC2,   /* U+11A2D */
  0xFBC2,   /* U+11A2E */
  0xFBC2,   /* U+11A2F */
  0xFBC2,   /* U+11A30 */
  0xFBC2,   /* U+11A31 */
  0xFBC2,   /* U+11A32 */
  0xFBC2,   /* U+11A33 */
  0xFBC2,   /* U+11A34 */
  0xFBC2,   /* U+11A35 */
  0xFBC2,   /* U+11A36 */
  0xFBC2,   /* U+11A37 */
  0xFBC2,   /* U+11A38 */
  0xFBC2,   /* U+11A39 */
  0xFBC2,   /* U+11A3A */
  0xFBC2,   /* U+11A3B */
  0xFBC2,   /* U+11A3C */
  0xFBC2,   /* U+11A3D */
  0xFBC2,   /* U+11A3E */
  0xFBC2,   /* U+11A3F */
  0xFBC2,   /* U+11A40 */
  0xFBC2,   /* U+11A41 */
  0xFBC2,   /* U+11A42 */
  0xFBC2,   /* U+11A43 */
  0xFBC2,   /* U+11A44 */
  0xFBC2,   /* U+11A45 */
  0xFBC2,   /* U+11A46 */
  0xFBC2,   /* U+11A47 */
  0xFBC2,   /* U+11A48 */
  0xFBC2,   /* U+11A49 */
  0xFBC2,   /* U+11A4A */
  0xFBC2,   /* U+11A4B */
  0xFBC2,   /* U+11A4C */
  0xFBC2,   /* U+11A4D */
  0xFBC2,   /* U+11A4E */
  0xFBC2,   /* U+11A4F */
  0xFBC2,   /* U+11A50 */
  0xFBC2,   /* U+11A51 */
  0xFBC2,   /* U+11A52 */
  0xFBC2,   /* U+11A53 */
  0xFBC2,   /* U+11A54 */
  0xFBC2,   /* U+11A55 */
  0xFBC2,   /* U+11A56 */
  0xFBC2,   /* U+11A57 */
  0xFBC2,   /* U+11A58 */
  0xFBC2,   /* U+11A59 */
  0xFBC2,   /* U+11A5A */
  0xFBC2,   /* U+11A5B */
  0xFBC2,   /* U+11A5C */
  0xFBC2,   /* U+11A5D */
  0xFBC2,   /* U+11A5E */
  0xFBC2,   /* U+11A5F */
  0xFBC2,   /* U+11A60 */
  0xFBC2,   /* U+11A61 */
  0xFBC2,   /* U+11A62 */
  0xFBC2,   /* U+11A63 */
  0xFBC2,   /* U+11A64 */
  0xFBC2,   /* U+11A65 */
  0xFBC2,   /* U+11A66 */
  0xFBC2,   /* U+11A67 */
  0xFBC2,   /* U+11A68 */
  0xFBC2,   /* U+11A69 */
  0xFBC2,   /* U+11A6A */
  0xFBC2,   /* U+11A6B */
  0xFBC2,   /* U+11A6C */
  0xFBC2,   /* U+11A6D */
  0xFBC2,   /* U+11A6E */
  0xFBC2,   /* U+11A6F */
  0xFBC2,   /* U+11A70 */
  0xFBC2,   /* U+11A71 */
  0xFBC2,   /* U+11A72 */
  0xFBC2,   /* U+11A73 */
  0xFBC2,   /* U+11A74 */
  0xFBC2,   /* U+11A75 */
  0xFBC2,   /* U+11A76 */
  0xFBC2,   /* U+11A77 */
  0xFBC2,   /* U+11A78 */
  0xFBC2,   /* U+11A79 */
  0xFBC2,   /* U+11A7A */
  0xFBC2,   /* U+11A7B */
  0xFBC2,   /* U+11A7C */
  0xFBC2,   /* U+11A7D */
  0xFBC2,   /* U+11A7E */
  0xFBC2,   /* U+11A7F */
  0xFBC2,   /* U+11A80 */
  0xFBC2,   /* U+11A81 */
  0xFBC2,   /* U+11A82 */
  0xFBC2,   /* U+11A83 */
  0xFBC2,   /* U+11A84 */
  0xFBC2,   /* U+11A85 */
  0xFBC2,   /* U+11A86 */
  0xFBC2,   /* U+11A87 */
  0xFBC2,   /* U+11A88 */
  0xFBC2,   /* U+11A89 */
  0xFBC2,   /* U+11A8A */
  0xFBC2,   /* U+11A8B */
  0xFBC2,   /* U+11A8C */
  0xFBC2,   /* U+11A8D */
  0xFBC2,   /* U+11A8E */
  0xFBC2,   /* U+11A8F */
  0xFBC2,   /* U+11A90 */
  0xFBC2,   /* U+11A91 */
  0xFBC2,   /* U+11A92 */
  0xFBC2,   /* U+11A93 */
  0xFBC2,   /* U+11A94 */
  0xFBC2,   /* U+11A95 */
  0xFBC2,   /* U+11A96 */
  0xFBC2,   /* U+11A97 */
  0xFBC2,   /* U+11A98 */
  0xFBC2,   /* U+11A99 */
  0xFBC2,   /* U+11A9A */
  0xFBC2,   /* U+11A9B */
  0xFBC2,   /* U+11A9C */
  0xFBC2,   /* U+11A9D */
  0xFBC2,   /* U+11A9E */
  0xFBC2,   /* U+11A9F */
  0xFBC2,   /* U+11AA0 */
  0xFBC2,   /* U+11AA1 */
  0xFBC2,   /* U+11AA2 */
  0xFBC2,   /* U+11AA3 */
  0xFBC2,   /* U+11AA4 */
  0xFBC2,   /* U+11AA5 */
  0xFBC2,   /* U+11AA6 */
  0xFBC2,   /* U+11AA7 */
  0xFBC2,   /* U+11AA8 */
  0xFBC2,   /* U+11AA9 */
  0xFBC2,   /* U+11AAA */
  0xFBC2,   /* U+11AAB */
  0xFBC2,   /* U+11AAC */
  0xFBC2,   /* U+11AAD */
  0xFBC2,   /* U+11AAE */
  0xFBC2,   /* U+11AAF */
  0xFBC2,   /* U+11AB0 */
  0xFBC2,   /* U+11AB1 */
  0xFBC2,   /* U+11AB2 */
  0xFBC2,   /* U+11AB3 */
  0xFBC2,   /* U+11AB4 */
  0xFBC2,   /* U+11AB5 */
  0xFBC2,   /* U+11AB6 */
  0xFBC2,   /* U+11AB7 */
  0xFBC2,   /* U+11AB8 */
  0xFBC2,   /* U+11AB9 */
  0xFBC2,   /* U+11ABA */
  0xFBC2,   /* U+11ABB */
  0xFBC2,   /* U+11ABC */
  0xFBC2,   /* U+11ABD */
  0xFBC2,   /* U+11ABE */
  0xFBC2,   /* U+11ABF */
  0x4323,   /* U+11AC0 */
  0x4324,   /* U+11AC1 */
  0x4325,   /* U+11AC2 */
  0x4326,   /* U+11AC3 */
  0x4327,   /* U+11AC4 */
  0x4328,   /* U+11AC5 */
  0x4329,   /* U+11AC6 */
  0x432A,   /* U+11AC7 */
  0x432B,   /* U+11AC8 */
  0x432C,   /* U+11AC9 */
  0x432D,   /* U+11ACA */
  0x432E,   /* U+11ACB */
  0x432F,   /* U+11ACC */
  0x4330,   /* U+11ACD */
  0x4331,   /* U+11ACE */
  0x4332,   /* U+11ACF */
  0x4333,   /* U+11AD0 */
  0x4334,   /* U+11AD1 */
  0x4335,   /* U+11AD2 */
  0x4336,   /* U+11AD3 */
  0x4337,   /* U+11AD4 */
  0x431C,   /* U+11AD5 */
  0x431D,   /* U+11AD6 */
  0x431E,   /* U+11AD7 */
  0x431F,   /* U+11AD8 */
  0x4320,   /* U+11AD9 */
  0x4321,   /* U+11ADA */
  0x4322,   /* U+11ADB */
  0x4338,   /* U+11ADC */
  0x4339,   /* U+11ADD */
  0x433F,   /* U+11ADE */
  0x433A,   /* U+11ADF */
  0x433B,   /* U+11AE0 */
  0x433C,   /* U+11AE1 */
  0x433D,   /* U+11AE2 */
  0x433E,   /* U+11AE3 */
  0x4340,   /* U+11AE4 */
  0x4346,   /* U+11AE5 */
  0x4344,   /* U+11AE6 */
  0x4352,   /* U+11AE7 */
  0x4347,   /* U+11AE8 */
  0x4345,   /* U+11AE9 */
  0x4353,   /* U+11AEA */
  0x434E,   /* U+11AEB */
  0x434C,   /* U+11AEC */
  0x434F,   /* U+11AED */
  0x434D,   /* U+11AEE */
  0x4341,   /* U+11AEF */
  0x4354,   /* U+11AF0 */
  0x4343,   /* U+11AF1 */
  0x4342,   /* U+11AF2 */
  0x434A,   /* U+11AF3 */
  0x4348,   /* U+11AF4 */
  0x4350,   /* U+11AF5 */
  0x434B,   /* U+11AF6 */
  0x4349,   /* U+11AF7 */
  0x4351,   /* U+11AF8 */
  0xFBC2,   /* U+11AF9 */
  0xFBC2,   /* U+11AFA */
  0xFBC2,   /* U+11AFB */
  0xFBC2,   /* U+11AFC */
  0xFBC2,   /* U+11AFD */
  0xFBC2,   /* U+11AFE */
  0xFBC2,   /* U+11AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11A00 */
  0x0020,   /* U+11A01 */
  0x0020,   /* U+11A02 */
  0x0020,   /* U+11A03 */
  0x0020,   /* U+11A04 */
  0x0020,   /* U+11A05 */
  0x0020,   /* U+11A06 */
  0x0020,   /* U+11A07 */
  0x0020,   /* U+11A08 */
  0x0020,   /* U+11A09 */
  0x0020,   /* U+11A0A */
  0x0020,   /* U+11A0B */
  0x0020,   /* U+11A0C */
  0x0020,   /* U+11A0D */
  0x0020,   /* U+11A0E */
  0x0020,   /* U+11A0F */
  0x0020,   /* U+11A10 */
  0x0020,   /* U+11A11 */
  0x0020,   /* U+11A12 */
  0x0020,   /* U+11A13 */
  0x0020,   /* U+11A14 */
  0x0020,   /* U+11A15 */
  0x0020,   /* U+11A16 */
  0x0020,   /* U+11A17 */
  0x0020,   /* U+11A18 */
  0x0020,   /* U+11A19 */
  0x0020,   /* U+11A1A */
  0x0020,   /* U+11A1B */
  0x0020,   /* U+11A1C */
  0x0020,   /* U+11A1D */
  0x0020,   /* U+11A1E */
  0x0020,   /* U+11A1F */
  0x0020,   /* U+11A20 */
  0x0020,   /* U+11A21 */
  0x0020,   /* U+11A22 */
  0x0020,   /* U+11A23 */
  0x0020,   /* U+11A24 */
  0x0020,   /* U+11A25 */
  0x0020,   /* U+11A26 */
  0x0020,   /* U+11A27 */
  0x0020,   /* U+11A28 */
  0x0020,   /* U+11A29 */
  0x0020,   /* U+11A2A */
  0x0020,   /* U+11A2B */
  0x0020,   /* U+11A2C */
  0x0020,   /* U+11A2D */
  0x0020,   /* U+11A2E */
  0x0020,   /* U+11A2F */
  0x0020,   /* U+11A30 */
  0x0020,   /* U+11A31 */
  0x0020,   /* U+11A32 */
  0x0020,   /* U+11A33 */
  0x0020,   /* U+11A34 */
  0x0020,   /* U+11A35 */
  0x0020,   /* U+11A36 */
  0x0020,   /* U+11A37 */
  0x0020,   /* U+11A38 */
  0x0020,   /* U+11A39 */
  0x0020,   /* U+11A3A */
  0x0020,   /* U+11A3B */
  0x0020,   /* U+11A3C */
  0x0020,   /* U+11A3D */
  0x0020,   /* U+11A3E */
  0x0020,   /* U+11A3F */
  0x0020,   /* U+11A40 */
  0x0020,   /* U+11A41 */
  0x0020,   /* U+11A42 */
  0x0020,   /* U+11A43 */
  0x0020,   /* U+11A44 */
  0x0020,   /* U+11A45 */
  0x0020,   /* U+11A46 */
  0x0020,   /* U+11A47 */
  0x0020,   /* U+11A48 */
  0x0020,   /* U+11A49 */
  0x0020,   /* U+11A4A */
  0x0020,   /* U+11A4B */
  0x0020,   /* U+11A4C */
  0x0020,   /* U+11A4D */
  0x0020,   /* U+11A4E */
  0x0020,   /* U+11A4F */
  0x0020,   /* U+11A50 */
  0x0020,   /* U+11A51 */
  0x0020,   /* U+11A52 */
  0x0020,   /* U+11A53 */
  0x0020,   /* U+11A54 */
  0x0020,   /* U+11A55 */
  0x0020,   /* U+11A56 */
  0x0020,   /* U+11A57 */
  0x0020,   /* U+11A58 */
  0x0020,   /* U+11A59 */
  0x0020,   /* U+11A5A */
  0x0020,   /* U+11A5B */
  0x0020,   /* U+11A5C */
  0x0020,   /* U+11A5D */
  0x0020,   /* U+11A5E */
  0x0020,   /* U+11A5F */
  0x0020,   /* U+11A60 */
  0x0020,   /* U+11A61 */
  0x0020,   /* U+11A62 */
  0x0020,   /* U+11A63 */
  0x0020,   /* U+11A64 */
  0x0020,   /* U+11A65 */
  0x0020,   /* U+11A66 */
  0x0020,   /* U+11A67 */
  0x0020,   /* U+11A68 */
  0x0020,   /* U+11A69 */
  0x0020,   /* U+11A6A */
  0x0020,   /* U+11A6B */
  0x0020,   /* U+11A6C */
  0x0020,   /* U+11A6D */
  0x0020,   /* U+11A6E */
  0x0020,   /* U+11A6F */
  0x0020,   /* U+11A70 */
  0x0020,   /* U+11A71 */
  0x0020,   /* U+11A72 */
  0x0020,   /* U+11A73 */
  0x0020,   /* U+11A74 */
  0x0020,   /* U+11A75 */
  0x0020,   /* U+11A76 */
  0x0020,   /* U+11A77 */
  0x0020,   /* U+11A78 */
  0x0020,   /* U+11A79 */
  0x0020,   /* U+11A7A */
  0x0020,   /* U+11A7B */
  0x0020,   /* U+11A7C */
  0x0020,   /* U+11A7D */
  0x0020,   /* U+11A7E */
  0x0020,   /* U+11A7F */
  0x0020,   /* U+11A80 */
  0x0020,   /* U+11A81 */
  0x0020,   /* U+11A82 */
  0x0020,   /* U+11A83 */
  0x0020,   /* U+11A84 */
  0x0020,   /* U+11A85 */
  0x0020,   /* U+11A86 */
  0x0020,   /* U+11A87 */
  0x0020,   /* U+11A88 */
  0x0020,   /* U+11A89 */
  0x0020,   /* U+11A8A */
  0x0020,   /* U+11A8B */
  0x0020,   /* U+11A8C */
  0x0020,   /* U+11A8D */
  0x0020,   /* U+11A8E */
  0x0020,   /* U+11A8F */
  0x0020,   /* U+11A90 */
  0x0020,   /* U+11A91 */
  0x0020,   /* U+11A92 */
  0x0020,   /* U+11A93 */
  0x0020,   /* U+11A94 */
  0x0020,   /* U+11A95 */
  0x0020,   /* U+11A96 */
  0x0020,   /* U+11A97 */
  0x0020,   /* U+11A98 */
  0x0020,   /* U+11A99 */
  0x0020,   /* U+11A9A */
  0x0020,   /* U+11A9B */
  0x0020,   /* U+11A9C */
  0x0020,   /* U+11A9D */
  0x0020,   /* U+11A9E */
  0x0020,   /* U+11A9F */
  0x0020,   /* U+11AA0 */
  0x0020,   /* U+11AA1 */
  0x0020,   /* U+11AA2 */
  0x0020,   /* U+11AA3 */
  0x0020,   /* U+11AA4 */
  0x0020,   /* U+11AA5 */
  0x0020,   /* U+11AA6 */
  0x0020,   /* U+11AA7 */
  0x0020,   /* U+11AA8 */
  0x0020,   /* U+11AA9 */
  0x0020,   /* U+11AAA */
  0x0020,   /* U+11AAB */
  0x0020,   /* U+11AAC */
  0x0020,   /* U+11AAD */
  0x0020,   /* U+11AAE */
  0x0020,   /* U+11AAF */
  0x0020,   /* U+11AB0 */
  0x0020,   /* U+11AB1 */
  0x0020,   /* U+11AB2 */
  0x0020,   /* U+11AB3 */
  0x0020,   /* U+11AB4 */
  0x0020,   /* U+11AB5 */
  0x0020,   /* U+11AB6 */
  0x0020,   /* U+11AB7 */
  0x0020,   /* U+11AB8 */
  0x0020,   /* U+11AB9 */
  0x0020,   /* U+11ABA */
  0x0020,   /* U+11ABB */
  0x0020,   /* U+11ABC */
  0x0020,   /* U+11ABD */
  0x0020,   /* U+11ABE */
  0x0020,   /* U+11ABF */
  0x0020,   /* U+11AC0 */
  0x0020,   /* U+11AC1 */
  0x0020,   /* U+11AC2 */
  0x0020,   /* U+11AC3 */
  0x0020,   /* U+11AC4 */
  0x0020,   /* U+11AC5 */
  0x0020,   /* U+11AC6 */
  0x0020,   /* U+11AC7 */
  0x0020,   /* U+11AC8 */
  0x0020,   /* U+11AC9 */
  0x0020,   /* U+11ACA */
  0x0020,   /* U+11ACB */
  0x0020,   /* U+11ACC */
  0x0020,   /* U+11ACD */
  0x0020,   /* U+11ACE */
  0x0020,   /* U+11ACF */
  0x0020,   /* U+11AD0 */
  0x0020,   /* U+11AD1 */
  0x0020,   /* U+11AD2 */
  0x0020,   /* U+11AD3 */
  0x0020,   /* U+11AD4 */
  0x0020,   /* U+11AD5 */
  0x0020,   /* U+11AD6 */
  0x0020,   /* U+11AD7 */
  0x0020,   /* U+11AD8 */
  0x0020,   /* U+11AD9 */
  0x0020,   /* U+11ADA */
  0x0020,   /* U+11ADB */
  0x0020,   /* U+11ADC */
  0x0020,   /* U+11ADD */
  0x0020,   /* U+11ADE */
  0x0020,   /* U+11ADF */
  0x0020,   /* U+11AE0 */
  0x0020,   /* U+11AE1 */
  0x0020,   /* U+11AE2 */
  0x0020,   /* U+11AE3 */
  0x0020,   /* U+11AE4 */
  0x0020,   /* U+11AE5 */
  0x0020,   /* U+11AE6 */
  0x0020,   /* U+11AE7 */
  0x0020,   /* U+11AE8 */
  0x0020,   /* U+11AE9 */
  0x0020,   /* U+11AEA */
  0x0020,   /* U+11AEB */
  0x0020,   /* U+11AEC */
  0x0020,   /* U+11AED */
  0x0020,   /* U+11AEE */
  0x0020,   /* U+11AEF */
  0x0020,   /* U+11AF0 */
  0x0020,   /* U+11AF1 */
  0x0020,   /* U+11AF2 */
  0x0020,   /* U+11AF3 */
  0x0020,   /* U+11AF4 */
  0x0020,   /* U+11AF5 */
  0x0020,   /* U+11AF6 */
  0x0020,   /* U+11AF7 */
  0x0020,   /* U+11AF8 */
  0x0020,   /* U+11AF9 */
  0x0020,   /* U+11AFA */
  0x0020,   /* U+11AFB */
  0x0020,   /* U+11AFC */
  0x0020,   /* U+11AFD */
  0x0020,   /* U+11AFE */
  0x0020,   /* U+11AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11A00 */
  0x0002,   /* U+11A01 */
  0x0002,   /* U+11A02 */
  0x0002,   /* U+11A03 */
  0x0002,   /* U+11A04 */
  0x0002,   /* U+11A05 */
  0x0002,   /* U+11A06 */
  0x0002,   /* U+11A07 */
  0x0002,   /* U+11A08 */
  0x0002,   /* U+11A09 */
  0x0002,   /* U+11A0A */
  0x0002,   /* U+11A0B */
  0x0002,   /* U+11A0C */
  0x0002,   /* U+11A0D */
  0x0002,   /* U+11A0E */
  0x0002,   /* U+11A0F */
  0x0002,   /* U+11A10 */
  0x0002,   /* U+11A11 */
  0x0002,   /* U+11A12 */
  0x0002,   /* U+11A13 */
  0x0002,   /* U+11A14 */
  0x0002,   /* U+11A15 */
  0x0002,   /* U+11A16 */
  0x0002,   /* U+11A17 */
  0x0002,   /* U+11A18 */
  0x0002,   /* U+11A19 */
  0x0002,   /* U+11A1A */
  0x0002,   /* U+11A1B */
  0x0002,   /* U+11A1C */
  0x0002,   /* U+11A1D */
  0x0002,   /* U+11A1E */
  0x0002,   /* U+11A1F */
  0x0002,   /* U+11A20 */
  0x0002,   /* U+11A21 */
  0x0002,   /* U+11A22 */
  0x0002,   /* U+11A23 */
  0x0002,   /* U+11A24 */
  0x0002,   /* U+11A25 */
  0x0002,   /* U+11A26 */
  0x0002,   /* U+11A27 */
  0x0002,   /* U+11A28 */
  0x0002,   /* U+11A29 */
  0x0002,   /* U+11A2A */
  0x0002,   /* U+11A2B */
  0x0002,   /* U+11A2C */
  0x0002,   /* U+11A2D */
  0x0002,   /* U+11A2E */
  0x0002,   /* U+11A2F */
  0x0002,   /* U+11A30 */
  0x0002,   /* U+11A31 */
  0x0002,   /* U+11A32 */
  0x0002,   /* U+11A33 */
  0x0002,   /* U+11A34 */
  0x0002,   /* U+11A35 */
  0x0002,   /* U+11A36 */
  0x0002,   /* U+11A37 */
  0x0002,   /* U+11A38 */
  0x0002,   /* U+11A39 */
  0x0002,   /* U+11A3A */
  0x0002,   /* U+11A3B */
  0x0002,   /* U+11A3C */
  0x0002,   /* U+11A3D */
  0x0002,   /* U+11A3E */
  0x0002,   /* U+11A3F */
  0x0002,   /* U+11A40 */
  0x0002,   /* U+11A41 */
  0x0002,   /* U+11A42 */
  0x0002,   /* U+11A43 */
  0x0002,   /* U+11A44 */
  0x0002,   /* U+11A45 */
  0x0002,   /* U+11A46 */
  0x0002,   /* U+11A47 */
  0x0002,   /* U+11A48 */
  0x0002,   /* U+11A49 */
  0x0002,   /* U+11A4A */
  0x0002,   /* U+11A4B */
  0x0002,   /* U+11A4C */
  0x0002,   /* U+11A4D */
  0x0002,   /* U+11A4E */
  0x0002,   /* U+11A4F */
  0x0002,   /* U+11A50 */
  0x0002,   /* U+11A51 */
  0x0002,   /* U+11A52 */
  0x0002,   /* U+11A53 */
  0x0002,   /* U+11A54 */
  0x0002,   /* U+11A55 */
  0x0002,   /* U+11A56 */
  0x0002,   /* U+11A57 */
  0x0002,   /* U+11A58 */
  0x0002,   /* U+11A59 */
  0x0002,   /* U+11A5A */
  0x0002,   /* U+11A5B */
  0x0002,   /* U+11A5C */
  0x0002,   /* U+11A5D */
  0x0002,   /* U+11A5E */
  0x0002,   /* U+11A5F */
  0x0002,   /* U+11A60 */
  0x0002,   /* U+11A61 */
  0x0002,   /* U+11A62 */
  0x0002,   /* U+11A63 */
  0x0002,   /* U+11A64 */
  0x0002,   /* U+11A65 */
  0x0002,   /* U+11A66 */
  0x0002,   /* U+11A67 */
  0x0002,   /* U+11A68 */
  0x0002,   /* U+11A69 */
  0x0002,   /* U+11A6A */
  0x0002,   /* U+11A6B */
  0x0002,   /* U+11A6C */
  0x0002,   /* U+11A6D */
  0x0002,   /* U+11A6E */
  0x0002,   /* U+11A6F */
  0x0002,   /* U+11A70 */
  0x0002,   /* U+11A71 */
  0x0002,   /* U+11A72 */
  0x0002,   /* U+11A73 */
  0x0002,   /* U+11A74 */
  0x0002,   /* U+11A75 */
  0x0002,   /* U+11A76 */
  0x0002,   /* U+11A77 */
  0x0002,   /* U+11A78 */
  0x0002,   /* U+11A79 */
  0x0002,   /* U+11A7A */
  0x0002,   /* U+11A7B */
  0x0002,   /* U+11A7C */
  0x0002,   /* U+11A7D */
  0x0002,   /* U+11A7E */
  0x0002,   /* U+11A7F */
  0x0002,   /* U+11A80 */
  0x0002,   /* U+11A81 */
  0x0002,   /* U+11A82 */
  0x0002,   /* U+11A83 */
  0x0002,   /* U+11A84 */
  0x0002,   /* U+11A85 */
  0x0002,   /* U+11A86 */
  0x0002,   /* U+11A87 */
  0x0002,   /* U+11A88 */
  0x0002,   /* U+11A89 */
  0x0002,   /* U+11A8A */
  0x0002,   /* U+11A8B */
  0x0002,   /* U+11A8C */
  0x0002,   /* U+11A8D */
  0x0002,   /* U+11A8E */
  0x0002,   /* U+11A8F */
  0x0002,   /* U+11A90 */
  0x0002,   /* U+11A91 */
  0x0002,   /* U+11A92 */
  0x0002,   /* U+11A93 */
  0x0002,   /* U+11A94 */
  0x0002,   /* U+11A95 */
  0x0002,   /* U+11A96 */
  0x0002,   /* U+11A97 */
  0x0002,   /* U+11A98 */
  0x0002,   /* U+11A99 */
  0x0002,   /* U+11A9A */
  0x0002,   /* U+11A9B */
  0x0002,   /* U+11A9C */
  0x0002,   /* U+11A9D */
  0x0002,   /* U+11A9E */
  0x0002,   /* U+11A9F */
  0x0002,   /* U+11AA0 */
  0x0002,   /* U+11AA1 */
  0x0002,   /* U+11AA2 */
  0x0002,   /* U+11AA3 */
  0x0002,   /* U+11AA4 */
  0x0002,   /* U+11AA5 */
  0x0002,   /* U+11AA6 */
  0x0002,   /* U+11AA7 */
  0x0002,   /* U+11AA8 */
  0x0002,   /* U+11AA9 */
  0x0002,   /* U+11AAA */
  0x0002,   /* U+11AAB */
  0x0002,   /* U+11AAC */
  0x0002,   /* U+11AAD */
  0x0002,   /* U+11AAE */
  0x0002,   /* U+11AAF */
  0x0002,   /* U+11AB0 */
  0x0002,   /* U+11AB1 */
  0x0002,   /* U+11AB2 */
  0x0002,   /* U+11AB3 */
  0x0002,   /* U+11AB4 */
  0x0002,   /* U+11AB5 */
  0x0002,   /* U+11AB6 */
  0x0002,   /* U+11AB7 */
  0x0002,   /* U+11AB8 */
  0x0002,   /* U+11AB9 */
  0x0002,   /* U+11ABA */
  0x0002,   /* U+11ABB */
  0x0002,   /* U+11ABC */
  0x0002,   /* U+11ABD */
  0x0002,   /* U+11ABE */
  0x0002,   /* U+11ABF */
  0x0002,   /* U+11AC0 */
  0x0002,   /* U+11AC1 */
  0x0002,   /* U+11AC2 */
  0x0002,   /* U+11AC3 */
  0x0002,   /* U+11AC4 */
  0x0002,   /* U+11AC5 */
  0x0002,   /* U+11AC6 */
  0x0002,   /* U+11AC7 */
  0x0002,   /* U+11AC8 */
  0x0002,   /* U+11AC9 */
  0x0002,   /* U+11ACA */
  0x0002,   /* U+11ACB */
  0x0002,   /* U+11ACC */
  0x0002,   /* U+11ACD */
  0x0002,   /* U+11ACE */
  0x0002,   /* U+11ACF */
  0x0002,   /* U+11AD0 */
  0x0002,   /* U+11AD1 */
  0x0002,   /* U+11AD2 */
  0x0002,   /* U+11AD3 */
  0x0002,   /* U+11AD4 */
  0x0002,   /* U+11AD5 */
  0x0002,   /* U+11AD6 */
  0x0002,   /* U+11AD7 */
  0x0002,   /* U+11AD8 */
  0x0002,   /* U+11AD9 */
  0x0002,   /* U+11ADA */
  0x0002,   /* U+11ADB */
  0x0002,   /* U+11ADC */
  0x0002,   /* U+11ADD */
  0x0002,   /* U+11ADE */
  0x0002,   /* U+11ADF */
  0x0002,   /* U+11AE0 */
  0x0002,   /* U+11AE1 */
  0x0002,   /* U+11AE2 */
  0x0002,   /* U+11AE3 */
  0x0002,   /* U+11AE4 */
  0x0002,   /* U+11AE5 */
  0x0002,   /* U+11AE6 */
  0x0002,   /* U+11AE7 */
  0x0002,   /* U+11AE8 */
  0x0002,   /* U+11AE9 */
  0x0002,   /* U+11AEA */
  0x0002,   /* U+11AEB */
  0x0002,   /* U+11AEC */
  0x0002,   /* U+11AED */
  0x0002,   /* U+11AEE */
  0x0002,   /* U+11AEF */
  0x0002,   /* U+11AF0 */
  0x0002,   /* U+11AF1 */
  0x0002,   /* U+11AF2 */
  0x0002,   /* U+11AF3 */
  0x0002,   /* U+11AF4 */
  0x0002,   /* U+11AF5 */
  0x0002,   /* U+11AF6 */
  0x0002,   /* U+11AF7 */
  0x0002,   /* U+11AF8 */
  0x0002,   /* U+11AF9 */
  0x0002,   /* U+11AFA */
  0x0002,   /* U+11AFB */
  0x0002,   /* U+11AFC */
  0x0002,   /* U+11AFD */
  0x0002,   /* U+11AFE */
  0x0002,   /* U+11AFF */

  /* Primary weight 2 for each character. */
  0x9A00,   /* U+11A00 */
  0x9A01,   /* U+11A01 */
  0x9A02,   /* U+11A02 */
  0x9A03,   /* U+11A03 */
  0x9A04,   /* U+11A04 */
  0x9A05,   /* U+11A05 */
  0x9A06,   /* U+11A06 */
  0x9A07,   /* U+11A07 */
  0x9A08,   /* U+11A08 */
  0x9A09,   /* U+11A09 */
  0x9A0A,   /* U+11A0A */
  0x9A0B,   /* U+11A0B */
  0x9A0C,   /* U+11A0C */
  0x9A0D,   /* U+11A0D */
  0x9A0E,   /* U+11A0E */
  0x9A0F,   /* U+11A0F */
  0x9A10,   /* U+11A10 */
  0x9A11,   /* U+11A11 */
  0x9A12,   /* U+11A12 */
  0x9A13,   /* U+11A13 */
  0x9A14,   /* U+11A14 */
  0x9A15,   /* U+11A15 */
  0x9A16,   /* U+11A16 */
  0x9A17,   /* U+11A17 */
  0x9A18,   /* U+11A18 */
  0x9A19,   /* U+11A19 */
  0x9A1A,   /* U+11A1A */
  0x9A1B,   /* U+11A1B */
  0x9A1C,   /* U+11A1C */
  0x9A1D,   /* U+11A1D */
  0x9A1E,   /* U+11A1E */
  0x9A1F,   /* U+11A1F */
  0x9A20,   /* U+11A20 */
  0x9A21,   /* U+11A21 */
  0x9A22,   /* U+11A22 */
  0x9A23,   /* U+11A23 */
  0x9A24,   /* U+11A24 */
  0x9A25,   /* U+11A25 */
  0x9A26,   /* U+11A26 */
  0x9A27,   /* U+11A27 */
  0x9A28,   /* U+11A28 */
  0x9A29,   /* U+11A29 */
  0x9A2A,   /* U+11A2A */
  0x9A2B,   /* U+11A2B */
  0x9A2C,   /* U+11A2C */
  0x9A2D,   /* U+11A2D */
  0x9A2E,   /* U+11A2E */
  0x9A2F,   /* U+11A2F */
  0x9A30,   /* U+11A30 */
  0x9A31,   /* U+11A31 */
  0x9A32,   /* U+11A32 */
  0x9A33,   /* U+11A33 */
  0x9A34,   /* U+11A34 */
  0x9A35,   /* U+11A35 */
  0x9A36,   /* U+11A36 */
  0x9A37,   /* U+11A37 */
  0x9A38,   /* U+11A38 */
  0x9A39,   /* U+11A39 */
  0x9A3A,   /* U+11A3A */
  0x9A3B,   /* U+11A3B */
  0x9A3C,   /* U+11A3C */
  0x9A3D,   /* U+11A3D */
  0x9A3E,   /* U+11A3E */
  0x9A3F,   /* U+11A3F */
  0x9A40,   /* U+11A40 */
  0x9A41,   /* U+11A41 */
  0x9A42,   /* U+11A42 */
  0x9A43,   /* U+11A43 */
  0x9A44,   /* U+11A44 */
  0x9A45,   /* U+11A45 */
  0x9A46,   /* U+11A46 */
  0x9A47,   /* U+11A47 */
  0x9A48,   /* U+11A48 */
  0x9A49,   /* U+11A49 */
  0x9A4A,   /* U+11A4A */
  0x9A4B,   /* U+11A4B */
  0x9A4C,   /* U+11A4C */
  0x9A4D,   /* U+11A4D */
  0x9A4E,   /* U+11A4E */
  0x9A4F,   /* U+11A4F */
  0x9A50,   /* U+11A50 */
  0x9A51,   /* U+11A51 */
  0x9A52,   /* U+11A52 */
  0x9A53,   /* U+11A53 */
  0x9A54,   /* U+11A54 */
  0x9A55,   /* U+11A55 */
  0x9A56,   /* U+11A56 */
  0x9A57,   /* U+11A57 */
  0x9A58,   /* U+11A58 */
  0x9A59,   /* U+11A59 */
  0x9A5A,   /* U+11A5A */
  0x9A5B,   /* U+11A5B */
  0x9A5C,   /* U+11A5C */
  0x9A5D,   /* U+11A5D */
  0x9A5E,   /* U+11A5E */
  0x9A5F,   /* U+11A5F */
  0x9A60,   /* U+11A60 */
  0x9A61,   /* U+11A61 */
  0x9A62,   /* U+11A62 */
  0x9A63,   /* U+11A63 */
  0x9A64,   /* U+11A64 */
  0x9A65,   /* U+11A65 */
  0x9A66,   /* U+11A66 */
  0x9A67,   /* U+11A67 */
  0x9A68,   /* U+11A68 */
  0x9A69,   /* U+11A69 */
  0x9A6A,   /* U+11A6A */
  0x9A6B,   /* U+11A6B */
  0x9A6C,   /* U+11A6C */
  0x9A6D,   /* U+11A6D */
  0x9A6E,   /* U+11A6E */
  0x9A6F,   /* U+11A6F */
  0x9A70,   /* U+11A70 */
  0x9A71,   /* U+11A71 */
  0x9A72,   /* U+11A72 */
  0x9A73,   /* U+11A73 */
  0x9A74,   /* U+11A74 */
  0x9A75,   /* U+11A75 */
  0x9A76,   /* U+11A76 */
  0x9A77,   /* U+11A77 */
  0x9A78,   /* U+11A78 */
  0x9A79,   /* U+11A79 */
  0x9A7A,   /* U+11A7A */
  0x9A7B,   /* U+11A7B */
  0x9A7C,   /* U+11A7C */
  0x9A7D,   /* U+11A7D */
  0x9A7E,   /* U+11A7E */
  0x9A7F,   /* U+11A7F */
  0x9A80,   /* U+11A80 */
  0x9A81,   /* U+11A81 */
  0x9A82,   /* U+11A82 */
  0x9A83,   /* U+11A83 */
  0x9A84,   /* U+11A84 */
  0x9A85,   /* U+11A85 */
  0x9A86,   /* U+11A86 */
  0x9A87,   /* U+11A87 */
  0x9A88,   /* U+11A88 */
  0x9A89,   /* U+11A89 */
  0x9A8A,   /* U+11A8A */
  0x9A8B,   /* U+11A8B */
  0x9A8C,   /* U+11A8C */
  0x9A8D,   /* U+11A8D */
  0x9A8E,   /* U+11A8E */
  0x9A8F,   /* U+11A8F */
  0x9A90,   /* U+11A90 */
  0x9A91,   /* U+11A91 */
  0x9A92,   /* U+11A92 */
  0x9A93,   /* U+11A93 */
  0x9A94,   /* U+11A94 */
  0x9A95,   /* U+11A95 */
  0x9A96,   /* U+11A96 */
  0x9A97,   /* U+11A97 */
  0x9A98,   /* U+11A98 */
  0x9A99,   /* U+11A99 */
  0x9A9A,   /* U+11A9A */
  0x9A9B,   /* U+11A9B */
  0x9A9C,   /* U+11A9C */
  0x9A9D,   /* U+11A9D */
  0x9A9E,   /* U+11A9E */
  0x9A9F,   /* U+11A9F */
  0x9AA0,   /* U+11AA0 */
  0x9AA1,   /* U+11AA1 */
  0x9AA2,   /* U+11AA2 */
  0x9AA3,   /* U+11AA3 */
  0x9AA4,   /* U+11AA4 */
  0x9AA5,   /* U+11AA5 */
  0x9AA6,   /* U+11AA6 */
  0x9AA7,   /* U+11AA7 */
  0x9AA8,   /* U+11AA8 */
  0x9AA9,   /* U+11AA9 */
  0x9AAA,   /* U+11AAA */
  0x9AAB,   /* U+11AAB */
  0x9AAC,   /* U+11AAC */
  0x9AAD,   /* U+11AAD */
  0x9AAE,   /* U+11AAE */
  0x9AAF,   /* U+11AAF */
  0x9AB0,   /* U+11AB0 */
  0x9AB1,   /* U+11AB1 */
  0x9AB2,   /* U+11AB2 */
  0x9AB3,   /* U+11AB3 */
  0x9AB4,   /* U+11AB4 */
  0x9AB5,   /* U+11AB5 */
  0x9AB6,   /* U+11AB6 */
  0x9AB7,   /* U+11AB7 */
  0x9AB8,   /* U+11AB8 */
  0x9AB9,   /* U+11AB9 */
  0x9ABA,   /* U+11ABA */
  0x9ABB,   /* U+11ABB */
  0x9ABC,   /* U+11ABC */
  0x9ABD,   /* U+11ABD */
  0x9ABE,   /* U+11ABE */
  0x9ABF,   /* U+11ABF */
  0x0000,   /* U+11AC0 */
  0x0000,   /* U+11AC1 */
  0x0000,   /* U+11AC2 */
  0x0000,   /* U+11AC3 */
  0x0000,   /* U+11AC4 */
  0x0000,   /* U+11AC5 */
  0x0000,   /* U+11AC6 */
  0x0000,   /* U+11AC7 */
  0x0000,   /* U+11AC8 */
  0x0000,   /* U+11AC9 */
  0x0000,   /* U+11ACA */
  0x0000,   /* U+11ACB */
  0x0000,   /* U+11ACC */
  0x0000,   /* U+11ACD */
  0x0000,   /* U+11ACE */
  0x0000,   /* U+11ACF */
  0x0000,   /* U+11AD0 */
  0x0000,   /* U+11AD1 */
  0x0000,   /* U+11AD2 */
  0x0000,   /* U+11AD3 */
  0x0000,   /* U+11AD4 */
  0x0000,   /* U+11AD5 */
  0x0000,   /* U+11AD6 */
  0x0000,   /* U+11AD7 */
  0x0000,   /* U+11AD8 */
  0x0000,   /* U+11AD9 */
  0x0000,   /* U+11ADA */
  0x0000,   /* U+11ADB */
  0x0000,   /* U+11ADC */
  0x0000,   /* U+11ADD */
  0x0000,   /* U+11ADE */
  0x0000,   /* U+11ADF */
  0x0000,   /* U+11AE0 */
  0x0000,   /* U+11AE1 */
  0x0000,   /* U+11AE2 */
  0x0000,   /* U+11AE3 */
  0x0000,   /* U+11AE4 */
  0x0000,   /* U+11AE5 */
  0x0000,   /* U+11AE6 */
  0x0000,   /* U+11AE7 */
  0x0000,   /* U+11AE8 */
  0x0000,   /* U+11AE9 */
  0x0000,   /* U+11AEA */
  0x0000,   /* U+11AEB */
  0x0000,   /* U+11AEC */
  0x0000,   /* U+11AED */
  0x0000,   /* U+11AEE */
  0x0000,   /* U+11AEF */
  0x0000,   /* U+11AF0 */
  0x0000,   /* U+11AF1 */
  0x0000,   /* U+11AF2 */
  0x0000,   /* U+11AF3 */
  0x0000,   /* U+11AF4 */
  0x0000,   /* U+11AF5 */
  0x0000,   /* U+11AF6 */
  0x0000,   /* U+11AF7 */
  0x0000,   /* U+11AF8 */
  0x9AF9,   /* U+11AF9 */
  0x9AFA,   /* U+11AFA */
  0x9AFB,   /* U+11AFB */
  0x9AFC,   /* U+11AFC */
  0x9AFD,   /* U+11AFD */
  0x9AFE,   /* U+11AFE */
  0x9AFF,   /* U+11AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11A00 */
  0x0000,   /* U+11A01 */
  0x0000,   /* U+11A02 */
  0x0000,   /* U+11A03 */
  0x0000,   /* U+11A04 */
  0x0000,   /* U+11A05 */
  0x0000,   /* U+11A06 */
  0x0000,   /* U+11A07 */
  0x0000,   /* U+11A08 */
  0x0000,   /* U+11A09 */
  0x0000,   /* U+11A0A */
  0x0000,   /* U+11A0B */
  0x0000,   /* U+11A0C */
  0x0000,   /* U+11A0D */
  0x0000,   /* U+11A0E */
  0x0000,   /* U+11A0F */
  0x0000,   /* U+11A10 */
  0x0000,   /* U+11A11 */
  0x0000,   /* U+11A12 */
  0x0000,   /* U+11A13 */
  0x0000,   /* U+11A14 */
  0x0000,   /* U+11A15 */
  0x0000,   /* U+11A16 */
  0x0000,   /* U+11A17 */
  0x0000,   /* U+11A18 */
  0x0000,   /* U+11A19 */
  0x0000,   /* U+11A1A */
  0x0000,   /* U+11A1B */
  0x0000,   /* U+11A1C */
  0x0000,   /* U+11A1D */
  0x0000,   /* U+11A1E */
  0x0000,   /* U+11A1F */
  0x0000,   /* U+11A20 */
  0x0000,   /* U+11A21 */
  0x0000,   /* U+11A22 */
  0x0000,   /* U+11A23 */
  0x0000,   /* U+11A24 */
  0x0000,   /* U+11A25 */
  0x0000,   /* U+11A26 */
  0x0000,   /* U+11A27 */
  0x0000,   /* U+11A28 */
  0x0000,   /* U+11A29 */
  0x0000,   /* U+11A2A */
  0x0000,   /* U+11A2B */
  0x0000,   /* U+11A2C */
  0x0000,   /* U+11A2D */
  0x0000,   /* U+11A2E */
  0x0000,   /* U+11A2F */
  0x0000,   /* U+11A30 */
  0x0000,   /* U+11A31 */
  0x0000,   /* U+11A32 */
  0x0000,   /* U+11A33 */
  0x0000,   /* U+11A34 */
  0x0000,   /* U+11A35 */
  0x0000,   /* U+11A36 */
  0x0000,   /* U+11A37 */
  0x0000,   /* U+11A38 */
  0x0000,   /* U+11A39 */
  0x0000,   /* U+11A3A */
  0x0000,   /* U+11A3B */
  0x0000,   /* U+11A3C */
  0x0000,   /* U+11A3D */
  0x0000,   /* U+11A3E */
  0x0000,   /* U+11A3F */
  0x0000,   /* U+11A40 */
  0x0000,   /* U+11A41 */
  0x0000,   /* U+11A42 */
  0x0000,   /* U+11A43 */
  0x0000,   /* U+11A44 */
  0x0000,   /* U+11A45 */
  0x0000,   /* U+11A46 */
  0x0000,   /* U+11A47 */
  0x0000,   /* U+11A48 */
  0x0000,   /* U+11A49 */
  0x0000,   /* U+11A4A */
  0x0000,   /* U+11A4B */
  0x0000,   /* U+11A4C */
  0x0000,   /* U+11A4D */
  0x0000,   /* U+11A4E */
  0x0000,   /* U+11A4F */
  0x0000,   /* U+11A50 */
  0x0000,   /* U+11A51 */
  0x0000,   /* U+11A52 */
  0x0000,   /* U+11A53 */
  0x0000,   /* U+11A54 */
  0x0000,   /* U+11A55 */
  0x0000,   /* U+11A56 */
  0x0000,   /* U+11A57 */
  0x0000,   /* U+11A58 */
  0x0000,   /* U+11A59 */
  0x0000,   /* U+11A5A */
  0x0000,   /* U+11A5B */
  0x0000,   /* U+11A5C */
  0x0000,   /* U+11A5D */
  0x0000,   /* U+11A5E */
  0x0000,   /* U+11A5F */
  0x0000,   /* U+11A60 */
  0x0000,   /* U+11A61 */
  0x0000,   /* U+11A62 */
  0x0000,   /* U+11A63 */
  0x0000,   /* U+11A64 */
  0x0000,   /* U+11A65 */
  0x0000,   /* U+11A66 */
  0x0000,   /* U+11A67 */
  0x0000,   /* U+11A68 */
  0x0000,   /* U+11A69 */
  0x0000,   /* U+11A6A */
  0x0000,   /* U+11A6B */
  0x0000,   /* U+11A6C */
  0x0000,   /* U+11A6D */
  0x0000,   /* U+11A6E */
  0x0000,   /* U+11A6F */
  0x0000,   /* U+11A70 */
  0x0000,   /* U+11A71 */
  0x0000,   /* U+11A72 */
  0x0000,   /* U+11A73 */
  0x0000,   /* U+11A74 */
  0x0000,   /* U+11A75 */
  0x0000,   /* U+11A76 */
  0x0000,   /* U+11A77 */
  0x0000,   /* U+11A78 */
  0x0000,   /* U+11A79 */
  0x0000,   /* U+11A7A */
  0x0000,   /* U+11A7B */
  0x0000,   /* U+11A7C */
  0x0000,   /* U+11A7D */
  0x0000,   /* U+11A7E */
  0x0000,   /* U+11A7F */
  0x0000,   /* U+11A80 */
  0x0000,   /* U+11A81 */
  0x0000,   /* U+11A82 */
  0x0000,   /* U+11A83 */
  0x0000,   /* U+11A84 */
  0x0000,   /* U+11A85 */
  0x0000,   /* U+11A86 */
  0x0000,   /* U+11A87 */
  0x0000,   /* U+11A88 */
  0x0000,   /* U+11A89 */
  0x0000,   /* U+11A8A */
  0x0000,   /* U+11A8B */
  0x0000,   /* U+11A8C */
  0x0000,   /* U+11A8D */
  0x0000,   /* U+11A8E */
  0x0000,   /* U+11A8F */
  0x0000,   /* U+11A90 */
  0x0000,   /* U+11A91 */
  0x0000,   /* U+11A92 */
  0x0000,   /* U+11A93 */
  0x0000,   /* U+11A94 */
  0x0000,   /* U+11A95 */
  0x0000,   /* U+11A96 */
  0x0000,   /* U+11A97 */
  0x0000,   /* U+11A98 */
  0x0000,   /* U+11A99 */
  0x0000,   /* U+11A9A */
  0x0000,   /* U+11A9B */
  0x0000,   /* U+11A9C */
  0x0000,   /* U+11A9D */
  0x0000,   /* U+11A9E */
  0x0000,   /* U+11A9F */
  0x0000,   /* U+11AA0 */
  0x0000,   /* U+11AA1 */
  0x0000,   /* U+11AA2 */
  0x0000,   /* U+11AA3 */
  0x0000,   /* U+11AA4 */
  0x0000,   /* U+11AA5 */
  0x0000,   /* U+11AA6 */
  0x0000,   /* U+11AA7 */
  0x0000,   /* U+11AA8 */
  0x0000,   /* U+11AA9 */
  0x0000,   /* U+11AAA */
  0x0000,   /* U+11AAB */
  0x0000,   /* U+11AAC */
  0x0000,   /* U+11AAD */
  0x0000,   /* U+11AAE */
  0x0000,   /* U+11AAF */
  0x0000,   /* U+11AB0 */
  0x0000,   /* U+11AB1 */
  0x0000,   /* U+11AB2 */
  0x0000,   /* U+11AB3 */
  0x0000,   /* U+11AB4 */
  0x0000,   /* U+11AB5 */
  0x0000,   /* U+11AB6 */
  0x0000,   /* U+11AB7 */
  0x0000,   /* U+11AB8 */
  0x0000,   /* U+11AB9 */
  0x0000,   /* U+11ABA */
  0x0000,   /* U+11ABB */
  0x0000,   /* U+11ABC */
  0x0000,   /* U+11ABD */
  0x0000,   /* U+11ABE */
  0x0000,   /* U+11ABF */
  0x0000,   /* U+11AC0 */
  0x0000,   /* U+11AC1 */
  0x0000,   /* U+11AC2 */
  0x0000,   /* U+11AC3 */
  0x0000,   /* U+11AC4 */
  0x0000,   /* U+11AC5 */
  0x0000,   /* U+11AC6 */
  0x0000,   /* U+11AC7 */
  0x0000,   /* U+11AC8 */
  0x0000,   /* U+11AC9 */
  0x0000,   /* U+11ACA */
  0x0000,   /* U+11ACB */
  0x0000,   /* U+11ACC */
  0x0000,   /* U+11ACD */
  0x0000,   /* U+11ACE */
  0x0000,   /* U+11ACF */
  0x0000,   /* U+11AD0 */
  0x0000,   /* U+11AD1 */
  0x0000,   /* U+11AD2 */
  0x0000,   /* U+11AD3 */
  0x0000,   /* U+11AD4 */
  0x0000,   /* U+11AD5 */
  0x0000,   /* U+11AD6 */
  0x0000,   /* U+11AD7 */
  0x0000,   /* U+11AD8 */
  0x0000,   /* U+11AD9 */
  0x0000,   /* U+11ADA */
  0x0000,   /* U+11ADB */
  0x0000,   /* U+11ADC */
  0x0000,   /* U+11ADD */
  0x0000,   /* U+11ADE */
  0x0000,   /* U+11ADF */
  0x0000,   /* U+11AE0 */
  0x0000,   /* U+11AE1 */
  0x0000,   /* U+11AE2 */
  0x0000,   /* U+11AE3 */
  0x0000,   /* U+11AE4 */
  0x0000,   /* U+11AE5 */
  0x0000,   /* U+11AE6 */
  0x0000,   /* U+11AE7 */
  0x0000,   /* U+11AE8 */
  0x0000,   /* U+11AE9 */
  0x0000,   /* U+11AEA */
  0x0000,   /* U+11AEB */
  0x0000,   /* U+11AEC */
  0x0000,   /* U+11AED */
  0x0000,   /* U+11AEE */
  0x0000,   /* U+11AEF */
  0x0000,   /* U+11AF0 */
  0x0000,   /* U+11AF1 */
  0x0000,   /* U+11AF2 */
  0x0000,   /* U+11AF3 */
  0x0000,   /* U+11AF4 */
  0x0000,   /* U+11AF5 */
  0x0000,   /* U+11AF6 */
  0x0000,   /* U+11AF7 */
  0x0000,   /* U+11AF8 */
  0x0000,   /* U+11AF9 */
  0x0000,   /* U+11AFA */
  0x0000,   /* U+11AFB */
  0x0000,   /* U+11AFC */
  0x0000,   /* U+11AFD */
  0x0000,   /* U+11AFE */
  0x0000,   /* U+11AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11A00 */
  0x0000,   /* U+11A01 */
  0x0000,   /* U+11A02 */
  0x0000,   /* U+11A03 */
  0x0000,   /* U+11A04 */
  0x0000,   /* U+11A05 */
  0x0000,   /* U+11A06 */
  0x0000,   /* U+11A07 */
  0x0000,   /* U+11A08 */
  0x0000,   /* U+11A09 */
  0x0000,   /* U+11A0A */
  0x0000,   /* U+11A0B */
  0x0000,   /* U+11A0C */
  0x0000,   /* U+11A0D */
  0x0000,   /* U+11A0E */
  0x0000,   /* U+11A0F */
  0x0000,   /* U+11A10 */
  0x0000,   /* U+11A11 */
  0x0000,   /* U+11A12 */
  0x0000,   /* U+11A13 */
  0x0000,   /* U+11A14 */
  0x0000,   /* U+11A15 */
  0x0000,   /* U+11A16 */
  0x0000,   /* U+11A17 */
  0x0000,   /* U+11A18 */
  0x0000,   /* U+11A19 */
  0x0000,   /* U+11A1A */
  0x0000,   /* U+11A1B */
  0x0000,   /* U+11A1C */
  0x0000,   /* U+11A1D */
  0x0000,   /* U+11A1E */
  0x0000,   /* U+11A1F */
  0x0000,   /* U+11A20 */
  0x0000,   /* U+11A21 */
  0x0000,   /* U+11A22 */
  0x0000,   /* U+11A23 */
  0x0000,   /* U+11A24 */
  0x0000,   /* U+11A25 */
  0x0000,   /* U+11A26 */
  0x0000,   /* U+11A27 */
  0x0000,   /* U+11A28 */
  0x0000,   /* U+11A29 */
  0x0000,   /* U+11A2A */
  0x0000,   /* U+11A2B */
  0x0000,   /* U+11A2C */
  0x0000,   /* U+11A2D */
  0x0000,   /* U+11A2E */
  0x0000,   /* U+11A2F */
  0x0000,   /* U+11A30 */
  0x0000,   /* U+11A31 */
  0x0000,   /* U+11A32 */
  0x0000,   /* U+11A33 */
  0x0000,   /* U+11A34 */
  0x0000,   /* U+11A35 */
  0x0000,   /* U+11A36 */
  0x0000,   /* U+11A37 */
  0x0000,   /* U+11A38 */
  0x0000,   /* U+11A39 */
  0x0000,   /* U+11A3A */
  0x0000,   /* U+11A3B */
  0x0000,   /* U+11A3C */
  0x0000,   /* U+11A3D */
  0x0000,   /* U+11A3E */
  0x0000,   /* U+11A3F */
  0x0000,   /* U+11A40 */
  0x0000,   /* U+11A41 */
  0x0000,   /* U+11A42 */
  0x0000,   /* U+11A43 */
  0x0000,   /* U+11A44 */
  0x0000,   /* U+11A45 */
  0x0000,   /* U+11A46 */
  0x0000,   /* U+11A47 */
  0x0000,   /* U+11A48 */
  0x0000,   /* U+11A49 */
  0x0000,   /* U+11A4A */
  0x0000,   /* U+11A4B */
  0x0000,   /* U+11A4C */
  0x0000,   /* U+11A4D */
  0x0000,   /* U+11A4E */
  0x0000,   /* U+11A4F */
  0x0000,   /* U+11A50 */
  0x0000,   /* U+11A51 */
  0x0000,   /* U+11A52 */
  0x0000,   /* U+11A53 */
  0x0000,   /* U+11A54 */
  0x0000,   /* U+11A55 */
  0x0000,   /* U+11A56 */
  0x0000,   /* U+11A57 */
  0x0000,   /* U+11A58 */
  0x0000,   /* U+11A59 */
  0x0000,   /* U+11A5A */
  0x0000,   /* U+11A5B */
  0x0000,   /* U+11A5C */
  0x0000,   /* U+11A5D */
  0x0000,   /* U+11A5E */
  0x0000,   /* U+11A5F */
  0x0000,   /* U+11A60 */
  0x0000,   /* U+11A61 */
  0x0000,   /* U+11A62 */
  0x0000,   /* U+11A63 */
  0x0000,   /* U+11A64 */
  0x0000,   /* U+11A65 */
  0x0000,   /* U+11A66 */
  0x0000,   /* U+11A67 */
  0x0000,   /* U+11A68 */
  0x0000,   /* U+11A69 */
  0x0000,   /* U+11A6A */
  0x0000,   /* U+11A6B */
  0x0000,   /* U+11A6C */
  0x0000,   /* U+11A6D */
  0x0000,   /* U+11A6E */
  0x0000,   /* U+11A6F */
  0x0000,   /* U+11A70 */
  0x0000,   /* U+11A71 */
  0x0000,   /* U+11A72 */
  0x0000,   /* U+11A73 */
  0x0000,   /* U+11A74 */
  0x0000,   /* U+11A75 */
  0x0000,   /* U+11A76 */
  0x0000,   /* U+11A77 */
  0x0000,   /* U+11A78 */
  0x0000,   /* U+11A79 */
  0x0000,   /* U+11A7A */
  0x0000,   /* U+11A7B */
  0x0000,   /* U+11A7C */
  0x0000,   /* U+11A7D */
  0x0000,   /* U+11A7E */
  0x0000,   /* U+11A7F */
  0x0000,   /* U+11A80 */
  0x0000,   /* U+11A81 */
  0x0000,   /* U+11A82 */
  0x0000,   /* U+11A83 */
  0x0000,   /* U+11A84 */
  0x0000,   /* U+11A85 */
  0x0000,   /* U+11A86 */
  0x0000,   /* U+11A87 */
  0x0000,   /* U+11A88 */
  0x0000,   /* U+11A89 */
  0x0000,   /* U+11A8A */
  0x0000,   /* U+11A8B */
  0x0000,   /* U+11A8C */
  0x0000,   /* U+11A8D */
  0x0000,   /* U+11A8E */
  0x0000,   /* U+11A8F */
  0x0000,   /* U+11A90 */
  0x0000,   /* U+11A91 */
  0x0000,   /* U+11A92 */
  0x0000,   /* U+11A93 */
  0x0000,   /* U+11A94 */
  0x0000,   /* U+11A95 */
  0x0000,   /* U+11A96 */
  0x0000,   /* U+11A97 */
  0x0000,   /* U+11A98 */
  0x0000,   /* U+11A99 */
  0x0000,   /* U+11A9A */
  0x0000,   /* U+11A9B */
  0x0000,   /* U+11A9C */
  0x0000,   /* U+11A9D */
  0x0000,   /* U+11A9E */
  0x0000,   /* U+11A9F */
  0x0000,   /* U+11AA0 */
  0x0000,   /* U+11AA1 */
  0x0000,   /* U+11AA2 */
  0x0000,   /* U+11AA3 */
  0x0000,   /* U+11AA4 */
  0x0000,   /* U+11AA5 */
  0x0000,   /* U+11AA6 */
  0x0000,   /* U+11AA7 */
  0x0000,   /* U+11AA8 */
  0x0000,   /* U+11AA9 */
  0x0000,   /* U+11AAA */
  0x0000,   /* U+11AAB */
  0x0000,   /* U+11AAC */
  0x0000,   /* U+11AAD */
  0x0000,   /* U+11AAE */
  0x0000,   /* U+11AAF */
  0x0000,   /* U+11AB0 */
  0x0000,   /* U+11AB1 */
  0x0000,   /* U+11AB2 */
  0x0000,   /* U+11AB3 */
  0x0000,   /* U+11AB4 */
  0x0000,   /* U+11AB5 */
  0x0000,   /* U+11AB6 */
  0x0000,   /* U+11AB7 */
  0x0000,   /* U+11AB8 */
  0x0000,   /* U+11AB9 */
  0x0000,   /* U+11ABA */
  0x0000,   /* U+11ABB */
  0x0000,   /* U+11ABC */
  0x0000,   /* U+11ABD */
  0x0000,   /* U+11ABE */
  0x0000,   /* U+11ABF */
  0x0000,   /* U+11AC0 */
  0x0000,   /* U+11AC1 */
  0x0000,   /* U+11AC2 */
  0x0000,   /* U+11AC3 */
  0x0000,   /* U+11AC4 */
  0x0000,   /* U+11AC5 */
  0x0000,   /* U+11AC6 */
  0x0000,   /* U+11AC7 */
  0x0000,   /* U+11AC8 */
  0x0000,   /* U+11AC9 */
  0x0000,   /* U+11ACA */
  0x0000,   /* U+11ACB */
  0x0000,   /* U+11ACC */
  0x0000,   /* U+11ACD */
  0x0000,   /* U+11ACE */
  0x0000,   /* U+11ACF */
  0x0000,   /* U+11AD0 */
  0x0000,   /* U+11AD1 */
  0x0000,   /* U+11AD2 */
  0x0000,   /* U+11AD3 */
  0x0000,   /* U+11AD4 */
  0x0000,   /* U+11AD5 */
  0x0000,   /* U+11AD6 */
  0x0000,   /* U+11AD7 */
  0x0000,   /* U+11AD8 */
  0x0000,   /* U+11AD9 */
  0x0000,   /* U+11ADA */
  0x0000,   /* U+11ADB */
  0x0000,   /* U+11ADC */
  0x0000,   /* U+11ADD */
  0x0000,   /* U+11ADE */
  0x0000,   /* U+11ADF */
  0x0000,   /* U+11AE0 */
  0x0000,   /* U+11AE1 */
  0x0000,   /* U+11AE2 */
  0x0000,   /* U+11AE3 */
  0x0000,   /* U+11AE4 */
  0x0000,   /* U+11AE5 */
  0x0000,   /* U+11AE6 */
  0x0000,   /* U+11AE7 */
  0x0000,   /* U+11AE8 */
  0x0000,   /* U+11AE9 */
  0x0000,   /* U+11AEA */
  0x0000,   /* U+11AEB */
  0x0000,   /* U+11AEC */
  0x0000,   /* U+11AED */
  0x0000,   /* U+11AEE */
  0x0000,   /* U+11AEF */
  0x0000,   /* U+11AF0 */
  0x0000,   /* U+11AF1 */
  0x0000,   /* U+11AF2 */
  0x0000,   /* U+11AF3 */
  0x0000,   /* U+11AF4 */
  0x0000,   /* U+11AF5 */
  0x0000,   /* U+11AF6 */
  0x0000,   /* U+11AF7 */
  0x0000,   /* U+11AF8 */
  0x0000,   /* U+11AF9 */
  0x0000,   /* U+11AFA */
  0x0000,   /* U+11AFB */
  0x0000,   /* U+11AFC */
  0x0000,   /* U+11AFD */
  0x0000,   /* U+11AFE */
  0x0000,   /* U+11AFF */
};

uint16 uca900_p11C[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x2D37,   /* U+11C00 */
  0x2D38,   /* U+11C01 */
  0x2D39,   /* U+11C02 */
  0x2D3A,   /* U+11C03 */
  0x2D3B,   /* U+11C04 */
  0x2D3C,   /* U+11C05 */
  0x2D3D,   /* U+11C06 */
  0x2D3E,   /* U+11C07 */
  0x2D3F,   /* U+11C08 */
  0xFBC2,   /* U+11C09 */
  0x2D40,   /* U+11C0A */
  0x2D41,   /* U+11C0B */
  0x2D42,   /* U+11C0C */
  0x2D43,   /* U+11C0D */
  0x2D44,   /* U+11C0E */
  0x2D45,   /* U+11C0F */
  0x2D46,   /* U+11C10 */
  0x2D47,   /* U+11C11 */
  0x2D48,   /* U+11C12 */
  0x2D49,   /* U+11C13 */
  0x2D4A,   /* U+11C14 */
  0x2D4B,   /* U+11C15 */
  0x2D4C,   /* U+11C16 */
  0x2D4D,   /* U+11C17 */
  0x2D4E,   /* U+11C18 */
  0x2D4F,   /* U+11C19 */
  0x2D50,   /* U+11C1A */
  0x2D51,   /* U+11C1B */
  0x2D52,   /* U+11C1C */
  0x2D53,   /* U+11C1D */
  0x2D54,   /* U+11C1E */
  0x2D55,   /* U+11C1F */
  0x2D56,   /* U+11C20 */
  0x2D57,   /* U+11C21 */
  0x2D58,   /* U+11C22 */
  0x2D59,   /* U+11C23 */
  0x2D5A,   /* U+11C24 */
  0x2D5B,   /* U+11C25 */
  0x2D5C,   /* U+11C26 */
  0x2D5D,   /* U+11C27 */
  0x2D5E,   /* U+11C28 */
  0x2D5F,   /* U+11C29 */
  0x2D60,   /* U+11C2A */
  0x2D61,   /* U+11C2B */
  0x2D62,   /* U+11C2C */
  0x2D63,   /* U+11C2D */
  0x2D64,   /* U+11C2E */
  0x2D66,   /* U+11C2F */
  0x2D67,   /* U+11C30 */
  0x2D68,   /* U+11C31 */
  0x2D69,   /* U+11C32 */
  0x2D6A,   /* U+11C33 */
  0x2D6B,   /* U+11C34 */
  0x2D6C,   /* U+11C35 */
  0x2D6D,   /* U+11C36 */
  0xFBC2,   /* U+11C37 */
  0x2D6E,   /* U+11C38 */
  0x2D6F,   /* U+11C39 */
  0x2D70,   /* U+11C3A */
  0x2D71,   /* U+11C3B */
  0x0000,   /* U+11C3C */
  0x0000,   /* U+11C3D */
  0x0000,   /* U+11C3E */
  0x2D72,   /* U+11C3F */
  0x2D65,   /* U+11C40 */
  0x02BE,   /* U+11C41 */
  0x02BF,   /* U+11C42 */
  0x043F,   /* U+11C43 */
  0x0440,   /* U+11C44 */
  0x0441,   /* U+11C45 */
  0xFBC2,   /* U+11C46 */
  0xFBC2,   /* U+11C47 */
  0xFBC2,   /* U+11C48 */
  0xFBC2,   /* U+11C49 */
  0xFBC2,   /* U+11C4A */
  0xFBC2,   /* U+11C4B */
  0xFBC2,   /* U+11C4C */
  0xFBC2,   /* U+11C4D */
  0xFBC2,   /* U+11C4E */
  0xFBC2,   /* U+11C4F */
  0x1C3D,   /* U+11C50 */
  0x1C3E,   /* U+11C51 */
  0x1C3F,   /* U+11C52 */
  0x1C40,   /* U+11C53 */
  0x1C41,   /* U+11C54 */
  0x1C42,   /* U+11C55 */
  0x1C43,   /* U+11C56 */
  0x1C44,   /* U+11C57 */
  0x1C45,   /* U+11C58 */
  0x1C46,   /* U+11C59 */
  0x1C3E,   /* U+11C5A */
  0x1C3F,   /* U+11C5B */
  0x1C40,   /* U+11C5C */
  0x1C41,   /* U+11C5D */
  0x1C42,   /* U+11C5E */
  0x1C43,   /* U+11C5F */
  0x1C44,   /* U+11C60 */
  0x1C45,   /* U+11C61 */
  0x1C46,   /* U+11C62 */
  0x1B82,   /* U+11C63 */
  0x1B83,   /* U+11C64 */
  0x1B84,   /* U+11C65 */
  0x1B85,   /* U+11C66 */
  0x1B86,   /* U+11C67 */
  0x1B87,   /* U+11C68 */
  0x1B88,   /* U+11C69 */
  0x1B89,   /* U+11C6A */
  0x1B8A,   /* U+11C6B */
  0x1B8B,   /* U+11C6C */
  0xFBC2,   /* U+11C6D */
  0xFBC2,   /* U+11C6E */
  0xFBC2,   /* U+11C6F */
  0x0409,   /* U+11C70 */
  0x040A,   /* U+11C71 */
  0x2E86,   /* U+11C72 */
  0x2E88,   /* U+11C73 */
  0x2E8A,   /* U+11C74 */
  0x2E8C,   /* U+11C75 */
  0x2E8E,   /* U+11C76 */
  0x2E90,   /* U+11C77 */
  0x2E92,   /* U+11C78 */
  0x2E94,   /* U+11C79 */
  0x2E96,   /* U+11C7A */
  0x2E98,   /* U+11C7B */
  0x2E9A,   /* U+11C7C */
  0x2E9C,   /* U+11C7D */
  0x2E9E,   /* U+11C7E */
  0x2EA0,   /* U+11C7F */
  0x2EA2,   /* U+11C80 */
  0x2EA4,   /* U+11C81 */
  0x2EA6,   /* U+11C82 */
  0x2EA8,   /* U+11C83 */
  0x2EAA,   /* U+11C84 */
  0x2EAC,   /* U+11C85 */
  0x2EAE,   /* U+11C86 */
  0x2EB0,   /* U+11C87 */
  0x2EB2,   /* U+11C88 */
  0x2EB3,   /* U+11C89 */
  0x2EB5,   /* U+11C8A */
  0x2EB7,   /* U+11C8B */
  0x2EB9,   /* U+11C8C */
  0x2EBB,   /* U+11C8D */
  0x2EBD,   /* U+11C8E */
  0x2EBF,   /* U+11C8F */
  0xFBC2,   /* U+11C90 */
  0xFBC2,   /* U+11C91 */
  0x2E87,   /* U+11C92 */
  0x2E89,   /* U+11C93 */
  0x2E8B,   /* U+11C94 */
  0x2E8D,   /* U+11C95 */
  0x2E8F,   /* U+11C96 */
  0x2E91,   /* U+11C97 */
  0x2E93,   /* U+11C98 */
  0x2E95,   /* U+11C99 */
  0x2E97,   /* U+11C9A */
  0x2E99,   /* U+11C9B */
  0x2E9B,   /* U+11C9C */
  0x2E9D,   /* U+11C9D */
  0x2E9F,   /* U+11C9E */
  0x2EA1,   /* U+11C9F */
  0x2EA3,   /* U+11CA0 */
  0x2EA5,   /* U+11CA1 */
  0x2EA7,   /* U+11CA2 */
  0x2EA9,   /* U+11CA3 */
  0x2EAB,   /* U+11CA4 */
  0x2EAD,   /* U+11CA5 */
  0x2EAF,   /* U+11CA6 */
  0x2EB1,   /* U+11CA7 */
  0xFBC2,   /* U+11CA8 */
  0x2EB4,   /* U+11CA9 */
  0x2EB6,   /* U+11CAA */
  0x2EB8,   /* U+11CAB */
  0x2EBA,   /* U+11CAC */
  0x2EBC,   /* U+11CAD */
  0x2EBE,   /* U+11CAE */
  0x2EC0,   /* U+11CAF */
  0x2EC1,   /* U+11CB0 */
  0x2EC2,   /* U+11CB1 */
  0x2EC3,   /* U+11CB2 */
  0x2EC4,   /* U+11CB3 */
  0x2EC5,   /* U+11CB4 */
  0x0000,   /* U+11CB5 */
  0x0000,   /* U+11CB6 */
  0xFBC2,   /* U+11CB7 */
  0xFBC2,   /* U+11CB8 */
  0xFBC2,   /* U+11CB9 */
  0xFBC2,   /* U+11CBA */
  0xFBC2,   /* U+11CBB */
  0xFBC2,   /* U+11CBC */
  0xFBC2,   /* U+11CBD */
  0xFBC2,   /* U+11CBE */
  0xFBC2,   /* U+11CBF */
  0xFBC2,   /* U+11CC0 */
  0xFBC2,   /* U+11CC1 */
  0xFBC2,   /* U+11CC2 */
  0xFBC2,   /* U+11CC3 */
  0xFBC2,   /* U+11CC4 */
  0xFBC2,   /* U+11CC5 */
  0xFBC2,   /* U+11CC6 */
  0xFBC2,   /* U+11CC7 */
  0xFBC2,   /* U+11CC8 */
  0xFBC2,   /* U+11CC9 */
  0xFBC2,   /* U+11CCA */
  0xFBC2,   /* U+11CCB */
  0xFBC2,   /* U+11CCC */
  0xFBC2,   /* U+11CCD */
  0xFBC2,   /* U+11CCE */
  0xFBC2,   /* U+11CCF */
  0xFBC2,   /* U+11CD0 */
  0xFBC2,   /* U+11CD1 */
  0xFBC2,   /* U+11CD2 */
  0xFBC2,   /* U+11CD3 */
  0xFBC2,   /* U+11CD4 */
  0xFBC2,   /* U+11CD5 */
  0xFBC2,   /* U+11CD6 */
  0xFBC2,   /* U+11CD7 */
  0xFBC2,   /* U+11CD8 */
  0xFBC2,   /* U+11CD9 */
  0xFBC2,   /* U+11CDA */
  0xFBC2,   /* U+11CDB */
  0xFBC2,   /* U+11CDC */
  0xFBC2,   /* U+11CDD */
  0xFBC2,   /* U+11CDE */
  0xFBC2,   /* U+11CDF */
  0xFBC2,   /* U+11CE0 */
  0xFBC2,   /* U+11CE1 */
  0xFBC2,   /* U+11CE2 */
  0xFBC2,   /* U+11CE3 */
  0xFBC2,   /* U+11CE4 */
  0xFBC2,   /* U+11CE5 */
  0xFBC2,   /* U+11CE6 */
  0xFBC2,   /* U+11CE7 */
  0xFBC2,   /* U+11CE8 */
  0xFBC2,   /* U+11CE9 */
  0xFBC2,   /* U+11CEA */
  0xFBC2,   /* U+11CEB */
  0xFBC2,   /* U+11CEC */
  0xFBC2,   /* U+11CED */
  0xFBC2,   /* U+11CEE */
  0xFBC2,   /* U+11CEF */
  0xFBC2,   /* U+11CF0 */
  0xFBC2,   /* U+11CF1 */
  0xFBC2,   /* U+11CF2 */
  0xFBC2,   /* U+11CF3 */
  0xFBC2,   /* U+11CF4 */
  0xFBC2,   /* U+11CF5 */
  0xFBC2,   /* U+11CF6 */
  0xFBC2,   /* U+11CF7 */
  0xFBC2,   /* U+11CF8 */
  0xFBC2,   /* U+11CF9 */
  0xFBC2,   /* U+11CFA */
  0xFBC2,   /* U+11CFB */
  0xFBC2,   /* U+11CFC */
  0xFBC2,   /* U+11CFD */
  0xFBC2,   /* U+11CFE */
  0xFBC2,   /* U+11CFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+11C00 */
  0x0020,   /* U+11C01 */
  0x0020,   /* U+11C02 */
  0x0020,   /* U+11C03 */
  0x0020,   /* U+11C04 */
  0x0020,   /* U+11C05 */
  0x0020,   /* U+11C06 */
  0x0020,   /* U+11C07 */
  0x0020,   /* U+11C08 */
  0x0020,   /* U+11C09 */
  0x0020,   /* U+11C0A */
  0x0020,   /* U+11C0B */
  0x0020,   /* U+11C0C */
  0x0020,   /* U+11C0D */
  0x0020,   /* U+11C0E */
  0x0020,   /* U+11C0F */
  0x0020,   /* U+11C10 */
  0x0020,   /* U+11C11 */
  0x0020,   /* U+11C12 */
  0x0020,   /* U+11C13 */
  0x0020,   /* U+11C14 */
  0x0020,   /* U+11C15 */
  0x0020,   /* U+11C16 */
  0x0020,   /* U+11C17 */
  0x0020,   /* U+11C18 */
  0x0020,   /* U+11C19 */
  0x0020,   /* U+11C1A */
  0x0020,   /* U+11C1B */
  0x0020,   /* U+11C1C */
  0x0020,   /* U+11C1D */
  0x0020,   /* U+11C1E */
  0x0020,   /* U+11C1F */
  0x0020,   /* U+11C20 */
  0x0020,   /* U+11C21 */
  0x0020,   /* U+11C22 */
  0x0020,   /* U+11C23 */
  0x0020,   /* U+11C24 */
  0x0020,   /* U+11C25 */
  0x0020,   /* U+11C26 */
  0x0020,   /* U+11C27 */
  0x0020,   /* U+11C28 */
  0x0020,   /* U+11C29 */
  0x0020,   /* U+11C2A */
  0x0020,   /* U+11C2B */
  0x0020,   /* U+11C2C */
  0x0020,   /* U+11C2D */
  0x0020,   /* U+11C2E */
  0x0020,   /* U+11C2F */
  0x0020,   /* U+11C30 */
  0x0020,   /* U+11C31 */
  0x0020,   /* U+11C32 */
  0x0020,   /* U+11C33 */
  0x0020,   /* U+11C34 */
  0x0020,   /* U+11C35 */
  0x0020,   /* U+11C36 */
  0x0020,   /* U+11C37 */
  0x0020,   /* U+11C38 */
  0x0020,   /* U+11C39 */
  0x0020,   /* U+11C3A */
  0x0020,   /* U+11C3B */
  0x00C3,   /* U+11C3C */
  0x00C4,   /* U+11C3D */
  0x00C5,   /* U+11C3E */
  0x0020,   /* U+11C3F */
  0x0020,   /* U+11C40 */
  0x0020,   /* U+11C41 */
  0x0020,   /* U+11C42 */
  0x0020,   /* U+11C43 */
  0x0020,   /* U+11C44 */
  0x0020,   /* U+11C45 */
  0x0020,   /* U+11C46 */
  0x0020,   /* U+11C47 */
  0x0020,   /* U+11C48 */
  0x0020,   /* U+11C49 */
  0x0020,   /* U+11C4A */
  0x0020,   /* U+11C4B */
  0x0020,   /* U+11C4C */
  0x0020,   /* U+11C4D */
  0x0020,   /* U+11C4E */
  0x0020,   /* U+11C4F */
  0x0020,   /* U+11C50 */
  0x0020,   /* U+11C51 */
  0x0020,   /* U+11C52 */
  0x0020,   /* U+11C53 */
  0x0020,   /* U+11C54 */
  0x0020,   /* U+11C55 */
  0x0020,   /* U+11C56 */
  0x0020,   /* U+11C57 */
  0x0020,   /* U+11C58 */
  0x0020,   /* U+11C59 */
  0x0020,   /* U+11C5A */
  0x0020,   /* U+11C5B */
  0x0020,   /* U+11C5C */
  0x0020,   /* U+11C5D */
  0x0020,   /* U+11C5E */
  0x0020,   /* U+11C5F */
  0x0020,   /* U+11C60 */
  0x0020,   /* U+11C61 */
  0x0020,   /* U+11C62 */
  0x0020,   /* U+11C63 */
  0x0020,   /* U+11C64 */
  0x0020,   /* U+11C65 */
  0x0020,   /* U+11C66 */
  0x0020,   /* U+11C67 */
  0x0020,   /* U+11C68 */
  0x0020,   /* U+11C69 */
  0x0020,   /* U+11C6A */
  0x0020,   /* U+11C6B */
  0x0020,   /* U+11C6C */
  0x0020,   /* U+11C6D */
  0x0020,   /* U+11C6E */
  0x0020,   /* U+11C6F */
  0x0020,   /* U+11C70 */
  0x0020,   /* U+11C71 */
  0x0020,   /* U+11C72 */
  0x0020,   /* U+11C73 */
  0x0020,   /* U+11C74 */
  0x0020,   /* U+11C75 */
  0x0020,   /* U+11C76 */
  0x0020,   /* U+11C77 */
  0x0020,   /* U+11C78 */
  0x0020,   /* U+11C79 */
  0x0020,   /* U+11C7A */
  0x0020,   /* U+11C7B */
  0x0020,   /* U+11C7C */
  0x0020,   /* U+11C7D */
  0x0020,   /* U+11C7E */
  0x0020,   /* U+11C7F */
  0x0020,   /* U+11C80 */
  0x0020,   /* U+11C81 */
  0x0020,   /* U+11C82 */
  0x0020,   /* U+11C83 */
  0x0020,   /* U+11C84 */
  0x0020,   /* U+11C85 */
  0x0020,   /* U+11C86 */
  0x0020,   /* U+11C87 */
  0x0020,   /* U+11C88 */
  0x0020,   /* U+11C89 */
  0x0020,   /* U+11C8A */
  0x0020,   /* U+11C8B */
  0x0020,   /* U+11C8C */
  0x0020,   /* U+11C8D */
  0x0020,   /* U+11C8E */
  0x0020,   /* U+11C8F */
  0x0020,   /* U+11C90 */
  0x0020,   /* U+11C91 */
  0x0020,   /* U+11C92 */
  0x0020,   /* U+11C93 */
  0x0020,   /* U+11C94 */
  0x0020,   /* U+11C95 */
  0x0020,   /* U+11C96 */
  0x0020,   /* U+11C97 */
  0x0020,   /* U+11C98 */
  0x0020,   /* U+11C99 */
  0x0020,   /* U+11C9A */
  0x0020,   /* U+11C9B */
  0x0020,   /* U+11C9C */
  0x0020,   /* U+11C9D */
  0x0020,   /* U+11C9E */
  0x0020,   /* U+11C9F */
  0x0020,   /* U+11CA0 */
  0x0020,   /* U+11CA1 */
  0x0020,   /* U+11CA2 */
  0x0020,   /* U+11CA3 */
  0x0020,   /* U+11CA4 */
  0x0020,   /* U+11CA5 */
  0x0020,   /* U+11CA6 */
  0x0020,   /* U+11CA7 */
  0x0020,   /* U+11CA8 */
  0x0020,   /* U+11CA9 */
  0x0020,   /* U+11CAA */
  0x0020,   /* U+11CAB */
  0x0020,   /* U+11CAC */
  0x0020,   /* U+11CAD */
  0x0020,   /* U+11CAE */
  0x0020,   /* U+11CAF */
  0x0020,   /* U+11CB0 */
  0x0020,   /* U+11CB1 */
  0x0020,   /* U+11CB2 */
  0x0020,   /* U+11CB3 */
  0x0020,   /* U+11CB4 */
  0x00C4,   /* U+11CB5 */
  0x00C3,   /* U+11CB6 */
  0x0020,   /* U+11CB7 */
  0x0020,   /* U+11CB8 */
  0x0020,   /* U+11CB9 */
  0x0020,   /* U+11CBA */
  0x0020,   /* U+11CBB */
  0x0020,   /* U+11CBC */
  0x0020,   /* U+11CBD */
  0x0020,   /* U+11CBE */
  0x0020,   /* U+11CBF */
  0x0020,   /* U+11CC0 */
  0x0020,   /* U+11CC1 */
  0x0020,   /* U+11CC2 */
  0x0020,   /* U+11CC3 */
  0x0020,   /* U+11CC4 */
  0x0020,   /* U+11CC5 */
  0x0020,   /* U+11CC6 */
  0x0020,   /* U+11CC7 */
  0x0020,   /* U+11CC8 */
  0x0020,   /* U+11CC9 */
  0x0020,   /* U+11CCA */
  0x0020,   /* U+11CCB */
  0x0020,   /* U+11CCC */
  0x0020,   /* U+11CCD */
  0x0020,   /* U+11CCE */
  0x0020,   /* U+11CCF */
  0x0020,   /* U+11CD0 */
  0x0020,   /* U+11CD1 */
  0x0020,   /* U+11CD2 */
  0x0020,   /* U+11CD3 */
  0x0020,   /* U+11CD4 */
  0x0020,   /* U+11CD5 */
  0x0020,   /* U+11CD6 */
  0x0020,   /* U+11CD7 */
  0x0020,   /* U+11CD8 */
  0x0020,   /* U+11CD9 */
  0x0020,   /* U+11CDA */
  0x0020,   /* U+11CDB */
  0x0020,   /* U+11CDC */
  0x0020,   /* U+11CDD */
  0x0020,   /* U+11CDE */
  0x0020,   /* U+11CDF */
  0x0020,   /* U+11CE0 */
  0x0020,   /* U+11CE1 */
  0x0020,   /* U+11CE2 */
  0x0020,   /* U+11CE3 */
  0x0020,   /* U+11CE4 */
  0x0020,   /* U+11CE5 */
  0x0020,   /* U+11CE6 */
  0x0020,   /* U+11CE7 */
  0x0020,   /* U+11CE8 */
  0x0020,   /* U+11CE9 */
  0x0020,   /* U+11CEA */
  0x0020,   /* U+11CEB */
  0x0020,   /* U+11CEC */
  0x0020,   /* U+11CED */
  0x0020,   /* U+11CEE */
  0x0020,   /* U+11CEF */
  0x0020,   /* U+11CF0 */
  0x0020,   /* U+11CF1 */
  0x0020,   /* U+11CF2 */
  0x0020,   /* U+11CF3 */
  0x0020,   /* U+11CF4 */
  0x0020,   /* U+11CF5 */
  0x0020,   /* U+11CF6 */
  0x0020,   /* U+11CF7 */
  0x0020,   /* U+11CF8 */
  0x0020,   /* U+11CF9 */
  0x0020,   /* U+11CFA */
  0x0020,   /* U+11CFB */
  0x0020,   /* U+11CFC */
  0x0020,   /* U+11CFD */
  0x0020,   /* U+11CFE */
  0x0020,   /* U+11CFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+11C00 */
  0x0002,   /* U+11C01 */
  0x0002,   /* U+11C02 */
  0x0002,   /* U+11C03 */
  0x0002,   /* U+11C04 */
  0x0002,   /* U+11C05 */
  0x0002,   /* U+11C06 */
  0x0002,   /* U+11C07 */
  0x0002,   /* U+11C08 */
  0x0002,   /* U+11C09 */
  0x0002,   /* U+11C0A */
  0x0002,   /* U+11C0B */
  0x0002,   /* U+11C0C */
  0x0002,   /* U+11C0D */
  0x0002,   /* U+11C0E */
  0x0002,   /* U+11C0F */
  0x0002,   /* U+11C10 */
  0x0002,   /* U+11C11 */
  0x0002,   /* U+11C12 */
  0x0002,   /* U+11C13 */
  0x0002,   /* U+11C14 */
  0x0002,   /* U+11C15 */
  0x0002,   /* U+11C16 */
  0x0002,   /* U+11C17 */
  0x0002,   /* U+11C18 */
  0x0002,   /* U+11C19 */
  0x0002,   /* U+11C1A */
  0x0002,   /* U+11C1B */
  0x0002,   /* U+11C1C */
  0x0002,   /* U+11C1D */
  0x0002,   /* U+11C1E */
  0x0002,   /* U+11C1F */
  0x0002,   /* U+11C20 */
  0x0002,   /* U+11C21 */
  0x0002,   /* U+11C22 */
  0x0002,   /* U+11C23 */
  0x0002,   /* U+11C24 */
  0x0002,   /* U+11C25 */
  0x0002,   /* U+11C26 */
  0x0002,   /* U+11C27 */
  0x0002,   /* U+11C28 */
  0x0002,   /* U+11C29 */
  0x0002,   /* U+11C2A */
  0x0002,   /* U+11C2B */
  0x0002,   /* U+11C2C */
  0x0002,   /* U+11C2D */
  0x0002,   /* U+11C2E */
  0x0002,   /* U+11C2F */
  0x0002,   /* U+11C30 */
  0x0002,   /* U+11C31 */
  0x0002,   /* U+11C32 */
  0x0002,   /* U+11C33 */
  0x0002,   /* U+11C34 */
  0x0002,   /* U+11C35 */
  0x0002,   /* U+11C36 */
  0x0002,   /* U+11C37 */
  0x0002,   /* U+11C38 */
  0x0002,   /* U+11C39 */
  0x0002,   /* U+11C3A */
  0x0002,   /* U+11C3B */
  0x0002,   /* U+11C3C */
  0x0002,   /* U+11C3D */
  0x0002,   /* U+11C3E */
  0x0002,   /* U+11C3F */
  0x0002,   /* U+11C40 */
  0x0002,   /* U+11C41 */
  0x0002,   /* U+11C42 */
  0x0002,   /* U+11C43 */
  0x0002,   /* U+11C44 */
  0x0002,   /* U+11C45 */
  0x0002,   /* U+11C46 */
  0x0002,   /* U+11C47 */
  0x0002,   /* U+11C48 */
  0x0002,   /* U+11C49 */
  0x0002,   /* U+11C4A */
  0x0002,   /* U+11C4B */
  0x0002,   /* U+11C4C */
  0x0002,   /* U+11C4D */
  0x0002,   /* U+11C4E */
  0x0002,   /* U+11C4F */
  0x0002,   /* U+11C50 */
  0x0002,   /* U+11C51 */
  0x0002,   /* U+11C52 */
  0x0002,   /* U+11C53 */
  0x0002,   /* U+11C54 */
  0x0002,   /* U+11C55 */
  0x0002,   /* U+11C56 */
  0x0002,   /* U+11C57 */
  0x0002,   /* U+11C58 */
  0x0002,   /* U+11C59 */
  0x0002,   /* U+11C5A */
  0x0002,   /* U+11C5B */
  0x0002,   /* U+11C5C */
  0x0002,   /* U+11C5D */
  0x0002,   /* U+11C5E */
  0x0002,   /* U+11C5F */
  0x0002,   /* U+11C60 */
  0x0002,   /* U+11C61 */
  0x0002,   /* U+11C62 */
  0x0002,   /* U+11C63 */
  0x0002,   /* U+11C64 */
  0x0002,   /* U+11C65 */
  0x0002,   /* U+11C66 */
  0x0002,   /* U+11C67 */
  0x0002,   /* U+11C68 */
  0x0002,   /* U+11C69 */
  0x0002,   /* U+11C6A */
  0x0002,   /* U+11C6B */
  0x0002,   /* U+11C6C */
  0x0002,   /* U+11C6D */
  0x0002,   /* U+11C6E */
  0x0002,   /* U+11C6F */
  0x0002,   /* U+11C70 */
  0x0002,   /* U+11C71 */
  0x0002,   /* U+11C72 */
  0x0002,   /* U+11C73 */
  0x0002,   /* U+11C74 */
  0x0002,   /* U+11C75 */
  0x0002,   /* U+11C76 */
  0x0002,   /* U+11C77 */
  0x0002,   /* U+11C78 */
  0x0002,   /* U+11C79 */
  0x0002,   /* U+11C7A */
  0x0002,   /* U+11C7B */
  0x0002,   /* U+11C7C */
  0x0002,   /* U+11C7D */
  0x0002,   /* U+11C7E */
  0x0002,   /* U+11C7F */
  0x0002,   /* U+11C80 */
  0x0002,   /* U+11C81 */
  0x0002,   /* U+11C82 */
  0x0002,   /* U+11C83 */
  0x0002,   /* U+11C84 */
  0x0002,   /* U+11C85 */
  0x0002,   /* U+11C86 */
  0x0002,   /* U+11C87 */
  0x0002,   /* U+11C88 */
  0x0002,   /* U+11C89 */
  0x0002,   /* U+11C8A */
  0x0002,   /* U+11C8B */
  0x0002,   /* U+11C8C */
  0x0002,   /* U+11C8D */
  0x0002,   /* U+11C8E */
  0x0002,   /* U+11C8F */
  0x0002,   /* U+11C90 */
  0x0002,   /* U+11C91 */
  0x0002,   /* U+11C92 */
  0x0002,   /* U+11C93 */
  0x0002,   /* U+11C94 */
  0x0002,   /* U+11C95 */
  0x0002,   /* U+11C96 */
  0x0002,   /* U+11C97 */
  0x0002,   /* U+11C98 */
  0x0002,   /* U+11C99 */
  0x0002,   /* U+11C9A */
  0x0002,   /* U+11C9B */
  0x0002,   /* U+11C9C */
  0x0002,   /* U+11C9D */
  0x0002,   /* U+11C9E */
  0x0002,   /* U+11C9F */
  0x0002,   /* U+11CA0 */
  0x0002,   /* U+11CA1 */
  0x0002,   /* U+11CA2 */
  0x0002,   /* U+11CA3 */
  0x0002,   /* U+11CA4 */
  0x0002,   /* U+11CA5 */
  0x0002,   /* U+11CA6 */
  0x0002,   /* U+11CA7 */
  0x0002,   /* U+11CA8 */
  0x0002,   /* U+11CA9 */
  0x0002,   /* U+11CAA */
  0x0002,   /* U+11CAB */
  0x0002,   /* U+11CAC */
  0x0002,   /* U+11CAD */
  0x0002,   /* U+11CAE */
  0x0002,   /* U+11CAF */
  0x0002,   /* U+11CB0 */
  0x0002,   /* U+11CB1 */
  0x0002,   /* U+11CB2 */
  0x0002,   /* U+11CB3 */
  0x0002,   /* U+11CB4 */
  0x0002,   /* U+11CB5 */
  0x0002,   /* U+11CB6 */
  0x0002,   /* U+11CB7 */
  0x0002,   /* U+11CB8 */
  0x0002,   /* U+11CB9 */
  0x0002,   /* U+11CBA */
  0x0002,   /* U+11CBB */
  0x0002,   /* U+11CBC */
  0x0002,   /* U+11CBD */
  0x0002,   /* U+11CBE */
  0x0002,   /* U+11CBF */
  0x0002,   /* U+11CC0 */
  0x0002,   /* U+11CC1 */
  0x0002,   /* U+11CC2 */
  0x0002,   /* U+11CC3 */
  0x0002,   /* U+11CC4 */
  0x0002,   /* U+11CC5 */
  0x0002,   /* U+11CC6 */
  0x0002,   /* U+11CC7 */
  0x0002,   /* U+11CC8 */
  0x0002,   /* U+11CC9 */
  0x0002,   /* U+11CCA */
  0x0002,   /* U+11CCB */
  0x0002,   /* U+11CCC */
  0x0002,   /* U+11CCD */
  0x0002,   /* U+11CCE */
  0x0002,   /* U+11CCF */
  0x0002,   /* U+11CD0 */
  0x0002,   /* U+11CD1 */
  0x0002,   /* U+11CD2 */
  0x0002,   /* U+11CD3 */
  0x0002,   /* U+11CD4 */
  0x0002,   /* U+11CD5 */
  0x0002,   /* U+11CD6 */
  0x0002,   /* U+11CD7 */
  0x0002,   /* U+11CD8 */
  0x0002,   /* U+11CD9 */
  0x0002,   /* U+11CDA */
  0x0002,   /* U+11CDB */
  0x0002,   /* U+11CDC */
  0x0002,   /* U+11CDD */
  0x0002,   /* U+11CDE */
  0x0002,   /* U+11CDF */
  0x0002,   /* U+11CE0 */
  0x0002,   /* U+11CE1 */
  0x0002,   /* U+11CE2 */
  0x0002,   /* U+11CE3 */
  0x0002,   /* U+11CE4 */
  0x0002,   /* U+11CE5 */
  0x0002,   /* U+11CE6 */
  0x0002,   /* U+11CE7 */
  0x0002,   /* U+11CE8 */
  0x0002,   /* U+11CE9 */
  0x0002,   /* U+11CEA */
  0x0002,   /* U+11CEB */
  0x0002,   /* U+11CEC */
  0x0002,   /* U+11CED */
  0x0002,   /* U+11CEE */
  0x0002,   /* U+11CEF */
  0x0002,   /* U+11CF0 */
  0x0002,   /* U+11CF1 */
  0x0002,   /* U+11CF2 */
  0x0002,   /* U+11CF3 */
  0x0002,   /* U+11CF4 */
  0x0002,   /* U+11CF5 */
  0x0002,   /* U+11CF6 */
  0x0002,   /* U+11CF7 */
  0x0002,   /* U+11CF8 */
  0x0002,   /* U+11CF9 */
  0x0002,   /* U+11CFA */
  0x0002,   /* U+11CFB */
  0x0002,   /* U+11CFC */
  0x0002,   /* U+11CFD */
  0x0002,   /* U+11CFE */
  0x0002,   /* U+11CFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+11C00 */
  0x0000,   /* U+11C01 */
  0x0000,   /* U+11C02 */
  0x0000,   /* U+11C03 */
  0x0000,   /* U+11C04 */
  0x0000,   /* U+11C05 */
  0x0000,   /* U+11C06 */
  0x0000,   /* U+11C07 */
  0x0000,   /* U+11C08 */
  0x9C09,   /* U+11C09 */
  0x0000,   /* U+11C0A */
  0x0000,   /* U+11C0B */
  0x0000,   /* U+11C0C */
  0x0000,   /* U+11C0D */
  0x0000,   /* U+11C0E */
  0x0000,   /* U+11C0F */
  0x0000,   /* U+11C10 */
  0x0000,   /* U+11C11 */
  0x0000,   /* U+11C12 */
  0x0000,   /* U+11C13 */
  0x0000,   /* U+11C14 */
  0x0000,   /* U+11C15 */
  0x0000,   /* U+11C16 */
  0x0000,   /* U+11C17 */
  0x0000,   /* U+11C18 */
  0x0000,   /* U+11C19 */
  0x0000,   /* U+11C1A */
  0x0000,   /* U+11C1B */
  0x0000,   /* U+11C1C */
  0x0000,   /* U+11C1D */
  0x0000,   /* U+11C1E */
  0x0000,   /* U+11C1F */
  0x0000,   /* U+11C20 */
  0x0000,   /* U+11C21 */
  0x0000,   /* U+11C22 */
  0x0000,   /* U+11C23 */
  0x0000,   /* U+11C24 */
  0x0000,   /* U+11C25 */
  0x0000,   /* U+11C26 */
  0x0000,   /* U+11C27 */
  0x0000,   /* U+11C28 */
  0x0000,   /* U+11C29 */
  0x0000,   /* U+11C2A */
  0x0000,   /* U+11C2B */
  0x0000,   /* U+11C2C */
  0x0000,   /* U+11C2D */
  0x0000,   /* U+11C2E */
  0x0000,   /* U+11C2F */
  0x0000,   /* U+11C30 */
  0x0000,   /* U+11C31 */
  0x0000,   /* U+11C32 */
  0x0000,   /* U+11C33 */
  0x0000,   /* U+11C34 */
  0x0000,   /* U+11C35 */
  0x0000,   /* U+11C36 */
  0x9C37,   /* U+11C37 */
  0x0000,   /* U+11C38 */
  0x0000,   /* U+11C39 */
  0x0000,   /* U+11C3A */
  0x0000,   /* U+11C3B */
  0x0000,   /* U+11C3C */
  0x0000,   /* U+11C3D */
  0x0000,   /* U+11C3E */
  0x0000,   /* U+11C3F */
  0x0000,   /* U+11C40 */
  0x0000,   /* U+11C41 */
  0x0000,   /* U+11C42 */
  0x0000,   /* U+11C43 */
  0x0000,   /* U+11C44 */
  0x0000,   /* U+11C45 */
  0x9C46,   /* U+11C46 */
  0x9C47,   /* U+11C47 */
  0x9C48,   /* U+11C48 */
  0x9C49,   /* U+11C49 */
  0x9C4A,   /* U+11C4A */
  0x9C4B,   /* U+11C4B */
  0x9C4C,   /* U+11C4C */
  0x9C4D,   /* U+11C4D */
  0x9C4E,   /* U+11C4E */
  0x9C4F,   /* U+11C4F */
  0x0000,   /* U+11C50 */
  0x0000,   /* U+11C51 */
  0x0000,   /* U+11C52 */
  0x0000,   /* U+11C53 */
  0x0000,   /* U+11C54 */
  0x0000,   /* U+11C55 */
  0x0000,   /* U+11C56 */
  0x0000,   /* U+11C57 */
  0x0000,   /* U+11C58 */
  0x0000,   /* U+11C59 */
  0x0000,   /* U+11C5A */
  0x0000,   /* U+11C5B */
  0x0000,   /* U+11C5C */
  0x0000,   /* U+11C5D */
  0x0000,   /* U+11C5E */
  0x0000,   /* U+11C5F */
  0x0000,   /* U+11C60 */
  0x0000,   /* U+11C61 */
  0x0000,   /* U+11C62 */
  0x0000,   /* U+11C63 */
  0x0000,   /* U+11C64 */
  0x0000,   /* U+11C65 */
  0x0000,   /* U+11C66 */
  0x0000,   /* U+11C67 */
  0x0000,   /* U+11C68 */
  0x0000,   /* U+11C69 */
  0x0000,   /* U+11C6A */
  0x0000,   /* U+11C6B */
  0x0000,   /* U+11C6C */
  0x9C6D,   /* U+11C6D */
  0x9C6E,   /* U+11C6E */
  0x9C6F,   /* U+11C6F */
  0x0000,   /* U+11C70 */
  0x0000,   /* U+11C71 */
  0x0000,   /* U+11C72 */
  0x0000,   /* U+11C73 */
  0x0000,   /* U+11C74 */
  0x0000,   /* U+11C75 */
  0x0000,   /* U+11C76 */
  0x0000,   /* U+11C77 */
  0x0000,   /* U+11C78 */
  0x0000,   /* U+11C79 */
  0x0000,   /* U+11C7A */
  0x0000,   /* U+11C7B */
  0x0000,   /* U+11C7C */
  0x0000,   /* U+11C7D */
  0x0000,   /* U+11C7E */
  0x0000,   /* U+11C7F */
  0x0000,   /* U+11C80 */
  0x0000,   /* U+11C81 */
  0x0000,   /* U+11C82 */
  0x0000,   /* U+11C83 */
  0x0000,   /* U+11C84 */
  0x0000,   /* U+11C85 */
  0x0000,   /* U+11C86 */
  0x0000,   /* U+11C87 */
  0x0000,   /* U+11C88 */
  0x0000,   /* U+11C89 */
  0x0000,   /* U+11C8A */
  0x0000,   /* U+11C8B */
  0x0000,   /* U+11C8C */
  0x0000,   /* U+11C8D */
  0x0000,   /* U+11C8E */
  0x0000,   /* U+11C8F */
  0x9C90,   /* U+11C90 */
  0x9C91,   /* U+11C91 */
  0x0000,   /* U+11C92 */
  0x0000,   /* U+11C93 */
  0x0000,   /* U+11C94 */
  0x0000,   /* U+11C95 */
  0x0000,   /* U+11C96 */
  0x0000,   /* U+11C97 */
  0x0000,   /* U+11C98 */
  0x0000,   /* U+11C99 */
  0x0000,   /* U+11C9A */
  0x0000,   /* U+11C9B */
  0x0000,   /* U+11C9C */
  0x0000,   /* U+11C9D */
  0x0000,   /* U+11C9E */
  0x0000,   /* U+11C9F */
  0x0000,   /* U+11CA0 */
  0x0000,   /* U+11CA1 */
  0x0000,   /* U+11CA2 */
  0x0000,   /* U+11CA3 */
  0x0000,   /* U+11CA4 */
  0x0000,   /* U+11CA5 */
  0x0000,   /* U+11CA6 */
  0x0000,   /* U+11CA7 */
  0x9CA8,   /* U+11CA8 */
  0x0000,   /* U+11CA9 */
  0x0000,   /* U+11CAA */
  0x0000,   /* U+11CAB */
  0x0000,   /* U+11CAC */
  0x0000,   /* U+11CAD */
  0x0000,   /* U+11CAE */
  0x0000,   /* U+11CAF */
  0x0000,   /* U+11CB0 */
  0x0000,   /* U+11CB1 */
  0x0000,   /* U+11CB2 */
  0x0000,   /* U+11CB3 */
  0x0000,   /* U+11CB4 */
  0x0000,   /* U+11CB5 */
  0x0000,   /* U+11CB6 */
  0x9CB7,   /* U+11CB7 */
  0x9CB8,   /* U+11CB8 */
  0x9CB9,   /* U+11CB9 */
  0x9CBA,   /* U+11CBA */
  0x9CBB,   /* U+11CBB */
  0x9CBC,   /* U+11CBC */
  0x9CBD,   /* U+11CBD */
  0x9CBE,   /* U+11CBE */
  0x9CBF,   /* U+11CBF */
  0x9CC0,   /* U+11CC0 */
  0x9CC1,   /* U+11CC1 */
  0x9CC2,   /* U+11CC2 */
  0x9CC3,   /* U+11CC3 */
  0x9CC4,   /* U+11CC4 */
  0x9CC5,   /* U+11CC5 */
  0x9CC6,   /* U+11CC6 */
  0x9CC7,   /* U+11CC7 */
  0x9CC8,   /* U+11CC8 */
  0x9CC9,   /* U+11CC9 */
  0x9CCA,   /* U+11CCA */
  0x9CCB,   /* U+11CCB */
  0x9CCC,   /* U+11CCC */
  0x9CCD,   /* U+11CCD */
  0x9CCE,   /* U+11CCE */
  0x9CCF,   /* U+11CCF */
  0x9CD0,   /* U+11CD0 */
  0x9CD1,   /* U+11CD1 */
  0x9CD2,   /* U+11CD2 */
  0x9CD3,   /* U+11CD3 */
  0x9CD4,   /* U+11CD4 */
  0x9CD5,   /* U+11CD5 */
  0x9CD6,   /* U+11CD6 */
  0x9CD7,   /* U+11CD7 */
  0x9CD8,   /* U+11CD8 */
  0x9CD9,   /* U+11CD9 */
  0x9CDA,   /* U+11CDA */
  0x9CDB,   /* U+11CDB */
  0x9CDC,   /* U+11CDC */
  0x9CDD,   /* U+11CDD */
  0x9CDE,   /* U+11CDE */
  0x9CDF,   /* U+11CDF */
  0x9CE0,   /* U+11CE0 */
  0x9CE1,   /* U+11CE1 */
  0x9CE2,   /* U+11CE2 */
  0x9CE3,   /* U+11CE3 */
  0x9CE4,   /* U+11CE4 */
  0x9CE5,   /* U+11CE5 */
  0x9CE6,   /* U+11CE6 */
  0x9CE7,   /* U+11CE7 */
  0x9CE8,   /* U+11CE8 */
  0x9CE9,   /* U+11CE9 */
  0x9CEA,   /* U+11CEA */
  0x9CEB,   /* U+11CEB */
  0x9CEC,   /* U+11CEC */
  0x9CED,   /* U+11CED */
  0x9CEE,   /* U+11CEE */
  0x9CEF,   /* U+11CEF */
  0x9CF0,   /* U+11CF0 */
  0x9CF1,   /* U+11CF1 */
  0x9CF2,   /* U+11CF2 */
  0x9CF3,   /* U+11CF3 */
  0x9CF4,   /* U+11CF4 */
  0x9CF5,   /* U+11CF5 */
  0x9CF6,   /* U+11CF6 */
  0x9CF7,   /* U+11CF7 */
  0x9CF8,   /* U+11CF8 */
  0x9CF9,   /* U+11CF9 */
  0x9CFA,   /* U+11CFA */
  0x9CFB,   /* U+11CFB */
  0x9CFC,   /* U+11CFC */
  0x9CFD,   /* U+11CFD */
  0x9CFE,   /* U+11CFE */
  0x9CFF,   /* U+11CFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+11C00 */
  0x0000,   /* U+11C01 */
  0x0000,   /* U+11C02 */
  0x0000,   /* U+11C03 */
  0x0000,   /* U+11C04 */
  0x0000,   /* U+11C05 */
  0x0000,   /* U+11C06 */
  0x0000,   /* U+11C07 */
  0x0000,   /* U+11C08 */
  0x0000,   /* U+11C09 */
  0x0000,   /* U+11C0A */
  0x0000,   /* U+11C0B */
  0x0000,   /* U+11C0C */
  0x0000,   /* U+11C0D */
  0x0000,   /* U+11C0E */
  0x0000,   /* U+11C0F */
  0x0000,   /* U+11C10 */
  0x0000,   /* U+11C11 */
  0x0000,   /* U+11C12 */
  0x0000,   /* U+11C13 */
  0x0000,   /* U+11C14 */
  0x0000,   /* U+11C15 */
  0x0000,   /* U+11C16 */
  0x0000,   /* U+11C17 */
  0x0000,   /* U+11C18 */
  0x0000,   /* U+11C19 */
  0x0000,   /* U+11C1A */
  0x0000,   /* U+11C1B */
  0x0000,   /* U+11C1C */
  0x0000,   /* U+11C1D */
  0x0000,   /* U+11C1E */
  0x0000,   /* U+11C1F */
  0x0000,   /* U+11C20 */
  0x0000,   /* U+11C21 */
  0x0000,   /* U+11C22 */
  0x0000,   /* U+11C23 */
  0x0000,   /* U+11C24 */
  0x0000,   /* U+11C25 */
  0x0000,   /* U+11C26 */
  0x0000,   /* U+11C27 */
  0x0000,   /* U+11C28 */
  0x0000,   /* U+11C29 */
  0x0000,   /* U+11C2A */
  0x0000,   /* U+11C2B */
  0x0000,   /* U+11C2C */
  0x0000,   /* U+11C2D */
  0x0000,   /* U+11C2E */
  0x0000,   /* U+11C2F */
  0x0000,   /* U+11C30 */
  0x0000,   /* U+11C31 */
  0x0000,   /* U+11C32 */
  0x0000,   /* U+11C33 */
  0x0000,   /* U+11C34 */
  0x0000,   /* U+11C35 */
  0x0000,   /* U+11C36 */
  0x0000,   /* U+11C37 */
  0x0000,   /* U+11C38 */
  0x0000,   /* U+11C39 */
  0x0000,   /* U+11C3A */
  0x0000,   /* U+11C3B */
  0x0000,   /* U+11C3C */
  0x0000,   /* U+11C3D */
  0x0000,   /* U+11C3E */
  0x0000,   /* U+11C3F */
  0x0000,   /* U+11C40 */
  0x0000,   /* U+11C41 */
  0x0000,   /* U+11C42 */
  0x0000,   /* U+11C43 */
  0x0000,   /* U+11C44 */
  0x0000,   /* U+11C45 */
  0x0000,   /* U+11C46 */
  0x0000,   /* U+11C47 */
  0x0000,   /* U+11C48 */
  0x0000,   /* U+11C49 */
  0x0000,   /* U+11C4A */
  0x0000,   /* U+11C4B */
  0x0000,   /* U+11C4C */
  0x0000,   /* U+11C4D */
  0x0000,   /* U+11C4E */
  0x0000,   /* U+11C4F */
  0x0000,   /* U+11C50 */
  0x0000,   /* U+11C51 */
  0x0000,   /* U+11C52 */
  0x0000,   /* U+11C53 */
  0x0000,   /* U+11C54 */
  0x0000,   /* U+11C55 */
  0x0000,   /* U+11C56 */
  0x0000,   /* U+11C57 */
  0x0000,   /* U+11C58 */
  0x0000,   /* U+11C59 */
  0x0000,   /* U+11C5A */
  0x0000,   /* U+11C5B */
  0x0000,   /* U+11C5C */
  0x0000,   /* U+11C5D */
  0x0000,   /* U+11C5E */
  0x0000,   /* U+11C5F */
  0x0000,   /* U+11C60 */
  0x0000,   /* U+11C61 */
  0x0000,   /* U+11C62 */
  0x0000,   /* U+11C63 */
  0x0000,   /* U+11C64 */
  0x0000,   /* U+11C65 */
  0x0000,   /* U+11C66 */
  0x0000,   /* U+11C67 */
  0x0000,   /* U+11C68 */
  0x0000,   /* U+11C69 */
  0x0000,   /* U+11C6A */
  0x0000,   /* U+11C6B */
  0x0000,   /* U+11C6C */
  0x0000,   /* U+11C6D */
  0x0000,   /* U+11C6E */
  0x0000,   /* U+11C6F */
  0x0000,   /* U+11C70 */
  0x0000,   /* U+11C71 */
  0x0000,   /* U+11C72 */
  0x0000,   /* U+11C73 */
  0x0000,   /* U+11C74 */
  0x0000,   /* U+11C75 */
  0x0000,   /* U+11C76 */
  0x0000,   /* U+11C77 */
  0x0000,   /* U+11C78 */
  0x0000,   /* U+11C79 */
  0x0000,   /* U+11C7A */
  0x0000,   /* U+11C7B */
  0x0000,   /* U+11C7C */
  0x0000,   /* U+11C7D */
  0x0000,   /* U+11C7E */
  0x0000,   /* U+11C7F */
  0x0000,   /* U+11C80 */
  0x0000,   /* U+11C81 */
  0x0000,   /* U+11C82 */
  0x0000,   /* U+11C83 */
  0x0000,   /* U+11C84 */
  0x0000,   /* U+11C85 */
  0x0000,   /* U+11C86 */
  0x0000,   /* U+11C87 */
  0x0000,   /* U+11C88 */
  0x0000,   /* U+11C89 */
  0x0000,   /* U+11C8A */
  0x0000,   /* U+11C8B */
  0x0000,   /* U+11C8C */
  0x0000,   /* U+11C8D */
  0x0000,   /* U+11C8E */
  0x0000,   /* U+11C8F */
  0x0000,   /* U+11C90 */
  0x0000,   /* U+11C91 */
  0x0000,   /* U+11C92 */
  0x0000,   /* U+11C93 */
  0x0000,   /* U+11C94 */
  0x0000,   /* U+11C95 */
  0x0000,   /* U+11C96 */
  0x0000,   /* U+11C97 */
  0x0000,   /* U+11C98 */
  0x0000,   /* U+11C99 */
  0x0000,   /* U+11C9A */
  0x0000,   /* U+11C9B */
  0x0000,   /* U+11C9C */
  0x0000,   /* U+11C9D */
  0x0000,   /* U+11C9E */
  0x0000,   /* U+11C9F */
  0x0000,   /* U+11CA0 */
  0x0000,   /* U+11CA1 */
  0x0000,   /* U+11CA2 */
  0x0000,   /* U+11CA3 */
  0x0000,   /* U+11CA4 */
  0x0000,   /* U+11CA5 */
  0x0000,   /* U+11CA6 */
  0x0000,   /* U+11CA7 */
  0x0000,   /* U+11CA8 */
  0x0000,   /* U+11CA9 */
  0x0000,   /* U+11CAA */
  0x0000,   /* U+11CAB */
  0x0000,   /* U+11CAC */
  0x0000,   /* U+11CAD */
  0x0000,   /* U+11CAE */
  0x0000,   /* U+11CAF */
  0x0000,   /* U+11CB0 */
  0x0000,   /* U+11CB1 */
  0x0000,   /* U+11CB2 */
  0x0000,   /* U+11CB3 */
  0x0000,   /* U+11CB4 */
  0x0000,   /* U+11CB5 */
  0x0000,   /* U+11CB6 */
  0x0000,   /* U+11CB7 */
  0x0000,   /* U+11CB8 */
  0x0000,   /* U+11CB9 */
  0x0000,   /* U+11CBA */
  0x0000,   /* U+11CBB */
  0x0000,   /* U+11CBC */
  0x0000,   /* U+11CBD */
  0x0000,   /* U+11CBE */
  0x0000,   /* U+11CBF */
  0x0000,   /* U+11CC0 */
  0x0000,   /* U+11CC1 */
  0x0000,   /* U+11CC2 */
  0x0000,   /* U+11CC3 */
  0x0000,   /* U+11CC4 */
  0x0000,   /* U+11CC5 */
  0x0000,   /* U+11CC6 */
  0x0000,   /* U+11CC7 */
  0x0000,   /* U+11CC8 */
  0x0000,   /* U+11CC9 */
  0x0000,   /* U+11CCA */
  0x0000,   /* U+11CCB */
  0x0000,   /* U+11CCC */
  0x0000,   /* U+11CCD */
  0x0000,   /* U+11CCE */
  0x0000,   /* U+11CCF */
  0x0000,   /* U+11CD0 */
  0x0000,   /* U+11CD1 */
  0x0000,   /* U+11CD2 */
  0x0000,   /* U+11CD3 */
  0x0000,   /* U+11CD4 */
  0x0000,   /* U+11CD5 */
  0x0000,   /* U+11CD6 */
  0x0000,   /* U+11CD7 */
  0x0000,   /* U+11CD8 */
  0x0000,   /* U+11CD9 */
  0x0000,   /* U+11CDA */
  0x0000,   /* U+11CDB */
  0x0000,   /* U+11CDC */
  0x0000,   /* U+11CDD */
  0x0000,   /* U+11CDE */
  0x0000,   /* U+11CDF */
  0x0000,   /* U+11CE0 */
  0x0000,   /* U+11CE1 */
  0x0000,   /* U+11CE2 */
  0x0000,   /* U+11CE3 */
  0x0000,   /* U+11CE4 */
  0x0000,   /* U+11CE5 */
  0x0000,   /* U+11CE6 */
  0x0000,   /* U+11CE7 */
  0x0000,   /* U+11CE8 */
  0x0000,   /* U+11CE9 */
  0x0000,   /* U+11CEA */
  0x0000,   /* U+11CEB */
  0x0000,   /* U+11CEC */
  0x0000,   /* U+11CED */
  0x0000,   /* U+11CEE */
  0x0000,   /* U+11CEF */
  0x0000,   /* U+11CF0 */
  0x0000,   /* U+11CF1 */
  0x0000,   /* U+11CF2 */
  0x0000,   /* U+11CF3 */
  0x0000,   /* U+11CF4 */
  0x0000,   /* U+11CF5 */
  0x0000,   /* U+11CF6 */
  0x0000,   /* U+11CF7 */
  0x0000,   /* U+11CF8 */
  0x0000,   /* U+11CF9 */
  0x0000,   /* U+11CFA */
  0x0000,   /* U+11CFB */
  0x0000,   /* U+11CFC */
  0x0000,   /* U+11CFD */
  0x0000,   /* U+11CFE */
  0x0000,   /* U+11CFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+11C00 */
  0x0000,   /* U+11C01 */
  0x0000,   /* U+11C02 */
  0x0000,   /* U+11C03 */
  0x0000,   /* U+11C04 */
  0x0000,   /* U+11C05 */
  0x0000,   /* U+11C06 */
  0x0000,   /* U+11C07 */
  0x0000,   /* U+11C08 */
  0x0000,   /* U+11C09 */
  0x0000,   /* U+11C0A */
  0x0000,   /* U+11C0B */
  0x0000,   /* U+11C0C */
  0x0000,   /* U+11C0D */
  0x0000,   /* U+11C0E */
  0x0000,   /* U+11C0F */
  0x0000,   /* U+11C10 */
  0x0000,   /* U+11C11 */
  0x0000,   /* U+11C12 */
  0x0000,   /* U+11C13 */
  0x0000,   /* U+11C14 */
  0x0000,   /* U+11C15 */
  0x0000,   /* U+11C16 */
  0x0000,   /* U+11C17 */
  0x0000,   /* U+11C18 */
  0x0000,   /* U+11C19 */
  0x0000,   /* U+11C1A */
  0x0000,   /* U+11C1B */
  0x0000,   /* U+11C1C */
  0x0000,   /* U+11C1D */
  0x0000,   /* U+11C1E */
  0x0000,   /* U+11C1F */
  0x0000,   /* U+11C20 */
  0x0000,   /* U+11C21 */
  0x0000,   /* U+11C22 */
  0x0000,   /* U+11C23 */
  0x0000,   /* U+11C24 */
  0x0000,   /* U+11C25 */
  0x0000,   /* U+11C26 */
  0x0000,   /* U+11C27 */
  0x0000,   /* U+11C28 */
  0x0000,   /* U+11C29 */
  0x0000,   /* U+11C2A */
  0x0000,   /* U+11C2B */
  0x0000,   /* U+11C2C */
  0x0000,   /* U+11C2D */
  0x0000,   /* U+11C2E */
  0x0000,   /* U+11C2F */
  0x0000,   /* U+11C30 */
  0x0000,   /* U+11C31 */
  0x0000,   /* U+11C32 */
  0x0000,   /* U+11C33 */
  0x0000,   /* U+11C34 */
  0x0000,   /* U+11C35 */
  0x0000,   /* U+11C36 */
  0x0000,   /* U+11C37 */
  0x0000,   /* U+11C38 */
  0x0000,   /* U+11C39 */
  0x0000,   /* U+11C3A */
  0x0000,   /* U+11C3B */
  0x0000,   /* U+11C3C */
  0x0000,   /* U+11C3D */
  0x0000,   /* U+11C3E */
  0x0000,   /* U+11C3F */
  0x0000,   /* U+11C40 */
  0x0000,   /* U+11C41 */
  0x0000,   /* U+11C42 */
  0x0000,   /* U+11C43 */
  0x0000,   /* U+11C44 */
  0x0000,   /* U+11C45 */
  0x0000,   /* U+11C46 */
  0x0000,   /* U+11C47 */
  0x0000,   /* U+11C48 */
  0x0000,   /* U+11C49 */
  0x0000,   /* U+11C4A */
  0x0000,   /* U+11C4B */
  0x0000,   /* U+11C4C */
  0x0000,   /* U+11C4D */
  0x0000,   /* U+11C4E */
  0x0000,   /* U+11C4F */
  0x0000,   /* U+11C50 */
  0x0000,   /* U+11C51 */
  0x0000,   /* U+11C52 */
  0x0000,   /* U+11C53 */
  0x0000,   /* U+11C54 */
  0x0000,   /* U+11C55 */
  0x0000,   /* U+11C56 */
  0x0000,   /* U+11C57 */
  0x0000,   /* U+11C58 */
  0x0000,   /* U+11C59 */
  0x0000,   /* U+11C5A */
  0x0000,   /* U+11C5B */
  0x0000,   /* U+11C5C */
  0x0000,   /* U+11C5D */
  0x0000,   /* U+11C5E */
  0x0000,   /* U+11C5F */
  0x0000,   /* U+11C60 */
  0x0000,   /* U+11C61 */
  0x0000,   /* U+11C62 */
  0x0000,   /* U+11C63 */
  0x0000,   /* U+11C64 */
  0x0000,   /* U+11C65 */
  0x0000,   /* U+11C66 */
  0x0000,   /* U+11C67 */
  0x0000,   /* U+11C68 */
  0x0000,   /* U+11C69 */
  0x0000,   /* U+11C6A */
  0x0000,   /* U+11C6B */
  0x0000,   /* U+11C6C */
  0x0000,   /* U+11C6D */
  0x0000,   /* U+11C6E */
  0x0000,   /* U+11C6F */
  0x0000,   /* U+11C70 */
  0x0000,   /* U+11C71 */
  0x0000,   /* U+11C72 */
  0x0000,   /* U+11C73 */
  0x0000,   /* U+11C74 */
  0x0000,   /* U+11C75 */
  0x0000,   /* U+11C76 */
  0x0000,   /* U+11C77 */
  0x0000,   /* U+11C78 */
  0x0000,   /* U+11C79 */
  0x0000,   /* U+11C7A */
  0x0000,   /* U+11C7B */
  0x0000,   /* U+11C7C */
  0x0000,   /* U+11C7D */
  0x0000,   /* U+11C7E */
  0x0000,   /* U+11C7F */
  0x0000,   /* U+11C80 */
  0x0000,   /* U+11C81 */
  0x0000,   /* U+11C82 */
  0x0000,   /* U+11C83 */
  0x0000,   /* U+11C84 */
  0x0000,   /* U+11C85 */
  0x0000,   /* U+11C86 */
  0x0000,   /* U+11C87 */
  0x0000,   /* U+11C88 */
  0x0000,   /* U+11C89 */
  0x0000,   /* U+11C8A */
  0x0000,   /* U+11C8B */
  0x0000,   /* U+11C8C */
  0x0000,   /* U+11C8D */
  0x0000,   /* U+11C8E */
  0x0000,   /* U+11C8F */
  0x0000,   /* U+11C90 */
  0x0000,   /* U+11C91 */
  0x0000,   /* U+11C92 */
  0x0000,   /* U+11C93 */
  0x0000,   /* U+11C94 */
  0x0000,   /* U+11C95 */
  0x0000,   /* U+11C96 */
  0x0000,   /* U+11C97 */
  0x0000,   /* U+11C98 */
  0x0000,   /* U+11C99 */
  0x0000,   /* U+11C9A */
  0x0000,   /* U+11C9B */
  0x0000,   /* U+11C9C */
  0x0000,   /* U+11C9D */
  0x0000,   /* U+11C9E */
  0x0000,   /* U+11C9F */
  0x0000,   /* U+11CA0 */
  0x0000,   /* U+11CA1 */
  0x0000,   /* U+11CA2 */
  0x0000,   /* U+11CA3 */
  0x0000,   /* U+11CA4 */
  0x0000,   /* U+11CA5 */
  0x0000,   /* U+11CA6 */
  0x0000,   /* U+11CA7 */
  0x0000,   /* U+11CA8 */
  0x0000,   /* U+11CA9 */
  0x0000,   /* U+11CAA */
  0x0000,   /* U+11CAB */
  0x0000,   /* U+11CAC */
  0x0000,   /* U+11CAD */
  0x0000,   /* U+11CAE */
  0x0000,   /* U+11CAF */
  0x0000,   /* U+11CB0 */
  0x0000,   /* U+11CB1 */
  0x0000,   /* U+11CB2 */
  0x0000,   /* U+11CB3 */
  0x0000,   /* U+11CB4 */
  0x0000,   /* U+11CB5 */
  0x0000,   /* U+11CB6 */
  0x0000,   /* U+11CB7 */
  0x0000,   /* U+11CB8 */
  0x0000,   /* U+11CB9 */
  0x0000,   /* U+11CBA */
  0x0000,   /* U+11CBB */
  0x0000,   /* U+11CBC */
  0x0000,   /* U+11CBD */
  0x0000,   /* U+11CBE */
  0x0000,   /* U+11CBF */
  0x0000,   /* U+11CC0 */
  0x0000,   /* U+11CC1 */
  0x0000,   /* U+11CC2 */
  0x0000,   /* U+11CC3 */
  0x0000,   /* U+11CC4 */
  0x0000,   /* U+11CC5 */
  0x0000,   /* U+11CC6 */
  0x0000,   /* U+11CC7 */
  0x0000,   /* U+11CC8 */
  0x0000,   /* U+11CC9 */
  0x0000,   /* U+11CCA */
  0x0000,   /* U+11CCB */
  0x0000,   /* U+11CCC */
  0x0000,   /* U+11CCD */
  0x0000,   /* U+11CCE */
  0x0000,   /* U+11CCF */
  0x0000,   /* U+11CD0 */
  0x0000,   /* U+11CD1 */
  0x0000,   /* U+11CD2 */
  0x0000,   /* U+11CD3 */
  0x0000,   /* U+11CD4 */
  0x0000,   /* U+11CD5 */
  0x0000,   /* U+11CD6 */
  0x0000,   /* U+11CD7 */
  0x0000,   /* U+11CD8 */
  0x0000,   /* U+11CD9 */
  0x0000,   /* U+11CDA */
  0x0000,   /* U+11CDB */
  0x0000,   /* U+11CDC */
  0x0000,   /* U+11CDD */
  0x0000,   /* U+11CDE */
  0x0000,   /* U+11CDF */
  0x0000,   /* U+11CE0 */
  0x0000,   /* U+11CE1 */
  0x0000,   /* U+11CE2 */
  0x0000,   /* U+11CE3 */
  0x0000,   /* U+11CE4 */
  0x0000,   /* U+11CE5 */
  0x0000,   /* U+11CE6 */
  0x0000,   /* U+11CE7 */
  0x0000,   /* U+11CE8 */
  0x0000,   /* U+11CE9 */
  0x0000,   /* U+11CEA */
  0x0000,   /* U+11CEB */
  0x0000,   /* U+11CEC */
  0x0000,   /* U+11CED */
  0x0000,   /* U+11CEE */
  0x0000,   /* U+11CEF */
  0x0000,   /* U+11CF0 */
  0x0000,   /* U+11CF1 */
  0x0000,   /* U+11CF2 */
  0x0000,   /* U+11CF3 */
  0x0000,   /* U+11CF4 */
  0x0000,   /* U+11CF5 */
  0x0000,   /* U+11CF6 */
  0x0000,   /* U+11CF7 */
  0x0000,   /* U+11CF8 */
  0x0000,   /* U+11CF9 */
  0x0000,   /* U+11CFA */
  0x0000,   /* U+11CFB */
  0x0000,   /* U+11CFC */
  0x0000,   /* U+11CFD */
  0x0000,   /* U+11CFE */
  0x0000,   /* U+11CFF */
};

uint16 uca900_p120[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x49B5,   /* U+12000 */
  0x49B6,   /* U+12001 */
  0x49B7,   /* U+12002 */
  0x49B8,   /* U+12003 */
  0x49B9,   /* U+12004 */
  0x49BA,   /* U+12005 */
  0x49BB,   /* U+12006 */
  0x49BC,   /* U+12007 */
  0x49BD,   /* U+12008 */
  0x49BE,   /* U+12009 */
  0x49BF,   /* U+1200A */
  0x49C0,   /* U+1200B */
  0x49C1,   /* U+1200C */
  0x49C2,   /* U+1200D */
  0x49C3,   /* U+1200E */
  0x49C4,   /* U+1200F */
  0x49C5,   /* U+12010 */
  0x49C6,   /* U+12011 */
  0x49C7,   /* U+12012 */
  0x49C8,   /* U+12013 */
  0x49C9,   /* U+12014 */
  0x49CA,   /* U+12015 */
  0x49CB,   /* U+12016 */
  0x49CC,   /* U+12017 */
  0x49CD,   /* U+12018 */
  0x49CE,   /* U+12019 */
  0x49CF,   /* U+1201A */
  0x49D0,   /* U+1201B */
  0x49D1,   /* U+1201C */
  0x49D2,   /* U+1201D */
  0x49D3,   /* U+1201E */
  0x49D4,   /* U+1201F */
  0x49D5,   /* U+12020 */
  0x49D6,   /* U+12021 */
  0x49D7,   /* U+12022 */
  0x49D8,   /* U+12023 */
  0x49D9,   /* U+12024 */
  0x49DA,   /* U+12025 */
  0x49DB,   /* U+12026 */
  0x49DC,   /* U+12027 */
  0x49DD,   /* U+12028 */
  0x49DE,   /* U+12029 */
  0x49DF,   /* U+1202A */
  0x49E0,   /* U+1202B */
  0x49E1,   /* U+1202C */
  0x49E2,   /* U+1202D */
  0x49E3,   /* U+1202E */
  0x49E4,   /* U+1202F */
  0x49E5,   /* U+12030 */
  0x49E6,   /* U+12031 */
  0x49E7,   /* U+12032 */
  0x49E8,   /* U+12033 */
  0x49E9,   /* U+12034 */
  0x49EA,   /* U+12035 */
  0x49EB,   /* U+12036 */
  0x49EC,   /* U+12037 */
  0x49ED,   /* U+12038 */
  0x49EE,   /* U+12039 */
  0x49EF,   /* U+1203A */
  0x49F0,   /* U+1203B */
  0x49F1,   /* U+1203C */
  0x49F2,   /* U+1203D */
  0x49F3,   /* U+1203E */
  0x49F4,   /* U+1203F */
  0x49F5,   /* U+12040 */
  0x49F6,   /* U+12041 */
  0x49F7,   /* U+12042 */
  0x49F8,   /* U+12043 */
  0x49F9,   /* U+12044 */
  0x49FA,   /* U+12045 */
  0x49FB,   /* U+12046 */
  0x49FC,   /* U+12047 */
  0x49FD,   /* U+12048 */
  0x49FE,   /* U+12049 */
  0x49FF,   /* U+1204A */
  0x4A00,   /* U+1204B */
  0x4A01,   /* U+1204C */
  0x4A02,   /* U+1204D */
  0x4A03,   /* U+1204E */
  0x4A04,   /* U+1204F */
  0x4A05,   /* U+12050 */
  0x4A06,   /* U+12051 */
  0x4A07,   /* U+12052 */
  0x4A08,   /* U+12053 */
  0x4A09,   /* U+12054 */
  0x4A0A,   /* U+12055 */
  0x4A0B,   /* U+12056 */
  0x4A0C,   /* U+12057 */
  0x4A0D,   /* U+12058 */
  0x4A0E,   /* U+12059 */
  0x4A0F,   /* U+1205A */
  0x4A10,   /* U+1205B */
  0x4A11,   /* U+1205C */
  0x4A12,   /* U+1205D */
  0x4A13,   /* U+1205E */
  0x4A14,   /* U+1205F */
  0x4A15,   /* U+12060 */
  0x4A16,   /* U+12061 */
  0x4A17,   /* U+12062 */
  0x4A18,   /* U+12063 */
  0x4A19,   /* U+12064 */
  0x4A1A,   /* U+12065 */
  0x4A1B,   /* U+12066 */
  0x4A1C,   /* U+12067 */
  0x4A1D,   /* U+12068 */
  0x4A1E,   /* U+12069 */
  0x4A1F,   /* U+1206A */
  0x4A20,   /* U+1206B */
  0x4A21,   /* U+1206C */
  0x4A22,   /* U+1206D */
  0x4A23,   /* U+1206E */
  0x4A24,   /* U+1206F */
  0x4A25,   /* U+12070 */
  0x4A26,   /* U+12071 */
  0x4A27,   /* U+12072 */
  0x4A28,   /* U+12073 */
  0x4A29,   /* U+12074 */
  0x4A2A,   /* U+12075 */
  0x4A2B,   /* U+12076 */
  0x4A2C,   /* U+12077 */
  0x4A2D,   /* U+12078 */
  0x4A2E,   /* U+12079 */
  0x4A2F,   /* U+1207A */
  0x4A30,   /* U+1207B */
  0x4A31,   /* U+1207C */
  0x4A32,   /* U+1207D */
  0x4A33,   /* U+1207E */
  0x4A34,   /* U+1207F */
  0x4A35,   /* U+12080 */
  0x4A36,   /* U+12081 */
  0x4A37,   /* U+12082 */
  0x4A38,   /* U+12083 */
  0x4A39,   /* U+12084 */
  0x4A3A,   /* U+12085 */
  0x4A3B,   /* U+12086 */
  0x4A3C,   /* U+12087 */
  0x4A3D,   /* U+12088 */
  0x4A3E,   /* U+12089 */
  0x4A3F,   /* U+1208A */
  0x4A40,   /* U+1208B */
  0x4A41,   /* U+1208C */
  0x4A42,   /* U+1208D */
  0x4A43,   /* U+1208E */
  0x4A44,   /* U+1208F */
  0x4A45,   /* U+12090 */
  0x4A46,   /* U+12091 */
  0x4A47,   /* U+12092 */
  0x4A48,   /* U+12093 */
  0x4A49,   /* U+12094 */
  0x4A4A,   /* U+12095 */
  0x4A4B,   /* U+12096 */
  0x4A4C,   /* U+12097 */
  0x4A4D,   /* U+12098 */
  0x4A4E,   /* U+12099 */
  0x4A4F,   /* U+1209A */
  0x4A50,   /* U+1209B */
  0x4A51,   /* U+1209C */
  0x4A52,   /* U+1209D */
  0x4A53,   /* U+1209E */
  0x4A54,   /* U+1209F */
  0x4A55,   /* U+120A0 */
  0x4A56,   /* U+120A1 */
  0x4A57,   /* U+120A2 */
  0x4A58,   /* U+120A3 */
  0x4A59,   /* U+120A4 */
  0x4A5A,   /* U+120A5 */
  0x4A5B,   /* U+120A6 */
  0x4A5C,   /* U+120A7 */
  0x4A5D,   /* U+120A8 */
  0x4A5E,   /* U+120A9 */
  0x4A5F,   /* U+120AA */
  0x4A60,   /* U+120AB */
  0x4A61,   /* U+120AC */
  0x4A62,   /* U+120AD */
  0x4A63,   /* U+120AE */
  0x4A64,   /* U+120AF */
  0x4A65,   /* U+120B0 */
  0x4A66,   /* U+120B1 */
  0x4A67,   /* U+120B2 */
  0x4A68,   /* U+120B3 */
  0x4A69,   /* U+120B4 */
  0x4A6A,   /* U+120B5 */
  0x4A6B,   /* U+120B6 */
  0x4A6C,   /* U+120B7 */
  0x4A6D,   /* U+120B8 */
  0x4A6E,   /* U+120B9 */
  0x4A6F,   /* U+120BA */
  0x4A70,   /* U+120BB */
  0x4A71,   /* U+120BC */
  0x4A72,   /* U+120BD */
  0x4A73,   /* U+120BE */
  0x4A74,   /* U+120BF */
  0x4A75,   /* U+120C0 */
  0x4A76,   /* U+120C1 */
  0x4A77,   /* U+120C2 */
  0x4A78,   /* U+120C3 */
  0x4A79,   /* U+120C4 */
  0x4A7A,   /* U+120C5 */
  0x4A7B,   /* U+120C6 */
  0x4A7C,   /* U+120C7 */
  0x4A7D,   /* U+120C8 */
  0x4A7E,   /* U+120C9 */
  0x4A7F,   /* U+120CA */
  0x4A80,   /* U+120CB */
  0x4A81,   /* U+120CC */
  0x4A82,   /* U+120CD */
  0x4A83,   /* U+120CE */
  0x4A84,   /* U+120CF */
  0x4A85,   /* U+120D0 */
  0x4A86,   /* U+120D1 */
  0x4A87,   /* U+120D2 */
  0x4A88,   /* U+120D3 */
  0x4A89,   /* U+120D4 */
  0x4A8A,   /* U+120D5 */
  0x4A8B,   /* U+120D6 */
  0x4A8C,   /* U+120D7 */
  0x4A8D,   /* U+120D8 */
  0x4A8E,   /* U+120D9 */
  0x4A8F,   /* U+120DA */
  0x4A90,   /* U+120DB */
  0x4A91,   /* U+120DC */
  0x4A92,   /* U+120DD */
  0x4A93,   /* U+120DE */
  0x4A94,   /* U+120DF */
  0x4A95,   /* U+120E0 */
  0x4A96,   /* U+120E1 */
  0x4A97,   /* U+120E2 */
  0x4A98,   /* U+120E3 */
  0x4A99,   /* U+120E4 */
  0x4A9A,   /* U+120E5 */
  0x4A9B,   /* U+120E6 */
  0x4A9C,   /* U+120E7 */
  0x4A9D,   /* U+120E8 */
  0x4A9E,   /* U+120E9 */
  0x4A9F,   /* U+120EA */
  0x4AA0,   /* U+120EB */
  0x4AA1,   /* U+120EC */
  0x4AA2,   /* U+120ED */
  0x4AA3,   /* U+120EE */
  0x4AA4,   /* U+120EF */
  0x4AA5,   /* U+120F0 */
  0x4AA6,   /* U+120F1 */
  0x4AA7,   /* U+120F2 */
  0x4AA8,   /* U+120F3 */
  0x4AA9,   /* U+120F4 */
  0x4AAA,   /* U+120F5 */
  0x4AAB,   /* U+120F6 */
  0x4AAC,   /* U+120F7 */
  0x4AAD,   /* U+120F8 */
  0x4AAE,   /* U+120F9 */
  0x4AAF,   /* U+120FA */
  0x4AB0,   /* U+120FB */
  0x4AB1,   /* U+120FC */
  0x4AB2,   /* U+120FD */
  0x4AB3,   /* U+120FE */
  0x4AB4,   /* U+120FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12000 */
  0x0020,   /* U+12001 */
  0x0020,   /* U+12002 */
  0x0020,   /* U+12003 */
  0x0020,   /* U+12004 */
  0x0020,   /* U+12005 */
  0x0020,   /* U+12006 */
  0x0020,   /* U+12007 */
  0x0020,   /* U+12008 */
  0x0020,   /* U+12009 */
  0x0020,   /* U+1200A */
  0x0020,   /* U+1200B */
  0x0020,   /* U+1200C */
  0x0020,   /* U+1200D */
  0x0020,   /* U+1200E */
  0x0020,   /* U+1200F */
  0x0020,   /* U+12010 */
  0x0020,   /* U+12011 */
  0x0020,   /* U+12012 */
  0x0020,   /* U+12013 */
  0x0020,   /* U+12014 */
  0x0020,   /* U+12015 */
  0x0020,   /* U+12016 */
  0x0020,   /* U+12017 */
  0x0020,   /* U+12018 */
  0x0020,   /* U+12019 */
  0x0020,   /* U+1201A */
  0x0020,   /* U+1201B */
  0x0020,   /* U+1201C */
  0x0020,   /* U+1201D */
  0x0020,   /* U+1201E */
  0x0020,   /* U+1201F */
  0x0020,   /* U+12020 */
  0x0020,   /* U+12021 */
  0x0020,   /* U+12022 */
  0x0020,   /* U+12023 */
  0x0020,   /* U+12024 */
  0x0020,   /* U+12025 */
  0x0020,   /* U+12026 */
  0x0020,   /* U+12027 */
  0x0020,   /* U+12028 */
  0x0020,   /* U+12029 */
  0x0020,   /* U+1202A */
  0x0020,   /* U+1202B */
  0x0020,   /* U+1202C */
  0x0020,   /* U+1202D */
  0x0020,   /* U+1202E */
  0x0020,   /* U+1202F */
  0x0020,   /* U+12030 */
  0x0020,   /* U+12031 */
  0x0020,   /* U+12032 */
  0x0020,   /* U+12033 */
  0x0020,   /* U+12034 */
  0x0020,   /* U+12035 */
  0x0020,   /* U+12036 */
  0x0020,   /* U+12037 */
  0x0020,   /* U+12038 */
  0x0020,   /* U+12039 */
  0x0020,   /* U+1203A */
  0x0020,   /* U+1203B */
  0x0020,   /* U+1203C */
  0x0020,   /* U+1203D */
  0x0020,   /* U+1203E */
  0x0020,   /* U+1203F */
  0x0020,   /* U+12040 */
  0x0020,   /* U+12041 */
  0x0020,   /* U+12042 */
  0x0020,   /* U+12043 */
  0x0020,   /* U+12044 */
  0x0020,   /* U+12045 */
  0x0020,   /* U+12046 */
  0x0020,   /* U+12047 */
  0x0020,   /* U+12048 */
  0x0020,   /* U+12049 */
  0x0020,   /* U+1204A */
  0x0020,   /* U+1204B */
  0x0020,   /* U+1204C */
  0x0020,   /* U+1204D */
  0x0020,   /* U+1204E */
  0x0020,   /* U+1204F */
  0x0020,   /* U+12050 */
  0x0020,   /* U+12051 */
  0x0020,   /* U+12052 */
  0x0020,   /* U+12053 */
  0x0020,   /* U+12054 */
  0x0020,   /* U+12055 */
  0x0020,   /* U+12056 */
  0x0020,   /* U+12057 */
  0x0020,   /* U+12058 */
  0x0020,   /* U+12059 */
  0x0020,   /* U+1205A */
  0x0020,   /* U+1205B */
  0x0020,   /* U+1205C */
  0x0020,   /* U+1205D */
  0x0020,   /* U+1205E */
  0x0020,   /* U+1205F */
  0x0020,   /* U+12060 */
  0x0020,   /* U+12061 */
  0x0020,   /* U+12062 */
  0x0020,   /* U+12063 */
  0x0020,   /* U+12064 */
  0x0020,   /* U+12065 */
  0x0020,   /* U+12066 */
  0x0020,   /* U+12067 */
  0x0020,   /* U+12068 */
  0x0020,   /* U+12069 */
  0x0020,   /* U+1206A */
  0x0020,   /* U+1206B */
  0x0020,   /* U+1206C */
  0x0020,   /* U+1206D */
  0x0020,   /* U+1206E */
  0x0020,   /* U+1206F */
  0x0020,   /* U+12070 */
  0x0020,   /* U+12071 */
  0x0020,   /* U+12072 */
  0x0020,   /* U+12073 */
  0x0020,   /* U+12074 */
  0x0020,   /* U+12075 */
  0x0020,   /* U+12076 */
  0x0020,   /* U+12077 */
  0x0020,   /* U+12078 */
  0x0020,   /* U+12079 */
  0x0020,   /* U+1207A */
  0x0020,   /* U+1207B */
  0x0020,   /* U+1207C */
  0x0020,   /* U+1207D */
  0x0020,   /* U+1207E */
  0x0020,   /* U+1207F */
  0x0020,   /* U+12080 */
  0x0020,   /* U+12081 */
  0x0020,   /* U+12082 */
  0x0020,   /* U+12083 */
  0x0020,   /* U+12084 */
  0x0020,   /* U+12085 */
  0x0020,   /* U+12086 */
  0x0020,   /* U+12087 */
  0x0020,   /* U+12088 */
  0x0020,   /* U+12089 */
  0x0020,   /* U+1208A */
  0x0020,   /* U+1208B */
  0x0020,   /* U+1208C */
  0x0020,   /* U+1208D */
  0x0020,   /* U+1208E */
  0x0020,   /* U+1208F */
  0x0020,   /* U+12090 */
  0x0020,   /* U+12091 */
  0x0020,   /* U+12092 */
  0x0020,   /* U+12093 */
  0x0020,   /* U+12094 */
  0x0020,   /* U+12095 */
  0x0020,   /* U+12096 */
  0x0020,   /* U+12097 */
  0x0020,   /* U+12098 */
  0x0020,   /* U+12099 */
  0x0020,   /* U+1209A */
  0x0020,   /* U+1209B */
  0x0020,   /* U+1209C */
  0x0020,   /* U+1209D */
  0x0020,   /* U+1209E */
  0x0020,   /* U+1209F */
  0x0020,   /* U+120A0 */
  0x0020,   /* U+120A1 */
  0x0020,   /* U+120A2 */
  0x0020,   /* U+120A3 */
  0x0020,   /* U+120A4 */
  0x0020,   /* U+120A5 */
  0x0020,   /* U+120A6 */
  0x0020,   /* U+120A7 */
  0x0020,   /* U+120A8 */
  0x0020,   /* U+120A9 */
  0x0020,   /* U+120AA */
  0x0020,   /* U+120AB */
  0x0020,   /* U+120AC */
  0x0020,   /* U+120AD */
  0x0020,   /* U+120AE */
  0x0020,   /* U+120AF */
  0x0020,   /* U+120B0 */
  0x0020,   /* U+120B1 */
  0x0020,   /* U+120B2 */
  0x0020,   /* U+120B3 */
  0x0020,   /* U+120B4 */
  0x0020,   /* U+120B5 */
  0x0020,   /* U+120B6 */
  0x0020,   /* U+120B7 */
  0x0020,   /* U+120B8 */
  0x0020,   /* U+120B9 */
  0x0020,   /* U+120BA */
  0x0020,   /* U+120BB */
  0x0020,   /* U+120BC */
  0x0020,   /* U+120BD */
  0x0020,   /* U+120BE */
  0x0020,   /* U+120BF */
  0x0020,   /* U+120C0 */
  0x0020,   /* U+120C1 */
  0x0020,   /* U+120C2 */
  0x0020,   /* U+120C3 */
  0x0020,   /* U+120C4 */
  0x0020,   /* U+120C5 */
  0x0020,   /* U+120C6 */
  0x0020,   /* U+120C7 */
  0x0020,   /* U+120C8 */
  0x0020,   /* U+120C9 */
  0x0020,   /* U+120CA */
  0x0020,   /* U+120CB */
  0x0020,   /* U+120CC */
  0x0020,   /* U+120CD */
  0x0020,   /* U+120CE */
  0x0020,   /* U+120CF */
  0x0020,   /* U+120D0 */
  0x0020,   /* U+120D1 */
  0x0020,   /* U+120D2 */
  0x0020,   /* U+120D3 */
  0x0020,   /* U+120D4 */
  0x0020,   /* U+120D5 */
  0x0020,   /* U+120D6 */
  0x0020,   /* U+120D7 */
  0x0020,   /* U+120D8 */
  0x0020,   /* U+120D9 */
  0x0020,   /* U+120DA */
  0x0020,   /* U+120DB */
  0x0020,   /* U+120DC */
  0x0020,   /* U+120DD */
  0x0020,   /* U+120DE */
  0x0020,   /* U+120DF */
  0x0020,   /* U+120E0 */
  0x0020,   /* U+120E1 */
  0x0020,   /* U+120E2 */
  0x0020,   /* U+120E3 */
  0x0020,   /* U+120E4 */
  0x0020,   /* U+120E5 */
  0x0020,   /* U+120E6 */
  0x0020,   /* U+120E7 */
  0x0020,   /* U+120E8 */
  0x0020,   /* U+120E9 */
  0x0020,   /* U+120EA */
  0x0020,   /* U+120EB */
  0x0020,   /* U+120EC */
  0x0020,   /* U+120ED */
  0x0020,   /* U+120EE */
  0x0020,   /* U+120EF */
  0x0020,   /* U+120F0 */
  0x0020,   /* U+120F1 */
  0x0020,   /* U+120F2 */
  0x0020,   /* U+120F3 */
  0x0020,   /* U+120F4 */
  0x0020,   /* U+120F5 */
  0x0020,   /* U+120F6 */
  0x0020,   /* U+120F7 */
  0x0020,   /* U+120F8 */
  0x0020,   /* U+120F9 */
  0x0020,   /* U+120FA */
  0x0020,   /* U+120FB */
  0x0020,   /* U+120FC */
  0x0020,   /* U+120FD */
  0x0020,   /* U+120FE */
  0x0020,   /* U+120FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12000 */
  0x0002,   /* U+12001 */
  0x0002,   /* U+12002 */
  0x0002,   /* U+12003 */
  0x0002,   /* U+12004 */
  0x0002,   /* U+12005 */
  0x0002,   /* U+12006 */
  0x0002,   /* U+12007 */
  0x0002,   /* U+12008 */
  0x0002,   /* U+12009 */
  0x0002,   /* U+1200A */
  0x0002,   /* U+1200B */
  0x0002,   /* U+1200C */
  0x0002,   /* U+1200D */
  0x0002,   /* U+1200E */
  0x0002,   /* U+1200F */
  0x0002,   /* U+12010 */
  0x0002,   /* U+12011 */
  0x0002,   /* U+12012 */
  0x0002,   /* U+12013 */
  0x0002,   /* U+12014 */
  0x0002,   /* U+12015 */
  0x0002,   /* U+12016 */
  0x0002,   /* U+12017 */
  0x0002,   /* U+12018 */
  0x0002,   /* U+12019 */
  0x0002,   /* U+1201A */
  0x0002,   /* U+1201B */
  0x0002,   /* U+1201C */
  0x0002,   /* U+1201D */
  0x0002,   /* U+1201E */
  0x0002,   /* U+1201F */
  0x0002,   /* U+12020 */
  0x0002,   /* U+12021 */
  0x0002,   /* U+12022 */
  0x0002,   /* U+12023 */
  0x0002,   /* U+12024 */
  0x0002,   /* U+12025 */
  0x0002,   /* U+12026 */
  0x0002,   /* U+12027 */
  0x0002,   /* U+12028 */
  0x0002,   /* U+12029 */
  0x0002,   /* U+1202A */
  0x0002,   /* U+1202B */
  0x0002,   /* U+1202C */
  0x0002,   /* U+1202D */
  0x0002,   /* U+1202E */
  0x0002,   /* U+1202F */
  0x0002,   /* U+12030 */
  0x0002,   /* U+12031 */
  0x0002,   /* U+12032 */
  0x0002,   /* U+12033 */
  0x0002,   /* U+12034 */
  0x0002,   /* U+12035 */
  0x0002,   /* U+12036 */
  0x0002,   /* U+12037 */
  0x0002,   /* U+12038 */
  0x0002,   /* U+12039 */
  0x0002,   /* U+1203A */
  0x0002,   /* U+1203B */
  0x0002,   /* U+1203C */
  0x0002,   /* U+1203D */
  0x0002,   /* U+1203E */
  0x0002,   /* U+1203F */
  0x0002,   /* U+12040 */
  0x0002,   /* U+12041 */
  0x0002,   /* U+12042 */
  0x0002,   /* U+12043 */
  0x0002,   /* U+12044 */
  0x0002,   /* U+12045 */
  0x0002,   /* U+12046 */
  0x0002,   /* U+12047 */
  0x0002,   /* U+12048 */
  0x0002,   /* U+12049 */
  0x0002,   /* U+1204A */
  0x0002,   /* U+1204B */
  0x0002,   /* U+1204C */
  0x0002,   /* U+1204D */
  0x0002,   /* U+1204E */
  0x0002,   /* U+1204F */
  0x0002,   /* U+12050 */
  0x0002,   /* U+12051 */
  0x0002,   /* U+12052 */
  0x0002,   /* U+12053 */
  0x0002,   /* U+12054 */
  0x0002,   /* U+12055 */
  0x0002,   /* U+12056 */
  0x0002,   /* U+12057 */
  0x0002,   /* U+12058 */
  0x0002,   /* U+12059 */
  0x0002,   /* U+1205A */
  0x0002,   /* U+1205B */
  0x0002,   /* U+1205C */
  0x0002,   /* U+1205D */
  0x0002,   /* U+1205E */
  0x0002,   /* U+1205F */
  0x0002,   /* U+12060 */
  0x0002,   /* U+12061 */
  0x0002,   /* U+12062 */
  0x0002,   /* U+12063 */
  0x0002,   /* U+12064 */
  0x0002,   /* U+12065 */
  0x0002,   /* U+12066 */
  0x0002,   /* U+12067 */
  0x0002,   /* U+12068 */
  0x0002,   /* U+12069 */
  0x0002,   /* U+1206A */
  0x0002,   /* U+1206B */
  0x0002,   /* U+1206C */
  0x0002,   /* U+1206D */
  0x0002,   /* U+1206E */
  0x0002,   /* U+1206F */
  0x0002,   /* U+12070 */
  0x0002,   /* U+12071 */
  0x0002,   /* U+12072 */
  0x0002,   /* U+12073 */
  0x0002,   /* U+12074 */
  0x0002,   /* U+12075 */
  0x0002,   /* U+12076 */
  0x0002,   /* U+12077 */
  0x0002,   /* U+12078 */
  0x0002,   /* U+12079 */
  0x0002,   /* U+1207A */
  0x0002,   /* U+1207B */
  0x0002,   /* U+1207C */
  0x0002,   /* U+1207D */
  0x0002,   /* U+1207E */
  0x0002,   /* U+1207F */
  0x0002,   /* U+12080 */
  0x0002,   /* U+12081 */
  0x0002,   /* U+12082 */
  0x0002,   /* U+12083 */
  0x0002,   /* U+12084 */
  0x0002,   /* U+12085 */
  0x0002,   /* U+12086 */
  0x0002,   /* U+12087 */
  0x0002,   /* U+12088 */
  0x0002,   /* U+12089 */
  0x0002,   /* U+1208A */
  0x0002,   /* U+1208B */
  0x0002,   /* U+1208C */
  0x0002,   /* U+1208D */
  0x0002,   /* U+1208E */
  0x0002,   /* U+1208F */
  0x0002,   /* U+12090 */
  0x0002,   /* U+12091 */
  0x0002,   /* U+12092 */
  0x0002,   /* U+12093 */
  0x0002,   /* U+12094 */
  0x0002,   /* U+12095 */
  0x0002,   /* U+12096 */
  0x0002,   /* U+12097 */
  0x0002,   /* U+12098 */
  0x0002,   /* U+12099 */
  0x0002,   /* U+1209A */
  0x0002,   /* U+1209B */
  0x0002,   /* U+1209C */
  0x0002,   /* U+1209D */
  0x0002,   /* U+1209E */
  0x0002,   /* U+1209F */
  0x0002,   /* U+120A0 */
  0x0002,   /* U+120A1 */
  0x0002,   /* U+120A2 */
  0x0002,   /* U+120A3 */
  0x0002,   /* U+120A4 */
  0x0002,   /* U+120A5 */
  0x0002,   /* U+120A6 */
  0x0002,   /* U+120A7 */
  0x0002,   /* U+120A8 */
  0x0002,   /* U+120A9 */
  0x0002,   /* U+120AA */
  0x0002,   /* U+120AB */
  0x0002,   /* U+120AC */
  0x0002,   /* U+120AD */
  0x0002,   /* U+120AE */
  0x0002,   /* U+120AF */
  0x0002,   /* U+120B0 */
  0x0002,   /* U+120B1 */
  0x0002,   /* U+120B2 */
  0x0002,   /* U+120B3 */
  0x0002,   /* U+120B4 */
  0x0002,   /* U+120B5 */
  0x0002,   /* U+120B6 */
  0x0002,   /* U+120B7 */
  0x0002,   /* U+120B8 */
  0x0002,   /* U+120B9 */
  0x0002,   /* U+120BA */
  0x0002,   /* U+120BB */
  0x0002,   /* U+120BC */
  0x0002,   /* U+120BD */
  0x0002,   /* U+120BE */
  0x0002,   /* U+120BF */
  0x0002,   /* U+120C0 */
  0x0002,   /* U+120C1 */
  0x0002,   /* U+120C2 */
  0x0002,   /* U+120C3 */
  0x0002,   /* U+120C4 */
  0x0002,   /* U+120C5 */
  0x0002,   /* U+120C6 */
  0x0002,   /* U+120C7 */
  0x0002,   /* U+120C8 */
  0x0002,   /* U+120C9 */
  0x0002,   /* U+120CA */
  0x0002,   /* U+120CB */
  0x0002,   /* U+120CC */
  0x0002,   /* U+120CD */
  0x0002,   /* U+120CE */
  0x0002,   /* U+120CF */
  0x0002,   /* U+120D0 */
  0x0002,   /* U+120D1 */
  0x0002,   /* U+120D2 */
  0x0002,   /* U+120D3 */
  0x0002,   /* U+120D4 */
  0x0002,   /* U+120D5 */
  0x0002,   /* U+120D6 */
  0x0002,   /* U+120D7 */
  0x0002,   /* U+120D8 */
  0x0002,   /* U+120D9 */
  0x0002,   /* U+120DA */
  0x0002,   /* U+120DB */
  0x0002,   /* U+120DC */
  0x0002,   /* U+120DD */
  0x0002,   /* U+120DE */
  0x0002,   /* U+120DF */
  0x0002,   /* U+120E0 */
  0x0002,   /* U+120E1 */
  0x0002,   /* U+120E2 */
  0x0002,   /* U+120E3 */
  0x0002,   /* U+120E4 */
  0x0002,   /* U+120E5 */
  0x0002,   /* U+120E6 */
  0x0002,   /* U+120E7 */
  0x0002,   /* U+120E8 */
  0x0002,   /* U+120E9 */
  0x0002,   /* U+120EA */
  0x0002,   /* U+120EB */
  0x0002,   /* U+120EC */
  0x0002,   /* U+120ED */
  0x0002,   /* U+120EE */
  0x0002,   /* U+120EF */
  0x0002,   /* U+120F0 */
  0x0002,   /* U+120F1 */
  0x0002,   /* U+120F2 */
  0x0002,   /* U+120F3 */
  0x0002,   /* U+120F4 */
  0x0002,   /* U+120F5 */
  0x0002,   /* U+120F6 */
  0x0002,   /* U+120F7 */
  0x0002,   /* U+120F8 */
  0x0002,   /* U+120F9 */
  0x0002,   /* U+120FA */
  0x0002,   /* U+120FB */
  0x0002,   /* U+120FC */
  0x0002,   /* U+120FD */
  0x0002,   /* U+120FE */
  0x0002,   /* U+120FF */
};

uint16 uca900_p121[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x4AB5,   /* U+12100 */
  0x4AB6,   /* U+12101 */
  0x4AB7,   /* U+12102 */
  0x4AB8,   /* U+12103 */
  0x4AB9,   /* U+12104 */
  0x4ABA,   /* U+12105 */
  0x4ABB,   /* U+12106 */
  0x4ABC,   /* U+12107 */
  0x4ABD,   /* U+12108 */
  0x4ABE,   /* U+12109 */
  0x4ABF,   /* U+1210A */
  0x4AC0,   /* U+1210B */
  0x4AC1,   /* U+1210C */
  0x4AC2,   /* U+1210D */
  0x4AC3,   /* U+1210E */
  0x4AC4,   /* U+1210F */
  0x4AC5,   /* U+12110 */
  0x4AC6,   /* U+12111 */
  0x4AC7,   /* U+12112 */
  0x4AC8,   /* U+12113 */
  0x4AC9,   /* U+12114 */
  0x4ACA,   /* U+12115 */
  0x4ACB,   /* U+12116 */
  0x4ACC,   /* U+12117 */
  0x4ACD,   /* U+12118 */
  0x4ACE,   /* U+12119 */
  0x4ACF,   /* U+1211A */
  0x4AD0,   /* U+1211B */
  0x4AD1,   /* U+1211C */
  0x4AD2,   /* U+1211D */
  0x4AD3,   /* U+1211E */
  0x4AD4,   /* U+1211F */
  0x4AD5,   /* U+12120 */
  0x4AD6,   /* U+12121 */
  0x4AD7,   /* U+12122 */
  0x4AD8,   /* U+12123 */
  0x4AD9,   /* U+12124 */
  0x4ADA,   /* U+12125 */
  0x4ADB,   /* U+12126 */
  0x4ADC,   /* U+12127 */
  0x4ADD,   /* U+12128 */
  0x4ADE,   /* U+12129 */
  0x4ADF,   /* U+1212A */
  0x4AE0,   /* U+1212B */
  0x4AE1,   /* U+1212C */
  0x4AE2,   /* U+1212D */
  0x4AE3,   /* U+1212E */
  0x4AE4,   /* U+1212F */
  0x4AE5,   /* U+12130 */
  0x4AE6,   /* U+12131 */
  0x4AE7,   /* U+12132 */
  0x4AE8,   /* U+12133 */
  0x4AE9,   /* U+12134 */
  0x4AEA,   /* U+12135 */
  0x4AEB,   /* U+12136 */
  0x4AEC,   /* U+12137 */
  0x4AED,   /* U+12138 */
  0x4AEE,   /* U+12139 */
  0x4AEF,   /* U+1213A */
  0x4AF0,   /* U+1213B */
  0x4AF1,   /* U+1213C */
  0x4AF2,   /* U+1213D */
  0x4AF3,   /* U+1213E */
  0x4AF4,   /* U+1213F */
  0x4AF5,   /* U+12140 */
  0x4AF6,   /* U+12141 */
  0x4AF7,   /* U+12142 */
  0x4AF8,   /* U+12143 */
  0x4AF9,   /* U+12144 */
  0x4AFA,   /* U+12145 */
  0x4AFB,   /* U+12146 */
  0x4AFC,   /* U+12147 */
  0x4AFD,   /* U+12148 */
  0x4AFE,   /* U+12149 */
  0x4AFF,   /* U+1214A */
  0x4B00,   /* U+1214B */
  0x4B01,   /* U+1214C */
  0x4B02,   /* U+1214D */
  0x4B03,   /* U+1214E */
  0x4B04,   /* U+1214F */
  0x4B05,   /* U+12150 */
  0x4B06,   /* U+12151 */
  0x4B07,   /* U+12152 */
  0x4B08,   /* U+12153 */
  0x4B09,   /* U+12154 */
  0x4B0A,   /* U+12155 */
  0x4B0B,   /* U+12156 */
  0x4B0C,   /* U+12157 */
  0x4B0D,   /* U+12158 */
  0x4B0E,   /* U+12159 */
  0x4B0F,   /* U+1215A */
  0x4B10,   /* U+1215B */
  0x4B11,   /* U+1215C */
  0x4B12,   /* U+1215D */
  0x4B13,   /* U+1215E */
  0x4B14,   /* U+1215F */
  0x4B15,   /* U+12160 */
  0x4B16,   /* U+12161 */
  0x4B17,   /* U+12162 */
  0x4B18,   /* U+12163 */
  0x4B19,   /* U+12164 */
  0x4B1A,   /* U+12165 */
  0x4B1B,   /* U+12166 */
  0x4B1C,   /* U+12167 */
  0x4B1D,   /* U+12168 */
  0x4B1E,   /* U+12169 */
  0x4B1F,   /* U+1216A */
  0x4B20,   /* U+1216B */
  0x4B21,   /* U+1216C */
  0x4B22,   /* U+1216D */
  0x4B23,   /* U+1216E */
  0x4B24,   /* U+1216F */
  0x4B25,   /* U+12170 */
  0x4B26,   /* U+12171 */
  0x4B27,   /* U+12172 */
  0x4B28,   /* U+12173 */
  0x4B29,   /* U+12174 */
  0x4B2A,   /* U+12175 */
  0x4B2B,   /* U+12176 */
  0x4B2C,   /* U+12177 */
  0x4B2D,   /* U+12178 */
  0x4B2E,   /* U+12179 */
  0x4B2F,   /* U+1217A */
  0x4B30,   /* U+1217B */
  0x4B31,   /* U+1217C */
  0x4B32,   /* U+1217D */
  0x4B33,   /* U+1217E */
  0x4B34,   /* U+1217F */
  0x4B35,   /* U+12180 */
  0x4B36,   /* U+12181 */
  0x4B37,   /* U+12182 */
  0x4B38,   /* U+12183 */
  0x4B39,   /* U+12184 */
  0x4B3A,   /* U+12185 */
  0x4B3B,   /* U+12186 */
  0x4B3C,   /* U+12187 */
  0x4B3D,   /* U+12188 */
  0x4B3E,   /* U+12189 */
  0x4B3F,   /* U+1218A */
  0x4B40,   /* U+1218B */
  0x4B41,   /* U+1218C */
  0x4B42,   /* U+1218D */
  0x4B43,   /* U+1218E */
  0x4B44,   /* U+1218F */
  0x4B45,   /* U+12190 */
  0x4B46,   /* U+12191 */
  0x4B47,   /* U+12192 */
  0x4B48,   /* U+12193 */
  0x4B49,   /* U+12194 */
  0x4B4A,   /* U+12195 */
  0x4B4B,   /* U+12196 */
  0x4B4C,   /* U+12197 */
  0x4B4D,   /* U+12198 */
  0x4B4E,   /* U+12199 */
  0x4B4F,   /* U+1219A */
  0x4B50,   /* U+1219B */
  0x4B51,   /* U+1219C */
  0x4B52,   /* U+1219D */
  0x4B53,   /* U+1219E */
  0x4B54,   /* U+1219F */
  0x4B55,   /* U+121A0 */
  0x4B56,   /* U+121A1 */
  0x4B57,   /* U+121A2 */
  0x4B58,   /* U+121A3 */
  0x4B59,   /* U+121A4 */
  0x4B5A,   /* U+121A5 */
  0x4B5B,   /* U+121A6 */
  0x4B5C,   /* U+121A7 */
  0x4B5D,   /* U+121A8 */
  0x4B5E,   /* U+121A9 */
  0x4B5F,   /* U+121AA */
  0x4B60,   /* U+121AB */
  0x4B61,   /* U+121AC */
  0x4B62,   /* U+121AD */
  0x4B63,   /* U+121AE */
  0x4B64,   /* U+121AF */
  0x4B65,   /* U+121B0 */
  0x4B66,   /* U+121B1 */
  0x4B67,   /* U+121B2 */
  0x4B68,   /* U+121B3 */
  0x4B69,   /* U+121B4 */
  0x4B6A,   /* U+121B5 */
  0x4B6B,   /* U+121B6 */
  0x4B6C,   /* U+121B7 */
  0x4B6D,   /* U+121B8 */
  0x4B6E,   /* U+121B9 */
  0x4B6F,   /* U+121BA */
  0x4B70,   /* U+121BB */
  0x4B71,   /* U+121BC */
  0x4B72,   /* U+121BD */
  0x4B73,   /* U+121BE */
  0x4B74,   /* U+121BF */
  0x4B75,   /* U+121C0 */
  0x4B76,   /* U+121C1 */
  0x4B77,   /* U+121C2 */
  0x4B78,   /* U+121C3 */
  0x4B79,   /* U+121C4 */
  0x4B7A,   /* U+121C5 */
  0x4B7B,   /* U+121C6 */
  0x4B7C,   /* U+121C7 */
  0x4B7D,   /* U+121C8 */
  0x4B7E,   /* U+121C9 */
  0x4B7F,   /* U+121CA */
  0x4B80,   /* U+121CB */
  0x4B81,   /* U+121CC */
  0x4B82,   /* U+121CD */
  0x4B83,   /* U+121CE */
  0x4B84,   /* U+121CF */
  0x4B85,   /* U+121D0 */
  0x4B86,   /* U+121D1 */
  0x4B87,   /* U+121D2 */
  0x4B88,   /* U+121D3 */
  0x4B89,   /* U+121D4 */
  0x4B8A,   /* U+121D5 */
  0x4B8B,   /* U+121D6 */
  0x4B8C,   /* U+121D7 */
  0x4B8D,   /* U+121D8 */
  0x4B8E,   /* U+121D9 */
  0x4B8F,   /* U+121DA */
  0x4B90,   /* U+121DB */
  0x4B91,   /* U+121DC */
  0x4B92,   /* U+121DD */
  0x4B93,   /* U+121DE */
  0x4B94,   /* U+121DF */
  0x4B95,   /* U+121E0 */
  0x4B96,   /* U+121E1 */
  0x4B97,   /* U+121E2 */
  0x4B98,   /* U+121E3 */
  0x4B99,   /* U+121E4 */
  0x4B9A,   /* U+121E5 */
  0x4B9B,   /* U+121E6 */
  0x4B9C,   /* U+121E7 */
  0x4B9D,   /* U+121E8 */
  0x4B9E,   /* U+121E9 */
  0x4B9F,   /* U+121EA */
  0x4BA0,   /* U+121EB */
  0x4BA1,   /* U+121EC */
  0x4BA2,   /* U+121ED */
  0x4BA3,   /* U+121EE */
  0x4BA4,   /* U+121EF */
  0x4BA5,   /* U+121F0 */
  0x4BA6,   /* U+121F1 */
  0x4BA7,   /* U+121F2 */
  0x4BA8,   /* U+121F3 */
  0x4BA9,   /* U+121F4 */
  0x4BAA,   /* U+121F5 */
  0x4BAB,   /* U+121F6 */
  0x4BAC,   /* U+121F7 */
  0x4BAD,   /* U+121F8 */
  0x4BAE,   /* U+121F9 */
  0x4BAF,   /* U+121FA */
  0x4BB0,   /* U+121FB */
  0x4BB1,   /* U+121FC */
  0x4BB2,   /* U+121FD */
  0x4BB3,   /* U+121FE */
  0x4BB4,   /* U+121FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12100 */
  0x0020,   /* U+12101 */
  0x0020,   /* U+12102 */
  0x0020,   /* U+12103 */
  0x0020,   /* U+12104 */
  0x0020,   /* U+12105 */
  0x0020,   /* U+12106 */
  0x0020,   /* U+12107 */
  0x0020,   /* U+12108 */
  0x0020,   /* U+12109 */
  0x0020,   /* U+1210A */
  0x0020,   /* U+1210B */
  0x0020,   /* U+1210C */
  0x0020,   /* U+1210D */
  0x0020,   /* U+1210E */
  0x0020,   /* U+1210F */
  0x0020,   /* U+12110 */
  0x0020,   /* U+12111 */
  0x0020,   /* U+12112 */
  0x0020,   /* U+12113 */
  0x0020,   /* U+12114 */
  0x0020,   /* U+12115 */
  0x0020,   /* U+12116 */
  0x0020,   /* U+12117 */
  0x0020,   /* U+12118 */
  0x0020,   /* U+12119 */
  0x0020,   /* U+1211A */
  0x0020,   /* U+1211B */
  0x0020,   /* U+1211C */
  0x0020,   /* U+1211D */
  0x0020,   /* U+1211E */
  0x0020,   /* U+1211F */
  0x0020,   /* U+12120 */
  0x0020,   /* U+12121 */
  0x0020,   /* U+12122 */
  0x0020,   /* U+12123 */
  0x0020,   /* U+12124 */
  0x0020,   /* U+12125 */
  0x0020,   /* U+12126 */
  0x0020,   /* U+12127 */
  0x0020,   /* U+12128 */
  0x0020,   /* U+12129 */
  0x0020,   /* U+1212A */
  0x0020,   /* U+1212B */
  0x0020,   /* U+1212C */
  0x0020,   /* U+1212D */
  0x0020,   /* U+1212E */
  0x0020,   /* U+1212F */
  0x0020,   /* U+12130 */
  0x0020,   /* U+12131 */
  0x0020,   /* U+12132 */
  0x0020,   /* U+12133 */
  0x0020,   /* U+12134 */
  0x0020,   /* U+12135 */
  0x0020,   /* U+12136 */
  0x0020,   /* U+12137 */
  0x0020,   /* U+12138 */
  0x0020,   /* U+12139 */
  0x0020,   /* U+1213A */
  0x0020,   /* U+1213B */
  0x0020,   /* U+1213C */
  0x0020,   /* U+1213D */
  0x0020,   /* U+1213E */
  0x0020,   /* U+1213F */
  0x0020,   /* U+12140 */
  0x0020,   /* U+12141 */
  0x0020,   /* U+12142 */
  0x0020,   /* U+12143 */
  0x0020,   /* U+12144 */
  0x0020,   /* U+12145 */
  0x0020,   /* U+12146 */
  0x0020,   /* U+12147 */
  0x0020,   /* U+12148 */
  0x0020,   /* U+12149 */
  0x0020,   /* U+1214A */
  0x0020,   /* U+1214B */
  0x0020,   /* U+1214C */
  0x0020,   /* U+1214D */
  0x0020,   /* U+1214E */
  0x0020,   /* U+1214F */
  0x0020,   /* U+12150 */
  0x0020,   /* U+12151 */
  0x0020,   /* U+12152 */
  0x0020,   /* U+12153 */
  0x0020,   /* U+12154 */
  0x0020,   /* U+12155 */
  0x0020,   /* U+12156 */
  0x0020,   /* U+12157 */
  0x0020,   /* U+12158 */
  0x0020,   /* U+12159 */
  0x0020,   /* U+1215A */
  0x0020,   /* U+1215B */
  0x0020,   /* U+1215C */
  0x0020,   /* U+1215D */
  0x0020,   /* U+1215E */
  0x0020,   /* U+1215F */
  0x0020,   /* U+12160 */
  0x0020,   /* U+12161 */
  0x0020,   /* U+12162 */
  0x0020,   /* U+12163 */
  0x0020,   /* U+12164 */
  0x0020,   /* U+12165 */
  0x0020,   /* U+12166 */
  0x0020,   /* U+12167 */
  0x0020,   /* U+12168 */
  0x0020,   /* U+12169 */
  0x0020,   /* U+1216A */
  0x0020,   /* U+1216B */
  0x0020,   /* U+1216C */
  0x0020,   /* U+1216D */
  0x0020,   /* U+1216E */
  0x0020,   /* U+1216F */
  0x0020,   /* U+12170 */
  0x0020,   /* U+12171 */
  0x0020,   /* U+12172 */
  0x0020,   /* U+12173 */
  0x0020,   /* U+12174 */
  0x0020,   /* U+12175 */
  0x0020,   /* U+12176 */
  0x0020,   /* U+12177 */
  0x0020,   /* U+12178 */
  0x0020,   /* U+12179 */
  0x0020,   /* U+1217A */
  0x0020,   /* U+1217B */
  0x0020,   /* U+1217C */
  0x0020,   /* U+1217D */
  0x0020,   /* U+1217E */
  0x0020,   /* U+1217F */
  0x0020,   /* U+12180 */
  0x0020,   /* U+12181 */
  0x0020,   /* U+12182 */
  0x0020,   /* U+12183 */
  0x0020,   /* U+12184 */
  0x0020,   /* U+12185 */
  0x0020,   /* U+12186 */
  0x0020,   /* U+12187 */
  0x0020,   /* U+12188 */
  0x0020,   /* U+12189 */
  0x0020,   /* U+1218A */
  0x0020,   /* U+1218B */
  0x0020,   /* U+1218C */
  0x0020,   /* U+1218D */
  0x0020,   /* U+1218E */
  0x0020,   /* U+1218F */
  0x0020,   /* U+12190 */
  0x0020,   /* U+12191 */
  0x0020,   /* U+12192 */
  0x0020,   /* U+12193 */
  0x0020,   /* U+12194 */
  0x0020,   /* U+12195 */
  0x0020,   /* U+12196 */
  0x0020,   /* U+12197 */
  0x0020,   /* U+12198 */
  0x0020,   /* U+12199 */
  0x0020,   /* U+1219A */
  0x0020,   /* U+1219B */
  0x0020,   /* U+1219C */
  0x0020,   /* U+1219D */
  0x0020,   /* U+1219E */
  0x0020,   /* U+1219F */
  0x0020,   /* U+121A0 */
  0x0020,   /* U+121A1 */
  0x0020,   /* U+121A2 */
  0x0020,   /* U+121A3 */
  0x0020,   /* U+121A4 */
  0x0020,   /* U+121A5 */
  0x0020,   /* U+121A6 */
  0x0020,   /* U+121A7 */
  0x0020,   /* U+121A8 */
  0x0020,   /* U+121A9 */
  0x0020,   /* U+121AA */
  0x0020,   /* U+121AB */
  0x0020,   /* U+121AC */
  0x0020,   /* U+121AD */
  0x0020,   /* U+121AE */
  0x0020,   /* U+121AF */
  0x0020,   /* U+121B0 */
  0x0020,   /* U+121B1 */
  0x0020,   /* U+121B2 */
  0x0020,   /* U+121B3 */
  0x0020,   /* U+121B4 */
  0x0020,   /* U+121B5 */
  0x0020,   /* U+121B6 */
  0x0020,   /* U+121B7 */
  0x0020,   /* U+121B8 */
  0x0020,   /* U+121B9 */
  0x0020,   /* U+121BA */
  0x0020,   /* U+121BB */
  0x0020,   /* U+121BC */
  0x0020,   /* U+121BD */
  0x0020,   /* U+121BE */
  0x0020,   /* U+121BF */
  0x0020,   /* U+121C0 */
  0x0020,   /* U+121C1 */
  0x0020,   /* U+121C2 */
  0x0020,   /* U+121C3 */
  0x0020,   /* U+121C4 */
  0x0020,   /* U+121C5 */
  0x0020,   /* U+121C6 */
  0x0020,   /* U+121C7 */
  0x0020,   /* U+121C8 */
  0x0020,   /* U+121C9 */
  0x0020,   /* U+121CA */
  0x0020,   /* U+121CB */
  0x0020,   /* U+121CC */
  0x0020,   /* U+121CD */
  0x0020,   /* U+121CE */
  0x0020,   /* U+121CF */
  0x0020,   /* U+121D0 */
  0x0020,   /* U+121D1 */
  0x0020,   /* U+121D2 */
  0x0020,   /* U+121D3 */
  0x0020,   /* U+121D4 */
  0x0020,   /* U+121D5 */
  0x0020,   /* U+121D6 */
  0x0020,   /* U+121D7 */
  0x0020,   /* U+121D8 */
  0x0020,   /* U+121D9 */
  0x0020,   /* U+121DA */
  0x0020,   /* U+121DB */
  0x0020,   /* U+121DC */
  0x0020,   /* U+121DD */
  0x0020,   /* U+121DE */
  0x0020,   /* U+121DF */
  0x0020,   /* U+121E0 */
  0x0020,   /* U+121E1 */
  0x0020,   /* U+121E2 */
  0x0020,   /* U+121E3 */
  0x0020,   /* U+121E4 */
  0x0020,   /* U+121E5 */
  0x0020,   /* U+121E6 */
  0x0020,   /* U+121E7 */
  0x0020,   /* U+121E8 */
  0x0020,   /* U+121E9 */
  0x0020,   /* U+121EA */
  0x0020,   /* U+121EB */
  0x0020,   /* U+121EC */
  0x0020,   /* U+121ED */
  0x0020,   /* U+121EE */
  0x0020,   /* U+121EF */
  0x0020,   /* U+121F0 */
  0x0020,   /* U+121F1 */
  0x0020,   /* U+121F2 */
  0x0020,   /* U+121F3 */
  0x0020,   /* U+121F4 */
  0x0020,   /* U+121F5 */
  0x0020,   /* U+121F6 */
  0x0020,   /* U+121F7 */
  0x0020,   /* U+121F8 */
  0x0020,   /* U+121F9 */
  0x0020,   /* U+121FA */
  0x0020,   /* U+121FB */
  0x0020,   /* U+121FC */
  0x0020,   /* U+121FD */
  0x0020,   /* U+121FE */
  0x0020,   /* U+121FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12100 */
  0x0002,   /* U+12101 */
  0x0002,   /* U+12102 */
  0x0002,   /* U+12103 */
  0x0002,   /* U+12104 */
  0x0002,   /* U+12105 */
  0x0002,   /* U+12106 */
  0x0002,   /* U+12107 */
  0x0002,   /* U+12108 */
  0x0002,   /* U+12109 */
  0x0002,   /* U+1210A */
  0x0002,   /* U+1210B */
  0x0002,   /* U+1210C */
  0x0002,   /* U+1210D */
  0x0002,   /* U+1210E */
  0x0002,   /* U+1210F */
  0x0002,   /* U+12110 */
  0x0002,   /* U+12111 */
  0x0002,   /* U+12112 */
  0x0002,   /* U+12113 */
  0x0002,   /* U+12114 */
  0x0002,   /* U+12115 */
  0x0002,   /* U+12116 */
  0x0002,   /* U+12117 */
  0x0002,   /* U+12118 */
  0x0002,   /* U+12119 */
  0x0002,   /* U+1211A */
  0x0002,   /* U+1211B */
  0x0002,   /* U+1211C */
  0x0002,   /* U+1211D */
  0x0002,   /* U+1211E */
  0x0002,   /* U+1211F */
  0x0002,   /* U+12120 */
  0x0002,   /* U+12121 */
  0x0002,   /* U+12122 */
  0x0002,   /* U+12123 */
  0x0002,   /* U+12124 */
  0x0002,   /* U+12125 */
  0x0002,   /* U+12126 */
  0x0002,   /* U+12127 */
  0x0002,   /* U+12128 */
  0x0002,   /* U+12129 */
  0x0002,   /* U+1212A */
  0x0002,   /* U+1212B */
  0x0002,   /* U+1212C */
  0x0002,   /* U+1212D */
  0x0002,   /* U+1212E */
  0x0002,   /* U+1212F */
  0x0002,   /* U+12130 */
  0x0002,   /* U+12131 */
  0x0002,   /* U+12132 */
  0x0002,   /* U+12133 */
  0x0002,   /* U+12134 */
  0x0002,   /* U+12135 */
  0x0002,   /* U+12136 */
  0x0002,   /* U+12137 */
  0x0002,   /* U+12138 */
  0x0002,   /* U+12139 */
  0x0002,   /* U+1213A */
  0x0002,   /* U+1213B */
  0x0002,   /* U+1213C */
  0x0002,   /* U+1213D */
  0x0002,   /* U+1213E */
  0x0002,   /* U+1213F */
  0x0002,   /* U+12140 */
  0x0002,   /* U+12141 */
  0x0002,   /* U+12142 */
  0x0002,   /* U+12143 */
  0x0002,   /* U+12144 */
  0x0002,   /* U+12145 */
  0x0002,   /* U+12146 */
  0x0002,   /* U+12147 */
  0x0002,   /* U+12148 */
  0x0002,   /* U+12149 */
  0x0002,   /* U+1214A */
  0x0002,   /* U+1214B */
  0x0002,   /* U+1214C */
  0x0002,   /* U+1214D */
  0x0002,   /* U+1214E */
  0x0002,   /* U+1214F */
  0x0002,   /* U+12150 */
  0x0002,   /* U+12151 */
  0x0002,   /* U+12152 */
  0x0002,   /* U+12153 */
  0x0002,   /* U+12154 */
  0x0002,   /* U+12155 */
  0x0002,   /* U+12156 */
  0x0002,   /* U+12157 */
  0x0002,   /* U+12158 */
  0x0002,   /* U+12159 */
  0x0002,   /* U+1215A */
  0x0002,   /* U+1215B */
  0x0002,   /* U+1215C */
  0x0002,   /* U+1215D */
  0x0002,   /* U+1215E */
  0x0002,   /* U+1215F */
  0x0002,   /* U+12160 */
  0x0002,   /* U+12161 */
  0x0002,   /* U+12162 */
  0x0002,   /* U+12163 */
  0x0002,   /* U+12164 */
  0x0002,   /* U+12165 */
  0x0002,   /* U+12166 */
  0x0002,   /* U+12167 */
  0x0002,   /* U+12168 */
  0x0002,   /* U+12169 */
  0x0002,   /* U+1216A */
  0x0002,   /* U+1216B */
  0x0002,   /* U+1216C */
  0x0002,   /* U+1216D */
  0x0002,   /* U+1216E */
  0x0002,   /* U+1216F */
  0x0002,   /* U+12170 */
  0x0002,   /* U+12171 */
  0x0002,   /* U+12172 */
  0x0002,   /* U+12173 */
  0x0002,   /* U+12174 */
  0x0002,   /* U+12175 */
  0x0002,   /* U+12176 */
  0x0002,   /* U+12177 */
  0x0002,   /* U+12178 */
  0x0002,   /* U+12179 */
  0x0002,   /* U+1217A */
  0x0002,   /* U+1217B */
  0x0002,   /* U+1217C */
  0x0002,   /* U+1217D */
  0x0002,   /* U+1217E */
  0x0002,   /* U+1217F */
  0x0002,   /* U+12180 */
  0x0002,   /* U+12181 */
  0x0002,   /* U+12182 */
  0x0002,   /* U+12183 */
  0x0002,   /* U+12184 */
  0x0002,   /* U+12185 */
  0x0002,   /* U+12186 */
  0x0002,   /* U+12187 */
  0x0002,   /* U+12188 */
  0x0002,   /* U+12189 */
  0x0002,   /* U+1218A */
  0x0002,   /* U+1218B */
  0x0002,   /* U+1218C */
  0x0002,   /* U+1218D */
  0x0002,   /* U+1218E */
  0x0002,   /* U+1218F */
  0x0002,   /* U+12190 */
  0x0002,   /* U+12191 */
  0x0002,   /* U+12192 */
  0x0002,   /* U+12193 */
  0x0002,   /* U+12194 */
  0x0002,   /* U+12195 */
  0x0002,   /* U+12196 */
  0x0002,   /* U+12197 */
  0x0002,   /* U+12198 */
  0x0002,   /* U+12199 */
  0x0002,   /* U+1219A */
  0x0002,   /* U+1219B */
  0x0002,   /* U+1219C */
  0x0002,   /* U+1219D */
  0x0002,   /* U+1219E */
  0x0002,   /* U+1219F */
  0x0002,   /* U+121A0 */
  0x0002,   /* U+121A1 */
  0x0002,   /* U+121A2 */
  0x0002,   /* U+121A3 */
  0x0002,   /* U+121A4 */
  0x0002,   /* U+121A5 */
  0x0002,   /* U+121A6 */
  0x0002,   /* U+121A7 */
  0x0002,   /* U+121A8 */
  0x0002,   /* U+121A9 */
  0x0002,   /* U+121AA */
  0x0002,   /* U+121AB */
  0x0002,   /* U+121AC */
  0x0002,   /* U+121AD */
  0x0002,   /* U+121AE */
  0x0002,   /* U+121AF */
  0x0002,   /* U+121B0 */
  0x0002,   /* U+121B1 */
  0x0002,   /* U+121B2 */
  0x0002,   /* U+121B3 */
  0x0002,   /* U+121B4 */
  0x0002,   /* U+121B5 */
  0x0002,   /* U+121B6 */
  0x0002,   /* U+121B7 */
  0x0002,   /* U+121B8 */
  0x0002,   /* U+121B9 */
  0x0002,   /* U+121BA */
  0x0002,   /* U+121BB */
  0x0002,   /* U+121BC */
  0x0002,   /* U+121BD */
  0x0002,   /* U+121BE */
  0x0002,   /* U+121BF */
  0x0002,   /* U+121C0 */
  0x0002,   /* U+121C1 */
  0x0002,   /* U+121C2 */
  0x0002,   /* U+121C3 */
  0x0002,   /* U+121C4 */
  0x0002,   /* U+121C5 */
  0x0002,   /* U+121C6 */
  0x0002,   /* U+121C7 */
  0x0002,   /* U+121C8 */
  0x0002,   /* U+121C9 */
  0x0002,   /* U+121CA */
  0x0002,   /* U+121CB */
  0x0002,   /* U+121CC */
  0x0002,   /* U+121CD */
  0x0002,   /* U+121CE */
  0x0002,   /* U+121CF */
  0x0002,   /* U+121D0 */
  0x0002,   /* U+121D1 */
  0x0002,   /* U+121D2 */
  0x0002,   /* U+121D3 */
  0x0002,   /* U+121D4 */
  0x0002,   /* U+121D5 */
  0x0002,   /* U+121D6 */
  0x0002,   /* U+121D7 */
  0x0002,   /* U+121D8 */
  0x0002,   /* U+121D9 */
  0x0002,   /* U+121DA */
  0x0002,   /* U+121DB */
  0x0002,   /* U+121DC */
  0x0002,   /* U+121DD */
  0x0002,   /* U+121DE */
  0x0002,   /* U+121DF */
  0x0002,   /* U+121E0 */
  0x0002,   /* U+121E1 */
  0x0002,   /* U+121E2 */
  0x0002,   /* U+121E3 */
  0x0002,   /* U+121E4 */
  0x0002,   /* U+121E5 */
  0x0002,   /* U+121E6 */
  0x0002,   /* U+121E7 */
  0x0002,   /* U+121E8 */
  0x0002,   /* U+121E9 */
  0x0002,   /* U+121EA */
  0x0002,   /* U+121EB */
  0x0002,   /* U+121EC */
  0x0002,   /* U+121ED */
  0x0002,   /* U+121EE */
  0x0002,   /* U+121EF */
  0x0002,   /* U+121F0 */
  0x0002,   /* U+121F1 */
  0x0002,   /* U+121F2 */
  0x0002,   /* U+121F3 */
  0x0002,   /* U+121F4 */
  0x0002,   /* U+121F5 */
  0x0002,   /* U+121F6 */
  0x0002,   /* U+121F7 */
  0x0002,   /* U+121F8 */
  0x0002,   /* U+121F9 */
  0x0002,   /* U+121FA */
  0x0002,   /* U+121FB */
  0x0002,   /* U+121FC */
  0x0002,   /* U+121FD */
  0x0002,   /* U+121FE */
  0x0002,   /* U+121FF */
};

uint16 uca900_p122[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x4BB5,   /* U+12200 */
  0x4BB6,   /* U+12201 */
  0x4BB7,   /* U+12202 */
  0x4BB8,   /* U+12203 */
  0x4BB9,   /* U+12204 */
  0x4BBA,   /* U+12205 */
  0x4BBB,   /* U+12206 */
  0x4BBC,   /* U+12207 */
  0x4BBD,   /* U+12208 */
  0x4BBE,   /* U+12209 */
  0x4BBF,   /* U+1220A */
  0x4BC0,   /* U+1220B */
  0x4BC1,   /* U+1220C */
  0x4BC2,   /* U+1220D */
  0x4BC3,   /* U+1220E */
  0x4BC4,   /* U+1220F */
  0x4BC5,   /* U+12210 */
  0x4BC6,   /* U+12211 */
  0x4BC7,   /* U+12212 */
  0x4BC8,   /* U+12213 */
  0x4BC9,   /* U+12214 */
  0x4BCA,   /* U+12215 */
  0x4BCB,   /* U+12216 */
  0x4BCC,   /* U+12217 */
  0x4BCD,   /* U+12218 */
  0x4BCE,   /* U+12219 */
  0x4BCF,   /* U+1221A */
  0x4BD0,   /* U+1221B */
  0x4BD1,   /* U+1221C */
  0x4BD2,   /* U+1221D */
  0x4BD3,   /* U+1221E */
  0x4BD4,   /* U+1221F */
  0x4BD5,   /* U+12220 */
  0x4BD6,   /* U+12221 */
  0x4BD7,   /* U+12222 */
  0x4BD8,   /* U+12223 */
  0x4BD9,   /* U+12224 */
  0x4BDA,   /* U+12225 */
  0x4BDB,   /* U+12226 */
  0x4BDC,   /* U+12227 */
  0x4BDD,   /* U+12228 */
  0x4BDE,   /* U+12229 */
  0x4BDF,   /* U+1222A */
  0x4BE0,   /* U+1222B */
  0x4BE1,   /* U+1222C */
  0x4BE2,   /* U+1222D */
  0x4BE3,   /* U+1222E */
  0x4BE4,   /* U+1222F */
  0x4BE5,   /* U+12230 */
  0x4BE6,   /* U+12231 */
  0x4BE7,   /* U+12232 */
  0x4BE8,   /* U+12233 */
  0x4BE9,   /* U+12234 */
  0x4BEA,   /* U+12235 */
  0x4BEB,   /* U+12236 */
  0x4BEC,   /* U+12237 */
  0x4BED,   /* U+12238 */
  0x4BEE,   /* U+12239 */
  0x4BEF,   /* U+1223A */
  0x4BF0,   /* U+1223B */
  0x4BF1,   /* U+1223C */
  0x4BF2,   /* U+1223D */
  0x4BF3,   /* U+1223E */
  0x4BF4,   /* U+1223F */
  0x4BF5,   /* U+12240 */
  0x4BF6,   /* U+12241 */
  0x4BF7,   /* U+12242 */
  0x4BF8,   /* U+12243 */
  0x4BF9,   /* U+12244 */
  0x4BFA,   /* U+12245 */
  0x4BFB,   /* U+12246 */
  0x4BFC,   /* U+12247 */
  0x4BFD,   /* U+12248 */
  0x4BFE,   /* U+12249 */
  0x4BFF,   /* U+1224A */
  0x4C00,   /* U+1224B */
  0x4C01,   /* U+1224C */
  0x4C02,   /* U+1224D */
  0x4C03,   /* U+1224E */
  0x4C04,   /* U+1224F */
  0x4C05,   /* U+12250 */
  0x4C06,   /* U+12251 */
  0x4C07,   /* U+12252 */
  0x4C08,   /* U+12253 */
  0x4C09,   /* U+12254 */
  0x4C0A,   /* U+12255 */
  0x4C0B,   /* U+12256 */
  0x4C0C,   /* U+12257 */
  0x4C0D,   /* U+12258 */
  0x4C0E,   /* U+12259 */
  0x4C0F,   /* U+1225A */
  0x4C10,   /* U+1225B */
  0x4C11,   /* U+1225C */
  0x4C12,   /* U+1225D */
  0x4C13,   /* U+1225E */
  0x4C14,   /* U+1225F */
  0x4C15,   /* U+12260 */
  0x4C16,   /* U+12261 */
  0x4C17,   /* U+12262 */
  0x4C1A,   /* U+12263 */
  0x4C1B,   /* U+12264 */
  0x4C1C,   /* U+12265 */
  0x4C1D,   /* U+12266 */
  0x4C1E,   /* U+12267 */
  0x4C1F,   /* U+12268 */
  0x4C20,   /* U+12269 */
  0x4C21,   /* U+1226A */
  0x4C22,   /* U+1226B */
  0x4C23,   /* U+1226C */
  0x4C24,   /* U+1226D */
  0x4C25,   /* U+1226E */
  0x4C26,   /* U+1226F */
  0x4C27,   /* U+12270 */
  0x4C28,   /* U+12271 */
  0x4C29,   /* U+12272 */
  0x4C2A,   /* U+12273 */
  0x4C2B,   /* U+12274 */
  0x4C2C,   /* U+12275 */
  0x4C2D,   /* U+12276 */
  0x4C2E,   /* U+12277 */
  0x4C2F,   /* U+12278 */
  0x4C30,   /* U+12279 */
  0x4C31,   /* U+1227A */
  0x4C32,   /* U+1227B */
  0x4C33,   /* U+1227C */
  0x4C34,   /* U+1227D */
  0x4C35,   /* U+1227E */
  0x4C36,   /* U+1227F */
  0x4C37,   /* U+12280 */
  0x4C38,   /* U+12281 */
  0x4C39,   /* U+12282 */
  0x4C3A,   /* U+12283 */
  0x4C3B,   /* U+12284 */
  0x4C3C,   /* U+12285 */
  0x4C3D,   /* U+12286 */
  0x4C3E,   /* U+12287 */
  0x4C3F,   /* U+12288 */
  0x4C40,   /* U+12289 */
  0x4C41,   /* U+1228A */
  0x4C42,   /* U+1228B */
  0x4C43,   /* U+1228C */
  0x4C44,   /* U+1228D */
  0x4C45,   /* U+1228E */
  0x4C46,   /* U+1228F */
  0x4C47,   /* U+12290 */
  0x4C48,   /* U+12291 */
  0x4C49,   /* U+12292 */
  0x4C4A,   /* U+12293 */
  0x4C4B,   /* U+12294 */
  0x4C4C,   /* U+12295 */
  0x4C4D,   /* U+12296 */
  0x4C4E,   /* U+12297 */
  0x4C4F,   /* U+12298 */
  0x4C50,   /* U+12299 */
  0x4C51,   /* U+1229A */
  0x4C52,   /* U+1229B */
  0x4C53,   /* U+1229C */
  0x4C54,   /* U+1229D */
  0x4C55,   /* U+1229E */
  0x4C56,   /* U+1229F */
  0x4C57,   /* U+122A0 */
  0x4C58,   /* U+122A1 */
  0x4C59,   /* U+122A2 */
  0x4C5A,   /* U+122A3 */
  0x4C5B,   /* U+122A4 */
  0x4C5C,   /* U+122A5 */
  0x4C5D,   /* U+122A6 */
  0x4C5E,   /* U+122A7 */
  0x4C5F,   /* U+122A8 */
  0x4C60,   /* U+122A9 */
  0x4C61,   /* U+122AA */
  0x4C62,   /* U+122AB */
  0x4C63,   /* U+122AC */
  0x4C64,   /* U+122AD */
  0x4C65,   /* U+122AE */
  0x4C66,   /* U+122AF */
  0x4C67,   /* U+122B0 */
  0x4C68,   /* U+122B1 */
  0x4C69,   /* U+122B2 */
  0x4C6A,   /* U+122B3 */
  0x4C6B,   /* U+122B4 */
  0x4C6C,   /* U+122B5 */
  0x4C6D,   /* U+122B6 */
  0x4C6E,   /* U+122B7 */
  0x4C6F,   /* U+122B8 */
  0x4C70,   /* U+122B9 */
  0x4C71,   /* U+122BA */
  0x4C72,   /* U+122BB */
  0x4C73,   /* U+122BC */
  0x4C74,   /* U+122BD */
  0x4C75,   /* U+122BE */
  0x4C76,   /* U+122BF */
  0x4C77,   /* U+122C0 */
  0x4C78,   /* U+122C1 */
  0x4C79,   /* U+122C2 */
  0x4C7A,   /* U+122C3 */
  0x4C7B,   /* U+122C4 */
  0x4C7C,   /* U+122C5 */
  0x4C7D,   /* U+122C6 */
  0x4C7E,   /* U+122C7 */
  0x4C7F,   /* U+122C8 */
  0x4C80,   /* U+122C9 */
  0x4C81,   /* U+122CA */
  0x4C82,   /* U+122CB */
  0x4C83,   /* U+122CC */
  0x4C84,   /* U+122CD */
  0x4C85,   /* U+122CE */
  0x4C86,   /* U+122CF */
  0x4C87,   /* U+122D0 */
  0x4C88,   /* U+122D1 */
  0x4C89,   /* U+122D2 */
  0x4C8A,   /* U+122D3 */
  0x4C18,   /* U+122D4 */
  0x4C19,   /* U+122D5 */
  0x4C8B,   /* U+122D6 */
  0x4C8C,   /* U+122D7 */
  0x4C8D,   /* U+122D8 */
  0x4C8E,   /* U+122D9 */
  0x4C8F,   /* U+122DA */
  0x4C90,   /* U+122DB */
  0x4C91,   /* U+122DC */
  0x4C92,   /* U+122DD */
  0x4C93,   /* U+122DE */
  0x4C94,   /* U+122DF */
  0x4C95,   /* U+122E0 */
  0x4C96,   /* U+122E1 */
  0x4C97,   /* U+122E2 */
  0x4C98,   /* U+122E3 */
  0x4C99,   /* U+122E4 */
  0x4C9A,   /* U+122E5 */
  0x4C9B,   /* U+122E6 */
  0x4C9C,   /* U+122E7 */
  0x4C9D,   /* U+122E8 */
  0x4C9E,   /* U+122E9 */
  0x4C9F,   /* U+122EA */
  0x4CA0,   /* U+122EB */
  0x4CA1,   /* U+122EC */
  0x4CA2,   /* U+122ED */
  0x4CA3,   /* U+122EE */
  0x4CA4,   /* U+122EF */
  0x4CA5,   /* U+122F0 */
  0x4CA6,   /* U+122F1 */
  0x4CA7,   /* U+122F2 */
  0x4CA8,   /* U+122F3 */
  0x4CA9,   /* U+122F4 */
  0x4CAA,   /* U+122F5 */
  0x4CAB,   /* U+122F6 */
  0x4CAC,   /* U+122F7 */
  0x4CAD,   /* U+122F8 */
  0x4CAE,   /* U+122F9 */
  0x4CAF,   /* U+122FA */
  0x4CB0,   /* U+122FB */
  0x4CB1,   /* U+122FC */
  0x4CB2,   /* U+122FD */
  0x4CB3,   /* U+122FE */
  0x4CB4,   /* U+122FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12200 */
  0x0020,   /* U+12201 */
  0x0020,   /* U+12202 */
  0x0020,   /* U+12203 */
  0x0020,   /* U+12204 */
  0x0020,   /* U+12205 */
  0x0020,   /* U+12206 */
  0x0020,   /* U+12207 */
  0x0020,   /* U+12208 */
  0x0020,   /* U+12209 */
  0x0020,   /* U+1220A */
  0x0020,   /* U+1220B */
  0x0020,   /* U+1220C */
  0x0020,   /* U+1220D */
  0x0020,   /* U+1220E */
  0x0020,   /* U+1220F */
  0x0020,   /* U+12210 */
  0x0020,   /* U+12211 */
  0x0020,   /* U+12212 */
  0x0020,   /* U+12213 */
  0x0020,   /* U+12214 */
  0x0020,   /* U+12215 */
  0x0020,   /* U+12216 */
  0x0020,   /* U+12217 */
  0x0020,   /* U+12218 */
  0x0020,   /* U+12219 */
  0x0020,   /* U+1221A */
  0x0020,   /* U+1221B */
  0x0020,   /* U+1221C */
  0x0020,   /* U+1221D */
  0x0020,   /* U+1221E */
  0x0020,   /* U+1221F */
  0x0020,   /* U+12220 */
  0x0020,   /* U+12221 */
  0x0020,   /* U+12222 */
  0x0020,   /* U+12223 */
  0x0020,   /* U+12224 */
  0x0020,   /* U+12225 */
  0x0020,   /* U+12226 */
  0x0020,   /* U+12227 */
  0x0020,   /* U+12228 */
  0x0020,   /* U+12229 */
  0x0020,   /* U+1222A */
  0x0020,   /* U+1222B */
  0x0020,   /* U+1222C */
  0x0020,   /* U+1222D */
  0x0020,   /* U+1222E */
  0x0020,   /* U+1222F */
  0x0020,   /* U+12230 */
  0x0020,   /* U+12231 */
  0x0020,   /* U+12232 */
  0x0020,   /* U+12233 */
  0x0020,   /* U+12234 */
  0x0020,   /* U+12235 */
  0x0020,   /* U+12236 */
  0x0020,   /* U+12237 */
  0x0020,   /* U+12238 */
  0x0020,   /* U+12239 */
  0x0020,   /* U+1223A */
  0x0020,   /* U+1223B */
  0x0020,   /* U+1223C */
  0x0020,   /* U+1223D */
  0x0020,   /* U+1223E */
  0x0020,   /* U+1223F */
  0x0020,   /* U+12240 */
  0x0020,   /* U+12241 */
  0x0020,   /* U+12242 */
  0x0020,   /* U+12243 */
  0x0020,   /* U+12244 */
  0x0020,   /* U+12245 */
  0x0020,   /* U+12246 */
  0x0020,   /* U+12247 */
  0x0020,   /* U+12248 */
  0x0020,   /* U+12249 */
  0x0020,   /* U+1224A */
  0x0020,   /* U+1224B */
  0x0020,   /* U+1224C */
  0x0020,   /* U+1224D */
  0x0020,   /* U+1224E */
  0x0020,   /* U+1224F */
  0x0020,   /* U+12250 */
  0x0020,   /* U+12251 */
  0x0020,   /* U+12252 */
  0x0020,   /* U+12253 */
  0x0020,   /* U+12254 */
  0x0020,   /* U+12255 */
  0x0020,   /* U+12256 */
  0x0020,   /* U+12257 */
  0x0020,   /* U+12258 */
  0x0020,   /* U+12259 */
  0x0020,   /* U+1225A */
  0x0020,   /* U+1225B */
  0x0020,   /* U+1225C */
  0x0020,   /* U+1225D */
  0x0020,   /* U+1225E */
  0x0020,   /* U+1225F */
  0x0020,   /* U+12260 */
  0x0020,   /* U+12261 */
  0x0020,   /* U+12262 */
  0x0020,   /* U+12263 */
  0x0020,   /* U+12264 */
  0x0020,   /* U+12265 */
  0x0020,   /* U+12266 */
  0x0020,   /* U+12267 */
  0x0020,   /* U+12268 */
  0x0020,   /* U+12269 */
  0x0020,   /* U+1226A */
  0x0020,   /* U+1226B */
  0x0020,   /* U+1226C */
  0x0020,   /* U+1226D */
  0x0020,   /* U+1226E */
  0x0020,   /* U+1226F */
  0x0020,   /* U+12270 */
  0x0020,   /* U+12271 */
  0x0020,   /* U+12272 */
  0x0020,   /* U+12273 */
  0x0020,   /* U+12274 */
  0x0020,   /* U+12275 */
  0x0020,   /* U+12276 */
  0x0020,   /* U+12277 */
  0x0020,   /* U+12278 */
  0x0020,   /* U+12279 */
  0x0020,   /* U+1227A */
  0x0020,   /* U+1227B */
  0x0020,   /* U+1227C */
  0x0020,   /* U+1227D */
  0x0020,   /* U+1227E */
  0x0020,   /* U+1227F */
  0x0020,   /* U+12280 */
  0x0020,   /* U+12281 */
  0x0020,   /* U+12282 */
  0x0020,   /* U+12283 */
  0x0020,   /* U+12284 */
  0x0020,   /* U+12285 */
  0x0020,   /* U+12286 */
  0x0020,   /* U+12287 */
  0x0020,   /* U+12288 */
  0x0020,   /* U+12289 */
  0x0020,   /* U+1228A */
  0x0020,   /* U+1228B */
  0x0020,   /* U+1228C */
  0x0020,   /* U+1228D */
  0x0020,   /* U+1228E */
  0x0020,   /* U+1228F */
  0x0020,   /* U+12290 */
  0x0020,   /* U+12291 */
  0x0020,   /* U+12292 */
  0x0020,   /* U+12293 */
  0x0020,   /* U+12294 */
  0x0020,   /* U+12295 */
  0x0020,   /* U+12296 */
  0x0020,   /* U+12297 */
  0x0020,   /* U+12298 */
  0x0020,   /* U+12299 */
  0x0020,   /* U+1229A */
  0x0020,   /* U+1229B */
  0x0020,   /* U+1229C */
  0x0020,   /* U+1229D */
  0x0020,   /* U+1229E */
  0x0020,   /* U+1229F */
  0x0020,   /* U+122A0 */
  0x0020,   /* U+122A1 */
  0x0020,   /* U+122A2 */
  0x0020,   /* U+122A3 */
  0x0020,   /* U+122A4 */
  0x0020,   /* U+122A5 */
  0x0020,   /* U+122A6 */
  0x0020,   /* U+122A7 */
  0x0020,   /* U+122A8 */
  0x0020,   /* U+122A9 */
  0x0020,   /* U+122AA */
  0x0020,   /* U+122AB */
  0x0020,   /* U+122AC */
  0x0020,   /* U+122AD */
  0x0020,   /* U+122AE */
  0x0020,   /* U+122AF */
  0x0020,   /* U+122B0 */
  0x0020,   /* U+122B1 */
  0x0020,   /* U+122B2 */
  0x0020,   /* U+122B3 */
  0x0020,   /* U+122B4 */
  0x0020,   /* U+122B5 */
  0x0020,   /* U+122B6 */
  0x0020,   /* U+122B7 */
  0x0020,   /* U+122B8 */
  0x0020,   /* U+122B9 */
  0x0020,   /* U+122BA */
  0x0020,   /* U+122BB */
  0x0020,   /* U+122BC */
  0x0020,   /* U+122BD */
  0x0020,   /* U+122BE */
  0x0020,   /* U+122BF */
  0x0020,   /* U+122C0 */
  0x0020,   /* U+122C1 */
  0x0020,   /* U+122C2 */
  0x0020,   /* U+122C3 */
  0x0020,   /* U+122C4 */
  0x0020,   /* U+122C5 */
  0x0020,   /* U+122C6 */
  0x0020,   /* U+122C7 */
  0x0020,   /* U+122C8 */
  0x0020,   /* U+122C9 */
  0x0020,   /* U+122CA */
  0x0020,   /* U+122CB */
  0x0020,   /* U+122CC */
  0x0020,   /* U+122CD */
  0x0020,   /* U+122CE */
  0x0020,   /* U+122CF */
  0x0020,   /* U+122D0 */
  0x0020,   /* U+122D1 */
  0x0020,   /* U+122D2 */
  0x0020,   /* U+122D3 */
  0x0020,   /* U+122D4 */
  0x0020,   /* U+122D5 */
  0x0020,   /* U+122D6 */
  0x0020,   /* U+122D7 */
  0x0020,   /* U+122D8 */
  0x0020,   /* U+122D9 */
  0x0020,   /* U+122DA */
  0x0020,   /* U+122DB */
  0x0020,   /* U+122DC */
  0x0020,   /* U+122DD */
  0x0020,   /* U+122DE */
  0x0020,   /* U+122DF */
  0x0020,   /* U+122E0 */
  0x0020,   /* U+122E1 */
  0x0020,   /* U+122E2 */
  0x0020,   /* U+122E3 */
  0x0020,   /* U+122E4 */
  0x0020,   /* U+122E5 */
  0x0020,   /* U+122E6 */
  0x0020,   /* U+122E7 */
  0x0020,   /* U+122E8 */
  0x0020,   /* U+122E9 */
  0x0020,   /* U+122EA */
  0x0020,   /* U+122EB */
  0x0020,   /* U+122EC */
  0x0020,   /* U+122ED */
  0x0020,   /* U+122EE */
  0x0020,   /* U+122EF */
  0x0020,   /* U+122F0 */
  0x0020,   /* U+122F1 */
  0x0020,   /* U+122F2 */
  0x0020,   /* U+122F3 */
  0x0020,   /* U+122F4 */
  0x0020,   /* U+122F5 */
  0x0020,   /* U+122F6 */
  0x0020,   /* U+122F7 */
  0x0020,   /* U+122F8 */
  0x0020,   /* U+122F9 */
  0x0020,   /* U+122FA */
  0x0020,   /* U+122FB */
  0x0020,   /* U+122FC */
  0x0020,   /* U+122FD */
  0x0020,   /* U+122FE */
  0x0020,   /* U+122FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12200 */
  0x0002,   /* U+12201 */
  0x0002,   /* U+12202 */
  0x0002,   /* U+12203 */
  0x0002,   /* U+12204 */
  0x0002,   /* U+12205 */
  0x0002,   /* U+12206 */
  0x0002,   /* U+12207 */
  0x0002,   /* U+12208 */
  0x0002,   /* U+12209 */
  0x0002,   /* U+1220A */
  0x0002,   /* U+1220B */
  0x0002,   /* U+1220C */
  0x0002,   /* U+1220D */
  0x0002,   /* U+1220E */
  0x0002,   /* U+1220F */
  0x0002,   /* U+12210 */
  0x0002,   /* U+12211 */
  0x0002,   /* U+12212 */
  0x0002,   /* U+12213 */
  0x0002,   /* U+12214 */
  0x0002,   /* U+12215 */
  0x0002,   /* U+12216 */
  0x0002,   /* U+12217 */
  0x0002,   /* U+12218 */
  0x0002,   /* U+12219 */
  0x0002,   /* U+1221A */
  0x0002,   /* U+1221B */
  0x0002,   /* U+1221C */
  0x0002,   /* U+1221D */
  0x0002,   /* U+1221E */
  0x0002,   /* U+1221F */
  0x0002,   /* U+12220 */
  0x0002,   /* U+12221 */
  0x0002,   /* U+12222 */
  0x0002,   /* U+12223 */
  0x0002,   /* U+12224 */
  0x0002,   /* U+12225 */
  0x0002,   /* U+12226 */
  0x0002,   /* U+12227 */
  0x0002,   /* U+12228 */
  0x0002,   /* U+12229 */
  0x0002,   /* U+1222A */
  0x0002,   /* U+1222B */
  0x0002,   /* U+1222C */
  0x0002,   /* U+1222D */
  0x0002,   /* U+1222E */
  0x0002,   /* U+1222F */
  0x0002,   /* U+12230 */
  0x0002,   /* U+12231 */
  0x0002,   /* U+12232 */
  0x0002,   /* U+12233 */
  0x0002,   /* U+12234 */
  0x0002,   /* U+12235 */
  0x0002,   /* U+12236 */
  0x0002,   /* U+12237 */
  0x0002,   /* U+12238 */
  0x0002,   /* U+12239 */
  0x0002,   /* U+1223A */
  0x0002,   /* U+1223B */
  0x0002,   /* U+1223C */
  0x0002,   /* U+1223D */
  0x0002,   /* U+1223E */
  0x0002,   /* U+1223F */
  0x0002,   /* U+12240 */
  0x0002,   /* U+12241 */
  0x0002,   /* U+12242 */
  0x0002,   /* U+12243 */
  0x0002,   /* U+12244 */
  0x0002,   /* U+12245 */
  0x0002,   /* U+12246 */
  0x0002,   /* U+12247 */
  0x0002,   /* U+12248 */
  0x0002,   /* U+12249 */
  0x0002,   /* U+1224A */
  0x0002,   /* U+1224B */
  0x0002,   /* U+1224C */
  0x0002,   /* U+1224D */
  0x0002,   /* U+1224E */
  0x0002,   /* U+1224F */
  0x0002,   /* U+12250 */
  0x0002,   /* U+12251 */
  0x0002,   /* U+12252 */
  0x0002,   /* U+12253 */
  0x0002,   /* U+12254 */
  0x0002,   /* U+12255 */
  0x0002,   /* U+12256 */
  0x0002,   /* U+12257 */
  0x0002,   /* U+12258 */
  0x0002,   /* U+12259 */
  0x0002,   /* U+1225A */
  0x0002,   /* U+1225B */
  0x0002,   /* U+1225C */
  0x0002,   /* U+1225D */
  0x0002,   /* U+1225E */
  0x0002,   /* U+1225F */
  0x0002,   /* U+12260 */
  0x0002,   /* U+12261 */
  0x0002,   /* U+12262 */
  0x0002,   /* U+12263 */
  0x0002,   /* U+12264 */
  0x0002,   /* U+12265 */
  0x0002,   /* U+12266 */
  0x0002,   /* U+12267 */
  0x0002,   /* U+12268 */
  0x0002,   /* U+12269 */
  0x0002,   /* U+1226A */
  0x0002,   /* U+1226B */
  0x0002,   /* U+1226C */
  0x0002,   /* U+1226D */
  0x0002,   /* U+1226E */
  0x0002,   /* U+1226F */
  0x0002,   /* U+12270 */
  0x0002,   /* U+12271 */
  0x0002,   /* U+12272 */
  0x0002,   /* U+12273 */
  0x0002,   /* U+12274 */
  0x0002,   /* U+12275 */
  0x0002,   /* U+12276 */
  0x0002,   /* U+12277 */
  0x0002,   /* U+12278 */
  0x0002,   /* U+12279 */
  0x0002,   /* U+1227A */
  0x0002,   /* U+1227B */
  0x0002,   /* U+1227C */
  0x0002,   /* U+1227D */
  0x0002,   /* U+1227E */
  0x0002,   /* U+1227F */
  0x0002,   /* U+12280 */
  0x0002,   /* U+12281 */
  0x0002,   /* U+12282 */
  0x0002,   /* U+12283 */
  0x0002,   /* U+12284 */
  0x0002,   /* U+12285 */
  0x0002,   /* U+12286 */
  0x0002,   /* U+12287 */
  0x0002,   /* U+12288 */
  0x0002,   /* U+12289 */
  0x0002,   /* U+1228A */
  0x0002,   /* U+1228B */
  0x0002,   /* U+1228C */
  0x0002,   /* U+1228D */
  0x0002,   /* U+1228E */
  0x0002,   /* U+1228F */
  0x0002,   /* U+12290 */
  0x0002,   /* U+12291 */
  0x0002,   /* U+12292 */
  0x0002,   /* U+12293 */
  0x0002,   /* U+12294 */
  0x0002,   /* U+12295 */
  0x0002,   /* U+12296 */
  0x0002,   /* U+12297 */
  0x0002,   /* U+12298 */
  0x0002,   /* U+12299 */
  0x0002,   /* U+1229A */
  0x0002,   /* U+1229B */
  0x0002,   /* U+1229C */
  0x0002,   /* U+1229D */
  0x0002,   /* U+1229E */
  0x0002,   /* U+1229F */
  0x0002,   /* U+122A0 */
  0x0002,   /* U+122A1 */
  0x0002,   /* U+122A2 */
  0x0002,   /* U+122A3 */
  0x0002,   /* U+122A4 */
  0x0002,   /* U+122A5 */
  0x0002,   /* U+122A6 */
  0x0002,   /* U+122A7 */
  0x0002,   /* U+122A8 */
  0x0002,   /* U+122A9 */
  0x0002,   /* U+122AA */
  0x0002,   /* U+122AB */
  0x0002,   /* U+122AC */
  0x0002,   /* U+122AD */
  0x0002,   /* U+122AE */
  0x0002,   /* U+122AF */
  0x0002,   /* U+122B0 */
  0x0002,   /* U+122B1 */
  0x0002,   /* U+122B2 */
  0x0002,   /* U+122B3 */
  0x0002,   /* U+122B4 */
  0x0002,   /* U+122B5 */
  0x0002,   /* U+122B6 */
  0x0002,   /* U+122B7 */
  0x0002,   /* U+122B8 */
  0x0002,   /* U+122B9 */
  0x0002,   /* U+122BA */
  0x0002,   /* U+122BB */
  0x0002,   /* U+122BC */
  0x0002,   /* U+122BD */
  0x0002,   /* U+122BE */
  0x0002,   /* U+122BF */
  0x0002,   /* U+122C0 */
  0x0002,   /* U+122C1 */
  0x0002,   /* U+122C2 */
  0x0002,   /* U+122C3 */
  0x0002,   /* U+122C4 */
  0x0002,   /* U+122C5 */
  0x0002,   /* U+122C6 */
  0x0002,   /* U+122C7 */
  0x0002,   /* U+122C8 */
  0x0002,   /* U+122C9 */
  0x0002,   /* U+122CA */
  0x0002,   /* U+122CB */
  0x0002,   /* U+122CC */
  0x0002,   /* U+122CD */
  0x0002,   /* U+122CE */
  0x0002,   /* U+122CF */
  0x0002,   /* U+122D0 */
  0x0002,   /* U+122D1 */
  0x0002,   /* U+122D2 */
  0x0002,   /* U+122D3 */
  0x0002,   /* U+122D4 */
  0x0002,   /* U+122D5 */
  0x0002,   /* U+122D6 */
  0x0002,   /* U+122D7 */
  0x0002,   /* U+122D8 */
  0x0002,   /* U+122D9 */
  0x0002,   /* U+122DA */
  0x0002,   /* U+122DB */
  0x0002,   /* U+122DC */
  0x0002,   /* U+122DD */
  0x0002,   /* U+122DE */
  0x0002,   /* U+122DF */
  0x0002,   /* U+122E0 */
  0x0002,   /* U+122E1 */
  0x0002,   /* U+122E2 */
  0x0002,   /* U+122E3 */
  0x0002,   /* U+122E4 */
  0x0002,   /* U+122E5 */
  0x0002,   /* U+122E6 */
  0x0002,   /* U+122E7 */
  0x0002,   /* U+122E8 */
  0x0002,   /* U+122E9 */
  0x0002,   /* U+122EA */
  0x0002,   /* U+122EB */
  0x0002,   /* U+122EC */
  0x0002,   /* U+122ED */
  0x0002,   /* U+122EE */
  0x0002,   /* U+122EF */
  0x0002,   /* U+122F0 */
  0x0002,   /* U+122F1 */
  0x0002,   /* U+122F2 */
  0x0002,   /* U+122F3 */
  0x0002,   /* U+122F4 */
  0x0002,   /* U+122F5 */
  0x0002,   /* U+122F6 */
  0x0002,   /* U+122F7 */
  0x0002,   /* U+122F8 */
  0x0002,   /* U+122F9 */
  0x0002,   /* U+122FA */
  0x0002,   /* U+122FB */
  0x0002,   /* U+122FC */
  0x0002,   /* U+122FD */
  0x0002,   /* U+122FE */
  0x0002,   /* U+122FF */
};

uint16 uca900_p123[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4CB5,   /* U+12300 */
  0x4CB6,   /* U+12301 */
  0x4CB7,   /* U+12302 */
  0x4CB8,   /* U+12303 */
  0x4CB9,   /* U+12304 */
  0x4CBA,   /* U+12305 */
  0x4CBB,   /* U+12306 */
  0x4CBC,   /* U+12307 */
  0x4CBD,   /* U+12308 */
  0x4CBE,   /* U+12309 */
  0x4CBF,   /* U+1230A */
  0x4CC0,   /* U+1230B */
  0x4CC1,   /* U+1230C */
  0x4CC2,   /* U+1230D */
  0x4CC3,   /* U+1230E */
  0x4CC4,   /* U+1230F */
  0x4CC5,   /* U+12310 */
  0x4CC6,   /* U+12311 */
  0x4CC7,   /* U+12312 */
  0x4CC8,   /* U+12313 */
  0x4CC9,   /* U+12314 */
  0x4CCA,   /* U+12315 */
  0x4CCB,   /* U+12316 */
  0x4CCC,   /* U+12317 */
  0x4CCD,   /* U+12318 */
  0x4CCE,   /* U+12319 */
  0x4CCF,   /* U+1231A */
  0x4CD0,   /* U+1231B */
  0x4CD1,   /* U+1231C */
  0x4CD2,   /* U+1231D */
  0x4CD3,   /* U+1231E */
  0x4CD4,   /* U+1231F */
  0x4CD5,   /* U+12320 */
  0x4CD6,   /* U+12321 */
  0x4CD7,   /* U+12322 */
  0x4CD8,   /* U+12323 */
  0x4CD9,   /* U+12324 */
  0x4CDA,   /* U+12325 */
  0x4CDB,   /* U+12326 */
  0x4CDC,   /* U+12327 */
  0x4CDD,   /* U+12328 */
  0x4CDE,   /* U+12329 */
  0x4CDF,   /* U+1232A */
  0x4CE0,   /* U+1232B */
  0x4CE1,   /* U+1232C */
  0x4CE2,   /* U+1232D */
  0x4CE3,   /* U+1232E */
  0x4CE4,   /* U+1232F */
  0x4CE5,   /* U+12330 */
  0x4CE6,   /* U+12331 */
  0x4CE7,   /* U+12332 */
  0x4CE8,   /* U+12333 */
  0x4CE9,   /* U+12334 */
  0x4CEA,   /* U+12335 */
  0x4CEB,   /* U+12336 */
  0x4CEC,   /* U+12337 */
  0x4CED,   /* U+12338 */
  0x4CEE,   /* U+12339 */
  0x4CEF,   /* U+1233A */
  0x4CF0,   /* U+1233B */
  0x4CF1,   /* U+1233C */
  0x4CF2,   /* U+1233D */
  0x4CF3,   /* U+1233E */
  0x4CF4,   /* U+1233F */
  0x4CF5,   /* U+12340 */
  0x4CF6,   /* U+12341 */
  0x4CF7,   /* U+12342 */
  0x4CF8,   /* U+12343 */
  0x4CF9,   /* U+12344 */
  0x4CFA,   /* U+12345 */
  0x4CFB,   /* U+12346 */
  0x4CFC,   /* U+12347 */
  0x4CFD,   /* U+12348 */
  0x4CFE,   /* U+12349 */
  0x4CFF,   /* U+1234A */
  0x4D00,   /* U+1234B */
  0x4D01,   /* U+1234C */
  0x4D02,   /* U+1234D */
  0x4D03,   /* U+1234E */
  0x4D04,   /* U+1234F */
  0x4D05,   /* U+12350 */
  0x4D06,   /* U+12351 */
  0x4D07,   /* U+12352 */
  0x4D08,   /* U+12353 */
  0x4D09,   /* U+12354 */
  0x4D0A,   /* U+12355 */
  0x4D0B,   /* U+12356 */
  0x4D0C,   /* U+12357 */
  0x4D0D,   /* U+12358 */
  0x4D0E,   /* U+12359 */
  0x4D0F,   /* U+1235A */
  0x4D10,   /* U+1235B */
  0x4D11,   /* U+1235C */
  0x4D12,   /* U+1235D */
  0x4D13,   /* U+1235E */
  0x4D14,   /* U+1235F */
  0x4D15,   /* U+12360 */
  0x4D16,   /* U+12361 */
  0x4D17,   /* U+12362 */
  0x4D18,   /* U+12363 */
  0x4D19,   /* U+12364 */
  0x4D1A,   /* U+12365 */
  0x4D1B,   /* U+12366 */
  0x4D1C,   /* U+12367 */
  0x4D1D,   /* U+12368 */
  0x4D1E,   /* U+12369 */
  0x4D1F,   /* U+1236A */
  0x4D20,   /* U+1236B */
  0x4D21,   /* U+1236C */
  0x4D22,   /* U+1236D */
  0x4D23,   /* U+1236E */
  0x4D24,   /* U+1236F */
  0x4D25,   /* U+12370 */
  0x4D26,   /* U+12371 */
  0x4D27,   /* U+12372 */
  0x4D28,   /* U+12373 */
  0x4D29,   /* U+12374 */
  0x4D2A,   /* U+12375 */
  0x4D2B,   /* U+12376 */
  0x4D2C,   /* U+12377 */
  0x4D2D,   /* U+12378 */
  0x4D2E,   /* U+12379 */
  0x4D2F,   /* U+1237A */
  0x4D30,   /* U+1237B */
  0x4D31,   /* U+1237C */
  0x4D32,   /* U+1237D */
  0x4D33,   /* U+1237E */
  0x4D34,   /* U+1237F */
  0x4D35,   /* U+12380 */
  0x4D36,   /* U+12381 */
  0x4D37,   /* U+12382 */
  0x4D38,   /* U+12383 */
  0x4D39,   /* U+12384 */
  0x4D3A,   /* U+12385 */
  0x4D3B,   /* U+12386 */
  0x4D3C,   /* U+12387 */
  0x4D3D,   /* U+12388 */
  0x4D3E,   /* U+12389 */
  0x4D3F,   /* U+1238A */
  0x4D40,   /* U+1238B */
  0x4D41,   /* U+1238C */
  0x4D42,   /* U+1238D */
  0x4D43,   /* U+1238E */
  0x4D44,   /* U+1238F */
  0x4D45,   /* U+12390 */
  0x4D46,   /* U+12391 */
  0x4D47,   /* U+12392 */
  0x4D48,   /* U+12393 */
  0x4D49,   /* U+12394 */
  0x4D4A,   /* U+12395 */
  0x4D4B,   /* U+12396 */
  0x4D4C,   /* U+12397 */
  0x4D4D,   /* U+12398 */
  0x4D4E,   /* U+12399 */
  0xFBC2,   /* U+1239A */
  0xFBC2,   /* U+1239B */
  0xFBC2,   /* U+1239C */
  0xFBC2,   /* U+1239D */
  0xFBC2,   /* U+1239E */
  0xFBC2,   /* U+1239F */
  0xFBC2,   /* U+123A0 */
  0xFBC2,   /* U+123A1 */
  0xFBC2,   /* U+123A2 */
  0xFBC2,   /* U+123A3 */
  0xFBC2,   /* U+123A4 */
  0xFBC2,   /* U+123A5 */
  0xFBC2,   /* U+123A6 */
  0xFBC2,   /* U+123A7 */
  0xFBC2,   /* U+123A8 */
  0xFBC2,   /* U+123A9 */
  0xFBC2,   /* U+123AA */
  0xFBC2,   /* U+123AB */
  0xFBC2,   /* U+123AC */
  0xFBC2,   /* U+123AD */
  0xFBC2,   /* U+123AE */
  0xFBC2,   /* U+123AF */
  0xFBC2,   /* U+123B0 */
  0xFBC2,   /* U+123B1 */
  0xFBC2,   /* U+123B2 */
  0xFBC2,   /* U+123B3 */
  0xFBC2,   /* U+123B4 */
  0xFBC2,   /* U+123B5 */
  0xFBC2,   /* U+123B6 */
  0xFBC2,   /* U+123B7 */
  0xFBC2,   /* U+123B8 */
  0xFBC2,   /* U+123B9 */
  0xFBC2,   /* U+123BA */
  0xFBC2,   /* U+123BB */
  0xFBC2,   /* U+123BC */
  0xFBC2,   /* U+123BD */
  0xFBC2,   /* U+123BE */
  0xFBC2,   /* U+123BF */
  0xFBC2,   /* U+123C0 */
  0xFBC2,   /* U+123C1 */
  0xFBC2,   /* U+123C2 */
  0xFBC2,   /* U+123C3 */
  0xFBC2,   /* U+123C4 */
  0xFBC2,   /* U+123C5 */
  0xFBC2,   /* U+123C6 */
  0xFBC2,   /* U+123C7 */
  0xFBC2,   /* U+123C8 */
  0xFBC2,   /* U+123C9 */
  0xFBC2,   /* U+123CA */
  0xFBC2,   /* U+123CB */
  0xFBC2,   /* U+123CC */
  0xFBC2,   /* U+123CD */
  0xFBC2,   /* U+123CE */
  0xFBC2,   /* U+123CF */
  0xFBC2,   /* U+123D0 */
  0xFBC2,   /* U+123D1 */
  0xFBC2,   /* U+123D2 */
  0xFBC2,   /* U+123D3 */
  0xFBC2,   /* U+123D4 */
  0xFBC2,   /* U+123D5 */
  0xFBC2,   /* U+123D6 */
  0xFBC2,   /* U+123D7 */
  0xFBC2,   /* U+123D8 */
  0xFBC2,   /* U+123D9 */
  0xFBC2,   /* U+123DA */
  0xFBC2,   /* U+123DB */
  0xFBC2,   /* U+123DC */
  0xFBC2,   /* U+123DD */
  0xFBC2,   /* U+123DE */
  0xFBC2,   /* U+123DF */
  0xFBC2,   /* U+123E0 */
  0xFBC2,   /* U+123E1 */
  0xFBC2,   /* U+123E2 */
  0xFBC2,   /* U+123E3 */
  0xFBC2,   /* U+123E4 */
  0xFBC2,   /* U+123E5 */
  0xFBC2,   /* U+123E6 */
  0xFBC2,   /* U+123E7 */
  0xFBC2,   /* U+123E8 */
  0xFBC2,   /* U+123E9 */
  0xFBC2,   /* U+123EA */
  0xFBC2,   /* U+123EB */
  0xFBC2,   /* U+123EC */
  0xFBC2,   /* U+123ED */
  0xFBC2,   /* U+123EE */
  0xFBC2,   /* U+123EF */
  0xFBC2,   /* U+123F0 */
  0xFBC2,   /* U+123F1 */
  0xFBC2,   /* U+123F2 */
  0xFBC2,   /* U+123F3 */
  0xFBC2,   /* U+123F4 */
  0xFBC2,   /* U+123F5 */
  0xFBC2,   /* U+123F6 */
  0xFBC2,   /* U+123F7 */
  0xFBC2,   /* U+123F8 */
  0xFBC2,   /* U+123F9 */
  0xFBC2,   /* U+123FA */
  0xFBC2,   /* U+123FB */
  0xFBC2,   /* U+123FC */
  0xFBC2,   /* U+123FD */
  0xFBC2,   /* U+123FE */
  0xFBC2,   /* U+123FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12300 */
  0x0020,   /* U+12301 */
  0x0020,   /* U+12302 */
  0x0020,   /* U+12303 */
  0x0020,   /* U+12304 */
  0x0020,   /* U+12305 */
  0x0020,   /* U+12306 */
  0x0020,   /* U+12307 */
  0x0020,   /* U+12308 */
  0x0020,   /* U+12309 */
  0x0020,   /* U+1230A */
  0x0020,   /* U+1230B */
  0x0020,   /* U+1230C */
  0x0020,   /* U+1230D */
  0x0020,   /* U+1230E */
  0x0020,   /* U+1230F */
  0x0020,   /* U+12310 */
  0x0020,   /* U+12311 */
  0x0020,   /* U+12312 */
  0x0020,   /* U+12313 */
  0x0020,   /* U+12314 */
  0x0020,   /* U+12315 */
  0x0020,   /* U+12316 */
  0x0020,   /* U+12317 */
  0x0020,   /* U+12318 */
  0x0020,   /* U+12319 */
  0x0020,   /* U+1231A */
  0x0020,   /* U+1231B */
  0x0020,   /* U+1231C */
  0x0020,   /* U+1231D */
  0x0020,   /* U+1231E */
  0x0020,   /* U+1231F */
  0x0020,   /* U+12320 */
  0x0020,   /* U+12321 */
  0x0020,   /* U+12322 */
  0x0020,   /* U+12323 */
  0x0020,   /* U+12324 */
  0x0020,   /* U+12325 */
  0x0020,   /* U+12326 */
  0x0020,   /* U+12327 */
  0x0020,   /* U+12328 */
  0x0020,   /* U+12329 */
  0x0020,   /* U+1232A */
  0x0020,   /* U+1232B */
  0x0020,   /* U+1232C */
  0x0020,   /* U+1232D */
  0x0020,   /* U+1232E */
  0x0020,   /* U+1232F */
  0x0020,   /* U+12330 */
  0x0020,   /* U+12331 */
  0x0020,   /* U+12332 */
  0x0020,   /* U+12333 */
  0x0020,   /* U+12334 */
  0x0020,   /* U+12335 */
  0x0020,   /* U+12336 */
  0x0020,   /* U+12337 */
  0x0020,   /* U+12338 */
  0x0020,   /* U+12339 */
  0x0020,   /* U+1233A */
  0x0020,   /* U+1233B */
  0x0020,   /* U+1233C */
  0x0020,   /* U+1233D */
  0x0020,   /* U+1233E */
  0x0020,   /* U+1233F */
  0x0020,   /* U+12340 */
  0x0020,   /* U+12341 */
  0x0020,   /* U+12342 */
  0x0020,   /* U+12343 */
  0x0020,   /* U+12344 */
  0x0020,   /* U+12345 */
  0x0020,   /* U+12346 */
  0x0020,   /* U+12347 */
  0x0020,   /* U+12348 */
  0x0020,   /* U+12349 */
  0x0020,   /* U+1234A */
  0x0020,   /* U+1234B */
  0x0020,   /* U+1234C */
  0x0020,   /* U+1234D */
  0x0020,   /* U+1234E */
  0x0020,   /* U+1234F */
  0x0020,   /* U+12350 */
  0x0020,   /* U+12351 */
  0x0020,   /* U+12352 */
  0x0020,   /* U+12353 */
  0x0020,   /* U+12354 */
  0x0020,   /* U+12355 */
  0x0020,   /* U+12356 */
  0x0020,   /* U+12357 */
  0x0020,   /* U+12358 */
  0x0020,   /* U+12359 */
  0x0020,   /* U+1235A */
  0x0020,   /* U+1235B */
  0x0020,   /* U+1235C */
  0x0020,   /* U+1235D */
  0x0020,   /* U+1235E */
  0x0020,   /* U+1235F */
  0x0020,   /* U+12360 */
  0x0020,   /* U+12361 */
  0x0020,   /* U+12362 */
  0x0020,   /* U+12363 */
  0x0020,   /* U+12364 */
  0x0020,   /* U+12365 */
  0x0020,   /* U+12366 */
  0x0020,   /* U+12367 */
  0x0020,   /* U+12368 */
  0x0020,   /* U+12369 */
  0x0020,   /* U+1236A */
  0x0020,   /* U+1236B */
  0x0020,   /* U+1236C */
  0x0020,   /* U+1236D */
  0x0020,   /* U+1236E */
  0x0020,   /* U+1236F */
  0x0020,   /* U+12370 */
  0x0020,   /* U+12371 */
  0x0020,   /* U+12372 */
  0x0020,   /* U+12373 */
  0x0020,   /* U+12374 */
  0x0020,   /* U+12375 */
  0x0020,   /* U+12376 */
  0x0020,   /* U+12377 */
  0x0020,   /* U+12378 */
  0x0020,   /* U+12379 */
  0x0020,   /* U+1237A */
  0x0020,   /* U+1237B */
  0x0020,   /* U+1237C */
  0x0020,   /* U+1237D */
  0x0020,   /* U+1237E */
  0x0020,   /* U+1237F */
  0x0020,   /* U+12380 */
  0x0020,   /* U+12381 */
  0x0020,   /* U+12382 */
  0x0020,   /* U+12383 */
  0x0020,   /* U+12384 */
  0x0020,   /* U+12385 */
  0x0020,   /* U+12386 */
  0x0020,   /* U+12387 */
  0x0020,   /* U+12388 */
  0x0020,   /* U+12389 */
  0x0020,   /* U+1238A */
  0x0020,   /* U+1238B */
  0x0020,   /* U+1238C */
  0x0020,   /* U+1238D */
  0x0020,   /* U+1238E */
  0x0020,   /* U+1238F */
  0x0020,   /* U+12390 */
  0x0020,   /* U+12391 */
  0x0020,   /* U+12392 */
  0x0020,   /* U+12393 */
  0x0020,   /* U+12394 */
  0x0020,   /* U+12395 */
  0x0020,   /* U+12396 */
  0x0020,   /* U+12397 */
  0x0020,   /* U+12398 */
  0x0020,   /* U+12399 */
  0x0020,   /* U+1239A */
  0x0020,   /* U+1239B */
  0x0020,   /* U+1239C */
  0x0020,   /* U+1239D */
  0x0020,   /* U+1239E */
  0x0020,   /* U+1239F */
  0x0020,   /* U+123A0 */
  0x0020,   /* U+123A1 */
  0x0020,   /* U+123A2 */
  0x0020,   /* U+123A3 */
  0x0020,   /* U+123A4 */
  0x0020,   /* U+123A5 */
  0x0020,   /* U+123A6 */
  0x0020,   /* U+123A7 */
  0x0020,   /* U+123A8 */
  0x0020,   /* U+123A9 */
  0x0020,   /* U+123AA */
  0x0020,   /* U+123AB */
  0x0020,   /* U+123AC */
  0x0020,   /* U+123AD */
  0x0020,   /* U+123AE */
  0x0020,   /* U+123AF */
  0x0020,   /* U+123B0 */
  0x0020,   /* U+123B1 */
  0x0020,   /* U+123B2 */
  0x0020,   /* U+123B3 */
  0x0020,   /* U+123B4 */
  0x0020,   /* U+123B5 */
  0x0020,   /* U+123B6 */
  0x0020,   /* U+123B7 */
  0x0020,   /* U+123B8 */
  0x0020,   /* U+123B9 */
  0x0020,   /* U+123BA */
  0x0020,   /* U+123BB */
  0x0020,   /* U+123BC */
  0x0020,   /* U+123BD */
  0x0020,   /* U+123BE */
  0x0020,   /* U+123BF */
  0x0020,   /* U+123C0 */
  0x0020,   /* U+123C1 */
  0x0020,   /* U+123C2 */
  0x0020,   /* U+123C3 */
  0x0020,   /* U+123C4 */
  0x0020,   /* U+123C5 */
  0x0020,   /* U+123C6 */
  0x0020,   /* U+123C7 */
  0x0020,   /* U+123C8 */
  0x0020,   /* U+123C9 */
  0x0020,   /* U+123CA */
  0x0020,   /* U+123CB */
  0x0020,   /* U+123CC */
  0x0020,   /* U+123CD */
  0x0020,   /* U+123CE */
  0x0020,   /* U+123CF */
  0x0020,   /* U+123D0 */
  0x0020,   /* U+123D1 */
  0x0020,   /* U+123D2 */
  0x0020,   /* U+123D3 */
  0x0020,   /* U+123D4 */
  0x0020,   /* U+123D5 */
  0x0020,   /* U+123D6 */
  0x0020,   /* U+123D7 */
  0x0020,   /* U+123D8 */
  0x0020,   /* U+123D9 */
  0x0020,   /* U+123DA */
  0x0020,   /* U+123DB */
  0x0020,   /* U+123DC */
  0x0020,   /* U+123DD */
  0x0020,   /* U+123DE */
  0x0020,   /* U+123DF */
  0x0020,   /* U+123E0 */
  0x0020,   /* U+123E1 */
  0x0020,   /* U+123E2 */
  0x0020,   /* U+123E3 */
  0x0020,   /* U+123E4 */
  0x0020,   /* U+123E5 */
  0x0020,   /* U+123E6 */
  0x0020,   /* U+123E7 */
  0x0020,   /* U+123E8 */
  0x0020,   /* U+123E9 */
  0x0020,   /* U+123EA */
  0x0020,   /* U+123EB */
  0x0020,   /* U+123EC */
  0x0020,   /* U+123ED */
  0x0020,   /* U+123EE */
  0x0020,   /* U+123EF */
  0x0020,   /* U+123F0 */
  0x0020,   /* U+123F1 */
  0x0020,   /* U+123F2 */
  0x0020,   /* U+123F3 */
  0x0020,   /* U+123F4 */
  0x0020,   /* U+123F5 */
  0x0020,   /* U+123F6 */
  0x0020,   /* U+123F7 */
  0x0020,   /* U+123F8 */
  0x0020,   /* U+123F9 */
  0x0020,   /* U+123FA */
  0x0020,   /* U+123FB */
  0x0020,   /* U+123FC */
  0x0020,   /* U+123FD */
  0x0020,   /* U+123FE */
  0x0020,   /* U+123FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12300 */
  0x0002,   /* U+12301 */
  0x0002,   /* U+12302 */
  0x0002,   /* U+12303 */
  0x0002,   /* U+12304 */
  0x0002,   /* U+12305 */
  0x0002,   /* U+12306 */
  0x0002,   /* U+12307 */
  0x0002,   /* U+12308 */
  0x0002,   /* U+12309 */
  0x0002,   /* U+1230A */
  0x0002,   /* U+1230B */
  0x0002,   /* U+1230C */
  0x0002,   /* U+1230D */
  0x0002,   /* U+1230E */
  0x0002,   /* U+1230F */
  0x0002,   /* U+12310 */
  0x0002,   /* U+12311 */
  0x0002,   /* U+12312 */
  0x0002,   /* U+12313 */
  0x0002,   /* U+12314 */
  0x0002,   /* U+12315 */
  0x0002,   /* U+12316 */
  0x0002,   /* U+12317 */
  0x0002,   /* U+12318 */
  0x0002,   /* U+12319 */
  0x0002,   /* U+1231A */
  0x0002,   /* U+1231B */
  0x0002,   /* U+1231C */
  0x0002,   /* U+1231D */
  0x0002,   /* U+1231E */
  0x0002,   /* U+1231F */
  0x0002,   /* U+12320 */
  0x0002,   /* U+12321 */
  0x0002,   /* U+12322 */
  0x0002,   /* U+12323 */
  0x0002,   /* U+12324 */
  0x0002,   /* U+12325 */
  0x0002,   /* U+12326 */
  0x0002,   /* U+12327 */
  0x0002,   /* U+12328 */
  0x0002,   /* U+12329 */
  0x0002,   /* U+1232A */
  0x0002,   /* U+1232B */
  0x0002,   /* U+1232C */
  0x0002,   /* U+1232D */
  0x0002,   /* U+1232E */
  0x0002,   /* U+1232F */
  0x0002,   /* U+12330 */
  0x0002,   /* U+12331 */
  0x0002,   /* U+12332 */
  0x0002,   /* U+12333 */
  0x0002,   /* U+12334 */
  0x0002,   /* U+12335 */
  0x0002,   /* U+12336 */
  0x0002,   /* U+12337 */
  0x0002,   /* U+12338 */
  0x0002,   /* U+12339 */
  0x0002,   /* U+1233A */
  0x0002,   /* U+1233B */
  0x0002,   /* U+1233C */
  0x0002,   /* U+1233D */
  0x0002,   /* U+1233E */
  0x0002,   /* U+1233F */
  0x0002,   /* U+12340 */
  0x0002,   /* U+12341 */
  0x0002,   /* U+12342 */
  0x0002,   /* U+12343 */
  0x0002,   /* U+12344 */
  0x0002,   /* U+12345 */
  0x0002,   /* U+12346 */
  0x0002,   /* U+12347 */
  0x0002,   /* U+12348 */
  0x0002,   /* U+12349 */
  0x0002,   /* U+1234A */
  0x0002,   /* U+1234B */
  0x0002,   /* U+1234C */
  0x0002,   /* U+1234D */
  0x0002,   /* U+1234E */
  0x0002,   /* U+1234F */
  0x0002,   /* U+12350 */
  0x0002,   /* U+12351 */
  0x0002,   /* U+12352 */
  0x0002,   /* U+12353 */
  0x0002,   /* U+12354 */
  0x0002,   /* U+12355 */
  0x0002,   /* U+12356 */
  0x0002,   /* U+12357 */
  0x0002,   /* U+12358 */
  0x0002,   /* U+12359 */
  0x0002,   /* U+1235A */
  0x0002,   /* U+1235B */
  0x0002,   /* U+1235C */
  0x0002,   /* U+1235D */
  0x0002,   /* U+1235E */
  0x0002,   /* U+1235F */
  0x0002,   /* U+12360 */
  0x0002,   /* U+12361 */
  0x0002,   /* U+12362 */
  0x0002,   /* U+12363 */
  0x0002,   /* U+12364 */
  0x0002,   /* U+12365 */
  0x0002,   /* U+12366 */
  0x0002,   /* U+12367 */
  0x0002,   /* U+12368 */
  0x0002,   /* U+12369 */
  0x0002,   /* U+1236A */
  0x0002,   /* U+1236B */
  0x0002,   /* U+1236C */
  0x0002,   /* U+1236D */
  0x0002,   /* U+1236E */
  0x0002,   /* U+1236F */
  0x0002,   /* U+12370 */
  0x0002,   /* U+12371 */
  0x0002,   /* U+12372 */
  0x0002,   /* U+12373 */
  0x0002,   /* U+12374 */
  0x0002,   /* U+12375 */
  0x0002,   /* U+12376 */
  0x0002,   /* U+12377 */
  0x0002,   /* U+12378 */
  0x0002,   /* U+12379 */
  0x0002,   /* U+1237A */
  0x0002,   /* U+1237B */
  0x0002,   /* U+1237C */
  0x0002,   /* U+1237D */
  0x0002,   /* U+1237E */
  0x0002,   /* U+1237F */
  0x0002,   /* U+12380 */
  0x0002,   /* U+12381 */
  0x0002,   /* U+12382 */
  0x0002,   /* U+12383 */
  0x0002,   /* U+12384 */
  0x0002,   /* U+12385 */
  0x0002,   /* U+12386 */
  0x0002,   /* U+12387 */
  0x0002,   /* U+12388 */
  0x0002,   /* U+12389 */
  0x0002,   /* U+1238A */
  0x0002,   /* U+1238B */
  0x0002,   /* U+1238C */
  0x0002,   /* U+1238D */
  0x0002,   /* U+1238E */
  0x0002,   /* U+1238F */
  0x0002,   /* U+12390 */
  0x0002,   /* U+12391 */
  0x0002,   /* U+12392 */
  0x0002,   /* U+12393 */
  0x0002,   /* U+12394 */
  0x0002,   /* U+12395 */
  0x0002,   /* U+12396 */
  0x0002,   /* U+12397 */
  0x0002,   /* U+12398 */
  0x0002,   /* U+12399 */
  0x0002,   /* U+1239A */
  0x0002,   /* U+1239B */
  0x0002,   /* U+1239C */
  0x0002,   /* U+1239D */
  0x0002,   /* U+1239E */
  0x0002,   /* U+1239F */
  0x0002,   /* U+123A0 */
  0x0002,   /* U+123A1 */
  0x0002,   /* U+123A2 */
  0x0002,   /* U+123A3 */
  0x0002,   /* U+123A4 */
  0x0002,   /* U+123A5 */
  0x0002,   /* U+123A6 */
  0x0002,   /* U+123A7 */
  0x0002,   /* U+123A8 */
  0x0002,   /* U+123A9 */
  0x0002,   /* U+123AA */
  0x0002,   /* U+123AB */
  0x0002,   /* U+123AC */
  0x0002,   /* U+123AD */
  0x0002,   /* U+123AE */
  0x0002,   /* U+123AF */
  0x0002,   /* U+123B0 */
  0x0002,   /* U+123B1 */
  0x0002,   /* U+123B2 */
  0x0002,   /* U+123B3 */
  0x0002,   /* U+123B4 */
  0x0002,   /* U+123B5 */
  0x0002,   /* U+123B6 */
  0x0002,   /* U+123B7 */
  0x0002,   /* U+123B8 */
  0x0002,   /* U+123B9 */
  0x0002,   /* U+123BA */
  0x0002,   /* U+123BB */
  0x0002,   /* U+123BC */
  0x0002,   /* U+123BD */
  0x0002,   /* U+123BE */
  0x0002,   /* U+123BF */
  0x0002,   /* U+123C0 */
  0x0002,   /* U+123C1 */
  0x0002,   /* U+123C2 */
  0x0002,   /* U+123C3 */
  0x0002,   /* U+123C4 */
  0x0002,   /* U+123C5 */
  0x0002,   /* U+123C6 */
  0x0002,   /* U+123C7 */
  0x0002,   /* U+123C8 */
  0x0002,   /* U+123C9 */
  0x0002,   /* U+123CA */
  0x0002,   /* U+123CB */
  0x0002,   /* U+123CC */
  0x0002,   /* U+123CD */
  0x0002,   /* U+123CE */
  0x0002,   /* U+123CF */
  0x0002,   /* U+123D0 */
  0x0002,   /* U+123D1 */
  0x0002,   /* U+123D2 */
  0x0002,   /* U+123D3 */
  0x0002,   /* U+123D4 */
  0x0002,   /* U+123D5 */
  0x0002,   /* U+123D6 */
  0x0002,   /* U+123D7 */
  0x0002,   /* U+123D8 */
  0x0002,   /* U+123D9 */
  0x0002,   /* U+123DA */
  0x0002,   /* U+123DB */
  0x0002,   /* U+123DC */
  0x0002,   /* U+123DD */
  0x0002,   /* U+123DE */
  0x0002,   /* U+123DF */
  0x0002,   /* U+123E0 */
  0x0002,   /* U+123E1 */
  0x0002,   /* U+123E2 */
  0x0002,   /* U+123E3 */
  0x0002,   /* U+123E4 */
  0x0002,   /* U+123E5 */
  0x0002,   /* U+123E6 */
  0x0002,   /* U+123E7 */
  0x0002,   /* U+123E8 */
  0x0002,   /* U+123E9 */
  0x0002,   /* U+123EA */
  0x0002,   /* U+123EB */
  0x0002,   /* U+123EC */
  0x0002,   /* U+123ED */
  0x0002,   /* U+123EE */
  0x0002,   /* U+123EF */
  0x0002,   /* U+123F0 */
  0x0002,   /* U+123F1 */
  0x0002,   /* U+123F2 */
  0x0002,   /* U+123F3 */
  0x0002,   /* U+123F4 */
  0x0002,   /* U+123F5 */
  0x0002,   /* U+123F6 */
  0x0002,   /* U+123F7 */
  0x0002,   /* U+123F8 */
  0x0002,   /* U+123F9 */
  0x0002,   /* U+123FA */
  0x0002,   /* U+123FB */
  0x0002,   /* U+123FC */
  0x0002,   /* U+123FD */
  0x0002,   /* U+123FE */
  0x0002,   /* U+123FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+12300 */
  0x0000,   /* U+12301 */
  0x0000,   /* U+12302 */
  0x0000,   /* U+12303 */
  0x0000,   /* U+12304 */
  0x0000,   /* U+12305 */
  0x0000,   /* U+12306 */
  0x0000,   /* U+12307 */
  0x0000,   /* U+12308 */
  0x0000,   /* U+12309 */
  0x0000,   /* U+1230A */
  0x0000,   /* U+1230B */
  0x0000,   /* U+1230C */
  0x0000,   /* U+1230D */
  0x0000,   /* U+1230E */
  0x0000,   /* U+1230F */
  0x0000,   /* U+12310 */
  0x0000,   /* U+12311 */
  0x0000,   /* U+12312 */
  0x0000,   /* U+12313 */
  0x0000,   /* U+12314 */
  0x0000,   /* U+12315 */
  0x0000,   /* U+12316 */
  0x0000,   /* U+12317 */
  0x0000,   /* U+12318 */
  0x0000,   /* U+12319 */
  0x0000,   /* U+1231A */
  0x0000,   /* U+1231B */
  0x0000,   /* U+1231C */
  0x0000,   /* U+1231D */
  0x0000,   /* U+1231E */
  0x0000,   /* U+1231F */
  0x0000,   /* U+12320 */
  0x0000,   /* U+12321 */
  0x0000,   /* U+12322 */
  0x0000,   /* U+12323 */
  0x0000,   /* U+12324 */
  0x0000,   /* U+12325 */
  0x0000,   /* U+12326 */
  0x0000,   /* U+12327 */
  0x0000,   /* U+12328 */
  0x0000,   /* U+12329 */
  0x0000,   /* U+1232A */
  0x0000,   /* U+1232B */
  0x0000,   /* U+1232C */
  0x0000,   /* U+1232D */
  0x0000,   /* U+1232E */
  0x0000,   /* U+1232F */
  0x0000,   /* U+12330 */
  0x0000,   /* U+12331 */
  0x0000,   /* U+12332 */
  0x0000,   /* U+12333 */
  0x0000,   /* U+12334 */
  0x0000,   /* U+12335 */
  0x0000,   /* U+12336 */
  0x0000,   /* U+12337 */
  0x0000,   /* U+12338 */
  0x0000,   /* U+12339 */
  0x0000,   /* U+1233A */
  0x0000,   /* U+1233B */
  0x0000,   /* U+1233C */
  0x0000,   /* U+1233D */
  0x0000,   /* U+1233E */
  0x0000,   /* U+1233F */
  0x0000,   /* U+12340 */
  0x0000,   /* U+12341 */
  0x0000,   /* U+12342 */
  0x0000,   /* U+12343 */
  0x0000,   /* U+12344 */
  0x0000,   /* U+12345 */
  0x0000,   /* U+12346 */
  0x0000,   /* U+12347 */
  0x0000,   /* U+12348 */
  0x0000,   /* U+12349 */
  0x0000,   /* U+1234A */
  0x0000,   /* U+1234B */
  0x0000,   /* U+1234C */
  0x0000,   /* U+1234D */
  0x0000,   /* U+1234E */
  0x0000,   /* U+1234F */
  0x0000,   /* U+12350 */
  0x0000,   /* U+12351 */
  0x0000,   /* U+12352 */
  0x0000,   /* U+12353 */
  0x0000,   /* U+12354 */
  0x0000,   /* U+12355 */
  0x0000,   /* U+12356 */
  0x0000,   /* U+12357 */
  0x0000,   /* U+12358 */
  0x0000,   /* U+12359 */
  0x0000,   /* U+1235A */
  0x0000,   /* U+1235B */
  0x0000,   /* U+1235C */
  0x0000,   /* U+1235D */
  0x0000,   /* U+1235E */
  0x0000,   /* U+1235F */
  0x0000,   /* U+12360 */
  0x0000,   /* U+12361 */
  0x0000,   /* U+12362 */
  0x0000,   /* U+12363 */
  0x0000,   /* U+12364 */
  0x0000,   /* U+12365 */
  0x0000,   /* U+12366 */
  0x0000,   /* U+12367 */
  0x0000,   /* U+12368 */
  0x0000,   /* U+12369 */
  0x0000,   /* U+1236A */
  0x0000,   /* U+1236B */
  0x0000,   /* U+1236C */
  0x0000,   /* U+1236D */
  0x0000,   /* U+1236E */
  0x0000,   /* U+1236F */
  0x0000,   /* U+12370 */
  0x0000,   /* U+12371 */
  0x0000,   /* U+12372 */
  0x0000,   /* U+12373 */
  0x0000,   /* U+12374 */
  0x0000,   /* U+12375 */
  0x0000,   /* U+12376 */
  0x0000,   /* U+12377 */
  0x0000,   /* U+12378 */
  0x0000,   /* U+12379 */
  0x0000,   /* U+1237A */
  0x0000,   /* U+1237B */
  0x0000,   /* U+1237C */
  0x0000,   /* U+1237D */
  0x0000,   /* U+1237E */
  0x0000,   /* U+1237F */
  0x0000,   /* U+12380 */
  0x0000,   /* U+12381 */
  0x0000,   /* U+12382 */
  0x0000,   /* U+12383 */
  0x0000,   /* U+12384 */
  0x0000,   /* U+12385 */
  0x0000,   /* U+12386 */
  0x0000,   /* U+12387 */
  0x0000,   /* U+12388 */
  0x0000,   /* U+12389 */
  0x0000,   /* U+1238A */
  0x0000,   /* U+1238B */
  0x0000,   /* U+1238C */
  0x0000,   /* U+1238D */
  0x0000,   /* U+1238E */
  0x0000,   /* U+1238F */
  0x0000,   /* U+12390 */
  0x0000,   /* U+12391 */
  0x0000,   /* U+12392 */
  0x0000,   /* U+12393 */
  0x0000,   /* U+12394 */
  0x0000,   /* U+12395 */
  0x0000,   /* U+12396 */
  0x0000,   /* U+12397 */
  0x0000,   /* U+12398 */
  0x0000,   /* U+12399 */
  0xA39A,   /* U+1239A */
  0xA39B,   /* U+1239B */
  0xA39C,   /* U+1239C */
  0xA39D,   /* U+1239D */
  0xA39E,   /* U+1239E */
  0xA39F,   /* U+1239F */
  0xA3A0,   /* U+123A0 */
  0xA3A1,   /* U+123A1 */
  0xA3A2,   /* U+123A2 */
  0xA3A3,   /* U+123A3 */
  0xA3A4,   /* U+123A4 */
  0xA3A5,   /* U+123A5 */
  0xA3A6,   /* U+123A6 */
  0xA3A7,   /* U+123A7 */
  0xA3A8,   /* U+123A8 */
  0xA3A9,   /* U+123A9 */
  0xA3AA,   /* U+123AA */
  0xA3AB,   /* U+123AB */
  0xA3AC,   /* U+123AC */
  0xA3AD,   /* U+123AD */
  0xA3AE,   /* U+123AE */
  0xA3AF,   /* U+123AF */
  0xA3B0,   /* U+123B0 */
  0xA3B1,   /* U+123B1 */
  0xA3B2,   /* U+123B2 */
  0xA3B3,   /* U+123B3 */
  0xA3B4,   /* U+123B4 */
  0xA3B5,   /* U+123B5 */
  0xA3B6,   /* U+123B6 */
  0xA3B7,   /* U+123B7 */
  0xA3B8,   /* U+123B8 */
  0xA3B9,   /* U+123B9 */
  0xA3BA,   /* U+123BA */
  0xA3BB,   /* U+123BB */
  0xA3BC,   /* U+123BC */
  0xA3BD,   /* U+123BD */
  0xA3BE,   /* U+123BE */
  0xA3BF,   /* U+123BF */
  0xA3C0,   /* U+123C0 */
  0xA3C1,   /* U+123C1 */
  0xA3C2,   /* U+123C2 */
  0xA3C3,   /* U+123C3 */
  0xA3C4,   /* U+123C4 */
  0xA3C5,   /* U+123C5 */
  0xA3C6,   /* U+123C6 */
  0xA3C7,   /* U+123C7 */
  0xA3C8,   /* U+123C8 */
  0xA3C9,   /* U+123C9 */
  0xA3CA,   /* U+123CA */
  0xA3CB,   /* U+123CB */
  0xA3CC,   /* U+123CC */
  0xA3CD,   /* U+123CD */
  0xA3CE,   /* U+123CE */
  0xA3CF,   /* U+123CF */
  0xA3D0,   /* U+123D0 */
  0xA3D1,   /* U+123D1 */
  0xA3D2,   /* U+123D2 */
  0xA3D3,   /* U+123D3 */
  0xA3D4,   /* U+123D4 */
  0xA3D5,   /* U+123D5 */
  0xA3D6,   /* U+123D6 */
  0xA3D7,   /* U+123D7 */
  0xA3D8,   /* U+123D8 */
  0xA3D9,   /* U+123D9 */
  0xA3DA,   /* U+123DA */
  0xA3DB,   /* U+123DB */
  0xA3DC,   /* U+123DC */
  0xA3DD,   /* U+123DD */
  0xA3DE,   /* U+123DE */
  0xA3DF,   /* U+123DF */
  0xA3E0,   /* U+123E0 */
  0xA3E1,   /* U+123E1 */
  0xA3E2,   /* U+123E2 */
  0xA3E3,   /* U+123E3 */
  0xA3E4,   /* U+123E4 */
  0xA3E5,   /* U+123E5 */
  0xA3E6,   /* U+123E6 */
  0xA3E7,   /* U+123E7 */
  0xA3E8,   /* U+123E8 */
  0xA3E9,   /* U+123E9 */
  0xA3EA,   /* U+123EA */
  0xA3EB,   /* U+123EB */
  0xA3EC,   /* U+123EC */
  0xA3ED,   /* U+123ED */
  0xA3EE,   /* U+123EE */
  0xA3EF,   /* U+123EF */
  0xA3F0,   /* U+123F0 */
  0xA3F1,   /* U+123F1 */
  0xA3F2,   /* U+123F2 */
  0xA3F3,   /* U+123F3 */
  0xA3F4,   /* U+123F4 */
  0xA3F5,   /* U+123F5 */
  0xA3F6,   /* U+123F6 */
  0xA3F7,   /* U+123F7 */
  0xA3F8,   /* U+123F8 */
  0xA3F9,   /* U+123F9 */
  0xA3FA,   /* U+123FA */
  0xA3FB,   /* U+123FB */
  0xA3FC,   /* U+123FC */
  0xA3FD,   /* U+123FD */
  0xA3FE,   /* U+123FE */
  0xA3FF,   /* U+123FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+12300 */
  0x0000,   /* U+12301 */
  0x0000,   /* U+12302 */
  0x0000,   /* U+12303 */
  0x0000,   /* U+12304 */
  0x0000,   /* U+12305 */
  0x0000,   /* U+12306 */
  0x0000,   /* U+12307 */
  0x0000,   /* U+12308 */
  0x0000,   /* U+12309 */
  0x0000,   /* U+1230A */
  0x0000,   /* U+1230B */
  0x0000,   /* U+1230C */
  0x0000,   /* U+1230D */
  0x0000,   /* U+1230E */
  0x0000,   /* U+1230F */
  0x0000,   /* U+12310 */
  0x0000,   /* U+12311 */
  0x0000,   /* U+12312 */
  0x0000,   /* U+12313 */
  0x0000,   /* U+12314 */
  0x0000,   /* U+12315 */
  0x0000,   /* U+12316 */
  0x0000,   /* U+12317 */
  0x0000,   /* U+12318 */
  0x0000,   /* U+12319 */
  0x0000,   /* U+1231A */
  0x0000,   /* U+1231B */
  0x0000,   /* U+1231C */
  0x0000,   /* U+1231D */
  0x0000,   /* U+1231E */
  0x0000,   /* U+1231F */
  0x0000,   /* U+12320 */
  0x0000,   /* U+12321 */
  0x0000,   /* U+12322 */
  0x0000,   /* U+12323 */
  0x0000,   /* U+12324 */
  0x0000,   /* U+12325 */
  0x0000,   /* U+12326 */
  0x0000,   /* U+12327 */
  0x0000,   /* U+12328 */
  0x0000,   /* U+12329 */
  0x0000,   /* U+1232A */
  0x0000,   /* U+1232B */
  0x0000,   /* U+1232C */
  0x0000,   /* U+1232D */
  0x0000,   /* U+1232E */
  0x0000,   /* U+1232F */
  0x0000,   /* U+12330 */
  0x0000,   /* U+12331 */
  0x0000,   /* U+12332 */
  0x0000,   /* U+12333 */
  0x0000,   /* U+12334 */
  0x0000,   /* U+12335 */
  0x0000,   /* U+12336 */
  0x0000,   /* U+12337 */
  0x0000,   /* U+12338 */
  0x0000,   /* U+12339 */
  0x0000,   /* U+1233A */
  0x0000,   /* U+1233B */
  0x0000,   /* U+1233C */
  0x0000,   /* U+1233D */
  0x0000,   /* U+1233E */
  0x0000,   /* U+1233F */
  0x0000,   /* U+12340 */
  0x0000,   /* U+12341 */
  0x0000,   /* U+12342 */
  0x0000,   /* U+12343 */
  0x0000,   /* U+12344 */
  0x0000,   /* U+12345 */
  0x0000,   /* U+12346 */
  0x0000,   /* U+12347 */
  0x0000,   /* U+12348 */
  0x0000,   /* U+12349 */
  0x0000,   /* U+1234A */
  0x0000,   /* U+1234B */
  0x0000,   /* U+1234C */
  0x0000,   /* U+1234D */
  0x0000,   /* U+1234E */
  0x0000,   /* U+1234F */
  0x0000,   /* U+12350 */
  0x0000,   /* U+12351 */
  0x0000,   /* U+12352 */
  0x0000,   /* U+12353 */
  0x0000,   /* U+12354 */
  0x0000,   /* U+12355 */
  0x0000,   /* U+12356 */
  0x0000,   /* U+12357 */
  0x0000,   /* U+12358 */
  0x0000,   /* U+12359 */
  0x0000,   /* U+1235A */
  0x0000,   /* U+1235B */
  0x0000,   /* U+1235C */
  0x0000,   /* U+1235D */
  0x0000,   /* U+1235E */
  0x0000,   /* U+1235F */
  0x0000,   /* U+12360 */
  0x0000,   /* U+12361 */
  0x0000,   /* U+12362 */
  0x0000,   /* U+12363 */
  0x0000,   /* U+12364 */
  0x0000,   /* U+12365 */
  0x0000,   /* U+12366 */
  0x0000,   /* U+12367 */
  0x0000,   /* U+12368 */
  0x0000,   /* U+12369 */
  0x0000,   /* U+1236A */
  0x0000,   /* U+1236B */
  0x0000,   /* U+1236C */
  0x0000,   /* U+1236D */
  0x0000,   /* U+1236E */
  0x0000,   /* U+1236F */
  0x0000,   /* U+12370 */
  0x0000,   /* U+12371 */
  0x0000,   /* U+12372 */
  0x0000,   /* U+12373 */
  0x0000,   /* U+12374 */
  0x0000,   /* U+12375 */
  0x0000,   /* U+12376 */
  0x0000,   /* U+12377 */
  0x0000,   /* U+12378 */
  0x0000,   /* U+12379 */
  0x0000,   /* U+1237A */
  0x0000,   /* U+1237B */
  0x0000,   /* U+1237C */
  0x0000,   /* U+1237D */
  0x0000,   /* U+1237E */
  0x0000,   /* U+1237F */
  0x0000,   /* U+12380 */
  0x0000,   /* U+12381 */
  0x0000,   /* U+12382 */
  0x0000,   /* U+12383 */
  0x0000,   /* U+12384 */
  0x0000,   /* U+12385 */
  0x0000,   /* U+12386 */
  0x0000,   /* U+12387 */
  0x0000,   /* U+12388 */
  0x0000,   /* U+12389 */
  0x0000,   /* U+1238A */
  0x0000,   /* U+1238B */
  0x0000,   /* U+1238C */
  0x0000,   /* U+1238D */
  0x0000,   /* U+1238E */
  0x0000,   /* U+1238F */
  0x0000,   /* U+12390 */
  0x0000,   /* U+12391 */
  0x0000,   /* U+12392 */
  0x0000,   /* U+12393 */
  0x0000,   /* U+12394 */
  0x0000,   /* U+12395 */
  0x0000,   /* U+12396 */
  0x0000,   /* U+12397 */
  0x0000,   /* U+12398 */
  0x0000,   /* U+12399 */
  0x0000,   /* U+1239A */
  0x0000,   /* U+1239B */
  0x0000,   /* U+1239C */
  0x0000,   /* U+1239D */
  0x0000,   /* U+1239E */
  0x0000,   /* U+1239F */
  0x0000,   /* U+123A0 */
  0x0000,   /* U+123A1 */
  0x0000,   /* U+123A2 */
  0x0000,   /* U+123A3 */
  0x0000,   /* U+123A4 */
  0x0000,   /* U+123A5 */
  0x0000,   /* U+123A6 */
  0x0000,   /* U+123A7 */
  0x0000,   /* U+123A8 */
  0x0000,   /* U+123A9 */
  0x0000,   /* U+123AA */
  0x0000,   /* U+123AB */
  0x0000,   /* U+123AC */
  0x0000,   /* U+123AD */
  0x0000,   /* U+123AE */
  0x0000,   /* U+123AF */
  0x0000,   /* U+123B0 */
  0x0000,   /* U+123B1 */
  0x0000,   /* U+123B2 */
  0x0000,   /* U+123B3 */
  0x0000,   /* U+123B4 */
  0x0000,   /* U+123B5 */
  0x0000,   /* U+123B6 */
  0x0000,   /* U+123B7 */
  0x0000,   /* U+123B8 */
  0x0000,   /* U+123B9 */
  0x0000,   /* U+123BA */
  0x0000,   /* U+123BB */
  0x0000,   /* U+123BC */
  0x0000,   /* U+123BD */
  0x0000,   /* U+123BE */
  0x0000,   /* U+123BF */
  0x0000,   /* U+123C0 */
  0x0000,   /* U+123C1 */
  0x0000,   /* U+123C2 */
  0x0000,   /* U+123C3 */
  0x0000,   /* U+123C4 */
  0x0000,   /* U+123C5 */
  0x0000,   /* U+123C6 */
  0x0000,   /* U+123C7 */
  0x0000,   /* U+123C8 */
  0x0000,   /* U+123C9 */
  0x0000,   /* U+123CA */
  0x0000,   /* U+123CB */
  0x0000,   /* U+123CC */
  0x0000,   /* U+123CD */
  0x0000,   /* U+123CE */
  0x0000,   /* U+123CF */
  0x0000,   /* U+123D0 */
  0x0000,   /* U+123D1 */
  0x0000,   /* U+123D2 */
  0x0000,   /* U+123D3 */
  0x0000,   /* U+123D4 */
  0x0000,   /* U+123D5 */
  0x0000,   /* U+123D6 */
  0x0000,   /* U+123D7 */
  0x0000,   /* U+123D8 */
  0x0000,   /* U+123D9 */
  0x0000,   /* U+123DA */
  0x0000,   /* U+123DB */
  0x0000,   /* U+123DC */
  0x0000,   /* U+123DD */
  0x0000,   /* U+123DE */
  0x0000,   /* U+123DF */
  0x0000,   /* U+123E0 */
  0x0000,   /* U+123E1 */
  0x0000,   /* U+123E2 */
  0x0000,   /* U+123E3 */
  0x0000,   /* U+123E4 */
  0x0000,   /* U+123E5 */
  0x0000,   /* U+123E6 */
  0x0000,   /* U+123E7 */
  0x0000,   /* U+123E8 */
  0x0000,   /* U+123E9 */
  0x0000,   /* U+123EA */
  0x0000,   /* U+123EB */
  0x0000,   /* U+123EC */
  0x0000,   /* U+123ED */
  0x0000,   /* U+123EE */
  0x0000,   /* U+123EF */
  0x0000,   /* U+123F0 */
  0x0000,   /* U+123F1 */
  0x0000,   /* U+123F2 */
  0x0000,   /* U+123F3 */
  0x0000,   /* U+123F4 */
  0x0000,   /* U+123F5 */
  0x0000,   /* U+123F6 */
  0x0000,   /* U+123F7 */
  0x0000,   /* U+123F8 */
  0x0000,   /* U+123F9 */
  0x0000,   /* U+123FA */
  0x0000,   /* U+123FB */
  0x0000,   /* U+123FC */
  0x0000,   /* U+123FD */
  0x0000,   /* U+123FE */
  0x0000,   /* U+123FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+12300 */
  0x0000,   /* U+12301 */
  0x0000,   /* U+12302 */
  0x0000,   /* U+12303 */
  0x0000,   /* U+12304 */
  0x0000,   /* U+12305 */
  0x0000,   /* U+12306 */
  0x0000,   /* U+12307 */
  0x0000,   /* U+12308 */
  0x0000,   /* U+12309 */
  0x0000,   /* U+1230A */
  0x0000,   /* U+1230B */
  0x0000,   /* U+1230C */
  0x0000,   /* U+1230D */
  0x0000,   /* U+1230E */
  0x0000,   /* U+1230F */
  0x0000,   /* U+12310 */
  0x0000,   /* U+12311 */
  0x0000,   /* U+12312 */
  0x0000,   /* U+12313 */
  0x0000,   /* U+12314 */
  0x0000,   /* U+12315 */
  0x0000,   /* U+12316 */
  0x0000,   /* U+12317 */
  0x0000,   /* U+12318 */
  0x0000,   /* U+12319 */
  0x0000,   /* U+1231A */
  0x0000,   /* U+1231B */
  0x0000,   /* U+1231C */
  0x0000,   /* U+1231D */
  0x0000,   /* U+1231E */
  0x0000,   /* U+1231F */
  0x0000,   /* U+12320 */
  0x0000,   /* U+12321 */
  0x0000,   /* U+12322 */
  0x0000,   /* U+12323 */
  0x0000,   /* U+12324 */
  0x0000,   /* U+12325 */
  0x0000,   /* U+12326 */
  0x0000,   /* U+12327 */
  0x0000,   /* U+12328 */
  0x0000,   /* U+12329 */
  0x0000,   /* U+1232A */
  0x0000,   /* U+1232B */
  0x0000,   /* U+1232C */
  0x0000,   /* U+1232D */
  0x0000,   /* U+1232E */
  0x0000,   /* U+1232F */
  0x0000,   /* U+12330 */
  0x0000,   /* U+12331 */
  0x0000,   /* U+12332 */
  0x0000,   /* U+12333 */
  0x0000,   /* U+12334 */
  0x0000,   /* U+12335 */
  0x0000,   /* U+12336 */
  0x0000,   /* U+12337 */
  0x0000,   /* U+12338 */
  0x0000,   /* U+12339 */
  0x0000,   /* U+1233A */
  0x0000,   /* U+1233B */
  0x0000,   /* U+1233C */
  0x0000,   /* U+1233D */
  0x0000,   /* U+1233E */
  0x0000,   /* U+1233F */
  0x0000,   /* U+12340 */
  0x0000,   /* U+12341 */
  0x0000,   /* U+12342 */
  0x0000,   /* U+12343 */
  0x0000,   /* U+12344 */
  0x0000,   /* U+12345 */
  0x0000,   /* U+12346 */
  0x0000,   /* U+12347 */
  0x0000,   /* U+12348 */
  0x0000,   /* U+12349 */
  0x0000,   /* U+1234A */
  0x0000,   /* U+1234B */
  0x0000,   /* U+1234C */
  0x0000,   /* U+1234D */
  0x0000,   /* U+1234E */
  0x0000,   /* U+1234F */
  0x0000,   /* U+12350 */
  0x0000,   /* U+12351 */
  0x0000,   /* U+12352 */
  0x0000,   /* U+12353 */
  0x0000,   /* U+12354 */
  0x0000,   /* U+12355 */
  0x0000,   /* U+12356 */
  0x0000,   /* U+12357 */
  0x0000,   /* U+12358 */
  0x0000,   /* U+12359 */
  0x0000,   /* U+1235A */
  0x0000,   /* U+1235B */
  0x0000,   /* U+1235C */
  0x0000,   /* U+1235D */
  0x0000,   /* U+1235E */
  0x0000,   /* U+1235F */
  0x0000,   /* U+12360 */
  0x0000,   /* U+12361 */
  0x0000,   /* U+12362 */
  0x0000,   /* U+12363 */
  0x0000,   /* U+12364 */
  0x0000,   /* U+12365 */
  0x0000,   /* U+12366 */
  0x0000,   /* U+12367 */
  0x0000,   /* U+12368 */
  0x0000,   /* U+12369 */
  0x0000,   /* U+1236A */
  0x0000,   /* U+1236B */
  0x0000,   /* U+1236C */
  0x0000,   /* U+1236D */
  0x0000,   /* U+1236E */
  0x0000,   /* U+1236F */
  0x0000,   /* U+12370 */
  0x0000,   /* U+12371 */
  0x0000,   /* U+12372 */
  0x0000,   /* U+12373 */
  0x0000,   /* U+12374 */
  0x0000,   /* U+12375 */
  0x0000,   /* U+12376 */
  0x0000,   /* U+12377 */
  0x0000,   /* U+12378 */
  0x0000,   /* U+12379 */
  0x0000,   /* U+1237A */
  0x0000,   /* U+1237B */
  0x0000,   /* U+1237C */
  0x0000,   /* U+1237D */
  0x0000,   /* U+1237E */
  0x0000,   /* U+1237F */
  0x0000,   /* U+12380 */
  0x0000,   /* U+12381 */
  0x0000,   /* U+12382 */
  0x0000,   /* U+12383 */
  0x0000,   /* U+12384 */
  0x0000,   /* U+12385 */
  0x0000,   /* U+12386 */
  0x0000,   /* U+12387 */
  0x0000,   /* U+12388 */
  0x0000,   /* U+12389 */
  0x0000,   /* U+1238A */
  0x0000,   /* U+1238B */
  0x0000,   /* U+1238C */
  0x0000,   /* U+1238D */
  0x0000,   /* U+1238E */
  0x0000,   /* U+1238F */
  0x0000,   /* U+12390 */
  0x0000,   /* U+12391 */
  0x0000,   /* U+12392 */
  0x0000,   /* U+12393 */
  0x0000,   /* U+12394 */
  0x0000,   /* U+12395 */
  0x0000,   /* U+12396 */
  0x0000,   /* U+12397 */
  0x0000,   /* U+12398 */
  0x0000,   /* U+12399 */
  0x0000,   /* U+1239A */
  0x0000,   /* U+1239B */
  0x0000,   /* U+1239C */
  0x0000,   /* U+1239D */
  0x0000,   /* U+1239E */
  0x0000,   /* U+1239F */
  0x0000,   /* U+123A0 */
  0x0000,   /* U+123A1 */
  0x0000,   /* U+123A2 */
  0x0000,   /* U+123A3 */
  0x0000,   /* U+123A4 */
  0x0000,   /* U+123A5 */
  0x0000,   /* U+123A6 */
  0x0000,   /* U+123A7 */
  0x0000,   /* U+123A8 */
  0x0000,   /* U+123A9 */
  0x0000,   /* U+123AA */
  0x0000,   /* U+123AB */
  0x0000,   /* U+123AC */
  0x0000,   /* U+123AD */
  0x0000,   /* U+123AE */
  0x0000,   /* U+123AF */
  0x0000,   /* U+123B0 */
  0x0000,   /* U+123B1 */
  0x0000,   /* U+123B2 */
  0x0000,   /* U+123B3 */
  0x0000,   /* U+123B4 */
  0x0000,   /* U+123B5 */
  0x0000,   /* U+123B6 */
  0x0000,   /* U+123B7 */
  0x0000,   /* U+123B8 */
  0x0000,   /* U+123B9 */
  0x0000,   /* U+123BA */
  0x0000,   /* U+123BB */
  0x0000,   /* U+123BC */
  0x0000,   /* U+123BD */
  0x0000,   /* U+123BE */
  0x0000,   /* U+123BF */
  0x0000,   /* U+123C0 */
  0x0000,   /* U+123C1 */
  0x0000,   /* U+123C2 */
  0x0000,   /* U+123C3 */
  0x0000,   /* U+123C4 */
  0x0000,   /* U+123C5 */
  0x0000,   /* U+123C6 */
  0x0000,   /* U+123C7 */
  0x0000,   /* U+123C8 */
  0x0000,   /* U+123C9 */
  0x0000,   /* U+123CA */
  0x0000,   /* U+123CB */
  0x0000,   /* U+123CC */
  0x0000,   /* U+123CD */
  0x0000,   /* U+123CE */
  0x0000,   /* U+123CF */
  0x0000,   /* U+123D0 */
  0x0000,   /* U+123D1 */
  0x0000,   /* U+123D2 */
  0x0000,   /* U+123D3 */
  0x0000,   /* U+123D4 */
  0x0000,   /* U+123D5 */
  0x0000,   /* U+123D6 */
  0x0000,   /* U+123D7 */
  0x0000,   /* U+123D8 */
  0x0000,   /* U+123D9 */
  0x0000,   /* U+123DA */
  0x0000,   /* U+123DB */
  0x0000,   /* U+123DC */
  0x0000,   /* U+123DD */
  0x0000,   /* U+123DE */
  0x0000,   /* U+123DF */
  0x0000,   /* U+123E0 */
  0x0000,   /* U+123E1 */
  0x0000,   /* U+123E2 */
  0x0000,   /* U+123E3 */
  0x0000,   /* U+123E4 */
  0x0000,   /* U+123E5 */
  0x0000,   /* U+123E6 */
  0x0000,   /* U+123E7 */
  0x0000,   /* U+123E8 */
  0x0000,   /* U+123E9 */
  0x0000,   /* U+123EA */
  0x0000,   /* U+123EB */
  0x0000,   /* U+123EC */
  0x0000,   /* U+123ED */
  0x0000,   /* U+123EE */
  0x0000,   /* U+123EF */
  0x0000,   /* U+123F0 */
  0x0000,   /* U+123F1 */
  0x0000,   /* U+123F2 */
  0x0000,   /* U+123F3 */
  0x0000,   /* U+123F4 */
  0x0000,   /* U+123F5 */
  0x0000,   /* U+123F6 */
  0x0000,   /* U+123F7 */
  0x0000,   /* U+123F8 */
  0x0000,   /* U+123F9 */
  0x0000,   /* U+123FA */
  0x0000,   /* U+123FB */
  0x0000,   /* U+123FC */
  0x0000,   /* U+123FD */
  0x0000,   /* U+123FE */
  0x0000,   /* U+123FF */
};

uint16 uca900_p124[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C3F,   /* U+12400 */
  0x1C40,   /* U+12401 */
  0x1C41,   /* U+12402 */
  0x1C42,   /* U+12403 */
  0x1C43,   /* U+12404 */
  0x1C44,   /* U+12405 */
  0x1C45,   /* U+12406 */
  0x1C46,   /* U+12407 */
  0x1C40,   /* U+12408 */
  0x1C41,   /* U+12409 */
  0x1C42,   /* U+1240A */
  0x1C43,   /* U+1240B */
  0x1C44,   /* U+1240C */
  0x1C45,   /* U+1240D */
  0x1C46,   /* U+1240E */
  0x1C41,   /* U+1240F */
  0x1C42,   /* U+12410 */
  0x1C43,   /* U+12411 */
  0x1C44,   /* U+12412 */
  0x1C45,   /* U+12413 */
  0x1C46,   /* U+12414 */
  0x1C3E,   /* U+12415 */
  0x1C3F,   /* U+12416 */
  0x1C40,   /* U+12417 */
  0x1C41,   /* U+12418 */
  0x1C42,   /* U+12419 */
  0x1C43,   /* U+1241A */
  0x1C44,   /* U+1241B */
  0x1C45,   /* U+1241C */
  0x1C46,   /* U+1241D */
  0x1C3E,   /* U+1241E */
  0x1C3F,   /* U+1241F */
  0x1C40,   /* U+12420 */
  0x1C41,   /* U+12421 */
  0x1C42,   /* U+12422 */
  0x1C3F,   /* U+12423 */
  0x1C40,   /* U+12424 */
  0x1C40,   /* U+12425 */
  0x1C41,   /* U+12426 */
  0x1C42,   /* U+12427 */
  0x1C43,   /* U+12428 */
  0x1C44,   /* U+12429 */
  0x1C45,   /* U+1242A */
  0x1C46,   /* U+1242B */
  0x1C3E,   /* U+1242C */
  0x1C3F,   /* U+1242D */
  0x1C40,   /* U+1242E */
  0x1C40,   /* U+1242F */
  0x1C41,   /* U+12430 */
  0x1C42,   /* U+12431 */
  0x1BDE,   /* U+12432 */
  0x1BDF,   /* U+12433 */
  0x1C3E,   /* U+12434 */
  0x1C3F,   /* U+12435 */
  0x1C40,   /* U+12436 */
  0x1C40,   /* U+12437 */
  0x1C41,   /* U+12438 */
  0x1C42,   /* U+12439 */
  0x1C40,   /* U+1243A */
  0x1C40,   /* U+1243B */
  0x1C41,   /* U+1243C */
  0x1C41,   /* U+1243D */
  0x1C41,   /* U+1243E */
  0x1C41,   /* U+1243F */
  0x1C43,   /* U+12440 */
  0x1C44,   /* U+12441 */
  0x1C44,   /* U+12442 */
  0x1C44,   /* U+12443 */
  0x1C45,   /* U+12444 */
  0x1C45,   /* U+12445 */
  0x1C46,   /* U+12446 */
  0x1C46,   /* U+12447 */
  0x1C46,   /* U+12448 */
  0x1C46,   /* U+12449 */
  0x1C3F,   /* U+1244A */
  0x1C40,   /* U+1244B */
  0x1C41,   /* U+1244C */
  0x1C42,   /* U+1244D */
  0x1C43,   /* U+1244E */
  0x1C3E,   /* U+1244F */
  0x1C3F,   /* U+12450 */
  0x1C40,   /* U+12451 */
  0x1C41,   /* U+12452 */
  0x1C41,   /* U+12453 */
  0x1C42,   /* U+12454 */
  0x1C42,   /* U+12455 */
  0x1C3F,   /* U+12456 */
  0x1C40,   /* U+12457 */
  0x1C3E,   /* U+12458 */
  0x1C3F,   /* U+12459 */
  0x1BE0,   /* U+1245A */
  0x1BE1,   /* U+1245B */
  0x1BE2,   /* U+1245C */
  0x1BE3,   /* U+1245D */
  0x1BE4,   /* U+1245E */
  0x1BE5,   /* U+1245F */
  0x1BE6,   /* U+12460 */
  0x1BE7,   /* U+12461 */
  0x1BE8,   /* U+12462 */
  0x1BE9,   /* U+12463 */
  0x1BEA,   /* U+12464 */
  0x1BEB,   /* U+12465 */
  0x1BEC,   /* U+12466 */
  0x1BED,   /* U+12467 */
  0x1BEE,   /* U+12468 */
  0x1C41,   /* U+12469 */
  0x1C42,   /* U+1246A */
  0x1C43,   /* U+1246B */
  0x1C44,   /* U+1246C */
  0x1C45,   /* U+1246D */
  0x1C46,   /* U+1246E */
  0xFBC2,   /* U+1246F */
  0x0300,   /* U+12470 */
  0x0301,   /* U+12471 */
  0x0302,   /* U+12472 */
  0x0303,   /* U+12473 */
  0x0304,   /* U+12474 */
  0xFBC2,   /* U+12475 */
  0xFBC2,   /* U+12476 */
  0xFBC2,   /* U+12477 */
  0xFBC2,   /* U+12478 */
  0xFBC2,   /* U+12479 */
  0xFBC2,   /* U+1247A */
  0xFBC2,   /* U+1247B */
  0xFBC2,   /* U+1247C */
  0xFBC2,   /* U+1247D */
  0xFBC2,   /* U+1247E */
  0xFBC2,   /* U+1247F */
  0x4D4F,   /* U+12480 */
  0x4D50,   /* U+12481 */
  0x4D51,   /* U+12482 */
  0x4D52,   /* U+12483 */
  0x4D53,   /* U+12484 */
  0x4D54,   /* U+12485 */
  0x4D55,   /* U+12486 */
  0x4D56,   /* U+12487 */
  0x4D57,   /* U+12488 */
  0x4D58,   /* U+12489 */
  0x4D59,   /* U+1248A */
  0x4D5A,   /* U+1248B */
  0x4D5B,   /* U+1248C */
  0x4D5C,   /* U+1248D */
  0x4D5D,   /* U+1248E */
  0x4D5E,   /* U+1248F */
  0x4D5F,   /* U+12490 */
  0x4D60,   /* U+12491 */
  0x4D61,   /* U+12492 */
  0x4D62,   /* U+12493 */
  0x4D63,   /* U+12494 */
  0x4D64,   /* U+12495 */
  0x4D65,   /* U+12496 */
  0x4D66,   /* U+12497 */
  0x4D67,   /* U+12498 */
  0x4D68,   /* U+12499 */
  0x4D69,   /* U+1249A */
  0x4D6A,   /* U+1249B */
  0x4D6B,   /* U+1249C */
  0x4D6C,   /* U+1249D */
  0x4D6D,   /* U+1249E */
  0x4D6E,   /* U+1249F */
  0x4D6F,   /* U+124A0 */
  0x4D70,   /* U+124A1 */
  0x4D71,   /* U+124A2 */
  0x4D72,   /* U+124A3 */
  0x4D73,   /* U+124A4 */
  0x4D74,   /* U+124A5 */
  0x4D75,   /* U+124A6 */
  0x4D76,   /* U+124A7 */
  0x4D77,   /* U+124A8 */
  0x4D78,   /* U+124A9 */
  0x4D79,   /* U+124AA */
  0x4D7A,   /* U+124AB */
  0x4D7B,   /* U+124AC */
  0x4D7C,   /* U+124AD */
  0x4D7D,   /* U+124AE */
  0x4D7E,   /* U+124AF */
  0x4D7F,   /* U+124B0 */
  0x4D80,   /* U+124B1 */
  0x4D81,   /* U+124B2 */
  0x4D82,   /* U+124B3 */
  0x4D83,   /* U+124B4 */
  0x4D84,   /* U+124B5 */
  0x4D85,   /* U+124B6 */
  0x4D86,   /* U+124B7 */
  0x4D87,   /* U+124B8 */
  0x4D88,   /* U+124B9 */
  0x4D89,   /* U+124BA */
  0x4D8A,   /* U+124BB */
  0x4D8B,   /* U+124BC */
  0x4D8C,   /* U+124BD */
  0x4D8D,   /* U+124BE */
  0x4D8E,   /* U+124BF */
  0x4D8F,   /* U+124C0 */
  0x4D90,   /* U+124C1 */
  0x4D91,   /* U+124C2 */
  0x4D92,   /* U+124C3 */
  0x4D93,   /* U+124C4 */
  0x4D94,   /* U+124C5 */
  0x4D95,   /* U+124C6 */
  0x4D96,   /* U+124C7 */
  0x4D97,   /* U+124C8 */
  0x4D98,   /* U+124C9 */
  0x4D99,   /* U+124CA */
  0x4D9A,   /* U+124CB */
  0x4D9B,   /* U+124CC */
  0x4D9C,   /* U+124CD */
  0x4D9D,   /* U+124CE */
  0x4D9E,   /* U+124CF */
  0x4D9F,   /* U+124D0 */
  0x4DA0,   /* U+124D1 */
  0x4DA1,   /* U+124D2 */
  0x4DA2,   /* U+124D3 */
  0x4DA3,   /* U+124D4 */
  0x4DA4,   /* U+124D5 */
  0x4DA5,   /* U+124D6 */
  0x4DA6,   /* U+124D7 */
  0x4DA7,   /* U+124D8 */
  0x4DA8,   /* U+124D9 */
  0x4DA9,   /* U+124DA */
  0x4DAA,   /* U+124DB */
  0x4DAB,   /* U+124DC */
  0x4DAC,   /* U+124DD */
  0x4DAD,   /* U+124DE */
  0x4DAE,   /* U+124DF */
  0x4DAF,   /* U+124E0 */
  0x4DB0,   /* U+124E1 */
  0x4DB1,   /* U+124E2 */
  0x4DB2,   /* U+124E3 */
  0x4DB3,   /* U+124E4 */
  0x4DB4,   /* U+124E5 */
  0x4DB5,   /* U+124E6 */
  0x4DB6,   /* U+124E7 */
  0x4DB7,   /* U+124E8 */
  0x4DB8,   /* U+124E9 */
  0x4DB9,   /* U+124EA */
  0x4DBA,   /* U+124EB */
  0x4DBB,   /* U+124EC */
  0x4DBC,   /* U+124ED */
  0x4DBD,   /* U+124EE */
  0x4DBE,   /* U+124EF */
  0x4DBF,   /* U+124F0 */
  0x4DC0,   /* U+124F1 */
  0x4DC1,   /* U+124F2 */
  0x4DC2,   /* U+124F3 */
  0x4DC3,   /* U+124F4 */
  0x4DC4,   /* U+124F5 */
  0x4DC5,   /* U+124F6 */
  0x4DC6,   /* U+124F7 */
  0x4DC7,   /* U+124F8 */
  0x4DC8,   /* U+124F9 */
  0x4DC9,   /* U+124FA */
  0x4DCA,   /* U+124FB */
  0x4DCB,   /* U+124FC */
  0x4DCC,   /* U+124FD */
  0x4DCD,   /* U+124FE */
  0x4DCE,   /* U+124FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12400 */
  0x0020,   /* U+12401 */
  0x0020,   /* U+12402 */
  0x0020,   /* U+12403 */
  0x0020,   /* U+12404 */
  0x0020,   /* U+12405 */
  0x0020,   /* U+12406 */
  0x0020,   /* U+12407 */
  0x0020,   /* U+12408 */
  0x0020,   /* U+12409 */
  0x0020,   /* U+1240A */
  0x0020,   /* U+1240B */
  0x0020,   /* U+1240C */
  0x0020,   /* U+1240D */
  0x0020,   /* U+1240E */
  0x0020,   /* U+1240F */
  0x0020,   /* U+12410 */
  0x0020,   /* U+12411 */
  0x0020,   /* U+12412 */
  0x0020,   /* U+12413 */
  0x0020,   /* U+12414 */
  0x0020,   /* U+12415 */
  0x0020,   /* U+12416 */
  0x0020,   /* U+12417 */
  0x0020,   /* U+12418 */
  0x0020,   /* U+12419 */
  0x0020,   /* U+1241A */
  0x0020,   /* U+1241B */
  0x0020,   /* U+1241C */
  0x0020,   /* U+1241D */
  0x0020,   /* U+1241E */
  0x0020,   /* U+1241F */
  0x0020,   /* U+12420 */
  0x0020,   /* U+12421 */
  0x0020,   /* U+12422 */
  0x0020,   /* U+12423 */
  0x0020,   /* U+12424 */
  0x0020,   /* U+12425 */
  0x0020,   /* U+12426 */
  0x0020,   /* U+12427 */
  0x0020,   /* U+12428 */
  0x0020,   /* U+12429 */
  0x0020,   /* U+1242A */
  0x0020,   /* U+1242B */
  0x0020,   /* U+1242C */
  0x0020,   /* U+1242D */
  0x0020,   /* U+1242E */
  0x0020,   /* U+1242F */
  0x0020,   /* U+12430 */
  0x0020,   /* U+12431 */
  0x0020,   /* U+12432 */
  0x0020,   /* U+12433 */
  0x0020,   /* U+12434 */
  0x0020,   /* U+12435 */
  0x0020,   /* U+12436 */
  0x0020,   /* U+12437 */
  0x0020,   /* U+12438 */
  0x0020,   /* U+12439 */
  0x0020,   /* U+1243A */
  0x0020,   /* U+1243B */
  0x0020,   /* U+1243C */
  0x0020,   /* U+1243D */
  0x0020,   /* U+1243E */
  0x0020,   /* U+1243F */
  0x0020,   /* U+12440 */
  0x0020,   /* U+12441 */
  0x0020,   /* U+12442 */
  0x0020,   /* U+12443 */
  0x0020,   /* U+12444 */
  0x0020,   /* U+12445 */
  0x0020,   /* U+12446 */
  0x0020,   /* U+12447 */
  0x0020,   /* U+12448 */
  0x0020,   /* U+12449 */
  0x0020,   /* U+1244A */
  0x0020,   /* U+1244B */
  0x0020,   /* U+1244C */
  0x0020,   /* U+1244D */
  0x0020,   /* U+1244E */
  0x0020,   /* U+1244F */
  0x0020,   /* U+12450 */
  0x0020,   /* U+12451 */
  0x0020,   /* U+12452 */
  0x0020,   /* U+12453 */
  0x0020,   /* U+12454 */
  0x0020,   /* U+12455 */
  0x0020,   /* U+12456 */
  0x0020,   /* U+12457 */
  0x0020,   /* U+12458 */
  0x0020,   /* U+12459 */
  0x0020,   /* U+1245A */
  0x0020,   /* U+1245B */
  0x0020,   /* U+1245C */
  0x0020,   /* U+1245D */
  0x0020,   /* U+1245E */
  0x0020,   /* U+1245F */
  0x0020,   /* U+12460 */
  0x0020,   /* U+12461 */
  0x0020,   /* U+12462 */
  0x0020,   /* U+12463 */
  0x0020,   /* U+12464 */
  0x0020,   /* U+12465 */
  0x0020,   /* U+12466 */
  0x0020,   /* U+12467 */
  0x0020,   /* U+12468 */
  0x0020,   /* U+12469 */
  0x0020,   /* U+1246A */
  0x0020,   /* U+1246B */
  0x0020,   /* U+1246C */
  0x0020,   /* U+1246D */
  0x0020,   /* U+1246E */
  0x0020,   /* U+1246F */
  0x0020,   /* U+12470 */
  0x0020,   /* U+12471 */
  0x0020,   /* U+12472 */
  0x0020,   /* U+12473 */
  0x0020,   /* U+12474 */
  0x0020,   /* U+12475 */
  0x0020,   /* U+12476 */
  0x0020,   /* U+12477 */
  0x0020,   /* U+12478 */
  0x0020,   /* U+12479 */
  0x0020,   /* U+1247A */
  0x0020,   /* U+1247B */
  0x0020,   /* U+1247C */
  0x0020,   /* U+1247D */
  0x0020,   /* U+1247E */
  0x0020,   /* U+1247F */
  0x0020,   /* U+12480 */
  0x0020,   /* U+12481 */
  0x0020,   /* U+12482 */
  0x0020,   /* U+12483 */
  0x0020,   /* U+12484 */
  0x0020,   /* U+12485 */
  0x0020,   /* U+12486 */
  0x0020,   /* U+12487 */
  0x0020,   /* U+12488 */
  0x0020,   /* U+12489 */
  0x0020,   /* U+1248A */
  0x0020,   /* U+1248B */
  0x0020,   /* U+1248C */
  0x0020,   /* U+1248D */
  0x0020,   /* U+1248E */
  0x0020,   /* U+1248F */
  0x0020,   /* U+12490 */
  0x0020,   /* U+12491 */
  0x0020,   /* U+12492 */
  0x0020,   /* U+12493 */
  0x0020,   /* U+12494 */
  0x0020,   /* U+12495 */
  0x0020,   /* U+12496 */
  0x0020,   /* U+12497 */
  0x0020,   /* U+12498 */
  0x0020,   /* U+12499 */
  0x0020,   /* U+1249A */
  0x0020,   /* U+1249B */
  0x0020,   /* U+1249C */
  0x0020,   /* U+1249D */
  0x0020,   /* U+1249E */
  0x0020,   /* U+1249F */
  0x0020,   /* U+124A0 */
  0x0020,   /* U+124A1 */
  0x0020,   /* U+124A2 */
  0x0020,   /* U+124A3 */
  0x0020,   /* U+124A4 */
  0x0020,   /* U+124A5 */
  0x0020,   /* U+124A6 */
  0x0020,   /* U+124A7 */
  0x0020,   /* U+124A8 */
  0x0020,   /* U+124A9 */
  0x0020,   /* U+124AA */
  0x0020,   /* U+124AB */
  0x0020,   /* U+124AC */
  0x0020,   /* U+124AD */
  0x0020,   /* U+124AE */
  0x0020,   /* U+124AF */
  0x0020,   /* U+124B0 */
  0x0020,   /* U+124B1 */
  0x0020,   /* U+124B2 */
  0x0020,   /* U+124B3 */
  0x0020,   /* U+124B4 */
  0x0020,   /* U+124B5 */
  0x0020,   /* U+124B6 */
  0x0020,   /* U+124B7 */
  0x0020,   /* U+124B8 */
  0x0020,   /* U+124B9 */
  0x0020,   /* U+124BA */
  0x0020,   /* U+124BB */
  0x0020,   /* U+124BC */
  0x0020,   /* U+124BD */
  0x0020,   /* U+124BE */
  0x0020,   /* U+124BF */
  0x0020,   /* U+124C0 */
  0x0020,   /* U+124C1 */
  0x0020,   /* U+124C2 */
  0x0020,   /* U+124C3 */
  0x0020,   /* U+124C4 */
  0x0020,   /* U+124C5 */
  0x0020,   /* U+124C6 */
  0x0020,   /* U+124C7 */
  0x0020,   /* U+124C8 */
  0x0020,   /* U+124C9 */
  0x0020,   /* U+124CA */
  0x0020,   /* U+124CB */
  0x0020,   /* U+124CC */
  0x0020,   /* U+124CD */
  0x0020,   /* U+124CE */
  0x0020,   /* U+124CF */
  0x0020,   /* U+124D0 */
  0x0020,   /* U+124D1 */
  0x0020,   /* U+124D2 */
  0x0020,   /* U+124D3 */
  0x0020,   /* U+124D4 */
  0x0020,   /* U+124D5 */
  0x0020,   /* U+124D6 */
  0x0020,   /* U+124D7 */
  0x0020,   /* U+124D8 */
  0x0020,   /* U+124D9 */
  0x0020,   /* U+124DA */
  0x0020,   /* U+124DB */
  0x0020,   /* U+124DC */
  0x0020,   /* U+124DD */
  0x0020,   /* U+124DE */
  0x0020,   /* U+124DF */
  0x0020,   /* U+124E0 */
  0x0020,   /* U+124E1 */
  0x0020,   /* U+124E2 */
  0x0020,   /* U+124E3 */
  0x0020,   /* U+124E4 */
  0x0020,   /* U+124E5 */
  0x0020,   /* U+124E6 */
  0x0020,   /* U+124E7 */
  0x0020,   /* U+124E8 */
  0x0020,   /* U+124E9 */
  0x0020,   /* U+124EA */
  0x0020,   /* U+124EB */
  0x0020,   /* U+124EC */
  0x0020,   /* U+124ED */
  0x0020,   /* U+124EE */
  0x0020,   /* U+124EF */
  0x0020,   /* U+124F0 */
  0x0020,   /* U+124F1 */
  0x0020,   /* U+124F2 */
  0x0020,   /* U+124F3 */
  0x0020,   /* U+124F4 */
  0x0020,   /* U+124F5 */
  0x0020,   /* U+124F6 */
  0x0020,   /* U+124F7 */
  0x0020,   /* U+124F8 */
  0x0020,   /* U+124F9 */
  0x0020,   /* U+124FA */
  0x0020,   /* U+124FB */
  0x0020,   /* U+124FC */
  0x0020,   /* U+124FD */
  0x0020,   /* U+124FE */
  0x0020,   /* U+124FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12400 */
  0x0002,   /* U+12401 */
  0x0002,   /* U+12402 */
  0x0002,   /* U+12403 */
  0x0002,   /* U+12404 */
  0x0002,   /* U+12405 */
  0x0002,   /* U+12406 */
  0x0002,   /* U+12407 */
  0x0002,   /* U+12408 */
  0x0002,   /* U+12409 */
  0x0002,   /* U+1240A */
  0x0002,   /* U+1240B */
  0x0002,   /* U+1240C */
  0x0002,   /* U+1240D */
  0x0002,   /* U+1240E */
  0x0002,   /* U+1240F */
  0x0002,   /* U+12410 */
  0x0002,   /* U+12411 */
  0x0002,   /* U+12412 */
  0x0002,   /* U+12413 */
  0x0002,   /* U+12414 */
  0x0002,   /* U+12415 */
  0x0002,   /* U+12416 */
  0x0002,   /* U+12417 */
  0x0002,   /* U+12418 */
  0x0002,   /* U+12419 */
  0x0002,   /* U+1241A */
  0x0002,   /* U+1241B */
  0x0002,   /* U+1241C */
  0x0002,   /* U+1241D */
  0x0002,   /* U+1241E */
  0x0002,   /* U+1241F */
  0x0002,   /* U+12420 */
  0x0002,   /* U+12421 */
  0x0002,   /* U+12422 */
  0x0002,   /* U+12423 */
  0x0002,   /* U+12424 */
  0x0002,   /* U+12425 */
  0x0002,   /* U+12426 */
  0x0002,   /* U+12427 */
  0x0002,   /* U+12428 */
  0x0002,   /* U+12429 */
  0x0002,   /* U+1242A */
  0x0002,   /* U+1242B */
  0x0002,   /* U+1242C */
  0x0002,   /* U+1242D */
  0x0002,   /* U+1242E */
  0x0002,   /* U+1242F */
  0x0002,   /* U+12430 */
  0x0002,   /* U+12431 */
  0x0002,   /* U+12432 */
  0x0002,   /* U+12433 */
  0x0002,   /* U+12434 */
  0x0002,   /* U+12435 */
  0x0002,   /* U+12436 */
  0x0002,   /* U+12437 */
  0x0002,   /* U+12438 */
  0x0002,   /* U+12439 */
  0x0002,   /* U+1243A */
  0x0002,   /* U+1243B */
  0x0002,   /* U+1243C */
  0x0002,   /* U+1243D */
  0x0002,   /* U+1243E */
  0x0002,   /* U+1243F */
  0x0002,   /* U+12440 */
  0x0002,   /* U+12441 */
  0x0002,   /* U+12442 */
  0x0002,   /* U+12443 */
  0x0002,   /* U+12444 */
  0x0002,   /* U+12445 */
  0x0002,   /* U+12446 */
  0x0002,   /* U+12447 */
  0x0002,   /* U+12448 */
  0x0002,   /* U+12449 */
  0x0002,   /* U+1244A */
  0x0002,   /* U+1244B */
  0x0002,   /* U+1244C */
  0x0002,   /* U+1244D */
  0x0002,   /* U+1244E */
  0x0002,   /* U+1244F */
  0x0002,   /* U+12450 */
  0x0002,   /* U+12451 */
  0x0002,   /* U+12452 */
  0x0002,   /* U+12453 */
  0x0002,   /* U+12454 */
  0x0002,   /* U+12455 */
  0x0002,   /* U+12456 */
  0x0002,   /* U+12457 */
  0x0002,   /* U+12458 */
  0x0002,   /* U+12459 */
  0x0002,   /* U+1245A */
  0x0002,   /* U+1245B */
  0x0002,   /* U+1245C */
  0x0002,   /* U+1245D */
  0x0002,   /* U+1245E */
  0x0002,   /* U+1245F */
  0x0002,   /* U+12460 */
  0x0002,   /* U+12461 */
  0x0002,   /* U+12462 */
  0x0002,   /* U+12463 */
  0x0002,   /* U+12464 */
  0x0002,   /* U+12465 */
  0x0002,   /* U+12466 */
  0x0002,   /* U+12467 */
  0x0002,   /* U+12468 */
  0x0002,   /* U+12469 */
  0x0002,   /* U+1246A */
  0x0002,   /* U+1246B */
  0x0002,   /* U+1246C */
  0x0002,   /* U+1246D */
  0x0002,   /* U+1246E */
  0x0002,   /* U+1246F */
  0x0002,   /* U+12470 */
  0x0002,   /* U+12471 */
  0x0002,   /* U+12472 */
  0x0002,   /* U+12473 */
  0x0002,   /* U+12474 */
  0x0002,   /* U+12475 */
  0x0002,   /* U+12476 */
  0x0002,   /* U+12477 */
  0x0002,   /* U+12478 */
  0x0002,   /* U+12479 */
  0x0002,   /* U+1247A */
  0x0002,   /* U+1247B */
  0x0002,   /* U+1247C */
  0x0002,   /* U+1247D */
  0x0002,   /* U+1247E */
  0x0002,   /* U+1247F */
  0x0002,   /* U+12480 */
  0x0002,   /* U+12481 */
  0x0002,   /* U+12482 */
  0x0002,   /* U+12483 */
  0x0002,   /* U+12484 */
  0x0002,   /* U+12485 */
  0x0002,   /* U+12486 */
  0x0002,   /* U+12487 */
  0x0002,   /* U+12488 */
  0x0002,   /* U+12489 */
  0x0002,   /* U+1248A */
  0x0002,   /* U+1248B */
  0x0002,   /* U+1248C */
  0x0002,   /* U+1248D */
  0x0002,   /* U+1248E */
  0x0002,   /* U+1248F */
  0x0002,   /* U+12490 */
  0x0002,   /* U+12491 */
  0x0002,   /* U+12492 */
  0x0002,   /* U+12493 */
  0x0002,   /* U+12494 */
  0x0002,   /* U+12495 */
  0x0002,   /* U+12496 */
  0x0002,   /* U+12497 */
  0x0002,   /* U+12498 */
  0x0002,   /* U+12499 */
  0x0002,   /* U+1249A */
  0x0002,   /* U+1249B */
  0x0002,   /* U+1249C */
  0x0002,   /* U+1249D */
  0x0002,   /* U+1249E */
  0x0002,   /* U+1249F */
  0x0002,   /* U+124A0 */
  0x0002,   /* U+124A1 */
  0x0002,   /* U+124A2 */
  0x0002,   /* U+124A3 */
  0x0002,   /* U+124A4 */
  0x0002,   /* U+124A5 */
  0x0002,   /* U+124A6 */
  0x0002,   /* U+124A7 */
  0x0002,   /* U+124A8 */
  0x0002,   /* U+124A9 */
  0x0002,   /* U+124AA */
  0x0002,   /* U+124AB */
  0x0002,   /* U+124AC */
  0x0002,   /* U+124AD */
  0x0002,   /* U+124AE */
  0x0002,   /* U+124AF */
  0x0002,   /* U+124B0 */
  0x0002,   /* U+124B1 */
  0x0002,   /* U+124B2 */
  0x0002,   /* U+124B3 */
  0x0002,   /* U+124B4 */
  0x0002,   /* U+124B5 */
  0x0002,   /* U+124B6 */
  0x0002,   /* U+124B7 */
  0x0002,   /* U+124B8 */
  0x0002,   /* U+124B9 */
  0x0002,   /* U+124BA */
  0x0002,   /* U+124BB */
  0x0002,   /* U+124BC */
  0x0002,   /* U+124BD */
  0x0002,   /* U+124BE */
  0x0002,   /* U+124BF */
  0x0002,   /* U+124C0 */
  0x0002,   /* U+124C1 */
  0x0002,   /* U+124C2 */
  0x0002,   /* U+124C3 */
  0x0002,   /* U+124C4 */
  0x0002,   /* U+124C5 */
  0x0002,   /* U+124C6 */
  0x0002,   /* U+124C7 */
  0x0002,   /* U+124C8 */
  0x0002,   /* U+124C9 */
  0x0002,   /* U+124CA */
  0x0002,   /* U+124CB */
  0x0002,   /* U+124CC */
  0x0002,   /* U+124CD */
  0x0002,   /* U+124CE */
  0x0002,   /* U+124CF */
  0x0002,   /* U+124D0 */
  0x0002,   /* U+124D1 */
  0x0002,   /* U+124D2 */
  0x0002,   /* U+124D3 */
  0x0002,   /* U+124D4 */
  0x0002,   /* U+124D5 */
  0x0002,   /* U+124D6 */
  0x0002,   /* U+124D7 */
  0x0002,   /* U+124D8 */
  0x0002,   /* U+124D9 */
  0x0002,   /* U+124DA */
  0x0002,   /* U+124DB */
  0x0002,   /* U+124DC */
  0x0002,   /* U+124DD */
  0x0002,   /* U+124DE */
  0x0002,   /* U+124DF */
  0x0002,   /* U+124E0 */
  0x0002,   /* U+124E1 */
  0x0002,   /* U+124E2 */
  0x0002,   /* U+124E3 */
  0x0002,   /* U+124E4 */
  0x0002,   /* U+124E5 */
  0x0002,   /* U+124E6 */
  0x0002,   /* U+124E7 */
  0x0002,   /* U+124E8 */
  0x0002,   /* U+124E9 */
  0x0002,   /* U+124EA */
  0x0002,   /* U+124EB */
  0x0002,   /* U+124EC */
  0x0002,   /* U+124ED */
  0x0002,   /* U+124EE */
  0x0002,   /* U+124EF */
  0x0002,   /* U+124F0 */
  0x0002,   /* U+124F1 */
  0x0002,   /* U+124F2 */
  0x0002,   /* U+124F3 */
  0x0002,   /* U+124F4 */
  0x0002,   /* U+124F5 */
  0x0002,   /* U+124F6 */
  0x0002,   /* U+124F7 */
  0x0002,   /* U+124F8 */
  0x0002,   /* U+124F9 */
  0x0002,   /* U+124FA */
  0x0002,   /* U+124FB */
  0x0002,   /* U+124FC */
  0x0002,   /* U+124FD */
  0x0002,   /* U+124FE */
  0x0002,   /* U+124FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+12400 */
  0x0000,   /* U+12401 */
  0x0000,   /* U+12402 */
  0x0000,   /* U+12403 */
  0x0000,   /* U+12404 */
  0x0000,   /* U+12405 */
  0x0000,   /* U+12406 */
  0x0000,   /* U+12407 */
  0x0000,   /* U+12408 */
  0x0000,   /* U+12409 */
  0x0000,   /* U+1240A */
  0x0000,   /* U+1240B */
  0x0000,   /* U+1240C */
  0x0000,   /* U+1240D */
  0x0000,   /* U+1240E */
  0x0000,   /* U+1240F */
  0x0000,   /* U+12410 */
  0x0000,   /* U+12411 */
  0x0000,   /* U+12412 */
  0x0000,   /* U+12413 */
  0x0000,   /* U+12414 */
  0x0000,   /* U+12415 */
  0x0000,   /* U+12416 */
  0x0000,   /* U+12417 */
  0x0000,   /* U+12418 */
  0x0000,   /* U+12419 */
  0x0000,   /* U+1241A */
  0x0000,   /* U+1241B */
  0x0000,   /* U+1241C */
  0x0000,   /* U+1241D */
  0x0000,   /* U+1241E */
  0x0000,   /* U+1241F */
  0x0000,   /* U+12420 */
  0x0000,   /* U+12421 */
  0x0000,   /* U+12422 */
  0x0000,   /* U+12423 */
  0x0000,   /* U+12424 */
  0x0000,   /* U+12425 */
  0x0000,   /* U+12426 */
  0x0000,   /* U+12427 */
  0x0000,   /* U+12428 */
  0x0000,   /* U+12429 */
  0x0000,   /* U+1242A */
  0x0000,   /* U+1242B */
  0x0000,   /* U+1242C */
  0x0000,   /* U+1242D */
  0x0000,   /* U+1242E */
  0x0000,   /* U+1242F */
  0x0000,   /* U+12430 */
  0x0000,   /* U+12431 */
  0x0000,   /* U+12432 */
  0x0000,   /* U+12433 */
  0x0000,   /* U+12434 */
  0x0000,   /* U+12435 */
  0x0000,   /* U+12436 */
  0x0000,   /* U+12437 */
  0x0000,   /* U+12438 */
  0x0000,   /* U+12439 */
  0x0000,   /* U+1243A */
  0x0000,   /* U+1243B */
  0x0000,   /* U+1243C */
  0x0000,   /* U+1243D */
  0x0000,   /* U+1243E */
  0x0000,   /* U+1243F */
  0x0000,   /* U+12440 */
  0x0000,   /* U+12441 */
  0x0000,   /* U+12442 */
  0x0000,   /* U+12443 */
  0x0000,   /* U+12444 */
  0x0000,   /* U+12445 */
  0x0000,   /* U+12446 */
  0x0000,   /* U+12447 */
  0x0000,   /* U+12448 */
  0x0000,   /* U+12449 */
  0x0000,   /* U+1244A */
  0x0000,   /* U+1244B */
  0x0000,   /* U+1244C */
  0x0000,   /* U+1244D */
  0x0000,   /* U+1244E */
  0x0000,   /* U+1244F */
  0x0000,   /* U+12450 */
  0x0000,   /* U+12451 */
  0x0000,   /* U+12452 */
  0x0000,   /* U+12453 */
  0x0000,   /* U+12454 */
  0x0000,   /* U+12455 */
  0x0000,   /* U+12456 */
  0x0000,   /* U+12457 */
  0x0000,   /* U+12458 */
  0x0000,   /* U+12459 */
  0x0000,   /* U+1245A */
  0x0000,   /* U+1245B */
  0x0000,   /* U+1245C */
  0x0000,   /* U+1245D */
  0x0000,   /* U+1245E */
  0x0000,   /* U+1245F */
  0x0000,   /* U+12460 */
  0x0000,   /* U+12461 */
  0x0000,   /* U+12462 */
  0x0000,   /* U+12463 */
  0x0000,   /* U+12464 */
  0x0000,   /* U+12465 */
  0x0000,   /* U+12466 */
  0x0000,   /* U+12467 */
  0x0000,   /* U+12468 */
  0x0000,   /* U+12469 */
  0x0000,   /* U+1246A */
  0x0000,   /* U+1246B */
  0x0000,   /* U+1246C */
  0x0000,   /* U+1246D */
  0x0000,   /* U+1246E */
  0xA46F,   /* U+1246F */
  0x0000,   /* U+12470 */
  0x0000,   /* U+12471 */
  0x0000,   /* U+12472 */
  0x0000,   /* U+12473 */
  0x0000,   /* U+12474 */
  0xA475,   /* U+12475 */
  0xA476,   /* U+12476 */
  0xA477,   /* U+12477 */
  0xA478,   /* U+12478 */
  0xA479,   /* U+12479 */
  0xA47A,   /* U+1247A */
  0xA47B,   /* U+1247B */
  0xA47C,   /* U+1247C */
  0xA47D,   /* U+1247D */
  0xA47E,   /* U+1247E */
  0xA47F,   /* U+1247F */
  0x0000,   /* U+12480 */
  0x0000,   /* U+12481 */
  0x0000,   /* U+12482 */
  0x0000,   /* U+12483 */
  0x0000,   /* U+12484 */
  0x0000,   /* U+12485 */
  0x0000,   /* U+12486 */
  0x0000,   /* U+12487 */
  0x0000,   /* U+12488 */
  0x0000,   /* U+12489 */
  0x0000,   /* U+1248A */
  0x0000,   /* U+1248B */
  0x0000,   /* U+1248C */
  0x0000,   /* U+1248D */
  0x0000,   /* U+1248E */
  0x0000,   /* U+1248F */
  0x0000,   /* U+12490 */
  0x0000,   /* U+12491 */
  0x0000,   /* U+12492 */
  0x0000,   /* U+12493 */
  0x0000,   /* U+12494 */
  0x0000,   /* U+12495 */
  0x0000,   /* U+12496 */
  0x0000,   /* U+12497 */
  0x0000,   /* U+12498 */
  0x0000,   /* U+12499 */
  0x0000,   /* U+1249A */
  0x0000,   /* U+1249B */
  0x0000,   /* U+1249C */
  0x0000,   /* U+1249D */
  0x0000,   /* U+1249E */
  0x0000,   /* U+1249F */
  0x0000,   /* U+124A0 */
  0x0000,   /* U+124A1 */
  0x0000,   /* U+124A2 */
  0x0000,   /* U+124A3 */
  0x0000,   /* U+124A4 */
  0x0000,   /* U+124A5 */
  0x0000,   /* U+124A6 */
  0x0000,   /* U+124A7 */
  0x0000,   /* U+124A8 */
  0x0000,   /* U+124A9 */
  0x0000,   /* U+124AA */
  0x0000,   /* U+124AB */
  0x0000,   /* U+124AC */
  0x0000,   /* U+124AD */
  0x0000,   /* U+124AE */
  0x0000,   /* U+124AF */
  0x0000,   /* U+124B0 */
  0x0000,   /* U+124B1 */
  0x0000,   /* U+124B2 */
  0x0000,   /* U+124B3 */
  0x0000,   /* U+124B4 */
  0x0000,   /* U+124B5 */
  0x0000,   /* U+124B6 */
  0x0000,   /* U+124B7 */
  0x0000,   /* U+124B8 */
  0x0000,   /* U+124B9 */
  0x0000,   /* U+124BA */
  0x0000,   /* U+124BB */
  0x0000,   /* U+124BC */
  0x0000,   /* U+124BD */
  0x0000,   /* U+124BE */
  0x0000,   /* U+124BF */
  0x0000,   /* U+124C0 */
  0x0000,   /* U+124C1 */
  0x0000,   /* U+124C2 */
  0x0000,   /* U+124C3 */
  0x0000,   /* U+124C4 */
  0x0000,   /* U+124C5 */
  0x0000,   /* U+124C6 */
  0x0000,   /* U+124C7 */
  0x0000,   /* U+124C8 */
  0x0000,   /* U+124C9 */
  0x0000,   /* U+124CA */
  0x0000,   /* U+124CB */
  0x0000,   /* U+124CC */
  0x0000,   /* U+124CD */
  0x0000,   /* U+124CE */
  0x0000,   /* U+124CF */
  0x0000,   /* U+124D0 */
  0x0000,   /* U+124D1 */
  0x0000,   /* U+124D2 */
  0x0000,   /* U+124D3 */
  0x0000,   /* U+124D4 */
  0x0000,   /* U+124D5 */
  0x0000,   /* U+124D6 */
  0x0000,   /* U+124D7 */
  0x0000,   /* U+124D8 */
  0x0000,   /* U+124D9 */
  0x0000,   /* U+124DA */
  0x0000,   /* U+124DB */
  0x0000,   /* U+124DC */
  0x0000,   /* U+124DD */
  0x0000,   /* U+124DE */
  0x0000,   /* U+124DF */
  0x0000,   /* U+124E0 */
  0x0000,   /* U+124E1 */
  0x0000,   /* U+124E2 */
  0x0000,   /* U+124E3 */
  0x0000,   /* U+124E4 */
  0x0000,   /* U+124E5 */
  0x0000,   /* U+124E6 */
  0x0000,   /* U+124E7 */
  0x0000,   /* U+124E8 */
  0x0000,   /* U+124E9 */
  0x0000,   /* U+124EA */
  0x0000,   /* U+124EB */
  0x0000,   /* U+124EC */
  0x0000,   /* U+124ED */
  0x0000,   /* U+124EE */
  0x0000,   /* U+124EF */
  0x0000,   /* U+124F0 */
  0x0000,   /* U+124F1 */
  0x0000,   /* U+124F2 */
  0x0000,   /* U+124F3 */
  0x0000,   /* U+124F4 */
  0x0000,   /* U+124F5 */
  0x0000,   /* U+124F6 */
  0x0000,   /* U+124F7 */
  0x0000,   /* U+124F8 */
  0x0000,   /* U+124F9 */
  0x0000,   /* U+124FA */
  0x0000,   /* U+124FB */
  0x0000,   /* U+124FC */
  0x0000,   /* U+124FD */
  0x0000,   /* U+124FE */
  0x0000,   /* U+124FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+12400 */
  0x0000,   /* U+12401 */
  0x0000,   /* U+12402 */
  0x0000,   /* U+12403 */
  0x0000,   /* U+12404 */
  0x0000,   /* U+12405 */
  0x0000,   /* U+12406 */
  0x0000,   /* U+12407 */
  0x0000,   /* U+12408 */
  0x0000,   /* U+12409 */
  0x0000,   /* U+1240A */
  0x0000,   /* U+1240B */
  0x0000,   /* U+1240C */
  0x0000,   /* U+1240D */
  0x0000,   /* U+1240E */
  0x0000,   /* U+1240F */
  0x0000,   /* U+12410 */
  0x0000,   /* U+12411 */
  0x0000,   /* U+12412 */
  0x0000,   /* U+12413 */
  0x0000,   /* U+12414 */
  0x0000,   /* U+12415 */
  0x0000,   /* U+12416 */
  0x0000,   /* U+12417 */
  0x0000,   /* U+12418 */
  0x0000,   /* U+12419 */
  0x0000,   /* U+1241A */
  0x0000,   /* U+1241B */
  0x0000,   /* U+1241C */
  0x0000,   /* U+1241D */
  0x0000,   /* U+1241E */
  0x0000,   /* U+1241F */
  0x0000,   /* U+12420 */
  0x0000,   /* U+12421 */
  0x0000,   /* U+12422 */
  0x0000,   /* U+12423 */
  0x0000,   /* U+12424 */
  0x0000,   /* U+12425 */
  0x0000,   /* U+12426 */
  0x0000,   /* U+12427 */
  0x0000,   /* U+12428 */
  0x0000,   /* U+12429 */
  0x0000,   /* U+1242A */
  0x0000,   /* U+1242B */
  0x0000,   /* U+1242C */
  0x0000,   /* U+1242D */
  0x0000,   /* U+1242E */
  0x0000,   /* U+1242F */
  0x0000,   /* U+12430 */
  0x0000,   /* U+12431 */
  0x0000,   /* U+12432 */
  0x0000,   /* U+12433 */
  0x0000,   /* U+12434 */
  0x0000,   /* U+12435 */
  0x0000,   /* U+12436 */
  0x0000,   /* U+12437 */
  0x0000,   /* U+12438 */
  0x0000,   /* U+12439 */
  0x0000,   /* U+1243A */
  0x0000,   /* U+1243B */
  0x0000,   /* U+1243C */
  0x0000,   /* U+1243D */
  0x0000,   /* U+1243E */
  0x0000,   /* U+1243F */
  0x0000,   /* U+12440 */
  0x0000,   /* U+12441 */
  0x0000,   /* U+12442 */
  0x0000,   /* U+12443 */
  0x0000,   /* U+12444 */
  0x0000,   /* U+12445 */
  0x0000,   /* U+12446 */
  0x0000,   /* U+12447 */
  0x0000,   /* U+12448 */
  0x0000,   /* U+12449 */
  0x0000,   /* U+1244A */
  0x0000,   /* U+1244B */
  0x0000,   /* U+1244C */
  0x0000,   /* U+1244D */
  0x0000,   /* U+1244E */
  0x0000,   /* U+1244F */
  0x0000,   /* U+12450 */
  0x0000,   /* U+12451 */
  0x0000,   /* U+12452 */
  0x0000,   /* U+12453 */
  0x0000,   /* U+12454 */
  0x0000,   /* U+12455 */
  0x0000,   /* U+12456 */
  0x0000,   /* U+12457 */
  0x0000,   /* U+12458 */
  0x0000,   /* U+12459 */
  0x0000,   /* U+1245A */
  0x0000,   /* U+1245B */
  0x0000,   /* U+1245C */
  0x0000,   /* U+1245D */
  0x0000,   /* U+1245E */
  0x0000,   /* U+1245F */
  0x0000,   /* U+12460 */
  0x0000,   /* U+12461 */
  0x0000,   /* U+12462 */
  0x0000,   /* U+12463 */
  0x0000,   /* U+12464 */
  0x0000,   /* U+12465 */
  0x0000,   /* U+12466 */
  0x0000,   /* U+12467 */
  0x0000,   /* U+12468 */
  0x0000,   /* U+12469 */
  0x0000,   /* U+1246A */
  0x0000,   /* U+1246B */
  0x0000,   /* U+1246C */
  0x0000,   /* U+1246D */
  0x0000,   /* U+1246E */
  0x0000,   /* U+1246F */
  0x0000,   /* U+12470 */
  0x0000,   /* U+12471 */
  0x0000,   /* U+12472 */
  0x0000,   /* U+12473 */
  0x0000,   /* U+12474 */
  0x0000,   /* U+12475 */
  0x0000,   /* U+12476 */
  0x0000,   /* U+12477 */
  0x0000,   /* U+12478 */
  0x0000,   /* U+12479 */
  0x0000,   /* U+1247A */
  0x0000,   /* U+1247B */
  0x0000,   /* U+1247C */
  0x0000,   /* U+1247D */
  0x0000,   /* U+1247E */
  0x0000,   /* U+1247F */
  0x0000,   /* U+12480 */
  0x0000,   /* U+12481 */
  0x0000,   /* U+12482 */
  0x0000,   /* U+12483 */
  0x0000,   /* U+12484 */
  0x0000,   /* U+12485 */
  0x0000,   /* U+12486 */
  0x0000,   /* U+12487 */
  0x0000,   /* U+12488 */
  0x0000,   /* U+12489 */
  0x0000,   /* U+1248A */
  0x0000,   /* U+1248B */
  0x0000,   /* U+1248C */
  0x0000,   /* U+1248D */
  0x0000,   /* U+1248E */
  0x0000,   /* U+1248F */
  0x0000,   /* U+12490 */
  0x0000,   /* U+12491 */
  0x0000,   /* U+12492 */
  0x0000,   /* U+12493 */
  0x0000,   /* U+12494 */
  0x0000,   /* U+12495 */
  0x0000,   /* U+12496 */
  0x0000,   /* U+12497 */
  0x0000,   /* U+12498 */
  0x0000,   /* U+12499 */
  0x0000,   /* U+1249A */
  0x0000,   /* U+1249B */
  0x0000,   /* U+1249C */
  0x0000,   /* U+1249D */
  0x0000,   /* U+1249E */
  0x0000,   /* U+1249F */
  0x0000,   /* U+124A0 */
  0x0000,   /* U+124A1 */
  0x0000,   /* U+124A2 */
  0x0000,   /* U+124A3 */
  0x0000,   /* U+124A4 */
  0x0000,   /* U+124A5 */
  0x0000,   /* U+124A6 */
  0x0000,   /* U+124A7 */
  0x0000,   /* U+124A8 */
  0x0000,   /* U+124A9 */
  0x0000,   /* U+124AA */
  0x0000,   /* U+124AB */
  0x0000,   /* U+124AC */
  0x0000,   /* U+124AD */
  0x0000,   /* U+124AE */
  0x0000,   /* U+124AF */
  0x0000,   /* U+124B0 */
  0x0000,   /* U+124B1 */
  0x0000,   /* U+124B2 */
  0x0000,   /* U+124B3 */
  0x0000,   /* U+124B4 */
  0x0000,   /* U+124B5 */
  0x0000,   /* U+124B6 */
  0x0000,   /* U+124B7 */
  0x0000,   /* U+124B8 */
  0x0000,   /* U+124B9 */
  0x0000,   /* U+124BA */
  0x0000,   /* U+124BB */
  0x0000,   /* U+124BC */
  0x0000,   /* U+124BD */
  0x0000,   /* U+124BE */
  0x0000,   /* U+124BF */
  0x0000,   /* U+124C0 */
  0x0000,   /* U+124C1 */
  0x0000,   /* U+124C2 */
  0x0000,   /* U+124C3 */
  0x0000,   /* U+124C4 */
  0x0000,   /* U+124C5 */
  0x0000,   /* U+124C6 */
  0x0000,   /* U+124C7 */
  0x0000,   /* U+124C8 */
  0x0000,   /* U+124C9 */
  0x0000,   /* U+124CA */
  0x0000,   /* U+124CB */
  0x0000,   /* U+124CC */
  0x0000,   /* U+124CD */
  0x0000,   /* U+124CE */
  0x0000,   /* U+124CF */
  0x0000,   /* U+124D0 */
  0x0000,   /* U+124D1 */
  0x0000,   /* U+124D2 */
  0x0000,   /* U+124D3 */
  0x0000,   /* U+124D4 */
  0x0000,   /* U+124D5 */
  0x0000,   /* U+124D6 */
  0x0000,   /* U+124D7 */
  0x0000,   /* U+124D8 */
  0x0000,   /* U+124D9 */
  0x0000,   /* U+124DA */
  0x0000,   /* U+124DB */
  0x0000,   /* U+124DC */
  0x0000,   /* U+124DD */
  0x0000,   /* U+124DE */
  0x0000,   /* U+124DF */
  0x0000,   /* U+124E0 */
  0x0000,   /* U+124E1 */
  0x0000,   /* U+124E2 */
  0x0000,   /* U+124E3 */
  0x0000,   /* U+124E4 */
  0x0000,   /* U+124E5 */
  0x0000,   /* U+124E6 */
  0x0000,   /* U+124E7 */
  0x0000,   /* U+124E8 */
  0x0000,   /* U+124E9 */
  0x0000,   /* U+124EA */
  0x0000,   /* U+124EB */
  0x0000,   /* U+124EC */
  0x0000,   /* U+124ED */
  0x0000,   /* U+124EE */
  0x0000,   /* U+124EF */
  0x0000,   /* U+124F0 */
  0x0000,   /* U+124F1 */
  0x0000,   /* U+124F2 */
  0x0000,   /* U+124F3 */
  0x0000,   /* U+124F4 */
  0x0000,   /* U+124F5 */
  0x0000,   /* U+124F6 */
  0x0000,   /* U+124F7 */
  0x0000,   /* U+124F8 */
  0x0000,   /* U+124F9 */
  0x0000,   /* U+124FA */
  0x0000,   /* U+124FB */
  0x0000,   /* U+124FC */
  0x0000,   /* U+124FD */
  0x0000,   /* U+124FE */
  0x0000,   /* U+124FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+12400 */
  0x0000,   /* U+12401 */
  0x0000,   /* U+12402 */
  0x0000,   /* U+12403 */
  0x0000,   /* U+12404 */
  0x0000,   /* U+12405 */
  0x0000,   /* U+12406 */
  0x0000,   /* U+12407 */
  0x0000,   /* U+12408 */
  0x0000,   /* U+12409 */
  0x0000,   /* U+1240A */
  0x0000,   /* U+1240B */
  0x0000,   /* U+1240C */
  0x0000,   /* U+1240D */
  0x0000,   /* U+1240E */
  0x0000,   /* U+1240F */
  0x0000,   /* U+12410 */
  0x0000,   /* U+12411 */
  0x0000,   /* U+12412 */
  0x0000,   /* U+12413 */
  0x0000,   /* U+12414 */
  0x0000,   /* U+12415 */
  0x0000,   /* U+12416 */
  0x0000,   /* U+12417 */
  0x0000,   /* U+12418 */
  0x0000,   /* U+12419 */
  0x0000,   /* U+1241A */
  0x0000,   /* U+1241B */
  0x0000,   /* U+1241C */
  0x0000,   /* U+1241D */
  0x0000,   /* U+1241E */
  0x0000,   /* U+1241F */
  0x0000,   /* U+12420 */
  0x0000,   /* U+12421 */
  0x0000,   /* U+12422 */
  0x0000,   /* U+12423 */
  0x0000,   /* U+12424 */
  0x0000,   /* U+12425 */
  0x0000,   /* U+12426 */
  0x0000,   /* U+12427 */
  0x0000,   /* U+12428 */
  0x0000,   /* U+12429 */
  0x0000,   /* U+1242A */
  0x0000,   /* U+1242B */
  0x0000,   /* U+1242C */
  0x0000,   /* U+1242D */
  0x0000,   /* U+1242E */
  0x0000,   /* U+1242F */
  0x0000,   /* U+12430 */
  0x0000,   /* U+12431 */
  0x0000,   /* U+12432 */
  0x0000,   /* U+12433 */
  0x0000,   /* U+12434 */
  0x0000,   /* U+12435 */
  0x0000,   /* U+12436 */
  0x0000,   /* U+12437 */
  0x0000,   /* U+12438 */
  0x0000,   /* U+12439 */
  0x0000,   /* U+1243A */
  0x0000,   /* U+1243B */
  0x0000,   /* U+1243C */
  0x0000,   /* U+1243D */
  0x0000,   /* U+1243E */
  0x0000,   /* U+1243F */
  0x0000,   /* U+12440 */
  0x0000,   /* U+12441 */
  0x0000,   /* U+12442 */
  0x0000,   /* U+12443 */
  0x0000,   /* U+12444 */
  0x0000,   /* U+12445 */
  0x0000,   /* U+12446 */
  0x0000,   /* U+12447 */
  0x0000,   /* U+12448 */
  0x0000,   /* U+12449 */
  0x0000,   /* U+1244A */
  0x0000,   /* U+1244B */
  0x0000,   /* U+1244C */
  0x0000,   /* U+1244D */
  0x0000,   /* U+1244E */
  0x0000,   /* U+1244F */
  0x0000,   /* U+12450 */
  0x0000,   /* U+12451 */
  0x0000,   /* U+12452 */
  0x0000,   /* U+12453 */
  0x0000,   /* U+12454 */
  0x0000,   /* U+12455 */
  0x0000,   /* U+12456 */
  0x0000,   /* U+12457 */
  0x0000,   /* U+12458 */
  0x0000,   /* U+12459 */
  0x0000,   /* U+1245A */
  0x0000,   /* U+1245B */
  0x0000,   /* U+1245C */
  0x0000,   /* U+1245D */
  0x0000,   /* U+1245E */
  0x0000,   /* U+1245F */
  0x0000,   /* U+12460 */
  0x0000,   /* U+12461 */
  0x0000,   /* U+12462 */
  0x0000,   /* U+12463 */
  0x0000,   /* U+12464 */
  0x0000,   /* U+12465 */
  0x0000,   /* U+12466 */
  0x0000,   /* U+12467 */
  0x0000,   /* U+12468 */
  0x0000,   /* U+12469 */
  0x0000,   /* U+1246A */
  0x0000,   /* U+1246B */
  0x0000,   /* U+1246C */
  0x0000,   /* U+1246D */
  0x0000,   /* U+1246E */
  0x0000,   /* U+1246F */
  0x0000,   /* U+12470 */
  0x0000,   /* U+12471 */
  0x0000,   /* U+12472 */
  0x0000,   /* U+12473 */
  0x0000,   /* U+12474 */
  0x0000,   /* U+12475 */
  0x0000,   /* U+12476 */
  0x0000,   /* U+12477 */
  0x0000,   /* U+12478 */
  0x0000,   /* U+12479 */
  0x0000,   /* U+1247A */
  0x0000,   /* U+1247B */
  0x0000,   /* U+1247C */
  0x0000,   /* U+1247D */
  0x0000,   /* U+1247E */
  0x0000,   /* U+1247F */
  0x0000,   /* U+12480 */
  0x0000,   /* U+12481 */
  0x0000,   /* U+12482 */
  0x0000,   /* U+12483 */
  0x0000,   /* U+12484 */
  0x0000,   /* U+12485 */
  0x0000,   /* U+12486 */
  0x0000,   /* U+12487 */
  0x0000,   /* U+12488 */
  0x0000,   /* U+12489 */
  0x0000,   /* U+1248A */
  0x0000,   /* U+1248B */
  0x0000,   /* U+1248C */
  0x0000,   /* U+1248D */
  0x0000,   /* U+1248E */
  0x0000,   /* U+1248F */
  0x0000,   /* U+12490 */
  0x0000,   /* U+12491 */
  0x0000,   /* U+12492 */
  0x0000,   /* U+12493 */
  0x0000,   /* U+12494 */
  0x0000,   /* U+12495 */
  0x0000,   /* U+12496 */
  0x0000,   /* U+12497 */
  0x0000,   /* U+12498 */
  0x0000,   /* U+12499 */
  0x0000,   /* U+1249A */
  0x0000,   /* U+1249B */
  0x0000,   /* U+1249C */
  0x0000,   /* U+1249D */
  0x0000,   /* U+1249E */
  0x0000,   /* U+1249F */
  0x0000,   /* U+124A0 */
  0x0000,   /* U+124A1 */
  0x0000,   /* U+124A2 */
  0x0000,   /* U+124A3 */
  0x0000,   /* U+124A4 */
  0x0000,   /* U+124A5 */
  0x0000,   /* U+124A6 */
  0x0000,   /* U+124A7 */
  0x0000,   /* U+124A8 */
  0x0000,   /* U+124A9 */
  0x0000,   /* U+124AA */
  0x0000,   /* U+124AB */
  0x0000,   /* U+124AC */
  0x0000,   /* U+124AD */
  0x0000,   /* U+124AE */
  0x0000,   /* U+124AF */
  0x0000,   /* U+124B0 */
  0x0000,   /* U+124B1 */
  0x0000,   /* U+124B2 */
  0x0000,   /* U+124B3 */
  0x0000,   /* U+124B4 */
  0x0000,   /* U+124B5 */
  0x0000,   /* U+124B6 */
  0x0000,   /* U+124B7 */
  0x0000,   /* U+124B8 */
  0x0000,   /* U+124B9 */
  0x0000,   /* U+124BA */
  0x0000,   /* U+124BB */
  0x0000,   /* U+124BC */
  0x0000,   /* U+124BD */
  0x0000,   /* U+124BE */
  0x0000,   /* U+124BF */
  0x0000,   /* U+124C0 */
  0x0000,   /* U+124C1 */
  0x0000,   /* U+124C2 */
  0x0000,   /* U+124C3 */
  0x0000,   /* U+124C4 */
  0x0000,   /* U+124C5 */
  0x0000,   /* U+124C6 */
  0x0000,   /* U+124C7 */
  0x0000,   /* U+124C8 */
  0x0000,   /* U+124C9 */
  0x0000,   /* U+124CA */
  0x0000,   /* U+124CB */
  0x0000,   /* U+124CC */
  0x0000,   /* U+124CD */
  0x0000,   /* U+124CE */
  0x0000,   /* U+124CF */
  0x0000,   /* U+124D0 */
  0x0000,   /* U+124D1 */
  0x0000,   /* U+124D2 */
  0x0000,   /* U+124D3 */
  0x0000,   /* U+124D4 */
  0x0000,   /* U+124D5 */
  0x0000,   /* U+124D6 */
  0x0000,   /* U+124D7 */
  0x0000,   /* U+124D8 */
  0x0000,   /* U+124D9 */
  0x0000,   /* U+124DA */
  0x0000,   /* U+124DB */
  0x0000,   /* U+124DC */
  0x0000,   /* U+124DD */
  0x0000,   /* U+124DE */
  0x0000,   /* U+124DF */
  0x0000,   /* U+124E0 */
  0x0000,   /* U+124E1 */
  0x0000,   /* U+124E2 */
  0x0000,   /* U+124E3 */
  0x0000,   /* U+124E4 */
  0x0000,   /* U+124E5 */
  0x0000,   /* U+124E6 */
  0x0000,   /* U+124E7 */
  0x0000,   /* U+124E8 */
  0x0000,   /* U+124E9 */
  0x0000,   /* U+124EA */
  0x0000,   /* U+124EB */
  0x0000,   /* U+124EC */
  0x0000,   /* U+124ED */
  0x0000,   /* U+124EE */
  0x0000,   /* U+124EF */
  0x0000,   /* U+124F0 */
  0x0000,   /* U+124F1 */
  0x0000,   /* U+124F2 */
  0x0000,   /* U+124F3 */
  0x0000,   /* U+124F4 */
  0x0000,   /* U+124F5 */
  0x0000,   /* U+124F6 */
  0x0000,   /* U+124F7 */
  0x0000,   /* U+124F8 */
  0x0000,   /* U+124F9 */
  0x0000,   /* U+124FA */
  0x0000,   /* U+124FB */
  0x0000,   /* U+124FC */
  0x0000,   /* U+124FD */
  0x0000,   /* U+124FE */
  0x0000,   /* U+124FF */
};

uint16 uca900_p125[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4DCF,   /* U+12500 */
  0x4DD0,   /* U+12501 */
  0x4DD1,   /* U+12502 */
  0x4DD2,   /* U+12503 */
  0x4DD3,   /* U+12504 */
  0x4DD4,   /* U+12505 */
  0x4DD5,   /* U+12506 */
  0x4DD6,   /* U+12507 */
  0x4DD7,   /* U+12508 */
  0x4DD8,   /* U+12509 */
  0x4DD9,   /* U+1250A */
  0x4DDA,   /* U+1250B */
  0x4DDB,   /* U+1250C */
  0x4DDC,   /* U+1250D */
  0x4DDD,   /* U+1250E */
  0x4DDE,   /* U+1250F */
  0x4DDF,   /* U+12510 */
  0x4DE0,   /* U+12511 */
  0x4DE1,   /* U+12512 */
  0x4DE2,   /* U+12513 */
  0x4DE3,   /* U+12514 */
  0x4DE4,   /* U+12515 */
  0x4DE5,   /* U+12516 */
  0x4DE6,   /* U+12517 */
  0x4DE7,   /* U+12518 */
  0x4DE8,   /* U+12519 */
  0x4DE9,   /* U+1251A */
  0x4DEA,   /* U+1251B */
  0x4DEB,   /* U+1251C */
  0x4DEC,   /* U+1251D */
  0x4DED,   /* U+1251E */
  0x4DEE,   /* U+1251F */
  0x4DEF,   /* U+12520 */
  0x4DF0,   /* U+12521 */
  0x4DF1,   /* U+12522 */
  0x4DF2,   /* U+12523 */
  0x4DF3,   /* U+12524 */
  0x4DF4,   /* U+12525 */
  0x4DF5,   /* U+12526 */
  0x4DF6,   /* U+12527 */
  0x4DF7,   /* U+12528 */
  0x4DF8,   /* U+12529 */
  0x4DF9,   /* U+1252A */
  0x4DFA,   /* U+1252B */
  0x4DFB,   /* U+1252C */
  0x4DFC,   /* U+1252D */
  0x4DFD,   /* U+1252E */
  0x4DFE,   /* U+1252F */
  0x4DFF,   /* U+12530 */
  0x4E00,   /* U+12531 */
  0x4E01,   /* U+12532 */
  0x4E02,   /* U+12533 */
  0x4E03,   /* U+12534 */
  0x4E04,   /* U+12535 */
  0x4E05,   /* U+12536 */
  0x4E06,   /* U+12537 */
  0x4E07,   /* U+12538 */
  0x4E08,   /* U+12539 */
  0x4E09,   /* U+1253A */
  0x4E0A,   /* U+1253B */
  0x4E0B,   /* U+1253C */
  0x4E0C,   /* U+1253D */
  0x4E0D,   /* U+1253E */
  0x4E0E,   /* U+1253F */
  0x4E0F,   /* U+12540 */
  0x4E10,   /* U+12541 */
  0x4E11,   /* U+12542 */
  0x4E12,   /* U+12543 */
  0xFBC2,   /* U+12544 */
  0xFBC2,   /* U+12545 */
  0xFBC2,   /* U+12546 */
  0xFBC2,   /* U+12547 */
  0xFBC2,   /* U+12548 */
  0xFBC2,   /* U+12549 */
  0xFBC2,   /* U+1254A */
  0xFBC2,   /* U+1254B */
  0xFBC2,   /* U+1254C */
  0xFBC2,   /* U+1254D */
  0xFBC2,   /* U+1254E */
  0xFBC2,   /* U+1254F */
  0xFBC2,   /* U+12550 */
  0xFBC2,   /* U+12551 */
  0xFBC2,   /* U+12552 */
  0xFBC2,   /* U+12553 */
  0xFBC2,   /* U+12554 */
  0xFBC2,   /* U+12555 */
  0xFBC2,   /* U+12556 */
  0xFBC2,   /* U+12557 */
  0xFBC2,   /* U+12558 */
  0xFBC2,   /* U+12559 */
  0xFBC2,   /* U+1255A */
  0xFBC2,   /* U+1255B */
  0xFBC2,   /* U+1255C */
  0xFBC2,   /* U+1255D */
  0xFBC2,   /* U+1255E */
  0xFBC2,   /* U+1255F */
  0xFBC2,   /* U+12560 */
  0xFBC2,   /* U+12561 */
  0xFBC2,   /* U+12562 */
  0xFBC2,   /* U+12563 */
  0xFBC2,   /* U+12564 */
  0xFBC2,   /* U+12565 */
  0xFBC2,   /* U+12566 */
  0xFBC2,   /* U+12567 */
  0xFBC2,   /* U+12568 */
  0xFBC2,   /* U+12569 */
  0xFBC2,   /* U+1256A */
  0xFBC2,   /* U+1256B */
  0xFBC2,   /* U+1256C */
  0xFBC2,   /* U+1256D */
  0xFBC2,   /* U+1256E */
  0xFBC2,   /* U+1256F */
  0xFBC2,   /* U+12570 */
  0xFBC2,   /* U+12571 */
  0xFBC2,   /* U+12572 */
  0xFBC2,   /* U+12573 */
  0xFBC2,   /* U+12574 */
  0xFBC2,   /* U+12575 */
  0xFBC2,   /* U+12576 */
  0xFBC2,   /* U+12577 */
  0xFBC2,   /* U+12578 */
  0xFBC2,   /* U+12579 */
  0xFBC2,   /* U+1257A */
  0xFBC2,   /* U+1257B */
  0xFBC2,   /* U+1257C */
  0xFBC2,   /* U+1257D */
  0xFBC2,   /* U+1257E */
  0xFBC2,   /* U+1257F */
  0xFBC2,   /* U+12580 */
  0xFBC2,   /* U+12581 */
  0xFBC2,   /* U+12582 */
  0xFBC2,   /* U+12583 */
  0xFBC2,   /* U+12584 */
  0xFBC2,   /* U+12585 */
  0xFBC2,   /* U+12586 */
  0xFBC2,   /* U+12587 */
  0xFBC2,   /* U+12588 */
  0xFBC2,   /* U+12589 */
  0xFBC2,   /* U+1258A */
  0xFBC2,   /* U+1258B */
  0xFBC2,   /* U+1258C */
  0xFBC2,   /* U+1258D */
  0xFBC2,   /* U+1258E */
  0xFBC2,   /* U+1258F */
  0xFBC2,   /* U+12590 */
  0xFBC2,   /* U+12591 */
  0xFBC2,   /* U+12592 */
  0xFBC2,   /* U+12593 */
  0xFBC2,   /* U+12594 */
  0xFBC2,   /* U+12595 */
  0xFBC2,   /* U+12596 */
  0xFBC2,   /* U+12597 */
  0xFBC2,   /* U+12598 */
  0xFBC2,   /* U+12599 */
  0xFBC2,   /* U+1259A */
  0xFBC2,   /* U+1259B */
  0xFBC2,   /* U+1259C */
  0xFBC2,   /* U+1259D */
  0xFBC2,   /* U+1259E */
  0xFBC2,   /* U+1259F */
  0xFBC2,   /* U+125A0 */
  0xFBC2,   /* U+125A1 */
  0xFBC2,   /* U+125A2 */
  0xFBC2,   /* U+125A3 */
  0xFBC2,   /* U+125A4 */
  0xFBC2,   /* U+125A5 */
  0xFBC2,   /* U+125A6 */
  0xFBC2,   /* U+125A7 */
  0xFBC2,   /* U+125A8 */
  0xFBC2,   /* U+125A9 */
  0xFBC2,   /* U+125AA */
  0xFBC2,   /* U+125AB */
  0xFBC2,   /* U+125AC */
  0xFBC2,   /* U+125AD */
  0xFBC2,   /* U+125AE */
  0xFBC2,   /* U+125AF */
  0xFBC2,   /* U+125B0 */
  0xFBC2,   /* U+125B1 */
  0xFBC2,   /* U+125B2 */
  0xFBC2,   /* U+125B3 */
  0xFBC2,   /* U+125B4 */
  0xFBC2,   /* U+125B5 */
  0xFBC2,   /* U+125B6 */
  0xFBC2,   /* U+125B7 */
  0xFBC2,   /* U+125B8 */
  0xFBC2,   /* U+125B9 */
  0xFBC2,   /* U+125BA */
  0xFBC2,   /* U+125BB */
  0xFBC2,   /* U+125BC */
  0xFBC2,   /* U+125BD */
  0xFBC2,   /* U+125BE */
  0xFBC2,   /* U+125BF */
  0xFBC2,   /* U+125C0 */
  0xFBC2,   /* U+125C1 */
  0xFBC2,   /* U+125C2 */
  0xFBC2,   /* U+125C3 */
  0xFBC2,   /* U+125C4 */
  0xFBC2,   /* U+125C5 */
  0xFBC2,   /* U+125C6 */
  0xFBC2,   /* U+125C7 */
  0xFBC2,   /* U+125C8 */
  0xFBC2,   /* U+125C9 */
  0xFBC2,   /* U+125CA */
  0xFBC2,   /* U+125CB */
  0xFBC2,   /* U+125CC */
  0xFBC2,   /* U+125CD */
  0xFBC2,   /* U+125CE */
  0xFBC2,   /* U+125CF */
  0xFBC2,   /* U+125D0 */
  0xFBC2,   /* U+125D1 */
  0xFBC2,   /* U+125D2 */
  0xFBC2,   /* U+125D3 */
  0xFBC2,   /* U+125D4 */
  0xFBC2,   /* U+125D5 */
  0xFBC2,   /* U+125D6 */
  0xFBC2,   /* U+125D7 */
  0xFBC2,   /* U+125D8 */
  0xFBC2,   /* U+125D9 */
  0xFBC2,   /* U+125DA */
  0xFBC2,   /* U+125DB */
  0xFBC2,   /* U+125DC */
  0xFBC2,   /* U+125DD */
  0xFBC2,   /* U+125DE */
  0xFBC2,   /* U+125DF */
  0xFBC2,   /* U+125E0 */
  0xFBC2,   /* U+125E1 */
  0xFBC2,   /* U+125E2 */
  0xFBC2,   /* U+125E3 */
  0xFBC2,   /* U+125E4 */
  0xFBC2,   /* U+125E5 */
  0xFBC2,   /* U+125E6 */
  0xFBC2,   /* U+125E7 */
  0xFBC2,   /* U+125E8 */
  0xFBC2,   /* U+125E9 */
  0xFBC2,   /* U+125EA */
  0xFBC2,   /* U+125EB */
  0xFBC2,   /* U+125EC */
  0xFBC2,   /* U+125ED */
  0xFBC2,   /* U+125EE */
  0xFBC2,   /* U+125EF */
  0xFBC2,   /* U+125F0 */
  0xFBC2,   /* U+125F1 */
  0xFBC2,   /* U+125F2 */
  0xFBC2,   /* U+125F3 */
  0xFBC2,   /* U+125F4 */
  0xFBC2,   /* U+125F5 */
  0xFBC2,   /* U+125F6 */
  0xFBC2,   /* U+125F7 */
  0xFBC2,   /* U+125F8 */
  0xFBC2,   /* U+125F9 */
  0xFBC2,   /* U+125FA */
  0xFBC2,   /* U+125FB */
  0xFBC2,   /* U+125FC */
  0xFBC2,   /* U+125FD */
  0xFBC2,   /* U+125FE */
  0xFBC2,   /* U+125FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+12500 */
  0x0020,   /* U+12501 */
  0x0020,   /* U+12502 */
  0x0020,   /* U+12503 */
  0x0020,   /* U+12504 */
  0x0020,   /* U+12505 */
  0x0020,   /* U+12506 */
  0x0020,   /* U+12507 */
  0x0020,   /* U+12508 */
  0x0020,   /* U+12509 */
  0x0020,   /* U+1250A */
  0x0020,   /* U+1250B */
  0x0020,   /* U+1250C */
  0x0020,   /* U+1250D */
  0x0020,   /* U+1250E */
  0x0020,   /* U+1250F */
  0x0020,   /* U+12510 */
  0x0020,   /* U+12511 */
  0x0020,   /* U+12512 */
  0x0020,   /* U+12513 */
  0x0020,   /* U+12514 */
  0x0020,   /* U+12515 */
  0x0020,   /* U+12516 */
  0x0020,   /* U+12517 */
  0x0020,   /* U+12518 */
  0x0020,   /* U+12519 */
  0x0020,   /* U+1251A */
  0x0020,   /* U+1251B */
  0x0020,   /* U+1251C */
  0x0020,   /* U+1251D */
  0x0020,   /* U+1251E */
  0x0020,   /* U+1251F */
  0x0020,   /* U+12520 */
  0x0020,   /* U+12521 */
  0x0020,   /* U+12522 */
  0x0020,   /* U+12523 */
  0x0020,   /* U+12524 */
  0x0020,   /* U+12525 */
  0x0020,   /* U+12526 */
  0x0020,   /* U+12527 */
  0x0020,   /* U+12528 */
  0x0020,   /* U+12529 */
  0x0020,   /* U+1252A */
  0x0020,   /* U+1252B */
  0x0020,   /* U+1252C */
  0x0020,   /* U+1252D */
  0x0020,   /* U+1252E */
  0x0020,   /* U+1252F */
  0x0020,   /* U+12530 */
  0x0020,   /* U+12531 */
  0x0020,   /* U+12532 */
  0x0020,   /* U+12533 */
  0x0020,   /* U+12534 */
  0x0020,   /* U+12535 */
  0x0020,   /* U+12536 */
  0x0020,   /* U+12537 */
  0x0020,   /* U+12538 */
  0x0020,   /* U+12539 */
  0x0020,   /* U+1253A */
  0x0020,   /* U+1253B */
  0x0020,   /* U+1253C */
  0x0020,   /* U+1253D */
  0x0020,   /* U+1253E */
  0x0020,   /* U+1253F */
  0x0020,   /* U+12540 */
  0x0020,   /* U+12541 */
  0x0020,   /* U+12542 */
  0x0020,   /* U+12543 */
  0x0020,   /* U+12544 */
  0x0020,   /* U+12545 */
  0x0020,   /* U+12546 */
  0x0020,   /* U+12547 */
  0x0020,   /* U+12548 */
  0x0020,   /* U+12549 */
  0x0020,   /* U+1254A */
  0x0020,   /* U+1254B */
  0x0020,   /* U+1254C */
  0x0020,   /* U+1254D */
  0x0020,   /* U+1254E */
  0x0020,   /* U+1254F */
  0x0020,   /* U+12550 */
  0x0020,   /* U+12551 */
  0x0020,   /* U+12552 */
  0x0020,   /* U+12553 */
  0x0020,   /* U+12554 */
  0x0020,   /* U+12555 */
  0x0020,   /* U+12556 */
  0x0020,   /* U+12557 */
  0x0020,   /* U+12558 */
  0x0020,   /* U+12559 */
  0x0020,   /* U+1255A */
  0x0020,   /* U+1255B */
  0x0020,   /* U+1255C */
  0x0020,   /* U+1255D */
  0x0020,   /* U+1255E */
  0x0020,   /* U+1255F */
  0x0020,   /* U+12560 */
  0x0020,   /* U+12561 */
  0x0020,   /* U+12562 */
  0x0020,   /* U+12563 */
  0x0020,   /* U+12564 */
  0x0020,   /* U+12565 */
  0x0020,   /* U+12566 */
  0x0020,   /* U+12567 */
  0x0020,   /* U+12568 */
  0x0020,   /* U+12569 */
  0x0020,   /* U+1256A */
  0x0020,   /* U+1256B */
  0x0020,   /* U+1256C */
  0x0020,   /* U+1256D */
  0x0020,   /* U+1256E */
  0x0020,   /* U+1256F */
  0x0020,   /* U+12570 */
  0x0020,   /* U+12571 */
  0x0020,   /* U+12572 */
  0x0020,   /* U+12573 */
  0x0020,   /* U+12574 */
  0x0020,   /* U+12575 */
  0x0020,   /* U+12576 */
  0x0020,   /* U+12577 */
  0x0020,   /* U+12578 */
  0x0020,   /* U+12579 */
  0x0020,   /* U+1257A */
  0x0020,   /* U+1257B */
  0x0020,   /* U+1257C */
  0x0020,   /* U+1257D */
  0x0020,   /* U+1257E */
  0x0020,   /* U+1257F */
  0x0020,   /* U+12580 */
  0x0020,   /* U+12581 */
  0x0020,   /* U+12582 */
  0x0020,   /* U+12583 */
  0x0020,   /* U+12584 */
  0x0020,   /* U+12585 */
  0x0020,   /* U+12586 */
  0x0020,   /* U+12587 */
  0x0020,   /* U+12588 */
  0x0020,   /* U+12589 */
  0x0020,   /* U+1258A */
  0x0020,   /* U+1258B */
  0x0020,   /* U+1258C */
  0x0020,   /* U+1258D */
  0x0020,   /* U+1258E */
  0x0020,   /* U+1258F */
  0x0020,   /* U+12590 */
  0x0020,   /* U+12591 */
  0x0020,   /* U+12592 */
  0x0020,   /* U+12593 */
  0x0020,   /* U+12594 */
  0x0020,   /* U+12595 */
  0x0020,   /* U+12596 */
  0x0020,   /* U+12597 */
  0x0020,   /* U+12598 */
  0x0020,   /* U+12599 */
  0x0020,   /* U+1259A */
  0x0020,   /* U+1259B */
  0x0020,   /* U+1259C */
  0x0020,   /* U+1259D */
  0x0020,   /* U+1259E */
  0x0020,   /* U+1259F */
  0x0020,   /* U+125A0 */
  0x0020,   /* U+125A1 */
  0x0020,   /* U+125A2 */
  0x0020,   /* U+125A3 */
  0x0020,   /* U+125A4 */
  0x0020,   /* U+125A5 */
  0x0020,   /* U+125A6 */
  0x0020,   /* U+125A7 */
  0x0020,   /* U+125A8 */
  0x0020,   /* U+125A9 */
  0x0020,   /* U+125AA */
  0x0020,   /* U+125AB */
  0x0020,   /* U+125AC */
  0x0020,   /* U+125AD */
  0x0020,   /* U+125AE */
  0x0020,   /* U+125AF */
  0x0020,   /* U+125B0 */
  0x0020,   /* U+125B1 */
  0x0020,   /* U+125B2 */
  0x0020,   /* U+125B3 */
  0x0020,   /* U+125B4 */
  0x0020,   /* U+125B5 */
  0x0020,   /* U+125B6 */
  0x0020,   /* U+125B7 */
  0x0020,   /* U+125B8 */
  0x0020,   /* U+125B9 */
  0x0020,   /* U+125BA */
  0x0020,   /* U+125BB */
  0x0020,   /* U+125BC */
  0x0020,   /* U+125BD */
  0x0020,   /* U+125BE */
  0x0020,   /* U+125BF */
  0x0020,   /* U+125C0 */
  0x0020,   /* U+125C1 */
  0x0020,   /* U+125C2 */
  0x0020,   /* U+125C3 */
  0x0020,   /* U+125C4 */
  0x0020,   /* U+125C5 */
  0x0020,   /* U+125C6 */
  0x0020,   /* U+125C7 */
  0x0020,   /* U+125C8 */
  0x0020,   /* U+125C9 */
  0x0020,   /* U+125CA */
  0x0020,   /* U+125CB */
  0x0020,   /* U+125CC */
  0x0020,   /* U+125CD */
  0x0020,   /* U+125CE */
  0x0020,   /* U+125CF */
  0x0020,   /* U+125D0 */
  0x0020,   /* U+125D1 */
  0x0020,   /* U+125D2 */
  0x0020,   /* U+125D3 */
  0x0020,   /* U+125D4 */
  0x0020,   /* U+125D5 */
  0x0020,   /* U+125D6 */
  0x0020,   /* U+125D7 */
  0x0020,   /* U+125D8 */
  0x0020,   /* U+125D9 */
  0x0020,   /* U+125DA */
  0x0020,   /* U+125DB */
  0x0020,   /* U+125DC */
  0x0020,   /* U+125DD */
  0x0020,   /* U+125DE */
  0x0020,   /* U+125DF */
  0x0020,   /* U+125E0 */
  0x0020,   /* U+125E1 */
  0x0020,   /* U+125E2 */
  0x0020,   /* U+125E3 */
  0x0020,   /* U+125E4 */
  0x0020,   /* U+125E5 */
  0x0020,   /* U+125E6 */
  0x0020,   /* U+125E7 */
  0x0020,   /* U+125E8 */
  0x0020,   /* U+125E9 */
  0x0020,   /* U+125EA */
  0x0020,   /* U+125EB */
  0x0020,   /* U+125EC */
  0x0020,   /* U+125ED */
  0x0020,   /* U+125EE */
  0x0020,   /* U+125EF */
  0x0020,   /* U+125F0 */
  0x0020,   /* U+125F1 */
  0x0020,   /* U+125F2 */
  0x0020,   /* U+125F3 */
  0x0020,   /* U+125F4 */
  0x0020,   /* U+125F5 */
  0x0020,   /* U+125F6 */
  0x0020,   /* U+125F7 */
  0x0020,   /* U+125F8 */
  0x0020,   /* U+125F9 */
  0x0020,   /* U+125FA */
  0x0020,   /* U+125FB */
  0x0020,   /* U+125FC */
  0x0020,   /* U+125FD */
  0x0020,   /* U+125FE */
  0x0020,   /* U+125FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+12500 */
  0x0002,   /* U+12501 */
  0x0002,   /* U+12502 */
  0x0002,   /* U+12503 */
  0x0002,   /* U+12504 */
  0x0002,   /* U+12505 */
  0x0002,   /* U+12506 */
  0x0002,   /* U+12507 */
  0x0002,   /* U+12508 */
  0x0002,   /* U+12509 */
  0x0002,   /* U+1250A */
  0x0002,   /* U+1250B */
  0x0002,   /* U+1250C */
  0x0002,   /* U+1250D */
  0x0002,   /* U+1250E */
  0x0002,   /* U+1250F */
  0x0002,   /* U+12510 */
  0x0002,   /* U+12511 */
  0x0002,   /* U+12512 */
  0x0002,   /* U+12513 */
  0x0002,   /* U+12514 */
  0x0002,   /* U+12515 */
  0x0002,   /* U+12516 */
  0x0002,   /* U+12517 */
  0x0002,   /* U+12518 */
  0x0002,   /* U+12519 */
  0x0002,   /* U+1251A */
  0x0002,   /* U+1251B */
  0x0002,   /* U+1251C */
  0x0002,   /* U+1251D */
  0x0002,   /* U+1251E */
  0x0002,   /* U+1251F */
  0x0002,   /* U+12520 */
  0x0002,   /* U+12521 */
  0x0002,   /* U+12522 */
  0x0002,   /* U+12523 */
  0x0002,   /* U+12524 */
  0x0002,   /* U+12525 */
  0x0002,   /* U+12526 */
  0x0002,   /* U+12527 */
  0x0002,   /* U+12528 */
  0x0002,   /* U+12529 */
  0x0002,   /* U+1252A */
  0x0002,   /* U+1252B */
  0x0002,   /* U+1252C */
  0x0002,   /* U+1252D */
  0x0002,   /* U+1252E */
  0x0002,   /* U+1252F */
  0x0002,   /* U+12530 */
  0x0002,   /* U+12531 */
  0x0002,   /* U+12532 */
  0x0002,   /* U+12533 */
  0x0002,   /* U+12534 */
  0x0002,   /* U+12535 */
  0x0002,   /* U+12536 */
  0x0002,   /* U+12537 */
  0x0002,   /* U+12538 */
  0x0002,   /* U+12539 */
  0x0002,   /* U+1253A */
  0x0002,   /* U+1253B */
  0x0002,   /* U+1253C */
  0x0002,   /* U+1253D */
  0x0002,   /* U+1253E */
  0x0002,   /* U+1253F */
  0x0002,   /* U+12540 */
  0x0002,   /* U+12541 */
  0x0002,   /* U+12542 */
  0x0002,   /* U+12543 */
  0x0002,   /* U+12544 */
  0x0002,   /* U+12545 */
  0x0002,   /* U+12546 */
  0x0002,   /* U+12547 */
  0x0002,   /* U+12548 */
  0x0002,   /* U+12549 */
  0x0002,   /* U+1254A */
  0x0002,   /* U+1254B */
  0x0002,   /* U+1254C */
  0x0002,   /* U+1254D */
  0x0002,   /* U+1254E */
  0x0002,   /* U+1254F */
  0x0002,   /* U+12550 */
  0x0002,   /* U+12551 */
  0x0002,   /* U+12552 */
  0x0002,   /* U+12553 */
  0x0002,   /* U+12554 */
  0x0002,   /* U+12555 */
  0x0002,   /* U+12556 */
  0x0002,   /* U+12557 */
  0x0002,   /* U+12558 */
  0x0002,   /* U+12559 */
  0x0002,   /* U+1255A */
  0x0002,   /* U+1255B */
  0x0002,   /* U+1255C */
  0x0002,   /* U+1255D */
  0x0002,   /* U+1255E */
  0x0002,   /* U+1255F */
  0x0002,   /* U+12560 */
  0x0002,   /* U+12561 */
  0x0002,   /* U+12562 */
  0x0002,   /* U+12563 */
  0x0002,   /* U+12564 */
  0x0002,   /* U+12565 */
  0x0002,   /* U+12566 */
  0x0002,   /* U+12567 */
  0x0002,   /* U+12568 */
  0x0002,   /* U+12569 */
  0x0002,   /* U+1256A */
  0x0002,   /* U+1256B */
  0x0002,   /* U+1256C */
  0x0002,   /* U+1256D */
  0x0002,   /* U+1256E */
  0x0002,   /* U+1256F */
  0x0002,   /* U+12570 */
  0x0002,   /* U+12571 */
  0x0002,   /* U+12572 */
  0x0002,   /* U+12573 */
  0x0002,   /* U+12574 */
  0x0002,   /* U+12575 */
  0x0002,   /* U+12576 */
  0x0002,   /* U+12577 */
  0x0002,   /* U+12578 */
  0x0002,   /* U+12579 */
  0x0002,   /* U+1257A */
  0x0002,   /* U+1257B */
  0x0002,   /* U+1257C */
  0x0002,   /* U+1257D */
  0x0002,   /* U+1257E */
  0x0002,   /* U+1257F */
  0x0002,   /* U+12580 */
  0x0002,   /* U+12581 */
  0x0002,   /* U+12582 */
  0x0002,   /* U+12583 */
  0x0002,   /* U+12584 */
  0x0002,   /* U+12585 */
  0x0002,   /* U+12586 */
  0x0002,   /* U+12587 */
  0x0002,   /* U+12588 */
  0x0002,   /* U+12589 */
  0x0002,   /* U+1258A */
  0x0002,   /* U+1258B */
  0x0002,   /* U+1258C */
  0x0002,   /* U+1258D */
  0x0002,   /* U+1258E */
  0x0002,   /* U+1258F */
  0x0002,   /* U+12590 */
  0x0002,   /* U+12591 */
  0x0002,   /* U+12592 */
  0x0002,   /* U+12593 */
  0x0002,   /* U+12594 */
  0x0002,   /* U+12595 */
  0x0002,   /* U+12596 */
  0x0002,   /* U+12597 */
  0x0002,   /* U+12598 */
  0x0002,   /* U+12599 */
  0x0002,   /* U+1259A */
  0x0002,   /* U+1259B */
  0x0002,   /* U+1259C */
  0x0002,   /* U+1259D */
  0x0002,   /* U+1259E */
  0x0002,   /* U+1259F */
  0x0002,   /* U+125A0 */
  0x0002,   /* U+125A1 */
  0x0002,   /* U+125A2 */
  0x0002,   /* U+125A3 */
  0x0002,   /* U+125A4 */
  0x0002,   /* U+125A5 */
  0x0002,   /* U+125A6 */
  0x0002,   /* U+125A7 */
  0x0002,   /* U+125A8 */
  0x0002,   /* U+125A9 */
  0x0002,   /* U+125AA */
  0x0002,   /* U+125AB */
  0x0002,   /* U+125AC */
  0x0002,   /* U+125AD */
  0x0002,   /* U+125AE */
  0x0002,   /* U+125AF */
  0x0002,   /* U+125B0 */
  0x0002,   /* U+125B1 */
  0x0002,   /* U+125B2 */
  0x0002,   /* U+125B3 */
  0x0002,   /* U+125B4 */
  0x0002,   /* U+125B5 */
  0x0002,   /* U+125B6 */
  0x0002,   /* U+125B7 */
  0x0002,   /* U+125B8 */
  0x0002,   /* U+125B9 */
  0x0002,   /* U+125BA */
  0x0002,   /* U+125BB */
  0x0002,   /* U+125BC */
  0x0002,   /* U+125BD */
  0x0002,   /* U+125BE */
  0x0002,   /* U+125BF */
  0x0002,   /* U+125C0 */
  0x0002,   /* U+125C1 */
  0x0002,   /* U+125C2 */
  0x0002,   /* U+125C3 */
  0x0002,   /* U+125C4 */
  0x0002,   /* U+125C5 */
  0x0002,   /* U+125C6 */
  0x0002,   /* U+125C7 */
  0x0002,   /* U+125C8 */
  0x0002,   /* U+125C9 */
  0x0002,   /* U+125CA */
  0x0002,   /* U+125CB */
  0x0002,   /* U+125CC */
  0x0002,   /* U+125CD */
  0x0002,   /* U+125CE */
  0x0002,   /* U+125CF */
  0x0002,   /* U+125D0 */
  0x0002,   /* U+125D1 */
  0x0002,   /* U+125D2 */
  0x0002,   /* U+125D3 */
  0x0002,   /* U+125D4 */
  0x0002,   /* U+125D5 */
  0x0002,   /* U+125D6 */
  0x0002,   /* U+125D7 */
  0x0002,   /* U+125D8 */
  0x0002,   /* U+125D9 */
  0x0002,   /* U+125DA */
  0x0002,   /* U+125DB */
  0x0002,   /* U+125DC */
  0x0002,   /* U+125DD */
  0x0002,   /* U+125DE */
  0x0002,   /* U+125DF */
  0x0002,   /* U+125E0 */
  0x0002,   /* U+125E1 */
  0x0002,   /* U+125E2 */
  0x0002,   /* U+125E3 */
  0x0002,   /* U+125E4 */
  0x0002,   /* U+125E5 */
  0x0002,   /* U+125E6 */
  0x0002,   /* U+125E7 */
  0x0002,   /* U+125E8 */
  0x0002,   /* U+125E9 */
  0x0002,   /* U+125EA */
  0x0002,   /* U+125EB */
  0x0002,   /* U+125EC */
  0x0002,   /* U+125ED */
  0x0002,   /* U+125EE */
  0x0002,   /* U+125EF */
  0x0002,   /* U+125F0 */
  0x0002,   /* U+125F1 */
  0x0002,   /* U+125F2 */
  0x0002,   /* U+125F3 */
  0x0002,   /* U+125F4 */
  0x0002,   /* U+125F5 */
  0x0002,   /* U+125F6 */
  0x0002,   /* U+125F7 */
  0x0002,   /* U+125F8 */
  0x0002,   /* U+125F9 */
  0x0002,   /* U+125FA */
  0x0002,   /* U+125FB */
  0x0002,   /* U+125FC */
  0x0002,   /* U+125FD */
  0x0002,   /* U+125FE */
  0x0002,   /* U+125FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+12500 */
  0x0000,   /* U+12501 */
  0x0000,   /* U+12502 */
  0x0000,   /* U+12503 */
  0x0000,   /* U+12504 */
  0x0000,   /* U+12505 */
  0x0000,   /* U+12506 */
  0x0000,   /* U+12507 */
  0x0000,   /* U+12508 */
  0x0000,   /* U+12509 */
  0x0000,   /* U+1250A */
  0x0000,   /* U+1250B */
  0x0000,   /* U+1250C */
  0x0000,   /* U+1250D */
  0x0000,   /* U+1250E */
  0x0000,   /* U+1250F */
  0x0000,   /* U+12510 */
  0x0000,   /* U+12511 */
  0x0000,   /* U+12512 */
  0x0000,   /* U+12513 */
  0x0000,   /* U+12514 */
  0x0000,   /* U+12515 */
  0x0000,   /* U+12516 */
  0x0000,   /* U+12517 */
  0x0000,   /* U+12518 */
  0x0000,   /* U+12519 */
  0x0000,   /* U+1251A */
  0x0000,   /* U+1251B */
  0x0000,   /* U+1251C */
  0x0000,   /* U+1251D */
  0x0000,   /* U+1251E */
  0x0000,   /* U+1251F */
  0x0000,   /* U+12520 */
  0x0000,   /* U+12521 */
  0x0000,   /* U+12522 */
  0x0000,   /* U+12523 */
  0x0000,   /* U+12524 */
  0x0000,   /* U+12525 */
  0x0000,   /* U+12526 */
  0x0000,   /* U+12527 */
  0x0000,   /* U+12528 */
  0x0000,   /* U+12529 */
  0x0000,   /* U+1252A */
  0x0000,   /* U+1252B */
  0x0000,   /* U+1252C */
  0x0000,   /* U+1252D */
  0x0000,   /* U+1252E */
  0x0000,   /* U+1252F */
  0x0000,   /* U+12530 */
  0x0000,   /* U+12531 */
  0x0000,   /* U+12532 */
  0x0000,   /* U+12533 */
  0x0000,   /* U+12534 */
  0x0000,   /* U+12535 */
  0x0000,   /* U+12536 */
  0x0000,   /* U+12537 */
  0x0000,   /* U+12538 */
  0x0000,   /* U+12539 */
  0x0000,   /* U+1253A */
  0x0000,   /* U+1253B */
  0x0000,   /* U+1253C */
  0x0000,   /* U+1253D */
  0x0000,   /* U+1253E */
  0x0000,   /* U+1253F */
  0x0000,   /* U+12540 */
  0x0000,   /* U+12541 */
  0x0000,   /* U+12542 */
  0x0000,   /* U+12543 */
  0xA544,   /* U+12544 */
  0xA545,   /* U+12545 */
  0xA546,   /* U+12546 */
  0xA547,   /* U+12547 */
  0xA548,   /* U+12548 */
  0xA549,   /* U+12549 */
  0xA54A,   /* U+1254A */
  0xA54B,   /* U+1254B */
  0xA54C,   /* U+1254C */
  0xA54D,   /* U+1254D */
  0xA54E,   /* U+1254E */
  0xA54F,   /* U+1254F */
  0xA550,   /* U+12550 */
  0xA551,   /* U+12551 */
  0xA552,   /* U+12552 */
  0xA553,   /* U+12553 */
  0xA554,   /* U+12554 */
  0xA555,   /* U+12555 */
  0xA556,   /* U+12556 */
  0xA557,   /* U+12557 */
  0xA558,   /* U+12558 */
  0xA559,   /* U+12559 */
  0xA55A,   /* U+1255A */
  0xA55B,   /* U+1255B */
  0xA55C,   /* U+1255C */
  0xA55D,   /* U+1255D */
  0xA55E,   /* U+1255E */
  0xA55F,   /* U+1255F */
  0xA560,   /* U+12560 */
  0xA561,   /* U+12561 */
  0xA562,   /* U+12562 */
  0xA563,   /* U+12563 */
  0xA564,   /* U+12564 */
  0xA565,   /* U+12565 */
  0xA566,   /* U+12566 */
  0xA567,   /* U+12567 */
  0xA568,   /* U+12568 */
  0xA569,   /* U+12569 */
  0xA56A,   /* U+1256A */
  0xA56B,   /* U+1256B */
  0xA56C,   /* U+1256C */
  0xA56D,   /* U+1256D */
  0xA56E,   /* U+1256E */
  0xA56F,   /* U+1256F */
  0xA570,   /* U+12570 */
  0xA571,   /* U+12571 */
  0xA572,   /* U+12572 */
  0xA573,   /* U+12573 */
  0xA574,   /* U+12574 */
  0xA575,   /* U+12575 */
  0xA576,   /* U+12576 */
  0xA577,   /* U+12577 */
  0xA578,   /* U+12578 */
  0xA579,   /* U+12579 */
  0xA57A,   /* U+1257A */
  0xA57B,   /* U+1257B */
  0xA57C,   /* U+1257C */
  0xA57D,   /* U+1257D */
  0xA57E,   /* U+1257E */
  0xA57F,   /* U+1257F */
  0xA580,   /* U+12580 */
  0xA581,   /* U+12581 */
  0xA582,   /* U+12582 */
  0xA583,   /* U+12583 */
  0xA584,   /* U+12584 */
  0xA585,   /* U+12585 */
  0xA586,   /* U+12586 */
  0xA587,   /* U+12587 */
  0xA588,   /* U+12588 */
  0xA589,   /* U+12589 */
  0xA58A,   /* U+1258A */
  0xA58B,   /* U+1258B */
  0xA58C,   /* U+1258C */
  0xA58D,   /* U+1258D */
  0xA58E,   /* U+1258E */
  0xA58F,   /* U+1258F */
  0xA590,   /* U+12590 */
  0xA591,   /* U+12591 */
  0xA592,   /* U+12592 */
  0xA593,   /* U+12593 */
  0xA594,   /* U+12594 */
  0xA595,   /* U+12595 */
  0xA596,   /* U+12596 */
  0xA597,   /* U+12597 */
  0xA598,   /* U+12598 */
  0xA599,   /* U+12599 */
  0xA59A,   /* U+1259A */
  0xA59B,   /* U+1259B */
  0xA59C,   /* U+1259C */
  0xA59D,   /* U+1259D */
  0xA59E,   /* U+1259E */
  0xA59F,   /* U+1259F */
  0xA5A0,   /* U+125A0 */
  0xA5A1,   /* U+125A1 */
  0xA5A2,   /* U+125A2 */
  0xA5A3,   /* U+125A3 */
  0xA5A4,   /* U+125A4 */
  0xA5A5,   /* U+125A5 */
  0xA5A6,   /* U+125A6 */
  0xA5A7,   /* U+125A7 */
  0xA5A8,   /* U+125A8 */
  0xA5A9,   /* U+125A9 */
  0xA5AA,   /* U+125AA */
  0xA5AB,   /* U+125AB */
  0xA5AC,   /* U+125AC */
  0xA5AD,   /* U+125AD */
  0xA5AE,   /* U+125AE */
  0xA5AF,   /* U+125AF */
  0xA5B0,   /* U+125B0 */
  0xA5B1,   /* U+125B1 */
  0xA5B2,   /* U+125B2 */
  0xA5B3,   /* U+125B3 */
  0xA5B4,   /* U+125B4 */
  0xA5B5,   /* U+125B5 */
  0xA5B6,   /* U+125B6 */
  0xA5B7,   /* U+125B7 */
  0xA5B8,   /* U+125B8 */
  0xA5B9,   /* U+125B9 */
  0xA5BA,   /* U+125BA */
  0xA5BB,   /* U+125BB */
  0xA5BC,   /* U+125BC */
  0xA5BD,   /* U+125BD */
  0xA5BE,   /* U+125BE */
  0xA5BF,   /* U+125BF */
  0xA5C0,   /* U+125C0 */
  0xA5C1,   /* U+125C1 */
  0xA5C2,   /* U+125C2 */
  0xA5C3,   /* U+125C3 */
  0xA5C4,   /* U+125C4 */
  0xA5C5,   /* U+125C5 */
  0xA5C6,   /* U+125C6 */
  0xA5C7,   /* U+125C7 */
  0xA5C8,   /* U+125C8 */
  0xA5C9,   /* U+125C9 */
  0xA5CA,   /* U+125CA */
  0xA5CB,   /* U+125CB */
  0xA5CC,   /* U+125CC */
  0xA5CD,   /* U+125CD */
  0xA5CE,   /* U+125CE */
  0xA5CF,   /* U+125CF */
  0xA5D0,   /* U+125D0 */
  0xA5D1,   /* U+125D1 */
  0xA5D2,   /* U+125D2 */
  0xA5D3,   /* U+125D3 */
  0xA5D4,   /* U+125D4 */
  0xA5D5,   /* U+125D5 */
  0xA5D6,   /* U+125D6 */
  0xA5D7,   /* U+125D7 */
  0xA5D8,   /* U+125D8 */
  0xA5D9,   /* U+125D9 */
  0xA5DA,   /* U+125DA */
  0xA5DB,   /* U+125DB */
  0xA5DC,   /* U+125DC */
  0xA5DD,   /* U+125DD */
  0xA5DE,   /* U+125DE */
  0xA5DF,   /* U+125DF */
  0xA5E0,   /* U+125E0 */
  0xA5E1,   /* U+125E1 */
  0xA5E2,   /* U+125E2 */
  0xA5E3,   /* U+125E3 */
  0xA5E4,   /* U+125E4 */
  0xA5E5,   /* U+125E5 */
  0xA5E6,   /* U+125E6 */
  0xA5E7,   /* U+125E7 */
  0xA5E8,   /* U+125E8 */
  0xA5E9,   /* U+125E9 */
  0xA5EA,   /* U+125EA */
  0xA5EB,   /* U+125EB */
  0xA5EC,   /* U+125EC */
  0xA5ED,   /* U+125ED */
  0xA5EE,   /* U+125EE */
  0xA5EF,   /* U+125EF */
  0xA5F0,   /* U+125F0 */
  0xA5F1,   /* U+125F1 */
  0xA5F2,   /* U+125F2 */
  0xA5F3,   /* U+125F3 */
  0xA5F4,   /* U+125F4 */
  0xA5F5,   /* U+125F5 */
  0xA5F6,   /* U+125F6 */
  0xA5F7,   /* U+125F7 */
  0xA5F8,   /* U+125F8 */
  0xA5F9,   /* U+125F9 */
  0xA5FA,   /* U+125FA */
  0xA5FB,   /* U+125FB */
  0xA5FC,   /* U+125FC */
  0xA5FD,   /* U+125FD */
  0xA5FE,   /* U+125FE */
  0xA5FF,   /* U+125FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+12500 */
  0x0000,   /* U+12501 */
  0x0000,   /* U+12502 */
  0x0000,   /* U+12503 */
  0x0000,   /* U+12504 */
  0x0000,   /* U+12505 */
  0x0000,   /* U+12506 */
  0x0000,   /* U+12507 */
  0x0000,   /* U+12508 */
  0x0000,   /* U+12509 */
  0x0000,   /* U+1250A */
  0x0000,   /* U+1250B */
  0x0000,   /* U+1250C */
  0x0000,   /* U+1250D */
  0x0000,   /* U+1250E */
  0x0000,   /* U+1250F */
  0x0000,   /* U+12510 */
  0x0000,   /* U+12511 */
  0x0000,   /* U+12512 */
  0x0000,   /* U+12513 */
  0x0000,   /* U+12514 */
  0x0000,   /* U+12515 */
  0x0000,   /* U+12516 */
  0x0000,   /* U+12517 */
  0x0000,   /* U+12518 */
  0x0000,   /* U+12519 */
  0x0000,   /* U+1251A */
  0x0000,   /* U+1251B */
  0x0000,   /* U+1251C */
  0x0000,   /* U+1251D */
  0x0000,   /* U+1251E */
  0x0000,   /* U+1251F */
  0x0000,   /* U+12520 */
  0x0000,   /* U+12521 */
  0x0000,   /* U+12522 */
  0x0000,   /* U+12523 */
  0x0000,   /* U+12524 */
  0x0000,   /* U+12525 */
  0x0000,   /* U+12526 */
  0x0000,   /* U+12527 */
  0x0000,   /* U+12528 */
  0x0000,   /* U+12529 */
  0x0000,   /* U+1252A */
  0x0000,   /* U+1252B */
  0x0000,   /* U+1252C */
  0x0000,   /* U+1252D */
  0x0000,   /* U+1252E */
  0x0000,   /* U+1252F */
  0x0000,   /* U+12530 */
  0x0000,   /* U+12531 */
  0x0000,   /* U+12532 */
  0x0000,   /* U+12533 */
  0x0000,   /* U+12534 */
  0x0000,   /* U+12535 */
  0x0000,   /* U+12536 */
  0x0000,   /* U+12537 */
  0x0000,   /* U+12538 */
  0x0000,   /* U+12539 */
  0x0000,   /* U+1253A */
  0x0000,   /* U+1253B */
  0x0000,   /* U+1253C */
  0x0000,   /* U+1253D */
  0x0000,   /* U+1253E */
  0x0000,   /* U+1253F */
  0x0000,   /* U+12540 */
  0x0000,   /* U+12541 */
  0x0000,   /* U+12542 */
  0x0000,   /* U+12543 */
  0x0000,   /* U+12544 */
  0x0000,   /* U+12545 */
  0x0000,   /* U+12546 */
  0x0000,   /* U+12547 */
  0x0000,   /* U+12548 */
  0x0000,   /* U+12549 */
  0x0000,   /* U+1254A */
  0x0000,   /* U+1254B */
  0x0000,   /* U+1254C */
  0x0000,   /* U+1254D */
  0x0000,   /* U+1254E */
  0x0000,   /* U+1254F */
  0x0000,   /* U+12550 */
  0x0000,   /* U+12551 */
  0x0000,   /* U+12552 */
  0x0000,   /* U+12553 */
  0x0000,   /* U+12554 */
  0x0000,   /* U+12555 */
  0x0000,   /* U+12556 */
  0x0000,   /* U+12557 */
  0x0000,   /* U+12558 */
  0x0000,   /* U+12559 */
  0x0000,   /* U+1255A */
  0x0000,   /* U+1255B */
  0x0000,   /* U+1255C */
  0x0000,   /* U+1255D */
  0x0000,   /* U+1255E */
  0x0000,   /* U+1255F */
  0x0000,   /* U+12560 */
  0x0000,   /* U+12561 */
  0x0000,   /* U+12562 */
  0x0000,   /* U+12563 */
  0x0000,   /* U+12564 */
  0x0000,   /* U+12565 */
  0x0000,   /* U+12566 */
  0x0000,   /* U+12567 */
  0x0000,   /* U+12568 */
  0x0000,   /* U+12569 */
  0x0000,   /* U+1256A */
  0x0000,   /* U+1256B */
  0x0000,   /* U+1256C */
  0x0000,   /* U+1256D */
  0x0000,   /* U+1256E */
  0x0000,   /* U+1256F */
  0x0000,   /* U+12570 */
  0x0000,   /* U+12571 */
  0x0000,   /* U+12572 */
  0x0000,   /* U+12573 */
  0x0000,   /* U+12574 */
  0x0000,   /* U+12575 */
  0x0000,   /* U+12576 */
  0x0000,   /* U+12577 */
  0x0000,   /* U+12578 */
  0x0000,   /* U+12579 */
  0x0000,   /* U+1257A */
  0x0000,   /* U+1257B */
  0x0000,   /* U+1257C */
  0x0000,   /* U+1257D */
  0x0000,   /* U+1257E */
  0x0000,   /* U+1257F */
  0x0000,   /* U+12580 */
  0x0000,   /* U+12581 */
  0x0000,   /* U+12582 */
  0x0000,   /* U+12583 */
  0x0000,   /* U+12584 */
  0x0000,   /* U+12585 */
  0x0000,   /* U+12586 */
  0x0000,   /* U+12587 */
  0x0000,   /* U+12588 */
  0x0000,   /* U+12589 */
  0x0000,   /* U+1258A */
  0x0000,   /* U+1258B */
  0x0000,   /* U+1258C */
  0x0000,   /* U+1258D */
  0x0000,   /* U+1258E */
  0x0000,   /* U+1258F */
  0x0000,   /* U+12590 */
  0x0000,   /* U+12591 */
  0x0000,   /* U+12592 */
  0x0000,   /* U+12593 */
  0x0000,   /* U+12594 */
  0x0000,   /* U+12595 */
  0x0000,   /* U+12596 */
  0x0000,   /* U+12597 */
  0x0000,   /* U+12598 */
  0x0000,   /* U+12599 */
  0x0000,   /* U+1259A */
  0x0000,   /* U+1259B */
  0x0000,   /* U+1259C */
  0x0000,   /* U+1259D */
  0x0000,   /* U+1259E */
  0x0000,   /* U+1259F */
  0x0000,   /* U+125A0 */
  0x0000,   /* U+125A1 */
  0x0000,   /* U+125A2 */
  0x0000,   /* U+125A3 */
  0x0000,   /* U+125A4 */
  0x0000,   /* U+125A5 */
  0x0000,   /* U+125A6 */
  0x0000,   /* U+125A7 */
  0x0000,   /* U+125A8 */
  0x0000,   /* U+125A9 */
  0x0000,   /* U+125AA */
  0x0000,   /* U+125AB */
  0x0000,   /* U+125AC */
  0x0000,   /* U+125AD */
  0x0000,   /* U+125AE */
  0x0000,   /* U+125AF */
  0x0000,   /* U+125B0 */
  0x0000,   /* U+125B1 */
  0x0000,   /* U+125B2 */
  0x0000,   /* U+125B3 */
  0x0000,   /* U+125B4 */
  0x0000,   /* U+125B5 */
  0x0000,   /* U+125B6 */
  0x0000,   /* U+125B7 */
  0x0000,   /* U+125B8 */
  0x0000,   /* U+125B9 */
  0x0000,   /* U+125BA */
  0x0000,   /* U+125BB */
  0x0000,   /* U+125BC */
  0x0000,   /* U+125BD */
  0x0000,   /* U+125BE */
  0x0000,   /* U+125BF */
  0x0000,   /* U+125C0 */
  0x0000,   /* U+125C1 */
  0x0000,   /* U+125C2 */
  0x0000,   /* U+125C3 */
  0x0000,   /* U+125C4 */
  0x0000,   /* U+125C5 */
  0x0000,   /* U+125C6 */
  0x0000,   /* U+125C7 */
  0x0000,   /* U+125C8 */
  0x0000,   /* U+125C9 */
  0x0000,   /* U+125CA */
  0x0000,   /* U+125CB */
  0x0000,   /* U+125CC */
  0x0000,   /* U+125CD */
  0x0000,   /* U+125CE */
  0x0000,   /* U+125CF */
  0x0000,   /* U+125D0 */
  0x0000,   /* U+125D1 */
  0x0000,   /* U+125D2 */
  0x0000,   /* U+125D3 */
  0x0000,   /* U+125D4 */
  0x0000,   /* U+125D5 */
  0x0000,   /* U+125D6 */
  0x0000,   /* U+125D7 */
  0x0000,   /* U+125D8 */
  0x0000,   /* U+125D9 */
  0x0000,   /* U+125DA */
  0x0000,   /* U+125DB */
  0x0000,   /* U+125DC */
  0x0000,   /* U+125DD */
  0x0000,   /* U+125DE */
  0x0000,   /* U+125DF */
  0x0000,   /* U+125E0 */
  0x0000,   /* U+125E1 */
  0x0000,   /* U+125E2 */
  0x0000,   /* U+125E3 */
  0x0000,   /* U+125E4 */
  0x0000,   /* U+125E5 */
  0x0000,   /* U+125E6 */
  0x0000,   /* U+125E7 */
  0x0000,   /* U+125E8 */
  0x0000,   /* U+125E9 */
  0x0000,   /* U+125EA */
  0x0000,   /* U+125EB */
  0x0000,   /* U+125EC */
  0x0000,   /* U+125ED */
  0x0000,   /* U+125EE */
  0x0000,   /* U+125EF */
  0x0000,   /* U+125F0 */
  0x0000,   /* U+125F1 */
  0x0000,   /* U+125F2 */
  0x0000,   /* U+125F3 */
  0x0000,   /* U+125F4 */
  0x0000,   /* U+125F5 */
  0x0000,   /* U+125F6 */
  0x0000,   /* U+125F7 */
  0x0000,   /* U+125F8 */
  0x0000,   /* U+125F9 */
  0x0000,   /* U+125FA */
  0x0000,   /* U+125FB */
  0x0000,   /* U+125FC */
  0x0000,   /* U+125FD */
  0x0000,   /* U+125FE */
  0x0000,   /* U+125FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+12500 */
  0x0000,   /* U+12501 */
  0x0000,   /* U+12502 */
  0x0000,   /* U+12503 */
  0x0000,   /* U+12504 */
  0x0000,   /* U+12505 */
  0x0000,   /* U+12506 */
  0x0000,   /* U+12507 */
  0x0000,   /* U+12508 */
  0x0000,   /* U+12509 */
  0x0000,   /* U+1250A */
  0x0000,   /* U+1250B */
  0x0000,   /* U+1250C */
  0x0000,   /* U+1250D */
  0x0000,   /* U+1250E */
  0x0000,   /* U+1250F */
  0x0000,   /* U+12510 */
  0x0000,   /* U+12511 */
  0x0000,   /* U+12512 */
  0x0000,   /* U+12513 */
  0x0000,   /* U+12514 */
  0x0000,   /* U+12515 */
  0x0000,   /* U+12516 */
  0x0000,   /* U+12517 */
  0x0000,   /* U+12518 */
  0x0000,   /* U+12519 */
  0x0000,   /* U+1251A */
  0x0000,   /* U+1251B */
  0x0000,   /* U+1251C */
  0x0000,   /* U+1251D */
  0x0000,   /* U+1251E */
  0x0000,   /* U+1251F */
  0x0000,   /* U+12520 */
  0x0000,   /* U+12521 */
  0x0000,   /* U+12522 */
  0x0000,   /* U+12523 */
  0x0000,   /* U+12524 */
  0x0000,   /* U+12525 */
  0x0000,   /* U+12526 */
  0x0000,   /* U+12527 */
  0x0000,   /* U+12528 */
  0x0000,   /* U+12529 */
  0x0000,   /* U+1252A */
  0x0000,   /* U+1252B */
  0x0000,   /* U+1252C */
  0x0000,   /* U+1252D */
  0x0000,   /* U+1252E */
  0x0000,   /* U+1252F */
  0x0000,   /* U+12530 */
  0x0000,   /* U+12531 */
  0x0000,   /* U+12532 */
  0x0000,   /* U+12533 */
  0x0000,   /* U+12534 */
  0x0000,   /* U+12535 */
  0x0000,   /* U+12536 */
  0x0000,   /* U+12537 */
  0x0000,   /* U+12538 */
  0x0000,   /* U+12539 */
  0x0000,   /* U+1253A */
  0x0000,   /* U+1253B */
  0x0000,   /* U+1253C */
  0x0000,   /* U+1253D */
  0x0000,   /* U+1253E */
  0x0000,   /* U+1253F */
  0x0000,   /* U+12540 */
  0x0000,   /* U+12541 */
  0x0000,   /* U+12542 */
  0x0000,   /* U+12543 */
  0x0000,   /* U+12544 */
  0x0000,   /* U+12545 */
  0x0000,   /* U+12546 */
  0x0000,   /* U+12547 */
  0x0000,   /* U+12548 */
  0x0000,   /* U+12549 */
  0x0000,   /* U+1254A */
  0x0000,   /* U+1254B */
  0x0000,   /* U+1254C */
  0x0000,   /* U+1254D */
  0x0000,   /* U+1254E */
  0x0000,   /* U+1254F */
  0x0000,   /* U+12550 */
  0x0000,   /* U+12551 */
  0x0000,   /* U+12552 */
  0x0000,   /* U+12553 */
  0x0000,   /* U+12554 */
  0x0000,   /* U+12555 */
  0x0000,   /* U+12556 */
  0x0000,   /* U+12557 */
  0x0000,   /* U+12558 */
  0x0000,   /* U+12559 */
  0x0000,   /* U+1255A */
  0x0000,   /* U+1255B */
  0x0000,   /* U+1255C */
  0x0000,   /* U+1255D */
  0x0000,   /* U+1255E */
  0x0000,   /* U+1255F */
  0x0000,   /* U+12560 */
  0x0000,   /* U+12561 */
  0x0000,   /* U+12562 */
  0x0000,   /* U+12563 */
  0x0000,   /* U+12564 */
  0x0000,   /* U+12565 */
  0x0000,   /* U+12566 */
  0x0000,   /* U+12567 */
  0x0000,   /* U+12568 */
  0x0000,   /* U+12569 */
  0x0000,   /* U+1256A */
  0x0000,   /* U+1256B */
  0x0000,   /* U+1256C */
  0x0000,   /* U+1256D */
  0x0000,   /* U+1256E */
  0x0000,   /* U+1256F */
  0x0000,   /* U+12570 */
  0x0000,   /* U+12571 */
  0x0000,   /* U+12572 */
  0x0000,   /* U+12573 */
  0x0000,   /* U+12574 */
  0x0000,   /* U+12575 */
  0x0000,   /* U+12576 */
  0x0000,   /* U+12577 */
  0x0000,   /* U+12578 */
  0x0000,   /* U+12579 */
  0x0000,   /* U+1257A */
  0x0000,   /* U+1257B */
  0x0000,   /* U+1257C */
  0x0000,   /* U+1257D */
  0x0000,   /* U+1257E */
  0x0000,   /* U+1257F */
  0x0000,   /* U+12580 */
  0x0000,   /* U+12581 */
  0x0000,   /* U+12582 */
  0x0000,   /* U+12583 */
  0x0000,   /* U+12584 */
  0x0000,   /* U+12585 */
  0x0000,   /* U+12586 */
  0x0000,   /* U+12587 */
  0x0000,   /* U+12588 */
  0x0000,   /* U+12589 */
  0x0000,   /* U+1258A */
  0x0000,   /* U+1258B */
  0x0000,   /* U+1258C */
  0x0000,   /* U+1258D */
  0x0000,   /* U+1258E */
  0x0000,   /* U+1258F */
  0x0000,   /* U+12590 */
  0x0000,   /* U+12591 */
  0x0000,   /* U+12592 */
  0x0000,   /* U+12593 */
  0x0000,   /* U+12594 */
  0x0000,   /* U+12595 */
  0x0000,   /* U+12596 */
  0x0000,   /* U+12597 */
  0x0000,   /* U+12598 */
  0x0000,   /* U+12599 */
  0x0000,   /* U+1259A */
  0x0000,   /* U+1259B */
  0x0000,   /* U+1259C */
  0x0000,   /* U+1259D */
  0x0000,   /* U+1259E */
  0x0000,   /* U+1259F */
  0x0000,   /* U+125A0 */
  0x0000,   /* U+125A1 */
  0x0000,   /* U+125A2 */
  0x0000,   /* U+125A3 */
  0x0000,   /* U+125A4 */
  0x0000,   /* U+125A5 */
  0x0000,   /* U+125A6 */
  0x0000,   /* U+125A7 */
  0x0000,   /* U+125A8 */
  0x0000,   /* U+125A9 */
  0x0000,   /* U+125AA */
  0x0000,   /* U+125AB */
  0x0000,   /* U+125AC */
  0x0000,   /* U+125AD */
  0x0000,   /* U+125AE */
  0x0000,   /* U+125AF */
  0x0000,   /* U+125B0 */
  0x0000,   /* U+125B1 */
  0x0000,   /* U+125B2 */
  0x0000,   /* U+125B3 */
  0x0000,   /* U+125B4 */
  0x0000,   /* U+125B5 */
  0x0000,   /* U+125B6 */
  0x0000,   /* U+125B7 */
  0x0000,   /* U+125B8 */
  0x0000,   /* U+125B9 */
  0x0000,   /* U+125BA */
  0x0000,   /* U+125BB */
  0x0000,   /* U+125BC */
  0x0000,   /* U+125BD */
  0x0000,   /* U+125BE */
  0x0000,   /* U+125BF */
  0x0000,   /* U+125C0 */
  0x0000,   /* U+125C1 */
  0x0000,   /* U+125C2 */
  0x0000,   /* U+125C3 */
  0x0000,   /* U+125C4 */
  0x0000,   /* U+125C5 */
  0x0000,   /* U+125C6 */
  0x0000,   /* U+125C7 */
  0x0000,   /* U+125C8 */
  0x0000,   /* U+125C9 */
  0x0000,   /* U+125CA */
  0x0000,   /* U+125CB */
  0x0000,   /* U+125CC */
  0x0000,   /* U+125CD */
  0x0000,   /* U+125CE */
  0x0000,   /* U+125CF */
  0x0000,   /* U+125D0 */
  0x0000,   /* U+125D1 */
  0x0000,   /* U+125D2 */
  0x0000,   /* U+125D3 */
  0x0000,   /* U+125D4 */
  0x0000,   /* U+125D5 */
  0x0000,   /* U+125D6 */
  0x0000,   /* U+125D7 */
  0x0000,   /* U+125D8 */
  0x0000,   /* U+125D9 */
  0x0000,   /* U+125DA */
  0x0000,   /* U+125DB */
  0x0000,   /* U+125DC */
  0x0000,   /* U+125DD */
  0x0000,   /* U+125DE */
  0x0000,   /* U+125DF */
  0x0000,   /* U+125E0 */
  0x0000,   /* U+125E1 */
  0x0000,   /* U+125E2 */
  0x0000,   /* U+125E3 */
  0x0000,   /* U+125E4 */
  0x0000,   /* U+125E5 */
  0x0000,   /* U+125E6 */
  0x0000,   /* U+125E7 */
  0x0000,   /* U+125E8 */
  0x0000,   /* U+125E9 */
  0x0000,   /* U+125EA */
  0x0000,   /* U+125EB */
  0x0000,   /* U+125EC */
  0x0000,   /* U+125ED */
  0x0000,   /* U+125EE */
  0x0000,   /* U+125EF */
  0x0000,   /* U+125F0 */
  0x0000,   /* U+125F1 */
  0x0000,   /* U+125F2 */
  0x0000,   /* U+125F3 */
  0x0000,   /* U+125F4 */
  0x0000,   /* U+125F5 */
  0x0000,   /* U+125F6 */
  0x0000,   /* U+125F7 */
  0x0000,   /* U+125F8 */
  0x0000,   /* U+125F9 */
  0x0000,   /* U+125FA */
  0x0000,   /* U+125FB */
  0x0000,   /* U+125FC */
  0x0000,   /* U+125FD */
  0x0000,   /* U+125FE */
  0x0000,   /* U+125FF */
};

uint16 uca900_p130[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x4E13,   /* U+13000 */
  0x4E14,   /* U+13001 */
  0x4E15,   /* U+13002 */
  0x4E16,   /* U+13003 */
  0x4E17,   /* U+13004 */
  0x4E18,   /* U+13005 */
  0x4E19,   /* U+13006 */
  0x4E1A,   /* U+13007 */
  0x4E1B,   /* U+13008 */
  0x4E1C,   /* U+13009 */
  0x4E1D,   /* U+1300A */
  0x4E1E,   /* U+1300B */
  0x4E1F,   /* U+1300C */
  0x4E20,   /* U+1300D */
  0x4E21,   /* U+1300E */
  0x4E22,   /* U+1300F */
  0x4E23,   /* U+13010 */
  0x4E24,   /* U+13011 */
  0x4E25,   /* U+13012 */
  0x4E26,   /* U+13013 */
  0x4E27,   /* U+13014 */
  0x4E28,   /* U+13015 */
  0x4E29,   /* U+13016 */
  0x4E2A,   /* U+13017 */
  0x4E2B,   /* U+13018 */
  0x4E2C,   /* U+13019 */
  0x4E2D,   /* U+1301A */
  0x4E2E,   /* U+1301B */
  0x4E2F,   /* U+1301C */
  0x4E30,   /* U+1301D */
  0x4E31,   /* U+1301E */
  0x4E32,   /* U+1301F */
  0x4E33,   /* U+13020 */
  0x4E34,   /* U+13021 */
  0x4E35,   /* U+13022 */
  0x4E36,   /* U+13023 */
  0x4E37,   /* U+13024 */
  0x4E38,   /* U+13025 */
  0x4E39,   /* U+13026 */
  0x4E3A,   /* U+13027 */
  0x4E3B,   /* U+13028 */
  0x4E3C,   /* U+13029 */
  0x4E3D,   /* U+1302A */
  0x4E3E,   /* U+1302B */
  0x4E3F,   /* U+1302C */
  0x4E40,   /* U+1302D */
  0x4E41,   /* U+1302E */
  0x4E42,   /* U+1302F */
  0x4E43,   /* U+13030 */
  0x4E44,   /* U+13031 */
  0x4E45,   /* U+13032 */
  0x4E46,   /* U+13033 */
  0x4E47,   /* U+13034 */
  0x4E48,   /* U+13035 */
  0x4E49,   /* U+13036 */
  0x4E4A,   /* U+13037 */
  0x4E4B,   /* U+13038 */
  0x4E4C,   /* U+13039 */
  0x4E4D,   /* U+1303A */
  0x4E4E,   /* U+1303B */
  0x4E4F,   /* U+1303C */
  0x4E50,   /* U+1303D */
  0x4E51,   /* U+1303E */
  0x4E52,   /* U+1303F */
  0x4E53,   /* U+13040 */
  0x4E54,   /* U+13041 */
  0x4E55,   /* U+13042 */
  0x4E56,   /* U+13043 */
  0x4E57,   /* U+13044 */
  0x4E58,   /* U+13045 */
  0x4E59,   /* U+13046 */
  0x4E5A,   /* U+13047 */
  0x4E5B,   /* U+13048 */
  0x4E5C,   /* U+13049 */
  0x4E5D,   /* U+1304A */
  0x4E5E,   /* U+1304B */
  0x4E5F,   /* U+1304C */
  0x4E60,   /* U+1304D */
  0x4E61,   /* U+1304E */
  0x4E62,   /* U+1304F */
  0x4E63,   /* U+13050 */
  0x4E64,   /* U+13051 */
  0x4E65,   /* U+13052 */
  0x4E66,   /* U+13053 */
  0x4E67,   /* U+13054 */
  0x4E68,   /* U+13055 */
  0x4E69,   /* U+13056 */
  0x4E6A,   /* U+13057 */
  0x4E6B,   /* U+13058 */
  0x4E6C,   /* U+13059 */
  0x4E6D,   /* U+1305A */
  0x4E6E,   /* U+1305B */
  0x4E6F,   /* U+1305C */
  0x4E70,   /* U+1305D */
  0x4E71,   /* U+1305E */
  0x4E72,   /* U+1305F */
  0x4E73,   /* U+13060 */
  0x4E74,   /* U+13061 */
  0x4E75,   /* U+13062 */
  0x4E76,   /* U+13063 */
  0x4E77,   /* U+13064 */
  0x4E78,   /* U+13065 */
  0x4E79,   /* U+13066 */
  0x4E7A,   /* U+13067 */
  0x4E7B,   /* U+13068 */
  0x4E7C,   /* U+13069 */
  0x4E7D,   /* U+1306A */
  0x4E7E,   /* U+1306B */
  0x4E7F,   /* U+1306C */
  0x4E80,   /* U+1306D */
  0x4E81,   /* U+1306E */
  0x4E82,   /* U+1306F */
  0x4E83,   /* U+13070 */
  0x4E84,   /* U+13071 */
  0x4E85,   /* U+13072 */
  0x4E86,   /* U+13073 */
  0x4E87,   /* U+13074 */
  0x4E88,   /* U+13075 */
  0x4E89,   /* U+13076 */
  0x4E8A,   /* U+13077 */
  0x4E8B,   /* U+13078 */
  0x4E8C,   /* U+13079 */
  0x4E8D,   /* U+1307A */
  0x4E8E,   /* U+1307B */
  0x4E8F,   /* U+1307C */
  0x4E90,   /* U+1307D */
  0x4E91,   /* U+1307E */
  0x4E92,   /* U+1307F */
  0x4E93,   /* U+13080 */
  0x4E94,   /* U+13081 */
  0x4E95,   /* U+13082 */
  0x4E96,   /* U+13083 */
  0x4E97,   /* U+13084 */
  0x4E98,   /* U+13085 */
  0x4E99,   /* U+13086 */
  0x4E9A,   /* U+13087 */
  0x4E9B,   /* U+13088 */
  0x4E9C,   /* U+13089 */
  0x4E9D,   /* U+1308A */
  0x4E9E,   /* U+1308B */
  0x4E9F,   /* U+1308C */
  0x4EA0,   /* U+1308D */
  0x4EA1,   /* U+1308E */
  0x4EA2,   /* U+1308F */
  0x4EA3,   /* U+13090 */
  0x4EA4,   /* U+13091 */
  0x4EA5,   /* U+13092 */
  0x4EA6,   /* U+13093 */
  0x4EA7,   /* U+13094 */
  0x4EA8,   /* U+13095 */
  0x4EA9,   /* U+13096 */
  0x4EAA,   /* U+13097 */
  0x4EAB,   /* U+13098 */
  0x4EAC,   /* U+13099 */
  0x4EAD,   /* U+1309A */
  0x4EAE,   /* U+1309B */
  0x4EAF,   /* U+1309C */
  0x4EB0,   /* U+1309D */
  0x4EB1,   /* U+1309E */
  0x4EB2,   /* U+1309F */
  0x4EB3,   /* U+130A0 */
  0x4EB4,   /* U+130A1 */
  0x4EB5,   /* U+130A2 */
  0x4EB6,   /* U+130A3 */
  0x4EB7,   /* U+130A4 */
  0x4EB8,   /* U+130A5 */
  0x4EB9,   /* U+130A6 */
  0x4EBA,   /* U+130A7 */
  0x4EBB,   /* U+130A8 */
  0x4EBC,   /* U+130A9 */
  0x4EBD,   /* U+130AA */
  0x4EBE,   /* U+130AB */
  0x4EBF,   /* U+130AC */
  0x4EC0,   /* U+130AD */
  0x4EC1,   /* U+130AE */
  0x4EC2,   /* U+130AF */
  0x4EC3,   /* U+130B0 */
  0x4EC4,   /* U+130B1 */
  0x4EC5,   /* U+130B2 */
  0x4EC6,   /* U+130B3 */
  0x4EC7,   /* U+130B4 */
  0x4EC8,   /* U+130B5 */
  0x4EC9,   /* U+130B6 */
  0x4ECA,   /* U+130B7 */
  0x4ECB,   /* U+130B8 */
  0x4ECC,   /* U+130B9 */
  0x4ECD,   /* U+130BA */
  0x4ECE,   /* U+130BB */
  0x4ECF,   /* U+130BC */
  0x4ED0,   /* U+130BD */
  0x4ED1,   /* U+130BE */
  0x4ED2,   /* U+130BF */
  0x4ED3,   /* U+130C0 */
  0x4ED4,   /* U+130C1 */
  0x4ED5,   /* U+130C2 */
  0x4ED6,   /* U+130C3 */
  0x4ED7,   /* U+130C4 */
  0x4ED8,   /* U+130C5 */
  0x4ED9,   /* U+130C6 */
  0x4EDA,   /* U+130C7 */
  0x4EDB,   /* U+130C8 */
  0x4EDC,   /* U+130C9 */
  0x4EDD,   /* U+130CA */
  0x4EDE,   /* U+130CB */
  0x4EDF,   /* U+130CC */
  0x4EE0,   /* U+130CD */
  0x4EE1,   /* U+130CE */
  0x4EE2,   /* U+130CF */
  0x4EE3,   /* U+130D0 */
  0x4EE4,   /* U+130D1 */
  0x4EE5,   /* U+130D2 */
  0x4EE6,   /* U+130D3 */
  0x4EE7,   /* U+130D4 */
  0x4EE8,   /* U+130D5 */
  0x4EE9,   /* U+130D6 */
  0x4EEA,   /* U+130D7 */
  0x4EEB,   /* U+130D8 */
  0x4EEC,   /* U+130D9 */
  0x4EED,   /* U+130DA */
  0x4EEE,   /* U+130DB */
  0x4EEF,   /* U+130DC */
  0x4EF0,   /* U+130DD */
  0x4EF1,   /* U+130DE */
  0x4EF2,   /* U+130DF */
  0x4EF3,   /* U+130E0 */
  0x4EF4,   /* U+130E1 */
  0x4EF5,   /* U+130E2 */
  0x4EF6,   /* U+130E3 */
  0x4EF7,   /* U+130E4 */
  0x4EF8,   /* U+130E5 */
  0x4EF9,   /* U+130E6 */
  0x4EFA,   /* U+130E7 */
  0x4EFB,   /* U+130E8 */
  0x4EFC,   /* U+130E9 */
  0x4EFD,   /* U+130EA */
  0x4EFE,   /* U+130EB */
  0x4EFF,   /* U+130EC */
  0x4F00,   /* U+130ED */
  0x4F01,   /* U+130EE */
  0x4F02,   /* U+130EF */
  0x4F03,   /* U+130F0 */
  0x4F04,   /* U+130F1 */
  0x4F05,   /* U+130F2 */
  0x4F06,   /* U+130F3 */
  0x4F07,   /* U+130F4 */
  0x4F08,   /* U+130F5 */
  0x4F09,   /* U+130F6 */
  0x4F0A,   /* U+130F7 */
  0x4F0B,   /* U+130F8 */
  0x4F0C,   /* U+130F9 */
  0x4F0D,   /* U+130FA */
  0x4F0E,   /* U+130FB */
  0x4F0F,   /* U+130FC */
  0x4F10,   /* U+130FD */
  0x4F11,   /* U+130FE */
  0x4F12,   /* U+130FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+13000 */
  0x0020,   /* U+13001 */
  0x0020,   /* U+13002 */
  0x0020,   /* U+13003 */
  0x0020,   /* U+13004 */
  0x0020,   /* U+13005 */
  0x0020,   /* U+13006 */
  0x0020,   /* U+13007 */
  0x0020,   /* U+13008 */
  0x0020,   /* U+13009 */
  0x0020,   /* U+1300A */
  0x0020,   /* U+1300B */
  0x0020,   /* U+1300C */
  0x0020,   /* U+1300D */
  0x0020,   /* U+1300E */
  0x0020,   /* U+1300F */
  0x0020,   /* U+13010 */
  0x0020,   /* U+13011 */
  0x0020,   /* U+13012 */
  0x0020,   /* U+13013 */
  0x0020,   /* U+13014 */
  0x0020,   /* U+13015 */
  0x0020,   /* U+13016 */
  0x0020,   /* U+13017 */
  0x0020,   /* U+13018 */
  0x0020,   /* U+13019 */
  0x0020,   /* U+1301A */
  0x0020,   /* U+1301B */
  0x0020,   /* U+1301C */
  0x0020,   /* U+1301D */
  0x0020,   /* U+1301E */
  0x0020,   /* U+1301F */
  0x0020,   /* U+13020 */
  0x0020,   /* U+13021 */
  0x0020,   /* U+13022 */
  0x0020,   /* U+13023 */
  0x0020,   /* U+13024 */
  0x0020,   /* U+13025 */
  0x0020,   /* U+13026 */
  0x0020,   /* U+13027 */
  0x0020,   /* U+13028 */
  0x0020,   /* U+13029 */
  0x0020,   /* U+1302A */
  0x0020,   /* U+1302B */
  0x0020,   /* U+1302C */
  0x0020,   /* U+1302D */
  0x0020,   /* U+1302E */
  0x0020,   /* U+1302F */
  0x0020,   /* U+13030 */
  0x0020,   /* U+13031 */
  0x0020,   /* U+13032 */
  0x0020,   /* U+13033 */
  0x0020,   /* U+13034 */
  0x0020,   /* U+13035 */
  0x0020,   /* U+13036 */
  0x0020,   /* U+13037 */
  0x0020,   /* U+13038 */
  0x0020,   /* U+13039 */
  0x0020,   /* U+1303A */
  0x0020,   /* U+1303B */
  0x0020,   /* U+1303C */
  0x0020,   /* U+1303D */
  0x0020,   /* U+1303E */
  0x0020,   /* U+1303F */
  0x0020,   /* U+13040 */
  0x0020,   /* U+13041 */
  0x0020,   /* U+13042 */
  0x0020,   /* U+13043 */
  0x0020,   /* U+13044 */
  0x0020,   /* U+13045 */
  0x0020,   /* U+13046 */
  0x0020,   /* U+13047 */
  0x0020,   /* U+13048 */
  0x0020,   /* U+13049 */
  0x0020,   /* U+1304A */
  0x0020,   /* U+1304B */
  0x0020,   /* U+1304C */
  0x0020,   /* U+1304D */
  0x0020,   /* U+1304E */
  0x0020,   /* U+1304F */
  0x0020,   /* U+13050 */
  0x0020,   /* U+13051 */
  0x0020,   /* U+13052 */
  0x0020,   /* U+13053 */
  0x0020,   /* U+13054 */
  0x0020,   /* U+13055 */
  0x0020,   /* U+13056 */
  0x0020,   /* U+13057 */
  0x0020,   /* U+13058 */
  0x0020,   /* U+13059 */
  0x0020,   /* U+1305A */
  0x0020,   /* U+1305B */
  0x0020,   /* U+1305C */
  0x0020,   /* U+1305D */
  0x0020,   /* U+1305E */
  0x0020,   /* U+1305F */
  0x0020,   /* U+13060 */
  0x0020,   /* U+13061 */
  0x0020,   /* U+13062 */
  0x0020,   /* U+13063 */
  0x0020,   /* U+13064 */
  0x0020,   /* U+13065 */
  0x0020,   /* U+13066 */
  0x0020,   /* U+13067 */
  0x0020,   /* U+13068 */
  0x0020,   /* U+13069 */
  0x0020,   /* U+1306A */
  0x0020,   /* U+1306B */
  0x0020,   /* U+1306C */
  0x0020,   /* U+1306D */
  0x0020,   /* U+1306E */
  0x0020,   /* U+1306F */
  0x0020,   /* U+13070 */
  0x0020,   /* U+13071 */
  0x0020,   /* U+13072 */
  0x0020,   /* U+13073 */
  0x0020,   /* U+13074 */
  0x0020,   /* U+13075 */
  0x0020,   /* U+13076 */
  0x0020,   /* U+13077 */
  0x0020,   /* U+13078 */
  0x0020,   /* U+13079 */
  0x0020,   /* U+1307A */
  0x0020,   /* U+1307B */
  0x0020,   /* U+1307C */
  0x0020,   /* U+1307D */
  0x0020,   /* U+1307E */
  0x0020,   /* U+1307F */
  0x0020,   /* U+13080 */
  0x0020,   /* U+13081 */
  0x0020,   /* U+13082 */
  0x0020,   /* U+13083 */
  0x0020,   /* U+13084 */
  0x0020,   /* U+13085 */
  0x0020,   /* U+13086 */
  0x0020,   /* U+13087 */
  0x0020,   /* U+13088 */
  0x0020,   /* U+13089 */
  0x0020,   /* U+1308A */
  0x0020,   /* U+1308B */
  0x0020,   /* U+1308C */
  0x0020,   /* U+1308D */
  0x0020,   /* U+1308E */
  0x0020,   /* U+1308F */
  0x0020,   /* U+13090 */
  0x0020,   /* U+13091 */
  0x0020,   /* U+13092 */
  0x0020,   /* U+13093 */
  0x0020,   /* U+13094 */
  0x0020,   /* U+13095 */
  0x0020,   /* U+13096 */
  0x0020,   /* U+13097 */
  0x0020,   /* U+13098 */
  0x0020,   /* U+13099 */
  0x0020,   /* U+1309A */
  0x0020,   /* U+1309B */
  0x0020,   /* U+1309C */
  0x0020,   /* U+1309D */
  0x0020,   /* U+1309E */
  0x0020,   /* U+1309F */
  0x0020,   /* U+130A0 */
  0x0020,   /* U+130A1 */
  0x0020,   /* U+130A2 */
  0x0020,   /* U+130A3 */
  0x0020,   /* U+130A4 */
  0x0020,   /* U+130A5 */
  0x0020,   /* U+130A6 */
  0x0020,   /* U+130A7 */
  0x0020,   /* U+130A8 */
  0x0020,   /* U+130A9 */
  0x0020,   /* U+130AA */
  0x0020,   /* U+130AB */
  0x0020,   /* U+130AC */
  0x0020,   /* U+130AD */
  0x0020,   /* U+130AE */
  0x0020,   /* U+130AF */
  0x0020,   /* U+130B0 */
  0x0020,   /* U+130B1 */
  0x0020,   /* U+130B2 */
  0x0020,   /* U+130B3 */
  0x0020,   /* U+130B4 */
  0x0020,   /* U+130B5 */
  0x0020,   /* U+130B6 */
  0x0020,   /* U+130B7 */
  0x0020,   /* U+130B8 */
  0x0020,   /* U+130B9 */
  0x0020,   /* U+130BA */
  0x0020,   /* U+130BB */
  0x0020,   /* U+130BC */
  0x0020,   /* U+130BD */
  0x0020,   /* U+130BE */
  0x0020,   /* U+130BF */
  0x0020,   /* U+130C0 */
  0x0020,   /* U+130C1 */
  0x0020,   /* U+130C2 */
  0x0020,   /* U+130C3 */
  0x0020,   /* U+130C4 */
  0x0020,   /* U+130C5 */
  0x0020,   /* U+130C6 */
  0x0020,   /* U+130C7 */
  0x0020,   /* U+130C8 */
  0x0020,   /* U+130C9 */
  0x0020,   /* U+130CA */
  0x0020,   /* U+130CB */
  0x0020,   /* U+130CC */
  0x0020,   /* U+130CD */
  0x0020,   /* U+130CE */
  0x0020,   /* U+130CF */
  0x0020,   /* U+130D0 */
  0x0020,   /* U+130D1 */
  0x0020,   /* U+130D2 */
  0x0020,   /* U+130D3 */
  0x0020,   /* U+130D4 */
  0x0020,   /* U+130D5 */
  0x0020,   /* U+130D6 */
  0x0020,   /* U+130D7 */
  0x0020,   /* U+130D8 */
  0x0020,   /* U+130D9 */
  0x0020,   /* U+130DA */
  0x0020,   /* U+130DB */
  0x0020,   /* U+130DC */
  0x0020,   /* U+130DD */
  0x0020,   /* U+130DE */
  0x0020,   /* U+130DF */
  0x0020,   /* U+130E0 */
  0x0020,   /* U+130E1 */
  0x0020,   /* U+130E2 */
  0x0020,   /* U+130E3 */
  0x0020,   /* U+130E4 */
  0x0020,   /* U+130E5 */
  0x0020,   /* U+130E6 */
  0x0020,   /* U+130E7 */
  0x0020,   /* U+130E8 */
  0x0020,   /* U+130E9 */
  0x0020,   /* U+130EA */
  0x0020,   /* U+130EB */
  0x0020,   /* U+130EC */
  0x0020,   /* U+130ED */
  0x0020,   /* U+130EE */
  0x0020,   /* U+130EF */
  0x0020,   /* U+130F0 */
  0x0020,   /* U+130F1 */
  0x0020,   /* U+130F2 */
  0x0020,   /* U+130F3 */
  0x0020,   /* U+130F4 */
  0x0020,   /* U+130F5 */
  0x0020,   /* U+130F6 */
  0x0020,   /* U+130F7 */
  0x0020,   /* U+130F8 */
  0x0020,   /* U+130F9 */
  0x0020,   /* U+130FA */
  0x0020,   /* U+130FB */
  0x0020,   /* U+130FC */
  0x0020,   /* U+130FD */
  0x0020,   /* U+130FE */
  0x0020,   /* U+130FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+13000 */
  0x0002,   /* U+13001 */
  0x0002,   /* U+13002 */
  0x0002,   /* U+13003 */
  0x0002,   /* U+13004 */
  0x0002,   /* U+13005 */
  0x0002,   /* U+13006 */
  0x0002,   /* U+13007 */
  0x0002,   /* U+13008 */
  0x0002,   /* U+13009 */
  0x0002,   /* U+1300A */
  0x0002,   /* U+1300B */
  0x0002,   /* U+1300C */
  0x0002,   /* U+1300D */
  0x0002,   /* U+1300E */
  0x0002,   /* U+1300F */
  0x0002,   /* U+13010 */
  0x0002,   /* U+13011 */
  0x0002,   /* U+13012 */
  0x0002,   /* U+13013 */
  0x0002,   /* U+13014 */
  0x0002,   /* U+13015 */
  0x0002,   /* U+13016 */
  0x0002,   /* U+13017 */
  0x0002,   /* U+13018 */
  0x0002,   /* U+13019 */
  0x0002,   /* U+1301A */
  0x0002,   /* U+1301B */
  0x0002,   /* U+1301C */
  0x0002,   /* U+1301D */
  0x0002,   /* U+1301E */
  0x0002,   /* U+1301F */
  0x0002,   /* U+13020 */
  0x0002,   /* U+13021 */
  0x0002,   /* U+13022 */
  0x0002,   /* U+13023 */
  0x0002,   /* U+13024 */
  0x0002,   /* U+13025 */
  0x0002,   /* U+13026 */
  0x0002,   /* U+13027 */
  0x0002,   /* U+13028 */
  0x0002,   /* U+13029 */
  0x0002,   /* U+1302A */
  0x0002,   /* U+1302B */
  0x0002,   /* U+1302C */
  0x0002,   /* U+1302D */
  0x0002,   /* U+1302E */
  0x0002,   /* U+1302F */
  0x0002,   /* U+13030 */
  0x0002,   /* U+13031 */
  0x0002,   /* U+13032 */
  0x0002,   /* U+13033 */
  0x0002,   /* U+13034 */
  0x0002,   /* U+13035 */
  0x0002,   /* U+13036 */
  0x0002,   /* U+13037 */
  0x0002,   /* U+13038 */
  0x0002,   /* U+13039 */
  0x0002,   /* U+1303A */
  0x0002,   /* U+1303B */
  0x0002,   /* U+1303C */
  0x0002,   /* U+1303D */
  0x0002,   /* U+1303E */
  0x0002,   /* U+1303F */
  0x0002,   /* U+13040 */
  0x0002,   /* U+13041 */
  0x0002,   /* U+13042 */
  0x0002,   /* U+13043 */
  0x0002,   /* U+13044 */
  0x0002,   /* U+13045 */
  0x0002,   /* U+13046 */
  0x0002,   /* U+13047 */
  0x0002,   /* U+13048 */
  0x0002,   /* U+13049 */
  0x0002,   /* U+1304A */
  0x0002,   /* U+1304B */
  0x0002,   /* U+1304C */
  0x0002,   /* U+1304D */
  0x0002,   /* U+1304E */
  0x0002,   /* U+1304F */
  0x0002,   /* U+13050 */
  0x0002,   /* U+13051 */
  0x0002,   /* U+13052 */
  0x0002,   /* U+13053 */
  0x0002,   /* U+13054 */
  0x0002,   /* U+13055 */
  0x0002,   /* U+13056 */
  0x0002,   /* U+13057 */
  0x0002,   /* U+13058 */
  0x0002,   /* U+13059 */
  0x0002,   /* U+1305A */
  0x0002,   /* U+1305B */
  0x0002,   /* U+1305C */
  0x0002,   /* U+1305D */
  0x0002,   /* U+1305E */
  0x0002,   /* U+1305F */
  0x0002,   /* U+13060 */
  0x0002,   /* U+13061 */
  0x0002,   /* U+13062 */
  0x0002,   /* U+13063 */
  0x0002,   /* U+13064 */
  0x0002,   /* U+13065 */
  0x0002,   /* U+13066 */
  0x0002,   /* U+13067 */
  0x0002,   /* U+13068 */
  0x0002,   /* U+13069 */
  0x0002,   /* U+1306A */
  0x0002,   /* U+1306B */
  0x0002,   /* U+1306C */
  0x0002,   /* U+1306D */
  0x0002,   /* U+1306E */
  0x0002,   /* U+1306F */
  0x0002,   /* U+13070 */
  0x0002,   /* U+13071 */
  0x0002,   /* U+13072 */
  0x0002,   /* U+13073 */
  0x0002,   /* U+13074 */
  0x0002,   /* U+13075 */
  0x0002,   /* U+13076 */
  0x0002,   /* U+13077 */
  0x0002,   /* U+13078 */
  0x0002,   /* U+13079 */
  0x0002,   /* U+1307A */
  0x0002,   /* U+1307B */
  0x0002,   /* U+1307C */
  0x0002,   /* U+1307D */
  0x0002,   /* U+1307E */
  0x0002,   /* U+1307F */
  0x0002,   /* U+13080 */
  0x0002,   /* U+13081 */
  0x0002,   /* U+13082 */
  0x0002,   /* U+13083 */
  0x0002,   /* U+13084 */
  0x0002,   /* U+13085 */
  0x0002,   /* U+13086 */
  0x0002,   /* U+13087 */
  0x0002,   /* U+13088 */
  0x0002,   /* U+13089 */
  0x0002,   /* U+1308A */
  0x0002,   /* U+1308B */
  0x0002,   /* U+1308C */
  0x0002,   /* U+1308D */
  0x0002,   /* U+1308E */
  0x0002,   /* U+1308F */
  0x0002,   /* U+13090 */
  0x0002,   /* U+13091 */
  0x0002,   /* U+13092 */
  0x0002,   /* U+13093 */
  0x0002,   /* U+13094 */
  0x0002,   /* U+13095 */
  0x0002,   /* U+13096 */
  0x0002,   /* U+13097 */
  0x0002,   /* U+13098 */
  0x0002,   /* U+13099 */
  0x0002,   /* U+1309A */
  0x0002,   /* U+1309B */
  0x0002,   /* U+1309C */
  0x0002,   /* U+1309D */
  0x0002,   /* U+1309E */
  0x0002,   /* U+1309F */
  0x0002,   /* U+130A0 */
  0x0002,   /* U+130A1 */
  0x0002,   /* U+130A2 */
  0x0002,   /* U+130A3 */
  0x0002,   /* U+130A4 */
  0x0002,   /* U+130A5 */
  0x0002,   /* U+130A6 */
  0x0002,   /* U+130A7 */
  0x0002,   /* U+130A8 */
  0x0002,   /* U+130A9 */
  0x0002,   /* U+130AA */
  0x0002,   /* U+130AB */
  0x0002,   /* U+130AC */
  0x0002,   /* U+130AD */
  0x0002,   /* U+130AE */
  0x0002,   /* U+130AF */
  0x0002,   /* U+130B0 */
  0x0002,   /* U+130B1 */
  0x0002,   /* U+130B2 */
  0x0002,   /* U+130B3 */
  0x0002,   /* U+130B4 */
  0x0002,   /* U+130B5 */
  0x0002,   /* U+130B6 */
  0x0002,   /* U+130B7 */
  0x0002,   /* U+130B8 */
  0x0002,   /* U+130B9 */
  0x0002,   /* U+130BA */
  0x0002,   /* U+130BB */
  0x0002,   /* U+130BC */
  0x0002,   /* U+130BD */
  0x0002,   /* U+130BE */
  0x0002,   /* U+130BF */
  0x0002,   /* U+130C0 */
  0x0002,   /* U+130C1 */
  0x0002,   /* U+130C2 */
  0x0002,   /* U+130C3 */
  0x0002,   /* U+130C4 */
  0x0002,   /* U+130C5 */
  0x0002,   /* U+130C6 */
  0x0002,   /* U+130C7 */
  0x0002,   /* U+130C8 */
  0x0002,   /* U+130C9 */
  0x0002,   /* U+130CA */
  0x0002,   /* U+130CB */
  0x0002,   /* U+130CC */
  0x0002,   /* U+130CD */
  0x0002,   /* U+130CE */
  0x0002,   /* U+130CF */
  0x0002,   /* U+130D0 */
  0x0002,   /* U+130D1 */
  0x0002,   /* U+130D2 */
  0x0002,   /* U+130D3 */
  0x0002,   /* U+130D4 */
  0x0002,   /* U+130D5 */
  0x0002,   /* U+130D6 */
  0x0002,   /* U+130D7 */
  0x0002,   /* U+130D8 */
  0x0002,   /* U+130D9 */
  0x0002,   /* U+130DA */
  0x0002,   /* U+130DB */
  0x0002,   /* U+130DC */
  0x0002,   /* U+130DD */
  0x0002,   /* U+130DE */
  0x0002,   /* U+130DF */
  0x0002,   /* U+130E0 */
  0x0002,   /* U+130E1 */
  0x0002,   /* U+130E2 */
  0x0002,   /* U+130E3 */
  0x0002,   /* U+130E4 */
  0x0002,   /* U+130E5 */
  0x0002,   /* U+130E6 */
  0x0002,   /* U+130E7 */
  0x0002,   /* U+130E8 */
  0x0002,   /* U+130E9 */
  0x0002,   /* U+130EA */
  0x0002,   /* U+130EB */
  0x0002,   /* U+130EC */
  0x0002,   /* U+130ED */
  0x0002,   /* U+130EE */
  0x0002,   /* U+130EF */
  0x0002,   /* U+130F0 */
  0x0002,   /* U+130F1 */
  0x0002,   /* U+130F2 */
  0x0002,   /* U+130F3 */
  0x0002,   /* U+130F4 */
  0x0002,   /* U+130F5 */
  0x0002,   /* U+130F6 */
  0x0002,   /* U+130F7 */
  0x0002,   /* U+130F8 */
  0x0002,   /* U+130F9 */
  0x0002,   /* U+130FA */
  0x0002,   /* U+130FB */
  0x0002,   /* U+130FC */
  0x0002,   /* U+130FD */
  0x0002,   /* U+130FE */
  0x0002,   /* U+130FF */
};

uint16 uca900_p131[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x4F13,   /* U+13100 */
  0x4F14,   /* U+13101 */
  0x4F15,   /* U+13102 */
  0x4F16,   /* U+13103 */
  0x4F17,   /* U+13104 */
  0x4F18,   /* U+13105 */
  0x4F19,   /* U+13106 */
  0x4F1A,   /* U+13107 */
  0x4F1B,   /* U+13108 */
  0x4F1C,   /* U+13109 */
  0x4F1D,   /* U+1310A */
  0x4F1E,   /* U+1310B */
  0x4F1F,   /* U+1310C */
  0x4F20,   /* U+1310D */
  0x4F21,   /* U+1310E */
  0x4F22,   /* U+1310F */
  0x4F23,   /* U+13110 */
  0x4F24,   /* U+13111 */
  0x4F25,   /* U+13112 */
  0x4F26,   /* U+13113 */
  0x4F27,   /* U+13114 */
  0x4F28,   /* U+13115 */
  0x4F29,   /* U+13116 */
  0x4F2A,   /* U+13117 */
  0x4F2B,   /* U+13118 */
  0x4F2C,   /* U+13119 */
  0x4F2D,   /* U+1311A */
  0x4F2E,   /* U+1311B */
  0x4F2F,   /* U+1311C */
  0x4F30,   /* U+1311D */
  0x4F31,   /* U+1311E */
  0x4F32,   /* U+1311F */
  0x4F33,   /* U+13120 */
  0x4F34,   /* U+13121 */
  0x4F35,   /* U+13122 */
  0x4F36,   /* U+13123 */
  0x4F37,   /* U+13124 */
  0x4F38,   /* U+13125 */
  0x4F39,   /* U+13126 */
  0x4F3A,   /* U+13127 */
  0x4F3B,   /* U+13128 */
  0x4F3C,   /* U+13129 */
  0x4F3D,   /* U+1312A */
  0x4F3E,   /* U+1312B */
  0x4F3F,   /* U+1312C */
  0x4F40,   /* U+1312D */
  0x4F41,   /* U+1312E */
  0x4F42,   /* U+1312F */
  0x4F43,   /* U+13130 */
  0x4F44,   /* U+13131 */
  0x4F45,   /* U+13132 */
  0x4F46,   /* U+13133 */
  0x4F47,   /* U+13134 */
  0x4F48,   /* U+13135 */
  0x4F49,   /* U+13136 */
  0x4F4A,   /* U+13137 */
  0x4F4B,   /* U+13138 */
  0x4F4C,   /* U+13139 */
  0x4F4D,   /* U+1313A */
  0x4F4E,   /* U+1313B */
  0x4F4F,   /* U+1313C */
  0x4F50,   /* U+1313D */
  0x4F51,   /* U+1313E */
  0x4F52,   /* U+1313F */
  0x4F53,   /* U+13140 */
  0x4F54,   /* U+13141 */
  0x4F55,   /* U+13142 */
  0x4F56,   /* U+13143 */
  0x4F57,   /* U+13144 */
  0x4F58,   /* U+13145 */
  0x4F59,   /* U+13146 */
  0x4F5A,   /* U+13147 */
  0x4F5B,   /* U+13148 */
  0x4F5C,   /* U+13149 */
  0x4F5D,   /* U+1314A */
  0x4F5E,   /* U+1314B */
  0x4F5F,   /* U+1314C */
  0x4F60,   /* U+1314D */
  0x4F61,   /* U+1314E */
  0x4F62,   /* U+1314F */
  0x4F63,   /* U+13150 */
  0x4F64,   /* U+13151 */
  0x4F65,   /* U+13152 */
  0x4F66,   /* U+13153 */
  0x4F67,   /* U+13154 */
  0x4F68,   /* U+13155 */
  0x4F69,   /* U+13156 */
  0x4F6A,   /* U+13157 */
  0x4F6B,   /* U+13158 */
  0x4F6C,   /* U+13159 */
  0x4F6D,   /* U+1315A */
  0x4F6E,   /* U+1315B */
  0x4F6F,   /* U+1315C */
  0x4F70,   /* U+1315D */
  0x4F71,   /* U+1315E */
  0x4F72,   /* U+1315F */
  0x4F73,   /* U+13160 */
  0x4F74,   /* U+13161 */
  0x4F75,   /* U+13162 */
  0x4F76,   /* U+13163 */
  0x4F77,   /* U+13164 */
  0x4F78,   /* U+13165 */
  0x4F79,   /* U+13166 */
  0x4F7A,   /* U+13167 */
  0x4F7B,   /* U+13168 */
  0x4F7C,   /* U+13169 */
  0x4F7D,   /* U+1316A */
  0x4F7E,   /* U+1316B */
  0x4F7F,   /* U+1316C */
  0x4F80,   /* U+1316D */
  0x4F81,   /* U+1316E */
  0x4F82,   /* U+1316F */
  0x4F83,   /* U+13170 */
  0x4F84,   /* U+13171 */
  0x4F85,   /* U+13172 */
  0x4F86,   /* U+13173 */
  0x4F87,   /* U+13174 */
  0x4F88,   /* U+13175 */
  0x4F89,   /* U+13176 */
  0x4F8A,   /* U+13177 */
  0x4F8B,   /* U+13178 */
  0x4F8C,   /* U+13179 */
  0x4F8D,   /* U+1317A */
  0x4F8E,   /* U+1317B */
  0x4F8F,   /* U+1317C */
  0x4F90,   /* U+1317D */
  0x4F91,   /* U+1317E */
  0x4F92,   /* U+1317F */
  0x4F93,   /* U+13180 */
  0x4F94,   /* U+13181 */
  0x4F95,   /* U+13182 */
  0x4F96,   /* U+13183 */
  0x4F97,   /* U+13184 */
  0x4F98,   /* U+13185 */
  0x4F99,   /* U+13186 */
  0x4F9A,   /* U+13187 */
  0x4F9B,   /* U+13188 */
  0x4F9C,   /* U+13189 */
  0x4F9D,   /* U+1318A */
  0x4F9E,   /* U+1318B */
  0x4F9F,   /* U+1318C */
  0x4FA0,   /* U+1318D */
  0x4FA1,   /* U+1318E */
  0x4FA2,   /* U+1318F */
  0x4FA3,   /* U+13190 */
  0x4FA4,   /* U+13191 */
  0x4FA5,   /* U+13192 */
  0x4FA6,   /* U+13193 */
  0x4FA7,   /* U+13194 */
  0x4FA8,   /* U+13195 */
  0x4FA9,   /* U+13196 */
  0x4FAA,   /* U+13197 */
  0x4FAB,   /* U+13198 */
  0x4FAC,   /* U+13199 */
  0x4FAD,   /* U+1319A */
  0x4FAE,   /* U+1319B */
  0x4FAF,   /* U+1319C */
  0x4FB0,   /* U+1319D */
  0x4FB1,   /* U+1319E */
  0x4FB2,   /* U+1319F */
  0x4FB3,   /* U+131A0 */
  0x4FB4,   /* U+131A1 */
  0x4FB5,   /* U+131A2 */
  0x4FB6,   /* U+131A3 */
  0x4FB7,   /* U+131A4 */
  0x4FB8,   /* U+131A5 */
  0x4FB9,   /* U+131A6 */
  0x4FBA,   /* U+131A7 */
  0x4FBB,   /* U+131A8 */
  0x4FBC,   /* U+131A9 */
  0x4FBD,   /* U+131AA */
  0x4FBE,   /* U+131AB */
  0x4FBF,   /* U+131AC */
  0x4FC0,   /* U+131AD */
  0x4FC1,   /* U+131AE */
  0x4FC2,   /* U+131AF */
  0x4FC3,   /* U+131B0 */
  0x4FC4,   /* U+131B1 */
  0x4FC5,   /* U+131B2 */
  0x4FC6,   /* U+131B3 */
  0x4FC7,   /* U+131B4 */
  0x4FC8,   /* U+131B5 */
  0x4FC9,   /* U+131B6 */
  0x4FCA,   /* U+131B7 */
  0x4FCB,   /* U+131B8 */
  0x4FCC,   /* U+131B9 */
  0x4FCD,   /* U+131BA */
  0x4FCE,   /* U+131BB */
  0x4FCF,   /* U+131BC */
  0x4FD0,   /* U+131BD */
  0x4FD1,   /* U+131BE */
  0x4FD2,   /* U+131BF */
  0x4FD3,   /* U+131C0 */
  0x4FD4,   /* U+131C1 */
  0x4FD5,   /* U+131C2 */
  0x4FD6,   /* U+131C3 */
  0x4FD7,   /* U+131C4 */
  0x4FD8,   /* U+131C5 */
  0x4FD9,   /* U+131C6 */
  0x4FDA,   /* U+131C7 */
  0x4FDB,   /* U+131C8 */
  0x4FDC,   /* U+131C9 */
  0x4FDD,   /* U+131CA */
  0x4FDE,   /* U+131CB */
  0x4FDF,   /* U+131CC */
  0x4FE0,   /* U+131CD */
  0x4FE1,   /* U+131CE */
  0x4FE2,   /* U+131CF */
  0x4FE3,   /* U+131D0 */
  0x4FE4,   /* U+131D1 */
  0x4FE5,   /* U+131D2 */
  0x4FE6,   /* U+131D3 */
  0x4FE7,   /* U+131D4 */
  0x4FE8,   /* U+131D5 */
  0x4FE9,   /* U+131D6 */
  0x4FEA,   /* U+131D7 */
  0x4FEB,   /* U+131D8 */
  0x4FEC,   /* U+131D9 */
  0x4FED,   /* U+131DA */
  0x4FEE,   /* U+131DB */
  0x4FEF,   /* U+131DC */
  0x4FF0,   /* U+131DD */
  0x4FF1,   /* U+131DE */
  0x4FF2,   /* U+131DF */
  0x4FF3,   /* U+131E0 */
  0x4FF4,   /* U+131E1 */
  0x4FF5,   /* U+131E2 */
  0x4FF6,   /* U+131E3 */
  0x4FF7,   /* U+131E4 */
  0x4FF8,   /* U+131E5 */
  0x4FF9,   /* U+131E6 */
  0x4FFA,   /* U+131E7 */
  0x4FFB,   /* U+131E8 */
  0x4FFC,   /* U+131E9 */
  0x4FFD,   /* U+131EA */
  0x4FFE,   /* U+131EB */
  0x4FFF,   /* U+131EC */
  0x5000,   /* U+131ED */
  0x5001,   /* U+131EE */
  0x5002,   /* U+131EF */
  0x5003,   /* U+131F0 */
  0x5004,   /* U+131F1 */
  0x5005,   /* U+131F2 */
  0x5006,   /* U+131F3 */
  0x5007,   /* U+131F4 */
  0x5008,   /* U+131F5 */
  0x5009,   /* U+131F6 */
  0x500A,   /* U+131F7 */
  0x500B,   /* U+131F8 */
  0x500C,   /* U+131F9 */
  0x500D,   /* U+131FA */
  0x500E,   /* U+131FB */
  0x500F,   /* U+131FC */
  0x5010,   /* U+131FD */
  0x5011,   /* U+131FE */
  0x5012,   /* U+131FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+13100 */
  0x0020,   /* U+13101 */
  0x0020,   /* U+13102 */
  0x0020,   /* U+13103 */
  0x0020,   /* U+13104 */
  0x0020,   /* U+13105 */
  0x0020,   /* U+13106 */
  0x0020,   /* U+13107 */
  0x0020,   /* U+13108 */
  0x0020,   /* U+13109 */
  0x0020,   /* U+1310A */
  0x0020,   /* U+1310B */
  0x0020,   /* U+1310C */
  0x0020,   /* U+1310D */
  0x0020,   /* U+1310E */
  0x0020,   /* U+1310F */
  0x0020,   /* U+13110 */
  0x0020,   /* U+13111 */
  0x0020,   /* U+13112 */
  0x0020,   /* U+13113 */
  0x0020,   /* U+13114 */
  0x0020,   /* U+13115 */
  0x0020,   /* U+13116 */
  0x0020,   /* U+13117 */
  0x0020,   /* U+13118 */
  0x0020,   /* U+13119 */
  0x0020,   /* U+1311A */
  0x0020,   /* U+1311B */
  0x0020,   /* U+1311C */
  0x0020,   /* U+1311D */
  0x0020,   /* U+1311E */
  0x0020,   /* U+1311F */
  0x0020,   /* U+13120 */
  0x0020,   /* U+13121 */
  0x0020,   /* U+13122 */
  0x0020,   /* U+13123 */
  0x0020,   /* U+13124 */
  0x0020,   /* U+13125 */
  0x0020,   /* U+13126 */
  0x0020,   /* U+13127 */
  0x0020,   /* U+13128 */
  0x0020,   /* U+13129 */
  0x0020,   /* U+1312A */
  0x0020,   /* U+1312B */
  0x0020,   /* U+1312C */
  0x0020,   /* U+1312D */
  0x0020,   /* U+1312E */
  0x0020,   /* U+1312F */
  0x0020,   /* U+13130 */
  0x0020,   /* U+13131 */
  0x0020,   /* U+13132 */
  0x0020,   /* U+13133 */
  0x0020,   /* U+13134 */
  0x0020,   /* U+13135 */
  0x0020,   /* U+13136 */
  0x0020,   /* U+13137 */
  0x0020,   /* U+13138 */
  0x0020,   /* U+13139 */
  0x0020,   /* U+1313A */
  0x0020,   /* U+1313B */
  0x0020,   /* U+1313C */
  0x0020,   /* U+1313D */
  0x0020,   /* U+1313E */
  0x0020,   /* U+1313F */
  0x0020,   /* U+13140 */
  0x0020,   /* U+13141 */
  0x0020,   /* U+13142 */
  0x0020,   /* U+13143 */
  0x0020,   /* U+13144 */
  0x0020,   /* U+13145 */
  0x0020,   /* U+13146 */
  0x0020,   /* U+13147 */
  0x0020,   /* U+13148 */
  0x0020,   /* U+13149 */
  0x0020,   /* U+1314A */
  0x0020,   /* U+1314B */
  0x0020,   /* U+1314C */
  0x0020,   /* U+1314D */
  0x0020,   /* U+1314E */
  0x0020,   /* U+1314F */
  0x0020,   /* U+13150 */
  0x0020,   /* U+13151 */
  0x0020,   /* U+13152 */
  0x0020,   /* U+13153 */
  0x0020,   /* U+13154 */
  0x0020,   /* U+13155 */
  0x0020,   /* U+13156 */
  0x0020,   /* U+13157 */
  0x0020,   /* U+13158 */
  0x0020,   /* U+13159 */
  0x0020,   /* U+1315A */
  0x0020,   /* U+1315B */
  0x0020,   /* U+1315C */
  0x0020,   /* U+1315D */
  0x0020,   /* U+1315E */
  0x0020,   /* U+1315F */
  0x0020,   /* U+13160 */
  0x0020,   /* U+13161 */
  0x0020,   /* U+13162 */
  0x0020,   /* U+13163 */
  0x0020,   /* U+13164 */
  0x0020,   /* U+13165 */
  0x0020,   /* U+13166 */
  0x0020,   /* U+13167 */
  0x0020,   /* U+13168 */
  0x0020,   /* U+13169 */
  0x0020,   /* U+1316A */
  0x0020,   /* U+1316B */
  0x0020,   /* U+1316C */
  0x0020,   /* U+1316D */
  0x0020,   /* U+1316E */
  0x0020,   /* U+1316F */
  0x0020,   /* U+13170 */
  0x0020,   /* U+13171 */
  0x0020,   /* U+13172 */
  0x0020,   /* U+13173 */
  0x0020,   /* U+13174 */
  0x0020,   /* U+13175 */
  0x0020,   /* U+13176 */
  0x0020,   /* U+13177 */
  0x0020,   /* U+13178 */
  0x0020,   /* U+13179 */
  0x0020,   /* U+1317A */
  0x0020,   /* U+1317B */
  0x0020,   /* U+1317C */
  0x0020,   /* U+1317D */
  0x0020,   /* U+1317E */
  0x0020,   /* U+1317F */
  0x0020,   /* U+13180 */
  0x0020,   /* U+13181 */
  0x0020,   /* U+13182 */
  0x0020,   /* U+13183 */
  0x0020,   /* U+13184 */
  0x0020,   /* U+13185 */
  0x0020,   /* U+13186 */
  0x0020,   /* U+13187 */
  0x0020,   /* U+13188 */
  0x0020,   /* U+13189 */
  0x0020,   /* U+1318A */
  0x0020,   /* U+1318B */
  0x0020,   /* U+1318C */
  0x0020,   /* U+1318D */
  0x0020,   /* U+1318E */
  0x0020,   /* U+1318F */
  0x0020,   /* U+13190 */
  0x0020,   /* U+13191 */
  0x0020,   /* U+13192 */
  0x0020,   /* U+13193 */
  0x0020,   /* U+13194 */
  0x0020,   /* U+13195 */
  0x0020,   /* U+13196 */
  0x0020,   /* U+13197 */
  0x0020,   /* U+13198 */
  0x0020,   /* U+13199 */
  0x0020,   /* U+1319A */
  0x0020,   /* U+1319B */
  0x0020,   /* U+1319C */
  0x0020,   /* U+1319D */
  0x0020,   /* U+1319E */
  0x0020,   /* U+1319F */
  0x0020,   /* U+131A0 */
  0x0020,   /* U+131A1 */
  0x0020,   /* U+131A2 */
  0x0020,   /* U+131A3 */
  0x0020,   /* U+131A4 */
  0x0020,   /* U+131A5 */
  0x0020,   /* U+131A6 */
  0x0020,   /* U+131A7 */
  0x0020,   /* U+131A8 */
  0x0020,   /* U+131A9 */
  0x0020,   /* U+131AA */
  0x0020,   /* U+131AB */
  0x0020,   /* U+131AC */
  0x0020,   /* U+131AD */
  0x0020,   /* U+131AE */
  0x0020,   /* U+131AF */
  0x0020,   /* U+131B0 */
  0x0020,   /* U+131B1 */
  0x0020,   /* U+131B2 */
  0x0020,   /* U+131B3 */
  0x0020,   /* U+131B4 */
  0x0020,   /* U+131B5 */
  0x0020,   /* U+131B6 */
  0x0020,   /* U+131B7 */
  0x0020,   /* U+131B8 */
  0x0020,   /* U+131B9 */
  0x0020,   /* U+131BA */
  0x0020,   /* U+131BB */
  0x0020,   /* U+131BC */
  0x0020,   /* U+131BD */
  0x0020,   /* U+131BE */
  0x0020,   /* U+131BF */
  0x0020,   /* U+131C0 */
  0x0020,   /* U+131C1 */
  0x0020,   /* U+131C2 */
  0x0020,   /* U+131C3 */
  0x0020,   /* U+131C4 */
  0x0020,   /* U+131C5 */
  0x0020,   /* U+131C6 */
  0x0020,   /* U+131C7 */
  0x0020,   /* U+131C8 */
  0x0020,   /* U+131C9 */
  0x0020,   /* U+131CA */
  0x0020,   /* U+131CB */
  0x0020,   /* U+131CC */
  0x0020,   /* U+131CD */
  0x0020,   /* U+131CE */
  0x0020,   /* U+131CF */
  0x0020,   /* U+131D0 */
  0x0020,   /* U+131D1 */
  0x0020,   /* U+131D2 */
  0x0020,   /* U+131D3 */
  0x0020,   /* U+131D4 */
  0x0020,   /* U+131D5 */
  0x0020,   /* U+131D6 */
  0x0020,   /* U+131D7 */
  0x0020,   /* U+131D8 */
  0x0020,   /* U+131D9 */
  0x0020,   /* U+131DA */
  0x0020,   /* U+131DB */
  0x0020,   /* U+131DC */
  0x0020,   /* U+131DD */
  0x0020,   /* U+131DE */
  0x0020,   /* U+131DF */
  0x0020,   /* U+131E0 */
  0x0020,   /* U+131E1 */
  0x0020,   /* U+131E2 */
  0x0020,   /* U+131E3 */
  0x0020,   /* U+131E4 */
  0x0020,   /* U+131E5 */
  0x0020,   /* U+131E6 */
  0x0020,   /* U+131E7 */
  0x0020,   /* U+131E8 */
  0x0020,   /* U+131E9 */
  0x0020,   /* U+131EA */
  0x0020,   /* U+131EB */
  0x0020,   /* U+131EC */
  0x0020,   /* U+131ED */
  0x0020,   /* U+131EE */
  0x0020,   /* U+131EF */
  0x0020,   /* U+131F0 */
  0x0020,   /* U+131F1 */
  0x0020,   /* U+131F2 */
  0x0020,   /* U+131F3 */
  0x0020,   /* U+131F4 */
  0x0020,   /* U+131F5 */
  0x0020,   /* U+131F6 */
  0x0020,   /* U+131F7 */
  0x0020,   /* U+131F8 */
  0x0020,   /* U+131F9 */
  0x0020,   /* U+131FA */
  0x0020,   /* U+131FB */
  0x0020,   /* U+131FC */
  0x0020,   /* U+131FD */
  0x0020,   /* U+131FE */
  0x0020,   /* U+131FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+13100 */
  0x0002,   /* U+13101 */
  0x0002,   /* U+13102 */
  0x0002,   /* U+13103 */
  0x0002,   /* U+13104 */
  0x0002,   /* U+13105 */
  0x0002,   /* U+13106 */
  0x0002,   /* U+13107 */
  0x0002,   /* U+13108 */
  0x0002,   /* U+13109 */
  0x0002,   /* U+1310A */
  0x0002,   /* U+1310B */
  0x0002,   /* U+1310C */
  0x0002,   /* U+1310D */
  0x0002,   /* U+1310E */
  0x0002,   /* U+1310F */
  0x0002,   /* U+13110 */
  0x0002,   /* U+13111 */
  0x0002,   /* U+13112 */
  0x0002,   /* U+13113 */
  0x0002,   /* U+13114 */
  0x0002,   /* U+13115 */
  0x0002,   /* U+13116 */
  0x0002,   /* U+13117 */
  0x0002,   /* U+13118 */
  0x0002,   /* U+13119 */
  0x0002,   /* U+1311A */
  0x0002,   /* U+1311B */
  0x0002,   /* U+1311C */
  0x0002,   /* U+1311D */
  0x0002,   /* U+1311E */
  0x0002,   /* U+1311F */
  0x0002,   /* U+13120 */
  0x0002,   /* U+13121 */
  0x0002,   /* U+13122 */
  0x0002,   /* U+13123 */
  0x0002,   /* U+13124 */
  0x0002,   /* U+13125 */
  0x0002,   /* U+13126 */
  0x0002,   /* U+13127 */
  0x0002,   /* U+13128 */
  0x0002,   /* U+13129 */
  0x0002,   /* U+1312A */
  0x0002,   /* U+1312B */
  0x0002,   /* U+1312C */
  0x0002,   /* U+1312D */
  0x0002,   /* U+1312E */
  0x0002,   /* U+1312F */
  0x0002,   /* U+13130 */
  0x0002,   /* U+13131 */
  0x0002,   /* U+13132 */
  0x0002,   /* U+13133 */
  0x0002,   /* U+13134 */
  0x0002,   /* U+13135 */
  0x0002,   /* U+13136 */
  0x0002,   /* U+13137 */
  0x0002,   /* U+13138 */
  0x0002,   /* U+13139 */
  0x0002,   /* U+1313A */
  0x0002,   /* U+1313B */
  0x0002,   /* U+1313C */
  0x0002,   /* U+1313D */
  0x0002,   /* U+1313E */
  0x0002,   /* U+1313F */
  0x0002,   /* U+13140 */
  0x0002,   /* U+13141 */
  0x0002,   /* U+13142 */
  0x0002,   /* U+13143 */
  0x0002,   /* U+13144 */
  0x0002,   /* U+13145 */
  0x0002,   /* U+13146 */
  0x0002,   /* U+13147 */
  0x0002,   /* U+13148 */
  0x0002,   /* U+13149 */
  0x0002,   /* U+1314A */
  0x0002,   /* U+1314B */
  0x0002,   /* U+1314C */
  0x0002,   /* U+1314D */
  0x0002,   /* U+1314E */
  0x0002,   /* U+1314F */
  0x0002,   /* U+13150 */
  0x0002,   /* U+13151 */
  0x0002,   /* U+13152 */
  0x0002,   /* U+13153 */
  0x0002,   /* U+13154 */
  0x0002,   /* U+13155 */
  0x0002,   /* U+13156 */
  0x0002,   /* U+13157 */
  0x0002,   /* U+13158 */
  0x0002,   /* U+13159 */
  0x0002,   /* U+1315A */
  0x0002,   /* U+1315B */
  0x0002,   /* U+1315C */
  0x0002,   /* U+1315D */
  0x0002,   /* U+1315E */
  0x0002,   /* U+1315F */
  0x0002,   /* U+13160 */
  0x0002,   /* U+13161 */
  0x0002,   /* U+13162 */
  0x0002,   /* U+13163 */
  0x0002,   /* U+13164 */
  0x0002,   /* U+13165 */
  0x0002,   /* U+13166 */
  0x0002,   /* U+13167 */
  0x0002,   /* U+13168 */
  0x0002,   /* U+13169 */
  0x0002,   /* U+1316A */
  0x0002,   /* U+1316B */
  0x0002,   /* U+1316C */
  0x0002,   /* U+1316D */
  0x0002,   /* U+1316E */
  0x0002,   /* U+1316F */
  0x0002,   /* U+13170 */
  0x0002,   /* U+13171 */
  0x0002,   /* U+13172 */
  0x0002,   /* U+13173 */
  0x0002,   /* U+13174 */
  0x0002,   /* U+13175 */
  0x0002,   /* U+13176 */
  0x0002,   /* U+13177 */
  0x0002,   /* U+13178 */
  0x0002,   /* U+13179 */
  0x0002,   /* U+1317A */
  0x0002,   /* U+1317B */
  0x0002,   /* U+1317C */
  0x0002,   /* U+1317D */
  0x0002,   /* U+1317E */
  0x0002,   /* U+1317F */
  0x0002,   /* U+13180 */
  0x0002,   /* U+13181 */
  0x0002,   /* U+13182 */
  0x0002,   /* U+13183 */
  0x0002,   /* U+13184 */
  0x0002,   /* U+13185 */
  0x0002,   /* U+13186 */
  0x0002,   /* U+13187 */
  0x0002,   /* U+13188 */
  0x0002,   /* U+13189 */
  0x0002,   /* U+1318A */
  0x0002,   /* U+1318B */
  0x0002,   /* U+1318C */
  0x0002,   /* U+1318D */
  0x0002,   /* U+1318E */
  0x0002,   /* U+1318F */
  0x0002,   /* U+13190 */
  0x0002,   /* U+13191 */
  0x0002,   /* U+13192 */
  0x0002,   /* U+13193 */
  0x0002,   /* U+13194 */
  0x0002,   /* U+13195 */
  0x0002,   /* U+13196 */
  0x0002,   /* U+13197 */
  0x0002,   /* U+13198 */
  0x0002,   /* U+13199 */
  0x0002,   /* U+1319A */
  0x0002,   /* U+1319B */
  0x0002,   /* U+1319C */
  0x0002,   /* U+1319D */
  0x0002,   /* U+1319E */
  0x0002,   /* U+1319F */
  0x0002,   /* U+131A0 */
  0x0002,   /* U+131A1 */
  0x0002,   /* U+131A2 */
  0x0002,   /* U+131A3 */
  0x0002,   /* U+131A4 */
  0x0002,   /* U+131A5 */
  0x0002,   /* U+131A6 */
  0x0002,   /* U+131A7 */
  0x0002,   /* U+131A8 */
  0x0002,   /* U+131A9 */
  0x0002,   /* U+131AA */
  0x0002,   /* U+131AB */
  0x0002,   /* U+131AC */
  0x0002,   /* U+131AD */
  0x0002,   /* U+131AE */
  0x0002,   /* U+131AF */
  0x0002,   /* U+131B0 */
  0x0002,   /* U+131B1 */
  0x0002,   /* U+131B2 */
  0x0002,   /* U+131B3 */
  0x0002,   /* U+131B4 */
  0x0002,   /* U+131B5 */
  0x0002,   /* U+131B6 */
  0x0002,   /* U+131B7 */
  0x0002,   /* U+131B8 */
  0x0002,   /* U+131B9 */
  0x0002,   /* U+131BA */
  0x0002,   /* U+131BB */
  0x0002,   /* U+131BC */
  0x0002,   /* U+131BD */
  0x0002,   /* U+131BE */
  0x0002,   /* U+131BF */
  0x0002,   /* U+131C0 */
  0x0002,   /* U+131C1 */
  0x0002,   /* U+131C2 */
  0x0002,   /* U+131C3 */
  0x0002,   /* U+131C4 */
  0x0002,   /* U+131C5 */
  0x0002,   /* U+131C6 */
  0x0002,   /* U+131C7 */
  0x0002,   /* U+131C8 */
  0x0002,   /* U+131C9 */
  0x0002,   /* U+131CA */
  0x0002,   /* U+131CB */
  0x0002,   /* U+131CC */
  0x0002,   /* U+131CD */
  0x0002,   /* U+131CE */
  0x0002,   /* U+131CF */
  0x0002,   /* U+131D0 */
  0x0002,   /* U+131D1 */
  0x0002,   /* U+131D2 */
  0x0002,   /* U+131D3 */
  0x0002,   /* U+131D4 */
  0x0002,   /* U+131D5 */
  0x0002,   /* U+131D6 */
  0x0002,   /* U+131D7 */
  0x0002,   /* U+131D8 */
  0x0002,   /* U+131D9 */
  0x0002,   /* U+131DA */
  0x0002,   /* U+131DB */
  0x0002,   /* U+131DC */
  0x0002,   /* U+131DD */
  0x0002,   /* U+131DE */
  0x0002,   /* U+131DF */
  0x0002,   /* U+131E0 */
  0x0002,   /* U+131E1 */
  0x0002,   /* U+131E2 */
  0x0002,   /* U+131E3 */
  0x0002,   /* U+131E4 */
  0x0002,   /* U+131E5 */
  0x0002,   /* U+131E6 */
  0x0002,   /* U+131E7 */
  0x0002,   /* U+131E8 */
  0x0002,   /* U+131E9 */
  0x0002,   /* U+131EA */
  0x0002,   /* U+131EB */
  0x0002,   /* U+131EC */
  0x0002,   /* U+131ED */
  0x0002,   /* U+131EE */
  0x0002,   /* U+131EF */
  0x0002,   /* U+131F0 */
  0x0002,   /* U+131F1 */
  0x0002,   /* U+131F2 */
  0x0002,   /* U+131F3 */
  0x0002,   /* U+131F4 */
  0x0002,   /* U+131F5 */
  0x0002,   /* U+131F6 */
  0x0002,   /* U+131F7 */
  0x0002,   /* U+131F8 */
  0x0002,   /* U+131F9 */
  0x0002,   /* U+131FA */
  0x0002,   /* U+131FB */
  0x0002,   /* U+131FC */
  0x0002,   /* U+131FD */
  0x0002,   /* U+131FE */
  0x0002,   /* U+131FF */
};

uint16 uca900_p132[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x5013,   /* U+13200 */
  0x5014,   /* U+13201 */
  0x5015,   /* U+13202 */
  0x5016,   /* U+13203 */
  0x5017,   /* U+13204 */
  0x5018,   /* U+13205 */
  0x5019,   /* U+13206 */
  0x501A,   /* U+13207 */
  0x501B,   /* U+13208 */
  0x501C,   /* U+13209 */
  0x501D,   /* U+1320A */
  0x501E,   /* U+1320B */
  0x501F,   /* U+1320C */
  0x5020,   /* U+1320D */
  0x5021,   /* U+1320E */
  0x5022,   /* U+1320F */
  0x5023,   /* U+13210 */
  0x5024,   /* U+13211 */
  0x5025,   /* U+13212 */
  0x5026,   /* U+13213 */
  0x5027,   /* U+13214 */
  0x5028,   /* U+13215 */
  0x5029,   /* U+13216 */
  0x502A,   /* U+13217 */
  0x502B,   /* U+13218 */
  0x502C,   /* U+13219 */
  0x502D,   /* U+1321A */
  0x502E,   /* U+1321B */
  0x502F,   /* U+1321C */
  0x5030,   /* U+1321D */
  0x5031,   /* U+1321E */
  0x5032,   /* U+1321F */
  0x5033,   /* U+13220 */
  0x5034,   /* U+13221 */
  0x5035,   /* U+13222 */
  0x5036,   /* U+13223 */
  0x5037,   /* U+13224 */
  0x5038,   /* U+13225 */
  0x5039,   /* U+13226 */
  0x503A,   /* U+13227 */
  0x503B,   /* U+13228 */
  0x503C,   /* U+13229 */
  0x503D,   /* U+1322A */
  0x503E,   /* U+1322B */
  0x503F,   /* U+1322C */
  0x5040,   /* U+1322D */
  0x5041,   /* U+1322E */
  0x5042,   /* U+1322F */
  0x5043,   /* U+13230 */
  0x5044,   /* U+13231 */
  0x5045,   /* U+13232 */
  0x5046,   /* U+13233 */
  0x5047,   /* U+13234 */
  0x5048,   /* U+13235 */
  0x5049,   /* U+13236 */
  0x504A,   /* U+13237 */
  0x504B,   /* U+13238 */
  0x504C,   /* U+13239 */
  0x504D,   /* U+1323A */
  0x504E,   /* U+1323B */
  0x504F,   /* U+1323C */
  0x5050,   /* U+1323D */
  0x5051,   /* U+1323E */
  0x5052,   /* U+1323F */
  0x5053,   /* U+13240 */
  0x5054,   /* U+13241 */
  0x5055,   /* U+13242 */
  0x5056,   /* U+13243 */
  0x5057,   /* U+13244 */
  0x5058,   /* U+13245 */
  0x5059,   /* U+13246 */
  0x505A,   /* U+13247 */
  0x505B,   /* U+13248 */
  0x505C,   /* U+13249 */
  0x505D,   /* U+1324A */
  0x505E,   /* U+1324B */
  0x505F,   /* U+1324C */
  0x5060,   /* U+1324D */
  0x5061,   /* U+1324E */
  0x5062,   /* U+1324F */
  0x5063,   /* U+13250 */
  0x5064,   /* U+13251 */
  0x5065,   /* U+13252 */
  0x5066,   /* U+13253 */
  0x5067,   /* U+13254 */
  0x5068,   /* U+13255 */
  0x5069,   /* U+13256 */
  0x506A,   /* U+13257 */
  0x506B,   /* U+13258 */
  0x506C,   /* U+13259 */
  0x506D,   /* U+1325A */
  0x506E,   /* U+1325B */
  0x506F,   /* U+1325C */
  0x5070,   /* U+1325D */
  0x5071,   /* U+1325E */
  0x5072,   /* U+1325F */
  0x5073,   /* U+13260 */
  0x5074,   /* U+13261 */
  0x5075,   /* U+13262 */
  0x5076,   /* U+13263 */
  0x5077,   /* U+13264 */
  0x5078,   /* U+13265 */
  0x5079,   /* U+13266 */
  0x507A,   /* U+13267 */
  0x507B,   /* U+13268 */
  0x507C,   /* U+13269 */
  0x507D,   /* U+1326A */
  0x507E,   /* U+1326B */
  0x507F,   /* U+1326C */
  0x5080,   /* U+1326D */
  0x5081,   /* U+1326E */
  0x5082,   /* U+1326F */
  0x5083,   /* U+13270 */
  0x5084,   /* U+13271 */
  0x5085,   /* U+13272 */
  0x5086,   /* U+13273 */
  0x5087,   /* U+13274 */
  0x5088,   /* U+13275 */
  0x5089,   /* U+13276 */
  0x508A,   /* U+13277 */
  0x508B,   /* U+13278 */
  0x508C,   /* U+13279 */
  0x508D,   /* U+1327A */
  0x508E,   /* U+1327B */
  0x508F,   /* U+1327C */
  0x5090,   /* U+1327D */
  0x5091,   /* U+1327E */
  0x5092,   /* U+1327F */
  0x5093,   /* U+13280 */
  0x5094,   /* U+13281 */
  0x5095,   /* U+13282 */
  0x5096,   /* U+13283 */
  0x5097,   /* U+13284 */
  0x5098,   /* U+13285 */
  0x5099,   /* U+13286 */
  0x509A,   /* U+13287 */
  0x509B,   /* U+13288 */
  0x509C,   /* U+13289 */
  0x509D,   /* U+1328A */
  0x509E,   /* U+1328B */
  0x509F,   /* U+1328C */
  0x50A0,   /* U+1328D */
  0x50A1,   /* U+1328E */
  0x50A2,   /* U+1328F */
  0x50A3,   /* U+13290 */
  0x50A4,   /* U+13291 */
  0x50A5,   /* U+13292 */
  0x50A6,   /* U+13293 */
  0x50A7,   /* U+13294 */
  0x50A8,   /* U+13295 */
  0x50A9,   /* U+13296 */
  0x50AA,   /* U+13297 */
  0x50AB,   /* U+13298 */
  0x50AC,   /* U+13299 */
  0x50AD,   /* U+1329A */
  0x50AE,   /* U+1329B */
  0x50AF,   /* U+1329C */
  0x50B0,   /* U+1329D */
  0x50B1,   /* U+1329E */
  0x50B2,   /* U+1329F */
  0x50B3,   /* U+132A0 */
  0x50B4,   /* U+132A1 */
  0x50B5,   /* U+132A2 */
  0x50B6,   /* U+132A3 */
  0x50B7,   /* U+132A4 */
  0x50B8,   /* U+132A5 */
  0x50B9,   /* U+132A6 */
  0x50BA,   /* U+132A7 */
  0x50BB,   /* U+132A8 */
  0x50BC,   /* U+132A9 */
  0x50BD,   /* U+132AA */
  0x50BE,   /* U+132AB */
  0x50BF,   /* U+132AC */
  0x50C0,   /* U+132AD */
  0x50C1,   /* U+132AE */
  0x50C2,   /* U+132AF */
  0x50C3,   /* U+132B0 */
  0x50C4,   /* U+132B1 */
  0x50C5,   /* U+132B2 */
  0x50C6,   /* U+132B3 */
  0x50C7,   /* U+132B4 */
  0x50C8,   /* U+132B5 */
  0x50C9,   /* U+132B6 */
  0x50CA,   /* U+132B7 */
  0x50CB,   /* U+132B8 */
  0x50CC,   /* U+132B9 */
  0x50CD,   /* U+132BA */
  0x50CE,   /* U+132BB */
  0x50CF,   /* U+132BC */
  0x50D0,   /* U+132BD */
  0x50D1,   /* U+132BE */
  0x50D2,   /* U+132BF */
  0x50D3,   /* U+132C0 */
  0x50D4,   /* U+132C1 */
  0x50D5,   /* U+132C2 */
  0x50D6,   /* U+132C3 */
  0x50D7,   /* U+132C4 */
  0x50D8,   /* U+132C5 */
  0x50D9,   /* U+132C6 */
  0x50DA,   /* U+132C7 */
  0x50DB,   /* U+132C8 */
  0x50DC,   /* U+132C9 */
  0x50DD,   /* U+132CA */
  0x50DE,   /* U+132CB */
  0x50DF,   /* U+132CC */
  0x50E0,   /* U+132CD */
  0x50E1,   /* U+132CE */
  0x50E2,   /* U+132CF */
  0x50E3,   /* U+132D0 */
  0x50E4,   /* U+132D1 */
  0x50E5,   /* U+132D2 */
  0x50E6,   /* U+132D3 */
  0x50E7,   /* U+132D4 */
  0x50E8,   /* U+132D5 */
  0x50E9,   /* U+132D6 */
  0x50EA,   /* U+132D7 */
  0x50EB,   /* U+132D8 */
  0x50EC,   /* U+132D9 */
  0x50ED,   /* U+132DA */
  0x50EE,   /* U+132DB */
  0x50EF,   /* U+132DC */
  0x50F0,   /* U+132DD */
  0x50F1,   /* U+132DE */
  0x50F2,   /* U+132DF */
  0x50F3,   /* U+132E0 */
  0x50F4,   /* U+132E1 */
  0x50F5,   /* U+132E2 */
  0x50F6,   /* U+132E3 */
  0x50F7,   /* U+132E4 */
  0x50F8,   /* U+132E5 */
  0x50F9,   /* U+132E6 */
  0x50FA,   /* U+132E7 */
  0x50FB,   /* U+132E8 */
  0x50FC,   /* U+132E9 */
  0x50FD,   /* U+132EA */
  0x50FE,   /* U+132EB */
  0x50FF,   /* U+132EC */
  0x5100,   /* U+132ED */
  0x5101,   /* U+132EE */
  0x5102,   /* U+132EF */
  0x5103,   /* U+132F0 */
  0x5104,   /* U+132F1 */
  0x5105,   /* U+132F2 */
  0x5106,   /* U+132F3 */
  0x5107,   /* U+132F4 */
  0x5108,   /* U+132F5 */
  0x5109,   /* U+132F6 */
  0x510A,   /* U+132F7 */
  0x510B,   /* U+132F8 */
  0x510C,   /* U+132F9 */
  0x510D,   /* U+132FA */
  0x510E,   /* U+132FB */
  0x510F,   /* U+132FC */
  0x5110,   /* U+132FD */
  0x5111,   /* U+132FE */
  0x5112,   /* U+132FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+13200 */
  0x0020,   /* U+13201 */
  0x0020,   /* U+13202 */
  0x0020,   /* U+13203 */
  0x0020,   /* U+13204 */
  0x0020,   /* U+13205 */
  0x0020,   /* U+13206 */
  0x0020,   /* U+13207 */
  0x0020,   /* U+13208 */
  0x0020,   /* U+13209 */
  0x0020,   /* U+1320A */
  0x0020,   /* U+1320B */
  0x0020,   /* U+1320C */
  0x0020,   /* U+1320D */
  0x0020,   /* U+1320E */
  0x0020,   /* U+1320F */
  0x0020,   /* U+13210 */
  0x0020,   /* U+13211 */
  0x0020,   /* U+13212 */
  0x0020,   /* U+13213 */
  0x0020,   /* U+13214 */
  0x0020,   /* U+13215 */
  0x0020,   /* U+13216 */
  0x0020,   /* U+13217 */
  0x0020,   /* U+13218 */
  0x0020,   /* U+13219 */
  0x0020,   /* U+1321A */
  0x0020,   /* U+1321B */
  0x0020,   /* U+1321C */
  0x0020,   /* U+1321D */
  0x0020,   /* U+1321E */
  0x0020,   /* U+1321F */
  0x0020,   /* U+13220 */
  0x0020,   /* U+13221 */
  0x0020,   /* U+13222 */
  0x0020,   /* U+13223 */
  0x0020,   /* U+13224 */
  0x0020,   /* U+13225 */
  0x0020,   /* U+13226 */
  0x0020,   /* U+13227 */
  0x0020,   /* U+13228 */
  0x0020,   /* U+13229 */
  0x0020,   /* U+1322A */
  0x0020,   /* U+1322B */
  0x0020,   /* U+1322C */
  0x0020,   /* U+1322D */
  0x0020,   /* U+1322E */
  0x0020,   /* U+1322F */
  0x0020,   /* U+13230 */
  0x0020,   /* U+13231 */
  0x0020,   /* U+13232 */
  0x0020,   /* U+13233 */
  0x0020,   /* U+13234 */
  0x0020,   /* U+13235 */
  0x0020,   /* U+13236 */
  0x0020,   /* U+13237 */
  0x0020,   /* U+13238 */
  0x0020,   /* U+13239 */
  0x0020,   /* U+1323A */
  0x0020,   /* U+1323B */
  0x0020,   /* U+1323C */
  0x0020,   /* U+1323D */
  0x0020,   /* U+1323E */
  0x0020,   /* U+1323F */
  0x0020,   /* U+13240 */
  0x0020,   /* U+13241 */
  0x0020,   /* U+13242 */
  0x0020,   /* U+13243 */
  0x0020,   /* U+13244 */
  0x0020,   /* U+13245 */
  0x0020,   /* U+13246 */
  0x0020,   /* U+13247 */
  0x0020,   /* U+13248 */
  0x0020,   /* U+13249 */
  0x0020,   /* U+1324A */
  0x0020,   /* U+1324B */
  0x0020,   /* U+1324C */
  0x0020,   /* U+1324D */
  0x0020,   /* U+1324E */
  0x0020,   /* U+1324F */
  0x0020,   /* U+13250 */
  0x0020,   /* U+13251 */
  0x0020,   /* U+13252 */
  0x0020,   /* U+13253 */
  0x0020,   /* U+13254 */
  0x0020,   /* U+13255 */
  0x0020,   /* U+13256 */
  0x0020,   /* U+13257 */
  0x0020,   /* U+13258 */
  0x0020,   /* U+13259 */
  0x0020,   /* U+1325A */
  0x0020,   /* U+1325B */
  0x0020,   /* U+1325C */
  0x0020,   /* U+1325D */
  0x0020,   /* U+1325E */
  0x0020,   /* U+1325F */
  0x0020,   /* U+13260 */
  0x0020,   /* U+13261 */
  0x0020,   /* U+13262 */
  0x0020,   /* U+13263 */
  0x0020,   /* U+13264 */
  0x0020,   /* U+13265 */
  0x0020,   /* U+13266 */
  0x0020,   /* U+13267 */
  0x0020,   /* U+13268 */
  0x0020,   /* U+13269 */
  0x0020,   /* U+1326A */
  0x0020,   /* U+1326B */
  0x0020,   /* U+1326C */
  0x0020,   /* U+1326D */
  0x0020,   /* U+1326E */
  0x0020,   /* U+1326F */
  0x0020,   /* U+13270 */
  0x0020,   /* U+13271 */
  0x0020,   /* U+13272 */
  0x0020,   /* U+13273 */
  0x0020,   /* U+13274 */
  0x0020,   /* U+13275 */
  0x0020,   /* U+13276 */
  0x0020,   /* U+13277 */
  0x0020,   /* U+13278 */
  0x0020,   /* U+13279 */
  0x0020,   /* U+1327A */
  0x0020,   /* U+1327B */
  0x0020,   /* U+1327C */
  0x0020,   /* U+1327D */
  0x0020,   /* U+1327E */
  0x0020,   /* U+1327F */
  0x0020,   /* U+13280 */
  0x0020,   /* U+13281 */
  0x0020,   /* U+13282 */
  0x0020,   /* U+13283 */
  0x0020,   /* U+13284 */
  0x0020,   /* U+13285 */
  0x0020,   /* U+13286 */
  0x0020,   /* U+13287 */
  0x0020,   /* U+13288 */
  0x0020,   /* U+13289 */
  0x0020,   /* U+1328A */
  0x0020,   /* U+1328B */
  0x0020,   /* U+1328C */
  0x0020,   /* U+1328D */
  0x0020,   /* U+1328E */
  0x0020,   /* U+1328F */
  0x0020,   /* U+13290 */
  0x0020,   /* U+13291 */
  0x0020,   /* U+13292 */
  0x0020,   /* U+13293 */
  0x0020,   /* U+13294 */
  0x0020,   /* U+13295 */
  0x0020,   /* U+13296 */
  0x0020,   /* U+13297 */
  0x0020,   /* U+13298 */
  0x0020,   /* U+13299 */
  0x0020,   /* U+1329A */
  0x0020,   /* U+1329B */
  0x0020,   /* U+1329C */
  0x0020,   /* U+1329D */
  0x0020,   /* U+1329E */
  0x0020,   /* U+1329F */
  0x0020,   /* U+132A0 */
  0x0020,   /* U+132A1 */
  0x0020,   /* U+132A2 */
  0x0020,   /* U+132A3 */
  0x0020,   /* U+132A4 */
  0x0020,   /* U+132A5 */
  0x0020,   /* U+132A6 */
  0x0020,   /* U+132A7 */
  0x0020,   /* U+132A8 */
  0x0020,   /* U+132A9 */
  0x0020,   /* U+132AA */
  0x0020,   /* U+132AB */
  0x0020,   /* U+132AC */
  0x0020,   /* U+132AD */
  0x0020,   /* U+132AE */
  0x0020,   /* U+132AF */
  0x0020,   /* U+132B0 */
  0x0020,   /* U+132B1 */
  0x0020,   /* U+132B2 */
  0x0020,   /* U+132B3 */
  0x0020,   /* U+132B4 */
  0x0020,   /* U+132B5 */
  0x0020,   /* U+132B6 */
  0x0020,   /* U+132B7 */
  0x0020,   /* U+132B8 */
  0x0020,   /* U+132B9 */
  0x0020,   /* U+132BA */
  0x0020,   /* U+132BB */
  0x0020,   /* U+132BC */
  0x0020,   /* U+132BD */
  0x0020,   /* U+132BE */
  0x0020,   /* U+132BF */
  0x0020,   /* U+132C0 */
  0x0020,   /* U+132C1 */
  0x0020,   /* U+132C2 */
  0x0020,   /* U+132C3 */
  0x0020,   /* U+132C4 */
  0x0020,   /* U+132C5 */
  0x0020,   /* U+132C6 */
  0x0020,   /* U+132C7 */
  0x0020,   /* U+132C8 */
  0x0020,   /* U+132C9 */
  0x0020,   /* U+132CA */
  0x0020,   /* U+132CB */
  0x0020,   /* U+132CC */
  0x0020,   /* U+132CD */
  0x0020,   /* U+132CE */
  0x0020,   /* U+132CF */
  0x0020,   /* U+132D0 */
  0x0020,   /* U+132D1 */
  0x0020,   /* U+132D2 */
  0x0020,   /* U+132D3 */
  0x0020,   /* U+132D4 */
  0x0020,   /* U+132D5 */
  0x0020,   /* U+132D6 */
  0x0020,   /* U+132D7 */
  0x0020,   /* U+132D8 */
  0x0020,   /* U+132D9 */
  0x0020,   /* U+132DA */
  0x0020,   /* U+132DB */
  0x0020,   /* U+132DC */
  0x0020,   /* U+132DD */
  0x0020,   /* U+132DE */
  0x0020,   /* U+132DF */
  0x0020,   /* U+132E0 */
  0x0020,   /* U+132E1 */
  0x0020,   /* U+132E2 */
  0x0020,   /* U+132E3 */
  0x0020,   /* U+132E4 */
  0x0020,   /* U+132E5 */
  0x0020,   /* U+132E6 */
  0x0020,   /* U+132E7 */
  0x0020,   /* U+132E8 */
  0x0020,   /* U+132E9 */
  0x0020,   /* U+132EA */
  0x0020,   /* U+132EB */
  0x0020,   /* U+132EC */
  0x0020,   /* U+132ED */
  0x0020,   /* U+132EE */
  0x0020,   /* U+132EF */
  0x0020,   /* U+132F0 */
  0x0020,   /* U+132F1 */
  0x0020,   /* U+132F2 */
  0x0020,   /* U+132F3 */
  0x0020,   /* U+132F4 */
  0x0020,   /* U+132F5 */
  0x0020,   /* U+132F6 */
  0x0020,   /* U+132F7 */
  0x0020,   /* U+132F8 */
  0x0020,   /* U+132F9 */
  0x0020,   /* U+132FA */
  0x0020,   /* U+132FB */
  0x0020,   /* U+132FC */
  0x0020,   /* U+132FD */
  0x0020,   /* U+132FE */
  0x0020,   /* U+132FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+13200 */
  0x0002,   /* U+13201 */
  0x0002,   /* U+13202 */
  0x0002,   /* U+13203 */
  0x0002,   /* U+13204 */
  0x0002,   /* U+13205 */
  0x0002,   /* U+13206 */
  0x0002,   /* U+13207 */
  0x0002,   /* U+13208 */
  0x0002,   /* U+13209 */
  0x0002,   /* U+1320A */
  0x0002,   /* U+1320B */
  0x0002,   /* U+1320C */
  0x0002,   /* U+1320D */
  0x0002,   /* U+1320E */
  0x0002,   /* U+1320F */
  0x0002,   /* U+13210 */
  0x0002,   /* U+13211 */
  0x0002,   /* U+13212 */
  0x0002,   /* U+13213 */
  0x0002,   /* U+13214 */
  0x0002,   /* U+13215 */
  0x0002,   /* U+13216 */
  0x0002,   /* U+13217 */
  0x0002,   /* U+13218 */
  0x0002,   /* U+13219 */
  0x0002,   /* U+1321A */
  0x0002,   /* U+1321B */
  0x0002,   /* U+1321C */
  0x0002,   /* U+1321D */
  0x0002,   /* U+1321E */
  0x0002,   /* U+1321F */
  0x0002,   /* U+13220 */
  0x0002,   /* U+13221 */
  0x0002,   /* U+13222 */
  0x0002,   /* U+13223 */
  0x0002,   /* U+13224 */
  0x0002,   /* U+13225 */
  0x0002,   /* U+13226 */
  0x0002,   /* U+13227 */
  0x0002,   /* U+13228 */
  0x0002,   /* U+13229 */
  0x0002,   /* U+1322A */
  0x0002,   /* U+1322B */
  0x0002,   /* U+1322C */
  0x0002,   /* U+1322D */
  0x0002,   /* U+1322E */
  0x0002,   /* U+1322F */
  0x0002,   /* U+13230 */
  0x0002,   /* U+13231 */
  0x0002,   /* U+13232 */
  0x0002,   /* U+13233 */
  0x0002,   /* U+13234 */
  0x0002,   /* U+13235 */
  0x0002,   /* U+13236 */
  0x0002,   /* U+13237 */
  0x0002,   /* U+13238 */
  0x0002,   /* U+13239 */
  0x0002,   /* U+1323A */
  0x0002,   /* U+1323B */
  0x0002,   /* U+1323C */
  0x0002,   /* U+1323D */
  0x0002,   /* U+1323E */
  0x0002,   /* U+1323F */
  0x0002,   /* U+13240 */
  0x0002,   /* U+13241 */
  0x0002,   /* U+13242 */
  0x0002,   /* U+13243 */
  0x0002,   /* U+13244 */
  0x0002,   /* U+13245 */
  0x0002,   /* U+13246 */
  0x0002,   /* U+13247 */
  0x0002,   /* U+13248 */
  0x0002,   /* U+13249 */
  0x0002,   /* U+1324A */
  0x0002,   /* U+1324B */
  0x0002,   /* U+1324C */
  0x0002,   /* U+1324D */
  0x0002,   /* U+1324E */
  0x0002,   /* U+1324F */
  0x0002,   /* U+13250 */
  0x0002,   /* U+13251 */
  0x0002,   /* U+13252 */
  0x0002,   /* U+13253 */
  0x0002,   /* U+13254 */
  0x0002,   /* U+13255 */
  0x0002,   /* U+13256 */
  0x0002,   /* U+13257 */
  0x0002,   /* U+13258 */
  0x0002,   /* U+13259 */
  0x0002,   /* U+1325A */
  0x0002,   /* U+1325B */
  0x0002,   /* U+1325C */
  0x0002,   /* U+1325D */
  0x0002,   /* U+1325E */
  0x0002,   /* U+1325F */
  0x0002,   /* U+13260 */
  0x0002,   /* U+13261 */
  0x0002,   /* U+13262 */
  0x0002,   /* U+13263 */
  0x0002,   /* U+13264 */
  0x0002,   /* U+13265 */
  0x0002,   /* U+13266 */
  0x0002,   /* U+13267 */
  0x0002,   /* U+13268 */
  0x0002,   /* U+13269 */
  0x0002,   /* U+1326A */
  0x0002,   /* U+1326B */
  0x0002,   /* U+1326C */
  0x0002,   /* U+1326D */
  0x0002,   /* U+1326E */
  0x0002,   /* U+1326F */
  0x0002,   /* U+13270 */
  0x0002,   /* U+13271 */
  0x0002,   /* U+13272 */
  0x0002,   /* U+13273 */
  0x0002,   /* U+13274 */
  0x0002,   /* U+13275 */
  0x0002,   /* U+13276 */
  0x0002,   /* U+13277 */
  0x0002,   /* U+13278 */
  0x0002,   /* U+13279 */
  0x0002,   /* U+1327A */
  0x0002,   /* U+1327B */
  0x0002,   /* U+1327C */
  0x0002,   /* U+1327D */
  0x0002,   /* U+1327E */
  0x0002,   /* U+1327F */
  0x0002,   /* U+13280 */
  0x0002,   /* U+13281 */
  0x0002,   /* U+13282 */
  0x0002,   /* U+13283 */
  0x0002,   /* U+13284 */
  0x0002,   /* U+13285 */
  0x0002,   /* U+13286 */
  0x0002,   /* U+13287 */
  0x0002,   /* U+13288 */
  0x0002,   /* U+13289 */
  0x0002,   /* U+1328A */
  0x0002,   /* U+1328B */
  0x0002,   /* U+1328C */
  0x0002,   /* U+1328D */
  0x0002,   /* U+1328E */
  0x0002,   /* U+1328F */
  0x0002,   /* U+13290 */
  0x0002,   /* U+13291 */
  0x0002,   /* U+13292 */
  0x0002,   /* U+13293 */
  0x0002,   /* U+13294 */
  0x0002,   /* U+13295 */
  0x0002,   /* U+13296 */
  0x0002,   /* U+13297 */
  0x0002,   /* U+13298 */
  0x0002,   /* U+13299 */
  0x0002,   /* U+1329A */
  0x0002,   /* U+1329B */
  0x0002,   /* U+1329C */
  0x0002,   /* U+1329D */
  0x0002,   /* U+1329E */
  0x0002,   /* U+1329F */
  0x0002,   /* U+132A0 */
  0x0002,   /* U+132A1 */
  0x0002,   /* U+132A2 */
  0x0002,   /* U+132A3 */
  0x0002,   /* U+132A4 */
  0x0002,   /* U+132A5 */
  0x0002,   /* U+132A6 */
  0x0002,   /* U+132A7 */
  0x0002,   /* U+132A8 */
  0x0002,   /* U+132A9 */
  0x0002,   /* U+132AA */
  0x0002,   /* U+132AB */
  0x0002,   /* U+132AC */
  0x0002,   /* U+132AD */
  0x0002,   /* U+132AE */
  0x0002,   /* U+132AF */
  0x0002,   /* U+132B0 */
  0x0002,   /* U+132B1 */
  0x0002,   /* U+132B2 */
  0x0002,   /* U+132B3 */
  0x0002,   /* U+132B4 */
  0x0002,   /* U+132B5 */
  0x0002,   /* U+132B6 */
  0x0002,   /* U+132B7 */
  0x0002,   /* U+132B8 */
  0x0002,   /* U+132B9 */
  0x0002,   /* U+132BA */
  0x0002,   /* U+132BB */
  0x0002,   /* U+132BC */
  0x0002,   /* U+132BD */
  0x0002,   /* U+132BE */
  0x0002,   /* U+132BF */
  0x0002,   /* U+132C0 */
  0x0002,   /* U+132C1 */
  0x0002,   /* U+132C2 */
  0x0002,   /* U+132C3 */
  0x0002,   /* U+132C4 */
  0x0002,   /* U+132C5 */
  0x0002,   /* U+132C6 */
  0x0002,   /* U+132C7 */
  0x0002,   /* U+132C8 */
  0x0002,   /* U+132C9 */
  0x0002,   /* U+132CA */
  0x0002,   /* U+132CB */
  0x0002,   /* U+132CC */
  0x0002,   /* U+132CD */
  0x0002,   /* U+132CE */
  0x0002,   /* U+132CF */
  0x0002,   /* U+132D0 */
  0x0002,   /* U+132D1 */
  0x0002,   /* U+132D2 */
  0x0002,   /* U+132D3 */
  0x0002,   /* U+132D4 */
  0x0002,   /* U+132D5 */
  0x0002,   /* U+132D6 */
  0x0002,   /* U+132D7 */
  0x0002,   /* U+132D8 */
  0x0002,   /* U+132D9 */
  0x0002,   /* U+132DA */
  0x0002,   /* U+132DB */
  0x0002,   /* U+132DC */
  0x0002,   /* U+132DD */
  0x0002,   /* U+132DE */
  0x0002,   /* U+132DF */
  0x0002,   /* U+132E0 */
  0x0002,   /* U+132E1 */
  0x0002,   /* U+132E2 */
  0x0002,   /* U+132E3 */
  0x0002,   /* U+132E4 */
  0x0002,   /* U+132E5 */
  0x0002,   /* U+132E6 */
  0x0002,   /* U+132E7 */
  0x0002,   /* U+132E8 */
  0x0002,   /* U+132E9 */
  0x0002,   /* U+132EA */
  0x0002,   /* U+132EB */
  0x0002,   /* U+132EC */
  0x0002,   /* U+132ED */
  0x0002,   /* U+132EE */
  0x0002,   /* U+132EF */
  0x0002,   /* U+132F0 */
  0x0002,   /* U+132F1 */
  0x0002,   /* U+132F2 */
  0x0002,   /* U+132F3 */
  0x0002,   /* U+132F4 */
  0x0002,   /* U+132F5 */
  0x0002,   /* U+132F6 */
  0x0002,   /* U+132F7 */
  0x0002,   /* U+132F8 */
  0x0002,   /* U+132F9 */
  0x0002,   /* U+132FA */
  0x0002,   /* U+132FB */
  0x0002,   /* U+132FC */
  0x0002,   /* U+132FD */
  0x0002,   /* U+132FE */
  0x0002,   /* U+132FF */
};

uint16 uca900_p133[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x5113,   /* U+13300 */
  0x5114,   /* U+13301 */
  0x5115,   /* U+13302 */
  0x5116,   /* U+13303 */
  0x5117,   /* U+13304 */
  0x5118,   /* U+13305 */
  0x5119,   /* U+13306 */
  0x511A,   /* U+13307 */
  0x511B,   /* U+13308 */
  0x511C,   /* U+13309 */
  0x511D,   /* U+1330A */
  0x511E,   /* U+1330B */
  0x511F,   /* U+1330C */
  0x5120,   /* U+1330D */
  0x5121,   /* U+1330E */
  0x5122,   /* U+1330F */
  0x5123,   /* U+13310 */
  0x5124,   /* U+13311 */
  0x5125,   /* U+13312 */
  0x5126,   /* U+13313 */
  0x5127,   /* U+13314 */
  0x5128,   /* U+13315 */
  0x5129,   /* U+13316 */
  0x512A,   /* U+13317 */
  0x512B,   /* U+13318 */
  0x512C,   /* U+13319 */
  0x512D,   /* U+1331A */
  0x512E,   /* U+1331B */
  0x512F,   /* U+1331C */
  0x5130,   /* U+1331D */
  0x5131,   /* U+1331E */
  0x5132,   /* U+1331F */
  0x5133,   /* U+13320 */
  0x5134,   /* U+13321 */
  0x5135,   /* U+13322 */
  0x5136,   /* U+13323 */
  0x5137,   /* U+13324 */
  0x5138,   /* U+13325 */
  0x5139,   /* U+13326 */
  0x513A,   /* U+13327 */
  0x513B,   /* U+13328 */
  0x513C,   /* U+13329 */
  0x513D,   /* U+1332A */
  0x513E,   /* U+1332B */
  0x513F,   /* U+1332C */
  0x5140,   /* U+1332D */
  0x5141,   /* U+1332E */
  0x5142,   /* U+1332F */
  0x5143,   /* U+13330 */
  0x5144,   /* U+13331 */
  0x5145,   /* U+13332 */
  0x5146,   /* U+13333 */
  0x5147,   /* U+13334 */
  0x5148,   /* U+13335 */
  0x5149,   /* U+13336 */
  0x514A,   /* U+13337 */
  0x514B,   /* U+13338 */
  0x514C,   /* U+13339 */
  0x514D,   /* U+1333A */
  0x514E,   /* U+1333B */
  0x514F,   /* U+1333C */
  0x5150,   /* U+1333D */
  0x5151,   /* U+1333E */
  0x5152,   /* U+1333F */
  0x5153,   /* U+13340 */
  0x5154,   /* U+13341 */
  0x5155,   /* U+13342 */
  0x5156,   /* U+13343 */
  0x5157,   /* U+13344 */
  0x5158,   /* U+13345 */
  0x5159,   /* U+13346 */
  0x515A,   /* U+13347 */
  0x515B,   /* U+13348 */
  0x515C,   /* U+13349 */
  0x515D,   /* U+1334A */
  0x515E,   /* U+1334B */
  0x515F,   /* U+1334C */
  0x5160,   /* U+1334D */
  0x5161,   /* U+1334E */
  0x5162,   /* U+1334F */
  0x5163,   /* U+13350 */
  0x5164,   /* U+13351 */
  0x5165,   /* U+13352 */
  0x5166,   /* U+13353 */
  0x5167,   /* U+13354 */
  0x5168,   /* U+13355 */
  0x5169,   /* U+13356 */
  0x516A,   /* U+13357 */
  0x516B,   /* U+13358 */
  0x516C,   /* U+13359 */
  0x516D,   /* U+1335A */
  0x516E,   /* U+1335B */
  0x516F,   /* U+1335C */
  0x5170,   /* U+1335D */
  0x5171,   /* U+1335E */
  0x5172,   /* U+1335F */
  0x5173,   /* U+13360 */
  0x5174,   /* U+13361 */
  0x5175,   /* U+13362 */
  0x5176,   /* U+13363 */
  0x5177,   /* U+13364 */
  0x5178,   /* U+13365 */
  0x5179,   /* U+13366 */
  0x517A,   /* U+13367 */
  0x517B,   /* U+13368 */
  0x517C,   /* U+13369 */
  0x517D,   /* U+1336A */
  0x517E,   /* U+1336B */
  0x517F,   /* U+1336C */
  0x5180,   /* U+1336D */
  0x5181,   /* U+1336E */
  0x5182,   /* U+1336F */
  0x5183,   /* U+13370 */
  0x5184,   /* U+13371 */
  0x5185,   /* U+13372 */
  0x5186,   /* U+13373 */
  0x5187,   /* U+13374 */
  0x5188,   /* U+13375 */
  0x5189,   /* U+13376 */
  0x518A,   /* U+13377 */
  0x518B,   /* U+13378 */
  0x518C,   /* U+13379 */
  0x518D,   /* U+1337A */
  0x518E,   /* U+1337B */
  0x518F,   /* U+1337C */
  0x5190,   /* U+1337D */
  0x5191,   /* U+1337E */
  0x5192,   /* U+1337F */
  0x5193,   /* U+13380 */
  0x5194,   /* U+13381 */
  0x5195,   /* U+13382 */
  0x5196,   /* U+13383 */
  0x5197,   /* U+13384 */
  0x5198,   /* U+13385 */
  0x5199,   /* U+13386 */
  0x519A,   /* U+13387 */
  0x519B,   /* U+13388 */
  0x519C,   /* U+13389 */
  0x519D,   /* U+1338A */
  0x519E,   /* U+1338B */
  0x519F,   /* U+1338C */
  0x51A0,   /* U+1338D */
  0x51A1,   /* U+1338E */
  0x51A2,   /* U+1338F */
  0x51A3,   /* U+13390 */
  0x51A4,   /* U+13391 */
  0x51A5,   /* U+13392 */
  0x51A6,   /* U+13393 */
  0x51A7,   /* U+13394 */
  0x51A8,   /* U+13395 */
  0x51A9,   /* U+13396 */
  0x51AA,   /* U+13397 */
  0x51AB,   /* U+13398 */
  0x51AC,   /* U+13399 */
  0x51AD,   /* U+1339A */
  0x51AE,   /* U+1339B */
  0x51AF,   /* U+1339C */
  0x51B0,   /* U+1339D */
  0x51B1,   /* U+1339E */
  0x51B2,   /* U+1339F */
  0x51B3,   /* U+133A0 */
  0x51B4,   /* U+133A1 */
  0x51B5,   /* U+133A2 */
  0x51B6,   /* U+133A3 */
  0x51B7,   /* U+133A4 */
  0x51B8,   /* U+133A5 */
  0x51B9,   /* U+133A6 */
  0x51BA,   /* U+133A7 */
  0x51BB,   /* U+133A8 */
  0x51BC,   /* U+133A9 */
  0x51BD,   /* U+133AA */
  0x51BE,   /* U+133AB */
  0x51BF,   /* U+133AC */
  0x51C0,   /* U+133AD */
  0x51C1,   /* U+133AE */
  0x51C2,   /* U+133AF */
  0x51C3,   /* U+133B0 */
  0x51C4,   /* U+133B1 */
  0x51C5,   /* U+133B2 */
  0x51C6,   /* U+133B3 */
  0x51C7,   /* U+133B4 */
  0x51C8,   /* U+133B5 */
  0x51C9,   /* U+133B6 */
  0x51CA,   /* U+133B7 */
  0x51CB,   /* U+133B8 */
  0x51CC,   /* U+133B9 */
  0x51CD,   /* U+133BA */
  0x51CE,   /* U+133BB */
  0x51CF,   /* U+133BC */
  0x51D0,   /* U+133BD */
  0x51D1,   /* U+133BE */
  0x51D2,   /* U+133BF */
  0x51D3,   /* U+133C0 */
  0x51D4,   /* U+133C1 */
  0x51D5,   /* U+133C2 */
  0x51D6,   /* U+133C3 */
  0x51D7,   /* U+133C4 */
  0x51D8,   /* U+133C5 */
  0x51D9,   /* U+133C6 */
  0x51DA,   /* U+133C7 */
  0x51DB,   /* U+133C8 */
  0x51DC,   /* U+133C9 */
  0x51DD,   /* U+133CA */
  0x51DE,   /* U+133CB */
  0x51DF,   /* U+133CC */
  0x51E0,   /* U+133CD */
  0x51E1,   /* U+133CE */
  0x51E2,   /* U+133CF */
  0x51E3,   /* U+133D0 */
  0x51E4,   /* U+133D1 */
  0x51E5,   /* U+133D2 */
  0x51E6,   /* U+133D3 */
  0x51E7,   /* U+133D4 */
  0x51E8,   /* U+133D5 */
  0x51E9,   /* U+133D6 */
  0x51EA,   /* U+133D7 */
  0x51EB,   /* U+133D8 */
  0x51EC,   /* U+133D9 */
  0x51ED,   /* U+133DA */
  0x51EE,   /* U+133DB */
  0x51EF,   /* U+133DC */
  0x51F0,   /* U+133DD */
  0x51F1,   /* U+133DE */
  0x51F2,   /* U+133DF */
  0x51F3,   /* U+133E0 */
  0x51F4,   /* U+133E1 */
  0x51F5,   /* U+133E2 */
  0x51F6,   /* U+133E3 */
  0x51F7,   /* U+133E4 */
  0x51F8,   /* U+133E5 */
  0x51F9,   /* U+133E6 */
  0x51FA,   /* U+133E7 */
  0x51FB,   /* U+133E8 */
  0x51FC,   /* U+133E9 */
  0x51FD,   /* U+133EA */
  0x51FE,   /* U+133EB */
  0x51FF,   /* U+133EC */
  0x5200,   /* U+133ED */
  0x5201,   /* U+133EE */
  0x5202,   /* U+133EF */
  0x5203,   /* U+133F0 */
  0x5204,   /* U+133F1 */
  0x5205,   /* U+133F2 */
  0x5206,   /* U+133F3 */
  0x5207,   /* U+133F4 */
  0x5208,   /* U+133F5 */
  0x5209,   /* U+133F6 */
  0x520A,   /* U+133F7 */
  0x520B,   /* U+133F8 */
  0x520C,   /* U+133F9 */
  0x520D,   /* U+133FA */
  0x520E,   /* U+133FB */
  0x520F,   /* U+133FC */
  0x5210,   /* U+133FD */
  0x5211,   /* U+133FE */
  0x5212,   /* U+133FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+13300 */
  0x0020,   /* U+13301 */
  0x0020,   /* U+13302 */
  0x0020,   /* U+13303 */
  0x0020,   /* U+13304 */
  0x0020,   /* U+13305 */
  0x0020,   /* U+13306 */
  0x0020,   /* U+13307 */
  0x0020,   /* U+13308 */
  0x0020,   /* U+13309 */
  0x0020,   /* U+1330A */
  0x0020,   /* U+1330B */
  0x0020,   /* U+1330C */
  0x0020,   /* U+1330D */
  0x0020,   /* U+1330E */
  0x0020,   /* U+1330F */
  0x0020,   /* U+13310 */
  0x0020,   /* U+13311 */
  0x0020,   /* U+13312 */
  0x0020,   /* U+13313 */
  0x0020,   /* U+13314 */
  0x0020,   /* U+13315 */
  0x0020,   /* U+13316 */
  0x0020,   /* U+13317 */
  0x0020,   /* U+13318 */
  0x0020,   /* U+13319 */
  0x0020,   /* U+1331A */
  0x0020,   /* U+1331B */
  0x0020,   /* U+1331C */
  0x0020,   /* U+1331D */
  0x0020,   /* U+1331E */
  0x0020,   /* U+1331F */
  0x0020,   /* U+13320 */
  0x0020,   /* U+13321 */
  0x0020,   /* U+13322 */
  0x0020,   /* U+13323 */
  0x0020,   /* U+13324 */
  0x0020,   /* U+13325 */
  0x0020,   /* U+13326 */
  0x0020,   /* U+13327 */
  0x0020,   /* U+13328 */
  0x0020,   /* U+13329 */
  0x0020,   /* U+1332A */
  0x0020,   /* U+1332B */
  0x0020,   /* U+1332C */
  0x0020,   /* U+1332D */
  0x0020,   /* U+1332E */
  0x0020,   /* U+1332F */
  0x0020,   /* U+13330 */
  0x0020,   /* U+13331 */
  0x0020,   /* U+13332 */
  0x0020,   /* U+13333 */
  0x0020,   /* U+13334 */
  0x0020,   /* U+13335 */
  0x0020,   /* U+13336 */
  0x0020,   /* U+13337 */
  0x0020,   /* U+13338 */
  0x0020,   /* U+13339 */
  0x0020,   /* U+1333A */
  0x0020,   /* U+1333B */
  0x0020,   /* U+1333C */
  0x0020,   /* U+1333D */
  0x0020,   /* U+1333E */
  0x0020,   /* U+1333F */
  0x0020,   /* U+13340 */
  0x0020,   /* U+13341 */
  0x0020,   /* U+13342 */
  0x0020,   /* U+13343 */
  0x0020,   /* U+13344 */
  0x0020,   /* U+13345 */
  0x0020,   /* U+13346 */
  0x0020,   /* U+13347 */
  0x0020,   /* U+13348 */
  0x0020,   /* U+13349 */
  0x0020,   /* U+1334A */
  0x0020,   /* U+1334B */
  0x0020,   /* U+1334C */
  0x0020,   /* U+1334D */
  0x0020,   /* U+1334E */
  0x0020,   /* U+1334F */
  0x0020,   /* U+13350 */
  0x0020,   /* U+13351 */
  0x0020,   /* U+13352 */
  0x0020,   /* U+13353 */
  0x0020,   /* U+13354 */
  0x0020,   /* U+13355 */
  0x0020,   /* U+13356 */
  0x0020,   /* U+13357 */
  0x0020,   /* U+13358 */
  0x0020,   /* U+13359 */
  0x0020,   /* U+1335A */
  0x0020,   /* U+1335B */
  0x0020,   /* U+1335C */
  0x0020,   /* U+1335D */
  0x0020,   /* U+1335E */
  0x0020,   /* U+1335F */
  0x0020,   /* U+13360 */
  0x0020,   /* U+13361 */
  0x0020,   /* U+13362 */
  0x0020,   /* U+13363 */
  0x0020,   /* U+13364 */
  0x0020,   /* U+13365 */
  0x0020,   /* U+13366 */
  0x0020,   /* U+13367 */
  0x0020,   /* U+13368 */
  0x0020,   /* U+13369 */
  0x0020,   /* U+1336A */
  0x0020,   /* U+1336B */
  0x0020,   /* U+1336C */
  0x0020,   /* U+1336D */
  0x0020,   /* U+1336E */
  0x0020,   /* U+1336F */
  0x0020,   /* U+13370 */
  0x0020,   /* U+13371 */
  0x0020,   /* U+13372 */
  0x0020,   /* U+13373 */
  0x0020,   /* U+13374 */
  0x0020,   /* U+13375 */
  0x0020,   /* U+13376 */
  0x0020,   /* U+13377 */
  0x0020,   /* U+13378 */
  0x0020,   /* U+13379 */
  0x0020,   /* U+1337A */
  0x0020,   /* U+1337B */
  0x0020,   /* U+1337C */
  0x0020,   /* U+1337D */
  0x0020,   /* U+1337E */
  0x0020,   /* U+1337F */
  0x0020,   /* U+13380 */
  0x0020,   /* U+13381 */
  0x0020,   /* U+13382 */
  0x0020,   /* U+13383 */
  0x0020,   /* U+13384 */
  0x0020,   /* U+13385 */
  0x0020,   /* U+13386 */
  0x0020,   /* U+13387 */
  0x0020,   /* U+13388 */
  0x0020,   /* U+13389 */
  0x0020,   /* U+1338A */
  0x0020,   /* U+1338B */
  0x0020,   /* U+1338C */
  0x0020,   /* U+1338D */
  0x0020,   /* U+1338E */
  0x0020,   /* U+1338F */
  0x0020,   /* U+13390 */
  0x0020,   /* U+13391 */
  0x0020,   /* U+13392 */
  0x0020,   /* U+13393 */
  0x0020,   /* U+13394 */
  0x0020,   /* U+13395 */
  0x0020,   /* U+13396 */
  0x0020,   /* U+13397 */
  0x0020,   /* U+13398 */
  0x0020,   /* U+13399 */
  0x0020,   /* U+1339A */
  0x0020,   /* U+1339B */
  0x0020,   /* U+1339C */
  0x0020,   /* U+1339D */
  0x0020,   /* U+1339E */
  0x0020,   /* U+1339F */
  0x0020,   /* U+133A0 */
  0x0020,   /* U+133A1 */
  0x0020,   /* U+133A2 */
  0x0020,   /* U+133A3 */
  0x0020,   /* U+133A4 */
  0x0020,   /* U+133A5 */
  0x0020,   /* U+133A6 */
  0x0020,   /* U+133A7 */
  0x0020,   /* U+133A8 */
  0x0020,   /* U+133A9 */
  0x0020,   /* U+133AA */
  0x0020,   /* U+133AB */
  0x0020,   /* U+133AC */
  0x0020,   /* U+133AD */
  0x0020,   /* U+133AE */
  0x0020,   /* U+133AF */
  0x0020,   /* U+133B0 */
  0x0020,   /* U+133B1 */
  0x0020,   /* U+133B2 */
  0x0020,   /* U+133B3 */
  0x0020,   /* U+133B4 */
  0x0020,   /* U+133B5 */
  0x0020,   /* U+133B6 */
  0x0020,   /* U+133B7 */
  0x0020,   /* U+133B8 */
  0x0020,   /* U+133B9 */
  0x0020,   /* U+133BA */
  0x0020,   /* U+133BB */
  0x0020,   /* U+133BC */
  0x0020,   /* U+133BD */
  0x0020,   /* U+133BE */
  0x0020,   /* U+133BF */
  0x0020,   /* U+133C0 */
  0x0020,   /* U+133C1 */
  0x0020,   /* U+133C2 */
  0x0020,   /* U+133C3 */
  0x0020,   /* U+133C4 */
  0x0020,   /* U+133C5 */
  0x0020,   /* U+133C6 */
  0x0020,   /* U+133C7 */
  0x0020,   /* U+133C8 */
  0x0020,   /* U+133C9 */
  0x0020,   /* U+133CA */
  0x0020,   /* U+133CB */
  0x0020,   /* U+133CC */
  0x0020,   /* U+133CD */
  0x0020,   /* U+133CE */
  0x0020,   /* U+133CF */
  0x0020,   /* U+133D0 */
  0x0020,   /* U+133D1 */
  0x0020,   /* U+133D2 */
  0x0020,   /* U+133D3 */
  0x0020,   /* U+133D4 */
  0x0020,   /* U+133D5 */
  0x0020,   /* U+133D6 */
  0x0020,   /* U+133D7 */
  0x0020,   /* U+133D8 */
  0x0020,   /* U+133D9 */
  0x0020,   /* U+133DA */
  0x0020,   /* U+133DB */
  0x0020,   /* U+133DC */
  0x0020,   /* U+133DD */
  0x0020,   /* U+133DE */
  0x0020,   /* U+133DF */
  0x0020,   /* U+133E0 */
  0x0020,   /* U+133E1 */
  0x0020,   /* U+133E2 */
  0x0020,   /* U+133E3 */
  0x0020,   /* U+133E4 */
  0x0020,   /* U+133E5 */
  0x0020,   /* U+133E6 */
  0x0020,   /* U+133E7 */
  0x0020,   /* U+133E8 */
  0x0020,   /* U+133E9 */
  0x0020,   /* U+133EA */
  0x0020,   /* U+133EB */
  0x0020,   /* U+133EC */
  0x0020,   /* U+133ED */
  0x0020,   /* U+133EE */
  0x0020,   /* U+133EF */
  0x0020,   /* U+133F0 */
  0x0020,   /* U+133F1 */
  0x0020,   /* U+133F2 */
  0x0020,   /* U+133F3 */
  0x0020,   /* U+133F4 */
  0x0020,   /* U+133F5 */
  0x0020,   /* U+133F6 */
  0x0020,   /* U+133F7 */
  0x0020,   /* U+133F8 */
  0x0020,   /* U+133F9 */
  0x0020,   /* U+133FA */
  0x0020,   /* U+133FB */
  0x0020,   /* U+133FC */
  0x0020,   /* U+133FD */
  0x0020,   /* U+133FE */
  0x0020,   /* U+133FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+13300 */
  0x0002,   /* U+13301 */
  0x0002,   /* U+13302 */
  0x0002,   /* U+13303 */
  0x0002,   /* U+13304 */
  0x0002,   /* U+13305 */
  0x0002,   /* U+13306 */
  0x0002,   /* U+13307 */
  0x0002,   /* U+13308 */
  0x0002,   /* U+13309 */
  0x0002,   /* U+1330A */
  0x0002,   /* U+1330B */
  0x0002,   /* U+1330C */
  0x0002,   /* U+1330D */
  0x0002,   /* U+1330E */
  0x0002,   /* U+1330F */
  0x0002,   /* U+13310 */
  0x0002,   /* U+13311 */
  0x0002,   /* U+13312 */
  0x0002,   /* U+13313 */
  0x0002,   /* U+13314 */
  0x0002,   /* U+13315 */
  0x0002,   /* U+13316 */
  0x0002,   /* U+13317 */
  0x0002,   /* U+13318 */
  0x0002,   /* U+13319 */
  0x0002,   /* U+1331A */
  0x0002,   /* U+1331B */
  0x0002,   /* U+1331C */
  0x0002,   /* U+1331D */
  0x0002,   /* U+1331E */
  0x0002,   /* U+1331F */
  0x0002,   /* U+13320 */
  0x0002,   /* U+13321 */
  0x0002,   /* U+13322 */
  0x0002,   /* U+13323 */
  0x0002,   /* U+13324 */
  0x0002,   /* U+13325 */
  0x0002,   /* U+13326 */
  0x0002,   /* U+13327 */
  0x0002,   /* U+13328 */
  0x0002,   /* U+13329 */
  0x0002,   /* U+1332A */
  0x0002,   /* U+1332B */
  0x0002,   /* U+1332C */
  0x0002,   /* U+1332D */
  0x0002,   /* U+1332E */
  0x0002,   /* U+1332F */
  0x0002,   /* U+13330 */
  0x0002,   /* U+13331 */
  0x0002,   /* U+13332 */
  0x0002,   /* U+13333 */
  0x0002,   /* U+13334 */
  0x0002,   /* U+13335 */
  0x0002,   /* U+13336 */
  0x0002,   /* U+13337 */
  0x0002,   /* U+13338 */
  0x0002,   /* U+13339 */
  0x0002,   /* U+1333A */
  0x0002,   /* U+1333B */
  0x0002,   /* U+1333C */
  0x0002,   /* U+1333D */
  0x0002,   /* U+1333E */
  0x0002,   /* U+1333F */
  0x0002,   /* U+13340 */
  0x0002,   /* U+13341 */
  0x0002,   /* U+13342 */
  0x0002,   /* U+13343 */
  0x0002,   /* U+13344 */
  0x0002,   /* U+13345 */
  0x0002,   /* U+13346 */
  0x0002,   /* U+13347 */
  0x0002,   /* U+13348 */
  0x0002,   /* U+13349 */
  0x0002,   /* U+1334A */
  0x0002,   /* U+1334B */
  0x0002,   /* U+1334C */
  0x0002,   /* U+1334D */
  0x0002,   /* U+1334E */
  0x0002,   /* U+1334F */
  0x0002,   /* U+13350 */
  0x0002,   /* U+13351 */
  0x0002,   /* U+13352 */
  0x0002,   /* U+13353 */
  0x0002,   /* U+13354 */
  0x0002,   /* U+13355 */
  0x0002,   /* U+13356 */
  0x0002,   /* U+13357 */
  0x0002,   /* U+13358 */
  0x0002,   /* U+13359 */
  0x0002,   /* U+1335A */
  0x0002,   /* U+1335B */
  0x0002,   /* U+1335C */
  0x0002,   /* U+1335D */
  0x0002,   /* U+1335E */
  0x0002,   /* U+1335F */
  0x0002,   /* U+13360 */
  0x0002,   /* U+13361 */
  0x0002,   /* U+13362 */
  0x0002,   /* U+13363 */
  0x0002,   /* U+13364 */
  0x0002,   /* U+13365 */
  0x0002,   /* U+13366 */
  0x0002,   /* U+13367 */
  0x0002,   /* U+13368 */
  0x0002,   /* U+13369 */
  0x0002,   /* U+1336A */
  0x0002,   /* U+1336B */
  0x0002,   /* U+1336C */
  0x0002,   /* U+1336D */
  0x0002,   /* U+1336E */
  0x0002,   /* U+1336F */
  0x0002,   /* U+13370 */
  0x0002,   /* U+13371 */
  0x0002,   /* U+13372 */
  0x0002,   /* U+13373 */
  0x0002,   /* U+13374 */
  0x0002,   /* U+13375 */
  0x0002,   /* U+13376 */
  0x0002,   /* U+13377 */
  0x0002,   /* U+13378 */
  0x0002,   /* U+13379 */
  0x0002,   /* U+1337A */
  0x0002,   /* U+1337B */
  0x0002,   /* U+1337C */
  0x0002,   /* U+1337D */
  0x0002,   /* U+1337E */
  0x0002,   /* U+1337F */
  0x0002,   /* U+13380 */
  0x0002,   /* U+13381 */
  0x0002,   /* U+13382 */
  0x0002,   /* U+13383 */
  0x0002,   /* U+13384 */
  0x0002,   /* U+13385 */
  0x0002,   /* U+13386 */
  0x0002,   /* U+13387 */
  0x0002,   /* U+13388 */
  0x0002,   /* U+13389 */
  0x0002,   /* U+1338A */
  0x0002,   /* U+1338B */
  0x0002,   /* U+1338C */
  0x0002,   /* U+1338D */
  0x0002,   /* U+1338E */
  0x0002,   /* U+1338F */
  0x0002,   /* U+13390 */
  0x0002,   /* U+13391 */
  0x0002,   /* U+13392 */
  0x0002,   /* U+13393 */
  0x0002,   /* U+13394 */
  0x0002,   /* U+13395 */
  0x0002,   /* U+13396 */
  0x0002,   /* U+13397 */
  0x0002,   /* U+13398 */
  0x0002,   /* U+13399 */
  0x0002,   /* U+1339A */
  0x0002,   /* U+1339B */
  0x0002,   /* U+1339C */
  0x0002,   /* U+1339D */
  0x0002,   /* U+1339E */
  0x0002,   /* U+1339F */
  0x0002,   /* U+133A0 */
  0x0002,   /* U+133A1 */
  0x0002,   /* U+133A2 */
  0x0002,   /* U+133A3 */
  0x0002,   /* U+133A4 */
  0x0002,   /* U+133A5 */
  0x0002,   /* U+133A6 */
  0x0002,   /* U+133A7 */
  0x0002,   /* U+133A8 */
  0x0002,   /* U+133A9 */
  0x0002,   /* U+133AA */
  0x0002,   /* U+133AB */
  0x0002,   /* U+133AC */
  0x0002,   /* U+133AD */
  0x0002,   /* U+133AE */
  0x0002,   /* U+133AF */
  0x0002,   /* U+133B0 */
  0x0002,   /* U+133B1 */
  0x0002,   /* U+133B2 */
  0x0002,   /* U+133B3 */
  0x0002,   /* U+133B4 */
  0x0002,   /* U+133B5 */
  0x0002,   /* U+133B6 */
  0x0002,   /* U+133B7 */
  0x0002,   /* U+133B8 */
  0x0002,   /* U+133B9 */
  0x0002,   /* U+133BA */
  0x0002,   /* U+133BB */
  0x0002,   /* U+133BC */
  0x0002,   /* U+133BD */
  0x0002,   /* U+133BE */
  0x0002,   /* U+133BF */
  0x0002,   /* U+133C0 */
  0x0002,   /* U+133C1 */
  0x0002,   /* U+133C2 */
  0x0002,   /* U+133C3 */
  0x0002,   /* U+133C4 */
  0x0002,   /* U+133C5 */
  0x0002,   /* U+133C6 */
  0x0002,   /* U+133C7 */
  0x0002,   /* U+133C8 */
  0x0002,   /* U+133C9 */
  0x0002,   /* U+133CA */
  0x0002,   /* U+133CB */
  0x0002,   /* U+133CC */
  0x0002,   /* U+133CD */
  0x0002,   /* U+133CE */
  0x0002,   /* U+133CF */
  0x0002,   /* U+133D0 */
  0x0002,   /* U+133D1 */
  0x0002,   /* U+133D2 */
  0x0002,   /* U+133D3 */
  0x0002,   /* U+133D4 */
  0x0002,   /* U+133D5 */
  0x0002,   /* U+133D6 */
  0x0002,   /* U+133D7 */
  0x0002,   /* U+133D8 */
  0x0002,   /* U+133D9 */
  0x0002,   /* U+133DA */
  0x0002,   /* U+133DB */
  0x0002,   /* U+133DC */
  0x0002,   /* U+133DD */
  0x0002,   /* U+133DE */
  0x0002,   /* U+133DF */
  0x0002,   /* U+133E0 */
  0x0002,   /* U+133E1 */
  0x0002,   /* U+133E2 */
  0x0002,   /* U+133E3 */
  0x0002,   /* U+133E4 */
  0x0002,   /* U+133E5 */
  0x0002,   /* U+133E6 */
  0x0002,   /* U+133E7 */
  0x0002,   /* U+133E8 */
  0x0002,   /* U+133E9 */
  0x0002,   /* U+133EA */
  0x0002,   /* U+133EB */
  0x0002,   /* U+133EC */
  0x0002,   /* U+133ED */
  0x0002,   /* U+133EE */
  0x0002,   /* U+133EF */
  0x0002,   /* U+133F0 */
  0x0002,   /* U+133F1 */
  0x0002,   /* U+133F2 */
  0x0002,   /* U+133F3 */
  0x0002,   /* U+133F4 */
  0x0002,   /* U+133F5 */
  0x0002,   /* U+133F6 */
  0x0002,   /* U+133F7 */
  0x0002,   /* U+133F8 */
  0x0002,   /* U+133F9 */
  0x0002,   /* U+133FA */
  0x0002,   /* U+133FB */
  0x0002,   /* U+133FC */
  0x0002,   /* U+133FD */
  0x0002,   /* U+133FE */
  0x0002,   /* U+133FF */
};

uint16 uca900_p134[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x5213,   /* U+13400 */
  0x5214,   /* U+13401 */
  0x5215,   /* U+13402 */
  0x5216,   /* U+13403 */
  0x5217,   /* U+13404 */
  0x5218,   /* U+13405 */
  0x5219,   /* U+13406 */
  0x521A,   /* U+13407 */
  0x521B,   /* U+13408 */
  0x521C,   /* U+13409 */
  0x521D,   /* U+1340A */
  0x521E,   /* U+1340B */
  0x521F,   /* U+1340C */
  0x5220,   /* U+1340D */
  0x5221,   /* U+1340E */
  0x5222,   /* U+1340F */
  0x5223,   /* U+13410 */
  0x5224,   /* U+13411 */
  0x5225,   /* U+13412 */
  0x5226,   /* U+13413 */
  0x5227,   /* U+13414 */
  0x5228,   /* U+13415 */
  0x5229,   /* U+13416 */
  0x522A,   /* U+13417 */
  0x522B,   /* U+13418 */
  0x522C,   /* U+13419 */
  0x522D,   /* U+1341A */
  0x522E,   /* U+1341B */
  0x522F,   /* U+1341C */
  0x5230,   /* U+1341D */
  0x5231,   /* U+1341E */
  0x5232,   /* U+1341F */
  0x5233,   /* U+13420 */
  0x5234,   /* U+13421 */
  0x5235,   /* U+13422 */
  0x5236,   /* U+13423 */
  0x5237,   /* U+13424 */
  0x5238,   /* U+13425 */
  0x5239,   /* U+13426 */
  0x523A,   /* U+13427 */
  0x523B,   /* U+13428 */
  0x523C,   /* U+13429 */
  0x523D,   /* U+1342A */
  0x523E,   /* U+1342B */
  0x523F,   /* U+1342C */
  0x5240,   /* U+1342D */
  0x5241,   /* U+1342E */
  0xFBC2,   /* U+1342F */
  0xFBC2,   /* U+13430 */
  0xFBC2,   /* U+13431 */
  0xFBC2,   /* U+13432 */
  0xFBC2,   /* U+13433 */
  0xFBC2,   /* U+13434 */
  0xFBC2,   /* U+13435 */
  0xFBC2,   /* U+13436 */
  0xFBC2,   /* U+13437 */
  0xFBC2,   /* U+13438 */
  0xFBC2,   /* U+13439 */
  0xFBC2,   /* U+1343A */
  0xFBC2,   /* U+1343B */
  0xFBC2,   /* U+1343C */
  0xFBC2,   /* U+1343D */
  0xFBC2,   /* U+1343E */
  0xFBC2,   /* U+1343F */
  0xFBC2,   /* U+13440 */
  0xFBC2,   /* U+13441 */
  0xFBC2,   /* U+13442 */
  0xFBC2,   /* U+13443 */
  0xFBC2,   /* U+13444 */
  0xFBC2,   /* U+13445 */
  0xFBC2,   /* U+13446 */
  0xFBC2,   /* U+13447 */
  0xFBC2,   /* U+13448 */
  0xFBC2,   /* U+13449 */
  0xFBC2,   /* U+1344A */
  0xFBC2,   /* U+1344B */
  0xFBC2,   /* U+1344C */
  0xFBC2,   /* U+1344D */
  0xFBC2,   /* U+1344E */
  0xFBC2,   /* U+1344F */
  0xFBC2,   /* U+13450 */
  0xFBC2,   /* U+13451 */
  0xFBC2,   /* U+13452 */
  0xFBC2,   /* U+13453 */
  0xFBC2,   /* U+13454 */
  0xFBC2,   /* U+13455 */
  0xFBC2,   /* U+13456 */
  0xFBC2,   /* U+13457 */
  0xFBC2,   /* U+13458 */
  0xFBC2,   /* U+13459 */
  0xFBC2,   /* U+1345A */
  0xFBC2,   /* U+1345B */
  0xFBC2,   /* U+1345C */
  0xFBC2,   /* U+1345D */
  0xFBC2,   /* U+1345E */
  0xFBC2,   /* U+1345F */
  0xFBC2,   /* U+13460 */
  0xFBC2,   /* U+13461 */
  0xFBC2,   /* U+13462 */
  0xFBC2,   /* U+13463 */
  0xFBC2,   /* U+13464 */
  0xFBC2,   /* U+13465 */
  0xFBC2,   /* U+13466 */
  0xFBC2,   /* U+13467 */
  0xFBC2,   /* U+13468 */
  0xFBC2,   /* U+13469 */
  0xFBC2,   /* U+1346A */
  0xFBC2,   /* U+1346B */
  0xFBC2,   /* U+1346C */
  0xFBC2,   /* U+1346D */
  0xFBC2,   /* U+1346E */
  0xFBC2,   /* U+1346F */
  0xFBC2,   /* U+13470 */
  0xFBC2,   /* U+13471 */
  0xFBC2,   /* U+13472 */
  0xFBC2,   /* U+13473 */
  0xFBC2,   /* U+13474 */
  0xFBC2,   /* U+13475 */
  0xFBC2,   /* U+13476 */
  0xFBC2,   /* U+13477 */
  0xFBC2,   /* U+13478 */
  0xFBC2,   /* U+13479 */
  0xFBC2,   /* U+1347A */
  0xFBC2,   /* U+1347B */
  0xFBC2,   /* U+1347C */
  0xFBC2,   /* U+1347D */
  0xFBC2,   /* U+1347E */
  0xFBC2,   /* U+1347F */
  0xFBC2,   /* U+13480 */
  0xFBC2,   /* U+13481 */
  0xFBC2,   /* U+13482 */
  0xFBC2,   /* U+13483 */
  0xFBC2,   /* U+13484 */
  0xFBC2,   /* U+13485 */
  0xFBC2,   /* U+13486 */
  0xFBC2,   /* U+13487 */
  0xFBC2,   /* U+13488 */
  0xFBC2,   /* U+13489 */
  0xFBC2,   /* U+1348A */
  0xFBC2,   /* U+1348B */
  0xFBC2,   /* U+1348C */
  0xFBC2,   /* U+1348D */
  0xFBC2,   /* U+1348E */
  0xFBC2,   /* U+1348F */
  0xFBC2,   /* U+13490 */
  0xFBC2,   /* U+13491 */
  0xFBC2,   /* U+13492 */
  0xFBC2,   /* U+13493 */
  0xFBC2,   /* U+13494 */
  0xFBC2,   /* U+13495 */
  0xFBC2,   /* U+13496 */
  0xFBC2,   /* U+13497 */
  0xFBC2,   /* U+13498 */
  0xFBC2,   /* U+13499 */
  0xFBC2,   /* U+1349A */
  0xFBC2,   /* U+1349B */
  0xFBC2,   /* U+1349C */
  0xFBC2,   /* U+1349D */
  0xFBC2,   /* U+1349E */
  0xFBC2,   /* U+1349F */
  0xFBC2,   /* U+134A0 */
  0xFBC2,   /* U+134A1 */
  0xFBC2,   /* U+134A2 */
  0xFBC2,   /* U+134A3 */
  0xFBC2,   /* U+134A4 */
  0xFBC2,   /* U+134A5 */
  0xFBC2,   /* U+134A6 */
  0xFBC2,   /* U+134A7 */
  0xFBC2,   /* U+134A8 */
  0xFBC2,   /* U+134A9 */
  0xFBC2,   /* U+134AA */
  0xFBC2,   /* U+134AB */
  0xFBC2,   /* U+134AC */
  0xFBC2,   /* U+134AD */
  0xFBC2,   /* U+134AE */
  0xFBC2,   /* U+134AF */
  0xFBC2,   /* U+134B0 */
  0xFBC2,   /* U+134B1 */
  0xFBC2,   /* U+134B2 */
  0xFBC2,   /* U+134B3 */
  0xFBC2,   /* U+134B4 */
  0xFBC2,   /* U+134B5 */
  0xFBC2,   /* U+134B6 */
  0xFBC2,   /* U+134B7 */
  0xFBC2,   /* U+134B8 */
  0xFBC2,   /* U+134B9 */
  0xFBC2,   /* U+134BA */
  0xFBC2,   /* U+134BB */
  0xFBC2,   /* U+134BC */
  0xFBC2,   /* U+134BD */
  0xFBC2,   /* U+134BE */
  0xFBC2,   /* U+134BF */
  0xFBC2,   /* U+134C0 */
  0xFBC2,   /* U+134C1 */
  0xFBC2,   /* U+134C2 */
  0xFBC2,   /* U+134C3 */
  0xFBC2,   /* U+134C4 */
  0xFBC2,   /* U+134C5 */
  0xFBC2,   /* U+134C6 */
  0xFBC2,   /* U+134C7 */
  0xFBC2,   /* U+134C8 */
  0xFBC2,   /* U+134C9 */
  0xFBC2,   /* U+134CA */
  0xFBC2,   /* U+134CB */
  0xFBC2,   /* U+134CC */
  0xFBC2,   /* U+134CD */
  0xFBC2,   /* U+134CE */
  0xFBC2,   /* U+134CF */
  0xFBC2,   /* U+134D0 */
  0xFBC2,   /* U+134D1 */
  0xFBC2,   /* U+134D2 */
  0xFBC2,   /* U+134D3 */
  0xFBC2,   /* U+134D4 */
  0xFBC2,   /* U+134D5 */
  0xFBC2,   /* U+134D6 */
  0xFBC2,   /* U+134D7 */
  0xFBC2,   /* U+134D8 */
  0xFBC2,   /* U+134D9 */
  0xFBC2,   /* U+134DA */
  0xFBC2,   /* U+134DB */
  0xFBC2,   /* U+134DC */
  0xFBC2,   /* U+134DD */
  0xFBC2,   /* U+134DE */
  0xFBC2,   /* U+134DF */
  0xFBC2,   /* U+134E0 */
  0xFBC2,   /* U+134E1 */
  0xFBC2,   /* U+134E2 */
  0xFBC2,   /* U+134E3 */
  0xFBC2,   /* U+134E4 */
  0xFBC2,   /* U+134E5 */
  0xFBC2,   /* U+134E6 */
  0xFBC2,   /* U+134E7 */
  0xFBC2,   /* U+134E8 */
  0xFBC2,   /* U+134E9 */
  0xFBC2,   /* U+134EA */
  0xFBC2,   /* U+134EB */
  0xFBC2,   /* U+134EC */
  0xFBC2,   /* U+134ED */
  0xFBC2,   /* U+134EE */
  0xFBC2,   /* U+134EF */
  0xFBC2,   /* U+134F0 */
  0xFBC2,   /* U+134F1 */
  0xFBC2,   /* U+134F2 */
  0xFBC2,   /* U+134F3 */
  0xFBC2,   /* U+134F4 */
  0xFBC2,   /* U+134F5 */
  0xFBC2,   /* U+134F6 */
  0xFBC2,   /* U+134F7 */
  0xFBC2,   /* U+134F8 */
  0xFBC2,   /* U+134F9 */
  0xFBC2,   /* U+134FA */
  0xFBC2,   /* U+134FB */
  0xFBC2,   /* U+134FC */
  0xFBC2,   /* U+134FD */
  0xFBC2,   /* U+134FE */
  0xFBC2,   /* U+134FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+13400 */
  0x0020,   /* U+13401 */
  0x0020,   /* U+13402 */
  0x0020,   /* U+13403 */
  0x0020,   /* U+13404 */
  0x0020,   /* U+13405 */
  0x0020,   /* U+13406 */
  0x0020,   /* U+13407 */
  0x0020,   /* U+13408 */
  0x0020,   /* U+13409 */
  0x0020,   /* U+1340A */
  0x0020,   /* U+1340B */
  0x0020,   /* U+1340C */
  0x0020,   /* U+1340D */
  0x0020,   /* U+1340E */
  0x0020,   /* U+1340F */
  0x0020,   /* U+13410 */
  0x0020,   /* U+13411 */
  0x0020,   /* U+13412 */
  0x0020,   /* U+13413 */
  0x0020,   /* U+13414 */
  0x0020,   /* U+13415 */
  0x0020,   /* U+13416 */
  0x0020,   /* U+13417 */
  0x0020,   /* U+13418 */
  0x0020,   /* U+13419 */
  0x0020,   /* U+1341A */
  0x0020,   /* U+1341B */
  0x0020,   /* U+1341C */
  0x0020,   /* U+1341D */
  0x0020,   /* U+1341E */
  0x0020,   /* U+1341F */
  0x0020,   /* U+13420 */
  0x0020,   /* U+13421 */
  0x0020,   /* U+13422 */
  0x0020,   /* U+13423 */
  0x0020,   /* U+13424 */
  0x0020,   /* U+13425 */
  0x0020,   /* U+13426 */
  0x0020,   /* U+13427 */
  0x0020,   /* U+13428 */
  0x0020,   /* U+13429 */
  0x0020,   /* U+1342A */
  0x0020,   /* U+1342B */
  0x0020,   /* U+1342C */
  0x0020,   /* U+1342D */
  0x0020,   /* U+1342E */
  0x0020,   /* U+1342F */
  0x0020,   /* U+13430 */
  0x0020,   /* U+13431 */
  0x0020,   /* U+13432 */
  0x0020,   /* U+13433 */
  0x0020,   /* U+13434 */
  0x0020,   /* U+13435 */
  0x0020,   /* U+13436 */
  0x0020,   /* U+13437 */
  0x0020,   /* U+13438 */
  0x0020,   /* U+13439 */
  0x0020,   /* U+1343A */
  0x0020,   /* U+1343B */
  0x0020,   /* U+1343C */
  0x0020,   /* U+1343D */
  0x0020,   /* U+1343E */
  0x0020,   /* U+1343F */
  0x0020,   /* U+13440 */
  0x0020,   /* U+13441 */
  0x0020,   /* U+13442 */
  0x0020,   /* U+13443 */
  0x0020,   /* U+13444 */
  0x0020,   /* U+13445 */
  0x0020,   /* U+13446 */
  0x0020,   /* U+13447 */
  0x0020,   /* U+13448 */
  0x0020,   /* U+13449 */
  0x0020,   /* U+1344A */
  0x0020,   /* U+1344B */
  0x0020,   /* U+1344C */
  0x0020,   /* U+1344D */
  0x0020,   /* U+1344E */
  0x0020,   /* U+1344F */
  0x0020,   /* U+13450 */
  0x0020,   /* U+13451 */
  0x0020,   /* U+13452 */
  0x0020,   /* U+13453 */
  0x0020,   /* U+13454 */
  0x0020,   /* U+13455 */
  0x0020,   /* U+13456 */
  0x0020,   /* U+13457 */
  0x0020,   /* U+13458 */
  0x0020,   /* U+13459 */
  0x0020,   /* U+1345A */
  0x0020,   /* U+1345B */
  0x0020,   /* U+1345C */
  0x0020,   /* U+1345D */
  0x0020,   /* U+1345E */
  0x0020,   /* U+1345F */
  0x0020,   /* U+13460 */
  0x0020,   /* U+13461 */
  0x0020,   /* U+13462 */
  0x0020,   /* U+13463 */
  0x0020,   /* U+13464 */
  0x0020,   /* U+13465 */
  0x0020,   /* U+13466 */
  0x0020,   /* U+13467 */
  0x0020,   /* U+13468 */
  0x0020,   /* U+13469 */
  0x0020,   /* U+1346A */
  0x0020,   /* U+1346B */
  0x0020,   /* U+1346C */
  0x0020,   /* U+1346D */
  0x0020,   /* U+1346E */
  0x0020,   /* U+1346F */
  0x0020,   /* U+13470 */
  0x0020,   /* U+13471 */
  0x0020,   /* U+13472 */
  0x0020,   /* U+13473 */
  0x0020,   /* U+13474 */
  0x0020,   /* U+13475 */
  0x0020,   /* U+13476 */
  0x0020,   /* U+13477 */
  0x0020,   /* U+13478 */
  0x0020,   /* U+13479 */
  0x0020,   /* U+1347A */
  0x0020,   /* U+1347B */
  0x0020,   /* U+1347C */
  0x0020,   /* U+1347D */
  0x0020,   /* U+1347E */
  0x0020,   /* U+1347F */
  0x0020,   /* U+13480 */
  0x0020,   /* U+13481 */
  0x0020,   /* U+13482 */
  0x0020,   /* U+13483 */
  0x0020,   /* U+13484 */
  0x0020,   /* U+13485 */
  0x0020,   /* U+13486 */
  0x0020,   /* U+13487 */
  0x0020,   /* U+13488 */
  0x0020,   /* U+13489 */
  0x0020,   /* U+1348A */
  0x0020,   /* U+1348B */
  0x0020,   /* U+1348C */
  0x0020,   /* U+1348D */
  0x0020,   /* U+1348E */
  0x0020,   /* U+1348F */
  0x0020,   /* U+13490 */
  0x0020,   /* U+13491 */
  0x0020,   /* U+13492 */
  0x0020,   /* U+13493 */
  0x0020,   /* U+13494 */
  0x0020,   /* U+13495 */
  0x0020,   /* U+13496 */
  0x0020,   /* U+13497 */
  0x0020,   /* U+13498 */
  0x0020,   /* U+13499 */
  0x0020,   /* U+1349A */
  0x0020,   /* U+1349B */
  0x0020,   /* U+1349C */
  0x0020,   /* U+1349D */
  0x0020,   /* U+1349E */
  0x0020,   /* U+1349F */
  0x0020,   /* U+134A0 */
  0x0020,   /* U+134A1 */
  0x0020,   /* U+134A2 */
  0x0020,   /* U+134A3 */
  0x0020,   /* U+134A4 */
  0x0020,   /* U+134A5 */
  0x0020,   /* U+134A6 */
  0x0020,   /* U+134A7 */
  0x0020,   /* U+134A8 */
  0x0020,   /* U+134A9 */
  0x0020,   /* U+134AA */
  0x0020,   /* U+134AB */
  0x0020,   /* U+134AC */
  0x0020,   /* U+134AD */
  0x0020,   /* U+134AE */
  0x0020,   /* U+134AF */
  0x0020,   /* U+134B0 */
  0x0020,   /* U+134B1 */
  0x0020,   /* U+134B2 */
  0x0020,   /* U+134B3 */
  0x0020,   /* U+134B4 */
  0x0020,   /* U+134B5 */
  0x0020,   /* U+134B6 */
  0x0020,   /* U+134B7 */
  0x0020,   /* U+134B8 */
  0x0020,   /* U+134B9 */
  0x0020,   /* U+134BA */
  0x0020,   /* U+134BB */
  0x0020,   /* U+134BC */
  0x0020,   /* U+134BD */
  0x0020,   /* U+134BE */
  0x0020,   /* U+134BF */
  0x0020,   /* U+134C0 */
  0x0020,   /* U+134C1 */
  0x0020,   /* U+134C2 */
  0x0020,   /* U+134C3 */
  0x0020,   /* U+134C4 */
  0x0020,   /* U+134C5 */
  0x0020,   /* U+134C6 */
  0x0020,   /* U+134C7 */
  0x0020,   /* U+134C8 */
  0x0020,   /* U+134C9 */
  0x0020,   /* U+134CA */
  0x0020,   /* U+134CB */
  0x0020,   /* U+134CC */
  0x0020,   /* U+134CD */
  0x0020,   /* U+134CE */
  0x0020,   /* U+134CF */
  0x0020,   /* U+134D0 */
  0x0020,   /* U+134D1 */
  0x0020,   /* U+134D2 */
  0x0020,   /* U+134D3 */
  0x0020,   /* U+134D4 */
  0x0020,   /* U+134D5 */
  0x0020,   /* U+134D6 */
  0x0020,   /* U+134D7 */
  0x0020,   /* U+134D8 */
  0x0020,   /* U+134D9 */
  0x0020,   /* U+134DA */
  0x0020,   /* U+134DB */
  0x0020,   /* U+134DC */
  0x0020,   /* U+134DD */
  0x0020,   /* U+134DE */
  0x0020,   /* U+134DF */
  0x0020,   /* U+134E0 */
  0x0020,   /* U+134E1 */
  0x0020,   /* U+134E2 */
  0x0020,   /* U+134E3 */
  0x0020,   /* U+134E4 */
  0x0020,   /* U+134E5 */
  0x0020,   /* U+134E6 */
  0x0020,   /* U+134E7 */
  0x0020,   /* U+134E8 */
  0x0020,   /* U+134E9 */
  0x0020,   /* U+134EA */
  0x0020,   /* U+134EB */
  0x0020,   /* U+134EC */
  0x0020,   /* U+134ED */
  0x0020,   /* U+134EE */
  0x0020,   /* U+134EF */
  0x0020,   /* U+134F0 */
  0x0020,   /* U+134F1 */
  0x0020,   /* U+134F2 */
  0x0020,   /* U+134F3 */
  0x0020,   /* U+134F4 */
  0x0020,   /* U+134F5 */
  0x0020,   /* U+134F6 */
  0x0020,   /* U+134F7 */
  0x0020,   /* U+134F8 */
  0x0020,   /* U+134F9 */
  0x0020,   /* U+134FA */
  0x0020,   /* U+134FB */
  0x0020,   /* U+134FC */
  0x0020,   /* U+134FD */
  0x0020,   /* U+134FE */
  0x0020,   /* U+134FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+13400 */
  0x0002,   /* U+13401 */
  0x0002,   /* U+13402 */
  0x0002,   /* U+13403 */
  0x0002,   /* U+13404 */
  0x0002,   /* U+13405 */
  0x0002,   /* U+13406 */
  0x0002,   /* U+13407 */
  0x0002,   /* U+13408 */
  0x0002,   /* U+13409 */
  0x0002,   /* U+1340A */
  0x0002,   /* U+1340B */
  0x0002,   /* U+1340C */
  0x0002,   /* U+1340D */
  0x0002,   /* U+1340E */
  0x0002,   /* U+1340F */
  0x0002,   /* U+13410 */
  0x0002,   /* U+13411 */
  0x0002,   /* U+13412 */
  0x0002,   /* U+13413 */
  0x0002,   /* U+13414 */
  0x0002,   /* U+13415 */
  0x0002,   /* U+13416 */
  0x0002,   /* U+13417 */
  0x0002,   /* U+13418 */
  0x0002,   /* U+13419 */
  0x0002,   /* U+1341A */
  0x0002,   /* U+1341B */
  0x0002,   /* U+1341C */
  0x0002,   /* U+1341D */
  0x0002,   /* U+1341E */
  0x0002,   /* U+1341F */
  0x0002,   /* U+13420 */
  0x0002,   /* U+13421 */
  0x0002,   /* U+13422 */
  0x0002,   /* U+13423 */
  0x0002,   /* U+13424 */
  0x0002,   /* U+13425 */
  0x0002,   /* U+13426 */
  0x0002,   /* U+13427 */
  0x0002,   /* U+13428 */
  0x0002,   /* U+13429 */
  0x0002,   /* U+1342A */
  0x0002,   /* U+1342B */
  0x0002,   /* U+1342C */
  0x0002,   /* U+1342D */
  0x0002,   /* U+1342E */
  0x0002,   /* U+1342F */
  0x0002,   /* U+13430 */
  0x0002,   /* U+13431 */
  0x0002,   /* U+13432 */
  0x0002,   /* U+13433 */
  0x0002,   /* U+13434 */
  0x0002,   /* U+13435 */
  0x0002,   /* U+13436 */
  0x0002,   /* U+13437 */
  0x0002,   /* U+13438 */
  0x0002,   /* U+13439 */
  0x0002,   /* U+1343A */
  0x0002,   /* U+1343B */
  0x0002,   /* U+1343C */
  0x0002,   /* U+1343D */
  0x0002,   /* U+1343E */
  0x0002,   /* U+1343F */
  0x0002,   /* U+13440 */
  0x0002,   /* U+13441 */
  0x0002,   /* U+13442 */
  0x0002,   /* U+13443 */
  0x0002,   /* U+13444 */
  0x0002,   /* U+13445 */
  0x0002,   /* U+13446 */
  0x0002,   /* U+13447 */
  0x0002,   /* U+13448 */
  0x0002,   /* U+13449 */
  0x0002,   /* U+1344A */
  0x0002,   /* U+1344B */
  0x0002,   /* U+1344C */
  0x0002,   /* U+1344D */
  0x0002,   /* U+1344E */
  0x0002,   /* U+1344F */
  0x0002,   /* U+13450 */
  0x0002,   /* U+13451 */
  0x0002,   /* U+13452 */
  0x0002,   /* U+13453 */
  0x0002,   /* U+13454 */
  0x0002,   /* U+13455 */
  0x0002,   /* U+13456 */
  0x0002,   /* U+13457 */
  0x0002,   /* U+13458 */
  0x0002,   /* U+13459 */
  0x0002,   /* U+1345A */
  0x0002,   /* U+1345B */
  0x0002,   /* U+1345C */
  0x0002,   /* U+1345D */
  0x0002,   /* U+1345E */
  0x0002,   /* U+1345F */
  0x0002,   /* U+13460 */
  0x0002,   /* U+13461 */
  0x0002,   /* U+13462 */
  0x0002,   /* U+13463 */
  0x0002,   /* U+13464 */
  0x0002,   /* U+13465 */
  0x0002,   /* U+13466 */
  0x0002,   /* U+13467 */
  0x0002,   /* U+13468 */
  0x0002,   /* U+13469 */
  0x0002,   /* U+1346A */
  0x0002,   /* U+1346B */
  0x0002,   /* U+1346C */
  0x0002,   /* U+1346D */
  0x0002,   /* U+1346E */
  0x0002,   /* U+1346F */
  0x0002,   /* U+13470 */
  0x0002,   /* U+13471 */
  0x0002,   /* U+13472 */
  0x0002,   /* U+13473 */
  0x0002,   /* U+13474 */
  0x0002,   /* U+13475 */
  0x0002,   /* U+13476 */
  0x0002,   /* U+13477 */
  0x0002,   /* U+13478 */
  0x0002,   /* U+13479 */
  0x0002,   /* U+1347A */
  0x0002,   /* U+1347B */
  0x0002,   /* U+1347C */
  0x0002,   /* U+1347D */
  0x0002,   /* U+1347E */
  0x0002,   /* U+1347F */
  0x0002,   /* U+13480 */
  0x0002,   /* U+13481 */
  0x0002,   /* U+13482 */
  0x0002,   /* U+13483 */
  0x0002,   /* U+13484 */
  0x0002,   /* U+13485 */
  0x0002,   /* U+13486 */
  0x0002,   /* U+13487 */
  0x0002,   /* U+13488 */
  0x0002,   /* U+13489 */
  0x0002,   /* U+1348A */
  0x0002,   /* U+1348B */
  0x0002,   /* U+1348C */
  0x0002,   /* U+1348D */
  0x0002,   /* U+1348E */
  0x0002,   /* U+1348F */
  0x0002,   /* U+13490 */
  0x0002,   /* U+13491 */
  0x0002,   /* U+13492 */
  0x0002,   /* U+13493 */
  0x0002,   /* U+13494 */
  0x0002,   /* U+13495 */
  0x0002,   /* U+13496 */
  0x0002,   /* U+13497 */
  0x0002,   /* U+13498 */
  0x0002,   /* U+13499 */
  0x0002,   /* U+1349A */
  0x0002,   /* U+1349B */
  0x0002,   /* U+1349C */
  0x0002,   /* U+1349D */
  0x0002,   /* U+1349E */
  0x0002,   /* U+1349F */
  0x0002,   /* U+134A0 */
  0x0002,   /* U+134A1 */
  0x0002,   /* U+134A2 */
  0x0002,   /* U+134A3 */
  0x0002,   /* U+134A4 */
  0x0002,   /* U+134A5 */
  0x0002,   /* U+134A6 */
  0x0002,   /* U+134A7 */
  0x0002,   /* U+134A8 */
  0x0002,   /* U+134A9 */
  0x0002,   /* U+134AA */
  0x0002,   /* U+134AB */
  0x0002,   /* U+134AC */
  0x0002,   /* U+134AD */
  0x0002,   /* U+134AE */
  0x0002,   /* U+134AF */
  0x0002,   /* U+134B0 */
  0x0002,   /* U+134B1 */
  0x0002,   /* U+134B2 */
  0x0002,   /* U+134B3 */
  0x0002,   /* U+134B4 */
  0x0002,   /* U+134B5 */
  0x0002,   /* U+134B6 */
  0x0002,   /* U+134B7 */
  0x0002,   /* U+134B8 */
  0x0002,   /* U+134B9 */
  0x0002,   /* U+134BA */
  0x0002,   /* U+134BB */
  0x0002,   /* U+134BC */
  0x0002,   /* U+134BD */
  0x0002,   /* U+134BE */
  0x0002,   /* U+134BF */
  0x0002,   /* U+134C0 */
  0x0002,   /* U+134C1 */
  0x0002,   /* U+134C2 */
  0x0002,   /* U+134C3 */
  0x0002,   /* U+134C4 */
  0x0002,   /* U+134C5 */
  0x0002,   /* U+134C6 */
  0x0002,   /* U+134C7 */
  0x0002,   /* U+134C8 */
  0x0002,   /* U+134C9 */
  0x0002,   /* U+134CA */
  0x0002,   /* U+134CB */
  0x0002,   /* U+134CC */
  0x0002,   /* U+134CD */
  0x0002,   /* U+134CE */
  0x0002,   /* U+134CF */
  0x0002,   /* U+134D0 */
  0x0002,   /* U+134D1 */
  0x0002,   /* U+134D2 */
  0x0002,   /* U+134D3 */
  0x0002,   /* U+134D4 */
  0x0002,   /* U+134D5 */
  0x0002,   /* U+134D6 */
  0x0002,   /* U+134D7 */
  0x0002,   /* U+134D8 */
  0x0002,   /* U+134D9 */
  0x0002,   /* U+134DA */
  0x0002,   /* U+134DB */
  0x0002,   /* U+134DC */
  0x0002,   /* U+134DD */
  0x0002,   /* U+134DE */
  0x0002,   /* U+134DF */
  0x0002,   /* U+134E0 */
  0x0002,   /* U+134E1 */
  0x0002,   /* U+134E2 */
  0x0002,   /* U+134E3 */
  0x0002,   /* U+134E4 */
  0x0002,   /* U+134E5 */
  0x0002,   /* U+134E6 */
  0x0002,   /* U+134E7 */
  0x0002,   /* U+134E8 */
  0x0002,   /* U+134E9 */
  0x0002,   /* U+134EA */
  0x0002,   /* U+134EB */
  0x0002,   /* U+134EC */
  0x0002,   /* U+134ED */
  0x0002,   /* U+134EE */
  0x0002,   /* U+134EF */
  0x0002,   /* U+134F0 */
  0x0002,   /* U+134F1 */
  0x0002,   /* U+134F2 */
  0x0002,   /* U+134F3 */
  0x0002,   /* U+134F4 */
  0x0002,   /* U+134F5 */
  0x0002,   /* U+134F6 */
  0x0002,   /* U+134F7 */
  0x0002,   /* U+134F8 */
  0x0002,   /* U+134F9 */
  0x0002,   /* U+134FA */
  0x0002,   /* U+134FB */
  0x0002,   /* U+134FC */
  0x0002,   /* U+134FD */
  0x0002,   /* U+134FE */
  0x0002,   /* U+134FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+13400 */
  0x0000,   /* U+13401 */
  0x0000,   /* U+13402 */
  0x0000,   /* U+13403 */
  0x0000,   /* U+13404 */
  0x0000,   /* U+13405 */
  0x0000,   /* U+13406 */
  0x0000,   /* U+13407 */
  0x0000,   /* U+13408 */
  0x0000,   /* U+13409 */
  0x0000,   /* U+1340A */
  0x0000,   /* U+1340B */
  0x0000,   /* U+1340C */
  0x0000,   /* U+1340D */
  0x0000,   /* U+1340E */
  0x0000,   /* U+1340F */
  0x0000,   /* U+13410 */
  0x0000,   /* U+13411 */
  0x0000,   /* U+13412 */
  0x0000,   /* U+13413 */
  0x0000,   /* U+13414 */
  0x0000,   /* U+13415 */
  0x0000,   /* U+13416 */
  0x0000,   /* U+13417 */
  0x0000,   /* U+13418 */
  0x0000,   /* U+13419 */
  0x0000,   /* U+1341A */
  0x0000,   /* U+1341B */
  0x0000,   /* U+1341C */
  0x0000,   /* U+1341D */
  0x0000,   /* U+1341E */
  0x0000,   /* U+1341F */
  0x0000,   /* U+13420 */
  0x0000,   /* U+13421 */
  0x0000,   /* U+13422 */
  0x0000,   /* U+13423 */
  0x0000,   /* U+13424 */
  0x0000,   /* U+13425 */
  0x0000,   /* U+13426 */
  0x0000,   /* U+13427 */
  0x0000,   /* U+13428 */
  0x0000,   /* U+13429 */
  0x0000,   /* U+1342A */
  0x0000,   /* U+1342B */
  0x0000,   /* U+1342C */
  0x0000,   /* U+1342D */
  0x0000,   /* U+1342E */
  0xB42F,   /* U+1342F */
  0xB430,   /* U+13430 */
  0xB431,   /* U+13431 */
  0xB432,   /* U+13432 */
  0xB433,   /* U+13433 */
  0xB434,   /* U+13434 */
  0xB435,   /* U+13435 */
  0xB436,   /* U+13436 */
  0xB437,   /* U+13437 */
  0xB438,   /* U+13438 */
  0xB439,   /* U+13439 */
  0xB43A,   /* U+1343A */
  0xB43B,   /* U+1343B */
  0xB43C,   /* U+1343C */
  0xB43D,   /* U+1343D */
  0xB43E,   /* U+1343E */
  0xB43F,   /* U+1343F */
  0xB440,   /* U+13440 */
  0xB441,   /* U+13441 */
  0xB442,   /* U+13442 */
  0xB443,   /* U+13443 */
  0xB444,   /* U+13444 */
  0xB445,   /* U+13445 */
  0xB446,   /* U+13446 */
  0xB447,   /* U+13447 */
  0xB448,   /* U+13448 */
  0xB449,   /* U+13449 */
  0xB44A,   /* U+1344A */
  0xB44B,   /* U+1344B */
  0xB44C,   /* U+1344C */
  0xB44D,   /* U+1344D */
  0xB44E,   /* U+1344E */
  0xB44F,   /* U+1344F */
  0xB450,   /* U+13450 */
  0xB451,   /* U+13451 */
  0xB452,   /* U+13452 */
  0xB453,   /* U+13453 */
  0xB454,   /* U+13454 */
  0xB455,   /* U+13455 */
  0xB456,   /* U+13456 */
  0xB457,   /* U+13457 */
  0xB458,   /* U+13458 */
  0xB459,   /* U+13459 */
  0xB45A,   /* U+1345A */
  0xB45B,   /* U+1345B */
  0xB45C,   /* U+1345C */
  0xB45D,   /* U+1345D */
  0xB45E,   /* U+1345E */
  0xB45F,   /* U+1345F */
  0xB460,   /* U+13460 */
  0xB461,   /* U+13461 */
  0xB462,   /* U+13462 */
  0xB463,   /* U+13463 */
  0xB464,   /* U+13464 */
  0xB465,   /* U+13465 */
  0xB466,   /* U+13466 */
  0xB467,   /* U+13467 */
  0xB468,   /* U+13468 */
  0xB469,   /* U+13469 */
  0xB46A,   /* U+1346A */
  0xB46B,   /* U+1346B */
  0xB46C,   /* U+1346C */
  0xB46D,   /* U+1346D */
  0xB46E,   /* U+1346E */
  0xB46F,   /* U+1346F */
  0xB470,   /* U+13470 */
  0xB471,   /* U+13471 */
  0xB472,   /* U+13472 */
  0xB473,   /* U+13473 */
  0xB474,   /* U+13474 */
  0xB475,   /* U+13475 */
  0xB476,   /* U+13476 */
  0xB477,   /* U+13477 */
  0xB478,   /* U+13478 */
  0xB479,   /* U+13479 */
  0xB47A,   /* U+1347A */
  0xB47B,   /* U+1347B */
  0xB47C,   /* U+1347C */
  0xB47D,   /* U+1347D */
  0xB47E,   /* U+1347E */
  0xB47F,   /* U+1347F */
  0xB480,   /* U+13480 */
  0xB481,   /* U+13481 */
  0xB482,   /* U+13482 */
  0xB483,   /* U+13483 */
  0xB484,   /* U+13484 */
  0xB485,   /* U+13485 */
  0xB486,   /* U+13486 */
  0xB487,   /* U+13487 */
  0xB488,   /* U+13488 */
  0xB489,   /* U+13489 */
  0xB48A,   /* U+1348A */
  0xB48B,   /* U+1348B */
  0xB48C,   /* U+1348C */
  0xB48D,   /* U+1348D */
  0xB48E,   /* U+1348E */
  0xB48F,   /* U+1348F */
  0xB490,   /* U+13490 */
  0xB491,   /* U+13491 */
  0xB492,   /* U+13492 */
  0xB493,   /* U+13493 */
  0xB494,   /* U+13494 */
  0xB495,   /* U+13495 */
  0xB496,   /* U+13496 */
  0xB497,   /* U+13497 */
  0xB498,   /* U+13498 */
  0xB499,   /* U+13499 */
  0xB49A,   /* U+1349A */
  0xB49B,   /* U+1349B */
  0xB49C,   /* U+1349C */
  0xB49D,   /* U+1349D */
  0xB49E,   /* U+1349E */
  0xB49F,   /* U+1349F */
  0xB4A0,   /* U+134A0 */
  0xB4A1,   /* U+134A1 */
  0xB4A2,   /* U+134A2 */
  0xB4A3,   /* U+134A3 */
  0xB4A4,   /* U+134A4 */
  0xB4A5,   /* U+134A5 */
  0xB4A6,   /* U+134A6 */
  0xB4A7,   /* U+134A7 */
  0xB4A8,   /* U+134A8 */
  0xB4A9,   /* U+134A9 */
  0xB4AA,   /* U+134AA */
  0xB4AB,   /* U+134AB */
  0xB4AC,   /* U+134AC */
  0xB4AD,   /* U+134AD */
  0xB4AE,   /* U+134AE */
  0xB4AF,   /* U+134AF */
  0xB4B0,   /* U+134B0 */
  0xB4B1,   /* U+134B1 */
  0xB4B2,   /* U+134B2 */
  0xB4B3,   /* U+134B3 */
  0xB4B4,   /* U+134B4 */
  0xB4B5,   /* U+134B5 */
  0xB4B6,   /* U+134B6 */
  0xB4B7,   /* U+134B7 */
  0xB4B8,   /* U+134B8 */
  0xB4B9,   /* U+134B9 */
  0xB4BA,   /* U+134BA */
  0xB4BB,   /* U+134BB */
  0xB4BC,   /* U+134BC */
  0xB4BD,   /* U+134BD */
  0xB4BE,   /* U+134BE */
  0xB4BF,   /* U+134BF */
  0xB4C0,   /* U+134C0 */
  0xB4C1,   /* U+134C1 */
  0xB4C2,   /* U+134C2 */
  0xB4C3,   /* U+134C3 */
  0xB4C4,   /* U+134C4 */
  0xB4C5,   /* U+134C5 */
  0xB4C6,   /* U+134C6 */
  0xB4C7,   /* U+134C7 */
  0xB4C8,   /* U+134C8 */
  0xB4C9,   /* U+134C9 */
  0xB4CA,   /* U+134CA */
  0xB4CB,   /* U+134CB */
  0xB4CC,   /* U+134CC */
  0xB4CD,   /* U+134CD */
  0xB4CE,   /* U+134CE */
  0xB4CF,   /* U+134CF */
  0xB4D0,   /* U+134D0 */
  0xB4D1,   /* U+134D1 */
  0xB4D2,   /* U+134D2 */
  0xB4D3,   /* U+134D3 */
  0xB4D4,   /* U+134D4 */
  0xB4D5,   /* U+134D5 */
  0xB4D6,   /* U+134D6 */
  0xB4D7,   /* U+134D7 */
  0xB4D8,   /* U+134D8 */
  0xB4D9,   /* U+134D9 */
  0xB4DA,   /* U+134DA */
  0xB4DB,   /* U+134DB */
  0xB4DC,   /* U+134DC */
  0xB4DD,   /* U+134DD */
  0xB4DE,   /* U+134DE */
  0xB4DF,   /* U+134DF */
  0xB4E0,   /* U+134E0 */
  0xB4E1,   /* U+134E1 */
  0xB4E2,   /* U+134E2 */
  0xB4E3,   /* U+134E3 */
  0xB4E4,   /* U+134E4 */
  0xB4E5,   /* U+134E5 */
  0xB4E6,   /* U+134E6 */
  0xB4E7,   /* U+134E7 */
  0xB4E8,   /* U+134E8 */
  0xB4E9,   /* U+134E9 */
  0xB4EA,   /* U+134EA */
  0xB4EB,   /* U+134EB */
  0xB4EC,   /* U+134EC */
  0xB4ED,   /* U+134ED */
  0xB4EE,   /* U+134EE */
  0xB4EF,   /* U+134EF */
  0xB4F0,   /* U+134F0 */
  0xB4F1,   /* U+134F1 */
  0xB4F2,   /* U+134F2 */
  0xB4F3,   /* U+134F3 */
  0xB4F4,   /* U+134F4 */
  0xB4F5,   /* U+134F5 */
  0xB4F6,   /* U+134F6 */
  0xB4F7,   /* U+134F7 */
  0xB4F8,   /* U+134F8 */
  0xB4F9,   /* U+134F9 */
  0xB4FA,   /* U+134FA */
  0xB4FB,   /* U+134FB */
  0xB4FC,   /* U+134FC */
  0xB4FD,   /* U+134FD */
  0xB4FE,   /* U+134FE */
  0xB4FF,   /* U+134FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+13400 */
  0x0000,   /* U+13401 */
  0x0000,   /* U+13402 */
  0x0000,   /* U+13403 */
  0x0000,   /* U+13404 */
  0x0000,   /* U+13405 */
  0x0000,   /* U+13406 */
  0x0000,   /* U+13407 */
  0x0000,   /* U+13408 */
  0x0000,   /* U+13409 */
  0x0000,   /* U+1340A */
  0x0000,   /* U+1340B */
  0x0000,   /* U+1340C */
  0x0000,   /* U+1340D */
  0x0000,   /* U+1340E */
  0x0000,   /* U+1340F */
  0x0000,   /* U+13410 */
  0x0000,   /* U+13411 */
  0x0000,   /* U+13412 */
  0x0000,   /* U+13413 */
  0x0000,   /* U+13414 */
  0x0000,   /* U+13415 */
  0x0000,   /* U+13416 */
  0x0000,   /* U+13417 */
  0x0000,   /* U+13418 */
  0x0000,   /* U+13419 */
  0x0000,   /* U+1341A */
  0x0000,   /* U+1341B */
  0x0000,   /* U+1341C */
  0x0000,   /* U+1341D */
  0x0000,   /* U+1341E */
  0x0000,   /* U+1341F */
  0x0000,   /* U+13420 */
  0x0000,   /* U+13421 */
  0x0000,   /* U+13422 */
  0x0000,   /* U+13423 */
  0x0000,   /* U+13424 */
  0x0000,   /* U+13425 */
  0x0000,   /* U+13426 */
  0x0000,   /* U+13427 */
  0x0000,   /* U+13428 */
  0x0000,   /* U+13429 */
  0x0000,   /* U+1342A */
  0x0000,   /* U+1342B */
  0x0000,   /* U+1342C */
  0x0000,   /* U+1342D */
  0x0000,   /* U+1342E */
  0x0000,   /* U+1342F */
  0x0000,   /* U+13430 */
  0x0000,   /* U+13431 */
  0x0000,   /* U+13432 */
  0x0000,   /* U+13433 */
  0x0000,   /* U+13434 */
  0x0000,   /* U+13435 */
  0x0000,   /* U+13436 */
  0x0000,   /* U+13437 */
  0x0000,   /* U+13438 */
  0x0000,   /* U+13439 */
  0x0000,   /* U+1343A */
  0x0000,   /* U+1343B */
  0x0000,   /* U+1343C */
  0x0000,   /* U+1343D */
  0x0000,   /* U+1343E */
  0x0000,   /* U+1343F */
  0x0000,   /* U+13440 */
  0x0000,   /* U+13441 */
  0x0000,   /* U+13442 */
  0x0000,   /* U+13443 */
  0x0000,   /* U+13444 */
  0x0000,   /* U+13445 */
  0x0000,   /* U+13446 */
  0x0000,   /* U+13447 */
  0x0000,   /* U+13448 */
  0x0000,   /* U+13449 */
  0x0000,   /* U+1344A */
  0x0000,   /* U+1344B */
  0x0000,   /* U+1344C */
  0x0000,   /* U+1344D */
  0x0000,   /* U+1344E */
  0x0000,   /* U+1344F */
  0x0000,   /* U+13450 */
  0x0000,   /* U+13451 */
  0x0000,   /* U+13452 */
  0x0000,   /* U+13453 */
  0x0000,   /* U+13454 */
  0x0000,   /* U+13455 */
  0x0000,   /* U+13456 */
  0x0000,   /* U+13457 */
  0x0000,   /* U+13458 */
  0x0000,   /* U+13459 */
  0x0000,   /* U+1345A */
  0x0000,   /* U+1345B */
  0x0000,   /* U+1345C */
  0x0000,   /* U+1345D */
  0x0000,   /* U+1345E */
  0x0000,   /* U+1345F */
  0x0000,   /* U+13460 */
  0x0000,   /* U+13461 */
  0x0000,   /* U+13462 */
  0x0000,   /* U+13463 */
  0x0000,   /* U+13464 */
  0x0000,   /* U+13465 */
  0x0000,   /* U+13466 */
  0x0000,   /* U+13467 */
  0x0000,   /* U+13468 */
  0x0000,   /* U+13469 */
  0x0000,   /* U+1346A */
  0x0000,   /* U+1346B */
  0x0000,   /* U+1346C */
  0x0000,   /* U+1346D */
  0x0000,   /* U+1346E */
  0x0000,   /* U+1346F */
  0x0000,   /* U+13470 */
  0x0000,   /* U+13471 */
  0x0000,   /* U+13472 */
  0x0000,   /* U+13473 */
  0x0000,   /* U+13474 */
  0x0000,   /* U+13475 */
  0x0000,   /* U+13476 */
  0x0000,   /* U+13477 */
  0x0000,   /* U+13478 */
  0x0000,   /* U+13479 */
  0x0000,   /* U+1347A */
  0x0000,   /* U+1347B */
  0x0000,   /* U+1347C */
  0x0000,   /* U+1347D */
  0x0000,   /* U+1347E */
  0x0000,   /* U+1347F */
  0x0000,   /* U+13480 */
  0x0000,   /* U+13481 */
  0x0000,   /* U+13482 */
  0x0000,   /* U+13483 */
  0x0000,   /* U+13484 */
  0x0000,   /* U+13485 */
  0x0000,   /* U+13486 */
  0x0000,   /* U+13487 */
  0x0000,   /* U+13488 */
  0x0000,   /* U+13489 */
  0x0000,   /* U+1348A */
  0x0000,   /* U+1348B */
  0x0000,   /* U+1348C */
  0x0000,   /* U+1348D */
  0x0000,   /* U+1348E */
  0x0000,   /* U+1348F */
  0x0000,   /* U+13490 */
  0x0000,   /* U+13491 */
  0x0000,   /* U+13492 */
  0x0000,   /* U+13493 */
  0x0000,   /* U+13494 */
  0x0000,   /* U+13495 */
  0x0000,   /* U+13496 */
  0x0000,   /* U+13497 */
  0x0000,   /* U+13498 */
  0x0000,   /* U+13499 */
  0x0000,   /* U+1349A */
  0x0000,   /* U+1349B */
  0x0000,   /* U+1349C */
  0x0000,   /* U+1349D */
  0x0000,   /* U+1349E */
  0x0000,   /* U+1349F */
  0x0000,   /* U+134A0 */
  0x0000,   /* U+134A1 */
  0x0000,   /* U+134A2 */
  0x0000,   /* U+134A3 */
  0x0000,   /* U+134A4 */
  0x0000,   /* U+134A5 */
  0x0000,   /* U+134A6 */
  0x0000,   /* U+134A7 */
  0x0000,   /* U+134A8 */
  0x0000,   /* U+134A9 */
  0x0000,   /* U+134AA */
  0x0000,   /* U+134AB */
  0x0000,   /* U+134AC */
  0x0000,   /* U+134AD */
  0x0000,   /* U+134AE */
  0x0000,   /* U+134AF */
  0x0000,   /* U+134B0 */
  0x0000,   /* U+134B1 */
  0x0000,   /* U+134B2 */
  0x0000,   /* U+134B3 */
  0x0000,   /* U+134B4 */
  0x0000,   /* U+134B5 */
  0x0000,   /* U+134B6 */
  0x0000,   /* U+134B7 */
  0x0000,   /* U+134B8 */
  0x0000,   /* U+134B9 */
  0x0000,   /* U+134BA */
  0x0000,   /* U+134BB */
  0x0000,   /* U+134BC */
  0x0000,   /* U+134BD */
  0x0000,   /* U+134BE */
  0x0000,   /* U+134BF */
  0x0000,   /* U+134C0 */
  0x0000,   /* U+134C1 */
  0x0000,   /* U+134C2 */
  0x0000,   /* U+134C3 */
  0x0000,   /* U+134C4 */
  0x0000,   /* U+134C5 */
  0x0000,   /* U+134C6 */
  0x0000,   /* U+134C7 */
  0x0000,   /* U+134C8 */
  0x0000,   /* U+134C9 */
  0x0000,   /* U+134CA */
  0x0000,   /* U+134CB */
  0x0000,   /* U+134CC */
  0x0000,   /* U+134CD */
  0x0000,   /* U+134CE */
  0x0000,   /* U+134CF */
  0x0000,   /* U+134D0 */
  0x0000,   /* U+134D1 */
  0x0000,   /* U+134D2 */
  0x0000,   /* U+134D3 */
  0x0000,   /* U+134D4 */
  0x0000,   /* U+134D5 */
  0x0000,   /* U+134D6 */
  0x0000,   /* U+134D7 */
  0x0000,   /* U+134D8 */
  0x0000,   /* U+134D9 */
  0x0000,   /* U+134DA */
  0x0000,   /* U+134DB */
  0x0000,   /* U+134DC */
  0x0000,   /* U+134DD */
  0x0000,   /* U+134DE */
  0x0000,   /* U+134DF */
  0x0000,   /* U+134E0 */
  0x0000,   /* U+134E1 */
  0x0000,   /* U+134E2 */
  0x0000,   /* U+134E3 */
  0x0000,   /* U+134E4 */
  0x0000,   /* U+134E5 */
  0x0000,   /* U+134E6 */
  0x0000,   /* U+134E7 */
  0x0000,   /* U+134E8 */
  0x0000,   /* U+134E9 */
  0x0000,   /* U+134EA */
  0x0000,   /* U+134EB */
  0x0000,   /* U+134EC */
  0x0000,   /* U+134ED */
  0x0000,   /* U+134EE */
  0x0000,   /* U+134EF */
  0x0000,   /* U+134F0 */
  0x0000,   /* U+134F1 */
  0x0000,   /* U+134F2 */
  0x0000,   /* U+134F3 */
  0x0000,   /* U+134F4 */
  0x0000,   /* U+134F5 */
  0x0000,   /* U+134F6 */
  0x0000,   /* U+134F7 */
  0x0000,   /* U+134F8 */
  0x0000,   /* U+134F9 */
  0x0000,   /* U+134FA */
  0x0000,   /* U+134FB */
  0x0000,   /* U+134FC */
  0x0000,   /* U+134FD */
  0x0000,   /* U+134FE */
  0x0000,   /* U+134FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+13400 */
  0x0000,   /* U+13401 */
  0x0000,   /* U+13402 */
  0x0000,   /* U+13403 */
  0x0000,   /* U+13404 */
  0x0000,   /* U+13405 */
  0x0000,   /* U+13406 */
  0x0000,   /* U+13407 */
  0x0000,   /* U+13408 */
  0x0000,   /* U+13409 */
  0x0000,   /* U+1340A */
  0x0000,   /* U+1340B */
  0x0000,   /* U+1340C */
  0x0000,   /* U+1340D */
  0x0000,   /* U+1340E */
  0x0000,   /* U+1340F */
  0x0000,   /* U+13410 */
  0x0000,   /* U+13411 */
  0x0000,   /* U+13412 */
  0x0000,   /* U+13413 */
  0x0000,   /* U+13414 */
  0x0000,   /* U+13415 */
  0x0000,   /* U+13416 */
  0x0000,   /* U+13417 */
  0x0000,   /* U+13418 */
  0x0000,   /* U+13419 */
  0x0000,   /* U+1341A */
  0x0000,   /* U+1341B */
  0x0000,   /* U+1341C */
  0x0000,   /* U+1341D */
  0x0000,   /* U+1341E */
  0x0000,   /* U+1341F */
  0x0000,   /* U+13420 */
  0x0000,   /* U+13421 */
  0x0000,   /* U+13422 */
  0x0000,   /* U+13423 */
  0x0000,   /* U+13424 */
  0x0000,   /* U+13425 */
  0x0000,   /* U+13426 */
  0x0000,   /* U+13427 */
  0x0000,   /* U+13428 */
  0x0000,   /* U+13429 */
  0x0000,   /* U+1342A */
  0x0000,   /* U+1342B */
  0x0000,   /* U+1342C */
  0x0000,   /* U+1342D */
  0x0000,   /* U+1342E */
  0x0000,   /* U+1342F */
  0x0000,   /* U+13430 */
  0x0000,   /* U+13431 */
  0x0000,   /* U+13432 */
  0x0000,   /* U+13433 */
  0x0000,   /* U+13434 */
  0x0000,   /* U+13435 */
  0x0000,   /* U+13436 */
  0x0000,   /* U+13437 */
  0x0000,   /* U+13438 */
  0x0000,   /* U+13439 */
  0x0000,   /* U+1343A */
  0x0000,   /* U+1343B */
  0x0000,   /* U+1343C */
  0x0000,   /* U+1343D */
  0x0000,   /* U+1343E */
  0x0000,   /* U+1343F */
  0x0000,   /* U+13440 */
  0x0000,   /* U+13441 */
  0x0000,   /* U+13442 */
  0x0000,   /* U+13443 */
  0x0000,   /* U+13444 */
  0x0000,   /* U+13445 */
  0x0000,   /* U+13446 */
  0x0000,   /* U+13447 */
  0x0000,   /* U+13448 */
  0x0000,   /* U+13449 */
  0x0000,   /* U+1344A */
  0x0000,   /* U+1344B */
  0x0000,   /* U+1344C */
  0x0000,   /* U+1344D */
  0x0000,   /* U+1344E */
  0x0000,   /* U+1344F */
  0x0000,   /* U+13450 */
  0x0000,   /* U+13451 */
  0x0000,   /* U+13452 */
  0x0000,   /* U+13453 */
  0x0000,   /* U+13454 */
  0x0000,   /* U+13455 */
  0x0000,   /* U+13456 */
  0x0000,   /* U+13457 */
  0x0000,   /* U+13458 */
  0x0000,   /* U+13459 */
  0x0000,   /* U+1345A */
  0x0000,   /* U+1345B */
  0x0000,   /* U+1345C */
  0x0000,   /* U+1345D */
  0x0000,   /* U+1345E */
  0x0000,   /* U+1345F */
  0x0000,   /* U+13460 */
  0x0000,   /* U+13461 */
  0x0000,   /* U+13462 */
  0x0000,   /* U+13463 */
  0x0000,   /* U+13464 */
  0x0000,   /* U+13465 */
  0x0000,   /* U+13466 */
  0x0000,   /* U+13467 */
  0x0000,   /* U+13468 */
  0x0000,   /* U+13469 */
  0x0000,   /* U+1346A */
  0x0000,   /* U+1346B */
  0x0000,   /* U+1346C */
  0x0000,   /* U+1346D */
  0x0000,   /* U+1346E */
  0x0000,   /* U+1346F */
  0x0000,   /* U+13470 */
  0x0000,   /* U+13471 */
  0x0000,   /* U+13472 */
  0x0000,   /* U+13473 */
  0x0000,   /* U+13474 */
  0x0000,   /* U+13475 */
  0x0000,   /* U+13476 */
  0x0000,   /* U+13477 */
  0x0000,   /* U+13478 */
  0x0000,   /* U+13479 */
  0x0000,   /* U+1347A */
  0x0000,   /* U+1347B */
  0x0000,   /* U+1347C */
  0x0000,   /* U+1347D */
  0x0000,   /* U+1347E */
  0x0000,   /* U+1347F */
  0x0000,   /* U+13480 */
  0x0000,   /* U+13481 */
  0x0000,   /* U+13482 */
  0x0000,   /* U+13483 */
  0x0000,   /* U+13484 */
  0x0000,   /* U+13485 */
  0x0000,   /* U+13486 */
  0x0000,   /* U+13487 */
  0x0000,   /* U+13488 */
  0x0000,   /* U+13489 */
  0x0000,   /* U+1348A */
  0x0000,   /* U+1348B */
  0x0000,   /* U+1348C */
  0x0000,   /* U+1348D */
  0x0000,   /* U+1348E */
  0x0000,   /* U+1348F */
  0x0000,   /* U+13490 */
  0x0000,   /* U+13491 */
  0x0000,   /* U+13492 */
  0x0000,   /* U+13493 */
  0x0000,   /* U+13494 */
  0x0000,   /* U+13495 */
  0x0000,   /* U+13496 */
  0x0000,   /* U+13497 */
  0x0000,   /* U+13498 */
  0x0000,   /* U+13499 */
  0x0000,   /* U+1349A */
  0x0000,   /* U+1349B */
  0x0000,   /* U+1349C */
  0x0000,   /* U+1349D */
  0x0000,   /* U+1349E */
  0x0000,   /* U+1349F */
  0x0000,   /* U+134A0 */
  0x0000,   /* U+134A1 */
  0x0000,   /* U+134A2 */
  0x0000,   /* U+134A3 */
  0x0000,   /* U+134A4 */
  0x0000,   /* U+134A5 */
  0x0000,   /* U+134A6 */
  0x0000,   /* U+134A7 */
  0x0000,   /* U+134A8 */
  0x0000,   /* U+134A9 */
  0x0000,   /* U+134AA */
  0x0000,   /* U+134AB */
  0x0000,   /* U+134AC */
  0x0000,   /* U+134AD */
  0x0000,   /* U+134AE */
  0x0000,   /* U+134AF */
  0x0000,   /* U+134B0 */
  0x0000,   /* U+134B1 */
  0x0000,   /* U+134B2 */
  0x0000,   /* U+134B3 */
  0x0000,   /* U+134B4 */
  0x0000,   /* U+134B5 */
  0x0000,   /* U+134B6 */
  0x0000,   /* U+134B7 */
  0x0000,   /* U+134B8 */
  0x0000,   /* U+134B9 */
  0x0000,   /* U+134BA */
  0x0000,   /* U+134BB */
  0x0000,   /* U+134BC */
  0x0000,   /* U+134BD */
  0x0000,   /* U+134BE */
  0x0000,   /* U+134BF */
  0x0000,   /* U+134C0 */
  0x0000,   /* U+134C1 */
  0x0000,   /* U+134C2 */
  0x0000,   /* U+134C3 */
  0x0000,   /* U+134C4 */
  0x0000,   /* U+134C5 */
  0x0000,   /* U+134C6 */
  0x0000,   /* U+134C7 */
  0x0000,   /* U+134C8 */
  0x0000,   /* U+134C9 */
  0x0000,   /* U+134CA */
  0x0000,   /* U+134CB */
  0x0000,   /* U+134CC */
  0x0000,   /* U+134CD */
  0x0000,   /* U+134CE */
  0x0000,   /* U+134CF */
  0x0000,   /* U+134D0 */
  0x0000,   /* U+134D1 */
  0x0000,   /* U+134D2 */
  0x0000,   /* U+134D3 */
  0x0000,   /* U+134D4 */
  0x0000,   /* U+134D5 */
  0x0000,   /* U+134D6 */
  0x0000,   /* U+134D7 */
  0x0000,   /* U+134D8 */
  0x0000,   /* U+134D9 */
  0x0000,   /* U+134DA */
  0x0000,   /* U+134DB */
  0x0000,   /* U+134DC */
  0x0000,   /* U+134DD */
  0x0000,   /* U+134DE */
  0x0000,   /* U+134DF */
  0x0000,   /* U+134E0 */
  0x0000,   /* U+134E1 */
  0x0000,   /* U+134E2 */
  0x0000,   /* U+134E3 */
  0x0000,   /* U+134E4 */
  0x0000,   /* U+134E5 */
  0x0000,   /* U+134E6 */
  0x0000,   /* U+134E7 */
  0x0000,   /* U+134E8 */
  0x0000,   /* U+134E9 */
  0x0000,   /* U+134EA */
  0x0000,   /* U+134EB */
  0x0000,   /* U+134EC */
  0x0000,   /* U+134ED */
  0x0000,   /* U+134EE */
  0x0000,   /* U+134EF */
  0x0000,   /* U+134F0 */
  0x0000,   /* U+134F1 */
  0x0000,   /* U+134F2 */
  0x0000,   /* U+134F3 */
  0x0000,   /* U+134F4 */
  0x0000,   /* U+134F5 */
  0x0000,   /* U+134F6 */
  0x0000,   /* U+134F7 */
  0x0000,   /* U+134F8 */
  0x0000,   /* U+134F9 */
  0x0000,   /* U+134FA */
  0x0000,   /* U+134FB */
  0x0000,   /* U+134FC */
  0x0000,   /* U+134FD */
  0x0000,   /* U+134FE */
  0x0000,   /* U+134FF */
};

uint16 uca900_p144[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x525D,   /* U+14400 */
  0x525E,   /* U+14401 */
  0x525F,   /* U+14402 */
  0x5260,   /* U+14403 */
  0x5261,   /* U+14404 */
  0x5262,   /* U+14405 */
  0x5263,   /* U+14406 */
  0x5264,   /* U+14407 */
  0x5265,   /* U+14408 */
  0x5266,   /* U+14409 */
  0x5267,   /* U+1440A */
  0x5268,   /* U+1440B */
  0x5269,   /* U+1440C */
  0x526A,   /* U+1440D */
  0x526B,   /* U+1440E */
  0x526C,   /* U+1440F */
  0x526D,   /* U+14410 */
  0x526E,   /* U+14411 */
  0x526F,   /* U+14412 */
  0x5270,   /* U+14413 */
  0x5271,   /* U+14414 */
  0x5272,   /* U+14415 */
  0x5273,   /* U+14416 */
  0x5274,   /* U+14417 */
  0x5275,   /* U+14418 */
  0x5276,   /* U+14419 */
  0x5277,   /* U+1441A */
  0x5278,   /* U+1441B */
  0x5279,   /* U+1441C */
  0x527A,   /* U+1441D */
  0x527B,   /* U+1441E */
  0x527C,   /* U+1441F */
  0x527D,   /* U+14420 */
  0x527E,   /* U+14421 */
  0x527F,   /* U+14422 */
  0x5280,   /* U+14423 */
  0x5281,   /* U+14424 */
  0x5282,   /* U+14425 */
  0x5283,   /* U+14426 */
  0x5284,   /* U+14427 */
  0x5285,   /* U+14428 */
  0x5286,   /* U+14429 */
  0x5287,   /* U+1442A */
  0x5288,   /* U+1442B */
  0x5289,   /* U+1442C */
  0x528A,   /* U+1442D */
  0x528B,   /* U+1442E */
  0x528C,   /* U+1442F */
  0x528D,   /* U+14430 */
  0x528E,   /* U+14431 */
  0x528F,   /* U+14432 */
  0x5290,   /* U+14433 */
  0x5291,   /* U+14434 */
  0x5292,   /* U+14435 */
  0x5293,   /* U+14436 */
  0x5294,   /* U+14437 */
  0x5295,   /* U+14438 */
  0x5296,   /* U+14439 */
  0x5297,   /* U+1443A */
  0x5298,   /* U+1443B */
  0x5299,   /* U+1443C */
  0x529A,   /* U+1443D */
  0x529B,   /* U+1443E */
  0x529C,   /* U+1443F */
  0x529D,   /* U+14440 */
  0x529E,   /* U+14441 */
  0x529F,   /* U+14442 */
  0x52A0,   /* U+14443 */
  0x52A1,   /* U+14444 */
  0x52A2,   /* U+14445 */
  0x52A3,   /* U+14446 */
  0x52A4,   /* U+14447 */
  0x52A5,   /* U+14448 */
  0x52A6,   /* U+14449 */
  0x52A7,   /* U+1444A */
  0x52A8,   /* U+1444B */
  0x52A9,   /* U+1444C */
  0x52AA,   /* U+1444D */
  0x52AB,   /* U+1444E */
  0x52AC,   /* U+1444F */
  0x52AD,   /* U+14450 */
  0x52AE,   /* U+14451 */
  0x52AF,   /* U+14452 */
  0x52B0,   /* U+14453 */
  0x52B1,   /* U+14454 */
  0x52B2,   /* U+14455 */
  0x52B3,   /* U+14456 */
  0x52B4,   /* U+14457 */
  0x52B5,   /* U+14458 */
  0x52B6,   /* U+14459 */
  0x52B7,   /* U+1445A */
  0x52B8,   /* U+1445B */
  0x52B9,   /* U+1445C */
  0x52BA,   /* U+1445D */
  0x52BB,   /* U+1445E */
  0x52BC,   /* U+1445F */
  0x52BD,   /* U+14460 */
  0x52BE,   /* U+14461 */
  0x52BF,   /* U+14462 */
  0x52C0,   /* U+14463 */
  0x52C1,   /* U+14464 */
  0x52C2,   /* U+14465 */
  0x52C3,   /* U+14466 */
  0x52C4,   /* U+14467 */
  0x52C5,   /* U+14468 */
  0x52C6,   /* U+14469 */
  0x52C7,   /* U+1446A */
  0x52C8,   /* U+1446B */
  0x52C9,   /* U+1446C */
  0x52CA,   /* U+1446D */
  0x52CB,   /* U+1446E */
  0x52CC,   /* U+1446F */
  0x52CD,   /* U+14470 */
  0x52CE,   /* U+14471 */
  0x52CF,   /* U+14472 */
  0x52D0,   /* U+14473 */
  0x52D1,   /* U+14474 */
  0x52D2,   /* U+14475 */
  0x52D3,   /* U+14476 */
  0x52D4,   /* U+14477 */
  0x52D5,   /* U+14478 */
  0x52D6,   /* U+14479 */
  0x52D7,   /* U+1447A */
  0x52D8,   /* U+1447B */
  0x52D9,   /* U+1447C */
  0x52DA,   /* U+1447D */
  0x52DB,   /* U+1447E */
  0x52DC,   /* U+1447F */
  0x52DD,   /* U+14480 */
  0x52DE,   /* U+14481 */
  0x52DF,   /* U+14482 */
  0x52E0,   /* U+14483 */
  0x52E1,   /* U+14484 */
  0x52E2,   /* U+14485 */
  0x52E3,   /* U+14486 */
  0x52E4,   /* U+14487 */
  0x52E5,   /* U+14488 */
  0x52E6,   /* U+14489 */
  0x52E7,   /* U+1448A */
  0x52E8,   /* U+1448B */
  0x52E9,   /* U+1448C */
  0x52EA,   /* U+1448D */
  0x52EB,   /* U+1448E */
  0x52EC,   /* U+1448F */
  0x52ED,   /* U+14490 */
  0x52EE,   /* U+14491 */
  0x52EF,   /* U+14492 */
  0x52F0,   /* U+14493 */
  0x52F1,   /* U+14494 */
  0x52F2,   /* U+14495 */
  0x52F3,   /* U+14496 */
  0x52F4,   /* U+14497 */
  0x52F5,   /* U+14498 */
  0x52F6,   /* U+14499 */
  0x52F7,   /* U+1449A */
  0x52F8,   /* U+1449B */
  0x52F9,   /* U+1449C */
  0x52FA,   /* U+1449D */
  0x52FB,   /* U+1449E */
  0x52FC,   /* U+1449F */
  0x52FD,   /* U+144A0 */
  0x52FE,   /* U+144A1 */
  0x52FF,   /* U+144A2 */
  0x5300,   /* U+144A3 */
  0x5301,   /* U+144A4 */
  0x5302,   /* U+144A5 */
  0x5303,   /* U+144A6 */
  0x5304,   /* U+144A7 */
  0x5305,   /* U+144A8 */
  0x5306,   /* U+144A9 */
  0x5307,   /* U+144AA */
  0x5308,   /* U+144AB */
  0x5309,   /* U+144AC */
  0x530A,   /* U+144AD */
  0x530B,   /* U+144AE */
  0x530C,   /* U+144AF */
  0x530D,   /* U+144B0 */
  0x530E,   /* U+144B1 */
  0x530F,   /* U+144B2 */
  0x5310,   /* U+144B3 */
  0x5311,   /* U+144B4 */
  0x5312,   /* U+144B5 */
  0x5313,   /* U+144B6 */
  0x5314,   /* U+144B7 */
  0x5315,   /* U+144B8 */
  0x5316,   /* U+144B9 */
  0x5317,   /* U+144BA */
  0x5318,   /* U+144BB */
  0x5319,   /* U+144BC */
  0x531A,   /* U+144BD */
  0x531B,   /* U+144BE */
  0x531C,   /* U+144BF */
  0x531D,   /* U+144C0 */
  0x531E,   /* U+144C1 */
  0x531F,   /* U+144C2 */
  0x5320,   /* U+144C3 */
  0x5321,   /* U+144C4 */
  0x5322,   /* U+144C5 */
  0x5323,   /* U+144C6 */
  0x5324,   /* U+144C7 */
  0x5325,   /* U+144C8 */
  0x5326,   /* U+144C9 */
  0x5327,   /* U+144CA */
  0x5328,   /* U+144CB */
  0x5329,   /* U+144CC */
  0x532A,   /* U+144CD */
  0x532B,   /* U+144CE */
  0x532C,   /* U+144CF */
  0x532D,   /* U+144D0 */
  0x532E,   /* U+144D1 */
  0x532F,   /* U+144D2 */
  0x5330,   /* U+144D3 */
  0x5331,   /* U+144D4 */
  0x5332,   /* U+144D5 */
  0x5333,   /* U+144D6 */
  0x5334,   /* U+144D7 */
  0x5335,   /* U+144D8 */
  0x5336,   /* U+144D9 */
  0x5337,   /* U+144DA */
  0x5338,   /* U+144DB */
  0x5339,   /* U+144DC */
  0x533A,   /* U+144DD */
  0x533B,   /* U+144DE */
  0x533C,   /* U+144DF */
  0x533D,   /* U+144E0 */
  0x533E,   /* U+144E1 */
  0x533F,   /* U+144E2 */
  0x5340,   /* U+144E3 */
  0x5341,   /* U+144E4 */
  0x5342,   /* U+144E5 */
  0x5343,   /* U+144E6 */
  0x5344,   /* U+144E7 */
  0x5345,   /* U+144E8 */
  0x5346,   /* U+144E9 */
  0x5347,   /* U+144EA */
  0x5348,   /* U+144EB */
  0x5349,   /* U+144EC */
  0x534A,   /* U+144ED */
  0x534B,   /* U+144EE */
  0x534C,   /* U+144EF */
  0x534D,   /* U+144F0 */
  0x534E,   /* U+144F1 */
  0x534F,   /* U+144F2 */
  0x5350,   /* U+144F3 */
  0x5351,   /* U+144F4 */
  0x5352,   /* U+144F5 */
  0x5353,   /* U+144F6 */
  0x5354,   /* U+144F7 */
  0x5355,   /* U+144F8 */
  0x5356,   /* U+144F9 */
  0x5357,   /* U+144FA */
  0x5358,   /* U+144FB */
  0x5359,   /* U+144FC */
  0x535A,   /* U+144FD */
  0x535B,   /* U+144FE */
  0x535C,   /* U+144FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+14400 */
  0x0020,   /* U+14401 */
  0x0020,   /* U+14402 */
  0x0020,   /* U+14403 */
  0x0020,   /* U+14404 */
  0x0020,   /* U+14405 */
  0x0020,   /* U+14406 */
  0x0020,   /* U+14407 */
  0x0020,   /* U+14408 */
  0x0020,   /* U+14409 */
  0x0020,   /* U+1440A */
  0x0020,   /* U+1440B */
  0x0020,   /* U+1440C */
  0x0020,   /* U+1440D */
  0x0020,   /* U+1440E */
  0x0020,   /* U+1440F */
  0x0020,   /* U+14410 */
  0x0020,   /* U+14411 */
  0x0020,   /* U+14412 */
  0x0020,   /* U+14413 */
  0x0020,   /* U+14414 */
  0x0020,   /* U+14415 */
  0x0020,   /* U+14416 */
  0x0020,   /* U+14417 */
  0x0020,   /* U+14418 */
  0x0020,   /* U+14419 */
  0x0020,   /* U+1441A */
  0x0020,   /* U+1441B */
  0x0020,   /* U+1441C */
  0x0020,   /* U+1441D */
  0x0020,   /* U+1441E */
  0x0020,   /* U+1441F */
  0x0020,   /* U+14420 */
  0x0020,   /* U+14421 */
  0x0020,   /* U+14422 */
  0x0020,   /* U+14423 */
  0x0020,   /* U+14424 */
  0x0020,   /* U+14425 */
  0x0020,   /* U+14426 */
  0x0020,   /* U+14427 */
  0x0020,   /* U+14428 */
  0x0020,   /* U+14429 */
  0x0020,   /* U+1442A */
  0x0020,   /* U+1442B */
  0x0020,   /* U+1442C */
  0x0020,   /* U+1442D */
  0x0020,   /* U+1442E */
  0x0020,   /* U+1442F */
  0x0020,   /* U+14430 */
  0x0020,   /* U+14431 */
  0x0020,   /* U+14432 */
  0x0020,   /* U+14433 */
  0x0020,   /* U+14434 */
  0x0020,   /* U+14435 */
  0x0020,   /* U+14436 */
  0x0020,   /* U+14437 */
  0x0020,   /* U+14438 */
  0x0020,   /* U+14439 */
  0x0020,   /* U+1443A */
  0x0020,   /* U+1443B */
  0x0020,   /* U+1443C */
  0x0020,   /* U+1443D */
  0x0020,   /* U+1443E */
  0x0020,   /* U+1443F */
  0x0020,   /* U+14440 */
  0x0020,   /* U+14441 */
  0x0020,   /* U+14442 */
  0x0020,   /* U+14443 */
  0x0020,   /* U+14444 */
  0x0020,   /* U+14445 */
  0x0020,   /* U+14446 */
  0x0020,   /* U+14447 */
  0x0020,   /* U+14448 */
  0x0020,   /* U+14449 */
  0x0020,   /* U+1444A */
  0x0020,   /* U+1444B */
  0x0020,   /* U+1444C */
  0x0020,   /* U+1444D */
  0x0020,   /* U+1444E */
  0x0020,   /* U+1444F */
  0x0020,   /* U+14450 */
  0x0020,   /* U+14451 */
  0x0020,   /* U+14452 */
  0x0020,   /* U+14453 */
  0x0020,   /* U+14454 */
  0x0020,   /* U+14455 */
  0x0020,   /* U+14456 */
  0x0020,   /* U+14457 */
  0x0020,   /* U+14458 */
  0x0020,   /* U+14459 */
  0x0020,   /* U+1445A */
  0x0020,   /* U+1445B */
  0x0020,   /* U+1445C */
  0x0020,   /* U+1445D */
  0x0020,   /* U+1445E */
  0x0020,   /* U+1445F */
  0x0020,   /* U+14460 */
  0x0020,   /* U+14461 */
  0x0020,   /* U+14462 */
  0x0020,   /* U+14463 */
  0x0020,   /* U+14464 */
  0x0020,   /* U+14465 */
  0x0020,   /* U+14466 */
  0x0020,   /* U+14467 */
  0x0020,   /* U+14468 */
  0x0020,   /* U+14469 */
  0x0020,   /* U+1446A */
  0x0020,   /* U+1446B */
  0x0020,   /* U+1446C */
  0x0020,   /* U+1446D */
  0x0020,   /* U+1446E */
  0x0020,   /* U+1446F */
  0x0020,   /* U+14470 */
  0x0020,   /* U+14471 */
  0x0020,   /* U+14472 */
  0x0020,   /* U+14473 */
  0x0020,   /* U+14474 */
  0x0020,   /* U+14475 */
  0x0020,   /* U+14476 */
  0x0020,   /* U+14477 */
  0x0020,   /* U+14478 */
  0x0020,   /* U+14479 */
  0x0020,   /* U+1447A */
  0x0020,   /* U+1447B */
  0x0020,   /* U+1447C */
  0x0020,   /* U+1447D */
  0x0020,   /* U+1447E */
  0x0020,   /* U+1447F */
  0x0020,   /* U+14480 */
  0x0020,   /* U+14481 */
  0x0020,   /* U+14482 */
  0x0020,   /* U+14483 */
  0x0020,   /* U+14484 */
  0x0020,   /* U+14485 */
  0x0020,   /* U+14486 */
  0x0020,   /* U+14487 */
  0x0020,   /* U+14488 */
  0x0020,   /* U+14489 */
  0x0020,   /* U+1448A */
  0x0020,   /* U+1448B */
  0x0020,   /* U+1448C */
  0x0020,   /* U+1448D */
  0x0020,   /* U+1448E */
  0x0020,   /* U+1448F */
  0x0020,   /* U+14490 */
  0x0020,   /* U+14491 */
  0x0020,   /* U+14492 */
  0x0020,   /* U+14493 */
  0x0020,   /* U+14494 */
  0x0020,   /* U+14495 */
  0x0020,   /* U+14496 */
  0x0020,   /* U+14497 */
  0x0020,   /* U+14498 */
  0x0020,   /* U+14499 */
  0x0020,   /* U+1449A */
  0x0020,   /* U+1449B */
  0x0020,   /* U+1449C */
  0x0020,   /* U+1449D */
  0x0020,   /* U+1449E */
  0x0020,   /* U+1449F */
  0x0020,   /* U+144A0 */
  0x0020,   /* U+144A1 */
  0x0020,   /* U+144A2 */
  0x0020,   /* U+144A3 */
  0x0020,   /* U+144A4 */
  0x0020,   /* U+144A5 */
  0x0020,   /* U+144A6 */
  0x0020,   /* U+144A7 */
  0x0020,   /* U+144A8 */
  0x0020,   /* U+144A9 */
  0x0020,   /* U+144AA */
  0x0020,   /* U+144AB */
  0x0020,   /* U+144AC */
  0x0020,   /* U+144AD */
  0x0020,   /* U+144AE */
  0x0020,   /* U+144AF */
  0x0020,   /* U+144B0 */
  0x0020,   /* U+144B1 */
  0x0020,   /* U+144B2 */
  0x0020,   /* U+144B3 */
  0x0020,   /* U+144B4 */
  0x0020,   /* U+144B5 */
  0x0020,   /* U+144B6 */
  0x0020,   /* U+144B7 */
  0x0020,   /* U+144B8 */
  0x0020,   /* U+144B9 */
  0x0020,   /* U+144BA */
  0x0020,   /* U+144BB */
  0x0020,   /* U+144BC */
  0x0020,   /* U+144BD */
  0x0020,   /* U+144BE */
  0x0020,   /* U+144BF */
  0x0020,   /* U+144C0 */
  0x0020,   /* U+144C1 */
  0x0020,   /* U+144C2 */
  0x0020,   /* U+144C3 */
  0x0020,   /* U+144C4 */
  0x0020,   /* U+144C5 */
  0x0020,   /* U+144C6 */
  0x0020,   /* U+144C7 */
  0x0020,   /* U+144C8 */
  0x0020,   /* U+144C9 */
  0x0020,   /* U+144CA */
  0x0020,   /* U+144CB */
  0x0020,   /* U+144CC */
  0x0020,   /* U+144CD */
  0x0020,   /* U+144CE */
  0x0020,   /* U+144CF */
  0x0020,   /* U+144D0 */
  0x0020,   /* U+144D1 */
  0x0020,   /* U+144D2 */
  0x0020,   /* U+144D3 */
  0x0020,   /* U+144D4 */
  0x0020,   /* U+144D5 */
  0x0020,   /* U+144D6 */
  0x0020,   /* U+144D7 */
  0x0020,   /* U+144D8 */
  0x0020,   /* U+144D9 */
  0x0020,   /* U+144DA */
  0x0020,   /* U+144DB */
  0x0020,   /* U+144DC */
  0x0020,   /* U+144DD */
  0x0020,   /* U+144DE */
  0x0020,   /* U+144DF */
  0x0020,   /* U+144E0 */
  0x0020,   /* U+144E1 */
  0x0020,   /* U+144E2 */
  0x0020,   /* U+144E3 */
  0x0020,   /* U+144E4 */
  0x0020,   /* U+144E5 */
  0x0020,   /* U+144E6 */
  0x0020,   /* U+144E7 */
  0x0020,   /* U+144E8 */
  0x0020,   /* U+144E9 */
  0x0020,   /* U+144EA */
  0x0020,   /* U+144EB */
  0x0020,   /* U+144EC */
  0x0020,   /* U+144ED */
  0x0020,   /* U+144EE */
  0x0020,   /* U+144EF */
  0x0020,   /* U+144F0 */
  0x0020,   /* U+144F1 */
  0x0020,   /* U+144F2 */
  0x0020,   /* U+144F3 */
  0x0020,   /* U+144F4 */
  0x0020,   /* U+144F5 */
  0x0020,   /* U+144F6 */
  0x0020,   /* U+144F7 */
  0x0020,   /* U+144F8 */
  0x0020,   /* U+144F9 */
  0x0020,   /* U+144FA */
  0x0020,   /* U+144FB */
  0x0020,   /* U+144FC */
  0x0020,   /* U+144FD */
  0x0020,   /* U+144FE */
  0x0020,   /* U+144FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+14400 */
  0x0002,   /* U+14401 */
  0x0002,   /* U+14402 */
  0x0002,   /* U+14403 */
  0x0002,   /* U+14404 */
  0x0002,   /* U+14405 */
  0x0002,   /* U+14406 */
  0x0002,   /* U+14407 */
  0x0002,   /* U+14408 */
  0x0002,   /* U+14409 */
  0x0002,   /* U+1440A */
  0x0002,   /* U+1440B */
  0x0002,   /* U+1440C */
  0x0002,   /* U+1440D */
  0x0002,   /* U+1440E */
  0x0002,   /* U+1440F */
  0x0002,   /* U+14410 */
  0x0002,   /* U+14411 */
  0x0002,   /* U+14412 */
  0x0002,   /* U+14413 */
  0x0002,   /* U+14414 */
  0x0002,   /* U+14415 */
  0x0002,   /* U+14416 */
  0x0002,   /* U+14417 */
  0x0002,   /* U+14418 */
  0x0002,   /* U+14419 */
  0x0002,   /* U+1441A */
  0x0002,   /* U+1441B */
  0x0002,   /* U+1441C */
  0x0002,   /* U+1441D */
  0x0002,   /* U+1441E */
  0x0002,   /* U+1441F */
  0x0002,   /* U+14420 */
  0x0002,   /* U+14421 */
  0x0002,   /* U+14422 */
  0x0002,   /* U+14423 */
  0x0002,   /* U+14424 */
  0x0002,   /* U+14425 */
  0x0002,   /* U+14426 */
  0x0002,   /* U+14427 */
  0x0002,   /* U+14428 */
  0x0002,   /* U+14429 */
  0x0002,   /* U+1442A */
  0x0002,   /* U+1442B */
  0x0002,   /* U+1442C */
  0x0002,   /* U+1442D */
  0x0002,   /* U+1442E */
  0x0002,   /* U+1442F */
  0x0002,   /* U+14430 */
  0x0002,   /* U+14431 */
  0x0002,   /* U+14432 */
  0x0002,   /* U+14433 */
  0x0002,   /* U+14434 */
  0x0002,   /* U+14435 */
  0x0002,   /* U+14436 */
  0x0002,   /* U+14437 */
  0x0002,   /* U+14438 */
  0x0002,   /* U+14439 */
  0x0002,   /* U+1443A */
  0x0002,   /* U+1443B */
  0x0002,   /* U+1443C */
  0x0002,   /* U+1443D */
  0x0002,   /* U+1443E */
  0x0002,   /* U+1443F */
  0x0002,   /* U+14440 */
  0x0002,   /* U+14441 */
  0x0002,   /* U+14442 */
  0x0002,   /* U+14443 */
  0x0002,   /* U+14444 */
  0x0002,   /* U+14445 */
  0x0002,   /* U+14446 */
  0x0002,   /* U+14447 */
  0x0002,   /* U+14448 */
  0x0002,   /* U+14449 */
  0x0002,   /* U+1444A */
  0x0002,   /* U+1444B */
  0x0002,   /* U+1444C */
  0x0002,   /* U+1444D */
  0x0002,   /* U+1444E */
  0x0002,   /* U+1444F */
  0x0002,   /* U+14450 */
  0x0002,   /* U+14451 */
  0x0002,   /* U+14452 */
  0x0002,   /* U+14453 */
  0x0002,   /* U+14454 */
  0x0002,   /* U+14455 */
  0x0002,   /* U+14456 */
  0x0002,   /* U+14457 */
  0x0002,   /* U+14458 */
  0x0002,   /* U+14459 */
  0x0002,   /* U+1445A */
  0x0002,   /* U+1445B */
  0x0002,   /* U+1445C */
  0x0002,   /* U+1445D */
  0x0002,   /* U+1445E */
  0x0002,   /* U+1445F */
  0x0002,   /* U+14460 */
  0x0002,   /* U+14461 */
  0x0002,   /* U+14462 */
  0x0002,   /* U+14463 */
  0x0002,   /* U+14464 */
  0x0002,   /* U+14465 */
  0x0002,   /* U+14466 */
  0x0002,   /* U+14467 */
  0x0002,   /* U+14468 */
  0x0002,   /* U+14469 */
  0x0002,   /* U+1446A */
  0x0002,   /* U+1446B */
  0x0002,   /* U+1446C */
  0x0002,   /* U+1446D */
  0x0002,   /* U+1446E */
  0x0002,   /* U+1446F */
  0x0002,   /* U+14470 */
  0x0002,   /* U+14471 */
  0x0002,   /* U+14472 */
  0x0002,   /* U+14473 */
  0x0002,   /* U+14474 */
  0x0002,   /* U+14475 */
  0x0002,   /* U+14476 */
  0x0002,   /* U+14477 */
  0x0002,   /* U+14478 */
  0x0002,   /* U+14479 */
  0x0002,   /* U+1447A */
  0x0002,   /* U+1447B */
  0x0002,   /* U+1447C */
  0x0002,   /* U+1447D */
  0x0002,   /* U+1447E */
  0x0002,   /* U+1447F */
  0x0002,   /* U+14480 */
  0x0002,   /* U+14481 */
  0x0002,   /* U+14482 */
  0x0002,   /* U+14483 */
  0x0002,   /* U+14484 */
  0x0002,   /* U+14485 */
  0x0002,   /* U+14486 */
  0x0002,   /* U+14487 */
  0x0002,   /* U+14488 */
  0x0002,   /* U+14489 */
  0x0002,   /* U+1448A */
  0x0002,   /* U+1448B */
  0x0002,   /* U+1448C */
  0x0002,   /* U+1448D */
  0x0002,   /* U+1448E */
  0x0002,   /* U+1448F */
  0x0002,   /* U+14490 */
  0x0002,   /* U+14491 */
  0x0002,   /* U+14492 */
  0x0002,   /* U+14493 */
  0x0002,   /* U+14494 */
  0x0002,   /* U+14495 */
  0x0002,   /* U+14496 */
  0x0002,   /* U+14497 */
  0x0002,   /* U+14498 */
  0x0002,   /* U+14499 */
  0x0002,   /* U+1449A */
  0x0002,   /* U+1449B */
  0x0002,   /* U+1449C */
  0x0002,   /* U+1449D */
  0x0002,   /* U+1449E */
  0x0002,   /* U+1449F */
  0x0002,   /* U+144A0 */
  0x0002,   /* U+144A1 */
  0x0002,   /* U+144A2 */
  0x0002,   /* U+144A3 */
  0x0002,   /* U+144A4 */
  0x0002,   /* U+144A5 */
  0x0002,   /* U+144A6 */
  0x0002,   /* U+144A7 */
  0x0002,   /* U+144A8 */
  0x0002,   /* U+144A9 */
  0x0002,   /* U+144AA */
  0x0002,   /* U+144AB */
  0x0002,   /* U+144AC */
  0x0002,   /* U+144AD */
  0x0002,   /* U+144AE */
  0x0002,   /* U+144AF */
  0x0002,   /* U+144B0 */
  0x0002,   /* U+144B1 */
  0x0002,   /* U+144B2 */
  0x0002,   /* U+144B3 */
  0x0002,   /* U+144B4 */
  0x0002,   /* U+144B5 */
  0x0002,   /* U+144B6 */
  0x0002,   /* U+144B7 */
  0x0002,   /* U+144B8 */
  0x0002,   /* U+144B9 */
  0x0002,   /* U+144BA */
  0x0002,   /* U+144BB */
  0x0002,   /* U+144BC */
  0x0002,   /* U+144BD */
  0x0002,   /* U+144BE */
  0x0002,   /* U+144BF */
  0x0002,   /* U+144C0 */
  0x0002,   /* U+144C1 */
  0x0002,   /* U+144C2 */
  0x0002,   /* U+144C3 */
  0x0002,   /* U+144C4 */
  0x0002,   /* U+144C5 */
  0x0002,   /* U+144C6 */
  0x0002,   /* U+144C7 */
  0x0002,   /* U+144C8 */
  0x0002,   /* U+144C9 */
  0x0002,   /* U+144CA */
  0x0002,   /* U+144CB */
  0x0002,   /* U+144CC */
  0x0002,   /* U+144CD */
  0x0002,   /* U+144CE */
  0x0002,   /* U+144CF */
  0x0002,   /* U+144D0 */
  0x0002,   /* U+144D1 */
  0x0002,   /* U+144D2 */
  0x0002,   /* U+144D3 */
  0x0002,   /* U+144D4 */
  0x0002,   /* U+144D5 */
  0x0002,   /* U+144D6 */
  0x0002,   /* U+144D7 */
  0x0002,   /* U+144D8 */
  0x0002,   /* U+144D9 */
  0x0002,   /* U+144DA */
  0x0002,   /* U+144DB */
  0x0002,   /* U+144DC */
  0x0002,   /* U+144DD */
  0x0002,   /* U+144DE */
  0x0002,   /* U+144DF */
  0x0002,   /* U+144E0 */
  0x0002,   /* U+144E1 */
  0x0002,   /* U+144E2 */
  0x0002,   /* U+144E3 */
  0x0002,   /* U+144E4 */
  0x0002,   /* U+144E5 */
  0x0002,   /* U+144E6 */
  0x0002,   /* U+144E7 */
  0x0002,   /* U+144E8 */
  0x0002,   /* U+144E9 */
  0x0002,   /* U+144EA */
  0x0002,   /* U+144EB */
  0x0002,   /* U+144EC */
  0x0002,   /* U+144ED */
  0x0002,   /* U+144EE */
  0x0002,   /* U+144EF */
  0x0002,   /* U+144F0 */
  0x0002,   /* U+144F1 */
  0x0002,   /* U+144F2 */
  0x0002,   /* U+144F3 */
  0x0002,   /* U+144F4 */
  0x0002,   /* U+144F5 */
  0x0002,   /* U+144F6 */
  0x0002,   /* U+144F7 */
  0x0002,   /* U+144F8 */
  0x0002,   /* U+144F9 */
  0x0002,   /* U+144FA */
  0x0002,   /* U+144FB */
  0x0002,   /* U+144FC */
  0x0002,   /* U+144FD */
  0x0002,   /* U+144FE */
  0x0002,   /* U+144FF */
};

uint16 uca900_p145[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x535D,   /* U+14500 */
  0x535E,   /* U+14501 */
  0x535F,   /* U+14502 */
  0x5360,   /* U+14503 */
  0x5361,   /* U+14504 */
  0x5362,   /* U+14505 */
  0x5363,   /* U+14506 */
  0x5364,   /* U+14507 */
  0x5365,   /* U+14508 */
  0x5366,   /* U+14509 */
  0x5367,   /* U+1450A */
  0x5368,   /* U+1450B */
  0x5369,   /* U+1450C */
  0x536A,   /* U+1450D */
  0x536B,   /* U+1450E */
  0x536C,   /* U+1450F */
  0x536D,   /* U+14510 */
  0x536E,   /* U+14511 */
  0x536F,   /* U+14512 */
  0x5370,   /* U+14513 */
  0x5371,   /* U+14514 */
  0x5372,   /* U+14515 */
  0x5373,   /* U+14516 */
  0x5374,   /* U+14517 */
  0x5375,   /* U+14518 */
  0x5376,   /* U+14519 */
  0x5377,   /* U+1451A */
  0x5378,   /* U+1451B */
  0x5379,   /* U+1451C */
  0x537A,   /* U+1451D */
  0x537B,   /* U+1451E */
  0x537C,   /* U+1451F */
  0x537D,   /* U+14520 */
  0x537E,   /* U+14521 */
  0x537F,   /* U+14522 */
  0x5380,   /* U+14523 */
  0x5381,   /* U+14524 */
  0x5382,   /* U+14525 */
  0x5383,   /* U+14526 */
  0x5384,   /* U+14527 */
  0x5385,   /* U+14528 */
  0x5386,   /* U+14529 */
  0x5387,   /* U+1452A */
  0x5388,   /* U+1452B */
  0x5389,   /* U+1452C */
  0x538A,   /* U+1452D */
  0x538B,   /* U+1452E */
  0x538C,   /* U+1452F */
  0x538D,   /* U+14530 */
  0x538E,   /* U+14531 */
  0x538F,   /* U+14532 */
  0x5390,   /* U+14533 */
  0x5391,   /* U+14534 */
  0x5392,   /* U+14535 */
  0x5393,   /* U+14536 */
  0x5394,   /* U+14537 */
  0x5395,   /* U+14538 */
  0x5396,   /* U+14539 */
  0x5397,   /* U+1453A */
  0x5398,   /* U+1453B */
  0x5399,   /* U+1453C */
  0x539A,   /* U+1453D */
  0x539B,   /* U+1453E */
  0x539C,   /* U+1453F */
  0x539D,   /* U+14540 */
  0x539E,   /* U+14541 */
  0x539F,   /* U+14542 */
  0x53A0,   /* U+14543 */
  0x53A1,   /* U+14544 */
  0x53A2,   /* U+14545 */
  0x53A3,   /* U+14546 */
  0x53A4,   /* U+14547 */
  0x53A5,   /* U+14548 */
  0x53A6,   /* U+14549 */
  0x53A7,   /* U+1454A */
  0x53A8,   /* U+1454B */
  0x53A9,   /* U+1454C */
  0x53AA,   /* U+1454D */
  0x53AB,   /* U+1454E */
  0x53AC,   /* U+1454F */
  0x53AD,   /* U+14550 */
  0x53AE,   /* U+14551 */
  0x53AF,   /* U+14552 */
  0x53B0,   /* U+14553 */
  0x53B1,   /* U+14554 */
  0x53B2,   /* U+14555 */
  0x53B3,   /* U+14556 */
  0x53B4,   /* U+14557 */
  0x53B5,   /* U+14558 */
  0x53B6,   /* U+14559 */
  0x53B7,   /* U+1455A */
  0x53B8,   /* U+1455B */
  0x53B9,   /* U+1455C */
  0x53BA,   /* U+1455D */
  0x53BB,   /* U+1455E */
  0x53BC,   /* U+1455F */
  0x53BD,   /* U+14560 */
  0x53BE,   /* U+14561 */
  0x53BF,   /* U+14562 */
  0x53C0,   /* U+14563 */
  0x53C1,   /* U+14564 */
  0x53C2,   /* U+14565 */
  0x53C3,   /* U+14566 */
  0x53C4,   /* U+14567 */
  0x53C5,   /* U+14568 */
  0x53C6,   /* U+14569 */
  0x53C7,   /* U+1456A */
  0x53C8,   /* U+1456B */
  0x53C9,   /* U+1456C */
  0x53CA,   /* U+1456D */
  0x53CB,   /* U+1456E */
  0x53CC,   /* U+1456F */
  0x53CD,   /* U+14570 */
  0x53CE,   /* U+14571 */
  0x53CF,   /* U+14572 */
  0x53D0,   /* U+14573 */
  0x53D1,   /* U+14574 */
  0x53D2,   /* U+14575 */
  0x53D3,   /* U+14576 */
  0x53D4,   /* U+14577 */
  0x53D5,   /* U+14578 */
  0x53D6,   /* U+14579 */
  0x53D7,   /* U+1457A */
  0x53D8,   /* U+1457B */
  0x53D9,   /* U+1457C */
  0x53DA,   /* U+1457D */
  0x53DB,   /* U+1457E */
  0x53DC,   /* U+1457F */
  0x53DD,   /* U+14580 */
  0x53DE,   /* U+14581 */
  0x53DF,   /* U+14582 */
  0x53E0,   /* U+14583 */
  0x53E1,   /* U+14584 */
  0x53E2,   /* U+14585 */
  0x53E3,   /* U+14586 */
  0x53E4,   /* U+14587 */
  0x53E5,   /* U+14588 */
  0x53E6,   /* U+14589 */
  0x53E7,   /* U+1458A */
  0x53E8,   /* U+1458B */
  0x53E9,   /* U+1458C */
  0x53EA,   /* U+1458D */
  0x53EB,   /* U+1458E */
  0x53EC,   /* U+1458F */
  0x53ED,   /* U+14590 */
  0x53EE,   /* U+14591 */
  0x53EF,   /* U+14592 */
  0x53F0,   /* U+14593 */
  0x53F1,   /* U+14594 */
  0x53F2,   /* U+14595 */
  0x53F3,   /* U+14596 */
  0x53F4,   /* U+14597 */
  0x53F5,   /* U+14598 */
  0x53F6,   /* U+14599 */
  0x53F7,   /* U+1459A */
  0x53F8,   /* U+1459B */
  0x53F9,   /* U+1459C */
  0x53FA,   /* U+1459D */
  0x53FB,   /* U+1459E */
  0x53FC,   /* U+1459F */
  0x53FD,   /* U+145A0 */
  0x53FE,   /* U+145A1 */
  0x53FF,   /* U+145A2 */
  0x5400,   /* U+145A3 */
  0x5401,   /* U+145A4 */
  0x5402,   /* U+145A5 */
  0x5403,   /* U+145A6 */
  0x5404,   /* U+145A7 */
  0x5405,   /* U+145A8 */
  0x5406,   /* U+145A9 */
  0x5407,   /* U+145AA */
  0x5408,   /* U+145AB */
  0x5409,   /* U+145AC */
  0x540A,   /* U+145AD */
  0x540B,   /* U+145AE */
  0x540C,   /* U+145AF */
  0x540D,   /* U+145B0 */
  0x540E,   /* U+145B1 */
  0x540F,   /* U+145B2 */
  0x5410,   /* U+145B3 */
  0x5411,   /* U+145B4 */
  0x5412,   /* U+145B5 */
  0x5413,   /* U+145B6 */
  0x5414,   /* U+145B7 */
  0x5415,   /* U+145B8 */
  0x5416,   /* U+145B9 */
  0x5417,   /* U+145BA */
  0x5418,   /* U+145BB */
  0x5419,   /* U+145BC */
  0x541A,   /* U+145BD */
  0x541B,   /* U+145BE */
  0x541C,   /* U+145BF */
  0x541D,   /* U+145C0 */
  0x541E,   /* U+145C1 */
  0x541F,   /* U+145C2 */
  0x5420,   /* U+145C3 */
  0x5421,   /* U+145C4 */
  0x5422,   /* U+145C5 */
  0x5423,   /* U+145C6 */
  0x5424,   /* U+145C7 */
  0x5425,   /* U+145C8 */
  0x5426,   /* U+145C9 */
  0x5427,   /* U+145CA */
  0x5428,   /* U+145CB */
  0x5429,   /* U+145CC */
  0x542A,   /* U+145CD */
  0x542B,   /* U+145CE */
  0x542C,   /* U+145CF */
  0x542D,   /* U+145D0 */
  0x542E,   /* U+145D1 */
  0x542F,   /* U+145D2 */
  0x5430,   /* U+145D3 */
  0x5431,   /* U+145D4 */
  0x5432,   /* U+145D5 */
  0x5433,   /* U+145D6 */
  0x5434,   /* U+145D7 */
  0x5435,   /* U+145D8 */
  0x5436,   /* U+145D9 */
  0x5437,   /* U+145DA */
  0x5438,   /* U+145DB */
  0x5439,   /* U+145DC */
  0x543A,   /* U+145DD */
  0x543B,   /* U+145DE */
  0x543C,   /* U+145DF */
  0x543D,   /* U+145E0 */
  0x543E,   /* U+145E1 */
  0x543F,   /* U+145E2 */
  0x5440,   /* U+145E3 */
  0x5441,   /* U+145E4 */
  0x5442,   /* U+145E5 */
  0x5443,   /* U+145E6 */
  0x5444,   /* U+145E7 */
  0x5445,   /* U+145E8 */
  0x5446,   /* U+145E9 */
  0x5447,   /* U+145EA */
  0x5448,   /* U+145EB */
  0x5449,   /* U+145EC */
  0x544A,   /* U+145ED */
  0x544B,   /* U+145EE */
  0x544C,   /* U+145EF */
  0x544D,   /* U+145F0 */
  0x544E,   /* U+145F1 */
  0x544F,   /* U+145F2 */
  0x5450,   /* U+145F3 */
  0x5451,   /* U+145F4 */
  0x5452,   /* U+145F5 */
  0x5453,   /* U+145F6 */
  0x5454,   /* U+145F7 */
  0x5455,   /* U+145F8 */
  0x5456,   /* U+145F9 */
  0x5457,   /* U+145FA */
  0x5458,   /* U+145FB */
  0x5459,   /* U+145FC */
  0x545A,   /* U+145FD */
  0x545B,   /* U+145FE */
  0x545C,   /* U+145FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+14500 */
  0x0020,   /* U+14501 */
  0x0020,   /* U+14502 */
  0x0020,   /* U+14503 */
  0x0020,   /* U+14504 */
  0x0020,   /* U+14505 */
  0x0020,   /* U+14506 */
  0x0020,   /* U+14507 */
  0x0020,   /* U+14508 */
  0x0020,   /* U+14509 */
  0x0020,   /* U+1450A */
  0x0020,   /* U+1450B */
  0x0020,   /* U+1450C */
  0x0020,   /* U+1450D */
  0x0020,   /* U+1450E */
  0x0020,   /* U+1450F */
  0x0020,   /* U+14510 */
  0x0020,   /* U+14511 */
  0x0020,   /* U+14512 */
  0x0020,   /* U+14513 */
  0x0020,   /* U+14514 */
  0x0020,   /* U+14515 */
  0x0020,   /* U+14516 */
  0x0020,   /* U+14517 */
  0x0020,   /* U+14518 */
  0x0020,   /* U+14519 */
  0x0020,   /* U+1451A */
  0x0020,   /* U+1451B */
  0x0020,   /* U+1451C */
  0x0020,   /* U+1451D */
  0x0020,   /* U+1451E */
  0x0020,   /* U+1451F */
  0x0020,   /* U+14520 */
  0x0020,   /* U+14521 */
  0x0020,   /* U+14522 */
  0x0020,   /* U+14523 */
  0x0020,   /* U+14524 */
  0x0020,   /* U+14525 */
  0x0020,   /* U+14526 */
  0x0020,   /* U+14527 */
  0x0020,   /* U+14528 */
  0x0020,   /* U+14529 */
  0x0020,   /* U+1452A */
  0x0020,   /* U+1452B */
  0x0020,   /* U+1452C */
  0x0020,   /* U+1452D */
  0x0020,   /* U+1452E */
  0x0020,   /* U+1452F */
  0x0020,   /* U+14530 */
  0x0020,   /* U+14531 */
  0x0020,   /* U+14532 */
  0x0020,   /* U+14533 */
  0x0020,   /* U+14534 */
  0x0020,   /* U+14535 */
  0x0020,   /* U+14536 */
  0x0020,   /* U+14537 */
  0x0020,   /* U+14538 */
  0x0020,   /* U+14539 */
  0x0020,   /* U+1453A */
  0x0020,   /* U+1453B */
  0x0020,   /* U+1453C */
  0x0020,   /* U+1453D */
  0x0020,   /* U+1453E */
  0x0020,   /* U+1453F */
  0x0020,   /* U+14540 */
  0x0020,   /* U+14541 */
  0x0020,   /* U+14542 */
  0x0020,   /* U+14543 */
  0x0020,   /* U+14544 */
  0x0020,   /* U+14545 */
  0x0020,   /* U+14546 */
  0x0020,   /* U+14547 */
  0x0020,   /* U+14548 */
  0x0020,   /* U+14549 */
  0x0020,   /* U+1454A */
  0x0020,   /* U+1454B */
  0x0020,   /* U+1454C */
  0x0020,   /* U+1454D */
  0x0020,   /* U+1454E */
  0x0020,   /* U+1454F */
  0x0020,   /* U+14550 */
  0x0020,   /* U+14551 */
  0x0020,   /* U+14552 */
  0x0020,   /* U+14553 */
  0x0020,   /* U+14554 */
  0x0020,   /* U+14555 */
  0x0020,   /* U+14556 */
  0x0020,   /* U+14557 */
  0x0020,   /* U+14558 */
  0x0020,   /* U+14559 */
  0x0020,   /* U+1455A */
  0x0020,   /* U+1455B */
  0x0020,   /* U+1455C */
  0x0020,   /* U+1455D */
  0x0020,   /* U+1455E */
  0x0020,   /* U+1455F */
  0x0020,   /* U+14560 */
  0x0020,   /* U+14561 */
  0x0020,   /* U+14562 */
  0x0020,   /* U+14563 */
  0x0020,   /* U+14564 */
  0x0020,   /* U+14565 */
  0x0020,   /* U+14566 */
  0x0020,   /* U+14567 */
  0x0020,   /* U+14568 */
  0x0020,   /* U+14569 */
  0x0020,   /* U+1456A */
  0x0020,   /* U+1456B */
  0x0020,   /* U+1456C */
  0x0020,   /* U+1456D */
  0x0020,   /* U+1456E */
  0x0020,   /* U+1456F */
  0x0020,   /* U+14570 */
  0x0020,   /* U+14571 */
  0x0020,   /* U+14572 */
  0x0020,   /* U+14573 */
  0x0020,   /* U+14574 */
  0x0020,   /* U+14575 */
  0x0020,   /* U+14576 */
  0x0020,   /* U+14577 */
  0x0020,   /* U+14578 */
  0x0020,   /* U+14579 */
  0x0020,   /* U+1457A */
  0x0020,   /* U+1457B */
  0x0020,   /* U+1457C */
  0x0020,   /* U+1457D */
  0x0020,   /* U+1457E */
  0x0020,   /* U+1457F */
  0x0020,   /* U+14580 */
  0x0020,   /* U+14581 */
  0x0020,   /* U+14582 */
  0x0020,   /* U+14583 */
  0x0020,   /* U+14584 */
  0x0020,   /* U+14585 */
  0x0020,   /* U+14586 */
  0x0020,   /* U+14587 */
  0x0020,   /* U+14588 */
  0x0020,   /* U+14589 */
  0x0020,   /* U+1458A */
  0x0020,   /* U+1458B */
  0x0020,   /* U+1458C */
  0x0020,   /* U+1458D */
  0x0020,   /* U+1458E */
  0x0020,   /* U+1458F */
  0x0020,   /* U+14590 */
  0x0020,   /* U+14591 */
  0x0020,   /* U+14592 */
  0x0020,   /* U+14593 */
  0x0020,   /* U+14594 */
  0x0020,   /* U+14595 */
  0x0020,   /* U+14596 */
  0x0020,   /* U+14597 */
  0x0020,   /* U+14598 */
  0x0020,   /* U+14599 */
  0x0020,   /* U+1459A */
  0x0020,   /* U+1459B */
  0x0020,   /* U+1459C */
  0x0020,   /* U+1459D */
  0x0020,   /* U+1459E */
  0x0020,   /* U+1459F */
  0x0020,   /* U+145A0 */
  0x0020,   /* U+145A1 */
  0x0020,   /* U+145A2 */
  0x0020,   /* U+145A3 */
  0x0020,   /* U+145A4 */
  0x0020,   /* U+145A5 */
  0x0020,   /* U+145A6 */
  0x0020,   /* U+145A7 */
  0x0020,   /* U+145A8 */
  0x0020,   /* U+145A9 */
  0x0020,   /* U+145AA */
  0x0020,   /* U+145AB */
  0x0020,   /* U+145AC */
  0x0020,   /* U+145AD */
  0x0020,   /* U+145AE */
  0x0020,   /* U+145AF */
  0x0020,   /* U+145B0 */
  0x0020,   /* U+145B1 */
  0x0020,   /* U+145B2 */
  0x0020,   /* U+145B3 */
  0x0020,   /* U+145B4 */
  0x0020,   /* U+145B5 */
  0x0020,   /* U+145B6 */
  0x0020,   /* U+145B7 */
  0x0020,   /* U+145B8 */
  0x0020,   /* U+145B9 */
  0x0020,   /* U+145BA */
  0x0020,   /* U+145BB */
  0x0020,   /* U+145BC */
  0x0020,   /* U+145BD */
  0x0020,   /* U+145BE */
  0x0020,   /* U+145BF */
  0x0020,   /* U+145C0 */
  0x0020,   /* U+145C1 */
  0x0020,   /* U+145C2 */
  0x0020,   /* U+145C3 */
  0x0020,   /* U+145C4 */
  0x0020,   /* U+145C5 */
  0x0020,   /* U+145C6 */
  0x0020,   /* U+145C7 */
  0x0020,   /* U+145C8 */
  0x0020,   /* U+145C9 */
  0x0020,   /* U+145CA */
  0x0020,   /* U+145CB */
  0x0020,   /* U+145CC */
  0x0020,   /* U+145CD */
  0x0020,   /* U+145CE */
  0x0020,   /* U+145CF */
  0x0020,   /* U+145D0 */
  0x0020,   /* U+145D1 */
  0x0020,   /* U+145D2 */
  0x0020,   /* U+145D3 */
  0x0020,   /* U+145D4 */
  0x0020,   /* U+145D5 */
  0x0020,   /* U+145D6 */
  0x0020,   /* U+145D7 */
  0x0020,   /* U+145D8 */
  0x0020,   /* U+145D9 */
  0x0020,   /* U+145DA */
  0x0020,   /* U+145DB */
  0x0020,   /* U+145DC */
  0x0020,   /* U+145DD */
  0x0020,   /* U+145DE */
  0x0020,   /* U+145DF */
  0x0020,   /* U+145E0 */
  0x0020,   /* U+145E1 */
  0x0020,   /* U+145E2 */
  0x0020,   /* U+145E3 */
  0x0020,   /* U+145E4 */
  0x0020,   /* U+145E5 */
  0x0020,   /* U+145E6 */
  0x0020,   /* U+145E7 */
  0x0020,   /* U+145E8 */
  0x0020,   /* U+145E9 */
  0x0020,   /* U+145EA */
  0x0020,   /* U+145EB */
  0x0020,   /* U+145EC */
  0x0020,   /* U+145ED */
  0x0020,   /* U+145EE */
  0x0020,   /* U+145EF */
  0x0020,   /* U+145F0 */
  0x0020,   /* U+145F1 */
  0x0020,   /* U+145F2 */
  0x0020,   /* U+145F3 */
  0x0020,   /* U+145F4 */
  0x0020,   /* U+145F5 */
  0x0020,   /* U+145F6 */
  0x0020,   /* U+145F7 */
  0x0020,   /* U+145F8 */
  0x0020,   /* U+145F9 */
  0x0020,   /* U+145FA */
  0x0020,   /* U+145FB */
  0x0020,   /* U+145FC */
  0x0020,   /* U+145FD */
  0x0020,   /* U+145FE */
  0x0020,   /* U+145FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+14500 */
  0x0002,   /* U+14501 */
  0x0002,   /* U+14502 */
  0x0002,   /* U+14503 */
  0x0002,   /* U+14504 */
  0x0002,   /* U+14505 */
  0x0002,   /* U+14506 */
  0x0002,   /* U+14507 */
  0x0002,   /* U+14508 */
  0x0002,   /* U+14509 */
  0x0002,   /* U+1450A */
  0x0002,   /* U+1450B */
  0x0002,   /* U+1450C */
  0x0002,   /* U+1450D */
  0x0002,   /* U+1450E */
  0x0002,   /* U+1450F */
  0x0002,   /* U+14510 */
  0x0002,   /* U+14511 */
  0x0002,   /* U+14512 */
  0x0002,   /* U+14513 */
  0x0002,   /* U+14514 */
  0x0002,   /* U+14515 */
  0x0002,   /* U+14516 */
  0x0002,   /* U+14517 */
  0x0002,   /* U+14518 */
  0x0002,   /* U+14519 */
  0x0002,   /* U+1451A */
  0x0002,   /* U+1451B */
  0x0002,   /* U+1451C */
  0x0002,   /* U+1451D */
  0x0002,   /* U+1451E */
  0x0002,   /* U+1451F */
  0x0002,   /* U+14520 */
  0x0002,   /* U+14521 */
  0x0002,   /* U+14522 */
  0x0002,   /* U+14523 */
  0x0002,   /* U+14524 */
  0x0002,   /* U+14525 */
  0x0002,   /* U+14526 */
  0x0002,   /* U+14527 */
  0x0002,   /* U+14528 */
  0x0002,   /* U+14529 */
  0x0002,   /* U+1452A */
  0x0002,   /* U+1452B */
  0x0002,   /* U+1452C */
  0x0002,   /* U+1452D */
  0x0002,   /* U+1452E */
  0x0002,   /* U+1452F */
  0x0002,   /* U+14530 */
  0x0002,   /* U+14531 */
  0x0002,   /* U+14532 */
  0x0002,   /* U+14533 */
  0x0002,   /* U+14534 */
  0x0002,   /* U+14535 */
  0x0002,   /* U+14536 */
  0x0002,   /* U+14537 */
  0x0002,   /* U+14538 */
  0x0002,   /* U+14539 */
  0x0002,   /* U+1453A */
  0x0002,   /* U+1453B */
  0x0002,   /* U+1453C */
  0x0002,   /* U+1453D */
  0x0002,   /* U+1453E */
  0x0002,   /* U+1453F */
  0x0002,   /* U+14540 */
  0x0002,   /* U+14541 */
  0x0002,   /* U+14542 */
  0x0002,   /* U+14543 */
  0x0002,   /* U+14544 */
  0x0002,   /* U+14545 */
  0x0002,   /* U+14546 */
  0x0002,   /* U+14547 */
  0x0002,   /* U+14548 */
  0x0002,   /* U+14549 */
  0x0002,   /* U+1454A */
  0x0002,   /* U+1454B */
  0x0002,   /* U+1454C */
  0x0002,   /* U+1454D */
  0x0002,   /* U+1454E */
  0x0002,   /* U+1454F */
  0x0002,   /* U+14550 */
  0x0002,   /* U+14551 */
  0x0002,   /* U+14552 */
  0x0002,   /* U+14553 */
  0x0002,   /* U+14554 */
  0x0002,   /* U+14555 */
  0x0002,   /* U+14556 */
  0x0002,   /* U+14557 */
  0x0002,   /* U+14558 */
  0x0002,   /* U+14559 */
  0x0002,   /* U+1455A */
  0x0002,   /* U+1455B */
  0x0002,   /* U+1455C */
  0x0002,   /* U+1455D */
  0x0002,   /* U+1455E */
  0x0002,   /* U+1455F */
  0x0002,   /* U+14560 */
  0x0002,   /* U+14561 */
  0x0002,   /* U+14562 */
  0x0002,   /* U+14563 */
  0x0002,   /* U+14564 */
  0x0002,   /* U+14565 */
  0x0002,   /* U+14566 */
  0x0002,   /* U+14567 */
  0x0002,   /* U+14568 */
  0x0002,   /* U+14569 */
  0x0002,   /* U+1456A */
  0x0002,   /* U+1456B */
  0x0002,   /* U+1456C */
  0x0002,   /* U+1456D */
  0x0002,   /* U+1456E */
  0x0002,   /* U+1456F */
  0x0002,   /* U+14570 */
  0x0002,   /* U+14571 */
  0x0002,   /* U+14572 */
  0x0002,   /* U+14573 */
  0x0002,   /* U+14574 */
  0x0002,   /* U+14575 */
  0x0002,   /* U+14576 */
  0x0002,   /* U+14577 */
  0x0002,   /* U+14578 */
  0x0002,   /* U+14579 */
  0x0002,   /* U+1457A */
  0x0002,   /* U+1457B */
  0x0002,   /* U+1457C */
  0x0002,   /* U+1457D */
  0x0002,   /* U+1457E */
  0x0002,   /* U+1457F */
  0x0002,   /* U+14580 */
  0x0002,   /* U+14581 */
  0x0002,   /* U+14582 */
  0x0002,   /* U+14583 */
  0x0002,   /* U+14584 */
  0x0002,   /* U+14585 */
  0x0002,   /* U+14586 */
  0x0002,   /* U+14587 */
  0x0002,   /* U+14588 */
  0x0002,   /* U+14589 */
  0x0002,   /* U+1458A */
  0x0002,   /* U+1458B */
  0x0002,   /* U+1458C */
  0x0002,   /* U+1458D */
  0x0002,   /* U+1458E */
  0x0002,   /* U+1458F */
  0x0002,   /* U+14590 */
  0x0002,   /* U+14591 */
  0x0002,   /* U+14592 */
  0x0002,   /* U+14593 */
  0x0002,   /* U+14594 */
  0x0002,   /* U+14595 */
  0x0002,   /* U+14596 */
  0x0002,   /* U+14597 */
  0x0002,   /* U+14598 */
  0x0002,   /* U+14599 */
  0x0002,   /* U+1459A */
  0x0002,   /* U+1459B */
  0x0002,   /* U+1459C */
  0x0002,   /* U+1459D */
  0x0002,   /* U+1459E */
  0x0002,   /* U+1459F */
  0x0002,   /* U+145A0 */
  0x0002,   /* U+145A1 */
  0x0002,   /* U+145A2 */
  0x0002,   /* U+145A3 */
  0x0002,   /* U+145A4 */
  0x0002,   /* U+145A5 */
  0x0002,   /* U+145A6 */
  0x0002,   /* U+145A7 */
  0x0002,   /* U+145A8 */
  0x0002,   /* U+145A9 */
  0x0002,   /* U+145AA */
  0x0002,   /* U+145AB */
  0x0002,   /* U+145AC */
  0x0002,   /* U+145AD */
  0x0002,   /* U+145AE */
  0x0002,   /* U+145AF */
  0x0002,   /* U+145B0 */
  0x0002,   /* U+145B1 */
  0x0002,   /* U+145B2 */
  0x0002,   /* U+145B3 */
  0x0002,   /* U+145B4 */
  0x0002,   /* U+145B5 */
  0x0002,   /* U+145B6 */
  0x0002,   /* U+145B7 */
  0x0002,   /* U+145B8 */
  0x0002,   /* U+145B9 */
  0x0002,   /* U+145BA */
  0x0002,   /* U+145BB */
  0x0002,   /* U+145BC */
  0x0002,   /* U+145BD */
  0x0002,   /* U+145BE */
  0x0002,   /* U+145BF */
  0x0002,   /* U+145C0 */
  0x0002,   /* U+145C1 */
  0x0002,   /* U+145C2 */
  0x0002,   /* U+145C3 */
  0x0002,   /* U+145C4 */
  0x0002,   /* U+145C5 */
  0x0002,   /* U+145C6 */
  0x0002,   /* U+145C7 */
  0x0002,   /* U+145C8 */
  0x0002,   /* U+145C9 */
  0x0002,   /* U+145CA */
  0x0002,   /* U+145CB */
  0x0002,   /* U+145CC */
  0x0002,   /* U+145CD */
  0x0002,   /* U+145CE */
  0x0002,   /* U+145CF */
  0x0002,   /* U+145D0 */
  0x0002,   /* U+145D1 */
  0x0002,   /* U+145D2 */
  0x0002,   /* U+145D3 */
  0x0002,   /* U+145D4 */
  0x0002,   /* U+145D5 */
  0x0002,   /* U+145D6 */
  0x0002,   /* U+145D7 */
  0x0002,   /* U+145D8 */
  0x0002,   /* U+145D9 */
  0x0002,   /* U+145DA */
  0x0002,   /* U+145DB */
  0x0002,   /* U+145DC */
  0x0002,   /* U+145DD */
  0x0002,   /* U+145DE */
  0x0002,   /* U+145DF */
  0x0002,   /* U+145E0 */
  0x0002,   /* U+145E1 */
  0x0002,   /* U+145E2 */
  0x0002,   /* U+145E3 */
  0x0002,   /* U+145E4 */
  0x0002,   /* U+145E5 */
  0x0002,   /* U+145E6 */
  0x0002,   /* U+145E7 */
  0x0002,   /* U+145E8 */
  0x0002,   /* U+145E9 */
  0x0002,   /* U+145EA */
  0x0002,   /* U+145EB */
  0x0002,   /* U+145EC */
  0x0002,   /* U+145ED */
  0x0002,   /* U+145EE */
  0x0002,   /* U+145EF */
  0x0002,   /* U+145F0 */
  0x0002,   /* U+145F1 */
  0x0002,   /* U+145F2 */
  0x0002,   /* U+145F3 */
  0x0002,   /* U+145F4 */
  0x0002,   /* U+145F5 */
  0x0002,   /* U+145F6 */
  0x0002,   /* U+145F7 */
  0x0002,   /* U+145F8 */
  0x0002,   /* U+145F9 */
  0x0002,   /* U+145FA */
  0x0002,   /* U+145FB */
  0x0002,   /* U+145FC */
  0x0002,   /* U+145FD */
  0x0002,   /* U+145FE */
  0x0002,   /* U+145FF */
};

uint16 uca900_p146[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x545D,   /* U+14600 */
  0x545E,   /* U+14601 */
  0x545F,   /* U+14602 */
  0x5460,   /* U+14603 */
  0x5461,   /* U+14604 */
  0x5462,   /* U+14605 */
  0x5463,   /* U+14606 */
  0x5464,   /* U+14607 */
  0x5465,   /* U+14608 */
  0x5466,   /* U+14609 */
  0x5467,   /* U+1460A */
  0x5468,   /* U+1460B */
  0x5469,   /* U+1460C */
  0x546A,   /* U+1460D */
  0x546B,   /* U+1460E */
  0x546C,   /* U+1460F */
  0x546D,   /* U+14610 */
  0x546E,   /* U+14611 */
  0x546F,   /* U+14612 */
  0x5470,   /* U+14613 */
  0x5471,   /* U+14614 */
  0x5472,   /* U+14615 */
  0x5473,   /* U+14616 */
  0x5474,   /* U+14617 */
  0x5475,   /* U+14618 */
  0x5476,   /* U+14619 */
  0x5477,   /* U+1461A */
  0x5478,   /* U+1461B */
  0x5479,   /* U+1461C */
  0x547A,   /* U+1461D */
  0x547B,   /* U+1461E */
  0x547C,   /* U+1461F */
  0x547D,   /* U+14620 */
  0x547E,   /* U+14621 */
  0x547F,   /* U+14622 */
  0x5480,   /* U+14623 */
  0x5481,   /* U+14624 */
  0x5482,   /* U+14625 */
  0x5483,   /* U+14626 */
  0x5484,   /* U+14627 */
  0x5485,   /* U+14628 */
  0x5486,   /* U+14629 */
  0x5487,   /* U+1462A */
  0x5488,   /* U+1462B */
  0x5489,   /* U+1462C */
  0x548A,   /* U+1462D */
  0x548B,   /* U+1462E */
  0x548C,   /* U+1462F */
  0x548D,   /* U+14630 */
  0x548E,   /* U+14631 */
  0x548F,   /* U+14632 */
  0x5490,   /* U+14633 */
  0x5491,   /* U+14634 */
  0x5492,   /* U+14635 */
  0x5493,   /* U+14636 */
  0x5494,   /* U+14637 */
  0x5495,   /* U+14638 */
  0x5496,   /* U+14639 */
  0x5497,   /* U+1463A */
  0x5498,   /* U+1463B */
  0x5499,   /* U+1463C */
  0x549A,   /* U+1463D */
  0x549B,   /* U+1463E */
  0x549C,   /* U+1463F */
  0x549D,   /* U+14640 */
  0x549E,   /* U+14641 */
  0x549F,   /* U+14642 */
  0x54A0,   /* U+14643 */
  0x54A1,   /* U+14644 */
  0x54A2,   /* U+14645 */
  0x54A3,   /* U+14646 */
  0xFBC2,   /* U+14647 */
  0xFBC2,   /* U+14648 */
  0xFBC2,   /* U+14649 */
  0xFBC2,   /* U+1464A */
  0xFBC2,   /* U+1464B */
  0xFBC2,   /* U+1464C */
  0xFBC2,   /* U+1464D */
  0xFBC2,   /* U+1464E */
  0xFBC2,   /* U+1464F */
  0xFBC2,   /* U+14650 */
  0xFBC2,   /* U+14651 */
  0xFBC2,   /* U+14652 */
  0xFBC2,   /* U+14653 */
  0xFBC2,   /* U+14654 */
  0xFBC2,   /* U+14655 */
  0xFBC2,   /* U+14656 */
  0xFBC2,   /* U+14657 */
  0xFBC2,   /* U+14658 */
  0xFBC2,   /* U+14659 */
  0xFBC2,   /* U+1465A */
  0xFBC2,   /* U+1465B */
  0xFBC2,   /* U+1465C */
  0xFBC2,   /* U+1465D */
  0xFBC2,   /* U+1465E */
  0xFBC2,   /* U+1465F */
  0xFBC2,   /* U+14660 */
  0xFBC2,   /* U+14661 */
  0xFBC2,   /* U+14662 */
  0xFBC2,   /* U+14663 */
  0xFBC2,   /* U+14664 */
  0xFBC2,   /* U+14665 */
  0xFBC2,   /* U+14666 */
  0xFBC2,   /* U+14667 */
  0xFBC2,   /* U+14668 */
  0xFBC2,   /* U+14669 */
  0xFBC2,   /* U+1466A */
  0xFBC2,   /* U+1466B */
  0xFBC2,   /* U+1466C */
  0xFBC2,   /* U+1466D */
  0xFBC2,   /* U+1466E */
  0xFBC2,   /* U+1466F */
  0xFBC2,   /* U+14670 */
  0xFBC2,   /* U+14671 */
  0xFBC2,   /* U+14672 */
  0xFBC2,   /* U+14673 */
  0xFBC2,   /* U+14674 */
  0xFBC2,   /* U+14675 */
  0xFBC2,   /* U+14676 */
  0xFBC2,   /* U+14677 */
  0xFBC2,   /* U+14678 */
  0xFBC2,   /* U+14679 */
  0xFBC2,   /* U+1467A */
  0xFBC2,   /* U+1467B */
  0xFBC2,   /* U+1467C */
  0xFBC2,   /* U+1467D */
  0xFBC2,   /* U+1467E */
  0xFBC2,   /* U+1467F */
  0xFBC2,   /* U+14680 */
  0xFBC2,   /* U+14681 */
  0xFBC2,   /* U+14682 */
  0xFBC2,   /* U+14683 */
  0xFBC2,   /* U+14684 */
  0xFBC2,   /* U+14685 */
  0xFBC2,   /* U+14686 */
  0xFBC2,   /* U+14687 */
  0xFBC2,   /* U+14688 */
  0xFBC2,   /* U+14689 */
  0xFBC2,   /* U+1468A */
  0xFBC2,   /* U+1468B */
  0xFBC2,   /* U+1468C */
  0xFBC2,   /* U+1468D */
  0xFBC2,   /* U+1468E */
  0xFBC2,   /* U+1468F */
  0xFBC2,   /* U+14690 */
  0xFBC2,   /* U+14691 */
  0xFBC2,   /* U+14692 */
  0xFBC2,   /* U+14693 */
  0xFBC2,   /* U+14694 */
  0xFBC2,   /* U+14695 */
  0xFBC2,   /* U+14696 */
  0xFBC2,   /* U+14697 */
  0xFBC2,   /* U+14698 */
  0xFBC2,   /* U+14699 */
  0xFBC2,   /* U+1469A */
  0xFBC2,   /* U+1469B */
  0xFBC2,   /* U+1469C */
  0xFBC2,   /* U+1469D */
  0xFBC2,   /* U+1469E */
  0xFBC2,   /* U+1469F */
  0xFBC2,   /* U+146A0 */
  0xFBC2,   /* U+146A1 */
  0xFBC2,   /* U+146A2 */
  0xFBC2,   /* U+146A3 */
  0xFBC2,   /* U+146A4 */
  0xFBC2,   /* U+146A5 */
  0xFBC2,   /* U+146A6 */
  0xFBC2,   /* U+146A7 */
  0xFBC2,   /* U+146A8 */
  0xFBC2,   /* U+146A9 */
  0xFBC2,   /* U+146AA */
  0xFBC2,   /* U+146AB */
  0xFBC2,   /* U+146AC */
  0xFBC2,   /* U+146AD */
  0xFBC2,   /* U+146AE */
  0xFBC2,   /* U+146AF */
  0xFBC2,   /* U+146B0 */
  0xFBC2,   /* U+146B1 */
  0xFBC2,   /* U+146B2 */
  0xFBC2,   /* U+146B3 */
  0xFBC2,   /* U+146B4 */
  0xFBC2,   /* U+146B5 */
  0xFBC2,   /* U+146B6 */
  0xFBC2,   /* U+146B7 */
  0xFBC2,   /* U+146B8 */
  0xFBC2,   /* U+146B9 */
  0xFBC2,   /* U+146BA */
  0xFBC2,   /* U+146BB */
  0xFBC2,   /* U+146BC */
  0xFBC2,   /* U+146BD */
  0xFBC2,   /* U+146BE */
  0xFBC2,   /* U+146BF */
  0xFBC2,   /* U+146C0 */
  0xFBC2,   /* U+146C1 */
  0xFBC2,   /* U+146C2 */
  0xFBC2,   /* U+146C3 */
  0xFBC2,   /* U+146C4 */
  0xFBC2,   /* U+146C5 */
  0xFBC2,   /* U+146C6 */
  0xFBC2,   /* U+146C7 */
  0xFBC2,   /* U+146C8 */
  0xFBC2,   /* U+146C9 */
  0xFBC2,   /* U+146CA */
  0xFBC2,   /* U+146CB */
  0xFBC2,   /* U+146CC */
  0xFBC2,   /* U+146CD */
  0xFBC2,   /* U+146CE */
  0xFBC2,   /* U+146CF */
  0xFBC2,   /* U+146D0 */
  0xFBC2,   /* U+146D1 */
  0xFBC2,   /* U+146D2 */
  0xFBC2,   /* U+146D3 */
  0xFBC2,   /* U+146D4 */
  0xFBC2,   /* U+146D5 */
  0xFBC2,   /* U+146D6 */
  0xFBC2,   /* U+146D7 */
  0xFBC2,   /* U+146D8 */
  0xFBC2,   /* U+146D9 */
  0xFBC2,   /* U+146DA */
  0xFBC2,   /* U+146DB */
  0xFBC2,   /* U+146DC */
  0xFBC2,   /* U+146DD */
  0xFBC2,   /* U+146DE */
  0xFBC2,   /* U+146DF */
  0xFBC2,   /* U+146E0 */
  0xFBC2,   /* U+146E1 */
  0xFBC2,   /* U+146E2 */
  0xFBC2,   /* U+146E3 */
  0xFBC2,   /* U+146E4 */
  0xFBC2,   /* U+146E5 */
  0xFBC2,   /* U+146E6 */
  0xFBC2,   /* U+146E7 */
  0xFBC2,   /* U+146E8 */
  0xFBC2,   /* U+146E9 */
  0xFBC2,   /* U+146EA */
  0xFBC2,   /* U+146EB */
  0xFBC2,   /* U+146EC */
  0xFBC2,   /* U+146ED */
  0xFBC2,   /* U+146EE */
  0xFBC2,   /* U+146EF */
  0xFBC2,   /* U+146F0 */
  0xFBC2,   /* U+146F1 */
  0xFBC2,   /* U+146F2 */
  0xFBC2,   /* U+146F3 */
  0xFBC2,   /* U+146F4 */
  0xFBC2,   /* U+146F5 */
  0xFBC2,   /* U+146F6 */
  0xFBC2,   /* U+146F7 */
  0xFBC2,   /* U+146F8 */
  0xFBC2,   /* U+146F9 */
  0xFBC2,   /* U+146FA */
  0xFBC2,   /* U+146FB */
  0xFBC2,   /* U+146FC */
  0xFBC2,   /* U+146FD */
  0xFBC2,   /* U+146FE */
  0xFBC2,   /* U+146FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+14600 */
  0x0020,   /* U+14601 */
  0x0020,   /* U+14602 */
  0x0020,   /* U+14603 */
  0x0020,   /* U+14604 */
  0x0020,   /* U+14605 */
  0x0020,   /* U+14606 */
  0x0020,   /* U+14607 */
  0x0020,   /* U+14608 */
  0x0020,   /* U+14609 */
  0x0020,   /* U+1460A */
  0x0020,   /* U+1460B */
  0x0020,   /* U+1460C */
  0x0020,   /* U+1460D */
  0x0020,   /* U+1460E */
  0x0020,   /* U+1460F */
  0x0020,   /* U+14610 */
  0x0020,   /* U+14611 */
  0x0020,   /* U+14612 */
  0x0020,   /* U+14613 */
  0x0020,   /* U+14614 */
  0x0020,   /* U+14615 */
  0x0020,   /* U+14616 */
  0x0020,   /* U+14617 */
  0x0020,   /* U+14618 */
  0x0020,   /* U+14619 */
  0x0020,   /* U+1461A */
  0x0020,   /* U+1461B */
  0x0020,   /* U+1461C */
  0x0020,   /* U+1461D */
  0x0020,   /* U+1461E */
  0x0020,   /* U+1461F */
  0x0020,   /* U+14620 */
  0x0020,   /* U+14621 */
  0x0020,   /* U+14622 */
  0x0020,   /* U+14623 */
  0x0020,   /* U+14624 */
  0x0020,   /* U+14625 */
  0x0020,   /* U+14626 */
  0x0020,   /* U+14627 */
  0x0020,   /* U+14628 */
  0x0020,   /* U+14629 */
  0x0020,   /* U+1462A */
  0x0020,   /* U+1462B */
  0x0020,   /* U+1462C */
  0x0020,   /* U+1462D */
  0x0020,   /* U+1462E */
  0x0020,   /* U+1462F */
  0x0020,   /* U+14630 */
  0x0020,   /* U+14631 */
  0x0020,   /* U+14632 */
  0x0020,   /* U+14633 */
  0x0020,   /* U+14634 */
  0x0020,   /* U+14635 */
  0x0020,   /* U+14636 */
  0x0020,   /* U+14637 */
  0x0020,   /* U+14638 */
  0x0020,   /* U+14639 */
  0x0020,   /* U+1463A */
  0x0020,   /* U+1463B */
  0x0020,   /* U+1463C */
  0x0020,   /* U+1463D */
  0x0020,   /* U+1463E */
  0x0020,   /* U+1463F */
  0x0020,   /* U+14640 */
  0x0020,   /* U+14641 */
  0x0020,   /* U+14642 */
  0x0020,   /* U+14643 */
  0x0020,   /* U+14644 */
  0x0020,   /* U+14645 */
  0x0020,   /* U+14646 */
  0x0020,   /* U+14647 */
  0x0020,   /* U+14648 */
  0x0020,   /* U+14649 */
  0x0020,   /* U+1464A */
  0x0020,   /* U+1464B */
  0x0020,   /* U+1464C */
  0x0020,   /* U+1464D */
  0x0020,   /* U+1464E */
  0x0020,   /* U+1464F */
  0x0020,   /* U+14650 */
  0x0020,   /* U+14651 */
  0x0020,   /* U+14652 */
  0x0020,   /* U+14653 */
  0x0020,   /* U+14654 */
  0x0020,   /* U+14655 */
  0x0020,   /* U+14656 */
  0x0020,   /* U+14657 */
  0x0020,   /* U+14658 */
  0x0020,   /* U+14659 */
  0x0020,   /* U+1465A */
  0x0020,   /* U+1465B */
  0x0020,   /* U+1465C */
  0x0020,   /* U+1465D */
  0x0020,   /* U+1465E */
  0x0020,   /* U+1465F */
  0x0020,   /* U+14660 */
  0x0020,   /* U+14661 */
  0x0020,   /* U+14662 */
  0x0020,   /* U+14663 */
  0x0020,   /* U+14664 */
  0x0020,   /* U+14665 */
  0x0020,   /* U+14666 */
  0x0020,   /* U+14667 */
  0x0020,   /* U+14668 */
  0x0020,   /* U+14669 */
  0x0020,   /* U+1466A */
  0x0020,   /* U+1466B */
  0x0020,   /* U+1466C */
  0x0020,   /* U+1466D */
  0x0020,   /* U+1466E */
  0x0020,   /* U+1466F */
  0x0020,   /* U+14670 */
  0x0020,   /* U+14671 */
  0x0020,   /* U+14672 */
  0x0020,   /* U+14673 */
  0x0020,   /* U+14674 */
  0x0020,   /* U+14675 */
  0x0020,   /* U+14676 */
  0x0020,   /* U+14677 */
  0x0020,   /* U+14678 */
  0x0020,   /* U+14679 */
  0x0020,   /* U+1467A */
  0x0020,   /* U+1467B */
  0x0020,   /* U+1467C */
  0x0020,   /* U+1467D */
  0x0020,   /* U+1467E */
  0x0020,   /* U+1467F */
  0x0020,   /* U+14680 */
  0x0020,   /* U+14681 */
  0x0020,   /* U+14682 */
  0x0020,   /* U+14683 */
  0x0020,   /* U+14684 */
  0x0020,   /* U+14685 */
  0x0020,   /* U+14686 */
  0x0020,   /* U+14687 */
  0x0020,   /* U+14688 */
  0x0020,   /* U+14689 */
  0x0020,   /* U+1468A */
  0x0020,   /* U+1468B */
  0x0020,   /* U+1468C */
  0x0020,   /* U+1468D */
  0x0020,   /* U+1468E */
  0x0020,   /* U+1468F */
  0x0020,   /* U+14690 */
  0x0020,   /* U+14691 */
  0x0020,   /* U+14692 */
  0x0020,   /* U+14693 */
  0x0020,   /* U+14694 */
  0x0020,   /* U+14695 */
  0x0020,   /* U+14696 */
  0x0020,   /* U+14697 */
  0x0020,   /* U+14698 */
  0x0020,   /* U+14699 */
  0x0020,   /* U+1469A */
  0x0020,   /* U+1469B */
  0x0020,   /* U+1469C */
  0x0020,   /* U+1469D */
  0x0020,   /* U+1469E */
  0x0020,   /* U+1469F */
  0x0020,   /* U+146A0 */
  0x0020,   /* U+146A1 */
  0x0020,   /* U+146A2 */
  0x0020,   /* U+146A3 */
  0x0020,   /* U+146A4 */
  0x0020,   /* U+146A5 */
  0x0020,   /* U+146A6 */
  0x0020,   /* U+146A7 */
  0x0020,   /* U+146A8 */
  0x0020,   /* U+146A9 */
  0x0020,   /* U+146AA */
  0x0020,   /* U+146AB */
  0x0020,   /* U+146AC */
  0x0020,   /* U+146AD */
  0x0020,   /* U+146AE */
  0x0020,   /* U+146AF */
  0x0020,   /* U+146B0 */
  0x0020,   /* U+146B1 */
  0x0020,   /* U+146B2 */
  0x0020,   /* U+146B3 */
  0x0020,   /* U+146B4 */
  0x0020,   /* U+146B5 */
  0x0020,   /* U+146B6 */
  0x0020,   /* U+146B7 */
  0x0020,   /* U+146B8 */
  0x0020,   /* U+146B9 */
  0x0020,   /* U+146BA */
  0x0020,   /* U+146BB */
  0x0020,   /* U+146BC */
  0x0020,   /* U+146BD */
  0x0020,   /* U+146BE */
  0x0020,   /* U+146BF */
  0x0020,   /* U+146C0 */
  0x0020,   /* U+146C1 */
  0x0020,   /* U+146C2 */
  0x0020,   /* U+146C3 */
  0x0020,   /* U+146C4 */
  0x0020,   /* U+146C5 */
  0x0020,   /* U+146C6 */
  0x0020,   /* U+146C7 */
  0x0020,   /* U+146C8 */
  0x0020,   /* U+146C9 */
  0x0020,   /* U+146CA */
  0x0020,   /* U+146CB */
  0x0020,   /* U+146CC */
  0x0020,   /* U+146CD */
  0x0020,   /* U+146CE */
  0x0020,   /* U+146CF */
  0x0020,   /* U+146D0 */
  0x0020,   /* U+146D1 */
  0x0020,   /* U+146D2 */
  0x0020,   /* U+146D3 */
  0x0020,   /* U+146D4 */
  0x0020,   /* U+146D5 */
  0x0020,   /* U+146D6 */
  0x0020,   /* U+146D7 */
  0x0020,   /* U+146D8 */
  0x0020,   /* U+146D9 */
  0x0020,   /* U+146DA */
  0x0020,   /* U+146DB */
  0x0020,   /* U+146DC */
  0x0020,   /* U+146DD */
  0x0020,   /* U+146DE */
  0x0020,   /* U+146DF */
  0x0020,   /* U+146E0 */
  0x0020,   /* U+146E1 */
  0x0020,   /* U+146E2 */
  0x0020,   /* U+146E3 */
  0x0020,   /* U+146E4 */
  0x0020,   /* U+146E5 */
  0x0020,   /* U+146E6 */
  0x0020,   /* U+146E7 */
  0x0020,   /* U+146E8 */
  0x0020,   /* U+146E9 */
  0x0020,   /* U+146EA */
  0x0020,   /* U+146EB */
  0x0020,   /* U+146EC */
  0x0020,   /* U+146ED */
  0x0020,   /* U+146EE */
  0x0020,   /* U+146EF */
  0x0020,   /* U+146F0 */
  0x0020,   /* U+146F1 */
  0x0020,   /* U+146F2 */
  0x0020,   /* U+146F3 */
  0x0020,   /* U+146F4 */
  0x0020,   /* U+146F5 */
  0x0020,   /* U+146F6 */
  0x0020,   /* U+146F7 */
  0x0020,   /* U+146F8 */
  0x0020,   /* U+146F9 */
  0x0020,   /* U+146FA */
  0x0020,   /* U+146FB */
  0x0020,   /* U+146FC */
  0x0020,   /* U+146FD */
  0x0020,   /* U+146FE */
  0x0020,   /* U+146FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+14600 */
  0x0002,   /* U+14601 */
  0x0002,   /* U+14602 */
  0x0002,   /* U+14603 */
  0x0002,   /* U+14604 */
  0x0002,   /* U+14605 */
  0x0002,   /* U+14606 */
  0x0002,   /* U+14607 */
  0x0002,   /* U+14608 */
  0x0002,   /* U+14609 */
  0x0002,   /* U+1460A */
  0x0002,   /* U+1460B */
  0x0002,   /* U+1460C */
  0x0002,   /* U+1460D */
  0x0002,   /* U+1460E */
  0x0002,   /* U+1460F */
  0x0002,   /* U+14610 */
  0x0002,   /* U+14611 */
  0x0002,   /* U+14612 */
  0x0002,   /* U+14613 */
  0x0002,   /* U+14614 */
  0x0002,   /* U+14615 */
  0x0002,   /* U+14616 */
  0x0002,   /* U+14617 */
  0x0002,   /* U+14618 */
  0x0002,   /* U+14619 */
  0x0002,   /* U+1461A */
  0x0002,   /* U+1461B */
  0x0002,   /* U+1461C */
  0x0002,   /* U+1461D */
  0x0002,   /* U+1461E */
  0x0002,   /* U+1461F */
  0x0002,   /* U+14620 */
  0x0002,   /* U+14621 */
  0x0002,   /* U+14622 */
  0x0002,   /* U+14623 */
  0x0002,   /* U+14624 */
  0x0002,   /* U+14625 */
  0x0002,   /* U+14626 */
  0x0002,   /* U+14627 */
  0x0002,   /* U+14628 */
  0x0002,   /* U+14629 */
  0x0002,   /* U+1462A */
  0x0002,   /* U+1462B */
  0x0002,   /* U+1462C */
  0x0002,   /* U+1462D */
  0x0002,   /* U+1462E */
  0x0002,   /* U+1462F */
  0x0002,   /* U+14630 */
  0x0002,   /* U+14631 */
  0x0002,   /* U+14632 */
  0x0002,   /* U+14633 */
  0x0002,   /* U+14634 */
  0x0002,   /* U+14635 */
  0x0002,   /* U+14636 */
  0x0002,   /* U+14637 */
  0x0002,   /* U+14638 */
  0x0002,   /* U+14639 */
  0x0002,   /* U+1463A */
  0x0002,   /* U+1463B */
  0x0002,   /* U+1463C */
  0x0002,   /* U+1463D */
  0x0002,   /* U+1463E */
  0x0002,   /* U+1463F */
  0x0002,   /* U+14640 */
  0x0002,   /* U+14641 */
  0x0002,   /* U+14642 */
  0x0002,   /* U+14643 */
  0x0002,   /* U+14644 */
  0x0002,   /* U+14645 */
  0x0002,   /* U+14646 */
  0x0002,   /* U+14647 */
  0x0002,   /* U+14648 */
  0x0002,   /* U+14649 */
  0x0002,   /* U+1464A */
  0x0002,   /* U+1464B */
  0x0002,   /* U+1464C */
  0x0002,   /* U+1464D */
  0x0002,   /* U+1464E */
  0x0002,   /* U+1464F */
  0x0002,   /* U+14650 */
  0x0002,   /* U+14651 */
  0x0002,   /* U+14652 */
  0x0002,   /* U+14653 */
  0x0002,   /* U+14654 */
  0x0002,   /* U+14655 */
  0x0002,   /* U+14656 */
  0x0002,   /* U+14657 */
  0x0002,   /* U+14658 */
  0x0002,   /* U+14659 */
  0x0002,   /* U+1465A */
  0x0002,   /* U+1465B */
  0x0002,   /* U+1465C */
  0x0002,   /* U+1465D */
  0x0002,   /* U+1465E */
  0x0002,   /* U+1465F */
  0x0002,   /* U+14660 */
  0x0002,   /* U+14661 */
  0x0002,   /* U+14662 */
  0x0002,   /* U+14663 */
  0x0002,   /* U+14664 */
  0x0002,   /* U+14665 */
  0x0002,   /* U+14666 */
  0x0002,   /* U+14667 */
  0x0002,   /* U+14668 */
  0x0002,   /* U+14669 */
  0x0002,   /* U+1466A */
  0x0002,   /* U+1466B */
  0x0002,   /* U+1466C */
  0x0002,   /* U+1466D */
  0x0002,   /* U+1466E */
  0x0002,   /* U+1466F */
  0x0002,   /* U+14670 */
  0x0002,   /* U+14671 */
  0x0002,   /* U+14672 */
  0x0002,   /* U+14673 */
  0x0002,   /* U+14674 */
  0x0002,   /* U+14675 */
  0x0002,   /* U+14676 */
  0x0002,   /* U+14677 */
  0x0002,   /* U+14678 */
  0x0002,   /* U+14679 */
  0x0002,   /* U+1467A */
  0x0002,   /* U+1467B */
  0x0002,   /* U+1467C */
  0x0002,   /* U+1467D */
  0x0002,   /* U+1467E */
  0x0002,   /* U+1467F */
  0x0002,   /* U+14680 */
  0x0002,   /* U+14681 */
  0x0002,   /* U+14682 */
  0x0002,   /* U+14683 */
  0x0002,   /* U+14684 */
  0x0002,   /* U+14685 */
  0x0002,   /* U+14686 */
  0x0002,   /* U+14687 */
  0x0002,   /* U+14688 */
  0x0002,   /* U+14689 */
  0x0002,   /* U+1468A */
  0x0002,   /* U+1468B */
  0x0002,   /* U+1468C */
  0x0002,   /* U+1468D */
  0x0002,   /* U+1468E */
  0x0002,   /* U+1468F */
  0x0002,   /* U+14690 */
  0x0002,   /* U+14691 */
  0x0002,   /* U+14692 */
  0x0002,   /* U+14693 */
  0x0002,   /* U+14694 */
  0x0002,   /* U+14695 */
  0x0002,   /* U+14696 */
  0x0002,   /* U+14697 */
  0x0002,   /* U+14698 */
  0x0002,   /* U+14699 */
  0x0002,   /* U+1469A */
  0x0002,   /* U+1469B */
  0x0002,   /* U+1469C */
  0x0002,   /* U+1469D */
  0x0002,   /* U+1469E */
  0x0002,   /* U+1469F */
  0x0002,   /* U+146A0 */
  0x0002,   /* U+146A1 */
  0x0002,   /* U+146A2 */
  0x0002,   /* U+146A3 */
  0x0002,   /* U+146A4 */
  0x0002,   /* U+146A5 */
  0x0002,   /* U+146A6 */
  0x0002,   /* U+146A7 */
  0x0002,   /* U+146A8 */
  0x0002,   /* U+146A9 */
  0x0002,   /* U+146AA */
  0x0002,   /* U+146AB */
  0x0002,   /* U+146AC */
  0x0002,   /* U+146AD */
  0x0002,   /* U+146AE */
  0x0002,   /* U+146AF */
  0x0002,   /* U+146B0 */
  0x0002,   /* U+146B1 */
  0x0002,   /* U+146B2 */
  0x0002,   /* U+146B3 */
  0x0002,   /* U+146B4 */
  0x0002,   /* U+146B5 */
  0x0002,   /* U+146B6 */
  0x0002,   /* U+146B7 */
  0x0002,   /* U+146B8 */
  0x0002,   /* U+146B9 */
  0x0002,   /* U+146BA */
  0x0002,   /* U+146BB */
  0x0002,   /* U+146BC */
  0x0002,   /* U+146BD */
  0x0002,   /* U+146BE */
  0x0002,   /* U+146BF */
  0x0002,   /* U+146C0 */
  0x0002,   /* U+146C1 */
  0x0002,   /* U+146C2 */
  0x0002,   /* U+146C3 */
  0x0002,   /* U+146C4 */
  0x0002,   /* U+146C5 */
  0x0002,   /* U+146C6 */
  0x0002,   /* U+146C7 */
  0x0002,   /* U+146C8 */
  0x0002,   /* U+146C9 */
  0x0002,   /* U+146CA */
  0x0002,   /* U+146CB */
  0x0002,   /* U+146CC */
  0x0002,   /* U+146CD */
  0x0002,   /* U+146CE */
  0x0002,   /* U+146CF */
  0x0002,   /* U+146D0 */
  0x0002,   /* U+146D1 */
  0x0002,   /* U+146D2 */
  0x0002,   /* U+146D3 */
  0x0002,   /* U+146D4 */
  0x0002,   /* U+146D5 */
  0x0002,   /* U+146D6 */
  0x0002,   /* U+146D7 */
  0x0002,   /* U+146D8 */
  0x0002,   /* U+146D9 */
  0x0002,   /* U+146DA */
  0x0002,   /* U+146DB */
  0x0002,   /* U+146DC */
  0x0002,   /* U+146DD */
  0x0002,   /* U+146DE */
  0x0002,   /* U+146DF */
  0x0002,   /* U+146E0 */
  0x0002,   /* U+146E1 */
  0x0002,   /* U+146E2 */
  0x0002,   /* U+146E3 */
  0x0002,   /* U+146E4 */
  0x0002,   /* U+146E5 */
  0x0002,   /* U+146E6 */
  0x0002,   /* U+146E7 */
  0x0002,   /* U+146E8 */
  0x0002,   /* U+146E9 */
  0x0002,   /* U+146EA */
  0x0002,   /* U+146EB */
  0x0002,   /* U+146EC */
  0x0002,   /* U+146ED */
  0x0002,   /* U+146EE */
  0x0002,   /* U+146EF */
  0x0002,   /* U+146F0 */
  0x0002,   /* U+146F1 */
  0x0002,   /* U+146F2 */
  0x0002,   /* U+146F3 */
  0x0002,   /* U+146F4 */
  0x0002,   /* U+146F5 */
  0x0002,   /* U+146F6 */
  0x0002,   /* U+146F7 */
  0x0002,   /* U+146F8 */
  0x0002,   /* U+146F9 */
  0x0002,   /* U+146FA */
  0x0002,   /* U+146FB */
  0x0002,   /* U+146FC */
  0x0002,   /* U+146FD */
  0x0002,   /* U+146FE */
  0x0002,   /* U+146FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+14600 */
  0x0000,   /* U+14601 */
  0x0000,   /* U+14602 */
  0x0000,   /* U+14603 */
  0x0000,   /* U+14604 */
  0x0000,   /* U+14605 */
  0x0000,   /* U+14606 */
  0x0000,   /* U+14607 */
  0x0000,   /* U+14608 */
  0x0000,   /* U+14609 */
  0x0000,   /* U+1460A */
  0x0000,   /* U+1460B */
  0x0000,   /* U+1460C */
  0x0000,   /* U+1460D */
  0x0000,   /* U+1460E */
  0x0000,   /* U+1460F */
  0x0000,   /* U+14610 */
  0x0000,   /* U+14611 */
  0x0000,   /* U+14612 */
  0x0000,   /* U+14613 */
  0x0000,   /* U+14614 */
  0x0000,   /* U+14615 */
  0x0000,   /* U+14616 */
  0x0000,   /* U+14617 */
  0x0000,   /* U+14618 */
  0x0000,   /* U+14619 */
  0x0000,   /* U+1461A */
  0x0000,   /* U+1461B */
  0x0000,   /* U+1461C */
  0x0000,   /* U+1461D */
  0x0000,   /* U+1461E */
  0x0000,   /* U+1461F */
  0x0000,   /* U+14620 */
  0x0000,   /* U+14621 */
  0x0000,   /* U+14622 */
  0x0000,   /* U+14623 */
  0x0000,   /* U+14624 */
  0x0000,   /* U+14625 */
  0x0000,   /* U+14626 */
  0x0000,   /* U+14627 */
  0x0000,   /* U+14628 */
  0x0000,   /* U+14629 */
  0x0000,   /* U+1462A */
  0x0000,   /* U+1462B */
  0x0000,   /* U+1462C */
  0x0000,   /* U+1462D */
  0x0000,   /* U+1462E */
  0x0000,   /* U+1462F */
  0x0000,   /* U+14630 */
  0x0000,   /* U+14631 */
  0x0000,   /* U+14632 */
  0x0000,   /* U+14633 */
  0x0000,   /* U+14634 */
  0x0000,   /* U+14635 */
  0x0000,   /* U+14636 */
  0x0000,   /* U+14637 */
  0x0000,   /* U+14638 */
  0x0000,   /* U+14639 */
  0x0000,   /* U+1463A */
  0x0000,   /* U+1463B */
  0x0000,   /* U+1463C */
  0x0000,   /* U+1463D */
  0x0000,   /* U+1463E */
  0x0000,   /* U+1463F */
  0x0000,   /* U+14640 */
  0x0000,   /* U+14641 */
  0x0000,   /* U+14642 */
  0x0000,   /* U+14643 */
  0x0000,   /* U+14644 */
  0x0000,   /* U+14645 */
  0x0000,   /* U+14646 */
  0xC647,   /* U+14647 */
  0xC648,   /* U+14648 */
  0xC649,   /* U+14649 */
  0xC64A,   /* U+1464A */
  0xC64B,   /* U+1464B */
  0xC64C,   /* U+1464C */
  0xC64D,   /* U+1464D */
  0xC64E,   /* U+1464E */
  0xC64F,   /* U+1464F */
  0xC650,   /* U+14650 */
  0xC651,   /* U+14651 */
  0xC652,   /* U+14652 */
  0xC653,   /* U+14653 */
  0xC654,   /* U+14654 */
  0xC655,   /* U+14655 */
  0xC656,   /* U+14656 */
  0xC657,   /* U+14657 */
  0xC658,   /* U+14658 */
  0xC659,   /* U+14659 */
  0xC65A,   /* U+1465A */
  0xC65B,   /* U+1465B */
  0xC65C,   /* U+1465C */
  0xC65D,   /* U+1465D */
  0xC65E,   /* U+1465E */
  0xC65F,   /* U+1465F */
  0xC660,   /* U+14660 */
  0xC661,   /* U+14661 */
  0xC662,   /* U+14662 */
  0xC663,   /* U+14663 */
  0xC664,   /* U+14664 */
  0xC665,   /* U+14665 */
  0xC666,   /* U+14666 */
  0xC667,   /* U+14667 */
  0xC668,   /* U+14668 */
  0xC669,   /* U+14669 */
  0xC66A,   /* U+1466A */
  0xC66B,   /* U+1466B */
  0xC66C,   /* U+1466C */
  0xC66D,   /* U+1466D */
  0xC66E,   /* U+1466E */
  0xC66F,   /* U+1466F */
  0xC670,   /* U+14670 */
  0xC671,   /* U+14671 */
  0xC672,   /* U+14672 */
  0xC673,   /* U+14673 */
  0xC674,   /* U+14674 */
  0xC675,   /* U+14675 */
  0xC676,   /* U+14676 */
  0xC677,   /* U+14677 */
  0xC678,   /* U+14678 */
  0xC679,   /* U+14679 */
  0xC67A,   /* U+1467A */
  0xC67B,   /* U+1467B */
  0xC67C,   /* U+1467C */
  0xC67D,   /* U+1467D */
  0xC67E,   /* U+1467E */
  0xC67F,   /* U+1467F */
  0xC680,   /* U+14680 */
  0xC681,   /* U+14681 */
  0xC682,   /* U+14682 */
  0xC683,   /* U+14683 */
  0xC684,   /* U+14684 */
  0xC685,   /* U+14685 */
  0xC686,   /* U+14686 */
  0xC687,   /* U+14687 */
  0xC688,   /* U+14688 */
  0xC689,   /* U+14689 */
  0xC68A,   /* U+1468A */
  0xC68B,   /* U+1468B */
  0xC68C,   /* U+1468C */
  0xC68D,   /* U+1468D */
  0xC68E,   /* U+1468E */
  0xC68F,   /* U+1468F */
  0xC690,   /* U+14690 */
  0xC691,   /* U+14691 */
  0xC692,   /* U+14692 */
  0xC693,   /* U+14693 */
  0xC694,   /* U+14694 */
  0xC695,   /* U+14695 */
  0xC696,   /* U+14696 */
  0xC697,   /* U+14697 */
  0xC698,   /* U+14698 */
  0xC699,   /* U+14699 */
  0xC69A,   /* U+1469A */
  0xC69B,   /* U+1469B */
  0xC69C,   /* U+1469C */
  0xC69D,   /* U+1469D */
  0xC69E,   /* U+1469E */
  0xC69F,   /* U+1469F */
  0xC6A0,   /* U+146A0 */
  0xC6A1,   /* U+146A1 */
  0xC6A2,   /* U+146A2 */
  0xC6A3,   /* U+146A3 */
  0xC6A4,   /* U+146A4 */
  0xC6A5,   /* U+146A5 */
  0xC6A6,   /* U+146A6 */
  0xC6A7,   /* U+146A7 */
  0xC6A8,   /* U+146A8 */
  0xC6A9,   /* U+146A9 */
  0xC6AA,   /* U+146AA */
  0xC6AB,   /* U+146AB */
  0xC6AC,   /* U+146AC */
  0xC6AD,   /* U+146AD */
  0xC6AE,   /* U+146AE */
  0xC6AF,   /* U+146AF */
  0xC6B0,   /* U+146B0 */
  0xC6B1,   /* U+146B1 */
  0xC6B2,   /* U+146B2 */
  0xC6B3,   /* U+146B3 */
  0xC6B4,   /* U+146B4 */
  0xC6B5,   /* U+146B5 */
  0xC6B6,   /* U+146B6 */
  0xC6B7,   /* U+146B7 */
  0xC6B8,   /* U+146B8 */
  0xC6B9,   /* U+146B9 */
  0xC6BA,   /* U+146BA */
  0xC6BB,   /* U+146BB */
  0xC6BC,   /* U+146BC */
  0xC6BD,   /* U+146BD */
  0xC6BE,   /* U+146BE */
  0xC6BF,   /* U+146BF */
  0xC6C0,   /* U+146C0 */
  0xC6C1,   /* U+146C1 */
  0xC6C2,   /* U+146C2 */
  0xC6C3,   /* U+146C3 */
  0xC6C4,   /* U+146C4 */
  0xC6C5,   /* U+146C5 */
  0xC6C6,   /* U+146C6 */
  0xC6C7,   /* U+146C7 */
  0xC6C8,   /* U+146C8 */
  0xC6C9,   /* U+146C9 */
  0xC6CA,   /* U+146CA */
  0xC6CB,   /* U+146CB */
  0xC6CC,   /* U+146CC */
  0xC6CD,   /* U+146CD */
  0xC6CE,   /* U+146CE */
  0xC6CF,   /* U+146CF */
  0xC6D0,   /* U+146D0 */
  0xC6D1,   /* U+146D1 */
  0xC6D2,   /* U+146D2 */
  0xC6D3,   /* U+146D3 */
  0xC6D4,   /* U+146D4 */
  0xC6D5,   /* U+146D5 */
  0xC6D6,   /* U+146D6 */
  0xC6D7,   /* U+146D7 */
  0xC6D8,   /* U+146D8 */
  0xC6D9,   /* U+146D9 */
  0xC6DA,   /* U+146DA */
  0xC6DB,   /* U+146DB */
  0xC6DC,   /* U+146DC */
  0xC6DD,   /* U+146DD */
  0xC6DE,   /* U+146DE */
  0xC6DF,   /* U+146DF */
  0xC6E0,   /* U+146E0 */
  0xC6E1,   /* U+146E1 */
  0xC6E2,   /* U+146E2 */
  0xC6E3,   /* U+146E3 */
  0xC6E4,   /* U+146E4 */
  0xC6E5,   /* U+146E5 */
  0xC6E6,   /* U+146E6 */
  0xC6E7,   /* U+146E7 */
  0xC6E8,   /* U+146E8 */
  0xC6E9,   /* U+146E9 */
  0xC6EA,   /* U+146EA */
  0xC6EB,   /* U+146EB */
  0xC6EC,   /* U+146EC */
  0xC6ED,   /* U+146ED */
  0xC6EE,   /* U+146EE */
  0xC6EF,   /* U+146EF */
  0xC6F0,   /* U+146F0 */
  0xC6F1,   /* U+146F1 */
  0xC6F2,   /* U+146F2 */
  0xC6F3,   /* U+146F3 */
  0xC6F4,   /* U+146F4 */
  0xC6F5,   /* U+146F5 */
  0xC6F6,   /* U+146F6 */
  0xC6F7,   /* U+146F7 */
  0xC6F8,   /* U+146F8 */
  0xC6F9,   /* U+146F9 */
  0xC6FA,   /* U+146FA */
  0xC6FB,   /* U+146FB */
  0xC6FC,   /* U+146FC */
  0xC6FD,   /* U+146FD */
  0xC6FE,   /* U+146FE */
  0xC6FF,   /* U+146FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+14600 */
  0x0000,   /* U+14601 */
  0x0000,   /* U+14602 */
  0x0000,   /* U+14603 */
  0x0000,   /* U+14604 */
  0x0000,   /* U+14605 */
  0x0000,   /* U+14606 */
  0x0000,   /* U+14607 */
  0x0000,   /* U+14608 */
  0x0000,   /* U+14609 */
  0x0000,   /* U+1460A */
  0x0000,   /* U+1460B */
  0x0000,   /* U+1460C */
  0x0000,   /* U+1460D */
  0x0000,   /* U+1460E */
  0x0000,   /* U+1460F */
  0x0000,   /* U+14610 */
  0x0000,   /* U+14611 */
  0x0000,   /* U+14612 */
  0x0000,   /* U+14613 */
  0x0000,   /* U+14614 */
  0x0000,   /* U+14615 */
  0x0000,   /* U+14616 */
  0x0000,   /* U+14617 */
  0x0000,   /* U+14618 */
  0x0000,   /* U+14619 */
  0x0000,   /* U+1461A */
  0x0000,   /* U+1461B */
  0x0000,   /* U+1461C */
  0x0000,   /* U+1461D */
  0x0000,   /* U+1461E */
  0x0000,   /* U+1461F */
  0x0000,   /* U+14620 */
  0x0000,   /* U+14621 */
  0x0000,   /* U+14622 */
  0x0000,   /* U+14623 */
  0x0000,   /* U+14624 */
  0x0000,   /* U+14625 */
  0x0000,   /* U+14626 */
  0x0000,   /* U+14627 */
  0x0000,   /* U+14628 */
  0x0000,   /* U+14629 */
  0x0000,   /* U+1462A */
  0x0000,   /* U+1462B */
  0x0000,   /* U+1462C */
  0x0000,   /* U+1462D */
  0x0000,   /* U+1462E */
  0x0000,   /* U+1462F */
  0x0000,   /* U+14630 */
  0x0000,   /* U+14631 */
  0x0000,   /* U+14632 */
  0x0000,   /* U+14633 */
  0x0000,   /* U+14634 */
  0x0000,   /* U+14635 */
  0x0000,   /* U+14636 */
  0x0000,   /* U+14637 */
  0x0000,   /* U+14638 */
  0x0000,   /* U+14639 */
  0x0000,   /* U+1463A */
  0x0000,   /* U+1463B */
  0x0000,   /* U+1463C */
  0x0000,   /* U+1463D */
  0x0000,   /* U+1463E */
  0x0000,   /* U+1463F */
  0x0000,   /* U+14640 */
  0x0000,   /* U+14641 */
  0x0000,   /* U+14642 */
  0x0000,   /* U+14643 */
  0x0000,   /* U+14644 */
  0x0000,   /* U+14645 */
  0x0000,   /* U+14646 */
  0x0000,   /* U+14647 */
  0x0000,   /* U+14648 */
  0x0000,   /* U+14649 */
  0x0000,   /* U+1464A */
  0x0000,   /* U+1464B */
  0x0000,   /* U+1464C */
  0x0000,   /* U+1464D */
  0x0000,   /* U+1464E */
  0x0000,   /* U+1464F */
  0x0000,   /* U+14650 */
  0x0000,   /* U+14651 */
  0x0000,   /* U+14652 */
  0x0000,   /* U+14653 */
  0x0000,   /* U+14654 */
  0x0000,   /* U+14655 */
  0x0000,   /* U+14656 */
  0x0000,   /* U+14657 */
  0x0000,   /* U+14658 */
  0x0000,   /* U+14659 */
  0x0000,   /* U+1465A */
  0x0000,   /* U+1465B */
  0x0000,   /* U+1465C */
  0x0000,   /* U+1465D */
  0x0000,   /* U+1465E */
  0x0000,   /* U+1465F */
  0x0000,   /* U+14660 */
  0x0000,   /* U+14661 */
  0x0000,   /* U+14662 */
  0x0000,   /* U+14663 */
  0x0000,   /* U+14664 */
  0x0000,   /* U+14665 */
  0x0000,   /* U+14666 */
  0x0000,   /* U+14667 */
  0x0000,   /* U+14668 */
  0x0000,   /* U+14669 */
  0x0000,   /* U+1466A */
  0x0000,   /* U+1466B */
  0x0000,   /* U+1466C */
  0x0000,   /* U+1466D */
  0x0000,   /* U+1466E */
  0x0000,   /* U+1466F */
  0x0000,   /* U+14670 */
  0x0000,   /* U+14671 */
  0x0000,   /* U+14672 */
  0x0000,   /* U+14673 */
  0x0000,   /* U+14674 */
  0x0000,   /* U+14675 */
  0x0000,   /* U+14676 */
  0x0000,   /* U+14677 */
  0x0000,   /* U+14678 */
  0x0000,   /* U+14679 */
  0x0000,   /* U+1467A */
  0x0000,   /* U+1467B */
  0x0000,   /* U+1467C */
  0x0000,   /* U+1467D */
  0x0000,   /* U+1467E */
  0x0000,   /* U+1467F */
  0x0000,   /* U+14680 */
  0x0000,   /* U+14681 */
  0x0000,   /* U+14682 */
  0x0000,   /* U+14683 */
  0x0000,   /* U+14684 */
  0x0000,   /* U+14685 */
  0x0000,   /* U+14686 */
  0x0000,   /* U+14687 */
  0x0000,   /* U+14688 */
  0x0000,   /* U+14689 */
  0x0000,   /* U+1468A */
  0x0000,   /* U+1468B */
  0x0000,   /* U+1468C */
  0x0000,   /* U+1468D */
  0x0000,   /* U+1468E */
  0x0000,   /* U+1468F */
  0x0000,   /* U+14690 */
  0x0000,   /* U+14691 */
  0x0000,   /* U+14692 */
  0x0000,   /* U+14693 */
  0x0000,   /* U+14694 */
  0x0000,   /* U+14695 */
  0x0000,   /* U+14696 */
  0x0000,   /* U+14697 */
  0x0000,   /* U+14698 */
  0x0000,   /* U+14699 */
  0x0000,   /* U+1469A */
  0x0000,   /* U+1469B */
  0x0000,   /* U+1469C */
  0x0000,   /* U+1469D */
  0x0000,   /* U+1469E */
  0x0000,   /* U+1469F */
  0x0000,   /* U+146A0 */
  0x0000,   /* U+146A1 */
  0x0000,   /* U+146A2 */
  0x0000,   /* U+146A3 */
  0x0000,   /* U+146A4 */
  0x0000,   /* U+146A5 */
  0x0000,   /* U+146A6 */
  0x0000,   /* U+146A7 */
  0x0000,   /* U+146A8 */
  0x0000,   /* U+146A9 */
  0x0000,   /* U+146AA */
  0x0000,   /* U+146AB */
  0x0000,   /* U+146AC */
  0x0000,   /* U+146AD */
  0x0000,   /* U+146AE */
  0x0000,   /* U+146AF */
  0x0000,   /* U+146B0 */
  0x0000,   /* U+146B1 */
  0x0000,   /* U+146B2 */
  0x0000,   /* U+146B3 */
  0x0000,   /* U+146B4 */
  0x0000,   /* U+146B5 */
  0x0000,   /* U+146B6 */
  0x0000,   /* U+146B7 */
  0x0000,   /* U+146B8 */
  0x0000,   /* U+146B9 */
  0x0000,   /* U+146BA */
  0x0000,   /* U+146BB */
  0x0000,   /* U+146BC */
  0x0000,   /* U+146BD */
  0x0000,   /* U+146BE */
  0x0000,   /* U+146BF */
  0x0000,   /* U+146C0 */
  0x0000,   /* U+146C1 */
  0x0000,   /* U+146C2 */
  0x0000,   /* U+146C3 */
  0x0000,   /* U+146C4 */
  0x0000,   /* U+146C5 */
  0x0000,   /* U+146C6 */
  0x0000,   /* U+146C7 */
  0x0000,   /* U+146C8 */
  0x0000,   /* U+146C9 */
  0x0000,   /* U+146CA */
  0x0000,   /* U+146CB */
  0x0000,   /* U+146CC */
  0x0000,   /* U+146CD */
  0x0000,   /* U+146CE */
  0x0000,   /* U+146CF */
  0x0000,   /* U+146D0 */
  0x0000,   /* U+146D1 */
  0x0000,   /* U+146D2 */
  0x0000,   /* U+146D3 */
  0x0000,   /* U+146D4 */
  0x0000,   /* U+146D5 */
  0x0000,   /* U+146D6 */
  0x0000,   /* U+146D7 */
  0x0000,   /* U+146D8 */
  0x0000,   /* U+146D9 */
  0x0000,   /* U+146DA */
  0x0000,   /* U+146DB */
  0x0000,   /* U+146DC */
  0x0000,   /* U+146DD */
  0x0000,   /* U+146DE */
  0x0000,   /* U+146DF */
  0x0000,   /* U+146E0 */
  0x0000,   /* U+146E1 */
  0x0000,   /* U+146E2 */
  0x0000,   /* U+146E3 */
  0x0000,   /* U+146E4 */
  0x0000,   /* U+146E5 */
  0x0000,   /* U+146E6 */
  0x0000,   /* U+146E7 */
  0x0000,   /* U+146E8 */
  0x0000,   /* U+146E9 */
  0x0000,   /* U+146EA */
  0x0000,   /* U+146EB */
  0x0000,   /* U+146EC */
  0x0000,   /* U+146ED */
  0x0000,   /* U+146EE */
  0x0000,   /* U+146EF */
  0x0000,   /* U+146F0 */
  0x0000,   /* U+146F1 */
  0x0000,   /* U+146F2 */
  0x0000,   /* U+146F3 */
  0x0000,   /* U+146F4 */
  0x0000,   /* U+146F5 */
  0x0000,   /* U+146F6 */
  0x0000,   /* U+146F7 */
  0x0000,   /* U+146F8 */
  0x0000,   /* U+146F9 */
  0x0000,   /* U+146FA */
  0x0000,   /* U+146FB */
  0x0000,   /* U+146FC */
  0x0000,   /* U+146FD */
  0x0000,   /* U+146FE */
  0x0000,   /* U+146FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+14600 */
  0x0000,   /* U+14601 */
  0x0000,   /* U+14602 */
  0x0000,   /* U+14603 */
  0x0000,   /* U+14604 */
  0x0000,   /* U+14605 */
  0x0000,   /* U+14606 */
  0x0000,   /* U+14607 */
  0x0000,   /* U+14608 */
  0x0000,   /* U+14609 */
  0x0000,   /* U+1460A */
  0x0000,   /* U+1460B */
  0x0000,   /* U+1460C */
  0x0000,   /* U+1460D */
  0x0000,   /* U+1460E */
  0x0000,   /* U+1460F */
  0x0000,   /* U+14610 */
  0x0000,   /* U+14611 */
  0x0000,   /* U+14612 */
  0x0000,   /* U+14613 */
  0x0000,   /* U+14614 */
  0x0000,   /* U+14615 */
  0x0000,   /* U+14616 */
  0x0000,   /* U+14617 */
  0x0000,   /* U+14618 */
  0x0000,   /* U+14619 */
  0x0000,   /* U+1461A */
  0x0000,   /* U+1461B */
  0x0000,   /* U+1461C */
  0x0000,   /* U+1461D */
  0x0000,   /* U+1461E */
  0x0000,   /* U+1461F */
  0x0000,   /* U+14620 */
  0x0000,   /* U+14621 */
  0x0000,   /* U+14622 */
  0x0000,   /* U+14623 */
  0x0000,   /* U+14624 */
  0x0000,   /* U+14625 */
  0x0000,   /* U+14626 */
  0x0000,   /* U+14627 */
  0x0000,   /* U+14628 */
  0x0000,   /* U+14629 */
  0x0000,   /* U+1462A */
  0x0000,   /* U+1462B */
  0x0000,   /* U+1462C */
  0x0000,   /* U+1462D */
  0x0000,   /* U+1462E */
  0x0000,   /* U+1462F */
  0x0000,   /* U+14630 */
  0x0000,   /* U+14631 */
  0x0000,   /* U+14632 */
  0x0000,   /* U+14633 */
  0x0000,   /* U+14634 */
  0x0000,   /* U+14635 */
  0x0000,   /* U+14636 */
  0x0000,   /* U+14637 */
  0x0000,   /* U+14638 */
  0x0000,   /* U+14639 */
  0x0000,   /* U+1463A */
  0x0000,   /* U+1463B */
  0x0000,   /* U+1463C */
  0x0000,   /* U+1463D */
  0x0000,   /* U+1463E */
  0x0000,   /* U+1463F */
  0x0000,   /* U+14640 */
  0x0000,   /* U+14641 */
  0x0000,   /* U+14642 */
  0x0000,   /* U+14643 */
  0x0000,   /* U+14644 */
  0x0000,   /* U+14645 */
  0x0000,   /* U+14646 */
  0x0000,   /* U+14647 */
  0x0000,   /* U+14648 */
  0x0000,   /* U+14649 */
  0x0000,   /* U+1464A */
  0x0000,   /* U+1464B */
  0x0000,   /* U+1464C */
  0x0000,   /* U+1464D */
  0x0000,   /* U+1464E */
  0x0000,   /* U+1464F */
  0x0000,   /* U+14650 */
  0x0000,   /* U+14651 */
  0x0000,   /* U+14652 */
  0x0000,   /* U+14653 */
  0x0000,   /* U+14654 */
  0x0000,   /* U+14655 */
  0x0000,   /* U+14656 */
  0x0000,   /* U+14657 */
  0x0000,   /* U+14658 */
  0x0000,   /* U+14659 */
  0x0000,   /* U+1465A */
  0x0000,   /* U+1465B */
  0x0000,   /* U+1465C */
  0x0000,   /* U+1465D */
  0x0000,   /* U+1465E */
  0x0000,   /* U+1465F */
  0x0000,   /* U+14660 */
  0x0000,   /* U+14661 */
  0x0000,   /* U+14662 */
  0x0000,   /* U+14663 */
  0x0000,   /* U+14664 */
  0x0000,   /* U+14665 */
  0x0000,   /* U+14666 */
  0x0000,   /* U+14667 */
  0x0000,   /* U+14668 */
  0x0000,   /* U+14669 */
  0x0000,   /* U+1466A */
  0x0000,   /* U+1466B */
  0x0000,   /* U+1466C */
  0x0000,   /* U+1466D */
  0x0000,   /* U+1466E */
  0x0000,   /* U+1466F */
  0x0000,   /* U+14670 */
  0x0000,   /* U+14671 */
  0x0000,   /* U+14672 */
  0x0000,   /* U+14673 */
  0x0000,   /* U+14674 */
  0x0000,   /* U+14675 */
  0x0000,   /* U+14676 */
  0x0000,   /* U+14677 */
  0x0000,   /* U+14678 */
  0x0000,   /* U+14679 */
  0x0000,   /* U+1467A */
  0x0000,   /* U+1467B */
  0x0000,   /* U+1467C */
  0x0000,   /* U+1467D */
  0x0000,   /* U+1467E */
  0x0000,   /* U+1467F */
  0x0000,   /* U+14680 */
  0x0000,   /* U+14681 */
  0x0000,   /* U+14682 */
  0x0000,   /* U+14683 */
  0x0000,   /* U+14684 */
  0x0000,   /* U+14685 */
  0x0000,   /* U+14686 */
  0x0000,   /* U+14687 */
  0x0000,   /* U+14688 */
  0x0000,   /* U+14689 */
  0x0000,   /* U+1468A */
  0x0000,   /* U+1468B */
  0x0000,   /* U+1468C */
  0x0000,   /* U+1468D */
  0x0000,   /* U+1468E */
  0x0000,   /* U+1468F */
  0x0000,   /* U+14690 */
  0x0000,   /* U+14691 */
  0x0000,   /* U+14692 */
  0x0000,   /* U+14693 */
  0x0000,   /* U+14694 */
  0x0000,   /* U+14695 */
  0x0000,   /* U+14696 */
  0x0000,   /* U+14697 */
  0x0000,   /* U+14698 */
  0x0000,   /* U+14699 */
  0x0000,   /* U+1469A */
  0x0000,   /* U+1469B */
  0x0000,   /* U+1469C */
  0x0000,   /* U+1469D */
  0x0000,   /* U+1469E */
  0x0000,   /* U+1469F */
  0x0000,   /* U+146A0 */
  0x0000,   /* U+146A1 */
  0x0000,   /* U+146A2 */
  0x0000,   /* U+146A3 */
  0x0000,   /* U+146A4 */
  0x0000,   /* U+146A5 */
  0x0000,   /* U+146A6 */
  0x0000,   /* U+146A7 */
  0x0000,   /* U+146A8 */
  0x0000,   /* U+146A9 */
  0x0000,   /* U+146AA */
  0x0000,   /* U+146AB */
  0x0000,   /* U+146AC */
  0x0000,   /* U+146AD */
  0x0000,   /* U+146AE */
  0x0000,   /* U+146AF */
  0x0000,   /* U+146B0 */
  0x0000,   /* U+146B1 */
  0x0000,   /* U+146B2 */
  0x0000,   /* U+146B3 */
  0x0000,   /* U+146B4 */
  0x0000,   /* U+146B5 */
  0x0000,   /* U+146B6 */
  0x0000,   /* U+146B7 */
  0x0000,   /* U+146B8 */
  0x0000,   /* U+146B9 */
  0x0000,   /* U+146BA */
  0x0000,   /* U+146BB */
  0x0000,   /* U+146BC */
  0x0000,   /* U+146BD */
  0x0000,   /* U+146BE */
  0x0000,   /* U+146BF */
  0x0000,   /* U+146C0 */
  0x0000,   /* U+146C1 */
  0x0000,   /* U+146C2 */
  0x0000,   /* U+146C3 */
  0x0000,   /* U+146C4 */
  0x0000,   /* U+146C5 */
  0x0000,   /* U+146C6 */
  0x0000,   /* U+146C7 */
  0x0000,   /* U+146C8 */
  0x0000,   /* U+146C9 */
  0x0000,   /* U+146CA */
  0x0000,   /* U+146CB */
  0x0000,   /* U+146CC */
  0x0000,   /* U+146CD */
  0x0000,   /* U+146CE */
  0x0000,   /* U+146CF */
  0x0000,   /* U+146D0 */
  0x0000,   /* U+146D1 */
  0x0000,   /* U+146D2 */
  0x0000,   /* U+146D3 */
  0x0000,   /* U+146D4 */
  0x0000,   /* U+146D5 */
  0x0000,   /* U+146D6 */
  0x0000,   /* U+146D7 */
  0x0000,   /* U+146D8 */
  0x0000,   /* U+146D9 */
  0x0000,   /* U+146DA */
  0x0000,   /* U+146DB */
  0x0000,   /* U+146DC */
  0x0000,   /* U+146DD */
  0x0000,   /* U+146DE */
  0x0000,   /* U+146DF */
  0x0000,   /* U+146E0 */
  0x0000,   /* U+146E1 */
  0x0000,   /* U+146E2 */
  0x0000,   /* U+146E3 */
  0x0000,   /* U+146E4 */
  0x0000,   /* U+146E5 */
  0x0000,   /* U+146E6 */
  0x0000,   /* U+146E7 */
  0x0000,   /* U+146E8 */
  0x0000,   /* U+146E9 */
  0x0000,   /* U+146EA */
  0x0000,   /* U+146EB */
  0x0000,   /* U+146EC */
  0x0000,   /* U+146ED */
  0x0000,   /* U+146EE */
  0x0000,   /* U+146EF */
  0x0000,   /* U+146F0 */
  0x0000,   /* U+146F1 */
  0x0000,   /* U+146F2 */
  0x0000,   /* U+146F3 */
  0x0000,   /* U+146F4 */
  0x0000,   /* U+146F5 */
  0x0000,   /* U+146F6 */
  0x0000,   /* U+146F7 */
  0x0000,   /* U+146F8 */
  0x0000,   /* U+146F9 */
  0x0000,   /* U+146FA */
  0x0000,   /* U+146FB */
  0x0000,   /* U+146FC */
  0x0000,   /* U+146FD */
  0x0000,   /* U+146FE */
  0x0000,   /* U+146FF */
};

uint16 uca900_p168[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x38B7,   /* U+16800 */
  0x38B8,   /* U+16801 */
  0x38B9,   /* U+16802 */
  0x38BA,   /* U+16803 */
  0x38BB,   /* U+16804 */
  0x38BC,   /* U+16805 */
  0x38BD,   /* U+16806 */
  0x38BE,   /* U+16807 */
  0x38BF,   /* U+16808 */
  0x38C0,   /* U+16809 */
  0x38C1,   /* U+1680A */
  0x38C2,   /* U+1680B */
  0x38C3,   /* U+1680C */
  0x38C4,   /* U+1680D */
  0x38C5,   /* U+1680E */
  0x38C6,   /* U+1680F */
  0x38C7,   /* U+16810 */
  0x38C8,   /* U+16811 */
  0x38C9,   /* U+16812 */
  0x38CA,   /* U+16813 */
  0x38CB,   /* U+16814 */
  0x38CC,   /* U+16815 */
  0x38CD,   /* U+16816 */
  0x38CE,   /* U+16817 */
  0x38CF,   /* U+16818 */
  0x38D0,   /* U+16819 */
  0x38D1,   /* U+1681A */
  0x38D2,   /* U+1681B */
  0x38D3,   /* U+1681C */
  0x38D4,   /* U+1681D */
  0x38D5,   /* U+1681E */
  0x38D6,   /* U+1681F */
  0x38D7,   /* U+16820 */
  0x38D8,   /* U+16821 */
  0x38D9,   /* U+16822 */
  0x38DA,   /* U+16823 */
  0x38DB,   /* U+16824 */
  0x38DC,   /* U+16825 */
  0x38DD,   /* U+16826 */
  0x38DE,   /* U+16827 */
  0x38DF,   /* U+16828 */
  0x38E0,   /* U+16829 */
  0x38E1,   /* U+1682A */
  0x38E2,   /* U+1682B */
  0x38E3,   /* U+1682C */
  0x38E4,   /* U+1682D */
  0x38E5,   /* U+1682E */
  0x38E6,   /* U+1682F */
  0x38E7,   /* U+16830 */
  0x38E8,   /* U+16831 */
  0x38E9,   /* U+16832 */
  0x38EA,   /* U+16833 */
  0x38EB,   /* U+16834 */
  0x38EC,   /* U+16835 */
  0x38ED,   /* U+16836 */
  0x38EE,   /* U+16837 */
  0x38EF,   /* U+16838 */
  0x38F0,   /* U+16839 */
  0x38F1,   /* U+1683A */
  0x38F2,   /* U+1683B */
  0x38F3,   /* U+1683C */
  0x38F4,   /* U+1683D */
  0x38F5,   /* U+1683E */
  0x38F6,   /* U+1683F */
  0x38F7,   /* U+16840 */
  0x38F8,   /* U+16841 */
  0x38F9,   /* U+16842 */
  0x38FA,   /* U+16843 */
  0x38FB,   /* U+16844 */
  0x38FC,   /* U+16845 */
  0x38FD,   /* U+16846 */
  0x38FE,   /* U+16847 */
  0x38FF,   /* U+16848 */
  0x3900,   /* U+16849 */
  0x3901,   /* U+1684A */
  0x3902,   /* U+1684B */
  0x3903,   /* U+1684C */
  0x3904,   /* U+1684D */
  0x3905,   /* U+1684E */
  0x3906,   /* U+1684F */
  0x3907,   /* U+16850 */
  0x3908,   /* U+16851 */
  0x3909,   /* U+16852 */
  0x390A,   /* U+16853 */
  0x390B,   /* U+16854 */
  0x390C,   /* U+16855 */
  0x390D,   /* U+16856 */
  0x390E,   /* U+16857 */
  0x390F,   /* U+16858 */
  0x3910,   /* U+16859 */
  0x3911,   /* U+1685A */
  0x3912,   /* U+1685B */
  0x3913,   /* U+1685C */
  0x3914,   /* U+1685D */
  0x3915,   /* U+1685E */
  0x3916,   /* U+1685F */
  0x3917,   /* U+16860 */
  0x3918,   /* U+16861 */
  0x3919,   /* U+16862 */
  0x391A,   /* U+16863 */
  0x391B,   /* U+16864 */
  0x391C,   /* U+16865 */
  0x391D,   /* U+16866 */
  0x391E,   /* U+16867 */
  0x391F,   /* U+16868 */
  0x3920,   /* U+16869 */
  0x3921,   /* U+1686A */
  0x3922,   /* U+1686B */
  0x3923,   /* U+1686C */
  0x3924,   /* U+1686D */
  0x3925,   /* U+1686E */
  0x3926,   /* U+1686F */
  0x3927,   /* U+16870 */
  0x3928,   /* U+16871 */
  0x3929,   /* U+16872 */
  0x392A,   /* U+16873 */
  0x392B,   /* U+16874 */
  0x392C,   /* U+16875 */
  0x392D,   /* U+16876 */
  0x392E,   /* U+16877 */
  0x392F,   /* U+16878 */
  0x3930,   /* U+16879 */
  0x3931,   /* U+1687A */
  0x3932,   /* U+1687B */
  0x3933,   /* U+1687C */
  0x3934,   /* U+1687D */
  0x3935,   /* U+1687E */
  0x3936,   /* U+1687F */
  0x3937,   /* U+16880 */
  0x3938,   /* U+16881 */
  0x3939,   /* U+16882 */
  0x393A,   /* U+16883 */
  0x393B,   /* U+16884 */
  0x393C,   /* U+16885 */
  0x393D,   /* U+16886 */
  0x393E,   /* U+16887 */
  0x393F,   /* U+16888 */
  0x3940,   /* U+16889 */
  0x3941,   /* U+1688A */
  0x3942,   /* U+1688B */
  0x3943,   /* U+1688C */
  0x3944,   /* U+1688D */
  0x3945,   /* U+1688E */
  0x3946,   /* U+1688F */
  0x3947,   /* U+16890 */
  0x3948,   /* U+16891 */
  0x3949,   /* U+16892 */
  0x394A,   /* U+16893 */
  0x394B,   /* U+16894 */
  0x394C,   /* U+16895 */
  0x394D,   /* U+16896 */
  0x394E,   /* U+16897 */
  0x394F,   /* U+16898 */
  0x3950,   /* U+16899 */
  0x3951,   /* U+1689A */
  0x3952,   /* U+1689B */
  0x3953,   /* U+1689C */
  0x3954,   /* U+1689D */
  0x3955,   /* U+1689E */
  0x3956,   /* U+1689F */
  0x3957,   /* U+168A0 */
  0x3958,   /* U+168A1 */
  0x3959,   /* U+168A2 */
  0x395A,   /* U+168A3 */
  0x395B,   /* U+168A4 */
  0x395C,   /* U+168A5 */
  0x395D,   /* U+168A6 */
  0x395E,   /* U+168A7 */
  0x395F,   /* U+168A8 */
  0x3960,   /* U+168A9 */
  0x3961,   /* U+168AA */
  0x3962,   /* U+168AB */
  0x3963,   /* U+168AC */
  0x3964,   /* U+168AD */
  0x3965,   /* U+168AE */
  0x3966,   /* U+168AF */
  0x3967,   /* U+168B0 */
  0x3968,   /* U+168B1 */
  0x3969,   /* U+168B2 */
  0x396A,   /* U+168B3 */
  0x396B,   /* U+168B4 */
  0x396C,   /* U+168B5 */
  0x396D,   /* U+168B6 */
  0x396E,   /* U+168B7 */
  0x396F,   /* U+168B8 */
  0x3970,   /* U+168B9 */
  0x3971,   /* U+168BA */
  0x3972,   /* U+168BB */
  0x3973,   /* U+168BC */
  0x3974,   /* U+168BD */
  0x3975,   /* U+168BE */
  0x3976,   /* U+168BF */
  0x3977,   /* U+168C0 */
  0x3978,   /* U+168C1 */
  0x3979,   /* U+168C2 */
  0x397A,   /* U+168C3 */
  0x397B,   /* U+168C4 */
  0x397C,   /* U+168C5 */
  0x397D,   /* U+168C6 */
  0x397E,   /* U+168C7 */
  0x397F,   /* U+168C8 */
  0x3980,   /* U+168C9 */
  0x3981,   /* U+168CA */
  0x3982,   /* U+168CB */
  0x3983,   /* U+168CC */
  0x3984,   /* U+168CD */
  0x3985,   /* U+168CE */
  0x3986,   /* U+168CF */
  0x3987,   /* U+168D0 */
  0x3988,   /* U+168D1 */
  0x3989,   /* U+168D2 */
  0x398A,   /* U+168D3 */
  0x398B,   /* U+168D4 */
  0x398C,   /* U+168D5 */
  0x398D,   /* U+168D6 */
  0x398E,   /* U+168D7 */
  0x398F,   /* U+168D8 */
  0x3990,   /* U+168D9 */
  0x3991,   /* U+168DA */
  0x3992,   /* U+168DB */
  0x3993,   /* U+168DC */
  0x3994,   /* U+168DD */
  0x3995,   /* U+168DE */
  0x3996,   /* U+168DF */
  0x3997,   /* U+168E0 */
  0x3998,   /* U+168E1 */
  0x3999,   /* U+168E2 */
  0x399A,   /* U+168E3 */
  0x399B,   /* U+168E4 */
  0x399C,   /* U+168E5 */
  0x399D,   /* U+168E6 */
  0x399E,   /* U+168E7 */
  0x399F,   /* U+168E8 */
  0x39A0,   /* U+168E9 */
  0x39A1,   /* U+168EA */
  0x39A2,   /* U+168EB */
  0x39A3,   /* U+168EC */
  0x39A4,   /* U+168ED */
  0x39A5,   /* U+168EE */
  0x39A6,   /* U+168EF */
  0x39A7,   /* U+168F0 */
  0x39A8,   /* U+168F1 */
  0x39A9,   /* U+168F2 */
  0x39AA,   /* U+168F3 */
  0x39AB,   /* U+168F4 */
  0x39AC,   /* U+168F5 */
  0x39AD,   /* U+168F6 */
  0x39AE,   /* U+168F7 */
  0x39AF,   /* U+168F8 */
  0x39B0,   /* U+168F9 */
  0x39B1,   /* U+168FA */
  0x39B2,   /* U+168FB */
  0x39B3,   /* U+168FC */
  0x39B4,   /* U+168FD */
  0x39B5,   /* U+168FE */
  0x39B6,   /* U+168FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+16800 */
  0x0020,   /* U+16801 */
  0x0020,   /* U+16802 */
  0x0020,   /* U+16803 */
  0x0020,   /* U+16804 */
  0x0020,   /* U+16805 */
  0x0020,   /* U+16806 */
  0x0020,   /* U+16807 */
  0x0020,   /* U+16808 */
  0x0020,   /* U+16809 */
  0x0020,   /* U+1680A */
  0x0020,   /* U+1680B */
  0x0020,   /* U+1680C */
  0x0020,   /* U+1680D */
  0x0020,   /* U+1680E */
  0x0020,   /* U+1680F */
  0x0020,   /* U+16810 */
  0x0020,   /* U+16811 */
  0x0020,   /* U+16812 */
  0x0020,   /* U+16813 */
  0x0020,   /* U+16814 */
  0x0020,   /* U+16815 */
  0x0020,   /* U+16816 */
  0x0020,   /* U+16817 */
  0x0020,   /* U+16818 */
  0x0020,   /* U+16819 */
  0x0020,   /* U+1681A */
  0x0020,   /* U+1681B */
  0x0020,   /* U+1681C */
  0x0020,   /* U+1681D */
  0x0020,   /* U+1681E */
  0x0020,   /* U+1681F */
  0x0020,   /* U+16820 */
  0x0020,   /* U+16821 */
  0x0020,   /* U+16822 */
  0x0020,   /* U+16823 */
  0x0020,   /* U+16824 */
  0x0020,   /* U+16825 */
  0x0020,   /* U+16826 */
  0x0020,   /* U+16827 */
  0x0020,   /* U+16828 */
  0x0020,   /* U+16829 */
  0x0020,   /* U+1682A */
  0x0020,   /* U+1682B */
  0x0020,   /* U+1682C */
  0x0020,   /* U+1682D */
  0x0020,   /* U+1682E */
  0x0020,   /* U+1682F */
  0x0020,   /* U+16830 */
  0x0020,   /* U+16831 */
  0x0020,   /* U+16832 */
  0x0020,   /* U+16833 */
  0x0020,   /* U+16834 */
  0x0020,   /* U+16835 */
  0x0020,   /* U+16836 */
  0x0020,   /* U+16837 */
  0x0020,   /* U+16838 */
  0x0020,   /* U+16839 */
  0x0020,   /* U+1683A */
  0x0020,   /* U+1683B */
  0x0020,   /* U+1683C */
  0x0020,   /* U+1683D */
  0x0020,   /* U+1683E */
  0x0020,   /* U+1683F */
  0x0020,   /* U+16840 */
  0x0020,   /* U+16841 */
  0x0020,   /* U+16842 */
  0x0020,   /* U+16843 */
  0x0020,   /* U+16844 */
  0x0020,   /* U+16845 */
  0x0020,   /* U+16846 */
  0x0020,   /* U+16847 */
  0x0020,   /* U+16848 */
  0x0020,   /* U+16849 */
  0x0020,   /* U+1684A */
  0x0020,   /* U+1684B */
  0x0020,   /* U+1684C */
  0x0020,   /* U+1684D */
  0x0020,   /* U+1684E */
  0x0020,   /* U+1684F */
  0x0020,   /* U+16850 */
  0x0020,   /* U+16851 */
  0x0020,   /* U+16852 */
  0x0020,   /* U+16853 */
  0x0020,   /* U+16854 */
  0x0020,   /* U+16855 */
  0x0020,   /* U+16856 */
  0x0020,   /* U+16857 */
  0x0020,   /* U+16858 */
  0x0020,   /* U+16859 */
  0x0020,   /* U+1685A */
  0x0020,   /* U+1685B */
  0x0020,   /* U+1685C */
  0x0020,   /* U+1685D */
  0x0020,   /* U+1685E */
  0x0020,   /* U+1685F */
  0x0020,   /* U+16860 */
  0x0020,   /* U+16861 */
  0x0020,   /* U+16862 */
  0x0020,   /* U+16863 */
  0x0020,   /* U+16864 */
  0x0020,   /* U+16865 */
  0x0020,   /* U+16866 */
  0x0020,   /* U+16867 */
  0x0020,   /* U+16868 */
  0x0020,   /* U+16869 */
  0x0020,   /* U+1686A */
  0x0020,   /* U+1686B */
  0x0020,   /* U+1686C */
  0x0020,   /* U+1686D */
  0x0020,   /* U+1686E */
  0x0020,   /* U+1686F */
  0x0020,   /* U+16870 */
  0x0020,   /* U+16871 */
  0x0020,   /* U+16872 */
  0x0020,   /* U+16873 */
  0x0020,   /* U+16874 */
  0x0020,   /* U+16875 */
  0x0020,   /* U+16876 */
  0x0020,   /* U+16877 */
  0x0020,   /* U+16878 */
  0x0020,   /* U+16879 */
  0x0020,   /* U+1687A */
  0x0020,   /* U+1687B */
  0x0020,   /* U+1687C */
  0x0020,   /* U+1687D */
  0x0020,   /* U+1687E */
  0x0020,   /* U+1687F */
  0x0020,   /* U+16880 */
  0x0020,   /* U+16881 */
  0x0020,   /* U+16882 */
  0x0020,   /* U+16883 */
  0x0020,   /* U+16884 */
  0x0020,   /* U+16885 */
  0x0020,   /* U+16886 */
  0x0020,   /* U+16887 */
  0x0020,   /* U+16888 */
  0x0020,   /* U+16889 */
  0x0020,   /* U+1688A */
  0x0020,   /* U+1688B */
  0x0020,   /* U+1688C */
  0x0020,   /* U+1688D */
  0x0020,   /* U+1688E */
  0x0020,   /* U+1688F */
  0x0020,   /* U+16890 */
  0x0020,   /* U+16891 */
  0x0020,   /* U+16892 */
  0x0020,   /* U+16893 */
  0x0020,   /* U+16894 */
  0x0020,   /* U+16895 */
  0x0020,   /* U+16896 */
  0x0020,   /* U+16897 */
  0x0020,   /* U+16898 */
  0x0020,   /* U+16899 */
  0x0020,   /* U+1689A */
  0x0020,   /* U+1689B */
  0x0020,   /* U+1689C */
  0x0020,   /* U+1689D */
  0x0020,   /* U+1689E */
  0x0020,   /* U+1689F */
  0x0020,   /* U+168A0 */
  0x0020,   /* U+168A1 */
  0x0020,   /* U+168A2 */
  0x0020,   /* U+168A3 */
  0x0020,   /* U+168A4 */
  0x0020,   /* U+168A5 */
  0x0020,   /* U+168A6 */
  0x0020,   /* U+168A7 */
  0x0020,   /* U+168A8 */
  0x0020,   /* U+168A9 */
  0x0020,   /* U+168AA */
  0x0020,   /* U+168AB */
  0x0020,   /* U+168AC */
  0x0020,   /* U+168AD */
  0x0020,   /* U+168AE */
  0x0020,   /* U+168AF */
  0x0020,   /* U+168B0 */
  0x0020,   /* U+168B1 */
  0x0020,   /* U+168B2 */
  0x0020,   /* U+168B3 */
  0x0020,   /* U+168B4 */
  0x0020,   /* U+168B5 */
  0x0020,   /* U+168B6 */
  0x0020,   /* U+168B7 */
  0x0020,   /* U+168B8 */
  0x0020,   /* U+168B9 */
  0x0020,   /* U+168BA */
  0x0020,   /* U+168BB */
  0x0020,   /* U+168BC */
  0x0020,   /* U+168BD */
  0x0020,   /* U+168BE */
  0x0020,   /* U+168BF */
  0x0020,   /* U+168C0 */
  0x0020,   /* U+168C1 */
  0x0020,   /* U+168C2 */
  0x0020,   /* U+168C3 */
  0x0020,   /* U+168C4 */
  0x0020,   /* U+168C5 */
  0x0020,   /* U+168C6 */
  0x0020,   /* U+168C7 */
  0x0020,   /* U+168C8 */
  0x0020,   /* U+168C9 */
  0x0020,   /* U+168CA */
  0x0020,   /* U+168CB */
  0x0020,   /* U+168CC */
  0x0020,   /* U+168CD */
  0x0020,   /* U+168CE */
  0x0020,   /* U+168CF */
  0x0020,   /* U+168D0 */
  0x0020,   /* U+168D1 */
  0x0020,   /* U+168D2 */
  0x0020,   /* U+168D3 */
  0x0020,   /* U+168D4 */
  0x0020,   /* U+168D5 */
  0x0020,   /* U+168D6 */
  0x0020,   /* U+168D7 */
  0x0020,   /* U+168D8 */
  0x0020,   /* U+168D9 */
  0x0020,   /* U+168DA */
  0x0020,   /* U+168DB */
  0x0020,   /* U+168DC */
  0x0020,   /* U+168DD */
  0x0020,   /* U+168DE */
  0x0020,   /* U+168DF */
  0x0020,   /* U+168E0 */
  0x0020,   /* U+168E1 */
  0x0020,   /* U+168E2 */
  0x0020,   /* U+168E3 */
  0x0020,   /* U+168E4 */
  0x0020,   /* U+168E5 */
  0x0020,   /* U+168E6 */
  0x0020,   /* U+168E7 */
  0x0020,   /* U+168E8 */
  0x0020,   /* U+168E9 */
  0x0020,   /* U+168EA */
  0x0020,   /* U+168EB */
  0x0020,   /* U+168EC */
  0x0020,   /* U+168ED */
  0x0020,   /* U+168EE */
  0x0020,   /* U+168EF */
  0x0020,   /* U+168F0 */
  0x0020,   /* U+168F1 */
  0x0020,   /* U+168F2 */
  0x0020,   /* U+168F3 */
  0x0020,   /* U+168F4 */
  0x0020,   /* U+168F5 */
  0x0020,   /* U+168F6 */
  0x0020,   /* U+168F7 */
  0x0020,   /* U+168F8 */
  0x0020,   /* U+168F9 */
  0x0020,   /* U+168FA */
  0x0020,   /* U+168FB */
  0x0020,   /* U+168FC */
  0x0020,   /* U+168FD */
  0x0020,   /* U+168FE */
  0x0020,   /* U+168FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+16800 */
  0x0002,   /* U+16801 */
  0x0002,   /* U+16802 */
  0x0002,   /* U+16803 */
  0x0002,   /* U+16804 */
  0x0002,   /* U+16805 */
  0x0002,   /* U+16806 */
  0x0002,   /* U+16807 */
  0x0002,   /* U+16808 */
  0x0002,   /* U+16809 */
  0x0002,   /* U+1680A */
  0x0002,   /* U+1680B */
  0x0002,   /* U+1680C */
  0x0002,   /* U+1680D */
  0x0002,   /* U+1680E */
  0x0002,   /* U+1680F */
  0x0002,   /* U+16810 */
  0x0002,   /* U+16811 */
  0x0002,   /* U+16812 */
  0x0002,   /* U+16813 */
  0x0002,   /* U+16814 */
  0x0002,   /* U+16815 */
  0x0002,   /* U+16816 */
  0x0002,   /* U+16817 */
  0x0002,   /* U+16818 */
  0x0002,   /* U+16819 */
  0x0002,   /* U+1681A */
  0x0002,   /* U+1681B */
  0x0002,   /* U+1681C */
  0x0002,   /* U+1681D */
  0x0002,   /* U+1681E */
  0x0002,   /* U+1681F */
  0x0002,   /* U+16820 */
  0x0002,   /* U+16821 */
  0x0002,   /* U+16822 */
  0x0002,   /* U+16823 */
  0x0002,   /* U+16824 */
  0x0002,   /* U+16825 */
  0x0002,   /* U+16826 */
  0x0002,   /* U+16827 */
  0x0002,   /* U+16828 */
  0x0002,   /* U+16829 */
  0x0002,   /* U+1682A */
  0x0002,   /* U+1682B */
  0x0002,   /* U+1682C */
  0x0002,   /* U+1682D */
  0x0002,   /* U+1682E */
  0x0002,   /* U+1682F */
  0x0002,   /* U+16830 */
  0x0002,   /* U+16831 */
  0x0002,   /* U+16832 */
  0x0002,   /* U+16833 */
  0x0002,   /* U+16834 */
  0x0002,   /* U+16835 */
  0x0002,   /* U+16836 */
  0x0002,   /* U+16837 */
  0x0002,   /* U+16838 */
  0x0002,   /* U+16839 */
  0x0002,   /* U+1683A */
  0x0002,   /* U+1683B */
  0x0002,   /* U+1683C */
  0x0002,   /* U+1683D */
  0x0002,   /* U+1683E */
  0x0002,   /* U+1683F */
  0x0002,   /* U+16840 */
  0x0002,   /* U+16841 */
  0x0002,   /* U+16842 */
  0x0002,   /* U+16843 */
  0x0002,   /* U+16844 */
  0x0002,   /* U+16845 */
  0x0002,   /* U+16846 */
  0x0002,   /* U+16847 */
  0x0002,   /* U+16848 */
  0x0002,   /* U+16849 */
  0x0002,   /* U+1684A */
  0x0002,   /* U+1684B */
  0x0002,   /* U+1684C */
  0x0002,   /* U+1684D */
  0x0002,   /* U+1684E */
  0x0002,   /* U+1684F */
  0x0002,   /* U+16850 */
  0x0002,   /* U+16851 */
  0x0002,   /* U+16852 */
  0x0002,   /* U+16853 */
  0x0002,   /* U+16854 */
  0x0002,   /* U+16855 */
  0x0002,   /* U+16856 */
  0x0002,   /* U+16857 */
  0x0002,   /* U+16858 */
  0x0002,   /* U+16859 */
  0x0002,   /* U+1685A */
  0x0002,   /* U+1685B */
  0x0002,   /* U+1685C */
  0x0002,   /* U+1685D */
  0x0002,   /* U+1685E */
  0x0002,   /* U+1685F */
  0x0002,   /* U+16860 */
  0x0002,   /* U+16861 */
  0x0002,   /* U+16862 */
  0x0002,   /* U+16863 */
  0x0002,   /* U+16864 */
  0x0002,   /* U+16865 */
  0x0002,   /* U+16866 */
  0x0002,   /* U+16867 */
  0x0002,   /* U+16868 */
  0x0002,   /* U+16869 */
  0x0002,   /* U+1686A */
  0x0002,   /* U+1686B */
  0x0002,   /* U+1686C */
  0x0002,   /* U+1686D */
  0x0002,   /* U+1686E */
  0x0002,   /* U+1686F */
  0x0002,   /* U+16870 */
  0x0002,   /* U+16871 */
  0x0002,   /* U+16872 */
  0x0002,   /* U+16873 */
  0x0002,   /* U+16874 */
  0x0002,   /* U+16875 */
  0x0002,   /* U+16876 */
  0x0002,   /* U+16877 */
  0x0002,   /* U+16878 */
  0x0002,   /* U+16879 */
  0x0002,   /* U+1687A */
  0x0002,   /* U+1687B */
  0x0002,   /* U+1687C */
  0x0002,   /* U+1687D */
  0x0002,   /* U+1687E */
  0x0002,   /* U+1687F */
  0x0002,   /* U+16880 */
  0x0002,   /* U+16881 */
  0x0002,   /* U+16882 */
  0x0002,   /* U+16883 */
  0x0002,   /* U+16884 */
  0x0002,   /* U+16885 */
  0x0002,   /* U+16886 */
  0x0002,   /* U+16887 */
  0x0002,   /* U+16888 */
  0x0002,   /* U+16889 */
  0x0002,   /* U+1688A */
  0x0002,   /* U+1688B */
  0x0002,   /* U+1688C */
  0x0002,   /* U+1688D */
  0x0002,   /* U+1688E */
  0x0002,   /* U+1688F */
  0x0002,   /* U+16890 */
  0x0002,   /* U+16891 */
  0x0002,   /* U+16892 */
  0x0002,   /* U+16893 */
  0x0002,   /* U+16894 */
  0x0002,   /* U+16895 */
  0x0002,   /* U+16896 */
  0x0002,   /* U+16897 */
  0x0002,   /* U+16898 */
  0x0002,   /* U+16899 */
  0x0002,   /* U+1689A */
  0x0002,   /* U+1689B */
  0x0002,   /* U+1689C */
  0x0002,   /* U+1689D */
  0x0002,   /* U+1689E */
  0x0002,   /* U+1689F */
  0x0002,   /* U+168A0 */
  0x0002,   /* U+168A1 */
  0x0002,   /* U+168A2 */
  0x0002,   /* U+168A3 */
  0x0002,   /* U+168A4 */
  0x0002,   /* U+168A5 */
  0x0002,   /* U+168A6 */
  0x0002,   /* U+168A7 */
  0x0002,   /* U+168A8 */
  0x0002,   /* U+168A9 */
  0x0002,   /* U+168AA */
  0x0002,   /* U+168AB */
  0x0002,   /* U+168AC */
  0x0002,   /* U+168AD */
  0x0002,   /* U+168AE */
  0x0002,   /* U+168AF */
  0x0002,   /* U+168B0 */
  0x0002,   /* U+168B1 */
  0x0002,   /* U+168B2 */
  0x0002,   /* U+168B3 */
  0x0002,   /* U+168B4 */
  0x0002,   /* U+168B5 */
  0x0002,   /* U+168B6 */
  0x0002,   /* U+168B7 */
  0x0002,   /* U+168B8 */
  0x0002,   /* U+168B9 */
  0x0002,   /* U+168BA */
  0x0002,   /* U+168BB */
  0x0002,   /* U+168BC */
  0x0002,   /* U+168BD */
  0x0002,   /* U+168BE */
  0x0002,   /* U+168BF */
  0x0002,   /* U+168C0 */
  0x0002,   /* U+168C1 */
  0x0002,   /* U+168C2 */
  0x0002,   /* U+168C3 */
  0x0002,   /* U+168C4 */
  0x0002,   /* U+168C5 */
  0x0002,   /* U+168C6 */
  0x0002,   /* U+168C7 */
  0x0002,   /* U+168C8 */
  0x0002,   /* U+168C9 */
  0x0002,   /* U+168CA */
  0x0002,   /* U+168CB */
  0x0002,   /* U+168CC */
  0x0002,   /* U+168CD */
  0x0002,   /* U+168CE */
  0x0002,   /* U+168CF */
  0x0002,   /* U+168D0 */
  0x0002,   /* U+168D1 */
  0x0002,   /* U+168D2 */
  0x0002,   /* U+168D3 */
  0x0002,   /* U+168D4 */
  0x0002,   /* U+168D5 */
  0x0002,   /* U+168D6 */
  0x0002,   /* U+168D7 */
  0x0002,   /* U+168D8 */
  0x0002,   /* U+168D9 */
  0x0002,   /* U+168DA */
  0x0002,   /* U+168DB */
  0x0002,   /* U+168DC */
  0x0002,   /* U+168DD */
  0x0002,   /* U+168DE */
  0x0002,   /* U+168DF */
  0x0002,   /* U+168E0 */
  0x0002,   /* U+168E1 */
  0x0002,   /* U+168E2 */
  0x0002,   /* U+168E3 */
  0x0002,   /* U+168E4 */
  0x0002,   /* U+168E5 */
  0x0002,   /* U+168E6 */
  0x0002,   /* U+168E7 */
  0x0002,   /* U+168E8 */
  0x0002,   /* U+168E9 */
  0x0002,   /* U+168EA */
  0x0002,   /* U+168EB */
  0x0002,   /* U+168EC */
  0x0002,   /* U+168ED */
  0x0002,   /* U+168EE */
  0x0002,   /* U+168EF */
  0x0002,   /* U+168F0 */
  0x0002,   /* U+168F1 */
  0x0002,   /* U+168F2 */
  0x0002,   /* U+168F3 */
  0x0002,   /* U+168F4 */
  0x0002,   /* U+168F5 */
  0x0002,   /* U+168F6 */
  0x0002,   /* U+168F7 */
  0x0002,   /* U+168F8 */
  0x0002,   /* U+168F9 */
  0x0002,   /* U+168FA */
  0x0002,   /* U+168FB */
  0x0002,   /* U+168FC */
  0x0002,   /* U+168FD */
  0x0002,   /* U+168FE */
  0x0002,   /* U+168FF */
};

uint16 uca900_p169[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x39B7,   /* U+16900 */
  0x39B8,   /* U+16901 */
  0x39B9,   /* U+16902 */
  0x39BA,   /* U+16903 */
  0x39BB,   /* U+16904 */
  0x39BC,   /* U+16905 */
  0x39BD,   /* U+16906 */
  0x39BE,   /* U+16907 */
  0x39BF,   /* U+16908 */
  0x39C0,   /* U+16909 */
  0x39C1,   /* U+1690A */
  0x39C2,   /* U+1690B */
  0x39C3,   /* U+1690C */
  0x39C4,   /* U+1690D */
  0x39C5,   /* U+1690E */
  0x39C6,   /* U+1690F */
  0x39C7,   /* U+16910 */
  0x39C8,   /* U+16911 */
  0x39C9,   /* U+16912 */
  0x39CA,   /* U+16913 */
  0x39CB,   /* U+16914 */
  0x39CC,   /* U+16915 */
  0x39CD,   /* U+16916 */
  0x39CE,   /* U+16917 */
  0x39CF,   /* U+16918 */
  0x39D0,   /* U+16919 */
  0x39D1,   /* U+1691A */
  0x39D2,   /* U+1691B */
  0x39D3,   /* U+1691C */
  0x39D4,   /* U+1691D */
  0x39D5,   /* U+1691E */
  0x39D6,   /* U+1691F */
  0x39D7,   /* U+16920 */
  0x39D8,   /* U+16921 */
  0x39D9,   /* U+16922 */
  0x39DA,   /* U+16923 */
  0x39DB,   /* U+16924 */
  0x39DC,   /* U+16925 */
  0x39DD,   /* U+16926 */
  0x39DE,   /* U+16927 */
  0x39DF,   /* U+16928 */
  0x39E0,   /* U+16929 */
  0x39E1,   /* U+1692A */
  0x39E2,   /* U+1692B */
  0x39E3,   /* U+1692C */
  0x39E4,   /* U+1692D */
  0x39E5,   /* U+1692E */
  0x39E6,   /* U+1692F */
  0x39E7,   /* U+16930 */
  0x39E8,   /* U+16931 */
  0x39E9,   /* U+16932 */
  0x39EA,   /* U+16933 */
  0x39EB,   /* U+16934 */
  0x39EC,   /* U+16935 */
  0x39ED,   /* U+16936 */
  0x39EE,   /* U+16937 */
  0x39EF,   /* U+16938 */
  0x39F0,   /* U+16939 */
  0x39F1,   /* U+1693A */
  0x39F2,   /* U+1693B */
  0x39F3,   /* U+1693C */
  0x39F4,   /* U+1693D */
  0x39F5,   /* U+1693E */
  0x39F6,   /* U+1693F */
  0x39F7,   /* U+16940 */
  0x39F8,   /* U+16941 */
  0x39F9,   /* U+16942 */
  0x39FA,   /* U+16943 */
  0x39FB,   /* U+16944 */
  0x39FC,   /* U+16945 */
  0x39FD,   /* U+16946 */
  0x39FE,   /* U+16947 */
  0x39FF,   /* U+16948 */
  0x3A00,   /* U+16949 */
  0x3A01,   /* U+1694A */
  0x3A02,   /* U+1694B */
  0x3A03,   /* U+1694C */
  0x3A04,   /* U+1694D */
  0x3A05,   /* U+1694E */
  0x3A06,   /* U+1694F */
  0x3A07,   /* U+16950 */
  0x3A08,   /* U+16951 */
  0x3A09,   /* U+16952 */
  0x3A0A,   /* U+16953 */
  0x3A0B,   /* U+16954 */
  0x3A0C,   /* U+16955 */
  0x3A0D,   /* U+16956 */
  0x3A0E,   /* U+16957 */
  0x3A0F,   /* U+16958 */
  0x3A10,   /* U+16959 */
  0x3A11,   /* U+1695A */
  0x3A12,   /* U+1695B */
  0x3A13,   /* U+1695C */
  0x3A14,   /* U+1695D */
  0x3A15,   /* U+1695E */
  0x3A16,   /* U+1695F */
  0x3A17,   /* U+16960 */
  0x3A18,   /* U+16961 */
  0x3A19,   /* U+16962 */
  0x3A1A,   /* U+16963 */
  0x3A1B,   /* U+16964 */
  0x3A1C,   /* U+16965 */
  0x3A1D,   /* U+16966 */
  0x3A1E,   /* U+16967 */
  0x3A1F,   /* U+16968 */
  0x3A20,   /* U+16969 */
  0x3A21,   /* U+1696A */
  0x3A22,   /* U+1696B */
  0x3A23,   /* U+1696C */
  0x3A24,   /* U+1696D */
  0x3A25,   /* U+1696E */
  0x3A26,   /* U+1696F */
  0x3A27,   /* U+16970 */
  0x3A28,   /* U+16971 */
  0x3A29,   /* U+16972 */
  0x3A2A,   /* U+16973 */
  0x3A2B,   /* U+16974 */
  0x3A2C,   /* U+16975 */
  0x3A2D,   /* U+16976 */
  0x3A2E,   /* U+16977 */
  0x3A2F,   /* U+16978 */
  0x3A30,   /* U+16979 */
  0x3A31,   /* U+1697A */
  0x3A32,   /* U+1697B */
  0x3A33,   /* U+1697C */
  0x3A34,   /* U+1697D */
  0x3A35,   /* U+1697E */
  0x3A36,   /* U+1697F */
  0x3A37,   /* U+16980 */
  0x3A38,   /* U+16981 */
  0x3A39,   /* U+16982 */
  0x3A3A,   /* U+16983 */
  0x3A3B,   /* U+16984 */
  0x3A3C,   /* U+16985 */
  0x3A3D,   /* U+16986 */
  0x3A3E,   /* U+16987 */
  0x3A3F,   /* U+16988 */
  0x3A40,   /* U+16989 */
  0x3A41,   /* U+1698A */
  0x3A42,   /* U+1698B */
  0x3A43,   /* U+1698C */
  0x3A44,   /* U+1698D */
  0x3A45,   /* U+1698E */
  0x3A46,   /* U+1698F */
  0x3A47,   /* U+16990 */
  0x3A48,   /* U+16991 */
  0x3A49,   /* U+16992 */
  0x3A4A,   /* U+16993 */
  0x3A4B,   /* U+16994 */
  0x3A4C,   /* U+16995 */
  0x3A4D,   /* U+16996 */
  0x3A4E,   /* U+16997 */
  0x3A4F,   /* U+16998 */
  0x3A50,   /* U+16999 */
  0x3A51,   /* U+1699A */
  0x3A52,   /* U+1699B */
  0x3A53,   /* U+1699C */
  0x3A54,   /* U+1699D */
  0x3A55,   /* U+1699E */
  0x3A56,   /* U+1699F */
  0x3A57,   /* U+169A0 */
  0x3A58,   /* U+169A1 */
  0x3A59,   /* U+169A2 */
  0x3A5A,   /* U+169A3 */
  0x3A5B,   /* U+169A4 */
  0x3A5C,   /* U+169A5 */
  0x3A5D,   /* U+169A6 */
  0x3A5E,   /* U+169A7 */
  0x3A5F,   /* U+169A8 */
  0x3A60,   /* U+169A9 */
  0x3A61,   /* U+169AA */
  0x3A62,   /* U+169AB */
  0x3A63,   /* U+169AC */
  0x3A64,   /* U+169AD */
  0x3A65,   /* U+169AE */
  0x3A66,   /* U+169AF */
  0x3A67,   /* U+169B0 */
  0x3A68,   /* U+169B1 */
  0x3A69,   /* U+169B2 */
  0x3A6A,   /* U+169B3 */
  0x3A6B,   /* U+169B4 */
  0x3A6C,   /* U+169B5 */
  0x3A6D,   /* U+169B6 */
  0x3A6E,   /* U+169B7 */
  0x3A6F,   /* U+169B8 */
  0x3A70,   /* U+169B9 */
  0x3A71,   /* U+169BA */
  0x3A72,   /* U+169BB */
  0x3A73,   /* U+169BC */
  0x3A74,   /* U+169BD */
  0x3A75,   /* U+169BE */
  0x3A76,   /* U+169BF */
  0x3A77,   /* U+169C0 */
  0x3A78,   /* U+169C1 */
  0x3A79,   /* U+169C2 */
  0x3A7A,   /* U+169C3 */
  0x3A7B,   /* U+169C4 */
  0x3A7C,   /* U+169C5 */
  0x3A7D,   /* U+169C6 */
  0x3A7E,   /* U+169C7 */
  0x3A7F,   /* U+169C8 */
  0x3A80,   /* U+169C9 */
  0x3A81,   /* U+169CA */
  0x3A82,   /* U+169CB */
  0x3A83,   /* U+169CC */
  0x3A84,   /* U+169CD */
  0x3A85,   /* U+169CE */
  0x3A86,   /* U+169CF */
  0x3A87,   /* U+169D0 */
  0x3A88,   /* U+169D1 */
  0x3A89,   /* U+169D2 */
  0x3A8A,   /* U+169D3 */
  0x3A8B,   /* U+169D4 */
  0x3A8C,   /* U+169D5 */
  0x3A8D,   /* U+169D6 */
  0x3A8E,   /* U+169D7 */
  0x3A8F,   /* U+169D8 */
  0x3A90,   /* U+169D9 */
  0x3A91,   /* U+169DA */
  0x3A92,   /* U+169DB */
  0x3A93,   /* U+169DC */
  0x3A94,   /* U+169DD */
  0x3A95,   /* U+169DE */
  0x3A96,   /* U+169DF */
  0x3A97,   /* U+169E0 */
  0x3A98,   /* U+169E1 */
  0x3A99,   /* U+169E2 */
  0x3A9A,   /* U+169E3 */
  0x3A9B,   /* U+169E4 */
  0x3A9C,   /* U+169E5 */
  0x3A9D,   /* U+169E6 */
  0x3A9E,   /* U+169E7 */
  0x3A9F,   /* U+169E8 */
  0x3AA0,   /* U+169E9 */
  0x3AA1,   /* U+169EA */
  0x3AA2,   /* U+169EB */
  0x3AA3,   /* U+169EC */
  0x3AA4,   /* U+169ED */
  0x3AA5,   /* U+169EE */
  0x3AA6,   /* U+169EF */
  0x3AA7,   /* U+169F0 */
  0x3AA8,   /* U+169F1 */
  0x3AA9,   /* U+169F2 */
  0x3AAA,   /* U+169F3 */
  0x3AAB,   /* U+169F4 */
  0x3AAC,   /* U+169F5 */
  0x3AAD,   /* U+169F6 */
  0x3AAE,   /* U+169F7 */
  0x3AAF,   /* U+169F8 */
  0x3AB0,   /* U+169F9 */
  0x3AB1,   /* U+169FA */
  0x3AB2,   /* U+169FB */
  0x3AB3,   /* U+169FC */
  0x3AB4,   /* U+169FD */
  0x3AB5,   /* U+169FE */
  0x3AB6,   /* U+169FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+16900 */
  0x0020,   /* U+16901 */
  0x0020,   /* U+16902 */
  0x0020,   /* U+16903 */
  0x0020,   /* U+16904 */
  0x0020,   /* U+16905 */
  0x0020,   /* U+16906 */
  0x0020,   /* U+16907 */
  0x0020,   /* U+16908 */
  0x0020,   /* U+16909 */
  0x0020,   /* U+1690A */
  0x0020,   /* U+1690B */
  0x0020,   /* U+1690C */
  0x0020,   /* U+1690D */
  0x0020,   /* U+1690E */
  0x0020,   /* U+1690F */
  0x0020,   /* U+16910 */
  0x0020,   /* U+16911 */
  0x0020,   /* U+16912 */
  0x0020,   /* U+16913 */
  0x0020,   /* U+16914 */
  0x0020,   /* U+16915 */
  0x0020,   /* U+16916 */
  0x0020,   /* U+16917 */
  0x0020,   /* U+16918 */
  0x0020,   /* U+16919 */
  0x0020,   /* U+1691A */
  0x0020,   /* U+1691B */
  0x0020,   /* U+1691C */
  0x0020,   /* U+1691D */
  0x0020,   /* U+1691E */
  0x0020,   /* U+1691F */
  0x0020,   /* U+16920 */
  0x0020,   /* U+16921 */
  0x0020,   /* U+16922 */
  0x0020,   /* U+16923 */
  0x0020,   /* U+16924 */
  0x0020,   /* U+16925 */
  0x0020,   /* U+16926 */
  0x0020,   /* U+16927 */
  0x0020,   /* U+16928 */
  0x0020,   /* U+16929 */
  0x0020,   /* U+1692A */
  0x0020,   /* U+1692B */
  0x0020,   /* U+1692C */
  0x0020,   /* U+1692D */
  0x0020,   /* U+1692E */
  0x0020,   /* U+1692F */
  0x0020,   /* U+16930 */
  0x0020,   /* U+16931 */
  0x0020,   /* U+16932 */
  0x0020,   /* U+16933 */
  0x0020,   /* U+16934 */
  0x0020,   /* U+16935 */
  0x0020,   /* U+16936 */
  0x0020,   /* U+16937 */
  0x0020,   /* U+16938 */
  0x0020,   /* U+16939 */
  0x0020,   /* U+1693A */
  0x0020,   /* U+1693B */
  0x0020,   /* U+1693C */
  0x0020,   /* U+1693D */
  0x0020,   /* U+1693E */
  0x0020,   /* U+1693F */
  0x0020,   /* U+16940 */
  0x0020,   /* U+16941 */
  0x0020,   /* U+16942 */
  0x0020,   /* U+16943 */
  0x0020,   /* U+16944 */
  0x0020,   /* U+16945 */
  0x0020,   /* U+16946 */
  0x0020,   /* U+16947 */
  0x0020,   /* U+16948 */
  0x0020,   /* U+16949 */
  0x0020,   /* U+1694A */
  0x0020,   /* U+1694B */
  0x0020,   /* U+1694C */
  0x0020,   /* U+1694D */
  0x0020,   /* U+1694E */
  0x0020,   /* U+1694F */
  0x0020,   /* U+16950 */
  0x0020,   /* U+16951 */
  0x0020,   /* U+16952 */
  0x0020,   /* U+16953 */
  0x0020,   /* U+16954 */
  0x0020,   /* U+16955 */
  0x0020,   /* U+16956 */
  0x0020,   /* U+16957 */
  0x0020,   /* U+16958 */
  0x0020,   /* U+16959 */
  0x0020,   /* U+1695A */
  0x0020,   /* U+1695B */
  0x0020,   /* U+1695C */
  0x0020,   /* U+1695D */
  0x0020,   /* U+1695E */
  0x0020,   /* U+1695F */
  0x0020,   /* U+16960 */
  0x0020,   /* U+16961 */
  0x0020,   /* U+16962 */
  0x0020,   /* U+16963 */
  0x0020,   /* U+16964 */
  0x0020,   /* U+16965 */
  0x0020,   /* U+16966 */
  0x0020,   /* U+16967 */
  0x0020,   /* U+16968 */
  0x0020,   /* U+16969 */
  0x0020,   /* U+1696A */
  0x0020,   /* U+1696B */
  0x0020,   /* U+1696C */
  0x0020,   /* U+1696D */
  0x0020,   /* U+1696E */
  0x0020,   /* U+1696F */
  0x0020,   /* U+16970 */
  0x0020,   /* U+16971 */
  0x0020,   /* U+16972 */
  0x0020,   /* U+16973 */
  0x0020,   /* U+16974 */
  0x0020,   /* U+16975 */
  0x0020,   /* U+16976 */
  0x0020,   /* U+16977 */
  0x0020,   /* U+16978 */
  0x0020,   /* U+16979 */
  0x0020,   /* U+1697A */
  0x0020,   /* U+1697B */
  0x0020,   /* U+1697C */
  0x0020,   /* U+1697D */
  0x0020,   /* U+1697E */
  0x0020,   /* U+1697F */
  0x0020,   /* U+16980 */
  0x0020,   /* U+16981 */
  0x0020,   /* U+16982 */
  0x0020,   /* U+16983 */
  0x0020,   /* U+16984 */
  0x0020,   /* U+16985 */
  0x0020,   /* U+16986 */
  0x0020,   /* U+16987 */
  0x0020,   /* U+16988 */
  0x0020,   /* U+16989 */
  0x0020,   /* U+1698A */
  0x0020,   /* U+1698B */
  0x0020,   /* U+1698C */
  0x0020,   /* U+1698D */
  0x0020,   /* U+1698E */
  0x0020,   /* U+1698F */
  0x0020,   /* U+16990 */
  0x0020,   /* U+16991 */
  0x0020,   /* U+16992 */
  0x0020,   /* U+16993 */
  0x0020,   /* U+16994 */
  0x0020,   /* U+16995 */
  0x0020,   /* U+16996 */
  0x0020,   /* U+16997 */
  0x0020,   /* U+16998 */
  0x0020,   /* U+16999 */
  0x0020,   /* U+1699A */
  0x0020,   /* U+1699B */
  0x0020,   /* U+1699C */
  0x0020,   /* U+1699D */
  0x0020,   /* U+1699E */
  0x0020,   /* U+1699F */
  0x0020,   /* U+169A0 */
  0x0020,   /* U+169A1 */
  0x0020,   /* U+169A2 */
  0x0020,   /* U+169A3 */
  0x0020,   /* U+169A4 */
  0x0020,   /* U+169A5 */
  0x0020,   /* U+169A6 */
  0x0020,   /* U+169A7 */
  0x0020,   /* U+169A8 */
  0x0020,   /* U+169A9 */
  0x0020,   /* U+169AA */
  0x0020,   /* U+169AB */
  0x0020,   /* U+169AC */
  0x0020,   /* U+169AD */
  0x0020,   /* U+169AE */
  0x0020,   /* U+169AF */
  0x0020,   /* U+169B0 */
  0x0020,   /* U+169B1 */
  0x0020,   /* U+169B2 */
  0x0020,   /* U+169B3 */
  0x0020,   /* U+169B4 */
  0x0020,   /* U+169B5 */
  0x0020,   /* U+169B6 */
  0x0020,   /* U+169B7 */
  0x0020,   /* U+169B8 */
  0x0020,   /* U+169B9 */
  0x0020,   /* U+169BA */
  0x0020,   /* U+169BB */
  0x0020,   /* U+169BC */
  0x0020,   /* U+169BD */
  0x0020,   /* U+169BE */
  0x0020,   /* U+169BF */
  0x0020,   /* U+169C0 */
  0x0020,   /* U+169C1 */
  0x0020,   /* U+169C2 */
  0x0020,   /* U+169C3 */
  0x0020,   /* U+169C4 */
  0x0020,   /* U+169C5 */
  0x0020,   /* U+169C6 */
  0x0020,   /* U+169C7 */
  0x0020,   /* U+169C8 */
  0x0020,   /* U+169C9 */
  0x0020,   /* U+169CA */
  0x0020,   /* U+169CB */
  0x0020,   /* U+169CC */
  0x0020,   /* U+169CD */
  0x0020,   /* U+169CE */
  0x0020,   /* U+169CF */
  0x0020,   /* U+169D0 */
  0x0020,   /* U+169D1 */
  0x0020,   /* U+169D2 */
  0x0020,   /* U+169D3 */
  0x0020,   /* U+169D4 */
  0x0020,   /* U+169D5 */
  0x0020,   /* U+169D6 */
  0x0020,   /* U+169D7 */
  0x0020,   /* U+169D8 */
  0x0020,   /* U+169D9 */
  0x0020,   /* U+169DA */
  0x0020,   /* U+169DB */
  0x0020,   /* U+169DC */
  0x0020,   /* U+169DD */
  0x0020,   /* U+169DE */
  0x0020,   /* U+169DF */
  0x0020,   /* U+169E0 */
  0x0020,   /* U+169E1 */
  0x0020,   /* U+169E2 */
  0x0020,   /* U+169E3 */
  0x0020,   /* U+169E4 */
  0x0020,   /* U+169E5 */
  0x0020,   /* U+169E6 */
  0x0020,   /* U+169E7 */
  0x0020,   /* U+169E8 */
  0x0020,   /* U+169E9 */
  0x0020,   /* U+169EA */
  0x0020,   /* U+169EB */
  0x0020,   /* U+169EC */
  0x0020,   /* U+169ED */
  0x0020,   /* U+169EE */
  0x0020,   /* U+169EF */
  0x0020,   /* U+169F0 */
  0x0020,   /* U+169F1 */
  0x0020,   /* U+169F2 */
  0x0020,   /* U+169F3 */
  0x0020,   /* U+169F4 */
  0x0020,   /* U+169F5 */
  0x0020,   /* U+169F6 */
  0x0020,   /* U+169F7 */
  0x0020,   /* U+169F8 */
  0x0020,   /* U+169F9 */
  0x0020,   /* U+169FA */
  0x0020,   /* U+169FB */
  0x0020,   /* U+169FC */
  0x0020,   /* U+169FD */
  0x0020,   /* U+169FE */
  0x0020,   /* U+169FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+16900 */
  0x0002,   /* U+16901 */
  0x0002,   /* U+16902 */
  0x0002,   /* U+16903 */
  0x0002,   /* U+16904 */
  0x0002,   /* U+16905 */
  0x0002,   /* U+16906 */
  0x0002,   /* U+16907 */
  0x0002,   /* U+16908 */
  0x0002,   /* U+16909 */
  0x0002,   /* U+1690A */
  0x0002,   /* U+1690B */
  0x0002,   /* U+1690C */
  0x0002,   /* U+1690D */
  0x0002,   /* U+1690E */
  0x0002,   /* U+1690F */
  0x0002,   /* U+16910 */
  0x0002,   /* U+16911 */
  0x0002,   /* U+16912 */
  0x0002,   /* U+16913 */
  0x0002,   /* U+16914 */
  0x0002,   /* U+16915 */
  0x0002,   /* U+16916 */
  0x0002,   /* U+16917 */
  0x0002,   /* U+16918 */
  0x0002,   /* U+16919 */
  0x0002,   /* U+1691A */
  0x0002,   /* U+1691B */
  0x0002,   /* U+1691C */
  0x0002,   /* U+1691D */
  0x0002,   /* U+1691E */
  0x0002,   /* U+1691F */
  0x0002,   /* U+16920 */
  0x0002,   /* U+16921 */
  0x0002,   /* U+16922 */
  0x0002,   /* U+16923 */
  0x0002,   /* U+16924 */
  0x0002,   /* U+16925 */
  0x0002,   /* U+16926 */
  0x0002,   /* U+16927 */
  0x0002,   /* U+16928 */
  0x0002,   /* U+16929 */
  0x0002,   /* U+1692A */
  0x0002,   /* U+1692B */
  0x0002,   /* U+1692C */
  0x0002,   /* U+1692D */
  0x0002,   /* U+1692E */
  0x0002,   /* U+1692F */
  0x0002,   /* U+16930 */
  0x0002,   /* U+16931 */
  0x0002,   /* U+16932 */
  0x0002,   /* U+16933 */
  0x0002,   /* U+16934 */
  0x0002,   /* U+16935 */
  0x0002,   /* U+16936 */
  0x0002,   /* U+16937 */
  0x0002,   /* U+16938 */
  0x0002,   /* U+16939 */
  0x0002,   /* U+1693A */
  0x0002,   /* U+1693B */
  0x0002,   /* U+1693C */
  0x0002,   /* U+1693D */
  0x0002,   /* U+1693E */
  0x0002,   /* U+1693F */
  0x0002,   /* U+16940 */
  0x0002,   /* U+16941 */
  0x0002,   /* U+16942 */
  0x0002,   /* U+16943 */
  0x0002,   /* U+16944 */
  0x0002,   /* U+16945 */
  0x0002,   /* U+16946 */
  0x0002,   /* U+16947 */
  0x0002,   /* U+16948 */
  0x0002,   /* U+16949 */
  0x0002,   /* U+1694A */
  0x0002,   /* U+1694B */
  0x0002,   /* U+1694C */
  0x0002,   /* U+1694D */
  0x0002,   /* U+1694E */
  0x0002,   /* U+1694F */
  0x0002,   /* U+16950 */
  0x0002,   /* U+16951 */
  0x0002,   /* U+16952 */
  0x0002,   /* U+16953 */
  0x0002,   /* U+16954 */
  0x0002,   /* U+16955 */
  0x0002,   /* U+16956 */
  0x0002,   /* U+16957 */
  0x0002,   /* U+16958 */
  0x0002,   /* U+16959 */
  0x0002,   /* U+1695A */
  0x0002,   /* U+1695B */
  0x0002,   /* U+1695C */
  0x0002,   /* U+1695D */
  0x0002,   /* U+1695E */
  0x0002,   /* U+1695F */
  0x0002,   /* U+16960 */
  0x0002,   /* U+16961 */
  0x0002,   /* U+16962 */
  0x0002,   /* U+16963 */
  0x0002,   /* U+16964 */
  0x0002,   /* U+16965 */
  0x0002,   /* U+16966 */
  0x0002,   /* U+16967 */
  0x0002,   /* U+16968 */
  0x0002,   /* U+16969 */
  0x0002,   /* U+1696A */
  0x0002,   /* U+1696B */
  0x0002,   /* U+1696C */
  0x0002,   /* U+1696D */
  0x0002,   /* U+1696E */
  0x0002,   /* U+1696F */
  0x0002,   /* U+16970 */
  0x0002,   /* U+16971 */
  0x0002,   /* U+16972 */
  0x0002,   /* U+16973 */
  0x0002,   /* U+16974 */
  0x0002,   /* U+16975 */
  0x0002,   /* U+16976 */
  0x0002,   /* U+16977 */
  0x0002,   /* U+16978 */
  0x0002,   /* U+16979 */
  0x0002,   /* U+1697A */
  0x0002,   /* U+1697B */
  0x0002,   /* U+1697C */
  0x0002,   /* U+1697D */
  0x0002,   /* U+1697E */
  0x0002,   /* U+1697F */
  0x0002,   /* U+16980 */
  0x0002,   /* U+16981 */
  0x0002,   /* U+16982 */
  0x0002,   /* U+16983 */
  0x0002,   /* U+16984 */
  0x0002,   /* U+16985 */
  0x0002,   /* U+16986 */
  0x0002,   /* U+16987 */
  0x0002,   /* U+16988 */
  0x0002,   /* U+16989 */
  0x0002,   /* U+1698A */
  0x0002,   /* U+1698B */
  0x0002,   /* U+1698C */
  0x0002,   /* U+1698D */
  0x0002,   /* U+1698E */
  0x0002,   /* U+1698F */
  0x0002,   /* U+16990 */
  0x0002,   /* U+16991 */
  0x0002,   /* U+16992 */
  0x0002,   /* U+16993 */
  0x0002,   /* U+16994 */
  0x0002,   /* U+16995 */
  0x0002,   /* U+16996 */
  0x0002,   /* U+16997 */
  0x0002,   /* U+16998 */
  0x0002,   /* U+16999 */
  0x0002,   /* U+1699A */
  0x0002,   /* U+1699B */
  0x0002,   /* U+1699C */
  0x0002,   /* U+1699D */
  0x0002,   /* U+1699E */
  0x0002,   /* U+1699F */
  0x0002,   /* U+169A0 */
  0x0002,   /* U+169A1 */
  0x0002,   /* U+169A2 */
  0x0002,   /* U+169A3 */
  0x0002,   /* U+169A4 */
  0x0002,   /* U+169A5 */
  0x0002,   /* U+169A6 */
  0x0002,   /* U+169A7 */
  0x0002,   /* U+169A8 */
  0x0002,   /* U+169A9 */
  0x0002,   /* U+169AA */
  0x0002,   /* U+169AB */
  0x0002,   /* U+169AC */
  0x0002,   /* U+169AD */
  0x0002,   /* U+169AE */
  0x0002,   /* U+169AF */
  0x0002,   /* U+169B0 */
  0x0002,   /* U+169B1 */
  0x0002,   /* U+169B2 */
  0x0002,   /* U+169B3 */
  0x0002,   /* U+169B4 */
  0x0002,   /* U+169B5 */
  0x0002,   /* U+169B6 */
  0x0002,   /* U+169B7 */
  0x0002,   /* U+169B8 */
  0x0002,   /* U+169B9 */
  0x0002,   /* U+169BA */
  0x0002,   /* U+169BB */
  0x0002,   /* U+169BC */
  0x0002,   /* U+169BD */
  0x0002,   /* U+169BE */
  0x0002,   /* U+169BF */
  0x0002,   /* U+169C0 */
  0x0002,   /* U+169C1 */
  0x0002,   /* U+169C2 */
  0x0002,   /* U+169C3 */
  0x0002,   /* U+169C4 */
  0x0002,   /* U+169C5 */
  0x0002,   /* U+169C6 */
  0x0002,   /* U+169C7 */
  0x0002,   /* U+169C8 */
  0x0002,   /* U+169C9 */
  0x0002,   /* U+169CA */
  0x0002,   /* U+169CB */
  0x0002,   /* U+169CC */
  0x0002,   /* U+169CD */
  0x0002,   /* U+169CE */
  0x0002,   /* U+169CF */
  0x0002,   /* U+169D0 */
  0x0002,   /* U+169D1 */
  0x0002,   /* U+169D2 */
  0x0002,   /* U+169D3 */
  0x0002,   /* U+169D4 */
  0x0002,   /* U+169D5 */
  0x0002,   /* U+169D6 */
  0x0002,   /* U+169D7 */
  0x0002,   /* U+169D8 */
  0x0002,   /* U+169D9 */
  0x0002,   /* U+169DA */
  0x0002,   /* U+169DB */
  0x0002,   /* U+169DC */
  0x0002,   /* U+169DD */
  0x0002,   /* U+169DE */
  0x0002,   /* U+169DF */
  0x0002,   /* U+169E0 */
  0x0002,   /* U+169E1 */
  0x0002,   /* U+169E2 */
  0x0002,   /* U+169E3 */
  0x0002,   /* U+169E4 */
  0x0002,   /* U+169E5 */
  0x0002,   /* U+169E6 */
  0x0002,   /* U+169E7 */
  0x0002,   /* U+169E8 */
  0x0002,   /* U+169E9 */
  0x0002,   /* U+169EA */
  0x0002,   /* U+169EB */
  0x0002,   /* U+169EC */
  0x0002,   /* U+169ED */
  0x0002,   /* U+169EE */
  0x0002,   /* U+169EF */
  0x0002,   /* U+169F0 */
  0x0002,   /* U+169F1 */
  0x0002,   /* U+169F2 */
  0x0002,   /* U+169F3 */
  0x0002,   /* U+169F4 */
  0x0002,   /* U+169F5 */
  0x0002,   /* U+169F6 */
  0x0002,   /* U+169F7 */
  0x0002,   /* U+169F8 */
  0x0002,   /* U+169F9 */
  0x0002,   /* U+169FA */
  0x0002,   /* U+169FB */
  0x0002,   /* U+169FC */
  0x0002,   /* U+169FD */
  0x0002,   /* U+169FE */
  0x0002,   /* U+169FF */
};

uint16 uca900_p16A[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3AB7,   /* U+16A00 */
  0x3AB8,   /* U+16A01 */
  0x3AB9,   /* U+16A02 */
  0x3ABA,   /* U+16A03 */
  0x3ABB,   /* U+16A04 */
  0x3ABC,   /* U+16A05 */
  0x3ABD,   /* U+16A06 */
  0x3ABE,   /* U+16A07 */
  0x3ABF,   /* U+16A08 */
  0x3AC0,   /* U+16A09 */
  0x3AC1,   /* U+16A0A */
  0x3AC2,   /* U+16A0B */
  0x3AC3,   /* U+16A0C */
  0x3AC4,   /* U+16A0D */
  0x3AC5,   /* U+16A0E */
  0x3AC6,   /* U+16A0F */
  0x3AC7,   /* U+16A10 */
  0x3AC8,   /* U+16A11 */
  0x3AC9,   /* U+16A12 */
  0x3ACA,   /* U+16A13 */
  0x3ACB,   /* U+16A14 */
  0x3ACC,   /* U+16A15 */
  0x3ACD,   /* U+16A16 */
  0x3ACE,   /* U+16A17 */
  0x3ACF,   /* U+16A18 */
  0x3AD0,   /* U+16A19 */
  0x3AD1,   /* U+16A1A */
  0x3AD2,   /* U+16A1B */
  0x3AD3,   /* U+16A1C */
  0x3AD4,   /* U+16A1D */
  0x3AD5,   /* U+16A1E */
  0x3AD6,   /* U+16A1F */
  0x3AD7,   /* U+16A20 */
  0x3AD8,   /* U+16A21 */
  0x3AD9,   /* U+16A22 */
  0x3ADA,   /* U+16A23 */
  0x3ADB,   /* U+16A24 */
  0x3ADC,   /* U+16A25 */
  0x3ADD,   /* U+16A26 */
  0x3ADE,   /* U+16A27 */
  0x3ADF,   /* U+16A28 */
  0x3AE0,   /* U+16A29 */
  0x3AE1,   /* U+16A2A */
  0x3AE2,   /* U+16A2B */
  0x3AE3,   /* U+16A2C */
  0x3AE4,   /* U+16A2D */
  0x3AE5,   /* U+16A2E */
  0x3AE6,   /* U+16A2F */
  0x3AE7,   /* U+16A30 */
  0x3AE8,   /* U+16A31 */
  0x3AE9,   /* U+16A32 */
  0x3AEA,   /* U+16A33 */
  0x3AEB,   /* U+16A34 */
  0x3AEC,   /* U+16A35 */
  0x3AED,   /* U+16A36 */
  0x3AEE,   /* U+16A37 */
  0x3AEF,   /* U+16A38 */
  0xFBC2,   /* U+16A39 */
  0xFBC2,   /* U+16A3A */
  0xFBC2,   /* U+16A3B */
  0xFBC2,   /* U+16A3C */
  0xFBC2,   /* U+16A3D */
  0xFBC2,   /* U+16A3E */
  0xFBC2,   /* U+16A3F */
  0x45C8,   /* U+16A40 */
  0x45C9,   /* U+16A41 */
  0x45CA,   /* U+16A42 */
  0x45CB,   /* U+16A43 */
  0x45CC,   /* U+16A44 */
  0x45CD,   /* U+16A45 */
  0x45CE,   /* U+16A46 */
  0x45CF,   /* U+16A47 */
  0x45D0,   /* U+16A48 */
  0x45D1,   /* U+16A49 */
  0x45D2,   /* U+16A4A */
  0x45D3,   /* U+16A4B */
  0x45D4,   /* U+16A4C */
  0x45D5,   /* U+16A4D */
  0x45D6,   /* U+16A4E */
  0x45D7,   /* U+16A4F */
  0x45D8,   /* U+16A50 */
  0x45D9,   /* U+16A51 */
  0x45DA,   /* U+16A52 */
  0x45DB,   /* U+16A53 */
  0x45DC,   /* U+16A54 */
  0x45DD,   /* U+16A55 */
  0x45DE,   /* U+16A56 */
  0x45DF,   /* U+16A57 */
  0x45E0,   /* U+16A58 */
  0x45E1,   /* U+16A59 */
  0x45E2,   /* U+16A5A */
  0x45E3,   /* U+16A5B */
  0x45E4,   /* U+16A5C */
  0x45E5,   /* U+16A5D */
  0x45E6,   /* U+16A5E */
  0xFBC2,   /* U+16A5F */
  0x1C3D,   /* U+16A60 */
  0x1C3E,   /* U+16A61 */
  0x1C3F,   /* U+16A62 */
  0x1C40,   /* U+16A63 */
  0x1C41,   /* U+16A64 */
  0x1C42,   /* U+16A65 */
  0x1C43,   /* U+16A66 */
  0x1C44,   /* U+16A67 */
  0x1C45,   /* U+16A68 */
  0x1C46,   /* U+16A69 */
  0xFBC2,   /* U+16A6A */
  0xFBC2,   /* U+16A6B */
  0xFBC2,   /* U+16A6C */
  0xFBC2,   /* U+16A6D */
  0x02C0,   /* U+16A6E */
  0x02C1,   /* U+16A6F */
  0xFBC2,   /* U+16A70 */
  0xFBC2,   /* U+16A71 */
  0xFBC2,   /* U+16A72 */
  0xFBC2,   /* U+16A73 */
  0xFBC2,   /* U+16A74 */
  0xFBC2,   /* U+16A75 */
  0xFBC2,   /* U+16A76 */
  0xFBC2,   /* U+16A77 */
  0xFBC2,   /* U+16A78 */
  0xFBC2,   /* U+16A79 */
  0xFBC2,   /* U+16A7A */
  0xFBC2,   /* U+16A7B */
  0xFBC2,   /* U+16A7C */
  0xFBC2,   /* U+16A7D */
  0xFBC2,   /* U+16A7E */
  0xFBC2,   /* U+16A7F */
  0xFBC2,   /* U+16A80 */
  0xFBC2,   /* U+16A81 */
  0xFBC2,   /* U+16A82 */
  0xFBC2,   /* U+16A83 */
  0xFBC2,   /* U+16A84 */
  0xFBC2,   /* U+16A85 */
  0xFBC2,   /* U+16A86 */
  0xFBC2,   /* U+16A87 */
  0xFBC2,   /* U+16A88 */
  0xFBC2,   /* U+16A89 */
  0xFBC2,   /* U+16A8A */
  0xFBC2,   /* U+16A8B */
  0xFBC2,   /* U+16A8C */
  0xFBC2,   /* U+16A8D */
  0xFBC2,   /* U+16A8E */
  0xFBC2,   /* U+16A8F */
  0xFBC2,   /* U+16A90 */
  0xFBC2,   /* U+16A91 */
  0xFBC2,   /* U+16A92 */
  0xFBC2,   /* U+16A93 */
  0xFBC2,   /* U+16A94 */
  0xFBC2,   /* U+16A95 */
  0xFBC2,   /* U+16A96 */
  0xFBC2,   /* U+16A97 */
  0xFBC2,   /* U+16A98 */
  0xFBC2,   /* U+16A99 */
  0xFBC2,   /* U+16A9A */
  0xFBC2,   /* U+16A9B */
  0xFBC2,   /* U+16A9C */
  0xFBC2,   /* U+16A9D */
  0xFBC2,   /* U+16A9E */
  0xFBC2,   /* U+16A9F */
  0xFBC2,   /* U+16AA0 */
  0xFBC2,   /* U+16AA1 */
  0xFBC2,   /* U+16AA2 */
  0xFBC2,   /* U+16AA3 */
  0xFBC2,   /* U+16AA4 */
  0xFBC2,   /* U+16AA5 */
  0xFBC2,   /* U+16AA6 */
  0xFBC2,   /* U+16AA7 */
  0xFBC2,   /* U+16AA8 */
  0xFBC2,   /* U+16AA9 */
  0xFBC2,   /* U+16AAA */
  0xFBC2,   /* U+16AAB */
  0xFBC2,   /* U+16AAC */
  0xFBC2,   /* U+16AAD */
  0xFBC2,   /* U+16AAE */
  0xFBC2,   /* U+16AAF */
  0xFBC2,   /* U+16AB0 */
  0xFBC2,   /* U+16AB1 */
  0xFBC2,   /* U+16AB2 */
  0xFBC2,   /* U+16AB3 */
  0xFBC2,   /* U+16AB4 */
  0xFBC2,   /* U+16AB5 */
  0xFBC2,   /* U+16AB6 */
  0xFBC2,   /* U+16AB7 */
  0xFBC2,   /* U+16AB8 */
  0xFBC2,   /* U+16AB9 */
  0xFBC2,   /* U+16ABA */
  0xFBC2,   /* U+16ABB */
  0xFBC2,   /* U+16ABC */
  0xFBC2,   /* U+16ABD */
  0xFBC2,   /* U+16ABE */
  0xFBC2,   /* U+16ABF */
  0xFBC2,   /* U+16AC0 */
  0xFBC2,   /* U+16AC1 */
  0xFBC2,   /* U+16AC2 */
  0xFBC2,   /* U+16AC3 */
  0xFBC2,   /* U+16AC4 */
  0xFBC2,   /* U+16AC5 */
  0xFBC2,   /* U+16AC6 */
  0xFBC2,   /* U+16AC7 */
  0xFBC2,   /* U+16AC8 */
  0xFBC2,   /* U+16AC9 */
  0xFBC2,   /* U+16ACA */
  0xFBC2,   /* U+16ACB */
  0xFBC2,   /* U+16ACC */
  0xFBC2,   /* U+16ACD */
  0xFBC2,   /* U+16ACE */
  0xFBC2,   /* U+16ACF */
  0x3AF0,   /* U+16AD0 */
  0x3AF1,   /* U+16AD1 */
  0x3AF2,   /* U+16AD2 */
  0x3AF3,   /* U+16AD3 */
  0x3AF4,   /* U+16AD4 */
  0x3AF5,   /* U+16AD5 */
  0x3AF6,   /* U+16AD6 */
  0x3AF7,   /* U+16AD7 */
  0x3AF8,   /* U+16AD8 */
  0x3AF9,   /* U+16AD9 */
  0x3AFA,   /* U+16ADA */
  0x3AFB,   /* U+16ADB */
  0x3AFC,   /* U+16ADC */
  0x3AFD,   /* U+16ADD */
  0x3AFE,   /* U+16ADE */
  0x3AFF,   /* U+16ADF */
  0x3B00,   /* U+16AE0 */
  0x3B01,   /* U+16AE1 */
  0x3B02,   /* U+16AE2 */
  0x3B03,   /* U+16AE3 */
  0x3B04,   /* U+16AE4 */
  0x3B05,   /* U+16AE5 */
  0x3B06,   /* U+16AE6 */
  0x3B07,   /* U+16AE7 */
  0x3B08,   /* U+16AE8 */
  0x3B09,   /* U+16AE9 */
  0x3B0A,   /* U+16AEA */
  0x3B0B,   /* U+16AEB */
  0x3B0C,   /* U+16AEC */
  0x3B0D,   /* U+16AED */
  0xFBC2,   /* U+16AEE */
  0xFBC2,   /* U+16AEF */
  0x0000,   /* U+16AF0 */
  0x0000,   /* U+16AF1 */
  0x0000,   /* U+16AF2 */
  0x0000,   /* U+16AF3 */
  0x0000,   /* U+16AF4 */
  0x0288,   /* U+16AF5 */
  0xFBC2,   /* U+16AF6 */
  0xFBC2,   /* U+16AF7 */
  0xFBC2,   /* U+16AF8 */
  0xFBC2,   /* U+16AF9 */
  0xFBC2,   /* U+16AFA */
  0xFBC2,   /* U+16AFB */
  0xFBC2,   /* U+16AFC */
  0xFBC2,   /* U+16AFD */
  0xFBC2,   /* U+16AFE */
  0xFBC2,   /* U+16AFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+16A00 */
  0x0020,   /* U+16A01 */
  0x0020,   /* U+16A02 */
  0x0020,   /* U+16A03 */
  0x0020,   /* U+16A04 */
  0x0020,   /* U+16A05 */
  0x0020,   /* U+16A06 */
  0x0020,   /* U+16A07 */
  0x0020,   /* U+16A08 */
  0x0020,   /* U+16A09 */
  0x0020,   /* U+16A0A */
  0x0020,   /* U+16A0B */
  0x0020,   /* U+16A0C */
  0x0020,   /* U+16A0D */
  0x0020,   /* U+16A0E */
  0x0020,   /* U+16A0F */
  0x0020,   /* U+16A10 */
  0x0020,   /* U+16A11 */
  0x0020,   /* U+16A12 */
  0x0020,   /* U+16A13 */
  0x0020,   /* U+16A14 */
  0x0020,   /* U+16A15 */
  0x0020,   /* U+16A16 */
  0x0020,   /* U+16A17 */
  0x0020,   /* U+16A18 */
  0x0020,   /* U+16A19 */
  0x0020,   /* U+16A1A */
  0x0020,   /* U+16A1B */
  0x0020,   /* U+16A1C */
  0x0020,   /* U+16A1D */
  0x0020,   /* U+16A1E */
  0x0020,   /* U+16A1F */
  0x0020,   /* U+16A20 */
  0x0020,   /* U+16A21 */
  0x0020,   /* U+16A22 */
  0x0020,   /* U+16A23 */
  0x0020,   /* U+16A24 */
  0x0020,   /* U+16A25 */
  0x0020,   /* U+16A26 */
  0x0020,   /* U+16A27 */
  0x0020,   /* U+16A28 */
  0x0020,   /* U+16A29 */
  0x0020,   /* U+16A2A */
  0x0020,   /* U+16A2B */
  0x0020,   /* U+16A2C */
  0x0020,   /* U+16A2D */
  0x0020,   /* U+16A2E */
  0x0020,   /* U+16A2F */
  0x0020,   /* U+16A30 */
  0x0020,   /* U+16A31 */
  0x0020,   /* U+16A32 */
  0x0020,   /* U+16A33 */
  0x0020,   /* U+16A34 */
  0x0020,   /* U+16A35 */
  0x0020,   /* U+16A36 */
  0x0020,   /* U+16A37 */
  0x0020,   /* U+16A38 */
  0x0020,   /* U+16A39 */
  0x0020,   /* U+16A3A */
  0x0020,   /* U+16A3B */
  0x0020,   /* U+16A3C */
  0x0020,   /* U+16A3D */
  0x0020,   /* U+16A3E */
  0x0020,   /* U+16A3F */
  0x0020,   /* U+16A40 */
  0x0020,   /* U+16A41 */
  0x0020,   /* U+16A42 */
  0x0020,   /* U+16A43 */
  0x0020,   /* U+16A44 */
  0x0020,   /* U+16A45 */
  0x0020,   /* U+16A46 */
  0x0020,   /* U+16A47 */
  0x0020,   /* U+16A48 */
  0x0020,   /* U+16A49 */
  0x0020,   /* U+16A4A */
  0x0020,   /* U+16A4B */
  0x0020,   /* U+16A4C */
  0x0020,   /* U+16A4D */
  0x0020,   /* U+16A4E */
  0x0020,   /* U+16A4F */
  0x0020,   /* U+16A50 */
  0x0020,   /* U+16A51 */
  0x0020,   /* U+16A52 */
  0x0020,   /* U+16A53 */
  0x0020,   /* U+16A54 */
  0x0020,   /* U+16A55 */
  0x0020,   /* U+16A56 */
  0x0020,   /* U+16A57 */
  0x0020,   /* U+16A58 */
  0x0020,   /* U+16A59 */
  0x0020,   /* U+16A5A */
  0x0020,   /* U+16A5B */
  0x0020,   /* U+16A5C */
  0x0020,   /* U+16A5D */
  0x0020,   /* U+16A5E */
  0x0020,   /* U+16A5F */
  0x0020,   /* U+16A60 */
  0x0020,   /* U+16A61 */
  0x0020,   /* U+16A62 */
  0x0020,   /* U+16A63 */
  0x0020,   /* U+16A64 */
  0x0020,   /* U+16A65 */
  0x0020,   /* U+16A66 */
  0x0020,   /* U+16A67 */
  0x0020,   /* U+16A68 */
  0x0020,   /* U+16A69 */
  0x0020,   /* U+16A6A */
  0x0020,   /* U+16A6B */
  0x0020,   /* U+16A6C */
  0x0020,   /* U+16A6D */
  0x0020,   /* U+16A6E */
  0x0020,   /* U+16A6F */
  0x0020,   /* U+16A70 */
  0x0020,   /* U+16A71 */
  0x0020,   /* U+16A72 */
  0x0020,   /* U+16A73 */
  0x0020,   /* U+16A74 */
  0x0020,   /* U+16A75 */
  0x0020,   /* U+16A76 */
  0x0020,   /* U+16A77 */
  0x0020,   /* U+16A78 */
  0x0020,   /* U+16A79 */
  0x0020,   /* U+16A7A */
  0x0020,   /* U+16A7B */
  0x0020,   /* U+16A7C */
  0x0020,   /* U+16A7D */
  0x0020,   /* U+16A7E */
  0x0020,   /* U+16A7F */
  0x0020,   /* U+16A80 */
  0x0020,   /* U+16A81 */
  0x0020,   /* U+16A82 */
  0x0020,   /* U+16A83 */
  0x0020,   /* U+16A84 */
  0x0020,   /* U+16A85 */
  0x0020,   /* U+16A86 */
  0x0020,   /* U+16A87 */
  0x0020,   /* U+16A88 */
  0x0020,   /* U+16A89 */
  0x0020,   /* U+16A8A */
  0x0020,   /* U+16A8B */
  0x0020,   /* U+16A8C */
  0x0020,   /* U+16A8D */
  0x0020,   /* U+16A8E */
  0x0020,   /* U+16A8F */
  0x0020,   /* U+16A90 */
  0x0020,   /* U+16A91 */
  0x0020,   /* U+16A92 */
  0x0020,   /* U+16A93 */
  0x0020,   /* U+16A94 */
  0x0020,   /* U+16A95 */
  0x0020,   /* U+16A96 */
  0x0020,   /* U+16A97 */
  0x0020,   /* U+16A98 */
  0x0020,   /* U+16A99 */
  0x0020,   /* U+16A9A */
  0x0020,   /* U+16A9B */
  0x0020,   /* U+16A9C */
  0x0020,   /* U+16A9D */
  0x0020,   /* U+16A9E */
  0x0020,   /* U+16A9F */
  0x0020,   /* U+16AA0 */
  0x0020,   /* U+16AA1 */
  0x0020,   /* U+16AA2 */
  0x0020,   /* U+16AA3 */
  0x0020,   /* U+16AA4 */
  0x0020,   /* U+16AA5 */
  0x0020,   /* U+16AA6 */
  0x0020,   /* U+16AA7 */
  0x0020,   /* U+16AA8 */
  0x0020,   /* U+16AA9 */
  0x0020,   /* U+16AAA */
  0x0020,   /* U+16AAB */
  0x0020,   /* U+16AAC */
  0x0020,   /* U+16AAD */
  0x0020,   /* U+16AAE */
  0x0020,   /* U+16AAF */
  0x0020,   /* U+16AB0 */
  0x0020,   /* U+16AB1 */
  0x0020,   /* U+16AB2 */
  0x0020,   /* U+16AB3 */
  0x0020,   /* U+16AB4 */
  0x0020,   /* U+16AB5 */
  0x0020,   /* U+16AB6 */
  0x0020,   /* U+16AB7 */
  0x0020,   /* U+16AB8 */
  0x0020,   /* U+16AB9 */
  0x0020,   /* U+16ABA */
  0x0020,   /* U+16ABB */
  0x0020,   /* U+16ABC */
  0x0020,   /* U+16ABD */
  0x0020,   /* U+16ABE */
  0x0020,   /* U+16ABF */
  0x0020,   /* U+16AC0 */
  0x0020,   /* U+16AC1 */
  0x0020,   /* U+16AC2 */
  0x0020,   /* U+16AC3 */
  0x0020,   /* U+16AC4 */
  0x0020,   /* U+16AC5 */
  0x0020,   /* U+16AC6 */
  0x0020,   /* U+16AC7 */
  0x0020,   /* U+16AC8 */
  0x0020,   /* U+16AC9 */
  0x0020,   /* U+16ACA */
  0x0020,   /* U+16ACB */
  0x0020,   /* U+16ACC */
  0x0020,   /* U+16ACD */
  0x0020,   /* U+16ACE */
  0x0020,   /* U+16ACF */
  0x0020,   /* U+16AD0 */
  0x0020,   /* U+16AD1 */
  0x0020,   /* U+16AD2 */
  0x0020,   /* U+16AD3 */
  0x0020,   /* U+16AD4 */
  0x0020,   /* U+16AD5 */
  0x0020,   /* U+16AD6 */
  0x0020,   /* U+16AD7 */
  0x0020,   /* U+16AD8 */
  0x0020,   /* U+16AD9 */
  0x0020,   /* U+16ADA */
  0x0020,   /* U+16ADB */
  0x0020,   /* U+16ADC */
  0x0020,   /* U+16ADD */
  0x0020,   /* U+16ADE */
  0x0020,   /* U+16ADF */
  0x0020,   /* U+16AE0 */
  0x0020,   /* U+16AE1 */
  0x0020,   /* U+16AE2 */
  0x0020,   /* U+16AE3 */
  0x0020,   /* U+16AE4 */
  0x0020,   /* U+16AE5 */
  0x0020,   /* U+16AE6 */
  0x0020,   /* U+16AE7 */
  0x0020,   /* U+16AE8 */
  0x0020,   /* U+16AE9 */
  0x0020,   /* U+16AEA */
  0x0020,   /* U+16AEB */
  0x0020,   /* U+16AEC */
  0x0020,   /* U+16AED */
  0x0020,   /* U+16AEE */
  0x0020,   /* U+16AEF */
  0x00B8,   /* U+16AF0 */
  0x00B9,   /* U+16AF1 */
  0x00BA,   /* U+16AF2 */
  0x00BB,   /* U+16AF3 */
  0x00BC,   /* U+16AF4 */
  0x0020,   /* U+16AF5 */
  0x0020,   /* U+16AF6 */
  0x0020,   /* U+16AF7 */
  0x0020,   /* U+16AF8 */
  0x0020,   /* U+16AF9 */
  0x0020,   /* U+16AFA */
  0x0020,   /* U+16AFB */
  0x0020,   /* U+16AFC */
  0x0020,   /* U+16AFD */
  0x0020,   /* U+16AFE */
  0x0020,   /* U+16AFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+16A00 */
  0x0002,   /* U+16A01 */
  0x0002,   /* U+16A02 */
  0x0002,   /* U+16A03 */
  0x0002,   /* U+16A04 */
  0x0002,   /* U+16A05 */
  0x0002,   /* U+16A06 */
  0x0002,   /* U+16A07 */
  0x0002,   /* U+16A08 */
  0x0002,   /* U+16A09 */
  0x0002,   /* U+16A0A */
  0x0002,   /* U+16A0B */
  0x0002,   /* U+16A0C */
  0x0002,   /* U+16A0D */
  0x0002,   /* U+16A0E */
  0x0002,   /* U+16A0F */
  0x0002,   /* U+16A10 */
  0x0002,   /* U+16A11 */
  0x0002,   /* U+16A12 */
  0x0002,   /* U+16A13 */
  0x0002,   /* U+16A14 */
  0x0002,   /* U+16A15 */
  0x0002,   /* U+16A16 */
  0x0002,   /* U+16A17 */
  0x0002,   /* U+16A18 */
  0x0002,   /* U+16A19 */
  0x0002,   /* U+16A1A */
  0x0002,   /* U+16A1B */
  0x0002,   /* U+16A1C */
  0x0002,   /* U+16A1D */
  0x0002,   /* U+16A1E */
  0x0002,   /* U+16A1F */
  0x0002,   /* U+16A20 */
  0x0002,   /* U+16A21 */
  0x0002,   /* U+16A22 */
  0x0002,   /* U+16A23 */
  0x0002,   /* U+16A24 */
  0x0002,   /* U+16A25 */
  0x0002,   /* U+16A26 */
  0x0002,   /* U+16A27 */
  0x0002,   /* U+16A28 */
  0x0002,   /* U+16A29 */
  0x0002,   /* U+16A2A */
  0x0002,   /* U+16A2B */
  0x0002,   /* U+16A2C */
  0x0002,   /* U+16A2D */
  0x0002,   /* U+16A2E */
  0x0002,   /* U+16A2F */
  0x0002,   /* U+16A30 */
  0x0002,   /* U+16A31 */
  0x0002,   /* U+16A32 */
  0x0002,   /* U+16A33 */
  0x0002,   /* U+16A34 */
  0x0002,   /* U+16A35 */
  0x0002,   /* U+16A36 */
  0x0002,   /* U+16A37 */
  0x0002,   /* U+16A38 */
  0x0002,   /* U+16A39 */
  0x0002,   /* U+16A3A */
  0x0002,   /* U+16A3B */
  0x0002,   /* U+16A3C */
  0x0002,   /* U+16A3D */
  0x0002,   /* U+16A3E */
  0x0002,   /* U+16A3F */
  0x0002,   /* U+16A40 */
  0x0002,   /* U+16A41 */
  0x0002,   /* U+16A42 */
  0x0002,   /* U+16A43 */
  0x0002,   /* U+16A44 */
  0x0002,   /* U+16A45 */
  0x0002,   /* U+16A46 */
  0x0002,   /* U+16A47 */
  0x0002,   /* U+16A48 */
  0x0002,   /* U+16A49 */
  0x0002,   /* U+16A4A */
  0x0002,   /* U+16A4B */
  0x0002,   /* U+16A4C */
  0x0002,   /* U+16A4D */
  0x0002,   /* U+16A4E */
  0x0002,   /* U+16A4F */
  0x0002,   /* U+16A50 */
  0x0002,   /* U+16A51 */
  0x0002,   /* U+16A52 */
  0x0002,   /* U+16A53 */
  0x0002,   /* U+16A54 */
  0x0002,   /* U+16A55 */
  0x0002,   /* U+16A56 */
  0x0002,   /* U+16A57 */
  0x0002,   /* U+16A58 */
  0x0002,   /* U+16A59 */
  0x0002,   /* U+16A5A */
  0x0002,   /* U+16A5B */
  0x0002,   /* U+16A5C */
  0x0002,   /* U+16A5D */
  0x0002,   /* U+16A5E */
  0x0002,   /* U+16A5F */
  0x0002,   /* U+16A60 */
  0x0002,   /* U+16A61 */
  0x0002,   /* U+16A62 */
  0x0002,   /* U+16A63 */
  0x0002,   /* U+16A64 */
  0x0002,   /* U+16A65 */
  0x0002,   /* U+16A66 */
  0x0002,   /* U+16A67 */
  0x0002,   /* U+16A68 */
  0x0002,   /* U+16A69 */
  0x0002,   /* U+16A6A */
  0x0002,   /* U+16A6B */
  0x0002,   /* U+16A6C */
  0x0002,   /* U+16A6D */
  0x0002,   /* U+16A6E */
  0x0002,   /* U+16A6F */
  0x0002,   /* U+16A70 */
  0x0002,   /* U+16A71 */
  0x0002,   /* U+16A72 */
  0x0002,   /* U+16A73 */
  0x0002,   /* U+16A74 */
  0x0002,   /* U+16A75 */
  0x0002,   /* U+16A76 */
  0x0002,   /* U+16A77 */
  0x0002,   /* U+16A78 */
  0x0002,   /* U+16A79 */
  0x0002,   /* U+16A7A */
  0x0002,   /* U+16A7B */
  0x0002,   /* U+16A7C */
  0x0002,   /* U+16A7D */
  0x0002,   /* U+16A7E */
  0x0002,   /* U+16A7F */
  0x0002,   /* U+16A80 */
  0x0002,   /* U+16A81 */
  0x0002,   /* U+16A82 */
  0x0002,   /* U+16A83 */
  0x0002,   /* U+16A84 */
  0x0002,   /* U+16A85 */
  0x0002,   /* U+16A86 */
  0x0002,   /* U+16A87 */
  0x0002,   /* U+16A88 */
  0x0002,   /* U+16A89 */
  0x0002,   /* U+16A8A */
  0x0002,   /* U+16A8B */
  0x0002,   /* U+16A8C */
  0x0002,   /* U+16A8D */
  0x0002,   /* U+16A8E */
  0x0002,   /* U+16A8F */
  0x0002,   /* U+16A90 */
  0x0002,   /* U+16A91 */
  0x0002,   /* U+16A92 */
  0x0002,   /* U+16A93 */
  0x0002,   /* U+16A94 */
  0x0002,   /* U+16A95 */
  0x0002,   /* U+16A96 */
  0x0002,   /* U+16A97 */
  0x0002,   /* U+16A98 */
  0x0002,   /* U+16A99 */
  0x0002,   /* U+16A9A */
  0x0002,   /* U+16A9B */
  0x0002,   /* U+16A9C */
  0x0002,   /* U+16A9D */
  0x0002,   /* U+16A9E */
  0x0002,   /* U+16A9F */
  0x0002,   /* U+16AA0 */
  0x0002,   /* U+16AA1 */
  0x0002,   /* U+16AA2 */
  0x0002,   /* U+16AA3 */
  0x0002,   /* U+16AA4 */
  0x0002,   /* U+16AA5 */
  0x0002,   /* U+16AA6 */
  0x0002,   /* U+16AA7 */
  0x0002,   /* U+16AA8 */
  0x0002,   /* U+16AA9 */
  0x0002,   /* U+16AAA */
  0x0002,   /* U+16AAB */
  0x0002,   /* U+16AAC */
  0x0002,   /* U+16AAD */
  0x0002,   /* U+16AAE */
  0x0002,   /* U+16AAF */
  0x0002,   /* U+16AB0 */
  0x0002,   /* U+16AB1 */
  0x0002,   /* U+16AB2 */
  0x0002,   /* U+16AB3 */
  0x0002,   /* U+16AB4 */
  0x0002,   /* U+16AB5 */
  0x0002,   /* U+16AB6 */
  0x0002,   /* U+16AB7 */
  0x0002,   /* U+16AB8 */
  0x0002,   /* U+16AB9 */
  0x0002,   /* U+16ABA */
  0x0002,   /* U+16ABB */
  0x0002,   /* U+16ABC */
  0x0002,   /* U+16ABD */
  0x0002,   /* U+16ABE */
  0x0002,   /* U+16ABF */
  0x0002,   /* U+16AC0 */
  0x0002,   /* U+16AC1 */
  0x0002,   /* U+16AC2 */
  0x0002,   /* U+16AC3 */
  0x0002,   /* U+16AC4 */
  0x0002,   /* U+16AC5 */
  0x0002,   /* U+16AC6 */
  0x0002,   /* U+16AC7 */
  0x0002,   /* U+16AC8 */
  0x0002,   /* U+16AC9 */
  0x0002,   /* U+16ACA */
  0x0002,   /* U+16ACB */
  0x0002,   /* U+16ACC */
  0x0002,   /* U+16ACD */
  0x0002,   /* U+16ACE */
  0x0002,   /* U+16ACF */
  0x0002,   /* U+16AD0 */
  0x0002,   /* U+16AD1 */
  0x0002,   /* U+16AD2 */
  0x0002,   /* U+16AD3 */
  0x0002,   /* U+16AD4 */
  0x0002,   /* U+16AD5 */
  0x0002,   /* U+16AD6 */
  0x0002,   /* U+16AD7 */
  0x0002,   /* U+16AD8 */
  0x0002,   /* U+16AD9 */
  0x0002,   /* U+16ADA */
  0x0002,   /* U+16ADB */
  0x0002,   /* U+16ADC */
  0x0002,   /* U+16ADD */
  0x0002,   /* U+16ADE */
  0x0002,   /* U+16ADF */
  0x0002,   /* U+16AE0 */
  0x0002,   /* U+16AE1 */
  0x0002,   /* U+16AE2 */
  0x0002,   /* U+16AE3 */
  0x0002,   /* U+16AE4 */
  0x0002,   /* U+16AE5 */
  0x0002,   /* U+16AE6 */
  0x0002,   /* U+16AE7 */
  0x0002,   /* U+16AE8 */
  0x0002,   /* U+16AE9 */
  0x0002,   /* U+16AEA */
  0x0002,   /* U+16AEB */
  0x0002,   /* U+16AEC */
  0x0002,   /* U+16AED */
  0x0002,   /* U+16AEE */
  0x0002,   /* U+16AEF */
  0x0002,   /* U+16AF0 */
  0x0002,   /* U+16AF1 */
  0x0002,   /* U+16AF2 */
  0x0002,   /* U+16AF3 */
  0x0002,   /* U+16AF4 */
  0x0002,   /* U+16AF5 */
  0x0002,   /* U+16AF6 */
  0x0002,   /* U+16AF7 */
  0x0002,   /* U+16AF8 */
  0x0002,   /* U+16AF9 */
  0x0002,   /* U+16AFA */
  0x0002,   /* U+16AFB */
  0x0002,   /* U+16AFC */
  0x0002,   /* U+16AFD */
  0x0002,   /* U+16AFE */
  0x0002,   /* U+16AFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+16A00 */
  0x0000,   /* U+16A01 */
  0x0000,   /* U+16A02 */
  0x0000,   /* U+16A03 */
  0x0000,   /* U+16A04 */
  0x0000,   /* U+16A05 */
  0x0000,   /* U+16A06 */
  0x0000,   /* U+16A07 */
  0x0000,   /* U+16A08 */
  0x0000,   /* U+16A09 */
  0x0000,   /* U+16A0A */
  0x0000,   /* U+16A0B */
  0x0000,   /* U+16A0C */
  0x0000,   /* U+16A0D */
  0x0000,   /* U+16A0E */
  0x0000,   /* U+16A0F */
  0x0000,   /* U+16A10 */
  0x0000,   /* U+16A11 */
  0x0000,   /* U+16A12 */
  0x0000,   /* U+16A13 */
  0x0000,   /* U+16A14 */
  0x0000,   /* U+16A15 */
  0x0000,   /* U+16A16 */
  0x0000,   /* U+16A17 */
  0x0000,   /* U+16A18 */
  0x0000,   /* U+16A19 */
  0x0000,   /* U+16A1A */
  0x0000,   /* U+16A1B */
  0x0000,   /* U+16A1C */
  0x0000,   /* U+16A1D */
  0x0000,   /* U+16A1E */
  0x0000,   /* U+16A1F */
  0x0000,   /* U+16A20 */
  0x0000,   /* U+16A21 */
  0x0000,   /* U+16A22 */
  0x0000,   /* U+16A23 */
  0x0000,   /* U+16A24 */
  0x0000,   /* U+16A25 */
  0x0000,   /* U+16A26 */
  0x0000,   /* U+16A27 */
  0x0000,   /* U+16A28 */
  0x0000,   /* U+16A29 */
  0x0000,   /* U+16A2A */
  0x0000,   /* U+16A2B */
  0x0000,   /* U+16A2C */
  0x0000,   /* U+16A2D */
  0x0000,   /* U+16A2E */
  0x0000,   /* U+16A2F */
  0x0000,   /* U+16A30 */
  0x0000,   /* U+16A31 */
  0x0000,   /* U+16A32 */
  0x0000,   /* U+16A33 */
  0x0000,   /* U+16A34 */
  0x0000,   /* U+16A35 */
  0x0000,   /* U+16A36 */
  0x0000,   /* U+16A37 */
  0x0000,   /* U+16A38 */
  0xEA39,   /* U+16A39 */
  0xEA3A,   /* U+16A3A */
  0xEA3B,   /* U+16A3B */
  0xEA3C,   /* U+16A3C */
  0xEA3D,   /* U+16A3D */
  0xEA3E,   /* U+16A3E */
  0xEA3F,   /* U+16A3F */
  0x0000,   /* U+16A40 */
  0x0000,   /* U+16A41 */
  0x0000,   /* U+16A42 */
  0x0000,   /* U+16A43 */
  0x0000,   /* U+16A44 */
  0x0000,   /* U+16A45 */
  0x0000,   /* U+16A46 */
  0x0000,   /* U+16A47 */
  0x0000,   /* U+16A48 */
  0x0000,   /* U+16A49 */
  0x0000,   /* U+16A4A */
  0x0000,   /* U+16A4B */
  0x0000,   /* U+16A4C */
  0x0000,   /* U+16A4D */
  0x0000,   /* U+16A4E */
  0x0000,   /* U+16A4F */
  0x0000,   /* U+16A50 */
  0x0000,   /* U+16A51 */
  0x0000,   /* U+16A52 */
  0x0000,   /* U+16A53 */
  0x0000,   /* U+16A54 */
  0x0000,   /* U+16A55 */
  0x0000,   /* U+16A56 */
  0x0000,   /* U+16A57 */
  0x0000,   /* U+16A58 */
  0x0000,   /* U+16A59 */
  0x0000,   /* U+16A5A */
  0x0000,   /* U+16A5B */
  0x0000,   /* U+16A5C */
  0x0000,   /* U+16A5D */
  0x0000,   /* U+16A5E */
  0xEA5F,   /* U+16A5F */
  0x0000,   /* U+16A60 */
  0x0000,   /* U+16A61 */
  0x0000,   /* U+16A62 */
  0x0000,   /* U+16A63 */
  0x0000,   /* U+16A64 */
  0x0000,   /* U+16A65 */
  0x0000,   /* U+16A66 */
  0x0000,   /* U+16A67 */
  0x0000,   /* U+16A68 */
  0x0000,   /* U+16A69 */
  0xEA6A,   /* U+16A6A */
  0xEA6B,   /* U+16A6B */
  0xEA6C,   /* U+16A6C */
  0xEA6D,   /* U+16A6D */
  0x0000,   /* U+16A6E */
  0x0000,   /* U+16A6F */
  0xEA70,   /* U+16A70 */
  0xEA71,   /* U+16A71 */
  0xEA72,   /* U+16A72 */
  0xEA73,   /* U+16A73 */
  0xEA74,   /* U+16A74 */
  0xEA75,   /* U+16A75 */
  0xEA76,   /* U+16A76 */
  0xEA77,   /* U+16A77 */
  0xEA78,   /* U+16A78 */
  0xEA79,   /* U+16A79 */
  0xEA7A,   /* U+16A7A */
  0xEA7B,   /* U+16A7B */
  0xEA7C,   /* U+16A7C */
  0xEA7D,   /* U+16A7D */
  0xEA7E,   /* U+16A7E */
  0xEA7F,   /* U+16A7F */
  0xEA80,   /* U+16A80 */
  0xEA81,   /* U+16A81 */
  0xEA82,   /* U+16A82 */
  0xEA83,   /* U+16A83 */
  0xEA84,   /* U+16A84 */
  0xEA85,   /* U+16A85 */
  0xEA86,   /* U+16A86 */
  0xEA87,   /* U+16A87 */
  0xEA88,   /* U+16A88 */
  0xEA89,   /* U+16A89 */
  0xEA8A,   /* U+16A8A */
  0xEA8B,   /* U+16A8B */
  0xEA8C,   /* U+16A8C */
  0xEA8D,   /* U+16A8D */
  0xEA8E,   /* U+16A8E */
  0xEA8F,   /* U+16A8F */
  0xEA90,   /* U+16A90 */
  0xEA91,   /* U+16A91 */
  0xEA92,   /* U+16A92 */
  0xEA93,   /* U+16A93 */
  0xEA94,   /* U+16A94 */
  0xEA95,   /* U+16A95 */
  0xEA96,   /* U+16A96 */
  0xEA97,   /* U+16A97 */
  0xEA98,   /* U+16A98 */
  0xEA99,   /* U+16A99 */
  0xEA9A,   /* U+16A9A */
  0xEA9B,   /* U+16A9B */
  0xEA9C,   /* U+16A9C */
  0xEA9D,   /* U+16A9D */
  0xEA9E,   /* U+16A9E */
  0xEA9F,   /* U+16A9F */
  0xEAA0,   /* U+16AA0 */
  0xEAA1,   /* U+16AA1 */
  0xEAA2,   /* U+16AA2 */
  0xEAA3,   /* U+16AA3 */
  0xEAA4,   /* U+16AA4 */
  0xEAA5,   /* U+16AA5 */
  0xEAA6,   /* U+16AA6 */
  0xEAA7,   /* U+16AA7 */
  0xEAA8,   /* U+16AA8 */
  0xEAA9,   /* U+16AA9 */
  0xEAAA,   /* U+16AAA */
  0xEAAB,   /* U+16AAB */
  0xEAAC,   /* U+16AAC */
  0xEAAD,   /* U+16AAD */
  0xEAAE,   /* U+16AAE */
  0xEAAF,   /* U+16AAF */
  0xEAB0,   /* U+16AB0 */
  0xEAB1,   /* U+16AB1 */
  0xEAB2,   /* U+16AB2 */
  0xEAB3,   /* U+16AB3 */
  0xEAB4,   /* U+16AB4 */
  0xEAB5,   /* U+16AB5 */
  0xEAB6,   /* U+16AB6 */
  0xEAB7,   /* U+16AB7 */
  0xEAB8,   /* U+16AB8 */
  0xEAB9,   /* U+16AB9 */
  0xEABA,   /* U+16ABA */
  0xEABB,   /* U+16ABB */
  0xEABC,   /* U+16ABC */
  0xEABD,   /* U+16ABD */
  0xEABE,   /* U+16ABE */
  0xEABF,   /* U+16ABF */
  0xEAC0,   /* U+16AC0 */
  0xEAC1,   /* U+16AC1 */
  0xEAC2,   /* U+16AC2 */
  0xEAC3,   /* U+16AC3 */
  0xEAC4,   /* U+16AC4 */
  0xEAC5,   /* U+16AC5 */
  0xEAC6,   /* U+16AC6 */
  0xEAC7,   /* U+16AC7 */
  0xEAC8,   /* U+16AC8 */
  0xEAC9,   /* U+16AC9 */
  0xEACA,   /* U+16ACA */
  0xEACB,   /* U+16ACB */
  0xEACC,   /* U+16ACC */
  0xEACD,   /* U+16ACD */
  0xEACE,   /* U+16ACE */
  0xEACF,   /* U+16ACF */
  0x0000,   /* U+16AD0 */
  0x0000,   /* U+16AD1 */
  0x0000,   /* U+16AD2 */
  0x0000,   /* U+16AD3 */
  0x0000,   /* U+16AD4 */
  0x0000,   /* U+16AD5 */
  0x0000,   /* U+16AD6 */
  0x0000,   /* U+16AD7 */
  0x0000,   /* U+16AD8 */
  0x0000,   /* U+16AD9 */
  0x0000,   /* U+16ADA */
  0x0000,   /* U+16ADB */
  0x0000,   /* U+16ADC */
  0x0000,   /* U+16ADD */
  0x0000,   /* U+16ADE */
  0x0000,   /* U+16ADF */
  0x0000,   /* U+16AE0 */
  0x0000,   /* U+16AE1 */
  0x0000,   /* U+16AE2 */
  0x0000,   /* U+16AE3 */
  0x0000,   /* U+16AE4 */
  0x0000,   /* U+16AE5 */
  0x0000,   /* U+16AE6 */
  0x0000,   /* U+16AE7 */
  0x0000,   /* U+16AE8 */
  0x0000,   /* U+16AE9 */
  0x0000,   /* U+16AEA */
  0x0000,   /* U+16AEB */
  0x0000,   /* U+16AEC */
  0x0000,   /* U+16AED */
  0xEAEE,   /* U+16AEE */
  0xEAEF,   /* U+16AEF */
  0x0000,   /* U+16AF0 */
  0x0000,   /* U+16AF1 */
  0x0000,   /* U+16AF2 */
  0x0000,   /* U+16AF3 */
  0x0000,   /* U+16AF4 */
  0x0000,   /* U+16AF5 */
  0xEAF6,   /* U+16AF6 */
  0xEAF7,   /* U+16AF7 */
  0xEAF8,   /* U+16AF8 */
  0xEAF9,   /* U+16AF9 */
  0xEAFA,   /* U+16AFA */
  0xEAFB,   /* U+16AFB */
  0xEAFC,   /* U+16AFC */
  0xEAFD,   /* U+16AFD */
  0xEAFE,   /* U+16AFE */
  0xEAFF,   /* U+16AFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+16A00 */
  0x0000,   /* U+16A01 */
  0x0000,   /* U+16A02 */
  0x0000,   /* U+16A03 */
  0x0000,   /* U+16A04 */
  0x0000,   /* U+16A05 */
  0x0000,   /* U+16A06 */
  0x0000,   /* U+16A07 */
  0x0000,   /* U+16A08 */
  0x0000,   /* U+16A09 */
  0x0000,   /* U+16A0A */
  0x0000,   /* U+16A0B */
  0x0000,   /* U+16A0C */
  0x0000,   /* U+16A0D */
  0x0000,   /* U+16A0E */
  0x0000,   /* U+16A0F */
  0x0000,   /* U+16A10 */
  0x0000,   /* U+16A11 */
  0x0000,   /* U+16A12 */
  0x0000,   /* U+16A13 */
  0x0000,   /* U+16A14 */
  0x0000,   /* U+16A15 */
  0x0000,   /* U+16A16 */
  0x0000,   /* U+16A17 */
  0x0000,   /* U+16A18 */
  0x0000,   /* U+16A19 */
  0x0000,   /* U+16A1A */
  0x0000,   /* U+16A1B */
  0x0000,   /* U+16A1C */
  0x0000,   /* U+16A1D */
  0x0000,   /* U+16A1E */
  0x0000,   /* U+16A1F */
  0x0000,   /* U+16A20 */
  0x0000,   /* U+16A21 */
  0x0000,   /* U+16A22 */
  0x0000,   /* U+16A23 */
  0x0000,   /* U+16A24 */
  0x0000,   /* U+16A25 */
  0x0000,   /* U+16A26 */
  0x0000,   /* U+16A27 */
  0x0000,   /* U+16A28 */
  0x0000,   /* U+16A29 */
  0x0000,   /* U+16A2A */
  0x0000,   /* U+16A2B */
  0x0000,   /* U+16A2C */
  0x0000,   /* U+16A2D */
  0x0000,   /* U+16A2E */
  0x0000,   /* U+16A2F */
  0x0000,   /* U+16A30 */
  0x0000,   /* U+16A31 */
  0x0000,   /* U+16A32 */
  0x0000,   /* U+16A33 */
  0x0000,   /* U+16A34 */
  0x0000,   /* U+16A35 */
  0x0000,   /* U+16A36 */
  0x0000,   /* U+16A37 */
  0x0000,   /* U+16A38 */
  0x0000,   /* U+16A39 */
  0x0000,   /* U+16A3A */
  0x0000,   /* U+16A3B */
  0x0000,   /* U+16A3C */
  0x0000,   /* U+16A3D */
  0x0000,   /* U+16A3E */
  0x0000,   /* U+16A3F */
  0x0000,   /* U+16A40 */
  0x0000,   /* U+16A41 */
  0x0000,   /* U+16A42 */
  0x0000,   /* U+16A43 */
  0x0000,   /* U+16A44 */
  0x0000,   /* U+16A45 */
  0x0000,   /* U+16A46 */
  0x0000,   /* U+16A47 */
  0x0000,   /* U+16A48 */
  0x0000,   /* U+16A49 */
  0x0000,   /* U+16A4A */
  0x0000,   /* U+16A4B */
  0x0000,   /* U+16A4C */
  0x0000,   /* U+16A4D */
  0x0000,   /* U+16A4E */
  0x0000,   /* U+16A4F */
  0x0000,   /* U+16A50 */
  0x0000,   /* U+16A51 */
  0x0000,   /* U+16A52 */
  0x0000,   /* U+16A53 */
  0x0000,   /* U+16A54 */
  0x0000,   /* U+16A55 */
  0x0000,   /* U+16A56 */
  0x0000,   /* U+16A57 */
  0x0000,   /* U+16A58 */
  0x0000,   /* U+16A59 */
  0x0000,   /* U+16A5A */
  0x0000,   /* U+16A5B */
  0x0000,   /* U+16A5C */
  0x0000,   /* U+16A5D */
  0x0000,   /* U+16A5E */
  0x0000,   /* U+16A5F */
  0x0000,   /* U+16A60 */
  0x0000,   /* U+16A61 */
  0x0000,   /* U+16A62 */
  0x0000,   /* U+16A63 */
  0x0000,   /* U+16A64 */
  0x0000,   /* U+16A65 */
  0x0000,   /* U+16A66 */
  0x0000,   /* U+16A67 */
  0x0000,   /* U+16A68 */
  0x0000,   /* U+16A69 */
  0x0000,   /* U+16A6A */
  0x0000,   /* U+16A6B */
  0x0000,   /* U+16A6C */
  0x0000,   /* U+16A6D */
  0x0000,   /* U+16A6E */
  0x0000,   /* U+16A6F */
  0x0000,   /* U+16A70 */
  0x0000,   /* U+16A71 */
  0x0000,   /* U+16A72 */
  0x0000,   /* U+16A73 */
  0x0000,   /* U+16A74 */
  0x0000,   /* U+16A75 */
  0x0000,   /* U+16A76 */
  0x0000,   /* U+16A77 */
  0x0000,   /* U+16A78 */
  0x0000,   /* U+16A79 */
  0x0000,   /* U+16A7A */
  0x0000,   /* U+16A7B */
  0x0000,   /* U+16A7C */
  0x0000,   /* U+16A7D */
  0x0000,   /* U+16A7E */
  0x0000,   /* U+16A7F */
  0x0000,   /* U+16A80 */
  0x0000,   /* U+16A81 */
  0x0000,   /* U+16A82 */
  0x0000,   /* U+16A83 */
  0x0000,   /* U+16A84 */
  0x0000,   /* U+16A85 */
  0x0000,   /* U+16A86 */
  0x0000,   /* U+16A87 */
  0x0000,   /* U+16A88 */
  0x0000,   /* U+16A89 */
  0x0000,   /* U+16A8A */
  0x0000,   /* U+16A8B */
  0x0000,   /* U+16A8C */
  0x0000,   /* U+16A8D */
  0x0000,   /* U+16A8E */
  0x0000,   /* U+16A8F */
  0x0000,   /* U+16A90 */
  0x0000,   /* U+16A91 */
  0x0000,   /* U+16A92 */
  0x0000,   /* U+16A93 */
  0x0000,   /* U+16A94 */
  0x0000,   /* U+16A95 */
  0x0000,   /* U+16A96 */
  0x0000,   /* U+16A97 */
  0x0000,   /* U+16A98 */
  0x0000,   /* U+16A99 */
  0x0000,   /* U+16A9A */
  0x0000,   /* U+16A9B */
  0x0000,   /* U+16A9C */
  0x0000,   /* U+16A9D */
  0x0000,   /* U+16A9E */
  0x0000,   /* U+16A9F */
  0x0000,   /* U+16AA0 */
  0x0000,   /* U+16AA1 */
  0x0000,   /* U+16AA2 */
  0x0000,   /* U+16AA3 */
  0x0000,   /* U+16AA4 */
  0x0000,   /* U+16AA5 */
  0x0000,   /* U+16AA6 */
  0x0000,   /* U+16AA7 */
  0x0000,   /* U+16AA8 */
  0x0000,   /* U+16AA9 */
  0x0000,   /* U+16AAA */
  0x0000,   /* U+16AAB */
  0x0000,   /* U+16AAC */
  0x0000,   /* U+16AAD */
  0x0000,   /* U+16AAE */
  0x0000,   /* U+16AAF */
  0x0000,   /* U+16AB0 */
  0x0000,   /* U+16AB1 */
  0x0000,   /* U+16AB2 */
  0x0000,   /* U+16AB3 */
  0x0000,   /* U+16AB4 */
  0x0000,   /* U+16AB5 */
  0x0000,   /* U+16AB6 */
  0x0000,   /* U+16AB7 */
  0x0000,   /* U+16AB8 */
  0x0000,   /* U+16AB9 */
  0x0000,   /* U+16ABA */
  0x0000,   /* U+16ABB */
  0x0000,   /* U+16ABC */
  0x0000,   /* U+16ABD */
  0x0000,   /* U+16ABE */
  0x0000,   /* U+16ABF */
  0x0000,   /* U+16AC0 */
  0x0000,   /* U+16AC1 */
  0x0000,   /* U+16AC2 */
  0x0000,   /* U+16AC3 */
  0x0000,   /* U+16AC4 */
  0x0000,   /* U+16AC5 */
  0x0000,   /* U+16AC6 */
  0x0000,   /* U+16AC7 */
  0x0000,   /* U+16AC8 */
  0x0000,   /* U+16AC9 */
  0x0000,   /* U+16ACA */
  0x0000,   /* U+16ACB */
  0x0000,   /* U+16ACC */
  0x0000,   /* U+16ACD */
  0x0000,   /* U+16ACE */
  0x0000,   /* U+16ACF */
  0x0000,   /* U+16AD0 */
  0x0000,   /* U+16AD1 */
  0x0000,   /* U+16AD2 */
  0x0000,   /* U+16AD3 */
  0x0000,   /* U+16AD4 */
  0x0000,   /* U+16AD5 */
  0x0000,   /* U+16AD6 */
  0x0000,   /* U+16AD7 */
  0x0000,   /* U+16AD8 */
  0x0000,   /* U+16AD9 */
  0x0000,   /* U+16ADA */
  0x0000,   /* U+16ADB */
  0x0000,   /* U+16ADC */
  0x0000,   /* U+16ADD */
  0x0000,   /* U+16ADE */
  0x0000,   /* U+16ADF */
  0x0000,   /* U+16AE0 */
  0x0000,   /* U+16AE1 */
  0x0000,   /* U+16AE2 */
  0x0000,   /* U+16AE3 */
  0x0000,   /* U+16AE4 */
  0x0000,   /* U+16AE5 */
  0x0000,   /* U+16AE6 */
  0x0000,   /* U+16AE7 */
  0x0000,   /* U+16AE8 */
  0x0000,   /* U+16AE9 */
  0x0000,   /* U+16AEA */
  0x0000,   /* U+16AEB */
  0x0000,   /* U+16AEC */
  0x0000,   /* U+16AED */
  0x0000,   /* U+16AEE */
  0x0000,   /* U+16AEF */
  0x0000,   /* U+16AF0 */
  0x0000,   /* U+16AF1 */
  0x0000,   /* U+16AF2 */
  0x0000,   /* U+16AF3 */
  0x0000,   /* U+16AF4 */
  0x0000,   /* U+16AF5 */
  0x0000,   /* U+16AF6 */
  0x0000,   /* U+16AF7 */
  0x0000,   /* U+16AF8 */
  0x0000,   /* U+16AF9 */
  0x0000,   /* U+16AFA */
  0x0000,   /* U+16AFB */
  0x0000,   /* U+16AFC */
  0x0000,   /* U+16AFD */
  0x0000,   /* U+16AFE */
  0x0000,   /* U+16AFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+16A00 */
  0x0000,   /* U+16A01 */
  0x0000,   /* U+16A02 */
  0x0000,   /* U+16A03 */
  0x0000,   /* U+16A04 */
  0x0000,   /* U+16A05 */
  0x0000,   /* U+16A06 */
  0x0000,   /* U+16A07 */
  0x0000,   /* U+16A08 */
  0x0000,   /* U+16A09 */
  0x0000,   /* U+16A0A */
  0x0000,   /* U+16A0B */
  0x0000,   /* U+16A0C */
  0x0000,   /* U+16A0D */
  0x0000,   /* U+16A0E */
  0x0000,   /* U+16A0F */
  0x0000,   /* U+16A10 */
  0x0000,   /* U+16A11 */
  0x0000,   /* U+16A12 */
  0x0000,   /* U+16A13 */
  0x0000,   /* U+16A14 */
  0x0000,   /* U+16A15 */
  0x0000,   /* U+16A16 */
  0x0000,   /* U+16A17 */
  0x0000,   /* U+16A18 */
  0x0000,   /* U+16A19 */
  0x0000,   /* U+16A1A */
  0x0000,   /* U+16A1B */
  0x0000,   /* U+16A1C */
  0x0000,   /* U+16A1D */
  0x0000,   /* U+16A1E */
  0x0000,   /* U+16A1F */
  0x0000,   /* U+16A20 */
  0x0000,   /* U+16A21 */
  0x0000,   /* U+16A22 */
  0x0000,   /* U+16A23 */
  0x0000,   /* U+16A24 */
  0x0000,   /* U+16A25 */
  0x0000,   /* U+16A26 */
  0x0000,   /* U+16A27 */
  0x0000,   /* U+16A28 */
  0x0000,   /* U+16A29 */
  0x0000,   /* U+16A2A */
  0x0000,   /* U+16A2B */
  0x0000,   /* U+16A2C */
  0x0000,   /* U+16A2D */
  0x0000,   /* U+16A2E */
  0x0000,   /* U+16A2F */
  0x0000,   /* U+16A30 */
  0x0000,   /* U+16A31 */
  0x0000,   /* U+16A32 */
  0x0000,   /* U+16A33 */
  0x0000,   /* U+16A34 */
  0x0000,   /* U+16A35 */
  0x0000,   /* U+16A36 */
  0x0000,   /* U+16A37 */
  0x0000,   /* U+16A38 */
  0x0000,   /* U+16A39 */
  0x0000,   /* U+16A3A */
  0x0000,   /* U+16A3B */
  0x0000,   /* U+16A3C */
  0x0000,   /* U+16A3D */
  0x0000,   /* U+16A3E */
  0x0000,   /* U+16A3F */
  0x0000,   /* U+16A40 */
  0x0000,   /* U+16A41 */
  0x0000,   /* U+16A42 */
  0x0000,   /* U+16A43 */
  0x0000,   /* U+16A44 */
  0x0000,   /* U+16A45 */
  0x0000,   /* U+16A46 */
  0x0000,   /* U+16A47 */
  0x0000,   /* U+16A48 */
  0x0000,   /* U+16A49 */
  0x0000,   /* U+16A4A */
  0x0000,   /* U+16A4B */
  0x0000,   /* U+16A4C */
  0x0000,   /* U+16A4D */
  0x0000,   /* U+16A4E */
  0x0000,   /* U+16A4F */
  0x0000,   /* U+16A50 */
  0x0000,   /* U+16A51 */
  0x0000,   /* U+16A52 */
  0x0000,   /* U+16A53 */
  0x0000,   /* U+16A54 */
  0x0000,   /* U+16A55 */
  0x0000,   /* U+16A56 */
  0x0000,   /* U+16A57 */
  0x0000,   /* U+16A58 */
  0x0000,   /* U+16A59 */
  0x0000,   /* U+16A5A */
  0x0000,   /* U+16A5B */
  0x0000,   /* U+16A5C */
  0x0000,   /* U+16A5D */
  0x0000,   /* U+16A5E */
  0x0000,   /* U+16A5F */
  0x0000,   /* U+16A60 */
  0x0000,   /* U+16A61 */
  0x0000,   /* U+16A62 */
  0x0000,   /* U+16A63 */
  0x0000,   /* U+16A64 */
  0x0000,   /* U+16A65 */
  0x0000,   /* U+16A66 */
  0x0000,   /* U+16A67 */
  0x0000,   /* U+16A68 */
  0x0000,   /* U+16A69 */
  0x0000,   /* U+16A6A */
  0x0000,   /* U+16A6B */
  0x0000,   /* U+16A6C */
  0x0000,   /* U+16A6D */
  0x0000,   /* U+16A6E */
  0x0000,   /* U+16A6F */
  0x0000,   /* U+16A70 */
  0x0000,   /* U+16A71 */
  0x0000,   /* U+16A72 */
  0x0000,   /* U+16A73 */
  0x0000,   /* U+16A74 */
  0x0000,   /* U+16A75 */
  0x0000,   /* U+16A76 */
  0x0000,   /* U+16A77 */
  0x0000,   /* U+16A78 */
  0x0000,   /* U+16A79 */
  0x0000,   /* U+16A7A */
  0x0000,   /* U+16A7B */
  0x0000,   /* U+16A7C */
  0x0000,   /* U+16A7D */
  0x0000,   /* U+16A7E */
  0x0000,   /* U+16A7F */
  0x0000,   /* U+16A80 */
  0x0000,   /* U+16A81 */
  0x0000,   /* U+16A82 */
  0x0000,   /* U+16A83 */
  0x0000,   /* U+16A84 */
  0x0000,   /* U+16A85 */
  0x0000,   /* U+16A86 */
  0x0000,   /* U+16A87 */
  0x0000,   /* U+16A88 */
  0x0000,   /* U+16A89 */
  0x0000,   /* U+16A8A */
  0x0000,   /* U+16A8B */
  0x0000,   /* U+16A8C */
  0x0000,   /* U+16A8D */
  0x0000,   /* U+16A8E */
  0x0000,   /* U+16A8F */
  0x0000,   /* U+16A90 */
  0x0000,   /* U+16A91 */
  0x0000,   /* U+16A92 */
  0x0000,   /* U+16A93 */
  0x0000,   /* U+16A94 */
  0x0000,   /* U+16A95 */
  0x0000,   /* U+16A96 */
  0x0000,   /* U+16A97 */
  0x0000,   /* U+16A98 */
  0x0000,   /* U+16A99 */
  0x0000,   /* U+16A9A */
  0x0000,   /* U+16A9B */
  0x0000,   /* U+16A9C */
  0x0000,   /* U+16A9D */
  0x0000,   /* U+16A9E */
  0x0000,   /* U+16A9F */
  0x0000,   /* U+16AA0 */
  0x0000,   /* U+16AA1 */
  0x0000,   /* U+16AA2 */
  0x0000,   /* U+16AA3 */
  0x0000,   /* U+16AA4 */
  0x0000,   /* U+16AA5 */
  0x0000,   /* U+16AA6 */
  0x0000,   /* U+16AA7 */
  0x0000,   /* U+16AA8 */
  0x0000,   /* U+16AA9 */
  0x0000,   /* U+16AAA */
  0x0000,   /* U+16AAB */
  0x0000,   /* U+16AAC */
  0x0000,   /* U+16AAD */
  0x0000,   /* U+16AAE */
  0x0000,   /* U+16AAF */
  0x0000,   /* U+16AB0 */
  0x0000,   /* U+16AB1 */
  0x0000,   /* U+16AB2 */
  0x0000,   /* U+16AB3 */
  0x0000,   /* U+16AB4 */
  0x0000,   /* U+16AB5 */
  0x0000,   /* U+16AB6 */
  0x0000,   /* U+16AB7 */
  0x0000,   /* U+16AB8 */
  0x0000,   /* U+16AB9 */
  0x0000,   /* U+16ABA */
  0x0000,   /* U+16ABB */
  0x0000,   /* U+16ABC */
  0x0000,   /* U+16ABD */
  0x0000,   /* U+16ABE */
  0x0000,   /* U+16ABF */
  0x0000,   /* U+16AC0 */
  0x0000,   /* U+16AC1 */
  0x0000,   /* U+16AC2 */
  0x0000,   /* U+16AC3 */
  0x0000,   /* U+16AC4 */
  0x0000,   /* U+16AC5 */
  0x0000,   /* U+16AC6 */
  0x0000,   /* U+16AC7 */
  0x0000,   /* U+16AC8 */
  0x0000,   /* U+16AC9 */
  0x0000,   /* U+16ACA */
  0x0000,   /* U+16ACB */
  0x0000,   /* U+16ACC */
  0x0000,   /* U+16ACD */
  0x0000,   /* U+16ACE */
  0x0000,   /* U+16ACF */
  0x0000,   /* U+16AD0 */
  0x0000,   /* U+16AD1 */
  0x0000,   /* U+16AD2 */
  0x0000,   /* U+16AD3 */
  0x0000,   /* U+16AD4 */
  0x0000,   /* U+16AD5 */
  0x0000,   /* U+16AD6 */
  0x0000,   /* U+16AD7 */
  0x0000,   /* U+16AD8 */
  0x0000,   /* U+16AD9 */
  0x0000,   /* U+16ADA */
  0x0000,   /* U+16ADB */
  0x0000,   /* U+16ADC */
  0x0000,   /* U+16ADD */
  0x0000,   /* U+16ADE */
  0x0000,   /* U+16ADF */
  0x0000,   /* U+16AE0 */
  0x0000,   /* U+16AE1 */
  0x0000,   /* U+16AE2 */
  0x0000,   /* U+16AE3 */
  0x0000,   /* U+16AE4 */
  0x0000,   /* U+16AE5 */
  0x0000,   /* U+16AE6 */
  0x0000,   /* U+16AE7 */
  0x0000,   /* U+16AE8 */
  0x0000,   /* U+16AE9 */
  0x0000,   /* U+16AEA */
  0x0000,   /* U+16AEB */
  0x0000,   /* U+16AEC */
  0x0000,   /* U+16AED */
  0x0000,   /* U+16AEE */
  0x0000,   /* U+16AEF */
  0x0000,   /* U+16AF0 */
  0x0000,   /* U+16AF1 */
  0x0000,   /* U+16AF2 */
  0x0000,   /* U+16AF3 */
  0x0000,   /* U+16AF4 */
  0x0000,   /* U+16AF5 */
  0x0000,   /* U+16AF6 */
  0x0000,   /* U+16AF7 */
  0x0000,   /* U+16AF8 */
  0x0000,   /* U+16AF9 */
  0x0000,   /* U+16AFA */
  0x0000,   /* U+16AFB */
  0x0000,   /* U+16AFC */
  0x0000,   /* U+16AFD */
  0x0000,   /* U+16AFE */
  0x0000,   /* U+16AFF */
};

uint16 uca900_p16B[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x4355,   /* U+16B00 */
  0x4356,   /* U+16B01 */
  0x4357,   /* U+16B02 */
  0x4358,   /* U+16B03 */
  0x4359,   /* U+16B04 */
  0x435A,   /* U+16B05 */
  0x435B,   /* U+16B06 */
  0x435C,   /* U+16B07 */
  0x435D,   /* U+16B08 */
  0x435E,   /* U+16B09 */
  0x435F,   /* U+16B0A */
  0x4360,   /* U+16B0B */
  0x4361,   /* U+16B0C */
  0x4362,   /* U+16B0D */
  0x4363,   /* U+16B0E */
  0x4364,   /* U+16B0F */
  0x4365,   /* U+16B10 */
  0x4366,   /* U+16B11 */
  0x4367,   /* U+16B12 */
  0x4368,   /* U+16B13 */
  0x4369,   /* U+16B14 */
  0x436A,   /* U+16B15 */
  0x436B,   /* U+16B16 */
  0x436C,   /* U+16B17 */
  0x436D,   /* U+16B18 */
  0x436E,   /* U+16B19 */
  0x436F,   /* U+16B1A */
  0x4370,   /* U+16B1B */
  0x4371,   /* U+16B1C */
  0x4372,   /* U+16B1D */
  0x4373,   /* U+16B1E */
  0x4374,   /* U+16B1F */
  0x4375,   /* U+16B20 */
  0x4376,   /* U+16B21 */
  0x4377,   /* U+16B22 */
  0x4378,   /* U+16B23 */
  0x4379,   /* U+16B24 */
  0x437A,   /* U+16B25 */
  0x437B,   /* U+16B26 */
  0x437C,   /* U+16B27 */
  0x437D,   /* U+16B28 */
  0x437E,   /* U+16B29 */
  0x437F,   /* U+16B2A */
  0x4380,   /* U+16B2B */
  0x4381,   /* U+16B2C */
  0x4382,   /* U+16B2D */
  0x4383,   /* U+16B2E */
  0x4384,   /* U+16B2F */
  0x0000,   /* U+16B30 */
  0x0000,   /* U+16B31 */
  0x0000,   /* U+16B32 */
  0x0000,   /* U+16B33 */
  0x0000,   /* U+16B34 */
  0x0000,   /* U+16B35 */
  0x0000,   /* U+16B36 */
  0x0477,   /* U+16B37 */
  0x0478,   /* U+16B38 */
  0x0479,   /* U+16B39 */
  0x047A,   /* U+16B3A */
  0x047B,   /* U+16B3B */
  0x0FD1,   /* U+16B3C */
  0x0FD2,   /* U+16B3D */
  0x0FD3,   /* U+16B3E */
  0x0FD4,   /* U+16B3F */
  0x4385,   /* U+16B40 */
  0x4386,   /* U+16B41 */
  0x1C05,   /* U+16B42 */
  0x1C06,   /* U+16B43 */
  0x047C,   /* U+16B44 */
  0x0FD5,   /* U+16B45 */
  0xFBC2,   /* U+16B46 */
  0xFBC2,   /* U+16B47 */
  0xFBC2,   /* U+16B48 */
  0xFBC2,   /* U+16B49 */
  0xFBC2,   /* U+16B4A */
  0xFBC2,   /* U+16B4B */
  0xFBC2,   /* U+16B4C */
  0xFBC2,   /* U+16B4D */
  0xFBC2,   /* U+16B4E */
  0xFBC2,   /* U+16B4F */
  0x1C3D,   /* U+16B50 */
  0x1C3E,   /* U+16B51 */
  0x1C3F,   /* U+16B52 */
  0x1C40,   /* U+16B53 */
  0x1C41,   /* U+16B54 */
  0x1C42,   /* U+16B55 */
  0x1C43,   /* U+16B56 */
  0x1C44,   /* U+16B57 */
  0x1C45,   /* U+16B58 */
  0x1C46,   /* U+16B59 */
  0xFBC2,   /* U+16B5A */
  0x1BA0,   /* U+16B5B */
  0x1BA1,   /* U+16B5C */
  0x1BA2,   /* U+16B5D */
  0x1BA3,   /* U+16B5E */
  0x1BA4,   /* U+16B5F */
  0x1BA5,   /* U+16B60 */
  0x1BA6,   /* U+16B61 */
  0xFBC2,   /* U+16B62 */
  0x4387,   /* U+16B63 */
  0x4388,   /* U+16B64 */
  0x4389,   /* U+16B65 */
  0x438A,   /* U+16B66 */
  0x438B,   /* U+16B67 */
  0x438C,   /* U+16B68 */
  0x438D,   /* U+16B69 */
  0x438E,   /* U+16B6A */
  0x438F,   /* U+16B6B */
  0x4390,   /* U+16B6C */
  0x4391,   /* U+16B6D */
  0x4392,   /* U+16B6E */
  0x4393,   /* U+16B6F */
  0x4394,   /* U+16B70 */
  0x4395,   /* U+16B71 */
  0x4396,   /* U+16B72 */
  0x4397,   /* U+16B73 */
  0x4398,   /* U+16B74 */
  0x4399,   /* U+16B75 */
  0x439A,   /* U+16B76 */
  0x439B,   /* U+16B77 */
  0xFBC2,   /* U+16B78 */
  0xFBC2,   /* U+16B79 */
  0xFBC2,   /* U+16B7A */
  0xFBC2,   /* U+16B7B */
  0xFBC2,   /* U+16B7C */
  0x439C,   /* U+16B7D */
  0x439D,   /* U+16B7E */
  0x439E,   /* U+16B7F */
  0x439F,   /* U+16B80 */
  0x43A0,   /* U+16B81 */
  0x43A1,   /* U+16B82 */
  0x43A2,   /* U+16B83 */
  0x43A3,   /* U+16B84 */
  0x43A4,   /* U+16B85 */
  0x43A5,   /* U+16B86 */
  0x43A6,   /* U+16B87 */
  0x43A7,   /* U+16B88 */
  0x43A8,   /* U+16B89 */
  0x43A9,   /* U+16B8A */
  0x43AA,   /* U+16B8B */
  0x43AB,   /* U+16B8C */
  0x43AC,   /* U+16B8D */
  0x43AD,   /* U+16B8E */
  0x43AE,   /* U+16B8F */
  0xFBC2,   /* U+16B90 */
  0xFBC2,   /* U+16B91 */
  0xFBC2,   /* U+16B92 */
  0xFBC2,   /* U+16B93 */
  0xFBC2,   /* U+16B94 */
  0xFBC2,   /* U+16B95 */
  0xFBC2,   /* U+16B96 */
  0xFBC2,   /* U+16B97 */
  0xFBC2,   /* U+16B98 */
  0xFBC2,   /* U+16B99 */
  0xFBC2,   /* U+16B9A */
  0xFBC2,   /* U+16B9B */
  0xFBC2,   /* U+16B9C */
  0xFBC2,   /* U+16B9D */
  0xFBC2,   /* U+16B9E */
  0xFBC2,   /* U+16B9F */
  0xFBC2,   /* U+16BA0 */
  0xFBC2,   /* U+16BA1 */
  0xFBC2,   /* U+16BA2 */
  0xFBC2,   /* U+16BA3 */
  0xFBC2,   /* U+16BA4 */
  0xFBC2,   /* U+16BA5 */
  0xFBC2,   /* U+16BA6 */
  0xFBC2,   /* U+16BA7 */
  0xFBC2,   /* U+16BA8 */
  0xFBC2,   /* U+16BA9 */
  0xFBC2,   /* U+16BAA */
  0xFBC2,   /* U+16BAB */
  0xFBC2,   /* U+16BAC */
  0xFBC2,   /* U+16BAD */
  0xFBC2,   /* U+16BAE */
  0xFBC2,   /* U+16BAF */
  0xFBC2,   /* U+16BB0 */
  0xFBC2,   /* U+16BB1 */
  0xFBC2,   /* U+16BB2 */
  0xFBC2,   /* U+16BB3 */
  0xFBC2,   /* U+16BB4 */
  0xFBC2,   /* U+16BB5 */
  0xFBC2,   /* U+16BB6 */
  0xFBC2,   /* U+16BB7 */
  0xFBC2,   /* U+16BB8 */
  0xFBC2,   /* U+16BB9 */
  0xFBC2,   /* U+16BBA */
  0xFBC2,   /* U+16BBB */
  0xFBC2,   /* U+16BBC */
  0xFBC2,   /* U+16BBD */
  0xFBC2,   /* U+16BBE */
  0xFBC2,   /* U+16BBF */
  0xFBC2,   /* U+16BC0 */
  0xFBC2,   /* U+16BC1 */
  0xFBC2,   /* U+16BC2 */
  0xFBC2,   /* U+16BC3 */
  0xFBC2,   /* U+16BC4 */
  0xFBC2,   /* U+16BC5 */
  0xFBC2,   /* U+16BC6 */
  0xFBC2,   /* U+16BC7 */
  0xFBC2,   /* U+16BC8 */
  0xFBC2,   /* U+16BC9 */
  0xFBC2,   /* U+16BCA */
  0xFBC2,   /* U+16BCB */
  0xFBC2,   /* U+16BCC */
  0xFBC2,   /* U+16BCD */
  0xFBC2,   /* U+16BCE */
  0xFBC2,   /* U+16BCF */
  0xFBC2,   /* U+16BD0 */
  0xFBC2,   /* U+16BD1 */
  0xFBC2,   /* U+16BD2 */
  0xFBC2,   /* U+16BD3 */
  0xFBC2,   /* U+16BD4 */
  0xFBC2,   /* U+16BD5 */
  0xFBC2,   /* U+16BD6 */
  0xFBC2,   /* U+16BD7 */
  0xFBC2,   /* U+16BD8 */
  0xFBC2,   /* U+16BD9 */
  0xFBC2,   /* U+16BDA */
  0xFBC2,   /* U+16BDB */
  0xFBC2,   /* U+16BDC */
  0xFBC2,   /* U+16BDD */
  0xFBC2,   /* U+16BDE */
  0xFBC2,   /* U+16BDF */
  0xFBC2,   /* U+16BE0 */
  0xFBC2,   /* U+16BE1 */
  0xFBC2,   /* U+16BE2 */
  0xFBC2,   /* U+16BE3 */
  0xFBC2,   /* U+16BE4 */
  0xFBC2,   /* U+16BE5 */
  0xFBC2,   /* U+16BE6 */
  0xFBC2,   /* U+16BE7 */
  0xFBC2,   /* U+16BE8 */
  0xFBC2,   /* U+16BE9 */
  0xFBC2,   /* U+16BEA */
  0xFBC2,   /* U+16BEB */
  0xFBC2,   /* U+16BEC */
  0xFBC2,   /* U+16BED */
  0xFBC2,   /* U+16BEE */
  0xFBC2,   /* U+16BEF */
  0xFBC2,   /* U+16BF0 */
  0xFBC2,   /* U+16BF1 */
  0xFBC2,   /* U+16BF2 */
  0xFBC2,   /* U+16BF3 */
  0xFBC2,   /* U+16BF4 */
  0xFBC2,   /* U+16BF5 */
  0xFBC2,   /* U+16BF6 */
  0xFBC2,   /* U+16BF7 */
  0xFBC2,   /* U+16BF8 */
  0xFBC2,   /* U+16BF9 */
  0xFBC2,   /* U+16BFA */
  0xFBC2,   /* U+16BFB */
  0xFBC2,   /* U+16BFC */
  0xFBC2,   /* U+16BFD */
  0xFBC2,   /* U+16BFE */
  0xFBC2,   /* U+16BFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+16B00 */
  0x0020,   /* U+16B01 */
  0x0020,   /* U+16B02 */
  0x0020,   /* U+16B03 */
  0x0020,   /* U+16B04 */
  0x0020,   /* U+16B05 */
  0x0020,   /* U+16B06 */
  0x0020,   /* U+16B07 */
  0x0020,   /* U+16B08 */
  0x0020,   /* U+16B09 */
  0x0020,   /* U+16B0A */
  0x0020,   /* U+16B0B */
  0x0020,   /* U+16B0C */
  0x0020,   /* U+16B0D */
  0x0020,   /* U+16B0E */
  0x0020,   /* U+16B0F */
  0x0020,   /* U+16B10 */
  0x0020,   /* U+16B11 */
  0x0020,   /* U+16B12 */
  0x0020,   /* U+16B13 */
  0x0020,   /* U+16B14 */
  0x0020,   /* U+16B15 */
  0x0020,   /* U+16B16 */
  0x0020,   /* U+16B17 */
  0x0020,   /* U+16B18 */
  0x0020,   /* U+16B19 */
  0x0020,   /* U+16B1A */
  0x0020,   /* U+16B1B */
  0x0020,   /* U+16B1C */
  0x0020,   /* U+16B1D */
  0x0020,   /* U+16B1E */
  0x0020,   /* U+16B1F */
  0x0020,   /* U+16B20 */
  0x0020,   /* U+16B21 */
  0x0020,   /* U+16B22 */
  0x0020,   /* U+16B23 */
  0x0020,   /* U+16B24 */
  0x0020,   /* U+16B25 */
  0x0020,   /* U+16B26 */
  0x0020,   /* U+16B27 */
  0x0020,   /* U+16B28 */
  0x0020,   /* U+16B29 */
  0x0020,   /* U+16B2A */
  0x0020,   /* U+16B2B */
  0x0020,   /* U+16B2C */
  0x0020,   /* U+16B2D */
  0x0020,   /* U+16B2E */
  0x0020,   /* U+16B2F */
  0x00F4,   /* U+16B30 */
  0x00F5,   /* U+16B31 */
  0x00F6,   /* U+16B32 */
  0x00F7,   /* U+16B33 */
  0x00F8,   /* U+16B34 */
  0x00F9,   /* U+16B35 */
  0x00FA,   /* U+16B36 */
  0x0020,   /* U+16B37 */
  0x0020,   /* U+16B38 */
  0x0020,   /* U+16B39 */
  0x0020,   /* U+16B3A */
  0x0020,   /* U+16B3B */
  0x0020,   /* U+16B3C */
  0x0020,   /* U+16B3D */
  0x0020,   /* U+16B3E */
  0x0020,   /* U+16B3F */
  0x0020,   /* U+16B40 */
  0x0020,   /* U+16B41 */
  0x0020,   /* U+16B42 */
  0x0020,   /* U+16B43 */
  0x0020,   /* U+16B44 */
  0x0020,   /* U+16B45 */
  0x0020,   /* U+16B46 */
  0x0020,   /* U+16B47 */
  0x0020,   /* U+16B48 */
  0x0020,   /* U+16B49 */
  0x0020,   /* U+16B4A */
  0x0020,   /* U+16B4B */
  0x0020,   /* U+16B4C */
  0x0020,   /* U+16B4D */
  0x0020,   /* U+16B4E */
  0x0020,   /* U+16B4F */
  0x0020,   /* U+16B50 */
  0x0020,   /* U+16B51 */
  0x0020,   /* U+16B52 */
  0x0020,   /* U+16B53 */
  0x0020,   /* U+16B54 */
  0x0020,   /* U+16B55 */
  0x0020,   /* U+16B56 */
  0x0020,   /* U+16B57 */
  0x0020,   /* U+16B58 */
  0x0020,   /* U+16B59 */
  0x0020,   /* U+16B5A */
  0x0020,   /* U+16B5B */
  0x0020,   /* U+16B5C */
  0x0020,   /* U+16B5D */
  0x0020,   /* U+16B5E */
  0x0020,   /* U+16B5F */
  0x0020,   /* U+16B60 */
  0x0020,   /* U+16B61 */
  0x0020,   /* U+16B62 */
  0x0020,   /* U+16B63 */
  0x0020,   /* U+16B64 */
  0x0020,   /* U+16B65 */
  0x0020,   /* U+16B66 */
  0x0020,   /* U+16B67 */
  0x0020,   /* U+16B68 */
  0x0020,   /* U+16B69 */
  0x0020,   /* U+16B6A */
  0x0020,   /* U+16B6B */
  0x0020,   /* U+16B6C */
  0x0020,   /* U+16B6D */
  0x0020,   /* U+16B6E */
  0x0020,   /* U+16B6F */
  0x0020,   /* U+16B70 */
  0x0020,   /* U+16B71 */
  0x0020,   /* U+16B72 */
  0x0020,   /* U+16B73 */
  0x0020,   /* U+16B74 */
  0x0020,   /* U+16B75 */
  0x0020,   /* U+16B76 */
  0x0020,   /* U+16B77 */
  0x0020,   /* U+16B78 */
  0x0020,   /* U+16B79 */
  0x0020,   /* U+16B7A */
  0x0020,   /* U+16B7B */
  0x0020,   /* U+16B7C */
  0x0020,   /* U+16B7D */
  0x0020,   /* U+16B7E */
  0x0020,   /* U+16B7F */
  0x0020,   /* U+16B80 */
  0x0020,   /* U+16B81 */
  0x0020,   /* U+16B82 */
  0x0020,   /* U+16B83 */
  0x0020,   /* U+16B84 */
  0x0020,   /* U+16B85 */
  0x0020,   /* U+16B86 */
  0x0020,   /* U+16B87 */
  0x0020,   /* U+16B88 */
  0x0020,   /* U+16B89 */
  0x0020,   /* U+16B8A */
  0x0020,   /* U+16B8B */
  0x0020,   /* U+16B8C */
  0x0020,   /* U+16B8D */
  0x0020,   /* U+16B8E */
  0x0020,   /* U+16B8F */
  0x0020,   /* U+16B90 */
  0x0020,   /* U+16B91 */
  0x0020,   /* U+16B92 */
  0x0020,   /* U+16B93 */
  0x0020,   /* U+16B94 */
  0x0020,   /* U+16B95 */
  0x0020,   /* U+16B96 */
  0x0020,   /* U+16B97 */
  0x0020,   /* U+16B98 */
  0x0020,   /* U+16B99 */
  0x0020,   /* U+16B9A */
  0x0020,   /* U+16B9B */
  0x0020,   /* U+16B9C */
  0x0020,   /* U+16B9D */
  0x0020,   /* U+16B9E */
  0x0020,   /* U+16B9F */
  0x0020,   /* U+16BA0 */
  0x0020,   /* U+16BA1 */
  0x0020,   /* U+16BA2 */
  0x0020,   /* U+16BA3 */
  0x0020,   /* U+16BA4 */
  0x0020,   /* U+16BA5 */
  0x0020,   /* U+16BA6 */
  0x0020,   /* U+16BA7 */
  0x0020,   /* U+16BA8 */
  0x0020,   /* U+16BA9 */
  0x0020,   /* U+16BAA */
  0x0020,   /* U+16BAB */
  0x0020,   /* U+16BAC */
  0x0020,   /* U+16BAD */
  0x0020,   /* U+16BAE */
  0x0020,   /* U+16BAF */
  0x0020,   /* U+16BB0 */
  0x0020,   /* U+16BB1 */
  0x0020,   /* U+16BB2 */
  0x0020,   /* U+16BB3 */
  0x0020,   /* U+16BB4 */
  0x0020,   /* U+16BB5 */
  0x0020,   /* U+16BB6 */
  0x0020,   /* U+16BB7 */
  0x0020,   /* U+16BB8 */
  0x0020,   /* U+16BB9 */
  0x0020,   /* U+16BBA */
  0x0020,   /* U+16BBB */
  0x0020,   /* U+16BBC */
  0x0020,   /* U+16BBD */
  0x0020,   /* U+16BBE */
  0x0020,   /* U+16BBF */
  0x0020,   /* U+16BC0 */
  0x0020,   /* U+16BC1 */
  0x0020,   /* U+16BC2 */
  0x0020,   /* U+16BC3 */
  0x0020,   /* U+16BC4 */
  0x0020,   /* U+16BC5 */
  0x0020,   /* U+16BC6 */
  0x0020,   /* U+16BC7 */
  0x0020,   /* U+16BC8 */
  0x0020,   /* U+16BC9 */
  0x0020,   /* U+16BCA */
  0x0020,   /* U+16BCB */
  0x0020,   /* U+16BCC */
  0x0020,   /* U+16BCD */
  0x0020,   /* U+16BCE */
  0x0020,   /* U+16BCF */
  0x0020,   /* U+16BD0 */
  0x0020,   /* U+16BD1 */
  0x0020,   /* U+16BD2 */
  0x0020,   /* U+16BD3 */
  0x0020,   /* U+16BD4 */
  0x0020,   /* U+16BD5 */
  0x0020,   /* U+16BD6 */
  0x0020,   /* U+16BD7 */
  0x0020,   /* U+16BD8 */
  0x0020,   /* U+16BD9 */
  0x0020,   /* U+16BDA */
  0x0020,   /* U+16BDB */
  0x0020,   /* U+16BDC */
  0x0020,   /* U+16BDD */
  0x0020,   /* U+16BDE */
  0x0020,   /* U+16BDF */
  0x0020,   /* U+16BE0 */
  0x0020,   /* U+16BE1 */
  0x0020,   /* U+16BE2 */
  0x0020,   /* U+16BE3 */
  0x0020,   /* U+16BE4 */
  0x0020,   /* U+16BE5 */
  0x0020,   /* U+16BE6 */
  0x0020,   /* U+16BE7 */
  0x0020,   /* U+16BE8 */
  0x0020,   /* U+16BE9 */
  0x0020,   /* U+16BEA */
  0x0020,   /* U+16BEB */
  0x0020,   /* U+16BEC */
  0x0020,   /* U+16BED */
  0x0020,   /* U+16BEE */
  0x0020,   /* U+16BEF */
  0x0020,   /* U+16BF0 */
  0x0020,   /* U+16BF1 */
  0x0020,   /* U+16BF2 */
  0x0020,   /* U+16BF3 */
  0x0020,   /* U+16BF4 */
  0x0020,   /* U+16BF5 */
  0x0020,   /* U+16BF6 */
  0x0020,   /* U+16BF7 */
  0x0020,   /* U+16BF8 */
  0x0020,   /* U+16BF9 */
  0x0020,   /* U+16BFA */
  0x0020,   /* U+16BFB */
  0x0020,   /* U+16BFC */
  0x0020,   /* U+16BFD */
  0x0020,   /* U+16BFE */
  0x0020,   /* U+16BFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+16B00 */
  0x0002,   /* U+16B01 */
  0x0002,   /* U+16B02 */
  0x0002,   /* U+16B03 */
  0x0002,   /* U+16B04 */
  0x0002,   /* U+16B05 */
  0x0002,   /* U+16B06 */
  0x0002,   /* U+16B07 */
  0x0002,   /* U+16B08 */
  0x0002,   /* U+16B09 */
  0x0002,   /* U+16B0A */
  0x0002,   /* U+16B0B */
  0x0002,   /* U+16B0C */
  0x0002,   /* U+16B0D */
  0x0002,   /* U+16B0E */
  0x0002,   /* U+16B0F */
  0x0002,   /* U+16B10 */
  0x0002,   /* U+16B11 */
  0x0002,   /* U+16B12 */
  0x0002,   /* U+16B13 */
  0x0002,   /* U+16B14 */
  0x0002,   /* U+16B15 */
  0x0002,   /* U+16B16 */
  0x0002,   /* U+16B17 */
  0x0002,   /* U+16B18 */
  0x0002,   /* U+16B19 */
  0x0002,   /* U+16B1A */
  0x0002,   /* U+16B1B */
  0x0002,   /* U+16B1C */
  0x0002,   /* U+16B1D */
  0x0002,   /* U+16B1E */
  0x0002,   /* U+16B1F */
  0x0002,   /* U+16B20 */
  0x0002,   /* U+16B21 */
  0x0002,   /* U+16B22 */
  0x0002,   /* U+16B23 */
  0x0002,   /* U+16B24 */
  0x0002,   /* U+16B25 */
  0x0002,   /* U+16B26 */
  0x0002,   /* U+16B27 */
  0x0002,   /* U+16B28 */
  0x0002,   /* U+16B29 */
  0x0002,   /* U+16B2A */
  0x0002,   /* U+16B2B */
  0x0002,   /* U+16B2C */
  0x0002,   /* U+16B2D */
  0x0002,   /* U+16B2E */
  0x0002,   /* U+16B2F */
  0x0002,   /* U+16B30 */
  0x0002,   /* U+16B31 */
  0x0002,   /* U+16B32 */
  0x0002,   /* U+16B33 */
  0x0002,   /* U+16B34 */
  0x0002,   /* U+16B35 */
  0x0002,   /* U+16B36 */
  0x0002,   /* U+16B37 */
  0x0002,   /* U+16B38 */
  0x0002,   /* U+16B39 */
  0x0002,   /* U+16B3A */
  0x0002,   /* U+16B3B */
  0x0002,   /* U+16B3C */
  0x0002,   /* U+16B3D */
  0x0002,   /* U+16B3E */
  0x0002,   /* U+16B3F */
  0x0002,   /* U+16B40 */
  0x0002,   /* U+16B41 */
  0x0002,   /* U+16B42 */
  0x0002,   /* U+16B43 */
  0x0002,   /* U+16B44 */
  0x0002,   /* U+16B45 */
  0x0002,   /* U+16B46 */
  0x0002,   /* U+16B47 */
  0x0002,   /* U+16B48 */
  0x0002,   /* U+16B49 */
  0x0002,   /* U+16B4A */
  0x0002,   /* U+16B4B */
  0x0002,   /* U+16B4C */
  0x0002,   /* U+16B4D */
  0x0002,   /* U+16B4E */
  0x0002,   /* U+16B4F */
  0x0002,   /* U+16B50 */
  0x0002,   /* U+16B51 */
  0x0002,   /* U+16B52 */
  0x0002,   /* U+16B53 */
  0x0002,   /* U+16B54 */
  0x0002,   /* U+16B55 */
  0x0002,   /* U+16B56 */
  0x0002,   /* U+16B57 */
  0x0002,   /* U+16B58 */
  0x0002,   /* U+16B59 */
  0x0002,   /* U+16B5A */
  0x0002,   /* U+16B5B */
  0x0002,   /* U+16B5C */
  0x0002,   /* U+16B5D */
  0x0002,   /* U+16B5E */
  0x0002,   /* U+16B5F */
  0x0002,   /* U+16B60 */
  0x0002,   /* U+16B61 */
  0x0002,   /* U+16B62 */
  0x0002,   /* U+16B63 */
  0x0002,   /* U+16B64 */
  0x0002,   /* U+16B65 */
  0x0002,   /* U+16B66 */
  0x0002,   /* U+16B67 */
  0x0002,   /* U+16B68 */
  0x0002,   /* U+16B69 */
  0x0002,   /* U+16B6A */
  0x0002,   /* U+16B6B */
  0x0002,   /* U+16B6C */
  0x0002,   /* U+16B6D */
  0x0002,   /* U+16B6E */
  0x0002,   /* U+16B6F */
  0x0002,   /* U+16B70 */
  0x0002,   /* U+16B71 */
  0x0002,   /* U+16B72 */
  0x0002,   /* U+16B73 */
  0x0002,   /* U+16B74 */
  0x0002,   /* U+16B75 */
  0x0002,   /* U+16B76 */
  0x0002,   /* U+16B77 */
  0x0002,   /* U+16B78 */
  0x0002,   /* U+16B79 */
  0x0002,   /* U+16B7A */
  0x0002,   /* U+16B7B */
  0x0002,   /* U+16B7C */
  0x0002,   /* U+16B7D */
  0x0002,   /* U+16B7E */
  0x0002,   /* U+16B7F */
  0x0002,   /* U+16B80 */
  0x0002,   /* U+16B81 */
  0x0002,   /* U+16B82 */
  0x0002,   /* U+16B83 */
  0x0002,   /* U+16B84 */
  0x0002,   /* U+16B85 */
  0x0002,   /* U+16B86 */
  0x0002,   /* U+16B87 */
  0x0002,   /* U+16B88 */
  0x0002,   /* U+16B89 */
  0x0002,   /* U+16B8A */
  0x0002,   /* U+16B8B */
  0x0002,   /* U+16B8C */
  0x0002,   /* U+16B8D */
  0x0002,   /* U+16B8E */
  0x0002,   /* U+16B8F */
  0x0002,   /* U+16B90 */
  0x0002,   /* U+16B91 */
  0x0002,   /* U+16B92 */
  0x0002,   /* U+16B93 */
  0x0002,   /* U+16B94 */
  0x0002,   /* U+16B95 */
  0x0002,   /* U+16B96 */
  0x0002,   /* U+16B97 */
  0x0002,   /* U+16B98 */
  0x0002,   /* U+16B99 */
  0x0002,   /* U+16B9A */
  0x0002,   /* U+16B9B */
  0x0002,   /* U+16B9C */
  0x0002,   /* U+16B9D */
  0x0002,   /* U+16B9E */
  0x0002,   /* U+16B9F */
  0x0002,   /* U+16BA0 */
  0x0002,   /* U+16BA1 */
  0x0002,   /* U+16BA2 */
  0x0002,   /* U+16BA3 */
  0x0002,   /* U+16BA4 */
  0x0002,   /* U+16BA5 */
  0x0002,   /* U+16BA6 */
  0x0002,   /* U+16BA7 */
  0x0002,   /* U+16BA8 */
  0x0002,   /* U+16BA9 */
  0x0002,   /* U+16BAA */
  0x0002,   /* U+16BAB */
  0x0002,   /* U+16BAC */
  0x0002,   /* U+16BAD */
  0x0002,   /* U+16BAE */
  0x0002,   /* U+16BAF */
  0x0002,   /* U+16BB0 */
  0x0002,   /* U+16BB1 */
  0x0002,   /* U+16BB2 */
  0x0002,   /* U+16BB3 */
  0x0002,   /* U+16BB4 */
  0x0002,   /* U+16BB5 */
  0x0002,   /* U+16BB6 */
  0x0002,   /* U+16BB7 */
  0x0002,   /* U+16BB8 */
  0x0002,   /* U+16BB9 */
  0x0002,   /* U+16BBA */
  0x0002,   /* U+16BBB */
  0x0002,   /* U+16BBC */
  0x0002,   /* U+16BBD */
  0x0002,   /* U+16BBE */
  0x0002,   /* U+16BBF */
  0x0002,   /* U+16BC0 */
  0x0002,   /* U+16BC1 */
  0x0002,   /* U+16BC2 */
  0x0002,   /* U+16BC3 */
  0x0002,   /* U+16BC4 */
  0x0002,   /* U+16BC5 */
  0x0002,   /* U+16BC6 */
  0x0002,   /* U+16BC7 */
  0x0002,   /* U+16BC8 */
  0x0002,   /* U+16BC9 */
  0x0002,   /* U+16BCA */
  0x0002,   /* U+16BCB */
  0x0002,   /* U+16BCC */
  0x0002,   /* U+16BCD */
  0x0002,   /* U+16BCE */
  0x0002,   /* U+16BCF */
  0x0002,   /* U+16BD0 */
  0x0002,   /* U+16BD1 */
  0x0002,   /* U+16BD2 */
  0x0002,   /* U+16BD3 */
  0x0002,   /* U+16BD4 */
  0x0002,   /* U+16BD5 */
  0x0002,   /* U+16BD6 */
  0x0002,   /* U+16BD7 */
  0x0002,   /* U+16BD8 */
  0x0002,   /* U+16BD9 */
  0x0002,   /* U+16BDA */
  0x0002,   /* U+16BDB */
  0x0002,   /* U+16BDC */
  0x0002,   /* U+16BDD */
  0x0002,   /* U+16BDE */
  0x0002,   /* U+16BDF */
  0x0002,   /* U+16BE0 */
  0x0002,   /* U+16BE1 */
  0x0002,   /* U+16BE2 */
  0x0002,   /* U+16BE3 */
  0x0002,   /* U+16BE4 */
  0x0002,   /* U+16BE5 */
  0x0002,   /* U+16BE6 */
  0x0002,   /* U+16BE7 */
  0x0002,   /* U+16BE8 */
  0x0002,   /* U+16BE9 */
  0x0002,   /* U+16BEA */
  0x0002,   /* U+16BEB */
  0x0002,   /* U+16BEC */
  0x0002,   /* U+16BED */
  0x0002,   /* U+16BEE */
  0x0002,   /* U+16BEF */
  0x0002,   /* U+16BF0 */
  0x0002,   /* U+16BF1 */
  0x0002,   /* U+16BF2 */
  0x0002,   /* U+16BF3 */
  0x0002,   /* U+16BF4 */
  0x0002,   /* U+16BF5 */
  0x0002,   /* U+16BF6 */
  0x0002,   /* U+16BF7 */
  0x0002,   /* U+16BF8 */
  0x0002,   /* U+16BF9 */
  0x0002,   /* U+16BFA */
  0x0002,   /* U+16BFB */
  0x0002,   /* U+16BFC */
  0x0002,   /* U+16BFD */
  0x0002,   /* U+16BFE */
  0x0002,   /* U+16BFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+16B00 */
  0x0000,   /* U+16B01 */
  0x0000,   /* U+16B02 */
  0x0000,   /* U+16B03 */
  0x0000,   /* U+16B04 */
  0x0000,   /* U+16B05 */
  0x0000,   /* U+16B06 */
  0x0000,   /* U+16B07 */
  0x0000,   /* U+16B08 */
  0x0000,   /* U+16B09 */
  0x0000,   /* U+16B0A */
  0x0000,   /* U+16B0B */
  0x0000,   /* U+16B0C */
  0x0000,   /* U+16B0D */
  0x0000,   /* U+16B0E */
  0x0000,   /* U+16B0F */
  0x0000,   /* U+16B10 */
  0x0000,   /* U+16B11 */
  0x0000,   /* U+16B12 */
  0x0000,   /* U+16B13 */
  0x0000,   /* U+16B14 */
  0x0000,   /* U+16B15 */
  0x0000,   /* U+16B16 */
  0x0000,   /* U+16B17 */
  0x0000,   /* U+16B18 */
  0x0000,   /* U+16B19 */
  0x0000,   /* U+16B1A */
  0x0000,   /* U+16B1B */
  0x0000,   /* U+16B1C */
  0x0000,   /* U+16B1D */
  0x0000,   /* U+16B1E */
  0x0000,   /* U+16B1F */
  0x0000,   /* U+16B20 */
  0x0000,   /* U+16B21 */
  0x0000,   /* U+16B22 */
  0x0000,   /* U+16B23 */
  0x0000,   /* U+16B24 */
  0x0000,   /* U+16B25 */
  0x0000,   /* U+16B26 */
  0x0000,   /* U+16B27 */
  0x0000,   /* U+16B28 */
  0x0000,   /* U+16B29 */
  0x0000,   /* U+16B2A */
  0x0000,   /* U+16B2B */
  0x0000,   /* U+16B2C */
  0x0000,   /* U+16B2D */
  0x0000,   /* U+16B2E */
  0x0000,   /* U+16B2F */
  0x0000,   /* U+16B30 */
  0x0000,   /* U+16B31 */
  0x0000,   /* U+16B32 */
  0x0000,   /* U+16B33 */
  0x0000,   /* U+16B34 */
  0x0000,   /* U+16B35 */
  0x0000,   /* U+16B36 */
  0x0000,   /* U+16B37 */
  0x0000,   /* U+16B38 */
  0x0000,   /* U+16B39 */
  0x0000,   /* U+16B3A */
  0x0000,   /* U+16B3B */
  0x0000,   /* U+16B3C */
  0x0000,   /* U+16B3D */
  0x0000,   /* U+16B3E */
  0x0000,   /* U+16B3F */
  0x0000,   /* U+16B40 */
  0x0000,   /* U+16B41 */
  0x0000,   /* U+16B42 */
  0x0000,   /* U+16B43 */
  0x0000,   /* U+16B44 */
  0x0000,   /* U+16B45 */
  0xEB46,   /* U+16B46 */
  0xEB47,   /* U+16B47 */
  0xEB48,   /* U+16B48 */
  0xEB49,   /* U+16B49 */
  0xEB4A,   /* U+16B4A */
  0xEB4B,   /* U+16B4B */
  0xEB4C,   /* U+16B4C */
  0xEB4D,   /* U+16B4D */
  0xEB4E,   /* U+16B4E */
  0xEB4F,   /* U+16B4F */
  0x0000,   /* U+16B50 */
  0x0000,   /* U+16B51 */
  0x0000,   /* U+16B52 */
  0x0000,   /* U+16B53 */
  0x0000,   /* U+16B54 */
  0x0000,   /* U+16B55 */
  0x0000,   /* U+16B56 */
  0x0000,   /* U+16B57 */
  0x0000,   /* U+16B58 */
  0x0000,   /* U+16B59 */
  0xEB5A,   /* U+16B5A */
  0x0000,   /* U+16B5B */
  0x0000,   /* U+16B5C */
  0x0000,   /* U+16B5D */
  0x0000,   /* U+16B5E */
  0x0000,   /* U+16B5F */
  0x0000,   /* U+16B60 */
  0x0000,   /* U+16B61 */
  0xEB62,   /* U+16B62 */
  0x0000,   /* U+16B63 */
  0x0000,   /* U+16B64 */
  0x0000,   /* U+16B65 */
  0x0000,   /* U+16B66 */
  0x0000,   /* U+16B67 */
  0x0000,   /* U+16B68 */
  0x0000,   /* U+16B69 */
  0x0000,   /* U+16B6A */
  0x0000,   /* U+16B6B */
  0x0000,   /* U+16B6C */
  0x0000,   /* U+16B6D */
  0x0000,   /* U+16B6E */
  0x0000,   /* U+16B6F */
  0x0000,   /* U+16B70 */
  0x0000,   /* U+16B71 */
  0x0000,   /* U+16B72 */
  0x0000,   /* U+16B73 */
  0x0000,   /* U+16B74 */
  0x0000,   /* U+16B75 */
  0x0000,   /* U+16B76 */
  0x0000,   /* U+16B77 */
  0xEB78,   /* U+16B78 */
  0xEB79,   /* U+16B79 */
  0xEB7A,   /* U+16B7A */
  0xEB7B,   /* U+16B7B */
  0xEB7C,   /* U+16B7C */
  0x0000,   /* U+16B7D */
  0x0000,   /* U+16B7E */
  0x0000,   /* U+16B7F */
  0x0000,   /* U+16B80 */
  0x0000,   /* U+16B81 */
  0x0000,   /* U+16B82 */
  0x0000,   /* U+16B83 */
  0x0000,   /* U+16B84 */
  0x0000,   /* U+16B85 */
  0x0000,   /* U+16B86 */
  0x0000,   /* U+16B87 */
  0x0000,   /* U+16B88 */
  0x0000,   /* U+16B89 */
  0x0000,   /* U+16B8A */
  0x0000,   /* U+16B8B */
  0x0000,   /* U+16B8C */
  0x0000,   /* U+16B8D */
  0x0000,   /* U+16B8E */
  0x0000,   /* U+16B8F */
  0xEB90,   /* U+16B90 */
  0xEB91,   /* U+16B91 */
  0xEB92,   /* U+16B92 */
  0xEB93,   /* U+16B93 */
  0xEB94,   /* U+16B94 */
  0xEB95,   /* U+16B95 */
  0xEB96,   /* U+16B96 */
  0xEB97,   /* U+16B97 */
  0xEB98,   /* U+16B98 */
  0xEB99,   /* U+16B99 */
  0xEB9A,   /* U+16B9A */
  0xEB9B,   /* U+16B9B */
  0xEB9C,   /* U+16B9C */
  0xEB9D,   /* U+16B9D */
  0xEB9E,   /* U+16B9E */
  0xEB9F,   /* U+16B9F */
  0xEBA0,   /* U+16BA0 */
  0xEBA1,   /* U+16BA1 */
  0xEBA2,   /* U+16BA2 */
  0xEBA3,   /* U+16BA3 */
  0xEBA4,   /* U+16BA4 */
  0xEBA5,   /* U+16BA5 */
  0xEBA6,   /* U+16BA6 */
  0xEBA7,   /* U+16BA7 */
  0xEBA8,   /* U+16BA8 */
  0xEBA9,   /* U+16BA9 */
  0xEBAA,   /* U+16BAA */
  0xEBAB,   /* U+16BAB */
  0xEBAC,   /* U+16BAC */
  0xEBAD,   /* U+16BAD */
  0xEBAE,   /* U+16BAE */
  0xEBAF,   /* U+16BAF */
  0xEBB0,   /* U+16BB0 */
  0xEBB1,   /* U+16BB1 */
  0xEBB2,   /* U+16BB2 */
  0xEBB3,   /* U+16BB3 */
  0xEBB4,   /* U+16BB4 */
  0xEBB5,   /* U+16BB5 */
  0xEBB6,   /* U+16BB6 */
  0xEBB7,   /* U+16BB7 */
  0xEBB8,   /* U+16BB8 */
  0xEBB9,   /* U+16BB9 */
  0xEBBA,   /* U+16BBA */
  0xEBBB,   /* U+16BBB */
  0xEBBC,   /* U+16BBC */
  0xEBBD,   /* U+16BBD */
  0xEBBE,   /* U+16BBE */
  0xEBBF,   /* U+16BBF */
  0xEBC0,   /* U+16BC0 */
  0xEBC1,   /* U+16BC1 */
  0xEBC2,   /* U+16BC2 */
  0xEBC3,   /* U+16BC3 */
  0xEBC4,   /* U+16BC4 */
  0xEBC5,   /* U+16BC5 */
  0xEBC6,   /* U+16BC6 */
  0xEBC7,   /* U+16BC7 */
  0xEBC8,   /* U+16BC8 */
  0xEBC9,   /* U+16BC9 */
  0xEBCA,   /* U+16BCA */
  0xEBCB,   /* U+16BCB */
  0xEBCC,   /* U+16BCC */
  0xEBCD,   /* U+16BCD */
  0xEBCE,   /* U+16BCE */
  0xEBCF,   /* U+16BCF */
  0xEBD0,   /* U+16BD0 */
  0xEBD1,   /* U+16BD1 */
  0xEBD2,   /* U+16BD2 */
  0xEBD3,   /* U+16BD3 */
  0xEBD4,   /* U+16BD4 */
  0xEBD5,   /* U+16BD5 */
  0xEBD6,   /* U+16BD6 */
  0xEBD7,   /* U+16BD7 */
  0xEBD8,   /* U+16BD8 */
  0xEBD9,   /* U+16BD9 */
  0xEBDA,   /* U+16BDA */
  0xEBDB,   /* U+16BDB */
  0xEBDC,   /* U+16BDC */
  0xEBDD,   /* U+16BDD */
  0xEBDE,   /* U+16BDE */
  0xEBDF,   /* U+16BDF */
  0xEBE0,   /* U+16BE0 */
  0xEBE1,   /* U+16BE1 */
  0xEBE2,   /* U+16BE2 */
  0xEBE3,   /* U+16BE3 */
  0xEBE4,   /* U+16BE4 */
  0xEBE5,   /* U+16BE5 */
  0xEBE6,   /* U+16BE6 */
  0xEBE7,   /* U+16BE7 */
  0xEBE8,   /* U+16BE8 */
  0xEBE9,   /* U+16BE9 */
  0xEBEA,   /* U+16BEA */
  0xEBEB,   /* U+16BEB */
  0xEBEC,   /* U+16BEC */
  0xEBED,   /* U+16BED */
  0xEBEE,   /* U+16BEE */
  0xEBEF,   /* U+16BEF */
  0xEBF0,   /* U+16BF0 */
  0xEBF1,   /* U+16BF1 */
  0xEBF2,   /* U+16BF2 */
  0xEBF3,   /* U+16BF3 */
  0xEBF4,   /* U+16BF4 */
  0xEBF5,   /* U+16BF5 */
  0xEBF6,   /* U+16BF6 */
  0xEBF7,   /* U+16BF7 */
  0xEBF8,   /* U+16BF8 */
  0xEBF9,   /* U+16BF9 */
  0xEBFA,   /* U+16BFA */
  0xEBFB,   /* U+16BFB */
  0xEBFC,   /* U+16BFC */
  0xEBFD,   /* U+16BFD */
  0xEBFE,   /* U+16BFE */
  0xEBFF,   /* U+16BFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+16B00 */
  0x0000,   /* U+16B01 */
  0x0000,   /* U+16B02 */
  0x0000,   /* U+16B03 */
  0x0000,   /* U+16B04 */
  0x0000,   /* U+16B05 */
  0x0000,   /* U+16B06 */
  0x0000,   /* U+16B07 */
  0x0000,   /* U+16B08 */
  0x0000,   /* U+16B09 */
  0x0000,   /* U+16B0A */
  0x0000,   /* U+16B0B */
  0x0000,   /* U+16B0C */
  0x0000,   /* U+16B0D */
  0x0000,   /* U+16B0E */
  0x0000,   /* U+16B0F */
  0x0000,   /* U+16B10 */
  0x0000,   /* U+16B11 */
  0x0000,   /* U+16B12 */
  0x0000,   /* U+16B13 */
  0x0000,   /* U+16B14 */
  0x0000,   /* U+16B15 */
  0x0000,   /* U+16B16 */
  0x0000,   /* U+16B17 */
  0x0000,   /* U+16B18 */
  0x0000,   /* U+16B19 */
  0x0000,   /* U+16B1A */
  0x0000,   /* U+16B1B */
  0x0000,   /* U+16B1C */
  0x0000,   /* U+16B1D */
  0x0000,   /* U+16B1E */
  0x0000,   /* U+16B1F */
  0x0000,   /* U+16B20 */
  0x0000,   /* U+16B21 */
  0x0000,   /* U+16B22 */
  0x0000,   /* U+16B23 */
  0x0000,   /* U+16B24 */
  0x0000,   /* U+16B25 */
  0x0000,   /* U+16B26 */
  0x0000,   /* U+16B27 */
  0x0000,   /* U+16B28 */
  0x0000,   /* U+16B29 */
  0x0000,   /* U+16B2A */
  0x0000,   /* U+16B2B */
  0x0000,   /* U+16B2C */
  0x0000,   /* U+16B2D */
  0x0000,   /* U+16B2E */
  0x0000,   /* U+16B2F */
  0x0000,   /* U+16B30 */
  0x0000,   /* U+16B31 */
  0x0000,   /* U+16B32 */
  0x0000,   /* U+16B33 */
  0x0000,   /* U+16B34 */
  0x0000,   /* U+16B35 */
  0x0000,   /* U+16B36 */
  0x0000,   /* U+16B37 */
  0x0000,   /* U+16B38 */
  0x0000,   /* U+16B39 */
  0x0000,   /* U+16B3A */
  0x0000,   /* U+16B3B */
  0x0000,   /* U+16B3C */
  0x0000,   /* U+16B3D */
  0x0000,   /* U+16B3E */
  0x0000,   /* U+16B3F */
  0x0000,   /* U+16B40 */
  0x0000,   /* U+16B41 */
  0x0000,   /* U+16B42 */
  0x0000,   /* U+16B43 */
  0x0000,   /* U+16B44 */
  0x0000,   /* U+16B45 */
  0x0000,   /* U+16B46 */
  0x0000,   /* U+16B47 */
  0x0000,   /* U+16B48 */
  0x0000,   /* U+16B49 */
  0x0000,   /* U+16B4A */
  0x0000,   /* U+16B4B */
  0x0000,   /* U+16B4C */
  0x0000,   /* U+16B4D */
  0x0000,   /* U+16B4E */
  0x0000,   /* U+16B4F */
  0x0000,   /* U+16B50 */
  0x0000,   /* U+16B51 */
  0x0000,   /* U+16B52 */
  0x0000,   /* U+16B53 */
  0x0000,   /* U+16B54 */
  0x0000,   /* U+16B55 */
  0x0000,   /* U+16B56 */
  0x0000,   /* U+16B57 */
  0x0000,   /* U+16B58 */
  0x0000,   /* U+16B59 */
  0x0000,   /* U+16B5A */
  0x0000,   /* U+16B5B */
  0x0000,   /* U+16B5C */
  0x0000,   /* U+16B5D */
  0x0000,   /* U+16B5E */
  0x0000,   /* U+16B5F */
  0x0000,   /* U+16B60 */
  0x0000,   /* U+16B61 */
  0x0000,   /* U+16B62 */
  0x0000,   /* U+16B63 */
  0x0000,   /* U+16B64 */
  0x0000,   /* U+16B65 */
  0x0000,   /* U+16B66 */
  0x0000,   /* U+16B67 */
  0x0000,   /* U+16B68 */
  0x0000,   /* U+16B69 */
  0x0000,   /* U+16B6A */
  0x0000,   /* U+16B6B */
  0x0000,   /* U+16B6C */
  0x0000,   /* U+16B6D */
  0x0000,   /* U+16B6E */
  0x0000,   /* U+16B6F */
  0x0000,   /* U+16B70 */
  0x0000,   /* U+16B71 */
  0x0000,   /* U+16B72 */
  0x0000,   /* U+16B73 */
  0x0000,   /* U+16B74 */
  0x0000,   /* U+16B75 */
  0x0000,   /* U+16B76 */
  0x0000,   /* U+16B77 */
  0x0000,   /* U+16B78 */
  0x0000,   /* U+16B79 */
  0x0000,   /* U+16B7A */
  0x0000,   /* U+16B7B */
  0x0000,   /* U+16B7C */
  0x0000,   /* U+16B7D */
  0x0000,   /* U+16B7E */
  0x0000,   /* U+16B7F */
  0x0000,   /* U+16B80 */
  0x0000,   /* U+16B81 */
  0x0000,   /* U+16B82 */
  0x0000,   /* U+16B83 */
  0x0000,   /* U+16B84 */
  0x0000,   /* U+16B85 */
  0x0000,   /* U+16B86 */
  0x0000,   /* U+16B87 */
  0x0000,   /* U+16B88 */
  0x0000,   /* U+16B89 */
  0x0000,   /* U+16B8A */
  0x0000,   /* U+16B8B */
  0x0000,   /* U+16B8C */
  0x0000,   /* U+16B8D */
  0x0000,   /* U+16B8E */
  0x0000,   /* U+16B8F */
  0x0000,   /* U+16B90 */
  0x0000,   /* U+16B91 */
  0x0000,   /* U+16B92 */
  0x0000,   /* U+16B93 */
  0x0000,   /* U+16B94 */
  0x0000,   /* U+16B95 */
  0x0000,   /* U+16B96 */
  0x0000,   /* U+16B97 */
  0x0000,   /* U+16B98 */
  0x0000,   /* U+16B99 */
  0x0000,   /* U+16B9A */
  0x0000,   /* U+16B9B */
  0x0000,   /* U+16B9C */
  0x0000,   /* U+16B9D */
  0x0000,   /* U+16B9E */
  0x0000,   /* U+16B9F */
  0x0000,   /* U+16BA0 */
  0x0000,   /* U+16BA1 */
  0x0000,   /* U+16BA2 */
  0x0000,   /* U+16BA3 */
  0x0000,   /* U+16BA4 */
  0x0000,   /* U+16BA5 */
  0x0000,   /* U+16BA6 */
  0x0000,   /* U+16BA7 */
  0x0000,   /* U+16BA8 */
  0x0000,   /* U+16BA9 */
  0x0000,   /* U+16BAA */
  0x0000,   /* U+16BAB */
  0x0000,   /* U+16BAC */
  0x0000,   /* U+16BAD */
  0x0000,   /* U+16BAE */
  0x0000,   /* U+16BAF */
  0x0000,   /* U+16BB0 */
  0x0000,   /* U+16BB1 */
  0x0000,   /* U+16BB2 */
  0x0000,   /* U+16BB3 */
  0x0000,   /* U+16BB4 */
  0x0000,   /* U+16BB5 */
  0x0000,   /* U+16BB6 */
  0x0000,   /* U+16BB7 */
  0x0000,   /* U+16BB8 */
  0x0000,   /* U+16BB9 */
  0x0000,   /* U+16BBA */
  0x0000,   /* U+16BBB */
  0x0000,   /* U+16BBC */
  0x0000,   /* U+16BBD */
  0x0000,   /* U+16BBE */
  0x0000,   /* U+16BBF */
  0x0000,   /* U+16BC0 */
  0x0000,   /* U+16BC1 */
  0x0000,   /* U+16BC2 */
  0x0000,   /* U+16BC3 */
  0x0000,   /* U+16BC4 */
  0x0000,   /* U+16BC5 */
  0x0000,   /* U+16BC6 */
  0x0000,   /* U+16BC7 */
  0x0000,   /* U+16BC8 */
  0x0000,   /* U+16BC9 */
  0x0000,   /* U+16BCA */
  0x0000,   /* U+16BCB */
  0x0000,   /* U+16BCC */
  0x0000,   /* U+16BCD */
  0x0000,   /* U+16BCE */
  0x0000,   /* U+16BCF */
  0x0000,   /* U+16BD0 */
  0x0000,   /* U+16BD1 */
  0x0000,   /* U+16BD2 */
  0x0000,   /* U+16BD3 */
  0x0000,   /* U+16BD4 */
  0x0000,   /* U+16BD5 */
  0x0000,   /* U+16BD6 */
  0x0000,   /* U+16BD7 */
  0x0000,   /* U+16BD8 */
  0x0000,   /* U+16BD9 */
  0x0000,   /* U+16BDA */
  0x0000,   /* U+16BDB */
  0x0000,   /* U+16BDC */
  0x0000,   /* U+16BDD */
  0x0000,   /* U+16BDE */
  0x0000,   /* U+16BDF */
  0x0000,   /* U+16BE0 */
  0x0000,   /* U+16BE1 */
  0x0000,   /* U+16BE2 */
  0x0000,   /* U+16BE3 */
  0x0000,   /* U+16BE4 */
  0x0000,   /* U+16BE5 */
  0x0000,   /* U+16BE6 */
  0x0000,   /* U+16BE7 */
  0x0000,   /* U+16BE8 */
  0x0000,   /* U+16BE9 */
  0x0000,   /* U+16BEA */
  0x0000,   /* U+16BEB */
  0x0000,   /* U+16BEC */
  0x0000,   /* U+16BED */
  0x0000,   /* U+16BEE */
  0x0000,   /* U+16BEF */
  0x0000,   /* U+16BF0 */
  0x0000,   /* U+16BF1 */
  0x0000,   /* U+16BF2 */
  0x0000,   /* U+16BF3 */
  0x0000,   /* U+16BF4 */
  0x0000,   /* U+16BF5 */
  0x0000,   /* U+16BF6 */
  0x0000,   /* U+16BF7 */
  0x0000,   /* U+16BF8 */
  0x0000,   /* U+16BF9 */
  0x0000,   /* U+16BFA */
  0x0000,   /* U+16BFB */
  0x0000,   /* U+16BFC */
  0x0000,   /* U+16BFD */
  0x0000,   /* U+16BFE */
  0x0000,   /* U+16BFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+16B00 */
  0x0000,   /* U+16B01 */
  0x0000,   /* U+16B02 */
  0x0000,   /* U+16B03 */
  0x0000,   /* U+16B04 */
  0x0000,   /* U+16B05 */
  0x0000,   /* U+16B06 */
  0x0000,   /* U+16B07 */
  0x0000,   /* U+16B08 */
  0x0000,   /* U+16B09 */
  0x0000,   /* U+16B0A */
  0x0000,   /* U+16B0B */
  0x0000,   /* U+16B0C */
  0x0000,   /* U+16B0D */
  0x0000,   /* U+16B0E */
  0x0000,   /* U+16B0F */
  0x0000,   /* U+16B10 */
  0x0000,   /* U+16B11 */
  0x0000,   /* U+16B12 */
  0x0000,   /* U+16B13 */
  0x0000,   /* U+16B14 */
  0x0000,   /* U+16B15 */
  0x0000,   /* U+16B16 */
  0x0000,   /* U+16B17 */
  0x0000,   /* U+16B18 */
  0x0000,   /* U+16B19 */
  0x0000,   /* U+16B1A */
  0x0000,   /* U+16B1B */
  0x0000,   /* U+16B1C */
  0x0000,   /* U+16B1D */
  0x0000,   /* U+16B1E */
  0x0000,   /* U+16B1F */
  0x0000,   /* U+16B20 */
  0x0000,   /* U+16B21 */
  0x0000,   /* U+16B22 */
  0x0000,   /* U+16B23 */
  0x0000,   /* U+16B24 */
  0x0000,   /* U+16B25 */
  0x0000,   /* U+16B26 */
  0x0000,   /* U+16B27 */
  0x0000,   /* U+16B28 */
  0x0000,   /* U+16B29 */
  0x0000,   /* U+16B2A */
  0x0000,   /* U+16B2B */
  0x0000,   /* U+16B2C */
  0x0000,   /* U+16B2D */
  0x0000,   /* U+16B2E */
  0x0000,   /* U+16B2F */
  0x0000,   /* U+16B30 */
  0x0000,   /* U+16B31 */
  0x0000,   /* U+16B32 */
  0x0000,   /* U+16B33 */
  0x0000,   /* U+16B34 */
  0x0000,   /* U+16B35 */
  0x0000,   /* U+16B36 */
  0x0000,   /* U+16B37 */
  0x0000,   /* U+16B38 */
  0x0000,   /* U+16B39 */
  0x0000,   /* U+16B3A */
  0x0000,   /* U+16B3B */
  0x0000,   /* U+16B3C */
  0x0000,   /* U+16B3D */
  0x0000,   /* U+16B3E */
  0x0000,   /* U+16B3F */
  0x0000,   /* U+16B40 */
  0x0000,   /* U+16B41 */
  0x0000,   /* U+16B42 */
  0x0000,   /* U+16B43 */
  0x0000,   /* U+16B44 */
  0x0000,   /* U+16B45 */
  0x0000,   /* U+16B46 */
  0x0000,   /* U+16B47 */
  0x0000,   /* U+16B48 */
  0x0000,   /* U+16B49 */
  0x0000,   /* U+16B4A */
  0x0000,   /* U+16B4B */
  0x0000,   /* U+16B4C */
  0x0000,   /* U+16B4D */
  0x0000,   /* U+16B4E */
  0x0000,   /* U+16B4F */
  0x0000,   /* U+16B50 */
  0x0000,   /* U+16B51 */
  0x0000,   /* U+16B52 */
  0x0000,   /* U+16B53 */
  0x0000,   /* U+16B54 */
  0x0000,   /* U+16B55 */
  0x0000,   /* U+16B56 */
  0x0000,   /* U+16B57 */
  0x0000,   /* U+16B58 */
  0x0000,   /* U+16B59 */
  0x0000,   /* U+16B5A */
  0x0000,   /* U+16B5B */
  0x0000,   /* U+16B5C */
  0x0000,   /* U+16B5D */
  0x0000,   /* U+16B5E */
  0x0000,   /* U+16B5F */
  0x0000,   /* U+16B60 */
  0x0000,   /* U+16B61 */
  0x0000,   /* U+16B62 */
  0x0000,   /* U+16B63 */
  0x0000,   /* U+16B64 */
  0x0000,   /* U+16B65 */
  0x0000,   /* U+16B66 */
  0x0000,   /* U+16B67 */
  0x0000,   /* U+16B68 */
  0x0000,   /* U+16B69 */
  0x0000,   /* U+16B6A */
  0x0000,   /* U+16B6B */
  0x0000,   /* U+16B6C */
  0x0000,   /* U+16B6D */
  0x0000,   /* U+16B6E */
  0x0000,   /* U+16B6F */
  0x0000,   /* U+16B70 */
  0x0000,   /* U+16B71 */
  0x0000,   /* U+16B72 */
  0x0000,   /* U+16B73 */
  0x0000,   /* U+16B74 */
  0x0000,   /* U+16B75 */
  0x0000,   /* U+16B76 */
  0x0000,   /* U+16B77 */
  0x0000,   /* U+16B78 */
  0x0000,   /* U+16B79 */
  0x0000,   /* U+16B7A */
  0x0000,   /* U+16B7B */
  0x0000,   /* U+16B7C */
  0x0000,   /* U+16B7D */
  0x0000,   /* U+16B7E */
  0x0000,   /* U+16B7F */
  0x0000,   /* U+16B80 */
  0x0000,   /* U+16B81 */
  0x0000,   /* U+16B82 */
  0x0000,   /* U+16B83 */
  0x0000,   /* U+16B84 */
  0x0000,   /* U+16B85 */
  0x0000,   /* U+16B86 */
  0x0000,   /* U+16B87 */
  0x0000,   /* U+16B88 */
  0x0000,   /* U+16B89 */
  0x0000,   /* U+16B8A */
  0x0000,   /* U+16B8B */
  0x0000,   /* U+16B8C */
  0x0000,   /* U+16B8D */
  0x0000,   /* U+16B8E */
  0x0000,   /* U+16B8F */
  0x0000,   /* U+16B90 */
  0x0000,   /* U+16B91 */
  0x0000,   /* U+16B92 */
  0x0000,   /* U+16B93 */
  0x0000,   /* U+16B94 */
  0x0000,   /* U+16B95 */
  0x0000,   /* U+16B96 */
  0x0000,   /* U+16B97 */
  0x0000,   /* U+16B98 */
  0x0000,   /* U+16B99 */
  0x0000,   /* U+16B9A */
  0x0000,   /* U+16B9B */
  0x0000,   /* U+16B9C */
  0x0000,   /* U+16B9D */
  0x0000,   /* U+16B9E */
  0x0000,   /* U+16B9F */
  0x0000,   /* U+16BA0 */
  0x0000,   /* U+16BA1 */
  0x0000,   /* U+16BA2 */
  0x0000,   /* U+16BA3 */
  0x0000,   /* U+16BA4 */
  0x0000,   /* U+16BA5 */
  0x0000,   /* U+16BA6 */
  0x0000,   /* U+16BA7 */
  0x0000,   /* U+16BA8 */
  0x0000,   /* U+16BA9 */
  0x0000,   /* U+16BAA */
  0x0000,   /* U+16BAB */
  0x0000,   /* U+16BAC */
  0x0000,   /* U+16BAD */
  0x0000,   /* U+16BAE */
  0x0000,   /* U+16BAF */
  0x0000,   /* U+16BB0 */
  0x0000,   /* U+16BB1 */
  0x0000,   /* U+16BB2 */
  0x0000,   /* U+16BB3 */
  0x0000,   /* U+16BB4 */
  0x0000,   /* U+16BB5 */
  0x0000,   /* U+16BB6 */
  0x0000,   /* U+16BB7 */
  0x0000,   /* U+16BB8 */
  0x0000,   /* U+16BB9 */
  0x0000,   /* U+16BBA */
  0x0000,   /* U+16BBB */
  0x0000,   /* U+16BBC */
  0x0000,   /* U+16BBD */
  0x0000,   /* U+16BBE */
  0x0000,   /* U+16BBF */
  0x0000,   /* U+16BC0 */
  0x0000,   /* U+16BC1 */
  0x0000,   /* U+16BC2 */
  0x0000,   /* U+16BC3 */
  0x0000,   /* U+16BC4 */
  0x0000,   /* U+16BC5 */
  0x0000,   /* U+16BC6 */
  0x0000,   /* U+16BC7 */
  0x0000,   /* U+16BC8 */
  0x0000,   /* U+16BC9 */
  0x0000,   /* U+16BCA */
  0x0000,   /* U+16BCB */
  0x0000,   /* U+16BCC */
  0x0000,   /* U+16BCD */
  0x0000,   /* U+16BCE */
  0x0000,   /* U+16BCF */
  0x0000,   /* U+16BD0 */
  0x0000,   /* U+16BD1 */
  0x0000,   /* U+16BD2 */
  0x0000,   /* U+16BD3 */
  0x0000,   /* U+16BD4 */
  0x0000,   /* U+16BD5 */
  0x0000,   /* U+16BD6 */
  0x0000,   /* U+16BD7 */
  0x0000,   /* U+16BD8 */
  0x0000,   /* U+16BD9 */
  0x0000,   /* U+16BDA */
  0x0000,   /* U+16BDB */
  0x0000,   /* U+16BDC */
  0x0000,   /* U+16BDD */
  0x0000,   /* U+16BDE */
  0x0000,   /* U+16BDF */
  0x0000,   /* U+16BE0 */
  0x0000,   /* U+16BE1 */
  0x0000,   /* U+16BE2 */
  0x0000,   /* U+16BE3 */
  0x0000,   /* U+16BE4 */
  0x0000,   /* U+16BE5 */
  0x0000,   /* U+16BE6 */
  0x0000,   /* U+16BE7 */
  0x0000,   /* U+16BE8 */
  0x0000,   /* U+16BE9 */
  0x0000,   /* U+16BEA */
  0x0000,   /* U+16BEB */
  0x0000,   /* U+16BEC */
  0x0000,   /* U+16BED */
  0x0000,   /* U+16BEE */
  0x0000,   /* U+16BEF */
  0x0000,   /* U+16BF0 */
  0x0000,   /* U+16BF1 */
  0x0000,   /* U+16BF2 */
  0x0000,   /* U+16BF3 */
  0x0000,   /* U+16BF4 */
  0x0000,   /* U+16BF5 */
  0x0000,   /* U+16BF6 */
  0x0000,   /* U+16BF7 */
  0x0000,   /* U+16BF8 */
  0x0000,   /* U+16BF9 */
  0x0000,   /* U+16BFA */
  0x0000,   /* U+16BFB */
  0x0000,   /* U+16BFC */
  0x0000,   /* U+16BFD */
  0x0000,   /* U+16BFE */
  0x0000,   /* U+16BFF */
};

uint16 uca900_p16F[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x427A,   /* U+16F00 */
  0x427B,   /* U+16F01 */
  0x427C,   /* U+16F02 */
  0x427D,   /* U+16F03 */
  0x427E,   /* U+16F04 */
  0x427F,   /* U+16F05 */
  0x427E,   /* U+16F06 */
  0x4280,   /* U+16F07 */
  0x4281,   /* U+16F08 */
  0x4282,   /* U+16F09 */
  0x4283,   /* U+16F0A */
  0x4284,   /* U+16F0B */
  0x4285,   /* U+16F0C */
  0x4286,   /* U+16F0D */
  0x4287,   /* U+16F0E */
  0x4288,   /* U+16F0F */
  0x4289,   /* U+16F10 */
  0x428A,   /* U+16F11 */
  0x428B,   /* U+16F12 */
  0x4289,   /* U+16F13 */
  0x428C,   /* U+16F14 */
  0x428D,   /* U+16F15 */
  0x428E,   /* U+16F16 */
  0x428F,   /* U+16F17 */
  0x4290,   /* U+16F18 */
  0x4291,   /* U+16F19 */
  0x4292,   /* U+16F1A */
  0x4293,   /* U+16F1B */
  0x4294,   /* U+16F1C */
  0x4295,   /* U+16F1D */
  0x4296,   /* U+16F1E */
  0x4297,   /* U+16F1F */
  0x4298,   /* U+16F20 */
  0x4299,   /* U+16F21 */
  0x429A,   /* U+16F22 */
  0x429B,   /* U+16F23 */
  0x429C,   /* U+16F24 */
  0x429B,   /* U+16F25 */
  0x429D,   /* U+16F26 */
  0x429E,   /* U+16F27 */
  0x429F,   /* U+16F28 */
  0x42A0,   /* U+16F29 */
  0x42A1,   /* U+16F2A */
  0x42A2,   /* U+16F2B */
  0x42A3,   /* U+16F2C */
  0x42A4,   /* U+16F2D */
  0x42A5,   /* U+16F2E */
  0x42A6,   /* U+16F2F */
  0x42A7,   /* U+16F30 */
  0x42A8,   /* U+16F31 */
  0x42A9,   /* U+16F32 */
  0x42AA,   /* U+16F33 */
  0x42AB,   /* U+16F34 */
  0x42AC,   /* U+16F35 */
  0x42AD,   /* U+16F36 */
  0x42AE,   /* U+16F37 */
  0x42AF,   /* U+16F38 */
  0x42B0,   /* U+16F39 */
  0x42B1,   /* U+16F3A */
  0x42B2,   /* U+16F3B */
  0x42B3,   /* U+16F3C */
  0x42B4,   /* U+16F3D */
  0x42B5,   /* U+16F3E */
  0x42B4,   /* U+16F3F */
  0x42B6,   /* U+16F40 */
  0x42B7,   /* U+16F41 */
  0x42B8,   /* U+16F42 */
  0x42B9,   /* U+16F43 */
  0x42BA,   /* U+16F44 */
  0xFBC2,   /* U+16F45 */
  0xFBC2,   /* U+16F46 */
  0xFBC2,   /* U+16F47 */
  0xFBC2,   /* U+16F48 */
  0xFBC2,   /* U+16F49 */
  0xFBC2,   /* U+16F4A */
  0xFBC2,   /* U+16F4B */
  0xFBC2,   /* U+16F4C */
  0xFBC2,   /* U+16F4D */
  0xFBC2,   /* U+16F4E */
  0xFBC2,   /* U+16F4F */
  0x42BB,   /* U+16F50 */
  0x42BC,   /* U+16F51 */
  0x42BD,   /* U+16F52 */
  0x42BE,   /* U+16F53 */
  0x42BF,   /* U+16F54 */
  0x42C0,   /* U+16F55 */
  0x42C1,   /* U+16F56 */
  0x42C2,   /* U+16F57 */
  0x42C3,   /* U+16F58 */
  0x42C4,   /* U+16F59 */
  0x42C5,   /* U+16F5A */
  0x42C6,   /* U+16F5B */
  0x42C7,   /* U+16F5C */
  0x42C8,   /* U+16F5D */
  0x42C9,   /* U+16F5E */
  0x42CA,   /* U+16F5F */
  0x42CB,   /* U+16F60 */
  0x42CC,   /* U+16F61 */
  0x42CD,   /* U+16F62 */
  0x42CE,   /* U+16F63 */
  0x42CF,   /* U+16F64 */
  0x42D0,   /* U+16F65 */
  0x42D1,   /* U+16F66 */
  0x42D2,   /* U+16F67 */
  0x42D3,   /* U+16F68 */
  0x42D4,   /* U+16F69 */
  0x42D5,   /* U+16F6A */
  0x42D6,   /* U+16F6B */
  0x42D7,   /* U+16F6C */
  0x42D8,   /* U+16F6D */
  0x42D9,   /* U+16F6E */
  0x42DA,   /* U+16F6F */
  0x42DB,   /* U+16F70 */
  0x42DC,   /* U+16F71 */
  0x42DD,   /* U+16F72 */
  0x42DE,   /* U+16F73 */
  0x42DF,   /* U+16F74 */
  0x42E0,   /* U+16F75 */
  0x42E1,   /* U+16F76 */
  0x42E2,   /* U+16F77 */
  0x42E3,   /* U+16F78 */
  0x42E4,   /* U+16F79 */
  0x42E5,   /* U+16F7A */
  0x42E6,   /* U+16F7B */
  0x42E7,   /* U+16F7C */
  0x42E8,   /* U+16F7D */
  0x42E9,   /* U+16F7E */
  0xFBC2,   /* U+16F7F */
  0xFBC2,   /* U+16F80 */
  0xFBC2,   /* U+16F81 */
  0xFBC2,   /* U+16F82 */
  0xFBC2,   /* U+16F83 */
  0xFBC2,   /* U+16F84 */
  0xFBC2,   /* U+16F85 */
  0xFBC2,   /* U+16F86 */
  0xFBC2,   /* U+16F87 */
  0xFBC2,   /* U+16F88 */
  0xFBC2,   /* U+16F89 */
  0xFBC2,   /* U+16F8A */
  0xFBC2,   /* U+16F8B */
  0xFBC2,   /* U+16F8C */
  0xFBC2,   /* U+16F8D */
  0xFBC2,   /* U+16F8E */
  0x42EA,   /* U+16F8F */
  0x42EB,   /* U+16F90 */
  0x42EC,   /* U+16F91 */
  0x42ED,   /* U+16F92 */
  0x42EE,   /* U+16F93 */
  0x42EF,   /* U+16F94 */
  0x42F0,   /* U+16F95 */
  0x42F1,   /* U+16F96 */
  0x42F2,   /* U+16F97 */
  0x42F3,   /* U+16F98 */
  0x42F4,   /* U+16F99 */
  0x42F5,   /* U+16F9A */
  0x42F6,   /* U+16F9B */
  0x42F7,   /* U+16F9C */
  0x42F8,   /* U+16F9D */
  0x42F9,   /* U+16F9E */
  0x42FA,   /* U+16F9F */
  0xFBC2,   /* U+16FA0 */
  0xFBC2,   /* U+16FA1 */
  0xFBC2,   /* U+16FA2 */
  0xFBC2,   /* U+16FA3 */
  0xFBC2,   /* U+16FA4 */
  0xFBC2,   /* U+16FA5 */
  0xFBC2,   /* U+16FA6 */
  0xFBC2,   /* U+16FA7 */
  0xFBC2,   /* U+16FA8 */
  0xFBC2,   /* U+16FA9 */
  0xFBC2,   /* U+16FAA */
  0xFBC2,   /* U+16FAB */
  0xFBC2,   /* U+16FAC */
  0xFBC2,   /* U+16FAD */
  0xFBC2,   /* U+16FAE */
  0xFBC2,   /* U+16FAF */
  0xFBC2,   /* U+16FB0 */
  0xFBC2,   /* U+16FB1 */
  0xFBC2,   /* U+16FB2 */
  0xFBC2,   /* U+16FB3 */
  0xFBC2,   /* U+16FB4 */
  0xFBC2,   /* U+16FB5 */
  0xFBC2,   /* U+16FB6 */
  0xFBC2,   /* U+16FB7 */
  0xFBC2,   /* U+16FB8 */
  0xFBC2,   /* U+16FB9 */
  0xFBC2,   /* U+16FBA */
  0xFBC2,   /* U+16FBB */
  0xFBC2,   /* U+16FBC */
  0xFBC2,   /* U+16FBD */
  0xFBC2,   /* U+16FBE */
  0xFBC2,   /* U+16FBF */
  0xFBC2,   /* U+16FC0 */
  0xFBC2,   /* U+16FC1 */
  0xFBC2,   /* U+16FC2 */
  0xFBC2,   /* U+16FC3 */
  0xFBC2,   /* U+16FC4 */
  0xFBC2,   /* U+16FC5 */
  0xFBC2,   /* U+16FC6 */
  0xFBC2,   /* U+16FC7 */
  0xFBC2,   /* U+16FC8 */
  0xFBC2,   /* U+16FC9 */
  0xFBC2,   /* U+16FCA */
  0xFBC2,   /* U+16FCB */
  0xFBC2,   /* U+16FCC */
  0xFBC2,   /* U+16FCD */
  0xFBC2,   /* U+16FCE */
  0xFBC2,   /* U+16FCF */
  0xFBC2,   /* U+16FD0 */
  0xFBC2,   /* U+16FD1 */
  0xFBC2,   /* U+16FD2 */
  0xFBC2,   /* U+16FD3 */
  0xFBC2,   /* U+16FD4 */
  0xFBC2,   /* U+16FD5 */
  0xFBC2,   /* U+16FD6 */
  0xFBC2,   /* U+16FD7 */
  0xFBC2,   /* U+16FD8 */
  0xFBC2,   /* U+16FD9 */
  0xFBC2,   /* U+16FDA */
  0xFBC2,   /* U+16FDB */
  0xFBC2,   /* U+16FDC */
  0xFBC2,   /* U+16FDD */
  0xFBC2,   /* U+16FDE */
  0xFBC2,   /* U+16FDF */
  0x1C09,   /* U+16FE0 */
  0xFBC2,   /* U+16FE1 */
  0xFBC2,   /* U+16FE2 */
  0xFBC2,   /* U+16FE3 */
  0xFBC2,   /* U+16FE4 */
  0xFBC2,   /* U+16FE5 */
  0xFBC2,   /* U+16FE6 */
  0xFBC2,   /* U+16FE7 */
  0xFBC2,   /* U+16FE8 */
  0xFBC2,   /* U+16FE9 */
  0xFBC2,   /* U+16FEA */
  0xFBC2,   /* U+16FEB */
  0xFBC2,   /* U+16FEC */
  0xFBC2,   /* U+16FED */
  0xFBC2,   /* U+16FEE */
  0xFBC2,   /* U+16FEF */
  0xFBC2,   /* U+16FF0 */
  0xFBC2,   /* U+16FF1 */
  0xFBC2,   /* U+16FF2 */
  0xFBC2,   /* U+16FF3 */
  0xFBC2,   /* U+16FF4 */
  0xFBC2,   /* U+16FF5 */
  0xFBC2,   /* U+16FF6 */
  0xFBC2,   /* U+16FF7 */
  0xFBC2,   /* U+16FF8 */
  0xFBC2,   /* U+16FF9 */
  0xFBC2,   /* U+16FFA */
  0xFBC2,   /* U+16FFB */
  0xFBC2,   /* U+16FFC */
  0xFBC2,   /* U+16FFD */
  0xFBC2,   /* U+16FFE */
  0xFBC2,   /* U+16FFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+16F00 */
  0x0020,   /* U+16F01 */
  0x0020,   /* U+16F02 */
  0x0020,   /* U+16F03 */
  0x0020,   /* U+16F04 */
  0x0020,   /* U+16F05 */
  0x0020,   /* U+16F06 */
  0x0020,   /* U+16F07 */
  0x0020,   /* U+16F08 */
  0x0020,   /* U+16F09 */
  0x0020,   /* U+16F0A */
  0x0020,   /* U+16F0B */
  0x0020,   /* U+16F0C */
  0x0020,   /* U+16F0D */
  0x0020,   /* U+16F0E */
  0x0020,   /* U+16F0F */
  0x0020,   /* U+16F10 */
  0x0020,   /* U+16F11 */
  0x0020,   /* U+16F12 */
  0x0020,   /* U+16F13 */
  0x0020,   /* U+16F14 */
  0x0020,   /* U+16F15 */
  0x0020,   /* U+16F16 */
  0x0020,   /* U+16F17 */
  0x0020,   /* U+16F18 */
  0x0020,   /* U+16F19 */
  0x0020,   /* U+16F1A */
  0x0020,   /* U+16F1B */
  0x0020,   /* U+16F1C */
  0x0020,   /* U+16F1D */
  0x0020,   /* U+16F1E */
  0x0020,   /* U+16F1F */
  0x0020,   /* U+16F20 */
  0x0020,   /* U+16F21 */
  0x0020,   /* U+16F22 */
  0x0020,   /* U+16F23 */
  0x0020,   /* U+16F24 */
  0x0020,   /* U+16F25 */
  0x0020,   /* U+16F26 */
  0x0020,   /* U+16F27 */
  0x0020,   /* U+16F28 */
  0x0020,   /* U+16F29 */
  0x0020,   /* U+16F2A */
  0x0020,   /* U+16F2B */
  0x0020,   /* U+16F2C */
  0x0020,   /* U+16F2D */
  0x0020,   /* U+16F2E */
  0x0020,   /* U+16F2F */
  0x0020,   /* U+16F30 */
  0x0020,   /* U+16F31 */
  0x0020,   /* U+16F32 */
  0x0020,   /* U+16F33 */
  0x0020,   /* U+16F34 */
  0x0020,   /* U+16F35 */
  0x0020,   /* U+16F36 */
  0x0020,   /* U+16F37 */
  0x0020,   /* U+16F38 */
  0x0020,   /* U+16F39 */
  0x0020,   /* U+16F3A */
  0x0020,   /* U+16F3B */
  0x0020,   /* U+16F3C */
  0x0020,   /* U+16F3D */
  0x0020,   /* U+16F3E */
  0x0020,   /* U+16F3F */
  0x0020,   /* U+16F40 */
  0x0020,   /* U+16F41 */
  0x0020,   /* U+16F42 */
  0x0020,   /* U+16F43 */
  0x0020,   /* U+16F44 */
  0x0020,   /* U+16F45 */
  0x0020,   /* U+16F46 */
  0x0020,   /* U+16F47 */
  0x0020,   /* U+16F48 */
  0x0020,   /* U+16F49 */
  0x0020,   /* U+16F4A */
  0x0020,   /* U+16F4B */
  0x0020,   /* U+16F4C */
  0x0020,   /* U+16F4D */
  0x0020,   /* U+16F4E */
  0x0020,   /* U+16F4F */
  0x0020,   /* U+16F50 */
  0x0020,   /* U+16F51 */
  0x0020,   /* U+16F52 */
  0x0020,   /* U+16F53 */
  0x0020,   /* U+16F54 */
  0x0020,   /* U+16F55 */
  0x0020,   /* U+16F56 */
  0x0020,   /* U+16F57 */
  0x0020,   /* U+16F58 */
  0x0020,   /* U+16F59 */
  0x0020,   /* U+16F5A */
  0x0020,   /* U+16F5B */
  0x0020,   /* U+16F5C */
  0x0020,   /* U+16F5D */
  0x0020,   /* U+16F5E */
  0x0020,   /* U+16F5F */
  0x0020,   /* U+16F60 */
  0x0020,   /* U+16F61 */
  0x0020,   /* U+16F62 */
  0x0020,   /* U+16F63 */
  0x0020,   /* U+16F64 */
  0x0020,   /* U+16F65 */
  0x0020,   /* U+16F66 */
  0x0020,   /* U+16F67 */
  0x0020,   /* U+16F68 */
  0x0020,   /* U+16F69 */
  0x0020,   /* U+16F6A */
  0x0020,   /* U+16F6B */
  0x0020,   /* U+16F6C */
  0x0020,   /* U+16F6D */
  0x0020,   /* U+16F6E */
  0x0020,   /* U+16F6F */
  0x0020,   /* U+16F70 */
  0x0020,   /* U+16F71 */
  0x0020,   /* U+16F72 */
  0x0020,   /* U+16F73 */
  0x0020,   /* U+16F74 */
  0x0020,   /* U+16F75 */
  0x0020,   /* U+16F76 */
  0x0020,   /* U+16F77 */
  0x0020,   /* U+16F78 */
  0x0020,   /* U+16F79 */
  0x0020,   /* U+16F7A */
  0x0020,   /* U+16F7B */
  0x0020,   /* U+16F7C */
  0x0020,   /* U+16F7D */
  0x0020,   /* U+16F7E */
  0x0020,   /* U+16F7F */
  0x0020,   /* U+16F80 */
  0x0020,   /* U+16F81 */
  0x0020,   /* U+16F82 */
  0x0020,   /* U+16F83 */
  0x0020,   /* U+16F84 */
  0x0020,   /* U+16F85 */
  0x0020,   /* U+16F86 */
  0x0020,   /* U+16F87 */
  0x0020,   /* U+16F88 */
  0x0020,   /* U+16F89 */
  0x0020,   /* U+16F8A */
  0x0020,   /* U+16F8B */
  0x0020,   /* U+16F8C */
  0x0020,   /* U+16F8D */
  0x0020,   /* U+16F8E */
  0x0020,   /* U+16F8F */
  0x0020,   /* U+16F90 */
  0x0020,   /* U+16F91 */
  0x0020,   /* U+16F92 */
  0x0020,   /* U+16F93 */
  0x0020,   /* U+16F94 */
  0x0020,   /* U+16F95 */
  0x0020,   /* U+16F96 */
  0x0020,   /* U+16F97 */
  0x0020,   /* U+16F98 */
  0x0020,   /* U+16F99 */
  0x0020,   /* U+16F9A */
  0x0020,   /* U+16F9B */
  0x0020,   /* U+16F9C */
  0x0020,   /* U+16F9D */
  0x0020,   /* U+16F9E */
  0x0020,   /* U+16F9F */
  0x0020,   /* U+16FA0 */
  0x0020,   /* U+16FA1 */
  0x0020,   /* U+16FA2 */
  0x0020,   /* U+16FA3 */
  0x0020,   /* U+16FA4 */
  0x0020,   /* U+16FA5 */
  0x0020,   /* U+16FA6 */
  0x0020,   /* U+16FA7 */
  0x0020,   /* U+16FA8 */
  0x0020,   /* U+16FA9 */
  0x0020,   /* U+16FAA */
  0x0020,   /* U+16FAB */
  0x0020,   /* U+16FAC */
  0x0020,   /* U+16FAD */
  0x0020,   /* U+16FAE */
  0x0020,   /* U+16FAF */
  0x0020,   /* U+16FB0 */
  0x0020,   /* U+16FB1 */
  0x0020,   /* U+16FB2 */
  0x0020,   /* U+16FB3 */
  0x0020,   /* U+16FB4 */
  0x0020,   /* U+16FB5 */
  0x0020,   /* U+16FB6 */
  0x0020,   /* U+16FB7 */
  0x0020,   /* U+16FB8 */
  0x0020,   /* U+16FB9 */
  0x0020,   /* U+16FBA */
  0x0020,   /* U+16FBB */
  0x0020,   /* U+16FBC */
  0x0020,   /* U+16FBD */
  0x0020,   /* U+16FBE */
  0x0020,   /* U+16FBF */
  0x0020,   /* U+16FC0 */
  0x0020,   /* U+16FC1 */
  0x0020,   /* U+16FC2 */
  0x0020,   /* U+16FC3 */
  0x0020,   /* U+16FC4 */
  0x0020,   /* U+16FC5 */
  0x0020,   /* U+16FC6 */
  0x0020,   /* U+16FC7 */
  0x0020,   /* U+16FC8 */
  0x0020,   /* U+16FC9 */
  0x0020,   /* U+16FCA */
  0x0020,   /* U+16FCB */
  0x0020,   /* U+16FCC */
  0x0020,   /* U+16FCD */
  0x0020,   /* U+16FCE */
  0x0020,   /* U+16FCF */
  0x0020,   /* U+16FD0 */
  0x0020,   /* U+16FD1 */
  0x0020,   /* U+16FD2 */
  0x0020,   /* U+16FD3 */
  0x0020,   /* U+16FD4 */
  0x0020,   /* U+16FD5 */
  0x0020,   /* U+16FD6 */
  0x0020,   /* U+16FD7 */
  0x0020,   /* U+16FD8 */
  0x0020,   /* U+16FD9 */
  0x0020,   /* U+16FDA */
  0x0020,   /* U+16FDB */
  0x0020,   /* U+16FDC */
  0x0020,   /* U+16FDD */
  0x0020,   /* U+16FDE */
  0x0020,   /* U+16FDF */
  0x0020,   /* U+16FE0 */
  0x0020,   /* U+16FE1 */
  0x0020,   /* U+16FE2 */
  0x0020,   /* U+16FE3 */
  0x0020,   /* U+16FE4 */
  0x0020,   /* U+16FE5 */
  0x0020,   /* U+16FE6 */
  0x0020,   /* U+16FE7 */
  0x0020,   /* U+16FE8 */
  0x0020,   /* U+16FE9 */
  0x0020,   /* U+16FEA */
  0x0020,   /* U+16FEB */
  0x0020,   /* U+16FEC */
  0x0020,   /* U+16FED */
  0x0020,   /* U+16FEE */
  0x0020,   /* U+16FEF */
  0x0020,   /* U+16FF0 */
  0x0020,   /* U+16FF1 */
  0x0020,   /* U+16FF2 */
  0x0020,   /* U+16FF3 */
  0x0020,   /* U+16FF4 */
  0x0020,   /* U+16FF5 */
  0x0020,   /* U+16FF6 */
  0x0020,   /* U+16FF7 */
  0x0020,   /* U+16FF8 */
  0x0020,   /* U+16FF9 */
  0x0020,   /* U+16FFA */
  0x0020,   /* U+16FFB */
  0x0020,   /* U+16FFC */
  0x0020,   /* U+16FFD */
  0x0020,   /* U+16FFE */
  0x0020,   /* U+16FFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+16F00 */
  0x0002,   /* U+16F01 */
  0x0002,   /* U+16F02 */
  0x0002,   /* U+16F03 */
  0x0002,   /* U+16F04 */
  0x0002,   /* U+16F05 */
  0x0004,   /* U+16F06 */
  0x0002,   /* U+16F07 */
  0x0002,   /* U+16F08 */
  0x0002,   /* U+16F09 */
  0x0002,   /* U+16F0A */
  0x0002,   /* U+16F0B */
  0x0002,   /* U+16F0C */
  0x0002,   /* U+16F0D */
  0x0002,   /* U+16F0E */
  0x0002,   /* U+16F0F */
  0x0002,   /* U+16F10 */
  0x0002,   /* U+16F11 */
  0x0002,   /* U+16F12 */
  0x0004,   /* U+16F13 */
  0x0002,   /* U+16F14 */
  0x0002,   /* U+16F15 */
  0x0002,   /* U+16F16 */
  0x0002,   /* U+16F17 */
  0x0002,   /* U+16F18 */
  0x0002,   /* U+16F19 */
  0x0002,   /* U+16F1A */
  0x0002,   /* U+16F1B */
  0x0002,   /* U+16F1C */
  0x0002,   /* U+16F1D */
  0x0002,   /* U+16F1E */
  0x0002,   /* U+16F1F */
  0x0002,   /* U+16F20 */
  0x0002,   /* U+16F21 */
  0x0002,   /* U+16F22 */
  0x0002,   /* U+16F23 */
  0x0002,   /* U+16F24 */
  0x0004,   /* U+16F25 */
  0x0002,   /* U+16F26 */
  0x0002,   /* U+16F27 */
  0x0002,   /* U+16F28 */
  0x0002,   /* U+16F29 */
  0x0002,   /* U+16F2A */
  0x0002,   /* U+16F2B */
  0x0002,   /* U+16F2C */
  0x0002,   /* U+16F2D */
  0x0002,   /* U+16F2E */
  0x0002,   /* U+16F2F */
  0x0002,   /* U+16F30 */
  0x0002,   /* U+16F31 */
  0x0002,   /* U+16F32 */
  0x0002,   /* U+16F33 */
  0x0002,   /* U+16F34 */
  0x0002,   /* U+16F35 */
  0x0002,   /* U+16F36 */
  0x0002,   /* U+16F37 */
  0x0002,   /* U+16F38 */
  0x0002,   /* U+16F39 */
  0x0002,   /* U+16F3A */
  0x0002,   /* U+16F3B */
  0x0002,   /* U+16F3C */
  0x0002,   /* U+16F3D */
  0x0002,   /* U+16F3E */
  0x0004,   /* U+16F3F */
  0x0002,   /* U+16F40 */
  0x0002,   /* U+16F41 */
  0x0002,   /* U+16F42 */
  0x0002,   /* U+16F43 */
  0x0002,   /* U+16F44 */
  0x0002,   /* U+16F45 */
  0x0002,   /* U+16F46 */
  0x0002,   /* U+16F47 */
  0x0002,   /* U+16F48 */
  0x0002,   /* U+16F49 */
  0x0002,   /* U+16F4A */
  0x0002,   /* U+16F4B */
  0x0002,   /* U+16F4C */
  0x0002,   /* U+16F4D */
  0x0002,   /* U+16F4E */
  0x0002,   /* U+16F4F */
  0x0002,   /* U+16F50 */
  0x0002,   /* U+16F51 */
  0x0002,   /* U+16F52 */
  0x0002,   /* U+16F53 */
  0x0002,   /* U+16F54 */
  0x0002,   /* U+16F55 */
  0x0002,   /* U+16F56 */
  0x0002,   /* U+16F57 */
  0x0002,   /* U+16F58 */
  0x0002,   /* U+16F59 */
  0x0002,   /* U+16F5A */
  0x0002,   /* U+16F5B */
  0x0002,   /* U+16F5C */
  0x0002,   /* U+16F5D */
  0x0002,   /* U+16F5E */
  0x0002,   /* U+16F5F */
  0x0002,   /* U+16F60 */
  0x0002,   /* U+16F61 */
  0x0002,   /* U+16F62 */
  0x0002,   /* U+16F63 */
  0x0002,   /* U+16F64 */
  0x0002,   /* U+16F65 */
  0x0002,   /* U+16F66 */
  0x0002,   /* U+16F67 */
  0x0002,   /* U+16F68 */
  0x0002,   /* U+16F69 */
  0x0002,   /* U+16F6A */
  0x0002,   /* U+16F6B */
  0x0002,   /* U+16F6C */
  0x0002,   /* U+16F6D */
  0x0002,   /* U+16F6E */
  0x0002,   /* U+16F6F */
  0x0002,   /* U+16F70 */
  0x0002,   /* U+16F71 */
  0x0002,   /* U+16F72 */
  0x0002,   /* U+16F73 */
  0x0002,   /* U+16F74 */
  0x0002,   /* U+16F75 */
  0x0002,   /* U+16F76 */
  0x0002,   /* U+16F77 */
  0x0002,   /* U+16F78 */
  0x0002,   /* U+16F79 */
  0x0002,   /* U+16F7A */
  0x0002,   /* U+16F7B */
  0x0002,   /* U+16F7C */
  0x0002,   /* U+16F7D */
  0x0002,   /* U+16F7E */
  0x0002,   /* U+16F7F */
  0x0002,   /* U+16F80 */
  0x0002,   /* U+16F81 */
  0x0002,   /* U+16F82 */
  0x0002,   /* U+16F83 */
  0x0002,   /* U+16F84 */
  0x0002,   /* U+16F85 */
  0x0002,   /* U+16F86 */
  0x0002,   /* U+16F87 */
  0x0002,   /* U+16F88 */
  0x0002,   /* U+16F89 */
  0x0002,   /* U+16F8A */
  0x0002,   /* U+16F8B */
  0x0002,   /* U+16F8C */
  0x0002,   /* U+16F8D */
  0x0002,   /* U+16F8E */
  0x0002,   /* U+16F8F */
  0x0002,   /* U+16F90 */
  0x0002,   /* U+16F91 */
  0x0002,   /* U+16F92 */
  0x0002,   /* U+16F93 */
  0x0002,   /* U+16F94 */
  0x0002,   /* U+16F95 */
  0x0002,   /* U+16F96 */
  0x0002,   /* U+16F97 */
  0x0002,   /* U+16F98 */
  0x0002,   /* U+16F99 */
  0x0002,   /* U+16F9A */
  0x0002,   /* U+16F9B */
  0x0002,   /* U+16F9C */
  0x0002,   /* U+16F9D */
  0x0002,   /* U+16F9E */
  0x0002,   /* U+16F9F */
  0x0002,   /* U+16FA0 */
  0x0002,   /* U+16FA1 */
  0x0002,   /* U+16FA2 */
  0x0002,   /* U+16FA3 */
  0x0002,   /* U+16FA4 */
  0x0002,   /* U+16FA5 */
  0x0002,   /* U+16FA6 */
  0x0002,   /* U+16FA7 */
  0x0002,   /* U+16FA8 */
  0x0002,   /* U+16FA9 */
  0x0002,   /* U+16FAA */
  0x0002,   /* U+16FAB */
  0x0002,   /* U+16FAC */
  0x0002,   /* U+16FAD */
  0x0002,   /* U+16FAE */
  0x0002,   /* U+16FAF */
  0x0002,   /* U+16FB0 */
  0x0002,   /* U+16FB1 */
  0x0002,   /* U+16FB2 */
  0x0002,   /* U+16FB3 */
  0x0002,   /* U+16FB4 */
  0x0002,   /* U+16FB5 */
  0x0002,   /* U+16FB6 */
  0x0002,   /* U+16FB7 */
  0x0002,   /* U+16FB8 */
  0x0002,   /* U+16FB9 */
  0x0002,   /* U+16FBA */
  0x0002,   /* U+16FBB */
  0x0002,   /* U+16FBC */
  0x0002,   /* U+16FBD */
  0x0002,   /* U+16FBE */
  0x0002,   /* U+16FBF */
  0x0002,   /* U+16FC0 */
  0x0002,   /* U+16FC1 */
  0x0002,   /* U+16FC2 */
  0x0002,   /* U+16FC3 */
  0x0002,   /* U+16FC4 */
  0x0002,   /* U+16FC5 */
  0x0002,   /* U+16FC6 */
  0x0002,   /* U+16FC7 */
  0x0002,   /* U+16FC8 */
  0x0002,   /* U+16FC9 */
  0x0002,   /* U+16FCA */
  0x0002,   /* U+16FCB */
  0x0002,   /* U+16FCC */
  0x0002,   /* U+16FCD */
  0x0002,   /* U+16FCE */
  0x0002,   /* U+16FCF */
  0x0002,   /* U+16FD0 */
  0x0002,   /* U+16FD1 */
  0x0002,   /* U+16FD2 */
  0x0002,   /* U+16FD3 */
  0x0002,   /* U+16FD4 */
  0x0002,   /* U+16FD5 */
  0x0002,   /* U+16FD6 */
  0x0002,   /* U+16FD7 */
  0x0002,   /* U+16FD8 */
  0x0002,   /* U+16FD9 */
  0x0002,   /* U+16FDA */
  0x0002,   /* U+16FDB */
  0x0002,   /* U+16FDC */
  0x0002,   /* U+16FDD */
  0x0002,   /* U+16FDE */
  0x0002,   /* U+16FDF */
  0x0002,   /* U+16FE0 */
  0x0002,   /* U+16FE1 */
  0x0002,   /* U+16FE2 */
  0x0002,   /* U+16FE3 */
  0x0002,   /* U+16FE4 */
  0x0002,   /* U+16FE5 */
  0x0002,   /* U+16FE6 */
  0x0002,   /* U+16FE7 */
  0x0002,   /* U+16FE8 */
  0x0002,   /* U+16FE9 */
  0x0002,   /* U+16FEA */
  0x0002,   /* U+16FEB */
  0x0002,   /* U+16FEC */
  0x0002,   /* U+16FED */
  0x0002,   /* U+16FEE */
  0x0002,   /* U+16FEF */
  0x0002,   /* U+16FF0 */
  0x0002,   /* U+16FF1 */
  0x0002,   /* U+16FF2 */
  0x0002,   /* U+16FF3 */
  0x0002,   /* U+16FF4 */
  0x0002,   /* U+16FF5 */
  0x0002,   /* U+16FF6 */
  0x0002,   /* U+16FF7 */
  0x0002,   /* U+16FF8 */
  0x0002,   /* U+16FF9 */
  0x0002,   /* U+16FFA */
  0x0002,   /* U+16FFB */
  0x0002,   /* U+16FFC */
  0x0002,   /* U+16FFD */
  0x0002,   /* U+16FFE */
  0x0002,   /* U+16FFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+16F00 */
  0x0000,   /* U+16F01 */
  0x0000,   /* U+16F02 */
  0x0000,   /* U+16F03 */
  0x0000,   /* U+16F04 */
  0x0000,   /* U+16F05 */
  0x0000,   /* U+16F06 */
  0x0000,   /* U+16F07 */
  0x0000,   /* U+16F08 */
  0x0000,   /* U+16F09 */
  0x0000,   /* U+16F0A */
  0x0000,   /* U+16F0B */
  0x0000,   /* U+16F0C */
  0x0000,   /* U+16F0D */
  0x0000,   /* U+16F0E */
  0x0000,   /* U+16F0F */
  0x0000,   /* U+16F10 */
  0x0000,   /* U+16F11 */
  0x0000,   /* U+16F12 */
  0x0000,   /* U+16F13 */
  0x0000,   /* U+16F14 */
  0x0000,   /* U+16F15 */
  0x0000,   /* U+16F16 */
  0x0000,   /* U+16F17 */
  0x0000,   /* U+16F18 */
  0x0000,   /* U+16F19 */
  0x0000,   /* U+16F1A */
  0x0000,   /* U+16F1B */
  0x0000,   /* U+16F1C */
  0x0000,   /* U+16F1D */
  0x0000,   /* U+16F1E */
  0x0000,   /* U+16F1F */
  0x0000,   /* U+16F20 */
  0x0000,   /* U+16F21 */
  0x0000,   /* U+16F22 */
  0x0000,   /* U+16F23 */
  0x0000,   /* U+16F24 */
  0x0000,   /* U+16F25 */
  0x0000,   /* U+16F26 */
  0x0000,   /* U+16F27 */
  0x0000,   /* U+16F28 */
  0x0000,   /* U+16F29 */
  0x0000,   /* U+16F2A */
  0x0000,   /* U+16F2B */
  0x0000,   /* U+16F2C */
  0x0000,   /* U+16F2D */
  0x0000,   /* U+16F2E */
  0x0000,   /* U+16F2F */
  0x0000,   /* U+16F30 */
  0x0000,   /* U+16F31 */
  0x0000,   /* U+16F32 */
  0x0000,   /* U+16F33 */
  0x0000,   /* U+16F34 */
  0x0000,   /* U+16F35 */
  0x0000,   /* U+16F36 */
  0x0000,   /* U+16F37 */
  0x0000,   /* U+16F38 */
  0x0000,   /* U+16F39 */
  0x0000,   /* U+16F3A */
  0x0000,   /* U+16F3B */
  0x0000,   /* U+16F3C */
  0x0000,   /* U+16F3D */
  0x0000,   /* U+16F3E */
  0x0000,   /* U+16F3F */
  0x0000,   /* U+16F40 */
  0x0000,   /* U+16F41 */
  0x0000,   /* U+16F42 */
  0x0000,   /* U+16F43 */
  0x0000,   /* U+16F44 */
  0xEF45,   /* U+16F45 */
  0xEF46,   /* U+16F46 */
  0xEF47,   /* U+16F47 */
  0xEF48,   /* U+16F48 */
  0xEF49,   /* U+16F49 */
  0xEF4A,   /* U+16F4A */
  0xEF4B,   /* U+16F4B */
  0xEF4C,   /* U+16F4C */
  0xEF4D,   /* U+16F4D */
  0xEF4E,   /* U+16F4E */
  0xEF4F,   /* U+16F4F */
  0x0000,   /* U+16F50 */
  0x0000,   /* U+16F51 */
  0x0000,   /* U+16F52 */
  0x0000,   /* U+16F53 */
  0x0000,   /* U+16F54 */
  0x0000,   /* U+16F55 */
  0x0000,   /* U+16F56 */
  0x0000,   /* U+16F57 */
  0x0000,   /* U+16F58 */
  0x0000,   /* U+16F59 */
  0x0000,   /* U+16F5A */
  0x0000,   /* U+16F5B */
  0x0000,   /* U+16F5C */
  0x0000,   /* U+16F5D */
  0x0000,   /* U+16F5E */
  0x0000,   /* U+16F5F */
  0x0000,   /* U+16F60 */
  0x0000,   /* U+16F61 */
  0x0000,   /* U+16F62 */
  0x0000,   /* U+16F63 */
  0x0000,   /* U+16F64 */
  0x0000,   /* U+16F65 */
  0x0000,   /* U+16F66 */
  0x0000,   /* U+16F67 */
  0x0000,   /* U+16F68 */
  0x0000,   /* U+16F69 */
  0x0000,   /* U+16F6A */
  0x0000,   /* U+16F6B */
  0x0000,   /* U+16F6C */
  0x0000,   /* U+16F6D */
  0x0000,   /* U+16F6E */
  0x0000,   /* U+16F6F */
  0x0000,   /* U+16F70 */
  0x0000,   /* U+16F71 */
  0x0000,   /* U+16F72 */
  0x0000,   /* U+16F73 */
  0x0000,   /* U+16F74 */
  0x0000,   /* U+16F75 */
  0x0000,   /* U+16F76 */
  0x0000,   /* U+16F77 */
  0x0000,   /* U+16F78 */
  0x0000,   /* U+16F79 */
  0x0000,   /* U+16F7A */
  0x0000,   /* U+16F7B */
  0x0000,   /* U+16F7C */
  0x0000,   /* U+16F7D */
  0x0000,   /* U+16F7E */
  0xEF7F,   /* U+16F7F */
  0xEF80,   /* U+16F80 */
  0xEF81,   /* U+16F81 */
  0xEF82,   /* U+16F82 */
  0xEF83,   /* U+16F83 */
  0xEF84,   /* U+16F84 */
  0xEF85,   /* U+16F85 */
  0xEF86,   /* U+16F86 */
  0xEF87,   /* U+16F87 */
  0xEF88,   /* U+16F88 */
  0xEF89,   /* U+16F89 */
  0xEF8A,   /* U+16F8A */
  0xEF8B,   /* U+16F8B */
  0xEF8C,   /* U+16F8C */
  0xEF8D,   /* U+16F8D */
  0xEF8E,   /* U+16F8E */
  0x0000,   /* U+16F8F */
  0x0000,   /* U+16F90 */
  0x0000,   /* U+16F91 */
  0x0000,   /* U+16F92 */
  0x0000,   /* U+16F93 */
  0x0000,   /* U+16F94 */
  0x0000,   /* U+16F95 */
  0x0000,   /* U+16F96 */
  0x0000,   /* U+16F97 */
  0x0000,   /* U+16F98 */
  0x0000,   /* U+16F99 */
  0x0000,   /* U+16F9A */
  0x0000,   /* U+16F9B */
  0x0000,   /* U+16F9C */
  0x0000,   /* U+16F9D */
  0x0000,   /* U+16F9E */
  0x0000,   /* U+16F9F */
  0xEFA0,   /* U+16FA0 */
  0xEFA1,   /* U+16FA1 */
  0xEFA2,   /* U+16FA2 */
  0xEFA3,   /* U+16FA3 */
  0xEFA4,   /* U+16FA4 */
  0xEFA5,   /* U+16FA5 */
  0xEFA6,   /* U+16FA6 */
  0xEFA7,   /* U+16FA7 */
  0xEFA8,   /* U+16FA8 */
  0xEFA9,   /* U+16FA9 */
  0xEFAA,   /* U+16FAA */
  0xEFAB,   /* U+16FAB */
  0xEFAC,   /* U+16FAC */
  0xEFAD,   /* U+16FAD */
  0xEFAE,   /* U+16FAE */
  0xEFAF,   /* U+16FAF */
  0xEFB0,   /* U+16FB0 */
  0xEFB1,   /* U+16FB1 */
  0xEFB2,   /* U+16FB2 */
  0xEFB3,   /* U+16FB3 */
  0xEFB4,   /* U+16FB4 */
  0xEFB5,   /* U+16FB5 */
  0xEFB6,   /* U+16FB6 */
  0xEFB7,   /* U+16FB7 */
  0xEFB8,   /* U+16FB8 */
  0xEFB9,   /* U+16FB9 */
  0xEFBA,   /* U+16FBA */
  0xEFBB,   /* U+16FBB */
  0xEFBC,   /* U+16FBC */
  0xEFBD,   /* U+16FBD */
  0xEFBE,   /* U+16FBE */
  0xEFBF,   /* U+16FBF */
  0xEFC0,   /* U+16FC0 */
  0xEFC1,   /* U+16FC1 */
  0xEFC2,   /* U+16FC2 */
  0xEFC3,   /* U+16FC3 */
  0xEFC4,   /* U+16FC4 */
  0xEFC5,   /* U+16FC5 */
  0xEFC6,   /* U+16FC6 */
  0xEFC7,   /* U+16FC7 */
  0xEFC8,   /* U+16FC8 */
  0xEFC9,   /* U+16FC9 */
  0xEFCA,   /* U+16FCA */
  0xEFCB,   /* U+16FCB */
  0xEFCC,   /* U+16FCC */
  0xEFCD,   /* U+16FCD */
  0xEFCE,   /* U+16FCE */
  0xEFCF,   /* U+16FCF */
  0xEFD0,   /* U+16FD0 */
  0xEFD1,   /* U+16FD1 */
  0xEFD2,   /* U+16FD2 */
  0xEFD3,   /* U+16FD3 */
  0xEFD4,   /* U+16FD4 */
  0xEFD5,   /* U+16FD5 */
  0xEFD6,   /* U+16FD6 */
  0xEFD7,   /* U+16FD7 */
  0xEFD8,   /* U+16FD8 */
  0xEFD9,   /* U+16FD9 */
  0xEFDA,   /* U+16FDA */
  0xEFDB,   /* U+16FDB */
  0xEFDC,   /* U+16FDC */
  0xEFDD,   /* U+16FDD */
  0xEFDE,   /* U+16FDE */
  0xEFDF,   /* U+16FDF */
  0x0000,   /* U+16FE0 */
  0xEFE1,   /* U+16FE1 */
  0xEFE2,   /* U+16FE2 */
  0xEFE3,   /* U+16FE3 */
  0xEFE4,   /* U+16FE4 */
  0xEFE5,   /* U+16FE5 */
  0xEFE6,   /* U+16FE6 */
  0xEFE7,   /* U+16FE7 */
  0xEFE8,   /* U+16FE8 */
  0xEFE9,   /* U+16FE9 */
  0xEFEA,   /* U+16FEA */
  0xEFEB,   /* U+16FEB */
  0xEFEC,   /* U+16FEC */
  0xEFED,   /* U+16FED */
  0xEFEE,   /* U+16FEE */
  0xEFEF,   /* U+16FEF */
  0xEFF0,   /* U+16FF0 */
  0xEFF1,   /* U+16FF1 */
  0xEFF2,   /* U+16FF2 */
  0xEFF3,   /* U+16FF3 */
  0xEFF4,   /* U+16FF4 */
  0xEFF5,   /* U+16FF5 */
  0xEFF6,   /* U+16FF6 */
  0xEFF7,   /* U+16FF7 */
  0xEFF8,   /* U+16FF8 */
  0xEFF9,   /* U+16FF9 */
  0xEFFA,   /* U+16FFA */
  0xEFFB,   /* U+16FFB */
  0xEFFC,   /* U+16FFC */
  0xEFFD,   /* U+16FFD */
  0xEFFE,   /* U+16FFE */
  0xEFFF,   /* U+16FFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+16F00 */
  0x0000,   /* U+16F01 */
  0x0000,   /* U+16F02 */
  0x0000,   /* U+16F03 */
  0x0000,   /* U+16F04 */
  0x0000,   /* U+16F05 */
  0x0000,   /* U+16F06 */
  0x0000,   /* U+16F07 */
  0x0000,   /* U+16F08 */
  0x0000,   /* U+16F09 */
  0x0000,   /* U+16F0A */
  0x0000,   /* U+16F0B */
  0x0000,   /* U+16F0C */
  0x0000,   /* U+16F0D */
  0x0000,   /* U+16F0E */
  0x0000,   /* U+16F0F */
  0x0000,   /* U+16F10 */
  0x0000,   /* U+16F11 */
  0x0000,   /* U+16F12 */
  0x0000,   /* U+16F13 */
  0x0000,   /* U+16F14 */
  0x0000,   /* U+16F15 */
  0x0000,   /* U+16F16 */
  0x0000,   /* U+16F17 */
  0x0000,   /* U+16F18 */
  0x0000,   /* U+16F19 */
  0x0000,   /* U+16F1A */
  0x0000,   /* U+16F1B */
  0x0000,   /* U+16F1C */
  0x0000,   /* U+16F1D */
  0x0000,   /* U+16F1E */
  0x0000,   /* U+16F1F */
  0x0000,   /* U+16F20 */
  0x0000,   /* U+16F21 */
  0x0000,   /* U+16F22 */
  0x0000,   /* U+16F23 */
  0x0000,   /* U+16F24 */
  0x0000,   /* U+16F25 */
  0x0000,   /* U+16F26 */
  0x0000,   /* U+16F27 */
  0x0000,   /* U+16F28 */
  0x0000,   /* U+16F29 */
  0x0000,   /* U+16F2A */
  0x0000,   /* U+16F2B */
  0x0000,   /* U+16F2C */
  0x0000,   /* U+16F2D */
  0x0000,   /* U+16F2E */
  0x0000,   /* U+16F2F */
  0x0000,   /* U+16F30 */
  0x0000,   /* U+16F31 */
  0x0000,   /* U+16F32 */
  0x0000,   /* U+16F33 */
  0x0000,   /* U+16F34 */
  0x0000,   /* U+16F35 */
  0x0000,   /* U+16F36 */
  0x0000,   /* U+16F37 */
  0x0000,   /* U+16F38 */
  0x0000,   /* U+16F39 */
  0x0000,   /* U+16F3A */
  0x0000,   /* U+16F3B */
  0x0000,   /* U+16F3C */
  0x0000,   /* U+16F3D */
  0x0000,   /* U+16F3E */
  0x0000,   /* U+16F3F */
  0x0000,   /* U+16F40 */
  0x0000,   /* U+16F41 */
  0x0000,   /* U+16F42 */
  0x0000,   /* U+16F43 */
  0x0000,   /* U+16F44 */
  0x0000,   /* U+16F45 */
  0x0000,   /* U+16F46 */
  0x0000,   /* U+16F47 */
  0x0000,   /* U+16F48 */
  0x0000,   /* U+16F49 */
  0x0000,   /* U+16F4A */
  0x0000,   /* U+16F4B */
  0x0000,   /* U+16F4C */
  0x0000,   /* U+16F4D */
  0x0000,   /* U+16F4E */
  0x0000,   /* U+16F4F */
  0x0000,   /* U+16F50 */
  0x0000,   /* U+16F51 */
  0x0000,   /* U+16F52 */
  0x0000,   /* U+16F53 */
  0x0000,   /* U+16F54 */
  0x0000,   /* U+16F55 */
  0x0000,   /* U+16F56 */
  0x0000,   /* U+16F57 */
  0x0000,   /* U+16F58 */
  0x0000,   /* U+16F59 */
  0x0000,   /* U+16F5A */
  0x0000,   /* U+16F5B */
  0x0000,   /* U+16F5C */
  0x0000,   /* U+16F5D */
  0x0000,   /* U+16F5E */
  0x0000,   /* U+16F5F */
  0x0000,   /* U+16F60 */
  0x0000,   /* U+16F61 */
  0x0000,   /* U+16F62 */
  0x0000,   /* U+16F63 */
  0x0000,   /* U+16F64 */
  0x0000,   /* U+16F65 */
  0x0000,   /* U+16F66 */
  0x0000,   /* U+16F67 */
  0x0000,   /* U+16F68 */
  0x0000,   /* U+16F69 */
  0x0000,   /* U+16F6A */
  0x0000,   /* U+16F6B */
  0x0000,   /* U+16F6C */
  0x0000,   /* U+16F6D */
  0x0000,   /* U+16F6E */
  0x0000,   /* U+16F6F */
  0x0000,   /* U+16F70 */
  0x0000,   /* U+16F71 */
  0x0000,   /* U+16F72 */
  0x0000,   /* U+16F73 */
  0x0000,   /* U+16F74 */
  0x0000,   /* U+16F75 */
  0x0000,   /* U+16F76 */
  0x0000,   /* U+16F77 */
  0x0000,   /* U+16F78 */
  0x0000,   /* U+16F79 */
  0x0000,   /* U+16F7A */
  0x0000,   /* U+16F7B */
  0x0000,   /* U+16F7C */
  0x0000,   /* U+16F7D */
  0x0000,   /* U+16F7E */
  0x0000,   /* U+16F7F */
  0x0000,   /* U+16F80 */
  0x0000,   /* U+16F81 */
  0x0000,   /* U+16F82 */
  0x0000,   /* U+16F83 */
  0x0000,   /* U+16F84 */
  0x0000,   /* U+16F85 */
  0x0000,   /* U+16F86 */
  0x0000,   /* U+16F87 */
  0x0000,   /* U+16F88 */
  0x0000,   /* U+16F89 */
  0x0000,   /* U+16F8A */
  0x0000,   /* U+16F8B */
  0x0000,   /* U+16F8C */
  0x0000,   /* U+16F8D */
  0x0000,   /* U+16F8E */
  0x0000,   /* U+16F8F */
  0x0000,   /* U+16F90 */
  0x0000,   /* U+16F91 */
  0x0000,   /* U+16F92 */
  0x0000,   /* U+16F93 */
  0x0000,   /* U+16F94 */
  0x0000,   /* U+16F95 */
  0x0000,   /* U+16F96 */
  0x0000,   /* U+16F97 */
  0x0000,   /* U+16F98 */
  0x0000,   /* U+16F99 */
  0x0000,   /* U+16F9A */
  0x0000,   /* U+16F9B */
  0x0000,   /* U+16F9C */
  0x0000,   /* U+16F9D */
  0x0000,   /* U+16F9E */
  0x0000,   /* U+16F9F */
  0x0000,   /* U+16FA0 */
  0x0000,   /* U+16FA1 */
  0x0000,   /* U+16FA2 */
  0x0000,   /* U+16FA3 */
  0x0000,   /* U+16FA4 */
  0x0000,   /* U+16FA5 */
  0x0000,   /* U+16FA6 */
  0x0000,   /* U+16FA7 */
  0x0000,   /* U+16FA8 */
  0x0000,   /* U+16FA9 */
  0x0000,   /* U+16FAA */
  0x0000,   /* U+16FAB */
  0x0000,   /* U+16FAC */
  0x0000,   /* U+16FAD */
  0x0000,   /* U+16FAE */
  0x0000,   /* U+16FAF */
  0x0000,   /* U+16FB0 */
  0x0000,   /* U+16FB1 */
  0x0000,   /* U+16FB2 */
  0x0000,   /* U+16FB3 */
  0x0000,   /* U+16FB4 */
  0x0000,   /* U+16FB5 */
  0x0000,   /* U+16FB6 */
  0x0000,   /* U+16FB7 */
  0x0000,   /* U+16FB8 */
  0x0000,   /* U+16FB9 */
  0x0000,   /* U+16FBA */
  0x0000,   /* U+16FBB */
  0x0000,   /* U+16FBC */
  0x0000,   /* U+16FBD */
  0x0000,   /* U+16FBE */
  0x0000,   /* U+16FBF */
  0x0000,   /* U+16FC0 */
  0x0000,   /* U+16FC1 */
  0x0000,   /* U+16FC2 */
  0x0000,   /* U+16FC3 */
  0x0000,   /* U+16FC4 */
  0x0000,   /* U+16FC5 */
  0x0000,   /* U+16FC6 */
  0x0000,   /* U+16FC7 */
  0x0000,   /* U+16FC8 */
  0x0000,   /* U+16FC9 */
  0x0000,   /* U+16FCA */
  0x0000,   /* U+16FCB */
  0x0000,   /* U+16FCC */
  0x0000,   /* U+16FCD */
  0x0000,   /* U+16FCE */
  0x0000,   /* U+16FCF */
  0x0000,   /* U+16FD0 */
  0x0000,   /* U+16FD1 */
  0x0000,   /* U+16FD2 */
  0x0000,   /* U+16FD3 */
  0x0000,   /* U+16FD4 */
  0x0000,   /* U+16FD5 */
  0x0000,   /* U+16FD6 */
  0x0000,   /* U+16FD7 */
  0x0000,   /* U+16FD8 */
  0x0000,   /* U+16FD9 */
  0x0000,   /* U+16FDA */
  0x0000,   /* U+16FDB */
  0x0000,   /* U+16FDC */
  0x0000,   /* U+16FDD */
  0x0000,   /* U+16FDE */
  0x0000,   /* U+16FDF */
  0x0000,   /* U+16FE0 */
  0x0000,   /* U+16FE1 */
  0x0000,   /* U+16FE2 */
  0x0000,   /* U+16FE3 */
  0x0000,   /* U+16FE4 */
  0x0000,   /* U+16FE5 */
  0x0000,   /* U+16FE6 */
  0x0000,   /* U+16FE7 */
  0x0000,   /* U+16FE8 */
  0x0000,   /* U+16FE9 */
  0x0000,   /* U+16FEA */
  0x0000,   /* U+16FEB */
  0x0000,   /* U+16FEC */
  0x0000,   /* U+16FED */
  0x0000,   /* U+16FEE */
  0x0000,   /* U+16FEF */
  0x0000,   /* U+16FF0 */
  0x0000,   /* U+16FF1 */
  0x0000,   /* U+16FF2 */
  0x0000,   /* U+16FF3 */
  0x0000,   /* U+16FF4 */
  0x0000,   /* U+16FF5 */
  0x0000,   /* U+16FF6 */
  0x0000,   /* U+16FF7 */
  0x0000,   /* U+16FF8 */
  0x0000,   /* U+16FF9 */
  0x0000,   /* U+16FFA */
  0x0000,   /* U+16FFB */
  0x0000,   /* U+16FFC */
  0x0000,   /* U+16FFD */
  0x0000,   /* U+16FFE */
  0x0000,   /* U+16FFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+16F00 */
  0x0000,   /* U+16F01 */
  0x0000,   /* U+16F02 */
  0x0000,   /* U+16F03 */
  0x0000,   /* U+16F04 */
  0x0000,   /* U+16F05 */
  0x0000,   /* U+16F06 */
  0x0000,   /* U+16F07 */
  0x0000,   /* U+16F08 */
  0x0000,   /* U+16F09 */
  0x0000,   /* U+16F0A */
  0x0000,   /* U+16F0B */
  0x0000,   /* U+16F0C */
  0x0000,   /* U+16F0D */
  0x0000,   /* U+16F0E */
  0x0000,   /* U+16F0F */
  0x0000,   /* U+16F10 */
  0x0000,   /* U+16F11 */
  0x0000,   /* U+16F12 */
  0x0000,   /* U+16F13 */
  0x0000,   /* U+16F14 */
  0x0000,   /* U+16F15 */
  0x0000,   /* U+16F16 */
  0x0000,   /* U+16F17 */
  0x0000,   /* U+16F18 */
  0x0000,   /* U+16F19 */
  0x0000,   /* U+16F1A */
  0x0000,   /* U+16F1B */
  0x0000,   /* U+16F1C */
  0x0000,   /* U+16F1D */
  0x0000,   /* U+16F1E */
  0x0000,   /* U+16F1F */
  0x0000,   /* U+16F20 */
  0x0000,   /* U+16F21 */
  0x0000,   /* U+16F22 */
  0x0000,   /* U+16F23 */
  0x0000,   /* U+16F24 */
  0x0000,   /* U+16F25 */
  0x0000,   /* U+16F26 */
  0x0000,   /* U+16F27 */
  0x0000,   /* U+16F28 */
  0x0000,   /* U+16F29 */
  0x0000,   /* U+16F2A */
  0x0000,   /* U+16F2B */
  0x0000,   /* U+16F2C */
  0x0000,   /* U+16F2D */
  0x0000,   /* U+16F2E */
  0x0000,   /* U+16F2F */
  0x0000,   /* U+16F30 */
  0x0000,   /* U+16F31 */
  0x0000,   /* U+16F32 */
  0x0000,   /* U+16F33 */
  0x0000,   /* U+16F34 */
  0x0000,   /* U+16F35 */
  0x0000,   /* U+16F36 */
  0x0000,   /* U+16F37 */
  0x0000,   /* U+16F38 */
  0x0000,   /* U+16F39 */
  0x0000,   /* U+16F3A */
  0x0000,   /* U+16F3B */
  0x0000,   /* U+16F3C */
  0x0000,   /* U+16F3D */
  0x0000,   /* U+16F3E */
  0x0000,   /* U+16F3F */
  0x0000,   /* U+16F40 */
  0x0000,   /* U+16F41 */
  0x0000,   /* U+16F42 */
  0x0000,   /* U+16F43 */
  0x0000,   /* U+16F44 */
  0x0000,   /* U+16F45 */
  0x0000,   /* U+16F46 */
  0x0000,   /* U+16F47 */
  0x0000,   /* U+16F48 */
  0x0000,   /* U+16F49 */
  0x0000,   /* U+16F4A */
  0x0000,   /* U+16F4B */
  0x0000,   /* U+16F4C */
  0x0000,   /* U+16F4D */
  0x0000,   /* U+16F4E */
  0x0000,   /* U+16F4F */
  0x0000,   /* U+16F50 */
  0x0000,   /* U+16F51 */
  0x0000,   /* U+16F52 */
  0x0000,   /* U+16F53 */
  0x0000,   /* U+16F54 */
  0x0000,   /* U+16F55 */
  0x0000,   /* U+16F56 */
  0x0000,   /* U+16F57 */
  0x0000,   /* U+16F58 */
  0x0000,   /* U+16F59 */
  0x0000,   /* U+16F5A */
  0x0000,   /* U+16F5B */
  0x0000,   /* U+16F5C */
  0x0000,   /* U+16F5D */
  0x0000,   /* U+16F5E */
  0x0000,   /* U+16F5F */
  0x0000,   /* U+16F60 */
  0x0000,   /* U+16F61 */
  0x0000,   /* U+16F62 */
  0x0000,   /* U+16F63 */
  0x0000,   /* U+16F64 */
  0x0000,   /* U+16F65 */
  0x0000,   /* U+16F66 */
  0x0000,   /* U+16F67 */
  0x0000,   /* U+16F68 */
  0x0000,   /* U+16F69 */
  0x0000,   /* U+16F6A */
  0x0000,   /* U+16F6B */
  0x0000,   /* U+16F6C */
  0x0000,   /* U+16F6D */
  0x0000,   /* U+16F6E */
  0x0000,   /* U+16F6F */
  0x0000,   /* U+16F70 */
  0x0000,   /* U+16F71 */
  0x0000,   /* U+16F72 */
  0x0000,   /* U+16F73 */
  0x0000,   /* U+16F74 */
  0x0000,   /* U+16F75 */
  0x0000,   /* U+16F76 */
  0x0000,   /* U+16F77 */
  0x0000,   /* U+16F78 */
  0x0000,   /* U+16F79 */
  0x0000,   /* U+16F7A */
  0x0000,   /* U+16F7B */
  0x0000,   /* U+16F7C */
  0x0000,   /* U+16F7D */
  0x0000,   /* U+16F7E */
  0x0000,   /* U+16F7F */
  0x0000,   /* U+16F80 */
  0x0000,   /* U+16F81 */
  0x0000,   /* U+16F82 */
  0x0000,   /* U+16F83 */
  0x0000,   /* U+16F84 */
  0x0000,   /* U+16F85 */
  0x0000,   /* U+16F86 */
  0x0000,   /* U+16F87 */
  0x0000,   /* U+16F88 */
  0x0000,   /* U+16F89 */
  0x0000,   /* U+16F8A */
  0x0000,   /* U+16F8B */
  0x0000,   /* U+16F8C */
  0x0000,   /* U+16F8D */
  0x0000,   /* U+16F8E */
  0x0000,   /* U+16F8F */
  0x0000,   /* U+16F90 */
  0x0000,   /* U+16F91 */
  0x0000,   /* U+16F92 */
  0x0000,   /* U+16F93 */
  0x0000,   /* U+16F94 */
  0x0000,   /* U+16F95 */
  0x0000,   /* U+16F96 */
  0x0000,   /* U+16F97 */
  0x0000,   /* U+16F98 */
  0x0000,   /* U+16F99 */
  0x0000,   /* U+16F9A */
  0x0000,   /* U+16F9B */
  0x0000,   /* U+16F9C */
  0x0000,   /* U+16F9D */
  0x0000,   /* U+16F9E */
  0x0000,   /* U+16F9F */
  0x0000,   /* U+16FA0 */
  0x0000,   /* U+16FA1 */
  0x0000,   /* U+16FA2 */
  0x0000,   /* U+16FA3 */
  0x0000,   /* U+16FA4 */
  0x0000,   /* U+16FA5 */
  0x0000,   /* U+16FA6 */
  0x0000,   /* U+16FA7 */
  0x0000,   /* U+16FA8 */
  0x0000,   /* U+16FA9 */
  0x0000,   /* U+16FAA */
  0x0000,   /* U+16FAB */
  0x0000,   /* U+16FAC */
  0x0000,   /* U+16FAD */
  0x0000,   /* U+16FAE */
  0x0000,   /* U+16FAF */
  0x0000,   /* U+16FB0 */
  0x0000,   /* U+16FB1 */
  0x0000,   /* U+16FB2 */
  0x0000,   /* U+16FB3 */
  0x0000,   /* U+16FB4 */
  0x0000,   /* U+16FB5 */
  0x0000,   /* U+16FB6 */
  0x0000,   /* U+16FB7 */
  0x0000,   /* U+16FB8 */
  0x0000,   /* U+16FB9 */
  0x0000,   /* U+16FBA */
  0x0000,   /* U+16FBB */
  0x0000,   /* U+16FBC */
  0x0000,   /* U+16FBD */
  0x0000,   /* U+16FBE */
  0x0000,   /* U+16FBF */
  0x0000,   /* U+16FC0 */
  0x0000,   /* U+16FC1 */
  0x0000,   /* U+16FC2 */
  0x0000,   /* U+16FC3 */
  0x0000,   /* U+16FC4 */
  0x0000,   /* U+16FC5 */
  0x0000,   /* U+16FC6 */
  0x0000,   /* U+16FC7 */
  0x0000,   /* U+16FC8 */
  0x0000,   /* U+16FC9 */
  0x0000,   /* U+16FCA */
  0x0000,   /* U+16FCB */
  0x0000,   /* U+16FCC */
  0x0000,   /* U+16FCD */
  0x0000,   /* U+16FCE */
  0x0000,   /* U+16FCF */
  0x0000,   /* U+16FD0 */
  0x0000,   /* U+16FD1 */
  0x0000,   /* U+16FD2 */
  0x0000,   /* U+16FD3 */
  0x0000,   /* U+16FD4 */
  0x0000,   /* U+16FD5 */
  0x0000,   /* U+16FD6 */
  0x0000,   /* U+16FD7 */
  0x0000,   /* U+16FD8 */
  0x0000,   /* U+16FD9 */
  0x0000,   /* U+16FDA */
  0x0000,   /* U+16FDB */
  0x0000,   /* U+16FDC */
  0x0000,   /* U+16FDD */
  0x0000,   /* U+16FDE */
  0x0000,   /* U+16FDF */
  0x0000,   /* U+16FE0 */
  0x0000,   /* U+16FE1 */
  0x0000,   /* U+16FE2 */
  0x0000,   /* U+16FE3 */
  0x0000,   /* U+16FE4 */
  0x0000,   /* U+16FE5 */
  0x0000,   /* U+16FE6 */
  0x0000,   /* U+16FE7 */
  0x0000,   /* U+16FE8 */
  0x0000,   /* U+16FE9 */
  0x0000,   /* U+16FEA */
  0x0000,   /* U+16FEB */
  0x0000,   /* U+16FEC */
  0x0000,   /* U+16FED */
  0x0000,   /* U+16FEE */
  0x0000,   /* U+16FEF */
  0x0000,   /* U+16FF0 */
  0x0000,   /* U+16FF1 */
  0x0000,   /* U+16FF2 */
  0x0000,   /* U+16FF3 */
  0x0000,   /* U+16FF4 */
  0x0000,   /* U+16FF5 */
  0x0000,   /* U+16FF6 */
  0x0000,   /* U+16FF7 */
  0x0000,   /* U+16FF8 */
  0x0000,   /* U+16FF9 */
  0x0000,   /* U+16FFA */
  0x0000,   /* U+16FFB */
  0x0000,   /* U+16FFC */
  0x0000,   /* U+16FFD */
  0x0000,   /* U+16FFE */
  0x0000,   /* U+16FFF */
};

uint16 uca900_p1B0[]= {
  /* Number of CEs for each character. */
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3D5D,   /* U+1B000 */
  0x3D80,   /* U+1B001 */
  0xFBC3,   /* U+1B002 */
  0xFBC3,   /* U+1B003 */
  0xFBC3,   /* U+1B004 */
  0xFBC3,   /* U+1B005 */
  0xFBC3,   /* U+1B006 */
  0xFBC3,   /* U+1B007 */
  0xFBC3,   /* U+1B008 */
  0xFBC3,   /* U+1B009 */
  0xFBC3,   /* U+1B00A */
  0xFBC3,   /* U+1B00B */
  0xFBC3,   /* U+1B00C */
  0xFBC3,   /* U+1B00D */
  0xFBC3,   /* U+1B00E */
  0xFBC3,   /* U+1B00F */
  0xFBC3,   /* U+1B010 */
  0xFBC3,   /* U+1B011 */
  0xFBC3,   /* U+1B012 */
  0xFBC3,   /* U+1B013 */
  0xFBC3,   /* U+1B014 */
  0xFBC3,   /* U+1B015 */
  0xFBC3,   /* U+1B016 */
  0xFBC3,   /* U+1B017 */
  0xFBC3,   /* U+1B018 */
  0xFBC3,   /* U+1B019 */
  0xFBC3,   /* U+1B01A */
  0xFBC3,   /* U+1B01B */
  0xFBC3,   /* U+1B01C */
  0xFBC3,   /* U+1B01D */
  0xFBC3,   /* U+1B01E */
  0xFBC3,   /* U+1B01F */
  0xFBC3,   /* U+1B020 */
  0xFBC3,   /* U+1B021 */
  0xFBC3,   /* U+1B022 */
  0xFBC3,   /* U+1B023 */
  0xFBC3,   /* U+1B024 */
  0xFBC3,   /* U+1B025 */
  0xFBC3,   /* U+1B026 */
  0xFBC3,   /* U+1B027 */
  0xFBC3,   /* U+1B028 */
  0xFBC3,   /* U+1B029 */
  0xFBC3,   /* U+1B02A */
  0xFBC3,   /* U+1B02B */
  0xFBC3,   /* U+1B02C */
  0xFBC3,   /* U+1B02D */
  0xFBC3,   /* U+1B02E */
  0xFBC3,   /* U+1B02F */
  0xFBC3,   /* U+1B030 */
  0xFBC3,   /* U+1B031 */
  0xFBC3,   /* U+1B032 */
  0xFBC3,   /* U+1B033 */
  0xFBC3,   /* U+1B034 */
  0xFBC3,   /* U+1B035 */
  0xFBC3,   /* U+1B036 */
  0xFBC3,   /* U+1B037 */
  0xFBC3,   /* U+1B038 */
  0xFBC3,   /* U+1B039 */
  0xFBC3,   /* U+1B03A */
  0xFBC3,   /* U+1B03B */
  0xFBC3,   /* U+1B03C */
  0xFBC3,   /* U+1B03D */
  0xFBC3,   /* U+1B03E */
  0xFBC3,   /* U+1B03F */
  0xFBC3,   /* U+1B040 */
  0xFBC3,   /* U+1B041 */
  0xFBC3,   /* U+1B042 */
  0xFBC3,   /* U+1B043 */
  0xFBC3,   /* U+1B044 */
  0xFBC3,   /* U+1B045 */
  0xFBC3,   /* U+1B046 */
  0xFBC3,   /* U+1B047 */
  0xFBC3,   /* U+1B048 */
  0xFBC3,   /* U+1B049 */
  0xFBC3,   /* U+1B04A */
  0xFBC3,   /* U+1B04B */
  0xFBC3,   /* U+1B04C */
  0xFBC3,   /* U+1B04D */
  0xFBC3,   /* U+1B04E */
  0xFBC3,   /* U+1B04F */
  0xFBC3,   /* U+1B050 */
  0xFBC3,   /* U+1B051 */
  0xFBC3,   /* U+1B052 */
  0xFBC3,   /* U+1B053 */
  0xFBC3,   /* U+1B054 */
  0xFBC3,   /* U+1B055 */
  0xFBC3,   /* U+1B056 */
  0xFBC3,   /* U+1B057 */
  0xFBC3,   /* U+1B058 */
  0xFBC3,   /* U+1B059 */
  0xFBC3,   /* U+1B05A */
  0xFBC3,   /* U+1B05B */
  0xFBC3,   /* U+1B05C */
  0xFBC3,   /* U+1B05D */
  0xFBC3,   /* U+1B05E */
  0xFBC3,   /* U+1B05F */
  0xFBC3,   /* U+1B060 */
  0xFBC3,   /* U+1B061 */
  0xFBC3,   /* U+1B062 */
  0xFBC3,   /* U+1B063 */
  0xFBC3,   /* U+1B064 */
  0xFBC3,   /* U+1B065 */
  0xFBC3,   /* U+1B066 */
  0xFBC3,   /* U+1B067 */
  0xFBC3,   /* U+1B068 */
  0xFBC3,   /* U+1B069 */
  0xFBC3,   /* U+1B06A */
  0xFBC3,   /* U+1B06B */
  0xFBC3,   /* U+1B06C */
  0xFBC3,   /* U+1B06D */
  0xFBC3,   /* U+1B06E */
  0xFBC3,   /* U+1B06F */
  0xFBC3,   /* U+1B070 */
  0xFBC3,   /* U+1B071 */
  0xFBC3,   /* U+1B072 */
  0xFBC3,   /* U+1B073 */
  0xFBC3,   /* U+1B074 */
  0xFBC3,   /* U+1B075 */
  0xFBC3,   /* U+1B076 */
  0xFBC3,   /* U+1B077 */
  0xFBC3,   /* U+1B078 */
  0xFBC3,   /* U+1B079 */
  0xFBC3,   /* U+1B07A */
  0xFBC3,   /* U+1B07B */
  0xFBC3,   /* U+1B07C */
  0xFBC3,   /* U+1B07D */
  0xFBC3,   /* U+1B07E */
  0xFBC3,   /* U+1B07F */
  0xFBC3,   /* U+1B080 */
  0xFBC3,   /* U+1B081 */
  0xFBC3,   /* U+1B082 */
  0xFBC3,   /* U+1B083 */
  0xFBC3,   /* U+1B084 */
  0xFBC3,   /* U+1B085 */
  0xFBC3,   /* U+1B086 */
  0xFBC3,   /* U+1B087 */
  0xFBC3,   /* U+1B088 */
  0xFBC3,   /* U+1B089 */
  0xFBC3,   /* U+1B08A */
  0xFBC3,   /* U+1B08B */
  0xFBC3,   /* U+1B08C */
  0xFBC3,   /* U+1B08D */
  0xFBC3,   /* U+1B08E */
  0xFBC3,   /* U+1B08F */
  0xFBC3,   /* U+1B090 */
  0xFBC3,   /* U+1B091 */
  0xFBC3,   /* U+1B092 */
  0xFBC3,   /* U+1B093 */
  0xFBC3,   /* U+1B094 */
  0xFBC3,   /* U+1B095 */
  0xFBC3,   /* U+1B096 */
  0xFBC3,   /* U+1B097 */
  0xFBC3,   /* U+1B098 */
  0xFBC3,   /* U+1B099 */
  0xFBC3,   /* U+1B09A */
  0xFBC3,   /* U+1B09B */
  0xFBC3,   /* U+1B09C */
  0xFBC3,   /* U+1B09D */
  0xFBC3,   /* U+1B09E */
  0xFBC3,   /* U+1B09F */
  0xFBC3,   /* U+1B0A0 */
  0xFBC3,   /* U+1B0A1 */
  0xFBC3,   /* U+1B0A2 */
  0xFBC3,   /* U+1B0A3 */
  0xFBC3,   /* U+1B0A4 */
  0xFBC3,   /* U+1B0A5 */
  0xFBC3,   /* U+1B0A6 */
  0xFBC3,   /* U+1B0A7 */
  0xFBC3,   /* U+1B0A8 */
  0xFBC3,   /* U+1B0A9 */
  0xFBC3,   /* U+1B0AA */
  0xFBC3,   /* U+1B0AB */
  0xFBC3,   /* U+1B0AC */
  0xFBC3,   /* U+1B0AD */
  0xFBC3,   /* U+1B0AE */
  0xFBC3,   /* U+1B0AF */
  0xFBC3,   /* U+1B0B0 */
  0xFBC3,   /* U+1B0B1 */
  0xFBC3,   /* U+1B0B2 */
  0xFBC3,   /* U+1B0B3 */
  0xFBC3,   /* U+1B0B4 */
  0xFBC3,   /* U+1B0B5 */
  0xFBC3,   /* U+1B0B6 */
  0xFBC3,   /* U+1B0B7 */
  0xFBC3,   /* U+1B0B8 */
  0xFBC3,   /* U+1B0B9 */
  0xFBC3,   /* U+1B0BA */
  0xFBC3,   /* U+1B0BB */
  0xFBC3,   /* U+1B0BC */
  0xFBC3,   /* U+1B0BD */
  0xFBC3,   /* U+1B0BE */
  0xFBC3,   /* U+1B0BF */
  0xFBC3,   /* U+1B0C0 */
  0xFBC3,   /* U+1B0C1 */
  0xFBC3,   /* U+1B0C2 */
  0xFBC3,   /* U+1B0C3 */
  0xFBC3,   /* U+1B0C4 */
  0xFBC3,   /* U+1B0C5 */
  0xFBC3,   /* U+1B0C6 */
  0xFBC3,   /* U+1B0C7 */
  0xFBC3,   /* U+1B0C8 */
  0xFBC3,   /* U+1B0C9 */
  0xFBC3,   /* U+1B0CA */
  0xFBC3,   /* U+1B0CB */
  0xFBC3,   /* U+1B0CC */
  0xFBC3,   /* U+1B0CD */
  0xFBC3,   /* U+1B0CE */
  0xFBC3,   /* U+1B0CF */
  0xFBC3,   /* U+1B0D0 */
  0xFBC3,   /* U+1B0D1 */
  0xFBC3,   /* U+1B0D2 */
  0xFBC3,   /* U+1B0D3 */
  0xFBC3,   /* U+1B0D4 */
  0xFBC3,   /* U+1B0D5 */
  0xFBC3,   /* U+1B0D6 */
  0xFBC3,   /* U+1B0D7 */
  0xFBC3,   /* U+1B0D8 */
  0xFBC3,   /* U+1B0D9 */
  0xFBC3,   /* U+1B0DA */
  0xFBC3,   /* U+1B0DB */
  0xFBC3,   /* U+1B0DC */
  0xFBC3,   /* U+1B0DD */
  0xFBC3,   /* U+1B0DE */
  0xFBC3,   /* U+1B0DF */
  0xFBC3,   /* U+1B0E0 */
  0xFBC3,   /* U+1B0E1 */
  0xFBC3,   /* U+1B0E2 */
  0xFBC3,   /* U+1B0E3 */
  0xFBC3,   /* U+1B0E4 */
  0xFBC3,   /* U+1B0E5 */
  0xFBC3,   /* U+1B0E6 */
  0xFBC3,   /* U+1B0E7 */
  0xFBC3,   /* U+1B0E8 */
  0xFBC3,   /* U+1B0E9 */
  0xFBC3,   /* U+1B0EA */
  0xFBC3,   /* U+1B0EB */
  0xFBC3,   /* U+1B0EC */
  0xFBC3,   /* U+1B0ED */
  0xFBC3,   /* U+1B0EE */
  0xFBC3,   /* U+1B0EF */
  0xFBC3,   /* U+1B0F0 */
  0xFBC3,   /* U+1B0F1 */
  0xFBC3,   /* U+1B0F2 */
  0xFBC3,   /* U+1B0F3 */
  0xFBC3,   /* U+1B0F4 */
  0xFBC3,   /* U+1B0F5 */
  0xFBC3,   /* U+1B0F6 */
  0xFBC3,   /* U+1B0F7 */
  0xFBC3,   /* U+1B0F8 */
  0xFBC3,   /* U+1B0F9 */
  0xFBC3,   /* U+1B0FA */
  0xFBC3,   /* U+1B0FB */
  0xFBC3,   /* U+1B0FC */
  0xFBC3,   /* U+1B0FD */
  0xFBC3,   /* U+1B0FE */
  0xFBC3,   /* U+1B0FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1B000 */
  0x0020,   /* U+1B001 */
  0x0020,   /* U+1B002 */
  0x0020,   /* U+1B003 */
  0x0020,   /* U+1B004 */
  0x0020,   /* U+1B005 */
  0x0020,   /* U+1B006 */
  0x0020,   /* U+1B007 */
  0x0020,   /* U+1B008 */
  0x0020,   /* U+1B009 */
  0x0020,   /* U+1B00A */
  0x0020,   /* U+1B00B */
  0x0020,   /* U+1B00C */
  0x0020,   /* U+1B00D */
  0x0020,   /* U+1B00E */
  0x0020,   /* U+1B00F */
  0x0020,   /* U+1B010 */
  0x0020,   /* U+1B011 */
  0x0020,   /* U+1B012 */
  0x0020,   /* U+1B013 */
  0x0020,   /* U+1B014 */
  0x0020,   /* U+1B015 */
  0x0020,   /* U+1B016 */
  0x0020,   /* U+1B017 */
  0x0020,   /* U+1B018 */
  0x0020,   /* U+1B019 */
  0x0020,   /* U+1B01A */
  0x0020,   /* U+1B01B */
  0x0020,   /* U+1B01C */
  0x0020,   /* U+1B01D */
  0x0020,   /* U+1B01E */
  0x0020,   /* U+1B01F */
  0x0020,   /* U+1B020 */
  0x0020,   /* U+1B021 */
  0x0020,   /* U+1B022 */
  0x0020,   /* U+1B023 */
  0x0020,   /* U+1B024 */
  0x0020,   /* U+1B025 */
  0x0020,   /* U+1B026 */
  0x0020,   /* U+1B027 */
  0x0020,   /* U+1B028 */
  0x0020,   /* U+1B029 */
  0x0020,   /* U+1B02A */
  0x0020,   /* U+1B02B */
  0x0020,   /* U+1B02C */
  0x0020,   /* U+1B02D */
  0x0020,   /* U+1B02E */
  0x0020,   /* U+1B02F */
  0x0020,   /* U+1B030 */
  0x0020,   /* U+1B031 */
  0x0020,   /* U+1B032 */
  0x0020,   /* U+1B033 */
  0x0020,   /* U+1B034 */
  0x0020,   /* U+1B035 */
  0x0020,   /* U+1B036 */
  0x0020,   /* U+1B037 */
  0x0020,   /* U+1B038 */
  0x0020,   /* U+1B039 */
  0x0020,   /* U+1B03A */
  0x0020,   /* U+1B03B */
  0x0020,   /* U+1B03C */
  0x0020,   /* U+1B03D */
  0x0020,   /* U+1B03E */
  0x0020,   /* U+1B03F */
  0x0020,   /* U+1B040 */
  0x0020,   /* U+1B041 */
  0x0020,   /* U+1B042 */
  0x0020,   /* U+1B043 */
  0x0020,   /* U+1B044 */
  0x0020,   /* U+1B045 */
  0x0020,   /* U+1B046 */
  0x0020,   /* U+1B047 */
  0x0020,   /* U+1B048 */
  0x0020,   /* U+1B049 */
  0x0020,   /* U+1B04A */
  0x0020,   /* U+1B04B */
  0x0020,   /* U+1B04C */
  0x0020,   /* U+1B04D */
  0x0020,   /* U+1B04E */
  0x0020,   /* U+1B04F */
  0x0020,   /* U+1B050 */
  0x0020,   /* U+1B051 */
  0x0020,   /* U+1B052 */
  0x0020,   /* U+1B053 */
  0x0020,   /* U+1B054 */
  0x0020,   /* U+1B055 */
  0x0020,   /* U+1B056 */
  0x0020,   /* U+1B057 */
  0x0020,   /* U+1B058 */
  0x0020,   /* U+1B059 */
  0x0020,   /* U+1B05A */
  0x0020,   /* U+1B05B */
  0x0020,   /* U+1B05C */
  0x0020,   /* U+1B05D */
  0x0020,   /* U+1B05E */
  0x0020,   /* U+1B05F */
  0x0020,   /* U+1B060 */
  0x0020,   /* U+1B061 */
  0x0020,   /* U+1B062 */
  0x0020,   /* U+1B063 */
  0x0020,   /* U+1B064 */
  0x0020,   /* U+1B065 */
  0x0020,   /* U+1B066 */
  0x0020,   /* U+1B067 */
  0x0020,   /* U+1B068 */
  0x0020,   /* U+1B069 */
  0x0020,   /* U+1B06A */
  0x0020,   /* U+1B06B */
  0x0020,   /* U+1B06C */
  0x0020,   /* U+1B06D */
  0x0020,   /* U+1B06E */
  0x0020,   /* U+1B06F */
  0x0020,   /* U+1B070 */
  0x0020,   /* U+1B071 */
  0x0020,   /* U+1B072 */
  0x0020,   /* U+1B073 */
  0x0020,   /* U+1B074 */
  0x0020,   /* U+1B075 */
  0x0020,   /* U+1B076 */
  0x0020,   /* U+1B077 */
  0x0020,   /* U+1B078 */
  0x0020,   /* U+1B079 */
  0x0020,   /* U+1B07A */
  0x0020,   /* U+1B07B */
  0x0020,   /* U+1B07C */
  0x0020,   /* U+1B07D */
  0x0020,   /* U+1B07E */
  0x0020,   /* U+1B07F */
  0x0020,   /* U+1B080 */
  0x0020,   /* U+1B081 */
  0x0020,   /* U+1B082 */
  0x0020,   /* U+1B083 */
  0x0020,   /* U+1B084 */
  0x0020,   /* U+1B085 */
  0x0020,   /* U+1B086 */
  0x0020,   /* U+1B087 */
  0x0020,   /* U+1B088 */
  0x0020,   /* U+1B089 */
  0x0020,   /* U+1B08A */
  0x0020,   /* U+1B08B */
  0x0020,   /* U+1B08C */
  0x0020,   /* U+1B08D */
  0x0020,   /* U+1B08E */
  0x0020,   /* U+1B08F */
  0x0020,   /* U+1B090 */
  0x0020,   /* U+1B091 */
  0x0020,   /* U+1B092 */
  0x0020,   /* U+1B093 */
  0x0020,   /* U+1B094 */
  0x0020,   /* U+1B095 */
  0x0020,   /* U+1B096 */
  0x0020,   /* U+1B097 */
  0x0020,   /* U+1B098 */
  0x0020,   /* U+1B099 */
  0x0020,   /* U+1B09A */
  0x0020,   /* U+1B09B */
  0x0020,   /* U+1B09C */
  0x0020,   /* U+1B09D */
  0x0020,   /* U+1B09E */
  0x0020,   /* U+1B09F */
  0x0020,   /* U+1B0A0 */
  0x0020,   /* U+1B0A1 */
  0x0020,   /* U+1B0A2 */
  0x0020,   /* U+1B0A3 */
  0x0020,   /* U+1B0A4 */
  0x0020,   /* U+1B0A5 */
  0x0020,   /* U+1B0A6 */
  0x0020,   /* U+1B0A7 */
  0x0020,   /* U+1B0A8 */
  0x0020,   /* U+1B0A9 */
  0x0020,   /* U+1B0AA */
  0x0020,   /* U+1B0AB */
  0x0020,   /* U+1B0AC */
  0x0020,   /* U+1B0AD */
  0x0020,   /* U+1B0AE */
  0x0020,   /* U+1B0AF */
  0x0020,   /* U+1B0B0 */
  0x0020,   /* U+1B0B1 */
  0x0020,   /* U+1B0B2 */
  0x0020,   /* U+1B0B3 */
  0x0020,   /* U+1B0B4 */
  0x0020,   /* U+1B0B5 */
  0x0020,   /* U+1B0B6 */
  0x0020,   /* U+1B0B7 */
  0x0020,   /* U+1B0B8 */
  0x0020,   /* U+1B0B9 */
  0x0020,   /* U+1B0BA */
  0x0020,   /* U+1B0BB */
  0x0020,   /* U+1B0BC */
  0x0020,   /* U+1B0BD */
  0x0020,   /* U+1B0BE */
  0x0020,   /* U+1B0BF */
  0x0020,   /* U+1B0C0 */
  0x0020,   /* U+1B0C1 */
  0x0020,   /* U+1B0C2 */
  0x0020,   /* U+1B0C3 */
  0x0020,   /* U+1B0C4 */
  0x0020,   /* U+1B0C5 */
  0x0020,   /* U+1B0C6 */
  0x0020,   /* U+1B0C7 */
  0x0020,   /* U+1B0C8 */
  0x0020,   /* U+1B0C9 */
  0x0020,   /* U+1B0CA */
  0x0020,   /* U+1B0CB */
  0x0020,   /* U+1B0CC */
  0x0020,   /* U+1B0CD */
  0x0020,   /* U+1B0CE */
  0x0020,   /* U+1B0CF */
  0x0020,   /* U+1B0D0 */
  0x0020,   /* U+1B0D1 */
  0x0020,   /* U+1B0D2 */
  0x0020,   /* U+1B0D3 */
  0x0020,   /* U+1B0D4 */
  0x0020,   /* U+1B0D5 */
  0x0020,   /* U+1B0D6 */
  0x0020,   /* U+1B0D7 */
  0x0020,   /* U+1B0D8 */
  0x0020,   /* U+1B0D9 */
  0x0020,   /* U+1B0DA */
  0x0020,   /* U+1B0DB */
  0x0020,   /* U+1B0DC */
  0x0020,   /* U+1B0DD */
  0x0020,   /* U+1B0DE */
  0x0020,   /* U+1B0DF */
  0x0020,   /* U+1B0E0 */
  0x0020,   /* U+1B0E1 */
  0x0020,   /* U+1B0E2 */
  0x0020,   /* U+1B0E3 */
  0x0020,   /* U+1B0E4 */
  0x0020,   /* U+1B0E5 */
  0x0020,   /* U+1B0E6 */
  0x0020,   /* U+1B0E7 */
  0x0020,   /* U+1B0E8 */
  0x0020,   /* U+1B0E9 */
  0x0020,   /* U+1B0EA */
  0x0020,   /* U+1B0EB */
  0x0020,   /* U+1B0EC */
  0x0020,   /* U+1B0ED */
  0x0020,   /* U+1B0EE */
  0x0020,   /* U+1B0EF */
  0x0020,   /* U+1B0F0 */
  0x0020,   /* U+1B0F1 */
  0x0020,   /* U+1B0F2 */
  0x0020,   /* U+1B0F3 */
  0x0020,   /* U+1B0F4 */
  0x0020,   /* U+1B0F5 */
  0x0020,   /* U+1B0F6 */
  0x0020,   /* U+1B0F7 */
  0x0020,   /* U+1B0F8 */
  0x0020,   /* U+1B0F9 */
  0x0020,   /* U+1B0FA */
  0x0020,   /* U+1B0FB */
  0x0020,   /* U+1B0FC */
  0x0020,   /* U+1B0FD */
  0x0020,   /* U+1B0FE */
  0x0020,   /* U+1B0FF */

  /* Tertiary weight 1 for each character. */
  0x0011,   /* U+1B000 */
  0x000E,   /* U+1B001 */
  0x0002,   /* U+1B002 */
  0x0002,   /* U+1B003 */
  0x0002,   /* U+1B004 */
  0x0002,   /* U+1B005 */
  0x0002,   /* U+1B006 */
  0x0002,   /* U+1B007 */
  0x0002,   /* U+1B008 */
  0x0002,   /* U+1B009 */
  0x0002,   /* U+1B00A */
  0x0002,   /* U+1B00B */
  0x0002,   /* U+1B00C */
  0x0002,   /* U+1B00D */
  0x0002,   /* U+1B00E */
  0x0002,   /* U+1B00F */
  0x0002,   /* U+1B010 */
  0x0002,   /* U+1B011 */
  0x0002,   /* U+1B012 */
  0x0002,   /* U+1B013 */
  0x0002,   /* U+1B014 */
  0x0002,   /* U+1B015 */
  0x0002,   /* U+1B016 */
  0x0002,   /* U+1B017 */
  0x0002,   /* U+1B018 */
  0x0002,   /* U+1B019 */
  0x0002,   /* U+1B01A */
  0x0002,   /* U+1B01B */
  0x0002,   /* U+1B01C */
  0x0002,   /* U+1B01D */
  0x0002,   /* U+1B01E */
  0x0002,   /* U+1B01F */
  0x0002,   /* U+1B020 */
  0x0002,   /* U+1B021 */
  0x0002,   /* U+1B022 */
  0x0002,   /* U+1B023 */
  0x0002,   /* U+1B024 */
  0x0002,   /* U+1B025 */
  0x0002,   /* U+1B026 */
  0x0002,   /* U+1B027 */
  0x0002,   /* U+1B028 */
  0x0002,   /* U+1B029 */
  0x0002,   /* U+1B02A */
  0x0002,   /* U+1B02B */
  0x0002,   /* U+1B02C */
  0x0002,   /* U+1B02D */
  0x0002,   /* U+1B02E */
  0x0002,   /* U+1B02F */
  0x0002,   /* U+1B030 */
  0x0002,   /* U+1B031 */
  0x0002,   /* U+1B032 */
  0x0002,   /* U+1B033 */
  0x0002,   /* U+1B034 */
  0x0002,   /* U+1B035 */
  0x0002,   /* U+1B036 */
  0x0002,   /* U+1B037 */
  0x0002,   /* U+1B038 */
  0x0002,   /* U+1B039 */
  0x0002,   /* U+1B03A */
  0x0002,   /* U+1B03B */
  0x0002,   /* U+1B03C */
  0x0002,   /* U+1B03D */
  0x0002,   /* U+1B03E */
  0x0002,   /* U+1B03F */
  0x0002,   /* U+1B040 */
  0x0002,   /* U+1B041 */
  0x0002,   /* U+1B042 */
  0x0002,   /* U+1B043 */
  0x0002,   /* U+1B044 */
  0x0002,   /* U+1B045 */
  0x0002,   /* U+1B046 */
  0x0002,   /* U+1B047 */
  0x0002,   /* U+1B048 */
  0x0002,   /* U+1B049 */
  0x0002,   /* U+1B04A */
  0x0002,   /* U+1B04B */
  0x0002,   /* U+1B04C */
  0x0002,   /* U+1B04D */
  0x0002,   /* U+1B04E */
  0x0002,   /* U+1B04F */
  0x0002,   /* U+1B050 */
  0x0002,   /* U+1B051 */
  0x0002,   /* U+1B052 */
  0x0002,   /* U+1B053 */
  0x0002,   /* U+1B054 */
  0x0002,   /* U+1B055 */
  0x0002,   /* U+1B056 */
  0x0002,   /* U+1B057 */
  0x0002,   /* U+1B058 */
  0x0002,   /* U+1B059 */
  0x0002,   /* U+1B05A */
  0x0002,   /* U+1B05B */
  0x0002,   /* U+1B05C */
  0x0002,   /* U+1B05D */
  0x0002,   /* U+1B05E */
  0x0002,   /* U+1B05F */
  0x0002,   /* U+1B060 */
  0x0002,   /* U+1B061 */
  0x0002,   /* U+1B062 */
  0x0002,   /* U+1B063 */
  0x0002,   /* U+1B064 */
  0x0002,   /* U+1B065 */
  0x0002,   /* U+1B066 */
  0x0002,   /* U+1B067 */
  0x0002,   /* U+1B068 */
  0x0002,   /* U+1B069 */
  0x0002,   /* U+1B06A */
  0x0002,   /* U+1B06B */
  0x0002,   /* U+1B06C */
  0x0002,   /* U+1B06D */
  0x0002,   /* U+1B06E */
  0x0002,   /* U+1B06F */
  0x0002,   /* U+1B070 */
  0x0002,   /* U+1B071 */
  0x0002,   /* U+1B072 */
  0x0002,   /* U+1B073 */
  0x0002,   /* U+1B074 */
  0x0002,   /* U+1B075 */
  0x0002,   /* U+1B076 */
  0x0002,   /* U+1B077 */
  0x0002,   /* U+1B078 */
  0x0002,   /* U+1B079 */
  0x0002,   /* U+1B07A */
  0x0002,   /* U+1B07B */
  0x0002,   /* U+1B07C */
  0x0002,   /* U+1B07D */
  0x0002,   /* U+1B07E */
  0x0002,   /* U+1B07F */
  0x0002,   /* U+1B080 */
  0x0002,   /* U+1B081 */
  0x0002,   /* U+1B082 */
  0x0002,   /* U+1B083 */
  0x0002,   /* U+1B084 */
  0x0002,   /* U+1B085 */
  0x0002,   /* U+1B086 */
  0x0002,   /* U+1B087 */
  0x0002,   /* U+1B088 */
  0x0002,   /* U+1B089 */
  0x0002,   /* U+1B08A */
  0x0002,   /* U+1B08B */
  0x0002,   /* U+1B08C */
  0x0002,   /* U+1B08D */
  0x0002,   /* U+1B08E */
  0x0002,   /* U+1B08F */
  0x0002,   /* U+1B090 */
  0x0002,   /* U+1B091 */
  0x0002,   /* U+1B092 */
  0x0002,   /* U+1B093 */
  0x0002,   /* U+1B094 */
  0x0002,   /* U+1B095 */
  0x0002,   /* U+1B096 */
  0x0002,   /* U+1B097 */
  0x0002,   /* U+1B098 */
  0x0002,   /* U+1B099 */
  0x0002,   /* U+1B09A */
  0x0002,   /* U+1B09B */
  0x0002,   /* U+1B09C */
  0x0002,   /* U+1B09D */
  0x0002,   /* U+1B09E */
  0x0002,   /* U+1B09F */
  0x0002,   /* U+1B0A0 */
  0x0002,   /* U+1B0A1 */
  0x0002,   /* U+1B0A2 */
  0x0002,   /* U+1B0A3 */
  0x0002,   /* U+1B0A4 */
  0x0002,   /* U+1B0A5 */
  0x0002,   /* U+1B0A6 */
  0x0002,   /* U+1B0A7 */
  0x0002,   /* U+1B0A8 */
  0x0002,   /* U+1B0A9 */
  0x0002,   /* U+1B0AA */
  0x0002,   /* U+1B0AB */
  0x0002,   /* U+1B0AC */
  0x0002,   /* U+1B0AD */
  0x0002,   /* U+1B0AE */
  0x0002,   /* U+1B0AF */
  0x0002,   /* U+1B0B0 */
  0x0002,   /* U+1B0B1 */
  0x0002,   /* U+1B0B2 */
  0x0002,   /* U+1B0B3 */
  0x0002,   /* U+1B0B4 */
  0x0002,   /* U+1B0B5 */
  0x0002,   /* U+1B0B6 */
  0x0002,   /* U+1B0B7 */
  0x0002,   /* U+1B0B8 */
  0x0002,   /* U+1B0B9 */
  0x0002,   /* U+1B0BA */
  0x0002,   /* U+1B0BB */
  0x0002,   /* U+1B0BC */
  0x0002,   /* U+1B0BD */
  0x0002,   /* U+1B0BE */
  0x0002,   /* U+1B0BF */
  0x0002,   /* U+1B0C0 */
  0x0002,   /* U+1B0C1 */
  0x0002,   /* U+1B0C2 */
  0x0002,   /* U+1B0C3 */
  0x0002,   /* U+1B0C4 */
  0x0002,   /* U+1B0C5 */
  0x0002,   /* U+1B0C6 */
  0x0002,   /* U+1B0C7 */
  0x0002,   /* U+1B0C8 */
  0x0002,   /* U+1B0C9 */
  0x0002,   /* U+1B0CA */
  0x0002,   /* U+1B0CB */
  0x0002,   /* U+1B0CC */
  0x0002,   /* U+1B0CD */
  0x0002,   /* U+1B0CE */
  0x0002,   /* U+1B0CF */
  0x0002,   /* U+1B0D0 */
  0x0002,   /* U+1B0D1 */
  0x0002,   /* U+1B0D2 */
  0x0002,   /* U+1B0D3 */
  0x0002,   /* U+1B0D4 */
  0x0002,   /* U+1B0D5 */
  0x0002,   /* U+1B0D6 */
  0x0002,   /* U+1B0D7 */
  0x0002,   /* U+1B0D8 */
  0x0002,   /* U+1B0D9 */
  0x0002,   /* U+1B0DA */
  0x0002,   /* U+1B0DB */
  0x0002,   /* U+1B0DC */
  0x0002,   /* U+1B0DD */
  0x0002,   /* U+1B0DE */
  0x0002,   /* U+1B0DF */
  0x0002,   /* U+1B0E0 */
  0x0002,   /* U+1B0E1 */
  0x0002,   /* U+1B0E2 */
  0x0002,   /* U+1B0E3 */
  0x0002,   /* U+1B0E4 */
  0x0002,   /* U+1B0E5 */
  0x0002,   /* U+1B0E6 */
  0x0002,   /* U+1B0E7 */
  0x0002,   /* U+1B0E8 */
  0x0002,   /* U+1B0E9 */
  0x0002,   /* U+1B0EA */
  0x0002,   /* U+1B0EB */
  0x0002,   /* U+1B0EC */
  0x0002,   /* U+1B0ED */
  0x0002,   /* U+1B0EE */
  0x0002,   /* U+1B0EF */
  0x0002,   /* U+1B0F0 */
  0x0002,   /* U+1B0F1 */
  0x0002,   /* U+1B0F2 */
  0x0002,   /* U+1B0F3 */
  0x0002,   /* U+1B0F4 */
  0x0002,   /* U+1B0F5 */
  0x0002,   /* U+1B0F6 */
  0x0002,   /* U+1B0F7 */
  0x0002,   /* U+1B0F8 */
  0x0002,   /* U+1B0F9 */
  0x0002,   /* U+1B0FA */
  0x0002,   /* U+1B0FB */
  0x0002,   /* U+1B0FC */
  0x0002,   /* U+1B0FD */
  0x0002,   /* U+1B0FE */
  0x0002,   /* U+1B0FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1B000 */
  0x0000,   /* U+1B001 */
  0xB002,   /* U+1B002 */
  0xB003,   /* U+1B003 */
  0xB004,   /* U+1B004 */
  0xB005,   /* U+1B005 */
  0xB006,   /* U+1B006 */
  0xB007,   /* U+1B007 */
  0xB008,   /* U+1B008 */
  0xB009,   /* U+1B009 */
  0xB00A,   /* U+1B00A */
  0xB00B,   /* U+1B00B */
  0xB00C,   /* U+1B00C */
  0xB00D,   /* U+1B00D */
  0xB00E,   /* U+1B00E */
  0xB00F,   /* U+1B00F */
  0xB010,   /* U+1B010 */
  0xB011,   /* U+1B011 */
  0xB012,   /* U+1B012 */
  0xB013,   /* U+1B013 */
  0xB014,   /* U+1B014 */
  0xB015,   /* U+1B015 */
  0xB016,   /* U+1B016 */
  0xB017,   /* U+1B017 */
  0xB018,   /* U+1B018 */
  0xB019,   /* U+1B019 */
  0xB01A,   /* U+1B01A */
  0xB01B,   /* U+1B01B */
  0xB01C,   /* U+1B01C */
  0xB01D,   /* U+1B01D */
  0xB01E,   /* U+1B01E */
  0xB01F,   /* U+1B01F */
  0xB020,   /* U+1B020 */
  0xB021,   /* U+1B021 */
  0xB022,   /* U+1B022 */
  0xB023,   /* U+1B023 */
  0xB024,   /* U+1B024 */
  0xB025,   /* U+1B025 */
  0xB026,   /* U+1B026 */
  0xB027,   /* U+1B027 */
  0xB028,   /* U+1B028 */
  0xB029,   /* U+1B029 */
  0xB02A,   /* U+1B02A */
  0xB02B,   /* U+1B02B */
  0xB02C,   /* U+1B02C */
  0xB02D,   /* U+1B02D */
  0xB02E,   /* U+1B02E */
  0xB02F,   /* U+1B02F */
  0xB030,   /* U+1B030 */
  0xB031,   /* U+1B031 */
  0xB032,   /* U+1B032 */
  0xB033,   /* U+1B033 */
  0xB034,   /* U+1B034 */
  0xB035,   /* U+1B035 */
  0xB036,   /* U+1B036 */
  0xB037,   /* U+1B037 */
  0xB038,   /* U+1B038 */
  0xB039,   /* U+1B039 */
  0xB03A,   /* U+1B03A */
  0xB03B,   /* U+1B03B */
  0xB03C,   /* U+1B03C */
  0xB03D,   /* U+1B03D */
  0xB03E,   /* U+1B03E */
  0xB03F,   /* U+1B03F */
  0xB040,   /* U+1B040 */
  0xB041,   /* U+1B041 */
  0xB042,   /* U+1B042 */
  0xB043,   /* U+1B043 */
  0xB044,   /* U+1B044 */
  0xB045,   /* U+1B045 */
  0xB046,   /* U+1B046 */
  0xB047,   /* U+1B047 */
  0xB048,   /* U+1B048 */
  0xB049,   /* U+1B049 */
  0xB04A,   /* U+1B04A */
  0xB04B,   /* U+1B04B */
  0xB04C,   /* U+1B04C */
  0xB04D,   /* U+1B04D */
  0xB04E,   /* U+1B04E */
  0xB04F,   /* U+1B04F */
  0xB050,   /* U+1B050 */
  0xB051,   /* U+1B051 */
  0xB052,   /* U+1B052 */
  0xB053,   /* U+1B053 */
  0xB054,   /* U+1B054 */
  0xB055,   /* U+1B055 */
  0xB056,   /* U+1B056 */
  0xB057,   /* U+1B057 */
  0xB058,   /* U+1B058 */
  0xB059,   /* U+1B059 */
  0xB05A,   /* U+1B05A */
  0xB05B,   /* U+1B05B */
  0xB05C,   /* U+1B05C */
  0xB05D,   /* U+1B05D */
  0xB05E,   /* U+1B05E */
  0xB05F,   /* U+1B05F */
  0xB060,   /* U+1B060 */
  0xB061,   /* U+1B061 */
  0xB062,   /* U+1B062 */
  0xB063,   /* U+1B063 */
  0xB064,   /* U+1B064 */
  0xB065,   /* U+1B065 */
  0xB066,   /* U+1B066 */
  0xB067,   /* U+1B067 */
  0xB068,   /* U+1B068 */
  0xB069,   /* U+1B069 */
  0xB06A,   /* U+1B06A */
  0xB06B,   /* U+1B06B */
  0xB06C,   /* U+1B06C */
  0xB06D,   /* U+1B06D */
  0xB06E,   /* U+1B06E */
  0xB06F,   /* U+1B06F */
  0xB070,   /* U+1B070 */
  0xB071,   /* U+1B071 */
  0xB072,   /* U+1B072 */
  0xB073,   /* U+1B073 */
  0xB074,   /* U+1B074 */
  0xB075,   /* U+1B075 */
  0xB076,   /* U+1B076 */
  0xB077,   /* U+1B077 */
  0xB078,   /* U+1B078 */
  0xB079,   /* U+1B079 */
  0xB07A,   /* U+1B07A */
  0xB07B,   /* U+1B07B */
  0xB07C,   /* U+1B07C */
  0xB07D,   /* U+1B07D */
  0xB07E,   /* U+1B07E */
  0xB07F,   /* U+1B07F */
  0xB080,   /* U+1B080 */
  0xB081,   /* U+1B081 */
  0xB082,   /* U+1B082 */
  0xB083,   /* U+1B083 */
  0xB084,   /* U+1B084 */
  0xB085,   /* U+1B085 */
  0xB086,   /* U+1B086 */
  0xB087,   /* U+1B087 */
  0xB088,   /* U+1B088 */
  0xB089,   /* U+1B089 */
  0xB08A,   /* U+1B08A */
  0xB08B,   /* U+1B08B */
  0xB08C,   /* U+1B08C */
  0xB08D,   /* U+1B08D */
  0xB08E,   /* U+1B08E */
  0xB08F,   /* U+1B08F */
  0xB090,   /* U+1B090 */
  0xB091,   /* U+1B091 */
  0xB092,   /* U+1B092 */
  0xB093,   /* U+1B093 */
  0xB094,   /* U+1B094 */
  0xB095,   /* U+1B095 */
  0xB096,   /* U+1B096 */
  0xB097,   /* U+1B097 */
  0xB098,   /* U+1B098 */
  0xB099,   /* U+1B099 */
  0xB09A,   /* U+1B09A */
  0xB09B,   /* U+1B09B */
  0xB09C,   /* U+1B09C */
  0xB09D,   /* U+1B09D */
  0xB09E,   /* U+1B09E */
  0xB09F,   /* U+1B09F */
  0xB0A0,   /* U+1B0A0 */
  0xB0A1,   /* U+1B0A1 */
  0xB0A2,   /* U+1B0A2 */
  0xB0A3,   /* U+1B0A3 */
  0xB0A4,   /* U+1B0A4 */
  0xB0A5,   /* U+1B0A5 */
  0xB0A6,   /* U+1B0A6 */
  0xB0A7,   /* U+1B0A7 */
  0xB0A8,   /* U+1B0A8 */
  0xB0A9,   /* U+1B0A9 */
  0xB0AA,   /* U+1B0AA */
  0xB0AB,   /* U+1B0AB */
  0xB0AC,   /* U+1B0AC */
  0xB0AD,   /* U+1B0AD */
  0xB0AE,   /* U+1B0AE */
  0xB0AF,   /* U+1B0AF */
  0xB0B0,   /* U+1B0B0 */
  0xB0B1,   /* U+1B0B1 */
  0xB0B2,   /* U+1B0B2 */
  0xB0B3,   /* U+1B0B3 */
  0xB0B4,   /* U+1B0B4 */
  0xB0B5,   /* U+1B0B5 */
  0xB0B6,   /* U+1B0B6 */
  0xB0B7,   /* U+1B0B7 */
  0xB0B8,   /* U+1B0B8 */
  0xB0B9,   /* U+1B0B9 */
  0xB0BA,   /* U+1B0BA */
  0xB0BB,   /* U+1B0BB */
  0xB0BC,   /* U+1B0BC */
  0xB0BD,   /* U+1B0BD */
  0xB0BE,   /* U+1B0BE */
  0xB0BF,   /* U+1B0BF */
  0xB0C0,   /* U+1B0C0 */
  0xB0C1,   /* U+1B0C1 */
  0xB0C2,   /* U+1B0C2 */
  0xB0C3,   /* U+1B0C3 */
  0xB0C4,   /* U+1B0C4 */
  0xB0C5,   /* U+1B0C5 */
  0xB0C6,   /* U+1B0C6 */
  0xB0C7,   /* U+1B0C7 */
  0xB0C8,   /* U+1B0C8 */
  0xB0C9,   /* U+1B0C9 */
  0xB0CA,   /* U+1B0CA */
  0xB0CB,   /* U+1B0CB */
  0xB0CC,   /* U+1B0CC */
  0xB0CD,   /* U+1B0CD */
  0xB0CE,   /* U+1B0CE */
  0xB0CF,   /* U+1B0CF */
  0xB0D0,   /* U+1B0D0 */
  0xB0D1,   /* U+1B0D1 */
  0xB0D2,   /* U+1B0D2 */
  0xB0D3,   /* U+1B0D3 */
  0xB0D4,   /* U+1B0D4 */
  0xB0D5,   /* U+1B0D5 */
  0xB0D6,   /* U+1B0D6 */
  0xB0D7,   /* U+1B0D7 */
  0xB0D8,   /* U+1B0D8 */
  0xB0D9,   /* U+1B0D9 */
  0xB0DA,   /* U+1B0DA */
  0xB0DB,   /* U+1B0DB */
  0xB0DC,   /* U+1B0DC */
  0xB0DD,   /* U+1B0DD */
  0xB0DE,   /* U+1B0DE */
  0xB0DF,   /* U+1B0DF */
  0xB0E0,   /* U+1B0E0 */
  0xB0E1,   /* U+1B0E1 */
  0xB0E2,   /* U+1B0E2 */
  0xB0E3,   /* U+1B0E3 */
  0xB0E4,   /* U+1B0E4 */
  0xB0E5,   /* U+1B0E5 */
  0xB0E6,   /* U+1B0E6 */
  0xB0E7,   /* U+1B0E7 */
  0xB0E8,   /* U+1B0E8 */
  0xB0E9,   /* U+1B0E9 */
  0xB0EA,   /* U+1B0EA */
  0xB0EB,   /* U+1B0EB */
  0xB0EC,   /* U+1B0EC */
  0xB0ED,   /* U+1B0ED */
  0xB0EE,   /* U+1B0EE */
  0xB0EF,   /* U+1B0EF */
  0xB0F0,   /* U+1B0F0 */
  0xB0F1,   /* U+1B0F1 */
  0xB0F2,   /* U+1B0F2 */
  0xB0F3,   /* U+1B0F3 */
  0xB0F4,   /* U+1B0F4 */
  0xB0F5,   /* U+1B0F5 */
  0xB0F6,   /* U+1B0F6 */
  0xB0F7,   /* U+1B0F7 */
  0xB0F8,   /* U+1B0F8 */
  0xB0F9,   /* U+1B0F9 */
  0xB0FA,   /* U+1B0FA */
  0xB0FB,   /* U+1B0FB */
  0xB0FC,   /* U+1B0FC */
  0xB0FD,   /* U+1B0FD */
  0xB0FE,   /* U+1B0FE */
  0xB0FF,   /* U+1B0FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1B000 */
  0x0000,   /* U+1B001 */
  0x0000,   /* U+1B002 */
  0x0000,   /* U+1B003 */
  0x0000,   /* U+1B004 */
  0x0000,   /* U+1B005 */
  0x0000,   /* U+1B006 */
  0x0000,   /* U+1B007 */
  0x0000,   /* U+1B008 */
  0x0000,   /* U+1B009 */
  0x0000,   /* U+1B00A */
  0x0000,   /* U+1B00B */
  0x0000,   /* U+1B00C */
  0x0000,   /* U+1B00D */
  0x0000,   /* U+1B00E */
  0x0000,   /* U+1B00F */
  0x0000,   /* U+1B010 */
  0x0000,   /* U+1B011 */
  0x0000,   /* U+1B012 */
  0x0000,   /* U+1B013 */
  0x0000,   /* U+1B014 */
  0x0000,   /* U+1B015 */
  0x0000,   /* U+1B016 */
  0x0000,   /* U+1B017 */
  0x0000,   /* U+1B018 */
  0x0000,   /* U+1B019 */
  0x0000,   /* U+1B01A */
  0x0000,   /* U+1B01B */
  0x0000,   /* U+1B01C */
  0x0000,   /* U+1B01D */
  0x0000,   /* U+1B01E */
  0x0000,   /* U+1B01F */
  0x0000,   /* U+1B020 */
  0x0000,   /* U+1B021 */
  0x0000,   /* U+1B022 */
  0x0000,   /* U+1B023 */
  0x0000,   /* U+1B024 */
  0x0000,   /* U+1B025 */
  0x0000,   /* U+1B026 */
  0x0000,   /* U+1B027 */
  0x0000,   /* U+1B028 */
  0x0000,   /* U+1B029 */
  0x0000,   /* U+1B02A */
  0x0000,   /* U+1B02B */
  0x0000,   /* U+1B02C */
  0x0000,   /* U+1B02D */
  0x0000,   /* U+1B02E */
  0x0000,   /* U+1B02F */
  0x0000,   /* U+1B030 */
  0x0000,   /* U+1B031 */
  0x0000,   /* U+1B032 */
  0x0000,   /* U+1B033 */
  0x0000,   /* U+1B034 */
  0x0000,   /* U+1B035 */
  0x0000,   /* U+1B036 */
  0x0000,   /* U+1B037 */
  0x0000,   /* U+1B038 */
  0x0000,   /* U+1B039 */
  0x0000,   /* U+1B03A */
  0x0000,   /* U+1B03B */
  0x0000,   /* U+1B03C */
  0x0000,   /* U+1B03D */
  0x0000,   /* U+1B03E */
  0x0000,   /* U+1B03F */
  0x0000,   /* U+1B040 */
  0x0000,   /* U+1B041 */
  0x0000,   /* U+1B042 */
  0x0000,   /* U+1B043 */
  0x0000,   /* U+1B044 */
  0x0000,   /* U+1B045 */
  0x0000,   /* U+1B046 */
  0x0000,   /* U+1B047 */
  0x0000,   /* U+1B048 */
  0x0000,   /* U+1B049 */
  0x0000,   /* U+1B04A */
  0x0000,   /* U+1B04B */
  0x0000,   /* U+1B04C */
  0x0000,   /* U+1B04D */
  0x0000,   /* U+1B04E */
  0x0000,   /* U+1B04F */
  0x0000,   /* U+1B050 */
  0x0000,   /* U+1B051 */
  0x0000,   /* U+1B052 */
  0x0000,   /* U+1B053 */
  0x0000,   /* U+1B054 */
  0x0000,   /* U+1B055 */
  0x0000,   /* U+1B056 */
  0x0000,   /* U+1B057 */
  0x0000,   /* U+1B058 */
  0x0000,   /* U+1B059 */
  0x0000,   /* U+1B05A */
  0x0000,   /* U+1B05B */
  0x0000,   /* U+1B05C */
  0x0000,   /* U+1B05D */
  0x0000,   /* U+1B05E */
  0x0000,   /* U+1B05F */
  0x0000,   /* U+1B060 */
  0x0000,   /* U+1B061 */
  0x0000,   /* U+1B062 */
  0x0000,   /* U+1B063 */
  0x0000,   /* U+1B064 */
  0x0000,   /* U+1B065 */
  0x0000,   /* U+1B066 */
  0x0000,   /* U+1B067 */
  0x0000,   /* U+1B068 */
  0x0000,   /* U+1B069 */
  0x0000,   /* U+1B06A */
  0x0000,   /* U+1B06B */
  0x0000,   /* U+1B06C */
  0x0000,   /* U+1B06D */
  0x0000,   /* U+1B06E */
  0x0000,   /* U+1B06F */
  0x0000,   /* U+1B070 */
  0x0000,   /* U+1B071 */
  0x0000,   /* U+1B072 */
  0x0000,   /* U+1B073 */
  0x0000,   /* U+1B074 */
  0x0000,   /* U+1B075 */
  0x0000,   /* U+1B076 */
  0x0000,   /* U+1B077 */
  0x0000,   /* U+1B078 */
  0x0000,   /* U+1B079 */
  0x0000,   /* U+1B07A */
  0x0000,   /* U+1B07B */
  0x0000,   /* U+1B07C */
  0x0000,   /* U+1B07D */
  0x0000,   /* U+1B07E */
  0x0000,   /* U+1B07F */
  0x0000,   /* U+1B080 */
  0x0000,   /* U+1B081 */
  0x0000,   /* U+1B082 */
  0x0000,   /* U+1B083 */
  0x0000,   /* U+1B084 */
  0x0000,   /* U+1B085 */
  0x0000,   /* U+1B086 */
  0x0000,   /* U+1B087 */
  0x0000,   /* U+1B088 */
  0x0000,   /* U+1B089 */
  0x0000,   /* U+1B08A */
  0x0000,   /* U+1B08B */
  0x0000,   /* U+1B08C */
  0x0000,   /* U+1B08D */
  0x0000,   /* U+1B08E */
  0x0000,   /* U+1B08F */
  0x0000,   /* U+1B090 */
  0x0000,   /* U+1B091 */
  0x0000,   /* U+1B092 */
  0x0000,   /* U+1B093 */
  0x0000,   /* U+1B094 */
  0x0000,   /* U+1B095 */
  0x0000,   /* U+1B096 */
  0x0000,   /* U+1B097 */
  0x0000,   /* U+1B098 */
  0x0000,   /* U+1B099 */
  0x0000,   /* U+1B09A */
  0x0000,   /* U+1B09B */
  0x0000,   /* U+1B09C */
  0x0000,   /* U+1B09D */
  0x0000,   /* U+1B09E */
  0x0000,   /* U+1B09F */
  0x0000,   /* U+1B0A0 */
  0x0000,   /* U+1B0A1 */
  0x0000,   /* U+1B0A2 */
  0x0000,   /* U+1B0A3 */
  0x0000,   /* U+1B0A4 */
  0x0000,   /* U+1B0A5 */
  0x0000,   /* U+1B0A6 */
  0x0000,   /* U+1B0A7 */
  0x0000,   /* U+1B0A8 */
  0x0000,   /* U+1B0A9 */
  0x0000,   /* U+1B0AA */
  0x0000,   /* U+1B0AB */
  0x0000,   /* U+1B0AC */
  0x0000,   /* U+1B0AD */
  0x0000,   /* U+1B0AE */
  0x0000,   /* U+1B0AF */
  0x0000,   /* U+1B0B0 */
  0x0000,   /* U+1B0B1 */
  0x0000,   /* U+1B0B2 */
  0x0000,   /* U+1B0B3 */
  0x0000,   /* U+1B0B4 */
  0x0000,   /* U+1B0B5 */
  0x0000,   /* U+1B0B6 */
  0x0000,   /* U+1B0B7 */
  0x0000,   /* U+1B0B8 */
  0x0000,   /* U+1B0B9 */
  0x0000,   /* U+1B0BA */
  0x0000,   /* U+1B0BB */
  0x0000,   /* U+1B0BC */
  0x0000,   /* U+1B0BD */
  0x0000,   /* U+1B0BE */
  0x0000,   /* U+1B0BF */
  0x0000,   /* U+1B0C0 */
  0x0000,   /* U+1B0C1 */
  0x0000,   /* U+1B0C2 */
  0x0000,   /* U+1B0C3 */
  0x0000,   /* U+1B0C4 */
  0x0000,   /* U+1B0C5 */
  0x0000,   /* U+1B0C6 */
  0x0000,   /* U+1B0C7 */
  0x0000,   /* U+1B0C8 */
  0x0000,   /* U+1B0C9 */
  0x0000,   /* U+1B0CA */
  0x0000,   /* U+1B0CB */
  0x0000,   /* U+1B0CC */
  0x0000,   /* U+1B0CD */
  0x0000,   /* U+1B0CE */
  0x0000,   /* U+1B0CF */
  0x0000,   /* U+1B0D0 */
  0x0000,   /* U+1B0D1 */
  0x0000,   /* U+1B0D2 */
  0x0000,   /* U+1B0D3 */
  0x0000,   /* U+1B0D4 */
  0x0000,   /* U+1B0D5 */
  0x0000,   /* U+1B0D6 */
  0x0000,   /* U+1B0D7 */
  0x0000,   /* U+1B0D8 */
  0x0000,   /* U+1B0D9 */
  0x0000,   /* U+1B0DA */
  0x0000,   /* U+1B0DB */
  0x0000,   /* U+1B0DC */
  0x0000,   /* U+1B0DD */
  0x0000,   /* U+1B0DE */
  0x0000,   /* U+1B0DF */
  0x0000,   /* U+1B0E0 */
  0x0000,   /* U+1B0E1 */
  0x0000,   /* U+1B0E2 */
  0x0000,   /* U+1B0E3 */
  0x0000,   /* U+1B0E4 */
  0x0000,   /* U+1B0E5 */
  0x0000,   /* U+1B0E6 */
  0x0000,   /* U+1B0E7 */
  0x0000,   /* U+1B0E8 */
  0x0000,   /* U+1B0E9 */
  0x0000,   /* U+1B0EA */
  0x0000,   /* U+1B0EB */
  0x0000,   /* U+1B0EC */
  0x0000,   /* U+1B0ED */
  0x0000,   /* U+1B0EE */
  0x0000,   /* U+1B0EF */
  0x0000,   /* U+1B0F0 */
  0x0000,   /* U+1B0F1 */
  0x0000,   /* U+1B0F2 */
  0x0000,   /* U+1B0F3 */
  0x0000,   /* U+1B0F4 */
  0x0000,   /* U+1B0F5 */
  0x0000,   /* U+1B0F6 */
  0x0000,   /* U+1B0F7 */
  0x0000,   /* U+1B0F8 */
  0x0000,   /* U+1B0F9 */
  0x0000,   /* U+1B0FA */
  0x0000,   /* U+1B0FB */
  0x0000,   /* U+1B0FC */
  0x0000,   /* U+1B0FD */
  0x0000,   /* U+1B0FE */
  0x0000,   /* U+1B0FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1B000 */
  0x0000,   /* U+1B001 */
  0x0000,   /* U+1B002 */
  0x0000,   /* U+1B003 */
  0x0000,   /* U+1B004 */
  0x0000,   /* U+1B005 */
  0x0000,   /* U+1B006 */
  0x0000,   /* U+1B007 */
  0x0000,   /* U+1B008 */
  0x0000,   /* U+1B009 */
  0x0000,   /* U+1B00A */
  0x0000,   /* U+1B00B */
  0x0000,   /* U+1B00C */
  0x0000,   /* U+1B00D */
  0x0000,   /* U+1B00E */
  0x0000,   /* U+1B00F */
  0x0000,   /* U+1B010 */
  0x0000,   /* U+1B011 */
  0x0000,   /* U+1B012 */
  0x0000,   /* U+1B013 */
  0x0000,   /* U+1B014 */
  0x0000,   /* U+1B015 */
  0x0000,   /* U+1B016 */
  0x0000,   /* U+1B017 */
  0x0000,   /* U+1B018 */
  0x0000,   /* U+1B019 */
  0x0000,   /* U+1B01A */
  0x0000,   /* U+1B01B */
  0x0000,   /* U+1B01C */
  0x0000,   /* U+1B01D */
  0x0000,   /* U+1B01E */
  0x0000,   /* U+1B01F */
  0x0000,   /* U+1B020 */
  0x0000,   /* U+1B021 */
  0x0000,   /* U+1B022 */
  0x0000,   /* U+1B023 */
  0x0000,   /* U+1B024 */
  0x0000,   /* U+1B025 */
  0x0000,   /* U+1B026 */
  0x0000,   /* U+1B027 */
  0x0000,   /* U+1B028 */
  0x0000,   /* U+1B029 */
  0x0000,   /* U+1B02A */
  0x0000,   /* U+1B02B */
  0x0000,   /* U+1B02C */
  0x0000,   /* U+1B02D */
  0x0000,   /* U+1B02E */
  0x0000,   /* U+1B02F */
  0x0000,   /* U+1B030 */
  0x0000,   /* U+1B031 */
  0x0000,   /* U+1B032 */
  0x0000,   /* U+1B033 */
  0x0000,   /* U+1B034 */
  0x0000,   /* U+1B035 */
  0x0000,   /* U+1B036 */
  0x0000,   /* U+1B037 */
  0x0000,   /* U+1B038 */
  0x0000,   /* U+1B039 */
  0x0000,   /* U+1B03A */
  0x0000,   /* U+1B03B */
  0x0000,   /* U+1B03C */
  0x0000,   /* U+1B03D */
  0x0000,   /* U+1B03E */
  0x0000,   /* U+1B03F */
  0x0000,   /* U+1B040 */
  0x0000,   /* U+1B041 */
  0x0000,   /* U+1B042 */
  0x0000,   /* U+1B043 */
  0x0000,   /* U+1B044 */
  0x0000,   /* U+1B045 */
  0x0000,   /* U+1B046 */
  0x0000,   /* U+1B047 */
  0x0000,   /* U+1B048 */
  0x0000,   /* U+1B049 */
  0x0000,   /* U+1B04A */
  0x0000,   /* U+1B04B */
  0x0000,   /* U+1B04C */
  0x0000,   /* U+1B04D */
  0x0000,   /* U+1B04E */
  0x0000,   /* U+1B04F */
  0x0000,   /* U+1B050 */
  0x0000,   /* U+1B051 */
  0x0000,   /* U+1B052 */
  0x0000,   /* U+1B053 */
  0x0000,   /* U+1B054 */
  0x0000,   /* U+1B055 */
  0x0000,   /* U+1B056 */
  0x0000,   /* U+1B057 */
  0x0000,   /* U+1B058 */
  0x0000,   /* U+1B059 */
  0x0000,   /* U+1B05A */
  0x0000,   /* U+1B05B */
  0x0000,   /* U+1B05C */
  0x0000,   /* U+1B05D */
  0x0000,   /* U+1B05E */
  0x0000,   /* U+1B05F */
  0x0000,   /* U+1B060 */
  0x0000,   /* U+1B061 */
  0x0000,   /* U+1B062 */
  0x0000,   /* U+1B063 */
  0x0000,   /* U+1B064 */
  0x0000,   /* U+1B065 */
  0x0000,   /* U+1B066 */
  0x0000,   /* U+1B067 */
  0x0000,   /* U+1B068 */
  0x0000,   /* U+1B069 */
  0x0000,   /* U+1B06A */
  0x0000,   /* U+1B06B */
  0x0000,   /* U+1B06C */
  0x0000,   /* U+1B06D */
  0x0000,   /* U+1B06E */
  0x0000,   /* U+1B06F */
  0x0000,   /* U+1B070 */
  0x0000,   /* U+1B071 */
  0x0000,   /* U+1B072 */
  0x0000,   /* U+1B073 */
  0x0000,   /* U+1B074 */
  0x0000,   /* U+1B075 */
  0x0000,   /* U+1B076 */
  0x0000,   /* U+1B077 */
  0x0000,   /* U+1B078 */
  0x0000,   /* U+1B079 */
  0x0000,   /* U+1B07A */
  0x0000,   /* U+1B07B */
  0x0000,   /* U+1B07C */
  0x0000,   /* U+1B07D */
  0x0000,   /* U+1B07E */
  0x0000,   /* U+1B07F */
  0x0000,   /* U+1B080 */
  0x0000,   /* U+1B081 */
  0x0000,   /* U+1B082 */
  0x0000,   /* U+1B083 */
  0x0000,   /* U+1B084 */
  0x0000,   /* U+1B085 */
  0x0000,   /* U+1B086 */
  0x0000,   /* U+1B087 */
  0x0000,   /* U+1B088 */
  0x0000,   /* U+1B089 */
  0x0000,   /* U+1B08A */
  0x0000,   /* U+1B08B */
  0x0000,   /* U+1B08C */
  0x0000,   /* U+1B08D */
  0x0000,   /* U+1B08E */
  0x0000,   /* U+1B08F */
  0x0000,   /* U+1B090 */
  0x0000,   /* U+1B091 */
  0x0000,   /* U+1B092 */
  0x0000,   /* U+1B093 */
  0x0000,   /* U+1B094 */
  0x0000,   /* U+1B095 */
  0x0000,   /* U+1B096 */
  0x0000,   /* U+1B097 */
  0x0000,   /* U+1B098 */
  0x0000,   /* U+1B099 */
  0x0000,   /* U+1B09A */
  0x0000,   /* U+1B09B */
  0x0000,   /* U+1B09C */
  0x0000,   /* U+1B09D */
  0x0000,   /* U+1B09E */
  0x0000,   /* U+1B09F */
  0x0000,   /* U+1B0A0 */
  0x0000,   /* U+1B0A1 */
  0x0000,   /* U+1B0A2 */
  0x0000,   /* U+1B0A3 */
  0x0000,   /* U+1B0A4 */
  0x0000,   /* U+1B0A5 */
  0x0000,   /* U+1B0A6 */
  0x0000,   /* U+1B0A7 */
  0x0000,   /* U+1B0A8 */
  0x0000,   /* U+1B0A9 */
  0x0000,   /* U+1B0AA */
  0x0000,   /* U+1B0AB */
  0x0000,   /* U+1B0AC */
  0x0000,   /* U+1B0AD */
  0x0000,   /* U+1B0AE */
  0x0000,   /* U+1B0AF */
  0x0000,   /* U+1B0B0 */
  0x0000,   /* U+1B0B1 */
  0x0000,   /* U+1B0B2 */
  0x0000,   /* U+1B0B3 */
  0x0000,   /* U+1B0B4 */
  0x0000,   /* U+1B0B5 */
  0x0000,   /* U+1B0B6 */
  0x0000,   /* U+1B0B7 */
  0x0000,   /* U+1B0B8 */
  0x0000,   /* U+1B0B9 */
  0x0000,   /* U+1B0BA */
  0x0000,   /* U+1B0BB */
  0x0000,   /* U+1B0BC */
  0x0000,   /* U+1B0BD */
  0x0000,   /* U+1B0BE */
  0x0000,   /* U+1B0BF */
  0x0000,   /* U+1B0C0 */
  0x0000,   /* U+1B0C1 */
  0x0000,   /* U+1B0C2 */
  0x0000,   /* U+1B0C3 */
  0x0000,   /* U+1B0C4 */
  0x0000,   /* U+1B0C5 */
  0x0000,   /* U+1B0C6 */
  0x0000,   /* U+1B0C7 */
  0x0000,   /* U+1B0C8 */
  0x0000,   /* U+1B0C9 */
  0x0000,   /* U+1B0CA */
  0x0000,   /* U+1B0CB */
  0x0000,   /* U+1B0CC */
  0x0000,   /* U+1B0CD */
  0x0000,   /* U+1B0CE */
  0x0000,   /* U+1B0CF */
  0x0000,   /* U+1B0D0 */
  0x0000,   /* U+1B0D1 */
  0x0000,   /* U+1B0D2 */
  0x0000,   /* U+1B0D3 */
  0x0000,   /* U+1B0D4 */
  0x0000,   /* U+1B0D5 */
  0x0000,   /* U+1B0D6 */
  0x0000,   /* U+1B0D7 */
  0x0000,   /* U+1B0D8 */
  0x0000,   /* U+1B0D9 */
  0x0000,   /* U+1B0DA */
  0x0000,   /* U+1B0DB */
  0x0000,   /* U+1B0DC */
  0x0000,   /* U+1B0DD */
  0x0000,   /* U+1B0DE */
  0x0000,   /* U+1B0DF */
  0x0000,   /* U+1B0E0 */
  0x0000,   /* U+1B0E1 */
  0x0000,   /* U+1B0E2 */
  0x0000,   /* U+1B0E3 */
  0x0000,   /* U+1B0E4 */
  0x0000,   /* U+1B0E5 */
  0x0000,   /* U+1B0E6 */
  0x0000,   /* U+1B0E7 */
  0x0000,   /* U+1B0E8 */
  0x0000,   /* U+1B0E9 */
  0x0000,   /* U+1B0EA */
  0x0000,   /* U+1B0EB */
  0x0000,   /* U+1B0EC */
  0x0000,   /* U+1B0ED */
  0x0000,   /* U+1B0EE */
  0x0000,   /* U+1B0EF */
  0x0000,   /* U+1B0F0 */
  0x0000,   /* U+1B0F1 */
  0x0000,   /* U+1B0F2 */
  0x0000,   /* U+1B0F3 */
  0x0000,   /* U+1B0F4 */
  0x0000,   /* U+1B0F5 */
  0x0000,   /* U+1B0F6 */
  0x0000,   /* U+1B0F7 */
  0x0000,   /* U+1B0F8 */
  0x0000,   /* U+1B0F9 */
  0x0000,   /* U+1B0FA */
  0x0000,   /* U+1B0FB */
  0x0000,   /* U+1B0FC */
  0x0000,   /* U+1B0FD */
  0x0000,   /* U+1B0FE */
  0x0000,   /* U+1B0FF */
};

uint16 uca900_p1BC[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x44AA,   /* U+1BC00 */
  0x44AB,   /* U+1BC01 */
  0x44AC,   /* U+1BC02 */
  0x44AD,   /* U+1BC03 */
  0x44AE,   /* U+1BC04 */
  0x44AF,   /* U+1BC05 */
  0x44B0,   /* U+1BC06 */
  0x44B1,   /* U+1BC07 */
  0x44B2,   /* U+1BC08 */
  0x44B3,   /* U+1BC09 */
  0x44B4,   /* U+1BC0A */
  0x44B5,   /* U+1BC0B */
  0x44B6,   /* U+1BC0C */
  0x44B7,   /* U+1BC0D */
  0x44B8,   /* U+1BC0E */
  0x44B9,   /* U+1BC0F */
  0x44BA,   /* U+1BC10 */
  0x44BB,   /* U+1BC11 */
  0x44BC,   /* U+1BC12 */
  0x44BD,   /* U+1BC13 */
  0x44BE,   /* U+1BC14 */
  0x44BF,   /* U+1BC15 */
  0x44C0,   /* U+1BC16 */
  0x44C1,   /* U+1BC17 */
  0x44C2,   /* U+1BC18 */
  0x44C3,   /* U+1BC19 */
  0x44C4,   /* U+1BC1A */
  0x44C5,   /* U+1BC1B */
  0x44C6,   /* U+1BC1C */
  0x44C7,   /* U+1BC1D */
  0x44C8,   /* U+1BC1E */
  0x44C9,   /* U+1BC1F */
  0x44CA,   /* U+1BC20 */
  0x44CB,   /* U+1BC21 */
  0x44CC,   /* U+1BC22 */
  0x44CD,   /* U+1BC23 */
  0x44CE,   /* U+1BC24 */
  0x44CF,   /* U+1BC25 */
  0x44D0,   /* U+1BC26 */
  0x44D1,   /* U+1BC27 */
  0x44D2,   /* U+1BC28 */
  0x44D3,   /* U+1BC29 */
  0x44D4,   /* U+1BC2A */
  0x44D5,   /* U+1BC2B */
  0x44D6,   /* U+1BC2C */
  0x44D7,   /* U+1BC2D */
  0x44D8,   /* U+1BC2E */
  0x44D9,   /* U+1BC2F */
  0x44DA,   /* U+1BC30 */
  0x44DB,   /* U+1BC31 */
  0x44DC,   /* U+1BC32 */
  0x44DD,   /* U+1BC33 */
  0x44DE,   /* U+1BC34 */
  0x44DF,   /* U+1BC35 */
  0x44E0,   /* U+1BC36 */
  0x44E1,   /* U+1BC37 */
  0x44E2,   /* U+1BC38 */
  0x44E3,   /* U+1BC39 */
  0x44E4,   /* U+1BC3A */
  0x44E5,   /* U+1BC3B */
  0x44E6,   /* U+1BC3C */
  0x44E7,   /* U+1BC3D */
  0x44E8,   /* U+1BC3E */
  0x44E9,   /* U+1BC3F */
  0x44EA,   /* U+1BC40 */
  0x44EB,   /* U+1BC41 */
  0x44EC,   /* U+1BC42 */
  0x44ED,   /* U+1BC43 */
  0x44EE,   /* U+1BC44 */
  0x44EF,   /* U+1BC45 */
  0x44F0,   /* U+1BC46 */
  0x44F1,   /* U+1BC47 */
  0x44F2,   /* U+1BC48 */
  0x44F3,   /* U+1BC49 */
  0x44F4,   /* U+1BC4A */
  0x44F5,   /* U+1BC4B */
  0x44F6,   /* U+1BC4C */
  0x44F7,   /* U+1BC4D */
  0x44F8,   /* U+1BC4E */
  0x44F9,   /* U+1BC4F */
  0x44FA,   /* U+1BC50 */
  0x44FB,   /* U+1BC51 */
  0x44FC,   /* U+1BC52 */
  0x44FD,   /* U+1BC53 */
  0x44FE,   /* U+1BC54 */
  0x44FF,   /* U+1BC55 */
  0x4500,   /* U+1BC56 */
  0x4501,   /* U+1BC57 */
  0x4502,   /* U+1BC58 */
  0x4503,   /* U+1BC59 */
  0x4504,   /* U+1BC5A */
  0x4505,   /* U+1BC5B */
  0x4506,   /* U+1BC5C */
  0x4507,   /* U+1BC5D */
  0x4508,   /* U+1BC5E */
  0x4509,   /* U+1BC5F */
  0x450A,   /* U+1BC60 */
  0x450B,   /* U+1BC61 */
  0x450C,   /* U+1BC62 */
  0x450D,   /* U+1BC63 */
  0x450E,   /* U+1BC64 */
  0x450F,   /* U+1BC65 */
  0x4510,   /* U+1BC66 */
  0x4511,   /* U+1BC67 */
  0x4512,   /* U+1BC68 */
  0x4513,   /* U+1BC69 */
  0x4514,   /* U+1BC6A */
  0xFBC3,   /* U+1BC6B */
  0xFBC3,   /* U+1BC6C */
  0xFBC3,   /* U+1BC6D */
  0xFBC3,   /* U+1BC6E */
  0xFBC3,   /* U+1BC6F */
  0x4515,   /* U+1BC70 */
  0x4516,   /* U+1BC71 */
  0x4517,   /* U+1BC72 */
  0x4518,   /* U+1BC73 */
  0x4519,   /* U+1BC74 */
  0x451A,   /* U+1BC75 */
  0x451B,   /* U+1BC76 */
  0x451C,   /* U+1BC77 */
  0x451D,   /* U+1BC78 */
  0x451E,   /* U+1BC79 */
  0x451F,   /* U+1BC7A */
  0x4520,   /* U+1BC7B */
  0x4521,   /* U+1BC7C */
  0xFBC3,   /* U+1BC7D */
  0xFBC3,   /* U+1BC7E */
  0xFBC3,   /* U+1BC7F */
  0x4522,   /* U+1BC80 */
  0x4523,   /* U+1BC81 */
  0x4524,   /* U+1BC82 */
  0x4525,   /* U+1BC83 */
  0x4526,   /* U+1BC84 */
  0x4527,   /* U+1BC85 */
  0x4528,   /* U+1BC86 */
  0x4529,   /* U+1BC87 */
  0x452A,   /* U+1BC88 */
  0xFBC3,   /* U+1BC89 */
  0xFBC3,   /* U+1BC8A */
  0xFBC3,   /* U+1BC8B */
  0xFBC3,   /* U+1BC8C */
  0xFBC3,   /* U+1BC8D */
  0xFBC3,   /* U+1BC8E */
  0xFBC3,   /* U+1BC8F */
  0x452B,   /* U+1BC90 */
  0x452C,   /* U+1BC91 */
  0x452D,   /* U+1BC92 */
  0x452E,   /* U+1BC93 */
  0x452F,   /* U+1BC94 */
  0x4530,   /* U+1BC95 */
  0x4531,   /* U+1BC96 */
  0x4532,   /* U+1BC97 */
  0x4533,   /* U+1BC98 */
  0x4534,   /* U+1BC99 */
  0xFBC3,   /* U+1BC9A */
  0xFBC3,   /* U+1BC9B */
  0x11C6,   /* U+1BC9C */
  0x0000,   /* U+1BC9D */
  0x0000,   /* U+1BC9E */
  0x0289,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0xFBC3,   /* U+1BCA4 */
  0xFBC3,   /* U+1BCA5 */
  0xFBC3,   /* U+1BCA6 */
  0xFBC3,   /* U+1BCA7 */
  0xFBC3,   /* U+1BCA8 */
  0xFBC3,   /* U+1BCA9 */
  0xFBC3,   /* U+1BCAA */
  0xFBC3,   /* U+1BCAB */
  0xFBC3,   /* U+1BCAC */
  0xFBC3,   /* U+1BCAD */
  0xFBC3,   /* U+1BCAE */
  0xFBC3,   /* U+1BCAF */
  0xFBC3,   /* U+1BCB0 */
  0xFBC3,   /* U+1BCB1 */
  0xFBC3,   /* U+1BCB2 */
  0xFBC3,   /* U+1BCB3 */
  0xFBC3,   /* U+1BCB4 */
  0xFBC3,   /* U+1BCB5 */
  0xFBC3,   /* U+1BCB6 */
  0xFBC3,   /* U+1BCB7 */
  0xFBC3,   /* U+1BCB8 */
  0xFBC3,   /* U+1BCB9 */
  0xFBC3,   /* U+1BCBA */
  0xFBC3,   /* U+1BCBB */
  0xFBC3,   /* U+1BCBC */
  0xFBC3,   /* U+1BCBD */
  0xFBC3,   /* U+1BCBE */
  0xFBC3,   /* U+1BCBF */
  0xFBC3,   /* U+1BCC0 */
  0xFBC3,   /* U+1BCC1 */
  0xFBC3,   /* U+1BCC2 */
  0xFBC3,   /* U+1BCC3 */
  0xFBC3,   /* U+1BCC4 */
  0xFBC3,   /* U+1BCC5 */
  0xFBC3,   /* U+1BCC6 */
  0xFBC3,   /* U+1BCC7 */
  0xFBC3,   /* U+1BCC8 */
  0xFBC3,   /* U+1BCC9 */
  0xFBC3,   /* U+1BCCA */
  0xFBC3,   /* U+1BCCB */
  0xFBC3,   /* U+1BCCC */
  0xFBC3,   /* U+1BCCD */
  0xFBC3,   /* U+1BCCE */
  0xFBC3,   /* U+1BCCF */
  0xFBC3,   /* U+1BCD0 */
  0xFBC3,   /* U+1BCD1 */
  0xFBC3,   /* U+1BCD2 */
  0xFBC3,   /* U+1BCD3 */
  0xFBC3,   /* U+1BCD4 */
  0xFBC3,   /* U+1BCD5 */
  0xFBC3,   /* U+1BCD6 */
  0xFBC3,   /* U+1BCD7 */
  0xFBC3,   /* U+1BCD8 */
  0xFBC3,   /* U+1BCD9 */
  0xFBC3,   /* U+1BCDA */
  0xFBC3,   /* U+1BCDB */
  0xFBC3,   /* U+1BCDC */
  0xFBC3,   /* U+1BCDD */
  0xFBC3,   /* U+1BCDE */
  0xFBC3,   /* U+1BCDF */
  0xFBC3,   /* U+1BCE0 */
  0xFBC3,   /* U+1BCE1 */
  0xFBC3,   /* U+1BCE2 */
  0xFBC3,   /* U+1BCE3 */
  0xFBC3,   /* U+1BCE4 */
  0xFBC3,   /* U+1BCE5 */
  0xFBC3,   /* U+1BCE6 */
  0xFBC3,   /* U+1BCE7 */
  0xFBC3,   /* U+1BCE8 */
  0xFBC3,   /* U+1BCE9 */
  0xFBC3,   /* U+1BCEA */
  0xFBC3,   /* U+1BCEB */
  0xFBC3,   /* U+1BCEC */
  0xFBC3,   /* U+1BCED */
  0xFBC3,   /* U+1BCEE */
  0xFBC3,   /* U+1BCEF */
  0xFBC3,   /* U+1BCF0 */
  0xFBC3,   /* U+1BCF1 */
  0xFBC3,   /* U+1BCF2 */
  0xFBC3,   /* U+1BCF3 */
  0xFBC3,   /* U+1BCF4 */
  0xFBC3,   /* U+1BCF5 */
  0xFBC3,   /* U+1BCF6 */
  0xFBC3,   /* U+1BCF7 */
  0xFBC3,   /* U+1BCF8 */
  0xFBC3,   /* U+1BCF9 */
  0xFBC3,   /* U+1BCFA */
  0xFBC3,   /* U+1BCFB */
  0xFBC3,   /* U+1BCFC */
  0xFBC3,   /* U+1BCFD */
  0xFBC3,   /* U+1BCFE */
  0xFBC3,   /* U+1BCFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1BC00 */
  0x0020,   /* U+1BC01 */
  0x0020,   /* U+1BC02 */
  0x0020,   /* U+1BC03 */
  0x0020,   /* U+1BC04 */
  0x0020,   /* U+1BC05 */
  0x0020,   /* U+1BC06 */
  0x0020,   /* U+1BC07 */
  0x0020,   /* U+1BC08 */
  0x0020,   /* U+1BC09 */
  0x0020,   /* U+1BC0A */
  0x0020,   /* U+1BC0B */
  0x0020,   /* U+1BC0C */
  0x0020,   /* U+1BC0D */
  0x0020,   /* U+1BC0E */
  0x0020,   /* U+1BC0F */
  0x0020,   /* U+1BC10 */
  0x0020,   /* U+1BC11 */
  0x0020,   /* U+1BC12 */
  0x0020,   /* U+1BC13 */
  0x0020,   /* U+1BC14 */
  0x0020,   /* U+1BC15 */
  0x0020,   /* U+1BC16 */
  0x0020,   /* U+1BC17 */
  0x0020,   /* U+1BC18 */
  0x0020,   /* U+1BC19 */
  0x0020,   /* U+1BC1A */
  0x0020,   /* U+1BC1B */
  0x0020,   /* U+1BC1C */
  0x0020,   /* U+1BC1D */
  0x0020,   /* U+1BC1E */
  0x0020,   /* U+1BC1F */
  0x0020,   /* U+1BC20 */
  0x0020,   /* U+1BC21 */
  0x0020,   /* U+1BC22 */
  0x0020,   /* U+1BC23 */
  0x0020,   /* U+1BC24 */
  0x0020,   /* U+1BC25 */
  0x0020,   /* U+1BC26 */
  0x0020,   /* U+1BC27 */
  0x0020,   /* U+1BC28 */
  0x0020,   /* U+1BC29 */
  0x0020,   /* U+1BC2A */
  0x0020,   /* U+1BC2B */
  0x0020,   /* U+1BC2C */
  0x0020,   /* U+1BC2D */
  0x0020,   /* U+1BC2E */
  0x0020,   /* U+1BC2F */
  0x0020,   /* U+1BC30 */
  0x0020,   /* U+1BC31 */
  0x0020,   /* U+1BC32 */
  0x0020,   /* U+1BC33 */
  0x0020,   /* U+1BC34 */
  0x0020,   /* U+1BC35 */
  0x0020,   /* U+1BC36 */
  0x0020,   /* U+1BC37 */
  0x0020,   /* U+1BC38 */
  0x0020,   /* U+1BC39 */
  0x0020,   /* U+1BC3A */
  0x0020,   /* U+1BC3B */
  0x0020,   /* U+1BC3C */
  0x0020,   /* U+1BC3D */
  0x0020,   /* U+1BC3E */
  0x0020,   /* U+1BC3F */
  0x0020,   /* U+1BC40 */
  0x0020,   /* U+1BC41 */
  0x0020,   /* U+1BC42 */
  0x0020,   /* U+1BC43 */
  0x0020,   /* U+1BC44 */
  0x0020,   /* U+1BC45 */
  0x0020,   /* U+1BC46 */
  0x0020,   /* U+1BC47 */
  0x0020,   /* U+1BC48 */
  0x0020,   /* U+1BC49 */
  0x0020,   /* U+1BC4A */
  0x0020,   /* U+1BC4B */
  0x0020,   /* U+1BC4C */
  0x0020,   /* U+1BC4D */
  0x0020,   /* U+1BC4E */
  0x0020,   /* U+1BC4F */
  0x0020,   /* U+1BC50 */
  0x0020,   /* U+1BC51 */
  0x0020,   /* U+1BC52 */
  0x0020,   /* U+1BC53 */
  0x0020,   /* U+1BC54 */
  0x0020,   /* U+1BC55 */
  0x0020,   /* U+1BC56 */
  0x0020,   /* U+1BC57 */
  0x0020,   /* U+1BC58 */
  0x0020,   /* U+1BC59 */
  0x0020,   /* U+1BC5A */
  0x0020,   /* U+1BC5B */
  0x0020,   /* U+1BC5C */
  0x0020,   /* U+1BC5D */
  0x0020,   /* U+1BC5E */
  0x0020,   /* U+1BC5F */
  0x0020,   /* U+1BC60 */
  0x0020,   /* U+1BC61 */
  0x0020,   /* U+1BC62 */
  0x0020,   /* U+1BC63 */
  0x0020,   /* U+1BC64 */
  0x0020,   /* U+1BC65 */
  0x0020,   /* U+1BC66 */
  0x0020,   /* U+1BC67 */
  0x0020,   /* U+1BC68 */
  0x0020,   /* U+1BC69 */
  0x0020,   /* U+1BC6A */
  0x0020,   /* U+1BC6B */
  0x0020,   /* U+1BC6C */
  0x0020,   /* U+1BC6D */
  0x0020,   /* U+1BC6E */
  0x0020,   /* U+1BC6F */
  0x0020,   /* U+1BC70 */
  0x0020,   /* U+1BC71 */
  0x0020,   /* U+1BC72 */
  0x0020,   /* U+1BC73 */
  0x0020,   /* U+1BC74 */
  0x0020,   /* U+1BC75 */
  0x0020,   /* U+1BC76 */
  0x0020,   /* U+1BC77 */
  0x0020,   /* U+1BC78 */
  0x0020,   /* U+1BC79 */
  0x0020,   /* U+1BC7A */
  0x0020,   /* U+1BC7B */
  0x0020,   /* U+1BC7C */
  0x0020,   /* U+1BC7D */
  0x0020,   /* U+1BC7E */
  0x0020,   /* U+1BC7F */
  0x0020,   /* U+1BC80 */
  0x0020,   /* U+1BC81 */
  0x0020,   /* U+1BC82 */
  0x0020,   /* U+1BC83 */
  0x0020,   /* U+1BC84 */
  0x0020,   /* U+1BC85 */
  0x0020,   /* U+1BC86 */
  0x0020,   /* U+1BC87 */
  0x0020,   /* U+1BC88 */
  0x0020,   /* U+1BC89 */
  0x0020,   /* U+1BC8A */
  0x0020,   /* U+1BC8B */
  0x0020,   /* U+1BC8C */
  0x0020,   /* U+1BC8D */
  0x0020,   /* U+1BC8E */
  0x0020,   /* U+1BC8F */
  0x0020,   /* U+1BC90 */
  0x0020,   /* U+1BC91 */
  0x0020,   /* U+1BC92 */
  0x0020,   /* U+1BC93 */
  0x0020,   /* U+1BC94 */
  0x0020,   /* U+1BC95 */
  0x0020,   /* U+1BC96 */
  0x0020,   /* U+1BC97 */
  0x0020,   /* U+1BC98 */
  0x0020,   /* U+1BC99 */
  0x0020,   /* U+1BC9A */
  0x0020,   /* U+1BC9B */
  0x0020,   /* U+1BC9C */
  0x0033,   /* U+1BC9D */
  0x0035,   /* U+1BC9E */
  0x0020,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0x0020,   /* U+1BCA4 */
  0x0020,   /* U+1BCA5 */
  0x0020,   /* U+1BCA6 */
  0x0020,   /* U+1BCA7 */
  0x0020,   /* U+1BCA8 */
  0x0020,   /* U+1BCA9 */
  0x0020,   /* U+1BCAA */
  0x0020,   /* U+1BCAB */
  0x0020,   /* U+1BCAC */
  0x0020,   /* U+1BCAD */
  0x0020,   /* U+1BCAE */
  0x0020,   /* U+1BCAF */
  0x0020,   /* U+1BCB0 */
  0x0020,   /* U+1BCB1 */
  0x0020,   /* U+1BCB2 */
  0x0020,   /* U+1BCB3 */
  0x0020,   /* U+1BCB4 */
  0x0020,   /* U+1BCB5 */
  0x0020,   /* U+1BCB6 */
  0x0020,   /* U+1BCB7 */
  0x0020,   /* U+1BCB8 */
  0x0020,   /* U+1BCB9 */
  0x0020,   /* U+1BCBA */
  0x0020,   /* U+1BCBB */
  0x0020,   /* U+1BCBC */
  0x0020,   /* U+1BCBD */
  0x0020,   /* U+1BCBE */
  0x0020,   /* U+1BCBF */
  0x0020,   /* U+1BCC0 */
  0x0020,   /* U+1BCC1 */
  0x0020,   /* U+1BCC2 */
  0x0020,   /* U+1BCC3 */
  0x0020,   /* U+1BCC4 */
  0x0020,   /* U+1BCC5 */
  0x0020,   /* U+1BCC6 */
  0x0020,   /* U+1BCC7 */
  0x0020,   /* U+1BCC8 */
  0x0020,   /* U+1BCC9 */
  0x0020,   /* U+1BCCA */
  0x0020,   /* U+1BCCB */
  0x0020,   /* U+1BCCC */
  0x0020,   /* U+1BCCD */
  0x0020,   /* U+1BCCE */
  0x0020,   /* U+1BCCF */
  0x0020,   /* U+1BCD0 */
  0x0020,   /* U+1BCD1 */
  0x0020,   /* U+1BCD2 */
  0x0020,   /* U+1BCD3 */
  0x0020,   /* U+1BCD4 */
  0x0020,   /* U+1BCD5 */
  0x0020,   /* U+1BCD6 */
  0x0020,   /* U+1BCD7 */
  0x0020,   /* U+1BCD8 */
  0x0020,   /* U+1BCD9 */
  0x0020,   /* U+1BCDA */
  0x0020,   /* U+1BCDB */
  0x0020,   /* U+1BCDC */
  0x0020,   /* U+1BCDD */
  0x0020,   /* U+1BCDE */
  0x0020,   /* U+1BCDF */
  0x0020,   /* U+1BCE0 */
  0x0020,   /* U+1BCE1 */
  0x0020,   /* U+1BCE2 */
  0x0020,   /* U+1BCE3 */
  0x0020,   /* U+1BCE4 */
  0x0020,   /* U+1BCE5 */
  0x0020,   /* U+1BCE6 */
  0x0020,   /* U+1BCE7 */
  0x0020,   /* U+1BCE8 */
  0x0020,   /* U+1BCE9 */
  0x0020,   /* U+1BCEA */
  0x0020,   /* U+1BCEB */
  0x0020,   /* U+1BCEC */
  0x0020,   /* U+1BCED */
  0x0020,   /* U+1BCEE */
  0x0020,   /* U+1BCEF */
  0x0020,   /* U+1BCF0 */
  0x0020,   /* U+1BCF1 */
  0x0020,   /* U+1BCF2 */
  0x0020,   /* U+1BCF3 */
  0x0020,   /* U+1BCF4 */
  0x0020,   /* U+1BCF5 */
  0x0020,   /* U+1BCF6 */
  0x0020,   /* U+1BCF7 */
  0x0020,   /* U+1BCF8 */
  0x0020,   /* U+1BCF9 */
  0x0020,   /* U+1BCFA */
  0x0020,   /* U+1BCFB */
  0x0020,   /* U+1BCFC */
  0x0020,   /* U+1BCFD */
  0x0020,   /* U+1BCFE */
  0x0020,   /* U+1BCFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1BC00 */
  0x0002,   /* U+1BC01 */
  0x0002,   /* U+1BC02 */
  0x0002,   /* U+1BC03 */
  0x0002,   /* U+1BC04 */
  0x0002,   /* U+1BC05 */
  0x0002,   /* U+1BC06 */
  0x0002,   /* U+1BC07 */
  0x0002,   /* U+1BC08 */
  0x0002,   /* U+1BC09 */
  0x0002,   /* U+1BC0A */
  0x0002,   /* U+1BC0B */
  0x0002,   /* U+1BC0C */
  0x0002,   /* U+1BC0D */
  0x0002,   /* U+1BC0E */
  0x0002,   /* U+1BC0F */
  0x0002,   /* U+1BC10 */
  0x0002,   /* U+1BC11 */
  0x0002,   /* U+1BC12 */
  0x0002,   /* U+1BC13 */
  0x0002,   /* U+1BC14 */
  0x0002,   /* U+1BC15 */
  0x0002,   /* U+1BC16 */
  0x0002,   /* U+1BC17 */
  0x0002,   /* U+1BC18 */
  0x0002,   /* U+1BC19 */
  0x0002,   /* U+1BC1A */
  0x0002,   /* U+1BC1B */
  0x0002,   /* U+1BC1C */
  0x0002,   /* U+1BC1D */
  0x0002,   /* U+1BC1E */
  0x0002,   /* U+1BC1F */
  0x0002,   /* U+1BC20 */
  0x0002,   /* U+1BC21 */
  0x0002,   /* U+1BC22 */
  0x0002,   /* U+1BC23 */
  0x0002,   /* U+1BC24 */
  0x0002,   /* U+1BC25 */
  0x0002,   /* U+1BC26 */
  0x0002,   /* U+1BC27 */
  0x0002,   /* U+1BC28 */
  0x0002,   /* U+1BC29 */
  0x0002,   /* U+1BC2A */
  0x0002,   /* U+1BC2B */
  0x0002,   /* U+1BC2C */
  0x0002,   /* U+1BC2D */
  0x0002,   /* U+1BC2E */
  0x0002,   /* U+1BC2F */
  0x0002,   /* U+1BC30 */
  0x0002,   /* U+1BC31 */
  0x0002,   /* U+1BC32 */
  0x0002,   /* U+1BC33 */
  0x0002,   /* U+1BC34 */
  0x0002,   /* U+1BC35 */
  0x0002,   /* U+1BC36 */
  0x0002,   /* U+1BC37 */
  0x0002,   /* U+1BC38 */
  0x0002,   /* U+1BC39 */
  0x0002,   /* U+1BC3A */
  0x0002,   /* U+1BC3B */
  0x0002,   /* U+1BC3C */
  0x0002,   /* U+1BC3D */
  0x0002,   /* U+1BC3E */
  0x0002,   /* U+1BC3F */
  0x0002,   /* U+1BC40 */
  0x0002,   /* U+1BC41 */
  0x0002,   /* U+1BC42 */
  0x0002,   /* U+1BC43 */
  0x0002,   /* U+1BC44 */
  0x0002,   /* U+1BC45 */
  0x0002,   /* U+1BC46 */
  0x0002,   /* U+1BC47 */
  0x0002,   /* U+1BC48 */
  0x0002,   /* U+1BC49 */
  0x0002,   /* U+1BC4A */
  0x0002,   /* U+1BC4B */
  0x0002,   /* U+1BC4C */
  0x0002,   /* U+1BC4D */
  0x0002,   /* U+1BC4E */
  0x0002,   /* U+1BC4F */
  0x0002,   /* U+1BC50 */
  0x0002,   /* U+1BC51 */
  0x0002,   /* U+1BC52 */
  0x0002,   /* U+1BC53 */
  0x0002,   /* U+1BC54 */
  0x0002,   /* U+1BC55 */
  0x0002,   /* U+1BC56 */
  0x0002,   /* U+1BC57 */
  0x0002,   /* U+1BC58 */
  0x0002,   /* U+1BC59 */
  0x0002,   /* U+1BC5A */
  0x0002,   /* U+1BC5B */
  0x0002,   /* U+1BC5C */
  0x0002,   /* U+1BC5D */
  0x0002,   /* U+1BC5E */
  0x0002,   /* U+1BC5F */
  0x0002,   /* U+1BC60 */
  0x0002,   /* U+1BC61 */
  0x0002,   /* U+1BC62 */
  0x0002,   /* U+1BC63 */
  0x0002,   /* U+1BC64 */
  0x0002,   /* U+1BC65 */
  0x0002,   /* U+1BC66 */
  0x0002,   /* U+1BC67 */
  0x0002,   /* U+1BC68 */
  0x0002,   /* U+1BC69 */
  0x0002,   /* U+1BC6A */
  0x0002,   /* U+1BC6B */
  0x0002,   /* U+1BC6C */
  0x0002,   /* U+1BC6D */
  0x0002,   /* U+1BC6E */
  0x0002,   /* U+1BC6F */
  0x0002,   /* U+1BC70 */
  0x0002,   /* U+1BC71 */
  0x0002,   /* U+1BC72 */
  0x0002,   /* U+1BC73 */
  0x0002,   /* U+1BC74 */
  0x0002,   /* U+1BC75 */
  0x0002,   /* U+1BC76 */
  0x0002,   /* U+1BC77 */
  0x0002,   /* U+1BC78 */
  0x0002,   /* U+1BC79 */
  0x0002,   /* U+1BC7A */
  0x0002,   /* U+1BC7B */
  0x0002,   /* U+1BC7C */
  0x0002,   /* U+1BC7D */
  0x0002,   /* U+1BC7E */
  0x0002,   /* U+1BC7F */
  0x0002,   /* U+1BC80 */
  0x0002,   /* U+1BC81 */
  0x0002,   /* U+1BC82 */
  0x0002,   /* U+1BC83 */
  0x0002,   /* U+1BC84 */
  0x0002,   /* U+1BC85 */
  0x0002,   /* U+1BC86 */
  0x0002,   /* U+1BC87 */
  0x0002,   /* U+1BC88 */
  0x0002,   /* U+1BC89 */
  0x0002,   /* U+1BC8A */
  0x0002,   /* U+1BC8B */
  0x0002,   /* U+1BC8C */
  0x0002,   /* U+1BC8D */
  0x0002,   /* U+1BC8E */
  0x0002,   /* U+1BC8F */
  0x0002,   /* U+1BC90 */
  0x0002,   /* U+1BC91 */
  0x0002,   /* U+1BC92 */
  0x0002,   /* U+1BC93 */
  0x0002,   /* U+1BC94 */
  0x0002,   /* U+1BC95 */
  0x0002,   /* U+1BC96 */
  0x0002,   /* U+1BC97 */
  0x0002,   /* U+1BC98 */
  0x0002,   /* U+1BC99 */
  0x0002,   /* U+1BC9A */
  0x0002,   /* U+1BC9B */
  0x0002,   /* U+1BC9C */
  0x0002,   /* U+1BC9D */
  0x0002,   /* U+1BC9E */
  0x0002,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0x0002,   /* U+1BCA4 */
  0x0002,   /* U+1BCA5 */
  0x0002,   /* U+1BCA6 */
  0x0002,   /* U+1BCA7 */
  0x0002,   /* U+1BCA8 */
  0x0002,   /* U+1BCA9 */
  0x0002,   /* U+1BCAA */
  0x0002,   /* U+1BCAB */
  0x0002,   /* U+1BCAC */
  0x0002,   /* U+1BCAD */
  0x0002,   /* U+1BCAE */
  0x0002,   /* U+1BCAF */
  0x0002,   /* U+1BCB0 */
  0x0002,   /* U+1BCB1 */
  0x0002,   /* U+1BCB2 */
  0x0002,   /* U+1BCB3 */
  0x0002,   /* U+1BCB4 */
  0x0002,   /* U+1BCB5 */
  0x0002,   /* U+1BCB6 */
  0x0002,   /* U+1BCB7 */
  0x0002,   /* U+1BCB8 */
  0x0002,   /* U+1BCB9 */
  0x0002,   /* U+1BCBA */
  0x0002,   /* U+1BCBB */
  0x0002,   /* U+1BCBC */
  0x0002,   /* U+1BCBD */
  0x0002,   /* U+1BCBE */
  0x0002,   /* U+1BCBF */
  0x0002,   /* U+1BCC0 */
  0x0002,   /* U+1BCC1 */
  0x0002,   /* U+1BCC2 */
  0x0002,   /* U+1BCC3 */
  0x0002,   /* U+1BCC4 */
  0x0002,   /* U+1BCC5 */
  0x0002,   /* U+1BCC6 */
  0x0002,   /* U+1BCC7 */
  0x0002,   /* U+1BCC8 */
  0x0002,   /* U+1BCC9 */
  0x0002,   /* U+1BCCA */
  0x0002,   /* U+1BCCB */
  0x0002,   /* U+1BCCC */
  0x0002,   /* U+1BCCD */
  0x0002,   /* U+1BCCE */
  0x0002,   /* U+1BCCF */
  0x0002,   /* U+1BCD0 */
  0x0002,   /* U+1BCD1 */
  0x0002,   /* U+1BCD2 */
  0x0002,   /* U+1BCD3 */
  0x0002,   /* U+1BCD4 */
  0x0002,   /* U+1BCD5 */
  0x0002,   /* U+1BCD6 */
  0x0002,   /* U+1BCD7 */
  0x0002,   /* U+1BCD8 */
  0x0002,   /* U+1BCD9 */
  0x0002,   /* U+1BCDA */
  0x0002,   /* U+1BCDB */
  0x0002,   /* U+1BCDC */
  0x0002,   /* U+1BCDD */
  0x0002,   /* U+1BCDE */
  0x0002,   /* U+1BCDF */
  0x0002,   /* U+1BCE0 */
  0x0002,   /* U+1BCE1 */
  0x0002,   /* U+1BCE2 */
  0x0002,   /* U+1BCE3 */
  0x0002,   /* U+1BCE4 */
  0x0002,   /* U+1BCE5 */
  0x0002,   /* U+1BCE6 */
  0x0002,   /* U+1BCE7 */
  0x0002,   /* U+1BCE8 */
  0x0002,   /* U+1BCE9 */
  0x0002,   /* U+1BCEA */
  0x0002,   /* U+1BCEB */
  0x0002,   /* U+1BCEC */
  0x0002,   /* U+1BCED */
  0x0002,   /* U+1BCEE */
  0x0002,   /* U+1BCEF */
  0x0002,   /* U+1BCF0 */
  0x0002,   /* U+1BCF1 */
  0x0002,   /* U+1BCF2 */
  0x0002,   /* U+1BCF3 */
  0x0002,   /* U+1BCF4 */
  0x0002,   /* U+1BCF5 */
  0x0002,   /* U+1BCF6 */
  0x0002,   /* U+1BCF7 */
  0x0002,   /* U+1BCF8 */
  0x0002,   /* U+1BCF9 */
  0x0002,   /* U+1BCFA */
  0x0002,   /* U+1BCFB */
  0x0002,   /* U+1BCFC */
  0x0002,   /* U+1BCFD */
  0x0002,   /* U+1BCFE */
  0x0002,   /* U+1BCFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1BC00 */
  0x0000,   /* U+1BC01 */
  0x0000,   /* U+1BC02 */
  0x0000,   /* U+1BC03 */
  0x0000,   /* U+1BC04 */
  0x0000,   /* U+1BC05 */
  0x0000,   /* U+1BC06 */
  0x0000,   /* U+1BC07 */
  0x0000,   /* U+1BC08 */
  0x0000,   /* U+1BC09 */
  0x0000,   /* U+1BC0A */
  0x0000,   /* U+1BC0B */
  0x0000,   /* U+1BC0C */
  0x0000,   /* U+1BC0D */
  0x0000,   /* U+1BC0E */
  0x0000,   /* U+1BC0F */
  0x0000,   /* U+1BC10 */
  0x0000,   /* U+1BC11 */
  0x0000,   /* U+1BC12 */
  0x0000,   /* U+1BC13 */
  0x0000,   /* U+1BC14 */
  0x0000,   /* U+1BC15 */
  0x0000,   /* U+1BC16 */
  0x0000,   /* U+1BC17 */
  0x0000,   /* U+1BC18 */
  0x0000,   /* U+1BC19 */
  0x0000,   /* U+1BC1A */
  0x0000,   /* U+1BC1B */
  0x0000,   /* U+1BC1C */
  0x0000,   /* U+1BC1D */
  0x0000,   /* U+1BC1E */
  0x0000,   /* U+1BC1F */
  0x0000,   /* U+1BC20 */
  0x0000,   /* U+1BC21 */
  0x0000,   /* U+1BC22 */
  0x0000,   /* U+1BC23 */
  0x0000,   /* U+1BC24 */
  0x0000,   /* U+1BC25 */
  0x0000,   /* U+1BC26 */
  0x0000,   /* U+1BC27 */
  0x0000,   /* U+1BC28 */
  0x0000,   /* U+1BC29 */
  0x0000,   /* U+1BC2A */
  0x0000,   /* U+1BC2B */
  0x0000,   /* U+1BC2C */
  0x0000,   /* U+1BC2D */
  0x0000,   /* U+1BC2E */
  0x0000,   /* U+1BC2F */
  0x0000,   /* U+1BC30 */
  0x0000,   /* U+1BC31 */
  0x0000,   /* U+1BC32 */
  0x0000,   /* U+1BC33 */
  0x0000,   /* U+1BC34 */
  0x0000,   /* U+1BC35 */
  0x0000,   /* U+1BC36 */
  0x0000,   /* U+1BC37 */
  0x0000,   /* U+1BC38 */
  0x0000,   /* U+1BC39 */
  0x0000,   /* U+1BC3A */
  0x0000,   /* U+1BC3B */
  0x0000,   /* U+1BC3C */
  0x0000,   /* U+1BC3D */
  0x0000,   /* U+1BC3E */
  0x0000,   /* U+1BC3F */
  0x0000,   /* U+1BC40 */
  0x0000,   /* U+1BC41 */
  0x0000,   /* U+1BC42 */
  0x0000,   /* U+1BC43 */
  0x0000,   /* U+1BC44 */
  0x0000,   /* U+1BC45 */
  0x0000,   /* U+1BC46 */
  0x0000,   /* U+1BC47 */
  0x0000,   /* U+1BC48 */
  0x0000,   /* U+1BC49 */
  0x0000,   /* U+1BC4A */
  0x0000,   /* U+1BC4B */
  0x0000,   /* U+1BC4C */
  0x0000,   /* U+1BC4D */
  0x0000,   /* U+1BC4E */
  0x0000,   /* U+1BC4F */
  0x0000,   /* U+1BC50 */
  0x0000,   /* U+1BC51 */
  0x0000,   /* U+1BC52 */
  0x0000,   /* U+1BC53 */
  0x0000,   /* U+1BC54 */
  0x0000,   /* U+1BC55 */
  0x0000,   /* U+1BC56 */
  0x0000,   /* U+1BC57 */
  0x0000,   /* U+1BC58 */
  0x0000,   /* U+1BC59 */
  0x0000,   /* U+1BC5A */
  0x0000,   /* U+1BC5B */
  0x0000,   /* U+1BC5C */
  0x0000,   /* U+1BC5D */
  0x0000,   /* U+1BC5E */
  0x0000,   /* U+1BC5F */
  0x0000,   /* U+1BC60 */
  0x0000,   /* U+1BC61 */
  0x0000,   /* U+1BC62 */
  0x0000,   /* U+1BC63 */
  0x0000,   /* U+1BC64 */
  0x0000,   /* U+1BC65 */
  0x0000,   /* U+1BC66 */
  0x0000,   /* U+1BC67 */
  0x0000,   /* U+1BC68 */
  0x0000,   /* U+1BC69 */
  0x0000,   /* U+1BC6A */
  0xBC6B,   /* U+1BC6B */
  0xBC6C,   /* U+1BC6C */
  0xBC6D,   /* U+1BC6D */
  0xBC6E,   /* U+1BC6E */
  0xBC6F,   /* U+1BC6F */
  0x0000,   /* U+1BC70 */
  0x0000,   /* U+1BC71 */
  0x0000,   /* U+1BC72 */
  0x0000,   /* U+1BC73 */
  0x0000,   /* U+1BC74 */
  0x0000,   /* U+1BC75 */
  0x0000,   /* U+1BC76 */
  0x0000,   /* U+1BC77 */
  0x0000,   /* U+1BC78 */
  0x0000,   /* U+1BC79 */
  0x0000,   /* U+1BC7A */
  0x0000,   /* U+1BC7B */
  0x0000,   /* U+1BC7C */
  0xBC7D,   /* U+1BC7D */
  0xBC7E,   /* U+1BC7E */
  0xBC7F,   /* U+1BC7F */
  0x0000,   /* U+1BC80 */
  0x0000,   /* U+1BC81 */
  0x0000,   /* U+1BC82 */
  0x0000,   /* U+1BC83 */
  0x0000,   /* U+1BC84 */
  0x0000,   /* U+1BC85 */
  0x0000,   /* U+1BC86 */
  0x0000,   /* U+1BC87 */
  0x0000,   /* U+1BC88 */
  0xBC89,   /* U+1BC89 */
  0xBC8A,   /* U+1BC8A */
  0xBC8B,   /* U+1BC8B */
  0xBC8C,   /* U+1BC8C */
  0xBC8D,   /* U+1BC8D */
  0xBC8E,   /* U+1BC8E */
  0xBC8F,   /* U+1BC8F */
  0x0000,   /* U+1BC90 */
  0x0000,   /* U+1BC91 */
  0x0000,   /* U+1BC92 */
  0x0000,   /* U+1BC93 */
  0x0000,   /* U+1BC94 */
  0x0000,   /* U+1BC95 */
  0x0000,   /* U+1BC96 */
  0x0000,   /* U+1BC97 */
  0x0000,   /* U+1BC98 */
  0x0000,   /* U+1BC99 */
  0xBC9A,   /* U+1BC9A */
  0xBC9B,   /* U+1BC9B */
  0x0000,   /* U+1BC9C */
  0x0000,   /* U+1BC9D */
  0x0000,   /* U+1BC9E */
  0x0000,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0xBCA4,   /* U+1BCA4 */
  0xBCA5,   /* U+1BCA5 */
  0xBCA6,   /* U+1BCA6 */
  0xBCA7,   /* U+1BCA7 */
  0xBCA8,   /* U+1BCA8 */
  0xBCA9,   /* U+1BCA9 */
  0xBCAA,   /* U+1BCAA */
  0xBCAB,   /* U+1BCAB */
  0xBCAC,   /* U+1BCAC */
  0xBCAD,   /* U+1BCAD */
  0xBCAE,   /* U+1BCAE */
  0xBCAF,   /* U+1BCAF */
  0xBCB0,   /* U+1BCB0 */
  0xBCB1,   /* U+1BCB1 */
  0xBCB2,   /* U+1BCB2 */
  0xBCB3,   /* U+1BCB3 */
  0xBCB4,   /* U+1BCB4 */
  0xBCB5,   /* U+1BCB5 */
  0xBCB6,   /* U+1BCB6 */
  0xBCB7,   /* U+1BCB7 */
  0xBCB8,   /* U+1BCB8 */
  0xBCB9,   /* U+1BCB9 */
  0xBCBA,   /* U+1BCBA */
  0xBCBB,   /* U+1BCBB */
  0xBCBC,   /* U+1BCBC */
  0xBCBD,   /* U+1BCBD */
  0xBCBE,   /* U+1BCBE */
  0xBCBF,   /* U+1BCBF */
  0xBCC0,   /* U+1BCC0 */
  0xBCC1,   /* U+1BCC1 */
  0xBCC2,   /* U+1BCC2 */
  0xBCC3,   /* U+1BCC3 */
  0xBCC4,   /* U+1BCC4 */
  0xBCC5,   /* U+1BCC5 */
  0xBCC6,   /* U+1BCC6 */
  0xBCC7,   /* U+1BCC7 */
  0xBCC8,   /* U+1BCC8 */
  0xBCC9,   /* U+1BCC9 */
  0xBCCA,   /* U+1BCCA */
  0xBCCB,   /* U+1BCCB */
  0xBCCC,   /* U+1BCCC */
  0xBCCD,   /* U+1BCCD */
  0xBCCE,   /* U+1BCCE */
  0xBCCF,   /* U+1BCCF */
  0xBCD0,   /* U+1BCD0 */
  0xBCD1,   /* U+1BCD1 */
  0xBCD2,   /* U+1BCD2 */
  0xBCD3,   /* U+1BCD3 */
  0xBCD4,   /* U+1BCD4 */
  0xBCD5,   /* U+1BCD5 */
  0xBCD6,   /* U+1BCD6 */
  0xBCD7,   /* U+1BCD7 */
  0xBCD8,   /* U+1BCD8 */
  0xBCD9,   /* U+1BCD9 */
  0xBCDA,   /* U+1BCDA */
  0xBCDB,   /* U+1BCDB */
  0xBCDC,   /* U+1BCDC */
  0xBCDD,   /* U+1BCDD */
  0xBCDE,   /* U+1BCDE */
  0xBCDF,   /* U+1BCDF */
  0xBCE0,   /* U+1BCE0 */
  0xBCE1,   /* U+1BCE1 */
  0xBCE2,   /* U+1BCE2 */
  0xBCE3,   /* U+1BCE3 */
  0xBCE4,   /* U+1BCE4 */
  0xBCE5,   /* U+1BCE5 */
  0xBCE6,   /* U+1BCE6 */
  0xBCE7,   /* U+1BCE7 */
  0xBCE8,   /* U+1BCE8 */
  0xBCE9,   /* U+1BCE9 */
  0xBCEA,   /* U+1BCEA */
  0xBCEB,   /* U+1BCEB */
  0xBCEC,   /* U+1BCEC */
  0xBCED,   /* U+1BCED */
  0xBCEE,   /* U+1BCEE */
  0xBCEF,   /* U+1BCEF */
  0xBCF0,   /* U+1BCF0 */
  0xBCF1,   /* U+1BCF1 */
  0xBCF2,   /* U+1BCF2 */
  0xBCF3,   /* U+1BCF3 */
  0xBCF4,   /* U+1BCF4 */
  0xBCF5,   /* U+1BCF5 */
  0xBCF6,   /* U+1BCF6 */
  0xBCF7,   /* U+1BCF7 */
  0xBCF8,   /* U+1BCF8 */
  0xBCF9,   /* U+1BCF9 */
  0xBCFA,   /* U+1BCFA */
  0xBCFB,   /* U+1BCFB */
  0xBCFC,   /* U+1BCFC */
  0xBCFD,   /* U+1BCFD */
  0xBCFE,   /* U+1BCFE */
  0xBCFF,   /* U+1BCFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1BC00 */
  0x0000,   /* U+1BC01 */
  0x0000,   /* U+1BC02 */
  0x0000,   /* U+1BC03 */
  0x0000,   /* U+1BC04 */
  0x0000,   /* U+1BC05 */
  0x0000,   /* U+1BC06 */
  0x0000,   /* U+1BC07 */
  0x0000,   /* U+1BC08 */
  0x0000,   /* U+1BC09 */
  0x0000,   /* U+1BC0A */
  0x0000,   /* U+1BC0B */
  0x0000,   /* U+1BC0C */
  0x0000,   /* U+1BC0D */
  0x0000,   /* U+1BC0E */
  0x0000,   /* U+1BC0F */
  0x0000,   /* U+1BC10 */
  0x0000,   /* U+1BC11 */
  0x0000,   /* U+1BC12 */
  0x0000,   /* U+1BC13 */
  0x0000,   /* U+1BC14 */
  0x0000,   /* U+1BC15 */
  0x0000,   /* U+1BC16 */
  0x0000,   /* U+1BC17 */
  0x0000,   /* U+1BC18 */
  0x0000,   /* U+1BC19 */
  0x0000,   /* U+1BC1A */
  0x0000,   /* U+1BC1B */
  0x0000,   /* U+1BC1C */
  0x0000,   /* U+1BC1D */
  0x0000,   /* U+1BC1E */
  0x0000,   /* U+1BC1F */
  0x0000,   /* U+1BC20 */
  0x0000,   /* U+1BC21 */
  0x0000,   /* U+1BC22 */
  0x0000,   /* U+1BC23 */
  0x0000,   /* U+1BC24 */
  0x0000,   /* U+1BC25 */
  0x0000,   /* U+1BC26 */
  0x0000,   /* U+1BC27 */
  0x0000,   /* U+1BC28 */
  0x0000,   /* U+1BC29 */
  0x0000,   /* U+1BC2A */
  0x0000,   /* U+1BC2B */
  0x0000,   /* U+1BC2C */
  0x0000,   /* U+1BC2D */
  0x0000,   /* U+1BC2E */
  0x0000,   /* U+1BC2F */
  0x0000,   /* U+1BC30 */
  0x0000,   /* U+1BC31 */
  0x0000,   /* U+1BC32 */
  0x0000,   /* U+1BC33 */
  0x0000,   /* U+1BC34 */
  0x0000,   /* U+1BC35 */
  0x0000,   /* U+1BC36 */
  0x0000,   /* U+1BC37 */
  0x0000,   /* U+1BC38 */
  0x0000,   /* U+1BC39 */
  0x0000,   /* U+1BC3A */
  0x0000,   /* U+1BC3B */
  0x0000,   /* U+1BC3C */
  0x0000,   /* U+1BC3D */
  0x0000,   /* U+1BC3E */
  0x0000,   /* U+1BC3F */
  0x0000,   /* U+1BC40 */
  0x0000,   /* U+1BC41 */
  0x0000,   /* U+1BC42 */
  0x0000,   /* U+1BC43 */
  0x0000,   /* U+1BC44 */
  0x0000,   /* U+1BC45 */
  0x0000,   /* U+1BC46 */
  0x0000,   /* U+1BC47 */
  0x0000,   /* U+1BC48 */
  0x0000,   /* U+1BC49 */
  0x0000,   /* U+1BC4A */
  0x0000,   /* U+1BC4B */
  0x0000,   /* U+1BC4C */
  0x0000,   /* U+1BC4D */
  0x0000,   /* U+1BC4E */
  0x0000,   /* U+1BC4F */
  0x0000,   /* U+1BC50 */
  0x0000,   /* U+1BC51 */
  0x0000,   /* U+1BC52 */
  0x0000,   /* U+1BC53 */
  0x0000,   /* U+1BC54 */
  0x0000,   /* U+1BC55 */
  0x0000,   /* U+1BC56 */
  0x0000,   /* U+1BC57 */
  0x0000,   /* U+1BC58 */
  0x0000,   /* U+1BC59 */
  0x0000,   /* U+1BC5A */
  0x0000,   /* U+1BC5B */
  0x0000,   /* U+1BC5C */
  0x0000,   /* U+1BC5D */
  0x0000,   /* U+1BC5E */
  0x0000,   /* U+1BC5F */
  0x0000,   /* U+1BC60 */
  0x0000,   /* U+1BC61 */
  0x0000,   /* U+1BC62 */
  0x0000,   /* U+1BC63 */
  0x0000,   /* U+1BC64 */
  0x0000,   /* U+1BC65 */
  0x0000,   /* U+1BC66 */
  0x0000,   /* U+1BC67 */
  0x0000,   /* U+1BC68 */
  0x0000,   /* U+1BC69 */
  0x0000,   /* U+1BC6A */
  0x0000,   /* U+1BC6B */
  0x0000,   /* U+1BC6C */
  0x0000,   /* U+1BC6D */
  0x0000,   /* U+1BC6E */
  0x0000,   /* U+1BC6F */
  0x0000,   /* U+1BC70 */
  0x0000,   /* U+1BC71 */
  0x0000,   /* U+1BC72 */
  0x0000,   /* U+1BC73 */
  0x0000,   /* U+1BC74 */
  0x0000,   /* U+1BC75 */
  0x0000,   /* U+1BC76 */
  0x0000,   /* U+1BC77 */
  0x0000,   /* U+1BC78 */
  0x0000,   /* U+1BC79 */
  0x0000,   /* U+1BC7A */
  0x0000,   /* U+1BC7B */
  0x0000,   /* U+1BC7C */
  0x0000,   /* U+1BC7D */
  0x0000,   /* U+1BC7E */
  0x0000,   /* U+1BC7F */
  0x0000,   /* U+1BC80 */
  0x0000,   /* U+1BC81 */
  0x0000,   /* U+1BC82 */
  0x0000,   /* U+1BC83 */
  0x0000,   /* U+1BC84 */
  0x0000,   /* U+1BC85 */
  0x0000,   /* U+1BC86 */
  0x0000,   /* U+1BC87 */
  0x0000,   /* U+1BC88 */
  0x0000,   /* U+1BC89 */
  0x0000,   /* U+1BC8A */
  0x0000,   /* U+1BC8B */
  0x0000,   /* U+1BC8C */
  0x0000,   /* U+1BC8D */
  0x0000,   /* U+1BC8E */
  0x0000,   /* U+1BC8F */
  0x0000,   /* U+1BC90 */
  0x0000,   /* U+1BC91 */
  0x0000,   /* U+1BC92 */
  0x0000,   /* U+1BC93 */
  0x0000,   /* U+1BC94 */
  0x0000,   /* U+1BC95 */
  0x0000,   /* U+1BC96 */
  0x0000,   /* U+1BC97 */
  0x0000,   /* U+1BC98 */
  0x0000,   /* U+1BC99 */
  0x0000,   /* U+1BC9A */
  0x0000,   /* U+1BC9B */
  0x0000,   /* U+1BC9C */
  0x0000,   /* U+1BC9D */
  0x0000,   /* U+1BC9E */
  0x0000,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0x0000,   /* U+1BCA4 */
  0x0000,   /* U+1BCA5 */
  0x0000,   /* U+1BCA6 */
  0x0000,   /* U+1BCA7 */
  0x0000,   /* U+1BCA8 */
  0x0000,   /* U+1BCA9 */
  0x0000,   /* U+1BCAA */
  0x0000,   /* U+1BCAB */
  0x0000,   /* U+1BCAC */
  0x0000,   /* U+1BCAD */
  0x0000,   /* U+1BCAE */
  0x0000,   /* U+1BCAF */
  0x0000,   /* U+1BCB0 */
  0x0000,   /* U+1BCB1 */
  0x0000,   /* U+1BCB2 */
  0x0000,   /* U+1BCB3 */
  0x0000,   /* U+1BCB4 */
  0x0000,   /* U+1BCB5 */
  0x0000,   /* U+1BCB6 */
  0x0000,   /* U+1BCB7 */
  0x0000,   /* U+1BCB8 */
  0x0000,   /* U+1BCB9 */
  0x0000,   /* U+1BCBA */
  0x0000,   /* U+1BCBB */
  0x0000,   /* U+1BCBC */
  0x0000,   /* U+1BCBD */
  0x0000,   /* U+1BCBE */
  0x0000,   /* U+1BCBF */
  0x0000,   /* U+1BCC0 */
  0x0000,   /* U+1BCC1 */
  0x0000,   /* U+1BCC2 */
  0x0000,   /* U+1BCC3 */
  0x0000,   /* U+1BCC4 */
  0x0000,   /* U+1BCC5 */
  0x0000,   /* U+1BCC6 */
  0x0000,   /* U+1BCC7 */
  0x0000,   /* U+1BCC8 */
  0x0000,   /* U+1BCC9 */
  0x0000,   /* U+1BCCA */
  0x0000,   /* U+1BCCB */
  0x0000,   /* U+1BCCC */
  0x0000,   /* U+1BCCD */
  0x0000,   /* U+1BCCE */
  0x0000,   /* U+1BCCF */
  0x0000,   /* U+1BCD0 */
  0x0000,   /* U+1BCD1 */
  0x0000,   /* U+1BCD2 */
  0x0000,   /* U+1BCD3 */
  0x0000,   /* U+1BCD4 */
  0x0000,   /* U+1BCD5 */
  0x0000,   /* U+1BCD6 */
  0x0000,   /* U+1BCD7 */
  0x0000,   /* U+1BCD8 */
  0x0000,   /* U+1BCD9 */
  0x0000,   /* U+1BCDA */
  0x0000,   /* U+1BCDB */
  0x0000,   /* U+1BCDC */
  0x0000,   /* U+1BCDD */
  0x0000,   /* U+1BCDE */
  0x0000,   /* U+1BCDF */
  0x0000,   /* U+1BCE0 */
  0x0000,   /* U+1BCE1 */
  0x0000,   /* U+1BCE2 */
  0x0000,   /* U+1BCE3 */
  0x0000,   /* U+1BCE4 */
  0x0000,   /* U+1BCE5 */
  0x0000,   /* U+1BCE6 */
  0x0000,   /* U+1BCE7 */
  0x0000,   /* U+1BCE8 */
  0x0000,   /* U+1BCE9 */
  0x0000,   /* U+1BCEA */
  0x0000,   /* U+1BCEB */
  0x0000,   /* U+1BCEC */
  0x0000,   /* U+1BCED */
  0x0000,   /* U+1BCEE */
  0x0000,   /* U+1BCEF */
  0x0000,   /* U+1BCF0 */
  0x0000,   /* U+1BCF1 */
  0x0000,   /* U+1BCF2 */
  0x0000,   /* U+1BCF3 */
  0x0000,   /* U+1BCF4 */
  0x0000,   /* U+1BCF5 */
  0x0000,   /* U+1BCF6 */
  0x0000,   /* U+1BCF7 */
  0x0000,   /* U+1BCF8 */
  0x0000,   /* U+1BCF9 */
  0x0000,   /* U+1BCFA */
  0x0000,   /* U+1BCFB */
  0x0000,   /* U+1BCFC */
  0x0000,   /* U+1BCFD */
  0x0000,   /* U+1BCFE */
  0x0000,   /* U+1BCFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1BC00 */
  0x0000,   /* U+1BC01 */
  0x0000,   /* U+1BC02 */
  0x0000,   /* U+1BC03 */
  0x0000,   /* U+1BC04 */
  0x0000,   /* U+1BC05 */
  0x0000,   /* U+1BC06 */
  0x0000,   /* U+1BC07 */
  0x0000,   /* U+1BC08 */
  0x0000,   /* U+1BC09 */
  0x0000,   /* U+1BC0A */
  0x0000,   /* U+1BC0B */
  0x0000,   /* U+1BC0C */
  0x0000,   /* U+1BC0D */
  0x0000,   /* U+1BC0E */
  0x0000,   /* U+1BC0F */
  0x0000,   /* U+1BC10 */
  0x0000,   /* U+1BC11 */
  0x0000,   /* U+1BC12 */
  0x0000,   /* U+1BC13 */
  0x0000,   /* U+1BC14 */
  0x0000,   /* U+1BC15 */
  0x0000,   /* U+1BC16 */
  0x0000,   /* U+1BC17 */
  0x0000,   /* U+1BC18 */
  0x0000,   /* U+1BC19 */
  0x0000,   /* U+1BC1A */
  0x0000,   /* U+1BC1B */
  0x0000,   /* U+1BC1C */
  0x0000,   /* U+1BC1D */
  0x0000,   /* U+1BC1E */
  0x0000,   /* U+1BC1F */
  0x0000,   /* U+1BC20 */
  0x0000,   /* U+1BC21 */
  0x0000,   /* U+1BC22 */
  0x0000,   /* U+1BC23 */
  0x0000,   /* U+1BC24 */
  0x0000,   /* U+1BC25 */
  0x0000,   /* U+1BC26 */
  0x0000,   /* U+1BC27 */
  0x0000,   /* U+1BC28 */
  0x0000,   /* U+1BC29 */
  0x0000,   /* U+1BC2A */
  0x0000,   /* U+1BC2B */
  0x0000,   /* U+1BC2C */
  0x0000,   /* U+1BC2D */
  0x0000,   /* U+1BC2E */
  0x0000,   /* U+1BC2F */
  0x0000,   /* U+1BC30 */
  0x0000,   /* U+1BC31 */
  0x0000,   /* U+1BC32 */
  0x0000,   /* U+1BC33 */
  0x0000,   /* U+1BC34 */
  0x0000,   /* U+1BC35 */
  0x0000,   /* U+1BC36 */
  0x0000,   /* U+1BC37 */
  0x0000,   /* U+1BC38 */
  0x0000,   /* U+1BC39 */
  0x0000,   /* U+1BC3A */
  0x0000,   /* U+1BC3B */
  0x0000,   /* U+1BC3C */
  0x0000,   /* U+1BC3D */
  0x0000,   /* U+1BC3E */
  0x0000,   /* U+1BC3F */
  0x0000,   /* U+1BC40 */
  0x0000,   /* U+1BC41 */
  0x0000,   /* U+1BC42 */
  0x0000,   /* U+1BC43 */
  0x0000,   /* U+1BC44 */
  0x0000,   /* U+1BC45 */
  0x0000,   /* U+1BC46 */
  0x0000,   /* U+1BC47 */
  0x0000,   /* U+1BC48 */
  0x0000,   /* U+1BC49 */
  0x0000,   /* U+1BC4A */
  0x0000,   /* U+1BC4B */
  0x0000,   /* U+1BC4C */
  0x0000,   /* U+1BC4D */
  0x0000,   /* U+1BC4E */
  0x0000,   /* U+1BC4F */
  0x0000,   /* U+1BC50 */
  0x0000,   /* U+1BC51 */
  0x0000,   /* U+1BC52 */
  0x0000,   /* U+1BC53 */
  0x0000,   /* U+1BC54 */
  0x0000,   /* U+1BC55 */
  0x0000,   /* U+1BC56 */
  0x0000,   /* U+1BC57 */
  0x0000,   /* U+1BC58 */
  0x0000,   /* U+1BC59 */
  0x0000,   /* U+1BC5A */
  0x0000,   /* U+1BC5B */
  0x0000,   /* U+1BC5C */
  0x0000,   /* U+1BC5D */
  0x0000,   /* U+1BC5E */
  0x0000,   /* U+1BC5F */
  0x0000,   /* U+1BC60 */
  0x0000,   /* U+1BC61 */
  0x0000,   /* U+1BC62 */
  0x0000,   /* U+1BC63 */
  0x0000,   /* U+1BC64 */
  0x0000,   /* U+1BC65 */
  0x0000,   /* U+1BC66 */
  0x0000,   /* U+1BC67 */
  0x0000,   /* U+1BC68 */
  0x0000,   /* U+1BC69 */
  0x0000,   /* U+1BC6A */
  0x0000,   /* U+1BC6B */
  0x0000,   /* U+1BC6C */
  0x0000,   /* U+1BC6D */
  0x0000,   /* U+1BC6E */
  0x0000,   /* U+1BC6F */
  0x0000,   /* U+1BC70 */
  0x0000,   /* U+1BC71 */
  0x0000,   /* U+1BC72 */
  0x0000,   /* U+1BC73 */
  0x0000,   /* U+1BC74 */
  0x0000,   /* U+1BC75 */
  0x0000,   /* U+1BC76 */
  0x0000,   /* U+1BC77 */
  0x0000,   /* U+1BC78 */
  0x0000,   /* U+1BC79 */
  0x0000,   /* U+1BC7A */
  0x0000,   /* U+1BC7B */
  0x0000,   /* U+1BC7C */
  0x0000,   /* U+1BC7D */
  0x0000,   /* U+1BC7E */
  0x0000,   /* U+1BC7F */
  0x0000,   /* U+1BC80 */
  0x0000,   /* U+1BC81 */
  0x0000,   /* U+1BC82 */
  0x0000,   /* U+1BC83 */
  0x0000,   /* U+1BC84 */
  0x0000,   /* U+1BC85 */
  0x0000,   /* U+1BC86 */
  0x0000,   /* U+1BC87 */
  0x0000,   /* U+1BC88 */
  0x0000,   /* U+1BC89 */
  0x0000,   /* U+1BC8A */
  0x0000,   /* U+1BC8B */
  0x0000,   /* U+1BC8C */
  0x0000,   /* U+1BC8D */
  0x0000,   /* U+1BC8E */
  0x0000,   /* U+1BC8F */
  0x0000,   /* U+1BC90 */
  0x0000,   /* U+1BC91 */
  0x0000,   /* U+1BC92 */
  0x0000,   /* U+1BC93 */
  0x0000,   /* U+1BC94 */
  0x0000,   /* U+1BC95 */
  0x0000,   /* U+1BC96 */
  0x0000,   /* U+1BC97 */
  0x0000,   /* U+1BC98 */
  0x0000,   /* U+1BC99 */
  0x0000,   /* U+1BC9A */
  0x0000,   /* U+1BC9B */
  0x0000,   /* U+1BC9C */
  0x0000,   /* U+1BC9D */
  0x0000,   /* U+1BC9E */
  0x0000,   /* U+1BC9F */
  0x0000,   /* U+1BCA0 */
  0x0000,   /* U+1BCA1 */
  0x0000,   /* U+1BCA2 */
  0x0000,   /* U+1BCA3 */
  0x0000,   /* U+1BCA4 */
  0x0000,   /* U+1BCA5 */
  0x0000,   /* U+1BCA6 */
  0x0000,   /* U+1BCA7 */
  0x0000,   /* U+1BCA8 */
  0x0000,   /* U+1BCA9 */
  0x0000,   /* U+1BCAA */
  0x0000,   /* U+1BCAB */
  0x0000,   /* U+1BCAC */
  0x0000,   /* U+1BCAD */
  0x0000,   /* U+1BCAE */
  0x0000,   /* U+1BCAF */
  0x0000,   /* U+1BCB0 */
  0x0000,   /* U+1BCB1 */
  0x0000,   /* U+1BCB2 */
  0x0000,   /* U+1BCB3 */
  0x0000,   /* U+1BCB4 */
  0x0000,   /* U+1BCB5 */
  0x0000,   /* U+1BCB6 */
  0x0000,   /* U+1BCB7 */
  0x0000,   /* U+1BCB8 */
  0x0000,   /* U+1BCB9 */
  0x0000,   /* U+1BCBA */
  0x0000,   /* U+1BCBB */
  0x0000,   /* U+1BCBC */
  0x0000,   /* U+1BCBD */
  0x0000,   /* U+1BCBE */
  0x0000,   /* U+1BCBF */
  0x0000,   /* U+1BCC0 */
  0x0000,   /* U+1BCC1 */
  0x0000,   /* U+1BCC2 */
  0x0000,   /* U+1BCC3 */
  0x0000,   /* U+1BCC4 */
  0x0000,   /* U+1BCC5 */
  0x0000,   /* U+1BCC6 */
  0x0000,   /* U+1BCC7 */
  0x0000,   /* U+1BCC8 */
  0x0000,   /* U+1BCC9 */
  0x0000,   /* U+1BCCA */
  0x0000,   /* U+1BCCB */
  0x0000,   /* U+1BCCC */
  0x0000,   /* U+1BCCD */
  0x0000,   /* U+1BCCE */
  0x0000,   /* U+1BCCF */
  0x0000,   /* U+1BCD0 */
  0x0000,   /* U+1BCD1 */
  0x0000,   /* U+1BCD2 */
  0x0000,   /* U+1BCD3 */
  0x0000,   /* U+1BCD4 */
  0x0000,   /* U+1BCD5 */
  0x0000,   /* U+1BCD6 */
  0x0000,   /* U+1BCD7 */
  0x0000,   /* U+1BCD8 */
  0x0000,   /* U+1BCD9 */
  0x0000,   /* U+1BCDA */
  0x0000,   /* U+1BCDB */
  0x0000,   /* U+1BCDC */
  0x0000,   /* U+1BCDD */
  0x0000,   /* U+1BCDE */
  0x0000,   /* U+1BCDF */
  0x0000,   /* U+1BCE0 */
  0x0000,   /* U+1BCE1 */
  0x0000,   /* U+1BCE2 */
  0x0000,   /* U+1BCE3 */
  0x0000,   /* U+1BCE4 */
  0x0000,   /* U+1BCE5 */
  0x0000,   /* U+1BCE6 */
  0x0000,   /* U+1BCE7 */
  0x0000,   /* U+1BCE8 */
  0x0000,   /* U+1BCE9 */
  0x0000,   /* U+1BCEA */
  0x0000,   /* U+1BCEB */
  0x0000,   /* U+1BCEC */
  0x0000,   /* U+1BCED */
  0x0000,   /* U+1BCEE */
  0x0000,   /* U+1BCEF */
  0x0000,   /* U+1BCF0 */
  0x0000,   /* U+1BCF1 */
  0x0000,   /* U+1BCF2 */
  0x0000,   /* U+1BCF3 */
  0x0000,   /* U+1BCF4 */
  0x0000,   /* U+1BCF5 */
  0x0000,   /* U+1BCF6 */
  0x0000,   /* U+1BCF7 */
  0x0000,   /* U+1BCF8 */
  0x0000,   /* U+1BCF9 */
  0x0000,   /* U+1BCFA */
  0x0000,   /* U+1BCFB */
  0x0000,   /* U+1BCFC */
  0x0000,   /* U+1BCFD */
  0x0000,   /* U+1BCFE */
  0x0000,   /* U+1BCFF */
};

uint16 uca900_p1D0[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0FD6,   /* U+1D000 */
  0x0FD7,   /* U+1D001 */
  0x0FD8,   /* U+1D002 */
  0x0FD9,   /* U+1D003 */
  0x0FDA,   /* U+1D004 */
  0x0FDB,   /* U+1D005 */
  0x0FDC,   /* U+1D006 */
  0x0FDD,   /* U+1D007 */
  0x0FDE,   /* U+1D008 */
  0x0FDF,   /* U+1D009 */
  0x0FE0,   /* U+1D00A */
  0x0FE1,   /* U+1D00B */
  0x0FE2,   /* U+1D00C */
  0x0FE3,   /* U+1D00D */
  0x0FE4,   /* U+1D00E */
  0x0FE5,   /* U+1D00F */
  0x0FE6,   /* U+1D010 */
  0x0FE7,   /* U+1D011 */
  0x0FE8,   /* U+1D012 */
  0x0FE9,   /* U+1D013 */
  0x0FEA,   /* U+1D014 */
  0x0FEB,   /* U+1D015 */
  0x0FEC,   /* U+1D016 */
  0x0FED,   /* U+1D017 */
  0x0FEE,   /* U+1D018 */
  0x0FEF,   /* U+1D019 */
  0x0FF0,   /* U+1D01A */
  0x0FF1,   /* U+1D01B */
  0x0FF2,   /* U+1D01C */
  0x0FF3,   /* U+1D01D */
  0x0FF4,   /* U+1D01E */
  0x0FF5,   /* U+1D01F */
  0x0FF6,   /* U+1D020 */
  0x0FF7,   /* U+1D021 */
  0x0FF8,   /* U+1D022 */
  0x0FF9,   /* U+1D023 */
  0x0FFA,   /* U+1D024 */
  0x0FFB,   /* U+1D025 */
  0x0FFC,   /* U+1D026 */
  0x0FFD,   /* U+1D027 */
  0x0FFE,   /* U+1D028 */
  0x0FFF,   /* U+1D029 */
  0x1000,   /* U+1D02A */
  0x1001,   /* U+1D02B */
  0x1002,   /* U+1D02C */
  0x1003,   /* U+1D02D */
  0x1004,   /* U+1D02E */
  0x1005,   /* U+1D02F */
  0x1006,   /* U+1D030 */
  0x1007,   /* U+1D031 */
  0x1008,   /* U+1D032 */
  0x1009,   /* U+1D033 */
  0x100A,   /* U+1D034 */
  0x100B,   /* U+1D035 */
  0x100C,   /* U+1D036 */
  0x100D,   /* U+1D037 */
  0x100E,   /* U+1D038 */
  0x100F,   /* U+1D039 */
  0x1010,   /* U+1D03A */
  0x1011,   /* U+1D03B */
  0x1012,   /* U+1D03C */
  0x1013,   /* U+1D03D */
  0x1014,   /* U+1D03E */
  0x1015,   /* U+1D03F */
  0x1016,   /* U+1D040 */
  0x1017,   /* U+1D041 */
  0x1018,   /* U+1D042 */
  0x1019,   /* U+1D043 */
  0x101A,   /* U+1D044 */
  0x101B,   /* U+1D045 */
  0x101C,   /* U+1D046 */
  0x101D,   /* U+1D047 */
  0x101E,   /* U+1D048 */
  0x101F,   /* U+1D049 */
  0x1020,   /* U+1D04A */
  0x1021,   /* U+1D04B */
  0x1022,   /* U+1D04C */
  0x1023,   /* U+1D04D */
  0x1024,   /* U+1D04E */
  0x1025,   /* U+1D04F */
  0x1026,   /* U+1D050 */
  0x1027,   /* U+1D051 */
  0x1028,   /* U+1D052 */
  0x1029,   /* U+1D053 */
  0x102A,   /* U+1D054 */
  0x102B,   /* U+1D055 */
  0x102C,   /* U+1D056 */
  0x102D,   /* U+1D057 */
  0x102E,   /* U+1D058 */
  0x102F,   /* U+1D059 */
  0x1030,   /* U+1D05A */
  0x1031,   /* U+1D05B */
  0x1032,   /* U+1D05C */
  0x1033,   /* U+1D05D */
  0x1034,   /* U+1D05E */
  0x1035,   /* U+1D05F */
  0x1036,   /* U+1D060 */
  0x1037,   /* U+1D061 */
  0x1038,   /* U+1D062 */
  0x1039,   /* U+1D063 */
  0x103A,   /* U+1D064 */
  0x103B,   /* U+1D065 */
  0x103C,   /* U+1D066 */
  0x103D,   /* U+1D067 */
  0x103E,   /* U+1D068 */
  0x103F,   /* U+1D069 */
  0x1040,   /* U+1D06A */
  0x1041,   /* U+1D06B */
  0x1042,   /* U+1D06C */
  0x1043,   /* U+1D06D */
  0x1044,   /* U+1D06E */
  0x1045,   /* U+1D06F */
  0x1046,   /* U+1D070 */
  0x1047,   /* U+1D071 */
  0x1048,   /* U+1D072 */
  0x1049,   /* U+1D073 */
  0x104A,   /* U+1D074 */
  0x104B,   /* U+1D075 */
  0x104C,   /* U+1D076 */
  0x104D,   /* U+1D077 */
  0x104E,   /* U+1D078 */
  0x104F,   /* U+1D079 */
  0x1050,   /* U+1D07A */
  0x1051,   /* U+1D07B */
  0x1052,   /* U+1D07C */
  0x1053,   /* U+1D07D */
  0x1054,   /* U+1D07E */
  0x1055,   /* U+1D07F */
  0x1056,   /* U+1D080 */
  0x1057,   /* U+1D081 */
  0x1058,   /* U+1D082 */
  0x1059,   /* U+1D083 */
  0x105A,   /* U+1D084 */
  0x105B,   /* U+1D085 */
  0x105C,   /* U+1D086 */
  0x105D,   /* U+1D087 */
  0x105E,   /* U+1D088 */
  0x105F,   /* U+1D089 */
  0x1060,   /* U+1D08A */
  0x1061,   /* U+1D08B */
  0x1062,   /* U+1D08C */
  0x1063,   /* U+1D08D */
  0x1064,   /* U+1D08E */
  0x1065,   /* U+1D08F */
  0x1066,   /* U+1D090 */
  0x1067,   /* U+1D091 */
  0x1068,   /* U+1D092 */
  0x1069,   /* U+1D093 */
  0x106A,   /* U+1D094 */
  0x106B,   /* U+1D095 */
  0x106C,   /* U+1D096 */
  0x106D,   /* U+1D097 */
  0x106E,   /* U+1D098 */
  0x106F,   /* U+1D099 */
  0x1070,   /* U+1D09A */
  0x1071,   /* U+1D09B */
  0x1072,   /* U+1D09C */
  0x1073,   /* U+1D09D */
  0x1074,   /* U+1D09E */
  0x1075,   /* U+1D09F */
  0x1076,   /* U+1D0A0 */
  0x1077,   /* U+1D0A1 */
  0x1078,   /* U+1D0A2 */
  0x1079,   /* U+1D0A3 */
  0x107A,   /* U+1D0A4 */
  0x107B,   /* U+1D0A5 */
  0x107C,   /* U+1D0A6 */
  0x107D,   /* U+1D0A7 */
  0x107E,   /* U+1D0A8 */
  0x107F,   /* U+1D0A9 */
  0x1080,   /* U+1D0AA */
  0x1081,   /* U+1D0AB */
  0x1082,   /* U+1D0AC */
  0x1083,   /* U+1D0AD */
  0x1084,   /* U+1D0AE */
  0x1085,   /* U+1D0AF */
  0x1086,   /* U+1D0B0 */
  0x1087,   /* U+1D0B1 */
  0x1088,   /* U+1D0B2 */
  0x1089,   /* U+1D0B3 */
  0x108A,   /* U+1D0B4 */
  0x108B,   /* U+1D0B5 */
  0x108C,   /* U+1D0B6 */
  0x108D,   /* U+1D0B7 */
  0x108E,   /* U+1D0B8 */
  0x108F,   /* U+1D0B9 */
  0x1090,   /* U+1D0BA */
  0x1091,   /* U+1D0BB */
  0x1092,   /* U+1D0BC */
  0x1093,   /* U+1D0BD */
  0x1094,   /* U+1D0BE */
  0x1095,   /* U+1D0BF */
  0x1096,   /* U+1D0C0 */
  0x1097,   /* U+1D0C1 */
  0x1098,   /* U+1D0C2 */
  0x1099,   /* U+1D0C3 */
  0x109A,   /* U+1D0C4 */
  0x109B,   /* U+1D0C5 */
  0x109C,   /* U+1D0C6 */
  0x109D,   /* U+1D0C7 */
  0x109E,   /* U+1D0C8 */
  0x109F,   /* U+1D0C9 */
  0x10A0,   /* U+1D0CA */
  0x10A1,   /* U+1D0CB */
  0x10A2,   /* U+1D0CC */
  0x10A3,   /* U+1D0CD */
  0x10A4,   /* U+1D0CE */
  0x10A5,   /* U+1D0CF */
  0x10A6,   /* U+1D0D0 */
  0x10A7,   /* U+1D0D1 */
  0x10A8,   /* U+1D0D2 */
  0x10A9,   /* U+1D0D3 */
  0x10AA,   /* U+1D0D4 */
  0x10AB,   /* U+1D0D5 */
  0x10AC,   /* U+1D0D6 */
  0x10AD,   /* U+1D0D7 */
  0x10AE,   /* U+1D0D8 */
  0x10AF,   /* U+1D0D9 */
  0x10B0,   /* U+1D0DA */
  0x10B1,   /* U+1D0DB */
  0x10B2,   /* U+1D0DC */
  0x10B3,   /* U+1D0DD */
  0x10B4,   /* U+1D0DE */
  0x10B5,   /* U+1D0DF */
  0x10B6,   /* U+1D0E0 */
  0x10B7,   /* U+1D0E1 */
  0x10B8,   /* U+1D0E2 */
  0x10B9,   /* U+1D0E3 */
  0x10BA,   /* U+1D0E4 */
  0x10BB,   /* U+1D0E5 */
  0x10BC,   /* U+1D0E6 */
  0x10BD,   /* U+1D0E7 */
  0x10BE,   /* U+1D0E8 */
  0x10BF,   /* U+1D0E9 */
  0x10C0,   /* U+1D0EA */
  0x10C1,   /* U+1D0EB */
  0x10C2,   /* U+1D0EC */
  0x10C3,   /* U+1D0ED */
  0x10C4,   /* U+1D0EE */
  0x10C5,   /* U+1D0EF */
  0x10C6,   /* U+1D0F0 */
  0x10C7,   /* U+1D0F1 */
  0x10C8,   /* U+1D0F2 */
  0x10C9,   /* U+1D0F3 */
  0x10CA,   /* U+1D0F4 */
  0x10CB,   /* U+1D0F5 */
  0xFBC3,   /* U+1D0F6 */
  0xFBC3,   /* U+1D0F7 */
  0xFBC3,   /* U+1D0F8 */
  0xFBC3,   /* U+1D0F9 */
  0xFBC3,   /* U+1D0FA */
  0xFBC3,   /* U+1D0FB */
  0xFBC3,   /* U+1D0FC */
  0xFBC3,   /* U+1D0FD */
  0xFBC3,   /* U+1D0FE */
  0xFBC3,   /* U+1D0FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D000 */
  0x0020,   /* U+1D001 */
  0x0020,   /* U+1D002 */
  0x0020,   /* U+1D003 */
  0x0020,   /* U+1D004 */
  0x0020,   /* U+1D005 */
  0x0020,   /* U+1D006 */
  0x0020,   /* U+1D007 */
  0x0020,   /* U+1D008 */
  0x0020,   /* U+1D009 */
  0x0020,   /* U+1D00A */
  0x0020,   /* U+1D00B */
  0x0020,   /* U+1D00C */
  0x0020,   /* U+1D00D */
  0x0020,   /* U+1D00E */
  0x0020,   /* U+1D00F */
  0x0020,   /* U+1D010 */
  0x0020,   /* U+1D011 */
  0x0020,   /* U+1D012 */
  0x0020,   /* U+1D013 */
  0x0020,   /* U+1D014 */
  0x0020,   /* U+1D015 */
  0x0020,   /* U+1D016 */
  0x0020,   /* U+1D017 */
  0x0020,   /* U+1D018 */
  0x0020,   /* U+1D019 */
  0x0020,   /* U+1D01A */
  0x0020,   /* U+1D01B */
  0x0020,   /* U+1D01C */
  0x0020,   /* U+1D01D */
  0x0020,   /* U+1D01E */
  0x0020,   /* U+1D01F */
  0x0020,   /* U+1D020 */
  0x0020,   /* U+1D021 */
  0x0020,   /* U+1D022 */
  0x0020,   /* U+1D023 */
  0x0020,   /* U+1D024 */
  0x0020,   /* U+1D025 */
  0x0020,   /* U+1D026 */
  0x0020,   /* U+1D027 */
  0x0020,   /* U+1D028 */
  0x0020,   /* U+1D029 */
  0x0020,   /* U+1D02A */
  0x0020,   /* U+1D02B */
  0x0020,   /* U+1D02C */
  0x0020,   /* U+1D02D */
  0x0020,   /* U+1D02E */
  0x0020,   /* U+1D02F */
  0x0020,   /* U+1D030 */
  0x0020,   /* U+1D031 */
  0x0020,   /* U+1D032 */
  0x0020,   /* U+1D033 */
  0x0020,   /* U+1D034 */
  0x0020,   /* U+1D035 */
  0x0020,   /* U+1D036 */
  0x0020,   /* U+1D037 */
  0x0020,   /* U+1D038 */
  0x0020,   /* U+1D039 */
  0x0020,   /* U+1D03A */
  0x0020,   /* U+1D03B */
  0x0020,   /* U+1D03C */
  0x0020,   /* U+1D03D */
  0x0020,   /* U+1D03E */
  0x0020,   /* U+1D03F */
  0x0020,   /* U+1D040 */
  0x0020,   /* U+1D041 */
  0x0020,   /* U+1D042 */
  0x0020,   /* U+1D043 */
  0x0020,   /* U+1D044 */
  0x0020,   /* U+1D045 */
  0x0020,   /* U+1D046 */
  0x0020,   /* U+1D047 */
  0x0020,   /* U+1D048 */
  0x0020,   /* U+1D049 */
  0x0020,   /* U+1D04A */
  0x0020,   /* U+1D04B */
  0x0020,   /* U+1D04C */
  0x0020,   /* U+1D04D */
  0x0020,   /* U+1D04E */
  0x0020,   /* U+1D04F */
  0x0020,   /* U+1D050 */
  0x0020,   /* U+1D051 */
  0x0020,   /* U+1D052 */
  0x0020,   /* U+1D053 */
  0x0020,   /* U+1D054 */
  0x0020,   /* U+1D055 */
  0x0020,   /* U+1D056 */
  0x0020,   /* U+1D057 */
  0x0020,   /* U+1D058 */
  0x0020,   /* U+1D059 */
  0x0020,   /* U+1D05A */
  0x0020,   /* U+1D05B */
  0x0020,   /* U+1D05C */
  0x0020,   /* U+1D05D */
  0x0020,   /* U+1D05E */
  0x0020,   /* U+1D05F */
  0x0020,   /* U+1D060 */
  0x0020,   /* U+1D061 */
  0x0020,   /* U+1D062 */
  0x0020,   /* U+1D063 */
  0x0020,   /* U+1D064 */
  0x0020,   /* U+1D065 */
  0x0020,   /* U+1D066 */
  0x0020,   /* U+1D067 */
  0x0020,   /* U+1D068 */
  0x0020,   /* U+1D069 */
  0x0020,   /* U+1D06A */
  0x0020,   /* U+1D06B */
  0x0020,   /* U+1D06C */
  0x0020,   /* U+1D06D */
  0x0020,   /* U+1D06E */
  0x0020,   /* U+1D06F */
  0x0020,   /* U+1D070 */
  0x0020,   /* U+1D071 */
  0x0020,   /* U+1D072 */
  0x0020,   /* U+1D073 */
  0x0020,   /* U+1D074 */
  0x0020,   /* U+1D075 */
  0x0020,   /* U+1D076 */
  0x0020,   /* U+1D077 */
  0x0020,   /* U+1D078 */
  0x0020,   /* U+1D079 */
  0x0020,   /* U+1D07A */
  0x0020,   /* U+1D07B */
  0x0020,   /* U+1D07C */
  0x0020,   /* U+1D07D */
  0x0020,   /* U+1D07E */
  0x0020,   /* U+1D07F */
  0x0020,   /* U+1D080 */
  0x0020,   /* U+1D081 */
  0x0020,   /* U+1D082 */
  0x0020,   /* U+1D083 */
  0x0020,   /* U+1D084 */
  0x0020,   /* U+1D085 */
  0x0020,   /* U+1D086 */
  0x0020,   /* U+1D087 */
  0x0020,   /* U+1D088 */
  0x0020,   /* U+1D089 */
  0x0020,   /* U+1D08A */
  0x0020,   /* U+1D08B */
  0x0020,   /* U+1D08C */
  0x0020,   /* U+1D08D */
  0x0020,   /* U+1D08E */
  0x0020,   /* U+1D08F */
  0x0020,   /* U+1D090 */
  0x0020,   /* U+1D091 */
  0x0020,   /* U+1D092 */
  0x0020,   /* U+1D093 */
  0x0020,   /* U+1D094 */
  0x0020,   /* U+1D095 */
  0x0020,   /* U+1D096 */
  0x0020,   /* U+1D097 */
  0x0020,   /* U+1D098 */
  0x0020,   /* U+1D099 */
  0x0020,   /* U+1D09A */
  0x0020,   /* U+1D09B */
  0x0020,   /* U+1D09C */
  0x0020,   /* U+1D09D */
  0x0020,   /* U+1D09E */
  0x0020,   /* U+1D09F */
  0x0020,   /* U+1D0A0 */
  0x0020,   /* U+1D0A1 */
  0x0020,   /* U+1D0A2 */
  0x0020,   /* U+1D0A3 */
  0x0020,   /* U+1D0A4 */
  0x0020,   /* U+1D0A5 */
  0x0020,   /* U+1D0A6 */
  0x0020,   /* U+1D0A7 */
  0x0020,   /* U+1D0A8 */
  0x0020,   /* U+1D0A9 */
  0x0020,   /* U+1D0AA */
  0x0020,   /* U+1D0AB */
  0x0020,   /* U+1D0AC */
  0x0020,   /* U+1D0AD */
  0x0020,   /* U+1D0AE */
  0x0020,   /* U+1D0AF */
  0x0020,   /* U+1D0B0 */
  0x0020,   /* U+1D0B1 */
  0x0020,   /* U+1D0B2 */
  0x0020,   /* U+1D0B3 */
  0x0020,   /* U+1D0B4 */
  0x0020,   /* U+1D0B5 */
  0x0020,   /* U+1D0B6 */
  0x0020,   /* U+1D0B7 */
  0x0020,   /* U+1D0B8 */
  0x0020,   /* U+1D0B9 */
  0x0020,   /* U+1D0BA */
  0x0020,   /* U+1D0BB */
  0x0020,   /* U+1D0BC */
  0x0020,   /* U+1D0BD */
  0x0020,   /* U+1D0BE */
  0x0020,   /* U+1D0BF */
  0x0020,   /* U+1D0C0 */
  0x0020,   /* U+1D0C1 */
  0x0020,   /* U+1D0C2 */
  0x0020,   /* U+1D0C3 */
  0x0020,   /* U+1D0C4 */
  0x0020,   /* U+1D0C5 */
  0x0020,   /* U+1D0C6 */
  0x0020,   /* U+1D0C7 */
  0x0020,   /* U+1D0C8 */
  0x0020,   /* U+1D0C9 */
  0x0020,   /* U+1D0CA */
  0x0020,   /* U+1D0CB */
  0x0020,   /* U+1D0CC */
  0x0020,   /* U+1D0CD */
  0x0020,   /* U+1D0CE */
  0x0020,   /* U+1D0CF */
  0x0020,   /* U+1D0D0 */
  0x0020,   /* U+1D0D1 */
  0x0020,   /* U+1D0D2 */
  0x0020,   /* U+1D0D3 */
  0x0020,   /* U+1D0D4 */
  0x0020,   /* U+1D0D5 */
  0x0020,   /* U+1D0D6 */
  0x0020,   /* U+1D0D7 */
  0x0020,   /* U+1D0D8 */
  0x0020,   /* U+1D0D9 */
  0x0020,   /* U+1D0DA */
  0x0020,   /* U+1D0DB */
  0x0020,   /* U+1D0DC */
  0x0020,   /* U+1D0DD */
  0x0020,   /* U+1D0DE */
  0x0020,   /* U+1D0DF */
  0x0020,   /* U+1D0E0 */
  0x0020,   /* U+1D0E1 */
  0x0020,   /* U+1D0E2 */
  0x0020,   /* U+1D0E3 */
  0x0020,   /* U+1D0E4 */
  0x0020,   /* U+1D0E5 */
  0x0020,   /* U+1D0E6 */
  0x0020,   /* U+1D0E7 */
  0x0020,   /* U+1D0E8 */
  0x0020,   /* U+1D0E9 */
  0x0020,   /* U+1D0EA */
  0x0020,   /* U+1D0EB */
  0x0020,   /* U+1D0EC */
  0x0020,   /* U+1D0ED */
  0x0020,   /* U+1D0EE */
  0x0020,   /* U+1D0EF */
  0x0020,   /* U+1D0F0 */
  0x0020,   /* U+1D0F1 */
  0x0020,   /* U+1D0F2 */
  0x0020,   /* U+1D0F3 */
  0x0020,   /* U+1D0F4 */
  0x0020,   /* U+1D0F5 */
  0x0020,   /* U+1D0F6 */
  0x0020,   /* U+1D0F7 */
  0x0020,   /* U+1D0F8 */
  0x0020,   /* U+1D0F9 */
  0x0020,   /* U+1D0FA */
  0x0020,   /* U+1D0FB */
  0x0020,   /* U+1D0FC */
  0x0020,   /* U+1D0FD */
  0x0020,   /* U+1D0FE */
  0x0020,   /* U+1D0FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D000 */
  0x0002,   /* U+1D001 */
  0x0002,   /* U+1D002 */
  0x0002,   /* U+1D003 */
  0x0002,   /* U+1D004 */
  0x0002,   /* U+1D005 */
  0x0002,   /* U+1D006 */
  0x0002,   /* U+1D007 */
  0x0002,   /* U+1D008 */
  0x0002,   /* U+1D009 */
  0x0002,   /* U+1D00A */
  0x0002,   /* U+1D00B */
  0x0002,   /* U+1D00C */
  0x0002,   /* U+1D00D */
  0x0002,   /* U+1D00E */
  0x0002,   /* U+1D00F */
  0x0002,   /* U+1D010 */
  0x0002,   /* U+1D011 */
  0x0002,   /* U+1D012 */
  0x0002,   /* U+1D013 */
  0x0002,   /* U+1D014 */
  0x0002,   /* U+1D015 */
  0x0002,   /* U+1D016 */
  0x0002,   /* U+1D017 */
  0x0002,   /* U+1D018 */
  0x0002,   /* U+1D019 */
  0x0002,   /* U+1D01A */
  0x0002,   /* U+1D01B */
  0x0002,   /* U+1D01C */
  0x0002,   /* U+1D01D */
  0x0002,   /* U+1D01E */
  0x0002,   /* U+1D01F */
  0x0002,   /* U+1D020 */
  0x0002,   /* U+1D021 */
  0x0002,   /* U+1D022 */
  0x0002,   /* U+1D023 */
  0x0002,   /* U+1D024 */
  0x0002,   /* U+1D025 */
  0x0002,   /* U+1D026 */
  0x0002,   /* U+1D027 */
  0x0002,   /* U+1D028 */
  0x0002,   /* U+1D029 */
  0x0002,   /* U+1D02A */
  0x0002,   /* U+1D02B */
  0x0002,   /* U+1D02C */
  0x0002,   /* U+1D02D */
  0x0002,   /* U+1D02E */
  0x0002,   /* U+1D02F */
  0x0002,   /* U+1D030 */
  0x0002,   /* U+1D031 */
  0x0002,   /* U+1D032 */
  0x0002,   /* U+1D033 */
  0x0002,   /* U+1D034 */
  0x0002,   /* U+1D035 */
  0x0002,   /* U+1D036 */
  0x0002,   /* U+1D037 */
  0x0002,   /* U+1D038 */
  0x0002,   /* U+1D039 */
  0x0002,   /* U+1D03A */
  0x0002,   /* U+1D03B */
  0x0002,   /* U+1D03C */
  0x0002,   /* U+1D03D */
  0x0002,   /* U+1D03E */
  0x0002,   /* U+1D03F */
  0x0002,   /* U+1D040 */
  0x0002,   /* U+1D041 */
  0x0002,   /* U+1D042 */
  0x0002,   /* U+1D043 */
  0x0002,   /* U+1D044 */
  0x0002,   /* U+1D045 */
  0x0002,   /* U+1D046 */
  0x0002,   /* U+1D047 */
  0x0002,   /* U+1D048 */
  0x0002,   /* U+1D049 */
  0x0002,   /* U+1D04A */
  0x0002,   /* U+1D04B */
  0x0002,   /* U+1D04C */
  0x0002,   /* U+1D04D */
  0x0002,   /* U+1D04E */
  0x0002,   /* U+1D04F */
  0x0002,   /* U+1D050 */
  0x0002,   /* U+1D051 */
  0x0002,   /* U+1D052 */
  0x0002,   /* U+1D053 */
  0x0002,   /* U+1D054 */
  0x0002,   /* U+1D055 */
  0x0002,   /* U+1D056 */
  0x0002,   /* U+1D057 */
  0x0002,   /* U+1D058 */
  0x0002,   /* U+1D059 */
  0x0002,   /* U+1D05A */
  0x0002,   /* U+1D05B */
  0x0002,   /* U+1D05C */
  0x0002,   /* U+1D05D */
  0x0002,   /* U+1D05E */
  0x0002,   /* U+1D05F */
  0x0002,   /* U+1D060 */
  0x0002,   /* U+1D061 */
  0x0002,   /* U+1D062 */
  0x0002,   /* U+1D063 */
  0x0002,   /* U+1D064 */
  0x0002,   /* U+1D065 */
  0x0002,   /* U+1D066 */
  0x0002,   /* U+1D067 */
  0x0002,   /* U+1D068 */
  0x0002,   /* U+1D069 */
  0x0002,   /* U+1D06A */
  0x0002,   /* U+1D06B */
  0x0002,   /* U+1D06C */
  0x0002,   /* U+1D06D */
  0x0002,   /* U+1D06E */
  0x0002,   /* U+1D06F */
  0x0002,   /* U+1D070 */
  0x0002,   /* U+1D071 */
  0x0002,   /* U+1D072 */
  0x0002,   /* U+1D073 */
  0x0002,   /* U+1D074 */
  0x0002,   /* U+1D075 */
  0x0002,   /* U+1D076 */
  0x0002,   /* U+1D077 */
  0x0002,   /* U+1D078 */
  0x0002,   /* U+1D079 */
  0x0002,   /* U+1D07A */
  0x0002,   /* U+1D07B */
  0x0002,   /* U+1D07C */
  0x0002,   /* U+1D07D */
  0x0002,   /* U+1D07E */
  0x0002,   /* U+1D07F */
  0x0002,   /* U+1D080 */
  0x0002,   /* U+1D081 */
  0x0002,   /* U+1D082 */
  0x0002,   /* U+1D083 */
  0x0002,   /* U+1D084 */
  0x0002,   /* U+1D085 */
  0x0002,   /* U+1D086 */
  0x0002,   /* U+1D087 */
  0x0002,   /* U+1D088 */
  0x0002,   /* U+1D089 */
  0x0002,   /* U+1D08A */
  0x0002,   /* U+1D08B */
  0x0002,   /* U+1D08C */
  0x0002,   /* U+1D08D */
  0x0002,   /* U+1D08E */
  0x0002,   /* U+1D08F */
  0x0002,   /* U+1D090 */
  0x0002,   /* U+1D091 */
  0x0002,   /* U+1D092 */
  0x0002,   /* U+1D093 */
  0x0002,   /* U+1D094 */
  0x0002,   /* U+1D095 */
  0x0002,   /* U+1D096 */
  0x0002,   /* U+1D097 */
  0x0002,   /* U+1D098 */
  0x0002,   /* U+1D099 */
  0x0002,   /* U+1D09A */
  0x0002,   /* U+1D09B */
  0x0002,   /* U+1D09C */
  0x0002,   /* U+1D09D */
  0x0002,   /* U+1D09E */
  0x0002,   /* U+1D09F */
  0x0002,   /* U+1D0A0 */
  0x0002,   /* U+1D0A1 */
  0x0002,   /* U+1D0A2 */
  0x0002,   /* U+1D0A3 */
  0x0002,   /* U+1D0A4 */
  0x0002,   /* U+1D0A5 */
  0x0002,   /* U+1D0A6 */
  0x0002,   /* U+1D0A7 */
  0x0002,   /* U+1D0A8 */
  0x0002,   /* U+1D0A9 */
  0x0002,   /* U+1D0AA */
  0x0002,   /* U+1D0AB */
  0x0002,   /* U+1D0AC */
  0x0002,   /* U+1D0AD */
  0x0002,   /* U+1D0AE */
  0x0002,   /* U+1D0AF */
  0x0002,   /* U+1D0B0 */
  0x0002,   /* U+1D0B1 */
  0x0002,   /* U+1D0B2 */
  0x0002,   /* U+1D0B3 */
  0x0002,   /* U+1D0B4 */
  0x0002,   /* U+1D0B5 */
  0x0002,   /* U+1D0B6 */
  0x0002,   /* U+1D0B7 */
  0x0002,   /* U+1D0B8 */
  0x0002,   /* U+1D0B9 */
  0x0002,   /* U+1D0BA */
  0x0002,   /* U+1D0BB */
  0x0002,   /* U+1D0BC */
  0x0002,   /* U+1D0BD */
  0x0002,   /* U+1D0BE */
  0x0002,   /* U+1D0BF */
  0x0002,   /* U+1D0C0 */
  0x0002,   /* U+1D0C1 */
  0x0002,   /* U+1D0C2 */
  0x0002,   /* U+1D0C3 */
  0x0002,   /* U+1D0C4 */
  0x0002,   /* U+1D0C5 */
  0x0002,   /* U+1D0C6 */
  0x0002,   /* U+1D0C7 */
  0x0002,   /* U+1D0C8 */
  0x0002,   /* U+1D0C9 */
  0x0002,   /* U+1D0CA */
  0x0002,   /* U+1D0CB */
  0x0002,   /* U+1D0CC */
  0x0002,   /* U+1D0CD */
  0x0002,   /* U+1D0CE */
  0x0002,   /* U+1D0CF */
  0x0002,   /* U+1D0D0 */
  0x0002,   /* U+1D0D1 */
  0x0002,   /* U+1D0D2 */
  0x0002,   /* U+1D0D3 */
  0x0002,   /* U+1D0D4 */
  0x0002,   /* U+1D0D5 */
  0x0002,   /* U+1D0D6 */
  0x0002,   /* U+1D0D7 */
  0x0002,   /* U+1D0D8 */
  0x0002,   /* U+1D0D9 */
  0x0002,   /* U+1D0DA */
  0x0002,   /* U+1D0DB */
  0x0002,   /* U+1D0DC */
  0x0002,   /* U+1D0DD */
  0x0002,   /* U+1D0DE */
  0x0002,   /* U+1D0DF */
  0x0002,   /* U+1D0E0 */
  0x0002,   /* U+1D0E1 */
  0x0002,   /* U+1D0E2 */
  0x0002,   /* U+1D0E3 */
  0x0002,   /* U+1D0E4 */
  0x0002,   /* U+1D0E5 */
  0x0002,   /* U+1D0E6 */
  0x0002,   /* U+1D0E7 */
  0x0002,   /* U+1D0E8 */
  0x0002,   /* U+1D0E9 */
  0x0002,   /* U+1D0EA */
  0x0002,   /* U+1D0EB */
  0x0002,   /* U+1D0EC */
  0x0002,   /* U+1D0ED */
  0x0002,   /* U+1D0EE */
  0x0002,   /* U+1D0EF */
  0x0002,   /* U+1D0F0 */
  0x0002,   /* U+1D0F1 */
  0x0002,   /* U+1D0F2 */
  0x0002,   /* U+1D0F3 */
  0x0002,   /* U+1D0F4 */
  0x0002,   /* U+1D0F5 */
  0x0002,   /* U+1D0F6 */
  0x0002,   /* U+1D0F7 */
  0x0002,   /* U+1D0F8 */
  0x0002,   /* U+1D0F9 */
  0x0002,   /* U+1D0FA */
  0x0002,   /* U+1D0FB */
  0x0002,   /* U+1D0FC */
  0x0002,   /* U+1D0FD */
  0x0002,   /* U+1D0FE */
  0x0002,   /* U+1D0FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D000 */
  0x0000,   /* U+1D001 */
  0x0000,   /* U+1D002 */
  0x0000,   /* U+1D003 */
  0x0000,   /* U+1D004 */
  0x0000,   /* U+1D005 */
  0x0000,   /* U+1D006 */
  0x0000,   /* U+1D007 */
  0x0000,   /* U+1D008 */
  0x0000,   /* U+1D009 */
  0x0000,   /* U+1D00A */
  0x0000,   /* U+1D00B */
  0x0000,   /* U+1D00C */
  0x0000,   /* U+1D00D */
  0x0000,   /* U+1D00E */
  0x0000,   /* U+1D00F */
  0x0000,   /* U+1D010 */
  0x0000,   /* U+1D011 */
  0x0000,   /* U+1D012 */
  0x0000,   /* U+1D013 */
  0x0000,   /* U+1D014 */
  0x0000,   /* U+1D015 */
  0x0000,   /* U+1D016 */
  0x0000,   /* U+1D017 */
  0x0000,   /* U+1D018 */
  0x0000,   /* U+1D019 */
  0x0000,   /* U+1D01A */
  0x0000,   /* U+1D01B */
  0x0000,   /* U+1D01C */
  0x0000,   /* U+1D01D */
  0x0000,   /* U+1D01E */
  0x0000,   /* U+1D01F */
  0x0000,   /* U+1D020 */
  0x0000,   /* U+1D021 */
  0x0000,   /* U+1D022 */
  0x0000,   /* U+1D023 */
  0x0000,   /* U+1D024 */
  0x0000,   /* U+1D025 */
  0x0000,   /* U+1D026 */
  0x0000,   /* U+1D027 */
  0x0000,   /* U+1D028 */
  0x0000,   /* U+1D029 */
  0x0000,   /* U+1D02A */
  0x0000,   /* U+1D02B */
  0x0000,   /* U+1D02C */
  0x0000,   /* U+1D02D */
  0x0000,   /* U+1D02E */
  0x0000,   /* U+1D02F */
  0x0000,   /* U+1D030 */
  0x0000,   /* U+1D031 */
  0x0000,   /* U+1D032 */
  0x0000,   /* U+1D033 */
  0x0000,   /* U+1D034 */
  0x0000,   /* U+1D035 */
  0x0000,   /* U+1D036 */
  0x0000,   /* U+1D037 */
  0x0000,   /* U+1D038 */
  0x0000,   /* U+1D039 */
  0x0000,   /* U+1D03A */
  0x0000,   /* U+1D03B */
  0x0000,   /* U+1D03C */
  0x0000,   /* U+1D03D */
  0x0000,   /* U+1D03E */
  0x0000,   /* U+1D03F */
  0x0000,   /* U+1D040 */
  0x0000,   /* U+1D041 */
  0x0000,   /* U+1D042 */
  0x0000,   /* U+1D043 */
  0x0000,   /* U+1D044 */
  0x0000,   /* U+1D045 */
  0x0000,   /* U+1D046 */
  0x0000,   /* U+1D047 */
  0x0000,   /* U+1D048 */
  0x0000,   /* U+1D049 */
  0x0000,   /* U+1D04A */
  0x0000,   /* U+1D04B */
  0x0000,   /* U+1D04C */
  0x0000,   /* U+1D04D */
  0x0000,   /* U+1D04E */
  0x0000,   /* U+1D04F */
  0x0000,   /* U+1D050 */
  0x0000,   /* U+1D051 */
  0x0000,   /* U+1D052 */
  0x0000,   /* U+1D053 */
  0x0000,   /* U+1D054 */
  0x0000,   /* U+1D055 */
  0x0000,   /* U+1D056 */
  0x0000,   /* U+1D057 */
  0x0000,   /* U+1D058 */
  0x0000,   /* U+1D059 */
  0x0000,   /* U+1D05A */
  0x0000,   /* U+1D05B */
  0x0000,   /* U+1D05C */
  0x0000,   /* U+1D05D */
  0x0000,   /* U+1D05E */
  0x0000,   /* U+1D05F */
  0x0000,   /* U+1D060 */
  0x0000,   /* U+1D061 */
  0x0000,   /* U+1D062 */
  0x0000,   /* U+1D063 */
  0x0000,   /* U+1D064 */
  0x0000,   /* U+1D065 */
  0x0000,   /* U+1D066 */
  0x0000,   /* U+1D067 */
  0x0000,   /* U+1D068 */
  0x0000,   /* U+1D069 */
  0x0000,   /* U+1D06A */
  0x0000,   /* U+1D06B */
  0x0000,   /* U+1D06C */
  0x0000,   /* U+1D06D */
  0x0000,   /* U+1D06E */
  0x0000,   /* U+1D06F */
  0x0000,   /* U+1D070 */
  0x0000,   /* U+1D071 */
  0x0000,   /* U+1D072 */
  0x0000,   /* U+1D073 */
  0x0000,   /* U+1D074 */
  0x0000,   /* U+1D075 */
  0x0000,   /* U+1D076 */
  0x0000,   /* U+1D077 */
  0x0000,   /* U+1D078 */
  0x0000,   /* U+1D079 */
  0x0000,   /* U+1D07A */
  0x0000,   /* U+1D07B */
  0x0000,   /* U+1D07C */
  0x0000,   /* U+1D07D */
  0x0000,   /* U+1D07E */
  0x0000,   /* U+1D07F */
  0x0000,   /* U+1D080 */
  0x0000,   /* U+1D081 */
  0x0000,   /* U+1D082 */
  0x0000,   /* U+1D083 */
  0x0000,   /* U+1D084 */
  0x0000,   /* U+1D085 */
  0x0000,   /* U+1D086 */
  0x0000,   /* U+1D087 */
  0x0000,   /* U+1D088 */
  0x0000,   /* U+1D089 */
  0x0000,   /* U+1D08A */
  0x0000,   /* U+1D08B */
  0x0000,   /* U+1D08C */
  0x0000,   /* U+1D08D */
  0x0000,   /* U+1D08E */
  0x0000,   /* U+1D08F */
  0x0000,   /* U+1D090 */
  0x0000,   /* U+1D091 */
  0x0000,   /* U+1D092 */
  0x0000,   /* U+1D093 */
  0x0000,   /* U+1D094 */
  0x0000,   /* U+1D095 */
  0x0000,   /* U+1D096 */
  0x0000,   /* U+1D097 */
  0x0000,   /* U+1D098 */
  0x0000,   /* U+1D099 */
  0x0000,   /* U+1D09A */
  0x0000,   /* U+1D09B */
  0x0000,   /* U+1D09C */
  0x0000,   /* U+1D09D */
  0x0000,   /* U+1D09E */
  0x0000,   /* U+1D09F */
  0x0000,   /* U+1D0A0 */
  0x0000,   /* U+1D0A1 */
  0x0000,   /* U+1D0A2 */
  0x0000,   /* U+1D0A3 */
  0x0000,   /* U+1D0A4 */
  0x0000,   /* U+1D0A5 */
  0x0000,   /* U+1D0A6 */
  0x0000,   /* U+1D0A7 */
  0x0000,   /* U+1D0A8 */
  0x0000,   /* U+1D0A9 */
  0x0000,   /* U+1D0AA */
  0x0000,   /* U+1D0AB */
  0x0000,   /* U+1D0AC */
  0x0000,   /* U+1D0AD */
  0x0000,   /* U+1D0AE */
  0x0000,   /* U+1D0AF */
  0x0000,   /* U+1D0B0 */
  0x0000,   /* U+1D0B1 */
  0x0000,   /* U+1D0B2 */
  0x0000,   /* U+1D0B3 */
  0x0000,   /* U+1D0B4 */
  0x0000,   /* U+1D0B5 */
  0x0000,   /* U+1D0B6 */
  0x0000,   /* U+1D0B7 */
  0x0000,   /* U+1D0B8 */
  0x0000,   /* U+1D0B9 */
  0x0000,   /* U+1D0BA */
  0x0000,   /* U+1D0BB */
  0x0000,   /* U+1D0BC */
  0x0000,   /* U+1D0BD */
  0x0000,   /* U+1D0BE */
  0x0000,   /* U+1D0BF */
  0x0000,   /* U+1D0C0 */
  0x0000,   /* U+1D0C1 */
  0x0000,   /* U+1D0C2 */
  0x0000,   /* U+1D0C3 */
  0x0000,   /* U+1D0C4 */
  0x0000,   /* U+1D0C5 */
  0x0000,   /* U+1D0C6 */
  0x0000,   /* U+1D0C7 */
  0x0000,   /* U+1D0C8 */
  0x0000,   /* U+1D0C9 */
  0x0000,   /* U+1D0CA */
  0x0000,   /* U+1D0CB */
  0x0000,   /* U+1D0CC */
  0x0000,   /* U+1D0CD */
  0x0000,   /* U+1D0CE */
  0x0000,   /* U+1D0CF */
  0x0000,   /* U+1D0D0 */
  0x0000,   /* U+1D0D1 */
  0x0000,   /* U+1D0D2 */
  0x0000,   /* U+1D0D3 */
  0x0000,   /* U+1D0D4 */
  0x0000,   /* U+1D0D5 */
  0x0000,   /* U+1D0D6 */
  0x0000,   /* U+1D0D7 */
  0x0000,   /* U+1D0D8 */
  0x0000,   /* U+1D0D9 */
  0x0000,   /* U+1D0DA */
  0x0000,   /* U+1D0DB */
  0x0000,   /* U+1D0DC */
  0x0000,   /* U+1D0DD */
  0x0000,   /* U+1D0DE */
  0x0000,   /* U+1D0DF */
  0x0000,   /* U+1D0E0 */
  0x0000,   /* U+1D0E1 */
  0x0000,   /* U+1D0E2 */
  0x0000,   /* U+1D0E3 */
  0x0000,   /* U+1D0E4 */
  0x0000,   /* U+1D0E5 */
  0x0000,   /* U+1D0E6 */
  0x0000,   /* U+1D0E7 */
  0x0000,   /* U+1D0E8 */
  0x0000,   /* U+1D0E9 */
  0x0000,   /* U+1D0EA */
  0x0000,   /* U+1D0EB */
  0x0000,   /* U+1D0EC */
  0x0000,   /* U+1D0ED */
  0x0000,   /* U+1D0EE */
  0x0000,   /* U+1D0EF */
  0x0000,   /* U+1D0F0 */
  0x0000,   /* U+1D0F1 */
  0x0000,   /* U+1D0F2 */
  0x0000,   /* U+1D0F3 */
  0x0000,   /* U+1D0F4 */
  0x0000,   /* U+1D0F5 */
  0xD0F6,   /* U+1D0F6 */
  0xD0F7,   /* U+1D0F7 */
  0xD0F8,   /* U+1D0F8 */
  0xD0F9,   /* U+1D0F9 */
  0xD0FA,   /* U+1D0FA */
  0xD0FB,   /* U+1D0FB */
  0xD0FC,   /* U+1D0FC */
  0xD0FD,   /* U+1D0FD */
  0xD0FE,   /* U+1D0FE */
  0xD0FF,   /* U+1D0FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D000 */
  0x0000,   /* U+1D001 */
  0x0000,   /* U+1D002 */
  0x0000,   /* U+1D003 */
  0x0000,   /* U+1D004 */
  0x0000,   /* U+1D005 */
  0x0000,   /* U+1D006 */
  0x0000,   /* U+1D007 */
  0x0000,   /* U+1D008 */
  0x0000,   /* U+1D009 */
  0x0000,   /* U+1D00A */
  0x0000,   /* U+1D00B */
  0x0000,   /* U+1D00C */
  0x0000,   /* U+1D00D */
  0x0000,   /* U+1D00E */
  0x0000,   /* U+1D00F */
  0x0000,   /* U+1D010 */
  0x0000,   /* U+1D011 */
  0x0000,   /* U+1D012 */
  0x0000,   /* U+1D013 */
  0x0000,   /* U+1D014 */
  0x0000,   /* U+1D015 */
  0x0000,   /* U+1D016 */
  0x0000,   /* U+1D017 */
  0x0000,   /* U+1D018 */
  0x0000,   /* U+1D019 */
  0x0000,   /* U+1D01A */
  0x0000,   /* U+1D01B */
  0x0000,   /* U+1D01C */
  0x0000,   /* U+1D01D */
  0x0000,   /* U+1D01E */
  0x0000,   /* U+1D01F */
  0x0000,   /* U+1D020 */
  0x0000,   /* U+1D021 */
  0x0000,   /* U+1D022 */
  0x0000,   /* U+1D023 */
  0x0000,   /* U+1D024 */
  0x0000,   /* U+1D025 */
  0x0000,   /* U+1D026 */
  0x0000,   /* U+1D027 */
  0x0000,   /* U+1D028 */
  0x0000,   /* U+1D029 */
  0x0000,   /* U+1D02A */
  0x0000,   /* U+1D02B */
  0x0000,   /* U+1D02C */
  0x0000,   /* U+1D02D */
  0x0000,   /* U+1D02E */
  0x0000,   /* U+1D02F */
  0x0000,   /* U+1D030 */
  0x0000,   /* U+1D031 */
  0x0000,   /* U+1D032 */
  0x0000,   /* U+1D033 */
  0x0000,   /* U+1D034 */
  0x0000,   /* U+1D035 */
  0x0000,   /* U+1D036 */
  0x0000,   /* U+1D037 */
  0x0000,   /* U+1D038 */
  0x0000,   /* U+1D039 */
  0x0000,   /* U+1D03A */
  0x0000,   /* U+1D03B */
  0x0000,   /* U+1D03C */
  0x0000,   /* U+1D03D */
  0x0000,   /* U+1D03E */
  0x0000,   /* U+1D03F */
  0x0000,   /* U+1D040 */
  0x0000,   /* U+1D041 */
  0x0000,   /* U+1D042 */
  0x0000,   /* U+1D043 */
  0x0000,   /* U+1D044 */
  0x0000,   /* U+1D045 */
  0x0000,   /* U+1D046 */
  0x0000,   /* U+1D047 */
  0x0000,   /* U+1D048 */
  0x0000,   /* U+1D049 */
  0x0000,   /* U+1D04A */
  0x0000,   /* U+1D04B */
  0x0000,   /* U+1D04C */
  0x0000,   /* U+1D04D */
  0x0000,   /* U+1D04E */
  0x0000,   /* U+1D04F */
  0x0000,   /* U+1D050 */
  0x0000,   /* U+1D051 */
  0x0000,   /* U+1D052 */
  0x0000,   /* U+1D053 */
  0x0000,   /* U+1D054 */
  0x0000,   /* U+1D055 */
  0x0000,   /* U+1D056 */
  0x0000,   /* U+1D057 */
  0x0000,   /* U+1D058 */
  0x0000,   /* U+1D059 */
  0x0000,   /* U+1D05A */
  0x0000,   /* U+1D05B */
  0x0000,   /* U+1D05C */
  0x0000,   /* U+1D05D */
  0x0000,   /* U+1D05E */
  0x0000,   /* U+1D05F */
  0x0000,   /* U+1D060 */
  0x0000,   /* U+1D061 */
  0x0000,   /* U+1D062 */
  0x0000,   /* U+1D063 */
  0x0000,   /* U+1D064 */
  0x0000,   /* U+1D065 */
  0x0000,   /* U+1D066 */
  0x0000,   /* U+1D067 */
  0x0000,   /* U+1D068 */
  0x0000,   /* U+1D069 */
  0x0000,   /* U+1D06A */
  0x0000,   /* U+1D06B */
  0x0000,   /* U+1D06C */
  0x0000,   /* U+1D06D */
  0x0000,   /* U+1D06E */
  0x0000,   /* U+1D06F */
  0x0000,   /* U+1D070 */
  0x0000,   /* U+1D071 */
  0x0000,   /* U+1D072 */
  0x0000,   /* U+1D073 */
  0x0000,   /* U+1D074 */
  0x0000,   /* U+1D075 */
  0x0000,   /* U+1D076 */
  0x0000,   /* U+1D077 */
  0x0000,   /* U+1D078 */
  0x0000,   /* U+1D079 */
  0x0000,   /* U+1D07A */
  0x0000,   /* U+1D07B */
  0x0000,   /* U+1D07C */
  0x0000,   /* U+1D07D */
  0x0000,   /* U+1D07E */
  0x0000,   /* U+1D07F */
  0x0000,   /* U+1D080 */
  0x0000,   /* U+1D081 */
  0x0000,   /* U+1D082 */
  0x0000,   /* U+1D083 */
  0x0000,   /* U+1D084 */
  0x0000,   /* U+1D085 */
  0x0000,   /* U+1D086 */
  0x0000,   /* U+1D087 */
  0x0000,   /* U+1D088 */
  0x0000,   /* U+1D089 */
  0x0000,   /* U+1D08A */
  0x0000,   /* U+1D08B */
  0x0000,   /* U+1D08C */
  0x0000,   /* U+1D08D */
  0x0000,   /* U+1D08E */
  0x0000,   /* U+1D08F */
  0x0000,   /* U+1D090 */
  0x0000,   /* U+1D091 */
  0x0000,   /* U+1D092 */
  0x0000,   /* U+1D093 */
  0x0000,   /* U+1D094 */
  0x0000,   /* U+1D095 */
  0x0000,   /* U+1D096 */
  0x0000,   /* U+1D097 */
  0x0000,   /* U+1D098 */
  0x0000,   /* U+1D099 */
  0x0000,   /* U+1D09A */
  0x0000,   /* U+1D09B */
  0x0000,   /* U+1D09C */
  0x0000,   /* U+1D09D */
  0x0000,   /* U+1D09E */
  0x0000,   /* U+1D09F */
  0x0000,   /* U+1D0A0 */
  0x0000,   /* U+1D0A1 */
  0x0000,   /* U+1D0A2 */
  0x0000,   /* U+1D0A3 */
  0x0000,   /* U+1D0A4 */
  0x0000,   /* U+1D0A5 */
  0x0000,   /* U+1D0A6 */
  0x0000,   /* U+1D0A7 */
  0x0000,   /* U+1D0A8 */
  0x0000,   /* U+1D0A9 */
  0x0000,   /* U+1D0AA */
  0x0000,   /* U+1D0AB */
  0x0000,   /* U+1D0AC */
  0x0000,   /* U+1D0AD */
  0x0000,   /* U+1D0AE */
  0x0000,   /* U+1D0AF */
  0x0000,   /* U+1D0B0 */
  0x0000,   /* U+1D0B1 */
  0x0000,   /* U+1D0B2 */
  0x0000,   /* U+1D0B3 */
  0x0000,   /* U+1D0B4 */
  0x0000,   /* U+1D0B5 */
  0x0000,   /* U+1D0B6 */
  0x0000,   /* U+1D0B7 */
  0x0000,   /* U+1D0B8 */
  0x0000,   /* U+1D0B9 */
  0x0000,   /* U+1D0BA */
  0x0000,   /* U+1D0BB */
  0x0000,   /* U+1D0BC */
  0x0000,   /* U+1D0BD */
  0x0000,   /* U+1D0BE */
  0x0000,   /* U+1D0BF */
  0x0000,   /* U+1D0C0 */
  0x0000,   /* U+1D0C1 */
  0x0000,   /* U+1D0C2 */
  0x0000,   /* U+1D0C3 */
  0x0000,   /* U+1D0C4 */
  0x0000,   /* U+1D0C5 */
  0x0000,   /* U+1D0C6 */
  0x0000,   /* U+1D0C7 */
  0x0000,   /* U+1D0C8 */
  0x0000,   /* U+1D0C9 */
  0x0000,   /* U+1D0CA */
  0x0000,   /* U+1D0CB */
  0x0000,   /* U+1D0CC */
  0x0000,   /* U+1D0CD */
  0x0000,   /* U+1D0CE */
  0x0000,   /* U+1D0CF */
  0x0000,   /* U+1D0D0 */
  0x0000,   /* U+1D0D1 */
  0x0000,   /* U+1D0D2 */
  0x0000,   /* U+1D0D3 */
  0x0000,   /* U+1D0D4 */
  0x0000,   /* U+1D0D5 */
  0x0000,   /* U+1D0D6 */
  0x0000,   /* U+1D0D7 */
  0x0000,   /* U+1D0D8 */
  0x0000,   /* U+1D0D9 */
  0x0000,   /* U+1D0DA */
  0x0000,   /* U+1D0DB */
  0x0000,   /* U+1D0DC */
  0x0000,   /* U+1D0DD */
  0x0000,   /* U+1D0DE */
  0x0000,   /* U+1D0DF */
  0x0000,   /* U+1D0E0 */
  0x0000,   /* U+1D0E1 */
  0x0000,   /* U+1D0E2 */
  0x0000,   /* U+1D0E3 */
  0x0000,   /* U+1D0E4 */
  0x0000,   /* U+1D0E5 */
  0x0000,   /* U+1D0E6 */
  0x0000,   /* U+1D0E7 */
  0x0000,   /* U+1D0E8 */
  0x0000,   /* U+1D0E9 */
  0x0000,   /* U+1D0EA */
  0x0000,   /* U+1D0EB */
  0x0000,   /* U+1D0EC */
  0x0000,   /* U+1D0ED */
  0x0000,   /* U+1D0EE */
  0x0000,   /* U+1D0EF */
  0x0000,   /* U+1D0F0 */
  0x0000,   /* U+1D0F1 */
  0x0000,   /* U+1D0F2 */
  0x0000,   /* U+1D0F3 */
  0x0000,   /* U+1D0F4 */
  0x0000,   /* U+1D0F5 */
  0x0000,   /* U+1D0F6 */
  0x0000,   /* U+1D0F7 */
  0x0000,   /* U+1D0F8 */
  0x0000,   /* U+1D0F9 */
  0x0000,   /* U+1D0FA */
  0x0000,   /* U+1D0FB */
  0x0000,   /* U+1D0FC */
  0x0000,   /* U+1D0FD */
  0x0000,   /* U+1D0FE */
  0x0000,   /* U+1D0FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D000 */
  0x0000,   /* U+1D001 */
  0x0000,   /* U+1D002 */
  0x0000,   /* U+1D003 */
  0x0000,   /* U+1D004 */
  0x0000,   /* U+1D005 */
  0x0000,   /* U+1D006 */
  0x0000,   /* U+1D007 */
  0x0000,   /* U+1D008 */
  0x0000,   /* U+1D009 */
  0x0000,   /* U+1D00A */
  0x0000,   /* U+1D00B */
  0x0000,   /* U+1D00C */
  0x0000,   /* U+1D00D */
  0x0000,   /* U+1D00E */
  0x0000,   /* U+1D00F */
  0x0000,   /* U+1D010 */
  0x0000,   /* U+1D011 */
  0x0000,   /* U+1D012 */
  0x0000,   /* U+1D013 */
  0x0000,   /* U+1D014 */
  0x0000,   /* U+1D015 */
  0x0000,   /* U+1D016 */
  0x0000,   /* U+1D017 */
  0x0000,   /* U+1D018 */
  0x0000,   /* U+1D019 */
  0x0000,   /* U+1D01A */
  0x0000,   /* U+1D01B */
  0x0000,   /* U+1D01C */
  0x0000,   /* U+1D01D */
  0x0000,   /* U+1D01E */
  0x0000,   /* U+1D01F */
  0x0000,   /* U+1D020 */
  0x0000,   /* U+1D021 */
  0x0000,   /* U+1D022 */
  0x0000,   /* U+1D023 */
  0x0000,   /* U+1D024 */
  0x0000,   /* U+1D025 */
  0x0000,   /* U+1D026 */
  0x0000,   /* U+1D027 */
  0x0000,   /* U+1D028 */
  0x0000,   /* U+1D029 */
  0x0000,   /* U+1D02A */
  0x0000,   /* U+1D02B */
  0x0000,   /* U+1D02C */
  0x0000,   /* U+1D02D */
  0x0000,   /* U+1D02E */
  0x0000,   /* U+1D02F */
  0x0000,   /* U+1D030 */
  0x0000,   /* U+1D031 */
  0x0000,   /* U+1D032 */
  0x0000,   /* U+1D033 */
  0x0000,   /* U+1D034 */
  0x0000,   /* U+1D035 */
  0x0000,   /* U+1D036 */
  0x0000,   /* U+1D037 */
  0x0000,   /* U+1D038 */
  0x0000,   /* U+1D039 */
  0x0000,   /* U+1D03A */
  0x0000,   /* U+1D03B */
  0x0000,   /* U+1D03C */
  0x0000,   /* U+1D03D */
  0x0000,   /* U+1D03E */
  0x0000,   /* U+1D03F */
  0x0000,   /* U+1D040 */
  0x0000,   /* U+1D041 */
  0x0000,   /* U+1D042 */
  0x0000,   /* U+1D043 */
  0x0000,   /* U+1D044 */
  0x0000,   /* U+1D045 */
  0x0000,   /* U+1D046 */
  0x0000,   /* U+1D047 */
  0x0000,   /* U+1D048 */
  0x0000,   /* U+1D049 */
  0x0000,   /* U+1D04A */
  0x0000,   /* U+1D04B */
  0x0000,   /* U+1D04C */
  0x0000,   /* U+1D04D */
  0x0000,   /* U+1D04E */
  0x0000,   /* U+1D04F */
  0x0000,   /* U+1D050 */
  0x0000,   /* U+1D051 */
  0x0000,   /* U+1D052 */
  0x0000,   /* U+1D053 */
  0x0000,   /* U+1D054 */
  0x0000,   /* U+1D055 */
  0x0000,   /* U+1D056 */
  0x0000,   /* U+1D057 */
  0x0000,   /* U+1D058 */
  0x0000,   /* U+1D059 */
  0x0000,   /* U+1D05A */
  0x0000,   /* U+1D05B */
  0x0000,   /* U+1D05C */
  0x0000,   /* U+1D05D */
  0x0000,   /* U+1D05E */
  0x0000,   /* U+1D05F */
  0x0000,   /* U+1D060 */
  0x0000,   /* U+1D061 */
  0x0000,   /* U+1D062 */
  0x0000,   /* U+1D063 */
  0x0000,   /* U+1D064 */
  0x0000,   /* U+1D065 */
  0x0000,   /* U+1D066 */
  0x0000,   /* U+1D067 */
  0x0000,   /* U+1D068 */
  0x0000,   /* U+1D069 */
  0x0000,   /* U+1D06A */
  0x0000,   /* U+1D06B */
  0x0000,   /* U+1D06C */
  0x0000,   /* U+1D06D */
  0x0000,   /* U+1D06E */
  0x0000,   /* U+1D06F */
  0x0000,   /* U+1D070 */
  0x0000,   /* U+1D071 */
  0x0000,   /* U+1D072 */
  0x0000,   /* U+1D073 */
  0x0000,   /* U+1D074 */
  0x0000,   /* U+1D075 */
  0x0000,   /* U+1D076 */
  0x0000,   /* U+1D077 */
  0x0000,   /* U+1D078 */
  0x0000,   /* U+1D079 */
  0x0000,   /* U+1D07A */
  0x0000,   /* U+1D07B */
  0x0000,   /* U+1D07C */
  0x0000,   /* U+1D07D */
  0x0000,   /* U+1D07E */
  0x0000,   /* U+1D07F */
  0x0000,   /* U+1D080 */
  0x0000,   /* U+1D081 */
  0x0000,   /* U+1D082 */
  0x0000,   /* U+1D083 */
  0x0000,   /* U+1D084 */
  0x0000,   /* U+1D085 */
  0x0000,   /* U+1D086 */
  0x0000,   /* U+1D087 */
  0x0000,   /* U+1D088 */
  0x0000,   /* U+1D089 */
  0x0000,   /* U+1D08A */
  0x0000,   /* U+1D08B */
  0x0000,   /* U+1D08C */
  0x0000,   /* U+1D08D */
  0x0000,   /* U+1D08E */
  0x0000,   /* U+1D08F */
  0x0000,   /* U+1D090 */
  0x0000,   /* U+1D091 */
  0x0000,   /* U+1D092 */
  0x0000,   /* U+1D093 */
  0x0000,   /* U+1D094 */
  0x0000,   /* U+1D095 */
  0x0000,   /* U+1D096 */
  0x0000,   /* U+1D097 */
  0x0000,   /* U+1D098 */
  0x0000,   /* U+1D099 */
  0x0000,   /* U+1D09A */
  0x0000,   /* U+1D09B */
  0x0000,   /* U+1D09C */
  0x0000,   /* U+1D09D */
  0x0000,   /* U+1D09E */
  0x0000,   /* U+1D09F */
  0x0000,   /* U+1D0A0 */
  0x0000,   /* U+1D0A1 */
  0x0000,   /* U+1D0A2 */
  0x0000,   /* U+1D0A3 */
  0x0000,   /* U+1D0A4 */
  0x0000,   /* U+1D0A5 */
  0x0000,   /* U+1D0A6 */
  0x0000,   /* U+1D0A7 */
  0x0000,   /* U+1D0A8 */
  0x0000,   /* U+1D0A9 */
  0x0000,   /* U+1D0AA */
  0x0000,   /* U+1D0AB */
  0x0000,   /* U+1D0AC */
  0x0000,   /* U+1D0AD */
  0x0000,   /* U+1D0AE */
  0x0000,   /* U+1D0AF */
  0x0000,   /* U+1D0B0 */
  0x0000,   /* U+1D0B1 */
  0x0000,   /* U+1D0B2 */
  0x0000,   /* U+1D0B3 */
  0x0000,   /* U+1D0B4 */
  0x0000,   /* U+1D0B5 */
  0x0000,   /* U+1D0B6 */
  0x0000,   /* U+1D0B7 */
  0x0000,   /* U+1D0B8 */
  0x0000,   /* U+1D0B9 */
  0x0000,   /* U+1D0BA */
  0x0000,   /* U+1D0BB */
  0x0000,   /* U+1D0BC */
  0x0000,   /* U+1D0BD */
  0x0000,   /* U+1D0BE */
  0x0000,   /* U+1D0BF */
  0x0000,   /* U+1D0C0 */
  0x0000,   /* U+1D0C1 */
  0x0000,   /* U+1D0C2 */
  0x0000,   /* U+1D0C3 */
  0x0000,   /* U+1D0C4 */
  0x0000,   /* U+1D0C5 */
  0x0000,   /* U+1D0C6 */
  0x0000,   /* U+1D0C7 */
  0x0000,   /* U+1D0C8 */
  0x0000,   /* U+1D0C9 */
  0x0000,   /* U+1D0CA */
  0x0000,   /* U+1D0CB */
  0x0000,   /* U+1D0CC */
  0x0000,   /* U+1D0CD */
  0x0000,   /* U+1D0CE */
  0x0000,   /* U+1D0CF */
  0x0000,   /* U+1D0D0 */
  0x0000,   /* U+1D0D1 */
  0x0000,   /* U+1D0D2 */
  0x0000,   /* U+1D0D3 */
  0x0000,   /* U+1D0D4 */
  0x0000,   /* U+1D0D5 */
  0x0000,   /* U+1D0D6 */
  0x0000,   /* U+1D0D7 */
  0x0000,   /* U+1D0D8 */
  0x0000,   /* U+1D0D9 */
  0x0000,   /* U+1D0DA */
  0x0000,   /* U+1D0DB */
  0x0000,   /* U+1D0DC */
  0x0000,   /* U+1D0DD */
  0x0000,   /* U+1D0DE */
  0x0000,   /* U+1D0DF */
  0x0000,   /* U+1D0E0 */
  0x0000,   /* U+1D0E1 */
  0x0000,   /* U+1D0E2 */
  0x0000,   /* U+1D0E3 */
  0x0000,   /* U+1D0E4 */
  0x0000,   /* U+1D0E5 */
  0x0000,   /* U+1D0E6 */
  0x0000,   /* U+1D0E7 */
  0x0000,   /* U+1D0E8 */
  0x0000,   /* U+1D0E9 */
  0x0000,   /* U+1D0EA */
  0x0000,   /* U+1D0EB */
  0x0000,   /* U+1D0EC */
  0x0000,   /* U+1D0ED */
  0x0000,   /* U+1D0EE */
  0x0000,   /* U+1D0EF */
  0x0000,   /* U+1D0F0 */
  0x0000,   /* U+1D0F1 */
  0x0000,   /* U+1D0F2 */
  0x0000,   /* U+1D0F3 */
  0x0000,   /* U+1D0F4 */
  0x0000,   /* U+1D0F5 */
  0x0000,   /* U+1D0F6 */
  0x0000,   /* U+1D0F7 */
  0x0000,   /* U+1D0F8 */
  0x0000,   /* U+1D0F9 */
  0x0000,   /* U+1D0FA */
  0x0000,   /* U+1D0FB */
  0x0000,   /* U+1D0FC */
  0x0000,   /* U+1D0FD */
  0x0000,   /* U+1D0FE */
  0x0000,   /* U+1D0FF */
};

uint16 uca900_p1D1[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 3, 3, 3, 
  3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x10CC,   /* U+1D100 */
  0x10CD,   /* U+1D101 */
  0x10CE,   /* U+1D102 */
  0x10CF,   /* U+1D103 */
  0x10D0,   /* U+1D104 */
  0x10D1,   /* U+1D105 */
  0x10D2,   /* U+1D106 */
  0x10D3,   /* U+1D107 */
  0x10D4,   /* U+1D108 */
  0x10D5,   /* U+1D109 */
  0x10D6,   /* U+1D10A */
  0x10D7,   /* U+1D10B */
  0x10D8,   /* U+1D10C */
  0x10D9,   /* U+1D10D */
  0x10DA,   /* U+1D10E */
  0x10DB,   /* U+1D10F */
  0x10DC,   /* U+1D110 */
  0x10DD,   /* U+1D111 */
  0x10DE,   /* U+1D112 */
  0x10DF,   /* U+1D113 */
  0x10E0,   /* U+1D114 */
  0x10E1,   /* U+1D115 */
  0x10E2,   /* U+1D116 */
  0x10E3,   /* U+1D117 */
  0x10E4,   /* U+1D118 */
  0x10E5,   /* U+1D119 */
  0x10E6,   /* U+1D11A */
  0x10E7,   /* U+1D11B */
  0x10E8,   /* U+1D11C */
  0x10E9,   /* U+1D11D */
  0x10EA,   /* U+1D11E */
  0x10EB,   /* U+1D11F */
  0x10EC,   /* U+1D120 */
  0x10ED,   /* U+1D121 */
  0x10EE,   /* U+1D122 */
  0x10EF,   /* U+1D123 */
  0x10F0,   /* U+1D124 */
  0x10F1,   /* U+1D125 */
  0x10F2,   /* U+1D126 */
  0xFBC3,   /* U+1D127 */
  0xFBC3,   /* U+1D128 */
  0x1106,   /* U+1D129 */
  0x10F6,   /* U+1D12A */
  0x10F7,   /* U+1D12B */
  0x10F8,   /* U+1D12C */
  0x10F9,   /* U+1D12D */
  0x10FA,   /* U+1D12E */
  0x10FB,   /* U+1D12F */
  0x10FC,   /* U+1D130 */
  0x10FD,   /* U+1D131 */
  0x10FE,   /* U+1D132 */
  0x10FF,   /* U+1D133 */
  0x1100,   /* U+1D134 */
  0x1101,   /* U+1D135 */
  0x1102,   /* U+1D136 */
  0x1103,   /* U+1D137 */
  0x1104,   /* U+1D138 */
  0x1105,   /* U+1D139 */
  0x1107,   /* U+1D13A */
  0x1108,   /* U+1D13B */
  0x1109,   /* U+1D13C */
  0x110A,   /* U+1D13D */
  0x110B,   /* U+1D13E */
  0x110C,   /* U+1D13F */
  0x110D,   /* U+1D140 */
  0x110E,   /* U+1D141 */
  0x110F,   /* U+1D142 */
  0x1110,   /* U+1D143 */
  0x1111,   /* U+1D144 */
  0x1112,   /* U+1D145 */
  0x1113,   /* U+1D146 */
  0x1114,   /* U+1D147 */
  0x1115,   /* U+1D148 */
  0x1116,   /* U+1D149 */
  0x1117,   /* U+1D14A */
  0x1118,   /* U+1D14B */
  0x1119,   /* U+1D14C */
  0x111A,   /* U+1D14D */
  0x111B,   /* U+1D14E */
  0x111C,   /* U+1D14F */
  0x111D,   /* U+1D150 */
  0x111E,   /* U+1D151 */
  0x111F,   /* U+1D152 */
  0x1120,   /* U+1D153 */
  0x1121,   /* U+1D154 */
  0x1122,   /* U+1D155 */
  0x1123,   /* U+1D156 */
  0x1124,   /* U+1D157 */
  0x1125,   /* U+1D158 */
  0x1126,   /* U+1D159 */
  0x1127,   /* U+1D15A */
  0x1128,   /* U+1D15B */
  0x1129,   /* U+1D15C */
  0x112A,   /* U+1D15D */
  0x1124,   /* U+1D15E */
  0x1125,   /* U+1D15F */
  0x1125,   /* U+1D160 */
  0x1125,   /* U+1D161 */
  0x1125,   /* U+1D162 */
  0x1125,   /* U+1D163 */
  0x1125,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x112B,   /* U+1D16A */
  0x112C,   /* U+1D16B */
  0x112D,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x112E,   /* U+1D183 */
  0x112F,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x1130,   /* U+1D18C */
  0x1131,   /* U+1D18D */
  0x1132,   /* U+1D18E */
  0x1133,   /* U+1D18F */
  0x1134,   /* U+1D190 */
  0x1135,   /* U+1D191 */
  0x1136,   /* U+1D192 */
  0x1137,   /* U+1D193 */
  0x1138,   /* U+1D194 */
  0x1139,   /* U+1D195 */
  0x113A,   /* U+1D196 */
  0x113B,   /* U+1D197 */
  0x113C,   /* U+1D198 */
  0x113D,   /* U+1D199 */
  0x113E,   /* U+1D19A */
  0x113F,   /* U+1D19B */
  0x1140,   /* U+1D19C */
  0x1141,   /* U+1D19D */
  0x1142,   /* U+1D19E */
  0x1143,   /* U+1D19F */
  0x1144,   /* U+1D1A0 */
  0x1145,   /* U+1D1A1 */
  0x1146,   /* U+1D1A2 */
  0x1147,   /* U+1D1A3 */
  0x1148,   /* U+1D1A4 */
  0x1149,   /* U+1D1A5 */
  0x114A,   /* U+1D1A6 */
  0x114B,   /* U+1D1A7 */
  0x114C,   /* U+1D1A8 */
  0x114D,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x114E,   /* U+1D1AE */
  0x114F,   /* U+1D1AF */
  0x1150,   /* U+1D1B0 */
  0x1151,   /* U+1D1B1 */
  0x1152,   /* U+1D1B2 */
  0x1153,   /* U+1D1B3 */
  0x1154,   /* U+1D1B4 */
  0x1155,   /* U+1D1B5 */
  0x1156,   /* U+1D1B6 */
  0x1157,   /* U+1D1B7 */
  0x1158,   /* U+1D1B8 */
  0x1159,   /* U+1D1B9 */
  0x115A,   /* U+1D1BA */
  0x1159,   /* U+1D1BB */
  0x115A,   /* U+1D1BC */
  0x1159,   /* U+1D1BD */
  0x115A,   /* U+1D1BE */
  0x1159,   /* U+1D1BF */
  0x115A,   /* U+1D1C0 */
  0x115B,   /* U+1D1C1 */
  0x115C,   /* U+1D1C2 */
  0x115D,   /* U+1D1C3 */
  0x115E,   /* U+1D1C4 */
  0x115F,   /* U+1D1C5 */
  0x1160,   /* U+1D1C6 */
  0x1161,   /* U+1D1C7 */
  0x1162,   /* U+1D1C8 */
  0x1163,   /* U+1D1C9 */
  0x1164,   /* U+1D1CA */
  0x1165,   /* U+1D1CB */
  0x1166,   /* U+1D1CC */
  0x1167,   /* U+1D1CD */
  0x1168,   /* U+1D1CE */
  0x1169,   /* U+1D1CF */
  0x116A,   /* U+1D1D0 */
  0x116B,   /* U+1D1D1 */
  0x116C,   /* U+1D1D2 */
  0x116D,   /* U+1D1D3 */
  0x116E,   /* U+1D1D4 */
  0x116F,   /* U+1D1D5 */
  0x1170,   /* U+1D1D6 */
  0x1171,   /* U+1D1D7 */
  0x1172,   /* U+1D1D8 */
  0x1173,   /* U+1D1D9 */
  0x1174,   /* U+1D1DA */
  0x1175,   /* U+1D1DB */
  0x1176,   /* U+1D1DC */
  0x1177,   /* U+1D1DD */
  0x1178,   /* U+1D1DE */
  0x1179,   /* U+1D1DF */
  0x117A,   /* U+1D1E0 */
  0x117B,   /* U+1D1E1 */
  0x117C,   /* U+1D1E2 */
  0x117D,   /* U+1D1E3 */
  0x117E,   /* U+1D1E4 */
  0x117F,   /* U+1D1E5 */
  0x1180,   /* U+1D1E6 */
  0x1181,   /* U+1D1E7 */
  0x1182,   /* U+1D1E8 */
  0xFBC3,   /* U+1D1E9 */
  0xFBC3,   /* U+1D1EA */
  0xFBC3,   /* U+1D1EB */
  0xFBC3,   /* U+1D1EC */
  0xFBC3,   /* U+1D1ED */
  0xFBC3,   /* U+1D1EE */
  0xFBC3,   /* U+1D1EF */
  0xFBC3,   /* U+1D1F0 */
  0xFBC3,   /* U+1D1F1 */
  0xFBC3,   /* U+1D1F2 */
  0xFBC3,   /* U+1D1F3 */
  0xFBC3,   /* U+1D1F4 */
  0xFBC3,   /* U+1D1F5 */
  0xFBC3,   /* U+1D1F6 */
  0xFBC3,   /* U+1D1F7 */
  0xFBC3,   /* U+1D1F8 */
  0xFBC3,   /* U+1D1F9 */
  0xFBC3,   /* U+1D1FA */
  0xFBC3,   /* U+1D1FB */
  0xFBC3,   /* U+1D1FC */
  0xFBC3,   /* U+1D1FD */
  0xFBC3,   /* U+1D1FE */
  0xFBC3,   /* U+1D1FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D100 */
  0x0020,   /* U+1D101 */
  0x0020,   /* U+1D102 */
  0x0020,   /* U+1D103 */
  0x0020,   /* U+1D104 */
  0x0020,   /* U+1D105 */
  0x0020,   /* U+1D106 */
  0x0020,   /* U+1D107 */
  0x0020,   /* U+1D108 */
  0x0020,   /* U+1D109 */
  0x0020,   /* U+1D10A */
  0x0020,   /* U+1D10B */
  0x0020,   /* U+1D10C */
  0x0020,   /* U+1D10D */
  0x0020,   /* U+1D10E */
  0x0020,   /* U+1D10F */
  0x0020,   /* U+1D110 */
  0x0020,   /* U+1D111 */
  0x0020,   /* U+1D112 */
  0x0020,   /* U+1D113 */
  0x0020,   /* U+1D114 */
  0x0020,   /* U+1D115 */
  0x0020,   /* U+1D116 */
  0x0020,   /* U+1D117 */
  0x0020,   /* U+1D118 */
  0x0020,   /* U+1D119 */
  0x0020,   /* U+1D11A */
  0x0020,   /* U+1D11B */
  0x0020,   /* U+1D11C */
  0x0020,   /* U+1D11D */
  0x0020,   /* U+1D11E */
  0x0020,   /* U+1D11F */
  0x0020,   /* U+1D120 */
  0x0020,   /* U+1D121 */
  0x0020,   /* U+1D122 */
  0x0020,   /* U+1D123 */
  0x0020,   /* U+1D124 */
  0x0020,   /* U+1D125 */
  0x0020,   /* U+1D126 */
  0x0020,   /* U+1D127 */
  0x0020,   /* U+1D128 */
  0x0020,   /* U+1D129 */
  0x0020,   /* U+1D12A */
  0x0020,   /* U+1D12B */
  0x0020,   /* U+1D12C */
  0x0020,   /* U+1D12D */
  0x0020,   /* U+1D12E */
  0x0020,   /* U+1D12F */
  0x0020,   /* U+1D130 */
  0x0020,   /* U+1D131 */
  0x0020,   /* U+1D132 */
  0x0020,   /* U+1D133 */
  0x0020,   /* U+1D134 */
  0x0020,   /* U+1D135 */
  0x0020,   /* U+1D136 */
  0x0020,   /* U+1D137 */
  0x0020,   /* U+1D138 */
  0x0020,   /* U+1D139 */
  0x0020,   /* U+1D13A */
  0x0020,   /* U+1D13B */
  0x0020,   /* U+1D13C */
  0x0020,   /* U+1D13D */
  0x0020,   /* U+1D13E */
  0x0020,   /* U+1D13F */
  0x0020,   /* U+1D140 */
  0x0020,   /* U+1D141 */
  0x0020,   /* U+1D142 */
  0x0020,   /* U+1D143 */
  0x0020,   /* U+1D144 */
  0x0020,   /* U+1D145 */
  0x0020,   /* U+1D146 */
  0x0020,   /* U+1D147 */
  0x0020,   /* U+1D148 */
  0x0020,   /* U+1D149 */
  0x0020,   /* U+1D14A */
  0x0020,   /* U+1D14B */
  0x0020,   /* U+1D14C */
  0x0020,   /* U+1D14D */
  0x0020,   /* U+1D14E */
  0x0020,   /* U+1D14F */
  0x0020,   /* U+1D150 */
  0x0020,   /* U+1D151 */
  0x0020,   /* U+1D152 */
  0x0020,   /* U+1D153 */
  0x0020,   /* U+1D154 */
  0x0020,   /* U+1D155 */
  0x0020,   /* U+1D156 */
  0x0020,   /* U+1D157 */
  0x0020,   /* U+1D158 */
  0x0020,   /* U+1D159 */
  0x0020,   /* U+1D15A */
  0x0020,   /* U+1D15B */
  0x0020,   /* U+1D15C */
  0x0020,   /* U+1D15D */
  0x0020,   /* U+1D15E */
  0x0020,   /* U+1D15F */
  0x0020,   /* U+1D160 */
  0x0020,   /* U+1D161 */
  0x0020,   /* U+1D162 */
  0x0020,   /* U+1D163 */
  0x0020,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0020,   /* U+1D16A */
  0x0020,   /* U+1D16B */
  0x0020,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0020,   /* U+1D183 */
  0x0020,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0020,   /* U+1D18C */
  0x0020,   /* U+1D18D */
  0x0020,   /* U+1D18E */
  0x0020,   /* U+1D18F */
  0x0020,   /* U+1D190 */
  0x0020,   /* U+1D191 */
  0x0020,   /* U+1D192 */
  0x0020,   /* U+1D193 */
  0x0020,   /* U+1D194 */
  0x0020,   /* U+1D195 */
  0x0020,   /* U+1D196 */
  0x0020,   /* U+1D197 */
  0x0020,   /* U+1D198 */
  0x0020,   /* U+1D199 */
  0x0020,   /* U+1D19A */
  0x0020,   /* U+1D19B */
  0x0020,   /* U+1D19C */
  0x0020,   /* U+1D19D */
  0x0020,   /* U+1D19E */
  0x0020,   /* U+1D19F */
  0x0020,   /* U+1D1A0 */
  0x0020,   /* U+1D1A1 */
  0x0020,   /* U+1D1A2 */
  0x0020,   /* U+1D1A3 */
  0x0020,   /* U+1D1A4 */
  0x0020,   /* U+1D1A5 */
  0x0020,   /* U+1D1A6 */
  0x0020,   /* U+1D1A7 */
  0x0020,   /* U+1D1A8 */
  0x0020,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0020,   /* U+1D1AE */
  0x0020,   /* U+1D1AF */
  0x0020,   /* U+1D1B0 */
  0x0020,   /* U+1D1B1 */
  0x0020,   /* U+1D1B2 */
  0x0020,   /* U+1D1B3 */
  0x0020,   /* U+1D1B4 */
  0x0020,   /* U+1D1B5 */
  0x0020,   /* U+1D1B6 */
  0x0020,   /* U+1D1B7 */
  0x0020,   /* U+1D1B8 */
  0x0020,   /* U+1D1B9 */
  0x0020,   /* U+1D1BA */
  0x0020,   /* U+1D1BB */
  0x0020,   /* U+1D1BC */
  0x0020,   /* U+1D1BD */
  0x0020,   /* U+1D1BE */
  0x0020,   /* U+1D1BF */
  0x0020,   /* U+1D1C0 */
  0x0020,   /* U+1D1C1 */
  0x0020,   /* U+1D1C2 */
  0x0020,   /* U+1D1C3 */
  0x0020,   /* U+1D1C4 */
  0x0020,   /* U+1D1C5 */
  0x0020,   /* U+1D1C6 */
  0x0020,   /* U+1D1C7 */
  0x0020,   /* U+1D1C8 */
  0x0020,   /* U+1D1C9 */
  0x0020,   /* U+1D1CA */
  0x0020,   /* U+1D1CB */
  0x0020,   /* U+1D1CC */
  0x0020,   /* U+1D1CD */
  0x0020,   /* U+1D1CE */
  0x0020,   /* U+1D1CF */
  0x0020,   /* U+1D1D0 */
  0x0020,   /* U+1D1D1 */
  0x0020,   /* U+1D1D2 */
  0x0020,   /* U+1D1D3 */
  0x0020,   /* U+1D1D4 */
  0x0020,   /* U+1D1D5 */
  0x0020,   /* U+1D1D6 */
  0x0020,   /* U+1D1D7 */
  0x0020,   /* U+1D1D8 */
  0x0020,   /* U+1D1D9 */
  0x0020,   /* U+1D1DA */
  0x0020,   /* U+1D1DB */
  0x0020,   /* U+1D1DC */
  0x0020,   /* U+1D1DD */
  0x0020,   /* U+1D1DE */
  0x0020,   /* U+1D1DF */
  0x0020,   /* U+1D1E0 */
  0x0020,   /* U+1D1E1 */
  0x0020,   /* U+1D1E2 */
  0x0020,   /* U+1D1E3 */
  0x0020,   /* U+1D1E4 */
  0x0020,   /* U+1D1E5 */
  0x0020,   /* U+1D1E6 */
  0x0020,   /* U+1D1E7 */
  0x0020,   /* U+1D1E8 */
  0x0020,   /* U+1D1E9 */
  0x0020,   /* U+1D1EA */
  0x0020,   /* U+1D1EB */
  0x0020,   /* U+1D1EC */
  0x0020,   /* U+1D1ED */
  0x0020,   /* U+1D1EE */
  0x0020,   /* U+1D1EF */
  0x0020,   /* U+1D1F0 */
  0x0020,   /* U+1D1F1 */
  0x0020,   /* U+1D1F2 */
  0x0020,   /* U+1D1F3 */
  0x0020,   /* U+1D1F4 */
  0x0020,   /* U+1D1F5 */
  0x0020,   /* U+1D1F6 */
  0x0020,   /* U+1D1F7 */
  0x0020,   /* U+1D1F8 */
  0x0020,   /* U+1D1F9 */
  0x0020,   /* U+1D1FA */
  0x0020,   /* U+1D1FB */
  0x0020,   /* U+1D1FC */
  0x0020,   /* U+1D1FD */
  0x0020,   /* U+1D1FE */
  0x0020,   /* U+1D1FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D100 */
  0x0002,   /* U+1D101 */
  0x0002,   /* U+1D102 */
  0x0002,   /* U+1D103 */
  0x0002,   /* U+1D104 */
  0x0002,   /* U+1D105 */
  0x0002,   /* U+1D106 */
  0x0002,   /* U+1D107 */
  0x0002,   /* U+1D108 */
  0x0002,   /* U+1D109 */
  0x0002,   /* U+1D10A */
  0x0002,   /* U+1D10B */
  0x0002,   /* U+1D10C */
  0x0002,   /* U+1D10D */
  0x0002,   /* U+1D10E */
  0x0002,   /* U+1D10F */
  0x0002,   /* U+1D110 */
  0x0002,   /* U+1D111 */
  0x0002,   /* U+1D112 */
  0x0002,   /* U+1D113 */
  0x0002,   /* U+1D114 */
  0x0002,   /* U+1D115 */
  0x0002,   /* U+1D116 */
  0x0002,   /* U+1D117 */
  0x0002,   /* U+1D118 */
  0x0002,   /* U+1D119 */
  0x0002,   /* U+1D11A */
  0x0002,   /* U+1D11B */
  0x0002,   /* U+1D11C */
  0x0002,   /* U+1D11D */
  0x0002,   /* U+1D11E */
  0x0002,   /* U+1D11F */
  0x0002,   /* U+1D120 */
  0x0002,   /* U+1D121 */
  0x0002,   /* U+1D122 */
  0x0002,   /* U+1D123 */
  0x0002,   /* U+1D124 */
  0x0002,   /* U+1D125 */
  0x0002,   /* U+1D126 */
  0x0002,   /* U+1D127 */
  0x0002,   /* U+1D128 */
  0x0002,   /* U+1D129 */
  0x0002,   /* U+1D12A */
  0x0002,   /* U+1D12B */
  0x0002,   /* U+1D12C */
  0x0002,   /* U+1D12D */
  0x0002,   /* U+1D12E */
  0x0002,   /* U+1D12F */
  0x0002,   /* U+1D130 */
  0x0002,   /* U+1D131 */
  0x0002,   /* U+1D132 */
  0x0002,   /* U+1D133 */
  0x0002,   /* U+1D134 */
  0x0002,   /* U+1D135 */
  0x0002,   /* U+1D136 */
  0x0002,   /* U+1D137 */
  0x0002,   /* U+1D138 */
  0x0002,   /* U+1D139 */
  0x0002,   /* U+1D13A */
  0x0002,   /* U+1D13B */
  0x0002,   /* U+1D13C */
  0x0002,   /* U+1D13D */
  0x0002,   /* U+1D13E */
  0x0002,   /* U+1D13F */
  0x0002,   /* U+1D140 */
  0x0002,   /* U+1D141 */
  0x0002,   /* U+1D142 */
  0x0002,   /* U+1D143 */
  0x0002,   /* U+1D144 */
  0x0002,   /* U+1D145 */
  0x0002,   /* U+1D146 */
  0x0002,   /* U+1D147 */
  0x0002,   /* U+1D148 */
  0x0002,   /* U+1D149 */
  0x0002,   /* U+1D14A */
  0x0002,   /* U+1D14B */
  0x0002,   /* U+1D14C */
  0x0002,   /* U+1D14D */
  0x0002,   /* U+1D14E */
  0x0002,   /* U+1D14F */
  0x0002,   /* U+1D150 */
  0x0002,   /* U+1D151 */
  0x0002,   /* U+1D152 */
  0x0002,   /* U+1D153 */
  0x0002,   /* U+1D154 */
  0x0002,   /* U+1D155 */
  0x0002,   /* U+1D156 */
  0x0002,   /* U+1D157 */
  0x0002,   /* U+1D158 */
  0x0002,   /* U+1D159 */
  0x0002,   /* U+1D15A */
  0x0002,   /* U+1D15B */
  0x0002,   /* U+1D15C */
  0x0002,   /* U+1D15D */
  0x0002,   /* U+1D15E */
  0x0002,   /* U+1D15F */
  0x0002,   /* U+1D160 */
  0x0002,   /* U+1D161 */
  0x0002,   /* U+1D162 */
  0x0002,   /* U+1D163 */
  0x0002,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0002,   /* U+1D16A */
  0x0002,   /* U+1D16B */
  0x0002,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0002,   /* U+1D183 */
  0x0002,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0002,   /* U+1D18C */
  0x0002,   /* U+1D18D */
  0x0002,   /* U+1D18E */
  0x0002,   /* U+1D18F */
  0x0002,   /* U+1D190 */
  0x0002,   /* U+1D191 */
  0x0002,   /* U+1D192 */
  0x0002,   /* U+1D193 */
  0x0002,   /* U+1D194 */
  0x0002,   /* U+1D195 */
  0x0002,   /* U+1D196 */
  0x0002,   /* U+1D197 */
  0x0002,   /* U+1D198 */
  0x0002,   /* U+1D199 */
  0x0002,   /* U+1D19A */
  0x0002,   /* U+1D19B */
  0x0002,   /* U+1D19C */
  0x0002,   /* U+1D19D */
  0x0002,   /* U+1D19E */
  0x0002,   /* U+1D19F */
  0x0002,   /* U+1D1A0 */
  0x0002,   /* U+1D1A1 */
  0x0002,   /* U+1D1A2 */
  0x0002,   /* U+1D1A3 */
  0x0002,   /* U+1D1A4 */
  0x0002,   /* U+1D1A5 */
  0x0002,   /* U+1D1A6 */
  0x0002,   /* U+1D1A7 */
  0x0002,   /* U+1D1A8 */
  0x0002,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0002,   /* U+1D1AE */
  0x0002,   /* U+1D1AF */
  0x0002,   /* U+1D1B0 */
  0x0002,   /* U+1D1B1 */
  0x0002,   /* U+1D1B2 */
  0x0002,   /* U+1D1B3 */
  0x0002,   /* U+1D1B4 */
  0x0002,   /* U+1D1B5 */
  0x0002,   /* U+1D1B6 */
  0x0002,   /* U+1D1B7 */
  0x0002,   /* U+1D1B8 */
  0x0002,   /* U+1D1B9 */
  0x0002,   /* U+1D1BA */
  0x0002,   /* U+1D1BB */
  0x0002,   /* U+1D1BC */
  0x0002,   /* U+1D1BD */
  0x0002,   /* U+1D1BE */
  0x0002,   /* U+1D1BF */
  0x0002,   /* U+1D1C0 */
  0x0002,   /* U+1D1C1 */
  0x0002,   /* U+1D1C2 */
  0x0002,   /* U+1D1C3 */
  0x0002,   /* U+1D1C4 */
  0x0002,   /* U+1D1C5 */
  0x0002,   /* U+1D1C6 */
  0x0002,   /* U+1D1C7 */
  0x0002,   /* U+1D1C8 */
  0x0002,   /* U+1D1C9 */
  0x0002,   /* U+1D1CA */
  0x0002,   /* U+1D1CB */
  0x0002,   /* U+1D1CC */
  0x0002,   /* U+1D1CD */
  0x0002,   /* U+1D1CE */
  0x0002,   /* U+1D1CF */
  0x0002,   /* U+1D1D0 */
  0x0002,   /* U+1D1D1 */
  0x0002,   /* U+1D1D2 */
  0x0002,   /* U+1D1D3 */
  0x0002,   /* U+1D1D4 */
  0x0002,   /* U+1D1D5 */
  0x0002,   /* U+1D1D6 */
  0x0002,   /* U+1D1D7 */
  0x0002,   /* U+1D1D8 */
  0x0002,   /* U+1D1D9 */
  0x0002,   /* U+1D1DA */
  0x0002,   /* U+1D1DB */
  0x0002,   /* U+1D1DC */
  0x0002,   /* U+1D1DD */
  0x0002,   /* U+1D1DE */
  0x0002,   /* U+1D1DF */
  0x0002,   /* U+1D1E0 */
  0x0002,   /* U+1D1E1 */
  0x0002,   /* U+1D1E2 */
  0x0002,   /* U+1D1E3 */
  0x0002,   /* U+1D1E4 */
  0x0002,   /* U+1D1E5 */
  0x0002,   /* U+1D1E6 */
  0x0002,   /* U+1D1E7 */
  0x0002,   /* U+1D1E8 */
  0x0002,   /* U+1D1E9 */
  0x0002,   /* U+1D1EA */
  0x0002,   /* U+1D1EB */
  0x0002,   /* U+1D1EC */
  0x0002,   /* U+1D1ED */
  0x0002,   /* U+1D1EE */
  0x0002,   /* U+1D1EF */
  0x0002,   /* U+1D1F0 */
  0x0002,   /* U+1D1F1 */
  0x0002,   /* U+1D1F2 */
  0x0002,   /* U+1D1F3 */
  0x0002,   /* U+1D1F4 */
  0x0002,   /* U+1D1F5 */
  0x0002,   /* U+1D1F6 */
  0x0002,   /* U+1D1F7 */
  0x0002,   /* U+1D1F8 */
  0x0002,   /* U+1D1F9 */
  0x0002,   /* U+1D1FA */
  0x0002,   /* U+1D1FB */
  0x0002,   /* U+1D1FC */
  0x0002,   /* U+1D1FD */
  0x0002,   /* U+1D1FE */
  0x0002,   /* U+1D1FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0xD127,   /* U+1D127 */
  0xD128,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0xD1E9,   /* U+1D1E9 */
  0xD1EA,   /* U+1D1EA */
  0xD1EB,   /* U+1D1EB */
  0xD1EC,   /* U+1D1EC */
  0xD1ED,   /* U+1D1ED */
  0xD1EE,   /* U+1D1EE */
  0xD1EF,   /* U+1D1EF */
  0xD1F0,   /* U+1D1F0 */
  0xD1F1,   /* U+1D1F1 */
  0xD1F2,   /* U+1D1F2 */
  0xD1F3,   /* U+1D1F3 */
  0xD1F4,   /* U+1D1F4 */
  0xD1F5,   /* U+1D1F5 */
  0xD1F6,   /* U+1D1F6 */
  0xD1F7,   /* U+1D1F7 */
  0xD1F8,   /* U+1D1F8 */
  0xD1F9,   /* U+1D1F9 */
  0xD1FA,   /* U+1D1FA */
  0xD1FB,   /* U+1D1FB */
  0xD1FC,   /* U+1D1FC */
  0xD1FD,   /* U+1D1FD */
  0xD1FE,   /* U+1D1FE */
  0xD1FF,   /* U+1D1FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0x0000,   /* U+1D127 */
  0x0000,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0x0000,   /* U+1D1E9 */
  0x0000,   /* U+1D1EA */
  0x0000,   /* U+1D1EB */
  0x0000,   /* U+1D1EC */
  0x0000,   /* U+1D1ED */
  0x0000,   /* U+1D1EE */
  0x0000,   /* U+1D1EF */
  0x0000,   /* U+1D1F0 */
  0x0000,   /* U+1D1F1 */
  0x0000,   /* U+1D1F2 */
  0x0000,   /* U+1D1F3 */
  0x0000,   /* U+1D1F4 */
  0x0000,   /* U+1D1F5 */
  0x0000,   /* U+1D1F6 */
  0x0000,   /* U+1D1F7 */
  0x0000,   /* U+1D1F8 */
  0x0000,   /* U+1D1F9 */
  0x0000,   /* U+1D1FA */
  0x0000,   /* U+1D1FB */
  0x0000,   /* U+1D1FC */
  0x0000,   /* U+1D1FD */
  0x0000,   /* U+1D1FE */
  0x0000,   /* U+1D1FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0x0000,   /* U+1D127 */
  0x0000,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0x0000,   /* U+1D1E9 */
  0x0000,   /* U+1D1EA */
  0x0000,   /* U+1D1EB */
  0x0000,   /* U+1D1EC */
  0x0000,   /* U+1D1ED */
  0x0000,   /* U+1D1EE */
  0x0000,   /* U+1D1EF */
  0x0000,   /* U+1D1F0 */
  0x0000,   /* U+1D1F1 */
  0x0000,   /* U+1D1F2 */
  0x0000,   /* U+1D1F3 */
  0x0000,   /* U+1D1F4 */
  0x0000,   /* U+1D1F5 */
  0x0000,   /* U+1D1F6 */
  0x0000,   /* U+1D1F7 */
  0x0000,   /* U+1D1F8 */
  0x0000,   /* U+1D1F9 */
  0x0000,   /* U+1D1FA */
  0x0000,   /* U+1D1FB */
  0x0000,   /* U+1D1FC */
  0x0000,   /* U+1D1FD */
  0x0000,   /* U+1D1FE */
  0x0000,   /* U+1D1FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0x0000,   /* U+1D127 */
  0x0000,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0x0000,   /* U+1D1E9 */
  0x0000,   /* U+1D1EA */
  0x0000,   /* U+1D1EB */
  0x0000,   /* U+1D1EC */
  0x0000,   /* U+1D1ED */
  0x0000,   /* U+1D1EE */
  0x0000,   /* U+1D1EF */
  0x0000,   /* U+1D1F0 */
  0x0000,   /* U+1D1F1 */
  0x0000,   /* U+1D1F2 */
  0x0000,   /* U+1D1F3 */
  0x0000,   /* U+1D1F4 */
  0x0000,   /* U+1D1F5 */
  0x0000,   /* U+1D1F6 */
  0x0000,   /* U+1D1F7 */
  0x0000,   /* U+1D1F8 */
  0x0000,   /* U+1D1F9 */
  0x0000,   /* U+1D1FA */
  0x0000,   /* U+1D1FB */
  0x0000,   /* U+1D1FC */
  0x0000,   /* U+1D1FD */
  0x0000,   /* U+1D1FE */
  0x0000,   /* U+1D1FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0x0000,   /* U+1D127 */
  0x0000,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0x0000,   /* U+1D1E9 */
  0x0000,   /* U+1D1EA */
  0x0000,   /* U+1D1EB */
  0x0000,   /* U+1D1EC */
  0x0000,   /* U+1D1ED */
  0x0000,   /* U+1D1EE */
  0x0000,   /* U+1D1EF */
  0x0000,   /* U+1D1F0 */
  0x0000,   /* U+1D1F1 */
  0x0000,   /* U+1D1F2 */
  0x0000,   /* U+1D1F3 */
  0x0000,   /* U+1D1F4 */
  0x0000,   /* U+1D1F5 */
  0x0000,   /* U+1D1F6 */
  0x0000,   /* U+1D1F7 */
  0x0000,   /* U+1D1F8 */
  0x0000,   /* U+1D1F9 */
  0x0000,   /* U+1D1FA */
  0x0000,   /* U+1D1FB */
  0x0000,   /* U+1D1FC */
  0x0000,   /* U+1D1FD */
  0x0000,   /* U+1D1FE */
  0x0000,   /* U+1D1FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1D100 */
  0x0000,   /* U+1D101 */
  0x0000,   /* U+1D102 */
  0x0000,   /* U+1D103 */
  0x0000,   /* U+1D104 */
  0x0000,   /* U+1D105 */
  0x0000,   /* U+1D106 */
  0x0000,   /* U+1D107 */
  0x0000,   /* U+1D108 */
  0x0000,   /* U+1D109 */
  0x0000,   /* U+1D10A */
  0x0000,   /* U+1D10B */
  0x0000,   /* U+1D10C */
  0x0000,   /* U+1D10D */
  0x0000,   /* U+1D10E */
  0x0000,   /* U+1D10F */
  0x0000,   /* U+1D110 */
  0x0000,   /* U+1D111 */
  0x0000,   /* U+1D112 */
  0x0000,   /* U+1D113 */
  0x0000,   /* U+1D114 */
  0x0000,   /* U+1D115 */
  0x0000,   /* U+1D116 */
  0x0000,   /* U+1D117 */
  0x0000,   /* U+1D118 */
  0x0000,   /* U+1D119 */
  0x0000,   /* U+1D11A */
  0x0000,   /* U+1D11B */
  0x0000,   /* U+1D11C */
  0x0000,   /* U+1D11D */
  0x0000,   /* U+1D11E */
  0x0000,   /* U+1D11F */
  0x0000,   /* U+1D120 */
  0x0000,   /* U+1D121 */
  0x0000,   /* U+1D122 */
  0x0000,   /* U+1D123 */
  0x0000,   /* U+1D124 */
  0x0000,   /* U+1D125 */
  0x0000,   /* U+1D126 */
  0x0000,   /* U+1D127 */
  0x0000,   /* U+1D128 */
  0x0000,   /* U+1D129 */
  0x0000,   /* U+1D12A */
  0x0000,   /* U+1D12B */
  0x0000,   /* U+1D12C */
  0x0000,   /* U+1D12D */
  0x0000,   /* U+1D12E */
  0x0000,   /* U+1D12F */
  0x0000,   /* U+1D130 */
  0x0000,   /* U+1D131 */
  0x0000,   /* U+1D132 */
  0x0000,   /* U+1D133 */
  0x0000,   /* U+1D134 */
  0x0000,   /* U+1D135 */
  0x0000,   /* U+1D136 */
  0x0000,   /* U+1D137 */
  0x0000,   /* U+1D138 */
  0x0000,   /* U+1D139 */
  0x0000,   /* U+1D13A */
  0x0000,   /* U+1D13B */
  0x0000,   /* U+1D13C */
  0x0000,   /* U+1D13D */
  0x0000,   /* U+1D13E */
  0x0000,   /* U+1D13F */
  0x0000,   /* U+1D140 */
  0x0000,   /* U+1D141 */
  0x0000,   /* U+1D142 */
  0x0000,   /* U+1D143 */
  0x0000,   /* U+1D144 */
  0x0000,   /* U+1D145 */
  0x0000,   /* U+1D146 */
  0x0000,   /* U+1D147 */
  0x0000,   /* U+1D148 */
  0x0000,   /* U+1D149 */
  0x0000,   /* U+1D14A */
  0x0000,   /* U+1D14B */
  0x0000,   /* U+1D14C */
  0x0000,   /* U+1D14D */
  0x0000,   /* U+1D14E */
  0x0000,   /* U+1D14F */
  0x0000,   /* U+1D150 */
  0x0000,   /* U+1D151 */
  0x0000,   /* U+1D152 */
  0x0000,   /* U+1D153 */
  0x0000,   /* U+1D154 */
  0x0000,   /* U+1D155 */
  0x0000,   /* U+1D156 */
  0x0000,   /* U+1D157 */
  0x0000,   /* U+1D158 */
  0x0000,   /* U+1D159 */
  0x0000,   /* U+1D15A */
  0x0000,   /* U+1D15B */
  0x0000,   /* U+1D15C */
  0x0000,   /* U+1D15D */
  0x0000,   /* U+1D15E */
  0x0000,   /* U+1D15F */
  0x0000,   /* U+1D160 */
  0x0000,   /* U+1D161 */
  0x0000,   /* U+1D162 */
  0x0000,   /* U+1D163 */
  0x0000,   /* U+1D164 */
  0x0000,   /* U+1D165 */
  0x0000,   /* U+1D166 */
  0x0000,   /* U+1D167 */
  0x0000,   /* U+1D168 */
  0x0000,   /* U+1D169 */
  0x0000,   /* U+1D16A */
  0x0000,   /* U+1D16B */
  0x0000,   /* U+1D16C */
  0x0000,   /* U+1D16D */
  0x0000,   /* U+1D16E */
  0x0000,   /* U+1D16F */
  0x0000,   /* U+1D170 */
  0x0000,   /* U+1D171 */
  0x0000,   /* U+1D172 */
  0x0000,   /* U+1D173 */
  0x0000,   /* U+1D174 */
  0x0000,   /* U+1D175 */
  0x0000,   /* U+1D176 */
  0x0000,   /* U+1D177 */
  0x0000,   /* U+1D178 */
  0x0000,   /* U+1D179 */
  0x0000,   /* U+1D17A */
  0x0000,   /* U+1D17B */
  0x0000,   /* U+1D17C */
  0x0000,   /* U+1D17D */
  0x0000,   /* U+1D17E */
  0x0000,   /* U+1D17F */
  0x0000,   /* U+1D180 */
  0x0000,   /* U+1D181 */
  0x0000,   /* U+1D182 */
  0x0000,   /* U+1D183 */
  0x0000,   /* U+1D184 */
  0x0000,   /* U+1D185 */
  0x0000,   /* U+1D186 */
  0x0000,   /* U+1D187 */
  0x0000,   /* U+1D188 */
  0x0000,   /* U+1D189 */
  0x0000,   /* U+1D18A */
  0x0000,   /* U+1D18B */
  0x0000,   /* U+1D18C */
  0x0000,   /* U+1D18D */
  0x0000,   /* U+1D18E */
  0x0000,   /* U+1D18F */
  0x0000,   /* U+1D190 */
  0x0000,   /* U+1D191 */
  0x0000,   /* U+1D192 */
  0x0000,   /* U+1D193 */
  0x0000,   /* U+1D194 */
  0x0000,   /* U+1D195 */
  0x0000,   /* U+1D196 */
  0x0000,   /* U+1D197 */
  0x0000,   /* U+1D198 */
  0x0000,   /* U+1D199 */
  0x0000,   /* U+1D19A */
  0x0000,   /* U+1D19B */
  0x0000,   /* U+1D19C */
  0x0000,   /* U+1D19D */
  0x0000,   /* U+1D19E */
  0x0000,   /* U+1D19F */
  0x0000,   /* U+1D1A0 */
  0x0000,   /* U+1D1A1 */
  0x0000,   /* U+1D1A2 */
  0x0000,   /* U+1D1A3 */
  0x0000,   /* U+1D1A4 */
  0x0000,   /* U+1D1A5 */
  0x0000,   /* U+1D1A6 */
  0x0000,   /* U+1D1A7 */
  0x0000,   /* U+1D1A8 */
  0x0000,   /* U+1D1A9 */
  0x0000,   /* U+1D1AA */
  0x0000,   /* U+1D1AB */
  0x0000,   /* U+1D1AC */
  0x0000,   /* U+1D1AD */
  0x0000,   /* U+1D1AE */
  0x0000,   /* U+1D1AF */
  0x0000,   /* U+1D1B0 */
  0x0000,   /* U+1D1B1 */
  0x0000,   /* U+1D1B2 */
  0x0000,   /* U+1D1B3 */
  0x0000,   /* U+1D1B4 */
  0x0000,   /* U+1D1B5 */
  0x0000,   /* U+1D1B6 */
  0x0000,   /* U+1D1B7 */
  0x0000,   /* U+1D1B8 */
  0x0000,   /* U+1D1B9 */
  0x0000,   /* U+1D1BA */
  0x0000,   /* U+1D1BB */
  0x0000,   /* U+1D1BC */
  0x0000,   /* U+1D1BD */
  0x0000,   /* U+1D1BE */
  0x0000,   /* U+1D1BF */
  0x0000,   /* U+1D1C0 */
  0x0000,   /* U+1D1C1 */
  0x0000,   /* U+1D1C2 */
  0x0000,   /* U+1D1C3 */
  0x0000,   /* U+1D1C4 */
  0x0000,   /* U+1D1C5 */
  0x0000,   /* U+1D1C6 */
  0x0000,   /* U+1D1C7 */
  0x0000,   /* U+1D1C8 */
  0x0000,   /* U+1D1C9 */
  0x0000,   /* U+1D1CA */
  0x0000,   /* U+1D1CB */
  0x0000,   /* U+1D1CC */
  0x0000,   /* U+1D1CD */
  0x0000,   /* U+1D1CE */
  0x0000,   /* U+1D1CF */
  0x0000,   /* U+1D1D0 */
  0x0000,   /* U+1D1D1 */
  0x0000,   /* U+1D1D2 */
  0x0000,   /* U+1D1D3 */
  0x0000,   /* U+1D1D4 */
  0x0000,   /* U+1D1D5 */
  0x0000,   /* U+1D1D6 */
  0x0000,   /* U+1D1D7 */
  0x0000,   /* U+1D1D8 */
  0x0000,   /* U+1D1D9 */
  0x0000,   /* U+1D1DA */
  0x0000,   /* U+1D1DB */
  0x0000,   /* U+1D1DC */
  0x0000,   /* U+1D1DD */
  0x0000,   /* U+1D1DE */
  0x0000,   /* U+1D1DF */
  0x0000,   /* U+1D1E0 */
  0x0000,   /* U+1D1E1 */
  0x0000,   /* U+1D1E2 */
  0x0000,   /* U+1D1E3 */
  0x0000,   /* U+1D1E4 */
  0x0000,   /* U+1D1E5 */
  0x0000,   /* U+1D1E6 */
  0x0000,   /* U+1D1E7 */
  0x0000,   /* U+1D1E8 */
  0x0000,   /* U+1D1E9 */
  0x0000,   /* U+1D1EA */
  0x0000,   /* U+1D1EB */
  0x0000,   /* U+1D1EC */
  0x0000,   /* U+1D1ED */
  0x0000,   /* U+1D1EE */
  0x0000,   /* U+1D1EF */
  0x0000,   /* U+1D1F0 */
  0x0000,   /* U+1D1F1 */
  0x0000,   /* U+1D1F2 */
  0x0000,   /* U+1D1F3 */
  0x0000,   /* U+1D1F4 */
  0x0000,   /* U+1D1F5 */
  0x0000,   /* U+1D1F6 */
  0x0000,   /* U+1D1F7 */
  0x0000,   /* U+1D1F8 */
  0x0000,   /* U+1D1F9 */
  0x0000,   /* U+1D1FA */
  0x0000,   /* U+1D1FB */
  0x0000,   /* U+1D1FC */
  0x0000,   /* U+1D1FD */
  0x0000,   /* U+1D1FE */
  0x0000,   /* U+1D1FF */
};

uint16 uca900_p1D2[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x1183,   /* U+1D200 */
  0x1184,   /* U+1D201 */
  0x1185,   /* U+1D202 */
  0x1186,   /* U+1D203 */
  0x1187,   /* U+1D204 */
  0x1188,   /* U+1D205 */
  0x1189,   /* U+1D206 */
  0x118A,   /* U+1D207 */
  0x118B,   /* U+1D208 */
  0x118C,   /* U+1D209 */
  0x118D,   /* U+1D20A */
  0x118E,   /* U+1D20B */
  0x118F,   /* U+1D20C */
  0x1190,   /* U+1D20D */
  0x1191,   /* U+1D20E */
  0x1192,   /* U+1D20F */
  0x1193,   /* U+1D210 */
  0x1194,   /* U+1D211 */
  0x1195,   /* U+1D212 */
  0x1196,   /* U+1D213 */
  0x1197,   /* U+1D214 */
  0x1198,   /* U+1D215 */
  0x1199,   /* U+1D216 */
  0x119A,   /* U+1D217 */
  0x119B,   /* U+1D218 */
  0x119C,   /* U+1D219 */
  0x119D,   /* U+1D21A */
  0x119E,   /* U+1D21B */
  0x119F,   /* U+1D21C */
  0x11A0,   /* U+1D21D */
  0x11A1,   /* U+1D21E */
  0x11A2,   /* U+1D21F */
  0x11A3,   /* U+1D220 */
  0x11A4,   /* U+1D221 */
  0x11A5,   /* U+1D222 */
  0x11A6,   /* U+1D223 */
  0x11A7,   /* U+1D224 */
  0x11A8,   /* U+1D225 */
  0x11A9,   /* U+1D226 */
  0x11AA,   /* U+1D227 */
  0x11AB,   /* U+1D228 */
  0x11AC,   /* U+1D229 */
  0x11AD,   /* U+1D22A */
  0x11AE,   /* U+1D22B */
  0x11AF,   /* U+1D22C */
  0x11B0,   /* U+1D22D */
  0x11B1,   /* U+1D22E */
  0x11B2,   /* U+1D22F */
  0x11B3,   /* U+1D230 */
  0x11B4,   /* U+1D231 */
  0x11B5,   /* U+1D232 */
  0x11B6,   /* U+1D233 */
  0x11B7,   /* U+1D234 */
  0x11B8,   /* U+1D235 */
  0x11B9,   /* U+1D236 */
  0x11BA,   /* U+1D237 */
  0x11BB,   /* U+1D238 */
  0x11BC,   /* U+1D239 */
  0x11BD,   /* U+1D23A */
  0x11BE,   /* U+1D23B */
  0x11BF,   /* U+1D23C */
  0x11C0,   /* U+1D23D */
  0x11C1,   /* U+1D23E */
  0x11C2,   /* U+1D23F */
  0x11C3,   /* U+1D240 */
  0x11C4,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x11C5,   /* U+1D245 */
  0xFBC3,   /* U+1D246 */
  0xFBC3,   /* U+1D247 */
  0xFBC3,   /* U+1D248 */
  0xFBC3,   /* U+1D249 */
  0xFBC3,   /* U+1D24A */
  0xFBC3,   /* U+1D24B */
  0xFBC3,   /* U+1D24C */
  0xFBC3,   /* U+1D24D */
  0xFBC3,   /* U+1D24E */
  0xFBC3,   /* U+1D24F */
  0xFBC3,   /* U+1D250 */
  0xFBC3,   /* U+1D251 */
  0xFBC3,   /* U+1D252 */
  0xFBC3,   /* U+1D253 */
  0xFBC3,   /* U+1D254 */
  0xFBC3,   /* U+1D255 */
  0xFBC3,   /* U+1D256 */
  0xFBC3,   /* U+1D257 */
  0xFBC3,   /* U+1D258 */
  0xFBC3,   /* U+1D259 */
  0xFBC3,   /* U+1D25A */
  0xFBC3,   /* U+1D25B */
  0xFBC3,   /* U+1D25C */
  0xFBC3,   /* U+1D25D */
  0xFBC3,   /* U+1D25E */
  0xFBC3,   /* U+1D25F */
  0xFBC3,   /* U+1D260 */
  0xFBC3,   /* U+1D261 */
  0xFBC3,   /* U+1D262 */
  0xFBC3,   /* U+1D263 */
  0xFBC3,   /* U+1D264 */
  0xFBC3,   /* U+1D265 */
  0xFBC3,   /* U+1D266 */
  0xFBC3,   /* U+1D267 */
  0xFBC3,   /* U+1D268 */
  0xFBC3,   /* U+1D269 */
  0xFBC3,   /* U+1D26A */
  0xFBC3,   /* U+1D26B */
  0xFBC3,   /* U+1D26C */
  0xFBC3,   /* U+1D26D */
  0xFBC3,   /* U+1D26E */
  0xFBC3,   /* U+1D26F */
  0xFBC3,   /* U+1D270 */
  0xFBC3,   /* U+1D271 */
  0xFBC3,   /* U+1D272 */
  0xFBC3,   /* U+1D273 */
  0xFBC3,   /* U+1D274 */
  0xFBC3,   /* U+1D275 */
  0xFBC3,   /* U+1D276 */
  0xFBC3,   /* U+1D277 */
  0xFBC3,   /* U+1D278 */
  0xFBC3,   /* U+1D279 */
  0xFBC3,   /* U+1D27A */
  0xFBC3,   /* U+1D27B */
  0xFBC3,   /* U+1D27C */
  0xFBC3,   /* U+1D27D */
  0xFBC3,   /* U+1D27E */
  0xFBC3,   /* U+1D27F */
  0xFBC3,   /* U+1D280 */
  0xFBC3,   /* U+1D281 */
  0xFBC3,   /* U+1D282 */
  0xFBC3,   /* U+1D283 */
  0xFBC3,   /* U+1D284 */
  0xFBC3,   /* U+1D285 */
  0xFBC3,   /* U+1D286 */
  0xFBC3,   /* U+1D287 */
  0xFBC3,   /* U+1D288 */
  0xFBC3,   /* U+1D289 */
  0xFBC3,   /* U+1D28A */
  0xFBC3,   /* U+1D28B */
  0xFBC3,   /* U+1D28C */
  0xFBC3,   /* U+1D28D */
  0xFBC3,   /* U+1D28E */
  0xFBC3,   /* U+1D28F */
  0xFBC3,   /* U+1D290 */
  0xFBC3,   /* U+1D291 */
  0xFBC3,   /* U+1D292 */
  0xFBC3,   /* U+1D293 */
  0xFBC3,   /* U+1D294 */
  0xFBC3,   /* U+1D295 */
  0xFBC3,   /* U+1D296 */
  0xFBC3,   /* U+1D297 */
  0xFBC3,   /* U+1D298 */
  0xFBC3,   /* U+1D299 */
  0xFBC3,   /* U+1D29A */
  0xFBC3,   /* U+1D29B */
  0xFBC3,   /* U+1D29C */
  0xFBC3,   /* U+1D29D */
  0xFBC3,   /* U+1D29E */
  0xFBC3,   /* U+1D29F */
  0xFBC3,   /* U+1D2A0 */
  0xFBC3,   /* U+1D2A1 */
  0xFBC3,   /* U+1D2A2 */
  0xFBC3,   /* U+1D2A3 */
  0xFBC3,   /* U+1D2A4 */
  0xFBC3,   /* U+1D2A5 */
  0xFBC3,   /* U+1D2A6 */
  0xFBC3,   /* U+1D2A7 */
  0xFBC3,   /* U+1D2A8 */
  0xFBC3,   /* U+1D2A9 */
  0xFBC3,   /* U+1D2AA */
  0xFBC3,   /* U+1D2AB */
  0xFBC3,   /* U+1D2AC */
  0xFBC3,   /* U+1D2AD */
  0xFBC3,   /* U+1D2AE */
  0xFBC3,   /* U+1D2AF */
  0xFBC3,   /* U+1D2B0 */
  0xFBC3,   /* U+1D2B1 */
  0xFBC3,   /* U+1D2B2 */
  0xFBC3,   /* U+1D2B3 */
  0xFBC3,   /* U+1D2B4 */
  0xFBC3,   /* U+1D2B5 */
  0xFBC3,   /* U+1D2B6 */
  0xFBC3,   /* U+1D2B7 */
  0xFBC3,   /* U+1D2B8 */
  0xFBC3,   /* U+1D2B9 */
  0xFBC3,   /* U+1D2BA */
  0xFBC3,   /* U+1D2BB */
  0xFBC3,   /* U+1D2BC */
  0xFBC3,   /* U+1D2BD */
  0xFBC3,   /* U+1D2BE */
  0xFBC3,   /* U+1D2BF */
  0xFBC3,   /* U+1D2C0 */
  0xFBC3,   /* U+1D2C1 */
  0xFBC3,   /* U+1D2C2 */
  0xFBC3,   /* U+1D2C3 */
  0xFBC3,   /* U+1D2C4 */
  0xFBC3,   /* U+1D2C5 */
  0xFBC3,   /* U+1D2C6 */
  0xFBC3,   /* U+1D2C7 */
  0xFBC3,   /* U+1D2C8 */
  0xFBC3,   /* U+1D2C9 */
  0xFBC3,   /* U+1D2CA */
  0xFBC3,   /* U+1D2CB */
  0xFBC3,   /* U+1D2CC */
  0xFBC3,   /* U+1D2CD */
  0xFBC3,   /* U+1D2CE */
  0xFBC3,   /* U+1D2CF */
  0xFBC3,   /* U+1D2D0 */
  0xFBC3,   /* U+1D2D1 */
  0xFBC3,   /* U+1D2D2 */
  0xFBC3,   /* U+1D2D3 */
  0xFBC3,   /* U+1D2D4 */
  0xFBC3,   /* U+1D2D5 */
  0xFBC3,   /* U+1D2D6 */
  0xFBC3,   /* U+1D2D7 */
  0xFBC3,   /* U+1D2D8 */
  0xFBC3,   /* U+1D2D9 */
  0xFBC3,   /* U+1D2DA */
  0xFBC3,   /* U+1D2DB */
  0xFBC3,   /* U+1D2DC */
  0xFBC3,   /* U+1D2DD */
  0xFBC3,   /* U+1D2DE */
  0xFBC3,   /* U+1D2DF */
  0xFBC3,   /* U+1D2E0 */
  0xFBC3,   /* U+1D2E1 */
  0xFBC3,   /* U+1D2E2 */
  0xFBC3,   /* U+1D2E3 */
  0xFBC3,   /* U+1D2E4 */
  0xFBC3,   /* U+1D2E5 */
  0xFBC3,   /* U+1D2E6 */
  0xFBC3,   /* U+1D2E7 */
  0xFBC3,   /* U+1D2E8 */
  0xFBC3,   /* U+1D2E9 */
  0xFBC3,   /* U+1D2EA */
  0xFBC3,   /* U+1D2EB */
  0xFBC3,   /* U+1D2EC */
  0xFBC3,   /* U+1D2ED */
  0xFBC3,   /* U+1D2EE */
  0xFBC3,   /* U+1D2EF */
  0xFBC3,   /* U+1D2F0 */
  0xFBC3,   /* U+1D2F1 */
  0xFBC3,   /* U+1D2F2 */
  0xFBC3,   /* U+1D2F3 */
  0xFBC3,   /* U+1D2F4 */
  0xFBC3,   /* U+1D2F5 */
  0xFBC3,   /* U+1D2F6 */
  0xFBC3,   /* U+1D2F7 */
  0xFBC3,   /* U+1D2F8 */
  0xFBC3,   /* U+1D2F9 */
  0xFBC3,   /* U+1D2FA */
  0xFBC3,   /* U+1D2FB */
  0xFBC3,   /* U+1D2FC */
  0xFBC3,   /* U+1D2FD */
  0xFBC3,   /* U+1D2FE */
  0xFBC3,   /* U+1D2FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D200 */
  0x0020,   /* U+1D201 */
  0x0020,   /* U+1D202 */
  0x0020,   /* U+1D203 */
  0x0020,   /* U+1D204 */
  0x0020,   /* U+1D205 */
  0x0020,   /* U+1D206 */
  0x0020,   /* U+1D207 */
  0x0020,   /* U+1D208 */
  0x0020,   /* U+1D209 */
  0x0020,   /* U+1D20A */
  0x0020,   /* U+1D20B */
  0x0020,   /* U+1D20C */
  0x0020,   /* U+1D20D */
  0x0020,   /* U+1D20E */
  0x0020,   /* U+1D20F */
  0x0020,   /* U+1D210 */
  0x0020,   /* U+1D211 */
  0x0020,   /* U+1D212 */
  0x0020,   /* U+1D213 */
  0x0020,   /* U+1D214 */
  0x0020,   /* U+1D215 */
  0x0020,   /* U+1D216 */
  0x0020,   /* U+1D217 */
  0x0020,   /* U+1D218 */
  0x0020,   /* U+1D219 */
  0x0020,   /* U+1D21A */
  0x0020,   /* U+1D21B */
  0x0020,   /* U+1D21C */
  0x0020,   /* U+1D21D */
  0x0020,   /* U+1D21E */
  0x0020,   /* U+1D21F */
  0x0020,   /* U+1D220 */
  0x0020,   /* U+1D221 */
  0x0020,   /* U+1D222 */
  0x0020,   /* U+1D223 */
  0x0020,   /* U+1D224 */
  0x0020,   /* U+1D225 */
  0x0020,   /* U+1D226 */
  0x0020,   /* U+1D227 */
  0x0020,   /* U+1D228 */
  0x0020,   /* U+1D229 */
  0x0020,   /* U+1D22A */
  0x0020,   /* U+1D22B */
  0x0020,   /* U+1D22C */
  0x0020,   /* U+1D22D */
  0x0020,   /* U+1D22E */
  0x0020,   /* U+1D22F */
  0x0020,   /* U+1D230 */
  0x0020,   /* U+1D231 */
  0x0020,   /* U+1D232 */
  0x0020,   /* U+1D233 */
  0x0020,   /* U+1D234 */
  0x0020,   /* U+1D235 */
  0x0020,   /* U+1D236 */
  0x0020,   /* U+1D237 */
  0x0020,   /* U+1D238 */
  0x0020,   /* U+1D239 */
  0x0020,   /* U+1D23A */
  0x0020,   /* U+1D23B */
  0x0020,   /* U+1D23C */
  0x0020,   /* U+1D23D */
  0x0020,   /* U+1D23E */
  0x0020,   /* U+1D23F */
  0x0020,   /* U+1D240 */
  0x0020,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x0020,   /* U+1D245 */
  0x0020,   /* U+1D246 */
  0x0020,   /* U+1D247 */
  0x0020,   /* U+1D248 */
  0x0020,   /* U+1D249 */
  0x0020,   /* U+1D24A */
  0x0020,   /* U+1D24B */
  0x0020,   /* U+1D24C */
  0x0020,   /* U+1D24D */
  0x0020,   /* U+1D24E */
  0x0020,   /* U+1D24F */
  0x0020,   /* U+1D250 */
  0x0020,   /* U+1D251 */
  0x0020,   /* U+1D252 */
  0x0020,   /* U+1D253 */
  0x0020,   /* U+1D254 */
  0x0020,   /* U+1D255 */
  0x0020,   /* U+1D256 */
  0x0020,   /* U+1D257 */
  0x0020,   /* U+1D258 */
  0x0020,   /* U+1D259 */
  0x0020,   /* U+1D25A */
  0x0020,   /* U+1D25B */
  0x0020,   /* U+1D25C */
  0x0020,   /* U+1D25D */
  0x0020,   /* U+1D25E */
  0x0020,   /* U+1D25F */
  0x0020,   /* U+1D260 */
  0x0020,   /* U+1D261 */
  0x0020,   /* U+1D262 */
  0x0020,   /* U+1D263 */
  0x0020,   /* U+1D264 */
  0x0020,   /* U+1D265 */
  0x0020,   /* U+1D266 */
  0x0020,   /* U+1D267 */
  0x0020,   /* U+1D268 */
  0x0020,   /* U+1D269 */
  0x0020,   /* U+1D26A */
  0x0020,   /* U+1D26B */
  0x0020,   /* U+1D26C */
  0x0020,   /* U+1D26D */
  0x0020,   /* U+1D26E */
  0x0020,   /* U+1D26F */
  0x0020,   /* U+1D270 */
  0x0020,   /* U+1D271 */
  0x0020,   /* U+1D272 */
  0x0020,   /* U+1D273 */
  0x0020,   /* U+1D274 */
  0x0020,   /* U+1D275 */
  0x0020,   /* U+1D276 */
  0x0020,   /* U+1D277 */
  0x0020,   /* U+1D278 */
  0x0020,   /* U+1D279 */
  0x0020,   /* U+1D27A */
  0x0020,   /* U+1D27B */
  0x0020,   /* U+1D27C */
  0x0020,   /* U+1D27D */
  0x0020,   /* U+1D27E */
  0x0020,   /* U+1D27F */
  0x0020,   /* U+1D280 */
  0x0020,   /* U+1D281 */
  0x0020,   /* U+1D282 */
  0x0020,   /* U+1D283 */
  0x0020,   /* U+1D284 */
  0x0020,   /* U+1D285 */
  0x0020,   /* U+1D286 */
  0x0020,   /* U+1D287 */
  0x0020,   /* U+1D288 */
  0x0020,   /* U+1D289 */
  0x0020,   /* U+1D28A */
  0x0020,   /* U+1D28B */
  0x0020,   /* U+1D28C */
  0x0020,   /* U+1D28D */
  0x0020,   /* U+1D28E */
  0x0020,   /* U+1D28F */
  0x0020,   /* U+1D290 */
  0x0020,   /* U+1D291 */
  0x0020,   /* U+1D292 */
  0x0020,   /* U+1D293 */
  0x0020,   /* U+1D294 */
  0x0020,   /* U+1D295 */
  0x0020,   /* U+1D296 */
  0x0020,   /* U+1D297 */
  0x0020,   /* U+1D298 */
  0x0020,   /* U+1D299 */
  0x0020,   /* U+1D29A */
  0x0020,   /* U+1D29B */
  0x0020,   /* U+1D29C */
  0x0020,   /* U+1D29D */
  0x0020,   /* U+1D29E */
  0x0020,   /* U+1D29F */
  0x0020,   /* U+1D2A0 */
  0x0020,   /* U+1D2A1 */
  0x0020,   /* U+1D2A2 */
  0x0020,   /* U+1D2A3 */
  0x0020,   /* U+1D2A4 */
  0x0020,   /* U+1D2A5 */
  0x0020,   /* U+1D2A6 */
  0x0020,   /* U+1D2A7 */
  0x0020,   /* U+1D2A8 */
  0x0020,   /* U+1D2A9 */
  0x0020,   /* U+1D2AA */
  0x0020,   /* U+1D2AB */
  0x0020,   /* U+1D2AC */
  0x0020,   /* U+1D2AD */
  0x0020,   /* U+1D2AE */
  0x0020,   /* U+1D2AF */
  0x0020,   /* U+1D2B0 */
  0x0020,   /* U+1D2B1 */
  0x0020,   /* U+1D2B2 */
  0x0020,   /* U+1D2B3 */
  0x0020,   /* U+1D2B4 */
  0x0020,   /* U+1D2B5 */
  0x0020,   /* U+1D2B6 */
  0x0020,   /* U+1D2B7 */
  0x0020,   /* U+1D2B8 */
  0x0020,   /* U+1D2B9 */
  0x0020,   /* U+1D2BA */
  0x0020,   /* U+1D2BB */
  0x0020,   /* U+1D2BC */
  0x0020,   /* U+1D2BD */
  0x0020,   /* U+1D2BE */
  0x0020,   /* U+1D2BF */
  0x0020,   /* U+1D2C0 */
  0x0020,   /* U+1D2C1 */
  0x0020,   /* U+1D2C2 */
  0x0020,   /* U+1D2C3 */
  0x0020,   /* U+1D2C4 */
  0x0020,   /* U+1D2C5 */
  0x0020,   /* U+1D2C6 */
  0x0020,   /* U+1D2C7 */
  0x0020,   /* U+1D2C8 */
  0x0020,   /* U+1D2C9 */
  0x0020,   /* U+1D2CA */
  0x0020,   /* U+1D2CB */
  0x0020,   /* U+1D2CC */
  0x0020,   /* U+1D2CD */
  0x0020,   /* U+1D2CE */
  0x0020,   /* U+1D2CF */
  0x0020,   /* U+1D2D0 */
  0x0020,   /* U+1D2D1 */
  0x0020,   /* U+1D2D2 */
  0x0020,   /* U+1D2D3 */
  0x0020,   /* U+1D2D4 */
  0x0020,   /* U+1D2D5 */
  0x0020,   /* U+1D2D6 */
  0x0020,   /* U+1D2D7 */
  0x0020,   /* U+1D2D8 */
  0x0020,   /* U+1D2D9 */
  0x0020,   /* U+1D2DA */
  0x0020,   /* U+1D2DB */
  0x0020,   /* U+1D2DC */
  0x0020,   /* U+1D2DD */
  0x0020,   /* U+1D2DE */
  0x0020,   /* U+1D2DF */
  0x0020,   /* U+1D2E0 */
  0x0020,   /* U+1D2E1 */
  0x0020,   /* U+1D2E2 */
  0x0020,   /* U+1D2E3 */
  0x0020,   /* U+1D2E4 */
  0x0020,   /* U+1D2E5 */
  0x0020,   /* U+1D2E6 */
  0x0020,   /* U+1D2E7 */
  0x0020,   /* U+1D2E8 */
  0x0020,   /* U+1D2E9 */
  0x0020,   /* U+1D2EA */
  0x0020,   /* U+1D2EB */
  0x0020,   /* U+1D2EC */
  0x0020,   /* U+1D2ED */
  0x0020,   /* U+1D2EE */
  0x0020,   /* U+1D2EF */
  0x0020,   /* U+1D2F0 */
  0x0020,   /* U+1D2F1 */
  0x0020,   /* U+1D2F2 */
  0x0020,   /* U+1D2F3 */
  0x0020,   /* U+1D2F4 */
  0x0020,   /* U+1D2F5 */
  0x0020,   /* U+1D2F6 */
  0x0020,   /* U+1D2F7 */
  0x0020,   /* U+1D2F8 */
  0x0020,   /* U+1D2F9 */
  0x0020,   /* U+1D2FA */
  0x0020,   /* U+1D2FB */
  0x0020,   /* U+1D2FC */
  0x0020,   /* U+1D2FD */
  0x0020,   /* U+1D2FE */
  0x0020,   /* U+1D2FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D200 */
  0x0002,   /* U+1D201 */
  0x0002,   /* U+1D202 */
  0x0002,   /* U+1D203 */
  0x0002,   /* U+1D204 */
  0x0002,   /* U+1D205 */
  0x0002,   /* U+1D206 */
  0x0002,   /* U+1D207 */
  0x0002,   /* U+1D208 */
  0x0002,   /* U+1D209 */
  0x0002,   /* U+1D20A */
  0x0002,   /* U+1D20B */
  0x0002,   /* U+1D20C */
  0x0002,   /* U+1D20D */
  0x0002,   /* U+1D20E */
  0x0002,   /* U+1D20F */
  0x0002,   /* U+1D210 */
  0x0002,   /* U+1D211 */
  0x0002,   /* U+1D212 */
  0x0002,   /* U+1D213 */
  0x0002,   /* U+1D214 */
  0x0002,   /* U+1D215 */
  0x0002,   /* U+1D216 */
  0x0002,   /* U+1D217 */
  0x0002,   /* U+1D218 */
  0x0002,   /* U+1D219 */
  0x0002,   /* U+1D21A */
  0x0002,   /* U+1D21B */
  0x0002,   /* U+1D21C */
  0x0002,   /* U+1D21D */
  0x0002,   /* U+1D21E */
  0x0002,   /* U+1D21F */
  0x0002,   /* U+1D220 */
  0x0002,   /* U+1D221 */
  0x0002,   /* U+1D222 */
  0x0002,   /* U+1D223 */
  0x0002,   /* U+1D224 */
  0x0002,   /* U+1D225 */
  0x0002,   /* U+1D226 */
  0x0002,   /* U+1D227 */
  0x0002,   /* U+1D228 */
  0x0002,   /* U+1D229 */
  0x0002,   /* U+1D22A */
  0x0002,   /* U+1D22B */
  0x0002,   /* U+1D22C */
  0x0002,   /* U+1D22D */
  0x0002,   /* U+1D22E */
  0x0002,   /* U+1D22F */
  0x0002,   /* U+1D230 */
  0x0002,   /* U+1D231 */
  0x0002,   /* U+1D232 */
  0x0002,   /* U+1D233 */
  0x0002,   /* U+1D234 */
  0x0002,   /* U+1D235 */
  0x0002,   /* U+1D236 */
  0x0002,   /* U+1D237 */
  0x0002,   /* U+1D238 */
  0x0002,   /* U+1D239 */
  0x0002,   /* U+1D23A */
  0x0002,   /* U+1D23B */
  0x0002,   /* U+1D23C */
  0x0002,   /* U+1D23D */
  0x0002,   /* U+1D23E */
  0x0002,   /* U+1D23F */
  0x0002,   /* U+1D240 */
  0x0002,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x0002,   /* U+1D245 */
  0x0002,   /* U+1D246 */
  0x0002,   /* U+1D247 */
  0x0002,   /* U+1D248 */
  0x0002,   /* U+1D249 */
  0x0002,   /* U+1D24A */
  0x0002,   /* U+1D24B */
  0x0002,   /* U+1D24C */
  0x0002,   /* U+1D24D */
  0x0002,   /* U+1D24E */
  0x0002,   /* U+1D24F */
  0x0002,   /* U+1D250 */
  0x0002,   /* U+1D251 */
  0x0002,   /* U+1D252 */
  0x0002,   /* U+1D253 */
  0x0002,   /* U+1D254 */
  0x0002,   /* U+1D255 */
  0x0002,   /* U+1D256 */
  0x0002,   /* U+1D257 */
  0x0002,   /* U+1D258 */
  0x0002,   /* U+1D259 */
  0x0002,   /* U+1D25A */
  0x0002,   /* U+1D25B */
  0x0002,   /* U+1D25C */
  0x0002,   /* U+1D25D */
  0x0002,   /* U+1D25E */
  0x0002,   /* U+1D25F */
  0x0002,   /* U+1D260 */
  0x0002,   /* U+1D261 */
  0x0002,   /* U+1D262 */
  0x0002,   /* U+1D263 */
  0x0002,   /* U+1D264 */
  0x0002,   /* U+1D265 */
  0x0002,   /* U+1D266 */
  0x0002,   /* U+1D267 */
  0x0002,   /* U+1D268 */
  0x0002,   /* U+1D269 */
  0x0002,   /* U+1D26A */
  0x0002,   /* U+1D26B */
  0x0002,   /* U+1D26C */
  0x0002,   /* U+1D26D */
  0x0002,   /* U+1D26E */
  0x0002,   /* U+1D26F */
  0x0002,   /* U+1D270 */
  0x0002,   /* U+1D271 */
  0x0002,   /* U+1D272 */
  0x0002,   /* U+1D273 */
  0x0002,   /* U+1D274 */
  0x0002,   /* U+1D275 */
  0x0002,   /* U+1D276 */
  0x0002,   /* U+1D277 */
  0x0002,   /* U+1D278 */
  0x0002,   /* U+1D279 */
  0x0002,   /* U+1D27A */
  0x0002,   /* U+1D27B */
  0x0002,   /* U+1D27C */
  0x0002,   /* U+1D27D */
  0x0002,   /* U+1D27E */
  0x0002,   /* U+1D27F */
  0x0002,   /* U+1D280 */
  0x0002,   /* U+1D281 */
  0x0002,   /* U+1D282 */
  0x0002,   /* U+1D283 */
  0x0002,   /* U+1D284 */
  0x0002,   /* U+1D285 */
  0x0002,   /* U+1D286 */
  0x0002,   /* U+1D287 */
  0x0002,   /* U+1D288 */
  0x0002,   /* U+1D289 */
  0x0002,   /* U+1D28A */
  0x0002,   /* U+1D28B */
  0x0002,   /* U+1D28C */
  0x0002,   /* U+1D28D */
  0x0002,   /* U+1D28E */
  0x0002,   /* U+1D28F */
  0x0002,   /* U+1D290 */
  0x0002,   /* U+1D291 */
  0x0002,   /* U+1D292 */
  0x0002,   /* U+1D293 */
  0x0002,   /* U+1D294 */
  0x0002,   /* U+1D295 */
  0x0002,   /* U+1D296 */
  0x0002,   /* U+1D297 */
  0x0002,   /* U+1D298 */
  0x0002,   /* U+1D299 */
  0x0002,   /* U+1D29A */
  0x0002,   /* U+1D29B */
  0x0002,   /* U+1D29C */
  0x0002,   /* U+1D29D */
  0x0002,   /* U+1D29E */
  0x0002,   /* U+1D29F */
  0x0002,   /* U+1D2A0 */
  0x0002,   /* U+1D2A1 */
  0x0002,   /* U+1D2A2 */
  0x0002,   /* U+1D2A3 */
  0x0002,   /* U+1D2A4 */
  0x0002,   /* U+1D2A5 */
  0x0002,   /* U+1D2A6 */
  0x0002,   /* U+1D2A7 */
  0x0002,   /* U+1D2A8 */
  0x0002,   /* U+1D2A9 */
  0x0002,   /* U+1D2AA */
  0x0002,   /* U+1D2AB */
  0x0002,   /* U+1D2AC */
  0x0002,   /* U+1D2AD */
  0x0002,   /* U+1D2AE */
  0x0002,   /* U+1D2AF */
  0x0002,   /* U+1D2B0 */
  0x0002,   /* U+1D2B1 */
  0x0002,   /* U+1D2B2 */
  0x0002,   /* U+1D2B3 */
  0x0002,   /* U+1D2B4 */
  0x0002,   /* U+1D2B5 */
  0x0002,   /* U+1D2B6 */
  0x0002,   /* U+1D2B7 */
  0x0002,   /* U+1D2B8 */
  0x0002,   /* U+1D2B9 */
  0x0002,   /* U+1D2BA */
  0x0002,   /* U+1D2BB */
  0x0002,   /* U+1D2BC */
  0x0002,   /* U+1D2BD */
  0x0002,   /* U+1D2BE */
  0x0002,   /* U+1D2BF */
  0x0002,   /* U+1D2C0 */
  0x0002,   /* U+1D2C1 */
  0x0002,   /* U+1D2C2 */
  0x0002,   /* U+1D2C3 */
  0x0002,   /* U+1D2C4 */
  0x0002,   /* U+1D2C5 */
  0x0002,   /* U+1D2C6 */
  0x0002,   /* U+1D2C7 */
  0x0002,   /* U+1D2C8 */
  0x0002,   /* U+1D2C9 */
  0x0002,   /* U+1D2CA */
  0x0002,   /* U+1D2CB */
  0x0002,   /* U+1D2CC */
  0x0002,   /* U+1D2CD */
  0x0002,   /* U+1D2CE */
  0x0002,   /* U+1D2CF */
  0x0002,   /* U+1D2D0 */
  0x0002,   /* U+1D2D1 */
  0x0002,   /* U+1D2D2 */
  0x0002,   /* U+1D2D3 */
  0x0002,   /* U+1D2D4 */
  0x0002,   /* U+1D2D5 */
  0x0002,   /* U+1D2D6 */
  0x0002,   /* U+1D2D7 */
  0x0002,   /* U+1D2D8 */
  0x0002,   /* U+1D2D9 */
  0x0002,   /* U+1D2DA */
  0x0002,   /* U+1D2DB */
  0x0002,   /* U+1D2DC */
  0x0002,   /* U+1D2DD */
  0x0002,   /* U+1D2DE */
  0x0002,   /* U+1D2DF */
  0x0002,   /* U+1D2E0 */
  0x0002,   /* U+1D2E1 */
  0x0002,   /* U+1D2E2 */
  0x0002,   /* U+1D2E3 */
  0x0002,   /* U+1D2E4 */
  0x0002,   /* U+1D2E5 */
  0x0002,   /* U+1D2E6 */
  0x0002,   /* U+1D2E7 */
  0x0002,   /* U+1D2E8 */
  0x0002,   /* U+1D2E9 */
  0x0002,   /* U+1D2EA */
  0x0002,   /* U+1D2EB */
  0x0002,   /* U+1D2EC */
  0x0002,   /* U+1D2ED */
  0x0002,   /* U+1D2EE */
  0x0002,   /* U+1D2EF */
  0x0002,   /* U+1D2F0 */
  0x0002,   /* U+1D2F1 */
  0x0002,   /* U+1D2F2 */
  0x0002,   /* U+1D2F3 */
  0x0002,   /* U+1D2F4 */
  0x0002,   /* U+1D2F5 */
  0x0002,   /* U+1D2F6 */
  0x0002,   /* U+1D2F7 */
  0x0002,   /* U+1D2F8 */
  0x0002,   /* U+1D2F9 */
  0x0002,   /* U+1D2FA */
  0x0002,   /* U+1D2FB */
  0x0002,   /* U+1D2FC */
  0x0002,   /* U+1D2FD */
  0x0002,   /* U+1D2FE */
  0x0002,   /* U+1D2FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D200 */
  0x0000,   /* U+1D201 */
  0x0000,   /* U+1D202 */
  0x0000,   /* U+1D203 */
  0x0000,   /* U+1D204 */
  0x0000,   /* U+1D205 */
  0x0000,   /* U+1D206 */
  0x0000,   /* U+1D207 */
  0x0000,   /* U+1D208 */
  0x0000,   /* U+1D209 */
  0x0000,   /* U+1D20A */
  0x0000,   /* U+1D20B */
  0x0000,   /* U+1D20C */
  0x0000,   /* U+1D20D */
  0x0000,   /* U+1D20E */
  0x0000,   /* U+1D20F */
  0x0000,   /* U+1D210 */
  0x0000,   /* U+1D211 */
  0x0000,   /* U+1D212 */
  0x0000,   /* U+1D213 */
  0x0000,   /* U+1D214 */
  0x0000,   /* U+1D215 */
  0x0000,   /* U+1D216 */
  0x0000,   /* U+1D217 */
  0x0000,   /* U+1D218 */
  0x0000,   /* U+1D219 */
  0x0000,   /* U+1D21A */
  0x0000,   /* U+1D21B */
  0x0000,   /* U+1D21C */
  0x0000,   /* U+1D21D */
  0x0000,   /* U+1D21E */
  0x0000,   /* U+1D21F */
  0x0000,   /* U+1D220 */
  0x0000,   /* U+1D221 */
  0x0000,   /* U+1D222 */
  0x0000,   /* U+1D223 */
  0x0000,   /* U+1D224 */
  0x0000,   /* U+1D225 */
  0x0000,   /* U+1D226 */
  0x0000,   /* U+1D227 */
  0x0000,   /* U+1D228 */
  0x0000,   /* U+1D229 */
  0x0000,   /* U+1D22A */
  0x0000,   /* U+1D22B */
  0x0000,   /* U+1D22C */
  0x0000,   /* U+1D22D */
  0x0000,   /* U+1D22E */
  0x0000,   /* U+1D22F */
  0x0000,   /* U+1D230 */
  0x0000,   /* U+1D231 */
  0x0000,   /* U+1D232 */
  0x0000,   /* U+1D233 */
  0x0000,   /* U+1D234 */
  0x0000,   /* U+1D235 */
  0x0000,   /* U+1D236 */
  0x0000,   /* U+1D237 */
  0x0000,   /* U+1D238 */
  0x0000,   /* U+1D239 */
  0x0000,   /* U+1D23A */
  0x0000,   /* U+1D23B */
  0x0000,   /* U+1D23C */
  0x0000,   /* U+1D23D */
  0x0000,   /* U+1D23E */
  0x0000,   /* U+1D23F */
  0x0000,   /* U+1D240 */
  0x0000,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x0000,   /* U+1D245 */
  0xD246,   /* U+1D246 */
  0xD247,   /* U+1D247 */
  0xD248,   /* U+1D248 */
  0xD249,   /* U+1D249 */
  0xD24A,   /* U+1D24A */
  0xD24B,   /* U+1D24B */
  0xD24C,   /* U+1D24C */
  0xD24D,   /* U+1D24D */
  0xD24E,   /* U+1D24E */
  0xD24F,   /* U+1D24F */
  0xD250,   /* U+1D250 */
  0xD251,   /* U+1D251 */
  0xD252,   /* U+1D252 */
  0xD253,   /* U+1D253 */
  0xD254,   /* U+1D254 */
  0xD255,   /* U+1D255 */
  0xD256,   /* U+1D256 */
  0xD257,   /* U+1D257 */
  0xD258,   /* U+1D258 */
  0xD259,   /* U+1D259 */
  0xD25A,   /* U+1D25A */
  0xD25B,   /* U+1D25B */
  0xD25C,   /* U+1D25C */
  0xD25D,   /* U+1D25D */
  0xD25E,   /* U+1D25E */
  0xD25F,   /* U+1D25F */
  0xD260,   /* U+1D260 */
  0xD261,   /* U+1D261 */
  0xD262,   /* U+1D262 */
  0xD263,   /* U+1D263 */
  0xD264,   /* U+1D264 */
  0xD265,   /* U+1D265 */
  0xD266,   /* U+1D266 */
  0xD267,   /* U+1D267 */
  0xD268,   /* U+1D268 */
  0xD269,   /* U+1D269 */
  0xD26A,   /* U+1D26A */
  0xD26B,   /* U+1D26B */
  0xD26C,   /* U+1D26C */
  0xD26D,   /* U+1D26D */
  0xD26E,   /* U+1D26E */
  0xD26F,   /* U+1D26F */
  0xD270,   /* U+1D270 */
  0xD271,   /* U+1D271 */
  0xD272,   /* U+1D272 */
  0xD273,   /* U+1D273 */
  0xD274,   /* U+1D274 */
  0xD275,   /* U+1D275 */
  0xD276,   /* U+1D276 */
  0xD277,   /* U+1D277 */
  0xD278,   /* U+1D278 */
  0xD279,   /* U+1D279 */
  0xD27A,   /* U+1D27A */
  0xD27B,   /* U+1D27B */
  0xD27C,   /* U+1D27C */
  0xD27D,   /* U+1D27D */
  0xD27E,   /* U+1D27E */
  0xD27F,   /* U+1D27F */
  0xD280,   /* U+1D280 */
  0xD281,   /* U+1D281 */
  0xD282,   /* U+1D282 */
  0xD283,   /* U+1D283 */
  0xD284,   /* U+1D284 */
  0xD285,   /* U+1D285 */
  0xD286,   /* U+1D286 */
  0xD287,   /* U+1D287 */
  0xD288,   /* U+1D288 */
  0xD289,   /* U+1D289 */
  0xD28A,   /* U+1D28A */
  0xD28B,   /* U+1D28B */
  0xD28C,   /* U+1D28C */
  0xD28D,   /* U+1D28D */
  0xD28E,   /* U+1D28E */
  0xD28F,   /* U+1D28F */
  0xD290,   /* U+1D290 */
  0xD291,   /* U+1D291 */
  0xD292,   /* U+1D292 */
  0xD293,   /* U+1D293 */
  0xD294,   /* U+1D294 */
  0xD295,   /* U+1D295 */
  0xD296,   /* U+1D296 */
  0xD297,   /* U+1D297 */
  0xD298,   /* U+1D298 */
  0xD299,   /* U+1D299 */
  0xD29A,   /* U+1D29A */
  0xD29B,   /* U+1D29B */
  0xD29C,   /* U+1D29C */
  0xD29D,   /* U+1D29D */
  0xD29E,   /* U+1D29E */
  0xD29F,   /* U+1D29F */
  0xD2A0,   /* U+1D2A0 */
  0xD2A1,   /* U+1D2A1 */
  0xD2A2,   /* U+1D2A2 */
  0xD2A3,   /* U+1D2A3 */
  0xD2A4,   /* U+1D2A4 */
  0xD2A5,   /* U+1D2A5 */
  0xD2A6,   /* U+1D2A6 */
  0xD2A7,   /* U+1D2A7 */
  0xD2A8,   /* U+1D2A8 */
  0xD2A9,   /* U+1D2A9 */
  0xD2AA,   /* U+1D2AA */
  0xD2AB,   /* U+1D2AB */
  0xD2AC,   /* U+1D2AC */
  0xD2AD,   /* U+1D2AD */
  0xD2AE,   /* U+1D2AE */
  0xD2AF,   /* U+1D2AF */
  0xD2B0,   /* U+1D2B0 */
  0xD2B1,   /* U+1D2B1 */
  0xD2B2,   /* U+1D2B2 */
  0xD2B3,   /* U+1D2B3 */
  0xD2B4,   /* U+1D2B4 */
  0xD2B5,   /* U+1D2B5 */
  0xD2B6,   /* U+1D2B6 */
  0xD2B7,   /* U+1D2B7 */
  0xD2B8,   /* U+1D2B8 */
  0xD2B9,   /* U+1D2B9 */
  0xD2BA,   /* U+1D2BA */
  0xD2BB,   /* U+1D2BB */
  0xD2BC,   /* U+1D2BC */
  0xD2BD,   /* U+1D2BD */
  0xD2BE,   /* U+1D2BE */
  0xD2BF,   /* U+1D2BF */
  0xD2C0,   /* U+1D2C0 */
  0xD2C1,   /* U+1D2C1 */
  0xD2C2,   /* U+1D2C2 */
  0xD2C3,   /* U+1D2C3 */
  0xD2C4,   /* U+1D2C4 */
  0xD2C5,   /* U+1D2C5 */
  0xD2C6,   /* U+1D2C6 */
  0xD2C7,   /* U+1D2C7 */
  0xD2C8,   /* U+1D2C8 */
  0xD2C9,   /* U+1D2C9 */
  0xD2CA,   /* U+1D2CA */
  0xD2CB,   /* U+1D2CB */
  0xD2CC,   /* U+1D2CC */
  0xD2CD,   /* U+1D2CD */
  0xD2CE,   /* U+1D2CE */
  0xD2CF,   /* U+1D2CF */
  0xD2D0,   /* U+1D2D0 */
  0xD2D1,   /* U+1D2D1 */
  0xD2D2,   /* U+1D2D2 */
  0xD2D3,   /* U+1D2D3 */
  0xD2D4,   /* U+1D2D4 */
  0xD2D5,   /* U+1D2D5 */
  0xD2D6,   /* U+1D2D6 */
  0xD2D7,   /* U+1D2D7 */
  0xD2D8,   /* U+1D2D8 */
  0xD2D9,   /* U+1D2D9 */
  0xD2DA,   /* U+1D2DA */
  0xD2DB,   /* U+1D2DB */
  0xD2DC,   /* U+1D2DC */
  0xD2DD,   /* U+1D2DD */
  0xD2DE,   /* U+1D2DE */
  0xD2DF,   /* U+1D2DF */
  0xD2E0,   /* U+1D2E0 */
  0xD2E1,   /* U+1D2E1 */
  0xD2E2,   /* U+1D2E2 */
  0xD2E3,   /* U+1D2E3 */
  0xD2E4,   /* U+1D2E4 */
  0xD2E5,   /* U+1D2E5 */
  0xD2E6,   /* U+1D2E6 */
  0xD2E7,   /* U+1D2E7 */
  0xD2E8,   /* U+1D2E8 */
  0xD2E9,   /* U+1D2E9 */
  0xD2EA,   /* U+1D2EA */
  0xD2EB,   /* U+1D2EB */
  0xD2EC,   /* U+1D2EC */
  0xD2ED,   /* U+1D2ED */
  0xD2EE,   /* U+1D2EE */
  0xD2EF,   /* U+1D2EF */
  0xD2F0,   /* U+1D2F0 */
  0xD2F1,   /* U+1D2F1 */
  0xD2F2,   /* U+1D2F2 */
  0xD2F3,   /* U+1D2F3 */
  0xD2F4,   /* U+1D2F4 */
  0xD2F5,   /* U+1D2F5 */
  0xD2F6,   /* U+1D2F6 */
  0xD2F7,   /* U+1D2F7 */
  0xD2F8,   /* U+1D2F8 */
  0xD2F9,   /* U+1D2F9 */
  0xD2FA,   /* U+1D2FA */
  0xD2FB,   /* U+1D2FB */
  0xD2FC,   /* U+1D2FC */
  0xD2FD,   /* U+1D2FD */
  0xD2FE,   /* U+1D2FE */
  0xD2FF,   /* U+1D2FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D200 */
  0x0000,   /* U+1D201 */
  0x0000,   /* U+1D202 */
  0x0000,   /* U+1D203 */
  0x0000,   /* U+1D204 */
  0x0000,   /* U+1D205 */
  0x0000,   /* U+1D206 */
  0x0000,   /* U+1D207 */
  0x0000,   /* U+1D208 */
  0x0000,   /* U+1D209 */
  0x0000,   /* U+1D20A */
  0x0000,   /* U+1D20B */
  0x0000,   /* U+1D20C */
  0x0000,   /* U+1D20D */
  0x0000,   /* U+1D20E */
  0x0000,   /* U+1D20F */
  0x0000,   /* U+1D210 */
  0x0000,   /* U+1D211 */
  0x0000,   /* U+1D212 */
  0x0000,   /* U+1D213 */
  0x0000,   /* U+1D214 */
  0x0000,   /* U+1D215 */
  0x0000,   /* U+1D216 */
  0x0000,   /* U+1D217 */
  0x0000,   /* U+1D218 */
  0x0000,   /* U+1D219 */
  0x0000,   /* U+1D21A */
  0x0000,   /* U+1D21B */
  0x0000,   /* U+1D21C */
  0x0000,   /* U+1D21D */
  0x0000,   /* U+1D21E */
  0x0000,   /* U+1D21F */
  0x0000,   /* U+1D220 */
  0x0000,   /* U+1D221 */
  0x0000,   /* U+1D222 */
  0x0000,   /* U+1D223 */
  0x0000,   /* U+1D224 */
  0x0000,   /* U+1D225 */
  0x0000,   /* U+1D226 */
  0x0000,   /* U+1D227 */
  0x0000,   /* U+1D228 */
  0x0000,   /* U+1D229 */
  0x0000,   /* U+1D22A */
  0x0000,   /* U+1D22B */
  0x0000,   /* U+1D22C */
  0x0000,   /* U+1D22D */
  0x0000,   /* U+1D22E */
  0x0000,   /* U+1D22F */
  0x0000,   /* U+1D230 */
  0x0000,   /* U+1D231 */
  0x0000,   /* U+1D232 */
  0x0000,   /* U+1D233 */
  0x0000,   /* U+1D234 */
  0x0000,   /* U+1D235 */
  0x0000,   /* U+1D236 */
  0x0000,   /* U+1D237 */
  0x0000,   /* U+1D238 */
  0x0000,   /* U+1D239 */
  0x0000,   /* U+1D23A */
  0x0000,   /* U+1D23B */
  0x0000,   /* U+1D23C */
  0x0000,   /* U+1D23D */
  0x0000,   /* U+1D23E */
  0x0000,   /* U+1D23F */
  0x0000,   /* U+1D240 */
  0x0000,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x0000,   /* U+1D245 */
  0x0000,   /* U+1D246 */
  0x0000,   /* U+1D247 */
  0x0000,   /* U+1D248 */
  0x0000,   /* U+1D249 */
  0x0000,   /* U+1D24A */
  0x0000,   /* U+1D24B */
  0x0000,   /* U+1D24C */
  0x0000,   /* U+1D24D */
  0x0000,   /* U+1D24E */
  0x0000,   /* U+1D24F */
  0x0000,   /* U+1D250 */
  0x0000,   /* U+1D251 */
  0x0000,   /* U+1D252 */
  0x0000,   /* U+1D253 */
  0x0000,   /* U+1D254 */
  0x0000,   /* U+1D255 */
  0x0000,   /* U+1D256 */
  0x0000,   /* U+1D257 */
  0x0000,   /* U+1D258 */
  0x0000,   /* U+1D259 */
  0x0000,   /* U+1D25A */
  0x0000,   /* U+1D25B */
  0x0000,   /* U+1D25C */
  0x0000,   /* U+1D25D */
  0x0000,   /* U+1D25E */
  0x0000,   /* U+1D25F */
  0x0000,   /* U+1D260 */
  0x0000,   /* U+1D261 */
  0x0000,   /* U+1D262 */
  0x0000,   /* U+1D263 */
  0x0000,   /* U+1D264 */
  0x0000,   /* U+1D265 */
  0x0000,   /* U+1D266 */
  0x0000,   /* U+1D267 */
  0x0000,   /* U+1D268 */
  0x0000,   /* U+1D269 */
  0x0000,   /* U+1D26A */
  0x0000,   /* U+1D26B */
  0x0000,   /* U+1D26C */
  0x0000,   /* U+1D26D */
  0x0000,   /* U+1D26E */
  0x0000,   /* U+1D26F */
  0x0000,   /* U+1D270 */
  0x0000,   /* U+1D271 */
  0x0000,   /* U+1D272 */
  0x0000,   /* U+1D273 */
  0x0000,   /* U+1D274 */
  0x0000,   /* U+1D275 */
  0x0000,   /* U+1D276 */
  0x0000,   /* U+1D277 */
  0x0000,   /* U+1D278 */
  0x0000,   /* U+1D279 */
  0x0000,   /* U+1D27A */
  0x0000,   /* U+1D27B */
  0x0000,   /* U+1D27C */
  0x0000,   /* U+1D27D */
  0x0000,   /* U+1D27E */
  0x0000,   /* U+1D27F */
  0x0000,   /* U+1D280 */
  0x0000,   /* U+1D281 */
  0x0000,   /* U+1D282 */
  0x0000,   /* U+1D283 */
  0x0000,   /* U+1D284 */
  0x0000,   /* U+1D285 */
  0x0000,   /* U+1D286 */
  0x0000,   /* U+1D287 */
  0x0000,   /* U+1D288 */
  0x0000,   /* U+1D289 */
  0x0000,   /* U+1D28A */
  0x0000,   /* U+1D28B */
  0x0000,   /* U+1D28C */
  0x0000,   /* U+1D28D */
  0x0000,   /* U+1D28E */
  0x0000,   /* U+1D28F */
  0x0000,   /* U+1D290 */
  0x0000,   /* U+1D291 */
  0x0000,   /* U+1D292 */
  0x0000,   /* U+1D293 */
  0x0000,   /* U+1D294 */
  0x0000,   /* U+1D295 */
  0x0000,   /* U+1D296 */
  0x0000,   /* U+1D297 */
  0x0000,   /* U+1D298 */
  0x0000,   /* U+1D299 */
  0x0000,   /* U+1D29A */
  0x0000,   /* U+1D29B */
  0x0000,   /* U+1D29C */
  0x0000,   /* U+1D29D */
  0x0000,   /* U+1D29E */
  0x0000,   /* U+1D29F */
  0x0000,   /* U+1D2A0 */
  0x0000,   /* U+1D2A1 */
  0x0000,   /* U+1D2A2 */
  0x0000,   /* U+1D2A3 */
  0x0000,   /* U+1D2A4 */
  0x0000,   /* U+1D2A5 */
  0x0000,   /* U+1D2A6 */
  0x0000,   /* U+1D2A7 */
  0x0000,   /* U+1D2A8 */
  0x0000,   /* U+1D2A9 */
  0x0000,   /* U+1D2AA */
  0x0000,   /* U+1D2AB */
  0x0000,   /* U+1D2AC */
  0x0000,   /* U+1D2AD */
  0x0000,   /* U+1D2AE */
  0x0000,   /* U+1D2AF */
  0x0000,   /* U+1D2B0 */
  0x0000,   /* U+1D2B1 */
  0x0000,   /* U+1D2B2 */
  0x0000,   /* U+1D2B3 */
  0x0000,   /* U+1D2B4 */
  0x0000,   /* U+1D2B5 */
  0x0000,   /* U+1D2B6 */
  0x0000,   /* U+1D2B7 */
  0x0000,   /* U+1D2B8 */
  0x0000,   /* U+1D2B9 */
  0x0000,   /* U+1D2BA */
  0x0000,   /* U+1D2BB */
  0x0000,   /* U+1D2BC */
  0x0000,   /* U+1D2BD */
  0x0000,   /* U+1D2BE */
  0x0000,   /* U+1D2BF */
  0x0000,   /* U+1D2C0 */
  0x0000,   /* U+1D2C1 */
  0x0000,   /* U+1D2C2 */
  0x0000,   /* U+1D2C3 */
  0x0000,   /* U+1D2C4 */
  0x0000,   /* U+1D2C5 */
  0x0000,   /* U+1D2C6 */
  0x0000,   /* U+1D2C7 */
  0x0000,   /* U+1D2C8 */
  0x0000,   /* U+1D2C9 */
  0x0000,   /* U+1D2CA */
  0x0000,   /* U+1D2CB */
  0x0000,   /* U+1D2CC */
  0x0000,   /* U+1D2CD */
  0x0000,   /* U+1D2CE */
  0x0000,   /* U+1D2CF */
  0x0000,   /* U+1D2D0 */
  0x0000,   /* U+1D2D1 */
  0x0000,   /* U+1D2D2 */
  0x0000,   /* U+1D2D3 */
  0x0000,   /* U+1D2D4 */
  0x0000,   /* U+1D2D5 */
  0x0000,   /* U+1D2D6 */
  0x0000,   /* U+1D2D7 */
  0x0000,   /* U+1D2D8 */
  0x0000,   /* U+1D2D9 */
  0x0000,   /* U+1D2DA */
  0x0000,   /* U+1D2DB */
  0x0000,   /* U+1D2DC */
  0x0000,   /* U+1D2DD */
  0x0000,   /* U+1D2DE */
  0x0000,   /* U+1D2DF */
  0x0000,   /* U+1D2E0 */
  0x0000,   /* U+1D2E1 */
  0x0000,   /* U+1D2E2 */
  0x0000,   /* U+1D2E3 */
  0x0000,   /* U+1D2E4 */
  0x0000,   /* U+1D2E5 */
  0x0000,   /* U+1D2E6 */
  0x0000,   /* U+1D2E7 */
  0x0000,   /* U+1D2E8 */
  0x0000,   /* U+1D2E9 */
  0x0000,   /* U+1D2EA */
  0x0000,   /* U+1D2EB */
  0x0000,   /* U+1D2EC */
  0x0000,   /* U+1D2ED */
  0x0000,   /* U+1D2EE */
  0x0000,   /* U+1D2EF */
  0x0000,   /* U+1D2F0 */
  0x0000,   /* U+1D2F1 */
  0x0000,   /* U+1D2F2 */
  0x0000,   /* U+1D2F3 */
  0x0000,   /* U+1D2F4 */
  0x0000,   /* U+1D2F5 */
  0x0000,   /* U+1D2F6 */
  0x0000,   /* U+1D2F7 */
  0x0000,   /* U+1D2F8 */
  0x0000,   /* U+1D2F9 */
  0x0000,   /* U+1D2FA */
  0x0000,   /* U+1D2FB */
  0x0000,   /* U+1D2FC */
  0x0000,   /* U+1D2FD */
  0x0000,   /* U+1D2FE */
  0x0000,   /* U+1D2FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D200 */
  0x0000,   /* U+1D201 */
  0x0000,   /* U+1D202 */
  0x0000,   /* U+1D203 */
  0x0000,   /* U+1D204 */
  0x0000,   /* U+1D205 */
  0x0000,   /* U+1D206 */
  0x0000,   /* U+1D207 */
  0x0000,   /* U+1D208 */
  0x0000,   /* U+1D209 */
  0x0000,   /* U+1D20A */
  0x0000,   /* U+1D20B */
  0x0000,   /* U+1D20C */
  0x0000,   /* U+1D20D */
  0x0000,   /* U+1D20E */
  0x0000,   /* U+1D20F */
  0x0000,   /* U+1D210 */
  0x0000,   /* U+1D211 */
  0x0000,   /* U+1D212 */
  0x0000,   /* U+1D213 */
  0x0000,   /* U+1D214 */
  0x0000,   /* U+1D215 */
  0x0000,   /* U+1D216 */
  0x0000,   /* U+1D217 */
  0x0000,   /* U+1D218 */
  0x0000,   /* U+1D219 */
  0x0000,   /* U+1D21A */
  0x0000,   /* U+1D21B */
  0x0000,   /* U+1D21C */
  0x0000,   /* U+1D21D */
  0x0000,   /* U+1D21E */
  0x0000,   /* U+1D21F */
  0x0000,   /* U+1D220 */
  0x0000,   /* U+1D221 */
  0x0000,   /* U+1D222 */
  0x0000,   /* U+1D223 */
  0x0000,   /* U+1D224 */
  0x0000,   /* U+1D225 */
  0x0000,   /* U+1D226 */
  0x0000,   /* U+1D227 */
  0x0000,   /* U+1D228 */
  0x0000,   /* U+1D229 */
  0x0000,   /* U+1D22A */
  0x0000,   /* U+1D22B */
  0x0000,   /* U+1D22C */
  0x0000,   /* U+1D22D */
  0x0000,   /* U+1D22E */
  0x0000,   /* U+1D22F */
  0x0000,   /* U+1D230 */
  0x0000,   /* U+1D231 */
  0x0000,   /* U+1D232 */
  0x0000,   /* U+1D233 */
  0x0000,   /* U+1D234 */
  0x0000,   /* U+1D235 */
  0x0000,   /* U+1D236 */
  0x0000,   /* U+1D237 */
  0x0000,   /* U+1D238 */
  0x0000,   /* U+1D239 */
  0x0000,   /* U+1D23A */
  0x0000,   /* U+1D23B */
  0x0000,   /* U+1D23C */
  0x0000,   /* U+1D23D */
  0x0000,   /* U+1D23E */
  0x0000,   /* U+1D23F */
  0x0000,   /* U+1D240 */
  0x0000,   /* U+1D241 */
  0x0000,   /* U+1D242 */
  0x0000,   /* U+1D243 */
  0x0000,   /* U+1D244 */
  0x0000,   /* U+1D245 */
  0x0000,   /* U+1D246 */
  0x0000,   /* U+1D247 */
  0x0000,   /* U+1D248 */
  0x0000,   /* U+1D249 */
  0x0000,   /* U+1D24A */
  0x0000,   /* U+1D24B */
  0x0000,   /* U+1D24C */
  0x0000,   /* U+1D24D */
  0x0000,   /* U+1D24E */
  0x0000,   /* U+1D24F */
  0x0000,   /* U+1D250 */
  0x0000,   /* U+1D251 */
  0x0000,   /* U+1D252 */
  0x0000,   /* U+1D253 */
  0x0000,   /* U+1D254 */
  0x0000,   /* U+1D255 */
  0x0000,   /* U+1D256 */
  0x0000,   /* U+1D257 */
  0x0000,   /* U+1D258 */
  0x0000,   /* U+1D259 */
  0x0000,   /* U+1D25A */
  0x0000,   /* U+1D25B */
  0x0000,   /* U+1D25C */
  0x0000,   /* U+1D25D */
  0x0000,   /* U+1D25E */
  0x0000,   /* U+1D25F */
  0x0000,   /* U+1D260 */
  0x0000,   /* U+1D261 */
  0x0000,   /* U+1D262 */
  0x0000,   /* U+1D263 */
  0x0000,   /* U+1D264 */
  0x0000,   /* U+1D265 */
  0x0000,   /* U+1D266 */
  0x0000,   /* U+1D267 */
  0x0000,   /* U+1D268 */
  0x0000,   /* U+1D269 */
  0x0000,   /* U+1D26A */
  0x0000,   /* U+1D26B */
  0x0000,   /* U+1D26C */
  0x0000,   /* U+1D26D */
  0x0000,   /* U+1D26E */
  0x0000,   /* U+1D26F */
  0x0000,   /* U+1D270 */
  0x0000,   /* U+1D271 */
  0x0000,   /* U+1D272 */
  0x0000,   /* U+1D273 */
  0x0000,   /* U+1D274 */
  0x0000,   /* U+1D275 */
  0x0000,   /* U+1D276 */
  0x0000,   /* U+1D277 */
  0x0000,   /* U+1D278 */
  0x0000,   /* U+1D279 */
  0x0000,   /* U+1D27A */
  0x0000,   /* U+1D27B */
  0x0000,   /* U+1D27C */
  0x0000,   /* U+1D27D */
  0x0000,   /* U+1D27E */
  0x0000,   /* U+1D27F */
  0x0000,   /* U+1D280 */
  0x0000,   /* U+1D281 */
  0x0000,   /* U+1D282 */
  0x0000,   /* U+1D283 */
  0x0000,   /* U+1D284 */
  0x0000,   /* U+1D285 */
  0x0000,   /* U+1D286 */
  0x0000,   /* U+1D287 */
  0x0000,   /* U+1D288 */
  0x0000,   /* U+1D289 */
  0x0000,   /* U+1D28A */
  0x0000,   /* U+1D28B */
  0x0000,   /* U+1D28C */
  0x0000,   /* U+1D28D */
  0x0000,   /* U+1D28E */
  0x0000,   /* U+1D28F */
  0x0000,   /* U+1D290 */
  0x0000,   /* U+1D291 */
  0x0000,   /* U+1D292 */
  0x0000,   /* U+1D293 */
  0x0000,   /* U+1D294 */
  0x0000,   /* U+1D295 */
  0x0000,   /* U+1D296 */
  0x0000,   /* U+1D297 */
  0x0000,   /* U+1D298 */
  0x0000,   /* U+1D299 */
  0x0000,   /* U+1D29A */
  0x0000,   /* U+1D29B */
  0x0000,   /* U+1D29C */
  0x0000,   /* U+1D29D */
  0x0000,   /* U+1D29E */
  0x0000,   /* U+1D29F */
  0x0000,   /* U+1D2A0 */
  0x0000,   /* U+1D2A1 */
  0x0000,   /* U+1D2A2 */
  0x0000,   /* U+1D2A3 */
  0x0000,   /* U+1D2A4 */
  0x0000,   /* U+1D2A5 */
  0x0000,   /* U+1D2A6 */
  0x0000,   /* U+1D2A7 */
  0x0000,   /* U+1D2A8 */
  0x0000,   /* U+1D2A9 */
  0x0000,   /* U+1D2AA */
  0x0000,   /* U+1D2AB */
  0x0000,   /* U+1D2AC */
  0x0000,   /* U+1D2AD */
  0x0000,   /* U+1D2AE */
  0x0000,   /* U+1D2AF */
  0x0000,   /* U+1D2B0 */
  0x0000,   /* U+1D2B1 */
  0x0000,   /* U+1D2B2 */
  0x0000,   /* U+1D2B3 */
  0x0000,   /* U+1D2B4 */
  0x0000,   /* U+1D2B5 */
  0x0000,   /* U+1D2B6 */
  0x0000,   /* U+1D2B7 */
  0x0000,   /* U+1D2B8 */
  0x0000,   /* U+1D2B9 */
  0x0000,   /* U+1D2BA */
  0x0000,   /* U+1D2BB */
  0x0000,   /* U+1D2BC */
  0x0000,   /* U+1D2BD */
  0x0000,   /* U+1D2BE */
  0x0000,   /* U+1D2BF */
  0x0000,   /* U+1D2C0 */
  0x0000,   /* U+1D2C1 */
  0x0000,   /* U+1D2C2 */
  0x0000,   /* U+1D2C3 */
  0x0000,   /* U+1D2C4 */
  0x0000,   /* U+1D2C5 */
  0x0000,   /* U+1D2C6 */
  0x0000,   /* U+1D2C7 */
  0x0000,   /* U+1D2C8 */
  0x0000,   /* U+1D2C9 */
  0x0000,   /* U+1D2CA */
  0x0000,   /* U+1D2CB */
  0x0000,   /* U+1D2CC */
  0x0000,   /* U+1D2CD */
  0x0000,   /* U+1D2CE */
  0x0000,   /* U+1D2CF */
  0x0000,   /* U+1D2D0 */
  0x0000,   /* U+1D2D1 */
  0x0000,   /* U+1D2D2 */
  0x0000,   /* U+1D2D3 */
  0x0000,   /* U+1D2D4 */
  0x0000,   /* U+1D2D5 */
  0x0000,   /* U+1D2D6 */
  0x0000,   /* U+1D2D7 */
  0x0000,   /* U+1D2D8 */
  0x0000,   /* U+1D2D9 */
  0x0000,   /* U+1D2DA */
  0x0000,   /* U+1D2DB */
  0x0000,   /* U+1D2DC */
  0x0000,   /* U+1D2DD */
  0x0000,   /* U+1D2DE */
  0x0000,   /* U+1D2DF */
  0x0000,   /* U+1D2E0 */
  0x0000,   /* U+1D2E1 */
  0x0000,   /* U+1D2E2 */
  0x0000,   /* U+1D2E3 */
  0x0000,   /* U+1D2E4 */
  0x0000,   /* U+1D2E5 */
  0x0000,   /* U+1D2E6 */
  0x0000,   /* U+1D2E7 */
  0x0000,   /* U+1D2E8 */
  0x0000,   /* U+1D2E9 */
  0x0000,   /* U+1D2EA */
  0x0000,   /* U+1D2EB */
  0x0000,   /* U+1D2EC */
  0x0000,   /* U+1D2ED */
  0x0000,   /* U+1D2EE */
  0x0000,   /* U+1D2EF */
  0x0000,   /* U+1D2F0 */
  0x0000,   /* U+1D2F1 */
  0x0000,   /* U+1D2F2 */
  0x0000,   /* U+1D2F3 */
  0x0000,   /* U+1D2F4 */
  0x0000,   /* U+1D2F5 */
  0x0000,   /* U+1D2F6 */
  0x0000,   /* U+1D2F7 */
  0x0000,   /* U+1D2F8 */
  0x0000,   /* U+1D2F9 */
  0x0000,   /* U+1D2FA */
  0x0000,   /* U+1D2FB */
  0x0000,   /* U+1D2FC */
  0x0000,   /* U+1D2FD */
  0x0000,   /* U+1D2FE */
  0x0000,   /* U+1D2FF */
};

uint16 uca900_p1D3[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0EEA,   /* U+1D300 */
  0x0EEB,   /* U+1D301 */
  0x0EEC,   /* U+1D302 */
  0x0EED,   /* U+1D303 */
  0x0EEE,   /* U+1D304 */
  0x0EEF,   /* U+1D305 */
  0x0EF0,   /* U+1D306 */
  0x0EF1,   /* U+1D307 */
  0x0EF2,   /* U+1D308 */
  0x0EF3,   /* U+1D309 */
  0x0EF4,   /* U+1D30A */
  0x0EF5,   /* U+1D30B */
  0x0EF6,   /* U+1D30C */
  0x0EF7,   /* U+1D30D */
  0x0EF8,   /* U+1D30E */
  0x0EF9,   /* U+1D30F */
  0x0EFA,   /* U+1D310 */
  0x0EFB,   /* U+1D311 */
  0x0EFC,   /* U+1D312 */
  0x0EFD,   /* U+1D313 */
  0x0EFE,   /* U+1D314 */
  0x0EFF,   /* U+1D315 */
  0x0F00,   /* U+1D316 */
  0x0F01,   /* U+1D317 */
  0x0F02,   /* U+1D318 */
  0x0F03,   /* U+1D319 */
  0x0F04,   /* U+1D31A */
  0x0F05,   /* U+1D31B */
  0x0F06,   /* U+1D31C */
  0x0F07,   /* U+1D31D */
  0x0F08,   /* U+1D31E */
  0x0F09,   /* U+1D31F */
  0x0F0A,   /* U+1D320 */
  0x0F0B,   /* U+1D321 */
  0x0F0C,   /* U+1D322 */
  0x0F0D,   /* U+1D323 */
  0x0F0E,   /* U+1D324 */
  0x0F0F,   /* U+1D325 */
  0x0F10,   /* U+1D326 */
  0x0F11,   /* U+1D327 */
  0x0F12,   /* U+1D328 */
  0x0F13,   /* U+1D329 */
  0x0F14,   /* U+1D32A */
  0x0F15,   /* U+1D32B */
  0x0F16,   /* U+1D32C */
  0x0F17,   /* U+1D32D */
  0x0F18,   /* U+1D32E */
  0x0F19,   /* U+1D32F */
  0x0F1A,   /* U+1D330 */
  0x0F1B,   /* U+1D331 */
  0x0F1C,   /* U+1D332 */
  0x0F1D,   /* U+1D333 */
  0x0F1E,   /* U+1D334 */
  0x0F1F,   /* U+1D335 */
  0x0F20,   /* U+1D336 */
  0x0F21,   /* U+1D337 */
  0x0F22,   /* U+1D338 */
  0x0F23,   /* U+1D339 */
  0x0F24,   /* U+1D33A */
  0x0F25,   /* U+1D33B */
  0x0F26,   /* U+1D33C */
  0x0F27,   /* U+1D33D */
  0x0F28,   /* U+1D33E */
  0x0F29,   /* U+1D33F */
  0x0F2A,   /* U+1D340 */
  0x0F2B,   /* U+1D341 */
  0x0F2C,   /* U+1D342 */
  0x0F2D,   /* U+1D343 */
  0x0F2E,   /* U+1D344 */
  0x0F2F,   /* U+1D345 */
  0x0F30,   /* U+1D346 */
  0x0F31,   /* U+1D347 */
  0x0F32,   /* U+1D348 */
  0x0F33,   /* U+1D349 */
  0x0F34,   /* U+1D34A */
  0x0F35,   /* U+1D34B */
  0x0F36,   /* U+1D34C */
  0x0F37,   /* U+1D34D */
  0x0F38,   /* U+1D34E */
  0x0F39,   /* U+1D34F */
  0x0F3A,   /* U+1D350 */
  0x0F3B,   /* U+1D351 */
  0x0F3C,   /* U+1D352 */
  0x0F3D,   /* U+1D353 */
  0x0F3E,   /* U+1D354 */
  0x0F3F,   /* U+1D355 */
  0x0F40,   /* U+1D356 */
  0xFBC3,   /* U+1D357 */
  0xFBC3,   /* U+1D358 */
  0xFBC3,   /* U+1D359 */
  0xFBC3,   /* U+1D35A */
  0xFBC3,   /* U+1D35B */
  0xFBC3,   /* U+1D35C */
  0xFBC3,   /* U+1D35D */
  0xFBC3,   /* U+1D35E */
  0xFBC3,   /* U+1D35F */
  0x1C3E,   /* U+1D360 */
  0x1C3F,   /* U+1D361 */
  0x1C40,   /* U+1D362 */
  0x1C41,   /* U+1D363 */
  0x1C42,   /* U+1D364 */
  0x1C43,   /* U+1D365 */
  0x1C44,   /* U+1D366 */
  0x1C45,   /* U+1D367 */
  0x1C46,   /* U+1D368 */
  0x1BEF,   /* U+1D369 */
  0x1BF0,   /* U+1D36A */
  0x1BF1,   /* U+1D36B */
  0x1BF2,   /* U+1D36C */
  0x1BF3,   /* U+1D36D */
  0x1BF4,   /* U+1D36E */
  0x1BF5,   /* U+1D36F */
  0x1BF6,   /* U+1D370 */
  0x1BF7,   /* U+1D371 */
  0xFBC3,   /* U+1D372 */
  0xFBC3,   /* U+1D373 */
  0xFBC3,   /* U+1D374 */
  0xFBC3,   /* U+1D375 */
  0xFBC3,   /* U+1D376 */
  0xFBC3,   /* U+1D377 */
  0xFBC3,   /* U+1D378 */
  0xFBC3,   /* U+1D379 */
  0xFBC3,   /* U+1D37A */
  0xFBC3,   /* U+1D37B */
  0xFBC3,   /* U+1D37C */
  0xFBC3,   /* U+1D37D */
  0xFBC3,   /* U+1D37E */
  0xFBC3,   /* U+1D37F */
  0xFBC3,   /* U+1D380 */
  0xFBC3,   /* U+1D381 */
  0xFBC3,   /* U+1D382 */
  0xFBC3,   /* U+1D383 */
  0xFBC3,   /* U+1D384 */
  0xFBC3,   /* U+1D385 */
  0xFBC3,   /* U+1D386 */
  0xFBC3,   /* U+1D387 */
  0xFBC3,   /* U+1D388 */
  0xFBC3,   /* U+1D389 */
  0xFBC3,   /* U+1D38A */
  0xFBC3,   /* U+1D38B */
  0xFBC3,   /* U+1D38C */
  0xFBC3,   /* U+1D38D */
  0xFBC3,   /* U+1D38E */
  0xFBC3,   /* U+1D38F */
  0xFBC3,   /* U+1D390 */
  0xFBC3,   /* U+1D391 */
  0xFBC3,   /* U+1D392 */
  0xFBC3,   /* U+1D393 */
  0xFBC3,   /* U+1D394 */
  0xFBC3,   /* U+1D395 */
  0xFBC3,   /* U+1D396 */
  0xFBC3,   /* U+1D397 */
  0xFBC3,   /* U+1D398 */
  0xFBC3,   /* U+1D399 */
  0xFBC3,   /* U+1D39A */
  0xFBC3,   /* U+1D39B */
  0xFBC3,   /* U+1D39C */
  0xFBC3,   /* U+1D39D */
  0xFBC3,   /* U+1D39E */
  0xFBC3,   /* U+1D39F */
  0xFBC3,   /* U+1D3A0 */
  0xFBC3,   /* U+1D3A1 */
  0xFBC3,   /* U+1D3A2 */
  0xFBC3,   /* U+1D3A3 */
  0xFBC3,   /* U+1D3A4 */
  0xFBC3,   /* U+1D3A5 */
  0xFBC3,   /* U+1D3A6 */
  0xFBC3,   /* U+1D3A7 */
  0xFBC3,   /* U+1D3A8 */
  0xFBC3,   /* U+1D3A9 */
  0xFBC3,   /* U+1D3AA */
  0xFBC3,   /* U+1D3AB */
  0xFBC3,   /* U+1D3AC */
  0xFBC3,   /* U+1D3AD */
  0xFBC3,   /* U+1D3AE */
  0xFBC3,   /* U+1D3AF */
  0xFBC3,   /* U+1D3B0 */
  0xFBC3,   /* U+1D3B1 */
  0xFBC3,   /* U+1D3B2 */
  0xFBC3,   /* U+1D3B3 */
  0xFBC3,   /* U+1D3B4 */
  0xFBC3,   /* U+1D3B5 */
  0xFBC3,   /* U+1D3B6 */
  0xFBC3,   /* U+1D3B7 */
  0xFBC3,   /* U+1D3B8 */
  0xFBC3,   /* U+1D3B9 */
  0xFBC3,   /* U+1D3BA */
  0xFBC3,   /* U+1D3BB */
  0xFBC3,   /* U+1D3BC */
  0xFBC3,   /* U+1D3BD */
  0xFBC3,   /* U+1D3BE */
  0xFBC3,   /* U+1D3BF */
  0xFBC3,   /* U+1D3C0 */
  0xFBC3,   /* U+1D3C1 */
  0xFBC3,   /* U+1D3C2 */
  0xFBC3,   /* U+1D3C3 */
  0xFBC3,   /* U+1D3C4 */
  0xFBC3,   /* U+1D3C5 */
  0xFBC3,   /* U+1D3C6 */
  0xFBC3,   /* U+1D3C7 */
  0xFBC3,   /* U+1D3C8 */
  0xFBC3,   /* U+1D3C9 */
  0xFBC3,   /* U+1D3CA */
  0xFBC3,   /* U+1D3CB */
  0xFBC3,   /* U+1D3CC */
  0xFBC3,   /* U+1D3CD */
  0xFBC3,   /* U+1D3CE */
  0xFBC3,   /* U+1D3CF */
  0xFBC3,   /* U+1D3D0 */
  0xFBC3,   /* U+1D3D1 */
  0xFBC3,   /* U+1D3D2 */
  0xFBC3,   /* U+1D3D3 */
  0xFBC3,   /* U+1D3D4 */
  0xFBC3,   /* U+1D3D5 */
  0xFBC3,   /* U+1D3D6 */
  0xFBC3,   /* U+1D3D7 */
  0xFBC3,   /* U+1D3D8 */
  0xFBC3,   /* U+1D3D9 */
  0xFBC3,   /* U+1D3DA */
  0xFBC3,   /* U+1D3DB */
  0xFBC3,   /* U+1D3DC */
  0xFBC3,   /* U+1D3DD */
  0xFBC3,   /* U+1D3DE */
  0xFBC3,   /* U+1D3DF */
  0xFBC3,   /* U+1D3E0 */
  0xFBC3,   /* U+1D3E1 */
  0xFBC3,   /* U+1D3E2 */
  0xFBC3,   /* U+1D3E3 */
  0xFBC3,   /* U+1D3E4 */
  0xFBC3,   /* U+1D3E5 */
  0xFBC3,   /* U+1D3E6 */
  0xFBC3,   /* U+1D3E7 */
  0xFBC3,   /* U+1D3E8 */
  0xFBC3,   /* U+1D3E9 */
  0xFBC3,   /* U+1D3EA */
  0xFBC3,   /* U+1D3EB */
  0xFBC3,   /* U+1D3EC */
  0xFBC3,   /* U+1D3ED */
  0xFBC3,   /* U+1D3EE */
  0xFBC3,   /* U+1D3EF */
  0xFBC3,   /* U+1D3F0 */
  0xFBC3,   /* U+1D3F1 */
  0xFBC3,   /* U+1D3F2 */
  0xFBC3,   /* U+1D3F3 */
  0xFBC3,   /* U+1D3F4 */
  0xFBC3,   /* U+1D3F5 */
  0xFBC3,   /* U+1D3F6 */
  0xFBC3,   /* U+1D3F7 */
  0xFBC3,   /* U+1D3F8 */
  0xFBC3,   /* U+1D3F9 */
  0xFBC3,   /* U+1D3FA */
  0xFBC3,   /* U+1D3FB */
  0xFBC3,   /* U+1D3FC */
  0xFBC3,   /* U+1D3FD */
  0xFBC3,   /* U+1D3FE */
  0xFBC3,   /* U+1D3FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D300 */
  0x0020,   /* U+1D301 */
  0x0020,   /* U+1D302 */
  0x0020,   /* U+1D303 */
  0x0020,   /* U+1D304 */
  0x0020,   /* U+1D305 */
  0x0020,   /* U+1D306 */
  0x0020,   /* U+1D307 */
  0x0020,   /* U+1D308 */
  0x0020,   /* U+1D309 */
  0x0020,   /* U+1D30A */
  0x0020,   /* U+1D30B */
  0x0020,   /* U+1D30C */
  0x0020,   /* U+1D30D */
  0x0020,   /* U+1D30E */
  0x0020,   /* U+1D30F */
  0x0020,   /* U+1D310 */
  0x0020,   /* U+1D311 */
  0x0020,   /* U+1D312 */
  0x0020,   /* U+1D313 */
  0x0020,   /* U+1D314 */
  0x0020,   /* U+1D315 */
  0x0020,   /* U+1D316 */
  0x0020,   /* U+1D317 */
  0x0020,   /* U+1D318 */
  0x0020,   /* U+1D319 */
  0x0020,   /* U+1D31A */
  0x0020,   /* U+1D31B */
  0x0020,   /* U+1D31C */
  0x0020,   /* U+1D31D */
  0x0020,   /* U+1D31E */
  0x0020,   /* U+1D31F */
  0x0020,   /* U+1D320 */
  0x0020,   /* U+1D321 */
  0x0020,   /* U+1D322 */
  0x0020,   /* U+1D323 */
  0x0020,   /* U+1D324 */
  0x0020,   /* U+1D325 */
  0x0020,   /* U+1D326 */
  0x0020,   /* U+1D327 */
  0x0020,   /* U+1D328 */
  0x0020,   /* U+1D329 */
  0x0020,   /* U+1D32A */
  0x0020,   /* U+1D32B */
  0x0020,   /* U+1D32C */
  0x0020,   /* U+1D32D */
  0x0020,   /* U+1D32E */
  0x0020,   /* U+1D32F */
  0x0020,   /* U+1D330 */
  0x0020,   /* U+1D331 */
  0x0020,   /* U+1D332 */
  0x0020,   /* U+1D333 */
  0x0020,   /* U+1D334 */
  0x0020,   /* U+1D335 */
  0x0020,   /* U+1D336 */
  0x0020,   /* U+1D337 */
  0x0020,   /* U+1D338 */
  0x0020,   /* U+1D339 */
  0x0020,   /* U+1D33A */
  0x0020,   /* U+1D33B */
  0x0020,   /* U+1D33C */
  0x0020,   /* U+1D33D */
  0x0020,   /* U+1D33E */
  0x0020,   /* U+1D33F */
  0x0020,   /* U+1D340 */
  0x0020,   /* U+1D341 */
  0x0020,   /* U+1D342 */
  0x0020,   /* U+1D343 */
  0x0020,   /* U+1D344 */
  0x0020,   /* U+1D345 */
  0x0020,   /* U+1D346 */
  0x0020,   /* U+1D347 */
  0x0020,   /* U+1D348 */
  0x0020,   /* U+1D349 */
  0x0020,   /* U+1D34A */
  0x0020,   /* U+1D34B */
  0x0020,   /* U+1D34C */
  0x0020,   /* U+1D34D */
  0x0020,   /* U+1D34E */
  0x0020,   /* U+1D34F */
  0x0020,   /* U+1D350 */
  0x0020,   /* U+1D351 */
  0x0020,   /* U+1D352 */
  0x0020,   /* U+1D353 */
  0x0020,   /* U+1D354 */
  0x0020,   /* U+1D355 */
  0x0020,   /* U+1D356 */
  0x0020,   /* U+1D357 */
  0x0020,   /* U+1D358 */
  0x0020,   /* U+1D359 */
  0x0020,   /* U+1D35A */
  0x0020,   /* U+1D35B */
  0x0020,   /* U+1D35C */
  0x0020,   /* U+1D35D */
  0x0020,   /* U+1D35E */
  0x0020,   /* U+1D35F */
  0x0020,   /* U+1D360 */
  0x0020,   /* U+1D361 */
  0x0020,   /* U+1D362 */
  0x0020,   /* U+1D363 */
  0x0020,   /* U+1D364 */
  0x0020,   /* U+1D365 */
  0x0020,   /* U+1D366 */
  0x0020,   /* U+1D367 */
  0x0020,   /* U+1D368 */
  0x0020,   /* U+1D369 */
  0x0020,   /* U+1D36A */
  0x0020,   /* U+1D36B */
  0x0020,   /* U+1D36C */
  0x0020,   /* U+1D36D */
  0x0020,   /* U+1D36E */
  0x0020,   /* U+1D36F */
  0x0020,   /* U+1D370 */
  0x0020,   /* U+1D371 */
  0x0020,   /* U+1D372 */
  0x0020,   /* U+1D373 */
  0x0020,   /* U+1D374 */
  0x0020,   /* U+1D375 */
  0x0020,   /* U+1D376 */
  0x0020,   /* U+1D377 */
  0x0020,   /* U+1D378 */
  0x0020,   /* U+1D379 */
  0x0020,   /* U+1D37A */
  0x0020,   /* U+1D37B */
  0x0020,   /* U+1D37C */
  0x0020,   /* U+1D37D */
  0x0020,   /* U+1D37E */
  0x0020,   /* U+1D37F */
  0x0020,   /* U+1D380 */
  0x0020,   /* U+1D381 */
  0x0020,   /* U+1D382 */
  0x0020,   /* U+1D383 */
  0x0020,   /* U+1D384 */
  0x0020,   /* U+1D385 */
  0x0020,   /* U+1D386 */
  0x0020,   /* U+1D387 */
  0x0020,   /* U+1D388 */
  0x0020,   /* U+1D389 */
  0x0020,   /* U+1D38A */
  0x0020,   /* U+1D38B */
  0x0020,   /* U+1D38C */
  0x0020,   /* U+1D38D */
  0x0020,   /* U+1D38E */
  0x0020,   /* U+1D38F */
  0x0020,   /* U+1D390 */
  0x0020,   /* U+1D391 */
  0x0020,   /* U+1D392 */
  0x0020,   /* U+1D393 */
  0x0020,   /* U+1D394 */
  0x0020,   /* U+1D395 */
  0x0020,   /* U+1D396 */
  0x0020,   /* U+1D397 */
  0x0020,   /* U+1D398 */
  0x0020,   /* U+1D399 */
  0x0020,   /* U+1D39A */
  0x0020,   /* U+1D39B */
  0x0020,   /* U+1D39C */
  0x0020,   /* U+1D39D */
  0x0020,   /* U+1D39E */
  0x0020,   /* U+1D39F */
  0x0020,   /* U+1D3A0 */
  0x0020,   /* U+1D3A1 */
  0x0020,   /* U+1D3A2 */
  0x0020,   /* U+1D3A3 */
  0x0020,   /* U+1D3A4 */
  0x0020,   /* U+1D3A5 */
  0x0020,   /* U+1D3A6 */
  0x0020,   /* U+1D3A7 */
  0x0020,   /* U+1D3A8 */
  0x0020,   /* U+1D3A9 */
  0x0020,   /* U+1D3AA */
  0x0020,   /* U+1D3AB */
  0x0020,   /* U+1D3AC */
  0x0020,   /* U+1D3AD */
  0x0020,   /* U+1D3AE */
  0x0020,   /* U+1D3AF */
  0x0020,   /* U+1D3B0 */
  0x0020,   /* U+1D3B1 */
  0x0020,   /* U+1D3B2 */
  0x0020,   /* U+1D3B3 */
  0x0020,   /* U+1D3B4 */
  0x0020,   /* U+1D3B5 */
  0x0020,   /* U+1D3B6 */
  0x0020,   /* U+1D3B7 */
  0x0020,   /* U+1D3B8 */
  0x0020,   /* U+1D3B9 */
  0x0020,   /* U+1D3BA */
  0x0020,   /* U+1D3BB */
  0x0020,   /* U+1D3BC */
  0x0020,   /* U+1D3BD */
  0x0020,   /* U+1D3BE */
  0x0020,   /* U+1D3BF */
  0x0020,   /* U+1D3C0 */
  0x0020,   /* U+1D3C1 */
  0x0020,   /* U+1D3C2 */
  0x0020,   /* U+1D3C3 */
  0x0020,   /* U+1D3C4 */
  0x0020,   /* U+1D3C5 */
  0x0020,   /* U+1D3C6 */
  0x0020,   /* U+1D3C7 */
  0x0020,   /* U+1D3C8 */
  0x0020,   /* U+1D3C9 */
  0x0020,   /* U+1D3CA */
  0x0020,   /* U+1D3CB */
  0x0020,   /* U+1D3CC */
  0x0020,   /* U+1D3CD */
  0x0020,   /* U+1D3CE */
  0x0020,   /* U+1D3CF */
  0x0020,   /* U+1D3D0 */
  0x0020,   /* U+1D3D1 */
  0x0020,   /* U+1D3D2 */
  0x0020,   /* U+1D3D3 */
  0x0020,   /* U+1D3D4 */
  0x0020,   /* U+1D3D5 */
  0x0020,   /* U+1D3D6 */
  0x0020,   /* U+1D3D7 */
  0x0020,   /* U+1D3D8 */
  0x0020,   /* U+1D3D9 */
  0x0020,   /* U+1D3DA */
  0x0020,   /* U+1D3DB */
  0x0020,   /* U+1D3DC */
  0x0020,   /* U+1D3DD */
  0x0020,   /* U+1D3DE */
  0x0020,   /* U+1D3DF */
  0x0020,   /* U+1D3E0 */
  0x0020,   /* U+1D3E1 */
  0x0020,   /* U+1D3E2 */
  0x0020,   /* U+1D3E3 */
  0x0020,   /* U+1D3E4 */
  0x0020,   /* U+1D3E5 */
  0x0020,   /* U+1D3E6 */
  0x0020,   /* U+1D3E7 */
  0x0020,   /* U+1D3E8 */
  0x0020,   /* U+1D3E9 */
  0x0020,   /* U+1D3EA */
  0x0020,   /* U+1D3EB */
  0x0020,   /* U+1D3EC */
  0x0020,   /* U+1D3ED */
  0x0020,   /* U+1D3EE */
  0x0020,   /* U+1D3EF */
  0x0020,   /* U+1D3F0 */
  0x0020,   /* U+1D3F1 */
  0x0020,   /* U+1D3F2 */
  0x0020,   /* U+1D3F3 */
  0x0020,   /* U+1D3F4 */
  0x0020,   /* U+1D3F5 */
  0x0020,   /* U+1D3F6 */
  0x0020,   /* U+1D3F7 */
  0x0020,   /* U+1D3F8 */
  0x0020,   /* U+1D3F9 */
  0x0020,   /* U+1D3FA */
  0x0020,   /* U+1D3FB */
  0x0020,   /* U+1D3FC */
  0x0020,   /* U+1D3FD */
  0x0020,   /* U+1D3FE */
  0x0020,   /* U+1D3FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D300 */
  0x0002,   /* U+1D301 */
  0x0002,   /* U+1D302 */
  0x0002,   /* U+1D303 */
  0x0002,   /* U+1D304 */
  0x0002,   /* U+1D305 */
  0x0002,   /* U+1D306 */
  0x0002,   /* U+1D307 */
  0x0002,   /* U+1D308 */
  0x0002,   /* U+1D309 */
  0x0002,   /* U+1D30A */
  0x0002,   /* U+1D30B */
  0x0002,   /* U+1D30C */
  0x0002,   /* U+1D30D */
  0x0002,   /* U+1D30E */
  0x0002,   /* U+1D30F */
  0x0002,   /* U+1D310 */
  0x0002,   /* U+1D311 */
  0x0002,   /* U+1D312 */
  0x0002,   /* U+1D313 */
  0x0002,   /* U+1D314 */
  0x0002,   /* U+1D315 */
  0x0002,   /* U+1D316 */
  0x0002,   /* U+1D317 */
  0x0002,   /* U+1D318 */
  0x0002,   /* U+1D319 */
  0x0002,   /* U+1D31A */
  0x0002,   /* U+1D31B */
  0x0002,   /* U+1D31C */
  0x0002,   /* U+1D31D */
  0x0002,   /* U+1D31E */
  0x0002,   /* U+1D31F */
  0x0002,   /* U+1D320 */
  0x0002,   /* U+1D321 */
  0x0002,   /* U+1D322 */
  0x0002,   /* U+1D323 */
  0x0002,   /* U+1D324 */
  0x0002,   /* U+1D325 */
  0x0002,   /* U+1D326 */
  0x0002,   /* U+1D327 */
  0x0002,   /* U+1D328 */
  0x0002,   /* U+1D329 */
  0x0002,   /* U+1D32A */
  0x0002,   /* U+1D32B */
  0x0002,   /* U+1D32C */
  0x0002,   /* U+1D32D */
  0x0002,   /* U+1D32E */
  0x0002,   /* U+1D32F */
  0x0002,   /* U+1D330 */
  0x0002,   /* U+1D331 */
  0x0002,   /* U+1D332 */
  0x0002,   /* U+1D333 */
  0x0002,   /* U+1D334 */
  0x0002,   /* U+1D335 */
  0x0002,   /* U+1D336 */
  0x0002,   /* U+1D337 */
  0x0002,   /* U+1D338 */
  0x0002,   /* U+1D339 */
  0x0002,   /* U+1D33A */
  0x0002,   /* U+1D33B */
  0x0002,   /* U+1D33C */
  0x0002,   /* U+1D33D */
  0x0002,   /* U+1D33E */
  0x0002,   /* U+1D33F */
  0x0002,   /* U+1D340 */
  0x0002,   /* U+1D341 */
  0x0002,   /* U+1D342 */
  0x0002,   /* U+1D343 */
  0x0002,   /* U+1D344 */
  0x0002,   /* U+1D345 */
  0x0002,   /* U+1D346 */
  0x0002,   /* U+1D347 */
  0x0002,   /* U+1D348 */
  0x0002,   /* U+1D349 */
  0x0002,   /* U+1D34A */
  0x0002,   /* U+1D34B */
  0x0002,   /* U+1D34C */
  0x0002,   /* U+1D34D */
  0x0002,   /* U+1D34E */
  0x0002,   /* U+1D34F */
  0x0002,   /* U+1D350 */
  0x0002,   /* U+1D351 */
  0x0002,   /* U+1D352 */
  0x0002,   /* U+1D353 */
  0x0002,   /* U+1D354 */
  0x0002,   /* U+1D355 */
  0x0002,   /* U+1D356 */
  0x0002,   /* U+1D357 */
  0x0002,   /* U+1D358 */
  0x0002,   /* U+1D359 */
  0x0002,   /* U+1D35A */
  0x0002,   /* U+1D35B */
  0x0002,   /* U+1D35C */
  0x0002,   /* U+1D35D */
  0x0002,   /* U+1D35E */
  0x0002,   /* U+1D35F */
  0x0002,   /* U+1D360 */
  0x0002,   /* U+1D361 */
  0x0002,   /* U+1D362 */
  0x0002,   /* U+1D363 */
  0x0002,   /* U+1D364 */
  0x0002,   /* U+1D365 */
  0x0002,   /* U+1D366 */
  0x0002,   /* U+1D367 */
  0x0002,   /* U+1D368 */
  0x0002,   /* U+1D369 */
  0x0002,   /* U+1D36A */
  0x0002,   /* U+1D36B */
  0x0002,   /* U+1D36C */
  0x0002,   /* U+1D36D */
  0x0002,   /* U+1D36E */
  0x0002,   /* U+1D36F */
  0x0002,   /* U+1D370 */
  0x0002,   /* U+1D371 */
  0x0002,   /* U+1D372 */
  0x0002,   /* U+1D373 */
  0x0002,   /* U+1D374 */
  0x0002,   /* U+1D375 */
  0x0002,   /* U+1D376 */
  0x0002,   /* U+1D377 */
  0x0002,   /* U+1D378 */
  0x0002,   /* U+1D379 */
  0x0002,   /* U+1D37A */
  0x0002,   /* U+1D37B */
  0x0002,   /* U+1D37C */
  0x0002,   /* U+1D37D */
  0x0002,   /* U+1D37E */
  0x0002,   /* U+1D37F */
  0x0002,   /* U+1D380 */
  0x0002,   /* U+1D381 */
  0x0002,   /* U+1D382 */
  0x0002,   /* U+1D383 */
  0x0002,   /* U+1D384 */
  0x0002,   /* U+1D385 */
  0x0002,   /* U+1D386 */
  0x0002,   /* U+1D387 */
  0x0002,   /* U+1D388 */
  0x0002,   /* U+1D389 */
  0x0002,   /* U+1D38A */
  0x0002,   /* U+1D38B */
  0x0002,   /* U+1D38C */
  0x0002,   /* U+1D38D */
  0x0002,   /* U+1D38E */
  0x0002,   /* U+1D38F */
  0x0002,   /* U+1D390 */
  0x0002,   /* U+1D391 */
  0x0002,   /* U+1D392 */
  0x0002,   /* U+1D393 */
  0x0002,   /* U+1D394 */
  0x0002,   /* U+1D395 */
  0x0002,   /* U+1D396 */
  0x0002,   /* U+1D397 */
  0x0002,   /* U+1D398 */
  0x0002,   /* U+1D399 */
  0x0002,   /* U+1D39A */
  0x0002,   /* U+1D39B */
  0x0002,   /* U+1D39C */
  0x0002,   /* U+1D39D */
  0x0002,   /* U+1D39E */
  0x0002,   /* U+1D39F */
  0x0002,   /* U+1D3A0 */
  0x0002,   /* U+1D3A1 */
  0x0002,   /* U+1D3A2 */
  0x0002,   /* U+1D3A3 */
  0x0002,   /* U+1D3A4 */
  0x0002,   /* U+1D3A5 */
  0x0002,   /* U+1D3A6 */
  0x0002,   /* U+1D3A7 */
  0x0002,   /* U+1D3A8 */
  0x0002,   /* U+1D3A9 */
  0x0002,   /* U+1D3AA */
  0x0002,   /* U+1D3AB */
  0x0002,   /* U+1D3AC */
  0x0002,   /* U+1D3AD */
  0x0002,   /* U+1D3AE */
  0x0002,   /* U+1D3AF */
  0x0002,   /* U+1D3B0 */
  0x0002,   /* U+1D3B1 */
  0x0002,   /* U+1D3B2 */
  0x0002,   /* U+1D3B3 */
  0x0002,   /* U+1D3B4 */
  0x0002,   /* U+1D3B5 */
  0x0002,   /* U+1D3B6 */
  0x0002,   /* U+1D3B7 */
  0x0002,   /* U+1D3B8 */
  0x0002,   /* U+1D3B9 */
  0x0002,   /* U+1D3BA */
  0x0002,   /* U+1D3BB */
  0x0002,   /* U+1D3BC */
  0x0002,   /* U+1D3BD */
  0x0002,   /* U+1D3BE */
  0x0002,   /* U+1D3BF */
  0x0002,   /* U+1D3C0 */
  0x0002,   /* U+1D3C1 */
  0x0002,   /* U+1D3C2 */
  0x0002,   /* U+1D3C3 */
  0x0002,   /* U+1D3C4 */
  0x0002,   /* U+1D3C5 */
  0x0002,   /* U+1D3C6 */
  0x0002,   /* U+1D3C7 */
  0x0002,   /* U+1D3C8 */
  0x0002,   /* U+1D3C9 */
  0x0002,   /* U+1D3CA */
  0x0002,   /* U+1D3CB */
  0x0002,   /* U+1D3CC */
  0x0002,   /* U+1D3CD */
  0x0002,   /* U+1D3CE */
  0x0002,   /* U+1D3CF */
  0x0002,   /* U+1D3D0 */
  0x0002,   /* U+1D3D1 */
  0x0002,   /* U+1D3D2 */
  0x0002,   /* U+1D3D3 */
  0x0002,   /* U+1D3D4 */
  0x0002,   /* U+1D3D5 */
  0x0002,   /* U+1D3D6 */
  0x0002,   /* U+1D3D7 */
  0x0002,   /* U+1D3D8 */
  0x0002,   /* U+1D3D9 */
  0x0002,   /* U+1D3DA */
  0x0002,   /* U+1D3DB */
  0x0002,   /* U+1D3DC */
  0x0002,   /* U+1D3DD */
  0x0002,   /* U+1D3DE */
  0x0002,   /* U+1D3DF */
  0x0002,   /* U+1D3E0 */
  0x0002,   /* U+1D3E1 */
  0x0002,   /* U+1D3E2 */
  0x0002,   /* U+1D3E3 */
  0x0002,   /* U+1D3E4 */
  0x0002,   /* U+1D3E5 */
  0x0002,   /* U+1D3E6 */
  0x0002,   /* U+1D3E7 */
  0x0002,   /* U+1D3E8 */
  0x0002,   /* U+1D3E9 */
  0x0002,   /* U+1D3EA */
  0x0002,   /* U+1D3EB */
  0x0002,   /* U+1D3EC */
  0x0002,   /* U+1D3ED */
  0x0002,   /* U+1D3EE */
  0x0002,   /* U+1D3EF */
  0x0002,   /* U+1D3F0 */
  0x0002,   /* U+1D3F1 */
  0x0002,   /* U+1D3F2 */
  0x0002,   /* U+1D3F3 */
  0x0002,   /* U+1D3F4 */
  0x0002,   /* U+1D3F5 */
  0x0002,   /* U+1D3F6 */
  0x0002,   /* U+1D3F7 */
  0x0002,   /* U+1D3F8 */
  0x0002,   /* U+1D3F9 */
  0x0002,   /* U+1D3FA */
  0x0002,   /* U+1D3FB */
  0x0002,   /* U+1D3FC */
  0x0002,   /* U+1D3FD */
  0x0002,   /* U+1D3FE */
  0x0002,   /* U+1D3FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D300 */
  0x0000,   /* U+1D301 */
  0x0000,   /* U+1D302 */
  0x0000,   /* U+1D303 */
  0x0000,   /* U+1D304 */
  0x0000,   /* U+1D305 */
  0x0000,   /* U+1D306 */
  0x0000,   /* U+1D307 */
  0x0000,   /* U+1D308 */
  0x0000,   /* U+1D309 */
  0x0000,   /* U+1D30A */
  0x0000,   /* U+1D30B */
  0x0000,   /* U+1D30C */
  0x0000,   /* U+1D30D */
  0x0000,   /* U+1D30E */
  0x0000,   /* U+1D30F */
  0x0000,   /* U+1D310 */
  0x0000,   /* U+1D311 */
  0x0000,   /* U+1D312 */
  0x0000,   /* U+1D313 */
  0x0000,   /* U+1D314 */
  0x0000,   /* U+1D315 */
  0x0000,   /* U+1D316 */
  0x0000,   /* U+1D317 */
  0x0000,   /* U+1D318 */
  0x0000,   /* U+1D319 */
  0x0000,   /* U+1D31A */
  0x0000,   /* U+1D31B */
  0x0000,   /* U+1D31C */
  0x0000,   /* U+1D31D */
  0x0000,   /* U+1D31E */
  0x0000,   /* U+1D31F */
  0x0000,   /* U+1D320 */
  0x0000,   /* U+1D321 */
  0x0000,   /* U+1D322 */
  0x0000,   /* U+1D323 */
  0x0000,   /* U+1D324 */
  0x0000,   /* U+1D325 */
  0x0000,   /* U+1D326 */
  0x0000,   /* U+1D327 */
  0x0000,   /* U+1D328 */
  0x0000,   /* U+1D329 */
  0x0000,   /* U+1D32A */
  0x0000,   /* U+1D32B */
  0x0000,   /* U+1D32C */
  0x0000,   /* U+1D32D */
  0x0000,   /* U+1D32E */
  0x0000,   /* U+1D32F */
  0x0000,   /* U+1D330 */
  0x0000,   /* U+1D331 */
  0x0000,   /* U+1D332 */
  0x0000,   /* U+1D333 */
  0x0000,   /* U+1D334 */
  0x0000,   /* U+1D335 */
  0x0000,   /* U+1D336 */
  0x0000,   /* U+1D337 */
  0x0000,   /* U+1D338 */
  0x0000,   /* U+1D339 */
  0x0000,   /* U+1D33A */
  0x0000,   /* U+1D33B */
  0x0000,   /* U+1D33C */
  0x0000,   /* U+1D33D */
  0x0000,   /* U+1D33E */
  0x0000,   /* U+1D33F */
  0x0000,   /* U+1D340 */
  0x0000,   /* U+1D341 */
  0x0000,   /* U+1D342 */
  0x0000,   /* U+1D343 */
  0x0000,   /* U+1D344 */
  0x0000,   /* U+1D345 */
  0x0000,   /* U+1D346 */
  0x0000,   /* U+1D347 */
  0x0000,   /* U+1D348 */
  0x0000,   /* U+1D349 */
  0x0000,   /* U+1D34A */
  0x0000,   /* U+1D34B */
  0x0000,   /* U+1D34C */
  0x0000,   /* U+1D34D */
  0x0000,   /* U+1D34E */
  0x0000,   /* U+1D34F */
  0x0000,   /* U+1D350 */
  0x0000,   /* U+1D351 */
  0x0000,   /* U+1D352 */
  0x0000,   /* U+1D353 */
  0x0000,   /* U+1D354 */
  0x0000,   /* U+1D355 */
  0x0000,   /* U+1D356 */
  0xD357,   /* U+1D357 */
  0xD358,   /* U+1D358 */
  0xD359,   /* U+1D359 */
  0xD35A,   /* U+1D35A */
  0xD35B,   /* U+1D35B */
  0xD35C,   /* U+1D35C */
  0xD35D,   /* U+1D35D */
  0xD35E,   /* U+1D35E */
  0xD35F,   /* U+1D35F */
  0x0000,   /* U+1D360 */
  0x0000,   /* U+1D361 */
  0x0000,   /* U+1D362 */
  0x0000,   /* U+1D363 */
  0x0000,   /* U+1D364 */
  0x0000,   /* U+1D365 */
  0x0000,   /* U+1D366 */
  0x0000,   /* U+1D367 */
  0x0000,   /* U+1D368 */
  0x0000,   /* U+1D369 */
  0x0000,   /* U+1D36A */
  0x0000,   /* U+1D36B */
  0x0000,   /* U+1D36C */
  0x0000,   /* U+1D36D */
  0x0000,   /* U+1D36E */
  0x0000,   /* U+1D36F */
  0x0000,   /* U+1D370 */
  0x0000,   /* U+1D371 */
  0xD372,   /* U+1D372 */
  0xD373,   /* U+1D373 */
  0xD374,   /* U+1D374 */
  0xD375,   /* U+1D375 */
  0xD376,   /* U+1D376 */
  0xD377,   /* U+1D377 */
  0xD378,   /* U+1D378 */
  0xD379,   /* U+1D379 */
  0xD37A,   /* U+1D37A */
  0xD37B,   /* U+1D37B */
  0xD37C,   /* U+1D37C */
  0xD37D,   /* U+1D37D */
  0xD37E,   /* U+1D37E */
  0xD37F,   /* U+1D37F */
  0xD380,   /* U+1D380 */
  0xD381,   /* U+1D381 */
  0xD382,   /* U+1D382 */
  0xD383,   /* U+1D383 */
  0xD384,   /* U+1D384 */
  0xD385,   /* U+1D385 */
  0xD386,   /* U+1D386 */
  0xD387,   /* U+1D387 */
  0xD388,   /* U+1D388 */
  0xD389,   /* U+1D389 */
  0xD38A,   /* U+1D38A */
  0xD38B,   /* U+1D38B */
  0xD38C,   /* U+1D38C */
  0xD38D,   /* U+1D38D */
  0xD38E,   /* U+1D38E */
  0xD38F,   /* U+1D38F */
  0xD390,   /* U+1D390 */
  0xD391,   /* U+1D391 */
  0xD392,   /* U+1D392 */
  0xD393,   /* U+1D393 */
  0xD394,   /* U+1D394 */
  0xD395,   /* U+1D395 */
  0xD396,   /* U+1D396 */
  0xD397,   /* U+1D397 */
  0xD398,   /* U+1D398 */
  0xD399,   /* U+1D399 */
  0xD39A,   /* U+1D39A */
  0xD39B,   /* U+1D39B */
  0xD39C,   /* U+1D39C */
  0xD39D,   /* U+1D39D */
  0xD39E,   /* U+1D39E */
  0xD39F,   /* U+1D39F */
  0xD3A0,   /* U+1D3A0 */
  0xD3A1,   /* U+1D3A1 */
  0xD3A2,   /* U+1D3A2 */
  0xD3A3,   /* U+1D3A3 */
  0xD3A4,   /* U+1D3A4 */
  0xD3A5,   /* U+1D3A5 */
  0xD3A6,   /* U+1D3A6 */
  0xD3A7,   /* U+1D3A7 */
  0xD3A8,   /* U+1D3A8 */
  0xD3A9,   /* U+1D3A9 */
  0xD3AA,   /* U+1D3AA */
  0xD3AB,   /* U+1D3AB */
  0xD3AC,   /* U+1D3AC */
  0xD3AD,   /* U+1D3AD */
  0xD3AE,   /* U+1D3AE */
  0xD3AF,   /* U+1D3AF */
  0xD3B0,   /* U+1D3B0 */
  0xD3B1,   /* U+1D3B1 */
  0xD3B2,   /* U+1D3B2 */
  0xD3B3,   /* U+1D3B3 */
  0xD3B4,   /* U+1D3B4 */
  0xD3B5,   /* U+1D3B5 */
  0xD3B6,   /* U+1D3B6 */
  0xD3B7,   /* U+1D3B7 */
  0xD3B8,   /* U+1D3B8 */
  0xD3B9,   /* U+1D3B9 */
  0xD3BA,   /* U+1D3BA */
  0xD3BB,   /* U+1D3BB */
  0xD3BC,   /* U+1D3BC */
  0xD3BD,   /* U+1D3BD */
  0xD3BE,   /* U+1D3BE */
  0xD3BF,   /* U+1D3BF */
  0xD3C0,   /* U+1D3C0 */
  0xD3C1,   /* U+1D3C1 */
  0xD3C2,   /* U+1D3C2 */
  0xD3C3,   /* U+1D3C3 */
  0xD3C4,   /* U+1D3C4 */
  0xD3C5,   /* U+1D3C5 */
  0xD3C6,   /* U+1D3C6 */
  0xD3C7,   /* U+1D3C7 */
  0xD3C8,   /* U+1D3C8 */
  0xD3C9,   /* U+1D3C9 */
  0xD3CA,   /* U+1D3CA */
  0xD3CB,   /* U+1D3CB */
  0xD3CC,   /* U+1D3CC */
  0xD3CD,   /* U+1D3CD */
  0xD3CE,   /* U+1D3CE */
  0xD3CF,   /* U+1D3CF */
  0xD3D0,   /* U+1D3D0 */
  0xD3D1,   /* U+1D3D1 */
  0xD3D2,   /* U+1D3D2 */
  0xD3D3,   /* U+1D3D3 */
  0xD3D4,   /* U+1D3D4 */
  0xD3D5,   /* U+1D3D5 */
  0xD3D6,   /* U+1D3D6 */
  0xD3D7,   /* U+1D3D7 */
  0xD3D8,   /* U+1D3D8 */
  0xD3D9,   /* U+1D3D9 */
  0xD3DA,   /* U+1D3DA */
  0xD3DB,   /* U+1D3DB */
  0xD3DC,   /* U+1D3DC */
  0xD3DD,   /* U+1D3DD */
  0xD3DE,   /* U+1D3DE */
  0xD3DF,   /* U+1D3DF */
  0xD3E0,   /* U+1D3E0 */
  0xD3E1,   /* U+1D3E1 */
  0xD3E2,   /* U+1D3E2 */
  0xD3E3,   /* U+1D3E3 */
  0xD3E4,   /* U+1D3E4 */
  0xD3E5,   /* U+1D3E5 */
  0xD3E6,   /* U+1D3E6 */
  0xD3E7,   /* U+1D3E7 */
  0xD3E8,   /* U+1D3E8 */
  0xD3E9,   /* U+1D3E9 */
  0xD3EA,   /* U+1D3EA */
  0xD3EB,   /* U+1D3EB */
  0xD3EC,   /* U+1D3EC */
  0xD3ED,   /* U+1D3ED */
  0xD3EE,   /* U+1D3EE */
  0xD3EF,   /* U+1D3EF */
  0xD3F0,   /* U+1D3F0 */
  0xD3F1,   /* U+1D3F1 */
  0xD3F2,   /* U+1D3F2 */
  0xD3F3,   /* U+1D3F3 */
  0xD3F4,   /* U+1D3F4 */
  0xD3F5,   /* U+1D3F5 */
  0xD3F6,   /* U+1D3F6 */
  0xD3F7,   /* U+1D3F7 */
  0xD3F8,   /* U+1D3F8 */
  0xD3F9,   /* U+1D3F9 */
  0xD3FA,   /* U+1D3FA */
  0xD3FB,   /* U+1D3FB */
  0xD3FC,   /* U+1D3FC */
  0xD3FD,   /* U+1D3FD */
  0xD3FE,   /* U+1D3FE */
  0xD3FF,   /* U+1D3FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D300 */
  0x0000,   /* U+1D301 */
  0x0000,   /* U+1D302 */
  0x0000,   /* U+1D303 */
  0x0000,   /* U+1D304 */
  0x0000,   /* U+1D305 */
  0x0000,   /* U+1D306 */
  0x0000,   /* U+1D307 */
  0x0000,   /* U+1D308 */
  0x0000,   /* U+1D309 */
  0x0000,   /* U+1D30A */
  0x0000,   /* U+1D30B */
  0x0000,   /* U+1D30C */
  0x0000,   /* U+1D30D */
  0x0000,   /* U+1D30E */
  0x0000,   /* U+1D30F */
  0x0000,   /* U+1D310 */
  0x0000,   /* U+1D311 */
  0x0000,   /* U+1D312 */
  0x0000,   /* U+1D313 */
  0x0000,   /* U+1D314 */
  0x0000,   /* U+1D315 */
  0x0000,   /* U+1D316 */
  0x0000,   /* U+1D317 */
  0x0000,   /* U+1D318 */
  0x0000,   /* U+1D319 */
  0x0000,   /* U+1D31A */
  0x0000,   /* U+1D31B */
  0x0000,   /* U+1D31C */
  0x0000,   /* U+1D31D */
  0x0000,   /* U+1D31E */
  0x0000,   /* U+1D31F */
  0x0000,   /* U+1D320 */
  0x0000,   /* U+1D321 */
  0x0000,   /* U+1D322 */
  0x0000,   /* U+1D323 */
  0x0000,   /* U+1D324 */
  0x0000,   /* U+1D325 */
  0x0000,   /* U+1D326 */
  0x0000,   /* U+1D327 */
  0x0000,   /* U+1D328 */
  0x0000,   /* U+1D329 */
  0x0000,   /* U+1D32A */
  0x0000,   /* U+1D32B */
  0x0000,   /* U+1D32C */
  0x0000,   /* U+1D32D */
  0x0000,   /* U+1D32E */
  0x0000,   /* U+1D32F */
  0x0000,   /* U+1D330 */
  0x0000,   /* U+1D331 */
  0x0000,   /* U+1D332 */
  0x0000,   /* U+1D333 */
  0x0000,   /* U+1D334 */
  0x0000,   /* U+1D335 */
  0x0000,   /* U+1D336 */
  0x0000,   /* U+1D337 */
  0x0000,   /* U+1D338 */
  0x0000,   /* U+1D339 */
  0x0000,   /* U+1D33A */
  0x0000,   /* U+1D33B */
  0x0000,   /* U+1D33C */
  0x0000,   /* U+1D33D */
  0x0000,   /* U+1D33E */
  0x0000,   /* U+1D33F */
  0x0000,   /* U+1D340 */
  0x0000,   /* U+1D341 */
  0x0000,   /* U+1D342 */
  0x0000,   /* U+1D343 */
  0x0000,   /* U+1D344 */
  0x0000,   /* U+1D345 */
  0x0000,   /* U+1D346 */
  0x0000,   /* U+1D347 */
  0x0000,   /* U+1D348 */
  0x0000,   /* U+1D349 */
  0x0000,   /* U+1D34A */
  0x0000,   /* U+1D34B */
  0x0000,   /* U+1D34C */
  0x0000,   /* U+1D34D */
  0x0000,   /* U+1D34E */
  0x0000,   /* U+1D34F */
  0x0000,   /* U+1D350 */
  0x0000,   /* U+1D351 */
  0x0000,   /* U+1D352 */
  0x0000,   /* U+1D353 */
  0x0000,   /* U+1D354 */
  0x0000,   /* U+1D355 */
  0x0000,   /* U+1D356 */
  0x0000,   /* U+1D357 */
  0x0000,   /* U+1D358 */
  0x0000,   /* U+1D359 */
  0x0000,   /* U+1D35A */
  0x0000,   /* U+1D35B */
  0x0000,   /* U+1D35C */
  0x0000,   /* U+1D35D */
  0x0000,   /* U+1D35E */
  0x0000,   /* U+1D35F */
  0x0000,   /* U+1D360 */
  0x0000,   /* U+1D361 */
  0x0000,   /* U+1D362 */
  0x0000,   /* U+1D363 */
  0x0000,   /* U+1D364 */
  0x0000,   /* U+1D365 */
  0x0000,   /* U+1D366 */
  0x0000,   /* U+1D367 */
  0x0000,   /* U+1D368 */
  0x0000,   /* U+1D369 */
  0x0000,   /* U+1D36A */
  0x0000,   /* U+1D36B */
  0x0000,   /* U+1D36C */
  0x0000,   /* U+1D36D */
  0x0000,   /* U+1D36E */
  0x0000,   /* U+1D36F */
  0x0000,   /* U+1D370 */
  0x0000,   /* U+1D371 */
  0x0000,   /* U+1D372 */
  0x0000,   /* U+1D373 */
  0x0000,   /* U+1D374 */
  0x0000,   /* U+1D375 */
  0x0000,   /* U+1D376 */
  0x0000,   /* U+1D377 */
  0x0000,   /* U+1D378 */
  0x0000,   /* U+1D379 */
  0x0000,   /* U+1D37A */
  0x0000,   /* U+1D37B */
  0x0000,   /* U+1D37C */
  0x0000,   /* U+1D37D */
  0x0000,   /* U+1D37E */
  0x0000,   /* U+1D37F */
  0x0000,   /* U+1D380 */
  0x0000,   /* U+1D381 */
  0x0000,   /* U+1D382 */
  0x0000,   /* U+1D383 */
  0x0000,   /* U+1D384 */
  0x0000,   /* U+1D385 */
  0x0000,   /* U+1D386 */
  0x0000,   /* U+1D387 */
  0x0000,   /* U+1D388 */
  0x0000,   /* U+1D389 */
  0x0000,   /* U+1D38A */
  0x0000,   /* U+1D38B */
  0x0000,   /* U+1D38C */
  0x0000,   /* U+1D38D */
  0x0000,   /* U+1D38E */
  0x0000,   /* U+1D38F */
  0x0000,   /* U+1D390 */
  0x0000,   /* U+1D391 */
  0x0000,   /* U+1D392 */
  0x0000,   /* U+1D393 */
  0x0000,   /* U+1D394 */
  0x0000,   /* U+1D395 */
  0x0000,   /* U+1D396 */
  0x0000,   /* U+1D397 */
  0x0000,   /* U+1D398 */
  0x0000,   /* U+1D399 */
  0x0000,   /* U+1D39A */
  0x0000,   /* U+1D39B */
  0x0000,   /* U+1D39C */
  0x0000,   /* U+1D39D */
  0x0000,   /* U+1D39E */
  0x0000,   /* U+1D39F */
  0x0000,   /* U+1D3A0 */
  0x0000,   /* U+1D3A1 */
  0x0000,   /* U+1D3A2 */
  0x0000,   /* U+1D3A3 */
  0x0000,   /* U+1D3A4 */
  0x0000,   /* U+1D3A5 */
  0x0000,   /* U+1D3A6 */
  0x0000,   /* U+1D3A7 */
  0x0000,   /* U+1D3A8 */
  0x0000,   /* U+1D3A9 */
  0x0000,   /* U+1D3AA */
  0x0000,   /* U+1D3AB */
  0x0000,   /* U+1D3AC */
  0x0000,   /* U+1D3AD */
  0x0000,   /* U+1D3AE */
  0x0000,   /* U+1D3AF */
  0x0000,   /* U+1D3B0 */
  0x0000,   /* U+1D3B1 */
  0x0000,   /* U+1D3B2 */
  0x0000,   /* U+1D3B3 */
  0x0000,   /* U+1D3B4 */
  0x0000,   /* U+1D3B5 */
  0x0000,   /* U+1D3B6 */
  0x0000,   /* U+1D3B7 */
  0x0000,   /* U+1D3B8 */
  0x0000,   /* U+1D3B9 */
  0x0000,   /* U+1D3BA */
  0x0000,   /* U+1D3BB */
  0x0000,   /* U+1D3BC */
  0x0000,   /* U+1D3BD */
  0x0000,   /* U+1D3BE */
  0x0000,   /* U+1D3BF */
  0x0000,   /* U+1D3C0 */
  0x0000,   /* U+1D3C1 */
  0x0000,   /* U+1D3C2 */
  0x0000,   /* U+1D3C3 */
  0x0000,   /* U+1D3C4 */
  0x0000,   /* U+1D3C5 */
  0x0000,   /* U+1D3C6 */
  0x0000,   /* U+1D3C7 */
  0x0000,   /* U+1D3C8 */
  0x0000,   /* U+1D3C9 */
  0x0000,   /* U+1D3CA */
  0x0000,   /* U+1D3CB */
  0x0000,   /* U+1D3CC */
  0x0000,   /* U+1D3CD */
  0x0000,   /* U+1D3CE */
  0x0000,   /* U+1D3CF */
  0x0000,   /* U+1D3D0 */
  0x0000,   /* U+1D3D1 */
  0x0000,   /* U+1D3D2 */
  0x0000,   /* U+1D3D3 */
  0x0000,   /* U+1D3D4 */
  0x0000,   /* U+1D3D5 */
  0x0000,   /* U+1D3D6 */
  0x0000,   /* U+1D3D7 */
  0x0000,   /* U+1D3D8 */
  0x0000,   /* U+1D3D9 */
  0x0000,   /* U+1D3DA */
  0x0000,   /* U+1D3DB */
  0x0000,   /* U+1D3DC */
  0x0000,   /* U+1D3DD */
  0x0000,   /* U+1D3DE */
  0x0000,   /* U+1D3DF */
  0x0000,   /* U+1D3E0 */
  0x0000,   /* U+1D3E1 */
  0x0000,   /* U+1D3E2 */
  0x0000,   /* U+1D3E3 */
  0x0000,   /* U+1D3E4 */
  0x0000,   /* U+1D3E5 */
  0x0000,   /* U+1D3E6 */
  0x0000,   /* U+1D3E7 */
  0x0000,   /* U+1D3E8 */
  0x0000,   /* U+1D3E9 */
  0x0000,   /* U+1D3EA */
  0x0000,   /* U+1D3EB */
  0x0000,   /* U+1D3EC */
  0x0000,   /* U+1D3ED */
  0x0000,   /* U+1D3EE */
  0x0000,   /* U+1D3EF */
  0x0000,   /* U+1D3F0 */
  0x0000,   /* U+1D3F1 */
  0x0000,   /* U+1D3F2 */
  0x0000,   /* U+1D3F3 */
  0x0000,   /* U+1D3F4 */
  0x0000,   /* U+1D3F5 */
  0x0000,   /* U+1D3F6 */
  0x0000,   /* U+1D3F7 */
  0x0000,   /* U+1D3F8 */
  0x0000,   /* U+1D3F9 */
  0x0000,   /* U+1D3FA */
  0x0000,   /* U+1D3FB */
  0x0000,   /* U+1D3FC */
  0x0000,   /* U+1D3FD */
  0x0000,   /* U+1D3FE */
  0x0000,   /* U+1D3FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D300 */
  0x0000,   /* U+1D301 */
  0x0000,   /* U+1D302 */
  0x0000,   /* U+1D303 */
  0x0000,   /* U+1D304 */
  0x0000,   /* U+1D305 */
  0x0000,   /* U+1D306 */
  0x0000,   /* U+1D307 */
  0x0000,   /* U+1D308 */
  0x0000,   /* U+1D309 */
  0x0000,   /* U+1D30A */
  0x0000,   /* U+1D30B */
  0x0000,   /* U+1D30C */
  0x0000,   /* U+1D30D */
  0x0000,   /* U+1D30E */
  0x0000,   /* U+1D30F */
  0x0000,   /* U+1D310 */
  0x0000,   /* U+1D311 */
  0x0000,   /* U+1D312 */
  0x0000,   /* U+1D313 */
  0x0000,   /* U+1D314 */
  0x0000,   /* U+1D315 */
  0x0000,   /* U+1D316 */
  0x0000,   /* U+1D317 */
  0x0000,   /* U+1D318 */
  0x0000,   /* U+1D319 */
  0x0000,   /* U+1D31A */
  0x0000,   /* U+1D31B */
  0x0000,   /* U+1D31C */
  0x0000,   /* U+1D31D */
  0x0000,   /* U+1D31E */
  0x0000,   /* U+1D31F */
  0x0000,   /* U+1D320 */
  0x0000,   /* U+1D321 */
  0x0000,   /* U+1D322 */
  0x0000,   /* U+1D323 */
  0x0000,   /* U+1D324 */
  0x0000,   /* U+1D325 */
  0x0000,   /* U+1D326 */
  0x0000,   /* U+1D327 */
  0x0000,   /* U+1D328 */
  0x0000,   /* U+1D329 */
  0x0000,   /* U+1D32A */
  0x0000,   /* U+1D32B */
  0x0000,   /* U+1D32C */
  0x0000,   /* U+1D32D */
  0x0000,   /* U+1D32E */
  0x0000,   /* U+1D32F */
  0x0000,   /* U+1D330 */
  0x0000,   /* U+1D331 */
  0x0000,   /* U+1D332 */
  0x0000,   /* U+1D333 */
  0x0000,   /* U+1D334 */
  0x0000,   /* U+1D335 */
  0x0000,   /* U+1D336 */
  0x0000,   /* U+1D337 */
  0x0000,   /* U+1D338 */
  0x0000,   /* U+1D339 */
  0x0000,   /* U+1D33A */
  0x0000,   /* U+1D33B */
  0x0000,   /* U+1D33C */
  0x0000,   /* U+1D33D */
  0x0000,   /* U+1D33E */
  0x0000,   /* U+1D33F */
  0x0000,   /* U+1D340 */
  0x0000,   /* U+1D341 */
  0x0000,   /* U+1D342 */
  0x0000,   /* U+1D343 */
  0x0000,   /* U+1D344 */
  0x0000,   /* U+1D345 */
  0x0000,   /* U+1D346 */
  0x0000,   /* U+1D347 */
  0x0000,   /* U+1D348 */
  0x0000,   /* U+1D349 */
  0x0000,   /* U+1D34A */
  0x0000,   /* U+1D34B */
  0x0000,   /* U+1D34C */
  0x0000,   /* U+1D34D */
  0x0000,   /* U+1D34E */
  0x0000,   /* U+1D34F */
  0x0000,   /* U+1D350 */
  0x0000,   /* U+1D351 */
  0x0000,   /* U+1D352 */
  0x0000,   /* U+1D353 */
  0x0000,   /* U+1D354 */
  0x0000,   /* U+1D355 */
  0x0000,   /* U+1D356 */
  0x0000,   /* U+1D357 */
  0x0000,   /* U+1D358 */
  0x0000,   /* U+1D359 */
  0x0000,   /* U+1D35A */
  0x0000,   /* U+1D35B */
  0x0000,   /* U+1D35C */
  0x0000,   /* U+1D35D */
  0x0000,   /* U+1D35E */
  0x0000,   /* U+1D35F */
  0x0000,   /* U+1D360 */
  0x0000,   /* U+1D361 */
  0x0000,   /* U+1D362 */
  0x0000,   /* U+1D363 */
  0x0000,   /* U+1D364 */
  0x0000,   /* U+1D365 */
  0x0000,   /* U+1D366 */
  0x0000,   /* U+1D367 */
  0x0000,   /* U+1D368 */
  0x0000,   /* U+1D369 */
  0x0000,   /* U+1D36A */
  0x0000,   /* U+1D36B */
  0x0000,   /* U+1D36C */
  0x0000,   /* U+1D36D */
  0x0000,   /* U+1D36E */
  0x0000,   /* U+1D36F */
  0x0000,   /* U+1D370 */
  0x0000,   /* U+1D371 */
  0x0000,   /* U+1D372 */
  0x0000,   /* U+1D373 */
  0x0000,   /* U+1D374 */
  0x0000,   /* U+1D375 */
  0x0000,   /* U+1D376 */
  0x0000,   /* U+1D377 */
  0x0000,   /* U+1D378 */
  0x0000,   /* U+1D379 */
  0x0000,   /* U+1D37A */
  0x0000,   /* U+1D37B */
  0x0000,   /* U+1D37C */
  0x0000,   /* U+1D37D */
  0x0000,   /* U+1D37E */
  0x0000,   /* U+1D37F */
  0x0000,   /* U+1D380 */
  0x0000,   /* U+1D381 */
  0x0000,   /* U+1D382 */
  0x0000,   /* U+1D383 */
  0x0000,   /* U+1D384 */
  0x0000,   /* U+1D385 */
  0x0000,   /* U+1D386 */
  0x0000,   /* U+1D387 */
  0x0000,   /* U+1D388 */
  0x0000,   /* U+1D389 */
  0x0000,   /* U+1D38A */
  0x0000,   /* U+1D38B */
  0x0000,   /* U+1D38C */
  0x0000,   /* U+1D38D */
  0x0000,   /* U+1D38E */
  0x0000,   /* U+1D38F */
  0x0000,   /* U+1D390 */
  0x0000,   /* U+1D391 */
  0x0000,   /* U+1D392 */
  0x0000,   /* U+1D393 */
  0x0000,   /* U+1D394 */
  0x0000,   /* U+1D395 */
  0x0000,   /* U+1D396 */
  0x0000,   /* U+1D397 */
  0x0000,   /* U+1D398 */
  0x0000,   /* U+1D399 */
  0x0000,   /* U+1D39A */
  0x0000,   /* U+1D39B */
  0x0000,   /* U+1D39C */
  0x0000,   /* U+1D39D */
  0x0000,   /* U+1D39E */
  0x0000,   /* U+1D39F */
  0x0000,   /* U+1D3A0 */
  0x0000,   /* U+1D3A1 */
  0x0000,   /* U+1D3A2 */
  0x0000,   /* U+1D3A3 */
  0x0000,   /* U+1D3A4 */
  0x0000,   /* U+1D3A5 */
  0x0000,   /* U+1D3A6 */
  0x0000,   /* U+1D3A7 */
  0x0000,   /* U+1D3A8 */
  0x0000,   /* U+1D3A9 */
  0x0000,   /* U+1D3AA */
  0x0000,   /* U+1D3AB */
  0x0000,   /* U+1D3AC */
  0x0000,   /* U+1D3AD */
  0x0000,   /* U+1D3AE */
  0x0000,   /* U+1D3AF */
  0x0000,   /* U+1D3B0 */
  0x0000,   /* U+1D3B1 */
  0x0000,   /* U+1D3B2 */
  0x0000,   /* U+1D3B3 */
  0x0000,   /* U+1D3B4 */
  0x0000,   /* U+1D3B5 */
  0x0000,   /* U+1D3B6 */
  0x0000,   /* U+1D3B7 */
  0x0000,   /* U+1D3B8 */
  0x0000,   /* U+1D3B9 */
  0x0000,   /* U+1D3BA */
  0x0000,   /* U+1D3BB */
  0x0000,   /* U+1D3BC */
  0x0000,   /* U+1D3BD */
  0x0000,   /* U+1D3BE */
  0x0000,   /* U+1D3BF */
  0x0000,   /* U+1D3C0 */
  0x0000,   /* U+1D3C1 */
  0x0000,   /* U+1D3C2 */
  0x0000,   /* U+1D3C3 */
  0x0000,   /* U+1D3C4 */
  0x0000,   /* U+1D3C5 */
  0x0000,   /* U+1D3C6 */
  0x0000,   /* U+1D3C7 */
  0x0000,   /* U+1D3C8 */
  0x0000,   /* U+1D3C9 */
  0x0000,   /* U+1D3CA */
  0x0000,   /* U+1D3CB */
  0x0000,   /* U+1D3CC */
  0x0000,   /* U+1D3CD */
  0x0000,   /* U+1D3CE */
  0x0000,   /* U+1D3CF */
  0x0000,   /* U+1D3D0 */
  0x0000,   /* U+1D3D1 */
  0x0000,   /* U+1D3D2 */
  0x0000,   /* U+1D3D3 */
  0x0000,   /* U+1D3D4 */
  0x0000,   /* U+1D3D5 */
  0x0000,   /* U+1D3D6 */
  0x0000,   /* U+1D3D7 */
  0x0000,   /* U+1D3D8 */
  0x0000,   /* U+1D3D9 */
  0x0000,   /* U+1D3DA */
  0x0000,   /* U+1D3DB */
  0x0000,   /* U+1D3DC */
  0x0000,   /* U+1D3DD */
  0x0000,   /* U+1D3DE */
  0x0000,   /* U+1D3DF */
  0x0000,   /* U+1D3E0 */
  0x0000,   /* U+1D3E1 */
  0x0000,   /* U+1D3E2 */
  0x0000,   /* U+1D3E3 */
  0x0000,   /* U+1D3E4 */
  0x0000,   /* U+1D3E5 */
  0x0000,   /* U+1D3E6 */
  0x0000,   /* U+1D3E7 */
  0x0000,   /* U+1D3E8 */
  0x0000,   /* U+1D3E9 */
  0x0000,   /* U+1D3EA */
  0x0000,   /* U+1D3EB */
  0x0000,   /* U+1D3EC */
  0x0000,   /* U+1D3ED */
  0x0000,   /* U+1D3EE */
  0x0000,   /* U+1D3EF */
  0x0000,   /* U+1D3F0 */
  0x0000,   /* U+1D3F1 */
  0x0000,   /* U+1D3F2 */
  0x0000,   /* U+1D3F3 */
  0x0000,   /* U+1D3F4 */
  0x0000,   /* U+1D3F5 */
  0x0000,   /* U+1D3F6 */
  0x0000,   /* U+1D3F7 */
  0x0000,   /* U+1D3F8 */
  0x0000,   /* U+1D3F9 */
  0x0000,   /* U+1D3FA */
  0x0000,   /* U+1D3FB */
  0x0000,   /* U+1D3FC */
  0x0000,   /* U+1D3FD */
  0x0000,   /* U+1D3FE */
  0x0000,   /* U+1D3FF */
};

uint16 uca900_p1D4[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  2, 2, 1, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C47,   /* U+1D400 */
  0x1C60,   /* U+1D401 */
  0x1C7A,   /* U+1D402 */
  0x1C8F,   /* U+1D403 */
  0x1CAA,   /* U+1D404 */
  0x1CE5,   /* U+1D405 */
  0x1CF4,   /* U+1D406 */
  0x1D18,   /* U+1D407 */
  0x1D32,   /* U+1D408 */
  0x1D4C,   /* U+1D409 */
  0x1D65,   /* U+1D40A */
  0x1D77,   /* U+1D40B */
  0x1DAA,   /* U+1D40C */
  0x1DB9,   /* U+1D40D */
  0x1DDD,   /* U+1D40E */
  0x1E0C,   /* U+1D40F */
  0x1E21,   /* U+1D410 */
  0x1E33,   /* U+1D411 */
  0x1E71,   /* U+1D412 */
  0x1E95,   /* U+1D413 */
  0x1EB5,   /* U+1D414 */
  0x1EE3,   /* U+1D415 */
  0x1EF5,   /* U+1D416 */
  0x1EFF,   /* U+1D417 */
  0x1F0B,   /* U+1D418 */
  0x1F21,   /* U+1D419 */
  0x1C47,   /* U+1D41A */
  0x1C60,   /* U+1D41B */
  0x1C7A,   /* U+1D41C */
  0x1C8F,   /* U+1D41D */
  0x1CAA,   /* U+1D41E */
  0x1CE5,   /* U+1D41F */
  0x1CF4,   /* U+1D420 */
  0x1D18,   /* U+1D421 */
  0x1D32,   /* U+1D422 */
  0x1D4C,   /* U+1D423 */
  0x1D65,   /* U+1D424 */
  0x1D77,   /* U+1D425 */
  0x1DAA,   /* U+1D426 */
  0x1DB9,   /* U+1D427 */
  0x1DDD,   /* U+1D428 */
  0x1E0C,   /* U+1D429 */
  0x1E21,   /* U+1D42A */
  0x1E33,   /* U+1D42B */
  0x1E71,   /* U+1D42C */
  0x1E95,   /* U+1D42D */
  0x1EB5,   /* U+1D42E */
  0x1EE3,   /* U+1D42F */
  0x1EF5,   /* U+1D430 */
  0x1EFF,   /* U+1D431 */
  0x1F0B,   /* U+1D432 */
  0x1F21,   /* U+1D433 */
  0x1C47,   /* U+1D434 */
  0x1C60,   /* U+1D435 */
  0x1C7A,   /* U+1D436 */
  0x1C8F,   /* U+1D437 */
  0x1CAA,   /* U+1D438 */
  0x1CE5,   /* U+1D439 */
  0x1CF4,   /* U+1D43A */
  0x1D18,   /* U+1D43B */
  0x1D32,   /* U+1D43C */
  0x1D4C,   /* U+1D43D */
  0x1D65,   /* U+1D43E */
  0x1D77,   /* U+1D43F */
  0x1DAA,   /* U+1D440 */
  0x1DB9,   /* U+1D441 */
  0x1DDD,   /* U+1D442 */
  0x1E0C,   /* U+1D443 */
  0x1E21,   /* U+1D444 */
  0x1E33,   /* U+1D445 */
  0x1E71,   /* U+1D446 */
  0x1E95,   /* U+1D447 */
  0x1EB5,   /* U+1D448 */
  0x1EE3,   /* U+1D449 */
  0x1EF5,   /* U+1D44A */
  0x1EFF,   /* U+1D44B */
  0x1F0B,   /* U+1D44C */
  0x1F21,   /* U+1D44D */
  0x1C47,   /* U+1D44E */
  0x1C60,   /* U+1D44F */
  0x1C7A,   /* U+1D450 */
  0x1C8F,   /* U+1D451 */
  0x1CAA,   /* U+1D452 */
  0x1CE5,   /* U+1D453 */
  0x1CF4,   /* U+1D454 */
  0xFBC3,   /* U+1D455 */
  0x1D32,   /* U+1D456 */
  0x1D4C,   /* U+1D457 */
  0x1D65,   /* U+1D458 */
  0x1D77,   /* U+1D459 */
  0x1DAA,   /* U+1D45A */
  0x1DB9,   /* U+1D45B */
  0x1DDD,   /* U+1D45C */
  0x1E0C,   /* U+1D45D */
  0x1E21,   /* U+1D45E */
  0x1E33,   /* U+1D45F */
  0x1E71,   /* U+1D460 */
  0x1E95,   /* U+1D461 */
  0x1EB5,   /* U+1D462 */
  0x1EE3,   /* U+1D463 */
  0x1EF5,   /* U+1D464 */
  0x1EFF,   /* U+1D465 */
  0x1F0B,   /* U+1D466 */
  0x1F21,   /* U+1D467 */
  0x1C47,   /* U+1D468 */
  0x1C60,   /* U+1D469 */
  0x1C7A,   /* U+1D46A */
  0x1C8F,   /* U+1D46B */
  0x1CAA,   /* U+1D46C */
  0x1CE5,   /* U+1D46D */
  0x1CF4,   /* U+1D46E */
  0x1D18,   /* U+1D46F */
  0x1D32,   /* U+1D470 */
  0x1D4C,   /* U+1D471 */
  0x1D65,   /* U+1D472 */
  0x1D77,   /* U+1D473 */
  0x1DAA,   /* U+1D474 */
  0x1DB9,   /* U+1D475 */
  0x1DDD,   /* U+1D476 */
  0x1E0C,   /* U+1D477 */
  0x1E21,   /* U+1D478 */
  0x1E33,   /* U+1D479 */
  0x1E71,   /* U+1D47A */
  0x1E95,   /* U+1D47B */
  0x1EB5,   /* U+1D47C */
  0x1EE3,   /* U+1D47D */
  0x1EF5,   /* U+1D47E */
  0x1EFF,   /* U+1D47F */
  0x1F0B,   /* U+1D480 */
  0x1F21,   /* U+1D481 */
  0x1C47,   /* U+1D482 */
  0x1C60,   /* U+1D483 */
  0x1C7A,   /* U+1D484 */
  0x1C8F,   /* U+1D485 */
  0x1CAA,   /* U+1D486 */
  0x1CE5,   /* U+1D487 */
  0x1CF4,   /* U+1D488 */
  0x1D18,   /* U+1D489 */
  0x1D32,   /* U+1D48A */
  0x1D4C,   /* U+1D48B */
  0x1D65,   /* U+1D48C */
  0x1D77,   /* U+1D48D */
  0x1DAA,   /* U+1D48E */
  0x1DB9,   /* U+1D48F */
  0x1DDD,   /* U+1D490 */
  0x1E0C,   /* U+1D491 */
  0x1E21,   /* U+1D492 */
  0x1E33,   /* U+1D493 */
  0x1E71,   /* U+1D494 */
  0x1E95,   /* U+1D495 */
  0x1EB5,   /* U+1D496 */
  0x1EE3,   /* U+1D497 */
  0x1EF5,   /* U+1D498 */
  0x1EFF,   /* U+1D499 */
  0x1F0B,   /* U+1D49A */
  0x1F21,   /* U+1D49B */
  0x1C47,   /* U+1D49C */
  0xFBC3,   /* U+1D49D */
  0x1C7A,   /* U+1D49E */
  0x1C8F,   /* U+1D49F */
  0xFBC3,   /* U+1D4A0 */
  0xFBC3,   /* U+1D4A1 */
  0x1CF4,   /* U+1D4A2 */
  0xFBC3,   /* U+1D4A3 */
  0xFBC3,   /* U+1D4A4 */
  0x1D4C,   /* U+1D4A5 */
  0x1D65,   /* U+1D4A6 */
  0xFBC3,   /* U+1D4A7 */
  0xFBC3,   /* U+1D4A8 */
  0x1DB9,   /* U+1D4A9 */
  0x1DDD,   /* U+1D4AA */
  0x1E0C,   /* U+1D4AB */
  0x1E21,   /* U+1D4AC */
  0xFBC3,   /* U+1D4AD */
  0x1E71,   /* U+1D4AE */
  0x1E95,   /* U+1D4AF */
  0x1EB5,   /* U+1D4B0 */
  0x1EE3,   /* U+1D4B1 */
  0x1EF5,   /* U+1D4B2 */
  0x1EFF,   /* U+1D4B3 */
  0x1F0B,   /* U+1D4B4 */
  0x1F21,   /* U+1D4B5 */
  0x1C47,   /* U+1D4B6 */
  0x1C60,   /* U+1D4B7 */
  0x1C7A,   /* U+1D4B8 */
  0x1C8F,   /* U+1D4B9 */
  0xFBC3,   /* U+1D4BA */
  0x1CE5,   /* U+1D4BB */
  0xFBC3,   /* U+1D4BC */
  0x1D18,   /* U+1D4BD */
  0x1D32,   /* U+1D4BE */
  0x1D4C,   /* U+1D4BF */
  0x1D65,   /* U+1D4C0 */
  0x1D77,   /* U+1D4C1 */
  0x1DAA,   /* U+1D4C2 */
  0x1DB9,   /* U+1D4C3 */
  0xFBC3,   /* U+1D4C4 */
  0x1E0C,   /* U+1D4C5 */
  0x1E21,   /* U+1D4C6 */
  0x1E33,   /* U+1D4C7 */
  0x1E71,   /* U+1D4C8 */
  0x1E95,   /* U+1D4C9 */
  0x1EB5,   /* U+1D4CA */
  0x1EE3,   /* U+1D4CB */
  0x1EF5,   /* U+1D4CC */
  0x1EFF,   /* U+1D4CD */
  0x1F0B,   /* U+1D4CE */
  0x1F21,   /* U+1D4CF */
  0x1C47,   /* U+1D4D0 */
  0x1C60,   /* U+1D4D1 */
  0x1C7A,   /* U+1D4D2 */
  0x1C8F,   /* U+1D4D3 */
  0x1CAA,   /* U+1D4D4 */
  0x1CE5,   /* U+1D4D5 */
  0x1CF4,   /* U+1D4D6 */
  0x1D18,   /* U+1D4D7 */
  0x1D32,   /* U+1D4D8 */
  0x1D4C,   /* U+1D4D9 */
  0x1D65,   /* U+1D4DA */
  0x1D77,   /* U+1D4DB */
  0x1DAA,   /* U+1D4DC */
  0x1DB9,   /* U+1D4DD */
  0x1DDD,   /* U+1D4DE */
  0x1E0C,   /* U+1D4DF */
  0x1E21,   /* U+1D4E0 */
  0x1E33,   /* U+1D4E1 */
  0x1E71,   /* U+1D4E2 */
  0x1E95,   /* U+1D4E3 */
  0x1EB5,   /* U+1D4E4 */
  0x1EE3,   /* U+1D4E5 */
  0x1EF5,   /* U+1D4E6 */
  0x1EFF,   /* U+1D4E7 */
  0x1F0B,   /* U+1D4E8 */
  0x1F21,   /* U+1D4E9 */
  0x1C47,   /* U+1D4EA */
  0x1C60,   /* U+1D4EB */
  0x1C7A,   /* U+1D4EC */
  0x1C8F,   /* U+1D4ED */
  0x1CAA,   /* U+1D4EE */
  0x1CE5,   /* U+1D4EF */
  0x1CF4,   /* U+1D4F0 */
  0x1D18,   /* U+1D4F1 */
  0x1D32,   /* U+1D4F2 */
  0x1D4C,   /* U+1D4F3 */
  0x1D65,   /* U+1D4F4 */
  0x1D77,   /* U+1D4F5 */
  0x1DAA,   /* U+1D4F6 */
  0x1DB9,   /* U+1D4F7 */
  0x1DDD,   /* U+1D4F8 */
  0x1E0C,   /* U+1D4F9 */
  0x1E21,   /* U+1D4FA */
  0x1E33,   /* U+1D4FB */
  0x1E71,   /* U+1D4FC */
  0x1E95,   /* U+1D4FD */
  0x1EB5,   /* U+1D4FE */
  0x1EE3,   /* U+1D4FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D400 */
  0x0020,   /* U+1D401 */
  0x0020,   /* U+1D402 */
  0x0020,   /* U+1D403 */
  0x0020,   /* U+1D404 */
  0x0020,   /* U+1D405 */
  0x0020,   /* U+1D406 */
  0x0020,   /* U+1D407 */
  0x0020,   /* U+1D408 */
  0x0020,   /* U+1D409 */
  0x0020,   /* U+1D40A */
  0x0020,   /* U+1D40B */
  0x0020,   /* U+1D40C */
  0x0020,   /* U+1D40D */
  0x0020,   /* U+1D40E */
  0x0020,   /* U+1D40F */
  0x0020,   /* U+1D410 */
  0x0020,   /* U+1D411 */
  0x0020,   /* U+1D412 */
  0x0020,   /* U+1D413 */
  0x0020,   /* U+1D414 */
  0x0020,   /* U+1D415 */
  0x0020,   /* U+1D416 */
  0x0020,   /* U+1D417 */
  0x0020,   /* U+1D418 */
  0x0020,   /* U+1D419 */
  0x0020,   /* U+1D41A */
  0x0020,   /* U+1D41B */
  0x0020,   /* U+1D41C */
  0x0020,   /* U+1D41D */
  0x0020,   /* U+1D41E */
  0x0020,   /* U+1D41F */
  0x0020,   /* U+1D420 */
  0x0020,   /* U+1D421 */
  0x0020,   /* U+1D422 */
  0x0020,   /* U+1D423 */
  0x0020,   /* U+1D424 */
  0x0020,   /* U+1D425 */
  0x0020,   /* U+1D426 */
  0x0020,   /* U+1D427 */
  0x0020,   /* U+1D428 */
  0x0020,   /* U+1D429 */
  0x0020,   /* U+1D42A */
  0x0020,   /* U+1D42B */
  0x0020,   /* U+1D42C */
  0x0020,   /* U+1D42D */
  0x0020,   /* U+1D42E */
  0x0020,   /* U+1D42F */
  0x0020,   /* U+1D430 */
  0x0020,   /* U+1D431 */
  0x0020,   /* U+1D432 */
  0x0020,   /* U+1D433 */
  0x0020,   /* U+1D434 */
  0x0020,   /* U+1D435 */
  0x0020,   /* U+1D436 */
  0x0020,   /* U+1D437 */
  0x0020,   /* U+1D438 */
  0x0020,   /* U+1D439 */
  0x0020,   /* U+1D43A */
  0x0020,   /* U+1D43B */
  0x0020,   /* U+1D43C */
  0x0020,   /* U+1D43D */
  0x0020,   /* U+1D43E */
  0x0020,   /* U+1D43F */
  0x0020,   /* U+1D440 */
  0x0020,   /* U+1D441 */
  0x0020,   /* U+1D442 */
  0x0020,   /* U+1D443 */
  0x0020,   /* U+1D444 */
  0x0020,   /* U+1D445 */
  0x0020,   /* U+1D446 */
  0x0020,   /* U+1D447 */
  0x0020,   /* U+1D448 */
  0x0020,   /* U+1D449 */
  0x0020,   /* U+1D44A */
  0x0020,   /* U+1D44B */
  0x0020,   /* U+1D44C */
  0x0020,   /* U+1D44D */
  0x0020,   /* U+1D44E */
  0x0020,   /* U+1D44F */
  0x0020,   /* U+1D450 */
  0x0020,   /* U+1D451 */
  0x0020,   /* U+1D452 */
  0x0020,   /* U+1D453 */
  0x0020,   /* U+1D454 */
  0x0020,   /* U+1D455 */
  0x0020,   /* U+1D456 */
  0x0020,   /* U+1D457 */
  0x0020,   /* U+1D458 */
  0x0020,   /* U+1D459 */
  0x0020,   /* U+1D45A */
  0x0020,   /* U+1D45B */
  0x0020,   /* U+1D45C */
  0x0020,   /* U+1D45D */
  0x0020,   /* U+1D45E */
  0x0020,   /* U+1D45F */
  0x0020,   /* U+1D460 */
  0x0020,   /* U+1D461 */
  0x0020,   /* U+1D462 */
  0x0020,   /* U+1D463 */
  0x0020,   /* U+1D464 */
  0x0020,   /* U+1D465 */
  0x0020,   /* U+1D466 */
  0x0020,   /* U+1D467 */
  0x0020,   /* U+1D468 */
  0x0020,   /* U+1D469 */
  0x0020,   /* U+1D46A */
  0x0020,   /* U+1D46B */
  0x0020,   /* U+1D46C */
  0x0020,   /* U+1D46D */
  0x0020,   /* U+1D46E */
  0x0020,   /* U+1D46F */
  0x0020,   /* U+1D470 */
  0x0020,   /* U+1D471 */
  0x0020,   /* U+1D472 */
  0x0020,   /* U+1D473 */
  0x0020,   /* U+1D474 */
  0x0020,   /* U+1D475 */
  0x0020,   /* U+1D476 */
  0x0020,   /* U+1D477 */
  0x0020,   /* U+1D478 */
  0x0020,   /* U+1D479 */
  0x0020,   /* U+1D47A */
  0x0020,   /* U+1D47B */
  0x0020,   /* U+1D47C */
  0x0020,   /* U+1D47D */
  0x0020,   /* U+1D47E */
  0x0020,   /* U+1D47F */
  0x0020,   /* U+1D480 */
  0x0020,   /* U+1D481 */
  0x0020,   /* U+1D482 */
  0x0020,   /* U+1D483 */
  0x0020,   /* U+1D484 */
  0x0020,   /* U+1D485 */
  0x0020,   /* U+1D486 */
  0x0020,   /* U+1D487 */
  0x0020,   /* U+1D488 */
  0x0020,   /* U+1D489 */
  0x0020,   /* U+1D48A */
  0x0020,   /* U+1D48B */
  0x0020,   /* U+1D48C */
  0x0020,   /* U+1D48D */
  0x0020,   /* U+1D48E */
  0x0020,   /* U+1D48F */
  0x0020,   /* U+1D490 */
  0x0020,   /* U+1D491 */
  0x0020,   /* U+1D492 */
  0x0020,   /* U+1D493 */
  0x0020,   /* U+1D494 */
  0x0020,   /* U+1D495 */
  0x0020,   /* U+1D496 */
  0x0020,   /* U+1D497 */
  0x0020,   /* U+1D498 */
  0x0020,   /* U+1D499 */
  0x0020,   /* U+1D49A */
  0x0020,   /* U+1D49B */
  0x0020,   /* U+1D49C */
  0x0020,   /* U+1D49D */
  0x0020,   /* U+1D49E */
  0x0020,   /* U+1D49F */
  0x0020,   /* U+1D4A0 */
  0x0020,   /* U+1D4A1 */
  0x0020,   /* U+1D4A2 */
  0x0020,   /* U+1D4A3 */
  0x0020,   /* U+1D4A4 */
  0x0020,   /* U+1D4A5 */
  0x0020,   /* U+1D4A6 */
  0x0020,   /* U+1D4A7 */
  0x0020,   /* U+1D4A8 */
  0x0020,   /* U+1D4A9 */
  0x0020,   /* U+1D4AA */
  0x0020,   /* U+1D4AB */
  0x0020,   /* U+1D4AC */
  0x0020,   /* U+1D4AD */
  0x0020,   /* U+1D4AE */
  0x0020,   /* U+1D4AF */
  0x0020,   /* U+1D4B0 */
  0x0020,   /* U+1D4B1 */
  0x0020,   /* U+1D4B2 */
  0x0020,   /* U+1D4B3 */
  0x0020,   /* U+1D4B4 */
  0x0020,   /* U+1D4B5 */
  0x0020,   /* U+1D4B6 */
  0x0020,   /* U+1D4B7 */
  0x0020,   /* U+1D4B8 */
  0x0020,   /* U+1D4B9 */
  0x0020,   /* U+1D4BA */
  0x0020,   /* U+1D4BB */
  0x0020,   /* U+1D4BC */
  0x0020,   /* U+1D4BD */
  0x0020,   /* U+1D4BE */
  0x0020,   /* U+1D4BF */
  0x0020,   /* U+1D4C0 */
  0x0020,   /* U+1D4C1 */
  0x0020,   /* U+1D4C2 */
  0x0020,   /* U+1D4C3 */
  0x0020,   /* U+1D4C4 */
  0x0020,   /* U+1D4C5 */
  0x0020,   /* U+1D4C6 */
  0x0020,   /* U+1D4C7 */
  0x0020,   /* U+1D4C8 */
  0x0020,   /* U+1D4C9 */
  0x0020,   /* U+1D4CA */
  0x0020,   /* U+1D4CB */
  0x0020,   /* U+1D4CC */
  0x0020,   /* U+1D4CD */
  0x0020,   /* U+1D4CE */
  0x0020,   /* U+1D4CF */
  0x0020,   /* U+1D4D0 */
  0x0020,   /* U+1D4D1 */
  0x0020,   /* U+1D4D2 */
  0x0020,   /* U+1D4D3 */
  0x0020,   /* U+1D4D4 */
  0x0020,   /* U+1D4D5 */
  0x0020,   /* U+1D4D6 */
  0x0020,   /* U+1D4D7 */
  0x0020,   /* U+1D4D8 */
  0x0020,   /* U+1D4D9 */
  0x0020,   /* U+1D4DA */
  0x0020,   /* U+1D4DB */
  0x0020,   /* U+1D4DC */
  0x0020,   /* U+1D4DD */
  0x0020,   /* U+1D4DE */
  0x0020,   /* U+1D4DF */
  0x0020,   /* U+1D4E0 */
  0x0020,   /* U+1D4E1 */
  0x0020,   /* U+1D4E2 */
  0x0020,   /* U+1D4E3 */
  0x0020,   /* U+1D4E4 */
  0x0020,   /* U+1D4E5 */
  0x0020,   /* U+1D4E6 */
  0x0020,   /* U+1D4E7 */
  0x0020,   /* U+1D4E8 */
  0x0020,   /* U+1D4E9 */
  0x0020,   /* U+1D4EA */
  0x0020,   /* U+1D4EB */
  0x0020,   /* U+1D4EC */
  0x0020,   /* U+1D4ED */
  0x0020,   /* U+1D4EE */
  0x0020,   /* U+1D4EF */
  0x0020,   /* U+1D4F0 */
  0x0020,   /* U+1D4F1 */
  0x0020,   /* U+1D4F2 */
  0x0020,   /* U+1D4F3 */
  0x0020,   /* U+1D4F4 */
  0x0020,   /* U+1D4F5 */
  0x0020,   /* U+1D4F6 */
  0x0020,   /* U+1D4F7 */
  0x0020,   /* U+1D4F8 */
  0x0020,   /* U+1D4F9 */
  0x0020,   /* U+1D4FA */
  0x0020,   /* U+1D4FB */
  0x0020,   /* U+1D4FC */
  0x0020,   /* U+1D4FD */
  0x0020,   /* U+1D4FE */
  0x0020,   /* U+1D4FF */

  /* Tertiary weight 1 for each character. */
  0x000B,   /* U+1D400 */
  0x000B,   /* U+1D401 */
  0x000B,   /* U+1D402 */
  0x000B,   /* U+1D403 */
  0x000B,   /* U+1D404 */
  0x000B,   /* U+1D405 */
  0x000B,   /* U+1D406 */
  0x000B,   /* U+1D407 */
  0x000B,   /* U+1D408 */
  0x000B,   /* U+1D409 */
  0x000B,   /* U+1D40A */
  0x000B,   /* U+1D40B */
  0x000B,   /* U+1D40C */
  0x000B,   /* U+1D40D */
  0x000B,   /* U+1D40E */
  0x000B,   /* U+1D40F */
  0x000B,   /* U+1D410 */
  0x000B,   /* U+1D411 */
  0x000B,   /* U+1D412 */
  0x000B,   /* U+1D413 */
  0x000B,   /* U+1D414 */
  0x000B,   /* U+1D415 */
  0x000B,   /* U+1D416 */
  0x000B,   /* U+1D417 */
  0x000B,   /* U+1D418 */
  0x000B,   /* U+1D419 */
  0x0005,   /* U+1D41A */
  0x0005,   /* U+1D41B */
  0x0005,   /* U+1D41C */
  0x0005,   /* U+1D41D */
  0x0005,   /* U+1D41E */
  0x0005,   /* U+1D41F */
  0x0005,   /* U+1D420 */
  0x0005,   /* U+1D421 */
  0x0005,   /* U+1D422 */
  0x0005,   /* U+1D423 */
  0x0005,   /* U+1D424 */
  0x0005,   /* U+1D425 */
  0x0005,   /* U+1D426 */
  0x0005,   /* U+1D427 */
  0x0005,   /* U+1D428 */
  0x0005,   /* U+1D429 */
  0x0005,   /* U+1D42A */
  0x0005,   /* U+1D42B */
  0x0005,   /* U+1D42C */
  0x0005,   /* U+1D42D */
  0x0005,   /* U+1D42E */
  0x0005,   /* U+1D42F */
  0x0005,   /* U+1D430 */
  0x0005,   /* U+1D431 */
  0x0005,   /* U+1D432 */
  0x0005,   /* U+1D433 */
  0x000B,   /* U+1D434 */
  0x000B,   /* U+1D435 */
  0x000B,   /* U+1D436 */
  0x000B,   /* U+1D437 */
  0x000B,   /* U+1D438 */
  0x000B,   /* U+1D439 */
  0x000B,   /* U+1D43A */
  0x000B,   /* U+1D43B */
  0x000B,   /* U+1D43C */
  0x000B,   /* U+1D43D */
  0x000B,   /* U+1D43E */
  0x000B,   /* U+1D43F */
  0x000B,   /* U+1D440 */
  0x000B,   /* U+1D441 */
  0x000B,   /* U+1D442 */
  0x000B,   /* U+1D443 */
  0x000B,   /* U+1D444 */
  0x000B,   /* U+1D445 */
  0x000B,   /* U+1D446 */
  0x000B,   /* U+1D447 */
  0x000B,   /* U+1D448 */
  0x000B,   /* U+1D449 */
  0x000B,   /* U+1D44A */
  0x000B,   /* U+1D44B */
  0x000B,   /* U+1D44C */
  0x000B,   /* U+1D44D */
  0x0005,   /* U+1D44E */
  0x0005,   /* U+1D44F */
  0x0005,   /* U+1D450 */
  0x0005,   /* U+1D451 */
  0x0005,   /* U+1D452 */
  0x0005,   /* U+1D453 */
  0x0005,   /* U+1D454 */
  0x0002,   /* U+1D455 */
  0x0005,   /* U+1D456 */
  0x0005,   /* U+1D457 */
  0x0005,   /* U+1D458 */
  0x0005,   /* U+1D459 */
  0x0005,   /* U+1D45A */
  0x0005,   /* U+1D45B */
  0x0005,   /* U+1D45C */
  0x0005,   /* U+1D45D */
  0x0005,   /* U+1D45E */
  0x0005,   /* U+1D45F */
  0x0005,   /* U+1D460 */
  0x0005,   /* U+1D461 */
  0x0005,   /* U+1D462 */
  0x0005,   /* U+1D463 */
  0x0005,   /* U+1D464 */
  0x0005,   /* U+1D465 */
  0x0005,   /* U+1D466 */
  0x0005,   /* U+1D467 */
  0x000B,   /* U+1D468 */
  0x000B,   /* U+1D469 */
  0x000B,   /* U+1D46A */
  0x000B,   /* U+1D46B */
  0x000B,   /* U+1D46C */
  0x000B,   /* U+1D46D */
  0x000B,   /* U+1D46E */
  0x000B,   /* U+1D46F */
  0x000B,   /* U+1D470 */
  0x000B,   /* U+1D471 */
  0x000B,   /* U+1D472 */
  0x000B,   /* U+1D473 */
  0x000B,   /* U+1D474 */
  0x000B,   /* U+1D475 */
  0x000B,   /* U+1D476 */
  0x000B,   /* U+1D477 */
  0x000B,   /* U+1D478 */
  0x000B,   /* U+1D479 */
  0x000B,   /* U+1D47A */
  0x000B,   /* U+1D47B */
  0x000B,   /* U+1D47C */
  0x000B,   /* U+1D47D */
  0x000B,   /* U+1D47E */
  0x000B,   /* U+1D47F */
  0x000B,   /* U+1D480 */
  0x000B,   /* U+1D481 */
  0x0005,   /* U+1D482 */
  0x0005,   /* U+1D483 */
  0x0005,   /* U+1D484 */
  0x0005,   /* U+1D485 */
  0x0005,   /* U+1D486 */
  0x0005,   /* U+1D487 */
  0x0005,   /* U+1D488 */
  0x0005,   /* U+1D489 */
  0x0005,   /* U+1D48A */
  0x0005,   /* U+1D48B */
  0x0005,   /* U+1D48C */
  0x0005,   /* U+1D48D */
  0x0005,   /* U+1D48E */
  0x0005,   /* U+1D48F */
  0x0005,   /* U+1D490 */
  0x0005,   /* U+1D491 */
  0x0005,   /* U+1D492 */
  0x0005,   /* U+1D493 */
  0x0005,   /* U+1D494 */
  0x0005,   /* U+1D495 */
  0x0005,   /* U+1D496 */
  0x0005,   /* U+1D497 */
  0x0005,   /* U+1D498 */
  0x0005,   /* U+1D499 */
  0x0005,   /* U+1D49A */
  0x0005,   /* U+1D49B */
  0x000B,   /* U+1D49C */
  0x0002,   /* U+1D49D */
  0x000B,   /* U+1D49E */
  0x000B,   /* U+1D49F */
  0x0002,   /* U+1D4A0 */
  0x0002,   /* U+1D4A1 */
  0x000B,   /* U+1D4A2 */
  0x0002,   /* U+1D4A3 */
  0x0002,   /* U+1D4A4 */
  0x000B,   /* U+1D4A5 */
  0x000B,   /* U+1D4A6 */
  0x0002,   /* U+1D4A7 */
  0x0002,   /* U+1D4A8 */
  0x000B,   /* U+1D4A9 */
  0x000B,   /* U+1D4AA */
  0x000B,   /* U+1D4AB */
  0x000B,   /* U+1D4AC */
  0x0002,   /* U+1D4AD */
  0x000B,   /* U+1D4AE */
  0x000B,   /* U+1D4AF */
  0x000B,   /* U+1D4B0 */
  0x000B,   /* U+1D4B1 */
  0x000B,   /* U+1D4B2 */
  0x000B,   /* U+1D4B3 */
  0x000B,   /* U+1D4B4 */
  0x000B,   /* U+1D4B5 */
  0x0005,   /* U+1D4B6 */
  0x0005,   /* U+1D4B7 */
  0x0005,   /* U+1D4B8 */
  0x0005,   /* U+1D4B9 */
  0x0002,   /* U+1D4BA */
  0x0005,   /* U+1D4BB */
  0x0002,   /* U+1D4BC */
  0x0005,   /* U+1D4BD */
  0x0005,   /* U+1D4BE */
  0x0005,   /* U+1D4BF */
  0x0005,   /* U+1D4C0 */
  0x0005,   /* U+1D4C1 */
  0x0005,   /* U+1D4C2 */
  0x0005,   /* U+1D4C3 */
  0x0002,   /* U+1D4C4 */
  0x0005,   /* U+1D4C5 */
  0x0005,   /* U+1D4C6 */
  0x0005,   /* U+1D4C7 */
  0x0005,   /* U+1D4C8 */
  0x0005,   /* U+1D4C9 */
  0x0005,   /* U+1D4CA */
  0x0005,   /* U+1D4CB */
  0x0005,   /* U+1D4CC */
  0x0005,   /* U+1D4CD */
  0x0005,   /* U+1D4CE */
  0x0005,   /* U+1D4CF */
  0x000B,   /* U+1D4D0 */
  0x000B,   /* U+1D4D1 */
  0x000B,   /* U+1D4D2 */
  0x000B,   /* U+1D4D3 */
  0x000B,   /* U+1D4D4 */
  0x000B,   /* U+1D4D5 */
  0x000B,   /* U+1D4D6 */
  0x000B,   /* U+1D4D7 */
  0x000B,   /* U+1D4D8 */
  0x000B,   /* U+1D4D9 */
  0x000B,   /* U+1D4DA */
  0x000B,   /* U+1D4DB */
  0x000B,   /* U+1D4DC */
  0x000B,   /* U+1D4DD */
  0x000B,   /* U+1D4DE */
  0x000B,   /* U+1D4DF */
  0x000B,   /* U+1D4E0 */
  0x000B,   /* U+1D4E1 */
  0x000B,   /* U+1D4E2 */
  0x000B,   /* U+1D4E3 */
  0x000B,   /* U+1D4E4 */
  0x000B,   /* U+1D4E5 */
  0x000B,   /* U+1D4E6 */
  0x000B,   /* U+1D4E7 */
  0x000B,   /* U+1D4E8 */
  0x000B,   /* U+1D4E9 */
  0x0005,   /* U+1D4EA */
  0x0005,   /* U+1D4EB */
  0x0005,   /* U+1D4EC */
  0x0005,   /* U+1D4ED */
  0x0005,   /* U+1D4EE */
  0x0005,   /* U+1D4EF */
  0x0005,   /* U+1D4F0 */
  0x0005,   /* U+1D4F1 */
  0x0005,   /* U+1D4F2 */
  0x0005,   /* U+1D4F3 */
  0x0005,   /* U+1D4F4 */
  0x0005,   /* U+1D4F5 */
  0x0005,   /* U+1D4F6 */
  0x0005,   /* U+1D4F7 */
  0x0005,   /* U+1D4F8 */
  0x0005,   /* U+1D4F9 */
  0x0005,   /* U+1D4FA */
  0x0005,   /* U+1D4FB */
  0x0005,   /* U+1D4FC */
  0x0005,   /* U+1D4FD */
  0x0005,   /* U+1D4FE */
  0x0005,   /* U+1D4FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D400 */
  0x0000,   /* U+1D401 */
  0x0000,   /* U+1D402 */
  0x0000,   /* U+1D403 */
  0x0000,   /* U+1D404 */
  0x0000,   /* U+1D405 */
  0x0000,   /* U+1D406 */
  0x0000,   /* U+1D407 */
  0x0000,   /* U+1D408 */
  0x0000,   /* U+1D409 */
  0x0000,   /* U+1D40A */
  0x0000,   /* U+1D40B */
  0x0000,   /* U+1D40C */
  0x0000,   /* U+1D40D */
  0x0000,   /* U+1D40E */
  0x0000,   /* U+1D40F */
  0x0000,   /* U+1D410 */
  0x0000,   /* U+1D411 */
  0x0000,   /* U+1D412 */
  0x0000,   /* U+1D413 */
  0x0000,   /* U+1D414 */
  0x0000,   /* U+1D415 */
  0x0000,   /* U+1D416 */
  0x0000,   /* U+1D417 */
  0x0000,   /* U+1D418 */
  0x0000,   /* U+1D419 */
  0x0000,   /* U+1D41A */
  0x0000,   /* U+1D41B */
  0x0000,   /* U+1D41C */
  0x0000,   /* U+1D41D */
  0x0000,   /* U+1D41E */
  0x0000,   /* U+1D41F */
  0x0000,   /* U+1D420 */
  0x0000,   /* U+1D421 */
  0x0000,   /* U+1D422 */
  0x0000,   /* U+1D423 */
  0x0000,   /* U+1D424 */
  0x0000,   /* U+1D425 */
  0x0000,   /* U+1D426 */
  0x0000,   /* U+1D427 */
  0x0000,   /* U+1D428 */
  0x0000,   /* U+1D429 */
  0x0000,   /* U+1D42A */
  0x0000,   /* U+1D42B */
  0x0000,   /* U+1D42C */
  0x0000,   /* U+1D42D */
  0x0000,   /* U+1D42E */
  0x0000,   /* U+1D42F */
  0x0000,   /* U+1D430 */
  0x0000,   /* U+1D431 */
  0x0000,   /* U+1D432 */
  0x0000,   /* U+1D433 */
  0x0000,   /* U+1D434 */
  0x0000,   /* U+1D435 */
  0x0000,   /* U+1D436 */
  0x0000,   /* U+1D437 */
  0x0000,   /* U+1D438 */
  0x0000,   /* U+1D439 */
  0x0000,   /* U+1D43A */
  0x0000,   /* U+1D43B */
  0x0000,   /* U+1D43C */
  0x0000,   /* U+1D43D */
  0x0000,   /* U+1D43E */
  0x0000,   /* U+1D43F */
  0x0000,   /* U+1D440 */
  0x0000,   /* U+1D441 */
  0x0000,   /* U+1D442 */
  0x0000,   /* U+1D443 */
  0x0000,   /* U+1D444 */
  0x0000,   /* U+1D445 */
  0x0000,   /* U+1D446 */
  0x0000,   /* U+1D447 */
  0x0000,   /* U+1D448 */
  0x0000,   /* U+1D449 */
  0x0000,   /* U+1D44A */
  0x0000,   /* U+1D44B */
  0x0000,   /* U+1D44C */
  0x0000,   /* U+1D44D */
  0x0000,   /* U+1D44E */
  0x0000,   /* U+1D44F */
  0x0000,   /* U+1D450 */
  0x0000,   /* U+1D451 */
  0x0000,   /* U+1D452 */
  0x0000,   /* U+1D453 */
  0x0000,   /* U+1D454 */
  0xD455,   /* U+1D455 */
  0x0000,   /* U+1D456 */
  0x0000,   /* U+1D457 */
  0x0000,   /* U+1D458 */
  0x0000,   /* U+1D459 */
  0x0000,   /* U+1D45A */
  0x0000,   /* U+1D45B */
  0x0000,   /* U+1D45C */
  0x0000,   /* U+1D45D */
  0x0000,   /* U+1D45E */
  0x0000,   /* U+1D45F */
  0x0000,   /* U+1D460 */
  0x0000,   /* U+1D461 */
  0x0000,   /* U+1D462 */
  0x0000,   /* U+1D463 */
  0x0000,   /* U+1D464 */
  0x0000,   /* U+1D465 */
  0x0000,   /* U+1D466 */
  0x0000,   /* U+1D467 */
  0x0000,   /* U+1D468 */
  0x0000,   /* U+1D469 */
  0x0000,   /* U+1D46A */
  0x0000,   /* U+1D46B */
  0x0000,   /* U+1D46C */
  0x0000,   /* U+1D46D */
  0x0000,   /* U+1D46E */
  0x0000,   /* U+1D46F */
  0x0000,   /* U+1D470 */
  0x0000,   /* U+1D471 */
  0x0000,   /* U+1D472 */
  0x0000,   /* U+1D473 */
  0x0000,   /* U+1D474 */
  0x0000,   /* U+1D475 */
  0x0000,   /* U+1D476 */
  0x0000,   /* U+1D477 */
  0x0000,   /* U+1D478 */
  0x0000,   /* U+1D479 */
  0x0000,   /* U+1D47A */
  0x0000,   /* U+1D47B */
  0x0000,   /* U+1D47C */
  0x0000,   /* U+1D47D */
  0x0000,   /* U+1D47E */
  0x0000,   /* U+1D47F */
  0x0000,   /* U+1D480 */
  0x0000,   /* U+1D481 */
  0x0000,   /* U+1D482 */
  0x0000,   /* U+1D483 */
  0x0000,   /* U+1D484 */
  0x0000,   /* U+1D485 */
  0x0000,   /* U+1D486 */
  0x0000,   /* U+1D487 */
  0x0000,   /* U+1D488 */
  0x0000,   /* U+1D489 */
  0x0000,   /* U+1D48A */
  0x0000,   /* U+1D48B */
  0x0000,   /* U+1D48C */
  0x0000,   /* U+1D48D */
  0x0000,   /* U+1D48E */
  0x0000,   /* U+1D48F */
  0x0000,   /* U+1D490 */
  0x0000,   /* U+1D491 */
  0x0000,   /* U+1D492 */
  0x0000,   /* U+1D493 */
  0x0000,   /* U+1D494 */
  0x0000,   /* U+1D495 */
  0x0000,   /* U+1D496 */
  0x0000,   /* U+1D497 */
  0x0000,   /* U+1D498 */
  0x0000,   /* U+1D499 */
  0x0000,   /* U+1D49A */
  0x0000,   /* U+1D49B */
  0x0000,   /* U+1D49C */
  0xD49D,   /* U+1D49D */
  0x0000,   /* U+1D49E */
  0x0000,   /* U+1D49F */
  0xD4A0,   /* U+1D4A0 */
  0xD4A1,   /* U+1D4A1 */
  0x0000,   /* U+1D4A2 */
  0xD4A3,   /* U+1D4A3 */
  0xD4A4,   /* U+1D4A4 */
  0x0000,   /* U+1D4A5 */
  0x0000,   /* U+1D4A6 */
  0xD4A7,   /* U+1D4A7 */
  0xD4A8,   /* U+1D4A8 */
  0x0000,   /* U+1D4A9 */
  0x0000,   /* U+1D4AA */
  0x0000,   /* U+1D4AB */
  0x0000,   /* U+1D4AC */
  0xD4AD,   /* U+1D4AD */
  0x0000,   /* U+1D4AE */
  0x0000,   /* U+1D4AF */
  0x0000,   /* U+1D4B0 */
  0x0000,   /* U+1D4B1 */
  0x0000,   /* U+1D4B2 */
  0x0000,   /* U+1D4B3 */
  0x0000,   /* U+1D4B4 */
  0x0000,   /* U+1D4B5 */
  0x0000,   /* U+1D4B6 */
  0x0000,   /* U+1D4B7 */
  0x0000,   /* U+1D4B8 */
  0x0000,   /* U+1D4B9 */
  0xD4BA,   /* U+1D4BA */
  0x0000,   /* U+1D4BB */
  0xD4BC,   /* U+1D4BC */
  0x0000,   /* U+1D4BD */
  0x0000,   /* U+1D4BE */
  0x0000,   /* U+1D4BF */
  0x0000,   /* U+1D4C0 */
  0x0000,   /* U+1D4C1 */
  0x0000,   /* U+1D4C2 */
  0x0000,   /* U+1D4C3 */
  0xD4C4,   /* U+1D4C4 */
  0x0000,   /* U+1D4C5 */
  0x0000,   /* U+1D4C6 */
  0x0000,   /* U+1D4C7 */
  0x0000,   /* U+1D4C8 */
  0x0000,   /* U+1D4C9 */
  0x0000,   /* U+1D4CA */
  0x0000,   /* U+1D4CB */
  0x0000,   /* U+1D4CC */
  0x0000,   /* U+1D4CD */
  0x0000,   /* U+1D4CE */
  0x0000,   /* U+1D4CF */
  0x0000,   /* U+1D4D0 */
  0x0000,   /* U+1D4D1 */
  0x0000,   /* U+1D4D2 */
  0x0000,   /* U+1D4D3 */
  0x0000,   /* U+1D4D4 */
  0x0000,   /* U+1D4D5 */
  0x0000,   /* U+1D4D6 */
  0x0000,   /* U+1D4D7 */
  0x0000,   /* U+1D4D8 */
  0x0000,   /* U+1D4D9 */
  0x0000,   /* U+1D4DA */
  0x0000,   /* U+1D4DB */
  0x0000,   /* U+1D4DC */
  0x0000,   /* U+1D4DD */
  0x0000,   /* U+1D4DE */
  0x0000,   /* U+1D4DF */
  0x0000,   /* U+1D4E0 */
  0x0000,   /* U+1D4E1 */
  0x0000,   /* U+1D4E2 */
  0x0000,   /* U+1D4E3 */
  0x0000,   /* U+1D4E4 */
  0x0000,   /* U+1D4E5 */
  0x0000,   /* U+1D4E6 */
  0x0000,   /* U+1D4E7 */
  0x0000,   /* U+1D4E8 */
  0x0000,   /* U+1D4E9 */
  0x0000,   /* U+1D4EA */
  0x0000,   /* U+1D4EB */
  0x0000,   /* U+1D4EC */
  0x0000,   /* U+1D4ED */
  0x0000,   /* U+1D4EE */
  0x0000,   /* U+1D4EF */
  0x0000,   /* U+1D4F0 */
  0x0000,   /* U+1D4F1 */
  0x0000,   /* U+1D4F2 */
  0x0000,   /* U+1D4F3 */
  0x0000,   /* U+1D4F4 */
  0x0000,   /* U+1D4F5 */
  0x0000,   /* U+1D4F6 */
  0x0000,   /* U+1D4F7 */
  0x0000,   /* U+1D4F8 */
  0x0000,   /* U+1D4F9 */
  0x0000,   /* U+1D4FA */
  0x0000,   /* U+1D4FB */
  0x0000,   /* U+1D4FC */
  0x0000,   /* U+1D4FD */
  0x0000,   /* U+1D4FE */
  0x0000,   /* U+1D4FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D400 */
  0x0000,   /* U+1D401 */
  0x0000,   /* U+1D402 */
  0x0000,   /* U+1D403 */
  0x0000,   /* U+1D404 */
  0x0000,   /* U+1D405 */
  0x0000,   /* U+1D406 */
  0x0000,   /* U+1D407 */
  0x0000,   /* U+1D408 */
  0x0000,   /* U+1D409 */
  0x0000,   /* U+1D40A */
  0x0000,   /* U+1D40B */
  0x0000,   /* U+1D40C */
  0x0000,   /* U+1D40D */
  0x0000,   /* U+1D40E */
  0x0000,   /* U+1D40F */
  0x0000,   /* U+1D410 */
  0x0000,   /* U+1D411 */
  0x0000,   /* U+1D412 */
  0x0000,   /* U+1D413 */
  0x0000,   /* U+1D414 */
  0x0000,   /* U+1D415 */
  0x0000,   /* U+1D416 */
  0x0000,   /* U+1D417 */
  0x0000,   /* U+1D418 */
  0x0000,   /* U+1D419 */
  0x0000,   /* U+1D41A */
  0x0000,   /* U+1D41B */
  0x0000,   /* U+1D41C */
  0x0000,   /* U+1D41D */
  0x0000,   /* U+1D41E */
  0x0000,   /* U+1D41F */
  0x0000,   /* U+1D420 */
  0x0000,   /* U+1D421 */
  0x0000,   /* U+1D422 */
  0x0000,   /* U+1D423 */
  0x0000,   /* U+1D424 */
  0x0000,   /* U+1D425 */
  0x0000,   /* U+1D426 */
  0x0000,   /* U+1D427 */
  0x0000,   /* U+1D428 */
  0x0000,   /* U+1D429 */
  0x0000,   /* U+1D42A */
  0x0000,   /* U+1D42B */
  0x0000,   /* U+1D42C */
  0x0000,   /* U+1D42D */
  0x0000,   /* U+1D42E */
  0x0000,   /* U+1D42F */
  0x0000,   /* U+1D430 */
  0x0000,   /* U+1D431 */
  0x0000,   /* U+1D432 */
  0x0000,   /* U+1D433 */
  0x0000,   /* U+1D434 */
  0x0000,   /* U+1D435 */
  0x0000,   /* U+1D436 */
  0x0000,   /* U+1D437 */
  0x0000,   /* U+1D438 */
  0x0000,   /* U+1D439 */
  0x0000,   /* U+1D43A */
  0x0000,   /* U+1D43B */
  0x0000,   /* U+1D43C */
  0x0000,   /* U+1D43D */
  0x0000,   /* U+1D43E */
  0x0000,   /* U+1D43F */
  0x0000,   /* U+1D440 */
  0x0000,   /* U+1D441 */
  0x0000,   /* U+1D442 */
  0x0000,   /* U+1D443 */
  0x0000,   /* U+1D444 */
  0x0000,   /* U+1D445 */
  0x0000,   /* U+1D446 */
  0x0000,   /* U+1D447 */
  0x0000,   /* U+1D448 */
  0x0000,   /* U+1D449 */
  0x0000,   /* U+1D44A */
  0x0000,   /* U+1D44B */
  0x0000,   /* U+1D44C */
  0x0000,   /* U+1D44D */
  0x0000,   /* U+1D44E */
  0x0000,   /* U+1D44F */
  0x0000,   /* U+1D450 */
  0x0000,   /* U+1D451 */
  0x0000,   /* U+1D452 */
  0x0000,   /* U+1D453 */
  0x0000,   /* U+1D454 */
  0x0000,   /* U+1D455 */
  0x0000,   /* U+1D456 */
  0x0000,   /* U+1D457 */
  0x0000,   /* U+1D458 */
  0x0000,   /* U+1D459 */
  0x0000,   /* U+1D45A */
  0x0000,   /* U+1D45B */
  0x0000,   /* U+1D45C */
  0x0000,   /* U+1D45D */
  0x0000,   /* U+1D45E */
  0x0000,   /* U+1D45F */
  0x0000,   /* U+1D460 */
  0x0000,   /* U+1D461 */
  0x0000,   /* U+1D462 */
  0x0000,   /* U+1D463 */
  0x0000,   /* U+1D464 */
  0x0000,   /* U+1D465 */
  0x0000,   /* U+1D466 */
  0x0000,   /* U+1D467 */
  0x0000,   /* U+1D468 */
  0x0000,   /* U+1D469 */
  0x0000,   /* U+1D46A */
  0x0000,   /* U+1D46B */
  0x0000,   /* U+1D46C */
  0x0000,   /* U+1D46D */
  0x0000,   /* U+1D46E */
  0x0000,   /* U+1D46F */
  0x0000,   /* U+1D470 */
  0x0000,   /* U+1D471 */
  0x0000,   /* U+1D472 */
  0x0000,   /* U+1D473 */
  0x0000,   /* U+1D474 */
  0x0000,   /* U+1D475 */
  0x0000,   /* U+1D476 */
  0x0000,   /* U+1D477 */
  0x0000,   /* U+1D478 */
  0x0000,   /* U+1D479 */
  0x0000,   /* U+1D47A */
  0x0000,   /* U+1D47B */
  0x0000,   /* U+1D47C */
  0x0000,   /* U+1D47D */
  0x0000,   /* U+1D47E */
  0x0000,   /* U+1D47F */
  0x0000,   /* U+1D480 */
  0x0000,   /* U+1D481 */
  0x0000,   /* U+1D482 */
  0x0000,   /* U+1D483 */
  0x0000,   /* U+1D484 */
  0x0000,   /* U+1D485 */
  0x0000,   /* U+1D486 */
  0x0000,   /* U+1D487 */
  0x0000,   /* U+1D488 */
  0x0000,   /* U+1D489 */
  0x0000,   /* U+1D48A */
  0x0000,   /* U+1D48B */
  0x0000,   /* U+1D48C */
  0x0000,   /* U+1D48D */
  0x0000,   /* U+1D48E */
  0x0000,   /* U+1D48F */
  0x0000,   /* U+1D490 */
  0x0000,   /* U+1D491 */
  0x0000,   /* U+1D492 */
  0x0000,   /* U+1D493 */
  0x0000,   /* U+1D494 */
  0x0000,   /* U+1D495 */
  0x0000,   /* U+1D496 */
  0x0000,   /* U+1D497 */
  0x0000,   /* U+1D498 */
  0x0000,   /* U+1D499 */
  0x0000,   /* U+1D49A */
  0x0000,   /* U+1D49B */
  0x0000,   /* U+1D49C */
  0x0000,   /* U+1D49D */
  0x0000,   /* U+1D49E */
  0x0000,   /* U+1D49F */
  0x0000,   /* U+1D4A0 */
  0x0000,   /* U+1D4A1 */
  0x0000,   /* U+1D4A2 */
  0x0000,   /* U+1D4A3 */
  0x0000,   /* U+1D4A4 */
  0x0000,   /* U+1D4A5 */
  0x0000,   /* U+1D4A6 */
  0x0000,   /* U+1D4A7 */
  0x0000,   /* U+1D4A8 */
  0x0000,   /* U+1D4A9 */
  0x0000,   /* U+1D4AA */
  0x0000,   /* U+1D4AB */
  0x0000,   /* U+1D4AC */
  0x0000,   /* U+1D4AD */
  0x0000,   /* U+1D4AE */
  0x0000,   /* U+1D4AF */
  0x0000,   /* U+1D4B0 */
  0x0000,   /* U+1D4B1 */
  0x0000,   /* U+1D4B2 */
  0x0000,   /* U+1D4B3 */
  0x0000,   /* U+1D4B4 */
  0x0000,   /* U+1D4B5 */
  0x0000,   /* U+1D4B6 */
  0x0000,   /* U+1D4B7 */
  0x0000,   /* U+1D4B8 */
  0x0000,   /* U+1D4B9 */
  0x0000,   /* U+1D4BA */
  0x0000,   /* U+1D4BB */
  0x0000,   /* U+1D4BC */
  0x0000,   /* U+1D4BD */
  0x0000,   /* U+1D4BE */
  0x0000,   /* U+1D4BF */
  0x0000,   /* U+1D4C0 */
  0x0000,   /* U+1D4C1 */
  0x0000,   /* U+1D4C2 */
  0x0000,   /* U+1D4C3 */
  0x0000,   /* U+1D4C4 */
  0x0000,   /* U+1D4C5 */
  0x0000,   /* U+1D4C6 */
  0x0000,   /* U+1D4C7 */
  0x0000,   /* U+1D4C8 */
  0x0000,   /* U+1D4C9 */
  0x0000,   /* U+1D4CA */
  0x0000,   /* U+1D4CB */
  0x0000,   /* U+1D4CC */
  0x0000,   /* U+1D4CD */
  0x0000,   /* U+1D4CE */
  0x0000,   /* U+1D4CF */
  0x0000,   /* U+1D4D0 */
  0x0000,   /* U+1D4D1 */
  0x0000,   /* U+1D4D2 */
  0x0000,   /* U+1D4D3 */
  0x0000,   /* U+1D4D4 */
  0x0000,   /* U+1D4D5 */
  0x0000,   /* U+1D4D6 */
  0x0000,   /* U+1D4D7 */
  0x0000,   /* U+1D4D8 */
  0x0000,   /* U+1D4D9 */
  0x0000,   /* U+1D4DA */
  0x0000,   /* U+1D4DB */
  0x0000,   /* U+1D4DC */
  0x0000,   /* U+1D4DD */
  0x0000,   /* U+1D4DE */
  0x0000,   /* U+1D4DF */
  0x0000,   /* U+1D4E0 */
  0x0000,   /* U+1D4E1 */
  0x0000,   /* U+1D4E2 */
  0x0000,   /* U+1D4E3 */
  0x0000,   /* U+1D4E4 */
  0x0000,   /* U+1D4E5 */
  0x0000,   /* U+1D4E6 */
  0x0000,   /* U+1D4E7 */
  0x0000,   /* U+1D4E8 */
  0x0000,   /* U+1D4E9 */
  0x0000,   /* U+1D4EA */
  0x0000,   /* U+1D4EB */
  0x0000,   /* U+1D4EC */
  0x0000,   /* U+1D4ED */
  0x0000,   /* U+1D4EE */
  0x0000,   /* U+1D4EF */
  0x0000,   /* U+1D4F0 */
  0x0000,   /* U+1D4F1 */
  0x0000,   /* U+1D4F2 */
  0x0000,   /* U+1D4F3 */
  0x0000,   /* U+1D4F4 */
  0x0000,   /* U+1D4F5 */
  0x0000,   /* U+1D4F6 */
  0x0000,   /* U+1D4F7 */
  0x0000,   /* U+1D4F8 */
  0x0000,   /* U+1D4F9 */
  0x0000,   /* U+1D4FA */
  0x0000,   /* U+1D4FB */
  0x0000,   /* U+1D4FC */
  0x0000,   /* U+1D4FD */
  0x0000,   /* U+1D4FE */
  0x0000,   /* U+1D4FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D400 */
  0x0000,   /* U+1D401 */
  0x0000,   /* U+1D402 */
  0x0000,   /* U+1D403 */
  0x0000,   /* U+1D404 */
  0x0000,   /* U+1D405 */
  0x0000,   /* U+1D406 */
  0x0000,   /* U+1D407 */
  0x0000,   /* U+1D408 */
  0x0000,   /* U+1D409 */
  0x0000,   /* U+1D40A */
  0x0000,   /* U+1D40B */
  0x0000,   /* U+1D40C */
  0x0000,   /* U+1D40D */
  0x0000,   /* U+1D40E */
  0x0000,   /* U+1D40F */
  0x0000,   /* U+1D410 */
  0x0000,   /* U+1D411 */
  0x0000,   /* U+1D412 */
  0x0000,   /* U+1D413 */
  0x0000,   /* U+1D414 */
  0x0000,   /* U+1D415 */
  0x0000,   /* U+1D416 */
  0x0000,   /* U+1D417 */
  0x0000,   /* U+1D418 */
  0x0000,   /* U+1D419 */
  0x0000,   /* U+1D41A */
  0x0000,   /* U+1D41B */
  0x0000,   /* U+1D41C */
  0x0000,   /* U+1D41D */
  0x0000,   /* U+1D41E */
  0x0000,   /* U+1D41F */
  0x0000,   /* U+1D420 */
  0x0000,   /* U+1D421 */
  0x0000,   /* U+1D422 */
  0x0000,   /* U+1D423 */
  0x0000,   /* U+1D424 */
  0x0000,   /* U+1D425 */
  0x0000,   /* U+1D426 */
  0x0000,   /* U+1D427 */
  0x0000,   /* U+1D428 */
  0x0000,   /* U+1D429 */
  0x0000,   /* U+1D42A */
  0x0000,   /* U+1D42B */
  0x0000,   /* U+1D42C */
  0x0000,   /* U+1D42D */
  0x0000,   /* U+1D42E */
  0x0000,   /* U+1D42F */
  0x0000,   /* U+1D430 */
  0x0000,   /* U+1D431 */
  0x0000,   /* U+1D432 */
  0x0000,   /* U+1D433 */
  0x0000,   /* U+1D434 */
  0x0000,   /* U+1D435 */
  0x0000,   /* U+1D436 */
  0x0000,   /* U+1D437 */
  0x0000,   /* U+1D438 */
  0x0000,   /* U+1D439 */
  0x0000,   /* U+1D43A */
  0x0000,   /* U+1D43B */
  0x0000,   /* U+1D43C */
  0x0000,   /* U+1D43D */
  0x0000,   /* U+1D43E */
  0x0000,   /* U+1D43F */
  0x0000,   /* U+1D440 */
  0x0000,   /* U+1D441 */
  0x0000,   /* U+1D442 */
  0x0000,   /* U+1D443 */
  0x0000,   /* U+1D444 */
  0x0000,   /* U+1D445 */
  0x0000,   /* U+1D446 */
  0x0000,   /* U+1D447 */
  0x0000,   /* U+1D448 */
  0x0000,   /* U+1D449 */
  0x0000,   /* U+1D44A */
  0x0000,   /* U+1D44B */
  0x0000,   /* U+1D44C */
  0x0000,   /* U+1D44D */
  0x0000,   /* U+1D44E */
  0x0000,   /* U+1D44F */
  0x0000,   /* U+1D450 */
  0x0000,   /* U+1D451 */
  0x0000,   /* U+1D452 */
  0x0000,   /* U+1D453 */
  0x0000,   /* U+1D454 */
  0x0000,   /* U+1D455 */
  0x0000,   /* U+1D456 */
  0x0000,   /* U+1D457 */
  0x0000,   /* U+1D458 */
  0x0000,   /* U+1D459 */
  0x0000,   /* U+1D45A */
  0x0000,   /* U+1D45B */
  0x0000,   /* U+1D45C */
  0x0000,   /* U+1D45D */
  0x0000,   /* U+1D45E */
  0x0000,   /* U+1D45F */
  0x0000,   /* U+1D460 */
  0x0000,   /* U+1D461 */
  0x0000,   /* U+1D462 */
  0x0000,   /* U+1D463 */
  0x0000,   /* U+1D464 */
  0x0000,   /* U+1D465 */
  0x0000,   /* U+1D466 */
  0x0000,   /* U+1D467 */
  0x0000,   /* U+1D468 */
  0x0000,   /* U+1D469 */
  0x0000,   /* U+1D46A */
  0x0000,   /* U+1D46B */
  0x0000,   /* U+1D46C */
  0x0000,   /* U+1D46D */
  0x0000,   /* U+1D46E */
  0x0000,   /* U+1D46F */
  0x0000,   /* U+1D470 */
  0x0000,   /* U+1D471 */
  0x0000,   /* U+1D472 */
  0x0000,   /* U+1D473 */
  0x0000,   /* U+1D474 */
  0x0000,   /* U+1D475 */
  0x0000,   /* U+1D476 */
  0x0000,   /* U+1D477 */
  0x0000,   /* U+1D478 */
  0x0000,   /* U+1D479 */
  0x0000,   /* U+1D47A */
  0x0000,   /* U+1D47B */
  0x0000,   /* U+1D47C */
  0x0000,   /* U+1D47D */
  0x0000,   /* U+1D47E */
  0x0000,   /* U+1D47F */
  0x0000,   /* U+1D480 */
  0x0000,   /* U+1D481 */
  0x0000,   /* U+1D482 */
  0x0000,   /* U+1D483 */
  0x0000,   /* U+1D484 */
  0x0000,   /* U+1D485 */
  0x0000,   /* U+1D486 */
  0x0000,   /* U+1D487 */
  0x0000,   /* U+1D488 */
  0x0000,   /* U+1D489 */
  0x0000,   /* U+1D48A */
  0x0000,   /* U+1D48B */
  0x0000,   /* U+1D48C */
  0x0000,   /* U+1D48D */
  0x0000,   /* U+1D48E */
  0x0000,   /* U+1D48F */
  0x0000,   /* U+1D490 */
  0x0000,   /* U+1D491 */
  0x0000,   /* U+1D492 */
  0x0000,   /* U+1D493 */
  0x0000,   /* U+1D494 */
  0x0000,   /* U+1D495 */
  0x0000,   /* U+1D496 */
  0x0000,   /* U+1D497 */
  0x0000,   /* U+1D498 */
  0x0000,   /* U+1D499 */
  0x0000,   /* U+1D49A */
  0x0000,   /* U+1D49B */
  0x0000,   /* U+1D49C */
  0x0000,   /* U+1D49D */
  0x0000,   /* U+1D49E */
  0x0000,   /* U+1D49F */
  0x0000,   /* U+1D4A0 */
  0x0000,   /* U+1D4A1 */
  0x0000,   /* U+1D4A2 */
  0x0000,   /* U+1D4A3 */
  0x0000,   /* U+1D4A4 */
  0x0000,   /* U+1D4A5 */
  0x0000,   /* U+1D4A6 */
  0x0000,   /* U+1D4A7 */
  0x0000,   /* U+1D4A8 */
  0x0000,   /* U+1D4A9 */
  0x0000,   /* U+1D4AA */
  0x0000,   /* U+1D4AB */
  0x0000,   /* U+1D4AC */
  0x0000,   /* U+1D4AD */
  0x0000,   /* U+1D4AE */
  0x0000,   /* U+1D4AF */
  0x0000,   /* U+1D4B0 */
  0x0000,   /* U+1D4B1 */
  0x0000,   /* U+1D4B2 */
  0x0000,   /* U+1D4B3 */
  0x0000,   /* U+1D4B4 */
  0x0000,   /* U+1D4B5 */
  0x0000,   /* U+1D4B6 */
  0x0000,   /* U+1D4B7 */
  0x0000,   /* U+1D4B8 */
  0x0000,   /* U+1D4B9 */
  0x0000,   /* U+1D4BA */
  0x0000,   /* U+1D4BB */
  0x0000,   /* U+1D4BC */
  0x0000,   /* U+1D4BD */
  0x0000,   /* U+1D4BE */
  0x0000,   /* U+1D4BF */
  0x0000,   /* U+1D4C0 */
  0x0000,   /* U+1D4C1 */
  0x0000,   /* U+1D4C2 */
  0x0000,   /* U+1D4C3 */
  0x0000,   /* U+1D4C4 */
  0x0000,   /* U+1D4C5 */
  0x0000,   /* U+1D4C6 */
  0x0000,   /* U+1D4C7 */
  0x0000,   /* U+1D4C8 */
  0x0000,   /* U+1D4C9 */
  0x0000,   /* U+1D4CA */
  0x0000,   /* U+1D4CB */
  0x0000,   /* U+1D4CC */
  0x0000,   /* U+1D4CD */
  0x0000,   /* U+1D4CE */
  0x0000,   /* U+1D4CF */
  0x0000,   /* U+1D4D0 */
  0x0000,   /* U+1D4D1 */
  0x0000,   /* U+1D4D2 */
  0x0000,   /* U+1D4D3 */
  0x0000,   /* U+1D4D4 */
  0x0000,   /* U+1D4D5 */
  0x0000,   /* U+1D4D6 */
  0x0000,   /* U+1D4D7 */
  0x0000,   /* U+1D4D8 */
  0x0000,   /* U+1D4D9 */
  0x0000,   /* U+1D4DA */
  0x0000,   /* U+1D4DB */
  0x0000,   /* U+1D4DC */
  0x0000,   /* U+1D4DD */
  0x0000,   /* U+1D4DE */
  0x0000,   /* U+1D4DF */
  0x0000,   /* U+1D4E0 */
  0x0000,   /* U+1D4E1 */
  0x0000,   /* U+1D4E2 */
  0x0000,   /* U+1D4E3 */
  0x0000,   /* U+1D4E4 */
  0x0000,   /* U+1D4E5 */
  0x0000,   /* U+1D4E6 */
  0x0000,   /* U+1D4E7 */
  0x0000,   /* U+1D4E8 */
  0x0000,   /* U+1D4E9 */
  0x0000,   /* U+1D4EA */
  0x0000,   /* U+1D4EB */
  0x0000,   /* U+1D4EC */
  0x0000,   /* U+1D4ED */
  0x0000,   /* U+1D4EE */
  0x0000,   /* U+1D4EF */
  0x0000,   /* U+1D4F0 */
  0x0000,   /* U+1D4F1 */
  0x0000,   /* U+1D4F2 */
  0x0000,   /* U+1D4F3 */
  0x0000,   /* U+1D4F4 */
  0x0000,   /* U+1D4F5 */
  0x0000,   /* U+1D4F6 */
  0x0000,   /* U+1D4F7 */
  0x0000,   /* U+1D4F8 */
  0x0000,   /* U+1D4F9 */
  0x0000,   /* U+1D4FA */
  0x0000,   /* U+1D4FB */
  0x0000,   /* U+1D4FC */
  0x0000,   /* U+1D4FD */
  0x0000,   /* U+1D4FE */
  0x0000,   /* U+1D4FF */
};

uint16 uca900_p1D5[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 2, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1, 
  1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1EF5,   /* U+1D500 */
  0x1EFF,   /* U+1D501 */
  0x1F0B,   /* U+1D502 */
  0x1F21,   /* U+1D503 */
  0x1C47,   /* U+1D504 */
  0x1C60,   /* U+1D505 */
  0xFBC3,   /* U+1D506 */
  0x1C8F,   /* U+1D507 */
  0x1CAA,   /* U+1D508 */
  0x1CE5,   /* U+1D509 */
  0x1CF4,   /* U+1D50A */
  0xFBC3,   /* U+1D50B */
  0xFBC3,   /* U+1D50C */
  0x1D4C,   /* U+1D50D */
  0x1D65,   /* U+1D50E */
  0x1D77,   /* U+1D50F */
  0x1DAA,   /* U+1D510 */
  0x1DB9,   /* U+1D511 */
  0x1DDD,   /* U+1D512 */
  0x1E0C,   /* U+1D513 */
  0x1E21,   /* U+1D514 */
  0xFBC3,   /* U+1D515 */
  0x1E71,   /* U+1D516 */
  0x1E95,   /* U+1D517 */
  0x1EB5,   /* U+1D518 */
  0x1EE3,   /* U+1D519 */
  0x1EF5,   /* U+1D51A */
  0x1EFF,   /* U+1D51B */
  0x1F0B,   /* U+1D51C */
  0xFBC3,   /* U+1D51D */
  0x1C47,   /* U+1D51E */
  0x1C60,   /* U+1D51F */
  0x1C7A,   /* U+1D520 */
  0x1C8F,   /* U+1D521 */
  0x1CAA,   /* U+1D522 */
  0x1CE5,   /* U+1D523 */
  0x1CF4,   /* U+1D524 */
  0x1D18,   /* U+1D525 */
  0x1D32,   /* U+1D526 */
  0x1D4C,   /* U+1D527 */
  0x1D65,   /* U+1D528 */
  0x1D77,   /* U+1D529 */
  0x1DAA,   /* U+1D52A */
  0x1DB9,   /* U+1D52B */
  0x1DDD,   /* U+1D52C */
  0x1E0C,   /* U+1D52D */
  0x1E21,   /* U+1D52E */
  0x1E33,   /* U+1D52F */
  0x1E71,   /* U+1D530 */
  0x1E95,   /* U+1D531 */
  0x1EB5,   /* U+1D532 */
  0x1EE3,   /* U+1D533 */
  0x1EF5,   /* U+1D534 */
  0x1EFF,   /* U+1D535 */
  0x1F0B,   /* U+1D536 */
  0x1F21,   /* U+1D537 */
  0x1C47,   /* U+1D538 */
  0x1C60,   /* U+1D539 */
  0xFBC3,   /* U+1D53A */
  0x1C8F,   /* U+1D53B */
  0x1CAA,   /* U+1D53C */
  0x1CE5,   /* U+1D53D */
  0x1CF4,   /* U+1D53E */
  0xFBC3,   /* U+1D53F */
  0x1D32,   /* U+1D540 */
  0x1D4C,   /* U+1D541 */
  0x1D65,   /* U+1D542 */
  0x1D77,   /* U+1D543 */
  0x1DAA,   /* U+1D544 */
  0xFBC3,   /* U+1D545 */
  0x1DDD,   /* U+1D546 */
  0xFBC3,   /* U+1D547 */
  0xFBC3,   /* U+1D548 */
  0xFBC3,   /* U+1D549 */
  0x1E71,   /* U+1D54A */
  0x1E95,   /* U+1D54B */
  0x1EB5,   /* U+1D54C */
  0x1EE3,   /* U+1D54D */
  0x1EF5,   /* U+1D54E */
  0x1EFF,   /* U+1D54F */
  0x1F0B,   /* U+1D550 */
  0xFBC3,   /* U+1D551 */
  0x1C47,   /* U+1D552 */
  0x1C60,   /* U+1D553 */
  0x1C7A,   /* U+1D554 */
  0x1C8F,   /* U+1D555 */
  0x1CAA,   /* U+1D556 */
  0x1CE5,   /* U+1D557 */
  0x1CF4,   /* U+1D558 */
  0x1D18,   /* U+1D559 */
  0x1D32,   /* U+1D55A */
  0x1D4C,   /* U+1D55B */
  0x1D65,   /* U+1D55C */
  0x1D77,   /* U+1D55D */
  0x1DAA,   /* U+1D55E */
  0x1DB9,   /* U+1D55F */
  0x1DDD,   /* U+1D560 */
  0x1E0C,   /* U+1D561 */
  0x1E21,   /* U+1D562 */
  0x1E33,   /* U+1D563 */
  0x1E71,   /* U+1D564 */
  0x1E95,   /* U+1D565 */
  0x1EB5,   /* U+1D566 */
  0x1EE3,   /* U+1D567 */
  0x1EF5,   /* U+1D568 */
  0x1EFF,   /* U+1D569 */
  0x1F0B,   /* U+1D56A */
  0x1F21,   /* U+1D56B */
  0x1C47,   /* U+1D56C */
  0x1C60,   /* U+1D56D */
  0x1C7A,   /* U+1D56E */
  0x1C8F,   /* U+1D56F */
  0x1CAA,   /* U+1D570 */
  0x1CE5,   /* U+1D571 */
  0x1CF4,   /* U+1D572 */
  0x1D18,   /* U+1D573 */
  0x1D32,   /* U+1D574 */
  0x1D4C,   /* U+1D575 */
  0x1D65,   /* U+1D576 */
  0x1D77,   /* U+1D577 */
  0x1DAA,   /* U+1D578 */
  0x1DB9,   /* U+1D579 */
  0x1DDD,   /* U+1D57A */
  0x1E0C,   /* U+1D57B */
  0x1E21,   /* U+1D57C */
  0x1E33,   /* U+1D57D */
  0x1E71,   /* U+1D57E */
  0x1E95,   /* U+1D57F */
  0x1EB5,   /* U+1D580 */
  0x1EE3,   /* U+1D581 */
  0x1EF5,   /* U+1D582 */
  0x1EFF,   /* U+1D583 */
  0x1F0B,   /* U+1D584 */
  0x1F21,   /* U+1D585 */
  0x1C47,   /* U+1D586 */
  0x1C60,   /* U+1D587 */
  0x1C7A,   /* U+1D588 */
  0x1C8F,   /* U+1D589 */
  0x1CAA,   /* U+1D58A */
  0x1CE5,   /* U+1D58B */
  0x1CF4,   /* U+1D58C */
  0x1D18,   /* U+1D58D */
  0x1D32,   /* U+1D58E */
  0x1D4C,   /* U+1D58F */
  0x1D65,   /* U+1D590 */
  0x1D77,   /* U+1D591 */
  0x1DAA,   /* U+1D592 */
  0x1DB9,   /* U+1D593 */
  0x1DDD,   /* U+1D594 */
  0x1E0C,   /* U+1D595 */
  0x1E21,   /* U+1D596 */
  0x1E33,   /* U+1D597 */
  0x1E71,   /* U+1D598 */
  0x1E95,   /* U+1D599 */
  0x1EB5,   /* U+1D59A */
  0x1EE3,   /* U+1D59B */
  0x1EF5,   /* U+1D59C */
  0x1EFF,   /* U+1D59D */
  0x1F0B,   /* U+1D59E */
  0x1F21,   /* U+1D59F */
  0x1C47,   /* U+1D5A0 */
  0x1C60,   /* U+1D5A1 */
  0x1C7A,   /* U+1D5A2 */
  0x1C8F,   /* U+1D5A3 */
  0x1CAA,   /* U+1D5A4 */
  0x1CE5,   /* U+1D5A5 */
  0x1CF4,   /* U+1D5A6 */
  0x1D18,   /* U+1D5A7 */
  0x1D32,   /* U+1D5A8 */
  0x1D4C,   /* U+1D5A9 */
  0x1D65,   /* U+1D5AA */
  0x1D77,   /* U+1D5AB */
  0x1DAA,   /* U+1D5AC */
  0x1DB9,   /* U+1D5AD */
  0x1DDD,   /* U+1D5AE */
  0x1E0C,   /* U+1D5AF */
  0x1E21,   /* U+1D5B0 */
  0x1E33,   /* U+1D5B1 */
  0x1E71,   /* U+1D5B2 */
  0x1E95,   /* U+1D5B3 */
  0x1EB5,   /* U+1D5B4 */
  0x1EE3,   /* U+1D5B5 */
  0x1EF5,   /* U+1D5B6 */
  0x1EFF,   /* U+1D5B7 */
  0x1F0B,   /* U+1D5B8 */
  0x1F21,   /* U+1D5B9 */
  0x1C47,   /* U+1D5BA */
  0x1C60,   /* U+1D5BB */
  0x1C7A,   /* U+1D5BC */
  0x1C8F,   /* U+1D5BD */
  0x1CAA,   /* U+1D5BE */
  0x1CE5,   /* U+1D5BF */
  0x1CF4,   /* U+1D5C0 */
  0x1D18,   /* U+1D5C1 */
  0x1D32,   /* U+1D5C2 */
  0x1D4C,   /* U+1D5C3 */
  0x1D65,   /* U+1D5C4 */
  0x1D77,   /* U+1D5C5 */
  0x1DAA,   /* U+1D5C6 */
  0x1DB9,   /* U+1D5C7 */
  0x1DDD,   /* U+1D5C8 */
  0x1E0C,   /* U+1D5C9 */
  0x1E21,   /* U+1D5CA */
  0x1E33,   /* U+1D5CB */
  0x1E71,   /* U+1D5CC */
  0x1E95,   /* U+1D5CD */
  0x1EB5,   /* U+1D5CE */
  0x1EE3,   /* U+1D5CF */
  0x1EF5,   /* U+1D5D0 */
  0x1EFF,   /* U+1D5D1 */
  0x1F0B,   /* U+1D5D2 */
  0x1F21,   /* U+1D5D3 */
  0x1C47,   /* U+1D5D4 */
  0x1C60,   /* U+1D5D5 */
  0x1C7A,   /* U+1D5D6 */
  0x1C8F,   /* U+1D5D7 */
  0x1CAA,   /* U+1D5D8 */
  0x1CE5,   /* U+1D5D9 */
  0x1CF4,   /* U+1D5DA */
  0x1D18,   /* U+1D5DB */
  0x1D32,   /* U+1D5DC */
  0x1D4C,   /* U+1D5DD */
  0x1D65,   /* U+1D5DE */
  0x1D77,   /* U+1D5DF */
  0x1DAA,   /* U+1D5E0 */
  0x1DB9,   /* U+1D5E1 */
  0x1DDD,   /* U+1D5E2 */
  0x1E0C,   /* U+1D5E3 */
  0x1E21,   /* U+1D5E4 */
  0x1E33,   /* U+1D5E5 */
  0x1E71,   /* U+1D5E6 */
  0x1E95,   /* U+1D5E7 */
  0x1EB5,   /* U+1D5E8 */
  0x1EE3,   /* U+1D5E9 */
  0x1EF5,   /* U+1D5EA */
  0x1EFF,   /* U+1D5EB */
  0x1F0B,   /* U+1D5EC */
  0x1F21,   /* U+1D5ED */
  0x1C47,   /* U+1D5EE */
  0x1C60,   /* U+1D5EF */
  0x1C7A,   /* U+1D5F0 */
  0x1C8F,   /* U+1D5F1 */
  0x1CAA,   /* U+1D5F2 */
  0x1CE5,   /* U+1D5F3 */
  0x1CF4,   /* U+1D5F4 */
  0x1D18,   /* U+1D5F5 */
  0x1D32,   /* U+1D5F6 */
  0x1D4C,   /* U+1D5F7 */
  0x1D65,   /* U+1D5F8 */
  0x1D77,   /* U+1D5F9 */
  0x1DAA,   /* U+1D5FA */
  0x1DB9,   /* U+1D5FB */
  0x1DDD,   /* U+1D5FC */
  0x1E0C,   /* U+1D5FD */
  0x1E21,   /* U+1D5FE */
  0x1E33,   /* U+1D5FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D500 */
  0x0020,   /* U+1D501 */
  0x0020,   /* U+1D502 */
  0x0020,   /* U+1D503 */
  0x0020,   /* U+1D504 */
  0x0020,   /* U+1D505 */
  0x0020,   /* U+1D506 */
  0x0020,   /* U+1D507 */
  0x0020,   /* U+1D508 */
  0x0020,   /* U+1D509 */
  0x0020,   /* U+1D50A */
  0x0020,   /* U+1D50B */
  0x0020,   /* U+1D50C */
  0x0020,   /* U+1D50D */
  0x0020,   /* U+1D50E */
  0x0020,   /* U+1D50F */
  0x0020,   /* U+1D510 */
  0x0020,   /* U+1D511 */
  0x0020,   /* U+1D512 */
  0x0020,   /* U+1D513 */
  0x0020,   /* U+1D514 */
  0x0020,   /* U+1D515 */
  0x0020,   /* U+1D516 */
  0x0020,   /* U+1D517 */
  0x0020,   /* U+1D518 */
  0x0020,   /* U+1D519 */
  0x0020,   /* U+1D51A */
  0x0020,   /* U+1D51B */
  0x0020,   /* U+1D51C */
  0x0020,   /* U+1D51D */
  0x0020,   /* U+1D51E */
  0x0020,   /* U+1D51F */
  0x0020,   /* U+1D520 */
  0x0020,   /* U+1D521 */
  0x0020,   /* U+1D522 */
  0x0020,   /* U+1D523 */
  0x0020,   /* U+1D524 */
  0x0020,   /* U+1D525 */
  0x0020,   /* U+1D526 */
  0x0020,   /* U+1D527 */
  0x0020,   /* U+1D528 */
  0x0020,   /* U+1D529 */
  0x0020,   /* U+1D52A */
  0x0020,   /* U+1D52B */
  0x0020,   /* U+1D52C */
  0x0020,   /* U+1D52D */
  0x0020,   /* U+1D52E */
  0x0020,   /* U+1D52F */
  0x0020,   /* U+1D530 */
  0x0020,   /* U+1D531 */
  0x0020,   /* U+1D532 */
  0x0020,   /* U+1D533 */
  0x0020,   /* U+1D534 */
  0x0020,   /* U+1D535 */
  0x0020,   /* U+1D536 */
  0x0020,   /* U+1D537 */
  0x0020,   /* U+1D538 */
  0x0020,   /* U+1D539 */
  0x0020,   /* U+1D53A */
  0x0020,   /* U+1D53B */
  0x0020,   /* U+1D53C */
  0x0020,   /* U+1D53D */
  0x0020,   /* U+1D53E */
  0x0020,   /* U+1D53F */
  0x0020,   /* U+1D540 */
  0x0020,   /* U+1D541 */
  0x0020,   /* U+1D542 */
  0x0020,   /* U+1D543 */
  0x0020,   /* U+1D544 */
  0x0020,   /* U+1D545 */
  0x0020,   /* U+1D546 */
  0x0020,   /* U+1D547 */
  0x0020,   /* U+1D548 */
  0x0020,   /* U+1D549 */
  0x0020,   /* U+1D54A */
  0x0020,   /* U+1D54B */
  0x0020,   /* U+1D54C */
  0x0020,   /* U+1D54D */
  0x0020,   /* U+1D54E */
  0x0020,   /* U+1D54F */
  0x0020,   /* U+1D550 */
  0x0020,   /* U+1D551 */
  0x0020,   /* U+1D552 */
  0x0020,   /* U+1D553 */
  0x0020,   /* U+1D554 */
  0x0020,   /* U+1D555 */
  0x0020,   /* U+1D556 */
  0x0020,   /* U+1D557 */
  0x0020,   /* U+1D558 */
  0x0020,   /* U+1D559 */
  0x0020,   /* U+1D55A */
  0x0020,   /* U+1D55B */
  0x0020,   /* U+1D55C */
  0x0020,   /* U+1D55D */
  0x0020,   /* U+1D55E */
  0x0020,   /* U+1D55F */
  0x0020,   /* U+1D560 */
  0x0020,   /* U+1D561 */
  0x0020,   /* U+1D562 */
  0x0020,   /* U+1D563 */
  0x0020,   /* U+1D564 */
  0x0020,   /* U+1D565 */
  0x0020,   /* U+1D566 */
  0x0020,   /* U+1D567 */
  0x0020,   /* U+1D568 */
  0x0020,   /* U+1D569 */
  0x0020,   /* U+1D56A */
  0x0020,   /* U+1D56B */
  0x0020,   /* U+1D56C */
  0x0020,   /* U+1D56D */
  0x0020,   /* U+1D56E */
  0x0020,   /* U+1D56F */
  0x0020,   /* U+1D570 */
  0x0020,   /* U+1D571 */
  0x0020,   /* U+1D572 */
  0x0020,   /* U+1D573 */
  0x0020,   /* U+1D574 */
  0x0020,   /* U+1D575 */
  0x0020,   /* U+1D576 */
  0x0020,   /* U+1D577 */
  0x0020,   /* U+1D578 */
  0x0020,   /* U+1D579 */
  0x0020,   /* U+1D57A */
  0x0020,   /* U+1D57B */
  0x0020,   /* U+1D57C */
  0x0020,   /* U+1D57D */
  0x0020,   /* U+1D57E */
  0x0020,   /* U+1D57F */
  0x0020,   /* U+1D580 */
  0x0020,   /* U+1D581 */
  0x0020,   /* U+1D582 */
  0x0020,   /* U+1D583 */
  0x0020,   /* U+1D584 */
  0x0020,   /* U+1D585 */
  0x0020,   /* U+1D586 */
  0x0020,   /* U+1D587 */
  0x0020,   /* U+1D588 */
  0x0020,   /* U+1D589 */
  0x0020,   /* U+1D58A */
  0x0020,   /* U+1D58B */
  0x0020,   /* U+1D58C */
  0x0020,   /* U+1D58D */
  0x0020,   /* U+1D58E */
  0x0020,   /* U+1D58F */
  0x0020,   /* U+1D590 */
  0x0020,   /* U+1D591 */
  0x0020,   /* U+1D592 */
  0x0020,   /* U+1D593 */
  0x0020,   /* U+1D594 */
  0x0020,   /* U+1D595 */
  0x0020,   /* U+1D596 */
  0x0020,   /* U+1D597 */
  0x0020,   /* U+1D598 */
  0x0020,   /* U+1D599 */
  0x0020,   /* U+1D59A */
  0x0020,   /* U+1D59B */
  0x0020,   /* U+1D59C */
  0x0020,   /* U+1D59D */
  0x0020,   /* U+1D59E */
  0x0020,   /* U+1D59F */
  0x0020,   /* U+1D5A0 */
  0x0020,   /* U+1D5A1 */
  0x0020,   /* U+1D5A2 */
  0x0020,   /* U+1D5A3 */
  0x0020,   /* U+1D5A4 */
  0x0020,   /* U+1D5A5 */
  0x0020,   /* U+1D5A6 */
  0x0020,   /* U+1D5A7 */
  0x0020,   /* U+1D5A8 */
  0x0020,   /* U+1D5A9 */
  0x0020,   /* U+1D5AA */
  0x0020,   /* U+1D5AB */
  0x0020,   /* U+1D5AC */
  0x0020,   /* U+1D5AD */
  0x0020,   /* U+1D5AE */
  0x0020,   /* U+1D5AF */
  0x0020,   /* U+1D5B0 */
  0x0020,   /* U+1D5B1 */
  0x0020,   /* U+1D5B2 */
  0x0020,   /* U+1D5B3 */
  0x0020,   /* U+1D5B4 */
  0x0020,   /* U+1D5B5 */
  0x0020,   /* U+1D5B6 */
  0x0020,   /* U+1D5B7 */
  0x0020,   /* U+1D5B8 */
  0x0020,   /* U+1D5B9 */
  0x0020,   /* U+1D5BA */
  0x0020,   /* U+1D5BB */
  0x0020,   /* U+1D5BC */
  0x0020,   /* U+1D5BD */
  0x0020,   /* U+1D5BE */
  0x0020,   /* U+1D5BF */
  0x0020,   /* U+1D5C0 */
  0x0020,   /* U+1D5C1 */
  0x0020,   /* U+1D5C2 */
  0x0020,   /* U+1D5C3 */
  0x0020,   /* U+1D5C4 */
  0x0020,   /* U+1D5C5 */
  0x0020,   /* U+1D5C6 */
  0x0020,   /* U+1D5C7 */
  0x0020,   /* U+1D5C8 */
  0x0020,   /* U+1D5C9 */
  0x0020,   /* U+1D5CA */
  0x0020,   /* U+1D5CB */
  0x0020,   /* U+1D5CC */
  0x0020,   /* U+1D5CD */
  0x0020,   /* U+1D5CE */
  0x0020,   /* U+1D5CF */
  0x0020,   /* U+1D5D0 */
  0x0020,   /* U+1D5D1 */
  0x0020,   /* U+1D5D2 */
  0x0020,   /* U+1D5D3 */
  0x0020,   /* U+1D5D4 */
  0x0020,   /* U+1D5D5 */
  0x0020,   /* U+1D5D6 */
  0x0020,   /* U+1D5D7 */
  0x0020,   /* U+1D5D8 */
  0x0020,   /* U+1D5D9 */
  0x0020,   /* U+1D5DA */
  0x0020,   /* U+1D5DB */
  0x0020,   /* U+1D5DC */
  0x0020,   /* U+1D5DD */
  0x0020,   /* U+1D5DE */
  0x0020,   /* U+1D5DF */
  0x0020,   /* U+1D5E0 */
  0x0020,   /* U+1D5E1 */
  0x0020,   /* U+1D5E2 */
  0x0020,   /* U+1D5E3 */
  0x0020,   /* U+1D5E4 */
  0x0020,   /* U+1D5E5 */
  0x0020,   /* U+1D5E6 */
  0x0020,   /* U+1D5E7 */
  0x0020,   /* U+1D5E8 */
  0x0020,   /* U+1D5E9 */
  0x0020,   /* U+1D5EA */
  0x0020,   /* U+1D5EB */
  0x0020,   /* U+1D5EC */
  0x0020,   /* U+1D5ED */
  0x0020,   /* U+1D5EE */
  0x0020,   /* U+1D5EF */
  0x0020,   /* U+1D5F0 */
  0x0020,   /* U+1D5F1 */
  0x0020,   /* U+1D5F2 */
  0x0020,   /* U+1D5F3 */
  0x0020,   /* U+1D5F4 */
  0x0020,   /* U+1D5F5 */
  0x0020,   /* U+1D5F6 */
  0x0020,   /* U+1D5F7 */
  0x0020,   /* U+1D5F8 */
  0x0020,   /* U+1D5F9 */
  0x0020,   /* U+1D5FA */
  0x0020,   /* U+1D5FB */
  0x0020,   /* U+1D5FC */
  0x0020,   /* U+1D5FD */
  0x0020,   /* U+1D5FE */
  0x0020,   /* U+1D5FF */

  /* Tertiary weight 1 for each character. */
  0x0005,   /* U+1D500 */
  0x0005,   /* U+1D501 */
  0x0005,   /* U+1D502 */
  0x0005,   /* U+1D503 */
  0x000B,   /* U+1D504 */
  0x000B,   /* U+1D505 */
  0x0002,   /* U+1D506 */
  0x000B,   /* U+1D507 */
  0x000B,   /* U+1D508 */
  0x000B,   /* U+1D509 */
  0x000B,   /* U+1D50A */
  0x0002,   /* U+1D50B */
  0x0002,   /* U+1D50C */
  0x000B,   /* U+1D50D */
  0x000B,   /* U+1D50E */
  0x000B,   /* U+1D50F */
  0x000B,   /* U+1D510 */
  0x000B,   /* U+1D511 */
  0x000B,   /* U+1D512 */
  0x000B,   /* U+1D513 */
  0x000B,   /* U+1D514 */
  0x0002,   /* U+1D515 */
  0x000B,   /* U+1D516 */
  0x000B,   /* U+1D517 */
  0x000B,   /* U+1D518 */
  0x000B,   /* U+1D519 */
  0x000B,   /* U+1D51A */
  0x000B,   /* U+1D51B */
  0x000B,   /* U+1D51C */
  0x0002,   /* U+1D51D */
  0x0005,   /* U+1D51E */
  0x0005,   /* U+1D51F */
  0x0005,   /* U+1D520 */
  0x0005,   /* U+1D521 */
  0x0005,   /* U+1D522 */
  0x0005,   /* U+1D523 */
  0x0005,   /* U+1D524 */
  0x0005,   /* U+1D525 */
  0x0005,   /* U+1D526 */
  0x0005,   /* U+1D527 */
  0x0005,   /* U+1D528 */
  0x0005,   /* U+1D529 */
  0x0005,   /* U+1D52A */
  0x0005,   /* U+1D52B */
  0x0005,   /* U+1D52C */
  0x0005,   /* U+1D52D */
  0x0005,   /* U+1D52E */
  0x0005,   /* U+1D52F */
  0x0005,   /* U+1D530 */
  0x0005,   /* U+1D531 */
  0x0005,   /* U+1D532 */
  0x0005,   /* U+1D533 */
  0x0005,   /* U+1D534 */
  0x0005,   /* U+1D535 */
  0x0005,   /* U+1D536 */
  0x0005,   /* U+1D537 */
  0x000B,   /* U+1D538 */
  0x000B,   /* U+1D539 */
  0x0002,   /* U+1D53A */
  0x000B,   /* U+1D53B */
  0x000B,   /* U+1D53C */
  0x000B,   /* U+1D53D */
  0x000B,   /* U+1D53E */
  0x0002,   /* U+1D53F */
  0x000B,   /* U+1D540 */
  0x000B,   /* U+1D541 */
  0x000B,   /* U+1D542 */
  0x000B,   /* U+1D543 */
  0x000B,   /* U+1D544 */
  0x0002,   /* U+1D545 */
  0x000B,   /* U+1D546 */
  0x0002,   /* U+1D547 */
  0x0002,   /* U+1D548 */
  0x0002,   /* U+1D549 */
  0x000B,   /* U+1D54A */
  0x000B,   /* U+1D54B */
  0x000B,   /* U+1D54C */
  0x000B,   /* U+1D54D */
  0x000B,   /* U+1D54E */
  0x000B,   /* U+1D54F */
  0x000B,   /* U+1D550 */
  0x0002,   /* U+1D551 */
  0x0005,   /* U+1D552 */
  0x0005,   /* U+1D553 */
  0x0005,   /* U+1D554 */
  0x0005,   /* U+1D555 */
  0x0005,   /* U+1D556 */
  0x0005,   /* U+1D557 */
  0x0005,   /* U+1D558 */
  0x0005,   /* U+1D559 */
  0x0005,   /* U+1D55A */
  0x0005,   /* U+1D55B */
  0x0005,   /* U+1D55C */
  0x0005,   /* U+1D55D */
  0x0005,   /* U+1D55E */
  0x0005,   /* U+1D55F */
  0x0005,   /* U+1D560 */
  0x0005,   /* U+1D561 */
  0x0005,   /* U+1D562 */
  0x0005,   /* U+1D563 */
  0x0005,   /* U+1D564 */
  0x0005,   /* U+1D565 */
  0x0005,   /* U+1D566 */
  0x0005,   /* U+1D567 */
  0x0005,   /* U+1D568 */
  0x0005,   /* U+1D569 */
  0x0005,   /* U+1D56A */
  0x0005,   /* U+1D56B */
  0x000B,   /* U+1D56C */
  0x000B,   /* U+1D56D */
  0x000B,   /* U+1D56E */
  0x000B,   /* U+1D56F */
  0x000B,   /* U+1D570 */
  0x000B,   /* U+1D571 */
  0x000B,   /* U+1D572 */
  0x000B,   /* U+1D573 */
  0x000B,   /* U+1D574 */
  0x000B,   /* U+1D575 */
  0x000B,   /* U+1D576 */
  0x000B,   /* U+1D577 */
  0x000B,   /* U+1D578 */
  0x000B,   /* U+1D579 */
  0x000B,   /* U+1D57A */
  0x000B,   /* U+1D57B */
  0x000B,   /* U+1D57C */
  0x000B,   /* U+1D57D */
  0x000B,   /* U+1D57E */
  0x000B,   /* U+1D57F */
  0x000B,   /* U+1D580 */
  0x000B,   /* U+1D581 */
  0x000B,   /* U+1D582 */
  0x000B,   /* U+1D583 */
  0x000B,   /* U+1D584 */
  0x000B,   /* U+1D585 */
  0x0005,   /* U+1D586 */
  0x0005,   /* U+1D587 */
  0x0005,   /* U+1D588 */
  0x0005,   /* U+1D589 */
  0x0005,   /* U+1D58A */
  0x0005,   /* U+1D58B */
  0x0005,   /* U+1D58C */
  0x0005,   /* U+1D58D */
  0x0005,   /* U+1D58E */
  0x0005,   /* U+1D58F */
  0x0005,   /* U+1D590 */
  0x0005,   /* U+1D591 */
  0x0005,   /* U+1D592 */
  0x0005,   /* U+1D593 */
  0x0005,   /* U+1D594 */
  0x0005,   /* U+1D595 */
  0x0005,   /* U+1D596 */
  0x0005,   /* U+1D597 */
  0x0005,   /* U+1D598 */
  0x0005,   /* U+1D599 */
  0x0005,   /* U+1D59A */
  0x0005,   /* U+1D59B */
  0x0005,   /* U+1D59C */
  0x0005,   /* U+1D59D */
  0x0005,   /* U+1D59E */
  0x0005,   /* U+1D59F */
  0x000B,   /* U+1D5A0 */
  0x000B,   /* U+1D5A1 */
  0x000B,   /* U+1D5A2 */
  0x000B,   /* U+1D5A3 */
  0x000B,   /* U+1D5A4 */
  0x000B,   /* U+1D5A5 */
  0x000B,   /* U+1D5A6 */
  0x000B,   /* U+1D5A7 */
  0x000B,   /* U+1D5A8 */
  0x000B,   /* U+1D5A9 */
  0x000B,   /* U+1D5AA */
  0x000B,   /* U+1D5AB */
  0x000B,   /* U+1D5AC */
  0x000B,   /* U+1D5AD */
  0x000B,   /* U+1D5AE */
  0x000B,   /* U+1D5AF */
  0x000B,   /* U+1D5B0 */
  0x000B,   /* U+1D5B1 */
  0x000B,   /* U+1D5B2 */
  0x000B,   /* U+1D5B3 */
  0x000B,   /* U+1D5B4 */
  0x000B,   /* U+1D5B5 */
  0x000B,   /* U+1D5B6 */
  0x000B,   /* U+1D5B7 */
  0x000B,   /* U+1D5B8 */
  0x000B,   /* U+1D5B9 */
  0x0005,   /* U+1D5BA */
  0x0005,   /* U+1D5BB */
  0x0005,   /* U+1D5BC */
  0x0005,   /* U+1D5BD */
  0x0005,   /* U+1D5BE */
  0x0005,   /* U+1D5BF */
  0x0005,   /* U+1D5C0 */
  0x0005,   /* U+1D5C1 */
  0x0005,   /* U+1D5C2 */
  0x0005,   /* U+1D5C3 */
  0x0005,   /* U+1D5C4 */
  0x0005,   /* U+1D5C5 */
  0x0005,   /* U+1D5C6 */
  0x0005,   /* U+1D5C7 */
  0x0005,   /* U+1D5C8 */
  0x0005,   /* U+1D5C9 */
  0x0005,   /* U+1D5CA */
  0x0005,   /* U+1D5CB */
  0x0005,   /* U+1D5CC */
  0x0005,   /* U+1D5CD */
  0x0005,   /* U+1D5CE */
  0x0005,   /* U+1D5CF */
  0x0005,   /* U+1D5D0 */
  0x0005,   /* U+1D5D1 */
  0x0005,   /* U+1D5D2 */
  0x0005,   /* U+1D5D3 */
  0x000B,   /* U+1D5D4 */
  0x000B,   /* U+1D5D5 */
  0x000B,   /* U+1D5D6 */
  0x000B,   /* U+1D5D7 */
  0x000B,   /* U+1D5D8 */
  0x000B,   /* U+1D5D9 */
  0x000B,   /* U+1D5DA */
  0x000B,   /* U+1D5DB */
  0x000B,   /* U+1D5DC */
  0x000B,   /* U+1D5DD */
  0x000B,   /* U+1D5DE */
  0x000B,   /* U+1D5DF */
  0x000B,   /* U+1D5E0 */
  0x000B,   /* U+1D5E1 */
  0x000B,   /* U+1D5E2 */
  0x000B,   /* U+1D5E3 */
  0x000B,   /* U+1D5E4 */
  0x000B,   /* U+1D5E5 */
  0x000B,   /* U+1D5E6 */
  0x000B,   /* U+1D5E7 */
  0x000B,   /* U+1D5E8 */
  0x000B,   /* U+1D5E9 */
  0x000B,   /* U+1D5EA */
  0x000B,   /* U+1D5EB */
  0x000B,   /* U+1D5EC */
  0x000B,   /* U+1D5ED */
  0x0005,   /* U+1D5EE */
  0x0005,   /* U+1D5EF */
  0x0005,   /* U+1D5F0 */
  0x0005,   /* U+1D5F1 */
  0x0005,   /* U+1D5F2 */
  0x0005,   /* U+1D5F3 */
  0x0005,   /* U+1D5F4 */
  0x0005,   /* U+1D5F5 */
  0x0005,   /* U+1D5F6 */
  0x0005,   /* U+1D5F7 */
  0x0005,   /* U+1D5F8 */
  0x0005,   /* U+1D5F9 */
  0x0005,   /* U+1D5FA */
  0x0005,   /* U+1D5FB */
  0x0005,   /* U+1D5FC */
  0x0005,   /* U+1D5FD */
  0x0005,   /* U+1D5FE */
  0x0005,   /* U+1D5FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D500 */
  0x0000,   /* U+1D501 */
  0x0000,   /* U+1D502 */
  0x0000,   /* U+1D503 */
  0x0000,   /* U+1D504 */
  0x0000,   /* U+1D505 */
  0xD506,   /* U+1D506 */
  0x0000,   /* U+1D507 */
  0x0000,   /* U+1D508 */
  0x0000,   /* U+1D509 */
  0x0000,   /* U+1D50A */
  0xD50B,   /* U+1D50B */
  0xD50C,   /* U+1D50C */
  0x0000,   /* U+1D50D */
  0x0000,   /* U+1D50E */
  0x0000,   /* U+1D50F */
  0x0000,   /* U+1D510 */
  0x0000,   /* U+1D511 */
  0x0000,   /* U+1D512 */
  0x0000,   /* U+1D513 */
  0x0000,   /* U+1D514 */
  0xD515,   /* U+1D515 */
  0x0000,   /* U+1D516 */
  0x0000,   /* U+1D517 */
  0x0000,   /* U+1D518 */
  0x0000,   /* U+1D519 */
  0x0000,   /* U+1D51A */
  0x0000,   /* U+1D51B */
  0x0000,   /* U+1D51C */
  0xD51D,   /* U+1D51D */
  0x0000,   /* U+1D51E */
  0x0000,   /* U+1D51F */
  0x0000,   /* U+1D520 */
  0x0000,   /* U+1D521 */
  0x0000,   /* U+1D522 */
  0x0000,   /* U+1D523 */
  0x0000,   /* U+1D524 */
  0x0000,   /* U+1D525 */
  0x0000,   /* U+1D526 */
  0x0000,   /* U+1D527 */
  0x0000,   /* U+1D528 */
  0x0000,   /* U+1D529 */
  0x0000,   /* U+1D52A */
  0x0000,   /* U+1D52B */
  0x0000,   /* U+1D52C */
  0x0000,   /* U+1D52D */
  0x0000,   /* U+1D52E */
  0x0000,   /* U+1D52F */
  0x0000,   /* U+1D530 */
  0x0000,   /* U+1D531 */
  0x0000,   /* U+1D532 */
  0x0000,   /* U+1D533 */
  0x0000,   /* U+1D534 */
  0x0000,   /* U+1D535 */
  0x0000,   /* U+1D536 */
  0x0000,   /* U+1D537 */
  0x0000,   /* U+1D538 */
  0x0000,   /* U+1D539 */
  0xD53A,   /* U+1D53A */
  0x0000,   /* U+1D53B */
  0x0000,   /* U+1D53C */
  0x0000,   /* U+1D53D */
  0x0000,   /* U+1D53E */
  0xD53F,   /* U+1D53F */
  0x0000,   /* U+1D540 */
  0x0000,   /* U+1D541 */
  0x0000,   /* U+1D542 */
  0x0000,   /* U+1D543 */
  0x0000,   /* U+1D544 */
  0xD545,   /* U+1D545 */
  0x0000,   /* U+1D546 */
  0xD547,   /* U+1D547 */
  0xD548,   /* U+1D548 */
  0xD549,   /* U+1D549 */
  0x0000,   /* U+1D54A */
  0x0000,   /* U+1D54B */
  0x0000,   /* U+1D54C */
  0x0000,   /* U+1D54D */
  0x0000,   /* U+1D54E */
  0x0000,   /* U+1D54F */
  0x0000,   /* U+1D550 */
  0xD551,   /* U+1D551 */
  0x0000,   /* U+1D552 */
  0x0000,   /* U+1D553 */
  0x0000,   /* U+1D554 */
  0x0000,   /* U+1D555 */
  0x0000,   /* U+1D556 */
  0x0000,   /* U+1D557 */
  0x0000,   /* U+1D558 */
  0x0000,   /* U+1D559 */
  0x0000,   /* U+1D55A */
  0x0000,   /* U+1D55B */
  0x0000,   /* U+1D55C */
  0x0000,   /* U+1D55D */
  0x0000,   /* U+1D55E */
  0x0000,   /* U+1D55F */
  0x0000,   /* U+1D560 */
  0x0000,   /* U+1D561 */
  0x0000,   /* U+1D562 */
  0x0000,   /* U+1D563 */
  0x0000,   /* U+1D564 */
  0x0000,   /* U+1D565 */
  0x0000,   /* U+1D566 */
  0x0000,   /* U+1D567 */
  0x0000,   /* U+1D568 */
  0x0000,   /* U+1D569 */
  0x0000,   /* U+1D56A */
  0x0000,   /* U+1D56B */
  0x0000,   /* U+1D56C */
  0x0000,   /* U+1D56D */
  0x0000,   /* U+1D56E */
  0x0000,   /* U+1D56F */
  0x0000,   /* U+1D570 */
  0x0000,   /* U+1D571 */
  0x0000,   /* U+1D572 */
  0x0000,   /* U+1D573 */
  0x0000,   /* U+1D574 */
  0x0000,   /* U+1D575 */
  0x0000,   /* U+1D576 */
  0x0000,   /* U+1D577 */
  0x0000,   /* U+1D578 */
  0x0000,   /* U+1D579 */
  0x0000,   /* U+1D57A */
  0x0000,   /* U+1D57B */
  0x0000,   /* U+1D57C */
  0x0000,   /* U+1D57D */
  0x0000,   /* U+1D57E */
  0x0000,   /* U+1D57F */
  0x0000,   /* U+1D580 */
  0x0000,   /* U+1D581 */
  0x0000,   /* U+1D582 */
  0x0000,   /* U+1D583 */
  0x0000,   /* U+1D584 */
  0x0000,   /* U+1D585 */
  0x0000,   /* U+1D586 */
  0x0000,   /* U+1D587 */
  0x0000,   /* U+1D588 */
  0x0000,   /* U+1D589 */
  0x0000,   /* U+1D58A */
  0x0000,   /* U+1D58B */
  0x0000,   /* U+1D58C */
  0x0000,   /* U+1D58D */
  0x0000,   /* U+1D58E */
  0x0000,   /* U+1D58F */
  0x0000,   /* U+1D590 */
  0x0000,   /* U+1D591 */
  0x0000,   /* U+1D592 */
  0x0000,   /* U+1D593 */
  0x0000,   /* U+1D594 */
  0x0000,   /* U+1D595 */
  0x0000,   /* U+1D596 */
  0x0000,   /* U+1D597 */
  0x0000,   /* U+1D598 */
  0x0000,   /* U+1D599 */
  0x0000,   /* U+1D59A */
  0x0000,   /* U+1D59B */
  0x0000,   /* U+1D59C */
  0x0000,   /* U+1D59D */
  0x0000,   /* U+1D59E */
  0x0000,   /* U+1D59F */
  0x0000,   /* U+1D5A0 */
  0x0000,   /* U+1D5A1 */
  0x0000,   /* U+1D5A2 */
  0x0000,   /* U+1D5A3 */
  0x0000,   /* U+1D5A4 */
  0x0000,   /* U+1D5A5 */
  0x0000,   /* U+1D5A6 */
  0x0000,   /* U+1D5A7 */
  0x0000,   /* U+1D5A8 */
  0x0000,   /* U+1D5A9 */
  0x0000,   /* U+1D5AA */
  0x0000,   /* U+1D5AB */
  0x0000,   /* U+1D5AC */
  0x0000,   /* U+1D5AD */
  0x0000,   /* U+1D5AE */
  0x0000,   /* U+1D5AF */
  0x0000,   /* U+1D5B0 */
  0x0000,   /* U+1D5B1 */
  0x0000,   /* U+1D5B2 */
  0x0000,   /* U+1D5B3 */
  0x0000,   /* U+1D5B4 */
  0x0000,   /* U+1D5B5 */
  0x0000,   /* U+1D5B6 */
  0x0000,   /* U+1D5B7 */
  0x0000,   /* U+1D5B8 */
  0x0000,   /* U+1D5B9 */
  0x0000,   /* U+1D5BA */
  0x0000,   /* U+1D5BB */
  0x0000,   /* U+1D5BC */
  0x0000,   /* U+1D5BD */
  0x0000,   /* U+1D5BE */
  0x0000,   /* U+1D5BF */
  0x0000,   /* U+1D5C0 */
  0x0000,   /* U+1D5C1 */
  0x0000,   /* U+1D5C2 */
  0x0000,   /* U+1D5C3 */
  0x0000,   /* U+1D5C4 */
  0x0000,   /* U+1D5C5 */
  0x0000,   /* U+1D5C6 */
  0x0000,   /* U+1D5C7 */
  0x0000,   /* U+1D5C8 */
  0x0000,   /* U+1D5C9 */
  0x0000,   /* U+1D5CA */
  0x0000,   /* U+1D5CB */
  0x0000,   /* U+1D5CC */
  0x0000,   /* U+1D5CD */
  0x0000,   /* U+1D5CE */
  0x0000,   /* U+1D5CF */
  0x0000,   /* U+1D5D0 */
  0x0000,   /* U+1D5D1 */
  0x0000,   /* U+1D5D2 */
  0x0000,   /* U+1D5D3 */
  0x0000,   /* U+1D5D4 */
  0x0000,   /* U+1D5D5 */
  0x0000,   /* U+1D5D6 */
  0x0000,   /* U+1D5D7 */
  0x0000,   /* U+1D5D8 */
  0x0000,   /* U+1D5D9 */
  0x0000,   /* U+1D5DA */
  0x0000,   /* U+1D5DB */
  0x0000,   /* U+1D5DC */
  0x0000,   /* U+1D5DD */
  0x0000,   /* U+1D5DE */
  0x0000,   /* U+1D5DF */
  0x0000,   /* U+1D5E0 */
  0x0000,   /* U+1D5E1 */
  0x0000,   /* U+1D5E2 */
  0x0000,   /* U+1D5E3 */
  0x0000,   /* U+1D5E4 */
  0x0000,   /* U+1D5E5 */
  0x0000,   /* U+1D5E6 */
  0x0000,   /* U+1D5E7 */
  0x0000,   /* U+1D5E8 */
  0x0000,   /* U+1D5E9 */
  0x0000,   /* U+1D5EA */
  0x0000,   /* U+1D5EB */
  0x0000,   /* U+1D5EC */
  0x0000,   /* U+1D5ED */
  0x0000,   /* U+1D5EE */
  0x0000,   /* U+1D5EF */
  0x0000,   /* U+1D5F0 */
  0x0000,   /* U+1D5F1 */
  0x0000,   /* U+1D5F2 */
  0x0000,   /* U+1D5F3 */
  0x0000,   /* U+1D5F4 */
  0x0000,   /* U+1D5F5 */
  0x0000,   /* U+1D5F6 */
  0x0000,   /* U+1D5F7 */
  0x0000,   /* U+1D5F8 */
  0x0000,   /* U+1D5F9 */
  0x0000,   /* U+1D5FA */
  0x0000,   /* U+1D5FB */
  0x0000,   /* U+1D5FC */
  0x0000,   /* U+1D5FD */
  0x0000,   /* U+1D5FE */
  0x0000,   /* U+1D5FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D500 */
  0x0000,   /* U+1D501 */
  0x0000,   /* U+1D502 */
  0x0000,   /* U+1D503 */
  0x0000,   /* U+1D504 */
  0x0000,   /* U+1D505 */
  0x0000,   /* U+1D506 */
  0x0000,   /* U+1D507 */
  0x0000,   /* U+1D508 */
  0x0000,   /* U+1D509 */
  0x0000,   /* U+1D50A */
  0x0000,   /* U+1D50B */
  0x0000,   /* U+1D50C */
  0x0000,   /* U+1D50D */
  0x0000,   /* U+1D50E */
  0x0000,   /* U+1D50F */
  0x0000,   /* U+1D510 */
  0x0000,   /* U+1D511 */
  0x0000,   /* U+1D512 */
  0x0000,   /* U+1D513 */
  0x0000,   /* U+1D514 */
  0x0000,   /* U+1D515 */
  0x0000,   /* U+1D516 */
  0x0000,   /* U+1D517 */
  0x0000,   /* U+1D518 */
  0x0000,   /* U+1D519 */
  0x0000,   /* U+1D51A */
  0x0000,   /* U+1D51B */
  0x0000,   /* U+1D51C */
  0x0000,   /* U+1D51D */
  0x0000,   /* U+1D51E */
  0x0000,   /* U+1D51F */
  0x0000,   /* U+1D520 */
  0x0000,   /* U+1D521 */
  0x0000,   /* U+1D522 */
  0x0000,   /* U+1D523 */
  0x0000,   /* U+1D524 */
  0x0000,   /* U+1D525 */
  0x0000,   /* U+1D526 */
  0x0000,   /* U+1D527 */
  0x0000,   /* U+1D528 */
  0x0000,   /* U+1D529 */
  0x0000,   /* U+1D52A */
  0x0000,   /* U+1D52B */
  0x0000,   /* U+1D52C */
  0x0000,   /* U+1D52D */
  0x0000,   /* U+1D52E */
  0x0000,   /* U+1D52F */
  0x0000,   /* U+1D530 */
  0x0000,   /* U+1D531 */
  0x0000,   /* U+1D532 */
  0x0000,   /* U+1D533 */
  0x0000,   /* U+1D534 */
  0x0000,   /* U+1D535 */
  0x0000,   /* U+1D536 */
  0x0000,   /* U+1D537 */
  0x0000,   /* U+1D538 */
  0x0000,   /* U+1D539 */
  0x0000,   /* U+1D53A */
  0x0000,   /* U+1D53B */
  0x0000,   /* U+1D53C */
  0x0000,   /* U+1D53D */
  0x0000,   /* U+1D53E */
  0x0000,   /* U+1D53F */
  0x0000,   /* U+1D540 */
  0x0000,   /* U+1D541 */
  0x0000,   /* U+1D542 */
  0x0000,   /* U+1D543 */
  0x0000,   /* U+1D544 */
  0x0000,   /* U+1D545 */
  0x0000,   /* U+1D546 */
  0x0000,   /* U+1D547 */
  0x0000,   /* U+1D548 */
  0x0000,   /* U+1D549 */
  0x0000,   /* U+1D54A */
  0x0000,   /* U+1D54B */
  0x0000,   /* U+1D54C */
  0x0000,   /* U+1D54D */
  0x0000,   /* U+1D54E */
  0x0000,   /* U+1D54F */
  0x0000,   /* U+1D550 */
  0x0000,   /* U+1D551 */
  0x0000,   /* U+1D552 */
  0x0000,   /* U+1D553 */
  0x0000,   /* U+1D554 */
  0x0000,   /* U+1D555 */
  0x0000,   /* U+1D556 */
  0x0000,   /* U+1D557 */
  0x0000,   /* U+1D558 */
  0x0000,   /* U+1D559 */
  0x0000,   /* U+1D55A */
  0x0000,   /* U+1D55B */
  0x0000,   /* U+1D55C */
  0x0000,   /* U+1D55D */
  0x0000,   /* U+1D55E */
  0x0000,   /* U+1D55F */
  0x0000,   /* U+1D560 */
  0x0000,   /* U+1D561 */
  0x0000,   /* U+1D562 */
  0x0000,   /* U+1D563 */
  0x0000,   /* U+1D564 */
  0x0000,   /* U+1D565 */
  0x0000,   /* U+1D566 */
  0x0000,   /* U+1D567 */
  0x0000,   /* U+1D568 */
  0x0000,   /* U+1D569 */
  0x0000,   /* U+1D56A */
  0x0000,   /* U+1D56B */
  0x0000,   /* U+1D56C */
  0x0000,   /* U+1D56D */
  0x0000,   /* U+1D56E */
  0x0000,   /* U+1D56F */
  0x0000,   /* U+1D570 */
  0x0000,   /* U+1D571 */
  0x0000,   /* U+1D572 */
  0x0000,   /* U+1D573 */
  0x0000,   /* U+1D574 */
  0x0000,   /* U+1D575 */
  0x0000,   /* U+1D576 */
  0x0000,   /* U+1D577 */
  0x0000,   /* U+1D578 */
  0x0000,   /* U+1D579 */
  0x0000,   /* U+1D57A */
  0x0000,   /* U+1D57B */
  0x0000,   /* U+1D57C */
  0x0000,   /* U+1D57D */
  0x0000,   /* U+1D57E */
  0x0000,   /* U+1D57F */
  0x0000,   /* U+1D580 */
  0x0000,   /* U+1D581 */
  0x0000,   /* U+1D582 */
  0x0000,   /* U+1D583 */
  0x0000,   /* U+1D584 */
  0x0000,   /* U+1D585 */
  0x0000,   /* U+1D586 */
  0x0000,   /* U+1D587 */
  0x0000,   /* U+1D588 */
  0x0000,   /* U+1D589 */
  0x0000,   /* U+1D58A */
  0x0000,   /* U+1D58B */
  0x0000,   /* U+1D58C */
  0x0000,   /* U+1D58D */
  0x0000,   /* U+1D58E */
  0x0000,   /* U+1D58F */
  0x0000,   /* U+1D590 */
  0x0000,   /* U+1D591 */
  0x0000,   /* U+1D592 */
  0x0000,   /* U+1D593 */
  0x0000,   /* U+1D594 */
  0x0000,   /* U+1D595 */
  0x0000,   /* U+1D596 */
  0x0000,   /* U+1D597 */
  0x0000,   /* U+1D598 */
  0x0000,   /* U+1D599 */
  0x0000,   /* U+1D59A */
  0x0000,   /* U+1D59B */
  0x0000,   /* U+1D59C */
  0x0000,   /* U+1D59D */
  0x0000,   /* U+1D59E */
  0x0000,   /* U+1D59F */
  0x0000,   /* U+1D5A0 */
  0x0000,   /* U+1D5A1 */
  0x0000,   /* U+1D5A2 */
  0x0000,   /* U+1D5A3 */
  0x0000,   /* U+1D5A4 */
  0x0000,   /* U+1D5A5 */
  0x0000,   /* U+1D5A6 */
  0x0000,   /* U+1D5A7 */
  0x0000,   /* U+1D5A8 */
  0x0000,   /* U+1D5A9 */
  0x0000,   /* U+1D5AA */
  0x0000,   /* U+1D5AB */
  0x0000,   /* U+1D5AC */
  0x0000,   /* U+1D5AD */
  0x0000,   /* U+1D5AE */
  0x0000,   /* U+1D5AF */
  0x0000,   /* U+1D5B0 */
  0x0000,   /* U+1D5B1 */
  0x0000,   /* U+1D5B2 */
  0x0000,   /* U+1D5B3 */
  0x0000,   /* U+1D5B4 */
  0x0000,   /* U+1D5B5 */
  0x0000,   /* U+1D5B6 */
  0x0000,   /* U+1D5B7 */
  0x0000,   /* U+1D5B8 */
  0x0000,   /* U+1D5B9 */
  0x0000,   /* U+1D5BA */
  0x0000,   /* U+1D5BB */
  0x0000,   /* U+1D5BC */
  0x0000,   /* U+1D5BD */
  0x0000,   /* U+1D5BE */
  0x0000,   /* U+1D5BF */
  0x0000,   /* U+1D5C0 */
  0x0000,   /* U+1D5C1 */
  0x0000,   /* U+1D5C2 */
  0x0000,   /* U+1D5C3 */
  0x0000,   /* U+1D5C4 */
  0x0000,   /* U+1D5C5 */
  0x0000,   /* U+1D5C6 */
  0x0000,   /* U+1D5C7 */
  0x0000,   /* U+1D5C8 */
  0x0000,   /* U+1D5C9 */
  0x0000,   /* U+1D5CA */
  0x0000,   /* U+1D5CB */
  0x0000,   /* U+1D5CC */
  0x0000,   /* U+1D5CD */
  0x0000,   /* U+1D5CE */
  0x0000,   /* U+1D5CF */
  0x0000,   /* U+1D5D0 */
  0x0000,   /* U+1D5D1 */
  0x0000,   /* U+1D5D2 */
  0x0000,   /* U+1D5D3 */
  0x0000,   /* U+1D5D4 */
  0x0000,   /* U+1D5D5 */
  0x0000,   /* U+1D5D6 */
  0x0000,   /* U+1D5D7 */
  0x0000,   /* U+1D5D8 */
  0x0000,   /* U+1D5D9 */
  0x0000,   /* U+1D5DA */
  0x0000,   /* U+1D5DB */
  0x0000,   /* U+1D5DC */
  0x0000,   /* U+1D5DD */
  0x0000,   /* U+1D5DE */
  0x0000,   /* U+1D5DF */
  0x0000,   /* U+1D5E0 */
  0x0000,   /* U+1D5E1 */
  0x0000,   /* U+1D5E2 */
  0x0000,   /* U+1D5E3 */
  0x0000,   /* U+1D5E4 */
  0x0000,   /* U+1D5E5 */
  0x0000,   /* U+1D5E6 */
  0x0000,   /* U+1D5E7 */
  0x0000,   /* U+1D5E8 */
  0x0000,   /* U+1D5E9 */
  0x0000,   /* U+1D5EA */
  0x0000,   /* U+1D5EB */
  0x0000,   /* U+1D5EC */
  0x0000,   /* U+1D5ED */
  0x0000,   /* U+1D5EE */
  0x0000,   /* U+1D5EF */
  0x0000,   /* U+1D5F0 */
  0x0000,   /* U+1D5F1 */
  0x0000,   /* U+1D5F2 */
  0x0000,   /* U+1D5F3 */
  0x0000,   /* U+1D5F4 */
  0x0000,   /* U+1D5F5 */
  0x0000,   /* U+1D5F6 */
  0x0000,   /* U+1D5F7 */
  0x0000,   /* U+1D5F8 */
  0x0000,   /* U+1D5F9 */
  0x0000,   /* U+1D5FA */
  0x0000,   /* U+1D5FB */
  0x0000,   /* U+1D5FC */
  0x0000,   /* U+1D5FD */
  0x0000,   /* U+1D5FE */
  0x0000,   /* U+1D5FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D500 */
  0x0000,   /* U+1D501 */
  0x0000,   /* U+1D502 */
  0x0000,   /* U+1D503 */
  0x0000,   /* U+1D504 */
  0x0000,   /* U+1D505 */
  0x0000,   /* U+1D506 */
  0x0000,   /* U+1D507 */
  0x0000,   /* U+1D508 */
  0x0000,   /* U+1D509 */
  0x0000,   /* U+1D50A */
  0x0000,   /* U+1D50B */
  0x0000,   /* U+1D50C */
  0x0000,   /* U+1D50D */
  0x0000,   /* U+1D50E */
  0x0000,   /* U+1D50F */
  0x0000,   /* U+1D510 */
  0x0000,   /* U+1D511 */
  0x0000,   /* U+1D512 */
  0x0000,   /* U+1D513 */
  0x0000,   /* U+1D514 */
  0x0000,   /* U+1D515 */
  0x0000,   /* U+1D516 */
  0x0000,   /* U+1D517 */
  0x0000,   /* U+1D518 */
  0x0000,   /* U+1D519 */
  0x0000,   /* U+1D51A */
  0x0000,   /* U+1D51B */
  0x0000,   /* U+1D51C */
  0x0000,   /* U+1D51D */
  0x0000,   /* U+1D51E */
  0x0000,   /* U+1D51F */
  0x0000,   /* U+1D520 */
  0x0000,   /* U+1D521 */
  0x0000,   /* U+1D522 */
  0x0000,   /* U+1D523 */
  0x0000,   /* U+1D524 */
  0x0000,   /* U+1D525 */
  0x0000,   /* U+1D526 */
  0x0000,   /* U+1D527 */
  0x0000,   /* U+1D528 */
  0x0000,   /* U+1D529 */
  0x0000,   /* U+1D52A */
  0x0000,   /* U+1D52B */
  0x0000,   /* U+1D52C */
  0x0000,   /* U+1D52D */
  0x0000,   /* U+1D52E */
  0x0000,   /* U+1D52F */
  0x0000,   /* U+1D530 */
  0x0000,   /* U+1D531 */
  0x0000,   /* U+1D532 */
  0x0000,   /* U+1D533 */
  0x0000,   /* U+1D534 */
  0x0000,   /* U+1D535 */
  0x0000,   /* U+1D536 */
  0x0000,   /* U+1D537 */
  0x0000,   /* U+1D538 */
  0x0000,   /* U+1D539 */
  0x0000,   /* U+1D53A */
  0x0000,   /* U+1D53B */
  0x0000,   /* U+1D53C */
  0x0000,   /* U+1D53D */
  0x0000,   /* U+1D53E */
  0x0000,   /* U+1D53F */
  0x0000,   /* U+1D540 */
  0x0000,   /* U+1D541 */
  0x0000,   /* U+1D542 */
  0x0000,   /* U+1D543 */
  0x0000,   /* U+1D544 */
  0x0000,   /* U+1D545 */
  0x0000,   /* U+1D546 */
  0x0000,   /* U+1D547 */
  0x0000,   /* U+1D548 */
  0x0000,   /* U+1D549 */
  0x0000,   /* U+1D54A */
  0x0000,   /* U+1D54B */
  0x0000,   /* U+1D54C */
  0x0000,   /* U+1D54D */
  0x0000,   /* U+1D54E */
  0x0000,   /* U+1D54F */
  0x0000,   /* U+1D550 */
  0x0000,   /* U+1D551 */
  0x0000,   /* U+1D552 */
  0x0000,   /* U+1D553 */
  0x0000,   /* U+1D554 */
  0x0000,   /* U+1D555 */
  0x0000,   /* U+1D556 */
  0x0000,   /* U+1D557 */
  0x0000,   /* U+1D558 */
  0x0000,   /* U+1D559 */
  0x0000,   /* U+1D55A */
  0x0000,   /* U+1D55B */
  0x0000,   /* U+1D55C */
  0x0000,   /* U+1D55D */
  0x0000,   /* U+1D55E */
  0x0000,   /* U+1D55F */
  0x0000,   /* U+1D560 */
  0x0000,   /* U+1D561 */
  0x0000,   /* U+1D562 */
  0x0000,   /* U+1D563 */
  0x0000,   /* U+1D564 */
  0x0000,   /* U+1D565 */
  0x0000,   /* U+1D566 */
  0x0000,   /* U+1D567 */
  0x0000,   /* U+1D568 */
  0x0000,   /* U+1D569 */
  0x0000,   /* U+1D56A */
  0x0000,   /* U+1D56B */
  0x0000,   /* U+1D56C */
  0x0000,   /* U+1D56D */
  0x0000,   /* U+1D56E */
  0x0000,   /* U+1D56F */
  0x0000,   /* U+1D570 */
  0x0000,   /* U+1D571 */
  0x0000,   /* U+1D572 */
  0x0000,   /* U+1D573 */
  0x0000,   /* U+1D574 */
  0x0000,   /* U+1D575 */
  0x0000,   /* U+1D576 */
  0x0000,   /* U+1D577 */
  0x0000,   /* U+1D578 */
  0x0000,   /* U+1D579 */
  0x0000,   /* U+1D57A */
  0x0000,   /* U+1D57B */
  0x0000,   /* U+1D57C */
  0x0000,   /* U+1D57D */
  0x0000,   /* U+1D57E */
  0x0000,   /* U+1D57F */
  0x0000,   /* U+1D580 */
  0x0000,   /* U+1D581 */
  0x0000,   /* U+1D582 */
  0x0000,   /* U+1D583 */
  0x0000,   /* U+1D584 */
  0x0000,   /* U+1D585 */
  0x0000,   /* U+1D586 */
  0x0000,   /* U+1D587 */
  0x0000,   /* U+1D588 */
  0x0000,   /* U+1D589 */
  0x0000,   /* U+1D58A */
  0x0000,   /* U+1D58B */
  0x0000,   /* U+1D58C */
  0x0000,   /* U+1D58D */
  0x0000,   /* U+1D58E */
  0x0000,   /* U+1D58F */
  0x0000,   /* U+1D590 */
  0x0000,   /* U+1D591 */
  0x0000,   /* U+1D592 */
  0x0000,   /* U+1D593 */
  0x0000,   /* U+1D594 */
  0x0000,   /* U+1D595 */
  0x0000,   /* U+1D596 */
  0x0000,   /* U+1D597 */
  0x0000,   /* U+1D598 */
  0x0000,   /* U+1D599 */
  0x0000,   /* U+1D59A */
  0x0000,   /* U+1D59B */
  0x0000,   /* U+1D59C */
  0x0000,   /* U+1D59D */
  0x0000,   /* U+1D59E */
  0x0000,   /* U+1D59F */
  0x0000,   /* U+1D5A0 */
  0x0000,   /* U+1D5A1 */
  0x0000,   /* U+1D5A2 */
  0x0000,   /* U+1D5A3 */
  0x0000,   /* U+1D5A4 */
  0x0000,   /* U+1D5A5 */
  0x0000,   /* U+1D5A6 */
  0x0000,   /* U+1D5A7 */
  0x0000,   /* U+1D5A8 */
  0x0000,   /* U+1D5A9 */
  0x0000,   /* U+1D5AA */
  0x0000,   /* U+1D5AB */
  0x0000,   /* U+1D5AC */
  0x0000,   /* U+1D5AD */
  0x0000,   /* U+1D5AE */
  0x0000,   /* U+1D5AF */
  0x0000,   /* U+1D5B0 */
  0x0000,   /* U+1D5B1 */
  0x0000,   /* U+1D5B2 */
  0x0000,   /* U+1D5B3 */
  0x0000,   /* U+1D5B4 */
  0x0000,   /* U+1D5B5 */
  0x0000,   /* U+1D5B6 */
  0x0000,   /* U+1D5B7 */
  0x0000,   /* U+1D5B8 */
  0x0000,   /* U+1D5B9 */
  0x0000,   /* U+1D5BA */
  0x0000,   /* U+1D5BB */
  0x0000,   /* U+1D5BC */
  0x0000,   /* U+1D5BD */
  0x0000,   /* U+1D5BE */
  0x0000,   /* U+1D5BF */
  0x0000,   /* U+1D5C0 */
  0x0000,   /* U+1D5C1 */
  0x0000,   /* U+1D5C2 */
  0x0000,   /* U+1D5C3 */
  0x0000,   /* U+1D5C4 */
  0x0000,   /* U+1D5C5 */
  0x0000,   /* U+1D5C6 */
  0x0000,   /* U+1D5C7 */
  0x0000,   /* U+1D5C8 */
  0x0000,   /* U+1D5C9 */
  0x0000,   /* U+1D5CA */
  0x0000,   /* U+1D5CB */
  0x0000,   /* U+1D5CC */
  0x0000,   /* U+1D5CD */
  0x0000,   /* U+1D5CE */
  0x0000,   /* U+1D5CF */
  0x0000,   /* U+1D5D0 */
  0x0000,   /* U+1D5D1 */
  0x0000,   /* U+1D5D2 */
  0x0000,   /* U+1D5D3 */
  0x0000,   /* U+1D5D4 */
  0x0000,   /* U+1D5D5 */
  0x0000,   /* U+1D5D6 */
  0x0000,   /* U+1D5D7 */
  0x0000,   /* U+1D5D8 */
  0x0000,   /* U+1D5D9 */
  0x0000,   /* U+1D5DA */
  0x0000,   /* U+1D5DB */
  0x0000,   /* U+1D5DC */
  0x0000,   /* U+1D5DD */
  0x0000,   /* U+1D5DE */
  0x0000,   /* U+1D5DF */
  0x0000,   /* U+1D5E0 */
  0x0000,   /* U+1D5E1 */
  0x0000,   /* U+1D5E2 */
  0x0000,   /* U+1D5E3 */
  0x0000,   /* U+1D5E4 */
  0x0000,   /* U+1D5E5 */
  0x0000,   /* U+1D5E6 */
  0x0000,   /* U+1D5E7 */
  0x0000,   /* U+1D5E8 */
  0x0000,   /* U+1D5E9 */
  0x0000,   /* U+1D5EA */
  0x0000,   /* U+1D5EB */
  0x0000,   /* U+1D5EC */
  0x0000,   /* U+1D5ED */
  0x0000,   /* U+1D5EE */
  0x0000,   /* U+1D5EF */
  0x0000,   /* U+1D5F0 */
  0x0000,   /* U+1D5F1 */
  0x0000,   /* U+1D5F2 */
  0x0000,   /* U+1D5F3 */
  0x0000,   /* U+1D5F4 */
  0x0000,   /* U+1D5F5 */
  0x0000,   /* U+1D5F6 */
  0x0000,   /* U+1D5F7 */
  0x0000,   /* U+1D5F8 */
  0x0000,   /* U+1D5F9 */
  0x0000,   /* U+1D5FA */
  0x0000,   /* U+1D5FB */
  0x0000,   /* U+1D5FC */
  0x0000,   /* U+1D5FD */
  0x0000,   /* U+1D5FE */
  0x0000,   /* U+1D5FF */
};

uint16 uca900_p1D6[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1E71,   /* U+1D600 */
  0x1E95,   /* U+1D601 */
  0x1EB5,   /* U+1D602 */
  0x1EE3,   /* U+1D603 */
  0x1EF5,   /* U+1D604 */
  0x1EFF,   /* U+1D605 */
  0x1F0B,   /* U+1D606 */
  0x1F21,   /* U+1D607 */
  0x1C47,   /* U+1D608 */
  0x1C60,   /* U+1D609 */
  0x1C7A,   /* U+1D60A */
  0x1C8F,   /* U+1D60B */
  0x1CAA,   /* U+1D60C */
  0x1CE5,   /* U+1D60D */
  0x1CF4,   /* U+1D60E */
  0x1D18,   /* U+1D60F */
  0x1D32,   /* U+1D610 */
  0x1D4C,   /* U+1D611 */
  0x1D65,   /* U+1D612 */
  0x1D77,   /* U+1D613 */
  0x1DAA,   /* U+1D614 */
  0x1DB9,   /* U+1D615 */
  0x1DDD,   /* U+1D616 */
  0x1E0C,   /* U+1D617 */
  0x1E21,   /* U+1D618 */
  0x1E33,   /* U+1D619 */
  0x1E71,   /* U+1D61A */
  0x1E95,   /* U+1D61B */
  0x1EB5,   /* U+1D61C */
  0x1EE3,   /* U+1D61D */
  0x1EF5,   /* U+1D61E */
  0x1EFF,   /* U+1D61F */
  0x1F0B,   /* U+1D620 */
  0x1F21,   /* U+1D621 */
  0x1C47,   /* U+1D622 */
  0x1C60,   /* U+1D623 */
  0x1C7A,   /* U+1D624 */
  0x1C8F,   /* U+1D625 */
  0x1CAA,   /* U+1D626 */
  0x1CE5,   /* U+1D627 */
  0x1CF4,   /* U+1D628 */
  0x1D18,   /* U+1D629 */
  0x1D32,   /* U+1D62A */
  0x1D4C,   /* U+1D62B */
  0x1D65,   /* U+1D62C */
  0x1D77,   /* U+1D62D */
  0x1DAA,   /* U+1D62E */
  0x1DB9,   /* U+1D62F */
  0x1DDD,   /* U+1D630 */
  0x1E0C,   /* U+1D631 */
  0x1E21,   /* U+1D632 */
  0x1E33,   /* U+1D633 */
  0x1E71,   /* U+1D634 */
  0x1E95,   /* U+1D635 */
  0x1EB5,   /* U+1D636 */
  0x1EE3,   /* U+1D637 */
  0x1EF5,   /* U+1D638 */
  0x1EFF,   /* U+1D639 */
  0x1F0B,   /* U+1D63A */
  0x1F21,   /* U+1D63B */
  0x1C47,   /* U+1D63C */
  0x1C60,   /* U+1D63D */
  0x1C7A,   /* U+1D63E */
  0x1C8F,   /* U+1D63F */
  0x1CAA,   /* U+1D640 */
  0x1CE5,   /* U+1D641 */
  0x1CF4,   /* U+1D642 */
  0x1D18,   /* U+1D643 */
  0x1D32,   /* U+1D644 */
  0x1D4C,   /* U+1D645 */
  0x1D65,   /* U+1D646 */
  0x1D77,   /* U+1D647 */
  0x1DAA,   /* U+1D648 */
  0x1DB9,   /* U+1D649 */
  0x1DDD,   /* U+1D64A */
  0x1E0C,   /* U+1D64B */
  0x1E21,   /* U+1D64C */
  0x1E33,   /* U+1D64D */
  0x1E71,   /* U+1D64E */
  0x1E95,   /* U+1D64F */
  0x1EB5,   /* U+1D650 */
  0x1EE3,   /* U+1D651 */
  0x1EF5,   /* U+1D652 */
  0x1EFF,   /* U+1D653 */
  0x1F0B,   /* U+1D654 */
  0x1F21,   /* U+1D655 */
  0x1C47,   /* U+1D656 */
  0x1C60,   /* U+1D657 */
  0x1C7A,   /* U+1D658 */
  0x1C8F,   /* U+1D659 */
  0x1CAA,   /* U+1D65A */
  0x1CE5,   /* U+1D65B */
  0x1CF4,   /* U+1D65C */
  0x1D18,   /* U+1D65D */
  0x1D32,   /* U+1D65E */
  0x1D4C,   /* U+1D65F */
  0x1D65,   /* U+1D660 */
  0x1D77,   /* U+1D661 */
  0x1DAA,   /* U+1D662 */
  0x1DB9,   /* U+1D663 */
  0x1DDD,   /* U+1D664 */
  0x1E0C,   /* U+1D665 */
  0x1E21,   /* U+1D666 */
  0x1E33,   /* U+1D667 */
  0x1E71,   /* U+1D668 */
  0x1E95,   /* U+1D669 */
  0x1EB5,   /* U+1D66A */
  0x1EE3,   /* U+1D66B */
  0x1EF5,   /* U+1D66C */
  0x1EFF,   /* U+1D66D */
  0x1F0B,   /* U+1D66E */
  0x1F21,   /* U+1D66F */
  0x1C47,   /* U+1D670 */
  0x1C60,   /* U+1D671 */
  0x1C7A,   /* U+1D672 */
  0x1C8F,   /* U+1D673 */
  0x1CAA,   /* U+1D674 */
  0x1CE5,   /* U+1D675 */
  0x1CF4,   /* U+1D676 */
  0x1D18,   /* U+1D677 */
  0x1D32,   /* U+1D678 */
  0x1D4C,   /* U+1D679 */
  0x1D65,   /* U+1D67A */
  0x1D77,   /* U+1D67B */
  0x1DAA,   /* U+1D67C */
  0x1DB9,   /* U+1D67D */
  0x1DDD,   /* U+1D67E */
  0x1E0C,   /* U+1D67F */
  0x1E21,   /* U+1D680 */
  0x1E33,   /* U+1D681 */
  0x1E71,   /* U+1D682 */
  0x1E95,   /* U+1D683 */
  0x1EB5,   /* U+1D684 */
  0x1EE3,   /* U+1D685 */
  0x1EF5,   /* U+1D686 */
  0x1EFF,   /* U+1D687 */
  0x1F0B,   /* U+1D688 */
  0x1F21,   /* U+1D689 */
  0x1C47,   /* U+1D68A */
  0x1C60,   /* U+1D68B */
  0x1C7A,   /* U+1D68C */
  0x1C8F,   /* U+1D68D */
  0x1CAA,   /* U+1D68E */
  0x1CE5,   /* U+1D68F */
  0x1CF4,   /* U+1D690 */
  0x1D18,   /* U+1D691 */
  0x1D32,   /* U+1D692 */
  0x1D4C,   /* U+1D693 */
  0x1D65,   /* U+1D694 */
  0x1D77,   /* U+1D695 */
  0x1DAA,   /* U+1D696 */
  0x1DB9,   /* U+1D697 */
  0x1DDD,   /* U+1D698 */
  0x1E0C,   /* U+1D699 */
  0x1E21,   /* U+1D69A */
  0x1E33,   /* U+1D69B */
  0x1E71,   /* U+1D69C */
  0x1E95,   /* U+1D69D */
  0x1EB5,   /* U+1D69E */
  0x1EE3,   /* U+1D69F */
  0x1EF5,   /* U+1D6A0 */
  0x1EFF,   /* U+1D6A1 */
  0x1F0B,   /* U+1D6A2 */
  0x1F21,   /* U+1D6A3 */
  0x1D36,   /* U+1D6A4 */
  0x1D50,   /* U+1D6A5 */
  0xFBC3,   /* U+1D6A6 */
  0xFBC3,   /* U+1D6A7 */
  0x1FB9,   /* U+1D6A8 */
  0x1FBA,   /* U+1D6A9 */
  0x1FBB,   /* U+1D6AA */
  0x1FBD,   /* U+1D6AB */
  0x1FBE,   /* U+1D6AC */
  0x1FC2,   /* U+1D6AD */
  0x1FC4,   /* U+1D6AE */
  0x1FC5,   /* U+1D6AF */
  0x1FC6,   /* U+1D6B0 */
  0x1FC8,   /* U+1D6B1 */
  0x1FC9,   /* U+1D6B2 */
  0x1FCB,   /* U+1D6B3 */
  0x1FCC,   /* U+1D6B4 */
  0x1FCD,   /* U+1D6B5 */
  0x1FCE,   /* U+1D6B6 */
  0x1FCF,   /* U+1D6B7 */
  0x1FD4,   /* U+1D6B8 */
  0x1FC5,   /* U+1D6B9 */
  0x1FD7,   /* U+1D6BA */
  0x1FDB,   /* U+1D6BB */
  0x1FDC,   /* U+1D6BC */
  0x1FDD,   /* U+1D6BD */
  0x1FDE,   /* U+1D6BE */
  0x1FDF,   /* U+1D6BF */
  0x1FE1,   /* U+1D6C0 */
  0x060C,   /* U+1D6C1 */
  0x1FB9,   /* U+1D6C2 */
  0x1FBA,   /* U+1D6C3 */
  0x1FBB,   /* U+1D6C4 */
  0x1FBD,   /* U+1D6C5 */
  0x1FBE,   /* U+1D6C6 */
  0x1FC2,   /* U+1D6C7 */
  0x1FC4,   /* U+1D6C8 */
  0x1FC5,   /* U+1D6C9 */
  0x1FC6,   /* U+1D6CA */
  0x1FC8,   /* U+1D6CB */
  0x1FC9,   /* U+1D6CC */
  0x1FCB,   /* U+1D6CD */
  0x1FCC,   /* U+1D6CE */
  0x1FCD,   /* U+1D6CF */
  0x1FCE,   /* U+1D6D0 */
  0x1FCF,   /* U+1D6D1 */
  0x1FD4,   /* U+1D6D2 */
  0x1FD7,   /* U+1D6D3 */
  0x1FD7,   /* U+1D6D4 */
  0x1FDB,   /* U+1D6D5 */
  0x1FDC,   /* U+1D6D6 */
  0x1FDD,   /* U+1D6D7 */
  0x1FDE,   /* U+1D6D8 */
  0x1FDF,   /* U+1D6D9 */
  0x1FE1,   /* U+1D6DA */
  0x0608,   /* U+1D6DB */
  0x1FBE,   /* U+1D6DC */
  0x1FC5,   /* U+1D6DD */
  0x1FC8,   /* U+1D6DE */
  0x1FDD,   /* U+1D6DF */
  0x1FD4,   /* U+1D6E0 */
  0x1FCF,   /* U+1D6E1 */
  0x1FB9,   /* U+1D6E2 */
  0x1FBA,   /* U+1D6E3 */
  0x1FBB,   /* U+1D6E4 */
  0x1FBD,   /* U+1D6E5 */
  0x1FBE,   /* U+1D6E6 */
  0x1FC2,   /* U+1D6E7 */
  0x1FC4,   /* U+1D6E8 */
  0x1FC5,   /* U+1D6E9 */
  0x1FC6,   /* U+1D6EA */
  0x1FC8,   /* U+1D6EB */
  0x1FC9,   /* U+1D6EC */
  0x1FCB,   /* U+1D6ED */
  0x1FCC,   /* U+1D6EE */
  0x1FCD,   /* U+1D6EF */
  0x1FCE,   /* U+1D6F0 */
  0x1FCF,   /* U+1D6F1 */
  0x1FD4,   /* U+1D6F2 */
  0x1FC5,   /* U+1D6F3 */
  0x1FD7,   /* U+1D6F4 */
  0x1FDB,   /* U+1D6F5 */
  0x1FDC,   /* U+1D6F6 */
  0x1FDD,   /* U+1D6F7 */
  0x1FDE,   /* U+1D6F8 */
  0x1FDF,   /* U+1D6F9 */
  0x1FE1,   /* U+1D6FA */
  0x060C,   /* U+1D6FB */
  0x1FB9,   /* U+1D6FC */
  0x1FBA,   /* U+1D6FD */
  0x1FBB,   /* U+1D6FE */
  0x1FBD,   /* U+1D6FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D600 */
  0x0020,   /* U+1D601 */
  0x0020,   /* U+1D602 */
  0x0020,   /* U+1D603 */
  0x0020,   /* U+1D604 */
  0x0020,   /* U+1D605 */
  0x0020,   /* U+1D606 */
  0x0020,   /* U+1D607 */
  0x0020,   /* U+1D608 */
  0x0020,   /* U+1D609 */
  0x0020,   /* U+1D60A */
  0x0020,   /* U+1D60B */
  0x0020,   /* U+1D60C */
  0x0020,   /* U+1D60D */
  0x0020,   /* U+1D60E */
  0x0020,   /* U+1D60F */
  0x0020,   /* U+1D610 */
  0x0020,   /* U+1D611 */
  0x0020,   /* U+1D612 */
  0x0020,   /* U+1D613 */
  0x0020,   /* U+1D614 */
  0x0020,   /* U+1D615 */
  0x0020,   /* U+1D616 */
  0x0020,   /* U+1D617 */
  0x0020,   /* U+1D618 */
  0x0020,   /* U+1D619 */
  0x0020,   /* U+1D61A */
  0x0020,   /* U+1D61B */
  0x0020,   /* U+1D61C */
  0x0020,   /* U+1D61D */
  0x0020,   /* U+1D61E */
  0x0020,   /* U+1D61F */
  0x0020,   /* U+1D620 */
  0x0020,   /* U+1D621 */
  0x0020,   /* U+1D622 */
  0x0020,   /* U+1D623 */
  0x0020,   /* U+1D624 */
  0x0020,   /* U+1D625 */
  0x0020,   /* U+1D626 */
  0x0020,   /* U+1D627 */
  0x0020,   /* U+1D628 */
  0x0020,   /* U+1D629 */
  0x0020,   /* U+1D62A */
  0x0020,   /* U+1D62B */
  0x0020,   /* U+1D62C */
  0x0020,   /* U+1D62D */
  0x0020,   /* U+1D62E */
  0x0020,   /* U+1D62F */
  0x0020,   /* U+1D630 */
  0x0020,   /* U+1D631 */
  0x0020,   /* U+1D632 */
  0x0020,   /* U+1D633 */
  0x0020,   /* U+1D634 */
  0x0020,   /* U+1D635 */
  0x0020,   /* U+1D636 */
  0x0020,   /* U+1D637 */
  0x0020,   /* U+1D638 */
  0x0020,   /* U+1D639 */
  0x0020,   /* U+1D63A */
  0x0020,   /* U+1D63B */
  0x0020,   /* U+1D63C */
  0x0020,   /* U+1D63D */
  0x0020,   /* U+1D63E */
  0x0020,   /* U+1D63F */
  0x0020,   /* U+1D640 */
  0x0020,   /* U+1D641 */
  0x0020,   /* U+1D642 */
  0x0020,   /* U+1D643 */
  0x0020,   /* U+1D644 */
  0x0020,   /* U+1D645 */
  0x0020,   /* U+1D646 */
  0x0020,   /* U+1D647 */
  0x0020,   /* U+1D648 */
  0x0020,   /* U+1D649 */
  0x0020,   /* U+1D64A */
  0x0020,   /* U+1D64B */
  0x0020,   /* U+1D64C */
  0x0020,   /* U+1D64D */
  0x0020,   /* U+1D64E */
  0x0020,   /* U+1D64F */
  0x0020,   /* U+1D650 */
  0x0020,   /* U+1D651 */
  0x0020,   /* U+1D652 */
  0x0020,   /* U+1D653 */
  0x0020,   /* U+1D654 */
  0x0020,   /* U+1D655 */
  0x0020,   /* U+1D656 */
  0x0020,   /* U+1D657 */
  0x0020,   /* U+1D658 */
  0x0020,   /* U+1D659 */
  0x0020,   /* U+1D65A */
  0x0020,   /* U+1D65B */
  0x0020,   /* U+1D65C */
  0x0020,   /* U+1D65D */
  0x0020,   /* U+1D65E */
  0x0020,   /* U+1D65F */
  0x0020,   /* U+1D660 */
  0x0020,   /* U+1D661 */
  0x0020,   /* U+1D662 */
  0x0020,   /* U+1D663 */
  0x0020,   /* U+1D664 */
  0x0020,   /* U+1D665 */
  0x0020,   /* U+1D666 */
  0x0020,   /* U+1D667 */
  0x0020,   /* U+1D668 */
  0x0020,   /* U+1D669 */
  0x0020,   /* U+1D66A */
  0x0020,   /* U+1D66B */
  0x0020,   /* U+1D66C */
  0x0020,   /* U+1D66D */
  0x0020,   /* U+1D66E */
  0x0020,   /* U+1D66F */
  0x0020,   /* U+1D670 */
  0x0020,   /* U+1D671 */
  0x0020,   /* U+1D672 */
  0x0020,   /* U+1D673 */
  0x0020,   /* U+1D674 */
  0x0020,   /* U+1D675 */
  0x0020,   /* U+1D676 */
  0x0020,   /* U+1D677 */
  0x0020,   /* U+1D678 */
  0x0020,   /* U+1D679 */
  0x0020,   /* U+1D67A */
  0x0020,   /* U+1D67B */
  0x0020,   /* U+1D67C */
  0x0020,   /* U+1D67D */
  0x0020,   /* U+1D67E */
  0x0020,   /* U+1D67F */
  0x0020,   /* U+1D680 */
  0x0020,   /* U+1D681 */
  0x0020,   /* U+1D682 */
  0x0020,   /* U+1D683 */
  0x0020,   /* U+1D684 */
  0x0020,   /* U+1D685 */
  0x0020,   /* U+1D686 */
  0x0020,   /* U+1D687 */
  0x0020,   /* U+1D688 */
  0x0020,   /* U+1D689 */
  0x0020,   /* U+1D68A */
  0x0020,   /* U+1D68B */
  0x0020,   /* U+1D68C */
  0x0020,   /* U+1D68D */
  0x0020,   /* U+1D68E */
  0x0020,   /* U+1D68F */
  0x0020,   /* U+1D690 */
  0x0020,   /* U+1D691 */
  0x0020,   /* U+1D692 */
  0x0020,   /* U+1D693 */
  0x0020,   /* U+1D694 */
  0x0020,   /* U+1D695 */
  0x0020,   /* U+1D696 */
  0x0020,   /* U+1D697 */
  0x0020,   /* U+1D698 */
  0x0020,   /* U+1D699 */
  0x0020,   /* U+1D69A */
  0x0020,   /* U+1D69B */
  0x0020,   /* U+1D69C */
  0x0020,   /* U+1D69D */
  0x0020,   /* U+1D69E */
  0x0020,   /* U+1D69F */
  0x0020,   /* U+1D6A0 */
  0x0020,   /* U+1D6A1 */
  0x0020,   /* U+1D6A2 */
  0x0020,   /* U+1D6A3 */
  0x0020,   /* U+1D6A4 */
  0x0020,   /* U+1D6A5 */
  0x0020,   /* U+1D6A6 */
  0x0020,   /* U+1D6A7 */
  0x0020,   /* U+1D6A8 */
  0x0020,   /* U+1D6A9 */
  0x0020,   /* U+1D6AA */
  0x0020,   /* U+1D6AB */
  0x0020,   /* U+1D6AC */
  0x0020,   /* U+1D6AD */
  0x0020,   /* U+1D6AE */
  0x0020,   /* U+1D6AF */
  0x0020,   /* U+1D6B0 */
  0x0020,   /* U+1D6B1 */
  0x0020,   /* U+1D6B2 */
  0x0020,   /* U+1D6B3 */
  0x0020,   /* U+1D6B4 */
  0x0020,   /* U+1D6B5 */
  0x0020,   /* U+1D6B6 */
  0x0020,   /* U+1D6B7 */
  0x0020,   /* U+1D6B8 */
  0x0020,   /* U+1D6B9 */
  0x0020,   /* U+1D6BA */
  0x0020,   /* U+1D6BB */
  0x0020,   /* U+1D6BC */
  0x0020,   /* U+1D6BD */
  0x0020,   /* U+1D6BE */
  0x0020,   /* U+1D6BF */
  0x0020,   /* U+1D6C0 */
  0x0020,   /* U+1D6C1 */
  0x0020,   /* U+1D6C2 */
  0x0020,   /* U+1D6C3 */
  0x0020,   /* U+1D6C4 */
  0x0020,   /* U+1D6C5 */
  0x0020,   /* U+1D6C6 */
  0x0020,   /* U+1D6C7 */
  0x0020,   /* U+1D6C8 */
  0x0020,   /* U+1D6C9 */
  0x0020,   /* U+1D6CA */
  0x0020,   /* U+1D6CB */
  0x0020,   /* U+1D6CC */
  0x0020,   /* U+1D6CD */
  0x0020,   /* U+1D6CE */
  0x0020,   /* U+1D6CF */
  0x0020,   /* U+1D6D0 */
  0x0020,   /* U+1D6D1 */
  0x0020,   /* U+1D6D2 */
  0x0020,   /* U+1D6D3 */
  0x0020,   /* U+1D6D4 */
  0x0020,   /* U+1D6D5 */
  0x0020,   /* U+1D6D6 */
  0x0020,   /* U+1D6D7 */
  0x0020,   /* U+1D6D8 */
  0x0020,   /* U+1D6D9 */
  0x0020,   /* U+1D6DA */
  0x0020,   /* U+1D6DB */
  0x0020,   /* U+1D6DC */
  0x0020,   /* U+1D6DD */
  0x0020,   /* U+1D6DE */
  0x0020,   /* U+1D6DF */
  0x0020,   /* U+1D6E0 */
  0x0020,   /* U+1D6E1 */
  0x0020,   /* U+1D6E2 */
  0x0020,   /* U+1D6E3 */
  0x0020,   /* U+1D6E4 */
  0x0020,   /* U+1D6E5 */
  0x0020,   /* U+1D6E6 */
  0x0020,   /* U+1D6E7 */
  0x0020,   /* U+1D6E8 */
  0x0020,   /* U+1D6E9 */
  0x0020,   /* U+1D6EA */
  0x0020,   /* U+1D6EB */
  0x0020,   /* U+1D6EC */
  0x0020,   /* U+1D6ED */
  0x0020,   /* U+1D6EE */
  0x0020,   /* U+1D6EF */
  0x0020,   /* U+1D6F0 */
  0x0020,   /* U+1D6F1 */
  0x0020,   /* U+1D6F2 */
  0x0020,   /* U+1D6F3 */
  0x0020,   /* U+1D6F4 */
  0x0020,   /* U+1D6F5 */
  0x0020,   /* U+1D6F6 */
  0x0020,   /* U+1D6F7 */
  0x0020,   /* U+1D6F8 */
  0x0020,   /* U+1D6F9 */
  0x0020,   /* U+1D6FA */
  0x0020,   /* U+1D6FB */
  0x0020,   /* U+1D6FC */
  0x0020,   /* U+1D6FD */
  0x0020,   /* U+1D6FE */
  0x0020,   /* U+1D6FF */

  /* Tertiary weight 1 for each character. */
  0x0005,   /* U+1D600 */
  0x0005,   /* U+1D601 */
  0x0005,   /* U+1D602 */
  0x0005,   /* U+1D603 */
  0x0005,   /* U+1D604 */
  0x0005,   /* U+1D605 */
  0x0005,   /* U+1D606 */
  0x0005,   /* U+1D607 */
  0x000B,   /* U+1D608 */
  0x000B,   /* U+1D609 */
  0x000B,   /* U+1D60A */
  0x000B,   /* U+1D60B */
  0x000B,   /* U+1D60C */
  0x000B,   /* U+1D60D */
  0x000B,   /* U+1D60E */
  0x000B,   /* U+1D60F */
  0x000B,   /* U+1D610 */
  0x000B,   /* U+1D611 */
  0x000B,   /* U+1D612 */
  0x000B,   /* U+1D613 */
  0x000B,   /* U+1D614 */
  0x000B,   /* U+1D615 */
  0x000B,   /* U+1D616 */
  0x000B,   /* U+1D617 */
  0x000B,   /* U+1D618 */
  0x000B,   /* U+1D619 */
  0x000B,   /* U+1D61A */
  0x000B,   /* U+1D61B */
  0x000B,   /* U+1D61C */
  0x000B,   /* U+1D61D */
  0x000B,   /* U+1D61E */
  0x000B,   /* U+1D61F */
  0x000B,   /* U+1D620 */
  0x000B,   /* U+1D621 */
  0x0005,   /* U+1D622 */
  0x0005,   /* U+1D623 */
  0x0005,   /* U+1D624 */
  0x0005,   /* U+1D625 */
  0x0005,   /* U+1D626 */
  0x0005,   /* U+1D627 */
  0x0005,   /* U+1D628 */
  0x0005,   /* U+1D629 */
  0x0005,   /* U+1D62A */
  0x0005,   /* U+1D62B */
  0x0005,   /* U+1D62C */
  0x0005,   /* U+1D62D */
  0x0005,   /* U+1D62E */
  0x0005,   /* U+1D62F */
  0x0005,   /* U+1D630 */
  0x0005,   /* U+1D631 */
  0x0005,   /* U+1D632 */
  0x0005,   /* U+1D633 */
  0x0005,   /* U+1D634 */
  0x0005,   /* U+1D635 */
  0x0005,   /* U+1D636 */
  0x0005,   /* U+1D637 */
  0x0005,   /* U+1D638 */
  0x0005,   /* U+1D639 */
  0x0005,   /* U+1D63A */
  0x0005,   /* U+1D63B */
  0x000B,   /* U+1D63C */
  0x000B,   /* U+1D63D */
  0x000B,   /* U+1D63E */
  0x000B,   /* U+1D63F */
  0x000B,   /* U+1D640 */
  0x000B,   /* U+1D641 */
  0x000B,   /* U+1D642 */
  0x000B,   /* U+1D643 */
  0x000B,   /* U+1D644 */
  0x000B,   /* U+1D645 */
  0x000B,   /* U+1D646 */
  0x000B,   /* U+1D647 */
  0x000B,   /* U+1D648 */
  0x000B,   /* U+1D649 */
  0x000B,   /* U+1D64A */
  0x000B,   /* U+1D64B */
  0x000B,   /* U+1D64C */
  0x000B,   /* U+1D64D */
  0x000B,   /* U+1D64E */
  0x000B,   /* U+1D64F */
  0x000B,   /* U+1D650 */
  0x000B,   /* U+1D651 */
  0x000B,   /* U+1D652 */
  0x000B,   /* U+1D653 */
  0x000B,   /* U+1D654 */
  0x000B,   /* U+1D655 */
  0x0005,   /* U+1D656 */
  0x0005,   /* U+1D657 */
  0x0005,   /* U+1D658 */
  0x0005,   /* U+1D659 */
  0x0005,   /* U+1D65A */
  0x0005,   /* U+1D65B */
  0x0005,   /* U+1D65C */
  0x0005,   /* U+1D65D */
  0x0005,   /* U+1D65E */
  0x0005,   /* U+1D65F */
  0x0005,   /* U+1D660 */
  0x0005,   /* U+1D661 */
  0x0005,   /* U+1D662 */
  0x0005,   /* U+1D663 */
  0x0005,   /* U+1D664 */
  0x0005,   /* U+1D665 */
  0x0005,   /* U+1D666 */
  0x0005,   /* U+1D667 */
  0x0005,   /* U+1D668 */
  0x0005,   /* U+1D669 */
  0x0005,   /* U+1D66A */
  0x0005,   /* U+1D66B */
  0x0005,   /* U+1D66C */
  0x0005,   /* U+1D66D */
  0x0005,   /* U+1D66E */
  0x0005,   /* U+1D66F */
  0x000B,   /* U+1D670 */
  0x000B,   /* U+1D671 */
  0x000B,   /* U+1D672 */
  0x000B,   /* U+1D673 */
  0x000B,   /* U+1D674 */
  0x000B,   /* U+1D675 */
  0x000B,   /* U+1D676 */
  0x000B,   /* U+1D677 */
  0x000B,   /* U+1D678 */
  0x000B,   /* U+1D679 */
  0x000B,   /* U+1D67A */
  0x000B,   /* U+1D67B */
  0x000B,   /* U+1D67C */
  0x000B,   /* U+1D67D */
  0x000B,   /* U+1D67E */
  0x000B,   /* U+1D67F */
  0x000B,   /* U+1D680 */
  0x000B,   /* U+1D681 */
  0x000B,   /* U+1D682 */
  0x000B,   /* U+1D683 */
  0x000B,   /* U+1D684 */
  0x000B,   /* U+1D685 */
  0x000B,   /* U+1D686 */
  0x000B,   /* U+1D687 */
  0x000B,   /* U+1D688 */
  0x000B,   /* U+1D689 */
  0x0005,   /* U+1D68A */
  0x0005,   /* U+1D68B */
  0x0005,   /* U+1D68C */
  0x0005,   /* U+1D68D */
  0x0005,   /* U+1D68E */
  0x0005,   /* U+1D68F */
  0x0005,   /* U+1D690 */
  0x0005,   /* U+1D691 */
  0x0005,   /* U+1D692 */
  0x0005,   /* U+1D693 */
  0x0005,   /* U+1D694 */
  0x0005,   /* U+1D695 */
  0x0005,   /* U+1D696 */
  0x0005,   /* U+1D697 */
  0x0005,   /* U+1D698 */
  0x0005,   /* U+1D699 */
  0x0005,   /* U+1D69A */
  0x0005,   /* U+1D69B */
  0x0005,   /* U+1D69C */
  0x0005,   /* U+1D69D */
  0x0005,   /* U+1D69E */
  0x0005,   /* U+1D69F */
  0x0005,   /* U+1D6A0 */
  0x0005,   /* U+1D6A1 */
  0x0005,   /* U+1D6A2 */
  0x0005,   /* U+1D6A3 */
  0x0005,   /* U+1D6A4 */
  0x0005,   /* U+1D6A5 */
  0x0002,   /* U+1D6A6 */
  0x0002,   /* U+1D6A7 */
  0x000B,   /* U+1D6A8 */
  0x000B,   /* U+1D6A9 */
  0x000B,   /* U+1D6AA */
  0x000B,   /* U+1D6AB */
  0x000B,   /* U+1D6AC */
  0x000B,   /* U+1D6AD */
  0x000B,   /* U+1D6AE */
  0x000B,   /* U+1D6AF */
  0x000B,   /* U+1D6B0 */
  0x000B,   /* U+1D6B1 */
  0x000B,   /* U+1D6B2 */
  0x000B,   /* U+1D6B3 */
  0x000B,   /* U+1D6B4 */
  0x000B,   /* U+1D6B5 */
  0x000B,   /* U+1D6B6 */
  0x000B,   /* U+1D6B7 */
  0x000B,   /* U+1D6B8 */
  0x000B,   /* U+1D6B9 */
  0x000B,   /* U+1D6BA */
  0x000B,   /* U+1D6BB */
  0x000B,   /* U+1D6BC */
  0x000B,   /* U+1D6BD */
  0x000B,   /* U+1D6BE */
  0x000B,   /* U+1D6BF */
  0x000B,   /* U+1D6C0 */
  0x0005,   /* U+1D6C1 */
  0x0005,   /* U+1D6C2 */
  0x0005,   /* U+1D6C3 */
  0x0005,   /* U+1D6C4 */
  0x0005,   /* U+1D6C5 */
  0x0005,   /* U+1D6C6 */
  0x0005,   /* U+1D6C7 */
  0x0005,   /* U+1D6C8 */
  0x0005,   /* U+1D6C9 */
  0x0005,   /* U+1D6CA */
  0x0005,   /* U+1D6CB */
  0x0005,   /* U+1D6CC */
  0x0005,   /* U+1D6CD */
  0x0005,   /* U+1D6CE */
  0x0005,   /* U+1D6CF */
  0x0005,   /* U+1D6D0 */
  0x0005,   /* U+1D6D1 */
  0x0005,   /* U+1D6D2 */
  0x0005,   /* U+1D6D3 */
  0x0005,   /* U+1D6D4 */
  0x0005,   /* U+1D6D5 */
  0x0005,   /* U+1D6D6 */
  0x0005,   /* U+1D6D7 */
  0x0005,   /* U+1D6D8 */
  0x0005,   /* U+1D6D9 */
  0x0005,   /* U+1D6DA */
  0x0005,   /* U+1D6DB */
  0x0005,   /* U+1D6DC */
  0x0005,   /* U+1D6DD */
  0x0005,   /* U+1D6DE */
  0x0005,   /* U+1D6DF */
  0x0005,   /* U+1D6E0 */
  0x0005,   /* U+1D6E1 */
  0x000B,   /* U+1D6E2 */
  0x000B,   /* U+1D6E3 */
  0x000B,   /* U+1D6E4 */
  0x000B,   /* U+1D6E5 */
  0x000B,   /* U+1D6E6 */
  0x000B,   /* U+1D6E7 */
  0x000B,   /* U+1D6E8 */
  0x000B,   /* U+1D6E9 */
  0x000B,   /* U+1D6EA */
  0x000B,   /* U+1D6EB */
  0x000B,   /* U+1D6EC */
  0x000B,   /* U+1D6ED */
  0x000B,   /* U+1D6EE */
  0x000B,   /* U+1D6EF */
  0x000B,   /* U+1D6F0 */
  0x000B,   /* U+1D6F1 */
  0x000B,   /* U+1D6F2 */
  0x000B,   /* U+1D6F3 */
  0x000B,   /* U+1D6F4 */
  0x000B,   /* U+1D6F5 */
  0x000B,   /* U+1D6F6 */
  0x000B,   /* U+1D6F7 */
  0x000B,   /* U+1D6F8 */
  0x000B,   /* U+1D6F9 */
  0x000B,   /* U+1D6FA */
  0x0005,   /* U+1D6FB */
  0x0005,   /* U+1D6FC */
  0x0005,   /* U+1D6FD */
  0x0005,   /* U+1D6FE */
  0x0005,   /* U+1D6FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D600 */
  0x0000,   /* U+1D601 */
  0x0000,   /* U+1D602 */
  0x0000,   /* U+1D603 */
  0x0000,   /* U+1D604 */
  0x0000,   /* U+1D605 */
  0x0000,   /* U+1D606 */
  0x0000,   /* U+1D607 */
  0x0000,   /* U+1D608 */
  0x0000,   /* U+1D609 */
  0x0000,   /* U+1D60A */
  0x0000,   /* U+1D60B */
  0x0000,   /* U+1D60C */
  0x0000,   /* U+1D60D */
  0x0000,   /* U+1D60E */
  0x0000,   /* U+1D60F */
  0x0000,   /* U+1D610 */
  0x0000,   /* U+1D611 */
  0x0000,   /* U+1D612 */
  0x0000,   /* U+1D613 */
  0x0000,   /* U+1D614 */
  0x0000,   /* U+1D615 */
  0x0000,   /* U+1D616 */
  0x0000,   /* U+1D617 */
  0x0000,   /* U+1D618 */
  0x0000,   /* U+1D619 */
  0x0000,   /* U+1D61A */
  0x0000,   /* U+1D61B */
  0x0000,   /* U+1D61C */
  0x0000,   /* U+1D61D */
  0x0000,   /* U+1D61E */
  0x0000,   /* U+1D61F */
  0x0000,   /* U+1D620 */
  0x0000,   /* U+1D621 */
  0x0000,   /* U+1D622 */
  0x0000,   /* U+1D623 */
  0x0000,   /* U+1D624 */
  0x0000,   /* U+1D625 */
  0x0000,   /* U+1D626 */
  0x0000,   /* U+1D627 */
  0x0000,   /* U+1D628 */
  0x0000,   /* U+1D629 */
  0x0000,   /* U+1D62A */
  0x0000,   /* U+1D62B */
  0x0000,   /* U+1D62C */
  0x0000,   /* U+1D62D */
  0x0000,   /* U+1D62E */
  0x0000,   /* U+1D62F */
  0x0000,   /* U+1D630 */
  0x0000,   /* U+1D631 */
  0x0000,   /* U+1D632 */
  0x0000,   /* U+1D633 */
  0x0000,   /* U+1D634 */
  0x0000,   /* U+1D635 */
  0x0000,   /* U+1D636 */
  0x0000,   /* U+1D637 */
  0x0000,   /* U+1D638 */
  0x0000,   /* U+1D639 */
  0x0000,   /* U+1D63A */
  0x0000,   /* U+1D63B */
  0x0000,   /* U+1D63C */
  0x0000,   /* U+1D63D */
  0x0000,   /* U+1D63E */
  0x0000,   /* U+1D63F */
  0x0000,   /* U+1D640 */
  0x0000,   /* U+1D641 */
  0x0000,   /* U+1D642 */
  0x0000,   /* U+1D643 */
  0x0000,   /* U+1D644 */
  0x0000,   /* U+1D645 */
  0x0000,   /* U+1D646 */
  0x0000,   /* U+1D647 */
  0x0000,   /* U+1D648 */
  0x0000,   /* U+1D649 */
  0x0000,   /* U+1D64A */
  0x0000,   /* U+1D64B */
  0x0000,   /* U+1D64C */
  0x0000,   /* U+1D64D */
  0x0000,   /* U+1D64E */
  0x0000,   /* U+1D64F */
  0x0000,   /* U+1D650 */
  0x0000,   /* U+1D651 */
  0x0000,   /* U+1D652 */
  0x0000,   /* U+1D653 */
  0x0000,   /* U+1D654 */
  0x0000,   /* U+1D655 */
  0x0000,   /* U+1D656 */
  0x0000,   /* U+1D657 */
  0x0000,   /* U+1D658 */
  0x0000,   /* U+1D659 */
  0x0000,   /* U+1D65A */
  0x0000,   /* U+1D65B */
  0x0000,   /* U+1D65C */
  0x0000,   /* U+1D65D */
  0x0000,   /* U+1D65E */
  0x0000,   /* U+1D65F */
  0x0000,   /* U+1D660 */
  0x0000,   /* U+1D661 */
  0x0000,   /* U+1D662 */
  0x0000,   /* U+1D663 */
  0x0000,   /* U+1D664 */
  0x0000,   /* U+1D665 */
  0x0000,   /* U+1D666 */
  0x0000,   /* U+1D667 */
  0x0000,   /* U+1D668 */
  0x0000,   /* U+1D669 */
  0x0000,   /* U+1D66A */
  0x0000,   /* U+1D66B */
  0x0000,   /* U+1D66C */
  0x0000,   /* U+1D66D */
  0x0000,   /* U+1D66E */
  0x0000,   /* U+1D66F */
  0x0000,   /* U+1D670 */
  0x0000,   /* U+1D671 */
  0x0000,   /* U+1D672 */
  0x0000,   /* U+1D673 */
  0x0000,   /* U+1D674 */
  0x0000,   /* U+1D675 */
  0x0000,   /* U+1D676 */
  0x0000,   /* U+1D677 */
  0x0000,   /* U+1D678 */
  0x0000,   /* U+1D679 */
  0x0000,   /* U+1D67A */
  0x0000,   /* U+1D67B */
  0x0000,   /* U+1D67C */
  0x0000,   /* U+1D67D */
  0x0000,   /* U+1D67E */
  0x0000,   /* U+1D67F */
  0x0000,   /* U+1D680 */
  0x0000,   /* U+1D681 */
  0x0000,   /* U+1D682 */
  0x0000,   /* U+1D683 */
  0x0000,   /* U+1D684 */
  0x0000,   /* U+1D685 */
  0x0000,   /* U+1D686 */
  0x0000,   /* U+1D687 */
  0x0000,   /* U+1D688 */
  0x0000,   /* U+1D689 */
  0x0000,   /* U+1D68A */
  0x0000,   /* U+1D68B */
  0x0000,   /* U+1D68C */
  0x0000,   /* U+1D68D */
  0x0000,   /* U+1D68E */
  0x0000,   /* U+1D68F */
  0x0000,   /* U+1D690 */
  0x0000,   /* U+1D691 */
  0x0000,   /* U+1D692 */
  0x0000,   /* U+1D693 */
  0x0000,   /* U+1D694 */
  0x0000,   /* U+1D695 */
  0x0000,   /* U+1D696 */
  0x0000,   /* U+1D697 */
  0x0000,   /* U+1D698 */
  0x0000,   /* U+1D699 */
  0x0000,   /* U+1D69A */
  0x0000,   /* U+1D69B */
  0x0000,   /* U+1D69C */
  0x0000,   /* U+1D69D */
  0x0000,   /* U+1D69E */
  0x0000,   /* U+1D69F */
  0x0000,   /* U+1D6A0 */
  0x0000,   /* U+1D6A1 */
  0x0000,   /* U+1D6A2 */
  0x0000,   /* U+1D6A3 */
  0x0000,   /* U+1D6A4 */
  0x0000,   /* U+1D6A5 */
  0xD6A6,   /* U+1D6A6 */
  0xD6A7,   /* U+1D6A7 */
  0x0000,   /* U+1D6A8 */
  0x0000,   /* U+1D6A9 */
  0x0000,   /* U+1D6AA */
  0x0000,   /* U+1D6AB */
  0x0000,   /* U+1D6AC */
  0x0000,   /* U+1D6AD */
  0x0000,   /* U+1D6AE */
  0x0000,   /* U+1D6AF */
  0x0000,   /* U+1D6B0 */
  0x0000,   /* U+1D6B1 */
  0x0000,   /* U+1D6B2 */
  0x0000,   /* U+1D6B3 */
  0x0000,   /* U+1D6B4 */
  0x0000,   /* U+1D6B5 */
  0x0000,   /* U+1D6B6 */
  0x0000,   /* U+1D6B7 */
  0x0000,   /* U+1D6B8 */
  0x0000,   /* U+1D6B9 */
  0x0000,   /* U+1D6BA */
  0x0000,   /* U+1D6BB */
  0x0000,   /* U+1D6BC */
  0x0000,   /* U+1D6BD */
  0x0000,   /* U+1D6BE */
  0x0000,   /* U+1D6BF */
  0x0000,   /* U+1D6C0 */
  0x0000,   /* U+1D6C1 */
  0x0000,   /* U+1D6C2 */
  0x0000,   /* U+1D6C3 */
  0x0000,   /* U+1D6C4 */
  0x0000,   /* U+1D6C5 */
  0x0000,   /* U+1D6C6 */
  0x0000,   /* U+1D6C7 */
  0x0000,   /* U+1D6C8 */
  0x0000,   /* U+1D6C9 */
  0x0000,   /* U+1D6CA */
  0x0000,   /* U+1D6CB */
  0x0000,   /* U+1D6CC */
  0x0000,   /* U+1D6CD */
  0x0000,   /* U+1D6CE */
  0x0000,   /* U+1D6CF */
  0x0000,   /* U+1D6D0 */
  0x0000,   /* U+1D6D1 */
  0x0000,   /* U+1D6D2 */
  0x0000,   /* U+1D6D3 */
  0x0000,   /* U+1D6D4 */
  0x0000,   /* U+1D6D5 */
  0x0000,   /* U+1D6D6 */
  0x0000,   /* U+1D6D7 */
  0x0000,   /* U+1D6D8 */
  0x0000,   /* U+1D6D9 */
  0x0000,   /* U+1D6DA */
  0x0000,   /* U+1D6DB */
  0x0000,   /* U+1D6DC */
  0x0000,   /* U+1D6DD */
  0x0000,   /* U+1D6DE */
  0x0000,   /* U+1D6DF */
  0x0000,   /* U+1D6E0 */
  0x0000,   /* U+1D6E1 */
  0x0000,   /* U+1D6E2 */
  0x0000,   /* U+1D6E3 */
  0x0000,   /* U+1D6E4 */
  0x0000,   /* U+1D6E5 */
  0x0000,   /* U+1D6E6 */
  0x0000,   /* U+1D6E7 */
  0x0000,   /* U+1D6E8 */
  0x0000,   /* U+1D6E9 */
  0x0000,   /* U+1D6EA */
  0x0000,   /* U+1D6EB */
  0x0000,   /* U+1D6EC */
  0x0000,   /* U+1D6ED */
  0x0000,   /* U+1D6EE */
  0x0000,   /* U+1D6EF */
  0x0000,   /* U+1D6F0 */
  0x0000,   /* U+1D6F1 */
  0x0000,   /* U+1D6F2 */
  0x0000,   /* U+1D6F3 */
  0x0000,   /* U+1D6F4 */
  0x0000,   /* U+1D6F5 */
  0x0000,   /* U+1D6F6 */
  0x0000,   /* U+1D6F7 */
  0x0000,   /* U+1D6F8 */
  0x0000,   /* U+1D6F9 */
  0x0000,   /* U+1D6FA */
  0x0000,   /* U+1D6FB */
  0x0000,   /* U+1D6FC */
  0x0000,   /* U+1D6FD */
  0x0000,   /* U+1D6FE */
  0x0000,   /* U+1D6FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D600 */
  0x0000,   /* U+1D601 */
  0x0000,   /* U+1D602 */
  0x0000,   /* U+1D603 */
  0x0000,   /* U+1D604 */
  0x0000,   /* U+1D605 */
  0x0000,   /* U+1D606 */
  0x0000,   /* U+1D607 */
  0x0000,   /* U+1D608 */
  0x0000,   /* U+1D609 */
  0x0000,   /* U+1D60A */
  0x0000,   /* U+1D60B */
  0x0000,   /* U+1D60C */
  0x0000,   /* U+1D60D */
  0x0000,   /* U+1D60E */
  0x0000,   /* U+1D60F */
  0x0000,   /* U+1D610 */
  0x0000,   /* U+1D611 */
  0x0000,   /* U+1D612 */
  0x0000,   /* U+1D613 */
  0x0000,   /* U+1D614 */
  0x0000,   /* U+1D615 */
  0x0000,   /* U+1D616 */
  0x0000,   /* U+1D617 */
  0x0000,   /* U+1D618 */
  0x0000,   /* U+1D619 */
  0x0000,   /* U+1D61A */
  0x0000,   /* U+1D61B */
  0x0000,   /* U+1D61C */
  0x0000,   /* U+1D61D */
  0x0000,   /* U+1D61E */
  0x0000,   /* U+1D61F */
  0x0000,   /* U+1D620 */
  0x0000,   /* U+1D621 */
  0x0000,   /* U+1D622 */
  0x0000,   /* U+1D623 */
  0x0000,   /* U+1D624 */
  0x0000,   /* U+1D625 */
  0x0000,   /* U+1D626 */
  0x0000,   /* U+1D627 */
  0x0000,   /* U+1D628 */
  0x0000,   /* U+1D629 */
  0x0000,   /* U+1D62A */
  0x0000,   /* U+1D62B */
  0x0000,   /* U+1D62C */
  0x0000,   /* U+1D62D */
  0x0000,   /* U+1D62E */
  0x0000,   /* U+1D62F */
  0x0000,   /* U+1D630 */
  0x0000,   /* U+1D631 */
  0x0000,   /* U+1D632 */
  0x0000,   /* U+1D633 */
  0x0000,   /* U+1D634 */
  0x0000,   /* U+1D635 */
  0x0000,   /* U+1D636 */
  0x0000,   /* U+1D637 */
  0x0000,   /* U+1D638 */
  0x0000,   /* U+1D639 */
  0x0000,   /* U+1D63A */
  0x0000,   /* U+1D63B */
  0x0000,   /* U+1D63C */
  0x0000,   /* U+1D63D */
  0x0000,   /* U+1D63E */
  0x0000,   /* U+1D63F */
  0x0000,   /* U+1D640 */
  0x0000,   /* U+1D641 */
  0x0000,   /* U+1D642 */
  0x0000,   /* U+1D643 */
  0x0000,   /* U+1D644 */
  0x0000,   /* U+1D645 */
  0x0000,   /* U+1D646 */
  0x0000,   /* U+1D647 */
  0x0000,   /* U+1D648 */
  0x0000,   /* U+1D649 */
  0x0000,   /* U+1D64A */
  0x0000,   /* U+1D64B */
  0x0000,   /* U+1D64C */
  0x0000,   /* U+1D64D */
  0x0000,   /* U+1D64E */
  0x0000,   /* U+1D64F */
  0x0000,   /* U+1D650 */
  0x0000,   /* U+1D651 */
  0x0000,   /* U+1D652 */
  0x0000,   /* U+1D653 */
  0x0000,   /* U+1D654 */
  0x0000,   /* U+1D655 */
  0x0000,   /* U+1D656 */
  0x0000,   /* U+1D657 */
  0x0000,   /* U+1D658 */
  0x0000,   /* U+1D659 */
  0x0000,   /* U+1D65A */
  0x0000,   /* U+1D65B */
  0x0000,   /* U+1D65C */
  0x0000,   /* U+1D65D */
  0x0000,   /* U+1D65E */
  0x0000,   /* U+1D65F */
  0x0000,   /* U+1D660 */
  0x0000,   /* U+1D661 */
  0x0000,   /* U+1D662 */
  0x0000,   /* U+1D663 */
  0x0000,   /* U+1D664 */
  0x0000,   /* U+1D665 */
  0x0000,   /* U+1D666 */
  0x0000,   /* U+1D667 */
  0x0000,   /* U+1D668 */
  0x0000,   /* U+1D669 */
  0x0000,   /* U+1D66A */
  0x0000,   /* U+1D66B */
  0x0000,   /* U+1D66C */
  0x0000,   /* U+1D66D */
  0x0000,   /* U+1D66E */
  0x0000,   /* U+1D66F */
  0x0000,   /* U+1D670 */
  0x0000,   /* U+1D671 */
  0x0000,   /* U+1D672 */
  0x0000,   /* U+1D673 */
  0x0000,   /* U+1D674 */
  0x0000,   /* U+1D675 */
  0x0000,   /* U+1D676 */
  0x0000,   /* U+1D677 */
  0x0000,   /* U+1D678 */
  0x0000,   /* U+1D679 */
  0x0000,   /* U+1D67A */
  0x0000,   /* U+1D67B */
  0x0000,   /* U+1D67C */
  0x0000,   /* U+1D67D */
  0x0000,   /* U+1D67E */
  0x0000,   /* U+1D67F */
  0x0000,   /* U+1D680 */
  0x0000,   /* U+1D681 */
  0x0000,   /* U+1D682 */
  0x0000,   /* U+1D683 */
  0x0000,   /* U+1D684 */
  0x0000,   /* U+1D685 */
  0x0000,   /* U+1D686 */
  0x0000,   /* U+1D687 */
  0x0000,   /* U+1D688 */
  0x0000,   /* U+1D689 */
  0x0000,   /* U+1D68A */
  0x0000,   /* U+1D68B */
  0x0000,   /* U+1D68C */
  0x0000,   /* U+1D68D */
  0x0000,   /* U+1D68E */
  0x0000,   /* U+1D68F */
  0x0000,   /* U+1D690 */
  0x0000,   /* U+1D691 */
  0x0000,   /* U+1D692 */
  0x0000,   /* U+1D693 */
  0x0000,   /* U+1D694 */
  0x0000,   /* U+1D695 */
  0x0000,   /* U+1D696 */
  0x0000,   /* U+1D697 */
  0x0000,   /* U+1D698 */
  0x0000,   /* U+1D699 */
  0x0000,   /* U+1D69A */
  0x0000,   /* U+1D69B */
  0x0000,   /* U+1D69C */
  0x0000,   /* U+1D69D */
  0x0000,   /* U+1D69E */
  0x0000,   /* U+1D69F */
  0x0000,   /* U+1D6A0 */
  0x0000,   /* U+1D6A1 */
  0x0000,   /* U+1D6A2 */
  0x0000,   /* U+1D6A3 */
  0x0000,   /* U+1D6A4 */
  0x0000,   /* U+1D6A5 */
  0x0000,   /* U+1D6A6 */
  0x0000,   /* U+1D6A7 */
  0x0000,   /* U+1D6A8 */
  0x0000,   /* U+1D6A9 */
  0x0000,   /* U+1D6AA */
  0x0000,   /* U+1D6AB */
  0x0000,   /* U+1D6AC */
  0x0000,   /* U+1D6AD */
  0x0000,   /* U+1D6AE */
  0x0000,   /* U+1D6AF */
  0x0000,   /* U+1D6B0 */
  0x0000,   /* U+1D6B1 */
  0x0000,   /* U+1D6B2 */
  0x0000,   /* U+1D6B3 */
  0x0000,   /* U+1D6B4 */
  0x0000,   /* U+1D6B5 */
  0x0000,   /* U+1D6B6 */
  0x0000,   /* U+1D6B7 */
  0x0000,   /* U+1D6B8 */
  0x0000,   /* U+1D6B9 */
  0x0000,   /* U+1D6BA */
  0x0000,   /* U+1D6BB */
  0x0000,   /* U+1D6BC */
  0x0000,   /* U+1D6BD */
  0x0000,   /* U+1D6BE */
  0x0000,   /* U+1D6BF */
  0x0000,   /* U+1D6C0 */
  0x0000,   /* U+1D6C1 */
  0x0000,   /* U+1D6C2 */
  0x0000,   /* U+1D6C3 */
  0x0000,   /* U+1D6C4 */
  0x0000,   /* U+1D6C5 */
  0x0000,   /* U+1D6C6 */
  0x0000,   /* U+1D6C7 */
  0x0000,   /* U+1D6C8 */
  0x0000,   /* U+1D6C9 */
  0x0000,   /* U+1D6CA */
  0x0000,   /* U+1D6CB */
  0x0000,   /* U+1D6CC */
  0x0000,   /* U+1D6CD */
  0x0000,   /* U+1D6CE */
  0x0000,   /* U+1D6CF */
  0x0000,   /* U+1D6D0 */
  0x0000,   /* U+1D6D1 */
  0x0000,   /* U+1D6D2 */
  0x0000,   /* U+1D6D3 */
  0x0000,   /* U+1D6D4 */
  0x0000,   /* U+1D6D5 */
  0x0000,   /* U+1D6D6 */
  0x0000,   /* U+1D6D7 */
  0x0000,   /* U+1D6D8 */
  0x0000,   /* U+1D6D9 */
  0x0000,   /* U+1D6DA */
  0x0000,   /* U+1D6DB */
  0x0000,   /* U+1D6DC */
  0x0000,   /* U+1D6DD */
  0x0000,   /* U+1D6DE */
  0x0000,   /* U+1D6DF */
  0x0000,   /* U+1D6E0 */
  0x0000,   /* U+1D6E1 */
  0x0000,   /* U+1D6E2 */
  0x0000,   /* U+1D6E3 */
  0x0000,   /* U+1D6E4 */
  0x0000,   /* U+1D6E5 */
  0x0000,   /* U+1D6E6 */
  0x0000,   /* U+1D6E7 */
  0x0000,   /* U+1D6E8 */
  0x0000,   /* U+1D6E9 */
  0x0000,   /* U+1D6EA */
  0x0000,   /* U+1D6EB */
  0x0000,   /* U+1D6EC */
  0x0000,   /* U+1D6ED */
  0x0000,   /* U+1D6EE */
  0x0000,   /* U+1D6EF */
  0x0000,   /* U+1D6F0 */
  0x0000,   /* U+1D6F1 */
  0x0000,   /* U+1D6F2 */
  0x0000,   /* U+1D6F3 */
  0x0000,   /* U+1D6F4 */
  0x0000,   /* U+1D6F5 */
  0x0000,   /* U+1D6F6 */
  0x0000,   /* U+1D6F7 */
  0x0000,   /* U+1D6F8 */
  0x0000,   /* U+1D6F9 */
  0x0000,   /* U+1D6FA */
  0x0000,   /* U+1D6FB */
  0x0000,   /* U+1D6FC */
  0x0000,   /* U+1D6FD */
  0x0000,   /* U+1D6FE */
  0x0000,   /* U+1D6FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D600 */
  0x0000,   /* U+1D601 */
  0x0000,   /* U+1D602 */
  0x0000,   /* U+1D603 */
  0x0000,   /* U+1D604 */
  0x0000,   /* U+1D605 */
  0x0000,   /* U+1D606 */
  0x0000,   /* U+1D607 */
  0x0000,   /* U+1D608 */
  0x0000,   /* U+1D609 */
  0x0000,   /* U+1D60A */
  0x0000,   /* U+1D60B */
  0x0000,   /* U+1D60C */
  0x0000,   /* U+1D60D */
  0x0000,   /* U+1D60E */
  0x0000,   /* U+1D60F */
  0x0000,   /* U+1D610 */
  0x0000,   /* U+1D611 */
  0x0000,   /* U+1D612 */
  0x0000,   /* U+1D613 */
  0x0000,   /* U+1D614 */
  0x0000,   /* U+1D615 */
  0x0000,   /* U+1D616 */
  0x0000,   /* U+1D617 */
  0x0000,   /* U+1D618 */
  0x0000,   /* U+1D619 */
  0x0000,   /* U+1D61A */
  0x0000,   /* U+1D61B */
  0x0000,   /* U+1D61C */
  0x0000,   /* U+1D61D */
  0x0000,   /* U+1D61E */
  0x0000,   /* U+1D61F */
  0x0000,   /* U+1D620 */
  0x0000,   /* U+1D621 */
  0x0000,   /* U+1D622 */
  0x0000,   /* U+1D623 */
  0x0000,   /* U+1D624 */
  0x0000,   /* U+1D625 */
  0x0000,   /* U+1D626 */
  0x0000,   /* U+1D627 */
  0x0000,   /* U+1D628 */
  0x0000,   /* U+1D629 */
  0x0000,   /* U+1D62A */
  0x0000,   /* U+1D62B */
  0x0000,   /* U+1D62C */
  0x0000,   /* U+1D62D */
  0x0000,   /* U+1D62E */
  0x0000,   /* U+1D62F */
  0x0000,   /* U+1D630 */
  0x0000,   /* U+1D631 */
  0x0000,   /* U+1D632 */
  0x0000,   /* U+1D633 */
  0x0000,   /* U+1D634 */
  0x0000,   /* U+1D635 */
  0x0000,   /* U+1D636 */
  0x0000,   /* U+1D637 */
  0x0000,   /* U+1D638 */
  0x0000,   /* U+1D639 */
  0x0000,   /* U+1D63A */
  0x0000,   /* U+1D63B */
  0x0000,   /* U+1D63C */
  0x0000,   /* U+1D63D */
  0x0000,   /* U+1D63E */
  0x0000,   /* U+1D63F */
  0x0000,   /* U+1D640 */
  0x0000,   /* U+1D641 */
  0x0000,   /* U+1D642 */
  0x0000,   /* U+1D643 */
  0x0000,   /* U+1D644 */
  0x0000,   /* U+1D645 */
  0x0000,   /* U+1D646 */
  0x0000,   /* U+1D647 */
  0x0000,   /* U+1D648 */
  0x0000,   /* U+1D649 */
  0x0000,   /* U+1D64A */
  0x0000,   /* U+1D64B */
  0x0000,   /* U+1D64C */
  0x0000,   /* U+1D64D */
  0x0000,   /* U+1D64E */
  0x0000,   /* U+1D64F */
  0x0000,   /* U+1D650 */
  0x0000,   /* U+1D651 */
  0x0000,   /* U+1D652 */
  0x0000,   /* U+1D653 */
  0x0000,   /* U+1D654 */
  0x0000,   /* U+1D655 */
  0x0000,   /* U+1D656 */
  0x0000,   /* U+1D657 */
  0x0000,   /* U+1D658 */
  0x0000,   /* U+1D659 */
  0x0000,   /* U+1D65A */
  0x0000,   /* U+1D65B */
  0x0000,   /* U+1D65C */
  0x0000,   /* U+1D65D */
  0x0000,   /* U+1D65E */
  0x0000,   /* U+1D65F */
  0x0000,   /* U+1D660 */
  0x0000,   /* U+1D661 */
  0x0000,   /* U+1D662 */
  0x0000,   /* U+1D663 */
  0x0000,   /* U+1D664 */
  0x0000,   /* U+1D665 */
  0x0000,   /* U+1D666 */
  0x0000,   /* U+1D667 */
  0x0000,   /* U+1D668 */
  0x0000,   /* U+1D669 */
  0x0000,   /* U+1D66A */
  0x0000,   /* U+1D66B */
  0x0000,   /* U+1D66C */
  0x0000,   /* U+1D66D */
  0x0000,   /* U+1D66E */
  0x0000,   /* U+1D66F */
  0x0000,   /* U+1D670 */
  0x0000,   /* U+1D671 */
  0x0000,   /* U+1D672 */
  0x0000,   /* U+1D673 */
  0x0000,   /* U+1D674 */
  0x0000,   /* U+1D675 */
  0x0000,   /* U+1D676 */
  0x0000,   /* U+1D677 */
  0x0000,   /* U+1D678 */
  0x0000,   /* U+1D679 */
  0x0000,   /* U+1D67A */
  0x0000,   /* U+1D67B */
  0x0000,   /* U+1D67C */
  0x0000,   /* U+1D67D */
  0x0000,   /* U+1D67E */
  0x0000,   /* U+1D67F */
  0x0000,   /* U+1D680 */
  0x0000,   /* U+1D681 */
  0x0000,   /* U+1D682 */
  0x0000,   /* U+1D683 */
  0x0000,   /* U+1D684 */
  0x0000,   /* U+1D685 */
  0x0000,   /* U+1D686 */
  0x0000,   /* U+1D687 */
  0x0000,   /* U+1D688 */
  0x0000,   /* U+1D689 */
  0x0000,   /* U+1D68A */
  0x0000,   /* U+1D68B */
  0x0000,   /* U+1D68C */
  0x0000,   /* U+1D68D */
  0x0000,   /* U+1D68E */
  0x0000,   /* U+1D68F */
  0x0000,   /* U+1D690 */
  0x0000,   /* U+1D691 */
  0x0000,   /* U+1D692 */
  0x0000,   /* U+1D693 */
  0x0000,   /* U+1D694 */
  0x0000,   /* U+1D695 */
  0x0000,   /* U+1D696 */
  0x0000,   /* U+1D697 */
  0x0000,   /* U+1D698 */
  0x0000,   /* U+1D699 */
  0x0000,   /* U+1D69A */
  0x0000,   /* U+1D69B */
  0x0000,   /* U+1D69C */
  0x0000,   /* U+1D69D */
  0x0000,   /* U+1D69E */
  0x0000,   /* U+1D69F */
  0x0000,   /* U+1D6A0 */
  0x0000,   /* U+1D6A1 */
  0x0000,   /* U+1D6A2 */
  0x0000,   /* U+1D6A3 */
  0x0000,   /* U+1D6A4 */
  0x0000,   /* U+1D6A5 */
  0x0000,   /* U+1D6A6 */
  0x0000,   /* U+1D6A7 */
  0x0000,   /* U+1D6A8 */
  0x0000,   /* U+1D6A9 */
  0x0000,   /* U+1D6AA */
  0x0000,   /* U+1D6AB */
  0x0000,   /* U+1D6AC */
  0x0000,   /* U+1D6AD */
  0x0000,   /* U+1D6AE */
  0x0000,   /* U+1D6AF */
  0x0000,   /* U+1D6B0 */
  0x0000,   /* U+1D6B1 */
  0x0000,   /* U+1D6B2 */
  0x0000,   /* U+1D6B3 */
  0x0000,   /* U+1D6B4 */
  0x0000,   /* U+1D6B5 */
  0x0000,   /* U+1D6B6 */
  0x0000,   /* U+1D6B7 */
  0x0000,   /* U+1D6B8 */
  0x0000,   /* U+1D6B9 */
  0x0000,   /* U+1D6BA */
  0x0000,   /* U+1D6BB */
  0x0000,   /* U+1D6BC */
  0x0000,   /* U+1D6BD */
  0x0000,   /* U+1D6BE */
  0x0000,   /* U+1D6BF */
  0x0000,   /* U+1D6C0 */
  0x0000,   /* U+1D6C1 */
  0x0000,   /* U+1D6C2 */
  0x0000,   /* U+1D6C3 */
  0x0000,   /* U+1D6C4 */
  0x0000,   /* U+1D6C5 */
  0x0000,   /* U+1D6C6 */
  0x0000,   /* U+1D6C7 */
  0x0000,   /* U+1D6C8 */
  0x0000,   /* U+1D6C9 */
  0x0000,   /* U+1D6CA */
  0x0000,   /* U+1D6CB */
  0x0000,   /* U+1D6CC */
  0x0000,   /* U+1D6CD */
  0x0000,   /* U+1D6CE */
  0x0000,   /* U+1D6CF */
  0x0000,   /* U+1D6D0 */
  0x0000,   /* U+1D6D1 */
  0x0000,   /* U+1D6D2 */
  0x0000,   /* U+1D6D3 */
  0x0000,   /* U+1D6D4 */
  0x0000,   /* U+1D6D5 */
  0x0000,   /* U+1D6D6 */
  0x0000,   /* U+1D6D7 */
  0x0000,   /* U+1D6D8 */
  0x0000,   /* U+1D6D9 */
  0x0000,   /* U+1D6DA */
  0x0000,   /* U+1D6DB */
  0x0000,   /* U+1D6DC */
  0x0000,   /* U+1D6DD */
  0x0000,   /* U+1D6DE */
  0x0000,   /* U+1D6DF */
  0x0000,   /* U+1D6E0 */
  0x0000,   /* U+1D6E1 */
  0x0000,   /* U+1D6E2 */
  0x0000,   /* U+1D6E3 */
  0x0000,   /* U+1D6E4 */
  0x0000,   /* U+1D6E5 */
  0x0000,   /* U+1D6E6 */
  0x0000,   /* U+1D6E7 */
  0x0000,   /* U+1D6E8 */
  0x0000,   /* U+1D6E9 */
  0x0000,   /* U+1D6EA */
  0x0000,   /* U+1D6EB */
  0x0000,   /* U+1D6EC */
  0x0000,   /* U+1D6ED */
  0x0000,   /* U+1D6EE */
  0x0000,   /* U+1D6EF */
  0x0000,   /* U+1D6F0 */
  0x0000,   /* U+1D6F1 */
  0x0000,   /* U+1D6F2 */
  0x0000,   /* U+1D6F3 */
  0x0000,   /* U+1D6F4 */
  0x0000,   /* U+1D6F5 */
  0x0000,   /* U+1D6F6 */
  0x0000,   /* U+1D6F7 */
  0x0000,   /* U+1D6F8 */
  0x0000,   /* U+1D6F9 */
  0x0000,   /* U+1D6FA */
  0x0000,   /* U+1D6FB */
  0x0000,   /* U+1D6FC */
  0x0000,   /* U+1D6FD */
  0x0000,   /* U+1D6FE */
  0x0000,   /* U+1D6FF */
};

uint16 uca900_p1D7[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1FBE,   /* U+1D700 */
  0x1FC2,   /* U+1D701 */
  0x1FC4,   /* U+1D702 */
  0x1FC5,   /* U+1D703 */
  0x1FC6,   /* U+1D704 */
  0x1FC8,   /* U+1D705 */
  0x1FC9,   /* U+1D706 */
  0x1FCB,   /* U+1D707 */
  0x1FCC,   /* U+1D708 */
  0x1FCD,   /* U+1D709 */
  0x1FCE,   /* U+1D70A */
  0x1FCF,   /* U+1D70B */
  0x1FD4,   /* U+1D70C */
  0x1FD7,   /* U+1D70D */
  0x1FD7,   /* U+1D70E */
  0x1FDB,   /* U+1D70F */
  0x1FDC,   /* U+1D710 */
  0x1FDD,   /* U+1D711 */
  0x1FDE,   /* U+1D712 */
  0x1FDF,   /* U+1D713 */
  0x1FE1,   /* U+1D714 */
  0x0608,   /* U+1D715 */
  0x1FBE,   /* U+1D716 */
  0x1FC5,   /* U+1D717 */
  0x1FC8,   /* U+1D718 */
  0x1FDD,   /* U+1D719 */
  0x1FD4,   /* U+1D71A */
  0x1FCF,   /* U+1D71B */
  0x1FB9,   /* U+1D71C */
  0x1FBA,   /* U+1D71D */
  0x1FBB,   /* U+1D71E */
  0x1FBD,   /* U+1D71F */
  0x1FBE,   /* U+1D720 */
  0x1FC2,   /* U+1D721 */
  0x1FC4,   /* U+1D722 */
  0x1FC5,   /* U+1D723 */
  0x1FC6,   /* U+1D724 */
  0x1FC8,   /* U+1D725 */
  0x1FC9,   /* U+1D726 */
  0x1FCB,   /* U+1D727 */
  0x1FCC,   /* U+1D728 */
  0x1FCD,   /* U+1D729 */
  0x1FCE,   /* U+1D72A */
  0x1FCF,   /* U+1D72B */
  0x1FD4,   /* U+1D72C */
  0x1FC5,   /* U+1D72D */
  0x1FD7,   /* U+1D72E */
  0x1FDB,   /* U+1D72F */
  0x1FDC,   /* U+1D730 */
  0x1FDD,   /* U+1D731 */
  0x1FDE,   /* U+1D732 */
  0x1FDF,   /* U+1D733 */
  0x1FE1,   /* U+1D734 */
  0x060C,   /* U+1D735 */
  0x1FB9,   /* U+1D736 */
  0x1FBA,   /* U+1D737 */
  0x1FBB,   /* U+1D738 */
  0x1FBD,   /* U+1D739 */
  0x1FBE,   /* U+1D73A */
  0x1FC2,   /* U+1D73B */
  0x1FC4,   /* U+1D73C */
  0x1FC5,   /* U+1D73D */
  0x1FC6,   /* U+1D73E */
  0x1FC8,   /* U+1D73F */
  0x1FC9,   /* U+1D740 */
  0x1FCB,   /* U+1D741 */
  0x1FCC,   /* U+1D742 */
  0x1FCD,   /* U+1D743 */
  0x1FCE,   /* U+1D744 */
  0x1FCF,   /* U+1D745 */
  0x1FD4,   /* U+1D746 */
  0x1FD7,   /* U+1D747 */
  0x1FD7,   /* U+1D748 */
  0x1FDB,   /* U+1D749 */
  0x1FDC,   /* U+1D74A */
  0x1FDD,   /* U+1D74B */
  0x1FDE,   /* U+1D74C */
  0x1FDF,   /* U+1D74D */
  0x1FE1,   /* U+1D74E */
  0x0608,   /* U+1D74F */
  0x1FBE,   /* U+1D750 */
  0x1FC5,   /* U+1D751 */
  0x1FC8,   /* U+1D752 */
  0x1FDD,   /* U+1D753 */
  0x1FD4,   /* U+1D754 */
  0x1FCF,   /* U+1D755 */
  0x1FB9,   /* U+1D756 */
  0x1FBA,   /* U+1D757 */
  0x1FBB,   /* U+1D758 */
  0x1FBD,   /* U+1D759 */
  0x1FBE,   /* U+1D75A */
  0x1FC2,   /* U+1D75B */
  0x1FC4,   /* U+1D75C */
  0x1FC5,   /* U+1D75D */
  0x1FC6,   /* U+1D75E */
  0x1FC8,   /* U+1D75F */
  0x1FC9,   /* U+1D760 */
  0x1FCB,   /* U+1D761 */
  0x1FCC,   /* U+1D762 */
  0x1FCD,   /* U+1D763 */
  0x1FCE,   /* U+1D764 */
  0x1FCF,   /* U+1D765 */
  0x1FD4,   /* U+1D766 */
  0x1FC5,   /* U+1D767 */
  0x1FD7,   /* U+1D768 */
  0x1FDB,   /* U+1D769 */
  0x1FDC,   /* U+1D76A */
  0x1FDD,   /* U+1D76B */
  0x1FDE,   /* U+1D76C */
  0x1FDF,   /* U+1D76D */
  0x1FE1,   /* U+1D76E */
  0x060C,   /* U+1D76F */
  0x1FB9,   /* U+1D770 */
  0x1FBA,   /* U+1D771 */
  0x1FBB,   /* U+1D772 */
  0x1FBD,   /* U+1D773 */
  0x1FBE,   /* U+1D774 */
  0x1FC2,   /* U+1D775 */
  0x1FC4,   /* U+1D776 */
  0x1FC5,   /* U+1D777 */
  0x1FC6,   /* U+1D778 */
  0x1FC8,   /* U+1D779 */
  0x1FC9,   /* U+1D77A */
  0x1FCB,   /* U+1D77B */
  0x1FCC,   /* U+1D77C */
  0x1FCD,   /* U+1D77D */
  0x1FCE,   /* U+1D77E */
  0x1FCF,   /* U+1D77F */
  0x1FD4,   /* U+1D780 */
  0x1FD7,   /* U+1D781 */
  0x1FD7,   /* U+1D782 */
  0x1FDB,   /* U+1D783 */
  0x1FDC,   /* U+1D784 */
  0x1FDD,   /* U+1D785 */
  0x1FDE,   /* U+1D786 */
  0x1FDF,   /* U+1D787 */
  0x1FE1,   /* U+1D788 */
  0x0608,   /* U+1D789 */
  0x1FBE,   /* U+1D78A */
  0x1FC5,   /* U+1D78B */
  0x1FC8,   /* U+1D78C */
  0x1FDD,   /* U+1D78D */
  0x1FD4,   /* U+1D78E */
  0x1FCF,   /* U+1D78F */
  0x1FB9,   /* U+1D790 */
  0x1FBA,   /* U+1D791 */
  0x1FBB,   /* U+1D792 */
  0x1FBD,   /* U+1D793 */
  0x1FBE,   /* U+1D794 */
  0x1FC2,   /* U+1D795 */
  0x1FC4,   /* U+1D796 */
  0x1FC5,   /* U+1D797 */
  0x1FC6,   /* U+1D798 */
  0x1FC8,   /* U+1D799 */
  0x1FC9,   /* U+1D79A */
  0x1FCB,   /* U+1D79B */
  0x1FCC,   /* U+1D79C */
  0x1FCD,   /* U+1D79D */
  0x1FCE,   /* U+1D79E */
  0x1FCF,   /* U+1D79F */
  0x1FD4,   /* U+1D7A0 */
  0x1FC5,   /* U+1D7A1 */
  0x1FD7,   /* U+1D7A2 */
  0x1FDB,   /* U+1D7A3 */
  0x1FDC,   /* U+1D7A4 */
  0x1FDD,   /* U+1D7A5 */
  0x1FDE,   /* U+1D7A6 */
  0x1FDF,   /* U+1D7A7 */
  0x1FE1,   /* U+1D7A8 */
  0x060C,   /* U+1D7A9 */
  0x1FB9,   /* U+1D7AA */
  0x1FBA,   /* U+1D7AB */
  0x1FBB,   /* U+1D7AC */
  0x1FBD,   /* U+1D7AD */
  0x1FBE,   /* U+1D7AE */
  0x1FC2,   /* U+1D7AF */
  0x1FC4,   /* U+1D7B0 */
  0x1FC5,   /* U+1D7B1 */
  0x1FC6,   /* U+1D7B2 */
  0x1FC8,   /* U+1D7B3 */
  0x1FC9,   /* U+1D7B4 */
  0x1FCB,   /* U+1D7B5 */
  0x1FCC,   /* U+1D7B6 */
  0x1FCD,   /* U+1D7B7 */
  0x1FCE,   /* U+1D7B8 */
  0x1FCF,   /* U+1D7B9 */
  0x1FD4,   /* U+1D7BA */
  0x1FD7,   /* U+1D7BB */
  0x1FD7,   /* U+1D7BC */
  0x1FDB,   /* U+1D7BD */
  0x1FDC,   /* U+1D7BE */
  0x1FDD,   /* U+1D7BF */
  0x1FDE,   /* U+1D7C0 */
  0x1FDF,   /* U+1D7C1 */
  0x1FE1,   /* U+1D7C2 */
  0x0608,   /* U+1D7C3 */
  0x1FBE,   /* U+1D7C4 */
  0x1FC5,   /* U+1D7C5 */
  0x1FC8,   /* U+1D7C6 */
  0x1FDD,   /* U+1D7C7 */
  0x1FD4,   /* U+1D7C8 */
  0x1FCF,   /* U+1D7C9 */
  0x1FBF,   /* U+1D7CA */
  0x1FBF,   /* U+1D7CB */
  0xFBC3,   /* U+1D7CC */
  0xFBC3,   /* U+1D7CD */
  0x1C3D,   /* U+1D7CE */
  0x1C3E,   /* U+1D7CF */
  0x1C3F,   /* U+1D7D0 */
  0x1C40,   /* U+1D7D1 */
  0x1C41,   /* U+1D7D2 */
  0x1C42,   /* U+1D7D3 */
  0x1C43,   /* U+1D7D4 */
  0x1C44,   /* U+1D7D5 */
  0x1C45,   /* U+1D7D6 */
  0x1C46,   /* U+1D7D7 */
  0x1C3D,   /* U+1D7D8 */
  0x1C3E,   /* U+1D7D9 */
  0x1C3F,   /* U+1D7DA */
  0x1C40,   /* U+1D7DB */
  0x1C41,   /* U+1D7DC */
  0x1C42,   /* U+1D7DD */
  0x1C43,   /* U+1D7DE */
  0x1C44,   /* U+1D7DF */
  0x1C45,   /* U+1D7E0 */
  0x1C46,   /* U+1D7E1 */
  0x1C3D,   /* U+1D7E2 */
  0x1C3E,   /* U+1D7E3 */
  0x1C3F,   /* U+1D7E4 */
  0x1C40,   /* U+1D7E5 */
  0x1C41,   /* U+1D7E6 */
  0x1C42,   /* U+1D7E7 */
  0x1C43,   /* U+1D7E8 */
  0x1C44,   /* U+1D7E9 */
  0x1C45,   /* U+1D7EA */
  0x1C46,   /* U+1D7EB */
  0x1C3D,   /* U+1D7EC */
  0x1C3E,   /* U+1D7ED */
  0x1C3F,   /* U+1D7EE */
  0x1C40,   /* U+1D7EF */
  0x1C41,   /* U+1D7F0 */
  0x1C42,   /* U+1D7F1 */
  0x1C43,   /* U+1D7F2 */
  0x1C44,   /* U+1D7F3 */
  0x1C45,   /* U+1D7F4 */
  0x1C46,   /* U+1D7F5 */
  0x1C3D,   /* U+1D7F6 */
  0x1C3E,   /* U+1D7F7 */
  0x1C3F,   /* U+1D7F8 */
  0x1C40,   /* U+1D7F9 */
  0x1C41,   /* U+1D7FA */
  0x1C42,   /* U+1D7FB */
  0x1C43,   /* U+1D7FC */
  0x1C44,   /* U+1D7FD */
  0x1C45,   /* U+1D7FE */
  0x1C46,   /* U+1D7FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D700 */
  0x0020,   /* U+1D701 */
  0x0020,   /* U+1D702 */
  0x0020,   /* U+1D703 */
  0x0020,   /* U+1D704 */
  0x0020,   /* U+1D705 */
  0x0020,   /* U+1D706 */
  0x0020,   /* U+1D707 */
  0x0020,   /* U+1D708 */
  0x0020,   /* U+1D709 */
  0x0020,   /* U+1D70A */
  0x0020,   /* U+1D70B */
  0x0020,   /* U+1D70C */
  0x0020,   /* U+1D70D */
  0x0020,   /* U+1D70E */
  0x0020,   /* U+1D70F */
  0x0020,   /* U+1D710 */
  0x0020,   /* U+1D711 */
  0x0020,   /* U+1D712 */
  0x0020,   /* U+1D713 */
  0x0020,   /* U+1D714 */
  0x0020,   /* U+1D715 */
  0x0020,   /* U+1D716 */
  0x0020,   /* U+1D717 */
  0x0020,   /* U+1D718 */
  0x0020,   /* U+1D719 */
  0x0020,   /* U+1D71A */
  0x0020,   /* U+1D71B */
  0x0020,   /* U+1D71C */
  0x0020,   /* U+1D71D */
  0x0020,   /* U+1D71E */
  0x0020,   /* U+1D71F */
  0x0020,   /* U+1D720 */
  0x0020,   /* U+1D721 */
  0x0020,   /* U+1D722 */
  0x0020,   /* U+1D723 */
  0x0020,   /* U+1D724 */
  0x0020,   /* U+1D725 */
  0x0020,   /* U+1D726 */
  0x0020,   /* U+1D727 */
  0x0020,   /* U+1D728 */
  0x0020,   /* U+1D729 */
  0x0020,   /* U+1D72A */
  0x0020,   /* U+1D72B */
  0x0020,   /* U+1D72C */
  0x0020,   /* U+1D72D */
  0x0020,   /* U+1D72E */
  0x0020,   /* U+1D72F */
  0x0020,   /* U+1D730 */
  0x0020,   /* U+1D731 */
  0x0020,   /* U+1D732 */
  0x0020,   /* U+1D733 */
  0x0020,   /* U+1D734 */
  0x0020,   /* U+1D735 */
  0x0020,   /* U+1D736 */
  0x0020,   /* U+1D737 */
  0x0020,   /* U+1D738 */
  0x0020,   /* U+1D739 */
  0x0020,   /* U+1D73A */
  0x0020,   /* U+1D73B */
  0x0020,   /* U+1D73C */
  0x0020,   /* U+1D73D */
  0x0020,   /* U+1D73E */
  0x0020,   /* U+1D73F */
  0x0020,   /* U+1D740 */
  0x0020,   /* U+1D741 */
  0x0020,   /* U+1D742 */
  0x0020,   /* U+1D743 */
  0x0020,   /* U+1D744 */
  0x0020,   /* U+1D745 */
  0x0020,   /* U+1D746 */
  0x0020,   /* U+1D747 */
  0x0020,   /* U+1D748 */
  0x0020,   /* U+1D749 */
  0x0020,   /* U+1D74A */
  0x0020,   /* U+1D74B */
  0x0020,   /* U+1D74C */
  0x0020,   /* U+1D74D */
  0x0020,   /* U+1D74E */
  0x0020,   /* U+1D74F */
  0x0020,   /* U+1D750 */
  0x0020,   /* U+1D751 */
  0x0020,   /* U+1D752 */
  0x0020,   /* U+1D753 */
  0x0020,   /* U+1D754 */
  0x0020,   /* U+1D755 */
  0x0020,   /* U+1D756 */
  0x0020,   /* U+1D757 */
  0x0020,   /* U+1D758 */
  0x0020,   /* U+1D759 */
  0x0020,   /* U+1D75A */
  0x0020,   /* U+1D75B */
  0x0020,   /* U+1D75C */
  0x0020,   /* U+1D75D */
  0x0020,   /* U+1D75E */
  0x0020,   /* U+1D75F */
  0x0020,   /* U+1D760 */
  0x0020,   /* U+1D761 */
  0x0020,   /* U+1D762 */
  0x0020,   /* U+1D763 */
  0x0020,   /* U+1D764 */
  0x0020,   /* U+1D765 */
  0x0020,   /* U+1D766 */
  0x0020,   /* U+1D767 */
  0x0020,   /* U+1D768 */
  0x0020,   /* U+1D769 */
  0x0020,   /* U+1D76A */
  0x0020,   /* U+1D76B */
  0x0020,   /* U+1D76C */
  0x0020,   /* U+1D76D */
  0x0020,   /* U+1D76E */
  0x0020,   /* U+1D76F */
  0x0020,   /* U+1D770 */
  0x0020,   /* U+1D771 */
  0x0020,   /* U+1D772 */
  0x0020,   /* U+1D773 */
  0x0020,   /* U+1D774 */
  0x0020,   /* U+1D775 */
  0x0020,   /* U+1D776 */
  0x0020,   /* U+1D777 */
  0x0020,   /* U+1D778 */
  0x0020,   /* U+1D779 */
  0x0020,   /* U+1D77A */
  0x0020,   /* U+1D77B */
  0x0020,   /* U+1D77C */
  0x0020,   /* U+1D77D */
  0x0020,   /* U+1D77E */
  0x0020,   /* U+1D77F */
  0x0020,   /* U+1D780 */
  0x0020,   /* U+1D781 */
  0x0020,   /* U+1D782 */
  0x0020,   /* U+1D783 */
  0x0020,   /* U+1D784 */
  0x0020,   /* U+1D785 */
  0x0020,   /* U+1D786 */
  0x0020,   /* U+1D787 */
  0x0020,   /* U+1D788 */
  0x0020,   /* U+1D789 */
  0x0020,   /* U+1D78A */
  0x0020,   /* U+1D78B */
  0x0020,   /* U+1D78C */
  0x0020,   /* U+1D78D */
  0x0020,   /* U+1D78E */
  0x0020,   /* U+1D78F */
  0x0020,   /* U+1D790 */
  0x0020,   /* U+1D791 */
  0x0020,   /* U+1D792 */
  0x0020,   /* U+1D793 */
  0x0020,   /* U+1D794 */
  0x0020,   /* U+1D795 */
  0x0020,   /* U+1D796 */
  0x0020,   /* U+1D797 */
  0x0020,   /* U+1D798 */
  0x0020,   /* U+1D799 */
  0x0020,   /* U+1D79A */
  0x0020,   /* U+1D79B */
  0x0020,   /* U+1D79C */
  0x0020,   /* U+1D79D */
  0x0020,   /* U+1D79E */
  0x0020,   /* U+1D79F */
  0x0020,   /* U+1D7A0 */
  0x0020,   /* U+1D7A1 */
  0x0020,   /* U+1D7A2 */
  0x0020,   /* U+1D7A3 */
  0x0020,   /* U+1D7A4 */
  0x0020,   /* U+1D7A5 */
  0x0020,   /* U+1D7A6 */
  0x0020,   /* U+1D7A7 */
  0x0020,   /* U+1D7A8 */
  0x0020,   /* U+1D7A9 */
  0x0020,   /* U+1D7AA */
  0x0020,   /* U+1D7AB */
  0x0020,   /* U+1D7AC */
  0x0020,   /* U+1D7AD */
  0x0020,   /* U+1D7AE */
  0x0020,   /* U+1D7AF */
  0x0020,   /* U+1D7B0 */
  0x0020,   /* U+1D7B1 */
  0x0020,   /* U+1D7B2 */
  0x0020,   /* U+1D7B3 */
  0x0020,   /* U+1D7B4 */
  0x0020,   /* U+1D7B5 */
  0x0020,   /* U+1D7B6 */
  0x0020,   /* U+1D7B7 */
  0x0020,   /* U+1D7B8 */
  0x0020,   /* U+1D7B9 */
  0x0020,   /* U+1D7BA */
  0x0020,   /* U+1D7BB */
  0x0020,   /* U+1D7BC */
  0x0020,   /* U+1D7BD */
  0x0020,   /* U+1D7BE */
  0x0020,   /* U+1D7BF */
  0x0020,   /* U+1D7C0 */
  0x0020,   /* U+1D7C1 */
  0x0020,   /* U+1D7C2 */
  0x0020,   /* U+1D7C3 */
  0x0020,   /* U+1D7C4 */
  0x0020,   /* U+1D7C5 */
  0x0020,   /* U+1D7C6 */
  0x0020,   /* U+1D7C7 */
  0x0020,   /* U+1D7C8 */
  0x0020,   /* U+1D7C9 */
  0x0020,   /* U+1D7CA */
  0x0020,   /* U+1D7CB */
  0x0020,   /* U+1D7CC */
  0x0020,   /* U+1D7CD */
  0x0020,   /* U+1D7CE */
  0x0020,   /* U+1D7CF */
  0x0020,   /* U+1D7D0 */
  0x0020,   /* U+1D7D1 */
  0x0020,   /* U+1D7D2 */
  0x0020,   /* U+1D7D3 */
  0x0020,   /* U+1D7D4 */
  0x0020,   /* U+1D7D5 */
  0x0020,   /* U+1D7D6 */
  0x0020,   /* U+1D7D7 */
  0x0020,   /* U+1D7D8 */
  0x0020,   /* U+1D7D9 */
  0x0020,   /* U+1D7DA */
  0x0020,   /* U+1D7DB */
  0x0020,   /* U+1D7DC */
  0x0020,   /* U+1D7DD */
  0x0020,   /* U+1D7DE */
  0x0020,   /* U+1D7DF */
  0x0020,   /* U+1D7E0 */
  0x0020,   /* U+1D7E1 */
  0x0020,   /* U+1D7E2 */
  0x0020,   /* U+1D7E3 */
  0x0020,   /* U+1D7E4 */
  0x0020,   /* U+1D7E5 */
  0x0020,   /* U+1D7E6 */
  0x0020,   /* U+1D7E7 */
  0x0020,   /* U+1D7E8 */
  0x0020,   /* U+1D7E9 */
  0x0020,   /* U+1D7EA */
  0x0020,   /* U+1D7EB */
  0x0020,   /* U+1D7EC */
  0x0020,   /* U+1D7ED */
  0x0020,   /* U+1D7EE */
  0x0020,   /* U+1D7EF */
  0x0020,   /* U+1D7F0 */
  0x0020,   /* U+1D7F1 */
  0x0020,   /* U+1D7F2 */
  0x0020,   /* U+1D7F3 */
  0x0020,   /* U+1D7F4 */
  0x0020,   /* U+1D7F5 */
  0x0020,   /* U+1D7F6 */
  0x0020,   /* U+1D7F7 */
  0x0020,   /* U+1D7F8 */
  0x0020,   /* U+1D7F9 */
  0x0020,   /* U+1D7FA */
  0x0020,   /* U+1D7FB */
  0x0020,   /* U+1D7FC */
  0x0020,   /* U+1D7FD */
  0x0020,   /* U+1D7FE */
  0x0020,   /* U+1D7FF */

  /* Tertiary weight 1 for each character. */
  0x0005,   /* U+1D700 */
  0x0005,   /* U+1D701 */
  0x0005,   /* U+1D702 */
  0x0005,   /* U+1D703 */
  0x0005,   /* U+1D704 */
  0x0005,   /* U+1D705 */
  0x0005,   /* U+1D706 */
  0x0005,   /* U+1D707 */
  0x0005,   /* U+1D708 */
  0x0005,   /* U+1D709 */
  0x0005,   /* U+1D70A */
  0x0005,   /* U+1D70B */
  0x0005,   /* U+1D70C */
  0x0005,   /* U+1D70D */
  0x0005,   /* U+1D70E */
  0x0005,   /* U+1D70F */
  0x0005,   /* U+1D710 */
  0x0005,   /* U+1D711 */
  0x0005,   /* U+1D712 */
  0x0005,   /* U+1D713 */
  0x0005,   /* U+1D714 */
  0x0005,   /* U+1D715 */
  0x0005,   /* U+1D716 */
  0x0005,   /* U+1D717 */
  0x0005,   /* U+1D718 */
  0x0005,   /* U+1D719 */
  0x0005,   /* U+1D71A */
  0x0005,   /* U+1D71B */
  0x000B,   /* U+1D71C */
  0x000B,   /* U+1D71D */
  0x000B,   /* U+1D71E */
  0x000B,   /* U+1D71F */
  0x000B,   /* U+1D720 */
  0x000B,   /* U+1D721 */
  0x000B,   /* U+1D722 */
  0x000B,   /* U+1D723 */
  0x000B,   /* U+1D724 */
  0x000B,   /* U+1D725 */
  0x000B,   /* U+1D726 */
  0x000B,   /* U+1D727 */
  0x000B,   /* U+1D728 */
  0x000B,   /* U+1D729 */
  0x000B,   /* U+1D72A */
  0x000B,   /* U+1D72B */
  0x000B,   /* U+1D72C */
  0x000B,   /* U+1D72D */
  0x000B,   /* U+1D72E */
  0x000B,   /* U+1D72F */
  0x000B,   /* U+1D730 */
  0x000B,   /* U+1D731 */
  0x000B,   /* U+1D732 */
  0x000B,   /* U+1D733 */
  0x000B,   /* U+1D734 */
  0x0005,   /* U+1D735 */
  0x0005,   /* U+1D736 */
  0x0005,   /* U+1D737 */
  0x0005,   /* U+1D738 */
  0x0005,   /* U+1D739 */
  0x0005,   /* U+1D73A */
  0x0005,   /* U+1D73B */
  0x0005,   /* U+1D73C */
  0x0005,   /* U+1D73D */
  0x0005,   /* U+1D73E */
  0x0005,   /* U+1D73F */
  0x0005,   /* U+1D740 */
  0x0005,   /* U+1D741 */
  0x0005,   /* U+1D742 */
  0x0005,   /* U+1D743 */
  0x0005,   /* U+1D744 */
  0x0005,   /* U+1D745 */
  0x0005,   /* U+1D746 */
  0x0005,   /* U+1D747 */
  0x0005,   /* U+1D748 */
  0x0005,   /* U+1D749 */
  0x0005,   /* U+1D74A */
  0x0005,   /* U+1D74B */
  0x0005,   /* U+1D74C */
  0x0005,   /* U+1D74D */
  0x0005,   /* U+1D74E */
  0x0005,   /* U+1D74F */
  0x0005,   /* U+1D750 */
  0x0005,   /* U+1D751 */
  0x0005,   /* U+1D752 */
  0x0005,   /* U+1D753 */
  0x0005,   /* U+1D754 */
  0x0005,   /* U+1D755 */
  0x000B,   /* U+1D756 */
  0x000B,   /* U+1D757 */
  0x000B,   /* U+1D758 */
  0x000B,   /* U+1D759 */
  0x000B,   /* U+1D75A */
  0x000B,   /* U+1D75B */
  0x000B,   /* U+1D75C */
  0x000B,   /* U+1D75D */
  0x000B,   /* U+1D75E */
  0x000B,   /* U+1D75F */
  0x000B,   /* U+1D760 */
  0x000B,   /* U+1D761 */
  0x000B,   /* U+1D762 */
  0x000B,   /* U+1D763 */
  0x000B,   /* U+1D764 */
  0x000B,   /* U+1D765 */
  0x000B,   /* U+1D766 */
  0x000B,   /* U+1D767 */
  0x000B,   /* U+1D768 */
  0x000B,   /* U+1D769 */
  0x000B,   /* U+1D76A */
  0x000B,   /* U+1D76B */
  0x000B,   /* U+1D76C */
  0x000B,   /* U+1D76D */
  0x000B,   /* U+1D76E */
  0x0005,   /* U+1D76F */
  0x0005,   /* U+1D770 */
  0x0005,   /* U+1D771 */
  0x0005,   /* U+1D772 */
  0x0005,   /* U+1D773 */
  0x0005,   /* U+1D774 */
  0x0005,   /* U+1D775 */
  0x0005,   /* U+1D776 */
  0x0005,   /* U+1D777 */
  0x0005,   /* U+1D778 */
  0x0005,   /* U+1D779 */
  0x0005,   /* U+1D77A */
  0x0005,   /* U+1D77B */
  0x0005,   /* U+1D77C */
  0x0005,   /* U+1D77D */
  0x0005,   /* U+1D77E */
  0x0005,   /* U+1D77F */
  0x0005,   /* U+1D780 */
  0x0005,   /* U+1D781 */
  0x0005,   /* U+1D782 */
  0x0005,   /* U+1D783 */
  0x0005,   /* U+1D784 */
  0x0005,   /* U+1D785 */
  0x0005,   /* U+1D786 */
  0x0005,   /* U+1D787 */
  0x0005,   /* U+1D788 */
  0x0005,   /* U+1D789 */
  0x0005,   /* U+1D78A */
  0x0005,   /* U+1D78B */
  0x0005,   /* U+1D78C */
  0x0005,   /* U+1D78D */
  0x0005,   /* U+1D78E */
  0x0005,   /* U+1D78F */
  0x000B,   /* U+1D790 */
  0x000B,   /* U+1D791 */
  0x000B,   /* U+1D792 */
  0x000B,   /* U+1D793 */
  0x000B,   /* U+1D794 */
  0x000B,   /* U+1D795 */
  0x000B,   /* U+1D796 */
  0x000B,   /* U+1D797 */
  0x000B,   /* U+1D798 */
  0x000B,   /* U+1D799 */
  0x000B,   /* U+1D79A */
  0x000B,   /* U+1D79B */
  0x000B,   /* U+1D79C */
  0x000B,   /* U+1D79D */
  0x000B,   /* U+1D79E */
  0x000B,   /* U+1D79F */
  0x000B,   /* U+1D7A0 */
  0x000B,   /* U+1D7A1 */
  0x000B,   /* U+1D7A2 */
  0x000B,   /* U+1D7A3 */
  0x000B,   /* U+1D7A4 */
  0x000B,   /* U+1D7A5 */
  0x000B,   /* U+1D7A6 */
  0x000B,   /* U+1D7A7 */
  0x000B,   /* U+1D7A8 */
  0x0005,   /* U+1D7A9 */
  0x0005,   /* U+1D7AA */
  0x0005,   /* U+1D7AB */
  0x0005,   /* U+1D7AC */
  0x0005,   /* U+1D7AD */
  0x0005,   /* U+1D7AE */
  0x0005,   /* U+1D7AF */
  0x0005,   /* U+1D7B0 */
  0x0005,   /* U+1D7B1 */
  0x0005,   /* U+1D7B2 */
  0x0005,   /* U+1D7B3 */
  0x0005,   /* U+1D7B4 */
  0x0005,   /* U+1D7B5 */
  0x0005,   /* U+1D7B6 */
  0x0005,   /* U+1D7B7 */
  0x0005,   /* U+1D7B8 */
  0x0005,   /* U+1D7B9 */
  0x0005,   /* U+1D7BA */
  0x0005,   /* U+1D7BB */
  0x0005,   /* U+1D7BC */
  0x0005,   /* U+1D7BD */
  0x0005,   /* U+1D7BE */
  0x0005,   /* U+1D7BF */
  0x0005,   /* U+1D7C0 */
  0x0005,   /* U+1D7C1 */
  0x0005,   /* U+1D7C2 */
  0x0005,   /* U+1D7C3 */
  0x0005,   /* U+1D7C4 */
  0x0005,   /* U+1D7C5 */
  0x0005,   /* U+1D7C6 */
  0x0005,   /* U+1D7C7 */
  0x0005,   /* U+1D7C8 */
  0x0005,   /* U+1D7C9 */
  0x000B,   /* U+1D7CA */
  0x0005,   /* U+1D7CB */
  0x0002,   /* U+1D7CC */
  0x0002,   /* U+1D7CD */
  0x0005,   /* U+1D7CE */
  0x0005,   /* U+1D7CF */
  0x0005,   /* U+1D7D0 */
  0x0005,   /* U+1D7D1 */
  0x0005,   /* U+1D7D2 */
  0x0005,   /* U+1D7D3 */
  0x0005,   /* U+1D7D4 */
  0x0005,   /* U+1D7D5 */
  0x0005,   /* U+1D7D6 */
  0x0005,   /* U+1D7D7 */
  0x0005,   /* U+1D7D8 */
  0x0005,   /* U+1D7D9 */
  0x0005,   /* U+1D7DA */
  0x0005,   /* U+1D7DB */
  0x0005,   /* U+1D7DC */
  0x0005,   /* U+1D7DD */
  0x0005,   /* U+1D7DE */
  0x0005,   /* U+1D7DF */
  0x0005,   /* U+1D7E0 */
  0x0005,   /* U+1D7E1 */
  0x0005,   /* U+1D7E2 */
  0x0005,   /* U+1D7E3 */
  0x0005,   /* U+1D7E4 */
  0x0005,   /* U+1D7E5 */
  0x0005,   /* U+1D7E6 */
  0x0005,   /* U+1D7E7 */
  0x0005,   /* U+1D7E8 */
  0x0005,   /* U+1D7E9 */
  0x0005,   /* U+1D7EA */
  0x0005,   /* U+1D7EB */
  0x0005,   /* U+1D7EC */
  0x0005,   /* U+1D7ED */
  0x0005,   /* U+1D7EE */
  0x0005,   /* U+1D7EF */
  0x0005,   /* U+1D7F0 */
  0x0005,   /* U+1D7F1 */
  0x0005,   /* U+1D7F2 */
  0x0005,   /* U+1D7F3 */
  0x0005,   /* U+1D7F4 */
  0x0005,   /* U+1D7F5 */
  0x0005,   /* U+1D7F6 */
  0x0005,   /* U+1D7F7 */
  0x0005,   /* U+1D7F8 */
  0x0005,   /* U+1D7F9 */
  0x0005,   /* U+1D7FA */
  0x0005,   /* U+1D7FB */
  0x0005,   /* U+1D7FC */
  0x0005,   /* U+1D7FD */
  0x0005,   /* U+1D7FE */
  0x0005,   /* U+1D7FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1D700 */
  0x0000,   /* U+1D701 */
  0x0000,   /* U+1D702 */
  0x0000,   /* U+1D703 */
  0x0000,   /* U+1D704 */
  0x0000,   /* U+1D705 */
  0x0000,   /* U+1D706 */
  0x0000,   /* U+1D707 */
  0x0000,   /* U+1D708 */
  0x0000,   /* U+1D709 */
  0x0000,   /* U+1D70A */
  0x0000,   /* U+1D70B */
  0x0000,   /* U+1D70C */
  0x0000,   /* U+1D70D */
  0x0000,   /* U+1D70E */
  0x0000,   /* U+1D70F */
  0x0000,   /* U+1D710 */
  0x0000,   /* U+1D711 */
  0x0000,   /* U+1D712 */
  0x0000,   /* U+1D713 */
  0x0000,   /* U+1D714 */
  0x0000,   /* U+1D715 */
  0x0000,   /* U+1D716 */
  0x0000,   /* U+1D717 */
  0x0000,   /* U+1D718 */
  0x0000,   /* U+1D719 */
  0x0000,   /* U+1D71A */
  0x0000,   /* U+1D71B */
  0x0000,   /* U+1D71C */
  0x0000,   /* U+1D71D */
  0x0000,   /* U+1D71E */
  0x0000,   /* U+1D71F */
  0x0000,   /* U+1D720 */
  0x0000,   /* U+1D721 */
  0x0000,   /* U+1D722 */
  0x0000,   /* U+1D723 */
  0x0000,   /* U+1D724 */
  0x0000,   /* U+1D725 */
  0x0000,   /* U+1D726 */
  0x0000,   /* U+1D727 */
  0x0000,   /* U+1D728 */
  0x0000,   /* U+1D729 */
  0x0000,   /* U+1D72A */
  0x0000,   /* U+1D72B */
  0x0000,   /* U+1D72C */
  0x0000,   /* U+1D72D */
  0x0000,   /* U+1D72E */
  0x0000,   /* U+1D72F */
  0x0000,   /* U+1D730 */
  0x0000,   /* U+1D731 */
  0x0000,   /* U+1D732 */
  0x0000,   /* U+1D733 */
  0x0000,   /* U+1D734 */
  0x0000,   /* U+1D735 */
  0x0000,   /* U+1D736 */
  0x0000,   /* U+1D737 */
  0x0000,   /* U+1D738 */
  0x0000,   /* U+1D739 */
  0x0000,   /* U+1D73A */
  0x0000,   /* U+1D73B */
  0x0000,   /* U+1D73C */
  0x0000,   /* U+1D73D */
  0x0000,   /* U+1D73E */
  0x0000,   /* U+1D73F */
  0x0000,   /* U+1D740 */
  0x0000,   /* U+1D741 */
  0x0000,   /* U+1D742 */
  0x0000,   /* U+1D743 */
  0x0000,   /* U+1D744 */
  0x0000,   /* U+1D745 */
  0x0000,   /* U+1D746 */
  0x0000,   /* U+1D747 */
  0x0000,   /* U+1D748 */
  0x0000,   /* U+1D749 */
  0x0000,   /* U+1D74A */
  0x0000,   /* U+1D74B */
  0x0000,   /* U+1D74C */
  0x0000,   /* U+1D74D */
  0x0000,   /* U+1D74E */
  0x0000,   /* U+1D74F */
  0x0000,   /* U+1D750 */
  0x0000,   /* U+1D751 */
  0x0000,   /* U+1D752 */
  0x0000,   /* U+1D753 */
  0x0000,   /* U+1D754 */
  0x0000,   /* U+1D755 */
  0x0000,   /* U+1D756 */
  0x0000,   /* U+1D757 */
  0x0000,   /* U+1D758 */
  0x0000,   /* U+1D759 */
  0x0000,   /* U+1D75A */
  0x0000,   /* U+1D75B */
  0x0000,   /* U+1D75C */
  0x0000,   /* U+1D75D */
  0x0000,   /* U+1D75E */
  0x0000,   /* U+1D75F */
  0x0000,   /* U+1D760 */
  0x0000,   /* U+1D761 */
  0x0000,   /* U+1D762 */
  0x0000,   /* U+1D763 */
  0x0000,   /* U+1D764 */
  0x0000,   /* U+1D765 */
  0x0000,   /* U+1D766 */
  0x0000,   /* U+1D767 */
  0x0000,   /* U+1D768 */
  0x0000,   /* U+1D769 */
  0x0000,   /* U+1D76A */
  0x0000,   /* U+1D76B */
  0x0000,   /* U+1D76C */
  0x0000,   /* U+1D76D */
  0x0000,   /* U+1D76E */
  0x0000,   /* U+1D76F */
  0x0000,   /* U+1D770 */
  0x0000,   /* U+1D771 */
  0x0000,   /* U+1D772 */
  0x0000,   /* U+1D773 */
  0x0000,   /* U+1D774 */
  0x0000,   /* U+1D775 */
  0x0000,   /* U+1D776 */
  0x0000,   /* U+1D777 */
  0x0000,   /* U+1D778 */
  0x0000,   /* U+1D779 */
  0x0000,   /* U+1D77A */
  0x0000,   /* U+1D77B */
  0x0000,   /* U+1D77C */
  0x0000,   /* U+1D77D */
  0x0000,   /* U+1D77E */
  0x0000,   /* U+1D77F */
  0x0000,   /* U+1D780 */
  0x0000,   /* U+1D781 */
  0x0000,   /* U+1D782 */
  0x0000,   /* U+1D783 */
  0x0000,   /* U+1D784 */
  0x0000,   /* U+1D785 */
  0x0000,   /* U+1D786 */
  0x0000,   /* U+1D787 */
  0x0000,   /* U+1D788 */
  0x0000,   /* U+1D789 */
  0x0000,   /* U+1D78A */
  0x0000,   /* U+1D78B */
  0x0000,   /* U+1D78C */
  0x0000,   /* U+1D78D */
  0x0000,   /* U+1D78E */
  0x0000,   /* U+1D78F */
  0x0000,   /* U+1D790 */
  0x0000,   /* U+1D791 */
  0x0000,   /* U+1D792 */
  0x0000,   /* U+1D793 */
  0x0000,   /* U+1D794 */
  0x0000,   /* U+1D795 */
  0x0000,   /* U+1D796 */
  0x0000,   /* U+1D797 */
  0x0000,   /* U+1D798 */
  0x0000,   /* U+1D799 */
  0x0000,   /* U+1D79A */
  0x0000,   /* U+1D79B */
  0x0000,   /* U+1D79C */
  0x0000,   /* U+1D79D */
  0x0000,   /* U+1D79E */
  0x0000,   /* U+1D79F */
  0x0000,   /* U+1D7A0 */
  0x0000,   /* U+1D7A1 */
  0x0000,   /* U+1D7A2 */
  0x0000,   /* U+1D7A3 */
  0x0000,   /* U+1D7A4 */
  0x0000,   /* U+1D7A5 */
  0x0000,   /* U+1D7A6 */
  0x0000,   /* U+1D7A7 */
  0x0000,   /* U+1D7A8 */
  0x0000,   /* U+1D7A9 */
  0x0000,   /* U+1D7AA */
  0x0000,   /* U+1D7AB */
  0x0000,   /* U+1D7AC */
  0x0000,   /* U+1D7AD */
  0x0000,   /* U+1D7AE */
  0x0000,   /* U+1D7AF */
  0x0000,   /* U+1D7B0 */
  0x0000,   /* U+1D7B1 */
  0x0000,   /* U+1D7B2 */
  0x0000,   /* U+1D7B3 */
  0x0000,   /* U+1D7B4 */
  0x0000,   /* U+1D7B5 */
  0x0000,   /* U+1D7B6 */
  0x0000,   /* U+1D7B7 */
  0x0000,   /* U+1D7B8 */
  0x0000,   /* U+1D7B9 */
  0x0000,   /* U+1D7BA */
  0x0000,   /* U+1D7BB */
  0x0000,   /* U+1D7BC */
  0x0000,   /* U+1D7BD */
  0x0000,   /* U+1D7BE */
  0x0000,   /* U+1D7BF */
  0x0000,   /* U+1D7C0 */
  0x0000,   /* U+1D7C1 */
  0x0000,   /* U+1D7C2 */
  0x0000,   /* U+1D7C3 */
  0x0000,   /* U+1D7C4 */
  0x0000,   /* U+1D7C5 */
  0x0000,   /* U+1D7C6 */
  0x0000,   /* U+1D7C7 */
  0x0000,   /* U+1D7C8 */
  0x0000,   /* U+1D7C9 */
  0x0000,   /* U+1D7CA */
  0x0000,   /* U+1D7CB */
  0xD7CC,   /* U+1D7CC */
  0xD7CD,   /* U+1D7CD */
  0x0000,   /* U+1D7CE */
  0x0000,   /* U+1D7CF */
  0x0000,   /* U+1D7D0 */
  0x0000,   /* U+1D7D1 */
  0x0000,   /* U+1D7D2 */
  0x0000,   /* U+1D7D3 */
  0x0000,   /* U+1D7D4 */
  0x0000,   /* U+1D7D5 */
  0x0000,   /* U+1D7D6 */
  0x0000,   /* U+1D7D7 */
  0x0000,   /* U+1D7D8 */
  0x0000,   /* U+1D7D9 */
  0x0000,   /* U+1D7DA */
  0x0000,   /* U+1D7DB */
  0x0000,   /* U+1D7DC */
  0x0000,   /* U+1D7DD */
  0x0000,   /* U+1D7DE */
  0x0000,   /* U+1D7DF */
  0x0000,   /* U+1D7E0 */
  0x0000,   /* U+1D7E1 */
  0x0000,   /* U+1D7E2 */
  0x0000,   /* U+1D7E3 */
  0x0000,   /* U+1D7E4 */
  0x0000,   /* U+1D7E5 */
  0x0000,   /* U+1D7E6 */
  0x0000,   /* U+1D7E7 */
  0x0000,   /* U+1D7E8 */
  0x0000,   /* U+1D7E9 */
  0x0000,   /* U+1D7EA */
  0x0000,   /* U+1D7EB */
  0x0000,   /* U+1D7EC */
  0x0000,   /* U+1D7ED */
  0x0000,   /* U+1D7EE */
  0x0000,   /* U+1D7EF */
  0x0000,   /* U+1D7F0 */
  0x0000,   /* U+1D7F1 */
  0x0000,   /* U+1D7F2 */
  0x0000,   /* U+1D7F3 */
  0x0000,   /* U+1D7F4 */
  0x0000,   /* U+1D7F5 */
  0x0000,   /* U+1D7F6 */
  0x0000,   /* U+1D7F7 */
  0x0000,   /* U+1D7F8 */
  0x0000,   /* U+1D7F9 */
  0x0000,   /* U+1D7FA */
  0x0000,   /* U+1D7FB */
  0x0000,   /* U+1D7FC */
  0x0000,   /* U+1D7FD */
  0x0000,   /* U+1D7FE */
  0x0000,   /* U+1D7FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1D700 */
  0x0000,   /* U+1D701 */
  0x0000,   /* U+1D702 */
  0x0000,   /* U+1D703 */
  0x0000,   /* U+1D704 */
  0x0000,   /* U+1D705 */
  0x0000,   /* U+1D706 */
  0x0000,   /* U+1D707 */
  0x0000,   /* U+1D708 */
  0x0000,   /* U+1D709 */
  0x0000,   /* U+1D70A */
  0x0000,   /* U+1D70B */
  0x0000,   /* U+1D70C */
  0x0000,   /* U+1D70D */
  0x0000,   /* U+1D70E */
  0x0000,   /* U+1D70F */
  0x0000,   /* U+1D710 */
  0x0000,   /* U+1D711 */
  0x0000,   /* U+1D712 */
  0x0000,   /* U+1D713 */
  0x0000,   /* U+1D714 */
  0x0000,   /* U+1D715 */
  0x0000,   /* U+1D716 */
  0x0000,   /* U+1D717 */
  0x0000,   /* U+1D718 */
  0x0000,   /* U+1D719 */
  0x0000,   /* U+1D71A */
  0x0000,   /* U+1D71B */
  0x0000,   /* U+1D71C */
  0x0000,   /* U+1D71D */
  0x0000,   /* U+1D71E */
  0x0000,   /* U+1D71F */
  0x0000,   /* U+1D720 */
  0x0000,   /* U+1D721 */
  0x0000,   /* U+1D722 */
  0x0000,   /* U+1D723 */
  0x0000,   /* U+1D724 */
  0x0000,   /* U+1D725 */
  0x0000,   /* U+1D726 */
  0x0000,   /* U+1D727 */
  0x0000,   /* U+1D728 */
  0x0000,   /* U+1D729 */
  0x0000,   /* U+1D72A */
  0x0000,   /* U+1D72B */
  0x0000,   /* U+1D72C */
  0x0000,   /* U+1D72D */
  0x0000,   /* U+1D72E */
  0x0000,   /* U+1D72F */
  0x0000,   /* U+1D730 */
  0x0000,   /* U+1D731 */
  0x0000,   /* U+1D732 */
  0x0000,   /* U+1D733 */
  0x0000,   /* U+1D734 */
  0x0000,   /* U+1D735 */
  0x0000,   /* U+1D736 */
  0x0000,   /* U+1D737 */
  0x0000,   /* U+1D738 */
  0x0000,   /* U+1D739 */
  0x0000,   /* U+1D73A */
  0x0000,   /* U+1D73B */
  0x0000,   /* U+1D73C */
  0x0000,   /* U+1D73D */
  0x0000,   /* U+1D73E */
  0x0000,   /* U+1D73F */
  0x0000,   /* U+1D740 */
  0x0000,   /* U+1D741 */
  0x0000,   /* U+1D742 */
  0x0000,   /* U+1D743 */
  0x0000,   /* U+1D744 */
  0x0000,   /* U+1D745 */
  0x0000,   /* U+1D746 */
  0x0000,   /* U+1D747 */
  0x0000,   /* U+1D748 */
  0x0000,   /* U+1D749 */
  0x0000,   /* U+1D74A */
  0x0000,   /* U+1D74B */
  0x0000,   /* U+1D74C */
  0x0000,   /* U+1D74D */
  0x0000,   /* U+1D74E */
  0x0000,   /* U+1D74F */
  0x0000,   /* U+1D750 */
  0x0000,   /* U+1D751 */
  0x0000,   /* U+1D752 */
  0x0000,   /* U+1D753 */
  0x0000,   /* U+1D754 */
  0x0000,   /* U+1D755 */
  0x0000,   /* U+1D756 */
  0x0000,   /* U+1D757 */
  0x0000,   /* U+1D758 */
  0x0000,   /* U+1D759 */
  0x0000,   /* U+1D75A */
  0x0000,   /* U+1D75B */
  0x0000,   /* U+1D75C */
  0x0000,   /* U+1D75D */
  0x0000,   /* U+1D75E */
  0x0000,   /* U+1D75F */
  0x0000,   /* U+1D760 */
  0x0000,   /* U+1D761 */
  0x0000,   /* U+1D762 */
  0x0000,   /* U+1D763 */
  0x0000,   /* U+1D764 */
  0x0000,   /* U+1D765 */
  0x0000,   /* U+1D766 */
  0x0000,   /* U+1D767 */
  0x0000,   /* U+1D768 */
  0x0000,   /* U+1D769 */
  0x0000,   /* U+1D76A */
  0x0000,   /* U+1D76B */
  0x0000,   /* U+1D76C */
  0x0000,   /* U+1D76D */
  0x0000,   /* U+1D76E */
  0x0000,   /* U+1D76F */
  0x0000,   /* U+1D770 */
  0x0000,   /* U+1D771 */
  0x0000,   /* U+1D772 */
  0x0000,   /* U+1D773 */
  0x0000,   /* U+1D774 */
  0x0000,   /* U+1D775 */
  0x0000,   /* U+1D776 */
  0x0000,   /* U+1D777 */
  0x0000,   /* U+1D778 */
  0x0000,   /* U+1D779 */
  0x0000,   /* U+1D77A */
  0x0000,   /* U+1D77B */
  0x0000,   /* U+1D77C */
  0x0000,   /* U+1D77D */
  0x0000,   /* U+1D77E */
  0x0000,   /* U+1D77F */
  0x0000,   /* U+1D780 */
  0x0000,   /* U+1D781 */
  0x0000,   /* U+1D782 */
  0x0000,   /* U+1D783 */
  0x0000,   /* U+1D784 */
  0x0000,   /* U+1D785 */
  0x0000,   /* U+1D786 */
  0x0000,   /* U+1D787 */
  0x0000,   /* U+1D788 */
  0x0000,   /* U+1D789 */
  0x0000,   /* U+1D78A */
  0x0000,   /* U+1D78B */
  0x0000,   /* U+1D78C */
  0x0000,   /* U+1D78D */
  0x0000,   /* U+1D78E */
  0x0000,   /* U+1D78F */
  0x0000,   /* U+1D790 */
  0x0000,   /* U+1D791 */
  0x0000,   /* U+1D792 */
  0x0000,   /* U+1D793 */
  0x0000,   /* U+1D794 */
  0x0000,   /* U+1D795 */
  0x0000,   /* U+1D796 */
  0x0000,   /* U+1D797 */
  0x0000,   /* U+1D798 */
  0x0000,   /* U+1D799 */
  0x0000,   /* U+1D79A */
  0x0000,   /* U+1D79B */
  0x0000,   /* U+1D79C */
  0x0000,   /* U+1D79D */
  0x0000,   /* U+1D79E */
  0x0000,   /* U+1D79F */
  0x0000,   /* U+1D7A0 */
  0x0000,   /* U+1D7A1 */
  0x0000,   /* U+1D7A2 */
  0x0000,   /* U+1D7A3 */
  0x0000,   /* U+1D7A4 */
  0x0000,   /* U+1D7A5 */
  0x0000,   /* U+1D7A6 */
  0x0000,   /* U+1D7A7 */
  0x0000,   /* U+1D7A8 */
  0x0000,   /* U+1D7A9 */
  0x0000,   /* U+1D7AA */
  0x0000,   /* U+1D7AB */
  0x0000,   /* U+1D7AC */
  0x0000,   /* U+1D7AD */
  0x0000,   /* U+1D7AE */
  0x0000,   /* U+1D7AF */
  0x0000,   /* U+1D7B0 */
  0x0000,   /* U+1D7B1 */
  0x0000,   /* U+1D7B2 */
  0x0000,   /* U+1D7B3 */
  0x0000,   /* U+1D7B4 */
  0x0000,   /* U+1D7B5 */
  0x0000,   /* U+1D7B6 */
  0x0000,   /* U+1D7B7 */
  0x0000,   /* U+1D7B8 */
  0x0000,   /* U+1D7B9 */
  0x0000,   /* U+1D7BA */
  0x0000,   /* U+1D7BB */
  0x0000,   /* U+1D7BC */
  0x0000,   /* U+1D7BD */
  0x0000,   /* U+1D7BE */
  0x0000,   /* U+1D7BF */
  0x0000,   /* U+1D7C0 */
  0x0000,   /* U+1D7C1 */
  0x0000,   /* U+1D7C2 */
  0x0000,   /* U+1D7C3 */
  0x0000,   /* U+1D7C4 */
  0x0000,   /* U+1D7C5 */
  0x0000,   /* U+1D7C6 */
  0x0000,   /* U+1D7C7 */
  0x0000,   /* U+1D7C8 */
  0x0000,   /* U+1D7C9 */
  0x0000,   /* U+1D7CA */
  0x0000,   /* U+1D7CB */
  0x0000,   /* U+1D7CC */
  0x0000,   /* U+1D7CD */
  0x0000,   /* U+1D7CE */
  0x0000,   /* U+1D7CF */
  0x0000,   /* U+1D7D0 */
  0x0000,   /* U+1D7D1 */
  0x0000,   /* U+1D7D2 */
  0x0000,   /* U+1D7D3 */
  0x0000,   /* U+1D7D4 */
  0x0000,   /* U+1D7D5 */
  0x0000,   /* U+1D7D6 */
  0x0000,   /* U+1D7D7 */
  0x0000,   /* U+1D7D8 */
  0x0000,   /* U+1D7D9 */
  0x0000,   /* U+1D7DA */
  0x0000,   /* U+1D7DB */
  0x0000,   /* U+1D7DC */
  0x0000,   /* U+1D7DD */
  0x0000,   /* U+1D7DE */
  0x0000,   /* U+1D7DF */
  0x0000,   /* U+1D7E0 */
  0x0000,   /* U+1D7E1 */
  0x0000,   /* U+1D7E2 */
  0x0000,   /* U+1D7E3 */
  0x0000,   /* U+1D7E4 */
  0x0000,   /* U+1D7E5 */
  0x0000,   /* U+1D7E6 */
  0x0000,   /* U+1D7E7 */
  0x0000,   /* U+1D7E8 */
  0x0000,   /* U+1D7E9 */
  0x0000,   /* U+1D7EA */
  0x0000,   /* U+1D7EB */
  0x0000,   /* U+1D7EC */
  0x0000,   /* U+1D7ED */
  0x0000,   /* U+1D7EE */
  0x0000,   /* U+1D7EF */
  0x0000,   /* U+1D7F0 */
  0x0000,   /* U+1D7F1 */
  0x0000,   /* U+1D7F2 */
  0x0000,   /* U+1D7F3 */
  0x0000,   /* U+1D7F4 */
  0x0000,   /* U+1D7F5 */
  0x0000,   /* U+1D7F6 */
  0x0000,   /* U+1D7F7 */
  0x0000,   /* U+1D7F8 */
  0x0000,   /* U+1D7F9 */
  0x0000,   /* U+1D7FA */
  0x0000,   /* U+1D7FB */
  0x0000,   /* U+1D7FC */
  0x0000,   /* U+1D7FD */
  0x0000,   /* U+1D7FE */
  0x0000,   /* U+1D7FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1D700 */
  0x0000,   /* U+1D701 */
  0x0000,   /* U+1D702 */
  0x0000,   /* U+1D703 */
  0x0000,   /* U+1D704 */
  0x0000,   /* U+1D705 */
  0x0000,   /* U+1D706 */
  0x0000,   /* U+1D707 */
  0x0000,   /* U+1D708 */
  0x0000,   /* U+1D709 */
  0x0000,   /* U+1D70A */
  0x0000,   /* U+1D70B */
  0x0000,   /* U+1D70C */
  0x0000,   /* U+1D70D */
  0x0000,   /* U+1D70E */
  0x0000,   /* U+1D70F */
  0x0000,   /* U+1D710 */
  0x0000,   /* U+1D711 */
  0x0000,   /* U+1D712 */
  0x0000,   /* U+1D713 */
  0x0000,   /* U+1D714 */
  0x0000,   /* U+1D715 */
  0x0000,   /* U+1D716 */
  0x0000,   /* U+1D717 */
  0x0000,   /* U+1D718 */
  0x0000,   /* U+1D719 */
  0x0000,   /* U+1D71A */
  0x0000,   /* U+1D71B */
  0x0000,   /* U+1D71C */
  0x0000,   /* U+1D71D */
  0x0000,   /* U+1D71E */
  0x0000,   /* U+1D71F */
  0x0000,   /* U+1D720 */
  0x0000,   /* U+1D721 */
  0x0000,   /* U+1D722 */
  0x0000,   /* U+1D723 */
  0x0000,   /* U+1D724 */
  0x0000,   /* U+1D725 */
  0x0000,   /* U+1D726 */
  0x0000,   /* U+1D727 */
  0x0000,   /* U+1D728 */
  0x0000,   /* U+1D729 */
  0x0000,   /* U+1D72A */
  0x0000,   /* U+1D72B */
  0x0000,   /* U+1D72C */
  0x0000,   /* U+1D72D */
  0x0000,   /* U+1D72E */
  0x0000,   /* U+1D72F */
  0x0000,   /* U+1D730 */
  0x0000,   /* U+1D731 */
  0x0000,   /* U+1D732 */
  0x0000,   /* U+1D733 */
  0x0000,   /* U+1D734 */
  0x0000,   /* U+1D735 */
  0x0000,   /* U+1D736 */
  0x0000,   /* U+1D737 */
  0x0000,   /* U+1D738 */
  0x0000,   /* U+1D739 */
  0x0000,   /* U+1D73A */
  0x0000,   /* U+1D73B */
  0x0000,   /* U+1D73C */
  0x0000,   /* U+1D73D */
  0x0000,   /* U+1D73E */
  0x0000,   /* U+1D73F */
  0x0000,   /* U+1D740 */
  0x0000,   /* U+1D741 */
  0x0000,   /* U+1D742 */
  0x0000,   /* U+1D743 */
  0x0000,   /* U+1D744 */
  0x0000,   /* U+1D745 */
  0x0000,   /* U+1D746 */
  0x0000,   /* U+1D747 */
  0x0000,   /* U+1D748 */
  0x0000,   /* U+1D749 */
  0x0000,   /* U+1D74A */
  0x0000,   /* U+1D74B */
  0x0000,   /* U+1D74C */
  0x0000,   /* U+1D74D */
  0x0000,   /* U+1D74E */
  0x0000,   /* U+1D74F */
  0x0000,   /* U+1D750 */
  0x0000,   /* U+1D751 */
  0x0000,   /* U+1D752 */
  0x0000,   /* U+1D753 */
  0x0000,   /* U+1D754 */
  0x0000,   /* U+1D755 */
  0x0000,   /* U+1D756 */
  0x0000,   /* U+1D757 */
  0x0000,   /* U+1D758 */
  0x0000,   /* U+1D759 */
  0x0000,   /* U+1D75A */
  0x0000,   /* U+1D75B */
  0x0000,   /* U+1D75C */
  0x0000,   /* U+1D75D */
  0x0000,   /* U+1D75E */
  0x0000,   /* U+1D75F */
  0x0000,   /* U+1D760 */
  0x0000,   /* U+1D761 */
  0x0000,   /* U+1D762 */
  0x0000,   /* U+1D763 */
  0x0000,   /* U+1D764 */
  0x0000,   /* U+1D765 */
  0x0000,   /* U+1D766 */
  0x0000,   /* U+1D767 */
  0x0000,   /* U+1D768 */
  0x0000,   /* U+1D769 */
  0x0000,   /* U+1D76A */
  0x0000,   /* U+1D76B */
  0x0000,   /* U+1D76C */
  0x0000,   /* U+1D76D */
  0x0000,   /* U+1D76E */
  0x0000,   /* U+1D76F */
  0x0000,   /* U+1D770 */
  0x0000,   /* U+1D771 */
  0x0000,   /* U+1D772 */
  0x0000,   /* U+1D773 */
  0x0000,   /* U+1D774 */
  0x0000,   /* U+1D775 */
  0x0000,   /* U+1D776 */
  0x0000,   /* U+1D777 */
  0x0000,   /* U+1D778 */
  0x0000,   /* U+1D779 */
  0x0000,   /* U+1D77A */
  0x0000,   /* U+1D77B */
  0x0000,   /* U+1D77C */
  0x0000,   /* U+1D77D */
  0x0000,   /* U+1D77E */
  0x0000,   /* U+1D77F */
  0x0000,   /* U+1D780 */
  0x0000,   /* U+1D781 */
  0x0000,   /* U+1D782 */
  0x0000,   /* U+1D783 */
  0x0000,   /* U+1D784 */
  0x0000,   /* U+1D785 */
  0x0000,   /* U+1D786 */
  0x0000,   /* U+1D787 */
  0x0000,   /* U+1D788 */
  0x0000,   /* U+1D789 */
  0x0000,   /* U+1D78A */
  0x0000,   /* U+1D78B */
  0x0000,   /* U+1D78C */
  0x0000,   /* U+1D78D */
  0x0000,   /* U+1D78E */
  0x0000,   /* U+1D78F */
  0x0000,   /* U+1D790 */
  0x0000,   /* U+1D791 */
  0x0000,   /* U+1D792 */
  0x0000,   /* U+1D793 */
  0x0000,   /* U+1D794 */
  0x0000,   /* U+1D795 */
  0x0000,   /* U+1D796 */
  0x0000,   /* U+1D797 */
  0x0000,   /* U+1D798 */
  0x0000,   /* U+1D799 */
  0x0000,   /* U+1D79A */
  0x0000,   /* U+1D79B */
  0x0000,   /* U+1D79C */
  0x0000,   /* U+1D79D */
  0x0000,   /* U+1D79E */
  0x0000,   /* U+1D79F */
  0x0000,   /* U+1D7A0 */
  0x0000,   /* U+1D7A1 */
  0x0000,   /* U+1D7A2 */
  0x0000,   /* U+1D7A3 */
  0x0000,   /* U+1D7A4 */
  0x0000,   /* U+1D7A5 */
  0x0000,   /* U+1D7A6 */
  0x0000,   /* U+1D7A7 */
  0x0000,   /* U+1D7A8 */
  0x0000,   /* U+1D7A9 */
  0x0000,   /* U+1D7AA */
  0x0000,   /* U+1D7AB */
  0x0000,   /* U+1D7AC */
  0x0000,   /* U+1D7AD */
  0x0000,   /* U+1D7AE */
  0x0000,   /* U+1D7AF */
  0x0000,   /* U+1D7B0 */
  0x0000,   /* U+1D7B1 */
  0x0000,   /* U+1D7B2 */
  0x0000,   /* U+1D7B3 */
  0x0000,   /* U+1D7B4 */
  0x0000,   /* U+1D7B5 */
  0x0000,   /* U+1D7B6 */
  0x0000,   /* U+1D7B7 */
  0x0000,   /* U+1D7B8 */
  0x0000,   /* U+1D7B9 */
  0x0000,   /* U+1D7BA */
  0x0000,   /* U+1D7BB */
  0x0000,   /* U+1D7BC */
  0x0000,   /* U+1D7BD */
  0x0000,   /* U+1D7BE */
  0x0000,   /* U+1D7BF */
  0x0000,   /* U+1D7C0 */
  0x0000,   /* U+1D7C1 */
  0x0000,   /* U+1D7C2 */
  0x0000,   /* U+1D7C3 */
  0x0000,   /* U+1D7C4 */
  0x0000,   /* U+1D7C5 */
  0x0000,   /* U+1D7C6 */
  0x0000,   /* U+1D7C7 */
  0x0000,   /* U+1D7C8 */
  0x0000,   /* U+1D7C9 */
  0x0000,   /* U+1D7CA */
  0x0000,   /* U+1D7CB */
  0x0000,   /* U+1D7CC */
  0x0000,   /* U+1D7CD */
  0x0000,   /* U+1D7CE */
  0x0000,   /* U+1D7CF */
  0x0000,   /* U+1D7D0 */
  0x0000,   /* U+1D7D1 */
  0x0000,   /* U+1D7D2 */
  0x0000,   /* U+1D7D3 */
  0x0000,   /* U+1D7D4 */
  0x0000,   /* U+1D7D5 */
  0x0000,   /* U+1D7D6 */
  0x0000,   /* U+1D7D7 */
  0x0000,   /* U+1D7D8 */
  0x0000,   /* U+1D7D9 */
  0x0000,   /* U+1D7DA */
  0x0000,   /* U+1D7DB */
  0x0000,   /* U+1D7DC */
  0x0000,   /* U+1D7DD */
  0x0000,   /* U+1D7DE */
  0x0000,   /* U+1D7DF */
  0x0000,   /* U+1D7E0 */
  0x0000,   /* U+1D7E1 */
  0x0000,   /* U+1D7E2 */
  0x0000,   /* U+1D7E3 */
  0x0000,   /* U+1D7E4 */
  0x0000,   /* U+1D7E5 */
  0x0000,   /* U+1D7E6 */
  0x0000,   /* U+1D7E7 */
  0x0000,   /* U+1D7E8 */
  0x0000,   /* U+1D7E9 */
  0x0000,   /* U+1D7EA */
  0x0000,   /* U+1D7EB */
  0x0000,   /* U+1D7EC */
  0x0000,   /* U+1D7ED */
  0x0000,   /* U+1D7EE */
  0x0000,   /* U+1D7EF */
  0x0000,   /* U+1D7F0 */
  0x0000,   /* U+1D7F1 */
  0x0000,   /* U+1D7F2 */
  0x0000,   /* U+1D7F3 */
  0x0000,   /* U+1D7F4 */
  0x0000,   /* U+1D7F5 */
  0x0000,   /* U+1D7F6 */
  0x0000,   /* U+1D7F7 */
  0x0000,   /* U+1D7F8 */
  0x0000,   /* U+1D7F9 */
  0x0000,   /* U+1D7FA */
  0x0000,   /* U+1D7FB */
  0x0000,   /* U+1D7FC */
  0x0000,   /* U+1D7FD */
  0x0000,   /* U+1D7FE */
  0x0000,   /* U+1D7FF */
};

uint16 uca900_p1D8[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x183F,   /* U+1D800 */
  0x1840,   /* U+1D801 */
  0x1841,   /* U+1D802 */
  0x1842,   /* U+1D803 */
  0x1843,   /* U+1D804 */
  0x1844,   /* U+1D805 */
  0x1845,   /* U+1D806 */
  0x1846,   /* U+1D807 */
  0x1847,   /* U+1D808 */
  0x1848,   /* U+1D809 */
  0x1849,   /* U+1D80A */
  0x184A,   /* U+1D80B */
  0x184B,   /* U+1D80C */
  0x184C,   /* U+1D80D */
  0x184D,   /* U+1D80E */
  0x184E,   /* U+1D80F */
  0x184F,   /* U+1D810 */
  0x1850,   /* U+1D811 */
  0x1851,   /* U+1D812 */
  0x1852,   /* U+1D813 */
  0x1853,   /* U+1D814 */
  0x1854,   /* U+1D815 */
  0x1855,   /* U+1D816 */
  0x1856,   /* U+1D817 */
  0x1857,   /* U+1D818 */
  0x1858,   /* U+1D819 */
  0x1859,   /* U+1D81A */
  0x185A,   /* U+1D81B */
  0x185B,   /* U+1D81C */
  0x185C,   /* U+1D81D */
  0x185D,   /* U+1D81E */
  0x185E,   /* U+1D81F */
  0x185F,   /* U+1D820 */
  0x1860,   /* U+1D821 */
  0x1861,   /* U+1D822 */
  0x1862,   /* U+1D823 */
  0x1863,   /* U+1D824 */
  0x1864,   /* U+1D825 */
  0x1865,   /* U+1D826 */
  0x1866,   /* U+1D827 */
  0x1867,   /* U+1D828 */
  0x1868,   /* U+1D829 */
  0x1869,   /* U+1D82A */
  0x186A,   /* U+1D82B */
  0x186B,   /* U+1D82C */
  0x186C,   /* U+1D82D */
  0x186D,   /* U+1D82E */
  0x186E,   /* U+1D82F */
  0x186F,   /* U+1D830 */
  0x1870,   /* U+1D831 */
  0x1871,   /* U+1D832 */
  0x1872,   /* U+1D833 */
  0x1873,   /* U+1D834 */
  0x1874,   /* U+1D835 */
  0x1875,   /* U+1D836 */
  0x1876,   /* U+1D837 */
  0x1877,   /* U+1D838 */
  0x1878,   /* U+1D839 */
  0x1879,   /* U+1D83A */
  0x187A,   /* U+1D83B */
  0x187B,   /* U+1D83C */
  0x187C,   /* U+1D83D */
  0x187D,   /* U+1D83E */
  0x187E,   /* U+1D83F */
  0x187F,   /* U+1D840 */
  0x1880,   /* U+1D841 */
  0x1881,   /* U+1D842 */
  0x1882,   /* U+1D843 */
  0x1883,   /* U+1D844 */
  0x1884,   /* U+1D845 */
  0x1885,   /* U+1D846 */
  0x1886,   /* U+1D847 */
  0x1887,   /* U+1D848 */
  0x1888,   /* U+1D849 */
  0x1889,   /* U+1D84A */
  0x188A,   /* U+1D84B */
  0x188B,   /* U+1D84C */
  0x188C,   /* U+1D84D */
  0x188D,   /* U+1D84E */
  0x188E,   /* U+1D84F */
  0x188F,   /* U+1D850 */
  0x1890,   /* U+1D851 */
  0x1891,   /* U+1D852 */
  0x1892,   /* U+1D853 */
  0x1893,   /* U+1D854 */
  0x1894,   /* U+1D855 */
  0x1895,   /* U+1D856 */
  0x1896,   /* U+1D857 */
  0x1897,   /* U+1D858 */
  0x1898,   /* U+1D859 */
  0x1899,   /* U+1D85A */
  0x189A,   /* U+1D85B */
  0x189B,   /* U+1D85C */
  0x189C,   /* U+1D85D */
  0x189D,   /* U+1D85E */
  0x189E,   /* U+1D85F */
  0x189F,   /* U+1D860 */
  0x18A0,   /* U+1D861 */
  0x18A1,   /* U+1D862 */
  0x18A2,   /* U+1D863 */
  0x18A3,   /* U+1D864 */
  0x18A4,   /* U+1D865 */
  0x18A5,   /* U+1D866 */
  0x18A6,   /* U+1D867 */
  0x18A7,   /* U+1D868 */
  0x18A8,   /* U+1D869 */
  0x18A9,   /* U+1D86A */
  0x18AA,   /* U+1D86B */
  0x18AB,   /* U+1D86C */
  0x18AC,   /* U+1D86D */
  0x18AD,   /* U+1D86E */
  0x18AE,   /* U+1D86F */
  0x18AF,   /* U+1D870 */
  0x18B0,   /* U+1D871 */
  0x18B1,   /* U+1D872 */
  0x18B2,   /* U+1D873 */
  0x18B3,   /* U+1D874 */
  0x18B4,   /* U+1D875 */
  0x18B5,   /* U+1D876 */
  0x18B6,   /* U+1D877 */
  0x18B7,   /* U+1D878 */
  0x18B8,   /* U+1D879 */
  0x18B9,   /* U+1D87A */
  0x18BA,   /* U+1D87B */
  0x18BB,   /* U+1D87C */
  0x18BC,   /* U+1D87D */
  0x18BD,   /* U+1D87E */
  0x18BE,   /* U+1D87F */
  0x18BF,   /* U+1D880 */
  0x18C0,   /* U+1D881 */
  0x18C1,   /* U+1D882 */
  0x18C2,   /* U+1D883 */
  0x18C3,   /* U+1D884 */
  0x18C4,   /* U+1D885 */
  0x18C5,   /* U+1D886 */
  0x18C6,   /* U+1D887 */
  0x18C7,   /* U+1D888 */
  0x18C8,   /* U+1D889 */
  0x18C9,   /* U+1D88A */
  0x18CA,   /* U+1D88B */
  0x18CB,   /* U+1D88C */
  0x18CC,   /* U+1D88D */
  0x18CD,   /* U+1D88E */
  0x18CE,   /* U+1D88F */
  0x18CF,   /* U+1D890 */
  0x18D0,   /* U+1D891 */
  0x18D1,   /* U+1D892 */
  0x18D2,   /* U+1D893 */
  0x18D3,   /* U+1D894 */
  0x18D4,   /* U+1D895 */
  0x18D5,   /* U+1D896 */
  0x18D6,   /* U+1D897 */
  0x18D7,   /* U+1D898 */
  0x18D8,   /* U+1D899 */
  0x18D9,   /* U+1D89A */
  0x18DA,   /* U+1D89B */
  0x18DB,   /* U+1D89C */
  0x18DC,   /* U+1D89D */
  0x18DD,   /* U+1D89E */
  0x18DE,   /* U+1D89F */
  0x18DF,   /* U+1D8A0 */
  0x18E0,   /* U+1D8A1 */
  0x18E1,   /* U+1D8A2 */
  0x18E2,   /* U+1D8A3 */
  0x18E3,   /* U+1D8A4 */
  0x18E4,   /* U+1D8A5 */
  0x18E5,   /* U+1D8A6 */
  0x18E6,   /* U+1D8A7 */
  0x18E7,   /* U+1D8A8 */
  0x18E8,   /* U+1D8A9 */
  0x18E9,   /* U+1D8AA */
  0x18EA,   /* U+1D8AB */
  0x18EB,   /* U+1D8AC */
  0x18EC,   /* U+1D8AD */
  0x18ED,   /* U+1D8AE */
  0x18EE,   /* U+1D8AF */
  0x18EF,   /* U+1D8B0 */
  0x18F0,   /* U+1D8B1 */
  0x18F1,   /* U+1D8B2 */
  0x18F2,   /* U+1D8B3 */
  0x18F3,   /* U+1D8B4 */
  0x18F4,   /* U+1D8B5 */
  0x18F5,   /* U+1D8B6 */
  0x18F6,   /* U+1D8B7 */
  0x18F7,   /* U+1D8B8 */
  0x18F8,   /* U+1D8B9 */
  0x18F9,   /* U+1D8BA */
  0x18FA,   /* U+1D8BB */
  0x18FB,   /* U+1D8BC */
  0x18FC,   /* U+1D8BD */
  0x18FD,   /* U+1D8BE */
  0x18FE,   /* U+1D8BF */
  0x18FF,   /* U+1D8C0 */
  0x1900,   /* U+1D8C1 */
  0x1901,   /* U+1D8C2 */
  0x1902,   /* U+1D8C3 */
  0x1903,   /* U+1D8C4 */
  0x1904,   /* U+1D8C5 */
  0x1905,   /* U+1D8C6 */
  0x1906,   /* U+1D8C7 */
  0x1907,   /* U+1D8C8 */
  0x1908,   /* U+1D8C9 */
  0x1909,   /* U+1D8CA */
  0x190A,   /* U+1D8CB */
  0x190B,   /* U+1D8CC */
  0x190C,   /* U+1D8CD */
  0x190D,   /* U+1D8CE */
  0x190E,   /* U+1D8CF */
  0x190F,   /* U+1D8D0 */
  0x1910,   /* U+1D8D1 */
  0x1911,   /* U+1D8D2 */
  0x1912,   /* U+1D8D3 */
  0x1913,   /* U+1D8D4 */
  0x1914,   /* U+1D8D5 */
  0x1915,   /* U+1D8D6 */
  0x1916,   /* U+1D8D7 */
  0x1917,   /* U+1D8D8 */
  0x1918,   /* U+1D8D9 */
  0x1919,   /* U+1D8DA */
  0x191A,   /* U+1D8DB */
  0x191B,   /* U+1D8DC */
  0x191C,   /* U+1D8DD */
  0x191D,   /* U+1D8DE */
  0x191E,   /* U+1D8DF */
  0x191F,   /* U+1D8E0 */
  0x1920,   /* U+1D8E1 */
  0x1921,   /* U+1D8E2 */
  0x1922,   /* U+1D8E3 */
  0x1923,   /* U+1D8E4 */
  0x1924,   /* U+1D8E5 */
  0x1925,   /* U+1D8E6 */
  0x1926,   /* U+1D8E7 */
  0x1927,   /* U+1D8E8 */
  0x1928,   /* U+1D8E9 */
  0x1929,   /* U+1D8EA */
  0x192A,   /* U+1D8EB */
  0x192B,   /* U+1D8EC */
  0x192C,   /* U+1D8ED */
  0x192D,   /* U+1D8EE */
  0x192E,   /* U+1D8EF */
  0x192F,   /* U+1D8F0 */
  0x1930,   /* U+1D8F1 */
  0x1931,   /* U+1D8F2 */
  0x1932,   /* U+1D8F3 */
  0x1933,   /* U+1D8F4 */
  0x1934,   /* U+1D8F5 */
  0x1935,   /* U+1D8F6 */
  0x1936,   /* U+1D8F7 */
  0x1937,   /* U+1D8F8 */
  0x1938,   /* U+1D8F9 */
  0x1939,   /* U+1D8FA */
  0x193A,   /* U+1D8FB */
  0x193B,   /* U+1D8FC */
  0x193C,   /* U+1D8FD */
  0x193D,   /* U+1D8FE */
  0x193E,   /* U+1D8FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D800 */
  0x0020,   /* U+1D801 */
  0x0020,   /* U+1D802 */
  0x0020,   /* U+1D803 */
  0x0020,   /* U+1D804 */
  0x0020,   /* U+1D805 */
  0x0020,   /* U+1D806 */
  0x0020,   /* U+1D807 */
  0x0020,   /* U+1D808 */
  0x0020,   /* U+1D809 */
  0x0020,   /* U+1D80A */
  0x0020,   /* U+1D80B */
  0x0020,   /* U+1D80C */
  0x0020,   /* U+1D80D */
  0x0020,   /* U+1D80E */
  0x0020,   /* U+1D80F */
  0x0020,   /* U+1D810 */
  0x0020,   /* U+1D811 */
  0x0020,   /* U+1D812 */
  0x0020,   /* U+1D813 */
  0x0020,   /* U+1D814 */
  0x0020,   /* U+1D815 */
  0x0020,   /* U+1D816 */
  0x0020,   /* U+1D817 */
  0x0020,   /* U+1D818 */
  0x0020,   /* U+1D819 */
  0x0020,   /* U+1D81A */
  0x0020,   /* U+1D81B */
  0x0020,   /* U+1D81C */
  0x0020,   /* U+1D81D */
  0x0020,   /* U+1D81E */
  0x0020,   /* U+1D81F */
  0x0020,   /* U+1D820 */
  0x0020,   /* U+1D821 */
  0x0020,   /* U+1D822 */
  0x0020,   /* U+1D823 */
  0x0020,   /* U+1D824 */
  0x0020,   /* U+1D825 */
  0x0020,   /* U+1D826 */
  0x0020,   /* U+1D827 */
  0x0020,   /* U+1D828 */
  0x0020,   /* U+1D829 */
  0x0020,   /* U+1D82A */
  0x0020,   /* U+1D82B */
  0x0020,   /* U+1D82C */
  0x0020,   /* U+1D82D */
  0x0020,   /* U+1D82E */
  0x0020,   /* U+1D82F */
  0x0020,   /* U+1D830 */
  0x0020,   /* U+1D831 */
  0x0020,   /* U+1D832 */
  0x0020,   /* U+1D833 */
  0x0020,   /* U+1D834 */
  0x0020,   /* U+1D835 */
  0x0020,   /* U+1D836 */
  0x0020,   /* U+1D837 */
  0x0020,   /* U+1D838 */
  0x0020,   /* U+1D839 */
  0x0020,   /* U+1D83A */
  0x0020,   /* U+1D83B */
  0x0020,   /* U+1D83C */
  0x0020,   /* U+1D83D */
  0x0020,   /* U+1D83E */
  0x0020,   /* U+1D83F */
  0x0020,   /* U+1D840 */
  0x0020,   /* U+1D841 */
  0x0020,   /* U+1D842 */
  0x0020,   /* U+1D843 */
  0x0020,   /* U+1D844 */
  0x0020,   /* U+1D845 */
  0x0020,   /* U+1D846 */
  0x0020,   /* U+1D847 */
  0x0020,   /* U+1D848 */
  0x0020,   /* U+1D849 */
  0x0020,   /* U+1D84A */
  0x0020,   /* U+1D84B */
  0x0020,   /* U+1D84C */
  0x0020,   /* U+1D84D */
  0x0020,   /* U+1D84E */
  0x0020,   /* U+1D84F */
  0x0020,   /* U+1D850 */
  0x0020,   /* U+1D851 */
  0x0020,   /* U+1D852 */
  0x0020,   /* U+1D853 */
  0x0020,   /* U+1D854 */
  0x0020,   /* U+1D855 */
  0x0020,   /* U+1D856 */
  0x0020,   /* U+1D857 */
  0x0020,   /* U+1D858 */
  0x0020,   /* U+1D859 */
  0x0020,   /* U+1D85A */
  0x0020,   /* U+1D85B */
  0x0020,   /* U+1D85C */
  0x0020,   /* U+1D85D */
  0x0020,   /* U+1D85E */
  0x0020,   /* U+1D85F */
  0x0020,   /* U+1D860 */
  0x0020,   /* U+1D861 */
  0x0020,   /* U+1D862 */
  0x0020,   /* U+1D863 */
  0x0020,   /* U+1D864 */
  0x0020,   /* U+1D865 */
  0x0020,   /* U+1D866 */
  0x0020,   /* U+1D867 */
  0x0020,   /* U+1D868 */
  0x0020,   /* U+1D869 */
  0x0020,   /* U+1D86A */
  0x0020,   /* U+1D86B */
  0x0020,   /* U+1D86C */
  0x0020,   /* U+1D86D */
  0x0020,   /* U+1D86E */
  0x0020,   /* U+1D86F */
  0x0020,   /* U+1D870 */
  0x0020,   /* U+1D871 */
  0x0020,   /* U+1D872 */
  0x0020,   /* U+1D873 */
  0x0020,   /* U+1D874 */
  0x0020,   /* U+1D875 */
  0x0020,   /* U+1D876 */
  0x0020,   /* U+1D877 */
  0x0020,   /* U+1D878 */
  0x0020,   /* U+1D879 */
  0x0020,   /* U+1D87A */
  0x0020,   /* U+1D87B */
  0x0020,   /* U+1D87C */
  0x0020,   /* U+1D87D */
  0x0020,   /* U+1D87E */
  0x0020,   /* U+1D87F */
  0x0020,   /* U+1D880 */
  0x0020,   /* U+1D881 */
  0x0020,   /* U+1D882 */
  0x0020,   /* U+1D883 */
  0x0020,   /* U+1D884 */
  0x0020,   /* U+1D885 */
  0x0020,   /* U+1D886 */
  0x0020,   /* U+1D887 */
  0x0020,   /* U+1D888 */
  0x0020,   /* U+1D889 */
  0x0020,   /* U+1D88A */
  0x0020,   /* U+1D88B */
  0x0020,   /* U+1D88C */
  0x0020,   /* U+1D88D */
  0x0020,   /* U+1D88E */
  0x0020,   /* U+1D88F */
  0x0020,   /* U+1D890 */
  0x0020,   /* U+1D891 */
  0x0020,   /* U+1D892 */
  0x0020,   /* U+1D893 */
  0x0020,   /* U+1D894 */
  0x0020,   /* U+1D895 */
  0x0020,   /* U+1D896 */
  0x0020,   /* U+1D897 */
  0x0020,   /* U+1D898 */
  0x0020,   /* U+1D899 */
  0x0020,   /* U+1D89A */
  0x0020,   /* U+1D89B */
  0x0020,   /* U+1D89C */
  0x0020,   /* U+1D89D */
  0x0020,   /* U+1D89E */
  0x0020,   /* U+1D89F */
  0x0020,   /* U+1D8A0 */
  0x0020,   /* U+1D8A1 */
  0x0020,   /* U+1D8A2 */
  0x0020,   /* U+1D8A3 */
  0x0020,   /* U+1D8A4 */
  0x0020,   /* U+1D8A5 */
  0x0020,   /* U+1D8A6 */
  0x0020,   /* U+1D8A7 */
  0x0020,   /* U+1D8A8 */
  0x0020,   /* U+1D8A9 */
  0x0020,   /* U+1D8AA */
  0x0020,   /* U+1D8AB */
  0x0020,   /* U+1D8AC */
  0x0020,   /* U+1D8AD */
  0x0020,   /* U+1D8AE */
  0x0020,   /* U+1D8AF */
  0x0020,   /* U+1D8B0 */
  0x0020,   /* U+1D8B1 */
  0x0020,   /* U+1D8B2 */
  0x0020,   /* U+1D8B3 */
  0x0020,   /* U+1D8B4 */
  0x0020,   /* U+1D8B5 */
  0x0020,   /* U+1D8B6 */
  0x0020,   /* U+1D8B7 */
  0x0020,   /* U+1D8B8 */
  0x0020,   /* U+1D8B9 */
  0x0020,   /* U+1D8BA */
  0x0020,   /* U+1D8BB */
  0x0020,   /* U+1D8BC */
  0x0020,   /* U+1D8BD */
  0x0020,   /* U+1D8BE */
  0x0020,   /* U+1D8BF */
  0x0020,   /* U+1D8C0 */
  0x0020,   /* U+1D8C1 */
  0x0020,   /* U+1D8C2 */
  0x0020,   /* U+1D8C3 */
  0x0020,   /* U+1D8C4 */
  0x0020,   /* U+1D8C5 */
  0x0020,   /* U+1D8C6 */
  0x0020,   /* U+1D8C7 */
  0x0020,   /* U+1D8C8 */
  0x0020,   /* U+1D8C9 */
  0x0020,   /* U+1D8CA */
  0x0020,   /* U+1D8CB */
  0x0020,   /* U+1D8CC */
  0x0020,   /* U+1D8CD */
  0x0020,   /* U+1D8CE */
  0x0020,   /* U+1D8CF */
  0x0020,   /* U+1D8D0 */
  0x0020,   /* U+1D8D1 */
  0x0020,   /* U+1D8D2 */
  0x0020,   /* U+1D8D3 */
  0x0020,   /* U+1D8D4 */
  0x0020,   /* U+1D8D5 */
  0x0020,   /* U+1D8D6 */
  0x0020,   /* U+1D8D7 */
  0x0020,   /* U+1D8D8 */
  0x0020,   /* U+1D8D9 */
  0x0020,   /* U+1D8DA */
  0x0020,   /* U+1D8DB */
  0x0020,   /* U+1D8DC */
  0x0020,   /* U+1D8DD */
  0x0020,   /* U+1D8DE */
  0x0020,   /* U+1D8DF */
  0x0020,   /* U+1D8E0 */
  0x0020,   /* U+1D8E1 */
  0x0020,   /* U+1D8E2 */
  0x0020,   /* U+1D8E3 */
  0x0020,   /* U+1D8E4 */
  0x0020,   /* U+1D8E5 */
  0x0020,   /* U+1D8E6 */
  0x0020,   /* U+1D8E7 */
  0x0020,   /* U+1D8E8 */
  0x0020,   /* U+1D8E9 */
  0x0020,   /* U+1D8EA */
  0x0020,   /* U+1D8EB */
  0x0020,   /* U+1D8EC */
  0x0020,   /* U+1D8ED */
  0x0020,   /* U+1D8EE */
  0x0020,   /* U+1D8EF */
  0x0020,   /* U+1D8F0 */
  0x0020,   /* U+1D8F1 */
  0x0020,   /* U+1D8F2 */
  0x0020,   /* U+1D8F3 */
  0x0020,   /* U+1D8F4 */
  0x0020,   /* U+1D8F5 */
  0x0020,   /* U+1D8F6 */
  0x0020,   /* U+1D8F7 */
  0x0020,   /* U+1D8F8 */
  0x0020,   /* U+1D8F9 */
  0x0020,   /* U+1D8FA */
  0x0020,   /* U+1D8FB */
  0x0020,   /* U+1D8FC */
  0x0020,   /* U+1D8FD */
  0x0020,   /* U+1D8FE */
  0x0020,   /* U+1D8FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D800 */
  0x0002,   /* U+1D801 */
  0x0002,   /* U+1D802 */
  0x0002,   /* U+1D803 */
  0x0002,   /* U+1D804 */
  0x0002,   /* U+1D805 */
  0x0002,   /* U+1D806 */
  0x0002,   /* U+1D807 */
  0x0002,   /* U+1D808 */
  0x0002,   /* U+1D809 */
  0x0002,   /* U+1D80A */
  0x0002,   /* U+1D80B */
  0x0002,   /* U+1D80C */
  0x0002,   /* U+1D80D */
  0x0002,   /* U+1D80E */
  0x0002,   /* U+1D80F */
  0x0002,   /* U+1D810 */
  0x0002,   /* U+1D811 */
  0x0002,   /* U+1D812 */
  0x0002,   /* U+1D813 */
  0x0002,   /* U+1D814 */
  0x0002,   /* U+1D815 */
  0x0002,   /* U+1D816 */
  0x0002,   /* U+1D817 */
  0x0002,   /* U+1D818 */
  0x0002,   /* U+1D819 */
  0x0002,   /* U+1D81A */
  0x0002,   /* U+1D81B */
  0x0002,   /* U+1D81C */
  0x0002,   /* U+1D81D */
  0x0002,   /* U+1D81E */
  0x0002,   /* U+1D81F */
  0x0002,   /* U+1D820 */
  0x0002,   /* U+1D821 */
  0x0002,   /* U+1D822 */
  0x0002,   /* U+1D823 */
  0x0002,   /* U+1D824 */
  0x0002,   /* U+1D825 */
  0x0002,   /* U+1D826 */
  0x0002,   /* U+1D827 */
  0x0002,   /* U+1D828 */
  0x0002,   /* U+1D829 */
  0x0002,   /* U+1D82A */
  0x0002,   /* U+1D82B */
  0x0002,   /* U+1D82C */
  0x0002,   /* U+1D82D */
  0x0002,   /* U+1D82E */
  0x0002,   /* U+1D82F */
  0x0002,   /* U+1D830 */
  0x0002,   /* U+1D831 */
  0x0002,   /* U+1D832 */
  0x0002,   /* U+1D833 */
  0x0002,   /* U+1D834 */
  0x0002,   /* U+1D835 */
  0x0002,   /* U+1D836 */
  0x0002,   /* U+1D837 */
  0x0002,   /* U+1D838 */
  0x0002,   /* U+1D839 */
  0x0002,   /* U+1D83A */
  0x0002,   /* U+1D83B */
  0x0002,   /* U+1D83C */
  0x0002,   /* U+1D83D */
  0x0002,   /* U+1D83E */
  0x0002,   /* U+1D83F */
  0x0002,   /* U+1D840 */
  0x0002,   /* U+1D841 */
  0x0002,   /* U+1D842 */
  0x0002,   /* U+1D843 */
  0x0002,   /* U+1D844 */
  0x0002,   /* U+1D845 */
  0x0002,   /* U+1D846 */
  0x0002,   /* U+1D847 */
  0x0002,   /* U+1D848 */
  0x0002,   /* U+1D849 */
  0x0002,   /* U+1D84A */
  0x0002,   /* U+1D84B */
  0x0002,   /* U+1D84C */
  0x0002,   /* U+1D84D */
  0x0002,   /* U+1D84E */
  0x0002,   /* U+1D84F */
  0x0002,   /* U+1D850 */
  0x0002,   /* U+1D851 */
  0x0002,   /* U+1D852 */
  0x0002,   /* U+1D853 */
  0x0002,   /* U+1D854 */
  0x0002,   /* U+1D855 */
  0x0002,   /* U+1D856 */
  0x0002,   /* U+1D857 */
  0x0002,   /* U+1D858 */
  0x0002,   /* U+1D859 */
  0x0002,   /* U+1D85A */
  0x0002,   /* U+1D85B */
  0x0002,   /* U+1D85C */
  0x0002,   /* U+1D85D */
  0x0002,   /* U+1D85E */
  0x0002,   /* U+1D85F */
  0x0002,   /* U+1D860 */
  0x0002,   /* U+1D861 */
  0x0002,   /* U+1D862 */
  0x0002,   /* U+1D863 */
  0x0002,   /* U+1D864 */
  0x0002,   /* U+1D865 */
  0x0002,   /* U+1D866 */
  0x0002,   /* U+1D867 */
  0x0002,   /* U+1D868 */
  0x0002,   /* U+1D869 */
  0x0002,   /* U+1D86A */
  0x0002,   /* U+1D86B */
  0x0002,   /* U+1D86C */
  0x0002,   /* U+1D86D */
  0x0002,   /* U+1D86E */
  0x0002,   /* U+1D86F */
  0x0002,   /* U+1D870 */
  0x0002,   /* U+1D871 */
  0x0002,   /* U+1D872 */
  0x0002,   /* U+1D873 */
  0x0002,   /* U+1D874 */
  0x0002,   /* U+1D875 */
  0x0002,   /* U+1D876 */
  0x0002,   /* U+1D877 */
  0x0002,   /* U+1D878 */
  0x0002,   /* U+1D879 */
  0x0002,   /* U+1D87A */
  0x0002,   /* U+1D87B */
  0x0002,   /* U+1D87C */
  0x0002,   /* U+1D87D */
  0x0002,   /* U+1D87E */
  0x0002,   /* U+1D87F */
  0x0002,   /* U+1D880 */
  0x0002,   /* U+1D881 */
  0x0002,   /* U+1D882 */
  0x0002,   /* U+1D883 */
  0x0002,   /* U+1D884 */
  0x0002,   /* U+1D885 */
  0x0002,   /* U+1D886 */
  0x0002,   /* U+1D887 */
  0x0002,   /* U+1D888 */
  0x0002,   /* U+1D889 */
  0x0002,   /* U+1D88A */
  0x0002,   /* U+1D88B */
  0x0002,   /* U+1D88C */
  0x0002,   /* U+1D88D */
  0x0002,   /* U+1D88E */
  0x0002,   /* U+1D88F */
  0x0002,   /* U+1D890 */
  0x0002,   /* U+1D891 */
  0x0002,   /* U+1D892 */
  0x0002,   /* U+1D893 */
  0x0002,   /* U+1D894 */
  0x0002,   /* U+1D895 */
  0x0002,   /* U+1D896 */
  0x0002,   /* U+1D897 */
  0x0002,   /* U+1D898 */
  0x0002,   /* U+1D899 */
  0x0002,   /* U+1D89A */
  0x0002,   /* U+1D89B */
  0x0002,   /* U+1D89C */
  0x0002,   /* U+1D89D */
  0x0002,   /* U+1D89E */
  0x0002,   /* U+1D89F */
  0x0002,   /* U+1D8A0 */
  0x0002,   /* U+1D8A1 */
  0x0002,   /* U+1D8A2 */
  0x0002,   /* U+1D8A3 */
  0x0002,   /* U+1D8A4 */
  0x0002,   /* U+1D8A5 */
  0x0002,   /* U+1D8A6 */
  0x0002,   /* U+1D8A7 */
  0x0002,   /* U+1D8A8 */
  0x0002,   /* U+1D8A9 */
  0x0002,   /* U+1D8AA */
  0x0002,   /* U+1D8AB */
  0x0002,   /* U+1D8AC */
  0x0002,   /* U+1D8AD */
  0x0002,   /* U+1D8AE */
  0x0002,   /* U+1D8AF */
  0x0002,   /* U+1D8B0 */
  0x0002,   /* U+1D8B1 */
  0x0002,   /* U+1D8B2 */
  0x0002,   /* U+1D8B3 */
  0x0002,   /* U+1D8B4 */
  0x0002,   /* U+1D8B5 */
  0x0002,   /* U+1D8B6 */
  0x0002,   /* U+1D8B7 */
  0x0002,   /* U+1D8B8 */
  0x0002,   /* U+1D8B9 */
  0x0002,   /* U+1D8BA */
  0x0002,   /* U+1D8BB */
  0x0002,   /* U+1D8BC */
  0x0002,   /* U+1D8BD */
  0x0002,   /* U+1D8BE */
  0x0002,   /* U+1D8BF */
  0x0002,   /* U+1D8C0 */
  0x0002,   /* U+1D8C1 */
  0x0002,   /* U+1D8C2 */
  0x0002,   /* U+1D8C3 */
  0x0002,   /* U+1D8C4 */
  0x0002,   /* U+1D8C5 */
  0x0002,   /* U+1D8C6 */
  0x0002,   /* U+1D8C7 */
  0x0002,   /* U+1D8C8 */
  0x0002,   /* U+1D8C9 */
  0x0002,   /* U+1D8CA */
  0x0002,   /* U+1D8CB */
  0x0002,   /* U+1D8CC */
  0x0002,   /* U+1D8CD */
  0x0002,   /* U+1D8CE */
  0x0002,   /* U+1D8CF */
  0x0002,   /* U+1D8D0 */
  0x0002,   /* U+1D8D1 */
  0x0002,   /* U+1D8D2 */
  0x0002,   /* U+1D8D3 */
  0x0002,   /* U+1D8D4 */
  0x0002,   /* U+1D8D5 */
  0x0002,   /* U+1D8D6 */
  0x0002,   /* U+1D8D7 */
  0x0002,   /* U+1D8D8 */
  0x0002,   /* U+1D8D9 */
  0x0002,   /* U+1D8DA */
  0x0002,   /* U+1D8DB */
  0x0002,   /* U+1D8DC */
  0x0002,   /* U+1D8DD */
  0x0002,   /* U+1D8DE */
  0x0002,   /* U+1D8DF */
  0x0002,   /* U+1D8E0 */
  0x0002,   /* U+1D8E1 */
  0x0002,   /* U+1D8E2 */
  0x0002,   /* U+1D8E3 */
  0x0002,   /* U+1D8E4 */
  0x0002,   /* U+1D8E5 */
  0x0002,   /* U+1D8E6 */
  0x0002,   /* U+1D8E7 */
  0x0002,   /* U+1D8E8 */
  0x0002,   /* U+1D8E9 */
  0x0002,   /* U+1D8EA */
  0x0002,   /* U+1D8EB */
  0x0002,   /* U+1D8EC */
  0x0002,   /* U+1D8ED */
  0x0002,   /* U+1D8EE */
  0x0002,   /* U+1D8EF */
  0x0002,   /* U+1D8F0 */
  0x0002,   /* U+1D8F1 */
  0x0002,   /* U+1D8F2 */
  0x0002,   /* U+1D8F3 */
  0x0002,   /* U+1D8F4 */
  0x0002,   /* U+1D8F5 */
  0x0002,   /* U+1D8F6 */
  0x0002,   /* U+1D8F7 */
  0x0002,   /* U+1D8F8 */
  0x0002,   /* U+1D8F9 */
  0x0002,   /* U+1D8FA */
  0x0002,   /* U+1D8FB */
  0x0002,   /* U+1D8FC */
  0x0002,   /* U+1D8FD */
  0x0002,   /* U+1D8FE */
  0x0002,   /* U+1D8FF */
};

uint16 uca900_p1D9[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x193F,   /* U+1D900 */
  0x1940,   /* U+1D901 */
  0x1941,   /* U+1D902 */
  0x1942,   /* U+1D903 */
  0x1943,   /* U+1D904 */
  0x1944,   /* U+1D905 */
  0x1945,   /* U+1D906 */
  0x1946,   /* U+1D907 */
  0x1947,   /* U+1D908 */
  0x1948,   /* U+1D909 */
  0x1949,   /* U+1D90A */
  0x194A,   /* U+1D90B */
  0x194B,   /* U+1D90C */
  0x194C,   /* U+1D90D */
  0x194D,   /* U+1D90E */
  0x194E,   /* U+1D90F */
  0x194F,   /* U+1D910 */
  0x1950,   /* U+1D911 */
  0x1951,   /* U+1D912 */
  0x1952,   /* U+1D913 */
  0x1953,   /* U+1D914 */
  0x1954,   /* U+1D915 */
  0x1955,   /* U+1D916 */
  0x1956,   /* U+1D917 */
  0x1957,   /* U+1D918 */
  0x1958,   /* U+1D919 */
  0x1959,   /* U+1D91A */
  0x195A,   /* U+1D91B */
  0x195B,   /* U+1D91C */
  0x195C,   /* U+1D91D */
  0x195D,   /* U+1D91E */
  0x195E,   /* U+1D91F */
  0x195F,   /* U+1D920 */
  0x1960,   /* U+1D921 */
  0x1961,   /* U+1D922 */
  0x1962,   /* U+1D923 */
  0x1963,   /* U+1D924 */
  0x1964,   /* U+1D925 */
  0x1965,   /* U+1D926 */
  0x1966,   /* U+1D927 */
  0x1967,   /* U+1D928 */
  0x1968,   /* U+1D929 */
  0x1969,   /* U+1D92A */
  0x196A,   /* U+1D92B */
  0x196B,   /* U+1D92C */
  0x196C,   /* U+1D92D */
  0x196D,   /* U+1D92E */
  0x196E,   /* U+1D92F */
  0x196F,   /* U+1D930 */
  0x1970,   /* U+1D931 */
  0x1971,   /* U+1D932 */
  0x1972,   /* U+1D933 */
  0x1973,   /* U+1D934 */
  0x1974,   /* U+1D935 */
  0x1975,   /* U+1D936 */
  0x1976,   /* U+1D937 */
  0x1977,   /* U+1D938 */
  0x1978,   /* U+1D939 */
  0x1979,   /* U+1D93A */
  0x197A,   /* U+1D93B */
  0x197B,   /* U+1D93C */
  0x197C,   /* U+1D93D */
  0x197D,   /* U+1D93E */
  0x197E,   /* U+1D93F */
  0x197F,   /* U+1D940 */
  0x1980,   /* U+1D941 */
  0x1981,   /* U+1D942 */
  0x1982,   /* U+1D943 */
  0x1983,   /* U+1D944 */
  0x1984,   /* U+1D945 */
  0x1985,   /* U+1D946 */
  0x1986,   /* U+1D947 */
  0x1987,   /* U+1D948 */
  0x1988,   /* U+1D949 */
  0x1989,   /* U+1D94A */
  0x198A,   /* U+1D94B */
  0x198B,   /* U+1D94C */
  0x198C,   /* U+1D94D */
  0x198D,   /* U+1D94E */
  0x198E,   /* U+1D94F */
  0x198F,   /* U+1D950 */
  0x1990,   /* U+1D951 */
  0x1991,   /* U+1D952 */
  0x1992,   /* U+1D953 */
  0x1993,   /* U+1D954 */
  0x1994,   /* U+1D955 */
  0x1995,   /* U+1D956 */
  0x1996,   /* U+1D957 */
  0x1997,   /* U+1D958 */
  0x1998,   /* U+1D959 */
  0x1999,   /* U+1D95A */
  0x199A,   /* U+1D95B */
  0x199B,   /* U+1D95C */
  0x199C,   /* U+1D95D */
  0x199D,   /* U+1D95E */
  0x199E,   /* U+1D95F */
  0x199F,   /* U+1D960 */
  0x19A0,   /* U+1D961 */
  0x19A1,   /* U+1D962 */
  0x19A2,   /* U+1D963 */
  0x19A3,   /* U+1D964 */
  0x19A4,   /* U+1D965 */
  0x19A5,   /* U+1D966 */
  0x19A6,   /* U+1D967 */
  0x19A7,   /* U+1D968 */
  0x19A8,   /* U+1D969 */
  0x19A9,   /* U+1D96A */
  0x19AA,   /* U+1D96B */
  0x19AB,   /* U+1D96C */
  0x19AC,   /* U+1D96D */
  0x19AD,   /* U+1D96E */
  0x19AE,   /* U+1D96F */
  0x19AF,   /* U+1D970 */
  0x19B0,   /* U+1D971 */
  0x19B1,   /* U+1D972 */
  0x19B2,   /* U+1D973 */
  0x19B3,   /* U+1D974 */
  0x19B4,   /* U+1D975 */
  0x19B5,   /* U+1D976 */
  0x19B6,   /* U+1D977 */
  0x19B7,   /* U+1D978 */
  0x19B8,   /* U+1D979 */
  0x19B9,   /* U+1D97A */
  0x19BA,   /* U+1D97B */
  0x19BB,   /* U+1D97C */
  0x19BC,   /* U+1D97D */
  0x19BD,   /* U+1D97E */
  0x19BE,   /* U+1D97F */
  0x19BF,   /* U+1D980 */
  0x19C0,   /* U+1D981 */
  0x19C1,   /* U+1D982 */
  0x19C2,   /* U+1D983 */
  0x19C3,   /* U+1D984 */
  0x19C4,   /* U+1D985 */
  0x19C5,   /* U+1D986 */
  0x19C6,   /* U+1D987 */
  0x19C7,   /* U+1D988 */
  0x19C8,   /* U+1D989 */
  0x19C9,   /* U+1D98A */
  0x19CA,   /* U+1D98B */
  0x19CB,   /* U+1D98C */
  0x19CC,   /* U+1D98D */
  0x19CD,   /* U+1D98E */
  0x19CE,   /* U+1D98F */
  0x19CF,   /* U+1D990 */
  0x19D0,   /* U+1D991 */
  0x19D1,   /* U+1D992 */
  0x19D2,   /* U+1D993 */
  0x19D3,   /* U+1D994 */
  0x19D4,   /* U+1D995 */
  0x19D5,   /* U+1D996 */
  0x19D6,   /* U+1D997 */
  0x19D7,   /* U+1D998 */
  0x19D8,   /* U+1D999 */
  0x19D9,   /* U+1D99A */
  0x19DA,   /* U+1D99B */
  0x19DB,   /* U+1D99C */
  0x19DC,   /* U+1D99D */
  0x19DD,   /* U+1D99E */
  0x19DE,   /* U+1D99F */
  0x19DF,   /* U+1D9A0 */
  0x19E0,   /* U+1D9A1 */
  0x19E1,   /* U+1D9A2 */
  0x19E2,   /* U+1D9A3 */
  0x19E3,   /* U+1D9A4 */
  0x19E4,   /* U+1D9A5 */
  0x19E5,   /* U+1D9A6 */
  0x19E6,   /* U+1D9A7 */
  0x19E7,   /* U+1D9A8 */
  0x19E8,   /* U+1D9A9 */
  0x19E9,   /* U+1D9AA */
  0x19EA,   /* U+1D9AB */
  0x19EB,   /* U+1D9AC */
  0x19EC,   /* U+1D9AD */
  0x19ED,   /* U+1D9AE */
  0x19EE,   /* U+1D9AF */
  0x19EF,   /* U+1D9B0 */
  0x19F0,   /* U+1D9B1 */
  0x19F1,   /* U+1D9B2 */
  0x19F2,   /* U+1D9B3 */
  0x19F3,   /* U+1D9B4 */
  0x19F4,   /* U+1D9B5 */
  0x19F5,   /* U+1D9B6 */
  0x19F6,   /* U+1D9B7 */
  0x19F7,   /* U+1D9B8 */
  0x19F8,   /* U+1D9B9 */
  0x19F9,   /* U+1D9BA */
  0x19FA,   /* U+1D9BB */
  0x19FB,   /* U+1D9BC */
  0x19FC,   /* U+1D9BD */
  0x19FD,   /* U+1D9BE */
  0x19FE,   /* U+1D9BF */
  0x19FF,   /* U+1D9C0 */
  0x1A00,   /* U+1D9C1 */
  0x1A01,   /* U+1D9C2 */
  0x1A02,   /* U+1D9C3 */
  0x1A03,   /* U+1D9C4 */
  0x1A04,   /* U+1D9C5 */
  0x1A05,   /* U+1D9C6 */
  0x1A06,   /* U+1D9C7 */
  0x1A07,   /* U+1D9C8 */
  0x1A08,   /* U+1D9C9 */
  0x1A09,   /* U+1D9CA */
  0x1A0A,   /* U+1D9CB */
  0x1A0B,   /* U+1D9CC */
  0x1A0C,   /* U+1D9CD */
  0x1A0D,   /* U+1D9CE */
  0x1A0E,   /* U+1D9CF */
  0x1A0F,   /* U+1D9D0 */
  0x1A10,   /* U+1D9D1 */
  0x1A11,   /* U+1D9D2 */
  0x1A12,   /* U+1D9D3 */
  0x1A13,   /* U+1D9D4 */
  0x1A14,   /* U+1D9D5 */
  0x1A15,   /* U+1D9D6 */
  0x1A16,   /* U+1D9D7 */
  0x1A17,   /* U+1D9D8 */
  0x1A18,   /* U+1D9D9 */
  0x1A19,   /* U+1D9DA */
  0x1A1A,   /* U+1D9DB */
  0x1A1B,   /* U+1D9DC */
  0x1A1C,   /* U+1D9DD */
  0x1A1D,   /* U+1D9DE */
  0x1A1E,   /* U+1D9DF */
  0x1A1F,   /* U+1D9E0 */
  0x1A20,   /* U+1D9E1 */
  0x1A21,   /* U+1D9E2 */
  0x1A22,   /* U+1D9E3 */
  0x1A23,   /* U+1D9E4 */
  0x1A24,   /* U+1D9E5 */
  0x1A25,   /* U+1D9E6 */
  0x1A26,   /* U+1D9E7 */
  0x1A27,   /* U+1D9E8 */
  0x1A28,   /* U+1D9E9 */
  0x1A29,   /* U+1D9EA */
  0x1A2A,   /* U+1D9EB */
  0x1A2B,   /* U+1D9EC */
  0x1A2C,   /* U+1D9ED */
  0x1A2D,   /* U+1D9EE */
  0x1A2E,   /* U+1D9EF */
  0x1A2F,   /* U+1D9F0 */
  0x1A30,   /* U+1D9F1 */
  0x1A31,   /* U+1D9F2 */
  0x1A32,   /* U+1D9F3 */
  0x1A33,   /* U+1D9F4 */
  0x1A34,   /* U+1D9F5 */
  0x1A35,   /* U+1D9F6 */
  0x1A36,   /* U+1D9F7 */
  0x1A37,   /* U+1D9F8 */
  0x1A38,   /* U+1D9F9 */
  0x1A39,   /* U+1D9FA */
  0x1A3A,   /* U+1D9FB */
  0x1A3B,   /* U+1D9FC */
  0x1A3C,   /* U+1D9FD */
  0x1A3D,   /* U+1D9FE */
  0x1A3E,   /* U+1D9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1D900 */
  0x0020,   /* U+1D901 */
  0x0020,   /* U+1D902 */
  0x0020,   /* U+1D903 */
  0x0020,   /* U+1D904 */
  0x0020,   /* U+1D905 */
  0x0020,   /* U+1D906 */
  0x0020,   /* U+1D907 */
  0x0020,   /* U+1D908 */
  0x0020,   /* U+1D909 */
  0x0020,   /* U+1D90A */
  0x0020,   /* U+1D90B */
  0x0020,   /* U+1D90C */
  0x0020,   /* U+1D90D */
  0x0020,   /* U+1D90E */
  0x0020,   /* U+1D90F */
  0x0020,   /* U+1D910 */
  0x0020,   /* U+1D911 */
  0x0020,   /* U+1D912 */
  0x0020,   /* U+1D913 */
  0x0020,   /* U+1D914 */
  0x0020,   /* U+1D915 */
  0x0020,   /* U+1D916 */
  0x0020,   /* U+1D917 */
  0x0020,   /* U+1D918 */
  0x0020,   /* U+1D919 */
  0x0020,   /* U+1D91A */
  0x0020,   /* U+1D91B */
  0x0020,   /* U+1D91C */
  0x0020,   /* U+1D91D */
  0x0020,   /* U+1D91E */
  0x0020,   /* U+1D91F */
  0x0020,   /* U+1D920 */
  0x0020,   /* U+1D921 */
  0x0020,   /* U+1D922 */
  0x0020,   /* U+1D923 */
  0x0020,   /* U+1D924 */
  0x0020,   /* U+1D925 */
  0x0020,   /* U+1D926 */
  0x0020,   /* U+1D927 */
  0x0020,   /* U+1D928 */
  0x0020,   /* U+1D929 */
  0x0020,   /* U+1D92A */
  0x0020,   /* U+1D92B */
  0x0020,   /* U+1D92C */
  0x0020,   /* U+1D92D */
  0x0020,   /* U+1D92E */
  0x0020,   /* U+1D92F */
  0x0020,   /* U+1D930 */
  0x0020,   /* U+1D931 */
  0x0020,   /* U+1D932 */
  0x0020,   /* U+1D933 */
  0x0020,   /* U+1D934 */
  0x0020,   /* U+1D935 */
  0x0020,   /* U+1D936 */
  0x0020,   /* U+1D937 */
  0x0020,   /* U+1D938 */
  0x0020,   /* U+1D939 */
  0x0020,   /* U+1D93A */
  0x0020,   /* U+1D93B */
  0x0020,   /* U+1D93C */
  0x0020,   /* U+1D93D */
  0x0020,   /* U+1D93E */
  0x0020,   /* U+1D93F */
  0x0020,   /* U+1D940 */
  0x0020,   /* U+1D941 */
  0x0020,   /* U+1D942 */
  0x0020,   /* U+1D943 */
  0x0020,   /* U+1D944 */
  0x0020,   /* U+1D945 */
  0x0020,   /* U+1D946 */
  0x0020,   /* U+1D947 */
  0x0020,   /* U+1D948 */
  0x0020,   /* U+1D949 */
  0x0020,   /* U+1D94A */
  0x0020,   /* U+1D94B */
  0x0020,   /* U+1D94C */
  0x0020,   /* U+1D94D */
  0x0020,   /* U+1D94E */
  0x0020,   /* U+1D94F */
  0x0020,   /* U+1D950 */
  0x0020,   /* U+1D951 */
  0x0020,   /* U+1D952 */
  0x0020,   /* U+1D953 */
  0x0020,   /* U+1D954 */
  0x0020,   /* U+1D955 */
  0x0020,   /* U+1D956 */
  0x0020,   /* U+1D957 */
  0x0020,   /* U+1D958 */
  0x0020,   /* U+1D959 */
  0x0020,   /* U+1D95A */
  0x0020,   /* U+1D95B */
  0x0020,   /* U+1D95C */
  0x0020,   /* U+1D95D */
  0x0020,   /* U+1D95E */
  0x0020,   /* U+1D95F */
  0x0020,   /* U+1D960 */
  0x0020,   /* U+1D961 */
  0x0020,   /* U+1D962 */
  0x0020,   /* U+1D963 */
  0x0020,   /* U+1D964 */
  0x0020,   /* U+1D965 */
  0x0020,   /* U+1D966 */
  0x0020,   /* U+1D967 */
  0x0020,   /* U+1D968 */
  0x0020,   /* U+1D969 */
  0x0020,   /* U+1D96A */
  0x0020,   /* U+1D96B */
  0x0020,   /* U+1D96C */
  0x0020,   /* U+1D96D */
  0x0020,   /* U+1D96E */
  0x0020,   /* U+1D96F */
  0x0020,   /* U+1D970 */
  0x0020,   /* U+1D971 */
  0x0020,   /* U+1D972 */
  0x0020,   /* U+1D973 */
  0x0020,   /* U+1D974 */
  0x0020,   /* U+1D975 */
  0x0020,   /* U+1D976 */
  0x0020,   /* U+1D977 */
  0x0020,   /* U+1D978 */
  0x0020,   /* U+1D979 */
  0x0020,   /* U+1D97A */
  0x0020,   /* U+1D97B */
  0x0020,   /* U+1D97C */
  0x0020,   /* U+1D97D */
  0x0020,   /* U+1D97E */
  0x0020,   /* U+1D97F */
  0x0020,   /* U+1D980 */
  0x0020,   /* U+1D981 */
  0x0020,   /* U+1D982 */
  0x0020,   /* U+1D983 */
  0x0020,   /* U+1D984 */
  0x0020,   /* U+1D985 */
  0x0020,   /* U+1D986 */
  0x0020,   /* U+1D987 */
  0x0020,   /* U+1D988 */
  0x0020,   /* U+1D989 */
  0x0020,   /* U+1D98A */
  0x0020,   /* U+1D98B */
  0x0020,   /* U+1D98C */
  0x0020,   /* U+1D98D */
  0x0020,   /* U+1D98E */
  0x0020,   /* U+1D98F */
  0x0020,   /* U+1D990 */
  0x0020,   /* U+1D991 */
  0x0020,   /* U+1D992 */
  0x0020,   /* U+1D993 */
  0x0020,   /* U+1D994 */
  0x0020,   /* U+1D995 */
  0x0020,   /* U+1D996 */
  0x0020,   /* U+1D997 */
  0x0020,   /* U+1D998 */
  0x0020,   /* U+1D999 */
  0x0020,   /* U+1D99A */
  0x0020,   /* U+1D99B */
  0x0020,   /* U+1D99C */
  0x0020,   /* U+1D99D */
  0x0020,   /* U+1D99E */
  0x0020,   /* U+1D99F */
  0x0020,   /* U+1D9A0 */
  0x0020,   /* U+1D9A1 */
  0x0020,   /* U+1D9A2 */
  0x0020,   /* U+1D9A3 */
  0x0020,   /* U+1D9A4 */
  0x0020,   /* U+1D9A5 */
  0x0020,   /* U+1D9A6 */
  0x0020,   /* U+1D9A7 */
  0x0020,   /* U+1D9A8 */
  0x0020,   /* U+1D9A9 */
  0x0020,   /* U+1D9AA */
  0x0020,   /* U+1D9AB */
  0x0020,   /* U+1D9AC */
  0x0020,   /* U+1D9AD */
  0x0020,   /* U+1D9AE */
  0x0020,   /* U+1D9AF */
  0x0020,   /* U+1D9B0 */
  0x0020,   /* U+1D9B1 */
  0x0020,   /* U+1D9B2 */
  0x0020,   /* U+1D9B3 */
  0x0020,   /* U+1D9B4 */
  0x0020,   /* U+1D9B5 */
  0x0020,   /* U+1D9B6 */
  0x0020,   /* U+1D9B7 */
  0x0020,   /* U+1D9B8 */
  0x0020,   /* U+1D9B9 */
  0x0020,   /* U+1D9BA */
  0x0020,   /* U+1D9BB */
  0x0020,   /* U+1D9BC */
  0x0020,   /* U+1D9BD */
  0x0020,   /* U+1D9BE */
  0x0020,   /* U+1D9BF */
  0x0020,   /* U+1D9C0 */
  0x0020,   /* U+1D9C1 */
  0x0020,   /* U+1D9C2 */
  0x0020,   /* U+1D9C3 */
  0x0020,   /* U+1D9C4 */
  0x0020,   /* U+1D9C5 */
  0x0020,   /* U+1D9C6 */
  0x0020,   /* U+1D9C7 */
  0x0020,   /* U+1D9C8 */
  0x0020,   /* U+1D9C9 */
  0x0020,   /* U+1D9CA */
  0x0020,   /* U+1D9CB */
  0x0020,   /* U+1D9CC */
  0x0020,   /* U+1D9CD */
  0x0020,   /* U+1D9CE */
  0x0020,   /* U+1D9CF */
  0x0020,   /* U+1D9D0 */
  0x0020,   /* U+1D9D1 */
  0x0020,   /* U+1D9D2 */
  0x0020,   /* U+1D9D3 */
  0x0020,   /* U+1D9D4 */
  0x0020,   /* U+1D9D5 */
  0x0020,   /* U+1D9D6 */
  0x0020,   /* U+1D9D7 */
  0x0020,   /* U+1D9D8 */
  0x0020,   /* U+1D9D9 */
  0x0020,   /* U+1D9DA */
  0x0020,   /* U+1D9DB */
  0x0020,   /* U+1D9DC */
  0x0020,   /* U+1D9DD */
  0x0020,   /* U+1D9DE */
  0x0020,   /* U+1D9DF */
  0x0020,   /* U+1D9E0 */
  0x0020,   /* U+1D9E1 */
  0x0020,   /* U+1D9E2 */
  0x0020,   /* U+1D9E3 */
  0x0020,   /* U+1D9E4 */
  0x0020,   /* U+1D9E5 */
  0x0020,   /* U+1D9E6 */
  0x0020,   /* U+1D9E7 */
  0x0020,   /* U+1D9E8 */
  0x0020,   /* U+1D9E9 */
  0x0020,   /* U+1D9EA */
  0x0020,   /* U+1D9EB */
  0x0020,   /* U+1D9EC */
  0x0020,   /* U+1D9ED */
  0x0020,   /* U+1D9EE */
  0x0020,   /* U+1D9EF */
  0x0020,   /* U+1D9F0 */
  0x0020,   /* U+1D9F1 */
  0x0020,   /* U+1D9F2 */
  0x0020,   /* U+1D9F3 */
  0x0020,   /* U+1D9F4 */
  0x0020,   /* U+1D9F5 */
  0x0020,   /* U+1D9F6 */
  0x0020,   /* U+1D9F7 */
  0x0020,   /* U+1D9F8 */
  0x0020,   /* U+1D9F9 */
  0x0020,   /* U+1D9FA */
  0x0020,   /* U+1D9FB */
  0x0020,   /* U+1D9FC */
  0x0020,   /* U+1D9FD */
  0x0020,   /* U+1D9FE */
  0x0020,   /* U+1D9FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1D900 */
  0x0002,   /* U+1D901 */
  0x0002,   /* U+1D902 */
  0x0002,   /* U+1D903 */
  0x0002,   /* U+1D904 */
  0x0002,   /* U+1D905 */
  0x0002,   /* U+1D906 */
  0x0002,   /* U+1D907 */
  0x0002,   /* U+1D908 */
  0x0002,   /* U+1D909 */
  0x0002,   /* U+1D90A */
  0x0002,   /* U+1D90B */
  0x0002,   /* U+1D90C */
  0x0002,   /* U+1D90D */
  0x0002,   /* U+1D90E */
  0x0002,   /* U+1D90F */
  0x0002,   /* U+1D910 */
  0x0002,   /* U+1D911 */
  0x0002,   /* U+1D912 */
  0x0002,   /* U+1D913 */
  0x0002,   /* U+1D914 */
  0x0002,   /* U+1D915 */
  0x0002,   /* U+1D916 */
  0x0002,   /* U+1D917 */
  0x0002,   /* U+1D918 */
  0x0002,   /* U+1D919 */
  0x0002,   /* U+1D91A */
  0x0002,   /* U+1D91B */
  0x0002,   /* U+1D91C */
  0x0002,   /* U+1D91D */
  0x0002,   /* U+1D91E */
  0x0002,   /* U+1D91F */
  0x0002,   /* U+1D920 */
  0x0002,   /* U+1D921 */
  0x0002,   /* U+1D922 */
  0x0002,   /* U+1D923 */
  0x0002,   /* U+1D924 */
  0x0002,   /* U+1D925 */
  0x0002,   /* U+1D926 */
  0x0002,   /* U+1D927 */
  0x0002,   /* U+1D928 */
  0x0002,   /* U+1D929 */
  0x0002,   /* U+1D92A */
  0x0002,   /* U+1D92B */
  0x0002,   /* U+1D92C */
  0x0002,   /* U+1D92D */
  0x0002,   /* U+1D92E */
  0x0002,   /* U+1D92F */
  0x0002,   /* U+1D930 */
  0x0002,   /* U+1D931 */
  0x0002,   /* U+1D932 */
  0x0002,   /* U+1D933 */
  0x0002,   /* U+1D934 */
  0x0002,   /* U+1D935 */
  0x0002,   /* U+1D936 */
  0x0002,   /* U+1D937 */
  0x0002,   /* U+1D938 */
  0x0002,   /* U+1D939 */
  0x0002,   /* U+1D93A */
  0x0002,   /* U+1D93B */
  0x0002,   /* U+1D93C */
  0x0002,   /* U+1D93D */
  0x0002,   /* U+1D93E */
  0x0002,   /* U+1D93F */
  0x0002,   /* U+1D940 */
  0x0002,   /* U+1D941 */
  0x0002,   /* U+1D942 */
  0x0002,   /* U+1D943 */
  0x0002,   /* U+1D944 */
  0x0002,   /* U+1D945 */
  0x0002,   /* U+1D946 */
  0x0002,   /* U+1D947 */
  0x0002,   /* U+1D948 */
  0x0002,   /* U+1D949 */
  0x0002,   /* U+1D94A */
  0x0002,   /* U+1D94B */
  0x0002,   /* U+1D94C */
  0x0002,   /* U+1D94D */
  0x0002,   /* U+1D94E */
  0x0002,   /* U+1D94F */
  0x0002,   /* U+1D950 */
  0x0002,   /* U+1D951 */
  0x0002,   /* U+1D952 */
  0x0002,   /* U+1D953 */
  0x0002,   /* U+1D954 */
  0x0002,   /* U+1D955 */
  0x0002,   /* U+1D956 */
  0x0002,   /* U+1D957 */
  0x0002,   /* U+1D958 */
  0x0002,   /* U+1D959 */
  0x0002,   /* U+1D95A */
  0x0002,   /* U+1D95B */
  0x0002,   /* U+1D95C */
  0x0002,   /* U+1D95D */
  0x0002,   /* U+1D95E */
  0x0002,   /* U+1D95F */
  0x0002,   /* U+1D960 */
  0x0002,   /* U+1D961 */
  0x0002,   /* U+1D962 */
  0x0002,   /* U+1D963 */
  0x0002,   /* U+1D964 */
  0x0002,   /* U+1D965 */
  0x0002,   /* U+1D966 */
  0x0002,   /* U+1D967 */
  0x0002,   /* U+1D968 */
  0x0002,   /* U+1D969 */
  0x0002,   /* U+1D96A */
  0x0002,   /* U+1D96B */
  0x0002,   /* U+1D96C */
  0x0002,   /* U+1D96D */
  0x0002,   /* U+1D96E */
  0x0002,   /* U+1D96F */
  0x0002,   /* U+1D970 */
  0x0002,   /* U+1D971 */
  0x0002,   /* U+1D972 */
  0x0002,   /* U+1D973 */
  0x0002,   /* U+1D974 */
  0x0002,   /* U+1D975 */
  0x0002,   /* U+1D976 */
  0x0002,   /* U+1D977 */
  0x0002,   /* U+1D978 */
  0x0002,   /* U+1D979 */
  0x0002,   /* U+1D97A */
  0x0002,   /* U+1D97B */
  0x0002,   /* U+1D97C */
  0x0002,   /* U+1D97D */
  0x0002,   /* U+1D97E */
  0x0002,   /* U+1D97F */
  0x0002,   /* U+1D980 */
  0x0002,   /* U+1D981 */
  0x0002,   /* U+1D982 */
  0x0002,   /* U+1D983 */
  0x0002,   /* U+1D984 */
  0x0002,   /* U+1D985 */
  0x0002,   /* U+1D986 */
  0x0002,   /* U+1D987 */
  0x0002,   /* U+1D988 */
  0x0002,   /* U+1D989 */
  0x0002,   /* U+1D98A */
  0x0002,   /* U+1D98B */
  0x0002,   /* U+1D98C */
  0x0002,   /* U+1D98D */
  0x0002,   /* U+1D98E */
  0x0002,   /* U+1D98F */
  0x0002,   /* U+1D990 */
  0x0002,   /* U+1D991 */
  0x0002,   /* U+1D992 */
  0x0002,   /* U+1D993 */
  0x0002,   /* U+1D994 */
  0x0002,   /* U+1D995 */
  0x0002,   /* U+1D996 */
  0x0002,   /* U+1D997 */
  0x0002,   /* U+1D998 */
  0x0002,   /* U+1D999 */
  0x0002,   /* U+1D99A */
  0x0002,   /* U+1D99B */
  0x0002,   /* U+1D99C */
  0x0002,   /* U+1D99D */
  0x0002,   /* U+1D99E */
  0x0002,   /* U+1D99F */
  0x0002,   /* U+1D9A0 */
  0x0002,   /* U+1D9A1 */
  0x0002,   /* U+1D9A2 */
  0x0002,   /* U+1D9A3 */
  0x0002,   /* U+1D9A4 */
  0x0002,   /* U+1D9A5 */
  0x0002,   /* U+1D9A6 */
  0x0002,   /* U+1D9A7 */
  0x0002,   /* U+1D9A8 */
  0x0002,   /* U+1D9A9 */
  0x0002,   /* U+1D9AA */
  0x0002,   /* U+1D9AB */
  0x0002,   /* U+1D9AC */
  0x0002,   /* U+1D9AD */
  0x0002,   /* U+1D9AE */
  0x0002,   /* U+1D9AF */
  0x0002,   /* U+1D9B0 */
  0x0002,   /* U+1D9B1 */
  0x0002,   /* U+1D9B2 */
  0x0002,   /* U+1D9B3 */
  0x0002,   /* U+1D9B4 */
  0x0002,   /* U+1D9B5 */
  0x0002,   /* U+1D9B6 */
  0x0002,   /* U+1D9B7 */
  0x0002,   /* U+1D9B8 */
  0x0002,   /* U+1D9B9 */
  0x0002,   /* U+1D9BA */
  0x0002,   /* U+1D9BB */
  0x0002,   /* U+1D9BC */
  0x0002,   /* U+1D9BD */
  0x0002,   /* U+1D9BE */
  0x0002,   /* U+1D9BF */
  0x0002,   /* U+1D9C0 */
  0x0002,   /* U+1D9C1 */
  0x0002,   /* U+1D9C2 */
  0x0002,   /* U+1D9C3 */
  0x0002,   /* U+1D9C4 */
  0x0002,   /* U+1D9C5 */
  0x0002,   /* U+1D9C6 */
  0x0002,   /* U+1D9C7 */
  0x0002,   /* U+1D9C8 */
  0x0002,   /* U+1D9C9 */
  0x0002,   /* U+1D9CA */
  0x0002,   /* U+1D9CB */
  0x0002,   /* U+1D9CC */
  0x0002,   /* U+1D9CD */
  0x0002,   /* U+1D9CE */
  0x0002,   /* U+1D9CF */
  0x0002,   /* U+1D9D0 */
  0x0002,   /* U+1D9D1 */
  0x0002,   /* U+1D9D2 */
  0x0002,   /* U+1D9D3 */
  0x0002,   /* U+1D9D4 */
  0x0002,   /* U+1D9D5 */
  0x0002,   /* U+1D9D6 */
  0x0002,   /* U+1D9D7 */
  0x0002,   /* U+1D9D8 */
  0x0002,   /* U+1D9D9 */
  0x0002,   /* U+1D9DA */
  0x0002,   /* U+1D9DB */
  0x0002,   /* U+1D9DC */
  0x0002,   /* U+1D9DD */
  0x0002,   /* U+1D9DE */
  0x0002,   /* U+1D9DF */
  0x0002,   /* U+1D9E0 */
  0x0002,   /* U+1D9E1 */
  0x0002,   /* U+1D9E2 */
  0x0002,   /* U+1D9E3 */
  0x0002,   /* U+1D9E4 */
  0x0002,   /* U+1D9E5 */
  0x0002,   /* U+1D9E6 */
  0x0002,   /* U+1D9E7 */
  0x0002,   /* U+1D9E8 */
  0x0002,   /* U+1D9E9 */
  0x0002,   /* U+1D9EA */
  0x0002,   /* U+1D9EB */
  0x0002,   /* U+1D9EC */
  0x0002,   /* U+1D9ED */
  0x0002,   /* U+1D9EE */
  0x0002,   /* U+1D9EF */
  0x0002,   /* U+1D9F0 */
  0x0002,   /* U+1D9F1 */
  0x0002,   /* U+1D9F2 */
  0x0002,   /* U+1D9F3 */
  0x0002,   /* U+1D9F4 */
  0x0002,   /* U+1D9F5 */
  0x0002,   /* U+1D9F6 */
  0x0002,   /* U+1D9F7 */
  0x0002,   /* U+1D9F8 */
  0x0002,   /* U+1D9F9 */
  0x0002,   /* U+1D9FA */
  0x0002,   /* U+1D9FB */
  0x0002,   /* U+1D9FC */
  0x0002,   /* U+1D9FD */
  0x0002,   /* U+1D9FE */
  0x0002,   /* U+1D9FF */
};

uint16 uca900_p1DA[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x1A3F,   /* U+1DA37 */
  0x1A40,   /* U+1DA38 */
  0x1A41,   /* U+1DA39 */
  0x1A42,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x1A43,   /* U+1DA6D */
  0x1A44,   /* U+1DA6E */
  0x1A45,   /* U+1DA6F */
  0x1A46,   /* U+1DA70 */
  0x1A47,   /* U+1DA71 */
  0x1A48,   /* U+1DA72 */
  0x1A49,   /* U+1DA73 */
  0x1A4A,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x1A4B,   /* U+1DA76 */
  0x1A4C,   /* U+1DA77 */
  0x1A4D,   /* U+1DA78 */
  0x1A4E,   /* U+1DA79 */
  0x1A4F,   /* U+1DA7A */
  0x1A50,   /* U+1DA7B */
  0x1A51,   /* U+1DA7C */
  0x1A52,   /* U+1DA7D */
  0x1A53,   /* U+1DA7E */
  0x1A54,   /* U+1DA7F */
  0x1A55,   /* U+1DA80 */
  0x1A56,   /* U+1DA81 */
  0x1A57,   /* U+1DA82 */
  0x1A58,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x1A59,   /* U+1DA85 */
  0x1A5A,   /* U+1DA86 */
  0x047D,   /* U+1DA87 */
  0x047E,   /* U+1DA88 */
  0x047F,   /* U+1DA89 */
  0x0480,   /* U+1DA8A */
  0x0481,   /* U+1DA8B */
  0xFBC3,   /* U+1DA8C */
  0xFBC3,   /* U+1DA8D */
  0xFBC3,   /* U+1DA8E */
  0xFBC3,   /* U+1DA8F */
  0xFBC3,   /* U+1DA90 */
  0xFBC3,   /* U+1DA91 */
  0xFBC3,   /* U+1DA92 */
  0xFBC3,   /* U+1DA93 */
  0xFBC3,   /* U+1DA94 */
  0xFBC3,   /* U+1DA95 */
  0xFBC3,   /* U+1DA96 */
  0xFBC3,   /* U+1DA97 */
  0xFBC3,   /* U+1DA98 */
  0xFBC3,   /* U+1DA99 */
  0xFBC3,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0xFBC3,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0xFBC3,   /* U+1DAB0 */
  0xFBC3,   /* U+1DAB1 */
  0xFBC3,   /* U+1DAB2 */
  0xFBC3,   /* U+1DAB3 */
  0xFBC3,   /* U+1DAB4 */
  0xFBC3,   /* U+1DAB5 */
  0xFBC3,   /* U+1DAB6 */
  0xFBC3,   /* U+1DAB7 */
  0xFBC3,   /* U+1DAB8 */
  0xFBC3,   /* U+1DAB9 */
  0xFBC3,   /* U+1DABA */
  0xFBC3,   /* U+1DABB */
  0xFBC3,   /* U+1DABC */
  0xFBC3,   /* U+1DABD */
  0xFBC3,   /* U+1DABE */
  0xFBC3,   /* U+1DABF */
  0xFBC3,   /* U+1DAC0 */
  0xFBC3,   /* U+1DAC1 */
  0xFBC3,   /* U+1DAC2 */
  0xFBC3,   /* U+1DAC3 */
  0xFBC3,   /* U+1DAC4 */
  0xFBC3,   /* U+1DAC5 */
  0xFBC3,   /* U+1DAC6 */
  0xFBC3,   /* U+1DAC7 */
  0xFBC3,   /* U+1DAC8 */
  0xFBC3,   /* U+1DAC9 */
  0xFBC3,   /* U+1DACA */
  0xFBC3,   /* U+1DACB */
  0xFBC3,   /* U+1DACC */
  0xFBC3,   /* U+1DACD */
  0xFBC3,   /* U+1DACE */
  0xFBC3,   /* U+1DACF */
  0xFBC3,   /* U+1DAD0 */
  0xFBC3,   /* U+1DAD1 */
  0xFBC3,   /* U+1DAD2 */
  0xFBC3,   /* U+1DAD3 */
  0xFBC3,   /* U+1DAD4 */
  0xFBC3,   /* U+1DAD5 */
  0xFBC3,   /* U+1DAD6 */
  0xFBC3,   /* U+1DAD7 */
  0xFBC3,   /* U+1DAD8 */
  0xFBC3,   /* U+1DAD9 */
  0xFBC3,   /* U+1DADA */
  0xFBC3,   /* U+1DADB */
  0xFBC3,   /* U+1DADC */
  0xFBC3,   /* U+1DADD */
  0xFBC3,   /* U+1DADE */
  0xFBC3,   /* U+1DADF */
  0xFBC3,   /* U+1DAE0 */
  0xFBC3,   /* U+1DAE1 */
  0xFBC3,   /* U+1DAE2 */
  0xFBC3,   /* U+1DAE3 */
  0xFBC3,   /* U+1DAE4 */
  0xFBC3,   /* U+1DAE5 */
  0xFBC3,   /* U+1DAE6 */
  0xFBC3,   /* U+1DAE7 */
  0xFBC3,   /* U+1DAE8 */
  0xFBC3,   /* U+1DAE9 */
  0xFBC3,   /* U+1DAEA */
  0xFBC3,   /* U+1DAEB */
  0xFBC3,   /* U+1DAEC */
  0xFBC3,   /* U+1DAED */
  0xFBC3,   /* U+1DAEE */
  0xFBC3,   /* U+1DAEF */
  0xFBC3,   /* U+1DAF0 */
  0xFBC3,   /* U+1DAF1 */
  0xFBC3,   /* U+1DAF2 */
  0xFBC3,   /* U+1DAF3 */
  0xFBC3,   /* U+1DAF4 */
  0xFBC3,   /* U+1DAF5 */
  0xFBC3,   /* U+1DAF6 */
  0xFBC3,   /* U+1DAF7 */
  0xFBC3,   /* U+1DAF8 */
  0xFBC3,   /* U+1DAF9 */
  0xFBC3,   /* U+1DAFA */
  0xFBC3,   /* U+1DAFB */
  0xFBC3,   /* U+1DAFC */
  0xFBC3,   /* U+1DAFD */
  0xFBC3,   /* U+1DAFE */
  0xFBC3,   /* U+1DAFF */

  /* Secondary weight 1 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x0020,   /* U+1DA37 */
  0x0020,   /* U+1DA38 */
  0x0020,   /* U+1DA39 */
  0x0020,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x0020,   /* U+1DA6D */
  0x0020,   /* U+1DA6E */
  0x0020,   /* U+1DA6F */
  0x0020,   /* U+1DA70 */
  0x0020,   /* U+1DA71 */
  0x0020,   /* U+1DA72 */
  0x0020,   /* U+1DA73 */
  0x0020,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x0020,   /* U+1DA76 */
  0x0020,   /* U+1DA77 */
  0x0020,   /* U+1DA78 */
  0x0020,   /* U+1DA79 */
  0x0020,   /* U+1DA7A */
  0x0020,   /* U+1DA7B */
  0x0020,   /* U+1DA7C */
  0x0020,   /* U+1DA7D */
  0x0020,   /* U+1DA7E */
  0x0020,   /* U+1DA7F */
  0x0020,   /* U+1DA80 */
  0x0020,   /* U+1DA81 */
  0x0020,   /* U+1DA82 */
  0x0020,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x0020,   /* U+1DA85 */
  0x0020,   /* U+1DA86 */
  0x0020,   /* U+1DA87 */
  0x0020,   /* U+1DA88 */
  0x0020,   /* U+1DA89 */
  0x0020,   /* U+1DA8A */
  0x0020,   /* U+1DA8B */
  0x0020,   /* U+1DA8C */
  0x0020,   /* U+1DA8D */
  0x0020,   /* U+1DA8E */
  0x0020,   /* U+1DA8F */
  0x0020,   /* U+1DA90 */
  0x0020,   /* U+1DA91 */
  0x0020,   /* U+1DA92 */
  0x0020,   /* U+1DA93 */
  0x0020,   /* U+1DA94 */
  0x0020,   /* U+1DA95 */
  0x0020,   /* U+1DA96 */
  0x0020,   /* U+1DA97 */
  0x0020,   /* U+1DA98 */
  0x0020,   /* U+1DA99 */
  0x0020,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0x0020,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0x0020,   /* U+1DAB0 */
  0x0020,   /* U+1DAB1 */
  0x0020,   /* U+1DAB2 */
  0x0020,   /* U+1DAB3 */
  0x0020,   /* U+1DAB4 */
  0x0020,   /* U+1DAB5 */
  0x0020,   /* U+1DAB6 */
  0x0020,   /* U+1DAB7 */
  0x0020,   /* U+1DAB8 */
  0x0020,   /* U+1DAB9 */
  0x0020,   /* U+1DABA */
  0x0020,   /* U+1DABB */
  0x0020,   /* U+1DABC */
  0x0020,   /* U+1DABD */
  0x0020,   /* U+1DABE */
  0x0020,   /* U+1DABF */
  0x0020,   /* U+1DAC0 */
  0x0020,   /* U+1DAC1 */
  0x0020,   /* U+1DAC2 */
  0x0020,   /* U+1DAC3 */
  0x0020,   /* U+1DAC4 */
  0x0020,   /* U+1DAC5 */
  0x0020,   /* U+1DAC6 */
  0x0020,   /* U+1DAC7 */
  0x0020,   /* U+1DAC8 */
  0x0020,   /* U+1DAC9 */
  0x0020,   /* U+1DACA */
  0x0020,   /* U+1DACB */
  0x0020,   /* U+1DACC */
  0x0020,   /* U+1DACD */
  0x0020,   /* U+1DACE */
  0x0020,   /* U+1DACF */
  0x0020,   /* U+1DAD0 */
  0x0020,   /* U+1DAD1 */
  0x0020,   /* U+1DAD2 */
  0x0020,   /* U+1DAD3 */
  0x0020,   /* U+1DAD4 */
  0x0020,   /* U+1DAD5 */
  0x0020,   /* U+1DAD6 */
  0x0020,   /* U+1DAD7 */
  0x0020,   /* U+1DAD8 */
  0x0020,   /* U+1DAD9 */
  0x0020,   /* U+1DADA */
  0x0020,   /* U+1DADB */
  0x0020,   /* U+1DADC */
  0x0020,   /* U+1DADD */
  0x0020,   /* U+1DADE */
  0x0020,   /* U+1DADF */
  0x0020,   /* U+1DAE0 */
  0x0020,   /* U+1DAE1 */
  0x0020,   /* U+1DAE2 */
  0x0020,   /* U+1DAE3 */
  0x0020,   /* U+1DAE4 */
  0x0020,   /* U+1DAE5 */
  0x0020,   /* U+1DAE6 */
  0x0020,   /* U+1DAE7 */
  0x0020,   /* U+1DAE8 */
  0x0020,   /* U+1DAE9 */
  0x0020,   /* U+1DAEA */
  0x0020,   /* U+1DAEB */
  0x0020,   /* U+1DAEC */
  0x0020,   /* U+1DAED */
  0x0020,   /* U+1DAEE */
  0x0020,   /* U+1DAEF */
  0x0020,   /* U+1DAF0 */
  0x0020,   /* U+1DAF1 */
  0x0020,   /* U+1DAF2 */
  0x0020,   /* U+1DAF3 */
  0x0020,   /* U+1DAF4 */
  0x0020,   /* U+1DAF5 */
  0x0020,   /* U+1DAF6 */
  0x0020,   /* U+1DAF7 */
  0x0020,   /* U+1DAF8 */
  0x0020,   /* U+1DAF9 */
  0x0020,   /* U+1DAFA */
  0x0020,   /* U+1DAFB */
  0x0020,   /* U+1DAFC */
  0x0020,   /* U+1DAFD */
  0x0020,   /* U+1DAFE */
  0x0020,   /* U+1DAFF */

  /* Tertiary weight 1 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x0002,   /* U+1DA37 */
  0x0002,   /* U+1DA38 */
  0x0002,   /* U+1DA39 */
  0x0002,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x0002,   /* U+1DA6D */
  0x0002,   /* U+1DA6E */
  0x0002,   /* U+1DA6F */
  0x0002,   /* U+1DA70 */
  0x0002,   /* U+1DA71 */
  0x0002,   /* U+1DA72 */
  0x0002,   /* U+1DA73 */
  0x0002,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x0002,   /* U+1DA76 */
  0x0002,   /* U+1DA77 */
  0x0002,   /* U+1DA78 */
  0x0002,   /* U+1DA79 */
  0x0002,   /* U+1DA7A */
  0x0002,   /* U+1DA7B */
  0x0002,   /* U+1DA7C */
  0x0002,   /* U+1DA7D */
  0x0002,   /* U+1DA7E */
  0x0002,   /* U+1DA7F */
  0x0002,   /* U+1DA80 */
  0x0002,   /* U+1DA81 */
  0x0002,   /* U+1DA82 */
  0x0002,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x0002,   /* U+1DA85 */
  0x0002,   /* U+1DA86 */
  0x0002,   /* U+1DA87 */
  0x0002,   /* U+1DA88 */
  0x0002,   /* U+1DA89 */
  0x0002,   /* U+1DA8A */
  0x0002,   /* U+1DA8B */
  0x0002,   /* U+1DA8C */
  0x0002,   /* U+1DA8D */
  0x0002,   /* U+1DA8E */
  0x0002,   /* U+1DA8F */
  0x0002,   /* U+1DA90 */
  0x0002,   /* U+1DA91 */
  0x0002,   /* U+1DA92 */
  0x0002,   /* U+1DA93 */
  0x0002,   /* U+1DA94 */
  0x0002,   /* U+1DA95 */
  0x0002,   /* U+1DA96 */
  0x0002,   /* U+1DA97 */
  0x0002,   /* U+1DA98 */
  0x0002,   /* U+1DA99 */
  0x0002,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0x0002,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0x0002,   /* U+1DAB0 */
  0x0002,   /* U+1DAB1 */
  0x0002,   /* U+1DAB2 */
  0x0002,   /* U+1DAB3 */
  0x0002,   /* U+1DAB4 */
  0x0002,   /* U+1DAB5 */
  0x0002,   /* U+1DAB6 */
  0x0002,   /* U+1DAB7 */
  0x0002,   /* U+1DAB8 */
  0x0002,   /* U+1DAB9 */
  0x0002,   /* U+1DABA */
  0x0002,   /* U+1DABB */
  0x0002,   /* U+1DABC */
  0x0002,   /* U+1DABD */
  0x0002,   /* U+1DABE */
  0x0002,   /* U+1DABF */
  0x0002,   /* U+1DAC0 */
  0x0002,   /* U+1DAC1 */
  0x0002,   /* U+1DAC2 */
  0x0002,   /* U+1DAC3 */
  0x0002,   /* U+1DAC4 */
  0x0002,   /* U+1DAC5 */
  0x0002,   /* U+1DAC6 */
  0x0002,   /* U+1DAC7 */
  0x0002,   /* U+1DAC8 */
  0x0002,   /* U+1DAC9 */
  0x0002,   /* U+1DACA */
  0x0002,   /* U+1DACB */
  0x0002,   /* U+1DACC */
  0x0002,   /* U+1DACD */
  0x0002,   /* U+1DACE */
  0x0002,   /* U+1DACF */
  0x0002,   /* U+1DAD0 */
  0x0002,   /* U+1DAD1 */
  0x0002,   /* U+1DAD2 */
  0x0002,   /* U+1DAD3 */
  0x0002,   /* U+1DAD4 */
  0x0002,   /* U+1DAD5 */
  0x0002,   /* U+1DAD6 */
  0x0002,   /* U+1DAD7 */
  0x0002,   /* U+1DAD8 */
  0x0002,   /* U+1DAD9 */
  0x0002,   /* U+1DADA */
  0x0002,   /* U+1DADB */
  0x0002,   /* U+1DADC */
  0x0002,   /* U+1DADD */
  0x0002,   /* U+1DADE */
  0x0002,   /* U+1DADF */
  0x0002,   /* U+1DAE0 */
  0x0002,   /* U+1DAE1 */
  0x0002,   /* U+1DAE2 */
  0x0002,   /* U+1DAE3 */
  0x0002,   /* U+1DAE4 */
  0x0002,   /* U+1DAE5 */
  0x0002,   /* U+1DAE6 */
  0x0002,   /* U+1DAE7 */
  0x0002,   /* U+1DAE8 */
  0x0002,   /* U+1DAE9 */
  0x0002,   /* U+1DAEA */
  0x0002,   /* U+1DAEB */
  0x0002,   /* U+1DAEC */
  0x0002,   /* U+1DAED */
  0x0002,   /* U+1DAEE */
  0x0002,   /* U+1DAEF */
  0x0002,   /* U+1DAF0 */
  0x0002,   /* U+1DAF1 */
  0x0002,   /* U+1DAF2 */
  0x0002,   /* U+1DAF3 */
  0x0002,   /* U+1DAF4 */
  0x0002,   /* U+1DAF5 */
  0x0002,   /* U+1DAF6 */
  0x0002,   /* U+1DAF7 */
  0x0002,   /* U+1DAF8 */
  0x0002,   /* U+1DAF9 */
  0x0002,   /* U+1DAFA */
  0x0002,   /* U+1DAFB */
  0x0002,   /* U+1DAFC */
  0x0002,   /* U+1DAFD */
  0x0002,   /* U+1DAFE */
  0x0002,   /* U+1DAFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x0000,   /* U+1DA37 */
  0x0000,   /* U+1DA38 */
  0x0000,   /* U+1DA39 */
  0x0000,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x0000,   /* U+1DA6D */
  0x0000,   /* U+1DA6E */
  0x0000,   /* U+1DA6F */
  0x0000,   /* U+1DA70 */
  0x0000,   /* U+1DA71 */
  0x0000,   /* U+1DA72 */
  0x0000,   /* U+1DA73 */
  0x0000,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x0000,   /* U+1DA76 */
  0x0000,   /* U+1DA77 */
  0x0000,   /* U+1DA78 */
  0x0000,   /* U+1DA79 */
  0x0000,   /* U+1DA7A */
  0x0000,   /* U+1DA7B */
  0x0000,   /* U+1DA7C */
  0x0000,   /* U+1DA7D */
  0x0000,   /* U+1DA7E */
  0x0000,   /* U+1DA7F */
  0x0000,   /* U+1DA80 */
  0x0000,   /* U+1DA81 */
  0x0000,   /* U+1DA82 */
  0x0000,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x0000,   /* U+1DA85 */
  0x0000,   /* U+1DA86 */
  0x0000,   /* U+1DA87 */
  0x0000,   /* U+1DA88 */
  0x0000,   /* U+1DA89 */
  0x0000,   /* U+1DA8A */
  0x0000,   /* U+1DA8B */
  0xDA8C,   /* U+1DA8C */
  0xDA8D,   /* U+1DA8D */
  0xDA8E,   /* U+1DA8E */
  0xDA8F,   /* U+1DA8F */
  0xDA90,   /* U+1DA90 */
  0xDA91,   /* U+1DA91 */
  0xDA92,   /* U+1DA92 */
  0xDA93,   /* U+1DA93 */
  0xDA94,   /* U+1DA94 */
  0xDA95,   /* U+1DA95 */
  0xDA96,   /* U+1DA96 */
  0xDA97,   /* U+1DA97 */
  0xDA98,   /* U+1DA98 */
  0xDA99,   /* U+1DA99 */
  0xDA9A,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0xDAA0,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0xDAB0,   /* U+1DAB0 */
  0xDAB1,   /* U+1DAB1 */
  0xDAB2,   /* U+1DAB2 */
  0xDAB3,   /* U+1DAB3 */
  0xDAB4,   /* U+1DAB4 */
  0xDAB5,   /* U+1DAB5 */
  0xDAB6,   /* U+1DAB6 */
  0xDAB7,   /* U+1DAB7 */
  0xDAB8,   /* U+1DAB8 */
  0xDAB9,   /* U+1DAB9 */
  0xDABA,   /* U+1DABA */
  0xDABB,   /* U+1DABB */
  0xDABC,   /* U+1DABC */
  0xDABD,   /* U+1DABD */
  0xDABE,   /* U+1DABE */
  0xDABF,   /* U+1DABF */
  0xDAC0,   /* U+1DAC0 */
  0xDAC1,   /* U+1DAC1 */
  0xDAC2,   /* U+1DAC2 */
  0xDAC3,   /* U+1DAC3 */
  0xDAC4,   /* U+1DAC4 */
  0xDAC5,   /* U+1DAC5 */
  0xDAC6,   /* U+1DAC6 */
  0xDAC7,   /* U+1DAC7 */
  0xDAC8,   /* U+1DAC8 */
  0xDAC9,   /* U+1DAC9 */
  0xDACA,   /* U+1DACA */
  0xDACB,   /* U+1DACB */
  0xDACC,   /* U+1DACC */
  0xDACD,   /* U+1DACD */
  0xDACE,   /* U+1DACE */
  0xDACF,   /* U+1DACF */
  0xDAD0,   /* U+1DAD0 */
  0xDAD1,   /* U+1DAD1 */
  0xDAD2,   /* U+1DAD2 */
  0xDAD3,   /* U+1DAD3 */
  0xDAD4,   /* U+1DAD4 */
  0xDAD5,   /* U+1DAD5 */
  0xDAD6,   /* U+1DAD6 */
  0xDAD7,   /* U+1DAD7 */
  0xDAD8,   /* U+1DAD8 */
  0xDAD9,   /* U+1DAD9 */
  0xDADA,   /* U+1DADA */
  0xDADB,   /* U+1DADB */
  0xDADC,   /* U+1DADC */
  0xDADD,   /* U+1DADD */
  0xDADE,   /* U+1DADE */
  0xDADF,   /* U+1DADF */
  0xDAE0,   /* U+1DAE0 */
  0xDAE1,   /* U+1DAE1 */
  0xDAE2,   /* U+1DAE2 */
  0xDAE3,   /* U+1DAE3 */
  0xDAE4,   /* U+1DAE4 */
  0xDAE5,   /* U+1DAE5 */
  0xDAE6,   /* U+1DAE6 */
  0xDAE7,   /* U+1DAE7 */
  0xDAE8,   /* U+1DAE8 */
  0xDAE9,   /* U+1DAE9 */
  0xDAEA,   /* U+1DAEA */
  0xDAEB,   /* U+1DAEB */
  0xDAEC,   /* U+1DAEC */
  0xDAED,   /* U+1DAED */
  0xDAEE,   /* U+1DAEE */
  0xDAEF,   /* U+1DAEF */
  0xDAF0,   /* U+1DAF0 */
  0xDAF1,   /* U+1DAF1 */
  0xDAF2,   /* U+1DAF2 */
  0xDAF3,   /* U+1DAF3 */
  0xDAF4,   /* U+1DAF4 */
  0xDAF5,   /* U+1DAF5 */
  0xDAF6,   /* U+1DAF6 */
  0xDAF7,   /* U+1DAF7 */
  0xDAF8,   /* U+1DAF8 */
  0xDAF9,   /* U+1DAF9 */
  0xDAFA,   /* U+1DAFA */
  0xDAFB,   /* U+1DAFB */
  0xDAFC,   /* U+1DAFC */
  0xDAFD,   /* U+1DAFD */
  0xDAFE,   /* U+1DAFE */
  0xDAFF,   /* U+1DAFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x0000,   /* U+1DA37 */
  0x0000,   /* U+1DA38 */
  0x0000,   /* U+1DA39 */
  0x0000,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x0000,   /* U+1DA6D */
  0x0000,   /* U+1DA6E */
  0x0000,   /* U+1DA6F */
  0x0000,   /* U+1DA70 */
  0x0000,   /* U+1DA71 */
  0x0000,   /* U+1DA72 */
  0x0000,   /* U+1DA73 */
  0x0000,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x0000,   /* U+1DA76 */
  0x0000,   /* U+1DA77 */
  0x0000,   /* U+1DA78 */
  0x0000,   /* U+1DA79 */
  0x0000,   /* U+1DA7A */
  0x0000,   /* U+1DA7B */
  0x0000,   /* U+1DA7C */
  0x0000,   /* U+1DA7D */
  0x0000,   /* U+1DA7E */
  0x0000,   /* U+1DA7F */
  0x0000,   /* U+1DA80 */
  0x0000,   /* U+1DA81 */
  0x0000,   /* U+1DA82 */
  0x0000,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x0000,   /* U+1DA85 */
  0x0000,   /* U+1DA86 */
  0x0000,   /* U+1DA87 */
  0x0000,   /* U+1DA88 */
  0x0000,   /* U+1DA89 */
  0x0000,   /* U+1DA8A */
  0x0000,   /* U+1DA8B */
  0x0000,   /* U+1DA8C */
  0x0000,   /* U+1DA8D */
  0x0000,   /* U+1DA8E */
  0x0000,   /* U+1DA8F */
  0x0000,   /* U+1DA90 */
  0x0000,   /* U+1DA91 */
  0x0000,   /* U+1DA92 */
  0x0000,   /* U+1DA93 */
  0x0000,   /* U+1DA94 */
  0x0000,   /* U+1DA95 */
  0x0000,   /* U+1DA96 */
  0x0000,   /* U+1DA97 */
  0x0000,   /* U+1DA98 */
  0x0000,   /* U+1DA99 */
  0x0000,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0x0000,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0x0000,   /* U+1DAB0 */
  0x0000,   /* U+1DAB1 */
  0x0000,   /* U+1DAB2 */
  0x0000,   /* U+1DAB3 */
  0x0000,   /* U+1DAB4 */
  0x0000,   /* U+1DAB5 */
  0x0000,   /* U+1DAB6 */
  0x0000,   /* U+1DAB7 */
  0x0000,   /* U+1DAB8 */
  0x0000,   /* U+1DAB9 */
  0x0000,   /* U+1DABA */
  0x0000,   /* U+1DABB */
  0x0000,   /* U+1DABC */
  0x0000,   /* U+1DABD */
  0x0000,   /* U+1DABE */
  0x0000,   /* U+1DABF */
  0x0000,   /* U+1DAC0 */
  0x0000,   /* U+1DAC1 */
  0x0000,   /* U+1DAC2 */
  0x0000,   /* U+1DAC3 */
  0x0000,   /* U+1DAC4 */
  0x0000,   /* U+1DAC5 */
  0x0000,   /* U+1DAC6 */
  0x0000,   /* U+1DAC7 */
  0x0000,   /* U+1DAC8 */
  0x0000,   /* U+1DAC9 */
  0x0000,   /* U+1DACA */
  0x0000,   /* U+1DACB */
  0x0000,   /* U+1DACC */
  0x0000,   /* U+1DACD */
  0x0000,   /* U+1DACE */
  0x0000,   /* U+1DACF */
  0x0000,   /* U+1DAD0 */
  0x0000,   /* U+1DAD1 */
  0x0000,   /* U+1DAD2 */
  0x0000,   /* U+1DAD3 */
  0x0000,   /* U+1DAD4 */
  0x0000,   /* U+1DAD5 */
  0x0000,   /* U+1DAD6 */
  0x0000,   /* U+1DAD7 */
  0x0000,   /* U+1DAD8 */
  0x0000,   /* U+1DAD9 */
  0x0000,   /* U+1DADA */
  0x0000,   /* U+1DADB */
  0x0000,   /* U+1DADC */
  0x0000,   /* U+1DADD */
  0x0000,   /* U+1DADE */
  0x0000,   /* U+1DADF */
  0x0000,   /* U+1DAE0 */
  0x0000,   /* U+1DAE1 */
  0x0000,   /* U+1DAE2 */
  0x0000,   /* U+1DAE3 */
  0x0000,   /* U+1DAE4 */
  0x0000,   /* U+1DAE5 */
  0x0000,   /* U+1DAE6 */
  0x0000,   /* U+1DAE7 */
  0x0000,   /* U+1DAE8 */
  0x0000,   /* U+1DAE9 */
  0x0000,   /* U+1DAEA */
  0x0000,   /* U+1DAEB */
  0x0000,   /* U+1DAEC */
  0x0000,   /* U+1DAED */
  0x0000,   /* U+1DAEE */
  0x0000,   /* U+1DAEF */
  0x0000,   /* U+1DAF0 */
  0x0000,   /* U+1DAF1 */
  0x0000,   /* U+1DAF2 */
  0x0000,   /* U+1DAF3 */
  0x0000,   /* U+1DAF4 */
  0x0000,   /* U+1DAF5 */
  0x0000,   /* U+1DAF6 */
  0x0000,   /* U+1DAF7 */
  0x0000,   /* U+1DAF8 */
  0x0000,   /* U+1DAF9 */
  0x0000,   /* U+1DAFA */
  0x0000,   /* U+1DAFB */
  0x0000,   /* U+1DAFC */
  0x0000,   /* U+1DAFD */
  0x0000,   /* U+1DAFE */
  0x0000,   /* U+1DAFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1DA00 */
  0x0000,   /* U+1DA01 */
  0x0000,   /* U+1DA02 */
  0x0000,   /* U+1DA03 */
  0x0000,   /* U+1DA04 */
  0x0000,   /* U+1DA05 */
  0x0000,   /* U+1DA06 */
  0x0000,   /* U+1DA07 */
  0x0000,   /* U+1DA08 */
  0x0000,   /* U+1DA09 */
  0x0000,   /* U+1DA0A */
  0x0000,   /* U+1DA0B */
  0x0000,   /* U+1DA0C */
  0x0000,   /* U+1DA0D */
  0x0000,   /* U+1DA0E */
  0x0000,   /* U+1DA0F */
  0x0000,   /* U+1DA10 */
  0x0000,   /* U+1DA11 */
  0x0000,   /* U+1DA12 */
  0x0000,   /* U+1DA13 */
  0x0000,   /* U+1DA14 */
  0x0000,   /* U+1DA15 */
  0x0000,   /* U+1DA16 */
  0x0000,   /* U+1DA17 */
  0x0000,   /* U+1DA18 */
  0x0000,   /* U+1DA19 */
  0x0000,   /* U+1DA1A */
  0x0000,   /* U+1DA1B */
  0x0000,   /* U+1DA1C */
  0x0000,   /* U+1DA1D */
  0x0000,   /* U+1DA1E */
  0x0000,   /* U+1DA1F */
  0x0000,   /* U+1DA20 */
  0x0000,   /* U+1DA21 */
  0x0000,   /* U+1DA22 */
  0x0000,   /* U+1DA23 */
  0x0000,   /* U+1DA24 */
  0x0000,   /* U+1DA25 */
  0x0000,   /* U+1DA26 */
  0x0000,   /* U+1DA27 */
  0x0000,   /* U+1DA28 */
  0x0000,   /* U+1DA29 */
  0x0000,   /* U+1DA2A */
  0x0000,   /* U+1DA2B */
  0x0000,   /* U+1DA2C */
  0x0000,   /* U+1DA2D */
  0x0000,   /* U+1DA2E */
  0x0000,   /* U+1DA2F */
  0x0000,   /* U+1DA30 */
  0x0000,   /* U+1DA31 */
  0x0000,   /* U+1DA32 */
  0x0000,   /* U+1DA33 */
  0x0000,   /* U+1DA34 */
  0x0000,   /* U+1DA35 */
  0x0000,   /* U+1DA36 */
  0x0000,   /* U+1DA37 */
  0x0000,   /* U+1DA38 */
  0x0000,   /* U+1DA39 */
  0x0000,   /* U+1DA3A */
  0x0000,   /* U+1DA3B */
  0x0000,   /* U+1DA3C */
  0x0000,   /* U+1DA3D */
  0x0000,   /* U+1DA3E */
  0x0000,   /* U+1DA3F */
  0x0000,   /* U+1DA40 */
  0x0000,   /* U+1DA41 */
  0x0000,   /* U+1DA42 */
  0x0000,   /* U+1DA43 */
  0x0000,   /* U+1DA44 */
  0x0000,   /* U+1DA45 */
  0x0000,   /* U+1DA46 */
  0x0000,   /* U+1DA47 */
  0x0000,   /* U+1DA48 */
  0x0000,   /* U+1DA49 */
  0x0000,   /* U+1DA4A */
  0x0000,   /* U+1DA4B */
  0x0000,   /* U+1DA4C */
  0x0000,   /* U+1DA4D */
  0x0000,   /* U+1DA4E */
  0x0000,   /* U+1DA4F */
  0x0000,   /* U+1DA50 */
  0x0000,   /* U+1DA51 */
  0x0000,   /* U+1DA52 */
  0x0000,   /* U+1DA53 */
  0x0000,   /* U+1DA54 */
  0x0000,   /* U+1DA55 */
  0x0000,   /* U+1DA56 */
  0x0000,   /* U+1DA57 */
  0x0000,   /* U+1DA58 */
  0x0000,   /* U+1DA59 */
  0x0000,   /* U+1DA5A */
  0x0000,   /* U+1DA5B */
  0x0000,   /* U+1DA5C */
  0x0000,   /* U+1DA5D */
  0x0000,   /* U+1DA5E */
  0x0000,   /* U+1DA5F */
  0x0000,   /* U+1DA60 */
  0x0000,   /* U+1DA61 */
  0x0000,   /* U+1DA62 */
  0x0000,   /* U+1DA63 */
  0x0000,   /* U+1DA64 */
  0x0000,   /* U+1DA65 */
  0x0000,   /* U+1DA66 */
  0x0000,   /* U+1DA67 */
  0x0000,   /* U+1DA68 */
  0x0000,   /* U+1DA69 */
  0x0000,   /* U+1DA6A */
  0x0000,   /* U+1DA6B */
  0x0000,   /* U+1DA6C */
  0x0000,   /* U+1DA6D */
  0x0000,   /* U+1DA6E */
  0x0000,   /* U+1DA6F */
  0x0000,   /* U+1DA70 */
  0x0000,   /* U+1DA71 */
  0x0000,   /* U+1DA72 */
  0x0000,   /* U+1DA73 */
  0x0000,   /* U+1DA74 */
  0x0000,   /* U+1DA75 */
  0x0000,   /* U+1DA76 */
  0x0000,   /* U+1DA77 */
  0x0000,   /* U+1DA78 */
  0x0000,   /* U+1DA79 */
  0x0000,   /* U+1DA7A */
  0x0000,   /* U+1DA7B */
  0x0000,   /* U+1DA7C */
  0x0000,   /* U+1DA7D */
  0x0000,   /* U+1DA7E */
  0x0000,   /* U+1DA7F */
  0x0000,   /* U+1DA80 */
  0x0000,   /* U+1DA81 */
  0x0000,   /* U+1DA82 */
  0x0000,   /* U+1DA83 */
  0x0000,   /* U+1DA84 */
  0x0000,   /* U+1DA85 */
  0x0000,   /* U+1DA86 */
  0x0000,   /* U+1DA87 */
  0x0000,   /* U+1DA88 */
  0x0000,   /* U+1DA89 */
  0x0000,   /* U+1DA8A */
  0x0000,   /* U+1DA8B */
  0x0000,   /* U+1DA8C */
  0x0000,   /* U+1DA8D */
  0x0000,   /* U+1DA8E */
  0x0000,   /* U+1DA8F */
  0x0000,   /* U+1DA90 */
  0x0000,   /* U+1DA91 */
  0x0000,   /* U+1DA92 */
  0x0000,   /* U+1DA93 */
  0x0000,   /* U+1DA94 */
  0x0000,   /* U+1DA95 */
  0x0000,   /* U+1DA96 */
  0x0000,   /* U+1DA97 */
  0x0000,   /* U+1DA98 */
  0x0000,   /* U+1DA99 */
  0x0000,   /* U+1DA9A */
  0x0000,   /* U+1DA9B */
  0x0000,   /* U+1DA9C */
  0x0000,   /* U+1DA9D */
  0x0000,   /* U+1DA9E */
  0x0000,   /* U+1DA9F */
  0x0000,   /* U+1DAA0 */
  0x0000,   /* U+1DAA1 */
  0x0000,   /* U+1DAA2 */
  0x0000,   /* U+1DAA3 */
  0x0000,   /* U+1DAA4 */
  0x0000,   /* U+1DAA5 */
  0x0000,   /* U+1DAA6 */
  0x0000,   /* U+1DAA7 */
  0x0000,   /* U+1DAA8 */
  0x0000,   /* U+1DAA9 */
  0x0000,   /* U+1DAAA */
  0x0000,   /* U+1DAAB */
  0x0000,   /* U+1DAAC */
  0x0000,   /* U+1DAAD */
  0x0000,   /* U+1DAAE */
  0x0000,   /* U+1DAAF */
  0x0000,   /* U+1DAB0 */
  0x0000,   /* U+1DAB1 */
  0x0000,   /* U+1DAB2 */
  0x0000,   /* U+1DAB3 */
  0x0000,   /* U+1DAB4 */
  0x0000,   /* U+1DAB5 */
  0x0000,   /* U+1DAB6 */
  0x0000,   /* U+1DAB7 */
  0x0000,   /* U+1DAB8 */
  0x0000,   /* U+1DAB9 */
  0x0000,   /* U+1DABA */
  0x0000,   /* U+1DABB */
  0x0000,   /* U+1DABC */
  0x0000,   /* U+1DABD */
  0x0000,   /* U+1DABE */
  0x0000,   /* U+1DABF */
  0x0000,   /* U+1DAC0 */
  0x0000,   /* U+1DAC1 */
  0x0000,   /* U+1DAC2 */
  0x0000,   /* U+1DAC3 */
  0x0000,   /* U+1DAC4 */
  0x0000,   /* U+1DAC5 */
  0x0000,   /* U+1DAC6 */
  0x0000,   /* U+1DAC7 */
  0x0000,   /* U+1DAC8 */
  0x0000,   /* U+1DAC9 */
  0x0000,   /* U+1DACA */
  0x0000,   /* U+1DACB */
  0x0000,   /* U+1DACC */
  0x0000,   /* U+1DACD */
  0x0000,   /* U+1DACE */
  0x0000,   /* U+1DACF */
  0x0000,   /* U+1DAD0 */
  0x0000,   /* U+1DAD1 */
  0x0000,   /* U+1DAD2 */
  0x0000,   /* U+1DAD3 */
  0x0000,   /* U+1DAD4 */
  0x0000,   /* U+1DAD5 */
  0x0000,   /* U+1DAD6 */
  0x0000,   /* U+1DAD7 */
  0x0000,   /* U+1DAD8 */
  0x0000,   /* U+1DAD9 */
  0x0000,   /* U+1DADA */
  0x0000,   /* U+1DADB */
  0x0000,   /* U+1DADC */
  0x0000,   /* U+1DADD */
  0x0000,   /* U+1DADE */
  0x0000,   /* U+1DADF */
  0x0000,   /* U+1DAE0 */
  0x0000,   /* U+1DAE1 */
  0x0000,   /* U+1DAE2 */
  0x0000,   /* U+1DAE3 */
  0x0000,   /* U+1DAE4 */
  0x0000,   /* U+1DAE5 */
  0x0000,   /* U+1DAE6 */
  0x0000,   /* U+1DAE7 */
  0x0000,   /* U+1DAE8 */
  0x0000,   /* U+1DAE9 */
  0x0000,   /* U+1DAEA */
  0x0000,   /* U+1DAEB */
  0x0000,   /* U+1DAEC */
  0x0000,   /* U+1DAED */
  0x0000,   /* U+1DAEE */
  0x0000,   /* U+1DAEF */
  0x0000,   /* U+1DAF0 */
  0x0000,   /* U+1DAF1 */
  0x0000,   /* U+1DAF2 */
  0x0000,   /* U+1DAF3 */
  0x0000,   /* U+1DAF4 */
  0x0000,   /* U+1DAF5 */
  0x0000,   /* U+1DAF6 */
  0x0000,   /* U+1DAF7 */
  0x0000,   /* U+1DAF8 */
  0x0000,   /* U+1DAF9 */
  0x0000,   /* U+1DAFA */
  0x0000,   /* U+1DAFB */
  0x0000,   /* U+1DAFC */
  0x0000,   /* U+1DAFD */
  0x0000,   /* U+1DAFE */
  0x0000,   /* U+1DAFF */
};

uint16 uca900_p1E0[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 
  1, 1, 2, 1, 1, 2, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x21E5,   /* U+1E000 */
  0x21E6,   /* U+1E001 */
  0x21E7,   /* U+1E002 */
  0x21E8,   /* U+1E003 */
  0x21E9,   /* U+1E004 */
  0x21EA,   /* U+1E005 */
  0x21EB,   /* U+1E006 */
  0xFBC3,   /* U+1E007 */
  0x21ED,   /* U+1E008 */
  0x21EE,   /* U+1E009 */
  0x21EF,   /* U+1E00A */
  0x21F0,   /* U+1E00B */
  0x21F1,   /* U+1E00C */
  0x21F2,   /* U+1E00D */
  0x21F3,   /* U+1E00E */
  0x21F4,   /* U+1E00F */
  0x21F5,   /* U+1E010 */
  0x21F6,   /* U+1E011 */
  0x21F7,   /* U+1E012 */
  0x21F8,   /* U+1E013 */
  0x21F9,   /* U+1E014 */
  0x21FA,   /* U+1E015 */
  0x21FB,   /* U+1E016 */
  0x21FC,   /* U+1E017 */
  0x21FD,   /* U+1E018 */
  0xFBC3,   /* U+1E019 */
  0xFBC3,   /* U+1E01A */
  0x2200,   /* U+1E01B */
  0x2201,   /* U+1E01C */
  0x2202,   /* U+1E01D */
  0x2203,   /* U+1E01E */
  0x2204,   /* U+1E01F */
  0x2205,   /* U+1E020 */
  0x2206,   /* U+1E021 */
  0xFBC3,   /* U+1E022 */
  0x2208,   /* U+1E023 */
  0x2209,   /* U+1E024 */
  0xFBC3,   /* U+1E025 */
  0x220B,   /* U+1E026 */
  0x220C,   /* U+1E027 */
  0x220D,   /* U+1E028 */
  0x220E,   /* U+1E029 */
  0x220F,   /* U+1E02A */
  0xFBC3,   /* U+1E02B */
  0xFBC3,   /* U+1E02C */
  0xFBC3,   /* U+1E02D */
  0xFBC3,   /* U+1E02E */
  0xFBC3,   /* U+1E02F */
  0xFBC3,   /* U+1E030 */
  0xFBC3,   /* U+1E031 */
  0xFBC3,   /* U+1E032 */
  0xFBC3,   /* U+1E033 */
  0xFBC3,   /* U+1E034 */
  0xFBC3,   /* U+1E035 */
  0xFBC3,   /* U+1E036 */
  0xFBC3,   /* U+1E037 */
  0xFBC3,   /* U+1E038 */
  0xFBC3,   /* U+1E039 */
  0xFBC3,   /* U+1E03A */
  0xFBC3,   /* U+1E03B */
  0xFBC3,   /* U+1E03C */
  0xFBC3,   /* U+1E03D */
  0xFBC3,   /* U+1E03E */
  0xFBC3,   /* U+1E03F */
  0xFBC3,   /* U+1E040 */
  0xFBC3,   /* U+1E041 */
  0xFBC3,   /* U+1E042 */
  0xFBC3,   /* U+1E043 */
  0xFBC3,   /* U+1E044 */
  0xFBC3,   /* U+1E045 */
  0xFBC3,   /* U+1E046 */
  0xFBC3,   /* U+1E047 */
  0xFBC3,   /* U+1E048 */
  0xFBC3,   /* U+1E049 */
  0xFBC3,   /* U+1E04A */
  0xFBC3,   /* U+1E04B */
  0xFBC3,   /* U+1E04C */
  0xFBC3,   /* U+1E04D */
  0xFBC3,   /* U+1E04E */
  0xFBC3,   /* U+1E04F */
  0xFBC3,   /* U+1E050 */
  0xFBC3,   /* U+1E051 */
  0xFBC3,   /* U+1E052 */
  0xFBC3,   /* U+1E053 */
  0xFBC3,   /* U+1E054 */
  0xFBC3,   /* U+1E055 */
  0xFBC3,   /* U+1E056 */
  0xFBC3,   /* U+1E057 */
  0xFBC3,   /* U+1E058 */
  0xFBC3,   /* U+1E059 */
  0xFBC3,   /* U+1E05A */
  0xFBC3,   /* U+1E05B */
  0xFBC3,   /* U+1E05C */
  0xFBC3,   /* U+1E05D */
  0xFBC3,   /* U+1E05E */
  0xFBC3,   /* U+1E05F */
  0xFBC3,   /* U+1E060 */
  0xFBC3,   /* U+1E061 */
  0xFBC3,   /* U+1E062 */
  0xFBC3,   /* U+1E063 */
  0xFBC3,   /* U+1E064 */
  0xFBC3,   /* U+1E065 */
  0xFBC3,   /* U+1E066 */
  0xFBC3,   /* U+1E067 */
  0xFBC3,   /* U+1E068 */
  0xFBC3,   /* U+1E069 */
  0xFBC3,   /* U+1E06A */
  0xFBC3,   /* U+1E06B */
  0xFBC3,   /* U+1E06C */
  0xFBC3,   /* U+1E06D */
  0xFBC3,   /* U+1E06E */
  0xFBC3,   /* U+1E06F */
  0xFBC3,   /* U+1E070 */
  0xFBC3,   /* U+1E071 */
  0xFBC3,   /* U+1E072 */
  0xFBC3,   /* U+1E073 */
  0xFBC3,   /* U+1E074 */
  0xFBC3,   /* U+1E075 */
  0xFBC3,   /* U+1E076 */
  0xFBC3,   /* U+1E077 */
  0xFBC3,   /* U+1E078 */
  0xFBC3,   /* U+1E079 */
  0xFBC3,   /* U+1E07A */
  0xFBC3,   /* U+1E07B */
  0xFBC3,   /* U+1E07C */
  0xFBC3,   /* U+1E07D */
  0xFBC3,   /* U+1E07E */
  0xFBC3,   /* U+1E07F */
  0xFBC3,   /* U+1E080 */
  0xFBC3,   /* U+1E081 */
  0xFBC3,   /* U+1E082 */
  0xFBC3,   /* U+1E083 */
  0xFBC3,   /* U+1E084 */
  0xFBC3,   /* U+1E085 */
  0xFBC3,   /* U+1E086 */
  0xFBC3,   /* U+1E087 */
  0xFBC3,   /* U+1E088 */
  0xFBC3,   /* U+1E089 */
  0xFBC3,   /* U+1E08A */
  0xFBC3,   /* U+1E08B */
  0xFBC3,   /* U+1E08C */
  0xFBC3,   /* U+1E08D */
  0xFBC3,   /* U+1E08E */
  0xFBC3,   /* U+1E08F */
  0xFBC3,   /* U+1E090 */
  0xFBC3,   /* U+1E091 */
  0xFBC3,   /* U+1E092 */
  0xFBC3,   /* U+1E093 */
  0xFBC3,   /* U+1E094 */
  0xFBC3,   /* U+1E095 */
  0xFBC3,   /* U+1E096 */
  0xFBC3,   /* U+1E097 */
  0xFBC3,   /* U+1E098 */
  0xFBC3,   /* U+1E099 */
  0xFBC3,   /* U+1E09A */
  0xFBC3,   /* U+1E09B */
  0xFBC3,   /* U+1E09C */
  0xFBC3,   /* U+1E09D */
  0xFBC3,   /* U+1E09E */
  0xFBC3,   /* U+1E09F */
  0xFBC3,   /* U+1E0A0 */
  0xFBC3,   /* U+1E0A1 */
  0xFBC3,   /* U+1E0A2 */
  0xFBC3,   /* U+1E0A3 */
  0xFBC3,   /* U+1E0A4 */
  0xFBC3,   /* U+1E0A5 */
  0xFBC3,   /* U+1E0A6 */
  0xFBC3,   /* U+1E0A7 */
  0xFBC3,   /* U+1E0A8 */
  0xFBC3,   /* U+1E0A9 */
  0xFBC3,   /* U+1E0AA */
  0xFBC3,   /* U+1E0AB */
  0xFBC3,   /* U+1E0AC */
  0xFBC3,   /* U+1E0AD */
  0xFBC3,   /* U+1E0AE */
  0xFBC3,   /* U+1E0AF */
  0xFBC3,   /* U+1E0B0 */
  0xFBC3,   /* U+1E0B1 */
  0xFBC3,   /* U+1E0B2 */
  0xFBC3,   /* U+1E0B3 */
  0xFBC3,   /* U+1E0B4 */
  0xFBC3,   /* U+1E0B5 */
  0xFBC3,   /* U+1E0B6 */
  0xFBC3,   /* U+1E0B7 */
  0xFBC3,   /* U+1E0B8 */
  0xFBC3,   /* U+1E0B9 */
  0xFBC3,   /* U+1E0BA */
  0xFBC3,   /* U+1E0BB */
  0xFBC3,   /* U+1E0BC */
  0xFBC3,   /* U+1E0BD */
  0xFBC3,   /* U+1E0BE */
  0xFBC3,   /* U+1E0BF */
  0xFBC3,   /* U+1E0C0 */
  0xFBC3,   /* U+1E0C1 */
  0xFBC3,   /* U+1E0C2 */
  0xFBC3,   /* U+1E0C3 */
  0xFBC3,   /* U+1E0C4 */
  0xFBC3,   /* U+1E0C5 */
  0xFBC3,   /* U+1E0C6 */
  0xFBC3,   /* U+1E0C7 */
  0xFBC3,   /* U+1E0C8 */
  0xFBC3,   /* U+1E0C9 */
  0xFBC3,   /* U+1E0CA */
  0xFBC3,   /* U+1E0CB */
  0xFBC3,   /* U+1E0CC */
  0xFBC3,   /* U+1E0CD */
  0xFBC3,   /* U+1E0CE */
  0xFBC3,   /* U+1E0CF */
  0xFBC3,   /* U+1E0D0 */
  0xFBC3,   /* U+1E0D1 */
  0xFBC3,   /* U+1E0D2 */
  0xFBC3,   /* U+1E0D3 */
  0xFBC3,   /* U+1E0D4 */
  0xFBC3,   /* U+1E0D5 */
  0xFBC3,   /* U+1E0D6 */
  0xFBC3,   /* U+1E0D7 */
  0xFBC3,   /* U+1E0D8 */
  0xFBC3,   /* U+1E0D9 */
  0xFBC3,   /* U+1E0DA */
  0xFBC3,   /* U+1E0DB */
  0xFBC3,   /* U+1E0DC */
  0xFBC3,   /* U+1E0DD */
  0xFBC3,   /* U+1E0DE */
  0xFBC3,   /* U+1E0DF */
  0xFBC3,   /* U+1E0E0 */
  0xFBC3,   /* U+1E0E1 */
  0xFBC3,   /* U+1E0E2 */
  0xFBC3,   /* U+1E0E3 */
  0xFBC3,   /* U+1E0E4 */
  0xFBC3,   /* U+1E0E5 */
  0xFBC3,   /* U+1E0E6 */
  0xFBC3,   /* U+1E0E7 */
  0xFBC3,   /* U+1E0E8 */
  0xFBC3,   /* U+1E0E9 */
  0xFBC3,   /* U+1E0EA */
  0xFBC3,   /* U+1E0EB */
  0xFBC3,   /* U+1E0EC */
  0xFBC3,   /* U+1E0ED */
  0xFBC3,   /* U+1E0EE */
  0xFBC3,   /* U+1E0EF */
  0xFBC3,   /* U+1E0F0 */
  0xFBC3,   /* U+1E0F1 */
  0xFBC3,   /* U+1E0F2 */
  0xFBC3,   /* U+1E0F3 */
  0xFBC3,   /* U+1E0F4 */
  0xFBC3,   /* U+1E0F5 */
  0xFBC3,   /* U+1E0F6 */
  0xFBC3,   /* U+1E0F7 */
  0xFBC3,   /* U+1E0F8 */
  0xFBC3,   /* U+1E0F9 */
  0xFBC3,   /* U+1E0FA */
  0xFBC3,   /* U+1E0FB */
  0xFBC3,   /* U+1E0FC */
  0xFBC3,   /* U+1E0FD */
  0xFBC3,   /* U+1E0FE */
  0xFBC3,   /* U+1E0FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1E000 */
  0x0020,   /* U+1E001 */
  0x0020,   /* U+1E002 */
  0x0020,   /* U+1E003 */
  0x0020,   /* U+1E004 */
  0x0020,   /* U+1E005 */
  0x0020,   /* U+1E006 */
  0x0020,   /* U+1E007 */
  0x0020,   /* U+1E008 */
  0x0020,   /* U+1E009 */
  0x0020,   /* U+1E00A */
  0x0020,   /* U+1E00B */
  0x0020,   /* U+1E00C */
  0x0020,   /* U+1E00D */
  0x0020,   /* U+1E00E */
  0x0020,   /* U+1E00F */
  0x0020,   /* U+1E010 */
  0x0020,   /* U+1E011 */
  0x0020,   /* U+1E012 */
  0x0020,   /* U+1E013 */
  0x0020,   /* U+1E014 */
  0x0020,   /* U+1E015 */
  0x0020,   /* U+1E016 */
  0x0020,   /* U+1E017 */
  0x0020,   /* U+1E018 */
  0x0020,   /* U+1E019 */
  0x0020,   /* U+1E01A */
  0x0020,   /* U+1E01B */
  0x0020,   /* U+1E01C */
  0x0020,   /* U+1E01D */
  0x0020,   /* U+1E01E */
  0x0020,   /* U+1E01F */
  0x0020,   /* U+1E020 */
  0x0020,   /* U+1E021 */
  0x0020,   /* U+1E022 */
  0x0020,   /* U+1E023 */
  0x0020,   /* U+1E024 */
  0x0020,   /* U+1E025 */
  0x0020,   /* U+1E026 */
  0x0020,   /* U+1E027 */
  0x0020,   /* U+1E028 */
  0x0020,   /* U+1E029 */
  0x0020,   /* U+1E02A */
  0x0020,   /* U+1E02B */
  0x0020,   /* U+1E02C */
  0x0020,   /* U+1E02D */
  0x0020,   /* U+1E02E */
  0x0020,   /* U+1E02F */
  0x0020,   /* U+1E030 */
  0x0020,   /* U+1E031 */
  0x0020,   /* U+1E032 */
  0x0020,   /* U+1E033 */
  0x0020,   /* U+1E034 */
  0x0020,   /* U+1E035 */
  0x0020,   /* U+1E036 */
  0x0020,   /* U+1E037 */
  0x0020,   /* U+1E038 */
  0x0020,   /* U+1E039 */
  0x0020,   /* U+1E03A */
  0x0020,   /* U+1E03B */
  0x0020,   /* U+1E03C */
  0x0020,   /* U+1E03D */
  0x0020,   /* U+1E03E */
  0x0020,   /* U+1E03F */
  0x0020,   /* U+1E040 */
  0x0020,   /* U+1E041 */
  0x0020,   /* U+1E042 */
  0x0020,   /* U+1E043 */
  0x0020,   /* U+1E044 */
  0x0020,   /* U+1E045 */
  0x0020,   /* U+1E046 */
  0x0020,   /* U+1E047 */
  0x0020,   /* U+1E048 */
  0x0020,   /* U+1E049 */
  0x0020,   /* U+1E04A */
  0x0020,   /* U+1E04B */
  0x0020,   /* U+1E04C */
  0x0020,   /* U+1E04D */
  0x0020,   /* U+1E04E */
  0x0020,   /* U+1E04F */
  0x0020,   /* U+1E050 */
  0x0020,   /* U+1E051 */
  0x0020,   /* U+1E052 */
  0x0020,   /* U+1E053 */
  0x0020,   /* U+1E054 */
  0x0020,   /* U+1E055 */
  0x0020,   /* U+1E056 */
  0x0020,   /* U+1E057 */
  0x0020,   /* U+1E058 */
  0x0020,   /* U+1E059 */
  0x0020,   /* U+1E05A */
  0x0020,   /* U+1E05B */
  0x0020,   /* U+1E05C */
  0x0020,   /* U+1E05D */
  0x0020,   /* U+1E05E */
  0x0020,   /* U+1E05F */
  0x0020,   /* U+1E060 */
  0x0020,   /* U+1E061 */
  0x0020,   /* U+1E062 */
  0x0020,   /* U+1E063 */
  0x0020,   /* U+1E064 */
  0x0020,   /* U+1E065 */
  0x0020,   /* U+1E066 */
  0x0020,   /* U+1E067 */
  0x0020,   /* U+1E068 */
  0x0020,   /* U+1E069 */
  0x0020,   /* U+1E06A */
  0x0020,   /* U+1E06B */
  0x0020,   /* U+1E06C */
  0x0020,   /* U+1E06D */
  0x0020,   /* U+1E06E */
  0x0020,   /* U+1E06F */
  0x0020,   /* U+1E070 */
  0x0020,   /* U+1E071 */
  0x0020,   /* U+1E072 */
  0x0020,   /* U+1E073 */
  0x0020,   /* U+1E074 */
  0x0020,   /* U+1E075 */
  0x0020,   /* U+1E076 */
  0x0020,   /* U+1E077 */
  0x0020,   /* U+1E078 */
  0x0020,   /* U+1E079 */
  0x0020,   /* U+1E07A */
  0x0020,   /* U+1E07B */
  0x0020,   /* U+1E07C */
  0x0020,   /* U+1E07D */
  0x0020,   /* U+1E07E */
  0x0020,   /* U+1E07F */
  0x0020,   /* U+1E080 */
  0x0020,   /* U+1E081 */
  0x0020,   /* U+1E082 */
  0x0020,   /* U+1E083 */
  0x0020,   /* U+1E084 */
  0x0020,   /* U+1E085 */
  0x0020,   /* U+1E086 */
  0x0020,   /* U+1E087 */
  0x0020,   /* U+1E088 */
  0x0020,   /* U+1E089 */
  0x0020,   /* U+1E08A */
  0x0020,   /* U+1E08B */
  0x0020,   /* U+1E08C */
  0x0020,   /* U+1E08D */
  0x0020,   /* U+1E08E */
  0x0020,   /* U+1E08F */
  0x0020,   /* U+1E090 */
  0x0020,   /* U+1E091 */
  0x0020,   /* U+1E092 */
  0x0020,   /* U+1E093 */
  0x0020,   /* U+1E094 */
  0x0020,   /* U+1E095 */
  0x0020,   /* U+1E096 */
  0x0020,   /* U+1E097 */
  0x0020,   /* U+1E098 */
  0x0020,   /* U+1E099 */
  0x0020,   /* U+1E09A */
  0x0020,   /* U+1E09B */
  0x0020,   /* U+1E09C */
  0x0020,   /* U+1E09D */
  0x0020,   /* U+1E09E */
  0x0020,   /* U+1E09F */
  0x0020,   /* U+1E0A0 */
  0x0020,   /* U+1E0A1 */
  0x0020,   /* U+1E0A2 */
  0x0020,   /* U+1E0A3 */
  0x0020,   /* U+1E0A4 */
  0x0020,   /* U+1E0A5 */
  0x0020,   /* U+1E0A6 */
  0x0020,   /* U+1E0A7 */
  0x0020,   /* U+1E0A8 */
  0x0020,   /* U+1E0A9 */
  0x0020,   /* U+1E0AA */
  0x0020,   /* U+1E0AB */
  0x0020,   /* U+1E0AC */
  0x0020,   /* U+1E0AD */
  0x0020,   /* U+1E0AE */
  0x0020,   /* U+1E0AF */
  0x0020,   /* U+1E0B0 */
  0x0020,   /* U+1E0B1 */
  0x0020,   /* U+1E0B2 */
  0x0020,   /* U+1E0B3 */
  0x0020,   /* U+1E0B4 */
  0x0020,   /* U+1E0B5 */
  0x0020,   /* U+1E0B6 */
  0x0020,   /* U+1E0B7 */
  0x0020,   /* U+1E0B8 */
  0x0020,   /* U+1E0B9 */
  0x0020,   /* U+1E0BA */
  0x0020,   /* U+1E0BB */
  0x0020,   /* U+1E0BC */
  0x0020,   /* U+1E0BD */
  0x0020,   /* U+1E0BE */
  0x0020,   /* U+1E0BF */
  0x0020,   /* U+1E0C0 */
  0x0020,   /* U+1E0C1 */
  0x0020,   /* U+1E0C2 */
  0x0020,   /* U+1E0C3 */
  0x0020,   /* U+1E0C4 */
  0x0020,   /* U+1E0C5 */
  0x0020,   /* U+1E0C6 */
  0x0020,   /* U+1E0C7 */
  0x0020,   /* U+1E0C8 */
  0x0020,   /* U+1E0C9 */
  0x0020,   /* U+1E0CA */
  0x0020,   /* U+1E0CB */
  0x0020,   /* U+1E0CC */
  0x0020,   /* U+1E0CD */
  0x0020,   /* U+1E0CE */
  0x0020,   /* U+1E0CF */
  0x0020,   /* U+1E0D0 */
  0x0020,   /* U+1E0D1 */
  0x0020,   /* U+1E0D2 */
  0x0020,   /* U+1E0D3 */
  0x0020,   /* U+1E0D4 */
  0x0020,   /* U+1E0D5 */
  0x0020,   /* U+1E0D6 */
  0x0020,   /* U+1E0D7 */
  0x0020,   /* U+1E0D8 */
  0x0020,   /* U+1E0D9 */
  0x0020,   /* U+1E0DA */
  0x0020,   /* U+1E0DB */
  0x0020,   /* U+1E0DC */
  0x0020,   /* U+1E0DD */
  0x0020,   /* U+1E0DE */
  0x0020,   /* U+1E0DF */
  0x0020,   /* U+1E0E0 */
  0x0020,   /* U+1E0E1 */
  0x0020,   /* U+1E0E2 */
  0x0020,   /* U+1E0E3 */
  0x0020,   /* U+1E0E4 */
  0x0020,   /* U+1E0E5 */
  0x0020,   /* U+1E0E6 */
  0x0020,   /* U+1E0E7 */
  0x0020,   /* U+1E0E8 */
  0x0020,   /* U+1E0E9 */
  0x0020,   /* U+1E0EA */
  0x0020,   /* U+1E0EB */
  0x0020,   /* U+1E0EC */
  0x0020,   /* U+1E0ED */
  0x0020,   /* U+1E0EE */
  0x0020,   /* U+1E0EF */
  0x0020,   /* U+1E0F0 */
  0x0020,   /* U+1E0F1 */
  0x0020,   /* U+1E0F2 */
  0x0020,   /* U+1E0F3 */
  0x0020,   /* U+1E0F4 */
  0x0020,   /* U+1E0F5 */
  0x0020,   /* U+1E0F6 */
  0x0020,   /* U+1E0F7 */
  0x0020,   /* U+1E0F8 */
  0x0020,   /* U+1E0F9 */
  0x0020,   /* U+1E0FA */
  0x0020,   /* U+1E0FB */
  0x0020,   /* U+1E0FC */
  0x0020,   /* U+1E0FD */
  0x0020,   /* U+1E0FE */
  0x0020,   /* U+1E0FF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+1E000 */
  0x0004,   /* U+1E001 */
  0x0004,   /* U+1E002 */
  0x0004,   /* U+1E003 */
  0x0004,   /* U+1E004 */
  0x0004,   /* U+1E005 */
  0x0004,   /* U+1E006 */
  0x0002,   /* U+1E007 */
  0x0004,   /* U+1E008 */
  0x0004,   /* U+1E009 */
  0x0004,   /* U+1E00A */
  0x0004,   /* U+1E00B */
  0x0004,   /* U+1E00C */
  0x0004,   /* U+1E00D */
  0x0004,   /* U+1E00E */
  0x0004,   /* U+1E00F */
  0x0004,   /* U+1E010 */
  0x0004,   /* U+1E011 */
  0x0004,   /* U+1E012 */
  0x0004,   /* U+1E013 */
  0x0004,   /* U+1E014 */
  0x0004,   /* U+1E015 */
  0x0004,   /* U+1E016 */
  0x0004,   /* U+1E017 */
  0x0004,   /* U+1E018 */
  0x0002,   /* U+1E019 */
  0x0002,   /* U+1E01A */
  0x0004,   /* U+1E01B */
  0x0004,   /* U+1E01C */
  0x0004,   /* U+1E01D */
  0x0004,   /* U+1E01E */
  0x0004,   /* U+1E01F */
  0x0004,   /* U+1E020 */
  0x0004,   /* U+1E021 */
  0x0002,   /* U+1E022 */
  0x0004,   /* U+1E023 */
  0x0004,   /* U+1E024 */
  0x0002,   /* U+1E025 */
  0x0004,   /* U+1E026 */
  0x0004,   /* U+1E027 */
  0x0004,   /* U+1E028 */
  0x0004,   /* U+1E029 */
  0x0004,   /* U+1E02A */
  0x0002,   /* U+1E02B */
  0x0002,   /* U+1E02C */
  0x0002,   /* U+1E02D */
  0x0002,   /* U+1E02E */
  0x0002,   /* U+1E02F */
  0x0002,   /* U+1E030 */
  0x0002,   /* U+1E031 */
  0x0002,   /* U+1E032 */
  0x0002,   /* U+1E033 */
  0x0002,   /* U+1E034 */
  0x0002,   /* U+1E035 */
  0x0002,   /* U+1E036 */
  0x0002,   /* U+1E037 */
  0x0002,   /* U+1E038 */
  0x0002,   /* U+1E039 */
  0x0002,   /* U+1E03A */
  0x0002,   /* U+1E03B */
  0x0002,   /* U+1E03C */
  0x0002,   /* U+1E03D */
  0x0002,   /* U+1E03E */
  0x0002,   /* U+1E03F */
  0x0002,   /* U+1E040 */
  0x0002,   /* U+1E041 */
  0x0002,   /* U+1E042 */
  0x0002,   /* U+1E043 */
  0x0002,   /* U+1E044 */
  0x0002,   /* U+1E045 */
  0x0002,   /* U+1E046 */
  0x0002,   /* U+1E047 */
  0x0002,   /* U+1E048 */
  0x0002,   /* U+1E049 */
  0x0002,   /* U+1E04A */
  0x0002,   /* U+1E04B */
  0x0002,   /* U+1E04C */
  0x0002,   /* U+1E04D */
  0x0002,   /* U+1E04E */
  0x0002,   /* U+1E04F */
  0x0002,   /* U+1E050 */
  0x0002,   /* U+1E051 */
  0x0002,   /* U+1E052 */
  0x0002,   /* U+1E053 */
  0x0002,   /* U+1E054 */
  0x0002,   /* U+1E055 */
  0x0002,   /* U+1E056 */
  0x0002,   /* U+1E057 */
  0x0002,   /* U+1E058 */
  0x0002,   /* U+1E059 */
  0x0002,   /* U+1E05A */
  0x0002,   /* U+1E05B */
  0x0002,   /* U+1E05C */
  0x0002,   /* U+1E05D */
  0x0002,   /* U+1E05E */
  0x0002,   /* U+1E05F */
  0x0002,   /* U+1E060 */
  0x0002,   /* U+1E061 */
  0x0002,   /* U+1E062 */
  0x0002,   /* U+1E063 */
  0x0002,   /* U+1E064 */
  0x0002,   /* U+1E065 */
  0x0002,   /* U+1E066 */
  0x0002,   /* U+1E067 */
  0x0002,   /* U+1E068 */
  0x0002,   /* U+1E069 */
  0x0002,   /* U+1E06A */
  0x0002,   /* U+1E06B */
  0x0002,   /* U+1E06C */
  0x0002,   /* U+1E06D */
  0x0002,   /* U+1E06E */
  0x0002,   /* U+1E06F */
  0x0002,   /* U+1E070 */
  0x0002,   /* U+1E071 */
  0x0002,   /* U+1E072 */
  0x0002,   /* U+1E073 */
  0x0002,   /* U+1E074 */
  0x0002,   /* U+1E075 */
  0x0002,   /* U+1E076 */
  0x0002,   /* U+1E077 */
  0x0002,   /* U+1E078 */
  0x0002,   /* U+1E079 */
  0x0002,   /* U+1E07A */
  0x0002,   /* U+1E07B */
  0x0002,   /* U+1E07C */
  0x0002,   /* U+1E07D */
  0x0002,   /* U+1E07E */
  0x0002,   /* U+1E07F */
  0x0002,   /* U+1E080 */
  0x0002,   /* U+1E081 */
  0x0002,   /* U+1E082 */
  0x0002,   /* U+1E083 */
  0x0002,   /* U+1E084 */
  0x0002,   /* U+1E085 */
  0x0002,   /* U+1E086 */
  0x0002,   /* U+1E087 */
  0x0002,   /* U+1E088 */
  0x0002,   /* U+1E089 */
  0x0002,   /* U+1E08A */
  0x0002,   /* U+1E08B */
  0x0002,   /* U+1E08C */
  0x0002,   /* U+1E08D */
  0x0002,   /* U+1E08E */
  0x0002,   /* U+1E08F */
  0x0002,   /* U+1E090 */
  0x0002,   /* U+1E091 */
  0x0002,   /* U+1E092 */
  0x0002,   /* U+1E093 */
  0x0002,   /* U+1E094 */
  0x0002,   /* U+1E095 */
  0x0002,   /* U+1E096 */
  0x0002,   /* U+1E097 */
  0x0002,   /* U+1E098 */
  0x0002,   /* U+1E099 */
  0x0002,   /* U+1E09A */
  0x0002,   /* U+1E09B */
  0x0002,   /* U+1E09C */
  0x0002,   /* U+1E09D */
  0x0002,   /* U+1E09E */
  0x0002,   /* U+1E09F */
  0x0002,   /* U+1E0A0 */
  0x0002,   /* U+1E0A1 */
  0x0002,   /* U+1E0A2 */
  0x0002,   /* U+1E0A3 */
  0x0002,   /* U+1E0A4 */
  0x0002,   /* U+1E0A5 */
  0x0002,   /* U+1E0A6 */
  0x0002,   /* U+1E0A7 */
  0x0002,   /* U+1E0A8 */
  0x0002,   /* U+1E0A9 */
  0x0002,   /* U+1E0AA */
  0x0002,   /* U+1E0AB */
  0x0002,   /* U+1E0AC */
  0x0002,   /* U+1E0AD */
  0x0002,   /* U+1E0AE */
  0x0002,   /* U+1E0AF */
  0x0002,   /* U+1E0B0 */
  0x0002,   /* U+1E0B1 */
  0x0002,   /* U+1E0B2 */
  0x0002,   /* U+1E0B3 */
  0x0002,   /* U+1E0B4 */
  0x0002,   /* U+1E0B5 */
  0x0002,   /* U+1E0B6 */
  0x0002,   /* U+1E0B7 */
  0x0002,   /* U+1E0B8 */
  0x0002,   /* U+1E0B9 */
  0x0002,   /* U+1E0BA */
  0x0002,   /* U+1E0BB */
  0x0002,   /* U+1E0BC */
  0x0002,   /* U+1E0BD */
  0x0002,   /* U+1E0BE */
  0x0002,   /* U+1E0BF */
  0x0002,   /* U+1E0C0 */
  0x0002,   /* U+1E0C1 */
  0x0002,   /* U+1E0C2 */
  0x0002,   /* U+1E0C3 */
  0x0002,   /* U+1E0C4 */
  0x0002,   /* U+1E0C5 */
  0x0002,   /* U+1E0C6 */
  0x0002,   /* U+1E0C7 */
  0x0002,   /* U+1E0C8 */
  0x0002,   /* U+1E0C9 */
  0x0002,   /* U+1E0CA */
  0x0002,   /* U+1E0CB */
  0x0002,   /* U+1E0CC */
  0x0002,   /* U+1E0CD */
  0x0002,   /* U+1E0CE */
  0x0002,   /* U+1E0CF */
  0x0002,   /* U+1E0D0 */
  0x0002,   /* U+1E0D1 */
  0x0002,   /* U+1E0D2 */
  0x0002,   /* U+1E0D3 */
  0x0002,   /* U+1E0D4 */
  0x0002,   /* U+1E0D5 */
  0x0002,   /* U+1E0D6 */
  0x0002,   /* U+1E0D7 */
  0x0002,   /* U+1E0D8 */
  0x0002,   /* U+1E0D9 */
  0x0002,   /* U+1E0DA */
  0x0002,   /* U+1E0DB */
  0x0002,   /* U+1E0DC */
  0x0002,   /* U+1E0DD */
  0x0002,   /* U+1E0DE */
  0x0002,   /* U+1E0DF */
  0x0002,   /* U+1E0E0 */
  0x0002,   /* U+1E0E1 */
  0x0002,   /* U+1E0E2 */
  0x0002,   /* U+1E0E3 */
  0x0002,   /* U+1E0E4 */
  0x0002,   /* U+1E0E5 */
  0x0002,   /* U+1E0E6 */
  0x0002,   /* U+1E0E7 */
  0x0002,   /* U+1E0E8 */
  0x0002,   /* U+1E0E9 */
  0x0002,   /* U+1E0EA */
  0x0002,   /* U+1E0EB */
  0x0002,   /* U+1E0EC */
  0x0002,   /* U+1E0ED */
  0x0002,   /* U+1E0EE */
  0x0002,   /* U+1E0EF */
  0x0002,   /* U+1E0F0 */
  0x0002,   /* U+1E0F1 */
  0x0002,   /* U+1E0F2 */
  0x0002,   /* U+1E0F3 */
  0x0002,   /* U+1E0F4 */
  0x0002,   /* U+1E0F5 */
  0x0002,   /* U+1E0F6 */
  0x0002,   /* U+1E0F7 */
  0x0002,   /* U+1E0F8 */
  0x0002,   /* U+1E0F9 */
  0x0002,   /* U+1E0FA */
  0x0002,   /* U+1E0FB */
  0x0002,   /* U+1E0FC */
  0x0002,   /* U+1E0FD */
  0x0002,   /* U+1E0FE */
  0x0002,   /* U+1E0FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1E000 */
  0x0000,   /* U+1E001 */
  0x0000,   /* U+1E002 */
  0x0000,   /* U+1E003 */
  0x0000,   /* U+1E004 */
  0x0000,   /* U+1E005 */
  0x0000,   /* U+1E006 */
  0xE007,   /* U+1E007 */
  0x0000,   /* U+1E008 */
  0x0000,   /* U+1E009 */
  0x0000,   /* U+1E00A */
  0x0000,   /* U+1E00B */
  0x0000,   /* U+1E00C */
  0x0000,   /* U+1E00D */
  0x0000,   /* U+1E00E */
  0x0000,   /* U+1E00F */
  0x0000,   /* U+1E010 */
  0x0000,   /* U+1E011 */
  0x0000,   /* U+1E012 */
  0x0000,   /* U+1E013 */
  0x0000,   /* U+1E014 */
  0x0000,   /* U+1E015 */
  0x0000,   /* U+1E016 */
  0x0000,   /* U+1E017 */
  0x0000,   /* U+1E018 */
  0xE019,   /* U+1E019 */
  0xE01A,   /* U+1E01A */
  0x0000,   /* U+1E01B */
  0x0000,   /* U+1E01C */
  0x0000,   /* U+1E01D */
  0x0000,   /* U+1E01E */
  0x0000,   /* U+1E01F */
  0x0000,   /* U+1E020 */
  0x0000,   /* U+1E021 */
  0xE022,   /* U+1E022 */
  0x0000,   /* U+1E023 */
  0x0000,   /* U+1E024 */
  0xE025,   /* U+1E025 */
  0x0000,   /* U+1E026 */
  0x0000,   /* U+1E027 */
  0x0000,   /* U+1E028 */
  0x0000,   /* U+1E029 */
  0x0000,   /* U+1E02A */
  0xE02B,   /* U+1E02B */
  0xE02C,   /* U+1E02C */
  0xE02D,   /* U+1E02D */
  0xE02E,   /* U+1E02E */
  0xE02F,   /* U+1E02F */
  0xE030,   /* U+1E030 */
  0xE031,   /* U+1E031 */
  0xE032,   /* U+1E032 */
  0xE033,   /* U+1E033 */
  0xE034,   /* U+1E034 */
  0xE035,   /* U+1E035 */
  0xE036,   /* U+1E036 */
  0xE037,   /* U+1E037 */
  0xE038,   /* U+1E038 */
  0xE039,   /* U+1E039 */
  0xE03A,   /* U+1E03A */
  0xE03B,   /* U+1E03B */
  0xE03C,   /* U+1E03C */
  0xE03D,   /* U+1E03D */
  0xE03E,   /* U+1E03E */
  0xE03F,   /* U+1E03F */
  0xE040,   /* U+1E040 */
  0xE041,   /* U+1E041 */
  0xE042,   /* U+1E042 */
  0xE043,   /* U+1E043 */
  0xE044,   /* U+1E044 */
  0xE045,   /* U+1E045 */
  0xE046,   /* U+1E046 */
  0xE047,   /* U+1E047 */
  0xE048,   /* U+1E048 */
  0xE049,   /* U+1E049 */
  0xE04A,   /* U+1E04A */
  0xE04B,   /* U+1E04B */
  0xE04C,   /* U+1E04C */
  0xE04D,   /* U+1E04D */
  0xE04E,   /* U+1E04E */
  0xE04F,   /* U+1E04F */
  0xE050,   /* U+1E050 */
  0xE051,   /* U+1E051 */
  0xE052,   /* U+1E052 */
  0xE053,   /* U+1E053 */
  0xE054,   /* U+1E054 */
  0xE055,   /* U+1E055 */
  0xE056,   /* U+1E056 */
  0xE057,   /* U+1E057 */
  0xE058,   /* U+1E058 */
  0xE059,   /* U+1E059 */
  0xE05A,   /* U+1E05A */
  0xE05B,   /* U+1E05B */
  0xE05C,   /* U+1E05C */
  0xE05D,   /* U+1E05D */
  0xE05E,   /* U+1E05E */
  0xE05F,   /* U+1E05F */
  0xE060,   /* U+1E060 */
  0xE061,   /* U+1E061 */
  0xE062,   /* U+1E062 */
  0xE063,   /* U+1E063 */
  0xE064,   /* U+1E064 */
  0xE065,   /* U+1E065 */
  0xE066,   /* U+1E066 */
  0xE067,   /* U+1E067 */
  0xE068,   /* U+1E068 */
  0xE069,   /* U+1E069 */
  0xE06A,   /* U+1E06A */
  0xE06B,   /* U+1E06B */
  0xE06C,   /* U+1E06C */
  0xE06D,   /* U+1E06D */
  0xE06E,   /* U+1E06E */
  0xE06F,   /* U+1E06F */
  0xE070,   /* U+1E070 */
  0xE071,   /* U+1E071 */
  0xE072,   /* U+1E072 */
  0xE073,   /* U+1E073 */
  0xE074,   /* U+1E074 */
  0xE075,   /* U+1E075 */
  0xE076,   /* U+1E076 */
  0xE077,   /* U+1E077 */
  0xE078,   /* U+1E078 */
  0xE079,   /* U+1E079 */
  0xE07A,   /* U+1E07A */
  0xE07B,   /* U+1E07B */
  0xE07C,   /* U+1E07C */
  0xE07D,   /* U+1E07D */
  0xE07E,   /* U+1E07E */
  0xE07F,   /* U+1E07F */
  0xE080,   /* U+1E080 */
  0xE081,   /* U+1E081 */
  0xE082,   /* U+1E082 */
  0xE083,   /* U+1E083 */
  0xE084,   /* U+1E084 */
  0xE085,   /* U+1E085 */
  0xE086,   /* U+1E086 */
  0xE087,   /* U+1E087 */
  0xE088,   /* U+1E088 */
  0xE089,   /* U+1E089 */
  0xE08A,   /* U+1E08A */
  0xE08B,   /* U+1E08B */
  0xE08C,   /* U+1E08C */
  0xE08D,   /* U+1E08D */
  0xE08E,   /* U+1E08E */
  0xE08F,   /* U+1E08F */
  0xE090,   /* U+1E090 */
  0xE091,   /* U+1E091 */
  0xE092,   /* U+1E092 */
  0xE093,   /* U+1E093 */
  0xE094,   /* U+1E094 */
  0xE095,   /* U+1E095 */
  0xE096,   /* U+1E096 */
  0xE097,   /* U+1E097 */
  0xE098,   /* U+1E098 */
  0xE099,   /* U+1E099 */
  0xE09A,   /* U+1E09A */
  0xE09B,   /* U+1E09B */
  0xE09C,   /* U+1E09C */
  0xE09D,   /* U+1E09D */
  0xE09E,   /* U+1E09E */
  0xE09F,   /* U+1E09F */
  0xE0A0,   /* U+1E0A0 */
  0xE0A1,   /* U+1E0A1 */
  0xE0A2,   /* U+1E0A2 */
  0xE0A3,   /* U+1E0A3 */
  0xE0A4,   /* U+1E0A4 */
  0xE0A5,   /* U+1E0A5 */
  0xE0A6,   /* U+1E0A6 */
  0xE0A7,   /* U+1E0A7 */
  0xE0A8,   /* U+1E0A8 */
  0xE0A9,   /* U+1E0A9 */
  0xE0AA,   /* U+1E0AA */
  0xE0AB,   /* U+1E0AB */
  0xE0AC,   /* U+1E0AC */
  0xE0AD,   /* U+1E0AD */
  0xE0AE,   /* U+1E0AE */
  0xE0AF,   /* U+1E0AF */
  0xE0B0,   /* U+1E0B0 */
  0xE0B1,   /* U+1E0B1 */
  0xE0B2,   /* U+1E0B2 */
  0xE0B3,   /* U+1E0B3 */
  0xE0B4,   /* U+1E0B4 */
  0xE0B5,   /* U+1E0B5 */
  0xE0B6,   /* U+1E0B6 */
  0xE0B7,   /* U+1E0B7 */
  0xE0B8,   /* U+1E0B8 */
  0xE0B9,   /* U+1E0B9 */
  0xE0BA,   /* U+1E0BA */
  0xE0BB,   /* U+1E0BB */
  0xE0BC,   /* U+1E0BC */
  0xE0BD,   /* U+1E0BD */
  0xE0BE,   /* U+1E0BE */
  0xE0BF,   /* U+1E0BF */
  0xE0C0,   /* U+1E0C0 */
  0xE0C1,   /* U+1E0C1 */
  0xE0C2,   /* U+1E0C2 */
  0xE0C3,   /* U+1E0C3 */
  0xE0C4,   /* U+1E0C4 */
  0xE0C5,   /* U+1E0C5 */
  0xE0C6,   /* U+1E0C6 */
  0xE0C7,   /* U+1E0C7 */
  0xE0C8,   /* U+1E0C8 */
  0xE0C9,   /* U+1E0C9 */
  0xE0CA,   /* U+1E0CA */
  0xE0CB,   /* U+1E0CB */
  0xE0CC,   /* U+1E0CC */
  0xE0CD,   /* U+1E0CD */
  0xE0CE,   /* U+1E0CE */
  0xE0CF,   /* U+1E0CF */
  0xE0D0,   /* U+1E0D0 */
  0xE0D1,   /* U+1E0D1 */
  0xE0D2,   /* U+1E0D2 */
  0xE0D3,   /* U+1E0D3 */
  0xE0D4,   /* U+1E0D4 */
  0xE0D5,   /* U+1E0D5 */
  0xE0D6,   /* U+1E0D6 */
  0xE0D7,   /* U+1E0D7 */
  0xE0D8,   /* U+1E0D8 */
  0xE0D9,   /* U+1E0D9 */
  0xE0DA,   /* U+1E0DA */
  0xE0DB,   /* U+1E0DB */
  0xE0DC,   /* U+1E0DC */
  0xE0DD,   /* U+1E0DD */
  0xE0DE,   /* U+1E0DE */
  0xE0DF,   /* U+1E0DF */
  0xE0E0,   /* U+1E0E0 */
  0xE0E1,   /* U+1E0E1 */
  0xE0E2,   /* U+1E0E2 */
  0xE0E3,   /* U+1E0E3 */
  0xE0E4,   /* U+1E0E4 */
  0xE0E5,   /* U+1E0E5 */
  0xE0E6,   /* U+1E0E6 */
  0xE0E7,   /* U+1E0E7 */
  0xE0E8,   /* U+1E0E8 */
  0xE0E9,   /* U+1E0E9 */
  0xE0EA,   /* U+1E0EA */
  0xE0EB,   /* U+1E0EB */
  0xE0EC,   /* U+1E0EC */
  0xE0ED,   /* U+1E0ED */
  0xE0EE,   /* U+1E0EE */
  0xE0EF,   /* U+1E0EF */
  0xE0F0,   /* U+1E0F0 */
  0xE0F1,   /* U+1E0F1 */
  0xE0F2,   /* U+1E0F2 */
  0xE0F3,   /* U+1E0F3 */
  0xE0F4,   /* U+1E0F4 */
  0xE0F5,   /* U+1E0F5 */
  0xE0F6,   /* U+1E0F6 */
  0xE0F7,   /* U+1E0F7 */
  0xE0F8,   /* U+1E0F8 */
  0xE0F9,   /* U+1E0F9 */
  0xE0FA,   /* U+1E0FA */
  0xE0FB,   /* U+1E0FB */
  0xE0FC,   /* U+1E0FC */
  0xE0FD,   /* U+1E0FD */
  0xE0FE,   /* U+1E0FE */
  0xE0FF,   /* U+1E0FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1E000 */
  0x0000,   /* U+1E001 */
  0x0000,   /* U+1E002 */
  0x0000,   /* U+1E003 */
  0x0000,   /* U+1E004 */
  0x0000,   /* U+1E005 */
  0x0000,   /* U+1E006 */
  0x0000,   /* U+1E007 */
  0x0000,   /* U+1E008 */
  0x0000,   /* U+1E009 */
  0x0000,   /* U+1E00A */
  0x0000,   /* U+1E00B */
  0x0000,   /* U+1E00C */
  0x0000,   /* U+1E00D */
  0x0000,   /* U+1E00E */
  0x0000,   /* U+1E00F */
  0x0000,   /* U+1E010 */
  0x0000,   /* U+1E011 */
  0x0000,   /* U+1E012 */
  0x0000,   /* U+1E013 */
  0x0000,   /* U+1E014 */
  0x0000,   /* U+1E015 */
  0x0000,   /* U+1E016 */
  0x0000,   /* U+1E017 */
  0x0000,   /* U+1E018 */
  0x0000,   /* U+1E019 */
  0x0000,   /* U+1E01A */
  0x0000,   /* U+1E01B */
  0x0000,   /* U+1E01C */
  0x0000,   /* U+1E01D */
  0x0000,   /* U+1E01E */
  0x0000,   /* U+1E01F */
  0x0000,   /* U+1E020 */
  0x0000,   /* U+1E021 */
  0x0000,   /* U+1E022 */
  0x0000,   /* U+1E023 */
  0x0000,   /* U+1E024 */
  0x0000,   /* U+1E025 */
  0x0000,   /* U+1E026 */
  0x0000,   /* U+1E027 */
  0x0000,   /* U+1E028 */
  0x0000,   /* U+1E029 */
  0x0000,   /* U+1E02A */
  0x0000,   /* U+1E02B */
  0x0000,   /* U+1E02C */
  0x0000,   /* U+1E02D */
  0x0000,   /* U+1E02E */
  0x0000,   /* U+1E02F */
  0x0000,   /* U+1E030 */
  0x0000,   /* U+1E031 */
  0x0000,   /* U+1E032 */
  0x0000,   /* U+1E033 */
  0x0000,   /* U+1E034 */
  0x0000,   /* U+1E035 */
  0x0000,   /* U+1E036 */
  0x0000,   /* U+1E037 */
  0x0000,   /* U+1E038 */
  0x0000,   /* U+1E039 */
  0x0000,   /* U+1E03A */
  0x0000,   /* U+1E03B */
  0x0000,   /* U+1E03C */
  0x0000,   /* U+1E03D */
  0x0000,   /* U+1E03E */
  0x0000,   /* U+1E03F */
  0x0000,   /* U+1E040 */
  0x0000,   /* U+1E041 */
  0x0000,   /* U+1E042 */
  0x0000,   /* U+1E043 */
  0x0000,   /* U+1E044 */
  0x0000,   /* U+1E045 */
  0x0000,   /* U+1E046 */
  0x0000,   /* U+1E047 */
  0x0000,   /* U+1E048 */
  0x0000,   /* U+1E049 */
  0x0000,   /* U+1E04A */
  0x0000,   /* U+1E04B */
  0x0000,   /* U+1E04C */
  0x0000,   /* U+1E04D */
  0x0000,   /* U+1E04E */
  0x0000,   /* U+1E04F */
  0x0000,   /* U+1E050 */
  0x0000,   /* U+1E051 */
  0x0000,   /* U+1E052 */
  0x0000,   /* U+1E053 */
  0x0000,   /* U+1E054 */
  0x0000,   /* U+1E055 */
  0x0000,   /* U+1E056 */
  0x0000,   /* U+1E057 */
  0x0000,   /* U+1E058 */
  0x0000,   /* U+1E059 */
  0x0000,   /* U+1E05A */
  0x0000,   /* U+1E05B */
  0x0000,   /* U+1E05C */
  0x0000,   /* U+1E05D */
  0x0000,   /* U+1E05E */
  0x0000,   /* U+1E05F */
  0x0000,   /* U+1E060 */
  0x0000,   /* U+1E061 */
  0x0000,   /* U+1E062 */
  0x0000,   /* U+1E063 */
  0x0000,   /* U+1E064 */
  0x0000,   /* U+1E065 */
  0x0000,   /* U+1E066 */
  0x0000,   /* U+1E067 */
  0x0000,   /* U+1E068 */
  0x0000,   /* U+1E069 */
  0x0000,   /* U+1E06A */
  0x0000,   /* U+1E06B */
  0x0000,   /* U+1E06C */
  0x0000,   /* U+1E06D */
  0x0000,   /* U+1E06E */
  0x0000,   /* U+1E06F */
  0x0000,   /* U+1E070 */
  0x0000,   /* U+1E071 */
  0x0000,   /* U+1E072 */
  0x0000,   /* U+1E073 */
  0x0000,   /* U+1E074 */
  0x0000,   /* U+1E075 */
  0x0000,   /* U+1E076 */
  0x0000,   /* U+1E077 */
  0x0000,   /* U+1E078 */
  0x0000,   /* U+1E079 */
  0x0000,   /* U+1E07A */
  0x0000,   /* U+1E07B */
  0x0000,   /* U+1E07C */
  0x0000,   /* U+1E07D */
  0x0000,   /* U+1E07E */
  0x0000,   /* U+1E07F */
  0x0000,   /* U+1E080 */
  0x0000,   /* U+1E081 */
  0x0000,   /* U+1E082 */
  0x0000,   /* U+1E083 */
  0x0000,   /* U+1E084 */
  0x0000,   /* U+1E085 */
  0x0000,   /* U+1E086 */
  0x0000,   /* U+1E087 */
  0x0000,   /* U+1E088 */
  0x0000,   /* U+1E089 */
  0x0000,   /* U+1E08A */
  0x0000,   /* U+1E08B */
  0x0000,   /* U+1E08C */
  0x0000,   /* U+1E08D */
  0x0000,   /* U+1E08E */
  0x0000,   /* U+1E08F */
  0x0000,   /* U+1E090 */
  0x0000,   /* U+1E091 */
  0x0000,   /* U+1E092 */
  0x0000,   /* U+1E093 */
  0x0000,   /* U+1E094 */
  0x0000,   /* U+1E095 */
  0x0000,   /* U+1E096 */
  0x0000,   /* U+1E097 */
  0x0000,   /* U+1E098 */
  0x0000,   /* U+1E099 */
  0x0000,   /* U+1E09A */
  0x0000,   /* U+1E09B */
  0x0000,   /* U+1E09C */
  0x0000,   /* U+1E09D */
  0x0000,   /* U+1E09E */
  0x0000,   /* U+1E09F */
  0x0000,   /* U+1E0A0 */
  0x0000,   /* U+1E0A1 */
  0x0000,   /* U+1E0A2 */
  0x0000,   /* U+1E0A3 */
  0x0000,   /* U+1E0A4 */
  0x0000,   /* U+1E0A5 */
  0x0000,   /* U+1E0A6 */
  0x0000,   /* U+1E0A7 */
  0x0000,   /* U+1E0A8 */
  0x0000,   /* U+1E0A9 */
  0x0000,   /* U+1E0AA */
  0x0000,   /* U+1E0AB */
  0x0000,   /* U+1E0AC */
  0x0000,   /* U+1E0AD */
  0x0000,   /* U+1E0AE */
  0x0000,   /* U+1E0AF */
  0x0000,   /* U+1E0B0 */
  0x0000,   /* U+1E0B1 */
  0x0000,   /* U+1E0B2 */
  0x0000,   /* U+1E0B3 */
  0x0000,   /* U+1E0B4 */
  0x0000,   /* U+1E0B5 */
  0x0000,   /* U+1E0B6 */
  0x0000,   /* U+1E0B7 */
  0x0000,   /* U+1E0B8 */
  0x0000,   /* U+1E0B9 */
  0x0000,   /* U+1E0BA */
  0x0000,   /* U+1E0BB */
  0x0000,   /* U+1E0BC */
  0x0000,   /* U+1E0BD */
  0x0000,   /* U+1E0BE */
  0x0000,   /* U+1E0BF */
  0x0000,   /* U+1E0C0 */
  0x0000,   /* U+1E0C1 */
  0x0000,   /* U+1E0C2 */
  0x0000,   /* U+1E0C3 */
  0x0000,   /* U+1E0C4 */
  0x0000,   /* U+1E0C5 */
  0x0000,   /* U+1E0C6 */
  0x0000,   /* U+1E0C7 */
  0x0000,   /* U+1E0C8 */
  0x0000,   /* U+1E0C9 */
  0x0000,   /* U+1E0CA */
  0x0000,   /* U+1E0CB */
  0x0000,   /* U+1E0CC */
  0x0000,   /* U+1E0CD */
  0x0000,   /* U+1E0CE */
  0x0000,   /* U+1E0CF */
  0x0000,   /* U+1E0D0 */
  0x0000,   /* U+1E0D1 */
  0x0000,   /* U+1E0D2 */
  0x0000,   /* U+1E0D3 */
  0x0000,   /* U+1E0D4 */
  0x0000,   /* U+1E0D5 */
  0x0000,   /* U+1E0D6 */
  0x0000,   /* U+1E0D7 */
  0x0000,   /* U+1E0D8 */
  0x0000,   /* U+1E0D9 */
  0x0000,   /* U+1E0DA */
  0x0000,   /* U+1E0DB */
  0x0000,   /* U+1E0DC */
  0x0000,   /* U+1E0DD */
  0x0000,   /* U+1E0DE */
  0x0000,   /* U+1E0DF */
  0x0000,   /* U+1E0E0 */
  0x0000,   /* U+1E0E1 */
  0x0000,   /* U+1E0E2 */
  0x0000,   /* U+1E0E3 */
  0x0000,   /* U+1E0E4 */
  0x0000,   /* U+1E0E5 */
  0x0000,   /* U+1E0E6 */
  0x0000,   /* U+1E0E7 */
  0x0000,   /* U+1E0E8 */
  0x0000,   /* U+1E0E9 */
  0x0000,   /* U+1E0EA */
  0x0000,   /* U+1E0EB */
  0x0000,   /* U+1E0EC */
  0x0000,   /* U+1E0ED */
  0x0000,   /* U+1E0EE */
  0x0000,   /* U+1E0EF */
  0x0000,   /* U+1E0F0 */
  0x0000,   /* U+1E0F1 */
  0x0000,   /* U+1E0F2 */
  0x0000,   /* U+1E0F3 */
  0x0000,   /* U+1E0F4 */
  0x0000,   /* U+1E0F5 */
  0x0000,   /* U+1E0F6 */
  0x0000,   /* U+1E0F7 */
  0x0000,   /* U+1E0F8 */
  0x0000,   /* U+1E0F9 */
  0x0000,   /* U+1E0FA */
  0x0000,   /* U+1E0FB */
  0x0000,   /* U+1E0FC */
  0x0000,   /* U+1E0FD */
  0x0000,   /* U+1E0FE */
  0x0000,   /* U+1E0FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1E000 */
  0x0000,   /* U+1E001 */
  0x0000,   /* U+1E002 */
  0x0000,   /* U+1E003 */
  0x0000,   /* U+1E004 */
  0x0000,   /* U+1E005 */
  0x0000,   /* U+1E006 */
  0x0000,   /* U+1E007 */
  0x0000,   /* U+1E008 */
  0x0000,   /* U+1E009 */
  0x0000,   /* U+1E00A */
  0x0000,   /* U+1E00B */
  0x0000,   /* U+1E00C */
  0x0000,   /* U+1E00D */
  0x0000,   /* U+1E00E */
  0x0000,   /* U+1E00F */
  0x0000,   /* U+1E010 */
  0x0000,   /* U+1E011 */
  0x0000,   /* U+1E012 */
  0x0000,   /* U+1E013 */
  0x0000,   /* U+1E014 */
  0x0000,   /* U+1E015 */
  0x0000,   /* U+1E016 */
  0x0000,   /* U+1E017 */
  0x0000,   /* U+1E018 */
  0x0000,   /* U+1E019 */
  0x0000,   /* U+1E01A */
  0x0000,   /* U+1E01B */
  0x0000,   /* U+1E01C */
  0x0000,   /* U+1E01D */
  0x0000,   /* U+1E01E */
  0x0000,   /* U+1E01F */
  0x0000,   /* U+1E020 */
  0x0000,   /* U+1E021 */
  0x0000,   /* U+1E022 */
  0x0000,   /* U+1E023 */
  0x0000,   /* U+1E024 */
  0x0000,   /* U+1E025 */
  0x0000,   /* U+1E026 */
  0x0000,   /* U+1E027 */
  0x0000,   /* U+1E028 */
  0x0000,   /* U+1E029 */
  0x0000,   /* U+1E02A */
  0x0000,   /* U+1E02B */
  0x0000,   /* U+1E02C */
  0x0000,   /* U+1E02D */
  0x0000,   /* U+1E02E */
  0x0000,   /* U+1E02F */
  0x0000,   /* U+1E030 */
  0x0000,   /* U+1E031 */
  0x0000,   /* U+1E032 */
  0x0000,   /* U+1E033 */
  0x0000,   /* U+1E034 */
  0x0000,   /* U+1E035 */
  0x0000,   /* U+1E036 */
  0x0000,   /* U+1E037 */
  0x0000,   /* U+1E038 */
  0x0000,   /* U+1E039 */
  0x0000,   /* U+1E03A */
  0x0000,   /* U+1E03B */
  0x0000,   /* U+1E03C */
  0x0000,   /* U+1E03D */
  0x0000,   /* U+1E03E */
  0x0000,   /* U+1E03F */
  0x0000,   /* U+1E040 */
  0x0000,   /* U+1E041 */
  0x0000,   /* U+1E042 */
  0x0000,   /* U+1E043 */
  0x0000,   /* U+1E044 */
  0x0000,   /* U+1E045 */
  0x0000,   /* U+1E046 */
  0x0000,   /* U+1E047 */
  0x0000,   /* U+1E048 */
  0x0000,   /* U+1E049 */
  0x0000,   /* U+1E04A */
  0x0000,   /* U+1E04B */
  0x0000,   /* U+1E04C */
  0x0000,   /* U+1E04D */
  0x0000,   /* U+1E04E */
  0x0000,   /* U+1E04F */
  0x0000,   /* U+1E050 */
  0x0000,   /* U+1E051 */
  0x0000,   /* U+1E052 */
  0x0000,   /* U+1E053 */
  0x0000,   /* U+1E054 */
  0x0000,   /* U+1E055 */
  0x0000,   /* U+1E056 */
  0x0000,   /* U+1E057 */
  0x0000,   /* U+1E058 */
  0x0000,   /* U+1E059 */
  0x0000,   /* U+1E05A */
  0x0000,   /* U+1E05B */
  0x0000,   /* U+1E05C */
  0x0000,   /* U+1E05D */
  0x0000,   /* U+1E05E */
  0x0000,   /* U+1E05F */
  0x0000,   /* U+1E060 */
  0x0000,   /* U+1E061 */
  0x0000,   /* U+1E062 */
  0x0000,   /* U+1E063 */
  0x0000,   /* U+1E064 */
  0x0000,   /* U+1E065 */
  0x0000,   /* U+1E066 */
  0x0000,   /* U+1E067 */
  0x0000,   /* U+1E068 */
  0x0000,   /* U+1E069 */
  0x0000,   /* U+1E06A */
  0x0000,   /* U+1E06B */
  0x0000,   /* U+1E06C */
  0x0000,   /* U+1E06D */
  0x0000,   /* U+1E06E */
  0x0000,   /* U+1E06F */
  0x0000,   /* U+1E070 */
  0x0000,   /* U+1E071 */
  0x0000,   /* U+1E072 */
  0x0000,   /* U+1E073 */
  0x0000,   /* U+1E074 */
  0x0000,   /* U+1E075 */
  0x0000,   /* U+1E076 */
  0x0000,   /* U+1E077 */
  0x0000,   /* U+1E078 */
  0x0000,   /* U+1E079 */
  0x0000,   /* U+1E07A */
  0x0000,   /* U+1E07B */
  0x0000,   /* U+1E07C */
  0x0000,   /* U+1E07D */
  0x0000,   /* U+1E07E */
  0x0000,   /* U+1E07F */
  0x0000,   /* U+1E080 */
  0x0000,   /* U+1E081 */
  0x0000,   /* U+1E082 */
  0x0000,   /* U+1E083 */
  0x0000,   /* U+1E084 */
  0x0000,   /* U+1E085 */
  0x0000,   /* U+1E086 */
  0x0000,   /* U+1E087 */
  0x0000,   /* U+1E088 */
  0x0000,   /* U+1E089 */
  0x0000,   /* U+1E08A */
  0x0000,   /* U+1E08B */
  0x0000,   /* U+1E08C */
  0x0000,   /* U+1E08D */
  0x0000,   /* U+1E08E */
  0x0000,   /* U+1E08F */
  0x0000,   /* U+1E090 */
  0x0000,   /* U+1E091 */
  0x0000,   /* U+1E092 */
  0x0000,   /* U+1E093 */
  0x0000,   /* U+1E094 */
  0x0000,   /* U+1E095 */
  0x0000,   /* U+1E096 */
  0x0000,   /* U+1E097 */
  0x0000,   /* U+1E098 */
  0x0000,   /* U+1E099 */
  0x0000,   /* U+1E09A */
  0x0000,   /* U+1E09B */
  0x0000,   /* U+1E09C */
  0x0000,   /* U+1E09D */
  0x0000,   /* U+1E09E */
  0x0000,   /* U+1E09F */
  0x0000,   /* U+1E0A0 */
  0x0000,   /* U+1E0A1 */
  0x0000,   /* U+1E0A2 */
  0x0000,   /* U+1E0A3 */
  0x0000,   /* U+1E0A4 */
  0x0000,   /* U+1E0A5 */
  0x0000,   /* U+1E0A6 */
  0x0000,   /* U+1E0A7 */
  0x0000,   /* U+1E0A8 */
  0x0000,   /* U+1E0A9 */
  0x0000,   /* U+1E0AA */
  0x0000,   /* U+1E0AB */
  0x0000,   /* U+1E0AC */
  0x0000,   /* U+1E0AD */
  0x0000,   /* U+1E0AE */
  0x0000,   /* U+1E0AF */
  0x0000,   /* U+1E0B0 */
  0x0000,   /* U+1E0B1 */
  0x0000,   /* U+1E0B2 */
  0x0000,   /* U+1E0B3 */
  0x0000,   /* U+1E0B4 */
  0x0000,   /* U+1E0B5 */
  0x0000,   /* U+1E0B6 */
  0x0000,   /* U+1E0B7 */
  0x0000,   /* U+1E0B8 */
  0x0000,   /* U+1E0B9 */
  0x0000,   /* U+1E0BA */
  0x0000,   /* U+1E0BB */
  0x0000,   /* U+1E0BC */
  0x0000,   /* U+1E0BD */
  0x0000,   /* U+1E0BE */
  0x0000,   /* U+1E0BF */
  0x0000,   /* U+1E0C0 */
  0x0000,   /* U+1E0C1 */
  0x0000,   /* U+1E0C2 */
  0x0000,   /* U+1E0C3 */
  0x0000,   /* U+1E0C4 */
  0x0000,   /* U+1E0C5 */
  0x0000,   /* U+1E0C6 */
  0x0000,   /* U+1E0C7 */
  0x0000,   /* U+1E0C8 */
  0x0000,   /* U+1E0C9 */
  0x0000,   /* U+1E0CA */
  0x0000,   /* U+1E0CB */
  0x0000,   /* U+1E0CC */
  0x0000,   /* U+1E0CD */
  0x0000,   /* U+1E0CE */
  0x0000,   /* U+1E0CF */
  0x0000,   /* U+1E0D0 */
  0x0000,   /* U+1E0D1 */
  0x0000,   /* U+1E0D2 */
  0x0000,   /* U+1E0D3 */
  0x0000,   /* U+1E0D4 */
  0x0000,   /* U+1E0D5 */
  0x0000,   /* U+1E0D6 */
  0x0000,   /* U+1E0D7 */
  0x0000,   /* U+1E0D8 */
  0x0000,   /* U+1E0D9 */
  0x0000,   /* U+1E0DA */
  0x0000,   /* U+1E0DB */
  0x0000,   /* U+1E0DC */
  0x0000,   /* U+1E0DD */
  0x0000,   /* U+1E0DE */
  0x0000,   /* U+1E0DF */
  0x0000,   /* U+1E0E0 */
  0x0000,   /* U+1E0E1 */
  0x0000,   /* U+1E0E2 */
  0x0000,   /* U+1E0E3 */
  0x0000,   /* U+1E0E4 */
  0x0000,   /* U+1E0E5 */
  0x0000,   /* U+1E0E6 */
  0x0000,   /* U+1E0E7 */
  0x0000,   /* U+1E0E8 */
  0x0000,   /* U+1E0E9 */
  0x0000,   /* U+1E0EA */
  0x0000,   /* U+1E0EB */
  0x0000,   /* U+1E0EC */
  0x0000,   /* U+1E0ED */
  0x0000,   /* U+1E0EE */
  0x0000,   /* U+1E0EF */
  0x0000,   /* U+1E0F0 */
  0x0000,   /* U+1E0F1 */
  0x0000,   /* U+1E0F2 */
  0x0000,   /* U+1E0F3 */
  0x0000,   /* U+1E0F4 */
  0x0000,   /* U+1E0F5 */
  0x0000,   /* U+1E0F6 */
  0x0000,   /* U+1E0F7 */
  0x0000,   /* U+1E0F8 */
  0x0000,   /* U+1E0F9 */
  0x0000,   /* U+1E0FA */
  0x0000,   /* U+1E0FB */
  0x0000,   /* U+1E0FC */
  0x0000,   /* U+1E0FD */
  0x0000,   /* U+1E0FE */
  0x0000,   /* U+1E0FF */
};

uint16 uca900_p1E8[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3B0E,   /* U+1E800 */
  0x3B0F,   /* U+1E801 */
  0x3B10,   /* U+1E802 */
  0x3B11,   /* U+1E803 */
  0x3B12,   /* U+1E804 */
  0x3B13,   /* U+1E805 */
  0x3B14,   /* U+1E806 */
  0x3B15,   /* U+1E807 */
  0x3B16,   /* U+1E808 */
  0x3B17,   /* U+1E809 */
  0x3B18,   /* U+1E80A */
  0x3B19,   /* U+1E80B */
  0x3B1A,   /* U+1E80C */
  0x3B1B,   /* U+1E80D */
  0x3B1C,   /* U+1E80E */
  0x3B1D,   /* U+1E80F */
  0x3B1E,   /* U+1E810 */
  0x3B1F,   /* U+1E811 */
  0x3B20,   /* U+1E812 */
  0x3B21,   /* U+1E813 */
  0x3B22,   /* U+1E814 */
  0x3B23,   /* U+1E815 */
  0x3B24,   /* U+1E816 */
  0x3B25,   /* U+1E817 */
  0x3B26,   /* U+1E818 */
  0x3B27,   /* U+1E819 */
  0x3B28,   /* U+1E81A */
  0x3B29,   /* U+1E81B */
  0x3B2A,   /* U+1E81C */
  0x3B2B,   /* U+1E81D */
  0x3B2C,   /* U+1E81E */
  0x3B2D,   /* U+1E81F */
  0x3B2E,   /* U+1E820 */
  0x3B2F,   /* U+1E821 */
  0x3B30,   /* U+1E822 */
  0x3B31,   /* U+1E823 */
  0x3B32,   /* U+1E824 */
  0x3B33,   /* U+1E825 */
  0x3B34,   /* U+1E826 */
  0x3B35,   /* U+1E827 */
  0x3B36,   /* U+1E828 */
  0x3B37,   /* U+1E829 */
  0x3B38,   /* U+1E82A */
  0x3B39,   /* U+1E82B */
  0x3B3A,   /* U+1E82C */
  0x3B3B,   /* U+1E82D */
  0x3B3C,   /* U+1E82E */
  0x3B3D,   /* U+1E82F */
  0x3B3E,   /* U+1E830 */
  0x3B3F,   /* U+1E831 */
  0x3B40,   /* U+1E832 */
  0x3B41,   /* U+1E833 */
  0x3B42,   /* U+1E834 */
  0x3B43,   /* U+1E835 */
  0x3B44,   /* U+1E836 */
  0x3B45,   /* U+1E837 */
  0x3B46,   /* U+1E838 */
  0x3B47,   /* U+1E839 */
  0x3B48,   /* U+1E83A */
  0x3B49,   /* U+1E83B */
  0x3B4A,   /* U+1E83C */
  0x3B4B,   /* U+1E83D */
  0x3B4C,   /* U+1E83E */
  0x3B4D,   /* U+1E83F */
  0x3B4E,   /* U+1E840 */
  0x3B4F,   /* U+1E841 */
  0x3B50,   /* U+1E842 */
  0x3B51,   /* U+1E843 */
  0x3B52,   /* U+1E844 */
  0x3B53,   /* U+1E845 */
  0x3B54,   /* U+1E846 */
  0x3B55,   /* U+1E847 */
  0x3B56,   /* U+1E848 */
  0x3B57,   /* U+1E849 */
  0x3B58,   /* U+1E84A */
  0x3B59,   /* U+1E84B */
  0x3B5A,   /* U+1E84C */
  0x3B5B,   /* U+1E84D */
  0x3B5C,   /* U+1E84E */
  0x3B5D,   /* U+1E84F */
  0x3B5E,   /* U+1E850 */
  0x3B5F,   /* U+1E851 */
  0x3B60,   /* U+1E852 */
  0x3B61,   /* U+1E853 */
  0x3B62,   /* U+1E854 */
  0x3B63,   /* U+1E855 */
  0x3B64,   /* U+1E856 */
  0x3B65,   /* U+1E857 */
  0x3B66,   /* U+1E858 */
  0x3B67,   /* U+1E859 */
  0x3B68,   /* U+1E85A */
  0x3B69,   /* U+1E85B */
  0x3B6A,   /* U+1E85C */
  0x3B6B,   /* U+1E85D */
  0x3B6C,   /* U+1E85E */
  0x3B6D,   /* U+1E85F */
  0x3B6E,   /* U+1E860 */
  0x3B6F,   /* U+1E861 */
  0x3B70,   /* U+1E862 */
  0x3B71,   /* U+1E863 */
  0x3B72,   /* U+1E864 */
  0x3B73,   /* U+1E865 */
  0x3B74,   /* U+1E866 */
  0x3B75,   /* U+1E867 */
  0x3B76,   /* U+1E868 */
  0x3B77,   /* U+1E869 */
  0x3B78,   /* U+1E86A */
  0x3B79,   /* U+1E86B */
  0x3B7A,   /* U+1E86C */
  0x3B7B,   /* U+1E86D */
  0x3B7C,   /* U+1E86E */
  0x3B7D,   /* U+1E86F */
  0x3B7E,   /* U+1E870 */
  0x3B7F,   /* U+1E871 */
  0x3B80,   /* U+1E872 */
  0x3B81,   /* U+1E873 */
  0x3B82,   /* U+1E874 */
  0x3B83,   /* U+1E875 */
  0x3B84,   /* U+1E876 */
  0x3B85,   /* U+1E877 */
  0x3B86,   /* U+1E878 */
  0x3B87,   /* U+1E879 */
  0x3B88,   /* U+1E87A */
  0x3B89,   /* U+1E87B */
  0x3B8A,   /* U+1E87C */
  0x3B8B,   /* U+1E87D */
  0x3B8C,   /* U+1E87E */
  0x3B8D,   /* U+1E87F */
  0x3B8E,   /* U+1E880 */
  0x3B8F,   /* U+1E881 */
  0x3B90,   /* U+1E882 */
  0x3B91,   /* U+1E883 */
  0x3B92,   /* U+1E884 */
  0x3B93,   /* U+1E885 */
  0x3B94,   /* U+1E886 */
  0x3B95,   /* U+1E887 */
  0x3B96,   /* U+1E888 */
  0x3B97,   /* U+1E889 */
  0x3B98,   /* U+1E88A */
  0x3B99,   /* U+1E88B */
  0x3B9A,   /* U+1E88C */
  0x3B9B,   /* U+1E88D */
  0x3B9C,   /* U+1E88E */
  0x3B9D,   /* U+1E88F */
  0x3B9E,   /* U+1E890 */
  0x3B9F,   /* U+1E891 */
  0x3BA0,   /* U+1E892 */
  0x3BA1,   /* U+1E893 */
  0x3BA2,   /* U+1E894 */
  0x3BA3,   /* U+1E895 */
  0x3BA4,   /* U+1E896 */
  0x3BA5,   /* U+1E897 */
  0x3BA6,   /* U+1E898 */
  0x3BA7,   /* U+1E899 */
  0x3BA8,   /* U+1E89A */
  0x3BA9,   /* U+1E89B */
  0x3BAA,   /* U+1E89C */
  0x3BAB,   /* U+1E89D */
  0x3BAC,   /* U+1E89E */
  0x3BAD,   /* U+1E89F */
  0x3BAE,   /* U+1E8A0 */
  0x3BAF,   /* U+1E8A1 */
  0x3BB0,   /* U+1E8A2 */
  0x3BB1,   /* U+1E8A3 */
  0x3BB2,   /* U+1E8A4 */
  0x3BB3,   /* U+1E8A5 */
  0x3BB4,   /* U+1E8A6 */
  0x3BB5,   /* U+1E8A7 */
  0x3BB6,   /* U+1E8A8 */
  0x3BB7,   /* U+1E8A9 */
  0x3BB8,   /* U+1E8AA */
  0x3BB9,   /* U+1E8AB */
  0x3BBA,   /* U+1E8AC */
  0x3BBB,   /* U+1E8AD */
  0x3BBC,   /* U+1E8AE */
  0x3BBD,   /* U+1E8AF */
  0x3BBE,   /* U+1E8B0 */
  0x3BBF,   /* U+1E8B1 */
  0x3BC0,   /* U+1E8B2 */
  0x3BC1,   /* U+1E8B3 */
  0x3BC2,   /* U+1E8B4 */
  0x3BC3,   /* U+1E8B5 */
  0x3BC4,   /* U+1E8B6 */
  0x3BC5,   /* U+1E8B7 */
  0x3BC6,   /* U+1E8B8 */
  0x3BC7,   /* U+1E8B9 */
  0x3BC8,   /* U+1E8BA */
  0x3BC9,   /* U+1E8BB */
  0x3BCA,   /* U+1E8BC */
  0x3BCB,   /* U+1E8BD */
  0x3BCC,   /* U+1E8BE */
  0x3BCD,   /* U+1E8BF */
  0x3BCE,   /* U+1E8C0 */
  0x3BCF,   /* U+1E8C1 */
  0x3BD0,   /* U+1E8C2 */
  0x3BD1,   /* U+1E8C3 */
  0x3BD2,   /* U+1E8C4 */
  0xFBC3,   /* U+1E8C5 */
  0xFBC3,   /* U+1E8C6 */
  0x1C3E,   /* U+1E8C7 */
  0x1C3F,   /* U+1E8C8 */
  0x1C40,   /* U+1E8C9 */
  0x1C41,   /* U+1E8CA */
  0x1C42,   /* U+1E8CB */
  0x1C43,   /* U+1E8CC */
  0x1C44,   /* U+1E8CD */
  0x1C45,   /* U+1E8CE */
  0x1C46,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0xFBC3,   /* U+1E8D7 */
  0xFBC3,   /* U+1E8D8 */
  0xFBC3,   /* U+1E8D9 */
  0xFBC3,   /* U+1E8DA */
  0xFBC3,   /* U+1E8DB */
  0xFBC3,   /* U+1E8DC */
  0xFBC3,   /* U+1E8DD */
  0xFBC3,   /* U+1E8DE */
  0xFBC3,   /* U+1E8DF */
  0xFBC3,   /* U+1E8E0 */
  0xFBC3,   /* U+1E8E1 */
  0xFBC3,   /* U+1E8E2 */
  0xFBC3,   /* U+1E8E3 */
  0xFBC3,   /* U+1E8E4 */
  0xFBC3,   /* U+1E8E5 */
  0xFBC3,   /* U+1E8E6 */
  0xFBC3,   /* U+1E8E7 */
  0xFBC3,   /* U+1E8E8 */
  0xFBC3,   /* U+1E8E9 */
  0xFBC3,   /* U+1E8EA */
  0xFBC3,   /* U+1E8EB */
  0xFBC3,   /* U+1E8EC */
  0xFBC3,   /* U+1E8ED */
  0xFBC3,   /* U+1E8EE */
  0xFBC3,   /* U+1E8EF */
  0xFBC3,   /* U+1E8F0 */
  0xFBC3,   /* U+1E8F1 */
  0xFBC3,   /* U+1E8F2 */
  0xFBC3,   /* U+1E8F3 */
  0xFBC3,   /* U+1E8F4 */
  0xFBC3,   /* U+1E8F5 */
  0xFBC3,   /* U+1E8F6 */
  0xFBC3,   /* U+1E8F7 */
  0xFBC3,   /* U+1E8F8 */
  0xFBC3,   /* U+1E8F9 */
  0xFBC3,   /* U+1E8FA */
  0xFBC3,   /* U+1E8FB */
  0xFBC3,   /* U+1E8FC */
  0xFBC3,   /* U+1E8FD */
  0xFBC3,   /* U+1E8FE */
  0xFBC3,   /* U+1E8FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1E800 */
  0x0020,   /* U+1E801 */
  0x0020,   /* U+1E802 */
  0x0020,   /* U+1E803 */
  0x0020,   /* U+1E804 */
  0x0020,   /* U+1E805 */
  0x0020,   /* U+1E806 */
  0x0020,   /* U+1E807 */
  0x0020,   /* U+1E808 */
  0x0020,   /* U+1E809 */
  0x0020,   /* U+1E80A */
  0x0020,   /* U+1E80B */
  0x0020,   /* U+1E80C */
  0x0020,   /* U+1E80D */
  0x0020,   /* U+1E80E */
  0x0020,   /* U+1E80F */
  0x0020,   /* U+1E810 */
  0x0020,   /* U+1E811 */
  0x0020,   /* U+1E812 */
  0x0020,   /* U+1E813 */
  0x0020,   /* U+1E814 */
  0x0020,   /* U+1E815 */
  0x0020,   /* U+1E816 */
  0x0020,   /* U+1E817 */
  0x0020,   /* U+1E818 */
  0x0020,   /* U+1E819 */
  0x0020,   /* U+1E81A */
  0x0020,   /* U+1E81B */
  0x0020,   /* U+1E81C */
  0x0020,   /* U+1E81D */
  0x0020,   /* U+1E81E */
  0x0020,   /* U+1E81F */
  0x0020,   /* U+1E820 */
  0x0020,   /* U+1E821 */
  0x0020,   /* U+1E822 */
  0x0020,   /* U+1E823 */
  0x0020,   /* U+1E824 */
  0x0020,   /* U+1E825 */
  0x0020,   /* U+1E826 */
  0x0020,   /* U+1E827 */
  0x0020,   /* U+1E828 */
  0x0020,   /* U+1E829 */
  0x0020,   /* U+1E82A */
  0x0020,   /* U+1E82B */
  0x0020,   /* U+1E82C */
  0x0020,   /* U+1E82D */
  0x0020,   /* U+1E82E */
  0x0020,   /* U+1E82F */
  0x0020,   /* U+1E830 */
  0x0020,   /* U+1E831 */
  0x0020,   /* U+1E832 */
  0x0020,   /* U+1E833 */
  0x0020,   /* U+1E834 */
  0x0020,   /* U+1E835 */
  0x0020,   /* U+1E836 */
  0x0020,   /* U+1E837 */
  0x0020,   /* U+1E838 */
  0x0020,   /* U+1E839 */
  0x0020,   /* U+1E83A */
  0x0020,   /* U+1E83B */
  0x0020,   /* U+1E83C */
  0x0020,   /* U+1E83D */
  0x0020,   /* U+1E83E */
  0x0020,   /* U+1E83F */
  0x0020,   /* U+1E840 */
  0x0020,   /* U+1E841 */
  0x0020,   /* U+1E842 */
  0x0020,   /* U+1E843 */
  0x0020,   /* U+1E844 */
  0x0020,   /* U+1E845 */
  0x0020,   /* U+1E846 */
  0x0020,   /* U+1E847 */
  0x0020,   /* U+1E848 */
  0x0020,   /* U+1E849 */
  0x0020,   /* U+1E84A */
  0x0020,   /* U+1E84B */
  0x0020,   /* U+1E84C */
  0x0020,   /* U+1E84D */
  0x0020,   /* U+1E84E */
  0x0020,   /* U+1E84F */
  0x0020,   /* U+1E850 */
  0x0020,   /* U+1E851 */
  0x0020,   /* U+1E852 */
  0x0020,   /* U+1E853 */
  0x0020,   /* U+1E854 */
  0x0020,   /* U+1E855 */
  0x0020,   /* U+1E856 */
  0x0020,   /* U+1E857 */
  0x0020,   /* U+1E858 */
  0x0020,   /* U+1E859 */
  0x0020,   /* U+1E85A */
  0x0020,   /* U+1E85B */
  0x0020,   /* U+1E85C */
  0x0020,   /* U+1E85D */
  0x0020,   /* U+1E85E */
  0x0020,   /* U+1E85F */
  0x0020,   /* U+1E860 */
  0x0020,   /* U+1E861 */
  0x0020,   /* U+1E862 */
  0x0020,   /* U+1E863 */
  0x0020,   /* U+1E864 */
  0x0020,   /* U+1E865 */
  0x0020,   /* U+1E866 */
  0x0020,   /* U+1E867 */
  0x0020,   /* U+1E868 */
  0x0020,   /* U+1E869 */
  0x0020,   /* U+1E86A */
  0x0020,   /* U+1E86B */
  0x0020,   /* U+1E86C */
  0x0020,   /* U+1E86D */
  0x0020,   /* U+1E86E */
  0x0020,   /* U+1E86F */
  0x0020,   /* U+1E870 */
  0x0020,   /* U+1E871 */
  0x0020,   /* U+1E872 */
  0x0020,   /* U+1E873 */
  0x0020,   /* U+1E874 */
  0x0020,   /* U+1E875 */
  0x0020,   /* U+1E876 */
  0x0020,   /* U+1E877 */
  0x0020,   /* U+1E878 */
  0x0020,   /* U+1E879 */
  0x0020,   /* U+1E87A */
  0x0020,   /* U+1E87B */
  0x0020,   /* U+1E87C */
  0x0020,   /* U+1E87D */
  0x0020,   /* U+1E87E */
  0x0020,   /* U+1E87F */
  0x0020,   /* U+1E880 */
  0x0020,   /* U+1E881 */
  0x0020,   /* U+1E882 */
  0x0020,   /* U+1E883 */
  0x0020,   /* U+1E884 */
  0x0020,   /* U+1E885 */
  0x0020,   /* U+1E886 */
  0x0020,   /* U+1E887 */
  0x0020,   /* U+1E888 */
  0x0020,   /* U+1E889 */
  0x0020,   /* U+1E88A */
  0x0020,   /* U+1E88B */
  0x0020,   /* U+1E88C */
  0x0020,   /* U+1E88D */
  0x0020,   /* U+1E88E */
  0x0020,   /* U+1E88F */
  0x0020,   /* U+1E890 */
  0x0020,   /* U+1E891 */
  0x0020,   /* U+1E892 */
  0x0020,   /* U+1E893 */
  0x0020,   /* U+1E894 */
  0x0020,   /* U+1E895 */
  0x0020,   /* U+1E896 */
  0x0020,   /* U+1E897 */
  0x0020,   /* U+1E898 */
  0x0020,   /* U+1E899 */
  0x0020,   /* U+1E89A */
  0x0020,   /* U+1E89B */
  0x0020,   /* U+1E89C */
  0x0020,   /* U+1E89D */
  0x0020,   /* U+1E89E */
  0x0020,   /* U+1E89F */
  0x0020,   /* U+1E8A0 */
  0x0020,   /* U+1E8A1 */
  0x0020,   /* U+1E8A2 */
  0x0020,   /* U+1E8A3 */
  0x0020,   /* U+1E8A4 */
  0x0020,   /* U+1E8A5 */
  0x0020,   /* U+1E8A6 */
  0x0020,   /* U+1E8A7 */
  0x0020,   /* U+1E8A8 */
  0x0020,   /* U+1E8A9 */
  0x0020,   /* U+1E8AA */
  0x0020,   /* U+1E8AB */
  0x0020,   /* U+1E8AC */
  0x0020,   /* U+1E8AD */
  0x0020,   /* U+1E8AE */
  0x0020,   /* U+1E8AF */
  0x0020,   /* U+1E8B0 */
  0x0020,   /* U+1E8B1 */
  0x0020,   /* U+1E8B2 */
  0x0020,   /* U+1E8B3 */
  0x0020,   /* U+1E8B4 */
  0x0020,   /* U+1E8B5 */
  0x0020,   /* U+1E8B6 */
  0x0020,   /* U+1E8B7 */
  0x0020,   /* U+1E8B8 */
  0x0020,   /* U+1E8B9 */
  0x0020,   /* U+1E8BA */
  0x0020,   /* U+1E8BB */
  0x0020,   /* U+1E8BC */
  0x0020,   /* U+1E8BD */
  0x0020,   /* U+1E8BE */
  0x0020,   /* U+1E8BF */
  0x0020,   /* U+1E8C0 */
  0x0020,   /* U+1E8C1 */
  0x0020,   /* U+1E8C2 */
  0x0020,   /* U+1E8C3 */
  0x0020,   /* U+1E8C4 */
  0x0020,   /* U+1E8C5 */
  0x0020,   /* U+1E8C6 */
  0x0020,   /* U+1E8C7 */
  0x0020,   /* U+1E8C8 */
  0x0020,   /* U+1E8C9 */
  0x0020,   /* U+1E8CA */
  0x0020,   /* U+1E8CB */
  0x0020,   /* U+1E8CC */
  0x0020,   /* U+1E8CD */
  0x0020,   /* U+1E8CE */
  0x0020,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0x0020,   /* U+1E8D7 */
  0x0020,   /* U+1E8D8 */
  0x0020,   /* U+1E8D9 */
  0x0020,   /* U+1E8DA */
  0x0020,   /* U+1E8DB */
  0x0020,   /* U+1E8DC */
  0x0020,   /* U+1E8DD */
  0x0020,   /* U+1E8DE */
  0x0020,   /* U+1E8DF */
  0x0020,   /* U+1E8E0 */
  0x0020,   /* U+1E8E1 */
  0x0020,   /* U+1E8E2 */
  0x0020,   /* U+1E8E3 */
  0x0020,   /* U+1E8E4 */
  0x0020,   /* U+1E8E5 */
  0x0020,   /* U+1E8E6 */
  0x0020,   /* U+1E8E7 */
  0x0020,   /* U+1E8E8 */
  0x0020,   /* U+1E8E9 */
  0x0020,   /* U+1E8EA */
  0x0020,   /* U+1E8EB */
  0x0020,   /* U+1E8EC */
  0x0020,   /* U+1E8ED */
  0x0020,   /* U+1E8EE */
  0x0020,   /* U+1E8EF */
  0x0020,   /* U+1E8F0 */
  0x0020,   /* U+1E8F1 */
  0x0020,   /* U+1E8F2 */
  0x0020,   /* U+1E8F3 */
  0x0020,   /* U+1E8F4 */
  0x0020,   /* U+1E8F5 */
  0x0020,   /* U+1E8F6 */
  0x0020,   /* U+1E8F7 */
  0x0020,   /* U+1E8F8 */
  0x0020,   /* U+1E8F9 */
  0x0020,   /* U+1E8FA */
  0x0020,   /* U+1E8FB */
  0x0020,   /* U+1E8FC */
  0x0020,   /* U+1E8FD */
  0x0020,   /* U+1E8FE */
  0x0020,   /* U+1E8FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1E800 */
  0x0002,   /* U+1E801 */
  0x0002,   /* U+1E802 */
  0x0002,   /* U+1E803 */
  0x0002,   /* U+1E804 */
  0x0002,   /* U+1E805 */
  0x0002,   /* U+1E806 */
  0x0002,   /* U+1E807 */
  0x0002,   /* U+1E808 */
  0x0002,   /* U+1E809 */
  0x0002,   /* U+1E80A */
  0x0002,   /* U+1E80B */
  0x0002,   /* U+1E80C */
  0x0002,   /* U+1E80D */
  0x0002,   /* U+1E80E */
  0x0002,   /* U+1E80F */
  0x0002,   /* U+1E810 */
  0x0002,   /* U+1E811 */
  0x0002,   /* U+1E812 */
  0x0002,   /* U+1E813 */
  0x0002,   /* U+1E814 */
  0x0002,   /* U+1E815 */
  0x0002,   /* U+1E816 */
  0x0002,   /* U+1E817 */
  0x0002,   /* U+1E818 */
  0x0002,   /* U+1E819 */
  0x0002,   /* U+1E81A */
  0x0002,   /* U+1E81B */
  0x0002,   /* U+1E81C */
  0x0002,   /* U+1E81D */
  0x0002,   /* U+1E81E */
  0x0002,   /* U+1E81F */
  0x0002,   /* U+1E820 */
  0x0002,   /* U+1E821 */
  0x0002,   /* U+1E822 */
  0x0002,   /* U+1E823 */
  0x0002,   /* U+1E824 */
  0x0002,   /* U+1E825 */
  0x0002,   /* U+1E826 */
  0x0002,   /* U+1E827 */
  0x0002,   /* U+1E828 */
  0x0002,   /* U+1E829 */
  0x0002,   /* U+1E82A */
  0x0002,   /* U+1E82B */
  0x0002,   /* U+1E82C */
  0x0002,   /* U+1E82D */
  0x0002,   /* U+1E82E */
  0x0002,   /* U+1E82F */
  0x0002,   /* U+1E830 */
  0x0002,   /* U+1E831 */
  0x0002,   /* U+1E832 */
  0x0002,   /* U+1E833 */
  0x0002,   /* U+1E834 */
  0x0002,   /* U+1E835 */
  0x0002,   /* U+1E836 */
  0x0002,   /* U+1E837 */
  0x0002,   /* U+1E838 */
  0x0002,   /* U+1E839 */
  0x0002,   /* U+1E83A */
  0x0002,   /* U+1E83B */
  0x0002,   /* U+1E83C */
  0x0002,   /* U+1E83D */
  0x0002,   /* U+1E83E */
  0x0002,   /* U+1E83F */
  0x0002,   /* U+1E840 */
  0x0002,   /* U+1E841 */
  0x0002,   /* U+1E842 */
  0x0002,   /* U+1E843 */
  0x0002,   /* U+1E844 */
  0x0002,   /* U+1E845 */
  0x0002,   /* U+1E846 */
  0x0002,   /* U+1E847 */
  0x0002,   /* U+1E848 */
  0x0002,   /* U+1E849 */
  0x0002,   /* U+1E84A */
  0x0002,   /* U+1E84B */
  0x0002,   /* U+1E84C */
  0x0002,   /* U+1E84D */
  0x0002,   /* U+1E84E */
  0x0002,   /* U+1E84F */
  0x0002,   /* U+1E850 */
  0x0002,   /* U+1E851 */
  0x0002,   /* U+1E852 */
  0x0002,   /* U+1E853 */
  0x0002,   /* U+1E854 */
  0x0002,   /* U+1E855 */
  0x0002,   /* U+1E856 */
  0x0002,   /* U+1E857 */
  0x0002,   /* U+1E858 */
  0x0002,   /* U+1E859 */
  0x0002,   /* U+1E85A */
  0x0002,   /* U+1E85B */
  0x0002,   /* U+1E85C */
  0x0002,   /* U+1E85D */
  0x0002,   /* U+1E85E */
  0x0002,   /* U+1E85F */
  0x0002,   /* U+1E860 */
  0x0002,   /* U+1E861 */
  0x0002,   /* U+1E862 */
  0x0002,   /* U+1E863 */
  0x0002,   /* U+1E864 */
  0x0002,   /* U+1E865 */
  0x0002,   /* U+1E866 */
  0x0002,   /* U+1E867 */
  0x0002,   /* U+1E868 */
  0x0002,   /* U+1E869 */
  0x0002,   /* U+1E86A */
  0x0002,   /* U+1E86B */
  0x0002,   /* U+1E86C */
  0x0002,   /* U+1E86D */
  0x0002,   /* U+1E86E */
  0x0002,   /* U+1E86F */
  0x0002,   /* U+1E870 */
  0x0002,   /* U+1E871 */
  0x0002,   /* U+1E872 */
  0x0002,   /* U+1E873 */
  0x0002,   /* U+1E874 */
  0x0002,   /* U+1E875 */
  0x0002,   /* U+1E876 */
  0x0002,   /* U+1E877 */
  0x0002,   /* U+1E878 */
  0x0002,   /* U+1E879 */
  0x0002,   /* U+1E87A */
  0x0002,   /* U+1E87B */
  0x0002,   /* U+1E87C */
  0x0002,   /* U+1E87D */
  0x0002,   /* U+1E87E */
  0x0002,   /* U+1E87F */
  0x0002,   /* U+1E880 */
  0x0002,   /* U+1E881 */
  0x0002,   /* U+1E882 */
  0x0002,   /* U+1E883 */
  0x0002,   /* U+1E884 */
  0x0002,   /* U+1E885 */
  0x0002,   /* U+1E886 */
  0x0002,   /* U+1E887 */
  0x0002,   /* U+1E888 */
  0x0002,   /* U+1E889 */
  0x0002,   /* U+1E88A */
  0x0002,   /* U+1E88B */
  0x0002,   /* U+1E88C */
  0x0002,   /* U+1E88D */
  0x0002,   /* U+1E88E */
  0x0002,   /* U+1E88F */
  0x0002,   /* U+1E890 */
  0x0002,   /* U+1E891 */
  0x0002,   /* U+1E892 */
  0x0002,   /* U+1E893 */
  0x0002,   /* U+1E894 */
  0x0002,   /* U+1E895 */
  0x0002,   /* U+1E896 */
  0x0002,   /* U+1E897 */
  0x0002,   /* U+1E898 */
  0x0002,   /* U+1E899 */
  0x0002,   /* U+1E89A */
  0x0002,   /* U+1E89B */
  0x0002,   /* U+1E89C */
  0x0002,   /* U+1E89D */
  0x0002,   /* U+1E89E */
  0x0002,   /* U+1E89F */
  0x0002,   /* U+1E8A0 */
  0x0002,   /* U+1E8A1 */
  0x0002,   /* U+1E8A2 */
  0x0002,   /* U+1E8A3 */
  0x0002,   /* U+1E8A4 */
  0x0002,   /* U+1E8A5 */
  0x0002,   /* U+1E8A6 */
  0x0002,   /* U+1E8A7 */
  0x0002,   /* U+1E8A8 */
  0x0002,   /* U+1E8A9 */
  0x0002,   /* U+1E8AA */
  0x0002,   /* U+1E8AB */
  0x0002,   /* U+1E8AC */
  0x0002,   /* U+1E8AD */
  0x0002,   /* U+1E8AE */
  0x0002,   /* U+1E8AF */
  0x0002,   /* U+1E8B0 */
  0x0002,   /* U+1E8B1 */
  0x0002,   /* U+1E8B2 */
  0x0002,   /* U+1E8B3 */
  0x0002,   /* U+1E8B4 */
  0x0002,   /* U+1E8B5 */
  0x0002,   /* U+1E8B6 */
  0x0002,   /* U+1E8B7 */
  0x0002,   /* U+1E8B8 */
  0x0002,   /* U+1E8B9 */
  0x0002,   /* U+1E8BA */
  0x0002,   /* U+1E8BB */
  0x0002,   /* U+1E8BC */
  0x0002,   /* U+1E8BD */
  0x0002,   /* U+1E8BE */
  0x0002,   /* U+1E8BF */
  0x0002,   /* U+1E8C0 */
  0x0002,   /* U+1E8C1 */
  0x0002,   /* U+1E8C2 */
  0x0002,   /* U+1E8C3 */
  0x0002,   /* U+1E8C4 */
  0x0002,   /* U+1E8C5 */
  0x0002,   /* U+1E8C6 */
  0x0002,   /* U+1E8C7 */
  0x0002,   /* U+1E8C8 */
  0x0002,   /* U+1E8C9 */
  0x0002,   /* U+1E8CA */
  0x0002,   /* U+1E8CB */
  0x0002,   /* U+1E8CC */
  0x0002,   /* U+1E8CD */
  0x0002,   /* U+1E8CE */
  0x0002,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0x0002,   /* U+1E8D7 */
  0x0002,   /* U+1E8D8 */
  0x0002,   /* U+1E8D9 */
  0x0002,   /* U+1E8DA */
  0x0002,   /* U+1E8DB */
  0x0002,   /* U+1E8DC */
  0x0002,   /* U+1E8DD */
  0x0002,   /* U+1E8DE */
  0x0002,   /* U+1E8DF */
  0x0002,   /* U+1E8E0 */
  0x0002,   /* U+1E8E1 */
  0x0002,   /* U+1E8E2 */
  0x0002,   /* U+1E8E3 */
  0x0002,   /* U+1E8E4 */
  0x0002,   /* U+1E8E5 */
  0x0002,   /* U+1E8E6 */
  0x0002,   /* U+1E8E7 */
  0x0002,   /* U+1E8E8 */
  0x0002,   /* U+1E8E9 */
  0x0002,   /* U+1E8EA */
  0x0002,   /* U+1E8EB */
  0x0002,   /* U+1E8EC */
  0x0002,   /* U+1E8ED */
  0x0002,   /* U+1E8EE */
  0x0002,   /* U+1E8EF */
  0x0002,   /* U+1E8F0 */
  0x0002,   /* U+1E8F1 */
  0x0002,   /* U+1E8F2 */
  0x0002,   /* U+1E8F3 */
  0x0002,   /* U+1E8F4 */
  0x0002,   /* U+1E8F5 */
  0x0002,   /* U+1E8F6 */
  0x0002,   /* U+1E8F7 */
  0x0002,   /* U+1E8F8 */
  0x0002,   /* U+1E8F9 */
  0x0002,   /* U+1E8FA */
  0x0002,   /* U+1E8FB */
  0x0002,   /* U+1E8FC */
  0x0002,   /* U+1E8FD */
  0x0002,   /* U+1E8FE */
  0x0002,   /* U+1E8FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1E800 */
  0x0000,   /* U+1E801 */
  0x0000,   /* U+1E802 */
  0x0000,   /* U+1E803 */
  0x0000,   /* U+1E804 */
  0x0000,   /* U+1E805 */
  0x0000,   /* U+1E806 */
  0x0000,   /* U+1E807 */
  0x0000,   /* U+1E808 */
  0x0000,   /* U+1E809 */
  0x0000,   /* U+1E80A */
  0x0000,   /* U+1E80B */
  0x0000,   /* U+1E80C */
  0x0000,   /* U+1E80D */
  0x0000,   /* U+1E80E */
  0x0000,   /* U+1E80F */
  0x0000,   /* U+1E810 */
  0x0000,   /* U+1E811 */
  0x0000,   /* U+1E812 */
  0x0000,   /* U+1E813 */
  0x0000,   /* U+1E814 */
  0x0000,   /* U+1E815 */
  0x0000,   /* U+1E816 */
  0x0000,   /* U+1E817 */
  0x0000,   /* U+1E818 */
  0x0000,   /* U+1E819 */
  0x0000,   /* U+1E81A */
  0x0000,   /* U+1E81B */
  0x0000,   /* U+1E81C */
  0x0000,   /* U+1E81D */
  0x0000,   /* U+1E81E */
  0x0000,   /* U+1E81F */
  0x0000,   /* U+1E820 */
  0x0000,   /* U+1E821 */
  0x0000,   /* U+1E822 */
  0x0000,   /* U+1E823 */
  0x0000,   /* U+1E824 */
  0x0000,   /* U+1E825 */
  0x0000,   /* U+1E826 */
  0x0000,   /* U+1E827 */
  0x0000,   /* U+1E828 */
  0x0000,   /* U+1E829 */
  0x0000,   /* U+1E82A */
  0x0000,   /* U+1E82B */
  0x0000,   /* U+1E82C */
  0x0000,   /* U+1E82D */
  0x0000,   /* U+1E82E */
  0x0000,   /* U+1E82F */
  0x0000,   /* U+1E830 */
  0x0000,   /* U+1E831 */
  0x0000,   /* U+1E832 */
  0x0000,   /* U+1E833 */
  0x0000,   /* U+1E834 */
  0x0000,   /* U+1E835 */
  0x0000,   /* U+1E836 */
  0x0000,   /* U+1E837 */
  0x0000,   /* U+1E838 */
  0x0000,   /* U+1E839 */
  0x0000,   /* U+1E83A */
  0x0000,   /* U+1E83B */
  0x0000,   /* U+1E83C */
  0x0000,   /* U+1E83D */
  0x0000,   /* U+1E83E */
  0x0000,   /* U+1E83F */
  0x0000,   /* U+1E840 */
  0x0000,   /* U+1E841 */
  0x0000,   /* U+1E842 */
  0x0000,   /* U+1E843 */
  0x0000,   /* U+1E844 */
  0x0000,   /* U+1E845 */
  0x0000,   /* U+1E846 */
  0x0000,   /* U+1E847 */
  0x0000,   /* U+1E848 */
  0x0000,   /* U+1E849 */
  0x0000,   /* U+1E84A */
  0x0000,   /* U+1E84B */
  0x0000,   /* U+1E84C */
  0x0000,   /* U+1E84D */
  0x0000,   /* U+1E84E */
  0x0000,   /* U+1E84F */
  0x0000,   /* U+1E850 */
  0x0000,   /* U+1E851 */
  0x0000,   /* U+1E852 */
  0x0000,   /* U+1E853 */
  0x0000,   /* U+1E854 */
  0x0000,   /* U+1E855 */
  0x0000,   /* U+1E856 */
  0x0000,   /* U+1E857 */
  0x0000,   /* U+1E858 */
  0x0000,   /* U+1E859 */
  0x0000,   /* U+1E85A */
  0x0000,   /* U+1E85B */
  0x0000,   /* U+1E85C */
  0x0000,   /* U+1E85D */
  0x0000,   /* U+1E85E */
  0x0000,   /* U+1E85F */
  0x0000,   /* U+1E860 */
  0x0000,   /* U+1E861 */
  0x0000,   /* U+1E862 */
  0x0000,   /* U+1E863 */
  0x0000,   /* U+1E864 */
  0x0000,   /* U+1E865 */
  0x0000,   /* U+1E866 */
  0x0000,   /* U+1E867 */
  0x0000,   /* U+1E868 */
  0x0000,   /* U+1E869 */
  0x0000,   /* U+1E86A */
  0x0000,   /* U+1E86B */
  0x0000,   /* U+1E86C */
  0x0000,   /* U+1E86D */
  0x0000,   /* U+1E86E */
  0x0000,   /* U+1E86F */
  0x0000,   /* U+1E870 */
  0x0000,   /* U+1E871 */
  0x0000,   /* U+1E872 */
  0x0000,   /* U+1E873 */
  0x0000,   /* U+1E874 */
  0x0000,   /* U+1E875 */
  0x0000,   /* U+1E876 */
  0x0000,   /* U+1E877 */
  0x0000,   /* U+1E878 */
  0x0000,   /* U+1E879 */
  0x0000,   /* U+1E87A */
  0x0000,   /* U+1E87B */
  0x0000,   /* U+1E87C */
  0x0000,   /* U+1E87D */
  0x0000,   /* U+1E87E */
  0x0000,   /* U+1E87F */
  0x0000,   /* U+1E880 */
  0x0000,   /* U+1E881 */
  0x0000,   /* U+1E882 */
  0x0000,   /* U+1E883 */
  0x0000,   /* U+1E884 */
  0x0000,   /* U+1E885 */
  0x0000,   /* U+1E886 */
  0x0000,   /* U+1E887 */
  0x0000,   /* U+1E888 */
  0x0000,   /* U+1E889 */
  0x0000,   /* U+1E88A */
  0x0000,   /* U+1E88B */
  0x0000,   /* U+1E88C */
  0x0000,   /* U+1E88D */
  0x0000,   /* U+1E88E */
  0x0000,   /* U+1E88F */
  0x0000,   /* U+1E890 */
  0x0000,   /* U+1E891 */
  0x0000,   /* U+1E892 */
  0x0000,   /* U+1E893 */
  0x0000,   /* U+1E894 */
  0x0000,   /* U+1E895 */
  0x0000,   /* U+1E896 */
  0x0000,   /* U+1E897 */
  0x0000,   /* U+1E898 */
  0x0000,   /* U+1E899 */
  0x0000,   /* U+1E89A */
  0x0000,   /* U+1E89B */
  0x0000,   /* U+1E89C */
  0x0000,   /* U+1E89D */
  0x0000,   /* U+1E89E */
  0x0000,   /* U+1E89F */
  0x0000,   /* U+1E8A0 */
  0x0000,   /* U+1E8A1 */
  0x0000,   /* U+1E8A2 */
  0x0000,   /* U+1E8A3 */
  0x0000,   /* U+1E8A4 */
  0x0000,   /* U+1E8A5 */
  0x0000,   /* U+1E8A6 */
  0x0000,   /* U+1E8A7 */
  0x0000,   /* U+1E8A8 */
  0x0000,   /* U+1E8A9 */
  0x0000,   /* U+1E8AA */
  0x0000,   /* U+1E8AB */
  0x0000,   /* U+1E8AC */
  0x0000,   /* U+1E8AD */
  0x0000,   /* U+1E8AE */
  0x0000,   /* U+1E8AF */
  0x0000,   /* U+1E8B0 */
  0x0000,   /* U+1E8B1 */
  0x0000,   /* U+1E8B2 */
  0x0000,   /* U+1E8B3 */
  0x0000,   /* U+1E8B4 */
  0x0000,   /* U+1E8B5 */
  0x0000,   /* U+1E8B6 */
  0x0000,   /* U+1E8B7 */
  0x0000,   /* U+1E8B8 */
  0x0000,   /* U+1E8B9 */
  0x0000,   /* U+1E8BA */
  0x0000,   /* U+1E8BB */
  0x0000,   /* U+1E8BC */
  0x0000,   /* U+1E8BD */
  0x0000,   /* U+1E8BE */
  0x0000,   /* U+1E8BF */
  0x0000,   /* U+1E8C0 */
  0x0000,   /* U+1E8C1 */
  0x0000,   /* U+1E8C2 */
  0x0000,   /* U+1E8C3 */
  0x0000,   /* U+1E8C4 */
  0xE8C5,   /* U+1E8C5 */
  0xE8C6,   /* U+1E8C6 */
  0x0000,   /* U+1E8C7 */
  0x0000,   /* U+1E8C8 */
  0x0000,   /* U+1E8C9 */
  0x0000,   /* U+1E8CA */
  0x0000,   /* U+1E8CB */
  0x0000,   /* U+1E8CC */
  0x0000,   /* U+1E8CD */
  0x0000,   /* U+1E8CE */
  0x0000,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0xE8D7,   /* U+1E8D7 */
  0xE8D8,   /* U+1E8D8 */
  0xE8D9,   /* U+1E8D9 */
  0xE8DA,   /* U+1E8DA */
  0xE8DB,   /* U+1E8DB */
  0xE8DC,   /* U+1E8DC */
  0xE8DD,   /* U+1E8DD */
  0xE8DE,   /* U+1E8DE */
  0xE8DF,   /* U+1E8DF */
  0xE8E0,   /* U+1E8E0 */
  0xE8E1,   /* U+1E8E1 */
  0xE8E2,   /* U+1E8E2 */
  0xE8E3,   /* U+1E8E3 */
  0xE8E4,   /* U+1E8E4 */
  0xE8E5,   /* U+1E8E5 */
  0xE8E6,   /* U+1E8E6 */
  0xE8E7,   /* U+1E8E7 */
  0xE8E8,   /* U+1E8E8 */
  0xE8E9,   /* U+1E8E9 */
  0xE8EA,   /* U+1E8EA */
  0xE8EB,   /* U+1E8EB */
  0xE8EC,   /* U+1E8EC */
  0xE8ED,   /* U+1E8ED */
  0xE8EE,   /* U+1E8EE */
  0xE8EF,   /* U+1E8EF */
  0xE8F0,   /* U+1E8F0 */
  0xE8F1,   /* U+1E8F1 */
  0xE8F2,   /* U+1E8F2 */
  0xE8F3,   /* U+1E8F3 */
  0xE8F4,   /* U+1E8F4 */
  0xE8F5,   /* U+1E8F5 */
  0xE8F6,   /* U+1E8F6 */
  0xE8F7,   /* U+1E8F7 */
  0xE8F8,   /* U+1E8F8 */
  0xE8F9,   /* U+1E8F9 */
  0xE8FA,   /* U+1E8FA */
  0xE8FB,   /* U+1E8FB */
  0xE8FC,   /* U+1E8FC */
  0xE8FD,   /* U+1E8FD */
  0xE8FE,   /* U+1E8FE */
  0xE8FF,   /* U+1E8FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1E800 */
  0x0000,   /* U+1E801 */
  0x0000,   /* U+1E802 */
  0x0000,   /* U+1E803 */
  0x0000,   /* U+1E804 */
  0x0000,   /* U+1E805 */
  0x0000,   /* U+1E806 */
  0x0000,   /* U+1E807 */
  0x0000,   /* U+1E808 */
  0x0000,   /* U+1E809 */
  0x0000,   /* U+1E80A */
  0x0000,   /* U+1E80B */
  0x0000,   /* U+1E80C */
  0x0000,   /* U+1E80D */
  0x0000,   /* U+1E80E */
  0x0000,   /* U+1E80F */
  0x0000,   /* U+1E810 */
  0x0000,   /* U+1E811 */
  0x0000,   /* U+1E812 */
  0x0000,   /* U+1E813 */
  0x0000,   /* U+1E814 */
  0x0000,   /* U+1E815 */
  0x0000,   /* U+1E816 */
  0x0000,   /* U+1E817 */
  0x0000,   /* U+1E818 */
  0x0000,   /* U+1E819 */
  0x0000,   /* U+1E81A */
  0x0000,   /* U+1E81B */
  0x0000,   /* U+1E81C */
  0x0000,   /* U+1E81D */
  0x0000,   /* U+1E81E */
  0x0000,   /* U+1E81F */
  0x0000,   /* U+1E820 */
  0x0000,   /* U+1E821 */
  0x0000,   /* U+1E822 */
  0x0000,   /* U+1E823 */
  0x0000,   /* U+1E824 */
  0x0000,   /* U+1E825 */
  0x0000,   /* U+1E826 */
  0x0000,   /* U+1E827 */
  0x0000,   /* U+1E828 */
  0x0000,   /* U+1E829 */
  0x0000,   /* U+1E82A */
  0x0000,   /* U+1E82B */
  0x0000,   /* U+1E82C */
  0x0000,   /* U+1E82D */
  0x0000,   /* U+1E82E */
  0x0000,   /* U+1E82F */
  0x0000,   /* U+1E830 */
  0x0000,   /* U+1E831 */
  0x0000,   /* U+1E832 */
  0x0000,   /* U+1E833 */
  0x0000,   /* U+1E834 */
  0x0000,   /* U+1E835 */
  0x0000,   /* U+1E836 */
  0x0000,   /* U+1E837 */
  0x0000,   /* U+1E838 */
  0x0000,   /* U+1E839 */
  0x0000,   /* U+1E83A */
  0x0000,   /* U+1E83B */
  0x0000,   /* U+1E83C */
  0x0000,   /* U+1E83D */
  0x0000,   /* U+1E83E */
  0x0000,   /* U+1E83F */
  0x0000,   /* U+1E840 */
  0x0000,   /* U+1E841 */
  0x0000,   /* U+1E842 */
  0x0000,   /* U+1E843 */
  0x0000,   /* U+1E844 */
  0x0000,   /* U+1E845 */
  0x0000,   /* U+1E846 */
  0x0000,   /* U+1E847 */
  0x0000,   /* U+1E848 */
  0x0000,   /* U+1E849 */
  0x0000,   /* U+1E84A */
  0x0000,   /* U+1E84B */
  0x0000,   /* U+1E84C */
  0x0000,   /* U+1E84D */
  0x0000,   /* U+1E84E */
  0x0000,   /* U+1E84F */
  0x0000,   /* U+1E850 */
  0x0000,   /* U+1E851 */
  0x0000,   /* U+1E852 */
  0x0000,   /* U+1E853 */
  0x0000,   /* U+1E854 */
  0x0000,   /* U+1E855 */
  0x0000,   /* U+1E856 */
  0x0000,   /* U+1E857 */
  0x0000,   /* U+1E858 */
  0x0000,   /* U+1E859 */
  0x0000,   /* U+1E85A */
  0x0000,   /* U+1E85B */
  0x0000,   /* U+1E85C */
  0x0000,   /* U+1E85D */
  0x0000,   /* U+1E85E */
  0x0000,   /* U+1E85F */
  0x0000,   /* U+1E860 */
  0x0000,   /* U+1E861 */
  0x0000,   /* U+1E862 */
  0x0000,   /* U+1E863 */
  0x0000,   /* U+1E864 */
  0x0000,   /* U+1E865 */
  0x0000,   /* U+1E866 */
  0x0000,   /* U+1E867 */
  0x0000,   /* U+1E868 */
  0x0000,   /* U+1E869 */
  0x0000,   /* U+1E86A */
  0x0000,   /* U+1E86B */
  0x0000,   /* U+1E86C */
  0x0000,   /* U+1E86D */
  0x0000,   /* U+1E86E */
  0x0000,   /* U+1E86F */
  0x0000,   /* U+1E870 */
  0x0000,   /* U+1E871 */
  0x0000,   /* U+1E872 */
  0x0000,   /* U+1E873 */
  0x0000,   /* U+1E874 */
  0x0000,   /* U+1E875 */
  0x0000,   /* U+1E876 */
  0x0000,   /* U+1E877 */
  0x0000,   /* U+1E878 */
  0x0000,   /* U+1E879 */
  0x0000,   /* U+1E87A */
  0x0000,   /* U+1E87B */
  0x0000,   /* U+1E87C */
  0x0000,   /* U+1E87D */
  0x0000,   /* U+1E87E */
  0x0000,   /* U+1E87F */
  0x0000,   /* U+1E880 */
  0x0000,   /* U+1E881 */
  0x0000,   /* U+1E882 */
  0x0000,   /* U+1E883 */
  0x0000,   /* U+1E884 */
  0x0000,   /* U+1E885 */
  0x0000,   /* U+1E886 */
  0x0000,   /* U+1E887 */
  0x0000,   /* U+1E888 */
  0x0000,   /* U+1E889 */
  0x0000,   /* U+1E88A */
  0x0000,   /* U+1E88B */
  0x0000,   /* U+1E88C */
  0x0000,   /* U+1E88D */
  0x0000,   /* U+1E88E */
  0x0000,   /* U+1E88F */
  0x0000,   /* U+1E890 */
  0x0000,   /* U+1E891 */
  0x0000,   /* U+1E892 */
  0x0000,   /* U+1E893 */
  0x0000,   /* U+1E894 */
  0x0000,   /* U+1E895 */
  0x0000,   /* U+1E896 */
  0x0000,   /* U+1E897 */
  0x0000,   /* U+1E898 */
  0x0000,   /* U+1E899 */
  0x0000,   /* U+1E89A */
  0x0000,   /* U+1E89B */
  0x0000,   /* U+1E89C */
  0x0000,   /* U+1E89D */
  0x0000,   /* U+1E89E */
  0x0000,   /* U+1E89F */
  0x0000,   /* U+1E8A0 */
  0x0000,   /* U+1E8A1 */
  0x0000,   /* U+1E8A2 */
  0x0000,   /* U+1E8A3 */
  0x0000,   /* U+1E8A4 */
  0x0000,   /* U+1E8A5 */
  0x0000,   /* U+1E8A6 */
  0x0000,   /* U+1E8A7 */
  0x0000,   /* U+1E8A8 */
  0x0000,   /* U+1E8A9 */
  0x0000,   /* U+1E8AA */
  0x0000,   /* U+1E8AB */
  0x0000,   /* U+1E8AC */
  0x0000,   /* U+1E8AD */
  0x0000,   /* U+1E8AE */
  0x0000,   /* U+1E8AF */
  0x0000,   /* U+1E8B0 */
  0x0000,   /* U+1E8B1 */
  0x0000,   /* U+1E8B2 */
  0x0000,   /* U+1E8B3 */
  0x0000,   /* U+1E8B4 */
  0x0000,   /* U+1E8B5 */
  0x0000,   /* U+1E8B6 */
  0x0000,   /* U+1E8B7 */
  0x0000,   /* U+1E8B8 */
  0x0000,   /* U+1E8B9 */
  0x0000,   /* U+1E8BA */
  0x0000,   /* U+1E8BB */
  0x0000,   /* U+1E8BC */
  0x0000,   /* U+1E8BD */
  0x0000,   /* U+1E8BE */
  0x0000,   /* U+1E8BF */
  0x0000,   /* U+1E8C0 */
  0x0000,   /* U+1E8C1 */
  0x0000,   /* U+1E8C2 */
  0x0000,   /* U+1E8C3 */
  0x0000,   /* U+1E8C4 */
  0x0000,   /* U+1E8C5 */
  0x0000,   /* U+1E8C6 */
  0x0000,   /* U+1E8C7 */
  0x0000,   /* U+1E8C8 */
  0x0000,   /* U+1E8C9 */
  0x0000,   /* U+1E8CA */
  0x0000,   /* U+1E8CB */
  0x0000,   /* U+1E8CC */
  0x0000,   /* U+1E8CD */
  0x0000,   /* U+1E8CE */
  0x0000,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0x0000,   /* U+1E8D7 */
  0x0000,   /* U+1E8D8 */
  0x0000,   /* U+1E8D9 */
  0x0000,   /* U+1E8DA */
  0x0000,   /* U+1E8DB */
  0x0000,   /* U+1E8DC */
  0x0000,   /* U+1E8DD */
  0x0000,   /* U+1E8DE */
  0x0000,   /* U+1E8DF */
  0x0000,   /* U+1E8E0 */
  0x0000,   /* U+1E8E1 */
  0x0000,   /* U+1E8E2 */
  0x0000,   /* U+1E8E3 */
  0x0000,   /* U+1E8E4 */
  0x0000,   /* U+1E8E5 */
  0x0000,   /* U+1E8E6 */
  0x0000,   /* U+1E8E7 */
  0x0000,   /* U+1E8E8 */
  0x0000,   /* U+1E8E9 */
  0x0000,   /* U+1E8EA */
  0x0000,   /* U+1E8EB */
  0x0000,   /* U+1E8EC */
  0x0000,   /* U+1E8ED */
  0x0000,   /* U+1E8EE */
  0x0000,   /* U+1E8EF */
  0x0000,   /* U+1E8F0 */
  0x0000,   /* U+1E8F1 */
  0x0000,   /* U+1E8F2 */
  0x0000,   /* U+1E8F3 */
  0x0000,   /* U+1E8F4 */
  0x0000,   /* U+1E8F5 */
  0x0000,   /* U+1E8F6 */
  0x0000,   /* U+1E8F7 */
  0x0000,   /* U+1E8F8 */
  0x0000,   /* U+1E8F9 */
  0x0000,   /* U+1E8FA */
  0x0000,   /* U+1E8FB */
  0x0000,   /* U+1E8FC */
  0x0000,   /* U+1E8FD */
  0x0000,   /* U+1E8FE */
  0x0000,   /* U+1E8FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1E800 */
  0x0000,   /* U+1E801 */
  0x0000,   /* U+1E802 */
  0x0000,   /* U+1E803 */
  0x0000,   /* U+1E804 */
  0x0000,   /* U+1E805 */
  0x0000,   /* U+1E806 */
  0x0000,   /* U+1E807 */
  0x0000,   /* U+1E808 */
  0x0000,   /* U+1E809 */
  0x0000,   /* U+1E80A */
  0x0000,   /* U+1E80B */
  0x0000,   /* U+1E80C */
  0x0000,   /* U+1E80D */
  0x0000,   /* U+1E80E */
  0x0000,   /* U+1E80F */
  0x0000,   /* U+1E810 */
  0x0000,   /* U+1E811 */
  0x0000,   /* U+1E812 */
  0x0000,   /* U+1E813 */
  0x0000,   /* U+1E814 */
  0x0000,   /* U+1E815 */
  0x0000,   /* U+1E816 */
  0x0000,   /* U+1E817 */
  0x0000,   /* U+1E818 */
  0x0000,   /* U+1E819 */
  0x0000,   /* U+1E81A */
  0x0000,   /* U+1E81B */
  0x0000,   /* U+1E81C */
  0x0000,   /* U+1E81D */
  0x0000,   /* U+1E81E */
  0x0000,   /* U+1E81F */
  0x0000,   /* U+1E820 */
  0x0000,   /* U+1E821 */
  0x0000,   /* U+1E822 */
  0x0000,   /* U+1E823 */
  0x0000,   /* U+1E824 */
  0x0000,   /* U+1E825 */
  0x0000,   /* U+1E826 */
  0x0000,   /* U+1E827 */
  0x0000,   /* U+1E828 */
  0x0000,   /* U+1E829 */
  0x0000,   /* U+1E82A */
  0x0000,   /* U+1E82B */
  0x0000,   /* U+1E82C */
  0x0000,   /* U+1E82D */
  0x0000,   /* U+1E82E */
  0x0000,   /* U+1E82F */
  0x0000,   /* U+1E830 */
  0x0000,   /* U+1E831 */
  0x0000,   /* U+1E832 */
  0x0000,   /* U+1E833 */
  0x0000,   /* U+1E834 */
  0x0000,   /* U+1E835 */
  0x0000,   /* U+1E836 */
  0x0000,   /* U+1E837 */
  0x0000,   /* U+1E838 */
  0x0000,   /* U+1E839 */
  0x0000,   /* U+1E83A */
  0x0000,   /* U+1E83B */
  0x0000,   /* U+1E83C */
  0x0000,   /* U+1E83D */
  0x0000,   /* U+1E83E */
  0x0000,   /* U+1E83F */
  0x0000,   /* U+1E840 */
  0x0000,   /* U+1E841 */
  0x0000,   /* U+1E842 */
  0x0000,   /* U+1E843 */
  0x0000,   /* U+1E844 */
  0x0000,   /* U+1E845 */
  0x0000,   /* U+1E846 */
  0x0000,   /* U+1E847 */
  0x0000,   /* U+1E848 */
  0x0000,   /* U+1E849 */
  0x0000,   /* U+1E84A */
  0x0000,   /* U+1E84B */
  0x0000,   /* U+1E84C */
  0x0000,   /* U+1E84D */
  0x0000,   /* U+1E84E */
  0x0000,   /* U+1E84F */
  0x0000,   /* U+1E850 */
  0x0000,   /* U+1E851 */
  0x0000,   /* U+1E852 */
  0x0000,   /* U+1E853 */
  0x0000,   /* U+1E854 */
  0x0000,   /* U+1E855 */
  0x0000,   /* U+1E856 */
  0x0000,   /* U+1E857 */
  0x0000,   /* U+1E858 */
  0x0000,   /* U+1E859 */
  0x0000,   /* U+1E85A */
  0x0000,   /* U+1E85B */
  0x0000,   /* U+1E85C */
  0x0000,   /* U+1E85D */
  0x0000,   /* U+1E85E */
  0x0000,   /* U+1E85F */
  0x0000,   /* U+1E860 */
  0x0000,   /* U+1E861 */
  0x0000,   /* U+1E862 */
  0x0000,   /* U+1E863 */
  0x0000,   /* U+1E864 */
  0x0000,   /* U+1E865 */
  0x0000,   /* U+1E866 */
  0x0000,   /* U+1E867 */
  0x0000,   /* U+1E868 */
  0x0000,   /* U+1E869 */
  0x0000,   /* U+1E86A */
  0x0000,   /* U+1E86B */
  0x0000,   /* U+1E86C */
  0x0000,   /* U+1E86D */
  0x0000,   /* U+1E86E */
  0x0000,   /* U+1E86F */
  0x0000,   /* U+1E870 */
  0x0000,   /* U+1E871 */
  0x0000,   /* U+1E872 */
  0x0000,   /* U+1E873 */
  0x0000,   /* U+1E874 */
  0x0000,   /* U+1E875 */
  0x0000,   /* U+1E876 */
  0x0000,   /* U+1E877 */
  0x0000,   /* U+1E878 */
  0x0000,   /* U+1E879 */
  0x0000,   /* U+1E87A */
  0x0000,   /* U+1E87B */
  0x0000,   /* U+1E87C */
  0x0000,   /* U+1E87D */
  0x0000,   /* U+1E87E */
  0x0000,   /* U+1E87F */
  0x0000,   /* U+1E880 */
  0x0000,   /* U+1E881 */
  0x0000,   /* U+1E882 */
  0x0000,   /* U+1E883 */
  0x0000,   /* U+1E884 */
  0x0000,   /* U+1E885 */
  0x0000,   /* U+1E886 */
  0x0000,   /* U+1E887 */
  0x0000,   /* U+1E888 */
  0x0000,   /* U+1E889 */
  0x0000,   /* U+1E88A */
  0x0000,   /* U+1E88B */
  0x0000,   /* U+1E88C */
  0x0000,   /* U+1E88D */
  0x0000,   /* U+1E88E */
  0x0000,   /* U+1E88F */
  0x0000,   /* U+1E890 */
  0x0000,   /* U+1E891 */
  0x0000,   /* U+1E892 */
  0x0000,   /* U+1E893 */
  0x0000,   /* U+1E894 */
  0x0000,   /* U+1E895 */
  0x0000,   /* U+1E896 */
  0x0000,   /* U+1E897 */
  0x0000,   /* U+1E898 */
  0x0000,   /* U+1E899 */
  0x0000,   /* U+1E89A */
  0x0000,   /* U+1E89B */
  0x0000,   /* U+1E89C */
  0x0000,   /* U+1E89D */
  0x0000,   /* U+1E89E */
  0x0000,   /* U+1E89F */
  0x0000,   /* U+1E8A0 */
  0x0000,   /* U+1E8A1 */
  0x0000,   /* U+1E8A2 */
  0x0000,   /* U+1E8A3 */
  0x0000,   /* U+1E8A4 */
  0x0000,   /* U+1E8A5 */
  0x0000,   /* U+1E8A6 */
  0x0000,   /* U+1E8A7 */
  0x0000,   /* U+1E8A8 */
  0x0000,   /* U+1E8A9 */
  0x0000,   /* U+1E8AA */
  0x0000,   /* U+1E8AB */
  0x0000,   /* U+1E8AC */
  0x0000,   /* U+1E8AD */
  0x0000,   /* U+1E8AE */
  0x0000,   /* U+1E8AF */
  0x0000,   /* U+1E8B0 */
  0x0000,   /* U+1E8B1 */
  0x0000,   /* U+1E8B2 */
  0x0000,   /* U+1E8B3 */
  0x0000,   /* U+1E8B4 */
  0x0000,   /* U+1E8B5 */
  0x0000,   /* U+1E8B6 */
  0x0000,   /* U+1E8B7 */
  0x0000,   /* U+1E8B8 */
  0x0000,   /* U+1E8B9 */
  0x0000,   /* U+1E8BA */
  0x0000,   /* U+1E8BB */
  0x0000,   /* U+1E8BC */
  0x0000,   /* U+1E8BD */
  0x0000,   /* U+1E8BE */
  0x0000,   /* U+1E8BF */
  0x0000,   /* U+1E8C0 */
  0x0000,   /* U+1E8C1 */
  0x0000,   /* U+1E8C2 */
  0x0000,   /* U+1E8C3 */
  0x0000,   /* U+1E8C4 */
  0x0000,   /* U+1E8C5 */
  0x0000,   /* U+1E8C6 */
  0x0000,   /* U+1E8C7 */
  0x0000,   /* U+1E8C8 */
  0x0000,   /* U+1E8C9 */
  0x0000,   /* U+1E8CA */
  0x0000,   /* U+1E8CB */
  0x0000,   /* U+1E8CC */
  0x0000,   /* U+1E8CD */
  0x0000,   /* U+1E8CE */
  0x0000,   /* U+1E8CF */
  0x0000,   /* U+1E8D0 */
  0x0000,   /* U+1E8D1 */
  0x0000,   /* U+1E8D2 */
  0x0000,   /* U+1E8D3 */
  0x0000,   /* U+1E8D4 */
  0x0000,   /* U+1E8D5 */
  0x0000,   /* U+1E8D6 */
  0x0000,   /* U+1E8D7 */
  0x0000,   /* U+1E8D8 */
  0x0000,   /* U+1E8D9 */
  0x0000,   /* U+1E8DA */
  0x0000,   /* U+1E8DB */
  0x0000,   /* U+1E8DC */
  0x0000,   /* U+1E8DD */
  0x0000,   /* U+1E8DE */
  0x0000,   /* U+1E8DF */
  0x0000,   /* U+1E8E0 */
  0x0000,   /* U+1E8E1 */
  0x0000,   /* U+1E8E2 */
  0x0000,   /* U+1E8E3 */
  0x0000,   /* U+1E8E4 */
  0x0000,   /* U+1E8E5 */
  0x0000,   /* U+1E8E6 */
  0x0000,   /* U+1E8E7 */
  0x0000,   /* U+1E8E8 */
  0x0000,   /* U+1E8E9 */
  0x0000,   /* U+1E8EA */
  0x0000,   /* U+1E8EB */
  0x0000,   /* U+1E8EC */
  0x0000,   /* U+1E8ED */
  0x0000,   /* U+1E8EE */
  0x0000,   /* U+1E8EF */
  0x0000,   /* U+1E8F0 */
  0x0000,   /* U+1E8F1 */
  0x0000,   /* U+1E8F2 */
  0x0000,   /* U+1E8F3 */
  0x0000,   /* U+1E8F4 */
  0x0000,   /* U+1E8F5 */
  0x0000,   /* U+1E8F6 */
  0x0000,   /* U+1E8F7 */
  0x0000,   /* U+1E8F8 */
  0x0000,   /* U+1E8F9 */
  0x0000,   /* U+1E8FA */
  0x0000,   /* U+1E8FB */
  0x0000,   /* U+1E8FC */
  0x0000,   /* U+1E8FD */
  0x0000,   /* U+1E8FE */
  0x0000,   /* U+1E8FF */
};

uint16 uca900_p1E9[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3BD3,   /* U+1E900 */
  0x3BD4,   /* U+1E901 */
  0x3BD5,   /* U+1E902 */
  0x3BD6,   /* U+1E903 */
  0x3BD7,   /* U+1E904 */
  0x3BD8,   /* U+1E905 */
  0x3BD9,   /* U+1E906 */
  0x3BDA,   /* U+1E907 */
  0x3BDB,   /* U+1E908 */
  0x3BDC,   /* U+1E909 */
  0x3BDD,   /* U+1E90A */
  0x3BDE,   /* U+1E90B */
  0x3BDF,   /* U+1E90C */
  0x3BE0,   /* U+1E90D */
  0x3BE1,   /* U+1E90E */
  0x3BE2,   /* U+1E90F */
  0x3BE3,   /* U+1E910 */
  0x3BE4,   /* U+1E911 */
  0x3BE5,   /* U+1E912 */
  0x3BE6,   /* U+1E913 */
  0x3BE7,   /* U+1E914 */
  0x3BE8,   /* U+1E915 */
  0x3BE9,   /* U+1E916 */
  0x3BEA,   /* U+1E917 */
  0x3BEB,   /* U+1E918 */
  0x3BEC,   /* U+1E919 */
  0x3BED,   /* U+1E91A */
  0x3BEE,   /* U+1E91B */
  0x3BEF,   /* U+1E91C */
  0x3BF0,   /* U+1E91D */
  0x3BF1,   /* U+1E91E */
  0x3BF2,   /* U+1E91F */
  0x3BF3,   /* U+1E920 */
  0x3BF4,   /* U+1E921 */
  0x3BD3,   /* U+1E922 */
  0x3BD4,   /* U+1E923 */
  0x3BD5,   /* U+1E924 */
  0x3BD6,   /* U+1E925 */
  0x3BD7,   /* U+1E926 */
  0x3BD8,   /* U+1E927 */
  0x3BD9,   /* U+1E928 */
  0x3BDA,   /* U+1E929 */
  0x3BDB,   /* U+1E92A */
  0x3BDC,   /* U+1E92B */
  0x3BDD,   /* U+1E92C */
  0x3BDE,   /* U+1E92D */
  0x3BDF,   /* U+1E92E */
  0x3BE0,   /* U+1E92F */
  0x3BE1,   /* U+1E930 */
  0x3BE2,   /* U+1E931 */
  0x3BE3,   /* U+1E932 */
  0x3BE4,   /* U+1E933 */
  0x3BE5,   /* U+1E934 */
  0x3BE6,   /* U+1E935 */
  0x3BE7,   /* U+1E936 */
  0x3BE8,   /* U+1E937 */
  0x3BE9,   /* U+1E938 */
  0x3BEA,   /* U+1E939 */
  0x3BEB,   /* U+1E93A */
  0x3BEC,   /* U+1E93B */
  0x3BED,   /* U+1E93C */
  0x3BEE,   /* U+1E93D */
  0x3BEF,   /* U+1E93E */
  0x3BF0,   /* U+1E93F */
  0x3BF1,   /* U+1E940 */
  0x3BF2,   /* U+1E941 */
  0x3BF3,   /* U+1E942 */
  0x3BF4,   /* U+1E943 */
  0x0000,   /* U+1E944 */
  0x0000,   /* U+1E945 */
  0x0000,   /* U+1E946 */
  0x0000,   /* U+1E947 */
  0x0000,   /* U+1E948 */
  0x0000,   /* U+1E949 */
  0x0000,   /* U+1E94A */
  0xFBC3,   /* U+1E94B */
  0xFBC3,   /* U+1E94C */
  0xFBC3,   /* U+1E94D */
  0xFBC3,   /* U+1E94E */
  0xFBC3,   /* U+1E94F */
  0x1C3D,   /* U+1E950 */
  0x1C3E,   /* U+1E951 */
  0x1C3F,   /* U+1E952 */
  0x1C40,   /* U+1E953 */
  0x1C41,   /* U+1E954 */
  0x1C42,   /* U+1E955 */
  0x1C43,   /* U+1E956 */
  0x1C44,   /* U+1E957 */
  0x1C45,   /* U+1E958 */
  0x1C46,   /* U+1E959 */
  0xFBC3,   /* U+1E95A */
  0xFBC3,   /* U+1E95B */
  0xFBC3,   /* U+1E95C */
  0xFBC3,   /* U+1E95D */
  0x0265,   /* U+1E95E */
  0x0274,   /* U+1E95F */
  0xFBC3,   /* U+1E960 */
  0xFBC3,   /* U+1E961 */
  0xFBC3,   /* U+1E962 */
  0xFBC3,   /* U+1E963 */
  0xFBC3,   /* U+1E964 */
  0xFBC3,   /* U+1E965 */
  0xFBC3,   /* U+1E966 */
  0xFBC3,   /* U+1E967 */
  0xFBC3,   /* U+1E968 */
  0xFBC3,   /* U+1E969 */
  0xFBC3,   /* U+1E96A */
  0xFBC3,   /* U+1E96B */
  0xFBC3,   /* U+1E96C */
  0xFBC3,   /* U+1E96D */
  0xFBC3,   /* U+1E96E */
  0xFBC3,   /* U+1E96F */
  0xFBC3,   /* U+1E970 */
  0xFBC3,   /* U+1E971 */
  0xFBC3,   /* U+1E972 */
  0xFBC3,   /* U+1E973 */
  0xFBC3,   /* U+1E974 */
  0xFBC3,   /* U+1E975 */
  0xFBC3,   /* U+1E976 */
  0xFBC3,   /* U+1E977 */
  0xFBC3,   /* U+1E978 */
  0xFBC3,   /* U+1E979 */
  0xFBC3,   /* U+1E97A */
  0xFBC3,   /* U+1E97B */
  0xFBC3,   /* U+1E97C */
  0xFBC3,   /* U+1E97D */
  0xFBC3,   /* U+1E97E */
  0xFBC3,   /* U+1E97F */
  0xFBC3,   /* U+1E980 */
  0xFBC3,   /* U+1E981 */
  0xFBC3,   /* U+1E982 */
  0xFBC3,   /* U+1E983 */
  0xFBC3,   /* U+1E984 */
  0xFBC3,   /* U+1E985 */
  0xFBC3,   /* U+1E986 */
  0xFBC3,   /* U+1E987 */
  0xFBC3,   /* U+1E988 */
  0xFBC3,   /* U+1E989 */
  0xFBC3,   /* U+1E98A */
  0xFBC3,   /* U+1E98B */
  0xFBC3,   /* U+1E98C */
  0xFBC3,   /* U+1E98D */
  0xFBC3,   /* U+1E98E */
  0xFBC3,   /* U+1E98F */
  0xFBC3,   /* U+1E990 */
  0xFBC3,   /* U+1E991 */
  0xFBC3,   /* U+1E992 */
  0xFBC3,   /* U+1E993 */
  0xFBC3,   /* U+1E994 */
  0xFBC3,   /* U+1E995 */
  0xFBC3,   /* U+1E996 */
  0xFBC3,   /* U+1E997 */
  0xFBC3,   /* U+1E998 */
  0xFBC3,   /* U+1E999 */
  0xFBC3,   /* U+1E99A */
  0xFBC3,   /* U+1E99B */
  0xFBC3,   /* U+1E99C */
  0xFBC3,   /* U+1E99D */
  0xFBC3,   /* U+1E99E */
  0xFBC3,   /* U+1E99F */
  0xFBC3,   /* U+1E9A0 */
  0xFBC3,   /* U+1E9A1 */
  0xFBC3,   /* U+1E9A2 */
  0xFBC3,   /* U+1E9A3 */
  0xFBC3,   /* U+1E9A4 */
  0xFBC3,   /* U+1E9A5 */
  0xFBC3,   /* U+1E9A6 */
  0xFBC3,   /* U+1E9A7 */
  0xFBC3,   /* U+1E9A8 */
  0xFBC3,   /* U+1E9A9 */
  0xFBC3,   /* U+1E9AA */
  0xFBC3,   /* U+1E9AB */
  0xFBC3,   /* U+1E9AC */
  0xFBC3,   /* U+1E9AD */
  0xFBC3,   /* U+1E9AE */
  0xFBC3,   /* U+1E9AF */
  0xFBC3,   /* U+1E9B0 */
  0xFBC3,   /* U+1E9B1 */
  0xFBC3,   /* U+1E9B2 */
  0xFBC3,   /* U+1E9B3 */
  0xFBC3,   /* U+1E9B4 */
  0xFBC3,   /* U+1E9B5 */
  0xFBC3,   /* U+1E9B6 */
  0xFBC3,   /* U+1E9B7 */
  0xFBC3,   /* U+1E9B8 */
  0xFBC3,   /* U+1E9B9 */
  0xFBC3,   /* U+1E9BA */
  0xFBC3,   /* U+1E9BB */
  0xFBC3,   /* U+1E9BC */
  0xFBC3,   /* U+1E9BD */
  0xFBC3,   /* U+1E9BE */
  0xFBC3,   /* U+1E9BF */
  0xFBC3,   /* U+1E9C0 */
  0xFBC3,   /* U+1E9C1 */
  0xFBC3,   /* U+1E9C2 */
  0xFBC3,   /* U+1E9C3 */
  0xFBC3,   /* U+1E9C4 */
  0xFBC3,   /* U+1E9C5 */
  0xFBC3,   /* U+1E9C6 */
  0xFBC3,   /* U+1E9C7 */
  0xFBC3,   /* U+1E9C8 */
  0xFBC3,   /* U+1E9C9 */
  0xFBC3,   /* U+1E9CA */
  0xFBC3,   /* U+1E9CB */
  0xFBC3,   /* U+1E9CC */
  0xFBC3,   /* U+1E9CD */
  0xFBC3,   /* U+1E9CE */
  0xFBC3,   /* U+1E9CF */
  0xFBC3,   /* U+1E9D0 */
  0xFBC3,   /* U+1E9D1 */
  0xFBC3,   /* U+1E9D2 */
  0xFBC3,   /* U+1E9D3 */
  0xFBC3,   /* U+1E9D4 */
  0xFBC3,   /* U+1E9D5 */
  0xFBC3,   /* U+1E9D6 */
  0xFBC3,   /* U+1E9D7 */
  0xFBC3,   /* U+1E9D8 */
  0xFBC3,   /* U+1E9D9 */
  0xFBC3,   /* U+1E9DA */
  0xFBC3,   /* U+1E9DB */
  0xFBC3,   /* U+1E9DC */
  0xFBC3,   /* U+1E9DD */
  0xFBC3,   /* U+1E9DE */
  0xFBC3,   /* U+1E9DF */
  0xFBC3,   /* U+1E9E0 */
  0xFBC3,   /* U+1E9E1 */
  0xFBC3,   /* U+1E9E2 */
  0xFBC3,   /* U+1E9E3 */
  0xFBC3,   /* U+1E9E4 */
  0xFBC3,   /* U+1E9E5 */
  0xFBC3,   /* U+1E9E6 */
  0xFBC3,   /* U+1E9E7 */
  0xFBC3,   /* U+1E9E8 */
  0xFBC3,   /* U+1E9E9 */
  0xFBC3,   /* U+1E9EA */
  0xFBC3,   /* U+1E9EB */
  0xFBC3,   /* U+1E9EC */
  0xFBC3,   /* U+1E9ED */
  0xFBC3,   /* U+1E9EE */
  0xFBC3,   /* U+1E9EF */
  0xFBC3,   /* U+1E9F0 */
  0xFBC3,   /* U+1E9F1 */
  0xFBC3,   /* U+1E9F2 */
  0xFBC3,   /* U+1E9F3 */
  0xFBC3,   /* U+1E9F4 */
  0xFBC3,   /* U+1E9F5 */
  0xFBC3,   /* U+1E9F6 */
  0xFBC3,   /* U+1E9F7 */
  0xFBC3,   /* U+1E9F8 */
  0xFBC3,   /* U+1E9F9 */
  0xFBC3,   /* U+1E9FA */
  0xFBC3,   /* U+1E9FB */
  0xFBC3,   /* U+1E9FC */
  0xFBC3,   /* U+1E9FD */
  0xFBC3,   /* U+1E9FE */
  0xFBC3,   /* U+1E9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1E900 */
  0x0020,   /* U+1E901 */
  0x0020,   /* U+1E902 */
  0x0020,   /* U+1E903 */
  0x0020,   /* U+1E904 */
  0x0020,   /* U+1E905 */
  0x0020,   /* U+1E906 */
  0x0020,   /* U+1E907 */
  0x0020,   /* U+1E908 */
  0x0020,   /* U+1E909 */
  0x0020,   /* U+1E90A */
  0x0020,   /* U+1E90B */
  0x0020,   /* U+1E90C */
  0x0020,   /* U+1E90D */
  0x0020,   /* U+1E90E */
  0x0020,   /* U+1E90F */
  0x0020,   /* U+1E910 */
  0x0020,   /* U+1E911 */
  0x0020,   /* U+1E912 */
  0x0020,   /* U+1E913 */
  0x0020,   /* U+1E914 */
  0x0020,   /* U+1E915 */
  0x0020,   /* U+1E916 */
  0x0020,   /* U+1E917 */
  0x0020,   /* U+1E918 */
  0x0020,   /* U+1E919 */
  0x0020,   /* U+1E91A */
  0x0020,   /* U+1E91B */
  0x0020,   /* U+1E91C */
  0x0020,   /* U+1E91D */
  0x0020,   /* U+1E91E */
  0x0020,   /* U+1E91F */
  0x0020,   /* U+1E920 */
  0x0020,   /* U+1E921 */
  0x0020,   /* U+1E922 */
  0x0020,   /* U+1E923 */
  0x0020,   /* U+1E924 */
  0x0020,   /* U+1E925 */
  0x0020,   /* U+1E926 */
  0x0020,   /* U+1E927 */
  0x0020,   /* U+1E928 */
  0x0020,   /* U+1E929 */
  0x0020,   /* U+1E92A */
  0x0020,   /* U+1E92B */
  0x0020,   /* U+1E92C */
  0x0020,   /* U+1E92D */
  0x0020,   /* U+1E92E */
  0x0020,   /* U+1E92F */
  0x0020,   /* U+1E930 */
  0x0020,   /* U+1E931 */
  0x0020,   /* U+1E932 */
  0x0020,   /* U+1E933 */
  0x0020,   /* U+1E934 */
  0x0020,   /* U+1E935 */
  0x0020,   /* U+1E936 */
  0x0020,   /* U+1E937 */
  0x0020,   /* U+1E938 */
  0x0020,   /* U+1E939 */
  0x0020,   /* U+1E93A */
  0x0020,   /* U+1E93B */
  0x0020,   /* U+1E93C */
  0x0020,   /* U+1E93D */
  0x0020,   /* U+1E93E */
  0x0020,   /* U+1E93F */
  0x0020,   /* U+1E940 */
  0x0020,   /* U+1E941 */
  0x0020,   /* U+1E942 */
  0x0020,   /* U+1E943 */
  0x00BD,   /* U+1E944 */
  0x00BD,   /* U+1E945 */
  0x00BD,   /* U+1E946 */
  0x00BF,   /* U+1E947 */
  0x00C0,   /* U+1E948 */
  0x00C1,   /* U+1E949 */
  0x00BE,   /* U+1E94A */
  0x0020,   /* U+1E94B */
  0x0020,   /* U+1E94C */
  0x0020,   /* U+1E94D */
  0x0020,   /* U+1E94E */
  0x0020,   /* U+1E94F */
  0x0020,   /* U+1E950 */
  0x0020,   /* U+1E951 */
  0x0020,   /* U+1E952 */
  0x0020,   /* U+1E953 */
  0x0020,   /* U+1E954 */
  0x0020,   /* U+1E955 */
  0x0020,   /* U+1E956 */
  0x0020,   /* U+1E957 */
  0x0020,   /* U+1E958 */
  0x0020,   /* U+1E959 */
  0x0020,   /* U+1E95A */
  0x0020,   /* U+1E95B */
  0x0020,   /* U+1E95C */
  0x0020,   /* U+1E95D */
  0x0020,   /* U+1E95E */
  0x0020,   /* U+1E95F */
  0x0020,   /* U+1E960 */
  0x0020,   /* U+1E961 */
  0x0020,   /* U+1E962 */
  0x0020,   /* U+1E963 */
  0x0020,   /* U+1E964 */
  0x0020,   /* U+1E965 */
  0x0020,   /* U+1E966 */
  0x0020,   /* U+1E967 */
  0x0020,   /* U+1E968 */
  0x0020,   /* U+1E969 */
  0x0020,   /* U+1E96A */
  0x0020,   /* U+1E96B */
  0x0020,   /* U+1E96C */
  0x0020,   /* U+1E96D */
  0x0020,   /* U+1E96E */
  0x0020,   /* U+1E96F */
  0x0020,   /* U+1E970 */
  0x0020,   /* U+1E971 */
  0x0020,   /* U+1E972 */
  0x0020,   /* U+1E973 */
  0x0020,   /* U+1E974 */
  0x0020,   /* U+1E975 */
  0x0020,   /* U+1E976 */
  0x0020,   /* U+1E977 */
  0x0020,   /* U+1E978 */
  0x0020,   /* U+1E979 */
  0x0020,   /* U+1E97A */
  0x0020,   /* U+1E97B */
  0x0020,   /* U+1E97C */
  0x0020,   /* U+1E97D */
  0x0020,   /* U+1E97E */
  0x0020,   /* U+1E97F */
  0x0020,   /* U+1E980 */
  0x0020,   /* U+1E981 */
  0x0020,   /* U+1E982 */
  0x0020,   /* U+1E983 */
  0x0020,   /* U+1E984 */
  0x0020,   /* U+1E985 */
  0x0020,   /* U+1E986 */
  0x0020,   /* U+1E987 */
  0x0020,   /* U+1E988 */
  0x0020,   /* U+1E989 */
  0x0020,   /* U+1E98A */
  0x0020,   /* U+1E98B */
  0x0020,   /* U+1E98C */
  0x0020,   /* U+1E98D */
  0x0020,   /* U+1E98E */
  0x0020,   /* U+1E98F */
  0x0020,   /* U+1E990 */
  0x0020,   /* U+1E991 */
  0x0020,   /* U+1E992 */
  0x0020,   /* U+1E993 */
  0x0020,   /* U+1E994 */
  0x0020,   /* U+1E995 */
  0x0020,   /* U+1E996 */
  0x0020,   /* U+1E997 */
  0x0020,   /* U+1E998 */
  0x0020,   /* U+1E999 */
  0x0020,   /* U+1E99A */
  0x0020,   /* U+1E99B */
  0x0020,   /* U+1E99C */
  0x0020,   /* U+1E99D */
  0x0020,   /* U+1E99E */
  0x0020,   /* U+1E99F */
  0x0020,   /* U+1E9A0 */
  0x0020,   /* U+1E9A1 */
  0x0020,   /* U+1E9A2 */
  0x0020,   /* U+1E9A3 */
  0x0020,   /* U+1E9A4 */
  0x0020,   /* U+1E9A5 */
  0x0020,   /* U+1E9A6 */
  0x0020,   /* U+1E9A7 */
  0x0020,   /* U+1E9A8 */
  0x0020,   /* U+1E9A9 */
  0x0020,   /* U+1E9AA */
  0x0020,   /* U+1E9AB */
  0x0020,   /* U+1E9AC */
  0x0020,   /* U+1E9AD */
  0x0020,   /* U+1E9AE */
  0x0020,   /* U+1E9AF */
  0x0020,   /* U+1E9B0 */
  0x0020,   /* U+1E9B1 */
  0x0020,   /* U+1E9B2 */
  0x0020,   /* U+1E9B3 */
  0x0020,   /* U+1E9B4 */
  0x0020,   /* U+1E9B5 */
  0x0020,   /* U+1E9B6 */
  0x0020,   /* U+1E9B7 */
  0x0020,   /* U+1E9B8 */
  0x0020,   /* U+1E9B9 */
  0x0020,   /* U+1E9BA */
  0x0020,   /* U+1E9BB */
  0x0020,   /* U+1E9BC */
  0x0020,   /* U+1E9BD */
  0x0020,   /* U+1E9BE */
  0x0020,   /* U+1E9BF */
  0x0020,   /* U+1E9C0 */
  0x0020,   /* U+1E9C1 */
  0x0020,   /* U+1E9C2 */
  0x0020,   /* U+1E9C3 */
  0x0020,   /* U+1E9C4 */
  0x0020,   /* U+1E9C5 */
  0x0020,   /* U+1E9C6 */
  0x0020,   /* U+1E9C7 */
  0x0020,   /* U+1E9C8 */
  0x0020,   /* U+1E9C9 */
  0x0020,   /* U+1E9CA */
  0x0020,   /* U+1E9CB */
  0x0020,   /* U+1E9CC */
  0x0020,   /* U+1E9CD */
  0x0020,   /* U+1E9CE */
  0x0020,   /* U+1E9CF */
  0x0020,   /* U+1E9D0 */
  0x0020,   /* U+1E9D1 */
  0x0020,   /* U+1E9D2 */
  0x0020,   /* U+1E9D3 */
  0x0020,   /* U+1E9D4 */
  0x0020,   /* U+1E9D5 */
  0x0020,   /* U+1E9D6 */
  0x0020,   /* U+1E9D7 */
  0x0020,   /* U+1E9D8 */
  0x0020,   /* U+1E9D9 */
  0x0020,   /* U+1E9DA */
  0x0020,   /* U+1E9DB */
  0x0020,   /* U+1E9DC */
  0x0020,   /* U+1E9DD */
  0x0020,   /* U+1E9DE */
  0x0020,   /* U+1E9DF */
  0x0020,   /* U+1E9E0 */
  0x0020,   /* U+1E9E1 */
  0x0020,   /* U+1E9E2 */
  0x0020,   /* U+1E9E3 */
  0x0020,   /* U+1E9E4 */
  0x0020,   /* U+1E9E5 */
  0x0020,   /* U+1E9E6 */
  0x0020,   /* U+1E9E7 */
  0x0020,   /* U+1E9E8 */
  0x0020,   /* U+1E9E9 */
  0x0020,   /* U+1E9EA */
  0x0020,   /* U+1E9EB */
  0x0020,   /* U+1E9EC */
  0x0020,   /* U+1E9ED */
  0x0020,   /* U+1E9EE */
  0x0020,   /* U+1E9EF */
  0x0020,   /* U+1E9F0 */
  0x0020,   /* U+1E9F1 */
  0x0020,   /* U+1E9F2 */
  0x0020,   /* U+1E9F3 */
  0x0020,   /* U+1E9F4 */
  0x0020,   /* U+1E9F5 */
  0x0020,   /* U+1E9F6 */
  0x0020,   /* U+1E9F7 */
  0x0020,   /* U+1E9F8 */
  0x0020,   /* U+1E9F9 */
  0x0020,   /* U+1E9FA */
  0x0020,   /* U+1E9FB */
  0x0020,   /* U+1E9FC */
  0x0020,   /* U+1E9FD */
  0x0020,   /* U+1E9FE */
  0x0020,   /* U+1E9FF */

  /* Tertiary weight 1 for each character. */
  0x0008,   /* U+1E900 */
  0x0008,   /* U+1E901 */
  0x0008,   /* U+1E902 */
  0x0008,   /* U+1E903 */
  0x0008,   /* U+1E904 */
  0x0008,   /* U+1E905 */
  0x0008,   /* U+1E906 */
  0x0008,   /* U+1E907 */
  0x0008,   /* U+1E908 */
  0x0008,   /* U+1E909 */
  0x0008,   /* U+1E90A */
  0x0008,   /* U+1E90B */
  0x0008,   /* U+1E90C */
  0x0008,   /* U+1E90D */
  0x0008,   /* U+1E90E */
  0x0008,   /* U+1E90F */
  0x0008,   /* U+1E910 */
  0x0008,   /* U+1E911 */
  0x0008,   /* U+1E912 */
  0x0008,   /* U+1E913 */
  0x0008,   /* U+1E914 */
  0x0008,   /* U+1E915 */
  0x0008,   /* U+1E916 */
  0x0008,   /* U+1E917 */
  0x0008,   /* U+1E918 */
  0x0008,   /* U+1E919 */
  0x0008,   /* U+1E91A */
  0x0008,   /* U+1E91B */
  0x0008,   /* U+1E91C */
  0x0008,   /* U+1E91D */
  0x0008,   /* U+1E91E */
  0x0008,   /* U+1E91F */
  0x0008,   /* U+1E920 */
  0x0008,   /* U+1E921 */
  0x0002,   /* U+1E922 */
  0x0002,   /* U+1E923 */
  0x0002,   /* U+1E924 */
  0x0002,   /* U+1E925 */
  0x0002,   /* U+1E926 */
  0x0002,   /* U+1E927 */
  0x0002,   /* U+1E928 */
  0x0002,   /* U+1E929 */
  0x0002,   /* U+1E92A */
  0x0002,   /* U+1E92B */
  0x0002,   /* U+1E92C */
  0x0002,   /* U+1E92D */
  0x0002,   /* U+1E92E */
  0x0002,   /* U+1E92F */
  0x0002,   /* U+1E930 */
  0x0002,   /* U+1E931 */
  0x0002,   /* U+1E932 */
  0x0002,   /* U+1E933 */
  0x0002,   /* U+1E934 */
  0x0002,   /* U+1E935 */
  0x0002,   /* U+1E936 */
  0x0002,   /* U+1E937 */
  0x0002,   /* U+1E938 */
  0x0002,   /* U+1E939 */
  0x0002,   /* U+1E93A */
  0x0002,   /* U+1E93B */
  0x0002,   /* U+1E93C */
  0x0002,   /* U+1E93D */
  0x0002,   /* U+1E93E */
  0x0002,   /* U+1E93F */
  0x0002,   /* U+1E940 */
  0x0002,   /* U+1E941 */
  0x0002,   /* U+1E942 */
  0x0002,   /* U+1E943 */
  0x0002,   /* U+1E944 */
  0x0002,   /* U+1E945 */
  0x0002,   /* U+1E946 */
  0x0002,   /* U+1E947 */
  0x0002,   /* U+1E948 */
  0x0002,   /* U+1E949 */
  0x0002,   /* U+1E94A */
  0x0002,   /* U+1E94B */
  0x0002,   /* U+1E94C */
  0x0002,   /* U+1E94D */
  0x0002,   /* U+1E94E */
  0x0002,   /* U+1E94F */
  0x0002,   /* U+1E950 */
  0x0002,   /* U+1E951 */
  0x0002,   /* U+1E952 */
  0x0002,   /* U+1E953 */
  0x0002,   /* U+1E954 */
  0x0002,   /* U+1E955 */
  0x0002,   /* U+1E956 */
  0x0002,   /* U+1E957 */
  0x0002,   /* U+1E958 */
  0x0002,   /* U+1E959 */
  0x0002,   /* U+1E95A */
  0x0002,   /* U+1E95B */
  0x0002,   /* U+1E95C */
  0x0002,   /* U+1E95D */
  0x0002,   /* U+1E95E */
  0x0002,   /* U+1E95F */
  0x0002,   /* U+1E960 */
  0x0002,   /* U+1E961 */
  0x0002,   /* U+1E962 */
  0x0002,   /* U+1E963 */
  0x0002,   /* U+1E964 */
  0x0002,   /* U+1E965 */
  0x0002,   /* U+1E966 */
  0x0002,   /* U+1E967 */
  0x0002,   /* U+1E968 */
  0x0002,   /* U+1E969 */
  0x0002,   /* U+1E96A */
  0x0002,   /* U+1E96B */
  0x0002,   /* U+1E96C */
  0x0002,   /* U+1E96D */
  0x0002,   /* U+1E96E */
  0x0002,   /* U+1E96F */
  0x0002,   /* U+1E970 */
  0x0002,   /* U+1E971 */
  0x0002,   /* U+1E972 */
  0x0002,   /* U+1E973 */
  0x0002,   /* U+1E974 */
  0x0002,   /* U+1E975 */
  0x0002,   /* U+1E976 */
  0x0002,   /* U+1E977 */
  0x0002,   /* U+1E978 */
  0x0002,   /* U+1E979 */
  0x0002,   /* U+1E97A */
  0x0002,   /* U+1E97B */
  0x0002,   /* U+1E97C */
  0x0002,   /* U+1E97D */
  0x0002,   /* U+1E97E */
  0x0002,   /* U+1E97F */
  0x0002,   /* U+1E980 */
  0x0002,   /* U+1E981 */
  0x0002,   /* U+1E982 */
  0x0002,   /* U+1E983 */
  0x0002,   /* U+1E984 */
  0x0002,   /* U+1E985 */
  0x0002,   /* U+1E986 */
  0x0002,   /* U+1E987 */
  0x0002,   /* U+1E988 */
  0x0002,   /* U+1E989 */
  0x0002,   /* U+1E98A */
  0x0002,   /* U+1E98B */
  0x0002,   /* U+1E98C */
  0x0002,   /* U+1E98D */
  0x0002,   /* U+1E98E */
  0x0002,   /* U+1E98F */
  0x0002,   /* U+1E990 */
  0x0002,   /* U+1E991 */
  0x0002,   /* U+1E992 */
  0x0002,   /* U+1E993 */
  0x0002,   /* U+1E994 */
  0x0002,   /* U+1E995 */
  0x0002,   /* U+1E996 */
  0x0002,   /* U+1E997 */
  0x0002,   /* U+1E998 */
  0x0002,   /* U+1E999 */
  0x0002,   /* U+1E99A */
  0x0002,   /* U+1E99B */
  0x0002,   /* U+1E99C */
  0x0002,   /* U+1E99D */
  0x0002,   /* U+1E99E */
  0x0002,   /* U+1E99F */
  0x0002,   /* U+1E9A0 */
  0x0002,   /* U+1E9A1 */
  0x0002,   /* U+1E9A2 */
  0x0002,   /* U+1E9A3 */
  0x0002,   /* U+1E9A4 */
  0x0002,   /* U+1E9A5 */
  0x0002,   /* U+1E9A6 */
  0x0002,   /* U+1E9A7 */
  0x0002,   /* U+1E9A8 */
  0x0002,   /* U+1E9A9 */
  0x0002,   /* U+1E9AA */
  0x0002,   /* U+1E9AB */
  0x0002,   /* U+1E9AC */
  0x0002,   /* U+1E9AD */
  0x0002,   /* U+1E9AE */
  0x0002,   /* U+1E9AF */
  0x0002,   /* U+1E9B0 */
  0x0002,   /* U+1E9B1 */
  0x0002,   /* U+1E9B2 */
  0x0002,   /* U+1E9B3 */
  0x0002,   /* U+1E9B4 */
  0x0002,   /* U+1E9B5 */
  0x0002,   /* U+1E9B6 */
  0x0002,   /* U+1E9B7 */
  0x0002,   /* U+1E9B8 */
  0x0002,   /* U+1E9B9 */
  0x0002,   /* U+1E9BA */
  0x0002,   /* U+1E9BB */
  0x0002,   /* U+1E9BC */
  0x0002,   /* U+1E9BD */
  0x0002,   /* U+1E9BE */
  0x0002,   /* U+1E9BF */
  0x0002,   /* U+1E9C0 */
  0x0002,   /* U+1E9C1 */
  0x0002,   /* U+1E9C2 */
  0x0002,   /* U+1E9C3 */
  0x0002,   /* U+1E9C4 */
  0x0002,   /* U+1E9C5 */
  0x0002,   /* U+1E9C6 */
  0x0002,   /* U+1E9C7 */
  0x0002,   /* U+1E9C8 */
  0x0002,   /* U+1E9C9 */
  0x0002,   /* U+1E9CA */
  0x0002,   /* U+1E9CB */
  0x0002,   /* U+1E9CC */
  0x0002,   /* U+1E9CD */
  0x0002,   /* U+1E9CE */
  0x0002,   /* U+1E9CF */
  0x0002,   /* U+1E9D0 */
  0x0002,   /* U+1E9D1 */
  0x0002,   /* U+1E9D2 */
  0x0002,   /* U+1E9D3 */
  0x0002,   /* U+1E9D4 */
  0x0002,   /* U+1E9D5 */
  0x0002,   /* U+1E9D6 */
  0x0002,   /* U+1E9D7 */
  0x0002,   /* U+1E9D8 */
  0x0002,   /* U+1E9D9 */
  0x0002,   /* U+1E9DA */
  0x0002,   /* U+1E9DB */
  0x0002,   /* U+1E9DC */
  0x0002,   /* U+1E9DD */
  0x0002,   /* U+1E9DE */
  0x0002,   /* U+1E9DF */
  0x0002,   /* U+1E9E0 */
  0x0002,   /* U+1E9E1 */
  0x0002,   /* U+1E9E2 */
  0x0002,   /* U+1E9E3 */
  0x0002,   /* U+1E9E4 */
  0x0002,   /* U+1E9E5 */
  0x0002,   /* U+1E9E6 */
  0x0002,   /* U+1E9E7 */
  0x0002,   /* U+1E9E8 */
  0x0002,   /* U+1E9E9 */
  0x0002,   /* U+1E9EA */
  0x0002,   /* U+1E9EB */
  0x0002,   /* U+1E9EC */
  0x0002,   /* U+1E9ED */
  0x0002,   /* U+1E9EE */
  0x0002,   /* U+1E9EF */
  0x0002,   /* U+1E9F0 */
  0x0002,   /* U+1E9F1 */
  0x0002,   /* U+1E9F2 */
  0x0002,   /* U+1E9F3 */
  0x0002,   /* U+1E9F4 */
  0x0002,   /* U+1E9F5 */
  0x0002,   /* U+1E9F6 */
  0x0002,   /* U+1E9F7 */
  0x0002,   /* U+1E9F8 */
  0x0002,   /* U+1E9F9 */
  0x0002,   /* U+1E9FA */
  0x0002,   /* U+1E9FB */
  0x0002,   /* U+1E9FC */
  0x0002,   /* U+1E9FD */
  0x0002,   /* U+1E9FE */
  0x0002,   /* U+1E9FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1E900 */
  0x0000,   /* U+1E901 */
  0x0000,   /* U+1E902 */
  0x0000,   /* U+1E903 */
  0x0000,   /* U+1E904 */
  0x0000,   /* U+1E905 */
  0x0000,   /* U+1E906 */
  0x0000,   /* U+1E907 */
  0x0000,   /* U+1E908 */
  0x0000,   /* U+1E909 */
  0x0000,   /* U+1E90A */
  0x0000,   /* U+1E90B */
  0x0000,   /* U+1E90C */
  0x0000,   /* U+1E90D */
  0x0000,   /* U+1E90E */
  0x0000,   /* U+1E90F */
  0x0000,   /* U+1E910 */
  0x0000,   /* U+1E911 */
  0x0000,   /* U+1E912 */
  0x0000,   /* U+1E913 */
  0x0000,   /* U+1E914 */
  0x0000,   /* U+1E915 */
  0x0000,   /* U+1E916 */
  0x0000,   /* U+1E917 */
  0x0000,   /* U+1E918 */
  0x0000,   /* U+1E919 */
  0x0000,   /* U+1E91A */
  0x0000,   /* U+1E91B */
  0x0000,   /* U+1E91C */
  0x0000,   /* U+1E91D */
  0x0000,   /* U+1E91E */
  0x0000,   /* U+1E91F */
  0x0000,   /* U+1E920 */
  0x0000,   /* U+1E921 */
  0x0000,   /* U+1E922 */
  0x0000,   /* U+1E923 */
  0x0000,   /* U+1E924 */
  0x0000,   /* U+1E925 */
  0x0000,   /* U+1E926 */
  0x0000,   /* U+1E927 */
  0x0000,   /* U+1E928 */
  0x0000,   /* U+1E929 */
  0x0000,   /* U+1E92A */
  0x0000,   /* U+1E92B */
  0x0000,   /* U+1E92C */
  0x0000,   /* U+1E92D */
  0x0000,   /* U+1E92E */
  0x0000,   /* U+1E92F */
  0x0000,   /* U+1E930 */
  0x0000,   /* U+1E931 */
  0x0000,   /* U+1E932 */
  0x0000,   /* U+1E933 */
  0x0000,   /* U+1E934 */
  0x0000,   /* U+1E935 */
  0x0000,   /* U+1E936 */
  0x0000,   /* U+1E937 */
  0x0000,   /* U+1E938 */
  0x0000,   /* U+1E939 */
  0x0000,   /* U+1E93A */
  0x0000,   /* U+1E93B */
  0x0000,   /* U+1E93C */
  0x0000,   /* U+1E93D */
  0x0000,   /* U+1E93E */
  0x0000,   /* U+1E93F */
  0x0000,   /* U+1E940 */
  0x0000,   /* U+1E941 */
  0x0000,   /* U+1E942 */
  0x0000,   /* U+1E943 */
  0x0000,   /* U+1E944 */
  0x0000,   /* U+1E945 */
  0x0000,   /* U+1E946 */
  0x0000,   /* U+1E947 */
  0x0000,   /* U+1E948 */
  0x0000,   /* U+1E949 */
  0x0000,   /* U+1E94A */
  0xE94B,   /* U+1E94B */
  0xE94C,   /* U+1E94C */
  0xE94D,   /* U+1E94D */
  0xE94E,   /* U+1E94E */
  0xE94F,   /* U+1E94F */
  0x0000,   /* U+1E950 */
  0x0000,   /* U+1E951 */
  0x0000,   /* U+1E952 */
  0x0000,   /* U+1E953 */
  0x0000,   /* U+1E954 */
  0x0000,   /* U+1E955 */
  0x0000,   /* U+1E956 */
  0x0000,   /* U+1E957 */
  0x0000,   /* U+1E958 */
  0x0000,   /* U+1E959 */
  0xE95A,   /* U+1E95A */
  0xE95B,   /* U+1E95B */
  0xE95C,   /* U+1E95C */
  0xE95D,   /* U+1E95D */
  0x0000,   /* U+1E95E */
  0x0000,   /* U+1E95F */
  0xE960,   /* U+1E960 */
  0xE961,   /* U+1E961 */
  0xE962,   /* U+1E962 */
  0xE963,   /* U+1E963 */
  0xE964,   /* U+1E964 */
  0xE965,   /* U+1E965 */
  0xE966,   /* U+1E966 */
  0xE967,   /* U+1E967 */
  0xE968,   /* U+1E968 */
  0xE969,   /* U+1E969 */
  0xE96A,   /* U+1E96A */
  0xE96B,   /* U+1E96B */
  0xE96C,   /* U+1E96C */
  0xE96D,   /* U+1E96D */
  0xE96E,   /* U+1E96E */
  0xE96F,   /* U+1E96F */
  0xE970,   /* U+1E970 */
  0xE971,   /* U+1E971 */
  0xE972,   /* U+1E972 */
  0xE973,   /* U+1E973 */
  0xE974,   /* U+1E974 */
  0xE975,   /* U+1E975 */
  0xE976,   /* U+1E976 */
  0xE977,   /* U+1E977 */
  0xE978,   /* U+1E978 */
  0xE979,   /* U+1E979 */
  0xE97A,   /* U+1E97A */
  0xE97B,   /* U+1E97B */
  0xE97C,   /* U+1E97C */
  0xE97D,   /* U+1E97D */
  0xE97E,   /* U+1E97E */
  0xE97F,   /* U+1E97F */
  0xE980,   /* U+1E980 */
  0xE981,   /* U+1E981 */
  0xE982,   /* U+1E982 */
  0xE983,   /* U+1E983 */
  0xE984,   /* U+1E984 */
  0xE985,   /* U+1E985 */
  0xE986,   /* U+1E986 */
  0xE987,   /* U+1E987 */
  0xE988,   /* U+1E988 */
  0xE989,   /* U+1E989 */
  0xE98A,   /* U+1E98A */
  0xE98B,   /* U+1E98B */
  0xE98C,   /* U+1E98C */
  0xE98D,   /* U+1E98D */
  0xE98E,   /* U+1E98E */
  0xE98F,   /* U+1E98F */
  0xE990,   /* U+1E990 */
  0xE991,   /* U+1E991 */
  0xE992,   /* U+1E992 */
  0xE993,   /* U+1E993 */
  0xE994,   /* U+1E994 */
  0xE995,   /* U+1E995 */
  0xE996,   /* U+1E996 */
  0xE997,   /* U+1E997 */
  0xE998,   /* U+1E998 */
  0xE999,   /* U+1E999 */
  0xE99A,   /* U+1E99A */
  0xE99B,   /* U+1E99B */
  0xE99C,   /* U+1E99C */
  0xE99D,   /* U+1E99D */
  0xE99E,   /* U+1E99E */
  0xE99F,   /* U+1E99F */
  0xE9A0,   /* U+1E9A0 */
  0xE9A1,   /* U+1E9A1 */
  0xE9A2,   /* U+1E9A2 */
  0xE9A3,   /* U+1E9A3 */
  0xE9A4,   /* U+1E9A4 */
  0xE9A5,   /* U+1E9A5 */
  0xE9A6,   /* U+1E9A6 */
  0xE9A7,   /* U+1E9A7 */
  0xE9A8,   /* U+1E9A8 */
  0xE9A9,   /* U+1E9A9 */
  0xE9AA,   /* U+1E9AA */
  0xE9AB,   /* U+1E9AB */
  0xE9AC,   /* U+1E9AC */
  0xE9AD,   /* U+1E9AD */
  0xE9AE,   /* U+1E9AE */
  0xE9AF,   /* U+1E9AF */
  0xE9B0,   /* U+1E9B0 */
  0xE9B1,   /* U+1E9B1 */
  0xE9B2,   /* U+1E9B2 */
  0xE9B3,   /* U+1E9B3 */
  0xE9B4,   /* U+1E9B4 */
  0xE9B5,   /* U+1E9B5 */
  0xE9B6,   /* U+1E9B6 */
  0xE9B7,   /* U+1E9B7 */
  0xE9B8,   /* U+1E9B8 */
  0xE9B9,   /* U+1E9B9 */
  0xE9BA,   /* U+1E9BA */
  0xE9BB,   /* U+1E9BB */
  0xE9BC,   /* U+1E9BC */
  0xE9BD,   /* U+1E9BD */
  0xE9BE,   /* U+1E9BE */
  0xE9BF,   /* U+1E9BF */
  0xE9C0,   /* U+1E9C0 */
  0xE9C1,   /* U+1E9C1 */
  0xE9C2,   /* U+1E9C2 */
  0xE9C3,   /* U+1E9C3 */
  0xE9C4,   /* U+1E9C4 */
  0xE9C5,   /* U+1E9C5 */
  0xE9C6,   /* U+1E9C6 */
  0xE9C7,   /* U+1E9C7 */
  0xE9C8,   /* U+1E9C8 */
  0xE9C9,   /* U+1E9C9 */
  0xE9CA,   /* U+1E9CA */
  0xE9CB,   /* U+1E9CB */
  0xE9CC,   /* U+1E9CC */
  0xE9CD,   /* U+1E9CD */
  0xE9CE,   /* U+1E9CE */
  0xE9CF,   /* U+1E9CF */
  0xE9D0,   /* U+1E9D0 */
  0xE9D1,   /* U+1E9D1 */
  0xE9D2,   /* U+1E9D2 */
  0xE9D3,   /* U+1E9D3 */
  0xE9D4,   /* U+1E9D4 */
  0xE9D5,   /* U+1E9D5 */
  0xE9D6,   /* U+1E9D6 */
  0xE9D7,   /* U+1E9D7 */
  0xE9D8,   /* U+1E9D8 */
  0xE9D9,   /* U+1E9D9 */
  0xE9DA,   /* U+1E9DA */
  0xE9DB,   /* U+1E9DB */
  0xE9DC,   /* U+1E9DC */
  0xE9DD,   /* U+1E9DD */
  0xE9DE,   /* U+1E9DE */
  0xE9DF,   /* U+1E9DF */
  0xE9E0,   /* U+1E9E0 */
  0xE9E1,   /* U+1E9E1 */
  0xE9E2,   /* U+1E9E2 */
  0xE9E3,   /* U+1E9E3 */
  0xE9E4,   /* U+1E9E4 */
  0xE9E5,   /* U+1E9E5 */
  0xE9E6,   /* U+1E9E6 */
  0xE9E7,   /* U+1E9E7 */
  0xE9E8,   /* U+1E9E8 */
  0xE9E9,   /* U+1E9E9 */
  0xE9EA,   /* U+1E9EA */
  0xE9EB,   /* U+1E9EB */
  0xE9EC,   /* U+1E9EC */
  0xE9ED,   /* U+1E9ED */
  0xE9EE,   /* U+1E9EE */
  0xE9EF,   /* U+1E9EF */
  0xE9F0,   /* U+1E9F0 */
  0xE9F1,   /* U+1E9F1 */
  0xE9F2,   /* U+1E9F2 */
  0xE9F3,   /* U+1E9F3 */
  0xE9F4,   /* U+1E9F4 */
  0xE9F5,   /* U+1E9F5 */
  0xE9F6,   /* U+1E9F6 */
  0xE9F7,   /* U+1E9F7 */
  0xE9F8,   /* U+1E9F8 */
  0xE9F9,   /* U+1E9F9 */
  0xE9FA,   /* U+1E9FA */
  0xE9FB,   /* U+1E9FB */
  0xE9FC,   /* U+1E9FC */
  0xE9FD,   /* U+1E9FD */
  0xE9FE,   /* U+1E9FE */
  0xE9FF,   /* U+1E9FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1E900 */
  0x0000,   /* U+1E901 */
  0x0000,   /* U+1E902 */
  0x0000,   /* U+1E903 */
  0x0000,   /* U+1E904 */
  0x0000,   /* U+1E905 */
  0x0000,   /* U+1E906 */
  0x0000,   /* U+1E907 */
  0x0000,   /* U+1E908 */
  0x0000,   /* U+1E909 */
  0x0000,   /* U+1E90A */
  0x0000,   /* U+1E90B */
  0x0000,   /* U+1E90C */
  0x0000,   /* U+1E90D */
  0x0000,   /* U+1E90E */
  0x0000,   /* U+1E90F */
  0x0000,   /* U+1E910 */
  0x0000,   /* U+1E911 */
  0x0000,   /* U+1E912 */
  0x0000,   /* U+1E913 */
  0x0000,   /* U+1E914 */
  0x0000,   /* U+1E915 */
  0x0000,   /* U+1E916 */
  0x0000,   /* U+1E917 */
  0x0000,   /* U+1E918 */
  0x0000,   /* U+1E919 */
  0x0000,   /* U+1E91A */
  0x0000,   /* U+1E91B */
  0x0000,   /* U+1E91C */
  0x0000,   /* U+1E91D */
  0x0000,   /* U+1E91E */
  0x0000,   /* U+1E91F */
  0x0000,   /* U+1E920 */
  0x0000,   /* U+1E921 */
  0x0000,   /* U+1E922 */
  0x0000,   /* U+1E923 */
  0x0000,   /* U+1E924 */
  0x0000,   /* U+1E925 */
  0x0000,   /* U+1E926 */
  0x0000,   /* U+1E927 */
  0x0000,   /* U+1E928 */
  0x0000,   /* U+1E929 */
  0x0000,   /* U+1E92A */
  0x0000,   /* U+1E92B */
  0x0000,   /* U+1E92C */
  0x0000,   /* U+1E92D */
  0x0000,   /* U+1E92E */
  0x0000,   /* U+1E92F */
  0x0000,   /* U+1E930 */
  0x0000,   /* U+1E931 */
  0x0000,   /* U+1E932 */
  0x0000,   /* U+1E933 */
  0x0000,   /* U+1E934 */
  0x0000,   /* U+1E935 */
  0x0000,   /* U+1E936 */
  0x0000,   /* U+1E937 */
  0x0000,   /* U+1E938 */
  0x0000,   /* U+1E939 */
  0x0000,   /* U+1E93A */
  0x0000,   /* U+1E93B */
  0x0000,   /* U+1E93C */
  0x0000,   /* U+1E93D */
  0x0000,   /* U+1E93E */
  0x0000,   /* U+1E93F */
  0x0000,   /* U+1E940 */
  0x0000,   /* U+1E941 */
  0x0000,   /* U+1E942 */
  0x0000,   /* U+1E943 */
  0x0000,   /* U+1E944 */
  0x0000,   /* U+1E945 */
  0x0000,   /* U+1E946 */
  0x0000,   /* U+1E947 */
  0x0000,   /* U+1E948 */
  0x0000,   /* U+1E949 */
  0x0000,   /* U+1E94A */
  0x0000,   /* U+1E94B */
  0x0000,   /* U+1E94C */
  0x0000,   /* U+1E94D */
  0x0000,   /* U+1E94E */
  0x0000,   /* U+1E94F */
  0x0000,   /* U+1E950 */
  0x0000,   /* U+1E951 */
  0x0000,   /* U+1E952 */
  0x0000,   /* U+1E953 */
  0x0000,   /* U+1E954 */
  0x0000,   /* U+1E955 */
  0x0000,   /* U+1E956 */
  0x0000,   /* U+1E957 */
  0x0000,   /* U+1E958 */
  0x0000,   /* U+1E959 */
  0x0000,   /* U+1E95A */
  0x0000,   /* U+1E95B */
  0x0000,   /* U+1E95C */
  0x0000,   /* U+1E95D */
  0x0000,   /* U+1E95E */
  0x0000,   /* U+1E95F */
  0x0000,   /* U+1E960 */
  0x0000,   /* U+1E961 */
  0x0000,   /* U+1E962 */
  0x0000,   /* U+1E963 */
  0x0000,   /* U+1E964 */
  0x0000,   /* U+1E965 */
  0x0000,   /* U+1E966 */
  0x0000,   /* U+1E967 */
  0x0000,   /* U+1E968 */
  0x0000,   /* U+1E969 */
  0x0000,   /* U+1E96A */
  0x0000,   /* U+1E96B */
  0x0000,   /* U+1E96C */
  0x0000,   /* U+1E96D */
  0x0000,   /* U+1E96E */
  0x0000,   /* U+1E96F */
  0x0000,   /* U+1E970 */
  0x0000,   /* U+1E971 */
  0x0000,   /* U+1E972 */
  0x0000,   /* U+1E973 */
  0x0000,   /* U+1E974 */
  0x0000,   /* U+1E975 */
  0x0000,   /* U+1E976 */
  0x0000,   /* U+1E977 */
  0x0000,   /* U+1E978 */
  0x0000,   /* U+1E979 */
  0x0000,   /* U+1E97A */
  0x0000,   /* U+1E97B */
  0x0000,   /* U+1E97C */
  0x0000,   /* U+1E97D */
  0x0000,   /* U+1E97E */
  0x0000,   /* U+1E97F */
  0x0000,   /* U+1E980 */
  0x0000,   /* U+1E981 */
  0x0000,   /* U+1E982 */
  0x0000,   /* U+1E983 */
  0x0000,   /* U+1E984 */
  0x0000,   /* U+1E985 */
  0x0000,   /* U+1E986 */
  0x0000,   /* U+1E987 */
  0x0000,   /* U+1E988 */
  0x0000,   /* U+1E989 */
  0x0000,   /* U+1E98A */
  0x0000,   /* U+1E98B */
  0x0000,   /* U+1E98C */
  0x0000,   /* U+1E98D */
  0x0000,   /* U+1E98E */
  0x0000,   /* U+1E98F */
  0x0000,   /* U+1E990 */
  0x0000,   /* U+1E991 */
  0x0000,   /* U+1E992 */
  0x0000,   /* U+1E993 */
  0x0000,   /* U+1E994 */
  0x0000,   /* U+1E995 */
  0x0000,   /* U+1E996 */
  0x0000,   /* U+1E997 */
  0x0000,   /* U+1E998 */
  0x0000,   /* U+1E999 */
  0x0000,   /* U+1E99A */
  0x0000,   /* U+1E99B */
  0x0000,   /* U+1E99C */
  0x0000,   /* U+1E99D */
  0x0000,   /* U+1E99E */
  0x0000,   /* U+1E99F */
  0x0000,   /* U+1E9A0 */
  0x0000,   /* U+1E9A1 */
  0x0000,   /* U+1E9A2 */
  0x0000,   /* U+1E9A3 */
  0x0000,   /* U+1E9A4 */
  0x0000,   /* U+1E9A5 */
  0x0000,   /* U+1E9A6 */
  0x0000,   /* U+1E9A7 */
  0x0000,   /* U+1E9A8 */
  0x0000,   /* U+1E9A9 */
  0x0000,   /* U+1E9AA */
  0x0000,   /* U+1E9AB */
  0x0000,   /* U+1E9AC */
  0x0000,   /* U+1E9AD */
  0x0000,   /* U+1E9AE */
  0x0000,   /* U+1E9AF */
  0x0000,   /* U+1E9B0 */
  0x0000,   /* U+1E9B1 */
  0x0000,   /* U+1E9B2 */
  0x0000,   /* U+1E9B3 */
  0x0000,   /* U+1E9B4 */
  0x0000,   /* U+1E9B5 */
  0x0000,   /* U+1E9B6 */
  0x0000,   /* U+1E9B7 */
  0x0000,   /* U+1E9B8 */
  0x0000,   /* U+1E9B9 */
  0x0000,   /* U+1E9BA */
  0x0000,   /* U+1E9BB */
  0x0000,   /* U+1E9BC */
  0x0000,   /* U+1E9BD */
  0x0000,   /* U+1E9BE */
  0x0000,   /* U+1E9BF */
  0x0000,   /* U+1E9C0 */
  0x0000,   /* U+1E9C1 */
  0x0000,   /* U+1E9C2 */
  0x0000,   /* U+1E9C3 */
  0x0000,   /* U+1E9C4 */
  0x0000,   /* U+1E9C5 */
  0x0000,   /* U+1E9C6 */
  0x0000,   /* U+1E9C7 */
  0x0000,   /* U+1E9C8 */
  0x0000,   /* U+1E9C9 */
  0x0000,   /* U+1E9CA */
  0x0000,   /* U+1E9CB */
  0x0000,   /* U+1E9CC */
  0x0000,   /* U+1E9CD */
  0x0000,   /* U+1E9CE */
  0x0000,   /* U+1E9CF */
  0x0000,   /* U+1E9D0 */
  0x0000,   /* U+1E9D1 */
  0x0000,   /* U+1E9D2 */
  0x0000,   /* U+1E9D3 */
  0x0000,   /* U+1E9D4 */
  0x0000,   /* U+1E9D5 */
  0x0000,   /* U+1E9D6 */
  0x0000,   /* U+1E9D7 */
  0x0000,   /* U+1E9D8 */
  0x0000,   /* U+1E9D9 */
  0x0000,   /* U+1E9DA */
  0x0000,   /* U+1E9DB */
  0x0000,   /* U+1E9DC */
  0x0000,   /* U+1E9DD */
  0x0000,   /* U+1E9DE */
  0x0000,   /* U+1E9DF */
  0x0000,   /* U+1E9E0 */
  0x0000,   /* U+1E9E1 */
  0x0000,   /* U+1E9E2 */
  0x0000,   /* U+1E9E3 */
  0x0000,   /* U+1E9E4 */
  0x0000,   /* U+1E9E5 */
  0x0000,   /* U+1E9E6 */
  0x0000,   /* U+1E9E7 */
  0x0000,   /* U+1E9E8 */
  0x0000,   /* U+1E9E9 */
  0x0000,   /* U+1E9EA */
  0x0000,   /* U+1E9EB */
  0x0000,   /* U+1E9EC */
  0x0000,   /* U+1E9ED */
  0x0000,   /* U+1E9EE */
  0x0000,   /* U+1E9EF */
  0x0000,   /* U+1E9F0 */
  0x0000,   /* U+1E9F1 */
  0x0000,   /* U+1E9F2 */
  0x0000,   /* U+1E9F3 */
  0x0000,   /* U+1E9F4 */
  0x0000,   /* U+1E9F5 */
  0x0000,   /* U+1E9F6 */
  0x0000,   /* U+1E9F7 */
  0x0000,   /* U+1E9F8 */
  0x0000,   /* U+1E9F9 */
  0x0000,   /* U+1E9FA */
  0x0000,   /* U+1E9FB */
  0x0000,   /* U+1E9FC */
  0x0000,   /* U+1E9FD */
  0x0000,   /* U+1E9FE */
  0x0000,   /* U+1E9FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1E900 */
  0x0000,   /* U+1E901 */
  0x0000,   /* U+1E902 */
  0x0000,   /* U+1E903 */
  0x0000,   /* U+1E904 */
  0x0000,   /* U+1E905 */
  0x0000,   /* U+1E906 */
  0x0000,   /* U+1E907 */
  0x0000,   /* U+1E908 */
  0x0000,   /* U+1E909 */
  0x0000,   /* U+1E90A */
  0x0000,   /* U+1E90B */
  0x0000,   /* U+1E90C */
  0x0000,   /* U+1E90D */
  0x0000,   /* U+1E90E */
  0x0000,   /* U+1E90F */
  0x0000,   /* U+1E910 */
  0x0000,   /* U+1E911 */
  0x0000,   /* U+1E912 */
  0x0000,   /* U+1E913 */
  0x0000,   /* U+1E914 */
  0x0000,   /* U+1E915 */
  0x0000,   /* U+1E916 */
  0x0000,   /* U+1E917 */
  0x0000,   /* U+1E918 */
  0x0000,   /* U+1E919 */
  0x0000,   /* U+1E91A */
  0x0000,   /* U+1E91B */
  0x0000,   /* U+1E91C */
  0x0000,   /* U+1E91D */
  0x0000,   /* U+1E91E */
  0x0000,   /* U+1E91F */
  0x0000,   /* U+1E920 */
  0x0000,   /* U+1E921 */
  0x0000,   /* U+1E922 */
  0x0000,   /* U+1E923 */
  0x0000,   /* U+1E924 */
  0x0000,   /* U+1E925 */
  0x0000,   /* U+1E926 */
  0x0000,   /* U+1E927 */
  0x0000,   /* U+1E928 */
  0x0000,   /* U+1E929 */
  0x0000,   /* U+1E92A */
  0x0000,   /* U+1E92B */
  0x0000,   /* U+1E92C */
  0x0000,   /* U+1E92D */
  0x0000,   /* U+1E92E */
  0x0000,   /* U+1E92F */
  0x0000,   /* U+1E930 */
  0x0000,   /* U+1E931 */
  0x0000,   /* U+1E932 */
  0x0000,   /* U+1E933 */
  0x0000,   /* U+1E934 */
  0x0000,   /* U+1E935 */
  0x0000,   /* U+1E936 */
  0x0000,   /* U+1E937 */
  0x0000,   /* U+1E938 */
  0x0000,   /* U+1E939 */
  0x0000,   /* U+1E93A */
  0x0000,   /* U+1E93B */
  0x0000,   /* U+1E93C */
  0x0000,   /* U+1E93D */
  0x0000,   /* U+1E93E */
  0x0000,   /* U+1E93F */
  0x0000,   /* U+1E940 */
  0x0000,   /* U+1E941 */
  0x0000,   /* U+1E942 */
  0x0000,   /* U+1E943 */
  0x0000,   /* U+1E944 */
  0x0000,   /* U+1E945 */
  0x0000,   /* U+1E946 */
  0x0000,   /* U+1E947 */
  0x0000,   /* U+1E948 */
  0x0000,   /* U+1E949 */
  0x0000,   /* U+1E94A */
  0x0000,   /* U+1E94B */
  0x0000,   /* U+1E94C */
  0x0000,   /* U+1E94D */
  0x0000,   /* U+1E94E */
  0x0000,   /* U+1E94F */
  0x0000,   /* U+1E950 */
  0x0000,   /* U+1E951 */
  0x0000,   /* U+1E952 */
  0x0000,   /* U+1E953 */
  0x0000,   /* U+1E954 */
  0x0000,   /* U+1E955 */
  0x0000,   /* U+1E956 */
  0x0000,   /* U+1E957 */
  0x0000,   /* U+1E958 */
  0x0000,   /* U+1E959 */
  0x0000,   /* U+1E95A */
  0x0000,   /* U+1E95B */
  0x0000,   /* U+1E95C */
  0x0000,   /* U+1E95D */
  0x0000,   /* U+1E95E */
  0x0000,   /* U+1E95F */
  0x0000,   /* U+1E960 */
  0x0000,   /* U+1E961 */
  0x0000,   /* U+1E962 */
  0x0000,   /* U+1E963 */
  0x0000,   /* U+1E964 */
  0x0000,   /* U+1E965 */
  0x0000,   /* U+1E966 */
  0x0000,   /* U+1E967 */
  0x0000,   /* U+1E968 */
  0x0000,   /* U+1E969 */
  0x0000,   /* U+1E96A */
  0x0000,   /* U+1E96B */
  0x0000,   /* U+1E96C */
  0x0000,   /* U+1E96D */
  0x0000,   /* U+1E96E */
  0x0000,   /* U+1E96F */
  0x0000,   /* U+1E970 */
  0x0000,   /* U+1E971 */
  0x0000,   /* U+1E972 */
  0x0000,   /* U+1E973 */
  0x0000,   /* U+1E974 */
  0x0000,   /* U+1E975 */
  0x0000,   /* U+1E976 */
  0x0000,   /* U+1E977 */
  0x0000,   /* U+1E978 */
  0x0000,   /* U+1E979 */
  0x0000,   /* U+1E97A */
  0x0000,   /* U+1E97B */
  0x0000,   /* U+1E97C */
  0x0000,   /* U+1E97D */
  0x0000,   /* U+1E97E */
  0x0000,   /* U+1E97F */
  0x0000,   /* U+1E980 */
  0x0000,   /* U+1E981 */
  0x0000,   /* U+1E982 */
  0x0000,   /* U+1E983 */
  0x0000,   /* U+1E984 */
  0x0000,   /* U+1E985 */
  0x0000,   /* U+1E986 */
  0x0000,   /* U+1E987 */
  0x0000,   /* U+1E988 */
  0x0000,   /* U+1E989 */
  0x0000,   /* U+1E98A */
  0x0000,   /* U+1E98B */
  0x0000,   /* U+1E98C */
  0x0000,   /* U+1E98D */
  0x0000,   /* U+1E98E */
  0x0000,   /* U+1E98F */
  0x0000,   /* U+1E990 */
  0x0000,   /* U+1E991 */
  0x0000,   /* U+1E992 */
  0x0000,   /* U+1E993 */
  0x0000,   /* U+1E994 */
  0x0000,   /* U+1E995 */
  0x0000,   /* U+1E996 */
  0x0000,   /* U+1E997 */
  0x0000,   /* U+1E998 */
  0x0000,   /* U+1E999 */
  0x0000,   /* U+1E99A */
  0x0000,   /* U+1E99B */
  0x0000,   /* U+1E99C */
  0x0000,   /* U+1E99D */
  0x0000,   /* U+1E99E */
  0x0000,   /* U+1E99F */
  0x0000,   /* U+1E9A0 */
  0x0000,   /* U+1E9A1 */
  0x0000,   /* U+1E9A2 */
  0x0000,   /* U+1E9A3 */
  0x0000,   /* U+1E9A4 */
  0x0000,   /* U+1E9A5 */
  0x0000,   /* U+1E9A6 */
  0x0000,   /* U+1E9A7 */
  0x0000,   /* U+1E9A8 */
  0x0000,   /* U+1E9A9 */
  0x0000,   /* U+1E9AA */
  0x0000,   /* U+1E9AB */
  0x0000,   /* U+1E9AC */
  0x0000,   /* U+1E9AD */
  0x0000,   /* U+1E9AE */
  0x0000,   /* U+1E9AF */
  0x0000,   /* U+1E9B0 */
  0x0000,   /* U+1E9B1 */
  0x0000,   /* U+1E9B2 */
  0x0000,   /* U+1E9B3 */
  0x0000,   /* U+1E9B4 */
  0x0000,   /* U+1E9B5 */
  0x0000,   /* U+1E9B6 */
  0x0000,   /* U+1E9B7 */
  0x0000,   /* U+1E9B8 */
  0x0000,   /* U+1E9B9 */
  0x0000,   /* U+1E9BA */
  0x0000,   /* U+1E9BB */
  0x0000,   /* U+1E9BC */
  0x0000,   /* U+1E9BD */
  0x0000,   /* U+1E9BE */
  0x0000,   /* U+1E9BF */
  0x0000,   /* U+1E9C0 */
  0x0000,   /* U+1E9C1 */
  0x0000,   /* U+1E9C2 */
  0x0000,   /* U+1E9C3 */
  0x0000,   /* U+1E9C4 */
  0x0000,   /* U+1E9C5 */
  0x0000,   /* U+1E9C6 */
  0x0000,   /* U+1E9C7 */
  0x0000,   /* U+1E9C8 */
  0x0000,   /* U+1E9C9 */
  0x0000,   /* U+1E9CA */
  0x0000,   /* U+1E9CB */
  0x0000,   /* U+1E9CC */
  0x0000,   /* U+1E9CD */
  0x0000,   /* U+1E9CE */
  0x0000,   /* U+1E9CF */
  0x0000,   /* U+1E9D0 */
  0x0000,   /* U+1E9D1 */
  0x0000,   /* U+1E9D2 */
  0x0000,   /* U+1E9D3 */
  0x0000,   /* U+1E9D4 */
  0x0000,   /* U+1E9D5 */
  0x0000,   /* U+1E9D6 */
  0x0000,   /* U+1E9D7 */
  0x0000,   /* U+1E9D8 */
  0x0000,   /* U+1E9D9 */
  0x0000,   /* U+1E9DA */
  0x0000,   /* U+1E9DB */
  0x0000,   /* U+1E9DC */
  0x0000,   /* U+1E9DD */
  0x0000,   /* U+1E9DE */
  0x0000,   /* U+1E9DF */
  0x0000,   /* U+1E9E0 */
  0x0000,   /* U+1E9E1 */
  0x0000,   /* U+1E9E2 */
  0x0000,   /* U+1E9E3 */
  0x0000,   /* U+1E9E4 */
  0x0000,   /* U+1E9E5 */
  0x0000,   /* U+1E9E6 */
  0x0000,   /* U+1E9E7 */
  0x0000,   /* U+1E9E8 */
  0x0000,   /* U+1E9E9 */
  0x0000,   /* U+1E9EA */
  0x0000,   /* U+1E9EB */
  0x0000,   /* U+1E9EC */
  0x0000,   /* U+1E9ED */
  0x0000,   /* U+1E9EE */
  0x0000,   /* U+1E9EF */
  0x0000,   /* U+1E9F0 */
  0x0000,   /* U+1E9F1 */
  0x0000,   /* U+1E9F2 */
  0x0000,   /* U+1E9F3 */
  0x0000,   /* U+1E9F4 */
  0x0000,   /* U+1E9F5 */
  0x0000,   /* U+1E9F6 */
  0x0000,   /* U+1E9F7 */
  0x0000,   /* U+1E9F8 */
  0x0000,   /* U+1E9F9 */
  0x0000,   /* U+1E9FA */
  0x0000,   /* U+1E9FB */
  0x0000,   /* U+1E9FC */
  0x0000,   /* U+1E9FD */
  0x0000,   /* U+1E9FE */
  0x0000,   /* U+1E9FF */
};

uint16 uca900_p1EE[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 2, 1, 2, 2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 2, 1, 2, 2, 2, 2, 
  2, 2, 1, 2, 2, 2, 2, 1, 2, 1, 2, 1, 2, 1, 1, 1, 
  2, 1, 1, 2, 1, 2, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 
  2, 1, 1, 2, 1, 2, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 
  1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  2, 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x230B,   /* U+1EE00 */
  0x230D,   /* U+1EE01 */
  0x2325,   /* U+1EE02 */
  0x2337,   /* U+1EE03 */
  0xFBC3,   /* U+1EE04 */
  0x23B7,   /* U+1EE05 */
  0x2347,   /* U+1EE06 */
  0x232C,   /* U+1EE07 */
  0x236A,   /* U+1EE08 */
  0x23C6,   /* U+1EE09 */
  0x2387,   /* U+1EE0A */
  0x239C,   /* U+1EE0B */
  0x23A3,   /* U+1EE0C */
  0x23A7,   /* U+1EE0D */
  0x2359,   /* U+1EE0E */
  0x236E,   /* U+1EE0F */
  0x2376,   /* U+1EE10 */
  0x2364,   /* U+1EE11 */
  0x2382,   /* U+1EE12 */
  0x2346,   /* U+1EE13 */
  0x235A,   /* U+1EE14 */
  0x231D,   /* U+1EE15 */
  0x231E,   /* U+1EE16 */
  0x232D,   /* U+1EE17 */
  0x2338,   /* U+1EE18 */
  0x2365,   /* U+1EE19 */
  0x236B,   /* U+1EE1A */
  0x236F,   /* U+1EE1B */
  0x230C,   /* U+1EE1C */
  0x23A8,   /* U+1EE1D */
  0x2377,   /* U+1EE1E */
  0x2381,   /* U+1EE1F */
  0xFBC3,   /* U+1EE20 */
  0x230D,   /* U+1EE21 */
  0x2325,   /* U+1EE22 */
  0xFBC3,   /* U+1EE23 */
  0x23B1,   /* U+1EE24 */
  0xFBC3,   /* U+1EE25 */
  0xFBC3,   /* U+1EE26 */
  0x232C,   /* U+1EE27 */
  0xFBC3,   /* U+1EE28 */
  0x23C6,   /* U+1EE29 */
  0x2387,   /* U+1EE2A */
  0x239C,   /* U+1EE2B */
  0x23A3,   /* U+1EE2C */
  0x23A7,   /* U+1EE2D */
  0x2359,   /* U+1EE2E */
  0x236E,   /* U+1EE2F */
  0x2376,   /* U+1EE30 */
  0x2364,   /* U+1EE31 */
  0x2382,   /* U+1EE32 */
  0xFBC3,   /* U+1EE33 */
  0x235A,   /* U+1EE34 */
  0x231D,   /* U+1EE35 */
  0x231E,   /* U+1EE36 */
  0x232D,   /* U+1EE37 */
  0xFBC3,   /* U+1EE38 */
  0x2365,   /* U+1EE39 */
  0xFBC3,   /* U+1EE3A */
  0x236F,   /* U+1EE3B */
  0xFBC3,   /* U+1EE3C */
  0xFBC3,   /* U+1EE3D */
  0xFBC3,   /* U+1EE3E */
  0xFBC3,   /* U+1EE3F */
  0xFBC3,   /* U+1EE40 */
  0xFBC3,   /* U+1EE41 */
  0x2325,   /* U+1EE42 */
  0xFBC3,   /* U+1EE43 */
  0xFBC3,   /* U+1EE44 */
  0xFBC3,   /* U+1EE45 */
  0xFBC3,   /* U+1EE46 */
  0x232C,   /* U+1EE47 */
  0xFBC3,   /* U+1EE48 */
  0x23C6,   /* U+1EE49 */
  0xFBC3,   /* U+1EE4A */
  0x239C,   /* U+1EE4B */
  0xFBC3,   /* U+1EE4C */
  0x23A7,   /* U+1EE4D */
  0x2359,   /* U+1EE4E */
  0x236E,   /* U+1EE4F */
  0xFBC3,   /* U+1EE50 */
  0x2364,   /* U+1EE51 */
  0x2382,   /* U+1EE52 */
  0xFBC3,   /* U+1EE53 */
  0x235A,   /* U+1EE54 */
  0xFBC3,   /* U+1EE55 */
  0xFBC3,   /* U+1EE56 */
  0x232D,   /* U+1EE57 */
  0xFBC3,   /* U+1EE58 */
  0x2365,   /* U+1EE59 */
  0xFBC3,   /* U+1EE5A */
  0x236F,   /* U+1EE5B */
  0xFBC3,   /* U+1EE5C */
  0x23A8,   /* U+1EE5D */
  0xFBC3,   /* U+1EE5E */
  0x2381,   /* U+1EE5F */
  0xFBC3,   /* U+1EE60 */
  0x230D,   /* U+1EE61 */
  0x2325,   /* U+1EE62 */
  0xFBC3,   /* U+1EE63 */
  0x23B1,   /* U+1EE64 */
  0xFBC3,   /* U+1EE65 */
  0xFBC3,   /* U+1EE66 */
  0x232C,   /* U+1EE67 */
  0x236A,   /* U+1EE68 */
  0x23C6,   /* U+1EE69 */
  0x2387,   /* U+1EE6A */
  0xFBC3,   /* U+1EE6B */
  0x23A3,   /* U+1EE6C */
  0x23A7,   /* U+1EE6D */
  0x2359,   /* U+1EE6E */
  0x236E,   /* U+1EE6F */
  0x2376,   /* U+1EE70 */
  0x2364,   /* U+1EE71 */
  0x2382,   /* U+1EE72 */
  0xFBC3,   /* U+1EE73 */
  0x235A,   /* U+1EE74 */
  0x231D,   /* U+1EE75 */
  0x231E,   /* U+1EE76 */
  0x232D,   /* U+1EE77 */
  0xFBC3,   /* U+1EE78 */
  0x2365,   /* U+1EE79 */
  0x236B,   /* U+1EE7A */
  0x236F,   /* U+1EE7B */
  0x230C,   /* U+1EE7C */
  0xFBC3,   /* U+1EE7D */
  0x2377,   /* U+1EE7E */
  0xFBC3,   /* U+1EE7F */
  0x230B,   /* U+1EE80 */
  0x230D,   /* U+1EE81 */
  0x2325,   /* U+1EE82 */
  0x2337,   /* U+1EE83 */
  0x23B1,   /* U+1EE84 */
  0x23B7,   /* U+1EE85 */
  0x2347,   /* U+1EE86 */
  0x232C,   /* U+1EE87 */
  0x236A,   /* U+1EE88 */
  0x23C6,   /* U+1EE89 */
  0xFBC3,   /* U+1EE8A */
  0x239C,   /* U+1EE8B */
  0x23A3,   /* U+1EE8C */
  0x23A7,   /* U+1EE8D */
  0x2359,   /* U+1EE8E */
  0x236E,   /* U+1EE8F */
  0x2376,   /* U+1EE90 */
  0x2364,   /* U+1EE91 */
  0x2382,   /* U+1EE92 */
  0x2346,   /* U+1EE93 */
  0x235A,   /* U+1EE94 */
  0x231D,   /* U+1EE95 */
  0x231E,   /* U+1EE96 */
  0x232D,   /* U+1EE97 */
  0x2338,   /* U+1EE98 */
  0x2365,   /* U+1EE99 */
  0x236B,   /* U+1EE9A */
  0x236F,   /* U+1EE9B */
  0xFBC3,   /* U+1EE9C */
  0xFBC3,   /* U+1EE9D */
  0xFBC3,   /* U+1EE9E */
  0xFBC3,   /* U+1EE9F */
  0xFBC3,   /* U+1EEA0 */
  0x230D,   /* U+1EEA1 */
  0x2325,   /* U+1EEA2 */
  0x2337,   /* U+1EEA3 */
  0xFBC3,   /* U+1EEA4 */
  0x23B7,   /* U+1EEA5 */
  0x2347,   /* U+1EEA6 */
  0x232C,   /* U+1EEA7 */
  0x236A,   /* U+1EEA8 */
  0x23C6,   /* U+1EEA9 */
  0xFBC3,   /* U+1EEAA */
  0x239C,   /* U+1EEAB */
  0x23A3,   /* U+1EEAC */
  0x23A7,   /* U+1EEAD */
  0x2359,   /* U+1EEAE */
  0x236E,   /* U+1EEAF */
  0x2376,   /* U+1EEB0 */
  0x2364,   /* U+1EEB1 */
  0x2382,   /* U+1EEB2 */
  0x2346,   /* U+1EEB3 */
  0x235A,   /* U+1EEB4 */
  0x231D,   /* U+1EEB5 */
  0x231E,   /* U+1EEB6 */
  0x232D,   /* U+1EEB7 */
  0x2338,   /* U+1EEB8 */
  0x2365,   /* U+1EEB9 */
  0x236B,   /* U+1EEBA */
  0x236F,   /* U+1EEBB */
  0xFBC3,   /* U+1EEBC */
  0xFBC3,   /* U+1EEBD */
  0xFBC3,   /* U+1EEBE */
  0xFBC3,   /* U+1EEBF */
  0xFBC3,   /* U+1EEC0 */
  0xFBC3,   /* U+1EEC1 */
  0xFBC3,   /* U+1EEC2 */
  0xFBC3,   /* U+1EEC3 */
  0xFBC3,   /* U+1EEC4 */
  0xFBC3,   /* U+1EEC5 */
  0xFBC3,   /* U+1EEC6 */
  0xFBC3,   /* U+1EEC7 */
  0xFBC3,   /* U+1EEC8 */
  0xFBC3,   /* U+1EEC9 */
  0xFBC3,   /* U+1EECA */
  0xFBC3,   /* U+1EECB */
  0xFBC3,   /* U+1EECC */
  0xFBC3,   /* U+1EECD */
  0xFBC3,   /* U+1EECE */
  0xFBC3,   /* U+1EECF */
  0xFBC3,   /* U+1EED0 */
  0xFBC3,   /* U+1EED1 */
  0xFBC3,   /* U+1EED2 */
  0xFBC3,   /* U+1EED3 */
  0xFBC3,   /* U+1EED4 */
  0xFBC3,   /* U+1EED5 */
  0xFBC3,   /* U+1EED6 */
  0xFBC3,   /* U+1EED7 */
  0xFBC3,   /* U+1EED8 */
  0xFBC3,   /* U+1EED9 */
  0xFBC3,   /* U+1EEDA */
  0xFBC3,   /* U+1EEDB */
  0xFBC3,   /* U+1EEDC */
  0xFBC3,   /* U+1EEDD */
  0xFBC3,   /* U+1EEDE */
  0xFBC3,   /* U+1EEDF */
  0xFBC3,   /* U+1EEE0 */
  0xFBC3,   /* U+1EEE1 */
  0xFBC3,   /* U+1EEE2 */
  0xFBC3,   /* U+1EEE3 */
  0xFBC3,   /* U+1EEE4 */
  0xFBC3,   /* U+1EEE5 */
  0xFBC3,   /* U+1EEE6 */
  0xFBC3,   /* U+1EEE7 */
  0xFBC3,   /* U+1EEE8 */
  0xFBC3,   /* U+1EEE9 */
  0xFBC3,   /* U+1EEEA */
  0xFBC3,   /* U+1EEEB */
  0xFBC3,   /* U+1EEEC */
  0xFBC3,   /* U+1EEED */
  0xFBC3,   /* U+1EEEE */
  0xFBC3,   /* U+1EEEF */
  0x04FB,   /* U+1EEF0 */
  0x04FC,   /* U+1EEF1 */
  0xFBC3,   /* U+1EEF2 */
  0xFBC3,   /* U+1EEF3 */
  0xFBC3,   /* U+1EEF4 */
  0xFBC3,   /* U+1EEF5 */
  0xFBC3,   /* U+1EEF6 */
  0xFBC3,   /* U+1EEF7 */
  0xFBC3,   /* U+1EEF8 */
  0xFBC3,   /* U+1EEF9 */
  0xFBC3,   /* U+1EEFA */
  0xFBC3,   /* U+1EEFB */
  0xFBC3,   /* U+1EEFC */
  0xFBC3,   /* U+1EEFD */
  0xFBC3,   /* U+1EEFE */
  0xFBC3,   /* U+1EEFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1EE00 */
  0x0020,   /* U+1EE01 */
  0x0020,   /* U+1EE02 */
  0x0020,   /* U+1EE03 */
  0x0020,   /* U+1EE04 */
  0x0020,   /* U+1EE05 */
  0x0020,   /* U+1EE06 */
  0x0020,   /* U+1EE07 */
  0x0020,   /* U+1EE08 */
  0x0020,   /* U+1EE09 */
  0x0020,   /* U+1EE0A */
  0x0020,   /* U+1EE0B */
  0x0020,   /* U+1EE0C */
  0x0020,   /* U+1EE0D */
  0x0020,   /* U+1EE0E */
  0x0020,   /* U+1EE0F */
  0x0020,   /* U+1EE10 */
  0x0020,   /* U+1EE11 */
  0x0020,   /* U+1EE12 */
  0x0020,   /* U+1EE13 */
  0x0020,   /* U+1EE14 */
  0x0020,   /* U+1EE15 */
  0x0020,   /* U+1EE16 */
  0x0020,   /* U+1EE17 */
  0x0020,   /* U+1EE18 */
  0x0020,   /* U+1EE19 */
  0x0020,   /* U+1EE1A */
  0x0020,   /* U+1EE1B */
  0x0020,   /* U+1EE1C */
  0x0020,   /* U+1EE1D */
  0x0020,   /* U+1EE1E */
  0x0020,   /* U+1EE1F */
  0x0020,   /* U+1EE20 */
  0x0020,   /* U+1EE21 */
  0x0020,   /* U+1EE22 */
  0x0020,   /* U+1EE23 */
  0x0020,   /* U+1EE24 */
  0x0020,   /* U+1EE25 */
  0x0020,   /* U+1EE26 */
  0x0020,   /* U+1EE27 */
  0x0020,   /* U+1EE28 */
  0x0020,   /* U+1EE29 */
  0x0020,   /* U+1EE2A */
  0x0020,   /* U+1EE2B */
  0x0020,   /* U+1EE2C */
  0x0020,   /* U+1EE2D */
  0x0020,   /* U+1EE2E */
  0x0020,   /* U+1EE2F */
  0x0020,   /* U+1EE30 */
  0x0020,   /* U+1EE31 */
  0x0020,   /* U+1EE32 */
  0x0020,   /* U+1EE33 */
  0x0020,   /* U+1EE34 */
  0x0020,   /* U+1EE35 */
  0x0020,   /* U+1EE36 */
  0x0020,   /* U+1EE37 */
  0x0020,   /* U+1EE38 */
  0x0020,   /* U+1EE39 */
  0x0020,   /* U+1EE3A */
  0x0020,   /* U+1EE3B */
  0x0020,   /* U+1EE3C */
  0x0020,   /* U+1EE3D */
  0x0020,   /* U+1EE3E */
  0x0020,   /* U+1EE3F */
  0x0020,   /* U+1EE40 */
  0x0020,   /* U+1EE41 */
  0x0020,   /* U+1EE42 */
  0x0020,   /* U+1EE43 */
  0x0020,   /* U+1EE44 */
  0x0020,   /* U+1EE45 */
  0x0020,   /* U+1EE46 */
  0x0020,   /* U+1EE47 */
  0x0020,   /* U+1EE48 */
  0x0020,   /* U+1EE49 */
  0x0020,   /* U+1EE4A */
  0x0020,   /* U+1EE4B */
  0x0020,   /* U+1EE4C */
  0x0020,   /* U+1EE4D */
  0x0020,   /* U+1EE4E */
  0x0020,   /* U+1EE4F */
  0x0020,   /* U+1EE50 */
  0x0020,   /* U+1EE51 */
  0x0020,   /* U+1EE52 */
  0x0020,   /* U+1EE53 */
  0x0020,   /* U+1EE54 */
  0x0020,   /* U+1EE55 */
  0x0020,   /* U+1EE56 */
  0x0020,   /* U+1EE57 */
  0x0020,   /* U+1EE58 */
  0x0020,   /* U+1EE59 */
  0x0020,   /* U+1EE5A */
  0x0020,   /* U+1EE5B */
  0x0020,   /* U+1EE5C */
  0x0020,   /* U+1EE5D */
  0x0020,   /* U+1EE5E */
  0x0020,   /* U+1EE5F */
  0x0020,   /* U+1EE60 */
  0x0020,   /* U+1EE61 */
  0x0020,   /* U+1EE62 */
  0x0020,   /* U+1EE63 */
  0x0020,   /* U+1EE64 */
  0x0020,   /* U+1EE65 */
  0x0020,   /* U+1EE66 */
  0x0020,   /* U+1EE67 */
  0x0020,   /* U+1EE68 */
  0x0020,   /* U+1EE69 */
  0x0020,   /* U+1EE6A */
  0x0020,   /* U+1EE6B */
  0x0020,   /* U+1EE6C */
  0x0020,   /* U+1EE6D */
  0x0020,   /* U+1EE6E */
  0x0020,   /* U+1EE6F */
  0x0020,   /* U+1EE70 */
  0x0020,   /* U+1EE71 */
  0x0020,   /* U+1EE72 */
  0x0020,   /* U+1EE73 */
  0x0020,   /* U+1EE74 */
  0x0020,   /* U+1EE75 */
  0x0020,   /* U+1EE76 */
  0x0020,   /* U+1EE77 */
  0x0020,   /* U+1EE78 */
  0x0020,   /* U+1EE79 */
  0x0020,   /* U+1EE7A */
  0x0020,   /* U+1EE7B */
  0x0020,   /* U+1EE7C */
  0x0020,   /* U+1EE7D */
  0x0020,   /* U+1EE7E */
  0x0020,   /* U+1EE7F */
  0x0020,   /* U+1EE80 */
  0x0020,   /* U+1EE81 */
  0x0020,   /* U+1EE82 */
  0x0020,   /* U+1EE83 */
  0x0020,   /* U+1EE84 */
  0x0020,   /* U+1EE85 */
  0x0020,   /* U+1EE86 */
  0x0020,   /* U+1EE87 */
  0x0020,   /* U+1EE88 */
  0x0020,   /* U+1EE89 */
  0x0020,   /* U+1EE8A */
  0x0020,   /* U+1EE8B */
  0x0020,   /* U+1EE8C */
  0x0020,   /* U+1EE8D */
  0x0020,   /* U+1EE8E */
  0x0020,   /* U+1EE8F */
  0x0020,   /* U+1EE90 */
  0x0020,   /* U+1EE91 */
  0x0020,   /* U+1EE92 */
  0x0020,   /* U+1EE93 */
  0x0020,   /* U+1EE94 */
  0x0020,   /* U+1EE95 */
  0x0020,   /* U+1EE96 */
  0x0020,   /* U+1EE97 */
  0x0020,   /* U+1EE98 */
  0x0020,   /* U+1EE99 */
  0x0020,   /* U+1EE9A */
  0x0020,   /* U+1EE9B */
  0x0020,   /* U+1EE9C */
  0x0020,   /* U+1EE9D */
  0x0020,   /* U+1EE9E */
  0x0020,   /* U+1EE9F */
  0x0020,   /* U+1EEA0 */
  0x0020,   /* U+1EEA1 */
  0x0020,   /* U+1EEA2 */
  0x0020,   /* U+1EEA3 */
  0x0020,   /* U+1EEA4 */
  0x0020,   /* U+1EEA5 */
  0x0020,   /* U+1EEA6 */
  0x0020,   /* U+1EEA7 */
  0x0020,   /* U+1EEA8 */
  0x0020,   /* U+1EEA9 */
  0x0020,   /* U+1EEAA */
  0x0020,   /* U+1EEAB */
  0x0020,   /* U+1EEAC */
  0x0020,   /* U+1EEAD */
  0x0020,   /* U+1EEAE */
  0x0020,   /* U+1EEAF */
  0x0020,   /* U+1EEB0 */
  0x0020,   /* U+1EEB1 */
  0x0020,   /* U+1EEB2 */
  0x0020,   /* U+1EEB3 */
  0x0020,   /* U+1EEB4 */
  0x0020,   /* U+1EEB5 */
  0x0020,   /* U+1EEB6 */
  0x0020,   /* U+1EEB7 */
  0x0020,   /* U+1EEB8 */
  0x0020,   /* U+1EEB9 */
  0x0020,   /* U+1EEBA */
  0x0020,   /* U+1EEBB */
  0x0020,   /* U+1EEBC */
  0x0020,   /* U+1EEBD */
  0x0020,   /* U+1EEBE */
  0x0020,   /* U+1EEBF */
  0x0020,   /* U+1EEC0 */
  0x0020,   /* U+1EEC1 */
  0x0020,   /* U+1EEC2 */
  0x0020,   /* U+1EEC3 */
  0x0020,   /* U+1EEC4 */
  0x0020,   /* U+1EEC5 */
  0x0020,   /* U+1EEC6 */
  0x0020,   /* U+1EEC7 */
  0x0020,   /* U+1EEC8 */
  0x0020,   /* U+1EEC9 */
  0x0020,   /* U+1EECA */
  0x0020,   /* U+1EECB */
  0x0020,   /* U+1EECC */
  0x0020,   /* U+1EECD */
  0x0020,   /* U+1EECE */
  0x0020,   /* U+1EECF */
  0x0020,   /* U+1EED0 */
  0x0020,   /* U+1EED1 */
  0x0020,   /* U+1EED2 */
  0x0020,   /* U+1EED3 */
  0x0020,   /* U+1EED4 */
  0x0020,   /* U+1EED5 */
  0x0020,   /* U+1EED6 */
  0x0020,   /* U+1EED7 */
  0x0020,   /* U+1EED8 */
  0x0020,   /* U+1EED9 */
  0x0020,   /* U+1EEDA */
  0x0020,   /* U+1EEDB */
  0x0020,   /* U+1EEDC */
  0x0020,   /* U+1EEDD */
  0x0020,   /* U+1EEDE */
  0x0020,   /* U+1EEDF */
  0x0020,   /* U+1EEE0 */
  0x0020,   /* U+1EEE1 */
  0x0020,   /* U+1EEE2 */
  0x0020,   /* U+1EEE3 */
  0x0020,   /* U+1EEE4 */
  0x0020,   /* U+1EEE5 */
  0x0020,   /* U+1EEE6 */
  0x0020,   /* U+1EEE7 */
  0x0020,   /* U+1EEE8 */
  0x0020,   /* U+1EEE9 */
  0x0020,   /* U+1EEEA */
  0x0020,   /* U+1EEEB */
  0x0020,   /* U+1EEEC */
  0x0020,   /* U+1EEED */
  0x0020,   /* U+1EEEE */
  0x0020,   /* U+1EEEF */
  0x0020,   /* U+1EEF0 */
  0x0020,   /* U+1EEF1 */
  0x0020,   /* U+1EEF2 */
  0x0020,   /* U+1EEF3 */
  0x0020,   /* U+1EEF4 */
  0x0020,   /* U+1EEF5 */
  0x0020,   /* U+1EEF6 */
  0x0020,   /* U+1EEF7 */
  0x0020,   /* U+1EEF8 */
  0x0020,   /* U+1EEF9 */
  0x0020,   /* U+1EEFA */
  0x0020,   /* U+1EEFB */
  0x0020,   /* U+1EEFC */
  0x0020,   /* U+1EEFD */
  0x0020,   /* U+1EEFE */
  0x0020,   /* U+1EEFF */

  /* Tertiary weight 1 for each character. */
  0x0005,   /* U+1EE00 */
  0x0005,   /* U+1EE01 */
  0x0005,   /* U+1EE02 */
  0x0005,   /* U+1EE03 */
  0x0002,   /* U+1EE04 */
  0x0005,   /* U+1EE05 */
  0x0005,   /* U+1EE06 */
  0x0005,   /* U+1EE07 */
  0x0005,   /* U+1EE08 */
  0x0005,   /* U+1EE09 */
  0x0005,   /* U+1EE0A */
  0x0005,   /* U+1EE0B */
  0x0005,   /* U+1EE0C */
  0x0005,   /* U+1EE0D */
  0x0005,   /* U+1EE0E */
  0x0005,   /* U+1EE0F */
  0x0005,   /* U+1EE10 */
  0x0005,   /* U+1EE11 */
  0x0005,   /* U+1EE12 */
  0x0005,   /* U+1EE13 */
  0x0005,   /* U+1EE14 */
  0x0005,   /* U+1EE15 */
  0x0005,   /* U+1EE16 */
  0x0005,   /* U+1EE17 */
  0x0005,   /* U+1EE18 */
  0x0005,   /* U+1EE19 */
  0x0005,   /* U+1EE1A */
  0x0005,   /* U+1EE1B */
  0x0005,   /* U+1EE1C */
  0x0005,   /* U+1EE1D */
  0x0005,   /* U+1EE1E */
  0x0005,   /* U+1EE1F */
  0x0002,   /* U+1EE20 */
  0x0005,   /* U+1EE21 */
  0x0005,   /* U+1EE22 */
  0x0002,   /* U+1EE23 */
  0x0005,   /* U+1EE24 */
  0x0002,   /* U+1EE25 */
  0x0002,   /* U+1EE26 */
  0x0005,   /* U+1EE27 */
  0x0002,   /* U+1EE28 */
  0x0005,   /* U+1EE29 */
  0x0005,   /* U+1EE2A */
  0x0005,   /* U+1EE2B */
  0x0005,   /* U+1EE2C */
  0x0005,   /* U+1EE2D */
  0x0005,   /* U+1EE2E */
  0x0005,   /* U+1EE2F */
  0x0005,   /* U+1EE30 */
  0x0005,   /* U+1EE31 */
  0x0005,   /* U+1EE32 */
  0x0002,   /* U+1EE33 */
  0x0005,   /* U+1EE34 */
  0x0005,   /* U+1EE35 */
  0x0005,   /* U+1EE36 */
  0x0005,   /* U+1EE37 */
  0x0002,   /* U+1EE38 */
  0x0005,   /* U+1EE39 */
  0x0002,   /* U+1EE3A */
  0x0005,   /* U+1EE3B */
  0x0002,   /* U+1EE3C */
  0x0002,   /* U+1EE3D */
  0x0002,   /* U+1EE3E */
  0x0002,   /* U+1EE3F */
  0x0002,   /* U+1EE40 */
  0x0002,   /* U+1EE41 */
  0x0005,   /* U+1EE42 */
  0x0002,   /* U+1EE43 */
  0x0002,   /* U+1EE44 */
  0x0002,   /* U+1EE45 */
  0x0002,   /* U+1EE46 */
  0x0005,   /* U+1EE47 */
  0x0002,   /* U+1EE48 */
  0x0005,   /* U+1EE49 */
  0x0002,   /* U+1EE4A */
  0x0005,   /* U+1EE4B */
  0x0002,   /* U+1EE4C */
  0x0005,   /* U+1EE4D */
  0x0005,   /* U+1EE4E */
  0x0005,   /* U+1EE4F */
  0x0002,   /* U+1EE50 */
  0x0005,   /* U+1EE51 */
  0x0005,   /* U+1EE52 */
  0x0002,   /* U+1EE53 */
  0x0005,   /* U+1EE54 */
  0x0002,   /* U+1EE55 */
  0x0002,   /* U+1EE56 */
  0x0005,   /* U+1EE57 */
  0x0002,   /* U+1EE58 */
  0x0005,   /* U+1EE59 */
  0x0002,   /* U+1EE5A */
  0x0005,   /* U+1EE5B */
  0x0002,   /* U+1EE5C */
  0x0005,   /* U+1EE5D */
  0x0002,   /* U+1EE5E */
  0x0005,   /* U+1EE5F */
  0x0002,   /* U+1EE60 */
  0x0005,   /* U+1EE61 */
  0x0005,   /* U+1EE62 */
  0x0002,   /* U+1EE63 */
  0x0005,   /* U+1EE64 */
  0x0002,   /* U+1EE65 */
  0x0002,   /* U+1EE66 */
  0x0005,   /* U+1EE67 */
  0x0005,   /* U+1EE68 */
  0x0005,   /* U+1EE69 */
  0x0005,   /* U+1EE6A */
  0x0002,   /* U+1EE6B */
  0x0005,   /* U+1EE6C */
  0x0005,   /* U+1EE6D */
  0x0005,   /* U+1EE6E */
  0x0005,   /* U+1EE6F */
  0x0005,   /* U+1EE70 */
  0x0005,   /* U+1EE71 */
  0x0005,   /* U+1EE72 */
  0x0002,   /* U+1EE73 */
  0x0005,   /* U+1EE74 */
  0x0005,   /* U+1EE75 */
  0x0005,   /* U+1EE76 */
  0x0005,   /* U+1EE77 */
  0x0002,   /* U+1EE78 */
  0x0005,   /* U+1EE79 */
  0x0005,   /* U+1EE7A */
  0x0005,   /* U+1EE7B */
  0x0005,   /* U+1EE7C */
  0x0002,   /* U+1EE7D */
  0x0005,   /* U+1EE7E */
  0x0002,   /* U+1EE7F */
  0x0005,   /* U+1EE80 */
  0x0005,   /* U+1EE81 */
  0x0005,   /* U+1EE82 */
  0x0005,   /* U+1EE83 */
  0x0005,   /* U+1EE84 */
  0x0005,   /* U+1EE85 */
  0x0005,   /* U+1EE86 */
  0x0005,   /* U+1EE87 */
  0x0005,   /* U+1EE88 */
  0x0005,   /* U+1EE89 */
  0x0002,   /* U+1EE8A */
  0x0005,   /* U+1EE8B */
  0x0005,   /* U+1EE8C */
  0x0005,   /* U+1EE8D */
  0x0005,   /* U+1EE8E */
  0x0005,   /* U+1EE8F */
  0x0005,   /* U+1EE90 */
  0x0005,   /* U+1EE91 */
  0x0005,   /* U+1EE92 */
  0x0005,   /* U+1EE93 */
  0x0005,   /* U+1EE94 */
  0x0005,   /* U+1EE95 */
  0x0005,   /* U+1EE96 */
  0x0005,   /* U+1EE97 */
  0x0005,   /* U+1EE98 */
  0x0005,   /* U+1EE99 */
  0x0005,   /* U+1EE9A */
  0x0005,   /* U+1EE9B */
  0x0002,   /* U+1EE9C */
  0x0002,   /* U+1EE9D */
  0x0002,   /* U+1EE9E */
  0x0002,   /* U+1EE9F */
  0x0002,   /* U+1EEA0 */
  0x0005,   /* U+1EEA1 */
  0x0005,   /* U+1EEA2 */
  0x0005,   /* U+1EEA3 */
  0x0002,   /* U+1EEA4 */
  0x0005,   /* U+1EEA5 */
  0x0005,   /* U+1EEA6 */
  0x0005,   /* U+1EEA7 */
  0x0005,   /* U+1EEA8 */
  0x0005,   /* U+1EEA9 */
  0x0002,   /* U+1EEAA */
  0x0005,   /* U+1EEAB */
  0x0005,   /* U+1EEAC */
  0x0005,   /* U+1EEAD */
  0x0005,   /* U+1EEAE */
  0x0005,   /* U+1EEAF */
  0x0005,   /* U+1EEB0 */
  0x0005,   /* U+1EEB1 */
  0x0005,   /* U+1EEB2 */
  0x0005,   /* U+1EEB3 */
  0x0005,   /* U+1EEB4 */
  0x0005,   /* U+1EEB5 */
  0x0005,   /* U+1EEB6 */
  0x0005,   /* U+1EEB7 */
  0x0005,   /* U+1EEB8 */
  0x0005,   /* U+1EEB9 */
  0x0005,   /* U+1EEBA */
  0x0005,   /* U+1EEBB */
  0x0002,   /* U+1EEBC */
  0x0002,   /* U+1EEBD */
  0x0002,   /* U+1EEBE */
  0x0002,   /* U+1EEBF */
  0x0002,   /* U+1EEC0 */
  0x0002,   /* U+1EEC1 */
  0x0002,   /* U+1EEC2 */
  0x0002,   /* U+1EEC3 */
  0x0002,   /* U+1EEC4 */
  0x0002,   /* U+1EEC5 */
  0x0002,   /* U+1EEC6 */
  0x0002,   /* U+1EEC7 */
  0x0002,   /* U+1EEC8 */
  0x0002,   /* U+1EEC9 */
  0x0002,   /* U+1EECA */
  0x0002,   /* U+1EECB */
  0x0002,   /* U+1EECC */
  0x0002,   /* U+1EECD */
  0x0002,   /* U+1EECE */
  0x0002,   /* U+1EECF */
  0x0002,   /* U+1EED0 */
  0x0002,   /* U+1EED1 */
  0x0002,   /* U+1EED2 */
  0x0002,   /* U+1EED3 */
  0x0002,   /* U+1EED4 */
  0x0002,   /* U+1EED5 */
  0x0002,   /* U+1EED6 */
  0x0002,   /* U+1EED7 */
  0x0002,   /* U+1EED8 */
  0x0002,   /* U+1EED9 */
  0x0002,   /* U+1EEDA */
  0x0002,   /* U+1EEDB */
  0x0002,   /* U+1EEDC */
  0x0002,   /* U+1EEDD */
  0x0002,   /* U+1EEDE */
  0x0002,   /* U+1EEDF */
  0x0002,   /* U+1EEE0 */
  0x0002,   /* U+1EEE1 */
  0x0002,   /* U+1EEE2 */
  0x0002,   /* U+1EEE3 */
  0x0002,   /* U+1EEE4 */
  0x0002,   /* U+1EEE5 */
  0x0002,   /* U+1EEE6 */
  0x0002,   /* U+1EEE7 */
  0x0002,   /* U+1EEE8 */
  0x0002,   /* U+1EEE9 */
  0x0002,   /* U+1EEEA */
  0x0002,   /* U+1EEEB */
  0x0002,   /* U+1EEEC */
  0x0002,   /* U+1EEED */
  0x0002,   /* U+1EEEE */
  0x0002,   /* U+1EEEF */
  0x0002,   /* U+1EEF0 */
  0x0002,   /* U+1EEF1 */
  0x0002,   /* U+1EEF2 */
  0x0002,   /* U+1EEF3 */
  0x0002,   /* U+1EEF4 */
  0x0002,   /* U+1EEF5 */
  0x0002,   /* U+1EEF6 */
  0x0002,   /* U+1EEF7 */
  0x0002,   /* U+1EEF8 */
  0x0002,   /* U+1EEF9 */
  0x0002,   /* U+1EEFA */
  0x0002,   /* U+1EEFB */
  0x0002,   /* U+1EEFC */
  0x0002,   /* U+1EEFD */
  0x0002,   /* U+1EEFE */
  0x0002,   /* U+1EEFF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1EE00 */
  0x0000,   /* U+1EE01 */
  0x0000,   /* U+1EE02 */
  0x0000,   /* U+1EE03 */
  0xEE04,   /* U+1EE04 */
  0x0000,   /* U+1EE05 */
  0x0000,   /* U+1EE06 */
  0x0000,   /* U+1EE07 */
  0x0000,   /* U+1EE08 */
  0x0000,   /* U+1EE09 */
  0x0000,   /* U+1EE0A */
  0x0000,   /* U+1EE0B */
  0x0000,   /* U+1EE0C */
  0x0000,   /* U+1EE0D */
  0x0000,   /* U+1EE0E */
  0x0000,   /* U+1EE0F */
  0x0000,   /* U+1EE10 */
  0x0000,   /* U+1EE11 */
  0x0000,   /* U+1EE12 */
  0x0000,   /* U+1EE13 */
  0x0000,   /* U+1EE14 */
  0x0000,   /* U+1EE15 */
  0x0000,   /* U+1EE16 */
  0x0000,   /* U+1EE17 */
  0x0000,   /* U+1EE18 */
  0x0000,   /* U+1EE19 */
  0x0000,   /* U+1EE1A */
  0x0000,   /* U+1EE1B */
  0x0000,   /* U+1EE1C */
  0x0000,   /* U+1EE1D */
  0x0000,   /* U+1EE1E */
  0x0000,   /* U+1EE1F */
  0xEE20,   /* U+1EE20 */
  0x0000,   /* U+1EE21 */
  0x0000,   /* U+1EE22 */
  0xEE23,   /* U+1EE23 */
  0x0000,   /* U+1EE24 */
  0xEE25,   /* U+1EE25 */
  0xEE26,   /* U+1EE26 */
  0x0000,   /* U+1EE27 */
  0xEE28,   /* U+1EE28 */
  0x0000,   /* U+1EE29 */
  0x0000,   /* U+1EE2A */
  0x0000,   /* U+1EE2B */
  0x0000,   /* U+1EE2C */
  0x0000,   /* U+1EE2D */
  0x0000,   /* U+1EE2E */
  0x0000,   /* U+1EE2F */
  0x0000,   /* U+1EE30 */
  0x0000,   /* U+1EE31 */
  0x0000,   /* U+1EE32 */
  0xEE33,   /* U+1EE33 */
  0x0000,   /* U+1EE34 */
  0x0000,   /* U+1EE35 */
  0x0000,   /* U+1EE36 */
  0x0000,   /* U+1EE37 */
  0xEE38,   /* U+1EE38 */
  0x0000,   /* U+1EE39 */
  0xEE3A,   /* U+1EE3A */
  0x0000,   /* U+1EE3B */
  0xEE3C,   /* U+1EE3C */
  0xEE3D,   /* U+1EE3D */
  0xEE3E,   /* U+1EE3E */
  0xEE3F,   /* U+1EE3F */
  0xEE40,   /* U+1EE40 */
  0xEE41,   /* U+1EE41 */
  0x0000,   /* U+1EE42 */
  0xEE43,   /* U+1EE43 */
  0xEE44,   /* U+1EE44 */
  0xEE45,   /* U+1EE45 */
  0xEE46,   /* U+1EE46 */
  0x0000,   /* U+1EE47 */
  0xEE48,   /* U+1EE48 */
  0x0000,   /* U+1EE49 */
  0xEE4A,   /* U+1EE4A */
  0x0000,   /* U+1EE4B */
  0xEE4C,   /* U+1EE4C */
  0x0000,   /* U+1EE4D */
  0x0000,   /* U+1EE4E */
  0x0000,   /* U+1EE4F */
  0xEE50,   /* U+1EE50 */
  0x0000,   /* U+1EE51 */
  0x0000,   /* U+1EE52 */
  0xEE53,   /* U+1EE53 */
  0x0000,   /* U+1EE54 */
  0xEE55,   /* U+1EE55 */
  0xEE56,   /* U+1EE56 */
  0x0000,   /* U+1EE57 */
  0xEE58,   /* U+1EE58 */
  0x0000,   /* U+1EE59 */
  0xEE5A,   /* U+1EE5A */
  0x0000,   /* U+1EE5B */
  0xEE5C,   /* U+1EE5C */
  0x0000,   /* U+1EE5D */
  0xEE5E,   /* U+1EE5E */
  0x0000,   /* U+1EE5F */
  0xEE60,   /* U+1EE60 */
  0x0000,   /* U+1EE61 */
  0x0000,   /* U+1EE62 */
  0xEE63,   /* U+1EE63 */
  0x0000,   /* U+1EE64 */
  0xEE65,   /* U+1EE65 */
  0xEE66,   /* U+1EE66 */
  0x0000,   /* U+1EE67 */
  0x0000,   /* U+1EE68 */
  0x0000,   /* U+1EE69 */
  0x0000,   /* U+1EE6A */
  0xEE6B,   /* U+1EE6B */
  0x0000,   /* U+1EE6C */
  0x0000,   /* U+1EE6D */
  0x0000,   /* U+1EE6E */
  0x0000,   /* U+1EE6F */
  0x0000,   /* U+1EE70 */
  0x0000,   /* U+1EE71 */
  0x0000,   /* U+1EE72 */
  0xEE73,   /* U+1EE73 */
  0x0000,   /* U+1EE74 */
  0x0000,   /* U+1EE75 */
  0x0000,   /* U+1EE76 */
  0x0000,   /* U+1EE77 */
  0xEE78,   /* U+1EE78 */
  0x0000,   /* U+1EE79 */
  0x0000,   /* U+1EE7A */
  0x0000,   /* U+1EE7B */
  0x0000,   /* U+1EE7C */
  0xEE7D,   /* U+1EE7D */
  0x0000,   /* U+1EE7E */
  0xEE7F,   /* U+1EE7F */
  0x0000,   /* U+1EE80 */
  0x0000,   /* U+1EE81 */
  0x0000,   /* U+1EE82 */
  0x0000,   /* U+1EE83 */
  0x0000,   /* U+1EE84 */
  0x0000,   /* U+1EE85 */
  0x0000,   /* U+1EE86 */
  0x0000,   /* U+1EE87 */
  0x0000,   /* U+1EE88 */
  0x0000,   /* U+1EE89 */
  0xEE8A,   /* U+1EE8A */
  0x0000,   /* U+1EE8B */
  0x0000,   /* U+1EE8C */
  0x0000,   /* U+1EE8D */
  0x0000,   /* U+1EE8E */
  0x0000,   /* U+1EE8F */
  0x0000,   /* U+1EE90 */
  0x0000,   /* U+1EE91 */
  0x0000,   /* U+1EE92 */
  0x0000,   /* U+1EE93 */
  0x0000,   /* U+1EE94 */
  0x0000,   /* U+1EE95 */
  0x0000,   /* U+1EE96 */
  0x0000,   /* U+1EE97 */
  0x0000,   /* U+1EE98 */
  0x0000,   /* U+1EE99 */
  0x0000,   /* U+1EE9A */
  0x0000,   /* U+1EE9B */
  0xEE9C,   /* U+1EE9C */
  0xEE9D,   /* U+1EE9D */
  0xEE9E,   /* U+1EE9E */
  0xEE9F,   /* U+1EE9F */
  0xEEA0,   /* U+1EEA0 */
  0x0000,   /* U+1EEA1 */
  0x0000,   /* U+1EEA2 */
  0x0000,   /* U+1EEA3 */
  0xEEA4,   /* U+1EEA4 */
  0x0000,   /* U+1EEA5 */
  0x0000,   /* U+1EEA6 */
  0x0000,   /* U+1EEA7 */
  0x0000,   /* U+1EEA8 */
  0x0000,   /* U+1EEA9 */
  0xEEAA,   /* U+1EEAA */
  0x0000,   /* U+1EEAB */
  0x0000,   /* U+1EEAC */
  0x0000,   /* U+1EEAD */
  0x0000,   /* U+1EEAE */
  0x0000,   /* U+1EEAF */
  0x0000,   /* U+1EEB0 */
  0x0000,   /* U+1EEB1 */
  0x0000,   /* U+1EEB2 */
  0x0000,   /* U+1EEB3 */
  0x0000,   /* U+1EEB4 */
  0x0000,   /* U+1EEB5 */
  0x0000,   /* U+1EEB6 */
  0x0000,   /* U+1EEB7 */
  0x0000,   /* U+1EEB8 */
  0x0000,   /* U+1EEB9 */
  0x0000,   /* U+1EEBA */
  0x0000,   /* U+1EEBB */
  0xEEBC,   /* U+1EEBC */
  0xEEBD,   /* U+1EEBD */
  0xEEBE,   /* U+1EEBE */
  0xEEBF,   /* U+1EEBF */
  0xEEC0,   /* U+1EEC0 */
  0xEEC1,   /* U+1EEC1 */
  0xEEC2,   /* U+1EEC2 */
  0xEEC3,   /* U+1EEC3 */
  0xEEC4,   /* U+1EEC4 */
  0xEEC5,   /* U+1EEC5 */
  0xEEC6,   /* U+1EEC6 */
  0xEEC7,   /* U+1EEC7 */
  0xEEC8,   /* U+1EEC8 */
  0xEEC9,   /* U+1EEC9 */
  0xEECA,   /* U+1EECA */
  0xEECB,   /* U+1EECB */
  0xEECC,   /* U+1EECC */
  0xEECD,   /* U+1EECD */
  0xEECE,   /* U+1EECE */
  0xEECF,   /* U+1EECF */
  0xEED0,   /* U+1EED0 */
  0xEED1,   /* U+1EED1 */
  0xEED2,   /* U+1EED2 */
  0xEED3,   /* U+1EED3 */
  0xEED4,   /* U+1EED4 */
  0xEED5,   /* U+1EED5 */
  0xEED6,   /* U+1EED6 */
  0xEED7,   /* U+1EED7 */
  0xEED8,   /* U+1EED8 */
  0xEED9,   /* U+1EED9 */
  0xEEDA,   /* U+1EEDA */
  0xEEDB,   /* U+1EEDB */
  0xEEDC,   /* U+1EEDC */
  0xEEDD,   /* U+1EEDD */
  0xEEDE,   /* U+1EEDE */
  0xEEDF,   /* U+1EEDF */
  0xEEE0,   /* U+1EEE0 */
  0xEEE1,   /* U+1EEE1 */
  0xEEE2,   /* U+1EEE2 */
  0xEEE3,   /* U+1EEE3 */
  0xEEE4,   /* U+1EEE4 */
  0xEEE5,   /* U+1EEE5 */
  0xEEE6,   /* U+1EEE6 */
  0xEEE7,   /* U+1EEE7 */
  0xEEE8,   /* U+1EEE8 */
  0xEEE9,   /* U+1EEE9 */
  0xEEEA,   /* U+1EEEA */
  0xEEEB,   /* U+1EEEB */
  0xEEEC,   /* U+1EEEC */
  0xEEED,   /* U+1EEED */
  0xEEEE,   /* U+1EEEE */
  0xEEEF,   /* U+1EEEF */
  0x0000,   /* U+1EEF0 */
  0x0000,   /* U+1EEF1 */
  0xEEF2,   /* U+1EEF2 */
  0xEEF3,   /* U+1EEF3 */
  0xEEF4,   /* U+1EEF4 */
  0xEEF5,   /* U+1EEF5 */
  0xEEF6,   /* U+1EEF6 */
  0xEEF7,   /* U+1EEF7 */
  0xEEF8,   /* U+1EEF8 */
  0xEEF9,   /* U+1EEF9 */
  0xEEFA,   /* U+1EEFA */
  0xEEFB,   /* U+1EEFB */
  0xEEFC,   /* U+1EEFC */
  0xEEFD,   /* U+1EEFD */
  0xEEFE,   /* U+1EEFE */
  0xEEFF,   /* U+1EEFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1EE00 */
  0x0000,   /* U+1EE01 */
  0x0000,   /* U+1EE02 */
  0x0000,   /* U+1EE03 */
  0x0000,   /* U+1EE04 */
  0x0000,   /* U+1EE05 */
  0x0000,   /* U+1EE06 */
  0x0000,   /* U+1EE07 */
  0x0000,   /* U+1EE08 */
  0x0000,   /* U+1EE09 */
  0x0000,   /* U+1EE0A */
  0x0000,   /* U+1EE0B */
  0x0000,   /* U+1EE0C */
  0x0000,   /* U+1EE0D */
  0x0000,   /* U+1EE0E */
  0x0000,   /* U+1EE0F */
  0x0000,   /* U+1EE10 */
  0x0000,   /* U+1EE11 */
  0x0000,   /* U+1EE12 */
  0x0000,   /* U+1EE13 */
  0x0000,   /* U+1EE14 */
  0x0000,   /* U+1EE15 */
  0x0000,   /* U+1EE16 */
  0x0000,   /* U+1EE17 */
  0x0000,   /* U+1EE18 */
  0x0000,   /* U+1EE19 */
  0x0000,   /* U+1EE1A */
  0x0000,   /* U+1EE1B */
  0x0000,   /* U+1EE1C */
  0x0000,   /* U+1EE1D */
  0x0000,   /* U+1EE1E */
  0x0000,   /* U+1EE1F */
  0x0000,   /* U+1EE20 */
  0x0000,   /* U+1EE21 */
  0x0000,   /* U+1EE22 */
  0x0000,   /* U+1EE23 */
  0x0000,   /* U+1EE24 */
  0x0000,   /* U+1EE25 */
  0x0000,   /* U+1EE26 */
  0x0000,   /* U+1EE27 */
  0x0000,   /* U+1EE28 */
  0x0000,   /* U+1EE29 */
  0x0000,   /* U+1EE2A */
  0x0000,   /* U+1EE2B */
  0x0000,   /* U+1EE2C */
  0x0000,   /* U+1EE2D */
  0x0000,   /* U+1EE2E */
  0x0000,   /* U+1EE2F */
  0x0000,   /* U+1EE30 */
  0x0000,   /* U+1EE31 */
  0x0000,   /* U+1EE32 */
  0x0000,   /* U+1EE33 */
  0x0000,   /* U+1EE34 */
  0x0000,   /* U+1EE35 */
  0x0000,   /* U+1EE36 */
  0x0000,   /* U+1EE37 */
  0x0000,   /* U+1EE38 */
  0x0000,   /* U+1EE39 */
  0x0000,   /* U+1EE3A */
  0x0000,   /* U+1EE3B */
  0x0000,   /* U+1EE3C */
  0x0000,   /* U+1EE3D */
  0x0000,   /* U+1EE3E */
  0x0000,   /* U+1EE3F */
  0x0000,   /* U+1EE40 */
  0x0000,   /* U+1EE41 */
  0x0000,   /* U+1EE42 */
  0x0000,   /* U+1EE43 */
  0x0000,   /* U+1EE44 */
  0x0000,   /* U+1EE45 */
  0x0000,   /* U+1EE46 */
  0x0000,   /* U+1EE47 */
  0x0000,   /* U+1EE48 */
  0x0000,   /* U+1EE49 */
  0x0000,   /* U+1EE4A */
  0x0000,   /* U+1EE4B */
  0x0000,   /* U+1EE4C */
  0x0000,   /* U+1EE4D */
  0x0000,   /* U+1EE4E */
  0x0000,   /* U+1EE4F */
  0x0000,   /* U+1EE50 */
  0x0000,   /* U+1EE51 */
  0x0000,   /* U+1EE52 */
  0x0000,   /* U+1EE53 */
  0x0000,   /* U+1EE54 */
  0x0000,   /* U+1EE55 */
  0x0000,   /* U+1EE56 */
  0x0000,   /* U+1EE57 */
  0x0000,   /* U+1EE58 */
  0x0000,   /* U+1EE59 */
  0x0000,   /* U+1EE5A */
  0x0000,   /* U+1EE5B */
  0x0000,   /* U+1EE5C */
  0x0000,   /* U+1EE5D */
  0x0000,   /* U+1EE5E */
  0x0000,   /* U+1EE5F */
  0x0000,   /* U+1EE60 */
  0x0000,   /* U+1EE61 */
  0x0000,   /* U+1EE62 */
  0x0000,   /* U+1EE63 */
  0x0000,   /* U+1EE64 */
  0x0000,   /* U+1EE65 */
  0x0000,   /* U+1EE66 */
  0x0000,   /* U+1EE67 */
  0x0000,   /* U+1EE68 */
  0x0000,   /* U+1EE69 */
  0x0000,   /* U+1EE6A */
  0x0000,   /* U+1EE6B */
  0x0000,   /* U+1EE6C */
  0x0000,   /* U+1EE6D */
  0x0000,   /* U+1EE6E */
  0x0000,   /* U+1EE6F */
  0x0000,   /* U+1EE70 */
  0x0000,   /* U+1EE71 */
  0x0000,   /* U+1EE72 */
  0x0000,   /* U+1EE73 */
  0x0000,   /* U+1EE74 */
  0x0000,   /* U+1EE75 */
  0x0000,   /* U+1EE76 */
  0x0000,   /* U+1EE77 */
  0x0000,   /* U+1EE78 */
  0x0000,   /* U+1EE79 */
  0x0000,   /* U+1EE7A */
  0x0000,   /* U+1EE7B */
  0x0000,   /* U+1EE7C */
  0x0000,   /* U+1EE7D */
  0x0000,   /* U+1EE7E */
  0x0000,   /* U+1EE7F */
  0x0000,   /* U+1EE80 */
  0x0000,   /* U+1EE81 */
  0x0000,   /* U+1EE82 */
  0x0000,   /* U+1EE83 */
  0x0000,   /* U+1EE84 */
  0x0000,   /* U+1EE85 */
  0x0000,   /* U+1EE86 */
  0x0000,   /* U+1EE87 */
  0x0000,   /* U+1EE88 */
  0x0000,   /* U+1EE89 */
  0x0000,   /* U+1EE8A */
  0x0000,   /* U+1EE8B */
  0x0000,   /* U+1EE8C */
  0x0000,   /* U+1EE8D */
  0x0000,   /* U+1EE8E */
  0x0000,   /* U+1EE8F */
  0x0000,   /* U+1EE90 */
  0x0000,   /* U+1EE91 */
  0x0000,   /* U+1EE92 */
  0x0000,   /* U+1EE93 */
  0x0000,   /* U+1EE94 */
  0x0000,   /* U+1EE95 */
  0x0000,   /* U+1EE96 */
  0x0000,   /* U+1EE97 */
  0x0000,   /* U+1EE98 */
  0x0000,   /* U+1EE99 */
  0x0000,   /* U+1EE9A */
  0x0000,   /* U+1EE9B */
  0x0000,   /* U+1EE9C */
  0x0000,   /* U+1EE9D */
  0x0000,   /* U+1EE9E */
  0x0000,   /* U+1EE9F */
  0x0000,   /* U+1EEA0 */
  0x0000,   /* U+1EEA1 */
  0x0000,   /* U+1EEA2 */
  0x0000,   /* U+1EEA3 */
  0x0000,   /* U+1EEA4 */
  0x0000,   /* U+1EEA5 */
  0x0000,   /* U+1EEA6 */
  0x0000,   /* U+1EEA7 */
  0x0000,   /* U+1EEA8 */
  0x0000,   /* U+1EEA9 */
  0x0000,   /* U+1EEAA */
  0x0000,   /* U+1EEAB */
  0x0000,   /* U+1EEAC */
  0x0000,   /* U+1EEAD */
  0x0000,   /* U+1EEAE */
  0x0000,   /* U+1EEAF */
  0x0000,   /* U+1EEB0 */
  0x0000,   /* U+1EEB1 */
  0x0000,   /* U+1EEB2 */
  0x0000,   /* U+1EEB3 */
  0x0000,   /* U+1EEB4 */
  0x0000,   /* U+1EEB5 */
  0x0000,   /* U+1EEB6 */
  0x0000,   /* U+1EEB7 */
  0x0000,   /* U+1EEB8 */
  0x0000,   /* U+1EEB9 */
  0x0000,   /* U+1EEBA */
  0x0000,   /* U+1EEBB */
  0x0000,   /* U+1EEBC */
  0x0000,   /* U+1EEBD */
  0x0000,   /* U+1EEBE */
  0x0000,   /* U+1EEBF */
  0x0000,   /* U+1EEC0 */
  0x0000,   /* U+1EEC1 */
  0x0000,   /* U+1EEC2 */
  0x0000,   /* U+1EEC3 */
  0x0000,   /* U+1EEC4 */
  0x0000,   /* U+1EEC5 */
  0x0000,   /* U+1EEC6 */
  0x0000,   /* U+1EEC7 */
  0x0000,   /* U+1EEC8 */
  0x0000,   /* U+1EEC9 */
  0x0000,   /* U+1EECA */
  0x0000,   /* U+1EECB */
  0x0000,   /* U+1EECC */
  0x0000,   /* U+1EECD */
  0x0000,   /* U+1EECE */
  0x0000,   /* U+1EECF */
  0x0000,   /* U+1EED0 */
  0x0000,   /* U+1EED1 */
  0x0000,   /* U+1EED2 */
  0x0000,   /* U+1EED3 */
  0x0000,   /* U+1EED4 */
  0x0000,   /* U+1EED5 */
  0x0000,   /* U+1EED6 */
  0x0000,   /* U+1EED7 */
  0x0000,   /* U+1EED8 */
  0x0000,   /* U+1EED9 */
  0x0000,   /* U+1EEDA */
  0x0000,   /* U+1EEDB */
  0x0000,   /* U+1EEDC */
  0x0000,   /* U+1EEDD */
  0x0000,   /* U+1EEDE */
  0x0000,   /* U+1EEDF */
  0x0000,   /* U+1EEE0 */
  0x0000,   /* U+1EEE1 */
  0x0000,   /* U+1EEE2 */
  0x0000,   /* U+1EEE3 */
  0x0000,   /* U+1EEE4 */
  0x0000,   /* U+1EEE5 */
  0x0000,   /* U+1EEE6 */
  0x0000,   /* U+1EEE7 */
  0x0000,   /* U+1EEE8 */
  0x0000,   /* U+1EEE9 */
  0x0000,   /* U+1EEEA */
  0x0000,   /* U+1EEEB */
  0x0000,   /* U+1EEEC */
  0x0000,   /* U+1EEED */
  0x0000,   /* U+1EEEE */
  0x0000,   /* U+1EEEF */
  0x0000,   /* U+1EEF0 */
  0x0000,   /* U+1EEF1 */
  0x0000,   /* U+1EEF2 */
  0x0000,   /* U+1EEF3 */
  0x0000,   /* U+1EEF4 */
  0x0000,   /* U+1EEF5 */
  0x0000,   /* U+1EEF6 */
  0x0000,   /* U+1EEF7 */
  0x0000,   /* U+1EEF8 */
  0x0000,   /* U+1EEF9 */
  0x0000,   /* U+1EEFA */
  0x0000,   /* U+1EEFB */
  0x0000,   /* U+1EEFC */
  0x0000,   /* U+1EEFD */
  0x0000,   /* U+1EEFE */
  0x0000,   /* U+1EEFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1EE00 */
  0x0000,   /* U+1EE01 */
  0x0000,   /* U+1EE02 */
  0x0000,   /* U+1EE03 */
  0x0000,   /* U+1EE04 */
  0x0000,   /* U+1EE05 */
  0x0000,   /* U+1EE06 */
  0x0000,   /* U+1EE07 */
  0x0000,   /* U+1EE08 */
  0x0000,   /* U+1EE09 */
  0x0000,   /* U+1EE0A */
  0x0000,   /* U+1EE0B */
  0x0000,   /* U+1EE0C */
  0x0000,   /* U+1EE0D */
  0x0000,   /* U+1EE0E */
  0x0000,   /* U+1EE0F */
  0x0000,   /* U+1EE10 */
  0x0000,   /* U+1EE11 */
  0x0000,   /* U+1EE12 */
  0x0000,   /* U+1EE13 */
  0x0000,   /* U+1EE14 */
  0x0000,   /* U+1EE15 */
  0x0000,   /* U+1EE16 */
  0x0000,   /* U+1EE17 */
  0x0000,   /* U+1EE18 */
  0x0000,   /* U+1EE19 */
  0x0000,   /* U+1EE1A */
  0x0000,   /* U+1EE1B */
  0x0000,   /* U+1EE1C */
  0x0000,   /* U+1EE1D */
  0x0000,   /* U+1EE1E */
  0x0000,   /* U+1EE1F */
  0x0000,   /* U+1EE20 */
  0x0000,   /* U+1EE21 */
  0x0000,   /* U+1EE22 */
  0x0000,   /* U+1EE23 */
  0x0000,   /* U+1EE24 */
  0x0000,   /* U+1EE25 */
  0x0000,   /* U+1EE26 */
  0x0000,   /* U+1EE27 */
  0x0000,   /* U+1EE28 */
  0x0000,   /* U+1EE29 */
  0x0000,   /* U+1EE2A */
  0x0000,   /* U+1EE2B */
  0x0000,   /* U+1EE2C */
  0x0000,   /* U+1EE2D */
  0x0000,   /* U+1EE2E */
  0x0000,   /* U+1EE2F */
  0x0000,   /* U+1EE30 */
  0x0000,   /* U+1EE31 */
  0x0000,   /* U+1EE32 */
  0x0000,   /* U+1EE33 */
  0x0000,   /* U+1EE34 */
  0x0000,   /* U+1EE35 */
  0x0000,   /* U+1EE36 */
  0x0000,   /* U+1EE37 */
  0x0000,   /* U+1EE38 */
  0x0000,   /* U+1EE39 */
  0x0000,   /* U+1EE3A */
  0x0000,   /* U+1EE3B */
  0x0000,   /* U+1EE3C */
  0x0000,   /* U+1EE3D */
  0x0000,   /* U+1EE3E */
  0x0000,   /* U+1EE3F */
  0x0000,   /* U+1EE40 */
  0x0000,   /* U+1EE41 */
  0x0000,   /* U+1EE42 */
  0x0000,   /* U+1EE43 */
  0x0000,   /* U+1EE44 */
  0x0000,   /* U+1EE45 */
  0x0000,   /* U+1EE46 */
  0x0000,   /* U+1EE47 */
  0x0000,   /* U+1EE48 */
  0x0000,   /* U+1EE49 */
  0x0000,   /* U+1EE4A */
  0x0000,   /* U+1EE4B */
  0x0000,   /* U+1EE4C */
  0x0000,   /* U+1EE4D */
  0x0000,   /* U+1EE4E */
  0x0000,   /* U+1EE4F */
  0x0000,   /* U+1EE50 */
  0x0000,   /* U+1EE51 */
  0x0000,   /* U+1EE52 */
  0x0000,   /* U+1EE53 */
  0x0000,   /* U+1EE54 */
  0x0000,   /* U+1EE55 */
  0x0000,   /* U+1EE56 */
  0x0000,   /* U+1EE57 */
  0x0000,   /* U+1EE58 */
  0x0000,   /* U+1EE59 */
  0x0000,   /* U+1EE5A */
  0x0000,   /* U+1EE5B */
  0x0000,   /* U+1EE5C */
  0x0000,   /* U+1EE5D */
  0x0000,   /* U+1EE5E */
  0x0000,   /* U+1EE5F */
  0x0000,   /* U+1EE60 */
  0x0000,   /* U+1EE61 */
  0x0000,   /* U+1EE62 */
  0x0000,   /* U+1EE63 */
  0x0000,   /* U+1EE64 */
  0x0000,   /* U+1EE65 */
  0x0000,   /* U+1EE66 */
  0x0000,   /* U+1EE67 */
  0x0000,   /* U+1EE68 */
  0x0000,   /* U+1EE69 */
  0x0000,   /* U+1EE6A */
  0x0000,   /* U+1EE6B */
  0x0000,   /* U+1EE6C */
  0x0000,   /* U+1EE6D */
  0x0000,   /* U+1EE6E */
  0x0000,   /* U+1EE6F */
  0x0000,   /* U+1EE70 */
  0x0000,   /* U+1EE71 */
  0x0000,   /* U+1EE72 */
  0x0000,   /* U+1EE73 */
  0x0000,   /* U+1EE74 */
  0x0000,   /* U+1EE75 */
  0x0000,   /* U+1EE76 */
  0x0000,   /* U+1EE77 */
  0x0000,   /* U+1EE78 */
  0x0000,   /* U+1EE79 */
  0x0000,   /* U+1EE7A */
  0x0000,   /* U+1EE7B */
  0x0000,   /* U+1EE7C */
  0x0000,   /* U+1EE7D */
  0x0000,   /* U+1EE7E */
  0x0000,   /* U+1EE7F */
  0x0000,   /* U+1EE80 */
  0x0000,   /* U+1EE81 */
  0x0000,   /* U+1EE82 */
  0x0000,   /* U+1EE83 */
  0x0000,   /* U+1EE84 */
  0x0000,   /* U+1EE85 */
  0x0000,   /* U+1EE86 */
  0x0000,   /* U+1EE87 */
  0x0000,   /* U+1EE88 */
  0x0000,   /* U+1EE89 */
  0x0000,   /* U+1EE8A */
  0x0000,   /* U+1EE8B */
  0x0000,   /* U+1EE8C */
  0x0000,   /* U+1EE8D */
  0x0000,   /* U+1EE8E */
  0x0000,   /* U+1EE8F */
  0x0000,   /* U+1EE90 */
  0x0000,   /* U+1EE91 */
  0x0000,   /* U+1EE92 */
  0x0000,   /* U+1EE93 */
  0x0000,   /* U+1EE94 */
  0x0000,   /* U+1EE95 */
  0x0000,   /* U+1EE96 */
  0x0000,   /* U+1EE97 */
  0x0000,   /* U+1EE98 */
  0x0000,   /* U+1EE99 */
  0x0000,   /* U+1EE9A */
  0x0000,   /* U+1EE9B */
  0x0000,   /* U+1EE9C */
  0x0000,   /* U+1EE9D */
  0x0000,   /* U+1EE9E */
  0x0000,   /* U+1EE9F */
  0x0000,   /* U+1EEA0 */
  0x0000,   /* U+1EEA1 */
  0x0000,   /* U+1EEA2 */
  0x0000,   /* U+1EEA3 */
  0x0000,   /* U+1EEA4 */
  0x0000,   /* U+1EEA5 */
  0x0000,   /* U+1EEA6 */
  0x0000,   /* U+1EEA7 */
  0x0000,   /* U+1EEA8 */
  0x0000,   /* U+1EEA9 */
  0x0000,   /* U+1EEAA */
  0x0000,   /* U+1EEAB */
  0x0000,   /* U+1EEAC */
  0x0000,   /* U+1EEAD */
  0x0000,   /* U+1EEAE */
  0x0000,   /* U+1EEAF */
  0x0000,   /* U+1EEB0 */
  0x0000,   /* U+1EEB1 */
  0x0000,   /* U+1EEB2 */
  0x0000,   /* U+1EEB3 */
  0x0000,   /* U+1EEB4 */
  0x0000,   /* U+1EEB5 */
  0x0000,   /* U+1EEB6 */
  0x0000,   /* U+1EEB7 */
  0x0000,   /* U+1EEB8 */
  0x0000,   /* U+1EEB9 */
  0x0000,   /* U+1EEBA */
  0x0000,   /* U+1EEBB */
  0x0000,   /* U+1EEBC */
  0x0000,   /* U+1EEBD */
  0x0000,   /* U+1EEBE */
  0x0000,   /* U+1EEBF */
  0x0000,   /* U+1EEC0 */
  0x0000,   /* U+1EEC1 */
  0x0000,   /* U+1EEC2 */
  0x0000,   /* U+1EEC3 */
  0x0000,   /* U+1EEC4 */
  0x0000,   /* U+1EEC5 */
  0x0000,   /* U+1EEC6 */
  0x0000,   /* U+1EEC7 */
  0x0000,   /* U+1EEC8 */
  0x0000,   /* U+1EEC9 */
  0x0000,   /* U+1EECA */
  0x0000,   /* U+1EECB */
  0x0000,   /* U+1EECC */
  0x0000,   /* U+1EECD */
  0x0000,   /* U+1EECE */
  0x0000,   /* U+1EECF */
  0x0000,   /* U+1EED0 */
  0x0000,   /* U+1EED1 */
  0x0000,   /* U+1EED2 */
  0x0000,   /* U+1EED3 */
  0x0000,   /* U+1EED4 */
  0x0000,   /* U+1EED5 */
  0x0000,   /* U+1EED6 */
  0x0000,   /* U+1EED7 */
  0x0000,   /* U+1EED8 */
  0x0000,   /* U+1EED9 */
  0x0000,   /* U+1EEDA */
  0x0000,   /* U+1EEDB */
  0x0000,   /* U+1EEDC */
  0x0000,   /* U+1EEDD */
  0x0000,   /* U+1EEDE */
  0x0000,   /* U+1EEDF */
  0x0000,   /* U+1EEE0 */
  0x0000,   /* U+1EEE1 */
  0x0000,   /* U+1EEE2 */
  0x0000,   /* U+1EEE3 */
  0x0000,   /* U+1EEE4 */
  0x0000,   /* U+1EEE5 */
  0x0000,   /* U+1EEE6 */
  0x0000,   /* U+1EEE7 */
  0x0000,   /* U+1EEE8 */
  0x0000,   /* U+1EEE9 */
  0x0000,   /* U+1EEEA */
  0x0000,   /* U+1EEEB */
  0x0000,   /* U+1EEEC */
  0x0000,   /* U+1EEED */
  0x0000,   /* U+1EEEE */
  0x0000,   /* U+1EEEF */
  0x0000,   /* U+1EEF0 */
  0x0000,   /* U+1EEF1 */
  0x0000,   /* U+1EEF2 */
  0x0000,   /* U+1EEF3 */
  0x0000,   /* U+1EEF4 */
  0x0000,   /* U+1EEF5 */
  0x0000,   /* U+1EEF6 */
  0x0000,   /* U+1EEF7 */
  0x0000,   /* U+1EEF8 */
  0x0000,   /* U+1EEF9 */
  0x0000,   /* U+1EEFA */
  0x0000,   /* U+1EEFB */
  0x0000,   /* U+1EEFC */
  0x0000,   /* U+1EEFD */
  0x0000,   /* U+1EEFE */
  0x0000,   /* U+1EEFF */
};

uint16 uca900_p1F0[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x11C7,   /* U+1F000 */
  0x11C8,   /* U+1F001 */
  0x11C9,   /* U+1F002 */
  0x11CA,   /* U+1F003 */
  0x11CB,   /* U+1F004 */
  0x11CC,   /* U+1F005 */
  0x11CD,   /* U+1F006 */
  0x11CE,   /* U+1F007 */
  0x11CF,   /* U+1F008 */
  0x11D0,   /* U+1F009 */
  0x11D1,   /* U+1F00A */
  0x11D2,   /* U+1F00B */
  0x11D3,   /* U+1F00C */
  0x11D4,   /* U+1F00D */
  0x11D5,   /* U+1F00E */
  0x11D6,   /* U+1F00F */
  0x11D7,   /* U+1F010 */
  0x11D8,   /* U+1F011 */
  0x11D9,   /* U+1F012 */
  0x11DA,   /* U+1F013 */
  0x11DB,   /* U+1F014 */
  0x11DC,   /* U+1F015 */
  0x11DD,   /* U+1F016 */
  0x11DE,   /* U+1F017 */
  0x11DF,   /* U+1F018 */
  0x11E0,   /* U+1F019 */
  0x11E1,   /* U+1F01A */
  0x11E2,   /* U+1F01B */
  0x11E3,   /* U+1F01C */
  0x11E4,   /* U+1F01D */
  0x11E5,   /* U+1F01E */
  0x11E6,   /* U+1F01F */
  0x11E7,   /* U+1F020 */
  0x11E8,   /* U+1F021 */
  0x11E9,   /* U+1F022 */
  0x11EA,   /* U+1F023 */
  0x11EB,   /* U+1F024 */
  0x11EC,   /* U+1F025 */
  0x11ED,   /* U+1F026 */
  0x11EE,   /* U+1F027 */
  0x11EF,   /* U+1F028 */
  0x11F0,   /* U+1F029 */
  0x11F1,   /* U+1F02A */
  0x11F2,   /* U+1F02B */
  0xFBC3,   /* U+1F02C */
  0xFBC3,   /* U+1F02D */
  0xFBC3,   /* U+1F02E */
  0xFBC3,   /* U+1F02F */
  0x11F3,   /* U+1F030 */
  0x11F4,   /* U+1F031 */
  0x11F5,   /* U+1F032 */
  0x11F6,   /* U+1F033 */
  0x11F7,   /* U+1F034 */
  0x11F8,   /* U+1F035 */
  0x11F9,   /* U+1F036 */
  0x11FA,   /* U+1F037 */
  0x11FB,   /* U+1F038 */
  0x11FC,   /* U+1F039 */
  0x11FD,   /* U+1F03A */
  0x11FE,   /* U+1F03B */
  0x11FF,   /* U+1F03C */
  0x1200,   /* U+1F03D */
  0x1201,   /* U+1F03E */
  0x1202,   /* U+1F03F */
  0x1203,   /* U+1F040 */
  0x1204,   /* U+1F041 */
  0x1205,   /* U+1F042 */
  0x1206,   /* U+1F043 */
  0x1207,   /* U+1F044 */
  0x1208,   /* U+1F045 */
  0x1209,   /* U+1F046 */
  0x120A,   /* U+1F047 */
  0x120B,   /* U+1F048 */
  0x120C,   /* U+1F049 */
  0x120D,   /* U+1F04A */
  0x120E,   /* U+1F04B */
  0x120F,   /* U+1F04C */
  0x1210,   /* U+1F04D */
  0x1211,   /* U+1F04E */
  0x1212,   /* U+1F04F */
  0x1213,   /* U+1F050 */
  0x1214,   /* U+1F051 */
  0x1215,   /* U+1F052 */
  0x1216,   /* U+1F053 */
  0x1217,   /* U+1F054 */
  0x1218,   /* U+1F055 */
  0x1219,   /* U+1F056 */
  0x121A,   /* U+1F057 */
  0x121B,   /* U+1F058 */
  0x121C,   /* U+1F059 */
  0x121D,   /* U+1F05A */
  0x121E,   /* U+1F05B */
  0x121F,   /* U+1F05C */
  0x1220,   /* U+1F05D */
  0x1221,   /* U+1F05E */
  0x1222,   /* U+1F05F */
  0x1223,   /* U+1F060 */
  0x1224,   /* U+1F061 */
  0x1225,   /* U+1F062 */
  0x1226,   /* U+1F063 */
  0x1227,   /* U+1F064 */
  0x1228,   /* U+1F065 */
  0x1229,   /* U+1F066 */
  0x122A,   /* U+1F067 */
  0x122B,   /* U+1F068 */
  0x122C,   /* U+1F069 */
  0x122D,   /* U+1F06A */
  0x122E,   /* U+1F06B */
  0x122F,   /* U+1F06C */
  0x1230,   /* U+1F06D */
  0x1231,   /* U+1F06E */
  0x1232,   /* U+1F06F */
  0x1233,   /* U+1F070 */
  0x1234,   /* U+1F071 */
  0x1235,   /* U+1F072 */
  0x1236,   /* U+1F073 */
  0x1237,   /* U+1F074 */
  0x1238,   /* U+1F075 */
  0x1239,   /* U+1F076 */
  0x123A,   /* U+1F077 */
  0x123B,   /* U+1F078 */
  0x123C,   /* U+1F079 */
  0x123D,   /* U+1F07A */
  0x123E,   /* U+1F07B */
  0x123F,   /* U+1F07C */
  0x1240,   /* U+1F07D */
  0x1241,   /* U+1F07E */
  0x1242,   /* U+1F07F */
  0x1243,   /* U+1F080 */
  0x1244,   /* U+1F081 */
  0x1245,   /* U+1F082 */
  0x1246,   /* U+1F083 */
  0x1247,   /* U+1F084 */
  0x1248,   /* U+1F085 */
  0x1249,   /* U+1F086 */
  0x124A,   /* U+1F087 */
  0x124B,   /* U+1F088 */
  0x124C,   /* U+1F089 */
  0x124D,   /* U+1F08A */
  0x124E,   /* U+1F08B */
  0x124F,   /* U+1F08C */
  0x1250,   /* U+1F08D */
  0x1251,   /* U+1F08E */
  0x1252,   /* U+1F08F */
  0x1253,   /* U+1F090 */
  0x1254,   /* U+1F091 */
  0x1255,   /* U+1F092 */
  0x1256,   /* U+1F093 */
  0xFBC3,   /* U+1F094 */
  0xFBC3,   /* U+1F095 */
  0xFBC3,   /* U+1F096 */
  0xFBC3,   /* U+1F097 */
  0xFBC3,   /* U+1F098 */
  0xFBC3,   /* U+1F099 */
  0xFBC3,   /* U+1F09A */
  0xFBC3,   /* U+1F09B */
  0xFBC3,   /* U+1F09C */
  0xFBC3,   /* U+1F09D */
  0xFBC3,   /* U+1F09E */
  0xFBC3,   /* U+1F09F */
  0x1257,   /* U+1F0A0 */
  0x1258,   /* U+1F0A1 */
  0x1259,   /* U+1F0A2 */
  0x125A,   /* U+1F0A3 */
  0x125B,   /* U+1F0A4 */
  0x125C,   /* U+1F0A5 */
  0x125D,   /* U+1F0A6 */
  0x125E,   /* U+1F0A7 */
  0x125F,   /* U+1F0A8 */
  0x1260,   /* U+1F0A9 */
  0x1261,   /* U+1F0AA */
  0x1262,   /* U+1F0AB */
  0x1263,   /* U+1F0AC */
  0x1264,   /* U+1F0AD */
  0x1265,   /* U+1F0AE */
  0xFBC3,   /* U+1F0AF */
  0xFBC3,   /* U+1F0B0 */
  0x1266,   /* U+1F0B1 */
  0x1267,   /* U+1F0B2 */
  0x1268,   /* U+1F0B3 */
  0x1269,   /* U+1F0B4 */
  0x126A,   /* U+1F0B5 */
  0x126B,   /* U+1F0B6 */
  0x126C,   /* U+1F0B7 */
  0x126D,   /* U+1F0B8 */
  0x126E,   /* U+1F0B9 */
  0x126F,   /* U+1F0BA */
  0x1270,   /* U+1F0BB */
  0x1271,   /* U+1F0BC */
  0x1272,   /* U+1F0BD */
  0x1273,   /* U+1F0BE */
  0x1274,   /* U+1F0BF */
  0xFBC3,   /* U+1F0C0 */
  0x1275,   /* U+1F0C1 */
  0x1276,   /* U+1F0C2 */
  0x1277,   /* U+1F0C3 */
  0x1278,   /* U+1F0C4 */
  0x1279,   /* U+1F0C5 */
  0x127A,   /* U+1F0C6 */
  0x127B,   /* U+1F0C7 */
  0x127C,   /* U+1F0C8 */
  0x127D,   /* U+1F0C9 */
  0x127E,   /* U+1F0CA */
  0x127F,   /* U+1F0CB */
  0x1280,   /* U+1F0CC */
  0x1281,   /* U+1F0CD */
  0x1282,   /* U+1F0CE */
  0x1283,   /* U+1F0CF */
  0xFBC3,   /* U+1F0D0 */
  0x1284,   /* U+1F0D1 */
  0x1285,   /* U+1F0D2 */
  0x1286,   /* U+1F0D3 */
  0x1287,   /* U+1F0D4 */
  0x1288,   /* U+1F0D5 */
  0x1289,   /* U+1F0D6 */
  0x128A,   /* U+1F0D7 */
  0x128B,   /* U+1F0D8 */
  0x128C,   /* U+1F0D9 */
  0x128D,   /* U+1F0DA */
  0x128E,   /* U+1F0DB */
  0x128F,   /* U+1F0DC */
  0x1290,   /* U+1F0DD */
  0x1291,   /* U+1F0DE */
  0x1292,   /* U+1F0DF */
  0x1293,   /* U+1F0E0 */
  0x1294,   /* U+1F0E1 */
  0x1295,   /* U+1F0E2 */
  0x1296,   /* U+1F0E3 */
  0x1297,   /* U+1F0E4 */
  0x1298,   /* U+1F0E5 */
  0x1299,   /* U+1F0E6 */
  0x129A,   /* U+1F0E7 */
  0x129B,   /* U+1F0E8 */
  0x129C,   /* U+1F0E9 */
  0x129D,   /* U+1F0EA */
  0x129E,   /* U+1F0EB */
  0x129F,   /* U+1F0EC */
  0x12A0,   /* U+1F0ED */
  0x12A1,   /* U+1F0EE */
  0x12A2,   /* U+1F0EF */
  0x12A3,   /* U+1F0F0 */
  0x12A4,   /* U+1F0F1 */
  0x12A5,   /* U+1F0F2 */
  0x12A6,   /* U+1F0F3 */
  0x12A7,   /* U+1F0F4 */
  0x12A8,   /* U+1F0F5 */
  0xFBC3,   /* U+1F0F6 */
  0xFBC3,   /* U+1F0F7 */
  0xFBC3,   /* U+1F0F8 */
  0xFBC3,   /* U+1F0F9 */
  0xFBC3,   /* U+1F0FA */
  0xFBC3,   /* U+1F0FB */
  0xFBC3,   /* U+1F0FC */
  0xFBC3,   /* U+1F0FD */
  0xFBC3,   /* U+1F0FE */
  0xFBC3,   /* U+1F0FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F000 */
  0x0020,   /* U+1F001 */
  0x0020,   /* U+1F002 */
  0x0020,   /* U+1F003 */
  0x0020,   /* U+1F004 */
  0x0020,   /* U+1F005 */
  0x0020,   /* U+1F006 */
  0x0020,   /* U+1F007 */
  0x0020,   /* U+1F008 */
  0x0020,   /* U+1F009 */
  0x0020,   /* U+1F00A */
  0x0020,   /* U+1F00B */
  0x0020,   /* U+1F00C */
  0x0020,   /* U+1F00D */
  0x0020,   /* U+1F00E */
  0x0020,   /* U+1F00F */
  0x0020,   /* U+1F010 */
  0x0020,   /* U+1F011 */
  0x0020,   /* U+1F012 */
  0x0020,   /* U+1F013 */
  0x0020,   /* U+1F014 */
  0x0020,   /* U+1F015 */
  0x0020,   /* U+1F016 */
  0x0020,   /* U+1F017 */
  0x0020,   /* U+1F018 */
  0x0020,   /* U+1F019 */
  0x0020,   /* U+1F01A */
  0x0020,   /* U+1F01B */
  0x0020,   /* U+1F01C */
  0x0020,   /* U+1F01D */
  0x0020,   /* U+1F01E */
  0x0020,   /* U+1F01F */
  0x0020,   /* U+1F020 */
  0x0020,   /* U+1F021 */
  0x0020,   /* U+1F022 */
  0x0020,   /* U+1F023 */
  0x0020,   /* U+1F024 */
  0x0020,   /* U+1F025 */
  0x0020,   /* U+1F026 */
  0x0020,   /* U+1F027 */
  0x0020,   /* U+1F028 */
  0x0020,   /* U+1F029 */
  0x0020,   /* U+1F02A */
  0x0020,   /* U+1F02B */
  0x0020,   /* U+1F02C */
  0x0020,   /* U+1F02D */
  0x0020,   /* U+1F02E */
  0x0020,   /* U+1F02F */
  0x0020,   /* U+1F030 */
  0x0020,   /* U+1F031 */
  0x0020,   /* U+1F032 */
  0x0020,   /* U+1F033 */
  0x0020,   /* U+1F034 */
  0x0020,   /* U+1F035 */
  0x0020,   /* U+1F036 */
  0x0020,   /* U+1F037 */
  0x0020,   /* U+1F038 */
  0x0020,   /* U+1F039 */
  0x0020,   /* U+1F03A */
  0x0020,   /* U+1F03B */
  0x0020,   /* U+1F03C */
  0x0020,   /* U+1F03D */
  0x0020,   /* U+1F03E */
  0x0020,   /* U+1F03F */
  0x0020,   /* U+1F040 */
  0x0020,   /* U+1F041 */
  0x0020,   /* U+1F042 */
  0x0020,   /* U+1F043 */
  0x0020,   /* U+1F044 */
  0x0020,   /* U+1F045 */
  0x0020,   /* U+1F046 */
  0x0020,   /* U+1F047 */
  0x0020,   /* U+1F048 */
  0x0020,   /* U+1F049 */
  0x0020,   /* U+1F04A */
  0x0020,   /* U+1F04B */
  0x0020,   /* U+1F04C */
  0x0020,   /* U+1F04D */
  0x0020,   /* U+1F04E */
  0x0020,   /* U+1F04F */
  0x0020,   /* U+1F050 */
  0x0020,   /* U+1F051 */
  0x0020,   /* U+1F052 */
  0x0020,   /* U+1F053 */
  0x0020,   /* U+1F054 */
  0x0020,   /* U+1F055 */
  0x0020,   /* U+1F056 */
  0x0020,   /* U+1F057 */
  0x0020,   /* U+1F058 */
  0x0020,   /* U+1F059 */
  0x0020,   /* U+1F05A */
  0x0020,   /* U+1F05B */
  0x0020,   /* U+1F05C */
  0x0020,   /* U+1F05D */
  0x0020,   /* U+1F05E */
  0x0020,   /* U+1F05F */
  0x0020,   /* U+1F060 */
  0x0020,   /* U+1F061 */
  0x0020,   /* U+1F062 */
  0x0020,   /* U+1F063 */
  0x0020,   /* U+1F064 */
  0x0020,   /* U+1F065 */
  0x0020,   /* U+1F066 */
  0x0020,   /* U+1F067 */
  0x0020,   /* U+1F068 */
  0x0020,   /* U+1F069 */
  0x0020,   /* U+1F06A */
  0x0020,   /* U+1F06B */
  0x0020,   /* U+1F06C */
  0x0020,   /* U+1F06D */
  0x0020,   /* U+1F06E */
  0x0020,   /* U+1F06F */
  0x0020,   /* U+1F070 */
  0x0020,   /* U+1F071 */
  0x0020,   /* U+1F072 */
  0x0020,   /* U+1F073 */
  0x0020,   /* U+1F074 */
  0x0020,   /* U+1F075 */
  0x0020,   /* U+1F076 */
  0x0020,   /* U+1F077 */
  0x0020,   /* U+1F078 */
  0x0020,   /* U+1F079 */
  0x0020,   /* U+1F07A */
  0x0020,   /* U+1F07B */
  0x0020,   /* U+1F07C */
  0x0020,   /* U+1F07D */
  0x0020,   /* U+1F07E */
  0x0020,   /* U+1F07F */
  0x0020,   /* U+1F080 */
  0x0020,   /* U+1F081 */
  0x0020,   /* U+1F082 */
  0x0020,   /* U+1F083 */
  0x0020,   /* U+1F084 */
  0x0020,   /* U+1F085 */
  0x0020,   /* U+1F086 */
  0x0020,   /* U+1F087 */
  0x0020,   /* U+1F088 */
  0x0020,   /* U+1F089 */
  0x0020,   /* U+1F08A */
  0x0020,   /* U+1F08B */
  0x0020,   /* U+1F08C */
  0x0020,   /* U+1F08D */
  0x0020,   /* U+1F08E */
  0x0020,   /* U+1F08F */
  0x0020,   /* U+1F090 */
  0x0020,   /* U+1F091 */
  0x0020,   /* U+1F092 */
  0x0020,   /* U+1F093 */
  0x0020,   /* U+1F094 */
  0x0020,   /* U+1F095 */
  0x0020,   /* U+1F096 */
  0x0020,   /* U+1F097 */
  0x0020,   /* U+1F098 */
  0x0020,   /* U+1F099 */
  0x0020,   /* U+1F09A */
  0x0020,   /* U+1F09B */
  0x0020,   /* U+1F09C */
  0x0020,   /* U+1F09D */
  0x0020,   /* U+1F09E */
  0x0020,   /* U+1F09F */
  0x0020,   /* U+1F0A0 */
  0x0020,   /* U+1F0A1 */
  0x0020,   /* U+1F0A2 */
  0x0020,   /* U+1F0A3 */
  0x0020,   /* U+1F0A4 */
  0x0020,   /* U+1F0A5 */
  0x0020,   /* U+1F0A6 */
  0x0020,   /* U+1F0A7 */
  0x0020,   /* U+1F0A8 */
  0x0020,   /* U+1F0A9 */
  0x0020,   /* U+1F0AA */
  0x0020,   /* U+1F0AB */
  0x0020,   /* U+1F0AC */
  0x0020,   /* U+1F0AD */
  0x0020,   /* U+1F0AE */
  0x0020,   /* U+1F0AF */
  0x0020,   /* U+1F0B0 */
  0x0020,   /* U+1F0B1 */
  0x0020,   /* U+1F0B2 */
  0x0020,   /* U+1F0B3 */
  0x0020,   /* U+1F0B4 */
  0x0020,   /* U+1F0B5 */
  0x0020,   /* U+1F0B6 */
  0x0020,   /* U+1F0B7 */
  0x0020,   /* U+1F0B8 */
  0x0020,   /* U+1F0B9 */
  0x0020,   /* U+1F0BA */
  0x0020,   /* U+1F0BB */
  0x0020,   /* U+1F0BC */
  0x0020,   /* U+1F0BD */
  0x0020,   /* U+1F0BE */
  0x0020,   /* U+1F0BF */
  0x0020,   /* U+1F0C0 */
  0x0020,   /* U+1F0C1 */
  0x0020,   /* U+1F0C2 */
  0x0020,   /* U+1F0C3 */
  0x0020,   /* U+1F0C4 */
  0x0020,   /* U+1F0C5 */
  0x0020,   /* U+1F0C6 */
  0x0020,   /* U+1F0C7 */
  0x0020,   /* U+1F0C8 */
  0x0020,   /* U+1F0C9 */
  0x0020,   /* U+1F0CA */
  0x0020,   /* U+1F0CB */
  0x0020,   /* U+1F0CC */
  0x0020,   /* U+1F0CD */
  0x0020,   /* U+1F0CE */
  0x0020,   /* U+1F0CF */
  0x0020,   /* U+1F0D0 */
  0x0020,   /* U+1F0D1 */
  0x0020,   /* U+1F0D2 */
  0x0020,   /* U+1F0D3 */
  0x0020,   /* U+1F0D4 */
  0x0020,   /* U+1F0D5 */
  0x0020,   /* U+1F0D6 */
  0x0020,   /* U+1F0D7 */
  0x0020,   /* U+1F0D8 */
  0x0020,   /* U+1F0D9 */
  0x0020,   /* U+1F0DA */
  0x0020,   /* U+1F0DB */
  0x0020,   /* U+1F0DC */
  0x0020,   /* U+1F0DD */
  0x0020,   /* U+1F0DE */
  0x0020,   /* U+1F0DF */
  0x0020,   /* U+1F0E0 */
  0x0020,   /* U+1F0E1 */
  0x0020,   /* U+1F0E2 */
  0x0020,   /* U+1F0E3 */
  0x0020,   /* U+1F0E4 */
  0x0020,   /* U+1F0E5 */
  0x0020,   /* U+1F0E6 */
  0x0020,   /* U+1F0E7 */
  0x0020,   /* U+1F0E8 */
  0x0020,   /* U+1F0E9 */
  0x0020,   /* U+1F0EA */
  0x0020,   /* U+1F0EB */
  0x0020,   /* U+1F0EC */
  0x0020,   /* U+1F0ED */
  0x0020,   /* U+1F0EE */
  0x0020,   /* U+1F0EF */
  0x0020,   /* U+1F0F0 */
  0x0020,   /* U+1F0F1 */
  0x0020,   /* U+1F0F2 */
  0x0020,   /* U+1F0F3 */
  0x0020,   /* U+1F0F4 */
  0x0020,   /* U+1F0F5 */
  0x0020,   /* U+1F0F6 */
  0x0020,   /* U+1F0F7 */
  0x0020,   /* U+1F0F8 */
  0x0020,   /* U+1F0F9 */
  0x0020,   /* U+1F0FA */
  0x0020,   /* U+1F0FB */
  0x0020,   /* U+1F0FC */
  0x0020,   /* U+1F0FD */
  0x0020,   /* U+1F0FE */
  0x0020,   /* U+1F0FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F000 */
  0x0002,   /* U+1F001 */
  0x0002,   /* U+1F002 */
  0x0002,   /* U+1F003 */
  0x0002,   /* U+1F004 */
  0x0002,   /* U+1F005 */
  0x0002,   /* U+1F006 */
  0x0002,   /* U+1F007 */
  0x0002,   /* U+1F008 */
  0x0002,   /* U+1F009 */
  0x0002,   /* U+1F00A */
  0x0002,   /* U+1F00B */
  0x0002,   /* U+1F00C */
  0x0002,   /* U+1F00D */
  0x0002,   /* U+1F00E */
  0x0002,   /* U+1F00F */
  0x0002,   /* U+1F010 */
  0x0002,   /* U+1F011 */
  0x0002,   /* U+1F012 */
  0x0002,   /* U+1F013 */
  0x0002,   /* U+1F014 */
  0x0002,   /* U+1F015 */
  0x0002,   /* U+1F016 */
  0x0002,   /* U+1F017 */
  0x0002,   /* U+1F018 */
  0x0002,   /* U+1F019 */
  0x0002,   /* U+1F01A */
  0x0002,   /* U+1F01B */
  0x0002,   /* U+1F01C */
  0x0002,   /* U+1F01D */
  0x0002,   /* U+1F01E */
  0x0002,   /* U+1F01F */
  0x0002,   /* U+1F020 */
  0x0002,   /* U+1F021 */
  0x0002,   /* U+1F022 */
  0x0002,   /* U+1F023 */
  0x0002,   /* U+1F024 */
  0x0002,   /* U+1F025 */
  0x0002,   /* U+1F026 */
  0x0002,   /* U+1F027 */
  0x0002,   /* U+1F028 */
  0x0002,   /* U+1F029 */
  0x0002,   /* U+1F02A */
  0x0002,   /* U+1F02B */
  0x0002,   /* U+1F02C */
  0x0002,   /* U+1F02D */
  0x0002,   /* U+1F02E */
  0x0002,   /* U+1F02F */
  0x0002,   /* U+1F030 */
  0x0002,   /* U+1F031 */
  0x0002,   /* U+1F032 */
  0x0002,   /* U+1F033 */
  0x0002,   /* U+1F034 */
  0x0002,   /* U+1F035 */
  0x0002,   /* U+1F036 */
  0x0002,   /* U+1F037 */
  0x0002,   /* U+1F038 */
  0x0002,   /* U+1F039 */
  0x0002,   /* U+1F03A */
  0x0002,   /* U+1F03B */
  0x0002,   /* U+1F03C */
  0x0002,   /* U+1F03D */
  0x0002,   /* U+1F03E */
  0x0002,   /* U+1F03F */
  0x0002,   /* U+1F040 */
  0x0002,   /* U+1F041 */
  0x0002,   /* U+1F042 */
  0x0002,   /* U+1F043 */
  0x0002,   /* U+1F044 */
  0x0002,   /* U+1F045 */
  0x0002,   /* U+1F046 */
  0x0002,   /* U+1F047 */
  0x0002,   /* U+1F048 */
  0x0002,   /* U+1F049 */
  0x0002,   /* U+1F04A */
  0x0002,   /* U+1F04B */
  0x0002,   /* U+1F04C */
  0x0002,   /* U+1F04D */
  0x0002,   /* U+1F04E */
  0x0002,   /* U+1F04F */
  0x0002,   /* U+1F050 */
  0x0002,   /* U+1F051 */
  0x0002,   /* U+1F052 */
  0x0002,   /* U+1F053 */
  0x0002,   /* U+1F054 */
  0x0002,   /* U+1F055 */
  0x0002,   /* U+1F056 */
  0x0002,   /* U+1F057 */
  0x0002,   /* U+1F058 */
  0x0002,   /* U+1F059 */
  0x0002,   /* U+1F05A */
  0x0002,   /* U+1F05B */
  0x0002,   /* U+1F05C */
  0x0002,   /* U+1F05D */
  0x0002,   /* U+1F05E */
  0x0002,   /* U+1F05F */
  0x0002,   /* U+1F060 */
  0x0002,   /* U+1F061 */
  0x0002,   /* U+1F062 */
  0x0002,   /* U+1F063 */
  0x0002,   /* U+1F064 */
  0x0002,   /* U+1F065 */
  0x0002,   /* U+1F066 */
  0x0002,   /* U+1F067 */
  0x0002,   /* U+1F068 */
  0x0002,   /* U+1F069 */
  0x0002,   /* U+1F06A */
  0x0002,   /* U+1F06B */
  0x0002,   /* U+1F06C */
  0x0002,   /* U+1F06D */
  0x0002,   /* U+1F06E */
  0x0002,   /* U+1F06F */
  0x0002,   /* U+1F070 */
  0x0002,   /* U+1F071 */
  0x0002,   /* U+1F072 */
  0x0002,   /* U+1F073 */
  0x0002,   /* U+1F074 */
  0x0002,   /* U+1F075 */
  0x0002,   /* U+1F076 */
  0x0002,   /* U+1F077 */
  0x0002,   /* U+1F078 */
  0x0002,   /* U+1F079 */
  0x0002,   /* U+1F07A */
  0x0002,   /* U+1F07B */
  0x0002,   /* U+1F07C */
  0x0002,   /* U+1F07D */
  0x0002,   /* U+1F07E */
  0x0002,   /* U+1F07F */
  0x0002,   /* U+1F080 */
  0x0002,   /* U+1F081 */
  0x0002,   /* U+1F082 */
  0x0002,   /* U+1F083 */
  0x0002,   /* U+1F084 */
  0x0002,   /* U+1F085 */
  0x0002,   /* U+1F086 */
  0x0002,   /* U+1F087 */
  0x0002,   /* U+1F088 */
  0x0002,   /* U+1F089 */
  0x0002,   /* U+1F08A */
  0x0002,   /* U+1F08B */
  0x0002,   /* U+1F08C */
  0x0002,   /* U+1F08D */
  0x0002,   /* U+1F08E */
  0x0002,   /* U+1F08F */
  0x0002,   /* U+1F090 */
  0x0002,   /* U+1F091 */
  0x0002,   /* U+1F092 */
  0x0002,   /* U+1F093 */
  0x0002,   /* U+1F094 */
  0x0002,   /* U+1F095 */
  0x0002,   /* U+1F096 */
  0x0002,   /* U+1F097 */
  0x0002,   /* U+1F098 */
  0x0002,   /* U+1F099 */
  0x0002,   /* U+1F09A */
  0x0002,   /* U+1F09B */
  0x0002,   /* U+1F09C */
  0x0002,   /* U+1F09D */
  0x0002,   /* U+1F09E */
  0x0002,   /* U+1F09F */
  0x0002,   /* U+1F0A0 */
  0x0002,   /* U+1F0A1 */
  0x0002,   /* U+1F0A2 */
  0x0002,   /* U+1F0A3 */
  0x0002,   /* U+1F0A4 */
  0x0002,   /* U+1F0A5 */
  0x0002,   /* U+1F0A6 */
  0x0002,   /* U+1F0A7 */
  0x0002,   /* U+1F0A8 */
  0x0002,   /* U+1F0A9 */
  0x0002,   /* U+1F0AA */
  0x0002,   /* U+1F0AB */
  0x0002,   /* U+1F0AC */
  0x0002,   /* U+1F0AD */
  0x0002,   /* U+1F0AE */
  0x0002,   /* U+1F0AF */
  0x0002,   /* U+1F0B0 */
  0x0002,   /* U+1F0B1 */
  0x0002,   /* U+1F0B2 */
  0x0002,   /* U+1F0B3 */
  0x0002,   /* U+1F0B4 */
  0x0002,   /* U+1F0B5 */
  0x0002,   /* U+1F0B6 */
  0x0002,   /* U+1F0B7 */
  0x0002,   /* U+1F0B8 */
  0x0002,   /* U+1F0B9 */
  0x0002,   /* U+1F0BA */
  0x0002,   /* U+1F0BB */
  0x0002,   /* U+1F0BC */
  0x0002,   /* U+1F0BD */
  0x0002,   /* U+1F0BE */
  0x0002,   /* U+1F0BF */
  0x0002,   /* U+1F0C0 */
  0x0002,   /* U+1F0C1 */
  0x0002,   /* U+1F0C2 */
  0x0002,   /* U+1F0C3 */
  0x0002,   /* U+1F0C4 */
  0x0002,   /* U+1F0C5 */
  0x0002,   /* U+1F0C6 */
  0x0002,   /* U+1F0C7 */
  0x0002,   /* U+1F0C8 */
  0x0002,   /* U+1F0C9 */
  0x0002,   /* U+1F0CA */
  0x0002,   /* U+1F0CB */
  0x0002,   /* U+1F0CC */
  0x0002,   /* U+1F0CD */
  0x0002,   /* U+1F0CE */
  0x0002,   /* U+1F0CF */
  0x0002,   /* U+1F0D0 */
  0x0002,   /* U+1F0D1 */
  0x0002,   /* U+1F0D2 */
  0x0002,   /* U+1F0D3 */
  0x0002,   /* U+1F0D4 */
  0x0002,   /* U+1F0D5 */
  0x0002,   /* U+1F0D6 */
  0x0002,   /* U+1F0D7 */
  0x0002,   /* U+1F0D8 */
  0x0002,   /* U+1F0D9 */
  0x0002,   /* U+1F0DA */
  0x0002,   /* U+1F0DB */
  0x0002,   /* U+1F0DC */
  0x0002,   /* U+1F0DD */
  0x0002,   /* U+1F0DE */
  0x0002,   /* U+1F0DF */
  0x0002,   /* U+1F0E0 */
  0x0002,   /* U+1F0E1 */
  0x0002,   /* U+1F0E2 */
  0x0002,   /* U+1F0E3 */
  0x0002,   /* U+1F0E4 */
  0x0002,   /* U+1F0E5 */
  0x0002,   /* U+1F0E6 */
  0x0002,   /* U+1F0E7 */
  0x0002,   /* U+1F0E8 */
  0x0002,   /* U+1F0E9 */
  0x0002,   /* U+1F0EA */
  0x0002,   /* U+1F0EB */
  0x0002,   /* U+1F0EC */
  0x0002,   /* U+1F0ED */
  0x0002,   /* U+1F0EE */
  0x0002,   /* U+1F0EF */
  0x0002,   /* U+1F0F0 */
  0x0002,   /* U+1F0F1 */
  0x0002,   /* U+1F0F2 */
  0x0002,   /* U+1F0F3 */
  0x0002,   /* U+1F0F4 */
  0x0002,   /* U+1F0F5 */
  0x0002,   /* U+1F0F6 */
  0x0002,   /* U+1F0F7 */
  0x0002,   /* U+1F0F8 */
  0x0002,   /* U+1F0F9 */
  0x0002,   /* U+1F0FA */
  0x0002,   /* U+1F0FB */
  0x0002,   /* U+1F0FC */
  0x0002,   /* U+1F0FD */
  0x0002,   /* U+1F0FE */
  0x0002,   /* U+1F0FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1F000 */
  0x0000,   /* U+1F001 */
  0x0000,   /* U+1F002 */
  0x0000,   /* U+1F003 */
  0x0000,   /* U+1F004 */
  0x0000,   /* U+1F005 */
  0x0000,   /* U+1F006 */
  0x0000,   /* U+1F007 */
  0x0000,   /* U+1F008 */
  0x0000,   /* U+1F009 */
  0x0000,   /* U+1F00A */
  0x0000,   /* U+1F00B */
  0x0000,   /* U+1F00C */
  0x0000,   /* U+1F00D */
  0x0000,   /* U+1F00E */
  0x0000,   /* U+1F00F */
  0x0000,   /* U+1F010 */
  0x0000,   /* U+1F011 */
  0x0000,   /* U+1F012 */
  0x0000,   /* U+1F013 */
  0x0000,   /* U+1F014 */
  0x0000,   /* U+1F015 */
  0x0000,   /* U+1F016 */
  0x0000,   /* U+1F017 */
  0x0000,   /* U+1F018 */
  0x0000,   /* U+1F019 */
  0x0000,   /* U+1F01A */
  0x0000,   /* U+1F01B */
  0x0000,   /* U+1F01C */
  0x0000,   /* U+1F01D */
  0x0000,   /* U+1F01E */
  0x0000,   /* U+1F01F */
  0x0000,   /* U+1F020 */
  0x0000,   /* U+1F021 */
  0x0000,   /* U+1F022 */
  0x0000,   /* U+1F023 */
  0x0000,   /* U+1F024 */
  0x0000,   /* U+1F025 */
  0x0000,   /* U+1F026 */
  0x0000,   /* U+1F027 */
  0x0000,   /* U+1F028 */
  0x0000,   /* U+1F029 */
  0x0000,   /* U+1F02A */
  0x0000,   /* U+1F02B */
  0xF02C,   /* U+1F02C */
  0xF02D,   /* U+1F02D */
  0xF02E,   /* U+1F02E */
  0xF02F,   /* U+1F02F */
  0x0000,   /* U+1F030 */
  0x0000,   /* U+1F031 */
  0x0000,   /* U+1F032 */
  0x0000,   /* U+1F033 */
  0x0000,   /* U+1F034 */
  0x0000,   /* U+1F035 */
  0x0000,   /* U+1F036 */
  0x0000,   /* U+1F037 */
  0x0000,   /* U+1F038 */
  0x0000,   /* U+1F039 */
  0x0000,   /* U+1F03A */
  0x0000,   /* U+1F03B */
  0x0000,   /* U+1F03C */
  0x0000,   /* U+1F03D */
  0x0000,   /* U+1F03E */
  0x0000,   /* U+1F03F */
  0x0000,   /* U+1F040 */
  0x0000,   /* U+1F041 */
  0x0000,   /* U+1F042 */
  0x0000,   /* U+1F043 */
  0x0000,   /* U+1F044 */
  0x0000,   /* U+1F045 */
  0x0000,   /* U+1F046 */
  0x0000,   /* U+1F047 */
  0x0000,   /* U+1F048 */
  0x0000,   /* U+1F049 */
  0x0000,   /* U+1F04A */
  0x0000,   /* U+1F04B */
  0x0000,   /* U+1F04C */
  0x0000,   /* U+1F04D */
  0x0000,   /* U+1F04E */
  0x0000,   /* U+1F04F */
  0x0000,   /* U+1F050 */
  0x0000,   /* U+1F051 */
  0x0000,   /* U+1F052 */
  0x0000,   /* U+1F053 */
  0x0000,   /* U+1F054 */
  0x0000,   /* U+1F055 */
  0x0000,   /* U+1F056 */
  0x0000,   /* U+1F057 */
  0x0000,   /* U+1F058 */
  0x0000,   /* U+1F059 */
  0x0000,   /* U+1F05A */
  0x0000,   /* U+1F05B */
  0x0000,   /* U+1F05C */
  0x0000,   /* U+1F05D */
  0x0000,   /* U+1F05E */
  0x0000,   /* U+1F05F */
  0x0000,   /* U+1F060 */
  0x0000,   /* U+1F061 */
  0x0000,   /* U+1F062 */
  0x0000,   /* U+1F063 */
  0x0000,   /* U+1F064 */
  0x0000,   /* U+1F065 */
  0x0000,   /* U+1F066 */
  0x0000,   /* U+1F067 */
  0x0000,   /* U+1F068 */
  0x0000,   /* U+1F069 */
  0x0000,   /* U+1F06A */
  0x0000,   /* U+1F06B */
  0x0000,   /* U+1F06C */
  0x0000,   /* U+1F06D */
  0x0000,   /* U+1F06E */
  0x0000,   /* U+1F06F */
  0x0000,   /* U+1F070 */
  0x0000,   /* U+1F071 */
  0x0000,   /* U+1F072 */
  0x0000,   /* U+1F073 */
  0x0000,   /* U+1F074 */
  0x0000,   /* U+1F075 */
  0x0000,   /* U+1F076 */
  0x0000,   /* U+1F077 */
  0x0000,   /* U+1F078 */
  0x0000,   /* U+1F079 */
  0x0000,   /* U+1F07A */
  0x0000,   /* U+1F07B */
  0x0000,   /* U+1F07C */
  0x0000,   /* U+1F07D */
  0x0000,   /* U+1F07E */
  0x0000,   /* U+1F07F */
  0x0000,   /* U+1F080 */
  0x0000,   /* U+1F081 */
  0x0000,   /* U+1F082 */
  0x0000,   /* U+1F083 */
  0x0000,   /* U+1F084 */
  0x0000,   /* U+1F085 */
  0x0000,   /* U+1F086 */
  0x0000,   /* U+1F087 */
  0x0000,   /* U+1F088 */
  0x0000,   /* U+1F089 */
  0x0000,   /* U+1F08A */
  0x0000,   /* U+1F08B */
  0x0000,   /* U+1F08C */
  0x0000,   /* U+1F08D */
  0x0000,   /* U+1F08E */
  0x0000,   /* U+1F08F */
  0x0000,   /* U+1F090 */
  0x0000,   /* U+1F091 */
  0x0000,   /* U+1F092 */
  0x0000,   /* U+1F093 */
  0xF094,   /* U+1F094 */
  0xF095,   /* U+1F095 */
  0xF096,   /* U+1F096 */
  0xF097,   /* U+1F097 */
  0xF098,   /* U+1F098 */
  0xF099,   /* U+1F099 */
  0xF09A,   /* U+1F09A */
  0xF09B,   /* U+1F09B */
  0xF09C,   /* U+1F09C */
  0xF09D,   /* U+1F09D */
  0xF09E,   /* U+1F09E */
  0xF09F,   /* U+1F09F */
  0x0000,   /* U+1F0A0 */
  0x0000,   /* U+1F0A1 */
  0x0000,   /* U+1F0A2 */
  0x0000,   /* U+1F0A3 */
  0x0000,   /* U+1F0A4 */
  0x0000,   /* U+1F0A5 */
  0x0000,   /* U+1F0A6 */
  0x0000,   /* U+1F0A7 */
  0x0000,   /* U+1F0A8 */
  0x0000,   /* U+1F0A9 */
  0x0000,   /* U+1F0AA */
  0x0000,   /* U+1F0AB */
  0x0000,   /* U+1F0AC */
  0x0000,   /* U+1F0AD */
  0x0000,   /* U+1F0AE */
  0xF0AF,   /* U+1F0AF */
  0xF0B0,   /* U+1F0B0 */
  0x0000,   /* U+1F0B1 */
  0x0000,   /* U+1F0B2 */
  0x0000,   /* U+1F0B3 */
  0x0000,   /* U+1F0B4 */
  0x0000,   /* U+1F0B5 */
  0x0000,   /* U+1F0B6 */
  0x0000,   /* U+1F0B7 */
  0x0000,   /* U+1F0B8 */
  0x0000,   /* U+1F0B9 */
  0x0000,   /* U+1F0BA */
  0x0000,   /* U+1F0BB */
  0x0000,   /* U+1F0BC */
  0x0000,   /* U+1F0BD */
  0x0000,   /* U+1F0BE */
  0x0000,   /* U+1F0BF */
  0xF0C0,   /* U+1F0C0 */
  0x0000,   /* U+1F0C1 */
  0x0000,   /* U+1F0C2 */
  0x0000,   /* U+1F0C3 */
  0x0000,   /* U+1F0C4 */
  0x0000,   /* U+1F0C5 */
  0x0000,   /* U+1F0C6 */
  0x0000,   /* U+1F0C7 */
  0x0000,   /* U+1F0C8 */
  0x0000,   /* U+1F0C9 */
  0x0000,   /* U+1F0CA */
  0x0000,   /* U+1F0CB */
  0x0000,   /* U+1F0CC */
  0x0000,   /* U+1F0CD */
  0x0000,   /* U+1F0CE */
  0x0000,   /* U+1F0CF */
  0xF0D0,   /* U+1F0D0 */
  0x0000,   /* U+1F0D1 */
  0x0000,   /* U+1F0D2 */
  0x0000,   /* U+1F0D3 */
  0x0000,   /* U+1F0D4 */
  0x0000,   /* U+1F0D5 */
  0x0000,   /* U+1F0D6 */
  0x0000,   /* U+1F0D7 */
  0x0000,   /* U+1F0D8 */
  0x0000,   /* U+1F0D9 */
  0x0000,   /* U+1F0DA */
  0x0000,   /* U+1F0DB */
  0x0000,   /* U+1F0DC */
  0x0000,   /* U+1F0DD */
  0x0000,   /* U+1F0DE */
  0x0000,   /* U+1F0DF */
  0x0000,   /* U+1F0E0 */
  0x0000,   /* U+1F0E1 */
  0x0000,   /* U+1F0E2 */
  0x0000,   /* U+1F0E3 */
  0x0000,   /* U+1F0E4 */
  0x0000,   /* U+1F0E5 */
  0x0000,   /* U+1F0E6 */
  0x0000,   /* U+1F0E7 */
  0x0000,   /* U+1F0E8 */
  0x0000,   /* U+1F0E9 */
  0x0000,   /* U+1F0EA */
  0x0000,   /* U+1F0EB */
  0x0000,   /* U+1F0EC */
  0x0000,   /* U+1F0ED */
  0x0000,   /* U+1F0EE */
  0x0000,   /* U+1F0EF */
  0x0000,   /* U+1F0F0 */
  0x0000,   /* U+1F0F1 */
  0x0000,   /* U+1F0F2 */
  0x0000,   /* U+1F0F3 */
  0x0000,   /* U+1F0F4 */
  0x0000,   /* U+1F0F5 */
  0xF0F6,   /* U+1F0F6 */
  0xF0F7,   /* U+1F0F7 */
  0xF0F8,   /* U+1F0F8 */
  0xF0F9,   /* U+1F0F9 */
  0xF0FA,   /* U+1F0FA */
  0xF0FB,   /* U+1F0FB */
  0xF0FC,   /* U+1F0FC */
  0xF0FD,   /* U+1F0FD */
  0xF0FE,   /* U+1F0FE */
  0xF0FF,   /* U+1F0FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1F000 */
  0x0000,   /* U+1F001 */
  0x0000,   /* U+1F002 */
  0x0000,   /* U+1F003 */
  0x0000,   /* U+1F004 */
  0x0000,   /* U+1F005 */
  0x0000,   /* U+1F006 */
  0x0000,   /* U+1F007 */
  0x0000,   /* U+1F008 */
  0x0000,   /* U+1F009 */
  0x0000,   /* U+1F00A */
  0x0000,   /* U+1F00B */
  0x0000,   /* U+1F00C */
  0x0000,   /* U+1F00D */
  0x0000,   /* U+1F00E */
  0x0000,   /* U+1F00F */
  0x0000,   /* U+1F010 */
  0x0000,   /* U+1F011 */
  0x0000,   /* U+1F012 */
  0x0000,   /* U+1F013 */
  0x0000,   /* U+1F014 */
  0x0000,   /* U+1F015 */
  0x0000,   /* U+1F016 */
  0x0000,   /* U+1F017 */
  0x0000,   /* U+1F018 */
  0x0000,   /* U+1F019 */
  0x0000,   /* U+1F01A */
  0x0000,   /* U+1F01B */
  0x0000,   /* U+1F01C */
  0x0000,   /* U+1F01D */
  0x0000,   /* U+1F01E */
  0x0000,   /* U+1F01F */
  0x0000,   /* U+1F020 */
  0x0000,   /* U+1F021 */
  0x0000,   /* U+1F022 */
  0x0000,   /* U+1F023 */
  0x0000,   /* U+1F024 */
  0x0000,   /* U+1F025 */
  0x0000,   /* U+1F026 */
  0x0000,   /* U+1F027 */
  0x0000,   /* U+1F028 */
  0x0000,   /* U+1F029 */
  0x0000,   /* U+1F02A */
  0x0000,   /* U+1F02B */
  0x0000,   /* U+1F02C */
  0x0000,   /* U+1F02D */
  0x0000,   /* U+1F02E */
  0x0000,   /* U+1F02F */
  0x0000,   /* U+1F030 */
  0x0000,   /* U+1F031 */
  0x0000,   /* U+1F032 */
  0x0000,   /* U+1F033 */
  0x0000,   /* U+1F034 */
  0x0000,   /* U+1F035 */
  0x0000,   /* U+1F036 */
  0x0000,   /* U+1F037 */
  0x0000,   /* U+1F038 */
  0x0000,   /* U+1F039 */
  0x0000,   /* U+1F03A */
  0x0000,   /* U+1F03B */
  0x0000,   /* U+1F03C */
  0x0000,   /* U+1F03D */
  0x0000,   /* U+1F03E */
  0x0000,   /* U+1F03F */
  0x0000,   /* U+1F040 */
  0x0000,   /* U+1F041 */
  0x0000,   /* U+1F042 */
  0x0000,   /* U+1F043 */
  0x0000,   /* U+1F044 */
  0x0000,   /* U+1F045 */
  0x0000,   /* U+1F046 */
  0x0000,   /* U+1F047 */
  0x0000,   /* U+1F048 */
  0x0000,   /* U+1F049 */
  0x0000,   /* U+1F04A */
  0x0000,   /* U+1F04B */
  0x0000,   /* U+1F04C */
  0x0000,   /* U+1F04D */
  0x0000,   /* U+1F04E */
  0x0000,   /* U+1F04F */
  0x0000,   /* U+1F050 */
  0x0000,   /* U+1F051 */
  0x0000,   /* U+1F052 */
  0x0000,   /* U+1F053 */
  0x0000,   /* U+1F054 */
  0x0000,   /* U+1F055 */
  0x0000,   /* U+1F056 */
  0x0000,   /* U+1F057 */
  0x0000,   /* U+1F058 */
  0x0000,   /* U+1F059 */
  0x0000,   /* U+1F05A */
  0x0000,   /* U+1F05B */
  0x0000,   /* U+1F05C */
  0x0000,   /* U+1F05D */
  0x0000,   /* U+1F05E */
  0x0000,   /* U+1F05F */
  0x0000,   /* U+1F060 */
  0x0000,   /* U+1F061 */
  0x0000,   /* U+1F062 */
  0x0000,   /* U+1F063 */
  0x0000,   /* U+1F064 */
  0x0000,   /* U+1F065 */
  0x0000,   /* U+1F066 */
  0x0000,   /* U+1F067 */
  0x0000,   /* U+1F068 */
  0x0000,   /* U+1F069 */
  0x0000,   /* U+1F06A */
  0x0000,   /* U+1F06B */
  0x0000,   /* U+1F06C */
  0x0000,   /* U+1F06D */
  0x0000,   /* U+1F06E */
  0x0000,   /* U+1F06F */
  0x0000,   /* U+1F070 */
  0x0000,   /* U+1F071 */
  0x0000,   /* U+1F072 */
  0x0000,   /* U+1F073 */
  0x0000,   /* U+1F074 */
  0x0000,   /* U+1F075 */
  0x0000,   /* U+1F076 */
  0x0000,   /* U+1F077 */
  0x0000,   /* U+1F078 */
  0x0000,   /* U+1F079 */
  0x0000,   /* U+1F07A */
  0x0000,   /* U+1F07B */
  0x0000,   /* U+1F07C */
  0x0000,   /* U+1F07D */
  0x0000,   /* U+1F07E */
  0x0000,   /* U+1F07F */
  0x0000,   /* U+1F080 */
  0x0000,   /* U+1F081 */
  0x0000,   /* U+1F082 */
  0x0000,   /* U+1F083 */
  0x0000,   /* U+1F084 */
  0x0000,   /* U+1F085 */
  0x0000,   /* U+1F086 */
  0x0000,   /* U+1F087 */
  0x0000,   /* U+1F088 */
  0x0000,   /* U+1F089 */
  0x0000,   /* U+1F08A */
  0x0000,   /* U+1F08B */
  0x0000,   /* U+1F08C */
  0x0000,   /* U+1F08D */
  0x0000,   /* U+1F08E */
  0x0000,   /* U+1F08F */
  0x0000,   /* U+1F090 */
  0x0000,   /* U+1F091 */
  0x0000,   /* U+1F092 */
  0x0000,   /* U+1F093 */
  0x0000,   /* U+1F094 */
  0x0000,   /* U+1F095 */
  0x0000,   /* U+1F096 */
  0x0000,   /* U+1F097 */
  0x0000,   /* U+1F098 */
  0x0000,   /* U+1F099 */
  0x0000,   /* U+1F09A */
  0x0000,   /* U+1F09B */
  0x0000,   /* U+1F09C */
  0x0000,   /* U+1F09D */
  0x0000,   /* U+1F09E */
  0x0000,   /* U+1F09F */
  0x0000,   /* U+1F0A0 */
  0x0000,   /* U+1F0A1 */
  0x0000,   /* U+1F0A2 */
  0x0000,   /* U+1F0A3 */
  0x0000,   /* U+1F0A4 */
  0x0000,   /* U+1F0A5 */
  0x0000,   /* U+1F0A6 */
  0x0000,   /* U+1F0A7 */
  0x0000,   /* U+1F0A8 */
  0x0000,   /* U+1F0A9 */
  0x0000,   /* U+1F0AA */
  0x0000,   /* U+1F0AB */
  0x0000,   /* U+1F0AC */
  0x0000,   /* U+1F0AD */
  0x0000,   /* U+1F0AE */
  0x0000,   /* U+1F0AF */
  0x0000,   /* U+1F0B0 */
  0x0000,   /* U+1F0B1 */
  0x0000,   /* U+1F0B2 */
  0x0000,   /* U+1F0B3 */
  0x0000,   /* U+1F0B4 */
  0x0000,   /* U+1F0B5 */
  0x0000,   /* U+1F0B6 */
  0x0000,   /* U+1F0B7 */
  0x0000,   /* U+1F0B8 */
  0x0000,   /* U+1F0B9 */
  0x0000,   /* U+1F0BA */
  0x0000,   /* U+1F0BB */
  0x0000,   /* U+1F0BC */
  0x0000,   /* U+1F0BD */
  0x0000,   /* U+1F0BE */
  0x0000,   /* U+1F0BF */
  0x0000,   /* U+1F0C0 */
  0x0000,   /* U+1F0C1 */
  0x0000,   /* U+1F0C2 */
  0x0000,   /* U+1F0C3 */
  0x0000,   /* U+1F0C4 */
  0x0000,   /* U+1F0C5 */
  0x0000,   /* U+1F0C6 */
  0x0000,   /* U+1F0C7 */
  0x0000,   /* U+1F0C8 */
  0x0000,   /* U+1F0C9 */
  0x0000,   /* U+1F0CA */
  0x0000,   /* U+1F0CB */
  0x0000,   /* U+1F0CC */
  0x0000,   /* U+1F0CD */
  0x0000,   /* U+1F0CE */
  0x0000,   /* U+1F0CF */
  0x0000,   /* U+1F0D0 */
  0x0000,   /* U+1F0D1 */
  0x0000,   /* U+1F0D2 */
  0x0000,   /* U+1F0D3 */
  0x0000,   /* U+1F0D4 */
  0x0000,   /* U+1F0D5 */
  0x0000,   /* U+1F0D6 */
  0x0000,   /* U+1F0D7 */
  0x0000,   /* U+1F0D8 */
  0x0000,   /* U+1F0D9 */
  0x0000,   /* U+1F0DA */
  0x0000,   /* U+1F0DB */
  0x0000,   /* U+1F0DC */
  0x0000,   /* U+1F0DD */
  0x0000,   /* U+1F0DE */
  0x0000,   /* U+1F0DF */
  0x0000,   /* U+1F0E0 */
  0x0000,   /* U+1F0E1 */
  0x0000,   /* U+1F0E2 */
  0x0000,   /* U+1F0E3 */
  0x0000,   /* U+1F0E4 */
  0x0000,   /* U+1F0E5 */
  0x0000,   /* U+1F0E6 */
  0x0000,   /* U+1F0E7 */
  0x0000,   /* U+1F0E8 */
  0x0000,   /* U+1F0E9 */
  0x0000,   /* U+1F0EA */
  0x0000,   /* U+1F0EB */
  0x0000,   /* U+1F0EC */
  0x0000,   /* U+1F0ED */
  0x0000,   /* U+1F0EE */
  0x0000,   /* U+1F0EF */
  0x0000,   /* U+1F0F0 */
  0x0000,   /* U+1F0F1 */
  0x0000,   /* U+1F0F2 */
  0x0000,   /* U+1F0F3 */
  0x0000,   /* U+1F0F4 */
  0x0000,   /* U+1F0F5 */
  0x0000,   /* U+1F0F6 */
  0x0000,   /* U+1F0F7 */
  0x0000,   /* U+1F0F8 */
  0x0000,   /* U+1F0F9 */
  0x0000,   /* U+1F0FA */
  0x0000,   /* U+1F0FB */
  0x0000,   /* U+1F0FC */
  0x0000,   /* U+1F0FD */
  0x0000,   /* U+1F0FE */
  0x0000,   /* U+1F0FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1F000 */
  0x0000,   /* U+1F001 */
  0x0000,   /* U+1F002 */
  0x0000,   /* U+1F003 */
  0x0000,   /* U+1F004 */
  0x0000,   /* U+1F005 */
  0x0000,   /* U+1F006 */
  0x0000,   /* U+1F007 */
  0x0000,   /* U+1F008 */
  0x0000,   /* U+1F009 */
  0x0000,   /* U+1F00A */
  0x0000,   /* U+1F00B */
  0x0000,   /* U+1F00C */
  0x0000,   /* U+1F00D */
  0x0000,   /* U+1F00E */
  0x0000,   /* U+1F00F */
  0x0000,   /* U+1F010 */
  0x0000,   /* U+1F011 */
  0x0000,   /* U+1F012 */
  0x0000,   /* U+1F013 */
  0x0000,   /* U+1F014 */
  0x0000,   /* U+1F015 */
  0x0000,   /* U+1F016 */
  0x0000,   /* U+1F017 */
  0x0000,   /* U+1F018 */
  0x0000,   /* U+1F019 */
  0x0000,   /* U+1F01A */
  0x0000,   /* U+1F01B */
  0x0000,   /* U+1F01C */
  0x0000,   /* U+1F01D */
  0x0000,   /* U+1F01E */
  0x0000,   /* U+1F01F */
  0x0000,   /* U+1F020 */
  0x0000,   /* U+1F021 */
  0x0000,   /* U+1F022 */
  0x0000,   /* U+1F023 */
  0x0000,   /* U+1F024 */
  0x0000,   /* U+1F025 */
  0x0000,   /* U+1F026 */
  0x0000,   /* U+1F027 */
  0x0000,   /* U+1F028 */
  0x0000,   /* U+1F029 */
  0x0000,   /* U+1F02A */
  0x0000,   /* U+1F02B */
  0x0000,   /* U+1F02C */
  0x0000,   /* U+1F02D */
  0x0000,   /* U+1F02E */
  0x0000,   /* U+1F02F */
  0x0000,   /* U+1F030 */
  0x0000,   /* U+1F031 */
  0x0000,   /* U+1F032 */
  0x0000,   /* U+1F033 */
  0x0000,   /* U+1F034 */
  0x0000,   /* U+1F035 */
  0x0000,   /* U+1F036 */
  0x0000,   /* U+1F037 */
  0x0000,   /* U+1F038 */
  0x0000,   /* U+1F039 */
  0x0000,   /* U+1F03A */
  0x0000,   /* U+1F03B */
  0x0000,   /* U+1F03C */
  0x0000,   /* U+1F03D */
  0x0000,   /* U+1F03E */
  0x0000,   /* U+1F03F */
  0x0000,   /* U+1F040 */
  0x0000,   /* U+1F041 */
  0x0000,   /* U+1F042 */
  0x0000,   /* U+1F043 */
  0x0000,   /* U+1F044 */
  0x0000,   /* U+1F045 */
  0x0000,   /* U+1F046 */
  0x0000,   /* U+1F047 */
  0x0000,   /* U+1F048 */
  0x0000,   /* U+1F049 */
  0x0000,   /* U+1F04A */
  0x0000,   /* U+1F04B */
  0x0000,   /* U+1F04C */
  0x0000,   /* U+1F04D */
  0x0000,   /* U+1F04E */
  0x0000,   /* U+1F04F */
  0x0000,   /* U+1F050 */
  0x0000,   /* U+1F051 */
  0x0000,   /* U+1F052 */
  0x0000,   /* U+1F053 */
  0x0000,   /* U+1F054 */
  0x0000,   /* U+1F055 */
  0x0000,   /* U+1F056 */
  0x0000,   /* U+1F057 */
  0x0000,   /* U+1F058 */
  0x0000,   /* U+1F059 */
  0x0000,   /* U+1F05A */
  0x0000,   /* U+1F05B */
  0x0000,   /* U+1F05C */
  0x0000,   /* U+1F05D */
  0x0000,   /* U+1F05E */
  0x0000,   /* U+1F05F */
  0x0000,   /* U+1F060 */
  0x0000,   /* U+1F061 */
  0x0000,   /* U+1F062 */
  0x0000,   /* U+1F063 */
  0x0000,   /* U+1F064 */
  0x0000,   /* U+1F065 */
  0x0000,   /* U+1F066 */
  0x0000,   /* U+1F067 */
  0x0000,   /* U+1F068 */
  0x0000,   /* U+1F069 */
  0x0000,   /* U+1F06A */
  0x0000,   /* U+1F06B */
  0x0000,   /* U+1F06C */
  0x0000,   /* U+1F06D */
  0x0000,   /* U+1F06E */
  0x0000,   /* U+1F06F */
  0x0000,   /* U+1F070 */
  0x0000,   /* U+1F071 */
  0x0000,   /* U+1F072 */
  0x0000,   /* U+1F073 */
  0x0000,   /* U+1F074 */
  0x0000,   /* U+1F075 */
  0x0000,   /* U+1F076 */
  0x0000,   /* U+1F077 */
  0x0000,   /* U+1F078 */
  0x0000,   /* U+1F079 */
  0x0000,   /* U+1F07A */
  0x0000,   /* U+1F07B */
  0x0000,   /* U+1F07C */
  0x0000,   /* U+1F07D */
  0x0000,   /* U+1F07E */
  0x0000,   /* U+1F07F */
  0x0000,   /* U+1F080 */
  0x0000,   /* U+1F081 */
  0x0000,   /* U+1F082 */
  0x0000,   /* U+1F083 */
  0x0000,   /* U+1F084 */
  0x0000,   /* U+1F085 */
  0x0000,   /* U+1F086 */
  0x0000,   /* U+1F087 */
  0x0000,   /* U+1F088 */
  0x0000,   /* U+1F089 */
  0x0000,   /* U+1F08A */
  0x0000,   /* U+1F08B */
  0x0000,   /* U+1F08C */
  0x0000,   /* U+1F08D */
  0x0000,   /* U+1F08E */
  0x0000,   /* U+1F08F */
  0x0000,   /* U+1F090 */
  0x0000,   /* U+1F091 */
  0x0000,   /* U+1F092 */
  0x0000,   /* U+1F093 */
  0x0000,   /* U+1F094 */
  0x0000,   /* U+1F095 */
  0x0000,   /* U+1F096 */
  0x0000,   /* U+1F097 */
  0x0000,   /* U+1F098 */
  0x0000,   /* U+1F099 */
  0x0000,   /* U+1F09A */
  0x0000,   /* U+1F09B */
  0x0000,   /* U+1F09C */
  0x0000,   /* U+1F09D */
  0x0000,   /* U+1F09E */
  0x0000,   /* U+1F09F */
  0x0000,   /* U+1F0A0 */
  0x0000,   /* U+1F0A1 */
  0x0000,   /* U+1F0A2 */
  0x0000,   /* U+1F0A3 */
  0x0000,   /* U+1F0A4 */
  0x0000,   /* U+1F0A5 */
  0x0000,   /* U+1F0A6 */
  0x0000,   /* U+1F0A7 */
  0x0000,   /* U+1F0A8 */
  0x0000,   /* U+1F0A9 */
  0x0000,   /* U+1F0AA */
  0x0000,   /* U+1F0AB */
  0x0000,   /* U+1F0AC */
  0x0000,   /* U+1F0AD */
  0x0000,   /* U+1F0AE */
  0x0000,   /* U+1F0AF */
  0x0000,   /* U+1F0B0 */
  0x0000,   /* U+1F0B1 */
  0x0000,   /* U+1F0B2 */
  0x0000,   /* U+1F0B3 */
  0x0000,   /* U+1F0B4 */
  0x0000,   /* U+1F0B5 */
  0x0000,   /* U+1F0B6 */
  0x0000,   /* U+1F0B7 */
  0x0000,   /* U+1F0B8 */
  0x0000,   /* U+1F0B9 */
  0x0000,   /* U+1F0BA */
  0x0000,   /* U+1F0BB */
  0x0000,   /* U+1F0BC */
  0x0000,   /* U+1F0BD */
  0x0000,   /* U+1F0BE */
  0x0000,   /* U+1F0BF */
  0x0000,   /* U+1F0C0 */
  0x0000,   /* U+1F0C1 */
  0x0000,   /* U+1F0C2 */
  0x0000,   /* U+1F0C3 */
  0x0000,   /* U+1F0C4 */
  0x0000,   /* U+1F0C5 */
  0x0000,   /* U+1F0C6 */
  0x0000,   /* U+1F0C7 */
  0x0000,   /* U+1F0C8 */
  0x0000,   /* U+1F0C9 */
  0x0000,   /* U+1F0CA */
  0x0000,   /* U+1F0CB */
  0x0000,   /* U+1F0CC */
  0x0000,   /* U+1F0CD */
  0x0000,   /* U+1F0CE */
  0x0000,   /* U+1F0CF */
  0x0000,   /* U+1F0D0 */
  0x0000,   /* U+1F0D1 */
  0x0000,   /* U+1F0D2 */
  0x0000,   /* U+1F0D3 */
  0x0000,   /* U+1F0D4 */
  0x0000,   /* U+1F0D5 */
  0x0000,   /* U+1F0D6 */
  0x0000,   /* U+1F0D7 */
  0x0000,   /* U+1F0D8 */
  0x0000,   /* U+1F0D9 */
  0x0000,   /* U+1F0DA */
  0x0000,   /* U+1F0DB */
  0x0000,   /* U+1F0DC */
  0x0000,   /* U+1F0DD */
  0x0000,   /* U+1F0DE */
  0x0000,   /* U+1F0DF */
  0x0000,   /* U+1F0E0 */
  0x0000,   /* U+1F0E1 */
  0x0000,   /* U+1F0E2 */
  0x0000,   /* U+1F0E3 */
  0x0000,   /* U+1F0E4 */
  0x0000,   /* U+1F0E5 */
  0x0000,   /* U+1F0E6 */
  0x0000,   /* U+1F0E7 */
  0x0000,   /* U+1F0E8 */
  0x0000,   /* U+1F0E9 */
  0x0000,   /* U+1F0EA */
  0x0000,   /* U+1F0EB */
  0x0000,   /* U+1F0EC */
  0x0000,   /* U+1F0ED */
  0x0000,   /* U+1F0EE */
  0x0000,   /* U+1F0EF */
  0x0000,   /* U+1F0F0 */
  0x0000,   /* U+1F0F1 */
  0x0000,   /* U+1F0F2 */
  0x0000,   /* U+1F0F3 */
  0x0000,   /* U+1F0F4 */
  0x0000,   /* U+1F0F5 */
  0x0000,   /* U+1F0F6 */
  0x0000,   /* U+1F0F7 */
  0x0000,   /* U+1F0F8 */
  0x0000,   /* U+1F0F9 */
  0x0000,   /* U+1F0FA */
  0x0000,   /* U+1F0FB */
  0x0000,   /* U+1F0FC */
  0x0000,   /* U+1F0FD */
  0x0000,   /* U+1F0FE */
  0x0000,   /* U+1F0FF */
};

uint16 uca900_p1F1[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 
  2, 2, 4, 4, 2, 3, 2, 2, 3, 3, 2, 2, 7, 2, 2, 2, 
  3, 3, 4, 3, 4, 1, 2, 3, 6, 8, 3, 3, 3, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x1C3D,   /* U+1F100 */
  0x1C3D,   /* U+1F101 */
  0x1C3E,   /* U+1F102 */
  0x1C3F,   /* U+1F103 */
  0x1C40,   /* U+1F104 */
  0x1C41,   /* U+1F105 */
  0x1C42,   /* U+1F106 */
  0x1C43,   /* U+1F107 */
  0x1C44,   /* U+1F108 */
  0x1C45,   /* U+1F109 */
  0x1C46,   /* U+1F10A */
  0x1C3D,   /* U+1F10B */
  0x1C3D,   /* U+1F10C */
  0xFBC3,   /* U+1F10D */
  0xFBC3,   /* U+1F10E */
  0xFBC3,   /* U+1F10F */
  0x0317,   /* U+1F110 */
  0x0317,   /* U+1F111 */
  0x0317,   /* U+1F112 */
  0x0317,   /* U+1F113 */
  0x0317,   /* U+1F114 */
  0x0317,   /* U+1F115 */
  0x0317,   /* U+1F116 */
  0x0317,   /* U+1F117 */
  0x0317,   /* U+1F118 */
  0x0317,   /* U+1F119 */
  0x0317,   /* U+1F11A */
  0x0317,   /* U+1F11B */
  0x0317,   /* U+1F11C */
  0x0317,   /* U+1F11D */
  0x0317,   /* U+1F11E */
  0x0317,   /* U+1F11F */
  0x0317,   /* U+1F120 */
  0x0317,   /* U+1F121 */
  0x0317,   /* U+1F122 */
  0x0317,   /* U+1F123 */
  0x0317,   /* U+1F124 */
  0x0317,   /* U+1F125 */
  0x0317,   /* U+1F126 */
  0x0317,   /* U+1F127 */
  0x0317,   /* U+1F128 */
  0x0317,   /* U+1F129 */
  0x0379,   /* U+1F12A */
  0x1C7A,   /* U+1F12B */
  0x1E33,   /* U+1F12C */
  0x1C7A,   /* U+1F12D */
  0x1EF5,   /* U+1F12E */
  0xFBC3,   /* U+1F12F */
  0x1C47,   /* U+1F130 */
  0x1C60,   /* U+1F131 */
  0x1C7A,   /* U+1F132 */
  0x1C8F,   /* U+1F133 */
  0x1CAA,   /* U+1F134 */
  0x1CE5,   /* U+1F135 */
  0x1CF4,   /* U+1F136 */
  0x1D18,   /* U+1F137 */
  0x1D32,   /* U+1F138 */
  0x1D4C,   /* U+1F139 */
  0x1D65,   /* U+1F13A */
  0x1D77,   /* U+1F13B */
  0x1DAA,   /* U+1F13C */
  0x1DB9,   /* U+1F13D */
  0x1DDD,   /* U+1F13E */
  0x1E0C,   /* U+1F13F */
  0x1E21,   /* U+1F140 */
  0x1E33,   /* U+1F141 */
  0x1E71,   /* U+1F142 */
  0x1E95,   /* U+1F143 */
  0x1EB5,   /* U+1F144 */
  0x1EE3,   /* U+1F145 */
  0x1EF5,   /* U+1F146 */
  0x1EFF,   /* U+1F147 */
  0x1F0B,   /* U+1F148 */
  0x1F21,   /* U+1F149 */
  0x1D18,   /* U+1F14A */
  0x1DAA,   /* U+1F14B */
  0x1E71,   /* U+1F14C */
  0x1E71,   /* U+1F14D */
  0x1E0C,   /* U+1F14E */
  0x1EF5,   /* U+1F14F */
  0x1C47,   /* U+1F150 */
  0x1C60,   /* U+1F151 */
  0x1C7A,   /* U+1F152 */
  0x1C8F,   /* U+1F153 */
  0x1CAA,   /* U+1F154 */
  0x1CE5,   /* U+1F155 */
  0x1CF4,   /* U+1F156 */
  0x1D18,   /* U+1F157 */
  0x1D32,   /* U+1F158 */
  0x1D4C,   /* U+1F159 */
  0x1D65,   /* U+1F15A */
  0x1D77,   /* U+1F15B */
  0x1DAA,   /* U+1F15C */
  0x1DB9,   /* U+1F15D */
  0x1DDD,   /* U+1F15E */
  0x1E0C,   /* U+1F15F */
  0x1E21,   /* U+1F160 */
  0x1E33,   /* U+1F161 */
  0x1E71,   /* U+1F162 */
  0x1E95,   /* U+1F163 */
  0x1EB5,   /* U+1F164 */
  0x1EE3,   /* U+1F165 */
  0x1EF5,   /* U+1F166 */
  0x1EFF,   /* U+1F167 */
  0x1F0B,   /* U+1F168 */
  0x1F21,   /* U+1F169 */
  0x1DAA,   /* U+1F16A */
  0x1DAA,   /* U+1F16B */
  0xFBC3,   /* U+1F16C */
  0xFBC3,   /* U+1F16D */
  0xFBC3,   /* U+1F16E */
  0xFBC3,   /* U+1F16F */
  0x1C47,   /* U+1F170 */
  0x1C60,   /* U+1F171 */
  0x1C7A,   /* U+1F172 */
  0x1C8F,   /* U+1F173 */
  0x1CAA,   /* U+1F174 */
  0x1CE5,   /* U+1F175 */
  0x1CF4,   /* U+1F176 */
  0x1D18,   /* U+1F177 */
  0x1D32,   /* U+1F178 */
  0x1D4C,   /* U+1F179 */
  0x1D65,   /* U+1F17A */
  0x1D77,   /* U+1F17B */
  0x1DAA,   /* U+1F17C */
  0x1DB9,   /* U+1F17D */
  0x1DDD,   /* U+1F17E */
  0x1E0C,   /* U+1F17F */
  0x1E21,   /* U+1F180 */
  0x1E33,   /* U+1F181 */
  0x1E71,   /* U+1F182 */
  0x1E95,   /* U+1F183 */
  0x1EB5,   /* U+1F184 */
  0x1EE3,   /* U+1F185 */
  0x1EF5,   /* U+1F186 */
  0x1EFF,   /* U+1F187 */
  0x1F0B,   /* U+1F188 */
  0x1F21,   /* U+1F189 */
  0x1E0C,   /* U+1F18A */
  0x1D32,   /* U+1F18B */
  0x1E0C,   /* U+1F18C */
  0x1E71,   /* U+1F18D */
  0x1C47,   /* U+1F18E */
  0x1EF5,   /* U+1F18F */
  0x1C8F,   /* U+1F190 */
  0x1C7A,   /* U+1F191 */
  0x1C7A,   /* U+1F192 */
  0x1CE5,   /* U+1F193 */
  0x1D32,   /* U+1F194 */
  0x1DB9,   /* U+1F195 */
  0x1DB9,   /* U+1F196 */
  0x1DDD,   /* U+1F197 */
  0x1E71,   /* U+1F198 */
  0x1EB5,   /* U+1F199 */
  0x1EE3,   /* U+1F19A */
  0x1C40,   /* U+1F19B */
  0x1C3F,   /* U+1F19C */
  0x1C3F,   /* U+1F19D */
  0x1C41,   /* U+1F19E */
  0x1C45,   /* U+1F19F */
  0x1C42,   /* U+1F1A0 */
  0x1C44,   /* U+1F1A1 */
  0x1C3F,   /* U+1F1A2 */
  0x1C43,   /* U+1F1A3 */
  0x1C3E,   /* U+1F1A4 */
  0x1C8F,   /* U+1F1A5 */
  0x1D18,   /* U+1F1A6 */
  0x1D18,   /* U+1F1A7 */
  0x1D18,   /* U+1F1A8 */
  0x1D77,   /* U+1F1A9 */
  0x1E71,   /* U+1F1AA */
  0x1EB5,   /* U+1F1AB */
  0x1EE3,   /* U+1F1AC */
  0xFBC3,   /* U+1F1AD */
  0xFBC3,   /* U+1F1AE */
  0xFBC3,   /* U+1F1AF */
  0xFBC3,   /* U+1F1B0 */
  0xFBC3,   /* U+1F1B1 */
  0xFBC3,   /* U+1F1B2 */
  0xFBC3,   /* U+1F1B3 */
  0xFBC3,   /* U+1F1B4 */
  0xFBC3,   /* U+1F1B5 */
  0xFBC3,   /* U+1F1B6 */
  0xFBC3,   /* U+1F1B7 */
  0xFBC3,   /* U+1F1B8 */
  0xFBC3,   /* U+1F1B9 */
  0xFBC3,   /* U+1F1BA */
  0xFBC3,   /* U+1F1BB */
  0xFBC3,   /* U+1F1BC */
  0xFBC3,   /* U+1F1BD */
  0xFBC3,   /* U+1F1BE */
  0xFBC3,   /* U+1F1BF */
  0xFBC3,   /* U+1F1C0 */
  0xFBC3,   /* U+1F1C1 */
  0xFBC3,   /* U+1F1C2 */
  0xFBC3,   /* U+1F1C3 */
  0xFBC3,   /* U+1F1C4 */
  0xFBC3,   /* U+1F1C5 */
  0xFBC3,   /* U+1F1C6 */
  0xFBC3,   /* U+1F1C7 */
  0xFBC3,   /* U+1F1C8 */
  0xFBC3,   /* U+1F1C9 */
  0xFBC3,   /* U+1F1CA */
  0xFBC3,   /* U+1F1CB */
  0xFBC3,   /* U+1F1CC */
  0xFBC3,   /* U+1F1CD */
  0xFBC3,   /* U+1F1CE */
  0xFBC3,   /* U+1F1CF */
  0xFBC3,   /* U+1F1D0 */
  0xFBC3,   /* U+1F1D1 */
  0xFBC3,   /* U+1F1D2 */
  0xFBC3,   /* U+1F1D3 */
  0xFBC3,   /* U+1F1D4 */
  0xFBC3,   /* U+1F1D5 */
  0xFBC3,   /* U+1F1D6 */
  0xFBC3,   /* U+1F1D7 */
  0xFBC3,   /* U+1F1D8 */
  0xFBC3,   /* U+1F1D9 */
  0xFBC3,   /* U+1F1DA */
  0xFBC3,   /* U+1F1DB */
  0xFBC3,   /* U+1F1DC */
  0xFBC3,   /* U+1F1DD */
  0xFBC3,   /* U+1F1DE */
  0xFBC3,   /* U+1F1DF */
  0xFBC3,   /* U+1F1E0 */
  0xFBC3,   /* U+1F1E1 */
  0xFBC3,   /* U+1F1E2 */
  0xFBC3,   /* U+1F1E3 */
  0xFBC3,   /* U+1F1E4 */
  0xFBC3,   /* U+1F1E5 */
  0x0A07,   /* U+1F1E6 */
  0x0A08,   /* U+1F1E7 */
  0x0A09,   /* U+1F1E8 */
  0x0A0A,   /* U+1F1E9 */
  0x0A0B,   /* U+1F1EA */
  0x0A0C,   /* U+1F1EB */
  0x0A0D,   /* U+1F1EC */
  0x0A0E,   /* U+1F1ED */
  0x0A0F,   /* U+1F1EE */
  0x0A10,   /* U+1F1EF */
  0x0A11,   /* U+1F1F0 */
  0x0A12,   /* U+1F1F1 */
  0x0A13,   /* U+1F1F2 */
  0x0A14,   /* U+1F1F3 */
  0x0A15,   /* U+1F1F4 */
  0x0A16,   /* U+1F1F5 */
  0x0A17,   /* U+1F1F6 */
  0x0A18,   /* U+1F1F7 */
  0x0A19,   /* U+1F1F8 */
  0x0A1A,   /* U+1F1F9 */
  0x0A1B,   /* U+1F1FA */
  0x0A1C,   /* U+1F1FB */
  0x0A1D,   /* U+1F1FC */
  0x0A1E,   /* U+1F1FD */
  0x0A1F,   /* U+1F1FE */
  0x0A20,   /* U+1F1FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F100 */
  0x0020,   /* U+1F101 */
  0x0020,   /* U+1F102 */
  0x0020,   /* U+1F103 */
  0x0020,   /* U+1F104 */
  0x0020,   /* U+1F105 */
  0x0020,   /* U+1F106 */
  0x0020,   /* U+1F107 */
  0x0020,   /* U+1F108 */
  0x0020,   /* U+1F109 */
  0x0020,   /* U+1F10A */
  0x0020,   /* U+1F10B */
  0x0020,   /* U+1F10C */
  0x0020,   /* U+1F10D */
  0x0020,   /* U+1F10E */
  0x0020,   /* U+1F10F */
  0x0020,   /* U+1F110 */
  0x0020,   /* U+1F111 */
  0x0020,   /* U+1F112 */
  0x0020,   /* U+1F113 */
  0x0020,   /* U+1F114 */
  0x0020,   /* U+1F115 */
  0x0020,   /* U+1F116 */
  0x0020,   /* U+1F117 */
  0x0020,   /* U+1F118 */
  0x0020,   /* U+1F119 */
  0x0020,   /* U+1F11A */
  0x0020,   /* U+1F11B */
  0x0020,   /* U+1F11C */
  0x0020,   /* U+1F11D */
  0x0020,   /* U+1F11E */
  0x0020,   /* U+1F11F */
  0x0020,   /* U+1F120 */
  0x0020,   /* U+1F121 */
  0x0020,   /* U+1F122 */
  0x0020,   /* U+1F123 */
  0x0020,   /* U+1F124 */
  0x0020,   /* U+1F125 */
  0x0020,   /* U+1F126 */
  0x0020,   /* U+1F127 */
  0x0020,   /* U+1F128 */
  0x0020,   /* U+1F129 */
  0x0020,   /* U+1F12A */
  0x0020,   /* U+1F12B */
  0x0020,   /* U+1F12C */
  0x0020,   /* U+1F12D */
  0x0020,   /* U+1F12E */
  0x0020,   /* U+1F12F */
  0x0020,   /* U+1F130 */
  0x0020,   /* U+1F131 */
  0x0020,   /* U+1F132 */
  0x0020,   /* U+1F133 */
  0x0020,   /* U+1F134 */
  0x0020,   /* U+1F135 */
  0x0020,   /* U+1F136 */
  0x0020,   /* U+1F137 */
  0x0020,   /* U+1F138 */
  0x0020,   /* U+1F139 */
  0x0020,   /* U+1F13A */
  0x0020,   /* U+1F13B */
  0x0020,   /* U+1F13C */
  0x0020,   /* U+1F13D */
  0x0020,   /* U+1F13E */
  0x0020,   /* U+1F13F */
  0x0020,   /* U+1F140 */
  0x0020,   /* U+1F141 */
  0x0020,   /* U+1F142 */
  0x0020,   /* U+1F143 */
  0x0020,   /* U+1F144 */
  0x0020,   /* U+1F145 */
  0x0020,   /* U+1F146 */
  0x0020,   /* U+1F147 */
  0x0020,   /* U+1F148 */
  0x0020,   /* U+1F149 */
  0x0020,   /* U+1F14A */
  0x0020,   /* U+1F14B */
  0x0020,   /* U+1F14C */
  0x0020,   /* U+1F14D */
  0x0020,   /* U+1F14E */
  0x0020,   /* U+1F14F */
  0x0020,   /* U+1F150 */
  0x0020,   /* U+1F151 */
  0x0020,   /* U+1F152 */
  0x0020,   /* U+1F153 */
  0x0020,   /* U+1F154 */
  0x0020,   /* U+1F155 */
  0x0020,   /* U+1F156 */
  0x0020,   /* U+1F157 */
  0x0020,   /* U+1F158 */
  0x0020,   /* U+1F159 */
  0x0020,   /* U+1F15A */
  0x0020,   /* U+1F15B */
  0x0020,   /* U+1F15C */
  0x0020,   /* U+1F15D */
  0x0020,   /* U+1F15E */
  0x0020,   /* U+1F15F */
  0x0020,   /* U+1F160 */
  0x0020,   /* U+1F161 */
  0x0020,   /* U+1F162 */
  0x0020,   /* U+1F163 */
  0x0020,   /* U+1F164 */
  0x0020,   /* U+1F165 */
  0x0020,   /* U+1F166 */
  0x0020,   /* U+1F167 */
  0x0020,   /* U+1F168 */
  0x0020,   /* U+1F169 */
  0x0020,   /* U+1F16A */
  0x0020,   /* U+1F16B */
  0x0020,   /* U+1F16C */
  0x0020,   /* U+1F16D */
  0x0020,   /* U+1F16E */
  0x0020,   /* U+1F16F */
  0x0020,   /* U+1F170 */
  0x0020,   /* U+1F171 */
  0x0020,   /* U+1F172 */
  0x0020,   /* U+1F173 */
  0x0020,   /* U+1F174 */
  0x0020,   /* U+1F175 */
  0x0020,   /* U+1F176 */
  0x0020,   /* U+1F177 */
  0x0020,   /* U+1F178 */
  0x0020,   /* U+1F179 */
  0x0020,   /* U+1F17A */
  0x0020,   /* U+1F17B */
  0x0020,   /* U+1F17C */
  0x0020,   /* U+1F17D */
  0x0020,   /* U+1F17E */
  0x0020,   /* U+1F17F */
  0x0020,   /* U+1F180 */
  0x0020,   /* U+1F181 */
  0x0020,   /* U+1F182 */
  0x0020,   /* U+1F183 */
  0x0020,   /* U+1F184 */
  0x0020,   /* U+1F185 */
  0x0020,   /* U+1F186 */
  0x0020,   /* U+1F187 */
  0x0020,   /* U+1F188 */
  0x0020,   /* U+1F189 */
  0x0020,   /* U+1F18A */
  0x0020,   /* U+1F18B */
  0x0020,   /* U+1F18C */
  0x0020,   /* U+1F18D */
  0x0020,   /* U+1F18E */
  0x0020,   /* U+1F18F */
  0x0020,   /* U+1F190 */
  0x0020,   /* U+1F191 */
  0x0020,   /* U+1F192 */
  0x0020,   /* U+1F193 */
  0x0020,   /* U+1F194 */
  0x0020,   /* U+1F195 */
  0x0020,   /* U+1F196 */
  0x0020,   /* U+1F197 */
  0x0020,   /* U+1F198 */
  0x0020,   /* U+1F199 */
  0x0020,   /* U+1F19A */
  0x0020,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0020,   /* U+1F19D */
  0x0020,   /* U+1F19E */
  0x0020,   /* U+1F19F */
  0x0020,   /* U+1F1A0 */
  0x0020,   /* U+1F1A1 */
  0x0020,   /* U+1F1A2 */
  0x0020,   /* U+1F1A3 */
  0x0020,   /* U+1F1A4 */
  0x0020,   /* U+1F1A5 */
  0x0020,   /* U+1F1A6 */
  0x0020,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0020,   /* U+1F1AA */
  0x0020,   /* U+1F1AB */
  0x0020,   /* U+1F1AC */
  0x0020,   /* U+1F1AD */
  0x0020,   /* U+1F1AE */
  0x0020,   /* U+1F1AF */
  0x0020,   /* U+1F1B0 */
  0x0020,   /* U+1F1B1 */
  0x0020,   /* U+1F1B2 */
  0x0020,   /* U+1F1B3 */
  0x0020,   /* U+1F1B4 */
  0x0020,   /* U+1F1B5 */
  0x0020,   /* U+1F1B6 */
  0x0020,   /* U+1F1B7 */
  0x0020,   /* U+1F1B8 */
  0x0020,   /* U+1F1B9 */
  0x0020,   /* U+1F1BA */
  0x0020,   /* U+1F1BB */
  0x0020,   /* U+1F1BC */
  0x0020,   /* U+1F1BD */
  0x0020,   /* U+1F1BE */
  0x0020,   /* U+1F1BF */
  0x0020,   /* U+1F1C0 */
  0x0020,   /* U+1F1C1 */
  0x0020,   /* U+1F1C2 */
  0x0020,   /* U+1F1C3 */
  0x0020,   /* U+1F1C4 */
  0x0020,   /* U+1F1C5 */
  0x0020,   /* U+1F1C6 */
  0x0020,   /* U+1F1C7 */
  0x0020,   /* U+1F1C8 */
  0x0020,   /* U+1F1C9 */
  0x0020,   /* U+1F1CA */
  0x0020,   /* U+1F1CB */
  0x0020,   /* U+1F1CC */
  0x0020,   /* U+1F1CD */
  0x0020,   /* U+1F1CE */
  0x0020,   /* U+1F1CF */
  0x0020,   /* U+1F1D0 */
  0x0020,   /* U+1F1D1 */
  0x0020,   /* U+1F1D2 */
  0x0020,   /* U+1F1D3 */
  0x0020,   /* U+1F1D4 */
  0x0020,   /* U+1F1D5 */
  0x0020,   /* U+1F1D6 */
  0x0020,   /* U+1F1D7 */
  0x0020,   /* U+1F1D8 */
  0x0020,   /* U+1F1D9 */
  0x0020,   /* U+1F1DA */
  0x0020,   /* U+1F1DB */
  0x0020,   /* U+1F1DC */
  0x0020,   /* U+1F1DD */
  0x0020,   /* U+1F1DE */
  0x0020,   /* U+1F1DF */
  0x0020,   /* U+1F1E0 */
  0x0020,   /* U+1F1E1 */
  0x0020,   /* U+1F1E2 */
  0x0020,   /* U+1F1E3 */
  0x0020,   /* U+1F1E4 */
  0x0020,   /* U+1F1E5 */
  0x0020,   /* U+1F1E6 */
  0x0020,   /* U+1F1E7 */
  0x0020,   /* U+1F1E8 */
  0x0020,   /* U+1F1E9 */
  0x0020,   /* U+1F1EA */
  0x0020,   /* U+1F1EB */
  0x0020,   /* U+1F1EC */
  0x0020,   /* U+1F1ED */
  0x0020,   /* U+1F1EE */
  0x0020,   /* U+1F1EF */
  0x0020,   /* U+1F1F0 */
  0x0020,   /* U+1F1F1 */
  0x0020,   /* U+1F1F2 */
  0x0020,   /* U+1F1F3 */
  0x0020,   /* U+1F1F4 */
  0x0020,   /* U+1F1F5 */
  0x0020,   /* U+1F1F6 */
  0x0020,   /* U+1F1F7 */
  0x0020,   /* U+1F1F8 */
  0x0020,   /* U+1F1F9 */
  0x0020,   /* U+1F1FA */
  0x0020,   /* U+1F1FB */
  0x0020,   /* U+1F1FC */
  0x0020,   /* U+1F1FD */
  0x0020,   /* U+1F1FE */
  0x0020,   /* U+1F1FF */

  /* Tertiary weight 1 for each character. */
  0x0004,   /* U+1F100 */
  0x0004,   /* U+1F101 */
  0x0004,   /* U+1F102 */
  0x0004,   /* U+1F103 */
  0x0004,   /* U+1F104 */
  0x0004,   /* U+1F105 */
  0x0004,   /* U+1F106 */
  0x0004,   /* U+1F107 */
  0x0004,   /* U+1F108 */
  0x0004,   /* U+1F109 */
  0x0004,   /* U+1F10A */
  0x0006,   /* U+1F10B */
  0x0006,   /* U+1F10C */
  0x0002,   /* U+1F10D */
  0x0002,   /* U+1F10E */
  0x0002,   /* U+1F10F */
  0x0004,   /* U+1F110 */
  0x0004,   /* U+1F111 */
  0x0004,   /* U+1F112 */
  0x0004,   /* U+1F113 */
  0x0004,   /* U+1F114 */
  0x0004,   /* U+1F115 */
  0x0004,   /* U+1F116 */
  0x0004,   /* U+1F117 */
  0x0004,   /* U+1F118 */
  0x0004,   /* U+1F119 */
  0x0004,   /* U+1F11A */
  0x0004,   /* U+1F11B */
  0x0004,   /* U+1F11C */
  0x0004,   /* U+1F11D */
  0x0004,   /* U+1F11E */
  0x0004,   /* U+1F11F */
  0x0004,   /* U+1F120 */
  0x0004,   /* U+1F121 */
  0x0004,   /* U+1F122 */
  0x0004,   /* U+1F123 */
  0x0004,   /* U+1F124 */
  0x0004,   /* U+1F125 */
  0x0004,   /* U+1F126 */
  0x0004,   /* U+1F127 */
  0x0004,   /* U+1F128 */
  0x0004,   /* U+1F129 */
  0x0004,   /* U+1F12A */
  0x000C,   /* U+1F12B */
  0x000C,   /* U+1F12C */
  0x000C,   /* U+1F12D */
  0x000C,   /* U+1F12E */
  0x0002,   /* U+1F12F */
  0x001D,   /* U+1F130 */
  0x001D,   /* U+1F131 */
  0x001D,   /* U+1F132 */
  0x001D,   /* U+1F133 */
  0x001D,   /* U+1F134 */
  0x001D,   /* U+1F135 */
  0x001D,   /* U+1F136 */
  0x001D,   /* U+1F137 */
  0x001D,   /* U+1F138 */
  0x001D,   /* U+1F139 */
  0x001D,   /* U+1F13A */
  0x001D,   /* U+1F13B */
  0x001D,   /* U+1F13C */
  0x001D,   /* U+1F13D */
  0x001D,   /* U+1F13E */
  0x001D,   /* U+1F13F */
  0x001D,   /* U+1F140 */
  0x001D,   /* U+1F141 */
  0x001D,   /* U+1F142 */
  0x001D,   /* U+1F143 */
  0x001D,   /* U+1F144 */
  0x001D,   /* U+1F145 */
  0x001D,   /* U+1F146 */
  0x001D,   /* U+1F147 */
  0x001D,   /* U+1F148 */
  0x001D,   /* U+1F149 */
  0x001D,   /* U+1F14A */
  0x001D,   /* U+1F14B */
  0x001D,   /* U+1F14C */
  0x001D,   /* U+1F14D */
  0x001D,   /* U+1F14E */
  0x001D,   /* U+1F14F */
  0x000C,   /* U+1F150 */
  0x000C,   /* U+1F151 */
  0x000C,   /* U+1F152 */
  0x000C,   /* U+1F153 */
  0x000C,   /* U+1F154 */
  0x000C,   /* U+1F155 */
  0x000C,   /* U+1F156 */
  0x000C,   /* U+1F157 */
  0x000C,   /* U+1F158 */
  0x000C,   /* U+1F159 */
  0x000C,   /* U+1F15A */
  0x000C,   /* U+1F15B */
  0x000C,   /* U+1F15C */
  0x000C,   /* U+1F15D */
  0x000C,   /* U+1F15E */
  0x000C,   /* U+1F15F */
  0x000C,   /* U+1F160 */
  0x000C,   /* U+1F161 */
  0x000C,   /* U+1F162 */
  0x000C,   /* U+1F163 */
  0x000C,   /* U+1F164 */
  0x000C,   /* U+1F165 */
  0x000C,   /* U+1F166 */
  0x000C,   /* U+1F167 */
  0x000C,   /* U+1F168 */
  0x000C,   /* U+1F169 */
  0x0014,   /* U+1F16A */
  0x0014,   /* U+1F16B */
  0x0002,   /* U+1F16C */
  0x0002,   /* U+1F16D */
  0x0002,   /* U+1F16E */
  0x0002,   /* U+1F16F */
  0x001D,   /* U+1F170 */
  0x001D,   /* U+1F171 */
  0x001D,   /* U+1F172 */
  0x001D,   /* U+1F173 */
  0x001D,   /* U+1F174 */
  0x001D,   /* U+1F175 */
  0x001D,   /* U+1F176 */
  0x001D,   /* U+1F177 */
  0x001D,   /* U+1F178 */
  0x001D,   /* U+1F179 */
  0x001D,   /* U+1F17A */
  0x001D,   /* U+1F17B */
  0x001D,   /* U+1F17C */
  0x001D,   /* U+1F17D */
  0x001D,   /* U+1F17E */
  0x001D,   /* U+1F17F */
  0x001D,   /* U+1F180 */
  0x001D,   /* U+1F181 */
  0x001D,   /* U+1F182 */
  0x001D,   /* U+1F183 */
  0x001D,   /* U+1F184 */
  0x001D,   /* U+1F185 */
  0x001D,   /* U+1F186 */
  0x001D,   /* U+1F187 */
  0x001D,   /* U+1F188 */
  0x001D,   /* U+1F189 */
  0x001D,   /* U+1F18A */
  0x001D,   /* U+1F18B */
  0x001D,   /* U+1F18C */
  0x001D,   /* U+1F18D */
  0x001D,   /* U+1F18E */
  0x001D,   /* U+1F18F */
  0x001D,   /* U+1F190 */
  0x001D,   /* U+1F191 */
  0x001D,   /* U+1F192 */
  0x001D,   /* U+1F193 */
  0x001D,   /* U+1F194 */
  0x001D,   /* U+1F195 */
  0x001D,   /* U+1F196 */
  0x001D,   /* U+1F197 */
  0x001D,   /* U+1F198 */
  0x001D,   /* U+1F199 */
  0x001D,   /* U+1F19A */
  0x001C,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x001C,   /* U+1F19D */
  0x001C,   /* U+1F19E */
  0x001C,   /* U+1F19F */
  0x001C,   /* U+1F1A0 */
  0x001C,   /* U+1F1A1 */
  0x001C,   /* U+1F1A2 */
  0x001C,   /* U+1F1A3 */
  0x001C,   /* U+1F1A4 */
  0x001C,   /* U+1F1A5 */
  0x001D,   /* U+1F1A6 */
  0x001D,   /* U+1F1A7 */
  0x001D,   /* U+1F1A8 */
  0x001D,   /* U+1F1A9 */
  0x001D,   /* U+1F1AA */
  0x001D,   /* U+1F1AB */
  0x001D,   /* U+1F1AC */
  0x0002,   /* U+1F1AD */
  0x0002,   /* U+1F1AE */
  0x0002,   /* U+1F1AF */
  0x0002,   /* U+1F1B0 */
  0x0002,   /* U+1F1B1 */
  0x0002,   /* U+1F1B2 */
  0x0002,   /* U+1F1B3 */
  0x0002,   /* U+1F1B4 */
  0x0002,   /* U+1F1B5 */
  0x0002,   /* U+1F1B6 */
  0x0002,   /* U+1F1B7 */
  0x0002,   /* U+1F1B8 */
  0x0002,   /* U+1F1B9 */
  0x0002,   /* U+1F1BA */
  0x0002,   /* U+1F1BB */
  0x0002,   /* U+1F1BC */
  0x0002,   /* U+1F1BD */
  0x0002,   /* U+1F1BE */
  0x0002,   /* U+1F1BF */
  0x0002,   /* U+1F1C0 */
  0x0002,   /* U+1F1C1 */
  0x0002,   /* U+1F1C2 */
  0x0002,   /* U+1F1C3 */
  0x0002,   /* U+1F1C4 */
  0x0002,   /* U+1F1C5 */
  0x0002,   /* U+1F1C6 */
  0x0002,   /* U+1F1C7 */
  0x0002,   /* U+1F1C8 */
  0x0002,   /* U+1F1C9 */
  0x0002,   /* U+1F1CA */
  0x0002,   /* U+1F1CB */
  0x0002,   /* U+1F1CC */
  0x0002,   /* U+1F1CD */
  0x0002,   /* U+1F1CE */
  0x0002,   /* U+1F1CF */
  0x0002,   /* U+1F1D0 */
  0x0002,   /* U+1F1D1 */
  0x0002,   /* U+1F1D2 */
  0x0002,   /* U+1F1D3 */
  0x0002,   /* U+1F1D4 */
  0x0002,   /* U+1F1D5 */
  0x0002,   /* U+1F1D6 */
  0x0002,   /* U+1F1D7 */
  0x0002,   /* U+1F1D8 */
  0x0002,   /* U+1F1D9 */
  0x0002,   /* U+1F1DA */
  0x0002,   /* U+1F1DB */
  0x0002,   /* U+1F1DC */
  0x0002,   /* U+1F1DD */
  0x0002,   /* U+1F1DE */
  0x0002,   /* U+1F1DF */
  0x0002,   /* U+1F1E0 */
  0x0002,   /* U+1F1E1 */
  0x0002,   /* U+1F1E2 */
  0x0002,   /* U+1F1E3 */
  0x0002,   /* U+1F1E4 */
  0x0002,   /* U+1F1E5 */
  0x0002,   /* U+1F1E6 */
  0x0002,   /* U+1F1E7 */
  0x0002,   /* U+1F1E8 */
  0x0002,   /* U+1F1E9 */
  0x0002,   /* U+1F1EA */
  0x0002,   /* U+1F1EB */
  0x0002,   /* U+1F1EC */
  0x0002,   /* U+1F1ED */
  0x0002,   /* U+1F1EE */
  0x0002,   /* U+1F1EF */
  0x0002,   /* U+1F1F0 */
  0x0002,   /* U+1F1F1 */
  0x0002,   /* U+1F1F2 */
  0x0002,   /* U+1F1F3 */
  0x0002,   /* U+1F1F4 */
  0x0002,   /* U+1F1F5 */
  0x0002,   /* U+1F1F6 */
  0x0002,   /* U+1F1F7 */
  0x0002,   /* U+1F1F8 */
  0x0002,   /* U+1F1F9 */
  0x0002,   /* U+1F1FA */
  0x0002,   /* U+1F1FB */
  0x0002,   /* U+1F1FC */
  0x0002,   /* U+1F1FD */
  0x0002,   /* U+1F1FE */
  0x0002,   /* U+1F1FF */

  /* Primary weight 2 for each character. */
  0x0277,   /* U+1F100 */
  0x0222,   /* U+1F101 */
  0x0222,   /* U+1F102 */
  0x0222,   /* U+1F103 */
  0x0222,   /* U+1F104 */
  0x0222,   /* U+1F105 */
  0x0222,   /* U+1F106 */
  0x0222,   /* U+1F107 */
  0x0222,   /* U+1F108 */
  0x0222,   /* U+1F109 */
  0x0222,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0xF10D,   /* U+1F10D */
  0xF10E,   /* U+1F10E */
  0xF10F,   /* U+1F10F */
  0x1C47,   /* U+1F110 */
  0x1C60,   /* U+1F111 */
  0x1C7A,   /* U+1F112 */
  0x1C8F,   /* U+1F113 */
  0x1CAA,   /* U+1F114 */
  0x1CE5,   /* U+1F115 */
  0x1CF4,   /* U+1F116 */
  0x1D18,   /* U+1F117 */
  0x1D32,   /* U+1F118 */
  0x1D4C,   /* U+1F119 */
  0x1D65,   /* U+1F11A */
  0x1D77,   /* U+1F11B */
  0x1DAA,   /* U+1F11C */
  0x1DB9,   /* U+1F11D */
  0x1DDD,   /* U+1F11E */
  0x1E0C,   /* U+1F11F */
  0x1E21,   /* U+1F120 */
  0x1E33,   /* U+1F121 */
  0x1E71,   /* U+1F122 */
  0x1E95,   /* U+1F123 */
  0x1EB5,   /* U+1F124 */
  0x1EE3,   /* U+1F125 */
  0x1EF5,   /* U+1F126 */
  0x1EFF,   /* U+1F127 */
  0x1F0B,   /* U+1F128 */
  0x1F21,   /* U+1F129 */
  0x1E71,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x1C8F,   /* U+1F12D */
  0x1F21,   /* U+1F12E */
  0xF12F,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x1EE3,   /* U+1F14A */
  0x1EE3,   /* U+1F14B */
  0x1C8F,   /* U+1F14C */
  0x1E71,   /* U+1F14D */
  0x1E0C,   /* U+1F14E */
  0x1C7A,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x1C7A,   /* U+1F16A */
  0x1C8F,   /* U+1F16B */
  0xF16C,   /* U+1F16C */
  0xF16D,   /* U+1F16D */
  0xF16E,   /* U+1F16E */
  0xF16F,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x1C7A,   /* U+1F18B */
  0x1C47,   /* U+1F18C */
  0x1C47,   /* U+1F18D */
  0x1C60,   /* U+1F18E */
  0x1C7A,   /* U+1F18F */
  0x1D4C,   /* U+1F190 */
  0x1D77,   /* U+1F191 */
  0x1DDD,   /* U+1F192 */
  0x1E33,   /* U+1F193 */
  0x1C8F,   /* U+1F194 */
  0x1CAA,   /* U+1F195 */
  0x1CF4,   /* U+1F196 */
  0x1D65,   /* U+1F197 */
  0x1DDD,   /* U+1F198 */
  0x1E0C,   /* U+1F199 */
  0x1E71,   /* U+1F19A */
  0x1C8F,   /* U+1F19B */
  0x1DB9,   /* U+1F19C */
  0x1D65,   /* U+1F19D */
  0x1D65,   /* U+1F19E */
  0x1D65,   /* U+1F19F */
  0x0277,   /* U+1F1A0 */
  0x0277,   /* U+1F1A1 */
  0x1C3F,   /* U+1F1A2 */
  0x1C3D,   /* U+1F1A3 */
  0x1C3F,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x1C7A,   /* U+1F1A6 */
  0x1C8F,   /* U+1F1A7 */
  0x1D32,   /* U+1F1A8 */
  0x1DDD,   /* U+1F1A9 */
  0x1D18,   /* U+1F1AA */
  0x1D18,   /* U+1F1AB */
  0x1DDD,   /* U+1F1AC */
  0xF1AD,   /* U+1F1AD */
  0xF1AE,   /* U+1F1AE */
  0xF1AF,   /* U+1F1AF */
  0xF1B0,   /* U+1F1B0 */
  0xF1B1,   /* U+1F1B1 */
  0xF1B2,   /* U+1F1B2 */
  0xF1B3,   /* U+1F1B3 */
  0xF1B4,   /* U+1F1B4 */
  0xF1B5,   /* U+1F1B5 */
  0xF1B6,   /* U+1F1B6 */
  0xF1B7,   /* U+1F1B7 */
  0xF1B8,   /* U+1F1B8 */
  0xF1B9,   /* U+1F1B9 */
  0xF1BA,   /* U+1F1BA */
  0xF1BB,   /* U+1F1BB */
  0xF1BC,   /* U+1F1BC */
  0xF1BD,   /* U+1F1BD */
  0xF1BE,   /* U+1F1BE */
  0xF1BF,   /* U+1F1BF */
  0xF1C0,   /* U+1F1C0 */
  0xF1C1,   /* U+1F1C1 */
  0xF1C2,   /* U+1F1C2 */
  0xF1C3,   /* U+1F1C3 */
  0xF1C4,   /* U+1F1C4 */
  0xF1C5,   /* U+1F1C5 */
  0xF1C6,   /* U+1F1C6 */
  0xF1C7,   /* U+1F1C7 */
  0xF1C8,   /* U+1F1C8 */
  0xF1C9,   /* U+1F1C9 */
  0xF1CA,   /* U+1F1CA */
  0xF1CB,   /* U+1F1CB */
  0xF1CC,   /* U+1F1CC */
  0xF1CD,   /* U+1F1CD */
  0xF1CE,   /* U+1F1CE */
  0xF1CF,   /* U+1F1CF */
  0xF1D0,   /* U+1F1D0 */
  0xF1D1,   /* U+1F1D1 */
  0xF1D2,   /* U+1F1D2 */
  0xF1D3,   /* U+1F1D3 */
  0xF1D4,   /* U+1F1D4 */
  0xF1D5,   /* U+1F1D5 */
  0xF1D6,   /* U+1F1D6 */
  0xF1D7,   /* U+1F1D7 */
  0xF1D8,   /* U+1F1D8 */
  0xF1D9,   /* U+1F1D9 */
  0xF1DA,   /* U+1F1DA */
  0xF1DB,   /* U+1F1DB */
  0xF1DC,   /* U+1F1DC */
  0xF1DD,   /* U+1F1DD */
  0xF1DE,   /* U+1F1DE */
  0xF1DF,   /* U+1F1DF */
  0xF1E0,   /* U+1F1E0 */
  0xF1E1,   /* U+1F1E1 */
  0xF1E2,   /* U+1F1E2 */
  0xF1E3,   /* U+1F1E3 */
  0xF1E4,   /* U+1F1E4 */
  0xF1E5,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+1F100 */
  0x0020,   /* U+1F101 */
  0x0020,   /* U+1F102 */
  0x0020,   /* U+1F103 */
  0x0020,   /* U+1F104 */
  0x0020,   /* U+1F105 */
  0x0020,   /* U+1F106 */
  0x0020,   /* U+1F107 */
  0x0020,   /* U+1F108 */
  0x0020,   /* U+1F109 */
  0x0020,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0020,   /* U+1F110 */
  0x0020,   /* U+1F111 */
  0x0020,   /* U+1F112 */
  0x0020,   /* U+1F113 */
  0x0020,   /* U+1F114 */
  0x0020,   /* U+1F115 */
  0x0020,   /* U+1F116 */
  0x0020,   /* U+1F117 */
  0x0020,   /* U+1F118 */
  0x0020,   /* U+1F119 */
  0x0020,   /* U+1F11A */
  0x0020,   /* U+1F11B */
  0x0020,   /* U+1F11C */
  0x0020,   /* U+1F11D */
  0x0020,   /* U+1F11E */
  0x0020,   /* U+1F11F */
  0x0020,   /* U+1F120 */
  0x0020,   /* U+1F121 */
  0x0020,   /* U+1F122 */
  0x0020,   /* U+1F123 */
  0x0020,   /* U+1F124 */
  0x0020,   /* U+1F125 */
  0x0020,   /* U+1F126 */
  0x0020,   /* U+1F127 */
  0x0020,   /* U+1F128 */
  0x0020,   /* U+1F129 */
  0x0020,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0020,   /* U+1F12D */
  0x0020,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0020,   /* U+1F14A */
  0x0020,   /* U+1F14B */
  0x0020,   /* U+1F14C */
  0x0020,   /* U+1F14D */
  0x0020,   /* U+1F14E */
  0x0020,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0020,   /* U+1F16A */
  0x0020,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0020,   /* U+1F18B */
  0x0020,   /* U+1F18C */
  0x0020,   /* U+1F18D */
  0x0020,   /* U+1F18E */
  0x0020,   /* U+1F18F */
  0x0020,   /* U+1F190 */
  0x0020,   /* U+1F191 */
  0x0020,   /* U+1F192 */
  0x0020,   /* U+1F193 */
  0x0020,   /* U+1F194 */
  0x0020,   /* U+1F195 */
  0x0020,   /* U+1F196 */
  0x0020,   /* U+1F197 */
  0x0020,   /* U+1F198 */
  0x0020,   /* U+1F199 */
  0x0020,   /* U+1F19A */
  0x0020,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0020,   /* U+1F19D */
  0x0020,   /* U+1F19E */
  0x0020,   /* U+1F19F */
  0x0020,   /* U+1F1A0 */
  0x0020,   /* U+1F1A1 */
  0x0020,   /* U+1F1A2 */
  0x0020,   /* U+1F1A3 */
  0x0020,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0020,   /* U+1F1A6 */
  0x0020,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0020,   /* U+1F1AA */
  0x0020,   /* U+1F1AB */
  0x0020,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 2 for each character. */
  0x0004,   /* U+1F100 */
  0x0004,   /* U+1F101 */
  0x0004,   /* U+1F102 */
  0x0004,   /* U+1F103 */
  0x0004,   /* U+1F104 */
  0x0004,   /* U+1F105 */
  0x0004,   /* U+1F106 */
  0x0004,   /* U+1F107 */
  0x0004,   /* U+1F108 */
  0x0004,   /* U+1F109 */
  0x0004,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x000A,   /* U+1F110 */
  0x000A,   /* U+1F111 */
  0x000A,   /* U+1F112 */
  0x000A,   /* U+1F113 */
  0x000A,   /* U+1F114 */
  0x000A,   /* U+1F115 */
  0x000A,   /* U+1F116 */
  0x000A,   /* U+1F117 */
  0x000A,   /* U+1F118 */
  0x000A,   /* U+1F119 */
  0x000A,   /* U+1F11A */
  0x000A,   /* U+1F11B */
  0x000A,   /* U+1F11C */
  0x000A,   /* U+1F11D */
  0x000A,   /* U+1F11E */
  0x000A,   /* U+1F11F */
  0x000A,   /* U+1F120 */
  0x000A,   /* U+1F121 */
  0x000A,   /* U+1F122 */
  0x000A,   /* U+1F123 */
  0x000A,   /* U+1F124 */
  0x000A,   /* U+1F125 */
  0x000A,   /* U+1F126 */
  0x000A,   /* U+1F127 */
  0x000A,   /* U+1F128 */
  0x000A,   /* U+1F129 */
  0x000A,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x000C,   /* U+1F12D */
  0x000C,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x001D,   /* U+1F14A */
  0x001D,   /* U+1F14B */
  0x001D,   /* U+1F14C */
  0x001D,   /* U+1F14D */
  0x001D,   /* U+1F14E */
  0x001D,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0014,   /* U+1F16A */
  0x0014,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x001D,   /* U+1F18B */
  0x001D,   /* U+1F18C */
  0x001D,   /* U+1F18D */
  0x001D,   /* U+1F18E */
  0x001D,   /* U+1F18F */
  0x001D,   /* U+1F190 */
  0x001D,   /* U+1F191 */
  0x001D,   /* U+1F192 */
  0x001D,   /* U+1F193 */
  0x001D,   /* U+1F194 */
  0x001D,   /* U+1F195 */
  0x001D,   /* U+1F196 */
  0x001D,   /* U+1F197 */
  0x001D,   /* U+1F198 */
  0x001D,   /* U+1F199 */
  0x001D,   /* U+1F19A */
  0x001D,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x001D,   /* U+1F19D */
  0x001D,   /* U+1F19E */
  0x001D,   /* U+1F19F */
  0x001C,   /* U+1F1A0 */
  0x001C,   /* U+1F1A1 */
  0x001C,   /* U+1F1A2 */
  0x001C,   /* U+1F1A3 */
  0x001C,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x001C,   /* U+1F1A6 */
  0x001C,   /* U+1F1A7 */
  0x001C,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x001D,   /* U+1F1AA */
  0x001D,   /* U+1F1AB */
  0x001D,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0318,   /* U+1F110 */
  0x0318,   /* U+1F111 */
  0x0318,   /* U+1F112 */
  0x0318,   /* U+1F113 */
  0x0318,   /* U+1F114 */
  0x0318,   /* U+1F115 */
  0x0318,   /* U+1F116 */
  0x0318,   /* U+1F117 */
  0x0318,   /* U+1F118 */
  0x0318,   /* U+1F119 */
  0x0318,   /* U+1F11A */
  0x0318,   /* U+1F11B */
  0x0318,   /* U+1F11C */
  0x0318,   /* U+1F11D */
  0x0318,   /* U+1F11E */
  0x0318,   /* U+1F11F */
  0x0318,   /* U+1F120 */
  0x0318,   /* U+1F121 */
  0x0318,   /* U+1F122 */
  0x0318,   /* U+1F123 */
  0x0318,   /* U+1F124 */
  0x0318,   /* U+1F125 */
  0x0318,   /* U+1F126 */
  0x0318,   /* U+1F127 */
  0x0318,   /* U+1F128 */
  0x0318,   /* U+1F129 */
  0x037A,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x1EE3,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x1DDD,   /* U+1F192 */
  0x1CAA,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x1EF5,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x1E71,   /* U+1F198 */
  0x0260,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x1C8F,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x1C3E,   /* U+1F1A0 */
  0x1C3E,   /* U+1F1A1 */
  0x0277,   /* U+1F1A2 */
  0x1E0C,   /* U+1F1A3 */
  0x1C3D,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x1E33,   /* U+1F1A7 */
  0x0209,   /* U+1F1A8 */
  0x1E71,   /* U+1F1A9 */
  0x1EE3,   /* U+1F1AA */
  0x1C8F,   /* U+1F1AB */
  0x1C8F,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0020,   /* U+1F110 */
  0x0020,   /* U+1F111 */
  0x0020,   /* U+1F112 */
  0x0020,   /* U+1F113 */
  0x0020,   /* U+1F114 */
  0x0020,   /* U+1F115 */
  0x0020,   /* U+1F116 */
  0x0020,   /* U+1F117 */
  0x0020,   /* U+1F118 */
  0x0020,   /* U+1F119 */
  0x0020,   /* U+1F11A */
  0x0020,   /* U+1F11B */
  0x0020,   /* U+1F11C */
  0x0020,   /* U+1F11D */
  0x0020,   /* U+1F11E */
  0x0020,   /* U+1F11F */
  0x0020,   /* U+1F120 */
  0x0020,   /* U+1F121 */
  0x0020,   /* U+1F122 */
  0x0020,   /* U+1F123 */
  0x0020,   /* U+1F124 */
  0x0020,   /* U+1F125 */
  0x0020,   /* U+1F126 */
  0x0020,   /* U+1F127 */
  0x0020,   /* U+1F128 */
  0x0020,   /* U+1F129 */
  0x0020,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0020,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0020,   /* U+1F192 */
  0x0020,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0020,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0020,   /* U+1F198 */
  0x0020,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0020,   /* U+1F1A0 */
  0x0020,   /* U+1F1A1 */
  0x0020,   /* U+1F1A2 */
  0x0020,   /* U+1F1A3 */
  0x0020,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0020,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0020,   /* U+1F1AA */
  0x0020,   /* U+1F1AB */
  0x0020,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0004,   /* U+1F110 */
  0x0004,   /* U+1F111 */
  0x0004,   /* U+1F112 */
  0x0004,   /* U+1F113 */
  0x0004,   /* U+1F114 */
  0x0004,   /* U+1F115 */
  0x0004,   /* U+1F116 */
  0x0004,   /* U+1F117 */
  0x0004,   /* U+1F118 */
  0x0004,   /* U+1F119 */
  0x0004,   /* U+1F11A */
  0x0004,   /* U+1F11B */
  0x0004,   /* U+1F11C */
  0x0004,   /* U+1F11D */
  0x0004,   /* U+1F11E */
  0x0004,   /* U+1F11F */
  0x0004,   /* U+1F120 */
  0x0004,   /* U+1F121 */
  0x0004,   /* U+1F122 */
  0x0004,   /* U+1F123 */
  0x0004,   /* U+1F124 */
  0x0004,   /* U+1F125 */
  0x0004,   /* U+1F126 */
  0x0004,   /* U+1F127 */
  0x0004,   /* U+1F128 */
  0x0004,   /* U+1F129 */
  0x0004,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x001D,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x001D,   /* U+1F192 */
  0x001D,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x001D,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x001D,   /* U+1F198 */
  0x001C,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x001C,   /* U+1F1A0 */
  0x001C,   /* U+1F1A1 */
  0x001C,   /* U+1F1A2 */
  0x001D,   /* U+1F1A3 */
  0x001C,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x001D,   /* U+1F1A7 */
  0x001C,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x001D,   /* U+1F1AA */
  0x001D,   /* U+1F1AB */
  0x001D,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x1D77,   /* U+1F192 */
  0x1CAA,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0209,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x1C3F,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x1E0C,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x1E33,   /* U+1F1A8 */
  0x1E71,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0020,   /* U+1F192 */
  0x0020,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0020,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0020,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x001D,   /* U+1F192 */
  0x001D,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x001C,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x001D,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x001D,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 5 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x1E71,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x1CAA,   /* U+1F1A8 */
  0x1D77,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 5 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 5 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x001D,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x001C,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 6 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x1C7A,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x1E71,   /* U+1F1A8 */
  0x1CAA,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 6 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0020,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 6 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x001C,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 7 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x1E33,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x1E71,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 7 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0020,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 7 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x001C,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Primary weight 8 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0000,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x1E71,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Secondary weight 8 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0000,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x0020,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */

  /* Tertiary weight 8 for each character. */
  0x0000,   /* U+1F100 */
  0x0000,   /* U+1F101 */
  0x0000,   /* U+1F102 */
  0x0000,   /* U+1F103 */
  0x0000,   /* U+1F104 */
  0x0000,   /* U+1F105 */
  0x0000,   /* U+1F106 */
  0x0000,   /* U+1F107 */
  0x0000,   /* U+1F108 */
  0x0000,   /* U+1F109 */
  0x0000,   /* U+1F10A */
  0x0000,   /* U+1F10B */
  0x0000,   /* U+1F10C */
  0x0000,   /* U+1F10D */
  0x0000,   /* U+1F10E */
  0x0000,   /* U+1F10F */
  0x0000,   /* U+1F110 */
  0x0000,   /* U+1F111 */
  0x0000,   /* U+1F112 */
  0x0000,   /* U+1F113 */
  0x0000,   /* U+1F114 */
  0x0000,   /* U+1F115 */
  0x0000,   /* U+1F116 */
  0x0000,   /* U+1F117 */
  0x0000,   /* U+1F118 */
  0x0000,   /* U+1F119 */
  0x0000,   /* U+1F11A */
  0x0000,   /* U+1F11B */
  0x0000,   /* U+1F11C */
  0x0000,   /* U+1F11D */
  0x0000,   /* U+1F11E */
  0x0000,   /* U+1F11F */
  0x0000,   /* U+1F120 */
  0x0000,   /* U+1F121 */
  0x0000,   /* U+1F122 */
  0x0000,   /* U+1F123 */
  0x0000,   /* U+1F124 */
  0x0000,   /* U+1F125 */
  0x0000,   /* U+1F126 */
  0x0000,   /* U+1F127 */
  0x0000,   /* U+1F128 */
  0x0000,   /* U+1F129 */
  0x0000,   /* U+1F12A */
  0x0000,   /* U+1F12B */
  0x0000,   /* U+1F12C */
  0x0000,   /* U+1F12D */
  0x0000,   /* U+1F12E */
  0x0000,   /* U+1F12F */
  0x0000,   /* U+1F130 */
  0x0000,   /* U+1F131 */
  0x0000,   /* U+1F132 */
  0x0000,   /* U+1F133 */
  0x0000,   /* U+1F134 */
  0x0000,   /* U+1F135 */
  0x0000,   /* U+1F136 */
  0x0000,   /* U+1F137 */
  0x0000,   /* U+1F138 */
  0x0000,   /* U+1F139 */
  0x0000,   /* U+1F13A */
  0x0000,   /* U+1F13B */
  0x0000,   /* U+1F13C */
  0x0000,   /* U+1F13D */
  0x0000,   /* U+1F13E */
  0x0000,   /* U+1F13F */
  0x0000,   /* U+1F140 */
  0x0000,   /* U+1F141 */
  0x0000,   /* U+1F142 */
  0x0000,   /* U+1F143 */
  0x0000,   /* U+1F144 */
  0x0000,   /* U+1F145 */
  0x0000,   /* U+1F146 */
  0x0000,   /* U+1F147 */
  0x0000,   /* U+1F148 */
  0x0000,   /* U+1F149 */
  0x0000,   /* U+1F14A */
  0x0000,   /* U+1F14B */
  0x0000,   /* U+1F14C */
  0x0000,   /* U+1F14D */
  0x0000,   /* U+1F14E */
  0x0000,   /* U+1F14F */
  0x0000,   /* U+1F150 */
  0x0000,   /* U+1F151 */
  0x0000,   /* U+1F152 */
  0x0000,   /* U+1F153 */
  0x0000,   /* U+1F154 */
  0x0000,   /* U+1F155 */
  0x0000,   /* U+1F156 */
  0x0000,   /* U+1F157 */
  0x0000,   /* U+1F158 */
  0x0000,   /* U+1F159 */
  0x0000,   /* U+1F15A */
  0x0000,   /* U+1F15B */
  0x0000,   /* U+1F15C */
  0x0000,   /* U+1F15D */
  0x0000,   /* U+1F15E */
  0x0000,   /* U+1F15F */
  0x0000,   /* U+1F160 */
  0x0000,   /* U+1F161 */
  0x0000,   /* U+1F162 */
  0x0000,   /* U+1F163 */
  0x0000,   /* U+1F164 */
  0x0000,   /* U+1F165 */
  0x0000,   /* U+1F166 */
  0x0000,   /* U+1F167 */
  0x0000,   /* U+1F168 */
  0x0000,   /* U+1F169 */
  0x0000,   /* U+1F16A */
  0x0000,   /* U+1F16B */
  0x0000,   /* U+1F16C */
  0x0000,   /* U+1F16D */
  0x0000,   /* U+1F16E */
  0x0000,   /* U+1F16F */
  0x0000,   /* U+1F170 */
  0x0000,   /* U+1F171 */
  0x0000,   /* U+1F172 */
  0x0000,   /* U+1F173 */
  0x0000,   /* U+1F174 */
  0x0000,   /* U+1F175 */
  0x0000,   /* U+1F176 */
  0x0000,   /* U+1F177 */
  0x0000,   /* U+1F178 */
  0x0000,   /* U+1F179 */
  0x0000,   /* U+1F17A */
  0x0000,   /* U+1F17B */
  0x0000,   /* U+1F17C */
  0x0000,   /* U+1F17D */
  0x0000,   /* U+1F17E */
  0x0000,   /* U+1F17F */
  0x0000,   /* U+1F180 */
  0x0000,   /* U+1F181 */
  0x0000,   /* U+1F182 */
  0x0000,   /* U+1F183 */
  0x0000,   /* U+1F184 */
  0x0000,   /* U+1F185 */
  0x0000,   /* U+1F186 */
  0x0000,   /* U+1F187 */
  0x0000,   /* U+1F188 */
  0x0000,   /* U+1F189 */
  0x0000,   /* U+1F18A */
  0x0000,   /* U+1F18B */
  0x0000,   /* U+1F18C */
  0x0000,   /* U+1F18D */
  0x0000,   /* U+1F18E */
  0x0000,   /* U+1F18F */
  0x0000,   /* U+1F190 */
  0x0000,   /* U+1F191 */
  0x0000,   /* U+1F192 */
  0x0000,   /* U+1F193 */
  0x0000,   /* U+1F194 */
  0x0000,   /* U+1F195 */
  0x0000,   /* U+1F196 */
  0x0000,   /* U+1F197 */
  0x0000,   /* U+1F198 */
  0x0000,   /* U+1F199 */
  0x0000,   /* U+1F19A */
  0x0000,   /* U+1F19B */
  0x0000,   /* U+1F19C */
  0x0000,   /* U+1F19D */
  0x0000,   /* U+1F19E */
  0x0000,   /* U+1F19F */
  0x0000,   /* U+1F1A0 */
  0x0000,   /* U+1F1A1 */
  0x0000,   /* U+1F1A2 */
  0x0000,   /* U+1F1A3 */
  0x0000,   /* U+1F1A4 */
  0x0000,   /* U+1F1A5 */
  0x0000,   /* U+1F1A6 */
  0x0000,   /* U+1F1A7 */
  0x0000,   /* U+1F1A8 */
  0x001C,   /* U+1F1A9 */
  0x0000,   /* U+1F1AA */
  0x0000,   /* U+1F1AB */
  0x0000,   /* U+1F1AC */
  0x0000,   /* U+1F1AD */
  0x0000,   /* U+1F1AE */
  0x0000,   /* U+1F1AF */
  0x0000,   /* U+1F1B0 */
  0x0000,   /* U+1F1B1 */
  0x0000,   /* U+1F1B2 */
  0x0000,   /* U+1F1B3 */
  0x0000,   /* U+1F1B4 */
  0x0000,   /* U+1F1B5 */
  0x0000,   /* U+1F1B6 */
  0x0000,   /* U+1F1B7 */
  0x0000,   /* U+1F1B8 */
  0x0000,   /* U+1F1B9 */
  0x0000,   /* U+1F1BA */
  0x0000,   /* U+1F1BB */
  0x0000,   /* U+1F1BC */
  0x0000,   /* U+1F1BD */
  0x0000,   /* U+1F1BE */
  0x0000,   /* U+1F1BF */
  0x0000,   /* U+1F1C0 */
  0x0000,   /* U+1F1C1 */
  0x0000,   /* U+1F1C2 */
  0x0000,   /* U+1F1C3 */
  0x0000,   /* U+1F1C4 */
  0x0000,   /* U+1F1C5 */
  0x0000,   /* U+1F1C6 */
  0x0000,   /* U+1F1C7 */
  0x0000,   /* U+1F1C8 */
  0x0000,   /* U+1F1C9 */
  0x0000,   /* U+1F1CA */
  0x0000,   /* U+1F1CB */
  0x0000,   /* U+1F1CC */
  0x0000,   /* U+1F1CD */
  0x0000,   /* U+1F1CE */
  0x0000,   /* U+1F1CF */
  0x0000,   /* U+1F1D0 */
  0x0000,   /* U+1F1D1 */
  0x0000,   /* U+1F1D2 */
  0x0000,   /* U+1F1D3 */
  0x0000,   /* U+1F1D4 */
  0x0000,   /* U+1F1D5 */
  0x0000,   /* U+1F1D6 */
  0x0000,   /* U+1F1D7 */
  0x0000,   /* U+1F1D8 */
  0x0000,   /* U+1F1D9 */
  0x0000,   /* U+1F1DA */
  0x0000,   /* U+1F1DB */
  0x0000,   /* U+1F1DC */
  0x0000,   /* U+1F1DD */
  0x0000,   /* U+1F1DE */
  0x0000,   /* U+1F1DF */
  0x0000,   /* U+1F1E0 */
  0x0000,   /* U+1F1E1 */
  0x0000,   /* U+1F1E2 */
  0x0000,   /* U+1F1E3 */
  0x0000,   /* U+1F1E4 */
  0x0000,   /* U+1F1E5 */
  0x0000,   /* U+1F1E6 */
  0x0000,   /* U+1F1E7 */
  0x0000,   /* U+1F1E8 */
  0x0000,   /* U+1F1E9 */
  0x0000,   /* U+1F1EA */
  0x0000,   /* U+1F1EB */
  0x0000,   /* U+1F1EC */
  0x0000,   /* U+1F1ED */
  0x0000,   /* U+1F1EE */
  0x0000,   /* U+1F1EF */
  0x0000,   /* U+1F1F0 */
  0x0000,   /* U+1F1F1 */
  0x0000,   /* U+1F1F2 */
  0x0000,   /* U+1F1F3 */
  0x0000,   /* U+1F1F4 */
  0x0000,   /* U+1F1F5 */
  0x0000,   /* U+1F1F6 */
  0x0000,   /* U+1F1F7 */
  0x0000,   /* U+1F1F8 */
  0x0000,   /* U+1F1F9 */
  0x0000,   /* U+1F1FA */
  0x0000,   /* U+1F1FB */
  0x0000,   /* U+1F1FC */
  0x0000,   /* U+1F1FD */
  0x0000,   /* U+1F1FE */
  0x0000,   /* U+1F1FF */
};

uint16 uca900_p1F2[]= {
  /* Number of CEs for each character. */
  2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x3D78,   /* U+1F200 */
  0x3D64,   /* U+1F201 */
  0x3D65,   /* U+1F202 */
  0xFBC3,   /* U+1F203 */
  0xFBC3,   /* U+1F204 */
  0xFBC3,   /* U+1F205 */
  0xFBC3,   /* U+1F206 */
  0xFBC3,   /* U+1F207 */
  0xFBC3,   /* U+1F208 */
  0xFBC3,   /* U+1F209 */
  0xFBC3,   /* U+1F20A */
  0xFBC3,   /* U+1F20B */
  0xFBC3,   /* U+1F20C */
  0xFBC3,   /* U+1F20D */
  0xFBC3,   /* U+1F20E */
  0xFBC3,   /* U+1F20F */
  0xFB40,   /* U+1F210 */
  0xFB40,   /* U+1F211 */
  0xFB40,   /* U+1F212 */
  0x3D6D,   /* U+1F213 */
  0xFB40,   /* U+1F214 */
  0xFB40,   /* U+1F215 */
  0xFB41,   /* U+1F216 */
  0xFB40,   /* U+1F217 */
  0xFB40,   /* U+1F218 */
  0xFB40,   /* U+1F219 */
  0xFB40,   /* U+1F21A */
  0xFB40,   /* U+1F21B */
  0xFB40,   /* U+1F21C */
  0xFB40,   /* U+1F21D */
  0xFB40,   /* U+1F21E */
  0xFB40,   /* U+1F21F */
  0xFB40,   /* U+1F220 */
  0xFB40,   /* U+1F221 */
  0xFB40,   /* U+1F222 */
  0xFB41,   /* U+1F223 */
  0xFB40,   /* U+1F224 */
  0xFB40,   /* U+1F225 */
  0xFB40,   /* U+1F226 */
  0xFB40,   /* U+1F227 */
  0xFB40,   /* U+1F228 */
  0xFB40,   /* U+1F229 */
  0xFB40,   /* U+1F22A */
  0xFB41,   /* U+1F22B */
  0xFB40,   /* U+1F22C */
  0xFB40,   /* U+1F22D */
  0xFB40,   /* U+1F22E */
  0xFB40,   /* U+1F22F */
  0xFB41,   /* U+1F230 */
  0xFB40,   /* U+1F231 */
  0xFB40,   /* U+1F232 */
  0xFB40,   /* U+1F233 */
  0xFB40,   /* U+1F234 */
  0xFB40,   /* U+1F235 */
  0xFB40,   /* U+1F236 */
  0xFB40,   /* U+1F237 */
  0xFB40,   /* U+1F238 */
  0xFB40,   /* U+1F239 */
  0xFB40,   /* U+1F23A */
  0xFB41,   /* U+1F23B */
  0xFBC3,   /* U+1F23C */
  0xFBC3,   /* U+1F23D */
  0xFBC3,   /* U+1F23E */
  0xFBC3,   /* U+1F23F */
  0x0379,   /* U+1F240 */
  0x0379,   /* U+1F241 */
  0x0379,   /* U+1F242 */
  0x0379,   /* U+1F243 */
  0x0379,   /* U+1F244 */
  0x0379,   /* U+1F245 */
  0x0379,   /* U+1F246 */
  0x0379,   /* U+1F247 */
  0x0379,   /* U+1F248 */
  0xFBC3,   /* U+1F249 */
  0xFBC3,   /* U+1F24A */
  0xFBC3,   /* U+1F24B */
  0xFBC3,   /* U+1F24C */
  0xFBC3,   /* U+1F24D */
  0xFBC3,   /* U+1F24E */
  0xFBC3,   /* U+1F24F */
  0xFB40,   /* U+1F250 */
  0xFB40,   /* U+1F251 */
  0xFBC3,   /* U+1F252 */
  0xFBC3,   /* U+1F253 */
  0xFBC3,   /* U+1F254 */
  0xFBC3,   /* U+1F255 */
  0xFBC3,   /* U+1F256 */
  0xFBC3,   /* U+1F257 */
  0xFBC3,   /* U+1F258 */
  0xFBC3,   /* U+1F259 */
  0xFBC3,   /* U+1F25A */
  0xFBC3,   /* U+1F25B */
  0xFBC3,   /* U+1F25C */
  0xFBC3,   /* U+1F25D */
  0xFBC3,   /* U+1F25E */
  0xFBC3,   /* U+1F25F */
  0xFBC3,   /* U+1F260 */
  0xFBC3,   /* U+1F261 */
  0xFBC3,   /* U+1F262 */
  0xFBC3,   /* U+1F263 */
  0xFBC3,   /* U+1F264 */
  0xFBC3,   /* U+1F265 */
  0xFBC3,   /* U+1F266 */
  0xFBC3,   /* U+1F267 */
  0xFBC3,   /* U+1F268 */
  0xFBC3,   /* U+1F269 */
  0xFBC3,   /* U+1F26A */
  0xFBC3,   /* U+1F26B */
  0xFBC3,   /* U+1F26C */
  0xFBC3,   /* U+1F26D */
  0xFBC3,   /* U+1F26E */
  0xFBC3,   /* U+1F26F */
  0xFBC3,   /* U+1F270 */
  0xFBC3,   /* U+1F271 */
  0xFBC3,   /* U+1F272 */
  0xFBC3,   /* U+1F273 */
  0xFBC3,   /* U+1F274 */
  0xFBC3,   /* U+1F275 */
  0xFBC3,   /* U+1F276 */
  0xFBC3,   /* U+1F277 */
  0xFBC3,   /* U+1F278 */
  0xFBC3,   /* U+1F279 */
  0xFBC3,   /* U+1F27A */
  0xFBC3,   /* U+1F27B */
  0xFBC3,   /* U+1F27C */
  0xFBC3,   /* U+1F27D */
  0xFBC3,   /* U+1F27E */
  0xFBC3,   /* U+1F27F */
  0xFBC3,   /* U+1F280 */
  0xFBC3,   /* U+1F281 */
  0xFBC3,   /* U+1F282 */
  0xFBC3,   /* U+1F283 */
  0xFBC3,   /* U+1F284 */
  0xFBC3,   /* U+1F285 */
  0xFBC3,   /* U+1F286 */
  0xFBC3,   /* U+1F287 */
  0xFBC3,   /* U+1F288 */
  0xFBC3,   /* U+1F289 */
  0xFBC3,   /* U+1F28A */
  0xFBC3,   /* U+1F28B */
  0xFBC3,   /* U+1F28C */
  0xFBC3,   /* U+1F28D */
  0xFBC3,   /* U+1F28E */
  0xFBC3,   /* U+1F28F */
  0xFBC3,   /* U+1F290 */
  0xFBC3,   /* U+1F291 */
  0xFBC3,   /* U+1F292 */
  0xFBC3,   /* U+1F293 */
  0xFBC3,   /* U+1F294 */
  0xFBC3,   /* U+1F295 */
  0xFBC3,   /* U+1F296 */
  0xFBC3,   /* U+1F297 */
  0xFBC3,   /* U+1F298 */
  0xFBC3,   /* U+1F299 */
  0xFBC3,   /* U+1F29A */
  0xFBC3,   /* U+1F29B */
  0xFBC3,   /* U+1F29C */
  0xFBC3,   /* U+1F29D */
  0xFBC3,   /* U+1F29E */
  0xFBC3,   /* U+1F29F */
  0xFBC3,   /* U+1F2A0 */
  0xFBC3,   /* U+1F2A1 */
  0xFBC3,   /* U+1F2A2 */
  0xFBC3,   /* U+1F2A3 */
  0xFBC3,   /* U+1F2A4 */
  0xFBC3,   /* U+1F2A5 */
  0xFBC3,   /* U+1F2A6 */
  0xFBC3,   /* U+1F2A7 */
  0xFBC3,   /* U+1F2A8 */
  0xFBC3,   /* U+1F2A9 */
  0xFBC3,   /* U+1F2AA */
  0xFBC3,   /* U+1F2AB */
  0xFBC3,   /* U+1F2AC */
  0xFBC3,   /* U+1F2AD */
  0xFBC3,   /* U+1F2AE */
  0xFBC3,   /* U+1F2AF */
  0xFBC3,   /* U+1F2B0 */
  0xFBC3,   /* U+1F2B1 */
  0xFBC3,   /* U+1F2B2 */
  0xFBC3,   /* U+1F2B3 */
  0xFBC3,   /* U+1F2B4 */
  0xFBC3,   /* U+1F2B5 */
  0xFBC3,   /* U+1F2B6 */
  0xFBC3,   /* U+1F2B7 */
  0xFBC3,   /* U+1F2B8 */
  0xFBC3,   /* U+1F2B9 */
  0xFBC3,   /* U+1F2BA */
  0xFBC3,   /* U+1F2BB */
  0xFBC3,   /* U+1F2BC */
  0xFBC3,   /* U+1F2BD */
  0xFBC3,   /* U+1F2BE */
  0xFBC3,   /* U+1F2BF */
  0xFBC3,   /* U+1F2C0 */
  0xFBC3,   /* U+1F2C1 */
  0xFBC3,   /* U+1F2C2 */
  0xFBC3,   /* U+1F2C3 */
  0xFBC3,   /* U+1F2C4 */
  0xFBC3,   /* U+1F2C5 */
  0xFBC3,   /* U+1F2C6 */
  0xFBC3,   /* U+1F2C7 */
  0xFBC3,   /* U+1F2C8 */
  0xFBC3,   /* U+1F2C9 */
  0xFBC3,   /* U+1F2CA */
  0xFBC3,   /* U+1F2CB */
  0xFBC3,   /* U+1F2CC */
  0xFBC3,   /* U+1F2CD */
  0xFBC3,   /* U+1F2CE */
  0xFBC3,   /* U+1F2CF */
  0xFBC3,   /* U+1F2D0 */
  0xFBC3,   /* U+1F2D1 */
  0xFBC3,   /* U+1F2D2 */
  0xFBC3,   /* U+1F2D3 */
  0xFBC3,   /* U+1F2D4 */
  0xFBC3,   /* U+1F2D5 */
  0xFBC3,   /* U+1F2D6 */
  0xFBC3,   /* U+1F2D7 */
  0xFBC3,   /* U+1F2D8 */
  0xFBC3,   /* U+1F2D9 */
  0xFBC3,   /* U+1F2DA */
  0xFBC3,   /* U+1F2DB */
  0xFBC3,   /* U+1F2DC */
  0xFBC3,   /* U+1F2DD */
  0xFBC3,   /* U+1F2DE */
  0xFBC3,   /* U+1F2DF */
  0xFBC3,   /* U+1F2E0 */
  0xFBC3,   /* U+1F2E1 */
  0xFBC3,   /* U+1F2E2 */
  0xFBC3,   /* U+1F2E3 */
  0xFBC3,   /* U+1F2E4 */
  0xFBC3,   /* U+1F2E5 */
  0xFBC3,   /* U+1F2E6 */
  0xFBC3,   /* U+1F2E7 */
  0xFBC3,   /* U+1F2E8 */
  0xFBC3,   /* U+1F2E9 */
  0xFBC3,   /* U+1F2EA */
  0xFBC3,   /* U+1F2EB */
  0xFBC3,   /* U+1F2EC */
  0xFBC3,   /* U+1F2ED */
  0xFBC3,   /* U+1F2EE */
  0xFBC3,   /* U+1F2EF */
  0xFBC3,   /* U+1F2F0 */
  0xFBC3,   /* U+1F2F1 */
  0xFBC3,   /* U+1F2F2 */
  0xFBC3,   /* U+1F2F3 */
  0xFBC3,   /* U+1F2F4 */
  0xFBC3,   /* U+1F2F5 */
  0xFBC3,   /* U+1F2F6 */
  0xFBC3,   /* U+1F2F7 */
  0xFBC3,   /* U+1F2F8 */
  0xFBC3,   /* U+1F2F9 */
  0xFBC3,   /* U+1F2FA */
  0xFBC3,   /* U+1F2FB */
  0xFBC3,   /* U+1F2FC */
  0xFBC3,   /* U+1F2FD */
  0xFBC3,   /* U+1F2FE */
  0xFBC3,   /* U+1F2FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F200 */
  0x0020,   /* U+1F201 */
  0x0020,   /* U+1F202 */
  0x0020,   /* U+1F203 */
  0x0020,   /* U+1F204 */
  0x0020,   /* U+1F205 */
  0x0020,   /* U+1F206 */
  0x0020,   /* U+1F207 */
  0x0020,   /* U+1F208 */
  0x0020,   /* U+1F209 */
  0x0020,   /* U+1F20A */
  0x0020,   /* U+1F20B */
  0x0020,   /* U+1F20C */
  0x0020,   /* U+1F20D */
  0x0020,   /* U+1F20E */
  0x0020,   /* U+1F20F */
  0x0020,   /* U+1F210 */
  0x0020,   /* U+1F211 */
  0x0020,   /* U+1F212 */
  0x0020,   /* U+1F213 */
  0x0020,   /* U+1F214 */
  0x0020,   /* U+1F215 */
  0x0020,   /* U+1F216 */
  0x0020,   /* U+1F217 */
  0x0020,   /* U+1F218 */
  0x0020,   /* U+1F219 */
  0x0020,   /* U+1F21A */
  0x0020,   /* U+1F21B */
  0x0020,   /* U+1F21C */
  0x0020,   /* U+1F21D */
  0x0020,   /* U+1F21E */
  0x0020,   /* U+1F21F */
  0x0020,   /* U+1F220 */
  0x0020,   /* U+1F221 */
  0x0020,   /* U+1F222 */
  0x0020,   /* U+1F223 */
  0x0020,   /* U+1F224 */
  0x0020,   /* U+1F225 */
  0x0020,   /* U+1F226 */
  0x0020,   /* U+1F227 */
  0x0020,   /* U+1F228 */
  0x0020,   /* U+1F229 */
  0x0020,   /* U+1F22A */
  0x0020,   /* U+1F22B */
  0x0020,   /* U+1F22C */
  0x0020,   /* U+1F22D */
  0x0020,   /* U+1F22E */
  0x0020,   /* U+1F22F */
  0x0020,   /* U+1F230 */
  0x0020,   /* U+1F231 */
  0x0020,   /* U+1F232 */
  0x0020,   /* U+1F233 */
  0x0020,   /* U+1F234 */
  0x0020,   /* U+1F235 */
  0x0020,   /* U+1F236 */
  0x0020,   /* U+1F237 */
  0x0020,   /* U+1F238 */
  0x0020,   /* U+1F239 */
  0x0020,   /* U+1F23A */
  0x0020,   /* U+1F23B */
  0x0020,   /* U+1F23C */
  0x0020,   /* U+1F23D */
  0x0020,   /* U+1F23E */
  0x0020,   /* U+1F23F */
  0x0020,   /* U+1F240 */
  0x0020,   /* U+1F241 */
  0x0020,   /* U+1F242 */
  0x0020,   /* U+1F243 */
  0x0020,   /* U+1F244 */
  0x0020,   /* U+1F245 */
  0x0020,   /* U+1F246 */
  0x0020,   /* U+1F247 */
  0x0020,   /* U+1F248 */
  0x0020,   /* U+1F249 */
  0x0020,   /* U+1F24A */
  0x0020,   /* U+1F24B */
  0x0020,   /* U+1F24C */
  0x0020,   /* U+1F24D */
  0x0020,   /* U+1F24E */
  0x0020,   /* U+1F24F */
  0x0020,   /* U+1F250 */
  0x0020,   /* U+1F251 */
  0x0020,   /* U+1F252 */
  0x0020,   /* U+1F253 */
  0x0020,   /* U+1F254 */
  0x0020,   /* U+1F255 */
  0x0020,   /* U+1F256 */
  0x0020,   /* U+1F257 */
  0x0020,   /* U+1F258 */
  0x0020,   /* U+1F259 */
  0x0020,   /* U+1F25A */
  0x0020,   /* U+1F25B */
  0x0020,   /* U+1F25C */
  0x0020,   /* U+1F25D */
  0x0020,   /* U+1F25E */
  0x0020,   /* U+1F25F */
  0x0020,   /* U+1F260 */
  0x0020,   /* U+1F261 */
  0x0020,   /* U+1F262 */
  0x0020,   /* U+1F263 */
  0x0020,   /* U+1F264 */
  0x0020,   /* U+1F265 */
  0x0020,   /* U+1F266 */
  0x0020,   /* U+1F267 */
  0x0020,   /* U+1F268 */
  0x0020,   /* U+1F269 */
  0x0020,   /* U+1F26A */
  0x0020,   /* U+1F26B */
  0x0020,   /* U+1F26C */
  0x0020,   /* U+1F26D */
  0x0020,   /* U+1F26E */
  0x0020,   /* U+1F26F */
  0x0020,   /* U+1F270 */
  0x0020,   /* U+1F271 */
  0x0020,   /* U+1F272 */
  0x0020,   /* U+1F273 */
  0x0020,   /* U+1F274 */
  0x0020,   /* U+1F275 */
  0x0020,   /* U+1F276 */
  0x0020,   /* U+1F277 */
  0x0020,   /* U+1F278 */
  0x0020,   /* U+1F279 */
  0x0020,   /* U+1F27A */
  0x0020,   /* U+1F27B */
  0x0020,   /* U+1F27C */
  0x0020,   /* U+1F27D */
  0x0020,   /* U+1F27E */
  0x0020,   /* U+1F27F */
  0x0020,   /* U+1F280 */
  0x0020,   /* U+1F281 */
  0x0020,   /* U+1F282 */
  0x0020,   /* U+1F283 */
  0x0020,   /* U+1F284 */
  0x0020,   /* U+1F285 */
  0x0020,   /* U+1F286 */
  0x0020,   /* U+1F287 */
  0x0020,   /* U+1F288 */
  0x0020,   /* U+1F289 */
  0x0020,   /* U+1F28A */
  0x0020,   /* U+1F28B */
  0x0020,   /* U+1F28C */
  0x0020,   /* U+1F28D */
  0x0020,   /* U+1F28E */
  0x0020,   /* U+1F28F */
  0x0020,   /* U+1F290 */
  0x0020,   /* U+1F291 */
  0x0020,   /* U+1F292 */
  0x0020,   /* U+1F293 */
  0x0020,   /* U+1F294 */
  0x0020,   /* U+1F295 */
  0x0020,   /* U+1F296 */
  0x0020,   /* U+1F297 */
  0x0020,   /* U+1F298 */
  0x0020,   /* U+1F299 */
  0x0020,   /* U+1F29A */
  0x0020,   /* U+1F29B */
  0x0020,   /* U+1F29C */
  0x0020,   /* U+1F29D */
  0x0020,   /* U+1F29E */
  0x0020,   /* U+1F29F */
  0x0020,   /* U+1F2A0 */
  0x0020,   /* U+1F2A1 */
  0x0020,   /* U+1F2A2 */
  0x0020,   /* U+1F2A3 */
  0x0020,   /* U+1F2A4 */
  0x0020,   /* U+1F2A5 */
  0x0020,   /* U+1F2A6 */
  0x0020,   /* U+1F2A7 */
  0x0020,   /* U+1F2A8 */
  0x0020,   /* U+1F2A9 */
  0x0020,   /* U+1F2AA */
  0x0020,   /* U+1F2AB */
  0x0020,   /* U+1F2AC */
  0x0020,   /* U+1F2AD */
  0x0020,   /* U+1F2AE */
  0x0020,   /* U+1F2AF */
  0x0020,   /* U+1F2B0 */
  0x0020,   /* U+1F2B1 */
  0x0020,   /* U+1F2B2 */
  0x0020,   /* U+1F2B3 */
  0x0020,   /* U+1F2B4 */
  0x0020,   /* U+1F2B5 */
  0x0020,   /* U+1F2B6 */
  0x0020,   /* U+1F2B7 */
  0x0020,   /* U+1F2B8 */
  0x0020,   /* U+1F2B9 */
  0x0020,   /* U+1F2BA */
  0x0020,   /* U+1F2BB */
  0x0020,   /* U+1F2BC */
  0x0020,   /* U+1F2BD */
  0x0020,   /* U+1F2BE */
  0x0020,   /* U+1F2BF */
  0x0020,   /* U+1F2C0 */
  0x0020,   /* U+1F2C1 */
  0x0020,   /* U+1F2C2 */
  0x0020,   /* U+1F2C3 */
  0x0020,   /* U+1F2C4 */
  0x0020,   /* U+1F2C5 */
  0x0020,   /* U+1F2C6 */
  0x0020,   /* U+1F2C7 */
  0x0020,   /* U+1F2C8 */
  0x0020,   /* U+1F2C9 */
  0x0020,   /* U+1F2CA */
  0x0020,   /* U+1F2CB */
  0x0020,   /* U+1F2CC */
  0x0020,   /* U+1F2CD */
  0x0020,   /* U+1F2CE */
  0x0020,   /* U+1F2CF */
  0x0020,   /* U+1F2D0 */
  0x0020,   /* U+1F2D1 */
  0x0020,   /* U+1F2D2 */
  0x0020,   /* U+1F2D3 */
  0x0020,   /* U+1F2D4 */
  0x0020,   /* U+1F2D5 */
  0x0020,   /* U+1F2D6 */
  0x0020,   /* U+1F2D7 */
  0x0020,   /* U+1F2D8 */
  0x0020,   /* U+1F2D9 */
  0x0020,   /* U+1F2DA */
  0x0020,   /* U+1F2DB */
  0x0020,   /* U+1F2DC */
  0x0020,   /* U+1F2DD */
  0x0020,   /* U+1F2DE */
  0x0020,   /* U+1F2DF */
  0x0020,   /* U+1F2E0 */
  0x0020,   /* U+1F2E1 */
  0x0020,   /* U+1F2E2 */
  0x0020,   /* U+1F2E3 */
  0x0020,   /* U+1F2E4 */
  0x0020,   /* U+1F2E5 */
  0x0020,   /* U+1F2E6 */
  0x0020,   /* U+1F2E7 */
  0x0020,   /* U+1F2E8 */
  0x0020,   /* U+1F2E9 */
  0x0020,   /* U+1F2EA */
  0x0020,   /* U+1F2EB */
  0x0020,   /* U+1F2EC */
  0x0020,   /* U+1F2ED */
  0x0020,   /* U+1F2EE */
  0x0020,   /* U+1F2EF */
  0x0020,   /* U+1F2F0 */
  0x0020,   /* U+1F2F1 */
  0x0020,   /* U+1F2F2 */
  0x0020,   /* U+1F2F3 */
  0x0020,   /* U+1F2F4 */
  0x0020,   /* U+1F2F5 */
  0x0020,   /* U+1F2F6 */
  0x0020,   /* U+1F2F7 */
  0x0020,   /* U+1F2F8 */
  0x0020,   /* U+1F2F9 */
  0x0020,   /* U+1F2FA */
  0x0020,   /* U+1F2FB */
  0x0020,   /* U+1F2FC */
  0x0020,   /* U+1F2FD */
  0x0020,   /* U+1F2FE */
  0x0020,   /* U+1F2FF */

  /* Tertiary weight 1 for each character. */
  0x001C,   /* U+1F200 */
  0x001C,   /* U+1F201 */
  0x001C,   /* U+1F202 */
  0x0002,   /* U+1F203 */
  0x0002,   /* U+1F204 */
  0x0002,   /* U+1F205 */
  0x0002,   /* U+1F206 */
  0x0002,   /* U+1F207 */
  0x0002,   /* U+1F208 */
  0x0002,   /* U+1F209 */
  0x0002,   /* U+1F20A */
  0x0002,   /* U+1F20B */
  0x0002,   /* U+1F20C */
  0x0002,   /* U+1F20D */
  0x0002,   /* U+1F20E */
  0x0002,   /* U+1F20F */
  0x001C,   /* U+1F210 */
  0x001C,   /* U+1F211 */
  0x001C,   /* U+1F212 */
  0x001C,   /* U+1F213 */
  0x001C,   /* U+1F214 */
  0x001C,   /* U+1F215 */
  0x001C,   /* U+1F216 */
  0x001C,   /* U+1F217 */
  0x001C,   /* U+1F218 */
  0x001C,   /* U+1F219 */
  0x001C,   /* U+1F21A */
  0x001C,   /* U+1F21B */
  0x001C,   /* U+1F21C */
  0x001C,   /* U+1F21D */
  0x001C,   /* U+1F21E */
  0x001C,   /* U+1F21F */
  0x001C,   /* U+1F220 */
  0x001C,   /* U+1F221 */
  0x001C,   /* U+1F222 */
  0x001C,   /* U+1F223 */
  0x001C,   /* U+1F224 */
  0x001C,   /* U+1F225 */
  0x001C,   /* U+1F226 */
  0x001C,   /* U+1F227 */
  0x001C,   /* U+1F228 */
  0x001C,   /* U+1F229 */
  0x001C,   /* U+1F22A */
  0x001C,   /* U+1F22B */
  0x001C,   /* U+1F22C */
  0x001C,   /* U+1F22D */
  0x001C,   /* U+1F22E */
  0x001C,   /* U+1F22F */
  0x001C,   /* U+1F230 */
  0x001C,   /* U+1F231 */
  0x001C,   /* U+1F232 */
  0x001C,   /* U+1F233 */
  0x001C,   /* U+1F234 */
  0x001C,   /* U+1F235 */
  0x001C,   /* U+1F236 */
  0x001C,   /* U+1F237 */
  0x001C,   /* U+1F238 */
  0x001C,   /* U+1F239 */
  0x001C,   /* U+1F23A */
  0x001C,   /* U+1F23B */
  0x0002,   /* U+1F23C */
  0x0002,   /* U+1F23D */
  0x0002,   /* U+1F23E */
  0x0002,   /* U+1F23F */
  0x0004,   /* U+1F240 */
  0x0004,   /* U+1F241 */
  0x0004,   /* U+1F242 */
  0x0004,   /* U+1F243 */
  0x0004,   /* U+1F244 */
  0x0004,   /* U+1F245 */
  0x0004,   /* U+1F246 */
  0x0004,   /* U+1F247 */
  0x0004,   /* U+1F248 */
  0x0002,   /* U+1F249 */
  0x0002,   /* U+1F24A */
  0x0002,   /* U+1F24B */
  0x0002,   /* U+1F24C */
  0x0002,   /* U+1F24D */
  0x0002,   /* U+1F24E */
  0x0002,   /* U+1F24F */
  0x0006,   /* U+1F250 */
  0x0006,   /* U+1F251 */
  0x0002,   /* U+1F252 */
  0x0002,   /* U+1F253 */
  0x0002,   /* U+1F254 */
  0x0002,   /* U+1F255 */
  0x0002,   /* U+1F256 */
  0x0002,   /* U+1F257 */
  0x0002,   /* U+1F258 */
  0x0002,   /* U+1F259 */
  0x0002,   /* U+1F25A */
  0x0002,   /* U+1F25B */
  0x0002,   /* U+1F25C */
  0x0002,   /* U+1F25D */
  0x0002,   /* U+1F25E */
  0x0002,   /* U+1F25F */
  0x0002,   /* U+1F260 */
  0x0002,   /* U+1F261 */
  0x0002,   /* U+1F262 */
  0x0002,   /* U+1F263 */
  0x0002,   /* U+1F264 */
  0x0002,   /* U+1F265 */
  0x0002,   /* U+1F266 */
  0x0002,   /* U+1F267 */
  0x0002,   /* U+1F268 */
  0x0002,   /* U+1F269 */
  0x0002,   /* U+1F26A */
  0x0002,   /* U+1F26B */
  0x0002,   /* U+1F26C */
  0x0002,   /* U+1F26D */
  0x0002,   /* U+1F26E */
  0x0002,   /* U+1F26F */
  0x0002,   /* U+1F270 */
  0x0002,   /* U+1F271 */
  0x0002,   /* U+1F272 */
  0x0002,   /* U+1F273 */
  0x0002,   /* U+1F274 */
  0x0002,   /* U+1F275 */
  0x0002,   /* U+1F276 */
  0x0002,   /* U+1F277 */
  0x0002,   /* U+1F278 */
  0x0002,   /* U+1F279 */
  0x0002,   /* U+1F27A */
  0x0002,   /* U+1F27B */
  0x0002,   /* U+1F27C */
  0x0002,   /* U+1F27D */
  0x0002,   /* U+1F27E */
  0x0002,   /* U+1F27F */
  0x0002,   /* U+1F280 */
  0x0002,   /* U+1F281 */
  0x0002,   /* U+1F282 */
  0x0002,   /* U+1F283 */
  0x0002,   /* U+1F284 */
  0x0002,   /* U+1F285 */
  0x0002,   /* U+1F286 */
  0x0002,   /* U+1F287 */
  0x0002,   /* U+1F288 */
  0x0002,   /* U+1F289 */
  0x0002,   /* U+1F28A */
  0x0002,   /* U+1F28B */
  0x0002,   /* U+1F28C */
  0x0002,   /* U+1F28D */
  0x0002,   /* U+1F28E */
  0x0002,   /* U+1F28F */
  0x0002,   /* U+1F290 */
  0x0002,   /* U+1F291 */
  0x0002,   /* U+1F292 */
  0x0002,   /* U+1F293 */
  0x0002,   /* U+1F294 */
  0x0002,   /* U+1F295 */
  0x0002,   /* U+1F296 */
  0x0002,   /* U+1F297 */
  0x0002,   /* U+1F298 */
  0x0002,   /* U+1F299 */
  0x0002,   /* U+1F29A */
  0x0002,   /* U+1F29B */
  0x0002,   /* U+1F29C */
  0x0002,   /* U+1F29D */
  0x0002,   /* U+1F29E */
  0x0002,   /* U+1F29F */
  0x0002,   /* U+1F2A0 */
  0x0002,   /* U+1F2A1 */
  0x0002,   /* U+1F2A2 */
  0x0002,   /* U+1F2A3 */
  0x0002,   /* U+1F2A4 */
  0x0002,   /* U+1F2A5 */
  0x0002,   /* U+1F2A6 */
  0x0002,   /* U+1F2A7 */
  0x0002,   /* U+1F2A8 */
  0x0002,   /* U+1F2A9 */
  0x0002,   /* U+1F2AA */
  0x0002,   /* U+1F2AB */
  0x0002,   /* U+1F2AC */
  0x0002,   /* U+1F2AD */
  0x0002,   /* U+1F2AE */
  0x0002,   /* U+1F2AF */
  0x0002,   /* U+1F2B0 */
  0x0002,   /* U+1F2B1 */
  0x0002,   /* U+1F2B2 */
  0x0002,   /* U+1F2B3 */
  0x0002,   /* U+1F2B4 */
  0x0002,   /* U+1F2B5 */
  0x0002,   /* U+1F2B6 */
  0x0002,   /* U+1F2B7 */
  0x0002,   /* U+1F2B8 */
  0x0002,   /* U+1F2B9 */
  0x0002,   /* U+1F2BA */
  0x0002,   /* U+1F2BB */
  0x0002,   /* U+1F2BC */
  0x0002,   /* U+1F2BD */
  0x0002,   /* U+1F2BE */
  0x0002,   /* U+1F2BF */
  0x0002,   /* U+1F2C0 */
  0x0002,   /* U+1F2C1 */
  0x0002,   /* U+1F2C2 */
  0x0002,   /* U+1F2C3 */
  0x0002,   /* U+1F2C4 */
  0x0002,   /* U+1F2C5 */
  0x0002,   /* U+1F2C6 */
  0x0002,   /* U+1F2C7 */
  0x0002,   /* U+1F2C8 */
  0x0002,   /* U+1F2C9 */
  0x0002,   /* U+1F2CA */
  0x0002,   /* U+1F2CB */
  0x0002,   /* U+1F2CC */
  0x0002,   /* U+1F2CD */
  0x0002,   /* U+1F2CE */
  0x0002,   /* U+1F2CF */
  0x0002,   /* U+1F2D0 */
  0x0002,   /* U+1F2D1 */
  0x0002,   /* U+1F2D2 */
  0x0002,   /* U+1F2D3 */
  0x0002,   /* U+1F2D4 */
  0x0002,   /* U+1F2D5 */
  0x0002,   /* U+1F2D6 */
  0x0002,   /* U+1F2D7 */
  0x0002,   /* U+1F2D8 */
  0x0002,   /* U+1F2D9 */
  0x0002,   /* U+1F2DA */
  0x0002,   /* U+1F2DB */
  0x0002,   /* U+1F2DC */
  0x0002,   /* U+1F2DD */
  0x0002,   /* U+1F2DE */
  0x0002,   /* U+1F2DF */
  0x0002,   /* U+1F2E0 */
  0x0002,   /* U+1F2E1 */
  0x0002,   /* U+1F2E2 */
  0x0002,   /* U+1F2E3 */
  0x0002,   /* U+1F2E4 */
  0x0002,   /* U+1F2E5 */
  0x0002,   /* U+1F2E6 */
  0x0002,   /* U+1F2E7 */
  0x0002,   /* U+1F2E8 */
  0x0002,   /* U+1F2E9 */
  0x0002,   /* U+1F2EA */
  0x0002,   /* U+1F2EB */
  0x0002,   /* U+1F2EC */
  0x0002,   /* U+1F2ED */
  0x0002,   /* U+1F2EE */
  0x0002,   /* U+1F2EF */
  0x0002,   /* U+1F2F0 */
  0x0002,   /* U+1F2F1 */
  0x0002,   /* U+1F2F2 */
  0x0002,   /* U+1F2F3 */
  0x0002,   /* U+1F2F4 */
  0x0002,   /* U+1F2F5 */
  0x0002,   /* U+1F2F6 */
  0x0002,   /* U+1F2F7 */
  0x0002,   /* U+1F2F8 */
  0x0002,   /* U+1F2F9 */
  0x0002,   /* U+1F2FA */
  0x0002,   /* U+1F2FB */
  0x0002,   /* U+1F2FC */
  0x0002,   /* U+1F2FD */
  0x0002,   /* U+1F2FE */
  0x0002,   /* U+1F2FF */

  /* Primary weight 2 for each character. */
  0x3D60,   /* U+1F200 */
  0x3D64,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0xF203,   /* U+1F203 */
  0xF204,   /* U+1F204 */
  0xF205,   /* U+1F205 */
  0xF206,   /* U+1F206 */
  0xF207,   /* U+1F207 */
  0xF208,   /* U+1F208 */
  0xF209,   /* U+1F209 */
  0xF20A,   /* U+1F20A */
  0xF20B,   /* U+1F20B */
  0xF20C,   /* U+1F20C */
  0xF20D,   /* U+1F20D */
  0xF20E,   /* U+1F20E */
  0xF20F,   /* U+1F20F */
  0xE24B,   /* U+1F210 */
  0xDB57,   /* U+1F211 */
  0xD3CC,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0xCE8C,   /* U+1F214 */
  0xD91A,   /* U+1F215 */
  0x89E3,   /* U+1F216 */
  0xD929,   /* U+1F217 */
  0xCEA4,   /* U+1F218 */
  0xE620,   /* U+1F219 */
  0xF121,   /* U+1F21A */
  0xE599,   /* U+1F21B */
  0xD24D,   /* U+1F21C */
  0xDF8C,   /* U+1F21D */
  0xD18D,   /* U+1F21E */
  0xE5B0,   /* U+1F21F */
  0xD21D,   /* U+1F220 */
  0xFD42,   /* U+1F221 */
  0xF51F,   /* U+1F222 */
  0x8CA9,   /* U+1F223 */
  0xD8F0,   /* U+1F224 */
  0xD439,   /* U+1F225 */
  0xEF14,   /* U+1F226 */
  0xE295,   /* U+1F227 */
  0xE355,   /* U+1F228 */
  0xCE00,   /* U+1F229 */
  0xCE09,   /* U+1F22A */
  0x904A,   /* U+1F22B */
  0xDDE6,   /* U+1F22C */
  0xCE2D,   /* U+1F22D */
  0xD3F3,   /* U+1F22E */
  0xE307,   /* U+1F22F */
  0x8D70,   /* U+1F230 */
  0xE253,   /* U+1F231 */
  0xF981,   /* U+1F232 */
  0xFA7A,   /* U+1F233 */
  0xD408,   /* U+1F234 */
  0xEE80,   /* U+1F235 */
  0xE709,   /* U+1F236 */
  0xE708,   /* U+1F237 */
  0xF533,   /* U+1F238 */
  0xD272,   /* U+1F239 */
  0xD5B6,   /* U+1F23A */
  0x914D,   /* U+1F23B */
  0xF23C,   /* U+1F23C */
  0xF23D,   /* U+1F23D */
  0xF23E,   /* U+1F23E */
  0xF23F,   /* U+1F23F */
  0xFB40,   /* U+1F240 */
  0xFB40,   /* U+1F241 */
  0xFB40,   /* U+1F242 */
  0xFB40,   /* U+1F243 */
  0xFB40,   /* U+1F244 */
  0xFB40,   /* U+1F245 */
  0xFB40,   /* U+1F246 */
  0xFB40,   /* U+1F247 */
  0xFB40,   /* U+1F248 */
  0xF249,   /* U+1F249 */
  0xF24A,   /* U+1F24A */
  0xF24B,   /* U+1F24B */
  0xF24C,   /* U+1F24C */
  0xF24D,   /* U+1F24D */
  0xF24E,   /* U+1F24E */
  0xF24F,   /* U+1F24F */
  0xDF97,   /* U+1F250 */
  0xD3EF,   /* U+1F251 */
  0xF252,   /* U+1F252 */
  0xF253,   /* U+1F253 */
  0xF254,   /* U+1F254 */
  0xF255,   /* U+1F255 */
  0xF256,   /* U+1F256 */
  0xF257,   /* U+1F257 */
  0xF258,   /* U+1F258 */
  0xF259,   /* U+1F259 */
  0xF25A,   /* U+1F25A */
  0xF25B,   /* U+1F25B */
  0xF25C,   /* U+1F25C */
  0xF25D,   /* U+1F25D */
  0xF25E,   /* U+1F25E */
  0xF25F,   /* U+1F25F */
  0xF260,   /* U+1F260 */
  0xF261,   /* U+1F261 */
  0xF262,   /* U+1F262 */
  0xF263,   /* U+1F263 */
  0xF264,   /* U+1F264 */
  0xF265,   /* U+1F265 */
  0xF266,   /* U+1F266 */
  0xF267,   /* U+1F267 */
  0xF268,   /* U+1F268 */
  0xF269,   /* U+1F269 */
  0xF26A,   /* U+1F26A */
  0xF26B,   /* U+1F26B */
  0xF26C,   /* U+1F26C */
  0xF26D,   /* U+1F26D */
  0xF26E,   /* U+1F26E */
  0xF26F,   /* U+1F26F */
  0xF270,   /* U+1F270 */
  0xF271,   /* U+1F271 */
  0xF272,   /* U+1F272 */
  0xF273,   /* U+1F273 */
  0xF274,   /* U+1F274 */
  0xF275,   /* U+1F275 */
  0xF276,   /* U+1F276 */
  0xF277,   /* U+1F277 */
  0xF278,   /* U+1F278 */
  0xF279,   /* U+1F279 */
  0xF27A,   /* U+1F27A */
  0xF27B,   /* U+1F27B */
  0xF27C,   /* U+1F27C */
  0xF27D,   /* U+1F27D */
  0xF27E,   /* U+1F27E */
  0xF27F,   /* U+1F27F */
  0xF280,   /* U+1F280 */
  0xF281,   /* U+1F281 */
  0xF282,   /* U+1F282 */
  0xF283,   /* U+1F283 */
  0xF284,   /* U+1F284 */
  0xF285,   /* U+1F285 */
  0xF286,   /* U+1F286 */
  0xF287,   /* U+1F287 */
  0xF288,   /* U+1F288 */
  0xF289,   /* U+1F289 */
  0xF28A,   /* U+1F28A */
  0xF28B,   /* U+1F28B */
  0xF28C,   /* U+1F28C */
  0xF28D,   /* U+1F28D */
  0xF28E,   /* U+1F28E */
  0xF28F,   /* U+1F28F */
  0xF290,   /* U+1F290 */
  0xF291,   /* U+1F291 */
  0xF292,   /* U+1F292 */
  0xF293,   /* U+1F293 */
  0xF294,   /* U+1F294 */
  0xF295,   /* U+1F295 */
  0xF296,   /* U+1F296 */
  0xF297,   /* U+1F297 */
  0xF298,   /* U+1F298 */
  0xF299,   /* U+1F299 */
  0xF29A,   /* U+1F29A */
  0xF29B,   /* U+1F29B */
  0xF29C,   /* U+1F29C */
  0xF29D,   /* U+1F29D */
  0xF29E,   /* U+1F29E */
  0xF29F,   /* U+1F29F */
  0xF2A0,   /* U+1F2A0 */
  0xF2A1,   /* U+1F2A1 */
  0xF2A2,   /* U+1F2A2 */
  0xF2A3,   /* U+1F2A3 */
  0xF2A4,   /* U+1F2A4 */
  0xF2A5,   /* U+1F2A5 */
  0xF2A6,   /* U+1F2A6 */
  0xF2A7,   /* U+1F2A7 */
  0xF2A8,   /* U+1F2A8 */
  0xF2A9,   /* U+1F2A9 */
  0xF2AA,   /* U+1F2AA */
  0xF2AB,   /* U+1F2AB */
  0xF2AC,   /* U+1F2AC */
  0xF2AD,   /* U+1F2AD */
  0xF2AE,   /* U+1F2AE */
  0xF2AF,   /* U+1F2AF */
  0xF2B0,   /* U+1F2B0 */
  0xF2B1,   /* U+1F2B1 */
  0xF2B2,   /* U+1F2B2 */
  0xF2B3,   /* U+1F2B3 */
  0xF2B4,   /* U+1F2B4 */
  0xF2B5,   /* U+1F2B5 */
  0xF2B6,   /* U+1F2B6 */
  0xF2B7,   /* U+1F2B7 */
  0xF2B8,   /* U+1F2B8 */
  0xF2B9,   /* U+1F2B9 */
  0xF2BA,   /* U+1F2BA */
  0xF2BB,   /* U+1F2BB */
  0xF2BC,   /* U+1F2BC */
  0xF2BD,   /* U+1F2BD */
  0xF2BE,   /* U+1F2BE */
  0xF2BF,   /* U+1F2BF */
  0xF2C0,   /* U+1F2C0 */
  0xF2C1,   /* U+1F2C1 */
  0xF2C2,   /* U+1F2C2 */
  0xF2C3,   /* U+1F2C3 */
  0xF2C4,   /* U+1F2C4 */
  0xF2C5,   /* U+1F2C5 */
  0xF2C6,   /* U+1F2C6 */
  0xF2C7,   /* U+1F2C7 */
  0xF2C8,   /* U+1F2C8 */
  0xF2C9,   /* U+1F2C9 */
  0xF2CA,   /* U+1F2CA */
  0xF2CB,   /* U+1F2CB */
  0xF2CC,   /* U+1F2CC */
  0xF2CD,   /* U+1F2CD */
  0xF2CE,   /* U+1F2CE */
  0xF2CF,   /* U+1F2CF */
  0xF2D0,   /* U+1F2D0 */
  0xF2D1,   /* U+1F2D1 */
  0xF2D2,   /* U+1F2D2 */
  0xF2D3,   /* U+1F2D3 */
  0xF2D4,   /* U+1F2D4 */
  0xF2D5,   /* U+1F2D5 */
  0xF2D6,   /* U+1F2D6 */
  0xF2D7,   /* U+1F2D7 */
  0xF2D8,   /* U+1F2D8 */
  0xF2D9,   /* U+1F2D9 */
  0xF2DA,   /* U+1F2DA */
  0xF2DB,   /* U+1F2DB */
  0xF2DC,   /* U+1F2DC */
  0xF2DD,   /* U+1F2DD */
  0xF2DE,   /* U+1F2DE */
  0xF2DF,   /* U+1F2DF */
  0xF2E0,   /* U+1F2E0 */
  0xF2E1,   /* U+1F2E1 */
  0xF2E2,   /* U+1F2E2 */
  0xF2E3,   /* U+1F2E3 */
  0xF2E4,   /* U+1F2E4 */
  0xF2E5,   /* U+1F2E5 */
  0xF2E6,   /* U+1F2E6 */
  0xF2E7,   /* U+1F2E7 */
  0xF2E8,   /* U+1F2E8 */
  0xF2E9,   /* U+1F2E9 */
  0xF2EA,   /* U+1F2EA */
  0xF2EB,   /* U+1F2EB */
  0xF2EC,   /* U+1F2EC */
  0xF2ED,   /* U+1F2ED */
  0xF2EE,   /* U+1F2EE */
  0xF2EF,   /* U+1F2EF */
  0xF2F0,   /* U+1F2F0 */
  0xF2F1,   /* U+1F2F1 */
  0xF2F2,   /* U+1F2F2 */
  0xF2F3,   /* U+1F2F3 */
  0xF2F4,   /* U+1F2F4 */
  0xF2F5,   /* U+1F2F5 */
  0xF2F6,   /* U+1F2F6 */
  0xF2F7,   /* U+1F2F7 */
  0xF2F8,   /* U+1F2F8 */
  0xF2F9,   /* U+1F2F9 */
  0xF2FA,   /* U+1F2FA */
  0xF2FB,   /* U+1F2FB */
  0xF2FC,   /* U+1F2FC */
  0xF2FD,   /* U+1F2FD */
  0xF2FE,   /* U+1F2FE */
  0xF2FF,   /* U+1F2FF */

  /* Secondary weight 2 for each character. */
  0x0020,   /* U+1F200 */
  0x0020,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0037,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0020,   /* U+1F240 */
  0x0020,   /* U+1F241 */
  0x0020,   /* U+1F242 */
  0x0020,   /* U+1F243 */
  0x0020,   /* U+1F244 */
  0x0020,   /* U+1F245 */
  0x0020,   /* U+1F246 */
  0x0020,   /* U+1F247 */
  0x0020,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Tertiary weight 2 for each character. */
  0x001C,   /* U+1F200 */
  0x001C,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x001C,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0004,   /* U+1F240 */
  0x0004,   /* U+1F241 */
  0x0004,   /* U+1F242 */
  0x0004,   /* U+1F243 */
  0x0004,   /* U+1F244 */
  0x0004,   /* U+1F245 */
  0x0004,   /* U+1F246 */
  0x0004,   /* U+1F247 */
  0x0004,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Primary weight 3 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0xE72C,   /* U+1F240 */
  0xCE09,   /* U+1F241 */
  0xCE8C,   /* U+1F242 */
  0xDB89,   /* U+1F243 */
  0xF0B9,   /* U+1F244 */
  0xE253,   /* U+1F245 */
  0xF6D7,   /* U+1F246 */
  0xD2DD,   /* U+1F247 */
  0xE557,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Secondary weight 3 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0000,   /* U+1F240 */
  0x0000,   /* U+1F241 */
  0x0000,   /* U+1F242 */
  0x0000,   /* U+1F243 */
  0x0000,   /* U+1F244 */
  0x0000,   /* U+1F245 */
  0x0000,   /* U+1F246 */
  0x0000,   /* U+1F247 */
  0x0000,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Tertiary weight 3 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0000,   /* U+1F240 */
  0x0000,   /* U+1F241 */
  0x0000,   /* U+1F242 */
  0x0000,   /* U+1F243 */
  0x0000,   /* U+1F244 */
  0x0000,   /* U+1F245 */
  0x0000,   /* U+1F246 */
  0x0000,   /* U+1F247 */
  0x0000,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Primary weight 4 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x037A,   /* U+1F240 */
  0x037A,   /* U+1F241 */
  0x037A,   /* U+1F242 */
  0x037A,   /* U+1F243 */
  0x037A,   /* U+1F244 */
  0x037A,   /* U+1F245 */
  0x037A,   /* U+1F246 */
  0x037A,   /* U+1F247 */
  0x037A,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Secondary weight 4 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0020,   /* U+1F240 */
  0x0020,   /* U+1F241 */
  0x0020,   /* U+1F242 */
  0x0020,   /* U+1F243 */
  0x0020,   /* U+1F244 */
  0x0020,   /* U+1F245 */
  0x0020,   /* U+1F246 */
  0x0020,   /* U+1F247 */
  0x0020,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */

  /* Tertiary weight 4 for each character. */
  0x0000,   /* U+1F200 */
  0x0000,   /* U+1F201 */
  0x0000,   /* U+1F202 */
  0x0000,   /* U+1F203 */
  0x0000,   /* U+1F204 */
  0x0000,   /* U+1F205 */
  0x0000,   /* U+1F206 */
  0x0000,   /* U+1F207 */
  0x0000,   /* U+1F208 */
  0x0000,   /* U+1F209 */
  0x0000,   /* U+1F20A */
  0x0000,   /* U+1F20B */
  0x0000,   /* U+1F20C */
  0x0000,   /* U+1F20D */
  0x0000,   /* U+1F20E */
  0x0000,   /* U+1F20F */
  0x0000,   /* U+1F210 */
  0x0000,   /* U+1F211 */
  0x0000,   /* U+1F212 */
  0x0000,   /* U+1F213 */
  0x0000,   /* U+1F214 */
  0x0000,   /* U+1F215 */
  0x0000,   /* U+1F216 */
  0x0000,   /* U+1F217 */
  0x0000,   /* U+1F218 */
  0x0000,   /* U+1F219 */
  0x0000,   /* U+1F21A */
  0x0000,   /* U+1F21B */
  0x0000,   /* U+1F21C */
  0x0000,   /* U+1F21D */
  0x0000,   /* U+1F21E */
  0x0000,   /* U+1F21F */
  0x0000,   /* U+1F220 */
  0x0000,   /* U+1F221 */
  0x0000,   /* U+1F222 */
  0x0000,   /* U+1F223 */
  0x0000,   /* U+1F224 */
  0x0000,   /* U+1F225 */
  0x0000,   /* U+1F226 */
  0x0000,   /* U+1F227 */
  0x0000,   /* U+1F228 */
  0x0000,   /* U+1F229 */
  0x0000,   /* U+1F22A */
  0x0000,   /* U+1F22B */
  0x0000,   /* U+1F22C */
  0x0000,   /* U+1F22D */
  0x0000,   /* U+1F22E */
  0x0000,   /* U+1F22F */
  0x0000,   /* U+1F230 */
  0x0000,   /* U+1F231 */
  0x0000,   /* U+1F232 */
  0x0000,   /* U+1F233 */
  0x0000,   /* U+1F234 */
  0x0000,   /* U+1F235 */
  0x0000,   /* U+1F236 */
  0x0000,   /* U+1F237 */
  0x0000,   /* U+1F238 */
  0x0000,   /* U+1F239 */
  0x0000,   /* U+1F23A */
  0x0000,   /* U+1F23B */
  0x0000,   /* U+1F23C */
  0x0000,   /* U+1F23D */
  0x0000,   /* U+1F23E */
  0x0000,   /* U+1F23F */
  0x0004,   /* U+1F240 */
  0x0004,   /* U+1F241 */
  0x0004,   /* U+1F242 */
  0x0004,   /* U+1F243 */
  0x0004,   /* U+1F244 */
  0x0004,   /* U+1F245 */
  0x0004,   /* U+1F246 */
  0x0004,   /* U+1F247 */
  0x0004,   /* U+1F248 */
  0x0000,   /* U+1F249 */
  0x0000,   /* U+1F24A */
  0x0000,   /* U+1F24B */
  0x0000,   /* U+1F24C */
  0x0000,   /* U+1F24D */
  0x0000,   /* U+1F24E */
  0x0000,   /* U+1F24F */
  0x0000,   /* U+1F250 */
  0x0000,   /* U+1F251 */
  0x0000,   /* U+1F252 */
  0x0000,   /* U+1F253 */
  0x0000,   /* U+1F254 */
  0x0000,   /* U+1F255 */
  0x0000,   /* U+1F256 */
  0x0000,   /* U+1F257 */
  0x0000,   /* U+1F258 */
  0x0000,   /* U+1F259 */
  0x0000,   /* U+1F25A */
  0x0000,   /* U+1F25B */
  0x0000,   /* U+1F25C */
  0x0000,   /* U+1F25D */
  0x0000,   /* U+1F25E */
  0x0000,   /* U+1F25F */
  0x0000,   /* U+1F260 */
  0x0000,   /* U+1F261 */
  0x0000,   /* U+1F262 */
  0x0000,   /* U+1F263 */
  0x0000,   /* U+1F264 */
  0x0000,   /* U+1F265 */
  0x0000,   /* U+1F266 */
  0x0000,   /* U+1F267 */
  0x0000,   /* U+1F268 */
  0x0000,   /* U+1F269 */
  0x0000,   /* U+1F26A */
  0x0000,   /* U+1F26B */
  0x0000,   /* U+1F26C */
  0x0000,   /* U+1F26D */
  0x0000,   /* U+1F26E */
  0x0000,   /* U+1F26F */
  0x0000,   /* U+1F270 */
  0x0000,   /* U+1F271 */
  0x0000,   /* U+1F272 */
  0x0000,   /* U+1F273 */
  0x0000,   /* U+1F274 */
  0x0000,   /* U+1F275 */
  0x0000,   /* U+1F276 */
  0x0000,   /* U+1F277 */
  0x0000,   /* U+1F278 */
  0x0000,   /* U+1F279 */
  0x0000,   /* U+1F27A */
  0x0000,   /* U+1F27B */
  0x0000,   /* U+1F27C */
  0x0000,   /* U+1F27D */
  0x0000,   /* U+1F27E */
  0x0000,   /* U+1F27F */
  0x0000,   /* U+1F280 */
  0x0000,   /* U+1F281 */
  0x0000,   /* U+1F282 */
  0x0000,   /* U+1F283 */
  0x0000,   /* U+1F284 */
  0x0000,   /* U+1F285 */
  0x0000,   /* U+1F286 */
  0x0000,   /* U+1F287 */
  0x0000,   /* U+1F288 */
  0x0000,   /* U+1F289 */
  0x0000,   /* U+1F28A */
  0x0000,   /* U+1F28B */
  0x0000,   /* U+1F28C */
  0x0000,   /* U+1F28D */
  0x0000,   /* U+1F28E */
  0x0000,   /* U+1F28F */
  0x0000,   /* U+1F290 */
  0x0000,   /* U+1F291 */
  0x0000,   /* U+1F292 */
  0x0000,   /* U+1F293 */
  0x0000,   /* U+1F294 */
  0x0000,   /* U+1F295 */
  0x0000,   /* U+1F296 */
  0x0000,   /* U+1F297 */
  0x0000,   /* U+1F298 */
  0x0000,   /* U+1F299 */
  0x0000,   /* U+1F29A */
  0x0000,   /* U+1F29B */
  0x0000,   /* U+1F29C */
  0x0000,   /* U+1F29D */
  0x0000,   /* U+1F29E */
  0x0000,   /* U+1F29F */
  0x0000,   /* U+1F2A0 */
  0x0000,   /* U+1F2A1 */
  0x0000,   /* U+1F2A2 */
  0x0000,   /* U+1F2A3 */
  0x0000,   /* U+1F2A4 */
  0x0000,   /* U+1F2A5 */
  0x0000,   /* U+1F2A6 */
  0x0000,   /* U+1F2A7 */
  0x0000,   /* U+1F2A8 */
  0x0000,   /* U+1F2A9 */
  0x0000,   /* U+1F2AA */
  0x0000,   /* U+1F2AB */
  0x0000,   /* U+1F2AC */
  0x0000,   /* U+1F2AD */
  0x0000,   /* U+1F2AE */
  0x0000,   /* U+1F2AF */
  0x0000,   /* U+1F2B0 */
  0x0000,   /* U+1F2B1 */
  0x0000,   /* U+1F2B2 */
  0x0000,   /* U+1F2B3 */
  0x0000,   /* U+1F2B4 */
  0x0000,   /* U+1F2B5 */
  0x0000,   /* U+1F2B6 */
  0x0000,   /* U+1F2B7 */
  0x0000,   /* U+1F2B8 */
  0x0000,   /* U+1F2B9 */
  0x0000,   /* U+1F2BA */
  0x0000,   /* U+1F2BB */
  0x0000,   /* U+1F2BC */
  0x0000,   /* U+1F2BD */
  0x0000,   /* U+1F2BE */
  0x0000,   /* U+1F2BF */
  0x0000,   /* U+1F2C0 */
  0x0000,   /* U+1F2C1 */
  0x0000,   /* U+1F2C2 */
  0x0000,   /* U+1F2C3 */
  0x0000,   /* U+1F2C4 */
  0x0000,   /* U+1F2C5 */
  0x0000,   /* U+1F2C6 */
  0x0000,   /* U+1F2C7 */
  0x0000,   /* U+1F2C8 */
  0x0000,   /* U+1F2C9 */
  0x0000,   /* U+1F2CA */
  0x0000,   /* U+1F2CB */
  0x0000,   /* U+1F2CC */
  0x0000,   /* U+1F2CD */
  0x0000,   /* U+1F2CE */
  0x0000,   /* U+1F2CF */
  0x0000,   /* U+1F2D0 */
  0x0000,   /* U+1F2D1 */
  0x0000,   /* U+1F2D2 */
  0x0000,   /* U+1F2D3 */
  0x0000,   /* U+1F2D4 */
  0x0000,   /* U+1F2D5 */
  0x0000,   /* U+1F2D6 */
  0x0000,   /* U+1F2D7 */
  0x0000,   /* U+1F2D8 */
  0x0000,   /* U+1F2D9 */
  0x0000,   /* U+1F2DA */
  0x0000,   /* U+1F2DB */
  0x0000,   /* U+1F2DC */
  0x0000,   /* U+1F2DD */
  0x0000,   /* U+1F2DE */
  0x0000,   /* U+1F2DF */
  0x0000,   /* U+1F2E0 */
  0x0000,   /* U+1F2E1 */
  0x0000,   /* U+1F2E2 */
  0x0000,   /* U+1F2E3 */
  0x0000,   /* U+1F2E4 */
  0x0000,   /* U+1F2E5 */
  0x0000,   /* U+1F2E6 */
  0x0000,   /* U+1F2E7 */
  0x0000,   /* U+1F2E8 */
  0x0000,   /* U+1F2E9 */
  0x0000,   /* U+1F2EA */
  0x0000,   /* U+1F2EB */
  0x0000,   /* U+1F2EC */
  0x0000,   /* U+1F2ED */
  0x0000,   /* U+1F2EE */
  0x0000,   /* U+1F2EF */
  0x0000,   /* U+1F2F0 */
  0x0000,   /* U+1F2F1 */
  0x0000,   /* U+1F2F2 */
  0x0000,   /* U+1F2F3 */
  0x0000,   /* U+1F2F4 */
  0x0000,   /* U+1F2F5 */
  0x0000,   /* U+1F2F6 */
  0x0000,   /* U+1F2F7 */
  0x0000,   /* U+1F2F8 */
  0x0000,   /* U+1F2F9 */
  0x0000,   /* U+1F2FA */
  0x0000,   /* U+1F2FB */
  0x0000,   /* U+1F2FC */
  0x0000,   /* U+1F2FD */
  0x0000,   /* U+1F2FE */
  0x0000,   /* U+1F2FF */
};

uint16 uca900_p1F3[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x12A9,   /* U+1F300 */
  0x12AA,   /* U+1F301 */
  0x12AB,   /* U+1F302 */
  0x12AC,   /* U+1F303 */
  0x12AD,   /* U+1F304 */
  0x12AE,   /* U+1F305 */
  0x12AF,   /* U+1F306 */
  0x12B0,   /* U+1F307 */
  0x12B1,   /* U+1F308 */
  0x12B2,   /* U+1F309 */
  0x12B3,   /* U+1F30A */
  0x12B4,   /* U+1F30B */
  0x12B5,   /* U+1F30C */
  0x12B6,   /* U+1F30D */
  0x12B7,   /* U+1F30E */
  0x12B8,   /* U+1F30F */
  0x12B9,   /* U+1F310 */
  0x12BA,   /* U+1F311 */
  0x12BB,   /* U+1F312 */
  0x12BC,   /* U+1F313 */
  0x12BD,   /* U+1F314 */
  0x12BE,   /* U+1F315 */
  0x12BF,   /* U+1F316 */
  0x12C0,   /* U+1F317 */
  0x12C1,   /* U+1F318 */
  0x12C2,   /* U+1F319 */
  0x12C3,   /* U+1F31A */
  0x12C4,   /* U+1F31B */
  0x12C5,   /* U+1F31C */
  0x12C6,   /* U+1F31D */
  0x12C7,   /* U+1F31E */
  0x12C8,   /* U+1F31F */
  0x12C9,   /* U+1F320 */
  0x12CA,   /* U+1F321 */
  0x12CB,   /* U+1F322 */
  0x12CC,   /* U+1F323 */
  0x12CD,   /* U+1F324 */
  0x12CE,   /* U+1F325 */
  0x12CF,   /* U+1F326 */
  0x12D0,   /* U+1F327 */
  0x12D1,   /* U+1F328 */
  0x12D2,   /* U+1F329 */
  0x12D3,   /* U+1F32A */
  0x12D4,   /* U+1F32B */
  0x12D5,   /* U+1F32C */
  0x12D6,   /* U+1F32D */
  0x12D7,   /* U+1F32E */
  0x12D8,   /* U+1F32F */
  0x12D9,   /* U+1F330 */
  0x12DA,   /* U+1F331 */
  0x12DB,   /* U+1F332 */
  0x12DC,   /* U+1F333 */
  0x12DD,   /* U+1F334 */
  0x12DE,   /* U+1F335 */
  0x12DF,   /* U+1F336 */
  0x12E0,   /* U+1F337 */
  0x12E1,   /* U+1F338 */
  0x12E2,   /* U+1F339 */
  0x12E3,   /* U+1F33A */
  0x12E4,   /* U+1F33B */
  0x12E5,   /* U+1F33C */
  0x12E6,   /* U+1F33D */
  0x12E7,   /* U+1F33E */
  0x12E8,   /* U+1F33F */
  0x12E9,   /* U+1F340 */
  0x12EA,   /* U+1F341 */
  0x12EB,   /* U+1F342 */
  0x12EC,   /* U+1F343 */
  0x12ED,   /* U+1F344 */
  0x12EE,   /* U+1F345 */
  0x12EF,   /* U+1F346 */
  0x12F0,   /* U+1F347 */
  0x12F1,   /* U+1F348 */
  0x12F2,   /* U+1F349 */
  0x12F3,   /* U+1F34A */
  0x12F4,   /* U+1F34B */
  0x12F5,   /* U+1F34C */
  0x12F6,   /* U+1F34D */
  0x12F7,   /* U+1F34E */
  0x12F8,   /* U+1F34F */
  0x12F9,   /* U+1F350 */
  0x12FA,   /* U+1F351 */
  0x12FB,   /* U+1F352 */
  0x12FC,   /* U+1F353 */
  0x12FD,   /* U+1F354 */
  0x12FE,   /* U+1F355 */
  0x12FF,   /* U+1F356 */
  0x1300,   /* U+1F357 */
  0x1301,   /* U+1F358 */
  0x1302,   /* U+1F359 */
  0x1303,   /* U+1F35A */
  0x1304,   /* U+1F35B */
  0x1305,   /* U+1F35C */
  0x1306,   /* U+1F35D */
  0x1307,   /* U+1F35E */
  0x1308,   /* U+1F35F */
  0x1309,   /* U+1F360 */
  0x130A,   /* U+1F361 */
  0x130B,   /* U+1F362 */
  0x130C,   /* U+1F363 */
  0x130D,   /* U+1F364 */
  0x130E,   /* U+1F365 */
  0x130F,   /* U+1F366 */
  0x1310,   /* U+1F367 */
  0x1311,   /* U+1F368 */
  0x1312,   /* U+1F369 */
  0x1313,   /* U+1F36A */
  0x1314,   /* U+1F36B */
  0x1315,   /* U+1F36C */
  0x1316,   /* U+1F36D */
  0x1317,   /* U+1F36E */
  0x1318,   /* U+1F36F */
  0x1319,   /* U+1F370 */
  0x131A,   /* U+1F371 */
  0x131B,   /* U+1F372 */
  0x131C,   /* U+1F373 */
  0x131D,   /* U+1F374 */
  0x131E,   /* U+1F375 */
  0x131F,   /* U+1F376 */
  0x1320,   /* U+1F377 */
  0x1321,   /* U+1F378 */
  0x1322,   /* U+1F379 */
  0x1323,   /* U+1F37A */
  0x1324,   /* U+1F37B */
  0x1325,   /* U+1F37C */
  0x1326,   /* U+1F37D */
  0x1327,   /* U+1F37E */
  0x1328,   /* U+1F37F */
  0x1329,   /* U+1F380 */
  0x132A,   /* U+1F381 */
  0x132B,   /* U+1F382 */
  0x132C,   /* U+1F383 */
  0x132D,   /* U+1F384 */
  0x132E,   /* U+1F385 */
  0x132F,   /* U+1F386 */
  0x1330,   /* U+1F387 */
  0x1331,   /* U+1F388 */
  0x1332,   /* U+1F389 */
  0x1333,   /* U+1F38A */
  0x1334,   /* U+1F38B */
  0x1335,   /* U+1F38C */
  0x1336,   /* U+1F38D */
  0x1337,   /* U+1F38E */
  0x1338,   /* U+1F38F */
  0x1339,   /* U+1F390 */
  0x133A,   /* U+1F391 */
  0x133B,   /* U+1F392 */
  0x133C,   /* U+1F393 */
  0x133D,   /* U+1F394 */
  0x133E,   /* U+1F395 */
  0x133F,   /* U+1F396 */
  0x1340,   /* U+1F397 */
  0x1341,   /* U+1F398 */
  0x1342,   /* U+1F399 */
  0x1343,   /* U+1F39A */
  0x1344,   /* U+1F39B */
  0x1345,   /* U+1F39C */
  0x1346,   /* U+1F39D */
  0x1347,   /* U+1F39E */
  0x1348,   /* U+1F39F */
  0x1349,   /* U+1F3A0 */
  0x134A,   /* U+1F3A1 */
  0x134B,   /* U+1F3A2 */
  0x134C,   /* U+1F3A3 */
  0x134D,   /* U+1F3A4 */
  0x134E,   /* U+1F3A5 */
  0x134F,   /* U+1F3A6 */
  0x1350,   /* U+1F3A7 */
  0x1351,   /* U+1F3A8 */
  0x1352,   /* U+1F3A9 */
  0x1353,   /* U+1F3AA */
  0x1354,   /* U+1F3AB */
  0x1355,   /* U+1F3AC */
  0x1356,   /* U+1F3AD */
  0x1357,   /* U+1F3AE */
  0x1358,   /* U+1F3AF */
  0x1359,   /* U+1F3B0 */
  0x135A,   /* U+1F3B1 */
  0x135B,   /* U+1F3B2 */
  0x135C,   /* U+1F3B3 */
  0x135D,   /* U+1F3B4 */
  0x135E,   /* U+1F3B5 */
  0x135F,   /* U+1F3B6 */
  0x1360,   /* U+1F3B7 */
  0x1361,   /* U+1F3B8 */
  0x1362,   /* U+1F3B9 */
  0x1363,   /* U+1F3BA */
  0x1364,   /* U+1F3BB */
  0x1365,   /* U+1F3BC */
  0x1366,   /* U+1F3BD */
  0x1367,   /* U+1F3BE */
  0x1368,   /* U+1F3BF */
  0x1369,   /* U+1F3C0 */
  0x136A,   /* U+1F3C1 */
  0x136B,   /* U+1F3C2 */
  0x136C,   /* U+1F3C3 */
  0x136D,   /* U+1F3C4 */
  0x136E,   /* U+1F3C5 */
  0x136F,   /* U+1F3C6 */
  0x1370,   /* U+1F3C7 */
  0x1371,   /* U+1F3C8 */
  0x1372,   /* U+1F3C9 */
  0x1373,   /* U+1F3CA */
  0x1374,   /* U+1F3CB */
  0x1375,   /* U+1F3CC */
  0x1376,   /* U+1F3CD */
  0x1377,   /* U+1F3CE */
  0x1378,   /* U+1F3CF */
  0x1379,   /* U+1F3D0 */
  0x137A,   /* U+1F3D1 */
  0x137B,   /* U+1F3D2 */
  0x137C,   /* U+1F3D3 */
  0x137D,   /* U+1F3D4 */
  0x137E,   /* U+1F3D5 */
  0x137F,   /* U+1F3D6 */
  0x1380,   /* U+1F3D7 */
  0x1381,   /* U+1F3D8 */
  0x1382,   /* U+1F3D9 */
  0x1383,   /* U+1F3DA */
  0x1384,   /* U+1F3DB */
  0x1385,   /* U+1F3DC */
  0x1386,   /* U+1F3DD */
  0x1387,   /* U+1F3DE */
  0x1388,   /* U+1F3DF */
  0x1389,   /* U+1F3E0 */
  0x138A,   /* U+1F3E1 */
  0x138B,   /* U+1F3E2 */
  0x138C,   /* U+1F3E3 */
  0x138D,   /* U+1F3E4 */
  0x138E,   /* U+1F3E5 */
  0x138F,   /* U+1F3E6 */
  0x1390,   /* U+1F3E7 */
  0x1391,   /* U+1F3E8 */
  0x1392,   /* U+1F3E9 */
  0x1393,   /* U+1F3EA */
  0x1394,   /* U+1F3EB */
  0x1395,   /* U+1F3EC */
  0x1396,   /* U+1F3ED */
  0x1397,   /* U+1F3EE */
  0x1398,   /* U+1F3EF */
  0x1399,   /* U+1F3F0 */
  0x139A,   /* U+1F3F1 */
  0x139B,   /* U+1F3F2 */
  0x139C,   /* U+1F3F3 */
  0x139D,   /* U+1F3F4 */
  0x139E,   /* U+1F3F5 */
  0x139F,   /* U+1F3F6 */
  0x13A0,   /* U+1F3F7 */
  0x13A1,   /* U+1F3F8 */
  0x13A2,   /* U+1F3F9 */
  0x13A3,   /* U+1F3FA */
  0x13A4,   /* U+1F3FB */
  0x13A5,   /* U+1F3FC */
  0x13A6,   /* U+1F3FD */
  0x13A7,   /* U+1F3FE */
  0x13A8,   /* U+1F3FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F300 */
  0x0020,   /* U+1F301 */
  0x0020,   /* U+1F302 */
  0x0020,   /* U+1F303 */
  0x0020,   /* U+1F304 */
  0x0020,   /* U+1F305 */
  0x0020,   /* U+1F306 */
  0x0020,   /* U+1F307 */
  0x0020,   /* U+1F308 */
  0x0020,   /* U+1F309 */
  0x0020,   /* U+1F30A */
  0x0020,   /* U+1F30B */
  0x0020,   /* U+1F30C */
  0x0020,   /* U+1F30D */
  0x0020,   /* U+1F30E */
  0x0020,   /* U+1F30F */
  0x0020,   /* U+1F310 */
  0x0020,   /* U+1F311 */
  0x0020,   /* U+1F312 */
  0x0020,   /* U+1F313 */
  0x0020,   /* U+1F314 */
  0x0020,   /* U+1F315 */
  0x0020,   /* U+1F316 */
  0x0020,   /* U+1F317 */
  0x0020,   /* U+1F318 */
  0x0020,   /* U+1F319 */
  0x0020,   /* U+1F31A */
  0x0020,   /* U+1F31B */
  0x0020,   /* U+1F31C */
  0x0020,   /* U+1F31D */
  0x0020,   /* U+1F31E */
  0x0020,   /* U+1F31F */
  0x0020,   /* U+1F320 */
  0x0020,   /* U+1F321 */
  0x0020,   /* U+1F322 */
  0x0020,   /* U+1F323 */
  0x0020,   /* U+1F324 */
  0x0020,   /* U+1F325 */
  0x0020,   /* U+1F326 */
  0x0020,   /* U+1F327 */
  0x0020,   /* U+1F328 */
  0x0020,   /* U+1F329 */
  0x0020,   /* U+1F32A */
  0x0020,   /* U+1F32B */
  0x0020,   /* U+1F32C */
  0x0020,   /* U+1F32D */
  0x0020,   /* U+1F32E */
  0x0020,   /* U+1F32F */
  0x0020,   /* U+1F330 */
  0x0020,   /* U+1F331 */
  0x0020,   /* U+1F332 */
  0x0020,   /* U+1F333 */
  0x0020,   /* U+1F334 */
  0x0020,   /* U+1F335 */
  0x0020,   /* U+1F336 */
  0x0020,   /* U+1F337 */
  0x0020,   /* U+1F338 */
  0x0020,   /* U+1F339 */
  0x0020,   /* U+1F33A */
  0x0020,   /* U+1F33B */
  0x0020,   /* U+1F33C */
  0x0020,   /* U+1F33D */
  0x0020,   /* U+1F33E */
  0x0020,   /* U+1F33F */
  0x0020,   /* U+1F340 */
  0x0020,   /* U+1F341 */
  0x0020,   /* U+1F342 */
  0x0020,   /* U+1F343 */
  0x0020,   /* U+1F344 */
  0x0020,   /* U+1F345 */
  0x0020,   /* U+1F346 */
  0x0020,   /* U+1F347 */
  0x0020,   /* U+1F348 */
  0x0020,   /* U+1F349 */
  0x0020,   /* U+1F34A */
  0x0020,   /* U+1F34B */
  0x0020,   /* U+1F34C */
  0x0020,   /* U+1F34D */
  0x0020,   /* U+1F34E */
  0x0020,   /* U+1F34F */
  0x0020,   /* U+1F350 */
  0x0020,   /* U+1F351 */
  0x0020,   /* U+1F352 */
  0x0020,   /* U+1F353 */
  0x0020,   /* U+1F354 */
  0x0020,   /* U+1F355 */
  0x0020,   /* U+1F356 */
  0x0020,   /* U+1F357 */
  0x0020,   /* U+1F358 */
  0x0020,   /* U+1F359 */
  0x0020,   /* U+1F35A */
  0x0020,   /* U+1F35B */
  0x0020,   /* U+1F35C */
  0x0020,   /* U+1F35D */
  0x0020,   /* U+1F35E */
  0x0020,   /* U+1F35F */
  0x0020,   /* U+1F360 */
  0x0020,   /* U+1F361 */
  0x0020,   /* U+1F362 */
  0x0020,   /* U+1F363 */
  0x0020,   /* U+1F364 */
  0x0020,   /* U+1F365 */
  0x0020,   /* U+1F366 */
  0x0020,   /* U+1F367 */
  0x0020,   /* U+1F368 */
  0x0020,   /* U+1F369 */
  0x0020,   /* U+1F36A */
  0x0020,   /* U+1F36B */
  0x0020,   /* U+1F36C */
  0x0020,   /* U+1F36D */
  0x0020,   /* U+1F36E */
  0x0020,   /* U+1F36F */
  0x0020,   /* U+1F370 */
  0x0020,   /* U+1F371 */
  0x0020,   /* U+1F372 */
  0x0020,   /* U+1F373 */
  0x0020,   /* U+1F374 */
  0x0020,   /* U+1F375 */
  0x0020,   /* U+1F376 */
  0x0020,   /* U+1F377 */
  0x0020,   /* U+1F378 */
  0x0020,   /* U+1F379 */
  0x0020,   /* U+1F37A */
  0x0020,   /* U+1F37B */
  0x0020,   /* U+1F37C */
  0x0020,   /* U+1F37D */
  0x0020,   /* U+1F37E */
  0x0020,   /* U+1F37F */
  0x0020,   /* U+1F380 */
  0x0020,   /* U+1F381 */
  0x0020,   /* U+1F382 */
  0x0020,   /* U+1F383 */
  0x0020,   /* U+1F384 */
  0x0020,   /* U+1F385 */
  0x0020,   /* U+1F386 */
  0x0020,   /* U+1F387 */
  0x0020,   /* U+1F388 */
  0x0020,   /* U+1F389 */
  0x0020,   /* U+1F38A */
  0x0020,   /* U+1F38B */
  0x0020,   /* U+1F38C */
  0x0020,   /* U+1F38D */
  0x0020,   /* U+1F38E */
  0x0020,   /* U+1F38F */
  0x0020,   /* U+1F390 */
  0x0020,   /* U+1F391 */
  0x0020,   /* U+1F392 */
  0x0020,   /* U+1F393 */
  0x0020,   /* U+1F394 */
  0x0020,   /* U+1F395 */
  0x0020,   /* U+1F396 */
  0x0020,   /* U+1F397 */
  0x0020,   /* U+1F398 */
  0x0020,   /* U+1F399 */
  0x0020,   /* U+1F39A */
  0x0020,   /* U+1F39B */
  0x0020,   /* U+1F39C */
  0x0020,   /* U+1F39D */
  0x0020,   /* U+1F39E */
  0x0020,   /* U+1F39F */
  0x0020,   /* U+1F3A0 */
  0x0020,   /* U+1F3A1 */
  0x0020,   /* U+1F3A2 */
  0x0020,   /* U+1F3A3 */
  0x0020,   /* U+1F3A4 */
  0x0020,   /* U+1F3A5 */
  0x0020,   /* U+1F3A6 */
  0x0020,   /* U+1F3A7 */
  0x0020,   /* U+1F3A8 */
  0x0020,   /* U+1F3A9 */
  0x0020,   /* U+1F3AA */
  0x0020,   /* U+1F3AB */
  0x0020,   /* U+1F3AC */
  0x0020,   /* U+1F3AD */
  0x0020,   /* U+1F3AE */
  0x0020,   /* U+1F3AF */
  0x0020,   /* U+1F3B0 */
  0x0020,   /* U+1F3B1 */
  0x0020,   /* U+1F3B2 */
  0x0020,   /* U+1F3B3 */
  0x0020,   /* U+1F3B4 */
  0x0020,   /* U+1F3B5 */
  0x0020,   /* U+1F3B6 */
  0x0020,   /* U+1F3B7 */
  0x0020,   /* U+1F3B8 */
  0x0020,   /* U+1F3B9 */
  0x0020,   /* U+1F3BA */
  0x0020,   /* U+1F3BB */
  0x0020,   /* U+1F3BC */
  0x0020,   /* U+1F3BD */
  0x0020,   /* U+1F3BE */
  0x0020,   /* U+1F3BF */
  0x0020,   /* U+1F3C0 */
  0x0020,   /* U+1F3C1 */
  0x0020,   /* U+1F3C2 */
  0x0020,   /* U+1F3C3 */
  0x0020,   /* U+1F3C4 */
  0x0020,   /* U+1F3C5 */
  0x0020,   /* U+1F3C6 */
  0x0020,   /* U+1F3C7 */
  0x0020,   /* U+1F3C8 */
  0x0020,   /* U+1F3C9 */
  0x0020,   /* U+1F3CA */
  0x0020,   /* U+1F3CB */
  0x0020,   /* U+1F3CC */
  0x0020,   /* U+1F3CD */
  0x0020,   /* U+1F3CE */
  0x0020,   /* U+1F3CF */
  0x0020,   /* U+1F3D0 */
  0x0020,   /* U+1F3D1 */
  0x0020,   /* U+1F3D2 */
  0x0020,   /* U+1F3D3 */
  0x0020,   /* U+1F3D4 */
  0x0020,   /* U+1F3D5 */
  0x0020,   /* U+1F3D6 */
  0x0020,   /* U+1F3D7 */
  0x0020,   /* U+1F3D8 */
  0x0020,   /* U+1F3D9 */
  0x0020,   /* U+1F3DA */
  0x0020,   /* U+1F3DB */
  0x0020,   /* U+1F3DC */
  0x0020,   /* U+1F3DD */
  0x0020,   /* U+1F3DE */
  0x0020,   /* U+1F3DF */
  0x0020,   /* U+1F3E0 */
  0x0020,   /* U+1F3E1 */
  0x0020,   /* U+1F3E2 */
  0x0020,   /* U+1F3E3 */
  0x0020,   /* U+1F3E4 */
  0x0020,   /* U+1F3E5 */
  0x0020,   /* U+1F3E6 */
  0x0020,   /* U+1F3E7 */
  0x0020,   /* U+1F3E8 */
  0x0020,   /* U+1F3E9 */
  0x0020,   /* U+1F3EA */
  0x0020,   /* U+1F3EB */
  0x0020,   /* U+1F3EC */
  0x0020,   /* U+1F3ED */
  0x0020,   /* U+1F3EE */
  0x0020,   /* U+1F3EF */
  0x0020,   /* U+1F3F0 */
  0x0020,   /* U+1F3F1 */
  0x0020,   /* U+1F3F2 */
  0x0020,   /* U+1F3F3 */
  0x0020,   /* U+1F3F4 */
  0x0020,   /* U+1F3F5 */
  0x0020,   /* U+1F3F6 */
  0x0020,   /* U+1F3F7 */
  0x0020,   /* U+1F3F8 */
  0x0020,   /* U+1F3F9 */
  0x0020,   /* U+1F3FA */
  0x0020,   /* U+1F3FB */
  0x0020,   /* U+1F3FC */
  0x0020,   /* U+1F3FD */
  0x0020,   /* U+1F3FE */
  0x0020,   /* U+1F3FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F300 */
  0x0002,   /* U+1F301 */
  0x0002,   /* U+1F302 */
  0x0002,   /* U+1F303 */
  0x0002,   /* U+1F304 */
  0x0002,   /* U+1F305 */
  0x0002,   /* U+1F306 */
  0x0002,   /* U+1F307 */
  0x0002,   /* U+1F308 */
  0x0002,   /* U+1F309 */
  0x0002,   /* U+1F30A */
  0x0002,   /* U+1F30B */
  0x0002,   /* U+1F30C */
  0x0002,   /* U+1F30D */
  0x0002,   /* U+1F30E */
  0x0002,   /* U+1F30F */
  0x0002,   /* U+1F310 */
  0x0002,   /* U+1F311 */
  0x0002,   /* U+1F312 */
  0x0002,   /* U+1F313 */
  0x0002,   /* U+1F314 */
  0x0002,   /* U+1F315 */
  0x0002,   /* U+1F316 */
  0x0002,   /* U+1F317 */
  0x0002,   /* U+1F318 */
  0x0002,   /* U+1F319 */
  0x0002,   /* U+1F31A */
  0x0002,   /* U+1F31B */
  0x0002,   /* U+1F31C */
  0x0002,   /* U+1F31D */
  0x0002,   /* U+1F31E */
  0x0002,   /* U+1F31F */
  0x0002,   /* U+1F320 */
  0x0002,   /* U+1F321 */
  0x0002,   /* U+1F322 */
  0x0002,   /* U+1F323 */
  0x0002,   /* U+1F324 */
  0x0002,   /* U+1F325 */
  0x0002,   /* U+1F326 */
  0x0002,   /* U+1F327 */
  0x0002,   /* U+1F328 */
  0x0002,   /* U+1F329 */
  0x0002,   /* U+1F32A */
  0x0002,   /* U+1F32B */
  0x0002,   /* U+1F32C */
  0x0002,   /* U+1F32D */
  0x0002,   /* U+1F32E */
  0x0002,   /* U+1F32F */
  0x0002,   /* U+1F330 */
  0x0002,   /* U+1F331 */
  0x0002,   /* U+1F332 */
  0x0002,   /* U+1F333 */
  0x0002,   /* U+1F334 */
  0x0002,   /* U+1F335 */
  0x0002,   /* U+1F336 */
  0x0002,   /* U+1F337 */
  0x0002,   /* U+1F338 */
  0x0002,   /* U+1F339 */
  0x0002,   /* U+1F33A */
  0x0002,   /* U+1F33B */
  0x0002,   /* U+1F33C */
  0x0002,   /* U+1F33D */
  0x0002,   /* U+1F33E */
  0x0002,   /* U+1F33F */
  0x0002,   /* U+1F340 */
  0x0002,   /* U+1F341 */
  0x0002,   /* U+1F342 */
  0x0002,   /* U+1F343 */
  0x0002,   /* U+1F344 */
  0x0002,   /* U+1F345 */
  0x0002,   /* U+1F346 */
  0x0002,   /* U+1F347 */
  0x0002,   /* U+1F348 */
  0x0002,   /* U+1F349 */
  0x0002,   /* U+1F34A */
  0x0002,   /* U+1F34B */
  0x0002,   /* U+1F34C */
  0x0002,   /* U+1F34D */
  0x0002,   /* U+1F34E */
  0x0002,   /* U+1F34F */
  0x0002,   /* U+1F350 */
  0x0002,   /* U+1F351 */
  0x0002,   /* U+1F352 */
  0x0002,   /* U+1F353 */
  0x0002,   /* U+1F354 */
  0x0002,   /* U+1F355 */
  0x0002,   /* U+1F356 */
  0x0002,   /* U+1F357 */
  0x0002,   /* U+1F358 */
  0x0002,   /* U+1F359 */
  0x0002,   /* U+1F35A */
  0x0002,   /* U+1F35B */
  0x0002,   /* U+1F35C */
  0x0002,   /* U+1F35D */
  0x0002,   /* U+1F35E */
  0x0002,   /* U+1F35F */
  0x0002,   /* U+1F360 */
  0x0002,   /* U+1F361 */
  0x0002,   /* U+1F362 */
  0x0002,   /* U+1F363 */
  0x0002,   /* U+1F364 */
  0x0002,   /* U+1F365 */
  0x0002,   /* U+1F366 */
  0x0002,   /* U+1F367 */
  0x0002,   /* U+1F368 */
  0x0002,   /* U+1F369 */
  0x0002,   /* U+1F36A */
  0x0002,   /* U+1F36B */
  0x0002,   /* U+1F36C */
  0x0002,   /* U+1F36D */
  0x0002,   /* U+1F36E */
  0x0002,   /* U+1F36F */
  0x0002,   /* U+1F370 */
  0x0002,   /* U+1F371 */
  0x0002,   /* U+1F372 */
  0x0002,   /* U+1F373 */
  0x0002,   /* U+1F374 */
  0x0002,   /* U+1F375 */
  0x0002,   /* U+1F376 */
  0x0002,   /* U+1F377 */
  0x0002,   /* U+1F378 */
  0x0002,   /* U+1F379 */
  0x0002,   /* U+1F37A */
  0x0002,   /* U+1F37B */
  0x0002,   /* U+1F37C */
  0x0002,   /* U+1F37D */
  0x0002,   /* U+1F37E */
  0x0002,   /* U+1F37F */
  0x0002,   /* U+1F380 */
  0x0002,   /* U+1F381 */
  0x0002,   /* U+1F382 */
  0x0002,   /* U+1F383 */
  0x0002,   /* U+1F384 */
  0x0002,   /* U+1F385 */
  0x0002,   /* U+1F386 */
  0x0002,   /* U+1F387 */
  0x0002,   /* U+1F388 */
  0x0002,   /* U+1F389 */
  0x0002,   /* U+1F38A */
  0x0002,   /* U+1F38B */
  0x0002,   /* U+1F38C */
  0x0002,   /* U+1F38D */
  0x0002,   /* U+1F38E */
  0x0002,   /* U+1F38F */
  0x0002,   /* U+1F390 */
  0x0002,   /* U+1F391 */
  0x0002,   /* U+1F392 */
  0x0002,   /* U+1F393 */
  0x0002,   /* U+1F394 */
  0x0002,   /* U+1F395 */
  0x0002,   /* U+1F396 */
  0x0002,   /* U+1F397 */
  0x0002,   /* U+1F398 */
  0x0002,   /* U+1F399 */
  0x0002,   /* U+1F39A */
  0x0002,   /* U+1F39B */
  0x0002,   /* U+1F39C */
  0x0002,   /* U+1F39D */
  0x0002,   /* U+1F39E */
  0x0002,   /* U+1F39F */
  0x0002,   /* U+1F3A0 */
  0x0002,   /* U+1F3A1 */
  0x0002,   /* U+1F3A2 */
  0x0002,   /* U+1F3A3 */
  0x0002,   /* U+1F3A4 */
  0x0002,   /* U+1F3A5 */
  0x0002,   /* U+1F3A6 */
  0x0002,   /* U+1F3A7 */
  0x0002,   /* U+1F3A8 */
  0x0002,   /* U+1F3A9 */
  0x0002,   /* U+1F3AA */
  0x0002,   /* U+1F3AB */
  0x0002,   /* U+1F3AC */
  0x0002,   /* U+1F3AD */
  0x0002,   /* U+1F3AE */
  0x0002,   /* U+1F3AF */
  0x0002,   /* U+1F3B0 */
  0x0002,   /* U+1F3B1 */
  0x0002,   /* U+1F3B2 */
  0x0002,   /* U+1F3B3 */
  0x0002,   /* U+1F3B4 */
  0x0002,   /* U+1F3B5 */
  0x0002,   /* U+1F3B6 */
  0x0002,   /* U+1F3B7 */
  0x0002,   /* U+1F3B8 */
  0x0002,   /* U+1F3B9 */
  0x0002,   /* U+1F3BA */
  0x0002,   /* U+1F3BB */
  0x0002,   /* U+1F3BC */
  0x0002,   /* U+1F3BD */
  0x0002,   /* U+1F3BE */
  0x0002,   /* U+1F3BF */
  0x0002,   /* U+1F3C0 */
  0x0002,   /* U+1F3C1 */
  0x0002,   /* U+1F3C2 */
  0x0002,   /* U+1F3C3 */
  0x0002,   /* U+1F3C4 */
  0x0002,   /* U+1F3C5 */
  0x0002,   /* U+1F3C6 */
  0x0002,   /* U+1F3C7 */
  0x0002,   /* U+1F3C8 */
  0x0002,   /* U+1F3C9 */
  0x0002,   /* U+1F3CA */
  0x0002,   /* U+1F3CB */
  0x0002,   /* U+1F3CC */
  0x0002,   /* U+1F3CD */
  0x0002,   /* U+1F3CE */
  0x0002,   /* U+1F3CF */
  0x0002,   /* U+1F3D0 */
  0x0002,   /* U+1F3D1 */
  0x0002,   /* U+1F3D2 */
  0x0002,   /* U+1F3D3 */
  0x0002,   /* U+1F3D4 */
  0x0002,   /* U+1F3D5 */
  0x0002,   /* U+1F3D6 */
  0x0002,   /* U+1F3D7 */
  0x0002,   /* U+1F3D8 */
  0x0002,   /* U+1F3D9 */
  0x0002,   /* U+1F3DA */
  0x0002,   /* U+1F3DB */
  0x0002,   /* U+1F3DC */
  0x0002,   /* U+1F3DD */
  0x0002,   /* U+1F3DE */
  0x0002,   /* U+1F3DF */
  0x0002,   /* U+1F3E0 */
  0x0002,   /* U+1F3E1 */
  0x0002,   /* U+1F3E2 */
  0x0002,   /* U+1F3E3 */
  0x0002,   /* U+1F3E4 */
  0x0002,   /* U+1F3E5 */
  0x0002,   /* U+1F3E6 */
  0x0002,   /* U+1F3E7 */
  0x0002,   /* U+1F3E8 */
  0x0002,   /* U+1F3E9 */
  0x0002,   /* U+1F3EA */
  0x0002,   /* U+1F3EB */
  0x0002,   /* U+1F3EC */
  0x0002,   /* U+1F3ED */
  0x0002,   /* U+1F3EE */
  0x0002,   /* U+1F3EF */
  0x0002,   /* U+1F3F0 */
  0x0002,   /* U+1F3F1 */
  0x0002,   /* U+1F3F2 */
  0x0002,   /* U+1F3F3 */
  0x0002,   /* U+1F3F4 */
  0x0002,   /* U+1F3F5 */
  0x0002,   /* U+1F3F6 */
  0x0002,   /* U+1F3F7 */
  0x0002,   /* U+1F3F8 */
  0x0002,   /* U+1F3F9 */
  0x0002,   /* U+1F3FA */
  0x0002,   /* U+1F3FB */
  0x0002,   /* U+1F3FC */
  0x0002,   /* U+1F3FD */
  0x0002,   /* U+1F3FE */
  0x0002,   /* U+1F3FF */
};

uint16 uca900_p1F4[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x13A9,   /* U+1F400 */
  0x13AA,   /* U+1F401 */
  0x13AB,   /* U+1F402 */
  0x13AC,   /* U+1F403 */
  0x13AD,   /* U+1F404 */
  0x13AE,   /* U+1F405 */
  0x13AF,   /* U+1F406 */
  0x13B0,   /* U+1F407 */
  0x13B1,   /* U+1F408 */
  0x13B2,   /* U+1F409 */
  0x13B3,   /* U+1F40A */
  0x13B4,   /* U+1F40B */
  0x13B5,   /* U+1F40C */
  0x13B6,   /* U+1F40D */
  0x13B7,   /* U+1F40E */
  0x13B8,   /* U+1F40F */
  0x13B9,   /* U+1F410 */
  0x13BA,   /* U+1F411 */
  0x13BB,   /* U+1F412 */
  0x13BC,   /* U+1F413 */
  0x13BD,   /* U+1F414 */
  0x13BE,   /* U+1F415 */
  0x13BF,   /* U+1F416 */
  0x13C0,   /* U+1F417 */
  0x13C1,   /* U+1F418 */
  0x13C2,   /* U+1F419 */
  0x13C3,   /* U+1F41A */
  0x13C4,   /* U+1F41B */
  0x13C5,   /* U+1F41C */
  0x13C6,   /* U+1F41D */
  0x13C7,   /* U+1F41E */
  0x13C8,   /* U+1F41F */
  0x13C9,   /* U+1F420 */
  0x13CA,   /* U+1F421 */
  0x13CB,   /* U+1F422 */
  0x13CC,   /* U+1F423 */
  0x13CD,   /* U+1F424 */
  0x13CE,   /* U+1F425 */
  0x13CF,   /* U+1F426 */
  0x13D0,   /* U+1F427 */
  0x13D1,   /* U+1F428 */
  0x13D2,   /* U+1F429 */
  0x13D3,   /* U+1F42A */
  0x13D4,   /* U+1F42B */
  0x13D5,   /* U+1F42C */
  0x13D6,   /* U+1F42D */
  0x13D7,   /* U+1F42E */
  0x13D8,   /* U+1F42F */
  0x13D9,   /* U+1F430 */
  0x13DA,   /* U+1F431 */
  0x13DB,   /* U+1F432 */
  0x13DC,   /* U+1F433 */
  0x13DD,   /* U+1F434 */
  0x13DE,   /* U+1F435 */
  0x13DF,   /* U+1F436 */
  0x13E0,   /* U+1F437 */
  0x13E1,   /* U+1F438 */
  0x13E2,   /* U+1F439 */
  0x13E3,   /* U+1F43A */
  0x13E4,   /* U+1F43B */
  0x13E5,   /* U+1F43C */
  0x13E6,   /* U+1F43D */
  0x13E7,   /* U+1F43E */
  0x13E8,   /* U+1F43F */
  0x13E9,   /* U+1F440 */
  0x13EA,   /* U+1F441 */
  0x13EB,   /* U+1F442 */
  0x13EC,   /* U+1F443 */
  0x13ED,   /* U+1F444 */
  0x13EE,   /* U+1F445 */
  0x13EF,   /* U+1F446 */
  0x13F0,   /* U+1F447 */
  0x13F1,   /* U+1F448 */
  0x13F2,   /* U+1F449 */
  0x13F3,   /* U+1F44A */
  0x13F4,   /* U+1F44B */
  0x13F5,   /* U+1F44C */
  0x13F6,   /* U+1F44D */
  0x13F7,   /* U+1F44E */
  0x13F8,   /* U+1F44F */
  0x13F9,   /* U+1F450 */
  0x13FA,   /* U+1F451 */
  0x13FB,   /* U+1F452 */
  0x13FC,   /* U+1F453 */
  0x13FD,   /* U+1F454 */
  0x13FE,   /* U+1F455 */
  0x13FF,   /* U+1F456 */
  0x1400,   /* U+1F457 */
  0x1401,   /* U+1F458 */
  0x1402,   /* U+1F459 */
  0x1403,   /* U+1F45A */
  0x1404,   /* U+1F45B */
  0x1405,   /* U+1F45C */
  0x1406,   /* U+1F45D */
  0x1407,   /* U+1F45E */
  0x1408,   /* U+1F45F */
  0x1409,   /* U+1F460 */
  0x140A,   /* U+1F461 */
  0x140B,   /* U+1F462 */
  0x140C,   /* U+1F463 */
  0x140D,   /* U+1F464 */
  0x140E,   /* U+1F465 */
  0x140F,   /* U+1F466 */
  0x1410,   /* U+1F467 */
  0x1411,   /* U+1F468 */
  0x1412,   /* U+1F469 */
  0x1413,   /* U+1F46A */
  0x1414,   /* U+1F46B */
  0x1415,   /* U+1F46C */
  0x1416,   /* U+1F46D */
  0x1417,   /* U+1F46E */
  0x1418,   /* U+1F46F */
  0x1419,   /* U+1F470 */
  0x141A,   /* U+1F471 */
  0x141B,   /* U+1F472 */
  0x141C,   /* U+1F473 */
  0x141D,   /* U+1F474 */
  0x141E,   /* U+1F475 */
  0x141F,   /* U+1F476 */
  0x1420,   /* U+1F477 */
  0x1421,   /* U+1F478 */
  0x1422,   /* U+1F479 */
  0x1423,   /* U+1F47A */
  0x1424,   /* U+1F47B */
  0x1425,   /* U+1F47C */
  0x1426,   /* U+1F47D */
  0x1427,   /* U+1F47E */
  0x1428,   /* U+1F47F */
  0x1429,   /* U+1F480 */
  0x142A,   /* U+1F481 */
  0x142B,   /* U+1F482 */
  0x142C,   /* U+1F483 */
  0x142D,   /* U+1F484 */
  0x142E,   /* U+1F485 */
  0x142F,   /* U+1F486 */
  0x1430,   /* U+1F487 */
  0x1431,   /* U+1F488 */
  0x1432,   /* U+1F489 */
  0x1433,   /* U+1F48A */
  0x1434,   /* U+1F48B */
  0x1435,   /* U+1F48C */
  0x1436,   /* U+1F48D */
  0x1437,   /* U+1F48E */
  0x1438,   /* U+1F48F */
  0x1439,   /* U+1F490 */
  0x143A,   /* U+1F491 */
  0x143B,   /* U+1F492 */
  0x143C,   /* U+1F493 */
  0x143D,   /* U+1F494 */
  0x143E,   /* U+1F495 */
  0x143F,   /* U+1F496 */
  0x1440,   /* U+1F497 */
  0x1441,   /* U+1F498 */
  0x1442,   /* U+1F499 */
  0x1443,   /* U+1F49A */
  0x1444,   /* U+1F49B */
  0x1445,   /* U+1F49C */
  0x1446,   /* U+1F49D */
  0x1447,   /* U+1F49E */
  0x1448,   /* U+1F49F */
  0x1449,   /* U+1F4A0 */
  0x144A,   /* U+1F4A1 */
  0x144B,   /* U+1F4A2 */
  0x144C,   /* U+1F4A3 */
  0x144D,   /* U+1F4A4 */
  0x144E,   /* U+1F4A5 */
  0x144F,   /* U+1F4A6 */
  0x1450,   /* U+1F4A7 */
  0x1451,   /* U+1F4A8 */
  0x1452,   /* U+1F4A9 */
  0x1453,   /* U+1F4AA */
  0x1454,   /* U+1F4AB */
  0x1455,   /* U+1F4AC */
  0x1456,   /* U+1F4AD */
  0x1457,   /* U+1F4AE */
  0x1458,   /* U+1F4AF */
  0x1459,   /* U+1F4B0 */
  0x145A,   /* U+1F4B1 */
  0x145B,   /* U+1F4B2 */
  0x145C,   /* U+1F4B3 */
  0x145D,   /* U+1F4B4 */
  0x145E,   /* U+1F4B5 */
  0x145F,   /* U+1F4B6 */
  0x1460,   /* U+1F4B7 */
  0x1461,   /* U+1F4B8 */
  0x1462,   /* U+1F4B9 */
  0x1463,   /* U+1F4BA */
  0x1464,   /* U+1F4BB */
  0x1465,   /* U+1F4BC */
  0x1466,   /* U+1F4BD */
  0x1467,   /* U+1F4BE */
  0x1468,   /* U+1F4BF */
  0x1469,   /* U+1F4C0 */
  0x146A,   /* U+1F4C1 */
  0x146B,   /* U+1F4C2 */
  0x146C,   /* U+1F4C3 */
  0x146D,   /* U+1F4C4 */
  0x146E,   /* U+1F4C5 */
  0x146F,   /* U+1F4C6 */
  0x1470,   /* U+1F4C7 */
  0x1471,   /* U+1F4C8 */
  0x1472,   /* U+1F4C9 */
  0x1473,   /* U+1F4CA */
  0x1474,   /* U+1F4CB */
  0x1475,   /* U+1F4CC */
  0x1476,   /* U+1F4CD */
  0x1477,   /* U+1F4CE */
  0x1478,   /* U+1F4CF */
  0x1479,   /* U+1F4D0 */
  0x147A,   /* U+1F4D1 */
  0x147B,   /* U+1F4D2 */
  0x147C,   /* U+1F4D3 */
  0x147D,   /* U+1F4D4 */
  0x147E,   /* U+1F4D5 */
  0x147F,   /* U+1F4D6 */
  0x1480,   /* U+1F4D7 */
  0x1481,   /* U+1F4D8 */
  0x1482,   /* U+1F4D9 */
  0x1483,   /* U+1F4DA */
  0x1484,   /* U+1F4DB */
  0x1485,   /* U+1F4DC */
  0x1486,   /* U+1F4DD */
  0x1487,   /* U+1F4DE */
  0x1488,   /* U+1F4DF */
  0x1489,   /* U+1F4E0 */
  0x148A,   /* U+1F4E1 */
  0x148B,   /* U+1F4E2 */
  0x148C,   /* U+1F4E3 */
  0x148D,   /* U+1F4E4 */
  0x148E,   /* U+1F4E5 */
  0x148F,   /* U+1F4E6 */
  0x1490,   /* U+1F4E7 */
  0x1491,   /* U+1F4E8 */
  0x1492,   /* U+1F4E9 */
  0x1493,   /* U+1F4EA */
  0x1494,   /* U+1F4EB */
  0x1495,   /* U+1F4EC */
  0x1496,   /* U+1F4ED */
  0x1497,   /* U+1F4EE */
  0x1498,   /* U+1F4EF */
  0x1499,   /* U+1F4F0 */
  0x149A,   /* U+1F4F1 */
  0x149B,   /* U+1F4F2 */
  0x149C,   /* U+1F4F3 */
  0x149D,   /* U+1F4F4 */
  0x149E,   /* U+1F4F5 */
  0x149F,   /* U+1F4F6 */
  0x14A0,   /* U+1F4F7 */
  0x14A1,   /* U+1F4F8 */
  0x14A2,   /* U+1F4F9 */
  0x14A3,   /* U+1F4FA */
  0x14A4,   /* U+1F4FB */
  0x14A5,   /* U+1F4FC */
  0x14A6,   /* U+1F4FD */
  0x14A7,   /* U+1F4FE */
  0x14A8,   /* U+1F4FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F400 */
  0x0020,   /* U+1F401 */
  0x0020,   /* U+1F402 */
  0x0020,   /* U+1F403 */
  0x0020,   /* U+1F404 */
  0x0020,   /* U+1F405 */
  0x0020,   /* U+1F406 */
  0x0020,   /* U+1F407 */
  0x0020,   /* U+1F408 */
  0x0020,   /* U+1F409 */
  0x0020,   /* U+1F40A */
  0x0020,   /* U+1F40B */
  0x0020,   /* U+1F40C */
  0x0020,   /* U+1F40D */
  0x0020,   /* U+1F40E */
  0x0020,   /* U+1F40F */
  0x0020,   /* U+1F410 */
  0x0020,   /* U+1F411 */
  0x0020,   /* U+1F412 */
  0x0020,   /* U+1F413 */
  0x0020,   /* U+1F414 */
  0x0020,   /* U+1F415 */
  0x0020,   /* U+1F416 */
  0x0020,   /* U+1F417 */
  0x0020,   /* U+1F418 */
  0x0020,   /* U+1F419 */
  0x0020,   /* U+1F41A */
  0x0020,   /* U+1F41B */
  0x0020,   /* U+1F41C */
  0x0020,   /* U+1F41D */
  0x0020,   /* U+1F41E */
  0x0020,   /* U+1F41F */
  0x0020,   /* U+1F420 */
  0x0020,   /* U+1F421 */
  0x0020,   /* U+1F422 */
  0x0020,   /* U+1F423 */
  0x0020,   /* U+1F424 */
  0x0020,   /* U+1F425 */
  0x0020,   /* U+1F426 */
  0x0020,   /* U+1F427 */
  0x0020,   /* U+1F428 */
  0x0020,   /* U+1F429 */
  0x0020,   /* U+1F42A */
  0x0020,   /* U+1F42B */
  0x0020,   /* U+1F42C */
  0x0020,   /* U+1F42D */
  0x0020,   /* U+1F42E */
  0x0020,   /* U+1F42F */
  0x0020,   /* U+1F430 */
  0x0020,   /* U+1F431 */
  0x0020,   /* U+1F432 */
  0x0020,   /* U+1F433 */
  0x0020,   /* U+1F434 */
  0x0020,   /* U+1F435 */
  0x0020,   /* U+1F436 */
  0x0020,   /* U+1F437 */
  0x0020,   /* U+1F438 */
  0x0020,   /* U+1F439 */
  0x0020,   /* U+1F43A */
  0x0020,   /* U+1F43B */
  0x0020,   /* U+1F43C */
  0x0020,   /* U+1F43D */
  0x0020,   /* U+1F43E */
  0x0020,   /* U+1F43F */
  0x0020,   /* U+1F440 */
  0x0020,   /* U+1F441 */
  0x0020,   /* U+1F442 */
  0x0020,   /* U+1F443 */
  0x0020,   /* U+1F444 */
  0x0020,   /* U+1F445 */
  0x0020,   /* U+1F446 */
  0x0020,   /* U+1F447 */
  0x0020,   /* U+1F448 */
  0x0020,   /* U+1F449 */
  0x0020,   /* U+1F44A */
  0x0020,   /* U+1F44B */
  0x0020,   /* U+1F44C */
  0x0020,   /* U+1F44D */
  0x0020,   /* U+1F44E */
  0x0020,   /* U+1F44F */
  0x0020,   /* U+1F450 */
  0x0020,   /* U+1F451 */
  0x0020,   /* U+1F452 */
  0x0020,   /* U+1F453 */
  0x0020,   /* U+1F454 */
  0x0020,   /* U+1F455 */
  0x0020,   /* U+1F456 */
  0x0020,   /* U+1F457 */
  0x0020,   /* U+1F458 */
  0x0020,   /* U+1F459 */
  0x0020,   /* U+1F45A */
  0x0020,   /* U+1F45B */
  0x0020,   /* U+1F45C */
  0x0020,   /* U+1F45D */
  0x0020,   /* U+1F45E */
  0x0020,   /* U+1F45F */
  0x0020,   /* U+1F460 */
  0x0020,   /* U+1F461 */
  0x0020,   /* U+1F462 */
  0x0020,   /* U+1F463 */
  0x0020,   /* U+1F464 */
  0x0020,   /* U+1F465 */
  0x0020,   /* U+1F466 */
  0x0020,   /* U+1F467 */
  0x0020,   /* U+1F468 */
  0x0020,   /* U+1F469 */
  0x0020,   /* U+1F46A */
  0x0020,   /* U+1F46B */
  0x0020,   /* U+1F46C */
  0x0020,   /* U+1F46D */
  0x0020,   /* U+1F46E */
  0x0020,   /* U+1F46F */
  0x0020,   /* U+1F470 */
  0x0020,   /* U+1F471 */
  0x0020,   /* U+1F472 */
  0x0020,   /* U+1F473 */
  0x0020,   /* U+1F474 */
  0x0020,   /* U+1F475 */
  0x0020,   /* U+1F476 */
  0x0020,   /* U+1F477 */
  0x0020,   /* U+1F478 */
  0x0020,   /* U+1F479 */
  0x0020,   /* U+1F47A */
  0x0020,   /* U+1F47B */
  0x0020,   /* U+1F47C */
  0x0020,   /* U+1F47D */
  0x0020,   /* U+1F47E */
  0x0020,   /* U+1F47F */
  0x0020,   /* U+1F480 */
  0x0020,   /* U+1F481 */
  0x0020,   /* U+1F482 */
  0x0020,   /* U+1F483 */
  0x0020,   /* U+1F484 */
  0x0020,   /* U+1F485 */
  0x0020,   /* U+1F486 */
  0x0020,   /* U+1F487 */
  0x0020,   /* U+1F488 */
  0x0020,   /* U+1F489 */
  0x0020,   /* U+1F48A */
  0x0020,   /* U+1F48B */
  0x0020,   /* U+1F48C */
  0x0020,   /* U+1F48D */
  0x0020,   /* U+1F48E */
  0x0020,   /* U+1F48F */
  0x0020,   /* U+1F490 */
  0x0020,   /* U+1F491 */
  0x0020,   /* U+1F492 */
  0x0020,   /* U+1F493 */
  0x0020,   /* U+1F494 */
  0x0020,   /* U+1F495 */
  0x0020,   /* U+1F496 */
  0x0020,   /* U+1F497 */
  0x0020,   /* U+1F498 */
  0x0020,   /* U+1F499 */
  0x0020,   /* U+1F49A */
  0x0020,   /* U+1F49B */
  0x0020,   /* U+1F49C */
  0x0020,   /* U+1F49D */
  0x0020,   /* U+1F49E */
  0x0020,   /* U+1F49F */
  0x0020,   /* U+1F4A0 */
  0x0020,   /* U+1F4A1 */
  0x0020,   /* U+1F4A2 */
  0x0020,   /* U+1F4A3 */
  0x0020,   /* U+1F4A4 */
  0x0020,   /* U+1F4A5 */
  0x0020,   /* U+1F4A6 */
  0x0020,   /* U+1F4A7 */
  0x0020,   /* U+1F4A8 */
  0x0020,   /* U+1F4A9 */
  0x0020,   /* U+1F4AA */
  0x0020,   /* U+1F4AB */
  0x0020,   /* U+1F4AC */
  0x0020,   /* U+1F4AD */
  0x0020,   /* U+1F4AE */
  0x0020,   /* U+1F4AF */
  0x0020,   /* U+1F4B0 */
  0x0020,   /* U+1F4B1 */
  0x0020,   /* U+1F4B2 */
  0x0020,   /* U+1F4B3 */
  0x0020,   /* U+1F4B4 */
  0x0020,   /* U+1F4B5 */
  0x0020,   /* U+1F4B6 */
  0x0020,   /* U+1F4B7 */
  0x0020,   /* U+1F4B8 */
  0x0020,   /* U+1F4B9 */
  0x0020,   /* U+1F4BA */
  0x0020,   /* U+1F4BB */
  0x0020,   /* U+1F4BC */
  0x0020,   /* U+1F4BD */
  0x0020,   /* U+1F4BE */
  0x0020,   /* U+1F4BF */
  0x0020,   /* U+1F4C0 */
  0x0020,   /* U+1F4C1 */
  0x0020,   /* U+1F4C2 */
  0x0020,   /* U+1F4C3 */
  0x0020,   /* U+1F4C4 */
  0x0020,   /* U+1F4C5 */
  0x0020,   /* U+1F4C6 */
  0x0020,   /* U+1F4C7 */
  0x0020,   /* U+1F4C8 */
  0x0020,   /* U+1F4C9 */
  0x0020,   /* U+1F4CA */
  0x0020,   /* U+1F4CB */
  0x0020,   /* U+1F4CC */
  0x0020,   /* U+1F4CD */
  0x0020,   /* U+1F4CE */
  0x0020,   /* U+1F4CF */
  0x0020,   /* U+1F4D0 */
  0x0020,   /* U+1F4D1 */
  0x0020,   /* U+1F4D2 */
  0x0020,   /* U+1F4D3 */
  0x0020,   /* U+1F4D4 */
  0x0020,   /* U+1F4D5 */
  0x0020,   /* U+1F4D6 */
  0x0020,   /* U+1F4D7 */
  0x0020,   /* U+1F4D8 */
  0x0020,   /* U+1F4D9 */
  0x0020,   /* U+1F4DA */
  0x0020,   /* U+1F4DB */
  0x0020,   /* U+1F4DC */
  0x0020,   /* U+1F4DD */
  0x0020,   /* U+1F4DE */
  0x0020,   /* U+1F4DF */
  0x0020,   /* U+1F4E0 */
  0x0020,   /* U+1F4E1 */
  0x0020,   /* U+1F4E2 */
  0x0020,   /* U+1F4E3 */
  0x0020,   /* U+1F4E4 */
  0x0020,   /* U+1F4E5 */
  0x0020,   /* U+1F4E6 */
  0x0020,   /* U+1F4E7 */
  0x0020,   /* U+1F4E8 */
  0x0020,   /* U+1F4E9 */
  0x0020,   /* U+1F4EA */
  0x0020,   /* U+1F4EB */
  0x0020,   /* U+1F4EC */
  0x0020,   /* U+1F4ED */
  0x0020,   /* U+1F4EE */
  0x0020,   /* U+1F4EF */
  0x0020,   /* U+1F4F0 */
  0x0020,   /* U+1F4F1 */
  0x0020,   /* U+1F4F2 */
  0x0020,   /* U+1F4F3 */
  0x0020,   /* U+1F4F4 */
  0x0020,   /* U+1F4F5 */
  0x0020,   /* U+1F4F6 */
  0x0020,   /* U+1F4F7 */
  0x0020,   /* U+1F4F8 */
  0x0020,   /* U+1F4F9 */
  0x0020,   /* U+1F4FA */
  0x0020,   /* U+1F4FB */
  0x0020,   /* U+1F4FC */
  0x0020,   /* U+1F4FD */
  0x0020,   /* U+1F4FE */
  0x0020,   /* U+1F4FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F400 */
  0x0002,   /* U+1F401 */
  0x0002,   /* U+1F402 */
  0x0002,   /* U+1F403 */
  0x0002,   /* U+1F404 */
  0x0002,   /* U+1F405 */
  0x0002,   /* U+1F406 */
  0x0002,   /* U+1F407 */
  0x0002,   /* U+1F408 */
  0x0002,   /* U+1F409 */
  0x0002,   /* U+1F40A */
  0x0002,   /* U+1F40B */
  0x0002,   /* U+1F40C */
  0x0002,   /* U+1F40D */
  0x0002,   /* U+1F40E */
  0x0002,   /* U+1F40F */
  0x0002,   /* U+1F410 */
  0x0002,   /* U+1F411 */
  0x0002,   /* U+1F412 */
  0x0002,   /* U+1F413 */
  0x0002,   /* U+1F414 */
  0x0002,   /* U+1F415 */
  0x0002,   /* U+1F416 */
  0x0002,   /* U+1F417 */
  0x0002,   /* U+1F418 */
  0x0002,   /* U+1F419 */
  0x0002,   /* U+1F41A */
  0x0002,   /* U+1F41B */
  0x0002,   /* U+1F41C */
  0x0002,   /* U+1F41D */
  0x0002,   /* U+1F41E */
  0x0002,   /* U+1F41F */
  0x0002,   /* U+1F420 */
  0x0002,   /* U+1F421 */
  0x0002,   /* U+1F422 */
  0x0002,   /* U+1F423 */
  0x0002,   /* U+1F424 */
  0x0002,   /* U+1F425 */
  0x0002,   /* U+1F426 */
  0x0002,   /* U+1F427 */
  0x0002,   /* U+1F428 */
  0x0002,   /* U+1F429 */
  0x0002,   /* U+1F42A */
  0x0002,   /* U+1F42B */
  0x0002,   /* U+1F42C */
  0x0002,   /* U+1F42D */
  0x0002,   /* U+1F42E */
  0x0002,   /* U+1F42F */
  0x0002,   /* U+1F430 */
  0x0002,   /* U+1F431 */
  0x0002,   /* U+1F432 */
  0x0002,   /* U+1F433 */
  0x0002,   /* U+1F434 */
  0x0002,   /* U+1F435 */
  0x0002,   /* U+1F436 */
  0x0002,   /* U+1F437 */
  0x0002,   /* U+1F438 */
  0x0002,   /* U+1F439 */
  0x0002,   /* U+1F43A */
  0x0002,   /* U+1F43B */
  0x0002,   /* U+1F43C */
  0x0002,   /* U+1F43D */
  0x0002,   /* U+1F43E */
  0x0002,   /* U+1F43F */
  0x0002,   /* U+1F440 */
  0x0002,   /* U+1F441 */
  0x0002,   /* U+1F442 */
  0x0002,   /* U+1F443 */
  0x0002,   /* U+1F444 */
  0x0002,   /* U+1F445 */
  0x0002,   /* U+1F446 */
  0x0002,   /* U+1F447 */
  0x0002,   /* U+1F448 */
  0x0002,   /* U+1F449 */
  0x0002,   /* U+1F44A */
  0x0002,   /* U+1F44B */
  0x0002,   /* U+1F44C */
  0x0002,   /* U+1F44D */
  0x0002,   /* U+1F44E */
  0x0002,   /* U+1F44F */
  0x0002,   /* U+1F450 */
  0x0002,   /* U+1F451 */
  0x0002,   /* U+1F452 */
  0x0002,   /* U+1F453 */
  0x0002,   /* U+1F454 */
  0x0002,   /* U+1F455 */
  0x0002,   /* U+1F456 */
  0x0002,   /* U+1F457 */
  0x0002,   /* U+1F458 */
  0x0002,   /* U+1F459 */
  0x0002,   /* U+1F45A */
  0x0002,   /* U+1F45B */
  0x0002,   /* U+1F45C */
  0x0002,   /* U+1F45D */
  0x0002,   /* U+1F45E */
  0x0002,   /* U+1F45F */
  0x0002,   /* U+1F460 */
  0x0002,   /* U+1F461 */
  0x0002,   /* U+1F462 */
  0x0002,   /* U+1F463 */
  0x0002,   /* U+1F464 */
  0x0002,   /* U+1F465 */
  0x0002,   /* U+1F466 */
  0x0002,   /* U+1F467 */
  0x0002,   /* U+1F468 */
  0x0002,   /* U+1F469 */
  0x0002,   /* U+1F46A */
  0x0002,   /* U+1F46B */
  0x0002,   /* U+1F46C */
  0x0002,   /* U+1F46D */
  0x0002,   /* U+1F46E */
  0x0002,   /* U+1F46F */
  0x0002,   /* U+1F470 */
  0x0002,   /* U+1F471 */
  0x0002,   /* U+1F472 */
  0x0002,   /* U+1F473 */
  0x0002,   /* U+1F474 */
  0x0002,   /* U+1F475 */
  0x0002,   /* U+1F476 */
  0x0002,   /* U+1F477 */
  0x0002,   /* U+1F478 */
  0x0002,   /* U+1F479 */
  0x0002,   /* U+1F47A */
  0x0002,   /* U+1F47B */
  0x0002,   /* U+1F47C */
  0x0002,   /* U+1F47D */
  0x0002,   /* U+1F47E */
  0x0002,   /* U+1F47F */
  0x0002,   /* U+1F480 */
  0x0002,   /* U+1F481 */
  0x0002,   /* U+1F482 */
  0x0002,   /* U+1F483 */
  0x0002,   /* U+1F484 */
  0x0002,   /* U+1F485 */
  0x0002,   /* U+1F486 */
  0x0002,   /* U+1F487 */
  0x0002,   /* U+1F488 */
  0x0002,   /* U+1F489 */
  0x0002,   /* U+1F48A */
  0x0002,   /* U+1F48B */
  0x0002,   /* U+1F48C */
  0x0002,   /* U+1F48D */
  0x0002,   /* U+1F48E */
  0x0002,   /* U+1F48F */
  0x0002,   /* U+1F490 */
  0x0002,   /* U+1F491 */
  0x0002,   /* U+1F492 */
  0x0002,   /* U+1F493 */
  0x0002,   /* U+1F494 */
  0x0002,   /* U+1F495 */
  0x0002,   /* U+1F496 */
  0x0002,   /* U+1F497 */
  0x0002,   /* U+1F498 */
  0x0002,   /* U+1F499 */
  0x0002,   /* U+1F49A */
  0x0002,   /* U+1F49B */
  0x0002,   /* U+1F49C */
  0x0002,   /* U+1F49D */
  0x0002,   /* U+1F49E */
  0x0002,   /* U+1F49F */
  0x0002,   /* U+1F4A0 */
  0x0002,   /* U+1F4A1 */
  0x0002,   /* U+1F4A2 */
  0x0002,   /* U+1F4A3 */
  0x0002,   /* U+1F4A4 */
  0x0002,   /* U+1F4A5 */
  0x0002,   /* U+1F4A6 */
  0x0002,   /* U+1F4A7 */
  0x0002,   /* U+1F4A8 */
  0x0002,   /* U+1F4A9 */
  0x0002,   /* U+1F4AA */
  0x0002,   /* U+1F4AB */
  0x0002,   /* U+1F4AC */
  0x0002,   /* U+1F4AD */
  0x0002,   /* U+1F4AE */
  0x0002,   /* U+1F4AF */
  0x0002,   /* U+1F4B0 */
  0x0002,   /* U+1F4B1 */
  0x0002,   /* U+1F4B2 */
  0x0002,   /* U+1F4B3 */
  0x0002,   /* U+1F4B4 */
  0x0002,   /* U+1F4B5 */
  0x0002,   /* U+1F4B6 */
  0x0002,   /* U+1F4B7 */
  0x0002,   /* U+1F4B8 */
  0x0002,   /* U+1F4B9 */
  0x0002,   /* U+1F4BA */
  0x0002,   /* U+1F4BB */
  0x0002,   /* U+1F4BC */
  0x0002,   /* U+1F4BD */
  0x0002,   /* U+1F4BE */
  0x0002,   /* U+1F4BF */
  0x0002,   /* U+1F4C0 */
  0x0002,   /* U+1F4C1 */
  0x0002,   /* U+1F4C2 */
  0x0002,   /* U+1F4C3 */
  0x0002,   /* U+1F4C4 */
  0x0002,   /* U+1F4C5 */
  0x0002,   /* U+1F4C6 */
  0x0002,   /* U+1F4C7 */
  0x0002,   /* U+1F4C8 */
  0x0002,   /* U+1F4C9 */
  0x0002,   /* U+1F4CA */
  0x0002,   /* U+1F4CB */
  0x0002,   /* U+1F4CC */
  0x0002,   /* U+1F4CD */
  0x0002,   /* U+1F4CE */
  0x0002,   /* U+1F4CF */
  0x0002,   /* U+1F4D0 */
  0x0002,   /* U+1F4D1 */
  0x0002,   /* U+1F4D2 */
  0x0002,   /* U+1F4D3 */
  0x0002,   /* U+1F4D4 */
  0x0002,   /* U+1F4D5 */
  0x0002,   /* U+1F4D6 */
  0x0002,   /* U+1F4D7 */
  0x0002,   /* U+1F4D8 */
  0x0002,   /* U+1F4D9 */
  0x0002,   /* U+1F4DA */
  0x0002,   /* U+1F4DB */
  0x0002,   /* U+1F4DC */
  0x0002,   /* U+1F4DD */
  0x0002,   /* U+1F4DE */
  0x0002,   /* U+1F4DF */
  0x0002,   /* U+1F4E0 */
  0x0002,   /* U+1F4E1 */
  0x0002,   /* U+1F4E2 */
  0x0002,   /* U+1F4E3 */
  0x0002,   /* U+1F4E4 */
  0x0002,   /* U+1F4E5 */
  0x0002,   /* U+1F4E6 */
  0x0002,   /* U+1F4E7 */
  0x0002,   /* U+1F4E8 */
  0x0002,   /* U+1F4E9 */
  0x0002,   /* U+1F4EA */
  0x0002,   /* U+1F4EB */
  0x0002,   /* U+1F4EC */
  0x0002,   /* U+1F4ED */
  0x0002,   /* U+1F4EE */
  0x0002,   /* U+1F4EF */
  0x0002,   /* U+1F4F0 */
  0x0002,   /* U+1F4F1 */
  0x0002,   /* U+1F4F2 */
  0x0002,   /* U+1F4F3 */
  0x0002,   /* U+1F4F4 */
  0x0002,   /* U+1F4F5 */
  0x0002,   /* U+1F4F6 */
  0x0002,   /* U+1F4F7 */
  0x0002,   /* U+1F4F8 */
  0x0002,   /* U+1F4F9 */
  0x0002,   /* U+1F4FA */
  0x0002,   /* U+1F4FB */
  0x0002,   /* U+1F4FC */
  0x0002,   /* U+1F4FD */
  0x0002,   /* U+1F4FE */
  0x0002,   /* U+1F4FF */
};

uint16 uca900_p1F5[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 

  /* Primary weight 1 for each character. */
  0x14A9,   /* U+1F500 */
  0x14AA,   /* U+1F501 */
  0x14AB,   /* U+1F502 */
  0x14AC,   /* U+1F503 */
  0x14AD,   /* U+1F504 */
  0x14AE,   /* U+1F505 */
  0x14AF,   /* U+1F506 */
  0x14B0,   /* U+1F507 */
  0x14B1,   /* U+1F508 */
  0x14B2,   /* U+1F509 */
  0x14B3,   /* U+1F50A */
  0x14B4,   /* U+1F50B */
  0x14B5,   /* U+1F50C */
  0x14B6,   /* U+1F50D */
  0x14B7,   /* U+1F50E */
  0x14B8,   /* U+1F50F */
  0x14B9,   /* U+1F510 */
  0x14BA,   /* U+1F511 */
  0x14BB,   /* U+1F512 */
  0x14BC,   /* U+1F513 */
  0x14BD,   /* U+1F514 */
  0x14BE,   /* U+1F515 */
  0x14BF,   /* U+1F516 */
  0x14C0,   /* U+1F517 */
  0x14C1,   /* U+1F518 */
  0x14C2,   /* U+1F519 */
  0x14C3,   /* U+1F51A */
  0x14C4,   /* U+1F51B */
  0x14C5,   /* U+1F51C */
  0x14C6,   /* U+1F51D */
  0x14C7,   /* U+1F51E */
  0x14C8,   /* U+1F51F */
  0x14C9,   /* U+1F520 */
  0x14CA,   /* U+1F521 */
  0x14CB,   /* U+1F522 */
  0x14CC,   /* U+1F523 */
  0x14CD,   /* U+1F524 */
  0x14CE,   /* U+1F525 */
  0x14CF,   /* U+1F526 */
  0x14D0,   /* U+1F527 */
  0x14D1,   /* U+1F528 */
  0x14D2,   /* U+1F529 */
  0x14D3,   /* U+1F52A */
  0x14D4,   /* U+1F52B */
  0x14D5,   /* U+1F52C */
  0x14D6,   /* U+1F52D */
  0x14D7,   /* U+1F52E */
  0x14D8,   /* U+1F52F */
  0x14D9,   /* U+1F530 */
  0x14DA,   /* U+1F531 */
  0x14DB,   /* U+1F532 */
  0x14DC,   /* U+1F533 */
  0x14DD,   /* U+1F534 */
  0x14DE,   /* U+1F535 */
  0x14DF,   /* U+1F536 */
  0x14E0,   /* U+1F537 */
  0x14E1,   /* U+1F538 */
  0x14E2,   /* U+1F539 */
  0x14E3,   /* U+1F53A */
  0x14E4,   /* U+1F53B */
  0x14E5,   /* U+1F53C */
  0x14E6,   /* U+1F53D */
  0x14E7,   /* U+1F53E */
  0x14E8,   /* U+1F53F */
  0x14E9,   /* U+1F540 */
  0x14EA,   /* U+1F541 */
  0x14EB,   /* U+1F542 */
  0x14EC,   /* U+1F543 */
  0x14ED,   /* U+1F544 */
  0x14EE,   /* U+1F545 */
  0x14EF,   /* U+1F546 */
  0x14F0,   /* U+1F547 */
  0x14F1,   /* U+1F548 */
  0x14F2,   /* U+1F549 */
  0x14F3,   /* U+1F54A */
  0x14F4,   /* U+1F54B */
  0x14F5,   /* U+1F54C */
  0x14F6,   /* U+1F54D */
  0x14F7,   /* U+1F54E */
  0x14F8,   /* U+1F54F */
  0x14F9,   /* U+1F550 */
  0x14FA,   /* U+1F551 */
  0x14FB,   /* U+1F552 */
  0x14FC,   /* U+1F553 */
  0x14FD,   /* U+1F554 */
  0x14FE,   /* U+1F555 */
  0x14FF,   /* U+1F556 */
  0x1500,   /* U+1F557 */
  0x1501,   /* U+1F558 */
  0x1502,   /* U+1F559 */
  0x1503,   /* U+1F55A */
  0x1504,   /* U+1F55B */
  0x1505,   /* U+1F55C */
  0x1506,   /* U+1F55D */
  0x1507,   /* U+1F55E */
  0x1508,   /* U+1F55F */
  0x1509,   /* U+1F560 */
  0x150A,   /* U+1F561 */
  0x150B,   /* U+1F562 */
  0x150C,   /* U+1F563 */
  0x150D,   /* U+1F564 */
  0x150E,   /* U+1F565 */
  0x150F,   /* U+1F566 */
  0x1510,   /* U+1F567 */
  0x1511,   /* U+1F568 */
  0x1512,   /* U+1F569 */
  0x1513,   /* U+1F56A */
  0x1514,   /* U+1F56B */
  0x1515,   /* U+1F56C */
  0x1516,   /* U+1F56D */
  0x1517,   /* U+1F56E */
  0x1518,   /* U+1F56F */
  0x1519,   /* U+1F570 */
  0x151A,   /* U+1F571 */
  0x151B,   /* U+1F572 */
  0x151C,   /* U+1F573 */
  0x151D,   /* U+1F574 */
  0x151E,   /* U+1F575 */
  0x151F,   /* U+1F576 */
  0x1520,   /* U+1F577 */
  0x1521,   /* U+1F578 */
  0x1522,   /* U+1F579 */
  0x1523,   /* U+1F57A */
  0x1524,   /* U+1F57B */
  0x1525,   /* U+1F57C */
  0x1526,   /* U+1F57D */
  0x1527,   /* U+1F57E */
  0x1528,   /* U+1F57F */
  0x1529,   /* U+1F580 */
  0x152A,   /* U+1F581 */
  0x152B,   /* U+1F582 */
  0x152C,   /* U+1F583 */
  0x152D,   /* U+1F584 */
  0x152E,   /* U+1F585 */
  0x152F,   /* U+1F586 */
  0x1530,   /* U+1F587 */
  0x1531,   /* U+1F588 */
  0x1532,   /* U+1F589 */
  0x1533,   /* U+1F58A */
  0x1534,   /* U+1F58B */
  0x1535,   /* U+1F58C */
  0x1536,   /* U+1F58D */
  0x1537,   /* U+1F58E */
  0x1538,   /* U+1F58F */
  0x1539,   /* U+1F590 */
  0x153A,   /* U+1F591 */
  0x153B,   /* U+1F592 */
  0x153C,   /* U+1F593 */
  0x153D,   /* U+1F594 */
  0x153E,   /* U+1F595 */
  0x153F,   /* U+1F596 */
  0x1540,   /* U+1F597 */
  0x1541,   /* U+1F598 */
  0x1542,   /* U+1F599 */
  0x1543,   /* U+1F59A */
  0x1544,   /* U+1F59B */
  0x1545,   /* U+1F59C */
  0x1546,   /* U+1F59D */
  0x1547,   /* U+1F59E */
  0x1548,   /* U+1F59F */
  0x1549,   /* U+1F5A0 */
  0x154A,   /* U+1F5A1 */
  0x154B,   /* U+1F5A2 */
  0x154C,   /* U+1F5A3 */
  0x154D,   /* U+1F5A4 */
  0x154E,   /* U+1F5A5 */
  0x154F,   /* U+1F5A6 */
  0x1550,   /* U+1F5A7 */
  0x1551,   /* U+1F5A8 */
  0x1552,   /* U+1F5A9 */
  0x1553,   /* U+1F5AA */
  0x1554,   /* U+1F5AB */
  0x1555,   /* U+1F5AC */
  0x1556,   /* U+1F5AD */
  0x1557,   /* U+1F5AE */
  0x1558,   /* U+1F5AF */
  0x1559,   /* U+1F5B0 */
  0x155A,   /* U+1F5B1 */
  0x155B,   /* U+1F5B2 */
  0x155C,   /* U+1F5B3 */
  0x155D,   /* U+1F5B4 */
  0x155E,   /* U+1F5B5 */
  0x155F,   /* U+1F5B6 */
  0x1560,   /* U+1F5B7 */
  0x1561,   /* U+1F5B8 */
  0x1562,   /* U+1F5B9 */
  0x1563,   /* U+1F5BA */
  0x1564,   /* U+1F5BB */
  0x1565,   /* U+1F5BC */
  0x1566,   /* U+1F5BD */
  0x1567,   /* U+1F5BE */
  0x1568,   /* U+1F5BF */
  0x1569,   /* U+1F5C0 */
  0x156A,   /* U+1F5C1 */
  0x156B,   /* U+1F5C2 */
  0x156C,   /* U+1F5C3 */
  0x156D,   /* U+1F5C4 */
  0x156E,   /* U+1F5C5 */
  0x156F,   /* U+1F5C6 */
  0x1570,   /* U+1F5C7 */
  0x1571,   /* U+1F5C8 */
  0x1572,   /* U+1F5C9 */
  0x1573,   /* U+1F5CA */
  0x1574,   /* U+1F5CB */
  0x1575,   /* U+1F5CC */
  0x1576,   /* U+1F5CD */
  0x1577,   /* U+1F5CE */
  0x1578,   /* U+1F5CF */
  0x1579,   /* U+1F5D0 */
  0x157A,   /* U+1F5D1 */
  0x157B,   /* U+1F5D2 */
  0x157C,   /* U+1F5D3 */
  0x157D,   /* U+1F5D4 */
  0x157E,   /* U+1F5D5 */
  0x157F,   /* U+1F5D6 */
  0x1580,   /* U+1F5D7 */
  0x1581,   /* U+1F5D8 */
  0x1582,   /* U+1F5D9 */
  0x1583,   /* U+1F5DA */
  0x1584,   /* U+1F5DB */
  0x1585,   /* U+1F5DC */
  0x1586,   /* U+1F5DD */
  0x1587,   /* U+1F5DE */
  0x1588,   /* U+1F5DF */
  0x1589,   /* U+1F5E0 */
  0x158A,   /* U+1F5E1 */
  0x158B,   /* U+1F5E2 */
  0x158C,   /* U+1F5E3 */
  0x158D,   /* U+1F5E4 */
  0x158E,   /* U+1F5E5 */
  0x158F,   /* U+1F5E6 */
  0x1590,   /* U+1F5E7 */
  0x1591,   /* U+1F5E8 */
  0x1592,   /* U+1F5E9 */
  0x1593,   /* U+1F5EA */
  0x1594,   /* U+1F5EB */
  0x1595,   /* U+1F5EC */
  0x1596,   /* U+1F5ED */
  0x1597,   /* U+1F5EE */
  0x1598,   /* U+1F5EF */
  0x1599,   /* U+1F5F0 */
  0x159A,   /* U+1F5F1 */
  0x159B,   /* U+1F5F2 */
  0x159C,   /* U+1F5F3 */
  0x159D,   /* U+1F5F4 */
  0x159E,   /* U+1F5F5 */
  0x159F,   /* U+1F5F6 */
  0x15A0,   /* U+1F5F7 */
  0x15A1,   /* U+1F5F8 */
  0x15A2,   /* U+1F5F9 */
  0x15A3,   /* U+1F5FA */
  0x15A4,   /* U+1F5FB */
  0x15A5,   /* U+1F5FC */
  0x15A6,   /* U+1F5FD */
  0x15A7,   /* U+1F5FE */
  0x15A8,   /* U+1F5FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F500 */
  0x0020,   /* U+1F501 */
  0x0020,   /* U+1F502 */
  0x0020,   /* U+1F503 */
  0x0020,   /* U+1F504 */
  0x0020,   /* U+1F505 */
  0x0020,   /* U+1F506 */
  0x0020,   /* U+1F507 */
  0x0020,   /* U+1F508 */
  0x0020,   /* U+1F509 */
  0x0020,   /* U+1F50A */
  0x0020,   /* U+1F50B */
  0x0020,   /* U+1F50C */
  0x0020,   /* U+1F50D */
  0x0020,   /* U+1F50E */
  0x0020,   /* U+1F50F */
  0x0020,   /* U+1F510 */
  0x0020,   /* U+1F511 */
  0x0020,   /* U+1F512 */
  0x0020,   /* U+1F513 */
  0x0020,   /* U+1F514 */
  0x0020,   /* U+1F515 */
  0x0020,   /* U+1F516 */
  0x0020,   /* U+1F517 */
  0x0020,   /* U+1F518 */
  0x0020,   /* U+1F519 */
  0x0020,   /* U+1F51A */
  0x0020,   /* U+1F51B */
  0x0020,   /* U+1F51C */
  0x0020,   /* U+1F51D */
  0x0020,   /* U+1F51E */
  0x0020,   /* U+1F51F */
  0x0020,   /* U+1F520 */
  0x0020,   /* U+1F521 */
  0x0020,   /* U+1F522 */
  0x0020,   /* U+1F523 */
  0x0020,   /* U+1F524 */
  0x0020,   /* U+1F525 */
  0x0020,   /* U+1F526 */
  0x0020,   /* U+1F527 */
  0x0020,   /* U+1F528 */
  0x0020,   /* U+1F529 */
  0x0020,   /* U+1F52A */
  0x0020,   /* U+1F52B */
  0x0020,   /* U+1F52C */
  0x0020,   /* U+1F52D */
  0x0020,   /* U+1F52E */
  0x0020,   /* U+1F52F */
  0x0020,   /* U+1F530 */
  0x0020,   /* U+1F531 */
  0x0020,   /* U+1F532 */
  0x0020,   /* U+1F533 */
  0x0020,   /* U+1F534 */
  0x0020,   /* U+1F535 */
  0x0020,   /* U+1F536 */
  0x0020,   /* U+1F537 */
  0x0020,   /* U+1F538 */
  0x0020,   /* U+1F539 */
  0x0020,   /* U+1F53A */
  0x0020,   /* U+1F53B */
  0x0020,   /* U+1F53C */
  0x0020,   /* U+1F53D */
  0x0020,   /* U+1F53E */
  0x0020,   /* U+1F53F */
  0x0020,   /* U+1F540 */
  0x0020,   /* U+1F541 */
  0x0020,   /* U+1F542 */
  0x0020,   /* U+1F543 */
  0x0020,   /* U+1F544 */
  0x0020,   /* U+1F545 */
  0x0020,   /* U+1F546 */
  0x0020,   /* U+1F547 */
  0x0020,   /* U+1F548 */
  0x0020,   /* U+1F549 */
  0x0020,   /* U+1F54A */
  0x0020,   /* U+1F54B */
  0x0020,   /* U+1F54C */
  0x0020,   /* U+1F54D */
  0x0020,   /* U+1F54E */
  0x0020,   /* U+1F54F */
  0x0020,   /* U+1F550 */
  0x0020,   /* U+1F551 */
  0x0020,   /* U+1F552 */
  0x0020,   /* U+1F553 */
  0x0020,   /* U+1F554 */
  0x0020,   /* U+1F555 */
  0x0020,   /* U+1F556 */
  0x0020,   /* U+1F557 */
  0x0020,   /* U+1F558 */
  0x0020,   /* U+1F559 */
  0x0020,   /* U+1F55A */
  0x0020,   /* U+1F55B */
  0x0020,   /* U+1F55C */
  0x0020,   /* U+1F55D */
  0x0020,   /* U+1F55E */
  0x0020,   /* U+1F55F */
  0x0020,   /* U+1F560 */
  0x0020,   /* U+1F561 */
  0x0020,   /* U+1F562 */
  0x0020,   /* U+1F563 */
  0x0020,   /* U+1F564 */
  0x0020,   /* U+1F565 */
  0x0020,   /* U+1F566 */
  0x0020,   /* U+1F567 */
  0x0020,   /* U+1F568 */
  0x0020,   /* U+1F569 */
  0x0020,   /* U+1F56A */
  0x0020,   /* U+1F56B */
  0x0020,   /* U+1F56C */
  0x0020,   /* U+1F56D */
  0x0020,   /* U+1F56E */
  0x0020,   /* U+1F56F */
  0x0020,   /* U+1F570 */
  0x0020,   /* U+1F571 */
  0x0020,   /* U+1F572 */
  0x0020,   /* U+1F573 */
  0x0020,   /* U+1F574 */
  0x0020,   /* U+1F575 */
  0x0020,   /* U+1F576 */
  0x0020,   /* U+1F577 */
  0x0020,   /* U+1F578 */
  0x0020,   /* U+1F579 */
  0x0020,   /* U+1F57A */
  0x0020,   /* U+1F57B */
  0x0020,   /* U+1F57C */
  0x0020,   /* U+1F57D */
  0x0020,   /* U+1F57E */
  0x0020,   /* U+1F57F */
  0x0020,   /* U+1F580 */
  0x0020,   /* U+1F581 */
  0x0020,   /* U+1F582 */
  0x0020,   /* U+1F583 */
  0x0020,   /* U+1F584 */
  0x0020,   /* U+1F585 */
  0x0020,   /* U+1F586 */
  0x0020,   /* U+1F587 */
  0x0020,   /* U+1F588 */
  0x0020,   /* U+1F589 */
  0x0020,   /* U+1F58A */
  0x0020,   /* U+1F58B */
  0x0020,   /* U+1F58C */
  0x0020,   /* U+1F58D */
  0x0020,   /* U+1F58E */
  0x0020,   /* U+1F58F */
  0x0020,   /* U+1F590 */
  0x0020,   /* U+1F591 */
  0x0020,   /* U+1F592 */
  0x0020,   /* U+1F593 */
  0x0020,   /* U+1F594 */
  0x0020,   /* U+1F595 */
  0x0020,   /* U+1F596 */
  0x0020,   /* U+1F597 */
  0x0020,   /* U+1F598 */
  0x0020,   /* U+1F599 */
  0x0020,   /* U+1F59A */
  0x0020,   /* U+1F59B */
  0x0020,   /* U+1F59C */
  0x0020,   /* U+1F59D */
  0x0020,   /* U+1F59E */
  0x0020,   /* U+1F59F */
  0x0020,   /* U+1F5A0 */
  0x0020,   /* U+1F5A1 */
  0x0020,   /* U+1F5A2 */
  0x0020,   /* U+1F5A3 */
  0x0020,   /* U+1F5A4 */
  0x0020,   /* U+1F5A5 */
  0x0020,   /* U+1F5A6 */
  0x0020,   /* U+1F5A7 */
  0x0020,   /* U+1F5A8 */
  0x0020,   /* U+1F5A9 */
  0x0020,   /* U+1F5AA */
  0x0020,   /* U+1F5AB */
  0x0020,   /* U+1F5AC */
  0x0020,   /* U+1F5AD */
  0x0020,   /* U+1F5AE */
  0x0020,   /* U+1F5AF */
  0x0020,   /* U+1F5B0 */
  0x0020,   /* U+1F5B1 */
  0x0020,   /* U+1F5B2 */
  0x0020,   /* U+1F5B3 */
  0x0020,   /* U+1F5B4 */
  0x0020,   /* U+1F5B5 */
  0x0020,   /* U+1F5B6 */
  0x0020,   /* U+1F5B7 */
  0x0020,   /* U+1F5B8 */
  0x0020,   /* U+1F5B9 */
  0x0020,   /* U+1F5BA */
  0x0020,   /* U+1F5BB */
  0x0020,   /* U+1F5BC */
  0x0020,   /* U+1F5BD */
  0x0020,   /* U+1F5BE */
  0x0020,   /* U+1F5BF */
  0x0020,   /* U+1F5C0 */
  0x0020,   /* U+1F5C1 */
  0x0020,   /* U+1F5C2 */
  0x0020,   /* U+1F5C3 */
  0x0020,   /* U+1F5C4 */
  0x0020,   /* U+1F5C5 */
  0x0020,   /* U+1F5C6 */
  0x0020,   /* U+1F5C7 */
  0x0020,   /* U+1F5C8 */
  0x0020,   /* U+1F5C9 */
  0x0020,   /* U+1F5CA */
  0x0020,   /* U+1F5CB */
  0x0020,   /* U+1F5CC */
  0x0020,   /* U+1F5CD */
  0x0020,   /* U+1F5CE */
  0x0020,   /* U+1F5CF */
  0x0020,   /* U+1F5D0 */
  0x0020,   /* U+1F5D1 */
  0x0020,   /* U+1F5D2 */
  0x0020,   /* U+1F5D3 */
  0x0020,   /* U+1F5D4 */
  0x0020,   /* U+1F5D5 */
  0x0020,   /* U+1F5D6 */
  0x0020,   /* U+1F5D7 */
  0x0020,   /* U+1F5D8 */
  0x0020,   /* U+1F5D9 */
  0x0020,   /* U+1F5DA */
  0x0020,   /* U+1F5DB */
  0x0020,   /* U+1F5DC */
  0x0020,   /* U+1F5DD */
  0x0020,   /* U+1F5DE */
  0x0020,   /* U+1F5DF */
  0x0020,   /* U+1F5E0 */
  0x0020,   /* U+1F5E1 */
  0x0020,   /* U+1F5E2 */
  0x0020,   /* U+1F5E3 */
  0x0020,   /* U+1F5E4 */
  0x0020,   /* U+1F5E5 */
  0x0020,   /* U+1F5E6 */
  0x0020,   /* U+1F5E7 */
  0x0020,   /* U+1F5E8 */
  0x0020,   /* U+1F5E9 */
  0x0020,   /* U+1F5EA */
  0x0020,   /* U+1F5EB */
  0x0020,   /* U+1F5EC */
  0x0020,   /* U+1F5ED */
  0x0020,   /* U+1F5EE */
  0x0020,   /* U+1F5EF */
  0x0020,   /* U+1F5F0 */
  0x0020,   /* U+1F5F1 */
  0x0020,   /* U+1F5F2 */
  0x0020,   /* U+1F5F3 */
  0x0020,   /* U+1F5F4 */
  0x0020,   /* U+1F5F5 */
  0x0020,   /* U+1F5F6 */
  0x0020,   /* U+1F5F7 */
  0x0020,   /* U+1F5F8 */
  0x0020,   /* U+1F5F9 */
  0x0020,   /* U+1F5FA */
  0x0020,   /* U+1F5FB */
  0x0020,   /* U+1F5FC */
  0x0020,   /* U+1F5FD */
  0x0020,   /* U+1F5FE */
  0x0020,   /* U+1F5FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F500 */
  0x0002,   /* U+1F501 */
  0x0002,   /* U+1F502 */
  0x0002,   /* U+1F503 */
  0x0002,   /* U+1F504 */
  0x0002,   /* U+1F505 */
  0x0002,   /* U+1F506 */
  0x0002,   /* U+1F507 */
  0x0002,   /* U+1F508 */
  0x0002,   /* U+1F509 */
  0x0002,   /* U+1F50A */
  0x0002,   /* U+1F50B */
  0x0002,   /* U+1F50C */
  0x0002,   /* U+1F50D */
  0x0002,   /* U+1F50E */
  0x0002,   /* U+1F50F */
  0x0002,   /* U+1F510 */
  0x0002,   /* U+1F511 */
  0x0002,   /* U+1F512 */
  0x0002,   /* U+1F513 */
  0x0002,   /* U+1F514 */
  0x0002,   /* U+1F515 */
  0x0002,   /* U+1F516 */
  0x0002,   /* U+1F517 */
  0x0002,   /* U+1F518 */
  0x0002,   /* U+1F519 */
  0x0002,   /* U+1F51A */
  0x0002,   /* U+1F51B */
  0x0002,   /* U+1F51C */
  0x0002,   /* U+1F51D */
  0x0002,   /* U+1F51E */
  0x0002,   /* U+1F51F */
  0x0002,   /* U+1F520 */
  0x0002,   /* U+1F521 */
  0x0002,   /* U+1F522 */
  0x0002,   /* U+1F523 */
  0x0002,   /* U+1F524 */
  0x0002,   /* U+1F525 */
  0x0002,   /* U+1F526 */
  0x0002,   /* U+1F527 */
  0x0002,   /* U+1F528 */
  0x0002,   /* U+1F529 */
  0x0002,   /* U+1F52A */
  0x0002,   /* U+1F52B */
  0x0002,   /* U+1F52C */
  0x0002,   /* U+1F52D */
  0x0002,   /* U+1F52E */
  0x0002,   /* U+1F52F */
  0x0002,   /* U+1F530 */
  0x0002,   /* U+1F531 */
  0x0002,   /* U+1F532 */
  0x0002,   /* U+1F533 */
  0x0002,   /* U+1F534 */
  0x0002,   /* U+1F535 */
  0x0002,   /* U+1F536 */
  0x0002,   /* U+1F537 */
  0x0002,   /* U+1F538 */
  0x0002,   /* U+1F539 */
  0x0002,   /* U+1F53A */
  0x0002,   /* U+1F53B */
  0x0002,   /* U+1F53C */
  0x0002,   /* U+1F53D */
  0x0002,   /* U+1F53E */
  0x0002,   /* U+1F53F */
  0x0002,   /* U+1F540 */
  0x0002,   /* U+1F541 */
  0x0002,   /* U+1F542 */
  0x0002,   /* U+1F543 */
  0x0002,   /* U+1F544 */
  0x0002,   /* U+1F545 */
  0x0002,   /* U+1F546 */
  0x0002,   /* U+1F547 */
  0x0002,   /* U+1F548 */
  0x0002,   /* U+1F549 */
  0x0002,   /* U+1F54A */
  0x0002,   /* U+1F54B */
  0x0002,   /* U+1F54C */
  0x0002,   /* U+1F54D */
  0x0002,   /* U+1F54E */
  0x0002,   /* U+1F54F */
  0x0002,   /* U+1F550 */
  0x0002,   /* U+1F551 */
  0x0002,   /* U+1F552 */
  0x0002,   /* U+1F553 */
  0x0002,   /* U+1F554 */
  0x0002,   /* U+1F555 */
  0x0002,   /* U+1F556 */
  0x0002,   /* U+1F557 */
  0x0002,   /* U+1F558 */
  0x0002,   /* U+1F559 */
  0x0002,   /* U+1F55A */
  0x0002,   /* U+1F55B */
  0x0002,   /* U+1F55C */
  0x0002,   /* U+1F55D */
  0x0002,   /* U+1F55E */
  0x0002,   /* U+1F55F */
  0x0002,   /* U+1F560 */
  0x0002,   /* U+1F561 */
  0x0002,   /* U+1F562 */
  0x0002,   /* U+1F563 */
  0x0002,   /* U+1F564 */
  0x0002,   /* U+1F565 */
  0x0002,   /* U+1F566 */
  0x0002,   /* U+1F567 */
  0x0002,   /* U+1F568 */
  0x0002,   /* U+1F569 */
  0x0002,   /* U+1F56A */
  0x0002,   /* U+1F56B */
  0x0002,   /* U+1F56C */
  0x0002,   /* U+1F56D */
  0x0002,   /* U+1F56E */
  0x0002,   /* U+1F56F */
  0x0002,   /* U+1F570 */
  0x0002,   /* U+1F571 */
  0x0002,   /* U+1F572 */
  0x0002,   /* U+1F573 */
  0x0002,   /* U+1F574 */
  0x0002,   /* U+1F575 */
  0x0002,   /* U+1F576 */
  0x0002,   /* U+1F577 */
  0x0002,   /* U+1F578 */
  0x0002,   /* U+1F579 */
  0x0002,   /* U+1F57A */
  0x0002,   /* U+1F57B */
  0x0002,   /* U+1F57C */
  0x0002,   /* U+1F57D */
  0x0002,   /* U+1F57E */
  0x0002,   /* U+1F57F */
  0x0002,   /* U+1F580 */
  0x0002,   /* U+1F581 */
  0x0002,   /* U+1F582 */
  0x0002,   /* U+1F583 */
  0x0002,   /* U+1F584 */
  0x0002,   /* U+1F585 */
  0x0002,   /* U+1F586 */
  0x0002,   /* U+1F587 */
  0x0002,   /* U+1F588 */
  0x0002,   /* U+1F589 */
  0x0002,   /* U+1F58A */
  0x0002,   /* U+1F58B */
  0x0002,   /* U+1F58C */
  0x0002,   /* U+1F58D */
  0x0002,   /* U+1F58E */
  0x0002,   /* U+1F58F */
  0x0002,   /* U+1F590 */
  0x0002,   /* U+1F591 */
  0x0002,   /* U+1F592 */
  0x0002,   /* U+1F593 */
  0x0002,   /* U+1F594 */
  0x0002,   /* U+1F595 */
  0x0002,   /* U+1F596 */
  0x0002,   /* U+1F597 */
  0x0002,   /* U+1F598 */
  0x0002,   /* U+1F599 */
  0x0002,   /* U+1F59A */
  0x0002,   /* U+1F59B */
  0x0002,   /* U+1F59C */
  0x0002,   /* U+1F59D */
  0x0002,   /* U+1F59E */
  0x0002,   /* U+1F59F */
  0x0002,   /* U+1F5A0 */
  0x0002,   /* U+1F5A1 */
  0x0002,   /* U+1F5A2 */
  0x0002,   /* U+1F5A3 */
  0x0002,   /* U+1F5A4 */
  0x0002,   /* U+1F5A5 */
  0x0002,   /* U+1F5A6 */
  0x0002,   /* U+1F5A7 */
  0x0002,   /* U+1F5A8 */
  0x0002,   /* U+1F5A9 */
  0x0002,   /* U+1F5AA */
  0x0002,   /* U+1F5AB */
  0x0002,   /* U+1F5AC */
  0x0002,   /* U+1F5AD */
  0x0002,   /* U+1F5AE */
  0x0002,   /* U+1F5AF */
  0x0002,   /* U+1F5B0 */
  0x0002,   /* U+1F5B1 */
  0x0002,   /* U+1F5B2 */
  0x0002,   /* U+1F5B3 */
  0x0002,   /* U+1F5B4 */
  0x0002,   /* U+1F5B5 */
  0x0002,   /* U+1F5B6 */
  0x0002,   /* U+1F5B7 */
  0x0002,   /* U+1F5B8 */
  0x0002,   /* U+1F5B9 */
  0x0002,   /* U+1F5BA */
  0x0002,   /* U+1F5BB */
  0x0002,   /* U+1F5BC */
  0x0002,   /* U+1F5BD */
  0x0002,   /* U+1F5BE */
  0x0002,   /* U+1F5BF */
  0x0002,   /* U+1F5C0 */
  0x0002,   /* U+1F5C1 */
  0x0002,   /* U+1F5C2 */
  0x0002,   /* U+1F5C3 */
  0x0002,   /* U+1F5C4 */
  0x0002,   /* U+1F5C5 */
  0x0002,   /* U+1F5C6 */
  0x0002,   /* U+1F5C7 */
  0x0002,   /* U+1F5C8 */
  0x0002,   /* U+1F5C9 */
  0x0002,   /* U+1F5CA */
  0x0002,   /* U+1F5CB */
  0x0002,   /* U+1F5CC */
  0x0002,   /* U+1F5CD */
  0x0002,   /* U+1F5CE */
  0x0002,   /* U+1F5CF */
  0x0002,   /* U+1F5D0 */
  0x0002,   /* U+1F5D1 */
  0x0002,   /* U+1F5D2 */
  0x0002,   /* U+1F5D3 */
  0x0002,   /* U+1F5D4 */
  0x0002,   /* U+1F5D5 */
  0x0002,   /* U+1F5D6 */
  0x0002,   /* U+1F5D7 */
  0x0002,   /* U+1F5D8 */
  0x0002,   /* U+1F5D9 */
  0x0002,   /* U+1F5DA */
  0x0002,   /* U+1F5DB */
  0x0002,   /* U+1F5DC */
  0x0002,   /* U+1F5DD */
  0x0002,   /* U+1F5DE */
  0x0002,   /* U+1F5DF */
  0x0002,   /* U+1F5E0 */
  0x0002,   /* U+1F5E1 */
  0x0002,   /* U+1F5E2 */
  0x0002,   /* U+1F5E3 */
  0x0002,   /* U+1F5E4 */
  0x0002,   /* U+1F5E5 */
  0x0002,   /* U+1F5E6 */
  0x0002,   /* U+1F5E7 */
  0x0002,   /* U+1F5E8 */
  0x0002,   /* U+1F5E9 */
  0x0002,   /* U+1F5EA */
  0x0002,   /* U+1F5EB */
  0x0002,   /* U+1F5EC */
  0x0002,   /* U+1F5ED */
  0x0002,   /* U+1F5EE */
  0x0002,   /* U+1F5EF */
  0x0002,   /* U+1F5F0 */
  0x0002,   /* U+1F5F1 */
  0x0002,   /* U+1F5F2 */
  0x0002,   /* U+1F5F3 */
  0x0002,   /* U+1F5F4 */
  0x0002,   /* U+1F5F5 */
  0x0002,   /* U+1F5F6 */
  0x0002,   /* U+1F5F7 */
  0x0002,   /* U+1F5F8 */
  0x0002,   /* U+1F5F9 */
  0x0002,   /* U+1F5FA */
  0x0002,   /* U+1F5FB */
  0x0002,   /* U+1F5FC */
  0x0002,   /* U+1F5FD */
  0x0002,   /* U+1F5FE */
  0x0002,   /* U+1F5FF */
};

uint16 uca900_p1F6[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x15FB,   /* U+1F600 */
  0x15FC,   /* U+1F601 */
  0x15FD,   /* U+1F602 */
  0x15FE,   /* U+1F603 */
  0x15FF,   /* U+1F604 */
  0x1600,   /* U+1F605 */
  0x1601,   /* U+1F606 */
  0x1602,   /* U+1F607 */
  0x1603,   /* U+1F608 */
  0x1604,   /* U+1F609 */
  0x1605,   /* U+1F60A */
  0x1606,   /* U+1F60B */
  0x1607,   /* U+1F60C */
  0x1608,   /* U+1F60D */
  0x1609,   /* U+1F60E */
  0x160A,   /* U+1F60F */
  0x160B,   /* U+1F610 */
  0x160C,   /* U+1F611 */
  0x160D,   /* U+1F612 */
  0x160E,   /* U+1F613 */
  0x160F,   /* U+1F614 */
  0x1610,   /* U+1F615 */
  0x1611,   /* U+1F616 */
  0x1612,   /* U+1F617 */
  0x1613,   /* U+1F618 */
  0x1614,   /* U+1F619 */
  0x1615,   /* U+1F61A */
  0x1616,   /* U+1F61B */
  0x1617,   /* U+1F61C */
  0x1618,   /* U+1F61D */
  0x1619,   /* U+1F61E */
  0x161A,   /* U+1F61F */
  0x161B,   /* U+1F620 */
  0x161C,   /* U+1F621 */
  0x161D,   /* U+1F622 */
  0x161E,   /* U+1F623 */
  0x161F,   /* U+1F624 */
  0x1620,   /* U+1F625 */
  0x1621,   /* U+1F626 */
  0x1622,   /* U+1F627 */
  0x1623,   /* U+1F628 */
  0x1624,   /* U+1F629 */
  0x1625,   /* U+1F62A */
  0x1626,   /* U+1F62B */
  0x1627,   /* U+1F62C */
  0x1628,   /* U+1F62D */
  0x1629,   /* U+1F62E */
  0x162A,   /* U+1F62F */
  0x162B,   /* U+1F630 */
  0x162C,   /* U+1F631 */
  0x162D,   /* U+1F632 */
  0x162E,   /* U+1F633 */
  0x162F,   /* U+1F634 */
  0x1630,   /* U+1F635 */
  0x1631,   /* U+1F636 */
  0x1632,   /* U+1F637 */
  0x1633,   /* U+1F638 */
  0x1634,   /* U+1F639 */
  0x1635,   /* U+1F63A */
  0x1636,   /* U+1F63B */
  0x1637,   /* U+1F63C */
  0x1638,   /* U+1F63D */
  0x1639,   /* U+1F63E */
  0x163A,   /* U+1F63F */
  0x163B,   /* U+1F640 */
  0x163C,   /* U+1F641 */
  0x163D,   /* U+1F642 */
  0x163E,   /* U+1F643 */
  0x163F,   /* U+1F644 */
  0x1640,   /* U+1F645 */
  0x1641,   /* U+1F646 */
  0x1642,   /* U+1F647 */
  0x1643,   /* U+1F648 */
  0x1644,   /* U+1F649 */
  0x1645,   /* U+1F64A */
  0x1646,   /* U+1F64B */
  0x1647,   /* U+1F64C */
  0x1648,   /* U+1F64D */
  0x1649,   /* U+1F64E */
  0x164A,   /* U+1F64F */
  0x164B,   /* U+1F650 */
  0x164C,   /* U+1F651 */
  0x164D,   /* U+1F652 */
  0x164E,   /* U+1F653 */
  0x164F,   /* U+1F654 */
  0x1650,   /* U+1F655 */
  0x1651,   /* U+1F656 */
  0x1652,   /* U+1F657 */
  0x1653,   /* U+1F658 */
  0x1654,   /* U+1F659 */
  0x1655,   /* U+1F65A */
  0x1656,   /* U+1F65B */
  0x1657,   /* U+1F65C */
  0x1658,   /* U+1F65D */
  0x1659,   /* U+1F65E */
  0x165A,   /* U+1F65F */
  0x165B,   /* U+1F660 */
  0x165C,   /* U+1F661 */
  0x165D,   /* U+1F662 */
  0x165E,   /* U+1F663 */
  0x165F,   /* U+1F664 */
  0x1660,   /* U+1F665 */
  0x1661,   /* U+1F666 */
  0x1662,   /* U+1F667 */
  0x1663,   /* U+1F668 */
  0x1664,   /* U+1F669 */
  0x1665,   /* U+1F66A */
  0x1666,   /* U+1F66B */
  0x1667,   /* U+1F66C */
  0x1668,   /* U+1F66D */
  0x1669,   /* U+1F66E */
  0x166A,   /* U+1F66F */
  0x166B,   /* U+1F670 */
  0x166C,   /* U+1F671 */
  0x166D,   /* U+1F672 */
  0x166E,   /* U+1F673 */
  0x166F,   /* U+1F674 */
  0x1670,   /* U+1F675 */
  0x1671,   /* U+1F676 */
  0x1672,   /* U+1F677 */
  0x1673,   /* U+1F678 */
  0x1674,   /* U+1F679 */
  0x1675,   /* U+1F67A */
  0x1676,   /* U+1F67B */
  0x1677,   /* U+1F67C */
  0x1678,   /* U+1F67D */
  0x1679,   /* U+1F67E */
  0x167A,   /* U+1F67F */
  0x167B,   /* U+1F680 */
  0x167C,   /* U+1F681 */
  0x167D,   /* U+1F682 */
  0x167E,   /* U+1F683 */
  0x167F,   /* U+1F684 */
  0x1680,   /* U+1F685 */
  0x1681,   /* U+1F686 */
  0x1682,   /* U+1F687 */
  0x1683,   /* U+1F688 */
  0x1684,   /* U+1F689 */
  0x1685,   /* U+1F68A */
  0x1686,   /* U+1F68B */
  0x1687,   /* U+1F68C */
  0x1688,   /* U+1F68D */
  0x1689,   /* U+1F68E */
  0x168A,   /* U+1F68F */
  0x168B,   /* U+1F690 */
  0x168C,   /* U+1F691 */
  0x168D,   /* U+1F692 */
  0x168E,   /* U+1F693 */
  0x168F,   /* U+1F694 */
  0x1690,   /* U+1F695 */
  0x1691,   /* U+1F696 */
  0x1692,   /* U+1F697 */
  0x1693,   /* U+1F698 */
  0x1694,   /* U+1F699 */
  0x1695,   /* U+1F69A */
  0x1696,   /* U+1F69B */
  0x1697,   /* U+1F69C */
  0x1698,   /* U+1F69D */
  0x1699,   /* U+1F69E */
  0x169A,   /* U+1F69F */
  0x169B,   /* U+1F6A0 */
  0x169C,   /* U+1F6A1 */
  0x169D,   /* U+1F6A2 */
  0x169E,   /* U+1F6A3 */
  0x169F,   /* U+1F6A4 */
  0x16A0,   /* U+1F6A5 */
  0x16A1,   /* U+1F6A6 */
  0x16A2,   /* U+1F6A7 */
  0x16A3,   /* U+1F6A8 */
  0x16A4,   /* U+1F6A9 */
  0x16A5,   /* U+1F6AA */
  0x16A6,   /* U+1F6AB */
  0x16A7,   /* U+1F6AC */
  0x16A8,   /* U+1F6AD */
  0x16A9,   /* U+1F6AE */
  0x16AA,   /* U+1F6AF */
  0x16AB,   /* U+1F6B0 */
  0x16AC,   /* U+1F6B1 */
  0x16AD,   /* U+1F6B2 */
  0x16AE,   /* U+1F6B3 */
  0x16AF,   /* U+1F6B4 */
  0x16B0,   /* U+1F6B5 */
  0x16B1,   /* U+1F6B6 */
  0x16B2,   /* U+1F6B7 */
  0x16B3,   /* U+1F6B8 */
  0x16B4,   /* U+1F6B9 */
  0x16B5,   /* U+1F6BA */
  0x16B6,   /* U+1F6BB */
  0x16B7,   /* U+1F6BC */
  0x16B8,   /* U+1F6BD */
  0x16B9,   /* U+1F6BE */
  0x16BA,   /* U+1F6BF */
  0x16BB,   /* U+1F6C0 */
  0x16BC,   /* U+1F6C1 */
  0x16BD,   /* U+1F6C2 */
  0x16BE,   /* U+1F6C3 */
  0x16BF,   /* U+1F6C4 */
  0x16C0,   /* U+1F6C5 */
  0x16C1,   /* U+1F6C6 */
  0x16C2,   /* U+1F6C7 */
  0x16C3,   /* U+1F6C8 */
  0x16C4,   /* U+1F6C9 */
  0x16C5,   /* U+1F6CA */
  0x16C6,   /* U+1F6CB */
  0x16C7,   /* U+1F6CC */
  0x16C8,   /* U+1F6CD */
  0x16C9,   /* U+1F6CE */
  0x16CA,   /* U+1F6CF */
  0x16CB,   /* U+1F6D0 */
  0x16CC,   /* U+1F6D1 */
  0x16CD,   /* U+1F6D2 */
  0xFBC3,   /* U+1F6D3 */
  0xFBC3,   /* U+1F6D4 */
  0xFBC3,   /* U+1F6D5 */
  0xFBC3,   /* U+1F6D6 */
  0xFBC3,   /* U+1F6D7 */
  0xFBC3,   /* U+1F6D8 */
  0xFBC3,   /* U+1F6D9 */
  0xFBC3,   /* U+1F6DA */
  0xFBC3,   /* U+1F6DB */
  0xFBC3,   /* U+1F6DC */
  0xFBC3,   /* U+1F6DD */
  0xFBC3,   /* U+1F6DE */
  0xFBC3,   /* U+1F6DF */
  0x16CE,   /* U+1F6E0 */
  0x16CF,   /* U+1F6E1 */
  0x16D0,   /* U+1F6E2 */
  0x16D1,   /* U+1F6E3 */
  0x16D2,   /* U+1F6E4 */
  0x16D3,   /* U+1F6E5 */
  0x16D4,   /* U+1F6E6 */
  0x16D5,   /* U+1F6E7 */
  0x16D6,   /* U+1F6E8 */
  0x16D7,   /* U+1F6E9 */
  0x16D8,   /* U+1F6EA */
  0x16D9,   /* U+1F6EB */
  0x16DA,   /* U+1F6EC */
  0xFBC3,   /* U+1F6ED */
  0xFBC3,   /* U+1F6EE */
  0xFBC3,   /* U+1F6EF */
  0x16DB,   /* U+1F6F0 */
  0x16DC,   /* U+1F6F1 */
  0x16DD,   /* U+1F6F2 */
  0x16DE,   /* U+1F6F3 */
  0x16DF,   /* U+1F6F4 */
  0x16E0,   /* U+1F6F5 */
  0x16E1,   /* U+1F6F6 */
  0xFBC3,   /* U+1F6F7 */
  0xFBC3,   /* U+1F6F8 */
  0xFBC3,   /* U+1F6F9 */
  0xFBC3,   /* U+1F6FA */
  0xFBC3,   /* U+1F6FB */
  0xFBC3,   /* U+1F6FC */
  0xFBC3,   /* U+1F6FD */
  0xFBC3,   /* U+1F6FE */
  0xFBC3,   /* U+1F6FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F600 */
  0x0020,   /* U+1F601 */
  0x0020,   /* U+1F602 */
  0x0020,   /* U+1F603 */
  0x0020,   /* U+1F604 */
  0x0020,   /* U+1F605 */
  0x0020,   /* U+1F606 */
  0x0020,   /* U+1F607 */
  0x0020,   /* U+1F608 */
  0x0020,   /* U+1F609 */
  0x0020,   /* U+1F60A */
  0x0020,   /* U+1F60B */
  0x0020,   /* U+1F60C */
  0x0020,   /* U+1F60D */
  0x0020,   /* U+1F60E */
  0x0020,   /* U+1F60F */
  0x0020,   /* U+1F610 */
  0x0020,   /* U+1F611 */
  0x0020,   /* U+1F612 */
  0x0020,   /* U+1F613 */
  0x0020,   /* U+1F614 */
  0x0020,   /* U+1F615 */
  0x0020,   /* U+1F616 */
  0x0020,   /* U+1F617 */
  0x0020,   /* U+1F618 */
  0x0020,   /* U+1F619 */
  0x0020,   /* U+1F61A */
  0x0020,   /* U+1F61B */
  0x0020,   /* U+1F61C */
  0x0020,   /* U+1F61D */
  0x0020,   /* U+1F61E */
  0x0020,   /* U+1F61F */
  0x0020,   /* U+1F620 */
  0x0020,   /* U+1F621 */
  0x0020,   /* U+1F622 */
  0x0020,   /* U+1F623 */
  0x0020,   /* U+1F624 */
  0x0020,   /* U+1F625 */
  0x0020,   /* U+1F626 */
  0x0020,   /* U+1F627 */
  0x0020,   /* U+1F628 */
  0x0020,   /* U+1F629 */
  0x0020,   /* U+1F62A */
  0x0020,   /* U+1F62B */
  0x0020,   /* U+1F62C */
  0x0020,   /* U+1F62D */
  0x0020,   /* U+1F62E */
  0x0020,   /* U+1F62F */
  0x0020,   /* U+1F630 */
  0x0020,   /* U+1F631 */
  0x0020,   /* U+1F632 */
  0x0020,   /* U+1F633 */
  0x0020,   /* U+1F634 */
  0x0020,   /* U+1F635 */
  0x0020,   /* U+1F636 */
  0x0020,   /* U+1F637 */
  0x0020,   /* U+1F638 */
  0x0020,   /* U+1F639 */
  0x0020,   /* U+1F63A */
  0x0020,   /* U+1F63B */
  0x0020,   /* U+1F63C */
  0x0020,   /* U+1F63D */
  0x0020,   /* U+1F63E */
  0x0020,   /* U+1F63F */
  0x0020,   /* U+1F640 */
  0x0020,   /* U+1F641 */
  0x0020,   /* U+1F642 */
  0x0020,   /* U+1F643 */
  0x0020,   /* U+1F644 */
  0x0020,   /* U+1F645 */
  0x0020,   /* U+1F646 */
  0x0020,   /* U+1F647 */
  0x0020,   /* U+1F648 */
  0x0020,   /* U+1F649 */
  0x0020,   /* U+1F64A */
  0x0020,   /* U+1F64B */
  0x0020,   /* U+1F64C */
  0x0020,   /* U+1F64D */
  0x0020,   /* U+1F64E */
  0x0020,   /* U+1F64F */
  0x0020,   /* U+1F650 */
  0x0020,   /* U+1F651 */
  0x0020,   /* U+1F652 */
  0x0020,   /* U+1F653 */
  0x0020,   /* U+1F654 */
  0x0020,   /* U+1F655 */
  0x0020,   /* U+1F656 */
  0x0020,   /* U+1F657 */
  0x0020,   /* U+1F658 */
  0x0020,   /* U+1F659 */
  0x0020,   /* U+1F65A */
  0x0020,   /* U+1F65B */
  0x0020,   /* U+1F65C */
  0x0020,   /* U+1F65D */
  0x0020,   /* U+1F65E */
  0x0020,   /* U+1F65F */
  0x0020,   /* U+1F660 */
  0x0020,   /* U+1F661 */
  0x0020,   /* U+1F662 */
  0x0020,   /* U+1F663 */
  0x0020,   /* U+1F664 */
  0x0020,   /* U+1F665 */
  0x0020,   /* U+1F666 */
  0x0020,   /* U+1F667 */
  0x0020,   /* U+1F668 */
  0x0020,   /* U+1F669 */
  0x0020,   /* U+1F66A */
  0x0020,   /* U+1F66B */
  0x0020,   /* U+1F66C */
  0x0020,   /* U+1F66D */
  0x0020,   /* U+1F66E */
  0x0020,   /* U+1F66F */
  0x0020,   /* U+1F670 */
  0x0020,   /* U+1F671 */
  0x0020,   /* U+1F672 */
  0x0020,   /* U+1F673 */
  0x0020,   /* U+1F674 */
  0x0020,   /* U+1F675 */
  0x0020,   /* U+1F676 */
  0x0020,   /* U+1F677 */
  0x0020,   /* U+1F678 */
  0x0020,   /* U+1F679 */
  0x0020,   /* U+1F67A */
  0x0020,   /* U+1F67B */
  0x0020,   /* U+1F67C */
  0x0020,   /* U+1F67D */
  0x0020,   /* U+1F67E */
  0x0020,   /* U+1F67F */
  0x0020,   /* U+1F680 */
  0x0020,   /* U+1F681 */
  0x0020,   /* U+1F682 */
  0x0020,   /* U+1F683 */
  0x0020,   /* U+1F684 */
  0x0020,   /* U+1F685 */
  0x0020,   /* U+1F686 */
  0x0020,   /* U+1F687 */
  0x0020,   /* U+1F688 */
  0x0020,   /* U+1F689 */
  0x0020,   /* U+1F68A */
  0x0020,   /* U+1F68B */
  0x0020,   /* U+1F68C */
  0x0020,   /* U+1F68D */
  0x0020,   /* U+1F68E */
  0x0020,   /* U+1F68F */
  0x0020,   /* U+1F690 */
  0x0020,   /* U+1F691 */
  0x0020,   /* U+1F692 */
  0x0020,   /* U+1F693 */
  0x0020,   /* U+1F694 */
  0x0020,   /* U+1F695 */
  0x0020,   /* U+1F696 */
  0x0020,   /* U+1F697 */
  0x0020,   /* U+1F698 */
  0x0020,   /* U+1F699 */
  0x0020,   /* U+1F69A */
  0x0020,   /* U+1F69B */
  0x0020,   /* U+1F69C */
  0x0020,   /* U+1F69D */
  0x0020,   /* U+1F69E */
  0x0020,   /* U+1F69F */
  0x0020,   /* U+1F6A0 */
  0x0020,   /* U+1F6A1 */
  0x0020,   /* U+1F6A2 */
  0x0020,   /* U+1F6A3 */
  0x0020,   /* U+1F6A4 */
  0x0020,   /* U+1F6A5 */
  0x0020,   /* U+1F6A6 */
  0x0020,   /* U+1F6A7 */
  0x0020,   /* U+1F6A8 */
  0x0020,   /* U+1F6A9 */
  0x0020,   /* U+1F6AA */
  0x0020,   /* U+1F6AB */
  0x0020,   /* U+1F6AC */
  0x0020,   /* U+1F6AD */
  0x0020,   /* U+1F6AE */
  0x0020,   /* U+1F6AF */
  0x0020,   /* U+1F6B0 */
  0x0020,   /* U+1F6B1 */
  0x0020,   /* U+1F6B2 */
  0x0020,   /* U+1F6B3 */
  0x0020,   /* U+1F6B4 */
  0x0020,   /* U+1F6B5 */
  0x0020,   /* U+1F6B6 */
  0x0020,   /* U+1F6B7 */
  0x0020,   /* U+1F6B8 */
  0x0020,   /* U+1F6B9 */
  0x0020,   /* U+1F6BA */
  0x0020,   /* U+1F6BB */
  0x0020,   /* U+1F6BC */
  0x0020,   /* U+1F6BD */
  0x0020,   /* U+1F6BE */
  0x0020,   /* U+1F6BF */
  0x0020,   /* U+1F6C0 */
  0x0020,   /* U+1F6C1 */
  0x0020,   /* U+1F6C2 */
  0x0020,   /* U+1F6C3 */
  0x0020,   /* U+1F6C4 */
  0x0020,   /* U+1F6C5 */
  0x0020,   /* U+1F6C6 */
  0x0020,   /* U+1F6C7 */
  0x0020,   /* U+1F6C8 */
  0x0020,   /* U+1F6C9 */
  0x0020,   /* U+1F6CA */
  0x0020,   /* U+1F6CB */
  0x0020,   /* U+1F6CC */
  0x0020,   /* U+1F6CD */
  0x0020,   /* U+1F6CE */
  0x0020,   /* U+1F6CF */
  0x0020,   /* U+1F6D0 */
  0x0020,   /* U+1F6D1 */
  0x0020,   /* U+1F6D2 */
  0x0020,   /* U+1F6D3 */
  0x0020,   /* U+1F6D4 */
  0x0020,   /* U+1F6D5 */
  0x0020,   /* U+1F6D6 */
  0x0020,   /* U+1F6D7 */
  0x0020,   /* U+1F6D8 */
  0x0020,   /* U+1F6D9 */
  0x0020,   /* U+1F6DA */
  0x0020,   /* U+1F6DB */
  0x0020,   /* U+1F6DC */
  0x0020,   /* U+1F6DD */
  0x0020,   /* U+1F6DE */
  0x0020,   /* U+1F6DF */
  0x0020,   /* U+1F6E0 */
  0x0020,   /* U+1F6E1 */
  0x0020,   /* U+1F6E2 */
  0x0020,   /* U+1F6E3 */
  0x0020,   /* U+1F6E4 */
  0x0020,   /* U+1F6E5 */
  0x0020,   /* U+1F6E6 */
  0x0020,   /* U+1F6E7 */
  0x0020,   /* U+1F6E8 */
  0x0020,   /* U+1F6E9 */
  0x0020,   /* U+1F6EA */
  0x0020,   /* U+1F6EB */
  0x0020,   /* U+1F6EC */
  0x0020,   /* U+1F6ED */
  0x0020,   /* U+1F6EE */
  0x0020,   /* U+1F6EF */
  0x0020,   /* U+1F6F0 */
  0x0020,   /* U+1F6F1 */
  0x0020,   /* U+1F6F2 */
  0x0020,   /* U+1F6F3 */
  0x0020,   /* U+1F6F4 */
  0x0020,   /* U+1F6F5 */
  0x0020,   /* U+1F6F6 */
  0x0020,   /* U+1F6F7 */
  0x0020,   /* U+1F6F8 */
  0x0020,   /* U+1F6F9 */
  0x0020,   /* U+1F6FA */
  0x0020,   /* U+1F6FB */
  0x0020,   /* U+1F6FC */
  0x0020,   /* U+1F6FD */
  0x0020,   /* U+1F6FE */
  0x0020,   /* U+1F6FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F600 */
  0x0002,   /* U+1F601 */
  0x0002,   /* U+1F602 */
  0x0002,   /* U+1F603 */
  0x0002,   /* U+1F604 */
  0x0002,   /* U+1F605 */
  0x0002,   /* U+1F606 */
  0x0002,   /* U+1F607 */
  0x0002,   /* U+1F608 */
  0x0002,   /* U+1F609 */
  0x0002,   /* U+1F60A */
  0x0002,   /* U+1F60B */
  0x0002,   /* U+1F60C */
  0x0002,   /* U+1F60D */
  0x0002,   /* U+1F60E */
  0x0002,   /* U+1F60F */
  0x0002,   /* U+1F610 */
  0x0002,   /* U+1F611 */
  0x0002,   /* U+1F612 */
  0x0002,   /* U+1F613 */
  0x0002,   /* U+1F614 */
  0x0002,   /* U+1F615 */
  0x0002,   /* U+1F616 */
  0x0002,   /* U+1F617 */
  0x0002,   /* U+1F618 */
  0x0002,   /* U+1F619 */
  0x0002,   /* U+1F61A */
  0x0002,   /* U+1F61B */
  0x0002,   /* U+1F61C */
  0x0002,   /* U+1F61D */
  0x0002,   /* U+1F61E */
  0x0002,   /* U+1F61F */
  0x0002,   /* U+1F620 */
  0x0002,   /* U+1F621 */
  0x0002,   /* U+1F622 */
  0x0002,   /* U+1F623 */
  0x0002,   /* U+1F624 */
  0x0002,   /* U+1F625 */
  0x0002,   /* U+1F626 */
  0x0002,   /* U+1F627 */
  0x0002,   /* U+1F628 */
  0x0002,   /* U+1F629 */
  0x0002,   /* U+1F62A */
  0x0002,   /* U+1F62B */
  0x0002,   /* U+1F62C */
  0x0002,   /* U+1F62D */
  0x0002,   /* U+1F62E */
  0x0002,   /* U+1F62F */
  0x0002,   /* U+1F630 */
  0x0002,   /* U+1F631 */
  0x0002,   /* U+1F632 */
  0x0002,   /* U+1F633 */
  0x0002,   /* U+1F634 */
  0x0002,   /* U+1F635 */
  0x0002,   /* U+1F636 */
  0x0002,   /* U+1F637 */
  0x0002,   /* U+1F638 */
  0x0002,   /* U+1F639 */
  0x0002,   /* U+1F63A */
  0x0002,   /* U+1F63B */
  0x0002,   /* U+1F63C */
  0x0002,   /* U+1F63D */
  0x0002,   /* U+1F63E */
  0x0002,   /* U+1F63F */
  0x0002,   /* U+1F640 */
  0x0002,   /* U+1F641 */
  0x0002,   /* U+1F642 */
  0x0002,   /* U+1F643 */
  0x0002,   /* U+1F644 */
  0x0002,   /* U+1F645 */
  0x0002,   /* U+1F646 */
  0x0002,   /* U+1F647 */
  0x0002,   /* U+1F648 */
  0x0002,   /* U+1F649 */
  0x0002,   /* U+1F64A */
  0x0002,   /* U+1F64B */
  0x0002,   /* U+1F64C */
  0x0002,   /* U+1F64D */
  0x0002,   /* U+1F64E */
  0x0002,   /* U+1F64F */
  0x0002,   /* U+1F650 */
  0x0002,   /* U+1F651 */
  0x0002,   /* U+1F652 */
  0x0002,   /* U+1F653 */
  0x0002,   /* U+1F654 */
  0x0002,   /* U+1F655 */
  0x0002,   /* U+1F656 */
  0x0002,   /* U+1F657 */
  0x0002,   /* U+1F658 */
  0x0002,   /* U+1F659 */
  0x0002,   /* U+1F65A */
  0x0002,   /* U+1F65B */
  0x0002,   /* U+1F65C */
  0x0002,   /* U+1F65D */
  0x0002,   /* U+1F65E */
  0x0002,   /* U+1F65F */
  0x0002,   /* U+1F660 */
  0x0002,   /* U+1F661 */
  0x0002,   /* U+1F662 */
  0x0002,   /* U+1F663 */
  0x0002,   /* U+1F664 */
  0x0002,   /* U+1F665 */
  0x0002,   /* U+1F666 */
  0x0002,   /* U+1F667 */
  0x0002,   /* U+1F668 */
  0x0002,   /* U+1F669 */
  0x0002,   /* U+1F66A */
  0x0002,   /* U+1F66B */
  0x0002,   /* U+1F66C */
  0x0002,   /* U+1F66D */
  0x0002,   /* U+1F66E */
  0x0002,   /* U+1F66F */
  0x0002,   /* U+1F670 */
  0x0002,   /* U+1F671 */
  0x0002,   /* U+1F672 */
  0x0002,   /* U+1F673 */
  0x0002,   /* U+1F674 */
  0x0002,   /* U+1F675 */
  0x0002,   /* U+1F676 */
  0x0002,   /* U+1F677 */
  0x0002,   /* U+1F678 */
  0x0002,   /* U+1F679 */
  0x0002,   /* U+1F67A */
  0x0002,   /* U+1F67B */
  0x0002,   /* U+1F67C */
  0x0002,   /* U+1F67D */
  0x0002,   /* U+1F67E */
  0x0002,   /* U+1F67F */
  0x0002,   /* U+1F680 */
  0x0002,   /* U+1F681 */
  0x0002,   /* U+1F682 */
  0x0002,   /* U+1F683 */
  0x0002,   /* U+1F684 */
  0x0002,   /* U+1F685 */
  0x0002,   /* U+1F686 */
  0x0002,   /* U+1F687 */
  0x0002,   /* U+1F688 */
  0x0002,   /* U+1F689 */
  0x0002,   /* U+1F68A */
  0x0002,   /* U+1F68B */
  0x0002,   /* U+1F68C */
  0x0002,   /* U+1F68D */
  0x0002,   /* U+1F68E */
  0x0002,   /* U+1F68F */
  0x0002,   /* U+1F690 */
  0x0002,   /* U+1F691 */
  0x0002,   /* U+1F692 */
  0x0002,   /* U+1F693 */
  0x0002,   /* U+1F694 */
  0x0002,   /* U+1F695 */
  0x0002,   /* U+1F696 */
  0x0002,   /* U+1F697 */
  0x0002,   /* U+1F698 */
  0x0002,   /* U+1F699 */
  0x0002,   /* U+1F69A */
  0x0002,   /* U+1F69B */
  0x0002,   /* U+1F69C */
  0x0002,   /* U+1F69D */
  0x0002,   /* U+1F69E */
  0x0002,   /* U+1F69F */
  0x0002,   /* U+1F6A0 */
  0x0002,   /* U+1F6A1 */
  0x0002,   /* U+1F6A2 */
  0x0002,   /* U+1F6A3 */
  0x0002,   /* U+1F6A4 */
  0x0002,   /* U+1F6A5 */
  0x0002,   /* U+1F6A6 */
  0x0002,   /* U+1F6A7 */
  0x0002,   /* U+1F6A8 */
  0x0002,   /* U+1F6A9 */
  0x0002,   /* U+1F6AA */
  0x0002,   /* U+1F6AB */
  0x0002,   /* U+1F6AC */
  0x0002,   /* U+1F6AD */
  0x0002,   /* U+1F6AE */
  0x0002,   /* U+1F6AF */
  0x0002,   /* U+1F6B0 */
  0x0002,   /* U+1F6B1 */
  0x0002,   /* U+1F6B2 */
  0x0002,   /* U+1F6B3 */
  0x0002,   /* U+1F6B4 */
  0x0002,   /* U+1F6B5 */
  0x0002,   /* U+1F6B6 */
  0x0002,   /* U+1F6B7 */
  0x0002,   /* U+1F6B8 */
  0x0002,   /* U+1F6B9 */
  0x0002,   /* U+1F6BA */
  0x0002,   /* U+1F6BB */
  0x0002,   /* U+1F6BC */
  0x0002,   /* U+1F6BD */
  0x0002,   /* U+1F6BE */
  0x0002,   /* U+1F6BF */
  0x0002,   /* U+1F6C0 */
  0x0002,   /* U+1F6C1 */
  0x0002,   /* U+1F6C2 */
  0x0002,   /* U+1F6C3 */
  0x0002,   /* U+1F6C4 */
  0x0002,   /* U+1F6C5 */
  0x0002,   /* U+1F6C6 */
  0x0002,   /* U+1F6C7 */
  0x0002,   /* U+1F6C8 */
  0x0002,   /* U+1F6C9 */
  0x0002,   /* U+1F6CA */
  0x0002,   /* U+1F6CB */
  0x0002,   /* U+1F6CC */
  0x0002,   /* U+1F6CD */
  0x0002,   /* U+1F6CE */
  0x0002,   /* U+1F6CF */
  0x0002,   /* U+1F6D0 */
  0x0002,   /* U+1F6D1 */
  0x0002,   /* U+1F6D2 */
  0x0002,   /* U+1F6D3 */
  0x0002,   /* U+1F6D4 */
  0x0002,   /* U+1F6D5 */
  0x0002,   /* U+1F6D6 */
  0x0002,   /* U+1F6D7 */
  0x0002,   /* U+1F6D8 */
  0x0002,   /* U+1F6D9 */
  0x0002,   /* U+1F6DA */
  0x0002,   /* U+1F6DB */
  0x0002,   /* U+1F6DC */
  0x0002,   /* U+1F6DD */
  0x0002,   /* U+1F6DE */
  0x0002,   /* U+1F6DF */
  0x0002,   /* U+1F6E0 */
  0x0002,   /* U+1F6E1 */
  0x0002,   /* U+1F6E2 */
  0x0002,   /* U+1F6E3 */
  0x0002,   /* U+1F6E4 */
  0x0002,   /* U+1F6E5 */
  0x0002,   /* U+1F6E6 */
  0x0002,   /* U+1F6E7 */
  0x0002,   /* U+1F6E8 */
  0x0002,   /* U+1F6E9 */
  0x0002,   /* U+1F6EA */
  0x0002,   /* U+1F6EB */
  0x0002,   /* U+1F6EC */
  0x0002,   /* U+1F6ED */
  0x0002,   /* U+1F6EE */
  0x0002,   /* U+1F6EF */
  0x0002,   /* U+1F6F0 */
  0x0002,   /* U+1F6F1 */
  0x0002,   /* U+1F6F2 */
  0x0002,   /* U+1F6F3 */
  0x0002,   /* U+1F6F4 */
  0x0002,   /* U+1F6F5 */
  0x0002,   /* U+1F6F6 */
  0x0002,   /* U+1F6F7 */
  0x0002,   /* U+1F6F8 */
  0x0002,   /* U+1F6F9 */
  0x0002,   /* U+1F6FA */
  0x0002,   /* U+1F6FB */
  0x0002,   /* U+1F6FC */
  0x0002,   /* U+1F6FD */
  0x0002,   /* U+1F6FE */
  0x0002,   /* U+1F6FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1F600 */
  0x0000,   /* U+1F601 */
  0x0000,   /* U+1F602 */
  0x0000,   /* U+1F603 */
  0x0000,   /* U+1F604 */
  0x0000,   /* U+1F605 */
  0x0000,   /* U+1F606 */
  0x0000,   /* U+1F607 */
  0x0000,   /* U+1F608 */
  0x0000,   /* U+1F609 */
  0x0000,   /* U+1F60A */
  0x0000,   /* U+1F60B */
  0x0000,   /* U+1F60C */
  0x0000,   /* U+1F60D */
  0x0000,   /* U+1F60E */
  0x0000,   /* U+1F60F */
  0x0000,   /* U+1F610 */
  0x0000,   /* U+1F611 */
  0x0000,   /* U+1F612 */
  0x0000,   /* U+1F613 */
  0x0000,   /* U+1F614 */
  0x0000,   /* U+1F615 */
  0x0000,   /* U+1F616 */
  0x0000,   /* U+1F617 */
  0x0000,   /* U+1F618 */
  0x0000,   /* U+1F619 */
  0x0000,   /* U+1F61A */
  0x0000,   /* U+1F61B */
  0x0000,   /* U+1F61C */
  0x0000,   /* U+1F61D */
  0x0000,   /* U+1F61E */
  0x0000,   /* U+1F61F */
  0x0000,   /* U+1F620 */
  0x0000,   /* U+1F621 */
  0x0000,   /* U+1F622 */
  0x0000,   /* U+1F623 */
  0x0000,   /* U+1F624 */
  0x0000,   /* U+1F625 */
  0x0000,   /* U+1F626 */
  0x0000,   /* U+1F627 */
  0x0000,   /* U+1F628 */
  0x0000,   /* U+1F629 */
  0x0000,   /* U+1F62A */
  0x0000,   /* U+1F62B */
  0x0000,   /* U+1F62C */
  0x0000,   /* U+1F62D */
  0x0000,   /* U+1F62E */
  0x0000,   /* U+1F62F */
  0x0000,   /* U+1F630 */
  0x0000,   /* U+1F631 */
  0x0000,   /* U+1F632 */
  0x0000,   /* U+1F633 */
  0x0000,   /* U+1F634 */
  0x0000,   /* U+1F635 */
  0x0000,   /* U+1F636 */
  0x0000,   /* U+1F637 */
  0x0000,   /* U+1F638 */
  0x0000,   /* U+1F639 */
  0x0000,   /* U+1F63A */
  0x0000,   /* U+1F63B */
  0x0000,   /* U+1F63C */
  0x0000,   /* U+1F63D */
  0x0000,   /* U+1F63E */
  0x0000,   /* U+1F63F */
  0x0000,   /* U+1F640 */
  0x0000,   /* U+1F641 */
  0x0000,   /* U+1F642 */
  0x0000,   /* U+1F643 */
  0x0000,   /* U+1F644 */
  0x0000,   /* U+1F645 */
  0x0000,   /* U+1F646 */
  0x0000,   /* U+1F647 */
  0x0000,   /* U+1F648 */
  0x0000,   /* U+1F649 */
  0x0000,   /* U+1F64A */
  0x0000,   /* U+1F64B */
  0x0000,   /* U+1F64C */
  0x0000,   /* U+1F64D */
  0x0000,   /* U+1F64E */
  0x0000,   /* U+1F64F */
  0x0000,   /* U+1F650 */
  0x0000,   /* U+1F651 */
  0x0000,   /* U+1F652 */
  0x0000,   /* U+1F653 */
  0x0000,   /* U+1F654 */
  0x0000,   /* U+1F655 */
  0x0000,   /* U+1F656 */
  0x0000,   /* U+1F657 */
  0x0000,   /* U+1F658 */
  0x0000,   /* U+1F659 */
  0x0000,   /* U+1F65A */
  0x0000,   /* U+1F65B */
  0x0000,   /* U+1F65C */
  0x0000,   /* U+1F65D */
  0x0000,   /* U+1F65E */
  0x0000,   /* U+1F65F */
  0x0000,   /* U+1F660 */
  0x0000,   /* U+1F661 */
  0x0000,   /* U+1F662 */
  0x0000,   /* U+1F663 */
  0x0000,   /* U+1F664 */
  0x0000,   /* U+1F665 */
  0x0000,   /* U+1F666 */
  0x0000,   /* U+1F667 */
  0x0000,   /* U+1F668 */
  0x0000,   /* U+1F669 */
  0x0000,   /* U+1F66A */
  0x0000,   /* U+1F66B */
  0x0000,   /* U+1F66C */
  0x0000,   /* U+1F66D */
  0x0000,   /* U+1F66E */
  0x0000,   /* U+1F66F */
  0x0000,   /* U+1F670 */
  0x0000,   /* U+1F671 */
  0x0000,   /* U+1F672 */
  0x0000,   /* U+1F673 */
  0x0000,   /* U+1F674 */
  0x0000,   /* U+1F675 */
  0x0000,   /* U+1F676 */
  0x0000,   /* U+1F677 */
  0x0000,   /* U+1F678 */
  0x0000,   /* U+1F679 */
  0x0000,   /* U+1F67A */
  0x0000,   /* U+1F67B */
  0x0000,   /* U+1F67C */
  0x0000,   /* U+1F67D */
  0x0000,   /* U+1F67E */
  0x0000,   /* U+1F67F */
  0x0000,   /* U+1F680 */
  0x0000,   /* U+1F681 */
  0x0000,   /* U+1F682 */
  0x0000,   /* U+1F683 */
  0x0000,   /* U+1F684 */
  0x0000,   /* U+1F685 */
  0x0000,   /* U+1F686 */
  0x0000,   /* U+1F687 */
  0x0000,   /* U+1F688 */
  0x0000,   /* U+1F689 */
  0x0000,   /* U+1F68A */
  0x0000,   /* U+1F68B */
  0x0000,   /* U+1F68C */
  0x0000,   /* U+1F68D */
  0x0000,   /* U+1F68E */
  0x0000,   /* U+1F68F */
  0x0000,   /* U+1F690 */
  0x0000,   /* U+1F691 */
  0x0000,   /* U+1F692 */
  0x0000,   /* U+1F693 */
  0x0000,   /* U+1F694 */
  0x0000,   /* U+1F695 */
  0x0000,   /* U+1F696 */
  0x0000,   /* U+1F697 */
  0x0000,   /* U+1F698 */
  0x0000,   /* U+1F699 */
  0x0000,   /* U+1F69A */
  0x0000,   /* U+1F69B */
  0x0000,   /* U+1F69C */
  0x0000,   /* U+1F69D */
  0x0000,   /* U+1F69E */
  0x0000,   /* U+1F69F */
  0x0000,   /* U+1F6A0 */
  0x0000,   /* U+1F6A1 */
  0x0000,   /* U+1F6A2 */
  0x0000,   /* U+1F6A3 */
  0x0000,   /* U+1F6A4 */
  0x0000,   /* U+1F6A5 */
  0x0000,   /* U+1F6A6 */
  0x0000,   /* U+1F6A7 */
  0x0000,   /* U+1F6A8 */
  0x0000,   /* U+1F6A9 */
  0x0000,   /* U+1F6AA */
  0x0000,   /* U+1F6AB */
  0x0000,   /* U+1F6AC */
  0x0000,   /* U+1F6AD */
  0x0000,   /* U+1F6AE */
  0x0000,   /* U+1F6AF */
  0x0000,   /* U+1F6B0 */
  0x0000,   /* U+1F6B1 */
  0x0000,   /* U+1F6B2 */
  0x0000,   /* U+1F6B3 */
  0x0000,   /* U+1F6B4 */
  0x0000,   /* U+1F6B5 */
  0x0000,   /* U+1F6B6 */
  0x0000,   /* U+1F6B7 */
  0x0000,   /* U+1F6B8 */
  0x0000,   /* U+1F6B9 */
  0x0000,   /* U+1F6BA */
  0x0000,   /* U+1F6BB */
  0x0000,   /* U+1F6BC */
  0x0000,   /* U+1F6BD */
  0x0000,   /* U+1F6BE */
  0x0000,   /* U+1F6BF */
  0x0000,   /* U+1F6C0 */
  0x0000,   /* U+1F6C1 */
  0x0000,   /* U+1F6C2 */
  0x0000,   /* U+1F6C3 */
  0x0000,   /* U+1F6C4 */
  0x0000,   /* U+1F6C5 */
  0x0000,   /* U+1F6C6 */
  0x0000,   /* U+1F6C7 */
  0x0000,   /* U+1F6C8 */
  0x0000,   /* U+1F6C9 */
  0x0000,   /* U+1F6CA */
  0x0000,   /* U+1F6CB */
  0x0000,   /* U+1F6CC */
  0x0000,   /* U+1F6CD */
  0x0000,   /* U+1F6CE */
  0x0000,   /* U+1F6CF */
  0x0000,   /* U+1F6D0 */
  0x0000,   /* U+1F6D1 */
  0x0000,   /* U+1F6D2 */
  0xF6D3,   /* U+1F6D3 */
  0xF6D4,   /* U+1F6D4 */
  0xF6D5,   /* U+1F6D5 */
  0xF6D6,   /* U+1F6D6 */
  0xF6D7,   /* U+1F6D7 */
  0xF6D8,   /* U+1F6D8 */
  0xF6D9,   /* U+1F6D9 */
  0xF6DA,   /* U+1F6DA */
  0xF6DB,   /* U+1F6DB */
  0xF6DC,   /* U+1F6DC */
  0xF6DD,   /* U+1F6DD */
  0xF6DE,   /* U+1F6DE */
  0xF6DF,   /* U+1F6DF */
  0x0000,   /* U+1F6E0 */
  0x0000,   /* U+1F6E1 */
  0x0000,   /* U+1F6E2 */
  0x0000,   /* U+1F6E3 */
  0x0000,   /* U+1F6E4 */
  0x0000,   /* U+1F6E5 */
  0x0000,   /* U+1F6E6 */
  0x0000,   /* U+1F6E7 */
  0x0000,   /* U+1F6E8 */
  0x0000,   /* U+1F6E9 */
  0x0000,   /* U+1F6EA */
  0x0000,   /* U+1F6EB */
  0x0000,   /* U+1F6EC */
  0xF6ED,   /* U+1F6ED */
  0xF6EE,   /* U+1F6EE */
  0xF6EF,   /* U+1F6EF */
  0x0000,   /* U+1F6F0 */
  0x0000,   /* U+1F6F1 */
  0x0000,   /* U+1F6F2 */
  0x0000,   /* U+1F6F3 */
  0x0000,   /* U+1F6F4 */
  0x0000,   /* U+1F6F5 */
  0x0000,   /* U+1F6F6 */
  0xF6F7,   /* U+1F6F7 */
  0xF6F8,   /* U+1F6F8 */
  0xF6F9,   /* U+1F6F9 */
  0xF6FA,   /* U+1F6FA */
  0xF6FB,   /* U+1F6FB */
  0xF6FC,   /* U+1F6FC */
  0xF6FD,   /* U+1F6FD */
  0xF6FE,   /* U+1F6FE */
  0xF6FF,   /* U+1F6FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1F600 */
  0x0000,   /* U+1F601 */
  0x0000,   /* U+1F602 */
  0x0000,   /* U+1F603 */
  0x0000,   /* U+1F604 */
  0x0000,   /* U+1F605 */
  0x0000,   /* U+1F606 */
  0x0000,   /* U+1F607 */
  0x0000,   /* U+1F608 */
  0x0000,   /* U+1F609 */
  0x0000,   /* U+1F60A */
  0x0000,   /* U+1F60B */
  0x0000,   /* U+1F60C */
  0x0000,   /* U+1F60D */
  0x0000,   /* U+1F60E */
  0x0000,   /* U+1F60F */
  0x0000,   /* U+1F610 */
  0x0000,   /* U+1F611 */
  0x0000,   /* U+1F612 */
  0x0000,   /* U+1F613 */
  0x0000,   /* U+1F614 */
  0x0000,   /* U+1F615 */
  0x0000,   /* U+1F616 */
  0x0000,   /* U+1F617 */
  0x0000,   /* U+1F618 */
  0x0000,   /* U+1F619 */
  0x0000,   /* U+1F61A */
  0x0000,   /* U+1F61B */
  0x0000,   /* U+1F61C */
  0x0000,   /* U+1F61D */
  0x0000,   /* U+1F61E */
  0x0000,   /* U+1F61F */
  0x0000,   /* U+1F620 */
  0x0000,   /* U+1F621 */
  0x0000,   /* U+1F622 */
  0x0000,   /* U+1F623 */
  0x0000,   /* U+1F624 */
  0x0000,   /* U+1F625 */
  0x0000,   /* U+1F626 */
  0x0000,   /* U+1F627 */
  0x0000,   /* U+1F628 */
  0x0000,   /* U+1F629 */
  0x0000,   /* U+1F62A */
  0x0000,   /* U+1F62B */
  0x0000,   /* U+1F62C */
  0x0000,   /* U+1F62D */
  0x0000,   /* U+1F62E */
  0x0000,   /* U+1F62F */
  0x0000,   /* U+1F630 */
  0x0000,   /* U+1F631 */
  0x0000,   /* U+1F632 */
  0x0000,   /* U+1F633 */
  0x0000,   /* U+1F634 */
  0x0000,   /* U+1F635 */
  0x0000,   /* U+1F636 */
  0x0000,   /* U+1F637 */
  0x0000,   /* U+1F638 */
  0x0000,   /* U+1F639 */
  0x0000,   /* U+1F63A */
  0x0000,   /* U+1F63B */
  0x0000,   /* U+1F63C */
  0x0000,   /* U+1F63D */
  0x0000,   /* U+1F63E */
  0x0000,   /* U+1F63F */
  0x0000,   /* U+1F640 */
  0x0000,   /* U+1F641 */
  0x0000,   /* U+1F642 */
  0x0000,   /* U+1F643 */
  0x0000,   /* U+1F644 */
  0x0000,   /* U+1F645 */
  0x0000,   /* U+1F646 */
  0x0000,   /* U+1F647 */
  0x0000,   /* U+1F648 */
  0x0000,   /* U+1F649 */
  0x0000,   /* U+1F64A */
  0x0000,   /* U+1F64B */
  0x0000,   /* U+1F64C */
  0x0000,   /* U+1F64D */
  0x0000,   /* U+1F64E */
  0x0000,   /* U+1F64F */
  0x0000,   /* U+1F650 */
  0x0000,   /* U+1F651 */
  0x0000,   /* U+1F652 */
  0x0000,   /* U+1F653 */
  0x0000,   /* U+1F654 */
  0x0000,   /* U+1F655 */
  0x0000,   /* U+1F656 */
  0x0000,   /* U+1F657 */
  0x0000,   /* U+1F658 */
  0x0000,   /* U+1F659 */
  0x0000,   /* U+1F65A */
  0x0000,   /* U+1F65B */
  0x0000,   /* U+1F65C */
  0x0000,   /* U+1F65D */
  0x0000,   /* U+1F65E */
  0x0000,   /* U+1F65F */
  0x0000,   /* U+1F660 */
  0x0000,   /* U+1F661 */
  0x0000,   /* U+1F662 */
  0x0000,   /* U+1F663 */
  0x0000,   /* U+1F664 */
  0x0000,   /* U+1F665 */
  0x0000,   /* U+1F666 */
  0x0000,   /* U+1F667 */
  0x0000,   /* U+1F668 */
  0x0000,   /* U+1F669 */
  0x0000,   /* U+1F66A */
  0x0000,   /* U+1F66B */
  0x0000,   /* U+1F66C */
  0x0000,   /* U+1F66D */
  0x0000,   /* U+1F66E */
  0x0000,   /* U+1F66F */
  0x0000,   /* U+1F670 */
  0x0000,   /* U+1F671 */
  0x0000,   /* U+1F672 */
  0x0000,   /* U+1F673 */
  0x0000,   /* U+1F674 */
  0x0000,   /* U+1F675 */
  0x0000,   /* U+1F676 */
  0x0000,   /* U+1F677 */
  0x0000,   /* U+1F678 */
  0x0000,   /* U+1F679 */
  0x0000,   /* U+1F67A */
  0x0000,   /* U+1F67B */
  0x0000,   /* U+1F67C */
  0x0000,   /* U+1F67D */
  0x0000,   /* U+1F67E */
  0x0000,   /* U+1F67F */
  0x0000,   /* U+1F680 */
  0x0000,   /* U+1F681 */
  0x0000,   /* U+1F682 */
  0x0000,   /* U+1F683 */
  0x0000,   /* U+1F684 */
  0x0000,   /* U+1F685 */
  0x0000,   /* U+1F686 */
  0x0000,   /* U+1F687 */
  0x0000,   /* U+1F688 */
  0x0000,   /* U+1F689 */
  0x0000,   /* U+1F68A */
  0x0000,   /* U+1F68B */
  0x0000,   /* U+1F68C */
  0x0000,   /* U+1F68D */
  0x0000,   /* U+1F68E */
  0x0000,   /* U+1F68F */
  0x0000,   /* U+1F690 */
  0x0000,   /* U+1F691 */
  0x0000,   /* U+1F692 */
  0x0000,   /* U+1F693 */
  0x0000,   /* U+1F694 */
  0x0000,   /* U+1F695 */
  0x0000,   /* U+1F696 */
  0x0000,   /* U+1F697 */
  0x0000,   /* U+1F698 */
  0x0000,   /* U+1F699 */
  0x0000,   /* U+1F69A */
  0x0000,   /* U+1F69B */
  0x0000,   /* U+1F69C */
  0x0000,   /* U+1F69D */
  0x0000,   /* U+1F69E */
  0x0000,   /* U+1F69F */
  0x0000,   /* U+1F6A0 */
  0x0000,   /* U+1F6A1 */
  0x0000,   /* U+1F6A2 */
  0x0000,   /* U+1F6A3 */
  0x0000,   /* U+1F6A4 */
  0x0000,   /* U+1F6A5 */
  0x0000,   /* U+1F6A6 */
  0x0000,   /* U+1F6A7 */
  0x0000,   /* U+1F6A8 */
  0x0000,   /* U+1F6A9 */
  0x0000,   /* U+1F6AA */
  0x0000,   /* U+1F6AB */
  0x0000,   /* U+1F6AC */
  0x0000,   /* U+1F6AD */
  0x0000,   /* U+1F6AE */
  0x0000,   /* U+1F6AF */
  0x0000,   /* U+1F6B0 */
  0x0000,   /* U+1F6B1 */
  0x0000,   /* U+1F6B2 */
  0x0000,   /* U+1F6B3 */
  0x0000,   /* U+1F6B4 */
  0x0000,   /* U+1F6B5 */
  0x0000,   /* U+1F6B6 */
  0x0000,   /* U+1F6B7 */
  0x0000,   /* U+1F6B8 */
  0x0000,   /* U+1F6B9 */
  0x0000,   /* U+1F6BA */
  0x0000,   /* U+1F6BB */
  0x0000,   /* U+1F6BC */
  0x0000,   /* U+1F6BD */
  0x0000,   /* U+1F6BE */
  0x0000,   /* U+1F6BF */
  0x0000,   /* U+1F6C0 */
  0x0000,   /* U+1F6C1 */
  0x0000,   /* U+1F6C2 */
  0x0000,   /* U+1F6C3 */
  0x0000,   /* U+1F6C4 */
  0x0000,   /* U+1F6C5 */
  0x0000,   /* U+1F6C6 */
  0x0000,   /* U+1F6C7 */
  0x0000,   /* U+1F6C8 */
  0x0000,   /* U+1F6C9 */
  0x0000,   /* U+1F6CA */
  0x0000,   /* U+1F6CB */
  0x0000,   /* U+1F6CC */
  0x0000,   /* U+1F6CD */
  0x0000,   /* U+1F6CE */
  0x0000,   /* U+1F6CF */
  0x0000,   /* U+1F6D0 */
  0x0000,   /* U+1F6D1 */
  0x0000,   /* U+1F6D2 */
  0x0000,   /* U+1F6D3 */
  0x0000,   /* U+1F6D4 */
  0x0000,   /* U+1F6D5 */
  0x0000,   /* U+1F6D6 */
  0x0000,   /* U+1F6D7 */
  0x0000,   /* U+1F6D8 */
  0x0000,   /* U+1F6D9 */
  0x0000,   /* U+1F6DA */
  0x0000,   /* U+1F6DB */
  0x0000,   /* U+1F6DC */
  0x0000,   /* U+1F6DD */
  0x0000,   /* U+1F6DE */
  0x0000,   /* U+1F6DF */
  0x0000,   /* U+1F6E0 */
  0x0000,   /* U+1F6E1 */
  0x0000,   /* U+1F6E2 */
  0x0000,   /* U+1F6E3 */
  0x0000,   /* U+1F6E4 */
  0x0000,   /* U+1F6E5 */
  0x0000,   /* U+1F6E6 */
  0x0000,   /* U+1F6E7 */
  0x0000,   /* U+1F6E8 */
  0x0000,   /* U+1F6E9 */
  0x0000,   /* U+1F6EA */
  0x0000,   /* U+1F6EB */
  0x0000,   /* U+1F6EC */
  0x0000,   /* U+1F6ED */
  0x0000,   /* U+1F6EE */
  0x0000,   /* U+1F6EF */
  0x0000,   /* U+1F6F0 */
  0x0000,   /* U+1F6F1 */
  0x0000,   /* U+1F6F2 */
  0x0000,   /* U+1F6F3 */
  0x0000,   /* U+1F6F4 */
  0x0000,   /* U+1F6F5 */
  0x0000,   /* U+1F6F6 */
  0x0000,   /* U+1F6F7 */
  0x0000,   /* U+1F6F8 */
  0x0000,   /* U+1F6F9 */
  0x0000,   /* U+1F6FA */
  0x0000,   /* U+1F6FB */
  0x0000,   /* U+1F6FC */
  0x0000,   /* U+1F6FD */
  0x0000,   /* U+1F6FE */
  0x0000,   /* U+1F6FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1F600 */
  0x0000,   /* U+1F601 */
  0x0000,   /* U+1F602 */
  0x0000,   /* U+1F603 */
  0x0000,   /* U+1F604 */
  0x0000,   /* U+1F605 */
  0x0000,   /* U+1F606 */
  0x0000,   /* U+1F607 */
  0x0000,   /* U+1F608 */
  0x0000,   /* U+1F609 */
  0x0000,   /* U+1F60A */
  0x0000,   /* U+1F60B */
  0x0000,   /* U+1F60C */
  0x0000,   /* U+1F60D */
  0x0000,   /* U+1F60E */
  0x0000,   /* U+1F60F */
  0x0000,   /* U+1F610 */
  0x0000,   /* U+1F611 */
  0x0000,   /* U+1F612 */
  0x0000,   /* U+1F613 */
  0x0000,   /* U+1F614 */
  0x0000,   /* U+1F615 */
  0x0000,   /* U+1F616 */
  0x0000,   /* U+1F617 */
  0x0000,   /* U+1F618 */
  0x0000,   /* U+1F619 */
  0x0000,   /* U+1F61A */
  0x0000,   /* U+1F61B */
  0x0000,   /* U+1F61C */
  0x0000,   /* U+1F61D */
  0x0000,   /* U+1F61E */
  0x0000,   /* U+1F61F */
  0x0000,   /* U+1F620 */
  0x0000,   /* U+1F621 */
  0x0000,   /* U+1F622 */
  0x0000,   /* U+1F623 */
  0x0000,   /* U+1F624 */
  0x0000,   /* U+1F625 */
  0x0000,   /* U+1F626 */
  0x0000,   /* U+1F627 */
  0x0000,   /* U+1F628 */
  0x0000,   /* U+1F629 */
  0x0000,   /* U+1F62A */
  0x0000,   /* U+1F62B */
  0x0000,   /* U+1F62C */
  0x0000,   /* U+1F62D */
  0x0000,   /* U+1F62E */
  0x0000,   /* U+1F62F */
  0x0000,   /* U+1F630 */
  0x0000,   /* U+1F631 */
  0x0000,   /* U+1F632 */
  0x0000,   /* U+1F633 */
  0x0000,   /* U+1F634 */
  0x0000,   /* U+1F635 */
  0x0000,   /* U+1F636 */
  0x0000,   /* U+1F637 */
  0x0000,   /* U+1F638 */
  0x0000,   /* U+1F639 */
  0x0000,   /* U+1F63A */
  0x0000,   /* U+1F63B */
  0x0000,   /* U+1F63C */
  0x0000,   /* U+1F63D */
  0x0000,   /* U+1F63E */
  0x0000,   /* U+1F63F */
  0x0000,   /* U+1F640 */
  0x0000,   /* U+1F641 */
  0x0000,   /* U+1F642 */
  0x0000,   /* U+1F643 */
  0x0000,   /* U+1F644 */
  0x0000,   /* U+1F645 */
  0x0000,   /* U+1F646 */
  0x0000,   /* U+1F647 */
  0x0000,   /* U+1F648 */
  0x0000,   /* U+1F649 */
  0x0000,   /* U+1F64A */
  0x0000,   /* U+1F64B */
  0x0000,   /* U+1F64C */
  0x0000,   /* U+1F64D */
  0x0000,   /* U+1F64E */
  0x0000,   /* U+1F64F */
  0x0000,   /* U+1F650 */
  0x0000,   /* U+1F651 */
  0x0000,   /* U+1F652 */
  0x0000,   /* U+1F653 */
  0x0000,   /* U+1F654 */
  0x0000,   /* U+1F655 */
  0x0000,   /* U+1F656 */
  0x0000,   /* U+1F657 */
  0x0000,   /* U+1F658 */
  0x0000,   /* U+1F659 */
  0x0000,   /* U+1F65A */
  0x0000,   /* U+1F65B */
  0x0000,   /* U+1F65C */
  0x0000,   /* U+1F65D */
  0x0000,   /* U+1F65E */
  0x0000,   /* U+1F65F */
  0x0000,   /* U+1F660 */
  0x0000,   /* U+1F661 */
  0x0000,   /* U+1F662 */
  0x0000,   /* U+1F663 */
  0x0000,   /* U+1F664 */
  0x0000,   /* U+1F665 */
  0x0000,   /* U+1F666 */
  0x0000,   /* U+1F667 */
  0x0000,   /* U+1F668 */
  0x0000,   /* U+1F669 */
  0x0000,   /* U+1F66A */
  0x0000,   /* U+1F66B */
  0x0000,   /* U+1F66C */
  0x0000,   /* U+1F66D */
  0x0000,   /* U+1F66E */
  0x0000,   /* U+1F66F */
  0x0000,   /* U+1F670 */
  0x0000,   /* U+1F671 */
  0x0000,   /* U+1F672 */
  0x0000,   /* U+1F673 */
  0x0000,   /* U+1F674 */
  0x0000,   /* U+1F675 */
  0x0000,   /* U+1F676 */
  0x0000,   /* U+1F677 */
  0x0000,   /* U+1F678 */
  0x0000,   /* U+1F679 */
  0x0000,   /* U+1F67A */
  0x0000,   /* U+1F67B */
  0x0000,   /* U+1F67C */
  0x0000,   /* U+1F67D */
  0x0000,   /* U+1F67E */
  0x0000,   /* U+1F67F */
  0x0000,   /* U+1F680 */
  0x0000,   /* U+1F681 */
  0x0000,   /* U+1F682 */
  0x0000,   /* U+1F683 */
  0x0000,   /* U+1F684 */
  0x0000,   /* U+1F685 */
  0x0000,   /* U+1F686 */
  0x0000,   /* U+1F687 */
  0x0000,   /* U+1F688 */
  0x0000,   /* U+1F689 */
  0x0000,   /* U+1F68A */
  0x0000,   /* U+1F68B */
  0x0000,   /* U+1F68C */
  0x0000,   /* U+1F68D */
  0x0000,   /* U+1F68E */
  0x0000,   /* U+1F68F */
  0x0000,   /* U+1F690 */
  0x0000,   /* U+1F691 */
  0x0000,   /* U+1F692 */
  0x0000,   /* U+1F693 */
  0x0000,   /* U+1F694 */
  0x0000,   /* U+1F695 */
  0x0000,   /* U+1F696 */
  0x0000,   /* U+1F697 */
  0x0000,   /* U+1F698 */
  0x0000,   /* U+1F699 */
  0x0000,   /* U+1F69A */
  0x0000,   /* U+1F69B */
  0x0000,   /* U+1F69C */
  0x0000,   /* U+1F69D */
  0x0000,   /* U+1F69E */
  0x0000,   /* U+1F69F */
  0x0000,   /* U+1F6A0 */
  0x0000,   /* U+1F6A1 */
  0x0000,   /* U+1F6A2 */
  0x0000,   /* U+1F6A3 */
  0x0000,   /* U+1F6A4 */
  0x0000,   /* U+1F6A5 */
  0x0000,   /* U+1F6A6 */
  0x0000,   /* U+1F6A7 */
  0x0000,   /* U+1F6A8 */
  0x0000,   /* U+1F6A9 */
  0x0000,   /* U+1F6AA */
  0x0000,   /* U+1F6AB */
  0x0000,   /* U+1F6AC */
  0x0000,   /* U+1F6AD */
  0x0000,   /* U+1F6AE */
  0x0000,   /* U+1F6AF */
  0x0000,   /* U+1F6B0 */
  0x0000,   /* U+1F6B1 */
  0x0000,   /* U+1F6B2 */
  0x0000,   /* U+1F6B3 */
  0x0000,   /* U+1F6B4 */
  0x0000,   /* U+1F6B5 */
  0x0000,   /* U+1F6B6 */
  0x0000,   /* U+1F6B7 */
  0x0000,   /* U+1F6B8 */
  0x0000,   /* U+1F6B9 */
  0x0000,   /* U+1F6BA */
  0x0000,   /* U+1F6BB */
  0x0000,   /* U+1F6BC */
  0x0000,   /* U+1F6BD */
  0x0000,   /* U+1F6BE */
  0x0000,   /* U+1F6BF */
  0x0000,   /* U+1F6C0 */
  0x0000,   /* U+1F6C1 */
  0x0000,   /* U+1F6C2 */
  0x0000,   /* U+1F6C3 */
  0x0000,   /* U+1F6C4 */
  0x0000,   /* U+1F6C5 */
  0x0000,   /* U+1F6C6 */
  0x0000,   /* U+1F6C7 */
  0x0000,   /* U+1F6C8 */
  0x0000,   /* U+1F6C9 */
  0x0000,   /* U+1F6CA */
  0x0000,   /* U+1F6CB */
  0x0000,   /* U+1F6CC */
  0x0000,   /* U+1F6CD */
  0x0000,   /* U+1F6CE */
  0x0000,   /* U+1F6CF */
  0x0000,   /* U+1F6D0 */
  0x0000,   /* U+1F6D1 */
  0x0000,   /* U+1F6D2 */
  0x0000,   /* U+1F6D3 */
  0x0000,   /* U+1F6D4 */
  0x0000,   /* U+1F6D5 */
  0x0000,   /* U+1F6D6 */
  0x0000,   /* U+1F6D7 */
  0x0000,   /* U+1F6D8 */
  0x0000,   /* U+1F6D9 */
  0x0000,   /* U+1F6DA */
  0x0000,   /* U+1F6DB */
  0x0000,   /* U+1F6DC */
  0x0000,   /* U+1F6DD */
  0x0000,   /* U+1F6DE */
  0x0000,   /* U+1F6DF */
  0x0000,   /* U+1F6E0 */
  0x0000,   /* U+1F6E1 */
  0x0000,   /* U+1F6E2 */
  0x0000,   /* U+1F6E3 */
  0x0000,   /* U+1F6E4 */
  0x0000,   /* U+1F6E5 */
  0x0000,   /* U+1F6E6 */
  0x0000,   /* U+1F6E7 */
  0x0000,   /* U+1F6E8 */
  0x0000,   /* U+1F6E9 */
  0x0000,   /* U+1F6EA */
  0x0000,   /* U+1F6EB */
  0x0000,   /* U+1F6EC */
  0x0000,   /* U+1F6ED */
  0x0000,   /* U+1F6EE */
  0x0000,   /* U+1F6EF */
  0x0000,   /* U+1F6F0 */
  0x0000,   /* U+1F6F1 */
  0x0000,   /* U+1F6F2 */
  0x0000,   /* U+1F6F3 */
  0x0000,   /* U+1F6F4 */
  0x0000,   /* U+1F6F5 */
  0x0000,   /* U+1F6F6 */
  0x0000,   /* U+1F6F7 */
  0x0000,   /* U+1F6F8 */
  0x0000,   /* U+1F6F9 */
  0x0000,   /* U+1F6FA */
  0x0000,   /* U+1F6FB */
  0x0000,   /* U+1F6FC */
  0x0000,   /* U+1F6FD */
  0x0000,   /* U+1F6FE */
  0x0000,   /* U+1F6FF */
};

uint16 uca900_p1F7[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x16E2,   /* U+1F700 */
  0x16E3,   /* U+1F701 */
  0x16E4,   /* U+1F702 */
  0x16E5,   /* U+1F703 */
  0x16E6,   /* U+1F704 */
  0x16E7,   /* U+1F705 */
  0x16E8,   /* U+1F706 */
  0x16E9,   /* U+1F707 */
  0x16EA,   /* U+1F708 */
  0x16EB,   /* U+1F709 */
  0x16EC,   /* U+1F70A */
  0x16ED,   /* U+1F70B */
  0x16EE,   /* U+1F70C */
  0x16EF,   /* U+1F70D */
  0x16F0,   /* U+1F70E */
  0x16F1,   /* U+1F70F */
  0x16F2,   /* U+1F710 */
  0x16F3,   /* U+1F711 */
  0x16F4,   /* U+1F712 */
  0x16F5,   /* U+1F713 */
  0x16F6,   /* U+1F714 */
  0x16F7,   /* U+1F715 */
  0x16F8,   /* U+1F716 */
  0x16F9,   /* U+1F717 */
  0x16FA,   /* U+1F718 */
  0x16FB,   /* U+1F719 */
  0x16FC,   /* U+1F71A */
  0x16FD,   /* U+1F71B */
  0x16FE,   /* U+1F71C */
  0x16FF,   /* U+1F71D */
  0x1700,   /* U+1F71E */
  0x1701,   /* U+1F71F */
  0x1702,   /* U+1F720 */
  0x1703,   /* U+1F721 */
  0x1704,   /* U+1F722 */
  0x1705,   /* U+1F723 */
  0x1706,   /* U+1F724 */
  0x1707,   /* U+1F725 */
  0x1708,   /* U+1F726 */
  0x1709,   /* U+1F727 */
  0x170A,   /* U+1F728 */
  0x170B,   /* U+1F729 */
  0x170C,   /* U+1F72A */
  0x170D,   /* U+1F72B */
  0x170E,   /* U+1F72C */
  0x170F,   /* U+1F72D */
  0x1710,   /* U+1F72E */
  0x1711,   /* U+1F72F */
  0x1712,   /* U+1F730 */
  0x1713,   /* U+1F731 */
  0x1714,   /* U+1F732 */
  0x1715,   /* U+1F733 */
  0x1716,   /* U+1F734 */
  0x1717,   /* U+1F735 */
  0x1718,   /* U+1F736 */
  0x1719,   /* U+1F737 */
  0x171A,   /* U+1F738 */
  0x171B,   /* U+1F739 */
  0x171C,   /* U+1F73A */
  0x171D,   /* U+1F73B */
  0x171E,   /* U+1F73C */
  0x171F,   /* U+1F73D */
  0x1720,   /* U+1F73E */
  0x1721,   /* U+1F73F */
  0x1722,   /* U+1F740 */
  0x1723,   /* U+1F741 */
  0x1724,   /* U+1F742 */
  0x1725,   /* U+1F743 */
  0x1726,   /* U+1F744 */
  0x1727,   /* U+1F745 */
  0x1728,   /* U+1F746 */
  0x1729,   /* U+1F747 */
  0x172A,   /* U+1F748 */
  0x172B,   /* U+1F749 */
  0x172C,   /* U+1F74A */
  0x172D,   /* U+1F74B */
  0x172E,   /* U+1F74C */
  0x172F,   /* U+1F74D */
  0x1730,   /* U+1F74E */
  0x1731,   /* U+1F74F */
  0x1732,   /* U+1F750 */
  0x1733,   /* U+1F751 */
  0x1734,   /* U+1F752 */
  0x1735,   /* U+1F753 */
  0x1736,   /* U+1F754 */
  0x1737,   /* U+1F755 */
  0x1738,   /* U+1F756 */
  0x1739,   /* U+1F757 */
  0x173A,   /* U+1F758 */
  0x173B,   /* U+1F759 */
  0x173C,   /* U+1F75A */
  0x173D,   /* U+1F75B */
  0x173E,   /* U+1F75C */
  0x173F,   /* U+1F75D */
  0x1740,   /* U+1F75E */
  0x1741,   /* U+1F75F */
  0x1742,   /* U+1F760 */
  0x1743,   /* U+1F761 */
  0x1744,   /* U+1F762 */
  0x1745,   /* U+1F763 */
  0x1746,   /* U+1F764 */
  0x1747,   /* U+1F765 */
  0x1748,   /* U+1F766 */
  0x1749,   /* U+1F767 */
  0x174A,   /* U+1F768 */
  0x174B,   /* U+1F769 */
  0x174C,   /* U+1F76A */
  0x174D,   /* U+1F76B */
  0x174E,   /* U+1F76C */
  0x174F,   /* U+1F76D */
  0x1750,   /* U+1F76E */
  0x1751,   /* U+1F76F */
  0x1752,   /* U+1F770 */
  0x1753,   /* U+1F771 */
  0x1754,   /* U+1F772 */
  0x1755,   /* U+1F773 */
  0xFBC3,   /* U+1F774 */
  0xFBC3,   /* U+1F775 */
  0xFBC3,   /* U+1F776 */
  0xFBC3,   /* U+1F777 */
  0xFBC3,   /* U+1F778 */
  0xFBC3,   /* U+1F779 */
  0xFBC3,   /* U+1F77A */
  0xFBC3,   /* U+1F77B */
  0xFBC3,   /* U+1F77C */
  0xFBC3,   /* U+1F77D */
  0xFBC3,   /* U+1F77E */
  0xFBC3,   /* U+1F77F */
  0x1756,   /* U+1F780 */
  0x1757,   /* U+1F781 */
  0x1758,   /* U+1F782 */
  0x1759,   /* U+1F783 */
  0x175A,   /* U+1F784 */
  0x175B,   /* U+1F785 */
  0x175C,   /* U+1F786 */
  0x175D,   /* U+1F787 */
  0x175E,   /* U+1F788 */
  0x175F,   /* U+1F789 */
  0x1760,   /* U+1F78A */
  0x1761,   /* U+1F78B */
  0x1762,   /* U+1F78C */
  0x1763,   /* U+1F78D */
  0x1764,   /* U+1F78E */
  0x1765,   /* U+1F78F */
  0x1766,   /* U+1F790 */
  0x1767,   /* U+1F791 */
  0x1768,   /* U+1F792 */
  0x1769,   /* U+1F793 */
  0x176A,   /* U+1F794 */
  0x176B,   /* U+1F795 */
  0x176C,   /* U+1F796 */
  0x176D,   /* U+1F797 */
  0x176E,   /* U+1F798 */
  0x176F,   /* U+1F799 */
  0x1770,   /* U+1F79A */
  0x1771,   /* U+1F79B */
  0x1772,   /* U+1F79C */
  0x1773,   /* U+1F79D */
  0x1774,   /* U+1F79E */
  0x1775,   /* U+1F79F */
  0x1776,   /* U+1F7A0 */
  0x1777,   /* U+1F7A1 */
  0x1778,   /* U+1F7A2 */
  0x1779,   /* U+1F7A3 */
  0x177A,   /* U+1F7A4 */
  0x177B,   /* U+1F7A5 */
  0x177C,   /* U+1F7A6 */
  0x177D,   /* U+1F7A7 */
  0x177E,   /* U+1F7A8 */
  0x177F,   /* U+1F7A9 */
  0x1780,   /* U+1F7AA */
  0x1781,   /* U+1F7AB */
  0x1782,   /* U+1F7AC */
  0x1783,   /* U+1F7AD */
  0x1784,   /* U+1F7AE */
  0x1785,   /* U+1F7AF */
  0x1786,   /* U+1F7B0 */
  0x1787,   /* U+1F7B1 */
  0x1788,   /* U+1F7B2 */
  0x1789,   /* U+1F7B3 */
  0x178A,   /* U+1F7B4 */
  0x178B,   /* U+1F7B5 */
  0x178C,   /* U+1F7B6 */
  0x178D,   /* U+1F7B7 */
  0x178E,   /* U+1F7B8 */
  0x178F,   /* U+1F7B9 */
  0x1790,   /* U+1F7BA */
  0x1791,   /* U+1F7BB */
  0x1792,   /* U+1F7BC */
  0x1793,   /* U+1F7BD */
  0x1794,   /* U+1F7BE */
  0x1795,   /* U+1F7BF */
  0x1796,   /* U+1F7C0 */
  0x1797,   /* U+1F7C1 */
  0x1798,   /* U+1F7C2 */
  0x1799,   /* U+1F7C3 */
  0x179A,   /* U+1F7C4 */
  0x179B,   /* U+1F7C5 */
  0x179C,   /* U+1F7C6 */
  0x179D,   /* U+1F7C7 */
  0x179E,   /* U+1F7C8 */
  0x179F,   /* U+1F7C9 */
  0x17A0,   /* U+1F7CA */
  0x17A1,   /* U+1F7CB */
  0x17A2,   /* U+1F7CC */
  0x17A3,   /* U+1F7CD */
  0x17A4,   /* U+1F7CE */
  0x17A5,   /* U+1F7CF */
  0x17A6,   /* U+1F7D0 */
  0x17A7,   /* U+1F7D1 */
  0x17A8,   /* U+1F7D2 */
  0x17A9,   /* U+1F7D3 */
  0x17AA,   /* U+1F7D4 */
  0xFBC3,   /* U+1F7D5 */
  0xFBC3,   /* U+1F7D6 */
  0xFBC3,   /* U+1F7D7 */
  0xFBC3,   /* U+1F7D8 */
  0xFBC3,   /* U+1F7D9 */
  0xFBC3,   /* U+1F7DA */
  0xFBC3,   /* U+1F7DB */
  0xFBC3,   /* U+1F7DC */
  0xFBC3,   /* U+1F7DD */
  0xFBC3,   /* U+1F7DE */
  0xFBC3,   /* U+1F7DF */
  0xFBC3,   /* U+1F7E0 */
  0xFBC3,   /* U+1F7E1 */
  0xFBC3,   /* U+1F7E2 */
  0xFBC3,   /* U+1F7E3 */
  0xFBC3,   /* U+1F7E4 */
  0xFBC3,   /* U+1F7E5 */
  0xFBC3,   /* U+1F7E6 */
  0xFBC3,   /* U+1F7E7 */
  0xFBC3,   /* U+1F7E8 */
  0xFBC3,   /* U+1F7E9 */
  0xFBC3,   /* U+1F7EA */
  0xFBC3,   /* U+1F7EB */
  0xFBC3,   /* U+1F7EC */
  0xFBC3,   /* U+1F7ED */
  0xFBC3,   /* U+1F7EE */
  0xFBC3,   /* U+1F7EF */
  0xFBC3,   /* U+1F7F0 */
  0xFBC3,   /* U+1F7F1 */
  0xFBC3,   /* U+1F7F2 */
  0xFBC3,   /* U+1F7F3 */
  0xFBC3,   /* U+1F7F4 */
  0xFBC3,   /* U+1F7F5 */
  0xFBC3,   /* U+1F7F6 */
  0xFBC3,   /* U+1F7F7 */
  0xFBC3,   /* U+1F7F8 */
  0xFBC3,   /* U+1F7F9 */
  0xFBC3,   /* U+1F7FA */
  0xFBC3,   /* U+1F7FB */
  0xFBC3,   /* U+1F7FC */
  0xFBC3,   /* U+1F7FD */
  0xFBC3,   /* U+1F7FE */
  0xFBC3,   /* U+1F7FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F700 */
  0x0020,   /* U+1F701 */
  0x0020,   /* U+1F702 */
  0x0020,   /* U+1F703 */
  0x0020,   /* U+1F704 */
  0x0020,   /* U+1F705 */
  0x0020,   /* U+1F706 */
  0x0020,   /* U+1F707 */
  0x0020,   /* U+1F708 */
  0x0020,   /* U+1F709 */
  0x0020,   /* U+1F70A */
  0x0020,   /* U+1F70B */
  0x0020,   /* U+1F70C */
  0x0020,   /* U+1F70D */
  0x0020,   /* U+1F70E */
  0x0020,   /* U+1F70F */
  0x0020,   /* U+1F710 */
  0x0020,   /* U+1F711 */
  0x0020,   /* U+1F712 */
  0x0020,   /* U+1F713 */
  0x0020,   /* U+1F714 */
  0x0020,   /* U+1F715 */
  0x0020,   /* U+1F716 */
  0x0020,   /* U+1F717 */
  0x0020,   /* U+1F718 */
  0x0020,   /* U+1F719 */
  0x0020,   /* U+1F71A */
  0x0020,   /* U+1F71B */
  0x0020,   /* U+1F71C */
  0x0020,   /* U+1F71D */
  0x0020,   /* U+1F71E */
  0x0020,   /* U+1F71F */
  0x0020,   /* U+1F720 */
  0x0020,   /* U+1F721 */
  0x0020,   /* U+1F722 */
  0x0020,   /* U+1F723 */
  0x0020,   /* U+1F724 */
  0x0020,   /* U+1F725 */
  0x0020,   /* U+1F726 */
  0x0020,   /* U+1F727 */
  0x0020,   /* U+1F728 */
  0x0020,   /* U+1F729 */
  0x0020,   /* U+1F72A */
  0x0020,   /* U+1F72B */
  0x0020,   /* U+1F72C */
  0x0020,   /* U+1F72D */
  0x0020,   /* U+1F72E */
  0x0020,   /* U+1F72F */
  0x0020,   /* U+1F730 */
  0x0020,   /* U+1F731 */
  0x0020,   /* U+1F732 */
  0x0020,   /* U+1F733 */
  0x0020,   /* U+1F734 */
  0x0020,   /* U+1F735 */
  0x0020,   /* U+1F736 */
  0x0020,   /* U+1F737 */
  0x0020,   /* U+1F738 */
  0x0020,   /* U+1F739 */
  0x0020,   /* U+1F73A */
  0x0020,   /* U+1F73B */
  0x0020,   /* U+1F73C */
  0x0020,   /* U+1F73D */
  0x0020,   /* U+1F73E */
  0x0020,   /* U+1F73F */
  0x0020,   /* U+1F740 */
  0x0020,   /* U+1F741 */
  0x0020,   /* U+1F742 */
  0x0020,   /* U+1F743 */
  0x0020,   /* U+1F744 */
  0x0020,   /* U+1F745 */
  0x0020,   /* U+1F746 */
  0x0020,   /* U+1F747 */
  0x0020,   /* U+1F748 */
  0x0020,   /* U+1F749 */
  0x0020,   /* U+1F74A */
  0x0020,   /* U+1F74B */
  0x0020,   /* U+1F74C */
  0x0020,   /* U+1F74D */
  0x0020,   /* U+1F74E */
  0x0020,   /* U+1F74F */
  0x0020,   /* U+1F750 */
  0x0020,   /* U+1F751 */
  0x0020,   /* U+1F752 */
  0x0020,   /* U+1F753 */
  0x0020,   /* U+1F754 */
  0x0020,   /* U+1F755 */
  0x0020,   /* U+1F756 */
  0x0020,   /* U+1F757 */
  0x0020,   /* U+1F758 */
  0x0020,   /* U+1F759 */
  0x0020,   /* U+1F75A */
  0x0020,   /* U+1F75B */
  0x0020,   /* U+1F75C */
  0x0020,   /* U+1F75D */
  0x0020,   /* U+1F75E */
  0x0020,   /* U+1F75F */
  0x0020,   /* U+1F760 */
  0x0020,   /* U+1F761 */
  0x0020,   /* U+1F762 */
  0x0020,   /* U+1F763 */
  0x0020,   /* U+1F764 */
  0x0020,   /* U+1F765 */
  0x0020,   /* U+1F766 */
  0x0020,   /* U+1F767 */
  0x0020,   /* U+1F768 */
  0x0020,   /* U+1F769 */
  0x0020,   /* U+1F76A */
  0x0020,   /* U+1F76B */
  0x0020,   /* U+1F76C */
  0x0020,   /* U+1F76D */
  0x0020,   /* U+1F76E */
  0x0020,   /* U+1F76F */
  0x0020,   /* U+1F770 */
  0x0020,   /* U+1F771 */
  0x0020,   /* U+1F772 */
  0x0020,   /* U+1F773 */
  0x0020,   /* U+1F774 */
  0x0020,   /* U+1F775 */
  0x0020,   /* U+1F776 */
  0x0020,   /* U+1F777 */
  0x0020,   /* U+1F778 */
  0x0020,   /* U+1F779 */
  0x0020,   /* U+1F77A */
  0x0020,   /* U+1F77B */
  0x0020,   /* U+1F77C */
  0x0020,   /* U+1F77D */
  0x0020,   /* U+1F77E */
  0x0020,   /* U+1F77F */
  0x0020,   /* U+1F780 */
  0x0020,   /* U+1F781 */
  0x0020,   /* U+1F782 */
  0x0020,   /* U+1F783 */
  0x0020,   /* U+1F784 */
  0x0020,   /* U+1F785 */
  0x0020,   /* U+1F786 */
  0x0020,   /* U+1F787 */
  0x0020,   /* U+1F788 */
  0x0020,   /* U+1F789 */
  0x0020,   /* U+1F78A */
  0x0020,   /* U+1F78B */
  0x0020,   /* U+1F78C */
  0x0020,   /* U+1F78D */
  0x0020,   /* U+1F78E */
  0x0020,   /* U+1F78F */
  0x0020,   /* U+1F790 */
  0x0020,   /* U+1F791 */
  0x0020,   /* U+1F792 */
  0x0020,   /* U+1F793 */
  0x0020,   /* U+1F794 */
  0x0020,   /* U+1F795 */
  0x0020,   /* U+1F796 */
  0x0020,   /* U+1F797 */
  0x0020,   /* U+1F798 */
  0x0020,   /* U+1F799 */
  0x0020,   /* U+1F79A */
  0x0020,   /* U+1F79B */
  0x0020,   /* U+1F79C */
  0x0020,   /* U+1F79D */
  0x0020,   /* U+1F79E */
  0x0020,   /* U+1F79F */
  0x0020,   /* U+1F7A0 */
  0x0020,   /* U+1F7A1 */
  0x0020,   /* U+1F7A2 */
  0x0020,   /* U+1F7A3 */
  0x0020,   /* U+1F7A4 */
  0x0020,   /* U+1F7A5 */
  0x0020,   /* U+1F7A6 */
  0x0020,   /* U+1F7A7 */
  0x0020,   /* U+1F7A8 */
  0x0020,   /* U+1F7A9 */
  0x0020,   /* U+1F7AA */
  0x0020,   /* U+1F7AB */
  0x0020,   /* U+1F7AC */
  0x0020,   /* U+1F7AD */
  0x0020,   /* U+1F7AE */
  0x0020,   /* U+1F7AF */
  0x0020,   /* U+1F7B0 */
  0x0020,   /* U+1F7B1 */
  0x0020,   /* U+1F7B2 */
  0x0020,   /* U+1F7B3 */
  0x0020,   /* U+1F7B4 */
  0x0020,   /* U+1F7B5 */
  0x0020,   /* U+1F7B6 */
  0x0020,   /* U+1F7B7 */
  0x0020,   /* U+1F7B8 */
  0x0020,   /* U+1F7B9 */
  0x0020,   /* U+1F7BA */
  0x0020,   /* U+1F7BB */
  0x0020,   /* U+1F7BC */
  0x0020,   /* U+1F7BD */
  0x0020,   /* U+1F7BE */
  0x0020,   /* U+1F7BF */
  0x0020,   /* U+1F7C0 */
  0x0020,   /* U+1F7C1 */
  0x0020,   /* U+1F7C2 */
  0x0020,   /* U+1F7C3 */
  0x0020,   /* U+1F7C4 */
  0x0020,   /* U+1F7C5 */
  0x0020,   /* U+1F7C6 */
  0x0020,   /* U+1F7C7 */
  0x0020,   /* U+1F7C8 */
  0x0020,   /* U+1F7C9 */
  0x0020,   /* U+1F7CA */
  0x0020,   /* U+1F7CB */
  0x0020,   /* U+1F7CC */
  0x0020,   /* U+1F7CD */
  0x0020,   /* U+1F7CE */
  0x0020,   /* U+1F7CF */
  0x0020,   /* U+1F7D0 */
  0x0020,   /* U+1F7D1 */
  0x0020,   /* U+1F7D2 */
  0x0020,   /* U+1F7D3 */
  0x0020,   /* U+1F7D4 */
  0x0020,   /* U+1F7D5 */
  0x0020,   /* U+1F7D6 */
  0x0020,   /* U+1F7D7 */
  0x0020,   /* U+1F7D8 */
  0x0020,   /* U+1F7D9 */
  0x0020,   /* U+1F7DA */
  0x0020,   /* U+1F7DB */
  0x0020,   /* U+1F7DC */
  0x0020,   /* U+1F7DD */
  0x0020,   /* U+1F7DE */
  0x0020,   /* U+1F7DF */
  0x0020,   /* U+1F7E0 */
  0x0020,   /* U+1F7E1 */
  0x0020,   /* U+1F7E2 */
  0x0020,   /* U+1F7E3 */
  0x0020,   /* U+1F7E4 */
  0x0020,   /* U+1F7E5 */
  0x0020,   /* U+1F7E6 */
  0x0020,   /* U+1F7E7 */
  0x0020,   /* U+1F7E8 */
  0x0020,   /* U+1F7E9 */
  0x0020,   /* U+1F7EA */
  0x0020,   /* U+1F7EB */
  0x0020,   /* U+1F7EC */
  0x0020,   /* U+1F7ED */
  0x0020,   /* U+1F7EE */
  0x0020,   /* U+1F7EF */
  0x0020,   /* U+1F7F0 */
  0x0020,   /* U+1F7F1 */
  0x0020,   /* U+1F7F2 */
  0x0020,   /* U+1F7F3 */
  0x0020,   /* U+1F7F4 */
  0x0020,   /* U+1F7F5 */
  0x0020,   /* U+1F7F6 */
  0x0020,   /* U+1F7F7 */
  0x0020,   /* U+1F7F8 */
  0x0020,   /* U+1F7F9 */
  0x0020,   /* U+1F7FA */
  0x0020,   /* U+1F7FB */
  0x0020,   /* U+1F7FC */
  0x0020,   /* U+1F7FD */
  0x0020,   /* U+1F7FE */
  0x0020,   /* U+1F7FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F700 */
  0x0002,   /* U+1F701 */
  0x0002,   /* U+1F702 */
  0x0002,   /* U+1F703 */
  0x0002,   /* U+1F704 */
  0x0002,   /* U+1F705 */
  0x0002,   /* U+1F706 */
  0x0002,   /* U+1F707 */
  0x0002,   /* U+1F708 */
  0x0002,   /* U+1F709 */
  0x0002,   /* U+1F70A */
  0x0002,   /* U+1F70B */
  0x0002,   /* U+1F70C */
  0x0002,   /* U+1F70D */
  0x0002,   /* U+1F70E */
  0x0002,   /* U+1F70F */
  0x0002,   /* U+1F710 */
  0x0002,   /* U+1F711 */
  0x0002,   /* U+1F712 */
  0x0002,   /* U+1F713 */
  0x0002,   /* U+1F714 */
  0x0002,   /* U+1F715 */
  0x0002,   /* U+1F716 */
  0x0002,   /* U+1F717 */
  0x0002,   /* U+1F718 */
  0x0002,   /* U+1F719 */
  0x0002,   /* U+1F71A */
  0x0002,   /* U+1F71B */
  0x0002,   /* U+1F71C */
  0x0002,   /* U+1F71D */
  0x0002,   /* U+1F71E */
  0x0002,   /* U+1F71F */
  0x0002,   /* U+1F720 */
  0x0002,   /* U+1F721 */
  0x0002,   /* U+1F722 */
  0x0002,   /* U+1F723 */
  0x0002,   /* U+1F724 */
  0x0002,   /* U+1F725 */
  0x0002,   /* U+1F726 */
  0x0002,   /* U+1F727 */
  0x0002,   /* U+1F728 */
  0x0002,   /* U+1F729 */
  0x0002,   /* U+1F72A */
  0x0002,   /* U+1F72B */
  0x0002,   /* U+1F72C */
  0x0002,   /* U+1F72D */
  0x0002,   /* U+1F72E */
  0x0002,   /* U+1F72F */
  0x0002,   /* U+1F730 */
  0x0002,   /* U+1F731 */
  0x0002,   /* U+1F732 */
  0x0002,   /* U+1F733 */
  0x0002,   /* U+1F734 */
  0x0002,   /* U+1F735 */
  0x0002,   /* U+1F736 */
  0x0002,   /* U+1F737 */
  0x0002,   /* U+1F738 */
  0x0002,   /* U+1F739 */
  0x0002,   /* U+1F73A */
  0x0002,   /* U+1F73B */
  0x0002,   /* U+1F73C */
  0x0002,   /* U+1F73D */
  0x0002,   /* U+1F73E */
  0x0002,   /* U+1F73F */
  0x0002,   /* U+1F740 */
  0x0002,   /* U+1F741 */
  0x0002,   /* U+1F742 */
  0x0002,   /* U+1F743 */
  0x0002,   /* U+1F744 */
  0x0002,   /* U+1F745 */
  0x0002,   /* U+1F746 */
  0x0002,   /* U+1F747 */
  0x0002,   /* U+1F748 */
  0x0002,   /* U+1F749 */
  0x0002,   /* U+1F74A */
  0x0002,   /* U+1F74B */
  0x0002,   /* U+1F74C */
  0x0002,   /* U+1F74D */
  0x0002,   /* U+1F74E */
  0x0002,   /* U+1F74F */
  0x0002,   /* U+1F750 */
  0x0002,   /* U+1F751 */
  0x0002,   /* U+1F752 */
  0x0002,   /* U+1F753 */
  0x0002,   /* U+1F754 */
  0x0002,   /* U+1F755 */
  0x0002,   /* U+1F756 */
  0x0002,   /* U+1F757 */
  0x0002,   /* U+1F758 */
  0x0002,   /* U+1F759 */
  0x0002,   /* U+1F75A */
  0x0002,   /* U+1F75B */
  0x0002,   /* U+1F75C */
  0x0002,   /* U+1F75D */
  0x0002,   /* U+1F75E */
  0x0002,   /* U+1F75F */
  0x0002,   /* U+1F760 */
  0x0002,   /* U+1F761 */
  0x0002,   /* U+1F762 */
  0x0002,   /* U+1F763 */
  0x0002,   /* U+1F764 */
  0x0002,   /* U+1F765 */
  0x0002,   /* U+1F766 */
  0x0002,   /* U+1F767 */
  0x0002,   /* U+1F768 */
  0x0002,   /* U+1F769 */
  0x0002,   /* U+1F76A */
  0x0002,   /* U+1F76B */
  0x0002,   /* U+1F76C */
  0x0002,   /* U+1F76D */
  0x0002,   /* U+1F76E */
  0x0002,   /* U+1F76F */
  0x0002,   /* U+1F770 */
  0x0002,   /* U+1F771 */
  0x0002,   /* U+1F772 */
  0x0002,   /* U+1F773 */
  0x0002,   /* U+1F774 */
  0x0002,   /* U+1F775 */
  0x0002,   /* U+1F776 */
  0x0002,   /* U+1F777 */
  0x0002,   /* U+1F778 */
  0x0002,   /* U+1F779 */
  0x0002,   /* U+1F77A */
  0x0002,   /* U+1F77B */
  0x0002,   /* U+1F77C */
  0x0002,   /* U+1F77D */
  0x0002,   /* U+1F77E */
  0x0002,   /* U+1F77F */
  0x0002,   /* U+1F780 */
  0x0002,   /* U+1F781 */
  0x0002,   /* U+1F782 */
  0x0002,   /* U+1F783 */
  0x0002,   /* U+1F784 */
  0x0002,   /* U+1F785 */
  0x0002,   /* U+1F786 */
  0x0002,   /* U+1F787 */
  0x0002,   /* U+1F788 */
  0x0002,   /* U+1F789 */
  0x0002,   /* U+1F78A */
  0x0002,   /* U+1F78B */
  0x0002,   /* U+1F78C */
  0x0002,   /* U+1F78D */
  0x0002,   /* U+1F78E */
  0x0002,   /* U+1F78F */
  0x0002,   /* U+1F790 */
  0x0002,   /* U+1F791 */
  0x0002,   /* U+1F792 */
  0x0002,   /* U+1F793 */
  0x0002,   /* U+1F794 */
  0x0002,   /* U+1F795 */
  0x0002,   /* U+1F796 */
  0x0002,   /* U+1F797 */
  0x0002,   /* U+1F798 */
  0x0002,   /* U+1F799 */
  0x0002,   /* U+1F79A */
  0x0002,   /* U+1F79B */
  0x0002,   /* U+1F79C */
  0x0002,   /* U+1F79D */
  0x0002,   /* U+1F79E */
  0x0002,   /* U+1F79F */
  0x0002,   /* U+1F7A0 */
  0x0002,   /* U+1F7A1 */
  0x0002,   /* U+1F7A2 */
  0x0002,   /* U+1F7A3 */
  0x0002,   /* U+1F7A4 */
  0x0002,   /* U+1F7A5 */
  0x0002,   /* U+1F7A6 */
  0x0002,   /* U+1F7A7 */
  0x0002,   /* U+1F7A8 */
  0x0002,   /* U+1F7A9 */
  0x0002,   /* U+1F7AA */
  0x0002,   /* U+1F7AB */
  0x0002,   /* U+1F7AC */
  0x0002,   /* U+1F7AD */
  0x0002,   /* U+1F7AE */
  0x0002,   /* U+1F7AF */
  0x0002,   /* U+1F7B0 */
  0x0002,   /* U+1F7B1 */
  0x0002,   /* U+1F7B2 */
  0x0002,   /* U+1F7B3 */
  0x0002,   /* U+1F7B4 */
  0x0002,   /* U+1F7B5 */
  0x0002,   /* U+1F7B6 */
  0x0002,   /* U+1F7B7 */
  0x0002,   /* U+1F7B8 */
  0x0002,   /* U+1F7B9 */
  0x0002,   /* U+1F7BA */
  0x0002,   /* U+1F7BB */
  0x0002,   /* U+1F7BC */
  0x0002,   /* U+1F7BD */
  0x0002,   /* U+1F7BE */
  0x0002,   /* U+1F7BF */
  0x0002,   /* U+1F7C0 */
  0x0002,   /* U+1F7C1 */
  0x0002,   /* U+1F7C2 */
  0x0002,   /* U+1F7C3 */
  0x0002,   /* U+1F7C4 */
  0x0002,   /* U+1F7C5 */
  0x0002,   /* U+1F7C6 */
  0x0002,   /* U+1F7C7 */
  0x0002,   /* U+1F7C8 */
  0x0002,   /* U+1F7C9 */
  0x0002,   /* U+1F7CA */
  0x0002,   /* U+1F7CB */
  0x0002,   /* U+1F7CC */
  0x0002,   /* U+1F7CD */
  0x0002,   /* U+1F7CE */
  0x0002,   /* U+1F7CF */
  0x0002,   /* U+1F7D0 */
  0x0002,   /* U+1F7D1 */
  0x0002,   /* U+1F7D2 */
  0x0002,   /* U+1F7D3 */
  0x0002,   /* U+1F7D4 */
  0x0002,   /* U+1F7D5 */
  0x0002,   /* U+1F7D6 */
  0x0002,   /* U+1F7D7 */
  0x0002,   /* U+1F7D8 */
  0x0002,   /* U+1F7D9 */
  0x0002,   /* U+1F7DA */
  0x0002,   /* U+1F7DB */
  0x0002,   /* U+1F7DC */
  0x0002,   /* U+1F7DD */
  0x0002,   /* U+1F7DE */
  0x0002,   /* U+1F7DF */
  0x0002,   /* U+1F7E0 */
  0x0002,   /* U+1F7E1 */
  0x0002,   /* U+1F7E2 */
  0x0002,   /* U+1F7E3 */
  0x0002,   /* U+1F7E4 */
  0x0002,   /* U+1F7E5 */
  0x0002,   /* U+1F7E6 */
  0x0002,   /* U+1F7E7 */
  0x0002,   /* U+1F7E8 */
  0x0002,   /* U+1F7E9 */
  0x0002,   /* U+1F7EA */
  0x0002,   /* U+1F7EB */
  0x0002,   /* U+1F7EC */
  0x0002,   /* U+1F7ED */
  0x0002,   /* U+1F7EE */
  0x0002,   /* U+1F7EF */
  0x0002,   /* U+1F7F0 */
  0x0002,   /* U+1F7F1 */
  0x0002,   /* U+1F7F2 */
  0x0002,   /* U+1F7F3 */
  0x0002,   /* U+1F7F4 */
  0x0002,   /* U+1F7F5 */
  0x0002,   /* U+1F7F6 */
  0x0002,   /* U+1F7F7 */
  0x0002,   /* U+1F7F8 */
  0x0002,   /* U+1F7F9 */
  0x0002,   /* U+1F7FA */
  0x0002,   /* U+1F7FB */
  0x0002,   /* U+1F7FC */
  0x0002,   /* U+1F7FD */
  0x0002,   /* U+1F7FE */
  0x0002,   /* U+1F7FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1F700 */
  0x0000,   /* U+1F701 */
  0x0000,   /* U+1F702 */
  0x0000,   /* U+1F703 */
  0x0000,   /* U+1F704 */
  0x0000,   /* U+1F705 */
  0x0000,   /* U+1F706 */
  0x0000,   /* U+1F707 */
  0x0000,   /* U+1F708 */
  0x0000,   /* U+1F709 */
  0x0000,   /* U+1F70A */
  0x0000,   /* U+1F70B */
  0x0000,   /* U+1F70C */
  0x0000,   /* U+1F70D */
  0x0000,   /* U+1F70E */
  0x0000,   /* U+1F70F */
  0x0000,   /* U+1F710 */
  0x0000,   /* U+1F711 */
  0x0000,   /* U+1F712 */
  0x0000,   /* U+1F713 */
  0x0000,   /* U+1F714 */
  0x0000,   /* U+1F715 */
  0x0000,   /* U+1F716 */
  0x0000,   /* U+1F717 */
  0x0000,   /* U+1F718 */
  0x0000,   /* U+1F719 */
  0x0000,   /* U+1F71A */
  0x0000,   /* U+1F71B */
  0x0000,   /* U+1F71C */
  0x0000,   /* U+1F71D */
  0x0000,   /* U+1F71E */
  0x0000,   /* U+1F71F */
  0x0000,   /* U+1F720 */
  0x0000,   /* U+1F721 */
  0x0000,   /* U+1F722 */
  0x0000,   /* U+1F723 */
  0x0000,   /* U+1F724 */
  0x0000,   /* U+1F725 */
  0x0000,   /* U+1F726 */
  0x0000,   /* U+1F727 */
  0x0000,   /* U+1F728 */
  0x0000,   /* U+1F729 */
  0x0000,   /* U+1F72A */
  0x0000,   /* U+1F72B */
  0x0000,   /* U+1F72C */
  0x0000,   /* U+1F72D */
  0x0000,   /* U+1F72E */
  0x0000,   /* U+1F72F */
  0x0000,   /* U+1F730 */
  0x0000,   /* U+1F731 */
  0x0000,   /* U+1F732 */
  0x0000,   /* U+1F733 */
  0x0000,   /* U+1F734 */
  0x0000,   /* U+1F735 */
  0x0000,   /* U+1F736 */
  0x0000,   /* U+1F737 */
  0x0000,   /* U+1F738 */
  0x0000,   /* U+1F739 */
  0x0000,   /* U+1F73A */
  0x0000,   /* U+1F73B */
  0x0000,   /* U+1F73C */
  0x0000,   /* U+1F73D */
  0x0000,   /* U+1F73E */
  0x0000,   /* U+1F73F */
  0x0000,   /* U+1F740 */
  0x0000,   /* U+1F741 */
  0x0000,   /* U+1F742 */
  0x0000,   /* U+1F743 */
  0x0000,   /* U+1F744 */
  0x0000,   /* U+1F745 */
  0x0000,   /* U+1F746 */
  0x0000,   /* U+1F747 */
  0x0000,   /* U+1F748 */
  0x0000,   /* U+1F749 */
  0x0000,   /* U+1F74A */
  0x0000,   /* U+1F74B */
  0x0000,   /* U+1F74C */
  0x0000,   /* U+1F74D */
  0x0000,   /* U+1F74E */
  0x0000,   /* U+1F74F */
  0x0000,   /* U+1F750 */
  0x0000,   /* U+1F751 */
  0x0000,   /* U+1F752 */
  0x0000,   /* U+1F753 */
  0x0000,   /* U+1F754 */
  0x0000,   /* U+1F755 */
  0x0000,   /* U+1F756 */
  0x0000,   /* U+1F757 */
  0x0000,   /* U+1F758 */
  0x0000,   /* U+1F759 */
  0x0000,   /* U+1F75A */
  0x0000,   /* U+1F75B */
  0x0000,   /* U+1F75C */
  0x0000,   /* U+1F75D */
  0x0000,   /* U+1F75E */
  0x0000,   /* U+1F75F */
  0x0000,   /* U+1F760 */
  0x0000,   /* U+1F761 */
  0x0000,   /* U+1F762 */
  0x0000,   /* U+1F763 */
  0x0000,   /* U+1F764 */
  0x0000,   /* U+1F765 */
  0x0000,   /* U+1F766 */
  0x0000,   /* U+1F767 */
  0x0000,   /* U+1F768 */
  0x0000,   /* U+1F769 */
  0x0000,   /* U+1F76A */
  0x0000,   /* U+1F76B */
  0x0000,   /* U+1F76C */
  0x0000,   /* U+1F76D */
  0x0000,   /* U+1F76E */
  0x0000,   /* U+1F76F */
  0x0000,   /* U+1F770 */
  0x0000,   /* U+1F771 */
  0x0000,   /* U+1F772 */
  0x0000,   /* U+1F773 */
  0xF774,   /* U+1F774 */
  0xF775,   /* U+1F775 */
  0xF776,   /* U+1F776 */
  0xF777,   /* U+1F777 */
  0xF778,   /* U+1F778 */
  0xF779,   /* U+1F779 */
  0xF77A,   /* U+1F77A */
  0xF77B,   /* U+1F77B */
  0xF77C,   /* U+1F77C */
  0xF77D,   /* U+1F77D */
  0xF77E,   /* U+1F77E */
  0xF77F,   /* U+1F77F */
  0x0000,   /* U+1F780 */
  0x0000,   /* U+1F781 */
  0x0000,   /* U+1F782 */
  0x0000,   /* U+1F783 */
  0x0000,   /* U+1F784 */
  0x0000,   /* U+1F785 */
  0x0000,   /* U+1F786 */
  0x0000,   /* U+1F787 */
  0x0000,   /* U+1F788 */
  0x0000,   /* U+1F789 */
  0x0000,   /* U+1F78A */
  0x0000,   /* U+1F78B */
  0x0000,   /* U+1F78C */
  0x0000,   /* U+1F78D */
  0x0000,   /* U+1F78E */
  0x0000,   /* U+1F78F */
  0x0000,   /* U+1F790 */
  0x0000,   /* U+1F791 */
  0x0000,   /* U+1F792 */
  0x0000,   /* U+1F793 */
  0x0000,   /* U+1F794 */
  0x0000,   /* U+1F795 */
  0x0000,   /* U+1F796 */
  0x0000,   /* U+1F797 */
  0x0000,   /* U+1F798 */
  0x0000,   /* U+1F799 */
  0x0000,   /* U+1F79A */
  0x0000,   /* U+1F79B */
  0x0000,   /* U+1F79C */
  0x0000,   /* U+1F79D */
  0x0000,   /* U+1F79E */
  0x0000,   /* U+1F79F */
  0x0000,   /* U+1F7A0 */
  0x0000,   /* U+1F7A1 */
  0x0000,   /* U+1F7A2 */
  0x0000,   /* U+1F7A3 */
  0x0000,   /* U+1F7A4 */
  0x0000,   /* U+1F7A5 */
  0x0000,   /* U+1F7A6 */
  0x0000,   /* U+1F7A7 */
  0x0000,   /* U+1F7A8 */
  0x0000,   /* U+1F7A9 */
  0x0000,   /* U+1F7AA */
  0x0000,   /* U+1F7AB */
  0x0000,   /* U+1F7AC */
  0x0000,   /* U+1F7AD */
  0x0000,   /* U+1F7AE */
  0x0000,   /* U+1F7AF */
  0x0000,   /* U+1F7B0 */
  0x0000,   /* U+1F7B1 */
  0x0000,   /* U+1F7B2 */
  0x0000,   /* U+1F7B3 */
  0x0000,   /* U+1F7B4 */
  0x0000,   /* U+1F7B5 */
  0x0000,   /* U+1F7B6 */
  0x0000,   /* U+1F7B7 */
  0x0000,   /* U+1F7B8 */
  0x0000,   /* U+1F7B9 */
  0x0000,   /* U+1F7BA */
  0x0000,   /* U+1F7BB */
  0x0000,   /* U+1F7BC */
  0x0000,   /* U+1F7BD */
  0x0000,   /* U+1F7BE */
  0x0000,   /* U+1F7BF */
  0x0000,   /* U+1F7C0 */
  0x0000,   /* U+1F7C1 */
  0x0000,   /* U+1F7C2 */
  0x0000,   /* U+1F7C3 */
  0x0000,   /* U+1F7C4 */
  0x0000,   /* U+1F7C5 */
  0x0000,   /* U+1F7C6 */
  0x0000,   /* U+1F7C7 */
  0x0000,   /* U+1F7C8 */
  0x0000,   /* U+1F7C9 */
  0x0000,   /* U+1F7CA */
  0x0000,   /* U+1F7CB */
  0x0000,   /* U+1F7CC */
  0x0000,   /* U+1F7CD */
  0x0000,   /* U+1F7CE */
  0x0000,   /* U+1F7CF */
  0x0000,   /* U+1F7D0 */
  0x0000,   /* U+1F7D1 */
  0x0000,   /* U+1F7D2 */
  0x0000,   /* U+1F7D3 */
  0x0000,   /* U+1F7D4 */
  0xF7D5,   /* U+1F7D5 */
  0xF7D6,   /* U+1F7D6 */
  0xF7D7,   /* U+1F7D7 */
  0xF7D8,   /* U+1F7D8 */
  0xF7D9,   /* U+1F7D9 */
  0xF7DA,   /* U+1F7DA */
  0xF7DB,   /* U+1F7DB */
  0xF7DC,   /* U+1F7DC */
  0xF7DD,   /* U+1F7DD */
  0xF7DE,   /* U+1F7DE */
  0xF7DF,   /* U+1F7DF */
  0xF7E0,   /* U+1F7E0 */
  0xF7E1,   /* U+1F7E1 */
  0xF7E2,   /* U+1F7E2 */
  0xF7E3,   /* U+1F7E3 */
  0xF7E4,   /* U+1F7E4 */
  0xF7E5,   /* U+1F7E5 */
  0xF7E6,   /* U+1F7E6 */
  0xF7E7,   /* U+1F7E7 */
  0xF7E8,   /* U+1F7E8 */
  0xF7E9,   /* U+1F7E9 */
  0xF7EA,   /* U+1F7EA */
  0xF7EB,   /* U+1F7EB */
  0xF7EC,   /* U+1F7EC */
  0xF7ED,   /* U+1F7ED */
  0xF7EE,   /* U+1F7EE */
  0xF7EF,   /* U+1F7EF */
  0xF7F0,   /* U+1F7F0 */
  0xF7F1,   /* U+1F7F1 */
  0xF7F2,   /* U+1F7F2 */
  0xF7F3,   /* U+1F7F3 */
  0xF7F4,   /* U+1F7F4 */
  0xF7F5,   /* U+1F7F5 */
  0xF7F6,   /* U+1F7F6 */
  0xF7F7,   /* U+1F7F7 */
  0xF7F8,   /* U+1F7F8 */
  0xF7F9,   /* U+1F7F9 */
  0xF7FA,   /* U+1F7FA */
  0xF7FB,   /* U+1F7FB */
  0xF7FC,   /* U+1F7FC */
  0xF7FD,   /* U+1F7FD */
  0xF7FE,   /* U+1F7FE */
  0xF7FF,   /* U+1F7FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1F700 */
  0x0000,   /* U+1F701 */
  0x0000,   /* U+1F702 */
  0x0000,   /* U+1F703 */
  0x0000,   /* U+1F704 */
  0x0000,   /* U+1F705 */
  0x0000,   /* U+1F706 */
  0x0000,   /* U+1F707 */
  0x0000,   /* U+1F708 */
  0x0000,   /* U+1F709 */
  0x0000,   /* U+1F70A */
  0x0000,   /* U+1F70B */
  0x0000,   /* U+1F70C */
  0x0000,   /* U+1F70D */
  0x0000,   /* U+1F70E */
  0x0000,   /* U+1F70F */
  0x0000,   /* U+1F710 */
  0x0000,   /* U+1F711 */
  0x0000,   /* U+1F712 */
  0x0000,   /* U+1F713 */
  0x0000,   /* U+1F714 */
  0x0000,   /* U+1F715 */
  0x0000,   /* U+1F716 */
  0x0000,   /* U+1F717 */
  0x0000,   /* U+1F718 */
  0x0000,   /* U+1F719 */
  0x0000,   /* U+1F71A */
  0x0000,   /* U+1F71B */
  0x0000,   /* U+1F71C */
  0x0000,   /* U+1F71D */
  0x0000,   /* U+1F71E */
  0x0000,   /* U+1F71F */
  0x0000,   /* U+1F720 */
  0x0000,   /* U+1F721 */
  0x0000,   /* U+1F722 */
  0x0000,   /* U+1F723 */
  0x0000,   /* U+1F724 */
  0x0000,   /* U+1F725 */
  0x0000,   /* U+1F726 */
  0x0000,   /* U+1F727 */
  0x0000,   /* U+1F728 */
  0x0000,   /* U+1F729 */
  0x0000,   /* U+1F72A */
  0x0000,   /* U+1F72B */
  0x0000,   /* U+1F72C */
  0x0000,   /* U+1F72D */
  0x0000,   /* U+1F72E */
  0x0000,   /* U+1F72F */
  0x0000,   /* U+1F730 */
  0x0000,   /* U+1F731 */
  0x0000,   /* U+1F732 */
  0x0000,   /* U+1F733 */
  0x0000,   /* U+1F734 */
  0x0000,   /* U+1F735 */
  0x0000,   /* U+1F736 */
  0x0000,   /* U+1F737 */
  0x0000,   /* U+1F738 */
  0x0000,   /* U+1F739 */
  0x0000,   /* U+1F73A */
  0x0000,   /* U+1F73B */
  0x0000,   /* U+1F73C */
  0x0000,   /* U+1F73D */
  0x0000,   /* U+1F73E */
  0x0000,   /* U+1F73F */
  0x0000,   /* U+1F740 */
  0x0000,   /* U+1F741 */
  0x0000,   /* U+1F742 */
  0x0000,   /* U+1F743 */
  0x0000,   /* U+1F744 */
  0x0000,   /* U+1F745 */
  0x0000,   /* U+1F746 */
  0x0000,   /* U+1F747 */
  0x0000,   /* U+1F748 */
  0x0000,   /* U+1F749 */
  0x0000,   /* U+1F74A */
  0x0000,   /* U+1F74B */
  0x0000,   /* U+1F74C */
  0x0000,   /* U+1F74D */
  0x0000,   /* U+1F74E */
  0x0000,   /* U+1F74F */
  0x0000,   /* U+1F750 */
  0x0000,   /* U+1F751 */
  0x0000,   /* U+1F752 */
  0x0000,   /* U+1F753 */
  0x0000,   /* U+1F754 */
  0x0000,   /* U+1F755 */
  0x0000,   /* U+1F756 */
  0x0000,   /* U+1F757 */
  0x0000,   /* U+1F758 */
  0x0000,   /* U+1F759 */
  0x0000,   /* U+1F75A */
  0x0000,   /* U+1F75B */
  0x0000,   /* U+1F75C */
  0x0000,   /* U+1F75D */
  0x0000,   /* U+1F75E */
  0x0000,   /* U+1F75F */
  0x0000,   /* U+1F760 */
  0x0000,   /* U+1F761 */
  0x0000,   /* U+1F762 */
  0x0000,   /* U+1F763 */
  0x0000,   /* U+1F764 */
  0x0000,   /* U+1F765 */
  0x0000,   /* U+1F766 */
  0x0000,   /* U+1F767 */
  0x0000,   /* U+1F768 */
  0x0000,   /* U+1F769 */
  0x0000,   /* U+1F76A */
  0x0000,   /* U+1F76B */
  0x0000,   /* U+1F76C */
  0x0000,   /* U+1F76D */
  0x0000,   /* U+1F76E */
  0x0000,   /* U+1F76F */
  0x0000,   /* U+1F770 */
  0x0000,   /* U+1F771 */
  0x0000,   /* U+1F772 */
  0x0000,   /* U+1F773 */
  0x0000,   /* U+1F774 */
  0x0000,   /* U+1F775 */
  0x0000,   /* U+1F776 */
  0x0000,   /* U+1F777 */
  0x0000,   /* U+1F778 */
  0x0000,   /* U+1F779 */
  0x0000,   /* U+1F77A */
  0x0000,   /* U+1F77B */
  0x0000,   /* U+1F77C */
  0x0000,   /* U+1F77D */
  0x0000,   /* U+1F77E */
  0x0000,   /* U+1F77F */
  0x0000,   /* U+1F780 */
  0x0000,   /* U+1F781 */
  0x0000,   /* U+1F782 */
  0x0000,   /* U+1F783 */
  0x0000,   /* U+1F784 */
  0x0000,   /* U+1F785 */
  0x0000,   /* U+1F786 */
  0x0000,   /* U+1F787 */
  0x0000,   /* U+1F788 */
  0x0000,   /* U+1F789 */
  0x0000,   /* U+1F78A */
  0x0000,   /* U+1F78B */
  0x0000,   /* U+1F78C */
  0x0000,   /* U+1F78D */
  0x0000,   /* U+1F78E */
  0x0000,   /* U+1F78F */
  0x0000,   /* U+1F790 */
  0x0000,   /* U+1F791 */
  0x0000,   /* U+1F792 */
  0x0000,   /* U+1F793 */
  0x0000,   /* U+1F794 */
  0x0000,   /* U+1F795 */
  0x0000,   /* U+1F796 */
  0x0000,   /* U+1F797 */
  0x0000,   /* U+1F798 */
  0x0000,   /* U+1F799 */
  0x0000,   /* U+1F79A */
  0x0000,   /* U+1F79B */
  0x0000,   /* U+1F79C */
  0x0000,   /* U+1F79D */
  0x0000,   /* U+1F79E */
  0x0000,   /* U+1F79F */
  0x0000,   /* U+1F7A0 */
  0x0000,   /* U+1F7A1 */
  0x0000,   /* U+1F7A2 */
  0x0000,   /* U+1F7A3 */
  0x0000,   /* U+1F7A4 */
  0x0000,   /* U+1F7A5 */
  0x0000,   /* U+1F7A6 */
  0x0000,   /* U+1F7A7 */
  0x0000,   /* U+1F7A8 */
  0x0000,   /* U+1F7A9 */
  0x0000,   /* U+1F7AA */
  0x0000,   /* U+1F7AB */
  0x0000,   /* U+1F7AC */
  0x0000,   /* U+1F7AD */
  0x0000,   /* U+1F7AE */
  0x0000,   /* U+1F7AF */
  0x0000,   /* U+1F7B0 */
  0x0000,   /* U+1F7B1 */
  0x0000,   /* U+1F7B2 */
  0x0000,   /* U+1F7B3 */
  0x0000,   /* U+1F7B4 */
  0x0000,   /* U+1F7B5 */
  0x0000,   /* U+1F7B6 */
  0x0000,   /* U+1F7B7 */
  0x0000,   /* U+1F7B8 */
  0x0000,   /* U+1F7B9 */
  0x0000,   /* U+1F7BA */
  0x0000,   /* U+1F7BB */
  0x0000,   /* U+1F7BC */
  0x0000,   /* U+1F7BD */
  0x0000,   /* U+1F7BE */
  0x0000,   /* U+1F7BF */
  0x0000,   /* U+1F7C0 */
  0x0000,   /* U+1F7C1 */
  0x0000,   /* U+1F7C2 */
  0x0000,   /* U+1F7C3 */
  0x0000,   /* U+1F7C4 */
  0x0000,   /* U+1F7C5 */
  0x0000,   /* U+1F7C6 */
  0x0000,   /* U+1F7C7 */
  0x0000,   /* U+1F7C8 */
  0x0000,   /* U+1F7C9 */
  0x0000,   /* U+1F7CA */
  0x0000,   /* U+1F7CB */
  0x0000,   /* U+1F7CC */
  0x0000,   /* U+1F7CD */
  0x0000,   /* U+1F7CE */
  0x0000,   /* U+1F7CF */
  0x0000,   /* U+1F7D0 */
  0x0000,   /* U+1F7D1 */
  0x0000,   /* U+1F7D2 */
  0x0000,   /* U+1F7D3 */
  0x0000,   /* U+1F7D4 */
  0x0000,   /* U+1F7D5 */
  0x0000,   /* U+1F7D6 */
  0x0000,   /* U+1F7D7 */
  0x0000,   /* U+1F7D8 */
  0x0000,   /* U+1F7D9 */
  0x0000,   /* U+1F7DA */
  0x0000,   /* U+1F7DB */
  0x0000,   /* U+1F7DC */
  0x0000,   /* U+1F7DD */
  0x0000,   /* U+1F7DE */
  0x0000,   /* U+1F7DF */
  0x0000,   /* U+1F7E0 */
  0x0000,   /* U+1F7E1 */
  0x0000,   /* U+1F7E2 */
  0x0000,   /* U+1F7E3 */
  0x0000,   /* U+1F7E4 */
  0x0000,   /* U+1F7E5 */
  0x0000,   /* U+1F7E6 */
  0x0000,   /* U+1F7E7 */
  0x0000,   /* U+1F7E8 */
  0x0000,   /* U+1F7E9 */
  0x0000,   /* U+1F7EA */
  0x0000,   /* U+1F7EB */
  0x0000,   /* U+1F7EC */
  0x0000,   /* U+1F7ED */
  0x0000,   /* U+1F7EE */
  0x0000,   /* U+1F7EF */
  0x0000,   /* U+1F7F0 */
  0x0000,   /* U+1F7F1 */
  0x0000,   /* U+1F7F2 */
  0x0000,   /* U+1F7F3 */
  0x0000,   /* U+1F7F4 */
  0x0000,   /* U+1F7F5 */
  0x0000,   /* U+1F7F6 */
  0x0000,   /* U+1F7F7 */
  0x0000,   /* U+1F7F8 */
  0x0000,   /* U+1F7F9 */
  0x0000,   /* U+1F7FA */
  0x0000,   /* U+1F7FB */
  0x0000,   /* U+1F7FC */
  0x0000,   /* U+1F7FD */
  0x0000,   /* U+1F7FE */
  0x0000,   /* U+1F7FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1F700 */
  0x0000,   /* U+1F701 */
  0x0000,   /* U+1F702 */
  0x0000,   /* U+1F703 */
  0x0000,   /* U+1F704 */
  0x0000,   /* U+1F705 */
  0x0000,   /* U+1F706 */
  0x0000,   /* U+1F707 */
  0x0000,   /* U+1F708 */
  0x0000,   /* U+1F709 */
  0x0000,   /* U+1F70A */
  0x0000,   /* U+1F70B */
  0x0000,   /* U+1F70C */
  0x0000,   /* U+1F70D */
  0x0000,   /* U+1F70E */
  0x0000,   /* U+1F70F */
  0x0000,   /* U+1F710 */
  0x0000,   /* U+1F711 */
  0x0000,   /* U+1F712 */
  0x0000,   /* U+1F713 */
  0x0000,   /* U+1F714 */
  0x0000,   /* U+1F715 */
  0x0000,   /* U+1F716 */
  0x0000,   /* U+1F717 */
  0x0000,   /* U+1F718 */
  0x0000,   /* U+1F719 */
  0x0000,   /* U+1F71A */
  0x0000,   /* U+1F71B */
  0x0000,   /* U+1F71C */
  0x0000,   /* U+1F71D */
  0x0000,   /* U+1F71E */
  0x0000,   /* U+1F71F */
  0x0000,   /* U+1F720 */
  0x0000,   /* U+1F721 */
  0x0000,   /* U+1F722 */
  0x0000,   /* U+1F723 */
  0x0000,   /* U+1F724 */
  0x0000,   /* U+1F725 */
  0x0000,   /* U+1F726 */
  0x0000,   /* U+1F727 */
  0x0000,   /* U+1F728 */
  0x0000,   /* U+1F729 */
  0x0000,   /* U+1F72A */
  0x0000,   /* U+1F72B */
  0x0000,   /* U+1F72C */
  0x0000,   /* U+1F72D */
  0x0000,   /* U+1F72E */
  0x0000,   /* U+1F72F */
  0x0000,   /* U+1F730 */
  0x0000,   /* U+1F731 */
  0x0000,   /* U+1F732 */
  0x0000,   /* U+1F733 */
  0x0000,   /* U+1F734 */
  0x0000,   /* U+1F735 */
  0x0000,   /* U+1F736 */
  0x0000,   /* U+1F737 */
  0x0000,   /* U+1F738 */
  0x0000,   /* U+1F739 */
  0x0000,   /* U+1F73A */
  0x0000,   /* U+1F73B */
  0x0000,   /* U+1F73C */
  0x0000,   /* U+1F73D */
  0x0000,   /* U+1F73E */
  0x0000,   /* U+1F73F */
  0x0000,   /* U+1F740 */
  0x0000,   /* U+1F741 */
  0x0000,   /* U+1F742 */
  0x0000,   /* U+1F743 */
  0x0000,   /* U+1F744 */
  0x0000,   /* U+1F745 */
  0x0000,   /* U+1F746 */
  0x0000,   /* U+1F747 */
  0x0000,   /* U+1F748 */
  0x0000,   /* U+1F749 */
  0x0000,   /* U+1F74A */
  0x0000,   /* U+1F74B */
  0x0000,   /* U+1F74C */
  0x0000,   /* U+1F74D */
  0x0000,   /* U+1F74E */
  0x0000,   /* U+1F74F */
  0x0000,   /* U+1F750 */
  0x0000,   /* U+1F751 */
  0x0000,   /* U+1F752 */
  0x0000,   /* U+1F753 */
  0x0000,   /* U+1F754 */
  0x0000,   /* U+1F755 */
  0x0000,   /* U+1F756 */
  0x0000,   /* U+1F757 */
  0x0000,   /* U+1F758 */
  0x0000,   /* U+1F759 */
  0x0000,   /* U+1F75A */
  0x0000,   /* U+1F75B */
  0x0000,   /* U+1F75C */
  0x0000,   /* U+1F75D */
  0x0000,   /* U+1F75E */
  0x0000,   /* U+1F75F */
  0x0000,   /* U+1F760 */
  0x0000,   /* U+1F761 */
  0x0000,   /* U+1F762 */
  0x0000,   /* U+1F763 */
  0x0000,   /* U+1F764 */
  0x0000,   /* U+1F765 */
  0x0000,   /* U+1F766 */
  0x0000,   /* U+1F767 */
  0x0000,   /* U+1F768 */
  0x0000,   /* U+1F769 */
  0x0000,   /* U+1F76A */
  0x0000,   /* U+1F76B */
  0x0000,   /* U+1F76C */
  0x0000,   /* U+1F76D */
  0x0000,   /* U+1F76E */
  0x0000,   /* U+1F76F */
  0x0000,   /* U+1F770 */
  0x0000,   /* U+1F771 */
  0x0000,   /* U+1F772 */
  0x0000,   /* U+1F773 */
  0x0000,   /* U+1F774 */
  0x0000,   /* U+1F775 */
  0x0000,   /* U+1F776 */
  0x0000,   /* U+1F777 */
  0x0000,   /* U+1F778 */
  0x0000,   /* U+1F779 */
  0x0000,   /* U+1F77A */
  0x0000,   /* U+1F77B */
  0x0000,   /* U+1F77C */
  0x0000,   /* U+1F77D */
  0x0000,   /* U+1F77E */
  0x0000,   /* U+1F77F */
  0x0000,   /* U+1F780 */
  0x0000,   /* U+1F781 */
  0x0000,   /* U+1F782 */
  0x0000,   /* U+1F783 */
  0x0000,   /* U+1F784 */
  0x0000,   /* U+1F785 */
  0x0000,   /* U+1F786 */
  0x0000,   /* U+1F787 */
  0x0000,   /* U+1F788 */
  0x0000,   /* U+1F789 */
  0x0000,   /* U+1F78A */
  0x0000,   /* U+1F78B */
  0x0000,   /* U+1F78C */
  0x0000,   /* U+1F78D */
  0x0000,   /* U+1F78E */
  0x0000,   /* U+1F78F */
  0x0000,   /* U+1F790 */
  0x0000,   /* U+1F791 */
  0x0000,   /* U+1F792 */
  0x0000,   /* U+1F793 */
  0x0000,   /* U+1F794 */
  0x0000,   /* U+1F795 */
  0x0000,   /* U+1F796 */
  0x0000,   /* U+1F797 */
  0x0000,   /* U+1F798 */
  0x0000,   /* U+1F799 */
  0x0000,   /* U+1F79A */
  0x0000,   /* U+1F79B */
  0x0000,   /* U+1F79C */
  0x0000,   /* U+1F79D */
  0x0000,   /* U+1F79E */
  0x0000,   /* U+1F79F */
  0x0000,   /* U+1F7A0 */
  0x0000,   /* U+1F7A1 */
  0x0000,   /* U+1F7A2 */
  0x0000,   /* U+1F7A3 */
  0x0000,   /* U+1F7A4 */
  0x0000,   /* U+1F7A5 */
  0x0000,   /* U+1F7A6 */
  0x0000,   /* U+1F7A7 */
  0x0000,   /* U+1F7A8 */
  0x0000,   /* U+1F7A9 */
  0x0000,   /* U+1F7AA */
  0x0000,   /* U+1F7AB */
  0x0000,   /* U+1F7AC */
  0x0000,   /* U+1F7AD */
  0x0000,   /* U+1F7AE */
  0x0000,   /* U+1F7AF */
  0x0000,   /* U+1F7B0 */
  0x0000,   /* U+1F7B1 */
  0x0000,   /* U+1F7B2 */
  0x0000,   /* U+1F7B3 */
  0x0000,   /* U+1F7B4 */
  0x0000,   /* U+1F7B5 */
  0x0000,   /* U+1F7B6 */
  0x0000,   /* U+1F7B7 */
  0x0000,   /* U+1F7B8 */
  0x0000,   /* U+1F7B9 */
  0x0000,   /* U+1F7BA */
  0x0000,   /* U+1F7BB */
  0x0000,   /* U+1F7BC */
  0x0000,   /* U+1F7BD */
  0x0000,   /* U+1F7BE */
  0x0000,   /* U+1F7BF */
  0x0000,   /* U+1F7C0 */
  0x0000,   /* U+1F7C1 */
  0x0000,   /* U+1F7C2 */
  0x0000,   /* U+1F7C3 */
  0x0000,   /* U+1F7C4 */
  0x0000,   /* U+1F7C5 */
  0x0000,   /* U+1F7C6 */
  0x0000,   /* U+1F7C7 */
  0x0000,   /* U+1F7C8 */
  0x0000,   /* U+1F7C9 */
  0x0000,   /* U+1F7CA */
  0x0000,   /* U+1F7CB */
  0x0000,   /* U+1F7CC */
  0x0000,   /* U+1F7CD */
  0x0000,   /* U+1F7CE */
  0x0000,   /* U+1F7CF */
  0x0000,   /* U+1F7D0 */
  0x0000,   /* U+1F7D1 */
  0x0000,   /* U+1F7D2 */
  0x0000,   /* U+1F7D3 */
  0x0000,   /* U+1F7D4 */
  0x0000,   /* U+1F7D5 */
  0x0000,   /* U+1F7D6 */
  0x0000,   /* U+1F7D7 */
  0x0000,   /* U+1F7D8 */
  0x0000,   /* U+1F7D9 */
  0x0000,   /* U+1F7DA */
  0x0000,   /* U+1F7DB */
  0x0000,   /* U+1F7DC */
  0x0000,   /* U+1F7DD */
  0x0000,   /* U+1F7DE */
  0x0000,   /* U+1F7DF */
  0x0000,   /* U+1F7E0 */
  0x0000,   /* U+1F7E1 */
  0x0000,   /* U+1F7E2 */
  0x0000,   /* U+1F7E3 */
  0x0000,   /* U+1F7E4 */
  0x0000,   /* U+1F7E5 */
  0x0000,   /* U+1F7E6 */
  0x0000,   /* U+1F7E7 */
  0x0000,   /* U+1F7E8 */
  0x0000,   /* U+1F7E9 */
  0x0000,   /* U+1F7EA */
  0x0000,   /* U+1F7EB */
  0x0000,   /* U+1F7EC */
  0x0000,   /* U+1F7ED */
  0x0000,   /* U+1F7EE */
  0x0000,   /* U+1F7EF */
  0x0000,   /* U+1F7F0 */
  0x0000,   /* U+1F7F1 */
  0x0000,   /* U+1F7F2 */
  0x0000,   /* U+1F7F3 */
  0x0000,   /* U+1F7F4 */
  0x0000,   /* U+1F7F5 */
  0x0000,   /* U+1F7F6 */
  0x0000,   /* U+1F7F7 */
  0x0000,   /* U+1F7F8 */
  0x0000,   /* U+1F7F9 */
  0x0000,   /* U+1F7FA */
  0x0000,   /* U+1F7FB */
  0x0000,   /* U+1F7FC */
  0x0000,   /* U+1F7FD */
  0x0000,   /* U+1F7FE */
  0x0000,   /* U+1F7FF */
};

uint16 uca900_p1F8[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x17AB,   /* U+1F800 */
  0x17AC,   /* U+1F801 */
  0x17AD,   /* U+1F802 */
  0x17AE,   /* U+1F803 */
  0x17AF,   /* U+1F804 */
  0x17B0,   /* U+1F805 */
  0x17B1,   /* U+1F806 */
  0x17B2,   /* U+1F807 */
  0x17B3,   /* U+1F808 */
  0x17B4,   /* U+1F809 */
  0x17B5,   /* U+1F80A */
  0x17B6,   /* U+1F80B */
  0xFBC3,   /* U+1F80C */
  0xFBC3,   /* U+1F80D */
  0xFBC3,   /* U+1F80E */
  0xFBC3,   /* U+1F80F */
  0x17B7,   /* U+1F810 */
  0x17B8,   /* U+1F811 */
  0x17B9,   /* U+1F812 */
  0x17BA,   /* U+1F813 */
  0x17BB,   /* U+1F814 */
  0x17BC,   /* U+1F815 */
  0x17BD,   /* U+1F816 */
  0x17BE,   /* U+1F817 */
  0x17BF,   /* U+1F818 */
  0x17C0,   /* U+1F819 */
  0x17C1,   /* U+1F81A */
  0x17C2,   /* U+1F81B */
  0x17C3,   /* U+1F81C */
  0x17C4,   /* U+1F81D */
  0x17C5,   /* U+1F81E */
  0x17C6,   /* U+1F81F */
  0x17C7,   /* U+1F820 */
  0x17C8,   /* U+1F821 */
  0x17C9,   /* U+1F822 */
  0x17CA,   /* U+1F823 */
  0x17CB,   /* U+1F824 */
  0x17CC,   /* U+1F825 */
  0x17CD,   /* U+1F826 */
  0x17CE,   /* U+1F827 */
  0x17CF,   /* U+1F828 */
  0x17D0,   /* U+1F829 */
  0x17D1,   /* U+1F82A */
  0x17D2,   /* U+1F82B */
  0x17D3,   /* U+1F82C */
  0x17D4,   /* U+1F82D */
  0x17D5,   /* U+1F82E */
  0x17D6,   /* U+1F82F */
  0x17D7,   /* U+1F830 */
  0x17D8,   /* U+1F831 */
  0x17D9,   /* U+1F832 */
  0x17DA,   /* U+1F833 */
  0x17DB,   /* U+1F834 */
  0x17DC,   /* U+1F835 */
  0x17DD,   /* U+1F836 */
  0x17DE,   /* U+1F837 */
  0x17DF,   /* U+1F838 */
  0x17E0,   /* U+1F839 */
  0x17E1,   /* U+1F83A */
  0x17E2,   /* U+1F83B */
  0x17E3,   /* U+1F83C */
  0x17E4,   /* U+1F83D */
  0x17E5,   /* U+1F83E */
  0x17E6,   /* U+1F83F */
  0x17E7,   /* U+1F840 */
  0x17E8,   /* U+1F841 */
  0x17E9,   /* U+1F842 */
  0x17EA,   /* U+1F843 */
  0x17EB,   /* U+1F844 */
  0x17EC,   /* U+1F845 */
  0x17ED,   /* U+1F846 */
  0x17EE,   /* U+1F847 */
  0xFBC3,   /* U+1F848 */
  0xFBC3,   /* U+1F849 */
  0xFBC3,   /* U+1F84A */
  0xFBC3,   /* U+1F84B */
  0xFBC3,   /* U+1F84C */
  0xFBC3,   /* U+1F84D */
  0xFBC3,   /* U+1F84E */
  0xFBC3,   /* U+1F84F */
  0x17EF,   /* U+1F850 */
  0x17F0,   /* U+1F851 */
  0x17F1,   /* U+1F852 */
  0x17F2,   /* U+1F853 */
  0x17F3,   /* U+1F854 */
  0x17F4,   /* U+1F855 */
  0x17F5,   /* U+1F856 */
  0x17F6,   /* U+1F857 */
  0x17F7,   /* U+1F858 */
  0x17F8,   /* U+1F859 */
  0xFBC3,   /* U+1F85A */
  0xFBC3,   /* U+1F85B */
  0xFBC3,   /* U+1F85C */
  0xFBC3,   /* U+1F85D */
  0xFBC3,   /* U+1F85E */
  0xFBC3,   /* U+1F85F */
  0x17F9,   /* U+1F860 */
  0x17FA,   /* U+1F861 */
  0x17FB,   /* U+1F862 */
  0x17FC,   /* U+1F863 */
  0x17FD,   /* U+1F864 */
  0x17FE,   /* U+1F865 */
  0x17FF,   /* U+1F866 */
  0x1800,   /* U+1F867 */
  0x1801,   /* U+1F868 */
  0x1802,   /* U+1F869 */
  0x1803,   /* U+1F86A */
  0x1804,   /* U+1F86B */
  0x1805,   /* U+1F86C */
  0x1806,   /* U+1F86D */
  0x1807,   /* U+1F86E */
  0x1808,   /* U+1F86F */
  0x1809,   /* U+1F870 */
  0x180A,   /* U+1F871 */
  0x180B,   /* U+1F872 */
  0x180C,   /* U+1F873 */
  0x180D,   /* U+1F874 */
  0x180E,   /* U+1F875 */
  0x180F,   /* U+1F876 */
  0x1810,   /* U+1F877 */
  0x1811,   /* U+1F878 */
  0x1812,   /* U+1F879 */
  0x1813,   /* U+1F87A */
  0x1814,   /* U+1F87B */
  0x1815,   /* U+1F87C */
  0x1816,   /* U+1F87D */
  0x1817,   /* U+1F87E */
  0x1818,   /* U+1F87F */
  0x1819,   /* U+1F880 */
  0x181A,   /* U+1F881 */
  0x181B,   /* U+1F882 */
  0x181C,   /* U+1F883 */
  0x181D,   /* U+1F884 */
  0x181E,   /* U+1F885 */
  0x181F,   /* U+1F886 */
  0x1820,   /* U+1F887 */
  0xFBC3,   /* U+1F888 */
  0xFBC3,   /* U+1F889 */
  0xFBC3,   /* U+1F88A */
  0xFBC3,   /* U+1F88B */
  0xFBC3,   /* U+1F88C */
  0xFBC3,   /* U+1F88D */
  0xFBC3,   /* U+1F88E */
  0xFBC3,   /* U+1F88F */
  0x1821,   /* U+1F890 */
  0x1822,   /* U+1F891 */
  0x1823,   /* U+1F892 */
  0x1824,   /* U+1F893 */
  0x1825,   /* U+1F894 */
  0x1826,   /* U+1F895 */
  0x1827,   /* U+1F896 */
  0x1828,   /* U+1F897 */
  0x1829,   /* U+1F898 */
  0x182A,   /* U+1F899 */
  0x182B,   /* U+1F89A */
  0x182C,   /* U+1F89B */
  0x182D,   /* U+1F89C */
  0x182E,   /* U+1F89D */
  0x182F,   /* U+1F89E */
  0x1830,   /* U+1F89F */
  0x1831,   /* U+1F8A0 */
  0x1832,   /* U+1F8A1 */
  0x1833,   /* U+1F8A2 */
  0x1834,   /* U+1F8A3 */
  0x1835,   /* U+1F8A4 */
  0x1836,   /* U+1F8A5 */
  0x1837,   /* U+1F8A6 */
  0x1838,   /* U+1F8A7 */
  0x1839,   /* U+1F8A8 */
  0x183A,   /* U+1F8A9 */
  0x183B,   /* U+1F8AA */
  0x183C,   /* U+1F8AB */
  0x183D,   /* U+1F8AC */
  0x183E,   /* U+1F8AD */
  0xFBC3,   /* U+1F8AE */
  0xFBC3,   /* U+1F8AF */
  0xFBC3,   /* U+1F8B0 */
  0xFBC3,   /* U+1F8B1 */
  0xFBC3,   /* U+1F8B2 */
  0xFBC3,   /* U+1F8B3 */
  0xFBC3,   /* U+1F8B4 */
  0xFBC3,   /* U+1F8B5 */
  0xFBC3,   /* U+1F8B6 */
  0xFBC3,   /* U+1F8B7 */
  0xFBC3,   /* U+1F8B8 */
  0xFBC3,   /* U+1F8B9 */
  0xFBC3,   /* U+1F8BA */
  0xFBC3,   /* U+1F8BB */
  0xFBC3,   /* U+1F8BC */
  0xFBC3,   /* U+1F8BD */
  0xFBC3,   /* U+1F8BE */
  0xFBC3,   /* U+1F8BF */
  0xFBC3,   /* U+1F8C0 */
  0xFBC3,   /* U+1F8C1 */
  0xFBC3,   /* U+1F8C2 */
  0xFBC3,   /* U+1F8C3 */
  0xFBC3,   /* U+1F8C4 */
  0xFBC3,   /* U+1F8C5 */
  0xFBC3,   /* U+1F8C6 */
  0xFBC3,   /* U+1F8C7 */
  0xFBC3,   /* U+1F8C8 */
  0xFBC3,   /* U+1F8C9 */
  0xFBC3,   /* U+1F8CA */
  0xFBC3,   /* U+1F8CB */
  0xFBC3,   /* U+1F8CC */
  0xFBC3,   /* U+1F8CD */
  0xFBC3,   /* U+1F8CE */
  0xFBC3,   /* U+1F8CF */
  0xFBC3,   /* U+1F8D0 */
  0xFBC3,   /* U+1F8D1 */
  0xFBC3,   /* U+1F8D2 */
  0xFBC3,   /* U+1F8D3 */
  0xFBC3,   /* U+1F8D4 */
  0xFBC3,   /* U+1F8D5 */
  0xFBC3,   /* U+1F8D6 */
  0xFBC3,   /* U+1F8D7 */
  0xFBC3,   /* U+1F8D8 */
  0xFBC3,   /* U+1F8D9 */
  0xFBC3,   /* U+1F8DA */
  0xFBC3,   /* U+1F8DB */
  0xFBC3,   /* U+1F8DC */
  0xFBC3,   /* U+1F8DD */
  0xFBC3,   /* U+1F8DE */
  0xFBC3,   /* U+1F8DF */
  0xFBC3,   /* U+1F8E0 */
  0xFBC3,   /* U+1F8E1 */
  0xFBC3,   /* U+1F8E2 */
  0xFBC3,   /* U+1F8E3 */
  0xFBC3,   /* U+1F8E4 */
  0xFBC3,   /* U+1F8E5 */
  0xFBC3,   /* U+1F8E6 */
  0xFBC3,   /* U+1F8E7 */
  0xFBC3,   /* U+1F8E8 */
  0xFBC3,   /* U+1F8E9 */
  0xFBC3,   /* U+1F8EA */
  0xFBC3,   /* U+1F8EB */
  0xFBC3,   /* U+1F8EC */
  0xFBC3,   /* U+1F8ED */
  0xFBC3,   /* U+1F8EE */
  0xFBC3,   /* U+1F8EF */
  0xFBC3,   /* U+1F8F0 */
  0xFBC3,   /* U+1F8F1 */
  0xFBC3,   /* U+1F8F2 */
  0xFBC3,   /* U+1F8F3 */
  0xFBC3,   /* U+1F8F4 */
  0xFBC3,   /* U+1F8F5 */
  0xFBC3,   /* U+1F8F6 */
  0xFBC3,   /* U+1F8F7 */
  0xFBC3,   /* U+1F8F8 */
  0xFBC3,   /* U+1F8F9 */
  0xFBC3,   /* U+1F8FA */
  0xFBC3,   /* U+1F8FB */
  0xFBC3,   /* U+1F8FC */
  0xFBC3,   /* U+1F8FD */
  0xFBC3,   /* U+1F8FE */
  0xFBC3,   /* U+1F8FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F800 */
  0x0020,   /* U+1F801 */
  0x0020,   /* U+1F802 */
  0x0020,   /* U+1F803 */
  0x0020,   /* U+1F804 */
  0x0020,   /* U+1F805 */
  0x0020,   /* U+1F806 */
  0x0020,   /* U+1F807 */
  0x0020,   /* U+1F808 */
  0x0020,   /* U+1F809 */
  0x0020,   /* U+1F80A */
  0x0020,   /* U+1F80B */
  0x0020,   /* U+1F80C */
  0x0020,   /* U+1F80D */
  0x0020,   /* U+1F80E */
  0x0020,   /* U+1F80F */
  0x0020,   /* U+1F810 */
  0x0020,   /* U+1F811 */
  0x0020,   /* U+1F812 */
  0x0020,   /* U+1F813 */
  0x0020,   /* U+1F814 */
  0x0020,   /* U+1F815 */
  0x0020,   /* U+1F816 */
  0x0020,   /* U+1F817 */
  0x0020,   /* U+1F818 */
  0x0020,   /* U+1F819 */
  0x0020,   /* U+1F81A */
  0x0020,   /* U+1F81B */
  0x0020,   /* U+1F81C */
  0x0020,   /* U+1F81D */
  0x0020,   /* U+1F81E */
  0x0020,   /* U+1F81F */
  0x0020,   /* U+1F820 */
  0x0020,   /* U+1F821 */
  0x0020,   /* U+1F822 */
  0x0020,   /* U+1F823 */
  0x0020,   /* U+1F824 */
  0x0020,   /* U+1F825 */
  0x0020,   /* U+1F826 */
  0x0020,   /* U+1F827 */
  0x0020,   /* U+1F828 */
  0x0020,   /* U+1F829 */
  0x0020,   /* U+1F82A */
  0x0020,   /* U+1F82B */
  0x0020,   /* U+1F82C */
  0x0020,   /* U+1F82D */
  0x0020,   /* U+1F82E */
  0x0020,   /* U+1F82F */
  0x0020,   /* U+1F830 */
  0x0020,   /* U+1F831 */
  0x0020,   /* U+1F832 */
  0x0020,   /* U+1F833 */
  0x0020,   /* U+1F834 */
  0x0020,   /* U+1F835 */
  0x0020,   /* U+1F836 */
  0x0020,   /* U+1F837 */
  0x0020,   /* U+1F838 */
  0x0020,   /* U+1F839 */
  0x0020,   /* U+1F83A */
  0x0020,   /* U+1F83B */
  0x0020,   /* U+1F83C */
  0x0020,   /* U+1F83D */
  0x0020,   /* U+1F83E */
  0x0020,   /* U+1F83F */
  0x0020,   /* U+1F840 */
  0x0020,   /* U+1F841 */
  0x0020,   /* U+1F842 */
  0x0020,   /* U+1F843 */
  0x0020,   /* U+1F844 */
  0x0020,   /* U+1F845 */
  0x0020,   /* U+1F846 */
  0x0020,   /* U+1F847 */
  0x0020,   /* U+1F848 */
  0x0020,   /* U+1F849 */
  0x0020,   /* U+1F84A */
  0x0020,   /* U+1F84B */
  0x0020,   /* U+1F84C */
  0x0020,   /* U+1F84D */
  0x0020,   /* U+1F84E */
  0x0020,   /* U+1F84F */
  0x0020,   /* U+1F850 */
  0x0020,   /* U+1F851 */
  0x0020,   /* U+1F852 */
  0x0020,   /* U+1F853 */
  0x0020,   /* U+1F854 */
  0x0020,   /* U+1F855 */
  0x0020,   /* U+1F856 */
  0x0020,   /* U+1F857 */
  0x0020,   /* U+1F858 */
  0x0020,   /* U+1F859 */
  0x0020,   /* U+1F85A */
  0x0020,   /* U+1F85B */
  0x0020,   /* U+1F85C */
  0x0020,   /* U+1F85D */
  0x0020,   /* U+1F85E */
  0x0020,   /* U+1F85F */
  0x0020,   /* U+1F860 */
  0x0020,   /* U+1F861 */
  0x0020,   /* U+1F862 */
  0x0020,   /* U+1F863 */
  0x0020,   /* U+1F864 */
  0x0020,   /* U+1F865 */
  0x0020,   /* U+1F866 */
  0x0020,   /* U+1F867 */
  0x0020,   /* U+1F868 */
  0x0020,   /* U+1F869 */
  0x0020,   /* U+1F86A */
  0x0020,   /* U+1F86B */
  0x0020,   /* U+1F86C */
  0x0020,   /* U+1F86D */
  0x0020,   /* U+1F86E */
  0x0020,   /* U+1F86F */
  0x0020,   /* U+1F870 */
  0x0020,   /* U+1F871 */
  0x0020,   /* U+1F872 */
  0x0020,   /* U+1F873 */
  0x0020,   /* U+1F874 */
  0x0020,   /* U+1F875 */
  0x0020,   /* U+1F876 */
  0x0020,   /* U+1F877 */
  0x0020,   /* U+1F878 */
  0x0020,   /* U+1F879 */
  0x0020,   /* U+1F87A */
  0x0020,   /* U+1F87B */
  0x0020,   /* U+1F87C */
  0x0020,   /* U+1F87D */
  0x0020,   /* U+1F87E */
  0x0020,   /* U+1F87F */
  0x0020,   /* U+1F880 */
  0x0020,   /* U+1F881 */
  0x0020,   /* U+1F882 */
  0x0020,   /* U+1F883 */
  0x0020,   /* U+1F884 */
  0x0020,   /* U+1F885 */
  0x0020,   /* U+1F886 */
  0x0020,   /* U+1F887 */
  0x0020,   /* U+1F888 */
  0x0020,   /* U+1F889 */
  0x0020,   /* U+1F88A */
  0x0020,   /* U+1F88B */
  0x0020,   /* U+1F88C */
  0x0020,   /* U+1F88D */
  0x0020,   /* U+1F88E */
  0x0020,   /* U+1F88F */
  0x0020,   /* U+1F890 */
  0x0020,   /* U+1F891 */
  0x0020,   /* U+1F892 */
  0x0020,   /* U+1F893 */
  0x0020,   /* U+1F894 */
  0x0020,   /* U+1F895 */
  0x0020,   /* U+1F896 */
  0x0020,   /* U+1F897 */
  0x0020,   /* U+1F898 */
  0x0020,   /* U+1F899 */
  0x0020,   /* U+1F89A */
  0x0020,   /* U+1F89B */
  0x0020,   /* U+1F89C */
  0x0020,   /* U+1F89D */
  0x0020,   /* U+1F89E */
  0x0020,   /* U+1F89F */
  0x0020,   /* U+1F8A0 */
  0x0020,   /* U+1F8A1 */
  0x0020,   /* U+1F8A2 */
  0x0020,   /* U+1F8A3 */
  0x0020,   /* U+1F8A4 */
  0x0020,   /* U+1F8A5 */
  0x0020,   /* U+1F8A6 */
  0x0020,   /* U+1F8A7 */
  0x0020,   /* U+1F8A8 */
  0x0020,   /* U+1F8A9 */
  0x0020,   /* U+1F8AA */
  0x0020,   /* U+1F8AB */
  0x0020,   /* U+1F8AC */
  0x0020,   /* U+1F8AD */
  0x0020,   /* U+1F8AE */
  0x0020,   /* U+1F8AF */
  0x0020,   /* U+1F8B0 */
  0x0020,   /* U+1F8B1 */
  0x0020,   /* U+1F8B2 */
  0x0020,   /* U+1F8B3 */
  0x0020,   /* U+1F8B4 */
  0x0020,   /* U+1F8B5 */
  0x0020,   /* U+1F8B6 */
  0x0020,   /* U+1F8B7 */
  0x0020,   /* U+1F8B8 */
  0x0020,   /* U+1F8B9 */
  0x0020,   /* U+1F8BA */
  0x0020,   /* U+1F8BB */
  0x0020,   /* U+1F8BC */
  0x0020,   /* U+1F8BD */
  0x0020,   /* U+1F8BE */
  0x0020,   /* U+1F8BF */
  0x0020,   /* U+1F8C0 */
  0x0020,   /* U+1F8C1 */
  0x0020,   /* U+1F8C2 */
  0x0020,   /* U+1F8C3 */
  0x0020,   /* U+1F8C4 */
  0x0020,   /* U+1F8C5 */
  0x0020,   /* U+1F8C6 */
  0x0020,   /* U+1F8C7 */
  0x0020,   /* U+1F8C8 */
  0x0020,   /* U+1F8C9 */
  0x0020,   /* U+1F8CA */
  0x0020,   /* U+1F8CB */
  0x0020,   /* U+1F8CC */
  0x0020,   /* U+1F8CD */
  0x0020,   /* U+1F8CE */
  0x0020,   /* U+1F8CF */
  0x0020,   /* U+1F8D0 */
  0x0020,   /* U+1F8D1 */
  0x0020,   /* U+1F8D2 */
  0x0020,   /* U+1F8D3 */
  0x0020,   /* U+1F8D4 */
  0x0020,   /* U+1F8D5 */
  0x0020,   /* U+1F8D6 */
  0x0020,   /* U+1F8D7 */
  0x0020,   /* U+1F8D8 */
  0x0020,   /* U+1F8D9 */
  0x0020,   /* U+1F8DA */
  0x0020,   /* U+1F8DB */
  0x0020,   /* U+1F8DC */
  0x0020,   /* U+1F8DD */
  0x0020,   /* U+1F8DE */
  0x0020,   /* U+1F8DF */
  0x0020,   /* U+1F8E0 */
  0x0020,   /* U+1F8E1 */
  0x0020,   /* U+1F8E2 */
  0x0020,   /* U+1F8E3 */
  0x0020,   /* U+1F8E4 */
  0x0020,   /* U+1F8E5 */
  0x0020,   /* U+1F8E6 */
  0x0020,   /* U+1F8E7 */
  0x0020,   /* U+1F8E8 */
  0x0020,   /* U+1F8E9 */
  0x0020,   /* U+1F8EA */
  0x0020,   /* U+1F8EB */
  0x0020,   /* U+1F8EC */
  0x0020,   /* U+1F8ED */
  0x0020,   /* U+1F8EE */
  0x0020,   /* U+1F8EF */
  0x0020,   /* U+1F8F0 */
  0x0020,   /* U+1F8F1 */
  0x0020,   /* U+1F8F2 */
  0x0020,   /* U+1F8F3 */
  0x0020,   /* U+1F8F4 */
  0x0020,   /* U+1F8F5 */
  0x0020,   /* U+1F8F6 */
  0x0020,   /* U+1F8F7 */
  0x0020,   /* U+1F8F8 */
  0x0020,   /* U+1F8F9 */
  0x0020,   /* U+1F8FA */
  0x0020,   /* U+1F8FB */
  0x0020,   /* U+1F8FC */
  0x0020,   /* U+1F8FD */
  0x0020,   /* U+1F8FE */
  0x0020,   /* U+1F8FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F800 */
  0x0002,   /* U+1F801 */
  0x0002,   /* U+1F802 */
  0x0002,   /* U+1F803 */
  0x0002,   /* U+1F804 */
  0x0002,   /* U+1F805 */
  0x0002,   /* U+1F806 */
  0x0002,   /* U+1F807 */
  0x0002,   /* U+1F808 */
  0x0002,   /* U+1F809 */
  0x0002,   /* U+1F80A */
  0x0002,   /* U+1F80B */
  0x0002,   /* U+1F80C */
  0x0002,   /* U+1F80D */
  0x0002,   /* U+1F80E */
  0x0002,   /* U+1F80F */
  0x0002,   /* U+1F810 */
  0x0002,   /* U+1F811 */
  0x0002,   /* U+1F812 */
  0x0002,   /* U+1F813 */
  0x0002,   /* U+1F814 */
  0x0002,   /* U+1F815 */
  0x0002,   /* U+1F816 */
  0x0002,   /* U+1F817 */
  0x0002,   /* U+1F818 */
  0x0002,   /* U+1F819 */
  0x0002,   /* U+1F81A */
  0x0002,   /* U+1F81B */
  0x0002,   /* U+1F81C */
  0x0002,   /* U+1F81D */
  0x0002,   /* U+1F81E */
  0x0002,   /* U+1F81F */
  0x0002,   /* U+1F820 */
  0x0002,   /* U+1F821 */
  0x0002,   /* U+1F822 */
  0x0002,   /* U+1F823 */
  0x0002,   /* U+1F824 */
  0x0002,   /* U+1F825 */
  0x0002,   /* U+1F826 */
  0x0002,   /* U+1F827 */
  0x0002,   /* U+1F828 */
  0x0002,   /* U+1F829 */
  0x0002,   /* U+1F82A */
  0x0002,   /* U+1F82B */
  0x0002,   /* U+1F82C */
  0x0002,   /* U+1F82D */
  0x0002,   /* U+1F82E */
  0x0002,   /* U+1F82F */
  0x0002,   /* U+1F830 */
  0x0002,   /* U+1F831 */
  0x0002,   /* U+1F832 */
  0x0002,   /* U+1F833 */
  0x0002,   /* U+1F834 */
  0x0002,   /* U+1F835 */
  0x0002,   /* U+1F836 */
  0x0002,   /* U+1F837 */
  0x0002,   /* U+1F838 */
  0x0002,   /* U+1F839 */
  0x0002,   /* U+1F83A */
  0x0002,   /* U+1F83B */
  0x0002,   /* U+1F83C */
  0x0002,   /* U+1F83D */
  0x0002,   /* U+1F83E */
  0x0002,   /* U+1F83F */
  0x0002,   /* U+1F840 */
  0x0002,   /* U+1F841 */
  0x0002,   /* U+1F842 */
  0x0002,   /* U+1F843 */
  0x0002,   /* U+1F844 */
  0x0002,   /* U+1F845 */
  0x0002,   /* U+1F846 */
  0x0002,   /* U+1F847 */
  0x0002,   /* U+1F848 */
  0x0002,   /* U+1F849 */
  0x0002,   /* U+1F84A */
  0x0002,   /* U+1F84B */
  0x0002,   /* U+1F84C */
  0x0002,   /* U+1F84D */
  0x0002,   /* U+1F84E */
  0x0002,   /* U+1F84F */
  0x0002,   /* U+1F850 */
  0x0002,   /* U+1F851 */
  0x0002,   /* U+1F852 */
  0x0002,   /* U+1F853 */
  0x0002,   /* U+1F854 */
  0x0002,   /* U+1F855 */
  0x0002,   /* U+1F856 */
  0x0002,   /* U+1F857 */
  0x0002,   /* U+1F858 */
  0x0002,   /* U+1F859 */
  0x0002,   /* U+1F85A */
  0x0002,   /* U+1F85B */
  0x0002,   /* U+1F85C */
  0x0002,   /* U+1F85D */
  0x0002,   /* U+1F85E */
  0x0002,   /* U+1F85F */
  0x0002,   /* U+1F860 */
  0x0002,   /* U+1F861 */
  0x0002,   /* U+1F862 */
  0x0002,   /* U+1F863 */
  0x0002,   /* U+1F864 */
  0x0002,   /* U+1F865 */
  0x0002,   /* U+1F866 */
  0x0002,   /* U+1F867 */
  0x0002,   /* U+1F868 */
  0x0002,   /* U+1F869 */
  0x0002,   /* U+1F86A */
  0x0002,   /* U+1F86B */
  0x0002,   /* U+1F86C */
  0x0002,   /* U+1F86D */
  0x0002,   /* U+1F86E */
  0x0002,   /* U+1F86F */
  0x0002,   /* U+1F870 */
  0x0002,   /* U+1F871 */
  0x0002,   /* U+1F872 */
  0x0002,   /* U+1F873 */
  0x0002,   /* U+1F874 */
  0x0002,   /* U+1F875 */
  0x0002,   /* U+1F876 */
  0x0002,   /* U+1F877 */
  0x0002,   /* U+1F878 */
  0x0002,   /* U+1F879 */
  0x0002,   /* U+1F87A */
  0x0002,   /* U+1F87B */
  0x0002,   /* U+1F87C */
  0x0002,   /* U+1F87D */
  0x0002,   /* U+1F87E */
  0x0002,   /* U+1F87F */
  0x0002,   /* U+1F880 */
  0x0002,   /* U+1F881 */
  0x0002,   /* U+1F882 */
  0x0002,   /* U+1F883 */
  0x0002,   /* U+1F884 */
  0x0002,   /* U+1F885 */
  0x0002,   /* U+1F886 */
  0x0002,   /* U+1F887 */
  0x0002,   /* U+1F888 */
  0x0002,   /* U+1F889 */
  0x0002,   /* U+1F88A */
  0x0002,   /* U+1F88B */
  0x0002,   /* U+1F88C */
  0x0002,   /* U+1F88D */
  0x0002,   /* U+1F88E */
  0x0002,   /* U+1F88F */
  0x0002,   /* U+1F890 */
  0x0002,   /* U+1F891 */
  0x0002,   /* U+1F892 */
  0x0002,   /* U+1F893 */
  0x0002,   /* U+1F894 */
  0x0002,   /* U+1F895 */
  0x0002,   /* U+1F896 */
  0x0002,   /* U+1F897 */
  0x0002,   /* U+1F898 */
  0x0002,   /* U+1F899 */
  0x0002,   /* U+1F89A */
  0x0002,   /* U+1F89B */
  0x0002,   /* U+1F89C */
  0x0002,   /* U+1F89D */
  0x0002,   /* U+1F89E */
  0x0002,   /* U+1F89F */
  0x0002,   /* U+1F8A0 */
  0x0002,   /* U+1F8A1 */
  0x0002,   /* U+1F8A2 */
  0x0002,   /* U+1F8A3 */
  0x0002,   /* U+1F8A4 */
  0x0002,   /* U+1F8A5 */
  0x0002,   /* U+1F8A6 */
  0x0002,   /* U+1F8A7 */
  0x0002,   /* U+1F8A8 */
  0x0002,   /* U+1F8A9 */
  0x0002,   /* U+1F8AA */
  0x0002,   /* U+1F8AB */
  0x0002,   /* U+1F8AC */
  0x0002,   /* U+1F8AD */
  0x0002,   /* U+1F8AE */
  0x0002,   /* U+1F8AF */
  0x0002,   /* U+1F8B0 */
  0x0002,   /* U+1F8B1 */
  0x0002,   /* U+1F8B2 */
  0x0002,   /* U+1F8B3 */
  0x0002,   /* U+1F8B4 */
  0x0002,   /* U+1F8B5 */
  0x0002,   /* U+1F8B6 */
  0x0002,   /* U+1F8B7 */
  0x0002,   /* U+1F8B8 */
  0x0002,   /* U+1F8B9 */
  0x0002,   /* U+1F8BA */
  0x0002,   /* U+1F8BB */
  0x0002,   /* U+1F8BC */
  0x0002,   /* U+1F8BD */
  0x0002,   /* U+1F8BE */
  0x0002,   /* U+1F8BF */
  0x0002,   /* U+1F8C0 */
  0x0002,   /* U+1F8C1 */
  0x0002,   /* U+1F8C2 */
  0x0002,   /* U+1F8C3 */
  0x0002,   /* U+1F8C4 */
  0x0002,   /* U+1F8C5 */
  0x0002,   /* U+1F8C6 */
  0x0002,   /* U+1F8C7 */
  0x0002,   /* U+1F8C8 */
  0x0002,   /* U+1F8C9 */
  0x0002,   /* U+1F8CA */
  0x0002,   /* U+1F8CB */
  0x0002,   /* U+1F8CC */
  0x0002,   /* U+1F8CD */
  0x0002,   /* U+1F8CE */
  0x0002,   /* U+1F8CF */
  0x0002,   /* U+1F8D0 */
  0x0002,   /* U+1F8D1 */
  0x0002,   /* U+1F8D2 */
  0x0002,   /* U+1F8D3 */
  0x0002,   /* U+1F8D4 */
  0x0002,   /* U+1F8D5 */
  0x0002,   /* U+1F8D6 */
  0x0002,   /* U+1F8D7 */
  0x0002,   /* U+1F8D8 */
  0x0002,   /* U+1F8D9 */
  0x0002,   /* U+1F8DA */
  0x0002,   /* U+1F8DB */
  0x0002,   /* U+1F8DC */
  0x0002,   /* U+1F8DD */
  0x0002,   /* U+1F8DE */
  0x0002,   /* U+1F8DF */
  0x0002,   /* U+1F8E0 */
  0x0002,   /* U+1F8E1 */
  0x0002,   /* U+1F8E2 */
  0x0002,   /* U+1F8E3 */
  0x0002,   /* U+1F8E4 */
  0x0002,   /* U+1F8E5 */
  0x0002,   /* U+1F8E6 */
  0x0002,   /* U+1F8E7 */
  0x0002,   /* U+1F8E8 */
  0x0002,   /* U+1F8E9 */
  0x0002,   /* U+1F8EA */
  0x0002,   /* U+1F8EB */
  0x0002,   /* U+1F8EC */
  0x0002,   /* U+1F8ED */
  0x0002,   /* U+1F8EE */
  0x0002,   /* U+1F8EF */
  0x0002,   /* U+1F8F0 */
  0x0002,   /* U+1F8F1 */
  0x0002,   /* U+1F8F2 */
  0x0002,   /* U+1F8F3 */
  0x0002,   /* U+1F8F4 */
  0x0002,   /* U+1F8F5 */
  0x0002,   /* U+1F8F6 */
  0x0002,   /* U+1F8F7 */
  0x0002,   /* U+1F8F8 */
  0x0002,   /* U+1F8F9 */
  0x0002,   /* U+1F8FA */
  0x0002,   /* U+1F8FB */
  0x0002,   /* U+1F8FC */
  0x0002,   /* U+1F8FD */
  0x0002,   /* U+1F8FE */
  0x0002,   /* U+1F8FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+1F800 */
  0x0000,   /* U+1F801 */
  0x0000,   /* U+1F802 */
  0x0000,   /* U+1F803 */
  0x0000,   /* U+1F804 */
  0x0000,   /* U+1F805 */
  0x0000,   /* U+1F806 */
  0x0000,   /* U+1F807 */
  0x0000,   /* U+1F808 */
  0x0000,   /* U+1F809 */
  0x0000,   /* U+1F80A */
  0x0000,   /* U+1F80B */
  0xF80C,   /* U+1F80C */
  0xF80D,   /* U+1F80D */
  0xF80E,   /* U+1F80E */
  0xF80F,   /* U+1F80F */
  0x0000,   /* U+1F810 */
  0x0000,   /* U+1F811 */
  0x0000,   /* U+1F812 */
  0x0000,   /* U+1F813 */
  0x0000,   /* U+1F814 */
  0x0000,   /* U+1F815 */
  0x0000,   /* U+1F816 */
  0x0000,   /* U+1F817 */
  0x0000,   /* U+1F818 */
  0x0000,   /* U+1F819 */
  0x0000,   /* U+1F81A */
  0x0000,   /* U+1F81B */
  0x0000,   /* U+1F81C */
  0x0000,   /* U+1F81D */
  0x0000,   /* U+1F81E */
  0x0000,   /* U+1F81F */
  0x0000,   /* U+1F820 */
  0x0000,   /* U+1F821 */
  0x0000,   /* U+1F822 */
  0x0000,   /* U+1F823 */
  0x0000,   /* U+1F824 */
  0x0000,   /* U+1F825 */
  0x0000,   /* U+1F826 */
  0x0000,   /* U+1F827 */
  0x0000,   /* U+1F828 */
  0x0000,   /* U+1F829 */
  0x0000,   /* U+1F82A */
  0x0000,   /* U+1F82B */
  0x0000,   /* U+1F82C */
  0x0000,   /* U+1F82D */
  0x0000,   /* U+1F82E */
  0x0000,   /* U+1F82F */
  0x0000,   /* U+1F830 */
  0x0000,   /* U+1F831 */
  0x0000,   /* U+1F832 */
  0x0000,   /* U+1F833 */
  0x0000,   /* U+1F834 */
  0x0000,   /* U+1F835 */
  0x0000,   /* U+1F836 */
  0x0000,   /* U+1F837 */
  0x0000,   /* U+1F838 */
  0x0000,   /* U+1F839 */
  0x0000,   /* U+1F83A */
  0x0000,   /* U+1F83B */
  0x0000,   /* U+1F83C */
  0x0000,   /* U+1F83D */
  0x0000,   /* U+1F83E */
  0x0000,   /* U+1F83F */
  0x0000,   /* U+1F840 */
  0x0000,   /* U+1F841 */
  0x0000,   /* U+1F842 */
  0x0000,   /* U+1F843 */
  0x0000,   /* U+1F844 */
  0x0000,   /* U+1F845 */
  0x0000,   /* U+1F846 */
  0x0000,   /* U+1F847 */
  0xF848,   /* U+1F848 */
  0xF849,   /* U+1F849 */
  0xF84A,   /* U+1F84A */
  0xF84B,   /* U+1F84B */
  0xF84C,   /* U+1F84C */
  0xF84D,   /* U+1F84D */
  0xF84E,   /* U+1F84E */
  0xF84F,   /* U+1F84F */
  0x0000,   /* U+1F850 */
  0x0000,   /* U+1F851 */
  0x0000,   /* U+1F852 */
  0x0000,   /* U+1F853 */
  0x0000,   /* U+1F854 */
  0x0000,   /* U+1F855 */
  0x0000,   /* U+1F856 */
  0x0000,   /* U+1F857 */
  0x0000,   /* U+1F858 */
  0x0000,   /* U+1F859 */
  0xF85A,   /* U+1F85A */
  0xF85B,   /* U+1F85B */
  0xF85C,   /* U+1F85C */
  0xF85D,   /* U+1F85D */
  0xF85E,   /* U+1F85E */
  0xF85F,   /* U+1F85F */
  0x0000,   /* U+1F860 */
  0x0000,   /* U+1F861 */
  0x0000,   /* U+1F862 */
  0x0000,   /* U+1F863 */
  0x0000,   /* U+1F864 */
  0x0000,   /* U+1F865 */
  0x0000,   /* U+1F866 */
  0x0000,   /* U+1F867 */
  0x0000,   /* U+1F868 */
  0x0000,   /* U+1F869 */
  0x0000,   /* U+1F86A */
  0x0000,   /* U+1F86B */
  0x0000,   /* U+1F86C */
  0x0000,   /* U+1F86D */
  0x0000,   /* U+1F86E */
  0x0000,   /* U+1F86F */
  0x0000,   /* U+1F870 */
  0x0000,   /* U+1F871 */
  0x0000,   /* U+1F872 */
  0x0000,   /* U+1F873 */
  0x0000,   /* U+1F874 */
  0x0000,   /* U+1F875 */
  0x0000,   /* U+1F876 */
  0x0000,   /* U+1F877 */
  0x0000,   /* U+1F878 */
  0x0000,   /* U+1F879 */
  0x0000,   /* U+1F87A */
  0x0000,   /* U+1F87B */
  0x0000,   /* U+1F87C */
  0x0000,   /* U+1F87D */
  0x0000,   /* U+1F87E */
  0x0000,   /* U+1F87F */
  0x0000,   /* U+1F880 */
  0x0000,   /* U+1F881 */
  0x0000,   /* U+1F882 */
  0x0000,   /* U+1F883 */
  0x0000,   /* U+1F884 */
  0x0000,   /* U+1F885 */
  0x0000,   /* U+1F886 */
  0x0000,   /* U+1F887 */
  0xF888,   /* U+1F888 */
  0xF889,   /* U+1F889 */
  0xF88A,   /* U+1F88A */
  0xF88B,   /* U+1F88B */
  0xF88C,   /* U+1F88C */
  0xF88D,   /* U+1F88D */
  0xF88E,   /* U+1F88E */
  0xF88F,   /* U+1F88F */
  0x0000,   /* U+1F890 */
  0x0000,   /* U+1F891 */
  0x0000,   /* U+1F892 */
  0x0000,   /* U+1F893 */
  0x0000,   /* U+1F894 */
  0x0000,   /* U+1F895 */
  0x0000,   /* U+1F896 */
  0x0000,   /* U+1F897 */
  0x0000,   /* U+1F898 */
  0x0000,   /* U+1F899 */
  0x0000,   /* U+1F89A */
  0x0000,   /* U+1F89B */
  0x0000,   /* U+1F89C */
  0x0000,   /* U+1F89D */
  0x0000,   /* U+1F89E */
  0x0000,   /* U+1F89F */
  0x0000,   /* U+1F8A0 */
  0x0000,   /* U+1F8A1 */
  0x0000,   /* U+1F8A2 */
  0x0000,   /* U+1F8A3 */
  0x0000,   /* U+1F8A4 */
  0x0000,   /* U+1F8A5 */
  0x0000,   /* U+1F8A6 */
  0x0000,   /* U+1F8A7 */
  0x0000,   /* U+1F8A8 */
  0x0000,   /* U+1F8A9 */
  0x0000,   /* U+1F8AA */
  0x0000,   /* U+1F8AB */
  0x0000,   /* U+1F8AC */
  0x0000,   /* U+1F8AD */
  0xF8AE,   /* U+1F8AE */
  0xF8AF,   /* U+1F8AF */
  0xF8B0,   /* U+1F8B0 */
  0xF8B1,   /* U+1F8B1 */
  0xF8B2,   /* U+1F8B2 */
  0xF8B3,   /* U+1F8B3 */
  0xF8B4,   /* U+1F8B4 */
  0xF8B5,   /* U+1F8B5 */
  0xF8B6,   /* U+1F8B6 */
  0xF8B7,   /* U+1F8B7 */
  0xF8B8,   /* U+1F8B8 */
  0xF8B9,   /* U+1F8B9 */
  0xF8BA,   /* U+1F8BA */
  0xF8BB,   /* U+1F8BB */
  0xF8BC,   /* U+1F8BC */
  0xF8BD,   /* U+1F8BD */
  0xF8BE,   /* U+1F8BE */
  0xF8BF,   /* U+1F8BF */
  0xF8C0,   /* U+1F8C0 */
  0xF8C1,   /* U+1F8C1 */
  0xF8C2,   /* U+1F8C2 */
  0xF8C3,   /* U+1F8C3 */
  0xF8C4,   /* U+1F8C4 */
  0xF8C5,   /* U+1F8C5 */
  0xF8C6,   /* U+1F8C6 */
  0xF8C7,   /* U+1F8C7 */
  0xF8C8,   /* U+1F8C8 */
  0xF8C9,   /* U+1F8C9 */
  0xF8CA,   /* U+1F8CA */
  0xF8CB,   /* U+1F8CB */
  0xF8CC,   /* U+1F8CC */
  0xF8CD,   /* U+1F8CD */
  0xF8CE,   /* U+1F8CE */
  0xF8CF,   /* U+1F8CF */
  0xF8D0,   /* U+1F8D0 */
  0xF8D1,   /* U+1F8D1 */
  0xF8D2,   /* U+1F8D2 */
  0xF8D3,   /* U+1F8D3 */
  0xF8D4,   /* U+1F8D4 */
  0xF8D5,   /* U+1F8D5 */
  0xF8D6,   /* U+1F8D6 */
  0xF8D7,   /* U+1F8D7 */
  0xF8D8,   /* U+1F8D8 */
  0xF8D9,   /* U+1F8D9 */
  0xF8DA,   /* U+1F8DA */
  0xF8DB,   /* U+1F8DB */
  0xF8DC,   /* U+1F8DC */
  0xF8DD,   /* U+1F8DD */
  0xF8DE,   /* U+1F8DE */
  0xF8DF,   /* U+1F8DF */
  0xF8E0,   /* U+1F8E0 */
  0xF8E1,   /* U+1F8E1 */
  0xF8E2,   /* U+1F8E2 */
  0xF8E3,   /* U+1F8E3 */
  0xF8E4,   /* U+1F8E4 */
  0xF8E5,   /* U+1F8E5 */
  0xF8E6,   /* U+1F8E6 */
  0xF8E7,   /* U+1F8E7 */
  0xF8E8,   /* U+1F8E8 */
  0xF8E9,   /* U+1F8E9 */
  0xF8EA,   /* U+1F8EA */
  0xF8EB,   /* U+1F8EB */
  0xF8EC,   /* U+1F8EC */
  0xF8ED,   /* U+1F8ED */
  0xF8EE,   /* U+1F8EE */
  0xF8EF,   /* U+1F8EF */
  0xF8F0,   /* U+1F8F0 */
  0xF8F1,   /* U+1F8F1 */
  0xF8F2,   /* U+1F8F2 */
  0xF8F3,   /* U+1F8F3 */
  0xF8F4,   /* U+1F8F4 */
  0xF8F5,   /* U+1F8F5 */
  0xF8F6,   /* U+1F8F6 */
  0xF8F7,   /* U+1F8F7 */
  0xF8F8,   /* U+1F8F8 */
  0xF8F9,   /* U+1F8F9 */
  0xF8FA,   /* U+1F8FA */
  0xF8FB,   /* U+1F8FB */
  0xF8FC,   /* U+1F8FC */
  0xF8FD,   /* U+1F8FD */
  0xF8FE,   /* U+1F8FE */
  0xF8FF,   /* U+1F8FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1F800 */
  0x0000,   /* U+1F801 */
  0x0000,   /* U+1F802 */
  0x0000,   /* U+1F803 */
  0x0000,   /* U+1F804 */
  0x0000,   /* U+1F805 */
  0x0000,   /* U+1F806 */
  0x0000,   /* U+1F807 */
  0x0000,   /* U+1F808 */
  0x0000,   /* U+1F809 */
  0x0000,   /* U+1F80A */
  0x0000,   /* U+1F80B */
  0x0000,   /* U+1F80C */
  0x0000,   /* U+1F80D */
  0x0000,   /* U+1F80E */
  0x0000,   /* U+1F80F */
  0x0000,   /* U+1F810 */
  0x0000,   /* U+1F811 */
  0x0000,   /* U+1F812 */
  0x0000,   /* U+1F813 */
  0x0000,   /* U+1F814 */
  0x0000,   /* U+1F815 */
  0x0000,   /* U+1F816 */
  0x0000,   /* U+1F817 */
  0x0000,   /* U+1F818 */
  0x0000,   /* U+1F819 */
  0x0000,   /* U+1F81A */
  0x0000,   /* U+1F81B */
  0x0000,   /* U+1F81C */
  0x0000,   /* U+1F81D */
  0x0000,   /* U+1F81E */
  0x0000,   /* U+1F81F */
  0x0000,   /* U+1F820 */
  0x0000,   /* U+1F821 */
  0x0000,   /* U+1F822 */
  0x0000,   /* U+1F823 */
  0x0000,   /* U+1F824 */
  0x0000,   /* U+1F825 */
  0x0000,   /* U+1F826 */
  0x0000,   /* U+1F827 */
  0x0000,   /* U+1F828 */
  0x0000,   /* U+1F829 */
  0x0000,   /* U+1F82A */
  0x0000,   /* U+1F82B */
  0x0000,   /* U+1F82C */
  0x0000,   /* U+1F82D */
  0x0000,   /* U+1F82E */
  0x0000,   /* U+1F82F */
  0x0000,   /* U+1F830 */
  0x0000,   /* U+1F831 */
  0x0000,   /* U+1F832 */
  0x0000,   /* U+1F833 */
  0x0000,   /* U+1F834 */
  0x0000,   /* U+1F835 */
  0x0000,   /* U+1F836 */
  0x0000,   /* U+1F837 */
  0x0000,   /* U+1F838 */
  0x0000,   /* U+1F839 */
  0x0000,   /* U+1F83A */
  0x0000,   /* U+1F83B */
  0x0000,   /* U+1F83C */
  0x0000,   /* U+1F83D */
  0x0000,   /* U+1F83E */
  0x0000,   /* U+1F83F */
  0x0000,   /* U+1F840 */
  0x0000,   /* U+1F841 */
  0x0000,   /* U+1F842 */
  0x0000,   /* U+1F843 */
  0x0000,   /* U+1F844 */
  0x0000,   /* U+1F845 */
  0x0000,   /* U+1F846 */
  0x0000,   /* U+1F847 */
  0x0000,   /* U+1F848 */
  0x0000,   /* U+1F849 */
  0x0000,   /* U+1F84A */
  0x0000,   /* U+1F84B */
  0x0000,   /* U+1F84C */
  0x0000,   /* U+1F84D */
  0x0000,   /* U+1F84E */
  0x0000,   /* U+1F84F */
  0x0000,   /* U+1F850 */
  0x0000,   /* U+1F851 */
  0x0000,   /* U+1F852 */
  0x0000,   /* U+1F853 */
  0x0000,   /* U+1F854 */
  0x0000,   /* U+1F855 */
  0x0000,   /* U+1F856 */
  0x0000,   /* U+1F857 */
  0x0000,   /* U+1F858 */
  0x0000,   /* U+1F859 */
  0x0000,   /* U+1F85A */
  0x0000,   /* U+1F85B */
  0x0000,   /* U+1F85C */
  0x0000,   /* U+1F85D */
  0x0000,   /* U+1F85E */
  0x0000,   /* U+1F85F */
  0x0000,   /* U+1F860 */
  0x0000,   /* U+1F861 */
  0x0000,   /* U+1F862 */
  0x0000,   /* U+1F863 */
  0x0000,   /* U+1F864 */
  0x0000,   /* U+1F865 */
  0x0000,   /* U+1F866 */
  0x0000,   /* U+1F867 */
  0x0000,   /* U+1F868 */
  0x0000,   /* U+1F869 */
  0x0000,   /* U+1F86A */
  0x0000,   /* U+1F86B */
  0x0000,   /* U+1F86C */
  0x0000,   /* U+1F86D */
  0x0000,   /* U+1F86E */
  0x0000,   /* U+1F86F */
  0x0000,   /* U+1F870 */
  0x0000,   /* U+1F871 */
  0x0000,   /* U+1F872 */
  0x0000,   /* U+1F873 */
  0x0000,   /* U+1F874 */
  0x0000,   /* U+1F875 */
  0x0000,   /* U+1F876 */
  0x0000,   /* U+1F877 */
  0x0000,   /* U+1F878 */
  0x0000,   /* U+1F879 */
  0x0000,   /* U+1F87A */
  0x0000,   /* U+1F87B */
  0x0000,   /* U+1F87C */
  0x0000,   /* U+1F87D */
  0x0000,   /* U+1F87E */
  0x0000,   /* U+1F87F */
  0x0000,   /* U+1F880 */
  0x0000,   /* U+1F881 */
  0x0000,   /* U+1F882 */
  0x0000,   /* U+1F883 */
  0x0000,   /* U+1F884 */
  0x0000,   /* U+1F885 */
  0x0000,   /* U+1F886 */
  0x0000,   /* U+1F887 */
  0x0000,   /* U+1F888 */
  0x0000,   /* U+1F889 */
  0x0000,   /* U+1F88A */
  0x0000,   /* U+1F88B */
  0x0000,   /* U+1F88C */
  0x0000,   /* U+1F88D */
  0x0000,   /* U+1F88E */
  0x0000,   /* U+1F88F */
  0x0000,   /* U+1F890 */
  0x0000,   /* U+1F891 */
  0x0000,   /* U+1F892 */
  0x0000,   /* U+1F893 */
  0x0000,   /* U+1F894 */
  0x0000,   /* U+1F895 */
  0x0000,   /* U+1F896 */
  0x0000,   /* U+1F897 */
  0x0000,   /* U+1F898 */
  0x0000,   /* U+1F899 */
  0x0000,   /* U+1F89A */
  0x0000,   /* U+1F89B */
  0x0000,   /* U+1F89C */
  0x0000,   /* U+1F89D */
  0x0000,   /* U+1F89E */
  0x0000,   /* U+1F89F */
  0x0000,   /* U+1F8A0 */
  0x0000,   /* U+1F8A1 */
  0x0000,   /* U+1F8A2 */
  0x0000,   /* U+1F8A3 */
  0x0000,   /* U+1F8A4 */
  0x0000,   /* U+1F8A5 */
  0x0000,   /* U+1F8A6 */
  0x0000,   /* U+1F8A7 */
  0x0000,   /* U+1F8A8 */
  0x0000,   /* U+1F8A9 */
  0x0000,   /* U+1F8AA */
  0x0000,   /* U+1F8AB */
  0x0000,   /* U+1F8AC */
  0x0000,   /* U+1F8AD */
  0x0000,   /* U+1F8AE */
  0x0000,   /* U+1F8AF */
  0x0000,   /* U+1F8B0 */
  0x0000,   /* U+1F8B1 */
  0x0000,   /* U+1F8B2 */
  0x0000,   /* U+1F8B3 */
  0x0000,   /* U+1F8B4 */
  0x0000,   /* U+1F8B5 */
  0x0000,   /* U+1F8B6 */
  0x0000,   /* U+1F8B7 */
  0x0000,   /* U+1F8B8 */
  0x0000,   /* U+1F8B9 */
  0x0000,   /* U+1F8BA */
  0x0000,   /* U+1F8BB */
  0x0000,   /* U+1F8BC */
  0x0000,   /* U+1F8BD */
  0x0000,   /* U+1F8BE */
  0x0000,   /* U+1F8BF */
  0x0000,   /* U+1F8C0 */
  0x0000,   /* U+1F8C1 */
  0x0000,   /* U+1F8C2 */
  0x0000,   /* U+1F8C3 */
  0x0000,   /* U+1F8C4 */
  0x0000,   /* U+1F8C5 */
  0x0000,   /* U+1F8C6 */
  0x0000,   /* U+1F8C7 */
  0x0000,   /* U+1F8C8 */
  0x0000,   /* U+1F8C9 */
  0x0000,   /* U+1F8CA */
  0x0000,   /* U+1F8CB */
  0x0000,   /* U+1F8CC */
  0x0000,   /* U+1F8CD */
  0x0000,   /* U+1F8CE */
  0x0000,   /* U+1F8CF */
  0x0000,   /* U+1F8D0 */
  0x0000,   /* U+1F8D1 */
  0x0000,   /* U+1F8D2 */
  0x0000,   /* U+1F8D3 */
  0x0000,   /* U+1F8D4 */
  0x0000,   /* U+1F8D5 */
  0x0000,   /* U+1F8D6 */
  0x0000,   /* U+1F8D7 */
  0x0000,   /* U+1F8D8 */
  0x0000,   /* U+1F8D9 */
  0x0000,   /* U+1F8DA */
  0x0000,   /* U+1F8DB */
  0x0000,   /* U+1F8DC */
  0x0000,   /* U+1F8DD */
  0x0000,   /* U+1F8DE */
  0x0000,   /* U+1F8DF */
  0x0000,   /* U+1F8E0 */
  0x0000,   /* U+1F8E1 */
  0x0000,   /* U+1F8E2 */
  0x0000,   /* U+1F8E3 */
  0x0000,   /* U+1F8E4 */
  0x0000,   /* U+1F8E5 */
  0x0000,   /* U+1F8E6 */
  0x0000,   /* U+1F8E7 */
  0x0000,   /* U+1F8E8 */
  0x0000,   /* U+1F8E9 */
  0x0000,   /* U+1F8EA */
  0x0000,   /* U+1F8EB */
  0x0000,   /* U+1F8EC */
  0x0000,   /* U+1F8ED */
  0x0000,   /* U+1F8EE */
  0x0000,   /* U+1F8EF */
  0x0000,   /* U+1F8F0 */
  0x0000,   /* U+1F8F1 */
  0x0000,   /* U+1F8F2 */
  0x0000,   /* U+1F8F3 */
  0x0000,   /* U+1F8F4 */
  0x0000,   /* U+1F8F5 */
  0x0000,   /* U+1F8F6 */
  0x0000,   /* U+1F8F7 */
  0x0000,   /* U+1F8F8 */
  0x0000,   /* U+1F8F9 */
  0x0000,   /* U+1F8FA */
  0x0000,   /* U+1F8FB */
  0x0000,   /* U+1F8FC */
  0x0000,   /* U+1F8FD */
  0x0000,   /* U+1F8FE */
  0x0000,   /* U+1F8FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1F800 */
  0x0000,   /* U+1F801 */
  0x0000,   /* U+1F802 */
  0x0000,   /* U+1F803 */
  0x0000,   /* U+1F804 */
  0x0000,   /* U+1F805 */
  0x0000,   /* U+1F806 */
  0x0000,   /* U+1F807 */
  0x0000,   /* U+1F808 */
  0x0000,   /* U+1F809 */
  0x0000,   /* U+1F80A */
  0x0000,   /* U+1F80B */
  0x0000,   /* U+1F80C */
  0x0000,   /* U+1F80D */
  0x0000,   /* U+1F80E */
  0x0000,   /* U+1F80F */
  0x0000,   /* U+1F810 */
  0x0000,   /* U+1F811 */
  0x0000,   /* U+1F812 */
  0x0000,   /* U+1F813 */
  0x0000,   /* U+1F814 */
  0x0000,   /* U+1F815 */
  0x0000,   /* U+1F816 */
  0x0000,   /* U+1F817 */
  0x0000,   /* U+1F818 */
  0x0000,   /* U+1F819 */
  0x0000,   /* U+1F81A */
  0x0000,   /* U+1F81B */
  0x0000,   /* U+1F81C */
  0x0000,   /* U+1F81D */
  0x0000,   /* U+1F81E */
  0x0000,   /* U+1F81F */
  0x0000,   /* U+1F820 */
  0x0000,   /* U+1F821 */
  0x0000,   /* U+1F822 */
  0x0000,   /* U+1F823 */
  0x0000,   /* U+1F824 */
  0x0000,   /* U+1F825 */
  0x0000,   /* U+1F826 */
  0x0000,   /* U+1F827 */
  0x0000,   /* U+1F828 */
  0x0000,   /* U+1F829 */
  0x0000,   /* U+1F82A */
  0x0000,   /* U+1F82B */
  0x0000,   /* U+1F82C */
  0x0000,   /* U+1F82D */
  0x0000,   /* U+1F82E */
  0x0000,   /* U+1F82F */
  0x0000,   /* U+1F830 */
  0x0000,   /* U+1F831 */
  0x0000,   /* U+1F832 */
  0x0000,   /* U+1F833 */
  0x0000,   /* U+1F834 */
  0x0000,   /* U+1F835 */
  0x0000,   /* U+1F836 */
  0x0000,   /* U+1F837 */
  0x0000,   /* U+1F838 */
  0x0000,   /* U+1F839 */
  0x0000,   /* U+1F83A */
  0x0000,   /* U+1F83B */
  0x0000,   /* U+1F83C */
  0x0000,   /* U+1F83D */
  0x0000,   /* U+1F83E */
  0x0000,   /* U+1F83F */
  0x0000,   /* U+1F840 */
  0x0000,   /* U+1F841 */
  0x0000,   /* U+1F842 */
  0x0000,   /* U+1F843 */
  0x0000,   /* U+1F844 */
  0x0000,   /* U+1F845 */
  0x0000,   /* U+1F846 */
  0x0000,   /* U+1F847 */
  0x0000,   /* U+1F848 */
  0x0000,   /* U+1F849 */
  0x0000,   /* U+1F84A */
  0x0000,   /* U+1F84B */
  0x0000,   /* U+1F84C */
  0x0000,   /* U+1F84D */
  0x0000,   /* U+1F84E */
  0x0000,   /* U+1F84F */
  0x0000,   /* U+1F850 */
  0x0000,   /* U+1F851 */
  0x0000,   /* U+1F852 */
  0x0000,   /* U+1F853 */
  0x0000,   /* U+1F854 */
  0x0000,   /* U+1F855 */
  0x0000,   /* U+1F856 */
  0x0000,   /* U+1F857 */
  0x0000,   /* U+1F858 */
  0x0000,   /* U+1F859 */
  0x0000,   /* U+1F85A */
  0x0000,   /* U+1F85B */
  0x0000,   /* U+1F85C */
  0x0000,   /* U+1F85D */
  0x0000,   /* U+1F85E */
  0x0000,   /* U+1F85F */
  0x0000,   /* U+1F860 */
  0x0000,   /* U+1F861 */
  0x0000,   /* U+1F862 */
  0x0000,   /* U+1F863 */
  0x0000,   /* U+1F864 */
  0x0000,   /* U+1F865 */
  0x0000,   /* U+1F866 */
  0x0000,   /* U+1F867 */
  0x0000,   /* U+1F868 */
  0x0000,   /* U+1F869 */
  0x0000,   /* U+1F86A */
  0x0000,   /* U+1F86B */
  0x0000,   /* U+1F86C */
  0x0000,   /* U+1F86D */
  0x0000,   /* U+1F86E */
  0x0000,   /* U+1F86F */
  0x0000,   /* U+1F870 */
  0x0000,   /* U+1F871 */
  0x0000,   /* U+1F872 */
  0x0000,   /* U+1F873 */
  0x0000,   /* U+1F874 */
  0x0000,   /* U+1F875 */
  0x0000,   /* U+1F876 */
  0x0000,   /* U+1F877 */
  0x0000,   /* U+1F878 */
  0x0000,   /* U+1F879 */
  0x0000,   /* U+1F87A */
  0x0000,   /* U+1F87B */
  0x0000,   /* U+1F87C */
  0x0000,   /* U+1F87D */
  0x0000,   /* U+1F87E */
  0x0000,   /* U+1F87F */
  0x0000,   /* U+1F880 */
  0x0000,   /* U+1F881 */
  0x0000,   /* U+1F882 */
  0x0000,   /* U+1F883 */
  0x0000,   /* U+1F884 */
  0x0000,   /* U+1F885 */
  0x0000,   /* U+1F886 */
  0x0000,   /* U+1F887 */
  0x0000,   /* U+1F888 */
  0x0000,   /* U+1F889 */
  0x0000,   /* U+1F88A */
  0x0000,   /* U+1F88B */
  0x0000,   /* U+1F88C */
  0x0000,   /* U+1F88D */
  0x0000,   /* U+1F88E */
  0x0000,   /* U+1F88F */
  0x0000,   /* U+1F890 */
  0x0000,   /* U+1F891 */
  0x0000,   /* U+1F892 */
  0x0000,   /* U+1F893 */
  0x0000,   /* U+1F894 */
  0x0000,   /* U+1F895 */
  0x0000,   /* U+1F896 */
  0x0000,   /* U+1F897 */
  0x0000,   /* U+1F898 */
  0x0000,   /* U+1F899 */
  0x0000,   /* U+1F89A */
  0x0000,   /* U+1F89B */
  0x0000,   /* U+1F89C */
  0x0000,   /* U+1F89D */
  0x0000,   /* U+1F89E */
  0x0000,   /* U+1F89F */
  0x0000,   /* U+1F8A0 */
  0x0000,   /* U+1F8A1 */
  0x0000,   /* U+1F8A2 */
  0x0000,   /* U+1F8A3 */
  0x0000,   /* U+1F8A4 */
  0x0000,   /* U+1F8A5 */
  0x0000,   /* U+1F8A6 */
  0x0000,   /* U+1F8A7 */
  0x0000,   /* U+1F8A8 */
  0x0000,   /* U+1F8A9 */
  0x0000,   /* U+1F8AA */
  0x0000,   /* U+1F8AB */
  0x0000,   /* U+1F8AC */
  0x0000,   /* U+1F8AD */
  0x0000,   /* U+1F8AE */
  0x0000,   /* U+1F8AF */
  0x0000,   /* U+1F8B0 */
  0x0000,   /* U+1F8B1 */
  0x0000,   /* U+1F8B2 */
  0x0000,   /* U+1F8B3 */
  0x0000,   /* U+1F8B4 */
  0x0000,   /* U+1F8B5 */
  0x0000,   /* U+1F8B6 */
  0x0000,   /* U+1F8B7 */
  0x0000,   /* U+1F8B8 */
  0x0000,   /* U+1F8B9 */
  0x0000,   /* U+1F8BA */
  0x0000,   /* U+1F8BB */
  0x0000,   /* U+1F8BC */
  0x0000,   /* U+1F8BD */
  0x0000,   /* U+1F8BE */
  0x0000,   /* U+1F8BF */
  0x0000,   /* U+1F8C0 */
  0x0000,   /* U+1F8C1 */
  0x0000,   /* U+1F8C2 */
  0x0000,   /* U+1F8C3 */
  0x0000,   /* U+1F8C4 */
  0x0000,   /* U+1F8C5 */
  0x0000,   /* U+1F8C6 */
  0x0000,   /* U+1F8C7 */
  0x0000,   /* U+1F8C8 */
  0x0000,   /* U+1F8C9 */
  0x0000,   /* U+1F8CA */
  0x0000,   /* U+1F8CB */
  0x0000,   /* U+1F8CC */
  0x0000,   /* U+1F8CD */
  0x0000,   /* U+1F8CE */
  0x0000,   /* U+1F8CF */
  0x0000,   /* U+1F8D0 */
  0x0000,   /* U+1F8D1 */
  0x0000,   /* U+1F8D2 */
  0x0000,   /* U+1F8D3 */
  0x0000,   /* U+1F8D4 */
  0x0000,   /* U+1F8D5 */
  0x0000,   /* U+1F8D6 */
  0x0000,   /* U+1F8D7 */
  0x0000,   /* U+1F8D8 */
  0x0000,   /* U+1F8D9 */
  0x0000,   /* U+1F8DA */
  0x0000,   /* U+1F8DB */
  0x0000,   /* U+1F8DC */
  0x0000,   /* U+1F8DD */
  0x0000,   /* U+1F8DE */
  0x0000,   /* U+1F8DF */
  0x0000,   /* U+1F8E0 */
  0x0000,   /* U+1F8E1 */
  0x0000,   /* U+1F8E2 */
  0x0000,   /* U+1F8E3 */
  0x0000,   /* U+1F8E4 */
  0x0000,   /* U+1F8E5 */
  0x0000,   /* U+1F8E6 */
  0x0000,   /* U+1F8E7 */
  0x0000,   /* U+1F8E8 */
  0x0000,   /* U+1F8E9 */
  0x0000,   /* U+1F8EA */
  0x0000,   /* U+1F8EB */
  0x0000,   /* U+1F8EC */
  0x0000,   /* U+1F8ED */
  0x0000,   /* U+1F8EE */
  0x0000,   /* U+1F8EF */
  0x0000,   /* U+1F8F0 */
  0x0000,   /* U+1F8F1 */
  0x0000,   /* U+1F8F2 */
  0x0000,   /* U+1F8F3 */
  0x0000,   /* U+1F8F4 */
  0x0000,   /* U+1F8F5 */
  0x0000,   /* U+1F8F6 */
  0x0000,   /* U+1F8F7 */
  0x0000,   /* U+1F8F8 */
  0x0000,   /* U+1F8F9 */
  0x0000,   /* U+1F8FA */
  0x0000,   /* U+1F8FB */
  0x0000,   /* U+1F8FC */
  0x0000,   /* U+1F8FD */
  0x0000,   /* U+1F8FE */
  0x0000,   /* U+1F8FF */
};

uint16 uca900_p1F9[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBC3,   /* U+1F900 */
  0xFBC3,   /* U+1F901 */
  0xFBC3,   /* U+1F902 */
  0xFBC3,   /* U+1F903 */
  0xFBC3,   /* U+1F904 */
  0xFBC3,   /* U+1F905 */
  0xFBC3,   /* U+1F906 */
  0xFBC3,   /* U+1F907 */
  0xFBC3,   /* U+1F908 */
  0xFBC3,   /* U+1F909 */
  0xFBC3,   /* U+1F90A */
  0xFBC3,   /* U+1F90B */
  0xFBC3,   /* U+1F90C */
  0xFBC3,   /* U+1F90D */
  0xFBC3,   /* U+1F90E */
  0xFBC3,   /* U+1F90F */
  0x15A9,   /* U+1F910 */
  0x15AA,   /* U+1F911 */
  0x15AB,   /* U+1F912 */
  0x15AC,   /* U+1F913 */
  0x15AD,   /* U+1F914 */
  0x15AE,   /* U+1F915 */
  0x15AF,   /* U+1F916 */
  0x15B0,   /* U+1F917 */
  0x15B1,   /* U+1F918 */
  0x15B2,   /* U+1F919 */
  0x15B3,   /* U+1F91A */
  0x15B4,   /* U+1F91B */
  0x15B5,   /* U+1F91C */
  0x15B6,   /* U+1F91D */
  0x15B7,   /* U+1F91E */
  0xFBC3,   /* U+1F91F */
  0x15B8,   /* U+1F920 */
  0x15B9,   /* U+1F921 */
  0x15BA,   /* U+1F922 */
  0x15BB,   /* U+1F923 */
  0x15BC,   /* U+1F924 */
  0x15BD,   /* U+1F925 */
  0x15BE,   /* U+1F926 */
  0x15BF,   /* U+1F927 */
  0xFBC3,   /* U+1F928 */
  0xFBC3,   /* U+1F929 */
  0xFBC3,   /* U+1F92A */
  0xFBC3,   /* U+1F92B */
  0xFBC3,   /* U+1F92C */
  0xFBC3,   /* U+1F92D */
  0xFBC3,   /* U+1F92E */
  0xFBC3,   /* U+1F92F */
  0x15C0,   /* U+1F930 */
  0xFBC3,   /* U+1F931 */
  0xFBC3,   /* U+1F932 */
  0x15C1,   /* U+1F933 */
  0x15C2,   /* U+1F934 */
  0x15C3,   /* U+1F935 */
  0x15C4,   /* U+1F936 */
  0x15C5,   /* U+1F937 */
  0x15C6,   /* U+1F938 */
  0x15C7,   /* U+1F939 */
  0x15C8,   /* U+1F93A */
  0x15C9,   /* U+1F93B */
  0x15CA,   /* U+1F93C */
  0x15CB,   /* U+1F93D */
  0x15CC,   /* U+1F93E */
  0xFBC3,   /* U+1F93F */
  0x15CD,   /* U+1F940 */
  0x15CE,   /* U+1F941 */
  0x15CF,   /* U+1F942 */
  0x15D0,   /* U+1F943 */
  0x15D1,   /* U+1F944 */
  0x15D2,   /* U+1F945 */
  0x15D3,   /* U+1F946 */
  0x15D4,   /* U+1F947 */
  0x15D5,   /* U+1F948 */
  0x15D6,   /* U+1F949 */
  0x15D7,   /* U+1F94A */
  0x15D8,   /* U+1F94B */
  0xFBC3,   /* U+1F94C */
  0xFBC3,   /* U+1F94D */
  0xFBC3,   /* U+1F94E */
  0xFBC3,   /* U+1F94F */
  0x15D9,   /* U+1F950 */
  0x15DA,   /* U+1F951 */
  0x15DB,   /* U+1F952 */
  0x15DC,   /* U+1F953 */
  0x15DD,   /* U+1F954 */
  0x15DE,   /* U+1F955 */
  0x15DF,   /* U+1F956 */
  0x15E0,   /* U+1F957 */
  0x15E1,   /* U+1F958 */
  0x15E2,   /* U+1F959 */
  0x15E3,   /* U+1F95A */
  0x15E4,   /* U+1F95B */
  0x15E5,   /* U+1F95C */
  0x15E6,   /* U+1F95D */
  0x15E7,   /* U+1F95E */
  0xFBC3,   /* U+1F95F */
  0xFBC3,   /* U+1F960 */
  0xFBC3,   /* U+1F961 */
  0xFBC3,   /* U+1F962 */
  0xFBC3,   /* U+1F963 */
  0xFBC3,   /* U+1F964 */
  0xFBC3,   /* U+1F965 */
  0xFBC3,   /* U+1F966 */
  0xFBC3,   /* U+1F967 */
  0xFBC3,   /* U+1F968 */
  0xFBC3,   /* U+1F969 */
  0xFBC3,   /* U+1F96A */
  0xFBC3,   /* U+1F96B */
  0xFBC3,   /* U+1F96C */
  0xFBC3,   /* U+1F96D */
  0xFBC3,   /* U+1F96E */
  0xFBC3,   /* U+1F96F */
  0xFBC3,   /* U+1F970 */
  0xFBC3,   /* U+1F971 */
  0xFBC3,   /* U+1F972 */
  0xFBC3,   /* U+1F973 */
  0xFBC3,   /* U+1F974 */
  0xFBC3,   /* U+1F975 */
  0xFBC3,   /* U+1F976 */
  0xFBC3,   /* U+1F977 */
  0xFBC3,   /* U+1F978 */
  0xFBC3,   /* U+1F979 */
  0xFBC3,   /* U+1F97A */
  0xFBC3,   /* U+1F97B */
  0xFBC3,   /* U+1F97C */
  0xFBC3,   /* U+1F97D */
  0xFBC3,   /* U+1F97E */
  0xFBC3,   /* U+1F97F */
  0x15E8,   /* U+1F980 */
  0x15E9,   /* U+1F981 */
  0x15EA,   /* U+1F982 */
  0x15EB,   /* U+1F983 */
  0x15EC,   /* U+1F984 */
  0x15ED,   /* U+1F985 */
  0x15EE,   /* U+1F986 */
  0x15EF,   /* U+1F987 */
  0x15F0,   /* U+1F988 */
  0x15F1,   /* U+1F989 */
  0x15F2,   /* U+1F98A */
  0x15F3,   /* U+1F98B */
  0x15F4,   /* U+1F98C */
  0x15F5,   /* U+1F98D */
  0x15F6,   /* U+1F98E */
  0x15F7,   /* U+1F98F */
  0x15F8,   /* U+1F990 */
  0x15F9,   /* U+1F991 */
  0xFBC3,   /* U+1F992 */
  0xFBC3,   /* U+1F993 */
  0xFBC3,   /* U+1F994 */
  0xFBC3,   /* U+1F995 */
  0xFBC3,   /* U+1F996 */
  0xFBC3,   /* U+1F997 */
  0xFBC3,   /* U+1F998 */
  0xFBC3,   /* U+1F999 */
  0xFBC3,   /* U+1F99A */
  0xFBC3,   /* U+1F99B */
  0xFBC3,   /* U+1F99C */
  0xFBC3,   /* U+1F99D */
  0xFBC3,   /* U+1F99E */
  0xFBC3,   /* U+1F99F */
  0xFBC3,   /* U+1F9A0 */
  0xFBC3,   /* U+1F9A1 */
  0xFBC3,   /* U+1F9A2 */
  0xFBC3,   /* U+1F9A3 */
  0xFBC3,   /* U+1F9A4 */
  0xFBC3,   /* U+1F9A5 */
  0xFBC3,   /* U+1F9A6 */
  0xFBC3,   /* U+1F9A7 */
  0xFBC3,   /* U+1F9A8 */
  0xFBC3,   /* U+1F9A9 */
  0xFBC3,   /* U+1F9AA */
  0xFBC3,   /* U+1F9AB */
  0xFBC3,   /* U+1F9AC */
  0xFBC3,   /* U+1F9AD */
  0xFBC3,   /* U+1F9AE */
  0xFBC3,   /* U+1F9AF */
  0xFBC3,   /* U+1F9B0 */
  0xFBC3,   /* U+1F9B1 */
  0xFBC3,   /* U+1F9B2 */
  0xFBC3,   /* U+1F9B3 */
  0xFBC3,   /* U+1F9B4 */
  0xFBC3,   /* U+1F9B5 */
  0xFBC3,   /* U+1F9B6 */
  0xFBC3,   /* U+1F9B7 */
  0xFBC3,   /* U+1F9B8 */
  0xFBC3,   /* U+1F9B9 */
  0xFBC3,   /* U+1F9BA */
  0xFBC3,   /* U+1F9BB */
  0xFBC3,   /* U+1F9BC */
  0xFBC3,   /* U+1F9BD */
  0xFBC3,   /* U+1F9BE */
  0xFBC3,   /* U+1F9BF */
  0x15FA,   /* U+1F9C0 */
  0xFBC3,   /* U+1F9C1 */
  0xFBC3,   /* U+1F9C2 */
  0xFBC3,   /* U+1F9C3 */
  0xFBC3,   /* U+1F9C4 */
  0xFBC3,   /* U+1F9C5 */
  0xFBC3,   /* U+1F9C6 */
  0xFBC3,   /* U+1F9C7 */
  0xFBC3,   /* U+1F9C8 */
  0xFBC3,   /* U+1F9C9 */
  0xFBC3,   /* U+1F9CA */
  0xFBC3,   /* U+1F9CB */
  0xFBC3,   /* U+1F9CC */
  0xFBC3,   /* U+1F9CD */
  0xFBC3,   /* U+1F9CE */
  0xFBC3,   /* U+1F9CF */
  0xFBC3,   /* U+1F9D0 */
  0xFBC3,   /* U+1F9D1 */
  0xFBC3,   /* U+1F9D2 */
  0xFBC3,   /* U+1F9D3 */
  0xFBC3,   /* U+1F9D4 */
  0xFBC3,   /* U+1F9D5 */
  0xFBC3,   /* U+1F9D6 */
  0xFBC3,   /* U+1F9D7 */
  0xFBC3,   /* U+1F9D8 */
  0xFBC3,   /* U+1F9D9 */
  0xFBC3,   /* U+1F9DA */
  0xFBC3,   /* U+1F9DB */
  0xFBC3,   /* U+1F9DC */
  0xFBC3,   /* U+1F9DD */
  0xFBC3,   /* U+1F9DE */
  0xFBC3,   /* U+1F9DF */
  0xFBC3,   /* U+1F9E0 */
  0xFBC3,   /* U+1F9E1 */
  0xFBC3,   /* U+1F9E2 */
  0xFBC3,   /* U+1F9E3 */
  0xFBC3,   /* U+1F9E4 */
  0xFBC3,   /* U+1F9E5 */
  0xFBC3,   /* U+1F9E6 */
  0xFBC3,   /* U+1F9E7 */
  0xFBC3,   /* U+1F9E8 */
  0xFBC3,   /* U+1F9E9 */
  0xFBC3,   /* U+1F9EA */
  0xFBC3,   /* U+1F9EB */
  0xFBC3,   /* U+1F9EC */
  0xFBC3,   /* U+1F9ED */
  0xFBC3,   /* U+1F9EE */
  0xFBC3,   /* U+1F9EF */
  0xFBC3,   /* U+1F9F0 */
  0xFBC3,   /* U+1F9F1 */
  0xFBC3,   /* U+1F9F2 */
  0xFBC3,   /* U+1F9F3 */
  0xFBC3,   /* U+1F9F4 */
  0xFBC3,   /* U+1F9F5 */
  0xFBC3,   /* U+1F9F6 */
  0xFBC3,   /* U+1F9F7 */
  0xFBC3,   /* U+1F9F8 */
  0xFBC3,   /* U+1F9F9 */
  0xFBC3,   /* U+1F9FA */
  0xFBC3,   /* U+1F9FB */
  0xFBC3,   /* U+1F9FC */
  0xFBC3,   /* U+1F9FD */
  0xFBC3,   /* U+1F9FE */
  0xFBC3,   /* U+1F9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+1F900 */
  0x0020,   /* U+1F901 */
  0x0020,   /* U+1F902 */
  0x0020,   /* U+1F903 */
  0x0020,   /* U+1F904 */
  0x0020,   /* U+1F905 */
  0x0020,   /* U+1F906 */
  0x0020,   /* U+1F907 */
  0x0020,   /* U+1F908 */
  0x0020,   /* U+1F909 */
  0x0020,   /* U+1F90A */
  0x0020,   /* U+1F90B */
  0x0020,   /* U+1F90C */
  0x0020,   /* U+1F90D */
  0x0020,   /* U+1F90E */
  0x0020,   /* U+1F90F */
  0x0020,   /* U+1F910 */
  0x0020,   /* U+1F911 */
  0x0020,   /* U+1F912 */
  0x0020,   /* U+1F913 */
  0x0020,   /* U+1F914 */
  0x0020,   /* U+1F915 */
  0x0020,   /* U+1F916 */
  0x0020,   /* U+1F917 */
  0x0020,   /* U+1F918 */
  0x0020,   /* U+1F919 */
  0x0020,   /* U+1F91A */
  0x0020,   /* U+1F91B */
  0x0020,   /* U+1F91C */
  0x0020,   /* U+1F91D */
  0x0020,   /* U+1F91E */
  0x0020,   /* U+1F91F */
  0x0020,   /* U+1F920 */
  0x0020,   /* U+1F921 */
  0x0020,   /* U+1F922 */
  0x0020,   /* U+1F923 */
  0x0020,   /* U+1F924 */
  0x0020,   /* U+1F925 */
  0x0020,   /* U+1F926 */
  0x0020,   /* U+1F927 */
  0x0020,   /* U+1F928 */
  0x0020,   /* U+1F929 */
  0x0020,   /* U+1F92A */
  0x0020,   /* U+1F92B */
  0x0020,   /* U+1F92C */
  0x0020,   /* U+1F92D */
  0x0020,   /* U+1F92E */
  0x0020,   /* U+1F92F */
  0x0020,   /* U+1F930 */
  0x0020,   /* U+1F931 */
  0x0020,   /* U+1F932 */
  0x0020,   /* U+1F933 */
  0x0020,   /* U+1F934 */
  0x0020,   /* U+1F935 */
  0x0020,   /* U+1F936 */
  0x0020,   /* U+1F937 */
  0x0020,   /* U+1F938 */
  0x0020,   /* U+1F939 */
  0x0020,   /* U+1F93A */
  0x0020,   /* U+1F93B */
  0x0020,   /* U+1F93C */
  0x0020,   /* U+1F93D */
  0x0020,   /* U+1F93E */
  0x0020,   /* U+1F93F */
  0x0020,   /* U+1F940 */
  0x0020,   /* U+1F941 */
  0x0020,   /* U+1F942 */
  0x0020,   /* U+1F943 */
  0x0020,   /* U+1F944 */
  0x0020,   /* U+1F945 */
  0x0020,   /* U+1F946 */
  0x0020,   /* U+1F947 */
  0x0020,   /* U+1F948 */
  0x0020,   /* U+1F949 */
  0x0020,   /* U+1F94A */
  0x0020,   /* U+1F94B */
  0x0020,   /* U+1F94C */
  0x0020,   /* U+1F94D */
  0x0020,   /* U+1F94E */
  0x0020,   /* U+1F94F */
  0x0020,   /* U+1F950 */
  0x0020,   /* U+1F951 */
  0x0020,   /* U+1F952 */
  0x0020,   /* U+1F953 */
  0x0020,   /* U+1F954 */
  0x0020,   /* U+1F955 */
  0x0020,   /* U+1F956 */
  0x0020,   /* U+1F957 */
  0x0020,   /* U+1F958 */
  0x0020,   /* U+1F959 */
  0x0020,   /* U+1F95A */
  0x0020,   /* U+1F95B */
  0x0020,   /* U+1F95C */
  0x0020,   /* U+1F95D */
  0x0020,   /* U+1F95E */
  0x0020,   /* U+1F95F */
  0x0020,   /* U+1F960 */
  0x0020,   /* U+1F961 */
  0x0020,   /* U+1F962 */
  0x0020,   /* U+1F963 */
  0x0020,   /* U+1F964 */
  0x0020,   /* U+1F965 */
  0x0020,   /* U+1F966 */
  0x0020,   /* U+1F967 */
  0x0020,   /* U+1F968 */
  0x0020,   /* U+1F969 */
  0x0020,   /* U+1F96A */
  0x0020,   /* U+1F96B */
  0x0020,   /* U+1F96C */
  0x0020,   /* U+1F96D */
  0x0020,   /* U+1F96E */
  0x0020,   /* U+1F96F */
  0x0020,   /* U+1F970 */
  0x0020,   /* U+1F971 */
  0x0020,   /* U+1F972 */
  0x0020,   /* U+1F973 */
  0x0020,   /* U+1F974 */
  0x0020,   /* U+1F975 */
  0x0020,   /* U+1F976 */
  0x0020,   /* U+1F977 */
  0x0020,   /* U+1F978 */
  0x0020,   /* U+1F979 */
  0x0020,   /* U+1F97A */
  0x0020,   /* U+1F97B */
  0x0020,   /* U+1F97C */
  0x0020,   /* U+1F97D */
  0x0020,   /* U+1F97E */
  0x0020,   /* U+1F97F */
  0x0020,   /* U+1F980 */
  0x0020,   /* U+1F981 */
  0x0020,   /* U+1F982 */
  0x0020,   /* U+1F983 */
  0x0020,   /* U+1F984 */
  0x0020,   /* U+1F985 */
  0x0020,   /* U+1F986 */
  0x0020,   /* U+1F987 */
  0x0020,   /* U+1F988 */
  0x0020,   /* U+1F989 */
  0x0020,   /* U+1F98A */
  0x0020,   /* U+1F98B */
  0x0020,   /* U+1F98C */
  0x0020,   /* U+1F98D */
  0x0020,   /* U+1F98E */
  0x0020,   /* U+1F98F */
  0x0020,   /* U+1F990 */
  0x0020,   /* U+1F991 */
  0x0020,   /* U+1F992 */
  0x0020,   /* U+1F993 */
  0x0020,   /* U+1F994 */
  0x0020,   /* U+1F995 */
  0x0020,   /* U+1F996 */
  0x0020,   /* U+1F997 */
  0x0020,   /* U+1F998 */
  0x0020,   /* U+1F999 */
  0x0020,   /* U+1F99A */
  0x0020,   /* U+1F99B */
  0x0020,   /* U+1F99C */
  0x0020,   /* U+1F99D */
  0x0020,   /* U+1F99E */
  0x0020,   /* U+1F99F */
  0x0020,   /* U+1F9A0 */
  0x0020,   /* U+1F9A1 */
  0x0020,   /* U+1F9A2 */
  0x0020,   /* U+1F9A3 */
  0x0020,   /* U+1F9A4 */
  0x0020,   /* U+1F9A5 */
  0x0020,   /* U+1F9A6 */
  0x0020,   /* U+1F9A7 */
  0x0020,   /* U+1F9A8 */
  0x0020,   /* U+1F9A9 */
  0x0020,   /* U+1F9AA */
  0x0020,   /* U+1F9AB */
  0x0020,   /* U+1F9AC */
  0x0020,   /* U+1F9AD */
  0x0020,   /* U+1F9AE */
  0x0020,   /* U+1F9AF */
  0x0020,   /* U+1F9B0 */
  0x0020,   /* U+1F9B1 */
  0x0020,   /* U+1F9B2 */
  0x0020,   /* U+1F9B3 */
  0x0020,   /* U+1F9B4 */
  0x0020,   /* U+1F9B5 */
  0x0020,   /* U+1F9B6 */
  0x0020,   /* U+1F9B7 */
  0x0020,   /* U+1F9B8 */
  0x0020,   /* U+1F9B9 */
  0x0020,   /* U+1F9BA */
  0x0020,   /* U+1F9BB */
  0x0020,   /* U+1F9BC */
  0x0020,   /* U+1F9BD */
  0x0020,   /* U+1F9BE */
  0x0020,   /* U+1F9BF */
  0x0020,   /* U+1F9C0 */
  0x0020,   /* U+1F9C1 */
  0x0020,   /* U+1F9C2 */
  0x0020,   /* U+1F9C3 */
  0x0020,   /* U+1F9C4 */
  0x0020,   /* U+1F9C5 */
  0x0020,   /* U+1F9C6 */
  0x0020,   /* U+1F9C7 */
  0x0020,   /* U+1F9C8 */
  0x0020,   /* U+1F9C9 */
  0x0020,   /* U+1F9CA */
  0x0020,   /* U+1F9CB */
  0x0020,   /* U+1F9CC */
  0x0020,   /* U+1F9CD */
  0x0020,   /* U+1F9CE */
  0x0020,   /* U+1F9CF */
  0x0020,   /* U+1F9D0 */
  0x0020,   /* U+1F9D1 */
  0x0020,   /* U+1F9D2 */
  0x0020,   /* U+1F9D3 */
  0x0020,   /* U+1F9D4 */
  0x0020,   /* U+1F9D5 */
  0x0020,   /* U+1F9D6 */
  0x0020,   /* U+1F9D7 */
  0x0020,   /* U+1F9D8 */
  0x0020,   /* U+1F9D9 */
  0x0020,   /* U+1F9DA */
  0x0020,   /* U+1F9DB */
  0x0020,   /* U+1F9DC */
  0x0020,   /* U+1F9DD */
  0x0020,   /* U+1F9DE */
  0x0020,   /* U+1F9DF */
  0x0020,   /* U+1F9E0 */
  0x0020,   /* U+1F9E1 */
  0x0020,   /* U+1F9E2 */
  0x0020,   /* U+1F9E3 */
  0x0020,   /* U+1F9E4 */
  0x0020,   /* U+1F9E5 */
  0x0020,   /* U+1F9E6 */
  0x0020,   /* U+1F9E7 */
  0x0020,   /* U+1F9E8 */
  0x0020,   /* U+1F9E9 */
  0x0020,   /* U+1F9EA */
  0x0020,   /* U+1F9EB */
  0x0020,   /* U+1F9EC */
  0x0020,   /* U+1F9ED */
  0x0020,   /* U+1F9EE */
  0x0020,   /* U+1F9EF */
  0x0020,   /* U+1F9F0 */
  0x0020,   /* U+1F9F1 */
  0x0020,   /* U+1F9F2 */
  0x0020,   /* U+1F9F3 */
  0x0020,   /* U+1F9F4 */
  0x0020,   /* U+1F9F5 */
  0x0020,   /* U+1F9F6 */
  0x0020,   /* U+1F9F7 */
  0x0020,   /* U+1F9F8 */
  0x0020,   /* U+1F9F9 */
  0x0020,   /* U+1F9FA */
  0x0020,   /* U+1F9FB */
  0x0020,   /* U+1F9FC */
  0x0020,   /* U+1F9FD */
  0x0020,   /* U+1F9FE */
  0x0020,   /* U+1F9FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+1F900 */
  0x0002,   /* U+1F901 */
  0x0002,   /* U+1F902 */
  0x0002,   /* U+1F903 */
  0x0002,   /* U+1F904 */
  0x0002,   /* U+1F905 */
  0x0002,   /* U+1F906 */
  0x0002,   /* U+1F907 */
  0x0002,   /* U+1F908 */
  0x0002,   /* U+1F909 */
  0x0002,   /* U+1F90A */
  0x0002,   /* U+1F90B */
  0x0002,   /* U+1F90C */
  0x0002,   /* U+1F90D */
  0x0002,   /* U+1F90E */
  0x0002,   /* U+1F90F */
  0x0002,   /* U+1F910 */
  0x0002,   /* U+1F911 */
  0x0002,   /* U+1F912 */
  0x0002,   /* U+1F913 */
  0x0002,   /* U+1F914 */
  0x0002,   /* U+1F915 */
  0x0002,   /* U+1F916 */
  0x0002,   /* U+1F917 */
  0x0002,   /* U+1F918 */
  0x0002,   /* U+1F919 */
  0x0002,   /* U+1F91A */
  0x0002,   /* U+1F91B */
  0x0002,   /* U+1F91C */
  0x0002,   /* U+1F91D */
  0x0002,   /* U+1F91E */
  0x0002,   /* U+1F91F */
  0x0002,   /* U+1F920 */
  0x0002,   /* U+1F921 */
  0x0002,   /* U+1F922 */
  0x0002,   /* U+1F923 */
  0x0002,   /* U+1F924 */
  0x0002,   /* U+1F925 */
  0x0002,   /* U+1F926 */
  0x0002,   /* U+1F927 */
  0x0002,   /* U+1F928 */
  0x0002,   /* U+1F929 */
  0x0002,   /* U+1F92A */
  0x0002,   /* U+1F92B */
  0x0002,   /* U+1F92C */
  0x0002,   /* U+1F92D */
  0x0002,   /* U+1F92E */
  0x0002,   /* U+1F92F */
  0x0002,   /* U+1F930 */
  0x0002,   /* U+1F931 */
  0x0002,   /* U+1F932 */
  0x0002,   /* U+1F933 */
  0x0002,   /* U+1F934 */
  0x0002,   /* U+1F935 */
  0x0002,   /* U+1F936 */
  0x0002,   /* U+1F937 */
  0x0002,   /* U+1F938 */
  0x0002,   /* U+1F939 */
  0x0002,   /* U+1F93A */
  0x0002,   /* U+1F93B */
  0x0002,   /* U+1F93C */
  0x0002,   /* U+1F93D */
  0x0002,   /* U+1F93E */
  0x0002,   /* U+1F93F */
  0x0002,   /* U+1F940 */
  0x0002,   /* U+1F941 */
  0x0002,   /* U+1F942 */
  0x0002,   /* U+1F943 */
  0x0002,   /* U+1F944 */
  0x0002,   /* U+1F945 */
  0x0002,   /* U+1F946 */
  0x0002,   /* U+1F947 */
  0x0002,   /* U+1F948 */
  0x0002,   /* U+1F949 */
  0x0002,   /* U+1F94A */
  0x0002,   /* U+1F94B */
  0x0002,   /* U+1F94C */
  0x0002,   /* U+1F94D */
  0x0002,   /* U+1F94E */
  0x0002,   /* U+1F94F */
  0x0002,   /* U+1F950 */
  0x0002,   /* U+1F951 */
  0x0002,   /* U+1F952 */
  0x0002,   /* U+1F953 */
  0x0002,   /* U+1F954 */
  0x0002,   /* U+1F955 */
  0x0002,   /* U+1F956 */
  0x0002,   /* U+1F957 */
  0x0002,   /* U+1F958 */
  0x0002,   /* U+1F959 */
  0x0002,   /* U+1F95A */
  0x0002,   /* U+1F95B */
  0x0002,   /* U+1F95C */
  0x0002,   /* U+1F95D */
  0x0002,   /* U+1F95E */
  0x0002,   /* U+1F95F */
  0x0002,   /* U+1F960 */
  0x0002,   /* U+1F961 */
  0x0002,   /* U+1F962 */
  0x0002,   /* U+1F963 */
  0x0002,   /* U+1F964 */
  0x0002,   /* U+1F965 */
  0x0002,   /* U+1F966 */
  0x0002,   /* U+1F967 */
  0x0002,   /* U+1F968 */
  0x0002,   /* U+1F969 */
  0x0002,   /* U+1F96A */
  0x0002,   /* U+1F96B */
  0x0002,   /* U+1F96C */
  0x0002,   /* U+1F96D */
  0x0002,   /* U+1F96E */
  0x0002,   /* U+1F96F */
  0x0002,   /* U+1F970 */
  0x0002,   /* U+1F971 */
  0x0002,   /* U+1F972 */
  0x0002,   /* U+1F973 */
  0x0002,   /* U+1F974 */
  0x0002,   /* U+1F975 */
  0x0002,   /* U+1F976 */
  0x0002,   /* U+1F977 */
  0x0002,   /* U+1F978 */
  0x0002,   /* U+1F979 */
  0x0002,   /* U+1F97A */
  0x0002,   /* U+1F97B */
  0x0002,   /* U+1F97C */
  0x0002,   /* U+1F97D */
  0x0002,   /* U+1F97E */
  0x0002,   /* U+1F97F */
  0x0002,   /* U+1F980 */
  0x0002,   /* U+1F981 */
  0x0002,   /* U+1F982 */
  0x0002,   /* U+1F983 */
  0x0002,   /* U+1F984 */
  0x0002,   /* U+1F985 */
  0x0002,   /* U+1F986 */
  0x0002,   /* U+1F987 */
  0x0002,   /* U+1F988 */
  0x0002,   /* U+1F989 */
  0x0002,   /* U+1F98A */
  0x0002,   /* U+1F98B */
  0x0002,   /* U+1F98C */
  0x0002,   /* U+1F98D */
  0x0002,   /* U+1F98E */
  0x0002,   /* U+1F98F */
  0x0002,   /* U+1F990 */
  0x0002,   /* U+1F991 */
  0x0002,   /* U+1F992 */
  0x0002,   /* U+1F993 */
  0x0002,   /* U+1F994 */
  0x0002,   /* U+1F995 */
  0x0002,   /* U+1F996 */
  0x0002,   /* U+1F997 */
  0x0002,   /* U+1F998 */
  0x0002,   /* U+1F999 */
  0x0002,   /* U+1F99A */
  0x0002,   /* U+1F99B */
  0x0002,   /* U+1F99C */
  0x0002,   /* U+1F99D */
  0x0002,   /* U+1F99E */
  0x0002,   /* U+1F99F */
  0x0002,   /* U+1F9A0 */
  0x0002,   /* U+1F9A1 */
  0x0002,   /* U+1F9A2 */
  0x0002,   /* U+1F9A3 */
  0x0002,   /* U+1F9A4 */
  0x0002,   /* U+1F9A5 */
  0x0002,   /* U+1F9A6 */
  0x0002,   /* U+1F9A7 */
  0x0002,   /* U+1F9A8 */
  0x0002,   /* U+1F9A9 */
  0x0002,   /* U+1F9AA */
  0x0002,   /* U+1F9AB */
  0x0002,   /* U+1F9AC */
  0x0002,   /* U+1F9AD */
  0x0002,   /* U+1F9AE */
  0x0002,   /* U+1F9AF */
  0x0002,   /* U+1F9B0 */
  0x0002,   /* U+1F9B1 */
  0x0002,   /* U+1F9B2 */
  0x0002,   /* U+1F9B3 */
  0x0002,   /* U+1F9B4 */
  0x0002,   /* U+1F9B5 */
  0x0002,   /* U+1F9B6 */
  0x0002,   /* U+1F9B7 */
  0x0002,   /* U+1F9B8 */
  0x0002,   /* U+1F9B9 */
  0x0002,   /* U+1F9BA */
  0x0002,   /* U+1F9BB */
  0x0002,   /* U+1F9BC */
  0x0002,   /* U+1F9BD */
  0x0002,   /* U+1F9BE */
  0x0002,   /* U+1F9BF */
  0x0002,   /* U+1F9C0 */
  0x0002,   /* U+1F9C1 */
  0x0002,   /* U+1F9C2 */
  0x0002,   /* U+1F9C3 */
  0x0002,   /* U+1F9C4 */
  0x0002,   /* U+1F9C5 */
  0x0002,   /* U+1F9C6 */
  0x0002,   /* U+1F9C7 */
  0x0002,   /* U+1F9C8 */
  0x0002,   /* U+1F9C9 */
  0x0002,   /* U+1F9CA */
  0x0002,   /* U+1F9CB */
  0x0002,   /* U+1F9CC */
  0x0002,   /* U+1F9CD */
  0x0002,   /* U+1F9CE */
  0x0002,   /* U+1F9CF */
  0x0002,   /* U+1F9D0 */
  0x0002,   /* U+1F9D1 */
  0x0002,   /* U+1F9D2 */
  0x0002,   /* U+1F9D3 */
  0x0002,   /* U+1F9D4 */
  0x0002,   /* U+1F9D5 */
  0x0002,   /* U+1F9D6 */
  0x0002,   /* U+1F9D7 */
  0x0002,   /* U+1F9D8 */
  0x0002,   /* U+1F9D9 */
  0x0002,   /* U+1F9DA */
  0x0002,   /* U+1F9DB */
  0x0002,   /* U+1F9DC */
  0x0002,   /* U+1F9DD */
  0x0002,   /* U+1F9DE */
  0x0002,   /* U+1F9DF */
  0x0002,   /* U+1F9E0 */
  0x0002,   /* U+1F9E1 */
  0x0002,   /* U+1F9E2 */
  0x0002,   /* U+1F9E3 */
  0x0002,   /* U+1F9E4 */
  0x0002,   /* U+1F9E5 */
  0x0002,   /* U+1F9E6 */
  0x0002,   /* U+1F9E7 */
  0x0002,   /* U+1F9E8 */
  0x0002,   /* U+1F9E9 */
  0x0002,   /* U+1F9EA */
  0x0002,   /* U+1F9EB */
  0x0002,   /* U+1F9EC */
  0x0002,   /* U+1F9ED */
  0x0002,   /* U+1F9EE */
  0x0002,   /* U+1F9EF */
  0x0002,   /* U+1F9F0 */
  0x0002,   /* U+1F9F1 */
  0x0002,   /* U+1F9F2 */
  0x0002,   /* U+1F9F3 */
  0x0002,   /* U+1F9F4 */
  0x0002,   /* U+1F9F5 */
  0x0002,   /* U+1F9F6 */
  0x0002,   /* U+1F9F7 */
  0x0002,   /* U+1F9F8 */
  0x0002,   /* U+1F9F9 */
  0x0002,   /* U+1F9FA */
  0x0002,   /* U+1F9FB */
  0x0002,   /* U+1F9FC */
  0x0002,   /* U+1F9FD */
  0x0002,   /* U+1F9FE */
  0x0002,   /* U+1F9FF */

  /* Primary weight 2 for each character. */
  0xF900,   /* U+1F900 */
  0xF901,   /* U+1F901 */
  0xF902,   /* U+1F902 */
  0xF903,   /* U+1F903 */
  0xF904,   /* U+1F904 */
  0xF905,   /* U+1F905 */
  0xF906,   /* U+1F906 */
  0xF907,   /* U+1F907 */
  0xF908,   /* U+1F908 */
  0xF909,   /* U+1F909 */
  0xF90A,   /* U+1F90A */
  0xF90B,   /* U+1F90B */
  0xF90C,   /* U+1F90C */
  0xF90D,   /* U+1F90D */
  0xF90E,   /* U+1F90E */
  0xF90F,   /* U+1F90F */
  0x0000,   /* U+1F910 */
  0x0000,   /* U+1F911 */
  0x0000,   /* U+1F912 */
  0x0000,   /* U+1F913 */
  0x0000,   /* U+1F914 */
  0x0000,   /* U+1F915 */
  0x0000,   /* U+1F916 */
  0x0000,   /* U+1F917 */
  0x0000,   /* U+1F918 */
  0x0000,   /* U+1F919 */
  0x0000,   /* U+1F91A */
  0x0000,   /* U+1F91B */
  0x0000,   /* U+1F91C */
  0x0000,   /* U+1F91D */
  0x0000,   /* U+1F91E */
  0xF91F,   /* U+1F91F */
  0x0000,   /* U+1F920 */
  0x0000,   /* U+1F921 */
  0x0000,   /* U+1F922 */
  0x0000,   /* U+1F923 */
  0x0000,   /* U+1F924 */
  0x0000,   /* U+1F925 */
  0x0000,   /* U+1F926 */
  0x0000,   /* U+1F927 */
  0xF928,   /* U+1F928 */
  0xF929,   /* U+1F929 */
  0xF92A,   /* U+1F92A */
  0xF92B,   /* U+1F92B */
  0xF92C,   /* U+1F92C */
  0xF92D,   /* U+1F92D */
  0xF92E,   /* U+1F92E */
  0xF92F,   /* U+1F92F */
  0x0000,   /* U+1F930 */
  0xF931,   /* U+1F931 */
  0xF932,   /* U+1F932 */
  0x0000,   /* U+1F933 */
  0x0000,   /* U+1F934 */
  0x0000,   /* U+1F935 */
  0x0000,   /* U+1F936 */
  0x0000,   /* U+1F937 */
  0x0000,   /* U+1F938 */
  0x0000,   /* U+1F939 */
  0x0000,   /* U+1F93A */
  0x0000,   /* U+1F93B */
  0x0000,   /* U+1F93C */
  0x0000,   /* U+1F93D */
  0x0000,   /* U+1F93E */
  0xF93F,   /* U+1F93F */
  0x0000,   /* U+1F940 */
  0x0000,   /* U+1F941 */
  0x0000,   /* U+1F942 */
  0x0000,   /* U+1F943 */
  0x0000,   /* U+1F944 */
  0x0000,   /* U+1F945 */
  0x0000,   /* U+1F946 */
  0x0000,   /* U+1F947 */
  0x0000,   /* U+1F948 */
  0x0000,   /* U+1F949 */
  0x0000,   /* U+1F94A */
  0x0000,   /* U+1F94B */
  0xF94C,   /* U+1F94C */
  0xF94D,   /* U+1F94D */
  0xF94E,   /* U+1F94E */
  0xF94F,   /* U+1F94F */
  0x0000,   /* U+1F950 */
  0x0000,   /* U+1F951 */
  0x0000,   /* U+1F952 */
  0x0000,   /* U+1F953 */
  0x0000,   /* U+1F954 */
  0x0000,   /* U+1F955 */
  0x0000,   /* U+1F956 */
  0x0000,   /* U+1F957 */
  0x0000,   /* U+1F958 */
  0x0000,   /* U+1F959 */
  0x0000,   /* U+1F95A */
  0x0000,   /* U+1F95B */
  0x0000,   /* U+1F95C */
  0x0000,   /* U+1F95D */
  0x0000,   /* U+1F95E */
  0xF95F,   /* U+1F95F */
  0xF960,   /* U+1F960 */
  0xF961,   /* U+1F961 */
  0xF962,   /* U+1F962 */
  0xF963,   /* U+1F963 */
  0xF964,   /* U+1F964 */
  0xF965,   /* U+1F965 */
  0xF966,   /* U+1F966 */
  0xF967,   /* U+1F967 */
  0xF968,   /* U+1F968 */
  0xF969,   /* U+1F969 */
  0xF96A,   /* U+1F96A */
  0xF96B,   /* U+1F96B */
  0xF96C,   /* U+1F96C */
  0xF96D,   /* U+1F96D */
  0xF96E,   /* U+1F96E */
  0xF96F,   /* U+1F96F */
  0xF970,   /* U+1F970 */
  0xF971,   /* U+1F971 */
  0xF972,   /* U+1F972 */
  0xF973,   /* U+1F973 */
  0xF974,   /* U+1F974 */
  0xF975,   /* U+1F975 */
  0xF976,   /* U+1F976 */
  0xF977,   /* U+1F977 */
  0xF978,   /* U+1F978 */
  0xF979,   /* U+1F979 */
  0xF97A,   /* U+1F97A */
  0xF97B,   /* U+1F97B */
  0xF97C,   /* U+1F97C */
  0xF97D,   /* U+1F97D */
  0xF97E,   /* U+1F97E */
  0xF97F,   /* U+1F97F */
  0x0000,   /* U+1F980 */
  0x0000,   /* U+1F981 */
  0x0000,   /* U+1F982 */
  0x0000,   /* U+1F983 */
  0x0000,   /* U+1F984 */
  0x0000,   /* U+1F985 */
  0x0000,   /* U+1F986 */
  0x0000,   /* U+1F987 */
  0x0000,   /* U+1F988 */
  0x0000,   /* U+1F989 */
  0x0000,   /* U+1F98A */
  0x0000,   /* U+1F98B */
  0x0000,   /* U+1F98C */
  0x0000,   /* U+1F98D */
  0x0000,   /* U+1F98E */
  0x0000,   /* U+1F98F */
  0x0000,   /* U+1F990 */
  0x0000,   /* U+1F991 */
  0xF992,   /* U+1F992 */
  0xF993,   /* U+1F993 */
  0xF994,   /* U+1F994 */
  0xF995,   /* U+1F995 */
  0xF996,   /* U+1F996 */
  0xF997,   /* U+1F997 */
  0xF998,   /* U+1F998 */
  0xF999,   /* U+1F999 */
  0xF99A,   /* U+1F99A */
  0xF99B,   /* U+1F99B */
  0xF99C,   /* U+1F99C */
  0xF99D,   /* U+1F99D */
  0xF99E,   /* U+1F99E */
  0xF99F,   /* U+1F99F */
  0xF9A0,   /* U+1F9A0 */
  0xF9A1,   /* U+1F9A1 */
  0xF9A2,   /* U+1F9A2 */
  0xF9A3,   /* U+1F9A3 */
  0xF9A4,   /* U+1F9A4 */
  0xF9A5,   /* U+1F9A5 */
  0xF9A6,   /* U+1F9A6 */
  0xF9A7,   /* U+1F9A7 */
  0xF9A8,   /* U+1F9A8 */
  0xF9A9,   /* U+1F9A9 */
  0xF9AA,   /* U+1F9AA */
  0xF9AB,   /* U+1F9AB */
  0xF9AC,   /* U+1F9AC */
  0xF9AD,   /* U+1F9AD */
  0xF9AE,   /* U+1F9AE */
  0xF9AF,   /* U+1F9AF */
  0xF9B0,   /* U+1F9B0 */
  0xF9B1,   /* U+1F9B1 */
  0xF9B2,   /* U+1F9B2 */
  0xF9B3,   /* U+1F9B3 */
  0xF9B4,   /* U+1F9B4 */
  0xF9B5,   /* U+1F9B5 */
  0xF9B6,   /* U+1F9B6 */
  0xF9B7,   /* U+1F9B7 */
  0xF9B8,   /* U+1F9B8 */
  0xF9B9,   /* U+1F9B9 */
  0xF9BA,   /* U+1F9BA */
  0xF9BB,   /* U+1F9BB */
  0xF9BC,   /* U+1F9BC */
  0xF9BD,   /* U+1F9BD */
  0xF9BE,   /* U+1F9BE */
  0xF9BF,   /* U+1F9BF */
  0x0000,   /* U+1F9C0 */
  0xF9C1,   /* U+1F9C1 */
  0xF9C2,   /* U+1F9C2 */
  0xF9C3,   /* U+1F9C3 */
  0xF9C4,   /* U+1F9C4 */
  0xF9C5,   /* U+1F9C5 */
  0xF9C6,   /* U+1F9C6 */
  0xF9C7,   /* U+1F9C7 */
  0xF9C8,   /* U+1F9C8 */
  0xF9C9,   /* U+1F9C9 */
  0xF9CA,   /* U+1F9CA */
  0xF9CB,   /* U+1F9CB */
  0xF9CC,   /* U+1F9CC */
  0xF9CD,   /* U+1F9CD */
  0xF9CE,   /* U+1F9CE */
  0xF9CF,   /* U+1F9CF */
  0xF9D0,   /* U+1F9D0 */
  0xF9D1,   /* U+1F9D1 */
  0xF9D2,   /* U+1F9D2 */
  0xF9D3,   /* U+1F9D3 */
  0xF9D4,   /* U+1F9D4 */
  0xF9D5,   /* U+1F9D5 */
  0xF9D6,   /* U+1F9D6 */
  0xF9D7,   /* U+1F9D7 */
  0xF9D8,   /* U+1F9D8 */
  0xF9D9,   /* U+1F9D9 */
  0xF9DA,   /* U+1F9DA */
  0xF9DB,   /* U+1F9DB */
  0xF9DC,   /* U+1F9DC */
  0xF9DD,   /* U+1F9DD */
  0xF9DE,   /* U+1F9DE */
  0xF9DF,   /* U+1F9DF */
  0xF9E0,   /* U+1F9E0 */
  0xF9E1,   /* U+1F9E1 */
  0xF9E2,   /* U+1F9E2 */
  0xF9E3,   /* U+1F9E3 */
  0xF9E4,   /* U+1F9E4 */
  0xF9E5,   /* U+1F9E5 */
  0xF9E6,   /* U+1F9E6 */
  0xF9E7,   /* U+1F9E7 */
  0xF9E8,   /* U+1F9E8 */
  0xF9E9,   /* U+1F9E9 */
  0xF9EA,   /* U+1F9EA */
  0xF9EB,   /* U+1F9EB */
  0xF9EC,   /* U+1F9EC */
  0xF9ED,   /* U+1F9ED */
  0xF9EE,   /* U+1F9EE */
  0xF9EF,   /* U+1F9EF */
  0xF9F0,   /* U+1F9F0 */
  0xF9F1,   /* U+1F9F1 */
  0xF9F2,   /* U+1F9F2 */
  0xF9F3,   /* U+1F9F3 */
  0xF9F4,   /* U+1F9F4 */
  0xF9F5,   /* U+1F9F5 */
  0xF9F6,   /* U+1F9F6 */
  0xF9F7,   /* U+1F9F7 */
  0xF9F8,   /* U+1F9F8 */
  0xF9F9,   /* U+1F9F9 */
  0xF9FA,   /* U+1F9FA */
  0xF9FB,   /* U+1F9FB */
  0xF9FC,   /* U+1F9FC */
  0xF9FD,   /* U+1F9FD */
  0xF9FE,   /* U+1F9FE */
  0xF9FF,   /* U+1F9FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+1F900 */
  0x0000,   /* U+1F901 */
  0x0000,   /* U+1F902 */
  0x0000,   /* U+1F903 */
  0x0000,   /* U+1F904 */
  0x0000,   /* U+1F905 */
  0x0000,   /* U+1F906 */
  0x0000,   /* U+1F907 */
  0x0000,   /* U+1F908 */
  0x0000,   /* U+1F909 */
  0x0000,   /* U+1F90A */
  0x0000,   /* U+1F90B */
  0x0000,   /* U+1F90C */
  0x0000,   /* U+1F90D */
  0x0000,   /* U+1F90E */
  0x0000,   /* U+1F90F */
  0x0000,   /* U+1F910 */
  0x0000,   /* U+1F911 */
  0x0000,   /* U+1F912 */
  0x0000,   /* U+1F913 */
  0x0000,   /* U+1F914 */
  0x0000,   /* U+1F915 */
  0x0000,   /* U+1F916 */
  0x0000,   /* U+1F917 */
  0x0000,   /* U+1F918 */
  0x0000,   /* U+1F919 */
  0x0000,   /* U+1F91A */
  0x0000,   /* U+1F91B */
  0x0000,   /* U+1F91C */
  0x0000,   /* U+1F91D */
  0x0000,   /* U+1F91E */
  0x0000,   /* U+1F91F */
  0x0000,   /* U+1F920 */
  0x0000,   /* U+1F921 */
  0x0000,   /* U+1F922 */
  0x0000,   /* U+1F923 */
  0x0000,   /* U+1F924 */
  0x0000,   /* U+1F925 */
  0x0000,   /* U+1F926 */
  0x0000,   /* U+1F927 */
  0x0000,   /* U+1F928 */
  0x0000,   /* U+1F929 */
  0x0000,   /* U+1F92A */
  0x0000,   /* U+1F92B */
  0x0000,   /* U+1F92C */
  0x0000,   /* U+1F92D */
  0x0000,   /* U+1F92E */
  0x0000,   /* U+1F92F */
  0x0000,   /* U+1F930 */
  0x0000,   /* U+1F931 */
  0x0000,   /* U+1F932 */
  0x0000,   /* U+1F933 */
  0x0000,   /* U+1F934 */
  0x0000,   /* U+1F935 */
  0x0000,   /* U+1F936 */
  0x0000,   /* U+1F937 */
  0x0000,   /* U+1F938 */
  0x0000,   /* U+1F939 */
  0x0000,   /* U+1F93A */
  0x0000,   /* U+1F93B */
  0x0000,   /* U+1F93C */
  0x0000,   /* U+1F93D */
  0x0000,   /* U+1F93E */
  0x0000,   /* U+1F93F */
  0x0000,   /* U+1F940 */
  0x0000,   /* U+1F941 */
  0x0000,   /* U+1F942 */
  0x0000,   /* U+1F943 */
  0x0000,   /* U+1F944 */
  0x0000,   /* U+1F945 */
  0x0000,   /* U+1F946 */
  0x0000,   /* U+1F947 */
  0x0000,   /* U+1F948 */
  0x0000,   /* U+1F949 */
  0x0000,   /* U+1F94A */
  0x0000,   /* U+1F94B */
  0x0000,   /* U+1F94C */
  0x0000,   /* U+1F94D */
  0x0000,   /* U+1F94E */
  0x0000,   /* U+1F94F */
  0x0000,   /* U+1F950 */
  0x0000,   /* U+1F951 */
  0x0000,   /* U+1F952 */
  0x0000,   /* U+1F953 */
  0x0000,   /* U+1F954 */
  0x0000,   /* U+1F955 */
  0x0000,   /* U+1F956 */
  0x0000,   /* U+1F957 */
  0x0000,   /* U+1F958 */
  0x0000,   /* U+1F959 */
  0x0000,   /* U+1F95A */
  0x0000,   /* U+1F95B */
  0x0000,   /* U+1F95C */
  0x0000,   /* U+1F95D */
  0x0000,   /* U+1F95E */
  0x0000,   /* U+1F95F */
  0x0000,   /* U+1F960 */
  0x0000,   /* U+1F961 */
  0x0000,   /* U+1F962 */
  0x0000,   /* U+1F963 */
  0x0000,   /* U+1F964 */
  0x0000,   /* U+1F965 */
  0x0000,   /* U+1F966 */
  0x0000,   /* U+1F967 */
  0x0000,   /* U+1F968 */
  0x0000,   /* U+1F969 */
  0x0000,   /* U+1F96A */
  0x0000,   /* U+1F96B */
  0x0000,   /* U+1F96C */
  0x0000,   /* U+1F96D */
  0x0000,   /* U+1F96E */
  0x0000,   /* U+1F96F */
  0x0000,   /* U+1F970 */
  0x0000,   /* U+1F971 */
  0x0000,   /* U+1F972 */
  0x0000,   /* U+1F973 */
  0x0000,   /* U+1F974 */
  0x0000,   /* U+1F975 */
  0x0000,   /* U+1F976 */
  0x0000,   /* U+1F977 */
  0x0000,   /* U+1F978 */
  0x0000,   /* U+1F979 */
  0x0000,   /* U+1F97A */
  0x0000,   /* U+1F97B */
  0x0000,   /* U+1F97C */
  0x0000,   /* U+1F97D */
  0x0000,   /* U+1F97E */
  0x0000,   /* U+1F97F */
  0x0000,   /* U+1F980 */
  0x0000,   /* U+1F981 */
  0x0000,   /* U+1F982 */
  0x0000,   /* U+1F983 */
  0x0000,   /* U+1F984 */
  0x0000,   /* U+1F985 */
  0x0000,   /* U+1F986 */
  0x0000,   /* U+1F987 */
  0x0000,   /* U+1F988 */
  0x0000,   /* U+1F989 */
  0x0000,   /* U+1F98A */
  0x0000,   /* U+1F98B */
  0x0000,   /* U+1F98C */
  0x0000,   /* U+1F98D */
  0x0000,   /* U+1F98E */
  0x0000,   /* U+1F98F */
  0x0000,   /* U+1F990 */
  0x0000,   /* U+1F991 */
  0x0000,   /* U+1F992 */
  0x0000,   /* U+1F993 */
  0x0000,   /* U+1F994 */
  0x0000,   /* U+1F995 */
  0x0000,   /* U+1F996 */
  0x0000,   /* U+1F997 */
  0x0000,   /* U+1F998 */
  0x0000,   /* U+1F999 */
  0x0000,   /* U+1F99A */
  0x0000,   /* U+1F99B */
  0x0000,   /* U+1F99C */
  0x0000,   /* U+1F99D */
  0x0000,   /* U+1F99E */
  0x0000,   /* U+1F99F */
  0x0000,   /* U+1F9A0 */
  0x0000,   /* U+1F9A1 */
  0x0000,   /* U+1F9A2 */
  0x0000,   /* U+1F9A3 */
  0x0000,   /* U+1F9A4 */
  0x0000,   /* U+1F9A5 */
  0x0000,   /* U+1F9A6 */
  0x0000,   /* U+1F9A7 */
  0x0000,   /* U+1F9A8 */
  0x0000,   /* U+1F9A9 */
  0x0000,   /* U+1F9AA */
  0x0000,   /* U+1F9AB */
  0x0000,   /* U+1F9AC */
  0x0000,   /* U+1F9AD */
  0x0000,   /* U+1F9AE */
  0x0000,   /* U+1F9AF */
  0x0000,   /* U+1F9B0 */
  0x0000,   /* U+1F9B1 */
  0x0000,   /* U+1F9B2 */
  0x0000,   /* U+1F9B3 */
  0x0000,   /* U+1F9B4 */
  0x0000,   /* U+1F9B5 */
  0x0000,   /* U+1F9B6 */
  0x0000,   /* U+1F9B7 */
  0x0000,   /* U+1F9B8 */
  0x0000,   /* U+1F9B9 */
  0x0000,   /* U+1F9BA */
  0x0000,   /* U+1F9BB */
  0x0000,   /* U+1F9BC */
  0x0000,   /* U+1F9BD */
  0x0000,   /* U+1F9BE */
  0x0000,   /* U+1F9BF */
  0x0000,   /* U+1F9C0 */
  0x0000,   /* U+1F9C1 */
  0x0000,   /* U+1F9C2 */
  0x0000,   /* U+1F9C3 */
  0x0000,   /* U+1F9C4 */
  0x0000,   /* U+1F9C5 */
  0x0000,   /* U+1F9C6 */
  0x0000,   /* U+1F9C7 */
  0x0000,   /* U+1F9C8 */
  0x0000,   /* U+1F9C9 */
  0x0000,   /* U+1F9CA */
  0x0000,   /* U+1F9CB */
  0x0000,   /* U+1F9CC */
  0x0000,   /* U+1F9CD */
  0x0000,   /* U+1F9CE */
  0x0000,   /* U+1F9CF */
  0x0000,   /* U+1F9D0 */
  0x0000,   /* U+1F9D1 */
  0x0000,   /* U+1F9D2 */
  0x0000,   /* U+1F9D3 */
  0x0000,   /* U+1F9D4 */
  0x0000,   /* U+1F9D5 */
  0x0000,   /* U+1F9D6 */
  0x0000,   /* U+1F9D7 */
  0x0000,   /* U+1F9D8 */
  0x0000,   /* U+1F9D9 */
  0x0000,   /* U+1F9DA */
  0x0000,   /* U+1F9DB */
  0x0000,   /* U+1F9DC */
  0x0000,   /* U+1F9DD */
  0x0000,   /* U+1F9DE */
  0x0000,   /* U+1F9DF */
  0x0000,   /* U+1F9E0 */
  0x0000,   /* U+1F9E1 */
  0x0000,   /* U+1F9E2 */
  0x0000,   /* U+1F9E3 */
  0x0000,   /* U+1F9E4 */
  0x0000,   /* U+1F9E5 */
  0x0000,   /* U+1F9E6 */
  0x0000,   /* U+1F9E7 */
  0x0000,   /* U+1F9E8 */
  0x0000,   /* U+1F9E9 */
  0x0000,   /* U+1F9EA */
  0x0000,   /* U+1F9EB */
  0x0000,   /* U+1F9EC */
  0x0000,   /* U+1F9ED */
  0x0000,   /* U+1F9EE */
  0x0000,   /* U+1F9EF */
  0x0000,   /* U+1F9F0 */
  0x0000,   /* U+1F9F1 */
  0x0000,   /* U+1F9F2 */
  0x0000,   /* U+1F9F3 */
  0x0000,   /* U+1F9F4 */
  0x0000,   /* U+1F9F5 */
  0x0000,   /* U+1F9F6 */
  0x0000,   /* U+1F9F7 */
  0x0000,   /* U+1F9F8 */
  0x0000,   /* U+1F9F9 */
  0x0000,   /* U+1F9FA */
  0x0000,   /* U+1F9FB */
  0x0000,   /* U+1F9FC */
  0x0000,   /* U+1F9FD */
  0x0000,   /* U+1F9FE */
  0x0000,   /* U+1F9FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+1F900 */
  0x0000,   /* U+1F901 */
  0x0000,   /* U+1F902 */
  0x0000,   /* U+1F903 */
  0x0000,   /* U+1F904 */
  0x0000,   /* U+1F905 */
  0x0000,   /* U+1F906 */
  0x0000,   /* U+1F907 */
  0x0000,   /* U+1F908 */
  0x0000,   /* U+1F909 */
  0x0000,   /* U+1F90A */
  0x0000,   /* U+1F90B */
  0x0000,   /* U+1F90C */
  0x0000,   /* U+1F90D */
  0x0000,   /* U+1F90E */
  0x0000,   /* U+1F90F */
  0x0000,   /* U+1F910 */
  0x0000,   /* U+1F911 */
  0x0000,   /* U+1F912 */
  0x0000,   /* U+1F913 */
  0x0000,   /* U+1F914 */
  0x0000,   /* U+1F915 */
  0x0000,   /* U+1F916 */
  0x0000,   /* U+1F917 */
  0x0000,   /* U+1F918 */
  0x0000,   /* U+1F919 */
  0x0000,   /* U+1F91A */
  0x0000,   /* U+1F91B */
  0x0000,   /* U+1F91C */
  0x0000,   /* U+1F91D */
  0x0000,   /* U+1F91E */
  0x0000,   /* U+1F91F */
  0x0000,   /* U+1F920 */
  0x0000,   /* U+1F921 */
  0x0000,   /* U+1F922 */
  0x0000,   /* U+1F923 */
  0x0000,   /* U+1F924 */
  0x0000,   /* U+1F925 */
  0x0000,   /* U+1F926 */
  0x0000,   /* U+1F927 */
  0x0000,   /* U+1F928 */
  0x0000,   /* U+1F929 */
  0x0000,   /* U+1F92A */
  0x0000,   /* U+1F92B */
  0x0000,   /* U+1F92C */
  0x0000,   /* U+1F92D */
  0x0000,   /* U+1F92E */
  0x0000,   /* U+1F92F */
  0x0000,   /* U+1F930 */
  0x0000,   /* U+1F931 */
  0x0000,   /* U+1F932 */
  0x0000,   /* U+1F933 */
  0x0000,   /* U+1F934 */
  0x0000,   /* U+1F935 */
  0x0000,   /* U+1F936 */
  0x0000,   /* U+1F937 */
  0x0000,   /* U+1F938 */
  0x0000,   /* U+1F939 */
  0x0000,   /* U+1F93A */
  0x0000,   /* U+1F93B */
  0x0000,   /* U+1F93C */
  0x0000,   /* U+1F93D */
  0x0000,   /* U+1F93E */
  0x0000,   /* U+1F93F */
  0x0000,   /* U+1F940 */
  0x0000,   /* U+1F941 */
  0x0000,   /* U+1F942 */
  0x0000,   /* U+1F943 */
  0x0000,   /* U+1F944 */
  0x0000,   /* U+1F945 */
  0x0000,   /* U+1F946 */
  0x0000,   /* U+1F947 */
  0x0000,   /* U+1F948 */
  0x0000,   /* U+1F949 */
  0x0000,   /* U+1F94A */
  0x0000,   /* U+1F94B */
  0x0000,   /* U+1F94C */
  0x0000,   /* U+1F94D */
  0x0000,   /* U+1F94E */
  0x0000,   /* U+1F94F */
  0x0000,   /* U+1F950 */
  0x0000,   /* U+1F951 */
  0x0000,   /* U+1F952 */
  0x0000,   /* U+1F953 */
  0x0000,   /* U+1F954 */
  0x0000,   /* U+1F955 */
  0x0000,   /* U+1F956 */
  0x0000,   /* U+1F957 */
  0x0000,   /* U+1F958 */
  0x0000,   /* U+1F959 */
  0x0000,   /* U+1F95A */
  0x0000,   /* U+1F95B */
  0x0000,   /* U+1F95C */
  0x0000,   /* U+1F95D */
  0x0000,   /* U+1F95E */
  0x0000,   /* U+1F95F */
  0x0000,   /* U+1F960 */
  0x0000,   /* U+1F961 */
  0x0000,   /* U+1F962 */
  0x0000,   /* U+1F963 */
  0x0000,   /* U+1F964 */
  0x0000,   /* U+1F965 */
  0x0000,   /* U+1F966 */
  0x0000,   /* U+1F967 */
  0x0000,   /* U+1F968 */
  0x0000,   /* U+1F969 */
  0x0000,   /* U+1F96A */
  0x0000,   /* U+1F96B */
  0x0000,   /* U+1F96C */
  0x0000,   /* U+1F96D */
  0x0000,   /* U+1F96E */
  0x0000,   /* U+1F96F */
  0x0000,   /* U+1F970 */
  0x0000,   /* U+1F971 */
  0x0000,   /* U+1F972 */
  0x0000,   /* U+1F973 */
  0x0000,   /* U+1F974 */
  0x0000,   /* U+1F975 */
  0x0000,   /* U+1F976 */
  0x0000,   /* U+1F977 */
  0x0000,   /* U+1F978 */
  0x0000,   /* U+1F979 */
  0x0000,   /* U+1F97A */
  0x0000,   /* U+1F97B */
  0x0000,   /* U+1F97C */
  0x0000,   /* U+1F97D */
  0x0000,   /* U+1F97E */
  0x0000,   /* U+1F97F */
  0x0000,   /* U+1F980 */
  0x0000,   /* U+1F981 */
  0x0000,   /* U+1F982 */
  0x0000,   /* U+1F983 */
  0x0000,   /* U+1F984 */
  0x0000,   /* U+1F985 */
  0x0000,   /* U+1F986 */
  0x0000,   /* U+1F987 */
  0x0000,   /* U+1F988 */
  0x0000,   /* U+1F989 */
  0x0000,   /* U+1F98A */
  0x0000,   /* U+1F98B */
  0x0000,   /* U+1F98C */
  0x0000,   /* U+1F98D */
  0x0000,   /* U+1F98E */
  0x0000,   /* U+1F98F */
  0x0000,   /* U+1F990 */
  0x0000,   /* U+1F991 */
  0x0000,   /* U+1F992 */
  0x0000,   /* U+1F993 */
  0x0000,   /* U+1F994 */
  0x0000,   /* U+1F995 */
  0x0000,   /* U+1F996 */
  0x0000,   /* U+1F997 */
  0x0000,   /* U+1F998 */
  0x0000,   /* U+1F999 */
  0x0000,   /* U+1F99A */
  0x0000,   /* U+1F99B */
  0x0000,   /* U+1F99C */
  0x0000,   /* U+1F99D */
  0x0000,   /* U+1F99E */
  0x0000,   /* U+1F99F */
  0x0000,   /* U+1F9A0 */
  0x0000,   /* U+1F9A1 */
  0x0000,   /* U+1F9A2 */
  0x0000,   /* U+1F9A3 */
  0x0000,   /* U+1F9A4 */
  0x0000,   /* U+1F9A5 */
  0x0000,   /* U+1F9A6 */
  0x0000,   /* U+1F9A7 */
  0x0000,   /* U+1F9A8 */
  0x0000,   /* U+1F9A9 */
  0x0000,   /* U+1F9AA */
  0x0000,   /* U+1F9AB */
  0x0000,   /* U+1F9AC */
  0x0000,   /* U+1F9AD */
  0x0000,   /* U+1F9AE */
  0x0000,   /* U+1F9AF */
  0x0000,   /* U+1F9B0 */
  0x0000,   /* U+1F9B1 */
  0x0000,   /* U+1F9B2 */
  0x0000,   /* U+1F9B3 */
  0x0000,   /* U+1F9B4 */
  0x0000,   /* U+1F9B5 */
  0x0000,   /* U+1F9B6 */
  0x0000,   /* U+1F9B7 */
  0x0000,   /* U+1F9B8 */
  0x0000,   /* U+1F9B9 */
  0x0000,   /* U+1F9BA */
  0x0000,   /* U+1F9BB */
  0x0000,   /* U+1F9BC */
  0x0000,   /* U+1F9BD */
  0x0000,   /* U+1F9BE */
  0x0000,   /* U+1F9BF */
  0x0000,   /* U+1F9C0 */
  0x0000,   /* U+1F9C1 */
  0x0000,   /* U+1F9C2 */
  0x0000,   /* U+1F9C3 */
  0x0000,   /* U+1F9C4 */
  0x0000,   /* U+1F9C5 */
  0x0000,   /* U+1F9C6 */
  0x0000,   /* U+1F9C7 */
  0x0000,   /* U+1F9C8 */
  0x0000,   /* U+1F9C9 */
  0x0000,   /* U+1F9CA */
  0x0000,   /* U+1F9CB */
  0x0000,   /* U+1F9CC */
  0x0000,   /* U+1F9CD */
  0x0000,   /* U+1F9CE */
  0x0000,   /* U+1F9CF */
  0x0000,   /* U+1F9D0 */
  0x0000,   /* U+1F9D1 */
  0x0000,   /* U+1F9D2 */
  0x0000,   /* U+1F9D3 */
  0x0000,   /* U+1F9D4 */
  0x0000,   /* U+1F9D5 */
  0x0000,   /* U+1F9D6 */
  0x0000,   /* U+1F9D7 */
  0x0000,   /* U+1F9D8 */
  0x0000,   /* U+1F9D9 */
  0x0000,   /* U+1F9DA */
  0x0000,   /* U+1F9DB */
  0x0000,   /* U+1F9DC */
  0x0000,   /* U+1F9DD */
  0x0000,   /* U+1F9DE */
  0x0000,   /* U+1F9DF */
  0x0000,   /* U+1F9E0 */
  0x0000,   /* U+1F9E1 */
  0x0000,   /* U+1F9E2 */
  0x0000,   /* U+1F9E3 */
  0x0000,   /* U+1F9E4 */
  0x0000,   /* U+1F9E5 */
  0x0000,   /* U+1F9E6 */
  0x0000,   /* U+1F9E7 */
  0x0000,   /* U+1F9E8 */
  0x0000,   /* U+1F9E9 */
  0x0000,   /* U+1F9EA */
  0x0000,   /* U+1F9EB */
  0x0000,   /* U+1F9EC */
  0x0000,   /* U+1F9ED */
  0x0000,   /* U+1F9EE */
  0x0000,   /* U+1F9EF */
  0x0000,   /* U+1F9F0 */
  0x0000,   /* U+1F9F1 */
  0x0000,   /* U+1F9F2 */
  0x0000,   /* U+1F9F3 */
  0x0000,   /* U+1F9F4 */
  0x0000,   /* U+1F9F5 */
  0x0000,   /* U+1F9F6 */
  0x0000,   /* U+1F9F7 */
  0x0000,   /* U+1F9F8 */
  0x0000,   /* U+1F9F9 */
  0x0000,   /* U+1F9FA */
  0x0000,   /* U+1F9FB */
  0x0000,   /* U+1F9FC */
  0x0000,   /* U+1F9FD */
  0x0000,   /* U+1F9FE */
  0x0000,   /* U+1F9FF */
};

uint16 uca900_p2F8[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB40,   /* U+2F800 */
  0xFB40,   /* U+2F801 */
  0xFB40,   /* U+2F802 */
  0xFB84,   /* U+2F803 */
  0xFB40,   /* U+2F804 */
  0xFB40,   /* U+2F805 */
  0xFB40,   /* U+2F806 */
  0xFB40,   /* U+2F807 */
  0xFB40,   /* U+2F808 */
  0xFB40,   /* U+2F809 */
  0xFB40,   /* U+2F80A */
  0xFB40,   /* U+2F80B */
  0xFB80,   /* U+2F80C */
  0xFB84,   /* U+2F80D */
  0xFB40,   /* U+2F80E */
  0xFB40,   /* U+2F80F */
  0xFB40,   /* U+2F810 */
  0xFB40,   /* U+2F811 */
  0xFB84,   /* U+2F812 */
  0xFB80,   /* U+2F813 */
  0xFB40,   /* U+2F814 */
  0xFB40,   /* U+2F815 */
  0xFB84,   /* U+2F816 */
  0xFB40,   /* U+2F817 */
  0xFB40,   /* U+2F818 */
  0xFB40,   /* U+2F819 */
  0xFB40,   /* U+2F81A */
  0xFB40,   /* U+2F81B */
  0xFB85,   /* U+2F81C */
  0xFB40,   /* U+2F81D */
  0xFB40,   /* U+2F81E */
  0xFB80,   /* U+2F81F */
  0xFB40,   /* U+2F820 */
  0xFB40,   /* U+2F821 */
  0xFB40,   /* U+2F822 */
  0xFB40,   /* U+2F823 */
  0xFB80,   /* U+2F824 */
  0xFB40,   /* U+2F825 */
  0xFB40,   /* U+2F826 */
  0xFB40,   /* U+2F827 */
  0xFB40,   /* U+2F828 */
  0xFB40,   /* U+2F829 */
  0xFB40,   /* U+2F82A */
  0xFB40,   /* U+2F82B */
  0xFB40,   /* U+2F82C */
  0xFB40,   /* U+2F82D */
  0xFB40,   /* U+2F82E */
  0xFB40,   /* U+2F82F */
  0xFB40,   /* U+2F830 */
  0xFB40,   /* U+2F831 */
  0xFB40,   /* U+2F832 */
  0xFB40,   /* U+2F833 */
  0xFB84,   /* U+2F834 */
  0xFB40,   /* U+2F835 */
  0xFB40,   /* U+2F836 */
  0xFB40,   /* U+2F837 */
  0xFB84,   /* U+2F838 */
  0xFB40,   /* U+2F839 */
  0xFB40,   /* U+2F83A */
  0xFB40,   /* U+2F83B */
  0xFB40,   /* U+2F83C */
  0xFB40,   /* U+2F83D */
  0xFB40,   /* U+2F83E */
  0xFB40,   /* U+2F83F */
  0xFB40,   /* U+2F840 */
  0xFB40,   /* U+2F841 */
  0xFB40,   /* U+2F842 */
  0xFB40,   /* U+2F843 */
  0xFB40,   /* U+2F844 */
  0xFB40,   /* U+2F845 */
  0xFB40,   /* U+2F846 */
  0xFB40,   /* U+2F847 */
  0xFB40,   /* U+2F848 */
  0xFB40,   /* U+2F849 */
  0xFB40,   /* U+2F84A */
  0xFB40,   /* U+2F84B */
  0xFB40,   /* U+2F84C */
  0xFB40,   /* U+2F84D */
  0xFB40,   /* U+2F84E */
  0xFB40,   /* U+2F84F */
  0xFB40,   /* U+2F850 */
  0xFB40,   /* U+2F851 */
  0xFB40,   /* U+2F852 */
  0xFB40,   /* U+2F853 */
  0xFB40,   /* U+2F854 */
  0xFB40,   /* U+2F855 */
  0xFB40,   /* U+2F856 */
  0xFB40,   /* U+2F857 */
  0xFB40,   /* U+2F858 */
  0xFB84,   /* U+2F859 */
  0xFB40,   /* U+2F85A */
  0xFB40,   /* U+2F85B */
  0xFB40,   /* U+2F85C */
  0xFB40,   /* U+2F85D */
  0xFB40,   /* U+2F85E */
  0xFB40,   /* U+2F85F */
  0xFB84,   /* U+2F860 */
  0xFB84,   /* U+2F861 */
  0xFB40,   /* U+2F862 */
  0xFB40,   /* U+2F863 */
  0xFB40,   /* U+2F864 */
  0xFB40,   /* U+2F865 */
  0xFB40,   /* U+2F866 */
  0xFB80,   /* U+2F867 */
  0xFB80,   /* U+2F868 */
  0xFB40,   /* U+2F869 */
  0xFB40,   /* U+2F86A */
  0xFB40,   /* U+2F86B */
  0xFB84,   /* U+2F86C */
  0xFB40,   /* U+2F86D */
  0xFB40,   /* U+2F86E */
  0xFB40,   /* U+2F86F */
  0xFB40,   /* U+2F870 */
  0xFB84,   /* U+2F871 */
  0xFB40,   /* U+2F872 */
  0xFB40,   /* U+2F873 */
  0xFB40,   /* U+2F874 */
  0xFB40,   /* U+2F875 */
  0xFB80,   /* U+2F876 */
  0xFB40,   /* U+2F877 */
  0xFB40,   /* U+2F878 */
  0xFB40,   /* U+2F879 */
  0xFB40,   /* U+2F87A */
  0xFB84,   /* U+2F87B */
  0xFB40,   /* U+2F87C */
  0xFB84,   /* U+2F87D */
  0xFB40,   /* U+2F87E */
  0xFB40,   /* U+2F87F */
  0xFB40,   /* U+2F880 */
  0xFB40,   /* U+2F881 */
  0xFB40,   /* U+2F882 */
  0xFB80,   /* U+2F883 */
  0xFB40,   /* U+2F884 */
  0xFB40,   /* U+2F885 */
  0xFB40,   /* U+2F886 */
  0xFB40,   /* U+2F887 */
  0xFB80,   /* U+2F888 */
  0xFB84,   /* U+2F889 */
  0xFB80,   /* U+2F88A */
  0xFB40,   /* U+2F88B */
  0xFB40,   /* U+2F88C */
  0xFB40,   /* U+2F88D */
  0xFB40,   /* U+2F88E */
  0xFB85,   /* U+2F88F */
  0xFB40,   /* U+2F890 */
  0xFB84,   /* U+2F891 */
  0xFB84,   /* U+2F892 */
  0xFB41,   /* U+2F893 */
  0xFB40,   /* U+2F894 */
  0xFB40,   /* U+2F895 */
  0xFB80,   /* U+2F896 */
  0xFB84,   /* U+2F897 */
  0xFB84,   /* U+2F898 */
  0xFB40,   /* U+2F899 */
  0xFB40,   /* U+2F89A */
  0xFB80,   /* U+2F89B */
  0xFB40,   /* U+2F89C */
  0xFB40,   /* U+2F89D */
  0xFB40,   /* U+2F89E */
  0xFB40,   /* U+2F89F */
  0xFB40,   /* U+2F8A0 */
  0xFB80,   /* U+2F8A1 */
  0xFB80,   /* U+2F8A2 */
  0xFB40,   /* U+2F8A3 */
  0xFB84,   /* U+2F8A4 */
  0xFB40,   /* U+2F8A5 */
  0xFB40,   /* U+2F8A6 */
  0xFB40,   /* U+2F8A7 */
  0xFB40,   /* U+2F8A8 */
  0xFB40,   /* U+2F8A9 */
  0xFB40,   /* U+2F8AA */
  0xFB40,   /* U+2F8AB */
  0xFB40,   /* U+2F8AC */
  0xFB40,   /* U+2F8AD */
  0xFB40,   /* U+2F8AE */
  0xFB40,   /* U+2F8AF */
  0xFB40,   /* U+2F8B0 */
  0xFB40,   /* U+2F8B1 */
  0xFB40,   /* U+2F8B2 */
  0xFB40,   /* U+2F8B3 */
  0xFB40,   /* U+2F8B4 */
  0xFB40,   /* U+2F8B5 */
  0xFB40,   /* U+2F8B6 */
  0xFB40,   /* U+2F8B7 */
  0xFB84,   /* U+2F8B8 */
  0xFB40,   /* U+2F8B9 */
  0xFB40,   /* U+2F8BA */
  0xFB40,   /* U+2F8BB */
  0xFB40,   /* U+2F8BC */
  0xFB40,   /* U+2F8BD */
  0xFB84,   /* U+2F8BE */
  0xFB40,   /* U+2F8BF */
  0xFB40,   /* U+2F8C0 */
  0xFB40,   /* U+2F8C1 */
  0xFB80,   /* U+2F8C2 */
  0xFB40,   /* U+2F8C3 */
  0xFB40,   /* U+2F8C4 */
  0xFB40,   /* U+2F8C5 */
  0xFB40,   /* U+2F8C6 */
  0xFB80,   /* U+2F8C7 */
  0xFB40,   /* U+2F8C8 */
  0xFB40,   /* U+2F8C9 */
  0xFB84,   /* U+2F8CA */
  0xFB40,   /* U+2F8CB */
  0xFB40,   /* U+2F8CC */
  0xFB40,   /* U+2F8CD */
  0xFB80,   /* U+2F8CE */
  0xFB40,   /* U+2F8CF */
  0xFB80,   /* U+2F8D0 */
  0xFB80,   /* U+2F8D1 */
  0xFB40,   /* U+2F8D2 */
  0xFB40,   /* U+2F8D3 */
  0xFB40,   /* U+2F8D4 */
  0xFB40,   /* U+2F8D5 */
  0xFB41,   /* U+2F8D6 */
  0xFB80,   /* U+2F8D7 */
  0xFB40,   /* U+2F8D8 */
  0xFB40,   /* U+2F8D9 */
  0xFB40,   /* U+2F8DA */
  0xFB40,   /* U+2F8DB */
  0xFB40,   /* U+2F8DC */
  0xFB84,   /* U+2F8DD */
  0xFB80,   /* U+2F8DE */
  0xFB40,   /* U+2F8DF */
  0xFB40,   /* U+2F8E0 */
  0xFB40,   /* U+2F8E1 */
  0xFB40,   /* U+2F8E2 */
  0xFB84,   /* U+2F8E3 */
  0xFB40,   /* U+2F8E4 */
  0xFB40,   /* U+2F8E5 */
  0xFB40,   /* U+2F8E6 */
  0xFB80,   /* U+2F8E7 */
  0xFB40,   /* U+2F8E8 */
  0xFB40,   /* U+2F8E9 */
  0xFB40,   /* U+2F8EA */
  0xFB40,   /* U+2F8EB */
  0xFB84,   /* U+2F8EC */
  0xFB40,   /* U+2F8ED */
  0xFB80,   /* U+2F8EE */
  0xFB40,   /* U+2F8EF */
  0xFB84,   /* U+2F8F0 */
  0xFB40,   /* U+2F8F1 */
  0xFB80,   /* U+2F8F2 */
  0xFB40,   /* U+2F8F3 */
  0xFB40,   /* U+2F8F4 */
  0xFB40,   /* U+2F8F5 */
  0xFB40,   /* U+2F8F6 */
  0xFB84,   /* U+2F8F7 */
  0xFB84,   /* U+2F8F8 */
  0xFB84,   /* U+2F8F9 */
  0xFB40,   /* U+2F8FA */
  0xFB84,   /* U+2F8FB */
  0xFB40,   /* U+2F8FC */
  0xFB40,   /* U+2F8FD */
  0xFB40,   /* U+2F8FE */
  0xFB40,   /* U+2F8FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2F800 */
  0x0020,   /* U+2F801 */
  0x0020,   /* U+2F802 */
  0x0020,   /* U+2F803 */
  0x0020,   /* U+2F804 */
  0x0020,   /* U+2F805 */
  0x0020,   /* U+2F806 */
  0x0020,   /* U+2F807 */
  0x0020,   /* U+2F808 */
  0x0020,   /* U+2F809 */
  0x0020,   /* U+2F80A */
  0x0020,   /* U+2F80B */
  0x0020,   /* U+2F80C */
  0x0020,   /* U+2F80D */
  0x0020,   /* U+2F80E */
  0x0020,   /* U+2F80F */
  0x0020,   /* U+2F810 */
  0x0020,   /* U+2F811 */
  0x0020,   /* U+2F812 */
  0x0020,   /* U+2F813 */
  0x0020,   /* U+2F814 */
  0x0020,   /* U+2F815 */
  0x0020,   /* U+2F816 */
  0x0020,   /* U+2F817 */
  0x0020,   /* U+2F818 */
  0x0020,   /* U+2F819 */
  0x0020,   /* U+2F81A */
  0x0020,   /* U+2F81B */
  0x0020,   /* U+2F81C */
  0x0020,   /* U+2F81D */
  0x0020,   /* U+2F81E */
  0x0020,   /* U+2F81F */
  0x0020,   /* U+2F820 */
  0x0020,   /* U+2F821 */
  0x0020,   /* U+2F822 */
  0x0020,   /* U+2F823 */
  0x0020,   /* U+2F824 */
  0x0020,   /* U+2F825 */
  0x0020,   /* U+2F826 */
  0x0020,   /* U+2F827 */
  0x0020,   /* U+2F828 */
  0x0020,   /* U+2F829 */
  0x0020,   /* U+2F82A */
  0x0020,   /* U+2F82B */
  0x0020,   /* U+2F82C */
  0x0020,   /* U+2F82D */
  0x0020,   /* U+2F82E */
  0x0020,   /* U+2F82F */
  0x0020,   /* U+2F830 */
  0x0020,   /* U+2F831 */
  0x0020,   /* U+2F832 */
  0x0020,   /* U+2F833 */
  0x0020,   /* U+2F834 */
  0x0020,   /* U+2F835 */
  0x0020,   /* U+2F836 */
  0x0020,   /* U+2F837 */
  0x0020,   /* U+2F838 */
  0x0020,   /* U+2F839 */
  0x0020,   /* U+2F83A */
  0x0020,   /* U+2F83B */
  0x0020,   /* U+2F83C */
  0x0020,   /* U+2F83D */
  0x0020,   /* U+2F83E */
  0x0020,   /* U+2F83F */
  0x0020,   /* U+2F840 */
  0x0020,   /* U+2F841 */
  0x0020,   /* U+2F842 */
  0x0020,   /* U+2F843 */
  0x0020,   /* U+2F844 */
  0x0020,   /* U+2F845 */
  0x0020,   /* U+2F846 */
  0x0020,   /* U+2F847 */
  0x0020,   /* U+2F848 */
  0x0020,   /* U+2F849 */
  0x0020,   /* U+2F84A */
  0x0020,   /* U+2F84B */
  0x0020,   /* U+2F84C */
  0x0020,   /* U+2F84D */
  0x0020,   /* U+2F84E */
  0x0020,   /* U+2F84F */
  0x0020,   /* U+2F850 */
  0x0020,   /* U+2F851 */
  0x0020,   /* U+2F852 */
  0x0020,   /* U+2F853 */
  0x0020,   /* U+2F854 */
  0x0020,   /* U+2F855 */
  0x0020,   /* U+2F856 */
  0x0020,   /* U+2F857 */
  0x0020,   /* U+2F858 */
  0x0020,   /* U+2F859 */
  0x0020,   /* U+2F85A */
  0x0020,   /* U+2F85B */
  0x0020,   /* U+2F85C */
  0x0020,   /* U+2F85D */
  0x0020,   /* U+2F85E */
  0x0020,   /* U+2F85F */
  0x0020,   /* U+2F860 */
  0x0020,   /* U+2F861 */
  0x0020,   /* U+2F862 */
  0x0020,   /* U+2F863 */
  0x0020,   /* U+2F864 */
  0x0020,   /* U+2F865 */
  0x0020,   /* U+2F866 */
  0x0020,   /* U+2F867 */
  0x0020,   /* U+2F868 */
  0x0020,   /* U+2F869 */
  0x0020,   /* U+2F86A */
  0x0020,   /* U+2F86B */
  0x0020,   /* U+2F86C */
  0x0020,   /* U+2F86D */
  0x0020,   /* U+2F86E */
  0x0020,   /* U+2F86F */
  0x0020,   /* U+2F870 */
  0x0020,   /* U+2F871 */
  0x0020,   /* U+2F872 */
  0x0020,   /* U+2F873 */
  0x0020,   /* U+2F874 */
  0x0020,   /* U+2F875 */
  0x0020,   /* U+2F876 */
  0x0020,   /* U+2F877 */
  0x0020,   /* U+2F878 */
  0x0020,   /* U+2F879 */
  0x0020,   /* U+2F87A */
  0x0020,   /* U+2F87B */
  0x0020,   /* U+2F87C */
  0x0020,   /* U+2F87D */
  0x0020,   /* U+2F87E */
  0x0020,   /* U+2F87F */
  0x0020,   /* U+2F880 */
  0x0020,   /* U+2F881 */
  0x0020,   /* U+2F882 */
  0x0020,   /* U+2F883 */
  0x0020,   /* U+2F884 */
  0x0020,   /* U+2F885 */
  0x0020,   /* U+2F886 */
  0x0020,   /* U+2F887 */
  0x0020,   /* U+2F888 */
  0x0020,   /* U+2F889 */
  0x0020,   /* U+2F88A */
  0x0020,   /* U+2F88B */
  0x0020,   /* U+2F88C */
  0x0020,   /* U+2F88D */
  0x0020,   /* U+2F88E */
  0x0020,   /* U+2F88F */
  0x0020,   /* U+2F890 */
  0x0020,   /* U+2F891 */
  0x0020,   /* U+2F892 */
  0x0020,   /* U+2F893 */
  0x0020,   /* U+2F894 */
  0x0020,   /* U+2F895 */
  0x0020,   /* U+2F896 */
  0x0020,   /* U+2F897 */
  0x0020,   /* U+2F898 */
  0x0020,   /* U+2F899 */
  0x0020,   /* U+2F89A */
  0x0020,   /* U+2F89B */
  0x0020,   /* U+2F89C */
  0x0020,   /* U+2F89D */
  0x0020,   /* U+2F89E */
  0x0020,   /* U+2F89F */
  0x0020,   /* U+2F8A0 */
  0x0020,   /* U+2F8A1 */
  0x0020,   /* U+2F8A2 */
  0x0020,   /* U+2F8A3 */
  0x0020,   /* U+2F8A4 */
  0x0020,   /* U+2F8A5 */
  0x0020,   /* U+2F8A6 */
  0x0020,   /* U+2F8A7 */
  0x0020,   /* U+2F8A8 */
  0x0020,   /* U+2F8A9 */
  0x0020,   /* U+2F8AA */
  0x0020,   /* U+2F8AB */
  0x0020,   /* U+2F8AC */
  0x0020,   /* U+2F8AD */
  0x0020,   /* U+2F8AE */
  0x0020,   /* U+2F8AF */
  0x0020,   /* U+2F8B0 */
  0x0020,   /* U+2F8B1 */
  0x0020,   /* U+2F8B2 */
  0x0020,   /* U+2F8B3 */
  0x0020,   /* U+2F8B4 */
  0x0020,   /* U+2F8B5 */
  0x0020,   /* U+2F8B6 */
  0x0020,   /* U+2F8B7 */
  0x0020,   /* U+2F8B8 */
  0x0020,   /* U+2F8B9 */
  0x0020,   /* U+2F8BA */
  0x0020,   /* U+2F8BB */
  0x0020,   /* U+2F8BC */
  0x0020,   /* U+2F8BD */
  0x0020,   /* U+2F8BE */
  0x0020,   /* U+2F8BF */
  0x0020,   /* U+2F8C0 */
  0x0020,   /* U+2F8C1 */
  0x0020,   /* U+2F8C2 */
  0x0020,   /* U+2F8C3 */
  0x0020,   /* U+2F8C4 */
  0x0020,   /* U+2F8C5 */
  0x0020,   /* U+2F8C6 */
  0x0020,   /* U+2F8C7 */
  0x0020,   /* U+2F8C8 */
  0x0020,   /* U+2F8C9 */
  0x0020,   /* U+2F8CA */
  0x0020,   /* U+2F8CB */
  0x0020,   /* U+2F8CC */
  0x0020,   /* U+2F8CD */
  0x0020,   /* U+2F8CE */
  0x0020,   /* U+2F8CF */
  0x0020,   /* U+2F8D0 */
  0x0020,   /* U+2F8D1 */
  0x0020,   /* U+2F8D2 */
  0x0020,   /* U+2F8D3 */
  0x0020,   /* U+2F8D4 */
  0x0020,   /* U+2F8D5 */
  0x0020,   /* U+2F8D6 */
  0x0020,   /* U+2F8D7 */
  0x0020,   /* U+2F8D8 */
  0x0020,   /* U+2F8D9 */
  0x0020,   /* U+2F8DA */
  0x0020,   /* U+2F8DB */
  0x0020,   /* U+2F8DC */
  0x0020,   /* U+2F8DD */
  0x0020,   /* U+2F8DE */
  0x0020,   /* U+2F8DF */
  0x0020,   /* U+2F8E0 */
  0x0020,   /* U+2F8E1 */
  0x0020,   /* U+2F8E2 */
  0x0020,   /* U+2F8E3 */
  0x0020,   /* U+2F8E4 */
  0x0020,   /* U+2F8E5 */
  0x0020,   /* U+2F8E6 */
  0x0020,   /* U+2F8E7 */
  0x0020,   /* U+2F8E8 */
  0x0020,   /* U+2F8E9 */
  0x0020,   /* U+2F8EA */
  0x0020,   /* U+2F8EB */
  0x0020,   /* U+2F8EC */
  0x0020,   /* U+2F8ED */
  0x0020,   /* U+2F8EE */
  0x0020,   /* U+2F8EF */
  0x0020,   /* U+2F8F0 */
  0x0020,   /* U+2F8F1 */
  0x0020,   /* U+2F8F2 */
  0x0020,   /* U+2F8F3 */
  0x0020,   /* U+2F8F4 */
  0x0020,   /* U+2F8F5 */
  0x0020,   /* U+2F8F6 */
  0x0020,   /* U+2F8F7 */
  0x0020,   /* U+2F8F8 */
  0x0020,   /* U+2F8F9 */
  0x0020,   /* U+2F8FA */
  0x0020,   /* U+2F8FB */
  0x0020,   /* U+2F8FC */
  0x0020,   /* U+2F8FD */
  0x0020,   /* U+2F8FE */
  0x0020,   /* U+2F8FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2F800 */
  0x0002,   /* U+2F801 */
  0x0002,   /* U+2F802 */
  0x0002,   /* U+2F803 */
  0x0002,   /* U+2F804 */
  0x0002,   /* U+2F805 */
  0x0002,   /* U+2F806 */
  0x0002,   /* U+2F807 */
  0x0002,   /* U+2F808 */
  0x0002,   /* U+2F809 */
  0x0002,   /* U+2F80A */
  0x0002,   /* U+2F80B */
  0x0002,   /* U+2F80C */
  0x0002,   /* U+2F80D */
  0x0002,   /* U+2F80E */
  0x0002,   /* U+2F80F */
  0x0002,   /* U+2F810 */
  0x0002,   /* U+2F811 */
  0x0002,   /* U+2F812 */
  0x0002,   /* U+2F813 */
  0x0002,   /* U+2F814 */
  0x0002,   /* U+2F815 */
  0x0002,   /* U+2F816 */
  0x0002,   /* U+2F817 */
  0x0002,   /* U+2F818 */
  0x0002,   /* U+2F819 */
  0x0002,   /* U+2F81A */
  0x0002,   /* U+2F81B */
  0x0002,   /* U+2F81C */
  0x0002,   /* U+2F81D */
  0x0002,   /* U+2F81E */
  0x0002,   /* U+2F81F */
  0x0002,   /* U+2F820 */
  0x0002,   /* U+2F821 */
  0x0002,   /* U+2F822 */
  0x0002,   /* U+2F823 */
  0x0002,   /* U+2F824 */
  0x0002,   /* U+2F825 */
  0x0002,   /* U+2F826 */
  0x0002,   /* U+2F827 */
  0x0002,   /* U+2F828 */
  0x0002,   /* U+2F829 */
  0x0002,   /* U+2F82A */
  0x0002,   /* U+2F82B */
  0x0002,   /* U+2F82C */
  0x0002,   /* U+2F82D */
  0x0002,   /* U+2F82E */
  0x0002,   /* U+2F82F */
  0x0002,   /* U+2F830 */
  0x0002,   /* U+2F831 */
  0x0002,   /* U+2F832 */
  0x0002,   /* U+2F833 */
  0x0002,   /* U+2F834 */
  0x0002,   /* U+2F835 */
  0x0002,   /* U+2F836 */
  0x0002,   /* U+2F837 */
  0x0002,   /* U+2F838 */
  0x0002,   /* U+2F839 */
  0x0002,   /* U+2F83A */
  0x0002,   /* U+2F83B */
  0x0002,   /* U+2F83C */
  0x0002,   /* U+2F83D */
  0x0002,   /* U+2F83E */
  0x0002,   /* U+2F83F */
  0x0002,   /* U+2F840 */
  0x0002,   /* U+2F841 */
  0x0002,   /* U+2F842 */
  0x0002,   /* U+2F843 */
  0x0002,   /* U+2F844 */
  0x0002,   /* U+2F845 */
  0x0002,   /* U+2F846 */
  0x0002,   /* U+2F847 */
  0x0002,   /* U+2F848 */
  0x0002,   /* U+2F849 */
  0x0002,   /* U+2F84A */
  0x0002,   /* U+2F84B */
  0x0002,   /* U+2F84C */
  0x0002,   /* U+2F84D */
  0x0002,   /* U+2F84E */
  0x0002,   /* U+2F84F */
  0x0002,   /* U+2F850 */
  0x0002,   /* U+2F851 */
  0x0002,   /* U+2F852 */
  0x0002,   /* U+2F853 */
  0x0002,   /* U+2F854 */
  0x0002,   /* U+2F855 */
  0x0002,   /* U+2F856 */
  0x0002,   /* U+2F857 */
  0x0002,   /* U+2F858 */
  0x0002,   /* U+2F859 */
  0x0002,   /* U+2F85A */
  0x0002,   /* U+2F85B */
  0x0002,   /* U+2F85C */
  0x0002,   /* U+2F85D */
  0x0002,   /* U+2F85E */
  0x0002,   /* U+2F85F */
  0x0002,   /* U+2F860 */
  0x0002,   /* U+2F861 */
  0x0002,   /* U+2F862 */
  0x0002,   /* U+2F863 */
  0x0002,   /* U+2F864 */
  0x0002,   /* U+2F865 */
  0x0002,   /* U+2F866 */
  0x0002,   /* U+2F867 */
  0x0002,   /* U+2F868 */
  0x0002,   /* U+2F869 */
  0x0002,   /* U+2F86A */
  0x0002,   /* U+2F86B */
  0x0002,   /* U+2F86C */
  0x0002,   /* U+2F86D */
  0x0002,   /* U+2F86E */
  0x0002,   /* U+2F86F */
  0x0002,   /* U+2F870 */
  0x0002,   /* U+2F871 */
  0x0002,   /* U+2F872 */
  0x0002,   /* U+2F873 */
  0x0002,   /* U+2F874 */
  0x0002,   /* U+2F875 */
  0x0002,   /* U+2F876 */
  0x0002,   /* U+2F877 */
  0x0002,   /* U+2F878 */
  0x0002,   /* U+2F879 */
  0x0002,   /* U+2F87A */
  0x0002,   /* U+2F87B */
  0x0002,   /* U+2F87C */
  0x0002,   /* U+2F87D */
  0x0002,   /* U+2F87E */
  0x0002,   /* U+2F87F */
  0x0002,   /* U+2F880 */
  0x0002,   /* U+2F881 */
  0x0002,   /* U+2F882 */
  0x0002,   /* U+2F883 */
  0x0002,   /* U+2F884 */
  0x0002,   /* U+2F885 */
  0x0002,   /* U+2F886 */
  0x0002,   /* U+2F887 */
  0x0002,   /* U+2F888 */
  0x0002,   /* U+2F889 */
  0x0002,   /* U+2F88A */
  0x0002,   /* U+2F88B */
  0x0002,   /* U+2F88C */
  0x0002,   /* U+2F88D */
  0x0002,   /* U+2F88E */
  0x0002,   /* U+2F88F */
  0x0002,   /* U+2F890 */
  0x0002,   /* U+2F891 */
  0x0002,   /* U+2F892 */
  0x0002,   /* U+2F893 */
  0x0002,   /* U+2F894 */
  0x0002,   /* U+2F895 */
  0x0002,   /* U+2F896 */
  0x0002,   /* U+2F897 */
  0x0002,   /* U+2F898 */
  0x0002,   /* U+2F899 */
  0x0002,   /* U+2F89A */
  0x0002,   /* U+2F89B */
  0x0002,   /* U+2F89C */
  0x0002,   /* U+2F89D */
  0x0002,   /* U+2F89E */
  0x0002,   /* U+2F89F */
  0x0002,   /* U+2F8A0 */
  0x0002,   /* U+2F8A1 */
  0x0002,   /* U+2F8A2 */
  0x0002,   /* U+2F8A3 */
  0x0002,   /* U+2F8A4 */
  0x0002,   /* U+2F8A5 */
  0x0002,   /* U+2F8A6 */
  0x0002,   /* U+2F8A7 */
  0x0002,   /* U+2F8A8 */
  0x0002,   /* U+2F8A9 */
  0x0002,   /* U+2F8AA */
  0x0002,   /* U+2F8AB */
  0x0002,   /* U+2F8AC */
  0x0002,   /* U+2F8AD */
  0x0002,   /* U+2F8AE */
  0x0002,   /* U+2F8AF */
  0x0002,   /* U+2F8B0 */
  0x0002,   /* U+2F8B1 */
  0x0002,   /* U+2F8B2 */
  0x0002,   /* U+2F8B3 */
  0x0002,   /* U+2F8B4 */
  0x0002,   /* U+2F8B5 */
  0x0002,   /* U+2F8B6 */
  0x0002,   /* U+2F8B7 */
  0x0002,   /* U+2F8B8 */
  0x0002,   /* U+2F8B9 */
  0x0002,   /* U+2F8BA */
  0x0002,   /* U+2F8BB */
  0x0002,   /* U+2F8BC */
  0x0002,   /* U+2F8BD */
  0x0002,   /* U+2F8BE */
  0x0002,   /* U+2F8BF */
  0x0002,   /* U+2F8C0 */
  0x0002,   /* U+2F8C1 */
  0x0002,   /* U+2F8C2 */
  0x0002,   /* U+2F8C3 */
  0x0002,   /* U+2F8C4 */
  0x0002,   /* U+2F8C5 */
  0x0002,   /* U+2F8C6 */
  0x0002,   /* U+2F8C7 */
  0x0002,   /* U+2F8C8 */
  0x0002,   /* U+2F8C9 */
  0x0002,   /* U+2F8CA */
  0x0002,   /* U+2F8CB */
  0x0002,   /* U+2F8CC */
  0x0002,   /* U+2F8CD */
  0x0002,   /* U+2F8CE */
  0x0002,   /* U+2F8CF */
  0x0002,   /* U+2F8D0 */
  0x0002,   /* U+2F8D1 */
  0x0002,   /* U+2F8D2 */
  0x0002,   /* U+2F8D3 */
  0x0002,   /* U+2F8D4 */
  0x0002,   /* U+2F8D5 */
  0x0002,   /* U+2F8D6 */
  0x0002,   /* U+2F8D7 */
  0x0002,   /* U+2F8D8 */
  0x0002,   /* U+2F8D9 */
  0x0002,   /* U+2F8DA */
  0x0002,   /* U+2F8DB */
  0x0002,   /* U+2F8DC */
  0x0002,   /* U+2F8DD */
  0x0002,   /* U+2F8DE */
  0x0002,   /* U+2F8DF */
  0x0002,   /* U+2F8E0 */
  0x0002,   /* U+2F8E1 */
  0x0002,   /* U+2F8E2 */
  0x0002,   /* U+2F8E3 */
  0x0002,   /* U+2F8E4 */
  0x0002,   /* U+2F8E5 */
  0x0002,   /* U+2F8E6 */
  0x0002,   /* U+2F8E7 */
  0x0002,   /* U+2F8E8 */
  0x0002,   /* U+2F8E9 */
  0x0002,   /* U+2F8EA */
  0x0002,   /* U+2F8EB */
  0x0002,   /* U+2F8EC */
  0x0002,   /* U+2F8ED */
  0x0002,   /* U+2F8EE */
  0x0002,   /* U+2F8EF */
  0x0002,   /* U+2F8F0 */
  0x0002,   /* U+2F8F1 */
  0x0002,   /* U+2F8F2 */
  0x0002,   /* U+2F8F3 */
  0x0002,   /* U+2F8F4 */
  0x0002,   /* U+2F8F5 */
  0x0002,   /* U+2F8F6 */
  0x0002,   /* U+2F8F7 */
  0x0002,   /* U+2F8F8 */
  0x0002,   /* U+2F8F9 */
  0x0002,   /* U+2F8FA */
  0x0002,   /* U+2F8FB */
  0x0002,   /* U+2F8FC */
  0x0002,   /* U+2F8FD */
  0x0002,   /* U+2F8FE */
  0x0002,   /* U+2F8FF */

  /* Primary weight 2 for each character. */
  0xCE3D,   /* U+2F800 */
  0xCE38,   /* U+2F801 */
  0xCE41,   /* U+2F802 */
  0x8122,   /* U+2F803 */
  0xCF60,   /* U+2F804 */
  0xCFAE,   /* U+2F805 */
  0xCFBB,   /* U+2F806 */
  0xD002,   /* U+2F807 */
  0xD07A,   /* U+2F808 */
  0xD099,   /* U+2F809 */
  0xD0E7,   /* U+2F80A */
  0xD0CF,   /* U+2F80B */
  0xB49E,   /* U+2F80C */
  0x863A,   /* U+2F80D */
  0xD14D,   /* U+2F80E */
  0xD154,   /* U+2F80F */
  0xD164,   /* U+2F810 */
  0xD177,   /* U+2F811 */
  0x851C,   /* U+2F812 */
  0xB4B9,   /* U+2F813 */
  0xD167,   /* U+2F814 */
  0xD18D,   /* U+2F815 */
  0x854B,   /* U+2F816 */
  0xD197,   /* U+2F817 */
  0xD1A4,   /* U+2F818 */
  0xCECC,   /* U+2F819 */
  0xD1AC,   /* U+2F81A */
  0xD1B5,   /* U+2F81B */
  0x91DF,   /* U+2F81C */
  0xD1F5,   /* U+2F81D */
  0xD203,   /* U+2F81E */
  0xB4DF,   /* U+2F81F */
  0xD23B,   /* U+2F820 */
  0xD246,   /* U+2F821 */
  0xD272,   /* U+2F822 */
  0xD277,   /* U+2F823 */
  0xB515,   /* U+2F824 */
  0xD2C7,   /* U+2F825 */
  0xD2C9,   /* U+2F826 */
  0xD2E4,   /* U+2F827 */
  0xD2FA,   /* U+2F828 */
  0xD305,   /* U+2F829 */
  0xD306,   /* U+2F82A */
  0xD317,   /* U+2F82B */
  0xD349,   /* U+2F82C */
  0xD351,   /* U+2F82D */
  0xD35A,   /* U+2F82E */
  0xD373,   /* U+2F82F */
  0xD37D,   /* U+2F830 */
  0xD37F,   /* U+2F831 */
  0xD37F,   /* U+2F832 */
  0xD37F,   /* U+2F833 */
  0x8A2C,   /* U+2F834 */
  0xF070,   /* U+2F835 */
  0xD3CA,   /* U+2F836 */
  0xD3DF,   /* U+2F837 */
  0x8B63,   /* U+2F838 */
  0xD3EB,   /* U+2F839 */
  0xD3F1,   /* U+2F83A */
  0xD406,   /* U+2F83B */
  0xD49E,   /* U+2F83C */
  0xD438,   /* U+2F83D */
  0xD448,   /* U+2F83E */
  0xD468,   /* U+2F83F */
  0xD4A2,   /* U+2F840 */
  0xD4F6,   /* U+2F841 */
  0xD510,   /* U+2F842 */
  0xD553,   /* U+2F843 */
  0xD563,   /* U+2F844 */
  0xD584,   /* U+2F845 */
  0xD584,   /* U+2F846 */
  0xD599,   /* U+2F847 */
  0xD5AB,   /* U+2F848 */
  0xD5B3,   /* U+2F849 */
  0xD5C2,   /* U+2F84A */
  0xD716,   /* U+2F84B */
  0xD606,   /* U+2F84C */
  0xD717,   /* U+2F84D */
  0xD651,   /* U+2F84E */
  0xD674,   /* U+2F84F */
  0xD207,   /* U+2F850 */
  0xD8EE,   /* U+2F851 */
  0xD7CE,   /* U+2F852 */
  0xD7F4,   /* U+2F853 */
  0xD80D,   /* U+2F854 */
  0xD78B,   /* U+2F855 */
  0xD832,   /* U+2F856 */
  0xD831,   /* U+2F857 */
  0xD8AC,   /* U+2F858 */
  0x94E4,   /* U+2F859 */
  0xD8F2,   /* U+2F85A */
  0xD8F7,   /* U+2F85B */
  0xD906,   /* U+2F85C */
  0xD91A,   /* U+2F85D */
  0xD922,   /* U+2F85E */
  0xD962,   /* U+2F85F */
  0x96A8,   /* U+2F860 */
  0x96EA,   /* U+2F861 */
  0xD9EC,   /* U+2F862 */
  0xDA1B,   /* U+2F863 */
  0xDA27,   /* U+2F864 */
  0xD9D8,   /* U+2F865 */
  0xDA66,   /* U+2F866 */
  0xB6EE,   /* U+2F867 */
  0xB6FC,   /* U+2F868 */
  0xDB08,   /* U+2F869 */
  0xDB3E,   /* U+2F86A */
  0xDB3E,   /* U+2F86B */
  0x99C8,   /* U+2F86C */
  0xDBC3,   /* U+2F86D */
  0xDBD8,   /* U+2F86E */
  0xDBE7,   /* U+2F86F */
  0xDBF3,   /* U+2F870 */
  0x9B18,   /* U+2F871 */
  0xDBFF,   /* U+2F872 */
  0xDC06,   /* U+2F873 */
  0xDF53,   /* U+2F874 */
  0xDC22,   /* U+2F875 */
  0xB781,   /* U+2F876 */
  0xDC60,   /* U+2F877 */
  0xDC6E,   /* U+2F878 */
  0xDCC0,   /* U+2F879 */
  0xDC8D,   /* U+2F87A */
  0x9DE4,   /* U+2F87B */
  0xDD43,   /* U+2F87C */
  0x9DE6,   /* U+2F87D */
  0xDD6E,   /* U+2F87E */
  0xDD6B,   /* U+2F87F */
  0xDD7C,   /* U+2F880 */
  0xDDE1,   /* U+2F881 */
  0xDDE2,   /* U+2F882 */
  0xB82F,   /* U+2F883 */
  0xDDFD,   /* U+2F884 */
  0xDE28,   /* U+2F885 */
  0xDE3D,   /* U+2F886 */
  0xDE69,   /* U+2F887 */
  0xB862,   /* U+2F888 */
  0xA183,   /* U+2F889 */
  0xB87C,   /* U+2F88A */
  0xDEB0,   /* U+2F88B */
  0xDEB3,   /* U+2F88C */
  0xDEB6,   /* U+2F88D */
  0xDECA,   /* U+2F88E */
  0xA392,   /* U+2F88F */
  0xDEFE,   /* U+2F890 */
  0xA331,   /* U+2F891 */
  0xA331,   /* U+2F892 */
  0x8201,   /* U+2F893 */
  0xDF22,   /* U+2F894 */
  0xDF22,   /* U+2F895 */
  0xB8C7,   /* U+2F896 */
  0xB2B8,   /* U+2F897 */
  0xE1DA,   /* U+2F898 */
  0xDF62,   /* U+2F899 */
  0xDF6B,   /* U+2F89A */
  0xB8E3,   /* U+2F89B */
  0xDF9A,   /* U+2F89C */
  0xDFCD,   /* U+2F89D */
  0xDFD7,   /* U+2F89E */
  0xDFF9,   /* U+2F89F */
  0xE081,   /* U+2F8A0 */
  0xB93A,   /* U+2F8A1 */
  0xB91C,   /* U+2F8A2 */
  0xE094,   /* U+2F8A3 */
  0xA6D4,   /* U+2F8A4 */
  0xE0C7,   /* U+2F8A5 */
  0xE148,   /* U+2F8A6 */
  0xE14C,   /* U+2F8A7 */
  0xE14E,   /* U+2F8A8 */
  0xE14C,   /* U+2F8A9 */
  0xE17A,   /* U+2F8AA */
  0xE18E,   /* U+2F8AB */
  0xE1B2,   /* U+2F8AC */
  0xE1A4,   /* U+2F8AD */
  0xE1AF,   /* U+2F8AE */
  0xE1DE,   /* U+2F8AF */
  0xE1F2,   /* U+2F8B0 */
  0xE1F6,   /* U+2F8B1 */
  0xE210,   /* U+2F8B2 */
  0xE21B,   /* U+2F8B3 */
  0xE25D,   /* U+2F8B4 */
  0xE2B1,   /* U+2F8B5 */
  0xE2D4,   /* U+2F8B6 */
  0xE350,   /* U+2F8B7 */
  0xAB0C,   /* U+2F8B8 */
  0xE33D,   /* U+2F8B9 */
  0xE2FC,   /* U+2F8BA */
  0xE368,   /* U+2F8BB */
  0xE383,   /* U+2F8BC */
  0xE3E4,   /* U+2F8BD */
  0xABF1,   /* U+2F8BE */
  0xE422,   /* U+2F8BF */
  0xE3C5,   /* U+2F8C0 */
  0xE3A9,   /* U+2F8C1 */
  0xBA2E,   /* U+2F8C2 */
  0xE469,   /* U+2F8C3 */
  0xE47E,   /* U+2F8C4 */
  0xE49D,   /* U+2F8C5 */
  0xE477,   /* U+2F8C6 */
  0xBA6C,   /* U+2F8C7 */
  0xE54F,   /* U+2F8C8 */
  0xE56C,   /* U+2F8C9 */
  0xB00A,   /* U+2F8CA */
  0xE5E3,   /* U+2F8CB */
  0xE6F8,   /* U+2F8CC */
  0xE649,   /* U+2F8CD */
  0xBB19,   /* U+2F8CE */
  0xE691,   /* U+2F8CF */
  0xBB08,   /* U+2F8D0 */
  0xBAE4,   /* U+2F8D1 */
  0xD192,   /* U+2F8D2 */
  0xD195,   /* U+2F8D3 */
  0xE700,   /* U+2F8D4 */
  0xE69C,   /* U+2F8D5 */
  0x80AD,   /* U+2F8D6 */
  0xC3D9,   /* U+2F8D7 */
  0xE717,   /* U+2F8D8 */
  0xE71B,   /* U+2F8D9 */
  0xE721,   /* U+2F8DA */
  0xE75E,   /* U+2F8DB */
  0xE753,   /* U+2F8DC */
  0xB3C3,   /* U+2F8DD */
  0xBB49,   /* U+2F8DE */
  0xE7FA,   /* U+2F8DF */
  0xE785,   /* U+2F8E0 */
  0xE852,   /* U+2F8E1 */
  0xE885,   /* U+2F8E2 */
  0xB46D,   /* U+2F8E3 */
  0xE88E,   /* U+2F8E4 */
  0xE81F,   /* U+2F8E5 */
  0xE914,   /* U+2F8E6 */
  0xBB9D,   /* U+2F8E7 */
  0xE942,   /* U+2F8E8 */
  0xE9A3,   /* U+2F8E9 */
  0xE9EA,   /* U+2F8EA */
  0xEAA8,   /* U+2F8EB */
  0xB6A3,   /* U+2F8EC */
  0xEADB,   /* U+2F8ED */
  0xBC18,   /* U+2F8EE */
  0xEB21,   /* U+2F8EF */
  0xB8A7,   /* U+2F8F0 */
  0xEB54,   /* U+2F8F1 */
  0xBC4E,   /* U+2F8F2 */
  0xEB72,   /* U+2F8F3 */
  0xEB9F,   /* U+2F8F4 */
  0xEBBA,   /* U+2F8F5 */
  0xEBBB,   /* U+2F8F6 */
  0xBA8D,   /* U+2F8F7 */
  0x9D0B,   /* U+2F8F8 */
  0xBAFA,   /* U+2F8F9 */
  0xEC4E,   /* U+2F8FA */
  0xBCBC,   /* U+2F8FB */
  0xECBF,   /* U+2F8FC */
  0xECCD,   /* U+2F8FD */
  0xEC67,   /* U+2F8FE */
  0xED16,   /* U+2F8FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2F800 */
  0x0000,   /* U+2F801 */
  0x0000,   /* U+2F802 */
  0x0000,   /* U+2F803 */
  0x0000,   /* U+2F804 */
  0x0000,   /* U+2F805 */
  0x0000,   /* U+2F806 */
  0x0000,   /* U+2F807 */
  0x0000,   /* U+2F808 */
  0x0000,   /* U+2F809 */
  0x0000,   /* U+2F80A */
  0x0000,   /* U+2F80B */
  0x0000,   /* U+2F80C */
  0x0000,   /* U+2F80D */
  0x0000,   /* U+2F80E */
  0x0000,   /* U+2F80F */
  0x0000,   /* U+2F810 */
  0x0000,   /* U+2F811 */
  0x0000,   /* U+2F812 */
  0x0000,   /* U+2F813 */
  0x0000,   /* U+2F814 */
  0x0000,   /* U+2F815 */
  0x0000,   /* U+2F816 */
  0x0000,   /* U+2F817 */
  0x0000,   /* U+2F818 */
  0x0000,   /* U+2F819 */
  0x0000,   /* U+2F81A */
  0x0000,   /* U+2F81B */
  0x0000,   /* U+2F81C */
  0x0000,   /* U+2F81D */
  0x0000,   /* U+2F81E */
  0x0000,   /* U+2F81F */
  0x0000,   /* U+2F820 */
  0x0000,   /* U+2F821 */
  0x0000,   /* U+2F822 */
  0x0000,   /* U+2F823 */
  0x0000,   /* U+2F824 */
  0x0000,   /* U+2F825 */
  0x0000,   /* U+2F826 */
  0x0000,   /* U+2F827 */
  0x0000,   /* U+2F828 */
  0x0000,   /* U+2F829 */
  0x0000,   /* U+2F82A */
  0x0000,   /* U+2F82B */
  0x0000,   /* U+2F82C */
  0x0000,   /* U+2F82D */
  0x0000,   /* U+2F82E */
  0x0000,   /* U+2F82F */
  0x0000,   /* U+2F830 */
  0x0000,   /* U+2F831 */
  0x0000,   /* U+2F832 */
  0x0000,   /* U+2F833 */
  0x0000,   /* U+2F834 */
  0x0000,   /* U+2F835 */
  0x0000,   /* U+2F836 */
  0x0000,   /* U+2F837 */
  0x0000,   /* U+2F838 */
  0x0000,   /* U+2F839 */
  0x0000,   /* U+2F83A */
  0x0000,   /* U+2F83B */
  0x0000,   /* U+2F83C */
  0x0000,   /* U+2F83D */
  0x0000,   /* U+2F83E */
  0x0000,   /* U+2F83F */
  0x0000,   /* U+2F840 */
  0x0000,   /* U+2F841 */
  0x0000,   /* U+2F842 */
  0x0000,   /* U+2F843 */
  0x0000,   /* U+2F844 */
  0x0000,   /* U+2F845 */
  0x0000,   /* U+2F846 */
  0x0000,   /* U+2F847 */
  0x0000,   /* U+2F848 */
  0x0000,   /* U+2F849 */
  0x0000,   /* U+2F84A */
  0x0000,   /* U+2F84B */
  0x0000,   /* U+2F84C */
  0x0000,   /* U+2F84D */
  0x0000,   /* U+2F84E */
  0x0000,   /* U+2F84F */
  0x0000,   /* U+2F850 */
  0x0000,   /* U+2F851 */
  0x0000,   /* U+2F852 */
  0x0000,   /* U+2F853 */
  0x0000,   /* U+2F854 */
  0x0000,   /* U+2F855 */
  0x0000,   /* U+2F856 */
  0x0000,   /* U+2F857 */
  0x0000,   /* U+2F858 */
  0x0000,   /* U+2F859 */
  0x0000,   /* U+2F85A */
  0x0000,   /* U+2F85B */
  0x0000,   /* U+2F85C */
  0x0000,   /* U+2F85D */
  0x0000,   /* U+2F85E */
  0x0000,   /* U+2F85F */
  0x0000,   /* U+2F860 */
  0x0000,   /* U+2F861 */
  0x0000,   /* U+2F862 */
  0x0000,   /* U+2F863 */
  0x0000,   /* U+2F864 */
  0x0000,   /* U+2F865 */
  0x0000,   /* U+2F866 */
  0x0000,   /* U+2F867 */
  0x0000,   /* U+2F868 */
  0x0000,   /* U+2F869 */
  0x0000,   /* U+2F86A */
  0x0000,   /* U+2F86B */
  0x0000,   /* U+2F86C */
  0x0000,   /* U+2F86D */
  0x0000,   /* U+2F86E */
  0x0000,   /* U+2F86F */
  0x0000,   /* U+2F870 */
  0x0000,   /* U+2F871 */
  0x0000,   /* U+2F872 */
  0x0000,   /* U+2F873 */
  0x0000,   /* U+2F874 */
  0x0000,   /* U+2F875 */
  0x0000,   /* U+2F876 */
  0x0000,   /* U+2F877 */
  0x0000,   /* U+2F878 */
  0x0000,   /* U+2F879 */
  0x0000,   /* U+2F87A */
  0x0000,   /* U+2F87B */
  0x0000,   /* U+2F87C */
  0x0000,   /* U+2F87D */
  0x0000,   /* U+2F87E */
  0x0000,   /* U+2F87F */
  0x0000,   /* U+2F880 */
  0x0000,   /* U+2F881 */
  0x0000,   /* U+2F882 */
  0x0000,   /* U+2F883 */
  0x0000,   /* U+2F884 */
  0x0000,   /* U+2F885 */
  0x0000,   /* U+2F886 */
  0x0000,   /* U+2F887 */
  0x0000,   /* U+2F888 */
  0x0000,   /* U+2F889 */
  0x0000,   /* U+2F88A */
  0x0000,   /* U+2F88B */
  0x0000,   /* U+2F88C */
  0x0000,   /* U+2F88D */
  0x0000,   /* U+2F88E */
  0x0000,   /* U+2F88F */
  0x0000,   /* U+2F890 */
  0x0000,   /* U+2F891 */
  0x0000,   /* U+2F892 */
  0x0000,   /* U+2F893 */
  0x0000,   /* U+2F894 */
  0x0000,   /* U+2F895 */
  0x0000,   /* U+2F896 */
  0x0000,   /* U+2F897 */
  0x0000,   /* U+2F898 */
  0x0000,   /* U+2F899 */
  0x0000,   /* U+2F89A */
  0x0000,   /* U+2F89B */
  0x0000,   /* U+2F89C */
  0x0000,   /* U+2F89D */
  0x0000,   /* U+2F89E */
  0x0000,   /* U+2F89F */
  0x0000,   /* U+2F8A0 */
  0x0000,   /* U+2F8A1 */
  0x0000,   /* U+2F8A2 */
  0x0000,   /* U+2F8A3 */
  0x0000,   /* U+2F8A4 */
  0x0000,   /* U+2F8A5 */
  0x0000,   /* U+2F8A6 */
  0x0000,   /* U+2F8A7 */
  0x0000,   /* U+2F8A8 */
  0x0000,   /* U+2F8A9 */
  0x0000,   /* U+2F8AA */
  0x0000,   /* U+2F8AB */
  0x0000,   /* U+2F8AC */
  0x0000,   /* U+2F8AD */
  0x0000,   /* U+2F8AE */
  0x0000,   /* U+2F8AF */
  0x0000,   /* U+2F8B0 */
  0x0000,   /* U+2F8B1 */
  0x0000,   /* U+2F8B2 */
  0x0000,   /* U+2F8B3 */
  0x0000,   /* U+2F8B4 */
  0x0000,   /* U+2F8B5 */
  0x0000,   /* U+2F8B6 */
  0x0000,   /* U+2F8B7 */
  0x0000,   /* U+2F8B8 */
  0x0000,   /* U+2F8B9 */
  0x0000,   /* U+2F8BA */
  0x0000,   /* U+2F8BB */
  0x0000,   /* U+2F8BC */
  0x0000,   /* U+2F8BD */
  0x0000,   /* U+2F8BE */
  0x0000,   /* U+2F8BF */
  0x0000,   /* U+2F8C0 */
  0x0000,   /* U+2F8C1 */
  0x0000,   /* U+2F8C2 */
  0x0000,   /* U+2F8C3 */
  0x0000,   /* U+2F8C4 */
  0x0000,   /* U+2F8C5 */
  0x0000,   /* U+2F8C6 */
  0x0000,   /* U+2F8C7 */
  0x0000,   /* U+2F8C8 */
  0x0000,   /* U+2F8C9 */
  0x0000,   /* U+2F8CA */
  0x0000,   /* U+2F8CB */
  0x0000,   /* U+2F8CC */
  0x0000,   /* U+2F8CD */
  0x0000,   /* U+2F8CE */
  0x0000,   /* U+2F8CF */
  0x0000,   /* U+2F8D0 */
  0x0000,   /* U+2F8D1 */
  0x0000,   /* U+2F8D2 */
  0x0000,   /* U+2F8D3 */
  0x0000,   /* U+2F8D4 */
  0x0000,   /* U+2F8D5 */
  0x0000,   /* U+2F8D6 */
  0x0000,   /* U+2F8D7 */
  0x0000,   /* U+2F8D8 */
  0x0000,   /* U+2F8D9 */
  0x0000,   /* U+2F8DA */
  0x0000,   /* U+2F8DB */
  0x0000,   /* U+2F8DC */
  0x0000,   /* U+2F8DD */
  0x0000,   /* U+2F8DE */
  0x0000,   /* U+2F8DF */
  0x0000,   /* U+2F8E0 */
  0x0000,   /* U+2F8E1 */
  0x0000,   /* U+2F8E2 */
  0x0000,   /* U+2F8E3 */
  0x0000,   /* U+2F8E4 */
  0x0000,   /* U+2F8E5 */
  0x0000,   /* U+2F8E6 */
  0x0000,   /* U+2F8E7 */
  0x0000,   /* U+2F8E8 */
  0x0000,   /* U+2F8E9 */
  0x0000,   /* U+2F8EA */
  0x0000,   /* U+2F8EB */
  0x0000,   /* U+2F8EC */
  0x0000,   /* U+2F8ED */
  0x0000,   /* U+2F8EE */
  0x0000,   /* U+2F8EF */
  0x0000,   /* U+2F8F0 */
  0x0000,   /* U+2F8F1 */
  0x0000,   /* U+2F8F2 */
  0x0000,   /* U+2F8F3 */
  0x0000,   /* U+2F8F4 */
  0x0000,   /* U+2F8F5 */
  0x0000,   /* U+2F8F6 */
  0x0000,   /* U+2F8F7 */
  0x0000,   /* U+2F8F8 */
  0x0000,   /* U+2F8F9 */
  0x0000,   /* U+2F8FA */
  0x0000,   /* U+2F8FB */
  0x0000,   /* U+2F8FC */
  0x0000,   /* U+2F8FD */
  0x0000,   /* U+2F8FE */
  0x0000,   /* U+2F8FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2F800 */
  0x0000,   /* U+2F801 */
  0x0000,   /* U+2F802 */
  0x0000,   /* U+2F803 */
  0x0000,   /* U+2F804 */
  0x0000,   /* U+2F805 */
  0x0000,   /* U+2F806 */
  0x0000,   /* U+2F807 */
  0x0000,   /* U+2F808 */
  0x0000,   /* U+2F809 */
  0x0000,   /* U+2F80A */
  0x0000,   /* U+2F80B */
  0x0000,   /* U+2F80C */
  0x0000,   /* U+2F80D */
  0x0000,   /* U+2F80E */
  0x0000,   /* U+2F80F */
  0x0000,   /* U+2F810 */
  0x0000,   /* U+2F811 */
  0x0000,   /* U+2F812 */
  0x0000,   /* U+2F813 */
  0x0000,   /* U+2F814 */
  0x0000,   /* U+2F815 */
  0x0000,   /* U+2F816 */
  0x0000,   /* U+2F817 */
  0x0000,   /* U+2F818 */
  0x0000,   /* U+2F819 */
  0x0000,   /* U+2F81A */
  0x0000,   /* U+2F81B */
  0x0000,   /* U+2F81C */
  0x0000,   /* U+2F81D */
  0x0000,   /* U+2F81E */
  0x0000,   /* U+2F81F */
  0x0000,   /* U+2F820 */
  0x0000,   /* U+2F821 */
  0x0000,   /* U+2F822 */
  0x0000,   /* U+2F823 */
  0x0000,   /* U+2F824 */
  0x0000,   /* U+2F825 */
  0x0000,   /* U+2F826 */
  0x0000,   /* U+2F827 */
  0x0000,   /* U+2F828 */
  0x0000,   /* U+2F829 */
  0x0000,   /* U+2F82A */
  0x0000,   /* U+2F82B */
  0x0000,   /* U+2F82C */
  0x0000,   /* U+2F82D */
  0x0000,   /* U+2F82E */
  0x0000,   /* U+2F82F */
  0x0000,   /* U+2F830 */
  0x0000,   /* U+2F831 */
  0x0000,   /* U+2F832 */
  0x0000,   /* U+2F833 */
  0x0000,   /* U+2F834 */
  0x0000,   /* U+2F835 */
  0x0000,   /* U+2F836 */
  0x0000,   /* U+2F837 */
  0x0000,   /* U+2F838 */
  0x0000,   /* U+2F839 */
  0x0000,   /* U+2F83A */
  0x0000,   /* U+2F83B */
  0x0000,   /* U+2F83C */
  0x0000,   /* U+2F83D */
  0x0000,   /* U+2F83E */
  0x0000,   /* U+2F83F */
  0x0000,   /* U+2F840 */
  0x0000,   /* U+2F841 */
  0x0000,   /* U+2F842 */
  0x0000,   /* U+2F843 */
  0x0000,   /* U+2F844 */
  0x0000,   /* U+2F845 */
  0x0000,   /* U+2F846 */
  0x0000,   /* U+2F847 */
  0x0000,   /* U+2F848 */
  0x0000,   /* U+2F849 */
  0x0000,   /* U+2F84A */
  0x0000,   /* U+2F84B */
  0x0000,   /* U+2F84C */
  0x0000,   /* U+2F84D */
  0x0000,   /* U+2F84E */
  0x0000,   /* U+2F84F */
  0x0000,   /* U+2F850 */
  0x0000,   /* U+2F851 */
  0x0000,   /* U+2F852 */
  0x0000,   /* U+2F853 */
  0x0000,   /* U+2F854 */
  0x0000,   /* U+2F855 */
  0x0000,   /* U+2F856 */
  0x0000,   /* U+2F857 */
  0x0000,   /* U+2F858 */
  0x0000,   /* U+2F859 */
  0x0000,   /* U+2F85A */
  0x0000,   /* U+2F85B */
  0x0000,   /* U+2F85C */
  0x0000,   /* U+2F85D */
  0x0000,   /* U+2F85E */
  0x0000,   /* U+2F85F */
  0x0000,   /* U+2F860 */
  0x0000,   /* U+2F861 */
  0x0000,   /* U+2F862 */
  0x0000,   /* U+2F863 */
  0x0000,   /* U+2F864 */
  0x0000,   /* U+2F865 */
  0x0000,   /* U+2F866 */
  0x0000,   /* U+2F867 */
  0x0000,   /* U+2F868 */
  0x0000,   /* U+2F869 */
  0x0000,   /* U+2F86A */
  0x0000,   /* U+2F86B */
  0x0000,   /* U+2F86C */
  0x0000,   /* U+2F86D */
  0x0000,   /* U+2F86E */
  0x0000,   /* U+2F86F */
  0x0000,   /* U+2F870 */
  0x0000,   /* U+2F871 */
  0x0000,   /* U+2F872 */
  0x0000,   /* U+2F873 */
  0x0000,   /* U+2F874 */
  0x0000,   /* U+2F875 */
  0x0000,   /* U+2F876 */
  0x0000,   /* U+2F877 */
  0x0000,   /* U+2F878 */
  0x0000,   /* U+2F879 */
  0x0000,   /* U+2F87A */
  0x0000,   /* U+2F87B */
  0x0000,   /* U+2F87C */
  0x0000,   /* U+2F87D */
  0x0000,   /* U+2F87E */
  0x0000,   /* U+2F87F */
  0x0000,   /* U+2F880 */
  0x0000,   /* U+2F881 */
  0x0000,   /* U+2F882 */
  0x0000,   /* U+2F883 */
  0x0000,   /* U+2F884 */
  0x0000,   /* U+2F885 */
  0x0000,   /* U+2F886 */
  0x0000,   /* U+2F887 */
  0x0000,   /* U+2F888 */
  0x0000,   /* U+2F889 */
  0x0000,   /* U+2F88A */
  0x0000,   /* U+2F88B */
  0x0000,   /* U+2F88C */
  0x0000,   /* U+2F88D */
  0x0000,   /* U+2F88E */
  0x0000,   /* U+2F88F */
  0x0000,   /* U+2F890 */
  0x0000,   /* U+2F891 */
  0x0000,   /* U+2F892 */
  0x0000,   /* U+2F893 */
  0x0000,   /* U+2F894 */
  0x0000,   /* U+2F895 */
  0x0000,   /* U+2F896 */
  0x0000,   /* U+2F897 */
  0x0000,   /* U+2F898 */
  0x0000,   /* U+2F899 */
  0x0000,   /* U+2F89A */
  0x0000,   /* U+2F89B */
  0x0000,   /* U+2F89C */
  0x0000,   /* U+2F89D */
  0x0000,   /* U+2F89E */
  0x0000,   /* U+2F89F */
  0x0000,   /* U+2F8A0 */
  0x0000,   /* U+2F8A1 */
  0x0000,   /* U+2F8A2 */
  0x0000,   /* U+2F8A3 */
  0x0000,   /* U+2F8A4 */
  0x0000,   /* U+2F8A5 */
  0x0000,   /* U+2F8A6 */
  0x0000,   /* U+2F8A7 */
  0x0000,   /* U+2F8A8 */
  0x0000,   /* U+2F8A9 */
  0x0000,   /* U+2F8AA */
  0x0000,   /* U+2F8AB */
  0x0000,   /* U+2F8AC */
  0x0000,   /* U+2F8AD */
  0x0000,   /* U+2F8AE */
  0x0000,   /* U+2F8AF */
  0x0000,   /* U+2F8B0 */
  0x0000,   /* U+2F8B1 */
  0x0000,   /* U+2F8B2 */
  0x0000,   /* U+2F8B3 */
  0x0000,   /* U+2F8B4 */
  0x0000,   /* U+2F8B5 */
  0x0000,   /* U+2F8B6 */
  0x0000,   /* U+2F8B7 */
  0x0000,   /* U+2F8B8 */
  0x0000,   /* U+2F8B9 */
  0x0000,   /* U+2F8BA */
  0x0000,   /* U+2F8BB */
  0x0000,   /* U+2F8BC */
  0x0000,   /* U+2F8BD */
  0x0000,   /* U+2F8BE */
  0x0000,   /* U+2F8BF */
  0x0000,   /* U+2F8C0 */
  0x0000,   /* U+2F8C1 */
  0x0000,   /* U+2F8C2 */
  0x0000,   /* U+2F8C3 */
  0x0000,   /* U+2F8C4 */
  0x0000,   /* U+2F8C5 */
  0x0000,   /* U+2F8C6 */
  0x0000,   /* U+2F8C7 */
  0x0000,   /* U+2F8C8 */
  0x0000,   /* U+2F8C9 */
  0x0000,   /* U+2F8CA */
  0x0000,   /* U+2F8CB */
  0x0000,   /* U+2F8CC */
  0x0000,   /* U+2F8CD */
  0x0000,   /* U+2F8CE */
  0x0000,   /* U+2F8CF */
  0x0000,   /* U+2F8D0 */
  0x0000,   /* U+2F8D1 */
  0x0000,   /* U+2F8D2 */
  0x0000,   /* U+2F8D3 */
  0x0000,   /* U+2F8D4 */
  0x0000,   /* U+2F8D5 */
  0x0000,   /* U+2F8D6 */
  0x0000,   /* U+2F8D7 */
  0x0000,   /* U+2F8D8 */
  0x0000,   /* U+2F8D9 */
  0x0000,   /* U+2F8DA */
  0x0000,   /* U+2F8DB */
  0x0000,   /* U+2F8DC */
  0x0000,   /* U+2F8DD */
  0x0000,   /* U+2F8DE */
  0x0000,   /* U+2F8DF */
  0x0000,   /* U+2F8E0 */
  0x0000,   /* U+2F8E1 */
  0x0000,   /* U+2F8E2 */
  0x0000,   /* U+2F8E3 */
  0x0000,   /* U+2F8E4 */
  0x0000,   /* U+2F8E5 */
  0x0000,   /* U+2F8E6 */
  0x0000,   /* U+2F8E7 */
  0x0000,   /* U+2F8E8 */
  0x0000,   /* U+2F8E9 */
  0x0000,   /* U+2F8EA */
  0x0000,   /* U+2F8EB */
  0x0000,   /* U+2F8EC */
  0x0000,   /* U+2F8ED */
  0x0000,   /* U+2F8EE */
  0x0000,   /* U+2F8EF */
  0x0000,   /* U+2F8F0 */
  0x0000,   /* U+2F8F1 */
  0x0000,   /* U+2F8F2 */
  0x0000,   /* U+2F8F3 */
  0x0000,   /* U+2F8F4 */
  0x0000,   /* U+2F8F5 */
  0x0000,   /* U+2F8F6 */
  0x0000,   /* U+2F8F7 */
  0x0000,   /* U+2F8F8 */
  0x0000,   /* U+2F8F9 */
  0x0000,   /* U+2F8FA */
  0x0000,   /* U+2F8FB */
  0x0000,   /* U+2F8FC */
  0x0000,   /* U+2F8FD */
  0x0000,   /* U+2F8FE */
  0x0000,   /* U+2F8FF */
};

uint16 uca900_p2F9[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB40,   /* U+2F900 */
  0xFB40,   /* U+2F901 */
  0xFB40,   /* U+2F902 */
  0xFB40,   /* U+2F903 */
  0xFB40,   /* U+2F904 */
  0xFB40,   /* U+2F905 */
  0xFB84,   /* U+2F906 */
  0xFB40,   /* U+2F907 */
  0xFB40,   /* U+2F908 */
  0xFB40,   /* U+2F909 */
  0xFB80,   /* U+2F90A */
  0xFB40,   /* U+2F90B */
  0xFB40,   /* U+2F90C */
  0xFB84,   /* U+2F90D */
  0xFB40,   /* U+2F90E */
  0xFB40,   /* U+2F90F */
  0xFB84,   /* U+2F910 */
  0xFB84,   /* U+2F911 */
  0xFB40,   /* U+2F912 */
  0xFB40,   /* U+2F913 */
  0xFB40,   /* U+2F914 */
  0xFB40,   /* U+2F915 */
  0xFB80,   /* U+2F916 */
  0xFB40,   /* U+2F917 */
  0xFB40,   /* U+2F918 */
  0xFB40,   /* U+2F919 */
  0xFB40,   /* U+2F91A */
  0xFB84,   /* U+2F91B */
  0xFB40,   /* U+2F91C */
  0xFB84,   /* U+2F91D */
  0xFB40,   /* U+2F91E */
  0xFB84,   /* U+2F91F */
  0xFB40,   /* U+2F920 */
  0xFB40,   /* U+2F921 */
  0xFB40,   /* U+2F922 */
  0xFB84,   /* U+2F923 */
  0xFB40,   /* U+2F924 */
  0xFB40,   /* U+2F925 */
  0xFB84,   /* U+2F926 */
  0xFB84,   /* U+2F927 */
  0xFB40,   /* U+2F928 */
  0xFB40,   /* U+2F929 */
  0xFB80,   /* U+2F92A */
  0xFB40,   /* U+2F92B */
  0xFB80,   /* U+2F92C */
  0xFB80,   /* U+2F92D */
  0xFB40,   /* U+2F92E */
  0xFB40,   /* U+2F92F */
  0xFB40,   /* U+2F930 */
  0xFB40,   /* U+2F931 */
  0xFB40,   /* U+2F932 */
  0xFB80,   /* U+2F933 */
  0xFB40,   /* U+2F934 */
  0xFB84,   /* U+2F935 */
  0xFB40,   /* U+2F936 */
  0xFB84,   /* U+2F937 */
  0xFB40,   /* U+2F938 */
  0xFB84,   /* U+2F939 */
  0xFB40,   /* U+2F93A */
  0xFB84,   /* U+2F93B */
  0xFB84,   /* U+2F93C */
  0xFB84,   /* U+2F93D */
  0xFB80,   /* U+2F93E */
  0xFB80,   /* U+2F93F */
  0xFB40,   /* U+2F940 */
  0xFB84,   /* U+2F941 */
  0xFB84,   /* U+2F942 */
  0xFB84,   /* U+2F943 */
  0xFB84,   /* U+2F944 */
  0xFB40,   /* U+2F945 */
  0xFB40,   /* U+2F946 */
  0xFB40,   /* U+2F947 */
  0xFB40,   /* U+2F948 */
  0xFB80,   /* U+2F949 */
  0xFB40,   /* U+2F94A */
  0xFB80,   /* U+2F94B */
  0xFB80,   /* U+2F94C */
  0xFB84,   /* U+2F94D */
  0xFB40,   /* U+2F94E */
  0xFB40,   /* U+2F94F */
  0xFB40,   /* U+2F950 */
  0xFB80,   /* U+2F951 */
  0xFB84,   /* U+2F952 */
  0xFB40,   /* U+2F953 */
  0xFB84,   /* U+2F954 */
  0xFB84,   /* U+2F955 */
  0xFB40,   /* U+2F956 */
  0xFB40,   /* U+2F957 */
  0xFB80,   /* U+2F958 */
  0xFB40,   /* U+2F959 */
  0xFB40,   /* U+2F95A */
  0xFB40,   /* U+2F95B */
  0xFB84,   /* U+2F95C */
  0xFB84,   /* U+2F95D */
  0xFB84,   /* U+2F95E */
  0xFB40,   /* U+2F95F */
  0xFB80,   /* U+2F960 */
  0xFB84,   /* U+2F961 */
  0xFB40,   /* U+2F962 */
  0xFB40,   /* U+2F963 */
  0xFB80,   /* U+2F964 */
  0xFB84,   /* U+2F965 */
  0xFB40,   /* U+2F966 */
  0xFB80,   /* U+2F967 */
  0xFB40,   /* U+2F968 */
  0xFB40,   /* U+2F969 */
  0xFB40,   /* U+2F96A */
  0xFB84,   /* U+2F96B */
  0xFB40,   /* U+2F96C */
  0xFB80,   /* U+2F96D */
  0xFB40,   /* U+2F96E */
  0xFB40,   /* U+2F96F */
  0xFB40,   /* U+2F970 */
  0xFB80,   /* U+2F971 */
  0xFB84,   /* U+2F972 */
  0xFB84,   /* U+2F973 */
  0xFB80,   /* U+2F974 */
  0xFB84,   /* U+2F975 */
  0xFB40,   /* U+2F976 */
  0xFB84,   /* U+2F977 */
  0xFB40,   /* U+2F978 */
  0xFB40,   /* U+2F979 */
  0xFB41,   /* U+2F97A */
  0xFB84,   /* U+2F97B */
  0xFB84,   /* U+2F97C */
  0xFB41,   /* U+2F97D */
  0xFB84,   /* U+2F97E */
  0xFB41,   /* U+2F97F */
  0xFB84,   /* U+2F980 */
  0xFB80,   /* U+2F981 */
  0xFB41,   /* U+2F982 */
  0xFB41,   /* U+2F983 */
  0xFB80,   /* U+2F984 */
  0xFB41,   /* U+2F985 */
  0xFB40,   /* U+2F986 */
  0xFB84,   /* U+2F987 */
  0xFB84,   /* U+2F988 */
  0xFB84,   /* U+2F989 */
  0xFB84,   /* U+2F98A */
  0xFB41,   /* U+2F98B */
  0xFB41,   /* U+2F98C */
  0xFB41,   /* U+2F98D */
  0xFB80,   /* U+2F98E */
  0xFB41,   /* U+2F98F */
  0xFB41,   /* U+2F990 */
  0xFB41,   /* U+2F991 */
  0xFB40,   /* U+2F992 */
  0xFB41,   /* U+2F993 */
  0xFB41,   /* U+2F994 */
  0xFB41,   /* U+2F995 */
  0xFB41,   /* U+2F996 */
  0xFB84,   /* U+2F997 */
  0xFB41,   /* U+2F998 */
  0xFB41,   /* U+2F999 */
  0xFB41,   /* U+2F99A */
  0xFB41,   /* U+2F99B */
  0xFB41,   /* U+2F99C */
  0xFB41,   /* U+2F99D */
  0xFB41,   /* U+2F99E */
  0xFB41,   /* U+2F99F */
  0xFB41,   /* U+2F9A0 */
  0xFB41,   /* U+2F9A1 */
  0xFB41,   /* U+2F9A2 */
  0xFB41,   /* U+2F9A3 */
  0xFB84,   /* U+2F9A4 */
  0xFB84,   /* U+2F9A5 */
  0xFB84,   /* U+2F9A6 */
  0xFB80,   /* U+2F9A7 */
  0xFB41,   /* U+2F9A8 */
  0xFB41,   /* U+2F9A9 */
  0xFB41,   /* U+2F9AA */
  0xFB84,   /* U+2F9AB */
  0xFB41,   /* U+2F9AC */
  0xFB84,   /* U+2F9AD */
  0xFB80,   /* U+2F9AE */
  0xFB80,   /* U+2F9AF */
  0xFB84,   /* U+2F9B0 */
  0xFB84,   /* U+2F9B1 */
  0xFB80,   /* U+2F9B2 */
  0xFB41,   /* U+2F9B3 */
  0xFB41,   /* U+2F9B4 */
  0xFB41,   /* U+2F9B5 */
  0xFB41,   /* U+2F9B6 */
  0xFB41,   /* U+2F9B7 */
  0xFB41,   /* U+2F9B8 */
  0xFB41,   /* U+2F9B9 */
  0xFB41,   /* U+2F9BA */
  0xFB41,   /* U+2F9BB */
  0xFB41,   /* U+2F9BC */
  0xFB41,   /* U+2F9BD */
  0xFB41,   /* U+2F9BE */
  0xFB80,   /* U+2F9BF */
  0xFB41,   /* U+2F9C0 */
  0xFB41,   /* U+2F9C1 */
  0xFB80,   /* U+2F9C2 */
  0xFB41,   /* U+2F9C3 */
  0xFB41,   /* U+2F9C4 */
  0xFB84,   /* U+2F9C5 */
  0xFB41,   /* U+2F9C6 */
  0xFB41,   /* U+2F9C7 */
  0xFB80,   /* U+2F9C8 */
  0xFB41,   /* U+2F9C9 */
  0xFB80,   /* U+2F9CA */
  0xFB84,   /* U+2F9CB */
  0xFB84,   /* U+2F9CC */
  0xFB80,   /* U+2F9CD */
  0xFB80,   /* U+2F9CE */
  0xFB41,   /* U+2F9CF */
  0xFB41,   /* U+2F9D0 */
  0xFB41,   /* U+2F9D1 */
  0xFB41,   /* U+2F9D2 */
  0xFB84,   /* U+2F9D3 */
  0xFB41,   /* U+2F9D4 */
  0xFB41,   /* U+2F9D5 */
  0xFB41,   /* U+2F9D6 */
  0xFB41,   /* U+2F9D7 */
  0xFB84,   /* U+2F9D8 */
  0xFB84,   /* U+2F9D9 */
  0xFB41,   /* U+2F9DA */
  0xFB41,   /* U+2F9DB */
  0xFB41,   /* U+2F9DC */
  0xFB84,   /* U+2F9DD */
  0xFB41,   /* U+2F9DE */
  0xFB41,   /* U+2F9DF */
  0xFB85,   /* U+2F9E0 */
  0xFB85,   /* U+2F9E1 */
  0xFB41,   /* U+2F9E2 */
  0xFB41,   /* U+2F9E3 */
  0xFB41,   /* U+2F9E4 */
  0xFB85,   /* U+2F9E5 */
  0xFB41,   /* U+2F9E6 */
  0xFB41,   /* U+2F9E7 */
  0xFB41,   /* U+2F9E8 */
  0xFB41,   /* U+2F9E9 */
  0xFB41,   /* U+2F9EA */
  0xFB41,   /* U+2F9EB */
  0xFB41,   /* U+2F9EC */
  0xFB85,   /* U+2F9ED */
  0xFB41,   /* U+2F9EE */
  0xFB80,   /* U+2F9EF */
  0xFB41,   /* U+2F9F0 */
  0xFB85,   /* U+2F9F1 */
  0xFB80,   /* U+2F9F2 */
  0xFB41,   /* U+2F9F3 */
  0xFB40,   /* U+2F9F4 */
  0xFB41,   /* U+2F9F5 */
  0xFB85,   /* U+2F9F6 */
  0xFB85,   /* U+2F9F7 */
  0xFB80,   /* U+2F9F8 */
  0xFB80,   /* U+2F9F9 */
  0xFB41,   /* U+2F9FA */
  0xFB85,   /* U+2F9FB */
  0xFB80,   /* U+2F9FC */
  0xFB85,   /* U+2F9FD */
  0xFB41,   /* U+2F9FE */
  0xFB41,   /* U+2F9FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2F900 */
  0x0020,   /* U+2F901 */
  0x0020,   /* U+2F902 */
  0x0020,   /* U+2F903 */
  0x0020,   /* U+2F904 */
  0x0020,   /* U+2F905 */
  0x0020,   /* U+2F906 */
  0x0020,   /* U+2F907 */
  0x0020,   /* U+2F908 */
  0x0020,   /* U+2F909 */
  0x0020,   /* U+2F90A */
  0x0020,   /* U+2F90B */
  0x0020,   /* U+2F90C */
  0x0020,   /* U+2F90D */
  0x0020,   /* U+2F90E */
  0x0020,   /* U+2F90F */
  0x0020,   /* U+2F910 */
  0x0020,   /* U+2F911 */
  0x0020,   /* U+2F912 */
  0x0020,   /* U+2F913 */
  0x0020,   /* U+2F914 */
  0x0020,   /* U+2F915 */
  0x0020,   /* U+2F916 */
  0x0020,   /* U+2F917 */
  0x0020,   /* U+2F918 */
  0x0020,   /* U+2F919 */
  0x0020,   /* U+2F91A */
  0x0020,   /* U+2F91B */
  0x0020,   /* U+2F91C */
  0x0020,   /* U+2F91D */
  0x0020,   /* U+2F91E */
  0x0020,   /* U+2F91F */
  0x0020,   /* U+2F920 */
  0x0020,   /* U+2F921 */
  0x0020,   /* U+2F922 */
  0x0020,   /* U+2F923 */
  0x0020,   /* U+2F924 */
  0x0020,   /* U+2F925 */
  0x0020,   /* U+2F926 */
  0x0020,   /* U+2F927 */
  0x0020,   /* U+2F928 */
  0x0020,   /* U+2F929 */
  0x0020,   /* U+2F92A */
  0x0020,   /* U+2F92B */
  0x0020,   /* U+2F92C */
  0x0020,   /* U+2F92D */
  0x0020,   /* U+2F92E */
  0x0020,   /* U+2F92F */
  0x0020,   /* U+2F930 */
  0x0020,   /* U+2F931 */
  0x0020,   /* U+2F932 */
  0x0020,   /* U+2F933 */
  0x0020,   /* U+2F934 */
  0x0020,   /* U+2F935 */
  0x0020,   /* U+2F936 */
  0x0020,   /* U+2F937 */
  0x0020,   /* U+2F938 */
  0x0020,   /* U+2F939 */
  0x0020,   /* U+2F93A */
  0x0020,   /* U+2F93B */
  0x0020,   /* U+2F93C */
  0x0020,   /* U+2F93D */
  0x0020,   /* U+2F93E */
  0x0020,   /* U+2F93F */
  0x0020,   /* U+2F940 */
  0x0020,   /* U+2F941 */
  0x0020,   /* U+2F942 */
  0x0020,   /* U+2F943 */
  0x0020,   /* U+2F944 */
  0x0020,   /* U+2F945 */
  0x0020,   /* U+2F946 */
  0x0020,   /* U+2F947 */
  0x0020,   /* U+2F948 */
  0x0020,   /* U+2F949 */
  0x0020,   /* U+2F94A */
  0x0020,   /* U+2F94B */
  0x0020,   /* U+2F94C */
  0x0020,   /* U+2F94D */
  0x0020,   /* U+2F94E */
  0x0020,   /* U+2F94F */
  0x0020,   /* U+2F950 */
  0x0020,   /* U+2F951 */
  0x0020,   /* U+2F952 */
  0x0020,   /* U+2F953 */
  0x0020,   /* U+2F954 */
  0x0020,   /* U+2F955 */
  0x0020,   /* U+2F956 */
  0x0020,   /* U+2F957 */
  0x0020,   /* U+2F958 */
  0x0020,   /* U+2F959 */
  0x0020,   /* U+2F95A */
  0x0020,   /* U+2F95B */
  0x0020,   /* U+2F95C */
  0x0020,   /* U+2F95D */
  0x0020,   /* U+2F95E */
  0x0020,   /* U+2F95F */
  0x0020,   /* U+2F960 */
  0x0020,   /* U+2F961 */
  0x0020,   /* U+2F962 */
  0x0020,   /* U+2F963 */
  0x0020,   /* U+2F964 */
  0x0020,   /* U+2F965 */
  0x0020,   /* U+2F966 */
  0x0020,   /* U+2F967 */
  0x0020,   /* U+2F968 */
  0x0020,   /* U+2F969 */
  0x0020,   /* U+2F96A */
  0x0020,   /* U+2F96B */
  0x0020,   /* U+2F96C */
  0x0020,   /* U+2F96D */
  0x0020,   /* U+2F96E */
  0x0020,   /* U+2F96F */
  0x0020,   /* U+2F970 */
  0x0020,   /* U+2F971 */
  0x0020,   /* U+2F972 */
  0x0020,   /* U+2F973 */
  0x0020,   /* U+2F974 */
  0x0020,   /* U+2F975 */
  0x0020,   /* U+2F976 */
  0x0020,   /* U+2F977 */
  0x0020,   /* U+2F978 */
  0x0020,   /* U+2F979 */
  0x0020,   /* U+2F97A */
  0x0020,   /* U+2F97B */
  0x0020,   /* U+2F97C */
  0x0020,   /* U+2F97D */
  0x0020,   /* U+2F97E */
  0x0020,   /* U+2F97F */
  0x0020,   /* U+2F980 */
  0x0020,   /* U+2F981 */
  0x0020,   /* U+2F982 */
  0x0020,   /* U+2F983 */
  0x0020,   /* U+2F984 */
  0x0020,   /* U+2F985 */
  0x0020,   /* U+2F986 */
  0x0020,   /* U+2F987 */
  0x0020,   /* U+2F988 */
  0x0020,   /* U+2F989 */
  0x0020,   /* U+2F98A */
  0x0020,   /* U+2F98B */
  0x0020,   /* U+2F98C */
  0x0020,   /* U+2F98D */
  0x0020,   /* U+2F98E */
  0x0020,   /* U+2F98F */
  0x0020,   /* U+2F990 */
  0x0020,   /* U+2F991 */
  0x0020,   /* U+2F992 */
  0x0020,   /* U+2F993 */
  0x0020,   /* U+2F994 */
  0x0020,   /* U+2F995 */
  0x0020,   /* U+2F996 */
  0x0020,   /* U+2F997 */
  0x0020,   /* U+2F998 */
  0x0020,   /* U+2F999 */
  0x0020,   /* U+2F99A */
  0x0020,   /* U+2F99B */
  0x0020,   /* U+2F99C */
  0x0020,   /* U+2F99D */
  0x0020,   /* U+2F99E */
  0x0020,   /* U+2F99F */
  0x0020,   /* U+2F9A0 */
  0x0020,   /* U+2F9A1 */
  0x0020,   /* U+2F9A2 */
  0x0020,   /* U+2F9A3 */
  0x0020,   /* U+2F9A4 */
  0x0020,   /* U+2F9A5 */
  0x0020,   /* U+2F9A6 */
  0x0020,   /* U+2F9A7 */
  0x0020,   /* U+2F9A8 */
  0x0020,   /* U+2F9A9 */
  0x0020,   /* U+2F9AA */
  0x0020,   /* U+2F9AB */
  0x0020,   /* U+2F9AC */
  0x0020,   /* U+2F9AD */
  0x0020,   /* U+2F9AE */
  0x0020,   /* U+2F9AF */
  0x0020,   /* U+2F9B0 */
  0x0020,   /* U+2F9B1 */
  0x0020,   /* U+2F9B2 */
  0x0020,   /* U+2F9B3 */
  0x0020,   /* U+2F9B4 */
  0x0020,   /* U+2F9B5 */
  0x0020,   /* U+2F9B6 */
  0x0020,   /* U+2F9B7 */
  0x0020,   /* U+2F9B8 */
  0x0020,   /* U+2F9B9 */
  0x0020,   /* U+2F9BA */
  0x0020,   /* U+2F9BB */
  0x0020,   /* U+2F9BC */
  0x0020,   /* U+2F9BD */
  0x0020,   /* U+2F9BE */
  0x0020,   /* U+2F9BF */
  0x0020,   /* U+2F9C0 */
  0x0020,   /* U+2F9C1 */
  0x0020,   /* U+2F9C2 */
  0x0020,   /* U+2F9C3 */
  0x0020,   /* U+2F9C4 */
  0x0020,   /* U+2F9C5 */
  0x0020,   /* U+2F9C6 */
  0x0020,   /* U+2F9C7 */
  0x0020,   /* U+2F9C8 */
  0x0020,   /* U+2F9C9 */
  0x0020,   /* U+2F9CA */
  0x0020,   /* U+2F9CB */
  0x0020,   /* U+2F9CC */
  0x0020,   /* U+2F9CD */
  0x0020,   /* U+2F9CE */
  0x0020,   /* U+2F9CF */
  0x0020,   /* U+2F9D0 */
  0x0020,   /* U+2F9D1 */
  0x0020,   /* U+2F9D2 */
  0x0020,   /* U+2F9D3 */
  0x0020,   /* U+2F9D4 */
  0x0020,   /* U+2F9D5 */
  0x0020,   /* U+2F9D6 */
  0x0020,   /* U+2F9D7 */
  0x0020,   /* U+2F9D8 */
  0x0020,   /* U+2F9D9 */
  0x0020,   /* U+2F9DA */
  0x0020,   /* U+2F9DB */
  0x0020,   /* U+2F9DC */
  0x0020,   /* U+2F9DD */
  0x0020,   /* U+2F9DE */
  0x0020,   /* U+2F9DF */
  0x0020,   /* U+2F9E0 */
  0x0020,   /* U+2F9E1 */
  0x0020,   /* U+2F9E2 */
  0x0020,   /* U+2F9E3 */
  0x0020,   /* U+2F9E4 */
  0x0020,   /* U+2F9E5 */
  0x0020,   /* U+2F9E6 */
  0x0020,   /* U+2F9E7 */
  0x0020,   /* U+2F9E8 */
  0x0020,   /* U+2F9E9 */
  0x0020,   /* U+2F9EA */
  0x0020,   /* U+2F9EB */
  0x0020,   /* U+2F9EC */
  0x0020,   /* U+2F9ED */
  0x0020,   /* U+2F9EE */
  0x0020,   /* U+2F9EF */
  0x0020,   /* U+2F9F0 */
  0x0020,   /* U+2F9F1 */
  0x0020,   /* U+2F9F2 */
  0x0020,   /* U+2F9F3 */
  0x0020,   /* U+2F9F4 */
  0x0020,   /* U+2F9F5 */
  0x0020,   /* U+2F9F6 */
  0x0020,   /* U+2F9F7 */
  0x0020,   /* U+2F9F8 */
  0x0020,   /* U+2F9F9 */
  0x0020,   /* U+2F9FA */
  0x0020,   /* U+2F9FB */
  0x0020,   /* U+2F9FC */
  0x0020,   /* U+2F9FD */
  0x0020,   /* U+2F9FE */
  0x0020,   /* U+2F9FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2F900 */
  0x0002,   /* U+2F901 */
  0x0002,   /* U+2F902 */
  0x0002,   /* U+2F903 */
  0x0002,   /* U+2F904 */
  0x0002,   /* U+2F905 */
  0x0002,   /* U+2F906 */
  0x0002,   /* U+2F907 */
  0x0002,   /* U+2F908 */
  0x0002,   /* U+2F909 */
  0x0002,   /* U+2F90A */
  0x0002,   /* U+2F90B */
  0x0002,   /* U+2F90C */
  0x0002,   /* U+2F90D */
  0x0002,   /* U+2F90E */
  0x0002,   /* U+2F90F */
  0x0002,   /* U+2F910 */
  0x0002,   /* U+2F911 */
  0x0002,   /* U+2F912 */
  0x0002,   /* U+2F913 */
  0x0002,   /* U+2F914 */
  0x0002,   /* U+2F915 */
  0x0002,   /* U+2F916 */
  0x0002,   /* U+2F917 */
  0x0002,   /* U+2F918 */
  0x0002,   /* U+2F919 */
  0x0002,   /* U+2F91A */
  0x0002,   /* U+2F91B */
  0x0002,   /* U+2F91C */
  0x0002,   /* U+2F91D */
  0x0002,   /* U+2F91E */
  0x0002,   /* U+2F91F */
  0x0002,   /* U+2F920 */
  0x0002,   /* U+2F921 */
  0x0002,   /* U+2F922 */
  0x0002,   /* U+2F923 */
  0x0002,   /* U+2F924 */
  0x0002,   /* U+2F925 */
  0x0002,   /* U+2F926 */
  0x0002,   /* U+2F927 */
  0x0002,   /* U+2F928 */
  0x0002,   /* U+2F929 */
  0x0002,   /* U+2F92A */
  0x0002,   /* U+2F92B */
  0x0002,   /* U+2F92C */
  0x0002,   /* U+2F92D */
  0x0002,   /* U+2F92E */
  0x0002,   /* U+2F92F */
  0x0002,   /* U+2F930 */
  0x0002,   /* U+2F931 */
  0x0002,   /* U+2F932 */
  0x0002,   /* U+2F933 */
  0x0002,   /* U+2F934 */
  0x0002,   /* U+2F935 */
  0x0002,   /* U+2F936 */
  0x0002,   /* U+2F937 */
  0x0002,   /* U+2F938 */
  0x0002,   /* U+2F939 */
  0x0002,   /* U+2F93A */
  0x0002,   /* U+2F93B */
  0x0002,   /* U+2F93C */
  0x0002,   /* U+2F93D */
  0x0002,   /* U+2F93E */
  0x0002,   /* U+2F93F */
  0x0002,   /* U+2F940 */
  0x0002,   /* U+2F941 */
  0x0002,   /* U+2F942 */
  0x0002,   /* U+2F943 */
  0x0002,   /* U+2F944 */
  0x0002,   /* U+2F945 */
  0x0002,   /* U+2F946 */
  0x0002,   /* U+2F947 */
  0x0002,   /* U+2F948 */
  0x0002,   /* U+2F949 */
  0x0002,   /* U+2F94A */
  0x0002,   /* U+2F94B */
  0x0002,   /* U+2F94C */
  0x0002,   /* U+2F94D */
  0x0002,   /* U+2F94E */
  0x0002,   /* U+2F94F */
  0x0002,   /* U+2F950 */
  0x0002,   /* U+2F951 */
  0x0002,   /* U+2F952 */
  0x0002,   /* U+2F953 */
  0x0002,   /* U+2F954 */
  0x0002,   /* U+2F955 */
  0x0002,   /* U+2F956 */
  0x0002,   /* U+2F957 */
  0x0002,   /* U+2F958 */
  0x0002,   /* U+2F959 */
  0x0002,   /* U+2F95A */
  0x0002,   /* U+2F95B */
  0x0002,   /* U+2F95C */
  0x0002,   /* U+2F95D */
  0x0002,   /* U+2F95E */
  0x0002,   /* U+2F95F */
  0x0002,   /* U+2F960 */
  0x0002,   /* U+2F961 */
  0x0002,   /* U+2F962 */
  0x0002,   /* U+2F963 */
  0x0002,   /* U+2F964 */
  0x0002,   /* U+2F965 */
  0x0002,   /* U+2F966 */
  0x0002,   /* U+2F967 */
  0x0002,   /* U+2F968 */
  0x0002,   /* U+2F969 */
  0x0002,   /* U+2F96A */
  0x0002,   /* U+2F96B */
  0x0002,   /* U+2F96C */
  0x0002,   /* U+2F96D */
  0x0002,   /* U+2F96E */
  0x0002,   /* U+2F96F */
  0x0002,   /* U+2F970 */
  0x0002,   /* U+2F971 */
  0x0002,   /* U+2F972 */
  0x0002,   /* U+2F973 */
  0x0002,   /* U+2F974 */
  0x0002,   /* U+2F975 */
  0x0002,   /* U+2F976 */
  0x0002,   /* U+2F977 */
  0x0002,   /* U+2F978 */
  0x0002,   /* U+2F979 */
  0x0002,   /* U+2F97A */
  0x0002,   /* U+2F97B */
  0x0002,   /* U+2F97C */
  0x0002,   /* U+2F97D */
  0x0002,   /* U+2F97E */
  0x0002,   /* U+2F97F */
  0x0002,   /* U+2F980 */
  0x0002,   /* U+2F981 */
  0x0002,   /* U+2F982 */
  0x0002,   /* U+2F983 */
  0x0002,   /* U+2F984 */
  0x0002,   /* U+2F985 */
  0x0002,   /* U+2F986 */
  0x0002,   /* U+2F987 */
  0x0002,   /* U+2F988 */
  0x0002,   /* U+2F989 */
  0x0002,   /* U+2F98A */
  0x0002,   /* U+2F98B */
  0x0002,   /* U+2F98C */
  0x0002,   /* U+2F98D */
  0x0002,   /* U+2F98E */
  0x0002,   /* U+2F98F */
  0x0002,   /* U+2F990 */
  0x0002,   /* U+2F991 */
  0x0002,   /* U+2F992 */
  0x0002,   /* U+2F993 */
  0x0002,   /* U+2F994 */
  0x0002,   /* U+2F995 */
  0x0002,   /* U+2F996 */
  0x0002,   /* U+2F997 */
  0x0002,   /* U+2F998 */
  0x0002,   /* U+2F999 */
  0x0002,   /* U+2F99A */
  0x0002,   /* U+2F99B */
  0x0002,   /* U+2F99C */
  0x0002,   /* U+2F99D */
  0x0002,   /* U+2F99E */
  0x0002,   /* U+2F99F */
  0x0002,   /* U+2F9A0 */
  0x0002,   /* U+2F9A1 */
  0x0002,   /* U+2F9A2 */
  0x0002,   /* U+2F9A3 */
  0x0002,   /* U+2F9A4 */
  0x0002,   /* U+2F9A5 */
  0x0002,   /* U+2F9A6 */
  0x0002,   /* U+2F9A7 */
  0x0002,   /* U+2F9A8 */
  0x0002,   /* U+2F9A9 */
  0x0002,   /* U+2F9AA */
  0x0002,   /* U+2F9AB */
  0x0002,   /* U+2F9AC */
  0x0002,   /* U+2F9AD */
  0x0002,   /* U+2F9AE */
  0x0002,   /* U+2F9AF */
  0x0002,   /* U+2F9B0 */
  0x0002,   /* U+2F9B1 */
  0x0002,   /* U+2F9B2 */
  0x0002,   /* U+2F9B3 */
  0x0002,   /* U+2F9B4 */
  0x0002,   /* U+2F9B5 */
  0x0002,   /* U+2F9B6 */
  0x0002,   /* U+2F9B7 */
  0x0002,   /* U+2F9B8 */
  0x0002,   /* U+2F9B9 */
  0x0002,   /* U+2F9BA */
  0x0002,   /* U+2F9BB */
  0x0002,   /* U+2F9BC */
  0x0002,   /* U+2F9BD */
  0x0002,   /* U+2F9BE */
  0x0002,   /* U+2F9BF */
  0x0002,   /* U+2F9C0 */
  0x0002,   /* U+2F9C1 */
  0x0002,   /* U+2F9C2 */
  0x0002,   /* U+2F9C3 */
  0x0002,   /* U+2F9C4 */
  0x0002,   /* U+2F9C5 */
  0x0002,   /* U+2F9C6 */
  0x0002,   /* U+2F9C7 */
  0x0002,   /* U+2F9C8 */
  0x0002,   /* U+2F9C9 */
  0x0002,   /* U+2F9CA */
  0x0002,   /* U+2F9CB */
  0x0002,   /* U+2F9CC */
  0x0002,   /* U+2F9CD */
  0x0002,   /* U+2F9CE */
  0x0002,   /* U+2F9CF */
  0x0002,   /* U+2F9D0 */
  0x0002,   /* U+2F9D1 */
  0x0002,   /* U+2F9D2 */
  0x0002,   /* U+2F9D3 */
  0x0002,   /* U+2F9D4 */
  0x0002,   /* U+2F9D5 */
  0x0002,   /* U+2F9D6 */
  0x0002,   /* U+2F9D7 */
  0x0002,   /* U+2F9D8 */
  0x0002,   /* U+2F9D9 */
  0x0002,   /* U+2F9DA */
  0x0002,   /* U+2F9DB */
  0x0002,   /* U+2F9DC */
  0x0002,   /* U+2F9DD */
  0x0002,   /* U+2F9DE */
  0x0002,   /* U+2F9DF */
  0x0002,   /* U+2F9E0 */
  0x0002,   /* U+2F9E1 */
  0x0002,   /* U+2F9E2 */
  0x0002,   /* U+2F9E3 */
  0x0002,   /* U+2F9E4 */
  0x0002,   /* U+2F9E5 */
  0x0002,   /* U+2F9E6 */
  0x0002,   /* U+2F9E7 */
  0x0002,   /* U+2F9E8 */
  0x0002,   /* U+2F9E9 */
  0x0002,   /* U+2F9EA */
  0x0002,   /* U+2F9EB */
  0x0002,   /* U+2F9EC */
  0x0002,   /* U+2F9ED */
  0x0002,   /* U+2F9EE */
  0x0002,   /* U+2F9EF */
  0x0002,   /* U+2F9F0 */
  0x0002,   /* U+2F9F1 */
  0x0002,   /* U+2F9F2 */
  0x0002,   /* U+2F9F3 */
  0x0002,   /* U+2F9F4 */
  0x0002,   /* U+2F9F5 */
  0x0002,   /* U+2F9F6 */
  0x0002,   /* U+2F9F7 */
  0x0002,   /* U+2F9F8 */
  0x0002,   /* U+2F9F9 */
  0x0002,   /* U+2F9FA */
  0x0002,   /* U+2F9FB */
  0x0002,   /* U+2F9FC */
  0x0002,   /* U+2F9FD */
  0x0002,   /* U+2F9FE */
  0x0002,   /* U+2F9FF */

  /* Primary weight 2 for each character. */
  0xED3E,   /* U+2F900 */
  0xED77,   /* U+2F901 */
  0xED41,   /* U+2F902 */
  0xED69,   /* U+2F903 */
  0xED78,   /* U+2F904 */
  0xED85,   /* U+2F905 */
  0xBD1E,   /* U+2F906 */
  0xED34,   /* U+2F907 */
  0xEE2F,   /* U+2F908 */
  0xEE6E,   /* U+2F909 */
  0xBD33,   /* U+2F90A */
  0xEECB,   /* U+2F90B */
  0xEEC7,   /* U+2F90C */
  0xBED1,   /* U+2F90D */
  0xEDF9,   /* U+2F90E */
  0xEF6E,   /* U+2F90F */
  0xBF5E,   /* U+2F910 */
  0xBF8E,   /* U+2F911 */
  0xEFC6,   /* U+2F912 */
  0xF039,   /* U+2F913 */
  0xF01E,   /* U+2F914 */
  0xF01B,   /* U+2F915 */
  0xBD96,   /* U+2F916 */
  0xF04A,   /* U+2F917 */
  0xF07D,   /* U+2F918 */
  0xF077,   /* U+2F919 */
  0xF0AD,   /* U+2F91A */
  0x8525,   /* U+2F91B */
  0xF145,   /* U+2F91C */
  0xC263,   /* U+2F91D */
  0xF19C,   /* U+2F91E */
  0xC3AB,   /* U+2F91F */
  0xF228,   /* U+2F920 */
  0xF235,   /* U+2F921 */
  0xF250,   /* U+2F922 */
  0xC608,   /* U+2F923 */
  0xF280,   /* U+2F924 */
  0xF295,   /* U+2F925 */
  0xC735,   /* U+2F926 */
  0xC814,   /* U+2F927 */
  0xF37A,   /* U+2F928 */
  0xF38B,   /* U+2F929 */
  0xBEAC,   /* U+2F92A */
  0xF3A5,   /* U+2F92B */
  0xBEB8,   /* U+2F92C */
  0xBEB8,   /* U+2F92D */
  0xF447,   /* U+2F92E */
  0xF45C,   /* U+2F92F */
  0xF471,   /* U+2F930 */
  0xF485,   /* U+2F931 */
  0xF4CA,   /* U+2F932 */
  0xBF1B,   /* U+2F933 */
  0xF524,   /* U+2F934 */
  0xCC36,   /* U+2F935 */
  0xF53E,   /* U+2F936 */
  0xCC92,   /* U+2F937 */
  0xF570,   /* U+2F938 */
  0xA19F,   /* U+2F939 */
  0xF610,   /* U+2F93A */
  0xCFA1,   /* U+2F93B */
  0xCFB8,   /* U+2F93C */
  0xD044,   /* U+2F93D */
  0xBFFC,   /* U+2F93E */
  0xC008,   /* U+2F93F */
  0xF6F4,   /* U+2F940 */
  0xD0F3,   /* U+2F941 */
  0xD0F2,   /* U+2F942 */
  0xD119,   /* U+2F943 */
  0xD133,   /* U+2F944 */
  0xF71E,   /* U+2F945 */
  0xF71F,   /* U+2F946 */
  0xF71F,   /* U+2F947 */
  0xF74A,   /* U+2F948 */
  0xC039,   /* U+2F949 */
  0xF78B,   /* U+2F94A */
  0xC046,   /* U+2F94B */
  0xC096,   /* U+2F94C */
  0xD41D,   /* U+2F94D */
  0xF84E,   /* U+2F94E */
  0xF88C,   /* U+2F94F */
  0xF8CC,   /* U+2F950 */
  0xC0E3,   /* U+2F951 */
  0xD626,   /* U+2F952 */
  0xF956,   /* U+2F953 */
  0xD69A,   /* U+2F954 */
  0xD6C5,   /* U+2F955 */
  0xF98F,   /* U+2F956 */
  0xF9EB,   /* U+2F957 */
  0xC12F,   /* U+2F958 */
  0xFA40,   /* U+2F959 */
  0xFA4A,   /* U+2F95A */
  0xFA4F,   /* U+2F95B */
  0xD97C,   /* U+2F95C */
  0xDAA7,   /* U+2F95D */
  0xDAA7,   /* U+2F95E */
  0xFAEE,   /* U+2F95F */
  0xC202,   /* U+2F960 */
  0xDBAB,   /* U+2F961 */
  0xFBC6,   /* U+2F962 */
  0xFBC9,   /* U+2F963 */
  0xC227,   /* U+2F964 */
  0xDC80,   /* U+2F965 */
  0xFCD2,   /* U+2F966 */
  0xC2A0,   /* U+2F967 */
  0xFCE8,   /* U+2F968 */
  0xFCE3,   /* U+2F969 */
  0xFD00,   /* U+2F96A */
  0xDF86,   /* U+2F96B */
  0xFD63,   /* U+2F96C */
  0xC301,   /* U+2F96D */
  0xFDC7,   /* U+2F96E */
  0xFE02,   /* U+2F96F */
  0xFE45,   /* U+2F970 */
  0xC334,   /* U+2F971 */
  0xE228,   /* U+2F972 */
  0xE247,   /* U+2F973 */
  0xC359,   /* U+2F974 */
  0xE2D9,   /* U+2F975 */
  0xFF7A,   /* U+2F976 */
  0xE33E,   /* U+2F977 */
  0xFF95,   /* U+2F978 */
  0xFFFA,   /* U+2F979 */
  0x8005,   /* U+2F97A */
  0xE4DA,   /* U+2F97B */
  0xE523,   /* U+2F97C */
  0x8060,   /* U+2F97D */
  0xE5A8,   /* U+2F97E */
  0x8070,   /* U+2F97F */
  0xB35F,   /* U+2F980 */
  0xC3D5,   /* U+2F981 */
  0x80B2,   /* U+2F982 */
  0x8103,   /* U+2F983 */
  0xC40B,   /* U+2F984 */
  0x813E,   /* U+2F985 */
  0xDAB5,   /* U+2F986 */
  0xE7A7,   /* U+2F987 */
  0xE7B5,   /* U+2F988 */
  0xB393,   /* U+2F989 */
  0xB39C,   /* U+2F98A */
  0x8201,   /* U+2F98B */
  0x8204,   /* U+2F98C */
  0x8F9E,   /* U+2F98D */
  0xC46B,   /* U+2F98E */
  0x8291,   /* U+2F98F */
  0x828B,   /* U+2F990 */
  0x829D,   /* U+2F991 */
  0xD2B3,   /* U+2F992 */
  0x82B1,   /* U+2F993 */
  0x82B3,   /* U+2F994 */
  0x82BD,   /* U+2F995 */
  0x82E6,   /* U+2F996 */
  0xEB3C,   /* U+2F997 */
  0x82E5,   /* U+2F998 */
  0x831D,   /* U+2F999 */
  0x8363,   /* U+2F99A */
  0x83AD,   /* U+2F99B */
  0x8323,   /* U+2F99C */
  0x83BD,   /* U+2F99D */
  0x83E7,   /* U+2F99E */
  0x8457,   /* U+2F99F */
  0x8353,   /* U+2F9A0 */
  0x83CA,   /* U+2F9A1 */
  0x83CC,   /* U+2F9A2 */
  0x83DC,   /* U+2F9A3 */
  0xEC36,   /* U+2F9A4 */
  0xED6B,   /* U+2F9A5 */
  0xECD5,   /* U+2F9A6 */
  0xC52B,   /* U+2F9A7 */
  0x84F1,   /* U+2F9A8 */
  0x84F3,   /* U+2F9A9 */
  0x8516,   /* U+2F9AA */
  0xF3CA,   /* U+2F9AB */
  0x8564,   /* U+2F9AC */
  0xEF2C,   /* U+2F9AD */
  0xC55D,   /* U+2F9AE */
  0xC561,   /* U+2F9AF */
  0xEFB1,   /* U+2F9B0 */
  0xF0D2,   /* U+2F9B1 */
  0xC56B,   /* U+2F9B2 */
  0x8650,   /* U+2F9B3 */
  0x865C,   /* U+2F9B4 */
  0x8667,   /* U+2F9B5 */
  0x8669,   /* U+2F9B6 */
  0x86A9,   /* U+2F9B7 */
  0x8688,   /* U+2F9B8 */
  0x870E,   /* U+2F9B9 */
  0x86E2,   /* U+2F9BA */
  0x8779,   /* U+2F9BB */
  0x8728,   /* U+2F9BC */
  0x876B,   /* U+2F9BD */
  0x8786,   /* U+2F9BE */
  0xC5D7,   /* U+2F9BF */
  0x87E1,   /* U+2F9C0 */
  0x8801,   /* U+2F9C1 */
  0xC5F9,   /* U+2F9C2 */
  0x8860,   /* U+2F9C3 */
  0x8863,   /* U+2F9C4 */
  0xF667,   /* U+2F9C5 */
  0x88D7,   /* U+2F9C6 */
  0x88DE,   /* U+2F9C7 */
  0xC635,   /* U+2F9C8 */
  0x88FA,   /* U+2F9C9 */
  0xB4BB,   /* U+2F9CA */
  0xF8AE,   /* U+2F9CB */
  0xF966,   /* U+2F9CC */
  0xC6BE,   /* U+2F9CD */
  0xC6C7,   /* U+2F9CE */
  0x8AA0,   /* U+2F9CF */
  0x8AED,   /* U+2F9D0 */
  0x8B8A,   /* U+2F9D1 */
  0x8C55,   /* U+2F9D2 */
  0xFCA8,   /* U+2F9D3 */
  0x8CAB,   /* U+2F9D4 */
  0x8CC1,   /* U+2F9D5 */
  0x8D1B,   /* U+2F9D6 */
  0x8D77,   /* U+2F9D7 */
  0xFF2F,   /* U+2F9D8 */
  0x8804,   /* U+2F9D9 */
  0x8DCB,   /* U+2F9DA */
  0x8DBC,   /* U+2F9DB */
  0x8DF0,   /* U+2F9DC */
  0x88DE,   /* U+2F9DD */
  0x8ED4,   /* U+2F9DE */
  0x8F38,   /* U+2F9DF */
  0x85D2,   /* U+2F9E0 */
  0x85ED,   /* U+2F9E1 */
  0x9094,   /* U+2F9E2 */
  0x90F1,   /* U+2F9E3 */
  0x9111,   /* U+2F9E4 */
  0x872E,   /* U+2F9E5 */
  0x911B,   /* U+2F9E6 */
  0x9238,   /* U+2F9E7 */
  0x92D7,   /* U+2F9E8 */
  0x92D8,   /* U+2F9E9 */
  0x927C,   /* U+2F9EA */
  0x93F9,   /* U+2F9EB */
  0x9415,   /* U+2F9EC */
  0x8BFA,   /* U+2F9ED */
  0x958B,   /* U+2F9EE */
  0xC995,   /* U+2F9EF */
  0x95B7,   /* U+2F9F0 */
  0x8D77,   /* U+2F9F1 */
  0xC9E6,   /* U+2F9F2 */
  0x96C3,   /* U+2F9F3 */
  0xDDB2,   /* U+2F9F4 */
  0x9723,   /* U+2F9F5 */
  0x9145,   /* U+2F9F6 */
  0x921A,   /* U+2F9F7 */
  0xCA6E,   /* U+2F9F8 */
  0xCA76,   /* U+2F9F9 */
  0x97E0,   /* U+2F9FA */
  0x940A,   /* U+2F9FB */
  0xCAB2,   /* U+2F9FC */
  0x9496,   /* U+2F9FD */
  0x980B,   /* U+2F9FE */
  0x980B,   /* U+2F9FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2F900 */
  0x0000,   /* U+2F901 */
  0x0000,   /* U+2F902 */
  0x0000,   /* U+2F903 */
  0x0000,   /* U+2F904 */
  0x0000,   /* U+2F905 */
  0x0000,   /* U+2F906 */
  0x0000,   /* U+2F907 */
  0x0000,   /* U+2F908 */
  0x0000,   /* U+2F909 */
  0x0000,   /* U+2F90A */
  0x0000,   /* U+2F90B */
  0x0000,   /* U+2F90C */
  0x0000,   /* U+2F90D */
  0x0000,   /* U+2F90E */
  0x0000,   /* U+2F90F */
  0x0000,   /* U+2F910 */
  0x0000,   /* U+2F911 */
  0x0000,   /* U+2F912 */
  0x0000,   /* U+2F913 */
  0x0000,   /* U+2F914 */
  0x0000,   /* U+2F915 */
  0x0000,   /* U+2F916 */
  0x0000,   /* U+2F917 */
  0x0000,   /* U+2F918 */
  0x0000,   /* U+2F919 */
  0x0000,   /* U+2F91A */
  0x0000,   /* U+2F91B */
  0x0000,   /* U+2F91C */
  0x0000,   /* U+2F91D */
  0x0000,   /* U+2F91E */
  0x0000,   /* U+2F91F */
  0x0000,   /* U+2F920 */
  0x0000,   /* U+2F921 */
  0x0000,   /* U+2F922 */
  0x0000,   /* U+2F923 */
  0x0000,   /* U+2F924 */
  0x0000,   /* U+2F925 */
  0x0000,   /* U+2F926 */
  0x0000,   /* U+2F927 */
  0x0000,   /* U+2F928 */
  0x0000,   /* U+2F929 */
  0x0000,   /* U+2F92A */
  0x0000,   /* U+2F92B */
  0x0000,   /* U+2F92C */
  0x0000,   /* U+2F92D */
  0x0000,   /* U+2F92E */
  0x0000,   /* U+2F92F */
  0x0000,   /* U+2F930 */
  0x0000,   /* U+2F931 */
  0x0000,   /* U+2F932 */
  0x0000,   /* U+2F933 */
  0x0000,   /* U+2F934 */
  0x0000,   /* U+2F935 */
  0x0000,   /* U+2F936 */
  0x0000,   /* U+2F937 */
  0x0000,   /* U+2F938 */
  0x0000,   /* U+2F939 */
  0x0000,   /* U+2F93A */
  0x0000,   /* U+2F93B */
  0x0000,   /* U+2F93C */
  0x0000,   /* U+2F93D */
  0x0000,   /* U+2F93E */
  0x0000,   /* U+2F93F */
  0x0000,   /* U+2F940 */
  0x0000,   /* U+2F941 */
  0x0000,   /* U+2F942 */
  0x0000,   /* U+2F943 */
  0x0000,   /* U+2F944 */
  0x0000,   /* U+2F945 */
  0x0000,   /* U+2F946 */
  0x0000,   /* U+2F947 */
  0x0000,   /* U+2F948 */
  0x0000,   /* U+2F949 */
  0x0000,   /* U+2F94A */
  0x0000,   /* U+2F94B */
  0x0000,   /* U+2F94C */
  0x0000,   /* U+2F94D */
  0x0000,   /* U+2F94E */
  0x0000,   /* U+2F94F */
  0x0000,   /* U+2F950 */
  0x0000,   /* U+2F951 */
  0x0000,   /* U+2F952 */
  0x0000,   /* U+2F953 */
  0x0000,   /* U+2F954 */
  0x0000,   /* U+2F955 */
  0x0000,   /* U+2F956 */
  0x0000,   /* U+2F957 */
  0x0000,   /* U+2F958 */
  0x0000,   /* U+2F959 */
  0x0000,   /* U+2F95A */
  0x0000,   /* U+2F95B */
  0x0000,   /* U+2F95C */
  0x0000,   /* U+2F95D */
  0x0000,   /* U+2F95E */
  0x0000,   /* U+2F95F */
  0x0000,   /* U+2F960 */
  0x0000,   /* U+2F961 */
  0x0000,   /* U+2F962 */
  0x0000,   /* U+2F963 */
  0x0000,   /* U+2F964 */
  0x0000,   /* U+2F965 */
  0x0000,   /* U+2F966 */
  0x0000,   /* U+2F967 */
  0x0000,   /* U+2F968 */
  0x0000,   /* U+2F969 */
  0x0000,   /* U+2F96A */
  0x0000,   /* U+2F96B */
  0x0000,   /* U+2F96C */
  0x0000,   /* U+2F96D */
  0x0000,   /* U+2F96E */
  0x0000,   /* U+2F96F */
  0x0000,   /* U+2F970 */
  0x0000,   /* U+2F971 */
  0x0000,   /* U+2F972 */
  0x0000,   /* U+2F973 */
  0x0000,   /* U+2F974 */
  0x0000,   /* U+2F975 */
  0x0000,   /* U+2F976 */
  0x0000,   /* U+2F977 */
  0x0000,   /* U+2F978 */
  0x0000,   /* U+2F979 */
  0x0000,   /* U+2F97A */
  0x0000,   /* U+2F97B */
  0x0000,   /* U+2F97C */
  0x0000,   /* U+2F97D */
  0x0000,   /* U+2F97E */
  0x0000,   /* U+2F97F */
  0x0000,   /* U+2F980 */
  0x0000,   /* U+2F981 */
  0x0000,   /* U+2F982 */
  0x0000,   /* U+2F983 */
  0x0000,   /* U+2F984 */
  0x0000,   /* U+2F985 */
  0x0000,   /* U+2F986 */
  0x0000,   /* U+2F987 */
  0x0000,   /* U+2F988 */
  0x0000,   /* U+2F989 */
  0x0000,   /* U+2F98A */
  0x0000,   /* U+2F98B */
  0x0000,   /* U+2F98C */
  0x0000,   /* U+2F98D */
  0x0000,   /* U+2F98E */
  0x0000,   /* U+2F98F */
  0x0000,   /* U+2F990 */
  0x0000,   /* U+2F991 */
  0x0000,   /* U+2F992 */
  0x0000,   /* U+2F993 */
  0x0000,   /* U+2F994 */
  0x0000,   /* U+2F995 */
  0x0000,   /* U+2F996 */
  0x0000,   /* U+2F997 */
  0x0000,   /* U+2F998 */
  0x0000,   /* U+2F999 */
  0x0000,   /* U+2F99A */
  0x0000,   /* U+2F99B */
  0x0000,   /* U+2F99C */
  0x0000,   /* U+2F99D */
  0x0000,   /* U+2F99E */
  0x0000,   /* U+2F99F */
  0x0000,   /* U+2F9A0 */
  0x0000,   /* U+2F9A1 */
  0x0000,   /* U+2F9A2 */
  0x0000,   /* U+2F9A3 */
  0x0000,   /* U+2F9A4 */
  0x0000,   /* U+2F9A5 */
  0x0000,   /* U+2F9A6 */
  0x0000,   /* U+2F9A7 */
  0x0000,   /* U+2F9A8 */
  0x0000,   /* U+2F9A9 */
  0x0000,   /* U+2F9AA */
  0x0000,   /* U+2F9AB */
  0x0000,   /* U+2F9AC */
  0x0000,   /* U+2F9AD */
  0x0000,   /* U+2F9AE */
  0x0000,   /* U+2F9AF */
  0x0000,   /* U+2F9B0 */
  0x0000,   /* U+2F9B1 */
  0x0000,   /* U+2F9B2 */
  0x0000,   /* U+2F9B3 */
  0x0000,   /* U+2F9B4 */
  0x0000,   /* U+2F9B5 */
  0x0000,   /* U+2F9B6 */
  0x0000,   /* U+2F9B7 */
  0x0000,   /* U+2F9B8 */
  0x0000,   /* U+2F9B9 */
  0x0000,   /* U+2F9BA */
  0x0000,   /* U+2F9BB */
  0x0000,   /* U+2F9BC */
  0x0000,   /* U+2F9BD */
  0x0000,   /* U+2F9BE */
  0x0000,   /* U+2F9BF */
  0x0000,   /* U+2F9C0 */
  0x0000,   /* U+2F9C1 */
  0x0000,   /* U+2F9C2 */
  0x0000,   /* U+2F9C3 */
  0x0000,   /* U+2F9C4 */
  0x0000,   /* U+2F9C5 */
  0x0000,   /* U+2F9C6 */
  0x0000,   /* U+2F9C7 */
  0x0000,   /* U+2F9C8 */
  0x0000,   /* U+2F9C9 */
  0x0000,   /* U+2F9CA */
  0x0000,   /* U+2F9CB */
  0x0000,   /* U+2F9CC */
  0x0000,   /* U+2F9CD */
  0x0000,   /* U+2F9CE */
  0x0000,   /* U+2F9CF */
  0x0000,   /* U+2F9D0 */
  0x0000,   /* U+2F9D1 */
  0x0000,   /* U+2F9D2 */
  0x0000,   /* U+2F9D3 */
  0x0000,   /* U+2F9D4 */
  0x0000,   /* U+2F9D5 */
  0x0000,   /* U+2F9D6 */
  0x0000,   /* U+2F9D7 */
  0x0000,   /* U+2F9D8 */
  0x0000,   /* U+2F9D9 */
  0x0000,   /* U+2F9DA */
  0x0000,   /* U+2F9DB */
  0x0000,   /* U+2F9DC */
  0x0000,   /* U+2F9DD */
  0x0000,   /* U+2F9DE */
  0x0000,   /* U+2F9DF */
  0x0000,   /* U+2F9E0 */
  0x0000,   /* U+2F9E1 */
  0x0000,   /* U+2F9E2 */
  0x0000,   /* U+2F9E3 */
  0x0000,   /* U+2F9E4 */
  0x0000,   /* U+2F9E5 */
  0x0000,   /* U+2F9E6 */
  0x0000,   /* U+2F9E7 */
  0x0000,   /* U+2F9E8 */
  0x0000,   /* U+2F9E9 */
  0x0000,   /* U+2F9EA */
  0x0000,   /* U+2F9EB */
  0x0000,   /* U+2F9EC */
  0x0000,   /* U+2F9ED */
  0x0000,   /* U+2F9EE */
  0x0000,   /* U+2F9EF */
  0x0000,   /* U+2F9F0 */
  0x0000,   /* U+2F9F1 */
  0x0000,   /* U+2F9F2 */
  0x0000,   /* U+2F9F3 */
  0x0000,   /* U+2F9F4 */
  0x0000,   /* U+2F9F5 */
  0x0000,   /* U+2F9F6 */
  0x0000,   /* U+2F9F7 */
  0x0000,   /* U+2F9F8 */
  0x0000,   /* U+2F9F9 */
  0x0000,   /* U+2F9FA */
  0x0000,   /* U+2F9FB */
  0x0000,   /* U+2F9FC */
  0x0000,   /* U+2F9FD */
  0x0000,   /* U+2F9FE */
  0x0000,   /* U+2F9FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2F900 */
  0x0000,   /* U+2F901 */
  0x0000,   /* U+2F902 */
  0x0000,   /* U+2F903 */
  0x0000,   /* U+2F904 */
  0x0000,   /* U+2F905 */
  0x0000,   /* U+2F906 */
  0x0000,   /* U+2F907 */
  0x0000,   /* U+2F908 */
  0x0000,   /* U+2F909 */
  0x0000,   /* U+2F90A */
  0x0000,   /* U+2F90B */
  0x0000,   /* U+2F90C */
  0x0000,   /* U+2F90D */
  0x0000,   /* U+2F90E */
  0x0000,   /* U+2F90F */
  0x0000,   /* U+2F910 */
  0x0000,   /* U+2F911 */
  0x0000,   /* U+2F912 */
  0x0000,   /* U+2F913 */
  0x0000,   /* U+2F914 */
  0x0000,   /* U+2F915 */
  0x0000,   /* U+2F916 */
  0x0000,   /* U+2F917 */
  0x0000,   /* U+2F918 */
  0x0000,   /* U+2F919 */
  0x0000,   /* U+2F91A */
  0x0000,   /* U+2F91B */
  0x0000,   /* U+2F91C */
  0x0000,   /* U+2F91D */
  0x0000,   /* U+2F91E */
  0x0000,   /* U+2F91F */
  0x0000,   /* U+2F920 */
  0x0000,   /* U+2F921 */
  0x0000,   /* U+2F922 */
  0x0000,   /* U+2F923 */
  0x0000,   /* U+2F924 */
  0x0000,   /* U+2F925 */
  0x0000,   /* U+2F926 */
  0x0000,   /* U+2F927 */
  0x0000,   /* U+2F928 */
  0x0000,   /* U+2F929 */
  0x0000,   /* U+2F92A */
  0x0000,   /* U+2F92B */
  0x0000,   /* U+2F92C */
  0x0000,   /* U+2F92D */
  0x0000,   /* U+2F92E */
  0x0000,   /* U+2F92F */
  0x0000,   /* U+2F930 */
  0x0000,   /* U+2F931 */
  0x0000,   /* U+2F932 */
  0x0000,   /* U+2F933 */
  0x0000,   /* U+2F934 */
  0x0000,   /* U+2F935 */
  0x0000,   /* U+2F936 */
  0x0000,   /* U+2F937 */
  0x0000,   /* U+2F938 */
  0x0000,   /* U+2F939 */
  0x0000,   /* U+2F93A */
  0x0000,   /* U+2F93B */
  0x0000,   /* U+2F93C */
  0x0000,   /* U+2F93D */
  0x0000,   /* U+2F93E */
  0x0000,   /* U+2F93F */
  0x0000,   /* U+2F940 */
  0x0000,   /* U+2F941 */
  0x0000,   /* U+2F942 */
  0x0000,   /* U+2F943 */
  0x0000,   /* U+2F944 */
  0x0000,   /* U+2F945 */
  0x0000,   /* U+2F946 */
  0x0000,   /* U+2F947 */
  0x0000,   /* U+2F948 */
  0x0000,   /* U+2F949 */
  0x0000,   /* U+2F94A */
  0x0000,   /* U+2F94B */
  0x0000,   /* U+2F94C */
  0x0000,   /* U+2F94D */
  0x0000,   /* U+2F94E */
  0x0000,   /* U+2F94F */
  0x0000,   /* U+2F950 */
  0x0000,   /* U+2F951 */
  0x0000,   /* U+2F952 */
  0x0000,   /* U+2F953 */
  0x0000,   /* U+2F954 */
  0x0000,   /* U+2F955 */
  0x0000,   /* U+2F956 */
  0x0000,   /* U+2F957 */
  0x0000,   /* U+2F958 */
  0x0000,   /* U+2F959 */
  0x0000,   /* U+2F95A */
  0x0000,   /* U+2F95B */
  0x0000,   /* U+2F95C */
  0x0000,   /* U+2F95D */
  0x0000,   /* U+2F95E */
  0x0000,   /* U+2F95F */
  0x0000,   /* U+2F960 */
  0x0000,   /* U+2F961 */
  0x0000,   /* U+2F962 */
  0x0000,   /* U+2F963 */
  0x0000,   /* U+2F964 */
  0x0000,   /* U+2F965 */
  0x0000,   /* U+2F966 */
  0x0000,   /* U+2F967 */
  0x0000,   /* U+2F968 */
  0x0000,   /* U+2F969 */
  0x0000,   /* U+2F96A */
  0x0000,   /* U+2F96B */
  0x0000,   /* U+2F96C */
  0x0000,   /* U+2F96D */
  0x0000,   /* U+2F96E */
  0x0000,   /* U+2F96F */
  0x0000,   /* U+2F970 */
  0x0000,   /* U+2F971 */
  0x0000,   /* U+2F972 */
  0x0000,   /* U+2F973 */
  0x0000,   /* U+2F974 */
  0x0000,   /* U+2F975 */
  0x0000,   /* U+2F976 */
  0x0000,   /* U+2F977 */
  0x0000,   /* U+2F978 */
  0x0000,   /* U+2F979 */
  0x0000,   /* U+2F97A */
  0x0000,   /* U+2F97B */
  0x0000,   /* U+2F97C */
  0x0000,   /* U+2F97D */
  0x0000,   /* U+2F97E */
  0x0000,   /* U+2F97F */
  0x0000,   /* U+2F980 */
  0x0000,   /* U+2F981 */
  0x0000,   /* U+2F982 */
  0x0000,   /* U+2F983 */
  0x0000,   /* U+2F984 */
  0x0000,   /* U+2F985 */
  0x0000,   /* U+2F986 */
  0x0000,   /* U+2F987 */
  0x0000,   /* U+2F988 */
  0x0000,   /* U+2F989 */
  0x0000,   /* U+2F98A */
  0x0000,   /* U+2F98B */
  0x0000,   /* U+2F98C */
  0x0000,   /* U+2F98D */
  0x0000,   /* U+2F98E */
  0x0000,   /* U+2F98F */
  0x0000,   /* U+2F990 */
  0x0000,   /* U+2F991 */
  0x0000,   /* U+2F992 */
  0x0000,   /* U+2F993 */
  0x0000,   /* U+2F994 */
  0x0000,   /* U+2F995 */
  0x0000,   /* U+2F996 */
  0x0000,   /* U+2F997 */
  0x0000,   /* U+2F998 */
  0x0000,   /* U+2F999 */
  0x0000,   /* U+2F99A */
  0x0000,   /* U+2F99B */
  0x0000,   /* U+2F99C */
  0x0000,   /* U+2F99D */
  0x0000,   /* U+2F99E */
  0x0000,   /* U+2F99F */
  0x0000,   /* U+2F9A0 */
  0x0000,   /* U+2F9A1 */
  0x0000,   /* U+2F9A2 */
  0x0000,   /* U+2F9A3 */
  0x0000,   /* U+2F9A4 */
  0x0000,   /* U+2F9A5 */
  0x0000,   /* U+2F9A6 */
  0x0000,   /* U+2F9A7 */
  0x0000,   /* U+2F9A8 */
  0x0000,   /* U+2F9A9 */
  0x0000,   /* U+2F9AA */
  0x0000,   /* U+2F9AB */
  0x0000,   /* U+2F9AC */
  0x0000,   /* U+2F9AD */
  0x0000,   /* U+2F9AE */
  0x0000,   /* U+2F9AF */
  0x0000,   /* U+2F9B0 */
  0x0000,   /* U+2F9B1 */
  0x0000,   /* U+2F9B2 */
  0x0000,   /* U+2F9B3 */
  0x0000,   /* U+2F9B4 */
  0x0000,   /* U+2F9B5 */
  0x0000,   /* U+2F9B6 */
  0x0000,   /* U+2F9B7 */
  0x0000,   /* U+2F9B8 */
  0x0000,   /* U+2F9B9 */
  0x0000,   /* U+2F9BA */
  0x0000,   /* U+2F9BB */
  0x0000,   /* U+2F9BC */
  0x0000,   /* U+2F9BD */
  0x0000,   /* U+2F9BE */
  0x0000,   /* U+2F9BF */
  0x0000,   /* U+2F9C0 */
  0x0000,   /* U+2F9C1 */
  0x0000,   /* U+2F9C2 */
  0x0000,   /* U+2F9C3 */
  0x0000,   /* U+2F9C4 */
  0x0000,   /* U+2F9C5 */
  0x0000,   /* U+2F9C6 */
  0x0000,   /* U+2F9C7 */
  0x0000,   /* U+2F9C8 */
  0x0000,   /* U+2F9C9 */
  0x0000,   /* U+2F9CA */
  0x0000,   /* U+2F9CB */
  0x0000,   /* U+2F9CC */
  0x0000,   /* U+2F9CD */
  0x0000,   /* U+2F9CE */
  0x0000,   /* U+2F9CF */
  0x0000,   /* U+2F9D0 */
  0x0000,   /* U+2F9D1 */
  0x0000,   /* U+2F9D2 */
  0x0000,   /* U+2F9D3 */
  0x0000,   /* U+2F9D4 */
  0x0000,   /* U+2F9D5 */
  0x0000,   /* U+2F9D6 */
  0x0000,   /* U+2F9D7 */
  0x0000,   /* U+2F9D8 */
  0x0000,   /* U+2F9D9 */
  0x0000,   /* U+2F9DA */
  0x0000,   /* U+2F9DB */
  0x0000,   /* U+2F9DC */
  0x0000,   /* U+2F9DD */
  0x0000,   /* U+2F9DE */
  0x0000,   /* U+2F9DF */
  0x0000,   /* U+2F9E0 */
  0x0000,   /* U+2F9E1 */
  0x0000,   /* U+2F9E2 */
  0x0000,   /* U+2F9E3 */
  0x0000,   /* U+2F9E4 */
  0x0000,   /* U+2F9E5 */
  0x0000,   /* U+2F9E6 */
  0x0000,   /* U+2F9E7 */
  0x0000,   /* U+2F9E8 */
  0x0000,   /* U+2F9E9 */
  0x0000,   /* U+2F9EA */
  0x0000,   /* U+2F9EB */
  0x0000,   /* U+2F9EC */
  0x0000,   /* U+2F9ED */
  0x0000,   /* U+2F9EE */
  0x0000,   /* U+2F9EF */
  0x0000,   /* U+2F9F0 */
  0x0000,   /* U+2F9F1 */
  0x0000,   /* U+2F9F2 */
  0x0000,   /* U+2F9F3 */
  0x0000,   /* U+2F9F4 */
  0x0000,   /* U+2F9F5 */
  0x0000,   /* U+2F9F6 */
  0x0000,   /* U+2F9F7 */
  0x0000,   /* U+2F9F8 */
  0x0000,   /* U+2F9F9 */
  0x0000,   /* U+2F9FA */
  0x0000,   /* U+2F9FB */
  0x0000,   /* U+2F9FC */
  0x0000,   /* U+2F9FD */
  0x0000,   /* U+2F9FE */
  0x0000,   /* U+2F9FF */
};

uint16 uca900_p2FA[]= {
  /* Number of CEs for each character. */
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFB41,   /* U+2FA00 */
  0xFB85,   /* U+2FA01 */
  0xFB41,   /* U+2FA02 */
  0xFB80,   /* U+2FA03 */
  0xFB41,   /* U+2FA04 */
  0xFB41,   /* U+2FA05 */
  0xFB41,   /* U+2FA06 */
  0xFB41,   /* U+2FA07 */
  0xFB80,   /* U+2FA08 */
  0xFB85,   /* U+2FA09 */
  0xFB41,   /* U+2FA0A */
  0xFB41,   /* U+2FA0B */
  0xFB41,   /* U+2FA0C */
  0xFB80,   /* U+2FA0D */
  0xFB80,   /* U+2FA0E */
  0xFB41,   /* U+2FA0F */
  0xFB85,   /* U+2FA10 */
  0xFB80,   /* U+2FA11 */
  0xFB85,   /* U+2FA12 */
  0xFB85,   /* U+2FA13 */
  0xFB85,   /* U+2FA14 */
  0xFB41,   /* U+2FA15 */
  0xFB80,   /* U+2FA16 */
  0xFB41,   /* U+2FA17 */
  0xFB41,   /* U+2FA18 */
  0xFB41,   /* U+2FA19 */
  0xFB41,   /* U+2FA1A */
  0xFB41,   /* U+2FA1B */
  0xFB41,   /* U+2FA1C */
  0xFB85,   /* U+2FA1D */
  0xFBC5,   /* U+2FA1E */
  0xFBC5,   /* U+2FA1F */
  0xFBC5,   /* U+2FA20 */
  0xFBC5,   /* U+2FA21 */
  0xFBC5,   /* U+2FA22 */
  0xFBC5,   /* U+2FA23 */
  0xFBC5,   /* U+2FA24 */
  0xFBC5,   /* U+2FA25 */
  0xFBC5,   /* U+2FA26 */
  0xFBC5,   /* U+2FA27 */
  0xFBC5,   /* U+2FA28 */
  0xFBC5,   /* U+2FA29 */
  0xFBC5,   /* U+2FA2A */
  0xFBC5,   /* U+2FA2B */
  0xFBC5,   /* U+2FA2C */
  0xFBC5,   /* U+2FA2D */
  0xFBC5,   /* U+2FA2E */
  0xFBC5,   /* U+2FA2F */
  0xFBC5,   /* U+2FA30 */
  0xFBC5,   /* U+2FA31 */
  0xFBC5,   /* U+2FA32 */
  0xFBC5,   /* U+2FA33 */
  0xFBC5,   /* U+2FA34 */
  0xFBC5,   /* U+2FA35 */
  0xFBC5,   /* U+2FA36 */
  0xFBC5,   /* U+2FA37 */
  0xFBC5,   /* U+2FA38 */
  0xFBC5,   /* U+2FA39 */
  0xFBC5,   /* U+2FA3A */
  0xFBC5,   /* U+2FA3B */
  0xFBC5,   /* U+2FA3C */
  0xFBC5,   /* U+2FA3D */
  0xFBC5,   /* U+2FA3E */
  0xFBC5,   /* U+2FA3F */
  0xFBC5,   /* U+2FA40 */
  0xFBC5,   /* U+2FA41 */
  0xFBC5,   /* U+2FA42 */
  0xFBC5,   /* U+2FA43 */
  0xFBC5,   /* U+2FA44 */
  0xFBC5,   /* U+2FA45 */
  0xFBC5,   /* U+2FA46 */
  0xFBC5,   /* U+2FA47 */
  0xFBC5,   /* U+2FA48 */
  0xFBC5,   /* U+2FA49 */
  0xFBC5,   /* U+2FA4A */
  0xFBC5,   /* U+2FA4B */
  0xFBC5,   /* U+2FA4C */
  0xFBC5,   /* U+2FA4D */
  0xFBC5,   /* U+2FA4E */
  0xFBC5,   /* U+2FA4F */
  0xFBC5,   /* U+2FA50 */
  0xFBC5,   /* U+2FA51 */
  0xFBC5,   /* U+2FA52 */
  0xFBC5,   /* U+2FA53 */
  0xFBC5,   /* U+2FA54 */
  0xFBC5,   /* U+2FA55 */
  0xFBC5,   /* U+2FA56 */
  0xFBC5,   /* U+2FA57 */
  0xFBC5,   /* U+2FA58 */
  0xFBC5,   /* U+2FA59 */
  0xFBC5,   /* U+2FA5A */
  0xFBC5,   /* U+2FA5B */
  0xFBC5,   /* U+2FA5C */
  0xFBC5,   /* U+2FA5D */
  0xFBC5,   /* U+2FA5E */
  0xFBC5,   /* U+2FA5F */
  0xFBC5,   /* U+2FA60 */
  0xFBC5,   /* U+2FA61 */
  0xFBC5,   /* U+2FA62 */
  0xFBC5,   /* U+2FA63 */
  0xFBC5,   /* U+2FA64 */
  0xFBC5,   /* U+2FA65 */
  0xFBC5,   /* U+2FA66 */
  0xFBC5,   /* U+2FA67 */
  0xFBC5,   /* U+2FA68 */
  0xFBC5,   /* U+2FA69 */
  0xFBC5,   /* U+2FA6A */
  0xFBC5,   /* U+2FA6B */
  0xFBC5,   /* U+2FA6C */
  0xFBC5,   /* U+2FA6D */
  0xFBC5,   /* U+2FA6E */
  0xFBC5,   /* U+2FA6F */
  0xFBC5,   /* U+2FA70 */
  0xFBC5,   /* U+2FA71 */
  0xFBC5,   /* U+2FA72 */
  0xFBC5,   /* U+2FA73 */
  0xFBC5,   /* U+2FA74 */
  0xFBC5,   /* U+2FA75 */
  0xFBC5,   /* U+2FA76 */
  0xFBC5,   /* U+2FA77 */
  0xFBC5,   /* U+2FA78 */
  0xFBC5,   /* U+2FA79 */
  0xFBC5,   /* U+2FA7A */
  0xFBC5,   /* U+2FA7B */
  0xFBC5,   /* U+2FA7C */
  0xFBC5,   /* U+2FA7D */
  0xFBC5,   /* U+2FA7E */
  0xFBC5,   /* U+2FA7F */
  0xFBC5,   /* U+2FA80 */
  0xFBC5,   /* U+2FA81 */
  0xFBC5,   /* U+2FA82 */
  0xFBC5,   /* U+2FA83 */
  0xFBC5,   /* U+2FA84 */
  0xFBC5,   /* U+2FA85 */
  0xFBC5,   /* U+2FA86 */
  0xFBC5,   /* U+2FA87 */
  0xFBC5,   /* U+2FA88 */
  0xFBC5,   /* U+2FA89 */
  0xFBC5,   /* U+2FA8A */
  0xFBC5,   /* U+2FA8B */
  0xFBC5,   /* U+2FA8C */
  0xFBC5,   /* U+2FA8D */
  0xFBC5,   /* U+2FA8E */
  0xFBC5,   /* U+2FA8F */
  0xFBC5,   /* U+2FA90 */
  0xFBC5,   /* U+2FA91 */
  0xFBC5,   /* U+2FA92 */
  0xFBC5,   /* U+2FA93 */
  0xFBC5,   /* U+2FA94 */
  0xFBC5,   /* U+2FA95 */
  0xFBC5,   /* U+2FA96 */
  0xFBC5,   /* U+2FA97 */
  0xFBC5,   /* U+2FA98 */
  0xFBC5,   /* U+2FA99 */
  0xFBC5,   /* U+2FA9A */
  0xFBC5,   /* U+2FA9B */
  0xFBC5,   /* U+2FA9C */
  0xFBC5,   /* U+2FA9D */
  0xFBC5,   /* U+2FA9E */
  0xFBC5,   /* U+2FA9F */
  0xFBC5,   /* U+2FAA0 */
  0xFBC5,   /* U+2FAA1 */
  0xFBC5,   /* U+2FAA2 */
  0xFBC5,   /* U+2FAA3 */
  0xFBC5,   /* U+2FAA4 */
  0xFBC5,   /* U+2FAA5 */
  0xFBC5,   /* U+2FAA6 */
  0xFBC5,   /* U+2FAA7 */
  0xFBC5,   /* U+2FAA8 */
  0xFBC5,   /* U+2FAA9 */
  0xFBC5,   /* U+2FAAA */
  0xFBC5,   /* U+2FAAB */
  0xFBC5,   /* U+2FAAC */
  0xFBC5,   /* U+2FAAD */
  0xFBC5,   /* U+2FAAE */
  0xFBC5,   /* U+2FAAF */
  0xFBC5,   /* U+2FAB0 */
  0xFBC5,   /* U+2FAB1 */
  0xFBC5,   /* U+2FAB2 */
  0xFBC5,   /* U+2FAB3 */
  0xFBC5,   /* U+2FAB4 */
  0xFBC5,   /* U+2FAB5 */
  0xFBC5,   /* U+2FAB6 */
  0xFBC5,   /* U+2FAB7 */
  0xFBC5,   /* U+2FAB8 */
  0xFBC5,   /* U+2FAB9 */
  0xFBC5,   /* U+2FABA */
  0xFBC5,   /* U+2FABB */
  0xFBC5,   /* U+2FABC */
  0xFBC5,   /* U+2FABD */
  0xFBC5,   /* U+2FABE */
  0xFBC5,   /* U+2FABF */
  0xFBC5,   /* U+2FAC0 */
  0xFBC5,   /* U+2FAC1 */
  0xFBC5,   /* U+2FAC2 */
  0xFBC5,   /* U+2FAC3 */
  0xFBC5,   /* U+2FAC4 */
  0xFBC5,   /* U+2FAC5 */
  0xFBC5,   /* U+2FAC6 */
  0xFBC5,   /* U+2FAC7 */
  0xFBC5,   /* U+2FAC8 */
  0xFBC5,   /* U+2FAC9 */
  0xFBC5,   /* U+2FACA */
  0xFBC5,   /* U+2FACB */
  0xFBC5,   /* U+2FACC */
  0xFBC5,   /* U+2FACD */
  0xFBC5,   /* U+2FACE */
  0xFBC5,   /* U+2FACF */
  0xFBC5,   /* U+2FAD0 */
  0xFBC5,   /* U+2FAD1 */
  0xFBC5,   /* U+2FAD2 */
  0xFBC5,   /* U+2FAD3 */
  0xFBC5,   /* U+2FAD4 */
  0xFBC5,   /* U+2FAD5 */
  0xFBC5,   /* U+2FAD6 */
  0xFBC5,   /* U+2FAD7 */
  0xFBC5,   /* U+2FAD8 */
  0xFBC5,   /* U+2FAD9 */
  0xFBC5,   /* U+2FADA */
  0xFBC5,   /* U+2FADB */
  0xFBC5,   /* U+2FADC */
  0xFBC5,   /* U+2FADD */
  0xFBC5,   /* U+2FADE */
  0xFBC5,   /* U+2FADF */
  0xFBC5,   /* U+2FAE0 */
  0xFBC5,   /* U+2FAE1 */
  0xFBC5,   /* U+2FAE2 */
  0xFBC5,   /* U+2FAE3 */
  0xFBC5,   /* U+2FAE4 */
  0xFBC5,   /* U+2FAE5 */
  0xFBC5,   /* U+2FAE6 */
  0xFBC5,   /* U+2FAE7 */
  0xFBC5,   /* U+2FAE8 */
  0xFBC5,   /* U+2FAE9 */
  0xFBC5,   /* U+2FAEA */
  0xFBC5,   /* U+2FAEB */
  0xFBC5,   /* U+2FAEC */
  0xFBC5,   /* U+2FAED */
  0xFBC5,   /* U+2FAEE */
  0xFBC5,   /* U+2FAEF */
  0xFBC5,   /* U+2FAF0 */
  0xFBC5,   /* U+2FAF1 */
  0xFBC5,   /* U+2FAF2 */
  0xFBC5,   /* U+2FAF3 */
  0xFBC5,   /* U+2FAF4 */
  0xFBC5,   /* U+2FAF5 */
  0xFBC5,   /* U+2FAF6 */
  0xFBC5,   /* U+2FAF7 */
  0xFBC5,   /* U+2FAF8 */
  0xFBC5,   /* U+2FAF9 */
  0xFBC5,   /* U+2FAFA */
  0xFBC5,   /* U+2FAFB */
  0xFBC5,   /* U+2FAFC */
  0xFBC5,   /* U+2FAFD */
  0xFBC5,   /* U+2FAFE */
  0xFBC5,   /* U+2FAFF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+2FA00 */
  0x0020,   /* U+2FA01 */
  0x0020,   /* U+2FA02 */
  0x0020,   /* U+2FA03 */
  0x0020,   /* U+2FA04 */
  0x0020,   /* U+2FA05 */
  0x0020,   /* U+2FA06 */
  0x0020,   /* U+2FA07 */
  0x0020,   /* U+2FA08 */
  0x0020,   /* U+2FA09 */
  0x0020,   /* U+2FA0A */
  0x0020,   /* U+2FA0B */
  0x0020,   /* U+2FA0C */
  0x0020,   /* U+2FA0D */
  0x0020,   /* U+2FA0E */
  0x0020,   /* U+2FA0F */
  0x0020,   /* U+2FA10 */
  0x0020,   /* U+2FA11 */
  0x0020,   /* U+2FA12 */
  0x0020,   /* U+2FA13 */
  0x0020,   /* U+2FA14 */
  0x0020,   /* U+2FA15 */
  0x0020,   /* U+2FA16 */
  0x0020,   /* U+2FA17 */
  0x0020,   /* U+2FA18 */
  0x0020,   /* U+2FA19 */
  0x0020,   /* U+2FA1A */
  0x0020,   /* U+2FA1B */
  0x0020,   /* U+2FA1C */
  0x0020,   /* U+2FA1D */
  0x0020,   /* U+2FA1E */
  0x0020,   /* U+2FA1F */
  0x0020,   /* U+2FA20 */
  0x0020,   /* U+2FA21 */
  0x0020,   /* U+2FA22 */
  0x0020,   /* U+2FA23 */
  0x0020,   /* U+2FA24 */
  0x0020,   /* U+2FA25 */
  0x0020,   /* U+2FA26 */
  0x0020,   /* U+2FA27 */
  0x0020,   /* U+2FA28 */
  0x0020,   /* U+2FA29 */
  0x0020,   /* U+2FA2A */
  0x0020,   /* U+2FA2B */
  0x0020,   /* U+2FA2C */
  0x0020,   /* U+2FA2D */
  0x0020,   /* U+2FA2E */
  0x0020,   /* U+2FA2F */
  0x0020,   /* U+2FA30 */
  0x0020,   /* U+2FA31 */
  0x0020,   /* U+2FA32 */
  0x0020,   /* U+2FA33 */
  0x0020,   /* U+2FA34 */
  0x0020,   /* U+2FA35 */
  0x0020,   /* U+2FA36 */
  0x0020,   /* U+2FA37 */
  0x0020,   /* U+2FA38 */
  0x0020,   /* U+2FA39 */
  0x0020,   /* U+2FA3A */
  0x0020,   /* U+2FA3B */
  0x0020,   /* U+2FA3C */
  0x0020,   /* U+2FA3D */
  0x0020,   /* U+2FA3E */
  0x0020,   /* U+2FA3F */
  0x0020,   /* U+2FA40 */
  0x0020,   /* U+2FA41 */
  0x0020,   /* U+2FA42 */
  0x0020,   /* U+2FA43 */
  0x0020,   /* U+2FA44 */
  0x0020,   /* U+2FA45 */
  0x0020,   /* U+2FA46 */
  0x0020,   /* U+2FA47 */
  0x0020,   /* U+2FA48 */
  0x0020,   /* U+2FA49 */
  0x0020,   /* U+2FA4A */
  0x0020,   /* U+2FA4B */
  0x0020,   /* U+2FA4C */
  0x0020,   /* U+2FA4D */
  0x0020,   /* U+2FA4E */
  0x0020,   /* U+2FA4F */
  0x0020,   /* U+2FA50 */
  0x0020,   /* U+2FA51 */
  0x0020,   /* U+2FA52 */
  0x0020,   /* U+2FA53 */
  0x0020,   /* U+2FA54 */
  0x0020,   /* U+2FA55 */
  0x0020,   /* U+2FA56 */
  0x0020,   /* U+2FA57 */
  0x0020,   /* U+2FA58 */
  0x0020,   /* U+2FA59 */
  0x0020,   /* U+2FA5A */
  0x0020,   /* U+2FA5B */
  0x0020,   /* U+2FA5C */
  0x0020,   /* U+2FA5D */
  0x0020,   /* U+2FA5E */
  0x0020,   /* U+2FA5F */
  0x0020,   /* U+2FA60 */
  0x0020,   /* U+2FA61 */
  0x0020,   /* U+2FA62 */
  0x0020,   /* U+2FA63 */
  0x0020,   /* U+2FA64 */
  0x0020,   /* U+2FA65 */
  0x0020,   /* U+2FA66 */
  0x0020,   /* U+2FA67 */
  0x0020,   /* U+2FA68 */
  0x0020,   /* U+2FA69 */
  0x0020,   /* U+2FA6A */
  0x0020,   /* U+2FA6B */
  0x0020,   /* U+2FA6C */
  0x0020,   /* U+2FA6D */
  0x0020,   /* U+2FA6E */
  0x0020,   /* U+2FA6F */
  0x0020,   /* U+2FA70 */
  0x0020,   /* U+2FA71 */
  0x0020,   /* U+2FA72 */
  0x0020,   /* U+2FA73 */
  0x0020,   /* U+2FA74 */
  0x0020,   /* U+2FA75 */
  0x0020,   /* U+2FA76 */
  0x0020,   /* U+2FA77 */
  0x0020,   /* U+2FA78 */
  0x0020,   /* U+2FA79 */
  0x0020,   /* U+2FA7A */
  0x0020,   /* U+2FA7B */
  0x0020,   /* U+2FA7C */
  0x0020,   /* U+2FA7D */
  0x0020,   /* U+2FA7E */
  0x0020,   /* U+2FA7F */
  0x0020,   /* U+2FA80 */
  0x0020,   /* U+2FA81 */
  0x0020,   /* U+2FA82 */
  0x0020,   /* U+2FA83 */
  0x0020,   /* U+2FA84 */
  0x0020,   /* U+2FA85 */
  0x0020,   /* U+2FA86 */
  0x0020,   /* U+2FA87 */
  0x0020,   /* U+2FA88 */
  0x0020,   /* U+2FA89 */
  0x0020,   /* U+2FA8A */
  0x0020,   /* U+2FA8B */
  0x0020,   /* U+2FA8C */
  0x0020,   /* U+2FA8D */
  0x0020,   /* U+2FA8E */
  0x0020,   /* U+2FA8F */
  0x0020,   /* U+2FA90 */
  0x0020,   /* U+2FA91 */
  0x0020,   /* U+2FA92 */
  0x0020,   /* U+2FA93 */
  0x0020,   /* U+2FA94 */
  0x0020,   /* U+2FA95 */
  0x0020,   /* U+2FA96 */
  0x0020,   /* U+2FA97 */
  0x0020,   /* U+2FA98 */
  0x0020,   /* U+2FA99 */
  0x0020,   /* U+2FA9A */
  0x0020,   /* U+2FA9B */
  0x0020,   /* U+2FA9C */
  0x0020,   /* U+2FA9D */
  0x0020,   /* U+2FA9E */
  0x0020,   /* U+2FA9F */
  0x0020,   /* U+2FAA0 */
  0x0020,   /* U+2FAA1 */
  0x0020,   /* U+2FAA2 */
  0x0020,   /* U+2FAA3 */
  0x0020,   /* U+2FAA4 */
  0x0020,   /* U+2FAA5 */
  0x0020,   /* U+2FAA6 */
  0x0020,   /* U+2FAA7 */
  0x0020,   /* U+2FAA8 */
  0x0020,   /* U+2FAA9 */
  0x0020,   /* U+2FAAA */
  0x0020,   /* U+2FAAB */
  0x0020,   /* U+2FAAC */
  0x0020,   /* U+2FAAD */
  0x0020,   /* U+2FAAE */
  0x0020,   /* U+2FAAF */
  0x0020,   /* U+2FAB0 */
  0x0020,   /* U+2FAB1 */
  0x0020,   /* U+2FAB2 */
  0x0020,   /* U+2FAB3 */
  0x0020,   /* U+2FAB4 */
  0x0020,   /* U+2FAB5 */
  0x0020,   /* U+2FAB6 */
  0x0020,   /* U+2FAB7 */
  0x0020,   /* U+2FAB8 */
  0x0020,   /* U+2FAB9 */
  0x0020,   /* U+2FABA */
  0x0020,   /* U+2FABB */
  0x0020,   /* U+2FABC */
  0x0020,   /* U+2FABD */
  0x0020,   /* U+2FABE */
  0x0020,   /* U+2FABF */
  0x0020,   /* U+2FAC0 */
  0x0020,   /* U+2FAC1 */
  0x0020,   /* U+2FAC2 */
  0x0020,   /* U+2FAC3 */
  0x0020,   /* U+2FAC4 */
  0x0020,   /* U+2FAC5 */
  0x0020,   /* U+2FAC6 */
  0x0020,   /* U+2FAC7 */
  0x0020,   /* U+2FAC8 */
  0x0020,   /* U+2FAC9 */
  0x0020,   /* U+2FACA */
  0x0020,   /* U+2FACB */
  0x0020,   /* U+2FACC */
  0x0020,   /* U+2FACD */
  0x0020,   /* U+2FACE */
  0x0020,   /* U+2FACF */
  0x0020,   /* U+2FAD0 */
  0x0020,   /* U+2FAD1 */
  0x0020,   /* U+2FAD2 */
  0x0020,   /* U+2FAD3 */
  0x0020,   /* U+2FAD4 */
  0x0020,   /* U+2FAD5 */
  0x0020,   /* U+2FAD6 */
  0x0020,   /* U+2FAD7 */
  0x0020,   /* U+2FAD8 */
  0x0020,   /* U+2FAD9 */
  0x0020,   /* U+2FADA */
  0x0020,   /* U+2FADB */
  0x0020,   /* U+2FADC */
  0x0020,   /* U+2FADD */
  0x0020,   /* U+2FADE */
  0x0020,   /* U+2FADF */
  0x0020,   /* U+2FAE0 */
  0x0020,   /* U+2FAE1 */
  0x0020,   /* U+2FAE2 */
  0x0020,   /* U+2FAE3 */
  0x0020,   /* U+2FAE4 */
  0x0020,   /* U+2FAE5 */
  0x0020,   /* U+2FAE6 */
  0x0020,   /* U+2FAE7 */
  0x0020,   /* U+2FAE8 */
  0x0020,   /* U+2FAE9 */
  0x0020,   /* U+2FAEA */
  0x0020,   /* U+2FAEB */
  0x0020,   /* U+2FAEC */
  0x0020,   /* U+2FAED */
  0x0020,   /* U+2FAEE */
  0x0020,   /* U+2FAEF */
  0x0020,   /* U+2FAF0 */
  0x0020,   /* U+2FAF1 */
  0x0020,   /* U+2FAF2 */
  0x0020,   /* U+2FAF3 */
  0x0020,   /* U+2FAF4 */
  0x0020,   /* U+2FAF5 */
  0x0020,   /* U+2FAF6 */
  0x0020,   /* U+2FAF7 */
  0x0020,   /* U+2FAF8 */
  0x0020,   /* U+2FAF9 */
  0x0020,   /* U+2FAFA */
  0x0020,   /* U+2FAFB */
  0x0020,   /* U+2FAFC */
  0x0020,   /* U+2FAFD */
  0x0020,   /* U+2FAFE */
  0x0020,   /* U+2FAFF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+2FA00 */
  0x0002,   /* U+2FA01 */
  0x0002,   /* U+2FA02 */
  0x0002,   /* U+2FA03 */
  0x0002,   /* U+2FA04 */
  0x0002,   /* U+2FA05 */
  0x0002,   /* U+2FA06 */
  0x0002,   /* U+2FA07 */
  0x0002,   /* U+2FA08 */
  0x0002,   /* U+2FA09 */
  0x0002,   /* U+2FA0A */
  0x0002,   /* U+2FA0B */
  0x0002,   /* U+2FA0C */
  0x0002,   /* U+2FA0D */
  0x0002,   /* U+2FA0E */
  0x0002,   /* U+2FA0F */
  0x0002,   /* U+2FA10 */
  0x0002,   /* U+2FA11 */
  0x0002,   /* U+2FA12 */
  0x0002,   /* U+2FA13 */
  0x0002,   /* U+2FA14 */
  0x0002,   /* U+2FA15 */
  0x0002,   /* U+2FA16 */
  0x0002,   /* U+2FA17 */
  0x0002,   /* U+2FA18 */
  0x0002,   /* U+2FA19 */
  0x0002,   /* U+2FA1A */
  0x0002,   /* U+2FA1B */
  0x0002,   /* U+2FA1C */
  0x0002,   /* U+2FA1D */
  0x0002,   /* U+2FA1E */
  0x0002,   /* U+2FA1F */
  0x0002,   /* U+2FA20 */
  0x0002,   /* U+2FA21 */
  0x0002,   /* U+2FA22 */
  0x0002,   /* U+2FA23 */
  0x0002,   /* U+2FA24 */
  0x0002,   /* U+2FA25 */
  0x0002,   /* U+2FA26 */
  0x0002,   /* U+2FA27 */
  0x0002,   /* U+2FA28 */
  0x0002,   /* U+2FA29 */
  0x0002,   /* U+2FA2A */
  0x0002,   /* U+2FA2B */
  0x0002,   /* U+2FA2C */
  0x0002,   /* U+2FA2D */
  0x0002,   /* U+2FA2E */
  0x0002,   /* U+2FA2F */
  0x0002,   /* U+2FA30 */
  0x0002,   /* U+2FA31 */
  0x0002,   /* U+2FA32 */
  0x0002,   /* U+2FA33 */
  0x0002,   /* U+2FA34 */
  0x0002,   /* U+2FA35 */
  0x0002,   /* U+2FA36 */
  0x0002,   /* U+2FA37 */
  0x0002,   /* U+2FA38 */
  0x0002,   /* U+2FA39 */
  0x0002,   /* U+2FA3A */
  0x0002,   /* U+2FA3B */
  0x0002,   /* U+2FA3C */
  0x0002,   /* U+2FA3D */
  0x0002,   /* U+2FA3E */
  0x0002,   /* U+2FA3F */
  0x0002,   /* U+2FA40 */
  0x0002,   /* U+2FA41 */
  0x0002,   /* U+2FA42 */
  0x0002,   /* U+2FA43 */
  0x0002,   /* U+2FA44 */
  0x0002,   /* U+2FA45 */
  0x0002,   /* U+2FA46 */
  0x0002,   /* U+2FA47 */
  0x0002,   /* U+2FA48 */
  0x0002,   /* U+2FA49 */
  0x0002,   /* U+2FA4A */
  0x0002,   /* U+2FA4B */
  0x0002,   /* U+2FA4C */
  0x0002,   /* U+2FA4D */
  0x0002,   /* U+2FA4E */
  0x0002,   /* U+2FA4F */
  0x0002,   /* U+2FA50 */
  0x0002,   /* U+2FA51 */
  0x0002,   /* U+2FA52 */
  0x0002,   /* U+2FA53 */
  0x0002,   /* U+2FA54 */
  0x0002,   /* U+2FA55 */
  0x0002,   /* U+2FA56 */
  0x0002,   /* U+2FA57 */
  0x0002,   /* U+2FA58 */
  0x0002,   /* U+2FA59 */
  0x0002,   /* U+2FA5A */
  0x0002,   /* U+2FA5B */
  0x0002,   /* U+2FA5C */
  0x0002,   /* U+2FA5D */
  0x0002,   /* U+2FA5E */
  0x0002,   /* U+2FA5F */
  0x0002,   /* U+2FA60 */
  0x0002,   /* U+2FA61 */
  0x0002,   /* U+2FA62 */
  0x0002,   /* U+2FA63 */
  0x0002,   /* U+2FA64 */
  0x0002,   /* U+2FA65 */
  0x0002,   /* U+2FA66 */
  0x0002,   /* U+2FA67 */
  0x0002,   /* U+2FA68 */
  0x0002,   /* U+2FA69 */
  0x0002,   /* U+2FA6A */
  0x0002,   /* U+2FA6B */
  0x0002,   /* U+2FA6C */
  0x0002,   /* U+2FA6D */
  0x0002,   /* U+2FA6E */
  0x0002,   /* U+2FA6F */
  0x0002,   /* U+2FA70 */
  0x0002,   /* U+2FA71 */
  0x0002,   /* U+2FA72 */
  0x0002,   /* U+2FA73 */
  0x0002,   /* U+2FA74 */
  0x0002,   /* U+2FA75 */
  0x0002,   /* U+2FA76 */
  0x0002,   /* U+2FA77 */
  0x0002,   /* U+2FA78 */
  0x0002,   /* U+2FA79 */
  0x0002,   /* U+2FA7A */
  0x0002,   /* U+2FA7B */
  0x0002,   /* U+2FA7C */
  0x0002,   /* U+2FA7D */
  0x0002,   /* U+2FA7E */
  0x0002,   /* U+2FA7F */
  0x0002,   /* U+2FA80 */
  0x0002,   /* U+2FA81 */
  0x0002,   /* U+2FA82 */
  0x0002,   /* U+2FA83 */
  0x0002,   /* U+2FA84 */
  0x0002,   /* U+2FA85 */
  0x0002,   /* U+2FA86 */
  0x0002,   /* U+2FA87 */
  0x0002,   /* U+2FA88 */
  0x0002,   /* U+2FA89 */
  0x0002,   /* U+2FA8A */
  0x0002,   /* U+2FA8B */
  0x0002,   /* U+2FA8C */
  0x0002,   /* U+2FA8D */
  0x0002,   /* U+2FA8E */
  0x0002,   /* U+2FA8F */
  0x0002,   /* U+2FA90 */
  0x0002,   /* U+2FA91 */
  0x0002,   /* U+2FA92 */
  0x0002,   /* U+2FA93 */
  0x0002,   /* U+2FA94 */
  0x0002,   /* U+2FA95 */
  0x0002,   /* U+2FA96 */
  0x0002,   /* U+2FA97 */
  0x0002,   /* U+2FA98 */
  0x0002,   /* U+2FA99 */
  0x0002,   /* U+2FA9A */
  0x0002,   /* U+2FA9B */
  0x0002,   /* U+2FA9C */
  0x0002,   /* U+2FA9D */
  0x0002,   /* U+2FA9E */
  0x0002,   /* U+2FA9F */
  0x0002,   /* U+2FAA0 */
  0x0002,   /* U+2FAA1 */
  0x0002,   /* U+2FAA2 */
  0x0002,   /* U+2FAA3 */
  0x0002,   /* U+2FAA4 */
  0x0002,   /* U+2FAA5 */
  0x0002,   /* U+2FAA6 */
  0x0002,   /* U+2FAA7 */
  0x0002,   /* U+2FAA8 */
  0x0002,   /* U+2FAA9 */
  0x0002,   /* U+2FAAA */
  0x0002,   /* U+2FAAB */
  0x0002,   /* U+2FAAC */
  0x0002,   /* U+2FAAD */
  0x0002,   /* U+2FAAE */
  0x0002,   /* U+2FAAF */
  0x0002,   /* U+2FAB0 */
  0x0002,   /* U+2FAB1 */
  0x0002,   /* U+2FAB2 */
  0x0002,   /* U+2FAB3 */
  0x0002,   /* U+2FAB4 */
  0x0002,   /* U+2FAB5 */
  0x0002,   /* U+2FAB6 */
  0x0002,   /* U+2FAB7 */
  0x0002,   /* U+2FAB8 */
  0x0002,   /* U+2FAB9 */
  0x0002,   /* U+2FABA */
  0x0002,   /* U+2FABB */
  0x0002,   /* U+2FABC */
  0x0002,   /* U+2FABD */
  0x0002,   /* U+2FABE */
  0x0002,   /* U+2FABF */
  0x0002,   /* U+2FAC0 */
  0x0002,   /* U+2FAC1 */
  0x0002,   /* U+2FAC2 */
  0x0002,   /* U+2FAC3 */
  0x0002,   /* U+2FAC4 */
  0x0002,   /* U+2FAC5 */
  0x0002,   /* U+2FAC6 */
  0x0002,   /* U+2FAC7 */
  0x0002,   /* U+2FAC8 */
  0x0002,   /* U+2FAC9 */
  0x0002,   /* U+2FACA */
  0x0002,   /* U+2FACB */
  0x0002,   /* U+2FACC */
  0x0002,   /* U+2FACD */
  0x0002,   /* U+2FACE */
  0x0002,   /* U+2FACF */
  0x0002,   /* U+2FAD0 */
  0x0002,   /* U+2FAD1 */
  0x0002,   /* U+2FAD2 */
  0x0002,   /* U+2FAD3 */
  0x0002,   /* U+2FAD4 */
  0x0002,   /* U+2FAD5 */
  0x0002,   /* U+2FAD6 */
  0x0002,   /* U+2FAD7 */
  0x0002,   /* U+2FAD8 */
  0x0002,   /* U+2FAD9 */
  0x0002,   /* U+2FADA */
  0x0002,   /* U+2FADB */
  0x0002,   /* U+2FADC */
  0x0002,   /* U+2FADD */
  0x0002,   /* U+2FADE */
  0x0002,   /* U+2FADF */
  0x0002,   /* U+2FAE0 */
  0x0002,   /* U+2FAE1 */
  0x0002,   /* U+2FAE2 */
  0x0002,   /* U+2FAE3 */
  0x0002,   /* U+2FAE4 */
  0x0002,   /* U+2FAE5 */
  0x0002,   /* U+2FAE6 */
  0x0002,   /* U+2FAE7 */
  0x0002,   /* U+2FAE8 */
  0x0002,   /* U+2FAE9 */
  0x0002,   /* U+2FAEA */
  0x0002,   /* U+2FAEB */
  0x0002,   /* U+2FAEC */
  0x0002,   /* U+2FAED */
  0x0002,   /* U+2FAEE */
  0x0002,   /* U+2FAEF */
  0x0002,   /* U+2FAF0 */
  0x0002,   /* U+2FAF1 */
  0x0002,   /* U+2FAF2 */
  0x0002,   /* U+2FAF3 */
  0x0002,   /* U+2FAF4 */
  0x0002,   /* U+2FAF5 */
  0x0002,   /* U+2FAF6 */
  0x0002,   /* U+2FAF7 */
  0x0002,   /* U+2FAF8 */
  0x0002,   /* U+2FAF9 */
  0x0002,   /* U+2FAFA */
  0x0002,   /* U+2FAFB */
  0x0002,   /* U+2FAFC */
  0x0002,   /* U+2FAFD */
  0x0002,   /* U+2FAFE */
  0x0002,   /* U+2FAFF */

  /* Primary weight 2 for each character. */
  0x9829,   /* U+2FA00 */
  0x95B6,   /* U+2FA01 */
  0x98E2,   /* U+2FA02 */
  0xCB33,   /* U+2FA03 */
  0x9929,   /* U+2FA04 */
  0x99A7,   /* U+2FA05 */
  0x99C2,   /* U+2FA06 */
  0x99FE,   /* U+2FA07 */
  0xCBCE,   /* U+2FA08 */
  0x9B30,   /* U+2FA09 */
  0x9B12,   /* U+2FA0A */
  0x9C40,   /* U+2FA0B */
  0x9CFD,   /* U+2FA0C */
  0xCCCE,   /* U+2FA0D */
  0xCCED,   /* U+2FA0E */
  0x9D67,   /* U+2FA0F */
  0xA0CE,   /* U+2FA10 */
  0xCCF8,   /* U+2FA11 */
  0xA105,   /* U+2FA12 */
  0xA20E,   /* U+2FA13 */
  0xA291,   /* U+2FA14 */
  0x9EBB,   /* U+2FA15 */
  0xCD56,   /* U+2FA16 */
  0x9EF9,   /* U+2FA17 */
  0x9EFE,   /* U+2FA18 */
  0x9F05,   /* U+2FA19 */
  0x9F0F,   /* U+2FA1A */
  0x9F16,   /* U+2FA1B */
  0x9F3B,   /* U+2FA1C */
  0xA600,   /* U+2FA1D */
  0xFA1E,   /* U+2FA1E */
  0xFA1F,   /* U+2FA1F */
  0xFA20,   /* U+2FA20 */
  0xFA21,   /* U+2FA21 */
  0xFA22,   /* U+2FA22 */
  0xFA23,   /* U+2FA23 */
  0xFA24,   /* U+2FA24 */
  0xFA25,   /* U+2FA25 */
  0xFA26,   /* U+2FA26 */
  0xFA27,   /* U+2FA27 */
  0xFA28,   /* U+2FA28 */
  0xFA29,   /* U+2FA29 */
  0xFA2A,   /* U+2FA2A */
  0xFA2B,   /* U+2FA2B */
  0xFA2C,   /* U+2FA2C */
  0xFA2D,   /* U+2FA2D */
  0xFA2E,   /* U+2FA2E */
  0xFA2F,   /* U+2FA2F */
  0xFA30,   /* U+2FA30 */
  0xFA31,   /* U+2FA31 */
  0xFA32,   /* U+2FA32 */
  0xFA33,   /* U+2FA33 */
  0xFA34,   /* U+2FA34 */
  0xFA35,   /* U+2FA35 */
  0xFA36,   /* U+2FA36 */
  0xFA37,   /* U+2FA37 */
  0xFA38,   /* U+2FA38 */
  0xFA39,   /* U+2FA39 */
  0xFA3A,   /* U+2FA3A */
  0xFA3B,   /* U+2FA3B */
  0xFA3C,   /* U+2FA3C */
  0xFA3D,   /* U+2FA3D */
  0xFA3E,   /* U+2FA3E */
  0xFA3F,   /* U+2FA3F */
  0xFA40,   /* U+2FA40 */
  0xFA41,   /* U+2FA41 */
  0xFA42,   /* U+2FA42 */
  0xFA43,   /* U+2FA43 */
  0xFA44,   /* U+2FA44 */
  0xFA45,   /* U+2FA45 */
  0xFA46,   /* U+2FA46 */
  0xFA47,   /* U+2FA47 */
  0xFA48,   /* U+2FA48 */
  0xFA49,   /* U+2FA49 */
  0xFA4A,   /* U+2FA4A */
  0xFA4B,   /* U+2FA4B */
  0xFA4C,   /* U+2FA4C */
  0xFA4D,   /* U+2FA4D */
  0xFA4E,   /* U+2FA4E */
  0xFA4F,   /* U+2FA4F */
  0xFA50,   /* U+2FA50 */
  0xFA51,   /* U+2FA51 */
  0xFA52,   /* U+2FA52 */
  0xFA53,   /* U+2FA53 */
  0xFA54,   /* U+2FA54 */
  0xFA55,   /* U+2FA55 */
  0xFA56,   /* U+2FA56 */
  0xFA57,   /* U+2FA57 */
  0xFA58,   /* U+2FA58 */
  0xFA59,   /* U+2FA59 */
  0xFA5A,   /* U+2FA5A */
  0xFA5B,   /* U+2FA5B */
  0xFA5C,   /* U+2FA5C */
  0xFA5D,   /* U+2FA5D */
  0xFA5E,   /* U+2FA5E */
  0xFA5F,   /* U+2FA5F */
  0xFA60,   /* U+2FA60 */
  0xFA61,   /* U+2FA61 */
  0xFA62,   /* U+2FA62 */
  0xFA63,   /* U+2FA63 */
  0xFA64,   /* U+2FA64 */
  0xFA65,   /* U+2FA65 */
  0xFA66,   /* U+2FA66 */
  0xFA67,   /* U+2FA67 */
  0xFA68,   /* U+2FA68 */
  0xFA69,   /* U+2FA69 */
  0xFA6A,   /* U+2FA6A */
  0xFA6B,   /* U+2FA6B */
  0xFA6C,   /* U+2FA6C */
  0xFA6D,   /* U+2FA6D */
  0xFA6E,   /* U+2FA6E */
  0xFA6F,   /* U+2FA6F */
  0xFA70,   /* U+2FA70 */
  0xFA71,   /* U+2FA71 */
  0xFA72,   /* U+2FA72 */
  0xFA73,   /* U+2FA73 */
  0xFA74,   /* U+2FA74 */
  0xFA75,   /* U+2FA75 */
  0xFA76,   /* U+2FA76 */
  0xFA77,   /* U+2FA77 */
  0xFA78,   /* U+2FA78 */
  0xFA79,   /* U+2FA79 */
  0xFA7A,   /* U+2FA7A */
  0xFA7B,   /* U+2FA7B */
  0xFA7C,   /* U+2FA7C */
  0xFA7D,   /* U+2FA7D */
  0xFA7E,   /* U+2FA7E */
  0xFA7F,   /* U+2FA7F */
  0xFA80,   /* U+2FA80 */
  0xFA81,   /* U+2FA81 */
  0xFA82,   /* U+2FA82 */
  0xFA83,   /* U+2FA83 */
  0xFA84,   /* U+2FA84 */
  0xFA85,   /* U+2FA85 */
  0xFA86,   /* U+2FA86 */
  0xFA87,   /* U+2FA87 */
  0xFA88,   /* U+2FA88 */
  0xFA89,   /* U+2FA89 */
  0xFA8A,   /* U+2FA8A */
  0xFA8B,   /* U+2FA8B */
  0xFA8C,   /* U+2FA8C */
  0xFA8D,   /* U+2FA8D */
  0xFA8E,   /* U+2FA8E */
  0xFA8F,   /* U+2FA8F */
  0xFA90,   /* U+2FA90 */
  0xFA91,   /* U+2FA91 */
  0xFA92,   /* U+2FA92 */
  0xFA93,   /* U+2FA93 */
  0xFA94,   /* U+2FA94 */
  0xFA95,   /* U+2FA95 */
  0xFA96,   /* U+2FA96 */
  0xFA97,   /* U+2FA97 */
  0xFA98,   /* U+2FA98 */
  0xFA99,   /* U+2FA99 */
  0xFA9A,   /* U+2FA9A */
  0xFA9B,   /* U+2FA9B */
  0xFA9C,   /* U+2FA9C */
  0xFA9D,   /* U+2FA9D */
  0xFA9E,   /* U+2FA9E */
  0xFA9F,   /* U+2FA9F */
  0xFAA0,   /* U+2FAA0 */
  0xFAA1,   /* U+2FAA1 */
  0xFAA2,   /* U+2FAA2 */
  0xFAA3,   /* U+2FAA3 */
  0xFAA4,   /* U+2FAA4 */
  0xFAA5,   /* U+2FAA5 */
  0xFAA6,   /* U+2FAA6 */
  0xFAA7,   /* U+2FAA7 */
  0xFAA8,   /* U+2FAA8 */
  0xFAA9,   /* U+2FAA9 */
  0xFAAA,   /* U+2FAAA */
  0xFAAB,   /* U+2FAAB */
  0xFAAC,   /* U+2FAAC */
  0xFAAD,   /* U+2FAAD */
  0xFAAE,   /* U+2FAAE */
  0xFAAF,   /* U+2FAAF */
  0xFAB0,   /* U+2FAB0 */
  0xFAB1,   /* U+2FAB1 */
  0xFAB2,   /* U+2FAB2 */
  0xFAB3,   /* U+2FAB3 */
  0xFAB4,   /* U+2FAB4 */
  0xFAB5,   /* U+2FAB5 */
  0xFAB6,   /* U+2FAB6 */
  0xFAB7,   /* U+2FAB7 */
  0xFAB8,   /* U+2FAB8 */
  0xFAB9,   /* U+2FAB9 */
  0xFABA,   /* U+2FABA */
  0xFABB,   /* U+2FABB */
  0xFABC,   /* U+2FABC */
  0xFABD,   /* U+2FABD */
  0xFABE,   /* U+2FABE */
  0xFABF,   /* U+2FABF */
  0xFAC0,   /* U+2FAC0 */
  0xFAC1,   /* U+2FAC1 */
  0xFAC2,   /* U+2FAC2 */
  0xFAC3,   /* U+2FAC3 */
  0xFAC4,   /* U+2FAC4 */
  0xFAC5,   /* U+2FAC5 */
  0xFAC6,   /* U+2FAC6 */
  0xFAC7,   /* U+2FAC7 */
  0xFAC8,   /* U+2FAC8 */
  0xFAC9,   /* U+2FAC9 */
  0xFACA,   /* U+2FACA */
  0xFACB,   /* U+2FACB */
  0xFACC,   /* U+2FACC */
  0xFACD,   /* U+2FACD */
  0xFACE,   /* U+2FACE */
  0xFACF,   /* U+2FACF */
  0xFAD0,   /* U+2FAD0 */
  0xFAD1,   /* U+2FAD1 */
  0xFAD2,   /* U+2FAD2 */
  0xFAD3,   /* U+2FAD3 */
  0xFAD4,   /* U+2FAD4 */
  0xFAD5,   /* U+2FAD5 */
  0xFAD6,   /* U+2FAD6 */
  0xFAD7,   /* U+2FAD7 */
  0xFAD8,   /* U+2FAD8 */
  0xFAD9,   /* U+2FAD9 */
  0xFADA,   /* U+2FADA */
  0xFADB,   /* U+2FADB */
  0xFADC,   /* U+2FADC */
  0xFADD,   /* U+2FADD */
  0xFADE,   /* U+2FADE */
  0xFADF,   /* U+2FADF */
  0xFAE0,   /* U+2FAE0 */
  0xFAE1,   /* U+2FAE1 */
  0xFAE2,   /* U+2FAE2 */
  0xFAE3,   /* U+2FAE3 */
  0xFAE4,   /* U+2FAE4 */
  0xFAE5,   /* U+2FAE5 */
  0xFAE6,   /* U+2FAE6 */
  0xFAE7,   /* U+2FAE7 */
  0xFAE8,   /* U+2FAE8 */
  0xFAE9,   /* U+2FAE9 */
  0xFAEA,   /* U+2FAEA */
  0xFAEB,   /* U+2FAEB */
  0xFAEC,   /* U+2FAEC */
  0xFAED,   /* U+2FAED */
  0xFAEE,   /* U+2FAEE */
  0xFAEF,   /* U+2FAEF */
  0xFAF0,   /* U+2FAF0 */
  0xFAF1,   /* U+2FAF1 */
  0xFAF2,   /* U+2FAF2 */
  0xFAF3,   /* U+2FAF3 */
  0xFAF4,   /* U+2FAF4 */
  0xFAF5,   /* U+2FAF5 */
  0xFAF6,   /* U+2FAF6 */
  0xFAF7,   /* U+2FAF7 */
  0xFAF8,   /* U+2FAF8 */
  0xFAF9,   /* U+2FAF9 */
  0xFAFA,   /* U+2FAFA */
  0xFAFB,   /* U+2FAFB */
  0xFAFC,   /* U+2FAFC */
  0xFAFD,   /* U+2FAFD */
  0xFAFE,   /* U+2FAFE */
  0xFAFF,   /* U+2FAFF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+2FA00 */
  0x0000,   /* U+2FA01 */
  0x0000,   /* U+2FA02 */
  0x0000,   /* U+2FA03 */
  0x0000,   /* U+2FA04 */
  0x0000,   /* U+2FA05 */
  0x0000,   /* U+2FA06 */
  0x0000,   /* U+2FA07 */
  0x0000,   /* U+2FA08 */
  0x0000,   /* U+2FA09 */
  0x0000,   /* U+2FA0A */
  0x0000,   /* U+2FA0B */
  0x0000,   /* U+2FA0C */
  0x0000,   /* U+2FA0D */
  0x0000,   /* U+2FA0E */
  0x0000,   /* U+2FA0F */
  0x0000,   /* U+2FA10 */
  0x0000,   /* U+2FA11 */
  0x0000,   /* U+2FA12 */
  0x0000,   /* U+2FA13 */
  0x0000,   /* U+2FA14 */
  0x0000,   /* U+2FA15 */
  0x0000,   /* U+2FA16 */
  0x0000,   /* U+2FA17 */
  0x0000,   /* U+2FA18 */
  0x0000,   /* U+2FA19 */
  0x0000,   /* U+2FA1A */
  0x0000,   /* U+2FA1B */
  0x0000,   /* U+2FA1C */
  0x0000,   /* U+2FA1D */
  0x0000,   /* U+2FA1E */
  0x0000,   /* U+2FA1F */
  0x0000,   /* U+2FA20 */
  0x0000,   /* U+2FA21 */
  0x0000,   /* U+2FA22 */
  0x0000,   /* U+2FA23 */
  0x0000,   /* U+2FA24 */
  0x0000,   /* U+2FA25 */
  0x0000,   /* U+2FA26 */
  0x0000,   /* U+2FA27 */
  0x0000,   /* U+2FA28 */
  0x0000,   /* U+2FA29 */
  0x0000,   /* U+2FA2A */
  0x0000,   /* U+2FA2B */
  0x0000,   /* U+2FA2C */
  0x0000,   /* U+2FA2D */
  0x0000,   /* U+2FA2E */
  0x0000,   /* U+2FA2F */
  0x0000,   /* U+2FA30 */
  0x0000,   /* U+2FA31 */
  0x0000,   /* U+2FA32 */
  0x0000,   /* U+2FA33 */
  0x0000,   /* U+2FA34 */
  0x0000,   /* U+2FA35 */
  0x0000,   /* U+2FA36 */
  0x0000,   /* U+2FA37 */
  0x0000,   /* U+2FA38 */
  0x0000,   /* U+2FA39 */
  0x0000,   /* U+2FA3A */
  0x0000,   /* U+2FA3B */
  0x0000,   /* U+2FA3C */
  0x0000,   /* U+2FA3D */
  0x0000,   /* U+2FA3E */
  0x0000,   /* U+2FA3F */
  0x0000,   /* U+2FA40 */
  0x0000,   /* U+2FA41 */
  0x0000,   /* U+2FA42 */
  0x0000,   /* U+2FA43 */
  0x0000,   /* U+2FA44 */
  0x0000,   /* U+2FA45 */
  0x0000,   /* U+2FA46 */
  0x0000,   /* U+2FA47 */
  0x0000,   /* U+2FA48 */
  0x0000,   /* U+2FA49 */
  0x0000,   /* U+2FA4A */
  0x0000,   /* U+2FA4B */
  0x0000,   /* U+2FA4C */
  0x0000,   /* U+2FA4D */
  0x0000,   /* U+2FA4E */
  0x0000,   /* U+2FA4F */
  0x0000,   /* U+2FA50 */
  0x0000,   /* U+2FA51 */
  0x0000,   /* U+2FA52 */
  0x0000,   /* U+2FA53 */
  0x0000,   /* U+2FA54 */
  0x0000,   /* U+2FA55 */
  0x0000,   /* U+2FA56 */
  0x0000,   /* U+2FA57 */
  0x0000,   /* U+2FA58 */
  0x0000,   /* U+2FA59 */
  0x0000,   /* U+2FA5A */
  0x0000,   /* U+2FA5B */
  0x0000,   /* U+2FA5C */
  0x0000,   /* U+2FA5D */
  0x0000,   /* U+2FA5E */
  0x0000,   /* U+2FA5F */
  0x0000,   /* U+2FA60 */
  0x0000,   /* U+2FA61 */
  0x0000,   /* U+2FA62 */
  0x0000,   /* U+2FA63 */
  0x0000,   /* U+2FA64 */
  0x0000,   /* U+2FA65 */
  0x0000,   /* U+2FA66 */
  0x0000,   /* U+2FA67 */
  0x0000,   /* U+2FA68 */
  0x0000,   /* U+2FA69 */
  0x0000,   /* U+2FA6A */
  0x0000,   /* U+2FA6B */
  0x0000,   /* U+2FA6C */
  0x0000,   /* U+2FA6D */
  0x0000,   /* U+2FA6E */
  0x0000,   /* U+2FA6F */
  0x0000,   /* U+2FA70 */
  0x0000,   /* U+2FA71 */
  0x0000,   /* U+2FA72 */
  0x0000,   /* U+2FA73 */
  0x0000,   /* U+2FA74 */
  0x0000,   /* U+2FA75 */
  0x0000,   /* U+2FA76 */
  0x0000,   /* U+2FA77 */
  0x0000,   /* U+2FA78 */
  0x0000,   /* U+2FA79 */
  0x0000,   /* U+2FA7A */
  0x0000,   /* U+2FA7B */
  0x0000,   /* U+2FA7C */
  0x0000,   /* U+2FA7D */
  0x0000,   /* U+2FA7E */
  0x0000,   /* U+2FA7F */
  0x0000,   /* U+2FA80 */
  0x0000,   /* U+2FA81 */
  0x0000,   /* U+2FA82 */
  0x0000,   /* U+2FA83 */
  0x0000,   /* U+2FA84 */
  0x0000,   /* U+2FA85 */
  0x0000,   /* U+2FA86 */
  0x0000,   /* U+2FA87 */
  0x0000,   /* U+2FA88 */
  0x0000,   /* U+2FA89 */
  0x0000,   /* U+2FA8A */
  0x0000,   /* U+2FA8B */
  0x0000,   /* U+2FA8C */
  0x0000,   /* U+2FA8D */
  0x0000,   /* U+2FA8E */
  0x0000,   /* U+2FA8F */
  0x0000,   /* U+2FA90 */
  0x0000,   /* U+2FA91 */
  0x0000,   /* U+2FA92 */
  0x0000,   /* U+2FA93 */
  0x0000,   /* U+2FA94 */
  0x0000,   /* U+2FA95 */
  0x0000,   /* U+2FA96 */
  0x0000,   /* U+2FA97 */
  0x0000,   /* U+2FA98 */
  0x0000,   /* U+2FA99 */
  0x0000,   /* U+2FA9A */
  0x0000,   /* U+2FA9B */
  0x0000,   /* U+2FA9C */
  0x0000,   /* U+2FA9D */
  0x0000,   /* U+2FA9E */
  0x0000,   /* U+2FA9F */
  0x0000,   /* U+2FAA0 */
  0x0000,   /* U+2FAA1 */
  0x0000,   /* U+2FAA2 */
  0x0000,   /* U+2FAA3 */
  0x0000,   /* U+2FAA4 */
  0x0000,   /* U+2FAA5 */
  0x0000,   /* U+2FAA6 */
  0x0000,   /* U+2FAA7 */
  0x0000,   /* U+2FAA8 */
  0x0000,   /* U+2FAA9 */
  0x0000,   /* U+2FAAA */
  0x0000,   /* U+2FAAB */
  0x0000,   /* U+2FAAC */
  0x0000,   /* U+2FAAD */
  0x0000,   /* U+2FAAE */
  0x0000,   /* U+2FAAF */
  0x0000,   /* U+2FAB0 */
  0x0000,   /* U+2FAB1 */
  0x0000,   /* U+2FAB2 */
  0x0000,   /* U+2FAB3 */
  0x0000,   /* U+2FAB4 */
  0x0000,   /* U+2FAB5 */
  0x0000,   /* U+2FAB6 */
  0x0000,   /* U+2FAB7 */
  0x0000,   /* U+2FAB8 */
  0x0000,   /* U+2FAB9 */
  0x0000,   /* U+2FABA */
  0x0000,   /* U+2FABB */
  0x0000,   /* U+2FABC */
  0x0000,   /* U+2FABD */
  0x0000,   /* U+2FABE */
  0x0000,   /* U+2FABF */
  0x0000,   /* U+2FAC0 */
  0x0000,   /* U+2FAC1 */
  0x0000,   /* U+2FAC2 */
  0x0000,   /* U+2FAC3 */
  0x0000,   /* U+2FAC4 */
  0x0000,   /* U+2FAC5 */
  0x0000,   /* U+2FAC6 */
  0x0000,   /* U+2FAC7 */
  0x0000,   /* U+2FAC8 */
  0x0000,   /* U+2FAC9 */
  0x0000,   /* U+2FACA */
  0x0000,   /* U+2FACB */
  0x0000,   /* U+2FACC */
  0x0000,   /* U+2FACD */
  0x0000,   /* U+2FACE */
  0x0000,   /* U+2FACF */
  0x0000,   /* U+2FAD0 */
  0x0000,   /* U+2FAD1 */
  0x0000,   /* U+2FAD2 */
  0x0000,   /* U+2FAD3 */
  0x0000,   /* U+2FAD4 */
  0x0000,   /* U+2FAD5 */
  0x0000,   /* U+2FAD6 */
  0x0000,   /* U+2FAD7 */
  0x0000,   /* U+2FAD8 */
  0x0000,   /* U+2FAD9 */
  0x0000,   /* U+2FADA */
  0x0000,   /* U+2FADB */
  0x0000,   /* U+2FADC */
  0x0000,   /* U+2FADD */
  0x0000,   /* U+2FADE */
  0x0000,   /* U+2FADF */
  0x0000,   /* U+2FAE0 */
  0x0000,   /* U+2FAE1 */
  0x0000,   /* U+2FAE2 */
  0x0000,   /* U+2FAE3 */
  0x0000,   /* U+2FAE4 */
  0x0000,   /* U+2FAE5 */
  0x0000,   /* U+2FAE6 */
  0x0000,   /* U+2FAE7 */
  0x0000,   /* U+2FAE8 */
  0x0000,   /* U+2FAE9 */
  0x0000,   /* U+2FAEA */
  0x0000,   /* U+2FAEB */
  0x0000,   /* U+2FAEC */
  0x0000,   /* U+2FAED */
  0x0000,   /* U+2FAEE */
  0x0000,   /* U+2FAEF */
  0x0000,   /* U+2FAF0 */
  0x0000,   /* U+2FAF1 */
  0x0000,   /* U+2FAF2 */
  0x0000,   /* U+2FAF3 */
  0x0000,   /* U+2FAF4 */
  0x0000,   /* U+2FAF5 */
  0x0000,   /* U+2FAF6 */
  0x0000,   /* U+2FAF7 */
  0x0000,   /* U+2FAF8 */
  0x0000,   /* U+2FAF9 */
  0x0000,   /* U+2FAFA */
  0x0000,   /* U+2FAFB */
  0x0000,   /* U+2FAFC */
  0x0000,   /* U+2FAFD */
  0x0000,   /* U+2FAFE */
  0x0000,   /* U+2FAFF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+2FA00 */
  0x0000,   /* U+2FA01 */
  0x0000,   /* U+2FA02 */
  0x0000,   /* U+2FA03 */
  0x0000,   /* U+2FA04 */
  0x0000,   /* U+2FA05 */
  0x0000,   /* U+2FA06 */
  0x0000,   /* U+2FA07 */
  0x0000,   /* U+2FA08 */
  0x0000,   /* U+2FA09 */
  0x0000,   /* U+2FA0A */
  0x0000,   /* U+2FA0B */
  0x0000,   /* U+2FA0C */
  0x0000,   /* U+2FA0D */
  0x0000,   /* U+2FA0E */
  0x0000,   /* U+2FA0F */
  0x0000,   /* U+2FA10 */
  0x0000,   /* U+2FA11 */
  0x0000,   /* U+2FA12 */
  0x0000,   /* U+2FA13 */
  0x0000,   /* U+2FA14 */
  0x0000,   /* U+2FA15 */
  0x0000,   /* U+2FA16 */
  0x0000,   /* U+2FA17 */
  0x0000,   /* U+2FA18 */
  0x0000,   /* U+2FA19 */
  0x0000,   /* U+2FA1A */
  0x0000,   /* U+2FA1B */
  0x0000,   /* U+2FA1C */
  0x0000,   /* U+2FA1D */
  0x0000,   /* U+2FA1E */
  0x0000,   /* U+2FA1F */
  0x0000,   /* U+2FA20 */
  0x0000,   /* U+2FA21 */
  0x0000,   /* U+2FA22 */
  0x0000,   /* U+2FA23 */
  0x0000,   /* U+2FA24 */
  0x0000,   /* U+2FA25 */
  0x0000,   /* U+2FA26 */
  0x0000,   /* U+2FA27 */
  0x0000,   /* U+2FA28 */
  0x0000,   /* U+2FA29 */
  0x0000,   /* U+2FA2A */
  0x0000,   /* U+2FA2B */
  0x0000,   /* U+2FA2C */
  0x0000,   /* U+2FA2D */
  0x0000,   /* U+2FA2E */
  0x0000,   /* U+2FA2F */
  0x0000,   /* U+2FA30 */
  0x0000,   /* U+2FA31 */
  0x0000,   /* U+2FA32 */
  0x0000,   /* U+2FA33 */
  0x0000,   /* U+2FA34 */
  0x0000,   /* U+2FA35 */
  0x0000,   /* U+2FA36 */
  0x0000,   /* U+2FA37 */
  0x0000,   /* U+2FA38 */
  0x0000,   /* U+2FA39 */
  0x0000,   /* U+2FA3A */
  0x0000,   /* U+2FA3B */
  0x0000,   /* U+2FA3C */
  0x0000,   /* U+2FA3D */
  0x0000,   /* U+2FA3E */
  0x0000,   /* U+2FA3F */
  0x0000,   /* U+2FA40 */
  0x0000,   /* U+2FA41 */
  0x0000,   /* U+2FA42 */
  0x0000,   /* U+2FA43 */
  0x0000,   /* U+2FA44 */
  0x0000,   /* U+2FA45 */
  0x0000,   /* U+2FA46 */
  0x0000,   /* U+2FA47 */
  0x0000,   /* U+2FA48 */
  0x0000,   /* U+2FA49 */
  0x0000,   /* U+2FA4A */
  0x0000,   /* U+2FA4B */
  0x0000,   /* U+2FA4C */
  0x0000,   /* U+2FA4D */
  0x0000,   /* U+2FA4E */
  0x0000,   /* U+2FA4F */
  0x0000,   /* U+2FA50 */
  0x0000,   /* U+2FA51 */
  0x0000,   /* U+2FA52 */
  0x0000,   /* U+2FA53 */
  0x0000,   /* U+2FA54 */
  0x0000,   /* U+2FA55 */
  0x0000,   /* U+2FA56 */
  0x0000,   /* U+2FA57 */
  0x0000,   /* U+2FA58 */
  0x0000,   /* U+2FA59 */
  0x0000,   /* U+2FA5A */
  0x0000,   /* U+2FA5B */
  0x0000,   /* U+2FA5C */
  0x0000,   /* U+2FA5D */
  0x0000,   /* U+2FA5E */
  0x0000,   /* U+2FA5F */
  0x0000,   /* U+2FA60 */
  0x0000,   /* U+2FA61 */
  0x0000,   /* U+2FA62 */
  0x0000,   /* U+2FA63 */
  0x0000,   /* U+2FA64 */
  0x0000,   /* U+2FA65 */
  0x0000,   /* U+2FA66 */
  0x0000,   /* U+2FA67 */
  0x0000,   /* U+2FA68 */
  0x0000,   /* U+2FA69 */
  0x0000,   /* U+2FA6A */
  0x0000,   /* U+2FA6B */
  0x0000,   /* U+2FA6C */
  0x0000,   /* U+2FA6D */
  0x0000,   /* U+2FA6E */
  0x0000,   /* U+2FA6F */
  0x0000,   /* U+2FA70 */
  0x0000,   /* U+2FA71 */
  0x0000,   /* U+2FA72 */
  0x0000,   /* U+2FA73 */
  0x0000,   /* U+2FA74 */
  0x0000,   /* U+2FA75 */
  0x0000,   /* U+2FA76 */
  0x0000,   /* U+2FA77 */
  0x0000,   /* U+2FA78 */
  0x0000,   /* U+2FA79 */
  0x0000,   /* U+2FA7A */
  0x0000,   /* U+2FA7B */
  0x0000,   /* U+2FA7C */
  0x0000,   /* U+2FA7D */
  0x0000,   /* U+2FA7E */
  0x0000,   /* U+2FA7F */
  0x0000,   /* U+2FA80 */
  0x0000,   /* U+2FA81 */
  0x0000,   /* U+2FA82 */
  0x0000,   /* U+2FA83 */
  0x0000,   /* U+2FA84 */
  0x0000,   /* U+2FA85 */
  0x0000,   /* U+2FA86 */
  0x0000,   /* U+2FA87 */
  0x0000,   /* U+2FA88 */
  0x0000,   /* U+2FA89 */
  0x0000,   /* U+2FA8A */
  0x0000,   /* U+2FA8B */
  0x0000,   /* U+2FA8C */
  0x0000,   /* U+2FA8D */
  0x0000,   /* U+2FA8E */
  0x0000,   /* U+2FA8F */
  0x0000,   /* U+2FA90 */
  0x0000,   /* U+2FA91 */
  0x0000,   /* U+2FA92 */
  0x0000,   /* U+2FA93 */
  0x0000,   /* U+2FA94 */
  0x0000,   /* U+2FA95 */
  0x0000,   /* U+2FA96 */
  0x0000,   /* U+2FA97 */
  0x0000,   /* U+2FA98 */
  0x0000,   /* U+2FA99 */
  0x0000,   /* U+2FA9A */
  0x0000,   /* U+2FA9B */
  0x0000,   /* U+2FA9C */
  0x0000,   /* U+2FA9D */
  0x0000,   /* U+2FA9E */
  0x0000,   /* U+2FA9F */
  0x0000,   /* U+2FAA0 */
  0x0000,   /* U+2FAA1 */
  0x0000,   /* U+2FAA2 */
  0x0000,   /* U+2FAA3 */
  0x0000,   /* U+2FAA4 */
  0x0000,   /* U+2FAA5 */
  0x0000,   /* U+2FAA6 */
  0x0000,   /* U+2FAA7 */
  0x0000,   /* U+2FAA8 */
  0x0000,   /* U+2FAA9 */
  0x0000,   /* U+2FAAA */
  0x0000,   /* U+2FAAB */
  0x0000,   /* U+2FAAC */
  0x0000,   /* U+2FAAD */
  0x0000,   /* U+2FAAE */
  0x0000,   /* U+2FAAF */
  0x0000,   /* U+2FAB0 */
  0x0000,   /* U+2FAB1 */
  0x0000,   /* U+2FAB2 */
  0x0000,   /* U+2FAB3 */
  0x0000,   /* U+2FAB4 */
  0x0000,   /* U+2FAB5 */
  0x0000,   /* U+2FAB6 */
  0x0000,   /* U+2FAB7 */
  0x0000,   /* U+2FAB8 */
  0x0000,   /* U+2FAB9 */
  0x0000,   /* U+2FABA */
  0x0000,   /* U+2FABB */
  0x0000,   /* U+2FABC */
  0x0000,   /* U+2FABD */
  0x0000,   /* U+2FABE */
  0x0000,   /* U+2FABF */
  0x0000,   /* U+2FAC0 */
  0x0000,   /* U+2FAC1 */
  0x0000,   /* U+2FAC2 */
  0x0000,   /* U+2FAC3 */
  0x0000,   /* U+2FAC4 */
  0x0000,   /* U+2FAC5 */
  0x0000,   /* U+2FAC6 */
  0x0000,   /* U+2FAC7 */
  0x0000,   /* U+2FAC8 */
  0x0000,   /* U+2FAC9 */
  0x0000,   /* U+2FACA */
  0x0000,   /* U+2FACB */
  0x0000,   /* U+2FACC */
  0x0000,   /* U+2FACD */
  0x0000,   /* U+2FACE */
  0x0000,   /* U+2FACF */
  0x0000,   /* U+2FAD0 */
  0x0000,   /* U+2FAD1 */
  0x0000,   /* U+2FAD2 */
  0x0000,   /* U+2FAD3 */
  0x0000,   /* U+2FAD4 */
  0x0000,   /* U+2FAD5 */
  0x0000,   /* U+2FAD6 */
  0x0000,   /* U+2FAD7 */
  0x0000,   /* U+2FAD8 */
  0x0000,   /* U+2FAD9 */
  0x0000,   /* U+2FADA */
  0x0000,   /* U+2FADB */
  0x0000,   /* U+2FADC */
  0x0000,   /* U+2FADD */
  0x0000,   /* U+2FADE */
  0x0000,   /* U+2FADF */
  0x0000,   /* U+2FAE0 */
  0x0000,   /* U+2FAE1 */
  0x0000,   /* U+2FAE2 */
  0x0000,   /* U+2FAE3 */
  0x0000,   /* U+2FAE4 */
  0x0000,   /* U+2FAE5 */
  0x0000,   /* U+2FAE6 */
  0x0000,   /* U+2FAE7 */
  0x0000,   /* U+2FAE8 */
  0x0000,   /* U+2FAE9 */
  0x0000,   /* U+2FAEA */
  0x0000,   /* U+2FAEB */
  0x0000,   /* U+2FAEC */
  0x0000,   /* U+2FAED */
  0x0000,   /* U+2FAEE */
  0x0000,   /* U+2FAEF */
  0x0000,   /* U+2FAF0 */
  0x0000,   /* U+2FAF1 */
  0x0000,   /* U+2FAF2 */
  0x0000,   /* U+2FAF3 */
  0x0000,   /* U+2FAF4 */
  0x0000,   /* U+2FAF5 */
  0x0000,   /* U+2FAF6 */
  0x0000,   /* U+2FAF7 */
  0x0000,   /* U+2FAF8 */
  0x0000,   /* U+2FAF9 */
  0x0000,   /* U+2FAFA */
  0x0000,   /* U+2FAFB */
  0x0000,   /* U+2FAFC */
  0x0000,   /* U+2FAFD */
  0x0000,   /* U+2FAFE */
  0x0000,   /* U+2FAFF */
};

uint16 uca900_pE00[]= {
  /* Number of CEs for each character. */
  2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0xFBDC,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0xFBDC,   /* U+E0002 */
  0xFBDC,   /* U+E0003 */
  0xFBDC,   /* U+E0004 */
  0xFBDC,   /* U+E0005 */
  0xFBDC,   /* U+E0006 */
  0xFBDC,   /* U+E0007 */
  0xFBDC,   /* U+E0008 */
  0xFBDC,   /* U+E0009 */
  0xFBDC,   /* U+E000A */
  0xFBDC,   /* U+E000B */
  0xFBDC,   /* U+E000C */
  0xFBDC,   /* U+E000D */
  0xFBDC,   /* U+E000E */
  0xFBDC,   /* U+E000F */
  0xFBDC,   /* U+E0010 */
  0xFBDC,   /* U+E0011 */
  0xFBDC,   /* U+E0012 */
  0xFBDC,   /* U+E0013 */
  0xFBDC,   /* U+E0014 */
  0xFBDC,   /* U+E0015 */
  0xFBDC,   /* U+E0016 */
  0xFBDC,   /* U+E0017 */
  0xFBDC,   /* U+E0018 */
  0xFBDC,   /* U+E0019 */
  0xFBDC,   /* U+E001A */
  0xFBDC,   /* U+E001B */
  0xFBDC,   /* U+E001C */
  0xFBDC,   /* U+E001D */
  0xFBDC,   /* U+E001E */
  0xFBDC,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0xFBDC,   /* U+E0080 */
  0xFBDC,   /* U+E0081 */
  0xFBDC,   /* U+E0082 */
  0xFBDC,   /* U+E0083 */
  0xFBDC,   /* U+E0084 */
  0xFBDC,   /* U+E0085 */
  0xFBDC,   /* U+E0086 */
  0xFBDC,   /* U+E0087 */
  0xFBDC,   /* U+E0088 */
  0xFBDC,   /* U+E0089 */
  0xFBDC,   /* U+E008A */
  0xFBDC,   /* U+E008B */
  0xFBDC,   /* U+E008C */
  0xFBDC,   /* U+E008D */
  0xFBDC,   /* U+E008E */
  0xFBDC,   /* U+E008F */
  0xFBDC,   /* U+E0090 */
  0xFBDC,   /* U+E0091 */
  0xFBDC,   /* U+E0092 */
  0xFBDC,   /* U+E0093 */
  0xFBDC,   /* U+E0094 */
  0xFBDC,   /* U+E0095 */
  0xFBDC,   /* U+E0096 */
  0xFBDC,   /* U+E0097 */
  0xFBDC,   /* U+E0098 */
  0xFBDC,   /* U+E0099 */
  0xFBDC,   /* U+E009A */
  0xFBDC,   /* U+E009B */
  0xFBDC,   /* U+E009C */
  0xFBDC,   /* U+E009D */
  0xFBDC,   /* U+E009E */
  0xFBDC,   /* U+E009F */
  0xFBDC,   /* U+E00A0 */
  0xFBDC,   /* U+E00A1 */
  0xFBDC,   /* U+E00A2 */
  0xFBDC,   /* U+E00A3 */
  0xFBDC,   /* U+E00A4 */
  0xFBDC,   /* U+E00A5 */
  0xFBDC,   /* U+E00A6 */
  0xFBDC,   /* U+E00A7 */
  0xFBDC,   /* U+E00A8 */
  0xFBDC,   /* U+E00A9 */
  0xFBDC,   /* U+E00AA */
  0xFBDC,   /* U+E00AB */
  0xFBDC,   /* U+E00AC */
  0xFBDC,   /* U+E00AD */
  0xFBDC,   /* U+E00AE */
  0xFBDC,   /* U+E00AF */
  0xFBDC,   /* U+E00B0 */
  0xFBDC,   /* U+E00B1 */
  0xFBDC,   /* U+E00B2 */
  0xFBDC,   /* U+E00B3 */
  0xFBDC,   /* U+E00B4 */
  0xFBDC,   /* U+E00B5 */
  0xFBDC,   /* U+E00B6 */
  0xFBDC,   /* U+E00B7 */
  0xFBDC,   /* U+E00B8 */
  0xFBDC,   /* U+E00B9 */
  0xFBDC,   /* U+E00BA */
  0xFBDC,   /* U+E00BB */
  0xFBDC,   /* U+E00BC */
  0xFBDC,   /* U+E00BD */
  0xFBDC,   /* U+E00BE */
  0xFBDC,   /* U+E00BF */
  0xFBDC,   /* U+E00C0 */
  0xFBDC,   /* U+E00C1 */
  0xFBDC,   /* U+E00C2 */
  0xFBDC,   /* U+E00C3 */
  0xFBDC,   /* U+E00C4 */
  0xFBDC,   /* U+E00C5 */
  0xFBDC,   /* U+E00C6 */
  0xFBDC,   /* U+E00C7 */
  0xFBDC,   /* U+E00C8 */
  0xFBDC,   /* U+E00C9 */
  0xFBDC,   /* U+E00CA */
  0xFBDC,   /* U+E00CB */
  0xFBDC,   /* U+E00CC */
  0xFBDC,   /* U+E00CD */
  0xFBDC,   /* U+E00CE */
  0xFBDC,   /* U+E00CF */
  0xFBDC,   /* U+E00D0 */
  0xFBDC,   /* U+E00D1 */
  0xFBDC,   /* U+E00D2 */
  0xFBDC,   /* U+E00D3 */
  0xFBDC,   /* U+E00D4 */
  0xFBDC,   /* U+E00D5 */
  0xFBDC,   /* U+E00D6 */
  0xFBDC,   /* U+E00D7 */
  0xFBDC,   /* U+E00D8 */
  0xFBDC,   /* U+E00D9 */
  0xFBDC,   /* U+E00DA */
  0xFBDC,   /* U+E00DB */
  0xFBDC,   /* U+E00DC */
  0xFBDC,   /* U+E00DD */
  0xFBDC,   /* U+E00DE */
  0xFBDC,   /* U+E00DF */
  0xFBDC,   /* U+E00E0 */
  0xFBDC,   /* U+E00E1 */
  0xFBDC,   /* U+E00E2 */
  0xFBDC,   /* U+E00E3 */
  0xFBDC,   /* U+E00E4 */
  0xFBDC,   /* U+E00E5 */
  0xFBDC,   /* U+E00E6 */
  0xFBDC,   /* U+E00E7 */
  0xFBDC,   /* U+E00E8 */
  0xFBDC,   /* U+E00E9 */
  0xFBDC,   /* U+E00EA */
  0xFBDC,   /* U+E00EB */
  0xFBDC,   /* U+E00EC */
  0xFBDC,   /* U+E00ED */
  0xFBDC,   /* U+E00EE */
  0xFBDC,   /* U+E00EF */
  0xFBDC,   /* U+E00F0 */
  0xFBDC,   /* U+E00F1 */
  0xFBDC,   /* U+E00F2 */
  0xFBDC,   /* U+E00F3 */
  0xFBDC,   /* U+E00F4 */
  0xFBDC,   /* U+E00F5 */
  0xFBDC,   /* U+E00F6 */
  0xFBDC,   /* U+E00F7 */
  0xFBDC,   /* U+E00F8 */
  0xFBDC,   /* U+E00F9 */
  0xFBDC,   /* U+E00FA */
  0xFBDC,   /* U+E00FB */
  0xFBDC,   /* U+E00FC */
  0xFBDC,   /* U+E00FD */
  0xFBDC,   /* U+E00FE */
  0xFBDC,   /* U+E00FF */

  /* Secondary weight 1 for each character. */
  0x0020,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0x0020,   /* U+E0002 */
  0x0020,   /* U+E0003 */
  0x0020,   /* U+E0004 */
  0x0020,   /* U+E0005 */
  0x0020,   /* U+E0006 */
  0x0020,   /* U+E0007 */
  0x0020,   /* U+E0008 */
  0x0020,   /* U+E0009 */
  0x0020,   /* U+E000A */
  0x0020,   /* U+E000B */
  0x0020,   /* U+E000C */
  0x0020,   /* U+E000D */
  0x0020,   /* U+E000E */
  0x0020,   /* U+E000F */
  0x0020,   /* U+E0010 */
  0x0020,   /* U+E0011 */
  0x0020,   /* U+E0012 */
  0x0020,   /* U+E0013 */
  0x0020,   /* U+E0014 */
  0x0020,   /* U+E0015 */
  0x0020,   /* U+E0016 */
  0x0020,   /* U+E0017 */
  0x0020,   /* U+E0018 */
  0x0020,   /* U+E0019 */
  0x0020,   /* U+E001A */
  0x0020,   /* U+E001B */
  0x0020,   /* U+E001C */
  0x0020,   /* U+E001D */
  0x0020,   /* U+E001E */
  0x0020,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0x0020,   /* U+E0080 */
  0x0020,   /* U+E0081 */
  0x0020,   /* U+E0082 */
  0x0020,   /* U+E0083 */
  0x0020,   /* U+E0084 */
  0x0020,   /* U+E0085 */
  0x0020,   /* U+E0086 */
  0x0020,   /* U+E0087 */
  0x0020,   /* U+E0088 */
  0x0020,   /* U+E0089 */
  0x0020,   /* U+E008A */
  0x0020,   /* U+E008B */
  0x0020,   /* U+E008C */
  0x0020,   /* U+E008D */
  0x0020,   /* U+E008E */
  0x0020,   /* U+E008F */
  0x0020,   /* U+E0090 */
  0x0020,   /* U+E0091 */
  0x0020,   /* U+E0092 */
  0x0020,   /* U+E0093 */
  0x0020,   /* U+E0094 */
  0x0020,   /* U+E0095 */
  0x0020,   /* U+E0096 */
  0x0020,   /* U+E0097 */
  0x0020,   /* U+E0098 */
  0x0020,   /* U+E0099 */
  0x0020,   /* U+E009A */
  0x0020,   /* U+E009B */
  0x0020,   /* U+E009C */
  0x0020,   /* U+E009D */
  0x0020,   /* U+E009E */
  0x0020,   /* U+E009F */
  0x0020,   /* U+E00A0 */
  0x0020,   /* U+E00A1 */
  0x0020,   /* U+E00A2 */
  0x0020,   /* U+E00A3 */
  0x0020,   /* U+E00A4 */
  0x0020,   /* U+E00A5 */
  0x0020,   /* U+E00A6 */
  0x0020,   /* U+E00A7 */
  0x0020,   /* U+E00A8 */
  0x0020,   /* U+E00A9 */
  0x0020,   /* U+E00AA */
  0x0020,   /* U+E00AB */
  0x0020,   /* U+E00AC */
  0x0020,   /* U+E00AD */
  0x0020,   /* U+E00AE */
  0x0020,   /* U+E00AF */
  0x0020,   /* U+E00B0 */
  0x0020,   /* U+E00B1 */
  0x0020,   /* U+E00B2 */
  0x0020,   /* U+E00B3 */
  0x0020,   /* U+E00B4 */
  0x0020,   /* U+E00B5 */
  0x0020,   /* U+E00B6 */
  0x0020,   /* U+E00B7 */
  0x0020,   /* U+E00B8 */
  0x0020,   /* U+E00B9 */
  0x0020,   /* U+E00BA */
  0x0020,   /* U+E00BB */
  0x0020,   /* U+E00BC */
  0x0020,   /* U+E00BD */
  0x0020,   /* U+E00BE */
  0x0020,   /* U+E00BF */
  0x0020,   /* U+E00C0 */
  0x0020,   /* U+E00C1 */
  0x0020,   /* U+E00C2 */
  0x0020,   /* U+E00C3 */
  0x0020,   /* U+E00C4 */
  0x0020,   /* U+E00C5 */
  0x0020,   /* U+E00C6 */
  0x0020,   /* U+E00C7 */
  0x0020,   /* U+E00C8 */
  0x0020,   /* U+E00C9 */
  0x0020,   /* U+E00CA */
  0x0020,   /* U+E00CB */
  0x0020,   /* U+E00CC */
  0x0020,   /* U+E00CD */
  0x0020,   /* U+E00CE */
  0x0020,   /* U+E00CF */
  0x0020,   /* U+E00D0 */
  0x0020,   /* U+E00D1 */
  0x0020,   /* U+E00D2 */
  0x0020,   /* U+E00D3 */
  0x0020,   /* U+E00D4 */
  0x0020,   /* U+E00D5 */
  0x0020,   /* U+E00D6 */
  0x0020,   /* U+E00D7 */
  0x0020,   /* U+E00D8 */
  0x0020,   /* U+E00D9 */
  0x0020,   /* U+E00DA */
  0x0020,   /* U+E00DB */
  0x0020,   /* U+E00DC */
  0x0020,   /* U+E00DD */
  0x0020,   /* U+E00DE */
  0x0020,   /* U+E00DF */
  0x0020,   /* U+E00E0 */
  0x0020,   /* U+E00E1 */
  0x0020,   /* U+E00E2 */
  0x0020,   /* U+E00E3 */
  0x0020,   /* U+E00E4 */
  0x0020,   /* U+E00E5 */
  0x0020,   /* U+E00E6 */
  0x0020,   /* U+E00E7 */
  0x0020,   /* U+E00E8 */
  0x0020,   /* U+E00E9 */
  0x0020,   /* U+E00EA */
  0x0020,   /* U+E00EB */
  0x0020,   /* U+E00EC */
  0x0020,   /* U+E00ED */
  0x0020,   /* U+E00EE */
  0x0020,   /* U+E00EF */
  0x0020,   /* U+E00F0 */
  0x0020,   /* U+E00F1 */
  0x0020,   /* U+E00F2 */
  0x0020,   /* U+E00F3 */
  0x0020,   /* U+E00F4 */
  0x0020,   /* U+E00F5 */
  0x0020,   /* U+E00F6 */
  0x0020,   /* U+E00F7 */
  0x0020,   /* U+E00F8 */
  0x0020,   /* U+E00F9 */
  0x0020,   /* U+E00FA */
  0x0020,   /* U+E00FB */
  0x0020,   /* U+E00FC */
  0x0020,   /* U+E00FD */
  0x0020,   /* U+E00FE */
  0x0020,   /* U+E00FF */

  /* Tertiary weight 1 for each character. */
  0x0002,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0x0002,   /* U+E0002 */
  0x0002,   /* U+E0003 */
  0x0002,   /* U+E0004 */
  0x0002,   /* U+E0005 */
  0x0002,   /* U+E0006 */
  0x0002,   /* U+E0007 */
  0x0002,   /* U+E0008 */
  0x0002,   /* U+E0009 */
  0x0002,   /* U+E000A */
  0x0002,   /* U+E000B */
  0x0002,   /* U+E000C */
  0x0002,   /* U+E000D */
  0x0002,   /* U+E000E */
  0x0002,   /* U+E000F */
  0x0002,   /* U+E0010 */
  0x0002,   /* U+E0011 */
  0x0002,   /* U+E0012 */
  0x0002,   /* U+E0013 */
  0x0002,   /* U+E0014 */
  0x0002,   /* U+E0015 */
  0x0002,   /* U+E0016 */
  0x0002,   /* U+E0017 */
  0x0002,   /* U+E0018 */
  0x0002,   /* U+E0019 */
  0x0002,   /* U+E001A */
  0x0002,   /* U+E001B */
  0x0002,   /* U+E001C */
  0x0002,   /* U+E001D */
  0x0002,   /* U+E001E */
  0x0002,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0x0002,   /* U+E0080 */
  0x0002,   /* U+E0081 */
  0x0002,   /* U+E0082 */
  0x0002,   /* U+E0083 */
  0x0002,   /* U+E0084 */
  0x0002,   /* U+E0085 */
  0x0002,   /* U+E0086 */
  0x0002,   /* U+E0087 */
  0x0002,   /* U+E0088 */
  0x0002,   /* U+E0089 */
  0x0002,   /* U+E008A */
  0x0002,   /* U+E008B */
  0x0002,   /* U+E008C */
  0x0002,   /* U+E008D */
  0x0002,   /* U+E008E */
  0x0002,   /* U+E008F */
  0x0002,   /* U+E0090 */
  0x0002,   /* U+E0091 */
  0x0002,   /* U+E0092 */
  0x0002,   /* U+E0093 */
  0x0002,   /* U+E0094 */
  0x0002,   /* U+E0095 */
  0x0002,   /* U+E0096 */
  0x0002,   /* U+E0097 */
  0x0002,   /* U+E0098 */
  0x0002,   /* U+E0099 */
  0x0002,   /* U+E009A */
  0x0002,   /* U+E009B */
  0x0002,   /* U+E009C */
  0x0002,   /* U+E009D */
  0x0002,   /* U+E009E */
  0x0002,   /* U+E009F */
  0x0002,   /* U+E00A0 */
  0x0002,   /* U+E00A1 */
  0x0002,   /* U+E00A2 */
  0x0002,   /* U+E00A3 */
  0x0002,   /* U+E00A4 */
  0x0002,   /* U+E00A5 */
  0x0002,   /* U+E00A6 */
  0x0002,   /* U+E00A7 */
  0x0002,   /* U+E00A8 */
  0x0002,   /* U+E00A9 */
  0x0002,   /* U+E00AA */
  0x0002,   /* U+E00AB */
  0x0002,   /* U+E00AC */
  0x0002,   /* U+E00AD */
  0x0002,   /* U+E00AE */
  0x0002,   /* U+E00AF */
  0x0002,   /* U+E00B0 */
  0x0002,   /* U+E00B1 */
  0x0002,   /* U+E00B2 */
  0x0002,   /* U+E00B3 */
  0x0002,   /* U+E00B4 */
  0x0002,   /* U+E00B5 */
  0x0002,   /* U+E00B6 */
  0x0002,   /* U+E00B7 */
  0x0002,   /* U+E00B8 */
  0x0002,   /* U+E00B9 */
  0x0002,   /* U+E00BA */
  0x0002,   /* U+E00BB */
  0x0002,   /* U+E00BC */
  0x0002,   /* U+E00BD */
  0x0002,   /* U+E00BE */
  0x0002,   /* U+E00BF */
  0x0002,   /* U+E00C0 */
  0x0002,   /* U+E00C1 */
  0x0002,   /* U+E00C2 */
  0x0002,   /* U+E00C3 */
  0x0002,   /* U+E00C4 */
  0x0002,   /* U+E00C5 */
  0x0002,   /* U+E00C6 */
  0x0002,   /* U+E00C7 */
  0x0002,   /* U+E00C8 */
  0x0002,   /* U+E00C9 */
  0x0002,   /* U+E00CA */
  0x0002,   /* U+E00CB */
  0x0002,   /* U+E00CC */
  0x0002,   /* U+E00CD */
  0x0002,   /* U+E00CE */
  0x0002,   /* U+E00CF */
  0x0002,   /* U+E00D0 */
  0x0002,   /* U+E00D1 */
  0x0002,   /* U+E00D2 */
  0x0002,   /* U+E00D3 */
  0x0002,   /* U+E00D4 */
  0x0002,   /* U+E00D5 */
  0x0002,   /* U+E00D6 */
  0x0002,   /* U+E00D7 */
  0x0002,   /* U+E00D8 */
  0x0002,   /* U+E00D9 */
  0x0002,   /* U+E00DA */
  0x0002,   /* U+E00DB */
  0x0002,   /* U+E00DC */
  0x0002,   /* U+E00DD */
  0x0002,   /* U+E00DE */
  0x0002,   /* U+E00DF */
  0x0002,   /* U+E00E0 */
  0x0002,   /* U+E00E1 */
  0x0002,   /* U+E00E2 */
  0x0002,   /* U+E00E3 */
  0x0002,   /* U+E00E4 */
  0x0002,   /* U+E00E5 */
  0x0002,   /* U+E00E6 */
  0x0002,   /* U+E00E7 */
  0x0002,   /* U+E00E8 */
  0x0002,   /* U+E00E9 */
  0x0002,   /* U+E00EA */
  0x0002,   /* U+E00EB */
  0x0002,   /* U+E00EC */
  0x0002,   /* U+E00ED */
  0x0002,   /* U+E00EE */
  0x0002,   /* U+E00EF */
  0x0002,   /* U+E00F0 */
  0x0002,   /* U+E00F1 */
  0x0002,   /* U+E00F2 */
  0x0002,   /* U+E00F3 */
  0x0002,   /* U+E00F4 */
  0x0002,   /* U+E00F5 */
  0x0002,   /* U+E00F6 */
  0x0002,   /* U+E00F7 */
  0x0002,   /* U+E00F8 */
  0x0002,   /* U+E00F9 */
  0x0002,   /* U+E00FA */
  0x0002,   /* U+E00FB */
  0x0002,   /* U+E00FC */
  0x0002,   /* U+E00FD */
  0x0002,   /* U+E00FE */
  0x0002,   /* U+E00FF */

  /* Primary weight 2 for each character. */
  0x8000,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0x8002,   /* U+E0002 */
  0x8003,   /* U+E0003 */
  0x8004,   /* U+E0004 */
  0x8005,   /* U+E0005 */
  0x8006,   /* U+E0006 */
  0x8007,   /* U+E0007 */
  0x8008,   /* U+E0008 */
  0x8009,   /* U+E0009 */
  0x800A,   /* U+E000A */
  0x800B,   /* U+E000B */
  0x800C,   /* U+E000C */
  0x800D,   /* U+E000D */
  0x800E,   /* U+E000E */
  0x800F,   /* U+E000F */
  0x8010,   /* U+E0010 */
  0x8011,   /* U+E0011 */
  0x8012,   /* U+E0012 */
  0x8013,   /* U+E0013 */
  0x8014,   /* U+E0014 */
  0x8015,   /* U+E0015 */
  0x8016,   /* U+E0016 */
  0x8017,   /* U+E0017 */
  0x8018,   /* U+E0018 */
  0x8019,   /* U+E0019 */
  0x801A,   /* U+E001A */
  0x801B,   /* U+E001B */
  0x801C,   /* U+E001C */
  0x801D,   /* U+E001D */
  0x801E,   /* U+E001E */
  0x801F,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0x8080,   /* U+E0080 */
  0x8081,   /* U+E0081 */
  0x8082,   /* U+E0082 */
  0x8083,   /* U+E0083 */
  0x8084,   /* U+E0084 */
  0x8085,   /* U+E0085 */
  0x8086,   /* U+E0086 */
  0x8087,   /* U+E0087 */
  0x8088,   /* U+E0088 */
  0x8089,   /* U+E0089 */
  0x808A,   /* U+E008A */
  0x808B,   /* U+E008B */
  0x808C,   /* U+E008C */
  0x808D,   /* U+E008D */
  0x808E,   /* U+E008E */
  0x808F,   /* U+E008F */
  0x8090,   /* U+E0090 */
  0x8091,   /* U+E0091 */
  0x8092,   /* U+E0092 */
  0x8093,   /* U+E0093 */
  0x8094,   /* U+E0094 */
  0x8095,   /* U+E0095 */
  0x8096,   /* U+E0096 */
  0x8097,   /* U+E0097 */
  0x8098,   /* U+E0098 */
  0x8099,   /* U+E0099 */
  0x809A,   /* U+E009A */
  0x809B,   /* U+E009B */
  0x809C,   /* U+E009C */
  0x809D,   /* U+E009D */
  0x809E,   /* U+E009E */
  0x809F,   /* U+E009F */
  0x80A0,   /* U+E00A0 */
  0x80A1,   /* U+E00A1 */
  0x80A2,   /* U+E00A2 */
  0x80A3,   /* U+E00A3 */
  0x80A4,   /* U+E00A4 */
  0x80A5,   /* U+E00A5 */
  0x80A6,   /* U+E00A6 */
  0x80A7,   /* U+E00A7 */
  0x80A8,   /* U+E00A8 */
  0x80A9,   /* U+E00A9 */
  0x80AA,   /* U+E00AA */
  0x80AB,   /* U+E00AB */
  0x80AC,   /* U+E00AC */
  0x80AD,   /* U+E00AD */
  0x80AE,   /* U+E00AE */
  0x80AF,   /* U+E00AF */
  0x80B0,   /* U+E00B0 */
  0x80B1,   /* U+E00B1 */
  0x80B2,   /* U+E00B2 */
  0x80B3,   /* U+E00B3 */
  0x80B4,   /* U+E00B4 */
  0x80B5,   /* U+E00B5 */
  0x80B6,   /* U+E00B6 */
  0x80B7,   /* U+E00B7 */
  0x80B8,   /* U+E00B8 */
  0x80B9,   /* U+E00B9 */
  0x80BA,   /* U+E00BA */
  0x80BB,   /* U+E00BB */
  0x80BC,   /* U+E00BC */
  0x80BD,   /* U+E00BD */
  0x80BE,   /* U+E00BE */
  0x80BF,   /* U+E00BF */
  0x80C0,   /* U+E00C0 */
  0x80C1,   /* U+E00C1 */
  0x80C2,   /* U+E00C2 */
  0x80C3,   /* U+E00C3 */
  0x80C4,   /* U+E00C4 */
  0x80C5,   /* U+E00C5 */
  0x80C6,   /* U+E00C6 */
  0x80C7,   /* U+E00C7 */
  0x80C8,   /* U+E00C8 */
  0x80C9,   /* U+E00C9 */
  0x80CA,   /* U+E00CA */
  0x80CB,   /* U+E00CB */
  0x80CC,   /* U+E00CC */
  0x80CD,   /* U+E00CD */
  0x80CE,   /* U+E00CE */
  0x80CF,   /* U+E00CF */
  0x80D0,   /* U+E00D0 */
  0x80D1,   /* U+E00D1 */
  0x80D2,   /* U+E00D2 */
  0x80D3,   /* U+E00D3 */
  0x80D4,   /* U+E00D4 */
  0x80D5,   /* U+E00D5 */
  0x80D6,   /* U+E00D6 */
  0x80D7,   /* U+E00D7 */
  0x80D8,   /* U+E00D8 */
  0x80D9,   /* U+E00D9 */
  0x80DA,   /* U+E00DA */
  0x80DB,   /* U+E00DB */
  0x80DC,   /* U+E00DC */
  0x80DD,   /* U+E00DD */
  0x80DE,   /* U+E00DE */
  0x80DF,   /* U+E00DF */
  0x80E0,   /* U+E00E0 */
  0x80E1,   /* U+E00E1 */
  0x80E2,   /* U+E00E2 */
  0x80E3,   /* U+E00E3 */
  0x80E4,   /* U+E00E4 */
  0x80E5,   /* U+E00E5 */
  0x80E6,   /* U+E00E6 */
  0x80E7,   /* U+E00E7 */
  0x80E8,   /* U+E00E8 */
  0x80E9,   /* U+E00E9 */
  0x80EA,   /* U+E00EA */
  0x80EB,   /* U+E00EB */
  0x80EC,   /* U+E00EC */
  0x80ED,   /* U+E00ED */
  0x80EE,   /* U+E00EE */
  0x80EF,   /* U+E00EF */
  0x80F0,   /* U+E00F0 */
  0x80F1,   /* U+E00F1 */
  0x80F2,   /* U+E00F2 */
  0x80F3,   /* U+E00F3 */
  0x80F4,   /* U+E00F4 */
  0x80F5,   /* U+E00F5 */
  0x80F6,   /* U+E00F6 */
  0x80F7,   /* U+E00F7 */
  0x80F8,   /* U+E00F8 */
  0x80F9,   /* U+E00F9 */
  0x80FA,   /* U+E00FA */
  0x80FB,   /* U+E00FB */
  0x80FC,   /* U+E00FC */
  0x80FD,   /* U+E00FD */
  0x80FE,   /* U+E00FE */
  0x80FF,   /* U+E00FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0x0000,   /* U+E0002 */
  0x0000,   /* U+E0003 */
  0x0000,   /* U+E0004 */
  0x0000,   /* U+E0005 */
  0x0000,   /* U+E0006 */
  0x0000,   /* U+E0007 */
  0x0000,   /* U+E0008 */
  0x0000,   /* U+E0009 */
  0x0000,   /* U+E000A */
  0x0000,   /* U+E000B */
  0x0000,   /* U+E000C */
  0x0000,   /* U+E000D */
  0x0000,   /* U+E000E */
  0x0000,   /* U+E000F */
  0x0000,   /* U+E0010 */
  0x0000,   /* U+E0011 */
  0x0000,   /* U+E0012 */
  0x0000,   /* U+E0013 */
  0x0000,   /* U+E0014 */
  0x0000,   /* U+E0015 */
  0x0000,   /* U+E0016 */
  0x0000,   /* U+E0017 */
  0x0000,   /* U+E0018 */
  0x0000,   /* U+E0019 */
  0x0000,   /* U+E001A */
  0x0000,   /* U+E001B */
  0x0000,   /* U+E001C */
  0x0000,   /* U+E001D */
  0x0000,   /* U+E001E */
  0x0000,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0x0000,   /* U+E0080 */
  0x0000,   /* U+E0081 */
  0x0000,   /* U+E0082 */
  0x0000,   /* U+E0083 */
  0x0000,   /* U+E0084 */
  0x0000,   /* U+E0085 */
  0x0000,   /* U+E0086 */
  0x0000,   /* U+E0087 */
  0x0000,   /* U+E0088 */
  0x0000,   /* U+E0089 */
  0x0000,   /* U+E008A */
  0x0000,   /* U+E008B */
  0x0000,   /* U+E008C */
  0x0000,   /* U+E008D */
  0x0000,   /* U+E008E */
  0x0000,   /* U+E008F */
  0x0000,   /* U+E0090 */
  0x0000,   /* U+E0091 */
  0x0000,   /* U+E0092 */
  0x0000,   /* U+E0093 */
  0x0000,   /* U+E0094 */
  0x0000,   /* U+E0095 */
  0x0000,   /* U+E0096 */
  0x0000,   /* U+E0097 */
  0x0000,   /* U+E0098 */
  0x0000,   /* U+E0099 */
  0x0000,   /* U+E009A */
  0x0000,   /* U+E009B */
  0x0000,   /* U+E009C */
  0x0000,   /* U+E009D */
  0x0000,   /* U+E009E */
  0x0000,   /* U+E009F */
  0x0000,   /* U+E00A0 */
  0x0000,   /* U+E00A1 */
  0x0000,   /* U+E00A2 */
  0x0000,   /* U+E00A3 */
  0x0000,   /* U+E00A4 */
  0x0000,   /* U+E00A5 */
  0x0000,   /* U+E00A6 */
  0x0000,   /* U+E00A7 */
  0x0000,   /* U+E00A8 */
  0x0000,   /* U+E00A9 */
  0x0000,   /* U+E00AA */
  0x0000,   /* U+E00AB */
  0x0000,   /* U+E00AC */
  0x0000,   /* U+E00AD */
  0x0000,   /* U+E00AE */
  0x0000,   /* U+E00AF */
  0x0000,   /* U+E00B0 */
  0x0000,   /* U+E00B1 */
  0x0000,   /* U+E00B2 */
  0x0000,   /* U+E00B3 */
  0x0000,   /* U+E00B4 */
  0x0000,   /* U+E00B5 */
  0x0000,   /* U+E00B6 */
  0x0000,   /* U+E00B7 */
  0x0000,   /* U+E00B8 */
  0x0000,   /* U+E00B9 */
  0x0000,   /* U+E00BA */
  0x0000,   /* U+E00BB */
  0x0000,   /* U+E00BC */
  0x0000,   /* U+E00BD */
  0x0000,   /* U+E00BE */
  0x0000,   /* U+E00BF */
  0x0000,   /* U+E00C0 */
  0x0000,   /* U+E00C1 */
  0x0000,   /* U+E00C2 */
  0x0000,   /* U+E00C3 */
  0x0000,   /* U+E00C4 */
  0x0000,   /* U+E00C5 */
  0x0000,   /* U+E00C6 */
  0x0000,   /* U+E00C7 */
  0x0000,   /* U+E00C8 */
  0x0000,   /* U+E00C9 */
  0x0000,   /* U+E00CA */
  0x0000,   /* U+E00CB */
  0x0000,   /* U+E00CC */
  0x0000,   /* U+E00CD */
  0x0000,   /* U+E00CE */
  0x0000,   /* U+E00CF */
  0x0000,   /* U+E00D0 */
  0x0000,   /* U+E00D1 */
  0x0000,   /* U+E00D2 */
  0x0000,   /* U+E00D3 */
  0x0000,   /* U+E00D4 */
  0x0000,   /* U+E00D5 */
  0x0000,   /* U+E00D6 */
  0x0000,   /* U+E00D7 */
  0x0000,   /* U+E00D8 */
  0x0000,   /* U+E00D9 */
  0x0000,   /* U+E00DA */
  0x0000,   /* U+E00DB */
  0x0000,   /* U+E00DC */
  0x0000,   /* U+E00DD */
  0x0000,   /* U+E00DE */
  0x0000,   /* U+E00DF */
  0x0000,   /* U+E00E0 */
  0x0000,   /* U+E00E1 */
  0x0000,   /* U+E00E2 */
  0x0000,   /* U+E00E3 */
  0x0000,   /* U+E00E4 */
  0x0000,   /* U+E00E5 */
  0x0000,   /* U+E00E6 */
  0x0000,   /* U+E00E7 */
  0x0000,   /* U+E00E8 */
  0x0000,   /* U+E00E9 */
  0x0000,   /* U+E00EA */
  0x0000,   /* U+E00EB */
  0x0000,   /* U+E00EC */
  0x0000,   /* U+E00ED */
  0x0000,   /* U+E00EE */
  0x0000,   /* U+E00EF */
  0x0000,   /* U+E00F0 */
  0x0000,   /* U+E00F1 */
  0x0000,   /* U+E00F2 */
  0x0000,   /* U+E00F3 */
  0x0000,   /* U+E00F4 */
  0x0000,   /* U+E00F5 */
  0x0000,   /* U+E00F6 */
  0x0000,   /* U+E00F7 */
  0x0000,   /* U+E00F8 */
  0x0000,   /* U+E00F9 */
  0x0000,   /* U+E00FA */
  0x0000,   /* U+E00FB */
  0x0000,   /* U+E00FC */
  0x0000,   /* U+E00FD */
  0x0000,   /* U+E00FE */
  0x0000,   /* U+E00FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+E0000 */
  0x0000,   /* U+E0001 */
  0x0000,   /* U+E0002 */
  0x0000,   /* U+E0003 */
  0x0000,   /* U+E0004 */
  0x0000,   /* U+E0005 */
  0x0000,   /* U+E0006 */
  0x0000,   /* U+E0007 */
  0x0000,   /* U+E0008 */
  0x0000,   /* U+E0009 */
  0x0000,   /* U+E000A */
  0x0000,   /* U+E000B */
  0x0000,   /* U+E000C */
  0x0000,   /* U+E000D */
  0x0000,   /* U+E000E */
  0x0000,   /* U+E000F */
  0x0000,   /* U+E0010 */
  0x0000,   /* U+E0011 */
  0x0000,   /* U+E0012 */
  0x0000,   /* U+E0013 */
  0x0000,   /* U+E0014 */
  0x0000,   /* U+E0015 */
  0x0000,   /* U+E0016 */
  0x0000,   /* U+E0017 */
  0x0000,   /* U+E0018 */
  0x0000,   /* U+E0019 */
  0x0000,   /* U+E001A */
  0x0000,   /* U+E001B */
  0x0000,   /* U+E001C */
  0x0000,   /* U+E001D */
  0x0000,   /* U+E001E */
  0x0000,   /* U+E001F */
  0x0000,   /* U+E0020 */
  0x0000,   /* U+E0021 */
  0x0000,   /* U+E0022 */
  0x0000,   /* U+E0023 */
  0x0000,   /* U+E0024 */
  0x0000,   /* U+E0025 */
  0x0000,   /* U+E0026 */
  0x0000,   /* U+E0027 */
  0x0000,   /* U+E0028 */
  0x0000,   /* U+E0029 */
  0x0000,   /* U+E002A */
  0x0000,   /* U+E002B */
  0x0000,   /* U+E002C */
  0x0000,   /* U+E002D */
  0x0000,   /* U+E002E */
  0x0000,   /* U+E002F */
  0x0000,   /* U+E0030 */
  0x0000,   /* U+E0031 */
  0x0000,   /* U+E0032 */
  0x0000,   /* U+E0033 */
  0x0000,   /* U+E0034 */
  0x0000,   /* U+E0035 */
  0x0000,   /* U+E0036 */
  0x0000,   /* U+E0037 */
  0x0000,   /* U+E0038 */
  0x0000,   /* U+E0039 */
  0x0000,   /* U+E003A */
  0x0000,   /* U+E003B */
  0x0000,   /* U+E003C */
  0x0000,   /* U+E003D */
  0x0000,   /* U+E003E */
  0x0000,   /* U+E003F */
  0x0000,   /* U+E0040 */
  0x0000,   /* U+E0041 */
  0x0000,   /* U+E0042 */
  0x0000,   /* U+E0043 */
  0x0000,   /* U+E0044 */
  0x0000,   /* U+E0045 */
  0x0000,   /* U+E0046 */
  0x0000,   /* U+E0047 */
  0x0000,   /* U+E0048 */
  0x0000,   /* U+E0049 */
  0x0000,   /* U+E004A */
  0x0000,   /* U+E004B */
  0x0000,   /* U+E004C */
  0x0000,   /* U+E004D */
  0x0000,   /* U+E004E */
  0x0000,   /* U+E004F */
  0x0000,   /* U+E0050 */
  0x0000,   /* U+E0051 */
  0x0000,   /* U+E0052 */
  0x0000,   /* U+E0053 */
  0x0000,   /* U+E0054 */
  0x0000,   /* U+E0055 */
  0x0000,   /* U+E0056 */
  0x0000,   /* U+E0057 */
  0x0000,   /* U+E0058 */
  0x0000,   /* U+E0059 */
  0x0000,   /* U+E005A */
  0x0000,   /* U+E005B */
  0x0000,   /* U+E005C */
  0x0000,   /* U+E005D */
  0x0000,   /* U+E005E */
  0x0000,   /* U+E005F */
  0x0000,   /* U+E0060 */
  0x0000,   /* U+E0061 */
  0x0000,   /* U+E0062 */
  0x0000,   /* U+E0063 */
  0x0000,   /* U+E0064 */
  0x0000,   /* U+E0065 */
  0x0000,   /* U+E0066 */
  0x0000,   /* U+E0067 */
  0x0000,   /* U+E0068 */
  0x0000,   /* U+E0069 */
  0x0000,   /* U+E006A */
  0x0000,   /* U+E006B */
  0x0000,   /* U+E006C */
  0x0000,   /* U+E006D */
  0x0000,   /* U+E006E */
  0x0000,   /* U+E006F */
  0x0000,   /* U+E0070 */
  0x0000,   /* U+E0071 */
  0x0000,   /* U+E0072 */
  0x0000,   /* U+E0073 */
  0x0000,   /* U+E0074 */
  0x0000,   /* U+E0075 */
  0x0000,   /* U+E0076 */
  0x0000,   /* U+E0077 */
  0x0000,   /* U+E0078 */
  0x0000,   /* U+E0079 */
  0x0000,   /* U+E007A */
  0x0000,   /* U+E007B */
  0x0000,   /* U+E007C */
  0x0000,   /* U+E007D */
  0x0000,   /* U+E007E */
  0x0000,   /* U+E007F */
  0x0000,   /* U+E0080 */
  0x0000,   /* U+E0081 */
  0x0000,   /* U+E0082 */
  0x0000,   /* U+E0083 */
  0x0000,   /* U+E0084 */
  0x0000,   /* U+E0085 */
  0x0000,   /* U+E0086 */
  0x0000,   /* U+E0087 */
  0x0000,   /* U+E0088 */
  0x0000,   /* U+E0089 */
  0x0000,   /* U+E008A */
  0x0000,   /* U+E008B */
  0x0000,   /* U+E008C */
  0x0000,   /* U+E008D */
  0x0000,   /* U+E008E */
  0x0000,   /* U+E008F */
  0x0000,   /* U+E0090 */
  0x0000,   /* U+E0091 */
  0x0000,   /* U+E0092 */
  0x0000,   /* U+E0093 */
  0x0000,   /* U+E0094 */
  0x0000,   /* U+E0095 */
  0x0000,   /* U+E0096 */
  0x0000,   /* U+E0097 */
  0x0000,   /* U+E0098 */
  0x0000,   /* U+E0099 */
  0x0000,   /* U+E009A */
  0x0000,   /* U+E009B */
  0x0000,   /* U+E009C */
  0x0000,   /* U+E009D */
  0x0000,   /* U+E009E */
  0x0000,   /* U+E009F */
  0x0000,   /* U+E00A0 */
  0x0000,   /* U+E00A1 */
  0x0000,   /* U+E00A2 */
  0x0000,   /* U+E00A3 */
  0x0000,   /* U+E00A4 */
  0x0000,   /* U+E00A5 */
  0x0000,   /* U+E00A6 */
  0x0000,   /* U+E00A7 */
  0x0000,   /* U+E00A8 */
  0x0000,   /* U+E00A9 */
  0x0000,   /* U+E00AA */
  0x0000,   /* U+E00AB */
  0x0000,   /* U+E00AC */
  0x0000,   /* U+E00AD */
  0x0000,   /* U+E00AE */
  0x0000,   /* U+E00AF */
  0x0000,   /* U+E00B0 */
  0x0000,   /* U+E00B1 */
  0x0000,   /* U+E00B2 */
  0x0000,   /* U+E00B3 */
  0x0000,   /* U+E00B4 */
  0x0000,   /* U+E00B5 */
  0x0000,   /* U+E00B6 */
  0x0000,   /* U+E00B7 */
  0x0000,   /* U+E00B8 */
  0x0000,   /* U+E00B9 */
  0x0000,   /* U+E00BA */
  0x0000,   /* U+E00BB */
  0x0000,   /* U+E00BC */
  0x0000,   /* U+E00BD */
  0x0000,   /* U+E00BE */
  0x0000,   /* U+E00BF */
  0x0000,   /* U+E00C0 */
  0x0000,   /* U+E00C1 */
  0x0000,   /* U+E00C2 */
  0x0000,   /* U+E00C3 */
  0x0000,   /* U+E00C4 */
  0x0000,   /* U+E00C5 */
  0x0000,   /* U+E00C6 */
  0x0000,   /* U+E00C7 */
  0x0000,   /* U+E00C8 */
  0x0000,   /* U+E00C9 */
  0x0000,   /* U+E00CA */
  0x0000,   /* U+E00CB */
  0x0000,   /* U+E00CC */
  0x0000,   /* U+E00CD */
  0x0000,   /* U+E00CE */
  0x0000,   /* U+E00CF */
  0x0000,   /* U+E00D0 */
  0x0000,   /* U+E00D1 */
  0x0000,   /* U+E00D2 */
  0x0000,   /* U+E00D3 */
  0x0000,   /* U+E00D4 */
  0x0000,   /* U+E00D5 */
  0x0000,   /* U+E00D6 */
  0x0000,   /* U+E00D7 */
  0x0000,   /* U+E00D8 */
  0x0000,   /* U+E00D9 */
  0x0000,   /* U+E00DA */
  0x0000,   /* U+E00DB */
  0x0000,   /* U+E00DC */
  0x0000,   /* U+E00DD */
  0x0000,   /* U+E00DE */
  0x0000,   /* U+E00DF */
  0x0000,   /* U+E00E0 */
  0x0000,   /* U+E00E1 */
  0x0000,   /* U+E00E2 */
  0x0000,   /* U+E00E3 */
  0x0000,   /* U+E00E4 */
  0x0000,   /* U+E00E5 */
  0x0000,   /* U+E00E6 */
  0x0000,   /* U+E00E7 */
  0x0000,   /* U+E00E8 */
  0x0000,   /* U+E00E9 */
  0x0000,   /* U+E00EA */
  0x0000,   /* U+E00EB */
  0x0000,   /* U+E00EC */
  0x0000,   /* U+E00ED */
  0x0000,   /* U+E00EE */
  0x0000,   /* U+E00EF */
  0x0000,   /* U+E00F0 */
  0x0000,   /* U+E00F1 */
  0x0000,   /* U+E00F2 */
  0x0000,   /* U+E00F3 */
  0x0000,   /* U+E00F4 */
  0x0000,   /* U+E00F5 */
  0x0000,   /* U+E00F6 */
  0x0000,   /* U+E00F7 */
  0x0000,   /* U+E00F8 */
  0x0000,   /* U+E00F9 */
  0x0000,   /* U+E00FA */
  0x0000,   /* U+E00FB */
  0x0000,   /* U+E00FC */
  0x0000,   /* U+E00FD */
  0x0000,   /* U+E00FE */
  0x0000,   /* U+E00FF */
};

uint16 uca900_pE01[]= {
  /* Number of CEs for each character. */
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 

  /* Primary weight 1 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0xFBDC,   /* U+E01F0 */
  0xFBDC,   /* U+E01F1 */
  0xFBDC,   /* U+E01F2 */
  0xFBDC,   /* U+E01F3 */
  0xFBDC,   /* U+E01F4 */
  0xFBDC,   /* U+E01F5 */
  0xFBDC,   /* U+E01F6 */
  0xFBDC,   /* U+E01F7 */
  0xFBDC,   /* U+E01F8 */
  0xFBDC,   /* U+E01F9 */
  0xFBDC,   /* U+E01FA */
  0xFBDC,   /* U+E01FB */
  0xFBDC,   /* U+E01FC */
  0xFBDC,   /* U+E01FD */
  0xFBDC,   /* U+E01FE */
  0xFBDC,   /* U+E01FF */

  /* Secondary weight 1 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0x0020,   /* U+E01F0 */
  0x0020,   /* U+E01F1 */
  0x0020,   /* U+E01F2 */
  0x0020,   /* U+E01F3 */
  0x0020,   /* U+E01F4 */
  0x0020,   /* U+E01F5 */
  0x0020,   /* U+E01F6 */
  0x0020,   /* U+E01F7 */
  0x0020,   /* U+E01F8 */
  0x0020,   /* U+E01F9 */
  0x0020,   /* U+E01FA */
  0x0020,   /* U+E01FB */
  0x0020,   /* U+E01FC */
  0x0020,   /* U+E01FD */
  0x0020,   /* U+E01FE */
  0x0020,   /* U+E01FF */

  /* Tertiary weight 1 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0x0002,   /* U+E01F0 */
  0x0002,   /* U+E01F1 */
  0x0002,   /* U+E01F2 */
  0x0002,   /* U+E01F3 */
  0x0002,   /* U+E01F4 */
  0x0002,   /* U+E01F5 */
  0x0002,   /* U+E01F6 */
  0x0002,   /* U+E01F7 */
  0x0002,   /* U+E01F8 */
  0x0002,   /* U+E01F9 */
  0x0002,   /* U+E01FA */
  0x0002,   /* U+E01FB */
  0x0002,   /* U+E01FC */
  0x0002,   /* U+E01FD */
  0x0002,   /* U+E01FE */
  0x0002,   /* U+E01FF */

  /* Primary weight 2 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0x81F0,   /* U+E01F0 */
  0x81F1,   /* U+E01F1 */
  0x81F2,   /* U+E01F2 */
  0x81F3,   /* U+E01F3 */
  0x81F4,   /* U+E01F4 */
  0x81F5,   /* U+E01F5 */
  0x81F6,   /* U+E01F6 */
  0x81F7,   /* U+E01F7 */
  0x81F8,   /* U+E01F8 */
  0x81F9,   /* U+E01F9 */
  0x81FA,   /* U+E01FA */
  0x81FB,   /* U+E01FB */
  0x81FC,   /* U+E01FC */
  0x81FD,   /* U+E01FD */
  0x81FE,   /* U+E01FE */
  0x81FF,   /* U+E01FF */

  /* Secondary weight 2 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0x0000,   /* U+E01F0 */
  0x0000,   /* U+E01F1 */
  0x0000,   /* U+E01F2 */
  0x0000,   /* U+E01F3 */
  0x0000,   /* U+E01F4 */
  0x0000,   /* U+E01F5 */
  0x0000,   /* U+E01F6 */
  0x0000,   /* U+E01F7 */
  0x0000,   /* U+E01F8 */
  0x0000,   /* U+E01F9 */
  0x0000,   /* U+E01FA */
  0x0000,   /* U+E01FB */
  0x0000,   /* U+E01FC */
  0x0000,   /* U+E01FD */
  0x0000,   /* U+E01FE */
  0x0000,   /* U+E01FF */

  /* Tertiary weight 2 for each character. */
  0x0000,   /* U+E0100 */
  0x0000,   /* U+E0101 */
  0x0000,   /* U+E0102 */
  0x0000,   /* U+E0103 */
  0x0000,   /* U+E0104 */
  0x0000,   /* U+E0105 */
  0x0000,   /* U+E0106 */
  0x0000,   /* U+E0107 */
  0x0000,   /* U+E0108 */
  0x0000,   /* U+E0109 */
  0x0000,   /* U+E010A */
  0x0000,   /* U+E010B */
  0x0000,   /* U+E010C */
  0x0000,   /* U+E010D */
  0x0000,   /* U+E010E */
  0x0000,   /* U+E010F */
  0x0000,   /* U+E0110 */
  0x0000,   /* U+E0111 */
  0x0000,   /* U+E0112 */
  0x0000,   /* U+E0113 */
  0x0000,   /* U+E0114 */
  0x0000,   /* U+E0115 */
  0x0000,   /* U+E0116 */
  0x0000,   /* U+E0117 */
  0x0000,   /* U+E0118 */
  0x0000,   /* U+E0119 */
  0x0000,   /* U+E011A */
  0x0000,   /* U+E011B */
  0x0000,   /* U+E011C */
  0x0000,   /* U+E011D */
  0x0000,   /* U+E011E */
  0x0000,   /* U+E011F */
  0x0000,   /* U+E0120 */
  0x0000,   /* U+E0121 */
  0x0000,   /* U+E0122 */
  0x0000,   /* U+E0123 */
  0x0000,   /* U+E0124 */
  0x0000,   /* U+E0125 */
  0x0000,   /* U+E0126 */
  0x0000,   /* U+E0127 */
  0x0000,   /* U+E0128 */
  0x0000,   /* U+E0129 */
  0x0000,   /* U+E012A */
  0x0000,   /* U+E012B */
  0x0000,   /* U+E012C */
  0x0000,   /* U+E012D */
  0x0000,   /* U+E012E */
  0x0000,   /* U+E012F */
  0x0000,   /* U+E0130 */
  0x0000,   /* U+E0131 */
  0x0000,   /* U+E0132 */
  0x0000,   /* U+E0133 */
  0x0000,   /* U+E0134 */
  0x0000,   /* U+E0135 */
  0x0000,   /* U+E0136 */
  0x0000,   /* U+E0137 */
  0x0000,   /* U+E0138 */
  0x0000,   /* U+E0139 */
  0x0000,   /* U+E013A */
  0x0000,   /* U+E013B */
  0x0000,   /* U+E013C */
  0x0000,   /* U+E013D */
  0x0000,   /* U+E013E */
  0x0000,   /* U+E013F */
  0x0000,   /* U+E0140 */
  0x0000,   /* U+E0141 */
  0x0000,   /* U+E0142 */
  0x0000,   /* U+E0143 */
  0x0000,   /* U+E0144 */
  0x0000,   /* U+E0145 */
  0x0000,   /* U+E0146 */
  0x0000,   /* U+E0147 */
  0x0000,   /* U+E0148 */
  0x0000,   /* U+E0149 */
  0x0000,   /* U+E014A */
  0x0000,   /* U+E014B */
  0x0000,   /* U+E014C */
  0x0000,   /* U+E014D */
  0x0000,   /* U+E014E */
  0x0000,   /* U+E014F */
  0x0000,   /* U+E0150 */
  0x0000,   /* U+E0151 */
  0x0000,   /* U+E0152 */
  0x0000,   /* U+E0153 */
  0x0000,   /* U+E0154 */
  0x0000,   /* U+E0155 */
  0x0000,   /* U+E0156 */
  0x0000,   /* U+E0157 */
  0x0000,   /* U+E0158 */
  0x0000,   /* U+E0159 */
  0x0000,   /* U+E015A */
  0x0000,   /* U+E015B */
  0x0000,   /* U+E015C */
  0x0000,   /* U+E015D */
  0x0000,   /* U+E015E */
  0x0000,   /* U+E015F */
  0x0000,   /* U+E0160 */
  0x0000,   /* U+E0161 */
  0x0000,   /* U+E0162 */
  0x0000,   /* U+E0163 */
  0x0000,   /* U+E0164 */
  0x0000,   /* U+E0165 */
  0x0000,   /* U+E0166 */
  0x0000,   /* U+E0167 */
  0x0000,   /* U+E0168 */
  0x0000,   /* U+E0169 */
  0x0000,   /* U+E016A */
  0x0000,   /* U+E016B */
  0x0000,   /* U+E016C */
  0x0000,   /* U+E016D */
  0x0000,   /* U+E016E */
  0x0000,   /* U+E016F */
  0x0000,   /* U+E0170 */
  0x0000,   /* U+E0171 */
  0x0000,   /* U+E0172 */
  0x0000,   /* U+E0173 */
  0x0000,   /* U+E0174 */
  0x0000,   /* U+E0175 */
  0x0000,   /* U+E0176 */
  0x0000,   /* U+E0177 */
  0x0000,   /* U+E0178 */
  0x0000,   /* U+E0179 */
  0x0000,   /* U+E017A */
  0x0000,   /* U+E017B */
  0x0000,   /* U+E017C */
  0x0000,   /* U+E017D */
  0x0000,   /* U+E017E */
  0x0000,   /* U+E017F */
  0x0000,   /* U+E0180 */
  0x0000,   /* U+E0181 */
  0x0000,   /* U+E0182 */
  0x0000,   /* U+E0183 */
  0x0000,   /* U+E0184 */
  0x0000,   /* U+E0185 */
  0x0000,   /* U+E0186 */
  0x0000,   /* U+E0187 */
  0x0000,   /* U+E0188 */
  0x0000,   /* U+E0189 */
  0x0000,   /* U+E018A */
  0x0000,   /* U+E018B */
  0x0000,   /* U+E018C */
  0x0000,   /* U+E018D */
  0x0000,   /* U+E018E */
  0x0000,   /* U+E018F */
  0x0000,   /* U+E0190 */
  0x0000,   /* U+E0191 */
  0x0000,   /* U+E0192 */
  0x0000,   /* U+E0193 */
  0x0000,   /* U+E0194 */
  0x0000,   /* U+E0195 */
  0x0000,   /* U+E0196 */
  0x0000,   /* U+E0197 */
  0x0000,   /* U+E0198 */
  0x0000,   /* U+E0199 */
  0x0000,   /* U+E019A */
  0x0000,   /* U+E019B */
  0x0000,   /* U+E019C */
  0x0000,   /* U+E019D */
  0x0000,   /* U+E019E */
  0x0000,   /* U+E019F */
  0x0000,   /* U+E01A0 */
  0x0000,   /* U+E01A1 */
  0x0000,   /* U+E01A2 */
  0x0000,   /* U+E01A3 */
  0x0000,   /* U+E01A4 */
  0x0000,   /* U+E01A5 */
  0x0000,   /* U+E01A6 */
  0x0000,   /* U+E01A7 */
  0x0000,   /* U+E01A8 */
  0x0000,   /* U+E01A9 */
  0x0000,   /* U+E01AA */
  0x0000,   /* U+E01AB */
  0x0000,   /* U+E01AC */
  0x0000,   /* U+E01AD */
  0x0000,   /* U+E01AE */
  0x0000,   /* U+E01AF */
  0x0000,   /* U+E01B0 */
  0x0000,   /* U+E01B1 */
  0x0000,   /* U+E01B2 */
  0x0000,   /* U+E01B3 */
  0x0000,   /* U+E01B4 */
  0x0000,   /* U+E01B5 */
  0x0000,   /* U+E01B6 */
  0x0000,   /* U+E01B7 */
  0x0000,   /* U+E01B8 */
  0x0000,   /* U+E01B9 */
  0x0000,   /* U+E01BA */
  0x0000,   /* U+E01BB */
  0x0000,   /* U+E01BC */
  0x0000,   /* U+E01BD */
  0x0000,   /* U+E01BE */
  0x0000,   /* U+E01BF */
  0x0000,   /* U+E01C0 */
  0x0000,   /* U+E01C1 */
  0x0000,   /* U+E01C2 */
  0x0000,   /* U+E01C3 */
  0x0000,   /* U+E01C4 */
  0x0000,   /* U+E01C5 */
  0x0000,   /* U+E01C6 */
  0x0000,   /* U+E01C7 */
  0x0000,   /* U+E01C8 */
  0x0000,   /* U+E01C9 */
  0x0000,   /* U+E01CA */
  0x0000,   /* U+E01CB */
  0x0000,   /* U+E01CC */
  0x0000,   /* U+E01CD */
  0x0000,   /* U+E01CE */
  0x0000,   /* U+E01CF */
  0x0000,   /* U+E01D0 */
  0x0000,   /* U+E01D1 */
  0x0000,   /* U+E01D2 */
  0x0000,   /* U+E01D3 */
  0x0000,   /* U+E01D4 */
  0x0000,   /* U+E01D5 */
  0x0000,   /* U+E01D6 */
  0x0000,   /* U+E01D7 */
  0x0000,   /* U+E01D8 */
  0x0000,   /* U+E01D9 */
  0x0000,   /* U+E01DA */
  0x0000,   /* U+E01DB */
  0x0000,   /* U+E01DC */
  0x0000,   /* U+E01DD */
  0x0000,   /* U+E01DE */
  0x0000,   /* U+E01DF */
  0x0000,   /* U+E01E0 */
  0x0000,   /* U+E01E1 */
  0x0000,   /* U+E01E2 */
  0x0000,   /* U+E01E3 */
  0x0000,   /* U+E01E4 */
  0x0000,   /* U+E01E5 */
  0x0000,   /* U+E01E6 */
  0x0000,   /* U+E01E7 */
  0x0000,   /* U+E01E8 */
  0x0000,   /* U+E01E9 */
  0x0000,   /* U+E01EA */
  0x0000,   /* U+E01EB */
  0x0000,   /* U+E01EC */
  0x0000,   /* U+E01ED */
  0x0000,   /* U+E01EE */
  0x0000,   /* U+E01EF */
  0x0000,   /* U+E01F0 */
  0x0000,   /* U+E01F1 */
  0x0000,   /* U+E01F2 */
  0x0000,   /* U+E01F3 */
  0x0000,   /* U+E01F4 */
  0x0000,   /* U+E01F5 */
  0x0000,   /* U+E01F6 */
  0x0000,   /* U+E01F7 */
  0x0000,   /* U+E01F8 */
  0x0000,   /* U+E01F9 */
  0x0000,   /* U+E01FA */
  0x0000,   /* U+E01FB */
  0x0000,   /* U+E01FC */
  0x0000,   /* U+E01FD */
  0x0000,   /* U+E01FE */
  0x0000,   /* U+E01FF */
};

uint16 *uca900_weight[4352] = {
    uca900_p000, uca900_p001, uca900_p002, uca900_p003, uca900_p004,
    uca900_p005, uca900_p006, uca900_p007, uca900_p008, uca900_p009,
    uca900_p00A, uca900_p00B, uca900_p00C, uca900_p00D, uca900_p00E,
    uca900_p00F, uca900_p010, uca900_p011, uca900_p012, uca900_p013,
    uca900_p014, uca900_p015, uca900_p016, uca900_p017, uca900_p018,
    uca900_p019, uca900_p01A, uca900_p01B, uca900_p01C, uca900_p01D,
    uca900_p01E, uca900_p01F, uca900_p020, uca900_p021, uca900_p022,
    uca900_p023, uca900_p024, uca900_p025, uca900_p026, uca900_p027,
    uca900_p028, uca900_p029, uca900_p02A, uca900_p02B, uca900_p02C,
    uca900_p02D, uca900_p02E, uca900_p02F, uca900_p030, uca900_p031,
    uca900_p032, uca900_p033, nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     uca900_p04D, nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    uca900_p0A0, uca900_p0A1, uca900_p0A2, uca900_p0A3, uca900_p0A4,
    uca900_p0A5, uca900_p0A6, uca900_p0A7, uca900_p0A8, uca900_p0A9,
    uca900_p0AA, uca900_p0AB, nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    uca900_p0D7, nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p0F9,
    uca900_p0FA, uca900_p0FB, uca900_p0FC, uca900_p0FD, uca900_p0FE,
    uca900_p0FF, uca900_p100, uca900_p101, uca900_p102, uca900_p103,
    uca900_p104, uca900_p105, uca900_p106, uca900_p107, uca900_p108,
    uca900_p109, uca900_p10A, uca900_p10B, uca900_p10C, nullptr,
    uca900_p10E, nullptr,     uca900_p110, uca900_p111, uca900_p112,
    uca900_p113, uca900_p114, uca900_p115, uca900_p116, uca900_p117,
    uca900_p118, nullptr,     uca900_p11A, nullptr,     uca900_p11C,
    nullptr,     nullptr,     nullptr,     uca900_p120, uca900_p121,
    uca900_p122, uca900_p123, uca900_p124, uca900_p125, nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p130,
    uca900_p131, uca900_p132, uca900_p133, uca900_p134, nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p144,
    uca900_p145, uca900_p146, nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    uca900_p168, uca900_p169, uca900_p16A, uca900_p16B, nullptr,
    nullptr,     nullptr,     uca900_p16F, nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     uca900_p1B0, nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p1BC,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p1D0,
    uca900_p1D1, uca900_p1D2, uca900_p1D3, uca900_p1D4, uca900_p1D5,
    uca900_p1D6, uca900_p1D7, uca900_p1D8, uca900_p1D9, uca900_p1DA,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    uca900_p1E0, nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     uca900_p1E8, uca900_p1E9,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_p1EE,
    nullptr,     uca900_p1F0, uca900_p1F1, uca900_p1F2, uca900_p1F3,
    uca900_p1F4, uca900_p1F5, uca900_p1F6, uca900_p1F7, uca900_p1F8,
    uca900_p1F9, nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    uca900_p2F8, uca900_p2F9, uca900_p2FA, nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     uca900_pE00,
    uca900_pE01, nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr,     nullptr,     nullptr,     nullptr,
    nullptr,     nullptr};

static const MY_UNICASE_CHARACTER u900p000[]= {
  {0x0000,0x0000,0x0000},  {0x0001,0x0001,0x0001}, /* 0000 */
  {0x0002,0x0002,0x0002},  {0x0003,0x0003,0x0003}, /* 0002 */
  {0x0004,0x0004,0x0004},  {0x0005,0x0005,0x0005}, /* 0004 */
  {0x0006,0x0006,0x0006},  {0x0007,0x0007,0x0007}, /* 0006 */
  {0x0008,0x0008,0x0008},  {0x0009,0x0009,0x0009}, /* 0008 */
  {0x000A,0x000A,0x000A},  {0x000B,0x000B,0x000B}, /* 000A */
  {0x000C,0x000C,0x000C},  {0x000D,0x000D,0x000D}, /* 000C */
  {0x000E,0x000E,0x000E},  {0x000F,0x000F,0x000F}, /* 000E */
  {0x0010,0x0010,0x0010},  {0x0011,0x0011,0x0011}, /* 0010 */
  {0x0012,0x0012,0x0012},  {0x0013,0x0013,0x0013}, /* 0012 */
  {0x0014,0x0014,0x0014},  {0x0015,0x0015,0x0015}, /* 0014 */
  {0x0016,0x0016,0x0016},  {0x0017,0x0017,0x0017}, /* 0016 */
  {0x0018,0x0018,0x0018},  {0x0019,0x0019,0x0019}, /* 0018 */
  {0x001A,0x001A,0x001A},  {0x001B,0x001B,0x001B}, /* 001A */
  {0x001C,0x001C,0x001C},  {0x001D,0x001D,0x001D}, /* 001C */
  {0x001E,0x001E,0x001E},  {0x001F,0x001F,0x001F}, /* 001E */
  {0x0020,0x0020,0x0020},  {0x0021,0x0021,0x0021}, /* 0020 */
  {0x0022,0x0022,0x0022},  {0x0023,0x0023,0x0023}, /* 0022 */
  {0x0024,0x0024,0x0024},  {0x0025,0x0025,0x0025}, /* 0024 */
  {0x0026,0x0026,0x0026},  {0x0027,0x0027,0x0027}, /* 0026 */
  {0x0028,0x0028,0x0028},  {0x0029,0x0029,0x0029}, /* 0028 */
  {0x002A,0x002A,0x002A},  {0x002B,0x002B,0x002B}, /* 002A */
  {0x002C,0x002C,0x002C},  {0x002D,0x002D,0x002D}, /* 002C */
  {0x002E,0x002E,0x002E},  {0x002F,0x002F,0x002F}, /* 002E */
  {0x0030,0x0030,0x0030},  {0x0031,0x0031,0x0031}, /* 0030 */
  {0x0032,0x0032,0x0032},  {0x0033,0x0033,0x0033}, /* 0032 */
  {0x0034,0x0034,0x0034},  {0x0035,0x0035,0x0035}, /* 0034 */
  {0x0036,0x0036,0x0036},  {0x0037,0x0037,0x0037}, /* 0036 */
  {0x0038,0x0038,0x0038},  {0x0039,0x0039,0x0039}, /* 0038 */
  {0x003A,0x003A,0x003A},  {0x003B,0x003B,0x003B}, /* 003A */
  {0x003C,0x003C,0x003C},  {0x003D,0x003D,0x003D}, /* 003C */
  {0x003E,0x003E,0x003E},  {0x003F,0x003F,0x003F}, /* 003E */
  {0x0040,0x0040,0x0040},  {0x0041,0x0061,0x0041}, /* 0040 */
  {0x0042,0x0062,0x0042},  {0x0043,0x0063,0x0043}, /* 0042 */
  {0x0044,0x0064,0x0044},  {0x0045,0x0065,0x0045}, /* 0044 */
  {0x0046,0x0066,0x0046},  {0x0047,0x0067,0x0047}, /* 0046 */
  {0x0048,0x0068,0x0048},  {0x0049,0x0069,0x0049}, /* 0048 */
  {0x004A,0x006A,0x004A},  {0x004B,0x006B,0x004B}, /* 004A */
  {0x004C,0x006C,0x004C},  {0x004D,0x006D,0x004D}, /* 004C */
  {0x004E,0x006E,0x004E},  {0x004F,0x006F,0x004F}, /* 004E */
  {0x0050,0x0070,0x0050},  {0x0051,0x0071,0x0051}, /* 0050 */
  {0x0052,0x0072,0x0052},  {0x0053,0x0073,0x0053}, /* 0052 */
  {0x0054,0x0074,0x0054},  {0x0055,0x0075,0x0055}, /* 0054 */
  {0x0056,0x0076,0x0056},  {0x0057,0x0077,0x0057}, /* 0056 */
  {0x0058,0x0078,0x0058},  {0x0059,0x0079,0x0059}, /* 0058 */
  {0x005A,0x007A,0x005A},  {0x005B,0x005B,0x005B}, /* 005A */
  {0x005C,0x005C,0x005C},  {0x005D,0x005D,0x005D}, /* 005C */
  {0x005E,0x005E,0x005E},  {0x005F,0x005F,0x005F}, /* 005E */
  {0x0060,0x0060,0x0060},  {0x0041,0x0061,0x0041}, /* 0060 */
  {0x0042,0x0062,0x0042},  {0x0043,0x0063,0x0043}, /* 0062 */
  {0x0044,0x0064,0x0044},  {0x0045,0x0065,0x0045}, /* 0064 */
  {0x0046,0x0066,0x0046},  {0x0047,0x0067,0x0047}, /* 0066 */
  {0x0048,0x0068,0x0048},  {0x0049,0x0069,0x0049}, /* 0068 */
  {0x004A,0x006A,0x004A},  {0x004B,0x006B,0x004B}, /* 006A */
  {0x004C,0x006C,0x004C},  {0x004D,0x006D,0x004D}, /* 006C */
  {0x004E,0x006E,0x004E},  {0x004F,0x006F,0x004F}, /* 006E */
  {0x0050,0x0070,0x0050},  {0x0051,0x0071,0x0051}, /* 0070 */
  {0x0052,0x0072,0x0052},  {0x0053,0x0073,0x0053}, /* 0072 */
  {0x0054,0x0074,0x0054},  {0x0055,0x0075,0x0055}, /* 0074 */
  {0x0056,0x0076,0x0056},  {0x0057,0x0077,0x0057}, /* 0076 */
  {0x0058,0x0078,0x0058},  {0x0059,0x0079,0x0059}, /* 0078 */
  {0x005A,0x007A,0x005A},  {0x007B,0x007B,0x007B}, /* 007A */
  {0x007C,0x007C,0x007C},  {0x007D,0x007D,0x007D}, /* 007C */
  {0x007E,0x007E,0x007E},  {0x007F,0x007F,0x007F}, /* 007E */
  {0x0080,0x0080,0x0080},  {0x0081,0x0081,0x0081}, /* 0080 */
  {0x0082,0x0082,0x0082},  {0x0083,0x0083,0x0083}, /* 0082 */
  {0x0084,0x0084,0x0084},  {0x0085,0x0085,0x0085}, /* 0084 */
  {0x0086,0x0086,0x0086},  {0x0087,0x0087,0x0087}, /* 0086 */
  {0x0088,0x0088,0x0088},  {0x0089,0x0089,0x0089}, /* 0088 */
  {0x008A,0x008A,0x008A},  {0x008B,0x008B,0x008B}, /* 008A */
  {0x008C,0x008C,0x008C},  {0x008D,0x008D,0x008D}, /* 008C */
  {0x008E,0x008E,0x008E},  {0x008F,0x008F,0x008F}, /* 008E */
  {0x0090,0x0090,0x0090},  {0x0091,0x0091,0x0091}, /* 0090 */
  {0x0092,0x0092,0x0092},  {0x0093,0x0093,0x0093}, /* 0092 */
  {0x0094,0x0094,0x0094},  {0x0095,0x0095,0x0095}, /* 0094 */
  {0x0096,0x0096,0x0096},  {0x0097,0x0097,0x0097}, /* 0096 */
  {0x0098,0x0098,0x0098},  {0x0099,0x0099,0x0099}, /* 0098 */
  {0x009A,0x009A,0x009A},  {0x009B,0x009B,0x009B}, /* 009A */
  {0x009C,0x009C,0x009C},  {0x009D,0x009D,0x009D}, /* 009C */
  {0x009E,0x009E,0x009E},  {0x009F,0x009F,0x009F}, /* 009E */
  {0x00A0,0x00A0,0x00A0},  {0x00A1,0x00A1,0x00A1}, /* 00A0 */
  {0x00A2,0x00A2,0x00A2},  {0x00A3,0x00A3,0x00A3}, /* 00A2 */
  {0x00A4,0x00A4,0x00A4},  {0x00A5,0x00A5,0x00A5}, /* 00A4 */
  {0x00A6,0x00A6,0x00A6},  {0x00A7,0x00A7,0x00A7}, /* 00A6 */
  {0x00A8,0x00A8,0x00A8},  {0x00A9,0x00A9,0x00A9}, /* 00A8 */
  {0x00AA,0x00AA,0x00AA},  {0x00AB,0x00AB,0x00AB}, /* 00AA */
  {0x00AC,0x00AC,0x00AC},  {0x00AD,0x00AD,0x00AD}, /* 00AC */
  {0x00AE,0x00AE,0x00AE},  {0x00AF,0x00AF,0x00AF}, /* 00AE */
  {0x00B0,0x00B0,0x00B0},  {0x00B1,0x00B1,0x00B1}, /* 00B0 */
  {0x00B2,0x00B2,0x00B2},  {0x00B3,0x00B3,0x00B3}, /* 00B2 */
  {0x00B4,0x00B4,0x00B4},  {0x039C,0x00B5,0x039C}, /* 00B4 */
  {0x00B6,0x00B6,0x00B6},  {0x00B7,0x00B7,0x00B7}, /* 00B6 */
  {0x00B8,0x00B8,0x00B8},  {0x00B9,0x00B9,0x00B9}, /* 00B8 */
  {0x00BA,0x00BA,0x00BA},  {0x00BB,0x00BB,0x00BB}, /* 00BA */
  {0x00BC,0x00BC,0x00BC},  {0x00BD,0x00BD,0x00BD}, /* 00BC */
  {0x00BE,0x00BE,0x00BE},  {0x00BF,0x00BF,0x00BF}, /* 00BE */
  {0x00C0,0x00E0,0x00C0},  {0x00C1,0x00E1,0x00C1}, /* 00C0 */
  {0x00C2,0x00E2,0x00C2},  {0x00C3,0x00E3,0x00C3}, /* 00C2 */
  {0x00C4,0x00E4,0x00C4},  {0x00C5,0x00E5,0x00C5}, /* 00C4 */
  {0x00C6,0x00E6,0x00C6},  {0x00C7,0x00E7,0x00C7}, /* 00C6 */
  {0x00C8,0x00E8,0x00C8},  {0x00C9,0x00E9,0x00C9}, /* 00C8 */
  {0x00CA,0x00EA,0x00CA},  {0x00CB,0x00EB,0x00CB}, /* 00CA */
  {0x00CC,0x00EC,0x00CC},  {0x00CD,0x00ED,0x00CD}, /* 00CC */
  {0x00CE,0x00EE,0x00CE},  {0x00CF,0x00EF,0x00CF}, /* 00CE */
  {0x00D0,0x00F0,0x00D0},  {0x00D1,0x00F1,0x00D1}, /* 00D0 */
  {0x00D2,0x00F2,0x00D2},  {0x00D3,0x00F3,0x00D3}, /* 00D2 */
  {0x00D4,0x00F4,0x00D4},  {0x00D5,0x00F5,0x00D5}, /* 00D4 */
  {0x00D6,0x00F6,0x00D6},  {0x00D7,0x00D7,0x00D7}, /* 00D6 */
  {0x00D8,0x00F8,0x00D8},  {0x00D9,0x00F9,0x00D9}, /* 00D8 */
  {0x00DA,0x00FA,0x00DA},  {0x00DB,0x00FB,0x00DB}, /* 00DA */
  {0x00DC,0x00FC,0x00DC},  {0x00DD,0x00FD,0x00DD}, /* 00DC */
  {0x00DE,0x00FE,0x00DE},  {0x00DF,0x00DF,0x00DF}, /* 00DE */
  {0x00C0,0x00E0,0x00C0},  {0x00C1,0x00E1,0x00C1}, /* 00E0 */
  {0x00C2,0x00E2,0x00C2},  {0x00C3,0x00E3,0x00C3}, /* 00E2 */
  {0x00C4,0x00E4,0x00C4},  {0x00C5,0x00E5,0x00C5}, /* 00E4 */
  {0x00C6,0x00E6,0x00C6},  {0x00C7,0x00E7,0x00C7}, /* 00E6 */
  {0x00C8,0x00E8,0x00C8},  {0x00C9,0x00E9,0x00C9}, /* 00E8 */
  {0x00CA,0x00EA,0x00CA},  {0x00CB,0x00EB,0x00CB}, /* 00EA */
  {0x00CC,0x00EC,0x00CC},  {0x00CD,0x00ED,0x00CD}, /* 00EC */
  {0x00CE,0x00EE,0x00CE},  {0x00CF,0x00EF,0x00CF}, /* 00EE */
  {0x00D0,0x00F0,0x00D0},  {0x00D1,0x00F1,0x00D1}, /* 00F0 */
  {0x00D2,0x00F2,0x00D2},  {0x00D3,0x00F3,0x00D3}, /* 00F2 */
  {0x00D4,0x00F4,0x00D4},  {0x00D5,0x00F5,0x00D5}, /* 00F4 */
  {0x00D6,0x00F6,0x00D6},  {0x00F7,0x00F7,0x00F7}, /* 00F6 */
  {0x00D8,0x00F8,0x00D8},  {0x00D9,0x00F9,0x00D9}, /* 00F8 */
  {0x00DA,0x00FA,0x00DA},  {0x00DB,0x00FB,0x00DB}, /* 00FA */
  {0x00DC,0x00FC,0x00DC},  {0x00DD,0x00FD,0x00DD}, /* 00FC */
  {0x00DE,0x00FE,0x00DE},  {0x0178,0x00FF,0x0178}  /* 00FE */
};

static const MY_UNICASE_CHARACTER u900p001[]= {
  {0x0100,0x0101,0x0100},  {0x0100,0x0101,0x0100}, /* 0100 */
  {0x0102,0x0103,0x0102},  {0x0102,0x0103,0x0102}, /* 0102 */
  {0x0104,0x0105,0x0104},  {0x0104,0x0105,0x0104}, /* 0104 */
  {0x0106,0x0107,0x0106},  {0x0106,0x0107,0x0106}, /* 0106 */
  {0x0108,0x0109,0x0108},  {0x0108,0x0109,0x0108}, /* 0108 */
  {0x010A,0x010B,0x010A},  {0x010A,0x010B,0x010A}, /* 010A */
  {0x010C,0x010D,0x010C},  {0x010C,0x010D,0x010C}, /* 010C */
  {0x010E,0x010F,0x010E},  {0x010E,0x010F,0x010E}, /* 010E */
  {0x0110,0x0111,0x0110},  {0x0110,0x0111,0x0110}, /* 0110 */
  {0x0112,0x0113,0x0112},  {0x0112,0x0113,0x0112}, /* 0112 */
  {0x0114,0x0115,0x0114},  {0x0114,0x0115,0x0114}, /* 0114 */
  {0x0116,0x0117,0x0116},  {0x0116,0x0117,0x0116}, /* 0116 */
  {0x0118,0x0119,0x0118},  {0x0118,0x0119,0x0118}, /* 0118 */
  {0x011A,0x011B,0x011A},  {0x011A,0x011B,0x011A}, /* 011A */
  {0x011C,0x011D,0x011C},  {0x011C,0x011D,0x011C}, /* 011C */
  {0x011E,0x011F,0x011E},  {0x011E,0x011F,0x011E}, /* 011E */
  {0x0120,0x0121,0x0120},  {0x0120,0x0121,0x0120}, /* 0120 */
  {0x0122,0x0123,0x0122},  {0x0122,0x0123,0x0122}, /* 0122 */
  {0x0124,0x0125,0x0124},  {0x0124,0x0125,0x0124}, /* 0124 */
  {0x0126,0x0127,0x0126},  {0x0126,0x0127,0x0126}, /* 0126 */
  {0x0128,0x0129,0x0128},  {0x0128,0x0129,0x0128}, /* 0128 */
  {0x012A,0x012B,0x012A},  {0x012A,0x012B,0x012A}, /* 012A */
  {0x012C,0x012D,0x012C},  {0x012C,0x012D,0x012C}, /* 012C */
  {0x012E,0x012F,0x012E},  {0x012E,0x012F,0x012E}, /* 012E */
  {0x0130,0x0069,0x0130},  {0x0049,0x0131,0x0049}, /* 0130 */
  {0x0132,0x0133,0x0132},  {0x0132,0x0133,0x0132}, /* 0132 */
  {0x0134,0x0135,0x0134},  {0x0134,0x0135,0x0134}, /* 0134 */
  {0x0136,0x0137,0x0136},  {0x0136,0x0137,0x0136}, /* 0136 */
  {0x0138,0x0138,0x0138},  {0x0139,0x013A,0x0139}, /* 0138 */
  {0x0139,0x013A,0x0139},  {0x013B,0x013C,0x013B}, /* 013A */
  {0x013B,0x013C,0x013B},  {0x013D,0x013E,0x013D}, /* 013C */
  {0x013D,0x013E,0x013D},  {0x013F,0x0140,0x013F}, /* 013E */
  {0x013F,0x0140,0x013F},  {0x0141,0x0142,0x0141}, /* 0140 */
  {0x0141,0x0142,0x0141},  {0x0143,0x0144,0x0143}, /* 0142 */
  {0x0143,0x0144,0x0143},  {0x0145,0x0146,0x0145}, /* 0144 */
  {0x0145,0x0146,0x0145},  {0x0147,0x0148,0x0147}, /* 0146 */
  {0x0147,0x0148,0x0147},  {0x0149,0x0149,0x0149}, /* 0148 */
  {0x014A,0x014B,0x014A},  {0x014A,0x014B,0x014A}, /* 014A */
  {0x014C,0x014D,0x014C},  {0x014C,0x014D,0x014C}, /* 014C */
  {0x014E,0x014F,0x014E},  {0x014E,0x014F,0x014E}, /* 014E */
  {0x0150,0x0151,0x0150},  {0x0150,0x0151,0x0150}, /* 0150 */
  {0x0152,0x0153,0x0152},  {0x0152,0x0153,0x0152}, /* 0152 */
  {0x0154,0x0155,0x0154},  {0x0154,0x0155,0x0154}, /* 0154 */
  {0x0156,0x0157,0x0156},  {0x0156,0x0157,0x0156}, /* 0156 */
  {0x0158,0x0159,0x0158},  {0x0158,0x0159,0x0158}, /* 0158 */
  {0x015A,0x015B,0x015A},  {0x015A,0x015B,0x015A}, /* 015A */
  {0x015C,0x015D,0x015C},  {0x015C,0x015D,0x015C}, /* 015C */
  {0x015E,0x015F,0x015E},  {0x015E,0x015F,0x015E}, /* 015E */
  {0x0160,0x0161,0x0160},  {0x0160,0x0161,0x0160}, /* 0160 */
  {0x0162,0x0163,0x0162},  {0x0162,0x0163,0x0162}, /* 0162 */
  {0x0164,0x0165,0x0164},  {0x0164,0x0165,0x0164}, /* 0164 */
  {0x0166,0x0167,0x0166},  {0x0166,0x0167,0x0166}, /* 0166 */
  {0x0168,0x0169,0x0168},  {0x0168,0x0169,0x0168}, /* 0168 */
  {0x016A,0x016B,0x016A},  {0x016A,0x016B,0x016A}, /* 016A */
  {0x016C,0x016D,0x016C},  {0x016C,0x016D,0x016C}, /* 016C */
  {0x016E,0x016F,0x016E},  {0x016E,0x016F,0x016E}, /* 016E */
  {0x0170,0x0171,0x0170},  {0x0170,0x0171,0x0170}, /* 0170 */
  {0x0172,0x0173,0x0172},  {0x0172,0x0173,0x0172}, /* 0172 */
  {0x0174,0x0175,0x0174},  {0x0174,0x0175,0x0174}, /* 0174 */
  {0x0176,0x0177,0x0176},  {0x0176,0x0177,0x0176}, /* 0176 */
  {0x0178,0x00FF,0x0178},  {0x0179,0x017A,0x0179}, /* 0178 */
  {0x0179,0x017A,0x0179},  {0x017B,0x017C,0x017B}, /* 017A */
  {0x017B,0x017C,0x017B},  {0x017D,0x017E,0x017D}, /* 017C */
  {0x017D,0x017E,0x017D},  {0x0053,0x017F,0x0053}, /* 017E */
  {0x0243,0x0180,0x0243},  {0x0181,0x0253,0x0181}, /* 0180 */
  {0x0182,0x0183,0x0182},  {0x0182,0x0183,0x0182}, /* 0182 */
  {0x0184,0x0185,0x0184},  {0x0184,0x0185,0x0184}, /* 0184 */
  {0x0186,0x0254,0x0186},  {0x0187,0x0188,0x0187}, /* 0186 */
  {0x0187,0x0188,0x0187},  {0x0189,0x0256,0x0189}, /* 0188 */
  {0x018A,0x0257,0x018A},  {0x018B,0x018C,0x018B}, /* 018A */
  {0x018B,0x018C,0x018B},  {0x018D,0x018D,0x018D}, /* 018C */
  {0x018E,0x01DD,0x018E},  {0x018F,0x0259,0x018F}, /* 018E */
  {0x0190,0x025B,0x0190},  {0x0191,0x0192,0x0191}, /* 0190 */
  {0x0191,0x0192,0x0191},  {0x0193,0x0260,0x0193}, /* 0192 */
  {0x0194,0x0263,0x0194},  {0x01F6,0x0195,0x01F6}, /* 0194 */
  {0x0196,0x0269,0x0196},  {0x0197,0x0268,0x0197}, /* 0196 */
  {0x0198,0x0199,0x0198},  {0x0198,0x0199,0x0198}, /* 0198 */
  {0x023D,0x019A,0x023D},  {0x019B,0x019B,0x019B}, /* 019A */
  {0x019C,0x026F,0x019C},  {0x019D,0x0272,0x019D}, /* 019C */
  {0x0220,0x019E,0x0220},  {0x019F,0x0275,0x019F}, /* 019E */
  {0x01A0,0x01A1,0x01A0},  {0x01A0,0x01A1,0x01A0}, /* 01A0 */
  {0x01A2,0x01A3,0x01A2},  {0x01A2,0x01A3,0x01A2}, /* 01A2 */
  {0x01A4,0x01A5,0x01A4},  {0x01A4,0x01A5,0x01A4}, /* 01A4 */
  {0x01A6,0x0280,0x01A6},  {0x01A7,0x01A8,0x01A7}, /* 01A6 */
  {0x01A7,0x01A8,0x01A7},  {0x01A9,0x0283,0x01A9}, /* 01A8 */
  {0x01AA,0x01AA,0x01AA},  {0x01AB,0x01AB,0x01AB}, /* 01AA */
  {0x01AC,0x01AD,0x01AC},  {0x01AC,0x01AD,0x01AC}, /* 01AC */
  {0x01AE,0x0288,0x01AE},  {0x01AF,0x01B0,0x01AF}, /* 01AE */
  {0x01AF,0x01B0,0x01AF},  {0x01B1,0x028A,0x01B1}, /* 01B0 */
  {0x01B2,0x028B,0x01B2},  {0x01B3,0x01B4,0x01B3}, /* 01B2 */
  {0x01B3,0x01B4,0x01B3},  {0x01B5,0x01B6,0x01B5}, /* 01B4 */
  {0x01B5,0x01B6,0x01B5},  {0x01B7,0x0292,0x01B7}, /* 01B6 */
  {0x01B8,0x01B9,0x01B8},  {0x01B8,0x01B9,0x01B8}, /* 01B8 */
  {0x01BA,0x01BA,0x01BA},  {0x01BB,0x01BB,0x01BB}, /* 01BA */
  {0x01BC,0x01BD,0x01BC},  {0x01BC,0x01BD,0x01BC}, /* 01BC */
  {0x01BE,0x01BE,0x01BE},  {0x01F7,0x01BF,0x01F7}, /* 01BE */
  {0x01C0,0x01C0,0x01C0},  {0x01C1,0x01C1,0x01C1}, /* 01C0 */
  {0x01C2,0x01C2,0x01C2},  {0x01C3,0x01C3,0x01C3}, /* 01C2 */
  {0x01C4,0x01C6,0x01C5},  {0x01C4,0x01C6,0x01C5}, /* 01C4 */
  {0x01C4,0x01C6,0x01C5},  {0x01C7,0x01C9,0x01C8}, /* 01C6 */
  {0x01C7,0x01C9,0x01C8},  {0x01C7,0x01C9,0x01C8}, /* 01C8 */
  {0x01CA,0x01CC,0x01CB},  {0x01CA,0x01CC,0x01CB}, /* 01CA */
  {0x01CA,0x01CC,0x01CB},  {0x01CD,0x01CE,0x01CD}, /* 01CC */
  {0x01CD,0x01CE,0x01CD},  {0x01CF,0x01D0,0x01CF}, /* 01CE */
  {0x01CF,0x01D0,0x01CF},  {0x01D1,0x01D2,0x01D1}, /* 01D0 */
  {0x01D1,0x01D2,0x01D1},  {0x01D3,0x01D4,0x01D3}, /* 01D2 */
  {0x01D3,0x01D4,0x01D3},  {0x01D5,0x01D6,0x01D5}, /* 01D4 */
  {0x01D5,0x01D6,0x01D5},  {0x01D7,0x01D8,0x01D7}, /* 01D6 */
  {0x01D7,0x01D8,0x01D7},  {0x01D9,0x01DA,0x01D9}, /* 01D8 */
  {0x01D9,0x01DA,0x01D9},  {0x01DB,0x01DC,0x01DB}, /* 01DA */
  {0x01DB,0x01DC,0x01DB},  {0x018E,0x01DD,0x018E}, /* 01DC */
  {0x01DE,0x01DF,0x01DE},  {0x01DE,0x01DF,0x01DE}, /* 01DE */
  {0x01E0,0x01E1,0x01E0},  {0x01E0,0x01E1,0x01E0}, /* 01E0 */
  {0x01E2,0x01E3,0x01E2},  {0x01E2,0x01E3,0x01E2}, /* 01E2 */
  {0x01E4,0x01E5,0x01E4},  {0x01E4,0x01E5,0x01E4}, /* 01E4 */
  {0x01E6,0x01E7,0x01E6},  {0x01E6,0x01E7,0x01E6}, /* 01E6 */
  {0x01E8,0x01E9,0x01E8},  {0x01E8,0x01E9,0x01E8}, /* 01E8 */
  {0x01EA,0x01EB,0x01EA},  {0x01EA,0x01EB,0x01EA}, /* 01EA */
  {0x01EC,0x01ED,0x01EC},  {0x01EC,0x01ED,0x01EC}, /* 01EC */
  {0x01EE,0x01EF,0x01EE},  {0x01EE,0x01EF,0x01EE}, /* 01EE */
  {0x01F0,0x01F0,0x01F0},  {0x01F1,0x01F3,0x01F2}, /* 01F0 */
  {0x01F1,0x01F3,0x01F2},  {0x01F1,0x01F3,0x01F2}, /* 01F2 */
  {0x01F4,0x01F5,0x01F4},  {0x01F4,0x01F5,0x01F4}, /* 01F4 */
  {0x01F6,0x0195,0x01F6},  {0x01F7,0x01BF,0x01F7}, /* 01F6 */
  {0x01F8,0x01F9,0x01F8},  {0x01F8,0x01F9,0x01F8}, /* 01F8 */
  {0x01FA,0x01FB,0x01FA},  {0x01FA,0x01FB,0x01FA}, /* 01FA */
  {0x01FC,0x01FD,0x01FC},  {0x01FC,0x01FD,0x01FC}, /* 01FC */
  {0x01FE,0x01FF,0x01FE},  {0x01FE,0x01FF,0x01FE}  /* 01FE */
};

static const MY_UNICASE_CHARACTER u900p002[]= {
  {0x0200,0x0201,0x0200},  {0x0200,0x0201,0x0200}, /* 0200 */
  {0x0202,0x0203,0x0202},  {0x0202,0x0203,0x0202}, /* 0202 */
  {0x0204,0x0205,0x0204},  {0x0204,0x0205,0x0204}, /* 0204 */
  {0x0206,0x0207,0x0206},  {0x0206,0x0207,0x0206}, /* 0206 */
  {0x0208,0x0209,0x0208},  {0x0208,0x0209,0x0208}, /* 0208 */
  {0x020A,0x020B,0x020A},  {0x020A,0x020B,0x020A}, /* 020A */
  {0x020C,0x020D,0x020C},  {0x020C,0x020D,0x020C}, /* 020C */
  {0x020E,0x020F,0x020E},  {0x020E,0x020F,0x020E}, /* 020E */
  {0x0210,0x0211,0x0210},  {0x0210,0x0211,0x0210}, /* 0210 */
  {0x0212,0x0213,0x0212},  {0x0212,0x0213,0x0212}, /* 0212 */
  {0x0214,0x0215,0x0214},  {0x0214,0x0215,0x0214}, /* 0214 */
  {0x0216,0x0217,0x0216},  {0x0216,0x0217,0x0216}, /* 0216 */
  {0x0218,0x0219,0x0218},  {0x0218,0x0219,0x0218}, /* 0218 */
  {0x021A,0x021B,0x021A},  {0x021A,0x021B,0x021A}, /* 021A */
  {0x021C,0x021D,0x021C},  {0x021C,0x021D,0x021C}, /* 021C */
  {0x021E,0x021F,0x021E},  {0x021E,0x021F,0x021E}, /* 021E */
  {0x0220,0x019E,0x0220},  {0x0221,0x0221,0x0221}, /* 0220 */
  {0x0222,0x0223,0x0222},  {0x0222,0x0223,0x0222}, /* 0222 */
  {0x0224,0x0225,0x0224},  {0x0224,0x0225,0x0224}, /* 0224 */
  {0x0226,0x0227,0x0226},  {0x0226,0x0227,0x0226}, /* 0226 */
  {0x0228,0x0229,0x0228},  {0x0228,0x0229,0x0228}, /* 0228 */
  {0x022A,0x022B,0x022A},  {0x022A,0x022B,0x022A}, /* 022A */
  {0x022C,0x022D,0x022C},  {0x022C,0x022D,0x022C}, /* 022C */
  {0x022E,0x022F,0x022E},  {0x022E,0x022F,0x022E}, /* 022E */
  {0x0230,0x0231,0x0230},  {0x0230,0x0231,0x0230}, /* 0230 */
  {0x0232,0x0233,0x0232},  {0x0232,0x0233,0x0232}, /* 0232 */
  {0x0234,0x0234,0x0234},  {0x0235,0x0235,0x0235}, /* 0234 */
  {0x0236,0x0236,0x0236},  {0x0237,0x0237,0x0237}, /* 0236 */
  {0x0238,0x0238,0x0238},  {0x0239,0x0239,0x0239}, /* 0238 */
  {0x023A,0x2C65,0x023A},  {0x023B,0x023C,0x023B}, /* 023A */
  {0x023B,0x023C,0x023B},  {0x023D,0x019A,0x023D}, /* 023C */
  {0x023E,0x2C66,0x023E},  {0x2C7E,0x023F,0x2C7E}, /* 023E */
  {0x2C7F,0x0240,0x2C7F},  {0x0241,0x0242,0x0241}, /* 0240 */
  {0x0241,0x0242,0x0241},  {0x0243,0x0180,0x0243}, /* 0242 */
  {0x0244,0x0289,0x0244},  {0x0245,0x028C,0x0245}, /* 0244 */
  {0x0246,0x0247,0x0246},  {0x0246,0x0247,0x0246}, /* 0246 */
  {0x0248,0x0249,0x0248},  {0x0248,0x0249,0x0248}, /* 0248 */
  {0x024A,0x024B,0x024A},  {0x024A,0x024B,0x024A}, /* 024A */
  {0x024C,0x024D,0x024C},  {0x024C,0x024D,0x024C}, /* 024C */
  {0x024E,0x024F,0x024E},  {0x024E,0x024F,0x024E}, /* 024E */
  {0x2C6F,0x0250,0x2C6F},  {0x2C6D,0x0251,0x2C6D}, /* 0250 */
  {0x2C70,0x0252,0x2C70},  {0x0181,0x0253,0x0181}, /* 0252 */
  {0x0186,0x0254,0x0186},  {0x0255,0x0255,0x0255}, /* 0254 */
  {0x0189,0x0256,0x0189},  {0x018A,0x0257,0x018A}, /* 0256 */
  {0x0258,0x0258,0x0258},  {0x018F,0x0259,0x018F}, /* 0258 */
  {0x025A,0x025A,0x025A},  {0x0190,0x025B,0x0190}, /* 025A */
  {0xA7AB,0x025C,0xA7AB},  {0x025D,0x025D,0x025D}, /* 025C */
  {0x025E,0x025E,0x025E},  {0x025F,0x025F,0x025F}, /* 025E */
  {0x0193,0x0260,0x0193},  {0xA7AC,0x0261,0xA7AC}, /* 0260 */
  {0x0262,0x0262,0x0262},  {0x0194,0x0263,0x0194}, /* 0262 */
  {0x0264,0x0264,0x0264},  {0xA78D,0x0265,0xA78D}, /* 0264 */
  {0xA7AA,0x0266,0xA7AA},  {0x0267,0x0267,0x0267}, /* 0266 */
  {0x0197,0x0268,0x0197},  {0x0196,0x0269,0x0196}, /* 0268 */
  {0xA7AE,0x026A,0xA7AE},  {0x2C62,0x026B,0x2C62}, /* 026A */
  {0xA7AD,0x026C,0xA7AD},  {0x026D,0x026D,0x026D}, /* 026C */
  {0x026E,0x026E,0x026E},  {0x019C,0x026F,0x019C}, /* 026E */
  {0x0270,0x0270,0x0270},  {0x2C6E,0x0271,0x2C6E}, /* 0270 */
  {0x019D,0x0272,0x019D},  {0x0273,0x0273,0x0273}, /* 0272 */
  {0x0274,0x0274,0x0274},  {0x019F,0x0275,0x019F}, /* 0274 */
  {0x0276,0x0276,0x0276},  {0x0277,0x0277,0x0277}, /* 0276 */
  {0x0278,0x0278,0x0278},  {0x0279,0x0279,0x0279}, /* 0278 */
  {0x027A,0x027A,0x027A},  {0x027B,0x027B,0x027B}, /* 027A */
  {0x027C,0x027C,0x027C},  {0x2C64,0x027D,0x2C64}, /* 027C */
  {0x027E,0x027E,0x027E},  {0x027F,0x027F,0x027F}, /* 027E */
  {0x01A6,0x0280,0x01A6},  {0x0281,0x0281,0x0281}, /* 0280 */
  {0x0282,0x0282,0x0282},  {0x01A9,0x0283,0x01A9}, /* 0282 */
  {0x0284,0x0284,0x0284},  {0x0285,0x0285,0x0285}, /* 0284 */
  {0x0286,0x0286,0x0286},  {0xA7B1,0x0287,0xA7B1}, /* 0286 */
  {0x01AE,0x0288,0x01AE},  {0x0244,0x0289,0x0244}, /* 0288 */
  {0x01B1,0x028A,0x01B1},  {0x01B2,0x028B,0x01B2}, /* 028A */
  {0x0245,0x028C,0x0245},  {0x028D,0x028D,0x028D}, /* 028C */
  {0x028E,0x028E,0x028E},  {0x028F,0x028F,0x028F}, /* 028E */
  {0x0290,0x0290,0x0290},  {0x0291,0x0291,0x0291}, /* 0290 */
  {0x01B7,0x0292,0x01B7},  {0x0293,0x0293,0x0293}, /* 0292 */
  {0x0294,0x0294,0x0294},  {0x0295,0x0295,0x0295}, /* 0294 */
  {0x0296,0x0296,0x0296},  {0x0297,0x0297,0x0297}, /* 0296 */
  {0x0298,0x0298,0x0298},  {0x0299,0x0299,0x0299}, /* 0298 */
  {0x029A,0x029A,0x029A},  {0x029B,0x029B,0x029B}, /* 029A */
  {0x029C,0x029C,0x029C},  {0xA7B2,0x029D,0xA7B2}, /* 029C */
  {0xA7B0,0x029E,0xA7B0},  {0x029F,0x029F,0x029F}, /* 029E */
  {0x02A0,0x02A0,0x02A0},  {0x02A1,0x02A1,0x02A1}, /* 02A0 */
  {0x02A2,0x02A2,0x02A2},  {0x02A3,0x02A3,0x02A3}, /* 02A2 */
  {0x02A4,0x02A4,0x02A4},  {0x02A5,0x02A5,0x02A5}, /* 02A4 */
  {0x02A6,0x02A6,0x02A6},  {0x02A7,0x02A7,0x02A7}, /* 02A6 */
  {0x02A8,0x02A8,0x02A8},  {0x02A9,0x02A9,0x02A9}, /* 02A8 */
  {0x02AA,0x02AA,0x02AA},  {0x02AB,0x02AB,0x02AB}, /* 02AA */
  {0x02AC,0x02AC,0x02AC},  {0x02AD,0x02AD,0x02AD}, /* 02AC */
  {0x02AE,0x02AE,0x02AE},  {0x02AF,0x02AF,0x02AF}, /* 02AE */
  {0x02B0,0x02B0,0x02B0},  {0x02B1,0x02B1,0x02B1}, /* 02B0 */
  {0x02B2,0x02B2,0x02B2},  {0x02B3,0x02B3,0x02B3}, /* 02B2 */
  {0x02B4,0x02B4,0x02B4},  {0x02B5,0x02B5,0x02B5}, /* 02B4 */
  {0x02B6,0x02B6,0x02B6},  {0x02B7,0x02B7,0x02B7}, /* 02B6 */
  {0x02B8,0x02B8,0x02B8},  {0x02B9,0x02B9,0x02B9}, /* 02B8 */
  {0x02BA,0x02BA,0x02BA},  {0x02BB,0x02BB,0x02BB}, /* 02BA */
  {0x02BC,0x02BC,0x02BC},  {0x02BD,0x02BD,0x02BD}, /* 02BC */
  {0x02BE,0x02BE,0x02BE},  {0x02BF,0x02BF,0x02BF}, /* 02BE */
  {0x02C0,0x02C0,0x02C0},  {0x02C1,0x02C1,0x02C1}, /* 02C0 */
  {0x02C2,0x02C2,0x02C2},  {0x02C3,0x02C3,0x02C3}, /* 02C2 */
  {0x02C4,0x02C4,0x02C4},  {0x02C5,0x02C5,0x02C5}, /* 02C4 */
  {0x02C6,0x02C6,0x02C6},  {0x02C7,0x02C7,0x02C7}, /* 02C6 */
  {0x02C8,0x02C8,0x02C8},  {0x02C9,0x02C9,0x02C9}, /* 02C8 */
  {0x02CA,0x02CA,0x02CA},  {0x02CB,0x02CB,0x02CB}, /* 02CA */
  {0x02CC,0x02CC,0x02CC},  {0x02CD,0x02CD,0x02CD}, /* 02CC */
  {0x02CE,0x02CE,0x02CE},  {0x02CF,0x02CF,0x02CF}, /* 02CE */
  {0x02D0,0x02D0,0x02D0},  {0x02D1,0x02D1,0x02D1}, /* 02D0 */
  {0x02D2,0x02D2,0x02D2},  {0x02D3,0x02D3,0x02D3}, /* 02D2 */
  {0x02D4,0x02D4,0x02D4},  {0x02D5,0x02D5,0x02D5}, /* 02D4 */
  {0x02D6,0x02D6,0x02D6},  {0x02D7,0x02D7,0x02D7}, /* 02D6 */
  {0x02D8,0x02D8,0x02D8},  {0x02D9,0x02D9,0x02D9}, /* 02D8 */
  {0x02DA,0x02DA,0x02DA},  {0x02DB,0x02DB,0x02DB}, /* 02DA */
  {0x02DC,0x02DC,0x02DC},  {0x02DD,0x02DD,0x02DD}, /* 02DC */
  {0x02DE,0x02DE,0x02DE},  {0x02DF,0x02DF,0x02DF}, /* 02DE */
  {0x02E0,0x02E0,0x02E0},  {0x02E1,0x02E1,0x02E1}, /* 02E0 */
  {0x02E2,0x02E2,0x02E2},  {0x02E3,0x02E3,0x02E3}, /* 02E2 */
  {0x02E4,0x02E4,0x02E4},  {0x02E5,0x02E5,0x02E5}, /* 02E4 */
  {0x02E6,0x02E6,0x02E6},  {0x02E7,0x02E7,0x02E7}, /* 02E6 */
  {0x02E8,0x02E8,0x02E8},  {0x02E9,0x02E9,0x02E9}, /* 02E8 */
  {0x02EA,0x02EA,0x02EA},  {0x02EB,0x02EB,0x02EB}, /* 02EA */
  {0x02EC,0x02EC,0x02EC},  {0x02ED,0x02ED,0x02ED}, /* 02EC */
  {0x02EE,0x02EE,0x02EE},  {0x02EF,0x02EF,0x02EF}, /* 02EE */
  {0x02F0,0x02F0,0x02F0},  {0x02F1,0x02F1,0x02F1}, /* 02F0 */
  {0x02F2,0x02F2,0x02F2},  {0x02F3,0x02F3,0x02F3}, /* 02F2 */
  {0x02F4,0x02F4,0x02F4},  {0x02F5,0x02F5,0x02F5}, /* 02F4 */
  {0x02F6,0x02F6,0x02F6},  {0x02F7,0x02F7,0x02F7}, /* 02F6 */
  {0x02F8,0x02F8,0x02F8},  {0x02F9,0x02F9,0x02F9}, /* 02F8 */
  {0x02FA,0x02FA,0x02FA},  {0x02FB,0x02FB,0x02FB}, /* 02FA */
  {0x02FC,0x02FC,0x02FC},  {0x02FD,0x02FD,0x02FD}, /* 02FC */
  {0x02FE,0x02FE,0x02FE},  {0x02FF,0x02FF,0x02FF}  /* 02FE */
};

static const MY_UNICASE_CHARACTER u900p003[]= {
  {0x0300,0x0300,0x0300},  {0x0301,0x0301,0x0301}, /* 0300 */
  {0x0302,0x0302,0x0302},  {0x0303,0x0303,0x0303}, /* 0302 */
  {0x0304,0x0304,0x0304},  {0x0305,0x0305,0x0305}, /* 0304 */
  {0x0306,0x0306,0x0306},  {0x0307,0x0307,0x0307}, /* 0306 */
  {0x0308,0x0308,0x0308},  {0x0309,0x0309,0x0309}, /* 0308 */
  {0x030A,0x030A,0x030A},  {0x030B,0x030B,0x030B}, /* 030A */
  {0x030C,0x030C,0x030C},  {0x030D,0x030D,0x030D}, /* 030C */
  {0x030E,0x030E,0x030E},  {0x030F,0x030F,0x030F}, /* 030E */
  {0x0310,0x0310,0x0310},  {0x0311,0x0311,0x0311}, /* 0310 */
  {0x0312,0x0312,0x0312},  {0x0313,0x0313,0x0313}, /* 0312 */
  {0x0314,0x0314,0x0314},  {0x0315,0x0315,0x0315}, /* 0314 */
  {0x0316,0x0316,0x0316},  {0x0317,0x0317,0x0317}, /* 0316 */
  {0x0318,0x0318,0x0318},  {0x0319,0x0319,0x0319}, /* 0318 */
  {0x031A,0x031A,0x031A},  {0x031B,0x031B,0x031B}, /* 031A */
  {0x031C,0x031C,0x031C},  {0x031D,0x031D,0x031D}, /* 031C */
  {0x031E,0x031E,0x031E},  {0x031F,0x031F,0x031F}, /* 031E */
  {0x0320,0x0320,0x0320},  {0x0321,0x0321,0x0321}, /* 0320 */
  {0x0322,0x0322,0x0322},  {0x0323,0x0323,0x0323}, /* 0322 */
  {0x0324,0x0324,0x0324},  {0x0325,0x0325,0x0325}, /* 0324 */
  {0x0326,0x0326,0x0326},  {0x0327,0x0327,0x0327}, /* 0326 */
  {0x0328,0x0328,0x0328},  {0x0329,0x0329,0x0329}, /* 0328 */
  {0x032A,0x032A,0x032A},  {0x032B,0x032B,0x032B}, /* 032A */
  {0x032C,0x032C,0x032C},  {0x032D,0x032D,0x032D}, /* 032C */
  {0x032E,0x032E,0x032E},  {0x032F,0x032F,0x032F}, /* 032E */
  {0x0330,0x0330,0x0330},  {0x0331,0x0331,0x0331}, /* 0330 */
  {0x0332,0x0332,0x0332},  {0x0333,0x0333,0x0333}, /* 0332 */
  {0x0334,0x0334,0x0334},  {0x0335,0x0335,0x0335}, /* 0334 */
  {0x0336,0x0336,0x0336},  {0x0337,0x0337,0x0337}, /* 0336 */
  {0x0338,0x0338,0x0338},  {0x0339,0x0339,0x0339}, /* 0338 */
  {0x033A,0x033A,0x033A},  {0x033B,0x033B,0x033B}, /* 033A */
  {0x033C,0x033C,0x033C},  {0x033D,0x033D,0x033D}, /* 033C */
  {0x033E,0x033E,0x033E},  {0x033F,0x033F,0x033F}, /* 033E */
  {0x0340,0x0340,0x0340},  {0x0341,0x0341,0x0341}, /* 0340 */
  {0x0342,0x0342,0x0342},  {0x0343,0x0343,0x0343}, /* 0342 */
  {0x0344,0x0344,0x0344},  {0x0399,0x0345,0x0399}, /* 0344 */
  {0x0346,0x0346,0x0346},  {0x0347,0x0347,0x0347}, /* 0346 */
  {0x0348,0x0348,0x0348},  {0x0349,0x0349,0x0349}, /* 0348 */
  {0x034A,0x034A,0x034A},  {0x034B,0x034B,0x034B}, /* 034A */
  {0x034C,0x034C,0x034C},  {0x034D,0x034D,0x034D}, /* 034C */
  {0x034E,0x034E,0x034E},  {0x034F,0x034F,0x034F}, /* 034E */
  {0x0350,0x0350,0x0350},  {0x0351,0x0351,0x0351}, /* 0350 */
  {0x0352,0x0352,0x0352},  {0x0353,0x0353,0x0353}, /* 0352 */
  {0x0354,0x0354,0x0354},  {0x0355,0x0355,0x0355}, /* 0354 */
  {0x0356,0x0356,0x0356},  {0x0357,0x0357,0x0357}, /* 0356 */
  {0x0358,0x0358,0x0358},  {0x0359,0x0359,0x0359}, /* 0358 */
  {0x035A,0x035A,0x035A},  {0x035B,0x035B,0x035B}, /* 035A */
  {0x035C,0x035C,0x035C},  {0x035D,0x035D,0x035D}, /* 035C */
  {0x035E,0x035E,0x035E},  {0x035F,0x035F,0x035F}, /* 035E */
  {0x0360,0x0360,0x0360},  {0x0361,0x0361,0x0361}, /* 0360 */
  {0x0362,0x0362,0x0362},  {0x0363,0x0363,0x0363}, /* 0362 */
  {0x0364,0x0364,0x0364},  {0x0365,0x0365,0x0365}, /* 0364 */
  {0x0366,0x0366,0x0366},  {0x0367,0x0367,0x0367}, /* 0366 */
  {0x0368,0x0368,0x0368},  {0x0369,0x0369,0x0369}, /* 0368 */
  {0x036A,0x036A,0x036A},  {0x036B,0x036B,0x036B}, /* 036A */
  {0x036C,0x036C,0x036C},  {0x036D,0x036D,0x036D}, /* 036C */
  {0x036E,0x036E,0x036E},  {0x036F,0x036F,0x036F}, /* 036E */
  {0x0370,0x0371,0x0370},  {0x0370,0x0371,0x0370}, /* 0370 */
  {0x0372,0x0373,0x0372},  {0x0372,0x0373,0x0372}, /* 0372 */
  {0x0374,0x0374,0x0374},  {0x0375,0x0375,0x0375}, /* 0374 */
  {0x0376,0x0377,0x0376},  {0x0376,0x0377,0x0376}, /* 0376 */
  {0x0378,0x0378,0x0378},  {0x0379,0x0379,0x0379}, /* 0378 */
  {0x037A,0x037A,0x037A},  {0x03FD,0x037B,0x03FD}, /* 037A */
  {0x03FE,0x037C,0x03FE},  {0x03FF,0x037D,0x03FF}, /* 037C */
  {0x037E,0x037E,0x037E},  {0x037F,0x03F3,0x037F}, /* 037E */
  {0x0380,0x0380,0x0380},  {0x0381,0x0381,0x0381}, /* 0380 */
  {0x0382,0x0382,0x0382},  {0x0383,0x0383,0x0383}, /* 0382 */
  {0x0384,0x0384,0x0384},  {0x0385,0x0385,0x0385}, /* 0384 */
  {0x0386,0x03AC,0x0386},  {0x0387,0x0387,0x0387}, /* 0386 */
  {0x0388,0x03AD,0x0388},  {0x0389,0x03AE,0x0389}, /* 0388 */
  {0x038A,0x03AF,0x038A},  {0x038B,0x038B,0x038B}, /* 038A */
  {0x038C,0x03CC,0x038C},  {0x038D,0x038D,0x038D}, /* 038C */
  {0x038E,0x03CD,0x038E},  {0x038F,0x03CE,0x038F}, /* 038E */
  {0x0390,0x0390,0x0390},  {0x0391,0x03B1,0x0391}, /* 0390 */
  {0x0392,0x03B2,0x0392},  {0x0393,0x03B3,0x0393}, /* 0392 */
  {0x0394,0x03B4,0x0394},  {0x0395,0x03B5,0x0395}, /* 0394 */
  {0x0396,0x03B6,0x0396},  {0x0397,0x03B7,0x0397}, /* 0396 */
  {0x0398,0x03B8,0x0398},  {0x0399,0x03B9,0x0399}, /* 0398 */
  {0x039A,0x03BA,0x039A},  {0x039B,0x03BB,0x039B}, /* 039A */
  {0x039C,0x03BC,0x039C},  {0x039D,0x03BD,0x039D}, /* 039C */
  {0x039E,0x03BE,0x039E},  {0x039F,0x03BF,0x039F}, /* 039E */
  {0x03A0,0x03C0,0x03A0},  {0x03A1,0x03C1,0x03A1}, /* 03A0 */
  {0x03A2,0x03A2,0x03A2},  {0x03A3,0x03C3,0x03A3}, /* 03A2 */
  {0x03A4,0x03C4,0x03A4},  {0x03A5,0x03C5,0x03A5}, /* 03A4 */
  {0x03A6,0x03C6,0x03A6},  {0x03A7,0x03C7,0x03A7}, /* 03A6 */
  {0x03A8,0x03C8,0x03A8},  {0x03A9,0x03C9,0x03A9}, /* 03A8 */
  {0x03AA,0x03CA,0x03AA},  {0x03AB,0x03CB,0x03AB}, /* 03AA */
  {0x0386,0x03AC,0x0386},  {0x0388,0x03AD,0x0388}, /* 03AC */
  {0x0389,0x03AE,0x0389},  {0x038A,0x03AF,0x038A}, /* 03AE */
  {0x03B0,0x03B0,0x03B0},  {0x0391,0x03B1,0x0391}, /* 03B0 */
  {0x0392,0x03B2,0x0392},  {0x0393,0x03B3,0x0393}, /* 03B2 */
  {0x0394,0x03B4,0x0394},  {0x0395,0x03B5,0x0395}, /* 03B4 */
  {0x0396,0x03B6,0x0396},  {0x0397,0x03B7,0x0397}, /* 03B6 */
  {0x0398,0x03B8,0x0398},  {0x0399,0x03B9,0x0399}, /* 03B8 */
  {0x039A,0x03BA,0x039A},  {0x039B,0x03BB,0x039B}, /* 03BA */
  {0x039C,0x03BC,0x039C},  {0x039D,0x03BD,0x039D}, /* 03BC */
  {0x039E,0x03BE,0x039E},  {0x039F,0x03BF,0x039F}, /* 03BE */
  {0x03A0,0x03C0,0x03A0},  {0x03A1,0x03C1,0x03A1}, /* 03C0 */
  {0x03A3,0x03C2,0x03A3},  {0x03A3,0x03C3,0x03A3}, /* 03C2 */
  {0x03A4,0x03C4,0x03A4},  {0x03A5,0x03C5,0x03A5}, /* 03C4 */
  {0x03A6,0x03C6,0x03A6},  {0x03A7,0x03C7,0x03A7}, /* 03C6 */
  {0x03A8,0x03C8,0x03A8},  {0x03A9,0x03C9,0x03A9}, /* 03C8 */
  {0x03AA,0x03CA,0x03AA},  {0x03AB,0x03CB,0x03AB}, /* 03CA */
  {0x038C,0x03CC,0x038C},  {0x038E,0x03CD,0x038E}, /* 03CC */
  {0x038F,0x03CE,0x038F},  {0x03CF,0x03D7,0x03CF}, /* 03CE */
  {0x0392,0x03D0,0x0392},  {0x0398,0x03D1,0x0398}, /* 03D0 */
  {0x03D2,0x03D2,0x03D2},  {0x03D3,0x03D3,0x03D3}, /* 03D2 */
  {0x03D4,0x03D4,0x03D4},  {0x03A6,0x03D5,0x03A6}, /* 03D4 */
  {0x03A0,0x03D6,0x03A0},  {0x03CF,0x03D7,0x03CF}, /* 03D6 */
  {0x03D8,0x03D9,0x03D8},  {0x03D8,0x03D9,0x03D8}, /* 03D8 */
  {0x03DA,0x03DB,0x03DA},  {0x03DA,0x03DB,0x03DA}, /* 03DA */
  {0x03DC,0x03DD,0x03DC},  {0x03DC,0x03DD,0x03DC}, /* 03DC */
  {0x03DE,0x03DF,0x03DE},  {0x03DE,0x03DF,0x03DE}, /* 03DE */
  {0x03E0,0x03E1,0x03E0},  {0x03E0,0x03E1,0x03E0}, /* 03E0 */
  {0x03E2,0x03E3,0x03E2},  {0x03E2,0x03E3,0x03E2}, /* 03E2 */
  {0x03E4,0x03E5,0x03E4},  {0x03E4,0x03E5,0x03E4}, /* 03E4 */
  {0x03E6,0x03E7,0x03E6},  {0x03E6,0x03E7,0x03E6}, /* 03E6 */
  {0x03E8,0x03E9,0x03E8},  {0x03E8,0x03E9,0x03E8}, /* 03E8 */
  {0x03EA,0x03EB,0x03EA},  {0x03EA,0x03EB,0x03EA}, /* 03EA */
  {0x03EC,0x03ED,0x03EC},  {0x03EC,0x03ED,0x03EC}, /* 03EC */
  {0x03EE,0x03EF,0x03EE},  {0x03EE,0x03EF,0x03EE}, /* 03EE */
  {0x039A,0x03F0,0x039A},  {0x03A1,0x03F1,0x03A1}, /* 03F0 */
  {0x03F9,0x03F2,0x03F9},  {0x037F,0x03F3,0x037F}, /* 03F2 */
  {0x03F4,0x03B8,0x03F4},  {0x0395,0x03F5,0x0395}, /* 03F4 */
  {0x03F6,0x03F6,0x03F6},  {0x03F7,0x03F8,0x03F7}, /* 03F6 */
  {0x03F7,0x03F8,0x03F7},  {0x03F9,0x03F2,0x03F9}, /* 03F8 */
  {0x03FA,0x03FB,0x03FA},  {0x03FA,0x03FB,0x03FA}, /* 03FA */
  {0x03FC,0x03FC,0x03FC},  {0x03FD,0x037B,0x03FD}, /* 03FC */
  {0x03FE,0x037C,0x03FE},  {0x03FF,0x037D,0x03FF}  /* 03FE */
};

static const MY_UNICASE_CHARACTER u900p004[]= {
  {0x0400,0x0450,0x0400},  {0x0401,0x0451,0x0401}, /* 0400 */
  {0x0402,0x0452,0x0402},  {0x0403,0x0453,0x0403}, /* 0402 */
  {0x0404,0x0454,0x0404},  {0x0405,0x0455,0x0405}, /* 0404 */
  {0x0406,0x0456,0x0406},  {0x0407,0x0457,0x0407}, /* 0406 */
  {0x0408,0x0458,0x0408},  {0x0409,0x0459,0x0409}, /* 0408 */
  {0x040A,0x045A,0x040A},  {0x040B,0x045B,0x040B}, /* 040A */
  {0x040C,0x045C,0x040C},  {0x040D,0x045D,0x040D}, /* 040C */
  {0x040E,0x045E,0x040E},  {0x040F,0x045F,0x040F}, /* 040E */
  {0x0410,0x0430,0x0410},  {0x0411,0x0431,0x0411}, /* 0410 */
  {0x0412,0x0432,0x0412},  {0x0413,0x0433,0x0413}, /* 0412 */
  {0x0414,0x0434,0x0414},  {0x0415,0x0435,0x0415}, /* 0414 */
  {0x0416,0x0436,0x0416},  {0x0417,0x0437,0x0417}, /* 0416 */
  {0x0418,0x0438,0x0418},  {0x0419,0x0439,0x0419}, /* 0418 */
  {0x041A,0x043A,0x041A},  {0x041B,0x043B,0x041B}, /* 041A */
  {0x041C,0x043C,0x041C},  {0x041D,0x043D,0x041D}, /* 041C */
  {0x041E,0x043E,0x041E},  {0x041F,0x043F,0x041F}, /* 041E */
  {0x0420,0x0440,0x0420},  {0x0421,0x0441,0x0421}, /* 0420 */
  {0x0422,0x0442,0x0422},  {0x0423,0x0443,0x0423}, /* 0422 */
  {0x0424,0x0444,0x0424},  {0x0425,0x0445,0x0425}, /* 0424 */
  {0x0426,0x0446,0x0426},  {0x0427,0x0447,0x0427}, /* 0426 */
  {0x0428,0x0448,0x0428},  {0x0429,0x0449,0x0429}, /* 0428 */
  {0x042A,0x044A,0x042A},  {0x042B,0x044B,0x042B}, /* 042A */
  {0x042C,0x044C,0x042C},  {0x042D,0x044D,0x042D}, /* 042C */
  {0x042E,0x044E,0x042E},  {0x042F,0x044F,0x042F}, /* 042E */
  {0x0410,0x0430,0x0410},  {0x0411,0x0431,0x0411}, /* 0430 */
  {0x0412,0x0432,0x0412},  {0x0413,0x0433,0x0413}, /* 0432 */
  {0x0414,0x0434,0x0414},  {0x0415,0x0435,0x0415}, /* 0434 */
  {0x0416,0x0436,0x0416},  {0x0417,0x0437,0x0417}, /* 0436 */
  {0x0418,0x0438,0x0418},  {0x0419,0x0439,0x0419}, /* 0438 */
  {0x041A,0x043A,0x041A},  {0x041B,0x043B,0x041B}, /* 043A */
  {0x041C,0x043C,0x041C},  {0x041D,0x043D,0x041D}, /* 043C */
  {0x041E,0x043E,0x041E},  {0x041F,0x043F,0x041F}, /* 043E */
  {0x0420,0x0440,0x0420},  {0x0421,0x0441,0x0421}, /* 0440 */
  {0x0422,0x0442,0x0422},  {0x0423,0x0443,0x0423}, /* 0442 */
  {0x0424,0x0444,0x0424},  {0x0425,0x0445,0x0425}, /* 0444 */
  {0x0426,0x0446,0x0426},  {0x0427,0x0447,0x0427}, /* 0446 */
  {0x0428,0x0448,0x0428},  {0x0429,0x0449,0x0429}, /* 0448 */
  {0x042A,0x044A,0x042A},  {0x042B,0x044B,0x042B}, /* 044A */
  {0x042C,0x044C,0x042C},  {0x042D,0x044D,0x042D}, /* 044C */
  {0x042E,0x044E,0x042E},  {0x042F,0x044F,0x042F}, /* 044E */
  {0x0400,0x0450,0x0400},  {0x0401,0x0451,0x0401}, /* 0450 */
  {0x0402,0x0452,0x0402},  {0x0403,0x0453,0x0403}, /* 0452 */
  {0x0404,0x0454,0x0404},  {0x0405,0x0455,0x0405}, /* 0454 */
  {0x0406,0x0456,0x0406},  {0x0407,0x0457,0x0407}, /* 0456 */
  {0x0408,0x0458,0x0408},  {0x0409,0x0459,0x0409}, /* 0458 */
  {0x040A,0x045A,0x040A},  {0x040B,0x045B,0x040B}, /* 045A */
  {0x040C,0x045C,0x040C},  {0x040D,0x045D,0x040D}, /* 045C */
  {0x040E,0x045E,0x040E},  {0x040F,0x045F,0x040F}, /* 045E */
  {0x0460,0x0461,0x0460},  {0x0460,0x0461,0x0460}, /* 0460 */
  {0x0462,0x0463,0x0462},  {0x0462,0x0463,0x0462}, /* 0462 */
  {0x0464,0x0465,0x0464},  {0x0464,0x0465,0x0464}, /* 0464 */
  {0x0466,0x0467,0x0466},  {0x0466,0x0467,0x0466}, /* 0466 */
  {0x0468,0x0469,0x0468},  {0x0468,0x0469,0x0468}, /* 0468 */
  {0x046A,0x046B,0x046A},  {0x046A,0x046B,0x046A}, /* 046A */
  {0x046C,0x046D,0x046C},  {0x046C,0x046D,0x046C}, /* 046C */
  {0x046E,0x046F,0x046E},  {0x046E,0x046F,0x046E}, /* 046E */
  {0x0470,0x0471,0x0470},  {0x0470,0x0471,0x0470}, /* 0470 */
  {0x0472,0x0473,0x0472},  {0x0472,0x0473,0x0472}, /* 0472 */
  {0x0474,0x0475,0x0474},  {0x0474,0x0475,0x0474}, /* 0474 */
  {0x0476,0x0477,0x0476},  {0x0476,0x0477,0x0476}, /* 0476 */
  {0x0478,0x0479,0x0478},  {0x0478,0x0479,0x0478}, /* 0478 */
  {0x047A,0x047B,0x047A},  {0x047A,0x047B,0x047A}, /* 047A */
  {0x047C,0x047D,0x047C},  {0x047C,0x047D,0x047C}, /* 047C */
  {0x047E,0x047F,0x047E},  {0x047E,0x047F,0x047E}, /* 047E */
  {0x0480,0x0481,0x0480},  {0x0480,0x0481,0x0480}, /* 0480 */
  {0x0482,0x0482,0x0482},  {0x0483,0x0483,0x0483}, /* 0482 */
  {0x0484,0x0484,0x0484},  {0x0485,0x0485,0x0485}, /* 0484 */
  {0x0486,0x0486,0x0486},  {0x0487,0x0487,0x0487}, /* 0486 */
  {0x0488,0x0488,0x0488},  {0x0489,0x0489,0x0489}, /* 0488 */
  {0x048A,0x048B,0x048A},  {0x048A,0x048B,0x048A}, /* 048A */
  {0x048C,0x048D,0x048C},  {0x048C,0x048D,0x048C}, /* 048C */
  {0x048E,0x048F,0x048E},  {0x048E,0x048F,0x048E}, /* 048E */
  {0x0490,0x0491,0x0490},  {0x0490,0x0491,0x0490}, /* 0490 */
  {0x0492,0x0493,0x0492},  {0x0492,0x0493,0x0492}, /* 0492 */
  {0x0494,0x0495,0x0494},  {0x0494,0x0495,0x0494}, /* 0494 */
  {0x0496,0x0497,0x0496},  {0x0496,0x0497,0x0496}, /* 0496 */
  {0x0498,0x0499,0x0498},  {0x0498,0x0499,0x0498}, /* 0498 */
  {0x049A,0x049B,0x049A},  {0x049A,0x049B,0x049A}, /* 049A */
  {0x049C,0x049D,0x049C},  {0x049C,0x049D,0x049C}, /* 049C */
  {0x049E,0x049F,0x049E},  {0x049E,0x049F,0x049E}, /* 049E */
  {0x04A0,0x04A1,0x04A0},  {0x04A0,0x04A1,0x04A0}, /* 04A0 */
  {0x04A2,0x04A3,0x04A2},  {0x04A2,0x04A3,0x04A2}, /* 04A2 */
  {0x04A4,0x04A5,0x04A4},  {0x04A4,0x04A5,0x04A4}, /* 04A4 */
  {0x04A6,0x04A7,0x04A6},  {0x04A6,0x04A7,0x04A6}, /* 04A6 */
  {0x04A8,0x04A9,0x04A8},  {0x04A8,0x04A9,0x04A8}, /* 04A8 */
  {0x04AA,0x04AB,0x04AA},  {0x04AA,0x04AB,0x04AA}, /* 04AA */
  {0x04AC,0x04AD,0x04AC},  {0x04AC,0x04AD,0x04AC}, /* 04AC */
  {0x04AE,0x04AF,0x04AE},  {0x04AE,0x04AF,0x04AE}, /* 04AE */
  {0x04B0,0x04B1,0x04B0},  {0x04B0,0x04B1,0x04B0}, /* 04B0 */
  {0x04B2,0x04B3,0x04B2},  {0x04B2,0x04B3,0x04B2}, /* 04B2 */
  {0x04B4,0x04B5,0x04B4},  {0x04B4,0x04B5,0x04B4}, /* 04B4 */
  {0x04B6,0x04B7,0x04B6},  {0x04B6,0x04B7,0x04B6}, /* 04B6 */
  {0x04B8,0x04B9,0x04B8},  {0x04B8,0x04B9,0x04B8}, /* 04B8 */
  {0x04BA,0x04BB,0x04BA},  {0x04BA,0x04BB,0x04BA}, /* 04BA */
  {0x04BC,0x04BD,0x04BC},  {0x04BC,0x04BD,0x04BC}, /* 04BC */
  {0x04BE,0x04BF,0x04BE},  {0x04BE,0x04BF,0x04BE}, /* 04BE */
  {0x04C0,0x04CF,0x04C0},  {0x04C1,0x04C2,0x04C1}, /* 04C0 */
  {0x04C1,0x04C2,0x04C1},  {0x04C3,0x04C4,0x04C3}, /* 04C2 */
  {0x04C3,0x04C4,0x04C3},  {0x04C5,0x04C6,0x04C5}, /* 04C4 */
  {0x04C5,0x04C6,0x04C5},  {0x04C7,0x04C8,0x04C7}, /* 04C6 */
  {0x04C7,0x04C8,0x04C7},  {0x04C9,0x04CA,0x04C9}, /* 04C8 */
  {0x04C9,0x04CA,0x04C9},  {0x04CB,0x04CC,0x04CB}, /* 04CA */
  {0x04CB,0x04CC,0x04CB},  {0x04CD,0x04CE,0x04CD}, /* 04CC */
  {0x04CD,0x04CE,0x04CD},  {0x04C0,0x04CF,0x04C0}, /* 04CE */
  {0x04D0,0x04D1,0x04D0},  {0x04D0,0x04D1,0x04D0}, /* 04D0 */
  {0x04D2,0x04D3,0x04D2},  {0x04D2,0x04D3,0x04D2}, /* 04D2 */
  {0x04D4,0x04D5,0x04D4},  {0x04D4,0x04D5,0x04D4}, /* 04D4 */
  {0x04D6,0x04D7,0x04D6},  {0x04D6,0x04D7,0x04D6}, /* 04D6 */
  {0x04D8,0x04D9,0x04D8},  {0x04D8,0x04D9,0x04D8}, /* 04D8 */
  {0x04DA,0x04DB,0x04DA},  {0x04DA,0x04DB,0x04DA}, /* 04DA */
  {0x04DC,0x04DD,0x04DC},  {0x04DC,0x04DD,0x04DC}, /* 04DC */
  {0x04DE,0x04DF,0x04DE},  {0x04DE,0x04DF,0x04DE}, /* 04DE */
  {0x04E0,0x04E1,0x04E0},  {0x04E0,0x04E1,0x04E0}, /* 04E0 */
  {0x04E2,0x04E3,0x04E2},  {0x04E2,0x04E3,0x04E2}, /* 04E2 */
  {0x04E4,0x04E5,0x04E4},  {0x04E4,0x04E5,0x04E4}, /* 04E4 */
  {0x04E6,0x04E7,0x04E6},  {0x04E6,0x04E7,0x04E6}, /* 04E6 */
  {0x04E8,0x04E9,0x04E8},  {0x04E8,0x04E9,0x04E8}, /* 04E8 */
  {0x04EA,0x04EB,0x04EA},  {0x04EA,0x04EB,0x04EA}, /* 04EA */
  {0x04EC,0x04ED,0x04EC},  {0x04EC,0x04ED,0x04EC}, /* 04EC */
  {0x04EE,0x04EF,0x04EE},  {0x04EE,0x04EF,0x04EE}, /* 04EE */
  {0x04F0,0x04F1,0x04F0},  {0x04F0,0x04F1,0x04F0}, /* 04F0 */
  {0x04F2,0x04F3,0x04F2},  {0x04F2,0x04F3,0x04F2}, /* 04F2 */
  {0x04F4,0x04F5,0x04F4},  {0x04F4,0x04F5,0x04F4}, /* 04F4 */
  {0x04F6,0x04F7,0x04F6},  {0x04F6,0x04F7,0x04F6}, /* 04F6 */
  {0x04F8,0x04F9,0x04F8},  {0x04F8,0x04F9,0x04F8}, /* 04F8 */
  {0x04FA,0x04FB,0x04FA},  {0x04FA,0x04FB,0x04FA}, /* 04FA */
  {0x04FC,0x04FD,0x04FC},  {0x04FC,0x04FD,0x04FC}, /* 04FC */
  {0x04FE,0x04FF,0x04FE},  {0x04FE,0x04FF,0x04FE}  /* 04FE */
};

static const MY_UNICASE_CHARACTER u900p005[]= {
  {0x0500,0x0501,0x0500},  {0x0500,0x0501,0x0500}, /* 0500 */
  {0x0502,0x0503,0x0502},  {0x0502,0x0503,0x0502}, /* 0502 */
  {0x0504,0x0505,0x0504},  {0x0504,0x0505,0x0504}, /* 0504 */
  {0x0506,0x0507,0x0506},  {0x0506,0x0507,0x0506}, /* 0506 */
  {0x0508,0x0509,0x0508},  {0x0508,0x0509,0x0508}, /* 0508 */
  {0x050A,0x050B,0x050A},  {0x050A,0x050B,0x050A}, /* 050A */
  {0x050C,0x050D,0x050C},  {0x050C,0x050D,0x050C}, /* 050C */
  {0x050E,0x050F,0x050E},  {0x050E,0x050F,0x050E}, /* 050E */
  {0x0510,0x0511,0x0510},  {0x0510,0x0511,0x0510}, /* 0510 */
  {0x0512,0x0513,0x0512},  {0x0512,0x0513,0x0512}, /* 0512 */
  {0x0514,0x0515,0x0514},  {0x0514,0x0515,0x0514}, /* 0514 */
  {0x0516,0x0517,0x0516},  {0x0516,0x0517,0x0516}, /* 0516 */
  {0x0518,0x0519,0x0518},  {0x0518,0x0519,0x0518}, /* 0518 */
  {0x051A,0x051B,0x051A},  {0x051A,0x051B,0x051A}, /* 051A */
  {0x051C,0x051D,0x051C},  {0x051C,0x051D,0x051C}, /* 051C */
  {0x051E,0x051F,0x051E},  {0x051E,0x051F,0x051E}, /* 051E */
  {0x0520,0x0521,0x0520},  {0x0520,0x0521,0x0520}, /* 0520 */
  {0x0522,0x0523,0x0522},  {0x0522,0x0523,0x0522}, /* 0522 */
  {0x0524,0x0525,0x0524},  {0x0524,0x0525,0x0524}, /* 0524 */
  {0x0526,0x0527,0x0526},  {0x0526,0x0527,0x0526}, /* 0526 */
  {0x0528,0x0529,0x0528},  {0x0528,0x0529,0x0528}, /* 0528 */
  {0x052A,0x052B,0x052A},  {0x052A,0x052B,0x052A}, /* 052A */
  {0x052C,0x052D,0x052C},  {0x052C,0x052D,0x052C}, /* 052C */
  {0x052E,0x052F,0x052E},  {0x052E,0x052F,0x052E}, /* 052E */
  {0x0530,0x0530,0x0530},  {0x0531,0x0561,0x0531}, /* 0530 */
  {0x0532,0x0562,0x0532},  {0x0533,0x0563,0x0533}, /* 0532 */
  {0x0534,0x0564,0x0534},  {0x0535,0x0565,0x0535}, /* 0534 */
  {0x0536,0x0566,0x0536},  {0x0537,0x0567,0x0537}, /* 0536 */
  {0x0538,0x0568,0x0538},  {0x0539,0x0569,0x0539}, /* 0538 */
  {0x053A,0x056A,0x053A},  {0x053B,0x056B,0x053B}, /* 053A */
  {0x053C,0x056C,0x053C},  {0x053D,0x056D,0x053D}, /* 053C */
  {0x053E,0x056E,0x053E},  {0x053F,0x056F,0x053F}, /* 053E */
  {0x0540,0x0570,0x0540},  {0x0541,0x0571,0x0541}, /* 0540 */
  {0x0542,0x0572,0x0542},  {0x0543,0x0573,0x0543}, /* 0542 */
  {0x0544,0x0574,0x0544},  {0x0545,0x0575,0x0545}, /* 0544 */
  {0x0546,0x0576,0x0546},  {0x0547,0x0577,0x0547}, /* 0546 */
  {0x0548,0x0578,0x0548},  {0x0549,0x0579,0x0549}, /* 0548 */
  {0x054A,0x057A,0x054A},  {0x054B,0x057B,0x054B}, /* 054A */
  {0x054C,0x057C,0x054C},  {0x054D,0x057D,0x054D}, /* 054C */
  {0x054E,0x057E,0x054E},  {0x054F,0x057F,0x054F}, /* 054E */
  {0x0550,0x0580,0x0550},  {0x0551,0x0581,0x0551}, /* 0550 */
  {0x0552,0x0582,0x0552},  {0x0553,0x0583,0x0553}, /* 0552 */
  {0x0554,0x0584,0x0554},  {0x0555,0x0585,0x0555}, /* 0554 */
  {0x0556,0x0586,0x0556},  {0x0557,0x0557,0x0557}, /* 0556 */
  {0x0558,0x0558,0x0558},  {0x0559,0x0559,0x0559}, /* 0558 */
  {0x055A,0x055A,0x055A},  {0x055B,0x055B,0x055B}, /* 055A */
  {0x055C,0x055C,0x055C},  {0x055D,0x055D,0x055D}, /* 055C */
  {0x055E,0x055E,0x055E},  {0x055F,0x055F,0x055F}, /* 055E */
  {0x0560,0x0560,0x0560},  {0x0531,0x0561,0x0531}, /* 0560 */
  {0x0532,0x0562,0x0532},  {0x0533,0x0563,0x0533}, /* 0562 */
  {0x0534,0x0564,0x0534},  {0x0535,0x0565,0x0535}, /* 0564 */
  {0x0536,0x0566,0x0536},  {0x0537,0x0567,0x0537}, /* 0566 */
  {0x0538,0x0568,0x0538},  {0x0539,0x0569,0x0539}, /* 0568 */
  {0x053A,0x056A,0x053A},  {0x053B,0x056B,0x053B}, /* 056A */
  {0x053C,0x056C,0x053C},  {0x053D,0x056D,0x053D}, /* 056C */
  {0x053E,0x056E,0x053E},  {0x053F,0x056F,0x053F}, /* 056E */
  {0x0540,0x0570,0x0540},  {0x0541,0x0571,0x0541}, /* 0570 */
  {0x0542,0x0572,0x0542},  {0x0543,0x0573,0x0543}, /* 0572 */
  {0x0544,0x0574,0x0544},  {0x0545,0x0575,0x0545}, /* 0574 */
  {0x0546,0x0576,0x0546},  {0x0547,0x0577,0x0547}, /* 0576 */
  {0x0548,0x0578,0x0548},  {0x0549,0x0579,0x0549}, /* 0578 */
  {0x054A,0x057A,0x054A},  {0x054B,0x057B,0x054B}, /* 057A */
  {0x054C,0x057C,0x054C},  {0x054D,0x057D,0x054D}, /* 057C */
  {0x054E,0x057E,0x054E},  {0x054F,0x057F,0x054F}, /* 057E */
  {0x0550,0x0580,0x0550},  {0x0551,0x0581,0x0551}, /* 0580 */
  {0x0552,0x0582,0x0552},  {0x0553,0x0583,0x0553}, /* 0582 */
  {0x0554,0x0584,0x0554},  {0x0555,0x0585,0x0555}, /* 0584 */
  {0x0556,0x0586,0x0556},  {0x0587,0x0587,0x0587}, /* 0586 */
  {0x0588,0x0588,0x0588},  {0x0589,0x0589,0x0589}, /* 0588 */
  {0x058A,0x058A,0x058A},  {0x058B,0x058B,0x058B}, /* 058A */
  {0x058C,0x058C,0x058C},  {0x058D,0x058D,0x058D}, /* 058C */
  {0x058E,0x058E,0x058E},  {0x058F,0x058F,0x058F}, /* 058E */
  {0x0590,0x0590,0x0590},  {0x0591,0x0591,0x0591}, /* 0590 */
  {0x0592,0x0592,0x0592},  {0x0593,0x0593,0x0593}, /* 0592 */
  {0x0594,0x0594,0x0594},  {0x0595,0x0595,0x0595}, /* 0594 */
  {0x0596,0x0596,0x0596},  {0x0597,0x0597,0x0597}, /* 0596 */
  {0x0598,0x0598,0x0598},  {0x0599,0x0599,0x0599}, /* 0598 */
  {0x059A,0x059A,0x059A},  {0x059B,0x059B,0x059B}, /* 059A */
  {0x059C,0x059C,0x059C},  {0x059D,0x059D,0x059D}, /* 059C */
  {0x059E,0x059E,0x059E},  {0x059F,0x059F,0x059F}, /* 059E */
  {0x05A0,0x05A0,0x05A0},  {0x05A1,0x05A1,0x05A1}, /* 05A0 */
  {0x05A2,0x05A2,0x05A2},  {0x05A3,0x05A3,0x05A3}, /* 05A2 */
  {0x05A4,0x05A4,0x05A4},  {0x05A5,0x05A5,0x05A5}, /* 05A4 */
  {0x05A6,0x05A6,0x05A6},  {0x05A7,0x05A7,0x05A7}, /* 05A6 */
  {0x05A8,0x05A8,0x05A8},  {0x05A9,0x05A9,0x05A9}, /* 05A8 */
  {0x05AA,0x05AA,0x05AA},  {0x05AB,0x05AB,0x05AB}, /* 05AA */
  {0x05AC,0x05AC,0x05AC},  {0x05AD,0x05AD,0x05AD}, /* 05AC */
  {0x05AE,0x05AE,0x05AE},  {0x05AF,0x05AF,0x05AF}, /* 05AE */
  {0x05B0,0x05B0,0x05B0},  {0x05B1,0x05B1,0x05B1}, /* 05B0 */
  {0x05B2,0x05B2,0x05B2},  {0x05B3,0x05B3,0x05B3}, /* 05B2 */
  {0x05B4,0x05B4,0x05B4},  {0x05B5,0x05B5,0x05B5}, /* 05B4 */
  {0x05B6,0x05B6,0x05B6},  {0x05B7,0x05B7,0x05B7}, /* 05B6 */
  {0x05B8,0x05B8,0x05B8},  {0x05B9,0x05B9,0x05B9}, /* 05B8 */
  {0x05BA,0x05BA,0x05BA},  {0x05BB,0x05BB,0x05BB}, /* 05BA */
  {0x05BC,0x05BC,0x05BC},  {0x05BD,0x05BD,0x05BD}, /* 05BC */
  {0x05BE,0x05BE,0x05BE},  {0x05BF,0x05BF,0x05BF}, /* 05BE */
  {0x05C0,0x05C0,0x05C0},  {0x05C1,0x05C1,0x05C1}, /* 05C0 */
  {0x05C2,0x05C2,0x05C2},  {0x05C3,0x05C3,0x05C3}, /* 05C2 */
  {0x05C4,0x05C4,0x05C4},  {0x05C5,0x05C5,0x05C5}, /* 05C4 */
  {0x05C6,0x05C6,0x05C6},  {0x05C7,0x05C7,0x05C7}, /* 05C6 */
  {0x05C8,0x05C8,0x05C8},  {0x05C9,0x05C9,0x05C9}, /* 05C8 */
  {0x05CA,0x05CA,0x05CA},  {0x05CB,0x05CB,0x05CB}, /* 05CA */
  {0x05CC,0x05CC,0x05CC},  {0x05CD,0x05CD,0x05CD}, /* 05CC */
  {0x05CE,0x05CE,0x05CE},  {0x05CF,0x05CF,0x05CF}, /* 05CE */
  {0x05D0,0x05D0,0x05D0},  {0x05D1,0x05D1,0x05D1}, /* 05D0 */
  {0x05D2,0x05D2,0x05D2},  {0x05D3,0x05D3,0x05D3}, /* 05D2 */
  {0x05D4,0x05D4,0x05D4},  {0x05D5,0x05D5,0x05D5}, /* 05D4 */
  {0x05D6,0x05D6,0x05D6},  {0x05D7,0x05D7,0x05D7}, /* 05D6 */
  {0x05D8,0x05D8,0x05D8},  {0x05D9,0x05D9,0x05D9}, /* 05D8 */
  {0x05DA,0x05DA,0x05DA},  {0x05DB,0x05DB,0x05DB}, /* 05DA */
  {0x05DC,0x05DC,0x05DC},  {0x05DD,0x05DD,0x05DD}, /* 05DC */
  {0x05DE,0x05DE,0x05DE},  {0x05DF,0x05DF,0x05DF}, /* 05DE */
  {0x05E0,0x05E0,0x05E0},  {0x05E1,0x05E1,0x05E1}, /* 05E0 */
  {0x05E2,0x05E2,0x05E2},  {0x05E3,0x05E3,0x05E3}, /* 05E2 */
  {0x05E4,0x05E4,0x05E4},  {0x05E5,0x05E5,0x05E5}, /* 05E4 */
  {0x05E6,0x05E6,0x05E6},  {0x05E7,0x05E7,0x05E7}, /* 05E6 */
  {0x05E8,0x05E8,0x05E8},  {0x05E9,0x05E9,0x05E9}, /* 05E8 */
  {0x05EA,0x05EA,0x05EA},  {0x05EB,0x05EB,0x05EB}, /* 05EA */
  {0x05EC,0x05EC,0x05EC},  {0x05ED,0x05ED,0x05ED}, /* 05EC */
  {0x05EE,0x05EE,0x05EE},  {0x05EF,0x05EF,0x05EF}, /* 05EE */
  {0x05F0,0x05F0,0x05F0},  {0x05F1,0x05F1,0x05F1}, /* 05F0 */
  {0x05F2,0x05F2,0x05F2},  {0x05F3,0x05F3,0x05F3}, /* 05F2 */
  {0x05F4,0x05F4,0x05F4},  {0x05F5,0x05F5,0x05F5}, /* 05F4 */
  {0x05F6,0x05F6,0x05F6},  {0x05F7,0x05F7,0x05F7}, /* 05F6 */
  {0x05F8,0x05F8,0x05F8},  {0x05F9,0x05F9,0x05F9}, /* 05F8 */
  {0x05FA,0x05FA,0x05FA},  {0x05FB,0x05FB,0x05FB}, /* 05FA */
  {0x05FC,0x05FC,0x05FC},  {0x05FD,0x05FD,0x05FD}, /* 05FC */
  {0x05FE,0x05FE,0x05FE},  {0x05FF,0x05FF,0x05FF}  /* 05FE */
};

static const MY_UNICASE_CHARACTER u900p010[]= {
  {0x1000,0x1000,0x1000},  {0x1001,0x1001,0x1001}, /* 1000 */
  {0x1002,0x1002,0x1002},  {0x1003,0x1003,0x1003}, /* 1002 */
  {0x1004,0x1004,0x1004},  {0x1005,0x1005,0x1005}, /* 1004 */
  {0x1006,0x1006,0x1006},  {0x1007,0x1007,0x1007}, /* 1006 */
  {0x1008,0x1008,0x1008},  {0x1009,0x1009,0x1009}, /* 1008 */
  {0x100A,0x100A,0x100A},  {0x100B,0x100B,0x100B}, /* 100A */
  {0x100C,0x100C,0x100C},  {0x100D,0x100D,0x100D}, /* 100C */
  {0x100E,0x100E,0x100E},  {0x100F,0x100F,0x100F}, /* 100E */
  {0x1010,0x1010,0x1010},  {0x1011,0x1011,0x1011}, /* 1010 */
  {0x1012,0x1012,0x1012},  {0x1013,0x1013,0x1013}, /* 1012 */
  {0x1014,0x1014,0x1014},  {0x1015,0x1015,0x1015}, /* 1014 */
  {0x1016,0x1016,0x1016},  {0x1017,0x1017,0x1017}, /* 1016 */
  {0x1018,0x1018,0x1018},  {0x1019,0x1019,0x1019}, /* 1018 */
  {0x101A,0x101A,0x101A},  {0x101B,0x101B,0x101B}, /* 101A */
  {0x101C,0x101C,0x101C},  {0x101D,0x101D,0x101D}, /* 101C */
  {0x101E,0x101E,0x101E},  {0x101F,0x101F,0x101F}, /* 101E */
  {0x1020,0x1020,0x1020},  {0x1021,0x1021,0x1021}, /* 1020 */
  {0x1022,0x1022,0x1022},  {0x1023,0x1023,0x1023}, /* 1022 */
  {0x1024,0x1024,0x1024},  {0x1025,0x1025,0x1025}, /* 1024 */
  {0x1026,0x1026,0x1026},  {0x1027,0x1027,0x1027}, /* 1026 */
  {0x1028,0x1028,0x1028},  {0x1029,0x1029,0x1029}, /* 1028 */
  {0x102A,0x102A,0x102A},  {0x102B,0x102B,0x102B}, /* 102A */
  {0x102C,0x102C,0x102C},  {0x102D,0x102D,0x102D}, /* 102C */
  {0x102E,0x102E,0x102E},  {0x102F,0x102F,0x102F}, /* 102E */
  {0x1030,0x1030,0x1030},  {0x1031,0x1031,0x1031}, /* 1030 */
  {0x1032,0x1032,0x1032},  {0x1033,0x1033,0x1033}, /* 1032 */
  {0x1034,0x1034,0x1034},  {0x1035,0x1035,0x1035}, /* 1034 */
  {0x1036,0x1036,0x1036},  {0x1037,0x1037,0x1037}, /* 1036 */
  {0x1038,0x1038,0x1038},  {0x1039,0x1039,0x1039}, /* 1038 */
  {0x103A,0x103A,0x103A},  {0x103B,0x103B,0x103B}, /* 103A */
  {0x103C,0x103C,0x103C},  {0x103D,0x103D,0x103D}, /* 103C */
  {0x103E,0x103E,0x103E},  {0x103F,0x103F,0x103F}, /* 103E */
  {0x1040,0x1040,0x1040},  {0x1041,0x1041,0x1041}, /* 1040 */
  {0x1042,0x1042,0x1042},  {0x1043,0x1043,0x1043}, /* 1042 */
  {0x1044,0x1044,0x1044},  {0x1045,0x1045,0x1045}, /* 1044 */
  {0x1046,0x1046,0x1046},  {0x1047,0x1047,0x1047}, /* 1046 */
  {0x1048,0x1048,0x1048},  {0x1049,0x1049,0x1049}, /* 1048 */
  {0x104A,0x104A,0x104A},  {0x104B,0x104B,0x104B}, /* 104A */
  {0x104C,0x104C,0x104C},  {0x104D,0x104D,0x104D}, /* 104C */
  {0x104E,0x104E,0x104E},  {0x104F,0x104F,0x104F}, /* 104E */
  {0x1050,0x1050,0x1050},  {0x1051,0x1051,0x1051}, /* 1050 */
  {0x1052,0x1052,0x1052},  {0x1053,0x1053,0x1053}, /* 1052 */
  {0x1054,0x1054,0x1054},  {0x1055,0x1055,0x1055}, /* 1054 */
  {0x1056,0x1056,0x1056},  {0x1057,0x1057,0x1057}, /* 1056 */
  {0x1058,0x1058,0x1058},  {0x1059,0x1059,0x1059}, /* 1058 */
  {0x105A,0x105A,0x105A},  {0x105B,0x105B,0x105B}, /* 105A */
  {0x105C,0x105C,0x105C},  {0x105D,0x105D,0x105D}, /* 105C */
  {0x105E,0x105E,0x105E},  {0x105F,0x105F,0x105F}, /* 105E */
  {0x1060,0x1060,0x1060},  {0x1061,0x1061,0x1061}, /* 1060 */
  {0x1062,0x1062,0x1062},  {0x1063,0x1063,0x1063}, /* 1062 */
  {0x1064,0x1064,0x1064},  {0x1065,0x1065,0x1065}, /* 1064 */
  {0x1066,0x1066,0x1066},  {0x1067,0x1067,0x1067}, /* 1066 */
  {0x1068,0x1068,0x1068},  {0x1069,0x1069,0x1069}, /* 1068 */
  {0x106A,0x106A,0x106A},  {0x106B,0x106B,0x106B}, /* 106A */
  {0x106C,0x106C,0x106C},  {0x106D,0x106D,0x106D}, /* 106C */
  {0x106E,0x106E,0x106E},  {0x106F,0x106F,0x106F}, /* 106E */
  {0x1070,0x1070,0x1070},  {0x1071,0x1071,0x1071}, /* 1070 */
  {0x1072,0x1072,0x1072},  {0x1073,0x1073,0x1073}, /* 1072 */
  {0x1074,0x1074,0x1074},  {0x1075,0x1075,0x1075}, /* 1074 */
  {0x1076,0x1076,0x1076},  {0x1077,0x1077,0x1077}, /* 1076 */
  {0x1078,0x1078,0x1078},  {0x1079,0x1079,0x1079}, /* 1078 */
  {0x107A,0x107A,0x107A},  {0x107B,0x107B,0x107B}, /* 107A */
  {0x107C,0x107C,0x107C},  {0x107D,0x107D,0x107D}, /* 107C */
  {0x107E,0x107E,0x107E},  {0x107F,0x107F,0x107F}, /* 107E */
  {0x1080,0x1080,0x1080},  {0x1081,0x1081,0x1081}, /* 1080 */
  {0x1082,0x1082,0x1082},  {0x1083,0x1083,0x1083}, /* 1082 */
  {0x1084,0x1084,0x1084},  {0x1085,0x1085,0x1085}, /* 1084 */
  {0x1086,0x1086,0x1086},  {0x1087,0x1087,0x1087}, /* 1086 */
  {0x1088,0x1088,0x1088},  {0x1089,0x1089,0x1089}, /* 1088 */
  {0x108A,0x108A,0x108A},  {0x108B,0x108B,0x108B}, /* 108A */
  {0x108C,0x108C,0x108C},  {0x108D,0x108D,0x108D}, /* 108C */
  {0x108E,0x108E,0x108E},  {0x108F,0x108F,0x108F}, /* 108E */
  {0x1090,0x1090,0x1090},  {0x1091,0x1091,0x1091}, /* 1090 */
  {0x1092,0x1092,0x1092},  {0x1093,0x1093,0x1093}, /* 1092 */
  {0x1094,0x1094,0x1094},  {0x1095,0x1095,0x1095}, /* 1094 */
  {0x1096,0x1096,0x1096},  {0x1097,0x1097,0x1097}, /* 1096 */
  {0x1098,0x1098,0x1098},  {0x1099,0x1099,0x1099}, /* 1098 */
  {0x109A,0x109A,0x109A},  {0x109B,0x109B,0x109B}, /* 109A */
  {0x109C,0x109C,0x109C},  {0x109D,0x109D,0x109D}, /* 109C */
  {0x109E,0x109E,0x109E},  {0x109F,0x109F,0x109F}, /* 109E */
  {0x10A0,0x2D00,0x10A0},  {0x10A1,0x2D01,0x10A1}, /* 10A0 */
  {0x10A2,0x2D02,0x10A2},  {0x10A3,0x2D03,0x10A3}, /* 10A2 */
  {0x10A4,0x2D04,0x10A4},  {0x10A5,0x2D05,0x10A5}, /* 10A4 */
  {0x10A6,0x2D06,0x10A6},  {0x10A7,0x2D07,0x10A7}, /* 10A6 */
  {0x10A8,0x2D08,0x10A8},  {0x10A9,0x2D09,0x10A9}, /* 10A8 */
  {0x10AA,0x2D0A,0x10AA},  {0x10AB,0x2D0B,0x10AB}, /* 10AA */
  {0x10AC,0x2D0C,0x10AC},  {0x10AD,0x2D0D,0x10AD}, /* 10AC */
  {0x10AE,0x2D0E,0x10AE},  {0x10AF,0x2D0F,0x10AF}, /* 10AE */
  {0x10B0,0x2D10,0x10B0},  {0x10B1,0x2D11,0x10B1}, /* 10B0 */
  {0x10B2,0x2D12,0x10B2},  {0x10B3,0x2D13,0x10B3}, /* 10B2 */
  {0x10B4,0x2D14,0x10B4},  {0x10B5,0x2D15,0x10B5}, /* 10B4 */
  {0x10B6,0x2D16,0x10B6},  {0x10B7,0x2D17,0x10B7}, /* 10B6 */
  {0x10B8,0x2D18,0x10B8},  {0x10B9,0x2D19,0x10B9}, /* 10B8 */
  {0x10BA,0x2D1A,0x10BA},  {0x10BB,0x2D1B,0x10BB}, /* 10BA */
  {0x10BC,0x2D1C,0x10BC},  {0x10BD,0x2D1D,0x10BD}, /* 10BC */
  {0x10BE,0x2D1E,0x10BE},  {0x10BF,0x2D1F,0x10BF}, /* 10BE */
  {0x10C0,0x2D20,0x10C0},  {0x10C1,0x2D21,0x10C1}, /* 10C0 */
  {0x10C2,0x2D22,0x10C2},  {0x10C3,0x2D23,0x10C3}, /* 10C2 */
  {0x10C4,0x2D24,0x10C4},  {0x10C5,0x2D25,0x10C5}, /* 10C4 */
  {0x10C6,0x10C6,0x10C6},  {0x10C7,0x2D27,0x10C7}, /* 10C6 */
  {0x10C8,0x10C8,0x10C8},  {0x10C9,0x10C9,0x10C9}, /* 10C8 */
  {0x10CA,0x10CA,0x10CA},  {0x10CB,0x10CB,0x10CB}, /* 10CA */
  {0x10CC,0x10CC,0x10CC},  {0x10CD,0x2D2D,0x10CD}, /* 10CC */
  {0x10CE,0x10CE,0x10CE},  {0x10CF,0x10CF,0x10CF}, /* 10CE */
  {0x10D0,0x10D0,0x10D0},  {0x10D1,0x10D1,0x10D1}, /* 10D0 */
  {0x10D2,0x10D2,0x10D2},  {0x10D3,0x10D3,0x10D3}, /* 10D2 */
  {0x10D4,0x10D4,0x10D4},  {0x10D5,0x10D5,0x10D5}, /* 10D4 */
  {0x10D6,0x10D6,0x10D6},  {0x10D7,0x10D7,0x10D7}, /* 10D6 */
  {0x10D8,0x10D8,0x10D8},  {0x10D9,0x10D9,0x10D9}, /* 10D8 */
  {0x10DA,0x10DA,0x10DA},  {0x10DB,0x10DB,0x10DB}, /* 10DA */
  {0x10DC,0x10DC,0x10DC},  {0x10DD,0x10DD,0x10DD}, /* 10DC */
  {0x10DE,0x10DE,0x10DE},  {0x10DF,0x10DF,0x10DF}, /* 10DE */
  {0x10E0,0x10E0,0x10E0},  {0x10E1,0x10E1,0x10E1}, /* 10E0 */
  {0x10E2,0x10E2,0x10E2},  {0x10E3,0x10E3,0x10E3}, /* 10E2 */
  {0x10E4,0x10E4,0x10E4},  {0x10E5,0x10E5,0x10E5}, /* 10E4 */
  {0x10E6,0x10E6,0x10E6},  {0x10E7,0x10E7,0x10E7}, /* 10E6 */
  {0x10E8,0x10E8,0x10E8},  {0x10E9,0x10E9,0x10E9}, /* 10E8 */
  {0x10EA,0x10EA,0x10EA},  {0x10EB,0x10EB,0x10EB}, /* 10EA */
  {0x10EC,0x10EC,0x10EC},  {0x10ED,0x10ED,0x10ED}, /* 10EC */
  {0x10EE,0x10EE,0x10EE},  {0x10EF,0x10EF,0x10EF}, /* 10EE */
  {0x10F0,0x10F0,0x10F0},  {0x10F1,0x10F1,0x10F1}, /* 10F0 */
  {0x10F2,0x10F2,0x10F2},  {0x10F3,0x10F3,0x10F3}, /* 10F2 */
  {0x10F4,0x10F4,0x10F4},  {0x10F5,0x10F5,0x10F5}, /* 10F4 */
  {0x10F6,0x10F6,0x10F6},  {0x10F7,0x10F7,0x10F7}, /* 10F6 */
  {0x10F8,0x10F8,0x10F8},  {0x10F9,0x10F9,0x10F9}, /* 10F8 */
  {0x10FA,0x10FA,0x10FA},  {0x10FB,0x10FB,0x10FB}, /* 10FA */
  {0x10FC,0x10FC,0x10FC},  {0x10FD,0x10FD,0x10FD}, /* 10FC */
  {0x10FE,0x10FE,0x10FE},  {0x10FF,0x10FF,0x10FF}  /* 10FE */
};

static const MY_UNICASE_CHARACTER u900p013[]= {
  {0x1300,0x1300,0x1300},  {0x1301,0x1301,0x1301}, /* 1300 */
  {0x1302,0x1302,0x1302},  {0x1303,0x1303,0x1303}, /* 1302 */
  {0x1304,0x1304,0x1304},  {0x1305,0x1305,0x1305}, /* 1304 */
  {0x1306,0x1306,0x1306},  {0x1307,0x1307,0x1307}, /* 1306 */
  {0x1308,0x1308,0x1308},  {0x1309,0x1309,0x1309}, /* 1308 */
  {0x130A,0x130A,0x130A},  {0x130B,0x130B,0x130B}, /* 130A */
  {0x130C,0x130C,0x130C},  {0x130D,0x130D,0x130D}, /* 130C */
  {0x130E,0x130E,0x130E},  {0x130F,0x130F,0x130F}, /* 130E */
  {0x1310,0x1310,0x1310},  {0x1311,0x1311,0x1311}, /* 1310 */
  {0x1312,0x1312,0x1312},  {0x1313,0x1313,0x1313}, /* 1312 */
  {0x1314,0x1314,0x1314},  {0x1315,0x1315,0x1315}, /* 1314 */
  {0x1316,0x1316,0x1316},  {0x1317,0x1317,0x1317}, /* 1316 */
  {0x1318,0x1318,0x1318},  {0x1319,0x1319,0x1319}, /* 1318 */
  {0x131A,0x131A,0x131A},  {0x131B,0x131B,0x131B}, /* 131A */
  {0x131C,0x131C,0x131C},  {0x131D,0x131D,0x131D}, /* 131C */
  {0x131E,0x131E,0x131E},  {0x131F,0x131F,0x131F}, /* 131E */
  {0x1320,0x1320,0x1320},  {0x1321,0x1321,0x1321}, /* 1320 */
  {0x1322,0x1322,0x1322},  {0x1323,0x1323,0x1323}, /* 1322 */
  {0x1324,0x1324,0x1324},  {0x1325,0x1325,0x1325}, /* 1324 */
  {0x1326,0x1326,0x1326},  {0x1327,0x1327,0x1327}, /* 1326 */
  {0x1328,0x1328,0x1328},  {0x1329,0x1329,0x1329}, /* 1328 */
  {0x132A,0x132A,0x132A},  {0x132B,0x132B,0x132B}, /* 132A */
  {0x132C,0x132C,0x132C},  {0x132D,0x132D,0x132D}, /* 132C */
  {0x132E,0x132E,0x132E},  {0x132F,0x132F,0x132F}, /* 132E */
  {0x1330,0x1330,0x1330},  {0x1331,0x1331,0x1331}, /* 1330 */
  {0x1332,0x1332,0x1332},  {0x1333,0x1333,0x1333}, /* 1332 */
  {0x1334,0x1334,0x1334},  {0x1335,0x1335,0x1335}, /* 1334 */
  {0x1336,0x1336,0x1336},  {0x1337,0x1337,0x1337}, /* 1336 */
  {0x1338,0x1338,0x1338},  {0x1339,0x1339,0x1339}, /* 1338 */
  {0x133A,0x133A,0x133A},  {0x133B,0x133B,0x133B}, /* 133A */
  {0x133C,0x133C,0x133C},  {0x133D,0x133D,0x133D}, /* 133C */
  {0x133E,0x133E,0x133E},  {0x133F,0x133F,0x133F}, /* 133E */
  {0x1340,0x1340,0x1340},  {0x1341,0x1341,0x1341}, /* 1340 */
  {0x1342,0x1342,0x1342},  {0x1343,0x1343,0x1343}, /* 1342 */
  {0x1344,0x1344,0x1344},  {0x1345,0x1345,0x1345}, /* 1344 */
  {0x1346,0x1346,0x1346},  {0x1347,0x1347,0x1347}, /* 1346 */
  {0x1348,0x1348,0x1348},  {0x1349,0x1349,0x1349}, /* 1348 */
  {0x134A,0x134A,0x134A},  {0x134B,0x134B,0x134B}, /* 134A */
  {0x134C,0x134C,0x134C},  {0x134D,0x134D,0x134D}, /* 134C */
  {0x134E,0x134E,0x134E},  {0x134F,0x134F,0x134F}, /* 134E */
  {0x1350,0x1350,0x1350},  {0x1351,0x1351,0x1351}, /* 1350 */
  {0x1352,0x1352,0x1352},  {0x1353,0x1353,0x1353}, /* 1352 */
  {0x1354,0x1354,0x1354},  {0x1355,0x1355,0x1355}, /* 1354 */
  {0x1356,0x1356,0x1356},  {0x1357,0x1357,0x1357}, /* 1356 */
  {0x1358,0x1358,0x1358},  {0x1359,0x1359,0x1359}, /* 1358 */
  {0x135A,0x135A,0x135A},  {0x135B,0x135B,0x135B}, /* 135A */
  {0x135C,0x135C,0x135C},  {0x135D,0x135D,0x135D}, /* 135C */
  {0x135E,0x135E,0x135E},  {0x135F,0x135F,0x135F}, /* 135E */
  {0x1360,0x1360,0x1360},  {0x1361,0x1361,0x1361}, /* 1360 */
  {0x1362,0x1362,0x1362},  {0x1363,0x1363,0x1363}, /* 1362 */
  {0x1364,0x1364,0x1364},  {0x1365,0x1365,0x1365}, /* 1364 */
  {0x1366,0x1366,0x1366},  {0x1367,0x1367,0x1367}, /* 1366 */
  {0x1368,0x1368,0x1368},  {0x1369,0x1369,0x1369}, /* 1368 */
  {0x136A,0x136A,0x136A},  {0x136B,0x136B,0x136B}, /* 136A */
  {0x136C,0x136C,0x136C},  {0x136D,0x136D,0x136D}, /* 136C */
  {0x136E,0x136E,0x136E},  {0x136F,0x136F,0x136F}, /* 136E */
  {0x1370,0x1370,0x1370},  {0x1371,0x1371,0x1371}, /* 1370 */
  {0x1372,0x1372,0x1372},  {0x1373,0x1373,0x1373}, /* 1372 */
  {0x1374,0x1374,0x1374},  {0x1375,0x1375,0x1375}, /* 1374 */
  {0x1376,0x1376,0x1376},  {0x1377,0x1377,0x1377}, /* 1376 */
  {0x1378,0x1378,0x1378},  {0x1379,0x1379,0x1379}, /* 1378 */
  {0x137A,0x137A,0x137A},  {0x137B,0x137B,0x137B}, /* 137A */
  {0x137C,0x137C,0x137C},  {0x137D,0x137D,0x137D}, /* 137C */
  {0x137E,0x137E,0x137E},  {0x137F,0x137F,0x137F}, /* 137E */
  {0x1380,0x1380,0x1380},  {0x1381,0x1381,0x1381}, /* 1380 */
  {0x1382,0x1382,0x1382},  {0x1383,0x1383,0x1383}, /* 1382 */
  {0x1384,0x1384,0x1384},  {0x1385,0x1385,0x1385}, /* 1384 */
  {0x1386,0x1386,0x1386},  {0x1387,0x1387,0x1387}, /* 1386 */
  {0x1388,0x1388,0x1388},  {0x1389,0x1389,0x1389}, /* 1388 */
  {0x138A,0x138A,0x138A},  {0x138B,0x138B,0x138B}, /* 138A */
  {0x138C,0x138C,0x138C},  {0x138D,0x138D,0x138D}, /* 138C */
  {0x138E,0x138E,0x138E},  {0x138F,0x138F,0x138F}, /* 138E */
  {0x1390,0x1390,0x1390},  {0x1391,0x1391,0x1391}, /* 1390 */
  {0x1392,0x1392,0x1392},  {0x1393,0x1393,0x1393}, /* 1392 */
  {0x1394,0x1394,0x1394},  {0x1395,0x1395,0x1395}, /* 1394 */
  {0x1396,0x1396,0x1396},  {0x1397,0x1397,0x1397}, /* 1396 */
  {0x1398,0x1398,0x1398},  {0x1399,0x1399,0x1399}, /* 1398 */
  {0x139A,0x139A,0x139A},  {0x139B,0x139B,0x139B}, /* 139A */
  {0x139C,0x139C,0x139C},  {0x139D,0x139D,0x139D}, /* 139C */
  {0x139E,0x139E,0x139E},  {0x139F,0x139F,0x139F}, /* 139E */
  {0x13A0,0xAB70,0x13A0},  {0x13A1,0xAB71,0x13A1}, /* 13A0 */
  {0x13A2,0xAB72,0x13A2},  {0x13A3,0xAB73,0x13A3}, /* 13A2 */
  {0x13A4,0xAB74,0x13A4},  {0x13A5,0xAB75,0x13A5}, /* 13A4 */
  {0x13A6,0xAB76,0x13A6},  {0x13A7,0xAB77,0x13A7}, /* 13A6 */
  {0x13A8,0xAB78,0x13A8},  {0x13A9,0xAB79,0x13A9}, /* 13A8 */
  {0x13AA,0xAB7A,0x13AA},  {0x13AB,0xAB7B,0x13AB}, /* 13AA */
  {0x13AC,0xAB7C,0x13AC},  {0x13AD,0xAB7D,0x13AD}, /* 13AC */
  {0x13AE,0xAB7E,0x13AE},  {0x13AF,0xAB7F,0x13AF}, /* 13AE */
  {0x13B0,0xAB80,0x13B0},  {0x13B1,0xAB81,0x13B1}, /* 13B0 */
  {0x13B2,0xAB82,0x13B2},  {0x13B3,0xAB83,0x13B3}, /* 13B2 */
  {0x13B4,0xAB84,0x13B4},  {0x13B5,0xAB85,0x13B5}, /* 13B4 */
  {0x13B6,0xAB86,0x13B6},  {0x13B7,0xAB87,0x13B7}, /* 13B6 */
  {0x13B8,0xAB88,0x13B8},  {0x13B9,0xAB89,0x13B9}, /* 13B8 */
  {0x13BA,0xAB8A,0x13BA},  {0x13BB,0xAB8B,0x13BB}, /* 13BA */
  {0x13BC,0xAB8C,0x13BC},  {0x13BD,0xAB8D,0x13BD}, /* 13BC */
  {0x13BE,0xAB8E,0x13BE},  {0x13BF,0xAB8F,0x13BF}, /* 13BE */
  {0x13C0,0xAB90,0x13C0},  {0x13C1,0xAB91,0x13C1}, /* 13C0 */
  {0x13C2,0xAB92,0x13C2},  {0x13C3,0xAB93,0x13C3}, /* 13C2 */
  {0x13C4,0xAB94,0x13C4},  {0x13C5,0xAB95,0x13C5}, /* 13C4 */
  {0x13C6,0xAB96,0x13C6},  {0x13C7,0xAB97,0x13C7}, /* 13C6 */
  {0x13C8,0xAB98,0x13C8},  {0x13C9,0xAB99,0x13C9}, /* 13C8 */
  {0x13CA,0xAB9A,0x13CA},  {0x13CB,0xAB9B,0x13CB}, /* 13CA */
  {0x13CC,0xAB9C,0x13CC},  {0x13CD,0xAB9D,0x13CD}, /* 13CC */
  {0x13CE,0xAB9E,0x13CE},  {0x13CF,0xAB9F,0x13CF}, /* 13CE */
  {0x13D0,0xABA0,0x13D0},  {0x13D1,0xABA1,0x13D1}, /* 13D0 */
  {0x13D2,0xABA2,0x13D2},  {0x13D3,0xABA3,0x13D3}, /* 13D2 */
  {0x13D4,0xABA4,0x13D4},  {0x13D5,0xABA5,0x13D5}, /* 13D4 */
  {0x13D6,0xABA6,0x13D6},  {0x13D7,0xABA7,0x13D7}, /* 13D6 */
  {0x13D8,0xABA8,0x13D8},  {0x13D9,0xABA9,0x13D9}, /* 13D8 */
  {0x13DA,0xABAA,0x13DA},  {0x13DB,0xABAB,0x13DB}, /* 13DA */
  {0x13DC,0xABAC,0x13DC},  {0x13DD,0xABAD,0x13DD}, /* 13DC */
  {0x13DE,0xABAE,0x13DE},  {0x13DF,0xABAF,0x13DF}, /* 13DE */
  {0x13E0,0xABB0,0x13E0},  {0x13E1,0xABB1,0x13E1}, /* 13E0 */
  {0x13E2,0xABB2,0x13E2},  {0x13E3,0xABB3,0x13E3}, /* 13E2 */
  {0x13E4,0xABB4,0x13E4},  {0x13E5,0xABB5,0x13E5}, /* 13E4 */
  {0x13E6,0xABB6,0x13E6},  {0x13E7,0xABB7,0x13E7}, /* 13E6 */
  {0x13E8,0xABB8,0x13E8},  {0x13E9,0xABB9,0x13E9}, /* 13E8 */
  {0x13EA,0xABBA,0x13EA},  {0x13EB,0xABBB,0x13EB}, /* 13EA */
  {0x13EC,0xABBC,0x13EC},  {0x13ED,0xABBD,0x13ED}, /* 13EC */
  {0x13EE,0xABBE,0x13EE},  {0x13EF,0xABBF,0x13EF}, /* 13EE */
  {0x13F0,0x13F8,0x13F0},  {0x13F1,0x13F9,0x13F1}, /* 13F0 */
  {0x13F2,0x13FA,0x13F2},  {0x13F3,0x13FB,0x13F3}, /* 13F2 */
  {0x13F4,0x13FC,0x13F4},  {0x13F5,0x13FD,0x13F5}, /* 13F4 */
  {0x13F6,0x13F6,0x13F6},  {0x13F7,0x13F7,0x13F7}, /* 13F6 */
  {0x13F0,0x13F8,0x13F0},  {0x13F1,0x13F9,0x13F1}, /* 13F8 */
  {0x13F2,0x13FA,0x13F2},  {0x13F3,0x13FB,0x13F3}, /* 13FA */
  {0x13F4,0x13FC,0x13F4},  {0x13F5,0x13FD,0x13F5}, /* 13FC */
  {0x13FE,0x13FE,0x13FE},  {0x13FF,0x13FF,0x13FF}  /* 13FE */
};

static const MY_UNICASE_CHARACTER u900p01C[]= {
  {0x1C00,0x1C00,0x1C00},  {0x1C01,0x1C01,0x1C01}, /* 1C00 */
  {0x1C02,0x1C02,0x1C02},  {0x1C03,0x1C03,0x1C03}, /* 1C02 */
  {0x1C04,0x1C04,0x1C04},  {0x1C05,0x1C05,0x1C05}, /* 1C04 */
  {0x1C06,0x1C06,0x1C06},  {0x1C07,0x1C07,0x1C07}, /* 1C06 */
  {0x1C08,0x1C08,0x1C08},  {0x1C09,0x1C09,0x1C09}, /* 1C08 */
  {0x1C0A,0x1C0A,0x1C0A},  {0x1C0B,0x1C0B,0x1C0B}, /* 1C0A */
  {0x1C0C,0x1C0C,0x1C0C},  {0x1C0D,0x1C0D,0x1C0D}, /* 1C0C */
  {0x1C0E,0x1C0E,0x1C0E},  {0x1C0F,0x1C0F,0x1C0F}, /* 1C0E */
  {0x1C10,0x1C10,0x1C10},  {0x1C11,0x1C11,0x1C11}, /* 1C10 */
  {0x1C12,0x1C12,0x1C12},  {0x1C13,0x1C13,0x1C13}, /* 1C12 */
  {0x1C14,0x1C14,0x1C14},  {0x1C15,0x1C15,0x1C15}, /* 1C14 */
  {0x1C16,0x1C16,0x1C16},  {0x1C17,0x1C17,0x1C17}, /* 1C16 */
  {0x1C18,0x1C18,0x1C18},  {0x1C19,0x1C19,0x1C19}, /* 1C18 */
  {0x1C1A,0x1C1A,0x1C1A},  {0x1C1B,0x1C1B,0x1C1B}, /* 1C1A */
  {0x1C1C,0x1C1C,0x1C1C},  {0x1C1D,0x1C1D,0x1C1D}, /* 1C1C */
  {0x1C1E,0x1C1E,0x1C1E},  {0x1C1F,0x1C1F,0x1C1F}, /* 1C1E */
  {0x1C20,0x1C20,0x1C20},  {0x1C21,0x1C21,0x1C21}, /* 1C20 */
  {0x1C22,0x1C22,0x1C22},  {0x1C23,0x1C23,0x1C23}, /* 1C22 */
  {0x1C24,0x1C24,0x1C24},  {0x1C25,0x1C25,0x1C25}, /* 1C24 */
  {0x1C26,0x1C26,0x1C26},  {0x1C27,0x1C27,0x1C27}, /* 1C26 */
  {0x1C28,0x1C28,0x1C28},  {0x1C29,0x1C29,0x1C29}, /* 1C28 */
  {0x1C2A,0x1C2A,0x1C2A},  {0x1C2B,0x1C2B,0x1C2B}, /* 1C2A */
  {0x1C2C,0x1C2C,0x1C2C},  {0x1C2D,0x1C2D,0x1C2D}, /* 1C2C */
  {0x1C2E,0x1C2E,0x1C2E},  {0x1C2F,0x1C2F,0x1C2F}, /* 1C2E */
  {0x1C30,0x1C30,0x1C30},  {0x1C31,0x1C31,0x1C31}, /* 1C30 */
  {0x1C32,0x1C32,0x1C32},  {0x1C33,0x1C33,0x1C33}, /* 1C32 */
  {0x1C34,0x1C34,0x1C34},  {0x1C35,0x1C35,0x1C35}, /* 1C34 */
  {0x1C36,0x1C36,0x1C36},  {0x1C37,0x1C37,0x1C37}, /* 1C36 */
  {0x1C38,0x1C38,0x1C38},  {0x1C39,0x1C39,0x1C39}, /* 1C38 */
  {0x1C3A,0x1C3A,0x1C3A},  {0x1C3B,0x1C3B,0x1C3B}, /* 1C3A */
  {0x1C3C,0x1C3C,0x1C3C},  {0x1C3D,0x1C3D,0x1C3D}, /* 1C3C */
  {0x1C3E,0x1C3E,0x1C3E},  {0x1C3F,0x1C3F,0x1C3F}, /* 1C3E */
  {0x1C40,0x1C40,0x1C40},  {0x1C41,0x1C41,0x1C41}, /* 1C40 */
  {0x1C42,0x1C42,0x1C42},  {0x1C43,0x1C43,0x1C43}, /* 1C42 */
  {0x1C44,0x1C44,0x1C44},  {0x1C45,0x1C45,0x1C45}, /* 1C44 */
  {0x1C46,0x1C46,0x1C46},  {0x1C47,0x1C47,0x1C47}, /* 1C46 */
  {0x1C48,0x1C48,0x1C48},  {0x1C49,0x1C49,0x1C49}, /* 1C48 */
  {0x1C4A,0x1C4A,0x1C4A},  {0x1C4B,0x1C4B,0x1C4B}, /* 1C4A */
  {0x1C4C,0x1C4C,0x1C4C},  {0x1C4D,0x1C4D,0x1C4D}, /* 1C4C */
  {0x1C4E,0x1C4E,0x1C4E},  {0x1C4F,0x1C4F,0x1C4F}, /* 1C4E */
  {0x1C50,0x1C50,0x1C50},  {0x1C51,0x1C51,0x1C51}, /* 1C50 */
  {0x1C52,0x1C52,0x1C52},  {0x1C53,0x1C53,0x1C53}, /* 1C52 */
  {0x1C54,0x1C54,0x1C54},  {0x1C55,0x1C55,0x1C55}, /* 1C54 */
  {0x1C56,0x1C56,0x1C56},  {0x1C57,0x1C57,0x1C57}, /* 1C56 */
  {0x1C58,0x1C58,0x1C58},  {0x1C59,0x1C59,0x1C59}, /* 1C58 */
  {0x1C5A,0x1C5A,0x1C5A},  {0x1C5B,0x1C5B,0x1C5B}, /* 1C5A */
  {0x1C5C,0x1C5C,0x1C5C},  {0x1C5D,0x1C5D,0x1C5D}, /* 1C5C */
  {0x1C5E,0x1C5E,0x1C5E},  {0x1C5F,0x1C5F,0x1C5F}, /* 1C5E */
  {0x1C60,0x1C60,0x1C60},  {0x1C61,0x1C61,0x1C61}, /* 1C60 */
  {0x1C62,0x1C62,0x1C62},  {0x1C63,0x1C63,0x1C63}, /* 1C62 */
  {0x1C64,0x1C64,0x1C64},  {0x1C65,0x1C65,0x1C65}, /* 1C64 */
  {0x1C66,0x1C66,0x1C66},  {0x1C67,0x1C67,0x1C67}, /* 1C66 */
  {0x1C68,0x1C68,0x1C68},  {0x1C69,0x1C69,0x1C69}, /* 1C68 */
  {0x1C6A,0x1C6A,0x1C6A},  {0x1C6B,0x1C6B,0x1C6B}, /* 1C6A */
  {0x1C6C,0x1C6C,0x1C6C},  {0x1C6D,0x1C6D,0x1C6D}, /* 1C6C */
  {0x1C6E,0x1C6E,0x1C6E},  {0x1C6F,0x1C6F,0x1C6F}, /* 1C6E */
  {0x1C70,0x1C70,0x1C70},  {0x1C71,0x1C71,0x1C71}, /* 1C70 */
  {0x1C72,0x1C72,0x1C72},  {0x1C73,0x1C73,0x1C73}, /* 1C72 */
  {0x1C74,0x1C74,0x1C74},  {0x1C75,0x1C75,0x1C75}, /* 1C74 */
  {0x1C76,0x1C76,0x1C76},  {0x1C77,0x1C77,0x1C77}, /* 1C76 */
  {0x1C78,0x1C78,0x1C78},  {0x1C79,0x1C79,0x1C79}, /* 1C78 */
  {0x1C7A,0x1C7A,0x1C7A},  {0x1C7B,0x1C7B,0x1C7B}, /* 1C7A */
  {0x1C7C,0x1C7C,0x1C7C},  {0x1C7D,0x1C7D,0x1C7D}, /* 1C7C */
  {0x1C7E,0x1C7E,0x1C7E},  {0x1C7F,0x1C7F,0x1C7F}, /* 1C7E */
  {0x0412,0x1C80,0x0412},  {0x0414,0x1C81,0x0414}, /* 1C80 */
  {0x041E,0x1C82,0x041E},  {0x0421,0x1C83,0x0421}, /* 1C82 */
  {0x0422,0x1C84,0x0422},  {0x0422,0x1C85,0x0422}, /* 1C84 */
  {0x042A,0x1C86,0x042A},  {0x0462,0x1C87,0x0462}, /* 1C86 */
  {0xA64A,0x1C88,0xA64A},  {0x1C89,0x1C89,0x1C89}, /* 1C88 */
  {0x1C8A,0x1C8A,0x1C8A},  {0x1C8B,0x1C8B,0x1C8B}, /* 1C8A */
  {0x1C8C,0x1C8C,0x1C8C},  {0x1C8D,0x1C8D,0x1C8D}, /* 1C8C */
  {0x1C8E,0x1C8E,0x1C8E},  {0x1C8F,0x1C8F,0x1C8F}, /* 1C8E */
  {0x1C90,0x1C90,0x1C90},  {0x1C91,0x1C91,0x1C91}, /* 1C90 */
  {0x1C92,0x1C92,0x1C92},  {0x1C93,0x1C93,0x1C93}, /* 1C92 */
  {0x1C94,0x1C94,0x1C94},  {0x1C95,0x1C95,0x1C95}, /* 1C94 */
  {0x1C96,0x1C96,0x1C96},  {0x1C97,0x1C97,0x1C97}, /* 1C96 */
  {0x1C98,0x1C98,0x1C98},  {0x1C99,0x1C99,0x1C99}, /* 1C98 */
  {0x1C9A,0x1C9A,0x1C9A},  {0x1C9B,0x1C9B,0x1C9B}, /* 1C9A */
  {0x1C9C,0x1C9C,0x1C9C},  {0x1C9D,0x1C9D,0x1C9D}, /* 1C9C */
  {0x1C9E,0x1C9E,0x1C9E},  {0x1C9F,0x1C9F,0x1C9F}, /* 1C9E */
  {0x1CA0,0x1CA0,0x1CA0},  {0x1CA1,0x1CA1,0x1CA1}, /* 1CA0 */
  {0x1CA2,0x1CA2,0x1CA2},  {0x1CA3,0x1CA3,0x1CA3}, /* 1CA2 */
  {0x1CA4,0x1CA4,0x1CA4},  {0x1CA5,0x1CA5,0x1CA5}, /* 1CA4 */
  {0x1CA6,0x1CA6,0x1CA6},  {0x1CA7,0x1CA7,0x1CA7}, /* 1CA6 */
  {0x1CA8,0x1CA8,0x1CA8},  {0x1CA9,0x1CA9,0x1CA9}, /* 1CA8 */
  {0x1CAA,0x1CAA,0x1CAA},  {0x1CAB,0x1CAB,0x1CAB}, /* 1CAA */
  {0x1CAC,0x1CAC,0x1CAC},  {0x1CAD,0x1CAD,0x1CAD}, /* 1CAC */
  {0x1CAE,0x1CAE,0x1CAE},  {0x1CAF,0x1CAF,0x1CAF}, /* 1CAE */
  {0x1CB0,0x1CB0,0x1CB0},  {0x1CB1,0x1CB1,0x1CB1}, /* 1CB0 */
  {0x1CB2,0x1CB2,0x1CB2},  {0x1CB3,0x1CB3,0x1CB3}, /* 1CB2 */
  {0x1CB4,0x1CB4,0x1CB4},  {0x1CB5,0x1CB5,0x1CB5}, /* 1CB4 */
  {0x1CB6,0x1CB6,0x1CB6},  {0x1CB7,0x1CB7,0x1CB7}, /* 1CB6 */
  {0x1CB8,0x1CB8,0x1CB8},  {0x1CB9,0x1CB9,0x1CB9}, /* 1CB8 */
  {0x1CBA,0x1CBA,0x1CBA},  {0x1CBB,0x1CBB,0x1CBB}, /* 1CBA */
  {0x1CBC,0x1CBC,0x1CBC},  {0x1CBD,0x1CBD,0x1CBD}, /* 1CBC */
  {0x1CBE,0x1CBE,0x1CBE},  {0x1CBF,0x1CBF,0x1CBF}, /* 1CBE */
  {0x1CC0,0x1CC0,0x1CC0},  {0x1CC1,0x1CC1,0x1CC1}, /* 1CC0 */
  {0x1CC2,0x1CC2,0x1CC2},  {0x1CC3,0x1CC3,0x1CC3}, /* 1CC2 */
  {0x1CC4,0x1CC4,0x1CC4},  {0x1CC5,0x1CC5,0x1CC5}, /* 1CC4 */
  {0x1CC6,0x1CC6,0x1CC6},  {0x1CC7,0x1CC7,0x1CC7}, /* 1CC6 */
  {0x1CC8,0x1CC8,0x1CC8},  {0x1CC9,0x1CC9,0x1CC9}, /* 1CC8 */
  {0x1CCA,0x1CCA,0x1CCA},  {0x1CCB,0x1CCB,0x1CCB}, /* 1CCA */
  {0x1CCC,0x1CCC,0x1CCC},  {0x1CCD,0x1CCD,0x1CCD}, /* 1CCC */
  {0x1CCE,0x1CCE,0x1CCE},  {0x1CCF,0x1CCF,0x1CCF}, /* 1CCE */
  {0x1CD0,0x1CD0,0x1CD0},  {0x1CD1,0x1CD1,0x1CD1}, /* 1CD0 */
  {0x1CD2,0x1CD2,0x1CD2},  {0x1CD3,0x1CD3,0x1CD3}, /* 1CD2 */
  {0x1CD4,0x1CD4,0x1CD4},  {0x1CD5,0x1CD5,0x1CD5}, /* 1CD4 */
  {0x1CD6,0x1CD6,0x1CD6},  {0x1CD7,0x1CD7,0x1CD7}, /* 1CD6 */
  {0x1CD8,0x1CD8,0x1CD8},  {0x1CD9,0x1CD9,0x1CD9}, /* 1CD8 */
  {0x1CDA,0x1CDA,0x1CDA},  {0x1CDB,0x1CDB,0x1CDB}, /* 1CDA */
  {0x1CDC,0x1CDC,0x1CDC},  {0x1CDD,0x1CDD,0x1CDD}, /* 1CDC */
  {0x1CDE,0x1CDE,0x1CDE},  {0x1CDF,0x1CDF,0x1CDF}, /* 1CDE */
  {0x1CE0,0x1CE0,0x1CE0},  {0x1CE1,0x1CE1,0x1CE1}, /* 1CE0 */
  {0x1CE2,0x1CE2,0x1CE2},  {0x1CE3,0x1CE3,0x1CE3}, /* 1CE2 */
  {0x1CE4,0x1CE4,0x1CE4},  {0x1CE5,0x1CE5,0x1CE5}, /* 1CE4 */
  {0x1CE6,0x1CE6,0x1CE6},  {0x1CE7,0x1CE7,0x1CE7}, /* 1CE6 */
  {0x1CE8,0x1CE8,0x1CE8},  {0x1CE9,0x1CE9,0x1CE9}, /* 1CE8 */
  {0x1CEA,0x1CEA,0x1CEA},  {0x1CEB,0x1CEB,0x1CEB}, /* 1CEA */
  {0x1CEC,0x1CEC,0x1CEC},  {0x1CED,0x1CED,0x1CED}, /* 1CEC */
  {0x1CEE,0x1CEE,0x1CEE},  {0x1CEF,0x1CEF,0x1CEF}, /* 1CEE */
  {0x1CF0,0x1CF0,0x1CF0},  {0x1CF1,0x1CF1,0x1CF1}, /* 1CF0 */
  {0x1CF2,0x1CF2,0x1CF2},  {0x1CF3,0x1CF3,0x1CF3}, /* 1CF2 */
  {0x1CF4,0x1CF4,0x1CF4},  {0x1CF5,0x1CF5,0x1CF5}, /* 1CF4 */
  {0x1CF6,0x1CF6,0x1CF6},  {0x1CF7,0x1CF7,0x1CF7}, /* 1CF6 */
  {0x1CF8,0x1CF8,0x1CF8},  {0x1CF9,0x1CF9,0x1CF9}, /* 1CF8 */
  {0x1CFA,0x1CFA,0x1CFA},  {0x1CFB,0x1CFB,0x1CFB}, /* 1CFA */
  {0x1CFC,0x1CFC,0x1CFC},  {0x1CFD,0x1CFD,0x1CFD}, /* 1CFC */
  {0x1CFE,0x1CFE,0x1CFE},  {0x1CFF,0x1CFF,0x1CFF}  /* 1CFE */
};

static const MY_UNICASE_CHARACTER u900p01D[]= {
  {0x1D00,0x1D00,0x1D00},  {0x1D01,0x1D01,0x1D01}, /* 1D00 */
  {0x1D02,0x1D02,0x1D02},  {0x1D03,0x1D03,0x1D03}, /* 1D02 */
  {0x1D04,0x1D04,0x1D04},  {0x1D05,0x1D05,0x1D05}, /* 1D04 */
  {0x1D06,0x1D06,0x1D06},  {0x1D07,0x1D07,0x1D07}, /* 1D06 */
  {0x1D08,0x1D08,0x1D08},  {0x1D09,0x1D09,0x1D09}, /* 1D08 */
  {0x1D0A,0x1D0A,0x1D0A},  {0x1D0B,0x1D0B,0x1D0B}, /* 1D0A */
  {0x1D0C,0x1D0C,0x1D0C},  {0x1D0D,0x1D0D,0x1D0D}, /* 1D0C */
  {0x1D0E,0x1D0E,0x1D0E},  {0x1D0F,0x1D0F,0x1D0F}, /* 1D0E */
  {0x1D10,0x1D10,0x1D10},  {0x1D11,0x1D11,0x1D11}, /* 1D10 */
  {0x1D12,0x1D12,0x1D12},  {0x1D13,0x1D13,0x1D13}, /* 1D12 */
  {0x1D14,0x1D14,0x1D14},  {0x1D15,0x1D15,0x1D15}, /* 1D14 */
  {0x1D16,0x1D16,0x1D16},  {0x1D17,0x1D17,0x1D17}, /* 1D16 */
  {0x1D18,0x1D18,0x1D18},  {0x1D19,0x1D19,0x1D19}, /* 1D18 */
  {0x1D1A,0x1D1A,0x1D1A},  {0x1D1B,0x1D1B,0x1D1B}, /* 1D1A */
  {0x1D1C,0x1D1C,0x1D1C},  {0x1D1D,0x1D1D,0x1D1D}, /* 1D1C */
  {0x1D1E,0x1D1E,0x1D1E},  {0x1D1F,0x1D1F,0x1D1F}, /* 1D1E */
  {0x1D20,0x1D20,0x1D20},  {0x1D21,0x1D21,0x1D21}, /* 1D20 */
  {0x1D22,0x1D22,0x1D22},  {0x1D23,0x1D23,0x1D23}, /* 1D22 */
  {0x1D24,0x1D24,0x1D24},  {0x1D25,0x1D25,0x1D25}, /* 1D24 */
  {0x1D26,0x1D26,0x1D26},  {0x1D27,0x1D27,0x1D27}, /* 1D26 */
  {0x1D28,0x1D28,0x1D28},  {0x1D29,0x1D29,0x1D29}, /* 1D28 */
  {0x1D2A,0x1D2A,0x1D2A},  {0x1D2B,0x1D2B,0x1D2B}, /* 1D2A */
  {0x1D2C,0x1D2C,0x1D2C},  {0x1D2D,0x1D2D,0x1D2D}, /* 1D2C */
  {0x1D2E,0x1D2E,0x1D2E},  {0x1D2F,0x1D2F,0x1D2F}, /* 1D2E */
  {0x1D30,0x1D30,0x1D30},  {0x1D31,0x1D31,0x1D31}, /* 1D30 */
  {0x1D32,0x1D32,0x1D32},  {0x1D33,0x1D33,0x1D33}, /* 1D32 */
  {0x1D34,0x1D34,0x1D34},  {0x1D35,0x1D35,0x1D35}, /* 1D34 */
  {0x1D36,0x1D36,0x1D36},  {0x1D37,0x1D37,0x1D37}, /* 1D36 */
  {0x1D38,0x1D38,0x1D38},  {0x1D39,0x1D39,0x1D39}, /* 1D38 */
  {0x1D3A,0x1D3A,0x1D3A},  {0x1D3B,0x1D3B,0x1D3B}, /* 1D3A */
  {0x1D3C,0x1D3C,0x1D3C},  {0x1D3D,0x1D3D,0x1D3D}, /* 1D3C */
  {0x1D3E,0x1D3E,0x1D3E},  {0x1D3F,0x1D3F,0x1D3F}, /* 1D3E */
  {0x1D40,0x1D40,0x1D40},  {0x1D41,0x1D41,0x1D41}, /* 1D40 */
  {0x1D42,0x1D42,0x1D42},  {0x1D43,0x1D43,0x1D43}, /* 1D42 */
  {0x1D44,0x1D44,0x1D44},  {0x1D45,0x1D45,0x1D45}, /* 1D44 */
  {0x1D46,0x1D46,0x1D46},  {0x1D47,0x1D47,0x1D47}, /* 1D46 */
  {0x1D48,0x1D48,0x1D48},  {0x1D49,0x1D49,0x1D49}, /* 1D48 */
  {0x1D4A,0x1D4A,0x1D4A},  {0x1D4B,0x1D4B,0x1D4B}, /* 1D4A */
  {0x1D4C,0x1D4C,0x1D4C},  {0x1D4D,0x1D4D,0x1D4D}, /* 1D4C */
  {0x1D4E,0x1D4E,0x1D4E},  {0x1D4F,0x1D4F,0x1D4F}, /* 1D4E */
  {0x1D50,0x1D50,0x1D50},  {0x1D51,0x1D51,0x1D51}, /* 1D50 */
  {0x1D52,0x1D52,0x1D52},  {0x1D53,0x1D53,0x1D53}, /* 1D52 */
  {0x1D54,0x1D54,0x1D54},  {0x1D55,0x1D55,0x1D55}, /* 1D54 */
  {0x1D56,0x1D56,0x1D56},  {0x1D57,0x1D57,0x1D57}, /* 1D56 */
  {0x1D58,0x1D58,0x1D58},  {0x1D59,0x1D59,0x1D59}, /* 1D58 */
  {0x1D5A,0x1D5A,0x1D5A},  {0x1D5B,0x1D5B,0x1D5B}, /* 1D5A */
  {0x1D5C,0x1D5C,0x1D5C},  {0x1D5D,0x1D5D,0x1D5D}, /* 1D5C */
  {0x1D5E,0x1D5E,0x1D5E},  {0x1D5F,0x1D5F,0x1D5F}, /* 1D5E */
  {0x1D60,0x1D60,0x1D60},  {0x1D61,0x1D61,0x1D61}, /* 1D60 */
  {0x1D62,0x1D62,0x1D62},  {0x1D63,0x1D63,0x1D63}, /* 1D62 */
  {0x1D64,0x1D64,0x1D64},  {0x1D65,0x1D65,0x1D65}, /* 1D64 */
  {0x1D66,0x1D66,0x1D66},  {0x1D67,0x1D67,0x1D67}, /* 1D66 */
  {0x1D68,0x1D68,0x1D68},  {0x1D69,0x1D69,0x1D69}, /* 1D68 */
  {0x1D6A,0x1D6A,0x1D6A},  {0x1D6B,0x1D6B,0x1D6B}, /* 1D6A */
  {0x1D6C,0x1D6C,0x1D6C},  {0x1D6D,0x1D6D,0x1D6D}, /* 1D6C */
  {0x1D6E,0x1D6E,0x1D6E},  {0x1D6F,0x1D6F,0x1D6F}, /* 1D6E */
  {0x1D70,0x1D70,0x1D70},  {0x1D71,0x1D71,0x1D71}, /* 1D70 */
  {0x1D72,0x1D72,0x1D72},  {0x1D73,0x1D73,0x1D73}, /* 1D72 */
  {0x1D74,0x1D74,0x1D74},  {0x1D75,0x1D75,0x1D75}, /* 1D74 */
  {0x1D76,0x1D76,0x1D76},  {0x1D77,0x1D77,0x1D77}, /* 1D76 */
  {0x1D78,0x1D78,0x1D78},  {0xA77D,0x1D79,0xA77D}, /* 1D78 */
  {0x1D7A,0x1D7A,0x1D7A},  {0x1D7B,0x1D7B,0x1D7B}, /* 1D7A */
  {0x1D7C,0x1D7C,0x1D7C},  {0x2C63,0x1D7D,0x2C63}, /* 1D7C */
  {0x1D7E,0x1D7E,0x1D7E},  {0x1D7F,0x1D7F,0x1D7F}, /* 1D7E */
  {0x1D80,0x1D80,0x1D80},  {0x1D81,0x1D81,0x1D81}, /* 1D80 */
  {0x1D82,0x1D82,0x1D82},  {0x1D83,0x1D83,0x1D83}, /* 1D82 */
  {0x1D84,0x1D84,0x1D84},  {0x1D85,0x1D85,0x1D85}, /* 1D84 */
  {0x1D86,0x1D86,0x1D86},  {0x1D87,0x1D87,0x1D87}, /* 1D86 */
  {0x1D88,0x1D88,0x1D88},  {0x1D89,0x1D89,0x1D89}, /* 1D88 */
  {0x1D8A,0x1D8A,0x1D8A},  {0x1D8B,0x1D8B,0x1D8B}, /* 1D8A */
  {0x1D8C,0x1D8C,0x1D8C},  {0x1D8D,0x1D8D,0x1D8D}, /* 1D8C */
  {0x1D8E,0x1D8E,0x1D8E},  {0x1D8F,0x1D8F,0x1D8F}, /* 1D8E */
  {0x1D90,0x1D90,0x1D90},  {0x1D91,0x1D91,0x1D91}, /* 1D90 */
  {0x1D92,0x1D92,0x1D92},  {0x1D93,0x1D93,0x1D93}, /* 1D92 */
  {0x1D94,0x1D94,0x1D94},  {0x1D95,0x1D95,0x1D95}, /* 1D94 */
  {0x1D96,0x1D96,0x1D96},  {0x1D97,0x1D97,0x1D97}, /* 1D96 */
  {0x1D98,0x1D98,0x1D98},  {0x1D99,0x1D99,0x1D99}, /* 1D98 */
  {0x1D9A,0x1D9A,0x1D9A},  {0x1D9B,0x1D9B,0x1D9B}, /* 1D9A */
  {0x1D9C,0x1D9C,0x1D9C},  {0x1D9D,0x1D9D,0x1D9D}, /* 1D9C */
  {0x1D9E,0x1D9E,0x1D9E},  {0x1D9F,0x1D9F,0x1D9F}, /* 1D9E */
  {0x1DA0,0x1DA0,0x1DA0},  {0x1DA1,0x1DA1,0x1DA1}, /* 1DA0 */
  {0x1DA2,0x1DA2,0x1DA2},  {0x1DA3,0x1DA3,0x1DA3}, /* 1DA2 */
  {0x1DA4,0x1DA4,0x1DA4},  {0x1DA5,0x1DA5,0x1DA5}, /* 1DA4 */
  {0x1DA6,0x1DA6,0x1DA6},  {0x1DA7,0x1DA7,0x1DA7}, /* 1DA6 */
  {0x1DA8,0x1DA8,0x1DA8},  {0x1DA9,0x1DA9,0x1DA9}, /* 1DA8 */
  {0x1DAA,0x1DAA,0x1DAA},  {0x1DAB,0x1DAB,0x1DAB}, /* 1DAA */
  {0x1DAC,0x1DAC,0x1DAC},  {0x1DAD,0x1DAD,0x1DAD}, /* 1DAC */
  {0x1DAE,0x1DAE,0x1DAE},  {0x1DAF,0x1DAF,0x1DAF}, /* 1DAE */
  {0x1DB0,0x1DB0,0x1DB0},  {0x1DB1,0x1DB1,0x1DB1}, /* 1DB0 */
  {0x1DB2,0x1DB2,0x1DB2},  {0x1DB3,0x1DB3,0x1DB3}, /* 1DB2 */
  {0x1DB4,0x1DB4,0x1DB4},  {0x1DB5,0x1DB5,0x1DB5}, /* 1DB4 */
  {0x1DB6,0x1DB6,0x1DB6},  {0x1DB7,0x1DB7,0x1DB7}, /* 1DB6 */
  {0x1DB8,0x1DB8,0x1DB8},  {0x1DB9,0x1DB9,0x1DB9}, /* 1DB8 */
  {0x1DBA,0x1DBA,0x1DBA},  {0x1DBB,0x1DBB,0x1DBB}, /* 1DBA */
  {0x1DBC,0x1DBC,0x1DBC},  {0x1DBD,0x1DBD,0x1DBD}, /* 1DBC */
  {0x1DBE,0x1DBE,0x1DBE},  {0x1DBF,0x1DBF,0x1DBF}, /* 1DBE */
  {0x1DC0,0x1DC0,0x1DC0},  {0x1DC1,0x1DC1,0x1DC1}, /* 1DC0 */
  {0x1DC2,0x1DC2,0x1DC2},  {0x1DC3,0x1DC3,0x1DC3}, /* 1DC2 */
  {0x1DC4,0x1DC4,0x1DC4},  {0x1DC5,0x1DC5,0x1DC5}, /* 1DC4 */
  {0x1DC6,0x1DC6,0x1DC6},  {0x1DC7,0x1DC7,0x1DC7}, /* 1DC6 */
  {0x1DC8,0x1DC8,0x1DC8},  {0x1DC9,0x1DC9,0x1DC9}, /* 1DC8 */
  {0x1DCA,0x1DCA,0x1DCA},  {0x1DCB,0x1DCB,0x1DCB}, /* 1DCA */
  {0x1DCC,0x1DCC,0x1DCC},  {0x1DCD,0x1DCD,0x1DCD}, /* 1DCC */
  {0x1DCE,0x1DCE,0x1DCE},  {0x1DCF,0x1DCF,0x1DCF}, /* 1DCE */
  {0x1DD0,0x1DD0,0x1DD0},  {0x1DD1,0x1DD1,0x1DD1}, /* 1DD0 */
  {0x1DD2,0x1DD2,0x1DD2},  {0x1DD3,0x1DD3,0x1DD3}, /* 1DD2 */
  {0x1DD4,0x1DD4,0x1DD4},  {0x1DD5,0x1DD5,0x1DD5}, /* 1DD4 */
  {0x1DD6,0x1DD6,0x1DD6},  {0x1DD7,0x1DD7,0x1DD7}, /* 1DD6 */
  {0x1DD8,0x1DD8,0x1DD8},  {0x1DD9,0x1DD9,0x1DD9}, /* 1DD8 */
  {0x1DDA,0x1DDA,0x1DDA},  {0x1DDB,0x1DDB,0x1DDB}, /* 1DDA */
  {0x1DDC,0x1DDC,0x1DDC},  {0x1DDD,0x1DDD,0x1DDD}, /* 1DDC */
  {0x1DDE,0x1DDE,0x1DDE},  {0x1DDF,0x1DDF,0x1DDF}, /* 1DDE */
  {0x1DE0,0x1DE0,0x1DE0},  {0x1DE1,0x1DE1,0x1DE1}, /* 1DE0 */
  {0x1DE2,0x1DE2,0x1DE2},  {0x1DE3,0x1DE3,0x1DE3}, /* 1DE2 */
  {0x1DE4,0x1DE4,0x1DE4},  {0x1DE5,0x1DE5,0x1DE5}, /* 1DE4 */
  {0x1DE6,0x1DE6,0x1DE6},  {0x1DE7,0x1DE7,0x1DE7}, /* 1DE6 */
  {0x1DE8,0x1DE8,0x1DE8},  {0x1DE9,0x1DE9,0x1DE9}, /* 1DE8 */
  {0x1DEA,0x1DEA,0x1DEA},  {0x1DEB,0x1DEB,0x1DEB}, /* 1DEA */
  {0x1DEC,0x1DEC,0x1DEC},  {0x1DED,0x1DED,0x1DED}, /* 1DEC */
  {0x1DEE,0x1DEE,0x1DEE},  {0x1DEF,0x1DEF,0x1DEF}, /* 1DEE */
  {0x1DF0,0x1DF0,0x1DF0},  {0x1DF1,0x1DF1,0x1DF1}, /* 1DF0 */
  {0x1DF2,0x1DF2,0x1DF2},  {0x1DF3,0x1DF3,0x1DF3}, /* 1DF2 */
  {0x1DF4,0x1DF4,0x1DF4},  {0x1DF5,0x1DF5,0x1DF5}, /* 1DF4 */
  {0x1DF6,0x1DF6,0x1DF6},  {0x1DF7,0x1DF7,0x1DF7}, /* 1DF6 */
  {0x1DF8,0x1DF8,0x1DF8},  {0x1DF9,0x1DF9,0x1DF9}, /* 1DF8 */
  {0x1DFA,0x1DFA,0x1DFA},  {0x1DFB,0x1DFB,0x1DFB}, /* 1DFA */
  {0x1DFC,0x1DFC,0x1DFC},  {0x1DFD,0x1DFD,0x1DFD}, /* 1DFC */
  {0x1DFE,0x1DFE,0x1DFE},  {0x1DFF,0x1DFF,0x1DFF}  /* 1DFE */
};

static const MY_UNICASE_CHARACTER u900p01E[]= {
  {0x1E00,0x1E01,0x1E00},  {0x1E00,0x1E01,0x1E00}, /* 1E00 */
  {0x1E02,0x1E03,0x1E02},  {0x1E02,0x1E03,0x1E02}, /* 1E02 */
  {0x1E04,0x1E05,0x1E04},  {0x1E04,0x1E05,0x1E04}, /* 1E04 */
  {0x1E06,0x1E07,0x1E06},  {0x1E06,0x1E07,0x1E06}, /* 1E06 */
  {0x1E08,0x1E09,0x1E08},  {0x1E08,0x1E09,0x1E08}, /* 1E08 */
  {0x1E0A,0x1E0B,0x1E0A},  {0x1E0A,0x1E0B,0x1E0A}, /* 1E0A */
  {0x1E0C,0x1E0D,0x1E0C},  {0x1E0C,0x1E0D,0x1E0C}, /* 1E0C */
  {0x1E0E,0x1E0F,0x1E0E},  {0x1E0E,0x1E0F,0x1E0E}, /* 1E0E */
  {0x1E10,0x1E11,0x1E10},  {0x1E10,0x1E11,0x1E10}, /* 1E10 */
  {0x1E12,0x1E13,0x1E12},  {0x1E12,0x1E13,0x1E12}, /* 1E12 */
  {0x1E14,0x1E15,0x1E14},  {0x1E14,0x1E15,0x1E14}, /* 1E14 */
  {0x1E16,0x1E17,0x1E16},  {0x1E16,0x1E17,0x1E16}, /* 1E16 */
  {0x1E18,0x1E19,0x1E18},  {0x1E18,0x1E19,0x1E18}, /* 1E18 */
  {0x1E1A,0x1E1B,0x1E1A},  {0x1E1A,0x1E1B,0x1E1A}, /* 1E1A */
  {0x1E1C,0x1E1D,0x1E1C},  {0x1E1C,0x1E1D,0x1E1C}, /* 1E1C */
  {0x1E1E,0x1E1F,0x1E1E},  {0x1E1E,0x1E1F,0x1E1E}, /* 1E1E */
  {0x1E20,0x1E21,0x1E20},  {0x1E20,0x1E21,0x1E20}, /* 1E20 */
  {0x1E22,0x1E23,0x1E22},  {0x1E22,0x1E23,0x1E22}, /* 1E22 */
  {0x1E24,0x1E25,0x1E24},  {0x1E24,0x1E25,0x1E24}, /* 1E24 */
  {0x1E26,0x1E27,0x1E26},  {0x1E26,0x1E27,0x1E26}, /* 1E26 */
  {0x1E28,0x1E29,0x1E28},  {0x1E28,0x1E29,0x1E28}, /* 1E28 */
  {0x1E2A,0x1E2B,0x1E2A},  {0x1E2A,0x1E2B,0x1E2A}, /* 1E2A */
  {0x1E2C,0x1E2D,0x1E2C},  {0x1E2C,0x1E2D,0x1E2C}, /* 1E2C */
  {0x1E2E,0x1E2F,0x1E2E},  {0x1E2E,0x1E2F,0x1E2E}, /* 1E2E */
  {0x1E30,0x1E31,0x1E30},  {0x1E30,0x1E31,0x1E30}, /* 1E30 */
  {0x1E32,0x1E33,0x1E32},  {0x1E32,0x1E33,0x1E32}, /* 1E32 */
  {0x1E34,0x1E35,0x1E34},  {0x1E34,0x1E35,0x1E34}, /* 1E34 */
  {0x1E36,0x1E37,0x1E36},  {0x1E36,0x1E37,0x1E36}, /* 1E36 */
  {0x1E38,0x1E39,0x1E38},  {0x1E38,0x1E39,0x1E38}, /* 1E38 */
  {0x1E3A,0x1E3B,0x1E3A},  {0x1E3A,0x1E3B,0x1E3A}, /* 1E3A */
  {0x1E3C,0x1E3D,0x1E3C},  {0x1E3C,0x1E3D,0x1E3C}, /* 1E3C */
  {0x1E3E,0x1E3F,0x1E3E},  {0x1E3E,0x1E3F,0x1E3E}, /* 1E3E */
  {0x1E40,0x1E41,0x1E40},  {0x1E40,0x1E41,0x1E40}, /* 1E40 */
  {0x1E42,0x1E43,0x1E42},  {0x1E42,0x1E43,0x1E42}, /* 1E42 */
  {0x1E44,0x1E45,0x1E44},  {0x1E44,0x1E45,0x1E44}, /* 1E44 */
  {0x1E46,0x1E47,0x1E46},  {0x1E46,0x1E47,0x1E46}, /* 1E46 */
  {0x1E48,0x1E49,0x1E48},  {0x1E48,0x1E49,0x1E48}, /* 1E48 */
  {0x1E4A,0x1E4B,0x1E4A},  {0x1E4A,0x1E4B,0x1E4A}, /* 1E4A */
  {0x1E4C,0x1E4D,0x1E4C},  {0x1E4C,0x1E4D,0x1E4C}, /* 1E4C */
  {0x1E4E,0x1E4F,0x1E4E},  {0x1E4E,0x1E4F,0x1E4E}, /* 1E4E */
  {0x1E50,0x1E51,0x1E50},  {0x1E50,0x1E51,0x1E50}, /* 1E50 */
  {0x1E52,0x1E53,0x1E52},  {0x1E52,0x1E53,0x1E52}, /* 1E52 */
  {0x1E54,0x1E55,0x1E54},  {0x1E54,0x1E55,0x1E54}, /* 1E54 */
  {0x1E56,0x1E57,0x1E56},  {0x1E56,0x1E57,0x1E56}, /* 1E56 */
  {0x1E58,0x1E59,0x1E58},  {0x1E58,0x1E59,0x1E58}, /* 1E58 */
  {0x1E5A,0x1E5B,0x1E5A},  {0x1E5A,0x1E5B,0x1E5A}, /* 1E5A */
  {0x1E5C,0x1E5D,0x1E5C},  {0x1E5C,0x1E5D,0x1E5C}, /* 1E5C */
  {0x1E5E,0x1E5F,0x1E5E},  {0x1E5E,0x1E5F,0x1E5E}, /* 1E5E */
  {0x1E60,0x1E61,0x1E60},  {0x1E60,0x1E61,0x1E60}, /* 1E60 */
  {0x1E62,0x1E63,0x1E62},  {0x1E62,0x1E63,0x1E62}, /* 1E62 */
  {0x1E64,0x1E65,0x1E64},  {0x1E64,0x1E65,0x1E64}, /* 1E64 */
  {0x1E66,0x1E67,0x1E66},  {0x1E66,0x1E67,0x1E66}, /* 1E66 */
  {0x1E68,0x1E69,0x1E68},  {0x1E68,0x1E69,0x1E68}, /* 1E68 */
  {0x1E6A,0x1E6B,0x1E6A},  {0x1E6A,0x1E6B,0x1E6A}, /* 1E6A */
  {0x1E6C,0x1E6D,0x1E6C},  {0x1E6C,0x1E6D,0x1E6C}, /* 1E6C */
  {0x1E6E,0x1E6F,0x1E6E},  {0x1E6E,0x1E6F,0x1E6E}, /* 1E6E */
  {0x1E70,0x1E71,0x1E70},  {0x1E70,0x1E71,0x1E70}, /* 1E70 */
  {0x1E72,0x1E73,0x1E72},  {0x1E72,0x1E73,0x1E72}, /* 1E72 */
  {0x1E74,0x1E75,0x1E74},  {0x1E74,0x1E75,0x1E74}, /* 1E74 */
  {0x1E76,0x1E77,0x1E76},  {0x1E76,0x1E77,0x1E76}, /* 1E76 */
  {0x1E78,0x1E79,0x1E78},  {0x1E78,0x1E79,0x1E78}, /* 1E78 */
  {0x1E7A,0x1E7B,0x1E7A},  {0x1E7A,0x1E7B,0x1E7A}, /* 1E7A */
  {0x1E7C,0x1E7D,0x1E7C},  {0x1E7C,0x1E7D,0x1E7C}, /* 1E7C */
  {0x1E7E,0x1E7F,0x1E7E},  {0x1E7E,0x1E7F,0x1E7E}, /* 1E7E */
  {0x1E80,0x1E81,0x1E80},  {0x1E80,0x1E81,0x1E80}, /* 1E80 */
  {0x1E82,0x1E83,0x1E82},  {0x1E82,0x1E83,0x1E82}, /* 1E82 */
  {0x1E84,0x1E85,0x1E84},  {0x1E84,0x1E85,0x1E84}, /* 1E84 */
  {0x1E86,0x1E87,0x1E86},  {0x1E86,0x1E87,0x1E86}, /* 1E86 */
  {0x1E88,0x1E89,0x1E88},  {0x1E88,0x1E89,0x1E88}, /* 1E88 */
  {0x1E8A,0x1E8B,0x1E8A},  {0x1E8A,0x1E8B,0x1E8A}, /* 1E8A */
  {0x1E8C,0x1E8D,0x1E8C},  {0x1E8C,0x1E8D,0x1E8C}, /* 1E8C */
  {0x1E8E,0x1E8F,0x1E8E},  {0x1E8E,0x1E8F,0x1E8E}, /* 1E8E */
  {0x1E90,0x1E91,0x1E90},  {0x1E90,0x1E91,0x1E90}, /* 1E90 */
  {0x1E92,0x1E93,0x1E92},  {0x1E92,0x1E93,0x1E92}, /* 1E92 */
  {0x1E94,0x1E95,0x1E94},  {0x1E94,0x1E95,0x1E94}, /* 1E94 */
  {0x1E96,0x1E96,0x1E96},  {0x1E97,0x1E97,0x1E97}, /* 1E96 */
  {0x1E98,0x1E98,0x1E98},  {0x1E99,0x1E99,0x1E99}, /* 1E98 */
  {0x1E9A,0x1E9A,0x1E9A},  {0x1E60,0x1E9B,0x1E60}, /* 1E9A */
  {0x1E9C,0x1E9C,0x1E9C},  {0x1E9D,0x1E9D,0x1E9D}, /* 1E9C */
  {0x1E9E,0x00DF,0x1E9E},  {0x1E9F,0x1E9F,0x1E9F}, /* 1E9E */
  {0x1EA0,0x1EA1,0x1EA0},  {0x1EA0,0x1EA1,0x1EA0}, /* 1EA0 */
  {0x1EA2,0x1EA3,0x1EA2},  {0x1EA2,0x1EA3,0x1EA2}, /* 1EA2 */
  {0x1EA4,0x1EA5,0x1EA4},  {0x1EA4,0x1EA5,0x1EA4}, /* 1EA4 */
  {0x1EA6,0x1EA7,0x1EA6},  {0x1EA6,0x1EA7,0x1EA6}, /* 1EA6 */
  {0x1EA8,0x1EA9,0x1EA8},  {0x1EA8,0x1EA9,0x1EA8}, /* 1EA8 */
  {0x1EAA,0x1EAB,0x1EAA},  {0x1EAA,0x1EAB,0x1EAA}, /* 1EAA */
  {0x1EAC,0x1EAD,0x1EAC},  {0x1EAC,0x1EAD,0x1EAC}, /* 1EAC */
  {0x1EAE,0x1EAF,0x1EAE},  {0x1EAE,0x1EAF,0x1EAE}, /* 1EAE */
  {0x1EB0,0x1EB1,0x1EB0},  {0x1EB0,0x1EB1,0x1EB0}, /* 1EB0 */
  {0x1EB2,0x1EB3,0x1EB2},  {0x1EB2,0x1EB3,0x1EB2}, /* 1EB2 */
  {0x1EB4,0x1EB5,0x1EB4},  {0x1EB4,0x1EB5,0x1EB4}, /* 1EB4 */
  {0x1EB6,0x1EB7,0x1EB6},  {0x1EB6,0x1EB7,0x1EB6}, /* 1EB6 */
  {0x1EB8,0x1EB9,0x1EB8},  {0x1EB8,0x1EB9,0x1EB8}, /* 1EB8 */
  {0x1EBA,0x1EBB,0x1EBA},  {0x1EBA,0x1EBB,0x1EBA}, /* 1EBA */
  {0x1EBC,0x1EBD,0x1EBC},  {0x1EBC,0x1EBD,0x1EBC}, /* 1EBC */
  {0x1EBE,0x1EBF,0x1EBE},  {0x1EBE,0x1EBF,0x1EBE}, /* 1EBE */
  {0x1EC0,0x1EC1,0x1EC0},  {0x1EC0,0x1EC1,0x1EC0}, /* 1EC0 */
  {0x1EC2,0x1EC3,0x1EC2},  {0x1EC2,0x1EC3,0x1EC2}, /* 1EC2 */
  {0x1EC4,0x1EC5,0x1EC4},  {0x1EC4,0x1EC5,0x1EC4}, /* 1EC4 */
  {0x1EC6,0x1EC7,0x1EC6},  {0x1EC6,0x1EC7,0x1EC6}, /* 1EC6 */
  {0x1EC8,0x1EC9,0x1EC8},  {0x1EC8,0x1EC9,0x1EC8}, /* 1EC8 */
  {0x1ECA,0x1ECB,0x1ECA},  {0x1ECA,0x1ECB,0x1ECA}, /* 1ECA */
  {0x1ECC,0x1ECD,0x1ECC},  {0x1ECC,0x1ECD,0x1ECC}, /* 1ECC */
  {0x1ECE,0x1ECF,0x1ECE},  {0x1ECE,0x1ECF,0x1ECE}, /* 1ECE */
  {0x1ED0,0x1ED1,0x1ED0},  {0x1ED0,0x1ED1,0x1ED0}, /* 1ED0 */
  {0x1ED2,0x1ED3,0x1ED2},  {0x1ED2,0x1ED3,0x1ED2}, /* 1ED2 */
  {0x1ED4,0x1ED5,0x1ED4},  {0x1ED4,0x1ED5,0x1ED4}, /* 1ED4 */
  {0x1ED6,0x1ED7,0x1ED6},  {0x1ED6,0x1ED7,0x1ED6}, /* 1ED6 */
  {0x1ED8,0x1ED9,0x1ED8},  {0x1ED8,0x1ED9,0x1ED8}, /* 1ED8 */
  {0x1EDA,0x1EDB,0x1EDA},  {0x1EDA,0x1EDB,0x1EDA}, /* 1EDA */
  {0x1EDC,0x1EDD,0x1EDC},  {0x1EDC,0x1EDD,0x1EDC}, /* 1EDC */
  {0x1EDE,0x1EDF,0x1EDE},  {0x1EDE,0x1EDF,0x1EDE}, /* 1EDE */
  {0x1EE0,0x1EE1,0x1EE0},  {0x1EE0,0x1EE1,0x1EE0}, /* 1EE0 */
  {0x1EE2,0x1EE3,0x1EE2},  {0x1EE2,0x1EE3,0x1EE2}, /* 1EE2 */
  {0x1EE4,0x1EE5,0x1EE4},  {0x1EE4,0x1EE5,0x1EE4}, /* 1EE4 */
  {0x1EE6,0x1EE7,0x1EE6},  {0x1EE6,0x1EE7,0x1EE6}, /* 1EE6 */
  {0x1EE8,0x1EE9,0x1EE8},  {0x1EE8,0x1EE9,0x1EE8}, /* 1EE8 */
  {0x1EEA,0x1EEB,0x1EEA},  {0x1EEA,0x1EEB,0x1EEA}, /* 1EEA */
  {0x1EEC,0x1EED,0x1EEC},  {0x1EEC,0x1EED,0x1EEC}, /* 1EEC */
  {0x1EEE,0x1EEF,0x1EEE},  {0x1EEE,0x1EEF,0x1EEE}, /* 1EEE */
  {0x1EF0,0x1EF1,0x1EF0},  {0x1EF0,0x1EF1,0x1EF0}, /* 1EF0 */
  {0x1EF2,0x1EF3,0x1EF2},  {0x1EF2,0x1EF3,0x1EF2}, /* 1EF2 */
  {0x1EF4,0x1EF5,0x1EF4},  {0x1EF4,0x1EF5,0x1EF4}, /* 1EF4 */
  {0x1EF6,0x1EF7,0x1EF6},  {0x1EF6,0x1EF7,0x1EF6}, /* 1EF6 */
  {0x1EF8,0x1EF9,0x1EF8},  {0x1EF8,0x1EF9,0x1EF8}, /* 1EF8 */
  {0x1EFA,0x1EFB,0x1EFA},  {0x1EFA,0x1EFB,0x1EFA}, /* 1EFA */
  {0x1EFC,0x1EFD,0x1EFC},  {0x1EFC,0x1EFD,0x1EFC}, /* 1EFC */
  {0x1EFE,0x1EFF,0x1EFE},  {0x1EFE,0x1EFF,0x1EFE}  /* 1EFE */
};

static const MY_UNICASE_CHARACTER u900p01F[]= {
  {0x1F08,0x1F00,0x1F08},  {0x1F09,0x1F01,0x1F09}, /* 1F00 */
  {0x1F0A,0x1F02,0x1F0A},  {0x1F0B,0x1F03,0x1F0B}, /* 1F02 */
  {0x1F0C,0x1F04,0x1F0C},  {0x1F0D,0x1F05,0x1F0D}, /* 1F04 */
  {0x1F0E,0x1F06,0x1F0E},  {0x1F0F,0x1F07,0x1F0F}, /* 1F06 */
  {0x1F08,0x1F00,0x1F08},  {0x1F09,0x1F01,0x1F09}, /* 1F08 */
  {0x1F0A,0x1F02,0x1F0A},  {0x1F0B,0x1F03,0x1F0B}, /* 1F0A */
  {0x1F0C,0x1F04,0x1F0C},  {0x1F0D,0x1F05,0x1F0D}, /* 1F0C */
  {0x1F0E,0x1F06,0x1F0E},  {0x1F0F,0x1F07,0x1F0F}, /* 1F0E */
  {0x1F18,0x1F10,0x1F18},  {0x1F19,0x1F11,0x1F19}, /* 1F10 */
  {0x1F1A,0x1F12,0x1F1A},  {0x1F1B,0x1F13,0x1F1B}, /* 1F12 */
  {0x1F1C,0x1F14,0x1F1C},  {0x1F1D,0x1F15,0x1F1D}, /* 1F14 */
  {0x1F16,0x1F16,0x1F16},  {0x1F17,0x1F17,0x1F17}, /* 1F16 */
  {0x1F18,0x1F10,0x1F18},  {0x1F19,0x1F11,0x1F19}, /* 1F18 */
  {0x1F1A,0x1F12,0x1F1A},  {0x1F1B,0x1F13,0x1F1B}, /* 1F1A */
  {0x1F1C,0x1F14,0x1F1C},  {0x1F1D,0x1F15,0x1F1D}, /* 1F1C */
  {0x1F1E,0x1F1E,0x1F1E},  {0x1F1F,0x1F1F,0x1F1F}, /* 1F1E */
  {0x1F28,0x1F20,0x1F28},  {0x1F29,0x1F21,0x1F29}, /* 1F20 */
  {0x1F2A,0x1F22,0x1F2A},  {0x1F2B,0x1F23,0x1F2B}, /* 1F22 */
  {0x1F2C,0x1F24,0x1F2C},  {0x1F2D,0x1F25,0x1F2D}, /* 1F24 */
  {0x1F2E,0x1F26,0x1F2E},  {0x1F2F,0x1F27,0x1F2F}, /* 1F26 */
  {0x1F28,0x1F20,0x1F28},  {0x1F29,0x1F21,0x1F29}, /* 1F28 */
  {0x1F2A,0x1F22,0x1F2A},  {0x1F2B,0x1F23,0x1F2B}, /* 1F2A */
  {0x1F2C,0x1F24,0x1F2C},  {0x1F2D,0x1F25,0x1F2D}, /* 1F2C */
  {0x1F2E,0x1F26,0x1F2E},  {0x1F2F,0x1F27,0x1F2F}, /* 1F2E */
  {0x1F38,0x1F30,0x1F38},  {0x1F39,0x1F31,0x1F39}, /* 1F30 */
  {0x1F3A,0x1F32,0x1F3A},  {0x1F3B,0x1F33,0x1F3B}, /* 1F32 */
  {0x1F3C,0x1F34,0x1F3C},  {0x1F3D,0x1F35,0x1F3D}, /* 1F34 */
  {0x1F3E,0x1F36,0x1F3E},  {0x1F3F,0x1F37,0x1F3F}, /* 1F36 */
  {0x1F38,0x1F30,0x1F38},  {0x1F39,0x1F31,0x1F39}, /* 1F38 */
  {0x1F3A,0x1F32,0x1F3A},  {0x1F3B,0x1F33,0x1F3B}, /* 1F3A */
  {0x1F3C,0x1F34,0x1F3C},  {0x1F3D,0x1F35,0x1F3D}, /* 1F3C */
  {0x1F3E,0x1F36,0x1F3E},  {0x1F3F,0x1F37,0x1F3F}, /* 1F3E */
  {0x1F48,0x1F40,0x1F48},  {0x1F49,0x1F41,0x1F49}, /* 1F40 */
  {0x1F4A,0x1F42,0x1F4A},  {0x1F4B,0x1F43,0x1F4B}, /* 1F42 */
  {0x1F4C,0x1F44,0x1F4C},  {0x1F4D,0x1F45,0x1F4D}, /* 1F44 */
  {0x1F46,0x1F46,0x1F46},  {0x1F47,0x1F47,0x1F47}, /* 1F46 */
  {0x1F48,0x1F40,0x1F48},  {0x1F49,0x1F41,0x1F49}, /* 1F48 */
  {0x1F4A,0x1F42,0x1F4A},  {0x1F4B,0x1F43,0x1F4B}, /* 1F4A */
  {0x1F4C,0x1F44,0x1F4C},  {0x1F4D,0x1F45,0x1F4D}, /* 1F4C */
  {0x1F4E,0x1F4E,0x1F4E},  {0x1F4F,0x1F4F,0x1F4F}, /* 1F4E */
  {0x1F50,0x1F50,0x1F50},  {0x1F59,0x1F51,0x1F59}, /* 1F50 */
  {0x1F52,0x1F52,0x1F52},  {0x1F5B,0x1F53,0x1F5B}, /* 1F52 */
  {0x1F54,0x1F54,0x1F54},  {0x1F5D,0x1F55,0x1F5D}, /* 1F54 */
  {0x1F56,0x1F56,0x1F56},  {0x1F5F,0x1F57,0x1F5F}, /* 1F56 */
  {0x1F58,0x1F58,0x1F58},  {0x1F59,0x1F51,0x1F59}, /* 1F58 */
  {0x1F5A,0x1F5A,0x1F5A},  {0x1F5B,0x1F53,0x1F5B}, /* 1F5A */
  {0x1F5C,0x1F5C,0x1F5C},  {0x1F5D,0x1F55,0x1F5D}, /* 1F5C */
  {0x1F5E,0x1F5E,0x1F5E},  {0x1F5F,0x1F57,0x1F5F}, /* 1F5E */
  {0x1F68,0x1F60,0x1F68},  {0x1F69,0x1F61,0x1F69}, /* 1F60 */
  {0x1F6A,0x1F62,0x1F6A},  {0x1F6B,0x1F63,0x1F6B}, /* 1F62 */
  {0x1F6C,0x1F64,0x1F6C},  {0x1F6D,0x1F65,0x1F6D}, /* 1F64 */
  {0x1F6E,0x1F66,0x1F6E},  {0x1F6F,0x1F67,0x1F6F}, /* 1F66 */
  {0x1F68,0x1F60,0x1F68},  {0x1F69,0x1F61,0x1F69}, /* 1F68 */
  {0x1F6A,0x1F62,0x1F6A},  {0x1F6B,0x1F63,0x1F6B}, /* 1F6A */
  {0x1F6C,0x1F64,0x1F6C},  {0x1F6D,0x1F65,0x1F6D}, /* 1F6C */
  {0x1F6E,0x1F66,0x1F6E},  {0x1F6F,0x1F67,0x1F6F}, /* 1F6E */
  {0x1FBA,0x1F70,0x1FBA},  {0x1FBB,0x1F71,0x1FBB}, /* 1F70 */
  {0x1FC8,0x1F72,0x1FC8},  {0x1FC9,0x1F73,0x1FC9}, /* 1F72 */
  {0x1FCA,0x1F74,0x1FCA},  {0x1FCB,0x1F75,0x1FCB}, /* 1F74 */
  {0x1FDA,0x1F76,0x1FDA},  {0x1FDB,0x1F77,0x1FDB}, /* 1F76 */
  {0x1FF8,0x1F78,0x1FF8},  {0x1FF9,0x1F79,0x1FF9}, /* 1F78 */
  {0x1FEA,0x1F7A,0x1FEA},  {0x1FEB,0x1F7B,0x1FEB}, /* 1F7A */
  {0x1FFA,0x1F7C,0x1FFA},  {0x1FFB,0x1F7D,0x1FFB}, /* 1F7C */
  {0x1F7E,0x1F7E,0x1F7E},  {0x1F7F,0x1F7F,0x1F7F}, /* 1F7E */
  {0x1F88,0x1F80,0x1F88},  {0x1F89,0x1F81,0x1F89}, /* 1F80 */
  {0x1F8A,0x1F82,0x1F8A},  {0x1F8B,0x1F83,0x1F8B}, /* 1F82 */
  {0x1F8C,0x1F84,0x1F8C},  {0x1F8D,0x1F85,0x1F8D}, /* 1F84 */
  {0x1F8E,0x1F86,0x1F8E},  {0x1F8F,0x1F87,0x1F8F}, /* 1F86 */
  {0x1F88,0x1F80,0x1F88},  {0x1F89,0x1F81,0x1F89}, /* 1F88 */
  {0x1F8A,0x1F82,0x1F8A},  {0x1F8B,0x1F83,0x1F8B}, /* 1F8A */
  {0x1F8C,0x1F84,0x1F8C},  {0x1F8D,0x1F85,0x1F8D}, /* 1F8C */
  {0x1F8E,0x1F86,0x1F8E},  {0x1F8F,0x1F87,0x1F8F}, /* 1F8E */
  {0x1F98,0x1F90,0x1F98},  {0x1F99,0x1F91,0x1F99}, /* 1F90 */
  {0x1F9A,0x1F92,0x1F9A},  {0x1F9B,0x1F93,0x1F9B}, /* 1F92 */
  {0x1F9C,0x1F94,0x1F9C},  {0x1F9D,0x1F95,0x1F9D}, /* 1F94 */
  {0x1F9E,0x1F96,0x1F9E},  {0x1F9F,0x1F97,0x1F9F}, /* 1F96 */
  {0x1F98,0x1F90,0x1F98},  {0x1F99,0x1F91,0x1F99}, /* 1F98 */
  {0x1F9A,0x1F92,0x1F9A},  {0x1F9B,0x1F93,0x1F9B}, /* 1F9A */
  {0x1F9C,0x1F94,0x1F9C},  {0x1F9D,0x1F95,0x1F9D}, /* 1F9C */
  {0x1F9E,0x1F96,0x1F9E},  {0x1F9F,0x1F97,0x1F9F}, /* 1F9E */
  {0x1FA8,0x1FA0,0x1FA8},  {0x1FA9,0x1FA1,0x1FA9}, /* 1FA0 */
  {0x1FAA,0x1FA2,0x1FAA},  {0x1FAB,0x1FA3,0x1FAB}, /* 1FA2 */
  {0x1FAC,0x1FA4,0x1FAC},  {0x1FAD,0x1FA5,0x1FAD}, /* 1FA4 */
  {0x1FAE,0x1FA6,0x1FAE},  {0x1FAF,0x1FA7,0x1FAF}, /* 1FA6 */
  {0x1FA8,0x1FA0,0x1FA8},  {0x1FA9,0x1FA1,0x1FA9}, /* 1FA8 */
  {0x1FAA,0x1FA2,0x1FAA},  {0x1FAB,0x1FA3,0x1FAB}, /* 1FAA */
  {0x1FAC,0x1FA4,0x1FAC},  {0x1FAD,0x1FA5,0x1FAD}, /* 1FAC */
  {0x1FAE,0x1FA6,0x1FAE},  {0x1FAF,0x1FA7,0x1FAF}, /* 1FAE */
  {0x1FB8,0x1FB0,0x1FB8},  {0x1FB9,0x1FB1,0x1FB9}, /* 1FB0 */
  {0x1FB2,0x1FB2,0x1FB2},  {0x1FBC,0x1FB3,0x1FBC}, /* 1FB2 */
  {0x1FB4,0x1FB4,0x1FB4},  {0x1FB5,0x1FB5,0x1FB5}, /* 1FB4 */
  {0x1FB6,0x1FB6,0x1FB6},  {0x1FB7,0x1FB7,0x1FB7}, /* 1FB6 */
  {0x1FB8,0x1FB0,0x1FB8},  {0x1FB9,0x1FB1,0x1FB9}, /* 1FB8 */
  {0x1FBA,0x1F70,0x1FBA},  {0x1FBB,0x1F71,0x1FBB}, /* 1FBA */
  {0x1FBC,0x1FB3,0x1FBC},  {0x1FBD,0x1FBD,0x1FBD}, /* 1FBC */
  {0x0399,0x1FBE,0x0399},  {0x1FBF,0x1FBF,0x1FBF}, /* 1FBE */
  {0x1FC0,0x1FC0,0x1FC0},  {0x1FC1,0x1FC1,0x1FC1}, /* 1FC0 */
  {0x1FC2,0x1FC2,0x1FC2},  {0x1FCC,0x1FC3,0x1FCC}, /* 1FC2 */
  {0x1FC4,0x1FC4,0x1FC4},  {0x1FC5,0x1FC5,0x1FC5}, /* 1FC4 */
  {0x1FC6,0x1FC6,0x1FC6},  {0x1FC7,0x1FC7,0x1FC7}, /* 1FC6 */
  {0x1FC8,0x1F72,0x1FC8},  {0x1FC9,0x1F73,0x1FC9}, /* 1FC8 */
  {0x1FCA,0x1F74,0x1FCA},  {0x1FCB,0x1F75,0x1FCB}, /* 1FCA */
  {0x1FCC,0x1FC3,0x1FCC},  {0x1FCD,0x1FCD,0x1FCD}, /* 1FCC */
  {0x1FCE,0x1FCE,0x1FCE},  {0x1FCF,0x1FCF,0x1FCF}, /* 1FCE */
  {0x1FD8,0x1FD0,0x1FD8},  {0x1FD9,0x1FD1,0x1FD9}, /* 1FD0 */
  {0x1FD2,0x1FD2,0x1FD2},  {0x1FD3,0x1FD3,0x1FD3}, /* 1FD2 */
  {0x1FD4,0x1FD4,0x1FD4},  {0x1FD5,0x1FD5,0x1FD5}, /* 1FD4 */
  {0x1FD6,0x1FD6,0x1FD6},  {0x1FD7,0x1FD7,0x1FD7}, /* 1FD6 */
  {0x1FD8,0x1FD0,0x1FD8},  {0x1FD9,0x1FD1,0x1FD9}, /* 1FD8 */
  {0x1FDA,0x1F76,0x1FDA},  {0x1FDB,0x1F77,0x1FDB}, /* 1FDA */
  {0x1FDC,0x1FDC,0x1FDC},  {0x1FDD,0x1FDD,0x1FDD}, /* 1FDC */
  {0x1FDE,0x1FDE,0x1FDE},  {0x1FDF,0x1FDF,0x1FDF}, /* 1FDE */
  {0x1FE8,0x1FE0,0x1FE8},  {0x1FE9,0x1FE1,0x1FE9}, /* 1FE0 */
  {0x1FE2,0x1FE2,0x1FE2},  {0x1FE3,0x1FE3,0x1FE3}, /* 1FE2 */
  {0x1FE4,0x1FE4,0x1FE4},  {0x1FEC,0x1FE5,0x1FEC}, /* 1FE4 */
  {0x1FE6,0x1FE6,0x1FE6},  {0x1FE7,0x1FE7,0x1FE7}, /* 1FE6 */
  {0x1FE8,0x1FE0,0x1FE8},  {0x1FE9,0x1FE1,0x1FE9}, /* 1FE8 */
  {0x1FEA,0x1F7A,0x1FEA},  {0x1FEB,0x1F7B,0x1FEB}, /* 1FEA */
  {0x1FEC,0x1FE5,0x1FEC},  {0x1FED,0x1FED,0x1FED}, /* 1FEC */
  {0x1FEE,0x1FEE,0x1FEE},  {0x1FEF,0x1FEF,0x1FEF}, /* 1FEE */
  {0x1FF0,0x1FF0,0x1FF0},  {0x1FF1,0x1FF1,0x1FF1}, /* 1FF0 */
  {0x1FF2,0x1FF2,0x1FF2},  {0x1FFC,0x1FF3,0x1FFC}, /* 1FF2 */
  {0x1FF4,0x1FF4,0x1FF4},  {0x1FF5,0x1FF5,0x1FF5}, /* 1FF4 */
  {0x1FF6,0x1FF6,0x1FF6},  {0x1FF7,0x1FF7,0x1FF7}, /* 1FF6 */
  {0x1FF8,0x1F78,0x1FF8},  {0x1FF9,0x1F79,0x1FF9}, /* 1FF8 */
  {0x1FFA,0x1F7C,0x1FFA},  {0x1FFB,0x1F7D,0x1FFB}, /* 1FFA */
  {0x1FFC,0x1FF3,0x1FFC},  {0x1FFD,0x1FFD,0x1FFD}, /* 1FFC */
  {0x1FFE,0x1FFE,0x1FFE},  {0x1FFF,0x1FFF,0x1FFF}  /* 1FFE */
};

static const MY_UNICASE_CHARACTER u900p021[]= {
  {0x2100,0x2100,0x2100},  {0x2101,0x2101,0x2101}, /* 2100 */
  {0x2102,0x2102,0x2102},  {0x2103,0x2103,0x2103}, /* 2102 */
  {0x2104,0x2104,0x2104},  {0x2105,0x2105,0x2105}, /* 2104 */
  {0x2106,0x2106,0x2106},  {0x2107,0x2107,0x2107}, /* 2106 */
  {0x2108,0x2108,0x2108},  {0x2109,0x2109,0x2109}, /* 2108 */
  {0x210A,0x210A,0x210A},  {0x210B,0x210B,0x210B}, /* 210A */
  {0x210C,0x210C,0x210C},  {0x210D,0x210D,0x210D}, /* 210C */
  {0x210E,0x210E,0x210E},  {0x210F,0x210F,0x210F}, /* 210E */
  {0x2110,0x2110,0x2110},  {0x2111,0x2111,0x2111}, /* 2110 */
  {0x2112,0x2112,0x2112},  {0x2113,0x2113,0x2113}, /* 2112 */
  {0x2114,0x2114,0x2114},  {0x2115,0x2115,0x2115}, /* 2114 */
  {0x2116,0x2116,0x2116},  {0x2117,0x2117,0x2117}, /* 2116 */
  {0x2118,0x2118,0x2118},  {0x2119,0x2119,0x2119}, /* 2118 */
  {0x211A,0x211A,0x211A},  {0x211B,0x211B,0x211B}, /* 211A */
  {0x211C,0x211C,0x211C},  {0x211D,0x211D,0x211D}, /* 211C */
  {0x211E,0x211E,0x211E},  {0x211F,0x211F,0x211F}, /* 211E */
  {0x2120,0x2120,0x2120},  {0x2121,0x2121,0x2121}, /* 2120 */
  {0x2122,0x2122,0x2122},  {0x2123,0x2123,0x2123}, /* 2122 */
  {0x2124,0x2124,0x2124},  {0x2125,0x2125,0x2125}, /* 2124 */
  {0x2126,0x03C9,0x2126},  {0x2127,0x2127,0x2127}, /* 2126 */
  {0x2128,0x2128,0x2128},  {0x2129,0x2129,0x2129}, /* 2128 */
  {0x212A,0x006B,0x212A},  {0x212B,0x00E5,0x212B}, /* 212A */
  {0x212C,0x212C,0x212C},  {0x212D,0x212D,0x212D}, /* 212C */
  {0x212E,0x212E,0x212E},  {0x212F,0x212F,0x212F}, /* 212E */
  {0x2130,0x2130,0x2130},  {0x2131,0x2131,0x2131}, /* 2130 */
  {0x2132,0x214E,0x2132},  {0x2133,0x2133,0x2133}, /* 2132 */
  {0x2134,0x2134,0x2134},  {0x2135,0x2135,0x2135}, /* 2134 */
  {0x2136,0x2136,0x2136},  {0x2137,0x2137,0x2137}, /* 2136 */
  {0x2138,0x2138,0x2138},  {0x2139,0x2139,0x2139}, /* 2138 */
  {0x213A,0x213A,0x213A},  {0x213B,0x213B,0x213B}, /* 213A */
  {0x213C,0x213C,0x213C},  {0x213D,0x213D,0x213D}, /* 213C */
  {0x213E,0x213E,0x213E},  {0x213F,0x213F,0x213F}, /* 213E */
  {0x2140,0x2140,0x2140},  {0x2141,0x2141,0x2141}, /* 2140 */
  {0x2142,0x2142,0x2142},  {0x2143,0x2143,0x2143}, /* 2142 */
  {0x2144,0x2144,0x2144},  {0x2145,0x2145,0x2145}, /* 2144 */
  {0x2146,0x2146,0x2146},  {0x2147,0x2147,0x2147}, /* 2146 */
  {0x2148,0x2148,0x2148},  {0x2149,0x2149,0x2149}, /* 2148 */
  {0x214A,0x214A,0x214A},  {0x214B,0x214B,0x214B}, /* 214A */
  {0x214C,0x214C,0x214C},  {0x214D,0x214D,0x214D}, /* 214C */
  {0x2132,0x214E,0x2132},  {0x214F,0x214F,0x214F}, /* 214E */
  {0x2150,0x2150,0x2150},  {0x2151,0x2151,0x2151}, /* 2150 */
  {0x2152,0x2152,0x2152},  {0x2153,0x2153,0x2153}, /* 2152 */
  {0x2154,0x2154,0x2154},  {0x2155,0x2155,0x2155}, /* 2154 */
  {0x2156,0x2156,0x2156},  {0x2157,0x2157,0x2157}, /* 2156 */
  {0x2158,0x2158,0x2158},  {0x2159,0x2159,0x2159}, /* 2158 */
  {0x215A,0x215A,0x215A},  {0x215B,0x215B,0x215B}, /* 215A */
  {0x215C,0x215C,0x215C},  {0x215D,0x215D,0x215D}, /* 215C */
  {0x215E,0x215E,0x215E},  {0x215F,0x215F,0x215F}, /* 215E */
  {0x2160,0x2170,0x2160},  {0x2161,0x2171,0x2161}, /* 2160 */
  {0x2162,0x2172,0x2162},  {0x2163,0x2173,0x2163}, /* 2162 */
  {0x2164,0x2174,0x2164},  {0x2165,0x2175,0x2165}, /* 2164 */
  {0x2166,0x2176,0x2166},  {0x2167,0x2177,0x2167}, /* 2166 */
  {0x2168,0x2178,0x2168},  {0x2169,0x2179,0x2169}, /* 2168 */
  {0x216A,0x217A,0x216A},  {0x216B,0x217B,0x216B}, /* 216A */
  {0x216C,0x217C,0x216C},  {0x216D,0x217D,0x216D}, /* 216C */
  {0x216E,0x217E,0x216E},  {0x216F,0x217F,0x216F}, /* 216E */
  {0x2160,0x2170,0x2160},  {0x2161,0x2171,0x2161}, /* 2170 */
  {0x2162,0x2172,0x2162},  {0x2163,0x2173,0x2163}, /* 2172 */
  {0x2164,0x2174,0x2164},  {0x2165,0x2175,0x2165}, /* 2174 */
  {0x2166,0x2176,0x2166},  {0x2167,0x2177,0x2167}, /* 2176 */
  {0x2168,0x2178,0x2168},  {0x2169,0x2179,0x2169}, /* 2178 */
  {0x216A,0x217A,0x216A},  {0x216B,0x217B,0x216B}, /* 217A */
  {0x216C,0x217C,0x216C},  {0x216D,0x217D,0x216D}, /* 217C */
  {0x216E,0x217E,0x216E},  {0x216F,0x217F,0x216F}, /* 217E */
  {0x2180,0x2180,0x2180},  {0x2181,0x2181,0x2181}, /* 2180 */
  {0x2182,0x2182,0x2182},  {0x2183,0x2184,0x2183}, /* 2182 */
  {0x2183,0x2184,0x2183},  {0x2185,0x2185,0x2185}, /* 2184 */
  {0x2186,0x2186,0x2186},  {0x2187,0x2187,0x2187}, /* 2186 */
  {0x2188,0x2188,0x2188},  {0x2189,0x2189,0x2189}, /* 2188 */
  {0x218A,0x218A,0x218A},  {0x218B,0x218B,0x218B}, /* 218A */
  {0x218C,0x218C,0x218C},  {0x218D,0x218D,0x218D}, /* 218C */
  {0x218E,0x218E,0x218E},  {0x218F,0x218F,0x218F}, /* 218E */
  {0x2190,0x2190,0x2190},  {0x2191,0x2191,0x2191}, /* 2190 */
  {0x2192,0x2192,0x2192},  {0x2193,0x2193,0x2193}, /* 2192 */
  {0x2194,0x2194,0x2194},  {0x2195,0x2195,0x2195}, /* 2194 */
  {0x2196,0x2196,0x2196},  {0x2197,0x2197,0x2197}, /* 2196 */
  {0x2198,0x2198,0x2198},  {0x2199,0x2199,0x2199}, /* 2198 */
  {0x219A,0x219A,0x219A},  {0x219B,0x219B,0x219B}, /* 219A */
  {0x219C,0x219C,0x219C},  {0x219D,0x219D,0x219D}, /* 219C */
  {0x219E,0x219E,0x219E},  {0x219F,0x219F,0x219F}, /* 219E */
  {0x21A0,0x21A0,0x21A0},  {0x21A1,0x21A1,0x21A1}, /* 21A0 */
  {0x21A2,0x21A2,0x21A2},  {0x21A3,0x21A3,0x21A3}, /* 21A2 */
  {0x21A4,0x21A4,0x21A4},  {0x21A5,0x21A5,0x21A5}, /* 21A4 */
  {0x21A6,0x21A6,0x21A6},  {0x21A7,0x21A7,0x21A7}, /* 21A6 */
  {0x21A8,0x21A8,0x21A8},  {0x21A9,0x21A9,0x21A9}, /* 21A8 */
  {0x21AA,0x21AA,0x21AA},  {0x21AB,0x21AB,0x21AB}, /* 21AA */
  {0x21AC,0x21AC,0x21AC},  {0x21AD,0x21AD,0x21AD}, /* 21AC */
  {0x21AE,0x21AE,0x21AE},  {0x21AF,0x21AF,0x21AF}, /* 21AE */
  {0x21B0,0x21B0,0x21B0},  {0x21B1,0x21B1,0x21B1}, /* 21B0 */
  {0x21B2,0x21B2,0x21B2},  {0x21B3,0x21B3,0x21B3}, /* 21B2 */
  {0x21B4,0x21B4,0x21B4},  {0x21B5,0x21B5,0x21B5}, /* 21B4 */
  {0x21B6,0x21B6,0x21B6},  {0x21B7,0x21B7,0x21B7}, /* 21B6 */
  {0x21B8,0x21B8,0x21B8},  {0x21B9,0x21B9,0x21B9}, /* 21B8 */
  {0x21BA,0x21BA,0x21BA},  {0x21BB,0x21BB,0x21BB}, /* 21BA */
  {0x21BC,0x21BC,0x21BC},  {0x21BD,0x21BD,0x21BD}, /* 21BC */
  {0x21BE,0x21BE,0x21BE},  {0x21BF,0x21BF,0x21BF}, /* 21BE */
  {0x21C0,0x21C0,0x21C0},  {0x21C1,0x21C1,0x21C1}, /* 21C0 */
  {0x21C2,0x21C2,0x21C2},  {0x21C3,0x21C3,0x21C3}, /* 21C2 */
  {0x21C4,0x21C4,0x21C4},  {0x21C5,0x21C5,0x21C5}, /* 21C4 */
  {0x21C6,0x21C6,0x21C6},  {0x21C7,0x21C7,0x21C7}, /* 21C6 */
  {0x21C8,0x21C8,0x21C8},  {0x21C9,0x21C9,0x21C9}, /* 21C8 */
  {0x21CA,0x21CA,0x21CA},  {0x21CB,0x21CB,0x21CB}, /* 21CA */
  {0x21CC,0x21CC,0x21CC},  {0x21CD,0x21CD,0x21CD}, /* 21CC */
  {0x21CE,0x21CE,0x21CE},  {0x21CF,0x21CF,0x21CF}, /* 21CE */
  {0x21D0,0x21D0,0x21D0},  {0x21D1,0x21D1,0x21D1}, /* 21D0 */
  {0x21D2,0x21D2,0x21D2},  {0x21D3,0x21D3,0x21D3}, /* 21D2 */
  {0x21D4,0x21D4,0x21D4},  {0x21D5,0x21D5,0x21D5}, /* 21D4 */
  {0x21D6,0x21D6,0x21D6},  {0x21D7,0x21D7,0x21D7}, /* 21D6 */
  {0x21D8,0x21D8,0x21D8},  {0x21D9,0x21D9,0x21D9}, /* 21D8 */
  {0x21DA,0x21DA,0x21DA},  {0x21DB,0x21DB,0x21DB}, /* 21DA */
  {0x21DC,0x21DC,0x21DC},  {0x21DD,0x21DD,0x21DD}, /* 21DC */
  {0x21DE,0x21DE,0x21DE},  {0x21DF,0x21DF,0x21DF}, /* 21DE */
  {0x21E0,0x21E0,0x21E0},  {0x21E1,0x21E1,0x21E1}, /* 21E0 */
  {0x21E2,0x21E2,0x21E2},  {0x21E3,0x21E3,0x21E3}, /* 21E2 */
  {0x21E4,0x21E4,0x21E4},  {0x21E5,0x21E5,0x21E5}, /* 21E4 */
  {0x21E6,0x21E6,0x21E6},  {0x21E7,0x21E7,0x21E7}, /* 21E6 */
  {0x21E8,0x21E8,0x21E8},  {0x21E9,0x21E9,0x21E9}, /* 21E8 */
  {0x21EA,0x21EA,0x21EA},  {0x21EB,0x21EB,0x21EB}, /* 21EA */
  {0x21EC,0x21EC,0x21EC},  {0x21ED,0x21ED,0x21ED}, /* 21EC */
  {0x21EE,0x21EE,0x21EE},  {0x21EF,0x21EF,0x21EF}, /* 21EE */
  {0x21F0,0x21F0,0x21F0},  {0x21F1,0x21F1,0x21F1}, /* 21F0 */
  {0x21F2,0x21F2,0x21F2},  {0x21F3,0x21F3,0x21F3}, /* 21F2 */
  {0x21F4,0x21F4,0x21F4},  {0x21F5,0x21F5,0x21F5}, /* 21F4 */
  {0x21F6,0x21F6,0x21F6},  {0x21F7,0x21F7,0x21F7}, /* 21F6 */
  {0x21F8,0x21F8,0x21F8},  {0x21F9,0x21F9,0x21F9}, /* 21F8 */
  {0x21FA,0x21FA,0x21FA},  {0x21FB,0x21FB,0x21FB}, /* 21FA */
  {0x21FC,0x21FC,0x21FC},  {0x21FD,0x21FD,0x21FD}, /* 21FC */
  {0x21FE,0x21FE,0x21FE},  {0x21FF,0x21FF,0x21FF}  /* 21FE */
};

static const MY_UNICASE_CHARACTER u900p024[]= {
  {0x2400,0x2400,0x2400},  {0x2401,0x2401,0x2401}, /* 2400 */
  {0x2402,0x2402,0x2402},  {0x2403,0x2403,0x2403}, /* 2402 */
  {0x2404,0x2404,0x2404},  {0x2405,0x2405,0x2405}, /* 2404 */
  {0x2406,0x2406,0x2406},  {0x2407,0x2407,0x2407}, /* 2406 */
  {0x2408,0x2408,0x2408},  {0x2409,0x2409,0x2409}, /* 2408 */
  {0x240A,0x240A,0x240A},  {0x240B,0x240B,0x240B}, /* 240A */
  {0x240C,0x240C,0x240C},  {0x240D,0x240D,0x240D}, /* 240C */
  {0x240E,0x240E,0x240E},  {0x240F,0x240F,0x240F}, /* 240E */
  {0x2410,0x2410,0x2410},  {0x2411,0x2411,0x2411}, /* 2410 */
  {0x2412,0x2412,0x2412},  {0x2413,0x2413,0x2413}, /* 2412 */
  {0x2414,0x2414,0x2414},  {0x2415,0x2415,0x2415}, /* 2414 */
  {0x2416,0x2416,0x2416},  {0x2417,0x2417,0x2417}, /* 2416 */
  {0x2418,0x2418,0x2418},  {0x2419,0x2419,0x2419}, /* 2418 */
  {0x241A,0x241A,0x241A},  {0x241B,0x241B,0x241B}, /* 241A */
  {0x241C,0x241C,0x241C},  {0x241D,0x241D,0x241D}, /* 241C */
  {0x241E,0x241E,0x241E},  {0x241F,0x241F,0x241F}, /* 241E */
  {0x2420,0x2420,0x2420},  {0x2421,0x2421,0x2421}, /* 2420 */
  {0x2422,0x2422,0x2422},  {0x2423,0x2423,0x2423}, /* 2422 */
  {0x2424,0x2424,0x2424},  {0x2425,0x2425,0x2425}, /* 2424 */
  {0x2426,0x2426,0x2426},  {0x2427,0x2427,0x2427}, /* 2426 */
  {0x2428,0x2428,0x2428},  {0x2429,0x2429,0x2429}, /* 2428 */
  {0x242A,0x242A,0x242A},  {0x242B,0x242B,0x242B}, /* 242A */
  {0x242C,0x242C,0x242C},  {0x242D,0x242D,0x242D}, /* 242C */
  {0x242E,0x242E,0x242E},  {0x242F,0x242F,0x242F}, /* 242E */
  {0x2430,0x2430,0x2430},  {0x2431,0x2431,0x2431}, /* 2430 */
  {0x2432,0x2432,0x2432},  {0x2433,0x2433,0x2433}, /* 2432 */
  {0x2434,0x2434,0x2434},  {0x2435,0x2435,0x2435}, /* 2434 */
  {0x2436,0x2436,0x2436},  {0x2437,0x2437,0x2437}, /* 2436 */
  {0x2438,0x2438,0x2438},  {0x2439,0x2439,0x2439}, /* 2438 */
  {0x243A,0x243A,0x243A},  {0x243B,0x243B,0x243B}, /* 243A */
  {0x243C,0x243C,0x243C},  {0x243D,0x243D,0x243D}, /* 243C */
  {0x243E,0x243E,0x243E},  {0x243F,0x243F,0x243F}, /* 243E */
  {0x2440,0x2440,0x2440},  {0x2441,0x2441,0x2441}, /* 2440 */
  {0x2442,0x2442,0x2442},  {0x2443,0x2443,0x2443}, /* 2442 */
  {0x2444,0x2444,0x2444},  {0x2445,0x2445,0x2445}, /* 2444 */
  {0x2446,0x2446,0x2446},  {0x2447,0x2447,0x2447}, /* 2446 */
  {0x2448,0x2448,0x2448},  {0x2449,0x2449,0x2449}, /* 2448 */
  {0x244A,0x244A,0x244A},  {0x244B,0x244B,0x244B}, /* 244A */
  {0x244C,0x244C,0x244C},  {0x244D,0x244D,0x244D}, /* 244C */
  {0x244E,0x244E,0x244E},  {0x244F,0x244F,0x244F}, /* 244E */
  {0x2450,0x2450,0x2450},  {0x2451,0x2451,0x2451}, /* 2450 */
  {0x2452,0x2452,0x2452},  {0x2453,0x2453,0x2453}, /* 2452 */
  {0x2454,0x2454,0x2454},  {0x2455,0x2455,0x2455}, /* 2454 */
  {0x2456,0x2456,0x2456},  {0x2457,0x2457,0x2457}, /* 2456 */
  {0x2458,0x2458,0x2458},  {0x2459,0x2459,0x2459}, /* 2458 */
  {0x245A,0x245A,0x245A},  {0x245B,0x245B,0x245B}, /* 245A */
  {0x245C,0x245C,0x245C},  {0x245D,0x245D,0x245D}, /* 245C */
  {0x245E,0x245E,0x245E},  {0x245F,0x245F,0x245F}, /* 245E */
  {0x2460,0x2460,0x2460},  {0x2461,0x2461,0x2461}, /* 2460 */
  {0x2462,0x2462,0x2462},  {0x2463,0x2463,0x2463}, /* 2462 */
  {0x2464,0x2464,0x2464},  {0x2465,0x2465,0x2465}, /* 2464 */
  {0x2466,0x2466,0x2466},  {0x2467,0x2467,0x2467}, /* 2466 */
  {0x2468,0x2468,0x2468},  {0x2469,0x2469,0x2469}, /* 2468 */
  {0x246A,0x246A,0x246A},  {0x246B,0x246B,0x246B}, /* 246A */
  {0x246C,0x246C,0x246C},  {0x246D,0x246D,0x246D}, /* 246C */
  {0x246E,0x246E,0x246E},  {0x246F,0x246F,0x246F}, /* 246E */
  {0x2470,0x2470,0x2470},  {0x2471,0x2471,0x2471}, /* 2470 */
  {0x2472,0x2472,0x2472},  {0x2473,0x2473,0x2473}, /* 2472 */
  {0x2474,0x2474,0x2474},  {0x2475,0x2475,0x2475}, /* 2474 */
  {0x2476,0x2476,0x2476},  {0x2477,0x2477,0x2477}, /* 2476 */
  {0x2478,0x2478,0x2478},  {0x2479,0x2479,0x2479}, /* 2478 */
  {0x247A,0x247A,0x247A},  {0x247B,0x247B,0x247B}, /* 247A */
  {0x247C,0x247C,0x247C},  {0x247D,0x247D,0x247D}, /* 247C */
  {0x247E,0x247E,0x247E},  {0x247F,0x247F,0x247F}, /* 247E */
  {0x2480,0x2480,0x2480},  {0x2481,0x2481,0x2481}, /* 2480 */
  {0x2482,0x2482,0x2482},  {0x2483,0x2483,0x2483}, /* 2482 */
  {0x2484,0x2484,0x2484},  {0x2485,0x2485,0x2485}, /* 2484 */
  {0x2486,0x2486,0x2486},  {0x2487,0x2487,0x2487}, /* 2486 */
  {0x2488,0x2488,0x2488},  {0x2489,0x2489,0x2489}, /* 2488 */
  {0x248A,0x248A,0x248A},  {0x248B,0x248B,0x248B}, /* 248A */
  {0x248C,0x248C,0x248C},  {0x248D,0x248D,0x248D}, /* 248C */
  {0x248E,0x248E,0x248E},  {0x248F,0x248F,0x248F}, /* 248E */
  {0x2490,0x2490,0x2490},  {0x2491,0x2491,0x2491}, /* 2490 */
  {0x2492,0x2492,0x2492},  {0x2493,0x2493,0x2493}, /* 2492 */
  {0x2494,0x2494,0x2494},  {0x2495,0x2495,0x2495}, /* 2494 */
  {0x2496,0x2496,0x2496},  {0x2497,0x2497,0x2497}, /* 2496 */
  {0x2498,0x2498,0x2498},  {0x2499,0x2499,0x2499}, /* 2498 */
  {0x249A,0x249A,0x249A},  {0x249B,0x249B,0x249B}, /* 249A */
  {0x249C,0x249C,0x249C},  {0x249D,0x249D,0x249D}, /* 249C */
  {0x249E,0x249E,0x249E},  {0x249F,0x249F,0x249F}, /* 249E */
  {0x24A0,0x24A0,0x24A0},  {0x24A1,0x24A1,0x24A1}, /* 24A0 */
  {0x24A2,0x24A2,0x24A2},  {0x24A3,0x24A3,0x24A3}, /* 24A2 */
  {0x24A4,0x24A4,0x24A4},  {0x24A5,0x24A5,0x24A5}, /* 24A4 */
  {0x24A6,0x24A6,0x24A6},  {0x24A7,0x24A7,0x24A7}, /* 24A6 */
  {0x24A8,0x24A8,0x24A8},  {0x24A9,0x24A9,0x24A9}, /* 24A8 */
  {0x24AA,0x24AA,0x24AA},  {0x24AB,0x24AB,0x24AB}, /* 24AA */
  {0x24AC,0x24AC,0x24AC},  {0x24AD,0x24AD,0x24AD}, /* 24AC */
  {0x24AE,0x24AE,0x24AE},  {0x24AF,0x24AF,0x24AF}, /* 24AE */
  {0x24B0,0x24B0,0x24B0},  {0x24B1,0x24B1,0x24B1}, /* 24B0 */
  {0x24B2,0x24B2,0x24B2},  {0x24B3,0x24B3,0x24B3}, /* 24B2 */
  {0x24B4,0x24B4,0x24B4},  {0x24B5,0x24B5,0x24B5}, /* 24B4 */
  {0x24B6,0x24D0,0x24B6},  {0x24B7,0x24D1,0x24B7}, /* 24B6 */
  {0x24B8,0x24D2,0x24B8},  {0x24B9,0x24D3,0x24B9}, /* 24B8 */
  {0x24BA,0x24D4,0x24BA},  {0x24BB,0x24D5,0x24BB}, /* 24BA */
  {0x24BC,0x24D6,0x24BC},  {0x24BD,0x24D7,0x24BD}, /* 24BC */
  {0x24BE,0x24D8,0x24BE},  {0x24BF,0x24D9,0x24BF}, /* 24BE */
  {0x24C0,0x24DA,0x24C0},  {0x24C1,0x24DB,0x24C1}, /* 24C0 */
  {0x24C2,0x24DC,0x24C2},  {0x24C3,0x24DD,0x24C3}, /* 24C2 */
  {0x24C4,0x24DE,0x24C4},  {0x24C5,0x24DF,0x24C5}, /* 24C4 */
  {0x24C6,0x24E0,0x24C6},  {0x24C7,0x24E1,0x24C7}, /* 24C6 */
  {0x24C8,0x24E2,0x24C8},  {0x24C9,0x24E3,0x24C9}, /* 24C8 */
  {0x24CA,0x24E4,0x24CA},  {0x24CB,0x24E5,0x24CB}, /* 24CA */
  {0x24CC,0x24E6,0x24CC},  {0x24CD,0x24E7,0x24CD}, /* 24CC */
  {0x24CE,0x24E8,0x24CE},  {0x24CF,0x24E9,0x24CF}, /* 24CE */
  {0x24B6,0x24D0,0x24B6},  {0x24B7,0x24D1,0x24B7}, /* 24D0 */
  {0x24B8,0x24D2,0x24B8},  {0x24B9,0x24D3,0x24B9}, /* 24D2 */
  {0x24BA,0x24D4,0x24BA},  {0x24BB,0x24D5,0x24BB}, /* 24D4 */
  {0x24BC,0x24D6,0x24BC},  {0x24BD,0x24D7,0x24BD}, /* 24D6 */
  {0x24BE,0x24D8,0x24BE},  {0x24BF,0x24D9,0x24BF}, /* 24D8 */
  {0x24C0,0x24DA,0x24C0},  {0x24C1,0x24DB,0x24C1}, /* 24DA */
  {0x24C2,0x24DC,0x24C2},  {0x24C3,0x24DD,0x24C3}, /* 24DC */
  {0x24C4,0x24DE,0x24C4},  {0x24C5,0x24DF,0x24C5}, /* 24DE */
  {0x24C6,0x24E0,0x24C6},  {0x24C7,0x24E1,0x24C7}, /* 24E0 */
  {0x24C8,0x24E2,0x24C8},  {0x24C9,0x24E3,0x24C9}, /* 24E2 */
  {0x24CA,0x24E4,0x24CA},  {0x24CB,0x24E5,0x24CB}, /* 24E4 */
  {0x24CC,0x24E6,0x24CC},  {0x24CD,0x24E7,0x24CD}, /* 24E6 */
  {0x24CE,0x24E8,0x24CE},  {0x24CF,0x24E9,0x24CF}, /* 24E8 */
  {0x24EA,0x24EA,0x24EA},  {0x24EB,0x24EB,0x24EB}, /* 24EA */
  {0x24EC,0x24EC,0x24EC},  {0x24ED,0x24ED,0x24ED}, /* 24EC */
  {0x24EE,0x24EE,0x24EE},  {0x24EF,0x24EF,0x24EF}, /* 24EE */
  {0x24F0,0x24F0,0x24F0},  {0x24F1,0x24F1,0x24F1}, /* 24F0 */
  {0x24F2,0x24F2,0x24F2},  {0x24F3,0x24F3,0x24F3}, /* 24F2 */
  {0x24F4,0x24F4,0x24F4},  {0x24F5,0x24F5,0x24F5}, /* 24F4 */
  {0x24F6,0x24F6,0x24F6},  {0x24F7,0x24F7,0x24F7}, /* 24F6 */
  {0x24F8,0x24F8,0x24F8},  {0x24F9,0x24F9,0x24F9}, /* 24F8 */
  {0x24FA,0x24FA,0x24FA},  {0x24FB,0x24FB,0x24FB}, /* 24FA */
  {0x24FC,0x24FC,0x24FC},  {0x24FD,0x24FD,0x24FD}, /* 24FC */
  {0x24FE,0x24FE,0x24FE},  {0x24FF,0x24FF,0x24FF}  /* 24FE */
};

static const MY_UNICASE_CHARACTER u900p02C[]= {
  {0x2C00,0x2C30,0x2C00},  {0x2C01,0x2C31,0x2C01}, /* 2C00 */
  {0x2C02,0x2C32,0x2C02},  {0x2C03,0x2C33,0x2C03}, /* 2C02 */
  {0x2C04,0x2C34,0x2C04},  {0x2C05,0x2C35,0x2C05}, /* 2C04 */
  {0x2C06,0x2C36,0x2C06},  {0x2C07,0x2C37,0x2C07}, /* 2C06 */
  {0x2C08,0x2C38,0x2C08},  {0x2C09,0x2C39,0x2C09}, /* 2C08 */
  {0x2C0A,0x2C3A,0x2C0A},  {0x2C0B,0x2C3B,0x2C0B}, /* 2C0A */
  {0x2C0C,0x2C3C,0x2C0C},  {0x2C0D,0x2C3D,0x2C0D}, /* 2C0C */
  {0x2C0E,0x2C3E,0x2C0E},  {0x2C0F,0x2C3F,0x2C0F}, /* 2C0E */
  {0x2C10,0x2C40,0x2C10},  {0x2C11,0x2C41,0x2C11}, /* 2C10 */
  {0x2C12,0x2C42,0x2C12},  {0x2C13,0x2C43,0x2C13}, /* 2C12 */
  {0x2C14,0x2C44,0x2C14},  {0x2C15,0x2C45,0x2C15}, /* 2C14 */
  {0x2C16,0x2C46,0x2C16},  {0x2C17,0x2C47,0x2C17}, /* 2C16 */
  {0x2C18,0x2C48,0x2C18},  {0x2C19,0x2C49,0x2C19}, /* 2C18 */
  {0x2C1A,0x2C4A,0x2C1A},  {0x2C1B,0x2C4B,0x2C1B}, /* 2C1A */
  {0x2C1C,0x2C4C,0x2C1C},  {0x2C1D,0x2C4D,0x2C1D}, /* 2C1C */
  {0x2C1E,0x2C4E,0x2C1E},  {0x2C1F,0x2C4F,0x2C1F}, /* 2C1E */
  {0x2C20,0x2C50,0x2C20},  {0x2C21,0x2C51,0x2C21}, /* 2C20 */
  {0x2C22,0x2C52,0x2C22},  {0x2C23,0x2C53,0x2C23}, /* 2C22 */
  {0x2C24,0x2C54,0x2C24},  {0x2C25,0x2C55,0x2C25}, /* 2C24 */
  {0x2C26,0x2C56,0x2C26},  {0x2C27,0x2C57,0x2C27}, /* 2C26 */
  {0x2C28,0x2C58,0x2C28},  {0x2C29,0x2C59,0x2C29}, /* 2C28 */
  {0x2C2A,0x2C5A,0x2C2A},  {0x2C2B,0x2C5B,0x2C2B}, /* 2C2A */
  {0x2C2C,0x2C5C,0x2C2C},  {0x2C2D,0x2C5D,0x2C2D}, /* 2C2C */
  {0x2C2E,0x2C5E,0x2C2E},  {0x2C2F,0x2C2F,0x2C2F}, /* 2C2E */
  {0x2C00,0x2C30,0x2C00},  {0x2C01,0x2C31,0x2C01}, /* 2C30 */
  {0x2C02,0x2C32,0x2C02},  {0x2C03,0x2C33,0x2C03}, /* 2C32 */
  {0x2C04,0x2C34,0x2C04},  {0x2C05,0x2C35,0x2C05}, /* 2C34 */
  {0x2C06,0x2C36,0x2C06},  {0x2C07,0x2C37,0x2C07}, /* 2C36 */
  {0x2C08,0x2C38,0x2C08},  {0x2C09,0x2C39,0x2C09}, /* 2C38 */
  {0x2C0A,0x2C3A,0x2C0A},  {0x2C0B,0x2C3B,0x2C0B}, /* 2C3A */
  {0x2C0C,0x2C3C,0x2C0C},  {0x2C0D,0x2C3D,0x2C0D}, /* 2C3C */
  {0x2C0E,0x2C3E,0x2C0E},  {0x2C0F,0x2C3F,0x2C0F}, /* 2C3E */
  {0x2C10,0x2C40,0x2C10},  {0x2C11,0x2C41,0x2C11}, /* 2C40 */
  {0x2C12,0x2C42,0x2C12},  {0x2C13,0x2C43,0x2C13}, /* 2C42 */
  {0x2C14,0x2C44,0x2C14},  {0x2C15,0x2C45,0x2C15}, /* 2C44 */
  {0x2C16,0x2C46,0x2C16},  {0x2C17,0x2C47,0x2C17}, /* 2C46 */
  {0x2C18,0x2C48,0x2C18},  {0x2C19,0x2C49,0x2C19}, /* 2C48 */
  {0x2C1A,0x2C4A,0x2C1A},  {0x2C1B,0x2C4B,0x2C1B}, /* 2C4A */
  {0x2C1C,0x2C4C,0x2C1C},  {0x2C1D,0x2C4D,0x2C1D}, /* 2C4C */
  {0x2C1E,0x2C4E,0x2C1E},  {0x2C1F,0x2C4F,0x2C1F}, /* 2C4E */
  {0x2C20,0x2C50,0x2C20},  {0x2C21,0x2C51,0x2C21}, /* 2C50 */
  {0x2C22,0x2C52,0x2C22},  {0x2C23,0x2C53,0x2C23}, /* 2C52 */
  {0x2C24,0x2C54,0x2C24},  {0x2C25,0x2C55,0x2C25}, /* 2C54 */
  {0x2C26,0x2C56,0x2C26},  {0x2C27,0x2C57,0x2C27}, /* 2C56 */
  {0x2C28,0x2C58,0x2C28},  {0x2C29,0x2C59,0x2C29}, /* 2C58 */
  {0x2C2A,0x2C5A,0x2C2A},  {0x2C2B,0x2C5B,0x2C2B}, /* 2C5A */
  {0x2C2C,0x2C5C,0x2C2C},  {0x2C2D,0x2C5D,0x2C2D}, /* 2C5C */
  {0x2C2E,0x2C5E,0x2C2E},  {0x2C5F,0x2C5F,0x2C5F}, /* 2C5E */
  {0x2C60,0x2C61,0x2C60},  {0x2C60,0x2C61,0x2C60}, /* 2C60 */
  {0x2C62,0x026B,0x2C62},  {0x2C63,0x1D7D,0x2C63}, /* 2C62 */
  {0x2C64,0x027D,0x2C64},  {0x023A,0x2C65,0x023A}, /* 2C64 */
  {0x023E,0x2C66,0x023E},  {0x2C67,0x2C68,0x2C67}, /* 2C66 */
  {0x2C67,0x2C68,0x2C67},  {0x2C69,0x2C6A,0x2C69}, /* 2C68 */
  {0x2C69,0x2C6A,0x2C69},  {0x2C6B,0x2C6C,0x2C6B}, /* 2C6A */
  {0x2C6B,0x2C6C,0x2C6B},  {0x2C6D,0x0251,0x2C6D}, /* 2C6C */
  {0x2C6E,0x0271,0x2C6E},  {0x2C6F,0x0250,0x2C6F}, /* 2C6E */
  {0x2C70,0x0252,0x2C70},  {0x2C71,0x2C71,0x2C71}, /* 2C70 */
  {0x2C72,0x2C73,0x2C72},  {0x2C72,0x2C73,0x2C72}, /* 2C72 */
  {0x2C74,0x2C74,0x2C74},  {0x2C75,0x2C76,0x2C75}, /* 2C74 */
  {0x2C75,0x2C76,0x2C75},  {0x2C77,0x2C77,0x2C77}, /* 2C76 */
  {0x2C78,0x2C78,0x2C78},  {0x2C79,0x2C79,0x2C79}, /* 2C78 */
  {0x2C7A,0x2C7A,0x2C7A},  {0x2C7B,0x2C7B,0x2C7B}, /* 2C7A */
  {0x2C7C,0x2C7C,0x2C7C},  {0x2C7D,0x2C7D,0x2C7D}, /* 2C7C */
  {0x2C7E,0x023F,0x2C7E},  {0x2C7F,0x0240,0x2C7F}, /* 2C7E */
  {0x2C80,0x2C81,0x2C80},  {0x2C80,0x2C81,0x2C80}, /* 2C80 */
  {0x2C82,0x2C83,0x2C82},  {0x2C82,0x2C83,0x2C82}, /* 2C82 */
  {0x2C84,0x2C85,0x2C84},  {0x2C84,0x2C85,0x2C84}, /* 2C84 */
  {0x2C86,0x2C87,0x2C86},  {0x2C86,0x2C87,0x2C86}, /* 2C86 */
  {0x2C88,0x2C89,0x2C88},  {0x2C88,0x2C89,0x2C88}, /* 2C88 */
  {0x2C8A,0x2C8B,0x2C8A},  {0x2C8A,0x2C8B,0x2C8A}, /* 2C8A */
  {0x2C8C,0x2C8D,0x2C8C},  {0x2C8C,0x2C8D,0x2C8C}, /* 2C8C */
  {0x2C8E,0x2C8F,0x2C8E},  {0x2C8E,0x2C8F,0x2C8E}, /* 2C8E */
  {0x2C90,0x2C91,0x2C90},  {0x2C90,0x2C91,0x2C90}, /* 2C90 */
  {0x2C92,0x2C93,0x2C92},  {0x2C92,0x2C93,0x2C92}, /* 2C92 */
  {0x2C94,0x2C95,0x2C94},  {0x2C94,0x2C95,0x2C94}, /* 2C94 */
  {0x2C96,0x2C97,0x2C96},  {0x2C96,0x2C97,0x2C96}, /* 2C96 */
  {0x2C98,0x2C99,0x2C98},  {0x2C98,0x2C99,0x2C98}, /* 2C98 */
  {0x2C9A,0x2C9B,0x2C9A},  {0x2C9A,0x2C9B,0x2C9A}, /* 2C9A */
  {0x2C9C,0x2C9D,0x2C9C},  {0x2C9C,0x2C9D,0x2C9C}, /* 2C9C */
  {0x2C9E,0x2C9F,0x2C9E},  {0x2C9E,0x2C9F,0x2C9E}, /* 2C9E */
  {0x2CA0,0x2CA1,0x2CA0},  {0x2CA0,0x2CA1,0x2CA0}, /* 2CA0 */
  {0x2CA2,0x2CA3,0x2CA2},  {0x2CA2,0x2CA3,0x2CA2}, /* 2CA2 */
  {0x2CA4,0x2CA5,0x2CA4},  {0x2CA4,0x2CA5,0x2CA4}, /* 2CA4 */
  {0x2CA6,0x2CA7,0x2CA6},  {0x2CA6,0x2CA7,0x2CA6}, /* 2CA6 */
  {0x2CA8,0x2CA9,0x2CA8},  {0x2CA8,0x2CA9,0x2CA8}, /* 2CA8 */
  {0x2CAA,0x2CAB,0x2CAA},  {0x2CAA,0x2CAB,0x2CAA}, /* 2CAA */
  {0x2CAC,0x2CAD,0x2CAC},  {0x2CAC,0x2CAD,0x2CAC}, /* 2CAC */
  {0x2CAE,0x2CAF,0x2CAE},  {0x2CAE,0x2CAF,0x2CAE}, /* 2CAE */
  {0x2CB0,0x2CB1,0x2CB0},  {0x2CB0,0x2CB1,0x2CB0}, /* 2CB0 */
  {0x2CB2,0x2CB3,0x2CB2},  {0x2CB2,0x2CB3,0x2CB2}, /* 2CB2 */
  {0x2CB4,0x2CB5,0x2CB4},  {0x2CB4,0x2CB5,0x2CB4}, /* 2CB4 */
  {0x2CB6,0x2CB7,0x2CB6},  {0x2CB6,0x2CB7,0x2CB6}, /* 2CB6 */
  {0x2CB8,0x2CB9,0x2CB8},  {0x2CB8,0x2CB9,0x2CB8}, /* 2CB8 */
  {0x2CBA,0x2CBB,0x2CBA},  {0x2CBA,0x2CBB,0x2CBA}, /* 2CBA */
  {0x2CBC,0x2CBD,0x2CBC},  {0x2CBC,0x2CBD,0x2CBC}, /* 2CBC */
  {0x2CBE,0x2CBF,0x2CBE},  {0x2CBE,0x2CBF,0x2CBE}, /* 2CBE */
  {0x2CC0,0x2CC1,0x2CC0},  {0x2CC0,0x2CC1,0x2CC0}, /* 2CC0 */
  {0x2CC2,0x2CC3,0x2CC2},  {0x2CC2,0x2CC3,0x2CC2}, /* 2CC2 */
  {0x2CC4,0x2CC5,0x2CC4},  {0x2CC4,0x2CC5,0x2CC4}, /* 2CC4 */
  {0x2CC6,0x2CC7,0x2CC6},  {0x2CC6,0x2CC7,0x2CC6}, /* 2CC6 */
  {0x2CC8,0x2CC9,0x2CC8},  {0x2CC8,0x2CC9,0x2CC8}, /* 2CC8 */
  {0x2CCA,0x2CCB,0x2CCA},  {0x2CCA,0x2CCB,0x2CCA}, /* 2CCA */
  {0x2CCC,0x2CCD,0x2CCC},  {0x2CCC,0x2CCD,0x2CCC}, /* 2CCC */
  {0x2CCE,0x2CCF,0x2CCE},  {0x2CCE,0x2CCF,0x2CCE}, /* 2CCE */
  {0x2CD0,0x2CD1,0x2CD0},  {0x2CD0,0x2CD1,0x2CD0}, /* 2CD0 */
  {0x2CD2,0x2CD3,0x2CD2},  {0x2CD2,0x2CD3,0x2CD2}, /* 2CD2 */
  {0x2CD4,0x2CD5,0x2CD4},  {0x2CD4,0x2CD5,0x2CD4}, /* 2CD4 */
  {0x2CD6,0x2CD7,0x2CD6},  {0x2CD6,0x2CD7,0x2CD6}, /* 2CD6 */
  {0x2CD8,0x2CD9,0x2CD8},  {0x2CD8,0x2CD9,0x2CD8}, /* 2CD8 */
  {0x2CDA,0x2CDB,0x2CDA},  {0x2CDA,0x2CDB,0x2CDA}, /* 2CDA */
  {0x2CDC,0x2CDD,0x2CDC},  {0x2CDC,0x2CDD,0x2CDC}, /* 2CDC */
  {0x2CDE,0x2CDF,0x2CDE},  {0x2CDE,0x2CDF,0x2CDE}, /* 2CDE */
  {0x2CE0,0x2CE1,0x2CE0},  {0x2CE0,0x2CE1,0x2CE0}, /* 2CE0 */
  {0x2CE2,0x2CE3,0x2CE2},  {0x2CE2,0x2CE3,0x2CE2}, /* 2CE2 */
  {0x2CE4,0x2CE4,0x2CE4},  {0x2CE5,0x2CE5,0x2CE5}, /* 2CE4 */
  {0x2CE6,0x2CE6,0x2CE6},  {0x2CE7,0x2CE7,0x2CE7}, /* 2CE6 */
  {0x2CE8,0x2CE8,0x2CE8},  {0x2CE9,0x2CE9,0x2CE9}, /* 2CE8 */
  {0x2CEA,0x2CEA,0x2CEA},  {0x2CEB,0x2CEC,0x2CEB}, /* 2CEA */
  {0x2CEB,0x2CEC,0x2CEB},  {0x2CED,0x2CEE,0x2CED}, /* 2CEC */
  {0x2CED,0x2CEE,0x2CED},  {0x2CEF,0x2CEF,0x2CEF}, /* 2CEE */
  {0x2CF0,0x2CF0,0x2CF0},  {0x2CF1,0x2CF1,0x2CF1}, /* 2CF0 */
  {0x2CF2,0x2CF3,0x2CF2},  {0x2CF2,0x2CF3,0x2CF2}, /* 2CF2 */
  {0x2CF4,0x2CF4,0x2CF4},  {0x2CF5,0x2CF5,0x2CF5}, /* 2CF4 */
  {0x2CF6,0x2CF6,0x2CF6},  {0x2CF7,0x2CF7,0x2CF7}, /* 2CF6 */
  {0x2CF8,0x2CF8,0x2CF8},  {0x2CF9,0x2CF9,0x2CF9}, /* 2CF8 */
  {0x2CFA,0x2CFA,0x2CFA},  {0x2CFB,0x2CFB,0x2CFB}, /* 2CFA */
  {0x2CFC,0x2CFC,0x2CFC},  {0x2CFD,0x2CFD,0x2CFD}, /* 2CFC */
  {0x2CFE,0x2CFE,0x2CFE},  {0x2CFF,0x2CFF,0x2CFF}  /* 2CFE */
};

static const MY_UNICASE_CHARACTER u900p02D[]= {
  {0x10A0,0x2D00,0x10A0},  {0x10A1,0x2D01,0x10A1}, /* 2D00 */
  {0x10A2,0x2D02,0x10A2},  {0x10A3,0x2D03,0x10A3}, /* 2D02 */
  {0x10A4,0x2D04,0x10A4},  {0x10A5,0x2D05,0x10A5}, /* 2D04 */
  {0x10A6,0x2D06,0x10A6},  {0x10A7,0x2D07,0x10A7}, /* 2D06 */
  {0x10A8,0x2D08,0x10A8},  {0x10A9,0x2D09,0x10A9}, /* 2D08 */
  {0x10AA,0x2D0A,0x10AA},  {0x10AB,0x2D0B,0x10AB}, /* 2D0A */
  {0x10AC,0x2D0C,0x10AC},  {0x10AD,0x2D0D,0x10AD}, /* 2D0C */
  {0x10AE,0x2D0E,0x10AE},  {0x10AF,0x2D0F,0x10AF}, /* 2D0E */
  {0x10B0,0x2D10,0x10B0},  {0x10B1,0x2D11,0x10B1}, /* 2D10 */
  {0x10B2,0x2D12,0x10B2},  {0x10B3,0x2D13,0x10B3}, /* 2D12 */
  {0x10B4,0x2D14,0x10B4},  {0x10B5,0x2D15,0x10B5}, /* 2D14 */
  {0x10B6,0x2D16,0x10B6},  {0x10B7,0x2D17,0x10B7}, /* 2D16 */
  {0x10B8,0x2D18,0x10B8},  {0x10B9,0x2D19,0x10B9}, /* 2D18 */
  {0x10BA,0x2D1A,0x10BA},  {0x10BB,0x2D1B,0x10BB}, /* 2D1A */
  {0x10BC,0x2D1C,0x10BC},  {0x10BD,0x2D1D,0x10BD}, /* 2D1C */
  {0x10BE,0x2D1E,0x10BE},  {0x10BF,0x2D1F,0x10BF}, /* 2D1E */
  {0x10C0,0x2D20,0x10C0},  {0x10C1,0x2D21,0x10C1}, /* 2D20 */
  {0x10C2,0x2D22,0x10C2},  {0x10C3,0x2D23,0x10C3}, /* 2D22 */
  {0x10C4,0x2D24,0x10C4},  {0x10C5,0x2D25,0x10C5}, /* 2D24 */
  {0x2D26,0x2D26,0x2D26},  {0x10C7,0x2D27,0x10C7}, /* 2D26 */
  {0x2D28,0x2D28,0x2D28},  {0x2D29,0x2D29,0x2D29}, /* 2D28 */
  {0x2D2A,0x2D2A,0x2D2A},  {0x2D2B,0x2D2B,0x2D2B}, /* 2D2A */
  {0x2D2C,0x2D2C,0x2D2C},  {0x10CD,0x2D2D,0x10CD}, /* 2D2C */
  {0x2D2E,0x2D2E,0x2D2E},  {0x2D2F,0x2D2F,0x2D2F}, /* 2D2E */
  {0x2D30,0x2D30,0x2D30},  {0x2D31,0x2D31,0x2D31}, /* 2D30 */
  {0x2D32,0x2D32,0x2D32},  {0x2D33,0x2D33,0x2D33}, /* 2D32 */
  {0x2D34,0x2D34,0x2D34},  {0x2D35,0x2D35,0x2D35}, /* 2D34 */
  {0x2D36,0x2D36,0x2D36},  {0x2D37,0x2D37,0x2D37}, /* 2D36 */
  {0x2D38,0x2D38,0x2D38},  {0x2D39,0x2D39,0x2D39}, /* 2D38 */
  {0x2D3A,0x2D3A,0x2D3A},  {0x2D3B,0x2D3B,0x2D3B}, /* 2D3A */
  {0x2D3C,0x2D3C,0x2D3C},  {0x2D3D,0x2D3D,0x2D3D}, /* 2D3C */
  {0x2D3E,0x2D3E,0x2D3E},  {0x2D3F,0x2D3F,0x2D3F}, /* 2D3E */
  {0x2D40,0x2D40,0x2D40},  {0x2D41,0x2D41,0x2D41}, /* 2D40 */
  {0x2D42,0x2D42,0x2D42},  {0x2D43,0x2D43,0x2D43}, /* 2D42 */
  {0x2D44,0x2D44,0x2D44},  {0x2D45,0x2D45,0x2D45}, /* 2D44 */
  {0x2D46,0x2D46,0x2D46},  {0x2D47,0x2D47,0x2D47}, /* 2D46 */
  {0x2D48,0x2D48,0x2D48},  {0x2D49,0x2D49,0x2D49}, /* 2D48 */
  {0x2D4A,0x2D4A,0x2D4A},  {0x2D4B,0x2D4B,0x2D4B}, /* 2D4A */
  {0x2D4C,0x2D4C,0x2D4C},  {0x2D4D,0x2D4D,0x2D4D}, /* 2D4C */
  {0x2D4E,0x2D4E,0x2D4E},  {0x2D4F,0x2D4F,0x2D4F}, /* 2D4E */
  {0x2D50,0x2D50,0x2D50},  {0x2D51,0x2D51,0x2D51}, /* 2D50 */
  {0x2D52,0x2D52,0x2D52},  {0x2D53,0x2D53,0x2D53}, /* 2D52 */
  {0x2D54,0x2D54,0x2D54},  {0x2D55,0x2D55,0x2D55}, /* 2D54 */
  {0x2D56,0x2D56,0x2D56},  {0x2D57,0x2D57,0x2D57}, /* 2D56 */
  {0x2D58,0x2D58,0x2D58},  {0x2D59,0x2D59,0x2D59}, /* 2D58 */
  {0x2D5A,0x2D5A,0x2D5A},  {0x2D5B,0x2D5B,0x2D5B}, /* 2D5A */
  {0x2D5C,0x2D5C,0x2D5C},  {0x2D5D,0x2D5D,0x2D5D}, /* 2D5C */
  {0x2D5E,0x2D5E,0x2D5E},  {0x2D5F,0x2D5F,0x2D5F}, /* 2D5E */
  {0x2D60,0x2D60,0x2D60},  {0x2D61,0x2D61,0x2D61}, /* 2D60 */
  {0x2D62,0x2D62,0x2D62},  {0x2D63,0x2D63,0x2D63}, /* 2D62 */
  {0x2D64,0x2D64,0x2D64},  {0x2D65,0x2D65,0x2D65}, /* 2D64 */
  {0x2D66,0x2D66,0x2D66},  {0x2D67,0x2D67,0x2D67}, /* 2D66 */
  {0x2D68,0x2D68,0x2D68},  {0x2D69,0x2D69,0x2D69}, /* 2D68 */
  {0x2D6A,0x2D6A,0x2D6A},  {0x2D6B,0x2D6B,0x2D6B}, /* 2D6A */
  {0x2D6C,0x2D6C,0x2D6C},  {0x2D6D,0x2D6D,0x2D6D}, /* 2D6C */
  {0x2D6E,0x2D6E,0x2D6E},  {0x2D6F,0x2D6F,0x2D6F}, /* 2D6E */
  {0x2D70,0x2D70,0x2D70},  {0x2D71,0x2D71,0x2D71}, /* 2D70 */
  {0x2D72,0x2D72,0x2D72},  {0x2D73,0x2D73,0x2D73}, /* 2D72 */
  {0x2D74,0x2D74,0x2D74},  {0x2D75,0x2D75,0x2D75}, /* 2D74 */
  {0x2D76,0x2D76,0x2D76},  {0x2D77,0x2D77,0x2D77}, /* 2D76 */
  {0x2D78,0x2D78,0x2D78},  {0x2D79,0x2D79,0x2D79}, /* 2D78 */
  {0x2D7A,0x2D7A,0x2D7A},  {0x2D7B,0x2D7B,0x2D7B}, /* 2D7A */
  {0x2D7C,0x2D7C,0x2D7C},  {0x2D7D,0x2D7D,0x2D7D}, /* 2D7C */
  {0x2D7E,0x2D7E,0x2D7E},  {0x2D7F,0x2D7F,0x2D7F}, /* 2D7E */
  {0x2D80,0x2D80,0x2D80},  {0x2D81,0x2D81,0x2D81}, /* 2D80 */
  {0x2D82,0x2D82,0x2D82},  {0x2D83,0x2D83,0x2D83}, /* 2D82 */
  {0x2D84,0x2D84,0x2D84},  {0x2D85,0x2D85,0x2D85}, /* 2D84 */
  {0x2D86,0x2D86,0x2D86},  {0x2D87,0x2D87,0x2D87}, /* 2D86 */
  {0x2D88,0x2D88,0x2D88},  {0x2D89,0x2D89,0x2D89}, /* 2D88 */
  {0x2D8A,0x2D8A,0x2D8A},  {0x2D8B,0x2D8B,0x2D8B}, /* 2D8A */
  {0x2D8C,0x2D8C,0x2D8C},  {0x2D8D,0x2D8D,0x2D8D}, /* 2D8C */
  {0x2D8E,0x2D8E,0x2D8E},  {0x2D8F,0x2D8F,0x2D8F}, /* 2D8E */
  {0x2D90,0x2D90,0x2D90},  {0x2D91,0x2D91,0x2D91}, /* 2D90 */
  {0x2D92,0x2D92,0x2D92},  {0x2D93,0x2D93,0x2D93}, /* 2D92 */
  {0x2D94,0x2D94,0x2D94},  {0x2D95,0x2D95,0x2D95}, /* 2D94 */
  {0x2D96,0x2D96,0x2D96},  {0x2D97,0x2D97,0x2D97}, /* 2D96 */
  {0x2D98,0x2D98,0x2D98},  {0x2D99,0x2D99,0x2D99}, /* 2D98 */
  {0x2D9A,0x2D9A,0x2D9A},  {0x2D9B,0x2D9B,0x2D9B}, /* 2D9A */
  {0x2D9C,0x2D9C,0x2D9C},  {0x2D9D,0x2D9D,0x2D9D}, /* 2D9C */
  {0x2D9E,0x2D9E,0x2D9E},  {0x2D9F,0x2D9F,0x2D9F}, /* 2D9E */
  {0x2DA0,0x2DA0,0x2DA0},  {0x2DA1,0x2DA1,0x2DA1}, /* 2DA0 */
  {0x2DA2,0x2DA2,0x2DA2},  {0x2DA3,0x2DA3,0x2DA3}, /* 2DA2 */
  {0x2DA4,0x2DA4,0x2DA4},  {0x2DA5,0x2DA5,0x2DA5}, /* 2DA4 */
  {0x2DA6,0x2DA6,0x2DA6},  {0x2DA7,0x2DA7,0x2DA7}, /* 2DA6 */
  {0x2DA8,0x2DA8,0x2DA8},  {0x2DA9,0x2DA9,0x2DA9}, /* 2DA8 */
  {0x2DAA,0x2DAA,0x2DAA},  {0x2DAB,0x2DAB,0x2DAB}, /* 2DAA */
  {0x2DAC,0x2DAC,0x2DAC},  {0x2DAD,0x2DAD,0x2DAD}, /* 2DAC */
  {0x2DAE,0x2DAE,0x2DAE},  {0x2DAF,0x2DAF,0x2DAF}, /* 2DAE */
  {0x2DB0,0x2DB0,0x2DB0},  {0x2DB1,0x2DB1,0x2DB1}, /* 2DB0 */
  {0x2DB2,0x2DB2,0x2DB2},  {0x2DB3,0x2DB3,0x2DB3}, /* 2DB2 */
  {0x2DB4,0x2DB4,0x2DB4},  {0x2DB5,0x2DB5,0x2DB5}, /* 2DB4 */
  {0x2DB6,0x2DB6,0x2DB6},  {0x2DB7,0x2DB7,0x2DB7}, /* 2DB6 */
  {0x2DB8,0x2DB8,0x2DB8},  {0x2DB9,0x2DB9,0x2DB9}, /* 2DB8 */
  {0x2DBA,0x2DBA,0x2DBA},  {0x2DBB,0x2DBB,0x2DBB}, /* 2DBA */
  {0x2DBC,0x2DBC,0x2DBC},  {0x2DBD,0x2DBD,0x2DBD}, /* 2DBC */
  {0x2DBE,0x2DBE,0x2DBE},  {0x2DBF,0x2DBF,0x2DBF}, /* 2DBE */
  {0x2DC0,0x2DC0,0x2DC0},  {0x2DC1,0x2DC1,0x2DC1}, /* 2DC0 */
  {0x2DC2,0x2DC2,0x2DC2},  {0x2DC3,0x2DC3,0x2DC3}, /* 2DC2 */
  {0x2DC4,0x2DC4,0x2DC4},  {0x2DC5,0x2DC5,0x2DC5}, /* 2DC4 */
  {0x2DC6,0x2DC6,0x2DC6},  {0x2DC7,0x2DC7,0x2DC7}, /* 2DC6 */
  {0x2DC8,0x2DC8,0x2DC8},  {0x2DC9,0x2DC9,0x2DC9}, /* 2DC8 */
  {0x2DCA,0x2DCA,0x2DCA},  {0x2DCB,0x2DCB,0x2DCB}, /* 2DCA */
  {0x2DCC,0x2DCC,0x2DCC},  {0x2DCD,0x2DCD,0x2DCD}, /* 2DCC */
  {0x2DCE,0x2DCE,0x2DCE},  {0x2DCF,0x2DCF,0x2DCF}, /* 2DCE */
  {0x2DD0,0x2DD0,0x2DD0},  {0x2DD1,0x2DD1,0x2DD1}, /* 2DD0 */
  {0x2DD2,0x2DD2,0x2DD2},  {0x2DD3,0x2DD3,0x2DD3}, /* 2DD2 */
  {0x2DD4,0x2DD4,0x2DD4},  {0x2DD5,0x2DD5,0x2DD5}, /* 2DD4 */
  {0x2DD6,0x2DD6,0x2DD6},  {0x2DD7,0x2DD7,0x2DD7}, /* 2DD6 */
  {0x2DD8,0x2DD8,0x2DD8},  {0x2DD9,0x2DD9,0x2DD9}, /* 2DD8 */
  {0x2DDA,0x2DDA,0x2DDA},  {0x2DDB,0x2DDB,0x2DDB}, /* 2DDA */
  {0x2DDC,0x2DDC,0x2DDC},  {0x2DDD,0x2DDD,0x2DDD}, /* 2DDC */
  {0x2DDE,0x2DDE,0x2DDE},  {0x2DDF,0x2DDF,0x2DDF}, /* 2DDE */
  {0x2DE0,0x2DE0,0x2DE0},  {0x2DE1,0x2DE1,0x2DE1}, /* 2DE0 */
  {0x2DE2,0x2DE2,0x2DE2},  {0x2DE3,0x2DE3,0x2DE3}, /* 2DE2 */
  {0x2DE4,0x2DE4,0x2DE4},  {0x2DE5,0x2DE5,0x2DE5}, /* 2DE4 */
  {0x2DE6,0x2DE6,0x2DE6},  {0x2DE7,0x2DE7,0x2DE7}, /* 2DE6 */
  {0x2DE8,0x2DE8,0x2DE8},  {0x2DE9,0x2DE9,0x2DE9}, /* 2DE8 */
  {0x2DEA,0x2DEA,0x2DEA},  {0x2DEB,0x2DEB,0x2DEB}, /* 2DEA */
  {0x2DEC,0x2DEC,0x2DEC},  {0x2DED,0x2DED,0x2DED}, /* 2DEC */
  {0x2DEE,0x2DEE,0x2DEE},  {0x2DEF,0x2DEF,0x2DEF}, /* 2DEE */
  {0x2DF0,0x2DF0,0x2DF0},  {0x2DF1,0x2DF1,0x2DF1}, /* 2DF0 */
  {0x2DF2,0x2DF2,0x2DF2},  {0x2DF3,0x2DF3,0x2DF3}, /* 2DF2 */
  {0x2DF4,0x2DF4,0x2DF4},  {0x2DF5,0x2DF5,0x2DF5}, /* 2DF4 */
  {0x2DF6,0x2DF6,0x2DF6},  {0x2DF7,0x2DF7,0x2DF7}, /* 2DF6 */
  {0x2DF8,0x2DF8,0x2DF8},  {0x2DF9,0x2DF9,0x2DF9}, /* 2DF8 */
  {0x2DFA,0x2DFA,0x2DFA},  {0x2DFB,0x2DFB,0x2DFB}, /* 2DFA */
  {0x2DFC,0x2DFC,0x2DFC},  {0x2DFD,0x2DFD,0x2DFD}, /* 2DFC */
  {0x2DFE,0x2DFE,0x2DFE},  {0x2DFF,0x2DFF,0x2DFF}  /* 2DFE */
};

static const MY_UNICASE_CHARACTER u900p0A6[]= {
  {0xA600,0xA600,0xA600},  {0xA601,0xA601,0xA601}, /* A600 */
  {0xA602,0xA602,0xA602},  {0xA603,0xA603,0xA603}, /* A602 */
  {0xA604,0xA604,0xA604},  {0xA605,0xA605,0xA605}, /* A604 */
  {0xA606,0xA606,0xA606},  {0xA607,0xA607,0xA607}, /* A606 */
  {0xA608,0xA608,0xA608},  {0xA609,0xA609,0xA609}, /* A608 */
  {0xA60A,0xA60A,0xA60A},  {0xA60B,0xA60B,0xA60B}, /* A60A */
  {0xA60C,0xA60C,0xA60C},  {0xA60D,0xA60D,0xA60D}, /* A60C */
  {0xA60E,0xA60E,0xA60E},  {0xA60F,0xA60F,0xA60F}, /* A60E */
  {0xA610,0xA610,0xA610},  {0xA611,0xA611,0xA611}, /* A610 */
  {0xA612,0xA612,0xA612},  {0xA613,0xA613,0xA613}, /* A612 */
  {0xA614,0xA614,0xA614},  {0xA615,0xA615,0xA615}, /* A614 */
  {0xA616,0xA616,0xA616},  {0xA617,0xA617,0xA617}, /* A616 */
  {0xA618,0xA618,0xA618},  {0xA619,0xA619,0xA619}, /* A618 */
  {0xA61A,0xA61A,0xA61A},  {0xA61B,0xA61B,0xA61B}, /* A61A */
  {0xA61C,0xA61C,0xA61C},  {0xA61D,0xA61D,0xA61D}, /* A61C */
  {0xA61E,0xA61E,0xA61E},  {0xA61F,0xA61F,0xA61F}, /* A61E */
  {0xA620,0xA620,0xA620},  {0xA621,0xA621,0xA621}, /* A620 */
  {0xA622,0xA622,0xA622},  {0xA623,0xA623,0xA623}, /* A622 */
  {0xA624,0xA624,0xA624},  {0xA625,0xA625,0xA625}, /* A624 */
  {0xA626,0xA626,0xA626},  {0xA627,0xA627,0xA627}, /* A626 */
  {0xA628,0xA628,0xA628},  {0xA629,0xA629,0xA629}, /* A628 */
  {0xA62A,0xA62A,0xA62A},  {0xA62B,0xA62B,0xA62B}, /* A62A */
  {0xA62C,0xA62C,0xA62C},  {0xA62D,0xA62D,0xA62D}, /* A62C */
  {0xA62E,0xA62E,0xA62E},  {0xA62F,0xA62F,0xA62F}, /* A62E */
  {0xA630,0xA630,0xA630},  {0xA631,0xA631,0xA631}, /* A630 */
  {0xA632,0xA632,0xA632},  {0xA633,0xA633,0xA633}, /* A632 */
  {0xA634,0xA634,0xA634},  {0xA635,0xA635,0xA635}, /* A634 */
  {0xA636,0xA636,0xA636},  {0xA637,0xA637,0xA637}, /* A636 */
  {0xA638,0xA638,0xA638},  {0xA639,0xA639,0xA639}, /* A638 */
  {0xA63A,0xA63A,0xA63A},  {0xA63B,0xA63B,0xA63B}, /* A63A */
  {0xA63C,0xA63C,0xA63C},  {0xA63D,0xA63D,0xA63D}, /* A63C */
  {0xA63E,0xA63E,0xA63E},  {0xA63F,0xA63F,0xA63F}, /* A63E */
  {0xA640,0xA641,0xA640},  {0xA640,0xA641,0xA640}, /* A640 */
  {0xA642,0xA643,0xA642},  {0xA642,0xA643,0xA642}, /* A642 */
  {0xA644,0xA645,0xA644},  {0xA644,0xA645,0xA644}, /* A644 */
  {0xA646,0xA647,0xA646},  {0xA646,0xA647,0xA646}, /* A646 */
  {0xA648,0xA649,0xA648},  {0xA648,0xA649,0xA648}, /* A648 */
  {0xA64A,0xA64B,0xA64A},  {0xA64A,0xA64B,0xA64A}, /* A64A */
  {0xA64C,0xA64D,0xA64C},  {0xA64C,0xA64D,0xA64C}, /* A64C */
  {0xA64E,0xA64F,0xA64E},  {0xA64E,0xA64F,0xA64E}, /* A64E */
  {0xA650,0xA651,0xA650},  {0xA650,0xA651,0xA650}, /* A650 */
  {0xA652,0xA653,0xA652},  {0xA652,0xA653,0xA652}, /* A652 */
  {0xA654,0xA655,0xA654},  {0xA654,0xA655,0xA654}, /* A654 */
  {0xA656,0xA657,0xA656},  {0xA656,0xA657,0xA656}, /* A656 */
  {0xA658,0xA659,0xA658},  {0xA658,0xA659,0xA658}, /* A658 */
  {0xA65A,0xA65B,0xA65A},  {0xA65A,0xA65B,0xA65A}, /* A65A */
  {0xA65C,0xA65D,0xA65C},  {0xA65C,0xA65D,0xA65C}, /* A65C */
  {0xA65E,0xA65F,0xA65E},  {0xA65E,0xA65F,0xA65E}, /* A65E */
  {0xA660,0xA661,0xA660},  {0xA660,0xA661,0xA660}, /* A660 */
  {0xA662,0xA663,0xA662},  {0xA662,0xA663,0xA662}, /* A662 */
  {0xA664,0xA665,0xA664},  {0xA664,0xA665,0xA664}, /* A664 */
  {0xA666,0xA667,0xA666},  {0xA666,0xA667,0xA666}, /* A666 */
  {0xA668,0xA669,0xA668},  {0xA668,0xA669,0xA668}, /* A668 */
  {0xA66A,0xA66B,0xA66A},  {0xA66A,0xA66B,0xA66A}, /* A66A */
  {0xA66C,0xA66D,0xA66C},  {0xA66C,0xA66D,0xA66C}, /* A66C */
  {0xA66E,0xA66E,0xA66E},  {0xA66F,0xA66F,0xA66F}, /* A66E */
  {0xA670,0xA670,0xA670},  {0xA671,0xA671,0xA671}, /* A670 */
  {0xA672,0xA672,0xA672},  {0xA673,0xA673,0xA673}, /* A672 */
  {0xA674,0xA674,0xA674},  {0xA675,0xA675,0xA675}, /* A674 */
  {0xA676,0xA676,0xA676},  {0xA677,0xA677,0xA677}, /* A676 */
  {0xA678,0xA678,0xA678},  {0xA679,0xA679,0xA679}, /* A678 */
  {0xA67A,0xA67A,0xA67A},  {0xA67B,0xA67B,0xA67B}, /* A67A */
  {0xA67C,0xA67C,0xA67C},  {0xA67D,0xA67D,0xA67D}, /* A67C */
  {0xA67E,0xA67E,0xA67E},  {0xA67F,0xA67F,0xA67F}, /* A67E */
  {0xA680,0xA681,0xA680},  {0xA680,0xA681,0xA680}, /* A680 */
  {0xA682,0xA683,0xA682},  {0xA682,0xA683,0xA682}, /* A682 */
  {0xA684,0xA685,0xA684},  {0xA684,0xA685,0xA684}, /* A684 */
  {0xA686,0xA687,0xA686},  {0xA686,0xA687,0xA686}, /* A686 */
  {0xA688,0xA689,0xA688},  {0xA688,0xA689,0xA688}, /* A688 */
  {0xA68A,0xA68B,0xA68A},  {0xA68A,0xA68B,0xA68A}, /* A68A */
  {0xA68C,0xA68D,0xA68C},  {0xA68C,0xA68D,0xA68C}, /* A68C */
  {0xA68E,0xA68F,0xA68E},  {0xA68E,0xA68F,0xA68E}, /* A68E */
  {0xA690,0xA691,0xA690},  {0xA690,0xA691,0xA690}, /* A690 */
  {0xA692,0xA693,0xA692},  {0xA692,0xA693,0xA692}, /* A692 */
  {0xA694,0xA695,0xA694},  {0xA694,0xA695,0xA694}, /* A694 */
  {0xA696,0xA697,0xA696},  {0xA696,0xA697,0xA696}, /* A696 */
  {0xA698,0xA699,0xA698},  {0xA698,0xA699,0xA698}, /* A698 */
  {0xA69A,0xA69B,0xA69A},  {0xA69A,0xA69B,0xA69A}, /* A69A */
  {0xA69C,0xA69C,0xA69C},  {0xA69D,0xA69D,0xA69D}, /* A69C */
  {0xA69E,0xA69E,0xA69E},  {0xA69F,0xA69F,0xA69F}, /* A69E */
  {0xA6A0,0xA6A0,0xA6A0},  {0xA6A1,0xA6A1,0xA6A1}, /* A6A0 */
  {0xA6A2,0xA6A2,0xA6A2},  {0xA6A3,0xA6A3,0xA6A3}, /* A6A2 */
  {0xA6A4,0xA6A4,0xA6A4},  {0xA6A5,0xA6A5,0xA6A5}, /* A6A4 */
  {0xA6A6,0xA6A6,0xA6A6},  {0xA6A7,0xA6A7,0xA6A7}, /* A6A6 */
  {0xA6A8,0xA6A8,0xA6A8},  {0xA6A9,0xA6A9,0xA6A9}, /* A6A8 */
  {0xA6AA,0xA6AA,0xA6AA},  {0xA6AB,0xA6AB,0xA6AB}, /* A6AA */
  {0xA6AC,0xA6AC,0xA6AC},  {0xA6AD,0xA6AD,0xA6AD}, /* A6AC */
  {0xA6AE,0xA6AE,0xA6AE},  {0xA6AF,0xA6AF,0xA6AF}, /* A6AE */
  {0xA6B0,0xA6B0,0xA6B0},  {0xA6B1,0xA6B1,0xA6B1}, /* A6B0 */
  {0xA6B2,0xA6B2,0xA6B2},  {0xA6B3,0xA6B3,0xA6B3}, /* A6B2 */
  {0xA6B4,0xA6B4,0xA6B4},  {0xA6B5,0xA6B5,0xA6B5}, /* A6B4 */
  {0xA6B6,0xA6B6,0xA6B6},  {0xA6B7,0xA6B7,0xA6B7}, /* A6B6 */
  {0xA6B8,0xA6B8,0xA6B8},  {0xA6B9,0xA6B9,0xA6B9}, /* A6B8 */
  {0xA6BA,0xA6BA,0xA6BA},  {0xA6BB,0xA6BB,0xA6BB}, /* A6BA */
  {0xA6BC,0xA6BC,0xA6BC},  {0xA6BD,0xA6BD,0xA6BD}, /* A6BC */
  {0xA6BE,0xA6BE,0xA6BE},  {0xA6BF,0xA6BF,0xA6BF}, /* A6BE */
  {0xA6C0,0xA6C0,0xA6C0},  {0xA6C1,0xA6C1,0xA6C1}, /* A6C0 */
  {0xA6C2,0xA6C2,0xA6C2},  {0xA6C3,0xA6C3,0xA6C3}, /* A6C2 */
  {0xA6C4,0xA6C4,0xA6C4},  {0xA6C5,0xA6C5,0xA6C5}, /* A6C4 */
  {0xA6C6,0xA6C6,0xA6C6},  {0xA6C7,0xA6C7,0xA6C7}, /* A6C6 */
  {0xA6C8,0xA6C8,0xA6C8},  {0xA6C9,0xA6C9,0xA6C9}, /* A6C8 */
  {0xA6CA,0xA6CA,0xA6CA},  {0xA6CB,0xA6CB,0xA6CB}, /* A6CA */
  {0xA6CC,0xA6CC,0xA6CC},  {0xA6CD,0xA6CD,0xA6CD}, /* A6CC */
  {0xA6CE,0xA6CE,0xA6CE},  {0xA6CF,0xA6CF,0xA6CF}, /* A6CE */
  {0xA6D0,0xA6D0,0xA6D0},  {0xA6D1,0xA6D1,0xA6D1}, /* A6D0 */
  {0xA6D2,0xA6D2,0xA6D2},  {0xA6D3,0xA6D3,0xA6D3}, /* A6D2 */
  {0xA6D4,0xA6D4,0xA6D4},  {0xA6D5,0xA6D5,0xA6D5}, /* A6D4 */
  {0xA6D6,0xA6D6,0xA6D6},  {0xA6D7,0xA6D7,0xA6D7}, /* A6D6 */
  {0xA6D8,0xA6D8,0xA6D8},  {0xA6D9,0xA6D9,0xA6D9}, /* A6D8 */
  {0xA6DA,0xA6DA,0xA6DA},  {0xA6DB,0xA6DB,0xA6DB}, /* A6DA */
  {0xA6DC,0xA6DC,0xA6DC},  {0xA6DD,0xA6DD,0xA6DD}, /* A6DC */
  {0xA6DE,0xA6DE,0xA6DE},  {0xA6DF,0xA6DF,0xA6DF}, /* A6DE */
  {0xA6E0,0xA6E0,0xA6E0},  {0xA6E1,0xA6E1,0xA6E1}, /* A6E0 */
  {0xA6E2,0xA6E2,0xA6E2},  {0xA6E3,0xA6E3,0xA6E3}, /* A6E2 */
  {0xA6E4,0xA6E4,0xA6E4},  {0xA6E5,0xA6E5,0xA6E5}, /* A6E4 */
  {0xA6E6,0xA6E6,0xA6E6},  {0xA6E7,0xA6E7,0xA6E7}, /* A6E6 */
  {0xA6E8,0xA6E8,0xA6E8},  {0xA6E9,0xA6E9,0xA6E9}, /* A6E8 */
  {0xA6EA,0xA6EA,0xA6EA},  {0xA6EB,0xA6EB,0xA6EB}, /* A6EA */
  {0xA6EC,0xA6EC,0xA6EC},  {0xA6ED,0xA6ED,0xA6ED}, /* A6EC */
  {0xA6EE,0xA6EE,0xA6EE},  {0xA6EF,0xA6EF,0xA6EF}, /* A6EE */
  {0xA6F0,0xA6F0,0xA6F0},  {0xA6F1,0xA6F1,0xA6F1}, /* A6F0 */
  {0xA6F2,0xA6F2,0xA6F2},  {0xA6F3,0xA6F3,0xA6F3}, /* A6F2 */
  {0xA6F4,0xA6F4,0xA6F4},  {0xA6F5,0xA6F5,0xA6F5}, /* A6F4 */
  {0xA6F6,0xA6F6,0xA6F6},  {0xA6F7,0xA6F7,0xA6F7}, /* A6F6 */
  {0xA6F8,0xA6F8,0xA6F8},  {0xA6F9,0xA6F9,0xA6F9}, /* A6F8 */
  {0xA6FA,0xA6FA,0xA6FA},  {0xA6FB,0xA6FB,0xA6FB}, /* A6FA */
  {0xA6FC,0xA6FC,0xA6FC},  {0xA6FD,0xA6FD,0xA6FD}, /* A6FC */
  {0xA6FE,0xA6FE,0xA6FE},  {0xA6FF,0xA6FF,0xA6FF}  /* A6FE */
};

static const MY_UNICASE_CHARACTER u900p0A7[]= {
  {0xA700,0xA700,0xA700},  {0xA701,0xA701,0xA701}, /* A700 */
  {0xA702,0xA702,0xA702},  {0xA703,0xA703,0xA703}, /* A702 */
  {0xA704,0xA704,0xA704},  {0xA705,0xA705,0xA705}, /* A704 */
  {0xA706,0xA706,0xA706},  {0xA707,0xA707,0xA707}, /* A706 */
  {0xA708,0xA708,0xA708},  {0xA709,0xA709,0xA709}, /* A708 */
  {0xA70A,0xA70A,0xA70A},  {0xA70B,0xA70B,0xA70B}, /* A70A */
  {0xA70C,0xA70C,0xA70C},  {0xA70D,0xA70D,0xA70D}, /* A70C */
  {0xA70E,0xA70E,0xA70E},  {0xA70F,0xA70F,0xA70F}, /* A70E */
  {0xA710,0xA710,0xA710},  {0xA711,0xA711,0xA711}, /* A710 */
  {0xA712,0xA712,0xA712},  {0xA713,0xA713,0xA713}, /* A712 */
  {0xA714,0xA714,0xA714},  {0xA715,0xA715,0xA715}, /* A714 */
  {0xA716,0xA716,0xA716},  {0xA717,0xA717,0xA717}, /* A716 */
  {0xA718,0xA718,0xA718},  {0xA719,0xA719,0xA719}, /* A718 */
  {0xA71A,0xA71A,0xA71A},  {0xA71B,0xA71B,0xA71B}, /* A71A */
  {0xA71C,0xA71C,0xA71C},  {0xA71D,0xA71D,0xA71D}, /* A71C */
  {0xA71E,0xA71E,0xA71E},  {0xA71F,0xA71F,0xA71F}, /* A71E */
  {0xA720,0xA720,0xA720},  {0xA721,0xA721,0xA721}, /* A720 */
  {0xA722,0xA723,0xA722},  {0xA722,0xA723,0xA722}, /* A722 */
  {0xA724,0xA725,0xA724},  {0xA724,0xA725,0xA724}, /* A724 */
  {0xA726,0xA727,0xA726},  {0xA726,0xA727,0xA726}, /* A726 */
  {0xA728,0xA729,0xA728},  {0xA728,0xA729,0xA728}, /* A728 */
  {0xA72A,0xA72B,0xA72A},  {0xA72A,0xA72B,0xA72A}, /* A72A */
  {0xA72C,0xA72D,0xA72C},  {0xA72C,0xA72D,0xA72C}, /* A72C */
  {0xA72E,0xA72F,0xA72E},  {0xA72E,0xA72F,0xA72E}, /* A72E */
  {0xA730,0xA730,0xA730},  {0xA731,0xA731,0xA731}, /* A730 */
  {0xA732,0xA733,0xA732},  {0xA732,0xA733,0xA732}, /* A732 */
  {0xA734,0xA735,0xA734},  {0xA734,0xA735,0xA734}, /* A734 */
  {0xA736,0xA737,0xA736},  {0xA736,0xA737,0xA736}, /* A736 */
  {0xA738,0xA739,0xA738},  {0xA738,0xA739,0xA738}, /* A738 */
  {0xA73A,0xA73B,0xA73A},  {0xA73A,0xA73B,0xA73A}, /* A73A */
  {0xA73C,0xA73D,0xA73C},  {0xA73C,0xA73D,0xA73C}, /* A73C */
  {0xA73E,0xA73F,0xA73E},  {0xA73E,0xA73F,0xA73E}, /* A73E */
  {0xA740,0xA741,0xA740},  {0xA740,0xA741,0xA740}, /* A740 */
  {0xA742,0xA743,0xA742},  {0xA742,0xA743,0xA742}, /* A742 */
  {0xA744,0xA745,0xA744},  {0xA744,0xA745,0xA744}, /* A744 */
  {0xA746,0xA747,0xA746},  {0xA746,0xA747,0xA746}, /* A746 */
  {0xA748,0xA749,0xA748},  {0xA748,0xA749,0xA748}, /* A748 */
  {0xA74A,0xA74B,0xA74A},  {0xA74A,0xA74B,0xA74A}, /* A74A */
  {0xA74C,0xA74D,0xA74C},  {0xA74C,0xA74D,0xA74C}, /* A74C */
  {0xA74E,0xA74F,0xA74E},  {0xA74E,0xA74F,0xA74E}, /* A74E */
  {0xA750,0xA751,0xA750},  {0xA750,0xA751,0xA750}, /* A750 */
  {0xA752,0xA753,0xA752},  {0xA752,0xA753,0xA752}, /* A752 */
  {0xA754,0xA755,0xA754},  {0xA754,0xA755,0xA754}, /* A754 */
  {0xA756,0xA757,0xA756},  {0xA756,0xA757,0xA756}, /* A756 */
  {0xA758,0xA759,0xA758},  {0xA758,0xA759,0xA758}, /* A758 */
  {0xA75A,0xA75B,0xA75A},  {0xA75A,0xA75B,0xA75A}, /* A75A */
  {0xA75C,0xA75D,0xA75C},  {0xA75C,0xA75D,0xA75C}, /* A75C */
  {0xA75E,0xA75F,0xA75E},  {0xA75E,0xA75F,0xA75E}, /* A75E */
  {0xA760,0xA761,0xA760},  {0xA760,0xA761,0xA760}, /* A760 */
  {0xA762,0xA763,0xA762},  {0xA762,0xA763,0xA762}, /* A762 */
  {0xA764,0xA765,0xA764},  {0xA764,0xA765,0xA764}, /* A764 */
  {0xA766,0xA767,0xA766},  {0xA766,0xA767,0xA766}, /* A766 */
  {0xA768,0xA769,0xA768},  {0xA768,0xA769,0xA768}, /* A768 */
  {0xA76A,0xA76B,0xA76A},  {0xA76A,0xA76B,0xA76A}, /* A76A */
  {0xA76C,0xA76D,0xA76C},  {0xA76C,0xA76D,0xA76C}, /* A76C */
  {0xA76E,0xA76F,0xA76E},  {0xA76E,0xA76F,0xA76E}, /* A76E */
  {0xA770,0xA770,0xA770},  {0xA771,0xA771,0xA771}, /* A770 */
  {0xA772,0xA772,0xA772},  {0xA773,0xA773,0xA773}, /* A772 */
  {0xA774,0xA774,0xA774},  {0xA775,0xA775,0xA775}, /* A774 */
  {0xA776,0xA776,0xA776},  {0xA777,0xA777,0xA777}, /* A776 */
  {0xA778,0xA778,0xA778},  {0xA779,0xA77A,0xA779}, /* A778 */
  {0xA779,0xA77A,0xA779},  {0xA77B,0xA77C,0xA77B}, /* A77A */
  {0xA77B,0xA77C,0xA77B},  {0xA77D,0x1D79,0xA77D}, /* A77C */
  {0xA77E,0xA77F,0xA77E},  {0xA77E,0xA77F,0xA77E}, /* A77E */
  {0xA780,0xA781,0xA780},  {0xA780,0xA781,0xA780}, /* A780 */
  {0xA782,0xA783,0xA782},  {0xA782,0xA783,0xA782}, /* A782 */
  {0xA784,0xA785,0xA784},  {0xA784,0xA785,0xA784}, /* A784 */
  {0xA786,0xA787,0xA786},  {0xA786,0xA787,0xA786}, /* A786 */
  {0xA788,0xA788,0xA788},  {0xA789,0xA789,0xA789}, /* A788 */
  {0xA78A,0xA78A,0xA78A},  {0xA78B,0xA78C,0xA78B}, /* A78A */
  {0xA78B,0xA78C,0xA78B},  {0xA78D,0x0265,0xA78D}, /* A78C */
  {0xA78E,0xA78E,0xA78E},  {0xA78F,0xA78F,0xA78F}, /* A78E */
  {0xA790,0xA791,0xA790},  {0xA790,0xA791,0xA790}, /* A790 */
  {0xA792,0xA793,0xA792},  {0xA792,0xA793,0xA792}, /* A792 */
  {0xA794,0xA794,0xA794},  {0xA795,0xA795,0xA795}, /* A794 */
  {0xA796,0xA797,0xA796},  {0xA796,0xA797,0xA796}, /* A796 */
  {0xA798,0xA799,0xA798},  {0xA798,0xA799,0xA798}, /* A798 */
  {0xA79A,0xA79B,0xA79A},  {0xA79A,0xA79B,0xA79A}, /* A79A */
  {0xA79C,0xA79D,0xA79C},  {0xA79C,0xA79D,0xA79C}, /* A79C */
  {0xA79E,0xA79F,0xA79E},  {0xA79E,0xA79F,0xA79E}, /* A79E */
  {0xA7A0,0xA7A1,0xA7A0},  {0xA7A0,0xA7A1,0xA7A0}, /* A7A0 */
  {0xA7A2,0xA7A3,0xA7A2},  {0xA7A2,0xA7A3,0xA7A2}, /* A7A2 */
  {0xA7A4,0xA7A5,0xA7A4},  {0xA7A4,0xA7A5,0xA7A4}, /* A7A4 */
  {0xA7A6,0xA7A7,0xA7A6},  {0xA7A6,0xA7A7,0xA7A6}, /* A7A6 */
  {0xA7A8,0xA7A9,0xA7A8},  {0xA7A8,0xA7A9,0xA7A8}, /* A7A8 */
  {0xA7AA,0x0266,0xA7AA},  {0xA7AB,0x025C,0xA7AB}, /* A7AA */
  {0xA7AC,0x0261,0xA7AC},  {0xA7AD,0x026C,0xA7AD}, /* A7AC */
  {0xA7AE,0x026A,0xA7AE},  {0xA7AF,0xA7AF,0xA7AF}, /* A7AE */
  {0xA7B0,0x029E,0xA7B0},  {0xA7B1,0x0287,0xA7B1}, /* A7B0 */
  {0xA7B2,0x029D,0xA7B2},  {0xA7B3,0xAB53,0xA7B3}, /* A7B2 */
  {0xA7B4,0xA7B5,0xA7B4},  {0xA7B4,0xA7B5,0xA7B4}, /* A7B4 */
  {0xA7B6,0xA7B7,0xA7B6},  {0xA7B6,0xA7B7,0xA7B6}, /* A7B6 */
  {0xA7B8,0xA7B8,0xA7B8},  {0xA7B9,0xA7B9,0xA7B9}, /* A7B8 */
  {0xA7BA,0xA7BA,0xA7BA},  {0xA7BB,0xA7BB,0xA7BB}, /* A7BA */
  {0xA7BC,0xA7BC,0xA7BC},  {0xA7BD,0xA7BD,0xA7BD}, /* A7BC */
  {0xA7BE,0xA7BE,0xA7BE},  {0xA7BF,0xA7BF,0xA7BF}, /* A7BE */
  {0xA7C0,0xA7C0,0xA7C0},  {0xA7C1,0xA7C1,0xA7C1}, /* A7C0 */
  {0xA7C2,0xA7C2,0xA7C2},  {0xA7C3,0xA7C3,0xA7C3}, /* A7C2 */
  {0xA7C4,0xA7C4,0xA7C4},  {0xA7C5,0xA7C5,0xA7C5}, /* A7C4 */
  {0xA7C6,0xA7C6,0xA7C6},  {0xA7C7,0xA7C7,0xA7C7}, /* A7C6 */
  {0xA7C8,0xA7C8,0xA7C8},  {0xA7C9,0xA7C9,0xA7C9}, /* A7C8 */
  {0xA7CA,0xA7CA,0xA7CA},  {0xA7CB,0xA7CB,0xA7CB}, /* A7CA */
  {0xA7CC,0xA7CC,0xA7CC},  {0xA7CD,0xA7CD,0xA7CD}, /* A7CC */
  {0xA7CE,0xA7CE,0xA7CE},  {0xA7CF,0xA7CF,0xA7CF}, /* A7CE */
  {0xA7D0,0xA7D0,0xA7D0},  {0xA7D1,0xA7D1,0xA7D1}, /* A7D0 */
  {0xA7D2,0xA7D2,0xA7D2},  {0xA7D3,0xA7D3,0xA7D3}, /* A7D2 */
  {0xA7D4,0xA7D4,0xA7D4},  {0xA7D5,0xA7D5,0xA7D5}, /* A7D4 */
  {0xA7D6,0xA7D6,0xA7D6},  {0xA7D7,0xA7D7,0xA7D7}, /* A7D6 */
  {0xA7D8,0xA7D8,0xA7D8},  {0xA7D9,0xA7D9,0xA7D9}, /* A7D8 */
  {0xA7DA,0xA7DA,0xA7DA},  {0xA7DB,0xA7DB,0xA7DB}, /* A7DA */
  {0xA7DC,0xA7DC,0xA7DC},  {0xA7DD,0xA7DD,0xA7DD}, /* A7DC */
  {0xA7DE,0xA7DE,0xA7DE},  {0xA7DF,0xA7DF,0xA7DF}, /* A7DE */
  {0xA7E0,0xA7E0,0xA7E0},  {0xA7E1,0xA7E1,0xA7E1}, /* A7E0 */
  {0xA7E2,0xA7E2,0xA7E2},  {0xA7E3,0xA7E3,0xA7E3}, /* A7E2 */
  {0xA7E4,0xA7E4,0xA7E4},  {0xA7E5,0xA7E5,0xA7E5}, /* A7E4 */
  {0xA7E6,0xA7E6,0xA7E6},  {0xA7E7,0xA7E7,0xA7E7}, /* A7E6 */
  {0xA7E8,0xA7E8,0xA7E8},  {0xA7E9,0xA7E9,0xA7E9}, /* A7E8 */
  {0xA7EA,0xA7EA,0xA7EA},  {0xA7EB,0xA7EB,0xA7EB}, /* A7EA */
  {0xA7EC,0xA7EC,0xA7EC},  {0xA7ED,0xA7ED,0xA7ED}, /* A7EC */
  {0xA7EE,0xA7EE,0xA7EE},  {0xA7EF,0xA7EF,0xA7EF}, /* A7EE */
  {0xA7F0,0xA7F0,0xA7F0},  {0xA7F1,0xA7F1,0xA7F1}, /* A7F0 */
  {0xA7F2,0xA7F2,0xA7F2},  {0xA7F3,0xA7F3,0xA7F3}, /* A7F2 */
  {0xA7F4,0xA7F4,0xA7F4},  {0xA7F5,0xA7F5,0xA7F5}, /* A7F4 */
  {0xA7F6,0xA7F6,0xA7F6},  {0xA7F7,0xA7F7,0xA7F7}, /* A7F6 */
  {0xA7F8,0xA7F8,0xA7F8},  {0xA7F9,0xA7F9,0xA7F9}, /* A7F8 */
  {0xA7FA,0xA7FA,0xA7FA},  {0xA7FB,0xA7FB,0xA7FB}, /* A7FA */
  {0xA7FC,0xA7FC,0xA7FC},  {0xA7FD,0xA7FD,0xA7FD}, /* A7FC */
  {0xA7FE,0xA7FE,0xA7FE},  {0xA7FF,0xA7FF,0xA7FF}  /* A7FE */
};

static const MY_UNICASE_CHARACTER u900p0AB[]= {
  {0xAB00,0xAB00,0xAB00},  {0xAB01,0xAB01,0xAB01}, /* AB00 */
  {0xAB02,0xAB02,0xAB02},  {0xAB03,0xAB03,0xAB03}, /* AB02 */
  {0xAB04,0xAB04,0xAB04},  {0xAB05,0xAB05,0xAB05}, /* AB04 */
  {0xAB06,0xAB06,0xAB06},  {0xAB07,0xAB07,0xAB07}, /* AB06 */
  {0xAB08,0xAB08,0xAB08},  {0xAB09,0xAB09,0xAB09}, /* AB08 */
  {0xAB0A,0xAB0A,0xAB0A},  {0xAB0B,0xAB0B,0xAB0B}, /* AB0A */
  {0xAB0C,0xAB0C,0xAB0C},  {0xAB0D,0xAB0D,0xAB0D}, /* AB0C */
  {0xAB0E,0xAB0E,0xAB0E},  {0xAB0F,0xAB0F,0xAB0F}, /* AB0E */
  {0xAB10,0xAB10,0xAB10},  {0xAB11,0xAB11,0xAB11}, /* AB10 */
  {0xAB12,0xAB12,0xAB12},  {0xAB13,0xAB13,0xAB13}, /* AB12 */
  {0xAB14,0xAB14,0xAB14},  {0xAB15,0xAB15,0xAB15}, /* AB14 */
  {0xAB16,0xAB16,0xAB16},  {0xAB17,0xAB17,0xAB17}, /* AB16 */
  {0xAB18,0xAB18,0xAB18},  {0xAB19,0xAB19,0xAB19}, /* AB18 */
  {0xAB1A,0xAB1A,0xAB1A},  {0xAB1B,0xAB1B,0xAB1B}, /* AB1A */
  {0xAB1C,0xAB1C,0xAB1C},  {0xAB1D,0xAB1D,0xAB1D}, /* AB1C */
  {0xAB1E,0xAB1E,0xAB1E},  {0xAB1F,0xAB1F,0xAB1F}, /* AB1E */
  {0xAB20,0xAB20,0xAB20},  {0xAB21,0xAB21,0xAB21}, /* AB20 */
  {0xAB22,0xAB22,0xAB22},  {0xAB23,0xAB23,0xAB23}, /* AB22 */
  {0xAB24,0xAB24,0xAB24},  {0xAB25,0xAB25,0xAB25}, /* AB24 */
  {0xAB26,0xAB26,0xAB26},  {0xAB27,0xAB27,0xAB27}, /* AB26 */
  {0xAB28,0xAB28,0xAB28},  {0xAB29,0xAB29,0xAB29}, /* AB28 */
  {0xAB2A,0xAB2A,0xAB2A},  {0xAB2B,0xAB2B,0xAB2B}, /* AB2A */
  {0xAB2C,0xAB2C,0xAB2C},  {0xAB2D,0xAB2D,0xAB2D}, /* AB2C */
  {0xAB2E,0xAB2E,0xAB2E},  {0xAB2F,0xAB2F,0xAB2F}, /* AB2E */
  {0xAB30,0xAB30,0xAB30},  {0xAB31,0xAB31,0xAB31}, /* AB30 */
  {0xAB32,0xAB32,0xAB32},  {0xAB33,0xAB33,0xAB33}, /* AB32 */
  {0xAB34,0xAB34,0xAB34},  {0xAB35,0xAB35,0xAB35}, /* AB34 */
  {0xAB36,0xAB36,0xAB36},  {0xAB37,0xAB37,0xAB37}, /* AB36 */
  {0xAB38,0xAB38,0xAB38},  {0xAB39,0xAB39,0xAB39}, /* AB38 */
  {0xAB3A,0xAB3A,0xAB3A},  {0xAB3B,0xAB3B,0xAB3B}, /* AB3A */
  {0xAB3C,0xAB3C,0xAB3C},  {0xAB3D,0xAB3D,0xAB3D}, /* AB3C */
  {0xAB3E,0xAB3E,0xAB3E},  {0xAB3F,0xAB3F,0xAB3F}, /* AB3E */
  {0xAB40,0xAB40,0xAB40},  {0xAB41,0xAB41,0xAB41}, /* AB40 */
  {0xAB42,0xAB42,0xAB42},  {0xAB43,0xAB43,0xAB43}, /* AB42 */
  {0xAB44,0xAB44,0xAB44},  {0xAB45,0xAB45,0xAB45}, /* AB44 */
  {0xAB46,0xAB46,0xAB46},  {0xAB47,0xAB47,0xAB47}, /* AB46 */
  {0xAB48,0xAB48,0xAB48},  {0xAB49,0xAB49,0xAB49}, /* AB48 */
  {0xAB4A,0xAB4A,0xAB4A},  {0xAB4B,0xAB4B,0xAB4B}, /* AB4A */
  {0xAB4C,0xAB4C,0xAB4C},  {0xAB4D,0xAB4D,0xAB4D}, /* AB4C */
  {0xAB4E,0xAB4E,0xAB4E},  {0xAB4F,0xAB4F,0xAB4F}, /* AB4E */
  {0xAB50,0xAB50,0xAB50},  {0xAB51,0xAB51,0xAB51}, /* AB50 */
  {0xAB52,0xAB52,0xAB52},  {0xA7B3,0xAB53,0xA7B3}, /* AB52 */
  {0xAB54,0xAB54,0xAB54},  {0xAB55,0xAB55,0xAB55}, /* AB54 */
  {0xAB56,0xAB56,0xAB56},  {0xAB57,0xAB57,0xAB57}, /* AB56 */
  {0xAB58,0xAB58,0xAB58},  {0xAB59,0xAB59,0xAB59}, /* AB58 */
  {0xAB5A,0xAB5A,0xAB5A},  {0xAB5B,0xAB5B,0xAB5B}, /* AB5A */
  {0xAB5C,0xAB5C,0xAB5C},  {0xAB5D,0xAB5D,0xAB5D}, /* AB5C */
  {0xAB5E,0xAB5E,0xAB5E},  {0xAB5F,0xAB5F,0xAB5F}, /* AB5E */
  {0xAB60,0xAB60,0xAB60},  {0xAB61,0xAB61,0xAB61}, /* AB60 */
  {0xAB62,0xAB62,0xAB62},  {0xAB63,0xAB63,0xAB63}, /* AB62 */
  {0xAB64,0xAB64,0xAB64},  {0xAB65,0xAB65,0xAB65}, /* AB64 */
  {0xAB66,0xAB66,0xAB66},  {0xAB67,0xAB67,0xAB67}, /* AB66 */
  {0xAB68,0xAB68,0xAB68},  {0xAB69,0xAB69,0xAB69}, /* AB68 */
  {0xAB6A,0xAB6A,0xAB6A},  {0xAB6B,0xAB6B,0xAB6B}, /* AB6A */
  {0xAB6C,0xAB6C,0xAB6C},  {0xAB6D,0xAB6D,0xAB6D}, /* AB6C */
  {0xAB6E,0xAB6E,0xAB6E},  {0xAB6F,0xAB6F,0xAB6F}, /* AB6E */
  {0x13A0,0xAB70,0x13A0},  {0x13A1,0xAB71,0x13A1}, /* AB70 */
  {0x13A2,0xAB72,0x13A2},  {0x13A3,0xAB73,0x13A3}, /* AB72 */
  {0x13A4,0xAB74,0x13A4},  {0x13A5,0xAB75,0x13A5}, /* AB74 */
  {0x13A6,0xAB76,0x13A6},  {0x13A7,0xAB77,0x13A7}, /* AB76 */
  {0x13A8,0xAB78,0x13A8},  {0x13A9,0xAB79,0x13A9}, /* AB78 */
  {0x13AA,0xAB7A,0x13AA},  {0x13AB,0xAB7B,0x13AB}, /* AB7A */
  {0x13AC,0xAB7C,0x13AC},  {0x13AD,0xAB7D,0x13AD}, /* AB7C */
  {0x13AE,0xAB7E,0x13AE},  {0x13AF,0xAB7F,0x13AF}, /* AB7E */
  {0x13B0,0xAB80,0x13B0},  {0x13B1,0xAB81,0x13B1}, /* AB80 */
  {0x13B2,0xAB82,0x13B2},  {0x13B3,0xAB83,0x13B3}, /* AB82 */
  {0x13B4,0xAB84,0x13B4},  {0x13B5,0xAB85,0x13B5}, /* AB84 */
  {0x13B6,0xAB86,0x13B6},  {0x13B7,0xAB87,0x13B7}, /* AB86 */
  {0x13B8,0xAB88,0x13B8},  {0x13B9,0xAB89,0x13B9}, /* AB88 */
  {0x13BA,0xAB8A,0x13BA},  {0x13BB,0xAB8B,0x13BB}, /* AB8A */
  {0x13BC,0xAB8C,0x13BC},  {0x13BD,0xAB8D,0x13BD}, /* AB8C */
  {0x13BE,0xAB8E,0x13BE},  {0x13BF,0xAB8F,0x13BF}, /* AB8E */
  {0x13C0,0xAB90,0x13C0},  {0x13C1,0xAB91,0x13C1}, /* AB90 */
  {0x13C2,0xAB92,0x13C2},  {0x13C3,0xAB93,0x13C3}, /* AB92 */
  {0x13C4,0xAB94,0x13C4},  {0x13C5,0xAB95,0x13C5}, /* AB94 */
  {0x13C6,0xAB96,0x13C6},  {0x13C7,0xAB97,0x13C7}, /* AB96 */
  {0x13C8,0xAB98,0x13C8},  {0x13C9,0xAB99,0x13C9}, /* AB98 */
  {0x13CA,0xAB9A,0x13CA},  {0x13CB,0xAB9B,0x13CB}, /* AB9A */
  {0x13CC,0xAB9C,0x13CC},  {0x13CD,0xAB9D,0x13CD}, /* AB9C */
  {0x13CE,0xAB9E,0x13CE},  {0x13CF,0xAB9F,0x13CF}, /* AB9E */
  {0x13D0,0xABA0,0x13D0},  {0x13D1,0xABA1,0x13D1}, /* ABA0 */
  {0x13D2,0xABA2,0x13D2},  {0x13D3,0xABA3,0x13D3}, /* ABA2 */
  {0x13D4,0xABA4,0x13D4},  {0x13D5,0xABA5,0x13D5}, /* ABA4 */
  {0x13D6,0xABA6,0x13D6},  {0x13D7,0xABA7,0x13D7}, /* ABA6 */
  {0x13D8,0xABA8,0x13D8},  {0x13D9,0xABA9,0x13D9}, /* ABA8 */
  {0x13DA,0xABAA,0x13DA},  {0x13DB,0xABAB,0x13DB}, /* ABAA */
  {0x13DC,0xABAC,0x13DC},  {0x13DD,0xABAD,0x13DD}, /* ABAC */
  {0x13DE,0xABAE,0x13DE},  {0x13DF,0xABAF,0x13DF}, /* ABAE */
  {0x13E0,0xABB0,0x13E0},  {0x13E1,0xABB1,0x13E1}, /* ABB0 */
  {0x13E2,0xABB2,0x13E2},  {0x13E3,0xABB3,0x13E3}, /* ABB2 */
  {0x13E4,0xABB4,0x13E4},  {0x13E5,0xABB5,0x13E5}, /* ABB4 */
  {0x13E6,0xABB6,0x13E6},  {0x13E7,0xABB7,0x13E7}, /* ABB6 */
  {0x13E8,0xABB8,0x13E8},  {0x13E9,0xABB9,0x13E9}, /* ABB8 */
  {0x13EA,0xABBA,0x13EA},  {0x13EB,0xABBB,0x13EB}, /* ABBA */
  {0x13EC,0xABBC,0x13EC},  {0x13ED,0xABBD,0x13ED}, /* ABBC */
  {0x13EE,0xABBE,0x13EE},  {0x13EF,0xABBF,0x13EF}, /* ABBE */
  {0xABC0,0xABC0,0xABC0},  {0xABC1,0xABC1,0xABC1}, /* ABC0 */
  {0xABC2,0xABC2,0xABC2},  {0xABC3,0xABC3,0xABC3}, /* ABC2 */
  {0xABC4,0xABC4,0xABC4},  {0xABC5,0xABC5,0xABC5}, /* ABC4 */
  {0xABC6,0xABC6,0xABC6},  {0xABC7,0xABC7,0xABC7}, /* ABC6 */
  {0xABC8,0xABC8,0xABC8},  {0xABC9,0xABC9,0xABC9}, /* ABC8 */
  {0xABCA,0xABCA,0xABCA},  {0xABCB,0xABCB,0xABCB}, /* ABCA */
  {0xABCC,0xABCC,0xABCC},  {0xABCD,0xABCD,0xABCD}, /* ABCC */
  {0xABCE,0xABCE,0xABCE},  {0xABCF,0xABCF,0xABCF}, /* ABCE */
  {0xABD0,0xABD0,0xABD0},  {0xABD1,0xABD1,0xABD1}, /* ABD0 */
  {0xABD2,0xABD2,0xABD2},  {0xABD3,0xABD3,0xABD3}, /* ABD2 */
  {0xABD4,0xABD4,0xABD4},  {0xABD5,0xABD5,0xABD5}, /* ABD4 */
  {0xABD6,0xABD6,0xABD6},  {0xABD7,0xABD7,0xABD7}, /* ABD6 */
  {0xABD8,0xABD8,0xABD8},  {0xABD9,0xABD9,0xABD9}, /* ABD8 */
  {0xABDA,0xABDA,0xABDA},  {0xABDB,0xABDB,0xABDB}, /* ABDA */
  {0xABDC,0xABDC,0xABDC},  {0xABDD,0xABDD,0xABDD}, /* ABDC */
  {0xABDE,0xABDE,0xABDE},  {0xABDF,0xABDF,0xABDF}, /* ABDE */
  {0xABE0,0xABE0,0xABE0},  {0xABE1,0xABE1,0xABE1}, /* ABE0 */
  {0xABE2,0xABE2,0xABE2},  {0xABE3,0xABE3,0xABE3}, /* ABE2 */
  {0xABE4,0xABE4,0xABE4},  {0xABE5,0xABE5,0xABE5}, /* ABE4 */
  {0xABE6,0xABE6,0xABE6},  {0xABE7,0xABE7,0xABE7}, /* ABE6 */
  {0xABE8,0xABE8,0xABE8},  {0xABE9,0xABE9,0xABE9}, /* ABE8 */
  {0xABEA,0xABEA,0xABEA},  {0xABEB,0xABEB,0xABEB}, /* ABEA */
  {0xABEC,0xABEC,0xABEC},  {0xABED,0xABED,0xABED}, /* ABEC */
  {0xABEE,0xABEE,0xABEE},  {0xABEF,0xABEF,0xABEF}, /* ABEE */
  {0xABF0,0xABF0,0xABF0},  {0xABF1,0xABF1,0xABF1}, /* ABF0 */
  {0xABF2,0xABF2,0xABF2},  {0xABF3,0xABF3,0xABF3}, /* ABF2 */
  {0xABF4,0xABF4,0xABF4},  {0xABF5,0xABF5,0xABF5}, /* ABF4 */
  {0xABF6,0xABF6,0xABF6},  {0xABF7,0xABF7,0xABF7}, /* ABF6 */
  {0xABF8,0xABF8,0xABF8},  {0xABF9,0xABF9,0xABF9}, /* ABF8 */
  {0xABFA,0xABFA,0xABFA},  {0xABFB,0xABFB,0xABFB}, /* ABFA */
  {0xABFC,0xABFC,0xABFC},  {0xABFD,0xABFD,0xABFD}, /* ABFC */
  {0xABFE,0xABFE,0xABFE},  {0xABFF,0xABFF,0xABFF}  /* ABFE */
};

static const MY_UNICASE_CHARACTER u900p0FF[]= {
  {0xFF00,0xFF00,0xFF00},  {0xFF01,0xFF01,0xFF01}, /* FF00 */
  {0xFF02,0xFF02,0xFF02},  {0xFF03,0xFF03,0xFF03}, /* FF02 */
  {0xFF04,0xFF04,0xFF04},  {0xFF05,0xFF05,0xFF05}, /* FF04 */
  {0xFF06,0xFF06,0xFF06},  {0xFF07,0xFF07,0xFF07}, /* FF06 */
  {0xFF08,0xFF08,0xFF08},  {0xFF09,0xFF09,0xFF09}, /* FF08 */
  {0xFF0A,0xFF0A,0xFF0A},  {0xFF0B,0xFF0B,0xFF0B}, /* FF0A */
  {0xFF0C,0xFF0C,0xFF0C},  {0xFF0D,0xFF0D,0xFF0D}, /* FF0C */
  {0xFF0E,0xFF0E,0xFF0E},  {0xFF0F,0xFF0F,0xFF0F}, /* FF0E */
  {0xFF10,0xFF10,0xFF10},  {0xFF11,0xFF11,0xFF11}, /* FF10 */
  {0xFF12,0xFF12,0xFF12},  {0xFF13,0xFF13,0xFF13}, /* FF12 */
  {0xFF14,0xFF14,0xFF14},  {0xFF15,0xFF15,0xFF15}, /* FF14 */
  {0xFF16,0xFF16,0xFF16},  {0xFF17,0xFF17,0xFF17}, /* FF16 */
  {0xFF18,0xFF18,0xFF18},  {0xFF19,0xFF19,0xFF19}, /* FF18 */
  {0xFF1A,0xFF1A,0xFF1A},  {0xFF1B,0xFF1B,0xFF1B}, /* FF1A */
  {0xFF1C,0xFF1C,0xFF1C},  {0xFF1D,0xFF1D,0xFF1D}, /* FF1C */
  {0xFF1E,0xFF1E,0xFF1E},  {0xFF1F,0xFF1F,0xFF1F}, /* FF1E */
  {0xFF20,0xFF20,0xFF20},  {0xFF21,0xFF41,0xFF21}, /* FF20 */
  {0xFF22,0xFF42,0xFF22},  {0xFF23,0xFF43,0xFF23}, /* FF22 */
  {0xFF24,0xFF44,0xFF24},  {0xFF25,0xFF45,0xFF25}, /* FF24 */
  {0xFF26,0xFF46,0xFF26},  {0xFF27,0xFF47,0xFF27}, /* FF26 */
  {0xFF28,0xFF48,0xFF28},  {0xFF29,0xFF49,0xFF29}, /* FF28 */
  {0xFF2A,0xFF4A,0xFF2A},  {0xFF2B,0xFF4B,0xFF2B}, /* FF2A */
  {0xFF2C,0xFF4C,0xFF2C},  {0xFF2D,0xFF4D,0xFF2D}, /* FF2C */
  {0xFF2E,0xFF4E,0xFF2E},  {0xFF2F,0xFF4F,0xFF2F}, /* FF2E */
  {0xFF30,0xFF50,0xFF30},  {0xFF31,0xFF51,0xFF31}, /* FF30 */
  {0xFF32,0xFF52,0xFF32},  {0xFF33,0xFF53,0xFF33}, /* FF32 */
  {0xFF34,0xFF54,0xFF34},  {0xFF35,0xFF55,0xFF35}, /* FF34 */
  {0xFF36,0xFF56,0xFF36},  {0xFF37,0xFF57,0xFF37}, /* FF36 */
  {0xFF38,0xFF58,0xFF38},  {0xFF39,0xFF59,0xFF39}, /* FF38 */
  {0xFF3A,0xFF5A,0xFF3A},  {0xFF3B,0xFF3B,0xFF3B}, /* FF3A */
  {0xFF3C,0xFF3C,0xFF3C},  {0xFF3D,0xFF3D,0xFF3D}, /* FF3C */
  {0xFF3E,0xFF3E,0xFF3E},  {0xFF3F,0xFF3F,0xFF3F}, /* FF3E */
  {0xFF40,0xFF40,0xFF40},  {0xFF21,0xFF41,0xFF21}, /* FF40 */
  {0xFF22,0xFF42,0xFF22},  {0xFF23,0xFF43,0xFF23}, /* FF42 */
  {0xFF24,0xFF44,0xFF24},  {0xFF25,0xFF45,0xFF25}, /* FF44 */
  {0xFF26,0xFF46,0xFF26},  {0xFF27,0xFF47,0xFF27}, /* FF46 */
  {0xFF28,0xFF48,0xFF28},  {0xFF29,0xFF49,0xFF29}, /* FF48 */
  {0xFF2A,0xFF4A,0xFF2A},  {0xFF2B,0xFF4B,0xFF2B}, /* FF4A */
  {0xFF2C,0xFF4C,0xFF2C},  {0xFF2D,0xFF4D,0xFF2D}, /* FF4C */
  {0xFF2E,0xFF4E,0xFF2E},  {0xFF2F,0xFF4F,0xFF2F}, /* FF4E */
  {0xFF30,0xFF50,0xFF30},  {0xFF31,0xFF51,0xFF31}, /* FF50 */
  {0xFF32,0xFF52,0xFF32},  {0xFF33,0xFF53,0xFF33}, /* FF52 */
  {0xFF34,0xFF54,0xFF34},  {0xFF35,0xFF55,0xFF35}, /* FF54 */
  {0xFF36,0xFF56,0xFF36},  {0xFF37,0xFF57,0xFF37}, /* FF56 */
  {0xFF38,0xFF58,0xFF38},  {0xFF39,0xFF59,0xFF39}, /* FF58 */
  {0xFF3A,0xFF5A,0xFF3A},  {0xFF5B,0xFF5B,0xFF5B}, /* FF5A */
  {0xFF5C,0xFF5C,0xFF5C},  {0xFF5D,0xFF5D,0xFF5D}, /* FF5C */
  {0xFF5E,0xFF5E,0xFF5E},  {0xFF5F,0xFF5F,0xFF5F}, /* FF5E */
  {0xFF60,0xFF60,0xFF60},  {0xFF61,0xFF61,0xFF61}, /* FF60 */
  {0xFF62,0xFF62,0xFF62},  {0xFF63,0xFF63,0xFF63}, /* FF62 */
  {0xFF64,0xFF64,0xFF64},  {0xFF65,0xFF65,0xFF65}, /* FF64 */
  {0xFF66,0xFF66,0xFF66},  {0xFF67,0xFF67,0xFF67}, /* FF66 */
  {0xFF68,0xFF68,0xFF68},  {0xFF69,0xFF69,0xFF69}, /* FF68 */
  {0xFF6A,0xFF6A,0xFF6A},  {0xFF6B,0xFF6B,0xFF6B}, /* FF6A */
  {0xFF6C,0xFF6C,0xFF6C},  {0xFF6D,0xFF6D,0xFF6D}, /* FF6C */
  {0xFF6E,0xFF6E,0xFF6E},  {0xFF6F,0xFF6F,0xFF6F}, /* FF6E */
  {0xFF70,0xFF70,0xFF70},  {0xFF71,0xFF71,0xFF71}, /* FF70 */
  {0xFF72,0xFF72,0xFF72},  {0xFF73,0xFF73,0xFF73}, /* FF72 */
  {0xFF74,0xFF74,0xFF74},  {0xFF75,0xFF75,0xFF75}, /* FF74 */
  {0xFF76,0xFF76,0xFF76},  {0xFF77,0xFF77,0xFF77}, /* FF76 */
  {0xFF78,0xFF78,0xFF78},  {0xFF79,0xFF79,0xFF79}, /* FF78 */
  {0xFF7A,0xFF7A,0xFF7A},  {0xFF7B,0xFF7B,0xFF7B}, /* FF7A */
  {0xFF7C,0xFF7C,0xFF7C},  {0xFF7D,0xFF7D,0xFF7D}, /* FF7C */
  {0xFF7E,0xFF7E,0xFF7E},  {0xFF7F,0xFF7F,0xFF7F}, /* FF7E */
  {0xFF80,0xFF80,0xFF80},  {0xFF81,0xFF81,0xFF81}, /* FF80 */
  {0xFF82,0xFF82,0xFF82},  {0xFF83,0xFF83,0xFF83}, /* FF82 */
  {0xFF84,0xFF84,0xFF84},  {0xFF85,0xFF85,0xFF85}, /* FF84 */
  {0xFF86,0xFF86,0xFF86},  {0xFF87,0xFF87,0xFF87}, /* FF86 */
  {0xFF88,0xFF88,0xFF88},  {0xFF89,0xFF89,0xFF89}, /* FF88 */
  {0xFF8A,0xFF8A,0xFF8A},  {0xFF8B,0xFF8B,0xFF8B}, /* FF8A */
  {0xFF8C,0xFF8C,0xFF8C},  {0xFF8D,0xFF8D,0xFF8D}, /* FF8C */
  {0xFF8E,0xFF8E,0xFF8E},  {0xFF8F,0xFF8F,0xFF8F}, /* FF8E */
  {0xFF90,0xFF90,0xFF90},  {0xFF91,0xFF91,0xFF91}, /* FF90 */
  {0xFF92,0xFF92,0xFF92},  {0xFF93,0xFF93,0xFF93}, /* FF92 */
  {0xFF94,0xFF94,0xFF94},  {0xFF95,0xFF95,0xFF95}, /* FF94 */
  {0xFF96,0xFF96,0xFF96},  {0xFF97,0xFF97,0xFF97}, /* FF96 */
  {0xFF98,0xFF98,0xFF98},  {0xFF99,0xFF99,0xFF99}, /* FF98 */
  {0xFF9A,0xFF9A,0xFF9A},  {0xFF9B,0xFF9B,0xFF9B}, /* FF9A */
  {0xFF9C,0xFF9C,0xFF9C},  {0xFF9D,0xFF9D,0xFF9D}, /* FF9C */
  {0xFF9E,0xFF9E,0xFF9E},  {0xFF9F,0xFF9F,0xFF9F}, /* FF9E */
  {0xFFA0,0xFFA0,0xFFA0},  {0xFFA1,0xFFA1,0xFFA1}, /* FFA0 */
  {0xFFA2,0xFFA2,0xFFA2},  {0xFFA3,0xFFA3,0xFFA3}, /* FFA2 */
  {0xFFA4,0xFFA4,0xFFA4},  {0xFFA5,0xFFA5,0xFFA5}, /* FFA4 */
  {0xFFA6,0xFFA6,0xFFA6},  {0xFFA7,0xFFA7,0xFFA7}, /* FFA6 */
  {0xFFA8,0xFFA8,0xFFA8},  {0xFFA9,0xFFA9,0xFFA9}, /* FFA8 */
  {0xFFAA,0xFFAA,0xFFAA},  {0xFFAB,0xFFAB,0xFFAB}, /* FFAA */
  {0xFFAC,0xFFAC,0xFFAC},  {0xFFAD,0xFFAD,0xFFAD}, /* FFAC */
  {0xFFAE,0xFFAE,0xFFAE},  {0xFFAF,0xFFAF,0xFFAF}, /* FFAE */
  {0xFFB0,0xFFB0,0xFFB0},  {0xFFB1,0xFFB1,0xFFB1}, /* FFB0 */
  {0xFFB2,0xFFB2,0xFFB2},  {0xFFB3,0xFFB3,0xFFB3}, /* FFB2 */
  {0xFFB4,0xFFB4,0xFFB4},  {0xFFB5,0xFFB5,0xFFB5}, /* FFB4 */
  {0xFFB6,0xFFB6,0xFFB6},  {0xFFB7,0xFFB7,0xFFB7}, /* FFB6 */
  {0xFFB8,0xFFB8,0xFFB8},  {0xFFB9,0xFFB9,0xFFB9}, /* FFB8 */
  {0xFFBA,0xFFBA,0xFFBA},  {0xFFBB,0xFFBB,0xFFBB}, /* FFBA */
  {0xFFBC,0xFFBC,0xFFBC},  {0xFFBD,0xFFBD,0xFFBD}, /* FFBC */
  {0xFFBE,0xFFBE,0xFFBE},  {0xFFBF,0xFFBF,0xFFBF}, /* FFBE */
  {0xFFC0,0xFFC0,0xFFC0},  {0xFFC1,0xFFC1,0xFFC1}, /* FFC0 */
  {0xFFC2,0xFFC2,0xFFC2},  {0xFFC3,0xFFC3,0xFFC3}, /* FFC2 */
  {0xFFC4,0xFFC4,0xFFC4},  {0xFFC5,0xFFC5,0xFFC5}, /* FFC4 */
  {0xFFC6,0xFFC6,0xFFC6},  {0xFFC7,0xFFC7,0xFFC7}, /* FFC6 */
  {0xFFC8,0xFFC8,0xFFC8},  {0xFFC9,0xFFC9,0xFFC9}, /* FFC8 */
  {0xFFCA,0xFFCA,0xFFCA},  {0xFFCB,0xFFCB,0xFFCB}, /* FFCA */
  {0xFFCC,0xFFCC,0xFFCC},  {0xFFCD,0xFFCD,0xFFCD}, /* FFCC */
  {0xFFCE,0xFFCE,0xFFCE},  {0xFFCF,0xFFCF,0xFFCF}, /* FFCE */
  {0xFFD0,0xFFD0,0xFFD0},  {0xFFD1,0xFFD1,0xFFD1}, /* FFD0 */
  {0xFFD2,0xFFD2,0xFFD2},  {0xFFD3,0xFFD3,0xFFD3}, /* FFD2 */
  {0xFFD4,0xFFD4,0xFFD4},  {0xFFD5,0xFFD5,0xFFD5}, /* FFD4 */
  {0xFFD6,0xFFD6,0xFFD6},  {0xFFD7,0xFFD7,0xFFD7}, /* FFD6 */
  {0xFFD8,0xFFD8,0xFFD8},  {0xFFD9,0xFFD9,0xFFD9}, /* FFD8 */
  {0xFFDA,0xFFDA,0xFFDA},  {0xFFDB,0xFFDB,0xFFDB}, /* FFDA */
  {0xFFDC,0xFFDC,0xFFDC},  {0xFFDD,0xFFDD,0xFFDD}, /* FFDC */
  {0xFFDE,0xFFDE,0xFFDE},  {0xFFDF,0xFFDF,0xFFDF}, /* FFDE */
  {0xFFE0,0xFFE0,0xFFE0},  {0xFFE1,0xFFE1,0xFFE1}, /* FFE0 */
  {0xFFE2,0xFFE2,0xFFE2},  {0xFFE3,0xFFE3,0xFFE3}, /* FFE2 */
  {0xFFE4,0xFFE4,0xFFE4},  {0xFFE5,0xFFE5,0xFFE5}, /* FFE4 */
  {0xFFE6,0xFFE6,0xFFE6},  {0xFFE7,0xFFE7,0xFFE7}, /* FFE6 */
  {0xFFE8,0xFFE8,0xFFE8},  {0xFFE9,0xFFE9,0xFFE9}, /* FFE8 */
  {0xFFEA,0xFFEA,0xFFEA},  {0xFFEB,0xFFEB,0xFFEB}, /* FFEA */
  {0xFFEC,0xFFEC,0xFFEC},  {0xFFED,0xFFED,0xFFED}, /* FFEC */
  {0xFFEE,0xFFEE,0xFFEE},  {0xFFEF,0xFFEF,0xFFEF}, /* FFEE */
  {0xFFF0,0xFFF0,0xFFF0},  {0xFFF1,0xFFF1,0xFFF1}, /* FFF0 */
  {0xFFF2,0xFFF2,0xFFF2},  {0xFFF3,0xFFF3,0xFFF3}, /* FFF2 */
  {0xFFF4,0xFFF4,0xFFF4},  {0xFFF5,0xFFF5,0xFFF5}, /* FFF4 */
  {0xFFF6,0xFFF6,0xFFF6},  {0xFFF7,0xFFF7,0xFFF7}, /* FFF6 */
  {0xFFF8,0xFFF8,0xFFF8},  {0xFFF9,0xFFF9,0xFFF9}, /* FFF8 */
  {0xFFFA,0xFFFA,0xFFFA},  {0xFFFB,0xFFFB,0xFFFB}, /* FFFA */
  {0xFFFC,0xFFFC,0xFFFC},  {0xFFFD,0xFFFD,0xFFFD}, /* FFFC */
  {0xFFFE,0xFFFE,0xFFFE},  {0xFFFF,0xFFFF,0xFFFF}  /* FFFE */
};

static const MY_UNICASE_CHARACTER u900p104[]= {
  {0x10400,0x10428,0x10400},  {0x10401,0x10429,0x10401}, /* 10400 */
  {0x10402,0x1042A,0x10402},  {0x10403,0x1042B,0x10403}, /* 10402 */
  {0x10404,0x1042C,0x10404},  {0x10405,0x1042D,0x10405}, /* 10404 */
  {0x10406,0x1042E,0x10406},  {0x10407,0x1042F,0x10407}, /* 10406 */
  {0x10408,0x10430,0x10408},  {0x10409,0x10431,0x10409}, /* 10408 */
  {0x1040A,0x10432,0x1040A},  {0x1040B,0x10433,0x1040B}, /* 1040A */
  {0x1040C,0x10434,0x1040C},  {0x1040D,0x10435,0x1040D}, /* 1040C */
  {0x1040E,0x10436,0x1040E},  {0x1040F,0x10437,0x1040F}, /* 1040E */
  {0x10410,0x10438,0x10410},  {0x10411,0x10439,0x10411}, /* 10410 */
  {0x10412,0x1043A,0x10412},  {0x10413,0x1043B,0x10413}, /* 10412 */
  {0x10414,0x1043C,0x10414},  {0x10415,0x1043D,0x10415}, /* 10414 */
  {0x10416,0x1043E,0x10416},  {0x10417,0x1043F,0x10417}, /* 10416 */
  {0x10418,0x10440,0x10418},  {0x10419,0x10441,0x10419}, /* 10418 */
  {0x1041A,0x10442,0x1041A},  {0x1041B,0x10443,0x1041B}, /* 1041A */
  {0x1041C,0x10444,0x1041C},  {0x1041D,0x10445,0x1041D}, /* 1041C */
  {0x1041E,0x10446,0x1041E},  {0x1041F,0x10447,0x1041F}, /* 1041E */
  {0x10420,0x10448,0x10420},  {0x10421,0x10449,0x10421}, /* 10420 */
  {0x10422,0x1044A,0x10422},  {0x10423,0x1044B,0x10423}, /* 10422 */
  {0x10424,0x1044C,0x10424},  {0x10425,0x1044D,0x10425}, /* 10424 */
  {0x10426,0x1044E,0x10426},  {0x10427,0x1044F,0x10427}, /* 10426 */
  {0x10400,0x10428,0x10400},  {0x10401,0x10429,0x10401}, /* 10428 */
  {0x10402,0x1042A,0x10402},  {0x10403,0x1042B,0x10403}, /* 1042A */
  {0x10404,0x1042C,0x10404},  {0x10405,0x1042D,0x10405}, /* 1042C */
  {0x10406,0x1042E,0x10406},  {0x10407,0x1042F,0x10407}, /* 1042E */
  {0x10408,0x10430,0x10408},  {0x10409,0x10431,0x10409}, /* 10430 */
  {0x1040A,0x10432,0x1040A},  {0x1040B,0x10433,0x1040B}, /* 10432 */
  {0x1040C,0x10434,0x1040C},  {0x1040D,0x10435,0x1040D}, /* 10434 */
  {0x1040E,0x10436,0x1040E},  {0x1040F,0x10437,0x1040F}, /* 10436 */
  {0x10410,0x10438,0x10410},  {0x10411,0x10439,0x10411}, /* 10438 */
  {0x10412,0x1043A,0x10412},  {0x10413,0x1043B,0x10413}, /* 1043A */
  {0x10414,0x1043C,0x10414},  {0x10415,0x1043D,0x10415}, /* 1043C */
  {0x10416,0x1043E,0x10416},  {0x10417,0x1043F,0x10417}, /* 1043E */
  {0x10418,0x10440,0x10418},  {0x10419,0x10441,0x10419}, /* 10440 */
  {0x1041A,0x10442,0x1041A},  {0x1041B,0x10443,0x1041B}, /* 10442 */
  {0x1041C,0x10444,0x1041C},  {0x1041D,0x10445,0x1041D}, /* 10444 */
  {0x1041E,0x10446,0x1041E},  {0x1041F,0x10447,0x1041F}, /* 10446 */
  {0x10420,0x10448,0x10420},  {0x10421,0x10449,0x10421}, /* 10448 */
  {0x10422,0x1044A,0x10422},  {0x10423,0x1044B,0x10423}, /* 1044A */
  {0x10424,0x1044C,0x10424},  {0x10425,0x1044D,0x10425}, /* 1044C */
  {0x10426,0x1044E,0x10426},  {0x10427,0x1044F,0x10427}, /* 1044E */
  {0x10450,0x10450,0x10450},  {0x10451,0x10451,0x10451}, /* 10450 */
  {0x10452,0x10452,0x10452},  {0x10453,0x10453,0x10453}, /* 10452 */
  {0x10454,0x10454,0x10454},  {0x10455,0x10455,0x10455}, /* 10454 */
  {0x10456,0x10456,0x10456},  {0x10457,0x10457,0x10457}, /* 10456 */
  {0x10458,0x10458,0x10458},  {0x10459,0x10459,0x10459}, /* 10458 */
  {0x1045A,0x1045A,0x1045A},  {0x1045B,0x1045B,0x1045B}, /* 1045A */
  {0x1045C,0x1045C,0x1045C},  {0x1045D,0x1045D,0x1045D}, /* 1045C */
  {0x1045E,0x1045E,0x1045E},  {0x1045F,0x1045F,0x1045F}, /* 1045E */
  {0x10460,0x10460,0x10460},  {0x10461,0x10461,0x10461}, /* 10460 */
  {0x10462,0x10462,0x10462},  {0x10463,0x10463,0x10463}, /* 10462 */
  {0x10464,0x10464,0x10464},  {0x10465,0x10465,0x10465}, /* 10464 */
  {0x10466,0x10466,0x10466},  {0x10467,0x10467,0x10467}, /* 10466 */
  {0x10468,0x10468,0x10468},  {0x10469,0x10469,0x10469}, /* 10468 */
  {0x1046A,0x1046A,0x1046A},  {0x1046B,0x1046B,0x1046B}, /* 1046A */
  {0x1046C,0x1046C,0x1046C},  {0x1046D,0x1046D,0x1046D}, /* 1046C */
  {0x1046E,0x1046E,0x1046E},  {0x1046F,0x1046F,0x1046F}, /* 1046E */
  {0x10470,0x10470,0x10470},  {0x10471,0x10471,0x10471}, /* 10470 */
  {0x10472,0x10472,0x10472},  {0x10473,0x10473,0x10473}, /* 10472 */
  {0x10474,0x10474,0x10474},  {0x10475,0x10475,0x10475}, /* 10474 */
  {0x10476,0x10476,0x10476},  {0x10477,0x10477,0x10477}, /* 10476 */
  {0x10478,0x10478,0x10478},  {0x10479,0x10479,0x10479}, /* 10478 */
  {0x1047A,0x1047A,0x1047A},  {0x1047B,0x1047B,0x1047B}, /* 1047A */
  {0x1047C,0x1047C,0x1047C},  {0x1047D,0x1047D,0x1047D}, /* 1047C */
  {0x1047E,0x1047E,0x1047E},  {0x1047F,0x1047F,0x1047F}, /* 1047E */
  {0x10480,0x10480,0x10480},  {0x10481,0x10481,0x10481}, /* 10480 */
  {0x10482,0x10482,0x10482},  {0x10483,0x10483,0x10483}, /* 10482 */
  {0x10484,0x10484,0x10484},  {0x10485,0x10485,0x10485}, /* 10484 */
  {0x10486,0x10486,0x10486},  {0x10487,0x10487,0x10487}, /* 10486 */
  {0x10488,0x10488,0x10488},  {0x10489,0x10489,0x10489}, /* 10488 */
  {0x1048A,0x1048A,0x1048A},  {0x1048B,0x1048B,0x1048B}, /* 1048A */
  {0x1048C,0x1048C,0x1048C},  {0x1048D,0x1048D,0x1048D}, /* 1048C */
  {0x1048E,0x1048E,0x1048E},  {0x1048F,0x1048F,0x1048F}, /* 1048E */
  {0x10490,0x10490,0x10490},  {0x10491,0x10491,0x10491}, /* 10490 */
  {0x10492,0x10492,0x10492},  {0x10493,0x10493,0x10493}, /* 10492 */
  {0x10494,0x10494,0x10494},  {0x10495,0x10495,0x10495}, /* 10494 */
  {0x10496,0x10496,0x10496},  {0x10497,0x10497,0x10497}, /* 10496 */
  {0x10498,0x10498,0x10498},  {0x10499,0x10499,0x10499}, /* 10498 */
  {0x1049A,0x1049A,0x1049A},  {0x1049B,0x1049B,0x1049B}, /* 1049A */
  {0x1049C,0x1049C,0x1049C},  {0x1049D,0x1049D,0x1049D}, /* 1049C */
  {0x1049E,0x1049E,0x1049E},  {0x1049F,0x1049F,0x1049F}, /* 1049E */
  {0x104A0,0x104A0,0x104A0},  {0x104A1,0x104A1,0x104A1}, /* 104A0 */
  {0x104A2,0x104A2,0x104A2},  {0x104A3,0x104A3,0x104A3}, /* 104A2 */
  {0x104A4,0x104A4,0x104A4},  {0x104A5,0x104A5,0x104A5}, /* 104A4 */
  {0x104A6,0x104A6,0x104A6},  {0x104A7,0x104A7,0x104A7}, /* 104A6 */
  {0x104A8,0x104A8,0x104A8},  {0x104A9,0x104A9,0x104A9}, /* 104A8 */
  {0x104AA,0x104AA,0x104AA},  {0x104AB,0x104AB,0x104AB}, /* 104AA */
  {0x104AC,0x104AC,0x104AC},  {0x104AD,0x104AD,0x104AD}, /* 104AC */
  {0x104AE,0x104AE,0x104AE},  {0x104AF,0x104AF,0x104AF}, /* 104AE */
  {0x104B0,0x104D8,0x104B0},  {0x104B1,0x104D9,0x104B1}, /* 104B0 */
  {0x104B2,0x104DA,0x104B2},  {0x104B3,0x104DB,0x104B3}, /* 104B2 */
  {0x104B4,0x104DC,0x104B4},  {0x104B5,0x104DD,0x104B5}, /* 104B4 */
  {0x104B6,0x104DE,0x104B6},  {0x104B7,0x104DF,0x104B7}, /* 104B6 */
  {0x104B8,0x104E0,0x104B8},  {0x104B9,0x104E1,0x104B9}, /* 104B8 */
  {0x104BA,0x104E2,0x104BA},  {0x104BB,0x104E3,0x104BB}, /* 104BA */
  {0x104BC,0x104E4,0x104BC},  {0x104BD,0x104E5,0x104BD}, /* 104BC */
  {0x104BE,0x104E6,0x104BE},  {0x104BF,0x104E7,0x104BF}, /* 104BE */
  {0x104C0,0x104E8,0x104C0},  {0x104C1,0x104E9,0x104C1}, /* 104C0 */
  {0x104C2,0x104EA,0x104C2},  {0x104C3,0x104EB,0x104C3}, /* 104C2 */
  {0x104C4,0x104EC,0x104C4},  {0x104C5,0x104ED,0x104C5}, /* 104C4 */
  {0x104C6,0x104EE,0x104C6},  {0x104C7,0x104EF,0x104C7}, /* 104C6 */
  {0x104C8,0x104F0,0x104C8},  {0x104C9,0x104F1,0x104C9}, /* 104C8 */
  {0x104CA,0x104F2,0x104CA},  {0x104CB,0x104F3,0x104CB}, /* 104CA */
  {0x104CC,0x104F4,0x104CC},  {0x104CD,0x104F5,0x104CD}, /* 104CC */
  {0x104CE,0x104F6,0x104CE},  {0x104CF,0x104F7,0x104CF}, /* 104CE */
  {0x104D0,0x104F8,0x104D0},  {0x104D1,0x104F9,0x104D1}, /* 104D0 */
  {0x104D2,0x104FA,0x104D2},  {0x104D3,0x104FB,0x104D3}, /* 104D2 */
  {0x104D4,0x104D4,0x104D4},  {0x104D5,0x104D5,0x104D5}, /* 104D4 */
  {0x104D6,0x104D6,0x104D6},  {0x104D7,0x104D7,0x104D7}, /* 104D6 */
  {0x104B0,0x104D8,0x104B0},  {0x104B1,0x104D9,0x104B1}, /* 104D8 */
  {0x104B2,0x104DA,0x104B2},  {0x104B3,0x104DB,0x104B3}, /* 104DA */
  {0x104B4,0x104DC,0x104B4},  {0x104B5,0x104DD,0x104B5}, /* 104DC */
  {0x104B6,0x104DE,0x104B6},  {0x104B7,0x104DF,0x104B7}, /* 104DE */
  {0x104B8,0x104E0,0x104B8},  {0x104B9,0x104E1,0x104B9}, /* 104E0 */
  {0x104BA,0x104E2,0x104BA},  {0x104BB,0x104E3,0x104BB}, /* 104E2 */
  {0x104BC,0x104E4,0x104BC},  {0x104BD,0x104E5,0x104BD}, /* 104E4 */
  {0x104BE,0x104E6,0x104BE},  {0x104BF,0x104E7,0x104BF}, /* 104E6 */
  {0x104C0,0x104E8,0x104C0},  {0x104C1,0x104E9,0x104C1}, /* 104E8 */
  {0x104C2,0x104EA,0x104C2},  {0x104C3,0x104EB,0x104C3}, /* 104EA */
  {0x104C4,0x104EC,0x104C4},  {0x104C5,0x104ED,0x104C5}, /* 104EC */
  {0x104C6,0x104EE,0x104C6},  {0x104C7,0x104EF,0x104C7}, /* 104EE */
  {0x104C8,0x104F0,0x104C8},  {0x104C9,0x104F1,0x104C9}, /* 104F0 */
  {0x104CA,0x104F2,0x104CA},  {0x104CB,0x104F3,0x104CB}, /* 104F2 */
  {0x104CC,0x104F4,0x104CC},  {0x104CD,0x104F5,0x104CD}, /* 104F4 */
  {0x104CE,0x104F6,0x104CE},  {0x104CF,0x104F7,0x104CF}, /* 104F6 */
  {0x104D0,0x104F8,0x104D0},  {0x104D1,0x104F9,0x104D1}, /* 104F8 */
  {0x104D2,0x104FA,0x104D2},  {0x104D3,0x104FB,0x104D3}, /* 104FA */
  {0x104FC,0x104FC,0x104FC},  {0x104FD,0x104FD,0x104FD}, /* 104FC */
  {0x104FE,0x104FE,0x104FE},  {0x104FF,0x104FF,0x104FF}  /* 104FE */
};

static const MY_UNICASE_CHARACTER u900p10C[]= {
  {0x10C00,0x10C00,0x10C00},  {0x10C01,0x10C01,0x10C01}, /* 10C00 */
  {0x10C02,0x10C02,0x10C02},  {0x10C03,0x10C03,0x10C03}, /* 10C02 */
  {0x10C04,0x10C04,0x10C04},  {0x10C05,0x10C05,0x10C05}, /* 10C04 */
  {0x10C06,0x10C06,0x10C06},  {0x10C07,0x10C07,0x10C07}, /* 10C06 */
  {0x10C08,0x10C08,0x10C08},  {0x10C09,0x10C09,0x10C09}, /* 10C08 */
  {0x10C0A,0x10C0A,0x10C0A},  {0x10C0B,0x10C0B,0x10C0B}, /* 10C0A */
  {0x10C0C,0x10C0C,0x10C0C},  {0x10C0D,0x10C0D,0x10C0D}, /* 10C0C */
  {0x10C0E,0x10C0E,0x10C0E},  {0x10C0F,0x10C0F,0x10C0F}, /* 10C0E */
  {0x10C10,0x10C10,0x10C10},  {0x10C11,0x10C11,0x10C11}, /* 10C10 */
  {0x10C12,0x10C12,0x10C12},  {0x10C13,0x10C13,0x10C13}, /* 10C12 */
  {0x10C14,0x10C14,0x10C14},  {0x10C15,0x10C15,0x10C15}, /* 10C14 */
  {0x10C16,0x10C16,0x10C16},  {0x10C17,0x10C17,0x10C17}, /* 10C16 */
  {0x10C18,0x10C18,0x10C18},  {0x10C19,0x10C19,0x10C19}, /* 10C18 */
  {0x10C1A,0x10C1A,0x10C1A},  {0x10C1B,0x10C1B,0x10C1B}, /* 10C1A */
  {0x10C1C,0x10C1C,0x10C1C},  {0x10C1D,0x10C1D,0x10C1D}, /* 10C1C */
  {0x10C1E,0x10C1E,0x10C1E},  {0x10C1F,0x10C1F,0x10C1F}, /* 10C1E */
  {0x10C20,0x10C20,0x10C20},  {0x10C21,0x10C21,0x10C21}, /* 10C20 */
  {0x10C22,0x10C22,0x10C22},  {0x10C23,0x10C23,0x10C23}, /* 10C22 */
  {0x10C24,0x10C24,0x10C24},  {0x10C25,0x10C25,0x10C25}, /* 10C24 */
  {0x10C26,0x10C26,0x10C26},  {0x10C27,0x10C27,0x10C27}, /* 10C26 */
  {0x10C28,0x10C28,0x10C28},  {0x10C29,0x10C29,0x10C29}, /* 10C28 */
  {0x10C2A,0x10C2A,0x10C2A},  {0x10C2B,0x10C2B,0x10C2B}, /* 10C2A */
  {0x10C2C,0x10C2C,0x10C2C},  {0x10C2D,0x10C2D,0x10C2D}, /* 10C2C */
  {0x10C2E,0x10C2E,0x10C2E},  {0x10C2F,0x10C2F,0x10C2F}, /* 10C2E */
  {0x10C30,0x10C30,0x10C30},  {0x10C31,0x10C31,0x10C31}, /* 10C30 */
  {0x10C32,0x10C32,0x10C32},  {0x10C33,0x10C33,0x10C33}, /* 10C32 */
  {0x10C34,0x10C34,0x10C34},  {0x10C35,0x10C35,0x10C35}, /* 10C34 */
  {0x10C36,0x10C36,0x10C36},  {0x10C37,0x10C37,0x10C37}, /* 10C36 */
  {0x10C38,0x10C38,0x10C38},  {0x10C39,0x10C39,0x10C39}, /* 10C38 */
  {0x10C3A,0x10C3A,0x10C3A},  {0x10C3B,0x10C3B,0x10C3B}, /* 10C3A */
  {0x10C3C,0x10C3C,0x10C3C},  {0x10C3D,0x10C3D,0x10C3D}, /* 10C3C */
  {0x10C3E,0x10C3E,0x10C3E},  {0x10C3F,0x10C3F,0x10C3F}, /* 10C3E */
  {0x10C40,0x10C40,0x10C40},  {0x10C41,0x10C41,0x10C41}, /* 10C40 */
  {0x10C42,0x10C42,0x10C42},  {0x10C43,0x10C43,0x10C43}, /* 10C42 */
  {0x10C44,0x10C44,0x10C44},  {0x10C45,0x10C45,0x10C45}, /* 10C44 */
  {0x10C46,0x10C46,0x10C46},  {0x10C47,0x10C47,0x10C47}, /* 10C46 */
  {0x10C48,0x10C48,0x10C48},  {0x10C49,0x10C49,0x10C49}, /* 10C48 */
  {0x10C4A,0x10C4A,0x10C4A},  {0x10C4B,0x10C4B,0x10C4B}, /* 10C4A */
  {0x10C4C,0x10C4C,0x10C4C},  {0x10C4D,0x10C4D,0x10C4D}, /* 10C4C */
  {0x10C4E,0x10C4E,0x10C4E},  {0x10C4F,0x10C4F,0x10C4F}, /* 10C4E */
  {0x10C50,0x10C50,0x10C50},  {0x10C51,0x10C51,0x10C51}, /* 10C50 */
  {0x10C52,0x10C52,0x10C52},  {0x10C53,0x10C53,0x10C53}, /* 10C52 */
  {0x10C54,0x10C54,0x10C54},  {0x10C55,0x10C55,0x10C55}, /* 10C54 */
  {0x10C56,0x10C56,0x10C56},  {0x10C57,0x10C57,0x10C57}, /* 10C56 */
  {0x10C58,0x10C58,0x10C58},  {0x10C59,0x10C59,0x10C59}, /* 10C58 */
  {0x10C5A,0x10C5A,0x10C5A},  {0x10C5B,0x10C5B,0x10C5B}, /* 10C5A */
  {0x10C5C,0x10C5C,0x10C5C},  {0x10C5D,0x10C5D,0x10C5D}, /* 10C5C */
  {0x10C5E,0x10C5E,0x10C5E},  {0x10C5F,0x10C5F,0x10C5F}, /* 10C5E */
  {0x10C60,0x10C60,0x10C60},  {0x10C61,0x10C61,0x10C61}, /* 10C60 */
  {0x10C62,0x10C62,0x10C62},  {0x10C63,0x10C63,0x10C63}, /* 10C62 */
  {0x10C64,0x10C64,0x10C64},  {0x10C65,0x10C65,0x10C65}, /* 10C64 */
  {0x10C66,0x10C66,0x10C66},  {0x10C67,0x10C67,0x10C67}, /* 10C66 */
  {0x10C68,0x10C68,0x10C68},  {0x10C69,0x10C69,0x10C69}, /* 10C68 */
  {0x10C6A,0x10C6A,0x10C6A},  {0x10C6B,0x10C6B,0x10C6B}, /* 10C6A */
  {0x10C6C,0x10C6C,0x10C6C},  {0x10C6D,0x10C6D,0x10C6D}, /* 10C6C */
  {0x10C6E,0x10C6E,0x10C6E},  {0x10C6F,0x10C6F,0x10C6F}, /* 10C6E */
  {0x10C70,0x10C70,0x10C70},  {0x10C71,0x10C71,0x10C71}, /* 10C70 */
  {0x10C72,0x10C72,0x10C72},  {0x10C73,0x10C73,0x10C73}, /* 10C72 */
  {0x10C74,0x10C74,0x10C74},  {0x10C75,0x10C75,0x10C75}, /* 10C74 */
  {0x10C76,0x10C76,0x10C76},  {0x10C77,0x10C77,0x10C77}, /* 10C76 */
  {0x10C78,0x10C78,0x10C78},  {0x10C79,0x10C79,0x10C79}, /* 10C78 */
  {0x10C7A,0x10C7A,0x10C7A},  {0x10C7B,0x10C7B,0x10C7B}, /* 10C7A */
  {0x10C7C,0x10C7C,0x10C7C},  {0x10C7D,0x10C7D,0x10C7D}, /* 10C7C */
  {0x10C7E,0x10C7E,0x10C7E},  {0x10C7F,0x10C7F,0x10C7F}, /* 10C7E */
  {0x10C80,0x10CC0,0x10C80},  {0x10C81,0x10CC1,0x10C81}, /* 10C80 */
  {0x10C82,0x10CC2,0x10C82},  {0x10C83,0x10CC3,0x10C83}, /* 10C82 */
  {0x10C84,0x10CC4,0x10C84},  {0x10C85,0x10CC5,0x10C85}, /* 10C84 */
  {0x10C86,0x10CC6,0x10C86},  {0x10C87,0x10CC7,0x10C87}, /* 10C86 */
  {0x10C88,0x10CC8,0x10C88},  {0x10C89,0x10CC9,0x10C89}, /* 10C88 */
  {0x10C8A,0x10CCA,0x10C8A},  {0x10C8B,0x10CCB,0x10C8B}, /* 10C8A */
  {0x10C8C,0x10CCC,0x10C8C},  {0x10C8D,0x10CCD,0x10C8D}, /* 10C8C */
  {0x10C8E,0x10CCE,0x10C8E},  {0x10C8F,0x10CCF,0x10C8F}, /* 10C8E */
  {0x10C90,0x10CD0,0x10C90},  {0x10C91,0x10CD1,0x10C91}, /* 10C90 */
  {0x10C92,0x10CD2,0x10C92},  {0x10C93,0x10CD3,0x10C93}, /* 10C92 */
  {0x10C94,0x10CD4,0x10C94},  {0x10C95,0x10CD5,0x10C95}, /* 10C94 */
  {0x10C96,0x10CD6,0x10C96},  {0x10C97,0x10CD7,0x10C97}, /* 10C96 */
  {0x10C98,0x10CD8,0x10C98},  {0x10C99,0x10CD9,0x10C99}, /* 10C98 */
  {0x10C9A,0x10CDA,0x10C9A},  {0x10C9B,0x10CDB,0x10C9B}, /* 10C9A */
  {0x10C9C,0x10CDC,0x10C9C},  {0x10C9D,0x10CDD,0x10C9D}, /* 10C9C */
  {0x10C9E,0x10CDE,0x10C9E},  {0x10C9F,0x10CDF,0x10C9F}, /* 10C9E */
  {0x10CA0,0x10CE0,0x10CA0},  {0x10CA1,0x10CE1,0x10CA1}, /* 10CA0 */
  {0x10CA2,0x10CE2,0x10CA2},  {0x10CA3,0x10CE3,0x10CA3}, /* 10CA2 */
  {0x10CA4,0x10CE4,0x10CA4},  {0x10CA5,0x10CE5,0x10CA5}, /* 10CA4 */
  {0x10CA6,0x10CE6,0x10CA6},  {0x10CA7,0x10CE7,0x10CA7}, /* 10CA6 */
  {0x10CA8,0x10CE8,0x10CA8},  {0x10CA9,0x10CE9,0x10CA9}, /* 10CA8 */
  {0x10CAA,0x10CEA,0x10CAA},  {0x10CAB,0x10CEB,0x10CAB}, /* 10CAA */
  {0x10CAC,0x10CEC,0x10CAC},  {0x10CAD,0x10CED,0x10CAD}, /* 10CAC */
  {0x10CAE,0x10CEE,0x10CAE},  {0x10CAF,0x10CEF,0x10CAF}, /* 10CAE */
  {0x10CB0,0x10CF0,0x10CB0},  {0x10CB1,0x10CF1,0x10CB1}, /* 10CB0 */
  {0x10CB2,0x10CF2,0x10CB2},  {0x10CB3,0x10CB3,0x10CB3}, /* 10CB2 */
  {0x10CB4,0x10CB4,0x10CB4},  {0x10CB5,0x10CB5,0x10CB5}, /* 10CB4 */
  {0x10CB6,0x10CB6,0x10CB6},  {0x10CB7,0x10CB7,0x10CB7}, /* 10CB6 */
  {0x10CB8,0x10CB8,0x10CB8},  {0x10CB9,0x10CB9,0x10CB9}, /* 10CB8 */
  {0x10CBA,0x10CBA,0x10CBA},  {0x10CBB,0x10CBB,0x10CBB}, /* 10CBA */
  {0x10CBC,0x10CBC,0x10CBC},  {0x10CBD,0x10CBD,0x10CBD}, /* 10CBC */
  {0x10CBE,0x10CBE,0x10CBE},  {0x10CBF,0x10CBF,0x10CBF}, /* 10CBE */
  {0x10C80,0x10CC0,0x10C80},  {0x10C81,0x10CC1,0x10C81}, /* 10CC0 */
  {0x10C82,0x10CC2,0x10C82},  {0x10C83,0x10CC3,0x10C83}, /* 10CC2 */
  {0x10C84,0x10CC4,0x10C84},  {0x10C85,0x10CC5,0x10C85}, /* 10CC4 */
  {0x10C86,0x10CC6,0x10C86},  {0x10C87,0x10CC7,0x10C87}, /* 10CC6 */
  {0x10C88,0x10CC8,0x10C88},  {0x10C89,0x10CC9,0x10C89}, /* 10CC8 */
  {0x10C8A,0x10CCA,0x10C8A},  {0x10C8B,0x10CCB,0x10C8B}, /* 10CCA */
  {0x10C8C,0x10CCC,0x10C8C},  {0x10C8D,0x10CCD,0x10C8D}, /* 10CCC */
  {0x10C8E,0x10CCE,0x10C8E},  {0x10C8F,0x10CCF,0x10C8F}, /* 10CCE */
  {0x10C90,0x10CD0,0x10C90},  {0x10C91,0x10CD1,0x10C91}, /* 10CD0 */
  {0x10C92,0x10CD2,0x10C92},  {0x10C93,0x10CD3,0x10C93}, /* 10CD2 */
  {0x10C94,0x10CD4,0x10C94},  {0x10C95,0x10CD5,0x10C95}, /* 10CD4 */
  {0x10C96,0x10CD6,0x10C96},  {0x10C97,0x10CD7,0x10C97}, /* 10CD6 */
  {0x10C98,0x10CD8,0x10C98},  {0x10C99,0x10CD9,0x10C99}, /* 10CD8 */
  {0x10C9A,0x10CDA,0x10C9A},  {0x10C9B,0x10CDB,0x10C9B}, /* 10CDA */
  {0x10C9C,0x10CDC,0x10C9C},  {0x10C9D,0x10CDD,0x10C9D}, /* 10CDC */
  {0x10C9E,0x10CDE,0x10C9E},  {0x10C9F,0x10CDF,0x10C9F}, /* 10CDE */
  {0x10CA0,0x10CE0,0x10CA0},  {0x10CA1,0x10CE1,0x10CA1}, /* 10CE0 */
  {0x10CA2,0x10CE2,0x10CA2},  {0x10CA3,0x10CE3,0x10CA3}, /* 10CE2 */
  {0x10CA4,0x10CE4,0x10CA4},  {0x10CA5,0x10CE5,0x10CA5}, /* 10CE4 */
  {0x10CA6,0x10CE6,0x10CA6},  {0x10CA7,0x10CE7,0x10CA7}, /* 10CE6 */
  {0x10CA8,0x10CE8,0x10CA8},  {0x10CA9,0x10CE9,0x10CA9}, /* 10CE8 */
  {0x10CAA,0x10CEA,0x10CAA},  {0x10CAB,0x10CEB,0x10CAB}, /* 10CEA */
  {0x10CAC,0x10CEC,0x10CAC},  {0x10CAD,0x10CED,0x10CAD}, /* 10CEC */
  {0x10CAE,0x10CEE,0x10CAE},  {0x10CAF,0x10CEF,0x10CAF}, /* 10CEE */
  {0x10CB0,0x10CF0,0x10CB0},  {0x10CB1,0x10CF1,0x10CB1}, /* 10CF0 */
  {0x10CB2,0x10CF2,0x10CB2},  {0x10CF3,0x10CF3,0x10CF3}, /* 10CF2 */
  {0x10CF4,0x10CF4,0x10CF4},  {0x10CF5,0x10CF5,0x10CF5}, /* 10CF4 */
  {0x10CF6,0x10CF6,0x10CF6},  {0x10CF7,0x10CF7,0x10CF7}, /* 10CF6 */
  {0x10CF8,0x10CF8,0x10CF8},  {0x10CF9,0x10CF9,0x10CF9}, /* 10CF8 */
  {0x10CFA,0x10CFA,0x10CFA},  {0x10CFB,0x10CFB,0x10CFB}, /* 10CFA */
  {0x10CFC,0x10CFC,0x10CFC},  {0x10CFD,0x10CFD,0x10CFD}, /* 10CFC */
  {0x10CFE,0x10CFE,0x10CFE},  {0x10CFF,0x10CFF,0x10CFF}  /* 10CFE */
};

static const MY_UNICASE_CHARACTER u900p118[]= {
  {0x11800,0x11800,0x11800},  {0x11801,0x11801,0x11801}, /* 11800 */
  {0x11802,0x11802,0x11802},  {0x11803,0x11803,0x11803}, /* 11802 */
  {0x11804,0x11804,0x11804},  {0x11805,0x11805,0x11805}, /* 11804 */
  {0x11806,0x11806,0x11806},  {0x11807,0x11807,0x11807}, /* 11806 */
  {0x11808,0x11808,0x11808},  {0x11809,0x11809,0x11809}, /* 11808 */
  {0x1180A,0x1180A,0x1180A},  {0x1180B,0x1180B,0x1180B}, /* 1180A */
  {0x1180C,0x1180C,0x1180C},  {0x1180D,0x1180D,0x1180D}, /* 1180C */
  {0x1180E,0x1180E,0x1180E},  {0x1180F,0x1180F,0x1180F}, /* 1180E */
  {0x11810,0x11810,0x11810},  {0x11811,0x11811,0x11811}, /* 11810 */
  {0x11812,0x11812,0x11812},  {0x11813,0x11813,0x11813}, /* 11812 */
  {0x11814,0x11814,0x11814},  {0x11815,0x11815,0x11815}, /* 11814 */
  {0x11816,0x11816,0x11816},  {0x11817,0x11817,0x11817}, /* 11816 */
  {0x11818,0x11818,0x11818},  {0x11819,0x11819,0x11819}, /* 11818 */
  {0x1181A,0x1181A,0x1181A},  {0x1181B,0x1181B,0x1181B}, /* 1181A */
  {0x1181C,0x1181C,0x1181C},  {0x1181D,0x1181D,0x1181D}, /* 1181C */
  {0x1181E,0x1181E,0x1181E},  {0x1181F,0x1181F,0x1181F}, /* 1181E */
  {0x11820,0x11820,0x11820},  {0x11821,0x11821,0x11821}, /* 11820 */
  {0x11822,0x11822,0x11822},  {0x11823,0x11823,0x11823}, /* 11822 */
  {0x11824,0x11824,0x11824},  {0x11825,0x11825,0x11825}, /* 11824 */
  {0x11826,0x11826,0x11826},  {0x11827,0x11827,0x11827}, /* 11826 */
  {0x11828,0x11828,0x11828},  {0x11829,0x11829,0x11829}, /* 11828 */
  {0x1182A,0x1182A,0x1182A},  {0x1182B,0x1182B,0x1182B}, /* 1182A */
  {0x1182C,0x1182C,0x1182C},  {0x1182D,0x1182D,0x1182D}, /* 1182C */
  {0x1182E,0x1182E,0x1182E},  {0x1182F,0x1182F,0x1182F}, /* 1182E */
  {0x11830,0x11830,0x11830},  {0x11831,0x11831,0x11831}, /* 11830 */
  {0x11832,0x11832,0x11832},  {0x11833,0x11833,0x11833}, /* 11832 */
  {0x11834,0x11834,0x11834},  {0x11835,0x11835,0x11835}, /* 11834 */
  {0x11836,0x11836,0x11836},  {0x11837,0x11837,0x11837}, /* 11836 */
  {0x11838,0x11838,0x11838},  {0x11839,0x11839,0x11839}, /* 11838 */
  {0x1183A,0x1183A,0x1183A},  {0x1183B,0x1183B,0x1183B}, /* 1183A */
  {0x1183C,0x1183C,0x1183C},  {0x1183D,0x1183D,0x1183D}, /* 1183C */
  {0x1183E,0x1183E,0x1183E},  {0x1183F,0x1183F,0x1183F}, /* 1183E */
  {0x11840,0x11840,0x11840},  {0x11841,0x11841,0x11841}, /* 11840 */
  {0x11842,0x11842,0x11842},  {0x11843,0x11843,0x11843}, /* 11842 */
  {0x11844,0x11844,0x11844},  {0x11845,0x11845,0x11845}, /* 11844 */
  {0x11846,0x11846,0x11846},  {0x11847,0x11847,0x11847}, /* 11846 */
  {0x11848,0x11848,0x11848},  {0x11849,0x11849,0x11849}, /* 11848 */
  {0x1184A,0x1184A,0x1184A},  {0x1184B,0x1184B,0x1184B}, /* 1184A */
  {0x1184C,0x1184C,0x1184C},  {0x1184D,0x1184D,0x1184D}, /* 1184C */
  {0x1184E,0x1184E,0x1184E},  {0x1184F,0x1184F,0x1184F}, /* 1184E */
  {0x11850,0x11850,0x11850},  {0x11851,0x11851,0x11851}, /* 11850 */
  {0x11852,0x11852,0x11852},  {0x11853,0x11853,0x11853}, /* 11852 */
  {0x11854,0x11854,0x11854},  {0x11855,0x11855,0x11855}, /* 11854 */
  {0x11856,0x11856,0x11856},  {0x11857,0x11857,0x11857}, /* 11856 */
  {0x11858,0x11858,0x11858},  {0x11859,0x11859,0x11859}, /* 11858 */
  {0x1185A,0x1185A,0x1185A},  {0x1185B,0x1185B,0x1185B}, /* 1185A */
  {0x1185C,0x1185C,0x1185C},  {0x1185D,0x1185D,0x1185D}, /* 1185C */
  {0x1185E,0x1185E,0x1185E},  {0x1185F,0x1185F,0x1185F}, /* 1185E */
  {0x11860,0x11860,0x11860},  {0x11861,0x11861,0x11861}, /* 11860 */
  {0x11862,0x11862,0x11862},  {0x11863,0x11863,0x11863}, /* 11862 */
  {0x11864,0x11864,0x11864},  {0x11865,0x11865,0x11865}, /* 11864 */
  {0x11866,0x11866,0x11866},  {0x11867,0x11867,0x11867}, /* 11866 */
  {0x11868,0x11868,0x11868},  {0x11869,0x11869,0x11869}, /* 11868 */
  {0x1186A,0x1186A,0x1186A},  {0x1186B,0x1186B,0x1186B}, /* 1186A */
  {0x1186C,0x1186C,0x1186C},  {0x1186D,0x1186D,0x1186D}, /* 1186C */
  {0x1186E,0x1186E,0x1186E},  {0x1186F,0x1186F,0x1186F}, /* 1186E */
  {0x11870,0x11870,0x11870},  {0x11871,0x11871,0x11871}, /* 11870 */
  {0x11872,0x11872,0x11872},  {0x11873,0x11873,0x11873}, /* 11872 */
  {0x11874,0x11874,0x11874},  {0x11875,0x11875,0x11875}, /* 11874 */
  {0x11876,0x11876,0x11876},  {0x11877,0x11877,0x11877}, /* 11876 */
  {0x11878,0x11878,0x11878},  {0x11879,0x11879,0x11879}, /* 11878 */
  {0x1187A,0x1187A,0x1187A},  {0x1187B,0x1187B,0x1187B}, /* 1187A */
  {0x1187C,0x1187C,0x1187C},  {0x1187D,0x1187D,0x1187D}, /* 1187C */
  {0x1187E,0x1187E,0x1187E},  {0x1187F,0x1187F,0x1187F}, /* 1187E */
  {0x11880,0x11880,0x11880},  {0x11881,0x11881,0x11881}, /* 11880 */
  {0x11882,0x11882,0x11882},  {0x11883,0x11883,0x11883}, /* 11882 */
  {0x11884,0x11884,0x11884},  {0x11885,0x11885,0x11885}, /* 11884 */
  {0x11886,0x11886,0x11886},  {0x11887,0x11887,0x11887}, /* 11886 */
  {0x11888,0x11888,0x11888},  {0x11889,0x11889,0x11889}, /* 11888 */
  {0x1188A,0x1188A,0x1188A},  {0x1188B,0x1188B,0x1188B}, /* 1188A */
  {0x1188C,0x1188C,0x1188C},  {0x1188D,0x1188D,0x1188D}, /* 1188C */
  {0x1188E,0x1188E,0x1188E},  {0x1188F,0x1188F,0x1188F}, /* 1188E */
  {0x11890,0x11890,0x11890},  {0x11891,0x11891,0x11891}, /* 11890 */
  {0x11892,0x11892,0x11892},  {0x11893,0x11893,0x11893}, /* 11892 */
  {0x11894,0x11894,0x11894},  {0x11895,0x11895,0x11895}, /* 11894 */
  {0x11896,0x11896,0x11896},  {0x11897,0x11897,0x11897}, /* 11896 */
  {0x11898,0x11898,0x11898},  {0x11899,0x11899,0x11899}, /* 11898 */
  {0x1189A,0x1189A,0x1189A},  {0x1189B,0x1189B,0x1189B}, /* 1189A */
  {0x1189C,0x1189C,0x1189C},  {0x1189D,0x1189D,0x1189D}, /* 1189C */
  {0x1189E,0x1189E,0x1189E},  {0x1189F,0x1189F,0x1189F}, /* 1189E */
  {0x118A0,0x118C0,0x118A0},  {0x118A1,0x118C1,0x118A1}, /* 118A0 */
  {0x118A2,0x118C2,0x118A2},  {0x118A3,0x118C3,0x118A3}, /* 118A2 */
  {0x118A4,0x118C4,0x118A4},  {0x118A5,0x118C5,0x118A5}, /* 118A4 */
  {0x118A6,0x118C6,0x118A6},  {0x118A7,0x118C7,0x118A7}, /* 118A6 */
  {0x118A8,0x118C8,0x118A8},  {0x118A9,0x118C9,0x118A9}, /* 118A8 */
  {0x118AA,0x118CA,0x118AA},  {0x118AB,0x118CB,0x118AB}, /* 118AA */
  {0x118AC,0x118CC,0x118AC},  {0x118AD,0x118CD,0x118AD}, /* 118AC */
  {0x118AE,0x118CE,0x118AE},  {0x118AF,0x118CF,0x118AF}, /* 118AE */
  {0x118B0,0x118D0,0x118B0},  {0x118B1,0x118D1,0x118B1}, /* 118B0 */
  {0x118B2,0x118D2,0x118B2},  {0x118B3,0x118D3,0x118B3}, /* 118B2 */
  {0x118B4,0x118D4,0x118B4},  {0x118B5,0x118D5,0x118B5}, /* 118B4 */
  {0x118B6,0x118D6,0x118B6},  {0x118B7,0x118D7,0x118B7}, /* 118B6 */
  {0x118B8,0x118D8,0x118B8},  {0x118B9,0x118D9,0x118B9}, /* 118B8 */
  {0x118BA,0x118DA,0x118BA},  {0x118BB,0x118DB,0x118BB}, /* 118BA */
  {0x118BC,0x118DC,0x118BC},  {0x118BD,0x118DD,0x118BD}, /* 118BC */
  {0x118BE,0x118DE,0x118BE},  {0x118BF,0x118DF,0x118BF}, /* 118BE */
  {0x118A0,0x118C0,0x118A0},  {0x118A1,0x118C1,0x118A1}, /* 118C0 */
  {0x118A2,0x118C2,0x118A2},  {0x118A3,0x118C3,0x118A3}, /* 118C2 */
  {0x118A4,0x118C4,0x118A4},  {0x118A5,0x118C5,0x118A5}, /* 118C4 */
  {0x118A6,0x118C6,0x118A6},  {0x118A7,0x118C7,0x118A7}, /* 118C6 */
  {0x118A8,0x118C8,0x118A8},  {0x118A9,0x118C9,0x118A9}, /* 118C8 */
  {0x118AA,0x118CA,0x118AA},  {0x118AB,0x118CB,0x118AB}, /* 118CA */
  {0x118AC,0x118CC,0x118AC},  {0x118AD,0x118CD,0x118AD}, /* 118CC */
  {0x118AE,0x118CE,0x118AE},  {0x118AF,0x118CF,0x118AF}, /* 118CE */
  {0x118B0,0x118D0,0x118B0},  {0x118B1,0x118D1,0x118B1}, /* 118D0 */
  {0x118B2,0x118D2,0x118B2},  {0x118B3,0x118D3,0x118B3}, /* 118D2 */
  {0x118B4,0x118D4,0x118B4},  {0x118B5,0x118D5,0x118B5}, /* 118D4 */
  {0x118B6,0x118D6,0x118B6},  {0x118B7,0x118D7,0x118B7}, /* 118D6 */
  {0x118B8,0x118D8,0x118B8},  {0x118B9,0x118D9,0x118B9}, /* 118D8 */
  {0x118BA,0x118DA,0x118BA},  {0x118BB,0x118DB,0x118BB}, /* 118DA */
  {0x118BC,0x118DC,0x118BC},  {0x118BD,0x118DD,0x118BD}, /* 118DC */
  {0x118BE,0x118DE,0x118BE},  {0x118BF,0x118DF,0x118BF}, /* 118DE */
  {0x118E0,0x118E0,0x118E0},  {0x118E1,0x118E1,0x118E1}, /* 118E0 */
  {0x118E2,0x118E2,0x118E2},  {0x118E3,0x118E3,0x118E3}, /* 118E2 */
  {0x118E4,0x118E4,0x118E4},  {0x118E5,0x118E5,0x118E5}, /* 118E4 */
  {0x118E6,0x118E6,0x118E6},  {0x118E7,0x118E7,0x118E7}, /* 118E6 */
  {0x118E8,0x118E8,0x118E8},  {0x118E9,0x118E9,0x118E9}, /* 118E8 */
  {0x118EA,0x118EA,0x118EA},  {0x118EB,0x118EB,0x118EB}, /* 118EA */
  {0x118EC,0x118EC,0x118EC},  {0x118ED,0x118ED,0x118ED}, /* 118EC */
  {0x118EE,0x118EE,0x118EE},  {0x118EF,0x118EF,0x118EF}, /* 118EE */
  {0x118F0,0x118F0,0x118F0},  {0x118F1,0x118F1,0x118F1}, /* 118F0 */
  {0x118F2,0x118F2,0x118F2},  {0x118F3,0x118F3,0x118F3}, /* 118F2 */
  {0x118F4,0x118F4,0x118F4},  {0x118F5,0x118F5,0x118F5}, /* 118F4 */
  {0x118F6,0x118F6,0x118F6},  {0x118F7,0x118F7,0x118F7}, /* 118F6 */
  {0x118F8,0x118F8,0x118F8},  {0x118F9,0x118F9,0x118F9}, /* 118F8 */
  {0x118FA,0x118FA,0x118FA},  {0x118FB,0x118FB,0x118FB}, /* 118FA */
  {0x118FC,0x118FC,0x118FC},  {0x118FD,0x118FD,0x118FD}, /* 118FC */
  {0x118FE,0x118FE,0x118FE},  {0x118FF,0x118FF,0x118FF}  /* 118FE */
};

static const MY_UNICASE_CHARACTER u900p1E9[]= {
  {0x1E900,0x1E922,0x1E900},  {0x1E901,0x1E923,0x1E901}, /* 1E900 */
  {0x1E902,0x1E924,0x1E902},  {0x1E903,0x1E925,0x1E903}, /* 1E902 */
  {0x1E904,0x1E926,0x1E904},  {0x1E905,0x1E927,0x1E905}, /* 1E904 */
  {0x1E906,0x1E928,0x1E906},  {0x1E907,0x1E929,0x1E907}, /* 1E906 */
  {0x1E908,0x1E92A,0x1E908},  {0x1E909,0x1E92B,0x1E909}, /* 1E908 */
  {0x1E90A,0x1E92C,0x1E90A},  {0x1E90B,0x1E92D,0x1E90B}, /* 1E90A */
  {0x1E90C,0x1E92E,0x1E90C},  {0x1E90D,0x1E92F,0x1E90D}, /* 1E90C */
  {0x1E90E,0x1E930,0x1E90E},  {0x1E90F,0x1E931,0x1E90F}, /* 1E90E */
  {0x1E910,0x1E932,0x1E910},  {0x1E911,0x1E933,0x1E911}, /* 1E910 */
  {0x1E912,0x1E934,0x1E912},  {0x1E913,0x1E935,0x1E913}, /* 1E912 */
  {0x1E914,0x1E936,0x1E914},  {0x1E915,0x1E937,0x1E915}, /* 1E914 */
  {0x1E916,0x1E938,0x1E916},  {0x1E917,0x1E939,0x1E917}, /* 1E916 */
  {0x1E918,0x1E93A,0x1E918},  {0x1E919,0x1E93B,0x1E919}, /* 1E918 */
  {0x1E91A,0x1E93C,0x1E91A},  {0x1E91B,0x1E93D,0x1E91B}, /* 1E91A */
  {0x1E91C,0x1E93E,0x1E91C},  {0x1E91D,0x1E93F,0x1E91D}, /* 1E91C */
  {0x1E91E,0x1E940,0x1E91E},  {0x1E91F,0x1E941,0x1E91F}, /* 1E91E */
  {0x1E920,0x1E942,0x1E920},  {0x1E921,0x1E943,0x1E921}, /* 1E920 */
  {0x1E900,0x1E922,0x1E900},  {0x1E901,0x1E923,0x1E901}, /* 1E922 */
  {0x1E902,0x1E924,0x1E902},  {0x1E903,0x1E925,0x1E903}, /* 1E924 */
  {0x1E904,0x1E926,0x1E904},  {0x1E905,0x1E927,0x1E905}, /* 1E926 */
  {0x1E906,0x1E928,0x1E906},  {0x1E907,0x1E929,0x1E907}, /* 1E928 */
  {0x1E908,0x1E92A,0x1E908},  {0x1E909,0x1E92B,0x1E909}, /* 1E92A */
  {0x1E90A,0x1E92C,0x1E90A},  {0x1E90B,0x1E92D,0x1E90B}, /* 1E92C */
  {0x1E90C,0x1E92E,0x1E90C},  {0x1E90D,0x1E92F,0x1E90D}, /* 1E92E */
  {0x1E90E,0x1E930,0x1E90E},  {0x1E90F,0x1E931,0x1E90F}, /* 1E930 */
  {0x1E910,0x1E932,0x1E910},  {0x1E911,0x1E933,0x1E911}, /* 1E932 */
  {0x1E912,0x1E934,0x1E912},  {0x1E913,0x1E935,0x1E913}, /* 1E934 */
  {0x1E914,0x1E936,0x1E914},  {0x1E915,0x1E937,0x1E915}, /* 1E936 */
  {0x1E916,0x1E938,0x1E916},  {0x1E917,0x1E939,0x1E917}, /* 1E938 */
  {0x1E918,0x1E93A,0x1E918},  {0x1E919,0x1E93B,0x1E919}, /* 1E93A */
  {0x1E91A,0x1E93C,0x1E91A},  {0x1E91B,0x1E93D,0x1E91B}, /* 1E93C */
  {0x1E91C,0x1E93E,0x1E91C},  {0x1E91D,0x1E93F,0x1E91D}, /* 1E93E */
  {0x1E91E,0x1E940,0x1E91E},  {0x1E91F,0x1E941,0x1E91F}, /* 1E940 */
  {0x1E920,0x1E942,0x1E920},  {0x1E921,0x1E943,0x1E921}, /* 1E942 */
  {0x1E944,0x1E944,0x1E944},  {0x1E945,0x1E945,0x1E945}, /* 1E944 */
  {0x1E946,0x1E946,0x1E946},  {0x1E947,0x1E947,0x1E947}, /* 1E946 */
  {0x1E948,0x1E948,0x1E948},  {0x1E949,0x1E949,0x1E949}, /* 1E948 */
  {0x1E94A,0x1E94A,0x1E94A},  {0x1E94B,0x1E94B,0x1E94B}, /* 1E94A */
  {0x1E94C,0x1E94C,0x1E94C},  {0x1E94D,0x1E94D,0x1E94D}, /* 1E94C */
  {0x1E94E,0x1E94E,0x1E94E},  {0x1E94F,0x1E94F,0x1E94F}, /* 1E94E */
  {0x1E950,0x1E950,0x1E950},  {0x1E951,0x1E951,0x1E951}, /* 1E950 */
  {0x1E952,0x1E952,0x1E952},  {0x1E953,0x1E953,0x1E953}, /* 1E952 */
  {0x1E954,0x1E954,0x1E954},  {0x1E955,0x1E955,0x1E955}, /* 1E954 */
  {0x1E956,0x1E956,0x1E956},  {0x1E957,0x1E957,0x1E957}, /* 1E956 */
  {0x1E958,0x1E958,0x1E958},  {0x1E959,0x1E959,0x1E959}, /* 1E958 */
  {0x1E95A,0x1E95A,0x1E95A},  {0x1E95B,0x1E95B,0x1E95B}, /* 1E95A */
  {0x1E95C,0x1E95C,0x1E95C},  {0x1E95D,0x1E95D,0x1E95D}, /* 1E95C */
  {0x1E95E,0x1E95E,0x1E95E},  {0x1E95F,0x1E95F,0x1E95F}, /* 1E95E */
  {0x1E960,0x1E960,0x1E960},  {0x1E961,0x1E961,0x1E961}, /* 1E960 */
  {0x1E962,0x1E962,0x1E962},  {0x1E963,0x1E963,0x1E963}, /* 1E962 */
  {0x1E964,0x1E964,0x1E964},  {0x1E965,0x1E965,0x1E965}, /* 1E964 */
  {0x1E966,0x1E966,0x1E966},  {0x1E967,0x1E967,0x1E967}, /* 1E966 */
  {0x1E968,0x1E968,0x1E968},  {0x1E969,0x1E969,0x1E969}, /* 1E968 */
  {0x1E96A,0x1E96A,0x1E96A},  {0x1E96B,0x1E96B,0x1E96B}, /* 1E96A */
  {0x1E96C,0x1E96C,0x1E96C},  {0x1E96D,0x1E96D,0x1E96D}, /* 1E96C */
  {0x1E96E,0x1E96E,0x1E96E},  {0x1E96F,0x1E96F,0x1E96F}, /* 1E96E */
  {0x1E970,0x1E970,0x1E970},  {0x1E971,0x1E971,0x1E971}, /* 1E970 */
  {0x1E972,0x1E972,0x1E972},  {0x1E973,0x1E973,0x1E973}, /* 1E972 */
  {0x1E974,0x1E974,0x1E974},  {0x1E975,0x1E975,0x1E975}, /* 1E974 */
  {0x1E976,0x1E976,0x1E976},  {0x1E977,0x1E977,0x1E977}, /* 1E976 */
  {0x1E978,0x1E978,0x1E978},  {0x1E979,0x1E979,0x1E979}, /* 1E978 */
  {0x1E97A,0x1E97A,0x1E97A},  {0x1E97B,0x1E97B,0x1E97B}, /* 1E97A */
  {0x1E97C,0x1E97C,0x1E97C},  {0x1E97D,0x1E97D,0x1E97D}, /* 1E97C */
  {0x1E97E,0x1E97E,0x1E97E},  {0x1E97F,0x1E97F,0x1E97F}, /* 1E97E */
  {0x1E980,0x1E980,0x1E980},  {0x1E981,0x1E981,0x1E981}, /* 1E980 */
  {0x1E982,0x1E982,0x1E982},  {0x1E983,0x1E983,0x1E983}, /* 1E982 */
  {0x1E984,0x1E984,0x1E984},  {0x1E985,0x1E985,0x1E985}, /* 1E984 */
  {0x1E986,0x1E986,0x1E986},  {0x1E987,0x1E987,0x1E987}, /* 1E986 */
  {0x1E988,0x1E988,0x1E988},  {0x1E989,0x1E989,0x1E989}, /* 1E988 */
  {0x1E98A,0x1E98A,0x1E98A},  {0x1E98B,0x1E98B,0x1E98B}, /* 1E98A */
  {0x1E98C,0x1E98C,0x1E98C},  {0x1E98D,0x1E98D,0x1E98D}, /* 1E98C */
  {0x1E98E,0x1E98E,0x1E98E},  {0x1E98F,0x1E98F,0x1E98F}, /* 1E98E */
  {0x1E990,0x1E990,0x1E990},  {0x1E991,0x1E991,0x1E991}, /* 1E990 */
  {0x1E992,0x1E992,0x1E992},  {0x1E993,0x1E993,0x1E993}, /* 1E992 */
  {0x1E994,0x1E994,0x1E994},  {0x1E995,0x1E995,0x1E995}, /* 1E994 */
  {0x1E996,0x1E996,0x1E996},  {0x1E997,0x1E997,0x1E997}, /* 1E996 */
  {0x1E998,0x1E998,0x1E998},  {0x1E999,0x1E999,0x1E999}, /* 1E998 */
  {0x1E99A,0x1E99A,0x1E99A},  {0x1E99B,0x1E99B,0x1E99B}, /* 1E99A */
  {0x1E99C,0x1E99C,0x1E99C},  {0x1E99D,0x1E99D,0x1E99D}, /* 1E99C */
  {0x1E99E,0x1E99E,0x1E99E},  {0x1E99F,0x1E99F,0x1E99F}, /* 1E99E */
  {0x1E9A0,0x1E9A0,0x1E9A0},  {0x1E9A1,0x1E9A1,0x1E9A1}, /* 1E9A0 */
  {0x1E9A2,0x1E9A2,0x1E9A2},  {0x1E9A3,0x1E9A3,0x1E9A3}, /* 1E9A2 */
  {0x1E9A4,0x1E9A4,0x1E9A4},  {0x1E9A5,0x1E9A5,0x1E9A5}, /* 1E9A4 */
  {0x1E9A6,0x1E9A6,0x1E9A6},  {0x1E9A7,0x1E9A7,0x1E9A7}, /* 1E9A6 */
  {0x1E9A8,0x1E9A8,0x1E9A8},  {0x1E9A9,0x1E9A9,0x1E9A9}, /* 1E9A8 */
  {0x1E9AA,0x1E9AA,0x1E9AA},  {0x1E9AB,0x1E9AB,0x1E9AB}, /* 1E9AA */
  {0x1E9AC,0x1E9AC,0x1E9AC},  {0x1E9AD,0x1E9AD,0x1E9AD}, /* 1E9AC */
  {0x1E9AE,0x1E9AE,0x1E9AE},  {0x1E9AF,0x1E9AF,0x1E9AF}, /* 1E9AE */
  {0x1E9B0,0x1E9B0,0x1E9B0},  {0x1E9B1,0x1E9B1,0x1E9B1}, /* 1E9B0 */
  {0x1E9B2,0x1E9B2,0x1E9B2},  {0x1E9B3,0x1E9B3,0x1E9B3}, /* 1E9B2 */
  {0x1E9B4,0x1E9B4,0x1E9B4},  {0x1E9B5,0x1E9B5,0x1E9B5}, /* 1E9B4 */
  {0x1E9B6,0x1E9B6,0x1E9B6},  {0x1E9B7,0x1E9B7,0x1E9B7}, /* 1E9B6 */
  {0x1E9B8,0x1E9B8,0x1E9B8},  {0x1E9B9,0x1E9B9,0x1E9B9}, /* 1E9B8 */
  {0x1E9BA,0x1E9BA,0x1E9BA},  {0x1E9BB,0x1E9BB,0x1E9BB}, /* 1E9BA */
  {0x1E9BC,0x1E9BC,0x1E9BC},  {0x1E9BD,0x1E9BD,0x1E9BD}, /* 1E9BC */
  {0x1E9BE,0x1E9BE,0x1E9BE},  {0x1E9BF,0x1E9BF,0x1E9BF}, /* 1E9BE */
  {0x1E9C0,0x1E9C0,0x1E9C0},  {0x1E9C1,0x1E9C1,0x1E9C1}, /* 1E9C0 */
  {0x1E9C2,0x1E9C2,0x1E9C2},  {0x1E9C3,0x1E9C3,0x1E9C3}, /* 1E9C2 */
  {0x1E9C4,0x1E9C4,0x1E9C4},  {0x1E9C5,0x1E9C5,0x1E9C5}, /* 1E9C4 */
  {0x1E9C6,0x1E9C6,0x1E9C6},  {0x1E9C7,0x1E9C7,0x1E9C7}, /* 1E9C6 */
  {0x1E9C8,0x1E9C8,0x1E9C8},  {0x1E9C9,0x1E9C9,0x1E9C9}, /* 1E9C8 */
  {0x1E9CA,0x1E9CA,0x1E9CA},  {0x1E9CB,0x1E9CB,0x1E9CB}, /* 1E9CA */
  {0x1E9CC,0x1E9CC,0x1E9CC},  {0x1E9CD,0x1E9CD,0x1E9CD}, /* 1E9CC */
  {0x1E9CE,0x1E9CE,0x1E9CE},  {0x1E9CF,0x1E9CF,0x1E9CF}, /* 1E9CE */
  {0x1E9D0,0x1E9D0,0x1E9D0},  {0x1E9D1,0x1E9D1,0x1E9D1}, /* 1E9D0 */
  {0x1E9D2,0x1E9D2,0x1E9D2},  {0x1E9D3,0x1E9D3,0x1E9D3}, /* 1E9D2 */
  {0x1E9D4,0x1E9D4,0x1E9D4},  {0x1E9D5,0x1E9D5,0x1E9D5}, /* 1E9D4 */
  {0x1E9D6,0x1E9D6,0x1E9D6},  {0x1E9D7,0x1E9D7,0x1E9D7}, /* 1E9D6 */
  {0x1E9D8,0x1E9D8,0x1E9D8},  {0x1E9D9,0x1E9D9,0x1E9D9}, /* 1E9D8 */
  {0x1E9DA,0x1E9DA,0x1E9DA},  {0x1E9DB,0x1E9DB,0x1E9DB}, /* 1E9DA */
  {0x1E9DC,0x1E9DC,0x1E9DC},  {0x1E9DD,0x1E9DD,0x1E9DD}, /* 1E9DC */
  {0x1E9DE,0x1E9DE,0x1E9DE},  {0x1E9DF,0x1E9DF,0x1E9DF}, /* 1E9DE */
  {0x1E9E0,0x1E9E0,0x1E9E0},  {0x1E9E1,0x1E9E1,0x1E9E1}, /* 1E9E0 */
  {0x1E9E2,0x1E9E2,0x1E9E2},  {0x1E9E3,0x1E9E3,0x1E9E3}, /* 1E9E2 */
  {0x1E9E4,0x1E9E4,0x1E9E4},  {0x1E9E5,0x1E9E5,0x1E9E5}, /* 1E9E4 */
  {0x1E9E6,0x1E9E6,0x1E9E6},  {0x1E9E7,0x1E9E7,0x1E9E7}, /* 1E9E6 */
  {0x1E9E8,0x1E9E8,0x1E9E8},  {0x1E9E9,0x1E9E9,0x1E9E9}, /* 1E9E8 */
  {0x1E9EA,0x1E9EA,0x1E9EA},  {0x1E9EB,0x1E9EB,0x1E9EB}, /* 1E9EA */
  {0x1E9EC,0x1E9EC,0x1E9EC},  {0x1E9ED,0x1E9ED,0x1E9ED}, /* 1E9EC */
  {0x1E9EE,0x1E9EE,0x1E9EE},  {0x1E9EF,0x1E9EF,0x1E9EF}, /* 1E9EE */
  {0x1E9F0,0x1E9F0,0x1E9F0},  {0x1E9F1,0x1E9F1,0x1E9F1}, /* 1E9F0 */
  {0x1E9F2,0x1E9F2,0x1E9F2},  {0x1E9F3,0x1E9F3,0x1E9F3}, /* 1E9F2 */
  {0x1E9F4,0x1E9F4,0x1E9F4},  {0x1E9F5,0x1E9F5,0x1E9F5}, /* 1E9F4 */
  {0x1E9F6,0x1E9F6,0x1E9F6},  {0x1E9F7,0x1E9F7,0x1E9F7}, /* 1E9F6 */
  {0x1E9F8,0x1E9F8,0x1E9F8},  {0x1E9F9,0x1E9F9,0x1E9F9}, /* 1E9F8 */
  {0x1E9FA,0x1E9FA,0x1E9FA},  {0x1E9FB,0x1E9FB,0x1E9FB}, /* 1E9FA */
  {0x1E9FC,0x1E9FC,0x1E9FC},  {0x1E9FD,0x1E9FD,0x1E9FD}, /* 1E9FC */
  {0x1E9FE,0x1E9FE,0x1E9FE},  {0x1E9FF,0x1E9FF,0x1E9FF}  /* 1E9FE */
};

const MY_UNICASE_CHARACTER *my_unicase_pages_unicode900[4352] = {
    u900p000, u900p001, u900p002, u900p003, u900p004, u900p005, nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  u900p010, nullptr,  nullptr,  u900p013, nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    u900p01C, u900p01D, u900p01E, u900p01F, nullptr,  u900p021, nullptr,
    nullptr,  u900p024, nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  u900p02C, u900p02D, nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  u900p0A6, u900p0A7,
    nullptr,  nullptr,  nullptr,  u900p0AB, nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  u900p0FF, nullptr,  nullptr,  nullptr,
    nullptr,  u900p104, nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  u900p10C, nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    u900p118, nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  u900p1E9,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
    nullptr,  nullptr,  nullptr,  nullptr,  nullptr,
};

MY_UNICASE_INFO my_unicase_unicode900=
{
  0x10FFFF,
  my_unicase_pages_unicode900
};

MY_UCA_INFO my_uca_v900 = {
    UCA_V900,

    0x10FFFF,                      /* maxchar           */
    nullptr,                       /* length - not used */
    uca900_weight, false, nullptr, /* contractions      */
    nullptr,

    0x0009,  /* first_non_ignorable       p != ignore                       */
    0x14646, /* last_non_ignorable        Not a CJK and not UASSIGNED       */

    0x0332,  /* first_primary_ignorable   p == ignore                       */
    0x101FD, /* last_primary_ignorable                                      */

    0x0000, /* first_secondary_ignorable p,s= ignore                       */
    0xFE73, /* last_secondary_ignorable                                    */

    0x0000, /* first_tertiary_ignorable  p,s,t == ignore                   */
    0xFE73, /* last_tertiary_ignorable                                     */

    0x0000, /* first_trailing                                              */
    0x0000, /* last_trailing                                               */

    0x0009,  /* first_variable            if alt=non-ignorable: p != ignore */
    0x1D371, /* last_variable             if alt=shifter: p,s,t == ignore   */
    /*
      By definition of DUCET 9.0.0, the weight range is:
        Primary weight range:   0200..54A3 (21156)
        Secondary weight range: 0020..0114 (245)
        Variant secondaries:    0110..0114 (5)
        Tertiary weight range:  0002..001F (30)
      To make the extra CE's weight bigger than any weight else, we define it
      as the 'max + 1'.
    */
    0x54A4, /* extra_ce_pri_base  */
    0x0115, /* extra_ce_sec_base  */
    0x0020, /* extra_ce_ter_base  */
};
#endif
